/*------------------------------------------------------------------------------
 * MDK Middleware - Component ::USB:Device
 * Copyright (c) 2004-2017 ARM Germany GmbH. All rights reserved.
 *------------------------------------------------------------------------------
 * Name:    usbd_cmsis_rtos1.c
 * Purpose: USB Device (USBD) - RTOS abstraction implemented on CMSIS-RTOS1
 *----------------------------------------------------------------------------*/

#include "RTE_Components.h"
#include "usb_os.h"

#ifdef        RTE_USB_Device_CustomClass_0
#include "..\USB\USBD_Config_CustomClass_0.h"
#endif
#ifdef        RTE_USB_Device_CustomClass_1
#include "..\USB\USBD_Config_CustomClass_1.h"
#endif
#ifdef        RTE_USB_Device_CustomClass_2
#include "..\USB\USBD_Config_CustomClass_2.h"
#endif
#ifdef        RTE_USB_Device_CustomClass_3
#include "..\USB\USBD_Config_CustomClass_3.h"
#endif

#ifdef        RTE_USB_Device_ADC_0
#include "..\USB\USBD_Config_ADC_0.h"
#endif
#ifdef        RTE_USB_Device_ADC_1
#include "..\USB\USBD_Config_ADC_1.h"
#endif
#ifdef        RTE_USB_Device_ADC_2
#include "..\USB\USBD_Config_ADC_2.h"
#endif
#ifdef        RTE_USB_Device_ADC_3
#include "..\USB\USBD_Config_ADC_3.h"
#endif

#ifdef        RTE_USB_Device_CDC_0
#include "..\USB\USBD_Config_CDC_0.h"
#endif
#ifdef        RTE_USB_Device_CDC_1
#include "..\USB\USBD_Config_CDC_1.h"
#endif
#ifdef        RTE_USB_Device_CDC_2
#include "..\USB\USBD_Config_CDC_2.h"
#endif
#ifdef        RTE_USB_Device_CDC_3
#include "..\USB\USBD_Config_CDC_3.h"
#endif

#ifdef        RTE_USB_Device_HID_0
#include "..\USB\USBD_Config_HID_0.h"
#endif
#ifdef        RTE_USB_Device_HID_1
#include "..\USB\USBD_Config_HID_1.h"
#endif
#ifdef        RTE_USB_Device_HID_2
#include "..\USB\USBD_Config_HID_2.h"
#endif
#ifdef        RTE_USB_Device_HID_3
#include "..\USB\USBD_Config_HID_3.h"
#endif

#ifdef        RTE_USB_Device_MSC_0
#include "..\USB\USBD_Config_MSC_0.h"
#endif
#ifdef        RTE_USB_Device_MSC_1
#include "..\USB\USBD_Config_MSC_1.h"
#endif
#ifdef        RTE_USB_Device_MSC_2
#include "..\USB\USBD_Config_MSC_2.h"
#endif
#ifdef        RTE_USB_Device_MSC_3
#include "..\USB\USBD_Config_MSC_3.h"
#endif

#include "usbd_compatibility.h"


// Local macro definitions
#define osThreadExt(name)                                                  extern const osThreadDef_t    os_thread_def_##name
#define osTimerExt(name)        extern uint32_t os_timer_cb_##name[6];     extern const osTimerDef_t     os_timer_def_##name
#define osMutexExt(name)        extern uint32_t os_mutex_cb_##name[4];     extern const osMutexDef_t     os_mutex_def_##name
#define osSemaphoreExt(name)    extern uint32_t os_semaphore_cb_##name[2]; extern const osSemaphoreDef_t os_semaphore_def_##name


// Resources definition

// Import thread definitions generated by each Device instance
// in usbd_config_dev_n.c file (n = 0 .. 3)
#ifdef RTE_USB_Device_0
osThreadExt(USBD0_Core_Thread);
#endif
#ifdef RTE_USB_Device_1
osThreadExt(USBD1_Core_Thread);
#endif
#ifdef RTE_USB_Device_2
osThreadExt(USBD2_Core_Thread);
#endif
#ifdef RTE_USB_Device_3
osThreadExt(USBD3_Core_Thread);
#endif

static const osThreadDef_t * const usbd_core_thread_def_ptr[USBD_DEV_NUM] = {
#ifdef RTE_USB_Device_0
    osThread(USBD0_Core_Thread)
#endif
#ifdef RTE_USB_Device_1
  , osThread(USBD1_Core_Thread)
#endif
#ifdef RTE_USB_Device_2
  , osThread(USBD2_Core_Thread)
#endif
#ifdef RTE_USB_Device_3
  , osThread(USBD3_Core_Thread)
#endif
};


// Create thread definitions for classes requiring threads
#if   (USBD_CUSTOM_CLASS_NUM > 0)

#ifdef RTE_USB_Device_CustomClass_0
#if   (USBD_EP_LAST_NUM >=      1)
extern void USBD_CustomClass0_EP1_Thread   (void const *arg);
       void USBD_CustomClass0_EP1_Thread   (void const *arg) { USBD_CustomClass_EP_Thread((void *)0U); }
osThreadExt(USBD_CustomClass0_EP1_Thread);
osThreadDef(USBD_CustomClass0_EP1_Thread,  USBD_CUSTOM_CLASS0_EP1_THREAD_PRIORITY,  1U, USBD_CUSTOM_CLASS0_EP1_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      2)
extern void USBD_CustomClass0_EP2_Thread   (void const *arg);
       void USBD_CustomClass0_EP2_Thread   (void const *arg) { USBD_CustomClass_EP_Thread((void *)0U); }
osThreadExt(USBD_CustomClass0_EP2_Thread);
osThreadDef(USBD_CustomClass0_EP2_Thread,  USBD_CUSTOM_CLASS0_EP2_THREAD_PRIORITY,  1U, USBD_CUSTOM_CLASS0_EP2_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      3)
extern void USBD_CustomClass0_EP3_Thread   (void const *arg);
       void USBD_CustomClass0_EP3_Thread   (void const *arg) { USBD_CustomClass_EP_Thread((void *)0U); }
osThreadExt(USBD_CustomClass0_EP3_Thread);
osThreadDef(USBD_CustomClass0_EP3_Thread,  USBD_CUSTOM_CLASS0_EP3_THREAD_PRIORITY,  1U, USBD_CUSTOM_CLASS0_EP3_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      4)
extern void USBD_CustomClass0_EP4_Thread   (void const *arg);
       void USBD_CustomClass0_EP4_Thread   (void const *arg) { USBD_CustomClass_EP_Thread((void *)0U); }
osThreadExt(USBD_CustomClass0_EP4_Thread);
osThreadDef(USBD_CustomClass0_EP4_Thread,  USBD_CUSTOM_CLASS0_EP4_THREAD_PRIORITY,  1U, USBD_CUSTOM_CLASS0_EP4_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      5)
extern void USBD_CustomClass0_EP5_Thread   (void const *arg);
       void USBD_CustomClass0_EP5_Thread   (void const *arg) { USBD_CustomClass_EP_Thread((void *)0U); }
osThreadExt(USBD_CustomClass0_EP5_Thread);
osThreadDef(USBD_CustomClass0_EP5_Thread,  USBD_CUSTOM_CLASS0_EP5_THREAD_PRIORITY,  1U, USBD_CUSTOM_CLASS0_EP5_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      6)
extern void USBD_CustomClass0_EP6_Thread   (void const *arg);
       void USBD_CustomClass0_EP6_Thread   (void const *arg) { USBD_CustomClass_EP_Thread((void *)0U); }
osThreadExt(USBD_CustomClass0_EP6_Thread);
osThreadDef(USBD_CustomClass0_EP6_Thread,  USBD_CUSTOM_CLASS0_EP6_THREAD_PRIORITY,  1U, USBD_CUSTOM_CLASS0_EP6_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      7)
extern void USBD_CustomClass0_EP7_Thread   (void const *arg);
       void USBD_CustomClass0_EP7_Thread   (void const *arg) { USBD_CustomClass_EP_Thread((void *)0U); }
osThreadExt(USBD_CustomClass0_EP7_Thread);
osThreadDef(USBD_CustomClass0_EP7_Thread,  USBD_CUSTOM_CLASS0_EP7_THREAD_PRIORITY,  1U, USBD_CUSTOM_CLASS0_EP7_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      8)
extern void USBD_CustomClass0_EP8_Thread   (void const *arg);
       void USBD_CustomClass0_EP8_Thread   (void const *arg) { USBD_CustomClass_EP_Thread((void *)0U); }
osThreadExt(USBD_CustomClass0_EP8_Thread);
osThreadDef(USBD_CustomClass0_EP8_Thread,  USBD_CUSTOM_CLASS0_EP8_THREAD_PRIORITY,  1U, USBD_CUSTOM_CLASS0_EP8_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      9)
extern void USBD_CustomClass0_EP9_Thread   (void const *arg);
       void USBD_CustomClass0_EP9_Thread   (void const *arg) { USBD_CustomClass_EP_Thread((void *)0U); }
osThreadExt(USBD_CustomClass0_EP9_Thread);
osThreadDef(USBD_CustomClass0_EP9_Thread,  USBD_CUSTOM_CLASS0_EP9_THREAD_PRIORITY,  1U, USBD_CUSTOM_CLASS0_EP9_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      10)
extern void USBD_CustomClass0_EP10_Thread  (void const *arg);
       void USBD_CustomClass0_EP10_Thread  (void const *arg) { USBD_CustomClass_EP_Thread((void *)0U); }
osThreadExt(USBD_CustomClass0_EP10_Thread);
osThreadDef(USBD_CustomClass0_EP10_Thread, USBD_CUSTOM_CLASS0_EP10_THREAD_PRIORITY, 1U, USBD_CUSTOM_CLASS0_EP10_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      11)
extern void USBD_CustomClass0_EP11_Thread  (void const *arg);
       void USBD_CustomClass0_EP11_Thread  (void const *arg) { USBD_CustomClass_EP_Thread((void *)0U); }
osThreadExt(USBD_CustomClass0_EP11_Thread);
osThreadDef(USBD_CustomClass0_EP11_Thread, USBD_CUSTOM_CLASS0_EP11_THREAD_PRIORITY, 1U, USBD_CUSTOM_CLASS0_EP11_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      12)
extern void USBD_CustomClass0_EP12_Thread  (void const *arg);
       void USBD_CustomClass0_EP12_Thread  (void const *arg) { USBD_CustomClass_EP_Thread((void *)0U); }
osThreadExt(USBD_CustomClass0_EP12_Thread);
osThreadDef(USBD_CustomClass0_EP12_Thread, USBD_CUSTOM_CLASS0_EP12_THREAD_PRIORITY, 1U, USBD_CUSTOM_CLASS0_EP12_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      13)
extern void USBD_CustomClass0_EP13_Thread  (void const *arg);
       void USBD_CustomClass0_EP13_Thread  (void const *arg) { USBD_CustomClass_EP_Thread((void *)0U); }
osThreadExt(USBD_CustomClass0_EP13_Thread);
osThreadDef(USBD_CustomClass0_EP13_Thread, USBD_CUSTOM_CLASS0_EP13_THREAD_PRIORITY, 1U, USBD_CUSTOM_CLASS0_EP13_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      14)
extern void USBD_CustomClass0_EP14_Thread  (void const *arg);
       void USBD_CustomClass0_EP14_Thread  (void const *arg) { USBD_CustomClass_EP_Thread((void *)0U); }
osThreadExt(USBD_CustomClass0_EP14_Thread);
osThreadDef(USBD_CustomClass0_EP14_Thread, USBD_CUSTOM_CLASS0_EP14_THREAD_PRIORITY, 1U, USBD_CUSTOM_CLASS0_EP14_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      15)
extern void USBD_CustomClass0_EP15_Thread  (void const *arg);
       void USBD_CustomClass0_EP15_Thread  (void const *arg) { USBD_CustomClass_EP_Thread((void *)0U); }
osThreadExt(USBD_CustomClass0_EP15_Thread);
osThreadDef(USBD_CustomClass0_EP15_Thread, USBD_CUSTOM_CLASS0_EP15_THREAD_PRIORITY, 1U, USBD_CUSTOM_CLASS0_EP15_THREAD_STACK_SIZE);
#endif
#endif

#ifdef RTE_USB_Device_CustomClass_1
#if   (USBD_EP_LAST_NUM >=      1)
extern void USBD_CustomClass1_EP1_Thread   (void const *arg);
       void USBD_CustomClass1_EP1_Thread   (void const *arg) { USBD_CustomClass_EP_Thread((void *)1U); }
osThreadExt(USBD_CustomClass1_EP1_Thread);
osThreadDef(USBD_CustomClass1_EP1_Thread,  USBD_CUSTOM_CLASS1_EP1_THREAD_PRIORITY,  1U, USBD_CUSTOM_CLASS1_EP1_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      2)
extern void USBD_CustomClass1_EP2_Thread   (void const *arg);
       void USBD_CustomClass1_EP2_Thread   (void const *arg) { USBD_CustomClass_EP_Thread((void *)1U); }
osThreadExt(USBD_CustomClass1_EP2_Thread);
osThreadDef(USBD_CustomClass1_EP2_Thread,  USBD_CUSTOM_CLASS1_EP2_THREAD_PRIORITY,  1U, USBD_CUSTOM_CLASS1_EP2_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      3)
extern void USBD_CustomClass1_EP3_Thread   (void const *arg);
       void USBD_CustomClass1_EP3_Thread   (void const *arg) { USBD_CustomClass_EP_Thread((void *)1U); }
osThreadExt(USBD_CustomClass1_EP3_Thread);
osThreadDef(USBD_CustomClass1_EP3_Thread,  USBD_CUSTOM_CLASS1_EP3_THREAD_PRIORITY,  1U, USBD_CUSTOM_CLASS1_EP3_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      4)
extern void USBD_CustomClass1_EP4_Thread   (void const *arg);
       void USBD_CustomClass1_EP4_Thread   (void const *arg) { USBD_CustomClass_EP_Thread((void *)1U); }
osThreadExt(USBD_CustomClass1_EP4_Thread);
osThreadDef(USBD_CustomClass1_EP4_Thread,  USBD_CUSTOM_CLASS1_EP4_THREAD_PRIORITY,  1U, USBD_CUSTOM_CLASS1_EP4_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      5)
extern void USBD_CustomClass1_EP5_Thread   (void const *arg);
       void USBD_CustomClass1_EP5_Thread   (void const *arg) { USBD_CustomClass_EP_Thread((void *)1U); }
osThreadExt(USBD_CustomClass1_EP5_Thread);
osThreadDef(USBD_CustomClass1_EP5_Thread,  USBD_CUSTOM_CLASS1_EP5_THREAD_PRIORITY,  1U, USBD_CUSTOM_CLASS1_EP5_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      6)
extern void USBD_CustomClass1_EP6_Thread   (void const *arg);
       void USBD_CustomClass1_EP6_Thread   (void const *arg) { USBD_CustomClass_EP_Thread((void *)1U); }
osThreadExt(USBD_CustomClass1_EP6_Thread);
osThreadDef(USBD_CustomClass1_EP6_Thread,  USBD_CUSTOM_CLASS1_EP6_THREAD_PRIORITY,  1U, USBD_CUSTOM_CLASS1_EP6_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      7)
extern void USBD_CustomClass1_EP7_Thread   (void const *arg);
       void USBD_CustomClass1_EP7_Thread   (void const *arg) { USBD_CustomClass_EP_Thread((void *)1U); }
osThreadExt(USBD_CustomClass1_EP7_Thread);
osThreadDef(USBD_CustomClass1_EP7_Thread,  USBD_CUSTOM_CLASS1_EP7_THREAD_PRIORITY,  1U, USBD_CUSTOM_CLASS1_EP7_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      8)
extern void USBD_CustomClass1_EP8_Thread   (void const *arg);
       void USBD_CustomClass1_EP8_Thread   (void const *arg) { USBD_CustomClass_EP_Thread((void *)1U); }
osThreadExt(USBD_CustomClass1_EP8_Thread);
osThreadDef(USBD_CustomClass1_EP8_Thread,  USBD_CUSTOM_CLASS1_EP8_THREAD_PRIORITY,  1U, USBD_CUSTOM_CLASS1_EP8_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      9)
extern void USBD_CustomClass1_EP9_Thread   (void const *arg);
       void USBD_CustomClass1_EP9_Thread   (void const *arg) { USBD_CustomClass_EP_Thread((void *)1U); }
osThreadExt(USBD_CustomClass1_EP9_Thread);
osThreadDef(USBD_CustomClass1_EP9_Thread,  USBD_CUSTOM_CLASS1_EP9_THREAD_PRIORITY,  1U, USBD_CUSTOM_CLASS1_EP9_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      10)
extern void USBD_CustomClass1_EP10_Thread  (void const *arg);
       void USBD_CustomClass1_EP10_Thread  (void const *arg) { USBD_CustomClass_EP_Thread((void *)1U); }
osThreadExt(USBD_CustomClass1_EP10_Thread);
osThreadDef(USBD_CustomClass1_EP10_Thread, USBD_CUSTOM_CLASS1_EP10_THREAD_PRIORITY, 1U, USBD_CUSTOM_CLASS1_EP10_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      11)
extern void USBD_CustomClass1_EP11_Thread  (void const *arg);
       void USBD_CustomClass1_EP11_Thread  (void const *arg) { USBD_CustomClass_EP_Thread((void *)1U); }
osThreadExt(USBD_CustomClass1_EP11_Thread);
osThreadDef(USBD_CustomClass1_EP11_Thread, USBD_CUSTOM_CLASS1_EP11_THREAD_PRIORITY, 1U, USBD_CUSTOM_CLASS1_EP11_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      12)
extern void USBD_CustomClass1_EP12_Thread  (void const *arg);
       void USBD_CustomClass1_EP12_Thread  (void const *arg) { USBD_CustomClass_EP_Thread((void *)1U); }
osThreadExt(USBD_CustomClass1_EP12_Thread);
osThreadDef(USBD_CustomClass1_EP12_Thread, USBD_CUSTOM_CLASS1_EP12_THREAD_PRIORITY, 1U, USBD_CUSTOM_CLASS1_EP12_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      13)
extern void USBD_CustomClass1_EP13_Thread  (void const *arg);
       void USBD_CustomClass1_EP13_Thread  (void const *arg) { USBD_CustomClass_EP_Thread((void *)1U); }
osThreadExt(USBD_CustomClass1_EP13_Thread);
osThreadDef(USBD_CustomClass1_EP13_Thread, USBD_CUSTOM_CLASS1_EP13_THREAD_PRIORITY, 1U, USBD_CUSTOM_CLASS1_EP13_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      14)
extern void USBD_CustomClass1_EP14_Thread  (void const *arg);
       void USBD_CustomClass1_EP14_Thread  (void const *arg) { USBD_CustomClass_EP_Thread((void *)1U); }
osThreadExt(USBD_CustomClass1_EP14_Thread);
osThreadDef(USBD_CustomClass1_EP14_Thread, USBD_CUSTOM_CLASS1_EP14_THREAD_PRIORITY, 1U, USBD_CUSTOM_CLASS1_EP14_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      15)
extern void USBD_CustomClass1_EP15_Thread  (void const *arg);
       void USBD_CustomClass1_EP15_Thread  (void const *arg) { USBD_CustomClass_EP_Thread((void *)1U); }
osThreadExt(USBD_CustomClass1_EP15_Thread);
osThreadDef(USBD_CustomClass1_EP15_Thread, USBD_CUSTOM_CLASS1_EP15_THREAD_PRIORITY, 1U, USBD_CUSTOM_CLASS1_EP15_THREAD_STACK_SIZE);
#endif
#endif

#ifdef RTE_USB_Device_CustomClass_2
#if   (USBD_EP_LAST_NUM >=      1)
extern void USBD_CustomClass2_EP1_Thread   (void const *arg);
       void USBD_CustomClass2_EP1_Thread   (void const *arg) { USBD_CustomClass_EP_Thread((void *)2U); }
osThreadExt(USBD_CustomClass2_EP1_Thread);
osThreadDef(USBD_CustomClass2_EP1_Thread,  USBD_CUSTOM_CLASS2_EP1_THREAD_PRIORITY,  1U, USBD_CUSTOM_CLASS2_EP1_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      2)
extern void USBD_CustomClass2_EP2_Thread   (void const *arg);
       void USBD_CustomClass2_EP2_Thread   (void const *arg) { USBD_CustomClass_EP_Thread((void *)2U); }
osThreadExt(USBD_CustomClass2_EP2_Thread);
osThreadDef(USBD_CustomClass2_EP2_Thread,  USBD_CUSTOM_CLASS2_EP2_THREAD_PRIORITY,  1U, USBD_CUSTOM_CLASS2_EP2_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      3)
extern void USBD_CustomClass2_EP3_Thread   (void const *arg);
       void USBD_CustomClass2_EP3_Thread   (void const *arg) { USBD_CustomClass_EP_Thread((void *)2U); }
osThreadExt(USBD_CustomClass2_EP3_Thread);
osThreadDef(USBD_CustomClass2_EP3_Thread,  USBD_CUSTOM_CLASS2_EP3_THREAD_PRIORITY,  1U, USBD_CUSTOM_CLASS2_EP3_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      4)
extern void USBD_CustomClass2_EP4_Thread   (void const *arg);
       void USBD_CustomClass2_EP4_Thread   (void const *arg) { USBD_CustomClass_EP_Thread((void *)2U); }
osThreadExt(USBD_CustomClass2_EP4_Thread);
osThreadDef(USBD_CustomClass2_EP4_Thread,  USBD_CUSTOM_CLASS2_EP4_THREAD_PRIORITY,  1U, USBD_CUSTOM_CLASS2_EP4_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      5)
extern void USBD_CustomClass2_EP5_Thread   (void const *arg);
       void USBD_CustomClass2_EP5_Thread   (void const *arg) { USBD_CustomClass_EP_Thread((void *)2U); }
osThreadExt(USBD_CustomClass2_EP5_Thread);
osThreadDef(USBD_CustomClass2_EP5_Thread,  USBD_CUSTOM_CLASS2_EP5_THREAD_PRIORITY,  1U, USBD_CUSTOM_CLASS2_EP5_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      6)
extern void USBD_CustomClass2_EP6_Thread   (void const *arg);
       void USBD_CustomClass2_EP6_Thread   (void const *arg) { USBD_CustomClass_EP_Thread((void *)2U); }
osThreadExt(USBD_CustomClass2_EP6_Thread);
osThreadDef(USBD_CustomClass2_EP6_Thread,  USBD_CUSTOM_CLASS2_EP6_THREAD_PRIORITY,  1U, USBD_CUSTOM_CLASS2_EP6_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      7)
extern void USBD_CustomClass2_EP7_Thread   (void const *arg);
       void USBD_CustomClass2_EP7_Thread   (void const *arg) { USBD_CustomClass_EP_Thread((void *)2U); }
osThreadExt(USBD_CustomClass2_EP7_Thread);
osThreadDef(USBD_CustomClass2_EP7_Thread,  USBD_CUSTOM_CLASS2_EP7_THREAD_PRIORITY,  1U, USBD_CUSTOM_CLASS2_EP7_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      8)
extern void USBD_CustomClass2_EP8_Thread   (void const *arg);
       void USBD_CustomClass2_EP8_Thread   (void const *arg) { USBD_CustomClass_EP_Thread((void *)2U); }
osThreadExt(USBD_CustomClass2_EP8_Thread);
osThreadDef(USBD_CustomClass2_EP8_Thread,  USBD_CUSTOM_CLASS2_EP8_THREAD_PRIORITY,  1U, USBD_CUSTOM_CLASS2_EP8_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      9)
extern void USBD_CustomClass2_EP9_Thread   (void const *arg);
       void USBD_CustomClass2_EP9_Thread   (void const *arg) { USBD_CustomClass_EP_Thread((void *)2U); }
osThreadExt(USBD_CustomClass2_EP9_Thread);
osThreadDef(USBD_CustomClass2_EP9_Thread,  USBD_CUSTOM_CLASS2_EP9_THREAD_PRIORITY,  1U, USBD_CUSTOM_CLASS2_EP9_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      10)
extern void USBD_CustomClass2_EP10_Thread  (void const *arg);
       void USBD_CustomClass2_EP10_Thread  (void const *arg) { USBD_CustomClass_EP_Thread((void *)2U); }
osThreadExt(USBD_CustomClass2_EP10_Thread);
osThreadDef(USBD_CustomClass2_EP10_Thread, USBD_CUSTOM_CLASS2_EP10_THREAD_PRIORITY, 1U, USBD_CUSTOM_CLASS2_EP10_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      11)
extern void USBD_CustomClass2_EP11_Thread  (void const *arg);
       void USBD_CustomClass2_EP11_Thread  (void const *arg) { USBD_CustomClass_EP_Thread((void *)2U); }
osThreadExt(USBD_CustomClass2_EP11_Thread);
osThreadDef(USBD_CustomClass2_EP11_Thread, USBD_CUSTOM_CLASS2_EP11_THREAD_PRIORITY, 1U, USBD_CUSTOM_CLASS2_EP11_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      12)
extern void USBD_CustomClass2_EP12_Thread  (void const *arg);
       void USBD_CustomClass2_EP12_Thread  (void const *arg) { USBD_CustomClass_EP_Thread((void *)2U); }
osThreadExt(USBD_CustomClass2_EP12_Thread);
osThreadDef(USBD_CustomClass2_EP12_Thread, USBD_CUSTOM_CLASS2_EP12_THREAD_PRIORITY, 1U, USBD_CUSTOM_CLASS2_EP12_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      13)
extern void USBD_CustomClass2_EP13_Thread  (void const *arg);
       void USBD_CustomClass2_EP13_Thread  (void const *arg) { USBD_CustomClass_EP_Thread((void *)2U); }
osThreadExt(USBD_CustomClass2_EP13_Thread);
osThreadDef(USBD_CustomClass2_EP13_Thread, USBD_CUSTOM_CLASS2_EP13_THREAD_PRIORITY, 1U, USBD_CUSTOM_CLASS2_EP13_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      14)
extern void USBD_CustomClass2_EP14_Thread  (void const *arg);
       void USBD_CustomClass2_EP14_Thread  (void const *arg) { USBD_CustomClass_EP_Thread((void *)2U); }
osThreadExt(USBD_CustomClass2_EP14_Thread);
osThreadDef(USBD_CustomClass2_EP14_Thread, USBD_CUSTOM_CLASS2_EP14_THREAD_PRIORITY, 1U, USBD_CUSTOM_CLASS2_EP14_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      15)
extern void USBD_CustomClass2_EP15_Thread  (void const *arg);
       void USBD_CustomClass2_EP15_Thread  (void const *arg) { USBD_CustomClass_EP_Thread((void *)2U); }
osThreadExt(USBD_CustomClass2_EP15_Thread);
osThreadDef(USBD_CustomClass2_EP15_Thread, USBD_CUSTOM_CLASS2_EP15_THREAD_PRIORITY, 1U, USBD_CUSTOM_CLASS2_EP15_THREAD_STACK_SIZE);
#endif
#endif

#ifdef RTE_USB_Device_CustomClass_3
#if   (USBD_EP_LAST_NUM >=      1)
       void USBD_CustomClass3_EP1_Thread   (void const *arg);
       void USBD_CustomClass3_EP1_Thread   (void const *arg) { USBD_CustomClass_EP_Thread((void *)3U); }
osThreadExt(USBD_CustomClass3_EP1_Thread);
osThreadDef(USBD_CustomClass3_EP1_Thread,  USBD_CUSTOM_CLASS3_EP1_THREAD_PRIORITY,  1U, USBD_CUSTOM_CLASS3_EP1_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      2)
extern void USBD_CustomClass3_EP2_Thread   (void const *arg);
       void USBD_CustomClass3_EP2_Thread   (void const *arg) { USBD_CustomClass_EP_Thread((void *)3U); }
osThreadExt(USBD_CustomClass3_EP2_Thread);
osThreadDef(USBD_CustomClass3_EP2_Thread,  USBD_CUSTOM_CLASS3_EP2_THREAD_PRIORITY,  1U, USBD_CUSTOM_CLASS3_EP2_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      3)
extern void USBD_CustomClass3_EP3_Thread   (void const *arg);
       void USBD_CustomClass3_EP3_Thread   (void const *arg) { USBD_CustomClass_EP_Thread((void *)3U); }
osThreadExt(USBD_CustomClass3_EP3_Thread);
osThreadDef(USBD_CustomClass3_EP3_Thread,  USBD_CUSTOM_CLASS3_EP3_THREAD_PRIORITY,  1U, USBD_CUSTOM_CLASS3_EP3_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      4)
extern void USBD_CustomClass3_EP4_Thread   (void const *arg);
       void USBD_CustomClass3_EP4_Thread   (void const *arg) { USBD_CustomClass_EP_Thread((void *)3U); }
osThreadExt(USBD_CustomClass3_EP4_Thread);
osThreadDef(USBD_CustomClass3_EP4_Thread,  USBD_CUSTOM_CLASS3_EP4_THREAD_PRIORITY,  1U, USBD_CUSTOM_CLASS3_EP4_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      5)
extern void USBD_CustomClass3_EP5_Thread   (void const *arg);
       void USBD_CustomClass3_EP5_Thread   (void const *arg) { USBD_CustomClass_EP_Thread((void *)3U); }
osThreadExt(USBD_CustomClass3_EP5_Thread);
osThreadDef(USBD_CustomClass3_EP5_Thread,  USBD_CUSTOM_CLASS3_EP5_THREAD_PRIORITY,  1U, USBD_CUSTOM_CLASS3_EP5_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      6)
extern void USBD_CustomClass3_EP6_Thread   (void const *arg);
       void USBD_CustomClass3_EP6_Thread   (void const *arg) { USBD_CustomClass_EP_Thread((void *)3U); }
osThreadExt(USBD_CustomClass3_EP6_Thread);
osThreadDef(USBD_CustomClass3_EP6_Thread,  USBD_CUSTOM_CLASS3_EP6_THREAD_PRIORITY,  1U, USBD_CUSTOM_CLASS3_EP6_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      7)
extern void USBD_CustomClass3_EP7_Thread   (void const *arg);
       void USBD_CustomClass3_EP7_Thread   (void const *arg) { USBD_CustomClass_EP_Thread((void *)3U); }
osThreadExt(USBD_CustomClass3_EP7_Thread);
osThreadDef(USBD_CustomClass3_EP7_Thread,  USBD_CUSTOM_CLASS3_EP7_THREAD_PRIORITY,  1U, USBD_CUSTOM_CLASS3_EP7_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      8)
extern void USBD_CustomClass3_EP8_Thread   (void const *arg);
       void USBD_CustomClass3_EP8_Thread   (void const *arg) { USBD_CustomClass_EP_Thread((void *)3U); }
osThreadExt(USBD_CustomClass3_EP8_Thread);
osThreadDef(USBD_CustomClass3_EP8_Thread,  USBD_CUSTOM_CLASS3_EP8_THREAD_PRIORITY,  1U, USBD_CUSTOM_CLASS3_EP8_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      9)
extern void USBD_CustomClass3_EP9_Thread   (void const *arg);
       void USBD_CustomClass3_EP9_Thread   (void const *arg) { USBD_CustomClass_EP_Thread((void *)3U); }
osThreadExt(USBD_CustomClass3_EP9_Thread);
osThreadDef(USBD_CustomClass3_EP9_Thread,  USBD_CUSTOM_CLASS3_EP9_THREAD_PRIORITY,  1U, USBD_CUSTOM_CLASS3_EP9_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      10)
extern void USBD_CustomClass3_EP10_Thread  (void const *arg);
       void USBD_CustomClass3_EP10_Thread  (void const *arg) { USBD_CustomClass_EP_Thread((void *)3U); }
osThreadExt(USBD_CustomClass3_EP10_Thread);
osThreadDef(USBD_CustomClass3_EP10_Thread, USBD_CUSTOM_CLASS3_EP10_THREAD_PRIORITY, 1U, USBD_CUSTOM_CLASS3_EP10_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      11)
extern void USBD_CustomClass3_EP11_Thread  (void const *arg);
       void USBD_CustomClass3_EP11_Thread  (void const *arg) { USBD_CustomClass_EP_Thread((void *)3U); }
osThreadExt(USBD_CustomClass3_EP11_Thread);
osThreadDef(USBD_CustomClass3_EP11_Thread, USBD_CUSTOM_CLASS3_EP11_THREAD_PRIORITY, 1U, USBD_CUSTOM_CLASS3_EP11_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      12)
extern void USBD_CustomClass3_EP12_Thread  (void const *arg);
       void USBD_CustomClass3_EP12_Thread  (void const *arg) { USBD_CustomClass_EP_Thread((void *)3U); }
osThreadExt(USBD_CustomClass3_EP12_Thread);
osThreadDef(USBD_CustomClass3_EP12_Thread, USBD_CUSTOM_CLASS3_EP12_THREAD_PRIORITY, 1U, USBD_CUSTOM_CLASS3_EP12_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      13)
extern void USBD_CustomClass3_EP13_Thread  (void const *arg);
       void USBD_CustomClass3_EP13_Thread  (void const *arg) { USBD_CustomClass_EP_Thread((void *)3U); }
osThreadExt(USBD_CustomClass3_EP13_Thread);
osThreadDef(USBD_CustomClass3_EP13_Thread, USBD_CUSTOM_CLASS3_EP13_THREAD_PRIORITY, 1U, USBD_CUSTOM_CLASS3_EP13_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      14)
extern void USBD_CustomClass3_EP14_Thread  (void const *arg);
       void USBD_CustomClass3_EP14_Thread  (void const *arg) { USBD_CustomClass_EP_Thread((void *)3U); }
osThreadExt(USBD_CustomClass3_EP14_Thread);
osThreadDef(USBD_CustomClass3_EP14_Thread, USBD_CUSTOM_CLASS3_EP14_THREAD_PRIORITY, 1U, USBD_CUSTOM_CLASS3_EP14_THREAD_STACK_SIZE);
#endif
#if   (USBD_EP_LAST_NUM >=      15)
extern void USBD_CustomClass3_EP15_Thread  (void const *arg);
       void USBD_CustomClass3_EP15_Thread  (void const *arg) { USBD_CustomClass_EP_Thread((void *)3U); }
osThreadExt(USBD_CustomClass3_EP15_Thread);
osThreadDef(USBD_CustomClass3_EP15_Thread, USBD_CUSTOM_CLASS3_EP15_THREAD_PRIORITY, 1U, USBD_CUSTOM_CLASS3_EP15_THREAD_STACK_SIZE);
#endif
#endif

static const osThreadDef_t * const usbd_custom_class_ep_thread_def_ptr[USBD_CUSTOM_CLASS_NUM * USBD_EP_NUM] = {
#ifdef RTE_USB_Device_CustomClass_0
    0U
#if   (USBD_EP_LAST_NUM >=       1)
  , osThread(USBD_CustomClass0_EP1_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       2)
  , osThread(USBD_CustomClass0_EP2_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       3)
  , osThread(USBD_CustomClass0_EP3_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       4)
  , osThread(USBD_CustomClass0_EP4_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       5)
  , osThread(USBD_CustomClass0_EP5_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       6)
  , osThread(USBD_CustomClass0_EP6_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       7)
  , osThread(USBD_CustomClass0_EP7_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       8)
  , osThread(USBD_CustomClass0_EP8_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       9)
  , osThread(USBD_CustomClass0_EP9_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       10)
  , osThread(USBD_CustomClass0_EP10_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       11)
  , osThread(USBD_CustomClass0_EP11_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       12)
  , osThread(USBD_CustomClass0_EP12_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       13)
  , osThread(USBD_CustomClass0_EP13_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       14)
  , osThread(USBD_CustomClass0_EP14_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       15)
  , osThread(USBD_CustomClass0_EP15_Thread)
#endif
#endif
#ifdef RTE_USB_Device_CustomClass_1
  , 0U
#if   (USBD_EP_LAST_NUM >=       1)
  , osThread(USBD_CustomClass1_EP1_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       2)
  , osThread(USBD_CustomClass1_EP2_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       3)
  , osThread(USBD_CustomClass1_EP3_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       4)
  , osThread(USBD_CustomClass1_EP4_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       5)
  , osThread(USBD_CustomClass1_EP5_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       6)
  , osThread(USBD_CustomClass1_EP6_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       7)
  , osThread(USBD_CustomClass1_EP7_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       8)
  , osThread(USBD_CustomClass1_EP8_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       9)
  , osThread(USBD_CustomClass1_EP9_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       10)
  , osThread(USBD_CustomClass1_EP10_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       11)
  , osThread(USBD_CustomClass1_EP11_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       12)
  , osThread(USBD_CustomClass1_EP12_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       13)
  , osThread(USBD_CustomClass1_EP13_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       14)
  , osThread(USBD_CustomClass1_EP14_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       15)
  , osThread(USBD_CustomClass1_EP15_Thread)
#endif
#endif
#ifdef RTE_USB_Device_CustomClass_2
  , 0U
#if   (USBD_EP_LAST_NUM >=       1)
  , osThread(USBD_CustomClass2_EP1_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       2)
  , osThread(USBD_CustomClass2_EP2_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       3)
  , osThread(USBD_CustomClass2_EP3_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       4)
  , osThread(USBD_CustomClass2_EP4_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       5)
  , osThread(USBD_CustomClass2_EP5_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       6)
  , osThread(USBD_CustomClass2_EP6_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       7)
  , osThread(USBD_CustomClass2_EP7_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       8)
  , osThread(USBD_CustomClass2_EP8_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       9)
  , osThread(USBD_CustomClass2_EP9_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       10)
  , osThread(USBD_CustomClass2_EP10_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       11)
  , osThread(USBD_CustomClass2_EP11_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       12)
  , osThread(USBD_CustomClass2_EP12_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       13)
  , osThread(USBD_CustomClass2_EP13_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       14)
  , osThread(USBD_CustomClass2_EP14_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       15)
  , osThread(USBD_CustomClass2_EP15_Thread)
#endif
#endif
#ifdef RTE_USB_Device_CustomClass_3
  , 0U
#if   (USBD_EP_LAST_NUM >=       1)
  , osThread(USBD_CustomClass3_EP1_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       2)
  , osThread(USBD_CustomClass3_EP2_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       3)
  , osThread(USBD_CustomClass3_EP3_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       4)
  , osThread(USBD_CustomClass3_EP4_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       5)
  , osThread(USBD_CustomClass3_EP5_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       6)
  , osThread(USBD_CustomClass3_EP6_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       7)
  , osThread(USBD_CustomClass3_EP7_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       8)
  , osThread(USBD_CustomClass3_EP8_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       9)
  , osThread(USBD_CustomClass3_EP9_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       10)
  , osThread(USBD_CustomClass3_EP10_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       11)
  , osThread(USBD_CustomClass3_EP11_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       12)
  , osThread(USBD_CustomClass3_EP12_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       13)
  , osThread(USBD_CustomClass3_EP13_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       14)
  , osThread(USBD_CustomClass3_EP14_Thread)
#endif
#if   (USBD_EP_LAST_NUM >=       15)
  , osThread(USBD_CustomClass3_EP15_Thread)
#endif
#endif
};
#endif

#if   (USBD_ADC_NUM > 0)
#ifdef RTE_USB_Device_ADC_0
osThreadExt(USBD_ADC0_Thread);
osThreadDef(USBD_ADC0_Thread, USBD_ADC0_THREAD_PRIORITY, 1U, USBD_ADC0_THREAD_STACK_SIZE);
#endif
#ifdef RTE_USB_Device_ADC_1
osThreadExt(USBD_ADC1_Thread);
osThreadDef(USBD_ADC1_Thread, USBD_ADC1_THREAD_PRIORITY, 1U, USBD_ADC1_THREAD_STACK_SIZE);
#endif
#ifdef RTE_USB_Device_ADC_2
osThreadExt(USBD_ADC2_Thread);
osThreadDef(USBD_ADC2_Thread, USBD_ADC2_THREAD_PRIORITY, 1U, USBD_ADC2_THREAD_STACK_SIZE);
#endif
#ifdef RTE_USB_Device_ADC_3
osThreadExt(USBD_ADC3_Thread);
osThreadDef(USBD_ADC3_Thread, USBD_ADC3_THREAD_PRIORITY, 1U, USBD_ADC3_THREAD_STACK_SIZE);
#endif

static const osThreadDef_t * const usbd_adc_thread_def_ptr[USBD_ADC_NUM] = {
#ifdef RTE_USB_Device_ADC_0
    osThread(USBD_ADC0_Thread)
#endif
#ifdef RTE_USB_Device_ADC_1
  , osThread(USBD_ADC1_Thread)
#endif
#ifdef RTE_USB_Device_ADC_2
  , osThread(USBD_ADC2_Thread)
#endif
#ifdef RTE_USB_Device_ADC_3
  , osThread(USBD_ADC3_Thread)
#endif
};
#endif

#if   (USBD_CDC_NUM > 0)
#ifdef RTE_USB_Device_CDC_0
osThreadExt(USBD_CDC0_Int_Thread);
osThreadDef(USBD_CDC0_Int_Thread,  USBD_CDC0_INT_THREAD_PRIORITY,  1U, USBD_CDC0_INT_THREAD_STACK_SIZE);
osThreadExt(USBD_CDC0_Bulk_Thread);
osThreadDef(USBD_CDC0_Bulk_Thread, USBD_CDC0_BULK_THREAD_PRIORITY, 1U, USBD_CDC0_BULK_THREAD_STACK_SIZE);
#endif
#ifdef RTE_USB_Device_CDC_1
osThreadExt(USBD_CDC1_Int_Thread);
osThreadDef(USBD_CDC1_Int_Thread,  USBD_CDC1_INT_THREAD_PRIORITY,  1U, USBD_CDC1_INT_THREAD_STACK_SIZE);
osThreadExt(USBD_CDC1_Bulk_Thread);
osThreadDef(USBD_CDC1_Bulk_Thread, USBD_CDC1_BULK_THREAD_PRIORITY, 1U, USBD_CDC1_BULK_THREAD_STACK_SIZE);
#endif
#ifdef RTE_USB_Device_CDC_2
osThreadExt(USBD_CDC2_Int_Thread);
osThreadDef(USBD_CDC2_Int_Thread,  USBD_CDC2_INT_THREAD_PRIORITY,  1U, USBD_CDC2_INT_THREAD_STACK_SIZE);
osThreadExt(USBD_CDC2_Bulk_Thread);
osThreadDef(USBD_CDC2_Bulk_Thread, USBD_CDC2_BULK_THREAD_PRIORITY, 1U, USBD_CDC2_BULK_THREAD_STACK_SIZE);
#endif
#ifdef RTE_USB_Device_CDC_3
osThreadExt(USBD_CDC3_Int_Thread);
osThreadDef(USBD_CDC3_Int_Thread,  USBD_CDC3_INT_THREAD_PRIORITY,  1U, USBD_CDC3_INT_THREAD_STACK_SIZE);
osThreadExt(USBD_CDC3_Bulk_Thread);
osThreadDef(USBD_CDC3_Bulk_Thread, USBD_CDC3_BULK_THREAD_PRIORITY, 1U, USBD_CDC3_BULK_THREAD_STACK_SIZE);
#endif

static const osThreadDef_t * const usbd_cdc_int_thread_def_ptr[USBD_CDC_NUM] = {
#ifdef  RTE_USB_Device_CDC_0
    osThread(USBD_CDC0_Int_Thread)
#endif
#ifdef  RTE_USB_Device_CDC_1
  , osThread(USBD_CDC1_Int_Thread)
#endif
#ifdef  RTE_USB_Device_CDC_2
  , osThread(USBD_CDC2_Int_Thread)
#endif
#ifdef  RTE_USB_Device_CDC_3
  , osThread(USBD_CDC3_Int_Thread)
#endif
};

static const osThreadDef_t * const usbd_cdc_bulk_thread_def_ptr[USBD_CDC_NUM] = {
#ifdef  RTE_USB_Device_CDC_0
    osThread(USBD_CDC0_Bulk_Thread)
#endif
#ifdef  RTE_USB_Device_CDC_1
  , osThread(USBD_CDC1_Bulk_Thread)
#endif
#ifdef  RTE_USB_Device_CDC_2
  , osThread(USBD_CDC2_Bulk_Thread)
#endif
#ifdef  RTE_USB_Device_CDC_3
  , osThread(USBD_CDC3_Bulk_Thread)
#endif
};
#endif

#if   (USBD_HID_NUM > 0)
#ifdef RTE_USB_Device_HID_0
osThreadExt(USBD_HID0_Thread);
osThreadDef(USBD_HID0_Thread, USBD_HID0_THREAD_PRIORITY, 1U, USBD_HID0_THREAD_STACK_SIZE);
#endif
#ifdef RTE_USB_Device_HID_1
osThreadExt(USBD_HID1_Thread);
osThreadDef(USBD_HID1_Thread, USBD_HID1_THREAD_PRIORITY, 1U, USBD_HID1_THREAD_STACK_SIZE);
#endif
#ifdef RTE_USB_Device_HID_2
osThreadExt(USBD_HID2_Thread);
osThreadDef(USBD_HID2_Thread, USBD_HID2_THREAD_PRIORITY, 1U, USBD_HID2_THREAD_STACK_SIZE);
#endif
#ifdef RTE_USB_Device_HID_3
osThreadExt(USBD_HID3_Thread);
osThreadDef(USBD_HID3_Thread, USBD_HID3_THREAD_PRIORITY, 1U, USBD_HID3_THREAD_STACK_SIZE);
#endif

static const osThreadDef_t * const usbd_hid_thread_def_ptr[USBD_HID_NUM] = {
#ifdef  RTE_USB_Device_HID_0
    osThread(USBD_HID0_Thread)
#endif
#ifdef  RTE_USB_Device_HID_1
  , osThread(USBD_HID1_Thread)
#endif
#ifdef  RTE_USB_Device_HID_2
  , osThread(USBD_HID2_Thread)
#endif
#ifdef  RTE_USB_Device_HID_3
  , osThread(USBD_HID3_Thread)
#endif
};
#endif

#if   (USBD_MSC_NUM > 0)
#ifdef RTE_USB_Device_MSC_0
osThreadExt(USBD_MSC0_Thread);
osThreadDef(USBD_MSC0_Thread, USBD_MSC0_THREAD_PRIORITY, 1U, USBD_MSC0_THREAD_STACK_SIZE);
#endif
#ifdef RTE_USB_Device_MSC_1
osThreadExt(USBD_MSC1_Thread);
osThreadDef(USBD_MSC1_Thread, USBD_MSC1_THREAD_PRIORITY, 1U, USBD_MSC1_THREAD_STACK_SIZE);
#endif
#ifdef RTE_USB_Device_MSC_2
osThreadExt(USBD_MSC2_Thread);
osThreadDef(USBD_MSC2_Thread, USBD_MSC2_THREAD_PRIORITY, 1U, USBD_MSC2_THREAD_STACK_SIZE);
#endif
#ifdef RTE_USB_Device_MSC_3
osThreadExt(USBD_MSC3_Thread);
osThreadDef(USBD_MSC3_Thread, USBD_MSC3_THREAD_PRIORITY, 1U, USBD_MSC3_THREAD_STACK_SIZE);
#endif

static const osThreadDef_t * const usbd_msc_thread_def_ptr[USBD_MSC_NUM] = {
#ifdef  RTE_USB_Device_MSC_0
    osThread(USBD_MSC0_Thread)
#endif
#ifdef  RTE_USB_Device_MSC_1
  , osThread(USBD_MSC1_Thread)
#endif
#ifdef  RTE_USB_Device_MSC_2
  , osThread(USBD_MSC2_Thread)
#endif
#ifdef  RTE_USB_Device_MSC_3
  , osThread(USBD_MSC3_Thread)
#endif
};
#endif

// Create timer definitions
#if   (USBD_HID_NUM > 0)
#ifdef RTE_USB_Device_HID_0
osTimerExt(usbd_hid0_timer);
osTimerDef(usbd_hid0_timer, USBD_HID_Timer);
#endif
#ifdef RTE_USB_Device_HID_1
osTimerExt(usbd_hid1_timer);
osTimerDef(usbd_hid1_timer, USBD_HID_Timer);
#endif
#ifdef RTE_USB_Device_HID_2
osTimerExt(usbd_hid2_timer);
osTimerDef(usbd_hid2_timer, USBD_HID_Timer);
#endif
#ifdef RTE_USB_Device_HID_3
osTimerExt(usbd_hid3_timer);
osTimerDef(usbd_hid3_timer, USBD_HID_Timer);
#endif

static const osTimerDef_t * const usbd_hid_timer_def_ptr[USBD_HID_NUM] = {
#ifdef RTE_USB_Device_HID_0
    osTimer(usbd_hid0_timer)
#endif
#ifdef RTE_USB_Device_HID_1
  , osTimer(usbd_hid1_timer)
#endif
#ifdef RTE_USB_Device_HID_2
  , osTimer(usbd_hid2_timer)
#endif
#ifdef RTE_USB_Device_HID_3
  , osTimer(usbd_hid3_timer)
#endif
};
#endif


// Create semaphore definitions
#ifdef RTE_USB_Device_0
osSemaphoreExt(usbd0_driver_semaphore);
osSemaphoreDef(usbd0_driver_semaphore);
#if   (USBD_EP_LAST_NUM >=    0)
osSemaphoreExt(usbd0_driver_ep0out_semaphore);
osSemaphoreDef(usbd0_driver_ep0out_semaphore);
osSemaphoreExt(usbd0_driver_ep0in_semaphore );
osSemaphoreDef(usbd0_driver_ep0in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    1)
osSemaphoreExt(usbd0_driver_ep1out_semaphore);
osSemaphoreDef(usbd0_driver_ep1out_semaphore);
osSemaphoreExt(usbd0_driver_ep1in_semaphore );
osSemaphoreDef(usbd0_driver_ep1in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    2)
osSemaphoreExt(usbd0_driver_ep2out_semaphore);
osSemaphoreDef(usbd0_driver_ep2out_semaphore);
osSemaphoreExt(usbd0_driver_ep2in_semaphore );
osSemaphoreDef(usbd0_driver_ep2in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    3)
osSemaphoreExt(usbd0_driver_ep3out_semaphore);
osSemaphoreDef(usbd0_driver_ep3out_semaphore);
osSemaphoreExt(usbd0_driver_ep3in_semaphore );
osSemaphoreDef(usbd0_driver_ep3in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    4)
osSemaphoreExt(usbd0_driver_ep4out_semaphore);
osSemaphoreDef(usbd0_driver_ep4out_semaphore);
osSemaphoreExt(usbd0_driver_ep4in_semaphore );
osSemaphoreDef(usbd0_driver_ep4in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    5)
osSemaphoreExt(usbd0_driver_ep5out_semaphore);
osSemaphoreDef(usbd0_driver_ep5out_semaphore);
osSemaphoreExt(usbd0_driver_ep5in_semaphore );
osSemaphoreDef(usbd0_driver_ep5in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    6)
osSemaphoreExt(usbd0_driver_ep6out_semaphore);
osSemaphoreDef(usbd0_driver_ep6out_semaphore);
osSemaphoreExt(usbd0_driver_ep6in_semaphore );
osSemaphoreDef(usbd0_driver_ep6in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    7)
osSemaphoreExt(usbd0_driver_ep7out_semaphore);
osSemaphoreDef(usbd0_driver_ep7out_semaphore);
osSemaphoreExt(usbd0_driver_ep7in_semaphore );
osSemaphoreDef(usbd0_driver_ep7in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    8)
osSemaphoreExt(usbd0_driver_ep8out_semaphore);
osSemaphoreDef(usbd0_driver_ep8out_semaphore);
osSemaphoreExt(usbd0_driver_ep8in_semaphore );
osSemaphoreDef(usbd0_driver_ep8in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    9)
osSemaphoreExt(usbd0_driver_ep9out_semaphore);
osSemaphoreDef(usbd0_driver_ep9out_semaphore);
osSemaphoreExt(usbd0_driver_ep9in_semaphore );
osSemaphoreDef(usbd0_driver_ep9in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    10)
osSemaphoreExt(usbd0_driver_ep10out_semaphore);
osSemaphoreDef(usbd0_driver_ep10out_semaphore);
osSemaphoreExt(usbd0_driver_ep10in_semaphore );
osSemaphoreDef(usbd0_driver_ep10in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    11)
osSemaphoreExt(usbd0_driver_ep11out_semaphore);
osSemaphoreDef(usbd0_driver_ep11out_semaphore);
osSemaphoreExt(usbd0_driver_ep11in_semaphore );
osSemaphoreDef(usbd0_driver_ep11in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    12)
osSemaphoreExt(usbd0_driver_ep12out_semaphore);
osSemaphoreDef(usbd0_driver_ep12out_semaphore);
osSemaphoreExt(usbd0_driver_ep12in_semaphore );
osSemaphoreDef(usbd0_driver_ep12in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    13)
osSemaphoreExt(usbd0_driver_ep13out_semaphore);
osSemaphoreDef(usbd0_driver_ep13out_semaphore);
osSemaphoreExt(usbd0_driver_ep13in_semaphore );
osSemaphoreDef(usbd0_driver_ep13in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    14)
osSemaphoreExt(usbd0_driver_ep14out_semaphore);
osSemaphoreDef(usbd0_driver_ep14out_semaphore);
osSemaphoreExt(usbd0_driver_ep14in_semaphore );
osSemaphoreDef(usbd0_driver_ep14in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    15)
osSemaphoreExt(usbd0_driver_ep15out_semaphore);
osSemaphoreDef(usbd0_driver_ep15out_semaphore);
osSemaphoreExt(usbd0_driver_ep15in_semaphore );
osSemaphoreDef(usbd0_driver_ep15in_semaphore );
#endif
#endif
#ifdef RTE_USB_Device_1
osSemaphoreExt(usbd1_driver_semaphore);
osSemaphoreDef(usbd1_driver_semaphore);
#if   (USBD_EP_LAST_NUM >=    0)
osSemaphoreExt(usbd1_driver_ep0out_semaphore);
osSemaphoreDef(usbd1_driver_ep0out_semaphore);
osSemaphoreExt(usbd1_driver_ep0in_semaphore );
osSemaphoreDef(usbd1_driver_ep0in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    1)
osSemaphoreExt(usbd1_driver_ep1out_semaphore);
osSemaphoreDef(usbd1_driver_ep1out_semaphore);
osSemaphoreExt(usbd1_driver_ep1in_semaphore );
osSemaphoreDef(usbd1_driver_ep1in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    2)
osSemaphoreExt(usbd1_driver_ep2out_semaphore);
osSemaphoreDef(usbd1_driver_ep2out_semaphore);
osSemaphoreExt(usbd1_driver_ep2in_semaphore );
osSemaphoreDef(usbd1_driver_ep2in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    3)
osSemaphoreExt(usbd1_driver_ep3out_semaphore);
osSemaphoreDef(usbd1_driver_ep3out_semaphore);
osSemaphoreExt(usbd1_driver_ep3in_semaphore );
osSemaphoreDef(usbd1_driver_ep3in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    4)
osSemaphoreExt(usbd1_driver_ep4out_semaphore);
osSemaphoreDef(usbd1_driver_ep4out_semaphore);
osSemaphoreExt(usbd1_driver_ep4in_semaphore );
osSemaphoreDef(usbd1_driver_ep4in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    5)
osSemaphoreExt(usbd1_driver_ep5out_semaphore);
osSemaphoreDef(usbd1_driver_ep5out_semaphore);
osSemaphoreExt(usbd1_driver_ep5in_semaphore );
osSemaphoreDef(usbd1_driver_ep5in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    6)
osSemaphoreExt(usbd1_driver_ep6out_semaphore);
osSemaphoreDef(usbd1_driver_ep6out_semaphore);
osSemaphoreExt(usbd1_driver_ep6in_semaphore );
osSemaphoreDef(usbd1_driver_ep6in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    7)
osSemaphoreExt(usbd1_driver_ep7out_semaphore);
osSemaphoreDef(usbd1_driver_ep7out_semaphore);
osSemaphoreExt(usbd1_driver_ep7in_semaphore );
osSemaphoreDef(usbd1_driver_ep7in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    8)
osSemaphoreExt(usbd1_driver_ep8out_semaphore);
osSemaphoreDef(usbd1_driver_ep8out_semaphore);
osSemaphoreExt(usbd1_driver_ep8in_semaphore );
osSemaphoreDef(usbd1_driver_ep8in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    9)
osSemaphoreExt(usbd1_driver_ep9out_semaphore);
osSemaphoreDef(usbd1_driver_ep9out_semaphore);
osSemaphoreExt(usbd1_driver_ep9in_semaphore );
osSemaphoreDef(usbd1_driver_ep9in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    10)
osSemaphoreExt(usbd1_driver_ep10out_semaphore);
osSemaphoreDef(usbd1_driver_ep10out_semaphore);
osSemaphoreExt(usbd1_driver_ep10in_semaphore );
osSemaphoreDef(usbd1_driver_ep10in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    11)
osSemaphoreExt(usbd1_driver_ep11out_semaphore);
osSemaphoreDef(usbd1_driver_ep11out_semaphore);
osSemaphoreExt(usbd1_driver_ep11in_semaphore );
osSemaphoreDef(usbd1_driver_ep11in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    12)
osSemaphoreExt(usbd1_driver_ep12out_semaphore);
osSemaphoreDef(usbd1_driver_ep12out_semaphore);
osSemaphoreExt(usbd1_driver_ep12in_semaphore );
osSemaphoreDef(usbd1_driver_ep12in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    13)
osSemaphoreExt(usbd1_driver_ep13out_semaphore);
osSemaphoreDef(usbd1_driver_ep13out_semaphore);
osSemaphoreExt(usbd1_driver_ep13in_semaphore );
osSemaphoreDef(usbd1_driver_ep13in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    14)
osSemaphoreExt(usbd1_driver_ep14out_semaphore);
osSemaphoreDef(usbd1_driver_ep14out_semaphore);
osSemaphoreExt(usbd1_driver_ep14in_semaphore );
osSemaphoreDef(usbd1_driver_ep14in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    15)
osSemaphoreExt(usbd1_driver_ep15out_semaphore);
osSemaphoreDef(usbd1_driver_ep15out_semaphore);
osSemaphoreExt(usbd1_driver_ep15in_semaphore );
osSemaphoreDef(usbd1_driver_ep15in_semaphore );
#endif
#endif
#ifdef RTE_USB_Device_2
osSemaphoreExt(usbd2_driver_semaphore);
osSemaphoreDef(usbd2_driver_semaphore);
#if   (USBD_EP_LAST_NUM >=    0)
osSemaphoreExt(usbd2_driver_ep0out_semaphore);
osSemaphoreDef(usbd2_driver_ep0out_semaphore);
osSemaphoreExt(usbd2_driver_ep0in_semaphore );
osSemaphoreDef(usbd2_driver_ep0in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    1)
osSemaphoreExt(usbd2_driver_ep1out_semaphore);
osSemaphoreDef(usbd2_driver_ep1out_semaphore);
osSemaphoreExt(usbd2_driver_ep1in_semaphore );
osSemaphoreDef(usbd2_driver_ep1in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    2)
osSemaphoreExt(usbd2_driver_ep2out_semaphore);
osSemaphoreDef(usbd2_driver_ep2out_semaphore);
osSemaphoreExt(usbd2_driver_ep2in_semaphore );
osSemaphoreDef(usbd2_driver_ep2in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    3)
osSemaphoreExt(usbd2_driver_ep3out_semaphore);
osSemaphoreDef(usbd2_driver_ep3out_semaphore);
osSemaphoreExt(usbd2_driver_ep3in_semaphore );
osSemaphoreDef(usbd2_driver_ep3in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    4)
osSemaphoreExt(usbd2_driver_ep4out_semaphore);
osSemaphoreDef(usbd2_driver_ep4out_semaphore);
osSemaphoreExt(usbd2_driver_ep4in_semaphore );
osSemaphoreDef(usbd2_driver_ep4in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    5)
osSemaphoreExt(usbd2_driver_ep5out_semaphore);
osSemaphoreDef(usbd2_driver_ep5out_semaphore);
osSemaphoreExt(usbd2_driver_ep5in_semaphore );
osSemaphoreDef(usbd2_driver_ep5in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    6)
osSemaphoreExt(usbd2_driver_ep6out_semaphore);
osSemaphoreDef(usbd2_driver_ep6out_semaphore);
osSemaphoreExt(usbd2_driver_ep6in_semaphore );
osSemaphoreDef(usbd2_driver_ep6in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    7)
osSemaphoreExt(usbd2_driver_ep7out_semaphore);
osSemaphoreDef(usbd2_driver_ep7out_semaphore);
osSemaphoreExt(usbd2_driver_ep7in_semaphore );
osSemaphoreDef(usbd2_driver_ep7in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    8)
osSemaphoreExt(usbd2_driver_ep8out_semaphore);
osSemaphoreDef(usbd2_driver_ep8out_semaphore);
osSemaphoreExt(usbd2_driver_ep8in_semaphore );
osSemaphoreDef(usbd2_driver_ep8in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    9)
osSemaphoreExt(usbd2_driver_ep9out_semaphore);
osSemaphoreDef(usbd2_driver_ep9out_semaphore);
osSemaphoreExt(usbd2_driver_ep9in_semaphore );
osSemaphoreDef(usbd2_driver_ep9in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    10)
osSemaphoreExt(usbd2_driver_ep10out_semaphore);
osSemaphoreDef(usbd2_driver_ep10out_semaphore);
osSemaphoreExt(usbd2_driver_ep10in_semaphore );
osSemaphoreDef(usbd2_driver_ep10in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    11)
osSemaphoreExt(usbd2_driver_ep11out_semaphore);
osSemaphoreDef(usbd2_driver_ep11out_semaphore);
osSemaphoreExt(usbd2_driver_ep11in_semaphore );
osSemaphoreDef(usbd2_driver_ep11in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    12)
osSemaphoreExt(usbd2_driver_ep12out_semaphore);
osSemaphoreDef(usbd2_driver_ep12out_semaphore);
osSemaphoreExt(usbd2_driver_ep12in_semaphore );
osSemaphoreDef(usbd2_driver_ep12in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    13)
osSemaphoreExt(usbd2_driver_ep13out_semaphore);
osSemaphoreDef(usbd2_driver_ep13out_semaphore);
osSemaphoreExt(usbd2_driver_ep13in_semaphore );
osSemaphoreDef(usbd2_driver_ep13in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    14)
osSemaphoreExt(usbd2_driver_ep14out_semaphore);
osSemaphoreDef(usbd2_driver_ep14out_semaphore);
osSemaphoreExt(usbd2_driver_ep14in_semaphore );
osSemaphoreDef(usbd2_driver_ep14in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    15)
osSemaphoreExt(usbd2_driver_ep15out_semaphore);
osSemaphoreDef(usbd2_driver_ep15out_semaphore);
osSemaphoreExt(usbd2_driver_ep15in_semaphore );
osSemaphoreDef(usbd2_driver_ep15in_semaphore );
#endif
#endif
#ifdef RTE_USB_Device_3
osSemaphoreExt(usbd3_driver_semaphore);
osSemaphoreDef(usbd3_driver_semaphore);
#if   (USBD_EP_LAST_NUM >=    0)
osSemaphoreExt(usbd3_driver_ep0out_semaphore);
osSemaphoreDef(usbd3_driver_ep0out_semaphore);
osSemaphoreExt(usbd3_driver_ep0in_semaphore );
osSemaphoreDef(usbd3_driver_ep0in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    1)
osSemaphoreExt(usbd3_driver_ep1out_semaphore);
osSemaphoreDef(usbd3_driver_ep1out_semaphore);
osSemaphoreExt(usbd3_driver_ep1in_semaphore );
osSemaphoreDef(usbd3_driver_ep1in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    2)
osSemaphoreExt(usbd3_driver_ep2out_semaphore);
osSemaphoreDef(usbd3_driver_ep2out_semaphore);
osSemaphoreExt(usbd3_driver_ep2in_semaphore );
osSemaphoreDef(usbd3_driver_ep2in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    3)
osSemaphoreExt(usbd3_driver_ep3out_semaphore);
osSemaphoreDef(usbd3_driver_ep3out_semaphore);
osSemaphoreExt(usbd3_driver_ep3in_semaphore );
osSemaphoreDef(usbd3_driver_ep3in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    4)
osSemaphoreExt(usbd3_driver_ep4out_semaphore);
osSemaphoreDef(usbd3_driver_ep4out_semaphore);
osSemaphoreExt(usbd3_driver_ep4in_semaphore );
osSemaphoreDef(usbd3_driver_ep4in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    5)
osSemaphoreExt(usbd3_driver_ep5out_semaphore);
osSemaphoreDef(usbd3_driver_ep5out_semaphore);
osSemaphoreExt(usbd3_driver_ep5in_semaphore );
osSemaphoreDef(usbd3_driver_ep5in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    6)
osSemaphoreExt(usbd3_driver_ep6out_semaphore);
osSemaphoreDef(usbd3_driver_ep6out_semaphore);
osSemaphoreExt(usbd3_driver_ep6in_semaphore );
osSemaphoreDef(usbd3_driver_ep6in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    7)
osSemaphoreExt(usbd3_driver_ep7out_semaphore);
osSemaphoreDef(usbd3_driver_ep7out_semaphore);
osSemaphoreExt(usbd3_driver_ep7in_semaphore );
osSemaphoreDef(usbd3_driver_ep7in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    8)
osSemaphoreExt(usbd3_driver_ep8out_semaphore);
osSemaphoreDef(usbd3_driver_ep8out_semaphore);
osSemaphoreExt(usbd3_driver_ep8in_semaphore );
osSemaphoreDef(usbd3_driver_ep8in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    9)
osSemaphoreExt(usbd3_driver_ep9out_semaphore);
osSemaphoreDef(usbd3_driver_ep9out_semaphore);
osSemaphoreExt(usbd3_driver_ep9in_semaphore );
osSemaphoreDef(usbd3_driver_ep9in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    10)
osSemaphoreExt(usbd3_driver_ep10out_semaphore);
osSemaphoreDef(usbd3_driver_ep10out_semaphore);
osSemaphoreExt(usbd3_driver_ep10in_semaphore );
osSemaphoreDef(usbd3_driver_ep10in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    11)
osSemaphoreExt(usbd3_driver_ep11out_semaphore);
osSemaphoreDef(usbd3_driver_ep11out_semaphore);
osSemaphoreExt(usbd3_driver_ep11in_semaphore );
osSemaphoreDef(usbd3_driver_ep11in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    12)
osSemaphoreExt(usbd3_driver_ep12out_semaphore);
osSemaphoreDef(usbd3_driver_ep12out_semaphore);
osSemaphoreExt(usbd3_driver_ep12in_semaphore );
osSemaphoreDef(usbd3_driver_ep12in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    13)
osSemaphoreExt(usbd3_driver_ep13out_semaphore);
osSemaphoreDef(usbd3_driver_ep13out_semaphore);
osSemaphoreExt(usbd3_driver_ep13in_semaphore );
osSemaphoreDef(usbd3_driver_ep13in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    14)
osSemaphoreExt(usbd3_driver_ep14out_semaphore);
osSemaphoreDef(usbd3_driver_ep14out_semaphore);
osSemaphoreExt(usbd3_driver_ep14in_semaphore );
osSemaphoreDef(usbd3_driver_ep14in_semaphore );
#endif
#if   (USBD_EP_LAST_NUM >=    15)
osSemaphoreExt(usbd3_driver_ep15out_semaphore);
osSemaphoreDef(usbd3_driver_ep15out_semaphore);
osSemaphoreExt(usbd3_driver_ep15in_semaphore );
osSemaphoreDef(usbd3_driver_ep15in_semaphore );
#endif
#endif

static const osSemaphoreDef_t * const usbd_driver_semaphore_def_ptr[USBD_DEV_NUM] = {
#ifdef RTE_USB_Device_0
    osSemaphore(usbd0_driver_semaphore)
#endif
#ifdef RTE_USB_Device_1
  , osSemaphore(usbd1_driver_semaphore)
#endif
#ifdef RTE_USB_Device_2
  , osSemaphore(usbd2_driver_semaphore)
#endif
#ifdef RTE_USB_Device_3
  , osSemaphore(usbd3_driver_semaphore)
#endif
};

static const osSemaphoreDef_t * const usbd_driver_ep_semaphore_def_ptr[USBD_DEV_NUM][USBD_EP_NUM * 2U] = {
#ifdef RTE_USB_Device_0
    {
#if   (USBD_EP_LAST_NUM >=       0)
      osSemaphore(usbd0_driver_ep0out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       1)
    , osSemaphore(usbd0_driver_ep1out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       2)
    , osSemaphore(usbd0_driver_ep2out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       3)
    , osSemaphore(usbd0_driver_ep3out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       4)
    , osSemaphore(usbd0_driver_ep4out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       5)
    , osSemaphore(usbd0_driver_ep5out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       6)
    , osSemaphore(usbd0_driver_ep6out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       7)
    , osSemaphore(usbd0_driver_ep7out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       8)
    , osSemaphore(usbd0_driver_ep8out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       9)
    , osSemaphore(usbd0_driver_ep9out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       10)
    , osSemaphore(usbd0_driver_ep10out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       11)
    , osSemaphore(usbd0_driver_ep11out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       12)
    , osSemaphore(usbd0_driver_ep12out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       13)
    , osSemaphore(usbd0_driver_ep13out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       14)
    , osSemaphore(usbd0_driver_ep14out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       15)
    , osSemaphore(usbd0_driver_ep15out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       0)
    , osSemaphore(usbd0_driver_ep0in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       1)
    , osSemaphore(usbd0_driver_ep1in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       2)
    , osSemaphore(usbd0_driver_ep2in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       3)
    , osSemaphore(usbd0_driver_ep3in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       4)
    , osSemaphore(usbd0_driver_ep4in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       5)
    , osSemaphore(usbd0_driver_ep5in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       6)
    , osSemaphore(usbd0_driver_ep6in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       7)
    , osSemaphore(usbd0_driver_ep7in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       8)
    , osSemaphore(usbd0_driver_ep8in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       9)
    , osSemaphore(usbd0_driver_ep9in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       10)
    , osSemaphore(usbd0_driver_ep10in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       11)
    , osSemaphore(usbd0_driver_ep11in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       12)
    , osSemaphore(usbd0_driver_ep12in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       13)
    , osSemaphore(usbd0_driver_ep13in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       14)
    , osSemaphore(usbd0_driver_ep14in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       15)
    , osSemaphore(usbd0_driver_ep15in_semaphore)
#endif
    }
#endif
#ifdef  RTE_USB_Device_1
  , {
#if   (USBD_EP_LAST_NUM >=       0)
      osSemaphore(usbd1_driver_ep0out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       1)
    , osSemaphore(usbd1_driver_ep1out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       2)
    , osSemaphore(usbd1_driver_ep2out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       3)
    , osSemaphore(usbd1_driver_ep3out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       4)
    , osSemaphore(usbd1_driver_ep4out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       5)
    , osSemaphore(usbd1_driver_ep5out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       6)
    , osSemaphore(usbd1_driver_ep6out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       7)
    , osSemaphore(usbd1_driver_ep7out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       8)
    , osSemaphore(usbd1_driver_ep8out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       9)
    , osSemaphore(usbd1_driver_ep9out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       10)
    , osSemaphore(usbd1_driver_ep10out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       11)
    , osSemaphore(usbd1_driver_ep11out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       12)
    , osSemaphore(usbd1_driver_ep12out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       13)
    , osSemaphore(usbd1_driver_ep13out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       14)
    , osSemaphore(usbd1_driver_ep14out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       15)
    , osSemaphore(usbd1_driver_ep15out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       0)
    , osSemaphore(usbd1_driver_ep0in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       1)
    , osSemaphore(usbd1_driver_ep1in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       2)
    , osSemaphore(usbd1_driver_ep2in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       3)
    , osSemaphore(usbd1_driver_ep3in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       4)
    , osSemaphore(usbd1_driver_ep4in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       5)
    , osSemaphore(usbd1_driver_ep5in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       6)
    , osSemaphore(usbd1_driver_ep6in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       7)
    , osSemaphore(usbd1_driver_ep7in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       8)
    , osSemaphore(usbd1_driver_ep8in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       9)
    , osSemaphore(usbd1_driver_ep9in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       10)
    , osSemaphore(usbd1_driver_ep10in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       11)
    , osSemaphore(usbd1_driver_ep11in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       12)
    , osSemaphore(usbd1_driver_ep12in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       13)
    , osSemaphore(usbd1_driver_ep13in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       14)
    , osSemaphore(usbd1_driver_ep14in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       15)
    , osSemaphore(usbd1_driver_ep15in_semaphore)
#endif
    }
#endif
#ifdef  RTE_USB_Device_2
  , {
#if   (USBD_EP_LAST_NUM >=       0)
      osSemaphore(usbd2_driver_ep0out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       1)
    , osSemaphore(usbd2_driver_ep1out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       2)
    , osSemaphore(usbd2_driver_ep2out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       3)
    , osSemaphore(usbd2_driver_ep3out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       4)
    , osSemaphore(usbd2_driver_ep4out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       5)
    , osSemaphore(usbd2_driver_ep5out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       6)
    , osSemaphore(usbd2_driver_ep6out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       7)
    , osSemaphore(usbd2_driver_ep7out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       8)
    , osSemaphore(usbd2_driver_ep8out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       9)
    , osSemaphore(usbd2_driver_ep9out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       10)
    , osSemaphore(usbd2_driver_ep10out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       11)
    , osSemaphore(usbd2_driver_ep11out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       12)
    , osSemaphore(usbd2_driver_ep12out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       13)
    , osSemaphore(usbd2_driver_ep13out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       14)
    , osSemaphore(usbd2_driver_ep14out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       15)
    , osSemaphore(usbd2_driver_ep15out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       0)
    , osSemaphore(usbd2_driver_ep0in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       1)
    , osSemaphore(usbd2_driver_ep1in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       2)
    , osSemaphore(usbd2_driver_ep2in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       3)
    , osSemaphore(usbd2_driver_ep3in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       4)
    , osSemaphore(usbd2_driver_ep4in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       5)
    , osSemaphore(usbd2_driver_ep5in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       6)
    , osSemaphore(usbd2_driver_ep6in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       7)
    , osSemaphore(usbd2_driver_ep7in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       8)
    , osSemaphore(usbd2_driver_ep8in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       9)
    , osSemaphore(usbd2_driver_ep9in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       10)
    , osSemaphore(usbd2_driver_ep10in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       11)
    , osSemaphore(usbd2_driver_ep11in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       12)
    , osSemaphore(usbd2_driver_ep12in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       13)
    , osSemaphore(usbd2_driver_ep13in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       14)
    , osSemaphore(usbd2_driver_ep14in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       15)
    , osSemaphore(usbd2_driver_ep15in_semaphore)
#endif
    }
#endif
#ifdef  RTE_USB_Device_3
  , {
#if   (USBD_EP_LAST_NUM >=       0)
      osSemaphore(usbd3_driver_ep0out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       1)
    , osSemaphore(usbd3_driver_ep1out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       2)
    , osSemaphore(usbd3_driver_ep2out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       3)
    , osSemaphore(usbd3_driver_ep3out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       4)
    , osSemaphore(usbd3_driver_ep4out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       5)
    , osSemaphore(usbd3_driver_ep5out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       6)
    , osSemaphore(usbd3_driver_ep6out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       7)
    , osSemaphore(usbd3_driver_ep7out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       8)
    , osSemaphore(usbd3_driver_ep8out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       9)
    , osSemaphore(usbd3_driver_ep9out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       10)
    , osSemaphore(usbd3_driver_ep10out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       11)
    , osSemaphore(usbd3_driver_ep11out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       12)
    , osSemaphore(usbd3_driver_ep12out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       13)
    , osSemaphore(usbd3_driver_ep13out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       14)
    , osSemaphore(usbd3_driver_ep14out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       15)
    , osSemaphore(usbd3_driver_ep15out_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       0)
    , osSemaphore(usbd3_driver_ep0in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       1)
    , osSemaphore(usbd3_driver_ep1in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       2)
    , osSemaphore(usbd3_driver_ep2in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       3)
    , osSemaphore(usbd3_driver_ep3in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       4)
    , osSemaphore(usbd3_driver_ep4in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       5)
    , osSemaphore(usbd3_driver_ep5in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       6)
    , osSemaphore(usbd3_driver_ep6in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       7)
    , osSemaphore(usbd3_driver_ep7in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       8)
    , osSemaphore(usbd3_driver_ep8in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       9)
    , osSemaphore(usbd3_driver_ep9in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       10)
    , osSemaphore(usbd3_driver_ep10in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       11)
    , osSemaphore(usbd3_driver_ep11in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       12)
    , osSemaphore(usbd3_driver_ep12in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       13)
    , osSemaphore(usbd3_driver_ep13in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       14)
    , osSemaphore(usbd3_driver_ep14in_semaphore)
#endif
#if   (USBD_EP_LAST_NUM >=       15)
    , osSemaphore(usbd3_driver_ep15in_semaphore)
#endif
    }
#endif
};

#if   (USBD_HID_NUM > 0)
#ifdef RTE_USB_Device_HID_0
osSemaphoreExt(usbd_hid0_semaphore);
osSemaphoreDef(usbd_hid0_semaphore);
#endif
#ifdef RTE_USB_Device_HID_1
osSemaphoreExt(usbd_hid1_semaphore);
osSemaphoreDef(usbd_hid1_semaphore);
#endif
#ifdef RTE_USB_Device_HID_2
osSemaphoreExt(usbd_hid2_semaphore);
osSemaphoreDef(usbd_hid2_semaphore);
#endif
#ifdef RTE_USB_Device_HID_3
osSemaphoreExt(usbd_hid3_semaphore);
osSemaphoreDef(usbd_hid3_semaphore);
#endif

static const osSemaphoreDef_t * const usbd_hid_semaphore_def_ptr[USBD_HID_NUM] = {
#ifdef RTE_USB_Device_HID_0
    osSemaphore(usbd_hid0_semaphore)
#endif
#ifdef RTE_USB_Device_HID_1
  , osSemaphore(usbd_hid1_semaphore)
#endif
#ifdef RTE_USB_Device_HID_2
  , osSemaphore(usbd_hid2_semaphore)
#endif
#ifdef RTE_USB_Device_HID_3
  , osSemaphore(usbd_hid3_semaphore)
#endif
};
#endif


// Functions definition

/// \brief Create a thread
/// \param[in]     thread               thread
/// \param[in]     index                parameter dependent on thread (device index or class instance or endpoint index)
/// \return
///                value != 0:          handle to created thread
///                value = 0:           thread creation failed
void *USBD_ThreadCreate (usbdThread_t thread, uint8_t index) {
  (void)index;

  switch (thread) {
    case usbdThreadCore:
      if (index >= USBD_DEV_NUM) { return NULL; }
      return ((void *)osThreadCreate (usbd_core_thread_def_ptr[index],            NULL));

    case usbdThreadCC:
#if (USBD_CUSTOM_CLASS_NUM > 0)
      if (index >= (USBD_CUSTOM_CLASS_NUM * USBD_EP_NUM)) { return NULL; }
      return ((void *)osThreadCreate (usbd_custom_class_ep_thread_def_ptr[index], NULL));
#else
      return NULL;
#endif

    case usbdThreadADC:
#if (USBD_ADC_NUM > 0)
      if (index >= USBD_ADC_NUM) { return NULL; }
      return ((void *)osThreadCreate (usbd_adc_thread_def_ptr[index],             NULL));
#else
      return NULL;
#endif

    case usbdThreadCDC_Int:
#if (USBD_CDC_NUM > 0)
      if (index >= USBD_CDC_NUM) { return NULL; }
      return ((void *)osThreadCreate (usbd_cdc_int_thread_def_ptr[index],         NULL));
#else
      return NULL;
#endif
    case usbdThreadCDC_Bulk:
#if (USBD_CDC_NUM > 0)
      if (index >= USBD_CDC_NUM) { return NULL; }
      return ((void *)osThreadCreate (usbd_cdc_bulk_thread_def_ptr[index],        NULL));
#else
      return NULL;
#endif

    case usbdThreadHID:
#if (USBD_HID_NUM > 0)
      if (index >= USBD_HID_NUM) { return NULL; }
      return ((void *)osThreadCreate (usbd_hid_thread_def_ptr[index],             NULL));
#else
      return NULL;
#endif

    case usbdThreadMSC:
#if (USBD_MSC_NUM > 0)
      if (index >= USBD_MSC_NUM) { return NULL; }
      return ((void *)osThreadCreate (usbd_msc_thread_def_ptr[index],             NULL));
#else
      return NULL;
#endif
  }
  return NULL;
}

/// \brief Terminate execution of a thread
/// \param[in]     thread_hndl          thread handle
/// \return
///                value 0:             thread terminated successfully
///                value < 0:           thread termination failed
int32_t USBD_ThreadTerminate (void *thread_hndl) {
  if (thread_hndl == NULL) { return -1; }
  return (-osThreadTerminate ((osThreadId)thread_hndl));
}

/// \brief Delay execution of a thread for specified number of milliseconds
/// \param[in]     millisec             number of milliseconds
/// \return
///                value 0:             delay finished
///                value < 0:           delay failed
int32_t USBD_Delay (uint32_t millisec) {
  return (-osDelay (millisec));
}

/// \brief Create and initialize a periodic timer for HID class handling
/// \param[in]     instance             class instance
/// \return
///                value != 0:          handle to created timer
///                value = 0:           timer creation failed
void *USBD_TimerCreate (uint8_t instance) {
#if (USBD_HID_NUM > 0)
  if (instance >= USBD_HID_NUM) { return NULL; }
  return ((void *)osTimerCreate (usbd_hid_timer_def_ptr[instance], osTimerPeriodic, (void *)(uint32_t)instance));
#else
  (void)instance;
  return NULL;
#endif
}

/// \brief Start or restart a timer
/// \param[in]     timer_hndl           timer handle
/// \return
///                value 0:             timer started or restarted successfully
///                value < 0:           timer start or restart failed
int32_t USBD_TimerStart (void *timer_hndl, uint32_t millisec) {
  if (timer_hndl == NULL) { return -1; }
  return (-osTimerStart ((osTimerId)timer_hndl, millisec));
}

/// \brief Stop a timer
/// \param[in]     timer_hndl           timer handle
/// \return
///                value 0:             timer stopped successfully
///                value < 0:           timer stop failed
int32_t USBD_TimerStop (void *timer_hndl) {
  if (timer_hndl == NULL) { return -1; }
  return (-osTimerStop ((osTimerId)timer_hndl));
}

/// \brief Delete a timer
/// \param[in]     timer_hndl           timer handle
/// \return
///                value 0:             timer deleted successfully
///                value < 0:           timer deletion failed
int32_t USBD_TimerDelete (void *timer_hndl) {
  if (timer_hndl == NULL) { return -1; }
  return (-osTimerDelete ((osTimerId)timer_hndl));
}

/// \brief Set the specified flags of a thread
/// \param[in]     thread_hndl          thread handle
/// \param[in]     flags                flags to be set
/// \return
///                value 0:             flags set successfully
///                value >= 0x80000000: setting of flags failed
uint32_t USBD_ThreadFlagsSet (void *thread_hndl, uint32_t flags) {
  if (thread_hndl == NULL) { return 0x80000000U; }
  return ((uint32_t)osSignalSet ((osThreadId)thread_hndl, (int32_t)flags));
}

/// \brief Wait for any USB related flag of currently running thread to become signaled
/// \param[in]     millisec             time-out in milliseconds, or 0 in case of no time-out
/// \return
///                value < 0x80000000:  flags
///                value 0:             time-out
///                value >= 0x80000000: error
uint32_t USBD_ThreadFlagsWait (uint32_t millisec) {
  osEvent event;

  event = osSignalWait (0U, millisec);
  if (event.status == osEventSignal) {
    return ((uint32_t)event.value.signals);
  } else if (event.status == osEventTimeout) {
    return 0U;
  } else {
    return 0x80000000U;
  }
}

/// \brief Create and initialize a binary semaphore
/// \param[in]     semaphore            semaphore
/// \param[in]     index1               semaphore dependent index1 (device index or class instance)
/// \param[in]     index2               semaphore dependent index2 (endpoint index)
/// \return
///                value != 0:          handle to created semaphore
///                value = 0:           semaphore creation failed
void *USBD_SemaphoreCreate (usbdSemaphore_t semaphore, uint8_t index1, uint8_t index2) {
  switch (semaphore) {
    case usbdSemaphoreCore:
      if (index1 >= USBD_DEV_NUM)       { return NULL; }
      return ((void *)osSemaphoreCreate (usbd_driver_semaphore_def_ptr[index1],            1U));

    case usbdSemaphoreEndpoint:
      if (index1 >=  USBD_DEV_NUM)      { return NULL; }
      if (index2 >= (USBD_EP_NUM * 2U)) { return NULL; }
      return ((void *)osSemaphoreCreate (usbd_driver_ep_semaphore_def_ptr[index1][index2], 1U));

    case usbdSemaphoreHID:
#if (USBD_HID_NUM > 0)
      if (index1 >= USBD_HID_NUM)       { return NULL; }
      return ((void *)osSemaphoreCreate (usbd_hid_semaphore_def_ptr[index1],               1U));
#else
      (void)semaphore;
      (void) index1;
      (void)index2;
      return NULL;
#endif
  }
  return NULL;
}

/// \brief Wait for a semaphore token to become available and acquire it
/// \param[in]     semaphore_hndl       semaphore handle
/// \param[in]     millisec             time-out in milliseconds, or 0 in case of no time-out
/// \return
///                value 0:             token acquired successfully
///                value < 0:           token acquire failed
int32_t USBD_SemaphoreAcquire (void *semaphore_hndl, uint32_t millisec) {
  int32_t result;

  if (semaphore_hndl == NULL) { return -1; }
  result = osSemaphoreWait ((osSemaphoreId)semaphore_hndl, millisec);
  if (result == 1) {
    return  0;
  } else {
    return -1;
  }
}

/// \brief Release a semaphore token
/// \param[in]     semaphore_hndl       semaphore handle
/// \return
///                value 0:             token released successfully
///                value < 0:           token release failed
int32_t USBD_SemaphoreRelease (void *semaphore_hndl) {
  if (semaphore_hndl == NULL) { return -1; }
  return (-osSemaphoreRelease ((osSemaphoreId)semaphore_hndl));
}

/// \brief Delete a semaphore
/// \param[in]     semaphore_hndl       semaphore handle
/// \return
///                value 0:             semaphore deleted successfully
///                value < 0:           semaphore deletion failed
int32_t USBD_SemaphoreDelete (void *semaphore_hndl) {
  if (semaphore_hndl == NULL) { return -1; }
  return (-osSemaphoreDelete ((osSemaphoreId)semaphore_hndl));
}
