/*------------------------------------------------------------------------------
 * MDK Middleware - Component ::USB:Device
 * Copyright (c) 2004-2017 ARM Germany GmbH. All rights reserved.
 *------------------------------------------------------------------------------
 * Name:    usbd_config_desc_3.c
 * Purpose: USB Device 3 Descriptor Creation
 *----------------------------------------------------------------------------*/

/* USB Device 3 Endpoint 0 Descriptor */
extern const uint8_t usbd3_ep0_descriptor[];
__WEAK const uint8_t usbd3_ep0_descriptor[] __ALIGNED(4) = { USBD_EP0(3) };

/* USB Device 3 Standard Descriptor */
extern const uint8_t usbd3_device_descriptor[];
__WEAK const uint8_t usbd3_device_descriptor[] __ALIGNED(4) = {
  USB_DEVICE_DESC_SIZE,                 /* bLength */
  USB_DEVICE_DESCRIPTOR_TYPE,           /* bDescriptorType */
  WBVAL(USBD3_BDC_USB),                 /* bcdUSB */
#if (USBD3_USE_IAD)
  USB_DEVICE_CLASS_MISCELLANEOUS,       /* bDeviceClass */
  0x02,                                 /* bDeviceSubClass */
  0x01,                                 /* bDeviceProtocol */
#elif ((USBD3_CDC_CNT == 1U) && (USBD3_CDC_ACM_CNT == 1U))
  CDC_COMMUNICATION_INTERFACE_CLASS,    /* bDeviceClass */
  CDC_ABSTRACT_CONTROL_MODEL,           /* bDeviceSubClass */
  0x01,                                 /* bDeviceProtocol */
#elif ((USBD3_CDC_CNT == 1U) && (USBD3_CDC_NCM_CNT == 1U))
  CDC_COMMUNICATION_INTERFACE_CLASS,    /* bDeviceClass */
  CDC_NETWORK_CONTROL_MODEL,            /* bDeviceSubClass */
  CDC_NCM_PROTOCOL_NONE,                /* bDeviceProtocol */
#else
  0x00,                                 /* bDeviceClass */
  0x00,                                 /* bDeviceSubClass */
  0x00,                                 /* bDeviceProtocol */
#endif
  USBD3_MAX_PACKET0,                    /* bMaxPacketSize0 */
  WBVAL(USBD3_DEV_DESC_IDVENDOR),       /* idVendor */
  WBVAL(USBD3_DEV_DESC_IDPRODUCT),      /* idProduct */
  WBVAL(USBD3_DEV_DESC_BCDDEVICE),      /* bcdDevice */
  0x01,                                 /* iManufacturer */
  0x02,                                 /* iProduct */
  0x03*USBD3_STR_DESC_SER_EN,           /* iSerialNumber */
  0x01                                  /* bNumConfigurations: one possible configuration */
};

/* USB Device 3 Configuration Descriptor (for Full Speed) */
/*   All Descriptors (Configuration, Interface, Endpoint, Class, Vendor) */
extern const uint8_t usbd3_config_descriptor_fs[];
__WEAK const uint8_t usbd3_config_descriptor_fs[] __ALIGNED(4) = {
  /* Configuration 1 */
  USB_CONFIGURATION_DESC_SIZE,          /* bLength */
  USB_CONFIGURATION_DESCRIPTOR_TYPE,    /* bDescriptorType */
  WBVAL(USBD3_WTOTALLENGTH),            /* wTotalLength */
  USBD3_IF_CNT,                         /* bNumInterfaces */
  0x01,                                 /* bConfigurationValue: 0x01 is used to select this configuration */
  0x00,                                 /* iConfiguration: no string to describe this configuration */
  USBD3_CFG_DESC_BMATTRIBUTES,          /* bmAttributes */
  USBD3_CFG_DESC_BMAXPOWER,             /* bMaxPower, device power consumption */

#if (USBD3_CUSTOM_CLASS0)
#if (USBD_CUSTOM_CLASS0_IAD_EN)
  USBD_CUSTOM_CLASSn_IAD                          (0)
#endif
#if (USBD_CUSTOM_CLASS0_IF0_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (0,0)
#if (USBD_CUSTOM_CLASS0_IF0_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,0,0,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF0_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,0,1,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF0_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,0,2,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF0_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,0,3,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF0_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,0,4,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF0_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,0,5,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF0_EP6_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,0,6,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF0_EP7_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,0,7,FS)
#endif
#endif
#if (USBD_CUSTOM_CLASS0_IF1_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (0,1)
#if (USBD_CUSTOM_CLASS0_IF1_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,1,0,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF1_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,1,1,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF1_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,1,2,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF1_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,1,3,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF1_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,1,4,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF1_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,1,5,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF1_EP6_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,1,6,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF1_EP7_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,1,7,FS)
#endif
#endif
#if (USBD_CUSTOM_CLASS0_IF2_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (0,2)
#if (USBD_CUSTOM_CLASS0_IF2_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,2,0,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF2_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,2,1,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF2_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,2,2,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF2_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,2,3,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF2_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,2,4,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF2_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,2,5,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF2_EP6_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,2,6,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF2_EP7_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,2,7,FS)
#endif
#endif
#if (USBD_CUSTOM_CLASS0_IF3_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (0,3)
#if (USBD_CUSTOM_CLASS0_IF3_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,3,0,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF3_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,3,1,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF3_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,3,2,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF3_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,3,3,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF3_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,3,4,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF3_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,3,5,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF3_EP6_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,3,6,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF3_EP7_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,3,7,FS)
#endif
#endif
#endif

#if (USBD3_CUSTOM_CLASS1)
#if (USBD_CUSTOM_CLASS1_IAD_EN)
  USBD_CUSTOM_CLASSn_IAD                          (1)
#endif
#if (USBD_CUSTOM_CLASS1_IF0_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (1,0)
#if (USBD_CUSTOM_CLASS1_IF0_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,0,0,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF0_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,0,1,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF0_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,0,2,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF0_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,0,3,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF0_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,0,4,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF0_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,0,5,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF0_EP6_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,0,6,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF0_EP7_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,0,7,FS)
#endif
#endif
#if (USBD_CUSTOM_CLASS1_IF1_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (1,1)
#if (USBD_CUSTOM_CLASS1_IF1_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,1,0,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF1_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,1,1,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF1_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,1,2,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF1_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,1,3,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF1_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,1,4,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF1_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,1,5,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF1_EP6_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,1,6,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF1_EP7_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,1,7,FS)
#endif
#endif
#if (USBD_CUSTOM_CLASS1_IF2_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (1,2)
#if (USBD_CUSTOM_CLASS1_IF2_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,2,0,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF2_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,2,1,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF2_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,2,2,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF2_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,2,3,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF2_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,2,4,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF2_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,2,5,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF2_EP6_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,2,6,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF2_EP7_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,2,7,FS)
#endif
#endif
#if (USBD_CUSTOM_CLASS1_IF3_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (1,3)
#if (USBD_CUSTOM_CLASS1_IF3_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,3,0,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF3_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,3,1,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF3_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,3,2,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF3_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,3,3,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF3_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,3,4,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF3_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,3,5,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF3_EP6_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,3,6,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF3_EP7_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,3,7,FS)
#endif
#endif
#endif

#if (USBD3_CUSTOM_CLASS2)
#if (USBD_CUSTOM_CLASS2_IAD_EN)
  USBD_CUSTOM_CLASSn_IAD                          (2)
#endif
#if (USBD_CUSTOM_CLASS2_IF0_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (2,0)
#if (USBD_CUSTOM_CLASS2_IF0_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,0,0,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF0_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,0,1,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF0_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,0,2,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF0_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,0,3,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF0_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,0,4,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF0_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,0,5,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF0_EP6_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,0,6,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF0_EP7_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,0,7,FS)
#endif
#endif
#if (USBD_CUSTOM_CLASS2_IF1_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (2,1)
#if (USBD_CUSTOM_CLASS2_IF1_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,1,0,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF1_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,1,1,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF1_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,1,2,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF1_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,1,3,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF1_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,1,4,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF1_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,1,5,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF1_EP6_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,1,6,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF1_EP7_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,1,7,FS)
#endif
#endif
#if (USBD_CUSTOM_CLASS2_IF2_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (2,2)
#if (USBD_CUSTOM_CLASS2_IF2_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,2,0,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF2_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,2,1,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF2_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,2,2,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF2_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,2,3,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF2_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,2,4,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF2_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,2,5,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF2_EP6_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,2,6,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF2_EP7_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,2,7,FS)
#endif
#endif
#if (USBD_CUSTOM_CLASS2_IF3_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (2,3)
#if (USBD_CUSTOM_CLASS2_IF3_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,3,0,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF3_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,3,1,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF3_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,3,2,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF3_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,3,3,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF3_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,3,4,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF3_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,3,5,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF3_EP6_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,3,6,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF3_EP7_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,3,7,FS)
#endif
#endif
#endif


#if (USBD3_CUSTOM_CLASS3)
#if (USBD_CUSTOM_CLASS3_IAD_EN)
  USBD_CUSTOM_CLASSn_IAD                          (3)
#endif
#if (USBD_CUSTOM_CLASS3_IF0_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (3,0)
#if (USBD_CUSTOM_CLASS3_IF0_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,0,0,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF0_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,0,1,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF0_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,0,2,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF0_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,0,3,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF0_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,0,4,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF0_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,0,5,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF0_EP6_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,0,6,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF0_EP7_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,0,7,FS)
#endif
#endif
#if (USBD_CUSTOM_CLASS3_IF1_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (3,1)
#if (USBD_CUSTOM_CLASS3_IF1_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,1,0,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF1_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,1,1,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF1_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,1,2,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF1_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,1,3,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF1_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,1,4,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF1_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,1,5,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF1_EP6_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,1,6,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF1_EP7_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,1,7,FS)
#endif
#endif
#if (USBD_CUSTOM_CLASS3_IF2_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (3,2)
#if (USBD_CUSTOM_CLASS3_IF2_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,2,0,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF2_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,2,1,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF2_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,2,2,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF2_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,2,3,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF2_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,2,4,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF2_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,2,5,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF2_EP6_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,2,6,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF2_EP7_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,2,7,FS)
#endif
#endif
#if (USBD_CUSTOM_CLASS3_IF3_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (3,3)
#if (USBD_CUSTOM_CLASS3_IF3_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,3,0,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF3_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,3,1,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF3_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,3,2,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF3_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,3,3,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF3_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,3,4,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF3_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,3,5,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF3_EP6_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,3,6,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF3_EP7_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,3,7,FS)
#endif
#endif
#endif

#if (USBD3_ADC0)
#if (USBD3_USE_IAD)
  USBD_ADCn_IAD                                   (0,USBD_ADC0_IF_CTRL_NUM,USBD_ADC0_IF_CNT)
#endif
  USBD_ADCn_IF_DESC_AC                            (0)
#if   (USBD_ADC0_IF_SPKR && USBD_ADC0_IF_MIC)
  USBD_ADCn_IF_DESC_AC_HEADER                     (0,USBD_ADC0_OUT_CH_NUM,USBD_ADC0_IN_CH_NUM)
#elif (USBD_ADC0_IF_SPKR)
  USBD_ADCn_IF_DESC_AC_HEADER_SPEAKER             (0,USBD_ADC0_OUT_CH_NUM)
#elif (USBD_ADC0_IF_MIC)
  USBD_ADCn_IF_DESC_AC_HEADER_MICROPHONE          (0,USBD_ADC0_IN_CH_NUM)
#endif
#if   (USBD_ADC0_IF_SPKR)
#if   (USBD_ADC0_OUT_CH_NUM == 1)
  USBD_ADCn_IF_DESC_AC_IT_FU_OT_MONO_SPEAKER      (0,1)
#elif (USBD_ADC0_OUT_CH_NUM == 2)
  USBD_ADCn_IF_DESC_AC_IT_FU_OT_STEREO_SPEAKERS   (0,1)
#endif
#endif
#if   (USBD_ADC0_IF_MIC)
#if   (USBD_ADC0_IN_CH_NUM == 1)
  USBD_ADCn_IF_DESC_AC_IT_FU_OT_MONO_MICROPHONE   (0,1+USBD_ADC0_IF_SPKR*3)
#elif (USBD_ADC0_IN_CH_NUM == 2)
  USBD_ADCn_IF_DESC_AC_IT_FU_OT_STEREO_MICROPHONES(0,1+USBD_ADC0_IF_SPKR*3)
#endif
#endif
#if   (USBD_ADC0_IF_SPKR)
  USBD_ADCn_IF_DESC_AS_SPEAKER                    (0)
  USBD_ADCn_IF_DESC_AS_IF_FT_SPEAKER              (0,1,USBD_ADC0_OUT_CH_NUM)
  USBD_ADCn_EP_DESC_SPEAKER                       (0)
#endif
#if   (USBD_ADC0_IF_MIC)
  USBD_ADCn_IF_DESC_AS_MICROPHONE                 (0)
  USBD_ADCn_IF_DESC_AS_IF_FT_MICROPHONE           (0,3+USBD_ADC0_IF_SPKR*3,USBD_ADC0_IN_CH_NUM)
  USBD_ADCn_EP_DESC_MICROPHONE                    (0)
#endif
#endif
#if (USBD3_ADC1)
#if (USBD3_USE_IAD)
  USBD_ADCn_IAD                                   (1,USBD_ADC1_IF_CTRL_NUM,USBD_ADC1_IF_CNT)
#endif
  USBD_ADCn_IF_DESC_AC                            (1)
#if   (USBD_ADC1_IF_SPKR && USBD_ADC1_IF_MIC)
  USBD_ADCn_IF_DESC_AC_HEADER                     (1,USBD_ADC1_OUT_CH_NUM,USBD_ADC1_IN_CH_NUM)
#elif (USBD_ADC1_IF_SPKR)
  USBD_ADCn_IF_DESC_AC_HEADER_SPEAKER             (1,USBD_ADC1_OUT_CH_NUM)
#elif (USBD_ADC1_IF_MIC)
  USBD_ADCn_IF_DESC_AC_HEADER_MICROPHONE          (1,USBD_ADC1_IN_CH_NUM)
#endif
#if   (USBD_ADC1_IF_SPKR)
#if   (USBD_ADC1_OUT_CH_NUM == 1)
  USBD_ADCn_IF_DESC_AC_IT_FU_OT_MONO_SPEAKER      (1,1)
#elif (USBD_ADC1_OUT_CH_NUM == 2)
  USBD_ADCn_IF_DESC_AC_IT_FU_OT_STEREO_SPEAKERS   (1,1)
#endif
#endif
#if   (USBD_ADC1_IF_MIC)
#if   (USBD_ADC1_IN_CH_NUM == 1)
  USBD_ADCn_IF_DESC_AC_IT_FU_OT_MONO_MICROPHONE   (1,1+USBD_ADC1_IF_SPKR*3)
#elif (USBD_ADC1_IN_CH_NUM == 2)
  USBD_ADCn_IF_DESC_AC_IT_FU_OT_STEREO_MICROPHONES(1,1+USBD_ADC1_IF_SPKR*3)
#endif
#endif
#if   (USBD_ADC1_IF_SPKR)
  USBD_ADCn_IF_DESC_AS_SPEAKER                    (1)
  USBD_ADCn_IF_DESC_AS_IF_FT_SPEAKER              (1,1,USBD_ADC1_OUT_CH_NUM)
  USBD_ADCn_EP_DESC_SPEAKER                       (1)
#endif
#if   (USBD_ADC1_IF_MIC)
  USBD_ADCn_IF_DESC_AS_MICROPHONE                 (1)
  USBD_ADCn_IF_DESC_AS_IF_FT_MICROPHONE           (1,3+USBD_ADC1_IF_SPKR*3,USBD_ADC1_IN_CH_NUM)
  USBD_ADCn_EP_DESC_MICROPHONE                    (1)
#endif
#endif
#if (USBD3_ADC2)
#if (USBD3_USE_IAD)
  USBD_ADCn_IAD                                   (2,USBD_ADC2_IF_CTRL_NUM,USBD_ADC2_IF_CNT)
#endif
  USBD_ADCn_IF_DESC_AC                            (2)
#if   (USBD_ADC2_IF_SPKR && USBD_ADC2_IF_MIC)
  USBD_ADCn_IF_DESC_AC_HEADER                     (2,USBD_ADC2_OUT_CH_NUM,USBD_ADC2_IN_CH_NUM)
#elif (USBD_ADC2_IF_SPKR)
  USBD_ADCn_IF_DESC_AC_HEADER_SPEAKER             (2,USBD_ADC2_OUT_CH_NUM)
#elif (USBD_ADC2_IF_MIC)
  USBD_ADCn_IF_DESC_AC_HEADER_MICROPHONE          (2,USBD_ADC2_IN_CH_NUM)
#endif
#if   (USBD_ADC2_IF_SPKR)
#if   (USBD_ADC2_OUT_CH_NUM == 1)
  USBD_ADCn_IF_DESC_AC_IT_FU_OT_MONO_SPEAKER      (2,1)
#elif (USBD_ADC2_OUT_CH_NUM == 2)
  USBD_ADCn_IF_DESC_AC_IT_FU_OT_STEREO_SPEAKERS   (2,1)
#endif
#endif
#if   (USBD_ADC2_IF_MIC)
#if   (USBD_ADC2_IN_CH_NUM == 1)
  USBD_ADCn_IF_DESC_AC_IT_FU_OT_MONO_MICROPHONE   (2,1+USBD_ADC2_IF_SPKR*3)
#elif (USBD_ADC2_IN_CH_NUM == 2)
  USBD_ADCn_IF_DESC_AC_IT_FU_OT_STEREO_MICROPHONES(2,1+USBD_ADC2_IF_SPKR*3)
#endif
#endif
#if   (USBD_ADC2_IF_SPKR)
  USBD_ADCn_IF_DESC_AS_SPEAKER                    (2)
  USBD_ADCn_IF_DESC_AS_IF_FT_SPEAKER              (2,1,USBD_ADC2_OUT_CH_NUM)
  USBD_ADCn_EP_DESC_SPEAKER                       (2)
#endif
#if   (USBD_ADC2_IF_MIC)
  USBD_ADCn_IF_DESC_AS_MICROPHONE                 (2)
  USBD_ADCn_IF_DESC_AS_IF_FT_MICROPHONE           (2,3+USBD_ADC2_IF_SPKR*3,USBD_ADC2_IN_CH_NUM)
  USBD_ADCn_EP_DESC_MICROPHONE                    (2)
#endif
#endif
#if (USBD3_ADC3)
#if (USBD3_USE_IAD)
  USBD_ADCn_IAD                                   (3,USBD_ADC3_IF_CTRL_NUM,USBD_ADC3_IF_CNT)
#endif
  USBD_ADCn_IF_DESC_AC                            (3)
#if   (USBD_ADC3_IF_SPKR && USBD_ADC3_IF_MIC)
  USBD_ADCn_IF_DESC_AC_HEADER                     (3,USBD_ADC3_OUT_CH_NUM,USBD_ADC3_IN_CH_NUM)
#elif (USBD_ADC3_IF_SPKR)
  USBD_ADCn_IF_DESC_AC_HEADER_SPEAKER             (3,USBD_ADC3_OUT_CH_NUM)
#elif (USBD_ADC3_IF_MIC)
  USBD_ADCn_IF_DESC_AC_HEADER_MICROPHONE          (3,USBD_ADC3_IN_CH_NUM)
#endif
#if   (USBD_ADC3_IF_SPKR)
#if   (USBD_ADC3_OUT_CH_NUM == 1)
  USBD_ADCn_IF_DESC_AC_IT_FU_OT_MONO_SPEAKER      (3,1)
#elif (USBD_ADC3_OUT_CH_NUM == 2)
  USBD_ADCn_IF_DESC_AC_IT_FU_OT_STEREO_SPEAKERS   (3,1)
#endif
#endif
#if   (USBD_ADC3_IF_MIC)
#if   (USBD_ADC3_IN_CH_NUM == 1)
  USBD_ADCn_IF_DESC_AC_IT_FU_OT_MONO_MICROPHONE   (3,1+USBD_ADC3_IF_SPKR*3)
#elif (USBD_ADC3_IN_CH_NUM == 2)
  USBD_ADCn_IF_DESC_AC_IT_FU_OT_STEREO_MICROPHONES(3,1+USBD_ADC3_IF_SPKR*3)
#endif
#endif
#if   (USBD_ADC3_IF_SPKR)
  USBD_ADCn_IF_DESC_AS_SPEAKER                    (3)
  USBD_ADCn_IF_DESC_AS_IF_FT_SPEAKER              (3,1,USBD_ADC3_OUT_CH_NUM)
  USBD_ADCn_EP_DESC_SPEAKER                       (3)
#endif
#if   (USBD_ADC3_IF_MIC)
  USBD_ADCn_IF_DESC_AS_MICROPHONE                 (3)
  USBD_ADCn_IF_DESC_AS_IF_FT_MICROPHONE           (3,3+USBD_ADC3_IF_SPKR*3,USBD_ADC3_IN_CH_NUM)
  USBD_ADCn_EP_DESC_MICROPHONE                    (3)
#endif
#endif

#if   (USBD3_CDC0)
#if   (USBD3_CDC0_ACM)
#if   (USBD3_USE_IAD)
  USBD_CDCn_ACM_IAD                               (0,USBD_CDC0_IF0_NUM,2)
#endif
  USBD_CDCn_ACM_DESC_IF0                          (0)
  USBD_CDCn_ACM_EP_IF0                            (0)
  USBD_CDCn_ACM_DESC_IF1                          (0)
  USBD_CDCn_ACM_EP_IF1                            (0)
#elif (USBD3_CDC0_NCM)
#if   (USBD3_USE_IAD)
  USBD_CDCn_NCM_IAD                               (0,USBD_CDC0_IF0_NUM,2)
#endif
  USBD_CDCn_NCM_DESC_IF0                          (0)
  USBD_CDCn_NCM_EP_IF0                            (0)
  USBD_CDCn_NCM_DESC_IF1                          (0)
  USBD_CDCn_NCM_EP_IF1                            (0)
#endif
#endif
#if   (USBD3_CDC1)
#if   (USBD3_CDC1_ACM)
#if   (USBD3_USE_IAD)
  USBD_CDCn_ACM_IAD                               (1,USBD_CDC1_IF0_NUM,2)
#endif
  USBD_CDCn_ACM_DESC_IF0                          (1)
  USBD_CDCn_ACM_EP_IF0                            (1)
  USBD_CDCn_ACM_DESC_IF1                          (1)
  USBD_CDCn_ACM_EP_IF1                            (1)
#elif (USBD3_CDC1_NCM)
#if   (USBD3_USE_IAD)
  USBD_CDCn_NCM_IAD                               (1,USBD_CDC1_IF0_NUM,2)
#endif
  USBD_CDCn_NCM_DESC_IF0                          (1)
  USBD_CDCn_NCM_EP_IF0                            (1)
  USBD_CDCn_NCM_DESC_IF1                          (1)
  USBD_CDCn_NCM_EP_IF1                            (1)
#endif
#endif
#if   (USBD3_CDC2)
#if   (USBD3_CDC2_ACM)
#if   (USBD3_USE_IAD)
  USBD_CDCn_ACM_IAD                               (2,USBD_CDC2_IF0_NUM,2)
#endif
  USBD_CDCn_ACM_DESC_IF0                          (2)
  USBD_CDCn_ACM_EP_IF0                            (2)
  USBD_CDCn_ACM_DESC_IF1                          (2)
  USBD_CDCn_ACM_EP_IF1                            (2)
#elif (USBD3_CDC2_NCM)
#if   (USBD3_USE_IAD)
  USBD_CDCn_NCM_IAD                               (2,USBD_CDC2_IF0_NUM,2)
#endif
  USBD_CDCn_NCM_DESC_IF0                          (2)
  USBD_CDCn_NCM_EP_IF0                            (2)
  USBD_CDCn_NCM_DESC_IF1                          (2)
  USBD_CDCn_NCM_EP_IF1                            (2)
#endif
#endif
#if   (USBD3_CDC3)
#if   (USBD3_CDC3_ACM)
#if   (USBD3_USE_IAD)
  USBD_CDCn_ACM_IAD                               (3,USBD_CDC3_IF0_NUM,2)
#endif
  USBD_CDCn_ACM_DESC_IF0                          (3)
  USBD_CDCn_ACM_EP_IF0                            (3)
  USBD_CDCn_ACM_DESC_IF1                          (3)
  USBD_CDCn_ACM_EP_IF1                            (3)
#elif (USBD3_CDC3_NCM)
#if   (USBD3_USE_IAD)
  USBD_CDCn_NCM_IAD                               (3,USBD_CDC3_IF0_NUM,2)
#endif
  USBD_CDCn_NCM_DESC_IF0                          (3)
  USBD_CDCn_NCM_EP_IF0                            (3)
  USBD_CDCn_NCM_DESC_IF1                          (3)
  USBD_CDCn_NCM_EP_IF1                            (3)
#endif
#endif

#if (USBD3_HID0)
  USBD_HIDn_DESC                                  (0)
#if (USBD_HID0_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                              (0)
#else
  USBD_HIDn_EP                                    (0)
#endif
#endif
#if (USBD3_HID1)
  USBD_HIDn_DESC                                  (1)
#if (USBD_HID1_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                              (1)
#else
  USBD_HIDn_EP                                    (1)
#endif
#endif
#if (USBD3_HID2)
  USBD_HIDn_DESC                                  (2)
#if (USBD_HID2_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                              (2)
#else
  USBD_HIDn_EP                                    (2)
#endif
#endif
#if (USBD3_HID3)
  USBD_HIDn_DESC                                  (3)
#if (USBD_HID3_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                              (3)
#else
  USBD_HIDn_EP                                    (3)
#endif
#endif

#if (USBD3_MSC0)
  USBD_MSCn_DESC                                  (0)
  USBD_MSCn_EP                                    (0)
#endif
#if (USBD3_MSC1)
  USBD_MSCn_DESC                                  (1)
  USBD_MSCn_EP                                    (1)
#endif
#if (USBD3_MSC2)
  USBD_MSCn_DESC                                  (2)
  USBD_MSCn_EP                                    (2)
#endif
#if (USBD3_MSC3)
  USBD_MSCn_DESC                                  (3)
  USBD_MSCn_EP                                    (3)
#endif

/* Terminator */
  0                                     /* bLength */
};

#if (!USBD3_HS)                         /* If High-speed not enabled, declare dummy descriptors for High-speed */
extern const uint8_t usbd3_device_qualifier_fs[];
__WEAK const uint8_t usbd3_device_qualifier_fs[]              = { 0 };
extern const uint8_t usbd3_device_qualifier_hs[];
__WEAK const uint8_t usbd3_device_qualifier_hs[]              = { 0 };
extern const uint8_t usbd3_config_descriptor_hs[];
__WEAK const uint8_t usbd3_config_descriptor_hs[]             = { 0 };
extern const uint8_t usbd3_other_speed_config_descriptor_fs[];
__WEAK const uint8_t usbd3_other_speed_config_descriptor_fs[] = { 0 };
extern const uint8_t usbd3_other_speed_config_descriptor_hs[];
__WEAK const uint8_t usbd3_other_speed_config_descriptor_hs[] = { 0 };

#else

#if (USBD3_ADC0 || USBD3_ADC1 || USBD3_ADC2 || USBD3_ADC3)
#error High-speed is not supported for Audio Device!
#endif

/* USB Device 3 Qualifier Descriptor (in Full Speed for High Speed) */
extern const uint8_t usbd3_device_qualifier_fs[];
__WEAK const uint8_t usbd3_device_qualifier_fs[] __ALIGNED(4) = {
  USB_DEVICE_QUALI_SIZE,                /* bLength */
  USB_DEVICE_QUALIFIER_DESCRIPTOR_TYPE, /* bDescriptorType */
  WBVAL(0x0200), /* 2.00 */             /* bcdUSB */
  0x00,                                 /* bDeviceClass */
  0x00,                                 /* bDeviceSubClass */
  0x00,                                 /* bDeviceProtocol */
  USBD3_MAX_PACKET0,                    /* bMaxPacketSize0 */
  0x01,                                 /* bNumConfigurations */
  0x00                                  /* bReserved */
};

/* USB Device 3 Qualifier Descriptor (in High Speed for Full Speed) */
extern const uint8_t usbd3_device_qualifier_hs[];
__WEAK const uint8_t usbd3_device_qualifier_hs[] __ALIGNED(4) = {
  USB_DEVICE_QUALI_SIZE,                /* bLength */
  USB_DEVICE_QUALIFIER_DESCRIPTOR_TYPE, /* bDescriptorType */
  WBVAL(0x0200), /* 2.00 */             /* bcdUSB */
  0x00,                                 /* bDeviceClass */
  0x00,                                 /* bDeviceSubClass */
  0x00,                                 /* bDeviceProtocol */
  USBD3_MAX_PACKET0,                    /* bMaxPacketSize0 */
  0x01,                                 /* bNumConfigurations */
  0x00                                  /* bReserved */
};

/* USB Device 3 Configuration Descriptor (for High Speed) */
/*   All Descriptors (Configuration, Interface, Endpoint, Class, Vendor) */
extern const uint8_t usbd3_config_descriptor_hs[];
__WEAK const uint8_t usbd3_config_descriptor_hs[] __ALIGNED(4) = {
  /* Configuration 1 */
  USB_CONFIGURATION_DESC_SIZE,          /* bLength */
  USB_CONFIGURATION_DESCRIPTOR_TYPE,    /* bDescriptorType */
  WBVAL(USBD3_WTOTALLENGTH),            /* wTotalLength */
  USBD3_IF_CNT,                         /* bNumInterfaces */
  0x01,                                 /* bConfigurationValue: 0x01 is used to select this configuration */
  0x00,                                 /* iConfiguration: no string to describe this configuration */
  USBD3_CFG_DESC_BMATTRIBUTES,          /* bmAttributes */
  USBD3_CFG_DESC_BMAXPOWER,             /* bMaxPower, device power consumption */

#if (USBD3_CUSTOM_CLASS0)
#if (USBD_CUSTOM_CLASS0_IAD_EN)
  USBD_CUSTOM_CLASSn_IAD                          (0)
#endif
#if (USBD_CUSTOM_CLASS0_IF0_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (0,0)
#if (USBD_CUSTOM_CLASS0_IF0_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,0,0,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF0_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,0,1,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF0_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,0,2,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF0_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,0,3,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF0_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,0,4,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF0_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,0,5,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF0_EP6_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,0,6,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF0_EP7_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,0,7,HS)
#endif
#endif
#if (USBD_CUSTOM_CLASS0_IF1_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (0,1)
#if (USBD_CUSTOM_CLASS0_IF1_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,1,0,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF1_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,1,1,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF1_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,1,2,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF1_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,1,3,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF1_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,1,4,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF1_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,1,5,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF1_EP6_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,1,6,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF1_EP7_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,1,7,HS)
#endif
#endif
#if (USBD_CUSTOM_CLASS0_IF2_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (0,2)
#if (USBD_CUSTOM_CLASS0_IF2_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,2,0,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF2_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,2,1,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF2_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,2,2,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF2_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,2,3,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF2_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,2,4,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF2_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,2,5,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF2_EP6_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,2,6,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF2_EP7_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,2,7,HS)
#endif
#endif
#if (USBD_CUSTOM_CLASS0_IF3_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (0,3)
#if (USBD_CUSTOM_CLASS0_IF3_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,3,0,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF3_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,3,1,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF3_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,3,2,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF3_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,3,3,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF3_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,3,4,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF3_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,3,5,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF3_EP6_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,3,6,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF3_EP7_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,3,7,HS)
#endif
#endif
#endif

#if (USBD3_CUSTOM_CLASS1)
#if (USBD_CUSTOM_CLASS1_IAD_EN)
  USBD_CUSTOM_CLASSn_IAD                          (1)
#endif
#if (USBD_CUSTOM_CLASS1_IF0_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (1,0)
#if (USBD_CUSTOM_CLASS1_IF0_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,0,0,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF0_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,0,1,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF0_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,0,2,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF0_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,0,3,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF0_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,0,4,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF0_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,0,5,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF0_EP6_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,0,6,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF0_EP7_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,0,7,HS)
#endif
#endif
#if (USBD_CUSTOM_CLASS1_IF1_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (1,1)
#if (USBD_CUSTOM_CLASS1_IF1_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,1,0,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF1_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,1,1,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF1_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,1,2,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF1_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,1,3,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF1_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,1,4,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF1_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,1,5,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF1_EP6_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,1,6,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF1_EP7_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,1,7,HS)
#endif
#endif
#if (USBD_CUSTOM_CLASS1_IF2_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (1,2)
#if (USBD_CUSTOM_CLASS1_IF2_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,2,0,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF2_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,2,1,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF2_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,2,2,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF2_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,2,3,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF2_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,2,4,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF2_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,2,5,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF2_EP6_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,2,6,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF2_EP7_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,2,7,HS)
#endif
#endif
#if (USBD_CUSTOM_CLASS1_IF3_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (1,3)
#if (USBD_CUSTOM_CLASS1_IF3_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,3,0,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF3_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,3,1,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF3_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,3,2,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF3_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,3,3,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF3_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,3,4,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF3_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,3,5,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF3_EP6_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,3,6,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF3_EP7_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,3,7,HS)
#endif
#endif
#endif

#if (USBD3_CUSTOM_CLASS2)
#if (USBD_CUSTOM_CLASS2_IAD_EN)
  USBD_CUSTOM_CLASSn_IAD                          (2)
#endif
#if (USBD_CUSTOM_CLASS2_IF0_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (2,0)
#if (USBD_CUSTOM_CLASS2_IF0_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,0,0,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF0_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,0,1,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF0_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,0,2,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF0_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,0,3,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF0_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,0,4,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF0_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,0,5,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF0_EP6_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,0,6,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF0_EP7_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,0,7,HS)
#endif
#endif
#if (USBD_CUSTOM_CLASS2_IF1_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (2,1)
#if (USBD_CUSTOM_CLASS2_IF1_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,1,0,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF1_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,1,1,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF1_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,1,2,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF1_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,1,3,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF1_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,1,4,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF1_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,1,5,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF1_EP6_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,1,6,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF1_EP7_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,1,7,HS)
#endif
#endif
#if (USBD_CUSTOM_CLASS2_IF2_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (2,2)
#if (USBD_CUSTOM_CLASS2_IF2_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,2,0,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF2_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,2,1,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF2_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,2,2,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF2_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,2,3,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF2_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,2,4,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF2_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,2,5,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF2_EP6_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,2,6,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF2_EP7_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,2,7,HS)
#endif
#endif
#if (USBD_CUSTOM_CLASS2_IF3_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (2,3)
#if (USBD_CUSTOM_CLASS2_IF3_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,3,0,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF3_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,3,1,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF3_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,3,2,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF3_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,3,3,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF3_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,3,4,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF3_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,3,5,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF3_EP6_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,3,6,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF3_EP7_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,3,7,HS)
#endif
#endif
#endif


#if (USBD3_CUSTOM_CLASS3)
#if (USBD_CUSTOM_CLASS3_IAD_EN)
  USBD_CUSTOM_CLASSn_IAD                          (3)
#endif
#if (USBD_CUSTOM_CLASS3_IF0_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (3,0)
#if (USBD_CUSTOM_CLASS3_IF0_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,0,0,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF0_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,0,1,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF0_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,0,2,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF0_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,0,3,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF0_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,0,4,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF0_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,0,5,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF0_EP6_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,0,6,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF0_EP7_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,0,7,HS)
#endif
#endif
#if (USBD_CUSTOM_CLASS3_IF1_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (3,1)
#if (USBD_CUSTOM_CLASS3_IF1_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,1,0,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF1_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,1,1,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF1_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,1,2,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF1_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,1,3,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF1_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,1,4,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF1_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,1,5,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF1_EP6_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,1,6,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF1_EP7_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,1,7,HS)
#endif
#endif
#if (USBD_CUSTOM_CLASS3_IF2_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (3,2)
#if (USBD_CUSTOM_CLASS3_IF2_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,2,0,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF2_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,2,1,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF2_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,2,2,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF2_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,2,3,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF2_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,2,4,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF2_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,2,5,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF2_EP6_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,2,6,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF2_EP7_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,2,7,HS)
#endif
#endif
#if (USBD_CUSTOM_CLASS3_IF3_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (3,3)
#if (USBD_CUSTOM_CLASS3_IF3_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,3,0,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF3_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,3,1,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF3_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,3,2,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF3_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,3,3,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF3_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,3,4,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF3_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,3,5,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF3_EP6_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,3,6,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF3_EP7_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,3,7,HS)
#endif
#endif
#endif

#if   (USBD3_CDC0)
#if   (USBD3_CDC0_ACM)
#if   (USBD3_USE_IAD)
  USBD_CDCn_ACM_IAD                               (0,USBD_CDC0_IF0_NUM,2)
#endif
  USBD_CDCn_ACM_DESC_IF0                          (0)
  USBD_CDCn_ACM_EP_IF0_HS                         (0)
  USBD_CDCn_ACM_DESC_IF1                          (0)
  USBD_CDCn_ACM_EP_IF1_HS                         (0)
#elif (USBD3_CDC0_NCM)
#if   (USBD3_USE_IAD)
  USBD_CDCn_NCM_IAD                               (0,USBD_CDC0_IF0_NUM,2)
#endif
  USBD_CDCn_NCM_DESC_IF0                          (0)
  USBD_CDCn_NCM_EP_IF0_HS                         (0)
  USBD_CDCn_NCM_DESC_IF1                          (0)
  USBD_CDCn_NCM_EP_IF1_HS                         (0)
#endif
#endif
#if   (USBD3_CDC1)
#if   (USBD3_CDC1_ACM)
#if   (USBD3_USE_IAD)
  USBD_CDCn_ACM_IAD                               (1,USBD_CDC1_IF0_NUM,2)
#endif
  USBD_CDCn_ACM_DESC_IF0                          (1)
  USBD_CDCn_ACM_EP_IF0_HS                         (1)
  USBD_CDCn_ACM_DESC_IF1                          (1)
  USBD_CDCn_ACM_EP_IF1_HS                         (1)
#elif (USBD3_CDC1_NCM)
#if   (USBD3_USE_IAD)
  USBD_CDCn_NCM_IAD                               (1,USBD_CDC1_IF0_NUM,2)
#endif
  USBD_CDCn_NCM_DESC_IF0                          (1)
  USBD_CDCn_NCM_EP_IF0_HS                         (1)
  USBD_CDCn_NCM_DESC_IF1                          (1)
  USBD_CDCn_NCM_EP_IF1_HS                         (1)
#endif
#endif
#if   (USBD3_CDC2)
#if   (USBD3_CDC2_ACM)
#if   (USBD3_USE_IAD)
  USBD_CDCn_ACM_IAD                               (2,USBD_CDC2_IF0_NUM,2)
#endif
  USBD_CDCn_ACM_DESC_IF0                          (2)
  USBD_CDCn_ACM_EP_IF0_HS                         (2)
  USBD_CDCn_ACM_DESC_IF1                          (2)
  USBD_CDCn_ACM_EP_IF1_HS                         (2)
#elif (USBD3_CDC2_NCM)
#if   (USBD3_USE_IAD)
  USBD_CDCn_NCM_IAD                               (2,USBD_CDC2_IF0_NUM,2)
#endif
  USBD_CDCn_NCM_DESC_IF0                          (2)
  USBD_CDCn_NCM_EP_IF0_HS                         (2)
  USBD_CDCn_NCM_DESC_IF1                          (2)
  USBD_CDCn_NCM_EP_IF1_HS                         (2)
#endif
#endif
#if   (USBD3_CDC3)
#if   (USBD3_CDC3_ACM)
#if   (USBD3_USE_IAD)
  USBD_CDCn_ACM_IAD                               (3,USBD_CDC3_IF0_NUM,2)
#endif
  USBD_CDCn_ACM_DESC_IF0                          (3)
  USBD_CDCn_ACM_EP_IF0_HS                         (3)
  USBD_CDCn_ACM_DESC_IF1                          (3)
  USBD_CDCn_ACM_EP_IF1_HS                         (3)
#elif (USBD3_CDC3_NCM)
#if   (USBD3_USE_IAD)
  USBD_CDCn_NCM_IAD                               (3,USBD_CDC3_IF0_NUM,2)
#endif
  USBD_CDCn_NCM_DESC_IF0                          (3)
  USBD_CDCn_NCM_EP_IF0_HS                         (3)
  USBD_CDCn_NCM_DESC_IF1                          (3)
  USBD_CDCn_NCM_EP_IF1_HS                         (3)
#endif
#endif

#if (USBD3_HID0)
  USBD_HIDn_DESC                                  (0)
#if (USBD_HID0_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                           (0)
#else
  USBD_HIDn_EP_HS                                 (0)
#endif
#endif
#if (USBD3_HID1)
  USBD_HIDn_DESC                                  (1)
#if (USBD_HID1_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                           (1)
#else
  USBD_HIDn_EP_HS                                 (1)
#endif
#endif
#if (USBD3_HID2)
  USBD_HIDn_DESC                                  (2)
#if (USBD_HID2_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                           (2)
#else
  USBD_HIDn_EP_HS                                 (2)
#endif
#endif
#if (USBD3_HID3)
  USBD_HIDn_DESC                                  (3)
#if (USBD_HID3_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                           (3)
#else
  USBD_HIDn_EP_HS                                 (3)
#endif
#endif

#if (USBD3_MSC0)
  USBD_MSCn_DESC                                  (0)
  USBD_MSCn_EP_HS                                 (0)
#endif
#if (USBD3_MSC1)
  USBD_MSCn_DESC                                  (1)
  USBD_MSCn_EP_HS                                 (1)
#endif
#if (USBD3_MSC2)
  USBD_MSCn_DESC                                  (2)
  USBD_MSCn_EP_HS                                 (2)
#endif
#if (USBD3_MSC3)
  USBD_MSCn_DESC                                  (3)
  USBD_MSCn_EP_HS                                 (3)
#endif

/* Terminator */
  0                                     /* bLength */
};

/* USB Device 3 Other Speed Configuration Descriptor (in Full Speed for High Speed) */
/*   All Descriptors (Configuration, Interface, Endpoint, Class, Vendor) */
extern const uint8_t usbd3_other_speed_config_descriptor_fs[];
__WEAK const uint8_t usbd3_other_speed_config_descriptor_fs[] __ALIGNED(4) = {
  /* Configuration 1 */
  USB_CONFIGURATION_DESC_SIZE,          /* bLength */
  USB_OTHER_SPEED_CONFIG_DESCRIPTOR_TYPE, /* bDescriptorType */
  WBVAL(USBD3_WTOTALLENGTH),            /* wTotalLength */
  USBD3_IF_CNT,                         /* bNumInterfaces */
  0x01,                                 /* bConfigurationValue: 0x01 is used to select this configuration */
  0x00,                                 /* iConfiguration: no string to describe this configuration */
  USBD3_CFG_DESC_BMATTRIBUTES,          /* bmAttributes */
  USBD3_CFG_DESC_BMAXPOWER,             /* bMaxPower, device power consumption */

#if (USBD3_CUSTOM_CLASS0)
#if (USBD_CUSTOM_CLASS0_IAD_EN)
  USBD_CUSTOM_CLASSn_IAD                          (0)
#endif
#if (USBD_CUSTOM_CLASS0_IF0_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (0,0)
#if (USBD_CUSTOM_CLASS0_IF0_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,0,0,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF0_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,0,1,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF0_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,0,2,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF0_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,0,3,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF0_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,0,4,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF0_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,0,5,HS)
#endif
#endif
#if (USBD_CUSTOM_CLASS0_IF1_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (0,1)
#if (USBD_CUSTOM_CLASS0_IF1_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,1,0,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF1_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,1,1,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF1_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,1,2,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF1_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,1,3,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF1_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,1,4,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF1_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,1,5,HS)
#endif
#endif
#if (USBD_CUSTOM_CLASS0_IF2_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (0,2)
#if (USBD_CUSTOM_CLASS0_IF2_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,2,0,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF2_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,2,1,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF2_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,2,2,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF2_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,2,3,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF2_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,2,4,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF2_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,2,5,HS)
#endif
#endif
#if (USBD_CUSTOM_CLASS0_IF3_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (0,3)
#if (USBD_CUSTOM_CLASS0_IF3_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,3,0,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF3_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,3,1,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF3_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,3,2,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF3_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,3,3,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF3_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,3,4,HS)
#endif
#if (USBD_CUSTOM_CLASS0_IF3_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,3,5,HS)
#endif
#endif
#endif

#if (USBD3_CUSTOM_CLASS1)
#if (USBD_CUSTOM_CLASS1_IAD_EN)
  USBD_CUSTOM_CLASSn_IAD                          (1)
#endif
#if (USBD_CUSTOM_CLASS1_IF0_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (1,0)
#if (USBD_CUSTOM_CLASS1_IF0_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,0,0,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF0_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,0,1,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF0_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,0,2,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF0_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,0,3,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF0_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,0,4,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF0_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,0,5,HS)
#endif
#endif
#if (USBD_CUSTOM_CLASS1_IF1_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (1,1)
#if (USBD_CUSTOM_CLASS1_IF1_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,1,0,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF1_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,1,1,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF1_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,1,2,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF1_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,1,3,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF1_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,1,4,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF1_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,1,5,HS)
#endif
#endif
#if (USBD_CUSTOM_CLASS1_IF2_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (1,2)
#if (USBD_CUSTOM_CLASS1_IF2_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,2,0,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF2_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,2,1,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF2_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,2,2,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF2_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,2,3,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF2_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,2,4,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF2_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,2,5,HS)
#endif
#endif
#if (USBD_CUSTOM_CLASS1_IF3_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (1,3)
#if (USBD_CUSTOM_CLASS1_IF3_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,3,0,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF3_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,3,1,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF3_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,3,2,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF3_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,3,3,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF3_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,3,4,HS)
#endif
#if (USBD_CUSTOM_CLASS1_IF3_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,3,5,HS)
#endif
#endif
#endif

#if (USBD3_CUSTOM_CLASS2)
#if (USBD_CUSTOM_CLASS2_IAD_EN)
  USBD_CUSTOM_CLASSn_IAD                          (2)
#endif
#if (USBD_CUSTOM_CLASS2_IF0_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (2,0)
#if (USBD_CUSTOM_CLASS2_IF0_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,0,0,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF0_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,0,1,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF0_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,0,2,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF0_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,0,3,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF0_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,0,4,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF0_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,0,5,HS)
#endif
#endif
#if (USBD_CUSTOM_CLASS2_IF1_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (2,1)
#if (USBD_CUSTOM_CLASS2_IF1_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,1,0,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF1_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,1,1,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF1_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,1,2,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF1_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,1,3,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF1_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,1,4,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF1_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,1,5,HS)
#endif
#endif
#if (USBD_CUSTOM_CLASS2_IF2_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (2,2)
#if (USBD_CUSTOM_CLASS2_IF2_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,2,0,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF2_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,2,1,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF2_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,2,2,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF2_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,2,3,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF2_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,2,4,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF2_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,2,5,HS)
#endif
#endif
#if (USBD_CUSTOM_CLASS2_IF3_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (2,3)
#if (USBD_CUSTOM_CLASS2_IF3_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,3,0,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF3_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,3,1,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF3_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,3,2,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF3_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,3,3,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF3_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,3,4,HS)
#endif
#if (USBD_CUSTOM_CLASS2_IF3_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,3,5,HS)
#endif
#endif
#endif


#if (USBD3_CUSTOM_CLASS3)
#if (USBD_CUSTOM_CLASS3_IAD_EN)
  USBD_CUSTOM_CLASSn_IAD                          (3)
#endif
#if (USBD_CUSTOM_CLASS3_IF0_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (3,0)
#if (USBD_CUSTOM_CLASS3_IF0_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,0,0,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF0_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,0,1,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF0_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,0,2,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF0_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,0,3,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF0_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,0,4,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF0_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,0,5,HS)
#endif
#endif
#if (USBD_CUSTOM_CLASS3_IF1_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (3,1)
#if (USBD_CUSTOM_CLASS3_IF1_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,1,0,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF1_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,1,1,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF1_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,1,2,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF1_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,1,3,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF1_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,1,4,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF1_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,1,5,HS)
#endif
#endif
#if (USBD_CUSTOM_CLASS3_IF2_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (3,2)
#if (USBD_CUSTOM_CLASS3_IF2_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,2,0,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF2_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,2,1,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF2_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,2,2,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF2_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,2,3,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF2_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,2,4,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF2_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,2,5,HS)
#endif
#endif
#if (USBD_CUSTOM_CLASS3_IF3_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (3,3)
#if (USBD_CUSTOM_CLASS3_IF3_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,3,0,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF3_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,3,1,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF3_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,3,2,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF3_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,3,3,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF3_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,3,4,HS)
#endif
#if (USBD_CUSTOM_CLASS3_IF3_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,3,5,HS)
#endif
#endif
#endif

#if   (USBD3_CDC0)
#if   (USBD3_CDC0_ACM)
#if   (USBD3_USE_IAD)
  USBD_CDCn_ACM_IAD                               (0,USBD_CDC0_IF0_NUM,2)
#endif
  USBD_CDCn_ACM_DESC_IF0                          (0)
  USBD_CDCn_ACM_EP_IF0_HS                         (0)
  USBD_CDCn_ACM_DESC_IF1                          (0)
  USBD_CDCn_ACM_EP_IF1_HS                         (0)
#elif (USBD3_CDC0_NCM)
#if   (USBD3_USE_IAD)
  USBD_CDCn_NCM_IAD                               (0,USBD_CDC0_IF0_NUM,2)
#endif
  USBD_CDCn_NCM_DESC_IF0                          (0)
  USBD_CDCn_NCM_EP_IF0_HS                         (0)
  USBD_CDCn_NCM_DESC_IF1                          (0)
  USBD_CDCn_NCM_EP_IF1_HS                         (0)
#endif
#endif
#if   (USBD3_CDC1)
#if   (USBD3_CDC1_ACM)
#if   (USBD3_USE_IAD)
  USBD_CDCn_ACM_IAD                               (1,USBD_CDC1_IF0_NUM,2)
#endif
  USBD_CDCn_ACM_DESC_IF0                          (1)
  USBD_CDCn_ACM_EP_IF0_HS                         (1)
  USBD_CDCn_ACM_DESC_IF1                          (1)
  USBD_CDCn_ACM_EP_IF1_HS                         (1)
#elif (USBD3_CDC1_NCM)
#if   (USBD3_USE_IAD)
  USBD_CDCn_NCM_IAD                               (1,USBD_CDC1_IF0_NUM,2)
#endif
  USBD_CDCn_NCM_DESC_IF0                          (1)
  USBD_CDCn_NCM_EP_IF0_HS                         (1)
  USBD_CDCn_NCM_DESC_IF1                          (1)
  USBD_CDCn_NCM_EP_IF1_HS                         (1)
#endif
#endif
#if   (USBD3_CDC2)
#if   (USBD3_CDC2_ACM)
#if   (USBD3_USE_IAD)
  USBD_CDCn_ACM_IAD                               (2,USBD_CDC2_IF0_NUM,2)
#endif
  USBD_CDCn_ACM_DESC_IF0                          (2)
  USBD_CDCn_ACM_EP_IF0_HS                         (2)
  USBD_CDCn_ACM_DESC_IF1                          (2)
  USBD_CDCn_ACM_EP_IF1_HS                         (2)
#elif (USBD3_CDC2_NCM)
#if   (USBD3_USE_IAD)
  USBD_CDCn_NCM_IAD                               (2,USBD_CDC2_IF0_NUM,2)
#endif
  USBD_CDCn_NCM_DESC_IF0                          (2)
  USBD_CDCn_NCM_EP_IF0_HS                         (2)
  USBD_CDCn_NCM_DESC_IF1                          (2)
  USBD_CDCn_NCM_EP_IF1_HS                         (2)
#endif
#endif
#if   (USBD3_CDC3)
#if   (USBD3_CDC3_ACM)
#if   (USBD3_USE_IAD)
  USBD_CDCn_ACM_IAD                               (3,USBD_CDC3_IF0_NUM,2)
#endif
  USBD_CDCn_ACM_DESC_IF0                          (3)
  USBD_CDCn_ACM_EP_IF0_HS                         (3)
  USBD_CDCn_ACM_DESC_IF1                          (3)
  USBD_CDCn_ACM_EP_IF1_HS                         (3)
#elif (USBD3_CDC3_NCM)
#if   (USBD3_USE_IAD)
  USBD_CDCn_NCM_IAD                               (3,USBD_CDC3_IF0_NUM,2)
#endif
  USBD_CDCn_NCM_DESC_IF0                          (3)
  USBD_CDCn_NCM_EP_IF0_HS                         (3)
  USBD_CDCn_NCM_DESC_IF1                          (3)
  USBD_CDCn_NCM_EP_IF1_HS                         (3)
#endif
#endif

#if (USBD3_HID0)
  USBD_HIDn_DESC                                  (0)
#if (USBD_HID0_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                           (0)
#else
  USBD_HIDn_EP_HS                                 (0)
#endif
#endif
#if (USBD3_HID1)
  USBD_HIDn_DESC                                  (1)
#if (USBD_HID1_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                           (1)
#else
  USBD_HIDn_EP_HS                                 (1)
#endif
#endif
#if (USBD3_HID2)
  USBD_HIDn_DESC                                  (2)
#if (USBD_HID2_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                           (2)
#else
  USBD_HIDn_EP_HS                                 (2)
#endif
#endif
#if (USBD3_HID3)
  USBD_HIDn_DESC                                  (3)
#if (USBD_HID3_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                           (3)
#else
  USBD_HIDn_EP_HS                                 (3)
#endif
#endif

#if (USBD3_MSC0)
  USBD_MSCn_DESC                                  (0)
  USBD_MSCn_EP_HS                                 (0)
#endif
#if (USBD3_MSC1)
  USBD_MSCn_DESC                                  (1)
  USBD_MSCn_EP_HS                                 (1)
#endif
#if (USBD3_MSC2)
  USBD_MSCn_DESC                                  (2)
  USBD_MSCn_EP_HS                                 (2)
#endif
#if (USBD3_MSC3)
  USBD_MSCn_DESC                                  (3)
  USBD_MSCn_EP_HS                                 (3)
#endif

/* Terminator */
  0                                     /* bLength */
};

/* USB Device 3 Other Speed Configuration Descriptor (in High Speed for Full Speed) */
/*   All Descriptors (Configuration, Interface, Endpoint, Class, Vendor) */
extern const uint8_t usbd3_other_speed_config_descriptor_hs[];
__WEAK const uint8_t usbd3_other_speed_config_descriptor_hs[] __ALIGNED(4) = {
/* Configuration 1 */
  USB_CONFIGURATION_DESC_SIZE,          /* bLength */
  USB_OTHER_SPEED_CONFIG_DESCRIPTOR_TYPE, /* bDescriptorType */
  WBVAL(USBD3_WTOTALLENGTH),            /* wTotalLength */
  USBD3_IF_CNT,                         /* bNumInterfaces */
  0x01,                                 /* bConfigurationValue: 0x01 is used to select this configuration */
  0x00,                                 /* iConfiguration: no string to describe this configuration */
  USBD3_CFG_DESC_BMATTRIBUTES,          /* bmAttributes */
  USBD3_CFG_DESC_BMAXPOWER,             /* bMaxPower, device power consumption */

#if (USBD3_CUSTOM_CLASS0)
#if (USBD_CUSTOM_CLASS0_IAD_EN)
  USBD_CUSTOM_CLASSn_IAD                          (0)
#endif
#if (USBD_CUSTOM_CLASS0_IF0_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (0,0)
#if (USBD_CUSTOM_CLASS0_IF0_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,0,0,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF0_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,0,1,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF0_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,0,2,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF0_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,0,3,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF0_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,0,4,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF0_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,0,5,FS)
#endif
#endif
#if (USBD_CUSTOM_CLASS0_IF1_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (0,1)
#if (USBD_CUSTOM_CLASS0_IF1_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,1,0,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF1_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,1,1,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF1_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,1,2,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF1_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,1,3,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF1_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,1,4,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF1_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,1,5,FS)
#endif
#endif
#if (USBD_CUSTOM_CLASS0_IF2_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (0,2)
#if (USBD_CUSTOM_CLASS0_IF2_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,2,0,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF2_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,2,1,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF2_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,2,2,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF2_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,2,3,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF2_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,2,4,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF2_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,2,5,FS)
#endif
#endif
#if (USBD_CUSTOM_CLASS0_IF3_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (0,3)
#if (USBD_CUSTOM_CLASS0_IF3_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,3,0,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF3_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,3,1,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF3_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,3,2,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF3_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,3,3,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF3_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,3,4,FS)
#endif
#if (USBD_CUSTOM_CLASS0_IF3_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (0,3,5,FS)
#endif
#endif
#endif

#if (USBD3_CUSTOM_CLASS1)
#if (USBD_CUSTOM_CLASS1_IAD_EN)
  USBD_CUSTOM_CLASSn_IAD                          (1)
#endif
#if (USBD_CUSTOM_CLASS1_IF0_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (1,0)
#if (USBD_CUSTOM_CLASS1_IF0_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,0,0,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF0_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,0,1,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF0_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,0,2,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF0_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,0,3,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF0_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,0,4,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF0_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,0,5,FS)
#endif
#endif
#if (USBD_CUSTOM_CLASS1_IF1_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (1,1)
#if (USBD_CUSTOM_CLASS1_IF1_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,1,0,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF1_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,1,1,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF1_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,1,2,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF1_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,1,3,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF1_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,1,4,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF1_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,1,5,FS)
#endif
#endif
#if (USBD_CUSTOM_CLASS1_IF2_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (1,2)
#if (USBD_CUSTOM_CLASS1_IF2_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,2,0,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF2_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,2,1,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF2_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,2,2,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF2_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,2,3,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF2_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,2,4,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF2_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,2,5,FS)
#endif
#endif
#if (USBD_CUSTOM_CLASS1_IF3_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (1,3)
#if (USBD_CUSTOM_CLASS1_IF3_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,3,0,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF3_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,3,1,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF3_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,3,2,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF3_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,3,3,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF3_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,3,4,FS)
#endif
#if (USBD_CUSTOM_CLASS1_IF3_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (1,3,5,FS)
#endif
#endif
#endif

#if (USBD3_CUSTOM_CLASS2)
#if (USBD_CUSTOM_CLASS2_IAD_EN)
  USBD_CUSTOM_CLASSn_IAD                          (2)
#endif
#if (USBD_CUSTOM_CLASS2_IF0_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (2,0)
#if (USBD_CUSTOM_CLASS2_IF0_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,0,0,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF0_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,0,1,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF0_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,0,2,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF0_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,0,3,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF0_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,0,4,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF0_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,0,5,FS)
#endif
#endif
#if (USBD_CUSTOM_CLASS2_IF1_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (2,1)
#if (USBD_CUSTOM_CLASS2_IF1_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,1,0,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF1_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,1,1,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF1_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,1,2,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF1_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,1,3,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF1_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,1,4,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF1_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,1,5,FS)
#endif
#endif
#if (USBD_CUSTOM_CLASS2_IF2_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (2,2)
#if (USBD_CUSTOM_CLASS2_IF2_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,2,0,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF2_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,2,1,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF2_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,2,2,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF2_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,2,3,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF2_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,2,4,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF2_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,2,5,FS)
#endif
#endif
#if (USBD_CUSTOM_CLASS2_IF3_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (2,3)
#if (USBD_CUSTOM_CLASS2_IF3_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,3,0,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF3_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,3,1,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF3_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,3,2,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF3_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,3,3,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF3_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,3,4,FS)
#endif
#if (USBD_CUSTOM_CLASS2_IF3_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (2,3,5,FS)
#endif
#endif
#endif


#if (USBD3_CUSTOM_CLASS3)
#if (USBD_CUSTOM_CLASS3_IAD_EN)
  USBD_CUSTOM_CLASSn_IAD                          (3)
#endif
#if (USBD_CUSTOM_CLASS3_IF0_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (3,0)
#if (USBD_CUSTOM_CLASS3_IF0_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,0,0,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF0_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,0,1,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF0_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,0,2,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF0_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,0,3,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF0_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,0,4,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF0_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,0,5,FS)
#endif
#endif
#if (USBD_CUSTOM_CLASS3_IF1_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (3,1)
#if (USBD_CUSTOM_CLASS3_IF1_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,1,0,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF1_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,1,1,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF1_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,1,2,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF1_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,1,3,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF1_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,1,4,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF1_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,1,5,FS)
#endif
#endif
#if (USBD_CUSTOM_CLASS3_IF2_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (3,2)
#if (USBD_CUSTOM_CLASS3_IF2_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,2,0,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF2_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,2,1,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF2_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,2,2,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF2_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,2,3,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF2_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,2,4,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF2_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,2,5,FS)
#endif
#endif
#if (USBD_CUSTOM_CLASS3_IF3_EN)
  USBD_CUSTOM_CLASSn_IFx_DESC                     (3,3)
#if (USBD_CUSTOM_CLASS3_IF3_EP0_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,3,0,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF3_EP1_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,3,1,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF3_EP2_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,3,2,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF3_EP3_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,3,3,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF3_EP4_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,3,4,FS)
#endif
#if (USBD_CUSTOM_CLASS3_IF3_EP5_EN)
  USBD_CUSTOM_CLASSn_IFx_EP                       (3,3,5,FS)
#endif
#endif
#endif

#if   (USBD3_CDC0)
#if   (USBD3_CDC0_ACM)
#if   (USBD3_USE_IAD)
  USBD_CDCn_ACM_IAD                               (0,USBD_CDC0_IF0_NUM,2)
#endif
  USBD_CDCn_ACM_DESC_IF0                          (0)
  USBD_CDCn_ACM_EP_IF0                            (0)
  USBD_CDCn_ACM_DESC_IF1                          (0)
  USBD_CDCn_ACM_EP_IF1                            (0)
#elif (USBD3_CDC0_NCM)
#if   (USBD3_USE_IAD)
  USBD_CDCn_NCM_IAD                               (0,USBD_CDC0_IF0_NUM,2)
#endif
  USBD_CDCn_NCM_DESC_IF0                          (0)
  USBD_CDCn_NCM_EP_IF0                            (0)
  USBD_CDCn_NCM_DESC_IF1                          (0)
  USBD_CDCn_NCM_EP_IF1                            (0)
#endif
#endif
#if   (USBD3_CDC1)
#if   (USBD3_CDC1_ACM)
#if   (USBD3_USE_IAD)
  USBD_CDCn_ACM_IAD                               (1,USBD_CDC1_IF0_NUM,2)
#endif
  USBD_CDCn_ACM_DESC_IF0                          (1)
  USBD_CDCn_ACM_EP_IF0                            (1)
  USBD_CDCn_ACM_DESC_IF1                          (1)
  USBD_CDCn_ACM_EP_IF1                            (1)
#elif (USBD3_CDC1_NCM)
#if   (USBD3_USE_IAD)
  USBD_CDCn_NCM_IAD                               (1,USBD_CDC1_IF0_NUM,2)
#endif
  USBD_CDCn_NCM_DESC_IF0                          (1)
  USBD_CDCn_NCM_EP_IF0                            (1)
  USBD_CDCn_NCM_DESC_IF1                          (1)
  USBD_CDCn_NCM_EP_IF1                            (1)
#endif
#endif
#if   (USBD3_CDC2)
#if   (USBD3_CDC2_ACM)
#if   (USBD3_USE_IAD)
  USBD_CDCn_ACM_IAD                               (2,USBD_CDC2_IF0_NUM,2)
#endif
  USBD_CDCn_ACM_DESC_IF0                          (2)
  USBD_CDCn_ACM_EP_IF0                            (2)
  USBD_CDCn_ACM_DESC_IF1                          (2)
  USBD_CDCn_ACM_EP_IF1                            (2)
#elif (USBD3_CDC2_NCM)
#if   (USBD3_USE_IAD)
  USBD_CDCn_NCM_IAD                               (2,USBD_CDC2_IF0_NUM,2)
#endif
  USBD_CDCn_NCM_DESC_IF0                          (2)
  USBD_CDCn_NCM_EP_IF0                            (2)
  USBD_CDCn_NCM_DESC_IF1                          (2)
  USBD_CDCn_NCM_EP_IF1                            (2)
#endif
#endif
#if   (USBD3_CDC3)
#if   (USBD3_CDC3_ACM)
#if   (USBD3_USE_IAD)
  USBD_CDCn_ACM_IAD                               (3,USBD_CDC3_IF0_NUM,2)
#endif
  USBD_CDCn_ACM_DESC_IF0                          (3)
  USBD_CDCn_ACM_EP_IF0                            (3)
  USBD_CDCn_ACM_DESC_IF1                          (3)
  USBD_CDCn_ACM_EP_IF1                            (3)
#elif (USBD3_CDC3_NCM)
#if   (USBD3_USE_IAD)
  USBD_CDCn_NCM_IAD                               (3,USBD_CDC3_IF0_NUM,2)
#endif
  USBD_CDCn_NCM_DESC_IF0                          (3)
  USBD_CDCn_NCM_EP_IF0                            (3)
  USBD_CDCn_NCM_DESC_IF1                          (3)
  USBD_CDCn_NCM_EP_IF1                            (3)
#endif
#endif

#if (USBD3_HID0)
  USBD_HIDn_DESC                                  (0)
#if (USBD_HID0_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                              (0)
#else
  USBD_HIDn_EP                                    (0)
#endif
#endif
#if (USBD3_HID1)
  USBD_HIDn_DESC                                  (1)
#if (USBD_HID1_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                              (1)
#else
  USBD_HIDn_EP                                    (1)
#endif
#endif
#if (USBD3_HID2)
  USBD_HIDn_DESC                                  (2)
#if (USBD_HID2_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                              (2)
#else
  USBD_HIDn_EP                                    (2)
#endif
#endif
#if (USBD3_HID3)
  USBD_HIDn_DESC                                  (3)
#if (USBD_HID3_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                              (3)
#else
  USBD_HIDn_EP                                    (3)
#endif
#endif

#if (USBD3_MSC0)
  USBD_MSCn_DESC                                  (0)
  USBD_MSCn_EP                                    (0)
#endif
#if (USBD3_MSC1)
  USBD_MSCn_DESC                                  (1)
  USBD_MSCn_EP                                    (1)
#endif
#if (USBD3_MSC2)
  USBD_MSCn_DESC                                  (2)
  USBD_MSCn_EP                                    (2)
#endif
#if (USBD3_MSC3)
  USBD_MSCn_DESC                                  (3)
  USBD_MSCn_EP                                    (3)
#endif

/* Terminator */
  0                                     /* bLength */
};

#endif

typedef const __PACKED_STRUCT {
  uint8_t  len;
  uint8_t  type;
  uint16_t langid;
  USBD_STR_DEF(0,  USBD3_STR_DESC_MAN);
  USBD_STR_DEF(1,  USBD3_STR_DESC_PROD);
#if  (USBD3_STR_DESC_SER_EN)
  USBD_STR_DEF(2,  USBD3_STR_DESC_SER);
#endif

#if (USBD3_CUSTOM_CLASS0)
#if ((USBD_CUSTOM_CLASS0_IF0_EN != 0) && (USBD_CUSTOM_CLASS0_IF0_STR_EN != 0))
  USBD_STR_DEF(3,  USBD_CUSTOM_CLASS0_IF0_STR);
#endif
#if ((USBD_CUSTOM_CLASS0_IF1_EN != 0) && (USBD_CUSTOM_CLASS0_IF1_STR_EN != 0))
  USBD_STR_DEF(4,  USBD_CUSTOM_CLASS0_IF1_STR);
#endif
#if ((USBD_CUSTOM_CLASS0_IF2_EN != 0) && (USBD_CUSTOM_CLASS0_IF2_STR_EN != 0))
  USBD_STR_DEF(5,  USBD_CUSTOM_CLASS0_IF2_STR);
#endif
#if ((USBD_CUSTOM_CLASS0_IF3_EN != 0) && (USBD_CUSTOM_CLASS0_IF3_STR_EN != 0))
  USBD_STR_DEF(6,  USBD_CUSTOM_CLASS0_IF3_STR);
#endif
#endif
#if (USBD3_CUSTOM_CLASS1)
#if ((USBD_CUSTOM_CLASS1_IF0_EN != 0) && (USBD_CUSTOM_CLASS1_IF0_STR_EN != 0))
  USBD_STR_DEF(7,  USBD_CUSTOM_CLASS1_IF0_STR);
#endif
#if ((USBD_CUSTOM_CLASS1_IF1_EN != 0) && (USBD_CUSTOM_CLASS1_IF1_STR_EN != 0))
  USBD_STR_DEF(8,  USBD_CUSTOM_CLASS1_IF1_STR);
#endif
#if ((USBD_CUSTOM_CLASS1_IF2_EN != 0) && (USBD_CUSTOM_CLASS1_IF2_STR_EN != 0))
  USBD_STR_DEF(9,  USBD_CUSTOM_CLASS1_IF2_STR);
#endif
#if ((USBD_CUSTOM_CLASS1_IF3_EN != 0) && (USBD_CUSTOM_CLASS1_IF3_STR_EN != 0))
  USBD_STR_DEF(10, USBD_CUSTOM_CLASS1_IF3_STR);
#endif
#endif
#if (USBD3_CUSTOM_CLASS2)
#if ((USBD_CUSTOM_CLASS2_IF0_EN != 0) && (USBD_CUSTOM_CLASS2_IF0_STR_EN != 0))
  USBD_STR_DEF(11, USBD_CUSTOM_CLASS2_IF0_STR);
#endif
#if ((USBD_CUSTOM_CLASS2_IF1_EN != 0) && (USBD_CUSTOM_CLASS2_IF1_STR_EN != 0))
  USBD_STR_DEF(12, USBD_CUSTOM_CLASS2_IF1_STR);
#endif
#if ((USBD_CUSTOM_CLASS2_IF2_EN != 0) && (USBD_CUSTOM_CLASS2_IF2_STR_EN != 0))
  USBD_STR_DEF(13, USBD_CUSTOM_CLASS2_IF2_STR);
#endif
#if ((USBD_CUSTOM_CLASS2_IF3_EN != 0) && (USBD_CUSTOM_CLASS2_IF3_STR_EN != 0))
  USBD_STR_DEF(14, USBD_CUSTOM_CLASS2_IF3_STR);
#endif
#endif
#if (USBD3_CUSTOM_CLASS3)
#if ((USBD_CUSTOM_CLASS3_IF0_EN != 0) && (USBD_CUSTOM_CLASS3_IF0_STR_EN != 0))
  USBD_STR_DEF(15, USBD_CUSTOM_CLASS3_IF0_STR);
#endif
#if ((USBD_CUSTOM_CLASS3_IF1_EN != 0) && (USBD_CUSTOM_CLASS3_IF1_STR_EN != 0))
  USBD_STR_DEF(16, USBD_CUSTOM_CLASS3_IF1_STR);
#endif
#if ((USBD_CUSTOM_CLASS3_IF2_EN != 0) && (USBD_CUSTOM_CLASS3_IF2_STR_EN != 0))
  USBD_STR_DEF(17, USBD_CUSTOM_CLASS3_IF2_STR);
#endif
#if ((USBD_CUSTOM_CLASS3_IF3_EN != 0) && (USBD_CUSTOM_CLASS3_IF3_STR_EN != 0))
  USBD_STR_DEF(18, USBD_CUSTOM_CLASS3_IF3_STR);
#endif
#endif

#if (USBD3_ADC0)
  USBD_STR_DEF(19, USBD_ADC0_CIF_STR_DESC);
#if (USBD_ADC0_IF_SPKR)
  USBD_STR_DEF(20, USBD_ADC0_OUT_SIF_STR0_DESC);
  USBD_STR_DEF(21, USBD_ADC0_OUT_SIF_STR1_DESC);
#endif
#if (USBD_ADC0_IF_MIC)
  USBD_STR_DEF(22, USBD_ADC0_IN_SIF_STR0_DESC);
  USBD_STR_DEF(23, USBD_ADC0_IN_SIF_STR1_DESC);
#endif
#endif
#if (USBD3_ADC1)
  USBD_STR_DEF(24, USBD_ADC1_CIF_STR_DESC);
#if (USBD_ADC1_IF_SPKR)
  USBD_STR_DEF(25, USBD_ADC1_OUT_SIF_STR0_DESC);
  USBD_STR_DEF(26, USBD_ADC1_OUT_SIF_STR1_DESC);
#endif
#if (USBD_ADC1_IF_MIC)
  USBD_STR_DEF(27, USBD_ADC1_IN_SIF_STR0_DESC);
  USBD_STR_DEF(28, USBD_ADC1_IN_SIF_STR1_DESC);
#endif
#endif
#if (USBD3_ADC2)
  USBD_STR_DEF(29, USBD_ADC2_CIF_STR_DESC);
#if (USBD_ADC2_IF_SPKR)
  USBD_STR_DEF(30, USBD_ADC2_OUT_SIF_STR0_DESC);
  USBD_STR_DEF(31, USBD_ADC2_OUT_SIF_STR1_DESC);
#endif
#if (USBD_ADC2_IF_MIC)
  USBD_STR_DEF(32, USBD_ADC2_IN_SIF_STR0_DESC);
  USBD_STR_DEF(33, USBD_ADC2_IN_SIF_STR1_DESC);
#endif
#endif
#if (USBD3_ADC3)
  USBD_STR_DEF(34, USBD_ADC3_CIF_STR_DESC);
#if (USBD_ADC3_IF_SPKR)
  USBD_STR_DEF(35, USBD_ADC3_OUT_SIF_STR0_DESC);
  USBD_STR_DEF(36, USBD_ADC3_OUT_SIF_STR1_DESC);
#endif
#if (USBD_ADC3_IF_MIC)
  USBD_STR_DEF(37, USBD_ADC3_IN_SIF_STR0_DESC);
  USBD_STR_DEF(38, USBD_ADC3_IN_SIF_STR1_DESC);
#endif
#endif

#if (USBD3_CDC0)
  USBD_STR_DEF(39, USBD_CDC0_CIF_STR_DESC);
  USBD_STR_DEF(40, USBD_CDC0_DIF_STR_DESC);
#if (USBD3_CDC0_NCM)
  USBD_STR_DEF(41, USBD_CDC0_NCM_MAC_ADDRESS);
#endif
#endif
#if (USBD3_CDC1)
  USBD_STR_DEF(42, USBD_CDC1_CIF_STR_DESC);
  USBD_STR_DEF(43, USBD_CDC1_DIF_STR_DESC);
#if (USBD3_CDC1_NCM)
  USBD_STR_DEF(44, USBD_CDC1_NCM_MAC_ADDRESS);
#endif
#endif
#if (USBD3_CDC2)
  USBD_STR_DEF(45, USBD_CDC2_CIF_STR_DESC);
  USBD_STR_DEF(46, USBD_CDC2_DIF_STR_DESC);
#if (USBD3_CDC2_NCM)
  USBD_STR_DEF(47, USBD_CDC2_NCM_MAC_ADDRESS);
#endif
#endif
#if (USBD3_CDC3)
  USBD_STR_DEF(48, USBD_CDC3_CIF_STR_DESC);
  USBD_STR_DEF(49, USBD_CDC3_DIF_STR_DESC);
#if (USBD3_CDC3_NCM)
  USBD_STR_DEF(50, USBD_CDC3_NCM_MAC_ADDRESS);
#endif
#endif

#if (USBD3_HID0)
  USBD_STR_DEF(51, USBD_HID0_STR_DESC);
#endif
#if (USBD3_HID1)
  USBD_STR_DEF(52, USBD_HID1_STR_DESC);
#endif
#if (USBD3_HID2)
  USBD_STR_DEF(53, USBD_HID2_STR_DESC);
#endif
#if (USBD3_HID3)
  USBD_STR_DEF(54, USBD_HID3_STR_DESC);
#endif

#if (USBD3_MSC0)
  USBD_STR_DEF(55, USBD_MSC0_STR_DESC);
#endif
#if (USBD3_MSC1)
  USBD_STR_DEF(56, USBD_MSC1_STR_DESC);
#endif
#if (USBD3_MSC2)
  USBD_STR_DEF(57, USBD_MSC2_STR_DESC);
#endif
#if (USBD3_MSC3)
  USBD_STR_DEF(58, USBD_MSC3_STR_DESC);
#endif
  uint8_t bLength0_termination;

} usbd3_string_descriptor_t;

extern usbd3_string_descriptor_t usbd3_string_descriptor;
__WEAK usbd3_string_descriptor_t usbd3_string_descriptor __ALIGNED(4) = {
  4, USB_STRING_DESCRIPTOR_TYPE, USBD3_STR_DESC_LANGID,
  USBD_STR_VAL(USBD3_STR_DESC_MAN),
  USBD_STR_VAL(USBD3_STR_DESC_PROD),
#if (USBD3_STR_DESC_SER_EN)
  USBD_STR_VAL(USBD3_STR_DESC_SER),
#endif

#if (USBD3_CUSTOM_CLASS0)
#if ((USBD_CUSTOM_CLASS0_IF0_EN != 0) && (USBD_CUSTOM_CLASS0_IF0_STR_EN != 0))
  USBD_STR_VAL(USBD_CUSTOM_CLASS0_IF0_STR),
#endif
#if ((USBD_CUSTOM_CLASS0_IF1_EN != 0) && (USBD_CUSTOM_CLASS0_IF1_STR_EN != 0))
  USBD_STR_VAL(USBD_CUSTOM_CLASS0_IF1_STR),
#endif
#if ((USBD_CUSTOM_CLASS0_IF2_EN != 0) && (USBD_CUSTOM_CLASS0_IF2_STR_EN != 0))
  USBD_STR_VAL(USBD_CUSTOM_CLASS0_IF2_STR),
#endif
#if ((USBD_CUSTOM_CLASS0_IF3_EN != 0) && (USBD_CUSTOM_CLASS0_IF3_STR_EN != 0))
  USBD_STR_VAL(USBD_CUSTOM_CLASS0_IF3_STR),
#endif
#endif
#if (USBD3_CUSTOM_CLASS1)
#if ((USBD_CUSTOM_CLASS1_IF0_EN != 0) && (USBD_CUSTOM_CLASS1_IF0_STR_EN != 0))
  USBD_STR_VAL(USBD_CUSTOM_CLASS1_IF0_STR),
#endif
#if ((USBD_CUSTOM_CLASS1_IF1_EN != 0) && (USBD_CUSTOM_CLASS1_IF1_STR_EN != 0))
  USBD_STR_VAL(USBD_CUSTOM_CLASS1_IF1_STR),
#endif
#if ((USBD_CUSTOM_CLASS1_IF2_EN != 0) && (USBD_CUSTOM_CLASS1_IF2_STR_EN != 0))
  USBD_STR_VAL(USBD_CUSTOM_CLASS1_IF2_STR),
#endif
#if ((USBD_CUSTOM_CLASS1_IF3_EN != 0) && (USBD_CUSTOM_CLASS1_IF3_STR_EN != 0))
  USBD_STR_VAL(USBD_CUSTOM_CLASS1_IF3_STR),
#endif
#endif
#if (USBD3_CUSTOM_CLASS2)
#if ((USBD_CUSTOM_CLASS2_IF0_EN != 0) && (USBD_CUSTOM_CLASS2_IF0_STR_EN != 0))
  USBD_STR_VAL(USBD_CUSTOM_CLASS2_IF0_STR),
#endif
#if ((USBD_CUSTOM_CLASS2_IF1_EN != 0) && (USBD_CUSTOM_CLASS2_IF1_STR_EN != 0))
  USBD_STR_VAL(USBD_CUSTOM_CLASS2_IF1_STR),
#endif
#if ((USBD_CUSTOM_CLASS2_IF2_EN != 0) && (USBD_CUSTOM_CLASS2_IF2_STR_EN != 0))
  USBD_STR_VAL(USBD_CUSTOM_CLASS2_IF2_STR),
#endif
#if ((USBD_CUSTOM_CLASS2_IF3_EN != 0) && (USBD_CUSTOM_CLASS2_IF3_STR_EN != 0))
  USBD_STR_VAL(USBD_CUSTOM_CLASS2_IF3_STR),
#endif
#endif
#if (USBD3_CUSTOM_CLASS3)
#if ((USBD_CUSTOM_CLASS3_IF0_EN != 0) && (USBD_CUSTOM_CLASS3_IF0_STR_EN != 0))
  USBD_STR_VAL(USBD_CUSTOM_CLASS3_IF0_STR),
#endif
#if ((USBD_CUSTOM_CLASS3_IF1_EN != 0) && (USBD_CUSTOM_CLASS3_IF1_STR_EN != 0))
  USBD_STR_VAL(USBD_CUSTOM_CLASS3_IF1_STR),
#endif
#if ((USBD_CUSTOM_CLASS3_IF2_EN != 0) && (USBD_CUSTOM_CLASS3_IF2_STR_EN != 0))
  USBD_STR_VAL(USBD_CUSTOM_CLASS3_IF2_STR),
#endif
#if ((USBD_CUSTOM_CLASS3_IF3_EN != 0) && (USBD_CUSTOM_CLASS3_IF3_STR_EN != 0))
  USBD_STR_VAL(USBD_CUSTOM_CLASS3_IF3_STR),
#endif
#endif

#if (USBD3_ADC0)
  USBD_STR_VAL(USBD_ADC0_CIF_STR_DESC),
#if (USBD_ADC0_IF_SPKR)
  USBD_STR_VAL(USBD_ADC0_OUT_SIF_STR0_DESC),
  USBD_STR_VAL(USBD_ADC0_OUT_SIF_STR1_DESC),
#endif
#if (USBD_ADC0_IF_MIC)
  USBD_STR_VAL(USBD_ADC0_IN_SIF_STR0_DESC),
  USBD_STR_VAL(USBD_ADC0_IN_SIF_STR1_DESC),
#endif
#endif
#if (USBD3_ADC1)
  USBD_STR_VAL(USBD_ADC1_CIF_STR_DESC),
#if (USBD_ADC1_IF_SPKR)
  USBD_STR_VAL(USBD_ADC1_OUT_SIF_STR0_DESC),
  USBD_STR_VAL(USBD_ADC1_OUT_SIF_STR1_DESC),
#endif
#if (USBD_ADC1_IF_MIC)
  USBD_STR_VAL(USBD_ADC1_IN_SIF_STR0_DESC),
  USBD_STR_VAL(USBD_ADC1_IN_SIF_STR1_DESC),
#endif
#endif
#if (USBD3_ADC2)
  USBD_STR_VAL(USBD_ADC2_CIF_STR_DESC),
#if (USBD_ADC2_IF_SPKR)
  USBD_STR_VAL(USBD_ADC2_OUT_SIF_STR0_DESC),
  USBD_STR_VAL(USBD_ADC2_OUT_SIF_STR1_DESC),
#endif
#if (USBD_ADC2_IF_MIC)
  USBD_STR_VAL(USBD_ADC2_IN_SIF_STR0_DESC),
  USBD_STR_VAL(USBD_ADC2_IN_SIF_STR1_DESC),
#endif
#endif
#if (USBD3_ADC3)
  USBD_STR_VAL(USBD_ADC3_CIF_STR_DESC),
#if (USBD_ADC3_IF_SPKR)
  USBD_STR_VAL(USBD_ADC3_OUT_SIF_STR0_DESC),
  USBD_STR_VAL(USBD_ADC3_OUT_SIF_STR1_DESC),
#endif
#if (USBD_ADC3_IF_MIC)
  USBD_STR_VAL(USBD_ADC3_IN_SIF_STR0_DESC),
  USBD_STR_VAL(USBD_ADC3_IN_SIF_STR1_DESC),
#endif
#endif

#if (USBD3_CDC0)
  USBD_STR_VAL(USBD_CDC0_CIF_STR_DESC),
  USBD_STR_VAL(USBD_CDC0_DIF_STR_DESC),
#if (USBD3_CDC0_NCM)
  USBD_STR_VAL(USBD_CDC0_NCM_MAC_ADDRESS),
#endif
#endif
#if (USBD3_CDC1)
  USBD_STR_VAL(USBD_CDC1_CIF_STR_DESC),
  USBD_STR_VAL(USBD_CDC1_DIF_STR_DESC),
#if (USBD3_CDC1_NCM)
  USBD_STR_VAL(USBD_CDC1_NCM_MAC_ADDRESS),
#endif
#endif
#if (USBD3_CDC2)
  USBD_STR_VAL(USBD_CDC2_CIF_STR_DESC),
  USBD_STR_VAL(USBD_CDC2_DIF_STR_DESC),
#if (USBD3_CDC2_NCM)
  USBD_STR_VAL(USBD_CDC2_NCM_MAC_ADDRESS),
#endif
#endif
#if (USBD3_CDC3)
  USBD_STR_VAL(USBD_CDC3_CIF_STR_DESC),
  USBD_STR_VAL(USBD_CDC3_DIF_STR_DESC),
#if (USBD3_CDC3_NCM)
  USBD_STR_VAL(USBD_CDC3_NCM_MAC_ADDRESS),
#endif
#endif

#if (USBD3_HID0)
  USBD_STR_VAL(USBD_HID0_STR_DESC),
#endif
#if (USBD3_HID1)
  USBD_STR_VAL(USBD_HID1_STR_DESC),
#endif
#if (USBD3_HID2)
  USBD_STR_VAL(USBD_HID2_STR_DESC),
#endif
#if (USBD3_HID3)
  USBD_STR_VAL(USBD_HID3_STR_DESC),
#endif

#if (USBD3_MSC0)
  USBD_STR_VAL(USBD_MSC0_STR_DESC),
#endif
#if (USBD3_MSC1)
  USBD_STR_VAL(USBD_MSC1_STR_DESC),
#endif
#if (USBD3_MSC2)
  USBD_STR_VAL(USBD_MSC2_STR_DESC),
#endif
#if (USBD3_MSC3)
  USBD_STR_VAL(USBD_MSC3_STR_DESC),
#endif
  0
};

#if ((USBD3_STR_DESC_SER_EN != 0) && (USBD3_STR_DESC_SER_MAX_LEN > 0))
typedef __PACKED_STRUCT {
  uint8_t  bLength;
  uint8_t  bDescriptorType;
  uint16_t bString[USBD3_STR_DESC_SER_MAX_LEN];
} usbd3_ser_num_string_descriptor_t;

static usbd3_ser_num_string_descriptor_t usbd3_ser_num_string_descriptor __ALIGNED(4);
#endif

#if (USBD3_OS_DESC_EN != 0)
// Microsoft OS String Descriptor
static const uint8_t usbd3_ms_os_string_descriptor[18] __ALIGNED(4) = {
  USBD_MS_OS_STR(0)
};

#if (USBD3_CUSTOM_CLASS_MS_COMPAT_CNT != 0)
// Microsoft Extended Compat ID OS Feature Descriptor
typedef const __PACKED_STRUCT {
  USBD_MS_OS_FEAT_EXT_COMPAT_ID_HEADER_DEF
#if  (USBD3_CUSTOM_CLASS0 == 1)
#if ((USBD_CUSTOM_CLASS0_IF0_EN == 1) && (USBD_CUSTOM_CLASS0_IF0_OS_EXT_COMPAT_ID_EN == 1))
  USBD_MS_OS_FEAT_EXT_COMPAT_ID_FUNCTION_DEF(0,0)
#endif
#if ((USBD_CUSTOM_CLASS0_IF1_EN == 1) && (USBD_CUSTOM_CLASS0_IF1_OS_EXT_COMPAT_ID_EN == 1))
  USBD_MS_OS_FEAT_EXT_COMPAT_ID_FUNCTION_DEF(0,1)
#endif
#if ((USBD_CUSTOM_CLASS0_IF2_EN == 1) && (USBD_CUSTOM_CLASS0_IF2_OS_EXT_COMPAT_ID_EN == 1))
  USBD_MS_OS_FEAT_EXT_COMPAT_ID_FUNCTION_DEF(0,2)
#endif
#if ((USBD_CUSTOM_CLASS0_IF3_EN == 1) && (USBD_CUSTOM_CLASS0_IF3_OS_EXT_COMPAT_ID_EN == 1))
  USBD_MS_OS_FEAT_EXT_COMPAT_ID_FUNCTION_DEF(0,3)
#endif
#endif

#if  (USBD3_CUSTOM_CLASS1 == 1)
#if ((USBD_CUSTOM_CLASS1_IF0_EN == 1) && (USBD_CUSTOM_CLASS1_IF0_OS_EXT_COMPAT_ID_EN == 1))
  USBD_MS_OS_FEAT_EXT_COMPAT_ID_FUNCTION_DEF(1,0)
#endif
#if ((USBD_CUSTOM_CLASS1_IF1_EN == 1) && (USBD_CUSTOM_CLASS1_IF1_OS_EXT_COMPAT_ID_EN == 1))
  USBD_MS_OS_FEAT_EXT_COMPAT_ID_FUNCTION_DEF(1,1)
#endif
#if ((USBD_CUSTOM_CLASS1_IF2_EN == 1) && (USBD_CUSTOM_CLASS1_IF2_OS_EXT_COMPAT_ID_EN == 1))
  USBD_MS_OS_FEAT_EXT_COMPAT_ID_FUNCTION_DEF(1,2)
#endif
#if ((USBD_CUSTOM_CLASS1_IF3_EN == 1) && (USBD_CUSTOM_CLASS1_IF3_OS_EXT_COMPAT_ID_EN == 1))
  USBD_MS_OS_FEAT_EXT_COMPAT_ID_FUNCTION_DEF(1,3)
#endif
#endif

#if  (USBD3_CUSTOM_CLASS2 == 1)
#if ((USBD_CUSTOM_CLASS2_IF0_EN == 1) && (USBD_CUSTOM_CLASS2_IF0_OS_EXT_COMPAT_ID_EN == 1))
  USBD_MS_OS_FEAT_EXT_COMPAT_ID_FUNCTION_DEF(2,0)
#endif
#if ((USBD_CUSTOM_CLASS2_IF1_EN == 1) && (USBD_CUSTOM_CLASS2_IF1_OS_EXT_COMPAT_ID_EN == 1))
  USBD_MS_OS_FEAT_EXT_COMPAT_ID_FUNCTION_DEF(2,1)
#endif
#if ((USBD_CUSTOM_CLASS2_IF2_EN == 1) && (USBD_CUSTOM_CLASS2_IF2_OS_EXT_COMPAT_ID_EN == 1))
  USBD_MS_OS_FEAT_EXT_COMPAT_ID_FUNCTION_DEF(2,2)
#endif
#if ((USBD_CUSTOM_CLASS2_IF3_EN == 1) && (USBD_CUSTOM_CLASS2_IF3_OS_EXT_COMPAT_ID_EN == 1))
  USBD_MS_OS_FEAT_EXT_COMPAT_ID_FUNCTION_DEF(2,3)
#endif
#endif

#if  (USBD3_CUSTOM_CLASS3 == 1)
#if ((USBD_CUSTOM_CLASS3_IF0_EN == 1) && (USBD_CUSTOM_CLASS3_IF0_OS_EXT_COMPAT_ID_EN == 1))
  USBD_MS_OS_FEAT_EXT_COMPAT_ID_FUNCTION_DEF(3,0)
#endif
#if ((USBD_CUSTOM_CLASS3_IF1_EN == 1) && (USBD_CUSTOM_CLASS3_IF1_OS_EXT_COMPAT_ID_EN == 1))
  USBD_MS_OS_FEAT_EXT_COMPAT_ID_FUNCTION_DEF(3,1)
#endif
#if ((USBD_CUSTOM_CLASS3_IF2_EN == 1) && (USBD_CUSTOM_CLASS3_IF2_OS_EXT_COMPAT_ID_EN == 1))
  USBD_MS_OS_FEAT_EXT_COMPAT_ID_FUNCTION_DEF(3,2)
#endif
#if ((USBD_CUSTOM_CLASS3_IF3_EN == 1) && (USBD_CUSTOM_CLASS3_IF3_OS_EXT_COMPAT_ID_EN == 1))
  USBD_MS_OS_FEAT_EXT_COMPAT_ID_FUNCTION_DEF(3,3)
#endif
#endif
} usbd3_ms_os_ext_compat_id_descriptor_t;

static const usbd3_ms_os_ext_compat_id_descriptor_t usbd3_ms_os_ext_compat_id_descriptor __ALIGNED(4) = {
  USBD_MS_OS_FEAT_EXT_COMPAT_ID_HEADER_VAL(0, USBD3_CUSTOM_CLASS_MS_COMPAT_CNT)
#if  (USBD3_CUSTOM_CLASS0 == 1)
#if ((USBD_CUSTOM_CLASS0_IF0_EN == 1) && (USBD_CUSTOM_CLASS0_IF0_OS_EXT_COMPAT_ID_EN == 1))
  USBD_MS_OS_FEAT_EXT_COMPAT_ID_FUNCTION_VAL(0,0)
#endif
#if ((USBD_CUSTOM_CLASS0_IF1_EN == 1) && (USBD_CUSTOM_CLASS0_IF1_OS_EXT_COMPAT_ID_EN == 1))
  USBD_MS_OS_FEAT_EXT_COMPAT_ID_FUNCTION_VAL(0,1)
#endif
#if ((USBD_CUSTOM_CLASS0_IF2_EN == 1) && (USBD_CUSTOM_CLASS0_IF2_OS_EXT_COMPAT_ID_EN == 1))
  USBD_MS_OS_FEAT_EXT_COMPAT_ID_FUNCTION_VAL(0,2)
#endif
#if ((USBD_CUSTOM_CLASS0_IF3_EN == 1) && (USBD_CUSTOM_CLASS0_IF3_OS_EXT_COMPAT_ID_EN == 1))
  USBD_MS_OS_FEAT_EXT_COMPAT_ID_FUNCTION_VAL(0,3)
#endif
#endif

#if  (USBD3_CUSTOM_CLASS1 == 1)
#if ((USBD_CUSTOM_CLASS1_IF0_EN == 1) && (USBD_CUSTOM_CLASS1_IF0_OS_EXT_COMPAT_ID_EN == 1))
  USBD_MS_OS_FEAT_EXT_COMPAT_ID_FUNCTION_VAL(1,0)
#endif
#if ((USBD_CUSTOM_CLASS1_IF1_EN == 1) && (USBD_CUSTOM_CLASS1_IF1_OS_EXT_COMPAT_ID_EN == 1))
  USBD_MS_OS_FEAT_EXT_COMPAT_ID_FUNCTION_VAL(1,1)
#endif
#if ((USBD_CUSTOM_CLASS1_IF2_EN == 1) && (USBD_CUSTOM_CLASS1_IF2_OS_EXT_COMPAT_ID_EN == 1))
  USBD_MS_OS_FEAT_EXT_COMPAT_ID_FUNCTION_VAL(1,2)
#endif
#if ((USBD_CUSTOM_CLASS1_IF3_EN == 1) && (USBD_CUSTOM_CLASS1_IF3_OS_EXT_COMPAT_ID_EN == 1))
  USBD_MS_OS_FEAT_EXT_COMPAT_ID_FUNCTION_VAL(1,3)
#endif
#endif

#if  (USBD3_CUSTOM_CLASS2 == 1)
#if ((USBD_CUSTOM_CLASS2_IF0_EN == 1) && (USBD_CUSTOM_CLASS2_IF0_OS_EXT_COMPAT_ID_EN == 1))
  USBD_MS_OS_FEAT_EXT_COMPAT_ID_FUNCTION_VAL(2,0)
#endif
#if ((USBD_CUSTOM_CLASS2_IF1_EN == 1) && (USBD_CUSTOM_CLASS2_IF1_OS_EXT_COMPAT_ID_EN == 1))
  USBD_MS_OS_FEAT_EXT_COMPAT_ID_FUNCTION_VAL(2,1)
#endif
#if ((USBD_CUSTOM_CLASS2_IF2_EN == 1) && (USBD_CUSTOM_CLASS2_IF2_OS_EXT_COMPAT_ID_EN == 1))
  USBD_MS_OS_FEAT_EXT_COMPAT_ID_FUNCTION_VAL(2,2)
#endif
#if ((USBD_CUSTOM_CLASS2_IF3_EN == 1) && (USBD_CUSTOM_CLASS2_IF3_OS_EXT_COMPAT_ID_EN == 1))
  USBD_MS_OS_FEAT_EXT_COMPAT_ID_FUNCTION_VAL(2,3)
#endif
#endif

#if  (USBD3_CUSTOM_CLASS3 == 1)
#if ((USBD_CUSTOM_CLASS3_IF0_EN == 1) && (USBD_CUSTOM_CLASS3_IF0_OS_EXT_COMPAT_ID_EN == 1))
  USBD_MS_OS_FEAT_EXT_COMPAT_ID_FUNCTION_VAL(3,0)
#endif
#if ((USBD_CUSTOM_CLASS3_IF1_EN == 1) && (USBD_CUSTOM_CLASS3_IF1_OS_EXT_COMPAT_ID_EN == 1))
  USBD_MS_OS_FEAT_EXT_COMPAT_ID_FUNCTION_VAL(3,1)
#endif
#if ((USBD_CUSTOM_CLASS3_IF2_EN == 1) && (USBD_CUSTOM_CLASS3_IF2_OS_EXT_COMPAT_ID_EN == 1))
  USBD_MS_OS_FEAT_EXT_COMPAT_ID_FUNCTION_VAL(3,2)
#endif
#if ((USBD_CUSTOM_CLASS3_IF3_EN == 1) && (USBD_CUSTOM_CLASS3_IF3_OS_EXT_COMPAT_ID_EN == 1))
  USBD_MS_OS_FEAT_EXT_COMPAT_ID_FUNCTION_VAL(3,3)
#endif
#endif
};
#endif // (USBD3_CUSTOM_CLASS_MS_CNT != 0)
#endif // (USBD3_OS_DESC_EN != 0)


// Pointers to all descriptors
extern 
usbd_desc_t usbd3_desc;
usbd_desc_t usbd3_desc = {
  usbd3_ep0_descriptor,
  usbd3_device_descriptor,
  usbd3_device_qualifier_fs,
  usbd3_device_qualifier_hs,
  usbd3_config_descriptor_fs,
  usbd3_config_descriptor_hs,
  usbd3_other_speed_config_descriptor_fs,
  usbd3_other_speed_config_descriptor_hs,
  (const uint8_t *)(&usbd3_string_descriptor),
#if ((USBD3_STR_DESC_SER_EN != 0) && (USBD3_STR_DESC_SER_MAX_LEN > 0))
  (uint8_t *)(&usbd3_ser_num_string_descriptor),
#else
  NULL,
#endif
#if (USBD3_OS_DESC_EN != 0)
  usbd3_ms_os_string_descriptor,
#else
  NULL,
#endif
#if ((USBD3_OS_DESC_EN != 0) && (USBD3_CUSTOM_CLASS_MS_COMPAT_CNT != 0))
  (const uint8_t *)(&usbd3_ms_os_ext_compat_id_descriptor),
#else
  NULL,
#endif
};
