/*
 * WinUsbIF.h: WinUSB Interface definitions
 * Copyright: Keil - An ARM Company
 * Version: 1.0.0
 */

#ifndef __WinUsbIF_h__
#define __WinUsbIF_h__

#include "windef.h"
#include "tchar.h"

#undef _MP  // Required for WinDDK USB headers

#include "winusb.h"

#define MAX_DEV_NUM      16                    // Maximum Number of Devices
#define MAX_IF_NUM        8                    // Maximum Number of Interfaces in Device
#define MAX_PIPE_NUM      8                    // Maximum Number of Pipes of each type
#define MAX_STR_SIZE    128                    // Maximum String Size (in UNICODE characters)

typedef struct {
  TCHAR                   DeviceInterfacePath  [MAX_PATH];
  HANDLE                  DeviceInterfaceHandle;
  WINUSB_INTERFACE_HANDLE WinUsbInterfaceHandle;
  TCHAR                   String               [MAX_STR_SIZE];
  UCHAR                   NumOutPipes;
  UCHAR                   NumInPipes;
  ULONG                   EpMask;
  UCHAR                   OutPipeID            [MAX_PIPE_NUM];
  UCHAR                   InPipeID             [MAX_PIPE_NUM];
} WinUsbInterface_t;

typedef struct {
  USHORT                  VID;
  USHORT                  PID;
  TCHAR                   SerialNumber         [MAX_STR_SIZE];
  UCHAR                   NumInterfaces;
  BOOL                    Open;
  TCHAR                   Manufacturer         [MAX_STR_SIZE];
  TCHAR                   Product              [MAX_STR_SIZE];
  UCHAR                   Speed;
  WinUsbInterface_t       Interface            [MAX_IF_NUM];
} WinUsbDevice_t;

extern void  WinUsbIF_Initialize               (void);
extern void  WinUsbIF_Uninitialize             (void);
extern BOOL  WinUsbIF_SetGUID                  (GUID *guid);
extern BOOL  WinUsbIF_GetGUID                  (GUID *guid);
extern int   WinUsbIF_FindDevices              (void);
extern int   WinUsbIF_GetNumOfInterfaces       (UCHAR dev);
extern ULONG WinUsbIF_GetEndpointsMask         (UCHAR dev, UCHAR intf);
extern BOOL  WinUsbIF_GetName                  (UCHAR dev, TCHAR *buf, UCHAR max_len);
extern BOOL  WinUsbIF_GetManufacturerString    (UCHAR dev, TCHAR *buf, UCHAR max_len);
extern BOOL  WinUsbIF_GetProductString         (UCHAR dev, TCHAR *buf, UCHAR max_len);
extern BOOL  WinUsbIF_GetSerialNumberString    (UCHAR dev, TCHAR *buf, UCHAR max_len);
extern BOOL  WinUsbIF_GetInterfaceString       (UCHAR dev, UCHAR intf, TCHAR *buf, UCHAR max_len);
extern BOOL  WinUsbIF_GetVID                   (UCHAR dev, WORD  *vid);
extern BOOL  WinUsbIF_GetPID                   (UCHAR dev, WORD  *pid);
extern BOOL  WinUsbIF_OpenDevice               (UCHAR dev);
extern void  WinUsbIF_CloseDevice              (UCHAR dev);
extern BOOL  WinUsbIF_ControlTransfer          (UCHAR dev, WINUSB_SETUP_PACKET *sp, UCHAR *buf, ULONG sz, ULONG *cnt, OVERLAPPED *ov);
extern BOOL  WinUsbIF_WritePipe                (UCHAR dev, UCHAR intf, UCHAR pipe,  UCHAR *buf, ULONG sz, ULONG *cnt, OVERLAPPED *ov);
extern BOOL  WinUsbIF_ReadPipe                 (UCHAR dev, UCHAR intf, UCHAR pipe,  UCHAR *buf, ULONG sz, ULONG *cnt, OVERLAPPED *ov);
extern BOOL  WinUsbIF_AbortPipe                (UCHAR dev, UCHAR intf, UCHAR pipe,  UCHAR pipe_dir);
extern BOOL  WinUsbIF_FlushPipe                (UCHAR dev, UCHAR intf, UCHAR pipe,  UCHAR pipe_dir);
extern BOOL  WinUsbIF_ResetPipe                (UCHAR dev, UCHAR intf, UCHAR pipe,  UCHAR pipe_dir);
extern BOOL  WinUsbIF_GetOverlappedResult      (UCHAR dev, OVERLAPPED *ov, DWORD *cnt, BOOL wait);

#endif /* __WinUsbIF_h__ */
