
// WinUSB_TestDlg.h : header file
//

#pragma once
#include "afxwin.h"

#include "WinUsbIF.h"
#include "afxeditbrowsectrl.h"

#define STRINGIFY(x) TEXT(#x)
#define TOSTRING(x)  STRINGIFY(x)

// Configuration of maximum transfer sizes
#define MAX_CONTROL_TRANSFER_BUF_SIZE    512
#define MAX_IN_TRANSFER_BUF_SIZE        4096
#define MAX_OUT_TRANSFER_BUF_SIZE       4096

#define ERROR_MSG_INVALID_GUID                          TEXT("Invalid GUID entered!\nReverted to previous GUID!")
#define ERROR_MSG_INVALID_SETUP_PACKET                  TEXT("Setup Packet must contain 8 bytes!")
#define ERROR_MSG_INVALID_CHAR_SETUP_PACKET             TEXT("Setup Packet contains invalid character(s)!")
#define ERROR_MSG_INVALID_BREQUEST                      TEXT("bRequest value is invalid!\nIt must contain 1 byte!")
#define ERROR_MSG_INVALID_CHAR_BREQUEST                 TEXT("bRequest value contains invalid character(s)!")
#define ERROR_MSG_INVALID_WVALUE                        TEXT("wValue value is invalid!")
#define ERROR_MSG_INVALID_WINDEX                        TEXT("wIndex value is invalid!")
#define ERROR_MSG_INVALID_WLENGTH                       TEXT("wLength value is invalid!")
#define ERROR_MSG_DEVICE_OPEN_FAILED                    TEXT("Could not open device!")
#define ERROR_MSG_NO_DEVICE                             TEXT("No device selected!")
#define ERROR_MSG_NO_INTERFACE                          TEXT("No Interface selected!")
#define ERROR_MSG_NO_IN_ENDPOINT                        TEXT("No In Endpoint selected!")
#define ERROR_MSG_NO_OUT_ENDPOINT                       TEXT("No Out Endpoint selected!")
#define ERROR_MSG_TOO_LARGE_WLENGTH                     TEXT("wLength value is too large!\nMaximum value is ")TOSTRING(MAX_CONTROL_TRANSFER_BUF_SIZE)TEXT("!")
#define ERROR_MSG_INVALID_CHAR_CTRL_TRANSFER_DATA       TEXT("Control Transfer data contains invalid character(s)!")
#define ERROR_MSG_TOO_MANY_CTRL_TRANSFER_DATA           TEXT("Too many Control Transfer data bytes!")
#define ERROR_MSG_NOT_ENOUGH_CTRL_TRANSFER_DATA         TEXT("Not enough Control Transfer data bytes!")
#define ERROR_MSG_CTRL_TRANSFER_FAILED                  TEXT("Control Transfer has failed!")
#define ERROR_MSG_NO_CTRL_TRANSFER_FILE                 TEXT("No file for Control Transfer data selected!")
#define ERROR_MSG_NOT_ENOUGH_CTRL_TRANSFER_FILE         TEXT("File does not contain required number of data bytes for Control Transfer!")
#define ERROR_MSG_OPEN_CTRL_TRANSFER_FILE_FAILED        TEXT("Could not open file for Control Transfer data!")
#define ERROR_MSG_TOO_LARGE_IN_TRANSFER_NUM_TO_RECEIVE  TEXT("Number of bytes to receive in In Transfer is too large!")
#define ERROR_MSG_NO_IN_TRANSFER_FILE                   TEXT("No file for In Transfer data selected!")
#define ERROR_MSG_OPEN_IN_TRANSFER_FILE_FAILED          TEXT("Could not open file for In Transfer data!")
#define ERROR_MSG_TOO_MANY_OUT_TRANSFER_DATA            TEXT("Too many Out Transfer data bytes!\nOut Transfer supports only up to ")TOSTRING(MAX_OUT_TRANSFER_BUF_SIZE)TEXT(" data bytes!")
#define ERROR_MSG_NO_OUT_TRANSFER_FILE                  TEXT("No file for Out Transfer data selected!")
#define ERROR_MSG_OPEN_OUT_TRANSFER_FILE_FAILED         TEXT("Could not open file for Out Transfer data!")
#define ERROR_MSG_OUT_TRANSFER_FAILED                   TEXT("Out Transfer has failed!")

typedef struct {
  HANDLE h;             // Thread Handle
  DWORD  id;            // Thread ID
  HANDLE evt;           // Thread event
  HANDLE kill;          // Thread kill
} THREAD;

// CWinUSB_TestDlg dialog
class CWinUSB_TestDlg : public CDialogEx
{
// Construction
public:
  CWinUSB_TestDlg(CWnd* pParent = NULL);  // standard constructor

// Dialog Data
  enum { IDD = IDD_WINUSB_TEST_DIALOG };

  protected:
  virtual void DoDataExchange(CDataExchange* pDX);  // DDX/DDV support


// Implementation
protected:
  // Variables used for application functionality
  UCHAR                 NumberOfDevices;
  UCHAR                 SelectedDevice;
  UCHAR                 SelectedInterface;
  UCHAR                 SelectedEndpointIn;
  UCHAR                 SelectedEndpointOut;
  UCHAR                 InTransferActive;
  ULONG                 InTransferNumToReceive;
  UCHAR                 OutTransferActive;
  ULONG                 OutTransferDataNumToSend;
  ULONG                 OutTransferFileNumToSend;
  WINUSB_SETUP_PACKET   ControlTransferSetupPacket;
  UCHAR                 ControlTransferData[MAX_CONTROL_TRANSFER_BUF_SIZE];
  UCHAR                 ControlTransferFile[MAX_CONTROL_TRANSFER_BUF_SIZE];
  UCHAR                 InTransferData     [MAX_IN_TRANSFER_BUF_SIZE];
  UCHAR                 InTransferFile     [MAX_IN_TRANSFER_BUF_SIZE];
  UCHAR                 OutTransferData    [MAX_OUT_TRANSFER_BUF_SIZE];
  UCHAR                 OutTransferFile    [MAX_OUT_TRANSFER_BUF_SIZE];
  THREAD                InTransferThread;
  THREAD                OutTransferThread;

  // Variables used for controls
  HICON                 m_hIcon;
  CEdit                 m_Edit_GUID;
  CComboBox             m_Combo_Device;
  CStatic               m_Static_Manufacturer;
  CStatic               m_Static_VendorId;
  CStatic               m_Static_ProductId;
  CEdit                 m_Edit_SetupPacket;
  CComboBox             m_Combo_SetupPacketDirection;
  CComboBox             m_Combo_SetupPacketType;
  CComboBox             m_Combo_SetupPacketRecipient;
  CEdit                 m_Edit_SetupPacketbRequest;
  CEdit                 m_Edit_SetupPacketwValue;
  CEdit                 m_Edit_SetupPacketwIndex;
  CEdit                 m_Edit_SetupPacketwLength;
  CEdit                 m_Edit_ControlTransferData;
  CButton               m_Button_ControlTransferData;
  CMFCEditBrowseCtrl    m_Browse_ControlTransferFile;
  CString               m_Browse_ControlTransferFilename;
  CButton               m_Button_ControlTransferFile;
  CComboBox             m_Combo_Interface;
  CStatic               m_Static_InterfaceString;
  CComboBox             m_Combo_EndpointIn;
  CEdit                 m_Edit_InTransferNumToReceive;
  CEdit                 m_Edit_InTransferData;
  CButton               m_Button_InTransferData;
  CMFCEditBrowseCtrl    m_Browse_InTransferFile;
  CString               m_Browse_InTransferFilename;
  CButton               m_Button_InTransferFile;
  CComboBox             m_Combo_EndpointOut;
  CEdit                 m_Edit_OutTransferData;
  CButton               m_Button_OutTransferData;
  CMFCEditBrowseCtrl    m_Browse_OutTransferFile;
  CString               m_Browse_OutTransferFilename;
  CButton               m_Button_OutTransferFile;

  // Functions handling Control's events
  // Generated message map functions
  virtual BOOL OnInitDialog();
  afx_msg void OnClose();
  afx_msg void OnOK();
  afx_msg void OnSysCommand(UINT nID, LPARAM lParam);
  afx_msg void OnPaint();
  afx_msg HCURSOR OnQueryDragIcon();
  afx_msg void OnEnKillfocusEditGUID();
  afx_msg void OnCbnSelchangeComboDevice();
  afx_msg void OnBnClickedButtonDeviceRefresh();
  afx_msg void OnEnKillfocusEditSetupPacket();
  afx_msg void OnCbnSelchangeComboSetupPacketDirection();
  afx_msg void OnCbnSelchangeComboSetupPacketType();
  afx_msg void OnCbnSelchangeComboSetupPacketRecipient();
  afx_msg void OnEnKillfocusEditSetupPacketbRequest();
  afx_msg void OnEnKillfocusEditSetupPacketwValue();
  afx_msg void OnEnKillfocusEditSetupPacketwIndex();
  afx_msg void OnEnKillfocusEditSetupPacketwLength();
  afx_msg void OnEnKillfocusEditControlTransferData();
  afx_msg void OnBnClickedButtonControlTransferData();
  afx_msg void OnEnChangeMfceditbrowseControlTransferFile();
  afx_msg void OnBnClickedButtonControlTransferFile();
  afx_msg void OnCbnSelchangeComboInterface();
  afx_msg void OnCbnSelchangeComboEndpointIn();
  afx_msg void OnEnKillfocusEditInTransferNumToReceive();
  afx_msg void OnBnClickedButtonInTransferData();
  afx_msg void OnEnChangeMfceditbrowseInTransferFile();
  afx_msg void OnBnClickedButtonInTransferFile();
  afx_msg void OnCbnSelchangeComboEndpointOut();
  afx_msg void OnEnKillfocusEditOutTransferData();
  afx_msg void OnBnClickedButtonOutTransferData();
  afx_msg void OnEnChangeMfceditbrowseOutTransferFile();
  afx_msg void OnBnClickedButtonOutTransferFile();
  DECLARE_MESSAGE_MAP()

  // Local auxiliary functions
  void   Initialize            (void);
  void   CloseCurrentDevice    (void);
  LONG   CountHexInCEdit       (CEdit *cedit,              ULONG max_num);
  ULONG  ParseHexFromCEdit     (CEdit *cedit, UCHAR  *buf, ULONG max_numz);
  ULONG  DisplayHexInCEdit     (CEdit *cedit, UCHAR  *buf, ULONG     num);
  BOOL   Check16bitHexInCEdit  (CEdit *cedit);
  USHORT Parse16bitHexFromCEdit(CEdit *cedit);
  void   Display16bitHexInCEdit(CEdit *cedit, USHORT  val);

  // Functions for reception Thread, static function (InTransferThreadFunc_) calling member function (InTransferThreadFunc)
         void InTransferThreadFunc (void);
  static void InTransferThreadFunc_(void *arg);

  // Functions for transmission Thread, static function (OutTransferThreadFunc_) calling member function (OutTransferThreadFunc)
         void OutTransferThreadFunc (void);
  static void OutTransferThreadFunc_(void *arg);
};
