/*********************************************************************
*                SEGGER Microcontroller GmbH & Co. KG                *
*        Solutions for real time microcontroller applications        *
**********************************************************************
*                                                                    *
*        (c) 1996 - 2016  SEGGER Microcontroller GmbH & Co. KG       *
*                                                                    *
*        Internet: www.segger.com    Support:  support@segger.com    *
*                                                                    *
**********************************************************************

** emWin V5.36 - Graphical user interface for embedded applications **
All  Intellectual Property rights  in the Software belongs to  SEGGER.
emWin is protected by  international copyright laws.  Knowledge of the
source code may not be used to write a similar product.  This file may
only be used in accordance with the following terms:

The software has been licensed to  ARM LIMITED whose registered office
is situated at  110 Fulbourn Road,  Cambridge CB1 9NJ,  England solely
for  the  purposes  of  creating  libraries  for  ARM7, ARM9, Cortex-M
series,  and   Cortex-R4   processor-based  devices,  sublicensed  and
distributed as part of the  MDK-ARM  Professional  under the terms and
conditions  of  the   End  User  License  supplied  with  the  MDK-ARM
Professional. 
Full source code is available at: www.segger.com

We appreciate your understanding and fairness.
----------------------------------------------------------------------
Licensing information

Licensor:                 SEGGER Software GmbH
Licensed to:              ARM Ltd, 110 Fulbourn Road, CB1 9NJ Cambridge, UK
Licensed SEGGER software: emWin
License number:           GUI-00181
License model:            LES-SLA-20007, Agreement, effective since October 1st 2011 
Licensed product:         MDK-ARM Professional
Licensed platform:        ARM7/9, Cortex-M/R4
Licensed number of seats: -
----------------------------------------------------------------------
File        : GUIDEMO_ZoomAndRotate.c
Purpose     : Demo of zooming and rotating images
---------------------------END-OF-HEADER------------------------------
*/

#include "GUIDEMO.h"

#if (SHOW_GUIDEMO_ZOOMANDROTATE && GUI_WINSUPPORT && GUI_SUPPORT_MEMDEV)

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
#define MAG                        300
#define TITLE_SIZE                 40
#define MIN_TIME_PER_FRAME_SHIFT   20
#define MIN_TIME_PER_FRAME_ENLARGE 30
#define BORDER_SIZE                4
#define FINAL_STEP                 100

//
// Memory device format in dependence of color format
//
#if (GUI_USE_ARGB)
  #define GUI_COLOR_CONV GUICC_M8888I
#else
  #define GUI_COLOR_CONV GUICC_8888
#endif

/*********************************************************************
*
*       Static const data
*
**********************************************************************
*/
/*********************************************************************
*
*       _bmSeggerLogo50
*/
static GUI_CONST_STORAGE GUI_COLOR _ColorsSeggerLogo50[] = {
#if (GUI_USE_ARGB)
  0xFFFFFFFF, 0xFFFFFFFF, 0xFF231F20, 0xFF989697,
  0xFF6767BE, 0xFF5858B8, 0xFF959494, 0xFF2020A0,
  0xFFD3D3EC, 0xFFC0BFC0, 0xFF929090, 0xFF1E1B1C,
  0xFF464344, 0xFF2323A1, 0xFF2A2AA4, 0xFF3333A8,
  0xFF949393, 0xFFABA9AA, 0xFFD6D6EE, 0xFFFAFAFD,
  0xFFFDFDFD, 0xFFFEFEFF, 0xFF7F7D7E, 0xFF5F5FBB,
  0xFF6464BD, 0xFF868485, 0xFF9F9E9E, 0xFFA3A2A2,
  0xFFC3C2C2, 0xFFFEFEFE, 0xFF1E1B1B, 0xFF242021,
  0xFF272425, 0xFF2F2B2C, 0xFF5C595A, 0xFF6F6C6D,
  0xFF2424A2, 0xFF2828A3, 0xFF2828A4, 0xFF2C2CA5,
  0xFF3939AB, 0xFF3F3FAD, 0xFF4141AE, 0xFF4242AE,
  0xFF4646B0, 0xFF4A4AB2, 0xFF5151B5, 0xFF6A6AC0,
  0xFF7171C3, 0xFF7979C6, 0xFF7E7EC8, 0xFF7F7FC8,
  0xFF939192, 0xFFA6A4A4, 0xFFABA9A9, 0xFFBAB9B9,
  0xFFBBBABA, 0xFFBEBDBD, 0xFF8383CA, 0xFF9090D0,
  0xFF9494D1, 0xFF9595D2, 0xFF9A9AD4, 0xFFA8A8DA,
  0xFFAFAFDD, 0xFFB0B0DD, 0xFFB3B3DF, 0xFFB5B5DF,
  0xFFB8B8E1, 0xFFC0BFBF, 0xFFC9C8C8, 0xFFCECDCD,
  0xFFD4D3D3, 0xFFC4C4E6, 0xFFCCCCE9, 0xFFD2D2EC,
  0xFFD8D8EE, 0xFFD9D9EF, 0xFFDFDFF1, 0xFFE8E8E8,
  0xFFECECEC, 0xFFE4E4F4, 0xFFE6E6F4, 0xFFE6E6F5,
  0xFFEAEAF6, 0xFFF0F0F0, 0xFFF5F5F5, 0xFFF0F0F9,
  0xFFF1F1F9, 0xFFF3F3FA, 0xFFF5F5FB, 0xFFF6F6FB,
  0xFFFBFBFB, 0xFFF8F8FC, 0xFFFCFCFE, 0xFFFDFDFE,
  0xFF080707, 0xFF141314, 0xFF262324, 0xFF272324,
  0xFF282425, 0xFF2A2627, 0xFF2E2A2B, 0xFF333031,
  0xFF343031, 0xFF373334, 0xFF393536, 0xFF3B3738,
  0xFF3C3839, 0xFF3E3B3B, 0xFF3F3C3D, 0xFF403D3E,
  0xFF454142, 0xFF4B4849, 0xFF4C494A, 0xFF4D494A,
  0xFF4E4B4C, 0xFF514E4F, 0xFF535051, 0xFF555253,
  0xFF585455, 0xFF595657, 0xFF5D5A5B, 0xFF5E5B5B,
  0xFF5E5B5C, 0xFF5F5D5D, 0xFF625F5F, 0xFF666364,
  0xFF676666, 0xFF686566, 0xFF6F6E6E, 0xFF737171,
  0xFF747172, 0xFF787576, 0xFF787676, 0xFF797677,
  0xFF797777, 0xFF3C3CAC, 0xFF3D3DAC, 0xFF6565BD,
  0xFF817F80, 0xFF828080, 0xFF848283, 0xFF888686,
  0xFF888687, 0xFF898787, 0xFF8E8B8C, 0xFF8F8D8D,
  0xFF908E8E, 0xFF929191, 0xFF949293, 0xFF959393,
  0xFF979596, 0xFF989696, 0xFF999797, 0xFF9E9C9D,
  0xFF9F9D9E, 0xFFA09E9E, 0xFFA4A2A3, 0xFFA5A4A4,
  0xFFA6A4A5, 0xFFA8A6A6, 0xFFA8A6A7, 0xFFA9A7A8,
  0xFFAAA9A9, 0xFFACAAAA, 0xFFACAAAB, 0xFFADABAC,
  0xFFAEADAD, 0xFFAFAEAE, 0xFFB0AFAF, 0xFFB1AFB0,
  0xFFB1B0B0, 0xFFB5B3B4, 0xFFB6B4B5, 0xFFB7B6B6,
  0xFFBDBCBC, 0xFFBDBCBD, 0xFFBFBEBE, 0xFFAAAADB,
  0xFFACACDC, 0xFFC1C0C0, 0xFFC8C7C8, 0xFFCAC9CA,
  0xFFCDCCCC, 0xFFCFCECE, 0xFFD0CFCF, 0xFFD2D1D1,
  0xFFD7D6D6, 0xFFD9D8D8, 0xFFDAD9D9, 0xFFDADADA,
  0xFFDBDBDB, 0xFFDCDCDC, 0xFFDEDDDE, 0xFFDEDEDE,
  0xFFDBDBF0, 0xFFDCDCF0, 0xFFE2E2E2, 0xFFE4E4E4,
  0xFFE5E4E5, 0xFFE9E9E9, 0xFFEBEAEA, 0xFFEBEBEB,
  0xFFEDECEC, 0xFFEEEEEE, 0xFFEDEDF7, 0xFFEDEDF8,
  0xFFF1F0F0, 0xFFF2F1F1, 0xFFF2F2F2, 0xFFF3F3F3,
  0xFFF4F3F3, 0xFFF4F4F4, 0xFFF7F7F7, 0xFFF8F8F8,
  0xFFF9F9F9, 0xFFFAFAFA, 0xFFFCFCFC
#else
  0xFFFFFF, 0xFFFFFF, 0x201F23, 0x979698,
  0xBE6767, 0xB85858, 0x949495, 0xA02020,
  0xECD3D3, 0xC0BFC0, 0x909092, 0x1C1B1E,
  0x444346, 0xA12323, 0xA42A2A, 0xA83333,
  0x939394, 0xAAA9AB, 0xEED6D6, 0xFDFAFA,
  0xFDFDFD, 0xFFFEFE, 0x7E7D7F, 0xBB5F5F,
  0xBD6464, 0x858486, 0x9E9E9F, 0xA2A2A3,
  0xC2C2C3, 0xFEFEFE, 0x1B1B1E, 0x212024,
  0x252427, 0x2C2B2F, 0x5A595C, 0x6D6C6F,
  0xA22424, 0xA32828, 0xA42828, 0xA52C2C,
  0xAB3939, 0xAD3F3F, 0xAE4141, 0xAE4242,
  0xB04646, 0xB24A4A, 0xB55151, 0xC06A6A,
  0xC37171, 0xC67979, 0xC87E7E, 0xC87F7F,
  0x929193, 0xA4A4A6, 0xA9A9AB, 0xB9B9BA,
  0xBABABB, 0xBDBDBE, 0xCA8383, 0xD09090,
  0xD19494, 0xD29595, 0xD49A9A, 0xDAA8A8,
  0xDDAFAF, 0xDDB0B0, 0xDFB3B3, 0xDFB5B5,
  0xE1B8B8, 0xBFBFC0, 0xC8C8C9, 0xCDCDCE,
  0xD3D3D4, 0xE6C4C4, 0xE9CCCC, 0xECD2D2,
  0xEED8D8, 0xEFD9D9, 0xF1DFDF, 0xE8E8E8,
  0xECECEC, 0xF4E4E4, 0xF4E6E6, 0xF5E6E6,
  0xF6EAEA, 0xF0F0F0, 0xF5F5F5, 0xF9F0F0,
  0xF9F1F1, 0xFAF3F3, 0xFBF5F5, 0xFBF6F6,
  0xFBFBFB, 0xFCF8F8, 0xFEFCFC, 0xFEFDFD,
  0x070708, 0x141314, 0x242326, 0x242327,
  0x252428, 0x27262A, 0x2B2A2E, 0x313033,
  0x313034, 0x343337, 0x363539, 0x38373B,
  0x39383C, 0x3B3B3E, 0x3D3C3F, 0x3E3D40,
  0x424145, 0x49484B, 0x4A494C, 0x4A494D,
  0x4C4B4E, 0x4F4E51, 0x515053, 0x535255,
  0x555458, 0x575659, 0x5B5A5D, 0x5B5B5E,
  0x5C5B5E, 0x5D5D5F, 0x5F5F62, 0x646366,
  0x666667, 0x666568, 0x6E6E6F, 0x717173,
  0x727174, 0x767578, 0x767678, 0x777679,
  0x777779, 0xAC3C3C, 0xAC3D3D, 0xBD6565,
  0x807F81, 0x808082, 0x838284, 0x868688,
  0x878688, 0x878789, 0x8C8B8E, 0x8D8D8F,
  0x8E8E90, 0x919192, 0x939294, 0x939395,
  0x969597, 0x969698, 0x979799, 0x9D9C9E,
  0x9E9D9F, 0x9E9EA0, 0xA3A2A4, 0xA4A4A5,
  0xA5A4A6, 0xA6A6A8, 0xA7A6A8, 0xA8A7A9,
  0xA9A9AA, 0xAAAAAC, 0xABAAAC, 0xACABAD,
  0xADADAE, 0xAEAEAF, 0xAFAFB0, 0xB0AFB1,
  0xB0B0B1, 0xB4B3B5, 0xB5B4B6, 0xB6B6B7,
  0xBCBCBD, 0xBDBCBD, 0xBEBEBF, 0xDBAAAA,
  0xDCACAC, 0xC0C0C1, 0xC8C7C8, 0xCAC9CA,
  0xCCCCCD, 0xCECECF, 0xCFCFD0, 0xD1D1D2,
  0xD6D6D7, 0xD8D8D9, 0xD9D9DA, 0xDADADA,
  0xDBDBDB, 0xDCDCDC, 0xDEDDDE, 0xDEDEDE,
  0xF0DBDB, 0xF0DCDC, 0xE2E2E2, 0xE4E4E4,
  0xE5E4E5, 0xE9E9E9, 0xEAEAEB, 0xEBEBEB,
  0xECECED, 0xEEEEEE, 0xF7EDED, 0xF8EDED,
  0xF0F0F1, 0xF1F1F2, 0xF2F2F2, 0xF3F3F3,
  0xF3F3F4, 0xF4F4F4, 0xF7F7F7, 0xF8F8F8,
  0xF9F9F9, 0xFAFAFA, 0xFCFCFC
#endif
};

static GUI_CONST_STORAGE GUI_LOGPALETTE _PalSeggerLogo50 = {
  219,	// Number of entries
  1, 	// Has transparency
  &_ColorsSeggerLogo50[0]
};

static GUI_CONST_STORAGE unsigned char _acSeggerLogo50[] = {
  0x00, 0x0B, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02,
        0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x0B, 0x00,
  0x1E, 0x0C, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03,
        0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x0C, 0x1E,
  0x02, 0x10, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01,
        0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x10, 0x02,
  0x02, 0x06, 0x5E, 0x42, 0x13, 0x01, 0x01, 0x4B, 0x4C, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01,
        0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x06, 0x02,
  0x02, 0x06, 0x51, 0x0D, 0x33, 0x01, 0x01, 0x2F, 0x0F, 0x53, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01,
        0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x06, 0x02,
  0x02, 0x06, 0x01, 0x3C, 0x07, 0x43, 0x01, 0x52, 0x0F, 0x2E, 0x13, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01,
        0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x06, 0x02,
  0x02, 0x06, 0x01, 0x5F, 0x17, 0x27, 0x4E, 0x01, 0x49, 0x0D, 0x32, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01,
        0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x06, 0x02,
  0x02, 0x06, 0x01, 0x01, 0xCE, 0x28, 0x2D, 0x5D, 0x01, 0x3D, 0x07, 0x41, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01,
        0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x06, 0x02,
  0x02, 0x06, 0x54, 0x5A, 0x01, 0x4A, 0x24, 0x31, 0x01, 0x15, 0x18, 0x0E, 0x4D, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01,
        0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x06, 0x02,
  0x02, 0x06, 0x08, 0x30, 0x01, 0x01, 0x3E, 0x07, 0x40, 0x01, 0x58, 0x29, 0x2A, 0x59, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01,
        0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x06, 0x02,
  0x02, 0x06, 0x08, 0x07, 0x3F, 0x01, 0x15, 0x18, 0x0E, 0xC5, 0x01, 0x12, 0x26, 0x2B, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05,
        0x05, 0x05, 0x05, 0x05, 0x3A, 0x01, 0x06, 0x02,
  0x02, 0x06, 0x08, 0x07, 0x25, 0x12, 0x01, 0x57, 0x89, 0x2C, 0x5B, 0x01, 0x44, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04,
        0x04, 0x04, 0x04, 0x04, 0x3B, 0x01, 0x06, 0x02,
  0x02, 0x06, 0x08, 0x07, 0x07, 0x17, 0x01, 0x01, 0xB4, 0x07, 0xB3, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01,
        0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x06, 0x02,
  0x02, 0x06, 0x08, 0x07, 0x25, 0x12, 0x01, 0x57, 0x8A, 0x2C, 0x5B, 0x01, 0x44, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04,
        0x04, 0x04, 0x04, 0x04, 0x3B, 0x01, 0x06, 0x02,
  0x02, 0x06, 0x08, 0x07, 0x3F, 0x01, 0x15, 0x8B, 0x0E, 0xC4, 0x01, 0x12, 0x26, 0x2B, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05,
        0x05, 0x05, 0x05, 0x05, 0x3A, 0x01, 0x06, 0x02,
  0x02, 0x06, 0x08, 0x30, 0x01, 0x01, 0x3E, 0x07, 0x40, 0x01, 0x58, 0x29, 0x2A, 0x59, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01,
        0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x06, 0x02,
  0x02, 0x06, 0x54, 0x5A, 0x01, 0x4A, 0x24, 0x31, 0x01, 0x15, 0x18, 0x0E, 0x4D, 0x01, 0xBB, 0xA2, 0x37, 0xD8, 0x01, 0xB8, 0x09, 0x09, 0x1C, 0x5C, 0x01, 0x47, 0xA1, 0xB1, 0xD9, 0x01, 0x01, 0xBD, 0xA3, 0xAF, 0xD5, 0x01, 0x4F, 0x09, 0x09, 0x09, 0xC6, 0xD1,
        0xB5, 0x09, 0x1C, 0xCB, 0x01, 0x01, 0x06, 0x02,
  0x02, 0x06, 0x01, 0x01, 0xCF, 0x28, 0x2D, 0x5D, 0x01, 0x3D, 0x07, 0x41, 0x01, 0x38, 0x6D, 0x9E, 0x81, 0x23, 0x14, 0x1F, 0x86, 0x19, 0x91, 0x55, 0x90, 0x70, 0x9A, 0x7C, 0x7A, 0x01, 0x35, 0x6A, 0x97, 0x23, 0x6E, 0x1D, 0x8C, 0x73, 0x19, 0x19, 0x39, 0x9F,
        0x6C, 0x8F, 0x84, 0x21, 0x55, 0x01, 0x06, 0x02,
  0x02, 0x06, 0x01, 0x5F, 0x17, 0x27, 0x4E, 0x01, 0x49, 0x0D, 0x32, 0x01, 0x01, 0xAD, 0x68, 0x94, 0x45, 0xCA, 0xDA, 0x02, 0x92, 0x1A, 0xA9, 0x56, 0x63, 0xBA, 0x1D, 0x4F, 0x47, 0x01, 0x6B, 0xAB, 0x01, 0xCC, 0x46, 0x1D, 0x16, 0x78, 0x1A, 0x9D, 0xC8, 0x1B,
        0x74, 0x48, 0xB2, 0x65, 0x50, 0x01, 0x06, 0x02,
  0x02, 0x06, 0x01, 0x3C, 0x07, 0x43, 0x01, 0x52, 0x0F, 0x2E, 0x13, 0x01, 0x01, 0x14, 0xB7, 0x96, 0x75, 0x6F, 0xD7, 0x02, 0x98, 0x36, 0x37, 0xD0, 0x02, 0xBE, 0xC2, 0x7D, 0x62, 0xC3, 0x67, 0x38, 0xD2, 0x7E, 0x21, 0x45, 0x16, 0x22, 0x36, 0x11, 0xC9, 0x1B,
        0x69, 0x83, 0x76, 0x77, 0x01, 0x01, 0x06, 0x02,
  0x02, 0x06, 0x51, 0x0D, 0x33, 0x01, 0x01, 0x2F, 0x0F, 0x53, 0x01, 0x01, 0x01, 0xAA, 0x72, 0x46, 0xA7, 0x66, 0x56, 0x02, 0x99, 0x11, 0xA5, 0x50, 0x7F, 0x7B, 0x1C, 0x34, 0x1F, 0xC0, 0x8E, 0x71, 0x39, 0x35, 0x64, 0xB0, 0x16, 0x22, 0x11, 0x11, 0xB6, 0x1B,
        0x79, 0x01, 0xBF, 0x02, 0xD4, 0x01, 0x06, 0x02,
  0x02, 0x06, 0x5E, 0x42, 0x13, 0x01, 0x01, 0x4B, 0x4C, 0x01, 0x01, 0x01, 0x01, 0x14, 0xA6, 0x88, 0x8D, 0x48, 0x01, 0x1A, 0x0A, 0x0A, 0x95, 0xC7, 0xD6, 0x9B, 0x85, 0x9C, 0xA0, 0xCD, 0x14, 0xA4, 0x87, 0x34, 0xAC, 0xC1, 0xB9, 0x0A, 0x0A, 0x0A, 0xAE, 0xBC,
        0xA8, 0x01, 0x5C, 0x93, 0xD3, 0x01, 0x06, 0x02,
  0x02, 0x10, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01,
        0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x10, 0x02,
  0x20, 0x0C, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03,
        0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x0C, 0x20,
  0x00, 0x0B, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02,
        0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x0B, 0x00
};

static GUI_CONST_STORAGE GUI_BITMAP _bmSeggerLogo50 = {
  50,                // XSize
  25,                // YSize
  50,                // BytesPerLine
  8,                 // BitsPerPixel
  _acSeggerLogo50,   // Pointer to picture data (indices)
  &_PalSeggerLogo50  // Pointer to palette
};

/*********************************************************************
*
*       bmPlatine_250x200
*/
#if (GUI_USE_ARGB)
static GUI_CONST_STORAGE unsigned long _acPlatine_250x200[] = {
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FEFEFE, 0x1BE8E8E8, 0x17EBEBE9, 0x3DD8D8D6, 0x45D4D4D2, 0x45D4D4D2, 0x45D4D4D2,
        0x39D9D9D7, 0x3DD7D7D5, 0x45D4D4D2, 0x3DD8DAD5, 0x41D8D9D4, 0x3DDADBD6, 0x3DD9DAD5, 0x45D6D5D1, 0x41D8D7D3, 0x35E0DCD9, 0x31E2DEDB, 0x45DCD9D2, 0x39E1DED7, 0x35E3E0D9, 0x3DE0DDD6, 0x45DCD9D2, 0x45DCD9D2, 0x3DE0DDD6, 0x35E3E0D9, 0x45DFDBD2,
        0x3DE3DFD6, 0x41DDDAD3, 0x41DDDAD3, 0x39DDDCD7, 0x41DAD9D4, 0x3DDBDCD6, 0x35DDDED9, 0x48DBD8CF, 0x39E4E1D8, 0x45DADAD2, 0x48D8D8D0, 0x45D7D8D2, 0x3DDADBD5, 0x1BECEDE7, 0x1BEAECE7, 0x00F8F8F8, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF,
        0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00F7F7F7, 0x48CFCFCF, 0xFFBABAB8, 0xFF949492, 0xFF848482, 0xFF858583, 0xFF8A8A88,
        0xFF90908E, 0xFF8A8A88, 0xFF888886, 0xFF959691, 0xFF979893, 0xFF989994, 0xFF989793, 0xFF979692, 0xFF9A9693, 0xFF989491, 0xFF938F8C, 0xFF9A9790, 0xFFA19E97, 0xFFA4A19A, 0xFFA09D96, 0xFF9F9C95, 0xFFA29F98, 0xFF9F9C95, 0xFF99968F, 0xFFA19D94,
        0xFFA5A198, 0xFFA29E95, 0xFFA6A39C, 0xFFA6A39C, 0xFF9D9C97, 0xFF9D9C97, 0xFF94938E, 0xFF99968D, 0xFFA09D94, 0xFF97948B, 0xFF919189, 0xFF8E8E86, 0xFF999A94, 0xFFB3B4AE, 0x72BFC0BA, 0x04F4F4F4, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF,
        0x00FFFFFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFCFC, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x04F4F4F4, 0x6EBCBCBC, 0xFFBDBEB9, 0xFFACADA8, 0xFFAEAFAA, 0xFFABACA7, 0xFFA5A6A1,
        0xFFA2A39E, 0xFF9FA09B, 0xFFADAEA9, 0xFFCACBC6, 0xFFD7D6D2, 0xFFDAD9D5, 0xFFDBDAD6, 0xFFE3DFDC, 0xFFDFDBD8, 0xFFCAC5C2, 0xFFB7B2AF, 0xFFCFCCC7, 0xFFDCD9D4, 0xFFE4E1DC, 0xFFE3E0DB, 0xFFE8E5E0, 0xFFECE9E4, 0xFFDDDAD5, 0xFFC6C3BE, 0xFFD7D0C8,
        0xFFD4CFC9, 0xFFD5D0CA, 0xFFE3DED8, 0xFFE0DDD8, 0xFFD9D6D1, 0xFFD6D5D1, 0xFFC1C0BC, 0xFFBCB8AF, 0xFFBEB9B3, 0xFFBAB7B0, 0xFFB5B2AB, 0xFFA8A7A2, 0xFFADACA7, 0xFFB7B6B2, 0xFFC4C5C0, 0x0BF0F0F0, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FCFCFC, 0x00FFFFFF, 0x00F5F5F5, 0x7AB6B6B6, 0xFFB0B1AC, 0xFFB8B9B4, 0xFFCBCCC7, 0xFFC0C1BC, 0xFFB2B3AE,
        0xFFADAEA9, 0xFFABACA7, 0xFFC0C1BC, 0xFFD5D4CF, 0xFFF4F3EE, 0xFFFFFFFA, 0xFFFFFFFA, 0xFFFFFFFA, 0xFFFFFAF6, 0xFFDDD8D4, 0xFFC6C1BD, 0xFFE6E1DE, 0xFFFBF6F3, 0xFFFFFEFB, 0xFFFFFBF8, 0xFFFFFEFB, 0xFFFFFEFB, 0xFFFFFBF8, 0xFFE6E1DE, 0xFFF1E9E6,
        0xFFEBE3E0, 0xFFEDE8E4, 0xFFFFFEFB, 0xFFFFFEFB, 0xFFFFFEFD, 0xFFFFFEFD, 0xFFECE8E7, 0xFFC4BFBB, 0xFFBFBAB6, 0xFFC3BEBA, 0xFFC1BDBA, 0xFFB5B1AE, 0xFFB4B3B1, 0xFFA8A7A5, 0xFFB3B2B0, 0x0FEDEDED, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFCFC, 0x00FFFFFF,
        0x00FFFFFF, 0x00FCFCFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FBFBFB, 0x00FFFFFF, 0x00F8F8F8, 0x76B8B8B8, 0xFFB0B1AC, 0xFFB6B7B2, 0xFFBDBEB9, 0xFFAAABA6, 0xFFA5A6A1,
        0xFFAAABA6, 0xFFA1A29D, 0xFFADAEA9, 0xFFBAB9B4, 0xFFEBEAE5, 0xFFFFFFFA, 0xFFFFFCF7, 0xFFFFFFFA, 0xFFF2EDE9, 0xFFD0CBC7, 0xFFBFBAB6, 0xFFCAC4C4, 0xFFEFE9E9, 0xFFFFFDFD, 0xFFFFFAFA, 0xFFFDF7F7, 0xFFFFFCFC, 0xFFF5EFEF, 0xFFDAD4D4, 0xFFBDB5B3,
        0xFFBEB6B4, 0xFFD7CFCD, 0xFFFFF9F9, 0xFFFFFDFD, 0xFFFFFBFC, 0xFFFAF6F7, 0xFFCDC9CA, 0xFFAFAAA7, 0xFFA29D9A, 0xFFA8A3A0, 0xFFACA8A7, 0xFFA9A5A4, 0xFFB1AFB0, 0xFFA19FA0, 0xFFB1AFB0, 0x13ECECEC, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFCFC, 0x00FFFFFF,
        0x00FFFFFF, 0x00FCFCFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FBFFFF, 0x00FCFFFF, 0x00FEFFFF, 0x00FCFCFC, 0x00FBFBFB, 0x00FDFFFE, 0x00FBFFFE, 0x00FBFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FDFFFE, 0x00FDFFFE, 0x00FCFEFD,
        0x00FDFFFE, 0x00FDFFFE, 0x00FDFFFE, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FDFFFE, 0x00FEFFFF,
        0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FBFFFD, 0x00FBFFFD, 0x00FBFFFD, 0x00FBFFFD, 0x00FBFFFD,
        0x00FBFFFD, 0x00FBFFFD, 0x00FBFFFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FBFFFE, 0x00FEFFFF, 0x00FEFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FFFBFD,
        0x00FFFDFF, 0x00FEFCFD, 0x00FDFDFD, 0x00FDFDFD, 0x00FEFCFD, 0x00FFFDFF, 0x00FFFBFD, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFCFF, 0x00FFFBFD, 0x00FFFAFC, 0x00FFFEFF, 0x00FFFEFF,
        0x00FEFCFD, 0x00FCFAFB, 0x00FCFCFC, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFBFC, 0x00FFFBFC, 0x00FFFCFD, 0x00FFFDFE, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFDFE,
        0x00FFFDFE, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FEFEFE, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFBFF, 0x00FFFBFE, 0x00FFFDFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFFFF, 0x00F5F7F6, 0x7AB6B8B7, 0xFFBABBB6, 0xFFB3B4AF, 0xFFBBBCB7, 0xFFAAABA6, 0xFF93948F,
        0xFF9A9B96, 0xFFA9AAA5, 0xFFB4B5B0, 0xFFB6B7B1, 0xFFEEEFE9, 0xFFF9FAF4, 0xFFF8F9F3, 0xFFFDFEF8, 0xFFD2D3CD, 0xFFA0A19B, 0xFF94958F, 0xFFA7A3A0, 0xFFACA8A5, 0xFFF4F0ED, 0xFFFFFEFB, 0xFFFAF6F3, 0xFFFFFEFB, 0xFFD2CECB, 0xFF9C9895, 0xFF92938E,
        0xFF92938E, 0xFFA1A29D, 0xFFFFFFFD, 0xFFFBFBF9, 0xFFFDFDFD, 0xFFF9F9F9, 0xFFAFAFAF, 0xFFA7A8A3, 0xFFA0A19C, 0xFF9C9D98, 0xFF9A9B96, 0xFFA0A19C, 0xFFA5A6A1, 0xFFAAABA6, 0xFFB9BAB5, 0x17F0EAEC, 0x00FFFDFF, 0x00FFFDFF, 0x00FFFDFF, 0x00FFFAFC,
        0x00FFFBFD, 0x00FFFDFF, 0x00FFF9FB, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FFFDFE, 0x00FEFCFD, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFDFE, 0x00FEFEFE, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFBFD, 0x00FFFDFF, 0x00FFFDFF, 0x00F9F7F8, 0x00FFFEFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFFFF, 0x00FDFFFE, 0x00FEFEFE, 0x00FFFDFE, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFFFF, 0x00FAFAFA, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFDFE,
        0x00FFFDFE, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FDFDFD, 0x00FDFDFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFCFC, 0x00FDFDFD, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FEFFFD, 0x00FEFFFD, 0x00FEFFFD, 0x00FEFFFD, 0x00FEFFFD,
        0x00FEFFFD, 0x00FEFFFD, 0x00FEFFFD, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FAFEFD, 0x00FBFDFC, 0x00FCFEFD, 0x00FEFEFE, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFEFF,
        0x00FFFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFE, 0x00FEFCFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFFFF,
        0x00FEFFFF, 0x00FDFFFE, 0x00FCFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FBFFFE, 0x00FBFFFE, 0x00FAFEFD,
        0x00FAFEFD, 0x00F9FDFC, 0x00F8FCFB, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FDFDFD, 0x00FCFCFC, 0x00FBFBFB, 0x00FBFFFE,
        0x00FBFFFE, 0x00FBFFFE, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FEFAFB, 0x00FFFEFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFFFF, 0x00FCFFFF, 0x08F2F8F6, 0x89ADB3B1, 0xFFB5B6B1, 0xFFABACA7, 0xFFB7B8B3, 0xFFB0B1AC, 0xFF9B9C97,
        0xFF9A9B96, 0xFFA2A39E, 0xFFA9AAA5, 0xFF979892, 0xFFA8A9A3, 0xFFA3A49E, 0xFFA1A29C, 0xFFA5A6A0, 0xFF9A9B95, 0xFF959690, 0xFF9C9D97, 0xFF8D8C88, 0xFF989793, 0xFFA8A7A3, 0xFFA4A39F, 0xFFA5A4A0, 0xFFAEADA9, 0xFFA9A8A4, 0xFFA6A5A1, 0xFF90918C,
        0xFFA5A6A1, 0xFFA5A6A1, 0xFFBBBBB9, 0xFFB7B7B5, 0xFFB6B6B6, 0xFFAFAFAF, 0xFFA7A7A7, 0xFFA1A29D, 0xFFAAABA6, 0xFFB2B3AE, 0xFFACADA8, 0xFFA8A9A4, 0xFFA9AAA5, 0xFFAFB0AB, 0xFFC1C2BD, 0x0FEDEDED, 0x00FDFDFD, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FCFEFD, 0x00FCFEFD, 0x00FDFFFE, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FCFEFD, 0x00FCFCFC, 0x00FEFEFE, 0x00FFFFFF, 0x00FEFFFF, 0x00FAFEFD, 0x00FBFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFBFC, 0x00FFFBFC, 0x00FFFDFF, 0x00FFFCFE, 0x00FFFDFF, 0x00FFFDFF, 0x00FFFDFF, 0x00FFFDFF, 0x00FFFDFF,
        0x00FFFDFF, 0x00FFFDFF, 0x00FFFDFF, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FEFCFD,
        0x00FEFCFD, 0x00FEFCFD, 0x00FEFCFD, 0x00FEFCFD, 0x00FEFCFD, 0x00FEFCFD, 0x00FEFCFD, 0x00FFFDFE, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FEFCFD, 0x00FFFDFC, 0x00FFFDFC, 0x00FFFDFC, 0x00FFFEFD, 0x00FFFEFD,
        0x00FFFEFD, 0x00FFFEFD, 0x00FFFEFD, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FDFFFE, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFFFF,
        0x00FEFFFF, 0x00FBFDFC, 0x00FDFFFE, 0x00FDFFFE, 0x00FBFDFC, 0x00FEFFFF, 0x00FCFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FBFFFE,
        0x00F9FDFC, 0x00FBFFFE, 0x00FCFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FBFDFC, 0x00FCFEFD, 0x00FCFEFD, 0x00FCFEFD, 0x00FDFFFE, 0x00FDFFFE, 0x00FDFFFE, 0x00FEFFFF, 0x00FCFEFD,
        0x00FCFEFD, 0x00FCFEFD, 0x00FCFEFD, 0x00FCFEFD, 0x00FCFEFD, 0x00FCFEFD, 0x00FCFEFD, 0x00FBFFFE, 0x00FBFFFE, 0x00FBFFFE, 0x00FBFFFE, 0x00FBFFFE, 0x00FBFFFE, 0x00FBFFFE, 0x00FBFFFE, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF,
        0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FBFFFE, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FCFFFF,
        0x00FCFFFF, 0x00FBFFFE, 0x00FBFFFE, 0x00FAFEFD, 0x00F9FDFC, 0x00F8FCFB, 0x00F7FBFA, 0x00FCFEFD, 0x00FEFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FBFFFF, 0x00F8FEFC, 0x0BEFF8F5, 0x91A9B2AF, 0xFFB9BAB5, 0xFFBBBCB7, 0xFFBCBDB8, 0xFFA8A9A4, 0xFF9A9B96,
        0xFFA1A29D, 0xFF9A9B96, 0xFF898A85, 0xFF989994, 0xFF91928D, 0xFF92938E, 0xFF9B9C97, 0xFF959691, 0xFF8D8E89, 0xFF959691, 0xFF92938E, 0xFF9B9C97, 0xFFACADA8, 0xFF9C9D98, 0xFF989994, 0xFF9B9C97, 0xFF8B8C87, 0xFF8F908B, 0xFF9E9F9A, 0xFF949590,
        0xFFA8A9A4, 0xFFA7A8A3, 0xFF8E8E8C, 0xFF9C9C9A, 0xFF9D9D9D, 0xFF838383, 0xFF969696, 0xFFA1A29D, 0xFFABACA7, 0xFFAFB0AB, 0xFFA1A29D, 0xFF9E9F9A, 0xFFA5A6A1, 0xFFA8A9A4, 0xFFB7B8B3, 0x0FEDF1F0, 0x00FBFFFE, 0x00FAFEFD, 0x00FAFEFD, 0x00FCFFFF,
        0x00FAFEFD, 0x00F9FDFC, 0x00F8FCFB, 0x00FDFFFE, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FDFFFE, 0x00FCFEFD, 0x00FEFEFE, 0x00FFFFFF, 0x00FAFCFB, 0x00FBFFFE, 0x00FAFFFE, 0x00F6FFFC, 0x08F2FBF8, 0x08F2FDF9, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFCFC, 0x00FCFAFB, 0x00FFFEFF, 0x00FFFDFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF,
        0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFDFF, 0x00FFFDFF, 0x00FFFDFF, 0x00FFFDFF, 0x00FFFDFF, 0x00FFFCFF, 0x00FFFCFF, 0x00FFFCFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFBFC, 0x00FFFBFC, 0x00FFFBFC, 0x00FFFCFD, 0x00FFFDFE,
        0x00FFFDFE, 0x00FFFDFE, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFBFF, 0x00FEF9FD, 0x00FEF9FD, 0x00FFFCFF, 0x00FFFDFF, 0x00FEFCFF, 0x00FDFBFE, 0x00FEFCFF, 0x00FEFFFF,
        0x00FDFEFF, 0x00FFFDFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFDFF, 0x00FDFEFF, 0x00FEFFFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FEFFFF, 0x00FEFEFE, 0x00FCFCFC, 0x00FCFCFC, 0x00FCFCFC,
        0x00FCFCFC, 0x00FDFBFC, 0x00FDFBFC, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF,
        0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FEFEFE, 0x00FEFEFE,
        0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FEFEFE, 0x00FEFEFE, 0x00FEFEFE, 0x00FEFEFE, 0x00FEFEFE, 0x00FEFEFE, 0x00FFFDFF, 0x00FFFDFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF,
        0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFCFC, 0x00FFFFFF, 0x00FEFFFF, 0x00FAFCFB, 0x00F7F9F8, 0x89AEB0AF, 0xFFABACA7, 0xFFBDBEB9, 0xFFB7B8B3, 0xFF9FA09B, 0xFF9E9F9A,
        0xFFA7A8A3, 0xFFA0A19C, 0xFF949590, 0xFF9D9F9A, 0xFF898B86, 0xFF848681, 0xFF8C8E89, 0xFF82847F, 0xFF81837E, 0xFF8F918C, 0xFF888A85, 0xFF999A95, 0xFF9C9D98, 0xFF858681, 0xFF8C8D88, 0xFF93948F, 0xFF7D7E79, 0xFF7D7E79, 0xFF878883, 0xFF848681,
        0xFF8D8F8A, 0xFFA3A5A0, 0xFF868885, 0xFF8F918E, 0xFF8D8F8E, 0xFF777978, 0xFF828483, 0xFF9C9E99, 0xFFA5A7A2, 0xFFA9ABA6, 0xFF999B96, 0xFF9EA09B, 0xFFACAEA9, 0xFFABADA8, 0xFFB5B7B2, 0x0FEDEDED, 0x00FEFEFE, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFDFE, 0x00FEF8FA, 0x00FFFEFF, 0x00FDFDFD, 0x00FEFFFF, 0x00FBFFFE, 0x00FCFFFF, 0x00FCFFFF, 0x00FEFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFCFF, 0x00FEFAFB, 0x00FEFFFF, 0x00F9FFFF, 0x00F7FFFE, 0x00F9FFFF, 0x00FBFFFF, 0x00F9FDFC, 0x00F8FEFC, 0x00F8FEFC, 0x00F8FEFC, 0x00F8FEFC, 0x00F8FEFC,
        0x00F8FEFC, 0x00F8FEFC, 0x00F8FEFC, 0x00FBFFFF, 0x00FBFFFF, 0x00FAFFFF, 0x00FAFFFF, 0x00FAFFFF, 0x00F9FFFF, 0x00F9FFFF, 0x00F9FFFF, 0x00FBFFFF, 0x00FBFFFF, 0x00FBFFFF, 0x00FBFFFF, 0x00FBFFFF, 0x00FBFFFF, 0x00FBFFFF, 0x00FBFFFF, 0x00F9FFFF,
        0x00F9FFFF, 0x00F9FFFF, 0x00F9FFFF, 0x00F9FFFF, 0x00F9FFFF, 0x00F9FFFF, 0x00F9FFFF, 0x00F9FFFF, 0x00F7FFFD, 0x00F5FEFB, 0x00F6FFFC, 0x00F8FFFE, 0x00F9FFFF, 0x00F9FFFF, 0x00F9FFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF,
        0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FBFFFF, 0x00FBFFFF, 0x00FBFFFF, 0x00FBFFFF, 0x00FBFFFF, 0x00FBFFFF, 0x00FBFFFF, 0x00FBFFFF, 0x00FFFDFF, 0x00FFFDFF, 0x00FFFDFF, 0x00FFFDFF, 0x00FFFEFF, 0x00FDFDFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FAFEFF,
        0x00FCFDFF, 0x00FFFDFF, 0x00FFFDFF, 0x00FFFDFF, 0x00FFFDFF, 0x00FCFDFF, 0x00FAFEFF, 0x00FEFEFF, 0x00FEFEFF, 0x00FEFEFF, 0x00FEFEFF, 0x00FEFEFF, 0x00FEFEFF, 0x00FEFEFF, 0x00FEFEFF, 0x00FDFFFE, 0x00FDFFFE, 0x00FDFFFE, 0x00FFFFFF, 0x00FFFEFF,
        0x00FFFEFF, 0x00FFFEFF, 0x00FFFBFC, 0x00FFFCFD, 0x00FFFCFD, 0x00FFFCFD, 0x00FFFCFD, 0x00FFFCFD, 0x00FFFCFD, 0x00FFFCFD, 0x00FFFCFD, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FFFCFD, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFEFF,
        0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF,
        0x00FFFDFE, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFDFF, 0x00FFFDFF, 0x00FFFDFF, 0x00FFFCFF, 0x00FFFBFF, 0x00FFFBFF, 0x00FFFCFF, 0x00FFFCFF, 0x00FFFDFF,
        0x00FFFDFF, 0x00FFFDFF, 0x00FFFDFF, 0x00FFFCFE, 0x00FFFBFD, 0x00FFFBFD, 0x00FFFAFC, 0x00FFFEFF, 0x00FDFBFC, 0x00FFFBFC, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FCF8F9, 0xA0A5A1A2, 0xFFA5A6A1, 0xFFB8B9B4, 0xFFB2B3AE, 0xFFAAABA6, 0xFFB0B1AC,
        0xFFA8A9A4, 0xFFA8A9A4, 0xFFBDBEB9, 0xFF91938E, 0xFF82847F, 0xFF777974, 0xFF777974, 0xFF72746F, 0xFF777974, 0xFF848681, 0xFF7E807B, 0xFF91938E, 0xFF868883, 0xFF6C6E69, 0xFF70726D, 0xFF858782, 0xFF7D7F7A, 0xFF797B76, 0xFF8B8D88, 0xFF9B9D98,
        0xFF8A8C87, 0xFF959792, 0xFF848683, 0xFF8A8C89, 0xFF858786, 0xFF808281, 0xFF7A7C7B, 0xFF7B7D78, 0xFF8D8F8A, 0xFF9FA19C, 0xFF989A95, 0xFFA1A39E, 0xFFACAEA9, 0xFFA3A5A0, 0x89B0B2AD, 0x13F0ECED, 0x00FFFCFD, 0x00FFFEFF, 0x00FEFAFB, 0x00FEFAFB,
        0x00FFFDFE, 0x00FFFBFC, 0x00FFFCFD, 0x00FFFDFF, 0x00FFFDFF, 0x00FFFDFF, 0x00FFFDFF, 0x00FFFDFF, 0x00FFFDFF, 0x00FFFDFF, 0x00FFFDFF, 0x00FFFAFE, 0x00FFFDFF, 0x00FBF9FA, 0x00FEFFFF, 0x00F7F9F8, 0x00FEFCFD, 0x00FFFAFC, 0x00FFFBFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFBFF, 0x00FFFBFC, 0x00F9FFFF, 0x00D1E4DE, 0x00B0CBC2, 0x00B7D2C9, 0x00B1CBC2, 0x00B3CAC2, 0x00B2C9C1, 0x00B1C8C0, 0x00B0C7BF, 0x00AFC6BE, 0x00AFC6BE,
        0x00AFC6BE, 0x00B0C7BF, 0x00B0C7BF, 0x00B0C6C3, 0x00B0C6C3, 0x00B0C6C3, 0x00B0C6C3, 0x00B0C6C3, 0x00B0C6C3, 0x00B1C7C4, 0x00B1C7C4, 0x00C1D2CC, 0x00C1D2CC, 0x00C2D3CD, 0x00C2D3CD, 0x00C2D3CD, 0x00C1D2CC, 0x00C0D1CB, 0x00BFD0CA, 0x00C1D6CF,
        0x00C1D6CF, 0x00C1D6CF, 0x00C1D6CF, 0x00C1D6CF, 0x00C1D6CF, 0x00C1D6CF, 0x00C1D6CF, 0x00C5DAD3, 0x00C3D8D1, 0x00C1D6CF, 0x00C3D8D1, 0x00C7DCD5, 0x00CBE0D9, 0x00CCE1DA, 0x00CCE1DA, 0x00CFE2DE, 0x00CFE2DE, 0x00CFE2DE, 0x00CFE2DE, 0x00CFE2DE,
        0x00CFE2DE, 0x00CFE2DE, 0x00CFE2DE, 0x00CFE2DC, 0x00CFE2DC, 0x00CFE2DC, 0x00CFE2DC, 0x00CFE2DC, 0x00CFE2DC, 0x00CFE2DC, 0x00CFE2DC, 0x00DBE0E3, 0x00D8E0E2, 0x00DBE5E6, 0x00E2EEEE, 0x00E2F0F0, 0x00DFEFEE, 0x00DDF0EE, 0x00E1F4F2, 0x00E0F4F2,
        0x00E2F0F0, 0x00E6EEF0, 0x00E5E6EA, 0x00E5E6EA, 0x00E6EEF0, 0x00E2F0F0, 0x00E0F4F2, 0x00E2F2F1, 0x00E2F2F1, 0x00E2F2F1, 0x00E2F2F1, 0x00E2F2F1, 0x00E2F2F1, 0x00E2F2F1, 0x00E2F2F1, 0x00EBFCF6, 0x00EBFCF6, 0x00EAF9F4, 0x00EAF6F2, 0x00EDF8F4,
        0x00F0F9F6, 0x00F2F8F6, 0x00EFF5F3, 0x00F1FAF7, 0x00F1FAF7, 0x00F1FAF7, 0x00F1FAF7, 0x00F1FAF7, 0x00F1FAF7, 0x00F1FAF7, 0x00F1FAF7, 0x00F0F9F6, 0x00F0F9F6, 0x00F0F9F6, 0x00F1FAF7, 0x00F3FCF9, 0x00F5FEFB, 0x00F7FFFD, 0x00F9FFFF, 0x00F8FFFE,
        0x00F8FFFE, 0x00F8FFFE, 0x00F8FFFE, 0x00F8FFFE, 0x00F8FFFE, 0x00F8FFFE, 0x00F8FFFE, 0x00FBFFFF, 0x00FBFFFF, 0x00FBFFFF, 0x00FBFFFF, 0x00FBFFFF, 0x00FBFFFF, 0x00FBFFFF, 0x00FBFFFF, 0x00F6FCFA, 0x00F6FCFA, 0x00F6FCFA, 0x00F7FDFB, 0x00F8FEFC,
        0x00FAFFFE, 0x00FBFFFF, 0x00FBFFFF, 0x00FBFFFF, 0x00FBFFFF, 0x00FBFFFF, 0x00FBFFFF, 0x00FBFFFF, 0x00FBFFFF, 0x00FBFFFF, 0x00FBFFFF, 0x00F9FFFF, 0x00F9FFFF, 0x00F9FFFF, 0x00F9FFFF, 0x00F9FFFF, 0x00F9FFFF, 0x00F9FFFF, 0x00F9FFFF, 0x00FCFFFF,
        0x00FCFFFF, 0x00FCFFFF, 0x00FBFFFE, 0x00FBFFFE, 0x00FBFFFE, 0x00FCFFFF, 0x00FCFFFF, 0x00FBFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFEFD, 0x00FDFFFE, 0x00FFFFFF, 0x00F5F5F5, 0x91ABA9AA, 0xFFA3A49F, 0xFFAFB0AB, 0xFFA1A29D, 0xFF9C9D98, 0xFFA7A8A3,
        0xFFA1A29D, 0xFF9C9D98, 0xFFA6A7A2, 0xFF838884, 0xFF808581, 0xFF767B77, 0xFF747975, 0xFF737874, 0xFF767B77, 0xFF7D827E, 0xFF747975, 0xFF7B807A, 0xFF797E78, 0xFF666B65, 0xFF616660, 0xFF7A7F79, 0xFF797E78, 0xFF70756F, 0xFF898E88, 0xFFA6ABA5,
        0xFF8D928C, 0xFF7D827C, 0xFF747975, 0xFF808581, 0xFF7A7E7D, 0xFF7B7F7E, 0xFF767A79, 0xFF757A74, 0xFF7F847E, 0xFF90958F, 0xFF90958F, 0xFFA2A7A1, 0xFFACB1AB, 0xFF9CA19B, 0x91AAAFA9, 0x00F6FAF9, 0x00FBFFFE, 0x00FCFFFF, 0x00FAFEFD, 0x00F9FDFC,
        0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FBFFFE, 0x00FAFEFD, 0x00FAFEFD, 0x00FAFEFD, 0x00FAFEFD, 0x00FAFEFD, 0x00FEFFFF, 0x00F8FEFC, 0x00F5FFFC, 0x00F7FFFF, 0x00F6FFFC, 0x00FFFFFF, 0x00FFF9FC, 0x00FFFAFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFAFF, 0x00FEFCFD, 0x0FEDFCF7, 0xD983A398, 0xFF335D4F, 0xFF316050, 0xFF235242, 0xFF234F40, 0xFF1F493B, 0xFF1E483A, 0xFF1C4638, 0xFF1A4436, 0xFF194335,
        0xFF194335, 0xFF1A4436, 0xFF1B4537, 0xFF1A4239, 0xFF1A4239, 0xFF1C443B, 0xFF1D453C, 0xFF1F473E, 0xFF20483F, 0xFF214940, 0xFF224A41, 0xFF325046, 0xFF345248, 0xFF38564C, 0xFF3B594F, 0xFF3D5B51, 0xFF3E5C52, 0xFF3E5C52, 0xFF3E5C52, 0xFF38594E,
        0xFF38594E, 0xFF38594E, 0xFF38594E, 0xFF38594E, 0xFF38594E, 0xFF38594E, 0xFF38594E, 0xFF38594E, 0xFF36574C, 0xFF35564B, 0xFF37584D, 0xFF3A5B50, 0xFF3C5D52, 0xFF3B5C51, 0xFF395A4F, 0xFF375B51, 0xFF365A50, 0xFF365A50, 0xFF35594F, 0xFF35594F,
        0xFF34584E, 0xFF34584E, 0xFF34584E, 0xFF39594E, 0xFF39594E, 0xFF3B5B50, 0xFF3C5C51, 0xFF3E5E53, 0xFF3F5F54, 0xFF406055, 0xFF416156, 0xFF40615A, 0xFF355851, 0xFF32574F, 0xFF386057, 0xFF3E695F, 0xFF3A685D, 0xFF356559, 0xFF336559, 0xFF346257,
        0xFF3E665D, 0xFF4C6B65, 0xFF49625E, 0xFF49625E, 0xFF4C6B65, 0xFF3E665D, 0xFF346257, 0xFF36665A, 0xFF36665A, 0xFF36665A, 0xFF36665A, 0xFF36665A, 0xFF36665A, 0xFF36665A, 0xFF36665A, 0xFF3D695A, 0xFF456F61, 0xFF497264, 0xFF486E61, 0xFF48695E,
        0xFF4B6B60, 0xFF506E64, 0xFF537167, 0xFF49695E, 0xFF49695E, 0xFF49695E, 0xFF49695E, 0xFF49695E, 0xFF49695E, 0xFF49695E, 0xFF49695E, 0xFF49695E, 0xFF49695E, 0xFF49695E, 0xFF4B6B60, 0xFF4D6D62, 0xFF517166, 0xFF547469, 0xFF56766B, 0xFF537368,
        0xFF537368, 0xFF537368, 0xFF537368, 0xFF537368, 0xFF537368, 0xFF537368, 0xFF537368, 0xFF547268, 0xFF547268, 0xFF547268, 0xFF547268, 0xFF547268, 0xFF547268, 0xFF547268, 0xFF547268, 0xFF537167, 0xFF537167, 0xFF547268, 0xFF557369, 0xFF57756B,
        0xFF5A786E, 0xFF5D7B71, 0xFF5F7D73, 0xFF607E74, 0xFF5F7D73, 0xFF5F7D73, 0xFF5E7C72, 0xFF5E7C72, 0xFF5D7B71, 0xFF5C7A70, 0xFF5C7A70, 0xFF597872, 0xFF5A7973, 0xFF5B7A74, 0xFF5B7A74, 0xFF5A7973, 0xFF587771, 0xFF56756F, 0xFF55746E, 0xFF567067,
        0xFF567067, 0xFF567067, 0xFF587269, 0xFF5A746B, 0xFF5E786F, 0xFF627C73, 0xFF647E75, 0xFF668077, 0xFF69837A, 0xFF6E857D, 0xFF667B74, 0xFF6B7E78, 0xFF6B7C76, 0xFB71827C, 0xFF6E7D78, 0xFF989994, 0xFF999A95, 0xFF868782, 0xFF7C7D78, 0xFF81827D,
        0xFF888984, 0xFF868782, 0xFF73746F, 0xFF727773, 0xFF6E736F, 0xFF616662, 0xFF5D625E, 0xFF5D625E, 0xFF656A66, 0xFF737874, 0xFF717672, 0xFF636A63, 0xFF6A716A, 0xFF616861, 0xFF5C635C, 0xFF666D66, 0xFF6B726B, 0xFF6F766F, 0xFF798079, 0xFF727771,
        0xFF70756F, 0xFF696E68, 0xFF666B67, 0xFF656A66, 0xFF5C605F, 0xFF626665, 0xFF6C706F, 0xFF70756F, 0xFF6A6F69, 0xFF717670, 0xFF747973, 0xFF8D928C, 0xFF9BA09A, 0xFF8E938D, 0x98A7ACA6, 0xCE89A098, 0xEC799088, 0xE87C938B, 0xE180978F, 0xDD829991,
        0xCA8CA39B, 0xC28FA69E, 0xBE91A8A0, 0xC68EA9A0, 0xC28FAAA1, 0xBE92ADA4, 0xBA93AEA5, 0xBA94AFA6, 0xBA94AFA6, 0xBA93AEA5, 0xBE92ADA4, 0xB398B3AA, 0xB796B6AB, 0xB796BAAE, 0xB397BBAF, 0x89AEC9C0, 0x00F5FFFF, 0x00FFFFFF, 0x00FFFAFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFF8FD, 0x00FFFEFF, 0x08F2FFFF, 0xEC799F92, 0xFF225745, 0xFF2F6A56, 0xFF1E5945, 0xFF17503D, 0xFF1B523F, 0xFF19503D, 0xFF154C39, 0xFF134A37, 0xFF114835,
        0xFF124936, 0xFF144B38, 0xFF154C39, 0xFF1A4C40, 0xFF1B4D41, 0xFF1D4F43, 0xFF1F5145, 0xFF225448, 0xFF25574B, 0xFF27594D, 0xFF285A4E, 0xFF3A6053, 0xFF3E6457, 0xFF456B5E, 0xFF4B7164, 0xFF51776A, 0xFF557B6E, 0xFF567C6F, 0xFF577D70, 0xFF537D6F,
        0xFF537D6F, 0xFF537D6F, 0xFF537D6F, 0xFF537D6F, 0xFF537D6F, 0xFF537D6F, 0xFF537D6F, 0xFF507A6C, 0xFF4E786A, 0xFF4D7769, 0xFF4E786A, 0xFF4F796B, 0xFF4D7769, 0xFF487264, 0xFF446E60, 0xFF3B695C, 0xFF3B695C, 0xFF3A685B, 0xFF39675A, 0xFF386659,
        0xFF376558, 0xFF376558, 0xFF366457, 0xFF416A5C, 0xFF426B5D, 0xFF446D5F, 0xFF466F61, 0xFF497264, 0xFF4C7567, 0xFF4E7769, 0xFF4F786A, 0xFF437C6D, 0xFF2A6455, 0xFF135040, 0xFF11503F, 0xFF145745, 0xFF145946, 0xFF0F5844, 0xFF0E5743, 0xFF105542,
        0xFF286555, 0xFF487C6F, 0xFF517C72, 0xFF517C72, 0xFF487C6F, 0xFF286555, 0xFF105542, 0xFF125A46, 0xFF125A46, 0xFF125A46, 0xFF125A46, 0xFF125A46, 0xFF125A46, 0xFF125A46, 0xFF125A46, 0xFF165741, 0xFF2B6A55, 0xFF407E69, 0xFF47826E, 0xFF467D6A,
        0xFF457A68, 0xFF4C7C6C, 0xFF4F7F6F, 0xFF467B69, 0xFF467B69, 0xFF467B69, 0xFF467B69, 0xFF467B69, 0xFF467B69, 0xFF467B69, 0xFF467B69, 0xFF477C6A, 0xFF467B69, 0xFF447967, 0xFF437866, 0xFF427765, 0xFF437866, 0xFF447967, 0xFF447967, 0xFF427765,
        0xFF427765, 0xFF427765, 0xFF427765, 0xFF427765, 0xFF427765, 0xFF427765, 0xFF427765, 0xFF407362, 0xFF407362, 0xFF407362, 0xFF407362, 0xFF407362, 0xFF407362, 0xFF407362, 0xFF407362, 0xFF417463, 0xFF407362, 0xFF3E7160, 0xFF3C6F5E, 0xFF3B6E5D,
        0xFF3C6F5E, 0xFF3D705F, 0xFF3D705F, 0xFF3B6E5D, 0xFF3B6E5D, 0xFF396C5B, 0xFF386B5A, 0xFF366958, 0xFF356857, 0xFF336655, 0xFF336655, 0xFF306457, 0xFF316558, 0xFF306457, 0xFF2D6154, 0xFF275B4E, 0xFF205447, 0xFF184C3F, 0xFF14483B, 0xFF124131,
        0xFF103F2F, 0xFF0C3B2B, 0xFF093828, 0xFF073626, 0xFF063525, 0xFF073626, 0xFF083727, 0xFF093325, 0xFF093325, 0xFF0E3729, 0xFF0D3326, 0xFF15392D, 0xFF0C2D22, 0xFF345449, 0xFF819F95, 0xFFA7A8A3, 0xFF979893, 0xFF898A85, 0xFF82837E, 0xFF70716C,
        0xFF6D6E69, 0xFF787974, 0xFF696A65, 0xFF686D69, 0xFF686D69, 0xFF686D69, 0xFF6D726E, 0xFF686D69, 0xFF646965, 0xFF686D69, 0xFF606561, 0xFF535A53, 0xFF575E57, 0xFF5C635C, 0xFF5E655E, 0xFF4E554E, 0xFF535A53, 0xFF646B64, 0xFF4E554E, 0xFF5B605A,
        0xFF5A5F59, 0xFF5D625C, 0xFF666B67, 0xFF666B67, 0xFF666A69, 0xFF606463, 0xFF616564, 0xFF646963, 0xFF5F645E, 0xFF6B706A, 0xFF6E736D, 0xFF848983, 0xFF919690, 0xFF90958F, 0xFFBABFB9, 0xFF2D5648, 0xFF083123, 0xFF042D1F, 0xFF0A3325, 0xFF093224,
        0xFF0E3729, 0xFF0F382A, 0xFF113A2C, 0xFF0D3D2D, 0xFF124232, 0xFF194939, 0xFF215141, 0xFF275747, 0xFF295949, 0xFF2A5A4A, 0xFF2A5A4A, 0xFF275747, 0xFF2E6351, 0xFF2A614E, 0xFF1A4F3D, 0xFF4B7768, 0x00D8F3EA, 0x00F8FFFF, 0x00FCFCFC, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FBFFFF, 0x00FFFCFF, 0x00FFFFFA, 0xE47E9C90, 0xFF295C53, 0xFF2A5C59, 0xFF1C4F48, 0xFF13503E, 0xFF1E4B37, 0xFF1C4935, 0xFF194431, 0xFF17402E, 0xFF173E2C,
        0xFF193B2B, 0xFF193B2B, 0xFF1A3A2B, 0xFF18422A, 0xFF1A4730, 0xFF194A37, 0xFF184B3C, 0xFF1A4E41, 0xFF1F5347, 0xFF22564A, 0xFF24564A, 0xFF29654C, 0xFF3C6756, 0xFF4C655F, 0xFF556B68, 0xFF567671, 0xFF4D766E, 0xFF4A7269, 0xFF55766D, 0xFF55796B,
        0xFF55796B, 0xFF55796B, 0xFF547A6B, 0xFF557B6C, 0xFF537C6C, 0xFF527B6B, 0xFF527B6B, 0xFF4A7965, 0xFF4C7766, 0xFF4F7365, 0xFF517164, 0xFF507065, 0xFF4A6E62, 0xFF416B5F, 0xFF3A685B, 0xFF2D6558, 0xFF2B6356, 0xFF286053, 0xFF286053, 0xFF296154,
        0xFF2A6255, 0xFF296154, 0xFF286053, 0xFF296657, 0xFF2E6758, 0xFF35675B, 0xFF3F6A60, 0xFF486E63, 0xFF4D7167, 0xFF527368, 0xFF537469, 0xFF5D7D70, 0xFF33594A, 0xFF204C3B, 0xFF1F523F, 0xFF265B47, 0xFF255643, 0xFF396453, 0xFF3E6757, 0xFF154C37,
        0xFF2E5F4C, 0xFF568170, 0xFF5E8274, 0xFF577B6D, 0xFF5A8071, 0xFF2D5948, 0xFF1E4D3B, 0xFF275C4A, 0xFF215644, 0xFF275C4A, 0xFF346957, 0xFF2A5F4D, 0xFF235846, 0xFF2F6452, 0xFF306553, 0xFF144F3B, 0xFF386D5B, 0xFF5A8677, 0xFF597F72, 0xFF587C70,
        0xFF5D8175, 0xFF537D6F, 0xFF578374, 0xFF5E8276, 0xFF597D71, 0xFF597F72, 0xFF517A6C, 0xFF568273, 0xFF4F7E6E, 0xFF528272, 0xFF4F7F6F, 0xFF558172, 0xFF568273, 0xFF558172, 0xFF527E6F, 0xFF537F70, 0xFF578374, 0xFF598576, 0xFF578374, 0xFF588274,
        0xFF578173, 0xFF557F71, 0xFF547E70, 0xFF547E70, 0xFF557F71, 0xFF568072, 0xFF578173, 0xFF4F7B6C, 0xFF518070, 0xFF538272, 0xFF518171, 0xFF4F8271, 0xFF508573, 0xFF4E8371, 0xFF4A7F6D, 0xFF567F6F, 0xFF578070, 0xFF598272, 0xFF5A8373, 0xFF588171,
        0xFF567F6F, 0xFF557E6E, 0xFF557E6E, 0xFF537C6E, 0xFF547D6F, 0xFF557E70, 0xFF547D6F, 0xFF50796B, 0xFF4E7769, 0xFF4D7668, 0xFF4D7668, 0xFF4A7466, 0xFF4A7466, 0xFF467062, 0xFF3F685A, 0xFF446D5F, 0xFF40695B, 0xFF2A5043, 0xFF22483B, 0xFF1B4136,
        0xFF1F4339, 0xFF1B392F, 0xFF284239, 0xFF253C32, 0xFF163025, 0xFF223E32, 0xFF224132, 0xFF1C4032, 0xFF1C3E30, 0xFF203E32, 0xFF284237, 0xFF2B4037, 0xFF3D4E46, 0xFF67766F, 0xFFACB9B2, 0xFFB8B5AE, 0xFF9E9B94, 0xFF87877F, 0xFF6E6E66, 0xFF5F6058,
        0xFF63645C, 0xFF60635A, 0xFF62655C, 0xFF5E655E, 0xFF596059, 0xFF5C635C, 0xFF60655F, 0xFF5E635D, 0xFF636560, 0xFF646661, 0xFF5E605B, 0xFF5C625E, 0xFF5C625E, 0xFF5D635F, 0xFF595F5B, 0xFF616763, 0xFF585E5A, 0xFF5A605C, 0xFF5E6460, 0xFF5D6661,
        0xFF5E6762, 0xFF616A67, 0xFF5A6463, 0xFF5F696A, 0xFF636E70, 0xFF5A6569, 0xFF5E6C6F, 0xFF5E6462, 0xFF626866, 0xFF676B6A, 0xFF707271, 0xFF7B797A, 0xFF898586, 0xFFABA5A7, 0xFFD4CBCE, 0xFF6E8478, 0xFF1C3427, 0xFF122C1F, 0xFF1C3B2C, 0xFF0C2E1E,
        0xFF0C3321, 0xFF18412F, 0xFF0F3A27, 0xFF1D3D2E, 0xFF224233, 0xFF375748, 0xFF4E6E5F, 0xFF587869, 0xFF567667, 0xFF547465, 0xFF587869, 0xFF54756A, 0xFF51776A, 0xFF517B6D, 0xFF447061, 0xFF496D61, 0x00E6F9F3, 0x00FEFFFF, 0x00FFFCFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F9FBFF, 0x00FFFAFF, 0x00FFFBF9, 0xD585968C, 0xFF2F544C, 0xFF315352, 0xFF22453E, 0xFF15402D, 0xFF1C3D1E, 0xFF1B3C1D, 0xFF1B3A1B, 0xFF1B381A, 0xFF1C3619,
        0xFF1C3619, 0xFF1D3619, 0xFF1E371A, 0xFF16311E, 0xFF1C3927, 0xFF1D3D2E, 0xFF1C4034, 0xFF1D453A, 0xFF204A40, 0xFF234E44, 0xFF285349, 0xFF2E644C, 0xFF3B6551, 0xFF4B6558, 0xFF536D62, 0xFF537567, 0xFF4E7563, 0xFF4E7362, 0xFF597566, 0xFF537769,
        0xFF537769, 0xFF527869, 0xFF527869, 0xFF527869, 0xFF507969, 0xFF4F7868, 0xFF4F7868, 0xFF497564, 0xFF4A7363, 0xFF4C7062, 0xFF4D6E63, 0xFF496D61, 0xFF416B5F, 0xFF376759, 0xFF306555, 0xFF266256, 0xFF235F53, 0xFF205C50, 0xFF1F5B4F, 0xFF205C50,
        0xFF205C50, 0xFF1F5B4F, 0xFF1E5A4E, 0xFF246152, 0xFF276153, 0xFF2E6255, 0xFF356559, 0xFF3F695D, 0xFF456D62, 0xFF4A7065, 0xFF4D7165, 0xFF4E7264, 0xFF2D5646, 0xFF1C4837, 0xFF306350, 0xFF376A57, 0xFF1F503D, 0xFF335F4E, 0xFF2F5848, 0xFF174C38,
        0xFF30614E, 0xFF547F6E, 0xFF587C6E, 0xFF537769, 0xFF587E6F, 0xFF2D5948, 0xFF1E4D3B, 0xFF346353, 0xFF396858, 0xFF3B6A5A, 0xFF396858, 0xFF316050, 0xFF366555, 0xFF3D6C5C, 0xFF2B5A4A, 0xFF144B38, 0xFF366958, 0xFF588475, 0xFF588173, 0xFF577D70,
        0xFF598274, 0xFF517D6E, 0xFF528272, 0xFF557E70, 0xFF547D6F, 0xFF598375, 0xFF568072, 0xFF5A8677, 0xFF548071, 0xFF568575, 0xFF568575, 0xFF537F70, 0xFF548071, 0xFF537F70, 0xFF507C6D, 0xFF517D6E, 0xFF548071, 0xFF548071, 0xFF517D6E, 0xFF527C6E,
        0xFF537D6F, 0xFF547E70, 0xFF557F71, 0xFF568072, 0xFF568072, 0xFF557F71, 0xFF547E70, 0xFF598274, 0xFF598375, 0xFF578173, 0xFF547E70, 0xFF547E70, 0xFF568072, 0xFF568072, 0xFF547E70, 0xFF547D6D, 0xFF557E6E, 0xFF557E6E, 0xFF547D6D, 0xFF527B6B,
        0xFF517A6A, 0xFF517A6A, 0xFF507969, 0xFF537C6E, 0xFF537C6E, 0xFF527B6D, 0xFF50796B, 0xFF4E7769, 0xFF4D7668, 0xFF4E7769, 0xFF4F786A, 0xFF497365, 0xFF497365, 0xFF487264, 0xFF426B5D, 0xFF3F685A, 0xFF315A4C, 0xFF23493C, 0xFF2C5245, 0xFF627D76,
        0xFF7B928C, 0xFF839692, 0xFF929E9A, 0xFF919C98, 0xFF84918A, 0xFF7F8E87, 0xFF6B7C74, 0xFF83978E, 0xFF8EA299, 0xFF7F8E87, 0xFF69746E, 0xFF777D79, 0xFF767B77, 0xFF80807E, 0xFFACACAA, 0xFFB9BAB5, 0xFFACADA8, 0xFFAAACA7, 0xFFA4A6A1, 0xFF9DA29C,
        0xFF9EA39D, 0xFF979C96, 0xFF959A94, 0xFF929C94, 0xFF8D948D, 0xFF8D948D, 0xFF8D948D, 0xFF898E88, 0xFF878C86, 0xFF888A85, 0xFF838580, 0xFF888983, 0xFF83847E, 0xFF878882, 0xFF858680, 0xFF8A8B85, 0xFF878882, 0xFF8F908A, 0xFF92938D, 0xFF8C8C80,
        0xFF8C8C80, 0xFF909086, 0xFF8D8D83, 0xFF8D8D85, 0xFF888880, 0xFF7E7D78, 0xFF868580, 0xFF858B89, 0xFF868C8A, 0xFF808483, 0xFF828685, 0xFF999B9A, 0xFFACAEAD, 0xFFB2B2B2, 0xFFB1B1B1, 0xFF96A098, 0xFF808B83, 0xFF6B786F, 0xFF55655B, 0xFF66776D,
        0xFF5C7065, 0xFF576B60, 0xFF647A6E, 0xFF6D7E74, 0xFF5F7066, 0xFF76877D, 0xFF8A9B91, 0xFF7F9086, 0xFF83948A, 0xFF8FA096, 0xFF84958B, 0xFF608176, 0xFF53796C, 0xFF4E786A, 0xFF487465, 0xFF52766A, 0x00EAFDF7, 0x00FEFFFF, 0x00FFF8FB, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x0FEEFFFF, 0x00FCFFFF, 0x08FDFFF2, 0xE17F9980, 0xFF244C34, 0xFF224531, 0xFF1E4028, 0xFF163F1D, 0xFF16391B, 0xFF16391B, 0xFF15381A, 0xFF16371A, 0xFF173519,
        0xFF173519, 0xFF183519, 0xFF183519, 0xFF133014, 0xFF18361A, 0xFF15371C, 0xFF11381D, 0xFF123E25, 0xFF13452C, 0xFF164C34, 0xFF1C563E, 0xFF25553D, 0xFF315843, 0xFF436451, 0xFF4F705B, 0xFF4D735C, 0xFF4B735B, 0xFF4F725A, 0xFF54715B, 0xFF4E7264,
        0xFF4D7364, 0xFF4E7465, 0xFF4E7465, 0xFF4C7565, 0xFF4B7464, 0xFF4A7564, 0xFF4A7564, 0xFF487161, 0xFF487161, 0xFF486E61, 0xFF446D5F, 0xFF3E6A5B, 0xFF336657, 0xFF286150, 0xFF215E4C, 0xFF1B5A4F, 0xFF1A594E, 0xFF18574C, 0xFF18574C, 0xFF19584D,
        0xFF1B5A4F, 0xFF1B5A4F, 0xFF1B5A4F, 0xFF1C5A4B, 0xFF1E5B4C, 0xFF235D4F, 0xFF295F51, 0xFF306256, 0xFF39675A, 0xFF416B5F, 0xFF456F63, 0xFF4C7263, 0xFF2C5746, 0xFF11402E, 0xFF336451, 0xFF3A6B58, 0xFF154432, 0xFF315C4B, 0xFF2C5545, 0xFF11402E,
        0xFF305C4B, 0xFF537C6C, 0xFF567C6D, 0xFF557B6C, 0xFF547D6D, 0xFF295443, 0xFF134230, 0xFF426C5E, 0xFF3A6456, 0xFF305A4C, 0xFF2D5749, 0xFF275143, 0xFF295345, 0xFF2F594B, 0xFF234D3F, 0xFF144736, 0xFF376656, 0xFF568273, 0xFF568072, 0xFF537D6F,
        0xFF548373, 0xFF4F8271, 0xFF4F8472, 0xFF518070, 0xFF507F6F, 0xFF558474, 0xFF568273, 0xFF578374, 0xFF507C6D, 0xFF537F70, 0xFF558172, 0xFF548071, 0xFF548071, 0xFF537F70, 0xFF517D6E, 0xFF527E6F, 0xFF548071, 0xFF537F70, 0xFF517D6E, 0xFF557F71,
        0xFF568072, 0xFF578173, 0xFF578173, 0xFF568072, 0xFF557F71, 0xFF547E70, 0xFF547E70, 0xFF567F71, 0xFF567F71, 0xFF567C6F, 0xFF557B6E, 0xFF577B6F, 0xFF5D7E73, 0xFF5D7E73, 0xFF5B7B70, 0xFF578070, 0xFF567F6F, 0xFF547D6D, 0xFF537C6C, 0xFF527B6B,
        0xFF517A6A, 0xFF517A6A, 0xFF517A6A, 0xFF527B6D, 0xFF517A6C, 0xFF4F786A, 0xFF4F786A, 0xFF4F786A, 0xFF4F786A, 0xFF4E7769, 0xFF4E7769, 0xFF497365, 0xFF477163, 0xFF477163, 0xFF466F61, 0xFF3F685A, 0xFF275042, 0xFF1A4033, 0xFF2F5548, 0xFF495855,
        0xFF596563, 0xFF5A6060, 0xFF676968, 0xFF757575, 0xFF757575, 0xFF696B68, 0xFF4A4F4B, 0xFF6B706C, 0xFF8F918E, 0xFFB5B5B3, 0xFFAAA6A5, 0xFFB0AAAA, 0xFFB3A9AA, 0xFFBBAFB1, 0xFFB9ADAF, 0xFF7C8280, 0xFF787E7C, 0xFF868C8A, 0xFF8F9895, 0xFF929B98,
        0xFF929D99, 0xFF8E9995, 0xFF8C9793, 0xFF7E8782, 0xFF79827D, 0xFF7A837E, 0xFF7F8581, 0xFF7D837F, 0xFF7C817D, 0xFF7D827E, 0xFF7D827E, 0xFF878880, 0xFF808179, 0xFF86877F, 0xFF86877F, 0xFF86877F, 0xFF85867E, 0xFF8F9088, 0xFF8F9088, 0xFF91948B,
        0xFF8E8F87, 0xFF8E8F87, 0xFF8D8E86, 0xFF8D8D85, 0xFF85857D, 0xFF7C7C74, 0xFF84817A, 0xFF9DA1A0, 0xFFABB1AF, 0xFFAFB5B3, 0xFFA6AFAC, 0xFFA3ACA9, 0xFF9CA7A3, 0xFF8A9591, 0xFF78837F, 0xFF908F8D, 0xFFB3B2B0, 0xFFBDBCBA, 0xFF8F8E8C, 0xFF9F9B9A,
        0xFF9A9695, 0xFFA29E9D, 0xFFABA7A6, 0xFF585A57, 0xFF4D4F4C, 0xFF696B68, 0xFF838582, 0xFF7A7C79, 0xFF858784, 0xFF999B98, 0xFF8B8D8A, 0xFF5F8075, 0xFF52786B, 0xFF4D7769, 0xFF4C7869, 0xFF55796D, 0x00E7FAF4, 0x00FDFFFE, 0x00FFF9FC, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x0FEEFEF4, 0x35F0EADA, 0x5FF1E4C4, 0xFF92986C, 0xFF4E5B30, 0xFF494D28, 0xFF4A4B21, 0xFF464E1B, 0xFF474D29, 0xFF484E2A, 0xFF474D29, 0xFF474D29, 0xFF464C28,
        0xFF444A26, 0xFF444825, 0xFF434724, 0xFF42431B, 0xFF43451D, 0xFF3C4118, 0xFF343F14, 0xFF314319, 0xFF29441B, 0xFF1B3E16, 0xFF163C15, 0xFF234E3B, 0xFF254F3B, 0xFF335A45, 0xFF3A644C, 0xFF356146, 0xFF356148, 0xFF3C6650, 0xFF3F6651, 0xFF466C5D,
        0xFF486E5F, 0xFF4A7061, 0xFF487161, 0xFF487161, 0xFF477261, 0xFF477261, 0xFF477261, 0xFF496F60, 0xFF466F61, 0xFF426C5E, 0xFF3C6B5B, 0xFF326757, 0xFF276250, 0xFF1C5C4B, 0xFF165947, 0xFF19584D, 0xFF18574C, 0xFF17564B, 0xFF16554A, 0xFF17564B,
        0xFF18574C, 0xFF19584D, 0xFF1A594E, 0xFF185649, 0xFF195748, 0xFF1B594A, 0xFF1F5C4D, 0xFF255F50, 0xFF2F6356, 0xFF39695B, 0xFF416D60, 0xFF4B7663, 0xFF285340, 0xFF0E3B27, 0xFF295642, 0xFF305D49, 0xFF17422F, 0xFF37604E, 0xFF396250, 0xFF174231,
        0xFF355E4E, 0xFF557E6E, 0xFF557E6E, 0xFF578070, 0xFF55806F, 0xFF2D5847, 0xFF154130, 0xFF335F4E, 0xFF2B5746, 0xFF2B5746, 0xFF396554, 0xFF3B6756, 0xFF386453, 0xFF3D6958, 0xFF396554, 0xFF184433, 0xFF396554, 0xFF54806F, 0xFF558170, 0xFF4F7E6C,
        0xFF528370, 0xFF528572, 0xFF508571, 0xFF548572, 0xFF528370, 0xFF538270, 0xFF52816F, 0xFF537F6E, 0xFF517C6B, 0xFF527D6C, 0xFF567F6F, 0xFF568273, 0xFF558172, 0xFF548071, 0xFF537F70, 0xFF558172, 0xFF578374, 0xFF578374, 0xFF568273, 0xFF547F6E,
        0xFF547F6E, 0xFF537E6D, 0xFF527D6C, 0xFF517C6B, 0xFF527D6C, 0xFF547F6E, 0xFF55806F, 0xFF507F6D, 0xFF527E6D, 0xFF537F6E, 0xFF567F6F, 0xFF5B8172, 0xFF5D8173, 0xFF5D7F71, 0xFF5A7C6E, 0xFF578070, 0xFF567F6F, 0xFF557E6E, 0xFF557E6E, 0xFF557E6E,
        0xFF557E6E, 0xFF547D6D, 0xFF537C6C, 0xFF527B6D, 0xFF517A6C, 0xFF517A6C, 0xFF517A6C, 0xFF527B6D, 0xFF517A6C, 0xFF4E7769, 0xFF4B7466, 0xFF4B7567, 0xFF456F61, 0xFF446E60, 0xFF487163, 0xFF466F61, 0xFF2A5345, 0xFF163C2F, 0xFF284E41, 0xFF364444,
        0xFF475353, 0xFF4C5252, 0xFF535456, 0xFF575757, 0xFF555756, 0xFF585C5B, 0xFF565B57, 0xFF444444, 0xFF595758, 0xFFCAC8C9, 0xFFF4EEF0, 0xFFF1E8EB, 0xFFFDF2F6, 0xFFFFF7FB, 0xFFAFA3A7, 0xFF374143, 0xFF2F3A3C, 0xFF333E40, 0xFF384345, 0xFF334142,
        0xFF2E3C3D, 0xFF2A3A3A, 0xFF293939, 0xFF4B5652, 0xFF47524E, 0xFF48534F, 0xFF4C5552, 0xFF49524F, 0xFF474D4B, 0xFF464C4A, 0xFF494F4D, 0xFF4A504E, 0xFF444A48, 0xFF494F4D, 0xFF4B514F, 0xFF464C4A, 0xFF434947, 0xFF484E4C, 0xFF464C4A, 0xFF425050,
        0xFF3F4B4B, 0xFF3E4A4A, 0xFF404C4C, 0xFF465051, 0xFF475152, 0xFF454F50, 0xFF495153, 0xFF4B4F50, 0xFF3D4343, 0xFF2D3635, 0xFF2B3534, 0xFF2D3C39, 0xFF2E3E3B, 0xFF273A36, 0xFF233833, 0xFF3B393A, 0xFF6C6A6B, 0xFFD0CCCD, 0xFFDBD5D7, 0xFFEAE1E4,
        0xFFEADFE3, 0xFFD8CCD0, 0xFF766A6E, 0xFF454545, 0xFF4A4A4A, 0xFF4F4F4F, 0xFF535353, 0xFF4D4D4D, 0xFF474747, 0xFF474747, 0xFF414141, 0xFF4D6E63, 0xFF4D7366, 0xFF4F796B, 0xFF4D796A, 0xFF4E7266, 0x00DFF2EC, 0x00FDFFFE, 0x00FFFCFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x7EE4C5B3, 0xFFD49C81, 0xFFE4A176, 0xFFD39C64, 0xFFCE9B64, 0xFFDB9E6F, 0xFFE2A06E, 0xFFDBA066, 0xFFD49C5F, 0xFFD49C5F, 0xFFD49E60, 0xFFD49E60, 0xFFD49E60,
        0xFFD29C5E, 0xFFD19B5D, 0xFFD09A5C, 0xFFD79A6D, 0xFFD6996A, 0xFFD49866, 0xFFCD9660, 0xFFC49561, 0xFFAF8958, 0xFF876B3C, 0xFF634E21, 0xFF204031, 0xFF193B2A, 0xFF1A4029, 0xFF1D432A, 0xFF194127, 0xFF1F4B34, 0xFF2C5D4A, 0xFF316757, 0xFF406657,
        0xFF43695A, 0xFF446D5D, 0xFF466F5F, 0xFF45705F, 0xFF45705F, 0xFF457160, 0xFF467261, 0xFF496D61, 0xFF446D5F, 0xFF3D6C5C, 0xFF346959, 0xFF296452, 0xFF1F5E4D, 0xFF175746, 0xFF125543, 0xFF19574A, 0xFF185649, 0xFF175548, 0xFF155346, 0xFF145245,
        0xFF145245, 0xFF155346, 0xFF165447, 0xFF195347, 0xFF195549, 0xFF19594B, 0xFF1B5B4B, 0xFF1F5D4E, 0xFF296253, 0xFF35675B, 0xFF3D6B60, 0xFF47725F, 0xFF234C3A, 0xFF153C2A, 0xFF193E2D, 0xFF1E4030, 0xFF143626, 0xFF234535, 0xFF264838, 0xFF183A2C,
        0xFF315547, 0xFF557B6C, 0xFF578070, 0xFF598473, 0xFF537E6D, 0xFF2D5847, 0xFF0E3928, 0xFF144331, 0xFF174634, 0xFF174634, 0xFF1C4B39, 0xFF194836, 0xFF174634, 0xFF1B4A38, 0xFF154432, 0xFF163F2F, 0xFF366150, 0xFF517D6C, 0xFF568271, 0xFF517D6C,
        0xFF537F6E, 0xFF568271, 0xFF527E6D, 0xFF588372, 0xFF568170, 0xFF547F6E, 0xFF55806F, 0xFF55806F, 0xFF588171, 0xFF567F6F, 0xFF557E6E, 0xFF558172, 0xFF537F70, 0xFF517D6E, 0xFF507C6D, 0xFF507C6D, 0xFF507C6D, 0xFF527E6F, 0xFF548071, 0xFF527D6C,
        0xFF547F6E, 0xFF568170, 0xFF568170, 0xFF568170, 0xFF55806F, 0xFF578271, 0xFF588372, 0xFF4D806D, 0xFF4B7E6B, 0xFF4D7E6B, 0xFF507F6D, 0xFF52816F, 0xFF54806F, 0xFF54806F, 0xFF547F6E, 0xFF537C6C, 0xFF547D6D, 0xFF557E6E, 0xFF578070, 0xFF578070,
        0xFF578070, 0xFF567F6F, 0xFF567F6F, 0xFF557E70, 0xFF537C6E, 0xFF527B6D, 0xFF527B6D, 0xFF527B6D, 0xFF517A6C, 0xFF4F786A, 0xFF4D7668, 0xFF4D7769, 0xFF497365, 0xFF456F61, 0xFF477062, 0xFF487163, 0xFF2E5749, 0xFF163C2F, 0xFF244A3D, 0xFF2E4743,
        0xFF465A58, 0xFF505E5E, 0xFF525C5B, 0xFF535C5B, 0xFF505B57, 0xFF535F5B, 0xFF54635E, 0xFF616765, 0xFF606664, 0xFFD2D8D6, 0xFFFCFFFF, 0xFFEEF0EF, 0xFFFAFCFB, 0xFFFFFFFF, 0xFFA6A6A6, 0xFF3D4B4C, 0xFF364445, 0xFF354344, 0xFF374747, 0xFF334343,
        0xFF2E4040, 0xFF314343, 0xFF2F4141, 0xFF525E5A, 0xFF535F5B, 0xFF55615D, 0xFF58635F, 0xFF58635F, 0xFF545D5A, 0xFF545D5A, 0xFF58615E, 0xFF556364, 0xFF536162, 0xFF515F60, 0xFF546263, 0xFF536162, 0xFF515F60, 0xFF505E5F, 0xFF526061, 0xFF525751,
        0xFF545B54, 0xFF575D59, 0xFF5A605E, 0xFF57605F, 0xFF586160, 0xFF5C6667, 0xFF5D6768, 0xFF5D6363, 0xFF3F4847, 0xFF293332, 0xFF283432, 0xFF2D3D3A, 0xFF2C3F3B, 0xFF293E39, 0xFF273E38, 0xFF444D4A, 0xFF767C7A, 0xFFF4FAF8, 0xFFFCFFFF, 0xFFF9FBFA,
        0xFFFFFFFF, 0xFFFFFEFF, 0xFF898788, 0xFF4C5552, 0xFF5C6562, 0xFF535C59, 0xFF4D5653, 0xFF4F5855, 0xFF49524F, 0xFF48514E, 0xFF49524F, 0xFF416257, 0xFF4E7467, 0xFF537D6F, 0xFF4D796A, 0xFF496D61, 0x00DAEDE7, 0x00FEFFFF, 0x00FFFCFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x89FBD0AE, 0xFFDC976D, 0xFFE99559, 0xFFEBA15A, 0xFFEEA862, 0xFFF3A165, 0xFFF6A063, 0xFFEA9D59, 0xFFE79758, 0xFFE89859, 0xFFE89A5A, 0xFFEA9C5C, 0xFFE99E5D,
        0xFFE99E5D, 0xFFE89E5D, 0xFFE89E5D, 0xFFE2975E, 0xFFE19559, 0xFFE79959, 0xFFEB9D5B, 0xFFE8A15F, 0xFFDEA063, 0xFFB38047, 0xFF7D4F1B, 0xFF27301B, 0xFF202C14, 0xFF1E2E11, 0xFF1F3010, 0xFF1E3115, 0xFF1D3A24, 0xFF1F4E3C, 0xFF1F5E4D, 0xFF3C6555,
        0xFF406959, 0xFF446D5D, 0xFF45705F, 0xFF446F5E, 0xFF436F5E, 0xFF44705F, 0xFF457160, 0xFF476D60, 0xFF426C60, 0xFF3A6A5C, 0xFF306656, 0xFF255F50, 0xFF1C5949, 0xFF175445, 0xFF175143, 0xFF184C3F, 0xFF184C3F, 0xFF184C3F, 0xFF174B3E, 0xFF174B3E,
        0xFF184C3F, 0xFF1A4E41, 0xFF1B4F42, 0xFF1C4F48, 0xFF1A5448, 0xFF19574A, 0xFF185A4C, 0xFF1C5C4C, 0xFF266052, 0xFF326458, 0xFF3C675D, 0xFF456E5C, 0xFF264B3A, 0xFF254536, 0xFF183426, 0xFF1B3326, 0xFF1D372A, 0xFF1D392B, 0xFF1C382A, 0xFF28463A,
        0xFF345447, 0xFF527869, 0xFF547F6E, 0xFF537F6E, 0xFF4F7B6A, 0xFF386352, 0xFF194232, 0xFF1B4432, 0xFF18412F, 0xFF153E2C, 0xFF1D4634, 0xFF1A4331, 0xFF113A28, 0xFF18412F, 0xFF1E4735, 0xFF234C3A, 0xFF426B59, 0xFF588370, 0xFF588571, 0xFF4F7A67,
        0xFF517665, 0xFF59796A, 0xFF547062, 0xFF567667, 0xFF5B7B6C, 0xFF587A6A, 0xFF567B6A, 0xFF547B69, 0xFF5A8371, 0xFF567F6D, 0xFF507B68, 0xFF507C6D, 0xFF4E7A6B, 0xFF4C7869, 0xFF487465, 0xFF416D5E, 0xFF3C6859, 0xFF3D695A, 0xFF406C5D, 0xFF426D5A,
        0xFF47725F, 0xFF4D7865, 0xFF4E7966, 0xFF4A7562, 0xFF436E5B, 0xFF3D6855, 0xFF3A6552, 0xFF396451, 0xFF37624F, 0xFF34614D, 0xFF36634F, 0xFF386753, 0xFF3C6B57, 0xFF44735F, 0xFF497A65, 0xFF4A7363, 0xFF4D7666, 0xFF517A6A, 0xFF527B6B, 0xFF527B6B,
        0xFF517A6A, 0xFF517A6A, 0xFF527B6B, 0xFF578072, 0xFF557E70, 0xFF527B6D, 0xFF50796B, 0xFF50796B, 0xFF517A6C, 0xFF527B6D, 0xFF527B6D, 0xFF4E786A, 0xFF4F796B, 0xFF497365, 0xFF456E60, 0xFF456E60, 0xFF2C5547, 0xFF183E31, 0xFF2A5043, 0xFF324F4B,
        0xFF455F5C, 0xFF445856, 0xFF3C4C4B, 0xFF455451, 0xFF4F5E5B, 0xFF4C5C59, 0xFF445751, 0xFF505F5C, 0xFF4F5E5B, 0xFF889492, 0xFF929E9C, 0xFF8A9694, 0xFF909A99, 0xFF9CA6A5, 0xFF798382, 0xFF566465, 0xFF546263, 0xFF516161, 0xFF536363, 0xFF4F5F5F,
        0xFF485A5A, 0xFF475959, 0xFF3D4F4F, 0xFF344340, 0xFF3D4C49, 0xFF45514F, 0xFF495553, 0xFF4D5957, 0xFF4F5958, 0xFF505A59, 0xFF555F5E, 0xFF4F595A, 0xFF545E5F, 0xFF495354, 0xFF4C5657, 0xFF566061, 0xFF566061, 0xFF4E5859, 0xFF596364, 0xFF87765C,
        0xFF8E7C64, 0xFF90816C, 0xFF897E6C, 0xFF71695C, 0xFF5C584D, 0xFF595951, 0xFF595853, 0xFF495354, 0xFF4B5556, 0xFF515D5D, 0xFF556161, 0xFF515F5F, 0xFF515F5F, 0xFF516160, 0xFF4E5E5D, 0xFF566562, 0xFF6A7976, 0xFFA2B1AE, 0xFFA1B0AD, 0xFFA3B2AF,
        0xFFA9B8B5, 0xFFA5B4B1, 0xFF4C5856, 0xFF445451, 0xFF566663, 0xFF536360, 0xFF4F5F5C, 0xFF4C5C59, 0xFF485855, 0xFF4C5C59, 0xFF41514E, 0xFF45665B, 0xFF53796C, 0xFF557F71, 0xFF4E7A6B, 0xFF4B6F63, 0x00DCEFE9, 0x00FEFFFF, 0x00FFFBFE, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x9CEAD2A4, 0xFFD5A06A, 0xFFE29C56, 0xFFE3A552, 0xFFE0A555, 0xFFE5A05B, 0xFFF0A969, 0xFFE4A560, 0xFFE19861, 0xFFE19A62, 0xFFE29B63, 0xFFE39E65, 0xFFE2A066,
        0xFFE2A167, 0xFFE1A368, 0xFFE1A368, 0xFFDCA765, 0xFFD89D59, 0xFFDF9F57, 0xFFE09E52, 0xFFDA9C51, 0xFFDAA360, 0xFFB4884B, 0xFF714C15, 0xFF634B27, 0xFF614E26, 0xFF5F4E22, 0xFF604F24, 0xFF5C4E2B, 0xFF4C4F32, 0xFF35573F, 0xFF2A644C, 0xFF3D6656,
        0xFF416A5A, 0xFF45705F, 0xFF467160, 0xFF436F5E, 0xFF426E5D, 0xFF426E5D, 0xFF44705F, 0xFF436B60, 0xFF3F6B5E, 0xFF356859, 0xFF2A6354, 0xFF225B4C, 0xFF1E5447, 0xFF1D4F44, 0xFF1E4D43, 0xFF1C4638, 0xFF1D4739, 0xFF1D4739, 0xFF1C4638, 0xFF1B4537,
        0xFF1B4537, 0xFF1D4739, 0xFF204A3C, 0xFF1D4C46, 0xFF1C4F48, 0xFF195549, 0xFF16584A, 0xFF185B4A, 0xFF225C4E, 0xFF305F55, 0xFF396359, 0xFF416654, 0xFF2D4E3D, 0xFF3C5446, 0xFF36483C, 0xFF3E4E43, 0xFF4C5C51, 0xFF495D51, 0xFF43594C, 0xFF52695F,
        0xFF446256, 0xFF54786A, 0xFF4E7A69, 0xFF4A7967, 0xFF4F7E6C, 0xFF517C6B, 0xFF3A6353, 0xFF41554A, 0xFF405449, 0xFF465A4F, 0xFF5C7065, 0xFF5C7065, 0xFF475B50, 0xFF43574C, 0xFF485C51, 0xFF406957, 0xFF537C6A, 0xFF55806D, 0xFF45705D, 0xFF2F5443,
        0xFF2C483A, 0xFF384C41, 0xFF344138, 0xFF2F4036, 0xFF3C5045, 0xFF3D5548, 0xFF3D594B, 0xFF3B5D4D, 0xFF4E7563, 0xFF527D6A, 0xFF527F6B, 0xFF4D796A, 0xFF4B7768, 0xFF487465, 0xFF3F6B5C, 0xFF315D4E, 0xFF234F40, 0xFF1F4B3C, 0xFF214D3E, 0xFF345F4C,
        0xFF3B6653, 0xFF436E5B, 0xFF436E5B, 0xFF3C6754, 0xFF315C49, 0xFF27523F, 0xFF214C39, 0xFF22382C, 0xFF1F372A, 0xFF1C3629, 0xFF1C3B2C, 0xFF1D3F2F, 0xFF204735, 0xFF285340, 0xFF335E4B, 0xFF4C7565, 0xFF507969, 0xFF547D6D, 0xFF547D6D, 0xFF527B6B,
        0xFF517A6A, 0xFF537C6C, 0xFF567F6F, 0xFF50796B, 0xFF517A6C, 0xFF527B6D, 0xFF527B6D, 0xFF527B6D, 0xFF527B6D, 0xFF527B6D, 0xFF537C6E, 0xFF4E786A, 0xFF4F796B, 0xFF4A7466, 0xFF477062, 0xFF466F61, 0xFF2A5345, 0xFF163C2F, 0xFF2C5245, 0xFF2E4744,
        0xFF455B59, 0xFF485858, 0xFF3A4646, 0xFF3A4445, 0xFF414B4A, 0xFF465250, 0xFF4D5C59, 0xFF4E5A58, 0xFF475351, 0xFF515D5B, 0xFF55615F, 0xFF5C6866, 0xFF606C6A, 0xFF586462, 0xFF4E5A58, 0xFF525E5E, 0xFF546060, 0xFF4E5C5C, 0xFF4F5D5D, 0xFF485857,
        0xFF435352, 0xFF40504F, 0xFF2D403E, 0xFF324241, 0xFF414F4F, 0xFF475555, 0xFF465454, 0xFF4B5757, 0xFF4C5858, 0xFF4C5657, 0xFF4D5758, 0xFF626054, 0xFF727064, 0xFF605E52, 0xFF636155, 0xFF78766A, 0xFF747266, 0xFF5E5C50, 0xFF706E62, 0xFFB3936D,
        0xFFAF916F, 0xFFAB9274, 0xFFA7957D, 0xFF887C6C, 0xFF605C51, 0xFF585752, 0xFF565855, 0xFF475555, 0xFF455353, 0xFF465252, 0xFF445050, 0xFF454F50, 0xFF4E5859, 0xFF545C5E, 0xFF50585A, 0xFF424E4E, 0xFF535F5F, 0xFF5E6C6C, 0xFF526261, 0xFF4F6260,
        0xFF495D5B, 0xFF4A605D, 0xFF2B413E, 0xFF364645, 0xFF4E5E5D, 0xFF516160, 0xFF536362, 0xFF4F5F5E, 0xFF4C5C5B, 0xFF516160, 0xFF3E4E4D, 0xFF4B6C61, 0xFF547A6D, 0xFF537D6F, 0xFF507C6D, 0xFF4E7266, 0x00DBEEE8, 0x00FEFFFF, 0x00FFFBFE, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xA0EACAA1, 0xFFDEA26E, 0xFFEB9E5A, 0xFFE8A153, 0xFFE4A057, 0xFFEA9E62, 0xFFF5A66E, 0xFFDC975C, 0xFFE89956, 0xFFE79A56, 0xFFE69B57, 0xFFE59C57, 0xFFE49E58,
        0xFFE39F58, 0xFFE3A059, 0xFFE3A059, 0xFFD8A065, 0xFFD59859, 0xFFE29D58, 0xFFE49A53, 0xFFDD9450, 0xFFE19F62, 0xFFBE8751, 0xFF744414, 0xFF714110, 0xFF734813, 0xFF724911, 0xFF774915, 0xFF774E22, 0xFF5F4F2D, 0xFF415637, 0xFF326546, 0xFF3F6858,
        0xFF436C5C, 0xFF477261, 0xFF477261, 0xFF436F5E, 0xFF416D5C, 0xFF416D5C, 0xFF406F5D, 0xFF41695E, 0xFF3B695C, 0xFF326757, 0xFF276051, 0xFF1F5849, 0xFF1D5145, 0xFF1F4D43, 0xFF234B43, 0xFF2A4F3E, 0xFF294E3D, 0xFF274C3B, 0xFF224736, 0xFF1D4231,
        0xFF1A3F2E, 0xFF1A3F2E, 0xFF1B402F, 0xFF1E4844, 0xFF1C4D47, 0xFF1A534A, 0xFF155749, 0xFF18584A, 0xFF1F594B, 0xFF2D5C52, 0xFF375F57, 0xFF3B5D4C, 0xFF355142, 0xFF4B5F53, 0xFF515E54, 0xFF5F6960, 0xFF677369, 0xFF647167, 0xFF536559, 0xFF52675E,
        0xFF405C50, 0xFF527668, 0xFF527E6D, 0xFF4C7B69, 0xFF4D7C6A, 0xFF527D6C, 0xFF376050, 0xFF565853, 0xFF636560, 0xFF6A6C67, 0xFF737570, 0xFF757772, 0xFF6C6E69, 0xFF62645F, 0xFF545651, 0xFF48715F, 0xFF57826F, 0xFF55806D, 0xFF48715F, 0xFF3C5C4D,
        0xFF495F53, 0xFF6A746C, 0xFF6F716C, 0xFF606A62, 0xFF6E7971, 0xFF63746A, 0xFF51695C, 0xFF3F5E4F, 0xFF4D7462, 0xFF517C69, 0xFF51806C, 0xFF4D796A, 0xFF4C7869, 0xFF487465, 0xFF3C6859, 0xFF275344, 0xFF133F30, 0xFF0A3627, 0xFF0B3728, 0xFF0F3A27,
        0xFF16412E, 0xFF1E4936, 0xFF204B38, 0xFF1A4532, 0xFF113C29, 0xFF093421, 0xFF05301D, 0xFF1E201B, 0xFF192019, 0xFF162119, 0xFF14251B, 0xFF10281B, 0xFF0E2A1C, 0xFF123424, 0xFF1A3F2E, 0xFF275040, 0xFF2B5444, 0xFF2E5747, 0xFF2D5646, 0xFF295242,
        0xFF285141, 0xFF2C5545, 0xFF305949, 0xFF466F61, 0xFF4C7567, 0xFF527B6D, 0xFF567F71, 0xFF567F71, 0xFF537C6E, 0xFF517A6C, 0xFF4F786A, 0xFF4D7769, 0xFF4B7567, 0xFF477163, 0xFF4A7365, 0xFF4C7567, 0xFF2C5547, 0xFF12382B, 0xFF294F42, 0xFF354747,
        0xFF465455, 0xFF4F595B, 0xFF515659, 0xFF575B5E, 0xFF525657, 0xFF4B5151, 0xFF545D5C, 0xFF4C5554, 0xFF404948, 0xFF47504F, 0xFF4B5453, 0xFF3F4847, 0xFF47504F, 0xFF495352, 0xFF4A5453, 0xFF535F5D, 0xFF566260, 0xFF4F5B59, 0xFF4B5A57, 0xFF475653,
        0xFF475653, 0xFF455552, 0xFF31413E, 0xFF384847, 0xFF495958, 0xFF4F5D5D, 0xFF495757, 0xFF495757, 0xFF4B5757, 0xFF475353, 0xFF465252, 0xFF71624D, 0xFF8A7B66, 0xFF756651, 0xFF786954, 0xFF93846F, 0xFF887964, 0xFF63543F, 0xFF776853, 0xFF957755,
        0xFF83684A, 0xFF7C684F, 0xFF887D6B, 0xFF757269, 0xFF4F5553, 0xFF495458, 0xFF48575E, 0xFF435352, 0xFF40504F, 0xFF465454, 0xFF4D5758, 0xFF4D5557, 0xFF515659, 0xFF54585B, 0xFF525357, 0xFF4F5457, 0xFF464E50, 0xFF3E4849, 0xFF3C4A4A, 0xFF3A4D4B,
        0xFF334946, 0xFF314A46, 0xFF1C3732, 0xFF344040, 0xFF525E5E, 0xFF4E5A5A, 0xFF4C5858, 0xFF4E5A5A, 0xFF4B5757, 0xFF4E5A5A, 0xFF424E4E, 0xFF4A6B60, 0xFF51776A, 0xFF507A6C, 0xFF517D6E, 0xFF4F7367, 0x00D7EAE4, 0x00FDFFFE, 0x00FFFCFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x91E9CBA9, 0xFFD69E65, 0xFFEEA159, 0xFFEBA25D, 0xFFE8A35E, 0xFFF1A55A, 0xFFF1A55A, 0xFFE09D57, 0xFFE39C5C, 0xFFE49D5D, 0xFFE7A060, 0xFFE7A060, 0xFFE69F5F,
        0xFFE59E5E, 0xFFE59E5E, 0xFFE59E5E, 0xFFE4A061, 0xFFDE9A59, 0xFFE09B58, 0xFFE29D5A, 0xFFDE9A59, 0xFFE0A364, 0xFFC28A4D, 0xFF794209, 0xFF64401C, 0xFF674522, 0xFF603E19, 0xFF643E11, 0xFF6D4917, 0xFF5A4B20, 0xFF405439, 0xFF39695D, 0xFF4C6D5A,
        0xFF416B57, 0xFF3F705D, 0xFF447265, 0xFF486C62, 0xFF4A6B62, 0xFF446F5E, 0xFF386C56, 0xFF386E57, 0xFF376A59, 0xFF2E5C51, 0xFF2F574F, 0xFF355552, 0xFF3B5552, 0xFF334946, 0xFF495E59, 0xFF54656C, 0xFF586970, 0xFF506266, 0xFF495B5D, 0xFF283A3A,
        0xFF314442, 0xFF3F524E, 0xFF2D403C, 0xFF1F443C, 0xFF1D483E, 0xFF175041, 0xFF165646, 0xFF195949, 0xFF1F5B4F, 0xFF2A5D54, 0xFF315E57, 0xFF356450, 0xFF405A4D, 0xFF8F918C, 0xFFA69E9B, 0xFFA19C98, 0xFFA4A59D, 0xFFA5A59D, 0xFFA5A29B, 0xFF89968F,
        0xFF456155, 0xFF4F7B6A, 0xFF467764, 0xFF4F7A69, 0xFF55796B, 0xFF4F7566, 0xFF426D5C, 0xFF79897F, 0xFFB1AEA9, 0xFFADA49D, 0xFFB0B0A4, 0xFFB1B3A6, 0xFFABA49C, 0xFFA5A6A0, 0xFF687C71, 0xFF4C6E5E, 0xFF597B6B, 0xFF5D7D6E, 0xFF476355, 0xFF566A5F,
        0xFF68726A, 0xFF80817C, 0xFF807B78, 0xFF756B6C, 0xFF908F8D, 0xFF606B65, 0xFF283F35, 0xFF3D5D50, 0xFF55796B, 0xFF4F7365, 0xFF55796B, 0xFF4E7A6D, 0xFF487669, 0xFF417161, 0xFF3F705D, 0xFF25503D, 0xFF0B2D1C, 0xFF132B1D, 0xFF182A1C, 0xFF192823,
        0xFF17221E, 0xFF191D1C, 0xFF1C1C1C, 0xFF1D1D1D, 0xFF1A1E1D, 0xFF15201C, 0xFF162520, 0xFF0C2417, 0xFF0D2317, 0xFF15261C, 0xFF16261C, 0xFF15251B, 0xFF14251B, 0xFF102419, 0xFF13291D, 0xFF1A2B23, 0xFF1D2A20, 0xFF343A2C, 0xFF524F3C, 0xFF847965,
        0xFF90816E, 0xFF877865, 0xFF837461, 0xFF657261, 0xFF2C4530, 0xFF1F452C, 0xFF294F38, 0xFF294934, 0xFF2E4936, 0xFF365744, 0xFF46725B, 0xFF4D7867, 0xFF4C7867, 0xFF477664, 0xFF477664, 0xFF4A7564, 0xFF2D5344, 0xFF163629, 0xFF2E4A3E, 0xFF2A3F3A,
        0xFF425551, 0xFF4E615D, 0xFF4D5D5A, 0xFF4F5E5B, 0xFF525E5C, 0xFF525C5B, 0xFF535D5C, 0xFF425454, 0xFF364848, 0xFF364848, 0xFF384A4A, 0xFF304242, 0xFF2F4141, 0xFF374949, 0xFF3B4D4D, 0xFF405955, 0xFF455957, 0xFF4E5A5A, 0xFF4D5557, 0xFF4A4F52,
        0xFF4A5254, 0xFF414F4F, 0xFF2F4240, 0xFF394747, 0xFF4E5A5A, 0xFF4B5757, 0xFF4D5959, 0xFF475152, 0xFF4A5455, 0xFF485052, 0xFF474F51, 0xFF4F5D46, 0xFF90825F, 0xFF635A49, 0xFF5C5250, 0xFFAB8C7A, 0xFF8D7C72, 0xFF4C5456, 0xFF6D6757, 0xFF8B8064,
        0xFF7E7355, 0xFF7F7454, 0xFF756C4D, 0xFF827D69, 0xFF5B5A55, 0xFF4F535E, 0xFF4D5467, 0xFF445252, 0xFF455151, 0xFF495354, 0xFF495354, 0xFF465051, 0xFF4B5757, 0xFF4C5C5B, 0xFF445755, 0xFF384C4B, 0xFF354747, 0xFF414F50, 0xFF475254, 0xFF3F4A4C,
        0xFF3C4A4B, 0xFF344646, 0xFF233736, 0xFF2F4342, 0xFF4B5B5B, 0xFF495555, 0xFF505558, 0xFF4F5356, 0xFF545859, 0xFF4F5555, 0xFF454E4D, 0xFF476D60, 0xFF547D6F, 0xFF578173, 0xFF577D70, 0xFF527267, 0x00DAEDE7, 0x00FCFFFF, 0x00FFFEFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x91E9CBA9, 0xFFD99E66, 0xFFEEA159, 0xFFEBA25D, 0xFFEAA45F, 0xFFF1A55A, 0xFFF1A559, 0xFFE19C57, 0xFFE39C5C, 0xFFE59E5E, 0xFFE7A060, 0xFFE7A060, 0xFFE69F5F,
        0xFFE59E5E, 0xFFE59E5E, 0xFFE59E5E, 0xFFE7A364, 0xFFE49F5E, 0xFFE29D5A, 0xFFE39E5B, 0xFFDE9A59, 0xFFDB9E5F, 0xFFC08549, 0xFF7F480F, 0xFF69390B, 0xFF64380B, 0xFF62370C, 0xFF6D4215, 0xFF6D4D1C, 0xFF4E451E, 0xFF334C36, 0xFF35685F, 0xFF456452,
        0xFF426956, 0xFF447361, 0xFF4A7669, 0xFF4B6C63, 0xFF46655D, 0xFF446D5D, 0xFF447560, 0xFF3B6C57, 0xFF396858, 0xFF355F55, 0xFF30534D, 0xFF4D6A68, 0xFF3A504E, 0xFF142725, 0xFF495956, 0xFF8899A0, 0xFF7B8C93, 0xFF85979B, 0xFF5A6C6E, 0xFF142626,
        0xFF394C4A, 0xFF586B67, 0xFF3A4D49, 0xFF23423C, 0xFF22473F, 0xFF1C4E42, 0xFF165344, 0xFF175546, 0xFF1E5B4C, 0xFF2A5F55, 0xFF306259, 0xFF32614D, 0xFF375144, 0xFF999B96, 0xFFCDC5C2, 0xFFCAC7C2, 0xFFB8B9B3, 0xFFB0B1A9, 0xFFBBB8B1, 0xFF9CA9A2,
        0xFF446054, 0xFF4D7968, 0xFF4F806D, 0xFF517C6B, 0xFF55796B, 0xFF507667, 0xFF2F5A49, 0xFF7A8A80, 0xFFD7D4CF, 0xFFE6DDD6, 0xFFE0E0D4, 0xFFDFE1D4, 0xFFE1DAD2, 0xFFC7C8C2, 0xFF62766B, 0xFF487360, 0xFF537E6B, 0xFF56816E, 0xFF486F5D, 0xFF507061,
        0xFF698174, 0xFF76867C, 0xFF657068, 0xFF59635B, 0xFF65756B, 0xFF425C4F, 0xFF1A3C2C, 0xFF2A5341, 0xFF4D7865, 0xFF537E6B, 0xFF4D7664, 0xFF4B7464, 0xFF4B7665, 0xFF45705F, 0xFF3C6756, 0xFF305647, 0xFF2A463A, 0xFF2E4239, 0xFF323F38, 0xFF2D3C37,
        0xFF17231F, 0xFF131917, 0xFF1B1D1C, 0xFF191B1A, 0xFF161C1A, 0xFF1F2B27, 0xFF2A3934, 0xFF30473F, 0xFF293E37, 0xFF2B3C36, 0xFF2A3934, 0xFF2B3A35, 0xFF2A3B35, 0xFF21362F, 0xFF213830, 0xFF34453F, 0xFF3B463E, 0xFF76796E, 0xFFA09D8E, 0xFFB5AC9B,
        0xFFB6A998, 0xFFB7A798, 0xFFB7A798, 0xFFA2A094, 0xFF747C6D, 0xFF5D725F, 0xFF586D5C, 0xFF525E50, 0xFF545F51, 0xFF536557, 0xFF4C6756, 0xFF507B6A, 0xFF4F7B6A, 0xFF4C7867, 0xFF4D7968, 0xFF4E7968, 0xFF315547, 0xFF18382B, 0xFF304C40, 0xFF2E413F,
        0xFF455856, 0xFF4F5F5E, 0xFF485656, 0xFF4B5757, 0xFF545E5F, 0xFF575F61, 0xFF555D5F, 0xFF404E4F, 0xFF3E4C4D, 0xFF414F50, 0xFF455354, 0xFF435152, 0xFF435152, 0xFF445253, 0xFF435152, 0xFF455957, 0xFF465655, 0xFF4A5455, 0xFF4A4F52, 0xFF4A4F52,
        0xFF4C5657, 0xFF455353, 0xFF344745, 0xFF303E3E, 0xFF445252, 0xFF435151, 0xFF4C5858, 0xFF4A5656, 0xFF4F595A, 0xFF4C5657, 0xFF4A5455, 0xFF5A6554, 0xFF91866A, 0xFF696657, 0xFF555450, 0xFF867062, 0xFF6E645B, 0xFF4C5658, 0xFF767266, 0xFFB18F6A,
        0xFFA3815B, 0xFFA4875D, 0xFFA38A62, 0xFF9D8D6C, 0xFF665E49, 0xFF4E4E46, 0xFF525355, 0xFF485454, 0xFF445050, 0xFF445050, 0xFF445050, 0xFF445050, 0xFF465454, 0xFF475756, 0xFF3F5250, 0xFF455958, 0xFF455757, 0xFF4D5B5C, 0xFF4E595B, 0xFF475254,
        0xFF465455, 0xFF3D4F4F, 0xFF273B3A, 0xFF334746, 0xFF4E5E5E, 0xFF4D5959, 0xFF4F5759, 0xFF4B5053, 0xFF515757, 0xFF4C5554, 0xFF454F4E, 0xFF466C5F, 0xFF527B6D, 0xFF557F71, 0xFF567C6F, 0xFF517166, 0x00DAEDE7, 0x00FCFFFF, 0x00FFFEFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x91EACCAA, 0xFFD99E66, 0xFFEFA159, 0xFFECA25B, 0xFFEAA45F, 0xFFF3A55B, 0xFFF2A459, 0xFFE19C57, 0xFFE39C5A, 0xFFE59E5C, 0xFFE7A05E, 0xFFE7A05E, 0xFFE69F5D,
        0xFFE59E5C, 0xFFE49D5B, 0xFFE59E5C, 0xFFE29E5F, 0xFFE19C5B, 0xFFE19A58, 0xFFE49E59, 0xFFDF9C58, 0xFFDB9B5B, 0xFFD1965A, 0xFFB47E42, 0xFFB9773A, 0xFFB5773C, 0xFFB37B42, 0xFFB5824D, 0xFF9F7B49, 0xFF6C6038, 0xFF41553C, 0xFF386256, 0xFF4D6857,
        0xFF4C6E5D, 0xFF45705F, 0xFF446A5F, 0xFF4C6A62, 0xFF536E67, 0xFF4B6F61, 0xFF406D59, 0xFF3D6656, 0xFF3E6457, 0xFF3E5F58, 0xFF355250, 0xFF435859, 0xFF7F9193, 0xFF1B2929, 0xFF4D5957, 0xFFA9B8BF, 0xFFA9B8BF, 0xFFB1C0C5, 0xFF48575A, 0xFF2B3B3B,
        0xFF819190, 0xFF546461, 0xFF162623, 0xFF273C37, 0xFF28463E, 0xFF244E44, 0xFF1B4F42, 0xFF155243, 0xFF1B5849, 0xFF255F51, 0xFF2C6459, 0xFF2C5B49, 0xFF2E4A3E, 0xFF949993, 0xFFC8C3C0, 0xFFBAB7B2, 0xFFA0A19B, 0xFF9FA09A, 0xFFBBB8B1, 0xFF95A29B,
        0xFF405C50, 0xFF477362, 0xFF4E7F6C, 0xFF4F7A69, 0xFF517567, 0xFF527869, 0xFF345F4E, 0xFF76857E, 0xFFD1D0CC, 0xFFD7CEC9, 0xFFC3C5BA, 0xFFC2C4B9, 0xFFCFCAC4, 0xFFC0C2BD, 0xFF576C63, 0xFF3A6B56, 0xFF457661, 0xFF4D816B, 0xFF457661, 0xFF305D49,
        0xFF3C6351, 0xFF3F5F50, 0xFF2C483A, 0xFF2A4637, 0xFF325342, 0xFF2C513F, 0xFF15402C, 0xFF113E29, 0xFF24513C, 0xFF2A5541, 0xFF1D4733, 0xFF254635, 0xFF254736, 0xFF1C3E2E, 0xFF18382B, 0xFF2D483F, 0xFF425752, 0xFF435151, 0xFF454F50, 0xFF394A44,
        0xFF192823, 0xFF0D1814, 0xFF141D1A, 0xFF131C19, 0xFF1B2622, 0xFF32413C, 0xFF41524C, 0xFF3C504E, 0xFF3B4E4C, 0xFF465454, 0xFF475353, 0xFF3F4D4D, 0xFF384847, 0xFF2D413F, 0xFF304945, 0xFF354441, 0xFF4B5650, 0xFF999C93, 0xFFBAB6AB, 0xFFBCB2A6,
        0xFFB6A89D, 0xFFB3A59A, 0xFFB7A9A0, 0xFFC7B4AE, 0xFFB8AFA6, 0xFFAFB1A4, 0xFFB1B1A7, 0xFFAFA8A0, 0xFFAFA69F, 0xFF9C9D95, 0xFF6F7B6F, 0xFF507B6A, 0xFF4F7A69, 0xFF497564, 0xFF497564, 0xFF4B7665, 0xFF315547, 0xFF1C3A2E, 0xFF365045, 0xFF334342,
        0xFF485857, 0xFF505E5E, 0xFF495555, 0xFF4B5556, 0xFF565E60, 0xFF596163, 0xFF555A5D, 0xFF566062, 0xFF5C6668, 0xFF5E686A, 0xFF5F696B, 0xFF616B6D, 0xFF606A6C, 0xFF5D6769, 0xFF5E686A, 0xFF515D5D, 0xFF4E5859, 0xFF4B5355, 0xFF4A5254, 0xFF4C5456,
        0xFF4D5959, 0xFF455554, 0xFF364A48, 0xFF364645, 0xFF4A5A59, 0xFF495757, 0xFF4D5B5B, 0xFF495555, 0xFF4A5656, 0xFF495354, 0xFF4A5455, 0xFF4C564B, 0xFF706D5A, 0xFF5B6157, 0xFF4D5651, 0xFF605C50, 0xFF50514B, 0xFF444F51, 0xFF5E5F59, 0xFFA17C5F,
        0xFFA68262, 0xFFA88764, 0xFF9B835F, 0xFF837455, 0xFF605B45, 0xFF4E5446, 0xFF4E5951, 0xFF495555, 0xFF455151, 0xFF445252, 0xFF455353, 0xFF435352, 0xFF455554, 0xFF485857, 0xFF445755, 0xFF435756, 0xFF455757, 0xFF4E5C5D, 0xFF4F5A5C, 0xFF4C5759,
        0xFF4F5D5E, 0xFF425454, 0xFF293D3C, 0xFF2C403F, 0xFF485858, 0xFF4C5858, 0xFF4F595A, 0xFF485253, 0xFF4C5856, 0xFF445350, 0xFF3B4B48, 0xFF476D60, 0xFF50796B, 0xFF537D6F, 0xFF557B6E, 0xFF517166, 0x00DAEDE7, 0x00FCFFFF, 0x00FFFEFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x94ECCBA8, 0xFFD99E64, 0xFFF0A258, 0xFFEDA35C, 0xFFEAA45E, 0xFFF3A55A, 0xFFF2A459, 0xFFE19D56, 0xFFE69C5B, 0xFFE79D5C, 0xFFE99F5E, 0xFFE99F5E, 0xFFE89E5D,
        0xFFE69C5B, 0xFFE69C5B, 0xFFE69C5B, 0xFFE8A362, 0xFFE6A15E, 0xFFE39D58, 0xFFE6A05B, 0xFFE09B58, 0xFFD79757, 0xFFDCA263, 0xFFD9A367, 0xFFDD9C5A, 0xFFE0A062, 0xFFD79C62, 0xFFBB8954, 0xFF8A6A3B, 0xFF564C28, 0xFF354427, 0xFF2D4A34, 0xFF384D3E,
        0xFF314D3E, 0xFF1E4234, 0xFF1F4037, 0xFF3F5852, 0xFF5D726D, 0xFF516F63, 0xFF355C4A, 0xFF466659, 0xFF537167, 0xFF263F3C, 0xFF495D5E, 0xFF46555A, 0xFF8F9A9E, 0xFF667072, 0xFF2D3635, 0xFFAAB7BF, 0xFFB8C5CD, 0xFFB2BFC5, 0xFF3B494C, 0xFF5E6C6D,
        0xFF9EACAC, 0xFF23322F, 0xFF142320, 0xFF1E2A28, 0xFF233632, 0xFF24433B, 0xFF1F4A40, 0xFF1A5042, 0xFF1F594B, 0xFF276455, 0xFF2E6B5C, 0xFF3B6C59, 0xFF3E5A4E, 0xFF999E9A, 0xFFB3AEAB, 0xFF9E9A97, 0xFF959891, 0xFF9E9F99, 0xFFB6B3AE, 0xFF909D96,
        0xFF526E62, 0xFF558170, 0xFF50816E, 0xFF537E6D, 0xFF507466, 0xFF557B6C, 0xFF497463, 0xFF7B8C84, 0xFFC4C3BF, 0xFFC0B9B3, 0xFFBABCB1, 0xFFB8BBB0, 0xFFBDB8B2, 0xFFB7B9B4, 0xFF5C7168, 0xFF4C7766, 0xFF4F7B6A, 0xFF538270, 0xFF4D7E6B, 0xFF21503E,
        0xFF244D3D, 0xFF25493B, 0xFF1B3B2E, 0xFF1A3D29, 0xFF1E442F, 0xFF1E442F, 0xFF143A25, 0xFF0D301C, 0xFF0D301C, 0xFF0F301D, 0xFF0A2917, 0xFF0F2A17, 0xFF0D2817, 0xFF0D2517, 0xFF10251C, 0xFF2F423E, 0xFF4C5A5A, 0xFF4F585D, 0xFF5C636B, 0xFF50605D,
        0xFF3B4B48, 0xFF243330, 0xFF1F2E2B, 0xFF21302D, 0xFF2B3A37, 0xFF3F4F4C, 0xFF495956, 0xFF425456, 0xFF3D4C4F, 0xFF475558, 0xFF485357, 0xFF3F4D50, 0xFF344648, 0xFF273C3D, 0xFF2C4645, 0xFF3D4949, 0xFF6A7370, 0xFFB1B2AC, 0xFFBBB7AE, 0xFFC5BBB2,
        0xFFC4B7AF, 0xFFBAADA7, 0xFFC2B4B1, 0xFFC1ACA7, 0xFFC3B6AE, 0xFFBBB4AA, 0xFFBFB6AF, 0xFFC1B2AD, 0xFFC4B5B0, 0xFFAFAAA4, 0xFF6D756A, 0xFF4E7767, 0xFF4D7867, 0xFF497564, 0xFF4A7665, 0xFF4B7464, 0xFF2D5143, 0xFF153327, 0xFF2B453A, 0xFF2E3E3D,
        0xFF3E4E4D, 0xFF475555, 0xFF445050, 0xFF465051, 0xFF4B5556, 0xFF4C5456, 0xFF484D50, 0xFF41494C, 0xFF474F52, 0xFF444C4F, 0xFF424A4D, 0xFF454D50, 0xFF41494C, 0xFF454D50, 0xFF575F62, 0xFF4F5759, 0xFF4D5557, 0xFF4C5657, 0xFF4E5859, 0xFF505C5C,
        0xFF4E5E5D, 0xFF495C5A, 0xFF415553, 0xFF465957, 0xFF5B6B6A, 0xFF586867, 0xFF5A6A69, 0xFF526060, 0xFF536161, 0xFF586464, 0xFF5E6A6A, 0xFF566159, 0xFF5B655A, 0xFF55645D, 0xFF596A64, 0xFF5F6C65, 0xFF5C6864, 0xFF5B6969, 0xFF5C6260, 0xFF625C50,
        0xFF6D675B, 0xFF6C695A, 0xFF666759, 0xFF5B6155, 0xFF57665F, 0xFF526768, 0xFF4F6B6E, 0xFF546060, 0xFF505E5E, 0xFF50605F, 0xFF4D605E, 0xFF485C5A, 0xFF445856, 0xFF475A58, 0xFF4C5C5B, 0xFF495D5C, 0xFF465858, 0xFF485657, 0xFF495456, 0xFF4A5557,
        0xFF4F5D5E, 0xFF465858, 0xFF344847, 0xFF3F5352, 0xFF536363, 0xFF546262, 0xFF566262, 0xFF505E5E, 0xFF586B67, 0xFF506761, 0xFF465F59, 0xFF4B7164, 0xFF527B6D, 0xFF537D6F, 0xFF567C6F, 0xFF537368, 0x00DBEEE8, 0x00FCFFFF, 0x00FFFEFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x94ECCBA8, 0xFFDA9D64, 0xFFF1A158, 0xFFEFA25A, 0xFFECA35E, 0xFFF3A55A, 0xFFF4A558, 0xFFE39D57, 0xFFE69D59, 0xFFE89F5B, 0xFFE9A05C, 0xFFE9A05C, 0xFFE89F5B,
        0xFFE69D59, 0xFFE69D59, 0xFFE69D59, 0xFFE5A05F, 0xFFE9A260, 0xFFE49B57, 0xFFE49B56, 0xFFE29D58, 0xFFDE9D5D, 0xFFD29658, 0xFFBA8247, 0xFFA27C4F, 0xFFA68155, 0xFF9E7D54, 0xFF7A6341, 0xFF463F25, 0xFF222B18, 0xFF19301E, 0xFF1B3A2A, 0xFF556356,
        0xFF395143, 0xFF224034, 0xFF254039, 0xFF384B47, 0xFF41504D, 0xFF364D43, 0xFF284A3A, 0xFF415B52, 0xFF3E554F, 0xFF445857, 0xFF718085, 0xFF6A777D, 0xFF636C71, 0xFF99A1A4, 0xFF515659, 0xFF849199, 0xFF8C99A1, 0xFF929FA5, 0xFF69777A, 0xFF808E8F,
        0xFF647272, 0xFF02110E, 0xFF23322F, 0xFF1B2421, 0xFF202F2A, 0xFF253C36, 0xFF22463C, 0xFF1E4C41, 0xFF1E5446, 0xFF245E50, 0xFF296657, 0xFF3A6A5A, 0xFF3B594F, 0xFF969C98, 0xFFABA7A6, 0xFF9D9C98, 0xFFA6A8A3, 0xFFA8ABA4, 0xFFB0AFAA, 0xFF88958E,
        0xFF4F6B5F, 0xFF578372, 0xFF4F806D, 0xFF537E6D, 0xFF55796B, 0xFF587E6F, 0xFF4A7564, 0xFF81928C, 0xFFC5C4C0, 0xFFBFB7B4, 0xFFBCBFB4, 0xFFBBBEB3, 0xFFBFBCB7, 0xFFBDC2BC, 0xFF647B73, 0xFF547668, 0xFF54786A, 0xFF4F7566, 0xFF406657, 0xFF15392B,
        0xFF1B3B2E, 0xFF1B372B, 0xFF0E251B, 0xFF22412F, 0xFF274432, 0xFF243C2C, 0xFF2A3F30, 0xFF384A3C, 0xFF354538, 0xFF2B3B2E, 0xFF313F32, 0xFF334A38, 0xFF2B4031, 0xFF2C3D33, 0xFF2C3932, 0xFF374140, 0xFF464E51, 0xFF585F65, 0xFF7C838B, 0xFF63726F,
        0xFF5D6C69, 0xFF41514E, 0xFF3F4F4C, 0xFF4A5A57, 0xFF455552, 0xFF4D5C59, 0xFF5A6966, 0xFF617073, 0xFF485659, 0xFF465155, 0xFF424D51, 0xFF414F52, 0xFF3A4E4F, 0xFF253D3D, 0xFF284543, 0xFF3D4746, 0xFF757B79, 0xFFB6B7B1, 0xFFBCB5AD, 0xFFC4B9B3,
        0xFFC4B7B1, 0xFFBAAFAB, 0xFFC1B6B4, 0xFFC4B7AF, 0xFFC4BDB3, 0xFFB6B4A8, 0xFFBAB3AB, 0xFFBEAFAA, 0xFFBFB4AE, 0xFFABACA4, 0xFF657568, 0xFF4C7565, 0xFF4F7868, 0xFF4F7A69, 0xFF527D6C, 0xFF4F7868, 0xFF2C4E40, 0xFF0C281C, 0xFF1E352B, 0xFF0B1F1E,
        0xFF102222, 0xFF152727, 0xFF1A2A2A, 0xFF1A2829, 0xFF1C2729, 0xFF1C2628, 0xFF1A2426, 0xFF172123, 0xFF1A2426, 0xFF162022, 0xFF162022, 0xFF172123, 0xFF0F191B, 0xFF1D2729, 0xFF414B4D, 0xFF212D2D, 0xFF1F2D2D, 0xFF1D2D2C, 0xFF1B2E2C, 0xFF192D2B,
        0xFF172B29, 0xFF172B29, 0xFF1A2E2C, 0xFF203432, 0xFF2D403E, 0xFF263937, 0xFF2C3F3D, 0xFF293938, 0xFF2A3A39, 0xFF2A3838, 0xFF2C3A3A, 0xFF303F3A, 0xFF213533, 0xFF213430, 0xFF293C36, 0xFF233C39, 0xFF2A3E3C, 0xFF394843, 0xFF2C3C3C, 0xFF2C4643,
        0xFF27403D, 0xFF233736, 0xFF2F4141, 0xFF344648, 0xFF2E4348, 0xFF1F383F, 0xFF1F3A45, 0xFF293334, 0xFF253333, 0xFF203432, 0xFF1D3632, 0xFF1F3834, 0xFF203633, 0xFF223533, 0xFF253534, 0xFF293D3C, 0xFF1C2E2E, 0xFF152324, 0xFF141F21, 0xFF162123,
        0xFF182627, 0xFF162828, 0xFF112524, 0xFF132525, 0xFF1C2C2C, 0xFF1C2C2B, 0xFF233332, 0xFF2F4240, 0xFF506963, 0xFF58776F, 0xFF587970, 0xFF51776A, 0xFF557E70, 0xFF547E70, 0xFF577D70, 0xFF55756A, 0x00DBEEE8, 0x00FBFFFE, 0x00FFFDFE, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x98EDCBA6, 0xFFDB9F63, 0xFFF1A257, 0xFFEFA25A, 0xFFEDA55D, 0xFFF4A558, 0xFFF4A558, 0xFFE39D55, 0xFFE79C59, 0xFFE99E5B, 0xFFEBA05D, 0xFFEA9F5C, 0xFFE99E5B,
        0xFFE79C59, 0xFFE69B58, 0xFFE69B58, 0xFFDE9755, 0xFFE6A05B, 0xFFE19853, 0xFFDF9651, 0xFFE7A15C, 0xFFE6A563, 0xFFC08445, 0xFF824A0D, 0xFF56472A, 0xFF4E3F22, 0xFF4A3E26, 0xFF3F4133, 0xFF293E39, 0xFF224445, 0xFF2A534F, 0xFF315951, 0xFF293126,
        0xFF213327, 0xFF375146, 0xFF5A6F6A, 0xFF5B6765, 0xFF404A49, 0xFF304139, 0xFF2C483A, 0xFF293E37, 0xFF192E29, 0xFF546668, 0xFF849398, 0xFF848E97, 0xFF6F787F, 0xFF98A0A3, 0xFF8A9294, 0xFF68727B, 0xFF6A747D, 0xFF778288, 0xFF929DA1, 0xFF8A9597,
        0xFF606C6C, 0xFF434F4D, 0xFF131F1D, 0xFF1D2623, 0xFF1C2824, 0xFF1E312B, 0xFF213F37, 0xFF22483D, 0xFF1F4D42, 0xFF23594B, 0xFF2B6455, 0xFF2A5A4A, 0xFF2C4A40, 0xFF959B99, 0xFFB4B0AF, 0xFFA6A5A3, 0xFFAAAFA9, 0xFFADAFAA, 0xFFBAB9B5, 0xFF84918A,
        0xFF365246, 0xFF4C7867, 0xFF4F806D, 0xFF517C6B, 0xFF5B7F71, 0xFF547A6B, 0xFF335E4D, 0xFF677A74, 0xFFC3C3C1, 0xFFC6C1BD, 0xFFC2C5BC, 0xFFBFC5BB, 0xFFC9C6C1, 0xFFBDC2BE, 0xFF485F57, 0xFF4D6E63, 0xFF5B7C71, 0xFF507468, 0xFF315247, 0xFF1A382E,
        0xFF3C534B, 0xFF5D6E68, 0xFF66726E, 0xFF53695D, 0xFF607167, 0xFF616E65, 0xFF666D66, 0xFF6F746E, 0xFF676C66, 0xFF5E635D, 0xFF646B64, 0xFF586C61, 0xFF56675D, 0xFF66736C, 0xFF6B7471, 0xFF5A6060, 0xFF4C5154, 0xFF535B5E, 0xFF687274, 0xFF697173,
        0xFF656F70, 0xFF4B5959, 0xFF566665, 0xFF677776, 0xFF526060, 0xFF545E5F, 0xFF6C7476, 0xFF5F6D6E, 0xFF465153, 0xFF465153, 0xFF434E50, 0xFF404E4F, 0xFF364A49, 0xFF1F3936, 0xFF25443F, 0xFF404944, 0xFF757A74, 0xFFB7B7AF, 0xFFC6BFB5, 0xFFC4BAB1,
        0xFFB9AEA8, 0xFFBBB0AE, 0xFFBDB3B2, 0xFFBAB3A9, 0xFFBBB9AD, 0xFFB4B4A8, 0xFFBFB8B0, 0xFFC3B6B0, 0xFFC2B9B2, 0xFFA8B0A5, 0xFF637A6A, 0xFF507667, 0xFF4F7868, 0xFF4D7867, 0xFF4E7968, 0xFF4F7566, 0xFF2E5042, 0xFF132F23, 0xFF273E34, 0xFF334C49,
        0xFF2D4643, 0xFF304644, 0xFF354948, 0xFF354747, 0xFF354747, 0xFF3A4A4A, 0xFF3B4B4B, 0xFF3A4C4C, 0xFF384A4A, 0xFF314343, 0xFF2D3F3F, 0xFF233535, 0xFF102222, 0xFF1C2E2E, 0xFF445656, 0xFF304643, 0xFF2D4642, 0xFF28433E, 0xFF24413C, 0xFF243F3A,
        0xFF223B37, 0xFF2B3F3D, 0xFF394948, 0xFF2D413F, 0xFF324644, 0xFF263A38, 0xFF314442, 0xFF354846, 0xFF374A48, 0xFF30403F, 0xFF293938, 0xFF283B35, 0xFF1D3B3D, 0xFF2D423D, 0xFF3C4C42, 0xFF254142, 0xFF243D39, 0xFF344439, 0xFF203D3B, 0xFF1D3C36,
        0xFF2A4540, 0xFF314543, 0xFF354544, 0xFF424E4E, 0xFF3D4949, 0xFF394747, 0xFF415150, 0xFF3F494A, 0xFF3A4848, 0xFF2E4441, 0xFF26433E, 0xFF2B4A44, 0xFF334E49, 0xFF344745, 0xFF334141, 0xFF3E5251, 0xFF354747, 0xFF324041, 0xFF374244, 0xFF394446,
        0xFF364445, 0xFF334545, 0xFF324645, 0xFF344646, 0xFF354545, 0xFF2E3E3D, 0xFF2C3F3D, 0xFF314744, 0xFF4F6E66, 0xFF53796E, 0xFF507A6E, 0xFF557B6E, 0xFF578072, 0xFF547E70, 0xFF587E71, 0xFF56766B, 0x00DCEFE9, 0x00FBFFFE, 0x00FFFCFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x94EECCA7, 0xFFDC9E63, 0xFFF1A257, 0xFFEFA25A, 0xFFEEA45D, 0xFFF5A659, 0xFFF4A556, 0xFFE59D55, 0xFFE89D59, 0xFFE99E5A, 0xFFEBA05C, 0xFFEA9F5B, 0xFFE99E5A,
        0xFFE79C58, 0xFFE69B57, 0xFFE69B57, 0xFFE59E5C, 0xFFEAA15D, 0xFFE59C57, 0xFFE49B56, 0xFFE49E59, 0xFFDF9E5C, 0xFFC18445, 0xFF8B5014, 0xFF765528, 0xFF67461B, 0xFF553B16, 0xFF3A341E, 0xFF1E2F29, 0xFF1E3E3D, 0xFF2B4E4A, 0xFF2C4C3F, 0xFF2E3128,
        0xFF334036, 0xFF3F544B, 0xFF3D4D4A, 0xFF2F3837, 0xFF313737, 0xFF2B3831, 0xFF11291C, 0xFF1A2F28, 0xFF1C2F2D, 0xFF1B2D2F, 0xFF627178, 0xFF828F97, 0xFF848F95, 0xFF919A9F, 0xFF7A8486, 0xFF757E87, 0xFF747D86, 0xFF757E85, 0xFF828B90, 0xFF828C8E,
        0xFF909A9B, 0xFF9EA8A7, 0xFF414B4A, 0xFF1C2824, 0xFF14201C, 0xFF13241E, 0xFF1E352D, 0xFF24433B, 0xFF254D42, 0xFF2D5D51, 0xFF3C6E62, 0xFF346756, 0xFF2E4C42, 0xFF999F9D, 0xFFBEBABB, 0xFFB1B1AF, 0xFFB3B8B4, 0xFFB5BAB4, 0xFFCACBC6, 0xFF8C9992,
        0xFF345044, 0xFF507C6B, 0xFF558673, 0xFF55806F, 0xFF577B6D, 0xFF446A5B, 0xFF2A5544, 0xFF7A8D89, 0xFFD3D3D1, 0xFFCFCAC7, 0xFFC3C9BF, 0xFFC1C7BD, 0xFFCFCBC8, 0xFFC8CDC9, 0xFF516862, 0xFF3E6A5B, 0xFF527E6F, 0xFF4B7567, 0xFF254E40, 0xFF27483D,
        0xFF4E685F, 0xFF8FA09A, 0xFFBAC5C1, 0xFFA0B5AE, 0xFFABBCB6, 0xFFB0BBB7, 0xFF9CA09F, 0xFF7E807F, 0xFF717574, 0xFF6D7673, 0xFF626E6A, 0xFF4F645F, 0xFF5B6B68, 0xFF778180, 0xFF787E7E, 0xFF4E5253, 0xFF414747, 0xFF5B6765, 0xFF677673, 0xFF6E6F73,
        0xFF686D70, 0xFF4E5A5A, 0xFF5A6868, 0xFF657373, 0xFF4D5959, 0xFF515659, 0xFF67686C, 0xFF5F6B6B, 0xFF4E5859, 0xFF5C6667, 0xFF5A6666, 0xFF4F5F5E, 0xFF3E5451, 0xFF24413C, 0xFF32534C, 0xFF4B524A, 0xFF888B82, 0xFFB7B7AB, 0xFFC1BBAF, 0xFFC4BCB1,
        0xFFBAB0A7, 0xFFC1B8B3, 0xFFC2BAB7, 0xFFC1B8AF, 0xFFBEBAAF, 0xFFB9B5AA, 0xFFBFB4AE, 0xFFC2B0AC, 0xFFC2B5AF, 0xFFA6ACA2, 0xFF667D6D, 0xFF517768, 0xFF4E7767, 0xFF4B7464, 0xFF4B7464, 0xFF4D7364, 0xFF315144, 0xFF193529, 0xFF31483E, 0xFF2B4947,
        0xFF254341, 0xFF264341, 0xFF2C4645, 0xFF2C4444, 0xFF324748, 0xFF354A4B, 0xFF344849, 0xFF2E4B47, 0xFF2C4945, 0xFF2A4743, 0xFF2A4743, 0xFF23403C, 0xFF16332F, 0xFF24413D, 0xFF46635F, 0xFF3D6059, 0xFF3B6058, 0xFF365E55, 0xFF386057, 0xFF3E615A,
        0xFF425D58, 0xFF4B5E5C, 0xFF5D6969, 0xFF3E5451, 0xFF415754, 0xFF354B48, 0xFF425654, 0xFF495D5B, 0xFF506361, 0xFF4B5E5C, 0xFF485B59, 0xFF324C41, 0xFF2D5258, 0xFF485B57, 0xFF5E665B, 0xFF486166, 0xFF3F5552, 0xFF445543, 0xFF29504B, 0xFF435853,
        0xFF61716E, 0xFF6C7876, 0xFF505A59, 0xFF505956, 0xFF3D4842, 0xFF405045, 0xFF4E6052, 0xFF61696B, 0xFF5E6E6D, 0xFF4B6460, 0xFF35564F, 0xFF33544D, 0xFF37544F, 0xFF3A4E4C, 0xFF3B4949, 0xFF394D4C, 0xFF3B4D4D, 0xFF414F50, 0xFF485355, 0xFF4A5557,
        0xFF465455, 0xFF405252, 0xFF3D5150, 0xFF415353, 0xFF425252, 0xFF3F5250, 0xFF3A4E4C, 0xFF37524D, 0xFF50746A, 0xFF4F7B6E, 0xFF4A7A6C, 0xFF567C6F, 0xFF567F71, 0xFF527C6E, 0xFF577D70, 0xFF56766B, 0x00DCEFE9, 0x00FBFFFE, 0x00FFFCFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x94EECCA7, 0xFFDC9E63, 0xFFF4A258, 0xFFF0A258, 0xFFEEA45D, 0xFFF5A659, 0xFFF6A556, 0xFFE59D55, 0xFFE89D59, 0xFFEA9F5B, 0xFFEBA05C, 0xFFEBA05C, 0xFFE99E5A,
        0xFFE79C58, 0xFFE69B57, 0xFFE69B57, 0xFFE69F5D, 0xFFE39A56, 0xFFE69B57, 0xFFEDA35C, 0xFFE29C56, 0xFFDB9756, 0xFFDB9E5F, 0xFFCE9357, 0xFFC9853C, 0xFFCE8B44, 0xFFBE8445, 0xFF7C5B30, 0xFF342F19, 0xFF1F2D1E, 0xFF253624, 0xFF1F2C12, 0xFF26271F,
        0xFF3E4A40, 0xFF3D4E46, 0xFF152421, 0xFF1A2020, 0xFF5C5D5F, 0xFF5F6A64, 0xFF192F23, 0xFF172C27, 0xFF122523, 0xFF1F3133, 0xFF37464D, 0xFF75828A, 0xFF7E8B91, 0xFF838E92, 0xFF808B8D, 0xFF747D86, 0xFF778089, 0xFF7B848B, 0xFF6B7479, 0xFF778183,
        0xFF929C9D, 0xFFA7B1B0, 0xFF9BA5A4, 0xFF2B3A33, 0xFF1A2924, 0xFF13241E, 0xFF1C312A, 0xFF223D36, 0xFF21453B, 0xFF2B554B, 0xFF3B695E, 0xFF3C6F60, 0xFF244439, 0xFF838C89, 0xFFADABAC, 0xFFB0B0AE, 0xFFB9BEBA, 0xFFB3B8B2, 0xFFBDBEB9, 0xFF7F8C85,
        0xFF304C40, 0xFF4F7B6A, 0xFF51826F, 0xFF55806F, 0xFF4D7163, 0xFF345A4B, 0xFF386352, 0xFF778A86, 0xFFC7C7C5, 0xFFC8C3C0, 0xFFCED4CA, 0xFFCAD0C6, 0xFFC3C2BE, 0xFFB5BBB7, 0xFF425B55, 0xFF417865, 0xFF508774, 0xFF4F8472, 0xFF346353, 0xFF345D4F,
        0xFF304E44, 0xFF4F625C, 0xFF72817C, 0xFF829B97, 0xFF90A3A1, 0xFFADB9B9, 0xFFA2AAAC, 0xFF7E8386, 0xFF818B8C, 0xFF839392, 0xFF5D716F, 0xFF526667, 0xFF7F8F8F, 0xFFB7C1C3, 0xFFB1B5B8, 0xFF626667, 0xFF424B48, 0xFF62716A, 0xFF64786F, 0xFF69686D,
        0xFF666A6D, 0xFF4F595A, 0xFF546262, 0xFF5A6868, 0xFF4E5859, 0xFF5A5E61, 0xFF656469, 0xFF606C6A, 0xFF4A5453, 0xFF555F5E, 0xFF535F5D, 0xFF495956, 0xFF384F49, 0xFF1B3931, 0xFF284940, 0xFF666E63, 0xFFB1B5A7, 0xFFC3C1B4, 0xFFB4AEA0, 0xFFC6BEB1,
        0xFFBEB6AB, 0xFFBEB7AF, 0xFFC1BAB4, 0xFFC7BAB2, 0xFFC3BAB1, 0xFFC3B9B0, 0xFFC5B3AF, 0xFFC6ACAB, 0xFFC6B3AF, 0xFFA6A69E, 0xFF67796B, 0xFF4F7566, 0xFF517768, 0xFF4F7868, 0xFF517A6A, 0xFF517768, 0xFF305043, 0xFF132D22, 0xFF263D33, 0xFF20433F,
        0xFF1E413D, 0xFF234340, 0xFF264442, 0xFF284543, 0xFF304A49, 0xFF334B4B, 0xFF2B4343, 0xFF254842, 0xFF234640, 0xFF22453F, 0xFF244741, 0xFF20433D, 0xFF1B3E38, 0xFF2B4E48, 0xFF466963, 0xFF416C62, 0xFF3B695E, 0xFF366459, 0xFF38665B, 0xFF3D655C,
        0xFF3A5752, 0xFF3C4F4D, 0xFF495354, 0xFF39524E, 0xFF405653, 0xFF324845, 0xFF374D4A, 0xFF374B49, 0xFF3C504E, 0xFF3F5250, 0xFF455856, 0xFF314D3F, 0xFF234C52, 0xFF32433D, 0xFF494B40, 0xFF3D545A, 0xFF3C524F, 0xFF41533D, 0xFF25524B, 0xFF3B4A4D,
        0xFF4C5A5D, 0xFF576266, 0xFF434E52, 0xFF4A5A59, 0xFF233831, 0xFF233F31, 0xFF395A47, 0xFF50585A, 0xFF566665, 0xFF46615C, 0xFF284B44, 0xFF1D4039, 0xFF25423D, 0xFF314543, 0xFF3B4949, 0xFF354948, 0xFF3B4D4D, 0xFF3E4C4D, 0xFF3D484A, 0xFF3D484A,
        0xFF3D4B4C, 0xFF3A4C4C, 0xFF384C4B, 0xFF3F4F4F, 0xFF405050, 0xFF405351, 0xFF384E4B, 0xFF304D48, 0xFF4B7166, 0xFF4D7B6E, 0xFF4C8171, 0xFF557B6E, 0xFF547D6F, 0xFF507A6C, 0xFF567C6F, 0xFF56766B, 0x00DCEFE9, 0x00FBFFFE, 0x00FFFDFE, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xB3EBC197, 0xFFE19F62, 0xFFF2A056, 0xFFEDA25E, 0xFFEDA664, 0xFFF3A55D, 0xFFF7A55B, 0xFFE69B57, 0xFFE69B58, 0xFFEA9F5C, 0xFFE89D59, 0xFFECA15D, 0xFFEBA05C,
        0xFFE79D56, 0xFFEBA15A, 0xFFE89E57, 0xFFE59F61, 0xFFEAA363, 0xFFE89E5D, 0xFFE19756, 0xFFE19A5A, 0xFFE39F62, 0xFFDB9D62, 0xFFD49760, 0xFFCF9C5D, 0xFFD59663, 0xFFC79359, 0xFFC2904B, 0xFFA6733C, 0xFF4E4A27, 0xFF234B31, 0xFF1B3824, 0xFF131F1B,
        0xFF192926, 0xFF1A2E2C, 0xFF344D4A, 0xFF152E2A, 0xFF344B45, 0xFF3B5047, 0xFF162A1E, 0xFF132926, 0xFF1C302E, 0xFF213432, 0xFF667675, 0xFF788484, 0xFF7B8586, 0xFF838B8D, 0xFF888D90, 0xFF828D93, 0xFF788188, 0xFF7E878C, 0xFF727B80, 0xFF727A7D,
        0xFF949C9F, 0xFFA2A7AA, 0xFF919699, 0xFF60686A, 0xFF132725, 0xFF152C26, 0xFF1C2826, 0xFF20332D, 0xFF204A3C, 0xFF225544, 0xFF396453, 0xFF466962, 0xFF244740, 0xFF2A4D46, 0xFF30514A, 0xFF466760, 0xFF587771, 0xFF4D6C66, 0xFF4B6A64, 0xFF5C716A,
        0xFF344F46, 0xFF638479, 0xFF597F72, 0xFF4C7567, 0xFF385C50, 0xFF4B6B60, 0xFF48635A, 0xFF5C6B68, 0xFF4B5B58, 0xFF60736F, 0xFF7F948F, 0xFF7B908B, 0xFF5A6D69, 0xFF566663, 0xFF465552, 0xFF4A7061, 0xFF587E6F, 0xFF557E6E, 0xFF4B7466, 0xFF315B4D,
        0xFF1A4639, 0xFF0F3D30, 0xFF1B493C, 0xFF326757, 0xFF2F6251, 0xFF2F5B4A, 0xFF2D5743, 0xFF31543E, 0xFF335139, 0xFF304A31, 0xFF2C442A, 0xFF2A4C31, 0xFF305038, 0xFF395640, 0xFF385042, 0xFF3A4E45, 0xFF465552, 0xFF566163, 0xFF6F787D, 0xFF6C7477,
        0xFF676F72, 0xFF4E585A, 0xFF5D686A, 0xFF576566, 0xFF4C5A5B, 0xFF526262, 0xFF5D6F6F, 0xFF5A6567, 0xFF475254, 0xFF596365, 0xFF616B6D, 0xFF4E595B, 0xFF364848, 0xFF243D3A, 0xFF294642, 0xFF697068, 0xFFB0B1A9, 0xFFBAB3AB, 0xFFBFB2AA, 0xFFC8BDB7,
        0xFFC5C5BD, 0xFFA7B4AB, 0xFF899E97, 0xFF6B9A88, 0xFF709486, 0xFF88A496, 0xFFAABAAF, 0xFFC8BDB9, 0xFFC7A9A9, 0xFFB1ADA2, 0xFF5B7E66, 0xFF4F7568, 0xFF527B6B, 0xFF517C69, 0xFF507A66, 0xFF4E7562, 0xFF2F4F42, 0xFF122D26, 0xFF253B38, 0xFF213E3C,
        0xFF22403E, 0xFF244441, 0xFF244743, 0xFF244743, 0xFF264442, 0xFF2A4443, 0xFF2C4444, 0xFF264540, 0xFF294644, 0xFF2A4445, 0xFF284145, 0xFF224042, 0xFF1C3F3B, 0xFF295349, 0xFF427063, 0xFF406C5D, 0xFF3C6B5B, 0xFF366958, 0xFF386B5A, 0xFF3E6A5B,
        0xFF39594E, 0xFF354A43, 0xFF3C4844, 0xFF43544A, 0xFF3C4B44, 0xFF39675A, 0xFF2D6355, 0xFF2F4542, 0xFF40504D, 0xFF2F493E, 0xFF465048, 0xFF34584C, 0xFF265D4A, 0xFF2F5347, 0xFF33443E, 0xFF3C564D, 0xFF344942, 0xFF435D54, 0xFF26614D, 0xFF305647,
        0xFF344B41, 0xFF444F49, 0xFF434C47, 0xFF4C6158, 0xFF3F5D51, 0xFF466659, 0xFF355145, 0xFF4C514B, 0xFF5C7369, 0xFF3D6C5C, 0xFF245947, 0xFF1A4639, 0xFF274640, 0xFF2F4845, 0xFF374D4B, 0xFF384E4B, 0xFF3A4E4C, 0xFF3B4F4D, 0xFF3B4E4C, 0xFF3A4D4B,
        0xFF3C4C4B, 0xFF3D4D4C, 0xFF404E4E, 0xFF384C4D, 0xFF3C5051, 0xFF3A4C4C, 0xFF374B49, 0xFF3A534D, 0xFF537368, 0xFF507669, 0xFF4C7766, 0xFF527C6E, 0xFF527E6F, 0xFF537F70, 0xFF568072, 0xFF527267, 0x00D9ECE6, 0x00FCFFFF, 0x00FFFDFE, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xB3EBC197, 0xFFE1A060, 0xFFF2A056, 0xFFEDA25E, 0xFFEDA664, 0xFFF3A55B, 0xFFF7A55B, 0xFFE69B57, 0xFFE69B58, 0xFFEA9F5C, 0xFFE89D59, 0xFFEBA05C, 0xFFE99E5A,
        0xFFE59B54, 0xFFE89E57, 0xFFE69C55, 0xFFDF995B, 0xFFE29B5B, 0xFFE59B5A, 0xFFE39958, 0xFFE59E5E, 0xFFE29E61, 0xFFC5874C, 0xFFA46730, 0xFF9C6930, 0xFFA16437, 0xFFB17E47, 0xFFCB9857, 0xFFBB8853, 0xFF605A3A, 0xFF31543C, 0xFF2B4431, 0xFF192521,
        0xFF293936, 0xFF243836, 0xFF465F5C, 0xFF253E3B, 0xFF203731, 0xFF243930, 0xFF14281C, 0xFF162C29, 0xFF102422, 0xFF2B3E3C, 0xFF738382, 0xFF808C8C, 0xFF869091, 0xFF7D8587, 0xFF767B7E, 0xFF859096, 0xFF798289, 0xFF7B8489, 0xFF727B80, 0xFF6F777A,
        0xFF878F92, 0xFF989DA0, 0xFF979C9F, 0xFF737B7D, 0xFF233735, 0xFF1A312B, 0xFF192523, 0xFF172A24, 0xFF1C4638, 0xFF255847, 0xFF426C5E, 0xFF426A61, 0xFF2E534B, 0xFF2E534B, 0xFF254A42, 0xFF21443D, 0xFF2F524B, 0xFF2A4B44, 0xFF33544D, 0xFF536D64,
        0xFF4F6D63, 0xFF54786C, 0xFF4D7668, 0xFF396355, 0xFF3F6558, 0xFF305146, 0xFF2A4A3F, 0xFF576E68, 0xFF39524C, 0xFF39544D, 0xFF435E57, 0xFF425D56, 0xFF39544D, 0xFF465F59, 0xFF435A54, 0xFF4B7665, 0xFF537E6D, 0xFF5B8675, 0xFF4F786A, 0xFF365C4F,
        0xFF1B3F35, 0xFF1A3E34, 0xFF25493F, 0xFF275145, 0xFF1E463B, 0xFF163C2F, 0xFF123426, 0xFF0C2B1C, 0xFF082314, 0xFF0A2212, 0xFF0E2515, 0xFF09220F, 0xFF0E2616, 0xFF091F12, 0xFF0F2318, 0xFF374842, 0xFF4B5A57, 0xFF424E4E, 0xFF485355, 0xFF556062,
        0xFF566163, 0xFF475254, 0xFF596466, 0xFF525D5F, 0xFF444F51, 0xFF4A5557, 0xFF586365, 0xFF5F6A6C, 0xFF4C5759, 0xFF5F696B, 0xFF667072, 0xFF535E60, 0xFF394B4B, 0xFF253E3B, 0xFF294642, 0xFF6B7069, 0xFFAEAEA6, 0xFFB3ACA4, 0xFFC4BAB1, 0xFFC6BFB7,
        0xFF9FA59B, 0xFF768A7F, 0xFF628076, 0xFF598473, 0xFF587C6E, 0xFF5E8070, 0xFF80988A, 0xFFB5B2AB, 0xFFCCB4B2, 0xFFAEAA9F, 0xFF5C7E66, 0xFF4F7568, 0xFF527B6B, 0xFF507B68, 0xFF4F7965, 0xFF4E7562, 0xFF2F4F42, 0xFF122D26, 0xFF253B38, 0xFF203D3B,
        0xFF213F3D, 0xFF22423F, 0xFF20433F, 0xFF224541, 0xFF274543, 0xFF2C4645, 0xFF2E4646, 0xFF274440, 0xFF284543, 0xFF294344, 0xFF284145, 0xFF224042, 0xFF1C3F3B, 0xFF285248, 0xFF406F5F, 0xFF3F6B5C, 0xFF3A6959, 0xFF336655, 0xFF376757, 0xFF3C6658,
        0xFF3B5B50, 0xFF3B5049, 0xFF46524E, 0xFF38483E, 0xFF3E4D46, 0xFF325E51, 0xFF2D6355, 0xFF304643, 0xFF3B4B48, 0xFF375146, 0xFF49534B, 0xFF3D5D52, 0xFF306352, 0xFF2D4E43, 0xFF36423E, 0xFF3E534C, 0xFF30413B, 0xFF445952, 0xFF2B624F, 0xFF325648,
        0xFF32493F, 0xFF4A534E, 0xFF434C47, 0xFF51655C, 0xFF4D6B5F, 0xFF4A685C, 0xFF365246, 0xFF4D524C, 0xFF5B7268, 0xFF3C6B5B, 0xFF245947, 0xFF1A4639, 0xFF274640, 0xFF2F4845, 0xFF374D4B, 0xFF354B48, 0xFF374B49, 0xFF394D4B, 0xFF394D4B, 0xFF394C4A,
        0xFF394C4A, 0xFF3C4C4B, 0xFF3D4D4C, 0xFF374B4C, 0xFF3B4F50, 0xFF3A4C4C, 0xFF384C4A, 0xFF3B544E, 0xFF55756A, 0xFF52786B, 0xFF4E7968, 0xFF517B6D, 0xFF517D6E, 0xFF527E6F, 0xFF557F71, 0xFF537368, 0x00DAEDE7, 0x00FCFFFF, 0x00FFFEFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xB7EBC296, 0xFFE09F5F, 0xFFF2A056, 0xFFEDA35C, 0xFFEDA762, 0xFFF3A55B, 0xFFF7A55B, 0xFFE59B54, 0xFFE69B58, 0xFFEA9F5C, 0xFFE79C58, 0xFFEA9F5B, 0xFFE89D59,
        0xFFE39952, 0xFFE69C55, 0xFFE39952, 0xFFE6A062, 0xFFE59E5E, 0xFFE79D5C, 0xFFE49B57, 0xFFE49D5B, 0xFFE29E5F, 0xFFBA7C41, 0xFF85480F, 0xFF693B09, 0xFF723812, 0xFF9E6C3B, 0xFFC69457, 0xFFBA8455, 0xFF635A3B, 0xFF32523B, 0xFF304534, 0xFF424E4A,
        0xFF5C6C69, 0xFF394D4C, 0xFF47605D, 0xFF2F4845, 0xFF223933, 0xFF293E35, 0xFF172B20, 0xFF152B29, 0xFF1D3130, 0xFF384A4A, 0xFF677777, 0xFF7F8A8C, 0xFF8F999B, 0xFF7C8487, 0xFF808589, 0xFF7F8A90, 0xFF767F86, 0xFF7A8388, 0xFF7B8489, 0xFF788083,
        0xFF7F878A, 0xFF92979A, 0xFFA3A8AB, 0xFF8A9294, 0xFF344846, 0xFF223835, 0xFF232F2D, 0xFF1A2D29, 0xFF1B4537, 0xFF215443, 0xFF416B5D, 0xFF49746A, 0xFF3A645A, 0xFF355F55, 0xFF335D53, 0xFF264E45, 0xFF3A6259, 0xFF2B5048, 0xFF375C54, 0xFF416156,
        0xFF4B6F63, 0xFF527B6D, 0xFF315B4D, 0xFF376354, 0xFF345E50, 0xFF456E60, 0xFF466C5F, 0xFF4E7268, 0xFF3A5E54, 0xFF3D6157, 0xFF40645A, 0xFF41655B, 0xFF43675D, 0xFF486C62, 0xFF41655B, 0xFF457865, 0xFF497A67, 0xFF558170, 0xFF466C5F, 0xFF3E5E53,
        0xFF2C453F, 0xFF324742, 0xFF2F423E, 0xFF314F47, 0xFF2F4A43, 0xFF324D46, 0xFF39524E, 0xFF394F4C, 0xFF314543, 0xFF2D413F, 0xFF324543, 0xFF374641, 0xFF31403B, 0xFF293833, 0xFF2A3936, 0xFF3F4E4B, 0xFF4B5A57, 0xFF404E4E, 0xFF404E4E, 0xFF354747,
        0xFF405050, 0xFF3F4F4F, 0xFF586667, 0xFF556062, 0xFF495355, 0xFF50585B, 0xFF64696D, 0xFF596466, 0xFF485355, 0xFF5B6567, 0xFF646E70, 0xFF515C5E, 0xFF374949, 0xFF223B38, 0xFF26433F, 0xFF6D6E68, 0xFFB4B1AA, 0xFFB7B0A8, 0xFFC6BDB4, 0xFFB3B3A9,
        0xFF778578, 0xFF587768, 0xFF547E70, 0xFF547A6B, 0xFF5B8172, 0xFF517A68, 0xFF557A68, 0xFF858F84, 0xFFC6B7B2, 0xFFACAA9E, 0xFF607D67, 0xFF4F7568, 0xFF517A6A, 0xFF4F7A67, 0xFF4F7965, 0xFF4E7562, 0xFF2E4E41, 0xFF122D26, 0xFF263C39, 0xFF1F3C3A,
        0xFF203E3C, 0xFF1F3F3C, 0xFF1C3F3B, 0xFF1F423E, 0xFF294745, 0xFF2F4948, 0xFF2F4747, 0xFF284540, 0xFF2A4441, 0xFF284243, 0xFF284145, 0xFF244041, 0xFF1D403C, 0xFF285246, 0xFF406C5D, 0xFF3F695B, 0xFF3A6959, 0xFF356555, 0xFF376757, 0xFF3D6759,
        0xFF3C5C51, 0xFF40534D, 0xFF4B5753, 0xFF3D4A41, 0xFF45524B, 0xFF366054, 0xFF2E6255, 0xFF344846, 0xFF42514E, 0xFF384F45, 0xFF4E534D, 0xFF3D584F, 0xFF336252, 0xFF2E4940, 0xFF3D4341, 0xFF475651, 0xFF424D49, 0xFF505F5A, 0xFF2C5C4C, 0xFF305446,
        0xFF33483F, 0xFF4C5550, 0xFF484E4A, 0xFF5A6B63, 0xFF597569, 0xFF4D6B5F, 0xFF375146, 0xFF4F544E, 0xFF5A7167, 0xFF3B6A5A, 0xFF235846, 0xFF194538, 0xFF26453F, 0xFF304946, 0xFF374D4B, 0xFF334746, 0xFF354948, 0xFF374B4A, 0xFF374B4A, 0xFF374B4A,
        0xFF364A49, 0xFF364A49, 0xFF374B4A, 0xFF34494A, 0xFF3A4E4F, 0xFF384C4B, 0xFF374B49, 0xFF3B544E, 0xFF56766B, 0xFF53796C, 0xFF507B6A, 0xFF527C6E, 0xFF517D6E, 0xFF517D6E, 0xFF557F71, 0xFF547469, 0x00DCEFE9, 0x00FCFFFF, 0x00FFFEFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xB7EBC296, 0xFFE09F5F, 0xFFF2A054, 0xFFECA25B, 0xFFEDA762, 0xFFF3A55B, 0xFFF7A559, 0xFFE59B54, 0xFFE59A57, 0xFFEA9F5C, 0xFFE79C58, 0xFFEA9F5B, 0xFFE89D59,
        0xFFE49A53, 0xFFE79D56, 0xFFE49A53, 0xFFE29D5C, 0xFFE29B59, 0xFFE69D59, 0xFFE49B57, 0xFFE39C5A, 0xFFE6A263, 0xFFC28447, 0xFF884B12, 0xFF704415, 0xFF78401D, 0xFFA8784A, 0xFFC9975C, 0xFFB47E50, 0xFF62593C, 0xFF36533D, 0xFF374939, 0xFF5A6664,
        0xFF788887, 0xFF455958, 0xFF445C5C, 0xFF354E4B, 0xFF283E3B, 0xFF364B44, 0xFF1F3328, 0xFF1A302E, 0xFF203433, 0xFF384A4A, 0xFF637373, 0xFF747F81, 0xFF7A8486, 0xFF70787B, 0xFF808589, 0xFF788389, 0xFF747D84, 0xFF7C858A, 0xFF879095, 0xFF848C8F,
        0xFF7D8588, 0xFF8D9295, 0xFFAAAFB2, 0xFF98A0A3, 0xFF3A4E4D, 0xFF213734, 0xFF303C3C, 0xFF243733, 0xFF1B4539, 0xFF1A4D3C, 0xFF3E685A, 0xFF457366, 0xFF477568, 0xFF4B796C, 0xFF537F72, 0xFF325E51, 0xFF446E62, 0xFF386256, 0xFF5A8478, 0xFF587E71,
        0xFF527B6D, 0xFF355F51, 0xFF3A6657, 0xFF2E5A4B, 0xFF487465, 0xFF517D6E, 0xFF527E6F, 0xFF528171, 0xFF4E7A6B, 0xFF588274, 0xFF598274, 0xFF598274, 0xFF5C8678, 0xFF5A8677, 0xFF518070, 0xFF4A846E, 0xFF497E6A, 0xFF477664, 0xFF284C40, 0xFF3A544B,
        0xFF485855, 0xFF707978, 0xFF7D8383, 0xFF5D706C, 0xFF5D706E, 0xFF617472, 0xFF657777, 0xFF637577, 0xFF5A6C70, 0xFF53646B, 0xFF516269, 0xFF64696D, 0xFF50585B, 0xFF61696C, 0xFF636D6E, 0xFF4A5656, 0xFF445350, 0xFF435350, 0xFF364643, 0xFF2E4442,
        0xFF394D4C, 0xFF3E5050, 0xFF5B696A, 0xFF5B6567, 0xFF51565A, 0xFF53565B, 0xFF66676C, 0xFF596466, 0xFF475254, 0xFF5C6668, 0xFF656F71, 0xFF535E60, 0xFF3B4D4D, 0xFF28413E, 0xFF2C4945, 0xFF6C6B66, 0xFFB9B4AE, 0xFFBBB4AC, 0xFFC1BAB0, 0xFF9CA296,
        0xFF627969, 0xFF547D6B, 0xFF508573, 0xFF5B7F71, 0xFF5E8475, 0xFF558470, 0xFF4B7B65, 0xFF627768, 0xFFB7AEA7, 0xFFACAA9E, 0xFF67806B, 0xFF507669, 0xFF507969, 0xFF4E7966, 0xFF4F7965, 0xFF4E7562, 0xFF2E4E41, 0xFF122D26, 0xFF273D3A, 0xFF1E3B39,
        0xFF203E3C, 0xFF1F3F3C, 0xFF1A3D39, 0xFF1D403C, 0xFF284644, 0xFF2E4847, 0xFF2D4545, 0xFF294641, 0xFF2B4542, 0xFF2A4242, 0xFF2B4345, 0xFF264243, 0xFF22423D, 0xFF2A5247, 0xFF3F6B5C, 0xFF406A5C, 0xFF3E6A5B, 0xFF396959, 0xFF3B6B5B, 0xFF3F695B,
        0xFF3D5D52, 0xFF3F524C, 0xFF4B5652, 0xFF465248, 0xFF4A544C, 0xFF396254, 0xFF2E5E50, 0xFF384845, 0xFF4D5955, 0xFF34483D, 0xFF52554E, 0xFF3A4F46, 0xFF234C3C, 0xFF32473E, 0xFF454744, 0xFF414C46, 0xFF444A46, 0xFF47524C, 0xFF2A5645, 0xFF315345,
        0xFF41554C, 0xFF505652, 0xFF484D49, 0xFF586961, 0xFF556F64, 0xFF577367, 0xFF3C5349, 0xFF535852, 0xFF5A7167, 0xFF396858, 0xFF235846, 0xFF184437, 0xFF26453F, 0xFF304946, 0xFF374D4B, 0xFF314544, 0xFF334746, 0xFF344A48, 0xFF354B49, 0xFF334947,
        0xFF324846, 0xFF314745, 0xFF2F4845, 0xFF304546, 0xFF354A4B, 0xFF364A49, 0xFF344A47, 0xFF39544D, 0xFF54756A, 0xFF517A6C, 0xFF507C6B, 0xFF547E70, 0xFF527E6F, 0xFF517D6E, 0xFF557F71, 0xFF537368, 0x00DBEEE8, 0x00FCFFFF, 0x00FFFEFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xBAEAC193, 0xFFE09F5D, 0xFFF19F53, 0xFFECA259, 0xFFECA661, 0xFFF3A55A, 0xFFF7A559, 0xFFE59B52, 0xFFE59A57, 0xFFEA9F5C, 0xFFE89D59, 0xFFEBA05C, 0xFFEA9F5B,
        0xFFE69C55, 0xFFEAA059, 0xFFE79D56, 0xFFE29D5C, 0xFFE39C5A, 0xFFE89F5B, 0xFFE69D58, 0xFFE49E59, 0xFFE4A061, 0xFFBD7F42, 0xFF83460D, 0xFF744615, 0xFF753B16, 0xFFA26F42, 0xFFC49257, 0xFFB37D4F, 0xFF635A3D, 0xFF365641, 0xFF394E3F, 0xFF54605E,
        0xFF6E7E7D, 0xFF4D6162, 0xFF536B6B, 0xFF425A5A, 0xFF273D3A, 0xFF394E47, 0xFF2D4138, 0xFF24393A, 0xFF15292A, 0xFF2E4042, 0xFF707F82, 0xFF7A8589, 0xFF70797E, 0xFF71787E, 0xFF767B81, 0xFF778288, 0xFF757E85, 0xFF778085, 0xFF818A8F, 0xFF80888B,
        0xFF777F82, 0xFF8A8F92, 0xFFA8ADB0, 0xFF8E9699, 0xFF334746, 0xFF1A302E, 0xFF384444, 0xFF283B39, 0xFF1D473B, 0xFF1C4F40, 0xFF467064, 0xFF49796B, 0xFF4A7A6C, 0xFF4C7A6D, 0xFF538174, 0xFF2C584B, 0xFF386457, 0xFF2F5B4E, 0xFF5F897D, 0xFF557F71,
        0xFF4B7567, 0xFF2A5446, 0xFF3F695B, 0xFF3B6557, 0xFF548071, 0xFF568575, 0xFF518171, 0xFF508070, 0xFF558172, 0xFF5B8476, 0xFF5F8377, 0xFF5A7E72, 0xFF5A8375, 0xFF5A8677, 0xFF598979, 0xFF4E8570, 0xFF4F8470, 0xFF4B7766, 0xFF284C40, 0xFF3A544B,
        0xFF495855, 0xFF828B8A, 0xFFACB0B1, 0xFFA2B5B1, 0xFFA7BAB6, 0xFFA4B7B5, 0xFF95A7A7, 0xFF809294, 0xFF6D7F81, 0xFF5D6F73, 0xFF53646B, 0xFF53585C, 0xFF4E5659, 0xFF747C7F, 0xFF798384, 0xFF525E5E, 0xFF4B5A57, 0xFF465653, 0xFF283835, 0xFF2D4341,
        0xFF344847, 0xFF3A4C4C, 0xFF586667, 0xFF5B6567, 0xFF54595D, 0xFF515459, 0xFF626368, 0xFF5A6567, 0xFF475254, 0xFF5A6466, 0xFF636D6F, 0xFF515C5E, 0xFF3A4C4C, 0xFF28413E, 0xFF2D4A46, 0xFF706D68, 0xFFB4AFA9, 0xFFB9B0A9, 0xFFBCB8AD, 0xFF91978B,
        0xFF5B7363, 0xFF537E6B, 0xFF487F6C, 0xFF608274, 0xFF4C7263, 0xFF4E7F6A, 0xFF4D816A, 0xFF597161, 0xFFAEA79F, 0xFFACAA9E, 0xFF69806C, 0xFF507669, 0xFF4F7868, 0xFF4D7865, 0xFF4F7965, 0xFF4E7562, 0xFF2D4D40, 0xFF122D26, 0xFF283E3B, 0xFF1F3C3A,
        0xFF203E3C, 0xFF1F3F3C, 0xFF1B3E3A, 0xFF1D403C, 0xFF254341, 0xFF2A4443, 0xFF2A4242, 0xFF2D4841, 0xFF2E4744, 0xFF2D4545, 0xFF2E4648, 0xFF2C4645, 0xFF24443F, 0xFF2B5446, 0xFF416C5B, 0xFF426B5D, 0xFF406C5D, 0xFF3C6B5B, 0xFF3D6C5C, 0xFF40695B,
        0xFF3E5C52, 0xFF43544E, 0xFF4F5A56, 0xFF576158, 0xFF676E67, 0xFF3A5E52, 0xFF346255, 0xFF3F4E4B, 0xFF4E5754, 0xFF4C5D53, 0xFF70716B, 0xFF4F635A, 0xFF204637, 0xFF40544B, 0xFF535250, 0xFF4F5551, 0xFF6E706D, 0xFF585E5A, 0xFF2D5646, 0xFF2F4F42,
        0xFF475850, 0xFF515652, 0xFF6B6D6A, 0xFF72817A, 0xFF465D53, 0xFF516B60, 0xFF3F544B, 0xFF595E58, 0xFF5B7268, 0xFF386757, 0xFF235846, 0xFF174336, 0xFF26453F, 0xFF314A47, 0xFF364C4A, 0xFF324647, 0xFF344849, 0xFF34494A, 0xFF324A4A, 0xFF2F4747,
        0xFF2B4544, 0xFF294342, 0xFF274442, 0xFF2B4343, 0xFF334849, 0xFF324846, 0xFF324845, 0xFF38534C, 0xFF54756A, 0xFF517A6C, 0xFF4F7B6A, 0xFF557F71, 0xFF548071, 0xFF527E6F, 0xFF557F71, 0xFF527267, 0x00D9ECE6, 0x00FCFFFF, 0x00FFFDFE, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xBAEAC193, 0xFFE09F5D, 0xFFF1A051, 0xFFECA259, 0xFFECA660, 0xFFF2A459, 0xFFF6A556, 0xFFE59B52, 0xFFE69B58, 0xFFEA9F5C, 0xFFE99E5A, 0xFFECA15D, 0xFFEBA05C,
        0xFFE79D56, 0xFFEBA15A, 0xFFE99F58, 0xFFEDA865, 0xFFE9A35E, 0xFFE89F5A, 0xFFE69D58, 0xFFE59F5A, 0xFFE39F5E, 0xFFC08344, 0xFF93571B, 0xFF86511B, 0xFF814217, 0xFF9E6937, 0xFFC28F50, 0xFFB68252, 0xFF605B3E, 0xFF2D5240, 0xFF2B4538, 0xFF556161,
        0xFF677777, 0xFF4F6364, 0xFF486062, 0xFF3E5656, 0xFF2E4442, 0xFF445954, 0xFF3F534A, 0xFF1D3233, 0xFF263A3B, 0xFF253739, 0xFF4A595C, 0xFF737E82, 0xFF7C858A, 0xFF798086, 0xFF7C8187, 0xFF747F85, 0xFF747D84, 0xFF6F787D, 0xFF737C81, 0xFF767E81,
        0xFF788083, 0xFF8E9396, 0xFFA0A5A8, 0xFF6D747A, 0xFF344847, 0xFF2C4240, 0xFF4B5658, 0xFF2D403E, 0xFF254F45, 0xFF245748, 0xFF4B7569, 0xFF4D7C6C, 0xFF4A7969, 0xFF487767, 0xFF507C6D, 0xFF325E4F, 0xFF3D6759, 0xFF2F594B, 0xFF578173, 0xFF517D6E,
        0xFF4D7769, 0xFF3F6558, 0xFF365C4F, 0xFF4C7265, 0xFF4E786A, 0xFF568575, 0xFF497969, 0xFF3D6857, 0xFF456B5C, 0xFF476559, 0xFF567065, 0xFF506A5F, 0xFF436155, 0xFF416758, 0xFF477261, 0xFF4F806D, 0xFF4E7D6B, 0xFF547F6E, 0xFF4B6F63, 0xFF506B62,
        0xFF3F5650, 0xFF5A6A67, 0xFF7B8A87, 0xFF67857B, 0xFF79948B, 0xFF809B92, 0xFF779188, 0xFF6B827A, 0xFF667B74, 0xFF637871, 0xFF61746E, 0xFF667570, 0xFF92A19C, 0xFFB2C1BC, 0xFF899895, 0xFF495855, 0xFF384744, 0xFF3E4C4C, 0xFF425050, 0xFF324444,
        0xFF364646, 0xFF3B4B4B, 0xFF566465, 0xFF5C6769, 0xFF566062, 0xFF51595C, 0xFF62676B, 0xFF5C6769, 0xFF485355, 0xFF5A6466, 0xFF616B6D, 0xFF4E595B, 0xFF354747, 0xFF233C39, 0xFF284541, 0xFF6F6E69, 0xFFB0ABA5, 0xFFBCB3AC, 0xFFC3BCB2, 0xFF93968B,
        0xFF556C5C, 0xFF4E7765, 0xFF538675, 0xFF5F8173, 0xFF55796B, 0xFF54836F, 0xFF43745E, 0xFF536859, 0xFFACA39C, 0xFFB2AEA3, 0xFF667D69, 0xFF507669, 0xFF4E7767, 0xFF4C7764, 0xFF4F7965, 0xFF4D7461, 0xFF2C4C3F, 0xFF122D26, 0xFF2A403D, 0xFF203D3B,
        0xFF1F3D3B, 0xFF1F3F3C, 0xFF1C3F3B, 0xFF1C3F3B, 0xFF203E3C, 0xFF26403F, 0xFF2A4242, 0xFF2E4942, 0xFF304945, 0xFF324748, 0xFF334849, 0xFF2D4746, 0xFF26453F, 0xFF2E5447, 0xFF426D5C, 0xFF446D5F, 0xFF416B5D, 0xFF3B6A5A, 0xFF3A6959, 0xFF3D6658,
        0xFF3F5D53, 0xFF4A5B55, 0xFF5B6461, 0xFFB7BDB3, 0xFFBEC3BD, 0xFF4B6D5F, 0xFF3C665A, 0xFF434F4D, 0xFF555B57, 0xFF96A69C, 0xFFD3D0C9, 0xFF5F7368, 0xFF294E3D, 0xFF43574C, 0xFF504F4B, 0xFF767D76, 0xFFD4D6D1, 0xFF8E958E, 0xFF1F4836, 0xFF3E5C50,
        0xFF44534C, 0xFF646663, 0xFFC4C6C3, 0xFFC6D3CC, 0xFF475C53, 0xFF435A50, 0xFF4F635A, 0xFF5F645E, 0xFF5C7369, 0xFF396858, 0xFF245947, 0xFF174336, 0xFF25443E, 0xFF314A47, 0xFF364C4A, 0xFF344849, 0xFF35494A, 0xFF34494A, 0xFF304848, 0xFF2C4645,
        0xFF274442, 0xFF244240, 0xFF234340, 0xFF2A4242, 0xFF314647, 0xFF314745, 0xFF304945, 0xFF35534B, 0xFF517569, 0xFF507A6C, 0xFF4D7C6A, 0xFF557F71, 0xFF548071, 0xFF548071, 0xFF557F71, 0xFF507065, 0x00D6E9E3, 0x00FBFFFE, 0x00FFFDFE, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xBAEAC193, 0xFFDF9E5A, 0xFFF1A051, 0xFFECA257, 0xFFECA660, 0xFFF2A557, 0xFFF6A556, 0xFFE49A4F, 0xFFE79C59, 0xFFEBA05D, 0xFFE99E5A, 0xFFECA15D, 0xFFEBA05C,
        0xFFE79D56, 0xFFEAA059, 0xFFE89E57, 0xFFE9A461, 0xFFE6A05B, 0xFFE49B56, 0xFFE49B56, 0xFFE7A15C, 0xFFE7A362, 0xFFD79A5B, 0xFFC4884C, 0xFFC28848, 0xFFC3804C, 0xFFC48C53, 0xFFC79250, 0xFFAE7C49, 0xFF5A593B, 0xFF2C5744, 0xFF214237, 0xFF263232,
        0xFF354545, 0xFF2B3F40, 0xFF1B3335, 0xFF223A3C, 0xFF253B39, 0xFF3F544F, 0xFF50645B, 0xFF516669, 0xFF697C80, 0xFF627478, 0xFF516065, 0xFF69747A, 0xFF808990, 0xFF7B828A, 0xFF80848D, 0xFF778288, 0xFF7E878E, 0xFF798287, 0xFF798287, 0xFF7D8588,
        0xFF858D90, 0xFF92979A, 0xFF8A8F92, 0xFF585F65, 0xFF536768, 0xFF5F7573, 0xFF677274, 0xFF2D3F3F, 0xFF2D574D, 0xFF2F6155, 0xFF4B756B, 0xFF477364, 0xFF517B6D, 0xFF588274, 0xFF517B6D, 0xFF345D4F, 0xFF3C6557, 0xFF375D50, 0xFF5E8477, 0xFF588274,
        0xFF4C7567, 0xFF305448, 0xFF416156, 0xFF416257, 0xFF62887B, 0xFF558172, 0xFF417161, 0xFF37594B, 0xFF415B50, 0xFF3F5048, 0xFF626D67, 0xFF5F6A64, 0xFF3F5048, 0xFF354F44, 0xFF385A4C, 0xFF597F70, 0xFF507667, 0xFF567C6D, 0xFF5A7E72, 0xFF4C6D62,
        0xFF38574F, 0xFF3A5951, 0xFF426159, 0xFF345F4E, 0xFF376050, 0xFF355C4A, 0xFF2C4E3D, 0xFF254432, 0xFF284330, 0xFF2E4732, 0xFF314834, 0xFF253E2B, 0xFF2E4636, 0xFF243A2D, 0xFF172B20, 0xFF354640, 0xFF4F5E5B, 0xFF475353, 0xFF414C4E, 0xFF434E50,
        0xFF434E50, 0xFF485355, 0xFF5D686A, 0xFF5C6769, 0xFF545F61, 0xFF4C5759, 0xFF5C6769, 0xFF616C6E, 0xFF4E595B, 0xFF616B6D, 0xFF687274, 0xFF545F61, 0xFF3B4D4D, 0xFF27403D, 0xFF2C4945, 0xFF6E6D68, 0xFFB1ACA6, 0xFFBEB5AE, 0xFFC7BEB5, 0xFFA4A49A,
        0xFF637366, 0xFF4A6A5B, 0xFF537F70, 0xFF557769, 0xFF587C6E, 0xFF4B7663, 0xFF345E4A, 0xFF69776A, 0xFFB8A9A4, 0xFFB0ACA1, 0xFF657E69, 0xFF507669, 0xFF4D7666, 0xFF4B7663, 0xFF4E7864, 0xFF4D7461, 0xFF2C4C3F, 0xFF112C25, 0xFF2A403D, 0xFF223F3D,
        0xFF1E3C3A, 0xFF1D3D3A, 0xFF1C3F3B, 0xFF1A3D39, 0xFF1B3937, 0xFF243E3D, 0xFF2E4646, 0xFF2D4640, 0xFF304945, 0xFF344A48, 0xFF34494A, 0xFF2E4744, 0xFF25443E, 0xFF2D5346, 0xFF426D5A, 0xFF426B5D, 0xFF416B5D, 0xFF3D695A, 0xFF3C6859, 0xFF3F6558,
        0xFF425D54, 0xFF4D5E58, 0xFF606966, 0xFFE7EAE1, 0xFFDCDFD8, 0xFF517164, 0xFF396254, 0xFF45504C, 0xFF626763, 0xFFABB8AE, 0xFFF8F3ED, 0xFF667A6F, 0xFF406755, 0xFF55695E, 0xFF5E5F5A, 0xFF8B958D, 0xFFF2F7F1, 0xFFA3ADA5, 0xFF2E5946, 0xFF526E62,
        0xFF4E5D56, 0xFF71716F, 0xFFD6D6D4, 0xFFD5E0DA, 0xFF4B6057, 0xFF465B52, 0xFF5F736A, 0xFF646963, 0xFF5E756B, 0xFF396858, 0xFF255A48, 0xFF174336, 0xFF25443E, 0xFF314A47, 0xFF354B49, 0xFF35484C, 0xFF35484C, 0xFF33484B, 0xFF2E4849, 0xFF294546,
        0xFF244443, 0xFF214342, 0xFF214342, 0xFF2B4343, 0xFF304848, 0xFF324846, 0xFF314A46, 0xFF36544C, 0xFF52766A, 0xFF517B6D, 0xFF4F7E6C, 0xFF537D6F, 0xFF537F70, 0xFF548071, 0xFF547E70, 0xFF4E6E63, 0x00D4E7E1, 0x00FAFEFD, 0x00FFFEFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xBAEAC193, 0xFFDF9E5A, 0xFFF1A051, 0xFFEBA156, 0xFFECA660, 0xFFF2A557, 0xFFF6A556, 0xFFE49A4F, 0xFFE89D5A, 0xFFECA15E, 0xFFE99E5A, 0xFFECA15D, 0xFFEA9F5B,
        0xFFE59B54, 0xFFE89E57, 0xFFE69C55, 0xFFE49F5C, 0xFFE7A15C, 0xFFE9A05B, 0xFFE8A058, 0xFFE59F59, 0xFFE09C5B, 0xFFDA9D5E, 0xFFDCA064, 0xFFD5964F, 0xFFE6A065, 0xFFD19758, 0xFFAE7933, 0xFF8C5A25, 0xFF484B2E, 0xFF2D5C4A, 0xFF1F453A, 0xFF475353,
        0xFF4F5F5F, 0xFF54676B, 0xFF455D5F, 0xFF3E5658, 0xFF203634, 0xFF2C413C, 0xFF5D706A, 0xFF576C6F, 0xFF506367, 0xFF809296, 0xFF7A898E, 0xFF586369, 0xFF707980, 0xFF868D95, 0xFF858992, 0xFF828D93, 0xFF9099A0, 0xFF90999E, 0xFF8E979C, 0xFF8E9699,
        0xFF929A9D, 0xFF8F9497, 0xFF6E7376, 0xFF5C6369, 0xFF7A8E8F, 0xFF8DA2A3, 0xFF778284, 0xFF243636, 0xFF2D574D, 0xFF386A5E, 0xFF4F796F, 0xFF527C6E, 0xFF547D6F, 0xFF4F786A, 0xFF3B6456, 0xFF294F42, 0xFF385E51, 0xFF315549, 0xFF4B6F63, 0xFF537D6F,
        0xFF33594C, 0xFF416156, 0xFF426056, 0xFF415F55, 0xFF4A6E62, 0xFF5B8577, 0xFF3C6B5B, 0xFF405A4F, 0xFF50645B, 0xFF58615C, 0xFF929491, 0xFF929491, 0xFF5D6661, 0xFF41554C, 0xFF3A5449, 0xFF59796C, 0xFF527466, 0xFF5C7E70, 0xFF567A6E, 0xFF2D5346,
        0xFF20483D, 0xFF204A3E, 0xFF164034, 0xFF174C38, 0xFF10442E, 0xFF0C3922, 0xFF0C361E, 0xFF12361C, 0xFF153317, 0xFF142E11, 0xFF10290B, 0xFF0E3015, 0xFF103018, 0xFF07240E, 0xFF0D2517, 0xFF354940, 0xFF495855, 0xFF3E494B, 0xFF434C51, 0xFF40484B,
        0xFF434B4E, 0xFF495355, 0xFF5C6769, 0xFF586667, 0xFF515F60, 0xFF495959, 0xFF5B6D6D, 0xFF5D686A, 0xFF4B5658, 0xFF5F696B, 0xFF687274, 0xFF556062, 0xFF3B4D4D, 0xFF28413E, 0xFF2B4844, 0xFF6E6F69, 0xFFB0ADA6, 0xFFB8AFA8, 0xFFC6BCB3, 0xFFBDBAB1,
        0xFF7D897D, 0xFF3D594B, 0xFF33594C, 0xFF44685A, 0xFF3C5E50, 0xFF244B39, 0xFF315342, 0xFF99A196, 0xFFC9B7B3, 0xFFA8A197, 0xFF6A836E, 0xFF507669, 0xFF4D7666, 0xFF4A7562, 0xFF4E7864, 0xFF4D7461, 0xFF2B4B3E, 0xFF112C25, 0xFF2B413E, 0xFF23403E,
        0xFF1D3B39, 0xFF1B3B38, 0xFF1C3F3B, 0xFF193C38, 0xFF183634, 0xFF243E3D, 0xFF324A4A, 0xFF2C453F, 0xFF324845, 0xFF344A48, 0xFF34494A, 0xFF2D4643, 0xFF23423C, 0xFF2C5245, 0xFF436C5A, 0xFF42685B, 0xFF416B5D, 0xFF3F6B5C, 0xFF406C5D, 0xFF42685B,
        0xFF435E55, 0xFF4D5C57, 0xFF5E6764, 0xFF8A8D84, 0xFF868781, 0xFF3A584C, 0xFF386153, 0xFF505956, 0xFF616360, 0xFF5D695F, 0xFF98938D, 0xFF4B6155, 0xFF355E4C, 0xFF546A5E, 0xFF5E605B, 0xFF58635B, 0xFF868D86, 0xFF5D675F, 0xFF3A6753, 0xFF496559,
        0xFF58655E, 0xFF5E5E5C, 0xFF7B7A78, 0xFF79847E, 0xFF374B42, 0xFF4B6057, 0xFF56675F, 0xFF676C66, 0xFF5F766C, 0xFF3A6959, 0xFF255A48, 0xFF174336, 0xFF25443E, 0xFF314A47, 0xFF344A48, 0xFF34474B, 0xFF34474B, 0xFF33484B, 0xFF2D4748, 0xFF274545,
        0xFF244443, 0xFF224443, 0xFF224644, 0xFF2C4444, 0xFF324A4A, 0xFF314A47, 0xFF324B47, 0xFF38564E, 0xFF54786C, 0xFF527C6E, 0xFF507F6D, 0xFF507A6C, 0xFF527E6F, 0xFF537F70, 0xFF547E70, 0xFF4D6D62, 0x00D3E6E0, 0x00FBFFFE, 0x00FFFEFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xBAEBC294, 0xFFE2A15D, 0xFFF3A253, 0xFFEDA35A, 0xFFECA661, 0xFFF2A45A, 0xFFF7A55B, 0xFFE79C58, 0xFFE99E5A, 0xFFEA9F5B, 0xFFECA15D, 0xFFEBA05C, 0xFFEA9F5B,
        0xFFE99E5A, 0xFFE99E5A, 0xFFE99E5A, 0xFFEBA162, 0xFFE99E5A, 0xFFE89E55, 0xFFE39B4F, 0xFFE8A25A, 0xFFE6A467, 0xFFC38755, 0xFFA46B40, 0xFF946B3D, 0xFF8F6F46, 0xFF726240, 0xFF4E4E34, 0xFF1D2C17, 0xFF142D1A, 0xFF486755, 0xFF30533F, 0xFF636361,
        0xFF85898A, 0xFF889294, 0xFF7B8A8D, 0xFF53666A, 0xFF293D3E, 0xFF283A3A, 0xFF617472, 0xFF7D8587, 0xFF707A7B, 0xFF647070, 0xFF303E3F, 0xFF263838, 0xFF4A5F60, 0xFF667E7E, 0xFF6A8282, 0xFF797D80, 0xFF969A9D, 0xFFBABBBF, 0xFFA5A9AC, 0xFF91999B,
        0xFF768685, 0xFF4E6461, 0xFF26413C, 0xFF5A5F62, 0xFF737D7E, 0xFFA3B1B1, 0xFF364B46, 0xFF2E4942, 0xFF4D6E65, 0xFF476D60, 0xFF517A6C, 0xFF517C6B, 0xFF517D6C, 0xFF557E6E, 0xFF203C30, 0xFF323F38, 0xFF535E58, 0xFF3D544A, 0xFF3E6254, 0xFF527668,
        0xFF374E44, 0xFF79847E, 0xFF65726B, 0xFF445E53, 0xFF446A5B, 0xFF547D6D, 0xFF517567, 0xFF6E6D51, 0xFF988A67, 0xFF947D5B, 0xFF958063, 0xFF927855, 0xFF947B52, 0xFF7C7D5E, 0xFF426358, 0xFF4C7867, 0xFF537E6D, 0xFF547A6B, 0xFF37594B, 0xFF4A685C,
        0xFF445E53, 0xFF3F564C, 0xFF253A31, 0xFF324B47, 0xFF38534E, 0xFF38534E, 0xFF385550, 0xFF375650, 0xFF2C4B45, 0xFF24453E, 0xFF24453E, 0xFF3F4B49, 0xFF4E5A58, 0xFF394543, 0xFF394545, 0xFF434D4E, 0xFF485254, 0xFF434D4F, 0xFF475153, 0xFF3E4A48,
        0xFF414D4B, 0xFF4B5554, 0xFF5B6564, 0xFF5F6968, 0xFF515A59, 0xFF4F5857, 0xFF616A69, 0xFF566465, 0xFF4B595A, 0xFF5B6B6B, 0xFF637575, 0xFF516564, 0xFF394F4D, 0xFF2E4442, 0xFF304946, 0xFF74736F, 0xFFB0ACA9, 0xFFBBB3B0, 0xFFC3B9B7, 0xFFC4BBB6,
        0xFFBDB6B0, 0xFF928F86, 0xFF57594E, 0xFF3F4537, 0xFF3D4536, 0xFF61695A, 0xFF9D9D91, 0xFFB9AFA6, 0xFFBBB1A8, 0xFFA3A799, 0xFF6D806D, 0xFF517A68, 0xFF537C6A, 0xFF517866, 0xFF557769, 0xFF567669, 0xFF2E4C42, 0xFF0F2A21, 0xFF284239, 0xFF1C443C,
        0xFF20403B, 0xFF203935, 0xFF2C3C39, 0xFF333C39, 0xFF727675, 0xFFBEC0BD, 0xFFC5C7C2, 0xFF4F6562, 0xFF293F3D, 0xFF2A4242, 0xFF2B4748, 0xFF2A4848, 0xFF21413C, 0xFF30544A, 0xFF476D5E, 0xFF4B695D, 0xFF527466, 0xFF426B5B, 0xFF416C5B, 0xFF406657,
        0xFF4C665B, 0xFF5C6761, 0xFF545653, 0xFF363B35, 0xFF384C41, 0xFF426756, 0xFF365F4D, 0xFF426253, 0xFF586E62, 0xFF374D41, 0xFF344C3F, 0xFF365D4B, 0xFF425C4F, 0xFF48534B, 0xFF707B73, 0xFF3F5549, 0xFF254536, 0xFF395B4B, 0xFF456455, 0xFF2A4F3E,
        0xFF64756B, 0xFF696B66, 0xFF3C4940, 0xFF2F5443, 0xFF3E6B57, 0xFF355445, 0xFF5B655D, 0xFF7F7E7A, 0xFF5C6D65, 0xFF446D5F, 0xFF255545, 0xFF22483D, 0xFF304945, 0xFF3D4F4F, 0xFF384848, 0xFF344C4E, 0xFF2E4648, 0xFF3C5154, 0xFF283C3D, 0xFF2D4243,
        0xFF284241, 0xFF1E3E39, 0xFF1D403A, 0xFF26453D, 0xFF314F47, 0xFF36514A, 0xFF3E584F, 0xFF3E584F, 0xFF557165, 0xFF5C7C6F, 0xFF54786A, 0xFF5A7E70, 0xFF597F70, 0xFF5A8373, 0xFF5C8273, 0xFF4E6C60, 0x00D5E6DE, 0x00FEFFFD, 0x00FFFDFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xBAEBC294, 0xFFE2A15D, 0xFFF4A354, 0xFFEDA35A, 0xFFECA661, 0xFFF2A45A, 0xFFF7A55B, 0xFFE79C58, 0xFFE99E5A, 0xFFEBA05C, 0xFFECA15D, 0xFFEBA05C, 0xFFEA9F5B,
        0xFFE99E5A, 0xFFE99E5A, 0xFFEA9F5B, 0xFFE89E5F, 0xFFE79C58, 0xFFEAA057, 0xFFE8A054, 0xFFE9A35B, 0xFFDF9D60, 0xFFB67A46, 0xFF955C2F, 0xFF847268, 0xFF71645B, 0xFF4E4B44, 0xFF373D39, 0xFF1F2E2B, 0xFF223531, 0xFF506761, 0xFF354C44, 0xFF3C423E,
        0xFF65706C, 0xFF839392, 0xFF6A7E7F, 0xFF385050, 0xFF536B6B, 0xFF6C827F, 0xFF576C67, 0xFF515D5D, 0xFF738181, 0xFF566464, 0xFF1D2D2D, 0xFF223232, 0xFF243636, 0xFF36484A, 0xFF4D6162, 0xFF526664, 0xFF728583, 0xFF859393, 0xFF697575, 0xFF5B6767,
        0xFF3F4D4D, 0xFF213432, 0xFF1A2E2C, 0xFF4A5254, 0xFF8C9898, 0xFF566665, 0xFF1F342F, 0xFF58736C, 0xFF4F7065, 0xFF547A6D, 0xFF588173, 0xFF588372, 0xFF54806F, 0xFF517A6A, 0xFF365246, 0xFF637069, 0xFF7C8781, 0xFF52695F, 0xFF3D6153, 0xFF55796B,
        0xFF42594F, 0xFF929D97, 0xFF8C9992, 0xFF4A6459, 0xFF446A5B, 0xFF5D8676, 0xFF486C5E, 0xFF777657, 0xFFAB9B77, 0xFFAA946F, 0xFFAB9474, 0xFFAC926D, 0xFFB59D71, 0xFF919270, 0xFF446658, 0xFF578372, 0xFF537E6D, 0xFF517567, 0xFF314F43, 0xFF475E54,
        0xFF42514A, 0xFF707B75, 0xFF7D8681, 0xFF5E7270, 0xFF627674, 0xFF5D716F, 0xFF596D6B, 0xFF586C6A, 0xFF516563, 0xFF465A58, 0xFF3F5552, 0xFF485855, 0xFF647370, 0xFF697875, 0xFF687676, 0xFF525E5E, 0xFF485355, 0xFF454F51, 0xFF434D4F, 0xFF445252,
        0xFF3E4C4C, 0xFF485454, 0xFF5E6A6A, 0xFF667071, 0xFF555F60, 0xFF545C5E, 0xFF6A7274, 0xFF616C6E, 0xFF495456, 0xFF435152, 0xFF465656, 0xFF3F5151, 0xFF324645, 0xFF293F3D, 0xFF314745, 0xFF6F6E6A, 0xFFB1ADAA, 0xFFBFB7B4, 0xFFBFB5B3, 0xFFBCB3AE,
        0xFFC1BAB4, 0xFFBFBCB3, 0xFFB5B5AB, 0xFFB4B6A9, 0xFFADB3A5, 0xFFB2B8AA, 0xFFC0BCB1, 0xFFC3B6AE, 0xFFC6B9B1, 0xFFA8AA9D, 0xFF657664, 0xFF517A68, 0xFF537C6A, 0xFF517866, 0xFF547668, 0xFF557568, 0xFF2E4C42, 0xFF0F2A21, 0xFF29433A, 0xFF194139,
        0xFF1F3F3A, 0xFF27403C, 0xFF2D3C39, 0xFF9FA8A5, 0xFFFCFFFF, 0xFFFCFFFD, 0xFFC6C8C3, 0xFF455B58, 0xFF2B413F, 0xFF2C4444, 0xFF2B4546, 0xFF284646, 0xFF20403B, 0xFF2E5248, 0xFF486E5F, 0xFF47695B, 0xFF4D7364, 0xFF45705F, 0xFF497463, 0xFF426658,
        0xFF485F55, 0xFF6D7671, 0xFF61615F, 0xFF2F362F, 0xFF34453B, 0xFF425E50, 0xFF3C584A, 0xFF63746A, 0xFF69736B, 0xFF323D35, 0xFF304137, 0xFF355A49, 0xFF485C51, 0xFF595E58, 0xFF797A75, 0xFF536057, 0xFF2A4235, 0xFF365244, 0xFF4A6457, 0xFF45564C,
        0xFF767873, 0xFF6D6966, 0xFF343E36, 0xFF284839, 0xFF4C7361, 0xFF546A5E, 0xFF73746F, 0xFF8A8985, 0xFF4C5D55, 0xFF3C6557, 0xFF295949, 0xFF20463B, 0xFF2A433F, 0xFF394B4B, 0xFF384848, 0xFF2D4746, 0xFF3A5252, 0xFF324645, 0xFF354747, 0xFF586A6A,
        0xFF263A38, 0xFF102925, 0xFF122D28, 0xFF253C36, 0xFF354A45, 0xFF364B46, 0xFF3A4F48, 0xFF3F564E, 0xFF5C786C, 0xFF618375, 0xFF567C6D, 0xFF5A7E70, 0xFF597F70, 0xFF5A8373, 0xFF5C8273, 0xFF4E6C60, 0x00D5E6DE, 0x00FDFFFC, 0x00FFFDFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xBAEBC294, 0xFFE1A05C, 0xFFF4A354, 0xFFEEA45B, 0xFFEDA762, 0xFFF3A55B, 0xFFF8A65C, 0xFFE79C58, 0xFFEA9F5B, 0xFFEBA05C, 0xFFECA15D, 0xFFECA15D, 0xFFEA9F5B,
        0xFFE99E5A, 0xFFE99E5A, 0xFFEA9F5B, 0xFFEAA05F, 0xFFE99E5A, 0xFFECA259, 0xFFE8A056, 0xFFE7A15B, 0xFFE09F5F, 0xFFC58A52, 0xFFB37B4A, 0xFFAC8252, 0xFF9D7B4D, 0xFF866F46, 0xFF6E6546, 0xFF49533B, 0xFF334B3B, 0xFF4A6F67, 0xFF2F5C55, 0xFF233329,
        0xFF354940, 0xFF3C5551, 0xFF2A4743, 0xFF395755, 0xFF93B2AD, 0xFFA4C1BC, 0xFF5A756C, 0xFF3D524D, 0xFF405550, 0xFF7B8E8C, 0xFF667675, 0xFF313F40, 0xFF243233, 0xFF293436, 0xFF293438, 0xFF284741, 0xFF223D38, 0xFF425855, 0xFF748483, 0xFF7E8A8A,
        0xFF455151, 0xFF2C3A3A, 0xFF50605F, 0xFF9DA9A9, 0xFF647272, 0xFF2B3E3A, 0xFF3A534D, 0xFF5A786E, 0xFF537469, 0xFF608679, 0xFF537C6C, 0xFF4D7867, 0xFF558170, 0xFF547D6D, 0xFF3A564A, 0xFF606D66, 0xFF69746E, 0xFF4B6258, 0xFF476B5D, 0xFF537769,
        0xFF40574D, 0xFF76817B, 0xFF75827B, 0xFF3A5449, 0xFF426859, 0xFF5B8474, 0xFF44685A, 0xFF75724F, 0xFFA99A71, 0xFFAF986F, 0xFFAF9974, 0xFFB3976F, 0xFFBA9F70, 0xFF8C8C66, 0xFF385946, 0xFF4D7867, 0xFF527B6B, 0xFF5B7D6F, 0xFF3B574B, 0xFF42564D,
        0xFF3F4A44, 0xFF838985, 0xFFA8AAA7, 0xFFA4B9B4, 0xFFA2B5B1, 0xFF91A4A0, 0xFF7A8A87, 0xFF6D7C79, 0xFF62716E, 0xFF576361, 0xFF515D5B, 0xFF425750, 0xFF637672, 0xFF7D908C, 0xFF7E8E8B, 0xFF526060, 0xFF455353, 0xFF465153, 0xFF3C4749, 0xFF3D4D4C,
        0xFF3C4C4B, 0xFF445453, 0xFF546262, 0xFF556363, 0xFF4B5757, 0xFF4E5A5A, 0xFF636F6F, 0xFF616B6D, 0xFF4C5658, 0xFF404B4D, 0xFF3F4D4E, 0xFF435353, 0xFF3A4C4C, 0xFF304242, 0xFF394D4C, 0xFF4F504B, 0xFF9D9C98, 0xFFBFBAB6, 0xFFC5BDBA, 0xFFC2B9B4,
        0xFFBBB4AE, 0xFFBAB7AE, 0xFFC4C4BA, 0xFFBFBBB0, 0xFFBDBDB1, 0xFFBBBBAF, 0xFFBBB2A9, 0xFFBCABA4, 0xFFC6B7B0, 0xFFACACA0, 0xFF687665, 0xFF527B69, 0xFF537C6A, 0xFF517866, 0xFF547668, 0xFF557568, 0xFF2E4C42, 0xFF102B22, 0xFF29433A, 0xFF224740,
        0xFF21403B, 0xFF3D5350, 0xFF273633, 0xFF828B88, 0xFFB9BFBD, 0xFFB8BDB9, 0xFF575C56, 0xFF344846, 0xFF2E4442, 0xFF2F4747, 0xFF2B4546, 0xFF284646, 0xFF1F3F3A, 0xFF2C5046, 0xFF4A6E60, 0xFF456E5E, 0xFF467160, 0xFF3F6B5A, 0xFF477060, 0xFF49695C,
        0xFF60746B, 0xFFB3B9B5, 0xFF969593, 0xFF607066, 0xFF42584C, 0xFF4C6457, 0xFF56665C, 0xFFA5A6A1, 0xFFAAA6A3, 0xFF636560, 0xFF4E5B52, 0xFF3E6352, 0xFF4A5E53, 0xFF83827E, 0xFFB1ACA9, 0xFF838882, 0xFF475B50, 0xFF435D50, 0xFF496557, 0xFF61605C,
        0xFFBBB1B0, 0xFFA8A09E, 0xFF4A554D, 0xFF335344, 0xFF537565, 0xFF66776D, 0xFFA29E9B, 0xFFBEBDB9, 0xFF495D54, 0xFF345D4F, 0xFF2C5C4C, 0xFF1B4338, 0xFF27403C, 0xFF354747, 0xFF364848, 0xFF37524D, 0xFF3D5350, 0xFF293C38, 0xFF51605D, 0xFF8A9692,
        0xFF2E3A36, 0xFF25342F, 0xFF293A32, 0xFF4C5655, 0xFF475150, 0xFF3A4443, 0xFF3F4E49, 0xFF485D56, 0xFF5B776B, 0xFF55796B, 0xFF527D6C, 0xFF597D6F, 0xFF587E6F, 0xFF5A8373, 0xFF5B8172, 0xFF4D6B5F, 0x00D4E5DD, 0x00FDFFFC, 0x00FFFCFC, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xBAEAC193, 0xFFE1A05C, 0xFFF4A354, 0xFFEEA45B, 0xFFEEA863, 0xFFF3A55B, 0xFFF8A65C, 0xFFE79C58, 0xFFEA9F5B, 0xFFEBA05C, 0xFFEDA25E, 0xFFECA15D, 0xFFEBA05C,
        0xFFEA9F5B, 0xFFEA9F5B, 0xFFEBA05C, 0xFFECA35F, 0xFFEA9F5B, 0xFFECA25B, 0xFFE69E56, 0xFFE49E59, 0xFFE4A363, 0xFFDA9F63, 0xFFD8A169, 0xFFE29E49, 0xFFDB9C4C, 0xFFCB974C, 0xFFA78545, 0xFF6F6331, 0xFF404B29, 0xFF496A55, 0xFF3E6A5B, 0xFF405B4C,
        0xFF233F33, 0xFF2C4D44, 0xFF294E46, 0xFF1C413A, 0xFF5F847C, 0xFF80A198, 0xFF3B5B50, 0xFF426058, 0xFF2A4840, 0xFF39524C, 0xFF768A88, 0xFF869695, 0xFF545F61, 0xFF485053, 0xFF34393D, 0xFF263C39, 0xFF1F3230, 0xFF445252, 0xFF768081, 0xFF81898B,
        0xFF778182, 0xFF8A9696, 0xFFAAB8B8, 0xFF50635F, 0xFF273C37, 0xFF344B45, 0xFF4D685F, 0xFF58786D, 0xFF64887C, 0xFF578070, 0xFF547F6E, 0xFF527D6C, 0xFF578372, 0xFF567F6F, 0xFF345044, 0xFF4F5C55, 0xFF4C5751, 0xFF364D43, 0xFF436759, 0xFF507466,
        0xFF384F45, 0xFF4D5852, 0xFF49564F, 0xFF304A3F, 0xFF4E7465, 0xFF588171, 0xFF4F7365, 0xFF86825C, 0xFFB2A175, 0xFFB59D71, 0xFFB69E78, 0xFFB79A6E, 0xFFBB9F6D, 0xFF94926B, 0xFF4F6F5A, 0xFF567F6D, 0xFF577E6C, 0xFF557A69, 0xFF3E5E4F, 0xFF3C584A,
        0xFF4D6558, 0xFF566A5F, 0xFF56675D, 0xFF658077, 0xFF708B82, 0xFF79938A, 0xFF758C84, 0xFF71867F, 0xFF6B7E78, 0xFF63746E, 0xFF65746F, 0xFF617C73, 0xFF86A097, 0xFFA3BAB2, 0xFF8FA49F, 0xFF4A5D59, 0xFF374746, 0xFF414F4F, 0xFF475353, 0xFF374B49,
        0xFF3A4E4C, 0xFF3D504E, 0xFF3D504E, 0xFF3F4F4E, 0xFF40504F, 0xFF445453, 0xFF4B5959, 0xFF4B5557, 0xFF454F51, 0xFF404B4D, 0xFF3F4D4E, 0xFF455555, 0xFF3B4B4B, 0xFF2E4040, 0xFF3D4F4F, 0xFF3F443E, 0xFF8C8D88, 0xFFB6B3AE, 0xFFC0B8B5, 0xFFC6BFB9,
        0xFFBFB8B2, 0xFFB5B1A8, 0xFFBCB9B0, 0xFFC3BAB1, 0xFFBDB9AE, 0xFFBBB7AC, 0xFFBFB5AC, 0xFFC0ADA7, 0xFFC1B0A9, 0xFFA4A498, 0xFF687665, 0xFF507967, 0xFF537C6A, 0xFF517866, 0xFF557769, 0xFF567669, 0xFF2E4C42, 0xFF0F2A21, 0xFF284239, 0xFF244741,
        0xFF1E3B37, 0xFF3E5451, 0xFF4A5956, 0xFF8E9794, 0xFFA4AAA8, 0xFFA7ADA9, 0xFF525952, 0xFF203432, 0xFF2F4342, 0xFF34494A, 0xFF2D4748, 0xFF2B4748, 0xFF22413C, 0xFF2C4D44, 0xFF4A6E60, 0xFF467563, 0xFF42715F, 0xFF3E6A59, 0xFF446A5B, 0xFF4D6B5F,
        0xFF899A92, 0xFFFCFFFD, 0xFFBEBDBB, 0xFF517164, 0xFF3E6052, 0xFF516D61, 0xFF76837C, 0xFFF4F3F1, 0xFFE2DEDD, 0xFF666F6A, 0xFF445950, 0xFF426E5D, 0xFF4D675C, 0xFFB5BAB6, 0xFFFFFFFD, 0xFF9FA8A3, 0xFF536D62, 0xFF3A5C4E, 0xFF2C5042, 0xFF776E6F,
        0xFFFFFAFB, 0xFFD5D5D3, 0xFF4D6259, 0xFF3B6152, 0xFF4A7061, 0xFF5F746B, 0xFFD3D5D2, 0xFFFEFFFA, 0xFF576B62, 0xFF305A4C, 0xFF2B5B4B, 0xFF1A4237, 0xFF29443F, 0xFF354948, 0xFF364848, 0xFF304B42, 0xFF1E352D, 0xFF2F3E39, 0xFF515A55, 0xFF5D635F,
        0xFF424741, 0xFF9A9F99, 0xFF777E77, 0xFF858587, 0xFF717274, 0xFF4D5152, 0xFF3F4845, 0xFF465953, 0xFF5C7A6E, 0xFF547A6B, 0xFF527E6D, 0xFF597D6F, 0xFF587E6F, 0xFF5A8373, 0xFF5B8172, 0xFF4D6B5F, 0x00D4E5DD, 0x00FDFFFC, 0x00FFFCFC, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xBEE9C092, 0xFFE1A05C, 0xFFF4A354, 0xFFEFA55C, 0xFFEFA964, 0xFFF4A65C, 0xFFF9A75D, 0xFFE89D59, 0xFFEBA05C, 0xFFECA15D, 0xFFEDA25E, 0xFFEDA25E, 0xFFEBA05C,
        0xFFEA9F5B, 0xFFEA9F5B, 0xFFEBA05C, 0xFFEAA15C, 0xFFE89D59, 0xFFEBA05C, 0xFFE89F5A, 0xFFE6A05B, 0xFFE1A05E, 0xFFCE9454, 0xFFC48E50, 0xFFC18C56, 0xFFC28E55, 0xFFC6935A, 0xFFBE905C, 0xFFA18256, 0xFF736848, 0xFF69715C, 0xFF5A6C60, 0xFF3B5C49,
        0xFF446958, 0xFF40695B, 0xFF376155, 0xFF396359, 0xFF345C51, 0xFF3A6053, 0xFF4E7264, 0xFF5D8376, 0xFF567A6E, 0xFF416058, 0xFF28433C, 0xFF748886, 0xFFADBDBC, 0xFF95A0A2, 0xFF818B8D, 0xFF7C8083, 0xFF6B6C70, 0xFF76777B, 0xFFA9AAAE, 0xFFCED3D6,
        0xFFC6D0D1, 0xFF829291, 0xFF344846, 0xFF253C36, 0xFF364F49, 0xFF3E5950, 0xFF57776C, 0xFF618277, 0xFF5D8374, 0xFF588171, 0xFF598473, 0xFF5A8574, 0xFF527E6D, 0xFF517A6A, 0xFF314D41, 0xFF48554E, 0xFF404B45, 0xFF334A40, 0xFF3F6355, 0xFF507466,
        0xFF344B41, 0xFF545F59, 0xFF48554E, 0xFF233D32, 0xFF466C5D, 0xFF557E6E, 0xFF527668, 0xFF87825C, 0xFFAD9A6F, 0xFFAF9469, 0xFFAD946C, 0xFFB09165, 0xFFB99D6B, 0xFF96936A, 0xFF57745E, 0xFF577E6C, 0xFF5A816F, 0xFF5A816F, 0xFF537A68, 0xFF3A6351,
        0xFF68917F, 0xFF466F5D, 0xFF3E6755, 0xFF5A8073, 0xFF5D8376, 0xFF62867A, 0xFF5E7F74, 0xFF5E7F74, 0xFF608075, 0xFF5D7B71, 0xFF68867C, 0xFF57796B, 0xFF58786D, 0xFF537167, 0xFF4D675E, 0xFF3C534D, 0xFF465955, 0xFF425251, 0xFF374545, 0xFF3C5552,
        0xFF37504D, 0xFF334947, 0xFF304644, 0xFF354948, 0xFF3B4F4E, 0xFF3D5150, 0xFF3B4D4D, 0xFF3F4A4C, 0xFF414C4E, 0xFF3F4D4E, 0xFF3C4C4C, 0xFF415151, 0xFF364848, 0xFF2B3F3E, 0xFF3D5150, 0xFF434D45, 0xFF848983, 0xFFB4B3AE, 0xFFB9B4B0, 0xFFC3BCB6,
        0xFFC4BDB7, 0xFFBDB9B0, 0xFFC1BEB5, 0xFFC5BBB2, 0xFFBEB7AD, 0xFFBAB6AB, 0xFFC0B6AD, 0xFFC2AFA9, 0xFFBEAFA8, 0xFFA3A598, 0xFF6E7F6D, 0xFF4F7866, 0xFF527B69, 0xFF527967, 0xFF57796B, 0xFF57776A, 0xFF2F4D43, 0xFF0E2920, 0xFF264037, 0xFF274641,
        0xFF26403D, 0xFF2D413F, 0xFF374643, 0xFF717C78, 0xFF8D9693, 0xFF565F5A, 0xFF121D15, 0xFF122523, 0xFF2F4342, 0xFF354A4B, 0xFF31494B, 0xFF2E4A4B, 0xFF25443F, 0xFF2C4D44, 0xFF4A6E60, 0xFF447361, 0xFF477362, 0xFF45705F, 0xFF416557, 0xFF476156,
        0xFF8D9E96, 0xFFF9FFFD, 0xFFA1A3A0, 0xFF32614F, 0xFF3D6C5A, 0xFF446A5B, 0xFF63786F, 0xFFF6FCF8, 0xFFD5DBD7, 0xFF4B6057, 0xFF3F6355, 0xFF3F725F, 0xFF486A5C, 0xFFA6B5AE, 0xFFF8FFFD, 0xFF6F847B, 0xFF375B4D, 0xFF295443, 0xFF335E4D, 0xFF6F716E,
        0xFFFBFFFD, 0xFFBFD0C8, 0xFF3E6052, 0xFF42715F, 0xFF497564, 0xFF526E62, 0xFFCDDAD3, 0xFFFFFFFB, 0xFF53685F, 0xFF2F594B, 0xFF2A5D4C, 0xFF1C463A, 0xFF2E4944, 0xFF374B4A, 0xFF364A49, 0xFF2A4139, 0xFF41544E, 0xFF707D76, 0xFF393F3B, 0xFF11130E,
        0xFF6A6C67, 0xFFFEFFFA, 0xFF8B8E87, 0xFF848285, 0xFFA8A6A9, 0xFF9FA0A2, 0xFF666F6C, 0xFF455650, 0xFF58766A, 0xFF567F6F, 0xFF548371, 0xFF597D6F, 0xFF587E6F, 0xFF5A8373, 0xFF5B8172, 0xFF4E6C60, 0x00D5E6DE, 0x00FDFFFC, 0x00FFFDFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xBEE9C092, 0xFFE09F5B, 0xFFF4A354, 0xFFF0A65D, 0xFFEFA964, 0xFFF5A75D, 0xFFF9A75D, 0xFFE89D59, 0xFFEBA05C, 0xFFEDA25E, 0xFFEEA35F, 0xFFEDA25E, 0xFFECA15D,
        0xFFEBA05C, 0xFFEBA05C, 0xFFECA15D, 0xFFEDA45F, 0xFFE89D59, 0xFFE89D5A, 0xFFE79E5A, 0xFFEAA361, 0xFFE1A05E, 0xFFBD843F, 0xFFA36E2A, 0xFFA17337, 0xFF9B6B2B, 0xFF9D6C2A, 0xFF986F31, 0xFF907947, 0xFF71714F, 0xFF627A6A, 0xFF59807D, 0xFF628871,
        0xFF5E8874, 0xFF638E7D, 0xFF628E81, 0xFF628E81, 0xFF5A8476, 0xFF5F8576, 0xFF648978, 0xFF527C6E, 0xFF648E80, 0xFF61877A, 0xFF55766D, 0xFF436159, 0xFF425B57, 0xFF889C9A, 0xFFA4B7B5, 0xFF94989B, 0xFF9CA0A3, 0xFFA0A4A7, 0xFFA7ACAF, 0xFF859191,
        0xFF4D605E, 0xFF27423D, 0xFF23423C, 0xFF415F57, 0xFF3C5A50, 0xFF345449, 0xFF426358, 0xFF3B6152, 0xFF335C4C, 0xFF3D6855, 0xFF35604D, 0xFF35604F, 0xFF2E5A49, 0xFF355E4E, 0xFF385448, 0xFF5C6962, 0xFF515C56, 0xFF50675D, 0xFF406456, 0xFF567A6C,
        0xFF4B6258, 0xFF939E98, 0xFF8F9C95, 0xFF496358, 0xFF4B7162, 0xFF5E8777, 0xFF4C7062, 0xFF847C58, 0xFFAE996E, 0xFFB3966E, 0xFFB19570, 0xFFB6956C, 0xFFC1A371, 0xFF8F8A64, 0xFF435E4B, 0xFF517A68, 0xFF5A8371, 0xFF628B79, 0xFF5D8674, 0xFF28513F,
        0xFF507967, 0xFF335C4A, 0xFF3E6956, 0xFF527668, 0xFF4F7566, 0xFF53796A, 0xFF4E7465, 0xFF567F6F, 0xFF5F8878, 0xFF537E6D, 0xFF568170, 0xFF547D6D, 0xFF587E6F, 0xFF54756A, 0xFF48665C, 0xFF365047, 0xFF3F544F, 0xFF3C4F4B, 0xFF435350, 0xFF33504C,
        0xFF2E4845, 0xFF2E4845, 0xFF314B48, 0xFF2C4542, 0xFF28413E, 0xFF344A48, 0xFF3F5553, 0xFF405050, 0xFF435353, 0xFF415353, 0xFF3D5150, 0xFF415755, 0xFF394F4D, 0xFF2A4340, 0xFF354F4C, 0xFF3F4A42, 0xFF7B827B, 0xFFBFC0BA, 0xFFC4C1BC, 0xFFBEB7B1,
        0xFFBDB6B0, 0xFFBCB5AD, 0xFFBCB8AF, 0xFFB9AFA6, 0xFFB9B5AA, 0xFFB8B4A9, 0xFFB9B0A7, 0xFFBAABA4, 0xFFBAB0A7, 0xFF9DA395, 0xFF647966, 0xFF4F7866, 0xFF527B69, 0xFF527967, 0xFF57796B, 0xFF57776A, 0xFF2F4D43, 0xFF0E2920, 0xFF264037, 0xFF284541,
        0xFF294340, 0xFF3A4E4C, 0xFF1E2D2A, 0xFF293430, 0xFF78817E, 0xFF444F49, 0xFF111E15, 0xFF122523, 0xFF334545, 0xFF35494A, 0xFF324A4C, 0xFF304A4B, 0xFF284541, 0xFF2E4F46, 0xFF4C6E60, 0xFF446F5E, 0xFF4A7363, 0xFF466A5C, 0xFF3A5A4D, 0xFF314B40,
        0xFF64786F, 0xFFB4C3BC, 0xFF69766F, 0xFF487767, 0xFF376757, 0xFF325C4E, 0xFF3E5950, 0xFF98A7A2, 0xFF98A7A2, 0xFF4F6D63, 0xFF436F60, 0xFF265545, 0xFF36564B, 0xFF738680, 0xFFBACDC7, 0xFF5A786E, 0xFF345D4F, 0xFF1E4739, 0xFF2D5145, 0xFF4F5E59,
        0xFFACBFB9, 0xFF7C978E, 0xFF2D5346, 0xFF3A6657, 0xFF416A5C, 0xFF415C53, 0xFF8DA09A, 0xFFBFC1BC, 0xFF3D5249, 0xFF315D4E, 0xFF2E6150, 0xFF204A3E, 0xFF2F4C47, 0xFF394D4C, 0xFF394D4C, 0xFF354844, 0xFF6E7D78, 0xFFA3AEAA, 0xFF909493, 0xFF898B88,
        0xFF868B87, 0xFFA5ACA5, 0xFF565D56, 0xFF626365, 0xFF98999B, 0xFFA9ADAE, 0xFF77827E, 0xFF485B55, 0xFF537165, 0xFF517A6A, 0xFF558170, 0xFF5A7E70, 0xFF597F70, 0xFF5A8373, 0xFF5C8273, 0xFF4E6C60, 0x00D5E6DE, 0x00FEFFFD, 0x00FFFDFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xBEE8BF91, 0xFFE09F5B, 0xFFF4A354, 0xFFF0A65D, 0xFFF0AA65, 0xFFF5A75D, 0xFFF9A75D, 0xFFE89D59, 0xFFECA15D, 0xFFEDA25E, 0xFFEEA35F, 0xFFEEA35F, 0xFFECA15D,
        0xFFEBA05C, 0xFFEBA05C, 0xFFECA15D, 0xFFEFA75F, 0xFFEA9F5B, 0xFFE99E5B, 0xFFE59B5A, 0xFFE9A262, 0xFFE2A15D, 0xFFBD843D, 0xFF9E6A21, 0xFFA8721C, 0xFFA87117, 0xFFAE771D, 0xFFA57722, 0xFF99803D, 0xFF6F7549, 0xFF527561, 0xFF487F79, 0xFF5D856D,
        0xFF658F79, 0xFF5E8B77, 0xFF518070, 0xFF638F80, 0xFF5D8678, 0xFF597E6D, 0xFF618372, 0xFF5B8A7A, 0xFF548071, 0xFF598375, 0xFF588173, 0xFF648A7F, 0xFF5B7C73, 0xFF506F69, 0xFF4D6C66, 0xFF21403A, 0xFF213E39, 0xFF17322D, 0xFF27403C, 0xFF2E4944,
        0xFF32514B, 0xFF33564F, 0xFF355D54, 0xFF36574C, 0xFF507166, 0xFF597D71, 0xFF426658, 0xFF4B7162, 0xFF4D7664, 0xFF446F5C, 0xFF45705D, 0xFF467160, 0xFF436F5E, 0xFF3E6757, 0xFF627E72, 0xFF8B9891, 0xFF707B75, 0xFF7E958B, 0xFF3F6355, 0xFF517567,
        0xFF52695F, 0xFF69746E, 0xFF5C6962, 0xFF405A4F, 0xFF2E5445, 0xFF436C5C, 0xFF3C6052, 0xFF7A7150, 0xFFA28A64, 0xFFAB8B65, 0xFFAA8C6A, 0xFFAC8A64, 0xFFB29164, 0xFF7C7652, 0xFF344F40, 0xFF45725D, 0xFF426D59, 0xFF446B58, 0xFF426352, 0xFF244031,
        0xFF445A4D, 0xFF3F5145, 0xFF3D4D42, 0xFF41594C, 0xFF3B5548, 0xFF385446, 0xFF29493A, 0xFF3B604F, 0xFF5A8371, 0xFF5B8673, 0xFF5E8B77, 0xFF578372, 0xFF517C6B, 0xFF507667, 0xFF4B6C61, 0xFF3A554C, 0xFF3F5650, 0xFF384B47, 0xFF394C48, 0xFF284644,
        0xFF264442, 0xFF2F4C4A, 0xFF324F4D, 0xFF274140, 0xFF1E3837, 0xFF2B4544, 0xFF3B5554, 0xFF394D4C, 0xFF394F4D, 0xFF3A504E, 0xFF354E4B, 0xFF36504D, 0xFF2E4B47, 0xFF22413C, 0xFF284742, 0xFF3F4C43, 0xFF69736B, 0xFFBBBEB7, 0xFFC5C2BD, 0xFFB2ABA5,
        0xFFAFA8A2, 0xFFB6AFA7, 0xFFB6B2A9, 0xFFB3AAA1, 0xFFBAB8AC, 0xFFBBBBAF, 0xFFC0B9AF, 0xFFC3B6AE, 0xFFC1BAB0, 0xFFA1AC9C, 0xFF66826C, 0xFF517A68, 0xFF537C6A, 0xFF517866, 0xFF557769, 0xFF557568, 0xFF2E4C42, 0xFF0F2A21, 0xFF284239, 0xFF2A4743,
        0xFF1D3633, 0xFF2F4240, 0xFF232F2D, 0xFF141F1B, 0xFF606B67, 0xFF4E5B54, 0xFF142118, 0xFF1F2F2E, 0xFF394B4B, 0xFF324647, 0xFF314649, 0xFF2B4546, 0xFF274440, 0xFF315048, 0xFF4E7062, 0xFF4C7062, 0xFF4E7062, 0xFF3C5C4F, 0xFF385448, 0xFF2F493E,
        0xFF3D544A, 0xFF637A70, 0xFF4E655B, 0xFF47655B, 0xFF2C5044, 0xFF365A4E, 0xFF3D574E, 0xFF4D5C57, 0xFF606F6A, 0xFF4E6960, 0xFF315B4D, 0xFF455F56, 0xFF4A5B55, 0xFF56625E, 0xFF6F807A, 0xFF4F6A61, 0xFF3E5E53, 0xFF365047, 0xFF3D4E48, 0xFF41544E,
        0xFF5E756D, 0xFF415C53, 0xFF28463C, 0xFF314C43, 0xFF3C534B, 0xFF3E4F49, 0xFF5D6965, 0xFF797B76, 0xFF30453C, 0xFF3A6657, 0xFF2E6351, 0xFF204C3F, 0xFF2F4C47, 0xFF394F4D, 0xFF3B4F4E, 0xFF465454, 0xFF4C5657, 0xFF5F6867, 0xFFADB3B3, 0xFFD8DEDC,
        0xFF606966, 0xFF424E4A, 0xFF778681, 0xFF4C5554, 0xFF4F5857, 0xFF525C5B, 0xFF45544F, 0xFF41564F, 0xFF5D7B6F, 0xFF577D6E, 0xFF54806F, 0xFF5B7F71, 0xFF5A8071, 0xFF5B8474, 0xFF5D8374, 0xFF4F6D61, 0x00D6E7DF, 0x00FEFFFD, 0x00FFFDFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xBEE8BF91, 0xFFE09F5B, 0xFFF4A354, 0xFFF0A65D, 0xFFF0AA65, 0xFFF6A85E, 0xFFFAA85E, 0xFFE89D59, 0xFFECA15D, 0xFFEDA25E, 0xFFEEA35F, 0xFFEEA35F, 0xFFEDA25E,
        0xFFEBA05C, 0xFFECA15D, 0xFFECA15D, 0xFFE9A159, 0xFFEBA05C, 0xFFEEA360, 0xFFE69C5D, 0xFFE59E5E, 0xFFE09F5B, 0xFFBF873D, 0xFFA47024, 0xFFA0763A, 0xFFA67533, 0xFFB37930, 0xFFAC7228, 0xFFA87838, 0xFF846C3C, 0xFF696C4F, 0xFF67796B, 0xFF628A70,
        0xFF557F69, 0xFF608D79, 0xFF5F8B7A, 0xFF578374, 0xFF567F6F, 0xFF638877, 0xFF597A69, 0xFF51806E, 0xFF558474, 0xFF568273, 0xFF568273, 0xFF5A8478, 0xFF577F74, 0xFF668E85, 0xFF567E75, 0xFF4A8374, 0xFF3F7567, 0xFF39695D, 0xFF4A756B, 0xFF3F695F,
        0xFF477167, 0xFF527D73, 0xFF4B766C, 0xFF496D61, 0xFF466A5E, 0xFF426658, 0xFF3E6455, 0xFF466F5F, 0xFF325D4A, 0xFF3C6754, 0xFF3C6754, 0xFF35604F, 0xFF416D5C, 0xFF2E5747, 0xFF526E62, 0xFF64716A, 0xFF414C46, 0xFF879E94, 0xFF517567, 0xFF527668,
        0xFF698076, 0xFF414C46, 0xFF38453E, 0xFF718B80, 0xFF4A7061, 0xFF3C6555, 0xFF3C6052, 0xFF7B7052, 0xFF947C58, 0xFF987754, 0xFF98795A, 0xFF987450, 0xFF9D7C51, 0xFF7A7150, 0xFF4A6255, 0xFF40705A, 0xFF416C58, 0xFF466857, 0xFF425A4C, 0xFF465349,
        0xFF676862, 0xFF88837F, 0xFF7B706E, 0xFF768078, 0xFF768179, 0xFF6B7C72, 0xFF4A6054, 0xFF4B6A5B, 0xFF688A7A, 0xFF5D8674, 0xFF57826F, 0xFF588773, 0xFF54806F, 0xFF5D8676, 0xFF587C6E, 0xFF3A584E, 0xFF39534A, 0xFF354A45, 0xFF394C48, 0xFF2C4C49,
        0xFF284845, 0xFF2C4A48, 0xFF2D4B49, 0xFF2A4745, 0xFF2A4745, 0xFF304A49, 0xFF2D4746, 0xFF344D4A, 0xFF334C49, 0xFF334D4A, 0xFF2A4743, 0xFF274641, 0xFF254540, 0xFF21413C, 0xFF294C46, 0xFF415147, 0xFF58635B, 0xFFA9AEA7, 0xFFBAB7B2, 0xFFA39C96,
        0xFF9D9690, 0xFFA29B93, 0xFF9D968E, 0xFFA19A90, 0xFFA4A296, 0xFFA5A79A, 0xFFAFABA0, 0xFFB4ABA2, 0xFFB2AEA3, 0xFF96A493, 0xFF64816B, 0xFF537C6A, 0xFF547D6B, 0xFF507765, 0xFF537567, 0xFF537366, 0xFF2D4B41, 0xFF102B22, 0xFF2A443B, 0xFF334D4A,
        0xFF314745, 0xFF2F4240, 0xFF5A6664, 0xFF6A7571, 0xFF6D7874, 0xFF5A6760, 0xFF18281E, 0xFF293938, 0xFF3D4F4F, 0xFF304445, 0xFF2E4346, 0xFF264041, 0xFF25423E, 0xFF33524A, 0xFF507264, 0xFF58786B, 0xFF516F63, 0xFF375347, 0xFF405A4F, 0xFF435A50,
        0xFF2F493E, 0xFF345044, 0xFF4A685C, 0xFF43524D, 0xFF50655E, 0xFF567067, 0xFF485B55, 0xFF3D4844, 0xFF4B5652, 0xFF4D625B, 0xFF507166, 0xFF606966, 0xFF5C605F, 0xFF595B5A, 0xFF46514D, 0xFF475C55, 0xFF435A52, 0xFF5E6965, 0xFF606060, 0xFF475A54,
        0xFF40554E, 0xFF344942, 0xFF465751, 0xFF515D59, 0xFF5B615F, 0xFF515755, 0xFF4F5352, 0xFF5C5E59, 0xFF334A40, 0xFF426E5F, 0xFF2C614F, 0xFF1E4A3D, 0xFF2F4C47, 0xFF3A504E, 0xFF3C5250, 0xFF424C4E, 0xFF495154, 0xFF494E51, 0xFF676C6F, 0xFF697173,
        0xFF212D2B, 0xFF2C3C39, 0xFF4E635E, 0xFF374643, 0xFF394845, 0xFF42514E, 0xFF3C4D47, 0xFF3B524A, 0xFF5F7D71, 0xFF5B8172, 0xFF568170, 0xFF5B7F71, 0xFF5A8071, 0xFF5C8575, 0xFF5D8374, 0xFF4F6D61, 0x00D6E7DF, 0x00FEFFFD, 0x00FFFDFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xCEF2BD89, 0xFFDA9F65, 0xFFEAA761, 0xFFF4AB5E, 0xFFF4A757, 0xFFF2A658, 0xFFF4AA63, 0xFFE29C57, 0xFFEA9F5C, 0xFFF0A562, 0xFFEDA25F, 0xFFEBA05D, 0xFFECA15E,
        0xFFE99E5B, 0xFFEA9F5C, 0xFFEEA360, 0xFFF1A05F, 0xFFEC9D5B, 0xFFEEA35F, 0xFFE9A35D, 0xFFE5A55E, 0xFFDDA25E, 0xFFBC8646, 0xFFA57233, 0xFFA57C2D, 0xFFAF7E32, 0xFFB47933, 0xFFB67C3D, 0xFFA6814C, 0xFF7D7A4D, 0xFF597B58, 0xFF589171, 0xFF5C8974,
        0xFF588872, 0xFF5E8D79, 0xFF5E8F7A, 0xFF568A74, 0xFF538874, 0xFF558A76, 0xFF518672, 0xFF5A8574, 0xFF598473, 0xFF588372, 0xFF578271, 0xFF578271, 0xFF598473, 0xFF5A8574, 0xFF5B8675, 0xFF5C8776, 0xFF598473, 0xFF5C8776, 0xFF5D8877, 0xFF588372,
        0xFF5A8574, 0xFF5F8A79, 0xFF5D8877, 0xFF668C7D, 0xFF5F8576, 0xFF63897A, 0xFF63897A, 0xFF5C8273, 0xFF5E8475, 0xFF608677, 0xFF5B8172, 0xFF63897A, 0xFF5D8374, 0xFF53796A, 0xFF4A7061, 0xFF466C5D, 0xFF466C5D, 0xFF4A7061, 0xFF4D7364, 0xFF547D6D,
        0xFF396252, 0xFF345D4D, 0xFF436C5C, 0xFF557E6E, 0xFF456E5E, 0xFF3D6656, 0xFF3A6353, 0xFF5C6D5D, 0xFF576858, 0xFF475848, 0xFF728373, 0xFF576858, 0xFF455646, 0xFF869787, 0xFF627363, 0xFF3E6B57, 0xFF3F6A57, 0xFF4A6F5E, 0xFF466556, 0xFF546A5E,
        0xFF7B8B81, 0xFF828C84, 0xFF6F766F, 0xFF7F8581, 0xFF8A938E, 0xFF56655E, 0xFF778C83, 0xFF658377, 0xFF5E8274, 0xFF5D8676, 0xFF568271, 0xFF5E8777, 0xFF5F8878, 0xFF5E8777, 0xFF5A8373, 0xFF547D6D, 0xFF4F7868, 0xFF4C7565, 0xFF4B7464, 0xFF4D7462,
        0xFF4C7361, 0xFF4B7260, 0xFF4A715F, 0xFF49705E, 0xFF49705E, 0xFF49705E, 0xFF49705E, 0xFF4D7261, 0xFF4D7261, 0xFF4D7261, 0xFF4C7160, 0xFF4C7160, 0xFF4B705F, 0xFF4B705F, 0xFF4B705F, 0xFF536F60, 0xFF597566, 0xFF688475, 0xFF718D7E, 0xFF6E8A7B,
        0xFF6C8879, 0xFF6D897A, 0xFF688475, 0xFF638877, 0xFF628776, 0xFF618675, 0xFF618675, 0xFF628776, 0xFF618675, 0xFF5E8372, 0xFF5C8170, 0xFF56816E, 0xFF57826F, 0xFF547B69, 0xFF587A6A, 0xFF5A796A, 0xFF344E41, 0xFF142C1F, 0xFF2D4337, 0xFF2B4B40,
        0xFF284239, 0xFF6B7A75, 0xFFA5AEAB, 0xFF868F8C, 0xFF54635E, 0xFF3D574E, 0xFF2A4A3F, 0xFF2F4845, 0xFF2A4340, 0xFF27413E, 0xFF2B4542, 0xFF274440, 0xFF24413D, 0xFF36534F, 0xFF51706B, 0xFF59806E, 0xFF517061, 0xFF364A3F, 0xFF5B665E, 0xFF49544C,
        0xFF2F4036, 0xFF2E483B, 0xFF517162, 0xFF4A5F56, 0xFF7A8F86, 0xFF50655C, 0xFF3F544B, 0xFF293E35, 0xFF4B6057, 0xFF5B7067, 0xFFA8BDB4, 0xFFBBBDB8, 0xFF677169, 0xFF30463A, 0xFF365246, 0xFF48665A, 0xFF7D948C, 0xFF8F9E99, 0xFF4B514F, 0xFF3F594E,
        0xFF203C30, 0xFF51685E, 0xFF77847D, 0xFFB3B3B1, 0xFF676C68, 0xFF4C6057, 0xFF1F4335, 0xFF425E50, 0xFF385A4C, 0xFF406A5C, 0xFF2E5E52, 0xFF204E44, 0xFF2E504F, 0xFF31494D, 0xFF38464F, 0xFF2D4A45, 0xFF33504B, 0xFF37544F, 0xFF324F4A, 0xFF284540,
        0xFF213E39, 0xFF223F3A, 0xFF26433E, 0xFF3E494F, 0xFF404D53, 0xFF3D4C4F, 0xFF334745, 0xFF445E55, 0xFF5D7B6F, 0xFF5F8171, 0xFF5C816F, 0xFF5C8072, 0xFF5C8273, 0xFF5B8474, 0xFF628879, 0xFF537165, 0x00D4E5DD, 0x00FEFFFD, 0x00FFFDFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xD9EEB484, 0xFFDE9F69, 0xFFE8A465, 0xFFECA25B, 0xFFF9AD62, 0xFFF7AA62, 0xFFF3AA66, 0xFFE59E5E, 0xFFEFA55E, 0xFFF2A861, 0xFFF2A861, 0xFFF2A861, 0xFFF3A962,
        0xFFEFA55E, 0xFFEDA35C, 0xFFEBA15A, 0xFFECA764, 0xFFEAA560, 0xFFEBA861, 0xFFE1A159, 0xFFDFA55C, 0xFFD9A45E, 0xFFBA8947, 0xFF9E7130, 0xFF827E4F, 0xFF8A8253, 0xFF8C8050, 0xFF8A7F51, 0xFF88865D, 0xFF748163, 0xFF627E68, 0xFF5C8072, 0xFF5B8571,
        0xFF57826E, 0xFF5A8572, 0xFF5D8875, 0xFF5B8673, 0xFF5E8978, 0xFF618C7B, 0xFF5D8877, 0xFF5A8574, 0xFF598473, 0xFF598473, 0xFF588372, 0xFF598473, 0xFF598473, 0xFF5B8675, 0xFF5B8675, 0xFF5B8675, 0xFF598473, 0xFF5D8877, 0xFF5E8978, 0xFF5A8574,
        0xFF5C8776, 0xFF608B7A, 0xFF5E8978, 0xFF5B8172, 0xFF5B8172, 0xFF5D8374, 0xFF5C8273, 0xFF587E6F, 0xFF587E6F, 0xFF5A8071, 0xFF5C8273, 0xFF5C8273, 0xFF5B8172, 0xFF5A8071, 0xFF597F70, 0xFF597F70, 0xFF597F70, 0xFF597F70, 0xFF597F70, 0xFF5E8475,
        0xFF5A8071, 0xFF5A8071, 0xFF53796A, 0xFF577D6E, 0xFF4F7566, 0xFF4C7263, 0xFF426859, 0xFF798E7F, 0xFF324738, 0xFF172C1D, 0xFF2F4435, 0xFF081D0E, 0xFF364B3C, 0xFF97AC9D, 0xFF526758, 0xFF45705D, 0xFF426B59, 0xFF466B5A, 0xFF385849, 0xFF385245,
        0xFF52685C, 0xFF596A60, 0xFF4B5B51, 0xFF4F5E57, 0xFF687971, 0xFF52675E, 0xFF5A7469, 0xFF4D6D60, 0xFF5A7E70, 0xFF5A8373, 0xFF588372, 0xFF567F6F, 0xFF588171, 0xFF5A8373, 0xFF5B8474, 0xFF5A8373, 0xFF598272, 0xFF598272, 0xFF5A8373, 0xFF5D8472,
        0xFF5C8371, 0xFF5B8270, 0xFF5A816F, 0xFF59806E, 0xFF59806E, 0xFF59806E, 0xFF59806E, 0xFF5D8271, 0xFF5D8271, 0xFF5D8271, 0xFF5D8271, 0xFF5D8271, 0xFF5D8271, 0xFF5D8271, 0xFF5D8271, 0xFF5C7D6C, 0xFF5D7E6D, 0xFF638473, 0xFF678877, 0xFF638473,
        0xFF638473, 0xFF668776, 0xFF638473, 0xFF5F8473, 0xFF5E8372, 0xFF5D8271, 0xFF5D8271, 0xFF5D8271, 0xFF5D8271, 0xFF5B806F, 0xFF597E6D, 0xFF527D6A, 0xFF56816E, 0xFF567D6B, 0xFF5A7C6C, 0xFF5D7C6D, 0xFF345042, 0xFF142C1F, 0xFF2E4639, 0xFF2B4947,
        0xFF263E3E, 0xFF455457, 0xFF636C71, 0xFF535C61, 0xFF3A494C, 0xFF304848, 0xFF294745, 0xFF314A47, 0xFF2D4744, 0xFF28423F, 0xFF294340, 0xFF284541, 0xFF25423E, 0xFF35544F, 0xFF54736E, 0xFF58816F, 0xFF5B7D6D, 0xFF4A6255, 0xFF7B8C82, 0xFF8A9A90,
        0xFF5D7367, 0xFF3E5D4E, 0xFF5D8271, 0xFF526C61, 0xFF627C71, 0xFF4E685D, 0xFF516B60, 0xFF405A4F, 0xFF607A6F, 0xFF6A8479, 0xFF88A297, 0xFFCAD1CA, 0xFF92A298, 0xFF4A6255, 0xFF537366, 0xFF59796C, 0xFF617C73, 0xFF92A59F, 0xFF7E8A86, 0xFF557367,
        0xFF355548, 0xFF607C70, 0xFF8C9D95, 0xFFE9EFEB, 0xFFA3ACA7, 0xFF5E756B, 0xFF305647, 0xFF4C6B5C, 0xFF3B5D4F, 0xFF416B5D, 0xFF315F52, 0xFF1D483F, 0xFF294C48, 0xFF344C4E, 0xFF384950, 0xFF394D4C, 0xFF3C504F, 0xFF3E5251, 0xFF3C504F, 0xFF374B4A,
        0xFF334746, 0xFF334746, 0xFF334746, 0xFF3D4C51, 0xFF3A4C4E, 0xFF354948, 0xFF304945, 0xFF426056, 0xFF597B6D, 0xFF577C6B, 0xFF547B68, 0xFF5D8173, 0xFF5C8273, 0xFF598272, 0xFF608677, 0xFF537165, 0x00D4E5DD, 0x00FEFFFD, 0x00FFFCFC, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x98EDC4A6, 0xAFE8BC99, 0xFFE0B185, 0xFFC19160, 0xFFD6A46F, 0xFFD3A470, 0xFFCD9F6E, 0xFFC89C6F, 0xFFCB9D62, 0xFFC99B60, 0xFFCD9F64, 0xFFCD9F64, 0xFFD2A469,
        0xFFD2A469, 0xFFD3A56A, 0xFFD2A469, 0xFFDBA173, 0xFFDA9D6E, 0xFFDCA06E, 0xFFD79B67, 0xFFD79D6B, 0xFFCC9868, 0xFFB68759, 0xFFA57A4D, 0xFF678778, 0xFF678A76, 0xFF698B72, 0xFF63856A, 0xFF6D8D75, 0xFF6F8C7A, 0xFF6F8882, 0xFF697F7D, 0xFF618B77,
        0xFF5D8773, 0xFF5D8472, 0xFF5D8472, 0xFF5B806F, 0xFF5E8072, 0xFF5F8173, 0xFF5E7E71, 0xFF5A8574, 0xFF5A8574, 0xFF5A8574, 0xFF5A8574, 0xFF5A8574, 0xFF5B8675, 0xFF5B8675, 0xFF5B8675, 0xFF5A8574, 0xFF5A8574, 0xFF5D8877, 0xFF5F8A79, 0xFF5C8776,
        0xFF5D8877, 0xFF5F8A79, 0xFF5D8877, 0xFF5C8273, 0xFF5F8576, 0xFF5C8273, 0xFF5A8071, 0xFF5C8273, 0xFF5A8071, 0xFF5B8172, 0xFF5E8475, 0xFF597F70, 0xFF5A8071, 0xFF5D8374, 0xFF5E8475, 0xFF5E8475, 0xFF5D8374, 0xFF5A8071, 0xFF587E6F, 0xFF54786A,
        0xFF5E8274, 0xFF64887A, 0xFF597D6F, 0xFF5A7E70, 0xFF5D8173, 0xFF5F8375, 0xFF577B6D, 0xFF4C6659, 0xFF4D675A, 0xFF3C5649, 0xFF3D574A, 0xFF3C5649, 0xFF496356, 0xFF637D70, 0xFF4F695C, 0xFF5F8876, 0xFF5F8674, 0xFF608775, 0xFF517665, 0xFF486A5A,
        0xFF547465, 0xFF5A796A, 0xFF537263, 0xFF4B695D, 0xFF607E72, 0xFF628275, 0xFF537567, 0xFF496D5F, 0xFF5B8172, 0xFF587E6F, 0xFF567F6F, 0xFF578070, 0xFF588171, 0xFF5A8373, 0xFF5B8474, 0xFF5B8474, 0xFF5C8575, 0xFF5C8575, 0xFF5D8676, 0xFF5A8371,
        0xFF5A8371, 0xFF598270, 0xFF58816F, 0xFF57806E, 0xFF57806E, 0xFF58816F, 0xFF58816F, 0xFF5B8270, 0xFF5B8270, 0xFF5C8371, 0xFF5C8371, 0xFF5D8472, 0xFF5D8472, 0xFF5E8573, 0xFF5E8573, 0xFF5F8473, 0xFF5C8170, 0xFF5D8271, 0xFF5D8271, 0xFF5A7F6E,
        0xFF5C8170, 0xFF5E8372, 0xFF5D8271, 0xFF5E8573, 0xFF5D8472, 0xFF5B8270, 0xFF5B8270, 0xFF5C8371, 0xFF5C8371, 0xFF5C8371, 0xFF5B8270, 0xFF527D6A, 0xFF57826F, 0xFF577E6C, 0xFF597E6D, 0xFF5E7E6F, 0xFF325142, 0xFF0F2B1D, 0xFF2C4639, 0xFF304E50,
        0xFF2F484C, 0xFF34454C, 0xFF39464E, 0xFF3A474F, 0xFF35464D, 0xFF2E474B, 0xFF2C4A4C, 0xFF2B4641, 0xFF334E49, 0xFF2F4C47, 0xFF2A4742, 0xFF274640, 0xFF22413B, 0xFF35544E, 0xFF5B7C75, 0xFF54816D, 0xFF608775, 0xFF607F70, 0xFF6C8679, 0xFF667E71,
        0xFF587466, 0xFF628474, 0xFF6C9381, 0xFF67877A, 0xFF57776A, 0xFF48685B, 0xFF567669, 0xFF567669, 0xFF668679, 0xFF57776A, 0xFF466659, 0xFF7B8C82, 0xFF71877B, 0xFF587768, 0xFF567A6C, 0xFF628678, 0xFF5A7A6F, 0xFF5C766D, 0xFF738881, 0xFF4E7062,
        0xFF325648, 0xFF4B6F61, 0xFF3B554A, 0xFF74837C, 0xFF6E7D76, 0xFF567468, 0xFF345F4E, 0xFF416154, 0xFF2F5345, 0xFF436D5F, 0xFF477366, 0xFF2C574D, 0xFF2A4D47, 0xFF314B4A, 0xFF374B4C, 0xFF3A494C, 0xFF39484B, 0xFF39484B, 0xFF3B4A4D, 0xFF3C4B4E,
        0xFF3C4B4E, 0xFF3A494C, 0xFF38474A, 0xFF384D4E, 0xFF344D4A, 0xFF324D48, 0xFF314F47, 0xFF46675C, 0xFF5C8273, 0xFF5B8472, 0xFF5C8573, 0xFF5D8173, 0xFF5C8273, 0xFF588171, 0xFF5F8576, 0xFF527064, 0x00D4E5DD, 0x00FEFFFD, 0x00FFFBFB, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFF8, 0x04FFFEF3, 0xFFD0CDBC, 0xFF76765E, 0xFF85856B, 0xFF83876C, 0xFF7D826B, 0xFF818670, 0xFF84876A, 0xFF818467, 0xFF888B6E, 0xFF828568, 0xFF878A6D,
        0xFF85886B, 0xFF85886B, 0xFF85886B, 0xFF7F8E6D, 0xFF7C8765, 0xFF838965, 0xFF858964, 0xFF898D6A, 0xFF787F5D, 0xFF6F7D5C, 0xFF728363, 0xFF5C847B, 0xFF5B8174, 0xFF648978, 0xFF5F8170, 0xFF698A79, 0xFF638376, 0xFF6A8983, 0xFF64837E, 0xFF5C8C76,
        0xFF5D8A75, 0xFF5C8975, 0xFF5D8674, 0xFF5C8371, 0xFF5E8274, 0xFF628476, 0xFF628476, 0xFF5A8574, 0xFF5A8574, 0xFF5B8675, 0xFF5B8675, 0xFF5B8675, 0xFF5C8776, 0xFF5B8675, 0xFF5B8675, 0xFF5A8574, 0xFF5B8675, 0xFF5D8877, 0xFF5F8A79, 0xFF5D8877,
        0xFF5A8574, 0xFF588372, 0xFF55806F, 0xFF5F8576, 0xFF5E8475, 0xFF587E6F, 0xFF597F70, 0xFF5B8172, 0xFF5A8071, 0xFF5B8172, 0xFF587E6F, 0xFF5A816F, 0xFF59806E, 0xFF587F6D, 0xFF577E6C, 0xFF567D6B, 0xFF567D6B, 0xFF557C6A, 0xFF557C6A, 0xFF587A6A,
        0xFF597B6B, 0xFF5C7E6E, 0xFF5C7E6E, 0xFF5E8070, 0xFF5F8171, 0xFF5F8171, 0xFF5C7E6E, 0xFF597B6D, 0xFF5F8173, 0xFF57796B, 0xFF537567, 0xFF5E8072, 0xFF597B6D, 0xFF517365, 0xFF5C7E70, 0xFF5B8270, 0xFF5E8573, 0xFF658C7A, 0xFF628977, 0xFF5A816F,
        0xFF5A8371, 0xFF5D8674, 0xFF598270, 0xFF5B8675, 0xFF5D8676, 0xFF648D7D, 0xFF598272, 0xFF5B8474, 0xFF618A7A, 0xFF5E8475, 0xFF5F8576, 0xFF5A8373, 0xFF5A8373, 0xFF5A8373, 0xFF5A8373, 0xFF5A8373, 0xFF598272, 0xFF598272, 0xFF598272, 0xFF598270,
        0xFF58816F, 0xFF58816F, 0xFF57806E, 0xFF567F6D, 0xFF567F6D, 0xFF57806E, 0xFF57806E, 0xFF5A816F, 0xFF5A816F, 0xFF5A816F, 0xFF5B8270, 0xFF5B8270, 0xFF5B8270, 0xFF5B8270, 0xFF5C8371, 0xFF58816F, 0xFF567F6D, 0xFF567F6D, 0xFF57806E, 0xFF58816F,
        0xFF598270, 0xFF598270, 0xFF57806E, 0xFF5E8573, 0xFF5C8371, 0xFF5A816F, 0xFF5A816F, 0xFF5B8270, 0xFF5C8371, 0xFF5C8371, 0xFF5C8371, 0xFF547F6C, 0xFF588370, 0xFF557E6C, 0xFF5B8270, 0xFF658777, 0xFF3D5D4E, 0xFF183728, 0xFF2F4E3F, 0xFF2B4A45,
        0xFF2E4B47, 0xFF2F4543, 0xFF2D4140, 0xFF334746, 0xFF344A48, 0xFF2A4743, 0xFF274641, 0xFF234139, 0xFF2D4B43, 0xFF27463E, 0xFF26453D, 0xFF2F5047, 0xFF2D4E45, 0xFF385950, 0xFF57786F, 0xFF51806C, 0xFF5B8673, 0xFF648B79, 0xFF648475, 0xFF5D7D6E,
        0xFF628474, 0xFF678E7C, 0xFF46715E, 0xFF54786A, 0xFF45695B, 0xFF34584A, 0xFF375B4D, 0xFF4F7365, 0xFF54786A, 0xFF496D5F, 0xFF406456, 0xFF577164, 0xFF446354, 0xFF5E8372, 0xFF6A9383, 0xFF618A7A, 0xFF52786B, 0xFF45665B, 0xFF5E7C72, 0xFF3C6052,
        0xFF335E4D, 0xFF4E7A69, 0xFF406456, 0xFF6B8278, 0xFF677E74, 0xFF537567, 0xFF2D5C4A, 0xFF486C5E, 0xFF3C6253, 0xFF4F7A69, 0xFF598576, 0xFF436D61, 0xFF30544A, 0xFF2B4843, 0xFF344D49, 0xFF354A4B, 0xFF34494A, 0xFF354A4B, 0xFF374C4D, 0xFF3A4F50,
        0xFF3B5051, 0xFF3A4F50, 0xFF384D4E, 0xFF314E49, 0xFF32514B, 0xFF2F5047, 0xFF284C42, 0xFF3C6255, 0xFF557E6E, 0xFF588372, 0xFF598471, 0xFF5F8375, 0xFF5D8374, 0xFF588171, 0xFF5F8576, 0xFF527064, 0x00D4E5DD, 0x00FEFFFD, 0x00FFFCFC, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x0FEDFFFF, 0x08F1FFFF, 0xFFB8D2C9, 0xFF466659, 0xFF638877, 0xFF6A917F, 0xFF567F6F, 0xFF4A7564, 0xFF496F64, 0xFF496F64, 0xFF53796E, 0xFF496F64, 0xFF52786D,
        0xFF4D7368, 0xFF4A7065, 0xFF4E7469, 0xFF628A72, 0xFF5A7A63, 0xFF576F57, 0xFF586750, 0xFF616D55, 0xFF495843, 0xFF3C543E, 0xFF3A5640, 0xFF35442D, 0xFF38412C, 0xFF515240, 0xFF504F3D, 0xFF596050, 0xFF465D4B, 0xFF527B69, 0xFF558A76, 0xFF538770,
        0xFF568A73, 0xFF588974, 0xFF578672, 0xFF54836F, 0xFF558170, 0xFF578372, 0xFF5C8776, 0xFF5A8574, 0xFF5A8574, 0xFF5A8574, 0xFF5B8675, 0xFF5B8675, 0xFF5B8675, 0xFF5B8675, 0xFF5B8675, 0xFF5A8574, 0xFF5B8675, 0xFF5C8776, 0xFF5D8877, 0xFF5C8776,
        0xFF568170, 0xFF4C7766, 0xFF467160, 0xFF456B5C, 0xFF446A5B, 0xFF406657, 0xFF446A5B, 0xFF426859, 0xFF406657, 0xFF476D5E, 0xFF406657, 0xFF406755, 0xFF3F6654, 0xFF3D6452, 0xFF3D6452, 0xFF3D6452, 0xFF3F6654, 0xFF416856, 0xFF436A58, 0xFF466858,
        0xFF426454, 0xFF406252, 0xFF456757, 0xFF476959, 0xFF466858, 0xFF426454, 0xFF426454, 0xFF466C5D, 0xFF3A6051, 0xFF416758, 0xFF466C5D, 0xFF486E5F, 0xFF476D5E, 0xFF3A6051, 0xFF416758, 0xFF476C5B, 0xFF4B7260, 0xFF557C6A, 0xFF5C8573, 0xFF598471,
        0xFF56816E, 0xFF55826E, 0xFF517E6A, 0xFF578674, 0xFF538270, 0xFF568271, 0xFF588473, 0xFF5F8A79, 0xFF5C8575, 0xFF5F8878, 0xFF5B8474, 0xFF5A8373, 0xFF598272, 0xFF598272, 0xFF5A8373, 0xFF5A8373, 0xFF5B8474, 0xFF5B8474, 0xFF5B8474, 0xFF598270,
        0xFF598270, 0xFF58816F, 0xFF58816F, 0xFF58816F, 0xFF58816F, 0xFF58816F, 0xFF598270, 0xFF598270, 0xFF598270, 0xFF598270, 0xFF598270, 0xFF58816F, 0xFF58816F, 0xFF58816F, 0xFF58816F, 0xFF55806D, 0xFF547F6C, 0xFF537E6B, 0xFF547F6C, 0xFF57826F,
        0xFF588370, 0xFF57826F, 0xFF56816E, 0xFF58816F, 0xFF567F6D, 0xFF547D6B, 0xFF547D6B, 0xFF557E6C, 0xFF567F6D, 0xFF567F6D, 0xFF567F6D, 0xFF57826F, 0xFF588370, 0xFF547D6B, 0xFF59806E, 0xFF5E8573, 0xFF385D4C, 0xFF0F3121, 0xFF193B2B, 0xFF1A3B2A,
        0xFF1A3929, 0xFF1B3728, 0xFF1A3526, 0xFF1A3526, 0xFF1B3728, 0xFF1A3929, 0xFF173827, 0xFF19392E, 0xFF19392E, 0xFF113227, 0xFF234439, 0xFF4C6D62, 0xFF587C70, 0xFF567A6E, 0xFF608478, 0xFF578672, 0xFF5D8A76, 0xFF5E8976, 0xFF6A917F, 0xFF789F8D,
        0xFF618876, 0xFF48715F, 0xFF3B6653, 0xFF497262, 0xFF517A6A, 0xFF4D7666, 0xFF436C5C, 0xFF507969, 0xFF466F5F, 0xFF456E5E, 0xFF507969, 0xFF517162, 0xFF557767, 0xFF648B79, 0xFF658E7E, 0xFF416C5B, 0xFF466F61, 0xFF5A8073, 0xFF2D5346, 0xFF1E4435,
        0xFF467261, 0xFF467764, 0xFF2C5746, 0xFF58786B, 0xFF536F63, 0xFF466C5D, 0xFF437461, 0xFF3B6454, 0xFF436C5C, 0xFF517C6B, 0xFF598473, 0xFF578072, 0xFF476B5F, 0xFF2E4E43, 0xFF234137, 0xFF20433D, 0xFF22453F, 0xFF244741, 0xFF254842, 0xFF254842,
        0xFF254842, 0xFF264943, 0xFF274A44, 0xFF24483C, 0xFF274B3F, 0xFF23473B, 0xFF1D4336, 0xFF365F51, 0xFF598272, 0xFF5D8877, 0xFF5A8574, 0xFF608476, 0xFF5E8475, 0xFF5A8373, 0xFF608677, 0xFF537165, 0x00D4E5DD, 0x00FEFFFD, 0x00FFFDFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F7FFF9, 0x00F5FFFB, 0xFFC0D8CB, 0xFF466756, 0xFF6A9480, 0xFF68937F, 0xFF3A6752, 0xFF14412C, 0xFF1E4739, 0xFF1B4436, 0xFF1F483A, 0xFF0D3628, 0xFF214A3C,
        0xFF1E4739, 0xFF1A4335, 0xFF254E40, 0xFF707869, 0xFF6A6B5B, 0xFF736656, 0xFF907667, 0xFFC5A598, 0xFFC4A89A, 0xFFC0AA9D, 0xFFB3A396, 0xFFB4A689, 0xFFAF9E84, 0xFFB39A86, 0xFF846D5D, 0xFF716859, 0xFF4A5946, 0xFF547E66, 0xFF599478, 0xFF608B77,
        0xFF628D79, 0xFF5E8B77, 0xFF5B8874, 0xFF598874, 0xFF578674, 0xFF578674, 0xFF598A77, 0xFF598473, 0xFF598473, 0xFF598473, 0xFF598473, 0xFF5A8574, 0xFF5A8574, 0xFF5A8574, 0xFF5B8675, 0xFF5A8574, 0xFF5B8675, 0xFF5A8574, 0xFF5B8675, 0xFF5C8776,
        0xFF527D6C, 0xFF416C5B, 0xFF366150, 0xFF395F50, 0xFF3F6556, 0xFF3A6051, 0xFF3F6556, 0xFF3E6455, 0xFF385E4F, 0xFF416758, 0xFF3D6354, 0xFF3A614F, 0xFF3A614F, 0xFF3B6250, 0xFF3C6351, 0xFF3C6351, 0xFF3D6452, 0xFF3E6553, 0xFF3E6553, 0xFF385F4D,
        0xFF3D6452, 0xFF3C6351, 0xFF3D6452, 0xFF3A614F, 0xFF406755, 0xFF3E6553, 0xFF3D6452, 0xFF365F4D, 0xFF3F6856, 0xFF406957, 0xFF355E4C, 0xFF3C6553, 0xFF3B6452, 0xFF305947, 0xFF436C5A, 0xFF3A614F, 0xFF436A58, 0xFF4E7563, 0xFF5B8472, 0xFF5F8876,
        0xFF5C8573, 0xFF5D8674, 0xFF598471, 0xFF54806F, 0xFF598574, 0xFF558170, 0xFF588473, 0xFF598473, 0xFF568170, 0xFF5F8A79, 0xFF55806F, 0xFF5D8676, 0xFF5C8575, 0xFF5B8474, 0xFF5B8474, 0xFF5C8575, 0xFF5D8676, 0xFF5D8676, 0xFF5D8676, 0xFF57826F,
        0xFF56816E, 0xFF56816E, 0xFF55806D, 0xFF55806D, 0xFF56816E, 0xFF56816E, 0xFF57826F, 0xFF58816F, 0xFF57806E, 0xFF57806E, 0xFF567F6D, 0xFF567F6D, 0xFF557E6C, 0xFF557E6C, 0xFF557E6C, 0xFF598473, 0xFF588372, 0xFF537E6D, 0xFF517C6B, 0xFF55806F,
        0xFF568170, 0xFF55806F, 0xFF578271, 0xFF598270, 0xFF58816F, 0xFF567F6D, 0xFF567F6D, 0xFF57806E, 0xFF57806E, 0xFF57806E, 0xFF567F6D, 0xFF57826F, 0xFF56816E, 0xFF557E6C, 0xFF57806E, 0xFF598270, 0xFF3C6351, 0xFF153C2A, 0xFF143B29, 0xFF173921,
        0xFF14361E, 0xFF15371F, 0xFF173921, 0xFF12341C, 0xFF12341C, 0xFF163820, 0xFF163820, 0xFF15362B, 0xFF15362B, 0xFF123328, 0xFF25493D, 0xFF4F7367, 0xFF5F8578, 0xFF5A8073, 0xFF5B8174, 0xFF5B8874, 0xFF5B8874, 0xFF5D8875, 0xFF648F7C, 0xFF57806E,
        0xFF436C5A, 0xFF3E6755, 0xFF4C7361, 0xFF3F6858, 0xFF477060, 0xFF4D7666, 0xFF456E5E, 0xFF456E5E, 0xFF3D6656, 0xFF416A5A, 0xFF456E5E, 0xFF5C8170, 0xFF739887, 0xFF678E7C, 0xFF496F60, 0xFF456E5E, 0xFF50796B, 0xFF3A6456, 0xFF123C2E, 0xFF55796B,
        0xFF416D5C, 0xFF1C4F3C, 0xFF1F4E3C, 0xFF55796B, 0xFF59796C, 0xFF496F60, 0xFF376654, 0xFF406C5B, 0xFF568170, 0xFF5A8574, 0xFF567F6F, 0xFF628879, 0xFF5B806F, 0xFF345646, 0xFF123424, 0xFF103A2E, 0xFF133D31, 0xFF153F33, 0xFF153F33, 0xFF123C30,
        0xFF123C30, 0xFF143E32, 0xFF174135, 0xFF1A3E30, 0xFF1A3E30, 0xFF163C2D, 0xFF173D2E, 0xFF365C4D, 0xFF5B8174, 0xFF5C8577, 0xFF567F71, 0xFF608476, 0xFF5F8576, 0xFF5B8474, 0xFF608677, 0xFF516F63, 0x00D2E3DB, 0x00FEFFFD, 0x00FFFDFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEF8, 0x00FDFEF8, 0xFFC1CEC5, 0xFF435D50, 0xFF6D8F7F, 0xFF6A917F, 0xFF3F6955, 0xFF1A442E, 0xFF456A58, 0xFF3E6351, 0xFF375C4A, 0xFF1E4331, 0xFF395E4C,
        0xFF305543, 0xFF1E4331, 0xFF284D3B, 0xFF617E6A, 0xFF556855, 0xFF686957, 0xFFA69988, 0xFFF8E3D2, 0xFFF6E5D3, 0xFFF0E5D3, 0xFFE2DDCA, 0xFFFAE2D5, 0xFFFDE7DA, 0xFFE9D7CD, 0xFF8B8279, 0xFF6A6D62, 0xFF445648, 0xFF547562, 0xFF527C66, 0xFF546A5D,
        0xFF51695B, 0xFF466053, 0xFF426152, 0xFF486A5A, 0xFF476D5E, 0xFF416C5B, 0xFF416C5B, 0xFF598473, 0xFF588372, 0xFF588372, 0xFF578271, 0xFF578271, 0xFF588372, 0xFF598473, 0xFF5A8574, 0xFF598473, 0xFF5B8675, 0xFF588372, 0xFF5B8675, 0xFF5E8978,
        0xFF507B6A, 0xFF396453, 0xFF2B5645, 0xFF1A4031, 0xFF264C3D, 0xFF13392A, 0xFF163C2D, 0xFF254B3C, 0xFF1B4132, 0xFF173D2E, 0xFF153B2C, 0xFF183F2C, 0xFF19402D, 0xFF1B422F, 0xFF1B422F, 0xFF1B422F, 0xFF1A412E, 0xFF183F2C, 0xFF173E2B, 0xFF16412D,
        0xFF1C4733, 0xFF194430, 0xFF194430, 0xFF123D29, 0xFF1A4531, 0xFF16412D, 0xFF143F2B, 0xFF1C4630, 0xFF17412B, 0xFF19432D, 0xFF16402A, 0xFF123C26, 0xFF18422C, 0xFF18422C, 0xFF0F3923, 0xFF234C3A, 0xFF345D4B, 0xFF446D5B, 0xFF59806E, 0xFF5F8674,
        0xFF5C8170, 0xFF608574, 0xFF608272, 0xFF597F70, 0xFF5D8374, 0xFF588171, 0xFF5F8878, 0xFF578271, 0xFF5B8776, 0xFF608C7B, 0xFF5C8877, 0xFF5F8878, 0xFF5D8676, 0xFF5B8474, 0xFF5A8373, 0xFF5B8474, 0xFF5C8575, 0xFF5B8474, 0xFF5A8373, 0xFF598471,
        0xFF588370, 0xFF588370, 0xFF588370, 0xFF588370, 0xFF588370, 0xFF598471, 0xFF598471, 0xFF5A8371, 0xFF598270, 0xFF598270, 0xFF598270, 0xFF598270, 0xFF598270, 0xFF598270, 0xFF598270, 0xFF5A8373, 0xFF5B8474, 0xFF557E6E, 0xFF537C6C, 0xFF578070,
        0xFF578070, 0xFF578070, 0xFF5B8474, 0xFF5C8573, 0xFF5B8472, 0xFF5A8371, 0xFF5A8371, 0xFF5A8371, 0xFF5A8371, 0xFF58816F, 0xFF57806E, 0xFF55806D, 0xFF547F6C, 0xFF588370, 0xFF598471, 0xFF5C8774, 0xFF537E6B, 0xFF416C59, 0xFF416C59, 0xFF4C6D5A,
        0xFF4B6E5A, 0xFF4A705B, 0xFF4B715C, 0xFF4B715C, 0xFF496F5A, 0xFF496C58, 0xFF4B6C59, 0xFF466A5C, 0xFF4A6E60, 0xFF496D5F, 0xFF4C7263, 0xFF597F70, 0xFF5D8374, 0xFF578070, 0xFF598272, 0xFF5A8371, 0xFF598270, 0xFF588370, 0xFF5A8572, 0xFF3E6956,
        0xFF47705E, 0xFF476C5B, 0xFF3F6453, 0xFF4C7263, 0xFF4D7364, 0xFF496F60, 0xFF406657, 0xFF406657, 0xFF456B5C, 0xFF4A7061, 0xFF496F60, 0xFF577E6C, 0xFF4C7160, 0xFF436857, 0xFF3E6254, 0xFF517567, 0xFF3F6558, 0xFF1F493B, 0xFF477163, 0xFF56786A,
        0xFF1C4736, 0xFF1F523F, 0xFF467764, 0xFF4F7365, 0xFF406053, 0xFF3F6355, 0xFF426D5C, 0xFF51806E, 0xFF5D8978, 0xFF588372, 0xFF557C6A, 0xFF5F8473, 0xFF628775, 0xFF527765, 0xFF3F6452, 0xFF43695A, 0xFF446A5B, 0xFF446A5B, 0xFF446A5B, 0xFF43695A,
        0xFF43695A, 0xFF446A5B, 0xFF466C5D, 0xFF446958, 0xFF436857, 0xFF436857, 0xFF436759, 0xFF54786A, 0xFF64887C, 0xFF5F8377, 0xFF5E8276, 0xFF5E8274, 0xFF5E8475, 0xFF5B8474, 0xFF5F8576, 0xFF4E6C60, 0x00CFE0D8, 0x00FDFFFC, 0x00FFFDFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFBFF, 0x00FAF8FB, 0xFFC6D1D3, 0xFF4B6363, 0xFF6E918D, 0xFF628B83, 0xFF3D675D, 0xFF1E483C, 0xFF4F7664, 0xFF4B7260, 0xFF466D5B, 0xFF305745, 0xFF537A68,
        0xFF416856, 0xFF1D4432, 0xFF1E4533, 0xFF5C8179, 0xFF425C53, 0xFF585D56, 0xFFA99E98, 0xFFFCE9E3, 0xFFE3D2CB, 0xFFD1C8C1, 0xFFCCC9C2, 0xFFDAC6C8, 0xFFEEE2E4, 0xFFDADCDB, 0xFF697976, 0xFF49625C, 0xFF2F4940, 0xFF3F544B, 0xFF31453A, 0xFF5F625B,
        0xFF525951, 0xFF3F4A42, 0xFF37483E, 0xFF3D5548, 0xFF3C584C, 0xFF2F5143, 0xFF294D3F, 0xFF598473, 0xFF588372, 0xFF568170, 0xFF568170, 0xFF568170, 0xFF578271, 0xFF598473, 0xFF5A8574, 0xFF598473, 0xFF5A8574, 0xFF578271, 0xFF5A8574, 0xFF5F8A79,
        0xFF507B6A, 0xFF35604F, 0xFF25503F, 0xFF33594A, 0xFF3F6556, 0xFF1C4233, 0xFF1F4536, 0xFF406657, 0xFF33594A, 0xFF1E4435, 0xFF1A4031, 0xFF19402D, 0xFF1A412E, 0xFF1B422F, 0xFF1C4330, 0xFF1D4431, 0xFF1C4330, 0xFF1C4330, 0xFF1B422F, 0xFF184832,
        0xFF184832, 0xFF13432D, 0xFF1A4A34, 0xFF164630, 0xFF1B4B35, 0xFF15452F, 0xFF164630, 0xFF1B452D, 0xFF1E4830, 0xFF19432B, 0xFF1B452D, 0xFF1B452D, 0xFF164028, 0xFF19432B, 0xFF18422A, 0xFF194431, 0xFF315C49, 0xFF47705E, 0xFF5E8573, 0xFF658777,
        0xFF5F7F70, 0xFF638273, 0xFF628172, 0xFF658779, 0xFF5C7E70, 0xFF587C6E, 0xFF628879, 0xFF547F6E, 0xFF5A8675, 0xFF52816F, 0xFF568573, 0xFF598272, 0xFF578070, 0xFF567F6F, 0xFF567F6F, 0xFF598272, 0xFF5B8474, 0xFF5B8474, 0xFF5A8373, 0xFF56816E,
        0xFF56816E, 0xFF55806D, 0xFF55806D, 0xFF55806D, 0xFF56816E, 0xFF57826F, 0xFF57826F, 0xFF56816E, 0xFF56816E, 0xFF56816E, 0xFF57826F, 0xFF57826F, 0xFF57826F, 0xFF588370, 0xFF588370, 0xFF577D6E, 0xFF5B8172, 0xFF587E6F, 0xFF587E6F, 0xFF5D8374,
        0xFF5C8273, 0xFF5A8071, 0xFF5E8475, 0xFF588370, 0xFF588370, 0xFF57826F, 0xFF57826F, 0xFF57826F, 0xFF56816E, 0xFF547F6C, 0xFF527D6A, 0xFF5A8371, 0xFF55806D, 0xFF57826F, 0xFF527D6A, 0xFF527D6A, 0xFF57826F, 0xFF57826F, 0xFF5C8774, 0xFF628277,
        0xFF65867B, 0xFF5D8175, 0xFF597F72, 0xFF608679, 0xFF618579, 0xFF5E7F74, 0xFF628277, 0xFF608574, 0xFF628776, 0xFF608574, 0xFF5C8371, 0xFF5D8472, 0xFF5B8472, 0xFF598270, 0xFF598270, 0xFF5B8270, 0xFF67907E, 0xFF4B7462, 0xFF3F6A57, 0xFF365F4D,
        0xFF4A715F, 0xFF3E6050, 0xFF4B6B5C, 0xFF3A5E50, 0xFF507466, 0xFF527668, 0xFF4E7264, 0xFF4F7365, 0xFF507466, 0xFF4A6E60, 0xFF507466, 0xFF507765, 0xFF4C7160, 0xFF527464, 0xFF5C7E70, 0xFF345648, 0xFF1C4235, 0xFF456E60, 0xFF4F796B, 0xFF204033,
        0xFF244F3E, 0xFF407360, 0xFF3C6D5A, 0xFF3F6355, 0xFF5F7D71, 0xFF5E8072, 0xFF517A6A, 0xFF4F806D, 0xFF4F7B6A, 0xFF517A6A, 0xFF587F6D, 0xFF577C6A, 0xFF517664, 0xFF557B66, 0xFF5B826D, 0xFF5E7D6D, 0xFF5C7B6B, 0xFF5B7A6A, 0xFF5B7A6A, 0xFF5D7C6C,
        0xFF5E7D6D, 0xFF5E7D6D, 0xFF5D7C6C, 0xFF597E6C, 0xFF5C7E6D, 0xFF628474, 0xFF628474, 0xFF628476, 0xFF5E7F74, 0xFF57786D, 0xFF608176, 0xFF5C8072, 0xFF5D8374, 0xFF5B8474, 0xFF5E8475, 0xFF4C6A5E, 0x00CCDDD5, 0x00FBFDFA, 0x00FFFDFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x00FBFBFB, 0xFFBACFC8, 0xFF3D6958, 0xFF679281, 0xFF708F80, 0xFF426756, 0xFF175139, 0xFF4D7462, 0xFF1F4634, 0xFF4F7664, 0xFF244B39, 0xFF3E6553,
        0xFF446B59, 0xFF1C4331, 0xFF234A38, 0xFF548065, 0xFF577961, 0xFF586957, 0xFFADAFA2, 0xFFE8E1D7, 0xFFE1D8CF, 0xFFD4D0C5, 0xFFBDBDB1, 0xFFDFCDC9, 0xFFE2D4D1, 0xFFDDD5D3, 0xFF9D9D9B, 0xFF717A75, 0xFF6E7F75, 0xFF4E6656, 0xFF5A7562, 0xFFA19D9C,
        0xFF7B7A78, 0xFF686866, 0xFF6D6F6C, 0xFF5B615D, 0xFF89928D, 0xFF79867F, 0xFF313E37, 0xFF5B8776, 0xFF558170, 0xFF54806F, 0xFF578372, 0xFF578372, 0xFF598574, 0xFF5A8675, 0xFF588473, 0xFF5C8776, 0xFF588372, 0xFF598473, 0xFF598272, 0xFF618A7A,
        0xFF4B7464, 0xFF395F50, 0xFF2A5041, 0xFF1F4634, 0xFF49705E, 0xFF406755, 0xFF113826, 0xFF355C4A, 0xFF547B69, 0xFF234A38, 0xFF183F2D, 0xFF19452E, 0xFF16422B, 0xFF15412A, 0xFF1A462F, 0xFF1E4A33, 0xFF1C4831, 0xFF18442D, 0xFF18442D, 0xFF17442F,
        0xFF16432E, 0xFF194631, 0xFF194631, 0xFF16432E, 0xFF1B4833, 0xFF1E4B36, 0xFF184530, 0xFF194631, 0xFF17442D, 0xFF17432C, 0xFF17432C, 0xFF18422A, 0xFF174129, 0xFF174227, 0xFF184026, 0xFF19432F, 0xFF39634F, 0xFF406957, 0xFF5F8876, 0xFF5B8472,
        0xFF5C8575, 0xFF5A8373, 0xFF5E8777, 0xFF5C8776, 0xFF5B8675, 0xFF5A8574, 0xFF5A8574, 0xFF5A8574, 0xFF5A8574, 0xFF5B8675, 0xFF5C8776, 0xFF5D8676, 0xFF5D8676, 0xFF5B8474, 0xFF5A8373, 0xFF5A8373, 0xFF598272, 0xFF598272, 0xFF598272, 0xFF578070,
        0xFF578070, 0xFF588171, 0xFF578070, 0xFF578070, 0xFF567F6F, 0xFF557E6E, 0xFF547D6D, 0xFF557E6C, 0xFF557E6C, 0xFF567F6D, 0xFF567F6D, 0xFF57806E, 0xFF58816F, 0xFF58816F, 0xFF598270, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF588171,
        0xFF588171, 0xFF588171, 0xFF588171, 0xFF598272, 0xFF588171, 0xFF588171, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF577D6E, 0xFF5A8373, 0xFF588171, 0xFF567F6D, 0xFF5A8572, 0xFF598471, 0xFF55826D, 0xFF588570, 0xFF538174,
        0xFF548373, 0xFF548071, 0xFF547F6E, 0xFF547D6B, 0xFF5D8471, 0xFF5A7F6D, 0xFF597E6C, 0xFF5A826A, 0xFF5C836E, 0xFF577E69, 0xFF5A816E, 0xFF5F8473, 0xFF618373, 0xFF628275, 0xFF5B796D, 0xFF618875, 0xFF577E6B, 0xFF567D6A, 0xFF416855, 0xFF3F6653,
        0xFF406754, 0xFF406754, 0xFF385F4C, 0xFF577E6B, 0xFF375E4B, 0xFF2B523F, 0xFF426956, 0xFF446B58, 0xFF2B523F, 0xFF426956, 0xFF416855, 0xFF294B3A, 0xFF4C6E5D, 0xFF3F6452, 0xFF2B503E, 0xFF436A57, 0xFF3F6955, 0xFF426C58, 0xFF26513D, 0xFF213F33,
        0xFF55796B, 0xFF305B4A, 0xFF476D60, 0xFF618176, 0xFF56776E, 0xFF528073, 0xFF3C7765, 0xFF506C60, 0xFF4B6B5E, 0xFF45695B, 0xFF406B5A, 0xFF3F6B5A, 0xFF3F6B5A, 0xFF3E6958, 0xFF3D6656, 0xFF44685A, 0xFF426658, 0xFF436759, 0xFF466A5C, 0xFF476B5D,
        0xFF45695B, 0xFF476B5D, 0xFF4B6F61, 0xFF4F7163, 0xFF486A5C, 0xFF597B6D, 0xFF547666, 0xFF557A69, 0xFF5E8371, 0xFF59806D, 0xFF507A66, 0xFF597E6D, 0xFF59806E, 0xFF57806E, 0xFF598272, 0xFF4E6E61, 0x00C3D8D1, 0x00F9FFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x00FDFDFD, 0xFFBCD1CA, 0xFF3B6756, 0xFF618C7B, 0xFF698879, 0xFF3C6150, 0xFF104A32, 0xFF577E6C, 0xFF3E6553, 0xFF4D7462, 0xFF214836, 0xFF49705E,
        0xFF4B7260, 0xFF163D2B, 0xFF1E4533, 0xFF61846E, 0xFF49624F, 0xFF5F6A5C, 0xFFBBB7AE, 0xFFEDE0DA, 0xFFDACDC7, 0xFFDACFC9, 0xFFD9D2CA, 0xFFDCCFC9, 0xFFECE3DE, 0xFFD6D2CF, 0xFF919390, 0xFFA8B3AD, 0xFF6D8176, 0xFF354D3D, 0xFF66836F, 0xFFAAAFAB,
        0xFF8A908C, 0xFF59625D, 0xFF5F6A64, 0xFF798881, 0xFF9FB0A8, 0xFF798D84, 0xFF495E55, 0xFF416D5C, 0xFF5F8B7A, 0xFF5D8978, 0xFF517D6C, 0xFF5C8877, 0xFF5C8877, 0xFF537F6E, 0xFF5B8776, 0xFF5D8877, 0xFF598473, 0xFF5B8675, 0xFF5A8373, 0xFF628B7B,
        0xFF4D7666, 0xFF3A6051, 0xFF2C5243, 0xFF113826, 0xFF2E5543, 0xFF4A715F, 0xFF2C5341, 0xFF163D2B, 0xFF416856, 0xFF476E5C, 0xFF133A28, 0xFF1C4831, 0xFF1B4730, 0xFF1A462F, 0xFF1B4730, 0xFF1B4730, 0xFF19452E, 0xFF19452E, 0xFF1A462F, 0xFF194430,
        0xFF194430, 0xFF1A4531, 0xFF1A4531, 0xFF18432F, 0xFF1A4531, 0xFF1B4632, 0xFF194430, 0xFF1C4934, 0xFF1E4935, 0xFF19452E, 0xFF16422B, 0xFF1A442C, 0xFF18422A, 0xFF173F25, 0xFF194127, 0xFF19432F, 0xFF39634F, 0xFF406957, 0xFF5E8775, 0xFF5A8371,
        0xFF5C8575, 0xFF5A8373, 0xFF5D8676, 0xFF5B8675, 0xFF5A8574, 0xFF5A8574, 0xFF5A8574, 0xFF5A8574, 0xFF5A8574, 0xFF5A8574, 0xFF5B8675, 0xFF5B8474, 0xFF5A8373, 0xFF5A8373, 0xFF598272, 0xFF598272, 0xFF598272, 0xFF598272, 0xFF5A8373, 0xFF5A8373,
        0xFF5A8373, 0xFF5A8373, 0xFF5A8373, 0xFF598272, 0xFF588171, 0xFF588171, 0xFF578070, 0xFF567F6D, 0xFF567F6D, 0xFF57806E, 0xFF57806E, 0xFF58816F, 0xFF598270, 0xFF598270, 0xFF598270, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF588171,
        0xFF588171, 0xFF588171, 0xFF588171, 0xFF598272, 0xFF588171, 0xFF588171, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF557E6E, 0xFF547D6D, 0xFF567F6F, 0xFF5A8371, 0xFF5A816F, 0xFF517866, 0xFF4F7663, 0xFF597E6C, 0xFF5C8776,
        0xFF4A7363, 0xFF567C6D, 0xFF577C6B, 0xFF688F7D, 0xFF4D7664, 0xFF57826F, 0xFF63907C, 0xFF567F6D, 0xFF56816E, 0xFF57826F, 0xFF537C6A, 0xFF5B8270, 0xFF5A7C6E, 0xFF4D6B5F, 0xFF597569, 0xFF587A69, 0xFF4F7160, 0xFF507261, 0xFF375948, 0xFF3E604F,
        0xFF426453, 0xFF3C5E4D, 0xFF476958, 0xFF395B4A, 0xFF305241, 0xFF355746, 0xFF446655, 0xFF385A49, 0xFF264837, 0xFF476958, 0xFF476958, 0xFF365545, 0xFF466555, 0xFF385948, 0xFF345645, 0xFF456A58, 0xFF406553, 0xFF3F6653, 0xFF3D6451, 0xFF406053,
        0xFF3C6253, 0xFF436E5D, 0xFF587C70, 0xFF567168, 0xFF47625B, 0xFF31594E, 0xFF3D7262, 0xFF44655C, 0xFF3D6358, 0xFF355F53, 0xFF2F5D50, 0xFF2C5C4E, 0xFF2A5A4C, 0xFF285649, 0xFF275346, 0xFF285246, 0xFF265044, 0xFF275145, 0xFF2A5448, 0xFF2B5549,
        0xFF2A5448, 0xFF2C564A, 0xFF315B4F, 0xFF355F51, 0xFF3C6557, 0xFF426358, 0xFF4A685C, 0xFF4E6C60, 0xFF527263, 0xFF527766, 0xFF517A68, 0xFF577C6B, 0xFF577E6C, 0xFF557E6C, 0xFF597F70, 0xFF4C6C5F, 0x00C3D6D0, 0x00FAFFFE, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFE, 0x00FEFEFE, 0xFFBBD0C9, 0xFF3E6A59, 0xFF679281, 0xFF6F8E7F, 0xFF426756, 0xFF124C34, 0xFF4E7563, 0xFF4B7260, 0xFF426957, 0xFF264D3B, 0xFF49705E,
        0xFF4C7361, 0xFF214836, 0xFF264D3B, 0xFF657F72, 0xFF3E5247, 0xFF5E655E, 0xFFACA7A4, 0xFFF6EAEA, 0xFFEDE1E1, 0xFFDFD5D4, 0xFFDBD3D1, 0xFFDEDBD2, 0xFFDEDDD8, 0xFFD4D6D1, 0xFF656E6B, 0xFF768580, 0xFF5D7367, 0xFF4A6657, 0xFF3A5A45, 0xFF40544B,
        0xFF5A6E65, 0xFF4A5F56, 0xFF435A50, 0xFF405A4F, 0xFF486458, 0xFF426054, 0xFF5A7A6D, 0xFF34604F, 0xFF4D7968, 0xFF5F8B7A, 0xFF5C8877, 0xFF578372, 0xFF598574, 0xFF5B8776, 0xFF588473, 0xFF5D8877, 0xFF5A8574, 0xFF5C8776, 0xFF5B8474, 0xFF638C7C,
        0xFF4E7767, 0xFF3C6253, 0xFF2E5445, 0xFF3B6250, 0xFF103725, 0xFF365D4B, 0xFF557C6A, 0xFF19402E, 0xFF1D4432, 0xFF537A68, 0xFF385F4D, 0xFF16412D, 0xFF17422E, 0xFF18432F, 0xFF17422E, 0xFF15402C, 0xFF17422E, 0xFF1A4531, 0xFF1B4632, 0xFF1B4632,
        0xFF1C4733, 0xFF1B4632, 0xFF1B4632, 0xFF1B4632, 0xFF1A4531, 0xFF194430, 0xFF1B4632, 0xFF194430, 0xFF1C4733, 0xFF16422B, 0xFF143E28, 0xFF19432D, 0xFF1A422A, 0xFF173F27, 0xFF194129, 0xFF18422E, 0xFF39634F, 0xFF3F6856, 0xFF5E8775, 0xFF598270,
        0xFF5B8474, 0xFF598272, 0xFF5C8575, 0xFF598473, 0xFF598473, 0xFF598473, 0xFF5A8574, 0xFF5A8574, 0xFF598473, 0xFF598473, 0xFF598473, 0xFF598272, 0xFF588171, 0xFF588171, 0xFF588171, 0xFF588171, 0xFF598272, 0xFF5A8373, 0xFF5A8373, 0xFF5A8373,
        0xFF5A8373, 0xFF5A8373, 0xFF5A8373, 0xFF5A8373, 0xFF598272, 0xFF588171, 0xFF578070, 0xFF57806E, 0xFF57806E, 0xFF58816F, 0xFF58816F, 0xFF598270, 0xFF598270, 0xFF5A8371, 0xFF5A8371, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF588171,
        0xFF588171, 0xFF588171, 0xFF588171, 0xFF598272, 0xFF588171, 0xFF588171, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF568271, 0xFF55806F, 0xFF557E6E, 0xFF517665, 0xFF426454, 0xFF395849, 0xFF3B5748, 0xFF405C4D, 0xFF436652,
        0xFF3E5F4C, 0xFF3D594A, 0xFF3B5749, 0xFF507061, 0xFF4E7767, 0xFF518171, 0xFF4B826F, 0xFF578177, 0xFF568275, 0xFF5B877A, 0xFF517D6E, 0xFF598272, 0xFF507264, 0xFF365244, 0xFF526A5D, 0xFF334E3F, 0xFF385344, 0xFF425D4E, 0xFF233E2F, 0xFF395445,
        0xFF3B5647, 0xFF213C2D, 0xFF445F50, 0xFF466152, 0xFF264132, 0xFF647F70, 0xFF3E594A, 0xFF254031, 0xFF415C4D, 0xFF264132, 0xFF3B5647, 0xFF5B7164, 0xFF2B4335, 0xFF385344, 0xFF516D5E, 0xFF244333, 0xFF325342, 0xFF4C6E5D, 0xFF325745, 0xFF46685A,
        0xFF315A4A, 0xFF467160, 0xFF45665B, 0xFF485F57, 0xFF697E79, 0xFF4C6B63, 0xFF2F594D, 0xFF366158, 0xFF2E5C52, 0xFF24584C, 0xFF205649, 0xFF1C5447, 0xFF1A5245, 0xFF1A4E42, 0xFF194B40, 0xFF194C43, 0xFF194C43, 0xFF1A4D44, 0xFF1B4E45, 0xFF1C4F46,
        0xFF1C4F46, 0xFF1F5249, 0xFF23564D, 0xFF205648, 0xFF38665B, 0xFF32534C, 0xFF49605A, 0xFF495E59, 0xFF4B655C, 0xFF4F6F64, 0xFF577D70, 0xFF577969, 0xFF557C6A, 0xFF547D6B, 0xFF587E6F, 0xFF4B6B5E, 0x00C2D5CF, 0x00FAFFFE, 0x00FFFEFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x00FEFEFE, 0xFFB8CDC6, 0xFF3B6756, 0xFF648F7E, 0xFF6A897A, 0xFF3D6251, 0xFF0B452D, 0xFF204735, 0xFF214836, 0xFF1C4331, 0xFF183F2D, 0xFF204735,
        0xFF264D3B, 0xFF19402E, 0xFF183F2D, 0xFF5C7A6E, 0xFF607A6F, 0xFF85968E, 0xFF97A09B, 0xFFDCDEDB, 0xFFDFE1DE, 0xFFD2D4D1, 0xFFDBE0DC, 0xFFD3DBD0, 0xFFECF6ED, 0xFFC5D2CB, 0xFF61726C, 0xFF9CB3AB, 0xFF779387, 0xFF537463, 0xFF6E8F7C, 0xFF5D7F71,
        0xFF5D7F71, 0xFF56786A, 0xFF5E8274, 0xFF5C8273, 0xFF5C8273, 0xFF446D5D, 0xFF396252, 0xFF376251, 0xFF376251, 0xFF4D7867, 0xFF608B7A, 0xFF5B8675, 0xFF598473, 0xFF5D8877, 0xFF598473, 0xFF5C8776, 0xFF5A8574, 0xFF5C8776, 0xFF5B8474, 0xFF628B7B,
        0xFF4D7666, 0xFF3C6253, 0xFF2E5445, 0xFF5C8371, 0xFF355C4A, 0xFF1C4331, 0xFF406755, 0xFF476E5C, 0xFF1B4230, 0xFF264D3B, 0xFF59806E, 0xFF244E3A, 0xFF224C38, 0xFF224C38, 0xFF204A36, 0xFF1D4733, 0xFF1F4935, 0xFF214B37, 0xFF1D4733, 0xFF1D4733,
        0xFF1E4834, 0xFF1C4632, 0xFF1D4733, 0xFF1F4935, 0xFF1D4733, 0xFF1B4531, 0xFF1F4935, 0xFF1C4733, 0xFF1B4632, 0xFF1B4531, 0xFF1A442E, 0xFF1B422D, 0xFF1D442F, 0xFF1B432B, 0xFF173D26, 0xFF18422E, 0xFF38624E, 0xFF3F6856, 0xFF5D8674, 0xFF598270,
        0xFF5A8373, 0xFF588171, 0xFF5B8474, 0xFF578271, 0xFF588372, 0xFF598473, 0xFF598473, 0xFF598473, 0xFF598473, 0xFF588372, 0xFF578271, 0xFF5A8373, 0xFF598272, 0xFF598272, 0xFF588171, 0xFF598272, 0xFF598272, 0xFF5A8373, 0xFF5A8373, 0xFF588171,
        0xFF598272, 0xFF598272, 0xFF598272, 0xFF588171, 0xFF578070, 0xFF567F6F, 0xFF567F6F, 0xFF57806E, 0xFF57806E, 0xFF58816F, 0xFF58816F, 0xFF58816F, 0xFF598270, 0xFF598270, 0xFF598270, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF588171,
        0xFF588171, 0xFF588171, 0xFF588171, 0xFF598272, 0xFF588171, 0xFF588171, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF537F6E, 0xFF598574, 0xFF5C8575, 0xFF4D7261, 0xFF3E5D4E, 0xFF455F52, 0xFF53695C, 0xFF52665A, 0xFF415E4A,
        0xFF4D6855, 0xFF425949, 0xFF586E61, 0xFF3B5749, 0xFF3B6152, 0xFF336655, 0xFF4E8776, 0xFF5A7E7A, 0xFF557E76, 0xFF568177, 0xFF528073, 0xFF598375, 0xFF4A6C5E, 0xFF344E41, 0xFF4E6257, 0xFF182C20, 0xFF34483C, 0xFF42564A, 0xFF162A1E, 0xFF3A4E42,
        0xFF3B4F43, 0xFF102418, 0xFF4A5E52, 0xFF3F5347, 0xFF1F3327, 0xFF586C60, 0xFF304438, 0xFF24382C, 0xFF576B5F, 0xFF2E4236, 0xFF2B3F33, 0xFF5E6E63, 0xFF283A2E, 0xFF35493D, 0xFF50685A, 0xFF1A3526, 0xFF335242, 0xFF476857, 0xFF385A49, 0xFF3C6052,
        0xFF3A6353, 0xFF396453, 0xFF3B5B50, 0xFF566963, 0xFF869592, 0xFF758C86, 0xFF2F5349, 0xFF30635C, 0xFF285D55, 0xFF1E574E, 0xFF18564B, 0xFF165449, 0xFF145046, 0xFF154D44, 0xFF174C44, 0xFF124B44, 0xFF134C45, 0xFF134C45, 0xFF144D46, 0xFF144D46,
        0xFF154E47, 0xFF175049, 0xFF1A534C, 0xFF1F5D4E, 0xFF215348, 0xFF2B4B46, 0xFF6A7D7B, 0xFF627070, 0xFF465955, 0xFF38564E, 0xFF5A8075, 0xFF577969, 0xFF567B6A, 0xFF567D6B, 0xFF587E6F, 0xFF4D6B5F, 0x00C3D6D0, 0x00FAFFFE, 0x00FFFEFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFE, 0x00FFFFFF, 0xFFB9CEC7, 0xFF3C6857, 0xFF618C7B, 0xFF688778, 0xFF4B705F, 0xFF266048, 0xFF345B49, 0xFF335A48, 0xFF305745, 0xFF3B6250, 0xFF325947,
        0xFF325947, 0xFF365D4B, 0xFF365D4B, 0xFF537C6E, 0xFF61877A, 0xFF7A9A8F, 0xFF5C776E, 0xFF748E85, 0xFF728981, 0xFF6E857D, 0xFF819B92, 0xFF799080, 0xFF6E8478, 0xFF6B8278, 0xFF536E67, 0xFF617F77, 0xFF557568, 0xFF537565, 0xFF5D836E, 0xFF5B8675,
        0xFF578271, 0xFF5E8978, 0xFF64907F, 0xFF537F6E, 0xFF588473, 0xFF578372, 0xFF487463, 0xFF3C6756, 0xFF376251, 0xFF345F4E, 0xFF497463, 0xFF638E7D, 0xFF5E8978, 0xFF537E6D, 0xFF5E8978, 0xFF5A8574, 0xFF588372, 0xFF5B8675, 0xFF5A8373, 0xFF618A7A,
        0xFF4B7464, 0xFF3A6051, 0xFF2E5445, 0xFF5A816F, 0xFF5D8472, 0xFF29503E, 0xFF1C4331, 0xFF486F5D, 0xFF385F4D, 0xFF183F2D, 0xFF325947, 0xFF4C7563, 0xFF466F5D, 0xFF47705E, 0xFF497260, 0xFF497260, 0xFF4C7563, 0xFF4D7664, 0xFF47705E, 0xFF4B725F,
        0xFF4C7360, 0xFF4C7360, 0xFF4D7461, 0xFF4E7562, 0xFF4E7562, 0xFF4E7562, 0xFF4F7663, 0xFF4B7462, 0xFF456E5C, 0xFF4A7460, 0xFF4D7461, 0xFF466D5A, 0xFF496F5A, 0xFF496F5A, 0xFF395F4A, 0xFF18422E, 0xFF38624E, 0xFF3F6856, 0xFF5D8674, 0xFF58816F,
        0xFF5A8373, 0xFF578070, 0xFF5A8373, 0xFF578271, 0xFF578271, 0xFF588372, 0xFF598473, 0xFF598473, 0xFF588372, 0xFF578271, 0xFF578271, 0xFF5D8676, 0xFF5C8575, 0xFF5B8474, 0xFF5A8373, 0xFF598272, 0xFF598272, 0xFF598272, 0xFF598272, 0xFF598272,
        0xFF598272, 0xFF598272, 0xFF598272, 0xFF598272, 0xFF588171, 0xFF578070, 0xFF567F6F, 0xFF567F6D, 0xFF57806E, 0xFF57806E, 0xFF57806E, 0xFF57806E, 0xFF57806E, 0xFF57806E, 0xFF57806E, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF588171,
        0xFF588171, 0xFF588171, 0xFF588171, 0xFF598272, 0xFF588171, 0xFF588171, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF537F6E, 0xFF598574, 0xFF5C8575, 0xFF4D7261, 0xFF3D5C4D, 0xFF445E51, 0xFF52685B, 0xFF52665A, 0xFF597461,
        0xFF587060, 0xFF3E5545, 0xFF5C7265, 0xFF4F6A5B, 0xFF557566, 0xFF365F4F, 0xFF285745, 0xFF36514C, 0xFF3C5B55, 0xFF41695E, 0xFF558172, 0xFF5D8779, 0xFF496B5D, 0xFF3B5248, 0xFF44554B, 0xFF1B281E, 0xFF445147, 0xFF4C594F, 0xFF17241A, 0xFF435046,
        0xFF465349, 0xFF1B281E, 0xFF5F6C62, 0xFF47544A, 0xFF233026, 0xFF57645A, 0xFF2B382E, 0xFF27342A, 0xFF657268, 0xFF2F3C32, 0xFF253228, 0xFF626E64, 0xFF253228, 0xFF35453A, 0xFF54685C, 0xFF173223, 0xFF335242, 0xFF466756, 0xFF385A49, 0xFF385C4E,
        0xFF3F6858, 0xFF3A6554, 0xFF3F5F54, 0xFF3C4F49, 0xFF2E3D3A, 0xFF324943, 0xFF2F5349, 0xFF2F675E, 0xFF276057, 0xFF1E5A50, 0xFF1A584D, 0xFF17554A, 0xFF165248, 0xFF1A5249, 0xFF20534C, 0xFF1A514B, 0xFF1B524C, 0xFF1B524C, 0xFF1B524C, 0xFF1B524C,
        0xFF1B524C, 0xFF1A514B, 0xFF19504A, 0xFF165344, 0xFF25574C, 0xFF34544F, 0xFF485B59, 0xFF3C4848, 0xFF50635F, 0xFF46645C, 0xFF4D7167, 0xFF5A7A6B, 0xFF587D6C, 0xFF587F6D, 0xFF5B7F71, 0xFF4E6C60, 0x00C6D7D1, 0x00FCFFFF, 0x00FFFEFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFCFD, 0x00FFFFFF, 0xFFBBD0C9, 0xFF3F6B5A, 0xFF608B7A, 0xFF6A897A, 0xFF648978, 0xFF528C74, 0xFF668D7B, 0xFF668D7B, 0xFF608775, 0xFF678E7C, 0xFF6A917F,
        0xFF628977, 0xFF628977, 0xFF6A917F, 0xFF679382, 0xFF558170, 0xFF618C7B, 0xFF568170, 0xFF5E8777, 0xFF5A8071, 0xFF5B8172, 0xFF658B7C, 0xFF5A7B6A, 0xFF5D7F6F, 0xFF5C7D72, 0xFF52766C, 0xFF4F7369, 0xFF5E8477, 0xFF638A78, 0xFF5E8572, 0xFF5B8A78,
        0xFF578674, 0xFF568573, 0xFF558472, 0xFF4F7B6A, 0xFF598574, 0xFF608C7B, 0xFF5C8877, 0xFF4C7565, 0xFF3B6454, 0xFF315A4A, 0xFF3C6555, 0xFF507969, 0xFF5C8575, 0xFF5B8474, 0xFF588171, 0xFF588372, 0xFF568170, 0xFF5A8574, 0xFF598272, 0xFF5F8878,
        0xFF4A7363, 0xFF3A6051, 0xFF2E5445, 0xFF587F6D, 0xFF638A78, 0xFF315846, 0xFF1A412F, 0xFF426957, 0xFF3C6351, 0xFF1A412F, 0xFF224937, 0xFF4A7361, 0xFF406957, 0xFF436C5A, 0xFF497260, 0xFF47705E, 0xFF497260, 0xFF4B7462, 0xFF466F5D, 0xFF446957,
        0xFF446957, 0xFF476C5A, 0xFF466B59, 0xFF446957, 0xFF486D5B, 0xFF4B705E, 0xFF466B59, 0xFF466F5D, 0xFF426B59, 0xFF4B7260, 0xFF4E7562, 0xFF496E5C, 0xFF4D7260, 0xFF4B715C, 0xFF3C5F4B, 0xFF19432F, 0xFF39634F, 0xFF3F6856, 0xFF5D8674, 0xFF58816F,
        0xFF598272, 0xFF578070, 0xFF5A8373, 0xFF578271, 0xFF588372, 0xFF588372, 0xFF588372, 0xFF588372, 0xFF588372, 0xFF588372, 0xFF578271, 0xFF5E8777, 0xFF5D8676, 0xFF5C8575, 0xFF5B8474, 0xFF5A8373, 0xFF598272, 0xFF598272, 0xFF598272, 0xFF5C8575,
        0xFF5C8575, 0xFF5C8575, 0xFF5C8575, 0xFF5B8474, 0xFF5A8373, 0xFF598272, 0xFF598272, 0xFF567F6D, 0xFF567F6D, 0xFF567F6D, 0xFF567F6D, 0xFF567F6D, 0xFF567F6D, 0xFF567F6D, 0xFF567F6D, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF588171,
        0xFF588171, 0xFF588171, 0xFF588171, 0xFF598272, 0xFF588171, 0xFF588171, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF578372, 0xFF55806F, 0xFF557E6E, 0xFF507564, 0xFF416353, 0xFF375647, 0xFF3A5647, 0xFF3E5A4B, 0xFF3A5648,
        0xFF3D594B, 0xFF3E5A4B, 0xFF3C5849, 0xFF496453, 0xFF466150, 0xFF415949, 0xFF132A18, 0xFF39433A, 0xFF3B4D41, 0xFF315041, 0xFF385F4D, 0xFF3A6353, 0xFF2C4E40, 0xFF2E453D, 0xFF384943, 0xFF131F15, 0xFF354137, 0xFF3C483E, 0xFF131F15, 0xFF38443A,
        0xFF39453B, 0xFF18241A, 0xFF4C584E, 0xFF3A463C, 0xFF18241A, 0xFF525E54, 0xFF232F25, 0xFF1B271D, 0xFF58645A, 0xFF1E2A20, 0xFF273329, 0xFF586259, 0xFF162218, 0xFF2A3A2F, 0xFF4D6356, 0xFF142F20, 0xFF2B4C3B, 0xFF426453, 0xFF2D5240, 0xFF3D5F51,
        0xFF325B4B, 0xFF3B6655, 0xFF4F7065, 0xFF6E857D, 0xFF5E736E, 0xFF36554D, 0xFF2D574B, 0xFF2F645A, 0xFF255D52, 0xFF1E584C, 0xFF1B574B, 0xFF1C564A, 0xFF22574D, 0xFF2D5F56, 0xFF3B6861, 0xFF3A685E, 0xFF3A685E, 0xFF3A685E, 0xFF3A685E, 0xFF3A685E,
        0xFF38665C, 0xFF326056, 0xFF2B594F, 0xFF25594C, 0xFF1C473D, 0xFF385751, 0xFF5F746F, 0xFF576A66, 0xFF5A6F68, 0xFF4A685E, 0xFF5B7F73, 0xFF5C7C6D, 0xFF5C7E6E, 0xFF5B806F, 0xFF5C8072, 0xFF516D61, 0x00C6D7D1, 0x00FCFFFF, 0x00FFFEFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x00FFFFFF, 0xFFB5CAC3, 0xFF3C6857, 0xFF5C8776, 0xFF638273, 0xFF618675, 0xFF569078, 0xFF608775, 0xFF5F8674, 0xFF5F8674, 0xFF5A816F, 0xFF688F7D,
        0xFF618876, 0xFF5C8371, 0xFF668D7B, 0xFF668D7B, 0xFF5B8270, 0xFF5F8876, 0xFF67927F, 0xFF648F7C, 0xFF5C8573, 0xFF648B79, 0xFF678E7C, 0xFF628D79, 0xFF638E7B, 0xFF628C7E, 0xFF648E82, 0xFF628C80, 0xFF5C8678, 0xFF5A8572, 0xFF5B8672, 0xFF548371,
        0xFF578372, 0xFF568271, 0xFF5A8675, 0xFF5E8978, 0xFF5E8978, 0xFF5A8373, 0xFF5A8373, 0xFF5D8676, 0xFF436C5C, 0xFF3E6757, 0xFF3C6555, 0xFF365F4F, 0xFF4C7565, 0xFF608979, 0xFF547D6D, 0xFF578271, 0xFF568170, 0xFF5A8574, 0xFF5A8373, 0xFF5F8878,
        0xFF4A7363, 0xFF3A6051, 0xFF2E5445, 0xFF567D6B, 0xFF577E6C, 0xFF1E4533, 0xFF2B5240, 0xFF557C6A, 0xFF264D3B, 0xFF153C2A, 0xFF49705E, 0xFF4C7361, 0xFF406755, 0xFF446B59, 0xFF4B7260, 0xFF426957, 0xFF3D6452, 0xFF446B59, 0xFF456C5A, 0xFF486A59,
        0xFF466857, 0xFF4C6E5D, 0xFF4A6C5B, 0xFF426453, 0xFF4C6E5D, 0xFF527463, 0xFF446655, 0xFF3B6454, 0xFF426859, 0xFF426957, 0xFF416856, 0xFF456A58, 0xFF476C5A, 0xFF416352, 0xFF365945, 0xFF1A4430, 0xFF3A6450, 0xFF406957, 0xFF5E8775, 0xFF598270,
        0xFF5A8373, 0xFF578070, 0xFF5A8373, 0xFF588372, 0xFF588372, 0xFF588372, 0xFF578271, 0xFF578271, 0xFF588372, 0xFF588372, 0xFF588372, 0xFF5C8575, 0xFF5B8474, 0xFF5A8373, 0xFF598272, 0xFF598272, 0xFF598272, 0xFF598272, 0xFF5A8373, 0xFF5C8575,
        0xFF5C8575, 0xFF5C8575, 0xFF5C8575, 0xFF5C8575, 0xFF5B8474, 0xFF5A8373, 0xFF598272, 0xFF58816F, 0xFF58816F, 0xFF57806E, 0xFF57806E, 0xFF57806E, 0xFF57806E, 0xFF57806E, 0xFF57806E, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF588171,
        0xFF588171, 0xFF588171, 0xFF588171, 0xFF598272, 0xFF588171, 0xFF588171, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF567F6F, 0xFF557E6E, 0xFF567F6F, 0xFF598270, 0xFF587F6D, 0xFF4E7563, 0xFF4C7360, 0xFF567B69, 0xFF3D5C56,
        0xFF163A30, 0xFF13392A, 0xFF244937, 0xFF42624D, 0xFF3E4F3C, 0xFF535746, 0xFF353221, 0xFF3E3727, 0xFF4C503F, 0xFF4A5D49, 0xFF3F604B, 0xFF3E6552, 0xFF2F5143, 0xFF223933, 0xFF314140, 0xFF172319, 0xFF1C281E, 0xFF222E24, 0xFF111D13, 0xFF253127,
        0xFF212D23, 0xFF101C12, 0xFF232F25, 0xFF1C281E, 0xFF1E2A20, 0xFF263228, 0xFF1F2B21, 0xFF142016, 0xFF263228, 0xFF1C281E, 0xFF152117, 0xFF273329, 0xFF1A271D, 0xFF15271B, 0xFF253B2E, 0xFF254132, 0xFF325443, 0xFF305744, 0xFF37614D, 0xFF3B5B4E,
        0xFF3A6051, 0xFF4C7766, 0xFF45695D, 0xFF4E6960, 0xFF4D6861, 0xFF375F54, 0xFF2E6353, 0xFF316056, 0xFF26584D, 0xFF1C5044, 0xFF194F42, 0xFF1A4E42, 0xFF245248, 0xFF39625A, 0xFF4F746D, 0xFF527466, 0xFF537567, 0xFF527466, 0xFF517365, 0xFF527466,
        0xFF4E7062, 0xFF426456, 0xFF36584A, 0xFF21473A, 0xFF22463A, 0xFF2E4C42, 0xFF3E584D, 0xFF486257, 0xFF557163, 0xFF557566, 0xFF577C6B, 0xFF5E7E6F, 0xFF5D7F6F, 0xFF5B806F, 0xFF5E8072, 0xFF516D61, 0x00C6D5D0, 0x00FDFFFE, 0x00FFFEFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x00FBFBFB, 0xFFAEC3BC, 0xFF3D6958, 0xFF648F7E, 0xFF698879, 0xFF638877, 0xFF538D75, 0xFF648B79, 0xFF5F8674, 0xFF6C9381, 0xFF5D8472, 0xFF678E7C,
        0xFF658C7A, 0xFF638A78, 0xFF668D7B, 0xFF668371, 0xFF71907E, 0xFF618470, 0xFF618772, 0xFF638974, 0xFF638974, 0xFF658874, 0xFF61826F, 0xFF55856F, 0xFF558472, 0xFF598878, 0xFF568177, 0xFF5E897F, 0xFF568273, 0xFF5F8A79, 0xFF57826E, 0xFF598574,
        0xFF588473, 0xFF598473, 0xFF5B8474, 0xFF588171, 0xFF547A6B, 0xFF567C6D, 0xFF5F8375, 0xFF618A7A, 0xFF598272, 0xFF466F5F, 0xFF365F4F, 0xFF345D4D, 0xFF3D6656, 0xFF4F7868, 0xFF608979, 0xFF578271, 0xFF578271, 0xFF5B8675, 0xFF5A8373, 0xFF608979,
        0xFF4A7363, 0xFF3B6152, 0xFF2F5546, 0xFF59806E, 0xFF244B39, 0xFF173E2C, 0xFF557C6A, 0xFF3F6654, 0xFF123927, 0xFF3F6654, 0xFF4A715F, 0xFF1B4132, 0xFF113728, 0xFF1B4132, 0xFF264C3D, 0xFF1E4435, 0xFF1C4233, 0xFF2B5142, 0xFF355B4C, 0xFF254736,
        0xFF224433, 0xFF2B4D3C, 0xFF274938, 0xFF1B3D2C, 0xFF274938, 0xFF305241, 0xFF1C3E2D, 0xFF294F40, 0xFF345A4B, 0xFF2A513F, 0xFF224736, 0xFF2B503E, 0xFF294B3A, 0xFF193B2A, 0xFF153824, 0xFF1A4430, 0xFF3A6450, 0xFF406957, 0xFF5E8775, 0xFF598270,
        0xFF5A8373, 0xFF578070, 0xFF5A8373, 0xFF598473, 0xFF588372, 0xFF588372, 0xFF578271, 0xFF578271, 0xFF588372, 0xFF588372, 0xFF598473, 0xFF598272, 0xFF598272, 0xFF588171, 0xFF588171, 0xFF588171, 0xFF598272, 0xFF5A8373, 0xFF5A8373, 0xFF5A8373,
        0xFF5A8373, 0xFF5A8373, 0xFF5A8373, 0xFF5A8373, 0xFF598272, 0xFF588171, 0xFF578070, 0xFF598270, 0xFF598270, 0xFF598270, 0xFF58816F, 0xFF58816F, 0xFF58816F, 0xFF58816F, 0xFF58816F, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF588171,
        0xFF588171, 0xFF588171, 0xFF588171, 0xFF598272, 0xFF588171, 0xFF588171, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF578070, 0xFF587E6F, 0xFF5B8474, 0xFF588171, 0xFF557E6C, 0xFF588370, 0xFF56816E, 0xFF517E69, 0xFF55826D, 0xFF244743,
        0xFF143C34, 0xFF0D392C, 0xFF1E4936, 0xFF22422D, 0xFF475440, 0xFF7E7765, 0xFF7F6C5B, 0xFF7D6C58, 0xFF87826C, 0xFF77856E, 0xFF4B6951, 0xFF3F6550, 0xFF325446, 0xFF1B342E, 0xFF3B4D4D, 0xFF354238, 0xFF1A271D, 0xFF1B281E, 0xFF19261C, 0xFF1E2B21,
        0xFF18251B, 0xFF18251B, 0xFF162319, 0xFF1A271D, 0xFF162319, 0xFF1D2A20, 0xFF1C291F, 0xFF18251B, 0xFF27342A, 0xFF17241A, 0xFF19261C, 0xFF18241A, 0xFF19261C, 0xFF17291D, 0xFF193123, 0xFF1E3D2D, 0xFF365B49, 0xFF3C6652, 0xFF2D5A45, 0xFF405E52,
        0xFF335749, 0xFF477261, 0xFF5D8376, 0xFF5B7B70, 0xFF45665D, 0xFF416F62, 0xFF326D5B, 0xFF356056, 0xFF27554A, 0xFF19493D, 0xFF114135, 0xFF0F3D32, 0xFF184238, 0xFF32554E, 0xFF4A6B64, 0xFF5B7665, 0xFF5B7665, 0xFF5A7564, 0xFF597463, 0xFF5A7564,
        0xFF55705F, 0xFF45604F, 0xFF365140, 0xFF224034, 0xFF2F4D41, 0xFF274539, 0xFF2B4A3B, 0xFF456455, 0xFF517261, 0xFF5F8170, 0xFF597B6A, 0xFF5F7E6F, 0xFF5D7F6F, 0xFF5B806F, 0xFF5E8072, 0xFF506C60, 0x00C5D4CF, 0x00FCFEFD, 0x00FFFCFE, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x00FEFFFF, 0xFFACC1BA, 0xFF396858, 0xFF5B8778, 0xFF6E8E83, 0xFF4E7266, 0xFF38715E, 0xFF4A7363, 0xFF4B7464, 0xFF4A7363, 0xFF466F5F, 0xFF456E5E,
        0xFF4E7767, 0xFF477060, 0xFF497262, 0xFF658570, 0xFF47624F, 0xFF3B4D3D, 0xFF6A7869, 0xFF4A5849, 0xFF314635, 0xFF2B4633, 0xFF2F503B, 0xFF364236, 0xFF2E3A2E, 0xFF344235, 0xFF425448, 0xFF40584A, 0xFF345445, 0xFF517C69, 0xFF598874, 0xFF5E8976,
        0xFF5D8674, 0xFF5C8573, 0xFF598270, 0xFF608775, 0xFF587F6D, 0xFF5B806F, 0xFF628776, 0xFF5F8576, 0xFF618778, 0xFF5D8374, 0xFF476D5E, 0xFF3B6152, 0xFF385E4F, 0xFF3D6354, 0xFF5B8172, 0xFF5D8374, 0xFF587E6F, 0xFF5E8475, 0xFF577D6E, 0xFF648A7B,
        0xFF507667, 0xFF325849, 0xFF315748, 0xFF254D42, 0xFF194136, 0xFF4A7365, 0xFF477062, 0xFF133C2C, 0xFF2E5747, 0xFF4C7563, 0xFF264F3D, 0xFF16402C, 0xFF123C28, 0xFF315B47, 0xFF446B58, 0xFF3D6451, 0xFF2E5341, 0xFF4C715F, 0xFF355A48, 0xFF274C3A,
        0xFF395E4C, 0xFF3D6451, 0xFF39634F, 0xFF234D39, 0xFF436E5A, 0xFF43705B, 0xFF2D5A45, 0xFF3E6755, 0xFF3E6755, 0xFF2C5642, 0xFF315845, 0xFF446B56, 0xFF426853, 0xFF2B513A, 0xFF143A23, 0xFF16402C, 0xFF37614D, 0xFF3E6854, 0xFF5C8672, 0xFF547E6A,
        0xFF5A8470, 0xFF5C8672, 0xFF59836F, 0xFF5B8270, 0xFF5C8371, 0xFF5C8371, 0xFF5D8472, 0xFF5D8472, 0xFF5E8573, 0xFF5E8573, 0xFF5E8573, 0xFF588475, 0xFF578374, 0xFF588274, 0xFF578173, 0xFF588173, 0xFF598274, 0xFF598274, 0xFF5B8174, 0xFF598272,
        0xFF598272, 0xFF5A8373, 0xFF5B8474, 0xFF5B8474, 0xFF5A8373, 0xFF598272, 0xFF598272, 0xFF5A8574, 0xFF5A8574, 0xFF598473, 0xFF588372, 0xFF588372, 0xFF588372, 0xFF598473, 0xFF598473, 0xFF547D6B, 0xFF567F6D, 0xFF57806E, 0xFF598270, 0xFF5A8371,
        0xFF598270, 0xFF58816F, 0xFF58816F, 0xFF56816E, 0xFF56816E, 0xFF57826F, 0xFF588370, 0xFF588370, 0xFF57826F, 0xFF56816E, 0xFF55806D, 0xFF5B8270, 0xFF5C8371, 0xFF5B8270, 0xFF587F6D, 0xFF59806E, 0xFF5A816F, 0xFF587F6D, 0xFF537A68, 0xFF143E34,
        0xFF254337, 0xFF294131, 0xFF2D4C3C, 0xFF517362, 0xFF5E6F5C, 0xFF9A856A, 0xFFB88162, 0xFFAC8A6F, 0xFF96886D, 0xFF39472E, 0xFF345A43, 0xFF33604C, 0xFF284A3C, 0xFF222F26, 0xFF4F4F47, 0xFF415146, 0xFF3E4E43, 0xFF35453A, 0xFF425247, 0xFF2E3E33,
        0xFF304035, 0xFF3E4E43, 0xFF2D3D32, 0xFF2B3F36, 0xFF364A41, 0xFF2D4138, 0xFF31463F, 0xFF31463F, 0xFF2E453F, 0xFF1B322C, 0xFF122923, 0xFF162927, 0xFF192C28, 0xFF132A24, 0xFF19322C, 0xFF203E34, 0xFF496A5F, 0xFF355B4C, 0xFF3A6051, 0xFF3D5E49,
        0xFF395C46, 0xFF476E5B, 0xFF537E6D, 0xFF49776A, 0xFF467265, 0xFF466F5F, 0xFF406754, 0xFF386359, 0xFF375B4F, 0xFF405546, 0xFF545B49, 0xFF9D9A87, 0xFFB3AC9C, 0xFFBBB1A7, 0xFFB9B0A7, 0xFFBEBCAD, 0xFFBEBCAD, 0xFFBEBCAD, 0xFFBDBBAC, 0xFFBBB9AA,
        0xFFB8B6A7, 0xFFB5B3A4, 0xFFB3B1A2, 0xFFB8B2A4, 0xFFB5AFA1, 0xFFB8B2A4, 0xFFB7B1A3, 0xFFBCB6A8, 0xFFC3BDAF, 0xFFC0BAAC, 0xFFC2BCAE, 0xFFC6C3BC, 0xFF7C8C7F, 0xFF5A7F6D, 0xFF5F8876, 0xFF4F6D61, 0x00C7D3CF, 0x00FEFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFAFB, 0x00FCFEFD, 0xFFACC1BA, 0xFF3D6C5C, 0xFF5E8A7B, 0xFF67877C, 0xFF365A4E, 0xFF144D3A, 0xFF184131, 0xFF174030, 0xFF194232, 0xFF184131, 0xFF1A4333,
        0xFF143D2D, 0xFF143D2D, 0xFF1C4535, 0xFF607966, 0xFF4E6050, 0xFF646F61, 0xFF8A8D82, 0xFF8C8E83, 0xFF848A7E, 0xFF778375, 0xFF768777, 0xFF6F796E, 0xFF879186, 0xFF869087, 0xFF525F55, 0xFF32483B, 0xFF3A594A, 0xFF5E8573, 0xFF5A8773, 0xFF5C8371,
        0xFF638A78, 0xFF628977, 0xFF648B79, 0xFF658E7C, 0xFF668F7D, 0xFF5E8775, 0xFF5C8774, 0xFF5D8374, 0xFF557B6C, 0xFF63897A, 0xFF618778, 0xFF43695A, 0xFF365C4D, 0xFF395F50, 0xFF385E4F, 0xFF5C8273, 0xFF5A8071, 0xFF648A7B, 0xFF608677, 0xFF597F70,
        0xFF4D7364, 0xFF416758, 0xFF1E4435, 0xFF123B2B, 0xFF3B6454, 0xFF436C5C, 0xFF1C4533, 0xFF1F4836, 0xFF466F5D, 0xFF3C6652, 0xFF153F2B, 0xFF153F2B, 0xFF18422E, 0xFF325946, 0xFF3B624F, 0xFF345947, 0xFF284D3B, 0xFF456A58, 0xFF2A4C3B, 0xFF143B28,
        0xFF345B48, 0xFF254F3B, 0xFF234D39, 0xFF2A5440, 0xFF355F4B, 0xFF1C4632, 0xFF1D4733, 0xFF416A58, 0xFF2E5745, 0xFF183F2C, 0xFF2A513E, 0xFF4A705B, 0xFF446A55, 0xFF294F38, 0xFF183B25, 0xFF204A36, 0xFF3B6551, 0xFF3F6955, 0xFF5E8874, 0xFF5B8571,
        0xFF618B77, 0xFF5E8874, 0xFF57816D, 0xFF5C8371, 0xFF5D8472, 0xFF5D8472, 0xFF5D8472, 0xFF5D8472, 0xFF5D8472, 0xFF5C8371, 0xFF5C8371, 0xFF5A8675, 0xFF5A8574, 0xFF5A8574, 0xFF598473, 0xFF598272, 0xFF598272, 0xFF5B8172, 0xFF5C8273, 0xFF588372,
        0xFF588372, 0xFF598473, 0xFF598473, 0xFF598473, 0xFF598473, 0xFF588372, 0xFF588372, 0xFF588370, 0xFF57826F, 0xFF57826F, 0xFF57826F, 0xFF57826F, 0xFF588370, 0xFF598471, 0xFF598471, 0xFF598270, 0xFF5A8371, 0xFF5A8371, 0xFF5A8371, 0xFF5A8371,
        0xFF598270, 0xFF58816F, 0xFF57806E, 0xFF598270, 0xFF598270, 0xFF598270, 0xFF598270, 0xFF598270, 0xFF58816F, 0xFF58816F, 0xFF57806E, 0xFF5B8270, 0xFF5C8371, 0xFF5B8270, 0xFF59806E, 0xFF5A816F, 0xFF5B8270, 0xFF587F6D, 0xFF537A68, 0xFF354C46,
        0xFF657268, 0xFF747F71, 0xFF465D4D, 0xFF375845, 0xFF40563F, 0xFF8C8163, 0xFFC39973, 0xFFB39271, 0xFF9E8F70, 0xFF516043, 0xFF4D735A, 0xFF4C7965, 0xFF426658, 0xFF27382E, 0xFF454A43, 0xFF405045, 0xFF2E3E33, 0xFF2F3F34, 0xFF3B4B40, 0xFF2D3D32,
        0xFF36463B, 0xFF3C4C41, 0xFF38483D, 0xFF364A41, 0xFF354940, 0xFF32463D, 0xFF374C45, 0xFF2C413A, 0xFF253A35, 0xFF162D27, 0xFF182F29, 0xFF1F2F2C, 0xFF1A2D29, 0xFF1F3630, 0xFF122C23, 0xFF36544A, 0xFF3F6153, 0xFF35594B, 0xFF355B4C, 0xFF42634E,
        0xFF3A604B, 0xFF416A5A, 0xFF487467, 0xFF427065, 0xFF3F6D60, 0xFF436F60, 0xFF3D6857, 0xFF3A655C, 0xFF45695D, 0xFF63796C, 0xFF7D8675, 0xFFC8C6B7, 0xFFE8E2D4, 0xFFF0E9DF, 0xFFF1EDE4, 0xFFE8E6DA, 0xFFE9E7DB, 0xFFE9E7DB, 0xFFEBE9DD, 0xFFEBE9DD,
        0xFFEBE9DD, 0xFFEAE8DC, 0xFFE9E7DB, 0xFFECE8DD, 0xFFE8E4D9, 0xFFECE8DD, 0xFFECE8DD, 0xFFEEEADF, 0xFFF1EDE2, 0xFFE8E4D9, 0xFFE6E2D7, 0xFFE3E0D9, 0xFF8C9C8F, 0xFF5B806E, 0xFF5B8472, 0xFF4D6B5F, 0x00C8D4D0, 0x00FEFFFF, 0x00FDFDFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x00FDFFFE, 0xFFABC0B9, 0xFF3C6B5B, 0xFF5F8B7C, 0xFF67877C, 0xFF32564A, 0xFF0C4532, 0xFF396252, 0xFF3C6555, 0xFF2E5747, 0xFF1B4434, 0xFF3D6656,
        0xFF2F5848, 0xFF1B4434, 0xFF254E3E, 0xFF829788, 0xFF4F5D50, 0xFF64675E, 0xFFA29B95, 0xFFE3D8D4, 0xFFEDE2DE, 0xFFDDD6D0, 0xFFDED9D3, 0xFFE5E8DF, 0xFFE6E9E2, 0xFFC9CEC7, 0xFF707A71, 0xFF4A5A50, 0xFF435B4E, 0xFF507264, 0xFF4D7364, 0xFF5F796C,
        0xFF4F6B5D, 0xFF385748, 0xFF446455, 0xFF416655, 0xFF436A58, 0xFF38614F, 0xFF436E5B, 0xFF5C8575, 0xFF5E8777, 0xFF5C8575, 0xFF5F8878, 0xFF588171, 0xFF456E5E, 0xFF3B6454, 0xFF386151, 0xFF416758, 0xFF618778, 0xFF5C8273, 0xFF5A8071, 0xFF678D7E,
        0xFF4E7465, 0xFF395F50, 0xFF284E3F, 0xFF224937, 0xFF4C7361, 0xFF315845, 0xFF0F3623, 0xFF3C634E, 0xFF49705B, 0xFF1E462E, 0xFF133B23, 0xFF19432F, 0xFF183F2C, 0xFF244B38, 0xFF375E4B, 0xFF3F6452, 0xFF345947, 0xFF466857, 0xFF204231, 0xFF15402C,
        0xFF2C5743, 0xFF36604C, 0xFF3D6451, 0xFF325946, 0xFF436856, 0xFF406251, 0xFF3A5C4B, 0xFF436C5A, 0xFF315846, 0xFF1A412E, 0xFF2A513E, 0xFF466C57, 0xFF3E644F, 0xFF2B4E38, 0xFF1B3E28, 0xFF163D2A, 0xFF365D4A, 0xFF436A57, 0xFF5F8673, 0xFF59806D,
        0xFF5D8471, 0xFF5F8673, 0xFF608774, 0xFF5C8371, 0xFF5C8371, 0xFF5D8472, 0xFF5D8472, 0xFF5C8371, 0xFF5B8270, 0xFF5A816F, 0xFF59806E, 0xFF5B8675, 0xFF5A8574, 0xFF598473, 0xFF598272, 0xFF598272, 0xFF5A8071, 0xFF5A8071, 0xFF5B8172, 0xFF55826E,
        0xFF55826E, 0xFF56836F, 0xFF56836F, 0xFF56836F, 0xFF56836F, 0xFF55826E, 0xFF55826E, 0xFF56816E, 0xFF56816E, 0xFF56816E, 0xFF56816E, 0xFF57826F, 0xFF588370, 0xFF598471, 0xFF598471, 0xFF5B8472, 0xFF5B8472, 0xFF5A8371, 0xFF5A8371, 0xFF598270,
        0xFF58816F, 0xFF58816F, 0xFF57806E, 0xFF5B8472, 0xFF5B8472, 0xFF5A8371, 0xFF598270, 0xFF598270, 0xFF598270, 0xFF598270, 0xFF598270, 0xFF5B8270, 0xFF5B8270, 0xFF5A816F, 0xFF59806E, 0xFF5B8270, 0xFF5D8472, 0xFF59806E, 0xFF537A68, 0xFF3B403C,
        0xFF9D9A93, 0xFFBEBAAE, 0xFF5F6C5B, 0xFF314E3C, 0xFF4A644B, 0xFF969672, 0xFFBA9F74, 0xFFBA9872, 0xFFAB9B77, 0xFF657352, 0xFF5A7E64, 0xFF4E7B66, 0xFF416758, 0xFF21382E, 0xFF404D44, 0xFF2F3F35, 0xFF213127, 0xFF27372D, 0xFF26362C, 0xFF223228,
        0xFF2D3D33, 0xFF25352B, 0xFF2F3F35, 0xFF2C3D35, 0xFF2A3B33, 0xFF2B3F36, 0xFF344741, 0xFF283D36, 0xFF273C37, 0xFF1C312C, 0xFF1A312B, 0xFF182825, 0xFF1C2F2B, 0xFF142922, 0xFF1B352C, 0xFF203E34, 0xFF3E6052, 0xFF2F5345, 0xFF3E6455, 0xFF3E5F4A,
        0xFF385F4D, 0xFF3C675D, 0xFF37665C, 0xFF305E53, 0xFF2F5B4E, 0xFF366255, 0xFF356056, 0xFF2B564C, 0xFF38594E, 0xFF60766A, 0xFF727C71, 0xFFBDBDB3, 0xFFE3E0D7, 0xFFE0E0D8, 0xFFDCDCD4, 0xFFE7E4DD, 0xFFE6E3DC, 0xFFE4E1DA, 0xFFE4E1DA, 0xFFE5E2DB,
        0xFFE4E1DA, 0xFFE2DFD8, 0xFFE0DDD6, 0xFFE1E1D9, 0xFFE1E1D9, 0xFFE7E7DF, 0xFFE4E4DC, 0xFFE1E1D9, 0xFFE2E2DA, 0xFFE0E0D8, 0xFFE6E6DE, 0xFFEEEBE4, 0xFF91A194, 0xFF5B806E, 0xFF598270, 0xFF4D6B5F, 0x00CBD7D3, 0x00FEFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x00FEFFFF, 0xFFABC0B9, 0xFF3B6A5A, 0xFF5E8A7B, 0xFF69897E, 0xFF395D51, 0xFF17503D, 0xFF4F7868, 0xFF456E5E, 0xFF477060, 0xFF224B3B, 0xFF487161,
        0xFF4C7565, 0xFF2F5848, 0xFF275040, 0xFF718E7C, 0xFF4B6051, 0xFF545C51, 0xFFAAA59F, 0xFFECDEDB, 0xFFE1D2CF, 0xFFD2C4C1, 0xFFD7CCC8, 0xFFD0CFCA, 0xFFE0DFDA, 0xFFCECFC9, 0xFF646963, 0xFF444F47, 0xFF384C43, 0xFF345044, 0xFF2B4D3F, 0xFF6A736E,
        0xFF57605B, 0xFF303D36, 0xFF35463E, 0xFF2C4339, 0xFF375347, 0xFF29493C, 0xFF335547, 0xFF557E6E, 0xFF5F8878, 0xFF527B6B, 0xFF578070, 0xFF648D7D, 0xFF567F6F, 0xFF446D5D, 0xFF396252, 0xFF416758, 0xFF476D5E, 0xFF567C6D, 0xFF688E7F, 0xFF618778,
        0xFF476D5E, 0xFF385E4F, 0xFF254B3C, 0xFF325944, 0xFF365D48, 0xFF1B422D, 0xFF1C442C, 0xFF406850, 0xFF315941, 0xFF0D351B, 0xFF1A4228, 0xFF153C29, 0xFF173E2B, 0xFF264D3A, 0xFF3F6452, 0xFF406553, 0xFF2F5140, 0xFF476958, 0xFF3D5F4E, 0xFF214C38,
        0xFF28533F, 0xFF3F6955, 0xFF426956, 0xFF1F4432, 0xFF3D5E4D, 0xFF4E6F5E, 0xFF365545, 0xFF3C6351, 0xFF3F6654, 0xFF315845, 0xFF2C513F, 0xFF315742, 0xFF2F523E, 0xFF264933, 0xFF183B25, 0xFF19402D, 0xFF385F4C, 0xFF4A715E, 0xFF608774, 0xFF587F6C,
        0xFF577E6B, 0xFF5B826F, 0xFF5F8673, 0xFF5A816F, 0xFF5B8270, 0xFF5C8371, 0xFF5C8371, 0xFF5C8371, 0xFF5B8270, 0xFF5A816F, 0xFF59806E, 0xFF598471, 0xFF598471, 0xFF598270, 0xFF58816F, 0xFF5A816F, 0xFF5A816F, 0xFF5A816F, 0xFF5B806F, 0xFF55826D,
        0xFF56836E, 0xFF56836E, 0xFF56836E, 0xFF56836E, 0xFF56836E, 0xFF56836E, 0xFF55826D, 0xFF58816F, 0xFF58816F, 0xFF58816F, 0xFF58816F, 0xFF598270, 0xFF598270, 0xFF5A8371, 0xFF5B8472, 0xFF58816F, 0xFF58816F, 0xFF58816F, 0xFF58816F, 0xFF58816F,
        0xFF58816F, 0xFF598270, 0xFF598270, 0xFF5C8573, 0xFF5B8472, 0xFF5A8371, 0xFF598270, 0xFF598270, 0xFF598270, 0xFF598270, 0xFF5A8371, 0xFF5B8270, 0xFF5B8270, 0xFF5A816F, 0xFF5A816F, 0xFF5C8371, 0xFF5E8573, 0xFF5A816F, 0xFF537A68, 0xFF3C3E39,
        0xFF706C61, 0xFF7E7669, 0xFF4C5445, 0xFF435E4B, 0xFF546E55, 0xFF979876, 0xFFB69F75, 0xFFB6976B, 0xFFA99B74, 0xFF5D6846, 0xFF54765B, 0xFF4A7762, 0xFF365F4F, 0xFF1C3A2E, 0xFF42564D, 0xFF36463C, 0xFF37473D, 0xFF3C4C42, 0xFF2F3F35, 0xFF324238,
        0xFF3A4A40, 0xFF2B3B31, 0xFF37473D, 0xFF203129, 0xFF2D3E36, 0xFF283931, 0xFF2F423C, 0xFF2E413B, 0xFF324742, 0xFF293E39, 0xFF152A25, 0xFF21312E, 0xFF162721, 0xFF20352E, 0xFF152F26, 0xFF2C483C, 0xFF39594C, 0xFF446958, 0xFF406554, 0xFF3C5D4A,
        0xFF3B6456, 0xFF3A6963, 0xFF295850, 0xFF1F493D, 0xFF1C4130, 0xFF254B3C, 0xFF254F43, 0xFF285143, 0xFF2D4D42, 0xFF5C7067, 0xFF6F7971, 0xFFBABBB5, 0xFFDDDED8, 0xFFC0C3BC, 0xFFA6ABA4, 0xFFA1A09C, 0xFF9D9C98, 0xFF999894, 0xFF979692, 0xFF979692,
        0xFF969591, 0xFF93928E, 0xFF908F8B, 0xFF9EA09B, 0xFFA2A49F, 0xFFABADA8, 0xFFA4A6A1, 0xFF9A9C97, 0xFFA0A29D, 0xFFB0B2AD, 0xFFC8CAC5, 0xFFE6E3DC, 0xFF8C9C8F, 0xFF5A7F6D, 0xFF5C8573, 0xFF4F6D61, 0x00CAD6D2, 0x00FEFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFBFC, 0x00FDFFFE, 0xFFADC2BB, 0xFF3D6C5C, 0xFF5D897A, 0xFF66867B, 0xFF35594D, 0xFF144D3A, 0xFF406959, 0xFF204939, 0xFF477060, 0xFF295242, 0xFF325B4B,
        0xFF436C5C, 0xFF2E5747, 0xFF174030, 0xFF547B68, 0xFF587767, 0xFF55675B, 0xFFB3B6AF, 0xFFE3DEDA, 0xFFD1C9C6, 0xFFC7C2BE, 0xFFC1BCB8, 0xFFD2CFCA, 0xFFDAD7D2, 0xFFD3D2CE, 0xFF848681, 0xFF7D8681, 0xFF5D6E66, 0xFF4B655C, 0xFF527267, 0xFF969291,
        0xFF878684, 0xFF565654, 0xFF4D534F, 0xFF57625C, 0xFF798881, 0xFF52665D, 0xFF2B4037, 0xFF5B8474, 0xFF5A8373, 0xFF598272, 0xFF5D8676, 0xFF567F6F, 0xFF588171, 0xFF5C8575, 0xFF406959, 0xFF426859, 0xFF345A4B, 0xFF476D5E, 0xFF5F8576, 0xFF608677,
        0xFF53796A, 0xFF385E4F, 0xFF2B5142, 0xFF193F2A, 0xFF173D28, 0xFF0F3520, 0xFF143A23, 0xFF1F452E, 0xFF193F28, 0xFF11371E, 0xFF153B22, 0xFF163D2A, 0xFF153A28, 0xFF163B29, 0xFF1F4432, 0xFF1E402F, 0xFF0F3120, 0xFF244534, 0xFF264736, 0xFF173E2B,
        0xFF1E4532, 0xFF1F4432, 0xFF1B402E, 0xFF143625, 0xFF234433, 0xFF234433, 0xFF183727, 0xFF1B4230, 0xFF2D5241, 0xFF284D3B, 0xFF1D4230, 0xFF1C3F2B, 0xFF1E412D, 0xFF21422D, 0xFF183924, 0xFF274C3A, 0xFF385D4B, 0xFF466B59, 0xFF5B806E, 0xFF5C816F,
        0xFF5D8270, 0xFF5A7F6D, 0xFF5A7F6D, 0xFF5A816F, 0xFF5B8270, 0xFF5B8270, 0xFF5B8270, 0xFF5B8270, 0xFF5B8270, 0xFF5A816F, 0xFF59806E, 0xFF58826E, 0xFF58826E, 0xFF57816D, 0xFF59806D, 0xFF59806D, 0xFF5A7F6D, 0xFF5B806E, 0xFF5C816F, 0xFF57836C,
        0xFF57836C, 0xFF57836C, 0xFF57836C, 0xFF57836C, 0xFF57836C, 0xFF57836C, 0xFF57836C, 0xFF58826E, 0xFF58826E, 0xFF58826E, 0xFF57816D, 0xFF58826E, 0xFF58826E, 0xFF59836F, 0xFF59836F, 0xFF567F6D, 0xFF567F6D, 0xFF567F6D, 0xFF567F6D, 0xFF57806E,
        0xFF58816F, 0xFF598270, 0xFF598270, 0xFF5D8472, 0xFF5C8371, 0xFF5B8270, 0xFF5A816F, 0xFF5A816F, 0xFF5A816F, 0xFF5B8270, 0xFF5B8270, 0xFF5A816F, 0xFF5B8270, 0xFF5A816F, 0xFF5A816F, 0xFF5B8270, 0xFF5D8472, 0xFF5A816F, 0xFF557C6A, 0xFF3B4F43,
        0xFF757D6E, 0xFF848577, 0xFF5F6A5C, 0xFF41594B, 0xFF304631, 0xFF868265, 0xFFC2A681, 0xFFB09970, 0xFFA09873, 0xFF495638, 0xFF52745B, 0xFF55806D, 0xFF396254, 0xFF1F3D31, 0xFF3F544B, 0xFF37473D, 0xFF2B3B31, 0xFF304036, 0xFF2C3C32, 0xFF2B3B31,
        0xFF2F3F35, 0xFF2C3C32, 0xFF2C3C32, 0xFF202F28, 0xFF36453E, 0xFF273830, 0xFF2F403A, 0xFF2E413B, 0xFF30433F, 0xFF293E39, 0xFF132823, 0xFF1A2924, 0xFF13241E, 0xFF152822, 0xFF0F261C, 0xFF2A463A, 0xFF375748, 0xFF3A5C4C, 0xFF2D5241, 0xFF4C6B59,
        0xFF467066, 0xFF396A66, 0xFF214C45, 0xFF244331, 0xFF293D24, 0xFF304830, 0xFF2C4935, 0xFF214332, 0xFF1D372A, 0xFF506056, 0xFF6B706A, 0xFFB7B8B3, 0xFFDBDCD6, 0xFFB4B9B2, 0xFF91998E, 0xFF9D9C98, 0xFF9A9995, 0xFF979692, 0xFF979692, 0xFF9A9995,
        0xFF9A9995, 0xFF989793, 0xFF969591, 0xFF969893, 0xFF989A95, 0xFF9FA19C, 0xFF959792, 0xFF8C8E89, 0xFF969893, 0xFFB2B4AF, 0xFFD4D6D1, 0xFFE5E2DB, 0xFF8B9B8E, 0xFF5B806E, 0xFF5E8775, 0xFF527064, 0x00CAD6D2, 0x00FDFFFE, 0x00FEFEFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFAFB, 0x00FCFEFD, 0xFFADC2BB, 0xFF3D6C5C, 0xFF5D897A, 0xFF65857A, 0xFF365A4E, 0xFF154E3B, 0xFF466F5F, 0xFF305949, 0xFF456E5E, 0xFF325B4B, 0xFF466F5F,
        0xFF436C5C, 0xFF1C4535, 0xFF174030, 0xFF4C7763, 0xFF4C6E5D, 0xFF4A6053, 0xFFA2ACA3, 0xFFE1E2DC, 0xFFD3D2CD, 0xFFCDCEC8, 0xFFC1C4BD, 0xFFD3CFCC, 0xFFD4D0CD, 0xFFD8D7D3, 0xFFA9ABA8, 0xFFB5BEB9, 0xFF6B7E78, 0xFF415C53, 0xFF4E6E63, 0xFF929493,
        0xFF898D8C, 0xFF646A68, 0xFF5C6562, 0xFF6B7773, 0xFF8B9A95, 0xFF72837D, 0xFF50635D, 0xFF4F7A69, 0xFF5B8675, 0xFF5F8A79, 0xFF5A8574, 0xFF537E6D, 0xFF588372, 0xFF5F8A79, 0xFF588372, 0xFF426859, 0xFF3C6253, 0xFF395F50, 0xFF43695A, 0xFF5D8374,
        0xFF648A7B, 0xFF406657, 0xFF3D6354, 0xFF395E4D, 0xFF436857, 0xFF426755, 0xFF3F6452, 0xFF3D634E, 0xFF3E644F, 0xFF446A53, 0xFF436952, 0xFF486D5B, 0xFF426755, 0xFF3B604E, 0xFF365847, 0xFF3C5E4D, 0xFF3D5E4D, 0xFF355645, 0xFF264736, 0xFF2A4939,
        0xFF2A4939, 0xFF2B4A3A, 0xFF385948, 0xFF3D5F4E, 0xFF3A5C4B, 0xFF355746, 0xFF3A5F4D, 0xFF3D6251, 0xFF426756, 0xFF3D6250, 0xFF416352, 0xFF436652, 0xFF41624F, 0xFF43644F, 0xFF3E5F4A, 0xFF365B49, 0xFF416654, 0xFF4E7361, 0xFF5C816F, 0xFF5C816F,
        0xFF5B806E, 0xFF587D6B, 0xFF597E6C, 0xFF59806E, 0xFF587F6D, 0xFF567D6B, 0xFF557C6A, 0xFF537A68, 0xFF517866, 0xFF507765, 0xFF4F7664, 0xFF4D7761, 0xFF4D7761, 0xFF4E7560, 0xFF4E7560, 0xFF4F7560, 0xFF4F7560, 0xFF507661, 0xFF527561, 0xFF4F775F,
        0xFF4F775F, 0xFF4E765E, 0xFF4E765E, 0xFF4E765E, 0xFF4E765E, 0xFF4F775F, 0xFF4F775F, 0xFF4B7561, 0xFF4A7460, 0xFF4A7460, 0xFF4A7460, 0xFF4B7561, 0xFF4C7662, 0xFF4D7763, 0xFF4E7864, 0xFF4E7765, 0xFF4F7866, 0xFF507967, 0xFF527B69, 0xFF537C6A,
        0xFF547D6B, 0xFF557E6C, 0xFF557E6C, 0xFF59806E, 0xFF59806E, 0xFF59806E, 0xFF59806E, 0xFF59806E, 0xFF59806E, 0xFF59806E, 0xFF5A816F, 0xFF5A816F, 0xFF5B8270, 0xFF5B8270, 0xFF59806E, 0xFF59806E, 0xFF5B8270, 0xFF5A816F, 0xFF577E6C, 0xFF486F5D,
        0xFF91A999, 0xFFAAB7A6, 0xFF788A7E, 0xFF4A665A, 0xFF354D3D, 0xFF888870, 0xFFC2AB8B, 0xFFAE9F7E, 0xFF979678, 0xFF4C5B44, 0xFF567764, 0xFF4B7464, 0xFF2E5246, 0xFF183428, 0xFF394D42, 0xFF32413A, 0xFF0F1E17, 0xFF15241D, 0xFF1A2922, 0xFF15241D,
        0xFF17261F, 0xFF1A2922, 0xFF14231C, 0xFF0F1E17, 0xFF1A2922, 0xFF17261F, 0xFF1E2F29, 0xFF1B2C26, 0xFF1F322E, 0xFF1D302C, 0xFF182B27, 0xFF1A2924, 0xFF2B3A33, 0xFF14281F, 0xFF21382E, 0xFF213D2F, 0xFF426152, 0xFF375948, 0xFF496B5A, 0xFF5D7865,
        0xFF4B736A, 0xFF31625F, 0xFF1F453A, 0xFF455437, 0xFF5B5C34, 0xFF63653E, 0xFF56603E, 0xFF2C452F, 0xFF263828, 0xFF606A5F, 0xFF848580, 0xFFCCC8C5, 0xFFE4E3DE, 0xFFBCBFB4, 0xFF959D8E, 0xFF999A94, 0xFF979892, 0xFF979892, 0xFF989993, 0xFF9A9B95,
        0xFF9C9D97, 0xFF9A9B95, 0xFF999A94, 0xFF9B9E97, 0xFF979A93, 0xFF999C95, 0xFF91948D, 0xFF8C8F88, 0xFF989B94, 0xFFB1B4AD, 0xFFD1D4CD, 0xFFEBE8E1, 0xFF8B9B8E, 0xFF577C6A, 0xFF5D8674, 0xFF547266, 0x00CDD9D5, 0x00FEFFFF, 0x00FEFEFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x00FEFFFF, 0xFFABC0B9, 0xFF3B6A5A, 0xFF5D897A, 0xFF68887D, 0xFF395D51, 0xFF17503D, 0xFF4D7666, 0xFF4F7868, 0xFF3C6555, 0xFF355E4E, 0xFF557E6E,
        0xFF466F5F, 0xFF1E4737, 0xFF244D3D, 0xFF577768, 0xFF4D675A, 0xFF828F86, 0xFFA7A9A4, 0xFFE6E5E1, 0xFFD0CCC9, 0xFFC5C6C1, 0xFFC3C5C0, 0xFFC3C2BE, 0xFFE1E0DC, 0xFFD2D2D0, 0xFF6F7470, 0xFF7D8884, 0xFF536861, 0xFF436157, 0xFF4B6C61, 0xFF415B52,
        0xFF4A645B, 0xFF486259, 0xFF445F56, 0xFF415C53, 0xFF456057, 0xFF4B695F, 0xFF59776D, 0xFF2F5A49, 0xFF4A7564, 0xFF578271, 0xFF578271, 0xFF5E8978, 0xFF588372, 0xFF4E7968, 0xFF5B8675, 0xFF5D8374, 0xFF446A5B, 0xFF3F6556, 0xFF456B5C, 0xFF3F6556,
        0xFF597F70, 0xFF5B8172, 0xFF43695A, 0xFF3D5F51, 0xFF436557, 0xFF3F6153, 0xFF3F6151, 0xFF3D5F4F, 0xFF395B4B, 0xFF3E604F, 0xFF3E604F, 0xFF325745, 0xFF395E4C, 0xFF446655, 0xFF385A49, 0xFF3B5C4B, 0xFF3B5C4B, 0xFF1E3F2E, 0xFF133222, 0xFF172B1F,
        0xFF0B2114, 0xFF193425, 0xFF365545, 0xFF3A5C4B, 0xFF355A48, 0xFF36604C, 0xFF396450, 0xFF3A5F4E, 0xFF395E4D, 0xFF395B4A, 0xFF426453, 0xFF446552, 0xFF3D5E4B, 0xFF3F604B, 0xFF40604B, 0xFF406251, 0xFF476958, 0xFF537564, 0xFF5B7D6C, 0xFF597B6A,
        0xFF557766, 0xFF537564, 0xFF587A69, 0xFF527967, 0xFF4F7664, 0xFF4B7260, 0xFF456C5A, 0xFF406755, 0xFF3B6250, 0xFF39604E, 0xFF385F4D, 0xFF345E46, 0xFF355D45, 0xFF345C44, 0xFF335B43, 0xFF335942, 0xFF325841, 0xFF345741, 0xFF345741, 0xFF34563E,
        0xFF34563E, 0xFF33553D, 0xFF32543C, 0xFF32543C, 0xFF33553D, 0xFF34563E, 0xFF34563E, 0xFF2F5641, 0xFF2F5641, 0xFF305742, 0xFF315843, 0xFF335A45, 0xFF355C47, 0xFF365D48, 0xFF375E49, 0xFF3B6452, 0xFF3D6654, 0xFF416A58, 0xFF456E5C, 0xFF497260,
        0xFF4D7664, 0xFF4F7866, 0xFF507967, 0xFF567B6A, 0xFF567B6A, 0xFF577C6B, 0xFF587D6C, 0xFF597E6D, 0xFF597E6D, 0xFF597E6D, 0xFF597E6D, 0xFF59806E, 0xFF5B8270, 0xFF5B8270, 0xFF587F6D, 0xFF577E6C, 0xFF59806E, 0xFF5A816F, 0xFF59806E, 0xFF265545,
        0xFF2C4B3B, 0xFF3C4E40, 0xFF374E44, 0xFF486D65, 0xFF45695B, 0xFF68765F, 0xFF8E8566, 0xFF959178, 0xFF777C66, 0xFF4D6251, 0xFF557568, 0xFF355B50, 0xFF2B4B40, 0xFF1F372A, 0xFF445447, 0xFF36453E, 0xFF26352E, 0xFF314039, 0xFF202F28, 0xFF2B3A33,
        0xFF303F38, 0xFF202F28, 0xFF2C3B34, 0xFF26352E, 0xFF1A2922, 0xFF32413A, 0xFF2F3E39, 0xFF21322C, 0xFF3C4C49, 0xFF2F423E, 0xFF2D403C, 0xFF303D36, 0xFF283730, 0xFF25362E, 0xFF263C30, 0xFF284235, 0xFF345343, 0xFF4C6D5C, 0xFF5C7E6D, 0xFF5F7865,
        0xFF487067, 0xFF305D5A, 0xFF294536, 0xFF6A6A44, 0xFF917E46, 0xFF98844F, 0xFF827847, 0xFF3C4F33, 0xFF37432D, 0xFF77796C, 0xFFA5A09C, 0xFFE4DCD9, 0xFFE9E4DE, 0xFFB8BAAC, 0xFF8B9280, 0xFF94958D, 0xFF94958D, 0xFF95968E, 0xFF96978F, 0xFF96978F,
        0xFF96978F, 0xFF95968E, 0xFF94958D, 0xFF9A9A92, 0xFF95958D, 0xFF95958D, 0xFF8F8F87, 0xFF8B8B83, 0xFF97978F, 0xFFB1B1A9, 0xFFD0D0C8, 0xFFE8E5DE, 0xFF8A9A8D, 0xFF577C6A, 0xFF5D8674, 0xFF527064, 0x00CBD7D3, 0x00FEFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x00FEFFFF, 0xFFA9BEB7, 0xFF3A6959, 0xFF5E8A7B, 0xFF67877C, 0xFF315549, 0xFF0A4330, 0xFF1D4636, 0xFF234C3C, 0xFF103929, 0xFF1A4333, 0xFF1C4535,
        0xFF1D4636, 0xFF244D3D, 0xFF143D2D, 0xFF5F7569, 0xFF59695F, 0xFFBFC4BE, 0xFFA6A29F, 0xFFF2EAE8, 0xFFEDE5E3, 0xFFDFDBD8, 0xFFD7D8D3, 0xFFE0E1DC, 0xFFF4F5F0, 0xFFCFD1CE, 0xFF666C68, 0xFF84908C, 0xFF667D75, 0xFF5B7B70, 0xFF5C8076, 0xFF4E7D6D,
        0xFF507F6F, 0xFF537F70, 0xFF507C6D, 0xFF578173, 0xFF4C7668, 0xFF3F695B, 0xFF355E50, 0xFF396453, 0xFF376251, 0xFF517C6B, 0xFF5D8877, 0xFF588372, 0xFF588372, 0xFF568170, 0xFF598473, 0xFF5D8374, 0xFF5A8071, 0xFF3E6455, 0xFF3D6354, 0xFF3D6354,
        0xFF4D7364, 0xFF5F8576, 0xFF63897A, 0xFF618279, 0xFF62837A, 0xFF5D7E73, 0xFF618277, 0xFF628476, 0xFF608274, 0xFF648676, 0xFF5C7E6E, 0xFF5F8472, 0xFF5A7C6B, 0xFF5F8170, 0xFF547665, 0xFF587968, 0xFF4F705F, 0xFF1E3D2D, 0xFF113020, 0xFF17241A,
        0xFF0F2115, 0xFF182E21, 0xFF385445, 0xFF557766, 0xFF58826E, 0xFF53806B, 0xFF52826C, 0xFF577C6B, 0xFF5C7E6E, 0xFF577968, 0xFF507261, 0xFF3C5D4A, 0xFF264734, 0xFF254530, 0xFF24442F, 0xFF254736, 0xFF244635, 0xFF2E503F, 0xFF3D5F4E, 0xFF4B6D5C,
        0xFF517362, 0xFF507261, 0xFF517362, 0xFF4B7260, 0xFF466D5B, 0xFF3F6654, 0xFF365D4B, 0xFF2E5543, 0xFF284F3D, 0xFF244B39, 0xFF224937, 0xFF1E462E, 0xFF1C442C, 0xFF1B432B, 0xFF193F28, 0xFF173D26, 0xFF183B25, 0xFF173A24, 0xFF173A24, 0xFF173720,
        0xFF173720, 0xFF16361F, 0xFF15351E, 0xFF15351E, 0xFF16361F, 0xFF173720, 0xFF173720, 0xFF143B26, 0xFF153C27, 0xFF163D28, 0xFF183F2A, 0xFF1B422D, 0xFF1E4530, 0xFF214833, 0xFF224934, 0xFF27503E, 0xFF2A5341, 0xFF315A48, 0xFF396250, 0xFF406957,
        0xFF466F5D, 0xFF4A7361, 0xFF4C7563, 0xFF537867, 0xFF547968, 0xFF567B6A, 0xFF577C6B, 0xFF587D6C, 0xFF587D6C, 0xFF587D6C, 0xFF587D6C, 0xFF587F6D, 0xFF5B8270, 0xFF5B8270, 0xFF577E6C, 0xFF557C6A, 0xFF577E6C, 0xFF5A816F, 0xFF5B8270, 0xFF2D5B4E,
        0xFF39594C, 0xFF5B6F64, 0xFF3C574E, 0xFF3A655C, 0xFF2A584B, 0xFF1F3B24, 0xFF303716, 0xFF6E6F5D, 0xFF455040, 0xFF425A4D, 0xFF517269, 0xFF32554E, 0xFF446258, 0xFF3B4F43, 0xFF5A6859, 0xFF1B2A23, 0xFF3A4942, 0xFF495851, 0xFF192821, 0xFF3D4C45,
        0xFF495851, 0xFF1F2E27, 0xFF44534C, 0xFF3D4A43, 0xFF1D2C25, 0xFF485750, 0xFF31403B, 0xFF182923, 0xFF485855, 0xFF273A36, 0xFF182B27, 0xFF55625B, 0xFF32413A, 0xFF22332B, 0xFF4D6357, 0xFF284235, 0xFF4E6D5D, 0xFF5A7B6A, 0xFF5B7D6C, 0xFF5D7462,
        0xFF4A6F67, 0xFF355F5B, 0xFF334C39, 0xFF7F7749, 0xFFA78848, 0xFFAC8A4B, 0xFF907A41, 0xFF526142, 0xFF404830, 0xFF737164, 0xFF9F9691, 0xFFDCD1CD, 0xFFE0D9D1, 0xFFB9BAAA, 0xFF909881, 0xFF8E9085, 0xFF909287, 0xFF929489, 0xFF94968B, 0xFF94968B,
        0xFF94968B, 0xFF93958A, 0xFF93958A, 0xFF959289, 0xFF918E85, 0xFF949188, 0xFF8D8A81, 0xFF88857C, 0xFF959289, 0xFFB3B0A7, 0xFFD8D5CC, 0xFFE3E0D9, 0xFF8B9B8E, 0xFF5D8270, 0xFF608977, 0xFF4E6C60, 0x00C2CECA, 0x00F9FBFA, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFF5FC, 0x00F8FFFF, 0xFFA7C6BE, 0xFF3B6758, 0xFF648E80, 0xFF5A8071, 0xFF406959, 0xFF275342, 0xFF27503E, 0xFF28513F, 0xFF295240, 0xFF2B5442, 0xFF2A5341,
        0xFF28513F, 0xFF2B5442, 0xFF2A5341, 0xFF577166, 0xFF586F65, 0xFF6E857B, 0xFF869B92, 0xFF9AAFA6, 0xFF8A9F96, 0xFF8CA399, 0xFFA0B7AD, 0xFF93A8A1, 0xFF8EA59D, 0xFF85A097, 0xFF517166, 0xFF45695D, 0xFF4B7466, 0xFF568273, 0xFF5B8778, 0xFF588976,
        0xFF558472, 0xFF568573, 0xFF598473, 0xFF578070, 0xFF5C8273, 0xFF5A7E70, 0xFF4E7264, 0xFF305949, 0xFF396252, 0xFF355E4E, 0xFF527B6B, 0xFF5B8474, 0xFF5D8676, 0xFF5D8676, 0xFF598272, 0xFF578271, 0xFF588372, 0xFF5A8574, 0xFF406959, 0xFF406959,
        0xFF376050, 0xFF486E5F, 0xFF608677, 0xFF5C8371, 0xFF5A816F, 0xFF59806E, 0xFF59806E, 0xFF5B8270, 0xFF5C8371, 0xFF5D8472, 0xFF5C8371, 0xFF5B7F73, 0xFF597F70, 0xFF527B69, 0xFF517B67, 0xFF517865, 0xFF426456, 0xFF112F27, 0xFF0F2824, 0xFF0F201A,
        0xFF182D26, 0xFF1B362D, 0xFF244437, 0xFF527668, 0xFF557A69, 0xFF587A6A, 0xFF5B7D6D, 0xFF588077, 0xFF416259, 0xFF475F52, 0xFF505E4F, 0xFF4F5644, 0xFF494B35, 0xFF3D3D23, 0xFF424226, 0xFF4D3E17, 0xFF372E0F, 0xFF333621, 0xFF253729, 0xFF27453B,
        0xFF284C3E, 0xFF385F4C, 0xFF4F775F, 0xFF497157, 0xFF3F6953, 0xFF315D4C, 0xFF255549, 0xFF1C5043, 0xFF184D39, 0xFF154B31, 0xFF14492B, 0xFF203D27, 0xFF1B3721, 0xFF182F1B, 0xFF182B18, 0xFF1B2918, 0xFF1D2818, 0xFF1C2415, 0xFF1B2113, 0xFF191E1A,
        0xFF191E1A, 0xFF181D19, 0xFF181D19, 0xFF191E1A, 0xFF1A1F1B, 0xFF1B201C, 0xFF1C211D, 0xFF1A2B1B, 0xFF182A1A, 0xFF162D19, 0xFF17331D, 0xFF15371E, 0xFF133A1F, 0xFF164125, 0xFF1D4A2B, 0xFF124B38, 0xFF144D3A, 0xFF17503D, 0xFF225946, 0xFF316453,
        0xFF436D5F, 0xFF4B6F63, 0xFF4F6D63, 0xFF527766, 0xFF547968, 0xFF557A69, 0xFF567B6A, 0xFF567B6A, 0xFF577C6B, 0xFF597E6D, 0xFF5B806F, 0xFF59806E, 0xFF59806E, 0xFF59806E, 0xFF59806E, 0xFF59806E, 0xFF587F6D, 0xFF587F6D, 0xFF587F6D, 0xFF5A7F6E,
        0xFF5C7E6E, 0xFF709282, 0xFF6B8D7D, 0xFF587869, 0xFF29493A, 0xFF153425, 0xFF234233, 0xFF314B3E, 0xFF274738, 0xFF335A48, 0xFF46715E, 0xFF3B604F, 0xFF395346, 0xFF46534A, 0xFF898E88, 0xFF1A2D19, 0xFF516450, 0xFF697C68, 0xFF1C2F1B, 0xFF5D705C,
        0xFF637662, 0xFF182B17, 0xFF637662, 0xFF586D5E, 0xFF1F3123, 0xFF728476, 0xFF4C5E50, 0xFF19291C, 0xFF7E8E81, 0xFF445245, 0xFF223023, 0xFF7C8C81, 0xFF314337, 0xFF283E31, 0xFF6C8778, 0xFF3B5A4A, 0xFF5B7D6C, 0xFF547967, 0xFF577E6B, 0xFF608271,
        0xFF476B5D, 0xFF2B4C3B, 0xFF3B4B30, 0xFF847F55, 0xFF9F8A55, 0xFF9C834B, 0xFF9D8652, 0xFF746C3E, 0xFF333F27, 0xFF455950, 0xFF7A7D72, 0xFFAEA89A, 0xFFE5E5DD, 0xFFBDC0B9, 0xFF949084, 0xFF90938A, 0xFF8D9087, 0xFF8A8D84, 0xFF8C8F86, 0xFF90938A,
        0xFF93968D, 0xFF92958C, 0xFF90938A, 0xFF8F9887, 0xFF8E9786, 0xFF909787, 0xFF878D7F, 0xFF818577, 0xFF909287, 0xFFB3B5AA, 0xFFD3D5CA, 0xFFDFE4DE, 0xFF7E8E84, 0xFF567566, 0xFF5C8575, 0xFF4B7162, 0x00B6CDC5, 0x00FAFEFD, 0x00FFFAFE, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFAFF, 0x00F9FFFF, 0xFFA2C1B9, 0xFF366253, 0xFF618B7D, 0xFF618778, 0xFF5E8777, 0xFF5B8776, 0xFF608977, 0xFF628B79, 0xFF618A78, 0xFF5F8876, 0xFF5E8775,
        0xFF5F8876, 0xFF5E8775, 0xFF598270, 0xFF638779, 0xFF5F8173, 0xFF5D7F71, 0xFF59796C, 0xFF5F8173, 0xFF55796B, 0xFF587E6F, 0xFF618A7A, 0xFF5F7A71, 0xFF537167, 0xFF5B7B70, 0xFF507468, 0xFF51776A, 0xFF4E786A, 0xFF548071, 0xFF5E8A7B, 0xFF558673,
        0xFF558673, 0xFF598876, 0xFF5B8776, 0xFF588372, 0xFF5E8777, 0xFF608979, 0xFF5D8374, 0xFF487161, 0xFF355E4E, 0xFF416A5A, 0xFF2F5848, 0xFF557E6E, 0xFF5E8777, 0xFF567F6F, 0xFF5A8373, 0xFF5A8574, 0xFF598473, 0xFF5F8A79, 0xFF4F7868, 0xFF3D6656,
        0xFF376050, 0xFF486E5F, 0xFF5F8576, 0xFF5D8472, 0xFF5B8270, 0xFF5A816F, 0xFF5A816F, 0xFF5B8270, 0xFF5C8371, 0xFF5C8371, 0xFF5C8371, 0xFF5E8276, 0xFF5B8172, 0xFF547D6B, 0xFF547E6A, 0xFF517865, 0xFF446658, 0xFF224038, 0xFF304945, 0xFF32433D,
        0xFF233831, 0xFF0D281F, 0xFF37574A, 0xFF537769, 0xFF4E7362, 0xFF547666, 0xFF567868, 0xFF537C6A, 0xFF446655, 0xFF394E3B, 0xFF5B634E, 0xFF8D876F, 0xFFA7977D, 0xFFB19E7E, 0xFFB09977, 0xFFAC906B, 0xFF978263, 0xFF7D7560, 0xFF43493D, 0xFF203129,
        0xFF142E23, 0xFF294839, 0xFF456452, 0xFF3C5D48, 0xFF345645, 0xFF294B3D, 0xFF1E4238, 0xFF1A3E34, 0xFF193D2F, 0xFF1B3D2C, 0xFF1B3E28, 0xFF1A3320, 0xFF172E1C, 0xFF132817, 0xFF142515, 0xFF182416, 0xFF1B2316, 0xFF1A2014, 0xFF191C11, 0xFF1A1C19,
        0xFF1A1C19, 0xFF191B18, 0xFF191B18, 0xFF191B18, 0xFF1A1C19, 0xFF1B1D1A, 0xFF1C1E1B, 0xFF1B1E17, 0xFF181E14, 0xFF172116, 0xFF19271A, 0xFF152A19, 0xFF142D1A, 0xFF16331D, 0xFF1C3923, 0xFF1A4637, 0xFF184838, 0xFF1A4F3D, 0xFF215845, 0xFF2A5F4D,
        0xFF366555, 0xFF43695C, 0xFF4C6C61, 0xFF4F7463, 0xFF517665, 0xFF537867, 0xFF537867, 0xFF547968, 0xFF557A69, 0xFF577C6B, 0xFF597E6D, 0xFF59806E, 0xFF59806E, 0xFF59806E, 0xFF59806E, 0xFF59806E, 0xFF59806E, 0xFF587F6D, 0xFF587F6D, 0xFF5A816F,
        0xFF567D6B, 0xFF628776, 0xFF5C8170, 0xFF597B6B, 0xFF416353, 0xFF3A5C4C, 0xFF4A6A5B, 0xFF527162, 0xFF3D5F4F, 0xFF426B59, 0xFF4B7663, 0xFF39604E, 0xFF405C4E, 0xFF6A7B71, 0xFFAEB8B0, 0xFF21362D, 0xFF6B8077, 0xFF81968D, 0xFF0D2219, 0xFF546960,
        0xFF52675E, 0xFF162B22, 0xFF647970, 0xFF546A5D, 0xFF162C1F, 0xFF7E9487, 0xFF3F5347, 0xFF293D31, 0xFFA5B7AB, 0xFF4A5C50, 0xFF2B3D31, 0xFF788A7E, 0xFF23372B, 0xFF4B6154, 0xFFAAC5B6, 0xFF416050, 0xFF597B6A, 0xFF5C816F, 0xFF577E6B, 0xFF4C6E5D,
        0xFF365A4C, 0xFF294A39, 0xFF45553A, 0xFF8B865C, 0xFFA8935E, 0xFFA68D55, 0xFFA08955, 0xFF776F41, 0xFF333F27, 0xFF44584F, 0xFF797C71, 0xFFACA698, 0xFFE5E5DD, 0xFFBFC2BB, 0xFF959185, 0xFF919187, 0xFF929288, 0xFF929288, 0xFF939389, 0xFF94948A,
        0xFF95958B, 0xFF95958B, 0xFF96968C, 0xFF909485, 0xFF8C9081, 0xFF8B8F80, 0xFF888A7D, 0xFF838578, 0xFF909086, 0xFFB5B5AB, 0xFFD8D8CE, 0xFFE0E7E0, 0xFF88998F, 0xFF5E7E6F, 0xFF5C8575, 0xFF486C5E, 0x00B5CAC3, 0x00FCFFFF, 0x00FFFCFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFF9FF, 0x00F7FFFF, 0xFFA3C2BA, 0xFF3A6657, 0xFF5F897B, 0xFF5C8273, 0xFF598272, 0xFF5D8978, 0xFF557E6C, 0xFF598270, 0xFF5A8371, 0xFF58816F, 0xFF5B8472,
        0xFF608977, 0xFF5F8876, 0xFF5C8573, 0xFF5D8978, 0xFF5F8A79, 0xFF5D8676, 0xFF588171, 0xFF628D7C, 0xFF61907E, 0xFF609380, 0xFF5B927D, 0xFF6C9084, 0xFF62887B, 0xFF668C7F, 0xFF638C7E, 0xFF5E887A, 0xFF5F897B, 0xFF5D897A, 0xFF598576, 0xFF588976,
        0xFF578875, 0xFF588775, 0xFF558472, 0xFF537F6E, 0xFF55806F, 0xFF588372, 0xFF55806F, 0xFF5D8676, 0xFF456E5E, 0xFF325B4B, 0xFF406959, 0xFF2D5646, 0xFF557E6E, 0xFF658E7E, 0xFF567F6F, 0xFF5A8574, 0xFF588372, 0xFF608B7A, 0xFF5C8575, 0xFF386151,
        0xFF376050, 0xFF486E5F, 0xFF618778, 0xFF5E8573, 0xFF5C8371, 0xFF5B8270, 0xFF5B8270, 0xFF5C8371, 0xFF5C8371, 0xFF5C8371, 0xFF5B8270, 0xFF5C8074, 0xFF587E6F, 0xFF557E6C, 0xFF56806C, 0xFF517865, 0xFF426456, 0xFF27453D, 0xFF405955, 0xFF5A6B65,
        0xFF1A2F28, 0xFF233E35, 0xFF103023, 0xFF416557, 0xFF5C8170, 0xFF5B7D6D, 0xFF577969, 0xFF57836C, 0xFF4B6D55, 0xFF35472F, 0xFF5D5D43, 0xFF9B8C6F, 0xFFAD9070, 0xFFB08C68, 0xFFB28B64, 0xFFB38F6B, 0xFF9F8264, 0xFF877561, 0xFF5F594D, 0xFF535851,
        0xFF495851, 0xFF3B5145, 0xFF2A4235, 0xFF244532, 0xFF1D3C2D, 0xFF173327, 0xFF162D25, 0xFF1A2D27, 0xFF1D2E26, 0xFF1C2C22, 0xFF1C2A1D, 0xFF16271D, 0xFF13241A, 0xFF112117, 0xFF131E16, 0xFF141E16, 0xFF171C16, 0xFF181A15, 0xFF171813, 0xFF1A1A18,
        0xFF1A1A18, 0xFF191917, 0xFF191917, 0xFF191917, 0xFF1A1A18, 0xFF1B1B19, 0xFF1C1C1A, 0xFF1D1415, 0xFF1B1515, 0xFF1A1915, 0xFF1B1D18, 0xFF1A211A, 0xFF172319, 0xFF18251B, 0xFF19291E, 0xFF163027, 0xFF143429, 0xFF144031, 0xFF184D3B, 0xFF1E5542,
        0xFF265948, 0xFF386254, 0xFF486C60, 0xFF4E7362, 0xFF507564, 0xFF527766, 0xFF537867, 0xFF547968, 0xFF557A69, 0xFF577C6B, 0xFF587D6C, 0xFF59806E, 0xFF59806E, 0xFF59806E, 0xFF59806E, 0xFF59806E, 0xFF59806E, 0xFF59806E, 0xFF59806E, 0xFF598471,
        0xFF557E6C, 0xFF598270, 0xFF537C6A, 0xFF5F8674, 0xFF5A816F, 0xFF5E8372, 0xFF688D7C, 0xFF638575, 0xFF5A7F6E, 0xFF5B8673, 0xFF4C7965, 0xFF365F4D, 0xFF3B5B4C, 0xFF8BA396, 0xFFE3F4EA, 0xFF253F3E, 0xFFA4BEBD, 0xFFC3DDDC, 0xFF2A4443, 0xFF597372,
        0xFF496362, 0xFF1A3433, 0xFF4E6867, 0xFF395549, 0xFF324E42, 0xFF7E9A8E, 0xFF274136, 0xFF2C463B, 0xFF889F95, 0xFF3E554B, 0xFF374E44, 0xFF4B5F53, 0xFF1F3528, 0xFF425A4C, 0xFF809C8D, 0xFF2D4C3C, 0xFF5E806F, 0xFF5B806E, 0xFF547967, 0xFF416352,
        0xFF2F5345, 0xFF2A4B3A, 0xFF46563B, 0xFF868157, 0xFFA7925D, 0xFFAC935B, 0xFFA08955, 0xFF7A7244, 0xFF333F27, 0xFF43574E, 0xFF787B70, 0xFFAAA496, 0xFFE5E5DD, 0xFFC1C4BD, 0xFF969286, 0xFF8D897D, 0xFF8F8B7F, 0xFF928E82, 0xFF979387, 0xFF9B978B,
        0xFF999589, 0xFF949084, 0xFF908C80, 0xFF8D8E80, 0xFF8D8E80, 0xFF908E81, 0xFF8C8A7E, 0xFF848276, 0xFF8E8B82, 0xFFB6B3AA, 0xFFDCD9D0, 0xFFB6C1B9, 0xFF788C81, 0xFF5F7F70, 0xFF608677, 0xFF4B6D5F, 0x00B5CAC3, 0x00FCFFFF, 0x00FFFCFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFF9FF, 0x00F9FFFF, 0xFFA3C2BA, 0xFF3B6758, 0xFF5F897B, 0xFF5F8576, 0xFF5A8373, 0xFF5C8877, 0xFF618A78, 0xFF628B79, 0xFF648D7B, 0xFF5F8876, 0xFF618A78,
        0xFF648D7B, 0xFF618A78, 0xFF668F7D, 0xFF5E8B77, 0xFF608977, 0xFF618675, 0xFF668677, 0xFF658777, 0xFF57806E, 0xFF548570, 0xFF538972, 0xFF598272, 0xFF578070, 0xFF5A8373, 0xFF5E8777, 0xFF598473, 0xFF5B8675, 0xFF5F8A79, 0xFF578271, 0xFF568573,
        0xFF558472, 0xFF558472, 0xFF578372, 0xFF578372, 0xFF588473, 0xFF588473, 0xFF568271, 0xFF588171, 0xFF618A7A, 0xFF436C5C, 0xFF325B4B, 0xFF406959, 0xFF3B6454, 0xFF567F6F, 0xFF628B7B, 0xFF588372, 0xFF578271, 0xFF598473, 0xFF5E8777, 0xFF355E4E,
        0xFF396252, 0xFF456B5C, 0xFF658B7C, 0xFF5E8573, 0xFF5D8472, 0xFF5C8371, 0xFF5C8371, 0xFF5C8371, 0xFF5D8472, 0xFF5C8371, 0xFF5A816F, 0xFF5A7E72, 0xFF587E6F, 0xFF537C6A, 0xFF557F6B, 0xFF527966, 0xFF456759, 0xFF26443C, 0xFF3A534F, 0xFF20312B,
        0xFF273C35, 0xFF223D34, 0xFF29493C, 0xFF284C3E, 0xFF4D7261, 0xFF587A6A, 0xFF537565, 0xFF5A8371, 0xFF4A6B5A, 0xFF334633, 0xFF41442F, 0xFF887E63, 0xFFB09D7D, 0xFFAB916E, 0xFFBCA07B, 0xFFAA906B, 0xFF998465, 0xFF7C6D56, 0xFF4B4839, 0xFF3B4338,
        0xFF36473D, 0xFF365045, 0xFF2B493D, 0xFF17402E, 0xFF113527, 0xFF0E2C20, 0xFF162B24, 0xFF222E2A, 0xFF252E29, 0xFF202622, 0xFF181D17, 0xFF182420, 0xFF16221E, 0xFF141F1B, 0xFF131C19, 0xFF151B19, 0xFF181A19, 0xFF181A19, 0xFF191919, 0xFF1B1B19,
        0xFF1B1B19, 0xFF1A1A18, 0xFF1A1A18, 0xFF1A1A18, 0xFF1B1B19, 0xFF1B1B19, 0xFF1B1B19, 0xFF1E181C, 0xFF1D181C, 0xFF1D1B1E, 0xFF1F1F1F, 0xFF1E2221, 0xFF1B241F, 0xFF19241E, 0xFF19241E, 0xFF1B211F, 0xFF14251F, 0xFF113227, 0xFF144434, 0xFF184F3C,
        0xFF1E5341, 0xFF2F5E4E, 0xFF416A5C, 0xFF4F7463, 0xFF517665, 0xFF547968, 0xFF557A69, 0xFF567B6A, 0xFF577C6B, 0xFF597E6D, 0xFF5A7F6E, 0xFF587F6D, 0xFF587F6D, 0xFF587F6D, 0xFF59806E, 0xFF59806E, 0xFF59806E, 0xFF59806E, 0xFF59806E, 0xFF578470,
        0xFF56836F, 0xFF588370, 0xFF4F7A67, 0xFF598270, 0xFF567F6D, 0xFF567F6D, 0xFF577E6C, 0xFF5C8170, 0xFF638A78, 0xFF598471, 0xFF406B58, 0xFF3D6654, 0xFF305242, 0xFF668274, 0xFFB0C8BB, 0xFF203E3C, 0xFF678583, 0xFF668482, 0xFF213F3D, 0xFF678583,
        0xFF577573, 0xFF274543, 0xFF2F4D4B, 0xFF25463B, 0xFF3C5D52, 0xFFABCBC0, 0xFF3E5E53, 0xFF234137, 0xFF4E6C62, 0xFF19342B, 0xFF445F56, 0xFF829A8C, 0xFF284032, 0xFF2B4637, 0xFF547061, 0xFF2C4B3B, 0xFF5B7C6B, 0xFF577968, 0xFF3C614F, 0xFF385A49,
        0xFF35594B, 0xFF335443, 0xFF425237, 0xFF7E794F, 0xFFA38E59, 0xFFAA9159, 0xFF9F8854, 0xFF7B7345, 0xFF313D25, 0xFF43574E, 0xFF797C71, 0xFFA8A294, 0xFFE6E6DE, 0xFFC4C7C0, 0xFF969286, 0xFF878070, 0xFF878070, 0xFF8B8474, 0xFF968F7F, 0xFFA09989,
        0xFF9F9888, 0xFF928B7B, 0xFF857E6E, 0xFF898377, 0xFF938D81, 0xFF969286, 0xFF8F8B80, 0xFF848075, 0xFF918D84, 0xFFBCB9B0, 0xFFDBD8CF, 0xFF829389, 0xFF637B6E, 0xFF5D7D6E, 0xFF618778, 0xFF4D6D60, 0x00B6C9C3, 0x00FCFFFF, 0x00FFFAFC, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFAFF, 0x00F9FFFF, 0xFFA4C3BB, 0xFF3C6859, 0xFF618B7D, 0xFF63897A, 0xFF517A6A, 0xFF457160, 0xFF4A7361, 0xFF48715F, 0xFF497260, 0xFF456E5C, 0xFF48715F,
        0xFF48715F, 0xFF436C5A, 0xFF4F7866, 0xFF5E8372, 0xFF537263, 0xFF51675B, 0xFF66766C, 0xFF57675D, 0xFF2E4438, 0xFF203F30, 0xFF254A39, 0xFF1C3C2F, 0xFF1B3D2F, 0xFF26483A, 0xFF35594B, 0xFF3E6254, 0xFF3A6051, 0xFF547D6D, 0xFF5D8676, 0xFF588171,
        0xFF5A8373, 0xFF5C8575, 0xFF5C8776, 0xFF5D8877, 0xFF5B8675, 0xFF5A8574, 0xFF5B8675, 0xFF5C8575, 0xFF5B8474, 0xFF5F8878, 0xFF3F6858, 0xFF3F6858, 0xFF3A6353, 0xFF396252, 0xFF5B8474, 0xFF598473, 0xFF5C8776, 0xFF568170, 0xFF5B8474, 0xFF376050,
        0xFF3D6656, 0xFF43695A, 0xFF678D7E, 0xFF5F8674, 0xFF5D8472, 0xFF5C8371, 0xFF5D8472, 0xFF5D8472, 0xFF5D8472, 0xFF5C8371, 0xFF5B8270, 0xFF5E8276, 0xFF5B8172, 0xFF547D6B, 0xFF547E6A, 0xFF567D6A, 0xFF4D6F61, 0xFF25433B, 0xFF2A433F, 0xFF34453F,
        0xFF334841, 0xFF334E45, 0xFF224235, 0xFF35594B, 0xFF234837, 0xFF547666, 0xFF5D7F6F, 0xFF537871, 0xFF4B6A62, 0xFF51685E, 0xFF4D5B4E, 0xFF929B88, 0xFFBFC4AD, 0xFF83876C, 0xFF7C8163, 0xFF838361, 0xFF838465, 0xFF6E745A, 0xFF384531, 0xFF172C1D,
        0xFF0C2B1C, 0xFF264C3D, 0xFF366052, 0xFF19503D, 0xFF174737, 0xFF183E2F, 0xFF1D372C, 0xFF23342C, 0xFF242F29, 0xFF1E2925, 0xFF18231F, 0xFF1D2726, 0xFF1B2524, 0xFF1A2322, 0xFF191F1F, 0xFF171D1D, 0xFF181C1D, 0xFF1A1E1F, 0xFF1E1F21, 0xFF1B1D1A,
        0xFF1C1E1B, 0xFF1C1E1B, 0xFF1D1F1C, 0xFF1D1F1C, 0xFF1C1E1B, 0xFF1B1D1A, 0xFF1B1D1A, 0xFF1B1F22, 0xFF1B1F22, 0xFF1C2124, 0xFF1C2524, 0xFF1D2927, 0xFF1C2B26, 0xFF192A24, 0xFF162721, 0xFF272526, 0xFF1D2623, 0xFF193028, 0xFF1A4033, 0xFF1B4B3B,
        0xFF1E5140, 0xFF2B5B4B, 0xFF386757, 0xFF4E7362, 0xFF507564, 0xFF537867, 0xFF557A69, 0xFF567B6A, 0xFF577C6B, 0xFF587D6C, 0xFF5A7F6E, 0xFF587F6D, 0xFF587F6D, 0xFF587F6D, 0xFF59806E, 0xFF59806E, 0xFF59806E, 0xFF5A816F, 0xFF5A816F, 0xFF507D69,
        0xFF55826E, 0xFF57826F, 0xFF507B68, 0xFF5A8371, 0xFF567F6D, 0xFF598270, 0xFF587F6D, 0xFF5C8170, 0xFF557C6A, 0xFF416A58, 0xFF335C4A, 0xFF466D5B, 0xFF2E5342, 0xFF2F4F40, 0xFF416051, 0xFF416557, 0xFF365A4C, 0xFF14382A, 0xFF305446, 0xFF54786A,
        0xFF325648, 0xFF284C3E, 0xFF426658, 0xFF34584C, 0xFF395D51, 0xFF62867A, 0xFF4A6B60, 0xFF44655A, 0xFF3C5C51, 0xFF416156, 0xFF27473C, 0xFF264132, 0xFF3B5748, 0xFF587465, 0xFF456454, 0xFF4B6C5B, 0xFF557665, 0xFF3F6150, 0xFF3B5D4C, 0xFF325443,
        0xFF416557, 0xFF3E5F4E, 0xFF435338, 0xFF807B51, 0xFFA5905B, 0xFFA88F57, 0xFF9E8753, 0xFF797143, 0xFF2D3921, 0xFF43574E, 0xFF7B7E73, 0xFFA7A193, 0xFFE5E5DD, 0xFFC6C9C2, 0xFF959185, 0xFF898272, 0xFF888171, 0xFF8B8474, 0xFF968F7F, 0xFFA09989,
        0xFFA09989, 0xFF958E7E, 0xFF8A8373, 0xFF888075, 0xFF958D82, 0xFF999387, 0xFF8F8B80, 0xFF858176, 0xFF9C9990, 0xFFC5C5BB, 0xFFCECEC4, 0xFF6F877A, 0xFF617D6F, 0xFF5F8171, 0xFF5F8375, 0xFF4B695D, 0x00B6C7C1, 0x00FCFFFF, 0x00FFFEFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFF5FC, 0x00F9FFFF, 0xFFA3C2BA, 0xFF416D5E, 0xFF648E80, 0xFF5E8475, 0xFF355E4E, 0xFF164231, 0xFF1A4331, 0xFF163F2D, 0xFF17402E, 0xFF123B29, 0xFF194230,
        0xFF1C4533, 0xFF103927, 0xFF1A4331, 0xFF59796A, 0xFF3C5447, 0xFF414C44, 0xFF8F8E8A, 0xFFADA9A6, 0xFF959691, 0xFF879189, 0xFF89998F, 0xFF87978D, 0xFF87988E, 0xFF72867B, 0xFF486053, 0xFF405C4E, 0xFF325243, 0xFF5A7F6E, 0xFF5F8473, 0xFF628476,
        0xFF66887A, 0xFF67897B, 0xFF64887A, 0xFF628879, 0xFF5E8475, 0xFF5D8676, 0xFF638C7C, 0xFF618A7A, 0xFF547D6D, 0xFF5A8373, 0xFF668F7F, 0xFF365F4F, 0xFF3D6656, 0xFF3C6555, 0xFF3B6454, 0xFF578271, 0xFF5C8776, 0xFF598473, 0xFF5A8373, 0xFF3A6353,
        0xFF3F6858, 0xFF446A5B, 0xFF668C7D, 0xFF5E8573, 0xFF5D8472, 0xFF5D8472, 0xFF5D8472, 0xFF5E8573, 0xFF5E8573, 0xFF5D8472, 0xFF5B8270, 0xFF5C8074, 0xFF5D8374, 0xFF598270, 0xFF5A8470, 0xFF5B826F, 0xFF4B6D5F, 0xFF16342C, 0xFF112A26, 0xFF1C2D27,
        0xFF122720, 0xFF0C271E, 0xFF234336, 0xFF315547, 0xFF385D4C, 0xFF305242, 0xFF486A5A, 0xFF5F847D, 0xFF4D6D68, 0xFF537169, 0xFF49635A, 0xFF6A8477, 0xFF6F8B7C, 0xFF24442F, 0xFF21422D, 0xFF466045, 0xFF496348, 0xFF415A44, 0xFF28442E, 0xFF1F3E2C,
        0xFF1C4331, 0xFF335F4E, 0xFF3A6B58, 0xFF1B5848, 0xFF1E5343, 0xFF1E4737, 0xFF1D372A, 0xFF18291F, 0xFF16231C, 0xFF152421, 0xFF182827, 0xFF1E2827, 0xFF1E2827, 0xFF1D2726, 0xFF1A2423, 0xFF182120, 0xFF182120, 0xFF1B2423, 0xFF1E2726, 0xFF191F1B,
        0xFF1A201C, 0xFF1D231F, 0xFF1E2420, 0xFF1E2420, 0xFF1C221E, 0xFF191F1B, 0xFF181E1A, 0xFF162021, 0xFF162021, 0xFF162220, 0xFF172623, 0xFF1A2B25, 0xFF1D2E28, 0xFF182C23, 0xFF13271E, 0xFF232122, 0xFF1D2120, 0xFF182722, 0xFF19332A, 0xFF1A3E32,
        0xFF1D493A, 0xFF275747, 0xFF316453, 0xFF4A6F5E, 0xFF4D7261, 0xFF507564, 0xFF537867, 0xFF547968, 0xFF547968, 0xFF567B6A, 0xFF577C6B, 0xFF577E6C, 0xFF587F6D, 0xFF587F6D, 0xFF587F6D, 0xFF59806E, 0xFF5A816F, 0xFF5A816F, 0xFF5A816F, 0xFF537E6B,
        0xFF58816F, 0xFF598270, 0xFF557E6C, 0xFF608775, 0xFF587F6D, 0xFF608574, 0xFF618675, 0xFF608272, 0xFF517363, 0xFF4B6D5D, 0xFF436555, 0xFF446656, 0xFF406051, 0xFF416152, 0xFF436354, 0xFF4C725D, 0xFF527863, 0xFF3D634E, 0xFF507661, 0xFF2F5540,
        0xFF1B412C, 0xFF375D48, 0xFF4A705B, 0xFF24483A, 0xFF2D5143, 0xFF4B6F61, 0xFF3C5E50, 0xFF547668, 0xFF49695C, 0xFF47675A, 0xFF537366, 0xFF537262, 0xFF365545, 0xFF577666, 0xFF5E7F6E, 0xFF587968, 0xFF436453, 0xFF406150, 0xFF375847, 0xFF426453,
        0xFF4E7264, 0xFF416251, 0xFF3F4F34, 0xFF7D784E, 0xFFA28D58, 0xFFA68D55, 0xFFA08955, 0xFF776F41, 0xFF29351D, 0xFF43574E, 0xFF7C7F74, 0xFFA49E90, 0xFFE4E4DC, 0xFFC9CCC5, 0xFF979387, 0xFF938D7D, 0xFF948E7E, 0xFF969080, 0xFF999383, 0xFF9C9686,
        0xFF9E9888, 0xFF9E9888, 0xFF9E9888, 0xFF999186, 0xFF9C968A, 0xFF999589, 0xFF908E82, 0xFF838377, 0xFFA0A297, 0xFFC5C8BD, 0xFFB2B5AA, 0xFF607F70, 0xFF5F7F70, 0xFF608272, 0xFF608274, 0xFF4D695D, 0x00B6C7C1, 0x00FCFFFF, 0x00FFFEFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFF9FF, 0x00F8FFFF, 0xFF9CBBB3, 0xFF406C5D, 0xFF638D7F, 0xFF5E8475, 0xFF325B4B, 0xFF164231, 0xFF3C6553, 0xFF38614F, 0xFF325B49, 0xFF254E3C, 0xFF325B49,
        0xFF396250, 0xFF1F4836, 0xFF194230, 0xFF58826E, 0xFF355142, 0xFF3F4B41, 0xFFADA8A4, 0xFFEBE0DE, 0xFFE8DEDC, 0xFFDEDBD6, 0xFFDCDFD8, 0xFFE5E1DE, 0xFFEDEEE9, 0xFFC1C6C0, 0xFF5E6961, 0xFF485C51, 0xFF2F493C, 0xFF59796A, 0xFF496B5B, 0xFF4E685D,
        0xFF496358, 0xFF405C50, 0xFF3E5C50, 0xFF466659, 0xFF46685A, 0xFF456759, 0xFF486C5E, 0xFF578070, 0xFF628B7B, 0xFF5B8474, 0xFF557E6E, 0xFF628B7B, 0xFF3C6555, 0xFF436C5C, 0xFF315A4A, 0xFF477261, 0xFF4F7A69, 0xFF5C8776, 0xFF5B8474, 0xFF3A6353,
        0xFF3C6555, 0xFF496F60, 0xFF618778, 0xFF5D8472, 0xFF5D8472, 0xFF5D8472, 0xFF5E8573, 0xFF5F8674, 0xFF5F8674, 0xFF5E8573, 0xFF5C8371, 0xFF5A7E72, 0xFF5C8273, 0xFF5B8472, 0xFF5D8773, 0xFF59806D, 0xFF446658, 0xFF133129, 0xFF152E2A, 0xFF162721,
        0xFF172C25, 0xFF142F26, 0xFF2A4A3D, 0xFF4C7062, 0xFF2E5342, 0xFF3E6050, 0xFF365848, 0xFF507967, 0xFF547A6B, 0xFF587A6C, 0xFF618174, 0xFF668679, 0xFF4D6F61, 0xFF305647, 0xFF376050, 0xFF325D4C, 0xFF496F60, 0xFF5A7C6C, 0xFF587767, 0xFF587968,
        0xFF4A6F5D, 0xFF47725F, 0xFF356450, 0xFF1C5245, 0xFF1C4A3D, 0xFF1C3C2F, 0xFF192B1F, 0xFF142016, 0xFF131C17, 0xFF15211F, 0xFF172526, 0xFF19261F, 0xFF1B2821, 0xFF1D2A23, 0xFF1C2922, 0xFF19261F, 0xFF18251E, 0xFF1A2720, 0xFF1D2A23, 0xFF14211A,
        0xFF17241D, 0xFF1B2821, 0xFF1D2A23, 0xFF1D2A23, 0xFF19261F, 0xFF15221B, 0xFF121F18, 0xFF131C1B, 0xFF141D1A, 0xFF141D1A, 0xFF16211D, 0xFF1C2721, 0xFF202D26, 0xFF1D2A21, 0xFF16231A, 0xFF171B1A, 0xFF171B1A, 0xFF161F1C, 0xFF13221D, 0xFF132A22,
        0xFF173B2F, 0xFF225242, 0xFF2D6451, 0xFF496E5D, 0xFF4C7160, 0xFF507564, 0xFF527766, 0xFF547968, 0xFF547968, 0xFF567B6A, 0xFF577C6B, 0xFF577E6C, 0xFF577E6C, 0xFF587F6D, 0xFF587F6D, 0xFF59806E, 0xFF5A816F, 0xFF5A816F, 0xFF5A816F, 0xFF5D8472,
        0xFF5D8472, 0xFF5A7F6E, 0xFF587D6C, 0xFF5B7D6D, 0xFF426454, 0xFF3E6050, 0xFF3B5B4C, 0xFF365647, 0xFF395849, 0xFF456455, 0xFF3F5B4D, 0xFF2B4739, 0xFF304F40, 0xFF355445, 0xFF426253, 0xFF254736, 0xFF3A5C4B, 0xFF3D5F4E, 0xFF345645, 0xFF244635,
        0xFF335544, 0xFF436554, 0xFF254736, 0xFF214634, 0xFF456756, 0xFF365847, 0xFF244635, 0xFF496A59, 0xFF2B4C3B, 0xFF1F3E2E, 0xFF1E3D2D, 0xFF3C5D4C, 0xFF4D6E5D, 0xFF557665, 0xFF547564, 0xFF486958, 0xFF446554, 0xFF406150, 0xFF456655, 0xFF547665,
        0xFF4A6E60, 0xFF355645, 0xFF39492E, 0xFF777248, 0xFF9E8954, 0xFFA58C54, 0xFFA08955, 0xFF756D3F, 0xFF27331B, 0xFF42564D, 0xFF7B7E73, 0xFFA09A8C, 0xFFE2E2DA, 0xFFCCCFC8, 0xFF9B978B, 0xFF9A968A, 0xFF9B978B, 0xFF9C988C, 0xFF9B978B, 0xFF999589,
        0xFF9B978B, 0xFF9F9B8F, 0xFFA4A094, 0xFFA6A094, 0xFF9F9B8F, 0xFF9C9A8D, 0xFF949488, 0xFF7D8173, 0xFF979D91, 0xFFBEC6B9, 0xFF9FAA9C, 0xFF557767, 0xFF5B806F, 0xFF5D8271, 0xFF618375, 0xFF516B60, 0x00B8C7C2, 0x00FCFFFF, 0x00FEFCFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFAFF, 0x00F5FEFD, 0xFF97B6AE, 0xFF3F6B5C, 0xFF638D7F, 0xFF5D8374, 0xFF335C4C, 0xFF1D4938, 0xFF4B7462, 0xFF48715F, 0xFF406957, 0xFF305947, 0xFF436C5A,
        0xFF517A68, 0xFF2F5846, 0xFF1B4432, 0xFF4C7D67, 0xFF3A5F4D, 0xFF546459, 0xFFC1BEB9, 0xFFF0E2E1, 0xFFDED0CF, 0xFFC9C4C0, 0xFFC2C1BC, 0xFFDACACB, 0xFFE8DCDC, 0xFFC8C3C0, 0xFF676964, 0xFF57645B, 0xFF22382C, 0xFF405C4E, 0xFF284839, 0xFF556A61,
        0xFF42574E, 0xFF294036, 0xFF264035, 0xFF365246, 0xFF38564A, 0xFF2F4F42, 0xFF2B4B3E, 0xFF598272, 0xFF5A8373, 0xFF5F8878, 0xFF5F8878, 0xFF5A8373, 0xFF5F8878, 0xFF416A5A, 0xFF396252, 0xFF366150, 0xFF3E6958, 0xFF5C8776, 0xFF5C8575, 0xFF376050,
        0xFF386151, 0xFF4E7465, 0xFF5D8374, 0xFF5D8472, 0xFF5C8371, 0xFF5D8472, 0xFF5E8573, 0xFF608775, 0xFF608775, 0xFF5F8674, 0xFF5D8472, 0xFF5F8377, 0xFF5B8172, 0xFF57806E, 0xFF57816D, 0xFF527966, 0xFF426456, 0xFF25433B, 0xFF3B5450, 0xFF42534D,
        0xFF1C312A, 0xFF243F36, 0xFF39594C, 0xFF537769, 0xFF4D7261, 0xFF335545, 0xFF3E6050, 0xFF295639, 0xFF568166, 0xFF51745E, 0xFF587765, 0xFF4E6D5D, 0xFF365546, 0xFF436557, 0xFF2F5347, 0xFF1F4E46, 0xFF3E6960, 0xFF567A70, 0xFF59776B, 0xFF5D796A,
        0xFF517060, 0xFF4D6F5E, 0xFF345B49, 0xFF1E4B44, 0xFF1B3E37, 0xFF192E25, 0xFF1B251C, 0xFF20231C, 0xFF222421, 0xFF1F2427, 0xFF1A2529, 0xFF112019, 0xFF15241D, 0xFF1A2922, 0xFF1B2A23, 0xFF182720, 0xFF15261E, 0xFF172820, 0xFF192A22, 0xFF12211A,
        0xFF15241D, 0xFF192821, 0xFF1C2B24, 0xFF1C2B24, 0xFF182720, 0xFF13221B, 0xFF0F1E17, 0xFF161817, 0xFF171918, 0xFF171916, 0xFF191B18, 0xFF1F241E, 0xFF242923, 0xFF222721, 0xFF1B2019, 0xFF16211D, 0xFF1C2220, 0xFF1C201F, 0xFF161C1A, 0xFF0E1F19,
        0xFF113126, 0xFF1E4D3D, 0xFF28614E, 0xFF4B705F, 0xFF4E7362, 0xFF527766, 0xFF557A69, 0xFF567B6A, 0xFF577C6B, 0xFF587D6C, 0xFF597E6D, 0xFF577E6C, 0xFF577E6C, 0xFF587F6D, 0xFF587F6D, 0xFF59806E, 0xFF5A816F, 0xFF5A816F, 0xFF5B8270, 0xFF5A7F6E,
        0xFF5B7D6D, 0xFF5A7C6C, 0xFF5D7F6F, 0xFF5C7C6D, 0xFF355546, 0xFF264536, 0xFF1B3A2B, 0xFF2B4A3B, 0xFF3D594B, 0xFF4C6659, 0xFF4E6659, 0xFF3F574A, 0xFF385245, 0xFF0F2E1F, 0xFF0F2F20, 0xFF2B4C43, 0xFF2C4D44, 0xFF46675E, 0xFF37584F, 0xFF2D4E45,
        0xFF1F4037, 0xFF27483F, 0xFF22433A, 0xFF345743, 0xFF3A5D49, 0xFF4C6D5A, 0xFF264734, 0xFF2B4A38, 0xFF365543, 0xFF2B4A38, 0xFF23402E, 0xFF2D4F3E, 0xFF446655, 0xFF456655, 0xFF5A7B6A, 0xFF375847, 0xFF456655, 0xFF486958, 0xFF557665, 0xFF507261,
        0xFF365A4C, 0xFF254635, 0xFF39492E, 0xFF7A754B, 0xFF9F8A55, 0xFFA78E56, 0xFF9D8652, 0xFF756D3F, 0xFF253119, 0xFF42564D, 0xFF7A7D72, 0xFF9D9789, 0xFFE0E0D8, 0xFFCDD0C9, 0xFF9E9A8E, 0xFF9B998D, 0xFF9B998D, 0xFF9A988C, 0xFF99978B, 0xFF99978B,
        0xFF9A988C, 0xFF9A988C, 0xFF9B998D, 0xFFA29E92, 0xFF9A988B, 0xFF9C9D8F, 0xFF97998C, 0xFF777D6F, 0xFF8D988A, 0xFFBDC9BB, 0xFFA3AFA1, 0xFF5A7F6E, 0xFF618675, 0xFF608574, 0xFF638376, 0xFF536D62, 0x00B8C7C2, 0x00FCFFFF, 0x00FEFCFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x04F3FEFA, 0xFF9EBCB2, 0xFF40695B, 0xFF62887B, 0xFF5D8175, 0xFF315B4D, 0xFF205040, 0xFF446D5B, 0xFF1C4533, 0xFF466F5D, 0xFF2D5644, 0xFF1E4735,
        0xFF517A68, 0xFF37604E, 0xFF194230, 0xFF59856E, 0xFF4B6E5A, 0xFF627466, 0xFFC6C9C0, 0xFFE3E0D9, 0xFFCEC9C3, 0xFFC3C4BC, 0xFFB8BEB4, 0xFFD5D0CC, 0xFFDEDBD6, 0xFFC8CBC4, 0xFF808A81, 0xFF637569, 0xFF597163, 0xFF3E5D4D, 0xFF628372, 0xFF9AA3A2,
        0xFF6F7877, 0xFF586160, 0xFF5B6463, 0xFF626B6A, 0xFF8F9897, 0xFF7E8786, 0xFF333C3B, 0xFF588372, 0xFF5C8776, 0xFF5A8574, 0xFF588372, 0xFF55806F, 0xFF608B7A, 0xFF618C7B, 0xFF3A6554, 0xFF39604E, 0xFF375E4C, 0xFF466D5B, 0xFF59806E, 0xFF3C6351,
        0xFF406755, 0xFF476E5C, 0xFF638A78, 0xFF5C8774, 0xFF5D8674, 0xFF5C8573, 0xFF5B8472, 0xFF5D8472, 0xFF5D8472, 0xFF5F8473, 0xFF608574, 0xFF648978, 0xFF587F6D, 0xFF5B8172, 0xFF5E8274, 0xFF617F75, 0xFF496058, 0xFF364540, 0xFF5D6766, 0xFF5A6966,
        0xFF2D403A, 0xFF243E35, 0xFF37554B, 0xFF547668, 0xFF5A7C6E, 0xFF517162, 0xFF335243, 0xFF3F604D, 0xFF2E5A43, 0xFF1D432E, 0xFF304033, 0xFF465246, 0xFF445C4C, 0xFF35503F, 0xFF39473A, 0xFF4E4D3B, 0xFF3A4934, 0xFF2B4F35, 0xFF517D64, 0xFF49735F,
        0xFF4C7263, 0xFF446E62, 0xFF2E6152, 0xFF2F4038, 0xFF12211A, 0xFF15201A, 0xFF222B26, 0xFF1B251C, 0xFF3C483A, 0xFF2F412B, 0xFF2C4027, 0xFF324238, 0xFF324238, 0xFF34453B, 0xFF2D3E36, 0xFF2A3E35, 0xFF31443E, 0xFF2D423B, 0xFF2C413A, 0xFF34483C,
        0xFF314539, 0xFF283C30, 0xFF2E4236, 0xFF304438, 0xFF263A2E, 0xFF25392D, 0xFF25392D, 0xFF1E3324, 0xFF253A2B, 0xFF253A2B, 0xFF203526, 0xFF263B2C, 0xFF304536, 0xFF2A3F30, 0xFF1B3021, 0xFF21381B, 0xFF203320, 0xFF2B3733, 0xFF1B2423, 0xFF172119,
        0xFF151F14, 0xFF25312D, 0xFF445255, 0xFF506E62, 0xFF4F7163, 0xFF496F60, 0xFF57806E, 0xFF517C69, 0xFF527B69, 0xFF547967, 0xFF5A7C6B, 0xFF557A68, 0xFF577C6A, 0xFF597E6C, 0xFF5A7F6D, 0xFF5A7F6D, 0xFF5A7F6D, 0xFF5B806E, 0xFF5C816F, 0xFF5B7F75,
        0xFF547967, 0xFF5C846A, 0xFF53836B, 0xFF518171, 0xFF2F5349, 0xFF434D44, 0xFF64584C, 0xFF6A6964, 0xFF4C5042, 0xFF30483A, 0xFF557C7B, 0xFF3C585B, 0xFF474A41, 0xFF474634, 0xFF2F3A2C, 0xFF505141, 0xFF395144, 0xFF39675D, 0xFF204A40, 0xFF334839,
        0xFF474838, 0xFF504E41, 0xFF444A3E, 0xFF4C483C, 0xFF3B5749, 0xFF3B655B, 0xFF3C514A, 0xFF4C4C40, 0xFF535540, 0xFF4A523D, 0xFF272F20, 0xFF35503D, 0xFF3F5F4A, 0xFF436652, 0xFF4E7560, 0xFF3A614E, 0xFF355A48, 0xFF5F8171, 0xFF5E7E6F, 0xFF3D594A,
        0xFF2D5646, 0xFF234E3A, 0xFF2F4629, 0xFF7D744B, 0xFF9E8250, 0xFFA68D55, 0xFF978A53, 0xFF7A6F42, 0xFF2E3820, 0xFF44554D, 0xFF7E8176, 0xFF9B9386, 0xFFE4E1DA, 0xFFCDCEC8, 0xFF999387, 0xFF95907D, 0xFF948F7C, 0xFF97927F, 0xFF9F9A87, 0xFFA8A390,
        0xFFA7A28F, 0xFF9D9885, 0xFF938E7B, 0xFF969082, 0xFF938D7F, 0xFF989586, 0xFF8F8B7F, 0xFF807E71, 0xFF939185, 0xFFA8A69A, 0xFF9D9D91, 0xFF5A7F6D, 0xFF5A7F6D, 0xFF577C6B, 0xFF648475, 0xFF577164, 0x00B6C5BE, 0x00F9FEFA, 0x00FFFEFC, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x00F6FFFD, 0xFF9DBBB1, 0xFF3F685A, 0xFF648A7D, 0xFF618579, 0xFF325C4E, 0xFF1B4B3B, 0xFF4A7361, 0xFF3F6856, 0xFF426B59, 0xFF2B5442, 0xFF3B6452,
        0xFF527B69, 0xFF295240, 0xFF194230, 0xFF567867, 0xFF456051, 0xFF626E64, 0xFFC5C4BF, 0xFFE0D6D4, 0xFFD1C7C5, 0xFFD5D0CC, 0xFFCDCCC7, 0xFFD4CAC9, 0xFFE3DBD9, 0xFFD0CCC9, 0xFF737570, 0xFF6E7971, 0xFF4A5B51, 0xFF465C50, 0xFF556F62, 0xFF8B9B98,
        0xFF7C8C89, 0xFF5D6D6A, 0xFF566663, 0xFF677774, 0xFF879794, 0xFF768683, 0xFF52625F, 0xFF45705F, 0xFF5C8776, 0xFF5A8574, 0xFF527D6C, 0xFF5A8574, 0xFF5E8978, 0xFF5B8675, 0xFF578271, 0xFF476E5C, 0xFF3C6351, 0xFF3B6250, 0xFF466D5B, 0xFF2E5543,
        0xFF3D6452, 0xFF49705E, 0xFF648B79, 0xFF5C8774, 0xFF5D8674, 0xFF5C8573, 0xFF5C8573, 0xFF5E8573, 0xFF5E8573, 0xFF608574, 0xFF618675, 0xFF628776, 0xFF628776, 0xFF648A7B, 0xFF5B8172, 0xFF5B7C71, 0xFF4E6960, 0xFF354C44, 0xFF41544E, 0xFF34453F,
        0xFF2D423B, 0xFF304B42, 0xFF36564B, 0xFF517567, 0xFF628678, 0xFF648676, 0xFF4D6F5F, 0xFF365543, 0xFF3B654F, 0xFF315440, 0xFF4C5A4D, 0xFF5F695E, 0xFF435A4A, 0xFF172F1F, 0xFF344034, 0xFF5C5948, 0xFF3F4C38, 0xFF24462E, 0xFF537B63, 0xFF507563,
        0xFF527466, 0xFF4A7063, 0xFF2E5A4D, 0xFF182720, 0xFF16211B, 0xFF131C17, 0xFF1A1F19, 0xFF232821, 0xFF5E695B, 0xFF6F7C6A, 0xFF768872, 0xFF5B6856, 0xFF6B7A67, 0xFF788774, 0xFF6C7A69, 0xFF657664, 0xFF6D7E6E, 0xFF677969, 0xFF617363, 0xFF6C7C6F,
        0xFF627265, 0xFF5D6D60, 0xFF667669, 0xFF647467, 0xFF607063, 0xFF67776A, 0xFF647467, 0xFF607161, 0xFF637464, 0xFF627363, 0xFF617262, 0xFF5F7060, 0xFF586959, 0xFF455646, 0xFF324333, 0xFF65795D, 0xFF707E6F, 0xFF5A6360, 0xFF1F2523, 0xFF181D16,
        0xFF10180D, 0xFF0E1712, 0xFF293237, 0xFF3F5D51, 0xFF557568, 0xFF517567, 0xFF4F7866, 0xFF4A7361, 0xFF527967, 0xFF557A68, 0xFF5A7C6B, 0xFF567B69, 0xFF587D6B, 0xFF5A7F6D, 0xFF5B806E, 0xFF5A7F6D, 0xFF5A7F6D, 0xFF5B806E, 0xFF5C816F, 0xFF5B8176,
        0xFF577C6A, 0xFF598167, 0xFF558369, 0xFF4C7867, 0xFF2D4B3F, 0xFF585C4D, 0xFF746351, 0xFF786556, 0xFF695F46, 0xFF47533B, 0xFF567669, 0xFF3A4F46, 0xFF56503A, 0xFF6B5A40, 0xFF5E563F, 0xFF5E523A, 0xFF3D4935, 0xFF375847, 0xFF284735, 0xFF586049,
        0xFF71654D, 0xFF695942, 0xFF635B46, 0xFF554D36, 0xFF445A43, 0xFF294C38, 0xFF3D4838, 0xFF726552, 0xFF67593E, 0xFF5E573A, 0xFF5E563F, 0xFF314C39, 0xFF466651, 0xFF436652, 0xFF426954, 0xFF39604D, 0xFF3B604E, 0xFF537565, 0xFF426253, 0xFF345041,
        0xFF315A4A, 0xFF2C5743, 0xFF30472A, 0xFF7B7249, 0xFF9D814F, 0xFFA58C54, 0xFF968952, 0xFF796E41, 0xFF2E3820, 0xFF42534B, 0xFF7D8075, 0xFF9A9285, 0xFFE4E1DA, 0xFFD1D2CC, 0xFF9D978B, 0xFF8A8572, 0xFF898471, 0xFF8D8875, 0xFF97927F, 0xFFA29D8A,
        0xFFA29D8A, 0xFF96917E, 0xFF8A8572, 0xFF8C8678, 0xFF908A7C, 0xFF979485, 0xFF908C80, 0xFF848275, 0xFF929084, 0xFFA4A296, 0xFF9E9E92, 0xFF618372, 0xFF618674, 0xFF608574, 0xFF638575, 0xFF506A5D, 0x00B1C0B9, 0x00FBFFFC, 0x00FFFFFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFCFE, 0x08F2FDF9, 0xFF99B7AD, 0xFF3F685A, 0xFF688E81, 0xFF678B7F, 0xFF355F51, 0xFF174737, 0xFF416A58, 0xFF4D7664, 0xFF355E4C, 0xFF244D3B, 0xFF436C5A,
        0xFF426B59, 0xFF1F4836, 0xFF1B4432, 0xFF5D796D, 0xFF4C6057, 0xFF5E6762, 0xFFBDBCBA, 0xFFEDE4E5, 0xFFE0D7D8, 0xFFD5D1D0, 0xFFCAC9C7, 0xFFDED5D8, 0xFFE9E3E5, 0xFFC7C5C6, 0xFF5C5E5D, 0xFF8B9491, 0xFF51605B, 0xFF50615B, 0xFF495E57, 0xFF2D4841,
        0xFF4B665F, 0xFF4B665F, 0xFF3E5952, 0xFF3D5851, 0xFF4A655E, 0xFF516C65, 0xFF607B74, 0xFF3E6958, 0xFF487362, 0xFF5D8877, 0xFF5F8A79, 0xFF547F6E, 0xFF578271, 0xFF5D8877, 0xFF5D8877, 0xFF59806E, 0xFF486F5D, 0xFF365D4B, 0xFF385F4D, 0xFF224937,
        0xFF3D6452, 0xFF49705E, 0xFF618876, 0xFF5C8774, 0xFF5C8573, 0xFF5C8573, 0xFF5C8573, 0xFF5F8674, 0xFF608775, 0xFF618675, 0xFF628776, 0xFF638575, 0xFF618675, 0xFF648B79, 0xFF5A8373, 0xFF5D8676, 0xFF587C70, 0xFF38584D, 0xFF27453B, 0xFF324740,
        0xFF375148, 0xFF3D5D52, 0xFF44685A, 0xFF567F6F, 0xFF5A8371, 0xFF5E8573, 0xFF608574, 0xFF4E695A, 0xFF365B49, 0xFF3F5E4E, 0xFF788279, 0xFF636861, 0xFF3D4F43, 0xFF22362A, 0xFF585F57, 0xFF8A8478, 0xFF596453, 0xFF24412B, 0xFF567965, 0xFF567565,
        0xFF557165, 0xFF4B695F, 0xFF1E4238, 0xFF131E18, 0xFF242A26, 0xFF1A1F19, 0xFF181913, 0xFF12130B, 0xFF373D31, 0xFF444C3D, 0xFF495645, 0xFF2F3920, 0xFF3D472E, 0xFF49533B, 0xFF3B472F, 0xFF35412B, 0xFF3D4B34, 0xFF3C4A33, 0xFF394632, 0xFF3F4A3C,
        0xFF323D2F, 0xFF323D2F, 0xFF3C4739, 0xFF354032, 0xFF343F31, 0xFF3F4A3C, 0xFF364133, 0xFF374232, 0xFF354030, 0xFF343F2F, 0xFF374232, 0xFF3A4535, 0xFF3A4535, 0xFF384333, 0xFF384333, 0xFF2E3D26, 0xFF303A2F, 0xFF313534, 0xFF0D0F0C, 0xFF101207,
        0xFF1A1D12, 0xFF161B17, 0xFF151A20, 0xFF243E33, 0xFF4E6C60, 0xFF527466, 0xFF4E7563, 0xFF527B69, 0xFF577E6C, 0xFF567B69, 0xFF5A7C6B, 0xFF587D6B, 0xFF5A7F6D, 0xFF5B806E, 0xFF5B806E, 0xFF5B806E, 0xFF5A7F6D, 0xFF5B806E, 0xFF5C816F, 0xFF588077,
        0xFF59806E, 0xFF597F66, 0xFF5B856D, 0xFF537A67, 0xFF3C5341, 0xFF817C66, 0xFF9E8567, 0xFFA37B61, 0xFF9B815C, 0xFF626641, 0xFF567057, 0xFF3C4B34, 0xFF716140, 0xFFA17F59, 0xFFA18361, 0xFF9A805F, 0xFF58573B, 0xFF445A43, 0xFF3D4F37, 0xFF767154,
        0xFFA88D6F, 0xFFA58667, 0xFFAA8F71, 0xFF7A6F4F, 0xFF465939, 0xFF3D5A3E, 0xFF74765E, 0xFFAC9478, 0xFFA18560, 0xFFA68E6A, 0xFF8C7354, 0xFF334E3B, 0xFF456550, 0xFF456854, 0xFF476E59, 0xFF3B624F, 0xFF2C513F, 0xFF385A4A, 0xFF365647, 0xFF355142,
        0xFF3E6757, 0xFF38634F, 0xFF334A2D, 0xFF7B7249, 0xFF9D814F, 0xFFA48B53, 0xFF958851, 0xFF7C7144, 0xFF303A22, 0xFF41524A, 0xFF7C7F74, 0xFF999184, 0xFFE3E0D9, 0xFFD2D3CD, 0xFF9F998D, 0xFF898471, 0xFF898471, 0xFF8D8875, 0xFF97927F, 0xFFA29D8A,
        0xFFA29D8A, 0xFF97927F, 0xFF8D8875, 0xFF8F897B, 0xFF969082, 0xFF999687, 0xFF938F83, 0xFF8D8B7E, 0xFF9F9D91, 0xFFB9B7AB, 0xFFC3C3B7, 0xFF5F7E6E, 0xFF5D7E6D, 0xFF648676, 0xFF658777, 0xFF4D695B, 0x00ACBDB5, 0x00FAFFFB, 0x00FFFEFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x00F8FFFF, 0xFF9DBBB1, 0xFF416A5C, 0xFF678D80, 0xFF62867A, 0xFF315B4D, 0xFF114131, 0xFF163F2D, 0xFF1F4836, 0xFF133C2A, 0xFF0D3624, 0xFF1C4533,
        0xFF1D4634, 0xFF153E2C, 0xFF194230, 0xFF597870, 0xFF556E68, 0xFF556864, 0xFFA5AFAE, 0xFFEDF3F3, 0xFFE3E9E9, 0xFFCFD8D7, 0xFFD3DDDC, 0xFFE0E6E6, 0xFFE3ECEB, 0xFFBEC8C7, 0xFF5D6C69, 0xFFA6B9B5, 0xFF758C86, 0xFF69827C, 0xFF647F78, 0xFF648D7F,
        0xFF5E8779, 0xFF567F71, 0xFF547D6F, 0xFF588173, 0xFF5B8476, 0xFF4A7365, 0xFF335C4E, 0xFF3A6554, 0xFF345F4E, 0xFF537E6D, 0xFF618C7B, 0xFF568170, 0xFF598473, 0xFF5B8675, 0xFF578271, 0xFF638A78, 0xFF557C6A, 0xFF3C6351, 0xFF39604E, 0xFF214836,
        0xFF406755, 0xFF486F5D, 0xFF5C8371, 0xFF5A8572, 0xFF5B8472, 0xFF5B8472, 0xFF5C8573, 0xFF5F8674, 0xFF5F8674, 0xFF618675, 0xFF628776, 0xFF648574, 0xFF5C816F, 0xFF5D8674, 0xFF5A8572, 0xFF5B8776, 0xFF5C8776, 0xFF477062, 0xFF3A6053, 0xFF435A52,
        0xFF425D54, 0xFF4A6C5E, 0xFF597F70, 0xFF618C79, 0xFF5A8572, 0xFF58826E, 0xFF5F8975, 0xFF6F8779, 0xFF5B7D6C, 0xFF355142, 0xFF555C54, 0xFF52554E, 0xFF314136, 0xFF16281C, 0xFF7A7F78, 0xFFC5BEB4, 0xFF808B7D, 0xFF294431, 0xFF496A59, 0xFF3D574A,
        0xFF40544B, 0xFF455A53, 0xFF18332A, 0xFF131917, 0xFF212622, 0xFF20211B, 0xFF2D2D23, 0xFF19190F, 0xFF202217, 0xFF272D23, 0xFF2F372C, 0xFF333A28, 0xFF303725, 0xFF343B29, 0xFF303928, 0xFF303928, 0xFF323D2C, 0xFF313C2C, 0xFF364131, 0xFF33392D,
        0xFF2C3226, 0xFF2D3327, 0xFF373D31, 0xFF32382C, 0xFF2E3428, 0xFF353B2F, 0xFF2C3226, 0xFF343A2E, 0xFF31372B, 0xFF2E3428, 0xFF2B3125, 0xFF2A3024, 0xFF2E3428, 0xFF383E32, 0xFF42483C, 0xFF2E3928, 0xFF1D241C, 0xFF212320, 0xFF181712, 0xFF2F2D20,
        0xFF323325, 0xFF191B16, 0xFF101318, 0xFF0E231A, 0xFF3F594E, 0xFF416154, 0xFF466B5A, 0xFF5B8270, 0xFF59806E, 0xFF547967, 0xFF5C7E6D, 0xFF597E6C, 0xFF5B806E, 0xFF5C816F, 0xFF5C816F, 0xFF5B806E, 0xFF5A7F6D, 0xFF5B806E, 0xFF5B806E, 0xFF598178,
        0xFF5A816F, 0xFF597F68, 0xFF588068, 0xFF557864, 0xFF445640, 0xFF887F60, 0xFFB3936C, 0xFFC08F6F, 0xFFB8986F, 0xFF6C6E46, 0xFF4F6D51, 0xFF3F5338, 0xFF82724E, 0xFFBA9368, 0xFFBA936C, 0xFFAF916F, 0xFF524E31, 0xFF40533D, 0xFF364832, 0xFF5D573D,
        0xFFAB8E6E, 0xFFB7936F, 0xFFBB9976, 0xFF898562, 0xFF4B6341, 0xFF2D4E2F, 0xFF51553C, 0xFFAB9274, 0xFFB3936D, 0xFFB2946E, 0xFFA58463, 0xFF35503D, 0xFF3A5A45, 0xFF426551, 0xFF4C735E, 0xFF3C6350, 0xFF294E3C, 0xFF365848, 0xFF3E5E4F, 0xFF486455,
        0xFF497262, 0xFF3B6652, 0xFF334A2D, 0xFF7D744B, 0xFFA08452, 0xFFA58C54, 0xFF968952, 0xFF82774A, 0xFF353F27, 0xFF41524A, 0xFF7B7E73, 0xFF978F82, 0xFFE0DDD6, 0xFFD2D3CD, 0xFF9E988C, 0xFF948F7C, 0xFF948F7C, 0xFF96917E, 0xFF9C9784, 0xFFA29D8A,
        0xFFA29D8A, 0xFF9E9986, 0xFF999481, 0xFF989284, 0xFF9D9789, 0xFF989586, 0xFF8D897D, 0xFF89877A, 0xFF9A988C, 0xFFBBB9AD, 0xFFD4D4C8, 0xFF7B9385, 0xFF617D6E, 0xFF628474, 0xFF638877, 0xFF4D6C5D, 0x00ACBDB5, 0x00F9FEFA, 0x00FFFDFC, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x00F8FFFF, 0xFF9DBBB1, 0xFF416A5C, 0xFF648A7D, 0xFF618579, 0xFF406A5C, 0xFF2A5A4A, 0xFF345D4B, 0xFF2E5745, 0xFF345D4B, 0xFF2F5846, 0xFF27503E,
        0xFF2A5341, 0xFF2B5442, 0xFF2F5846, 0xFF537D71, 0xFF53796E, 0xFF45665D, 0xFF63827A, 0xFF88A69E, 0xFF76948C, 0xFF718F87, 0xFF86A59D, 0xFF77958D, 0xFF76948C, 0xFF6F8E86, 0xFF395A51, 0xFF61857B, 0xFF52786D, 0xFF50786D, 0xFF5E867B, 0xFF568273,
        0xFF537F70, 0xFF5C8879, 0xFF608C7D, 0xFF5A8677, 0xFF5A8677, 0xFF588475, 0xFF497566, 0xFF335E4D, 0xFF315C4B, 0xFF3A6554, 0xFF497463, 0xFF5F8A79, 0xFF638E7D, 0xFF55806F, 0xFF5D8877, 0xFF618876, 0xFF5A816F, 0xFF416856, 0xFF3C6351, 0xFF224937,
        0xFF426957, 0xFF486F5D, 0xFF5D8472, 0xFF598471, 0xFF5A8371, 0xFF5B8472, 0xFF5C8573, 0xFF5E8573, 0xFF5F8674, 0xFF608574, 0xFF608574, 0xFF608271, 0xFF5F8472, 0xFF608A76, 0xFF578470, 0xFF54816D, 0xFF5C8877, 0xFF608979, 0xFF618778, 0xFF6B8278,
        0xFF668276, 0xFF628476, 0xFF618876, 0xFF5B8673, 0xFF5F8A76, 0xFF5F8975, 0xFF5A8470, 0xFF647C6F, 0xFF678979, 0xFF395547, 0xFF4C534C, 0xFF545651, 0xFF39493F, 0xFF0E1F15, 0xFF60655F, 0xFF79766F, 0xFF505C50, 0xFF193425, 0xFF597868, 0xFF5A7064,
        0xFF66756E, 0xFF6A7972, 0xFF30453C, 0xFF161A19, 0xFF1E201B, 0xFF292921, 0xFF49473B, 0xFF262418, 0xFF15150B, 0xFF151811, 0xFF161B17, 0xFF141B14, 0xFF0E150E, 0xFF111A15, 0xFF131C17, 0xFF141F1B, 0xFF131E1A, 0xFF0C1713, 0xFF0E1A18, 0xFF141B13,
        0xFF151C14, 0xFF121911, 0xFF171E16, 0xFF181F17, 0xFF131A12, 0xFF151C14, 0xFF131A12, 0xFF11170D, 0xFF141A10, 0xFF161C12, 0xFF151B11, 0xFF141A10, 0xFF151B11, 0xFF151B11, 0xFF151B11, 0xFF111C0E, 0xFF181D17, 0xFF1B1B19, 0xFF24211A, 0xFF646351,
        0xFF525341, 0xFF12130D, 0xFF1A1B20, 0xFF0C1D15, 0xFF43584F, 0xFF3E5A4E, 0xFF395B4B, 0xFF557A69, 0xFF537A68, 0xFF547967, 0xFF5F8170, 0xFF5A7F6D, 0xFF5B806E, 0xFF5C816F, 0xFF5C816F, 0xFF5B806E, 0xFF5B806E, 0xFF5B806E, 0xFF5C816F, 0xFF598178,
        0xFF597E6D, 0xFF5D806C, 0xFF547A65, 0xFF5B7D6C, 0xFF4D5F47, 0xFF837956, 0xFFBF9E71, 0xFFC09376, 0xFFBBA27A, 0xFF6C7854, 0xFF557D65, 0xFF496650, 0xFF8A8160, 0xFFBC9B72, 0xFFB5936D, 0xFFB29B7C, 0xFF595D46, 0xFF476354, 0xFF395546, 0xFF5B5E49,
        0xFFB59D81, 0xFFBC9B7A, 0xFFB89A76, 0xFF909479, 0xFF436447, 0xFF386046, 0xFF48543E, 0xFFA38F76, 0xFFBAA07D, 0xFFB49C78, 0xFFA78C6E, 0xFF445F4C, 0xFF3E5E49, 0xFF466955, 0xFF476E59, 0xFF385F4C, 0xFF375C4A, 0xFF416353, 0xFF39594A, 0xFF587465,
        0xFF477060, 0xFF2F5A46, 0xFF2D4427, 0xFF7D744B, 0xFFA18553, 0xFFA68D55, 0xFF988B54, 0xFF83784B, 0xFF374129, 0xFF3F5048, 0xFF7A7D72, 0xFF968E81, 0xFFDEDBD4, 0xFFD4D5CF, 0xFFA09A8E, 0xFF9D9885, 0xFF9D9885, 0xFF9D9885, 0xFF9C9784, 0xFF9D9885,
        0xFF9D9885, 0xFF9F9A87, 0xFFA09B88, 0xFF969082, 0xFF9B9587, 0xFF959283, 0xFF8B877B, 0xFF868477, 0xFF939185, 0xFFB7B5A9, 0xFFD9D9CD, 0xFFB4C6BA, 0xFF738B7D, 0xFF618172, 0xFF618876, 0xFF4F6F60, 0x00AABEB5, 0x00FBFFFC, 0x00FFFDFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFF9FB, 0x00F6FFFD, 0xFF98B6AC, 0xFF3E6759, 0xFF63897C, 0xFF678B7F, 0xFF5A8476, 0xFF548474, 0xFF5F8876, 0xFF567F6D, 0xFF618A78, 0xFF5F8876, 0xFF5C8573,
        0xFF618A78, 0xFF5B8472, 0xFF618A78, 0xFF5C887B, 0xFF537F72, 0xFF4C786B, 0xFF527E71, 0xFF548073, 0xFF477366, 0xFF558174, 0xFF618D80, 0xFF588777, 0xFF497868, 0xFF588777, 0xFF4D7C6C, 0xFF467565, 0xFF4F7E6E, 0xFF598878, 0xFF619080, 0xFF5E8D7B,
        0xFF5A8977, 0xFF588775, 0xFF558472, 0xFF5D8C7A, 0xFF5C8B79, 0xFF5C8B79, 0xFF578674, 0xFF45705F, 0xFF335E4D, 0xFF2F5A49, 0xFF3B6655, 0xFF4F7A69, 0xFF5D8877, 0xFF5D8877, 0xFF5C8776, 0xFF5B8270, 0xFF587F6D, 0xFF3D6452, 0xFF39604E, 0xFF1E4533,
        0xFF426957, 0xFF49705E, 0xFF648B79, 0xFF598471, 0xFF5B8472, 0xFF5B8472, 0xFF5C8573, 0xFF5F8674, 0xFF5F8674, 0xFF608574, 0xFF5F8473, 0xFF5D836E, 0xFF608774, 0xFF5D8874, 0xFF58836F, 0xFF598471, 0xFF618A78, 0xFF608476, 0xFF567669, 0xFF5D7269,
        0xFF556F64, 0xFF537364, 0xFF5C8170, 0xFF57816D, 0xFF5B8571, 0xFF5B826F, 0xFF5A806B, 0xFF637D70, 0xFF577C6B, 0xFF355445, 0xFF4B554D, 0xFF3D423C, 0xFF1D2E24, 0xFF12261B, 0xFF5D645D, 0xFF74756F, 0xFF4A5A4F, 0xFF123122, 0xFF537364, 0xFF586E62,
        0xFF75827B, 0xFF727F78, 0xFF283931, 0xFF141A18, 0xFF1B201A, 0xFF2E3025, 0xFF505141, 0xFF292A1A, 0xFF181A0F, 0xFF161B17, 0xFF171D1D, 0xFF263435, 0xFF253334, 0xFF273538, 0xFF202F32, 0xFF1F2E33, 0xFF223438, 0xFF1C2E32, 0xFF1A2C30, 0xFF202F28,
        0xFF213029, 0xFF17261F, 0xFF15241D, 0xFF16251E, 0xFF112019, 0xFF14231C, 0xFF15241D, 0xFF17221C, 0xFF16211B, 0xFF141F19, 0xFF15201A, 0xFF19241E, 0xFF1D2822, 0xFF1D2822, 0xFF1A251F, 0xFF1D271E, 0xFF171D19, 0xFF171914, 0xFF18180C, 0xFF50523C,
        0xFF4A4C37, 0xFF12150C, 0xFF1B1E23, 0xFF15221B, 0xFF596A62, 0xFF52695F, 0xFF3D5D4E, 0xFF557A69, 0xFF557A69, 0xFF597E6C, 0xFF5A7F6D, 0xFF597E6C, 0xFF5A7F6D, 0xFF5C816F, 0xFF5C816F, 0xFF5B806E, 0xFF5B806E, 0xFF5B806E, 0xFF5C816F, 0xFF5A8073,
        0xFF5A7C6C, 0xFF5F8170, 0xFF557C6A, 0xFF5F8375, 0xFF4E644F, 0xFF7D7956, 0xFFC5AA7B, 0xFFB99477, 0xFFB6A47E, 0xFF697A5A, 0xFF558170, 0xFF476959, 0xFF888467, 0xFFBCA37B, 0xFFB59C74, 0xFFB2A185, 0xFF67705B, 0xFF3D6354, 0xFF284E41, 0xFF55604F,
        0xFFB4A48A, 0xFFB69977, 0xFFBCA07B, 0xFF888D76, 0xFF33553D, 0xFF4A7561, 0xFF5B6A57, 0xFFA2947A, 0xFFB6A281, 0xFFB3A17D, 0xFF9E896A, 0xFF4D6855, 0xFF41614C, 0xFF4F725E, 0xFF4D745F, 0xFF375E4B, 0xFF375C4A, 0xFF416353, 0xFF3F5F50, 0xFF567263,
        0xFF3A6353, 0xFF224D39, 0xFF294023, 0xFF7A7148, 0xFFA08452, 0xFFA88F57, 0xFF998C55, 0xFF7F7447, 0xFF343E26, 0xFF3B4C44, 0xFF787B70, 0xFF958D80, 0xFFDEDBD4, 0xFFD9DAD4, 0xFFA49E92, 0xFF9F9A87, 0xFF9F9A87, 0xFF9F9A87, 0xFF9F9A87, 0xFF9E9986,
        0xFF9F9A87, 0xFF9F9A87, 0xFFA09B88, 0xFF9A9486, 0xFF9A9486, 0xFF959283, 0xFF8C887C, 0xFF868477, 0xFF959387, 0xFFBEBCB0, 0xFFE6E6DA, 0xFFD1DDD3, 0xFF768A7E, 0xFF5E7E6F, 0xFF618876, 0xFF517363, 0x00ABC0B7, 0x00FCFFFD, 0x00FFFCFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x00F8FFFF, 0xFF99B7AD, 0xFF416A5C, 0xFF648A7D, 0xFF65897D, 0xFF5E887A, 0xFF5F8F7F, 0xFF618A78, 0xFF5C8573, 0xFF5E8775, 0xFF5F8876, 0xFF608977,
        0xFF618A78, 0xFF5C8573, 0xFF5D8674, 0xFF63897A, 0xFF5A8373, 0xFF5F8A79, 0xFF638F7E, 0xFF639280, 0xFF5E8D7B, 0xFF6C9887, 0xFF64907F, 0xFF679883, 0xFF5C8B77, 0xFF5F8E7A, 0xFF669581, 0xFF598672, 0xFF5D8A76, 0xFF608B78, 0xFF5A8572, 0xFF5F8A79,
        0xFF5F8A79, 0xFF5E8978, 0xFF55806F, 0xFF5C8776, 0xFF588372, 0xFF5B8675, 0xFF5C8776, 0xFF5D8877, 0xFF426D5C, 0xFF396453, 0xFF3C6756, 0xFF386352, 0xFF4C7766, 0xFF638E7D, 0xFF588372, 0xFF5C8371, 0xFF587F6D, 0xFF3A614F, 0xFF385F4D, 0xFF1D4432,
        0xFF426957, 0xFF486F5D, 0xFF668D7B, 0xFF5B8673, 0xFF5D8674, 0xFF5E8775, 0xFF5E8775, 0xFF608775, 0xFF608775, 0xFF608574, 0xFF608574, 0xFF5C8670, 0xFF5B856F, 0xFF57826E, 0xFF598470, 0xFF5C8370, 0xFF517162, 0xFF3C584A, 0xFF2C4437, 0xFF213228,
        0xFF21372B, 0xFF233F31, 0xFF3D5E4D, 0xFF4F7462, 0xFF5C816F, 0xFF597C68, 0xFF5C7F6B, 0xFF567566, 0xFF5C8573, 0xFF3E6050, 0xFF606D64, 0xFF768078, 0xFF51675B, 0xFF526A5D, 0xFFA5B0A8, 0xFFB6BDB6, 0xFF7F9388, 0xFF2C4E3E, 0xFF446656, 0xFF344C3F,
        0xFF66736A, 0xFF78857C, 0xFF3C4C42, 0xFF0D1814, 0xFF131A13, 0xFF202618, 0xFF404330, 0xFF202312, 0xFF151B0F, 0xFF1E2422, 0xFF333D3F, 0xFF3A5350, 0xFF2E4744, 0xFF253E3B, 0xFF1B3333, 0xFF1D3736, 0xFF253F40, 0xFF203C3D, 0xFF203C3D, 0xFF2C423F,
        0xFF2A403D, 0xFF223835, 0xFF213734, 0xFF1D3330, 0xFF1C322F, 0xFF243A37, 0xFF213734, 0xFF1B2E2A, 0xFF142723, 0xFF10231F, 0xFF122521, 0xFF182B27, 0xFF20332F, 0xFF30433F, 0xFF3E514D, 0xFF28352C, 0xFF0B140F, 0xFF191E18, 0xFF212516, 0xFF44482F,
        0xFF42482E, 0xFF151B11, 0xFF0C1115, 0xFF1C2721, 0xFF5E6D66, 0xFF566B62, 0xFF426152, 0xFF5D7F6F, 0xFF587D6C, 0xFF5A7F6D, 0xFF577C6A, 0xFF577C6A, 0xFF597E6C, 0xFF5B806E, 0xFF5C816F, 0xFF5B806E, 0xFF5B806E, 0xFF5C816F, 0xFF5D8270, 0xFF608272,
        0xFF5D7E6D, 0xFF5C7E6D, 0xFF567F71, 0xFF547C73, 0xFF375241, 0xFF707354, 0xFFB5A478, 0xFFB3936D, 0xFFAF9D75, 0xFF5E6D4C, 0xFF496F60, 0xFF3A5648, 0xFF7E795B, 0xFFBCA37A, 0xFFB39C70, 0xFFB4A180, 0xFF6C765D, 0xFF375E4C, 0xFF224839, 0xFF4C5A43,
        0xFFAF9F7E, 0xFFB79770, 0xFFC4A376, 0xFF88846B, 0xFF536D54, 0xFF496C54, 0xFF3F4833, 0xFFA69579, 0xFFB39D76, 0xFFA9986D, 0xFFB39F7A, 0xFF435E4B, 0xFF43634E, 0xFF577A66, 0xFF507762, 0xFF3A614E, 0xFF385D4B, 0xFF3E6050, 0xFF446455, 0xFF4A6657,
        0xFF2F5848, 0xFF224D39, 0xFF2D4427, 0xFF786F46, 0xFF9D814F, 0xFFAA9159, 0xFF978A53, 0xFF7D7245, 0xFF343E26, 0xFF394A42, 0xFF777A6F, 0xFF958D80, 0xFFDEDBD4, 0xFFDADBD5, 0xFFA59F93, 0xFF999481, 0xFF999481, 0xFF9C9784, 0xFFA29D8A, 0xFFA7A28F,
        0xFFA6A18E, 0xFF9F9A87, 0xFF999481, 0xFF9C9688, 0xFF989284, 0xFF949182, 0xFF8D897D, 0xFF828073, 0xFF8E8C80, 0xFFBAB8AC, 0xFFE1E1D5, 0xFFD3DAD2, 0xFF728478, 0xFF5F7F70, 0xFF618A78, 0xFF4F7463, 0x00ABC0B7, 0x00FCFFFD, 0x00FFF9FA, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x00F8FFFF, 0xFF98B6AC, 0xFF436C5E, 0xFF678D80, 0xFF64887C, 0xFF5C8678, 0xFF5D8D7D, 0xFF638C7A, 0xFF608977, 0xFF5C8573, 0xFF628B79, 0xFF5E8775,
        0xFF5F8876, 0xFF68917F, 0xFF628B79, 0xFF658475, 0xFF678778, 0xFF678C7B, 0xFF5E8775, 0xFF608977, 0xFF547D6B, 0xFF517A68, 0xFF486F5D, 0xFF416E59, 0xFF517C68, 0xFF497460, 0xFF57816D, 0xFF658C79, 0xFF5F8472, 0xFF608573, 0xFF618372, 0xFF5B8474,
        0xFF567F6F, 0xFF5E8777, 0xFF5C8575, 0xFF608979, 0xFF5E8777, 0xFF618A7A, 0xFF567F6F, 0xFF5F8A79, 0xFF5C8776, 0xFF426D5C, 0xFF35604F, 0xFF386352, 0xFF426D5C, 0xFF588372, 0xFF608B7A, 0xFF618876, 0xFF5B8270, 0xFF3A614F, 0xFF3A614F, 0xFF204735,
        0xFF446B59, 0xFF466D5B, 0xFF648B79, 0xFF5D8875, 0xFF5F8876, 0xFF608977, 0xFF608977, 0xFF628977, 0xFF618876, 0xFF618675, 0xFF608574, 0xFF5E8A73, 0xFF59856E, 0xFF55816A, 0xFF5A8470, 0xFF4E7361, 0xFF294537, 0xFF13291D, 0xFF17281E, 0xFF3A4A40,
        0xFF3B4F44, 0xFF1A3427, 0xFF113020, 0xFF2D4F3E, 0xFF597C68, 0xFF5E7F6C, 0xFF5E7D6B, 0xFF648477, 0xFF527D6C, 0xFF315547, 0xFF45544D, 0xFF505B55, 0xFF42594F, 0xFF476156, 0xFF4E5B54, 0xFF333E38, 0xFF334A40, 0xFF1F4335, 0xFF4D7163, 0xFF324C3F,
        0xFF5E6E64, 0xFF6F7C73, 0xFF38483E, 0xFF1C2826, 0xFF202A21, 0xFF2E3726, 0xFF4E533D, 0xFF2E331F, 0xFF151D10, 0xFF151E1B, 0xFF3B464A, 0xFF436358, 0xFF24433B, 0xFF123129, 0xFF103128, 0xFF1A3B34, 0xFF1D3E37, 0xFF143731, 0xFF173A34, 0xFF13302E,
        0xFF0F2C2A, 0xFF112E2C, 0xFF183533, 0xFF122F2D, 0xFF163331, 0xFF23403E, 0xFF1B3836, 0xFF1B3431, 0xFF17302D, 0xFF1A3330, 0xFF203936, 0xFF1D3633, 0xFF1C3532, 0xFF314A47, 0xFF4C6562, 0xFF37463F, 0xFF16211D, 0xFF192019, 0xFF232A18, 0xFF555B3F,
        0xFF4D563B, 0xFF1C2417, 0xFF20282B, 0xFF202924, 0xFF525F58, 0xFF41564D, 0xFF385446, 0xFF597B6B, 0xFF517665, 0xFF567B69, 0xFF587D6B, 0xFF567B69, 0xFF587D6B, 0xFF5A7F6D, 0xFF5B806E, 0xFF5B806E, 0xFF5B806E, 0xFF5C816F, 0xFF5D8270, 0xFF618271,
        0xFF5F7E6C, 0xFF5B7D6D, 0xFF5E867D, 0xFF527D76, 0xFF2D4C3D, 0xFF778063, 0xFFB9AD83, 0xFFB7966B, 0xFFB29D70, 0xFF5E6843, 0xFF486958, 0xFF3B5041, 0xFF847959, 0xFFC2A778, 0xFFB09A68, 0xFFB29C73, 0xFF6B7251, 0xFF456852, 0xFF3A604B, 0xFF566045,
        0xFFB3A078, 0xFFBE9A6A, 0xFFC19C68, 0xFF887A5F, 0xFF5A6A4D, 0xFF4F6C50, 0xFF46492E, 0xFFAC9573, 0xFFBAA274, 0xFFB7A371, 0xFFA48E5F, 0xFF46614E, 0xFF597964, 0xFF658874, 0xFF466D58, 0xFF365D4A, 0xFF416654, 0xFF355747, 0xFF254536, 0xFF405C4D,
        0xFF2C5545, 0xFF2A5541, 0xFF364D30, 0xFF786F46, 0xFF9B7F4D, 0xFFAA9159, 0xFF958851, 0xFF7F7447, 0xFF353F27, 0xFF394A42, 0xFF777A6F, 0xFF948C7F, 0xFFDCD9D2, 0xFFD8D9D3, 0xFFA39D91, 0xFF8D8875, 0xFF8D8875, 0xFF938E7B, 0xFFA09B88, 0xFFACA794,
        0xFFAAA592, 0xFF9B9683, 0xFF8D8875, 0xFF8E887A, 0xFF8D8779, 0xFF939081, 0xFF959185, 0xFF8B897C, 0xFF949286, 0xFFBCBAAE, 0xFFDFDFD3, 0xFFDBE0D9, 0xFF7A8A7F, 0xFF688778, 0xFF628B79, 0xFF49705E, 0x00A7BEB4, 0x00FCFFFD, 0x00FFFCFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFBFF, 0x00F9FFFF, 0xFF99B7AD, 0xFF446A5D, 0xFF648A7D, 0xFF678D80, 0xFF477364, 0xFF417463, 0xFF47705E, 0xFF517A68, 0xFF406957, 0xFF4E7966, 0xFF4D7865,
        0xFF477460, 0xFF406D59, 0xFF4A7763, 0xFF628171, 0xFF4E6658, 0xFF425448, 0xFF626E64, 0xFF4D5A50, 0xFF34463A, 0xFF264132, 0xFF2B4A3A, 0xFF2B3B31, 0xFF2E3F35, 0xFF273D31, 0xFF395549, 0xFF345648, 0xFF3C6557, 0xFF558172, 0xFF629181, 0xFF5B8476,
        0xFF508070, 0xFF528976, 0xFF558575, 0xFF698F82, 0xFF5D7D72, 0xFF5C8275, 0xFF5A8979, 0xFF5E8777, 0xFF618A7A, 0xFF608979, 0xFF3F6858, 0xFF355E4E, 0xFF3F6858, 0xFF416A5A, 0xFF5F8878, 0xFF618C79, 0xFF5C8573, 0xFF37604E, 0xFF345D4B, 0xFF234A38,
        0xFF3E6553, 0xFF4D7261, 0xFF678C7B, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF5F8674, 0xFF5E8573, 0xFF5E8573, 0xFF628473, 0xFF5C8370, 0xFF5D8471, 0xFF5C7869, 0xFF29392E, 0xFF232D24, 0xFF35473B, 0xFF2E4A3B, 0xFF4D6158,
        0xFF465A4F, 0xFF314238, 0xFF293A30, 0xFF1B3124, 0xFF2C4839, 0xFF5E816D, 0xFF5A816C, 0xFF5C8672, 0xFF547967, 0xFF3B5C4B, 0xFF3C5849, 0xFF365142, 0xFF203C2D, 0xFF123322, 0xFF1A3F2D, 0xFF31523F, 0xFF163724, 0xFF1C3E2D, 0xFF4A6A5B, 0xFF3A564A,
        0xFF5F746D, 0xFF75847F, 0xFF475150, 0xFF25403B, 0xFF20342B, 0xFF222A1F, 0xFF3C3A2D, 0xFF211B0F, 0xFF1C1912, 0xFF1C211D, 0xFF1F292A, 0xFF1B372B, 0xFF264C3D, 0xFF255442, 0xFF1C4736, 0xFF19372B, 0xFF1E382D, 0xFF2B4D3F, 0xFF3A6655, 0xFF3D5D4E,
        0xFF364E41, 0xFF16261C, 0xFF222824, 0xFF202622, 0xFF182722, 0xFF1D342C, 0xFF214136, 0xFF233634, 0xFF1E3230, 0xFF1C322F, 0xFF1F3630, 0xFF1F342F, 0xFF20302D, 0xFF25302C, 0xFF2A3330, 0xFF19231B, 0xFF191F1D, 0xFF1B1D1C, 0xFF1F2116, 0xFF494C37,
        0xFF3C412B, 0xFF182015, 0xFF21292C, 0xFF1F2C25, 0xFF4B5C54, 0xFF536A60, 0xFF3B5B4C, 0xFF597E6D, 0xFF537A68, 0xFF547B68, 0xFF547B68, 0xFF55806C, 0xFF5A8571, 0xFF58826E, 0xFF58826E, 0xFF5E8572, 0xFF5C8370, 0xFF5C8370, 0xFF608573, 0xFF638273,
        0xFF5B7D6D, 0xFF557F6B, 0xFF5E8A73, 0xFF618770, 0xFF4D6651, 0xFF6D7867, 0xFF959887, 0xFFA59876, 0xFF9A9778, 0xFF707D61, 0xFF61795F, 0xFF3C543A, 0xFF7D8A6C, 0xFFA19E7B, 0xFFB0A47C, 0xFFA19984, 0xFF6A6A52, 0xFF606F52, 0xFF4A6242, 0xFF4C6142,
        0xFF8C9278, 0xFFAFA28F, 0xFFB89E8F, 0xFF687F63, 0xFF5F7357, 0xFF58684D, 0xFF465037, 0xFF83876E, 0xFF9F9D86, 0xFFB0AA92, 0xFF827A63, 0xFF466A5C, 0xFF5A7F6E, 0xFF608574, 0xFF4A6F5E, 0xFF325745, 0xFF365B49, 0xFF3E644F, 0xFF2B513C, 0xFF355643,
        0xFF3A5E50, 0xFF426658, 0xFF364C35, 0xFF72724C, 0xFF9C8B57, 0xFFA68D55, 0xFFA08751, 0xFF7F733F, 0xFF333C21, 0xFF37473D, 0xFF76776F, 0xFF938D81, 0xFFD9D9D1, 0xFFD5D8CF, 0xFFA3A08F, 0xFF8C866E, 0xFF837D65, 0xFF8A846C, 0xFF98927A, 0xFFA8A28A,
        0xFFABA58D, 0xFF9B957D, 0xFF948E76, 0xFF9F8878, 0xFF978676, 0xFF9C9382, 0xFF969583, 0xFF888C7D, 0xFF939588, 0xFFB8B8AE, 0xFFE4E1D8, 0xFFD9DAD4, 0xFF7B887E, 0xFF6B8A7B, 0xFF618A78, 0xFF486F5D, 0x00A7BEB4, 0x00FBFFFD, 0x00FFFCFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFF7FB, 0x00F9FFFF, 0xFF98B6AC, 0xFF456B5E, 0xFF688E81, 0xFF668C7F, 0xFF325E4F, 0xFF174A39, 0xFF1A4331, 0xFF18412F, 0xFF1B4432, 0xFF1D4835, 0xFF143F2C,
        0xFF174430, 0xFF184531, 0xFF1D4A36, 0xFF5F806F, 0xFF466152, 0xFF485A4E, 0xFF757F76, 0xFFA1ABA2, 0xFFABB7AD, 0xFFAABCB0, 0xFFA6BCAF, 0xFFB1BBB3, 0xFFB3C0B7, 0xFF9CADA3, 0xFF50655C, 0xFF294539, 0xFF37584D, 0xFF62887B, 0xFF557E70, 0xFF64887C,
        0xFF5E8D7D, 0xFF5B907E, 0xFF578777, 0xFF608679, 0xFF638479, 0xFF638C7E, 0xFF5A8D7C, 0xFF5F8878, 0xFF537C6C, 0xFF658E7E, 0xFF5F8878, 0xFF426B5B, 0xFF3B6454, 0xFF3B6454, 0xFF3E6757, 0xFF588370, 0xFF5B8472, 0xFF3E6755, 0xFF396250, 0xFF234A38,
        0xFF406755, 0xFF4E7362, 0xFF668B7A, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF5F8674, 0xFF5E8573, 0xFF5E8573, 0xFF5F8170, 0xFF5F8673, 0xFF5B826F, 0xFF3B5748, 0xFF213126, 0xFF3D473E, 0xFF3B4D41, 0xFF2A4637, 0xFF43584F,
        0xFF3D5146, 0xFF34453B, 0xFF32463B, 0xFF203629, 0xFF1C3829, 0xFF436451, 0xFF577D68, 0xFF527C68, 0xFF648977, 0xFF587767, 0xFF556D5F, 0xFF5A7264, 0xFF5B7667, 0xFF537463, 0xFF2A4F3D, 0xFF456653, 0xFF355643, 0xFF446655, 0xFF587869, 0xFF3D594D,
        0xFF627770, 0xFF687772, 0xFF3F4948, 0xFF243D39, 0xFF23342C, 0xFF272F24, 0xFF413F32, 0xFF211D11, 0xFF181810, 0xFF161B17, 0xFF182022, 0xFF0C1B14, 0xFF22392F, 0xFF28483B, 0xFF1A362A, 0xFF0B1C14, 0xFF0E1B14, 0xFF263B32, 0xFF436155, 0xFF597B6B,
        0xFF3F594C, 0xFF102016, 0xFF151E19, 0xFF1C221E, 0xFF1B2622, 0xFF162923, 0xFF132A22, 0xFF122825, 0xFF172D2A, 0xFF1A332F, 0xFF1A312B, 0xFF142723, 0xFF121E1A, 0xFF161C1A, 0xFF1D1F1E, 0xFF18221A, 0xFF181E1C, 0xFF1B1D1C, 0xFF1F2116, 0xFF4A4D38,
        0xFF3D422C, 0xFF171F14, 0xFF1E2629, 0xFF27342D, 0xFF54655D, 0xFF546B61, 0xFF3A5A4B, 0xFF5A7F6E, 0xFF537A68, 0xFF577E6B, 0xFF577E6B, 0xFF547F6B, 0xFF55806C, 0xFF56806C, 0xFF59836F, 0xFF5C8370, 0xFF577E6B, 0xFF587D6B, 0xFF5B806E, 0xFF628476,
        0xFF5F8576, 0xFF5B8472, 0xFF5D8773, 0xFF628571, 0xFF46614E, 0xFF3B4C3A, 0xFF333E2E, 0xFF4E533C, 0xFF55634C, 0xFF506956, 0xFF638374, 0xFF456655, 0xFF617A65, 0xFF5C6A50, 0xFF5C6344, 0xFF72786A, 0xFF586655, 0xFF5E7762, 0xFF486851, 0xFF35523C,
        0xFF556A57, 0xFF727A6B, 0xFF7D7D71, 0xFF3E5B49, 0xFF506B58, 0xFF5A7360, 0xFF425746, 0xFF536452, 0xFF576453, 0xFF65705F, 0xFF4E5746, 0xFF3F6355, 0xFF5D8271, 0xFF5D8271, 0xFF486D5C, 0xFF436856, 0xFF385D4B, 0xFF2C523D, 0xFF3D634E, 0xFF3A5B48,
        0xFF496D5F, 0xFF406456, 0xFF384E37, 0xFF787852, 0xFF988753, 0xFFA58C54, 0xFF9D844E, 0xFF847844, 0xFF374025, 0xFF3B4B41, 0xFF787971, 0xFF938D81, 0xFFD9D9D1, 0xFFD6D9D0, 0xFFA5A291, 0xFF9D9683, 0xFF9C9582, 0xFFA49D8A, 0xFFA69F8C, 0xFFA8A18E,
        0xFFA8A18E, 0xFFA39C89, 0xFFA59E8B, 0xFFA79483, 0xFF9E9180, 0xFFA09987, 0xFF949381, 0xFF858779, 0xFF919386, 0xFFB7B4AB, 0xFFDFDAD4, 0xFFD3D8D1, 0xFF728277, 0xFF628172, 0xFF638C7A, 0xFF517866, 0x00AAC1B7, 0x00FAFFFC, 0x00FFFBFB, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFF9FD, 0x00F9FFFF, 0xFF97B5AB, 0xFF43695C, 0xFF688E81, 0xFF688E81, 0xFF2F5B4C, 0xFF0D402F, 0xFF3A6351, 0xFF3B6452, 0xFF305947, 0xFF083320, 0xFF1D4835,
        0xFF2E5B47, 0xFF174430, 0xFF1C4935, 0xFF5C7E6E, 0xFF3C584A, 0xFF4E5E54, 0xFF99A099, 0xFFEEEFEA, 0xFFEFF1EC, 0xFFECF1EB, 0xFFE6F0E8, 0xFFE9EAE5, 0xFFEFF1EC, 0xFFC9D0C9, 0xFF57645D, 0xFF51655C, 0xFF435D54, 0xFF547268, 0xFF5A7A6F, 0xFF566D65,
        0xFF46675C, 0xFF396355, 0xFF436C5E, 0xFF4A6B60, 0xFF4B6B60, 0xFF40695B, 0xFF3B6E5D, 0xFF5F8878, 0xFF588171, 0xFF598272, 0xFF5E8777, 0xFF578070, 0xFF426B5B, 0xFF365F4F, 0xFF3B6454, 0xFF446F5C, 0xFF4D7664, 0xFF3B6452, 0xFF38614F, 0xFF1F4634,
        0xFF3E6553, 0xFF4D7261, 0xFF638877, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF5F8674, 0xFF5F8674, 0xFF5E8573, 0xFF5F8170, 0xFF5F8673, 0xFF547B68, 0xFF1D392A, 0xFF26362B, 0xFF555F56, 0xFF36483C, 0xFF213D2E, 0xFF294036,
        0xFF22382C, 0xFF273B30, 0xFF374B40, 0xFF2C4235, 0xFF1C3728, 0xFF2E4F3C, 0xFF5E816D, 0xFF5D8773, 0xFF5B7D6C, 0xFF4F6A5B, 0xFF64786C, 0xFF62766A, 0xFF3D5547, 0xFF4C6B5B, 0xFF365B49, 0xFF446552, 0xFF375845, 0xFF527463, 0xFF4D6D5E, 0xFF254135,
        0xFF596E67, 0xFF5D6C67, 0xFF293332, 0xFF102321, 0xFF17261F, 0xFF272D23, 0xFF47483A, 0xFF29271A, 0xFF1C1C14, 0xFF141915, 0xFF10181A, 0xFF1E201D, 0xFF1E2722, 0xFF16251E, 0xFF121D17, 0xFF161B17, 0xFF1D1F1C, 0xFF232925, 0xFF2A3730, 0xFF507564,
        0xFF3A5648, 0xFF0E1F15, 0xFF131C17, 0xFF1D221E, 0xFF222826, 0xFF212C28, 0xFF1F2B27, 0xFF112A26, 0xFF203935, 0xFF2B443E, 0xFF273E38, 0xFF1B2C26, 0xFF121D19, 0xFF171918, 0xFF1C1B19, 0xFF162018, 0xFF171B1A, 0xFF191B1A, 0xFF212117, 0xFF4D4F3A,
        0xFF3E432D, 0xFF161C12, 0xFF192124, 0xFF1E2B24, 0xFF51625A, 0xFF4C6359, 0xFF355546, 0xFF5A7F6E, 0xFF537A68, 0xFF59806D, 0xFF59806D, 0xFF557F6B, 0xFF537D69, 0xFF56806C, 0xFF5A816E, 0xFF59806D, 0xFF587D6B, 0xFF597E6C, 0xFF5D8270, 0xFF557F71,
        0xFF578072, 0xFF5C8575, 0xFF64887A, 0xFF5C7E6E, 0xFF375646, 0xFF264132, 0xFF233B2D, 0xFF254434, 0xFF325247, 0xFF3B6059, 0xFF5E8582, 0xFF466D68, 0xFF43695E, 0xFF274738, 0xFF23432E, 0xFF465D55, 0xFF4C675E, 0xFF5E7F74, 0xFF4C7265, 0xFF264F41,
        0xFF22483B, 0xFF2A4B40, 0xFF2F4D43, 0xFF305148, 0xFF507168, 0xFF607F77, 0xFF47675C, 0xFF335147, 0xFF234135, 0xFF2A483C, 0xFF233F33, 0xFF3E6254, 0xFF5C8170, 0xFF5C8170, 0xFF537867, 0xFF395E4C, 0xFF325745, 0xFF486E59, 0xFF335944, 0xFF3A5B48,
        0xFF4D7163, 0xFF35594B, 0xFF314730, 0xFF75754F, 0xFF94834F, 0xFFA78E56, 0xFFA18852, 0xFF847844, 0xFF374025, 0xFF3C4C42, 0xFF797A72, 0xFF918B7F, 0xFFD7D7CF, 0xFFD8DBD2, 0xFFA7A493, 0xFFAAA391, 0xFFA9A290, 0xFFADA694, 0xFFABA492, 0xFFADA694,
        0xFFB4AD9B, 0xFFB2AB99, 0xFFB5AE9C, 0xFFB3A896, 0xFFAAA390, 0xFFA6A390, 0xFF909181, 0xFF818274, 0xFF949488, 0xFFC1BDB4, 0xFFE8E3DD, 0xFFDDE7DE, 0xFF7B8D81, 0xFF607F70, 0xFF648B79, 0xFF527766, 0x00A7BEB4, 0x00FAFFFC, 0x00FFFDFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFAFE, 0x00F9FFFF, 0xFF98B6AC, 0xFF43695C, 0xFF658B7E, 0xFF658B7E, 0xFF356152, 0xFF1E5140, 0xFF517A6A, 0xFF477060, 0xFF446D5D, 0xFF1A4534, 0xFF436E5D,
        0xFF507C6B, 0xFF214D3C, 0xFF214D3C, 0xFF537867, 0xFF405C4E, 0xFF4E5E54, 0xFFADAFAA, 0xFFEBE7E4, 0xFFD8D3D0, 0xFFCDC9C6, 0xFFBFBEBA, 0xFFDBD6D3, 0xFFE3DFDC, 0xFFDADBD6, 0xFF666C68, 0xFF414E47, 0xFF364741, 0xFF435851, 0xFF334A42, 0xFF545857,
        0xFF495853, 0xFF2A443B, 0xFF29433A, 0xFF293C36, 0xFF2F423C, 0xFF26443A, 0xFF325B4D, 0xFF5A8373, 0xFF5D8676, 0xFF537C6C, 0xFF5B8474, 0xFF668F7F, 0xFF557E6E, 0xFF436C5C, 0xFF3A6353, 0xFF36614E, 0xFF3A6351, 0xFF315A48, 0xFF37604E, 0xFF1F4634,
        0xFF3E6553, 0xFF4D7261, 0xFF658A79, 0xFF5F8674, 0xFF5F8674, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF5F8674, 0xFF5F8674, 0xFF638574, 0xFF5F8673, 0xFF486F5C, 0xFF173324, 0xFF3D4D42, 0xFF59635A, 0xFF26382C, 0xFF183425, 0xFF213B30,
        0xFF1B3326, 0xFF21352A, 0xFF32463B, 0xFF344A3D, 0xFF233E2F, 0xFF1C3B29, 0xFF537461, 0xFF5A8470, 0xFF587968, 0xFF384E41, 0xFF415146, 0xFF455248, 0xFF1D3125, 0xFF446051, 0xFF365B49, 0xFF50715E, 0xFF345542, 0xFF527463, 0xFF577768, 0xFF2E4A3E,
        0xFF5C716A, 0xFF65746F, 0xFF2C3635, 0xFF0A1818, 0xFF111C16, 0xFF1E241A, 0xFF3B3D2F, 0xFF1E1F11, 0xFF191A12, 0xFF191E1A, 0xFF1C2124, 0xFF20221F, 0xFF1E231F, 0xFF191F1B, 0xFF1A201C, 0xFF252724, 0xFF252724, 0xFF1D221E, 0xFF1A201C, 0xFF3A5C4C,
        0xFF2E4A3C, 0xFF0F2318, 0xFF16211B, 0xFF1B201C, 0xFF202221, 0xFF1E201F, 0xFF1C201F, 0xFF112A24, 0xFF233C36, 0xFF314A44, 0xFF2D423B, 0xFF1F2E29, 0xFF181E1C, 0xFF1A1917, 0xFF1E1818, 0xFF171E17, 0xFF151918, 0xFF191919, 0xFF222218, 0xFF4F513C,
        0xFF40432E, 0xFF141A10, 0xFF161B1F, 0xFF121F18, 0xFF4C5D55, 0xFF496056, 0xFF355546, 0xFF5C8170, 0xFF537A68, 0xFF587F6C, 0xFF567D6A, 0xFF56806C, 0xFF547E6A, 0xFF5B826F, 0xFF59806D, 0xFF587D6B, 0xFF5E8371, 0xFF608573, 0xFF628775, 0xFF5F8B7C,
        0xFF568072, 0xFF587E71, 0xFF5F8377, 0xFF5D7F71, 0xFF47695B, 0xFF406254, 0xFF426658, 0xFF477165, 0xFF487268, 0xFF436A65, 0xFF59807D, 0xFF49706D, 0xFF456D65, 0xFF315B4D, 0xFF396451, 0xFF35564B, 0xFF46675C, 0xFF57786D, 0xFF53776B, 0xFF41675A,
        0xFF355F51, 0xFF366555, 0xFF346454, 0xFF42615B, 0xFF587771, 0xFF5C7B73, 0xFF54756C, 0xFF42665A, 0xFF3E6256, 0xFF3F6556, 0xFF395F50, 0xFF466A5C, 0xFF618675, 0xFF5E8372, 0xFF416655, 0xFF375C4A, 0xFF3C614F, 0xFF3C624D, 0xFF395F4A, 0xFF2F503D,
        0xFF3B5F51, 0xFF294D3F, 0xFF2B412A, 0xFF6E6E48, 0xFF968551, 0xFFA88F57, 0xFFA28953, 0xFF7D713D, 0xFF30391E, 0xFF38483E, 0xFF777870, 0xFF8E887C, 0xFFD6D6CE, 0xFFD9DCD3, 0xFFA7A493, 0xFFA29A8D, 0xFFA1998C, 0xFFA69E91, 0xFFA59D90, 0xFFACA497,
        0xFFB8B0A3, 0xFFB5ADA0, 0xFFB3AB9E, 0xFFB2AD99, 0xFFAFAC99, 0xFFADAC9A, 0xFF989687, 0xFF88897B, 0xFFA29F96, 0xFFCBC8C1, 0xFFEDEAE3, 0xFFCCDCD1, 0xFF7C9285, 0xFF638374, 0xFF648978, 0xFF4F7161, 0x009FB4AB, 0x00FAFFFC, 0x00FFFEFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFF9FD, 0x00F7FFFD, 0xFF96B4AA, 0xFF466C5F, 0xFF668C7F, 0xFF5E8477, 0xFF305C4D, 0xFF205342, 0xFF406959, 0xFF1B4434, 0xFF416A5A, 0xFF3C6756, 0xFF4E7968,
        0xFF507C6B, 0xFF285443, 0xFF1B4736, 0xFF5E8372, 0xFF577365, 0xFF57675D, 0xFFBABBB6, 0xFFE3DBD9, 0xFFD8CECD, 0xFFD3CBC9, 0xFFC0BBB8, 0xFFCDC5C3, 0xFFDAD5D2, 0xFFD4D3CF, 0xFF8A8F8B, 0xFF747F79, 0xFF5C6B66, 0xFF4B5E58, 0xFF556A63, 0xFF92898C,
        0xFF838584, 0xFF4C5854, 0xFF4D5955, 0xFF7F8583, 0xFF9DA3A1, 0xFF657670, 0xFF39544B, 0xFF608979, 0xFF5C8575, 0xFF5D8676, 0xFF618A7A, 0xFF578070, 0xFF5B8474, 0xFF5F8878, 0xFF3B6454, 0xFF386350, 0xFF2F5846, 0xFF2B5442, 0xFF3B6452, 0xFF254C3A,
        0xFF426957, 0xFF4F7463, 0xFF6A8F7E, 0xFF5F8674, 0xFF5F8674, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF5F8674, 0xFF5F8674, 0xFF648675, 0xFF628976, 0xFF3E6552, 0xFF1A3627, 0xFF506055, 0xFF515B52, 0xFF1E3024, 0xFF153122, 0xFF213D31,
        0xFF203A2D, 0xFF21372B, 0xFF2A3E33, 0xFF34483C, 0xFF2D4537, 0xFF163321, 0xFF486755, 0xFF58826E, 0xFF5F806F, 0xFF405246, 0xFF5C665D, 0xFF7B827A, 0xFF4A5A4F, 0xFF4A6657, 0xFF325745, 0xFF4A6B58, 0xFF335441, 0xFF436554, 0xFF517162, 0xFF395549,
        0xFF526760, 0xFF576661, 0xFF273130, 0xFF10181A, 0xFF171D19, 0xFF22251C, 0xFF3A3C2E, 0xFF1A1C0E, 0xFF161910, 0xFF181D19, 0xFF1C2023, 0xFF121D17, 0xFF18211C, 0xFF1D231F, 0xFF202924, 0xFF25302A, 0xFF212C26, 0xFF1A231E, 0xFF1A201C, 0xFF233F31,
        0xFF21392C, 0xFF0F2016, 0xFF141F19, 0xFF171C18, 0xFF1B1B1B, 0xFF141213, 0xFF121011, 0xFF0F241F, 0xFF1E332E, 0xFF293E37, 0xFF263933, 0xFF1F2B27, 0xFF1A201C, 0xFF1B1A18, 0xFF1C1616, 0xFF161D16, 0xFF151918, 0xFF191919, 0xFF222218, 0xFF50523D,
        0xFF41442F, 0xFF13190F, 0xFF14191D, 0xFF15221B, 0xFF506159, 0xFF52695F, 0xFF3B5B4C, 0xFF5C8170, 0xFF537A68, 0xFF577E6B, 0xFF547B68, 0xFF567D6A, 0xFF59806D, 0xFF608774, 0xFF597E6C, 0xFF597E6C, 0xFF628775, 0xFF5E806F, 0xFF5A7C6B, 0xFF578271,
        0xFF5E8777, 0xFF658B7C, 0xFF64887A, 0xFF608478, 0xFF65897D, 0xFF618A7C, 0xFF5E887A, 0xFF679080, 0xFF678D80, 0xFF5E7F74, 0xFF68877F, 0xFF628277, 0xFF66887A, 0xFF5D8472, 0xFF67917B, 0xFF658675, 0xFF6B8A7A, 0xFF6B8778, 0xFF6A8677, 0xFF648574,
        0xFF608573, 0xFF618C78, 0xFF5C8C76, 0xFF698174, 0xFF6D877A, 0xFF658071, 0xFF678676, 0xFF668776, 0xFF6D907C, 0xFF6A907B, 0xFF638974, 0xFF5A7E70, 0xFF5E8372, 0xFF456A59, 0xFF416655, 0xFF385D4B, 0xFF2B503E, 0xFF3D634E, 0xFF395F4A, 0xFF294A37,
        0xFF294D3F, 0xFF2A4E40, 0xFF2F452E, 0xFF696943, 0xFF9B8A56, 0xFFA48B53, 0xFF9D844E, 0xFF7B6F3B, 0xFF2D361B, 0xFF37473D, 0xFF777870, 0xFF8E887C, 0xFFD6D6CE, 0xFFD9DCD3, 0xFFA6A392, 0xFF999184, 0xFF9E9689, 0xFFA8A093, 0xFFA9A194, 0xFFADA598,
        0xFFB3AB9E, 0xFFAEA699, 0xFFABA396, 0xFFAEAD99, 0xFFB0AF9B, 0xFFB8B5A4, 0xFFABA99C, 0xFFA09E92, 0xFFA7A79D, 0xFFAEB1A8, 0xFFB4B7B0, 0xFF92A89B, 0xFF698475, 0xFF658576, 0xFF678C7B, 0xFF506F60, 0x009CB0A7, 0x00F9FFFB, 0x00FFFFFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFBFF, 0x04F3FCF9, 0xFF8FADA3, 0xFF446A5D, 0xFF698F82, 0xFF61877A, 0xFF305C4D, 0xFF1F5241, 0xFF50796B, 0xFF396254, 0xFF487163, 0xFF376153, 0xFF456F61,
        0xFF537F70, 0xFF2E5A4B, 0xFF1E4A3B, 0xFF587A6C, 0xFF476357, 0xFF4F5C55, 0xFFAFB1AE, 0xFFE4DEDE, 0xFFDDD4D5, 0xFFDBD5D5, 0xFFDAD6D5, 0xFFD9D4D1, 0xFFE8E7E3, 0xFFC3C5C0, 0xFF8B918D, 0xFFA2AFA8, 0xFF6A7D77, 0xFF3C534B, 0xFF728C83, 0xFF9A9899,
        0xFF868F8C, 0xFF566761, 0xFF5F6E69, 0xFFA4AAA8, 0xFFBAC0BE, 0xFF7A8984, 0xFF455C54, 0xFF517A6A, 0xFF5E8777, 0xFF628B7B, 0xFF5C8575, 0xFF537C6C, 0xFF5D8676, 0xFF668F7F, 0xFF4C7565, 0xFF3C6754, 0xFF295240, 0xFF28513F, 0xFF3C6553, 0xFF244B39,
        0xFF406755, 0xFF4B705F, 0xFF658A79, 0xFF5E8573, 0xFF5F8674, 0xFF5F8674, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF608271, 0xFF668D7A, 0xFF3B624F, 0xFF163223, 0xFF536358, 0xFF49534A, 0xFF213327, 0xFF173324, 0xFF153327,
        0xFF1F3B2D, 0xFF1F372A, 0xFF25392E, 0xFF314539, 0xFF344A3D, 0xFF1C3726, 0xFF506D5B, 0xFF5B8571, 0xFF618070, 0xFF5B6B60, 0xFFB0B3AC, 0xFFD4D7D0, 0xFF7A867C, 0xFF445F50, 0xFF3F6452, 0xFF3C5D4A, 0xFF4A6B58, 0xFF456756, 0xFF345445, 0xFF375347,
        0xFF5F746D, 0xFF5B6A65, 0xFF283231, 0xFF131418, 0xFF1C1E1B, 0xFF282B22, 0xFF414536, 0xFF212516, 0xFF1B1E15, 0xFF191B18, 0xFF161A1D, 0xFF13241C, 0xFF15201A, 0xFF151A16, 0xFF1A201C, 0xFF1E2D26, 0xFF1A2B23, 0xFF132019, 0xFF131915, 0xFF14251B,
        0xFF1C2D23, 0xFF18251C, 0xFF19221D, 0xFF151A16, 0xFF1A1A1A, 0xFF191718, 0xFF231F20, 0xFF14201E, 0xFF1B2A25, 0xFF20312B, 0xFF20312B, 0xFF1E2B24, 0xFF1A231E, 0xFF1C1E19, 0xFF1C1B17, 0xFF181D17, 0xFF171918, 0xFF1A1819, 0xFF232017, 0xFF51503C,
        0xFF41432E, 0xFF14170E, 0xFF15181D, 0xFF14211A, 0xFF4B5C54, 0xFF586F65, 0xFF3B5B4C, 0xFF547968, 0xFF527967, 0xFF577E6B, 0xFF587F6C, 0xFF537A67, 0xFF59806D, 0xFF638876, 0xFF5A7F6D, 0xFF5D8270, 0xFF618372, 0xFF4C6E5D, 0xFF466857, 0xFF3F604F,
        0xFF476857, 0xFF577968, 0xFF5E8573, 0xFF587E6F, 0xFF5A8373, 0xFF598274, 0xFF5F887A, 0xFF5D7F6E, 0xFF5F8170, 0xFF5D7C6C, 0xFF62816F, 0xFF607F6D, 0xFF658472, 0xFF60836D, 0xFF648771, 0xFF658570, 0xFF678470, 0xFF63806C, 0xFF647F6C, 0xFF64816D,
        0xFF63836E, 0xFF62856F, 0xFF5E846D, 0xFF6A8272, 0xFF6A8574, 0xFF627D6A, 0xFF63806C, 0xFF64846D, 0xFF678770, 0xFF678971, 0xFF5E8067, 0xFF628678, 0xFF496E5D, 0xFF3F6453, 0xFF3C6150, 0xFF2A4F3D, 0xFF2E5341, 0xFF3E644F, 0xFF325843, 0xFF365744,
        0xFF2D5143, 0xFF2E5244, 0xFF324831, 0xFF63633D, 0xFF9B8A56, 0xFFA48B53, 0xFF9E854F, 0xFF807440, 0xFF2F381D, 0xFF38483E, 0xFF787971, 0xFF8D877B, 0xFFD6D6CE, 0xFFDADDD4, 0xFFA5A291, 0xFF928B79, 0xFF918A78, 0xFF9A9381, 0xFFA19A88, 0xFFABA492,
        0xFFB1AA98, 0xFFA49D8B, 0xFF9C9583, 0xFF9F9C89, 0xFFA39D8D, 0xFFB3AD9F, 0xFFB7B3A7, 0xFFB4B4A8, 0xFFAAB0A6, 0xFF8A968C, 0xFF6D7D72, 0xFF6C8778, 0xFF607F6F, 0xFF698B7B, 0xFF6D8F7F, 0xFF536F61, 0x009DAEA6, 0x00FBFFFD, 0x00FFFFFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFBFF, 0x00F5FEFB, 0xFF8EACA2, 0xFF42685B, 0xFF6A9083, 0xFF678D80, 0xFF335F50, 0xFF1D503F, 0xFF497264, 0xFF4F786A, 0xFF386153, 0xFF164032, 0xFF275143,
        0xFF3C6859, 0xFF1B4738, 0xFF1B4738, 0xFF567669, 0xFF506A5F, 0xFF79867F, 0xFFA7A9A6, 0xFFF1EDEC, 0xFFE2DEDD, 0xFFD6D6D4, 0xFFDEE0DD, 0xFFD7D8D3, 0xFFE5E7E2, 0xFFCED5CE, 0xFF65726B, 0xFF5F736A, 0xFF415B52, 0xFF456359, 0xFF4E6E63, 0xFF4A5D57,
        0xFF4A685E, 0xFF406458, 0xFF3B5B50, 0xFF51665F, 0xFF556660, 0xFF577168, 0xFF638479, 0xFF355E4E, 0xFF507969, 0xFF5B8474, 0xFF5E8777, 0xFF648D7D, 0xFF638C7C, 0xFF5A8373, 0xFF4F7868, 0xFF396451, 0xFF264F3D, 0xFF2A5341, 0xFF3C6553, 0xFF204735,
        0xFF3F6654, 0xFF496E5D, 0xFF618675, 0xFF5E8573, 0xFF5E8573, 0xFF5F8674, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF618372, 0xFF648B78, 0xFF3E6552, 0xFF132F20, 0xFF506055, 0xFF454F46, 0xFF1F3125, 0xFF173324, 0xFF133326,
        0xFF1F3B2D, 0xFF1C3427, 0xFF263A2F, 0xFF314539, 0xFF304639, 0xFF1A3222, 0xFF506B5A, 0xFF5F8975, 0xFF5E7A6B, 0xFF445046, 0xFF7A7974, 0xFF8E8D88, 0xFF586259, 0xFF3A5244, 0xFF5E8371, 0xFF385946, 0xFF436451, 0xFF426453, 0xFF274738, 0xFF3A564A,
        0xFF738881, 0xFF62716C, 0xFF273130, 0xFF19171C, 0xFF1F1E1C, 0xFF26271F, 0xFF3A3E2F, 0xFF1A2111, 0xFF181E14, 0xFF1D1F1C, 0xFF1D1E22, 0xFF15201A, 0xFF1D1F1C, 0xFF1E1A19, 0xFF1C1B19, 0xFF151E19, 0xFF0E1B14, 0xFF101612, 0xFF1D1918, 0xFF171E17,
        0xFF222C24, 0xFF212B23, 0xFF1C2520, 0xFF121814, 0xFF191B1A, 0xFF1B1B1B, 0xFF282627, 0xFF161A19, 0xFF181E1C, 0xFF1A2521, 0xFF1C2922, 0xFF19261F, 0xFF162119, 0xFF171E17, 0xFF181D17, 0xFF191E18, 0xFF181A19, 0xFF1B191A, 0xFF232017, 0xFF504F3B,
        0xFF40422D, 0xFF15180F, 0xFF171A1F, 0xFF111E17, 0xFF45564E, 0xFF61786E, 0xFF406051, 0xFF507564, 0xFF537A68, 0xFF577E6B, 0xFF5B826F, 0xFF577E6B, 0xFF5A7F6D, 0xFF5F8472, 0xFF597E6C, 0xFF638574, 0xFF5B7D6C, 0xFF3D5E4D, 0xFF3E5F4E, 0xFF546956,
        0xFF465F4A, 0xFF4B6C57, 0xFF5F8973, 0xFF608B77, 0xFF628B79, 0xFF587E6F, 0xFF608274, 0xFF638577, 0xFF638575, 0xFF628775, 0xFF638974, 0xFF5F8570, 0xFF628873, 0xFF648773, 0xFF638574, 0xFF5D8270, 0xFF5C816F, 0xFF618674, 0xFF628775, 0xFF628473,
        0xFF638473, 0xFF628171, 0xFF607F6F, 0xFF5F8473, 0xFF638877, 0xFF628776, 0xFF608573, 0xFF658A78, 0xFF648773, 0xFF698C78, 0xFF5F826E, 0xFF4F7365, 0xFF3E6352, 0xFF456A59, 0xFF1E4332, 0xFF335846, 0xFF486D5B, 0xFF193F2A, 0xFF426853, 0xFF3B5C49,
        0xFF395D4F, 0xFF2E5244, 0xFF354B34, 0xFF676741, 0xFF998854, 0xFFA68D55, 0xFFA28953, 0xFF827642, 0xFF2E371C, 0xFF35453B, 0xFF74756D, 0xFF8A8478, 0xFFD5D5CD, 0xFFDCDFD6, 0xFFA6A392, 0xFF8E8774, 0xFF857E6B, 0xFF8A8370, 0xFF97907D, 0xFFA8A18E,
        0xFFAFA895, 0xFFA09986, 0xFF97907D, 0xFF97907E, 0xFF968D7E, 0xFFA59B8F, 0xFFB1A99E, 0xFFB5B5AB, 0xFFAAB4A9, 0xFF82968A, 0xFF5E786B, 0xFF678877, 0xFF648574, 0xFF6A8C7C, 0xFF6C8C7D, 0xFF536D60, 0x009CADA5, 0x00FBFFFD, 0x00FFFFFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFF8FC, 0x00F6FFFC, 0xFF93B1A7, 0xFF446A5D, 0xFF698F82, 0xFF658B7E, 0xFF2C5849, 0xFF0F4231, 0xFF194234, 0xFF1E4739, 0xFF194234, 0xFF184234, 0xFF113B2D,
        0xFF184435, 0xFF0C3829, 0xFF1C4839, 0xFF547467, 0xFF768D83, 0xFFBCC9C2, 0xFF959A96, 0xFFE8E8E6, 0xFFE3E3E1, 0xFFCFD4D0, 0xFFC5CEC9, 0xFFD1D6D0, 0xFFD7E1D9, 0xFFCBD8CF, 0xFF5C6D65, 0xFF72897F, 0xFF506E64, 0xFF65867B, 0xFF5F8377, 0xFF5B8476,
        0xFF548474, 0xFF528976, 0xFF4D7D6D, 0xFF62867A, 0xFF5C7C71, 0xFF4E7266, 0xFF335F50, 0xFF3D6656, 0xFF386151, 0xFF507969, 0xFF628B7B, 0xFF5C8575, 0xFF5F8878, 0xFF5F8878, 0xFF497262, 0xFF37624F, 0xFF295240, 0xFF315A48, 0xFF406957, 0xFF224937,
        0xFF446B59, 0xFF4F7463, 0xFF658A79, 0xFF5E8573, 0xFF5E8573, 0xFF5F8674, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF658776, 0xFF5F8673, 0xFF416855, 0xFF143021, 0xFF4F5F54, 0xFF424C43, 0xFF182A1E, 0xFF153122, 0xFF143427,
        0xFF173627, 0xFF132B1E, 0xFF273D31, 0xFF35493D, 0xFF304639, 0xFF193121, 0xFF516C5B, 0xFF5D8773, 0xFF5C7869, 0xFF4D574E, 0xFF85827D, 0xFF898480, 0xFF535A52, 0xFF284032, 0xFF577C6A, 0xFF4B6C59, 0xFF244532, 0xFF3A5C4B, 0xFF446455, 0xFF567266,
        0xFF7E938C, 0xFF5D6C67, 0xFF283231, 0xFF191219, 0xFF1F1B1A, 0xFF25261E, 0xFF383F2F, 0xFF1A2111, 0xFF161C12, 0xFF181A17, 0xFF15161A, 0xFF191E1A, 0xFF1E1818, 0xFF201114, 0xFF1F1516, 0xFF1B1D1A, 0xFF151B17, 0xFF181715, 0xFF27181B, 0xFF181914,
        0xFF1E201B, 0xFF161B15, 0xFF151B17, 0xFF141A16, 0xFF1F2322, 0xFF1B1D1C, 0xFF202020, 0xFF1D1B1C, 0xFF1D1D1D, 0xFF1D231F, 0xFF1E2923, 0xFF1B2821, 0xFF152219, 0xFF152018, 0xFF1A241C, 0xFF1A1F19, 0xFF191B1A, 0xFF1B191A, 0xFF232017, 0xFF4E4D39,
        0xFF3F412C, 0xFF15180F, 0xFF181B20, 0xFF17241D, 0xFF495A52, 0xFF70877D, 0xFF4D6D5E, 0xFF557A69, 0xFF577E6C, 0xFF567D6A, 0xFF59806D, 0xFF5E8371, 0xFF597E6C, 0xFF597E6C, 0xFF577C6A, 0xFF658776, 0xFF587A69, 0xFF375847, 0xFF426352, 0xFF43503C,
        0xFF435944, 0xFF50725A, 0xFF59856C, 0xFF527F68, 0xFF5E8975, 0xFF5E8070, 0xFF648374, 0xFF61847D, 0xFF5D8378, 0xFF5C8776, 0xFF5A8773, 0xFF56836F, 0xFF588370, 0xFF618778, 0xFF608478, 0xFF5C8678, 0xFF558172, 0xFF5C8879, 0xFF5A8677, 0xFF578173,
        0xFF608478, 0xFF638378, 0xFF69847B, 0xFF56867A, 0xFF588679, 0xFF59877A, 0xFF537F72, 0xFF5E887A, 0xFF578072, 0xFF5B8474, 0xFF4C7565, 0xFF3B5F51, 0xFF496E5D, 0xFF183D2C, 0xFF375C4B, 0xFF466B59, 0xFF204533, 0xFF395F4A, 0xFF395F4A, 0xFF2C4D3A,
        0xFF3C6052, 0xFF2C5042, 0xFF3F553E, 0xFF767650, 0xFF9C8B57, 0xFFA78E56, 0xFF9F8650, 0xFF807440, 0xFF283116, 0xFF2E3E34, 0xFF6E6F67, 0xFF857F73, 0xFFD4D4CC, 0xFFDDE0D7, 0xFFA8A594, 0xFF9E9880, 0xFF98927A, 0xFFA09A82, 0xFFA9A38B, 0xFFB3AD95,
        0xFFB6B098, 0xFFAEA890, 0xFFAFA991, 0xFFB2A698, 0xFFA79B8D, 0xFFA89C90, 0xFFA69E93, 0xFFA3A399, 0xFF9AA89B, 0xFF7B9687, 0xFF608272, 0xFF658776, 0xFF618372, 0xFF638575, 0xFF688879, 0xFF526C5F, 0x009AABA3, 0x00FBFFFD, 0x00FBFBF9, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x00F7FFFE, 0xFF8AA89E, 0xFF456E60, 0xFF668C7F, 0xFF608478, 0xFF497365, 0xFF2C5C4C, 0xFF325B4B, 0xFF345D4D, 0xFF325B4B, 0xFF325B4B, 0xFF335C4C,
        0xFF2E5747, 0xFF2B5444, 0xFF335C4C, 0xFF517D6C, 0xFF487362, 0xFF739789, 0xFF6D8B7F, 0xFF7A968A, 0xFF7B978B, 0xFF5F7D71, 0xFF749487, 0xFF58766A, 0xFF628074, 0xFF719186, 0xFF55766B, 0xFF42665C, 0xFF466C61, 0xFF628A7F, 0xFF5F877E, 0xFF638E7B,
        0xFF5D8875, 0xFF5C8774, 0xFF5E8777, 0xFF5F8576, 0xFF62887B, 0xFF5F8377, 0xFF517569, 0xFF325B4B, 0xFF446D5D, 0xFF365F4F, 0xFF557E6E, 0xFF638C7C, 0xFF628B7B, 0xFF618A7A, 0xFF4E7767, 0xFF3B6250, 0xFF2D5442, 0xFF325947, 0xFF3B6250, 0xFF264D3B,
        0xFF416856, 0xFF4C7361, 0xFF618876, 0xFF5C8573, 0xFF5C8573, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5C8573, 0xFF5C8573, 0xFF618674, 0xFF678877, 0xFF435F50, 0xFF192F22, 0xFF465A4E, 0xFF425448, 0xFF203428, 0xFF1B3124, 0xFF1E3229,
        0xFF1D372A, 0xFF143324, 0xFF243C2F, 0xFF3A473D, 0xFF323E34, 0xFF284030, 0xFF30533F, 0xFF3B5D4F, 0xFF3C564B, 0xFF4C5B54, 0xFFA0A9A4, 0xFFB9C2BD, 0xFF62716A, 0xFF314B40, 0xFF385A4C, 0xFF425A4C, 0xFF375243, 0xFF314D3F, 0xFF385748, 0xFF4F6B5D,
        0xFF60776D, 0xFF4C5D55, 0xFF202F28, 0xFF101721, 0xFF181E1A, 0xFF2F3322, 0xFF474838, 0xFF221F18, 0xFF1C1A1D, 0xFF191A1C, 0xFF181D19, 0xFF1B1D18, 0xFF161B15, 0xFF181F18, 0xFF18211C, 0xFF121B16, 0xFF141915, 0xFF1B1B1B, 0xFF1D1B1C, 0xFF1B1B1B,
        0xFF161817, 0xFF161C1A, 0xFF141D1A, 0xFF101916, 0xFF131C19, 0xFF1A201E, 0xFF191D1C, 0xFF211B1B, 0xFF1D1D1B, 0xFF242D28, 0xFF17261F, 0xFF16271F, 0xFF112019, 0xFF151E19, 0xFF1A1C19, 0xFF1B1A16, 0xFF171D1B, 0xFF15201C, 0xFF1D2317, 0xFF504F3B,
        0xFF454430, 0xFF141C11, 0xFF0C1E1E, 0xFF13221F, 0xFF4F625C, 0xFF5A746B, 0xFF426255, 0xFF5C8170, 0xFF557C69, 0xFF59806B, 0xFF527863, 0xFF5A7A6D, 0xFF5F8171, 0xFF608574, 0xFF5D8472, 0xFF567D6A, 0xFF406251, 0xFF375644, 0xFF567160, 0xFF486352,
        0xFF41604E, 0xFF517362, 0xFF5D8471, 0xFF5D8674, 0xFF5C8573, 0xFF5B8270, 0xFF628678, 0xFF5E8371, 0xFF5E8371, 0xFF5E8371, 0xFF5E8371, 0xFF5E8371, 0xFF5E8371, 0xFF5E8371, 0xFF5E8371, 0xFF638574, 0xFF638574, 0xFF638574, 0xFF638574, 0xFF628473,
        0xFF618372, 0xFF608271, 0xFF5F8170, 0xFF5B806F, 0xFF618675, 0xFF628776, 0xFF608574, 0xFF658A79, 0xFF608574, 0xFF527766, 0xFF355A49, 0xFF4A6F5E, 0xFF183D2C, 0xFF2E5341, 0xFF476C5A, 0xFF1F4530, 0xFF315742, 0xFF4E745D, 0xFF0E341D, 0xFF405E46,
        0xFF2E503F, 0xFF193F30, 0xFF2C452F, 0xFF6C704B, 0xFF9A8955, 0xFF9F854A, 0xFFAC8F55, 0xFF85713E, 0xFF36391E, 0xFF465349, 0xFF727469, 0xFFACA698, 0xFFDFDCD5, 0xFFDBD8CF, 0xFFB3A797, 0xFFA49C8F, 0xFFA9A194, 0xFFA49C8F, 0xFFACA497, 0xFFB1A99C,
        0xFFB1A99C, 0xFFB9B1A4, 0xFFB6AEA1, 0xFFB5AEA6, 0xFFA8AA9F, 0xFF979F92, 0xFF949A8C, 0xFF9C9C90, 0xFFA3A598, 0xFF7B8C79, 0xFF66846C, 0xFF638272, 0xFF678877, 0xFF5F8473, 0xFF648978, 0xFF4F6F60, 0x0098ADA4, 0x00F8FFFD, 0x00F8FAF7, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x00F7FFFE, 0xFF89A79D, 0xFF456E60, 0xFF648A7D, 0xFF64887C, 0xFF5E887A, 0xFF568676, 0xFF658E7E, 0xFF628B7B, 0xFF5E8777, 0xFF5F8878, 0xFF638C7C,
        0xFF638C7C, 0xFF618A7A, 0xFF628B7B, 0xFF5F927F, 0xFF4E7D6B, 0xFF568170, 0xFF547A6B, 0xFF5E8475, 0xFF588171, 0xFF537E6D, 0xFF689483, 0xFF618577, 0xFF54786A, 0xFF54786A, 0xFF53796C, 0xFF557E70, 0xFF547C71, 0xFF5B8579, 0xFF5A8478, 0xFF578470,
        0xFF598672, 0xFF5F8A77, 0xFF5E8978, 0xFF5B8474, 0xFF5E8779, 0xFF638C7E, 0xFF63897C, 0xFF4D7666, 0xFF315A4A, 0xFF416A5A, 0xFF3B6454, 0xFF578070, 0xFF5F8878, 0xFF628B7B, 0xFF497262, 0xFF3B6250, 0xFF2C5341, 0xFF305745, 0xFF39604E, 0xFF254C3A,
        0xFF406755, 0xFF4B7260, 0xFF618876, 0xFF5C8573, 0xFF5C8573, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5C8573, 0xFF5C8573, 0xFF608573, 0xFF678877, 0xFF435F50, 0xFF182E21, 0xFF44584C, 0xFF3E5044, 0xFF1C3024, 0xFF182E21, 0xFF172B22,
        0xFF152F22, 0xFF143324, 0xFF243C2F, 0xFF3D4A40, 0xFF404C42, 0xFF3C5444, 0xFF4B6E5A, 0xFF436555, 0xFF456455, 0xFF3A5245, 0xFF596D62, 0xFF5F7368, 0xFF324A3D, 0xFF2F4E3F, 0xFF486A5A, 0xFF344F40, 0xFF3F5A4B, 0xFF425E50, 0xFF3D5C4D, 0xFF486456,
        0xFF4B6258, 0xFF42534B, 0xFF415049, 0xFF2B343D, 0xFF1E2722, 0xFF252C1A, 0xFF3D402F, 0xFF1B1C14, 0xFF18181A, 0xFF141819, 0xFF181E1A, 0xFF1E231D, 0xFF171E17, 0xFF19231B, 0xFF1F2A24, 0xFF1B241F, 0xFF181C1B, 0xFF1D1D1D, 0xFF221E1F, 0xFF1D1B1C,
        0xFF1B1B1B, 0xFF1D1F1E, 0xFF1A1E1D, 0xFF141A18, 0xFF171D1B, 0xFF1D2120, 0xFF1B1F1E, 0xFF1C1817, 0xFF171916, 0xFF202B25, 0xFF1A2922, 0xFF192A22, 0xFF14211A, 0xFF181E1A, 0xFF232321, 0xFF181713, 0xFF151B19, 0xFF151E1B, 0xFF1A2014, 0xFF494834,
        0xFF403F2B, 0xFF1C2419, 0xFF213333, 0xFF263532, 0xFF576A64, 0xFF536D64, 0xFF3A5A4D, 0xFF577C6B, 0xFF517865, 0xFF587E69, 0xFF567C67, 0xFF58786B, 0xFF5D7F6F, 0xFF547968, 0xFF5A816F, 0xFF4B725F, 0xFF395B4A, 0xFF436250, 0xFF3A5544, 0xFF466150,
        0xFF506F5D, 0xFF658776, 0xFF648B78, 0xFF5B8472, 0xFF5D8674, 0xFF5C8371, 0xFF5A7E70, 0xFF608573, 0xFF608573, 0xFF608573, 0xFF608573, 0xFF608573, 0xFF608573, 0xFF608573, 0xFF608573, 0xFF628473, 0xFF638574, 0xFF638574, 0xFF638574, 0xFF628473,
        0xFF628473, 0xFF618372, 0xFF608271, 0xFF5F8473, 0xFF618675, 0xFF5E8372, 0xFF5F8473, 0xFF5F8473, 0xFF587D6C, 0xFF305544, 0xFF496E5D, 0xFF173C2B, 0xFF365B4A, 0xFF4B705E, 0xFF1B402E, 0xFF365C47, 0xFF4C725D, 0xFF133922, 0xFF3A6049, 0xFF436149,
        0xFF183A29, 0xFF1E4435, 0xFF3A533D, 0xFF575B36, 0xFF968551, 0xFFB3995E, 0xFF9C7F45, 0xFF8A7643, 0xFF45492E, 0xFF5F6C62, 0xFF8D9085, 0xFFBEB8AA, 0xFFEAE7E0, 0xFFDEDBD2, 0xFFAFA393, 0xFFA19A8A, 0xFFA7A090, 0xFFAAA393, 0xFFB0A999, 0xFFB2AB9B,
        0xFFB0A999, 0xFFB3AC9C, 0xFFB1AA9A, 0xFFADA399, 0xFFA6A497, 0xFF9B9F91, 0xFF9CA092, 0xFFA3A397, 0xFFA5A89D, 0xFF798B7B, 0xFF5D7E69, 0xFF698576, 0xFF6B8C7B, 0xFF618675, 0xFF648978, 0xFF4E6E5F, 0x0093A89F, 0x00F8FFFD, 0x00FEFFFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFAFC, 0x04F4FFFB, 0xFF8CAAA0, 0xFF497264, 0xFF63897C, 0xFF618579, 0xFF5C8678, 0xFF619181, 0xFF5B8474, 0xFF5E8777, 0xFF628B7B, 0xFF608979, 0xFF5C8575,
        0xFF5C8575, 0xFF5F8878, 0xFF618A7A, 0xFF578A77, 0xFF5E8F7C, 0xFF5A8977, 0xFF5C8877, 0xFF659180, 0xFF608F7D, 0xFF639683, 0xFF5B927D, 0xFF669180, 0xFF669180, 0xFF648F7E, 0xFF628C7E, 0xFF5E887A, 0xFF5E887A, 0xFF5E887C, 0xFF5D877B, 0xFF598672,
        0xFF5B8874, 0xFF5E8B77, 0xFF5B8776, 0xFF568271, 0xFF598375, 0xFF5D8779, 0xFF5D8779, 0xFF5A8373, 0xFF4F7868, 0xFF2E5747, 0xFF3E6757, 0xFF396252, 0xFF567F6F, 0xFF618A7A, 0xFF4B7464, 0xFF3B6250, 0xFF2C5341, 0xFF2F5644, 0xFF375E4C, 0xFF234A38,
        0xFF3F6654, 0xFF4B7260, 0xFF618876, 0xFF5C8573, 0xFF5C8573, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5C8573, 0xFF5C8573, 0xFF5F8472, 0xFF668776, 0xFF415D4E, 0xFF162C1F, 0xFF43574B, 0xFF3B4D41, 0xFF192D21, 0xFF172D20, 0xFF1C3027,
        0xFF193326, 0xFF1C3B2C, 0xFF273F32, 0xFF3A473D, 0xFF333F35, 0xFF233B2B, 0xFF3A5D49, 0xFF426454, 0xFF436555, 0xFF274939, 0xFF284839, 0xFF2B4B3C, 0xFF204232, 0xFF345646, 0xFF466858, 0xFF244031, 0xFF375344, 0xFF3E5D4E, 0xFF375647, 0xFF536F61,
        0xFF72897F, 0xFF5B6C64, 0xFF3A4740, 0xFF35424A, 0xFF202B25, 0xFF212A17, 0xFF3E4231, 0xFF1F2219, 0xFF191D1E, 0xFF1B2121, 0xFF242D28, 0xFF28322A, 0xFF18231B, 0xFF15221B, 0xFF1B2821, 0xFF18211E, 0xFF141817, 0xFF191718, 0xFF201A1E, 0xFF191718,
        0xFF1B191A, 0xFF1D1B1C, 0xFF191919, 0xFF161616, 0xFF181A19, 0xFF1D1F1E, 0xFF1B1D1C, 0xFF1C1C1A, 0xFF171C18, 0xFF1E2B24, 0xFF1A2B23, 0xFF172820, 0xFF111C16, 0xFF151A16, 0xFF252422, 0xFF1F1E1A, 0xFF171B1A, 0xFF131C19, 0xFF1E2418, 0xFF55523F,
        0xFF4D4A37, 0xFF252D22, 0xFF283838, 0xFF293835, 0xFF546761, 0xFF4A645B, 0xFF355548, 0xFF567B6A, 0xFF517664, 0xFF537964, 0xFF567C67, 0xFF567669, 0xFF5F8171, 0xFF5E8372, 0xFF385F4D, 0xFF3A614E, 0xFF345645, 0xFF143321, 0xFF45604F, 0xFF4D6857,
        0xFF547361, 0xFF648675, 0xFF5E8572, 0xFF557E6C, 0xFF5B8472, 0xFF5F8674, 0xFF618577, 0xFF618674, 0xFF618674, 0xFF618674, 0xFF618674, 0xFF618674, 0xFF618674, 0xFF618674, 0xFF618674, 0xFF608573, 0xFF608573, 0xFF608573, 0xFF618674, 0xFF618674,
        0xFF608573, 0xFF608573, 0xFF5F8472, 0xFF648978, 0xFF5A7F6E, 0xFF5A7F6E, 0xFF658A79, 0xFF527766, 0xFF2B503F, 0xFF4D7261, 0xFF204534, 0xFF345948, 0xFF517665, 0xFF183D2B, 0xFF375C4A, 0xFF496F5A, 0xFF193F2A, 0xFF365C45, 0xFF3A6049, 0xFF1B3921,
        0xFF3A5C4B, 0xFF315748, 0xFF233C26, 0xFF595D38, 0xFF9D8C58, 0xFFA88E53, 0xFFA5884E, 0xFF8D7C48, 0xFF4C5035, 0xFF5E6E63, 0xFF8B8E83, 0xFFB1AB9D, 0xFFE4E1DA, 0xFFDEDBD2, 0xFFAEA594, 0xFF99927F, 0xFF9A9380, 0xFFA29B88, 0xFFA9A28F, 0xFFAFA895,
        0xFFB0A996, 0xFFA7A08D, 0xFFA39C89, 0xFFAE9E8F, 0xFFA29B89, 0xFF90917F, 0xFF939587, 0xFF9B9B8F, 0xFF9FA59B, 0xFF75897D, 0xFF587A6A, 0xFF648071, 0xFF668776, 0xFF608272, 0xFF638877, 0xFF527162, 0x0093A89F, 0x00F8FFFC, 0x00FEFFFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x00F7FFFE, 0xFF8BA99F, 0xFF477062, 0xFF62887B, 0xFF668A7E, 0xFF5C8678, 0xFF619181, 0xFF608979, 0xFF608979, 0xFF638C7C, 0xFF648D7D, 0xFF638C7C,
        0xFF658E7E, 0xFF648D7D, 0xFF5D8676, 0xFF5B8673, 0xFF658E7C, 0xFF5F8473, 0xFF618675, 0xFF608775, 0xFF567F6D, 0xFF578672, 0xFF4C7D68, 0xFF507B68, 0xFF56816E, 0xFF598473, 0xFF5C8776, 0xFF578173, 0xFF608A7C, 0xFF5F897B, 0xFF598377, 0xFF5C8774,
        0xFF588370, 0xFF56836F, 0xFF578372, 0xFF5A8675, 0xFF5D897A, 0xFF598878, 0xFF558474, 0xFF5E8777, 0xFF648D7D, 0xFF436C5C, 0xFF325B4B, 0xFF3F6858, 0xFF376050, 0xFF5D8676, 0xFF4F7868, 0xFF3D6452, 0xFF2D5442, 0xFF305745, 0xFF375E4C, 0xFF234A38,
        0xFF3F6654, 0xFF4C7361, 0xFF638A78, 0xFF5C8573, 0xFF5C8573, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5C8573, 0xFF5C8573, 0xFF5D8270, 0xFF658675, 0xFF405C4D, 0xFF152B1E, 0xFF44584C, 0xFF3B4D41, 0xFF182C20, 0xFF192F22, 0xFF12261D,
        0xFF112B1E, 0xFF173627, 0xFF243C2F, 0xFF3F4C42, 0xFF303C32, 0xFF172F1F, 0xFF40634F, 0xFF618675, 0xFF5D8472, 0xFF3D6654, 0xFF37624F, 0xFF3A6552, 0xFF345D4B, 0xFF3D6452, 0xFF3D6251, 0xFF426151, 0xFF325342, 0xFF3C5C4D, 0xFF385849, 0xFF3A5648,
        0xFF52675E, 0xFF4D5E56, 0xFF3A4740, 0xFF2D3A42, 0xFF202D26, 0xFF293521, 0xFF444B39, 0xFF1E241A, 0xFF141A1A, 0xFF141D1C, 0xFF1F2A24, 0xFF25302A, 0xFF17241D, 0xFF12211A, 0xFF172621, 0xFF17221E, 0xFF161C1A, 0xFF19191B, 0xFF1B161A, 0xFF171717,
        0xFF181617, 0xFF1A1617, 0xFF191516, 0xFF191516, 0xFF1B191A, 0xFF1A1C1B, 0xFF161A19, 0xFF181E1A, 0xFF16231C, 0xFF1D2E26, 0xFF182D24, 0xFF172B22, 0xFF16231C, 0xFF191B18, 0xFF272322, 0xFF24201D, 0xFF181C1B, 0xFF141D1A, 0xFF23261B, 0xFF5A5744,
        0xFF4F4C39, 0xFF22281E, 0xFF203030, 0xFF2F3B39, 0xFF60716B, 0xFF546B63, 0xFF3B5B4E, 0xFF5C7E6E, 0xFF567B69, 0xFF567C67, 0xFF5B7E6A, 0xFF658578, 0xFF567868, 0xFF446958, 0xFF39604E, 0xFF3C6350, 0xFF335544, 0xFF2A4937, 0xFF425D4C, 0xFF466150,
        0xFF3F5E4C, 0xFF517362, 0xFF5F8673, 0xFF608977, 0xFF5E8775, 0xFF5B8270, 0xFF618577, 0xFF5F8472, 0xFF5F8472, 0xFF5F8472, 0xFF5F8472, 0xFF5F8472, 0xFF5F8472, 0xFF5F8472, 0xFF5F8472, 0xFF5F8472, 0xFF5F8472, 0xFF608573, 0xFF608573, 0xFF618674,
        0xFF608573, 0xFF608573, 0xFF608573, 0xFF5F8473, 0xFF5F8473, 0xFF628776, 0xFF507564, 0xFF3A5F4E, 0xFF496E5D, 0xFF193E2D, 0xFF335847, 0xFF466B5A, 0xFF1B402F, 0xFF375C4A, 0xFF416654, 0xFF1E442F, 0xFF305641, 0xFF40664F, 0xFF193F28, 0xFF426048,
        0xFF315342, 0xFF204637, 0xFF2B442E, 0xFF60643F, 0xFF978652, 0xFFA48A4F, 0xFFA78A50, 0xFF8C7B47, 0xFF44482D, 0xFF48584D, 0xFF72756A, 0xFF928F80, 0xFFD3D3CB, 0xFFDADAD0, 0xFFAFA695, 0xFF908A74, 0xFF89836D, 0xFF928C76, 0xFF9A947E, 0xFFA9A38D,
        0xFFAFA993, 0xFF9D9781, 0xFF948E78, 0xFF9E8C78, 0xFF9D927E, 0xFF93927E, 0xFF989989, 0xFF9C9A8E, 0xFF9B9E97, 0xFF768A7F, 0xFF5A7C6E, 0xFF648071, 0xFF678676, 0xFF608272, 0xFF638575, 0xFF537263, 0x0097ABA2, 0x00F5FEF9, 0x00FDFDFB, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x00F8FFFF, 0xFF8AA89E, 0xFF456E60, 0xFF63897C, 0xFF688C80, 0xFF467062, 0xFF3B6B5B, 0xFF3E6757, 0xFF416A5A, 0xFF456E5E, 0xFF446D5D, 0xFF416A5A,
        0xFF406959, 0xFF436C5C, 0xFF456E5E, 0xFF658173, 0xFF486255, 0xFF3B5145, 0xFF4D6156, 0xFF475B50, 0xFF2A4235, 0xFF1E3A2C, 0xFF254435, 0xFF143626, 0xFF173929, 0xFF234837, 0xFF2F5345, 0xFF2B5142, 0xFF3A6355, 0xFF527C6E, 0xFF5F897B, 0xFF5F8674,
        0xFF5F8674, 0xFF5D8674, 0xFF5D8877, 0xFF5F8A79, 0xFF5D897A, 0xFF578676, 0xFF568575, 0xFF5D8676, 0xFF5E8777, 0xFF638C7C, 0xFF3E6757, 0xFF3F6858, 0xFF396252, 0xFF436C5C, 0xFF456E5E, 0xFF3E6553, 0xFF2E5543, 0xFF315846, 0xFF39604E, 0xFF254C3A,
        0xFF416856, 0xFF4E7563, 0xFF648B79, 0xFF5C8573, 0xFF5C8573, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5C8573, 0xFF5C8573, 0xFF5D8270, 0xFF658675, 0xFF3F5B4C, 0xFF162C1F, 0xFF475B4F, 0xFF3D4F43, 0xFF192D21, 0xFF1D3326, 0xFF1E3229,
        0xFF1B3528, 0xFF183728, 0xFF263E31, 0xFF465349, 0xFF303C32, 0xFF172F1F, 0xFF4F725E, 0xFF5F8472, 0xFF658C79, 0xFF57816D, 0xFF598470, 0xFF5E8975, 0xFF4F7965, 0xFF426956, 0xFF345947, 0xFF325443, 0xFF3D5F4E, 0xFF5A7C6C, 0xFF517363, 0xFF3C5B4C,
        0xFF4A5F56, 0xFF46554E, 0xFF29342E, 0xFF142129, 0xFF15221B, 0xFF28311E, 0xFF404735, 0xFF1B1E15, 0xFF121617, 0xFF111717, 0xFF141F19, 0xFF151E19, 0xFF121D17, 0xFF17231F, 0xFF1D2C27, 0xFF202F2A, 0xFF242E2D, 0xFF212727, 0xFF18191B, 0xFF171918,
        0xFF181818, 0xFF1B1718, 0xFF1E181A, 0xFF201A1C, 0xFF1C1A1B, 0xFF141817, 0xFF0F1815, 0xFF0D2118, 0xFF152A21, 0xFF193529, 0xFF173327, 0xFF1C3329, 0xFF1F2E27, 0xFF151A16, 0xFF161513, 0xFF1C1815, 0xFF181A19, 0xFF161C1A, 0xFF1E2116, 0xFF4F4A37,
        0xFF423D2A, 0xFF161C12, 0xFF1A2829, 0xFF25312F, 0xFF5C6D67, 0xFF50675F, 0xFF304E42, 0xFF446656, 0xFF426755, 0xFF466955, 0xFF496C58, 0xFF4C6C5F, 0xFF3F6151, 0xFF325746, 0xFF345B49, 0xFF466D5A, 0xFF466857, 0xFF395846, 0xFF4A6554, 0xFF415C4B,
        0xFF345341, 0xFF466857, 0xFF5C8370, 0xFF628B79, 0xFF5D8674, 0xFF59806E, 0xFF618577, 0xFF5E8371, 0xFF5E8371, 0xFF5E8371, 0xFF5E8371, 0xFF5E8371, 0xFF5E8371, 0xFF5E8371, 0xFF5E8371, 0xFF5D8471, 0xFF5E8572, 0xFF5E8572, 0xFF5F8673, 0xFF5F8673,
        0xFF5F8673, 0xFF5E8572, 0xFF5E8572, 0xFF5E8372, 0xFF658A79, 0xFF567B6A, 0xFF365B4A, 0xFF4A6F5E, 0xFF274C3B, 0xFF294E3D, 0xFF466B5A, 0xFF1C4130, 0xFF335847, 0xFF496E5C, 0xFF1E4331, 0xFF335944, 0xFF476D58, 0xFF163C25, 0xFF3A6049, 0xFF405E46,
        0xFF1A3C2B, 0xFF1F4536, 0xFF425B45, 0xFF686C47, 0xFF9A8955, 0xFFAE9459, 0xFFA3864C, 0xFF8A7945, 0xFF3C4226, 0xFF3A4C40, 0xFF6B7165, 0xFF8A8778, 0xFFD4D4CC, 0xFFDBDBD1, 0xFFABA492, 0xFF908A74, 0xFF88826C, 0xFF8F8973, 0xFF96907A, 0xFFA59F89,
        0xFFACA690, 0xFF9B957F, 0xFF938D77, 0xFF9A8874, 0xFF9C917D, 0xFF92917C, 0xFF969485, 0xFF908C81, 0xFF909189, 0xFF79897F, 0xFF638175, 0xFF688374, 0xFF6A8979, 0xFF678778, 0xFF638575, 0xFF516D5F, 0x0099ADA4, 0x00FAFFFC, 0x00FFFFFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x00F8FFFF, 0xFF8AA89E, 0xFF477062, 0xFF658B7E, 0xFF64887C, 0xFF2C5648, 0xFF134333, 0xFF1C4535, 0xFF1B4434, 0xFF174030, 0xFF174030, 0xFF1C4535,
        0xFF194232, 0xFF174030, 0xFF1D4636, 0xFF687E72, 0xFF44554B, 0xFF46534A, 0xFF828982, 0xFFA0A59F, 0xFF99A099, 0xFF8E9890, 0xFF98A39B, 0xFF95A59A, 0xFF95A79B, 0xFF83978C, 0xFF4B6558, 0xFF2E4E41, 0xFF375B4D, 0xFF588171, 0xFF618B7D, 0xFF628273,
        0xFF678778, 0xFF688A7A, 0xFF64887A, 0xFF5E8475, 0xFF578072, 0xFF578173, 0xFF5E8A7B, 0xFF578070, 0xFF5D8676, 0xFF5D8676, 0xFF5E8777, 0xFF436C5C, 0xFF406959, 0xFF3D6656, 0xFF285141, 0xFF3E6553, 0xFF2F5644, 0xFF325947, 0xFF3A614F, 0xFF264D3B,
        0xFF426957, 0xFF4E7563, 0xFF648B79, 0xFF5C8573, 0xFF5C8573, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5C8573, 0xFF5C8573, 0xFF5E8371, 0xFF658675, 0xFF3D594A, 0xFF162C1F, 0xFF4B5F53, 0xFF405246, 0xFF1A2E22, 0xFF1F3528, 0xFF1F332A,
        0xFF1C3629, 0xFF173627, 0xFF273F32, 0xFF445147, 0xFF2E3A30, 0xFF172F1F, 0xFF476A56, 0xFF5F8170, 0xFF577968, 0xFF375948, 0xFF305140, 0xFF3B5C4B, 0xFF385A49, 0xFF365847, 0xFF365847, 0xFF3C614F, 0xFF577E6B, 0xFF5B806F, 0xFF416353, 0xFF456455,
        0xFF6B8077, 0xFF5F6E67, 0xFF27302B, 0xFF0B141D, 0xFF0E1712, 0xFF222917, 0xFF3C3F2E, 0xFF1B1C14, 0xFF1C1C1E, 0xFF181C1D, 0xFF161C18, 0xFF141613, 0xFF141817, 0xFF16211D, 0xFF1A2924, 0xFF1E2E2B, 0xFF233330, 0xFF202E2E, 0xFF122020, 0xFF111C18,
        0xFF141817, 0xFF181617, 0xFF1F191B, 0xFF1F1B1C, 0xFF151716, 0xFF101916, 0xFF13221D, 0xFF3F5F52, 0xFF47695B, 0xFF456B5C, 0xFF426859, 0xFF47675A, 0xFF455A51, 0xFF252E29, 0xFF161815, 0xFF181411, 0xFF151716, 0xFF151B19, 0xFF1E2015, 0xFF514C39,
        0xFF46412E, 0xFF161910, 0xFF0F1D1E, 0xFF1A2423, 0xFF576661, 0xFF516860, 0xFF335145, 0xFF3B5D4D, 0xFF3B5D4C, 0xFF3E614D, 0xFF3E5F4C, 0xFF3A5A4D, 0xFF3F6151, 0xFF3F6453, 0xFF2B5240, 0xFF4A715E, 0xFF597B6A, 0xFF375644, 0xFF5B7665, 0xFF3F5A49,
        0xFF3E5D4B, 0xFF507261, 0xFF59806D, 0xFF58816F, 0xFF598270, 0xFF5B8270, 0xFF618577, 0xFF608573, 0xFF608573, 0xFF608573, 0xFF608573, 0xFF608573, 0xFF608573, 0xFF608573, 0xFF608573, 0xFF5D8471, 0xFF5D8471, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572,
        0xFF5E8572, 0xFF5D8471, 0xFF5D8471, 0xFF608574, 0xFF567B6A, 0xFF305544, 0xFF4E7362, 0xFF264B3A, 0xFF254A39, 0xFF4B705F, 0xFF254A39, 0xFF335847, 0xFF4C7160, 0xFF1D4230, 0xFF315644, 0xFF476D58, 0xFF224833, 0xFF325841, 0xFF3D634C, 0xFF1A3820,
        0xFF2F5140, 0xFF33594A, 0xFF3B543E, 0xFF6C704B, 0xFF9F8E5A, 0xFFA68C51, 0xFFA88B51, 0xFF8B7B47, 0xFF3C4226, 0xFF36483C, 0xFF71776B, 0xFF8B897A, 0xFFD6D7CF, 0xFFDCDED3, 0xFFA8A18F, 0xFF99927F, 0xFF97907D, 0xFF9C9582, 0xFF9F9885, 0xFFA49D8A,
        0xFFA7A08D, 0xFFA09986, 0xFF9E9784, 0xFFA79885, 0xFF97907D, 0xFF797B66, 0xFF7A7869, 0xFF7C7469, 0xFF858178, 0xFF757F74, 0xFF5E7565, 0xFF617C6D, 0xFF607C6D, 0xFF658576, 0xFF678778, 0xFF506C5E, 0x009AABA3, 0x00FAFFFC, 0x00FFFFFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x00F8FFFF, 0xFF88A69C, 0xFF477062, 0xFF658B7E, 0xFF678B7F, 0xFF315B4D, 0xFF225242, 0xFF4A7363, 0xFF3B6454, 0xFF285141, 0xFF315A4A, 0xFF497262,
        0xFF426B5B, 0xFF295242, 0xFF214A3A, 0xFF5A7667, 0xFF3F5548, 0xFF4A5A4F, 0xFFA5ACA4, 0xFFDDE0D9, 0xFFE9EAE4, 0xFFE6E7E1, 0xFFE7E8E2, 0xFFEEE9E5, 0xFFF2EFEA, 0xFFBEC3BC, 0xFF525F56, 0xFF3F5549, 0xFF446256, 0xFF53796A, 0xFF416A5A, 0xFF466053,
        0xFF435D50, 0xFF3B5749, 0xFF38584B, 0xFF3D5F51, 0xFF3B6154, 0xFF3A6355, 0xFF416A5C, 0xFF5A8373, 0xFF5B8474, 0xFF5E8777, 0xFF598272, 0xFF648D7D, 0xFF355E4E, 0xFF4B7464, 0xFF153E2E, 0xFF3D6452, 0xFF2E5543, 0xFF325947, 0xFF3B6250, 0xFF264D3B,
        0xFF416856, 0xFF4D7462, 0xFF638A78, 0xFF5C8573, 0xFF5C8573, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5C8573, 0xFF5C8573, 0xFF608573, 0xFF668776, 0xFF3C5849, 0xFF152B1E, 0xFF4C6054, 0xFF415347, 0xFF192D21, 0xFF1E3427, 0xFF1D3128,
        0xFF1A3427, 0xFF183728, 0xFF2A4235, 0xFF3C493F, 0xFF2E3A30, 0xFF213929, 0xFF365945, 0xFF3D604C, 0xFF385745, 0xFF2B4333, 0xFF374C3D, 0xFF465B4C, 0xFF324A3A, 0xFF2C4B39, 0xFF436652, 0xFF5A816E, 0xFF58826E, 0xFF4D7462, 0xFF496E5D, 0xFF4C6B5C,
        0xFF586D64, 0xFF536059, 0xFF242D28, 0xFF181C27, 0xFF171C18, 0xFF262918, 0xFF3E3C2D, 0xFF1F1A14, 0xFF1F1A1E, 0xFF19191B, 0xFF151714, 0xFF201A1C, 0xFF181617, 0xFF161C1A, 0xFF152421, 0xFF162B26, 0xFF19322E, 0xFF19322E, 0xFF112A26, 0xFF13241E,
        0xFF141F1B, 0xFF171918, 0xFF1C1819, 0xFF191718, 0xFF131716, 0xFF1D2C27, 0xFF364B44, 0xFF5A8574, 0xFF588473, 0xFF4D7C6A, 0xFF4F7E6C, 0xFF597F70, 0xFF536F63, 0xFF26352E, 0xFF111713, 0xFF1B1613, 0xFF181A19, 0xFF171B1A, 0xFF1F2116, 0xFF57523F,
        0xFF514A38, 0xFF1A1D14, 0xFF091718, 0xFF111B1A, 0xFF4A5954, 0xFF4C615A, 0xFF385448, 0xFF365647, 0xFF325443, 0xFF375845, 0xFF365744, 0xFF39594C, 0xFF355747, 0xFF244938, 0xFF305745, 0xFF375E4B, 0xFF385A49, 0xFF3F5E4C, 0xFF294433, 0xFF344F3E,
        0xFF41604E, 0xFF5D7F6E, 0xFF618875, 0xFF5A8371, 0xFF598270, 0xFF587F6D, 0xFF597D6F, 0xFF5F8472, 0xFF5F8472, 0xFF5F8472, 0xFF5F8472, 0xFF5F8472, 0xFF5F8472, 0xFF5F8472, 0xFF5F8472, 0xFF5B8571, 0xFF5B8571, 0xFF5B8571, 0xFF5B8571, 0xFF5B8571,
        0xFF5A8470, 0xFF59836F, 0xFF59836F, 0xFF5A7F6E, 0xFF2B503F, 0xFF4D7261, 0xFF234837, 0xFF2D5241, 0xFF496E5D, 0xFF224736, 0xFF2D5241, 0xFF517665, 0xFF123726, 0xFF395E4C, 0xFF406553, 0xFF173D28, 0xFF375D48, 0xFF426851, 0xFF153B24, 0xFF436149,
        0xFF365847, 0xFF325849, 0xFF3D5640, 0xFF636742, 0xFF92814D, 0xFFA3894E, 0xFFA6894F, 0xFF8E7E4A, 0xFF394225, 0xFF314337, 0xFF727A6D, 0xFF868475, 0xFFD2D3CB, 0xFFDBDDD2, 0xFFAAA391, 0xFF9F9888, 0xFFA29B8B, 0xFFA29B8B, 0xFFA59E8E, 0xFFA59E8E,
        0xFFA49D8D, 0xFFA7A090, 0xFFA69F8F, 0xFFA2998A, 0xFF939280, 0xFF787B68, 0xFF7B796A, 0xFF7F7569, 0xFF90867A, 0xFF8D8F81, 0xFF7A8976, 0xFF7B9385, 0xFF607C6D, 0xFF5F7F70, 0xFF688879, 0xFF516D5F, 0x009AABA3, 0x00F9FFFB, 0x00FFFEFC, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x00F5FFFC, 0xFF89A79D, 0xFF4C7567, 0xFF668C7F, 0xFF62867A, 0xFF2B5547, 0xFF255545, 0xFF456E5E, 0xFF487161, 0xFF426B5B, 0xFF466F5F, 0xFF4E7767,
        0xFF355E4E, 0xFF1C4535, 0xFF224B3B, 0xFF597E6C, 0xFF3B5A4A, 0xFF475D50, 0xFFB7C4BA, 0xFFEAEFE8, 0xFFD9DAD4, 0xFFCECDC8, 0xFFCCCBC6, 0xFFE3CFD0, 0xFFF8E8E8, 0xFFCFC7C4, 0xFF595E58, 0xFF48584E, 0xFF2F4B3F, 0xFF2F5345, 0xFF234C3C, 0xFF596F63,
        0xFF445A4E, 0xFF2A4437, 0xFF2A463A, 0xFF3B5B4E, 0xFF416257, 0xFF365A4E, 0xFF31574A, 0xFF628B7B, 0xFF5E8777, 0xFF5D8676, 0xFF588171, 0xFF608979, 0xFF4F7868, 0xFF396252, 0xFF234C3C, 0xFF3B6250, 0xFF2D5442, 0xFF315846, 0xFF3A614F, 0xFF264D3B,
        0xFF416856, 0xFF4C7361, 0xFF618876, 0xFF5C8573, 0xFF5C8573, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5C8573, 0xFF5C8573, 0xFF628775, 0xFF678877, 0xFF3B5748, 0xFF142A1D, 0xFF4D6155, 0xFF415347, 0xFF172B1F, 0xFF1C3225, 0xFF21352C,
        0xFF1B3528, 0xFF1C3B2C, 0xFF2E4639, 0xFF3A473D, 0xFF3C483E, 0xFF405848, 0xFF436652, 0xFF3B5E4A, 0xFF466150, 0xFF5A6A5D, 0xFF7F897E, 0xFF7D877C, 0xFF405043, 0xFF304B3A, 0xFF5E816D, 0xFF507A66, 0xFF4C7662, 0xFF4B7462, 0xFF557A69, 0xFF4B6A5B,
        0xFF576C63, 0xFF616E67, 0xFF252E29, 0xFF151723, 0xFF161614, 0xFF262415, 0xFF3D3A2B, 0xFF1F1812, 0xFF1D171B, 0xFF1C171B, 0xFF1B1A18, 0xFF25191D, 0xFF1A1416, 0xFF181C1D, 0xFF1D2C29, 0xFF1D342E, 0xFF1B3833, 0xFF1A3933, 0xFF173630, 0xFF1D322B,
        0xFF1E2A26, 0xFF1B1F1E, 0xFF1B191A, 0xFF181818, 0xFF191F1D, 0xFF374842, 0xFF617B72, 0xFF427360, 0xFF386B58, 0xFF2A5F4B, 0xFF376A57, 0xFF4B7766, 0xFF4C6C5F, 0xFF23342C, 0xFF111A15, 0xFF1C1714, 0xFF1E1E1E, 0xFF1A1E1D, 0xFF1A1C11, 0xFF4D4634,
        0xFF48412F, 0xFF1A1D14, 0xFF101E1F, 0xFF26302F, 0xFF53625D, 0xFF51665F, 0xFF3E5A4E, 0xFF365647, 0xFF325443, 0xFF3B5C49, 0xFF3D5E4B, 0xFF3A5A4D, 0xFF3E6050, 0xFF3D6251, 0xFF3E6553, 0xFF446B58, 0xFF365847, 0xFF22412F, 0xFF2F4A39, 0xFF344F3E,
        0xFF3F5E4C, 0xFF597B6A, 0xFF5E8572, 0xFF57806E, 0xFF58816F, 0xFF5A816F, 0xFF5F8375, 0xFF5D8270, 0xFF5D8270, 0xFF5D8270, 0xFF5D8270, 0xFF5D8270, 0xFF5D8270, 0xFF5D8270, 0xFF5D8270, 0xFF5B8571, 0xFF5B8571, 0xFF5B8571, 0xFF5B8571, 0xFF5A8470,
        0xFF59836F, 0xFF58826E, 0xFF57816D, 0xFF365B4A, 0xFF527766, 0xFF244938, 0xFF315645, 0xFF4B705F, 0xFF2B503F, 0xFF2D5241, 0xFF4E7362, 0xFF1B402F, 0xFF406554, 0xFF426755, 0xFF1A3F2D, 0xFF3A604B, 0xFF416752, 0xFF123821, 0xFF3E644D, 0xFF39573F,
        0xFF426453, 0xFF446A5B, 0xFF415A44, 0xFF5E623D, 0xFF978652, 0xFFAB9156, 0xFFA6894F, 0xFF8B7B47, 0xFF374023, 0xFF2E4236, 0xFF757D70, 0xFF858374, 0xFFD0D1C9, 0xFFDADCD1, 0xFFACA593, 0xFF9E9689, 0xFFA29A8D, 0xFF9C9487, 0xFFA29A8D, 0xFFA59D90,
        0xFFA49C8F, 0xFFAAA295, 0xFFA69E91, 0xFF9B978B, 0xFF9FA193, 0xFF929685, 0xFF918F80, 0xFF8A7E72, 0xFFA29487, 0xFFC2BFAE, 0xFFCCD5C0, 0xFFB2CABC, 0xFF779384, 0xFF607F70, 0xFF658576, 0xFF516B5E, 0x009AABA3, 0x00FBFFFC, 0x00FFFFFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x00F8FFFF, 0xFF89A79D, 0xFF487163, 0xFF678D80, 0xFF668A7E, 0xFF2B5547, 0xFF265646, 0xFF446F5E, 0xFF244F3E, 0xFF406B5A, 0xFF3A6554, 0xFF3B6655,
        0xFF487362, 0xFF234E3D, 0xFF224D3C, 0xFF5B8472, 0xFF466556, 0xFF6A7A70, 0xFFBFC0BB, 0xFFE5E0DD, 0xFFEDE5E3, 0xFFD3D2CE, 0xFFC4C5C0, 0xFFDED8D8, 0xFFDAD6D5, 0xFFD2D2D0, 0xFF757A76, 0xFF66716B, 0xFF63746C, 0xFF475C53, 0xFF576E64, 0xFF8E908D,
        0xFF747673, 0xFF535552, 0xFF4F5450, 0xFF727773, 0xFFB3B9B5, 0xFFAFB5B1, 0xFF474D49, 0xFF456B5E, 0xFF62887B, 0xFF61877A, 0xFF5B8172, 0xFF5E8475, 0xFF4E7563, 0xFF3D6452, 0xFF204735, 0xFF3C6350, 0xFF325946, 0xFF2F5643, 0xFF3A614E, 0xFF264D3A,
        0xFF406754, 0xFF527966, 0xFF5F8673, 0xFF5B8472, 0xFF5C8573, 0xFF5E8775, 0xFF5F8876, 0xFF5E8775, 0xFF5D8674, 0xFF5D8674, 0xFF5F8876, 0xFF5E8274, 0xFF6A8F7E, 0xFF3B5B4C, 0xFF1C2D23, 0xFF585D56, 0xFF4F524B, 0xFF152719, 0xFF153623, 0xFF182E21,
        0xFF203C2D, 0xFF1D3C2C, 0xFF35493D, 0xFF414B42, 0xFF3C463D, 0xFF0D2819, 0xFF2D5844, 0xFF39634F, 0xFF3F5B4C, 0xFF535D54, 0xFFBBB8B3, 0xFFD4CFCB, 0xFF767D75, 0xFF3E5648, 0xFF5B7D6C, 0xFF567D6A, 0xFF507563, 0xFF587A6A, 0xFF4F6B5D, 0xFF354D40,
        0xFF74887F, 0xFF62716A, 0xFF26332C, 0xFF17141D, 0xFF1E1A17, 0xFF333124, 0xFF49473B, 0xFF1D1C18, 0xFF15191C, 0xFF151E1B, 0xFF18251B, 0xFF171C18, 0xFF171C18, 0xFF1A1F1B, 0xFF272D2B, 0xFF222E2A, 0xFF162B26, 0xFF1C3A32, 0xFF1B3C33, 0xFF1C3730,
        0xFF29403A, 0xFF1C2B28, 0xFF141F1B, 0xFF0F1B17, 0xFF0A2117, 0xFF3E6254, 0xFF507C6B, 0xFF415B52, 0xFF1A352C, 0xFF07271C, 0xFF0E2F24, 0xFF1C3A30, 0xFF243B33, 0xFF1B2723, 0xFF111A17, 0xFF121D15, 0xFF131917, 0xFF191B1A, 0xFF232319, 0xFF5B5A46,
        0xFF4F4E3A, 0xFF17170F, 0xFF1B1A20, 0xFF222826, 0xFF4C5751, 0xFF53645C, 0xFF41594C, 0xFF385445, 0xFF385745, 0xFF405D49, 0xFF405D49, 0xFF3A5647, 0xFF3D594A, 0xFF435E4F, 0xFF456051, 0xFF445C4E, 0xFF425A4C, 0xFF4A6254, 0xFF556B5E, 0xFF54695A,
        0xFF4C6353, 0xFF4E6A5B, 0xFF5D7F6E, 0xFF658C7A, 0xFF628977, 0xFF608574, 0xFF64887A, 0xFF5E8475, 0xFF5F8576, 0xFF618778, 0xFF628879, 0xFF618778, 0xFF608677, 0xFF608677, 0xFF608677, 0xFF628879, 0xFF648A7B, 0xFF63897A, 0xFF608677, 0xFF618778,
        0xFF5F8576, 0xFF517768, 0xFF3F6556, 0xFF4E7563, 0xFF234A38, 0xFF2C5341, 0xFF4B7260, 0xFF29503E, 0xFF2C5341, 0xFF4E7563, 0xFF1C4331, 0xFF37614D, 0xFF436D59, 0xFF153C29, 0xFF3D6451, 0xFF446957, 0xFF143927, 0xFF3B5D4C, 0xFF406251, 0xFF3F6453,
        0xFF597866, 0xFF527362, 0xFF365647, 0xFF596444, 0xFF9A854C, 0xFFAB8C4C, 0xFFA08B54, 0xFF8A794B, 0xFF38401B, 0xFF334633, 0xFF777C76, 0xFF84817A, 0xFFD3D6C5, 0xFFDDE0CF, 0xFFA9A097, 0xFF98917F, 0xFF958E7C, 0xFF9C9583, 0xFFA69F8D, 0xFFABA492,
        0xFFACA593, 0xFFA69F8D, 0xFF9A9381, 0xFF9B937E, 0xFF988F7D, 0xFF9C9583, 0xFF95907D, 0xFF8D8A79, 0xFF918F80, 0xFFC3C1B4, 0xFFDDDED0, 0xFF96B5A5, 0xFF6E8F7E, 0xFF668878, 0xFF6A8C7C, 0xFF4F6B5D, 0x0096A79F, 0x00F8FEFA, 0x00FFFFFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFCFE, 0x00F5FFFC, 0xFF84A298, 0xFF456E60, 0xFF658B7E, 0xFF63877B, 0xFF2A5446, 0xFF235343, 0xFF446E60, 0xFF345E50, 0xFF416B5D, 0xFF335D4F, 0xFF3B6557,
        0xFF497365, 0xFF254F41, 0xFF254F41, 0xFF587F6D, 0xFF3C5B4C, 0xFF6A7A70, 0xFFCACCC7, 0xFFE0DBD8, 0xFFDCD7D4, 0xFFDBDAD6, 0xFFD9DBD6, 0xFFD9D3D3, 0xFFECE8E7, 0xFFCACAC8, 0xFF707672, 0xFF7D8C85, 0xFF53685F, 0xFF3F594E, 0xFF627E72, 0xFFA3AEA8,
        0xFF89948E, 0xFF69746E, 0xFF6C7972, 0xFF839089, 0xFFA4B3AC, 0xFFAFBEB7, 0xFF82918A, 0xFF32584B, 0xFF4F7568, 0xFF597F72, 0xFF608677, 0xFF668C7D, 0xFF486F5D, 0xFF406755, 0xFF224937, 0xFF406754, 0xFF2B523F, 0xFF365D4A, 0xFF375E4B, 0xFF224936,
        0xFF466D5A, 0xFF4D7461, 0xFF648B78, 0xFF5D8674, 0xFF5E8775, 0xFF5F8876, 0xFF608977, 0xFF5F8876, 0xFF5E8775, 0xFF5D8674, 0xFF5D8674, 0xFF5D8173, 0xFF608775, 0xFF3B5B4C, 0xFF203429, 0xFF565B54, 0xFF53564F, 0xFF1E3022, 0xFF11301E, 0xFF172D20,
        0xFF112C1D, 0xFF1F3A2B, 0xFF34483C, 0xFF49534A, 0xFF323E34, 0xFF264233, 0xFF3E6955, 0xFF608A76, 0xFF4E6A5B, 0xFF525E54, 0xFF9F9E99, 0xFF9F9C97, 0xFF555F56, 0xFF40584A, 0xFF5A7C6B, 0xFF527966, 0xFF557A68, 0xFF537565, 0xFF496557, 0xFF455D50,
        0xFF6C8077, 0xFF4D5C55, 0xFF19261F, 0xFF14141C, 0xFF171813, 0xFF292A1C, 0xFF413F33, 0xFF1A1915, 0xFF16171B, 0xFF161C1A, 0xFF19231A, 0xFF202521, 0xFF1C211D, 0xFF191E1A, 0xFF1F2523, 0xFF17231F, 0xFF10231F, 0xFF19322C, 0xFF18372F, 0xFF122D26,
        0xFF1D322D, 0xFF192523, 0xFF151E1B, 0xFF18211E, 0xFF15261E, 0xFF3C584C, 0xFF47695B, 0xFF20332D, 0xFF11261F, 0xFF122C23, 0xFF173128, 0xFF162D25, 0xFF162721, 0xFF131C19, 0xFF171B1A, 0xFF171E17, 0xFF161A19, 0xFF191919, 0xFF1B1B11, 0xFF4F513C,
        0xFF464934, 0xFF13160D, 0xFF14191D, 0xFF151E1B, 0xFF58655E, 0xFF5C7168, 0xFF385446, 0xFF335242, 0xFF3A5B48, 0xFF385944, 0xFF31513C, 0xFF395848, 0xFF3B5A4A, 0xFF3F5E4E, 0xFF435F50, 0xFF415D4E, 0xFF415C4D, 0xFF476253, 0xFF4F6A5B, 0xFF4A6151,
        0xFF3D5847, 0xFF355645, 0xFF395E4C, 0xFF396250, 0xFF355E4C, 0xFF365D4B, 0xFF3B5F51, 0xFF406657, 0xFF416758, 0xFF426859, 0xFF426859, 0xFF406657, 0xFF406657, 0xFF416758, 0xFF43695A, 0xFF406657, 0xFF406657, 0xFF3B6152, 0xFF385E4F, 0xFF3D6354,
        0xFF446A5B, 0xFF426859, 0xFF3A6051, 0xFF264D3B, 0xFF305745, 0xFF4D7462, 0xFF204735, 0xFF2B5240, 0xFF4F7664, 0xFF234A38, 0xFF2D5442, 0xFF436D59, 0xFF1A4430, 0xFF406754, 0xFF385F4C, 0xFF1E4331, 0xFF416654, 0xFF416352, 0xFF1F4130, 0xFF3F6453,
        0xFF496856, 0xFF3B5C4B, 0xFF284839, 0xFF525D3D, 0xFF958047, 0xFFA98A4A, 0xFFA6915A, 0xFF89784A, 0xFF373F1A, 0xFF334633, 0xFF767B75, 0xFF84817A, 0xFFD3D6C5, 0xFFDCDFCE, 0xFFA9A097, 0xFF8D8674, 0xFF88816F, 0xFF8F8876, 0xFF9D9684, 0xFFA7A08E,
        0xFFA9A290, 0xFF9E9785, 0xFF8D8674, 0xFF948B79, 0xFF968D7B, 0xFF98917F, 0xFF8F8979, 0xFF898677, 0xFF939182, 0xFFC3C1B4, 0xFFD8D8CC, 0xFF739282, 0xFF5A7B6A, 0xFF5B7D6D, 0xFF658777, 0xFF516D5F, 0x009AABA3, 0x00FBFFFD, 0x00FFFFFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x00F7FFFE, 0xFF84A298, 0xFF4A7365, 0xFF6A9083, 0xFF65897D, 0xFF2D5749, 0xFF215141, 0xFF5A847A, 0xFF487268, 0xFF396359, 0xFF2D574D, 0xFF446E64,
        0xFF497369, 0xFF224C42, 0xFF1E483E, 0xFF5C7E6E, 0xFF385245, 0xFF536359, 0xFFC1C6C0, 0xFFE9E8E4, 0xFFDFDEDA, 0xFFD7D9D4, 0xFFD5DAD4, 0xFFDCD8D7, 0xFFE6E5E3, 0xFFBFC4C0, 0xFF637069, 0xFF697D74, 0xFF395549, 0xFF496B5D, 0xFF3D6153, 0xFF50675D,
        0xFF475E54, 0xFF395046, 0xFF445E53, 0xFF4D675C, 0xFF466256, 0xFF4B675B, 0xFF466256, 0xFF406659, 0xFF31574A, 0xFF5C8275, 0xFF5C8273, 0xFF5D8374, 0xFF4C7361, 0xFF39604E, 0xFF204735, 0xFF3A614F, 0xFF2D5442, 0xFF345B49, 0xFF3D6452, 0xFF284F3D,
        0xFF416856, 0xFF537A68, 0xFF608775, 0xFF5C8573, 0xFF5D8674, 0xFF5E8775, 0xFF5E8775, 0xFF5E8775, 0xFF5E8775, 0xFF5D8674, 0xFF5B8472, 0xFF5D8173, 0xFF6A917F, 0xFF335545, 0xFF142A1E, 0xFF5C635B, 0xFF5D6059, 0xFF223225, 0xFF0F2C1A, 0xFF1E3427,
        0xFF183022, 0xFF1F3528, 0xFF3D4D42, 0xFF475349, 0xFF203025, 0xFF163223, 0xFF547E6A, 0xFF5B8571, 0xFF597868, 0xFF516156, 0xFF8C8F88, 0xFF949790, 0xFF4A564C, 0xFF324D3E, 0xFF5B806E, 0xFF587F6C, 0xFF527765, 0xFF4A6C5C, 0xFF496557, 0xFF61796C,
        0xFF788C83, 0xFF5F6E67, 0xFF2D3A33, 0xFF131C21, 0xFF121912, 0xFF202415, 0xFF38382C, 0xFF181713, 0xFF19171C, 0xFF191B1A, 0xFF1D221B, 0xFF181E1A, 0xFF171D19, 0xFF171C18, 0xFF191F1D, 0xFF131E1A, 0xFF11201D, 0xFF192E29, 0xFF17302A, 0xFF18312B,
        0xFF1D302C, 0xFF1E2827, 0xFF1A1E1D, 0xFF1C201F, 0xFF141D18, 0xFF23322B, 0xFF23372E, 0xFF18231F, 0xFF15201C, 0xFF1E2A26, 0xFF1C2824, 0xFF131E1A, 0xFF111715, 0xFF141615, 0xFF1D1B1C, 0xFF1C1D18, 0xFF1C1C1C, 0xFF1C1A1B, 0xFF1A170E, 0xFF474934,
        0xFF3E462F, 0xFF0D170C, 0xFF101B1D, 0xFF1D2925, 0xFF72837B, 0xFF7B9288, 0xFF4B6A5B, 0xFF3C5E4D, 0xFF3E644F, 0xFF42654F, 0xFF466953, 0xFF416352, 0xFF416352, 0xFF436554, 0xFF456655, 0xFF456655, 0xFF466555, 0xFF4A6959, 0xFF4F6E5E, 0xFF597463,
        0xFF4F6E5C, 0xFF446957, 0xFF3F6955, 0xFF3E6956, 0xFF3D6855, 0xFF3F6856, 0xFF456B5C, 0xFF45695B, 0xFF466A5C, 0xFF45695B, 0xFF44685A, 0xFF436759, 0xFF436759, 0xFF45695B, 0xFF476B5D, 0xFF426658, 0xFF476B5D, 0xFF496D5F, 0xFF486C5E, 0xFF496D5F,
        0xFF486C5E, 0xFF406456, 0xFF35594B, 0xFF2D5442, 0xFF4E7563, 0xFF244B39, 0xFF335A48, 0xFF446B59, 0xFF284F3D, 0xFF3E6553, 0xFF486F5D, 0xFF254F3B, 0xFF2F5945, 0xFF426956, 0xFF274E3B, 0xFF375C4A, 0xFF406553, 0xFF1A3C2B, 0xFF385A49, 0xFF456A59,
        0xFF436250, 0xFF365746, 0xFF315142, 0xFF616C4C, 0xFF9B864D, 0xFFA98A4A, 0xFFA6915A, 0xFF887749, 0xFF373F1A, 0xFF314431, 0xFF757A74, 0xFF84817A, 0xFFD4D7C6, 0xFFDCDFCE, 0xFFA9A097, 0xFF908977, 0xFF88816F, 0xFF8C8573, 0xFF9A9381, 0xFFA69F8D,
        0xFFAAA391, 0xFFA09987, 0xFF8F8876, 0xFF928977, 0xFF968F7C, 0xFF9D9684, 0xFF958F7F, 0xFF8E8B7C, 0xFFA09E91, 0xFFCBCCBE, 0xFFD4D4C8, 0xFF6A8979, 0xFF618271, 0xFF628474, 0xFF688A7A, 0xFF557163, 0x009BACA4, 0x00F7FDF9, 0x00FEFDFB, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x00F8FFFF, 0xFF83A197, 0xFF4B7466, 0xFF678D80, 0xFF5C8074, 0xFF275143, 0xFF174737, 0xFF244C43, 0xFF1D453C, 0xFF153D34, 0xFF173F36, 0xFF264E45,
        0xFF1D453C, 0xFF10382F, 0xFF244C43, 0xFF658475, 0xFF506A5D, 0xFF526359, 0xFFAFBAB2, 0xFFEBF2EB, 0xFFE2E9E2, 0xFFD4DED6, 0xFFDAE5DD, 0xFFE2E4E1, 0xFFF3F8F4, 0xFFBFCAC4, 0xFF5F736A, 0xFF849E93, 0xFF527466, 0xFF638C7C, 0xFF537E6D, 0xFF5B7D6F,
        0xFF5F8173, 0xFF567A6C, 0xFF5C8072, 0xFF5D8374, 0xFF527869, 0xFF53796A, 0xFF537C6C, 0xFF3C6255, 0xFF476D60, 0xFF3A6053, 0xFF486E5F, 0xFF6E9485, 0xFF456C5A, 0xFF436A58, 0xFF224937, 0xFF3B6250, 0xFF2A513F, 0xFF375E4C, 0xFF3A614F, 0xFF274E3C,
        0xFF426957, 0xFF517866, 0xFF618876, 0xFF5B8472, 0xFF5C8573, 0xFF5C8573, 0xFF5B8472, 0xFF5C8573, 0xFF5E8775, 0xFF5E8775, 0xFF5C8573, 0xFF618577, 0xFF5E8573, 0xFF4E7362, 0xFF1A3225, 0xFF626C63, 0xFF878A83, 0xFF2B372B, 0xFF162D1D, 0xFF172D20,
        0xFF2A3E32, 0xFF0D1D12, 0xFF5D6A60, 0xFF505D53, 0xFF2A3E32, 0xFF1E3D2D, 0xFF466D5A, 0xFF537E6A, 0xFF5C7D6C, 0xFF5D7165, 0xFFA2ACA3, 0xFFB5BFB6, 0xFF5A6C60, 0xFF375344, 0xFF658A78, 0xFF446B58, 0xFF345947, 0xFF345646, 0xFF385446, 0xFF536B5E,
        0xFF64786F, 0xFF64736C, 0xFF38453E, 0xFF263437, 0xFF202B23, 0xFF262F1E, 0xFF3D3F32, 0xFF1F1E1A, 0xFF1E1C21, 0xFF1E1C1D, 0xFF23241E, 0xFF15201A, 0xFF18211C, 0xFF1F2521, 0xFF232726, 0xFF1F2523, 0xFF1C2625, 0xFF1B2B28, 0xFF142723, 0xFF162B26,
        0xFF172623, 0xFF1E2726, 0xFF1A1C1B, 0xFF1D1D1D, 0xFF161614, 0xFF181A17, 0xFF151A16, 0xFF1B1D1C, 0xFF151716, 0xFF181A19, 0xFF171918, 0xFF191919, 0xFF1E1C1D, 0xFF1C1819, 0xFF1F191B, 0xFF171612, 0xFF191516, 0xFF211D1E, 0xFF242118, 0xFF50533E,
        0xFF475139, 0xFF1F2D20, 0xFF2B3B3B, 0xFF25342F, 0xFF566A61, 0xFF516D61, 0xFF355747, 0xFF345947, 0xFF335A45, 0xFF315941, 0xFF365C45, 0xFF39604D, 0xFF385F4C, 0xFF3A5F4D, 0xFF3B604E, 0xFF3C614F, 0xFF3E604F, 0xFF406251, 0xFF426453, 0xFF405F4D,
        0xFF3E614D, 0xFF3B624F, 0xFF37624E, 0xFF36634F, 0xFF386350, 0xFF3A6351, 0xFF3C6253, 0xFF3E6352, 0xFF3E6352, 0xFF3F6453, 0xFF3D6251, 0xFF3C6150, 0xFF3C6150, 0xFF3D6251, 0xFF3F6453, 0xFF3E6352, 0xFF416655, 0xFF426756, 0xFF3F6453, 0xFF3E6352,
        0xFF3E6352, 0xFF3B604F, 0xFF365B4A, 0xFF446B59, 0xFF2A513F, 0xFF355C4A, 0xFF486F5D, 0xFF274E3C, 0xFF2B5240, 0xFF4A715F, 0xFF244B39, 0xFF36604C, 0xFF446E5A, 0xFF1D4431, 0xFF375E4B, 0xFF436856, 0xFF224735, 0xFF3E604F, 0xFF466857, 0xFF3B604F,
        0xFF385745, 0xFF315241, 0xFF345445, 0xFF657050, 0xFF9C874E, 0xFFA88949, 0xFFA28D56, 0xFF877648, 0xFF38401B, 0xFF304330, 0xFF747973, 0xFF85827B, 0xFFD4D7C6, 0xFFDBDECD, 0xFFAAA198, 0xFF9E9785, 0xFF968F7D, 0xFF968F7D, 0xFF9C9583, 0xFFA29B89,
        0xFFA8A18F, 0xFFA8A18F, 0xFFA09987, 0xFF9E9784, 0xFF9E9785, 0xFFA6A090, 0xFF9C9988, 0xFF868475, 0xFF9A9B8D, 0xFFC1C1B5, 0xFFB5B7AA, 0xFF607F6F, 0xFF638473, 0xFF628474, 0xFF638575, 0xFF516D5F, 0x0098A9A1, 0x00F7FDF9, 0x00FFFFFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFCFE, 0x00F6FFFD, 0xFF7C9A90, 0xFF497264, 0xFF688E81, 0xFF62867A, 0xFF3F695B, 0xFF346454, 0xFF325A51, 0xFF2F574E, 0xFF2C544B, 0xFF315950, 0xFF325A51,
        0xFF264E45, 0xFF274F46, 0xFF386057, 0xFF6B8A7B, 0xFF6F8B7D, 0xFF647C6F, 0xFF899D92, 0xFF99AAA0, 0xFF8C9D93, 0xFF8CA095, 0xFFA0B4A9, 0xFF8D9C95, 0xFF8A9B93, 0xFF758A81, 0xFF415B50, 0xFF69897C, 0xFF466C5D, 0xFF5E8978, 0xFF5A8675, 0xFF5C8776,
        0xFF5F8A79, 0xFF5D8877, 0xFF5C8877, 0xFF5A8675, 0xFF5A8675, 0xFF5D8C7A, 0xFF5B8A78, 0xFF547A6D, 0xFF3A6053, 0xFF42685B, 0xFF416758, 0xFF53796A, 0xFF4A715F, 0xFF436A58, 0xFF1E4533, 0xFF416856, 0xFF274E3C, 0xFF3E6553, 0xFF325947, 0xFF244B39,
        0xFF456C5A, 0xFF466D5B, 0xFF628977, 0xFF5E8775, 0xFF608977, 0xFF5F8876, 0xFF5C8573, 0xFF5C8573, 0xFF5F8876, 0xFF5F8876, 0xFF5C8573, 0xFF67897B, 0xFF598270, 0xFF38614F, 0xFF244032, 0xFF758177, 0xFFA9ACA5, 0xFF677166, 0xFF122416, 0xFF1A3023,
        0xFF112116, 0xFF3B453C, 0xFF8E988F, 0xFF68786D, 0xFF435B4D, 0xFF1D3E2D, 0xFF577C6A, 0xFF5E8975, 0xFF5A7F6D, 0xFF405B4C, 0xFF415347, 0xFF405045, 0xFF344A3D, 0xFF486958, 0xFF507764, 0xFF335A47, 0xFF305543, 0xFF395B4B, 0xFF3A5648, 0xFF51695C,
        0xFF596D64, 0xFF5E6D66, 0xFF3E4B44, 0xFF2D3C3F, 0xFF26332A, 0xFF293221, 0xFF3B3F31, 0xFF1D1C18, 0xFF19171C, 0xFF161616, 0xFF1E1F19, 0xFF15221B, 0xFF141F19, 0xFF1C221E, 0xFF1F2322, 0xFF1D2120, 0xFF202626, 0xFF1D2726, 0xFF182422, 0xFF14201E,
        0xFF131D1C, 0xFF1F2525, 0xFF191B1A, 0xFF1E1C1D, 0xFF181715, 0xFF191816, 0xFF191816, 0xFF1C1C1C, 0xFF161415, 0xFF161415, 0xFF181415, 0xFF1E1A1B, 0xFF201C1D, 0xFF181415, 0xFF161415, 0xFF1E1D19, 0xFF171516, 0xFF201C1D, 0xFF25251B, 0xFF4C513B,
        0xFF3B472F, 0xFF19291C, 0xFF2E4040, 0xFF2E3D38, 0xFF465A51, 0xFF3F5B4F, 0xFF345646, 0xFF365B49, 0xFF345B46, 0xFF345C44, 0xFF385E47, 0xFF355C49, 0xFF355C49, 0xFF345B48, 0xFF365B49, 0xFF375C4A, 0xFF395E4C, 0xFF3C5E4D, 0xFF3B5D4C, 0xFF3B5C49,
        0xFF3E614D, 0xFF3D6451, 0xFF3A6551, 0xFF396451, 0xFF3A6351, 0xFF3C6150, 0xFF3C5E50, 0xFF436555, 0xFF446656, 0xFF456757, 0xFF456757, 0xFF446656, 0xFF426454, 0xFF426454, 0xFF436555, 0xFF456757, 0xFF466858, 0xFF446656, 0xFF406252, 0xFF3F6151,
        0xFF426454, 0xFF456757, 0xFF476959, 0xFF3B6250, 0xFF244B39, 0xFF486F5D, 0xFF29503E, 0xFF2E5543, 0xFF4D7462, 0xFF1F4634, 0xFF2E5543, 0xFF3E6854, 0xFF224C38, 0xFF3E6552, 0xFF3A614E, 0xFF234836, 0xFF375C4A, 0xFF456756, 0xFF436554, 0xFF557A69,
        0xFF52715F, 0xFF456655, 0xFF3B5B4C, 0xFF616C4C, 0xFF9A854C, 0xFFA98A4A, 0xFFA08B54, 0xFF887749, 0xFF39411C, 0xFF304330, 0xFF737872, 0xFF87847D, 0xFFD5D8C7, 0xFFDADDCC, 0xFFA9A097, 0xFFA39C8A, 0xFF9F9886, 0xFFA09987, 0xFFA09987, 0xFF9D9684,
        0xFFA19A88, 0xFFA8A18F, 0xFFA9A290, 0xFFAAA391, 0xFF9F9A87, 0xFFA9A393, 0xFFA19E8F, 0xFF7E7C6F, 0xFF97978B, 0xFFBFC1B4, 0xFFA9ABA0, 0xFF5B7A6A, 0xFF648574, 0xFF618373, 0xFF618373, 0xFF516D5F, 0x0099AAA2, 0x00FAFFFC, 0x00FFFFFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFCFE, 0x00F6FFFD, 0xFF78968C, 0xFF4A7365, 0xFF6A9083, 0xFF6D9185, 0xFF5E887A, 0xFF5C8C7C, 0xFF679082, 0xFF628B7D, 0xFF5E8779, 0xFF5F887A, 0xFF628B7D,
        0xFF638C7E, 0xFF628B7D, 0xFF5C8577, 0xFF5F7F70, 0xFF648475, 0xFF5D7D6E, 0xFF688778, 0xFF5C7B6C, 0xFF5B7A6B, 0xFF658475, 0xFF6A897A, 0xFF668478, 0xFF567468, 0xFF68887B, 0xFF547668, 0xFF557B6C, 0xFF507969, 0xFF6B9685, 0xFF5A8574, 0xFF5C8776,
        0xFF5B8675, 0xFF5E8A79, 0xFF618D7C, 0xFF598876, 0xFF568573, 0xFF598876, 0xFF588976, 0xFF62887B, 0xFF577D70, 0xFF3E6457, 0xFF446A5B, 0xFF3A6051, 0xFF2C5341, 0xFF4D7462, 0xFF204735, 0xFF355B4C, 0xFF375D4E, 0xFF325849, 0xFF3B6152, 0xFF355B4C,
        0xFF365C4D, 0xFF496F60, 0xFF4D7364, 0xFF5D8674, 0xFF638C7A, 0xFF648D7B, 0xFF608977, 0xFF5E8775, 0xFF608977, 0xFF608977, 0xFF5C8573, 0xFF5E8072, 0xFF739C8A, 0xFF45705D, 0xFF204031, 0xFF68756B, 0xFFA8ABA4, 0xFFAEB4AA, 0xFF8A968A, 0xFF5D7366,
        0xFF6E7B71, 0xFFAAADA6, 0xFFA3A8A1, 0xFF8C9C91, 0xFF587465, 0xFF305241, 0xFF5F8170, 0xFF54816C, 0xFF5C8370, 0xFF395848, 0xFF21392B, 0xFF21372A, 0xFF223E2F, 0xFF385A49, 0xFF335A47, 0xFF335A47, 0xFF395E4C, 0xFF3B5D4D, 0xFF395547, 0xFF60786B,
        0xFF687C73, 0xFF4C5B54, 0xFF24312A, 0xFF202B2F, 0xFF1C271F, 0xFF232A1A, 0xFF383C2E, 0xFF1B1C17, 0xFF16171B, 0xFF151918, 0xFF232A22, 0xFF26352E, 0xFF1A2720, 0xFF191F1B, 0xFF171918, 0xFF161616, 0xFF1C1D1F, 0xFF1C2021, 0xFF1C2222, 0xFF1C2021,
        0xFF1B1F20, 0xFF222627, 0xFF1B1D1C, 0xFF1E1E1E, 0xFF171715, 0xFF1A1A18, 0xFF1C1B19, 0xFF1B1D1C, 0xFF1B1B1B, 0xFF1E1C1D, 0xFF1C1819, 0xFF1B1718, 0xFF1A1819, 0xFF1B1D1C, 0xFF292D2C, 0xFF212620, 0xFF121413, 0xFF161616, 0xFF1F2116, 0xFF464B35,
        0xFF364028, 0xFF162417, 0xFF2E3C3D, 0xFF27332F, 0xFF4C5D55, 0xFF51685E, 0xFF3F5E4F, 0xFF375948, 0xFF365C47, 0xFF3F624C, 0xFF41644E, 0xFF3D6451, 0xFF3F6452, 0xFF3E6351, 0xFF3E6351, 0xFF426453, 0xFF456756, 0xFF456655, 0xFF436453, 0xFF476654,
        0xFF486956, 0xFF456A58, 0xFF446B58, 0xFF446B59, 0xFF456A59, 0xFF476758, 0xFF4A665A, 0xFF456757, 0xFF476959, 0xFF486A5A, 0xFF486A5A, 0xFF466858, 0xFF456757, 0xFF446656, 0xFF446656, 0xFF456757, 0xFF486A5A, 0xFF4A6C5C, 0xFF496B5B, 0xFF476959,
        0xFF456757, 0xFF436555, 0xFF416353, 0xFF3E6553, 0xFF4F7664, 0xFF264D3B, 0xFF2F5644, 0xFF4A715F, 0xFF2E5543, 0xFF305745, 0xFF486F5D, 0xFF204A36, 0xFF335D49, 0xFF49705D, 0xFF254C39, 0xFF315644, 0xFF496E5C, 0xFF355746, 0xFF5B7D6C, 0xFF587D6C,
        0xFF597866, 0xFF4C6D5C, 0xFF385849, 0xFF596444, 0xFF99844B, 0xFFAD8E4E, 0xFFA18C55, 0xFF8A794B, 0xFF3C441F, 0xFF314431, 0xFF737872, 0xFF88857E, 0xFFD6D9C8, 0xFFD9DCCB, 0xFFA9A097, 0xFF9F9886, 0xFF9F9886, 0xFFA49D8B, 0xFFA69F8D, 0xFFA29B89,
        0xFFA29B89, 0xFFA69F8D, 0xFFA49D8B, 0xFFA6A18E, 0xFF9A9484, 0xFF9F9C8B, 0xFF9D9B8C, 0xFF7C7D6F, 0xFF99998D, 0xFFC0C2B7, 0xFFB0B2A7, 0xFF5E7D6D, 0xFF688978, 0xFF648676, 0xFF668878, 0xFF536F61, 0x0098A9A1, 0x00F9FFFB, 0x00FDFCFA, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x00F8FFFF, 0xFF7A988E, 0xFF4A7365, 0xFF63897C, 0xFF63877B, 0xFF5C8678, 0xFF5D8D7D, 0xFF5E8573, 0xFF658C7A, 0xFF6A917F, 0xFF638A78, 0xFF5E8573,
        0xFF5F8674, 0xFF628977, 0xFF648B79, 0xFF6B8D7D, 0xFF658A79, 0xFF618876, 0xFF699280, 0xFF648F7C, 0xFF6C9784, 0xFF6A9381, 0xFF628B79, 0xFF5F8B7A, 0xFF6F9B8A, 0xFF6E9988, 0xFF5F8A79, 0xFF628B7B, 0xFF638C7C, 0xFF5C8273, 0xFF5E8475, 0xFF648D7D,
        0xFF608979, 0xFF5D8877, 0xFF5C8776, 0xFF598574, 0xFF558170, 0xFF568271, 0xFF5B8A78, 0xFF5B8174, 0xFF688E81, 0xFF52786B, 0xFF355B4C, 0xFF406657, 0xFF3F6654, 0xFF254C3A, 0xFF466D5B, 0xFF2E5445, 0xFF3E6455, 0xFF2F5546, 0xFF3F6556, 0xFF406657,
        0xFF315748, 0xFF426859, 0xFF3E6455, 0xFF4F7866, 0xFF5A8371, 0xFF618A78, 0xFF5F8876, 0xFF5E8775, 0xFF618A78, 0xFF608977, 0xFF5D8674, 0xFF618375, 0xFF58816F, 0xFF5B8874, 0xFF224434, 0xFF5A6A5F, 0xFFABAEA7, 0xFF8C8F86, 0xFF858D82, 0xFF8EA497,
        0xFF97A198, 0xFF8A8984, 0xFFA6A9A2, 0xFF96A89C, 0xFF254635, 0xFF446655, 0xFF6A8B7A, 0xFF598671, 0xFF537D69, 0xFF456756, 0xFF597566, 0xFF5F7B6C, 0xFF274636, 0xFF244937, 0xFF3C6350, 0xFF2E5542, 0xFF2F5442, 0xFF305242, 0xFF375345, 0xFF6D8578,
        0xFF8EA299, 0xFF606F68, 0xFF26332C, 0xFF181D23, 0xFF1A1F19, 0xFF262A1B, 0xFF3D3F32, 0xFF20221D, 0xFF1A1F22, 0xFF1B2622, 0xFF2E3B31, 0xFF3C4D45, 0xFF27342D, 0xFF1E2722, 0xFF1B1D1C, 0xFF191718, 0xFF1C1A1D, 0xFF19191B, 0xFF1A1B1D, 0xFF1C1A1D,
        0xFF1C1A1D, 0xFF1D1E20, 0xFF171B1A, 0xFF1C201F, 0xFF161B17, 0xFF1D221E, 0xFF1B201C, 0xFF111A17, 0xFF151918, 0xFF1D1D1D, 0xFF1A1819, 0xFF191718, 0xFF191D1C, 0xFF252E2B, 0xFF434F4B, 0xFF2F3A32, 0xFF18211E, 0xFF181C1B, 0xFF22251A, 0xFF4C513B,
        0xFF3D452E, 0xFF151D12, 0xFF202A2C, 0xFF29322F, 0xFF58655E, 0xFF556A61, 0xFF3B5749, 0xFF3C5B4B, 0xFF426350, 0xFF41624D, 0xFF3B5B46, 0xFF3F6150, 0xFF416352, 0xFF416352, 0xFF416251, 0xFF426352, 0xFF476656, 0xFF466555, 0xFF436252, 0xFF486553,
        0xFF446351, 0xFF436453, 0xFF446655, 0xFF456757, 0xFF476657, 0xFF4B6558, 0xFF50655C, 0xFF446655, 0xFF456756, 0xFF466857, 0xFF456756, 0xFF436554, 0xFF426453, 0xFF426453, 0xFF436554, 0xFF436554, 0xFF446655, 0xFF456756, 0xFF456756, 0xFF456756,
        0xFF456756, 0xFF466857, 0xFF476958, 0xFF4A715F, 0xFF466D5B, 0xFF365D4B, 0xFF446B59, 0xFF2B5240, 0xFF264D3B, 0xFF4C7361, 0xFF234A38, 0xFF36604C, 0xFF4B7561, 0xFF1F4633, 0xFF325946, 0xFF436856, 0xFF385D4B, 0xFF638574, 0xFF5A7C6B, 0xFF597E6D,
        0xFF5C7B69, 0xFF4F705F, 0xFF3A5A4B, 0xFF5A6545, 0xFF9C874E, 0xFFB09151, 0xFFA08B54, 0xFF8C7B4D, 0xFF3E4621, 0xFF324532, 0xFF737872, 0xFF89867F, 0xFFD6D9C8, 0xFFD8DBCA, 0xFFA89F96, 0xFF98917F, 0xFF958E7C, 0xFF9C9583, 0xFFA69F8D, 0xFFA9A290,
        0xFFA9A290, 0xFFA49D8B, 0xFF999280, 0xFF999383, 0xFF969080, 0xFF949182, 0xFF949285, 0xFF7B7B6F, 0xFF8B8D82, 0xFFA6A89D, 0xFFA1A49B, 0xFF5A7969, 0xFF608170, 0xFF5C7E6E, 0xFF628474, 0xFF506C5E, 0x0095A69E, 0x00FAFFFC, 0x00FBFAF8, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFF9FB, 0x00F8FFFF, 0xFF7C9A90, 0xFF4F786A, 0xFF678D80, 0xFF63877B, 0xFF5F897B, 0xFF5F8F7F, 0xFF648B78, 0xFF648B78, 0xFF658C79, 0xFF658C79, 0xFF688F7C,
        0xFF658C79, 0xFF618875, 0xFF6B927F, 0xFF628776, 0xFF638C7A, 0xFF598672, 0xFF578672, 0xFF53846F, 0xFF52836E, 0xFF467561, 0xFF4F7E6A, 0xFF467B67, 0xFF407561, 0xFF447562, 0xFF608F7D, 0xFF55806F, 0xFF628879, 0xFF618577, 0xFF66887A, 0xFF608677,
        0xFF628879, 0xFF5D8374, 0xFF588171, 0xFF608979, 0xFF5F8A79, 0xFF588372, 0xFF5D8877, 0xFF648A7D, 0xFF5B8174, 0xFF658B7E, 0xFF5D8374, 0xFF3D6354, 0xFF426957, 0xFF2F5644, 0xFF365D4B, 0xFF3F6556, 0xFF2B5142, 0xFF456B5C, 0xFF2C5243, 0xFF325849,
        0xFF466C5D, 0xFF264C3D, 0xFF496F60, 0xFF3D6654, 0xFF4C7563, 0xFF598270, 0xFF5C8573, 0xFF5D8674, 0xFF608977, 0xFF618A78, 0xFF5D8674, 0xFF628476, 0xFF628B79, 0xFF64917D, 0xFF486A5A, 0xFF2B3D31, 0xFF5F625B, 0xFFAEAFA7, 0xFF8A9086, 0xFF788E81,
        0xFF8C968D, 0xFFB3B0AB, 0xFF90918B, 0xFF36483C, 0xFF325342, 0xFF5E8371, 0xFF658675, 0xFF55826D, 0xFF5B8571, 0xFF608573, 0xFF608170, 0xFF4F6E5E, 0xFF395A49, 0xFF456A58, 0xFF2B5541, 0xFF3B624F, 0xFF325745, 0xFF385A4A, 0xFF3B5749, 0xFF5B7366,
        0xFF8B9F96, 0xFF617069, 0xFF1B2821, 0xFF14141C, 0xFF181A15, 0xFF242618, 0xFF3A3C2F, 0xFF181D17, 0xFF0D1517, 0xFF0D1C17, 0xFF213529, 0xFF24382F, 0xFF101F18, 0xFF101914, 0xFF161817, 0xFF1A1819, 0xFF201B1F, 0xFF181619, 0xFF19191B, 0xFF201A1E,
        0xFF1E191D, 0xFF1A1A1C, 0xFF141817, 0xFF18211E, 0xFF0F1813, 0xFF161F1A, 0xFF0F1813, 0xFF14201C, 0xFF141D1A, 0xFF1A1E1D, 0xFF1B1B1B, 0xFF191B1A, 0xFF131C19, 0xFF172621, 0xFF31443E, 0xFF213127, 0xFF0C1814, 0xFF0E1412, 0xFF1C1F14, 0xFF4E513C,
        0xFF434832, 0xFF12180E, 0xFF101519, 0xFF262C2A, 0xFF646F69, 0xFF61726A, 0xFF3D5548, 0xFF395546, 0xFF3B5A48, 0xFF3D5A46, 0xFF425F4B, 0xFF3C5D4C, 0xFF3E5F4E, 0xFF3F604F, 0xFF3F5E4E, 0xFF405F4F, 0xFF466253, 0xFF456152, 0xFF425E4F, 0xFF466150,
        0xFF425F4D, 0xFF405F4F, 0xFF436453, 0xFF466556, 0xFF476154, 0xFF495F53, 0xFF4C6057, 0xFF446554, 0xFF446554, 0xFF436453, 0xFF426352, 0xFF406150, 0xFF3F604F, 0xFF406150, 0xFF426352, 0xFF446554, 0xFF446554, 0xFF446554, 0xFF456655, 0xFF446554,
        0xFF436453, 0xFF436453, 0xFF446554, 0xFF3E6553, 0xFF3F6654, 0xFF446B59, 0xFF274E3C, 0xFF335A48, 0xFF456C5A, 0xFF274E3C, 0xFF325947, 0xFF4D7763, 0xFF143E2A, 0xFF3E6552, 0xFF446B58, 0xFF264B39, 0xFF416654, 0xFF507261, 0xFF486A59, 0xFF456A59,
        0xFF456452, 0xFF395A49, 0xFF284839, 0xFF4E5939, 0xFF968148, 0xFFAE8F4F, 0xFFA08B54, 0xFF8D7C4E, 0xFF404823, 0xFF324532, 0xFF737872, 0xFF8A8780, 0xFFD7DAC9, 0xFFD8DBCA, 0xFFA89F96, 0xFF908977, 0xFF898270, 0xFF8D8674, 0xFF9C9583, 0xFFA9A290,
        0xFFACA593, 0xFFA09987, 0xFF8D8674, 0xFF8C8676, 0xFF93907F, 0xFF908D7E, 0xFF959386, 0xFF86867A, 0xFF8C8E83, 0xFF979A8F, 0xFF9DA097, 0xFF658474, 0xFF658675, 0xFF5E8070, 0xFF658777, 0xFF516D5F, 0x0098A9A1, 0x00FBFFFD, 0x00FFFFFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFCFE, 0x00F7FFFF, 0xFF79998E, 0xFF477163, 0xFF628B7D, 0xFF688E81, 0xFF3F6B5C, 0xFF396C5B, 0xFF406A5E, 0xFF446E62, 0xFF456F63, 0xFF436D61, 0xFF3F695D,
        0xFF3F695D, 0xFF426C60, 0xFF446E62, 0xFF5D7F6F, 0xFF3A594A, 0xFF3D5548, 0xFF53675C, 0xFF3F5549, 0xFF274335, 0xFF1E4332, 0xFF1E4936, 0xFF20382B, 0xFF243E31, 0xFF2B4739, 0xFF27473A, 0xFF315547, 0xFF3A6053, 0xFF5D8678, 0xFF628C7E, 0xFF5C8C76,
        0xFF5C8974, 0xFF588570, 0xFF618E7A, 0xFF5F8A77, 0xFF547F6E, 0xFF5D8676, 0xFF5E8777, 0xFF5C8575, 0xFF5F8878, 0xFF578070, 0xFF618A7A, 0xFF527B6B, 0xFF3D6656, 0xFF416A5A, 0xFF2E5747, 0xFF3A6051, 0xFF3E6455, 0xFF294F40, 0xFF476D5E, 0xFF305647,
        0xFF355B4C, 0xFF466C5D, 0xFF254B3C, 0xFF4F7566, 0xFF33594A, 0xFF4D7364, 0xFF688E7F, 0xFF5C8273, 0xFF63897A, 0xFF5B8172, 0xFF618778, 0xFF608B7A, 0xFF5B8474, 0xFF628B7B, 0xFF638C7C, 0xFF43695A, 0xFF1D4334, 0xFF2C5042, 0xFF577B6D, 0xFF648477,
        0xFF618174, 0xFF3F6153, 0xFF214335, 0xFF305446, 0xFF587C6E, 0xFF668A7C, 0xFF5F8375, 0xFF578070, 0xFF5A8373, 0xFF608979, 0xFF477060, 0xFF305949, 0xFF456E5E, 0xFF315A4A, 0xFF3B6454, 0xFF2E5445, 0xFF335749, 0xFF355749, 0xFF3A584C, 0xFF4E685D,
        0xFF667B72, 0xFF64756D, 0xFF2B3C34, 0xFF141611, 0xFF1E1F1A, 0xFF2A2925, 0xFF35312E, 0xFF191814, 0xFF1D1F1A, 0xFF111811, 0xFF141F17, 0xFF171918, 0xFF181A19, 0xFF191B1A, 0xFF191B1A, 0xFF181A19, 0xFF181A19, 0xFF181A19, 0xFF181A19, 0xFF181818,
        0xFF191919, 0xFF191919, 0xFF1A1A1A, 0xFF1A1A1A, 0xFF191919, 0xFF181818, 0xFF181818, 0xFF171B1A, 0xFF181C1B, 0xFF1B1D1C, 0xFF1A1C1B, 0xFF1A1A1A, 0xFF1A1819, 0xFF1A1819, 0xFF1A1819, 0xFF17181A, 0xFF141414, 0xFF1A1615, 0xFF1C1714, 0xFF3C3932,
        0xFF39392F, 0xFF1F2315, 0xFF121909, 0xFF1A251F, 0xFF606F68, 0xFF70857C, 0xFF4C685A, 0xFF416152, 0xFF456757, 0xFF406251, 0xFF416251, 0xFF456757, 0xFF436555, 0xFF426454, 0xFF496B5B, 0xFF4D6F5F, 0xFF4C6E5E, 0xFF4D6F5F, 0xFF4C6E5E, 0xFF476C5B,
        0xFF476C5B, 0xFF466B5A, 0xFF456A59, 0xFF456A59, 0xFF446958, 0xFF436857, 0xFF436857, 0xFF446957, 0xFF446957, 0xFF446957, 0xFF446957, 0xFF446957, 0xFF446957, 0xFF446957, 0xFF446957, 0xFF456A59, 0xFF456A59, 0xFF456A59, 0xFF446958, 0xFF446958,
        0xFF446958, 0xFF446958, 0xFF436857, 0xFF486D5B, 0xFF426755, 0xFF476C5A, 0xFF4C715F, 0xFF3E6351, 0xFF2B503E, 0xFF345947, 0xFF4D7260, 0xFF1E4331, 0xFF406553, 0xFF436856, 0xFF284D3B, 0xFF355A48, 0xFF496E5C, 0xFF416654, 0xFF426755, 0xFF436356,
        0xFF405D4B, 0xFF355445, 0xFF365448, 0xFF6B7457, 0xFF99824C, 0xFFAF8D50, 0xFFA28B55, 0xFF897749, 0xFF3A421D, 0xFF30412F, 0xFF727771, 0xFF8B8680, 0xFFDADBCB, 0xFFDBDCCC, 0xFFA79D94, 0xFF8D8674, 0xFF88816F, 0xFF867F6D, 0xFF938C7A, 0xFFA39C8A,
        0xFFA7A08E, 0xFF9F9886, 0xFF938C7A, 0xFF938C79, 0xFF8F8875, 0xFF989380, 0xFF969080, 0xFF8B8879, 0xFF979387, 0xFFB5B3A6, 0xFFD7D5C9, 0xFF788A7E, 0xFF627A6C, 0xFF688879, 0xFF638877, 0xFF4E6E5F, 0x0092A79E, 0x00FAFFFC, 0x00FFFEFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDF7F9, 0x00F7FFFF, 0xFF79998E, 0xFF4B7567, 0xFF668F81, 0xFF5F8578, 0xFF265243, 0xFF1A4D3C, 0xFF1B4539, 0xFF1E483C, 0xFF0D372B, 0xFF133D31, 0xFF234D41,
        0xFF174135, 0xFF143E32, 0xFF234D41, 0xFF607C6E, 0xFF4A6054, 0xFF556259, 0xFF929992, 0xFFA7AEA7, 0xFFA4AFA7, 0xFF9BAFA4, 0xFF9FB7AA, 0xFFA0B1A7, 0xFFA7BBB0, 0xFF82988C, 0xFF375146, 0xFF3A584C, 0xFF3B5B50, 0xFF63877B, 0xFF5F8578, 0xFF5E8975,
        0xFF638E7A, 0xFF608B77, 0xFF608B78, 0xFF608D79, 0xFF5F8B7A, 0xFF608C7B, 0xFF5D8978, 0xFF5C8575, 0xFF5E8777, 0xFF5A8373, 0xFF628B7B, 0xFF608979, 0xFF466F5F, 0xFF3D6656, 0xFF477060, 0xFF2A5041, 0xFF385E4F, 0xFF4A7061, 0xFF23493A, 0xFF4C7263,
        0xFF2E5445, 0xFF385E4F, 0xFF4B7162, 0xFF204637, 0xFF517768, 0xFF3D6354, 0xFF4D7364, 0xFF6F9586, 0xFF567C6D, 0xFF678D7E, 0xFF608677, 0xFF5E8978, 0xFF598272, 0xFF598272, 0xFF618A7A, 0xFF608677, 0xFF4A7061, 0xFF305446, 0xFF204436, 0xFF163629,
        0xFF254538, 0xFF345648, 0xFF486A5C, 0xFF608476, 0xFF6D9183, 0xFF688C7E, 0xFF608677, 0xFF5F8878, 0xFF557E6E, 0xFF517A6A, 0xFF345D4D, 0xFF456E5E, 0xFF2C5545, 0xFF406959, 0xFF2D5646, 0xFF3C6253, 0xFF3C6253, 0xFF355749, 0xFF4C6C5F, 0xFF415D51,
        0xFF5E756B, 0xFF64786F, 0xFF162A21, 0xFF181A15, 0xFF12130E, 0xFF2E2D29, 0xFF433F3C, 0xFF15110E, 0xFF161712, 0xFF181D17, 0xFF141E16, 0xFF181A17, 0xFF191B18, 0xFF191B18, 0xFF191B18, 0xFF181A17, 0xFF181A17, 0xFF191B18, 0xFF191B18, 0xFF1A1A18,
        0xFF1A1A18, 0xFF1A1A18, 0xFF1A1A18, 0xFF191917, 0xFF191917, 0xFF191917, 0xFF191917, 0xFF171916, 0xFF181A17, 0xFF1A1A18, 0xFF1A1A18, 0xFF1A1917, 0xFF1A1917, 0xFF1D1918, 0xFF1E1A19, 0xFF191917, 0xFF1D1C18, 0xFF1C1815, 0xFF292421, 0xFF5A5551,
        0xFF404038, 0xFF11140B, 0xFF161E13, 0xFF1F2C25, 0xFF607169, 0xFF6B8278, 0xFF426152, 0xFF365848, 0xFF3D5F4F, 0xFF3B5D4C, 0xFF3C5D4C, 0xFF3C5E4E, 0xFF345646, 0xFF3C5E4E, 0xFF446656, 0xFF3B5D4D, 0xFF3B5D4D, 0xFF426454, 0xFF3B5D4D, 0xFF406554,
        0xFF406554, 0xFF3F6453, 0xFF3F6453, 0xFF3E6352, 0xFF3D6251, 0xFF3D6251, 0xFF3D6251, 0xFF3E6351, 0xFF3E6351, 0xFF3E6351, 0xFF3E6351, 0xFF3E6351, 0xFF3E6351, 0xFF3E6351, 0xFF3E6351, 0xFF416655, 0xFF416655, 0xFF416655, 0xFF406554, 0xFF406554,
        0xFF406554, 0xFF406554, 0xFF406554, 0xFF3D6250, 0xFF436856, 0xFF436856, 0xFF3F6452, 0xFF406553, 0xFF426755, 0xFF385D4B, 0xFF274C3A, 0xFF395E4C, 0xFF456A58, 0xFF294E3C, 0xFF355A48, 0xFF4B705E, 0xFF3A5F4D, 0xFF3B604E, 0xFF416654, 0xFF3F6153,
        0xFF3F5C4A, 0xFF375647, 0xFF345246, 0xFF656E51, 0xFF98814B, 0xFFB18F52, 0xFFA28B55, 0xFF8D7B4D, 0xFF3C441F, 0xFF30412F, 0xFF717670, 0xFF8B8680, 0xFFD9DACA, 0xFFDADBCB, 0xFFA89E95, 0xFF9E9785, 0xFF9C9583, 0xFF9B9482, 0xFFA09987, 0xFFA59E8C,
        0xFFA7A08E, 0xFFA49D8B, 0xFFA09987, 0xFF9A9380, 0xFF948D7A, 0xFF9A9582, 0xFF958F7F, 0xFF888576, 0xFF949084, 0xFFB6B4A7, 0xFFDCDACE, 0xFF93A599, 0xFF657D6F, 0xFF5F7F70, 0xFF608574, 0xFF4F6F60, 0x0093A89F, 0x00F7FDF9, 0x00FFFCFB, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFCFE, 0x00F7FFFF, 0xFF77978C, 0xFF4A7466, 0xFF6A9385, 0xFF5E8477, 0xFF214D3E, 0xFF1B4E3D, 0xFF4D776B, 0xFF416B5F, 0xFF254F43, 0xFF2D574B, 0xFF3C665A,
        0xFF244E42, 0xFF1C463A, 0xFF2E584C, 0xFF61776B, 0xFF57675D, 0xFF696E68, 0xFFC6C2BF, 0xFFF3EBE9, 0xFFF7F2EF, 0xFFEAE9E5, 0xFFEDEFEA, 0xFFE9F0E9, 0xFFF1F8F1, 0xFFABB6AE, 0xFF3D4C45, 0xFF495D54, 0xFF425951, 0xFF5C776E, 0xFF516F65, 0xFF476656,
        0xFF3D5C4C, 0xFF375748, 0xFF385D4C, 0xFF3F6556, 0xFF376251, 0xFF335F4E, 0xFF417060, 0xFF628B7B, 0xFF5E8777, 0xFF5B8474, 0xFF588171, 0xFF648D7D, 0xFF618A7A, 0xFF446D5D, 0xFF3B6454, 0xFF486E5F, 0xFF294F40, 0xFF33594A, 0xFF456B5C, 0xFF1E4435,
        0xFF517768, 0xFF2B5142, 0xFF345A4B, 0xFF496F60, 0xFF204637, 0xFF4B7162, 0xFF3E6455, 0xFF4A7061, 0xFF688E7F, 0xFF5F8576, 0xFF577D6E, 0xFF598473, 0xFF608979, 0xFF628B7B, 0xFF618A7A, 0xFF668C7D, 0xFF6B9182, 0xFF668A7C, 0xFF5C8072, 0xFF618375,
        0xFF648678, 0xFF688A7C, 0xFF6D9183, 0xFF6E9284, 0xFF64887A, 0xFF5A8071, 0xFF5C8273, 0xFF648D7D, 0xFF527B6B, 0xFF3E6757, 0xFF497262, 0xFF2F5848, 0xFF507969, 0xFF2A5343, 0xFF3E6757, 0xFF2F5848, 0xFF355B4C, 0xFF426658, 0xFF305043, 0xFF4C6A5E,
        0xFF546E63, 0xFF41584E, 0xFF172E24, 0xFF161B15, 0xFF191B16, 0xFF1F1E1A, 0xFF231F1C, 0xFF130F0C, 0xFF23221E, 0xFF1A1B16, 0xFF10150F, 0xFF1A1914, 0xFF1A1914, 0xFF1A1914, 0xFF1A1914, 0xFF1A1914, 0xFF1A1914, 0xFF1A1914, 0xFF1A1914, 0xFF181712,
        0xFF171611, 0xFF171611, 0xFF161510, 0xFF161510, 0xFF161510, 0xFF161510, 0xFF15140F, 0xFF171611, 0xFF181712, 0xFF191813, 0xFF181712, 0xFF171611, 0xFF171611, 0xFF1A1712, 0xFF1B1813, 0xFF12120A, 0xFF1C1912, 0xFF1D1814, 0xFF15100C, 0xFF241F1C,
        0xFF21201C, 0xFF141915, 0xFF171D19, 0xFF1B2C24, 0xFF54685F, 0xFF607A6F, 0xFF406051, 0xFF345948, 0xFF395E4D, 0xFF375948, 0xFF365847, 0xFF355A49, 0xFF406554, 0xFF3E6352, 0xFF365B4A, 0xFF406554, 0xFF3C6150, 0xFF335847, 0xFF4A6F5E, 0xFF436857,
        0xFF426756, 0xFF426756, 0xFF426756, 0xFF416655, 0xFF406554, 0xFF406554, 0xFF406554, 0xFF416654, 0xFF416654, 0xFF416654, 0xFF416654, 0xFF416654, 0xFF416654, 0xFF416654, 0xFF416654, 0xFF406554, 0xFF406554, 0xFF406554, 0xFF3F6453, 0xFF3F6453,
        0xFF3F6453, 0xFF3F6453, 0xFF3F6453, 0xFF3F6452, 0xFF3F6452, 0xFF3E6351, 0xFF3E6351, 0xFF416654, 0xFF406553, 0xFF375C4A, 0xFF2C513F, 0xFF426755, 0xFF244937, 0xFF395E4C, 0xFF406553, 0xFF3E6351, 0xFF567B69, 0xFF5C816F, 0xFF608573, 0xFF5C7E70,
        0xFF577462, 0xFF486859, 0xFF345447, 0xFF5B6447, 0xFF927B45, 0xFFAE8E51, 0xFFA08953, 0xFF907E50, 0xFF3E4621, 0xFF2F402E, 0xFF70756F, 0xFF8B8680, 0xFFD8D9C9, 0xFFD9DACA, 0xFFA99F96, 0xFFA49D8B, 0xFFA49D8B, 0xFFA39C8A, 0xFF9F9886, 0xFF9E9785,
        0xFFA19A88, 0xFFA59E8C, 0xFFA9A290, 0xFFA29B88, 0xFF99927F, 0xFF9D9885, 0xFF958F7F, 0xFF858273, 0xFF908C80, 0xFFB5B3A6, 0xFFDFDDD1, 0xFFD5E7DB, 0xFF829A8C, 0xFF668677, 0xFF628776, 0xFF4F6F60, 0x0094A9A0, 0x00FAFFFC, 0x00FFFEFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x00F7FFFF, 0xFF75958A, 0xFF487264, 0xFF6C9587, 0xFF608679, 0xFF285445, 0xFF2C5F4E, 0xFF507A6E, 0xFF3E685C, 0xFF3E685C, 0xFF467064, 0xFF406A5E,
        0xFF265044, 0xFF184236, 0xFF2B5549, 0xFF5F7B6D, 0xFF5D7166, 0xFF777E77, 0xFFD5D1CE, 0xFFF3E9E8, 0xFFEFE3E3, 0xFFDDD5D3, 0xFFDAD5D2, 0xFFECE8E5, 0xFFF0EFEB, 0xFFB4B6B1, 0xFF595F5B, 0xFF58655E, 0xFF3D4E48, 0xFF334841, 0xFF344B43, 0xFF525D55,
        0xFF414E45, 0xFF35453B, 0xFF2E433A, 0xFF365045, 0xFF2E4C42, 0xFF214237, 0xFF32564A, 0xFF598473, 0xFF588372, 0xFF608B7A, 0xFF5A8574, 0xFF578271, 0xFF628D7C, 0xFF5C8776, 0xFF4B7665, 0xFF3C6253, 0xFF456B5C, 0xFF23493A, 0xFF3A6051, 0xFF456B5C,
        0xFF1F4536, 0xFF4F7566, 0xFF284E3F, 0xFF385E4F, 0xFF426859, 0xFF244A3B, 0xFF4B7162, 0xFF355B4C, 0xFF567C6D, 0xFF5F8576, 0xFF668C7D, 0xFF5D8877, 0xFF5F8878, 0xFF5D8676, 0xFF598272, 0xFF5B8172, 0xFF608677, 0xFF668A7C, 0xFF688C7E, 0xFF709284,
        0xFF719385, 0xFF6A8E80, 0xFF638779, 0xFF608476, 0xFF618778, 0xFF648A7B, 0xFF678D7E, 0xFF50796B, 0xFF416A5C, 0xFF40695B, 0xFF335C4E, 0xFF497264, 0xFF2D5648, 0xFF436C5E, 0xFF3A6355, 0xFF355E4E, 0xFF426859, 0xFF2A5041, 0xFF486A5C, 0xFF39594C,
        0xFF506E62, 0xFF607C70, 0xFF3C584C, 0xFF0E1911, 0xFF161D16, 0xFF141611, 0xFF161511, 0xFF1A1613, 0xFF302C29, 0xFF2B2A26, 0xFF31322D, 0xFF2F2F27, 0xFF2F2F27, 0xFF2F2F27, 0xFF303028, 0xFF313129, 0xFF303028, 0xFF2F2F27, 0xFF2E2E26, 0xFF32322A,
        0xFF33332B, 0xFF33332B, 0xFF33332B, 0xFF33332B, 0xFF313129, 0xFF303028, 0xFF303028, 0xFF35322B, 0xFF35322B, 0xFF33332B, 0xFF32322A, 0xFF303028, 0xFF303028, 0xFF313129, 0xFF31322A, 0xFF2F2F23, 0xFF343128, 0xFF2F2C25, 0xFF1E1915, 0xFF181715,
        0xFF202221, 0xFF1A2322, 0xFF0D1718, 0xFF2E433A, 0xFF536D62, 0xFF5E7C70, 0xFF4F7463, 0xFF4F7664, 0xFF557C6A, 0xFF547967, 0xFF577968, 0xFF517567, 0xFF5C8072, 0xFF4B6F61, 0xFF5B7F71, 0xFF45695B, 0xFF2D5143, 0xFF4D7163, 0xFF395D4F, 0xFF466B5A,
        0xFF466B5A, 0xFF466B5A, 0xFF456A59, 0xFF456A59, 0xFF456A59, 0xFF446958, 0xFF446958, 0xFF456A58, 0xFF456A58, 0xFF456A58, 0xFF456A58, 0xFF456A58, 0xFF456A58, 0xFF456A58, 0xFF456A58, 0xFF476C5B, 0xFF476C5B, 0xFF466B5A, 0xFF466B5A, 0xFF466B5A,
        0xFF466B5A, 0xFF456A59, 0xFF456A59, 0xFF496E5C, 0xFF436856, 0xFF436856, 0xFF496E5C, 0xFF476C5A, 0xFF406553, 0xFF416654, 0xFF4A6F5D, 0xFF193E2C, 0xFF385D4B, 0xFF3B604E, 0xFF446957, 0xFF5A7F6D, 0xFF5D8270, 0xFF5B806E, 0xFF587D6B, 0xFF5C7E70,
        0xFF5B7A68, 0xFF557566, 0xFF3E5E51, 0xFF5F6A4C, 0xFF98834C, 0xFFB19154, 0xFF9E8952, 0xFF8E7C4E, 0xFF3C441F, 0xFF2C3D2B, 0xFF6E736D, 0xFF8B8680, 0xFFD8D9C9, 0xFFD8D9C9, 0xFFAAA097, 0xFFA19A88, 0xFFA19A88, 0xFFA49D8B, 0xFFA19A88, 0xFFA29B89,
        0xFFA59E8C, 0xFFA19A88, 0xFFA09987, 0xFFA29B88, 0xFF99927F, 0xFF9D9885, 0xFF969080, 0xFF848172, 0xFF8C887C, 0xFFB1AFA2, 0xFFDFDDD1, 0xFFD6E8DC, 0xFF7A9284, 0xFF638374, 0xFF648978, 0xFF517162, 0x0098ADA4, 0x00FBFFFD, 0x00FFFEFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFBFD, 0x00F7FFFF, 0xFF77978C, 0xFF4B7567, 0xFF6C9587, 0xFF5C8275, 0xFF245041, 0xFF2B5E4D, 0xFF345E52, 0xFF2C564A, 0xFF467064, 0xFF4D776B, 0xFF4B7569,
        0xFF3A6458, 0xFF1A4438, 0xFF2F594D, 0xFF638877, 0xFF627E70, 0xFF86968C, 0xFFD4D6D1, 0xFFE2DEDB, 0xFFD9D4D1, 0xFFD1CDCA, 0xFFC9C8C4, 0xFFD8CECD, 0xFFE7DFDD, 0xFFC6C2BF, 0xFF919390, 0xFF7F8883, 0xFF5F6E69, 0xFF4A5F58, 0xFF677E76, 0xFF96928F,
        0xFF7B7A76, 0xFF595B58, 0xFF4D534F, 0xFF707C78, 0xFF859690, 0xFF5E736C, 0xFF3F5650, 0xFF638E7D, 0xFF5E8978, 0xFF5C8776, 0xFF608B7A, 0xFF598473, 0xFF598473, 0xFF65907F, 0xFF638E7D, 0xFF4F7566, 0xFF3B6152, 0xFF4D7364, 0xFF274D3E, 0xFF416758,
        0xFF43695A, 0xFF264C3D, 0xFF486E5F, 0xFF2A5041, 0xFF3F6556, 0xFF406657, 0xFF23493A, 0xFF527869, 0xFF3C6253, 0xFF5B8172, 0xFF587E6F, 0xFF5D8877, 0xFF5E8777, 0xFF5E8777, 0xFF5F8878, 0xFF628879, 0xFF608677, 0xFF618577, 0xFF618577, 0xFF608476,
        0xFF64887A, 0xFF65897B, 0xFF628678, 0xFF5F8576, 0xFF618778, 0xFF5C8575, 0xFF557E6E, 0xFF3D6658, 0xFF436C5E, 0xFF355E50, 0xFF446D5F, 0xFF3B6456, 0xFF3C6557, 0xFF3D6658, 0xFF3D6658, 0xFF446D5D, 0xFF2A5343, 0xFF476D5E, 0xFF3D6354, 0xFF517567,
        0xFF5E8072, 0xFF688A7C, 0xFF5F7F72, 0xFF41554A, 0xFF122218, 0xFF0B150D, 0xFF20221D, 0xFF191814, 0xFF21201C, 0xFF22211D, 0xFF1E1F1A, 0xFF1C1D15, 0xFF1D1E16, 0xFF1E1F17, 0xFF202119, 0xFF21221A, 0xFF202119, 0xFF1D1E16, 0xFF1A1B13, 0xFF202119,
        0xFF21221A, 0xFF23241C, 0xFF24251D, 0xFF24251D, 0xFF22231B, 0xFF202119, 0xFF1E1F17, 0xFF1E1E16, 0xFF1E1E16, 0xFF1E1E16, 0xFF1E1F17, 0xFF1D2017, 0xFF1E2118, 0xFF1C2218, 0xFF1C2218, 0xFF1B1D12, 0xFF25271C, 0xFF1B1B13, 0xFF1A1B16, 0xFF1F211E,
        0xFF0D1615, 0xFF0D1B1B, 0xFF2D403E, 0xFF617B70, 0xFF68867A, 0xFF66887A, 0xFF5E8573, 0xFF5E8775, 0xFF5F8674, 0xFF5F8472, 0xFF678978, 0xFF608677, 0xFF5E8475, 0xFF628879, 0xFF5A8071, 0xFF466C5D, 0xFF385E4F, 0xFF33594A, 0xFF345A4B, 0xFF3F6453,
        0xFF3F6453, 0xFF3F6453, 0xFF3F6453, 0xFF3F6453, 0xFF3E6352, 0xFF3E6352, 0xFF3E6352, 0xFF3F6452, 0xFF3F6452, 0xFF3F6452, 0xFF3F6452, 0xFF3F6452, 0xFF3F6452, 0xFF3F6452, 0xFF3F6452, 0xFF406554, 0xFF406554, 0xFF406554, 0xFF406554, 0xFF3F6453,
        0xFF3F6453, 0xFF3F6453, 0xFF3F6453, 0xFF375C4A, 0xFF3B604E, 0xFF3B604E, 0xFF385D4B, 0xFF3B604E, 0xFF406553, 0xFF3D6250, 0xFF335846, 0xFF3A5F4D, 0xFF486D5B, 0xFF456A58, 0xFF527765, 0xFF648977, 0xFF5F8472, 0xFF5B806E, 0xFF5C816F, 0xFF527668,
        0xFF547361, 0xFF4E7060, 0xFF335547, 0xFF596446, 0xFF98834C, 0xFFAF9153, 0xFF9F8A53, 0xFF897749, 0xFF39411C, 0xFF293A28, 0xFF6D726C, 0xFF8D8882, 0xFFD9DACA, 0xFFD6D7C7, 0xFFAAA097, 0xFF9B9482, 0xFF9A9381, 0xFFA49D8B, 0xFFA8A18F, 0xFFB0A997,
        0xFFB4AD9B, 0xFF9F9886, 0xFF928B79, 0xFF99927F, 0xFF918A77, 0xFF9A9582, 0xFF999383, 0xFF888576, 0xFF8D897D, 0xFFB1AFA2, 0xFFE0DED2, 0xFFBACCC0, 0xFF677F71, 0xFF5E7E6F, 0xFF618675, 0xFF4B6B5C, 0x0097ACA3, 0x00FBFFFD, 0x00FFFEFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFF9FB, 0x00F7FFFF, 0xFF77978C, 0xFF4D7769, 0xFF6B9486, 0xFF5D8376, 0xFF265243, 0xFF285B4A, 0xFF4C766A, 0xFF3F695D, 0xFF416B5F, 0xFF3E685C, 0xFF5C867A,
        0xFF568074, 0xFF214B3F, 0xFF2F594D, 0xFF557C6A, 0xFF577667, 0xFF91A298, 0xFFCCD3CC, 0xFFE1E2DD, 0xFFE3E2DE, 0xFFEBEDE8, 0xFFE5EAE4, 0xFFE4DAD9, 0xFFEDE5E3, 0xFFBFBEBA, 0xFF909591, 0xFF7D8A83, 0xFF50635D, 0xFF405A51, 0xFF577269, 0xFFA9ABA8,
        0xFF888D89, 0xFF606662, 0xFF5A6360, 0xFF828E8A, 0xFF99A8A3, 0xFF798986, 0xFF536662, 0xFF568271, 0xFF618D7C, 0xFF588473, 0xFF5A8675, 0xFF608C7B, 0xFF5A8675, 0xFF588473, 0xFF558170, 0xFF678D7E, 0xFF577D6E, 0xFF355B4C, 0xFF496F60, 0xFF274D3E,
        0xFF466C5D, 0xFF33594A, 0xFF345A4B, 0xFF43695A, 0xFF2A5041, 0xFF416758, 0xFF3B6152, 0xFF2D5344, 0xFF4F7566, 0xFF345A4B, 0xFF43695A, 0xFF3D6857, 0xFF446D5D, 0xFF446D5D, 0xFF3E6757, 0xFF3C6253, 0xFF3F6556, 0xFF44685A, 0xFF466A5C, 0xFF436759,
        0xFF3E6254, 0xFF3F6355, 0xFF43695A, 0xFF43695A, 0xFF406959, 0xFF406959, 0xFF3D6656, 0xFF41695E, 0xFF386055, 0xFF466E63, 0xFF3D655A, 0xFF3D655A, 0xFF497166, 0xFF375F54, 0xFF3D655A, 0xFF305949, 0xFF466F5F, 0xFF315A4A, 0xFF547D6D, 0xFF608979,
        0xFF63897A, 0xFF597F70, 0xFF587E6F, 0xFF638273, 0xFF2F493C, 0xFF102117, 0xFF101B13, 0xFF1B201A, 0xFF2B2D28, 0xFF23241F, 0xFF181A15, 0xFF1F241D, 0xFF1E231C, 0xFF1F241D, 0xFF21261F, 0xFF232821, 0xFF222720, 0xFF1F241D, 0xFF1B2019, 0xFF1A2119,
        0xFF1C231B, 0xFF1E251D, 0xFF1F261E, 0xFF1F261E, 0xFF1E251D, 0xFF1C231B, 0xFF1B221A, 0xFF1C1D17, 0xFF1B1E17, 0xFF1B1E17, 0xFF1D221B, 0xFF20271F, 0xFF202C22, 0xFF1F2B21, 0xFF1C291F, 0xFF192018, 0xFF1E231D, 0xFF191E18, 0xFF1E2420, 0xFF121D17,
        0xFF11221C, 0xFF374E46, 0xFF6B867D, 0xFF658578, 0xFF618375, 0xFF5B8172, 0xFF5B8673, 0xFF5E8976, 0xFF5B8472, 0xFF5C8370, 0xFF648675, 0xFF5E8477, 0xFF658B7E, 0xFF5A8073, 0xFF395F52, 0xFF456B5E, 0xFF3D6356, 0xFF1C4235, 0xFF466C5F, 0xFF3F6453,
        0xFF3F6453, 0xFF3F6453, 0xFF3F6453, 0xFF3F6453, 0xFF3F6453, 0xFF3F6453, 0xFF3F6453, 0xFF3F6452, 0xFF3F6452, 0xFF3F6452, 0xFF3F6452, 0xFF3F6452, 0xFF3F6452, 0xFF3F6452, 0xFF3F6452, 0xFF3E6352, 0xFF3E6352, 0xFF3D6251, 0xFF3D6251, 0xFF3D6251,
        0xFF3D6251, 0xFF3C6150, 0xFF3C6150, 0xFF3D6250, 0xFF446957, 0xFF446957, 0xFF3D6250, 0xFF3F6452, 0xFF466B59, 0xFF426755, 0xFF365B49, 0xFF426755, 0xFF315644, 0xFF5A7F6D, 0xFF648977, 0xFF547967, 0xFF5F8472, 0xFF5B806E, 0xFF527765, 0xFF5C8072,
        0xFF597A67, 0xFF496B5B, 0xFF26483A, 0xFF4E5B3D, 0xFF927F47, 0xFFA98B4D, 0xFFA08D55, 0xFF887648, 0xFF373F1A, 0xFF273826, 0xFF6C716B, 0xFF8F8A84, 0xFFD9DACA, 0xFFD5D6C6, 0xFFABA198, 0xFF938C7A, 0xFF8D8674, 0xFF958E7C, 0xFF999280, 0xFFA9A290,
        0xFFB6AF9D, 0xFFA29B89, 0xFF958E7C, 0xFF948D7A, 0xFF8C8572, 0xFF989380, 0xFF9C9686, 0xFF8D8A7B, 0xFF908C80, 0xFFB3B1A4, 0xFFE3E1D5, 0xFFB7C9BD, 0xFF698173, 0xFF648475, 0xFF628776, 0xFF49695A, 0x0094A9A0, 0x00FAFFFC, 0x00FFFEFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x00F5FFFD, 0xFF75958A, 0xFF4B7567, 0xFF6A9385, 0xFF61877A, 0xFF2A5647, 0xFF235645, 0xFF4F796D, 0xFF436D61, 0xFF2C564A, 0xFF265044, 0xFF467064,
        0xFF426C60, 0xFF1E483C, 0xFF2A5448, 0xFF617D6F, 0xFF6F8579, 0xFFB7C2BA, 0xFFD2D3CE, 0xFFE7E3E0, 0xFFDFDBD8, 0xFFE3E4DF, 0xFFDEE0DB, 0xFFE7E2DF, 0xFFECE8E5, 0xFFACAEA9, 0xFF7D8882, 0xFF92A39B, 0xFF466057, 0xFF4D6D62, 0xFF4A6B60, 0xFF4D675C,
        0xFF496358, 0xFF415B50, 0xFF445F56, 0xFF4D685F, 0xFF506B64, 0xFF516F67, 0xFF506E66, 0xFF305C4B, 0xFF4C7867, 0xFF5E8A79, 0xFF598574, 0xFF598574, 0xFF5B8776, 0xFF54806F, 0xFF578372, 0xFF5B8172, 0xFF658B7C, 0xFF4D7364, 0xFF385E4F, 0xFF4E7465,
        0xFF1A4031, 0xFF4A7061, 0xFF305647, 0xFF305647, 0xFF466C5D, 0xFF274D3E, 0xFF426859, 0xFF3C6253, 0xFF284E3F, 0xFF426859, 0xFF456B5C, 0xFF3C6756, 0xFF3D6656, 0xFF3C6555, 0xFF3B6454, 0xFF426859, 0xFF486E5F, 0xFF496D5F, 0xFF44685A, 0xFF4B6F61,
        0xFF426658, 0xFF426859, 0xFF486E5F, 0xFF446D5D, 0xFF426B5B, 0xFF436E5D, 0xFF436E5D, 0xFF3C6459, 0xFF3D655A, 0xFF396156, 0xFF3A6257, 0xFF40685D, 0xFF365E53, 0xFF3E665B, 0xFF355D52, 0xFF45705F, 0xFF376251, 0xFF588372, 0xFF578271, 0xFF598473,
        0xFF5C8776, 0xFF608B7A, 0xFF568170, 0xFF5E8775, 0xFF5E8372, 0xFF405A4D, 0xFF112218, 0xFF09140C, 0xFF171E17, 0xFF121711, 0xFF121711, 0xFF121D17, 0xFF101B15, 0xFF0F1A14, 0xFF101B15, 0xFF121D17, 0xFF131E18, 0xFF111C16, 0xFF0E1913, 0xFF0E1B14,
        0xFF0F1C15, 0xFF0F1C15, 0xFF101D16, 0xFF111E17, 0xFF111E17, 0xFF111E17, 0xFF111E17, 0xFF141915, 0xFF111713, 0xFF0F1813, 0xFF131E18, 0xFF18251E, 0xFF182921, 0xFF12261D, 0xFF0D2118, 0xFF162220, 0xFF121C1B, 0xFF0C1713, 0xFF14201C, 0xFF10241B,
        0xFF354F42, 0xFF5C7E6E, 0xFF5F8674, 0xFF608476, 0xFF5D8374, 0xFF5C8776, 0xFF5E8B77, 0xFF5C8975, 0xFF598471, 0xFF5A816E, 0xFF5C816F, 0xFF638C7E, 0xFF5D8678, 0xFF3B6456, 0xFF3D6658, 0xFF2E5749, 0xFF234C3E, 0xFF446D5F, 0xFF396254, 0xFF507564,
        0xFF507564, 0xFF507564, 0xFF507564, 0xFF507564, 0xFF507564, 0xFF507564, 0xFF507564, 0xFF517664, 0xFF517664, 0xFF517664, 0xFF517664, 0xFF517664, 0xFF517664, 0xFF517664, 0xFF517664, 0xFF537867, 0xFF537867, 0xFF527766, 0xFF527766, 0xFF527766,
        0xFF527766, 0xFF517665, 0xFF517665, 0xFF507563, 0xFF4F7462, 0xFF4F7462, 0xFF4E7361, 0xFF4B705E, 0xFF496E5C, 0xFF4B705E, 0xFF507563, 0xFF4A6F5D, 0xFF668B79, 0xFF597E6C, 0xFF597E6C, 0xFF5B806E, 0xFF537866, 0xFF618674, 0xFF618674, 0xFF567C6D,
        0xFF587966, 0xFF4B6D5D, 0xFF2B4D3F, 0xFF5C694B, 0xFF9B8850, 0xFFA7894B, 0xFF9F8C54, 0xFF8C7A4C, 0xFF3A421D, 0xFF273826, 0xFF6B706A, 0xFF908B85, 0xFFD9DACA, 0xFFD5D6C6, 0xFFADA39A, 0xFF999280, 0xFF908977, 0xFF948D7B, 0xFF918A78, 0xFF9F9886,
        0xFFB3AC9A, 0xFFA9A290, 0xFFA49D8B, 0xFF9C9582, 0xFF928B78, 0xFF9C9784, 0xFFA19B8B, 0xFF918E7F, 0xFF918D81, 0xFFB2B0A3, 0xFFE2E0D4, 0xFFAEC0B4, 0xFF627A6C, 0xFF628273, 0xFF658A79, 0xFF507061, 0x0097ACA3, 0x00F5FBF7, 0x00FFFDFC, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x04F3FFFB, 0xFF749489, 0xFF4C7668, 0xFF6A9385, 0xFF61877A, 0xFF234F40, 0xFF124534, 0xFF143E32, 0xFF1D473B, 0xFF153F33, 0xFF184236, 0xFF1B4539,
        0xFF113B2F, 0xFF1A4438, 0xFF2E584C, 0xFF64756B, 0xFF848F87, 0xFFD9DAD5, 0xFFDFD7D5, 0xFFF3E7E7, 0xFFDED4D3, 0xFFD9D5D2, 0xFFD6D5D1, 0xFFDCD8D5, 0xFFE9EAE5, 0xFFA9B0A9, 0xFF85928B, 0xFFB9D0C6, 0xFF4B695F, 0xFF6B8F83, 0xFF598274, 0xFF5B8776,
        0xFF608C7B, 0xFF5A8677, 0xFF5D8779, 0xFF5D877B, 0xFF578175, 0xFF4D756A, 0xFF325A51, 0xFF3E6A59, 0xFF34604F, 0xFF527E6D, 0xFF5A8675, 0xFF588473, 0xFF5F8B7A, 0xFF54806F, 0xFF5A8675, 0xFF608677, 0xFF5A8071, 0xFF6B9182, 0xFF4E7465, 0xFF375D4E,
        0xFF4D7364, 0xFF214738, 0xFF416758, 0xFF345A4B, 0xFF33594A, 0xFF3C6253, 0xFF2B5142, 0xFF3D6354, 0xFF496F60, 0xFF3F6556, 0xFF416758, 0xFF3F6A59, 0xFF3F6858, 0xFF416A5A, 0xFF436C5C, 0xFF426859, 0xFF3D6354, 0xFF416557, 0xFF476B5D, 0xFF45695B,
        0xFF416758, 0xFF43695A, 0xFF446A5B, 0xFF3D6656, 0xFF406959, 0xFF426D5C, 0xFF3F6A59, 0xFF436B60, 0xFF375F54, 0xFF446C61, 0xFF3D655A, 0xFF386055, 0xFF426A5F, 0xFF335B50, 0xFF456D62, 0xFF376251, 0xFF55806F, 0xFF5E8978, 0xFF5F8A79, 0xFF5B8675,
        0xFF588473, 0xFF568271, 0xFF5F8B7A, 0xFF558470, 0xFF5E8775, 0xFF648475, 0xFF445A4E, 0xFF2B382F, 0xFF3C463E, 0xFF4A514A, 0xFF404740, 0xFF424E4A, 0xFF3F4B47, 0xFF3B4743, 0xFF3C4844, 0xFF3E4A46, 0xFF404C48, 0xFF404C48, 0xFF3E4A46, 0xFF394A44,
        0xFF384943, 0xFF384943, 0xFF384943, 0xFF394A44, 0xFF3A4B45, 0xFF3C4D47, 0xFF3E4F49, 0xFF3F4543, 0xFF39423F, 0xFF37423E, 0xFF384742, 0xFF3E4F49, 0xFF3C514A, 0xFF344B43, 0xFF2B453C, 0xFF354344, 0xFF435152, 0xFF303E3E, 0xFF2F403A, 0xFF3C5349,
        0xFF587968, 0xFF618873, 0xFF5A866F, 0xFF5C8273, 0xFF5C8575, 0xFF5D8978, 0xFF598874, 0xFF578470, 0xFF5A8572, 0xFF628976, 0xFF648977, 0xFF5B8476, 0xFF325B4D, 0xFF446D5F, 0xFF30594B, 0xFF264F41, 0xFF416A5C, 0xFF386153, 0xFF487163, 0xFF608574,
        0xFF608574, 0xFF608574, 0xFF618675, 0xFF618675, 0xFF618675, 0xFF618675, 0xFF618675, 0xFF618674, 0xFF618674, 0xFF618674, 0xFF618674, 0xFF618674, 0xFF618674, 0xFF618674, 0xFF618674, 0xFF618675, 0xFF618675, 0xFF608574, 0xFF608574, 0xFF608574,
        0xFF608574, 0xFF5F8473, 0xFF5F8473, 0xFF608573, 0xFF5F8472, 0xFF5F8472, 0xFF608573, 0xFF628775, 0xFF628775, 0xFF618674, 0xFF618674, 0xFF5F8472, 0xFF5D8270, 0xFF5D8270, 0xFF577C6A, 0xFF5D8270, 0xFF5E8371, 0xFF547967, 0xFF5B806E, 0xFF597F70,
        0xFF5B7C69, 0xFF486D5C, 0xFF274B3D, 0xFF5E6B4D, 0xFF9C8951, 0xFFA38748, 0xFF9E8B53, 0xFF917F51, 0xFF3D4520, 0xFF283927, 0xFF6B706A, 0xFF908B85, 0xFFD9DACA, 0xFFD6D7C7, 0xFFAFA59C, 0xFFACA593, 0xFFA7A08E, 0xFFAEA795, 0xFFA49D8B, 0xFFAAA391,
        0xFFB9B2A0, 0xFFAFA896, 0xFFAEA795, 0xFFAAA390, 0xFF9B9481, 0xFFA29D8A, 0xFFA49E8E, 0xFF928F80, 0xFF908C80, 0xFFAFADA0, 0xFFE0DED2, 0xFFBBCDC1, 0xFF667E70, 0xFF5D7D6E, 0xFF5F8473, 0xFF4F6F60, 0x0098ADA4, 0x00F6FCF8, 0x00FFFEFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFAFC, 0x04F3FFFB, 0xFF749489, 0xFF507A6C, 0xFF679082, 0xFF658B7E, 0xFF477364, 0xFF3C6F5E, 0xFF3E6958, 0xFF3F6A59, 0xFF416C5B, 0xFF3A6554, 0xFF3B6655,
        0xFF3F6A59, 0xFF3C6756, 0xFF45705F, 0xFF588372, 0xFF426859, 0xFF739386, 0xFF9DB9AD, 0xFF7A9187, 0xFF5F796E, 0xFF728C81, 0xFF7B978B, 0xFF5D7E73, 0xFF5D8175, 0xFF597D73, 0xFF4F756A, 0xFF4F776C, 0xFF497369, 0xFF59847A, 0xFF5B897E, 0xFF628879,
        0xFF608979, 0xFF5C8575, 0xFF618A7A, 0xFF5F8A79, 0xFF5E8978, 0xFF5E8A79, 0xFF4B7766, 0xFF2D5948, 0xFF3F6B5A, 0xFF34604F, 0xFF517D6C, 0xFF5E8A79, 0xFF5A8675, 0xFF5D8978, 0xFF5A8675, 0xFF558472, 0xFF578674, 0xFF5C8B79, 0xFF5C8877, 0xFF467261,
        0xFF3D6857, 0xFF3B6655, 0xFF244F3E, 0xFF3D6656, 0xFF355E4E, 0xFF365F4F, 0xFF3A6353, 0xFF365F4F, 0xFF386151, 0xFF3C6555, 0xFF3A6353, 0xFF3B6152, 0xFF3C6253, 0xFF3C6253, 0xFF3C6253, 0xFF3C6253, 0xFF3D6354, 0xFF3D6354, 0xFF3D6354, 0xFF3D6354,
        0xFF3E6455, 0xFF3E6455, 0xFF3F6556, 0xFF3F6556, 0xFF3E6455, 0xFF3E6455, 0xFF3D6354, 0xFF396252, 0xFF3E6757, 0xFF416A5A, 0xFF396252, 0xFF3A6353, 0xFF3C6555, 0xFF416A5A, 0xFF3E6757, 0xFF598574, 0xFF588473, 0xFF588473, 0xFF588473, 0xFF588473,
        0xFF598574, 0xFF598574, 0xFF5A8675, 0xFF578372, 0xFF578372, 0xFF5A8574, 0xFF618778, 0xFF4E6C60, 0xFF778C83, 0xFF7F8A84, 0xFF808682, 0xFF99A1A3, 0xFF949C9E, 0xFF99A1A3, 0xFF959D9F, 0xFF9BA3A5, 0xFF939B9D, 0xFF929A9C, 0xFF90989A, 0xFF8B9499,
        0xFF97A0A5, 0xFF8E979C, 0xFF848D92, 0xFFA2ABB0, 0xFF929BA0, 0xFF8D969B, 0xFF9BA4A9, 0xFF949C9E, 0xFF899193, 0xFF979FA1, 0xFF8B9395, 0xFF90989A, 0xFFA5ADAF, 0xFF8D9597, 0xFF80888A, 0xFF9E98A2, 0xFF9EA3A9, 0xFF697D7C, 0xFF5E8278, 0xFF598576,
        0xFF608D79, 0xFF58826E, 0xFF5F8570, 0xFF5D8875, 0xFF588370, 0xFF5E8976, 0xFF5C8776, 0xFF5E8978, 0xFF618B7D, 0xFF5A8476, 0xFF5E887A, 0xFF375D4E, 0xFF3E6455, 0xFF345A4B, 0xFF214836, 0xFF3D6452, 0xFF3F6654, 0xFF446B58, 0xFF6B927F, 0xFF608574,
        0xFF5F8473, 0xFF5D8472, 0xFF5D8472, 0xFF5B8472, 0xFF5B8472, 0xFF5B8472, 0xFF5A8572, 0xFF608775, 0xFF5F8674, 0xFF5E8573, 0xFF5D8472, 0xFF5D8472, 0xFF5D8472, 0xFF5E8573, 0xFF5F8674, 0xFF5F8674, 0xFF5F8674, 0xFF5F8674, 0xFF5E8573, 0xFF5D8472,
        0xFF5C8371, 0xFF5B8270, 0xFF5A816F, 0xFF5F8673, 0xFF5F8673, 0xFF5E8572, 0xFF5E8572, 0xFF5D8471, 0xFF5C8370, 0xFF5C8370, 0xFF5B826F, 0xFF5C816F, 0xFF5C816F, 0xFF5C816F, 0xFF5C816F, 0xFF5B806E, 0xFF5B806E, 0xFF5B806E, 0xFF5B806E, 0xFF5E7E69,
        0xFF547A6B, 0xFF436D63, 0xFF345341, 0xFF596341, 0xFF958D5C, 0xFFA5945C, 0xFF9C8951, 0xFF897D4B, 0xFF4B4C2C, 0xFF2B382E, 0xFF687676, 0xFF888D89, 0xFFE4E0D4, 0xFFD5CEBB, 0xFFA9A490, 0xFFA9A091, 0xFFABA293, 0xFFA79E8F, 0xFFA49B8C, 0xFFA59C8D,
        0xFFA49B8C, 0xFFA39A8B, 0xFFA89F90, 0xFFA19C86, 0xFF9D9980, 0xFF9B977E, 0xFF938F76, 0xFF8D8874, 0xFF928C7C, 0xFFAEA79D, 0xFFDCD5CF, 0xFFCDD3C9, 0xFF819184, 0xFF5D796A, 0xFF608775, 0xFF4D7163, 0x0092ACA3, 0x00F4FFFB, 0x00FFFEFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x00F7FFFF, 0xFF729287, 0xFF4E786A, 0xFF648D7F, 0xFF698F82, 0xFF5C8879, 0xFF5A8D7C, 0xFF65907F, 0xFF628D7C, 0xFF618C7B, 0xFF628D7C, 0xFF628D7C,
        0xFF608B7A, 0xFF5F8A79, 0xFF669180, 0xFF5D8C7A, 0xFF54806F, 0xFF5B8172, 0xFF65897B, 0xFF638577, 0xFF5F8375, 0xFF618778, 0xFF638C7C, 0xFF61877A, 0xFF608679, 0xFF5D8376, 0xFF547C71, 0xFF5A8277, 0xFF578177, 0xFF628C82, 0xFF588379, 0xFF5C8575,
        0xFF5E8777, 0xFF5B8675, 0xFF5D8877, 0xFF5A8675, 0xFF5A8675, 0xFF608C7B, 0xFF538270, 0xFF4A7665, 0xFF2D5948, 0xFF406C5B, 0xFF3B6756, 0xFF588473, 0xFF608C7B, 0xFF618D7C, 0xFF568271, 0xFF598876, 0xFF588775, 0xFF52816F, 0xFF5D8978, 0xFF5B8776,
        0xFF487362, 0xFF3F6A59, 0xFF3C6756, 0xFF2A5343, 0xFF345D4D, 0xFF2F5848, 0xFF325B4B, 0xFF436C5C, 0xFF406959, 0xFF396252, 0xFF466F5F, 0xFF426859, 0xFF426859, 0xFF426859, 0xFF43695A, 0xFF43695A, 0xFF43695A, 0xFF43695A, 0xFF43695A, 0xFF426859,
        0xFF426859, 0xFF43695A, 0xFF43695A, 0xFF43695A, 0xFF43695A, 0xFF426859, 0xFF426859, 0xFF3C6555, 0xFF3C6555, 0xFF365F4F, 0xFF406959, 0xFF386151, 0xFF426B5B, 0xFF456E5E, 0xFF547D6D, 0xFF5B8776, 0xFF5B8776, 0xFF5B8776, 0xFF5B8776, 0xFF5B8776,
        0xFF5B8776, 0xFF5B8776, 0xFF5B8776, 0xFF5C8877, 0xFF5E8A79, 0xFF598473, 0xFF688E7F, 0xFF416154, 0xFF657C72, 0xFF68756E, 0xFF77807B, 0xFF7E8A8A, 0xFF7D8989, 0xFF778383, 0xFF737F7F, 0xFF7F8B8B, 0xFF828E8E, 0xFF8F9B9B, 0xFF899595, 0xFF7A8889,
        0xFF778586, 0xFF6E7C7D, 0xFF798788, 0xFF6D7B7C, 0xFF7D8B8C, 0xFF6B797A, 0xFF7F8D8E, 0xFF818D8B, 0xFF75817F, 0xFF84908E, 0xFF7B8785, 0xFF7C8886, 0xFF8A9694, 0xFF7B8785, 0xFF788482, 0xFF8B8B93, 0xFF848E90, 0xFF4A635F, 0xFF557B70, 0xFF588775,
        0xFF558470, 0xFF598470, 0xFF618873, 0xFF5D8674, 0xFF5D8674, 0xFF598270, 0xFF648D7D, 0xFF598272, 0xFF598274, 0xFF60897B, 0xFF2F584A, 0xFF426859, 0xFF355B4C, 0xFF1F4536, 0xFF466D5B, 0xFF3A614F, 0xFF4A715F, 0xFF608774, 0xFF5E8572, 0xFF608574,
        0xFF5F8473, 0xFF5E8573, 0xFF5D8472, 0xFF5B8472, 0xFF5B8472, 0xFF5B8472, 0xFF5A8572, 0xFF5F8674, 0xFF5E8573, 0xFF5E8573, 0xFF5D8472, 0xFF5D8472, 0xFF5E8573, 0xFF5F8674, 0xFF5F8674, 0xFF608775, 0xFF608775, 0xFF5F8674, 0xFF5E8573, 0xFF5D8472,
        0xFF5C8371, 0xFF5B8270, 0xFF5B8270, 0xFF5F8673, 0xFF5F8673, 0xFF5E8572, 0xFF5E8572, 0xFF5D8471, 0xFF5C8370, 0xFF5C8370, 0xFF5C8370, 0xFF5D8270, 0xFF5C816F, 0xFF5C816F, 0xFF5C816F, 0xFF5C816F, 0xFF5C816F, 0xFF5B806E, 0xFF5B806E, 0xFF5B7C69,
        0xFF547A6D, 0xFF487268, 0xFF3D5C4A, 0xFF5F6644, 0xFF928654, 0xFFA58F56, 0xFFA38A51, 0xFF8F7F4B, 0xFF53502D, 0xFF2E392B, 0xFF5D6766, 0xFF868883, 0xFFE4DED2, 0xFFD4CFBC, 0xFF918E7B, 0xFF8B836C, 0xFF8E866F, 0xFF8B836C, 0xFF89816A, 0xFF8B836C,
        0xFF89816A, 0xFF877F68, 0xFF8B836C, 0xFF88836F, 0xFF827D67, 0xFF817D64, 0xFF7A765D, 0xFF7A7561, 0xFF8A8474, 0xFFADA69C, 0xFFD8D1CB, 0xFFE8EEE4, 0xFF9CAA9D, 0xFF658172, 0xFF628977, 0xFF4B6F61, 0x008EA89F, 0x00F8FFFE, 0x00FFFEFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x00F7FFFF, 0xFF719186, 0xFF537D6F, 0xFF638C7E, 0xFF658B7E, 0xFF5D897A, 0xFF5A8D7C, 0xFF5E8978, 0xFF5F8A79, 0xFF5C8776, 0xFF5F8A79, 0xFF5D8877,
        0xFF5F8A79, 0xFF608B7A, 0xFF628D7C, 0xFF61927F, 0xFF659482, 0xFF5D8877, 0xFF5D8676, 0xFF648D7D, 0xFF699584, 0xFF629380, 0xFF5E917E, 0xFF6A9383, 0xFF689183, 0xFF6B9486, 0xFF699284, 0xFF658D82, 0xFF5A8277, 0xFF658D84, 0xFF5F877E, 0xFF5D8877,
        0xFF5D8877, 0xFF598574, 0xFF5A8675, 0xFF578674, 0xFF578674, 0xFF5C8D7A, 0xFF588976, 0xFF598574, 0xFF4A7665, 0xFF2B5746, 0xFF3D6958, 0xFF3A6655, 0xFF578372, 0xFF5C8877, 0xFF537F6E, 0xFF568573, 0xFF5B8A78, 0xFF558472, 0xFF598574, 0xFF5E8A79,
        0xFF55806F, 0xFF467160, 0xFF396453, 0xFF3B6454, 0xFF305949, 0xFF3C6555, 0xFF487161, 0xFF406959, 0xFF436C5C, 0xFF4A7363, 0xFF406959, 0xFF456E5E, 0xFF456E5E, 0xFF456E5E, 0xFF456E5E, 0xFF466F5F, 0xFF466F5F, 0xFF466F5F, 0xFF466F5F, 0xFF486E5F,
        0xFF476D5E, 0xFF476D5E, 0xFF476D5E, 0xFF476D5E, 0xFF476D5E, 0xFF476D5E, 0xFF486E5F, 0xFF4B7464, 0xFF3F6858, 0xFF446D5D, 0xFF386151, 0xFF466F5F, 0xFF3E6757, 0xFF537C6C, 0xFF5C8575, 0xFF598574, 0xFF598574, 0xFF5A8675, 0xFF5A8675, 0xFF5A8675,
        0xFF588473, 0xFF578372, 0xFF568271, 0xFF5A8574, 0xFF568170, 0xFF618C7B, 0xFF5A8373, 0xFF557769, 0xFF728C81, 0xFF9AAEA5, 0xFFB1BEB7, 0xFF7F948F, 0xFF869B96, 0xFF7F948F, 0xFF708580, 0xFF728782, 0xFF728782, 0xFF859A95, 0xFF7F948F, 0xFF798F8C,
        0xFF627875, 0xFF617774, 0xFF647A77, 0xFF5A706D, 0xFF5A706D, 0xFF627875, 0xFF5B716E, 0xFF596E67, 0xFF566B64, 0xFF71867F, 0xFF758A83, 0xFF788D86, 0xFF7A8F88, 0xFF6A7F78, 0xFF677C75, 0xFF7C8789, 0xFF788B89, 0xFF44635B, 0xFF588274, 0xFF5C8B79,
        0xFF51806C, 0xFF57846F, 0xFF5A8571, 0xFF57806E, 0xFF57806E, 0xFF5F8876, 0xFF567F6F, 0xFF5F8878, 0xFF5D8678, 0xFF3A6355, 0xFF40695B, 0xFF2D5344, 0xFF224839, 0xFF426859, 0xFF365D4B, 0xFF507765, 0xFF5D8472, 0xFF5F8673, 0xFF5D8471, 0xFF608574,
        0xFF5F8473, 0xFF5E8573, 0xFF5D8472, 0xFF5B8472, 0xFF5B8472, 0xFF5B8472, 0xFF5B8673, 0xFF5E8573, 0xFF5E8573, 0xFF5D8472, 0xFF5D8472, 0xFF5E8573, 0xFF5F8674, 0xFF608775, 0xFF608775, 0xFF618876, 0xFF608775, 0xFF5E8573, 0xFF5D8472, 0xFF5C8371,
        0xFF5C8371, 0xFF5C8371, 0xFF5D8472, 0xFF5F8673, 0xFF5F8673, 0xFF5E8572, 0xFF5E8572, 0xFF5D8471, 0xFF5D8471, 0xFF5C8370, 0xFF5C8370, 0xFF5D8270, 0xFF5D8270, 0xFF5D8270, 0xFF5D8270, 0xFF5D8270, 0xFF5C816F, 0xFF5C816F, 0xFF5C816F, 0xFF5D806C,
        0xFF5A8375, 0xFF517B71, 0xFF43604C, 0xFF646A46, 0xFF998955, 0xFFB2955B, 0xFFAB8D51, 0xFF927F47, 0xFF544C27, 0xFF343827, 0xFF5D645D, 0xFF95948F, 0xFFE4DED2, 0xFFD2CFBE, 0xFF7E7F6F, 0xFF726D50, 0xFF767154, 0xFF746F52, 0xFF736E51, 0xFF767154,
        0xFF757053, 0xFF716C4F, 0xFF726D50, 0xFF6F6C59, 0xFF6A6853, 0xFF6E6C55, 0xFF696750, 0xFF696752, 0xFF807D6C, 0xFFA6A297, 0xFFCDC8C2, 0xFFF0F3EA, 0xFFA8B6A9, 0xFF627E6F, 0xFF5F8473, 0xFF496D5F, 0x008AA199, 0x00F9FFFF, 0x00FFFEFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x00F7FFFF, 0xFF6A8A7F, 0xFF527C6E, 0xFF638C7E, 0xFF668C7F, 0xFF5E8A7B, 0xFF5D907F, 0xFF648E80, 0xFF6D9789, 0xFF669082, 0xFF658F81, 0xFF5E887A,
        0xFF648E80, 0xFF669082, 0xFF648E80, 0xFF658C7A, 0xFF628776, 0xFF658777, 0xFF6E8E7F, 0xFF597B6B, 0xFF4E7563, 0xFF4C7764, 0xFF46735F, 0xFF4A7061, 0xFF43695A, 0xFF4D7366, 0xFF597F72, 0xFF61877A, 0xFF5A8075, 0xFF668C81, 0xFF60867B, 0xFF5F8B7A,
        0xFF5C8877, 0xFF588473, 0xFF588775, 0xFF588775, 0xFF568774, 0xFF578875, 0xFF568774, 0xFF5F8A79, 0xFF628D7C, 0xFF436E5D, 0xFF345F4E, 0xFF3F6A59, 0xFF3D6857, 0xFF5B8675, 0xFF5E8978, 0xFF568573, 0xFF558472, 0xFF558472, 0xFF5A8675, 0xFF588473,
        0xFF5D8877, 0xFF5D8877, 0xFF467160, 0xFF3B6454, 0xFF3D6656, 0xFF3A6353, 0xFF3C6555, 0xFF416A5A, 0xFF3C6555, 0xFF355E4E, 0xFF396252, 0xFF386151, 0xFF386151, 0xFF396252, 0xFF396252, 0xFF396252, 0xFF396252, 0xFF3A6353, 0xFF3A6353, 0xFF3E6455,
        0xFF3D6354, 0xFF3C6253, 0xFF3C6253, 0xFF3C6253, 0xFF3C6253, 0xFF3D6354, 0xFF3E6455, 0xFF376050, 0xFF3E6757, 0xFF315A4A, 0xFF3D6656, 0xFF365F4F, 0xFF547D6D, 0xFF557E6E, 0xFF5E8777, 0xFF5C8776, 0xFF5C8776, 0xFF5D8877, 0xFF5E8978, 0xFF5D8877,
        0xFF5B8675, 0xFF598473, 0xFF588372, 0xFF567F6F, 0xFF638C7C, 0xFF547F6E, 0xFF578070, 0xFF335749, 0xFF5C7A6E, 0xFF647B71, 0xFF6C8077, 0xFF4B695F, 0xFF537167, 0xFF59776D, 0xFF47655B, 0xFF436157, 0xFF3F5D53, 0xFF446258, 0xFF405E54, 0xFF3E5C54,
        0xFF4E6C64, 0xFF2E4C44, 0xFF2C4A42, 0xFF56746C, 0xFF45635B, 0xFF4B6961, 0xFF527068, 0xFF516F65, 0xFF426056, 0xFF46645A, 0xFF3E5C52, 0xFF436157, 0xFF47655B, 0xFF446258, 0xFF405E54, 0xFF586F69, 0xFF5F7A73, 0xFF476B5F, 0xFF588274, 0xFF5A8977,
        0xFF578672, 0xFF5B8874, 0xFF5D8875, 0xFF608977, 0xFF5C8573, 0xFF5C8573, 0xFF638C7C, 0xFF547D6D, 0xFF40695B, 0xFF3E6759, 0xFF345D4F, 0xFF1C4233, 0xFF416758, 0xFF3C6253, 0xFF476E5C, 0xFF59806E, 0xFF628977, 0xFF5A816E, 0xFF5E8572, 0xFF608574,
        0xFF608574, 0xFF5E8573, 0xFF5D8472, 0xFF5B8472, 0xFF5B8472, 0xFF5C8573, 0xFF5B8673, 0xFF5D8472, 0xFF5D8472, 0xFF5D8472, 0xFF5E8573, 0xFF5F8674, 0xFF608775, 0xFF608775, 0xFF618876, 0xFF618876, 0xFF608775, 0xFF5E8573, 0xFF5C8371, 0xFF5C8371,
        0xFF5C8371, 0xFF5D8472, 0xFF5E8573, 0xFF5F8673, 0xFF5F8673, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF5D8471, 0xFF5D8471, 0xFF5D8471, 0xFF5E8371, 0xFF5E8371, 0xFF5E8371, 0xFF5E8371, 0xFF5E8371, 0xFF5D8270, 0xFF5D8270, 0xFF5D8270, 0xFF597C68,
        0xFF5A8476, 0xFF4C766C, 0xFF34513D, 0xFF5C603D, 0xFF9E8C5A, 0xFFBFA067, 0xFFAD8D52, 0xFF96834B, 0xFF564E27, 0xFF4A4B39, 0xFF80837A, 0xFFBEBBB2, 0xFFECE6DA, 0xFFD0CEC1, 0xFF737769, 0xFF67624C, 0xFF6D6852, 0xFF6C6751, 0xFF6C6751, 0xFF706B55,
        0xFF6F6A54, 0xFF6A654F, 0xFF69644E, 0xFF6A6756, 0xFF64614E, 0xFF66644F, 0xFF5D5B46, 0xFF5A5843, 0xFF767362, 0xFFA4A094, 0xFFCFCBC2, 0xFFE8EBE2, 0xFFABB9AC, 0xFF5E796A, 0xFF5E8372, 0xFF4E7062, 0x00889F97, 0x00F9FFFF, 0x00FEF9FD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x00F6FFFE, 0xFF67877C, 0xFF537D6F, 0xFF658E80, 0xFF61877A, 0xFF447061, 0xFF3E7160, 0xFF3B6557, 0xFF426C5E, 0xFF3A6456, 0xFF406A5C, 0xFF3C6658,
        0xFF416B5D, 0xFF406A5C, 0xFF4B7567, 0xFF6C8477, 0xFF52685C, 0xFF4D5E54, 0xFF637369, 0xFF45554B, 0xFF31453A, 0xFF2C4437, 0xFF203A2D, 0xFF1B372B, 0xFF244236, 0xFF2F4D41, 0xFF2B4B40, 0xFF315247, 0xFF3C6054, 0xFF5C8277, 0xFF5E8479, 0xFF5F8A79,
        0xFF5C8776, 0xFF5D8877, 0xFF5D8978, 0xFF5C8877, 0xFF568573, 0xFF558472, 0xFF5B8A78, 0xFF5D8877, 0xFF5B8675, 0xFF628D7C, 0xFF3D6857, 0xFF386352, 0xFF3C6756, 0xFF3F6A59, 0xFF5A8574, 0xFF5D8C7A, 0xFF538270, 0xFF507F6D, 0xFF5B8776, 0xFF578372,
        0xFF547F6E, 0xFF5E8978, 0xFF5D8877, 0xFF3A6353, 0xFF3C6555, 0xFF386151, 0xFF3A6353, 0xFF426B5B, 0xFF406959, 0xFF3E6757, 0xFF446D5D, 0xFF406959, 0xFF406959, 0xFF406959, 0xFF416A5A, 0xFF416A5A, 0xFF416A5A, 0xFF416A5A, 0xFF426B5B, 0xFF466C5D,
        0xFF456B5C, 0xFF446A5B, 0xFF446A5B, 0xFF446A5B, 0xFF446A5B, 0xFF456B5C, 0xFF466C5D, 0xFF3F6858, 0xFF3A6353, 0xFF436C5C, 0xFF386151, 0xFF537C6C, 0xFF537C6C, 0xFF608979, 0xFF567F6F, 0xFF5B8675, 0xFF5C8776, 0xFF5D8877, 0xFF5D8877, 0xFF5D8877,
        0xFF5B8675, 0xFF598473, 0xFF588372, 0xFF638779, 0xFF5B8172, 0xFF598272, 0xFF3D6857, 0xFF3B6454, 0xFF365A4C, 0xFF426054, 0xFF304A3F, 0xFF3F6357, 0xFF42665A, 0xFF5E8276, 0xFF577B6F, 0xFF5F8377, 0xFF63877B, 0xFF5B7F73, 0xFF5D8175, 0xFF5F8173,
        0xFF628476, 0xFF3C5E50, 0xFF426456, 0xFF325446, 0xFF46685A, 0xFF395B4D, 0xFF3C5E50, 0xFF3C6052, 0xFF34584A, 0xFF3E6254, 0xFF426658, 0xFF567A6C, 0xFF597D6F, 0xFF4F7365, 0xFF365A4C, 0xFF3A5A4F, 0xFF395B4D, 0xFF3B6152, 0xFF45705F, 0xFF497564,
        0xFF4F7B6A, 0xFF487561, 0xFF517C69, 0xFF537A68, 0xFF59806E, 0xFF567D6B, 0xFF53796A, 0xFF466C5D, 0xFF3D6356, 0xFF385E51, 0xFF22483B, 0xFF426859, 0xFF416758, 0xFF446A5B, 0xFF618876, 0xFF5E8573, 0xFF5C8371, 0xFF5F8673, 0xFF5D8471, 0xFF608574,
        0xFF608574, 0xFF5E8573, 0xFF5E8573, 0xFF5B8472, 0xFF5B8472, 0xFF5C8573, 0xFF5B8673, 0xFF5D8472, 0xFF5D8472, 0xFF5E8573, 0xFF5F8674, 0xFF5F8674, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF618876, 0xFF608775, 0xFF5E8573, 0xFF5D8472, 0xFF5C8371,
        0xFF5D8472, 0xFF5E8573, 0xFF5F8674, 0xFF5F8673, 0xFF5F8673, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF5F8472, 0xFF5F8472, 0xFF5F8472, 0xFF5F8472, 0xFF5F8472, 0xFF5E8371, 0xFF5E8371, 0xFF5E8371, 0xFF5C7F6B,
        0xFF5C8678, 0xFF4B766C, 0xFF32513F, 0xFF5F6444, 0xFFA19062, 0xFFCCB17C, 0xFFC1A36F, 0xFF968A58, 0xFF605C37, 0xFF676A59, 0xFF9A9D94, 0xFFD5D1C6, 0xFFF2ECDE, 0xFFCDCEC0, 0xFF6C7467, 0xFF5D594E, 0xFF666257, 0xFF686459, 0xFF676358, 0xFF6B675C,
        0xFF6B675C, 0xFF656156, 0xFF635F54, 0xFF656356, 0xFF615F50, 0xFF656450, 0xFF5E5D48, 0xFF585742, 0xFF6D6C5A, 0xFF9C9A8D, 0xFFCECCC0, 0xFFEBECE4, 0xFFB8C4B8, 0xFF647F70, 0xFF668878, 0xFF547668, 0x008A9F98, 0x00FBFFFF, 0x00FCF7FB, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x00F6FFFE, 0xFF69897E, 0xFF547E70, 0xFF689183, 0xFF5A8073, 0xFF214D3E, 0xFF1A4D3C, 0xFF1B4539, 0xFF164034, 0xFF0C362A, 0xFF1A4438, 0xFF1D473B,
        0xFF1C463A, 0xFF0D372B, 0xFF295347, 0xFF63746A, 0xFF49564D, 0xFF49534B, 0xFF939892, 0xFFB1B3AE, 0xFFBBBDB8, 0xFFB8BDB7, 0xFFB1B8B1, 0xFFA1AEA5, 0xFFABBBB1, 0xFF8A9E95, 0xFF465D53, 0xFF2F4D43, 0xFF45665B, 0xFF678D80, 0xFF628A7F, 0xFF63897A,
        0xFF618778, 0xFF648A7B, 0xFF628B7B, 0xFF618A7A, 0xFF5C8575, 0xFF5A8574, 0xFF699483, 0xFF5F8878, 0xFF628B7B, 0xFF638C7C, 0xFF608979, 0xFF3A6353, 0xFF426B5B, 0xFF376050, 0xFF3F6858, 0xFF538270, 0xFF608F7D, 0xFF598876, 0xFF5C8877, 0xFF608C7B,
        0xFF578271, 0xFF568170, 0xFF5F8A79, 0xFF578070, 0xFF466F5F, 0xFF4B7464, 0xFF4F7868, 0xFF426B5B, 0xFF446D5D, 0xFF4C7565, 0xFF436C5C, 0xFF436E5D, 0xFF446F5E, 0xFF446F5E, 0xFF446F5E, 0xFF446F5E, 0xFF446F5E, 0xFF45705F, 0xFF45705F, 0xFF476D5E,
        0xFF476D5E, 0xFF476D5E, 0xFF466C5D, 0xFF466C5D, 0xFF476D5E, 0xFF476D5E, 0xFF476D5E, 0xFF497262, 0xFF487161, 0xFF406959, 0xFF588171, 0xFF557E6E, 0xFF628B7B, 0xFF588171, 0xFF5C8575, 0xFF5F8878, 0xFF5F8878, 0xFF608979, 0xFF608979, 0xFF5F8878,
        0xFF5E8777, 0xFF5D8676, 0xFF5C8575, 0xFF67897B, 0xFF54786A, 0xFF365F4F, 0xFF3C6756, 0xFF2E5948, 0xFF3A6353, 0xFF3D6153, 0xFF406254, 0xFF3C6253, 0xFF446A5B, 0xFF668C7D, 0xFF5E8475, 0xFF5F8576, 0xFF5E8475, 0xFF597F70, 0xFF618778, 0xFF5F8473,
        0xFF426756, 0xFF466B5A, 0xFF3C6150, 0xFF1D4231, 0xFF416655, 0xFF395E4D, 0xFF3E6352, 0xFF3C6351, 0xFF355C4A, 0xFF3E6553, 0xFF486F5D, 0xFF658C7A, 0xFF658C7A, 0xFF5A816F, 0xFF365D4B, 0xFF3D6452, 0xFF284F3D, 0xFF315846, 0xFF3A614F, 0xFF3C6253,
        0xFF3F6556, 0xFF315748, 0xFF3E6455, 0xFF39604E, 0xFF406755, 0xFF456C5A, 0xFF345A4B, 0xFF3C6253, 0xFF3A6053, 0xFF22483B, 0xFF3A6053, 0xFF486E5F, 0xFF3B6152, 0xFF658B7C, 0xFF5D8472, 0xFF658C7A, 0xFF5C8371, 0xFF5C8370, 0xFF5E8572, 0xFF618675,
        0xFF608574, 0xFF5F8674, 0xFF5E8573, 0xFF5C8573, 0xFF5C8573, 0xFF5C8573, 0xFF5B8673, 0xFF5D8472, 0xFF5E8573, 0xFF5F8674, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF5F8674, 0xFF5E8573, 0xFF5D8472, 0xFF5D8472,
        0xFF5E8573, 0xFF5E8573, 0xFF5F8674, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF618674, 0xFF608573, 0xFF608573, 0xFF608573, 0xFF608573, 0xFF5F8472, 0xFF5F8472, 0xFF5F8472, 0xFF64866E,
        0xFF5F8878, 0xFF558076, 0xFF486859, 0xFF6E785D, 0xFF988F68, 0xFFC6B388, 0xFFCBB68B, 0xFF85835A, 0xFF5A6044, 0xFF657062, 0xFF838881, 0xFFB9B5AC, 0xFFEEE6D9, 0xFFCFD0C2, 0xFF70786D, 0xFF6F6C67, 0xFF7B7873, 0xFF7E7B76, 0xFF7D7A75, 0xFF807D78,
        0xFF807D78, 0xFF7C7974, 0xFF7A7772, 0xFF777569, 0xFF6F6D60, 0xFF6C6B59, 0xFF63624E, 0xFF5C5B47, 0xFF646351, 0xFF8F8D7E, 0xFFCCCABE, 0xFFECEDE5, 0xFFBECABE, 0xFF6A8274, 0xFF658777, 0xFF557568, 0x008BA099, 0x00FBFFFF, 0x00FFFBFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x00F7FFFF, 0xFF68887D, 0xFF4E786A, 0xFF689183, 0xFF61877A, 0xFF224E3F, 0xFF2B5E4D, 0xFF4C766A, 0xFF436D61, 0xFF325C50, 0xFF376155, 0xFF416B5F,
        0xFF436D61, 0xFF1C463A, 0xFF366054, 0xFF556D5F, 0xFF3B4F43, 0xFF4D594F, 0xFFBBC0B9, 0xFFEDECE7, 0xFFF3F0EB, 0xFFECE9E4, 0xFFEAE7E2, 0xFFF5F1EE, 0xFFF3F2EE, 0xFFAFB4AE, 0xFF495851, 0xFF2F463C, 0xFF3F5F54, 0xFF52786B, 0xFF406A5C, 0xFF4C6C5F,
        0xFF3F5F52, 0xFF3A5C4E, 0xFF3A5C4E, 0xFF3E6254, 0xFF3D6153, 0xFF385C4E, 0xFF466C5D, 0xFF628B7B, 0xFF5C8575, 0xFF608979, 0xFF5B8474, 0xFF5D8676, 0xFF406959, 0xFF4C7565, 0xFF345D4D, 0xFF3D6C5A, 0xFF588775, 0xFF588775, 0xFF578372, 0xFF5F8B7A,
        0xFF5D8877, 0xFF5A8574, 0xFF5A8574, 0xFF588171, 0xFF608979, 0xFF588171, 0xFF588171, 0xFF668F7F, 0xFF618A7A, 0xFF598272, 0xFF668F7F, 0xFF5F8A79, 0xFF608B7A, 0xFF608B7A, 0xFF608B7A, 0xFF608B7A, 0xFF608B7A, 0xFF618C7B, 0xFF618C7B, 0xFF628879,
        0xFF63897A, 0xFF63897A, 0xFF63897A, 0xFF63897A, 0xFF63897A, 0xFF63897A, 0xFF628879, 0xFF638C7C, 0xFF5C8575, 0xFF628B7B, 0xFF5F8878, 0xFF689181, 0xFF5B8474, 0xFF5F8878, 0xFF5E8777, 0xFF608979, 0xFF608979, 0xFF608979, 0xFF608979, 0xFF608979,
        0xFF608979, 0xFF5F8878, 0xFF5F8878, 0xFF5D7D70, 0xFF44685A, 0xFF3B6454, 0xFF34604F, 0xFF406C5B, 0xFF305B4A, 0xFF426B5B, 0xFF406657, 0xFF416856, 0xFF537A68, 0xFF668D7B, 0xFF5F8674, 0xFF608775, 0xFF5B8270, 0xFF628977, 0xFF5F8674, 0xFF3F6452,
        0xFF416654, 0xFF325745, 0xFF1C412F, 0xFF3D6250, 0xFF3A5F4D, 0xFF234836, 0xFF446957, 0xFF3A614E, 0xFF335A47, 0xFF3B624F, 0xFF486F5C, 0xFF638A77, 0xFF5C8370, 0xFF5B826F, 0xFF3D6451, 0xFF3C6652, 0xFF2C5340, 0xFF365B49, 0xFF3F6151, 0xFF3F6151,
        0xFF416355, 0xFF3B5D4F, 0xFF426456, 0xFF3F6453, 0xFF3A5F4E, 0xFF345948, 0xFF466A5C, 0xFF375B4D, 0xFF274B3F, 0xFF406458, 0xFF3F6357, 0xFF375D4E, 0xFF668C7D, 0xFF577D6E, 0xFF668D7B, 0xFF5C8371, 0xFF5F8674, 0xFF5D8471, 0xFF5F8673, 0xFF618675,
        0xFF608574, 0xFF5F8674, 0xFF5E8573, 0xFF5C8573, 0xFF5C8573, 0xFF5C8573, 0xFF5C8774, 0xFF5E8573, 0xFF5F8674, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF5F8674, 0xFF5F8674, 0xFF5F8674, 0xFF5F8674, 0xFF5F8674, 0xFF5E8573, 0xFF5E8573,
        0xFF5E8573, 0xFF5E8573, 0xFF5E8573, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF618674, 0xFF618674, 0xFF618674, 0xFF618674, 0xFF608573, 0xFF608573, 0xFF608573, 0xFF608573, 0xFF628368,
        0xFF5F8674, 0xFF5B867C, 0xFF54786C, 0xFF73826D, 0xFF818062, 0xFFA29978, 0xFFA29779, 0xFF646F4F, 0xFF455643, 0xFF56675F, 0xFF67706D, 0xFF9A978E, 0xFFE8E0D3, 0xFFCCCABD, 0xFF6E7669, 0xFF818177, 0xFF8F8F85, 0xFF95958B, 0xFF929288, 0xFF94948A,
        0xFF95958B, 0xFF929288, 0xFF919187, 0xFF908D86, 0xFF7B796D, 0xFF686657, 0xFF5E5D49, 0xFF5A5945, 0xFF595846, 0xFF817F70, 0xFFCBC9BC, 0xFFE8E9E1, 0xFFBFC9BE, 0xFF6B8375, 0xFF618373, 0xFF537366, 0x008BA099, 0x00FCFFFF, 0x00FFFDFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x00F5FFFD, 0xFF69897E, 0xFF4D7769, 0xFF699284, 0xFF62887B, 0xFF1C4839, 0xFF2F6251, 0xFF467064, 0xFF456F63, 0xFF335D51, 0xFF2D574B, 0xFF416B5F,
        0xFF537D71, 0xFF1F493D, 0xFF2C564A, 0xFF5B7C6B, 0xFF3A5647, 0xFF596B5F, 0xFFD1DBD2, 0xFFE5E6E0, 0xFFCECBC6, 0xFFC5C2BD, 0xFFCBC6C2, 0xFFDFCFD0, 0xFFF1E7E6, 0xFFC0BFBB, 0xFF5C6560, 0xFF2A3F36, 0xFF214134, 0xFF325B4D, 0xFF356152, 0xFF516D61,
        0xFF345246, 0xFF244236, 0xFF244236, 0xFF335346, 0xFF37574A, 0xFF2B4D3F, 0xFF345648, 0xFF4C7565, 0xFF426B5B, 0xFF456E5E, 0xFF487161, 0xFF507969, 0xFF5A8373, 0xFF3A6353, 0xFF436C5C, 0xFF386755, 0xFF3B6A58, 0xFF3D6C5A, 0xFF477362, 0xFF436F5E,
        0xFF3E6958, 0xFF467160, 0xFF497463, 0xFF487161, 0xFF426B5B, 0xFF487161, 0xFF4F7868, 0xFF4C7565, 0xFF4C7565, 0xFF4C7565, 0xFF487161, 0xFF4A7564, 0xFF4A7564, 0xFF4A7564, 0xFF4A7564, 0xFF4B7665, 0xFF4B7665, 0xFF4B7665, 0xFF4B7665, 0xFF4E7465,
        0xFF4F7566, 0xFF507667, 0xFF507667, 0xFF507667, 0xFF507667, 0xFF4F7566, 0xFF4E7465, 0xFF456E5E, 0xFF4D7666, 0xFF4E7767, 0xFF4B7464, 0xFF456E5E, 0xFF4C7565, 0xFF4E7767, 0xFF507969, 0xFF507969, 0xFF507969, 0xFF507969, 0xFF507969, 0xFF507969,
        0xFF517A6A, 0xFF517A6A, 0xFF517A6A, 0xFF446457, 0xFF406254, 0xFF386151, 0xFF3A6655, 0xFF396856, 0xFF34604F, 0xFF3E6958, 0xFF3A6353, 0xFF5A816F, 0xFF688F7D, 0xFF5C8371, 0xFF5A816F, 0xFF658C7A, 0xFF5E8573, 0xFF638A78, 0xFF416856, 0xFF3E644F,
        0xFF426853, 0xFF183E29, 0xFF426853, 0xFF426853, 0xFF446A55, 0xFF3A604B, 0xFF395F4A, 0xFF3F6653, 0xFF355C49, 0xFF3B624F, 0xFF486F5C, 0xFF618875, 0xFF59806D, 0xFF678E7B, 0xFF577E6B, 0xFF36604A, 0xFF3B624D, 0xFF466857, 0xFF49695A, 0xFF446354,
        0xFF48665A, 0xFF4C6C5F, 0xFF46665B, 0xFF426756, 0xFF456A59, 0xFF436857, 0xFF3D6153, 0xFF4B6F61, 0xFF4D7165, 0xFF3D6155, 0xFF43675B, 0xFF608677, 0xFF63897A, 0xFF658B7C, 0xFF547B69, 0xFF608775, 0xFF5F8674, 0xFF5B826F, 0xFF648B78, 0xFF618675,
        0xFF618675, 0xFF5F8674, 0xFF5E8573, 0xFF5C8573, 0xFF5C8573, 0xFF5C8573, 0xFF5C8774, 0xFF5F8674, 0xFF5F8674, 0xFF608775, 0xFF618876, 0xFF608775, 0xFF608775, 0xFF5F8674, 0xFF5E8573, 0xFF5E8573, 0xFF5F8674, 0xFF5F8674, 0xFF5F8674, 0xFF5F8674,
        0xFF5F8674, 0xFF5E8573, 0xFF5E8573, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF628775, 0xFF628775, 0xFF618674, 0xFF618674, 0xFF618674, 0xFF618674, 0xFF608573, 0xFF608573, 0xFF658367,
        0xFF648B78, 0xFF638E84, 0xFF5A8075, 0xFF748774, 0xFF787C63, 0xFF87836A, 0xFF716F58, 0xFF4D6148, 0xFF374F41, 0xFF556B68, 0xFF6B7775, 0xFF9A9A92, 0xFFECE4D7, 0xFFC1BFB0, 0xFF62685C, 0xFF6F705E, 0xFF7F806E, 0xFF858674, 0xFF828371, 0xFF838472,
        0xFF848573, 0xFF828371, 0xFF828371, 0xFF7F7F77, 0xFF6C6C62, 0xFF5F6052, 0xFF626351, 0xFF646651, 0xFF575846, 0xFF6D6E5E, 0xFFB3B4A6, 0xFFECECE4, 0xFFC4CEC3, 0xFF728A7C, 0xFF658576, 0xFF557568, 0x008DA09A, 0x00FCFFFF, 0x00FFFDFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x08F1FFFB, 0xFF65897D, 0xFF4C7869, 0xFF658F81, 0xFF5C8577, 0xFF1B4A3A, 0xFF346957, 0xFF325C50, 0xFF2F594D, 0xFF457164, 0xFF154134, 0xFF487669,
        0xFF336154, 0xFF154336, 0xFF2C5C4E, 0xFF5F7B6C, 0xFF465E50, 0xFF6A7A6F, 0xFFCBD2CA, 0xFFDFE2DB, 0xFFD8D9D3, 0xFFC0C1BB, 0xFFC5C6C0, 0xFFD4D1CC, 0xFFE5E2DD, 0xFFCCCBC6, 0xFF787B74, 0xFF7F847D, 0xFF606A61, 0xFF566258, 0xFF79867C, 0xFFA4A4A2,
        0xFF6F716E, 0xFF5D625E, 0xFF5E6460, 0xFF59645E, 0xFF93A29B, 0xFF697A72, 0xFF4A5B53, 0xFF3D6656, 0xFF456E5E, 0xFF3A6353, 0xFF426B5B, 0xFF3B6454, 0xFF477060, 0xFF557E6E, 0xFF315A4A, 0xFF3C6557, 0xFF416A5C, 0xFF466F61, 0xFF456E60, 0xFF416A5C,
        0xFF3E6759, 0xFF3F685A, 0xFF426B5D, 0xFF44685A, 0xFF436759, 0xFF426658, 0xFF44685A, 0xFF466A5C, 0xFF476B5D, 0xFF45695B, 0xFF436759, 0xFF3F675C, 0xFF40685D, 0xFF426A5F, 0xFF436B60, 0xFF446C61, 0xFF436B60, 0xFF426A5F, 0xFF41695E, 0xFF456B5C,
        0xFF456B5C, 0xFF456B5C, 0xFF456B5C, 0xFF456B5C, 0xFF456B5C, 0xFF456B5C, 0xFF456B5C, 0xFF396254, 0xFF3C6557, 0xFF40695B, 0xFF43695C, 0xFF446A5D, 0xFF45695D, 0xFF466A5E, 0xFF486C60, 0xFF416B5F, 0xFF476F64, 0xFF446C61, 0xFF426A5F, 0xFF486E63,
        0xFF466C61, 0xFF45695F, 0xFF4A6E64, 0xFF567A70, 0xFF3A5E52, 0xFF45695D, 0xFF416559, 0xFF385C4E, 0xFF476B5D, 0xFF3B604F, 0xFF618675, 0xFF628976, 0xFF628976, 0xFF59806D, 0xFF5F8673, 0xFF5B826F, 0xFF608774, 0xFF476E5B, 0xFF39604D, 0xFF4A715F,
        0xFF193E2D, 0xFF466657, 0xFF4C6B5C, 0xFF436253, 0xFF648475, 0xFF3D6251, 0xFF3D6452, 0xFF3E6A59, 0xFF305B4A, 0xFF43695A, 0xFF46685A, 0xFF6A8C7E, 0xFF5C8072, 0xFF577D6E, 0xFF658E7E, 0xFF5B7F71, 0xFF3F6355, 0xFF3F6355, 0xFF3D6153, 0xFF3C6052,
        0xFF416557, 0xFF3A5E50, 0xFF406456, 0xFF3D6251, 0xFF446958, 0xFF3B604F, 0xFF3D6251, 0xFF496E5D, 0xFF406554, 0xFF436857, 0xFF658A79, 0xFF608775, 0xFF5F8674, 0xFF5F8674, 0xFF5F8674, 0xFF5F8674, 0xFF5E8573, 0xFF5E8573, 0xFF5E8573, 0xFF608774,
        0xFF608774, 0xFF608774, 0xFF608774, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5D8471, 0xFF5D8471, 0xFF5D8471, 0xFF5D8471, 0xFF5D8471, 0xFF5D8471, 0xFF5D8471, 0xFF5D8471, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673,
        0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5A8572, 0xFF5F8876, 0xFF5A8371, 0xFF5E8775, 0xFF5F8674, 0xFF5C8371, 0xFF648978, 0xFF618675, 0xFF628776, 0xFF628776, 0xFF5F8473, 0xFF5F8473, 0xFF638877, 0xFF5F8473, 0xFF5B806F, 0xFF648978, 0xFF608775,
        0xFF5F8674, 0xFF547968, 0xFF577C6B, 0xFF5C7E6E, 0xFF5C7E6E, 0xFF557767, 0xFF39594A, 0xFF3E5B56, 0xFF384F49, 0xFF5B6660, 0xFF6D6E68, 0xFF9D998E, 0xFFE7E1D3, 0xFFC2BFAC, 0xFFA9A894, 0xFFABA197, 0xFFAEA49A, 0xFFAEA69B, 0xFFAFA79C, 0xFFB7AFA4,
        0xFFBAB2A7, 0xFFB6AEA3, 0xFFB9B1A6, 0xFFB4AAA0, 0xFFC7BFB4, 0xFFB6B2A6, 0xFFB4B2A5, 0xFFBAB8AB, 0xFFB3B1A4, 0xFFC1BBAF, 0xFFCAC4B8, 0xFFEFE7E4, 0xFFCBD0C9, 0xFF71877B, 0xFF638877, 0xFF527766, 0x0088A297, 0x00F9FFFD, 0x00FDFCFA, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x04F3FFFD, 0xFF68897E, 0xFF4C7869, 0xFF648E80, 0xFF5B8476, 0xFF1B4A3A, 0xFF2B604E, 0xFF456F63, 0xFF406A5E, 0xFF3E685C, 0xFF1C483B, 0xFF447063,
        0xFF265447, 0xFF174538, 0xFF305E51, 0xFF62786B, 0xFF4C5C51, 0xFF79837A, 0xFFDCDFD8, 0xFFE1DED9, 0xFFD6D1CD, 0xFFD3CECA, 0xFFDED9D5, 0xFFDEDAD7, 0xFFD9D8D4, 0xFFC1C3BE, 0xFF9AA19A, 0xFF939E96, 0xFF47574D, 0xFF4E6257, 0xFF8BA195, 0xFFA8B1AC,
        0xFF79847E, 0xFF525F58, 0xFF5B6A63, 0xFF798D84, 0xFF9FB4AB, 0xFF576E64, 0xFF384F45, 0xFF3D6656, 0xFF305949, 0xFF477060, 0xFF386151, 0xFF376050, 0xFF3D6656, 0xFF3D6656, 0xFF305949, 0xFF345D4F, 0xFF386153, 0xFF3C6557, 0xFF3D6658, 0xFF3B6456,
        0xFF396254, 0xFF3B6456, 0xFF3D6658, 0xFF406456, 0xFF3F6355, 0xFF3F6355, 0xFF406456, 0xFF426658, 0xFF426658, 0xFF416557, 0xFF406456, 0xFF396156, 0xFF3A6257, 0xFF3C6459, 0xFF3D655A, 0xFF3D655A, 0xFF3C6459, 0xFF3B6358, 0xFF3A6257, 0xFF3C6253,
        0xFF3C6253, 0xFF3C6253, 0xFF3B6152, 0xFF3B6152, 0xFF3A6051, 0xFF3A6051, 0xFF3A6051, 0xFF376052, 0xFF396254, 0xFF3D6658, 0xFF406659, 0xFF406659, 0xFF406659, 0xFF42665A, 0xFF43675B, 0xFF406A5E, 0xFF426A5F, 0xFF41695E, 0xFF41695E, 0xFF42685D,
        0xFF42685D, 0xFF43675D, 0xFF44685E, 0xFF31574A, 0xFF4D7366, 0xFF446A5D, 0xFF2F5548, 0xFF486E5F, 0xFF3B6152, 0xFF59806E, 0xFF648B79, 0xFF57806E, 0xFF5B8472, 0xFF668F7D, 0xFF58816F, 0xFF638C7A, 0xFF446D5B, 0xFF436C5A, 0xFF416A58, 0xFF1F4836,
        0xFF416856, 0xFF436555, 0xFF446455, 0xFF6C8C7D, 0xFF6F9181, 0xFF345B49, 0xFF3F6856, 0xFF36634F, 0xFF3E6956, 0xFF345D4B, 0xFF537C6A, 0xFF527967, 0xFF628977, 0xFF6F9684, 0xFF547B69, 0xFF608775, 0xFF638A78, 0xFF668D7B, 0xFF658C7A, 0xFF5F8674,
        0xFF608775, 0xFF638A78, 0xFF5E8573, 0xFF5F8473, 0xFF658A79, 0xFF678C7B, 0xFF608574, 0xFF5D8271, 0xFF638877, 0xFF668B7A, 0xFF638877, 0xFF5D8472, 0xFF5D8472, 0xFF5E8573, 0xFF5E8573, 0xFF5F8674, 0xFF608775, 0xFF618876, 0xFF618876, 0xFF608774,
        0xFF608774, 0xFF608774, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673,
        0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5A8572, 0xFF5E8775, 0xFF5A8371, 0xFF5E8775, 0xFF618876, 0xFF5E8573, 0xFF648978, 0xFF628776, 0xFF5F8473, 0xFF618675, 0xFF5F8473, 0xFF638877, 0xFF628776, 0xFF608574, 0xFF658A79, 0xFF628776, 0xFF4F7664,
        0xFF406755, 0xFF3F6453, 0xFF507564, 0xFF436555, 0xFF496B5B, 0xFF486A5A, 0xFF507061, 0xFF3C5D54, 0xFF3B564D, 0xFF5E6F67, 0xFF727C73, 0xFF999F95, 0xFFE2E6D8, 0xFFE3EADA, 0xFFE1EAD9, 0xFFE6E8DD, 0xFFE9EBE0, 0xFFE9EBE0, 0xFFE9EBE0, 0xFFEAECE1,
        0xFFE8EADF, 0xFFEAECE1, 0xFFF4F6EB, 0xFFF2EFE6, 0xFFE7E7DD, 0xFFEBEDE2, 0xFFEEF1E6, 0xFFE8EEE2, 0xFFE8EEE2, 0xFFE9ECE1, 0xFFE1E4D9, 0xFFE7E8E2, 0xFFC8D4CA, 0xFF779385, 0xFF678E7C, 0xFF4F7664, 0x00859F94, 0x00F9FFFD, 0x00FEFAF9, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFCFD, 0x00F5FFFD, 0xFF6A8B80, 0xFF517B6D, 0xFF6B9486, 0xFF60897B, 0xFF245041, 0xFF265B49, 0xFF598377, 0xFF4B7569, 0xFF2A5448, 0xFF214D40, 0xFF426E61,
        0xFF265245, 0xFF19473A, 0xFF275548, 0xFF617268, 0xFF4B584F, 0xFF6C736C, 0xFFD5D6D1, 0xFFEDE9E6, 0xFFDFDAD7, 0xFFD8D3D0, 0xFFDDD9D6, 0xFFDCDCDC, 0xFFEEF2F1, 0xFFB2BBB8, 0xFF62716C, 0xFF5D726B, 0xFF466158, 0xFF527267, 0xFF3E5F54, 0xFF50675D,
        0xFF576E64, 0xFF476156, 0xFF476357, 0xFF446256, 0xFF466659, 0xFF27493B, 0xFF3F6153, 0xFF5C8575, 0xFF5E8777, 0xFF557E6E, 0xFF618A7A, 0xFF3F6858, 0xFF3F6858, 0xFF416A5A, 0xFF3D6656, 0xFF3D6658, 0xFF40695B, 0xFF426B5D, 0xFF436C5E, 0xFF436C5E,
        0xFF426B5D, 0xFF436C5E, 0xFF446D5F, 0xFF466C5D, 0xFF466C5D, 0xFF466C5D, 0xFF476D5E, 0xFF486E5F, 0xFF486E5F, 0xFF476D5E, 0xFF466C5D, 0xFF466E63, 0xFF466E63, 0xFF476F64, 0xFF487065, 0xFF487065, 0xFF476F64, 0xFF456D62, 0xFF456D62, 0xFF456E5E,
        0xFF456E5E, 0xFF446D5D, 0xFF436C5C, 0xFF426B5B, 0xFF426B5B, 0xFF416A5A, 0xFF416A5A, 0xFF3F695B, 0xFF416B5D, 0xFF446D5F, 0xFF466F61, 0xFF486E61, 0xFF476D60, 0xFF486E61, 0xFF496D61, 0xFF477165, 0xFF466E63, 0xFF487065, 0xFF4A7267, 0xFF4B7166,
        0xFF4D7368, 0xFF50746A, 0xFF4E7268, 0xFF50796B, 0xFF40695B, 0xFF345D4F, 0xFF507969, 0xFF456E5E, 0xFF598270, 0xFF5E8775, 0xFF5E8775, 0xFF5F897B, 0xFF5B8577, 0xFF5D8779, 0xFF679183, 0xFF487264, 0xFF3E685A, 0xFF416B5D, 0xFF1D4739, 0xFF436E5B,
        0xFF3B6452, 0xFF4B705F, 0xFF6B8D7D, 0xFF6A8C7C, 0xFF5C8170, 0xFF426B59, 0xFF3D6855, 0xFF466F5D, 0xFF396451, 0xFF376450, 0xFF3C6955, 0xFF618E7A, 0xFF5B8472, 0xFF587F6D, 0xFF668B7A, 0xFF58816F, 0xFF618A78, 0xFF58816F, 0xFF5B8472, 0xFF598270,
        0xFF598270, 0xFF668F7D, 0xFF5A8371, 0xFF5E8372, 0xFF5B806F, 0xFF5F8473, 0xFF638877, 0xFF618675, 0xFF658A79, 0xFF678C7B, 0xFF608574, 0xFF5D8472, 0xFF5D8472, 0xFF5E8573, 0xFF5F8674, 0xFF608775, 0xFF608775, 0xFF618876, 0xFF618876, 0xFF608774,
        0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673,
        0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5A8572, 0xFF5E8775, 0xFF5B8472, 0xFF5E8775, 0xFF608775, 0xFF5E8573, 0xFF628776, 0xFF5F8473, 0xFF5E8372, 0xFF628776, 0xFF608574, 0xFF658A79, 0xFF608574, 0xFF5F8473, 0xFF648978, 0xFF547968, 0xFF466D5B,
        0xFF39604E, 0xFF476C5B, 0xFF436857, 0xFF3F6151, 0xFF486A5A, 0xFF416353, 0xFF3E5E4F, 0xFF476D5E, 0xFF46685A, 0xFF5D796D, 0xFF6E8478, 0xFF84988D, 0xFFA8B9AF, 0xFFA6BAAE, 0xFF99ADA1, 0xFF9EB2A6, 0xFF9DB1A5, 0xFF98ACA0, 0xFF99ADA1, 0xFF9DAFA3,
        0xFF96A89C, 0xFF91A397, 0xFF93A599, 0xFF97A399, 0xFF89958B, 0xFF909C92, 0xFF869389, 0xFF85958A, 0xFF9DAFA3, 0xFF9BADA1, 0xFF92A69A, 0xFF99A99E, 0xFF8CA496, 0xFF628776, 0xFF608B78, 0xFF507765, 0x0088A297, 0x00FAFFFC, 0x00FFFDFC, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFCFD, 0x00F6FFFE, 0xFF6A8B80, 0xFF4F796B, 0xFF668F81, 0xFF597F72, 0xFF1E4A3B, 0xFF164938, 0xFF1C4537, 0xFF214A3C, 0xFF0F392B, 0xFF174133, 0xFF224E3F,
        0xFF133F30, 0xFF174334, 0xFF2E5D4D, 0xFF5A7668, 0xFF4F675A, 0xFF55665C, 0xFFB2BDB5, 0xFFE9F0E9, 0xFFE0E7E0, 0xFFD1DBD3, 0xFFD8E2DA, 0xFFE4EDEC, 0xFFE1EBEA, 0xFFA6B6B3, 0xFF687F79, 0xFF5B7971, 0xFF42685D, 0xFF5E887C, 0xFF578578, 0xFF5B7F71,
        0xFF608476, 0xFF5A8071, 0xFF618778, 0xFF537C6C, 0xFF537E6D, 0xFF547F6E, 0xFF669180, 0xFF5E8777, 0xFF5E8777, 0xFF638C7C, 0xFF598272, 0xFF4F7868, 0xFF2C5545, 0xFF497262, 0xFF3C6555, 0xFF426C5E, 0xFF426C5E, 0xFF426C5E, 0xFF426C5E, 0xFF426C5E,
        0xFF416B5D, 0xFF406A5C, 0xFF3F695B, 0xFF426B5B, 0xFF426B5B, 0xFF426B5B, 0xFF436C5C, 0xFF436C5C, 0xFF436C5C, 0xFF426B5B, 0xFF426B5B, 0xFF3D6759, 0xFF3E685A, 0xFF3E685A, 0xFF3E685A, 0xFF3E685A, 0xFF3D6759, 0xFF3C6658, 0xFF3B6557, 0xFF3E6757,
        0xFF3E6757, 0xFF3E6757, 0xFF3D6656, 0xFF3D6656, 0xFF3C6555, 0xFF3C6555, 0xFF3C6555, 0xFF3C6756, 0xFF3D6857, 0xFF3E6958, 0xFF416A5A, 0xFF416A5A, 0xFF426859, 0xFF426859, 0xFF416758, 0xFF3F695B, 0xFF426B5D, 0xFF40695B, 0xFF3F685A, 0xFF43695C,
        0xFF42685B, 0xFF416559, 0xFF416559, 0xFF487163, 0xFF416A5C, 0xFF416A5A, 0xFF365F4F, 0xFF567F6D, 0xFF6B9482, 0xFF648E7A, 0xFF5E8874, 0xFF5F877C, 0xFF60887D, 0xFF60887D, 0xFF4F776C, 0xFF41695E, 0xFF4C7469, 0xFF1E463B, 0xFF345C51, 0xFF44715D,
        0xFF47725F, 0xFF608775, 0xFF648978, 0xFF5E8372, 0xFF678E7C, 0xFF46715E, 0xFF3E6B57, 0xFF406755, 0xFF285340, 0xFF44715D, 0xFF2F5E4A, 0xFF4E7B67, 0xFF5F8A77, 0xFF6C9381, 0xFF618373, 0xFF5D8674, 0xFF628B79, 0xFF5E8775, 0xFF658E7C, 0xFF628B79,
        0xFF598270, 0xFF648D7B, 0xFF638C7A, 0xFF688D7C, 0xFF648978, 0xFF5E8372, 0xFF628776, 0xFF678C7B, 0xFF608574, 0xFF5B806F, 0xFF638877, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF5F8674, 0xFF5F8674, 0xFF5F8674, 0xFF5F8674, 0xFF5F8674, 0xFF5F8673,
        0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF5E8572, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673,
        0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5C8774, 0xFF5E8775, 0xFF5D8674, 0xFF5E8775, 0xFF5F8674, 0xFF5D8472, 0xFF5F8473, 0xFF5E8372, 0xFF628776, 0xFF638877, 0xFF648978, 0xFF608574, 0xFF5F8473, 0xFF5F8473, 0xFF527766, 0xFF416655, 0xFF406755,
        0xFF406755, 0xFF4A6F5E, 0xFF2C5140, 0xFF426454, 0xFF4D6F5F, 0xFF577969, 0xFF4A6A5B, 0xFF396250, 0xFF517866, 0xFF688D7C, 0xFF6C8C7F, 0xFF6F8D81, 0xFF6F8A81, 0xFF6E8980, 0xFF68837A, 0xFF5F8473, 0xFF668878, 0xFF5F8171, 0xFF5B7B6C, 0xFF567566,
        0xFF456455, 0xFF395547, 0xFF304C3E, 0xFF2E4237, 0xFF4B5F54, 0xFF54685D, 0xFF405449, 0xFF53695D, 0xFF6E887B, 0xFF618071, 0xFF5B7D6D, 0xFF648071, 0xFF658776, 0xFF5C8774, 0xFF63907C, 0xFF557C6A, 0x008EA59B, 0x00FAFFFC, 0x00FFFDFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x00F8FFFF, 0xFF6A8A7F, 0xFF4F786A, 0xFF688E81, 0xFF5D8376, 0xFF345E50, 0xFF2A5D4C, 0xFF2D5648, 0xFF345D4F, 0xFF315A4C, 0xFF325C4E, 0xFF2D5749,
        0xFF2B5748, 0xFF285445, 0xFF3F6B5C, 0xFF537E6B, 0xFF527967, 0xFF446656, 0xFF718D7F, 0xFF8CA699, 0xFF809A8D, 0xFF7C988A, 0xFF86A596, 0xFF7A8A87, 0xFF899C98, 0xFF78918B, 0xFF45635B, 0xFF3B6156, 0xFF406C5F, 0xFF5D8D7F, 0xFF56897A, 0xFF64907F,
        0xFF5D8978, 0xFF558170, 0xFF618D7C, 0xFF5D8C7A, 0xFF608F7D, 0xFF608F7D, 0xFF558472, 0xFF608979, 0xFF5B8474, 0xFF618A7A, 0xFF5B8474, 0xFF376050, 0xFF487161, 0xFF386151, 0xFF487161, 0xFF3F695B, 0xFF3E685A, 0xFF3E685A, 0xFF3E685A, 0xFF3E685A,
        0xFF3D6759, 0xFF3C6658, 0xFF3B6557, 0xFF3A6554, 0xFF3B6655, 0xFF3C6756, 0xFF3C6756, 0xFF3B6655, 0xFF3B6655, 0xFF3A6554, 0xFF3A6554, 0xFF3D6759, 0xFF3D6759, 0xFF3D6759, 0xFF3D6759, 0xFF3D6759, 0xFF3C6658, 0xFF3B6557, 0xFF3B6557, 0xFF3D6656,
        0xFF3D6656, 0xFF3D6656, 0xFF3E6757, 0xFF3E6757, 0xFF3E6757, 0xFF3F6858, 0xFF3F6858, 0xFF3B6756, 0xFF3C6857, 0xFF3E6958, 0xFF3F6A59, 0xFF416A5A, 0xFF406959, 0xFF416758, 0xFF406657, 0xFF396355, 0xFF426B5D, 0xFF3E6759, 0xFF3D6658, 0xFF486E61,
        0xFF466C5F, 0xFF42665A, 0xFF496D61, 0xFF3C6253, 0xFF416758, 0xFF476D5E, 0xFF587F6D, 0xFF5F8674, 0xFF3F6653, 0xFF456C59, 0xFF4F7663, 0xFF456B5E, 0xFF4B7164, 0xFF4C7265, 0xFF395F52, 0xFF41675A, 0xFF254B3E, 0xFF375D50, 0xFF547A6D, 0xFF3D6A56,
        0xFF5C8573, 0xFF658C7A, 0xFF628474, 0xFF688A7A, 0xFF628977, 0xFF355E4C, 0xFF386551, 0xFF3D6451, 0xFF325C48, 0xFF224C38, 0xFF426C58, 0xFF3E6854, 0xFF3C6350, 0xFF486D5B, 0xFF486A59, 0xFF476E5B, 0xFF436A57, 0xFF476E5B, 0xFF577E6B, 0xFF658C79,
        0xFF658C79, 0xFF5C8370, 0xFF577E6B, 0xFF608574, 0xFF648978, 0xFF608574, 0xFF5C8170, 0xFF608574, 0xFF5F8473, 0xFF5C8170, 0xFF5E8372, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF5F8674, 0xFF5F8674, 0xFF5F8674, 0xFF608774,
        0xFF608774, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5E8572, 0xFF5E8572, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673,
        0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5C8774, 0xFF5E8775, 0xFF608977, 0xFF5F8876, 0xFF5F8674, 0xFF5F8674, 0xFF608574, 0xFF628776, 0xFF618675, 0xFF618675, 0xFF668B7A, 0xFF5B806F, 0xFF648978, 0xFF628776, 0xFF3F6453, 0xFF3B604F, 0xFF3E6553,
        0xFF577E6C, 0xFF4A6F5E, 0xFF3F6453, 0xFF416353, 0xFF456757, 0xFF426454, 0xFF416152, 0xFF4E7864, 0xFF668D7A, 0xFF6C9381, 0xFF567B6A, 0xFF5C7C6F, 0xFF526E62, 0xFF566D63, 0xFF5B7069, 0xFF416A5A, 0xFF4F7868, 0xFF476D5E, 0xFF395D4F, 0xFF305244,
        0xFF2F4F42, 0xFF3C5A4E, 0xFF435F53, 0xFF2D443A, 0xFF576C63, 0xFF52665D, 0xFF3F5048, 0xFF546960, 0xFF698579, 0xFF628678, 0xFF638C7C, 0xFF648675, 0xFF648B78, 0xFF648F7C, 0xFF648F7C, 0xFF527766, 0x008BA097, 0x00FAFFFC, 0x00FFFEFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x00F8FFFF, 0xFF69897E, 0xFF517A6C, 0xFF6E9487, 0xFF6C9084, 0xFF5B8577, 0xFF588878, 0xFF688E7F, 0xFF618778, 0xFF5E8777, 0xFF5C8575, 0xFF5A8574,
        0xFF65907F, 0xFF5B8675, 0xFF608C7B, 0xFF548774, 0xFF558673, 0xFF547F6E, 0xFF648A7B, 0xFF5D8374, 0xFF577D6E, 0xFF608979, 0xFF648F7E, 0xFF647F76, 0xFF5D7B71, 0xFF638378, 0xFF53776B, 0xFF578072, 0xFF578374, 0xFF5F8E7E, 0xFF5C8C7C, 0xFF568573,
        0xFF608F7D, 0xFF5D8C7A, 0xFF5B8A78, 0xFF588775, 0xFF54806F, 0xFF5A8675, 0xFF5A8675, 0xFF689181, 0xFF557E6E, 0xFF578070, 0xFF376050, 0xFF396252, 0xFF3C6555, 0xFF4E7767, 0xFF507969, 0xFF578374, 0xFF578374, 0xFF578374, 0xFF578374, 0xFF588475,
        0xFF598576, 0xFF588475, 0xFF588475, 0xFF568271, 0xFF578372, 0xFF588473, 0xFF588473, 0xFF578372, 0xFF568271, 0xFF568271, 0xFF568271, 0xFF54806F, 0xFF54806F, 0xFF54806F, 0xFF537F6E, 0xFF537F6E, 0xFF537F6E, 0xFF537F6E, 0xFF537F6E, 0xFF517C6B,
        0xFF527D6C, 0xFF527D6C, 0xFF537E6D, 0xFF547F6E, 0xFF55806F, 0xFF55806F, 0xFF568170, 0xFF517E6A, 0xFF517E6A, 0xFF527F6B, 0xFF55806D, 0xFF56816E, 0xFF57806E, 0xFF557E6C, 0xFF547D6B, 0xFF4D7867, 0xFF567F6F, 0xFF547D6D, 0xFF527B6B, 0xFF597F70,
        0xFF567C6D, 0xFF537769, 0xFF597D6F, 0xFF587A6C, 0xFF5D7F71, 0xFF5E8070, 0xFF5A7C6C, 0xFF325443, 0xFF4D6F5E, 0xFF4D6F5E, 0xFF40634F, 0xFF517162, 0xFF476758, 0xFF49695A, 0xFF49695A, 0xFF325243, 0xFF2F4F40, 0xFF4B6B5C, 0xFF365647, 0xFF638C7A,
        0xFF69907E, 0xFF5F8171, 0xFF709081, 0xFF608071, 0xFF476959, 0xFF406755, 0xFF3F6856, 0xFF2D5743, 0xFF466D5A, 0xFF385948, 0xFF2A4637, 0xFF466253, 0xFF517060, 0xFF4D6F5E, 0xFF486D5B, 0xFF557766, 0xFF4A6C5B, 0xFF3F6150, 0xFF406251, 0xFF547665,
        0xFF668877, 0xFF688A79, 0xFF688A79, 0xFF658A79, 0xFF5C8170, 0xFF608574, 0xFF688D7C, 0xFF638877, 0xFF5E8372, 0xFF5D8271, 0xFF5B806F, 0xFF5E8573, 0xFF5E8573, 0xFF5F8674, 0xFF5F8674, 0xFF608775, 0xFF618876, 0xFF628977, 0xFF628977, 0xFF608774,
        0xFF608774, 0xFF608774, 0xFF608774, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF608774, 0xFF608774, 0xFF608774, 0xFF608774, 0xFF608774, 0xFF608774, 0xFF608774, 0xFF608774, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673,
        0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5C8774, 0xFF5D8674, 0xFF628B79, 0xFF5E8775, 0xFF5C8371, 0xFF5D8472, 0xFF5E8372, 0xFF648978, 0xFF5E8372, 0xFF5F8473, 0xFF648978, 0xFF5C8170, 0xFF658A79, 0xFF5B806F, 0xFF375C4B, 0xFF406554, 0xFF4C7361,
        0xFF6E9583, 0xFF3F6453, 0xFF4E7362, 0xFF3B5D4D, 0xFF678979, 0xFF628474, 0xFF739384, 0xFF678C7B, 0xFF688D7B, 0xFF6C8E7D, 0xFF4D6E5D, 0xFF637E6F, 0xFF637567, 0xFF616B60, 0xFF666C62, 0xFF3E6352, 0xFF4E7060, 0xFF416152, 0xFF395547, 0xFF365043,
        0xFF3B5145, 0xFF4C6055, 0xFF4F6358, 0xFF526359, 0xFF5D6D63, 0xFF5A655D, 0xFF566058, 0xFF606D64, 0xFF657D70, 0xFF648475, 0xFF5E8775, 0xFF648675, 0xFF648977, 0xFF658C7A, 0xFF638A78, 0xFF517162, 0x00899D94, 0x00F9FFFD, 0x00FFFFFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFCFF, 0x00F8FFFF, 0xFF68867C, 0xFF4F786A, 0xFF698F82, 0xFF64887C, 0xFF618A7C, 0xFF619181, 0xFF658B7C, 0xFF618778, 0xFF688E7F, 0xFF638C7C, 0xFF5A8373,
        0xFF5E8978, 0xFF628D7C, 0xFF669180, 0xFF5E8F7C, 0xFF618D7C, 0xFF689181, 0xFF709687, 0xFF6C9082, 0xFF698F80, 0xFF6B9484, 0xFF689382, 0xFF6A917F, 0xFF688F7D, 0xFF709785, 0xFF69907E, 0xFF68917F, 0xFF618A78, 0xFF668F7D, 0xFF628B79, 0xFF638F7E,
        0xFF5E8A79, 0xFF568271, 0xFF547F6E, 0xFF618C7B, 0xFF5F8878, 0xFF628B7B, 0xFF5D8676, 0xFF5E8777, 0xFF578070, 0xFF376050, 0xFF3E6757, 0xFF3C6555, 0xFF527B6B, 0xFF668F7F, 0xFF658E7E, 0xFF598576, 0xFF598576, 0xFF598576, 0xFF598576, 0xFF598576,
        0xFF598576, 0xFF5A8677, 0xFF5A8677, 0xFF588473, 0xFF598574, 0xFF5B8776, 0xFF5A8675, 0xFF588473, 0xFF578372, 0xFF578372, 0xFF588473, 0xFF598574, 0xFF598574, 0xFF598574, 0xFF598574, 0xFF598574, 0xFF5A8675, 0xFF5A8675, 0xFF5A8675, 0xFF588372,
        0xFF598473, 0xFF598473, 0xFF5A8574, 0xFF5A8574, 0xFF5B8675, 0xFF5B8675, 0xFF5B8675, 0xFF5C8975, 0xFF5B8874, 0xFF5C8975, 0xFF5E8B77, 0xFF618C79, 0xFF618C79, 0xFF608977, 0xFF5F8876, 0xFF5F8A79, 0xFF5D8676, 0xFF5F8878, 0xFF5A8373, 0xFF4F7566,
        0xFF486E5F, 0xFF486C5E, 0xFF44685A, 0xFF496859, 0xFF3E5D4E, 0xFF4E6D5E, 0xFF395848, 0xFF496858, 0xFF325141, 0xFF22412F, 0xFF2C4B39, 0xFF365341, 0xFF2A4735, 0xFF405D4B, 0xFF324F3D, 0xFF2C4937, 0xFF54715F, 0xFF1E3B29, 0xFF3B5846, 0xFF476C5B,
        0xFF547666, 0xFF3E5D4E, 0xFF415D4F, 0xFF3E5A4C, 0xFF385748, 0xFF3C5E4E, 0xFF496E5D, 0xFF5E8B74, 0xFF385B47, 0xFF4F6757, 0xFF404E41, 0xFF546255, 0xFF3B5041, 0xFF2D4C3A, 0xFF284F3A, 0xFF234230, 0xFF335240, 0xFF375644, 0xFF466553, 0xFF4C6B59,
        0xFF446351, 0xFF496856, 0xFF4B6A58, 0xFF4B705F, 0xFF406554, 0xFF456A59, 0xFF597E6D, 0xFF638877, 0xFF5F8473, 0xFF5D8271, 0xFF628776, 0xFF5E8573, 0xFF5E8573, 0xFF5F8674, 0xFF608775, 0xFF618876, 0xFF618876, 0xFF628977, 0xFF628977, 0xFF618875,
        0xFF618875, 0xFF618875, 0xFF608774, 0xFF608774, 0xFF608774, 0xFF608774, 0xFF608774, 0xFF608774, 0xFF608774, 0xFF608774, 0xFF608774, 0xFF608774, 0xFF608774, 0xFF608774, 0xFF608774, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673,
        0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5C8774, 0xFF5C8573, 0xFF618A78, 0xFF58816F, 0xFF517866, 0xFF517866, 0xFF517665, 0xFF5A7F6E, 0xFF5D8271, 0xFF628776, 0xFF5F8473, 0xFF628776, 0xFF5C8170, 0xFF406554, 0xFF315645, 0xFF3D6251, 0xFF3F6654,
        0xFF4C7361, 0xFF325746, 0xFF416655, 0xFF305242, 0xFF527464, 0xFF4D6F5F, 0xFF5C7C6D, 0xFF5A786C, 0xFF4C6B5C, 0xFF668575, 0xFF4F6855, 0xFF657664, 0xFF6B6F5E, 0xFF6B6454, 0xFF786A5D, 0xFF435B4D, 0xFF566E60, 0xFF607468, 0xFF7D8F83, 0xFF94A197,
        0xFF949E95, 0xFF999E97, 0xFF90958E, 0xFF808A81, 0xFF828780, 0xFF9D9C97, 0xFFABA8A3, 0xFFA7AAA3, 0xFF99A69C, 0xFF779283, 0xFF648675, 0xFF6A8677, 0xFF6B8A7A, 0xFF6A8A7B, 0xFF6A8A7B, 0xFF5A7668, 0x008B9F96, 0x00F9FFFD, 0x00F9FEFA, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFCFF, 0x00F7FFFD, 0xFF69877D, 0xFF547A6D, 0xFF6B8F83, 0xFF618579, 0xFF618A7C, 0xFF629282, 0xFF6B9182, 0xFF628879, 0xFF648A7B, 0xFF668F7F, 0xFF638C7C,
        0xFF5E8978, 0xFF65907F, 0xFF5D8877, 0xFF608B7A, 0xFF668C7D, 0xFF67897B, 0xFF648477, 0xFF68867A, 0xFF618174, 0xFF537769, 0xFF577D6E, 0xFF547F6B, 0xFF527D69, 0xFF55806C, 0xFF5F8975, 0xFF688F7C, 0xFF5D8471, 0xFF668B79, 0xFF5F8472, 0xFF618C7B,
        0xFF588372, 0xFF5F8A79, 0xFF5F8878, 0xFF618778, 0xFF5A8071, 0xFF688C7E, 0xFF668A7C, 0xFF557E6E, 0xFF3A6353, 0xFF406959, 0xFF3B6454, 0xFF4E7767, 0xFF638C7C, 0xFF5A8373, 0xFF5C8575, 0xFF5E8A7B, 0xFF5E8A7B, 0xFF5E8A7B, 0xFF5C8879, 0xFF5A8677,
        0xFF598576, 0xFF598576, 0xFF5A8677, 0xFF568573, 0xFF588775, 0xFF598876, 0xFF598876, 0xFF578674, 0xFF558472, 0xFF568573, 0xFF568573, 0xFF588473, 0xFF588473, 0xFF588473, 0xFF588473, 0xFF588473, 0xFF598574, 0xFF5A8675, 0xFF5A8675, 0xFF5A8574,
        0xFF5A8574, 0xFF5A8574, 0xFF5A8574, 0xFF5A8574, 0xFF5A8574, 0xFF5A8574, 0xFF5A8574, 0xFF568571, 0xFF578470, 0xFF588571, 0xFF5A8773, 0xFF5D8875, 0xFF5E8976, 0xFF5D8674, 0xFF5B8472, 0xFF5F8A79, 0xFF557E6E, 0xFF5F8878, 0xFF5E8777, 0xFF476D5E,
        0xFF446A5B, 0xFF4C7062, 0xFF3F6355, 0xFF2B4538, 0xFF385245, 0xFF2F493C, 0xFF4B6657, 0xFF355041, 0xFF2D4837, 0xFF587362, 0xFF5D7867, 0xFF344D3A, 0xFF314A37, 0xFF425B48, 0xFF4F6855, 0xFF4A6350, 0xFF253E2B, 0xFF3D5643, 0xFF607966, 0xFF4A6A5B,
        0xFF2B4A3B, 0xFF3C5649, 0xFF556D60, 0xFF4B6356, 0xFF476154, 0xFF446354, 0xFF69897A, 0xFF5D8A73, 0xFF5C7F6B, 0xFF495B4D, 0xFF696F65, 0xFF7E8178, 0xFF526053, 0xFF274634, 0xFF29533D, 0xFF587362, 0xFF4B6655, 0xFF264130, 0xFF385342, 0xFF425D4C,
        0xFF284332, 0xFF2B4635, 0xFF233E2D, 0xFF375C4B, 0xFF456A59, 0xFF3D6251, 0xFF3E6352, 0xFF5D8271, 0xFF678C7B, 0xFF5C8170, 0xFF5E8372, 0xFF618876, 0xFF618876, 0xFF618876, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF5F8674, 0xFF628976,
        0xFF618875, 0xFF618875, 0xFF618875, 0xFF618875, 0xFF608774, 0xFF608774, 0xFF608774, 0xFF608774, 0xFF608774, 0xFF608774, 0xFF608774, 0xFF608774, 0xFF608774, 0xFF608774, 0xFF608774, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673,
        0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF5C8774, 0xFF5B8472, 0xFF5F8876, 0xFF517A68, 0xFF456C5A, 0xFF426957, 0xFF416655, 0xFF4A6F5E, 0xFF608574, 0xFF678C7B, 0xFF5C8170, 0xFF678C7B, 0xFF507564, 0xFF244938, 0xFF2A4F3E, 0xFF335847, 0xFF2D5442,
        0xFF0F3624, 0xFF254A39, 0xFF284D3C, 0xFF2B4D3D, 0xFF1C3E2E, 0xFF375949, 0xFF5E7E6F, 0xFF769286, 0xFF476355, 0xFF658071, 0xFF536855, 0xFF757E6B, 0xFF86816E, 0xFF83705F, 0xFF8B705F, 0xFF4E5E51, 0xFF5E6E61, 0xFF6B776B, 0xFF91998E, 0xFFA8ABA2,
        0xFFA0A098, 0xFFA5A29B, 0xFFA49F99, 0xFF9B9E95, 0xFF99968F, 0xFFB7AEA9, 0xFFB1A6A2, 0xFFAAA59F, 0xFFA0A89D, 0xFF718677, 0xFF658472, 0xFF6C8476, 0xFF6C8778, 0xFF688476, 0xFF698577, 0xFF5A7467, 0x008B9C94, 0x00F8FFFD, 0x00F5FBF7, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x00FEFEFF, 0xFF648678, 0xFF50846E, 0xFF688778, 0xFF6C9082, 0xFF3A7060, 0xFF457066, 0xFF446E62, 0xFF497367, 0xFF467064, 0xFF4D776B, 0xFF497367,
        0xFF487266, 0xFF436D61, 0xFF568074, 0xFF638877, 0xFF385849, 0xFF3C5649, 0xFF51675B, 0xFF344C3F, 0xFF234233, 0xFF1B4230, 0xFF204B38, 0xFF20382B, 0xFF233D30, 0xFF294537, 0xFF2E4E41, 0xFF385A4C, 0xFF466C5F, 0xFF668F81, 0xFF5A8476, 0xFF61927F,
        0xFF538471, 0xFF5E8D7B, 0xFF608B7A, 0xFF5B8474, 0xFF648A7B, 0xFF64887A, 0xFF628476, 0xFF487161, 0xFF386151, 0xFF396252, 0xFF517C6B, 0xFF638E7D, 0xFF5F8A79, 0xFF598574, 0xFF5B8776, 0xFF598876, 0xFF598876, 0xFF598876, 0xFF588775, 0xFF588775,
        0xFF578674, 0xFF578674, 0xFF578674, 0xFF5B8778, 0xFF5B8778, 0xFF5A8677, 0xFF5A8677, 0xFF5A8677, 0xFF5A8677, 0xFF598576, 0xFF598576, 0xFF588473, 0xFF578372, 0xFF578372, 0xFF578372, 0xFF588473, 0xFF598574, 0xFF588473, 0xFF578372, 0xFF5A8574,
        0xFF5B8675, 0xFF5B8675, 0xFF5C8776, 0xFF5C8776, 0xFF5B8675, 0xFF5B8675, 0xFF5A8574, 0xFF558472, 0xFF558472, 0xFF598574, 0xFF5A8675, 0xFF5C8776, 0xFF5D8877, 0xFF5D8877, 0xFF5E8777, 0xFF558973, 0xFF5A8B76, 0xFF57826F, 0xFF5D8270, 0xFF436252,
        0xFF4E6958, 0xFF2E4334, 0xFF637567, 0xFF3D6A53, 0xFF576E5E, 0xFF7A7B73, 0xFF797F73, 0xFF223D2A, 0xFF365941, 0xFF3E5541, 0xFF585C4E, 0xFF627A6D, 0xFF69796F, 0xFF727972, 0xFF6B776D, 0xFF445A4D, 0xFF1C3927, 0xFF4A6151, 0xFF344437, 0xFF587B67,
        0xFF55705F, 0xFF5A685B, 0xFF7E847A, 0xFF7A8277, 0xFF384D3E, 0xFF577A66, 0xFF5E8B74, 0xFF618C79, 0xFF5F8876, 0xFF385849, 0xFF4D5A51, 0xFF767571, 0xFF6F706B, 0xFF506459, 0xFF5E8372, 0xFF3F4F42, 0xFF435A4A, 0xFF395443, 0xFF213529, 0xFF5F6B61,
        0xFF606B63, 0xFF4C6457, 0xFF3F6151, 0xFF6D7067, 0xFF3A4237, 0xFF546459, 0xFF395445, 0xFF608071, 0xFF597B6B, 0xFF5F8473, 0xFF5F8375, 0xFF5F8472, 0xFF5F8472, 0xFF5F8472, 0xFF608573, 0xFF608573, 0xFF608573, 0xFF608573, 0xFF618674, 0xFF628977,
        0xFF618876, 0xFF608775, 0xFF608775, 0xFF5F8674, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF618876, 0xFF608775, 0xFF5F8674, 0xFF5E8573, 0xFF5E8573, 0xFF5F8674, 0xFF608775, 0xFF618876, 0xFF628977, 0xFF628977, 0xFF618876, 0xFF608775, 0xFF5E8573,
        0xFF5E8573, 0xFF5F8674, 0xFF618876, 0xFF628C76, 0xFF59836D, 0xFF658B76, 0xFF496C58, 0xFF4A6957, 0xFF385543, 0xFF536E5D, 0xFF445C4C, 0xFF698A7F, 0xFF68867C, 0xFF638175, 0xFF658779, 0xFF4C7563, 0xFF385948, 0xFF636F65, 0xFF675F5C, 0xFF3B604F,
        0xFF19291F, 0xFF474642, 0xFF646962, 0xFF4D6859, 0xFF284936, 0xFF39493C, 0xFF58534D, 0xFF7A6E72, 0xFF657978, 0xFF548278, 0xFF395546, 0xFF82725B, 0xFFBC896C, 0xFFBC8765, 0xFFB9926B, 0xFF666C5E, 0xFF496453, 0xFF5D7C6D, 0xFF888E8A, 0xFFB6ADB0,
        0xFF989795, 0xFFA5A6A1, 0xFFAAA59F, 0xFFBDB7AB, 0xFFB5B6A8, 0xFFA6AA9C, 0xFFABABA1, 0xFFADA8A2, 0xFF9FA29B, 0xFF72887B, 0xFF5E8573, 0xFF628B79, 0xFF658777, 0xFF698879, 0xFF658779, 0xFF527668, 0x0082A096, 0x00F8FFFF, 0x00FFFBFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x00FEFEFF, 0xFF658779, 0xFF558973, 0xFF6B8A7B, 0xFF5D8173, 0xFF194F3F, 0xFF204B41, 0xFF1B4539, 0xFF184236, 0xFF1B4539, 0xFF1B4539, 0xFF0F392D,
        0xFF1A4438, 0xFF1E483C, 0xFF2E584C, 0xFF587768, 0xFF445C4F, 0xFF4D5D53, 0xFF869088, 0xFF9AA49C, 0xFF9FACA3, 0xFF98ACA1, 0xFF93ADA0, 0xFF92A399, 0xFF9EAFA5, 0xFF687E72, 0xFF354F44, 0xFF38564A, 0xFF426257, 0xFF64887C, 0xFF6C9084, 0xFF618D7C,
        0xFF5E8A79, 0xFF618D7C, 0xFF669180, 0xFF5F8878, 0xFF5C8575, 0xFF587E6F, 0xFF43695A, 0xFF3B6454, 0xFF3F6858, 0xFF4D7666, 0xFF5C8776, 0xFF618C7B, 0xFF5A8675, 0xFF598574, 0xFF5E8A79, 0xFF598876, 0xFF598876, 0xFF598876, 0xFF588775, 0xFF588775,
        0xFF578674, 0xFF578674, 0xFF578674, 0xFF5B8778, 0xFF5A8677, 0xFF5A8677, 0xFF5A8677, 0xFF5A8677, 0xFF598576, 0xFF598576, 0xFF598576, 0xFF598574, 0xFF588473, 0xFF588473, 0xFF588473, 0xFF598574, 0xFF5A8675, 0xFF598574, 0xFF588473, 0xFF5A8574,
        0xFF5A8574, 0xFF5B8675, 0xFF5B8675, 0xFF5B8675, 0xFF5B8675, 0xFF5A8574, 0xFF5A8574, 0xFF548371, 0xFF558472, 0xFF568573, 0xFF5A8675, 0xFF5B8776, 0xFF5C8776, 0xFF5C8776, 0xFF5C8776, 0xFF4F836D, 0xFF568772, 0xFF6A9582, 0xFF5D8471, 0xFF3A5B4A,
        0xFF577263, 0xFF5B7262, 0xFF485D4E, 0xFF27503E, 0xFF5D7165, 0xFF6F726B, 0xFF727971, 0xFF264130, 0xFF355B46, 0xFF55705D, 0xFF5D695B, 0xFF2F4E3F, 0xFF48594F, 0xFF535A53, 0xFF525C53, 0xFF465A4E, 0xFF274334, 0xFF4E6958, 0xFF536859, 0xFF436554,
        0xFF365142, 0xFF4F5F54, 0xFF686F67, 0xFF677168, 0xFF3C5044, 0xFF4C6E5D, 0xFF63907B, 0xFF598471, 0xFF648D7B, 0xFF466657, 0xFF55655B, 0xFF7A7B76, 0xFF6D6F6A, 0xFF3B4F44, 0xFF305544, 0xFF607767, 0xFF4B6856, 0xFF325141, 0xFF2D4336, 0xFF636F65,
        0xFF5C665E, 0xFF51655A, 0xFF2B4B3C, 0xFF526A5A, 0xFF546F5E, 0xFF547564, 0xFF385D4B, 0xFF5F8674, 0xFF668D7B, 0xFF608574, 0xFF5D7F71, 0xFF608573, 0xFF608573, 0xFF608573, 0xFF608573, 0xFF608573, 0xFF618674, 0xFF618674, 0xFF618674, 0xFF618876,
        0xFF618876, 0xFF608775, 0xFF608775, 0xFF5F8674, 0xFF5F8674, 0xFF608775, 0xFF608775, 0xFF618876, 0xFF608775, 0xFF5F8674, 0xFF5E8573, 0xFF5E8573, 0xFF5F8674, 0xFF608775, 0xFF618876, 0xFF608775, 0xFF618876, 0xFF618876, 0xFF608775, 0xFF5F8674,
        0xFF5F8674, 0xFF608775, 0xFF618876, 0xFF618B75, 0xFF5F8973, 0xFF628974, 0xFF476A56, 0xFF456653, 0xFF43604E, 0xFF5C7766, 0xFF496453, 0xFF466F5F, 0xFF628678, 0xFF66887A, 0xFF608977, 0xFF517E6A, 0xFF375E4C, 0xFF57695D, 0xFF656660, 0xFF395B4B,
        0xFF3A4A40, 0xFF8D8C88, 0xFFABB0A9, 0xFF728A7C, 0xFF224131, 0xFF586659, 0xFFAEA9A3, 0xFF9F968D, 0xFF40564A, 0xFF245341, 0xFF476951, 0xFF928D70, 0xFFBC9876, 0xFFCBA882, 0xFFB09872, 0xFF6C7564, 0xFF395640, 0xFF385747, 0xFF616B63, 0xFFA29C9C,
        0xFF959490, 0xFFA6A9A0, 0xFFB0ADA4, 0xFFC5BFB3, 0xFFB3B4A6, 0xFFA3A799, 0xFF9D9D93, 0xFF99948E, 0xFF979A93, 0xFF748A7E, 0xFF638A78, 0xFF618A78, 0xFF658777, 0xFF698879, 0xFF66887A, 0xFF517567, 0x00809E94, 0x00F8FFFF, 0x00FFFCFE, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFBFF, 0x00F9F9FB, 0xFF5C7E70, 0xFF528670, 0xFF719081, 0xFF5D8173, 0xFF154B3B, 0xFF275248, 0xFF497367, 0xFF396357, 0xFF244E42, 0xFF295347, 0xFF3F695D,
        0xFF366054, 0xFF123C30, 0xFF366054, 0xFF5F776A, 0xFF4D5E54, 0xFF636A63, 0xFFC9C8C4, 0xFFF2EDEA, 0xFFE5E1DE, 0xFFD9DAD5, 0xFFDEE3DD, 0xFFECF3EC, 0xFFEDF7EF, 0xFF8C978F, 0xFF3E4D46, 0xFF4C6057, 0xFF4C665D, 0xFF567168, 0xFF516F65, 0xFF47675A,
        0xFF49695C, 0xFF406254, 0xFF47695B, 0xFF3E6254, 0xFF3F6556, 0xFF4F7566, 0xFF4C7263, 0xFF3C6555, 0xFF4E7767, 0xFF5E8978, 0xFF628D7C, 0xFF5B8776, 0xFF588473, 0xFF5A8675, 0xFF5C8B79, 0xFF598876, 0xFF598876, 0xFF588775, 0xFF588775, 0xFF588775,
        0xFF588775, 0xFF578674, 0xFF578674, 0xFF5A8677, 0xFF5A8677, 0xFF5A8677, 0xFF5A8677, 0xFF5A8677, 0xFF598576, 0xFF598576, 0xFF598576, 0xFF5A8675, 0xFF598574, 0xFF588473, 0xFF598574, 0xFF5A8675, 0xFF5A8675, 0xFF598574, 0xFF588473, 0xFF5A8574,
        0xFF5A8574, 0xFF5B8675, 0xFF5B8675, 0xFF5B8675, 0xFF5B8675, 0xFF5A8574, 0xFF5A8574, 0xFF548371, 0xFF558472, 0xFF568573, 0xFF598574, 0xFF5A8675, 0xFF5A8675, 0xFF5B8675, 0xFF5B8675, 0xFF598A77, 0xFF578875, 0xFF517E6A, 0xFF5F8674, 0xFF6A8C7B,
        0xFF334F40, 0xFF233B2D, 0xFF4C6454, 0xFF3B5F53, 0xFF3A4B43, 0xFF484D49, 0xFF49544E, 0xFF2D4C3D, 0xFF58816F, 0xFF476958, 0xFF4F6558, 0xFF597F70, 0xFF354C42, 0xFF444B44, 0xFF515650, 0xFF415146, 0xFF3D594A, 0xFF618271, 0xFF436250, 0xFF4C6E60,
        0xFF526E62, 0xFF3C4D45, 0xFF49544E, 0xFF48554E, 0xFF2A3F36, 0xFF5E8072, 0xFF5A8574, 0xFF618A78, 0xFF699280, 0xFF466858, 0xFF3D5146, 0xFF4F544E, 0xFF474E47, 0xFF3D5347, 0xFF517866, 0xFF436250, 0xFF3B5D4C, 0xFF5D8270, 0xFF375243, 0xFF47544B,
        0xFF4A544C, 0xFF374840, 0xFF435F53, 0xFF3B654F, 0xFF1F4B34, 0xFF2D5A45, 0xFF53836D, 0xFF5A8773, 0xFF547D6B, 0xFF608574, 0xFF638577, 0xFF5F8673, 0xFF5F8673, 0xFF5F8673, 0xFF608774, 0xFF608774, 0xFF608774, 0xFF608774, 0xFF608774, 0xFF618876,
        0xFF618876, 0xFF608775, 0xFF5F8674, 0xFF5F8674, 0xFF5F8674, 0xFF608775, 0xFF608775, 0xFF618876, 0xFF608775, 0xFF5F8674, 0xFF5E8573, 0xFF5E8573, 0xFF5F8674, 0xFF608775, 0xFF618876, 0xFF5E8573, 0xFF608775, 0xFF618876, 0xFF618876, 0xFF608775,
        0xFF5F8674, 0xFF608775, 0xFF618876, 0xFF5C8672, 0xFF5D8773, 0xFF658C79, 0xFF5D8270, 0xFF4D6F5E, 0xFF406150, 0xFF537262, 0xFF587465, 0xFF366251, 0xFF53796A, 0xFF5F8473, 0xFF648D7B, 0xFF558470, 0xFF345D4B, 0xFF40584B, 0xFF495049, 0xFF2A483C,
        0xFF29362F, 0xFF63645F, 0xFF6F746E, 0xFF485C50, 0xFF11291B, 0xFF616D63, 0xFFB1ACA6, 0xFF9B9A85, 0xFF38513B, 0xFF4A7A60, 0xFF476C4D, 0xFF7F805E, 0xFFC3AC83, 0xFFC0A97F, 0xFFC6BA90, 0xFF7B8471, 0xFF3C5A42, 0xFF2A4B36, 0xFF5C665D, 0xFFAEA9A5,
        0xFFA7A6A1, 0xFFB2B5AA, 0xFFBDBBAF, 0xFFC7C1B5, 0xFFBABAAE, 0xFFC2C6B8, 0xFFBEBEB4, 0xFFABA6A0, 0xFFA5A8A1, 0xFF768C80, 0xFF577E6C, 0xFF608977, 0xFF658777, 0xFF698879, 0xFF67897B, 0xFF517567, 0x007D9B91, 0x00F8FFFF, 0x00FFFCFE, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x00FCFCFE, 0xFF587A6C, 0xFF4E826C, 0xFF729182, 0xFF5C8072, 0xFF1A5040, 0xFF3B665C, 0xFF4E786C, 0xFF436D61, 0xFF325C50, 0xFF3F695D, 0xFF608A7E,
        0xFF497367, 0xFF0D372B, 0xFF366054, 0xFF628172, 0xFF4A6054, 0xFF707A72, 0xFFDEDDD9, 0xFFEDE5E3, 0xFFD1C7C6, 0xFFC3BEBB, 0xFFD3CFCC, 0xFFD9D5D2, 0xFFEBEAE6, 0xFFA8AAA5, 0xFF5B645F, 0xFF505D56, 0xFF344741, 0xFF374E46, 0xFF455F56, 0xFF637069,
        0xFF47564F, 0xFF2C3D35, 0xFF33473E, 0xFF3A5147, 0xFF445E53, 0xFF476357, 0xFF486458, 0xFF537E6D, 0xFF5B8675, 0xFF608B7A, 0xFF5D8978, 0xFF598574, 0xFF5A8675, 0xFF598876, 0xFF598876, 0xFF588775, 0xFF588775, 0xFF588775, 0xFF588775, 0xFF588775,
        0xFF588775, 0xFF588775, 0xFF588775, 0xFF588777, 0xFF588777, 0xFF588777, 0xFF588777, 0xFF578676, 0xFF578676, 0xFF578676, 0xFF578676, 0xFF578674, 0xFF568573, 0xFF558472, 0xFF568573, 0xFF578674, 0xFF588775, 0xFF578674, 0xFF568573, 0xFF588473,
        0xFF598574, 0xFF598574, 0xFF598574, 0xFF598574, 0xFF598574, 0xFF598574, 0xFF588473, 0xFF538471, 0xFF548572, 0xFF568573, 0xFF578674, 0xFF5A8675, 0xFF5A8675, 0xFF5A8675, 0xFF5B8675, 0xFF598A77, 0xFF4E7D6B, 0xFF659180, 0xFF668F7D, 0xFF446958,
        0xFF436453, 0xFF516D5E, 0xFF3C5748, 0xFF2A4941, 0xFF364945, 0xFF313A39, 0xFF3F4E49, 0xFF38584D, 0xFF598473, 0xFF5F8878, 0xFF5F8173, 0xFF4D7C6A, 0xFF345246, 0xFF414A45, 0xFF474C46, 0xFF435349, 0xFF426151, 0xFF5C816F, 0xFF5C8370, 0xFF5E8276,
        0xFF4D6B61, 0xFF3F544D, 0xFF3A4B45, 0xFF3A4B45, 0xFF324941, 0xFF517166, 0xFF5C8275, 0xFF5E8372, 0xFF587F6D, 0xFF385A4A, 0xFF385043, 0xFF47524A, 0xFF364139, 0xFF395144, 0xFF5D8472, 0xFF5E8371, 0xFF638D79, 0xFF507A66, 0xFF2C4B3C, 0xFF4A5A50,
        0xFF48514C, 0xFF3F4E47, 0xFF304A3F, 0xFF426350, 0xFF3A5D49, 0xFF3B624F, 0xFF48735F, 0xFF628D7A, 0xFF5C8573, 0xFF5C8371, 0xFF64887A, 0xFF5F8673, 0xFF608774, 0xFF608774, 0xFF608774, 0xFF608774, 0xFF618875, 0xFF618875, 0xFF618875, 0xFF618876,
        0xFF608775, 0xFF608775, 0xFF5F8674, 0xFF5F8674, 0xFF5F8674, 0xFF5F8674, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF5F8674, 0xFF5F8674, 0xFF5F8674, 0xFF5F8674, 0xFF608775, 0xFF608775, 0xFF5D8472, 0xFF5E8573, 0xFF608775, 0xFF618876, 0xFF608775,
        0xFF608775, 0xFF608775, 0xFF618876, 0xFF628C78, 0xFF5B8571, 0xFF5A816E, 0xFF698E7C, 0xFF638876, 0xFF597B6A, 0xFF517362, 0xFF476857, 0xFF59806E, 0xFF486859, 0xFF435F51, 0xFF4D6D5E, 0xFF426957, 0xFF345648, 0xFF42564D, 0xFF464C48, 0xFF314D41,
        0xFF25342D, 0xFF4F5450, 0xFF596059, 0xFF43544A, 0xFF152B1E, 0xFF5D695F, 0xFF7D7C77, 0xFF828A73, 0xFF5F7B64, 0xFF507A62, 0xFF284728, 0xFF787650, 0xFFC7AC81, 0xFFC6AA7B, 0xFFC6B484, 0xFF656E59, 0xFF335137, 0xFF2A4C34, 0xFF606A5F, 0xFFACA7A3,
        0xFFA2A39B, 0xFFB2B6A8, 0xFFCBC9BC, 0xFFC4BDB3, 0xFFB8B8AC, 0xFFCCCFC4, 0xFFC1C1B9, 0xFFA29D99, 0xFFA2A49F, 0xFF788E82, 0xFF5B8172, 0xFF5F8876, 0xFF668878, 0xFF698879, 0xFF688A7C, 0xFF507466, 0x0079978D, 0x00F8FFFF, 0x00FFFDFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x00FCFCFE, 0xFF5A7C6E, 0xFF4F836D, 0xFF719081, 0xFF597D6F, 0xFF174D3D, 0xFF3C675D, 0xFF2E584C, 0xFF2F594D, 0xFF426C60, 0xFF4A7468, 0xFF477165,
        0xFF446E62, 0xFF234D41, 0xFF345E52, 0xFF678E7C, 0xFF517061, 0xFF819288, 0xFFDDE2DC, 0xFFD7D6D2, 0xFFD2CECB, 0xFFCBCAC6, 0xFFCFD0CB, 0xFFDBD3D1, 0xFFE5E0DD, 0xFFB1B2AD, 0xFF838884, 0xFF828F88, 0xFF596A64, 0xFF516860, 0xFF738D84, 0xFF90908E,
        0xFF70726F, 0xFF5C615D, 0xFF616763, 0xFF6E7772, 0xFF7D8A83, 0xFF57665F, 0xFF4C5B54, 0xFF648F7E, 0xFF5F8A79, 0xFF5A8675, 0xFF598574, 0xFF5B8776, 0xFF5A8977, 0xFF598876, 0xFF598876, 0xFF588775, 0xFF588775, 0xFF588775, 0xFF588775, 0xFF588775,
        0xFF588775, 0xFF588775, 0xFF588775, 0xFF588777, 0xFF588777, 0xFF588777, 0xFF578676, 0xFF578676, 0xFF578676, 0xFF578676, 0xFF568575, 0xFF568573, 0xFF558472, 0xFF548371, 0xFF548371, 0xFF568573, 0xFF568573, 0xFF558472, 0xFF548371, 0xFF588473,
        0xFF588473, 0xFF588473, 0xFF598574, 0xFF598574, 0xFF588473, 0xFF588473, 0xFF588473, 0xFF548572, 0xFF548572, 0xFF558673, 0xFF578674, 0xFF578674, 0xFF5A8675, 0xFF598574, 0xFF598574, 0xFF558575, 0xFF598878, 0xFF578372, 0xFF547F6E, 0xFF3D6452,
        0xFF446656, 0xFF375847, 0xFF4F705F, 0xFF33524C, 0xFF778D8A, 0xFF8E9E9B, 0xFF6D827D, 0xFF315247, 0xFF315B4D, 0xFF386455, 0xFF3E6757, 0xFF2D604F, 0xFF507065, 0xFF98A59E, 0xFFA0A9A4, 0xFF6D8176, 0xFF3B5D4D, 0xFF355F4B, 0xFF396450, 0xFF42685B,
        0xFF395A4F, 0xFF567067, 0xFF859A93, 0xFF80958E, 0xFF476158, 0xFF39594E, 0xFF416559, 0xFF456556, 0xFF3D6251, 0xFF375C4B, 0xFF5F7B6D, 0xFF86968C, 0xFF637369, 0xFF395547, 0xFF335C4A, 0xFF345B48, 0xFF386550, 0xFF2D5A46, 0xFF4A6A5B, 0xFF95A69E,
        0xFF929D97, 0xFF6A7D77, 0xFF28433A, 0xFF596559, 0xFF4D5D50, 0xFF536B5D, 0xFF375646, 0xFF5A7F6E, 0xFF618876, 0xFF658C7A, 0xFF5E8475, 0xFF5D8773, 0xFF5E8874, 0xFF5E8874, 0xFF5E8874, 0xFF5E8874, 0xFF5F8975, 0xFF5F8975, 0xFF5F8975, 0xFF618876,
        0xFF608775, 0xFF5F8674, 0xFF5F8674, 0xFF5F8674, 0xFF5F8674, 0xFF5F8674, 0xFF5F8674, 0xFF608775, 0xFF608775, 0xFF5F8674, 0xFF5F8674, 0xFF5F8674, 0xFF5F8674, 0xFF608775, 0xFF608775, 0xFF5C8371, 0xFF5E8573, 0xFF608775, 0xFF618876, 0xFF608775,
        0xFF5F8674, 0xFF608775, 0xFF608775, 0xFF5C8573, 0xFF638C7A, 0xFF5E8573, 0xFF628977, 0xFF638A78, 0xFF6A917F, 0xFF5D8271, 0xFF406554, 0xFF4D6C5A, 0xFF5B7164, 0xFF5D6D62, 0xFF4D5E54, 0xFF3C564B, 0xFF536D62, 0xFF7A8682, 0xFF878988, 0xFF4F6960,
        0xFF374A44, 0xFF66736C, 0xFF808D86, 0xFF5F7368, 0xFF2C4035, 0xFF7A877D, 0xFF9FA49D, 0xFF768372, 0xFF536D60, 0xFF3E6052, 0xFF213420, 0xFF807555, 0xFFCFAE83, 0xFFCBA679, 0xFFCDAD7E, 0xFF636F59, 0xFF315139, 0xFF254631, 0xFF59655B, 0xFFA4A19C,
        0xFFA2A39B, 0xFFB1B7AB, 0xFFCACABE, 0xFFC1BAB0, 0xFFB9B9AF, 0xFFCED1C6, 0xFFBEBEB6, 0xFF999490, 0xFF999B96, 0xFF758A81, 0xFF5D8374, 0xFF5F8876, 0xFF678979, 0xFF688778, 0xFF6A8C7E, 0xFF517567, 0x0077958B, 0x00F8FFFF, 0x00FFFDFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFBFF, 0x00FCFCFE, 0xFF5B7D6F, 0xFF4F836D, 0xFF708F80, 0xFF5B7F71, 0xFF194F3F, 0xFF3A655B, 0xFF467064, 0xFF3E685C, 0xFF3D675B, 0xFF426C60, 0xFF477165,
        0xFF436D61, 0xFF234D41, 0xFF345E52, 0xFF567F6D, 0xFF4A6A5B, 0xFF72867B, 0xFFD1DBD3, 0xFFD5D7D2, 0xFFD6D7D2, 0xFFCED0CB, 0xFFC8CFC8, 0xFFDAD5D2, 0xFFECE8E5, 0xFFB1B3AE, 0xFF929B96, 0xFF8C9B94, 0xFF415850, 0xFF405B52, 0xFF75958A, 0xFF979D99,
        0xFF717773, 0xFF595F5B, 0xFF616A65, 0xFF8E9993, 0xFFB7C2BC, 0xFF74817A, 0xFF4C5952, 0xFF638E7D, 0xFF5B8776, 0xFF588473, 0xFF5B8776, 0xFF5C8B79, 0xFF5A8977, 0xFF588976, 0xFF5A8B78, 0xFF578674, 0xFF578674, 0xFF588775, 0xFF588775, 0xFF588775,
        0xFF588775, 0xFF598876, 0xFF598876, 0xFF578777, 0xFF568676, 0xFF568676, 0xFF568676, 0xFF568676, 0xFF558575, 0xFF558575, 0xFF558575, 0xFF548572, 0xFF538471, 0xFF528370, 0xFF538471, 0xFF548572, 0xFF548572, 0xFF548572, 0xFF528370, 0xFF558472,
        0xFF558472, 0xFF568573, 0xFF568573, 0xFF568573, 0xFF568573, 0xFF558472, 0xFF558472, 0xFF538673, 0xFF538673, 0xFF568774, 0xFF568774, 0xFF588775, 0xFF588775, 0xFF578674, 0xFF598574, 0xFF558575, 0xFF5B8A7A, 0xFF5B8A7A, 0xFF5A8574, 0xFF3B6454,
        0xFF4B7260, 0xFF395E4D, 0xFF4A6F5E, 0xFF486C62, 0xFF67867E, 0xFF8BA69F, 0xFF6A857C, 0xFF46675C, 0xFF4E7767, 0xFF3C6756, 0xFF477261, 0xFF417463, 0xFF496A5F, 0xFF859892, 0xFF91A29A, 0xFF5D796D, 0xFF456C5A, 0xFF4D7865, 0xFF466F5D, 0xFF466C5D,
        0xFF517567, 0xFF608073, 0xFF9BB7AB, 0xFF93ADA2, 0xFF547064, 0xFF5A786C, 0xFF57776A, 0xFF517061, 0xFF507564, 0xFF4B7260, 0xFF618172, 0xFF849A8E, 0xFF6F8378, 0xFF527162, 0xFF466F5D, 0xFF507765, 0xFF406B58, 0xFF45725E, 0xFF47695B, 0xFF556A61,
        0xFF566761, 0xFF587269, 0xFF4D7165, 0xFF70736A, 0xFF414B40, 0xFF586A5E, 0xFF4C6859, 0xFF618373, 0xFF59806E, 0xFF638C7A, 0xFF598272, 0xFF5D8773, 0xFF5D8773, 0xFF5D8773, 0xFF5E8874, 0xFF5E8874, 0xFF5E8874, 0xFF5E8874, 0xFF5E8874, 0xFF608775,
        0xFF608775, 0xFF5F8674, 0xFF5F8674, 0xFF5E8573, 0xFF5E8573, 0xFF5F8674, 0xFF5F8674, 0xFF608775, 0xFF608775, 0xFF5F8674, 0xFF5F8674, 0xFF5F8674, 0xFF5F8674, 0xFF608775, 0xFF608775, 0xFF5D8472, 0xFF5F8674, 0xFF608775, 0xFF608775, 0xFF5F8674,
        0xFF5F8674, 0xFF5F8674, 0xFF608775, 0xFF5B8270, 0xFF638C7A, 0xFF5F8876, 0xFF5D8674, 0xFF5B8472, 0xFF5F8876, 0xFF628B79, 0xFF58816F, 0xFF42624D, 0xFF485D4E, 0xFF556157, 0xFF69766D, 0xFF61766D, 0xFF4F665E, 0xFF5E6D6A, 0xFF989C9D, 0xFF85A097,
        0xFF627C73, 0xFF859A93, 0xFFAEC3BA, 0xFF97ACA3, 0xFF5C7266, 0xFF6D7E74, 0xFF616E65, 0xFF5F7163, 0xFF324C43, 0xFF5A756E, 0xFF526053, 0xFF847D63, 0xFFC5AB88, 0xFFC8AA84, 0xFFCBB18C, 0xFF5B6655, 0xFF4A6A53, 0xFF507261, 0xFF6D7870, 0xFF878382,
        0xFF747671, 0xFF91978D, 0xFFBABAB0, 0xFFB2ABA3, 0xFFACACA2, 0xFFB4B7AE, 0xFFA3A29D, 0xFF87827F, 0xFF898B88, 0xFF6E837A, 0xFF5F8578, 0xFF608977, 0xFF688A7A, 0xFF678677, 0xFF6B8D7F, 0xFF527668, 0x0076948A, 0x00F8FFFF, 0x00FFFDFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x00FFFFFF, 0xFF5D7F71, 0xFF4E826C, 0xFF6E8D7E, 0xFF5D8173, 0xFF174D3D, 0xFF2B564C, 0xFF568074, 0xFF477165, 0xFF265044, 0xFF305A4E, 0xFF4B7569,
        0xFF3B6559, 0xFF164034, 0xFF376155, 0xFF5A796A, 0xFF71897C, 0xFF7D8880, 0xFFDBDDD8, 0xFFEDE9E6, 0xFFDCDBD7, 0xFFD1D3CE, 0xFFD0D5CF, 0xFFE4E3DF, 0xFFE7E8E3, 0xFF939A93, 0xFF96A59E, 0xFF9CB1A8, 0xFF415F55, 0xFF43675B, 0xFF4F7568, 0xFF476357,
        0xFF4C685C, 0xFF496559, 0xFF496358, 0xFF4B655A, 0xFF637A70, 0xFF4A6157, 0xFF556A61, 0xFF5B8776, 0xFF598574, 0xFF5A8675, 0xFF5A8977, 0xFF5B8A78, 0xFF588976, 0xFF588976, 0xFF598A77, 0xFF578674, 0xFF578674, 0xFF578674, 0xFF588775, 0xFF588775,
        0xFF598876, 0xFF598876, 0xFF598876, 0xFF568676, 0xFF568676, 0xFF568676, 0xFF568676, 0xFF568676, 0xFF558575, 0xFF558575, 0xFF558575, 0xFF558673, 0xFF548572, 0xFF538471, 0xFF538471, 0xFF558673, 0xFF558673, 0xFF548572, 0xFF538471, 0xFF558472,
        0xFF558472, 0xFF568573, 0xFF568573, 0xFF568573, 0xFF568573, 0xFF558472, 0xFF558472, 0xFF538673, 0xFF548774, 0xFF558875, 0xFF578875, 0xFF578875, 0xFF588775, 0xFF588775, 0xFF578674, 0xFF598979, 0xFF548373, 0xFF568575, 0xFF5D897A, 0xFF4B7665,
        0xFF3D6656, 0xFF375E4C, 0xFF3C6351, 0xFF325C4E, 0xFF224B3B, 0xFF092F20, 0xFF1C3E30, 0xFF325444, 0xFF3E6352, 0xFF47715D, 0xFF3A6551, 0xFF3A6959, 0xFF2D5145, 0xFF223931, 0xFF1D372E, 0xFF2B4F41, 0xFF34604F, 0xFF386350, 0xFF456C5A, 0xFF456F5B,
        0xFF335A47, 0xFF2F5140, 0xFF224131, 0xFF1F3E2E, 0xFF345041, 0xFF4B6A5A, 0xFF658474, 0xFF496557, 0xFF496B5B, 0xFF2D5644, 0xFF183A2A, 0xFF203A2D, 0xFF20382B, 0xFF305041, 0xFF3E6956, 0xFF3C5E4E, 0xFF446D5B, 0xFF3C6756, 0xFF3C6052, 0xFF284239,
        0xFF0D241C, 0xFF3B5B50, 0xFF3D695A, 0xFF455A4B, 0xFF465E4E, 0xFF446353, 0xFF416654, 0xFF628B79, 0xFF5C8573, 0xFF567F6D, 0xFF5C8273, 0xFF5D8773, 0xFF5D8773, 0xFF5D8773, 0xFF5D8773, 0xFF5D8773, 0xFF5E8874, 0xFF5E8874, 0xFF5E8874, 0xFF608775,
        0xFF608775, 0xFF5F8674, 0xFF5E8573, 0xFF5E8573, 0xFF5E8573, 0xFF5F8674, 0xFF5F8674, 0xFF5F8674, 0xFF5F8674, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF5F8674, 0xFF5F8674, 0xFF5F8674, 0xFF608775, 0xFF608775, 0xFF5F8674, 0xFF5E8573,
        0xFF5D8472, 0xFF5E8573, 0xFF608775, 0xFF678E7C, 0xFF5A8371, 0xFF537C6A, 0xFF5A8371, 0xFF648F7C, 0xFF5C8774, 0xFF608D79, 0xFF5E8B77, 0xFF3F6951, 0xFF4F6F5A, 0xFF4B6051, 0xFF40564A, 0xFF466458, 0xFF517267, 0xFF39544D, 0xFF354846, 0xFF68887D,
        0xFF37574C, 0xFF224036, 0xFF314C43, 0xFF365045, 0xFF364D43, 0xFF3A5245, 0xFF1C3226, 0xFF50634D, 0xFF435B4D, 0xFF5A756E, 0xFF6C8077, 0xFF7D8170, 0xFF88836D, 0xFFA9A48E, 0xFF8A8977, 0xFF788476, 0xFF5F806D, 0xFF5F8173, 0xFF7E8A86, 0xFF9A989B,
        0xFF898B8A, 0xFF878D89, 0xFF8A8B85, 0xFF989189, 0xFFA0A096, 0xFFAAADA4, 0xFFADACA7, 0xFFA5A09D, 0xFF9FA19E, 0xFF778C85, 0xFF62887B, 0xFF628B79, 0xFF698B7B, 0xFF668576, 0xFF6C8E80, 0xFF54786A, 0x0076948A, 0x00F8FFFF, 0x00FFFCFE, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FBFAFF, 0x00F9F9FB, 0xFF5A7C6E, 0xFF4D816B, 0xFF6F8E7F, 0xFF618577, 0xFF154B3B, 0xFF1D483E, 0xFF1B4539, 0xFF1C463A, 0xFF113B2F, 0xFF194337, 0xFF204A3E,
        0xFF1C463A, 0xFF164034, 0xFF345E52, 0xFF63776C, 0xFFADB8B0, 0xFF999B96, 0xFFDBD6D3, 0xFFF3E9E8, 0xFFE4DCDA, 0xFFE2DEDB, 0xFFE1E2DD, 0xFFE7E8E3, 0xFFF3F8F2, 0xFFA6B1A9, 0xFFBDD1C8, 0xFFA7C3B7, 0xFF436459, 0xFF668F81, 0xFF558172, 0xFF598A77,
        0xFF608F7D, 0xFF598574, 0xFF65907F, 0xFF5E8777, 0xFF5F8375, 0xFF517365, 0xFF688A7C, 0xFF5B8776, 0xFF5B8776, 0xFF598876, 0xFF578674, 0xFF578674, 0xFF578875, 0xFF578875, 0xFF568774, 0xFF578674, 0xFF578674, 0xFF578674, 0xFF588775, 0xFF588775,
        0xFF598876, 0xFF598876, 0xFF598876, 0xFF568676, 0xFF568676, 0xFF568676, 0xFF568676, 0xFF558575, 0xFF558575, 0xFF558575, 0xFF558575, 0xFF568774, 0xFF558673, 0xFF548572, 0xFF548572, 0xFF568774, 0xFF568774, 0xFF558673, 0xFF548572, 0xFF548371,
        0xFF558472, 0xFF558472, 0xFF568573, 0xFF568573, 0xFF558472, 0xFF558472, 0xFF548371, 0xFF548774, 0xFF548774, 0xFF558875, 0xFF588976, 0xFF588976, 0xFF588775, 0xFF588775, 0xFF578674, 0xFF538375, 0xFF578676, 0xFF528171, 0xFF537F70, 0xFF679281,
        0xFF4C7766, 0xFF58816F, 0xFF58816F, 0xFF4E7F6C, 0xFF3A6955, 0xFF3F6A57, 0xFF406755, 0xFF507261, 0xFF668877, 0xFF618873, 0xFF618B75, 0xFF5C887B, 0xFF4C6D62, 0xFF4A655C, 0xFF4A6A5F, 0xFF446F5E, 0xFF51826F, 0xFF628D7A, 0xFF648676, 0xFF618B75,
        0xFF5C836E, 0xFF406651, 0xFF40634F, 0xFF42614F, 0xFF4B6A58, 0xFF698674, 0xFF63806E, 0xFF6C8679, 0xFF688A7A, 0xFF4E7765, 0xFF385D4C, 0xFF446052, 0xFF455D50, 0xFF537364, 0xFF5B8673, 0xFF698879, 0xFF5F8473, 0xFF5C8575, 0xFF486C5E, 0xFF4D685F,
        0xFF4B655C, 0xFF456B5E, 0xFF4B7E6F, 0xFF5B856F, 0xFF537F68, 0xFF45755F, 0xFF5D8E78, 0xFF598874, 0xFF588571, 0xFF608977, 0xFF5E8475, 0xFF5B8672, 0xFF5B8672, 0xFF5B8672, 0xFF5C8773, 0xFF5C8773, 0xFF5C8773, 0xFF5C8773, 0xFF5D8874, 0xFF608775,
        0xFF608775, 0xFF5F8674, 0xFF5E8573, 0xFF5E8573, 0xFF5E8573, 0xFF5E8573, 0xFF5F8674, 0xFF5F8674, 0xFF5F8674, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF5F8674, 0xFF5F8674, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF5F8674, 0xFF5D8472,
        0xFF5D8472, 0xFF5E8573, 0xFF608775, 0xFF5D8374, 0xFF5C8575, 0xFF608979, 0xFF5A8574, 0xFF648F7E, 0xFF578372, 0xFF608C7B, 0xFF598876, 0xFF538A6B, 0xFF4E7860, 0xFF5A7967, 0xFF628372, 0xFF4F7868, 0xFF437164, 0xFF335D53, 0xFF42635C, 0xFF44655C,
        0xFF3B5F53, 0xFF34584C, 0xFF3B5C51, 0xFF345246, 0xFF3B554A, 0xFF425C4F, 0xFF3B5749, 0xFF576A4A, 0xFF465F49, 0xFF557367, 0xFF4F6A61, 0xFF54685C, 0xFF485A4A, 0xFF4F6559, 0xFF4D6863, 0xFF5C685C, 0xFF547465, 0xFF65867B, 0xFF859191, 0xFF97959A,
        0xFF848587, 0xFF848A88, 0xFF838381, 0xFFA49D95, 0xFFA9A9A1, 0xFFA1A49B, 0xFFA2A19C, 0xFFA49F9C, 0xFF9C9E9B, 0xFF738881, 0xFF5F8578, 0xFF638C7A, 0xFF6A8C7C, 0xFF658475, 0xFF6C8E80, 0xFF55796B, 0x0076948A, 0x00F8FFFF, 0x00FFFCFE, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFF8FB, 0x04F4FFFC, 0xFF5A7E72, 0xFF548373, 0xFF638F80, 0xFF587E71, 0xFF386153, 0xFF305C4D, 0xFF345B49, 0xFF2F5644, 0xFF2E5543, 0xFF315846, 0xFF345B49,
        0xFF335A48, 0xFF345B49, 0xFF3E6553, 0xFF527A6F, 0xFF5E867B, 0xFF6C9489, 0xFF88B0A5, 0xFF82AA9F, 0xFF6A9287, 0xFF78A095, 0xFF83ABA0, 0xFF638B80, 0xFF7CA499, 0xFF9BC3B8, 0xFF9CC4B9, 0xFF86AEA3, 0xFF497166, 0xFF5A8277, 0xFF60887D, 0xFF5F8B7C,
        0xFF5F8B7C, 0xFF5F8B7C, 0xFF5E8A7B, 0xFF5E8A7B, 0xFF5E8A7B, 0xFF5E8A7B, 0xFF5D897A, 0xFF598878, 0xFF598878, 0xFF598878, 0xFF588878, 0xFF588878, 0xFF588878, 0xFF568978, 0xFF568978, 0xFF558575, 0xFF568676, 0xFF578777, 0xFF578777, 0xFF578777,
        0xFF578777, 0xFF568676, 0xFF558575, 0xFF588475, 0xFF598878, 0xFF558474, 0xFF528272, 0xFF558575, 0xFF538675, 0xFF528775, 0xFF528775, 0xFF518171, 0xFF598979, 0xFF568676, 0xFF508070, 0xFF558575, 0xFF578777, 0xFF528272, 0xFF538373, 0xFF528171,
        0xFF588777, 0xFF588777, 0xFF558474, 0xFF598878, 0xFF588777, 0xFF518070, 0xFF538272, 0xFF528272, 0xFF538373, 0xFF548474, 0xFF568575, 0xFF578676, 0xFF598576, 0xFF598576, 0xFF598576, 0xFF568676, 0xFF558575, 0xFF558575, 0xFF548474, 0xFF548474,
        0xFF558575, 0xFF558575, 0xFF568676, 0xFF5E8A79, 0xFF5E8A79, 0xFF5D8978, 0xFF5C8877, 0xFF5B8776, 0xFF5B8776, 0xFF5A8675, 0xFF5A8675, 0xFF598471, 0xFF5A8572, 0xFF608B78, 0xFF638C7C, 0xFF5F8878, 0xFF5D8678, 0xFF5F8578, 0xFF5C8275, 0xFF58816F,
        0xFF537C6A, 0xFF5C8573, 0xFF668F7D, 0xFF628B79, 0xFF5E8775, 0xFF5E8775, 0xFF598270, 0xFF5A8373, 0xFF598272, 0xFF5C8575, 0xFF608977, 0xFF5D8674, 0xFF5F8975, 0xFF618B77, 0xFF58826E, 0xFF56816E, 0xFF588370, 0xFF5A8572, 0xFF5C8774, 0xFF5D8875,
        0xFF5C8774, 0xFF5B8673, 0xFF5A8572, 0xFF5D8875, 0xFF5D8875, 0xFF5C8774, 0xFF5B8673, 0xFF5B8673, 0xFF5A8572, 0xFF598471, 0xFF598471, 0xFF5C8774, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5E8976,
        0xFF5E8976, 0xFF5C8774, 0xFF598471, 0xFF5B8673, 0xFF5B8673, 0xFF57826F, 0xFF5A8572, 0xFF588372, 0xFF5B8675, 0xFF5E8978, 0xFF598473, 0xFF547F6E, 0xFF5E8978, 0xFF5C8776, 0xFF608B7A, 0xFF5C8974, 0xFF5C8974, 0xFF5B8874, 0xFF5B8673, 0xFF5C8573,
        0xFF5B8474, 0xFF608677, 0xFF668C7D, 0xFF5C8573, 0xFF5D8674, 0xFF5E8775, 0xFF5F8876, 0xFF5E8775, 0xFF5D8674, 0xFF5B8472, 0xFF5A8371, 0xFF5F8876, 0xFF608977, 0xFF618A78, 0xFF628B79, 0xFF618A78, 0xFF5E8775, 0xFF5C8573, 0xFF5A8371, 0xFF628473,
        0xFF638574, 0xFF658776, 0xFF698A77, 0xFF6A8B78, 0xFF688873, 0xFF62826D, 0xFF5D7D68, 0xFF5D735E, 0xFF5D7460, 0xFF6D8472, 0xFF466150, 0xFF3A5648, 0xFF28463A, 0xFF35544C, 0xFF34534B, 0xFF456357, 0xFF607C70, 0xFF758F84, 0xFF72867D, 0xFF75847D,
        0xFF858E89, 0xFF8E938F, 0xFF80827F, 0xFF848D88, 0xFF7B8680, 0xFF84918A, 0xFF7C8D85, 0xFF6D8279, 0xFF7E988D, 0xFF6F8D81, 0xFF6B897D, 0xFF608D79, 0xFF628977, 0xFF678979, 0xFF65897B, 0xFF53796A, 0x00749288, 0x00F8FFFF, 0x00FFFDFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFCFF, 0x00F5FFFD, 0xFF577B6F, 0xFF518070, 0xFF679384, 0xFF6A9083, 0xFF5D8678, 0xFF608C7D, 0xFF638A78, 0xFF638A78, 0xFF658C7A, 0xFF668D7B, 0xFF648B79,
        0xFF5F8674, 0xFF5E8573, 0xFF628977, 0xFF598176, 0xFF577F74, 0xFF537B70, 0xFF60887D, 0xFF5C8479, 0xFF527A6F, 0xFF60887D, 0xFF60887D, 0xFF598176, 0xFF567E73, 0xFF557D72, 0xFF567E73, 0xFF5C8479, 0xFF4B7368, 0xFF648C81, 0xFF648C81, 0xFF5D897A,
        0xFF5D897A, 0xFF5D897A, 0xFF5C8879, 0xFF5C8879, 0xFF5C8879, 0xFF5C8879, 0xFF5B8778, 0xFF5A8979, 0xFF5A8979, 0xFF598878, 0xFF578777, 0xFF578777, 0xFF568676, 0xFF548776, 0xFF538675, 0xFF568676, 0xFF568676, 0xFF568676, 0xFF578777, 0xFF578777,
        0xFF568676, 0xFF568676, 0xFF568676, 0xFF528171, 0xFF578676, 0xFF588777, 0xFF578777, 0xFF568676, 0xFF518473, 0xFF4E8170, 0xFF4B7E6D, 0xFF578777, 0xFF548474, 0xFF558575, 0xFF538373, 0xFF538373, 0xFF568676, 0xFF538373, 0xFF548474, 0xFF5A8979,
        0xFF598878, 0xFF548373, 0xFF507F6F, 0xFF548373, 0xFF548373, 0xFF548373, 0xFF5B8A7A, 0xFF538373, 0xFF538373, 0xFF548474, 0xFF578676, 0xFF578676, 0xFF5A8677, 0xFF5A8677, 0xFF5A8677, 0xFF568676, 0xFF558575, 0xFF558575, 0xFF548474, 0xFF548474,
        0xFF558575, 0xFF558575, 0xFF568676, 0xFF5A8675, 0xFF598574, 0xFF598574, 0xFF588473, 0xFF588473, 0xFF578372, 0xFF578372, 0xFF578372, 0xFF5E8976, 0xFF5B8673, 0xFF5B8673, 0xFF5D8676, 0xFF5B8474, 0xFF5A8375, 0xFF5A8073, 0xFF587E71, 0xFF618A78,
        0xFF5E8775, 0xFF5B8472, 0xFF5A8371, 0xFF5B8472, 0xFF5C8573, 0xFF5C8573, 0xFF5A8371, 0xFF5C8575, 0xFF588171, 0xFF598272, 0xFF5C8573, 0xFF5A8371, 0xFF5B8571, 0xFF5E8874, 0xFF59836F, 0xFF598471, 0xFF598471, 0xFF5A8572, 0xFF5A8572, 0xFF5A8572,
        0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5C8774, 0xFF5C8774, 0xFF5B8673, 0xFF5B8673, 0xFF5A8572, 0xFF5A8572, 0xFF5A8572, 0xFF5A8572, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF608977,
        0xFF5D8674, 0xFF5B8472, 0xFF5B8472, 0xFF5D8674, 0xFF5B8472, 0xFF5B8472, 0xFF608977, 0xFF5D8676, 0xFF5B8474, 0xFF5D8676, 0xFF5D8676, 0xFF608979, 0xFF608979, 0xFF5B8474, 0xFF588171, 0xFF588570, 0xFF5D8874, 0xFF5D8875, 0xFF5D8875, 0xFF5F8876,
        0xFF5D8676, 0xFF5C8273, 0xFF5D8374, 0xFF5D8674, 0xFF5E8775, 0xFF5E8775, 0xFF5F8876, 0xFF5E8775, 0xFF5D8674, 0xFF5C8573, 0xFF5B8472, 0xFF5C8573, 0xFF5D8674, 0xFF5E8775, 0xFF5F8876, 0xFF5F8876, 0xFF5E8775, 0xFF5D8674, 0xFF5D8674, 0xFF5F8674,
        0xFF608573, 0xFF618674, 0xFF648977, 0xFF678A76, 0xFF668975, 0xFF648570, 0xFF61826D, 0xFF637C67, 0xFF637F69, 0xFF6E8976, 0xFF557260, 0xFF527162, 0xFF4D6D60, 0xFF517267, 0xFF4B6C63, 0xFF4C6E60, 0xFF668679, 0xFF6D897D, 0xFF5C7369, 0xFF52665D,
        0xFF59665F, 0xFF7B8680, 0xFF707974, 0xFF6D7A73, 0xFF4F5E57, 0xFF4F6058, 0xFF4E635A, 0xFF466055, 0xFF638175, 0xFF67877A, 0xFF688A7C, 0xFF608D79, 0xFF628977, 0xFF678979, 0xFF65897B, 0xFF53796A, 0x00749288, 0x00F8FFFF, 0x00FFFDFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFCFF, 0x04F4FFFC, 0xFF577B6F, 0xFF4F7E6E, 0xFF628E7F, 0xFF678D80, 0xFF5E8779, 0xFF638F80, 0xFF628B79, 0xFF628B79, 0xFF638C7A, 0xFF628B79, 0xFF5F8876,
        0xFF5F8876, 0xFF628B79, 0xFF618A78, 0xFF668E83, 0xFF699186, 0xFF5D857A, 0xFF61897E, 0xFF678F84, 0xFF658D82, 0xFF6B9388, 0xFF678F84, 0xFF678F84, 0xFF648C81, 0xFF5F877C, 0xFF61897E, 0xFF668E83, 0xFF61897E, 0xFF6A9287, 0xFF5E867B, 0xFF5D897A,
        0xFF5D897A, 0xFF5D897A, 0xFF5D897A, 0xFF5C8879, 0xFF5C8879, 0xFF5C8879, 0xFF5C8879, 0xFF5A8979, 0xFF5A8979, 0xFF598878, 0xFF578777, 0xFF568676, 0xFF558575, 0xFF528574, 0xFF528574, 0xFF578777, 0xFF568676, 0xFF568676, 0xFF568676, 0xFF568676,
        0xFF568676, 0xFF568676, 0xFF578777, 0xFF578676, 0xFF598878, 0xFF598878, 0xFF578676, 0xFF578777, 0xFF588878, 0xFF578777, 0xFF588878, 0xFF5B8A7A, 0xFF518070, 0xFF548373, 0xFF538272, 0xFF528171, 0xFF568575, 0xFF548373, 0xFF568575, 0xFF578676,
        0xFF558474, 0xFF548373, 0xFF568575, 0xFF5B8A7A, 0xFF568575, 0xFF548373, 0xFF538272, 0xFF538373, 0xFF548474, 0xFF558575, 0xFF578676, 0xFF588777, 0xFF5B8778, 0xFF5B8778, 0xFF5B8778, 0xFF568676, 0xFF568676, 0xFF558575, 0xFF558575, 0xFF558575,
        0xFF558575, 0xFF568676, 0xFF568676, 0xFF578372, 0xFF578372, 0xFF578372, 0xFF578372, 0xFF568271, 0xFF568271, 0xFF568271, 0xFF568271, 0xFF5C8774, 0xFF57826F, 0xFF537E6B, 0xFF598272, 0xFF588171, 0xFF5A8375, 0xFF5A8073, 0xFF5B8174, 0xFF57806E,
        0xFF5E8775, 0xFF5E8775, 0xFF598270, 0xFF598270, 0xFF598270, 0xFF5A8371, 0xFF5F8876, 0xFF5B8476, 0xFF598272, 0xFF5B8474, 0xFF5C8575, 0xFF5A8371, 0xFF5A8371, 0xFF5C8672, 0xFF5E8874, 0xFF5C8774, 0xFF5A8572, 0xFF598471, 0xFF588370, 0xFF588370,
        0xFF598471, 0xFF5B8673, 0xFF5C8774, 0xFF5B8673, 0xFF5B8673, 0xFF5A8572, 0xFF5A8572, 0xFF5A8572, 0xFF5A8572, 0xFF5A8572, 0xFF5A8572, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5E8775,
        0xFF5D8674, 0xFF5C8573, 0xFF5F8876, 0xFF608977, 0xFF5C8573, 0xFF5B8472, 0xFF5E8775, 0xFF5F8878, 0xFF5A8373, 0xFF598272, 0xFF598272, 0xFF608979, 0xFF5A8373, 0xFF5F8878, 0xFF5C8575, 0xFF5D8773, 0xFF608A76, 0xFF5B8472, 0xFF598270, 0xFF598270,
        0xFF5C8575, 0xFF5B8474, 0xFF5E8777, 0xFF5E8775, 0xFF5E8775, 0xFF5E8775, 0xFF5E8775, 0xFF5E8775, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5B8472, 0xFF5C8573, 0xFF5C8573, 0xFF5C8573, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5C8774,
        0xFF5C8774, 0xFF5D8773, 0xFF5E8874, 0xFF628974, 0xFF638A75, 0xFF638A75, 0xFF648A73, 0xFF698974, 0xFF698A75, 0xFF6E8F7C, 0xFF628473, 0xFF658777, 0xFF6B8F81, 0xFF688E81, 0xFF658B7E, 0xFF5F8576, 0xFF6F9385, 0xFF6B8D7F, 0xFF5F7D71, 0xFF5C766B,
        0xFF576C63, 0xFF7B8F86, 0xFF62736B, 0xFF6D8279, 0xFF4A5F56, 0xFF41584E, 0xFF456155, 0xFF3E5E51, 0xFF5B7D6F, 0xFF65897B, 0xFF678D7E, 0xFF608D79, 0xFF628977, 0xFF678979, 0xFF65897B, 0xFF53796A, 0x00749288, 0x00F8FFFF, 0x00FFFDFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFAFD, 0x04F4FFFC, 0xFF5B7F73, 0xFF548373, 0xFF649081, 0xFF678D80, 0xFF5D8678, 0xFF608C7D, 0xFF618A7A, 0xFF618A7A, 0xFF608979, 0xFF618A7A, 0xFF5F8878,
        0xFF648D7D, 0xFF668F7F, 0xFF578070, 0xFF568074, 0xFF648E82, 0xFF5D877B, 0xFF5D877B, 0xFF669084, 0xFF638D81, 0xFF628C80, 0xFF628C80, 0xFF628C80, 0xFF638D81, 0xFF628C80, 0xFF608A7E, 0xFF5C867A, 0xFF578175, 0xFF5D877B, 0xFF5E887C, 0xFF608C7D,
        0xFF608C7D, 0xFF5F8B7C, 0xFF5F8B7C, 0xFF5F8B7C, 0xFF5F8B7C, 0xFF5E8A7B, 0xFF5E8A7B, 0xFF5A8979, 0xFF5A8979, 0xFF598878, 0xFF588878, 0xFF578777, 0xFF568676, 0xFF538675, 0xFF538675, 0xFF578777, 0xFF578777, 0xFF568676, 0xFF558575, 0xFF558575,
        0xFF568676, 0xFF578777, 0xFF578777, 0xFF558575, 0xFF538373, 0xFF548373, 0xFF558474, 0xFF5A8979, 0xFF5A8979, 0xFF538272, 0xFF588475, 0xFF598576, 0xFF558172, 0xFF5A8677, 0xFF598576, 0xFF5B8778, 0xFF5E8A7B, 0xFF588475, 0xFF588475, 0xFF5D897A,
        0xFF5B8778, 0xFF5B8778, 0xFF5A8677, 0xFF5E8A7B, 0xFF5A8677, 0xFF618D7E, 0xFF5C8879, 0xFF548474, 0xFF558575, 0xFF568676, 0xFF588777, 0xFF598878, 0xFF5B8778, 0xFF5B8778, 0xFF5B8778, 0xFF578777, 0xFF568676, 0xFF568676, 0xFF558575, 0xFF558575,
        0xFF568676, 0xFF568676, 0xFF578777, 0xFF588473, 0xFF588473, 0xFF588473, 0xFF598574, 0xFF598574, 0xFF598574, 0xFF5A8675, 0xFF5A8675, 0xFF5E8976, 0xFF5D8875, 0xFF5D8875, 0xFF648D7D, 0xFF5C8575, 0xFF5F887A, 0xFF62887B, 0xFF668C7F, 0xFF608979,
        0xFF648D7D, 0xFF668F7F, 0xFF658E7E, 0xFF638C7C, 0xFF618A7A, 0xFF618A7A, 0xFF618A7A, 0xFF618A7C, 0xFF60897B, 0xFF618A7C, 0xFF5D8676, 0xFF5D8676, 0xFF5D8674, 0xFF598270, 0xFF5E8775, 0xFF5C8774, 0xFF5B8673, 0xFF5A8572, 0xFF588370, 0xFF588370,
        0xFF598471, 0xFF5A8572, 0xFF5B8673, 0xFF5A8572, 0xFF5A8572, 0xFF5A8572, 0xFF5A8572, 0xFF5A8572, 0xFF5A8572, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5C8774, 0xFF5C8774, 0xFF5C8774, 0xFF5C8774, 0xFF5C8774, 0xFF5D8674,
        0xFF628B79, 0xFF5D8674, 0xFF5E8775, 0xFF638C7A, 0xFF618A78, 0xFF608977, 0xFF5F8876, 0xFF628B79, 0xFF628B79, 0xFF618A78, 0xFF5B8472, 0xFF5E8775, 0xFF57806E, 0xFF628B79, 0xFF628B79, 0xFF5E8572, 0xFF628976, 0xFF5F8674, 0xFF618876, 0xFF598270,
        0xFF5D8676, 0xFF5C8575, 0xFF608979, 0xFF5F8876, 0xFF5F8876, 0xFF5E8775, 0xFF5E8775, 0xFF5E8775, 0xFF5E8775, 0xFF5E8775, 0xFF5E8775, 0xFF5F8876, 0xFF5E8775, 0xFF5D8674, 0xFF5C8573, 0xFF5B8472, 0xFF5B8472, 0xFF5B8472, 0xFF5B8472, 0xFF5B8874,
        0xFF5B8874, 0xFF5B8874, 0xFF5D8874, 0xFF5F8A76, 0xFF608C75, 0xFF628C76, 0xFF638D77, 0xFF69907B, 0xFF688F7A, 0xFF6A917E, 0xFF6A917F, 0xFF668D7B, 0xFF6A9383, 0xFF638C7E, 0xFF668F81, 0xFF6C9786, 0xFF689382, 0xFF658B7C, 0xFF5F8375, 0xFF648477,
        0xFF5C7A6E, 0xFF7B958A, 0xFF617B70, 0xFF688476, 0xFF5B7A6B, 0xFF577768, 0xFF5C7E6E, 0xFF527766, 0xFF5F8674, 0xFF5F8876, 0xFF5F8876, 0xFF608D79, 0xFF628977, 0xFF678979, 0xFF65897B, 0xFF53796A, 0x00749288, 0x00F8FFFF, 0x00FFFDFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFCFF, 0x04F4FFFC, 0xFF5A7E72, 0xFF538272, 0xFF618D7E, 0xFF668C7F, 0xFF618A7C, 0xFF659182, 0xFF5E8777, 0xFF5E8777, 0xFF5E8777, 0xFF638C7C, 0xFF618A7A,
        0xFF638C7C, 0xFF5A8373, 0xFF345D4D, 0xFF1D473B, 0xFF224C40, 0xFF224C40, 0xFF234D41, 0xFF214B3F, 0xFF214B3F, 0xFF265044, 0xFF224C40, 0xFF214B3F, 0xFF1F493D, 0xFF254F43, 0xFF265044, 0xFF2D574B, 0xFF224C40, 0xFF224C40, 0xFF2F594D, 0xFF5F8B7C,
        0xFF5F8B7C, 0xFF5F8B7C, 0xFF5F8B7C, 0xFF5F8B7C, 0xFF5E8A7B, 0xFF5E8A7B, 0xFF5E8A7B, 0xFF5A8979, 0xFF5A8979, 0xFF598878, 0xFF588878, 0xFF588878, 0xFF588878, 0xFF568978, 0xFF568978, 0xFF578777, 0xFF578777, 0xFF568676, 0xFF558575, 0xFF558575,
        0xFF568676, 0xFF578777, 0xFF578777, 0xFF5C8C7C, 0xFF598979, 0xFF588878, 0xFF598878, 0xFF5D897A, 0xFF467263, 0xFF244E40, 0xFF214B3D, 0xFF224E3F, 0xFF255142, 0xFF204C3D, 0xFF1D493A, 0xFF224E3F, 0xFF265243, 0xFF255142, 0xFF275344, 0xFF285445,
        0xFF275344, 0xFF285445, 0xFF214D3E, 0xFF214D3E, 0xFF275344, 0xFF507C6D, 0xFF5D897A, 0xFF548474, 0xFF558575, 0xFF568676, 0xFF588777, 0xFF598878, 0xFF5B8778, 0xFF5C8879, 0xFF5B8778, 0xFF578777, 0xFF578777, 0xFF568676, 0xFF568676, 0xFF568676,
        0xFF568676, 0xFF578777, 0xFF578777, 0xFF598574, 0xFF5A8675, 0xFF5A8675, 0xFF5A8675, 0xFF5B8776, 0xFF5B8776, 0xFF5C8877, 0xFF5C8877, 0xFF5B8673, 0xFF5B8673, 0xFF5A8572, 0xFF567F6F, 0xFF335C4C, 0xFF2F584A, 0xFF305649, 0xFF305649, 0xFF355E4E,
        0xFF2C5545, 0xFF2B5444, 0xFF305949, 0xFF325B4B, 0xFF335C4C, 0xFF325B4B, 0xFF2E5747, 0xFF30584D, 0xFF2F584A, 0xFF2E5749, 0xFF2D5648, 0xFF456E5E, 0xFF5D8676, 0xFF5B8472, 0xFF5E8775, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5A8572, 0xFF5A8572,
        0xFF5A8572, 0xFF5A8572, 0xFF5A8572, 0xFF5A8572, 0xFF5A8572, 0xFF5A8572, 0xFF5A8572, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5C8774, 0xFF5C8774, 0xFF5C8774, 0xFF5D8875, 0xFF5D8875, 0xFF5E8573,
        0xFF618876, 0xFF456C5A, 0xFF345B49, 0xFF385F4D, 0xFF385F4D, 0xFF385F4D, 0xFF365D4B, 0xFF355C4A, 0xFF3A614F, 0xFF3B6250, 0xFF385F4D, 0xFF355C4A, 0xFF345B49, 0xFF365D4B, 0xFF345B49, 0xFF315342, 0xFF355A48, 0xFF476C5B, 0xFF628977, 0xFF5E8573,
        0xFF5F8878, 0xFF5A8574, 0xFF5D8877, 0xFF5E8775, 0xFF5E8775, 0xFF5E8775, 0xFF5E8775, 0xFF5E8775, 0xFF5E8775, 0xFF5F8876, 0xFF5F8876, 0xFF608977, 0xFF5F8876, 0xFF5E8775, 0xFF5C8573, 0xFF5C8573, 0xFF5C8573, 0xFF5D8674, 0xFF5E8775, 0xFF5A8977,
        0xFF5A8977, 0xFF5D8A76, 0xFF5E8B77, 0xFF5F8A76, 0xFF608B77, 0xFF618C78, 0xFF618D76, 0xFF4B7663, 0xFF3D6855, 0xFF3E6956, 0xFF47705E, 0xFF426B5B, 0xFF436C5C, 0xFF3E6757, 0xFF446D5D, 0xFF477362, 0xFF426E5D, 0xFF497463, 0xFF477060, 0xFF476B5D,
        0xFF446658, 0xFF57776A, 0xFF527265, 0xFF4B6D5D, 0xFF6B8D7D, 0xFF6F9483, 0xFF719685, 0xFF69907E, 0xFF6A9381, 0xFF648D7B, 0xFF66917E, 0xFF608D79, 0xFF628977, 0xFF678979, 0xFF65897B, 0xFF53796A, 0x00749288, 0x00F8FFFF, 0x00FFFDFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFCFF, 0x08F1FDF9, 0xFF587C70, 0xFF518070, 0xFF5D897A, 0xFF61877A, 0xFF5C8577, 0xFF5E8A7B, 0xFF618B7D, 0xFF608A7C, 0xFF5C8678, 0xFF618B7D, 0xFF5F897B,
        0xFF648E80, 0xFF588274, 0xFF234D3F, 0xFF1A4639, 0xFF174336, 0xFF1F4B3E, 0xFF245043, 0xFF224E41, 0xFF2C584B, 0xFF325E51, 0xFF234F42, 0xFF265245, 0xFF1C483B, 0xFF1F4B3E, 0xFF133F32, 0xFF265245, 0xFF133F32, 0xFF144033, 0xFF315D50, 0xFF5C8879,
        0xFF5C8879, 0xFF5C8879, 0xFF5C8879, 0xFF5C8879, 0xFF5B8778, 0xFF5B8778, 0xFF5B8778, 0xFF598878, 0xFF598878, 0xFF598878, 0xFF588878, 0xFF588878, 0xFF588878, 0xFF568978, 0xFF568978, 0xFF578777, 0xFF568676, 0xFF568676, 0xFF568676, 0xFF568676,
        0xFF568676, 0xFF568676, 0xFF578777, 0xFF528574, 0xFF518473, 0xFF548474, 0xFF588777, 0xFF618D7E, 0xFF3F695B, 0xFF10392B, 0xFF1A4033, 0xFF113B2D, 0xFF1A4436, 0xFF174133, 0xFF1F493B, 0xFF1F493B, 0xFF174133, 0xFF1B4537, 0xFF133D2F, 0xFF143E30,
        0xFF153F31, 0xFF214B3D, 0xFF1E483A, 0xFF143E30, 0xFF0F392B, 0xFF477163, 0xFF5F897B, 0xFF548474, 0xFF558575, 0xFF568676, 0xFF588777, 0xFF598878, 0xFF5B8778, 0xFF5B8778, 0xFF5B8778, 0xFF588878, 0xFF578777, 0xFF578777, 0xFF568676, 0xFF568676,
        0xFF578777, 0xFF578777, 0xFF588878, 0xFF598574, 0xFF598574, 0xFF5A8675, 0xFF5A8675, 0xFF5A8675, 0xFF5B8776, 0xFF5B8776, 0xFF5B8776, 0xFF628D7A, 0xFF618C79, 0xFF618C79, 0xFF537C6C, 0xFF1A4333, 0xFF163F31, 0xFF1C4235, 0xFF153B2E, 0xFF1A4335,
        0xFF1E4739, 0xFF224B3D, 0xFF214A3C, 0xFF1C4537, 0xFF184133, 0xFF184133, 0xFF1A4335, 0xFF1A4237, 0xFF1A4237, 0xFF10392B, 0xFF083123, 0xFF315A4C, 0xFF5F8878, 0xFF5E8777, 0xFF5B8474, 0xFF5A8572, 0xFF5B8673, 0xFF5C8774, 0xFF5C8774, 0xFF5C8774,
        0xFF5C8774, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5C8774, 0xFF5C8774, 0xFF5D8875, 0xFF5D8875, 0xFF5E8976, 0xFF628977,
        0xFF5E8573, 0xFF284F3D, 0xFF0A311F, 0xFF143B29, 0xFF143B29, 0xFF143B29, 0xFF183F2D, 0xFF173E2C, 0xFF143B29, 0xFF103725, 0xFF153C2A, 0xFF0F3624, 0xFF1C4331, 0xFF113826, 0xFF163D2B, 0xFF153625, 0xFF0F301F, 0xFF2A4C3C, 0xFF5D8271, 0xFF5B8270,
        0xFF5D8676, 0xFF5B8675, 0xFF5C8877, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5E8775, 0xFF5E8775, 0xFF5E8775, 0xFF5E8775, 0xFF5E8775, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5E8775, 0xFF5F8876, 0xFF608977, 0xFF618A78, 0xFF5C8877,
        0xFF5E8A79, 0xFF5F8B7A, 0xFF608B78, 0xFF608B78, 0xFF608A76, 0xFF618B77, 0xFF628C78, 0xFF305C4B, 0xFF113C2B, 0xFF103B2A, 0xFF133C2C, 0xFF103929, 0xFF153B2C, 0xFF1C4032, 0xFF1E4234, 0xFF154130, 0xFF15402F, 0xFF164130, 0xFF184131, 0xFF123829,
        0xFF163A2C, 0xFF14382A, 0xFF153729, 0xFF234837, 0xFF5E8372, 0xFF678C7B, 0xFF638A78, 0xFF5F8674, 0xFF608775, 0xFF5E8573, 0xFF628B79, 0xFF608D79, 0xFF628977, 0xFF678979, 0xFF65897B, 0xFF53796A, 0x00749288, 0x00F8FFFF, 0x00FFFDFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFAFD, 0x0BEFFBF7, 0xFF5A7E72, 0xFF568575, 0xFF638F80, 0xFF688E81, 0xFF618A7C, 0xFF608C7D, 0xFF608A7C, 0xFF608A7C, 0xFF5B8577, 0xFF5E887A, 0xFF5B8577,
        0xFF648E80, 0xFF5D8779, 0xFF224C3E, 0xFF2A5649, 0xFF406C5F, 0xFF3B675A, 0xFF335F52, 0xFF477366, 0xFF507C6F, 0xFF406C5F, 0xFF3A6659, 0xFF315D50, 0xFF2D594C, 0xFF3C685B, 0xFF285447, 0xFF4A7669, 0xFF255144, 0xFF164235, 0xFF2E5A4D, 0xFF5C8879,
        0xFF5C8879, 0xFF5B8778, 0xFF5B8778, 0xFF5B8778, 0xFF5B8778, 0xFF5A8677, 0xFF5A8677, 0xFF588777, 0xFF588777, 0xFF588777, 0xFF578777, 0xFF568676, 0xFF568676, 0xFF538675, 0xFF538675, 0xFF568676, 0xFF568676, 0xFF568676, 0xFF578777, 0xFF578777,
        0xFF568676, 0xFF568676, 0xFF568676, 0xFF548977, 0xFF558877, 0xFF538373, 0xFF558474, 0xFF638D7F, 0xFF3E6759, 0xFF143A2D, 0xFF34584C, 0xFF315A4C, 0xFF345D4F, 0xFF386153, 0xFF557E70, 0xFF487163, 0xFF30594B, 0xFF3C6557, 0xFF285143, 0xFF396355,
        0xFF2D5749, 0xFF3C6658, 0xFF446E60, 0xFF345E50, 0xFF174133, 0xFF446E60, 0xFF5A8476, 0xFF548474, 0xFF558575, 0xFF568676, 0xFF588777, 0xFF598878, 0xFF5B8778, 0xFF5B8778, 0xFF5B8778, 0xFF588878, 0xFF588878, 0xFF578777, 0xFF578777, 0xFF578777,
        0xFF578777, 0xFF588878, 0xFF588878, 0xFF5A8675, 0xFF5A8675, 0xFF5A8675, 0xFF5B8776, 0xFF5B8776, 0xFF5B8776, 0xFF5B8776, 0xFF5B8776, 0xFF5B8673, 0xFF5A8572, 0xFF608B78, 0xFF567F6F, 0xFF1A4333, 0xFF285143, 0xFF3D6356, 0xFF345A4D, 0xFF2E5749,
        0xFF487163, 0xFF477062, 0xFF365F51, 0xFF345D4F, 0xFF2F584A, 0xFF285143, 0xFF30594B, 0xFF40685D, 0xFF497166, 0xFF396156, 0xFF194234, 0xFF365F51, 0xFF648D7D, 0xFF5E8777, 0xFF578070, 0xFF5B8673, 0xFF5B8673, 0xFF5C8774, 0xFF5D8875, 0xFF5D8875,
        0xFF5D8875, 0xFF5D8875, 0xFF5D8875, 0xFF5D8875, 0xFF5D8875, 0xFF5D8875, 0xFF5C8774, 0xFF5C8774, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5C8774, 0xFF5D8875, 0xFF5D8875, 0xFF5E8976, 0xFF5E8976, 0xFF678C7B,
        0xFF648978, 0xFF2A4F3E, 0xFF193E2D, 0xFF355A49, 0xFF315645, 0xFF2D5241, 0xFF385D4C, 0xFF4D7260, 0xFF3D6250, 0xFF2F5442, 0xFF375C4A, 0xFF1E4331, 0xFF2D5240, 0xFF113624, 0xFF254A38, 0xFF3B5A4A, 0xFF183727, 0xFF284839, 0xFF638877, 0xFF5B8270,
        0xFF5C8575, 0xFF5C8776, 0xFF5B8776, 0xFF5B8472, 0xFF5C8573, 0xFF5D8674, 0xFF5E8775, 0xFF5F8876, 0xFF5E8775, 0xFF5E8775, 0xFF5D8674, 0xFF5C8573, 0xFF5C8573, 0xFF5C8573, 0xFF5D8674, 0xFF5E8775, 0xFF5F8876, 0xFF608977, 0xFF618A78, 0xFF5E887A,
        0xFF5F8A79, 0xFF618A7A, 0xFF618A7A, 0xFF628977, 0xFF628977, 0xFF658A78, 0xFF678C7A, 0xFF366054, 0xFF184234, 0xFF2A5345, 0xFF2E5447, 0xFF315547, 0xFF395B4D, 0xFF4D6F5F, 0xFF466657, 0xFF295242, 0xFF345D4D, 0xFF1E4737, 0xFF315748, 0xFF355B4C,
        0xFF4C7062, 0xFF3F6355, 0xFF2E5244, 0xFF204533, 0xFF5E8371, 0xFF668B79, 0xFF658A78, 0xFF608573, 0xFF618674, 0xFF648977, 0xFF658A78, 0xFF608D79, 0xFF628977, 0xFF678979, 0xFF65897B, 0xFF53796A, 0x00749288, 0x00F8FFFF, 0x00FFFDFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFCFF, 0x08F2FEFA, 0xFF5A7E72, 0xFF548373, 0xFF608C7D, 0xFF668C7F, 0xFF60897B, 0xFF5F8B7C, 0xFF5C8678, 0xFF618B7D, 0xFF608A7C, 0xFF648E80, 0xFF5E887A,
        0xFF658F81, 0xFF5D8779, 0xFF1E483A, 0xFF285447, 0xFF5E8A7D, 0xFF3E6A5D, 0xFF1B473A, 0xFF467265, 0xFF426E61, 0xFF194538, 0xFF2B574A, 0xFF4F7B6E, 0xFF3E6A5D, 0xFF3F6B5E, 0xFF1E4A3D, 0xFF517D70, 0xFF2D594C, 0xFF184437, 0xFF2A5649, 0xFF5D897A,
        0xFF5D897A, 0xFF5D897A, 0xFF5D897A, 0xFF5D897A, 0xFF5C8879, 0xFF5C8879, 0xFF5C8879, 0xFF588777, 0xFF588777, 0xFF578676, 0xFF558575, 0xFF548474, 0xFF538373, 0xFF518473, 0xFF508372, 0xFF558575, 0xFF568676, 0xFF578777, 0xFF578777, 0xFF578777,
        0xFF578777, 0xFF568676, 0xFF558575, 0xFF548977, 0xFF588B7A, 0xFF548474, 0xFF568273, 0xFF648E80, 0xFF3F6558, 0xFF193D31, 0xFF486C60, 0xFF4C7567, 0xFF355E50, 0xFF275042, 0xFF436C5E, 0xFF295244, 0xFF174032, 0xFF446D5F, 0xFF386153, 0xFF497365,
        0xFF275143, 0xFF2D5749, 0xFF426C5E, 0xFF3C6658, 0xFF1A4436, 0xFF487264, 0xFF5F897B, 0xFF548474, 0xFF548474, 0xFF558575, 0xFF588777, 0xFF588777, 0xFF5B8778, 0xFF5B8778, 0xFF5B8778, 0xFF588878, 0xFF588878, 0xFF578777, 0xFF578777, 0xFF578777,
        0xFF578777, 0xFF588878, 0xFF588878, 0xFF5D8978, 0xFF5D8978, 0xFF5D8978, 0xFF5D8978, 0xFF5D8978, 0xFF5D8978, 0xFF5D8978, 0xFF5D8978, 0xFF5F8A77, 0xFF5A8572, 0xFF618C79, 0xFF578070, 0xFF1B4434, 0xFF355E50, 0xFF53796C, 0xFF476D60, 0xFF264F41,
        0xFF456E60, 0xFF325B4D, 0xFF1C4537, 0xFF355E50, 0xFF416A5C, 0xFF365F51, 0xFF3D6658, 0xFF295148, 0xFF456D62, 0xFF426A5F, 0xFF1A4237, 0xFF30594B, 0xFF60897B, 0xFF608979, 0xFF5E8777, 0xFF5B8673, 0xFF5C8774, 0xFF5C8774, 0xFF5D8875, 0xFF5D8875,
        0xFF5E8976, 0xFF5F8A77, 0xFF5F8A77, 0xFF5E8976, 0xFF5E8976, 0xFF5D8875, 0xFF5D8875, 0xFF5C8774, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5C8774, 0xFF5D8875, 0xFF5E8976, 0xFF5E8976, 0xFF5E8976, 0xFF5F8473,
        0xFF608574, 0xFF2C5140, 0xFF2C5140, 0xFF537867, 0xFF436857, 0xFF325746, 0xFF3F6453, 0xFF3A5F4D, 0xFF2C513F, 0xFF2C513F, 0xFF476C5A, 0xFF2D5240, 0xFF426755, 0xFF244937, 0xFF4A6F5D, 0xFF526E5F, 0xFF1A3929, 0xFF224233, 0xFF658777, 0xFF5B8270,
        0xFF5C8575, 0xFF5A8675, 0xFF598574, 0xFF5A8371, 0xFF5B8472, 0xFF5D8674, 0xFF5E8775, 0xFF5F8876, 0xFF5E8775, 0xFF5D8674, 0xFF5C8573, 0xFF5E8775, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5F887A,
        0xFF62887B, 0xFF628879, 0xFF628879, 0xFF628776, 0xFF648978, 0xFF698B7B, 0xFF6C8E7D, 0xFF315B4F, 0xFF234B40, 0xFF4B7466, 0xFF4A7063, 0xFF47695B, 0xFF3F5F52, 0xFF4E6D5E, 0xFF355445, 0xFF2B5142, 0xFF4D7364, 0xFF2E5445, 0xFF496F60, 0xFF4C7062,
        0xFF5D8173, 0xFF3F6355, 0xFF163A2C, 0xFF224735, 0xFF5A7F6D, 0xFF638876, 0xFF6B8D7C, 0xFF658776, 0xFF668877, 0xFF698B7A, 0xFF628473, 0xFF608D79, 0xFF628977, 0xFF678979, 0xFF65897B, 0xFF53796A, 0x00749288, 0x00F8FFFF, 0x00FFFDFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x0FEDF9F7, 0xFF53776D, 0xFF548373, 0xFF5F8B7C, 0xFF618A7A, 0xFF658E7E, 0xFF628E7D, 0xFF678D78, 0xFF608A76, 0xFF5E8A79, 0xFF608F7F, 0xFF5E8E7E,
        0xFF5D8C7A, 0xFF5E8B76, 0xFF214D36, 0xFF2A584D, 0xFF57857A, 0xFF3A685D, 0xFF154338, 0xFF467469, 0xFF3F6D62, 0xFF17453A, 0xFF154338, 0xFF38665C, 0xFF4F7D73, 0xFF18463C, 0xFF113F35, 0xFF406E64, 0xFF29574D, 0xFF0E3C32, 0xFF29574D, 0xFF628B79,
        0xFF608775, 0xFF608677, 0xFF63897A, 0xFF5E887A, 0xFF588679, 0xFF578A7B, 0xFF5B9080, 0xFF578676, 0xFF578676, 0xFF578676, 0xFF578676, 0xFF578676, 0xFF568575, 0xFF558474, 0xFF558474, 0xFF568573, 0xFF568573, 0xFF578674, 0xFF588775, 0xFF588775,
        0xFF598876, 0xFF598876, 0xFF598876, 0xFF56836F, 0xFF588370, 0xFF5D8875, 0xFF5B8571, 0xFF658F7B, 0xFF456C59, 0xFF193F2A, 0xFF456B56, 0xFF517E67, 0xFF24513A, 0xFF24513A, 0xFF4E7B64, 0xFF24513A, 0xFF0D3A23, 0xFF214E37, 0xFF507D66, 0xFF305A44,
        0xFF173E29, 0xFF395F4A, 0xFF426551, 0xFF284E39, 0xFF183F2A, 0xFF48755E, 0xFF5D8D75, 0xFF568A73, 0xFF568670, 0xFF58816F, 0xFF5C8371, 0xFF5E8475, 0xFF5A8476, 0xFF558575, 0xFF548B78, 0xFF558673, 0xFF558673, 0xFF558673, 0xFF568573, 0xFF578674,
        0xFF588775, 0xFF5A8675, 0xFF5B8776, 0xFF5C8877, 0xFF5E8A79, 0xFF5F8B7A, 0xFF5E8A79, 0xFF5D8978, 0xFF5B8776, 0xFF5C8877, 0xFF5F8B7A, 0xFF60816E, 0xFF668C77, 0xFF678E79, 0xFF5B8770, 0xFF18452E, 0xFF325F48, 0xFF55826B, 0xFF315D46, 0xFF1D4A36,
        0xFF497662, 0xFF33604C, 0xFF103D29, 0xFF174430, 0xFF497662, 0xFF487561, 0xFF1F4C38, 0xFF1C4533, 0xFF3A6351, 0xFF507B68, 0xFF17422F, 0xFF355E4C, 0xFF6B907F, 0xFF5E7E6F, 0xFF678677, 0xFF5E8978, 0xFF608B7A, 0xFF5D8877, 0xFF5D8877, 0xFF618C7B,
        0xFF5F8A79, 0xFF5C8776, 0xFF5E8978, 0xFF608977, 0xFF5F8876, 0xFF5E8775, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5E8775, 0xFF5E8775, 0xFF5A8979, 0xFF588473, 0xFF598473, 0xFF5D8674, 0xFF5F8674, 0xFF628473, 0xFF648572, 0xFF658673, 0xFF5E8570,
        0xFF6B917C, 0xFF2B4E3A, 0xFF274835, 0xFF638672, 0xFF3B614C, 0xFF0E3822, 0xFF3D6A53, 0xFF466E56, 0xFF1A422A, 0xFF143C24, 0xFF466E56, 0xFF527A62, 0xFF1F472F, 0xFF426A52, 0xFF4D755D, 0xFF507262, 0xFF0F3423, 0xFF214836, 0xFF5F8876, 0xFF628B79,
        0xFF597E6D, 0xFF648475, 0xFF668274, 0xFF608775, 0xFF5F8674, 0xFF5E8573, 0xFF5D8472, 0xFF5E8573, 0xFF5E8573, 0xFF5E8573, 0xFF5E8573, 0xFF608775, 0xFF608775, 0xFF618876, 0xFF618876, 0xFF628977, 0xFF618876, 0xFF618876, 0xFF618876, 0xFF5F8878,
        0xFF608677, 0xFF628678, 0xFF668878, 0xFF688978, 0xFF658776, 0xFF658B76, 0xFF69907B, 0xFF34634F, 0xFF194834, 0xFF588773, 0xFF477662, 0xFF184733, 0xFF32614D, 0xFF558470, 0xFF1D4C38, 0xFF174135, 0xFF355F53, 0xFF5F897D, 0xFF2C564A, 0xFF2F594D,
        0xFF446E62, 0xFF4A7468, 0xFF214B3F, 0xFF17442D, 0xFF5A846E, 0xFF698C78, 0xFF698876, 0xFF708F7D, 0xFF668975, 0xFF628C76, 0xFF5C8972, 0xFF628D7A, 0xFF668B7A, 0xFF678778, 0xFF67897B, 0xFF53796A, 0x00719186, 0x00F7FFFF, 0x00FFFBFC, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x13ECF8F6, 0xFF52766C, 0xFF538272, 0xFF5E8A7B, 0xFF618778, 0xFF628B7B, 0xFF5E8A79, 0xFF618674, 0xFF5C8573, 0xFF578676, 0xFF4F8273, 0xFF4A7A6C,
        0xFF467263, 0xFF49705E, 0xFF1B3C2B, 0xFF224332, 0xFF3C5D4C, 0xFF3A5B4A, 0xFF294A39, 0xFF406150, 0xFF3F604F, 0xFF2C4D3C, 0xFF1D3E2D, 0xFF264234, 0xFF3B5749, 0xFF1E3A2C, 0xFF264234, 0xFF486456, 0xFF324E40, 0xFF1B3729, 0xFF254133, 0xFF497564,
        0xFF4A7564, 0xFF4D7867, 0xFF517B6D, 0xFF558172, 0xFF558376, 0xFF538375, 0xFF4F8273, 0xFF568575, 0xFF568575, 0xFF578676, 0xFF578676, 0xFF578676, 0xFF578676, 0xFF568575, 0xFF568575, 0xFF568573, 0xFF568573, 0xFF568573, 0xFF568573, 0xFF568573,
        0xFF568573, 0xFF558472, 0xFF558472, 0xFF568976, 0xFF4F806D, 0xFF507D69, 0xFF48715F, 0xFF4F7463, 0xFF375847, 0xFF143021, 0xFF365243, 0xFF3C5849, 0xFF324E3F, 0xFF284435, 0xFF405C4D, 0xFF314D3E, 0xFF1D392A, 0xFF173324, 0xFF3D594A, 0xFF1B3A2A,
        0xFF264233, 0xFF486455, 0xFF385445, 0xFF1C3B2B, 0xFF103221, 0xFF38624E, 0xFF4B7863, 0xFF447B64, 0xFF487C66, 0xFF527F6B, 0xFF578070, 0xFF588171, 0xFF557F71, 0xFF518171, 0xFF508372, 0xFF548572, 0xFF548572, 0xFF548572, 0xFF568573, 0xFF568573,
        0xFF598574, 0xFF5A8675, 0xFF5A8675, 0xFF598574, 0xFF5A8675, 0xFF5B8776, 0xFF5B8776, 0xFF5B8776, 0xFF5A8675, 0xFF598574, 0xFF588473, 0xFF5C8773, 0xFF5A8470, 0xFF577E6B, 0xFF507563, 0xFF1C3E2D, 0xFF305241, 0xFF4C6E5D, 0xFF385A49, 0xFF294537,
        0xFF486456, 0xFF3D594B, 0xFF233F31, 0xFF173325, 0xFF324E40, 0xFF355143, 0xFF193527, 0xFF3E5E4F, 0xFF436354, 0xFF446455, 0xFF143425, 0xFF305242, 0xFF5E8372, 0xFF577E6C, 0xFF608977, 0xFF5C8575, 0xFF5F8878, 0xFF5D8676, 0xFF5D8676, 0xFF618A7A,
        0xFF5F8878, 0xFF5B8474, 0xFF5E8777, 0xFF5F8876, 0xFF5F8876, 0xFF5E8775, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5C8877, 0xFF5A8675, 0xFF5B8675, 0xFF5D8875, 0xFF5F8876, 0xFF5D8773, 0xFF5C8370, 0xFF5A816E, 0xFF5C8370,
        0xFF5D7F6E, 0xFF264535, 0xFF254132, 0xFF597465, 0xFF486455, 0xFF284938, 0xFF3F6150, 0xFF3E5B49, 0xFF23402E, 0xFF112E1C, 0xFF2F4C3A, 0xFF415E4C, 0xFF1D3A28, 0xFF3F5C4A, 0xFF4C6957, 0xFF546C5F, 0xFF1D392B, 0xFF244435, 0xFF547B69, 0xFF5F8876,
        0xFF5B8472, 0xFF618876, 0xFF5D8271, 0xFF5D8472, 0xFF5D8472, 0xFF5D8472, 0xFF5D8472, 0xFF5E8573, 0xFF5E8573, 0xFF5E8573, 0xFF5E8573, 0xFF5F8674, 0xFF608775, 0xFF608775, 0xFF618876, 0xFF618876, 0xFF618876, 0xFF608775, 0xFF608775, 0xFF618778,
        0xFF608677, 0xFF608476, 0xFF628776, 0xFF608775, 0xFF5C8672, 0xFF5D8874, 0xFF618C78, 0xFF335344, 0xFF1F3F30, 0xFF517162, 0xFF49695A, 0xFF315142, 0xFF3B5B4C, 0xFF4B6B5C, 0xFF254536, 0xFF1D3B2F, 0xFF2A483C, 0xFF4A685C, 0xFF234135, 0xFF2D4B3F,
        0xFF426054, 0xFF4D6B5F, 0xFF314F43, 0xFF1E4834, 0xFF57816D, 0xFF658A78, 0xFF648675, 0xFF688A79, 0xFF618674, 0xFF638D79, 0xFF638D79, 0xFF608B78, 0xFF648978, 0xFF668677, 0xFF67897B, 0xFF547A6B, 0x00729287, 0x00F7FFFF, 0x00FFFDFE, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFBFF, 0x13ECF8F6, 0xFF52766C, 0xFF538272, 0xFF5D8779, 0xFF5E8475, 0xFF5F8576, 0xFF5A8574, 0xFF587D6C, 0xFF547D6D, 0xFF49796B, 0xFF35695C, 0xFF2B5B4F,
        0xFF244C41, 0xFF2B473B, 0xFF0F2318, 0xFF142512, 0xFF192A17, 0xFF21321F, 0xFF1C2D1A, 0xFF1D2E1B, 0xFF1F301D, 0xFF1F301D, 0xFF122310, 0xFF1B2616, 0xFF222D1D, 0xFF141F0F, 0xFF1D2818, 0xFF2B3626, 0xFF1F2A1A, 0xFF1A2515, 0xFF1B2616, 0xFF215141,
        0xFF265646, 0xFF2C5C4C, 0xFF346454, 0xFF427161, 0xFF4D7C6C, 0xFF528073, 0xFF538174, 0xFF558474, 0xFF558474, 0xFF568575, 0xFF578676, 0xFF578676, 0xFF578676, 0xFF578676, 0xFF578676, 0xFF568573, 0xFF568573, 0xFF558472, 0xFF558472, 0xFF548371,
        0xFF52816F, 0xFF51806E, 0xFF507F6D, 0xFF417A67, 0xFF306754, 0xFF2E5E4E, 0xFF295242, 0xFF2D4D40, 0xFF223A2D, 0xFF0B1C12, 0xFF1D2A21, 0xFF192019, 0xFF282F28, 0xFF1C231C, 0xFF252C25, 0xFF282F28, 0xFF232A23, 0xFF0F160F, 0xFF222922, 0xFF0D1D13,
        0xFF16261C, 0xFF28382E, 0xFF1D2E24, 0xFF152B1F, 0xFF052113, 0xFF1C3E2E, 0xFF305745, 0xFF205B45, 0xFF2B654F, 0xFF3F725F, 0xFF4D7C6A, 0xFF537F6E, 0xFF558172, 0xFF578374, 0xFF568575, 0xFF538471, 0xFF538471, 0xFF548371, 0xFF558472, 0xFF578372,
        0xFF588473, 0xFF588473, 0xFF5A8574, 0xFF598574, 0xFF588473, 0xFF578372, 0xFF588473, 0xFF578372, 0xFF558170, 0xFF507C6B, 0xFF497564, 0xFF336A53, 0xFF2E5D49, 0xFF2E5342, 0xFF344E41, 0xFF13271C, 0xFF1B2C22, 0xFF203429, 0xFF1A3024, 0xFF2B322A,
        0xFF363D35, 0xFF373E36, 0xFF30372F, 0xFF1C231B, 0xFF222921, 0xFF282F27, 0xFF1C231B, 0xFF263A2E, 0xFF24362A, 0xFF223227, 0xFF0B1D11, 0xFF20382A, 0xFF395B4A, 0xFF32624C, 0xFF386E56, 0xFF507969, 0xFF567F6F, 0xFF578070, 0xFF588171, 0xFF5D8676,
        0xFF5D8676, 0xFF5A8373, 0xFF5D8676, 0xFF5E8775, 0xFF5E8775, 0xFF5E8775, 0xFF5E8775, 0xFF5D8674, 0xFF5C8573, 0xFF5B8472, 0xFF5B8472, 0xFF5C8575, 0xFF5A8574, 0xFF588372, 0xFF558170, 0xFF53806C, 0xFF487763, 0xFF3C6B57, 0xFF356450, 0xFF395B4B,
        0xFF335243, 0xFF162E21, 0xFF0F2318, 0xFF26362C, 0xFF25352B, 0xFF1E2E24, 0xFF23342A, 0xFF35453A, 0xFF314136, 0xFF19291E, 0xFF1A2A1F, 0xFF27372C, 0xFF07170C, 0xFF142419, 0xFF17271C, 0xFF2C362D, 0xFF132318, 0xFF1B3325, 0xFF385A49, 0xFF426C58,
        0xFF426F5A, 0xFF487862, 0xFF4F7F69, 0xFF59806E, 0xFF5A816F, 0xFF5C8371, 0xFF5D8472, 0xFF5D8472, 0xFF5D8472, 0xFF5D8472, 0xFF5D8472, 0xFF5F8674, 0xFF5F8674, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF5F8674, 0xFF5F8674, 0xFF5F8375,
        0xFF5B7F71, 0xFF557B6C, 0xFF4C7766, 0xFF416D5C, 0xFF376652, 0xFF33644F, 0xFF32634E, 0xFF2B3D31, 0xFF182A1E, 0xFF2C3E32, 0xFF27392D, 0xFF2D3F33, 0xFF334539, 0xFF405246, 0xFF3E5044, 0xFF1F3425, 0xFF1C3122, 0xFF314637, 0xFF1C3122, 0xFF283D2E,
        0xFF334839, 0xFF384D3E, 0xFF263B2C, 0xFF113625, 0xFF39604E, 0xFF456E5C, 0xFF487360, 0xFF517C69, 0xFF57806E, 0xFF618876, 0xFF618675, 0xFF5C8774, 0xFF618675, 0xFF658576, 0xFF688A7C, 0xFF557B6C, 0x00729287, 0x00F7FFFF, 0x00FFFEFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFBFF, 0x0FEDF7F6, 0xFF55766D, 0xFF558172, 0xFF5D8779, 0xFF5D8374, 0xFF5D8374, 0xFF568170, 0xFF608274, 0xFF567F71, 0xFF46766A, 0xFF2A5C51, 0xFF204E44,
        0xFF1C3D36, 0xFF22352F, 0xFF17201B, 0xFF162214, 0xFF172315, 0xFF162214, 0xFF131F11, 0xFF152113, 0xFF121E10, 0xFF121E10, 0xFF152113, 0xFF171A11, 0xFF181B12, 0xFF13160D, 0xFF1B1E15, 0xFF1D2017, 0xFF181B12, 0xFF1C1F16, 0xFF1C1F16, 0xFF0F3F31,
        0xFF154839, 0xFF1D5041, 0xFF295C4D, 0xFF3A6D5E, 0xFF497868, 0xFF527E6F, 0xFF598375, 0xFF568273, 0xFF568273, 0xFF578374, 0xFF598576, 0xFF5A8677, 0xFF5A8677, 0xFF5A8677, 0xFF5A8677, 0xFF598574, 0xFF598574, 0xFF588473, 0xFF578372, 0xFF558170,
        0xFF527E6D, 0xFF4F7B6A, 0xFF4E7A69, 0xFF3B7866, 0xFF255E4D, 0xFF1F4F41, 0xFF1A4033, 0xFF1A342B, 0xFF1A2922, 0xFF121814, 0xFF161815, 0xFF101211, 0xFF181A19, 0xFF131514, 0xFF181A19, 0xFF161817, 0xFF1F2120, 0xFF161817, 0xFF161817, 0xFF1A1F1B,
        0xFF121713, 0xFF141915, 0xFF0F1813, 0xFF18251E, 0xFF081D14, 0xFF0F2B1F, 0xFF1A3C2E, 0xFF0E4C37, 0xFF1D5643, 0xFF346B58, 0xFF467968, 0xFF4C7B6B, 0xFF507C6B, 0xFF547F6E, 0xFF537E6D, 0xFF538270, 0xFF538270, 0xFF548371, 0xFF578372, 0xFF578372,
        0xFF578372, 0xFF588372, 0xFF588372, 0xFF5D8877, 0xFF588372, 0xFF578271, 0xFF568170, 0xFF547F6E, 0xFF517C6B, 0xFF477261, 0xFF396453, 0xFF1C5A43, 0xFF174A37, 0xFF17392B, 0xFF203129, 0xFF141A16, 0xFF161B17, 0xFF0F1511, 0xFF101B15, 0xFF181B14,
        0xFF151811, 0xFF171A13, 0xFF1E211A, 0xFF181B14, 0xFF151811, 0xFF171A13, 0xFF161912, 0xFF0E180F, 0xFF111810, 0xFF161B14, 0xFF111810, 0xFF17291D, 0xFF1F402F, 0xFF1C4D37, 0xFF1E5D42, 0xFF406959, 0xFF4A7363, 0xFF4F7868, 0xFF527B6B, 0xFF588171,
        0xFF5B8474, 0xFF5B8474, 0xFF5E8777, 0xFF5D8674, 0xFF5E8775, 0xFF5E8775, 0xFF5E8775, 0xFF5D8674, 0xFF5C8573, 0xFF5A8371, 0xFF598270, 0xFF59806E, 0xFF58816F, 0xFF537E6D, 0xFF4E7A69, 0xFF467563, 0xFF336653, 0xFF1E533F, 0xFF134836, 0xFF203E32,
        0xFF1C362B, 0xFF13271E, 0xFF0E1913, 0xFF0D130F, 0xFF111612, 0xFF151A16, 0xFF151A16, 0xFF101612, 0xFF1C221E, 0xFF121814, 0xFF121814, 0xFF212723, 0xFF101612, 0xFF161C18, 0xFF121814, 0xFF1A1712, 0xFF141912, 0xFF17271C, 0xFF213D2E, 0xFF244E3A,
        0xFF265640, 0xFF356952, 0xFF4B8169, 0xFF557C6A, 0xFF577E6C, 0xFF5A816F, 0xFF5D8472, 0xFF5D8472, 0xFF5D8472, 0xFF5D8472, 0xFF5D8472, 0xFF5E8573, 0xFF5E8573, 0xFF5F8674, 0xFF5F8674, 0xFF5F8674, 0xFF5E8573, 0xFF5E8573, 0xFF5D8472, 0xFF5D7F71,
        0xFF55796B, 0xFF4B7665, 0xFF3D6C5A, 0xFF2A5D4C, 0xFF1E5140, 0xFF1A4D3C, 0xFF164938, 0xFF1C291F, 0xFF0F1C12, 0xFF121F15, 0xFF0C190F, 0xFF19261C, 0xFF132016, 0xFF121F15, 0xFF1C291F, 0xFF172919, 0xFF0E2010, 0xFF1B2D1D, 0xFF112313, 0xFF192B1B,
        0xFF1B2D1D, 0xFF1A2C1C, 0xFF102212, 0xFF0F2F22, 0xFF204436, 0xFF285342, 0xFF305F4D, 0xFF437260, 0xFF547F6E, 0xFF5D8173, 0xFF5B7B6E, 0xFF5C8774, 0xFF608574, 0xFF658576, 0xFF698B7D, 0xFF567C6D, 0x00719186, 0x00F7FFFF, 0x00FFFEFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFBFF, 0x0FEEF8F7, 0xFF56776E, 0xFF578374, 0xFF60897B, 0xFF5E8274, 0xFF5E8274, 0xFF578070, 0xFF5F8075, 0xFF4D726A, 0xFF3E675F, 0xFF224C48, 0xFF1A413E,
        0xFF18322F, 0xFF192928, 0xFF151E1B, 0xFF162623, 0xFF162623, 0xFF132320, 0xFF172724, 0xFF1A2A27, 0xFF152522, 0xFF10201D, 0xFF172724, 0xFF1B2526, 0xFF1C2627, 0xFF192324, 0xFF1A2425, 0xFF182223, 0xFF131D1E, 0xFF172122, 0xFF192324, 0xFF113A32,
        0xFF133E35, 0xFF144438, 0xFF225747, 0xFF3B6E5F, 0xFF487868, 0xFF4E786A, 0xFF587E6F, 0xFF558172, 0xFF568273, 0xFF578374, 0xFF598576, 0xFF5A8677, 0xFF5A8677, 0xFF5B8778, 0xFF5B8778, 0xFF5B8776, 0xFF5B8776, 0xFF5A8675, 0xFF588473, 0xFF558170,
        0xFF527E6D, 0xFF4F7B6A, 0xFF4D7968, 0xFF376D60, 0xFF225449, 0xFF174137, 0xFF11322B, 0xFF102520, 0xFF162220, 0xFF191D1E, 0xFF191919, 0xFF182022, 0xFF131B1D, 0xFF121A1C, 0xFF182022, 0xFF0E1618, 0xFF131B1D, 0xFF141C1E, 0xFF141C1E, 0xFF181C1D,
        0xFF181C1D, 0xFF1A1E1F, 0xFF111516, 0xFF121B1A, 0xFF0F1F1C, 0xFF142B25, 0xFF102B24, 0xFF124639, 0xFF194D40, 0xFF2E6353, 0xFF437667, 0xFF497969, 0xFF4F7B6A, 0xFF537E6D, 0xFF537C6A, 0xFF52816F, 0xFF538270, 0xFF568271, 0xFF578372, 0xFF588473,
        0xFF598473, 0xFF588372, 0xFF588372, 0xFF5D8877, 0xFF578271, 0xFF55806F, 0xFF55806F, 0xFF527D6C, 0xFF4F7A69, 0xFF436E5D, 0xFF305B4A, 0xFF144D3C, 0xFF124033, 0xFF0E2C24, 0xFF131F1D, 0xFF151618, 0xFF1B1B1D, 0xFF161719, 0xFF1B2121, 0xFF151E19,
        0xFF101914, 0xFF0F1813, 0xFF121B16, 0xFF19221D, 0xFF141D18, 0xFF0F1813, 0xFF151E19, 0xFF151B17, 0xFF1B201C, 0xFF1B1B19, 0xFF171916, 0xFF101D16, 0xFF0F2D21, 0xFF103F2D, 0xFF144E38, 0xFF3B6152, 0xFF496F60, 0xFF517768, 0xFF547A6B, 0xFF5A8071,
        0xFF5E8475, 0xFF5E8475, 0xFF608677, 0xFF5C8573, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5C8573, 0xFF5A8371, 0xFF598270, 0xFF567D6B, 0xFF59806E, 0xFF547D6B, 0xFF507B6A, 0xFF467565, 0xFF306050, 0xFF184B3C, 0xFF114636, 0xFF153029,
        0xFF112822, 0xFF12211E, 0xFF121B1A, 0xFF161719, 0xFF1A1A1C, 0xFF1A1A1C, 0xFF17181A, 0xFF111516, 0xFF161A1B, 0xFF131718, 0xFF111516, 0xFF181C1D, 0xFF121617, 0xFF1A1E1F, 0xFF141819, 0xFF1C1616, 0xFF161614, 0xFF0D1A13, 0xFF0D271C, 0xFF15392B,
        0xFF1C4837, 0xFF2E5F4C, 0xFF497C69, 0xFF527967, 0xFF567D6B, 0xFF5A816F, 0xFF5D8472, 0xFF5D8472, 0xFF5C8371, 0xFF5D8472, 0xFF5D8472, 0xFF5E8573, 0xFF5E8573, 0xFF5F8674, 0xFF5F8674, 0xFF5E8573, 0xFF5E8573, 0xFF5D8472, 0xFF5C8371, 0xFF608073,
        0xFF567C6D, 0xFF4C7867, 0xFF386B5A, 0xFF215646, 0xFF174A3B, 0xFF17473B, 0xFF144237, 0xFF162B22, 0xFF13281F, 0xFF162B22, 0xFF12271E, 0xFF1B3027, 0xFF12271E, 0xFF0C2118, 0xFF12271E, 0xFF122D1C, 0xFF0D2817, 0xFF132E1D, 0xFF0D2817, 0xFF0F2A19,
        0xFF122D1C, 0xFF15301F, 0xFF122D1C, 0xFF15332B, 0xFF17382F, 0xFF1B4338, 0xFF255144, 0xFF3A6659, 0xFF51796E, 0xFF5C7D74, 0xFF628078, 0xFF5D8875, 0xFF618675, 0xFF668677, 0xFF6A8C7E, 0xFF557B6C, 0x006F8F84, 0x00F7FFFF, 0x00FFFEFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFBFF, 0x0BEFF8F7, 0xFF57766E, 0xFF588274, 0xFF618A7C, 0xFF608476, 0xFF608476, 0xFF598272, 0xFF607F79, 0xFF4D6D68, 0xFF4D6D6C, 0xFF3F5D5F, 0xFF415D60,
        0xFF465B5E, 0xFF455757, 0xFF475555, 0xFF4C6164, 0xFF455A5D, 0xFF44595C, 0xFF4A5F62, 0xFF43585B, 0xFF44595C, 0xFF44595C, 0xFF44595C, 0xFF42555B, 0xFF475A60, 0xFF45585E, 0xFF44575D, 0xFF43565C, 0xFF3F5258, 0xFF3F5258, 0xFF44575D, 0xFF3A5453,
        0xFF2F4F4C, 0xFF1C453D, 0xFF1E4E42, 0xFF36695A, 0xFF487868, 0xFF4E7968, 0xFF557B6C, 0xFF568072, 0xFF578173, 0xFF588274, 0xFF5A8476, 0xFF5B8577, 0xFF5B8577, 0xFF5B8577, 0xFF5B8577, 0xFF5C8776, 0xFF5C8776, 0xFF5C8776, 0xFF5A8574, 0xFF578271,
        0xFF547F6E, 0xFF507B6A, 0xFF4E7968, 0xFF406762, 0xFF3D615D, 0xFF3B5957, 0xFF3E5855, 0xFF405453, 0xFF445252, 0xFF4C5657, 0xFF4A5254, 0xFF455759, 0xFF425456, 0xFF3F5153, 0xFF445658, 0xFF425456, 0xFF3E5052, 0xFF3C4E50, 0xFF46585A, 0xFF435151,
        0xFF424E4E, 0xFF475152, 0xFF464E50, 0xFF414B4C, 0xFF3B4949, 0xFF3B4F4D, 0xFF384E4B, 0xFF2A4E4A, 0xFF214A42, 0xFF305B51, 0xFF447466, 0xFF4B7B6B, 0xFF517D6C, 0xFF57826F, 0xFF58816F, 0xFF538270, 0xFF558170, 0xFF578372, 0xFF588473, 0xFF5A8574,
        0xFF5A8574, 0xFF5A8373, 0xFF598272, 0xFF5C8575, 0xFF557E6E, 0xFF557E6E, 0xFF557E6E, 0xFF537C6C, 0xFF527B6B, 0xFF477060, 0xFF305949, 0xFF1E483E, 0xFF264942, 0xFF2A433F, 0xFF324040, 0xFF393E41, 0xFF3C4144, 0xFF383D40, 0xFF31393B, 0xFF324541,
        0xFF314440, 0xFF30433F, 0xFF293C38, 0xFF324541, 0xFF2E413D, 0xFF2A3D39, 0xFF334642, 0xFF313B3A, 0xFF394241, 0xFF383E3E, 0xFF353E3D, 0xFF303F3C, 0xFF2E4741, 0xFF264A40, 0xFF234F42, 0xFF3D6354, 0xFF4D7364, 0xFF577D6E, 0xFF587E6F, 0xFF5C8273,
        0xFF5F8576, 0xFF5E8475, 0xFF608677, 0xFF5C8573, 0xFF5C8573, 0xFF5D8674, 0xFF5D8674, 0xFF5C8573, 0xFF5C8573, 0xFF5B8472, 0xFF5B8472, 0xFF5A816E, 0xFF5B826F, 0xFF567F6D, 0xFF537C6C, 0xFF497365, 0xFF305A4E, 0xFF1D483E, 0xFF224D44, 0xFF233C38,
        0xFF263A38, 0xFF2E3C3C, 0xFF30383A, 0xFF33383B, 0xFF33383B, 0xFF2E3336, 0xFF2F3739, 0xFF364042, 0xFF313B3D, 0xFF313B3D, 0xFF303A3C, 0xFF2E383A, 0xFF2C3638, 0xFF323C3E, 0xFF2B3537, 0xFF303032, 0xFF2D3132, 0xFF283734, 0xFF273E38, 0xFF26453D,
        0xFF274B41, 0xFF345A4F, 0xFF476D62, 0xFF527967, 0xFF567D6B, 0xFF5B8270, 0xFF5D8472, 0xFF5D8472, 0xFF5C8371, 0xFF5D8472, 0xFF5E8573, 0xFF5E8573, 0xFF5F8674, 0xFF5F8674, 0xFF5F8674, 0xFF5E8573, 0xFF5D8472, 0xFF5C8371, 0xFF5C8371, 0xFF5F8173,
        0xFF547D6D, 0xFF4A7969, 0xFF356859, 0xFF1D4F43, 0xFF1A483E, 0xFF274B47, 0xFF294946, 0xFF2B463F, 0xFF2A453E, 0xFF2A453E, 0xFF254039, 0xFF29443D, 0xFF28433C, 0xFF29443D, 0xFF29443D, 0xFF26443A, 0xFF28463C, 0xFF2B493F, 0xFF2B493F, 0xFF28463C,
        0xFF2B493F, 0xFF2C4A40, 0xFF2D4B41, 0xFF2F4A45, 0xFF294641, 0xFF2E4D47, 0xFF2F5049, 0xFF3C5D56, 0xFF4D6C66, 0xFF55726D, 0xFF68837E, 0xFF5F8A77, 0xFF628776, 0xFF668677, 0xFF6A8C7E, 0xFF547A6B, 0x006D8D82, 0x00F5FFFD, 0x00FFFEFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFAFF, 0x0FEEF7F6, 0xFF56756D, 0xFF588274, 0xFF60897B, 0xFF638577, 0xFF618577, 0xFF5B8474, 0xFF55746F, 0xFF46605F, 0xFF586D72, 0xFF55666E, 0xFF5A6B73,
        0xFF5E7177, 0xFF5A7272, 0xFF5B7572, 0xFF576A6E, 0xFF55686C, 0xFF53666A, 0xFF55686C, 0xFF475A5E, 0xFF4C5F63, 0xFF4C5F63, 0xFF4B5E62, 0xFF4C6166, 0xFF52676C, 0xFF5A6F74, 0xFF5D7277, 0xFF5E7378, 0xFF5B7075, 0xFF576C71, 0xFF576C71, 0xFF667079,
        0xFF5D7074, 0xFF3F5F5C, 0xFF2A554C, 0xFF36695A, 0xFF4B7B6B, 0xFF537E6D, 0xFF577E6C, 0xFF578173, 0xFF588274, 0xFF598375, 0xFF5A8476, 0xFF5B8577, 0xFF5B8577, 0xFF5B8577, 0xFF5B8577, 0xFF5B8675, 0xFF5B8675, 0xFF5B8675, 0xFF5A8574, 0xFF588372,
        0xFF547F6E, 0xFF517C6B, 0xFF4E7968, 0xFF4A5F62, 0xFF56696D, 0xFF55686C, 0xFF5A6E6F, 0xFF5C7071, 0xFF556968, 0xFF5C6E6E, 0xFF607272, 0xFF4A5D61, 0xFF4A5D61, 0xFF475A5E, 0xFF495C60, 0xFF4B5E62, 0xFF475A5E, 0xFF45585C, 0xFF4A5D61, 0xFF4A6461,
        0xFF4D6361, 0xFF536565, 0xFF5B696A, 0xFF5E6C6D, 0xFF5C6A6B, 0xFF5C6E6E, 0xFF617574, 0xFF52676A, 0xFF3B5554, 0xFF395C56, 0xFF4B756B, 0xFF4E7D6D, 0xFF4E7D6B, 0xFF537E6B, 0xFF547E6A, 0xFF558170, 0xFF568271, 0xFF588473, 0xFF5B8675, 0xFF5B8675,
        0xFF5C8575, 0xFF5B8474, 0xFF5A8373, 0xFF5C8575, 0xFF557E6E, 0xFF567F6F, 0xFF578070, 0xFF537C6C, 0xFF557E6E, 0xFF4A7363, 0xFF325B4B, 0xFF4C6260, 0xFF546A68, 0xFF556968, 0xFF5E7070, 0xFF5E6E6E, 0xFF5E6E6E, 0xFF647474, 0xFF586868, 0xFF516566,
        0xFF4C6061, 0xFF516566, 0xFF45595A, 0xFF495D5E, 0xFF4A5E5F, 0xFF4B5F60, 0xFF506465, 0xFF516365, 0xFF5A6C6E, 0xFF57696B, 0xFF5B6D6F, 0xFF5B6F70, 0xFF5E7374, 0xFF526A6A, 0xFF56706F, 0xFF3D6153, 0xFF507466, 0xFF5A7E70, 0xFF5B7F71, 0xFF5E8274,
        0xFF608476, 0xFF5F8375, 0xFF608476, 0xFF5C8573, 0xFF5C8573, 0xFF5C8573, 0xFF5C8573, 0xFF5C8573, 0xFF5D8674, 0xFF5D8674, 0xFF5E8775, 0xFF5B856F, 0xFF5B8571, 0xFF577E6C, 0xFF557B6C, 0xFF4C7066, 0xFF34554E, 0xFF2B4B46, 0xFF3C5C59, 0xFF4E6462,
        0xFF526665, 0xFF5A6A6A, 0xFF596466, 0xFF5C6769, 0xFF5C6A6B, 0xFF516161, 0xFF506262, 0xFF526466, 0xFF4C5E60, 0xFF506264, 0xFF506264, 0xFF4F6163, 0xFF526466, 0xFF536567, 0xFF506264, 0xFF5A6368, 0xFF576568, 0xFF586A6C, 0xFF536B6B, 0xFF4B6564,
        0xFF4A6463, 0xFF4B6363, 0xFF495E5F, 0xFF537A68, 0xFF577E6C, 0xFF5C8371, 0xFF5D8472, 0xFF5D8472, 0xFF5C8371, 0xFF5E8573, 0xFF608775, 0xFF5F8674, 0xFF5F8674, 0xFF5F8674, 0xFF5F8674, 0xFF5E8573, 0xFF5D8472, 0xFF5C8371, 0xFF5B8270, 0xFF597E6D,
        0xFF507B6A, 0xFF487868, 0xFF36695A, 0xFF235248, 0xFF2F534F, 0xFF4A6465, 0xFF55666D, 0xFF546E6D, 0xFF546E6D, 0xFF506A69, 0xFF4E6867, 0xFF4D6766, 0xFF486261, 0xFF405A59, 0xFF395352, 0xFF3F5759, 0xFF425A5C, 0xFF41595B, 0xFF486062, 0xFF445C5E,
        0xFF455D5F, 0xFF445C5E, 0xFF486062, 0xFF506A67, 0xFF4A6360, 0xFF4F6563, 0xFF4A5E5D, 0xFF4E6261, 0xFF516765, 0xFF4A6360, 0xFF5F7976, 0xFF5F8A77, 0xFF618675, 0xFF658576, 0xFF6A8C7E, 0xFF53796A, 0x006B8B80, 0x00F5FFFD, 0x00FFFEFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFF8FF, 0x0FEDF6F5, 0xFF55746C, 0xFF578173, 0xFF62887B, 0xFF638577, 0xFF628678, 0xFF5F8576, 0xFF5B7A75, 0xFF496163, 0xFF5E6D74, 0xFF535D69, 0xFF48525E,
        0xFF3B4C53, 0xFF284445, 0xFF22453F, 0xFF40504F, 0xFF50605F, 0xFF50605F, 0xFF516160, 0xFF485857, 0xFF485857, 0xFF3E4E4D, 0xFF415150, 0xFF4E6261, 0xFF495D5C, 0xFF475B5A, 0xFF405453, 0xFF384C4B, 0xFF394D4C, 0xFF3A4E4D, 0xFF3A4E4D, 0xFF565662,
        0xFF606A73, 0xFF4C6665, 0xFF305951, 0xFF38685A, 0xFF4E7E6E, 0xFF578372, 0xFF598270, 0xFF578173, 0xFF588274, 0xFF598375, 0xFF5A8476, 0xFF5A8476, 0xFF5B8577, 0xFF5A8476, 0xFF5A8476, 0xFF598473, 0xFF5A8574, 0xFF5A8574, 0xFF5A8574, 0xFF588372,
        0xFF547F6E, 0xFF517C6B, 0xFF4E7968, 0xFF535C65, 0xFF5F6972, 0xFF505D63, 0xFF49585D, 0xFF415556, 0xFF324748, 0xFF36504D, 0xFF425C59, 0xFF506168, 0xFF4A5B62, 0xFF4B5C63, 0xFF495A61, 0xFF43545B, 0xFF46575E, 0xFF47585F, 0xFF3D4E55, 0xFF3C5F5B,
        0xFF557371, 0xFF546E6D, 0xFF405455, 0xFF3C4B4E, 0xFF425154, 0xFF415355, 0xFF405254, 0xFF687379, 0xFF45575B, 0xFF3C5957, 0xFF4C746B, 0xFF537F70, 0xFF51806E, 0xFF57826F, 0xFF59836F, 0xFF568271, 0xFF578372, 0xFF5A8574, 0xFF5C8776, 0xFF5C8776,
        0xFF5D8676, 0xFF5B8474, 0xFF5B8474, 0xFF5F8878, 0xFF588171, 0xFF598272, 0xFF588171, 0xFF537C6C, 0xFF557E6E, 0xFF4A7363, 0xFF315A4A, 0xFF606B6F, 0xFF59676A, 0xFF46585A, 0xFF435859, 0xFF3D5253, 0xFF44595A, 0xFF637879, 0xFF607475, 0xFF586970,
        0xFF4A5B62, 0xFF52636A, 0xFF46575E, 0xFF46575E, 0xFF4A5B62, 0xFF4E5F66, 0xFF495A61, 0xFF4D6569, 0xFF4E666A, 0xFF415A5E, 0xFF3E575B, 0xFF41595D, 0xFF4F6268, 0xFF516067, 0xFF67747C, 0xFF385C4E, 0xFF4C7062, 0xFF587C6E, 0xFF597D6F, 0xFF5C8072,
        0xFF5E8274, 0xFF5E8274, 0xFF5F8375, 0xFF5C8573, 0xFF5C8573, 0xFF5B8472, 0xFF5B8472, 0xFF5C8573, 0xFF5D8674, 0xFF5F8876, 0xFF5F8876, 0xFF5A866F, 0xFF59836F, 0xFF567D6B, 0xFF567A6E, 0xFF527169, 0xFF3B5854, 0xFF3D5756, 0xFF5A7272, 0xFF526768,
        0xFF4D6162, 0xFF4C5E60, 0xFF4B5A5D, 0xFF5A696C, 0xFF647879, 0xFF506868, 0xFF3E5B59, 0xFF415B5C, 0xFF3E5859, 0xFF3D5758, 0xFF395354, 0xFF3D5758, 0xFF455F60, 0xFF435D5E, 0xFF455F60, 0xFF5F7077, 0xFF52656B, 0xFF4E6368, 0xFF4D6569, 0xFF53686D,
        0xFF607379, 0xFF5E6D74, 0xFF48525B, 0xFF547B69, 0xFF587F6D, 0xFF5C8371, 0xFF5E8573, 0xFF5C8371, 0xFF5C8371, 0xFF5E8573, 0xFF608775, 0xFF5F8674, 0xFF5F8674, 0xFF5F8674, 0xFF5F8674, 0xFF5E8573, 0xFF5D8472, 0xFF5C8371, 0xFF5B8270, 0xFF5D8472,
        0xFF547F6E, 0xFF4A7A6A, 0xFF35675B, 0xFF225046, 0xFF345453, 0xFF586970, 0xFF68707B, 0xFF43585D, 0xFF43585D, 0xFF41565B, 0xFF485D62, 0xFF52676C, 0xFF51666B, 0xFF4B6065, 0xFF455A5F, 0xFF4A5863, 0xFF485661, 0xFF3F4D58, 0xFF495762, 0xFF475560,
        0xFF4D5B66, 0xFF515F6A, 0xFF5D6B76, 0xFF486563, 0xFF465B5C, 0xFF516063, 0xFF535E62, 0xFF5E696D, 0xFF5C6B6E, 0xFF485D5E, 0xFF587573, 0xFF5F8A77, 0xFF618675, 0xFF648475, 0xFF698B7D, 0xFF53796A, 0x006B8B80, 0x00F6FFFE, 0x00FFFEFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x17E9F5F1, 0xFF52766A, 0xFF578676, 0xFF5F897B, 0xFF64887C, 0xFF698D81, 0xFF5A8375, 0xFF5A6F70, 0xFF4A5F64, 0xFF516570, 0xFF445A67, 0xFF3D535E,
        0xFF475E64, 0xFF304A47, 0xFF18332C, 0xFF1B393B, 0xFF234044, 0xFF2F4A51, 0xFF334B55, 0xFF314850, 0xFF374B52, 0xFF374A4E, 0xFF2B3F3E, 0xFF234745, 0xFF153935, 0xFF11342E, 0xFF1A3A35, 0xFF133330, 0xFF2F4D4F, 0xFF3B565D, 0xFF2E4954, 0xFF3A4F52,
        0xFF4D696A, 0xFF3D605C, 0xFF37605A, 0xFF3E6960, 0xFF4D766E, 0xFF557A73, 0xFF63847D, 0xFF5F887A, 0xFF567F71, 0xFF5B8476, 0xFF5D8779, 0xFF588274, 0xFF5A8677, 0xFF5E8A7B, 0xFF5D897A, 0xFF5C8371, 0xFF5C8573, 0xFF588370, 0xFF5B8776, 0xFF507C6B,
        0xFF638E7D, 0xFF547A6D, 0xFF517267, 0xFF4F6361, 0xFF536768, 0xFF3D5257, 0xFF374E56, 0xFF3C525D, 0xFF344C56, 0xFF1D3841, 0xFF2C4750, 0xFF4A5B62, 0xFF405359, 0xFF334A50, 0xFF2D484F, 0xFF304D51, 0xFF345051, 0xFF324B47, 0xFF2E453F, 0xFF3D544E,
        0xFF3E5753, 0xFF324C4B, 0xFF1E3C3E, 0xFF2A474B, 0xFF39545B, 0xFF314D51, 0xFF385156, 0xFF5E6263, 0xFF445857, 0xFF355C5B, 0xFF467271, 0xFF557D7C, 0xFF5D807A, 0xFF64887A, 0xFF5B8571, 0xFF578372, 0xFF578372, 0xFF588473, 0xFF588473, 0xFF598574,
        0xFF598574, 0xFF5A8675, 0xFF5A8675, 0xFF638574, 0xFF628776, 0xFF5B8472, 0xFF527E6D, 0xFF5D897A, 0xFF507C6F, 0xFF446E64, 0xFF376157, 0xFF566465, 0xFF425456, 0xFF334B4F, 0xFF355155, 0xFF253E45, 0xFF1E3239, 0xFF4D5C63, 0xFF7C868F, 0xFF818689,
        0xFF727C7E, 0xFF5D6C71, 0xFF4C6166, 0xFF455D61, 0xFF445E5F, 0xFF425B58, 0xFF415754, 0xFF2D5354, 0xFF1E4143, 0xFF193A3F, 0xFF2F4C52, 0xFF304B52, 0xFF2F474B, 0xFF33484B, 0xFF536768, 0xFF425961, 0xFF55726D, 0xFF5C7F6B, 0xFF567D68, 0xFF5B8476,
        0xFF5F897F, 0xFF588173, 0xFF557F69, 0xFF5B8472, 0xFF5B8472, 0xFF5B8472, 0xFF5B8472, 0xFF5B8472, 0xFF5A8371, 0xFF5A8371, 0xFF5A8371, 0xFF597F74, 0xFF5A7B70, 0xFF5C7E70, 0xFF57826F, 0xFF427562, 0xFF315F52, 0xFF3C5957, 0xFF586369, 0xFF385052,
        0xFF304C4F, 0xFF304F52, 0xFF294B4D, 0xFF1E4041, 0xFF284445, 0xFF3D5150, 0xFF455554, 0xFF39534A, 0xFF3B5450, 0xFF3A5453, 0xFF375054, 0xFF334F53, 0xFF345155, 0xFF39585A, 0xFF3D5D5C, 0xFF2B5152, 0xFF25484A, 0xFF26474C, 0xFF304B52, 0xFF395056,
        0xFF405359, 0xFF576669, 0xFF435152, 0xFF4E7465, 0xFF5A8373, 0xFF578372, 0xFF5C8B77, 0xFF5D8A76, 0xFF58836F, 0xFF628976, 0xFF5F8472, 0xFF5F8A77, 0xFF5F8876, 0xFF598270, 0xFF5E8775, 0xFF5F8674, 0xFF567D6B, 0xFF5A7F6E, 0xFF5D8271, 0xFF588477,
        0xFF547F75, 0xFF4F7A70, 0xFF396159, 0xFF30504D, 0xFF455A5D, 0xFF4E5B63, 0xFF4F5861, 0xFF384B5A, 0xFF39515B, 0xFF395759, 0xFF284B45, 0xFF2D5049, 0xFF36534E, 0xFF445A57, 0xFF3F5151, 0xFF394D4E, 0xFF364B4E, 0xFF30494E, 0xFF2D4A52, 0xFF335058,
        0xFF39545B, 0xFF395257, 0xFF334B4F, 0xFF2F5047, 0xFF33524D, 0xFF3C5559, 0xFF394D56, 0xFF3D515A, 0xFF556A6F, 0xFF4B6562, 0xFF56746C, 0xFF6C8C7F, 0xFF5B8474, 0xFF598A77, 0xFF5B8E7B, 0xFF55806F, 0x006E887D, 0x00F9FEFA, 0x00FFFCFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFF8FB, 0x17EAF5F1, 0xFF56776C, 0xFF598375, 0xFF60897B, 0xFF67887D, 0xFF618277, 0xFF446A5D, 0xFF425758, 0xFF465B60, 0xFF546871, 0xFF4A606B, 0xFF3F5560,
        0xFF4B6469, 0xFF455F5E, 0xFF1F3A35, 0xFF2F4D45, 0xFF314E4A, 0xFF355152, 0xFF345053, 0xFF2D494D, 0xFF2D494C, 0xFF2F4C4A, 0xFF2B4844, 0xFF1B3F3F, 0xFF193B3A, 0xFF173A36, 0xFF21413C, 0xFF23423D, 0xFF395556, 0xFF415A5F, 0xFF3B535D, 0xFF516063,
        0xFF5F7173, 0xFF47605D, 0xFF2A4944, 0xFF1C3F38, 0xFF234840, 0xFF274D42, 0xFF3D6358, 0xFF527B6D, 0xFF648D7F, 0xFF5C8577, 0xFF578173, 0xFF5B8577, 0xFF598576, 0xFF5C8879, 0xFF588475, 0xFF598270, 0xFF638E7B, 0xFF547F6C, 0xFF588372, 0xFF668F7F,
        0xFF4B6F63, 0xFF38564C, 0xFF365047, 0xFF55696A, 0xFF5D7172, 0xFF475C61, 0xFF3F565C, 0xFF425B60, 0xFF3F585D, 0xFF2A464A, 0xFF355256, 0xFF415651, 0xFF38514E, 0xFF2E4B49, 0xFF254544, 0xFF1E3E3D, 0xFF1E3C3A, 0xFF2A4742, 0xFF39544D, 0xFF385249,
        0xFF3E5751, 0xFF3A5451, 0xFF294546, 0xFF334F52, 0xFF445D62, 0xFF41585E, 0xFF42595F, 0xFF5A6567, 0xFF47615E, 0xFF2C534E, 0xFF1E4942, 0xFF294E47, 0xFF375B51, 0xFF487161, 0xFF5B8C77, 0xFF578372, 0xFF588473, 0xFF588473, 0xFF598574, 0xFF598574,
        0xFF5A8675, 0xFF5A8675, 0xFF5A8675, 0xFF608774, 0xFF4F7664, 0xFF5E8775, 0xFF588171, 0xFF3E6759, 0xFF294F44, 0xFF2C4F48, 0xFF2E4F48, 0xFF5A686B, 0xFF47595B, 0xFF405558, 0xFF496266, 0xFF3F585C, 0xFF374C4F, 0xFF596B6F, 0xFF7E8C8F, 0xFF8C908F,
        0xFF747C7E, 0xFF566465, 0xFF415458, 0xFF384D50, 0xFF354B49, 0xFF394D4B, 0xFF3E534C, 0xFF335352, 0xFF284749, 0xFF264446, 0xFF395559, 0xFF3F585D, 0xFF40585C, 0xFF3E5356, 0xFF54696C, 0xFF4E626B, 0xFF37504C, 0xFF2E4D3B, 0xFF3B5E4A, 0xFF4E7266,
        0xFF5D827A, 0xFF62887B, 0xFF5F8671, 0xFF5C8573, 0xFF5C8573, 0xFF5C8573, 0xFF5B8472, 0xFF5B8472, 0xFF5B8472, 0xFF5B8472, 0xFF5A8371, 0xFF598176, 0xFF698A7F, 0xFF4E6C60, 0xFF375949, 0xFF295443, 0xFF285045, 0xFF476160, 0xFF5B686E, 0xFF4C6166,
        0xFF374F53, 0xFF3D5B5D, 0xFF3C5B5D, 0xFF224040, 0xFF2B4544, 0xFF425654, 0xFF384845, 0xFF3D544A, 0xFF374E46, 0xFF354B48, 0xFF394E4F, 0xFF3B5051, 0xFF344A48, 0xFF2B423C, 0xFF273E36, 0xFF2D504C, 0xFF284847, 0xFF2D4B4D, 0xFF385457, 0xFF41595D,
        0xFF44595C, 0xFF586C6D, 0xFF46585A, 0xFF355347, 0xFF3E5E51, 0xFF496D5F, 0xFF598471, 0xFF598471, 0xFF5B8672, 0xFF66907C, 0xFF527966, 0xFF5A8572, 0xFF5D8674, 0xFF598270, 0xFF5D8674, 0xFF5E8573, 0xFF59806E, 0xFF5F8473, 0xFF5E8372, 0xFF517A6C,
        0xFF3A6458, 0xFF355F53, 0xFF2F574E, 0xFF2B4E48, 0xFF3C5957, 0xFF556A6D, 0xFF5E7074, 0xFF4E606C, 0xFF44585F, 0xFF3A5657, 0xFF30504B, 0xFF33544D, 0xFF2C4944, 0xFF324846, 0xFF344849, 0xFF324B47, 0xFF354F4C, 0xFF355152, 0xFF304E50, 0xFF2C4A4A,
        0xFF2F4948, 0xFF334946, 0xFF334841, 0xFF395850, 0xFF425C59, 0xFF4A5F64, 0xFF465960, 0xFF4D5E66, 0xFF607379, 0xFF495F5D, 0xFF4E6761, 0xFF5B756A, 0xFF5F7F72, 0xFF648F7E, 0xFF5E8D7B, 0xFF4F7868, 0x006B857A, 0x00FBFFFC, 0x00FFFCFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFBFF, 0x0FEEF7F4, 0xFF547268, 0xFF567F71, 0xFF688C80, 0xFF66847A, 0xFF4C675E, 0xFF26463B, 0xFF314546, 0xFF4E6368, 0xFF4D6168, 0xFF496068, 0xFF5E777E,
        0xFF577377, 0xFF3F5B5C, 0xFF264341, 0xFF2F4B3F, 0xFF2D4B43, 0xFF33514F, 0xFF3B5A5C, 0xFF3B5C61, 0xFF3D6064, 0xFF426569, 0xFF416768, 0xFF2C4F55, 0xFF2B4D4F, 0xFF20403F, 0xFF1B3937, 0xFF203D39, 0xFF3D5756, 0xFF566E70, 0xFF687D82, 0xFF647070,
        0xFF667272, 0xFF465454, 0xFF2A3A37, 0xFF182D28, 0xFF183229, 0xFF113126, 0xFF1F4337, 0xFF365F51, 0xFF648D7F, 0xFF5A8375, 0xFF547E70, 0xFF5E887A, 0xFF588475, 0xFF5C8879, 0xFF598576, 0xFF5B8874, 0xFF5A8773, 0xFF598473, 0xFF608979, 0xFF55766B,
        0xFF375148, 0xFF798C86, 0xFF94A09E, 0xFF4F6165, 0xFF506367, 0xFF4B6063, 0xFF637B7D, 0xFF688281, 0xFF466361, 0xFF203E3C, 0xFF294946, 0xFF2C473E, 0xFF26443C, 0xFF284B47, 0xFF365A5A, 0xFF3B6162, 0xFF385B5D, 0xFF325455, 0xFF345252, 0xFF39574F,
        0xFF3C5752, 0xFF38534E, 0xFF2F4747, 0xFF3A5254, 0xFF576C71, 0xFF63777E, 0xFF667882, 0xFF627677, 0xFF3A5753, 0xFF15392F, 0xFF052719, 0xFF153123, 0xFF213D2F, 0xFF264F3F, 0xFF4B806C, 0xFF588473, 0xFF588473, 0xFF588473, 0xFF598574, 0xFF598574,
        0xFF5A8675, 0xFF5A8675, 0xFF5B8776, 0xFF57826F, 0xFF618A78, 0xFF63897A, 0xFF3A5B50, 0xFF28463E, 0xFF26413C, 0xFF314744, 0xFF425856, 0xFF56656A, 0xFF4C5E62, 0xFF4F6465, 0xFF556F6E, 0xFF3F5958, 0xFF28413E, 0xFF3B514F, 0xFF556968, 0xFF757F7E,
        0xFF637172, 0xFF526466, 0xFF4A6266, 0xFF445D61, 0xFF3C5454, 0xFF39524F, 0xFF3E554F, 0xFF3A5453, 0xFF2E4847, 0xFF2B4544, 0xFF405A59, 0xFF526C6D, 0xFF5E7678, 0xFF51696D, 0xFF5B7377, 0xFF404E59, 0xFF576A68, 0xFF718878, 0xFF607B6A, 0xFF3F5A51,
        0xFF486761, 0xFF64857A, 0xFF5F8570, 0xFF5D8674, 0xFF5C8573, 0xFF5C8573, 0xFF5C8573, 0xFF5C8573, 0xFF5B8472, 0xFF5B8472, 0xFF5B8472, 0xFF5E887C, 0xFF4D6B61, 0xFF2E4239, 0xFF283C31, 0xFF183428, 0xFF113028, 0xFF3E5656, 0xFF5D6C71, 0xFF596C73,
        0xFF5B6F76, 0xFF577074, 0xFF466263, 0xFF2F4C4A, 0xFF294340, 0xFF364C49, 0xFF425654, 0xFF445F58, 0xFF3E5954, 0xFF3F5855, 0xFF465B5C, 0xFF455957, 0xFF3C4F49, 0xFF394A40, 0xFF3C4E40, 0xFF294641, 0xFF2B4843, 0xFF405D59, 0xFF5B7572, 0xFF5F7978,
        0xFF506A6B, 0xFF556D6F, 0xFF3D5559, 0xFF6D7C77, 0xFF63776E, 0xFF3E5A4E, 0xFF486A5C, 0xFF648D7B, 0xFF5A8572, 0xFF55806C, 0xFF618B77, 0xFF588370, 0xFF5D8674, 0xFF5A8371, 0xFF5C8573, 0xFF5E8573, 0xFF5D8472, 0xFF5F8473, 0xFF557A69, 0xFF365244,
        0xFF264438, 0xFF1E4032, 0xFF113529, 0xFF10342A, 0xFF385751, 0xFF647E7B, 0xFF5C7572, 0xFF697780, 0xFF5A6D71, 0xFF465E5E, 0xFF324F4A, 0xFF2F4C47, 0xFF264341, 0xFF31494B, 0xFF43575E, 0xFF4A6866, 0xFF4A6A69, 0xFF456768, 0xFF426261, 0xFF3F5D5B,
        0xFF3F5852, 0xFF3D5146, 0xFF384A3C, 0xFF2E4944, 0xFF425758, 0xFF5D6F73, 0xFF5E6D74, 0xFF57646C, 0xFF6A797E, 0xFF4A5C5E, 0xFF3F5351, 0xFF45524B, 0xFF566B62, 0xFF648678, 0xFF628D7C, 0xFF527869, 0x006F897E, 0x00F9FFFD, 0x00FFFEFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFAFE, 0x0BEFF8F5, 0xFF557369, 0xFF5D8376, 0xFF709085, 0xFF587269, 0xFF3A5149, 0xFF375249, 0xFF3C4F53, 0xFF4B5E62, 0xFF495E63, 0xFF3B5258, 0xFF435C61,
        0xFF2A474B, 0xFF1A383A, 0xFF29484A, 0xFF284543, 0xFF2C4A4C, 0xFF34555E, 0xFF3F606F, 0xFF42677A, 0xFF3E677B, 0xFF376273, 0xFF2E5C6C, 0xFF446876, 0xFF42646E, 0xFF36575E, 0xFF264547, 0xFF1C3937, 0xFF2A4242, 0xFF3F5455, 0xFF4D6064, 0xFF304741,
        0xFF51615E, 0xFF4D5756, 0xFF4D5353, 0xFF373D3B, 0xFF25302C, 0xFF2E3F37, 0xFF445950, 0xFF30594B, 0xFF537C6E, 0xFF598274, 0xFF588274, 0xFF5B8577, 0xFF578374, 0xFF5B8778, 0xFF5C8879, 0xFF5C8B79, 0xFF558472, 0xFF598574, 0xFF63897C, 0xFF436157,
        0xFF4D605A, 0xFF97A09F, 0xFF8A8B8D, 0xFF52636B, 0xFF44555C, 0xFF2F4344, 0xFF47605D, 0xFF4B6863, 0xFF2D4C46, 0xFF133630, 0xFF254842, 0xFF2D4F4E, 0xFF325557, 0xFF396067, 0xFF3B6571, 0xFF376171, 0xFF30596B, 0xFF365B6D, 0xFF3D6073, 0xFF406261,
        0xFF375754, 0xFF324F4D, 0xFF334947, 0xFF374B4C, 0xFF425458, 0xFF3F5057, 0xFF31444B, 0xFF516367, 0xFF445A58, 0xFF455C52, 0xFF304033, 0xFF343C2F, 0xFF465349, 0xFF39574B, 0xFF39675A, 0xFF588473, 0xFF588473, 0xFF598574, 0xFF598574, 0xFF5A8675,
        0xFF5A8675, 0xFF5A8675, 0xFF5B8776, 0xFF568772, 0xFF588473, 0xFF476B5D, 0xFF38534A, 0xFF465B56, 0xFF354544, 0xFF324041, 0xFF414F50, 0xFF4D5E65, 0xFF43565A, 0xFF455A5B, 0xFF4A635F, 0xFF37524D, 0xFF233E39, 0xFF2F4946, 0xFF455F5E, 0xFF4E6972,
        0xFF44606B, 0xFF395966, 0xFF355967, 0xFF355B68, 0xFF375B67, 0xFF3D5F68, 0xFF43646B, 0xFF3D5557, 0xFF304848, 0xFF29423F, 0xFF334C49, 0xFF3E5754, 0xFF455F5E, 0xFF364F53, 0xFF465F64, 0xFF4F5965, 0xFF626E6E, 0xFFAEBCAF, 0xFFB1C1B4, 0xFF4D625B,
        0xFF38534E, 0xFF6A8A7F, 0xFF618772, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5C8573, 0xFF5C8573, 0xFF5C8573, 0xFF5C8573, 0xFF4F7B6E, 0xFF435E55, 0xFF5F6A64, 0xFF545B54, 0xFF293831, 0xFF2E453F, 0xFF44595A, 0xFF4A5C60, 0xFF44555F,
        0xFF54676E, 0xFF4E6366, 0xFF3A5252, 0xFF2F4946, 0xFF223F3B, 0xFF264341, 0xFF3E5B59, 0xFF466F75, 0xFF3B6269, 0xFF365B64, 0xFF41626B, 0xFF4F6C72, 0xFF526C6D, 0xFF49625E, 0xFF425951, 0xFF354E48, 0xFF2E4741, 0xFF39524C, 0xFF4A655E, 0xFF4C6762,
        0xFF3E5B59, 0xFF4C686B, 0xFF3B575B, 0xFF9EA2A1, 0xFFB7C2BE, 0xFF6D807A, 0xFF355347, 0xFF55796B, 0xFF658E7C, 0xFF5A8572, 0xFF588370, 0xFF5B8673, 0xFF608977, 0xFF5D8674, 0xFF5D8674, 0xFF608775, 0xFF608775, 0xFF587D6C, 0xFF406554, 0xFF555C54,
        0xFF556058, 0xFF4A5A50, 0xFF2D4239, 0xFF1E382F, 0xFF38514B, 0xFF617774, 0xFF445856, 0xFF415053, 0xFF3E5052, 0xFF324645, 0xFF27403D, 0xFF314B4A, 0xFF395558, 0xFF45626A, 0xFF54707C, 0xFF456975, 0xFF456B78, 0xFF466C79, 0xFF466A76, 0xFF4B6971,
        0xFF4C6569, 0xFF485B59, 0xFF41504B, 0xFF364B4C, 0xFF3B4F50, 0xFF59686D, 0xFF59646A, 0xFF475057, 0xFF6A757B, 0xFF556467, 0xFF455759, 0xFF4F514E, 0xFF59665F, 0xFF607A6F, 0xFF688C7E, 0xFF567A6C, 0x006E887D, 0x00F8FFFD, 0x00FFFFFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFF9FC, 0x0BF0FBF7, 0xFF58786D, 0xFF5E8477, 0xFF6D8D82, 0xFF526961, 0xFF586D66, 0xFF9CB6AD, 0xFF819397, 0xFF495C60, 0xFF4B6063, 0xFF334D4E, 0xFF1C383B,
        0xFF173639, 0xFF1C3D42, 0xFF25484E, 0xFF355669, 0xFF496C82, 0xFF60839F, 0xFF6D93B7, 0xFF759FC5, 0xFF7AA6CD, 0xFF73A3C9, 0xFF6B9DC2, 0xFF4E7690, 0xFF476D82, 0xFF446876, 0xFF395A61, 0xFF2C4A4C, 0xFF2D4748, 0xFF364B4C, 0xFF324748, 0xFF284942,
        0xFF4D6863, 0xFF51615E, 0xFF818A89, 0xFF939496, 0xFF868686, 0xFF8C8E8D, 0xFF838786, 0xFF4D7668, 0xFF456E60, 0xFF588173, 0xFF5E887A, 0xFF578173, 0xFF5B8778, 0xFF5B8778, 0xFF5D897A, 0xFF5A8B78, 0xFF588775, 0xFF537F70, 0xFF63897C, 0xFF38534A,
        0xFF52625F, 0xFF727878, 0xFF666668, 0xFF4E5C67, 0xFF4A5B62, 0xFF2A3E3D, 0xFF263F3B, 0xFF24423A, 0xFF1F3F3A, 0xFF1C403E, 0xFF29504F, 0xFF42677A, 0xFF466F85, 0xFF507C97, 0xFF5E8CAD, 0xFF6E9CC0, 0xFF729EC5, 0xFF658FB5, 0xFF557CA3, 0xFF436D79,
        0xFF325861, 0xFF2E4D52, 0xFF385052, 0xFF415353, 0xFF425456, 0xFF384B4F, 0xFF26393F, 0xFF4B545D, 0xFF4F5A5C, 0xFF88918E, 0xFF93948E, 0xFF87807A, 0xFF8A8782, 0xFF62716A, 0xFF3A5B52, 0xFF588473, 0xFF588473, 0xFF588473, 0xFF598574, 0xFF598574,
        0xFF5A8675, 0xFF5A8675, 0xFF5A8675, 0xFF568976, 0xFF568271, 0xFF47685D, 0xFF49605A, 0xFF5E6E6D, 0xFF657171, 0xFF6D787A, 0xFF4E595D, 0xFF4F6268, 0xFF35484C, 0xFF2B413F, 0xFF324943, 0xFF314C47, 0xFF314E4C, 0xFF3C5B5E, 0xFF4D6C71, 0xFF335F78,
        0xFF3F6E88, 0xFF4B7E9B, 0xFF5388A7, 0xFF558AA9, 0xFF5285A2, 0xFF457790, 0xFF3A6A81, 0xFF425D66, 0xFF355057, 0xFF2B4748, 0xFF27413E, 0xFF213C37, 0xFF223C3B, 0xFF1B3438, 0xFF3F585D, 0xFF4D5562, 0xFF788082, 0xFF7C867B, 0xFF717B70, 0xFF606F6A,
        0xFF475D5A, 0xFF4B6B60, 0xFF648B76, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5C8573, 0xFF5C8573, 0xFF5C8573, 0xFF4B776A, 0xFF4D675E, 0xFFB1B7B3, 0xFFB1B2AD, 0xFF757E79, 0xFF647773, 0xFF45595A, 0xFF4F6165, 0xFF2E3F49,
        0xFF293C42, 0xFF304347, 0xFF314949, 0xFF294644, 0xFF304F52, 0xFF3B5E64, 0xFF3A5F68, 0xFF285E76, 0xFF3E748C, 0xFF5486A1, 0xFF55839D, 0xFF4A748C, 0xFF426B7D, 0xFF466C79, 0xFF496F78, 0xFF3F5757, 0xFF2C4542, 0xFF223B35, 0xFF243F36, 0xFF26413A,
        0xFF26433F, 0xFF446063, 0xFF3D585F, 0xFF5C5C5E, 0xFF838987, 0xFF65746F, 0xFF425C53, 0xFF4D6F61, 0xFF567F6F, 0xFF5A8572, 0xFF598471, 0xFF5C8774, 0xFF608977, 0xFF5D8674, 0xFF5F8876, 0xFF638A78, 0xFF628977, 0xFF547968, 0xFF305544, 0xFF7E7371,
        0xFF8E8684, 0xFF8B8C87, 0xFF828884, 0xFF6B7672, 0xFF4C5856, 0xFF596565, 0xFF3D4949, 0xFF2F3F3F, 0xFF304242, 0xFF2E4241, 0xFF344C4C, 0xFF425E62, 0xFF476772, 0xFF426676, 0xFF41657B, 0xFF4B7494, 0xFF557E9E, 0xFF5C85A5, 0xFF5B82A1, 0xFF577A96,
        0xFF567288, 0xFF546B7D, 0xFF526574, 0xFF485D60, 0xFF3D4F53, 0xFF475558, 0xFF4C555A, 0xFF454E53, 0xFF667175, 0xFF546167, 0xFF637277, 0xFF7A7977, 0xFF68716C, 0xFF597066, 0xFF698D7F, 0xFF5B7F71, 0x006B857A, 0x00F3FEF8, 0x00FFFFFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x13ECFBF6, 0xFF55766B, 0xFF5B8476, 0xFF6C8D82, 0xFF556F66, 0xFF627971, 0xFFB1CBC2, 0xFF8FA1A5, 0xFF4D5F63, 0xFF4B6061, 0xFF2E4847, 0xFF2C4A4A,
        0xFF335557, 0xFF2B4E54, 0xFF2A4F58, 0xFF4F779A, 0xFF6791B7, 0xFF7AA3CF, 0xFF7BA5D5, 0xFF79A4D7, 0xFF77A5D7, 0xFF74A4D4, 0xFF70A3D2, 0xFF77A3CA, 0xFF618CAE, 0xFF547C95, 0xFF406575, 0xFF32535A, 0xFF2E4B4F, 0xFF354F50, 0xFF2C4446, 0xFF264945,
        0xFF45645F, 0xFF405654, 0xFF7B898A, 0xFFADB5B7, 0xFFCED2D5, 0xFFF7F8FA, 0xFFF4F5F7, 0xFF769F91, 0xFF517A6C, 0xFF5E8779, 0xFF608A7C, 0xFF557F71, 0xFF5B8778, 0xFF558172, 0xFF517D6E, 0xFF568573, 0xFF5B8A7A, 0xFF5A8677, 0xFF608679, 0xFF3B5951,
        0xFF9AADA9, 0xFFBDC5C7, 0xFF929699, 0xFF485460, 0xFF526166, 0xFF394D4C, 0xFF334E47, 0xFF284741, 0xFF224445, 0xFF2B5058, 0xFF3E6471, 0xFF547EA4, 0xFF6894BB, 0xFF74A5D0, 0xFF71A4D3, 0xFF6A9ECE, 0xFF6D9FD0, 0xFF74A4D4, 0xFF79A5D4, 0xFF689BBA,
        0xFF528098, 0xFF3F6373, 0xFF425B62, 0xFF445859, 0xFF455957, 0xFF485E5C, 0xFF3E5656, 0xFF565F66, 0xFF4B565A, 0xFF879192, 0xFFBDBDBF, 0xFFE5DCDF, 0xFFFAF4F6, 0xFFA8B4B0, 0xFF425D56, 0xFF578372, 0xFF578372, 0xFF588473, 0xFF588473, 0xFF598574,
        0xFF598574, 0xFF5A8675, 0xFF5A8675, 0xFF588D79, 0xFF5E8A7B, 0xFF3F5F54, 0xFF506560, 0xFFB0BEBE, 0xFFDCE7E9, 0xFFABBABD, 0xFF415355, 0xFF4D6567, 0xFF34494A, 0xFF2C403E, 0xFF304741, 0xFF324A4A, 0xFF37545A, 0xFF406170, 0xFF4E7386, 0xFF558BB1,
        0xFF5F99BF, 0xFF6AA5CF, 0xFF6AA7D3, 0xFF6BA8D4, 0xFF6EA9D3, 0xFF69A0C7, 0xFF6094BB, 0xFF577A90, 0xFF416575, 0xFF2F5258, 0xFF244441, 0xFF22413B, 0xFF2D4744, 0xFF273F41, 0xFF4F676B, 0xFF49515E, 0xFF848C8E, 0xFFB2B8AE, 0xFFA0A89D, 0xFF606C68,
        0xFF475D5A, 0xFF5D8175, 0xFF5D8972, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5C8573, 0xFF5C8573, 0xFF5C8573, 0xFF4D776B, 0xFF49635A, 0xFFB5BBB7, 0xFFD4D6D1, 0xFFB5C0BA, 0xFF859894, 0xFF3C5051, 0xFF58676C, 0xFF3D5158,
        0xFF384D52, 0xFF354A4B, 0xFF304A49, 0xFF314E52, 0xFF385C68, 0xFF436E81, 0xFF4A7892, 0xFF67A3C7, 0xFF6CA6CC, 0xFF6FA6CF, 0xFF6EA3CD, 0xFF6FA3CB, 0xFF6D9FC2, 0xFF5B8CAC, 0xFF467893, 0xFF4E6875, 0xFF3F5A61, 0xFF34514F, 0xFF334E47, 0xFF37524B,
        0xFF3C5653, 0xFF566E72, 0xFF445B63, 0xFF8C8D8F, 0xFFC4CDCC, 0xFF8FA09A, 0xFF3D584F, 0xFF406458, 0xFF5D8676, 0xFF638E7D, 0xFF578271, 0xFF5A8572, 0xFF5D8674, 0xFF5C8573, 0xFF5F8876, 0xFF628977, 0xFF638A78, 0xFF567B6A, 0xFF315645, 0xFFA49696,
        0xFFC7BDBE, 0xFFB1ADAC, 0xFF959998, 0xFF828B8A, 0xFF525C5D, 0xFF5C6668, 0xFF4B5557, 0xFF384C4D, 0xFF3A4E4F, 0xFF3F5457, 0xFF4E656B, 0xFF4D6B76, 0xFF4E7386, 0xFF517D96, 0xFF5D8CAA, 0xFF719CC7, 0xFF75A1CE, 0xFF78A4D1, 0xFF74A0CD, 0xFF6F9AC5,
        0xFF6B91B8, 0xFF6586A9, 0xFF5F7C9C, 0xFF4D6267, 0xFF43565A, 0xFF425154, 0xFF4E5A5A, 0xFF515D5D, 0xFF606B6F, 0xFF48575C, 0xFF8899A0, 0xFFD3D5D2, 0xFF95A29B, 0xFF557165, 0xFF608677, 0xFF5A7E70, 0x00708A7F, 0x00F3FEF8, 0x00FCFCFA, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFEFD, 0x26E1F4EE, 0xFF4F786A, 0xFF598576, 0xFF6C9285, 0xFF5E7970, 0xFF486259, 0xFF5A756C, 0xFF536269, 0xFF495B5D, 0xFF4E6462, 0xFF2B4641, 0xFF33514F,
        0xFF284B4D, 0xFF264B54, 0xFF4A707D, 0xFF6798C0, 0xFF76A7D0, 0xFF7BABD9, 0xFF77A7D8, 0xFF74A4D5, 0xFF71A1D1, 0xFF70A0CE, 0xFF74A5CE, 0xFF7AACDD, 0xFF71A2CD, 0xFF6694B5, 0xFF4D778D, 0xFF3B616E, 0xFF2A4B52, 0xFF2B484C, 0xFF2A4649, 0xFF364B50,
        0xFF5D7277, 0xFF4B6365, 0xFF385052, 0xFF2B4041, 0xFF4C6061, 0xFF8D9C9F, 0xFFAEBCBD, 0xFF5F887A, 0xFF517A6C, 0xFF5A8375, 0xFF5B8577, 0xFF5A8476, 0xFF628E7F, 0xFF5F8B7C, 0xFF5E8A7B, 0xFF558474, 0xFF5E8D7D, 0xFF5E8A7B, 0xFF567E73, 0xFF597A71,
        0xFFD5EEEA, 0xFFECFCFB, 0xFF7E8889, 0xFF536069, 0xFF536267, 0xFF324644, 0xFF2F4A45, 0xFF254542, 0xFF25484E, 0xFF416679, 0xFF6388A3, 0xFF71A0CC, 0xFF72A2D0, 0xFF6FA3D3, 0xFF69A1D2, 0xFF6AA2D3, 0xFF6EA7D4, 0xFF6FA4D0, 0xFF6CA0C8, 0xFF6CA9D8,
        0xFF6A9EC6, 0xFF58809A, 0xFF496772, 0xFF395151, 0xFF334946, 0xFF36514A, 0xFF2F4C47, 0xFF5B6D71, 0xFF40585A, 0xFF395155, 0xFF48575E, 0xFF999DA6, 0xFFE7EFF2, 0xFFB4C9C4, 0xFF537469, 0xFF568271, 0xFF568271, 0xFF578372, 0xFF578372, 0xFF588473,
        0xFF588473, 0xFF598574, 0xFF598574, 0xFF528773, 0xFF5E8A7B, 0xFF4B6A62, 0xFF475C57, 0xFF839191, 0xFF819191, 0xFF506566, 0xFF375150, 0xFF4B6564, 0xFF3A504E, 0xFF3B504B, 0xFF3E5250, 0xFF3E565A, 0xFF446171, 0xFF4F728E, 0xFF6389AD, 0xFF689CC4,
        0xFF6BA2CB, 0xFF6BA4D1, 0xFF67A1CF, 0xFF68A0CF, 0xFF6DA3D1, 0xFF73A4CF, 0xFF73A2CC, 0xFF6D99BC, 0xFF56829D, 0xFF3B6571, 0xFF254B4C, 0xFF274742, 0xFF34514C, 0xFF2D4243, 0xFF4F6266, 0xFF505A66, 0xFF626A6C, 0xFF7E847A, 0xFF8D9389, 0xFF697571,
        0xFF4E6763, 0xFF5C8577, 0xFF578970, 0xFF5D8674, 0xFF5D8674, 0xFF5D8674, 0xFF5C8573, 0xFF5C8573, 0xFF5C8573, 0xFF5C8573, 0xFF5C8573, 0xFF547C71, 0xFF445E55, 0xFF65706A, 0xFF768078, 0xFF72817A, 0xFF5D726D, 0xFF405254, 0xFF556268, 0xFF324B4F,
        0xFF3B5554, 0xFF344E4D, 0xFF2F4B4C, 0xFF3D5E67, 0xFF477086, 0xFF5686AA, 0xFF6EA3CF, 0xFF689CC3, 0xFF6B9FC7, 0xFF6C9FCC, 0xFF6C9FCE, 0xFF6FA2D1, 0xFF71A4D1, 0xFF6CA0C8, 0xFF6498BF, 0xFF56748E, 0xFF496678, 0xFF38555B, 0xFF2B4844, 0xFF314A44,
        0xFF3F5552, 0xFF56696D, 0xFF3B4C54, 0xFF808A89, 0xFFA1B0AD, 0xFF69807A, 0xFF415F55, 0xFF5E8276, 0xFF5E8777, 0xFF527B6B, 0xFF5F8878, 0xFF598471, 0xFF5C8573, 0xFF5D8674, 0xFF5F8876, 0xFF5E8573, 0xFF608775, 0xFF5C8170, 0xFF3F6453, 0xFFC0C0C0,
        0xFFFBFFFF, 0xFFB8C2C1, 0xFF677776, 0xFF5E7070, 0xFF425657, 0xFF586A6E, 0xFF495B5F, 0xFF3B5355, 0xFF3F5759, 0xFF4A6167, 0xFF55707B, 0xFF4A6B7E, 0xFF557E9A, 0xFF6597BA, 0xFF71ABD1, 0xFF74A1CB, 0xFF71A0CC, 0xFF6C9FCC, 0xFF6CA0CF, 0xFF71A4D3,
        0xFF73A4CF, 0xFF6C99C2, 0xFF638DB5, 0xFF587176, 0xFF4E6368, 0xFF465858, 0xFF495956, 0xFF485754, 0xFF596969, 0xFF43545B, 0xFF889BA2, 0xFFE6F1EB, 0xFFA7BBB2, 0xFF5B7B6E, 0xFF5C8575, 0xFF517768, 0x006F897E, 0x00F8FFFD, 0x00FFFFFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FBFFFF, 0x2EDEF5ED, 0xFF4D7769, 0xFF4D7D6D, 0xFF668F81, 0xFF749288, 0xFF5C776E, 0xFF47655B, 0xFF505F66, 0xFF4F6163, 0xFF566C69, 0xFF344F4A, 0xFF2D4D48,
        0xFF1E4143, 0xFF3D636C, 0xFF7198A7, 0xFF74AED3, 0xFF74AED4, 0xFF70A7D0, 0xFF70A5D1, 0xFF75A8D3, 0xFF75A6CF, 0xFF73A3C9, 0xFF78A9CA, 0xFF699DD6, 0xFF77ABDB, 0xFF7AACD1, 0xFF618FA9, 0xFF4C7686, 0xFF2E535C, 0xFF2C4D52, 0xFF375659, 0xFF474D59,
        0xFF606D76, 0xFF50646B, 0xFF294749, 0xFF1A3C3D, 0xFF3C5E5D, 0xFF496767, 0xFF435F60, 0xFF477062, 0xFF5E8779, 0xFF628B7D, 0xFF5B8577, 0xFF5C8678, 0xFF598576, 0xFF588475, 0xFF598576, 0xFF5C8879, 0xFF568273, 0xFF537F72, 0xFF5E887C, 0xFF597D73,
        0xFF607D78, 0xFF7C908E, 0xFF718180, 0xFF4F5C64, 0xFF576669, 0xFF374B49, 0xFF2F4A45, 0xFF2D4D4C, 0xFF3A5E6A, 0xFF567B95, 0xFF678DB1, 0xFF73A4CF, 0xFF6EA3CF, 0xFF69A2CF, 0xFF68A3CF, 0xFF67A5CE, 0xFF68A4C9, 0xFF69A4C6, 0xFF6BA4C2, 0xFF5F9FDC,
        0xFF72AADB, 0xFF6B96B8, 0xFF557584, 0xFF3C5657, 0xFF314A44, 0xFF36544A, 0xFF32534A, 0xFF547270, 0xFF355959, 0xFF254B4E, 0xFF335159, 0xFF5F737C, 0xFF6E8185, 0xFF53726C, 0xFF4E7A69, 0xFF568271, 0xFF568271, 0xFF568271, 0xFF578372, 0xFF578372,
        0xFF588473, 0xFF588473, 0xFF598574, 0xFF508372, 0xFF598375, 0xFF68877F, 0xFF4B5F5D, 0xFF495959, 0xFF3E5251, 0xFF284241, 0xFF325251, 0xFF526C69, 0xFF39524E, 0xFF374C47, 0xFF3A4E4C, 0xFF415860, 0xFF516D83, 0xFF5F83A7, 0xFF729BC7, 0xFF74A1C8,
        0xFF72A2CA, 0xFF71A2CD, 0xFF6EA1CE, 0xFF6D9DCB, 0xFF6D9CC8, 0xFF729CC6, 0xFF779EC7, 0xFF6EA3CD, 0xFF6597B8, 0xFF507F91, 0xFF2E5759, 0xFF22453F, 0xFF2F4C47, 0xFF2B413F, 0xFF526466, 0xFF4F5B67, 0xFF7A8485, 0xFFA3A99F, 0xFFA1A79D, 0xFF687470,
        0xFF4C6762, 0xFF608A7C, 0xFF51876D, 0xFF5D8674, 0xFF5D8674, 0xFF5C8573, 0xFF5C8573, 0xFF5C8573, 0xFF5C8573, 0xFF5B8472, 0xFF5B8472, 0xFF62887D, 0xFF4B655C, 0xFF5D6A63, 0xFF95A299, 0xFF879C93, 0xFF4D6660, 0xFF485C5D, 0xFF576268, 0xFF335151,
        0xFF2B4846, 0xFF375450, 0xFF446264, 0xFF456975, 0xFF58849F, 0xFF6BA0CC, 0xFF68A4DA, 0xFF739EC1, 0xFF729CC2, 0xFF709BC6, 0xFF709CCB, 0xFF6F9DCE, 0xFF6F9FCD, 0xFF6FA0CB, 0xFF6EA1CC, 0xFF7091B2, 0xFF5F7F98, 0xFF41616C, 0xFF2A4745, 0xFF2F4842,
        0xFF465A58, 0xFF627174, 0xFF45525B, 0xFF657373, 0xFF718480, 0xFF556E68, 0xFF4B6C63, 0xFF5E8477, 0xFF608679, 0xFF5E8475, 0xFF5A8071, 0xFF5A8572, 0xFF5E8775, 0xFF608977, 0xFF608977, 0xFF59806E, 0xFF5B8270, 0xFF608574, 0xFF4C7160, 0xFF909F9C,
        0xFFE8FBF9, 0xFFA4BDB9, 0xFF4F6C68, 0xFF587674, 0xFF415F5F, 0xFF4D6B6D, 0xFF385457, 0xFF334F52, 0xFF3E575B, 0xFF4D666D, 0xFF5B7785, 0xFF577992, 0xFF6C99BA, 0xFF6FA6CD, 0xFF609FCB, 0xFF6E9EC2, 0xFF70A2C7, 0xFF6EA3CB, 0xFF6AA3CE, 0xFF6AA3CE,
        0xFF6DA4CD, 0xFF6DA2CA, 0xFF6A9EC5, 0xFF668188, 0xFF50686C, 0xFF4C605F, 0xFF4B5E5A, 0xFF435350, 0xFF617373, 0xFF3E5157, 0xFF52666F, 0xFF707F78, 0xFF698378, 0xFF54786A, 0xFF679382, 0xFF507969, 0x00688277, 0x00F5FEF9, 0x00FFFFFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFBFE, 0x2EDEEEEB, 0xFF4B7368, 0xFF538373, 0xFF618B7D, 0xFF64887A, 0xFF618577, 0xFF5F8576, 0xFF536C71, 0xFF575F6A, 0xFF585F69, 0xFF38514B, 0xFF2F4F42,
        0xFF2B4542, 0xFF58798C, 0xFF73A5C6, 0xFF72A5D2, 0xFF74A7D4, 0xFF75A8D5, 0xFF74A7D4, 0xFF73A6D3, 0xFF72A5D2, 0xFF73A6D3, 0xFF74A7D4, 0xFF6FA4C6, 0xFF71A7CD, 0xFF74A9D1, 0xFF6CA0C7, 0xFF4C7B99, 0xFF496F82, 0xFF314E52, 0xFF3C5551, 0xFF3B4E54,
        0xFF626C75, 0xFF4B5E62, 0xFF325D54, 0xFF336156, 0xFF45645C, 0xFF4C6A60, 0xFF517A6A, 0xFF568072, 0xFF5C8678, 0xFF598375, 0xFF558172, 0xFF598576, 0xFF598878, 0xFF578676, 0xFF578676, 0xFF5C8273, 0xFF618778, 0xFF5B8172, 0xFF598274, 0xFF588274,
        0xFF517B6F, 0xFF467265, 0xFF386457, 0xFF4D535F, 0xFF607276, 0xFF415C57, 0xFF304E46, 0xFF334D4E, 0xFF557280, 0xFF5D89A2, 0xFF6AA5C3, 0xFF6FA2CF, 0xFF6EA1CE, 0xFF6EA1CE, 0xFF6EA1CE, 0xFF6DA0CD, 0xFF6EA1CE, 0xFF6EA1CE, 0xFF6FA2CF, 0xFF6EA5C3,
        0xFF6EA2C9, 0xFF76A5D3, 0xFF638EB1, 0xFF3A5D70, 0xFF324E51, 0xFF3F5351, 0xFF3B4949, 0xFF5D666F, 0xFF485D60, 0xFF3C5D56, 0xFF3E6256, 0xFF3E6052, 0xFF45695B, 0xFF528073, 0xFF4B8674, 0xFF51826F, 0xFF528370, 0xFF538471, 0xFF548572, 0xFF558673,
        0xFF558673, 0xFF558673, 0xFF558673, 0xFF568275, 0xFF5A8677, 0xFF568170, 0xFF567F6D, 0xFF476D5E, 0xFF40645A, 0xFF355854, 0xFF375756, 0xFF576661, 0xFF42534D, 0xFF364B46, 0xFF314B4C, 0xFF496B75, 0xFF4E7A93, 0xFF6CA0C7, 0xFF659FCD, 0xFF6DA0CB,
        0xFF6DA0CB, 0xFF6EA1CC, 0xFF6EA1CC, 0xFF6EA1CC, 0xFF6EA1CC, 0xFF6DA0CB, 0xFF6DA0CB, 0xFF6EA3D7, 0xFF6DA0CB, 0xFF628EA9, 0xFF416571, 0xFF2B4B4A, 0xFF314C45, 0xFF29433A, 0xFF4A645B, 0xFF415B5C, 0xFF506F6A, 0xFF4F7568, 0xFF477460, 0xFF517E69,
        0xFF5D8674, 0xFF608274, 0xFF5F7D73, 0xFF5F8876, 0xFF5E8775, 0xFF5D8674, 0xFF5C8573, 0xFF5B8472, 0xFF598270, 0xFF58816F, 0xFF58816F, 0xFF59806E, 0xFF608476, 0xFF476B5F, 0xFF4A7669, 0xFF44766A, 0xFF27524B, 0xFF3C5456, 0xFF5E656D, 0xFF32524D,
        0xFF2E4E49, 0xFF2D4F50, 0xFF4B6C7F, 0xFF4F759A, 0xFF6E9AC7, 0xFF6EA4C8, 0xFF67A5BE, 0xFF6B9EC9, 0xFF6C9FCA, 0xFF6DA0CB, 0xFF6DA0CB, 0xFF6B9EC9, 0xFF6B9EC9, 0xFF6C9FCA, 0xFF6DA0CB, 0xFF6BA2C8, 0xFF5A8CA7, 0xFF466C77, 0xFF36504D, 0xFF364A41,
        0xFF42514C, 0xFF59676A, 0xFF44525D, 0xFF406055, 0xFF46665B, 0xFF496A5F, 0xFF5A7E70, 0xFF628879, 0xFF56816E, 0xFF57826F, 0xFF5C8975, 0xFF57806E, 0xFF5A8371, 0xFF5C8573, 0xFF5B8472, 0xFF5A8371, 0xFF598270, 0xFF58816F, 0xFF598270, 0xFF4F6F62,
        0xFF62887B, 0xFF437365, 0xFF2B5F52, 0xFF35645A, 0xFF395958, 0xFF58656D, 0xFF545662, 0xFF545F61, 0xFF485D5E, 0xFF4D7076, 0xFF46768C, 0xFF669BC3, 0xFF689ECC, 0xFF6CA0C7, 0xFF6EA0BB, 0xFF6DA0CB, 0xFF6EA1CC, 0xFF6EA1CC, 0xFF6DA0CB, 0xFF6C9FCA,
        0xFF6C9FCA, 0xFF6EA1CC, 0xFF6FA2CD, 0xFF6990B9, 0xFF5C7C91, 0xFF506462, 0xFF485247, 0xFF4E554E, 0xFF677274, 0xFF3D5559, 0xFF446366, 0xFF487360, 0xFF567D6B, 0xFF628776, 0xFF638C7C, 0xFF527D6C, 0x0066867B, 0x00F9FFFF, 0x00FFF9FD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x31DCECE9, 0xFF4B7166, 0xFF568676, 0xFF648E80, 0xFF668A7C, 0xFF638779, 0xFF608677, 0xFF516D70, 0xFF515E67, 0xFF545E67, 0xFF34514D, 0xFF31554B,
        0xFF365255, 0xFF608399, 0xFF75A7CC, 0xFF73A6D3, 0xFF74A7D4, 0xFF74A7D4, 0xFF74A7D4, 0xFF73A6D3, 0xFF72A5D2, 0xFF73A6D3, 0xFF73A6D3, 0xFF70A4CB, 0xFF72A7CF, 0xFF76ABD5, 0xFF73A7CF, 0xFF5784A5, 0xFF507588, 0xFF35525A, 0xFF3E5656, 0xFF364B50,
        0xFF66737B, 0xFF475C5F, 0xFF2F5D53, 0xFF3B6D61, 0xFF577D72, 0xFF608478, 0xFF558472, 0xFF578173, 0xFF5D8779, 0xFF5C8678, 0xFF598576, 0xFF5B8778, 0xFF588777, 0xFF568575, 0xFF598878, 0xFF5F8878, 0xFF5F8878, 0xFF598473, 0xFF598375, 0xFF537F70,
        0xFF507C6F, 0xFF507C6F, 0xFF427063, 0xFF4E5861, 0xFF5E7273, 0xFF375650, 0xFF284741, 0xFF2E4A4D, 0xFF577688, 0xFF6491AE, 0xFF71ABD0, 0xFF70A3D0, 0xFF70A3D0, 0xFF6FA2CF, 0xFF6FA2CF, 0xFF6FA2CF, 0xFF6EA1CE, 0xFF6EA1CE, 0xFF6EA1CE, 0xFF6AA2C3,
        0xFF6FA4CE, 0xFF76A6D4, 0xFF6892B8, 0xFF466B7D, 0xFF345155, 0xFF354B49, 0xFF334547, 0xFF636D76, 0xFF3A5254, 0xFF355D54, 0xFF497566, 0xFF547D6D, 0xFF578070, 0xFF4F7F71, 0xFF47806F, 0xFF51826F, 0xFF528370, 0xFF538471, 0xFF548572, 0xFF558673,
        0xFF558673, 0xFF558673, 0xFF558673, 0xFF548275, 0xFF568575, 0xFF558170, 0xFF5A8572, 0xFF557E6E, 0xFF577D72, 0xFF466965, 0xFF365857, 0xFF586867, 0xFF465655, 0xFF3C504F, 0xFF375054, 0xFF4E707C, 0xFF55819A, 0xFF70A4CB, 0xFF69A1D0, 0xFF6EA1CC,
        0xFF6DA0CB, 0xFF6DA0CB, 0xFF6DA0CB, 0xFF6DA0CB, 0xFF6DA0CB, 0xFF6DA0CB, 0xFF6EA1CC, 0xFF6CA1D5, 0xFF6EA1CC, 0xFF6E9AB7, 0xFF4E7280, 0xFF2F4C50, 0xFF344E4B, 0xFF334C46, 0xFF546D67, 0xFF425E5F, 0xFF476762, 0xFF4E7769, 0xFF54816D, 0xFF54816C,
        0xFF517C69, 0xFF577D6E, 0xFF638479, 0xFF5E8775, 0xFF5E8775, 0xFF5D8674, 0xFF5C8573, 0xFF5B8472, 0xFF5A8371, 0xFF598270, 0xFF58816F, 0xFF598471, 0xFF587E6F, 0xFF4D7366, 0xFF4A7669, 0xFF3D7164, 0xFF29564F, 0xFF395354, 0xFF5E676E, 0xFF355554,
        0xFF2D4D4A, 0xFF304F52, 0xFF517186, 0xFF597FA6, 0xFF74A0CF, 0xFF6EA3CB, 0xFF629FBB, 0xFF6B9EC9, 0xFF6C9FCA, 0xFF6EA1CC, 0xFF6DA0CB, 0xFF6C9FCA, 0xFF6B9EC9, 0xFF6B9EC9, 0xFF6C9FCA, 0xFF71A6D2, 0xFF6593B4, 0xFF4E7282, 0xFF3C5657, 0xFF40534D,
        0xFF4A5954, 0xFF5B6A6D, 0xFF44525B, 0xFF4C7064, 0xFF567A6E, 0xFF5A8073, 0xFF5C8273, 0xFF5B8474, 0xFF5D8875, 0xFF608B78, 0xFF5D8875, 0xFF5F8876, 0xFF5C8573, 0xFF5A8371, 0xFF5A8371, 0xFF5B8472, 0xFF5A8371, 0xFF5A8371, 0xFF5A8371, 0xFF57796B,
        0xFF537C6E, 0xFF407062, 0xFF2B5F52, 0xFF27564C, 0xFF345655, 0xFF5E6F76, 0xFF545B65, 0xFF586367, 0xFF4C6164, 0xFF53757E, 0xFF507E96, 0xFF6C9FCA, 0xFF6B9FCF, 0xFF6CA0C8, 0xFF6EA1C0, 0xFF6EA1CC, 0xFF6EA1CC, 0xFF6FA2CD, 0xFF6EA1CC, 0xFF6C9FCA,
        0xFF6C9FCA, 0xFF6DA0CB, 0xFF6EA1CC, 0xFF7298C5, 0xFF63859E, 0xFF586D70, 0xFF4E5B52, 0xFF505954, 0xFF677578, 0xFF435B5F, 0xFF4C6B6D, 0xFF5F8876, 0xFF628776, 0xFF628776, 0xFF618A7A, 0xFF517C6B, 0x0065857A, 0x00F9FFFF, 0x00FFFBFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x35D9E9E6, 0xFF4C7267, 0xFF5B8A7A, 0xFF638D7F, 0xFF628678, 0xFF638577, 0xFF5C8273, 0xFF506E6C, 0xFF4E5F66, 0xFF52636B, 0xFF2E504F, 0xFF285048,
        0xFF35535B, 0xFF6487A3, 0xFF7BACD5, 0xFF74A7D4, 0xFF73A6D3, 0xFF73A6D3, 0xFF73A6D3, 0xFF73A6D3, 0xFF73A6D3, 0xFF72A5D2, 0xFF72A5D2, 0xFF70A5D1, 0xFF70A5D1, 0xFF76A9D6, 0xFF7AA9D3, 0xFF608BAD, 0xFF54778D, 0xFF385460, 0xFF3E575B, 0xFF314A4E,
        0xFF5F6E75, 0xFF41595B, 0xFF346258, 0xFF3B6F62, 0xFF436D61, 0xFF4B7768, 0xFF4B856F, 0xFF557F71, 0xFF5A8476, 0xFF5B8577, 0xFF5A8677, 0xFF5C8879, 0xFF578676, 0xFF568575, 0xFF5B8A7A, 0xFF5C8877, 0xFF5A8675, 0xFF598574, 0xFF588777, 0xFF518070,
        0xFF4E7C6F, 0xFF507E71, 0xFF447265, 0xFF4B5A61, 0xFF566E6E, 0xFF2C4D46, 0xFF1E413B, 0xFF2B484E, 0xFF597990, 0xFF6997BB, 0xFF71ACD6, 0xFF71A4D1, 0xFF71A4D1, 0xFF71A4D1, 0xFF70A3D0, 0xFF6FA2CF, 0xFF6FA2CF, 0xFF6EA1CE, 0xFF6DA0CD, 0xFF67A3C7,
        0xFF6DA3CF, 0xFF72A2D2, 0xFF6A96BB, 0xFF54798C, 0xFF37585D, 0xFF2A4647, 0xFF2B4448, 0xFF59666E, 0xFF3B5758, 0xFF3E6C61, 0xFF437866, 0xFF467764, 0xFF537F6E, 0xFF4F7F71, 0xFF457E6D, 0xFF528370, 0xFF528370, 0xFF538471, 0xFF548572, 0xFF558673,
        0xFF558673, 0xFF558673, 0xFF548572, 0xFF568678, 0xFF558575, 0xFF538270, 0xFF54816D, 0xFF4F7A69, 0xFF537B70, 0xFF456C67, 0xFF325654, 0xFF58676A, 0xFF47595B, 0xFF3F5354, 0xFF3A5358, 0xFF527281, 0xFF6089A5, 0xFF76A7D0, 0xFF6CA1D3, 0xFF6EA1CC,
        0xFF6DA0CB, 0xFF6C9FCA, 0xFF6C9FCA, 0xFF6C9FCA, 0xFF6C9FCA, 0xFF6DA0CB, 0xFF6EA1CC, 0xFF6DA2D8, 0xFF70A0CE, 0xFF75A0C0, 0xFF55788B, 0xFF2A474D, 0xFF2F494A, 0xFF324A4A, 0xFF4F6465, 0xFF466464, 0xFF4A6A65, 0xFF4F7568, 0xFF527D6A, 0xFF53806B,
        0xFF5A8773, 0xFF5C8776, 0xFF578072, 0xFF5D8674, 0xFF5D8674, 0xFF5C8573, 0xFF5C8573, 0xFF5B8472, 0xFF5A8371, 0xFF5A8371, 0xFF5A8371, 0xFF568571, 0xFF527B6B, 0xFF557E70, 0xFF507C6F, 0xFF3E7265, 0xFF2F5E56, 0xFF355353, 0xFF59686D, 0xFF385559,
        0xFF314F51, 0xFF355459, 0xFF58788D, 0xFF6487AF, 0xFF76A2D3, 0xFF6DA2CC, 0xFF65A0C2, 0xFF6B9EC9, 0xFF6DA0CB, 0xFF6EA1CC, 0xFF6EA1CC, 0xFF6C9FCA, 0xFF6B9EC9, 0xFF6A9DC8, 0xFF6B9EC9, 0xFF72A4D9, 0xFF6E99C3, 0xFF54778D, 0xFF425A5E, 0xFF4C5C59,
        0xFF52615E, 0xFF5E6D70, 0xFF43545C, 0xFF467062, 0xFF507A6C, 0xFF568072, 0xFF55806F, 0xFF578271, 0xFF57806E, 0xFF557E6C, 0xFF5D8674, 0xFF618A78, 0xFF5A8371, 0xFF58816F, 0xFF5C8573, 0xFF5C8573, 0xFF5A8371, 0xFF58816F, 0xFF57806E, 0xFF5F8375,
        0xFF547E70, 0xFF487B6C, 0xFF326659, 0xFF245349, 0xFF325654, 0xFF5C7176, 0xFF58626B, 0xFF5A6770, 0xFF4D6168, 0xFF567681, 0xFF58849D, 0xFF71A2CD, 0xFF6B9FD1, 0xFF6B9ECB, 0xFF6FA1C4, 0xFF6EA1CC, 0xFF6FA2CD, 0xFF6FA2CD, 0xFF6EA1CC, 0xFF6C9FCA,
        0xFF6C9FCA, 0xFF6DA0CB, 0xFF6EA1CC, 0xFF729ACD, 0xFF6083A1, 0xFF536C73, 0xFF485855, 0xFF475653, 0xFF5F7173, 0xFF405A5B, 0xFF4D6D6A, 0xFF638A78, 0xFF618373, 0xFF608272, 0xFF628879, 0xFF537E6D, 0x0066867B, 0x00F8FFFF, 0x00FFFBFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x31DBEAE7, 0xFF4D7368, 0xFF578676, 0xFF5E8779, 0xFF658779, 0xFF698B7D, 0xFF65897B, 0xFF51726B, 0xFF4E6165, 0xFF4E6269, 0xFF224948, 0xFF1E4846,
        0xFF32525D, 0xFF6487A5, 0xFF7DACD8, 0xFF75A8D5, 0xFF73A6D3, 0xFF72A5D2, 0xFF72A5D2, 0xFF73A6D3, 0xFF73A6D3, 0xFF72A5D2, 0xFF70A3D0, 0xFF6FA5D4, 0xFF6EA2D1, 0xFF75A5D3, 0xFF7BA6D1, 0xFF658BAF, 0xFF53738C, 0xFF385462, 0xFF3B565D, 0xFF264446,
        0xFF576A70, 0xFF445C5E, 0xFF2E5C52, 0xFF3A6E61, 0xFF547E72, 0xFF518070, 0xFF498770, 0xFF547E70, 0xFF578173, 0xFF568072, 0xFF578374, 0xFF5B8778, 0xFF578676, 0xFF568575, 0xFF5B8A7A, 0xFF568774, 0xFF578875, 0xFF538471, 0xFF568676, 0xFF568575,
        0xFF517F72, 0xFF507E71, 0xFF477568, 0xFF45585C, 0xFF4D6A68, 0xFF21463E, 0xFF1B3F3B, 0xFF2A4850, 0xFF597992, 0xFF6996BD, 0xFF6EA6D5, 0xFF6FA2CF, 0xFF6FA2CF, 0xFF70A3D0, 0xFF70A3D0, 0xFF6FA2CF, 0xFF6FA2CF, 0xFF6EA1CE, 0xFF6DA0CD, 0xFF67A5CC,
        0xFF68A0CF, 0xFF6F9FD0, 0xFF6D97BD, 0xFF577C8E, 0xFF375A5E, 0xFF29484A, 0xFF28474C, 0xFF5A6970, 0xFF335151, 0xFF2D5F53, 0xFF427B68, 0xFF4B7E6B, 0xFF4E7D6B, 0xFF508072, 0xFF4E8474, 0xFF538270, 0xFF548371, 0xFF558472, 0xFF558472, 0xFF568573,
        0xFF568573, 0xFF558472, 0xFF558472, 0xFF578779, 0xFF518171, 0xFF51806E, 0xFF53806C, 0xFF4E7968, 0xFF4E786C, 0xFF446B66, 0xFF315855, 0xFF55666D, 0xFF44565A, 0xFF3B4E54, 0xFF375057, 0xFF527183, 0xFF6990AF, 0xFF7AA7D1, 0xFF6C9ED1, 0xFF6DA0CB,
        0xFF6DA0CB, 0xFF6DA0CB, 0xFF6DA0CB, 0xFF6DA0CB, 0xFF6DA0CB, 0xFF6DA0CB, 0xFF6DA0CB, 0xFF6EA1D8, 0xFF6E9ECC, 0xFF79A2C2, 0xFF597A8D, 0xFF27444C, 0xFF2D4549, 0xFF33484B, 0xFF4A5F62, 0xFF385656, 0xFF4B6B66, 0xFF587C70, 0xFF547D6B, 0xFF4F7A66,
        0xFF53806C, 0xFF588473, 0xFF588475, 0xFF5C8573, 0xFF5C8573, 0xFF5C8573, 0xFF5B8472, 0xFF5B8472, 0xFF5B8472, 0xFF5B8472, 0xFF5B8472, 0xFF538771, 0xFF568170, 0xFF587E71, 0xFF4F796D, 0xFF3E7064, 0xFF2D5C54, 0xFF335352, 0xFF57696D, 0xFF38555D,
        0xFF345155, 0xFF3D5C61, 0xFF5E7D92, 0xFF678AB2, 0xFF739DCF, 0xFF6DA0CF, 0xFF6AA4CA, 0xFF6B9EC9, 0xFF6DA0CB, 0xFF6EA1CC, 0xFF6EA1CC, 0xFF6C9FCA, 0xFF6B9EC9, 0xFF6A9DC8, 0xFF6A9DC8, 0xFF6F9ED6, 0xFF739CC8, 0xFF597992, 0xFF465D63, 0xFF50605D,
        0xFF52615E, 0xFF5D6C6F, 0xFF42535A, 0xFF477666, 0xFF558474, 0xFF5B8778, 0xFF578271, 0xFF5B8675, 0xFF5A8371, 0xFF547D6B, 0xFF638A78, 0xFF5B8472, 0xFF567F6D, 0xFF598270, 0xFF5F8876, 0xFF5C8573, 0xFF58816F, 0xFF567F6D, 0xFF547D6B, 0xFF4E7465,
        0xFF4C7668, 0xFF467668, 0xFF2F6155, 0xFF28564C, 0xFF335755, 0xFF546D71, 0xFF5D6E75, 0xFF576371, 0xFF4A5E67, 0xFF56747F, 0xFF5E869F, 0xFF729FC9, 0xFF6B9DD2, 0xFF6A9CCD, 0xFF6EA2CA, 0xFF6FA2CD, 0xFF6FA2CD, 0xFF6FA2CD, 0xFF6EA1CC, 0xFF6C9FCA,
        0xFF6B9EC9, 0xFF6C9FCA, 0xFF6EA1CC, 0xFF7A9FD3, 0xFF5F82A2, 0xFF4B6772, 0xFF435756, 0xFF425454, 0xFF5D7074, 0xFF405D5B, 0xFF50716A, 0xFF5F8171, 0xFF608071, 0xFF628273, 0xFF63897A, 0xFF537E6D, 0x0065867B, 0x00F8FFFF, 0x00FFFCFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x2EDEEDEA, 0xFF4C7066, 0xFF558172, 0xFF60897B, 0xFF6B8D7F, 0xFF6D8D80, 0xFF618577, 0xFF4B6C65, 0xFF495E61, 0xFF485F67, 0xFF1B4446, 0xFF1D4B49,
        0xFF35565F, 0xFF64839F, 0xFF7DA9D0, 0xFF76A9D6, 0xFF74A7D4, 0xFF73A6D3, 0xFF73A6D3, 0xFF73A6D3, 0xFF73A6D3, 0xFF72A5D2, 0xFF70A3D0, 0xFF6BA3D4, 0xFF6DA1D1, 0xFF76A5D1, 0xFF7EA5CE, 0xFF698CAC, 0xFF506E86, 0xFF365361, 0xFF36535B, 0xFF2E4D4F,
        0xFF51646A, 0xFF4C6164, 0xFF355D55, 0xFF366157, 0xFF496D63, 0xFF476D60, 0xFF538A75, 0xFF588274, 0xFF598375, 0xFF568072, 0xFF568273, 0xFF5C8879, 0xFF598878, 0xFF568575, 0xFF578676, 0xFF568976, 0xFF598C79, 0xFF51826F, 0xFF548373, 0xFF588777,
        0xFF527E71, 0xFF4C766A, 0xFF477165, 0xFF45585E, 0xFF486664, 0xFF1D423B, 0xFF1C3F39, 0xFF2B4850, 0xFF56748C, 0xFF6D97BD, 0xFF73A9D7, 0xFF6DA0CD, 0xFF6EA1CE, 0xFF6EA1CE, 0xFF6EA1CE, 0xFF6EA1CE, 0xFF6EA1CE, 0xFF6EA1CE, 0xFF6EA1CE, 0xFF65A3CC,
        0xFF6AA0CF, 0xFF78A4D3, 0xFF7399BD, 0xFF507180, 0xFF2F5153, 0xFF26484A, 0xFF25484E, 0xFF505F66, 0xFF405E5E, 0xFF366459, 0xFF457565, 0xFF4C7766, 0xFF507667, 0xFF578376, 0xFF4B7E6F, 0xFF548371, 0xFF558472, 0xFF558472, 0xFF568573, 0xFF568573,
        0xFF568573, 0xFF558472, 0xFF558472, 0xFF568477, 0xFF4E7D6D, 0xFF54806F, 0xFF598471, 0xFF568170, 0xFF4C7469, 0xFF436763, 0xFF365A58, 0xFF56676E, 0xFF41545A, 0xFF374A50, 0xFF354C54, 0xFF526F7F, 0xFF6E93AE, 0xFF7BA6D0, 0xFF6F9FCF, 0xFF6C9FCA,
        0xFF6DA0CB, 0xFF6DA0CB, 0xFF6EA1CC, 0xFF6EA1CC, 0xFF6DA0CB, 0xFF6DA0CB, 0xFF6C9FCA, 0xFF6DA1D3, 0xFF71A0CC, 0xFF7BA4C2, 0xFF597A8B, 0xFF2B464F, 0xFF334B4F, 0xFF3B5055, 0xFF556A6F, 0xFF435F60, 0xFF496662, 0xFF506E64, 0xFF597B6B, 0xFF5E8371,
        0xFF57806E, 0xFF527E6D, 0xFF578676, 0xFF5B8472, 0xFF5B8472, 0xFF5B8472, 0xFF5B8472, 0xFF5B8472, 0xFF5B8472, 0xFF5B8472, 0xFF5B8472, 0xFF4F836D, 0xFF5E8978, 0xFF577B6F, 0xFF51776C, 0xFF467167, 0xFF2F5A53, 0xFF375555, 0xFF54676B, 0xFF38535E,
        0xFF355057, 0xFF3F5C60, 0xFF5E7D8F, 0xFF6C8EB3, 0xFF749ECE, 0xFF6C9FCE, 0xFF6AA1CA, 0xFF6B9EC9, 0xFF6DA0CB, 0xFF6EA1CC, 0xFF6EA1CC, 0xFF6C9FCA, 0xFF6B9EC9, 0xFF6B9EC9, 0xFF6B9EC9, 0xFF739ED5, 0xFF789DC9, 0xFF5D7991, 0xFF4A5E65, 0xFF4F5F5C,
        0xFF4B5B58, 0xFF596B6D, 0xFF40535A, 0xFF487264, 0xFF517B6D, 0xFF5C8577, 0xFF5B8474, 0xFF588171, 0xFF5D8674, 0xFF5F8876, 0xFF5B8472, 0xFF5A8371, 0xFF567F6D, 0xFF5C8573, 0xFF608977, 0xFF5A8371, 0xFF57806E, 0xFF58816F, 0xFF58816F, 0xFF5C8072,
        0xFF527B6D, 0xFF4A7669, 0xFF336156, 0xFF2E574F, 0xFF375958, 0xFF546D71, 0xFF53686D, 0xFF4E5B6B, 0xFF445861, 0xFF55717C, 0xFF6589A1, 0xFF76A0CA, 0xFF70A0D1, 0xFF6B9DCE, 0xFF6BA0CA, 0xFF6EA1CC, 0xFF6FA2CD, 0xFF6FA2CD, 0xFF6EA1CC, 0xFF6C9FCA,
        0xFF6C9FCA, 0xFF6DA0CB, 0xFF6EA1CC, 0xFF81A3D1, 0xFF5C7E9A, 0xFF446169, 0xFF3D5253, 0xFF3E5253, 0xFF5A6F74, 0xFF405A59, 0xFF4E6B66, 0xFF628273, 0xFF698577, 0xFF6A897A, 0xFF65897B, 0xFF4F7A69, 0x00628378, 0x00F7FFFF, 0x00FFFDFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x2ADFEBE9, 0xFF4B6F65, 0xFF588475, 0xFF699284, 0xFF68887B, 0xFF527265, 0xFF3A5E50, 0xFF375650, 0xFF475A60, 0xFF516772, 0xFF224B4D, 0xFF1E4C49,
        0xFF2F4E51, 0xFF5A7588, 0xFF82A7C4, 0xFF76A9D6, 0xFF75A8D5, 0xFF74A7D4, 0xFF74A7D4, 0xFF74A7D4, 0xFF73A6D3, 0xFF72A5D2, 0xFF71A4D1, 0xFF6AA4D2, 0xFF71A6D2, 0xFF7BA8D1, 0xFF7DA3C7, 0xFF6887A3, 0xFF4B687A, 0xFF34525D, 0xFF314F57, 0xFF335757,
        0xFF506369, 0xFF47595D, 0xFF2D4A46, 0xFF26433E, 0xFF3E514D, 0xFF3A5149, 0xFF365F4F, 0xFF557F71, 0xFF5C8678, 0xFF5C8678, 0xFF598576, 0xFF5C8879, 0xFF5A8979, 0xFF568575, 0xFF558474, 0xFF568774, 0xFF5A8B78, 0xFF568573, 0xFF598576, 0xFF557F71,
        0xFF426A5F, 0xFF365C51, 0xFF2D5147, 0xFF495C63, 0xFF4A6667, 0xFF1D403A, 0xFF20413A, 0xFF284145, 0xFF4D6879, 0xFF6C93B0, 0xFF7BADD2, 0xFF6FA2CF, 0xFF6FA2CF, 0xFF6EA1CE, 0xFF6DA0CD, 0xFF6DA0CD, 0xFF6EA1CE, 0xFF6EA1CE, 0xFF6FA2CF, 0xFF659EC9,
        0xFF70A3D2, 0xFF83A9D6, 0xFF7493B2, 0xFF45636D, 0xFF274744, 0xFF244645, 0xFF24494F, 0xFF4F6067, 0xFF415B5C, 0xFF2E514A, 0xFF315247, 0xFF395348, 0xFF405A4F, 0xFF52766C, 0xFF5A887B, 0xFF588473, 0xFF588473, 0xFF598574, 0xFF598574, 0xFF598574,
        0xFF598574, 0xFF588473, 0xFF588473, 0xFF598377, 0xFF578173, 0xFF5A8373, 0xFF517866, 0xFF416557, 0xFF294D43, 0xFF2A4A47, 0xFF335151, 0xFF576A70, 0xFF425559, 0xFF384B4F, 0xFF354C52, 0xFF4C6874, 0xFF698DA5, 0xFF77A2C5, 0xFF76A3CD, 0xFF6DA0CB,
        0xFF6DA0CB, 0xFF6EA1CC, 0xFF6EA1CC, 0xFF6EA1CC, 0xFF6EA1CC, 0xFF6DA0CB, 0xFF6DA0CB, 0xFF6DA2CE, 0xFF75A5CB, 0xFF779FB9, 0xFF50707F, 0xFF294549, 0xFF34494C, 0xFF3A4D51, 0xFF5B6E74, 0xFF3E5356, 0xFF3D5351, 0xFF3D544C, 0xFF455D50, 0xFF517060,
        0xFF5E8372, 0xFF5D8877, 0xFF548373, 0xFF5B8472, 0xFF5B8472, 0xFF5B8472, 0xFF5B8472, 0xFF5B8472, 0xFF5B8472, 0xFF5B8472, 0xFF5B8472, 0xFF52836E, 0xFF587E6F, 0xFF456359, 0xFF37564E, 0xFF2A4F47, 0xFF204440, 0xFF385455, 0xFF586A6E, 0xFF3A5560,
        0xFF345053, 0xFF395757, 0xFF577580, 0xFF6D90AE, 0xFF78A1CD, 0xFF6F9FCD, 0xFF649BC4, 0xFF6B9EC9, 0xFF6C9FCA, 0xFF6DA0CB, 0xFF6C9FCA, 0xFF6B9EC9, 0xFF6B9EC9, 0xFF6C9FCA, 0xFF6DA0CB, 0xFF79A2D0, 0xFF779BBD, 0xFF577283, 0xFF495D5E, 0xFF4B5C56,
        0xFF475754, 0xFF596B6F, 0xFF3E525B, 0xFF3C5A50, 0xFF35554A, 0xFF44655A, 0xFF5F8173, 0xFF5A8071, 0xFF577E6C, 0xFF5F8876, 0xFF55806D, 0xFF5F8876, 0xFF5B8472, 0xFF5F8876, 0xFF5F8876, 0xFF58816F, 0xFF598270, 0xFF5D8674, 0xFF598270, 0xFF547467,
        0xFF3C5D52, 0xFF31554B, 0xFF254A42, 0xFF21443E, 0xFF375555, 0xFF5B7478, 0xFF475C61, 0xFF455564, 0xFF40545B, 0xFF546F76, 0xFF6B8C9D, 0xFF79A1C5, 0xFF77A6D4, 0xFF6CA0CF, 0xFF689FC6, 0xFF6DA0CB, 0xFF6EA1CC, 0xFF6EA1CC, 0xFF6DA0CB, 0xFF6C9FCA,
        0xFF6C9FCA, 0xFF6EA1CC, 0xFF6FA2CD, 0xFF809EC2, 0xFF59758A, 0xFF405C5F, 0xFF3C5250, 0xFF3C4F53, 0xFF596A71, 0xFF405357, 0xFF4F6562, 0xFF5C786A, 0xFF637D70, 0xFF698577, 0xFF698B7D, 0xFF517A6A, 0x00608176, 0x00F4FFFC, 0x00FFFCFE, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFBFF, 0x2ADFEBE9, 0xFF4C6D64, 0xFF5D8779, 0xFF6E9487, 0xFF5A7A6D, 0xFF38584B, 0xFF2A4C3E, 0xFF35504B, 0xFF4D5C63, 0xFF566C77, 0xFF224B4D, 0xFF1C4942,
        0xFF213E3A, 0xFF435A62, 0xFF7493A5, 0xFF76A9D6, 0xFF77AAD7, 0xFF76A9D6, 0xFF75A8D5, 0xFF74A7D4, 0xFF72A5D2, 0xFF72A5D2, 0xFF72A5D2, 0xFF6FAAD6, 0xFF76ABD5, 0xFF7AA6CB, 0xFF7497B5, 0xFF5A798E, 0xFF3F5C6A, 0xFF325058, 0xFF335257, 0xFF2B4F4F,
        0xFF5D7076, 0xFF505B61, 0xFF3D4F4F, 0xFF576766, 0xFF9C9D9F, 0xFF9BA19F, 0xFF5A6F66, 0xFF446E60, 0xFF568072, 0xFF5F897B, 0xFF5A8677, 0xFF5A8677, 0xFF598878, 0xFF578676, 0xFF578676, 0xFF568573, 0xFF578372, 0xFF5D8877, 0xFF5E8779, 0xFF486C60,
        0xFF34554C, 0xFF304F47, 0xFF224139, 0xFF4E5C69, 0xFF4E666A, 0xFF22413C, 0xFF25433B, 0xFF233937, 0xFF3D535E, 0xFF5E8195, 0xFF73A1BB, 0xFF74A7D4, 0xFF73A6D3, 0xFF70A3D0, 0xFF6EA1CE, 0xFF6DA0CD, 0xFF6DA0CD, 0xFF6EA1CE, 0xFF6FA2CF, 0xFF6DA2CC,
        0xFF7BA7D4, 0xFF7C9DC8, 0xFF617B96, 0xFF3E575C, 0xFF2A4742, 0xFF2A4A47, 0xFF2B4E52, 0xFF5A6B72, 0xFF33484B, 0xFF3D5350, 0xFF707F7A, 0xFF959B97, 0xFF7B8680, 0xFF4E6962, 0xFF588276, 0xFF598574, 0xFF598574, 0xFF5A8675, 0xFF5A8675, 0xFF5A8675,
        0xFF598574, 0xFF598574, 0xFF588473, 0xFF5E8479, 0xFF608478, 0xFF577B6D, 0xFF3E6050, 0xFF2E4E41, 0xFF19372F, 0xFF223C3B, 0xFF344E4F, 0xFF56696D, 0xFF425657, 0xFF3A504E, 0xFF344C4E, 0xFF3F5C64, 0xFF5B7E92, 0xFF6F98B6, 0xFF7AA6CB, 0xFF70A3CE,
        0xFF6FA2CD, 0xFF6EA1CC, 0xFF6DA0CB, 0xFF6DA0CB, 0xFF6EA1CC, 0xFF6FA2CD, 0xFF70A3CE, 0xFF72A6CD, 0xFF77A5C6, 0xFF638CA0, 0xFF3A5B64, 0xFF294344, 0xFF374D4B, 0xFF324647, 0xFF536768, 0xFF485A5E, 0xFF6A7A7A, 0xFF889792, 0xFF75867C, 0xFF4E6457,
        0xFF506F60, 0xFF63897A, 0xFF5D897A, 0xFF5C8573, 0xFF5C8573, 0xFF5C8573, 0xFF5B8472, 0xFF5B8472, 0xFF5B8472, 0xFF5B8472, 0xFF5A8371, 0xFF578672, 0xFF46685A, 0xFF395048, 0xFF213832, 0xFF0C2924, 0xFF183634, 0xFF3C5456, 0xFF5F6C72, 0xFF405B64,
        0xFF355152, 0xFF33504C, 0xFF47666B, 0xFF62869E, 0xFF749EC4, 0xFF72A3CC, 0xFF689FC5, 0xFF6B9EC9, 0xFF6C9FCA, 0xFF6C9FCA, 0xFF6B9EC9, 0xFF6A9DC8, 0xFF6B9EC9, 0xFF6EA1CC, 0xFF70A3CE, 0xFF79A1C4, 0xFF6A8EA6, 0xFF45626A, 0xFF3F544F, 0xFF46574F,
        0xFF465653, 0xFF5B6E74, 0xFF3E515F, 0xFF5E6F69, 0xFF637670, 0xFF4C635B, 0xFF567266, 0xFF658779, 0xFF59806E, 0xFF5B8673, 0xFF5B8874, 0xFF5F8876, 0xFF5B8472, 0xFF5E8775, 0xFF5E8775, 0xFF58816F, 0xFF5C8573, 0xFF5B8472, 0xFF4D7664, 0xFF304C40,
        0xFF233E35, 0xFF133129, 0xFF0F2C27, 0xFF16332F, 0xFF334D4E, 0xFF60757A, 0xFF4A5D63, 0xFF42545E, 0xFF3D5257, 0xFF506A6B, 0xFF63838E, 0xFF7398B5, 0xFF78A5CF, 0xFF6EA3CD, 0xFF67A1C6, 0xFF6C9FCA, 0xFF6DA0CB, 0xFF6DA0CB, 0xFF6DA0CB, 0xFF6DA0CB,
        0xFF6DA0CB, 0xFF6FA2CD, 0xFF71A4CF, 0xFF7791A8, 0xFF536E77, 0xFF445D59, 0xFF445954, 0xFF455457, 0xFF5F6D76, 0xFF4A595E, 0xFF5C6C6B, 0xFF5C7669, 0xFF5B7366, 0xFF617B6E, 0xFF6A8C7E, 0xFF567F6F, 0x005D7E73, 0x00EFFBF7, 0x00FFFDFE, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x26E1EDEB, 0xFF4B6C63, 0xFF5A8476, 0xFF698F82, 0xFF527265, 0xFF3D5B4F, 0xFF446658, 0xFF495F5D, 0xFF536069, 0xFF4E616F, 0xFF173F3F, 0xFF1D483E,
        0xFF1D3831, 0xFF2E4344, 0xFF57747C, 0xFF76A9D6, 0xFF77AAD7, 0xFF78ABD8, 0xFF76A9D6, 0xFF74A7D4, 0xFF72A5D2, 0xFF72A5D2, 0xFF73A6D3, 0xFF72B0D9, 0xFF79AED6, 0xFF75A2C3, 0xFF6488A2, 0xFF4A6779, 0xFF334F5A, 0xFF315055, 0xFF345658, 0xFF315857,
        0xFF5D6E75, 0xFF575E66, 0xFF6E797B, 0xFF7A7F82, 0xFF90878C, 0xFF9E989A, 0xFF717C76, 0xFF2F594B, 0xFF4C7668, 0xFF5D8779, 0xFF598576, 0xFF568273, 0xFF568575, 0xFF588777, 0xFF5A8979, 0xFF5E8A79, 0xFF547F6E, 0xFF5D8676, 0xFF5E8477, 0xFF3B5C51,
        0xFF325048, 0xFF456059, 0xFF3C5750, 0xFF4E5A6A, 0xFF50646B, 0xFF25423E, 0xFF2A453C, 0xFF1E332E, 0xFF2D4247, 0xFF4D6D7C, 0xFF638EA1, 0xFF79ACD9, 0xFF77AAD7, 0xFF73A6D3, 0xFF6FA2CF, 0xFF6DA0CD, 0xFF6DA0CD, 0xFF6EA1CE, 0xFF6FA2CF, 0xFF76AAD2,
        0xFF7FAAD5, 0xFF6D8AB2, 0xFF4B6179, 0xFF3B5053, 0xFF365047, 0xFF31504A, 0xFF315456, 0xFF59686F, 0xFF4B5D61, 0xFF647272, 0xFF717372, 0xFF969090, 0xFF999B98, 0xFF4F645F, 0xFF3A6257, 0xFF5A8675, 0xFF5A8675, 0xFF5A8675, 0xFF5B8776, 0xFF5A8675,
        0xFF5A8675, 0xFF598574, 0xFF598574, 0xFF62837A, 0xFF5D7E73, 0xFF4C6C5F, 0xFF365546, 0xFF405C50, 0xFF39524C, 0xFF385050, 0xFF415659, 0xFF516667, 0xFF405654, 0xFF3B514E, 0xFF324B48, 0xFF335056, 0xFF4E6F80, 0xFF658DA7, 0xFF7BA6C8, 0xFF73A6D1,
        0xFF71A4CF, 0xFF6DA0CB, 0xFF6B9EC9, 0xFF6B9EC9, 0xFF6DA0CB, 0xFF71A4CF, 0xFF73A6D1, 0xFF75A9CE, 0xFF72A1BD, 0xFF4F788A, 0xFF294A51, 0xFF2E4847, 0xFF435956, 0xFF344847, 0xFF506463, 0xFF445157, 0xFF737E80, 0xFFB6C1BD, 0xFFADB8B0, 0xFF5E7064,
        0xFF435D50, 0xFF5A7E70, 0xFF5C8678, 0xFF5D8674, 0xFF5C8573, 0xFF5C8573, 0xFF5C8573, 0xFF5B8472, 0xFF5B8472, 0xFF5A8371, 0xFF5A8371, 0xFF517E6A, 0xFF3D5D50, 0xFF556862, 0xFF495C58, 0xFF2D4642, 0xFF385553, 0xFF485B5F, 0xFF5A656B, 0xFF465F66,
        0xFF3B5554, 0xFF304E46, 0xFF39585A, 0xFF54778B, 0xFF6B96B8, 0xFF74A5CD, 0xFF75ABD1, 0xFF6B9EC9, 0xFF6B9EC9, 0xFF6B9EC9, 0xFF6A9DC8, 0xFF6A9DC8, 0xFF6B9EC9, 0xFF6FA2CD, 0xFF71A4CF, 0xFF729CB5, 0xFF5B7F8F, 0xFF345053, 0xFF344942, 0xFF435349,
        0xFF465653, 0xFF5F7279, 0xFF3F5261, 0xFF9CA5A2, 0xFFC9D4D0, 0xFF758680, 0xFF3D544A, 0xFF5D7D70, 0xFF628977, 0xFF588571, 0xFF588773, 0xFF5A8371, 0xFF57806E, 0xFF5C8573, 0xFF5D8674, 0xFF5A8371, 0xFF5E8775, 0xFF557E6C, 0xFF3C6553, 0xFF5B7268,
        0xFF5E786F, 0xFF405953, 0xFF2B4440, 0xFF304946, 0xFF394E51, 0xFF596C72, 0xFF4A5D63, 0xFF455761, 0xFF3C5152, 0xFF486261, 0xFF58767E, 0xFF678BA5, 0xFF74A0C7, 0xFF6FA4CC, 0xFF69A4C6, 0xFF6B9EC9, 0xFF6C9FCA, 0xFF6DA0CB, 0xFF6DA0CB, 0xFF6DA0CB,
        0xFF6EA1CC, 0xFF70A3CE, 0xFF71A4CF, 0xFF647B8B, 0xFF455D61, 0xFF3E554D, 0xFF435650, 0xFF425154, 0xFF5D6A73, 0xFF4D5760, 0xFF636E70, 0xFF728A7D, 0xFF62786C, 0xFF60786B, 0xFF6A8C7E, 0xFF578070, 0x005A7B70, 0x00ECFBF6, 0x00FFFEFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFBFF, 0x2ADFEEEB, 0xFF466E63, 0xFF538677, 0xFF608A7C, 0xFF516C63, 0xFF5C7067, 0xFFC4D3CC, 0xFFA09CAA, 0xFF585F69, 0xFF485D62, 0xFF2C4E4F, 0xFF1E4544,
        0xFF264547, 0xFF4A5F64, 0xFF4E5B63, 0xFF47717F, 0xFF6B95AB, 0xFF88B3D3, 0xFF88B5DE, 0xFF82B0E1, 0xFF83B6E5, 0xFF81B5E4, 0xFF79AEDA, 0xFF84A7C7, 0xFF7B9CBB, 0xFF61819A, 0xFF405F74, 0xFF324E5C, 0xFF35505B, 0xFF3D545A, 0xFF3C5458, 0xFF324E4F,
        0xFF597374, 0xFF495D5E, 0xFF7B8989, 0xFF979D9D, 0xFF818181, 0xFF979191, 0xFF7D7573, 0xFF3D6656, 0xFF4F7868, 0xFF5E8777, 0xFF5D8877, 0xFF5A8574, 0xFF5A8675, 0xFF5C8877, 0xFF5A8675, 0xFF5A8B78, 0xFF578674, 0xFF618C7B, 0xFF5D7E73, 0xFF364D45,
        0xFF7B8584, 0xFFC3C3C5, 0xFF999397, 0xFF4C5B62, 0xFF4E6167, 0xFF1D3B3D, 0xFF1F4645, 0xFF184240, 0xFF153F3D, 0xFF204746, 0xFF456969, 0xFF5F8EAA, 0xFF79A7C9, 0xFF82AFD9, 0xFF7CA7DA, 0xFF7EA9E0, 0xFF81ACE1, 0xFF7FABDC, 0xFF7EAAD9, 0xFF87A4C2,
        0xFF6D8BA5, 0xFF476379, 0xFF365361, 0xFF38555D, 0xFF304C4F, 0xFF28423F, 0xFF25403B, 0xFF52636A, 0xFF495C60, 0xFF819190, 0xFF858B89, 0xFF949391, 0xFF969694, 0xFF5B6B68, 0xFF406158, 0xFF5C8877, 0xFF5B8776, 0xFF5B8675, 0xFF5A8574, 0xFF5A8574,
        0xFF5B8474, 0xFF5C8575, 0xFF5D8676, 0xFF588571, 0xFF64887A, 0xFF4B6059, 0xFF727B78, 0xFF8E9293, 0xFF92979A, 0xFF96A1A3, 0xFF445454, 0xFF4B6863, 0xFF2E4B47, 0xFF324F4D, 0xFF365255, 0xFF38535A, 0xFF2F4A55, 0xFF4A6473, 0xFF758FA0, 0xFF75A6C7,
        0xFF7FAFD5, 0xFF79A8D4, 0xFF75A3D4, 0xFF73A1D2, 0xFF75A4D0, 0xFF7FAFD5, 0xFF7AABCC, 0xFF7C9CA7, 0xFF55767F, 0xFF39575F, 0xFF244346, 0xFF2A484A, 0xFF284543, 0xFF253F3C, 0xFF506B66, 0xFF49585D, 0xFF717C7E, 0xFF818785, 0xFF929791, 0xFF717B72,
        0xFF43594D, 0xFF55796B, 0xFF5C8879, 0xFF5C8774, 0xFF5C8774, 0xFF5C8774, 0xFF5C8774, 0xFF5C8774, 0xFF5C8774, 0xFF5C8774, 0xFF5C8774, 0xFF59796C, 0xFF42534D, 0xFF878988, 0xFF908E91, 0xFF909598, 0xFF889395, 0xFF464F54, 0xFF64696F, 0xFF3B5758,
        0xFF3B575A, 0xFF365158, 0xFF304C58, 0xFF3B5669, 0xFF58748C, 0xFF7691AE, 0xFF86A1BF, 0xFF77AED7, 0xFF75A9D8, 0xFF6FA1D2, 0xFF6B9BCC, 0xFF74A0CF, 0xFF7EAAD1, 0xFF7CA5C5, 0xFF749EB7, 0xFF6A797E, 0xFF405256, 0xFF395153, 0xFF334F50, 0xFF354F50,
        0xFF485D60, 0xFF5F6C72, 0xFF464F56, 0xFF6F6B6C, 0xFF777978, 0xFF77837F, 0xFF445B51, 0xFF507063, 0xFF5D8472, 0xFF5F8876, 0xFF5E8775, 0xFF5F8674, 0xFF5D8472, 0xFF5E8573, 0xFF5A816F, 0xFF587F6D, 0xFF618876, 0xFF567D6B, 0xFF355C4A, 0xFF8B8D8A,
        0xFF9C9E9D, 0xFF797A7C, 0xFF868A8D, 0xFF828C8E, 0xFF354449, 0xFF687B81, 0xFF51696D, 0xFF415965, 0xFF485E69, 0xFF465A63, 0xFF475963, 0xFF586F7F, 0xFF6F8DA5, 0xFF7DA1C1, 0xFF81ACCF, 0xFF71A7D6, 0xFF72A6D6, 0xFF72A1D5, 0xFF749FD2, 0xFF7DA6D4,
        0xFF83A9CE, 0xFF789DB8, 0xFF668B9E, 0xFF516063, 0xFF495D5E, 0xFF3A5453, 0xFF3F5D5B, 0xFF405E5C, 0xFF577170, 0xFF3C5051, 0xFF667578, 0xFF929992, 0xFF5F6C63, 0xFF556F62, 0xFF5E8978, 0xFF4F826F, 0x00597F72, 0x00F8FFFE, 0x00FFF5FB, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x2ADFEEEB, 0xFF446C61, 0xFF548474, 0xFF648E80, 0xFF597569, 0xFF4C6057, 0xFF859990, 0xFF778288, 0xFF4C5E62, 0xFF435F60, 0xFF1C403E, 0xFF133A37,
        0xFF1C403E, 0xFF2F4D4D, 0xFF304A4B, 0xFF223E3F, 0xFF405D63, 0xFF617E8E, 0xFF7391A9, 0xFF7B9AB7, 0xFF7DA0BE, 0xFF7BA0BD, 0xFF769BB6, 0xFF6590A3, 0xFF4C7788, 0xFF335D6D, 0xFF2D5360, 0xFF2D525B, 0xFF2A4D53, 0xFF234449, 0xFF203F42, 0xFF2F4749,
        0xFF586D70, 0xFF445859, 0xFF647272, 0xFF7A8483, 0xFF707473, 0xFF888A87, 0xFF777873, 0xFF436C5C, 0xFF517A6A, 0xFF5D8676, 0xFF5B8675, 0xFF588372, 0xFF588473, 0xFF598574, 0xFF578372, 0xFF588473, 0xFF5D8978, 0xFF5F8A79, 0xFF567C6F, 0xFF39574D,
        0xFF6D807C, 0xFFA5AEAD, 0xFF8C8D8F, 0xFF546169, 0xFF51646A, 0xFF1B373A, 0xFF1C3E3F, 0xFF1D4443, 0xFF183F3E, 0xFF103434, 0xFF1F4142, 0xFF345E6C, 0xFF4D7688, 0xFF6C94AE, 0xFF769DBE, 0xFF7197BB, 0xFF7399BD, 0xFF759CBB, 0xFF6D94B1, 0xFF577C8E,
        0xFF3E6373, 0xFF2C505E, 0xFF1F414A, 0xFF15363B, 0xFF193938, 0xFF1B3B38, 0xFF1A3934, 0xFF5B6870, 0xFF435559, 0xFF687B79, 0xFF848F8B, 0xFFB2B4B1, 0xFFAFB5B1, 0xFF637A74, 0xFF365E53, 0xFF588473, 0xFF588473, 0xFF578372, 0xFF588372, 0xFF598473,
        0xFF5B8474, 0xFF5C8575, 0xFF5C8575, 0xFF588773, 0xFF5B8172, 0xFF536D64, 0xFFB3C2BD, 0xFFF1FAF9, 0xFFACB6B7, 0xFF6B7B7B, 0xFF435756, 0xFF4C6B65, 0xFF21403B, 0xFF1F3F3C, 0xFF1E3D3F, 0xFF1C3D42, 0xFF21434C, 0xFF325760, 0xFF365A66, 0xFF56829B,
        0xFF6B97B4, 0xFF76A1C4, 0xFF7DA7CD, 0xFF7DA7CD, 0xFF79A4C7, 0xFF729EBB, 0xFF608CA5, 0xFF40676E, 0xFF30575C, 0xFF1F4548, 0xFF113436, 0xFF183A3B, 0xFF1C3C39, 0xFF1F3F3A, 0xFF44635E, 0xFF4F6165, 0xFF697778, 0xFF6F7875, 0xFF838A83, 0xFF6C796F,
        0xFF455D50, 0xFF55796B, 0xFF5C8879, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF496B5D, 0xFF556A63, 0xFFD9DFDD, 0xFFF6FAFB, 0xFFADB9B9, 0xFF718383, 0xFF445255, 0xFF5E656B, 0xFF325455,
        0xFF294B4C, 0xFF28494E, 0xFF30525B, 0xFF345662, 0xFF3A5B6C, 0xFF507386, 0xFF6C8FA3, 0xFF6899BA, 0xFF6F9FC3, 0xFF79A6CD, 0xFF7BA5CD, 0xFF7BA3C7, 0xFF799DBD, 0xFF6B8FA7, 0xFF5A7F91, 0xFF364E50, 0xFF365253, 0xFF30504F, 0xFF294B4A, 0xFF224443,
        0xFF2D4B4B, 0xFF556D6F, 0xFF425559, 0xFF646464, 0xFF6C706F, 0xFF6F7E79, 0xFF455F54, 0xFF527466, 0xFF5E8573, 0xFF608977, 0xFF608977, 0xFF5F8674, 0xFF5C8371, 0xFF5E8573, 0xFF5C8371, 0xFF5A816F, 0xFF618876, 0xFF577E6C, 0xFF3C6351, 0xFF6F7470,
        0xFFB6BAB9, 0xFFD9DFDF, 0xFF929A9C, 0xFF687375, 0xFF4E5D62, 0xFF576C71, 0xFF3E565A, 0xFF2E5059, 0xFF335159, 0xFF344F56, 0xFF3E555D, 0xFF4D6370, 0xFF587283, 0xFF658199, 0xFF7291AE, 0xFF73A2C0, 0xFF77A4C5, 0xFF7BA6C8, 0xFF7DA4C5, 0xFF799CB8,
        0xFF6B8BA0, 0xFF557580, 0xFF446368, 0xFF49575A, 0xFF4A5C5E, 0xFF3D5555, 0xFF395352, 0xFF3B5554, 0xFF576F6F, 0xFF47595B, 0xFF849295, 0xFFC2CDC5, 0xFF79897F, 0xFF5B7568, 0xFF608B7A, 0xFF508370, 0x00547A6D, 0x00F7FFFD, 0x00FFFAFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x2EDEEDEA, 0xFF476D60, 0xFF528272, 0xFF648D7F, 0xFF628074, 0xFF455C52, 0xFF52695F, 0xFF496163, 0xFF466061, 0xFF577374, 0xFF315150, 0xFF2E504F,
        0xFF345856, 0xFF274B49, 0xFF1D413F, 0xFF344B45, 0xFF304946, 0xFF324B50, 0xFF3C5762, 0xFF476371, 0xFF4A6777, 0xFF466675, 0xFF436372, 0xFF27585C, 0xFF1F4F53, 0xFF1B4B4F, 0xFF235053, 0xFF2A5357, 0xFF264E50, 0xFF20484A, 0xFF1F4548, 0xFF3D5054,
        0xFF607377, 0xFF4B5D5F, 0xFF526261, 0xFF657572, 0xFF697873, 0xFF7E8D86, 0xFF7B887F, 0xFF4D7666, 0xFF578070, 0xFF5D8676, 0xFF5A8574, 0xFF588372, 0xFF588473, 0xFF598574, 0xFF578372, 0xFF5D8877, 0xFF5C8776, 0xFF588372, 0xFF5C8678, 0xFF466A5E,
        0xFF405E56, 0xFF516862, 0xFF4D605C, 0xFF536068, 0xFF5C6D74, 0xFF34494E, 0xFF2E4A4D, 0xFF2C4B4D, 0xFF234244, 0xFF163537, 0xFF264446, 0xFF1C403C, 0xFF274B4B, 0xFF3A5D61, 0xFF4A6C76, 0xFF537581, 0xFF537581, 0xFF4A6C76, 0xFF3D5F68, 0xFF295256,
        0xFF20494D, 0xFF224A4C, 0xFF1B4142, 0xFF1F4343, 0xFF2A4C4B, 0xFF264945, 0xFF2A4A47, 0xFF656E77, 0xFF455459, 0xFF506361, 0xFF64736E, 0xFF838C87, 0xFF7C8B84, 0xFF527169, 0xFF427264, 0xFF558472, 0xFF578372, 0xFF578372, 0xFF588473, 0xFF598473,
        0xFF5B8675, 0xFF5D8676, 0xFF5D8676, 0xFF568772, 0xFF608B7A, 0xFF4B6B60, 0xFF728981, 0xFFA3B6B2, 0xFF768987, 0xFF435957, 0xFF415B58, 0xFF536D6A, 0xFF304D49, 0xFF2F4D4D, 0xFF264849, 0xFF1A4043, 0xFF163F43, 0xFF224D53, 0xFF214E54, 0xFF355665,
        0xFF436475, 0xFF4D6D82, 0xFF57778E, 0xFF5E7E95, 0xFF59798E, 0xFF48697A, 0xFF2F505F, 0xFF20494D, 0xFF1C4547, 0xFF123A3C, 0xFF123839, 0xFF1D413F, 0xFF1D403C, 0xFF284B47, 0xFF4A6A65, 0xFF475A5E, 0xFF556767, 0xFF596863, 0xFF758279, 0xFF708276,
        0xFF506C5E, 0xFF597F70, 0xFF5C8879, 0xFF5A8572, 0xFF5A8572, 0xFF5A8572, 0xFF5A8572, 0xFF5A8572, 0xFF5A8572, 0xFF5A8572, 0xFF5A8572, 0xFF598272, 0xFF506B62, 0xFFA3B2AD, 0xFFB2C1BE, 0xFF637775, 0xFF38524F, 0xFF405455, 0xFF606E71, 0xFF224446,
        0xFF1E4143, 0xFF204347, 0xFF2A4D51, 0xFF30555B, 0xFF31565C, 0xFF335A61, 0xFF375E65, 0xFF51778C, 0xFF587E95, 0xFF6488A0, 0xFF6789A2, 0xFF5D7D92, 0xFF517180, 0xFF415F6A, 0xFF2A474F, 0xFF264645, 0xFF3D5F5E, 0xFF244B48, 0xFF1A413E, 0xFF153C39,
        0xFF224443, 0xFF557373, 0xFF415D5E, 0xFF5B615F, 0xFF626D69, 0xFF687B75, 0xFF4A685C, 0xFF577B6D, 0xFF5F8674, 0xFF5E8775, 0xFF5F8876, 0xFF5E8775, 0xFF5A8371, 0xFF5C8573, 0xFF5D8674, 0xFF5A8371, 0xFF5E8775, 0xFF58816F, 0xFF466F5D, 0xFF85908A,
        0xFFF3FEFA, 0xFFD2DEDC, 0xFF576766, 0xFF405252, 0xFF3E5155, 0xFF51666B, 0xFF3E5358, 0xFF194248, 0xFF1C4147, 0xFF274649, 0xFF395257, 0xFF435A60, 0xFF405661, 0xFF445A68, 0xFF546B7B, 0xFF587A84, 0xFF577985, 0xFF5C7987, 0xFF607C88, 0xFF5D757F,
        0xFF4C6466, 0xFF394F4C, 0xFF2C413A, 0xFF3E4C4F, 0xFF455457, 0xFF3E5253, 0xFF314647, 0xFF354A4B, 0xFF546869, 0xFF425154, 0xFF829093, 0xFF94A49A, 0xFF607167, 0xFF557163, 0xFF648F7E, 0xFF5A8B78, 0x00587E71, 0x00EFFAF6, 0x00FFF5F9, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEF9FD, 0x31DBEAE5, 0xFF496F62, 0xFF568273, 0xFF618778, 0xFF68867A, 0xFF536F61, 0xFF4E6D5E, 0xFF3B5758, 0xFF405A5B, 0xFF607578, 0xFF45585C, 0xFF465B5E,
        0xFF496364, 0xFF2B4B4A, 0xFF1F4341, 0xFF304E4E, 0xFF203F41, 0xFF133235, 0xFF12333A, 0xFF193B44, 0xFF1F414B, 0xFF214551, 0xFF234753, 0xFF1F4A43, 0xFF214A44, 0xFF1E4743, 0xFF1E4542, 0xFF2C5050, 0xFF3B5E60, 0xFF3A5C5E, 0xFF2E4F54, 0xFF4B5A5F,
        0xFF5B6D71, 0xFF425456, 0xFF3A4E4C, 0xFF4D645E, 0xFF567067, 0xFF617D71, 0xFF637F71, 0xFF567F6F, 0xFF5B8474, 0xFF5D8676, 0xFF5A8574, 0xFF598473, 0xFF5B8776, 0xFF5B8776, 0xFF598574, 0xFF5E8475, 0xFF598272, 0xFF578271, 0xFF649081, 0xFF588475,
        0xFF3B6358, 0xFF33574D, 0xFF34554C, 0xFF556469, 0xFF617075, 0xFF43565A, 0xFF485D60, 0xFF4B6566, 0xFF3A5455, 0xFF223C3D, 0xFF274344, 0xFF37524B, 0xFF3B5651, 0xFF304A47, 0xFF304A49, 0xFF415B5C, 0xFF3C5657, 0xFF304A4B, 0xFF385253, 0xFF315450,
        0xFF30534F, 0xFF284845, 0xFF193735, 0xFF355152, 0xFF4B6566, 0xFF3D5758, 0xFF4E6668, 0xFF666F78, 0xFF46555A, 0xFF465C59, 0xFF556C64, 0xFF5A6E65, 0xFF546B61, 0xFF50786D, 0xFF5A9080, 0xFF578674, 0xFF578674, 0xFF598574, 0xFF5A8675, 0xFF5A8675,
        0xFF5C8776, 0xFF5D8877, 0xFF5D8877, 0xFF538771, 0xFF679684, 0xFF5F8578, 0xFF46665B, 0xFF4A655E, 0xFF4E6964, 0xFF3E5D58, 0xFF395954, 0xFF5D6B6C, 0xFF435255, 0xFF3A4D51, 0xFF395256, 0xFF3A595C, 0xFF1E4447, 0xFF1A4347, 0xFF255154, 0xFF405C5F,
        0xFF3C585C, 0xFF365256, 0xFF355057, 0xFF355057, 0xFF345054, 0xFF334F53, 0xFF2F4B4E, 0xFF294B4D, 0xFF224446, 0xFF1A393C, 0xFF325052, 0xFF446061, 0xFF385251, 0xFF3B5554, 0xFF4F6767, 0xFF4A6264, 0xFF586E6C, 0xFF596E67, 0xFF6F8579, 0xFF718C7D,
        0xFF5E7E6F, 0xFF5F8576, 0xFF5A8677, 0xFF5A8572, 0xFF5A8572, 0xFF5A8572, 0xFF5A8572, 0xFF5A8572, 0xFF5A8572, 0xFF5A8572, 0xFF5A8572, 0xFF5A8675, 0xFF4D6E63, 0xFF5A7169, 0xFF5F7670, 0xFF45645E, 0xFF31544E, 0xFF3D5756, 0xFF556769, 0xFF2D484F,
        0xFF314C53, 0xFF334F53, 0xFF365256, 0xFF3A585A, 0xFF3E5C5C, 0xFF375756, 0xFF2F4F4C, 0xFF335058, 0xFF3B5860, 0xFF446169, 0xFF48636A, 0xFF3F585D, 0xFF3A5357, 0xFF3D5555, 0xFF324B48, 0xFF264444, 0xFF40605F, 0xFF315150, 0xFF30504F, 0xFF314F4F,
        0xFF3C5859, 0xFF5C7879, 0xFF405A5B, 0xFF6C7B76, 0xFF6E817B, 0xFF748E85, 0xFF5E8072, 0xFF618778, 0xFF5C8573, 0xFF57806E, 0xFF57806E, 0xFF5E8775, 0xFF598270, 0xFF5C8573, 0xFF5F8876, 0xFF5D8674, 0xFF5D8674, 0xFF5A8371, 0xFF527B69, 0xFF6A7B73,
        0xFFBED1CB, 0xFF788F89, 0xFF3F5854, 0xFF516A67, 0xFF3E5658, 0xFF54696E, 0xFF394E53, 0xFF395A61, 0xFF38575C, 0xFF365357, 0xFF385457, 0xFF3D5758, 0xFF385253, 0xFF385253, 0xFF425C5D, 0xFF465E60, 0xFF465B5E, 0xFF46595D, 0xFF47595B, 0xFF4B5B5B,
        0xFF4D5C59, 0xFF4F5C55, 0xFF4E5B52, 0xFF3B494C, 0xFF404F52, 0xFF49585B, 0xFF405254, 0xFF455759, 0xFF617073, 0xFF425154, 0xFF6F7D80, 0xFF849A8E, 0xFF6B8376, 0xFF658475, 0xFF608979, 0xFF4F7E6C, 0x00597D71, 0x00F8FFFF, 0x00FFFBFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFBFC, 0x35D9E8E3, 0xFF4A6E62, 0xFF598473, 0xFF5C8273, 0xFF69897A, 0xFF628273, 0xFF5C8170, 0xFF4B6564, 0xFF425657, 0xFF647275, 0xFF5E656B, 0xFF5C656A,
        0xFF586669, 0xFF34494A, 0xFF264341, 0xFF1B3836, 0xFF1C3937, 0xFF1C3937, 0xFF1A3836, 0xFF1B3937, 0xFF1D3B3B, 0xFF1F3D3F, 0xFF1F3C40, 0xFF37524B, 0xFF3E5952, 0xFF36514C, 0xFF253F3C, 0xFF274142, 0xFF3D5559, 0xFF4A6266, 0xFF475E64, 0xFF57696D,
        0xFF55686C, 0xFF3C5152, 0xFF344F4A, 0xFF48675F, 0xFF517267, 0xFF4E7465, 0xFF517866, 0xFF578070, 0xFF5A8373, 0xFF5A8373, 0xFF588372, 0xFF598473, 0xFF5A8675, 0xFF5B8776, 0xFF5A8675, 0xFF597D6F, 0xFF5D8676, 0xFF5A8675, 0xFF568575, 0xFF568676,
        0xFF558376, 0xFF517B6F, 0xFF41695E, 0xFF495C60, 0xFF596B6F, 0xFF55676B, 0xFF607377, 0xFF5D7074, 0xFF395153, 0xFF193334, 0xFF203A3B, 0xFF3A4F52, 0xFF405556, 0xFF3B5051, 0xFF3C5152, 0xFF44595A, 0xFF3B5053, 0xFF34494C, 0xFF42575C, 0xFF495F5C,
        0xFF455B58, 0xFF364A49, 0xFF223635, 0xFF3C4E50, 0xFF556469, 0xFF49565C, 0xFF58656B, 0xFF67717A, 0xFF3A4F52, 0xFF32514B, 0xFF4F6F64, 0xFF567266, 0xFF557568, 0xFF568275, 0xFF4D8876, 0xFF578674, 0xFF588775, 0xFF588775, 0xFF5A8675, 0xFF5A8675,
        0xFF5A8675, 0xFF5A8574, 0xFF5A8574, 0xFF578873, 0xFF507F6D, 0xFF5E887A, 0xFF5A8073, 0xFF4D7167, 0xFF4C6F68, 0xFF3E615B, 0xFF32554F, 0xFF65686F, 0xFF5A5F65, 0xFF596269, 0xFF57666D, 0xFF4C6166, 0xFF233C41, 0xFF1D3A3E, 0xFF2C4D52, 0xFF4A5F60,
        0xFF455B59, 0xFF445A58, 0xFF415754, 0xFF374D4A, 0xFF304644, 0xFF374D4B, 0xFF495E5F, 0xFF475C61, 0xFF374C51, 0xFF24373B, 0xFF3C4F53, 0xFF57696D, 0xFF536267, 0xFF536164, 0xFF566467, 0xFF355152, 0xFF46635F, 0xFF49675D, 0xFF537263, 0xFF5C7D6C,
        0xFF597E6D, 0xFF5B8474, 0xFF598576, 0xFF5A8572, 0xFF5A8572, 0xFF5A8572, 0xFF5A8572, 0xFF5A8572, 0xFF5A8572, 0xFF5A8572, 0xFF5A8572, 0xFF578674, 0xFF5E8276, 0xFF4D6B61, 0xFF46655D, 0xFF3F675E, 0xFF2C554D, 0xFF385654, 0xFF5D7172, 0xFF515F6A,
        0xFF515F6A, 0xFF515F68, 0xFF4C5D64, 0xFF44565A, 0xFF3F5354, 0xFF455958, 0xFF506462, 0xFF395153, 0xFF364E50, 0xFF314949, 0xFF394F4C, 0xFF394F4C, 0xFF39504A, 0xFF3A504D, 0xFF2A403D, 0xFF2B4345, 0xFF3B5053, 0xFF44575B, 0xFF4E6064, 0xFF4A5C60,
        0xFF516367, 0xFF596B6F, 0xFF3E5054, 0xFF4A6159, 0xFF4E6960, 0xFF59796E, 0xFF547A6B, 0xFF5C8776, 0xFF5D8875, 0xFF5F8876, 0xFF648B79, 0xFF5F8876, 0xFF5A8371, 0xFF5C8573, 0xFF608977, 0xFF5E8775, 0xFF5C8573, 0xFF5C8573, 0xFF5A8371, 0xFF587267,
        0xFF557067, 0xFF44655C, 0xFF345750, 0xFF32524D, 0xFF385455, 0xFF526A6E, 0xFF56696F, 0xFF4E5F67, 0xFF4E6168, 0xFF41585E, 0xFF345053, 0xFF365454, 0xFF3A5A57, 0xFF355851, 0xFF32564C, 0xFF3F5057, 0xFF3D4C51, 0xFF374649, 0xFF384647, 0xFF485355,
        0xFF586365, 0xFF5D6769, 0xFF586264, 0xFF3E5052, 0xFF405254, 0xFF59686B, 0xFF5B6A6D, 0xFF576669, 0xFF69787B, 0xFF445658, 0xFF5A6C6E, 0xFF5F7E6F, 0xFF627E70, 0xFF69897A, 0xFF608979, 0xFF4C7B69, 0x00597D71, 0x00F5FFFD, 0x00FFF9FC, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x39D7E6DF, 0xFF486A5C, 0xFF5D8676, 0xFF5D8271, 0xFF678778, 0xFF628775, 0xFF5B8672, 0xFF516B6A, 0xFF384D4E, 0xFF536265, 0xFF5B6469, 0xFF596267,
        0xFF515F62, 0xFF334748, 0xFF284040, 0xFF3C4D47, 0xFF3E4F45, 0xFF405244, 0xFF455646, 0xFF50614F, 0xFF5A6B5B, 0xFF58685D, 0xFF516157, 0xFF324541, 0xFF3D504E, 0xFF425553, 0xFF384A4A, 0xFF2F4344, 0xFF374B4C, 0xFF4B5E62, 0xFF5A6D71, 0xFF4D6567,
        0xFF465E60, 0xFF395654, 0xFF405F59, 0xFF54786E, 0xFF598274, 0xFF55806F, 0xFF56836F, 0xFF557E6E, 0xFF557E6E, 0xFF567F6F, 0xFF568170, 0xFF578271, 0xFF588473, 0xFF598574, 0xFF598574, 0xFF608476, 0xFF5E8777, 0xFF5C8877, 0xFF528171, 0xFF548474,
        0xFF5A887B, 0xFF5C887B, 0xFF4C7469, 0xFF455F60, 0xFF546C6E, 0xFF607377, 0xFF6C7E82, 0xFF607377, 0xFF415659, 0xFF2E4849, 0xFF3C5859, 0xFF4B616C, 0xFF435A62, 0xFF556D71, 0xFF778F91, 0xFF88A0A0, 0xFF839B9D, 0xFF6B8288, 0xFF4F666E, 0xFF566A69,
        0xFF4B5F5E, 0xFF455757, 0xFF3A494C, 0xFF425053, 0xFF5E696D, 0xFF687377, 0xFF707980, 0xFF596C72, 0xFF3A5858, 0xFF396158, 0xFF578072, 0xFF5C8072, 0xFF618577, 0xFF5D8B7E, 0xFF508978, 0xFF558673, 0xFF558673, 0xFF578674, 0xFF578674, 0xFF598574,
        0xFF588473, 0xFF578372, 0xFF588372, 0xFF608D79, 0xFF568271, 0xFF5C8678, 0xFF5C8678, 0xFF598176, 0xFF5C8179, 0xFF4D726B, 0xFF3C5F59, 0xFF5D616A, 0xFF60676F, 0xFF727B82, 0xFF6A777D, 0xFF4D5E65, 0xFF2F4449, 0xFF3B5458, 0xFF40595D, 0xFF54696C,
        0xFF516765, 0xFF5F7572, 0xFF6C837B, 0xFF6B827A, 0xFF5E7471, 0xFF536967, 0xFF586D70, 0xFF4E6263, 0xFF465A5B, 0xFF334547, 0xFF404F52, 0xFF5C696F, 0xFF677278, 0xFF657076, 0xFF596269, 0xFF3B5B5A, 0xFF50736D, 0xFF577B6F, 0xFF557C6A, 0xFF59836F,
        0xFF5B8472, 0xFF598473, 0xFF588274, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8673, 0xFF5B8776, 0xFF648A7D, 0xFF5F8075, 0xFF5F857A, 0xFF568177, 0xFF38635A, 0xFF3A5856, 0xFF596B6D, 0xFF626C78,
        0xFF64717A, 0xFF616E77, 0xFF55626A, 0xFF4C5B60, 0xFF4E5D62, 0xFF506264, 0xFF516365, 0xFF4D6265, 0xFF596E6F, 0xFF576D6A, 0xFF586F69, 0xFF4F6960, 0xFF47605A, 0xFF516A67, 0xFF516969, 0xFF415659, 0xFF405256, 0xFF505D63, 0xFF646F75, 0xFF656E75,
        0xFF6B767C, 0xFF5D6A70, 0xFF47565B, 0xFF57776C, 0xFF5B7C71, 0xFF5F8578, 0xFF5B8675, 0xFF5C8877, 0xFF5A8572, 0xFF5B8472, 0xFF5D8472, 0xFF5E8976, 0xFF5B8673, 0xFF5C8774, 0xFF5F8A77, 0xFF5E8976, 0xFF5B8673, 0xFF5B8673, 0xFF5C8774, 0xFF69897C,
        0xFF567A6E, 0xFF598377, 0xFF467167, 0xFF2A534B, 0xFF375756, 0xFF4C6468, 0xFF67787F, 0xFF6F787F, 0xFF68757D, 0xFF4D6168, 0xFF345054, 0xFF355759, 0xFF436765, 0xFF486D65, 0xFF466C61, 0xFF5C6D77, 0xFF6D7E85, 0xFF75878B, 0xFF67797B, 0xFF5A696C,
        0xFF57666D, 0xFF5A6873, 0xFF5C6A77, 0xFF3E5656, 0xFF425758, 0xFF5C6E70, 0xFF647376, 0xFF576669, 0xFF607274, 0xFF4A5F60, 0xFF5B7373, 0xFF668B7A, 0xFF648676, 0xFF678979, 0xFF608979, 0xFF517D6C, 0x00587C70, 0x00F2FEFA, 0x00FFFDFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFD, 0x3DD5E4DD, 0xFF46685A, 0xFF608775, 0xFF5E8372, 0xFF648574, 0xFF5F8673, 0xFF54846E, 0xFF4F726C, 0xFF30504B, 0xFF314E4A, 0xFF304A47, 0xFF304946,
        0xFF334D4A, 0xFF274440, 0xFF264540, 0xFF233535, 0xFF1D2E28, 0xFF16281C, 0xFF192B1B, 0xFF283927, 0xFF314230, 0xFF2F3D30, 0xFF26332A, 0xFF172C2D, 0xFF172C2D, 0xFF1C3132, 0xFF233B3B, 0xFF2A4242, 0xFF2C4645, 0xFF2F4948, 0xFF324C49, 0xFF2C4A4A,
        0xFF2E4E4D, 0xFF335652, 0xFF42675F, 0xFF4F776C, 0xFF537D6F, 0xFF527E6D, 0xFF51806C, 0xFF547D6D, 0xFF557E6E, 0xFF578070, 0xFF588372, 0xFF598473, 0xFF588473, 0xFF598574, 0xFF5A8675, 0xFF628879, 0xFF5C8575, 0xFF5F8B7A, 0xFF5B8A7A, 0xFF608F7F,
        0xFF5D877B, 0xFF5F877C, 0xFF5F8379, 0xFF476764, 0xFF3D5A58, 0xFF3B5353, 0xFF465A5B, 0xFF4B5F60, 0xFF425A5A, 0xFF385553, 0xFF3A5A57, 0xFF2E4954, 0xFF1C373E, 0xFF2D494A, 0xFF53706C, 0xFF66837E, 0xFF668381, 0xFF4B676A, 0xFF203B42, 0xFF233F42,
        0xFF2B474A, 0xFF3F585C, 0xFF486062, 0xFF475F61, 0xFF4E6364, 0xFF4D6162, 0xFF455958, 0xFF324E51, 0xFF375E5B, 0xFF437569, 0xFF598979, 0xFF598272, 0xFF5D8374, 0xFF578376, 0xFF568B7B, 0xFF568774, 0xFF578875, 0xFF588976, 0xFF598876, 0xFF598876,
        0xFF5B8776, 0xFF5A8675, 0xFF5A8675, 0xFF537C6A, 0xFF5F8A79, 0xFF5D8779, 0xFF557F71, 0xFF557F73, 0xFF5D857C, 0xFF567B74, 0xFF385B55, 0xFF43585B, 0xFF43585B, 0xFF475F61, 0xFF466061, 0xFF466061, 0xFF405C5D, 0xFF496767, 0xFF3B5959, 0xFF294249,
        0xFF375054, 0xFF5E7875, 0xFF7C9792, 0xFF849F9A, 0xFF6D8784, 0xFF40595D, 0xFF294249, 0xFF36534F, 0xFF3F5C58, 0xFF405A59, 0xFF475F5F, 0xFF546C6C, 0xFF5B7073, 0xFF526569, 0xFF44575B, 0xFF3B5F5D, 0xFF507870, 0xFF5A8476, 0xFF578470, 0xFF5C8974,
        0xFF5D8A76, 0xFF588473, 0xFF5D8779, 0xFF5C8774, 0xFF5C8774, 0xFF5C8774, 0xFF5C8774, 0xFF5C8774, 0xFF5C8774, 0xFF5C8774, 0xFF5C8774, 0xFF5A8574, 0xFF597D71, 0xFF5D7E73, 0xFF5D857A, 0xFF59877C, 0xFF4A756C, 0xFF3C5A58, 0xFF465558, 0xFF50656A,
        0xFF586D72, 0xFF566A71, 0xFF4A6167, 0xFF4A6167, 0xFF50696E, 0xFF486166, 0xFF375055, 0xFF445B63, 0xFF799195, 0xFF92ACA9, 0xFF93AEA9, 0xFF708B86, 0xFF3F5C58, 0xFF395558, 0xFF3E5960, 0xFF506E6E, 0xFF526C6D, 0xFF586B6F, 0xFF67767B, 0xFF657278,
        0xFF606F74, 0xFF475C5F, 0xFF395153, 0xFF577D70, 0xFF5A8375, 0xFF5B8778, 0xFF598876, 0xFF588775, 0xFF598672, 0xFF5D8674, 0xFF5F8674, 0xFF5D8875, 0xFF5D8875, 0xFF5D8875, 0xFF5E8976, 0xFF5E8976, 0xFF5C8774, 0xFF5B8673, 0xFF5B8673, 0xFF5D8374,
        0xFF679183, 0xFF568678, 0xFF45796C, 0xFF326157, 0xFF315553, 0xFF3C5458, 0xFF4D5C63, 0xFF526261, 0xFF526667, 0xFF4A6368, 0xFF436169, 0xFF44626A, 0xFF3E5B5F, 0xFF314D4E, 0xFF2D4643, 0xFF597276, 0xFF799394, 0xFF8DA7A4, 0xFF76918C, 0xFF496360,
        0xFF314A4E, 0xFF3C545E, 0xFF506775, 0xFF4C6A68, 0xFF4C6967, 0xFF586D6E, 0xFF586A6C, 0xFF485A5C, 0xFF485D5E, 0xFF3E5B59, 0xFF587674, 0xFF618C79, 0xFF608574, 0xFF638877, 0xFF628B7B, 0xFF588473, 0x00597D71, 0x00ECFBF6, 0x00FFFBFC, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FBFBF9, 0x41D3E2DB, 0xFF47675A, 0xFF608775, 0xFF5F8171, 0xFF618372, 0xFF5D8773, 0xFF54856F, 0xFF5D8B81, 0xFF457067, 0xFF376259, 0xFF2D564E, 0xFF2B544C,
        0xFF315951, 0xFF2A524A, 0xFF2D524B, 0xFF2D4B56, 0xFF2E4C54, 0xFF314F4F, 0xFF33524A, 0xFF345248, 0xFF314C45, 0xFF2C4643, 0xFF294344, 0xFF274346, 0xFF294548, 0xFF2C4A4A, 0xFF304E4E, 0xFF30504D, 0xFF2E514B, 0xFF2E514B, 0xFF2F524B, 0xFF284A49,
        0xFF335755, 0xFF456965, 0xFF527A71, 0xFF537D71, 0xFF517D6E, 0xFF558170, 0xFF51806C, 0xFF578070, 0xFF598272, 0xFF5B8474, 0xFF5C8776, 0xFF5D8877, 0xFF5B8776, 0xFF5B8776, 0xFF5C8877, 0xFF5A8071, 0xFF608979, 0xFF659180, 0xFF588475, 0xFF5E8A7B,
        0xFF5F877C, 0xFF65897F, 0xFF67867E, 0xFF5B7F7B, 0xFF4C6C69, 0xFF476160, 0xFF485D5E, 0xFF495D5E, 0xFF486060, 0xFF456361, 0xFF416460, 0xFF46646C, 0xFF416063, 0xFF43635E, 0xFF44655A, 0xFF3E5F54, 0xFF3B5C53, 0xFF3E5E5B, 0xFF3F5E60, 0xFF355C61,
        0xFF3E6369, 0xFF3A6063, 0xFF385C5C, 0xFF3C5E5D, 0xFF3C5C59, 0xFF3D5C57, 0xFF405F5A, 0xFF466567, 0xFF537D79, 0xFF53897B, 0xFF578C7A, 0xFF5D8877, 0xFF658B7C, 0xFF588276, 0xFF5F8F81, 0xFF598A77, 0xFF5A8B78, 0xFF5B8C79, 0xFF5D8C7A, 0xFF5E8D7B,
        0xFF5E8D7B, 0xFF608C7B, 0xFF5F8B7A, 0xFF678E7C, 0xFF648A7B, 0xFF578173, 0xFF5F897B, 0xFF5E887C, 0xFF5D857C, 0xFF668983, 0xFF557570, 0xFF325654, 0xFF385C5A, 0xFF2D514F, 0xFF264A48, 0xFF335755, 0xFF2A4E4C, 0xFF2D514F, 0xFF284C48, 0xFF314D58,
        0xFF2F4C52, 0xFF345252, 0xFF2D4C47, 0xFF2B4A45, 0xFF314F4F, 0xFF2F4C52, 0xFF35515C, 0xFF265046, 0xFF2F574F, 0xFF335B53, 0xFF335753, 0xFF335652, 0xFF325251, 0xFF335151, 0xFF375557, 0xFF4C7370, 0xFF5B847C, 0xFF608C7D, 0xFF578672, 0xFF5A8B75,
        0xFF5B8A76, 0xFF5A8675, 0xFF679183, 0xFF5D8875, 0xFF5D8875, 0xFF5D8875, 0xFF5D8875, 0xFF5D8875, 0xFF5D8875, 0xFF5D8875, 0xFF5D8875, 0xFF5A8373, 0xFF608478, 0xFF67887D, 0xFF5A8277, 0xFF59897D, 0xFF5B867D, 0xFF476462, 0xFF475558, 0xFF2F4E50,
        0xFF2C4B4D, 0xFF2C4E4F, 0xFF335557, 0xFF325358, 0xFF284B51, 0xFF22454B, 0xFF22444D, 0xFF273F4B, 0xFF3A535A, 0xFF2C4849, 0xFF2F4C48, 0xFF31504B, 0xFF214042, 0xFF234149, 0xFF25434E, 0xFF224946, 0xFF355756, 0xFF324E4F, 0xFF3B5053, 0xFF364B4E,
        0xFF334B4D, 0xFF365454, 0xFF466867, 0xFF5D8678, 0xFF5F8B7C, 0xFF5D8C7C, 0xFF5B8C79, 0xFF578875, 0xFF5A8773, 0xFF608977, 0xFF608775, 0xFF5D8875, 0xFF5E8976, 0xFF5D8875, 0xFF5D8875, 0xFF5D8875, 0xFF5C8774, 0xFF5A8572, 0xFF5A8572, 0xFF5C8776,
        0xFF588777, 0xFF5A8F7F, 0xFF568C7E, 0xFF3E7065, 0xFF426966, 0xFF3E565A, 0xFF3B4A51, 0xFF37504A, 0xFF314E4A, 0xFF2D4C4F, 0xFF31525B, 0xFF36545E, 0xFF304852, 0xFF304149, 0xFF3D484E, 0xFF2D4B49, 0xFF2B4A45, 0xFF294840, 0xFF2A4A3F, 0xFF2A4B42,
        0xFF294946, 0xFF29484D, 0xFF294752, 0xFF2C4F4B, 0xFF304E4C, 0xFF324A4A, 0xFF384C4D, 0xFF384C4D, 0xFF3E5656, 0xFF446260, 0xFF688B87, 0xFF608D79, 0xFF648B79, 0xFF688D7C, 0xFF618A7A, 0xFF588372, 0x005E7F74, 0x00EFFEF9, 0x00FFFEFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x41D3DFDB, 0xFF446459, 0xFF5E8777, 0xFF628678, 0xFF658777, 0xFF5C8371, 0xFF5A8773, 0xFF598471, 0xFF588370, 0xFF588370, 0xFF57826F, 0xFF55806D,
        0xFF537E6B, 0xFF527D6A, 0xFF517C69, 0xFF507C6B, 0xFF517D6C, 0xFF527E6D, 0xFF537F6E, 0xFF537F6E, 0xFF537F6E, 0xFF537F6E, 0xFF527E6D, 0xFF4E7A69, 0xFF4D7968, 0xFF4D7968, 0xFF4C7867, 0xFF4B7766, 0xFF4B7766, 0xFF4A7665, 0xFF4A7665, 0xFF4D7666,
        0xFF4D7666, 0xFF4D7666, 0xFF4E7767, 0xFF517A6A, 0xFF547D6D, 0xFF578070, 0xFF598272, 0xFF598272, 0xFF598272, 0xFF5A8373, 0xFF5A8373, 0xFF5B8474, 0xFF5B8474, 0xFF5C8575, 0xFF5C8575, 0xFF5E8777, 0xFF5F8878, 0xFF5F8878, 0xFF5F8878, 0xFF5F8878,
        0xFF608979, 0xFF608979, 0xFF608979, 0xFF628B7B, 0xFF638C7C, 0xFF638C7C, 0xFF648D7D, 0xFF658E7E, 0xFF658E7E, 0xFF668F7F, 0xFF668F7F, 0xFF669082, 0xFF649081, 0xFF628E7F, 0xFF608C7D, 0xFF5C8B7B, 0xFF5A8979, 0xFF578777, 0xFF568676, 0xFF6A9383,
        0xFF699282, 0xFF668F7F, 0xFF648D7D, 0xFF628B7B, 0xFF628B7B, 0xFF638C7C, 0xFF648D7D, 0xFF5D8676, 0xFF5D8676, 0xFF5D8676, 0xFF5D8676, 0xFF5D8676, 0xFF5D8676, 0xFF5D8676, 0xFF5D8676, 0xFF5D8676, 0xFF5E8777, 0xFF5E8777, 0xFF5E8777, 0xFF5F8878,
        0xFF5F8878, 0xFF5F8878, 0xFF608979, 0xFF5F8B7A, 0xFF5E8A79, 0xFF5C8877, 0xFF5B8776, 0xFF5B8776, 0xFF5C8877, 0xFF5E8A79, 0xFF5F8B7A, 0xFF638577, 0xFF648678, 0xFF648678, 0xFF658779, 0xFF66887A, 0xFF66887A, 0xFF67897B, 0xFF67897B, 0xFF658B7C,
        0xFF628879, 0xFF5E8475, 0xFF5B8172, 0xFF5A8071, 0xFF5B8172, 0xFF5E8475, 0xFF608677, 0xFF65897B, 0xFF65897B, 0xFF64887A, 0xFF64887A, 0xFF64887A, 0xFF64887A, 0xFF64887A, 0xFF64887A, 0xFF658B7C, 0xFF63897A, 0xFF618778, 0xFF608677, 0xFF608677,
        0xFF618778, 0xFF63897A, 0xFF658B7C, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF608775, 0xFF5F8F79, 0xFF5E8E78, 0xFF5C8C76, 0xFF5C8C76, 0xFF5D8D77, 0xFF5B8B75, 0xFF588872, 0xFF55856F, 0xFF5D7D70,
        0xFF5E7E71, 0xFF608073, 0xFF608073, 0xFF608073, 0xFF5E7E71, 0xFF5C7C6F, 0xFF5B7B6E, 0xFF478071, 0xFF457B6D, 0xFF43776A, 0xFF49796D, 0xFF507E73, 0xFF588278, 0xFF588077, 0xFF557D74, 0xFF5C7B73, 0xFF597870, 0xFF56756D, 0xFF55746C, 0xFF57766E,
        0xFF5C7B73, 0xFF628179, 0xFF66857D, 0xFF598A77, 0xFF598A77, 0xFF588976, 0xFF588976, 0xFF588976, 0xFF588976, 0xFF578875, 0xFF578875, 0xFF598A77, 0xFF598A77, 0xFF598A77, 0xFF598A77, 0xFF598A77, 0xFF598A77, 0xFF598A77, 0xFF598A77, 0xFF608677,
        0xFF618778, 0xFF63897A, 0xFF63897A, 0xFF618778, 0xFF5D8374, 0xFF587E6F, 0xFF547A6B, 0xFF54756C, 0xFF55766D, 0xFF57786F, 0xFF597A71, 0xFF597A71, 0xFF597A71, 0xFF57786F, 0xFF57786F, 0xFF416F65, 0xFF457067, 0xFF49726A, 0xFF4D756D, 0xFF52756F,
        0xFF557570, 0xFF577470, 0xFF577470, 0xFF5A786C, 0xFF577569, 0xFF547266, 0xFF557367, 0xFF5A786C, 0xFF617F73, 0xFF678579, 0xFF6A887C, 0xFF5E8978, 0xFF688C7E, 0xFF628476, 0xFF648A7B, 0xFF598574, 0x005C8273, 0x00E7FCF3, 0x00FCFFFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFAFB, 0x41D4E3DE, 0xFF45695D, 0xFF598876, 0xFF5A8574, 0xFF5B806F, 0xFF587F6D, 0xFF598471, 0xFF5D8472, 0xFF5D8472, 0xFF5D8472, 0xFF5D8472, 0xFF5D8472,
        0xFF5C8371, 0xFF5B8270, 0xFF5B8270, 0xFF598272, 0xFF588171, 0xFF588171, 0xFF578070, 0xFF567F6F, 0xFF557E6E, 0xFF557E6E, 0xFF547D6D, 0xFF557E6E, 0xFF547D6D, 0xFF547D6D, 0xFF537C6C, 0xFF527B6B, 0xFF527B6B, 0xFF517A6A, 0xFF517A6A, 0xFF527D6C,
        0xFF517C6B, 0xFF517C6B, 0xFF517C6B, 0xFF527D6C, 0xFF547F6E, 0xFF568170, 0xFF578271, 0xFF578271, 0xFF578271, 0xFF588372, 0xFF598473, 0xFF598473, 0xFF5A8574, 0xFF5B8675, 0xFF5B8675, 0xFF5E8978, 0xFF5E8978, 0xFF5E8978, 0xFF5F8A79, 0xFF5F8A79,
        0xFF5F8A79, 0xFF5F8A79, 0xFF5F8A79, 0xFF628D7C, 0xFF628D7C, 0xFF628D7C, 0xFF628D7C, 0xFF638E7D, 0xFF638E7D, 0xFF638E7D, 0xFF638E7D, 0xFF628E7F, 0xFF628E7F, 0xFF608F7F, 0xFF608F7F, 0xFF5F8F7F, 0xFF5F8F7F, 0xFF5F8F7F, 0xFF5C8F7E, 0xFF618C7B,
        0xFF618C7B, 0xFF638E7D, 0xFF648F7E, 0xFF65907F, 0xFF648F7E, 0xFF648F7E, 0xFF638E7D, 0xFF628D7C, 0xFF628D7C, 0xFF628D7C, 0xFF628D7C, 0xFF628D7C, 0xFF628D7C, 0xFF628D7C, 0xFF628D7C, 0xFF5D8877, 0xFF5E8978, 0xFF5E8978, 0xFF5E8978, 0xFF5E8978,
        0xFF5E8978, 0xFF5E8978, 0xFF5E8978, 0xFF658B7C, 0xFF648A7B, 0xFF63897A, 0xFF628879, 0xFF628879, 0xFF63897A, 0xFF648A7B, 0xFF658B7C, 0xFF658B7C, 0xFF658B7C, 0xFF668C7D, 0xFF668C7D, 0xFF678D7E, 0xFF688E7F, 0xFF688E7F, 0xFF688E7F, 0xFF6B9484,
        0xFF6A9383, 0xFF689181, 0xFF668F7F, 0xFF658E7E, 0xFF668F7F, 0xFF679080, 0xFF689181, 0xFF668F7F, 0xFF668F7F, 0xFF668F7F, 0xFF658E7E, 0xFF658E7E, 0xFF658E7E, 0xFF648D7D, 0xFF648D7D, 0xFF638C7C, 0xFF628B7B, 0xFF618A7A, 0xFF5F8878, 0xFF5F8878,
        0xFF5F8878, 0xFF5F8878, 0xFF608979, 0xFF618A78, 0xFF618A78, 0xFF618A78, 0xFF628B79, 0xFF628B79, 0xFF638C7A, 0xFF638C7A, 0xFF638C7A, 0xFF618B77, 0xFF608A76, 0xFF5F8975, 0xFF608A76, 0xFF628C78, 0xFF638D79, 0xFF638D79, 0xFF628C78, 0xFF6B9182,
        0xFF6B9182, 0xFF6C9283, 0xFF6C9283, 0xFF6B9182, 0xFF6A9081, 0xFF688E7F, 0xFF678D7E, 0xFF5F8F81, 0xFF5C8C7E, 0xFF5B897C, 0xFF608A7E, 0xFF668E83, 0xFF6C9086, 0xFF6D8E85, 0xFF6C8D84, 0xFF698F82, 0xFF698F82, 0xFF688E81, 0xFF678D80, 0xFF668C7F,
        0xFF648A7D, 0xFF63897C, 0xFF62887B, 0xFF5F8878, 0xFF5E8777, 0xFF5E8777, 0xFF5E8777, 0xFF5E8777, 0xFF5D8676, 0xFF5D8676, 0xFF5D8676, 0xFF5E8777, 0xFF5E8777, 0xFF5E8777, 0xFF5E8777, 0xFF5E8777, 0xFF5E8777, 0xFF5E8777, 0xFF5E8777, 0xFF5C8575,
        0xFF5E8777, 0xFF608979, 0xFF638C7C, 0xFF648D7D, 0xFF658E7E, 0xFF648D7D, 0xFF648D7D, 0xFF668C7F, 0xFF678D80, 0xFF698F82, 0xFF6A9083, 0xFF6A9083, 0xFF698F82, 0xFF688E81, 0xFF678D80, 0xFF5B8B7F, 0xFF5D8B80, 0xFF618C82, 0xFF658F85, 0xFF678F86,
        0xFF6B8E87, 0xFF6C8D86, 0xFF6C8D86, 0xFF6B8F81, 0xFF698D7F, 0xFF678B7D, 0xFF65897B, 0xFF65897B, 0xFF668A7C, 0xFF688C7E, 0xFF688C7E, 0xFF638C7C, 0xFF6D8F81, 0xFF678579, 0xFF6A8C7E, 0xFF5B8172, 0x005F7F72, 0x00EEFBF4, 0x00FFFBFA, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFCFD, 0x39D7E8E2, 0xFF456F61, 0xFF548975, 0xFF5B8C79, 0xFF5C8573, 0xFF5F8674, 0xFF5F8876, 0xFF5C8170, 0xFF5C8170, 0xFF5D8271, 0xFF5D8271, 0xFF5C8170,
        0xFF5C8170, 0xFF5B806F, 0xFF5B806F, 0xFF5A8071, 0xFF587E6F, 0xFF567C6D, 0xFF547A6B, 0xFF53796A, 0xFF53796A, 0xFF53796A, 0xFF53796A, 0xFF557B6C, 0xFF557B6C, 0xFF557B6C, 0xFF557B6C, 0xFF557B6C, 0xFF557B6C, 0xFF557B6C, 0xFF557B6C, 0xFF537F6E,
        0xFF527E6D, 0xFF517D6C, 0xFF517D6C, 0xFF517D6C, 0xFF527E6D, 0xFF537F6E, 0xFF54806F, 0xFF558170, 0xFF558170, 0xFF568271, 0xFF578372, 0xFF588473, 0xFF598574, 0xFF5A8675, 0xFF5A8675, 0xFF5A8675, 0xFF5A8675, 0xFF5B8776, 0xFF5B8776, 0xFF5B8776,
        0xFF5B8776, 0xFF5C8877, 0xFF5C8877, 0xFF5E8A79, 0xFF5E8A79, 0xFF5E8A79, 0xFF5E8A79, 0xFF5D8978, 0xFF5D8978, 0xFF5D8978, 0xFF5D8978, 0xFF5B8A7A, 0xFF5C8B7B, 0xFF5C8B7B, 0xFF5C8C7C, 0xFF5D8D7D, 0xFF5C8F7E, 0xFF5C8F7E, 0xFF5C8F7E, 0xFF5D8978,
        0xFF5E8A79, 0xFF5F8B7A, 0xFF608C7B, 0xFF5F8B7A, 0xFF5E8A79, 0xFF5C8877, 0xFF5B8776, 0xFF5C8877, 0xFF5C8877, 0xFF5C8877, 0xFF5C8877, 0xFF5C8877, 0xFF5C8877, 0xFF5C8877, 0xFF5C8877, 0xFF5E8A79, 0xFF5E8A79, 0xFF5E8A79, 0xFF5E8A79, 0xFF5E8A79,
        0xFF5D8978, 0xFF5D8978, 0xFF5D8978, 0xFF658779, 0xFF658779, 0xFF648678, 0xFF638577, 0xFF638577, 0xFF648678, 0xFF658779, 0xFF658779, 0xFF638C7C, 0xFF638C7C, 0xFF638C7C, 0xFF648D7D, 0xFF648D7D, 0xFF658E7E, 0xFF658E7E, 0xFF658E7E, 0xFF608C7B,
        0xFF608C7B, 0xFF608C7B, 0xFF608C7B, 0xFF608C7B, 0xFF608C7B, 0xFF5F8B7A, 0xFF5F8B7A, 0xFF638E7D, 0xFF638E7D, 0xFF648F7E, 0xFF648F7E, 0xFF648F7E, 0xFF638E7D, 0xFF628D7C, 0xFF618C7B, 0xFF5E8A79, 0xFF5E8A79, 0xFF5E8A79, 0xFF5D8978, 0xFF5D8978,
        0xFF5D8978, 0xFF5D8978, 0xFF5D8978, 0xFF5B8874, 0xFF5C8975, 0xFF5C8975, 0xFF5D8A76, 0xFF5E8B77, 0xFF5F8C78, 0xFF5F8C78, 0xFF608D79, 0xFF638575, 0xFF628474, 0xFF618373, 0xFF628474, 0xFF648676, 0xFF668878, 0xFF688A7A, 0xFF688A7A, 0xFF638F7E,
        0xFF638F7E, 0xFF628E7D, 0xFF628E7D, 0xFF618D7C, 0xFF618D7C, 0xFF608C7B, 0xFF608C7B, 0xFF608B7A, 0xFF5F8A79, 0xFF5F8878, 0xFF628879, 0xFF65897B, 0xFF698B7D, 0xFF6A8A7D, 0xFF6A887C, 0xFF5E8B77, 0xFF5F8C78, 0xFF628F7B, 0xFF63907C, 0xFF63907C,
        0xFF618E7A, 0xFF5E8B77, 0xFF5D8A76, 0xFF638779, 0xFF638779, 0xFF638779, 0xFF638779, 0xFF638779, 0xFF628678, 0xFF628678, 0xFF628678, 0xFF638779, 0xFF638779, 0xFF638779, 0xFF638779, 0xFF638779, 0xFF638779, 0xFF638779, 0xFF638779, 0xFF5E8A79,
        0xFF5E8A79, 0xFF5F8B7A, 0xFF608C7B, 0xFF628E7D, 0xFF64907F, 0xFF659180, 0xFF669281, 0xFF62917D, 0xFF63927E, 0xFF64937F, 0xFF64937F, 0xFF64937F, 0xFF63927E, 0xFF62917D, 0xFF61907C, 0xFF5A8D7C, 0xFF5D8D7D, 0xFF5F8E7E, 0xFF628E7F, 0xFF648E80,
        0xFF648D7F, 0xFF668C7F, 0xFF658B7E, 0xFF618C7B, 0xFF628D7C, 0xFF638E7D, 0xFF648F7E, 0xFF638E7D, 0xFF638E7D, 0xFF628D7C, 0xFF628D7C, 0xFF618C7B, 0xFF698D7F, 0xFF668679, 0xFF6C8C7F, 0xFF5A7C6E, 0x00698378, 0x00FBFFFD, 0x00FFFCFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFE, 0x45D1E4DE, 0xFF447061, 0xFF4D846F, 0xFF5E917E, 0xFF648D7B, 0xFF668878, 0xFF5D8271, 0xFF5D8877, 0xFF5D8877, 0xFF5D8877, 0xFF5C8776, 0xFF5B8675,
        0xFF5A8574, 0xFF588372, 0xFF578271, 0xFF568271, 0xFF558170, 0xFF54806F, 0xFF54806F, 0xFF537F6E, 0xFF537F6E, 0xFF537F6E, 0xFF54806F, 0xFF517D6C, 0xFF517D6C, 0xFF517D6C, 0xFF527E6D, 0xFF527E6D, 0xFF537F6E, 0xFF537F6E, 0xFF537F6E, 0xFF537E6D,
        0xFF537E6D, 0xFF537E6D, 0xFF537E6D, 0xFF547F6E, 0xFF55806F, 0xFF568170, 0xFF578271, 0xFF598473, 0xFF598473, 0xFF5A8574, 0xFF5B8675, 0xFF5C8776, 0xFF5D8877, 0xFF5E8978, 0xFF5E8978, 0xFF5F8A79, 0xFF5F8A79, 0xFF5F8A79, 0xFF608B7A, 0xFF608B7A,
        0xFF608B7A, 0xFF608B7A, 0xFF608B7A, 0xFF628D7C, 0xFF628D7C, 0xFF628D7C, 0xFF628D7C, 0xFF628D7C, 0xFF628D7C, 0xFF628D7C, 0xFF628D7C, 0xFF628C7E, 0xFF628C7E, 0xFF618D7E, 0xFF618D7E, 0xFF5F8E7E, 0xFF5F8E7E, 0xFF5E8D7D, 0xFF5D8D7D, 0xFF65907F,
        0xFF648F7E, 0xFF628D7C, 0xFF618C7B, 0xFF608B7A, 0xFF618C7B, 0xFF618C7B, 0xFF628D7C, 0xFF618C7B, 0xFF618C7B, 0xFF618C7B, 0xFF618C7B, 0xFF618C7B, 0xFF618C7B, 0xFF618C7B, 0xFF618C7B, 0xFF628D7C, 0xFF628D7C, 0xFF618C7B, 0xFF618C7B, 0xFF608B7A,
        0xFF608B7A, 0xFF608B7A, 0xFF608B7A, 0xFF64887A, 0xFF64887A, 0xFF638779, 0xFF628678, 0xFF628678, 0xFF638779, 0xFF64887A, 0xFF64887A, 0xFF638C7C, 0xFF638C7C, 0xFF638C7C, 0xFF648D7D, 0xFF648D7D, 0xFF648D7D, 0xFF658E7E, 0xFF658E7E, 0xFF618C7B,
        0xFF628D7C, 0xFF638E7D, 0xFF648F7E, 0xFF648F7E, 0xFF648F7E, 0xFF648F7E, 0xFF638E7D, 0xFF648D7D, 0xFF658E7E, 0xFF668F7F, 0xFF679080, 0xFF668F7F, 0xFF658E7E, 0xFF638C7C, 0xFF628B7B, 0xFF628D7C, 0xFF628D7C, 0xFF628D7C, 0xFF628D7C, 0xFF628D7C,
        0xFF638E7D, 0xFF638E7D, 0xFF648F7E, 0xFF608B7A, 0xFF608B7A, 0xFF608B7A, 0xFF608B7A, 0xFF618C7B, 0xFF618C7B, 0xFF628D7C, 0xFF628D7C, 0xFF668D7B, 0xFF658C7A, 0xFF658C7A, 0xFF658C7A, 0xFF658C7A, 0xFF668D7B, 0xFF668D7B, 0xFF678E7C, 0xFF61907E,
        0xFF61907E, 0xFF608F7D, 0xFF608F7D, 0xFF608F7D, 0xFF608F7D, 0xFF61907E, 0xFF61907E, 0xFF638E7B, 0xFF638E7B, 0xFF648D7B, 0xFF648D7B, 0xFF668D7B, 0xFF688D7C, 0xFF688D7C, 0xFF688D7C, 0xFF61917B, 0xFF61917B, 0xFF61917B, 0xFF60907A, 0xFF5F8F79,
        0xFF5E8E78, 0xFF5D8D77, 0xFF5D8D77, 0xFF628879, 0xFF628879, 0xFF618778, 0xFF618778, 0xFF618778, 0xFF618778, 0xFF608677, 0xFF608677, 0xFF618778, 0xFF618778, 0xFF618778, 0xFF618778, 0xFF618778, 0xFF618778, 0xFF618778, 0xFF618778, 0xFF608C7B,
        0xFF608C7B, 0xFF5F8B7A, 0xFF5F8B7A, 0xFF5F8B7A, 0xFF608C7B, 0xFF608C7B, 0xFF618D7C, 0xFF5F907A, 0xFF5F907A, 0xFF60917B, 0xFF60917B, 0xFF5F907A, 0xFF5E8F79, 0xFF5D8E78, 0xFF5D8E78, 0xFF62917D, 0xFF62917D, 0xFF63927E, 0xFF65927E, 0xFF65907D,
        0xFF648F7C, 0xFF648D7B, 0xFF648D7B, 0xFF62917F, 0xFF649381, 0xFF669583, 0xFF659482, 0xFF639280, 0xFF608F7D, 0xFF5F8E7C, 0xFF5E8D7B, 0xFF578A77, 0xFF648F7E, 0xFF678D7E, 0xFF6D9384, 0xFF547D6D, 0x0069877B, 0x00F6FFFB, 0x00FEF5F6, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x48CFE0DA, 0xFF496F62, 0xFF406F5D, 0xFF4E7767, 0xFF59796A, 0xFF5E7A6C, 0xFF597869, 0xFF507C6B, 0xFF507C6B, 0xFF507C6B, 0xFF4F7B6A, 0xFF4E7A69,
        0xFF4D7968, 0xFF4C7867, 0xFF4B7766, 0xFF497866, 0xFF497866, 0xFF497866, 0xFF497866, 0xFF497866, 0xFF487765, 0xFF477664, 0xFF467563, 0xFF497866, 0xFF497866, 0xFF487765, 0xFF477664, 0xFF467563, 0xFF457462, 0xFF457462, 0xFF447361, 0xFF537567,
        0xFF537567, 0xFF537567, 0xFF537567, 0xFF537567, 0xFF547668, 0xFF557769, 0xFF557769, 0xFF587A6C, 0xFF587A6C, 0xFF597B6D, 0xFF5A7C6E, 0xFF5B7D6F, 0xFF5B7D6F, 0xFF5C7E70, 0xFF5C7E70, 0xFF608274, 0xFF608274, 0xFF608274, 0xFF608274, 0xFF618375,
        0xFF618375, 0xFF618375, 0xFF618375, 0xFF618375, 0xFF628476, 0xFF628476, 0xFF638577, 0xFF638577, 0xFF648678, 0xFF648678, 0xFF648678, 0xFF64857A, 0xFF638479, 0xFF638479, 0xFF608478, 0xFF608478, 0xFF5E8477, 0xFF5E8477, 0xFF5E8477, 0xFF618375,
        0xFF618375, 0xFF618375, 0xFF628476, 0xFF638577, 0xFF648678, 0xFF658779, 0xFF66887A, 0xFF67897B, 0xFF67897B, 0xFF67897B, 0xFF67897B, 0xFF67897B, 0xFF67897B, 0xFF67897B, 0xFF67897B, 0xFF648678, 0xFF648678, 0xFF638577, 0xFF638577, 0xFF628476,
        0xFF628476, 0xFF628476, 0xFF628476, 0xFF63897A, 0xFF63897A, 0xFF628879, 0xFF628879, 0xFF628879, 0xFF628879, 0xFF63897A, 0xFF63897A, 0xFF66887A, 0xFF66887A, 0xFF66887A, 0xFF66887A, 0xFF66887A, 0xFF66887A, 0xFF67897B, 0xFF67897B, 0xFF678B7D,
        0xFF678B7D, 0xFF678B7D, 0xFF688C7E, 0xFF688C7E, 0xFF698D7F, 0xFF698D7F, 0xFF6A8E80, 0xFF67897B, 0xFF688A7C, 0xFF698B7D, 0xFF6A8C7E, 0xFF6A8C7E, 0xFF698B7D, 0xFF67897B, 0xFF66887A, 0xFF668A7C, 0xFF65897B, 0xFF638779, 0xFF618577, 0xFF618577,
        0xFF628678, 0xFF638779, 0xFF64887A, 0xFF65897B, 0xFF64887A, 0xFF64887A, 0xFF64887A, 0xFF638779, 0xFF638779, 0xFF638779, 0xFF628678, 0xFF628B79, 0xFF638C7A, 0xFF638C7A, 0xFF638C7A, 0xFF628B79, 0xFF628B79, 0xFF618A78, 0xFF618A78, 0xFF628B7B,
        0xFF618A7A, 0xFF618A7A, 0xFF618A7A, 0xFF628B7B, 0xFF638C7C, 0xFF658E7E, 0xFF668F7F, 0xFF628D7A, 0xFF638E7B, 0xFF638E7B, 0xFF638E7B, 0xFF628D7A, 0xFF638C7A, 0xFF638C7A, 0xFF648D7B, 0xFF68927E, 0xFF68927E, 0xFF67917D, 0xFF67917D, 0xFF66907C,
        0xFF66907C, 0xFF66907C, 0xFF66907C, 0xFF638E7D, 0xFF638E7D, 0xFF638E7D, 0xFF638E7D, 0xFF638E7D, 0xFF628D7C, 0xFF628D7C, 0xFF628D7C, 0xFF638E7D, 0xFF638E7D, 0xFF638E7D, 0xFF638E7D, 0xFF638E7D, 0xFF638E7D, 0xFF638E7D, 0xFF638E7D, 0xFF678D7E,
        0xFF688E7F, 0xFF688E7F, 0xFF6A9081, 0xFF6B9182, 0xFF6C9283, 0xFF6C9283, 0xFF6D9384, 0xFF6B9682, 0xFF6B9682, 0xFF6B9682, 0xFF6B9682, 0xFF6A9581, 0xFF6A9581, 0xFF699480, 0xFF699480, 0xFF6B9280, 0xFF6B9280, 0xFF6B9280, 0xFF6A917F, 0xFF69907E,
        0xFF698E7D, 0xFF698E7D, 0xFF688D7C, 0xFF689181, 0xFF699282, 0xFF6B9484, 0xFF6A9383, 0xFF679080, 0xFF658E7E, 0xFF648D7D, 0xFF658E7E, 0xFF5C917D, 0xFF62917F, 0xFF618C7B, 0xFF5E8A79, 0xFF477664, 0x00678D7E, 0x00F1FFFA, 0x00FFFFFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFAFB, 0x39D7E2DE, 0xFF7E988F, 0xFF718F83, 0xFF72877E, 0xFF77877D, 0xFF74887D, 0xFF728A7D, 0xFF6E8F84, 0xFF6E8F84, 0xFF6F9085, 0xFB709186, 0xFB709186,
        0xFB709186, 0xFB709186, 0xFF6F9085, 0xFF6E9284, 0xFF6E9284, 0xFF6E9284, 0xFF6E9284, 0xFF6C9082, 0xFF6A8E80, 0xFF688C7E, 0xFF678B7D, 0xFF65897B, 0xFF64887A, 0xFF628678, 0xFF608476, 0xFF5D8173, 0xFF5A7E70, 0xFF587C6E, 0xFF577B6D, 0xFF60756C,
        0xFF5F746B, 0xFF5F746B, 0xFF5E736A, 0xFF5D7269, 0xFF5D7269, 0xFF5C7168, 0xFF5C7168, 0xFF5F746B, 0xFF5F746B, 0xFF60756C, 0xFF60756C, 0xFF61766D, 0xFF61766D, 0xFF61766D, 0xFF62776E, 0xFF61766D, 0xFF61766D, 0xFF61766D, 0xFF62776E, 0xFF62776E,
        0xFF62776E, 0xFF62776E, 0xFF62776E, 0xFF62776E, 0xFF62776E, 0xFF63786F, 0xFF63786F, 0xFF647970, 0xFF657A71, 0xFF667B72, 0xFF667B72, 0xFF657872, 0xFF657872, 0xFF647972, 0xFF647972, 0xFF647B73, 0xFF647B73, 0xFF647B73, 0xFF627C73, 0xFF60756C,
        0xFF62776E, 0xFF647970, 0xFF667B72, 0xFF667B72, 0xFF657A71, 0xFF62776E, 0xFF61766D, 0xFF60756C, 0xFF60756C, 0xFF60756C, 0xFF60756C, 0xFF60756C, 0xFF60756C, 0xFF60756C, 0xFF60756C, 0xFF61766D, 0xFF61766D, 0xFF61766D, 0xFF61766D, 0xFF61766D,
        0xFF60756C, 0xFF60756C, 0xFF60756C, 0xFF5D7B6F, 0xFF5D7B6F, 0xFF5D7B6F, 0xFF5D7B6F, 0xFF5D7B6F, 0xFF5D7B6F, 0xFF5D7B6F, 0xFF5D7B6F, 0xFF60776D, 0xFF60776D, 0xFF60776D, 0xFF60776D, 0xFF60776D, 0xFF60776D, 0xFF60776D, 0xFF60776D, 0xFF60776D,
        0xFF5F766C, 0xFF5F766C, 0xFF5F766C, 0xFF5F766C, 0xFF60776D, 0xFF61786E, 0xFF62796F, 0xFF63786F, 0xFF647970, 0xFF647970, 0xFF657A71, 0xFF647970, 0xFF647970, 0xFF62776E, 0xFF62776E, 0xFF62796F, 0xFF60776D, 0xFF5C7369, 0xFF597066, 0xFF576E64,
        0xFF576E64, 0xFF586F65, 0xFF597066, 0xFF5B726A, 0xFF5B726A, 0xFF5A7169, 0xFF597068, 0xFF597068, 0xFF586F67, 0xFF576E66, 0xFF576E66, 0xFF4F6F62, 0xFF517164, 0xFF537366, 0xFF537366, 0xFF537366, 0xFF527265, 0xFF517164, 0xFF517164, 0xFF526E62,
        0xFF526E62, 0xFF526E62, 0xFF526E62, 0xFF536F63, 0xFF557165, 0xFF567266, 0xFF577367, 0xFF507264, 0xFF507264, 0xFF517365, 0xFF517365, 0xFF507264, 0xFF507264, 0xFF4F7163, 0xFF4F7163, 0xFF536F61, 0xFF557163, 0xFF577365, 0xFF597567, 0xFF597567,
        0xFF587466, 0xFF567264, 0xFF557163, 0xFF537567, 0xFF537567, 0xFF537567, 0xFF527466, 0xFF527466, 0xFF527466, 0xFF527466, 0xFF517365, 0xFF527466, 0xFF527466, 0xFF527466, 0xFF527466, 0xFF527466, 0xFF527466, 0xFF527466, 0xFF527466, 0xFF547064,
        0xFF557165, 0xFF567266, 0xFF587468, 0xFF587468, 0xFF597569, 0xFF587468, 0xFF587468, 0xFF597567, 0xFF597567, 0xFF597567, 0xFF597567, 0xFF587466, 0xFF587466, 0xFF587466, 0xFF587466, 0xFF5C7669, 0xFF5C7669, 0xFF5C7669, 0xFF5B7568, 0xFF597567,
        0xFF597567, 0xFF587466, 0xFF587466, 0xFF597569, 0xFF597569, 0xFF587468, 0xFF587468, 0xFF577367, 0xFF567266, 0xFF577367, 0xFF587468, 0xFF507B6A, 0xFF55796B, 0xFF56786A, 0xFF53796A, 0xFF4D7867, 0x0074988A, 0x00E5F9F0, 0x00FEFFFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x00FEFFFF, 0x0BEFFAF6, 0x00F9FFFD, 0x00FFFFFD, 0x00FFFFFB, 0x00F7FFF9, 0x0BEFFFF8, 0x00F6FFFD, 0x00F7FFFE, 0x00F7FFFE, 0x00F8FFFF, 0x00F8FFFF,
        0x00F8FFFF, 0x00F8FFFF, 0x00F8FFFF, 0x00F7FFFD, 0x00F7FFFD, 0x00F7FFFD, 0x00F7FFFD, 0x00F6FFFC, 0x00F5FFFB, 0x04F4FFFA, 0x04F4FFFA, 0x00F7FFFD, 0x00F7FFFD, 0x00F7FFFD, 0x00F5FFFB, 0x04F3FFF9, 0x08F1FEF7, 0x0BF0FDF6, 0x0BEFFCF5, 0x00F6FFFA,
        0x00F6FFFA, 0x00F6FFFA, 0x00F6FFFA, 0x00F5FEF9, 0x04F4FDF8, 0x04F4FDF8, 0x04F3FCF7, 0x00F5FEF9, 0x00F5FEF9, 0x00F5FEF9, 0x00F5FEF9, 0x00F5FEF9, 0x00F5FEF9, 0x00F5FEF9, 0x00F5FEF9, 0x08F1FAF5, 0x08F1FAF5, 0x08F1FAF5, 0x08F2FBF6, 0x08F2FBF6,
        0x08F2FBF6, 0x08F2FBF6, 0x08F2FBF6, 0x08F1FAF5, 0x08F2FBF6, 0x08F2FBF6, 0x08F2FBF6, 0x04F3FCF7, 0x04F3FCF7, 0x04F3FCF7, 0x04F3FCF7, 0x00F5FBF9, 0x00F5FBF9, 0x00F5FBF9, 0x04F3FCF9, 0x04F4FDFA, 0x04F3FEFA, 0x04F3FEFA, 0x04F4FFFB, 0x08F2FBF6,
        0x04F3FCF7, 0x00F5FEF9, 0x00F6FFFA, 0x04F4FDF8, 0x08F1FAF5, 0x0FEDF6F1, 0x13EBF4EF, 0x17EAF3EE, 0x17EAF3EE, 0x17EAF3EE, 0x17EAF3EE, 0x17EAF3EE, 0x17EAF3EE, 0x17EAF3EE, 0x17EAF3EE, 0x17E9F2ED, 0x17E9F2ED, 0x17E9F2ED, 0x17E9F2ED, 0x17E9F2ED,
        0x17E9F2ED, 0x17EAF3EE, 0x17EAF3EE, 0x1BE7F2EC, 0x1BE7F2EC, 0x1BE7F2EC, 0x1BE8F3ED, 0x1BE8F3ED, 0x1BE7F2EC, 0x1BE7F2EC, 0x1BE7F2EC, 0x1BE7F2EC, 0x1BE7F2EC, 0x1EE6F1EB, 0x1EE6F1EB, 0x1EE6F1EB, 0x1EE6F1EB, 0x1EE6F1EB, 0x1EE6F1EB, 0x1EE6F1EB,
        0x1EE6F1EB, 0x1EE6F1EB, 0x1EE6F1EB, 0x1EE6F1EB, 0x1BE7F2EC, 0x1BE7F2EC, 0x1BE7F2EC, 0x1BE8F1EC, 0x1BE8F1EC, 0x1BE8F1EC, 0x1BE8F1EC, 0x1BE8F1EC, 0x1BE8F1EC, 0x1BE7F0EB, 0x1BE7F0EB, 0x1EE6F1EB, 0x1EE5F0EA, 0x26E2EDE7, 0x2AE0EBE5, 0x2EDEE9E3,
        0x2EDDE8E2, 0x2EDEE9E3, 0x2EDEE9E3, 0x2EDEE9E5, 0x2EDEE9E5, 0x2EDEE9E5, 0x2EDDE8E4, 0x2EDDE8E4, 0x31DCE7E3, 0x31DCE7E3, 0x31DCE7E3, 0x39D8E5DE, 0x35DAE7E0, 0x31DBE8E1, 0x31DBE8E1, 0x35DAE7E0, 0x35D9E6DF, 0x35DAE7E0, 0x31DBE8E1, 0x2EDDE8E2,
        0x31DCE7E1, 0x31DCE7E1, 0x31DCE7E1, 0x2EDDE8E2, 0x2EDDE8E2, 0x2EDEE9E3, 0x2EDEE9E3, 0x31DBE7E5, 0x31DBE7E5, 0x31DBE7E5, 0x31DCE8E6, 0x35DAE9E6, 0x35DAE9E6, 0x39D8E8E5, 0x39D7E7E4, 0x35D9E4E0, 0x31DBE6E2, 0x31DCE7E3, 0x31DCE7E3, 0x35DAE5E1,
        0x3DD6E1DD, 0x45D2DDD9, 0x48CFDAD6, 0x50CCDBD4, 0x50CCDBD4, 0x50CCDBD4, 0x50CCDBD4, 0x50CCDBD4, 0x50CBDAD3, 0x50CBDAD3, 0x50CBDAD3, 0x50CBDAD3, 0x50CBDAD3, 0x50CBDAD3, 0x50CBDAD3, 0x50CBDAD3, 0x50CBDAD3, 0x50CBDAD3, 0x50CBDAD3, 0x54CAD9D2,
        0x54CAD9D2, 0x54CAD9D2, 0x54CAD9D2, 0x54C9D8D1, 0x58C8D7D0, 0x58C8D7D0, 0x58C7D6CF, 0x4CCDD9D5, 0x4CCDD9D5, 0x4CCDD9D5, 0x4CCDD9D5, 0x4CCDD9D5, 0x4CCEDAD6, 0x4CCEDAD6, 0x4CCEDAD6, 0x4CCDD9D5, 0x50CCD8D4, 0x50CCD8D4, 0x54CAD9D4, 0x54C9D8D3,
        0x58C8D9D3, 0x54C9DAD4, 0x54C9DAD4, 0x4CCDDAD3, 0x50CBD8D1, 0x58C8D5CE, 0x5BC5D2CB, 0x5FC3D0C9, 0x63C1CEC7, 0x67C0CDC6, 0x67BFCCC5, 0x6EBCCDC5, 0x63C2CFC8, 0x54C9D4CE, 0x63C1D5CC, 0x72BAD6CA, 0x00C5DCD2, 0x00F7FFFB, 0x00FFF9FA, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF
};

static GUI_CONST_STORAGE GUI_BITMAP _bmPlatine_250x200 = {
  250,                                  // XSize
  200,                                  // YSize
  1000,                                 // BytesPerLine
  32,                                   // BitsPerPixel
  (unsigned char *)_acPlatine_250x200,  // Pointer to picture data
  NULL,                                 // Pointer to palette
  GUI_DRAW_BMPM8888I
};
#else
static GUI_CONST_STORAGE unsigned long _acPlatine_250x200[] = {
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFEFEFE, 0xE4E8E8E8, 0xE8E9EBEB, 0xC2D6D8D8, 0xBAD2D4D4, 0xBAD2D4D4, 0xBAD2D4D4,
        0xC6D7D9D9, 0xC2D5D7D7, 0xBAD2D4D4, 0xC2D5DAD8, 0xBED4D9D8, 0xC2D6DBDA, 0xC2D5DAD9, 0xBAD1D5D6, 0xBED3D7D8, 0xCAD9DCE0, 0xCEDBDEE2, 0xBAD2D9DC, 0xC6D7DEE1, 0xCAD9E0E3, 0xC2D6DDE0, 0xBAD2D9DC, 0xBAD2D9DC, 0xC2D6DDE0, 0xCAD9E0E3, 0xBAD2DBDF,
        0xC2D6DFE3, 0xBED3DADD, 0xBED3DADD, 0xC6D7DCDD, 0xBED4D9DA, 0xC2D6DCDB, 0xCAD9DEDD, 0xB7CFD8DB, 0xC6D8E1E4, 0xBAD2DADA, 0xB7D0D8D8, 0xBAD2D8D7, 0xC2D5DBDA, 0xE4E7EDEC, 0xE4E7ECEA, 0xFFF8F8F8, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFF7F7F7, 0xB7CFCFCF, 0x00B8BABA, 0x00929494, 0x00828484, 0x00838585, 0x00888A8A,
        0x008E9090, 0x00888A8A, 0x00868888, 0x00919695, 0x00939897, 0x00949998, 0x00939798, 0x00929697, 0x0093969A, 0x00919498, 0x008C8F93, 0x0090979A, 0x00979EA1, 0x009AA1A4, 0x00969DA0, 0x00959C9F, 0x00989FA2, 0x00959C9F, 0x008F9699, 0x00949DA1,
        0x0098A1A5, 0x00959EA2, 0x009CA3A6, 0x009CA3A6, 0x00979C9D, 0x00979C9D, 0x008E9394, 0x008D9699, 0x00949DA0, 0x008B9497, 0x00899191, 0x00868E8E, 0x00949A99, 0x00AEB4B3, 0x8DBAC0BF, 0xFBF4F4F4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFBF4F4F4, 0x91BCBCBC, 0x00B9BEBD, 0x00A8ADAC, 0x00AAAFAE, 0x00A7ACAB, 0x00A1A6A5,
        0x009EA3A2, 0x009BA09F, 0x00A9AEAD, 0x00C6CBCA, 0x00D2D6D7, 0x00D5D9DA, 0x00D6DADB, 0x00DCDFE3, 0x00D8DBDF, 0x00C2C5CA, 0x00AFB2B7, 0x00C7CCCF, 0x00D4D9DC, 0x00DCE1E4, 0x00DBE0E3, 0x00E0E5E8, 0x00E4E9EC, 0x00D5DADD, 0x00BEC3C6, 0x00C8D0D7,
        0x00C9CFD4, 0x00CAD0D5, 0x00D8DEE3, 0x00D8DDE0, 0x00D1D6D9, 0x00D1D5D6, 0x00BCC0C1, 0x00AFB8BC, 0x00B3B9BE, 0x00B0B7BA, 0x00ABB2B5, 0x00A2A7A8, 0x00A7ACAD, 0x00B2B6B7, 0x00C0C5C4, 0xF4F0F0F0, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFF5F5F5, 0x85B6B6B6, 0x00ACB1B0, 0x00B4B9B8, 0x00C7CCCB, 0x00BCC1C0, 0x00AEB3B2,
        0x00A9AEAD, 0x00A7ACAB, 0x00BCC1C0, 0x00CFD4D5, 0x00EEF3F4, 0x00FAFFFF, 0x00FAFFFF, 0x00FAFFFF, 0x00F6FAFF, 0x00D4D8DD, 0x00BDC1C6, 0x00DEE1E6, 0x00F3F6FB, 0x00FBFEFF, 0x00F8FBFF, 0x00FBFEFF, 0x00FBFEFF, 0x00F8FBFF, 0x00DEE1E6, 0x00E6E9F1,
        0x00E0E3EB, 0x00E4E8ED, 0x00FBFEFF, 0x00FBFEFF, 0x00FDFEFF, 0x00FDFEFF, 0x00E7E8EC, 0x00BBBFC4, 0x00B6BABF, 0x00BABEC3, 0x00BABDC1, 0x00AEB1B5, 0x00B1B3B4, 0x00A5A7A8, 0x00B0B2B3, 0xF0EDEDED, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFBFBFB, 0xFFFFFFFF, 0xFFF8F8F8, 0x89B8B8B8, 0x00ACB1B0, 0x00B2B7B6, 0x00B9BEBD, 0x00A6ABAA, 0x00A1A6A5,
        0x00A6ABAA, 0x009DA2A1, 0x00A9AEAD, 0x00B4B9BA, 0x00E5EAEB, 0x00FAFFFF, 0x00F7FCFF, 0x00FAFFFF, 0x00E9EDF2, 0x00C7CBD0, 0x00B6BABF, 0x00C4C4CA, 0x00E9E9EF, 0x00FDFDFF, 0x00FAFAFF, 0x00F7F7FD, 0x00FCFCFF, 0x00EFEFF5, 0x00D4D4DA, 0x00B3B5BD,
        0x00B4B6BE, 0x00CDCFD7, 0x00F9F9FF, 0x00FDFDFF, 0x00FCFBFF, 0x00F7F6FA, 0x00CAC9CD, 0x00A7AAAF, 0x009A9DA2, 0x00A0A3A8, 0x00A7A8AC, 0x00A4A5A9, 0x00B0AFB1, 0x00A09FA1, 0x00B0AFB1, 0xECECECEC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFB, 0xFFFFFFFC, 0xFFFFFFFE, 0xFFFCFCFC, 0xFFFBFBFB, 0xFFFEFFFD, 0xFFFEFFFB, 0xFFFFFFFB, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFEFFFD, 0xFFFEFFFD, 0xFFFDFEFC,
        0xFFFEFFFD, 0xFFFEFFFD, 0xFFFEFFFD, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFEFFFD, 0xFFFFFFFE,
        0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFDFFFB, 0xFFFDFFFB, 0xFFFDFFFB, 0xFFFDFFFB, 0xFFFDFFFB,
        0xFFFDFFFB, 0xFFFDFFFB, 0xFFFDFFFB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFFFB, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFDFBFF,
        0xFFFFFDFF, 0xFFFDFCFE, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFDFCFE, 0xFFFFFDFF, 0xFFFDFBFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFCFF, 0xFFFDFBFF, 0xFFFCFAFF, 0xFFFFFEFF, 0xFFFFFEFF,
        0xFFFDFCFE, 0xFFFBFAFC, 0xFFFCFCFC, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFCFBFF, 0xFFFCFBFF, 0xFFFDFCFF, 0xFFFEFDFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFEFDFF,
        0xFFFEFDFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFBFF, 0xFFFEFBFF, 0xFFFFFDFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFF6F7F5, 0x85B7B8B6, 0x00B6BBBA, 0x00AFB4B3, 0x00B7BCBB, 0x00A6ABAA, 0x008F9493,
        0x00969B9A, 0x00A5AAA9, 0x00B0B5B4, 0x00B1B7B6, 0x00E9EFEE, 0x00F4FAF9, 0x00F3F9F8, 0x00F8FEFD, 0x00CDD3D2, 0x009BA1A0, 0x008F9594, 0x00A0A3A7, 0x00A5A8AC, 0x00EDF0F4, 0x00FBFEFF, 0x00F3F6FA, 0x00FBFEFF, 0x00CBCED2, 0x0095989C, 0x008E9392,
        0x008E9392, 0x009DA2A1, 0x00FDFFFF, 0x00F9FBFB, 0x00FDFDFD, 0x00F9F9F9, 0x00AFAFAF, 0x00A3A8A7, 0x009CA1A0, 0x00989D9C, 0x00969B9A, 0x009CA1A0, 0x00A1A6A5, 0x00A6ABAA, 0x00B5BAB9, 0xE8ECEAF0, 0xFFFFFDFF, 0xFFFFFDFF, 0xFFFFFDFF, 0xFFFCFAFF,
        0xFFFDFBFF, 0xFFFFFDFF, 0xFFFBF9FF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFDFCFE, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFEFEFE, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFDFBFF, 0xFFFFFDFF, 0xFFFFFDFF, 0xFFF8F7F9, 0xFFFFFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFC, 0xFFFEFFFD, 0xFFFEFEFE, 0xFFFEFDFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFAFAFA, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFEFDFF,
        0xFFFEFDFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFDFDFD, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFDFFFE, 0xFFFDFFFE, 0xFFFDFFFE, 0xFFFDFFFE, 0xFFFDFFFE,
        0xFFFDFFFE, 0xFFFDFFFE, 0xFFFDFFFE, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFDFEFA, 0xFFFCFDFB, 0xFFFDFEFC, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFEFF,
        0xFFFFFFFF, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFF, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFDFF, 0xFFFDFCFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFE,
        0xFFFFFFFE, 0xFFFEFFFD, 0xFFFFFFFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFEFFFB, 0xFFFEFFFB, 0xFFFDFEFA,
        0xFFFDFEFA, 0xFFFCFDF9, 0xFFFBFCF8, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFDFDFD, 0xFFFCFCFC, 0xFFFBFBFB, 0xFFFEFFFB,
        0xFFFEFFFB, 0xFFFEFFFB, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFBFAFE, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFE, 0xFFFFFFFC, 0xF7F6F8F2, 0x76B1B3AD, 0x00B1B6B5, 0x00A7ACAB, 0x00B3B8B7, 0x00ACB1B0, 0x00979C9B,
        0x00969B9A, 0x009EA3A2, 0x00A5AAA9, 0x00929897, 0x00A3A9A8, 0x009EA4A3, 0x009CA2A1, 0x00A0A6A5, 0x00959B9A, 0x00909695, 0x00979D9C, 0x00888C8D, 0x00939798, 0x00A3A7A8, 0x009FA3A4, 0x00A0A4A5, 0x00A9ADAE, 0x00A4A8A9, 0x00A1A5A6, 0x008C9190,
        0x00A1A6A5, 0x00A1A6A5, 0x00B9BBBB, 0x00B5B7B7, 0x00B6B6B6, 0x00AFAFAF, 0x00A7A7A7, 0x009DA2A1, 0x00A6ABAA, 0x00AEB3B2, 0x00A8ADAC, 0x00A4A9A8, 0x00A5AAA9, 0x00ABB0AF, 0x00BDC2C1, 0xF0EDEDED, 0xFFFDFDFD, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFDFEFC, 0xFFFDFEFC, 0xFFFEFFFD, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFDFEFC, 0xFFFCFCFC, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFE, 0xFFFDFEFA, 0xFFFFFFFB, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFCFBFF, 0xFFFCFBFF, 0xFFFFFDFF, 0xFFFEFCFF, 0xFFFFFDFF, 0xFFFFFDFF, 0xFFFFFDFF, 0xFFFFFDFF, 0xFFFFFDFF,
        0xFFFFFDFF, 0xFFFFFDFF, 0xFFFFFDFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFDFCFE,
        0xFFFDFCFE, 0xFFFDFCFE, 0xFFFDFCFE, 0xFFFDFCFE, 0xFFFDFCFE, 0xFFFDFCFE, 0xFFFDFCFE, 0xFFFEFDFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFDFCFE, 0xFFFCFDFF, 0xFFFCFDFF, 0xFFFCFDFF, 0xFFFDFEFF, 0xFFFDFEFF,
        0xFFFDFEFF, 0xFFFDFEFF, 0xFFFDFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFEFFFD, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFC,
        0xFFFFFFFE, 0xFFFCFDFB, 0xFFFEFFFD, 0xFFFEFFFD, 0xFFFCFDFB, 0xFFFFFFFE, 0xFFFFFFFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFEFFFB,
        0xFFFCFDF9, 0xFFFEFFFB, 0xFFFFFFFC, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFCFDFB, 0xFFFDFEFC, 0xFFFDFEFC, 0xFFFDFEFC, 0xFFFEFFFD, 0xFFFEFFFD, 0xFFFEFFFD, 0xFFFFFFFE, 0xFFFDFEFC,
        0xFFFDFEFC, 0xFFFDFEFC, 0xFFFDFEFC, 0xFFFDFEFC, 0xFFFDFEFC, 0xFFFDFEFC, 0xFFFDFEFC, 0xFFFEFFFB, 0xFFFEFFFB, 0xFFFEFFFB, 0xFFFEFFFB, 0xFFFEFFFB, 0xFFFEFFFB, 0xFFFEFFFB, 0xFFFEFFFB, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC,
        0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFEFFFB, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFC,
        0xFFFFFFFC, 0xFFFEFFFB, 0xFFFEFFFB, 0xFFFDFEFA, 0xFFFCFDF9, 0xFFFBFCF8, 0xFFFAFBF7, 0xFFFDFEFC, 0xFFFFFFFE, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFB, 0xFFFCFEF8, 0xF4F5F8EF, 0x6EAFB2A9, 0x00B5BAB9, 0x00B7BCBB, 0x00B8BDBC, 0x00A4A9A8, 0x00969B9A,
        0x009DA2A1, 0x00969B9A, 0x00858A89, 0x00949998, 0x008D9291, 0x008E9392, 0x00979C9B, 0x00919695, 0x00898E8D, 0x00919695, 0x008E9392, 0x00979C9B, 0x00A8ADAC, 0x00989D9C, 0x00949998, 0x00979C9B, 0x00878C8B, 0x008B908F, 0x009A9F9E, 0x00909594,
        0x00A4A9A8, 0x00A3A8A7, 0x008C8E8E, 0x009A9C9C, 0x009D9D9D, 0x00838383, 0x00969696, 0x009DA2A1, 0x00A7ACAB, 0x00ABB0AF, 0x009DA2A1, 0x009A9F9E, 0x00A1A6A5, 0x00A4A9A8, 0x00B3B8B7, 0xF0F0F1ED, 0xFFFEFFFB, 0xFFFDFEFA, 0xFFFDFEFA, 0xFFFFFFFC,
        0xFFFDFEFA, 0xFFFCFDF9, 0xFFFBFCF8, 0xFFFEFFFD, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFEFFFD, 0xFFFDFEFC, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFBFCFA, 0xFFFEFFFB, 0xFFFEFFFA, 0xFFFCFFF6, 0xF7F8FBF2, 0xF7F9FDF2, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFBFAFC, 0xFFFFFEFF, 0xFFFFFDFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF,
        0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFDFF, 0xFFFFFDFF, 0xFFFFFDFF, 0xFFFFFDFF, 0xFFFFFDFF, 0xFFFFFCFF, 0xFFFFFCFF, 0xFFFFFCFF, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFCFBFF, 0xFFFCFBFF, 0xFFFCFBFF, 0xFFFDFCFF, 0xFFFEFDFF,
        0xFFFEFDFF, 0xFFFEFDFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFFFBFF, 0xFFFDF9FE, 0xFFFDF9FE, 0xFFFFFCFF, 0xFFFFFDFF, 0xFFFFFCFE, 0xFFFEFBFD, 0xFFFFFCFE, 0xFFFFFFFE,
        0xFFFFFEFD, 0xFFFFFDFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFDFF, 0xFFFFFEFD, 0xFFFFFFFE, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFFFE, 0xFFFEFEFE, 0xFFFCFCFC, 0xFFFCFCFC, 0xFFFCFCFC,
        0xFFFCFCFC, 0xFFFCFBFD, 0xFFFCFBFD, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF,
        0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFEFEFE,
        0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFFFDFF, 0xFFFFFDFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF,
        0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFFFFFE, 0xFFFBFCFA, 0xFFF8F9F7, 0x76AFB0AE, 0x00A7ACAB, 0x00B9BEBD, 0x00B3B8B7, 0x009BA09F, 0x009A9F9E,
        0x00A3A8A7, 0x009CA1A0, 0x00909594, 0x009A9F9D, 0x00868B89, 0x00818684, 0x00898E8C, 0x007F8482, 0x007E8381, 0x008C918F, 0x00858A88, 0x00959A99, 0x00989D9C, 0x00818685, 0x00888D8C, 0x008F9493, 0x00797E7D, 0x00797E7D, 0x00838887, 0x00818684,
        0x008A8F8D, 0x00A0A5A3, 0x00858886, 0x008E918F, 0x008E8F8D, 0x00787977, 0x00838482, 0x00999E9C, 0x00A2A7A5, 0x00A6ABA9, 0x00969B99, 0x009BA09E, 0x00A9AEAC, 0x00A8ADAB, 0x00B2B7B5, 0xF0EDEDED, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFAF8FE, 0xFFFFFEFF, 0xFFFDFDFD, 0xFFFFFFFE, 0xFFFEFFFB, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFE, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFCFF, 0xFFFBFAFE, 0xFFFFFFFE, 0xFFFFFFF9, 0xFFFEFFF7, 0xFFFFFFF9, 0xFFFFFFFB, 0xFFFCFDF9, 0xFFFCFEF8, 0xFFFCFEF8, 0xFFFCFEF8, 0xFFFCFEF8, 0xFFFCFEF8,
        0xFFFCFEF8, 0xFFFCFEF8, 0xFFFCFEF8, 0xFFFFFFFB, 0xFFFFFFFB, 0xFFFFFFFA, 0xFFFFFFFA, 0xFFFFFFFA, 0xFFFFFFF9, 0xFFFFFFF9, 0xFFFFFFF9, 0xFFFFFFFB, 0xFFFFFFFB, 0xFFFFFFFB, 0xFFFFFFFB, 0xFFFFFFFB, 0xFFFFFFFB, 0xFFFFFFFB, 0xFFFFFFFB, 0xFFFFFFF9,
        0xFFFFFFF9, 0xFFFFFFF9, 0xFFFFFFF9, 0xFFFFFFF9, 0xFFFFFFF9, 0xFFFFFFF9, 0xFFFFFFF9, 0xFFFFFFF9, 0xFFFDFFF7, 0xFFFBFEF5, 0xFFFCFFF6, 0xFFFEFFF8, 0xFFFFFFF9, 0xFFFFFFF9, 0xFFFFFFF9, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC,
        0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFB, 0xFFFFFFFB, 0xFFFFFFFB, 0xFFFFFFFB, 0xFFFFFFFB, 0xFFFFFFFB, 0xFFFFFFFB, 0xFFFFFFFB, 0xFFFFFDFF, 0xFFFFFDFF, 0xFFFFFDFF, 0xFFFFFDFF, 0xFFFFFEFF, 0xFFFFFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFA,
        0xFFFFFDFC, 0xFFFFFDFF, 0xFFFFFDFF, 0xFFFFFDFF, 0xFFFFFDFF, 0xFFFFFDFC, 0xFFFFFEFA, 0xFFFFFEFE, 0xFFFFFEFE, 0xFFFFFEFE, 0xFFFFFEFE, 0xFFFFFEFE, 0xFFFFFEFE, 0xFFFFFEFE, 0xFFFFFEFE, 0xFFFEFFFD, 0xFFFEFFFD, 0xFFFEFFFD, 0xFFFFFFFF, 0xFFFFFEFF,
        0xFFFFFEFF, 0xFFFFFEFF, 0xFFFCFBFF, 0xFFFDFCFF, 0xFFFDFCFF, 0xFFFDFCFF, 0xFFFDFCFF, 0xFFFDFCFF, 0xFFFDFCFF, 0xFFFDFCFF, 0xFFFDFCFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFDFCFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFFFEFF,
        0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF,
        0xFFFEFDFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFDFF, 0xFFFFFDFF, 0xFFFFFDFF, 0xFFFFFCFF, 0xFFFFFBFF, 0xFFFFFBFF, 0xFFFFFCFF, 0xFFFFFCFF, 0xFFFFFDFF,
        0xFFFFFDFF, 0xFFFFFDFF, 0xFFFFFDFF, 0xFFFEFCFF, 0xFFFDFBFF, 0xFFFDFBFF, 0xFFFCFAFF, 0xFFFFFEFF, 0xFFFCFBFD, 0xFFFCFBFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFF9F8FC, 0x5FA2A1A5, 0x00A1A6A5, 0x00B4B9B8, 0x00AEB3B2, 0x00A6ABAA, 0x00ACB1B0,
        0x00A4A9A8, 0x00A4A9A8, 0x00B9BEBD, 0x008E9391, 0x007F8482, 0x00747977, 0x00747977, 0x006F7472, 0x00747977, 0x00818684, 0x007B807E, 0x008E9391, 0x00838886, 0x00696E6C, 0x006D7270, 0x00828785, 0x007A7F7D, 0x00767B79, 0x00888D8B, 0x00989D9B,
        0x00878C8A, 0x00929795, 0x00838684, 0x00898C8A, 0x00868785, 0x00818280, 0x007B7C7A, 0x00787D7B, 0x008A8F8D, 0x009CA19F, 0x00959A98, 0x009EA3A1, 0x00A9AEAC, 0x00A0A5A3, 0x76ADB2B0, 0xECEDECF0, 0xFFFDFCFF, 0xFFFFFEFF, 0xFFFBFAFE, 0xFFFBFAFE,
        0xFFFEFDFF, 0xFFFCFBFF, 0xFFFDFCFF, 0xFFFFFDFF, 0xFFFFFDFF, 0xFFFFFDFF, 0xFFFFFDFF, 0xFFFFFDFF, 0xFFFFFDFF, 0xFFFFFDFF, 0xFFFFFDFF, 0xFFFEFAFF, 0xFFFFFDFF, 0xFFFAF9FB, 0xFFFFFFFE, 0xFFF8F9F7, 0xFFFDFCFE, 0xFFFCFAFF, 0xFFFFFBFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFBFF, 0xFFFCFBFF, 0xFFFFFFF9, 0xFFDEE4D1, 0xFFC2CBB0, 0xFFC9D2B7, 0xFFC2CBB1, 0xFFC2CAB3, 0xFFC1C9B2, 0xFFC0C8B1, 0xFFBFC7B0, 0xFFBEC6AF, 0xFFBEC6AF,
        0xFFBEC6AF, 0xFFBFC7B0, 0xFFBFC7B0, 0xFFC3C6B0, 0xFFC3C6B0, 0xFFC3C6B0, 0xFFC3C6B0, 0xFFC3C6B0, 0xFFC3C6B0, 0xFFC4C7B1, 0xFFC4C7B1, 0xFFCCD2C1, 0xFFCCD2C1, 0xFFCDD3C2, 0xFFCDD3C2, 0xFFCDD3C2, 0xFFCCD2C1, 0xFFCBD1C0, 0xFFCAD0BF, 0xFFCFD6C1,
        0xFFCFD6C1, 0xFFCFD6C1, 0xFFCFD6C1, 0xFFCFD6C1, 0xFFCFD6C1, 0xFFCFD6C1, 0xFFCFD6C1, 0xFFD3DAC5, 0xFFD1D8C3, 0xFFCFD6C1, 0xFFD1D8C3, 0xFFD5DCC7, 0xFFD9E0CB, 0xFFDAE1CC, 0xFFDAE1CC, 0xFFDEE2CF, 0xFFDEE2CF, 0xFFDEE2CF, 0xFFDEE2CF, 0xFFDEE2CF,
        0xFFDEE2CF, 0xFFDEE2CF, 0xFFDEE2CF, 0xFFDCE2CF, 0xFFDCE2CF, 0xFFDCE2CF, 0xFFDCE2CF, 0xFFDCE2CF, 0xFFDCE2CF, 0xFFDCE2CF, 0xFFDCE2CF, 0xFFE3E0DB, 0xFFE2E0D8, 0xFFE6E5DB, 0xFFEEEEE2, 0xFFF0F0E2, 0xFFEEEFDF, 0xFFEEF0DD, 0xFFF2F4E1, 0xFFF2F4E0,
        0xFFF0F0E2, 0xFFF0EEE6, 0xFFEAE6E5, 0xFFEAE6E5, 0xFFF0EEE6, 0xFFF0F0E2, 0xFFF2F4E0, 0xFFF1F2E2, 0xFFF1F2E2, 0xFFF1F2E2, 0xFFF1F2E2, 0xFFF1F2E2, 0xFFF1F2E2, 0xFFF1F2E2, 0xFFF1F2E2, 0xFFF6FCEB, 0xFFF6FCEB, 0xFFF4F9EA, 0xFFF2F6EA, 0xFFF4F8ED,
        0xFFF6F9F0, 0xFFF6F8F2, 0xFFF3F5EF, 0xFFF7FAF1, 0xFFF7FAF1, 0xFFF7FAF1, 0xFFF7FAF1, 0xFFF7FAF1, 0xFFF7FAF1, 0xFFF7FAF1, 0xFFF7FAF1, 0xFFF6F9F0, 0xFFF6F9F0, 0xFFF6F9F0, 0xFFF7FAF1, 0xFFF9FCF3, 0xFFFBFEF5, 0xFFFDFFF7, 0xFFFFFFF9, 0xFFFEFFF8,
        0xFFFEFFF8, 0xFFFEFFF8, 0xFFFEFFF8, 0xFFFEFFF8, 0xFFFEFFF8, 0xFFFEFFF8, 0xFFFEFFF8, 0xFFFFFFFB, 0xFFFFFFFB, 0xFFFFFFFB, 0xFFFFFFFB, 0xFFFFFFFB, 0xFFFFFFFB, 0xFFFFFFFB, 0xFFFFFFFB, 0xFFFAFCF6, 0xFFFAFCF6, 0xFFFAFCF6, 0xFFFBFDF7, 0xFFFCFEF8,
        0xFFFEFFFA, 0xFFFFFFFB, 0xFFFFFFFB, 0xFFFFFFFB, 0xFFFFFFFB, 0xFFFFFFFB, 0xFFFFFFFB, 0xFFFFFFFB, 0xFFFFFFFB, 0xFFFFFFFB, 0xFFFFFFFB, 0xFFFFFFF9, 0xFFFFFFF9, 0xFFFFFFF9, 0xFFFFFFF9, 0xFFFFFFF9, 0xFFFFFFF9, 0xFFFFFFF9, 0xFFFFFFF9, 0xFFFFFFFC,
        0xFFFFFFFC, 0xFFFFFFFC, 0xFFFEFFFB, 0xFFFEFFFB, 0xFFFEFFFB, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFB, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFDFEFC, 0xFFFEFFFD, 0xFFFFFFFF, 0xFFF5F5F5, 0x6EAAA9AB, 0x009FA4A3, 0x00ABB0AF, 0x009DA2A1, 0x00989D9C, 0x00A3A8A7,
        0x009DA2A1, 0x00989D9C, 0x00A2A7A6, 0x00848883, 0x00818580, 0x00777B76, 0x00757974, 0x00747873, 0x00777B76, 0x007E827D, 0x00757974, 0x007A807B, 0x00787E79, 0x00656B66, 0x00606661, 0x00797F7A, 0x00787E79, 0x006F7570, 0x00888E89, 0x00A5ABA6,
        0x008C928D, 0x007C827D, 0x00757974, 0x00818580, 0x007D7E7A, 0x007E7F7B, 0x00797A76, 0x00747A75, 0x007E847F, 0x008F9590, 0x008F9590, 0x00A1A7A2, 0x00ABB1AC, 0x009BA19C, 0x6EA9AFAA, 0xFFF9FAF6, 0xFFFEFFFB, 0xFFFFFFFC, 0xFFFDFEFA, 0xFFFCFDF9,
        0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFEFFFB, 0xFFFDFEFA, 0xFFFDFEFA, 0xFFFDFEFA, 0xFFFDFEFA, 0xFFFDFEFA, 0xFFFFFFFE, 0xFFFCFEF8, 0xFFFCFFF5, 0xFFFFFFF7, 0xFFFCFFF6, 0xFFFFFFFF, 0xFFFCF9FF, 0xFFFFFAFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFAFF, 0xFFFDFCFE, 0xF0F7FCED, 0x2698A383, 0x004F5D33, 0x00506031, 0x00425223, 0x00404F23, 0x003B491F, 0x003A481E, 0x0038461C, 0x0036441A, 0x00354319,
        0x00354319, 0x0036441A, 0x0037451B, 0x0039421A, 0x0039421A, 0x003B441C, 0x003C451D, 0x003E471F, 0x003F4820, 0x00404921, 0x00414A22, 0x00465032, 0x00485234, 0x004C5638, 0x004F593B, 0x00515B3D, 0x00525C3E, 0x00525C3E, 0x00525C3E, 0x004E5938,
        0x004E5938, 0x004E5938, 0x004E5938, 0x004E5938, 0x004E5938, 0x004E5938, 0x004E5938, 0x004E5938, 0x004C5736, 0x004B5635, 0x004D5837, 0x00505B3A, 0x00525D3C, 0x00515C3B, 0x004F5A39, 0x00515B37, 0x00505A36, 0x00505A36, 0x004F5935, 0x004F5935,
        0x004E5834, 0x004E5834, 0x004E5834, 0x004E5939, 0x004E5939, 0x00505B3B, 0x00515C3C, 0x00535E3E, 0x00545F3F, 0x00556040, 0x00566141, 0x005A6140, 0x00515835, 0x004F5732, 0x00576038, 0x005F693E, 0x005D683A, 0x00596535, 0x00596533, 0x00576234,
        0x005D663E, 0x00656B4C, 0x005E6249, 0x005E6249, 0x00656B4C, 0x005D663E, 0x00576234, 0x005A6636, 0x005A6636, 0x005A6636, 0x005A6636, 0x005A6636, 0x005A6636, 0x005A6636, 0x005A6636, 0x005A693D, 0x00616F45, 0x00647249, 0x00616E48, 0x005E6948,
        0x00606B4B, 0x00646E50, 0x00677153, 0x005E6949, 0x005E6949, 0x005E6949, 0x005E6949, 0x005E6949, 0x005E6949, 0x005E6949, 0x005E6949, 0x005E6949, 0x005E6949, 0x005E6949, 0x00606B4B, 0x00626D4D, 0x00667151, 0x00697454, 0x006B7656, 0x00687353,
        0x00687353, 0x00687353, 0x00687353, 0x00687353, 0x00687353, 0x00687353, 0x00687353, 0x00687254, 0x00687254, 0x00687254, 0x00687254, 0x00687254, 0x00687254, 0x00687254, 0x00687254, 0x00677153, 0x00677153, 0x00687254, 0x00697355, 0x006B7557,
        0x006E785A, 0x00717B5D, 0x00737D5F, 0x00747E60, 0x00737D5F, 0x00737D5F, 0x00727C5E, 0x00727C5E, 0x00717B5D, 0x00707A5C, 0x00707A5C, 0x00727859, 0x0073795A, 0x00747A5B, 0x00747A5B, 0x0073795A, 0x00717758, 0x006F7556, 0x006E7455, 0x00677056,
        0x00677056, 0x00677056, 0x00697258, 0x006B745A, 0x006F785E, 0x00737C62, 0x00757E64, 0x00778066, 0x007A8369, 0x007D856E, 0x00747B66, 0x00787E6B, 0x00767C6B, 0x047C8271, 0x00787D6E, 0x00949998, 0x00959A99, 0x00828786, 0x00787D7C, 0x007D8281,
        0x00848988, 0x00828786, 0x006F7473, 0x00737772, 0x006F736E, 0x00626661, 0x005E625D, 0x005E625D, 0x00666A65, 0x00747873, 0x00727671, 0x00636A63, 0x006A716A, 0x00616861, 0x005C635C, 0x00666D66, 0x006B726B, 0x006F766F, 0x00798079, 0x00717772,
        0x006F7570, 0x00686E69, 0x00676B66, 0x00666A65, 0x005F605C, 0x00656662, 0x006F706C, 0x006F7570, 0x00696F6A, 0x00707671, 0x00737974, 0x008C928D, 0x009AA09B, 0x008D938E, 0x67A6ACA7, 0x3198A089, 0x13889079, 0x178B937C, 0x1E8F9780, 0x22919982,
        0x359BA38C, 0x3D9EA68F, 0x41A0A891, 0x39A0A98E, 0x3DA1AA8F, 0x41A4AD92, 0x45A5AE93, 0x45A6AF94, 0x45A6AF94, 0x45A5AE93, 0x41A4AD92, 0x4CAAB398, 0x48ABB696, 0x48AEBA96, 0x4CAFBB97, 0x76C0C9AE, 0xFFFFFFF5, 0xFFFFFFFF, 0xFFFDFAFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDF8FF, 0xFFFFFEFF, 0xF7FFFFF2, 0x13929F79, 0x00455722, 0x00566A2F, 0x0045591E, 0x003D5017, 0x003F521B, 0x003D5019, 0x00394C15, 0x00374A13, 0x00354811,
        0x00364912, 0x00384B14, 0x00394C15, 0x00404C1A, 0x00414D1B, 0x00434F1D, 0x0045511F, 0x00485422, 0x004B5725, 0x004D5927, 0x004E5A28, 0x0053603A, 0x0057643E, 0x005E6B45, 0x0064714B, 0x006A7751, 0x006E7B55, 0x006F7C56, 0x00707D57, 0x006F7D53,
        0x006F7D53, 0x006F7D53, 0x006F7D53, 0x006F7D53, 0x006F7D53, 0x006F7D53, 0x006F7D53, 0x006C7A50, 0x006A784E, 0x0069774D, 0x006A784E, 0x006B794F, 0x0069774D, 0x00647248, 0x00606E44, 0x005C693B, 0x005C693B, 0x005B683A, 0x005A6739, 0x00596638,
        0x00586537, 0x00586537, 0x00576436, 0x005C6A41, 0x005D6B42, 0x005F6D44, 0x00616F46, 0x00647249, 0x0067754C, 0x0069774E, 0x006A784F, 0x006D7C43, 0x0055642A, 0x00405013, 0x003F5011, 0x00455714, 0x00465914, 0x0044580F, 0x0043570E, 0x00425510,
        0x00556528, 0x006F7C48, 0x00727C51, 0x00727C51, 0x006F7C48, 0x00556528, 0x00425510, 0x00465A12, 0x00465A12, 0x00465A12, 0x00465A12, 0x00465A12, 0x00465A12, 0x00465A12, 0x00465A12, 0x00415716, 0x00556A2B, 0x00697E40, 0x006E8247, 0x006A7D46,
        0x00687A45, 0x006C7C4C, 0x006F7F4F, 0x00697B46, 0x00697B46, 0x00697B46, 0x00697B46, 0x00697B46, 0x00697B46, 0x00697B46, 0x00697B46, 0x006A7C47, 0x00697B46, 0x00677944, 0x00667843, 0x00657742, 0x00667843, 0x00677944, 0x00677944, 0x00657742,
        0x00657742, 0x00657742, 0x00657742, 0x00657742, 0x00657742, 0x00657742, 0x00657742, 0x00627340, 0x00627340, 0x00627340, 0x00627340, 0x00627340, 0x00627340, 0x00627340, 0x00627340, 0x00637441, 0x00627340, 0x0060713E, 0x005E6F3C, 0x005D6E3B,
        0x005E6F3C, 0x005F703D, 0x005F703D, 0x005D6E3B, 0x005D6E3B, 0x005B6C39, 0x005A6B38, 0x00586936, 0x00576835, 0x00556633, 0x00556633, 0x00576430, 0x00586531, 0x00576430, 0x0054612D, 0x004E5B27, 0x00475420, 0x003F4C18, 0x003B4814, 0x00314112,
        0x002F3F10, 0x002B3B0C, 0x00283809, 0x00263607, 0x00253506, 0x00263607, 0x00273708, 0x00253309, 0x00253309, 0x0029370E, 0x0026330D, 0x002D3915, 0x00222D0C, 0x00495434, 0x00959F81, 0x00A3A8A7, 0x00939897, 0x00858A89, 0x007E8382, 0x006C7170,
        0x00696E6D, 0x00747978, 0x00656A69, 0x00696D68, 0x00696D68, 0x00696D68, 0x006E726D, 0x00696D68, 0x00656964, 0x00696D68, 0x00616560, 0x00535A53, 0x00575E57, 0x005C635C, 0x005E655E, 0x004E554E, 0x00535A53, 0x00646B64, 0x004E554E, 0x005A605B,
        0x00595F5A, 0x005C625D, 0x00676B66, 0x00676B66, 0x00696A66, 0x00636460, 0x00646561, 0x00636964, 0x005E645F, 0x006A706B, 0x006D736E, 0x00838984, 0x00909691, 0x008F9590, 0x00B9BFBA, 0x0048562D, 0x00233108, 0x001F2D04, 0x0025330A, 0x00243209,
        0x0029370E, 0x002A380F, 0x002C3A11, 0x002D3D0D, 0x00324212, 0x00394919, 0x00415121, 0x00475727, 0x00495929, 0x004A5A2A, 0x004A5A2A, 0x00475727, 0x0051632E, 0x004E612A, 0x003D4F1A, 0x0068774B, 0xFFEAF3D8, 0xFFFFFFF8, 0xFFFCFCFC, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFB, 0xFFFFFCFF, 0xFFFAFFFF, 0x1B909C7E, 0x00535C29, 0x00595C2A, 0x00484F1C, 0x003E5013, 0x00374B1E, 0x0035491C, 0x00314419, 0x002E4017, 0x002C3E17,
        0x002B3B19, 0x002B3B19, 0x002B3A1A, 0x002A4218, 0x0030471A, 0x00374A19, 0x003C4B18, 0x00414E1A, 0x0047531F, 0x004A5622, 0x004A5624, 0x004C6529, 0x0056673C, 0x005F654C, 0x00686B55, 0x00717656, 0x006E764D, 0x0069724A, 0x006D7655, 0x006B7955,
        0x006B7955, 0x006B7955, 0x006B7A54, 0x006C7B55, 0x006C7C53, 0x006B7B52, 0x006B7B52, 0x0065794A, 0x0066774C, 0x0065734F, 0x00647151, 0x00657050, 0x00626E4A, 0x005F6B41, 0x005B683A, 0x0058652D, 0x0056632B, 0x00536028, 0x00536028, 0x00546129,
        0x0055622A, 0x00546129, 0x00536028, 0x00576629, 0x0058672E, 0x005B6735, 0x00606A3F, 0x00636E48, 0x0067714D, 0x00687352, 0x00697453, 0x00707D5D, 0x004A5933, 0x003B4C20, 0x003F521F, 0x00475B26, 0x00435625, 0x00536439, 0x0057673E, 0x00374C15,
        0x004C5F2E, 0x00708156, 0x0074825E, 0x006D7B57, 0x0071805A, 0x0048592D, 0x003B4D1E, 0x004A5C27, 0x00445621, 0x004A5C27, 0x00576934, 0x004D5F2A, 0x00465823, 0x0052642F, 0x00536530, 0x003B4F14, 0x005B6D38, 0x0077865A, 0x00727F59, 0x00707C58,
        0x0075815D, 0x006F7D53, 0x00748357, 0x0076825E, 0x00717D59, 0x00727F59, 0x006C7A51, 0x00738256, 0x006E7E4F, 0x00728252, 0x006F7F4F, 0x00728155, 0x00738256, 0x00728155, 0x006F7E52, 0x00707F53, 0x00748357, 0x00768559, 0x00748357, 0x00748258,
        0x00738157, 0x00717F55, 0x00707E54, 0x00707E54, 0x00717F55, 0x00728056, 0x00738157, 0x006C7B4F, 0x00708051, 0x00728253, 0x00718151, 0x0071824F, 0x00738550, 0x0071834E, 0x006D7F4A, 0x006F7F56, 0x00708057, 0x00728259, 0x0073835A, 0x00718158,
        0x006F7F56, 0x006E7E55, 0x006E7E55, 0x006E7C53, 0x006F7D54, 0x00707E55, 0x006F7D54, 0x006B7950, 0x0069774E, 0x0068764D, 0x0068764D, 0x0066744A, 0x0066744A, 0x00627046, 0x005A683F, 0x005F6D44, 0x005B6940, 0x0043502A, 0x003B4822, 0x0036411B,
        0x0039431F, 0x002F391B, 0x00394228, 0x00323C25, 0x00253016, 0x00323E22, 0x00324122, 0x0032401C, 0x00303E1C, 0x00323E20, 0x00374228, 0x0037402B, 0x00464E3D, 0x006F7667, 0x00B2B9AC, 0x00AEB5B8, 0x00949B9E, 0x007F8787, 0x00666E6E, 0x0058605F,
        0x005C6463, 0x005A6360, 0x005C6562, 0x005E655E, 0x00596059, 0x005C635C, 0x005F6560, 0x005D635E, 0x00606563, 0x00616664, 0x005B605E, 0x005E625C, 0x005E625C, 0x005F635D, 0x005B5F59, 0x00636761, 0x005A5E58, 0x005C605A, 0x0060645E, 0x0061665D,
        0x0062675E, 0x00676A61, 0x0063645A, 0x006A695F, 0x00706E63, 0x0069655A, 0x006F6C5E, 0x0062645E, 0x00666862, 0x006A6B67, 0x00717270, 0x007A797B, 0x00868589, 0x00A7A5AB, 0x00CECBD4, 0x0078846E, 0x0027341C, 0x001F2C12, 0x002C3B1C, 0x001E2E0C,
        0x0021330C, 0x002F4118, 0x00273A0F, 0x002E3D1D, 0x00334222, 0x00485737, 0x005F6E4E, 0x00697858, 0x00677656, 0x00657454, 0x00697858, 0x006A7554, 0x006A7751, 0x006D7B51, 0x00617044, 0x00616D49, 0xFFF3F9E6, 0xFFFFFFFE, 0xFFFFFCFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFBF9, 0xFFFFFAFF, 0xFFF9FBFF, 0x2A8C9685, 0x004C542F, 0x00525331, 0x003E4522, 0x002D4015, 0x001E3D1C, 0x001D3C1B, 0x001B3A1B, 0x001A381B, 0x0019361C,
        0x0019361C, 0x0019361D, 0x001A371E, 0x001E3116, 0x0027391C, 0x002E3D1D, 0x0034401C, 0x003A451D, 0x00404A20, 0x00444E23, 0x00495328, 0x004C642E, 0x0051653B, 0x0058654B, 0x00626D53, 0x00677553, 0x0063754E, 0x0062734E, 0x00667559, 0x00697753,
        0x00697753, 0x00697852, 0x00697852, 0x00697852, 0x00697950, 0x0068784F, 0x0068784F, 0x00647549, 0x0063734A, 0x0062704C, 0x00636E4D, 0x00616D49, 0x005F6B41, 0x00596737, 0x00556530, 0x00566226, 0x00535F23, 0x00505C20, 0x004F5B1F, 0x00505C20,
        0x00505C20, 0x004F5B1F, 0x004E5A1E, 0x00526124, 0x00536127, 0x0055622E, 0x00596535, 0x005D693F, 0x00626D45, 0x0065704A, 0x0065714D, 0x0064724E, 0x0046562D, 0x0037481C, 0x00506330, 0x00576A37, 0x003D501F, 0x004E5F33, 0x0048582F, 0x00384C17,
        0x004E6130, 0x006E7F54, 0x006E7C58, 0x00697753, 0x006F7E58, 0x0048592D, 0x003B4D1E, 0x00536334, 0x00586839, 0x005A6A3B, 0x00586839, 0x00506031, 0x00556536, 0x005C6C3D, 0x004A5A2B, 0x00384B14, 0x00586936, 0x00758458, 0x00738158, 0x00707D57,
        0x00748259, 0x006E7D51, 0x00728252, 0x00707E55, 0x006F7D54, 0x00758359, 0x00728056, 0x0077865A, 0x00718054, 0x00758556, 0x00758556, 0x00707F53, 0x00718054, 0x00707F53, 0x006D7C50, 0x006E7D51, 0x00718054, 0x00718054, 0x006E7D51, 0x006E7C52,
        0x006F7D53, 0x00707E54, 0x00717F55, 0x00728056, 0x00728056, 0x00717F55, 0x00707E54, 0x00748259, 0x00758359, 0x00738157, 0x00707E54, 0x00707E54, 0x00728056, 0x00728056, 0x00707E54, 0x006D7D54, 0x006E7E55, 0x006E7E55, 0x006D7D54, 0x006B7B52,
        0x006A7A51, 0x006A7A51, 0x00697950, 0x006E7C53, 0x006E7C53, 0x006D7B52, 0x006B7950, 0x0069774E, 0x0068764D, 0x0069774E, 0x006A784F, 0x00657349, 0x00657349, 0x00647248, 0x005D6B42, 0x005A683F, 0x004C5A31, 0x003C4923, 0x0045522C, 0x00767D62,
        0x008C927B, 0x00929683, 0x009A9E92, 0x00989C91, 0x008A9184, 0x00878E7F, 0x00747C6B, 0x008E9783, 0x0099A28E, 0x00878E7F, 0x006E7469, 0x00797D77, 0x00777B76, 0x007E8080, 0x00AAACAC, 0x00B5BAB9, 0x00A8ADAC, 0x00A7ACAA, 0x00A1A6A4, 0x009CA29D,
        0x009DA39E, 0x00969C97, 0x00949A95, 0x00949C92, 0x008D948D, 0x008D948D, 0x008D948D, 0x00888E89, 0x00868C87, 0x00858A88, 0x00808583, 0x00838988, 0x007E8483, 0x00828887, 0x00808685, 0x00858B8A, 0x00828887, 0x008A908F, 0x008D9392, 0x00808C8C,
        0x00808C8C, 0x00869090, 0x00838D8D, 0x00858D8D, 0x00808888, 0x00787D7E, 0x00808586, 0x00898B85, 0x008A8C86, 0x00838480, 0x00858682, 0x009A9B99, 0x00ADAEAC, 0x00B2B2B2, 0x00B1B1B1, 0x0098A096, 0x00838B80, 0x006F786B, 0x005B6555, 0x006D7766,
        0x0065705C, 0x00606B57, 0x006E7A64, 0x00747E6D, 0x0066705F, 0x007D8776, 0x00919B8A, 0x0086907F, 0x008A9483, 0x0096A08F, 0x008B9584, 0x00768160, 0x006C7953, 0x006A784E, 0x00657448, 0x006A7652, 0xFFF7FDEA, 0xFFFFFFFE, 0xFFFBF8FF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xF0FFFFEE, 0xFFFFFFFC, 0xF7F2FFFD, 0x1E80997F, 0x00344C24, 0x00314522, 0x0028401E, 0x001D3F16, 0x001B3916, 0x001B3916, 0x001A3815, 0x001A3716, 0x00193517,
        0x00193517, 0x00193518, 0x00193518, 0x00143013, 0x001A3618, 0x001C3715, 0x001D3811, 0x00253E12, 0x002C4513, 0x00344C16, 0x003E561C, 0x003D5525, 0x00435831, 0x00516443, 0x005B704F, 0x005C734D, 0x005B734B, 0x005A724F, 0x005B7154, 0x0064724E,
        0x0064734D, 0x0065744E, 0x0065744E, 0x0065754C, 0x0064744B, 0x0064754A, 0x0064754A, 0x00617148, 0x00617148, 0x00616E48, 0x005F6D44, 0x005B6A3E, 0x00576633, 0x00506128, 0x004C5E21, 0x004F5A1B, 0x004E591A, 0x004C5718, 0x004C5718, 0x004D5819,
        0x004F5A1B, 0x004F5A1B, 0x004F5A1B, 0x004B5A1C, 0x004C5B1E, 0x004F5D23, 0x00515F29, 0x00566230, 0x005A6739, 0x005F6B41, 0x00636F45, 0x0063724C, 0x0046572C, 0x002E4011, 0x00516433, 0x00586B3A, 0x00324415, 0x004B5C31, 0x0045552C, 0x002E4011,
        0x004B5C30, 0x006C7C53, 0x006D7C56, 0x006C7B55, 0x006D7D54, 0x00435429, 0x00304213, 0x005E6C42, 0x0056643A, 0x004C5A30, 0x0049572D, 0x00435127, 0x00455329, 0x004B592F, 0x003F4D23, 0x00364714, 0x00566637, 0x00738256, 0x00728056, 0x006F7D53,
        0x00738354, 0x0071824F, 0x0072844F, 0x00708051, 0x006F7F50, 0x00748455, 0x00738256, 0x00748357, 0x006D7C50, 0x00707F53, 0x00728155, 0x00718054, 0x00718054, 0x00707F53, 0x006E7D51, 0x006F7E52, 0x00718054, 0x00707F53, 0x006E7D51, 0x00717F55,
        0x00728056, 0x00738157, 0x00738157, 0x00728056, 0x00717F55, 0x00707E54, 0x00707E54, 0x00717F56, 0x00717F56, 0x006F7C56, 0x006E7B55, 0x006F7B57, 0x00737E5D, 0x00737E5D, 0x00707B5B, 0x00708057, 0x006F7F56, 0x006D7D54, 0x006C7C53, 0x006B7B52,
        0x006A7A51, 0x006A7A51, 0x006A7A51, 0x006D7B52, 0x006C7A51, 0x006A784F, 0x006A784F, 0x006A784F, 0x006A784F, 0x0069774E, 0x0069774E, 0x00657349, 0x00637147, 0x00637147, 0x00616F46, 0x005A683F, 0x00425027, 0x0033401A, 0x0048552F, 0x00555849,
        0x00636559, 0x0060605A, 0x00686967, 0x00757575, 0x00757575, 0x00686B69, 0x004B4F4A, 0x006C706B, 0x008E918F, 0x00B3B5B5, 0x00A5A6AA, 0x00AAAAB0, 0x00AAA9B3, 0x00B1AFBB, 0x00AFADB9, 0x0080827C, 0x007C7E78, 0x008A8C86, 0x0095988F, 0x00989B92,
        0x00999D92, 0x0095998E, 0x0093978C, 0x0082877E, 0x007D8279, 0x007E837A, 0x0081857F, 0x007F837D, 0x007D817C, 0x007E827D, 0x007E827D, 0x00808887, 0x00798180, 0x007F8786, 0x007F8786, 0x007F8786, 0x007E8685, 0x0088908F, 0x0088908F, 0x008B9491,
        0x00878F8E, 0x00878F8E, 0x00868E8D, 0x00858D8D, 0x007D8585, 0x00747C7C, 0x007A8184, 0x00A0A19D, 0x00AFB1AB, 0x00B3B5AF, 0x00ACAFA6, 0x00A9ACA3, 0x00A3A79C, 0x0091958A, 0x007F8378, 0x008D8F90, 0x00B0B2B3, 0x00BABCBD, 0x008C8E8F, 0x009A9B9F,
        0x0095969A, 0x009D9EA2, 0x00A6A7AB, 0x00575A58, 0x004C4F4D, 0x00686B69, 0x00828583, 0x00797C7A, 0x00848785, 0x00989B99, 0x008A8D8B, 0x0075805F, 0x006B7852, 0x0069774D, 0x0069784C, 0x006D7955, 0xFFF4FAE7, 0xFFFEFFFD, 0xFFFCF9FF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xF0F4FEEE, 0xCADAEAF0, 0xA0C4E4F1, 0x006C9892, 0x00305B4E, 0x00284D49, 0x00214B4A, 0x001B4E46, 0x00294D47, 0x002A4E48, 0x00294D47, 0x00294D47, 0x00284C46,
        0x00264A44, 0x00254844, 0x00244743, 0x001B4342, 0x001D4543, 0x0018413C, 0x00143F34, 0x00194331, 0x001B4429, 0x00163E1B, 0x00153C16, 0x003B4E23, 0x003B4F25, 0x00455A33, 0x004C643A, 0x00466135, 0x00486135, 0x0050663C, 0x0051663F, 0x005D6C46,
        0x005F6E48, 0x0061704A, 0x00617148, 0x00617148, 0x00617247, 0x00617247, 0x00617247, 0x00606F49, 0x00616F46, 0x005E6C42, 0x005B6B3C, 0x00576732, 0x00506227, 0x004B5C1C, 0x00475916, 0x004D5819, 0x004C5718, 0x004B5617, 0x004A5516, 0x004B5617,
        0x004C5718, 0x004D5819, 0x004E591A, 0x00495618, 0x00485719, 0x004A591B, 0x004D5C1F, 0x00505F25, 0x0056632F, 0x005B6939, 0x00606D41, 0x0063764B, 0x00405328, 0x00273B0E, 0x00425629, 0x00495D30, 0x002F4217, 0x004E6037, 0x00506239, 0x00314217,
        0x004E5E35, 0x006E7E55, 0x006E7E55, 0x00708057, 0x006F8055, 0x0047582D, 0x00304115, 0x004E5F33, 0x0046572B, 0x0046572B, 0x00546539, 0x0056673B, 0x00536438, 0x0058693D, 0x00546539, 0x00334418, 0x00546539, 0x006F8054, 0x00708155, 0x006C7E4F,
        0x00708352, 0x00728552, 0x00718550, 0x00728554, 0x00708352, 0x00708253, 0x006F8152, 0x006E7F53, 0x006B7C51, 0x006C7D52, 0x006F7F56, 0x00738256, 0x00728155, 0x00718054, 0x00707F53, 0x00728155, 0x00748357, 0x00748357, 0x00738256, 0x006E7F54,
        0x006E7F54, 0x006D7E53, 0x006C7D52, 0x006B7C51, 0x006C7D52, 0x006E7F54, 0x006F8055, 0x006D7F50, 0x006D7E52, 0x006E7F53, 0x006F7F56, 0x0072815B, 0x0073815D, 0x00717F5D, 0x006E7C5A, 0x00708057, 0x006F7F56, 0x006E7E55, 0x006E7E55, 0x006E7E55,
        0x006E7E55, 0x006D7D54, 0x006C7C53, 0x006D7B52, 0x006C7A51, 0x006C7A51, 0x006C7A51, 0x006D7B52, 0x006C7A51, 0x0069774E, 0x0066744B, 0x0067754B, 0x00616F45, 0x00606E44, 0x00637148, 0x00616F46, 0x0045532A, 0x002F3C16, 0x00414E28, 0x00444436,
        0x00535347, 0x0052524C, 0x00565453, 0x00575757, 0x00565755, 0x005B5C58, 0x00575B56, 0x00444444, 0x00585759, 0x00C9C8CA, 0x00F0EEF4, 0x00EBE8F1, 0x00F6F2FD, 0x00FBF7FF, 0x00A7A3AF, 0x00434137, 0x003C3A2F, 0x00403E33, 0x00454338, 0x00424133,
        0x003D3C2E, 0x003A3A2A, 0x00393929, 0x0052564B, 0x004E5247, 0x004F5348, 0x0052554C, 0x004F5249, 0x004B4D47, 0x004A4C46, 0x004D4F49, 0x004E504A, 0x00484A44, 0x004D4F49, 0x004F514B, 0x004A4C46, 0x00474943, 0x004C4E48, 0x004A4C46, 0x00505042,
        0x004B4B3F, 0x004A4A3E, 0x004C4C40, 0x00515046, 0x00525147, 0x00504F45, 0x00535149, 0x00504F4B, 0x0043433D, 0x0035362D, 0x0034352B, 0x00393C2D, 0x003B3E2E, 0x00363A27, 0x00333823, 0x003A393B, 0x006B6A6C, 0x00CDCCD0, 0x00D7D5DB, 0x00E4E1EA,
        0x00E3DFEA, 0x00D0CCD8, 0x006E6A76, 0x00454545, 0x004A4A4A, 0x004F4F4F, 0x00535353, 0x004D4D4D, 0x00474747, 0x00474747, 0x00414141, 0x00636E4D, 0x0066734D, 0x006B794F, 0x006A794D, 0x0066724E, 0xFFECF2DF, 0xFFFEFFFD, 0xFFFFFCFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x81B3C5E4, 0x00819CD4, 0x0076A1E4, 0x00649CD3, 0x00649BCE, 0x006F9EDB, 0x006EA0E2, 0x0066A0DB, 0x005F9CD4, 0x005F9CD4, 0x00609ED4, 0x00609ED4, 0x00609ED4,
        0x005E9CD2, 0x005D9BD1, 0x005C9AD0, 0x006D9AD7, 0x006A99D6, 0x006698D4, 0x006096CD, 0x006195C4, 0x005889AF, 0x003C6B87, 0x00214E63, 0x00314020, 0x002A3B19, 0x0029401A, 0x002A431D, 0x00274119, 0x00344B1F, 0x004A5D2C, 0x00576731, 0x00576640,
        0x005A6943, 0x005D6D44, 0x005F6F46, 0x005F7045, 0x005F7045, 0x00607145, 0x00617246, 0x00616D49, 0x005F6D44, 0x005C6C3D, 0x00596934, 0x00526429, 0x004D5E1F, 0x00465717, 0x00435512, 0x004A5719, 0x00495618, 0x00485517, 0x00465315, 0x00455214,
        0x00455214, 0x00465315, 0x00475416, 0x00475319, 0x00495519, 0x004B5919, 0x004B5B1B, 0x004E5D1F, 0x00536229, 0x005B6735, 0x00606B3D, 0x005F7247, 0x003A4C23, 0x002A3C15, 0x002D3E19, 0x0030401E, 0x00263614, 0x00354523, 0x00384826, 0x002C3A18,
        0x00475531, 0x006C7B55, 0x00708057, 0x00738459, 0x006D7E53, 0x0047582D, 0x0028390E, 0x00314314, 0x00344617, 0x00344617, 0x00394B1C, 0x00364819, 0x00344617, 0x00384A1B, 0x00324415, 0x002F3F16, 0x00506136, 0x006C7D51, 0x00718256, 0x006C7D51,
        0x006E7F53, 0x00718256, 0x006D7E52, 0x00728358, 0x00708156, 0x006E7F54, 0x006F8055, 0x006F8055, 0x00718158, 0x006F7F56, 0x006E7E55, 0x00728155, 0x00707F53, 0x006E7D51, 0x006D7C50, 0x006D7C50, 0x006D7C50, 0x006F7E52, 0x00718054, 0x006C7D52,
        0x006E7F54, 0x00708156, 0x00708156, 0x00708156, 0x006F8055, 0x00718257, 0x00728358, 0x006D804D, 0x006B7E4B, 0x006B7E4D, 0x006D7F50, 0x006F8152, 0x006F8054, 0x006F8054, 0x006E7F54, 0x006C7C53, 0x006D7D54, 0x006E7E55, 0x00708057, 0x00708057,
        0x00708057, 0x006F7F56, 0x006F7F56, 0x00707E55, 0x006E7C53, 0x006D7B52, 0x006D7B52, 0x006D7B52, 0x006C7A51, 0x006A784F, 0x0068764D, 0x0069774D, 0x00657349, 0x00616F45, 0x00627047, 0x00637148, 0x0049572E, 0x002F3C16, 0x003D4A24, 0x0043472E,
        0x00585A46, 0x005E5E50, 0x005B5C52, 0x005B5C53, 0x00575B50, 0x005B5F53, 0x005E6354, 0x00656761, 0x00646660, 0x00D6D8D2, 0x00FFFFFC, 0x00EFF0EE, 0x00FBFCFA, 0x00FFFFFF, 0x00A6A6A6, 0x004C4B3D, 0x00454436, 0x00444335, 0x00474737, 0x00434333,
        0x0040402E, 0x00434331, 0x0041412F, 0x005A5E52, 0x005B5F53, 0x005D6155, 0x005F6358, 0x005F6358, 0x005A5D54, 0x005A5D54, 0x005E6158, 0x00646355, 0x00626153, 0x00605F51, 0x00636254, 0x00626153, 0x00605F51, 0x005F5E50, 0x00616052, 0x00515752,
        0x00545B54, 0x00595D57, 0x005E605A, 0x005F6057, 0x00606158, 0x0067665C, 0x0068675D, 0x0063635D, 0x0047483F, 0x00323329, 0x00323428, 0x003A3D2D, 0x003B3F2C, 0x00393E29, 0x00383E27, 0x004A4D44, 0x007A7C76, 0x00F8FAF4, 0x00FFFFFC, 0x00FAFBF9,
        0x00FFFFFF, 0x00FFFEFF, 0x00888789, 0x0052554C, 0x0062655C, 0x00595C53, 0x0053564D, 0x0055584F, 0x004F5249, 0x004E5148, 0x004F5249, 0x00576241, 0x0067744E, 0x006F7D53, 0x006A794D, 0x00616D49, 0xFFE7EDDA, 0xFFFFFFFE, 0xFFFFFCFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x76AED0FB, 0x006D97DC, 0x005995E9, 0x005AA1EB, 0x0062A8EE, 0x0065A1F3, 0x0063A0F6, 0x00599DEA, 0x005897E7, 0x005998E8, 0x005A9AE8, 0x005C9CEA, 0x005D9EE9,
        0x005D9EE9, 0x005D9EE8, 0x005D9EE8, 0x005E97E2, 0x005995E1, 0x005999E7, 0x005B9DEB, 0x005FA1E8, 0x0063A0DE, 0x004780B3, 0x001B4F7D, 0x001B3027, 0x00142C20, 0x00112E1E, 0x0010301F, 0x0015311E, 0x00243A1D, 0x003C4E1F, 0x004D5E1F, 0x0055653C,
        0x00596940, 0x005D6D44, 0x005F7045, 0x005E6F44, 0x005E6F43, 0x005F7044, 0x00607145, 0x00606D47, 0x00606C42, 0x005C6A3A, 0x00566630, 0x00505F25, 0x0049591C, 0x00455417, 0x00435117, 0x003F4C18, 0x003F4C18, 0x003F4C18, 0x003E4B17, 0x003E4B17,
        0x003F4C18, 0x00414E1A, 0x00424F1B, 0x00484F1C, 0x0048541A, 0x004A5719, 0x004C5A18, 0x004C5C1C, 0x00526026, 0x00586432, 0x005D673C, 0x005C6E45, 0x003A4B26, 0x00364525, 0x00263418, 0x0026331B, 0x002A371D, 0x002B391D, 0x002A381C, 0x003A4628,
        0x00475434, 0x00697852, 0x006E7F54, 0x006E7F53, 0x006A7B4F, 0x00526338, 0x00324219, 0x0032441B, 0x002F4118, 0x002C3E15, 0x0034461D, 0x0031431A, 0x00283A11, 0x002F4118, 0x0035471E, 0x003A4C23, 0x00596B42, 0x00708358, 0x00718558, 0x00677A4F,
        0x00657651, 0x006A7959, 0x00627054, 0x00677656, 0x006C7B5B, 0x006A7A58, 0x006A7B56, 0x00697B54, 0x0071835A, 0x006D7F56, 0x00687B50, 0x006D7C50, 0x006B7A4E, 0x0069784C, 0x00657448, 0x005E6D41, 0x0059683C, 0x005A693D, 0x005D6C40, 0x005A6D42,
        0x005F7247, 0x0065784D, 0x0066794E, 0x0062754A, 0x005B6E43, 0x0055683D, 0x0052653A, 0x00516439, 0x004F6237, 0x004D6134, 0x004F6336, 0x00536738, 0x00576B3C, 0x005F7344, 0x00657A49, 0x0063734A, 0x0066764D, 0x006A7A51, 0x006B7B52, 0x006B7B52,
        0x006A7A51, 0x006A7A51, 0x006B7B52, 0x00728057, 0x00707E55, 0x006D7B52, 0x006B7950, 0x006B7950, 0x006C7A51, 0x006D7B52, 0x006D7B52, 0x006A784E, 0x006B794F, 0x00657349, 0x00606E45, 0x00606E45, 0x0047552C, 0x00313E18, 0x0043502A, 0x004B4F32,
        0x005C5F45, 0x00565844, 0x004B4C3C, 0x00515445, 0x005B5E4F, 0x00595C4C, 0x00515744, 0x005C5F50, 0x005B5E4F, 0x00929488, 0x009C9E92, 0x0094968A, 0x00999A90, 0x00A5A69C, 0x00828379, 0x00656456, 0x00636254, 0x00616151, 0x00636353, 0x005F5F4F,
        0x005A5A48, 0x00595947, 0x004F4F3D, 0x00404334, 0x00494C3D, 0x004F5145, 0x00535549, 0x0057594D, 0x0058594F, 0x00595A50, 0x005E5F55, 0x005A594F, 0x005F5E54, 0x00545349, 0x0057564C, 0x00616056, 0x00616056, 0x0059584E, 0x00646359, 0x005C7687,
        0x00647C8E, 0x006C8190, 0x006C7E89, 0x005C6971, 0x004D585C, 0x00515959, 0x00535859, 0x00545349, 0x0056554B, 0x005D5D51, 0x00616155, 0x005F5F51, 0x005F5F51, 0x00606151, 0x005D5E4E, 0x00626556, 0x0076796A, 0x00AEB1A2, 0x00ADB0A1, 0x00AFB2A3,
        0x00B5B8A9, 0x00B1B4A5, 0x0056584C, 0x00515444, 0x00636656, 0x00606353, 0x005C5F4F, 0x00595C4C, 0x00555848, 0x00595C4C, 0x004E5141, 0x005B6645, 0x006C7953, 0x00717F55, 0x006B7A4E, 0x00636F4B, 0xFFE9EFDC, 0xFFFFFFFE, 0xFFFEFBFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x63A4D2EA, 0x006AA0D5, 0x00569CE2, 0x0052A5E3, 0x0055A5E0, 0x005BA0E5, 0x0069A9F0, 0x0060A5E4, 0x006198E1, 0x00629AE1, 0x00639BE2, 0x00659EE3, 0x0066A0E2,
        0x0067A1E2, 0x0068A3E1, 0x0068A3E1, 0x0065A7DC, 0x00599DD8, 0x00579FDF, 0x00529EE0, 0x00519CDA, 0x0060A3DA, 0x004B88B4, 0x00154C71, 0x00274B63, 0x00264E61, 0x00224E5F, 0x00244F60, 0x002B4E5C, 0x00324F4C, 0x003F5735, 0x004C642A, 0x0056663D,
        0x005A6A41, 0x005F7045, 0x00607146, 0x005E6F43, 0x005D6E42, 0x005D6E42, 0x005F7044, 0x00606B43, 0x005E6B3F, 0x00596835, 0x0054632A, 0x004C5B22, 0x0047541E, 0x00444F1D, 0x00434D1E, 0x0038461C, 0x0039471D, 0x0039471D, 0x0038461C, 0x0037451B,
        0x0037451B, 0x0039471D, 0x003C4A20, 0x00464C1D, 0x00484F1C, 0x00495519, 0x004A5816, 0x004A5B18, 0x004E5C22, 0x00555F30, 0x00596339, 0x00546641, 0x003D4E2D, 0x0046543C, 0x003C4836, 0x00434E3E, 0x00515C4C, 0x00515D49, 0x004C5943, 0x005F6952,
        0x00566244, 0x006A7854, 0x00697A4E, 0x0067794A, 0x006C7E4F, 0x006B7C51, 0x0053633A, 0x004A5541, 0x00495440, 0x004F5A46, 0x0065705C, 0x0065705C, 0x00505B47, 0x004C5743, 0x00515C48, 0x00576940, 0x006A7C53, 0x006D8055, 0x005D7045, 0x0043542F,
        0x003A482C, 0x00414C38, 0x00384134, 0x0036402F, 0x0045503C, 0x0048553D, 0x004B593D, 0x004D5D3B, 0x0063754E, 0x006A7D52, 0x006B7F52, 0x006A794D, 0x0068774B, 0x00657448, 0x005C6B3F, 0x004E5D31, 0x00404F23, 0x003C4B1F, 0x003E4D21, 0x004C5F34,
        0x0053663B, 0x005B6E43, 0x005B6E43, 0x0054673C, 0x00495C31, 0x003F5227, 0x00394C21, 0x002C3822, 0x002A371F, 0x0029361C, 0x002C3B1C, 0x002F3F1D, 0x00354720, 0x00405328, 0x004B5E33, 0x0065754C, 0x00697950, 0x006D7D54, 0x006D7D54, 0x006B7B52,
        0x006A7A51, 0x006C7C53, 0x006F7F56, 0x006B7950, 0x006C7A51, 0x006D7B52, 0x006D7B52, 0x006D7B52, 0x006D7B52, 0x006D7B52, 0x006E7C53, 0x006A784E, 0x006B794F, 0x0066744A, 0x00627047, 0x00616F46, 0x0045532A, 0x002F3C16, 0x0045522C, 0x0044472E,
        0x00595B45, 0x00585848, 0x0046463A, 0x0045443A, 0x004A4B41, 0x00505246, 0x00595C4D, 0x00585A4E, 0x00515347, 0x005B5D51, 0x005F6155, 0x0066685C, 0x006A6C60, 0x00626458, 0x00585A4E, 0x005E5E52, 0x00606054, 0x005C5C4E, 0x005D5D4F, 0x00575848,
        0x00525343, 0x004F5040, 0x003E402D, 0x00414232, 0x004F4F41, 0x00555547, 0x00545446, 0x0057574B, 0x0058584C, 0x0057564C, 0x0058574D, 0x00546062, 0x00647072, 0x00525E60, 0x00556163, 0x006A7678, 0x00667274, 0x00505C5E, 0x00626E70, 0x006D93B3,
        0x006F91AF, 0x007492AB, 0x007D95A7, 0x006C7C88, 0x00515C60, 0x00525758, 0x00555856, 0x00555547, 0x00535345, 0x00525246, 0x00505044, 0x00504F45, 0x0059584E, 0x005E5C54, 0x005A5850, 0x004E4E42, 0x005F5F53, 0x006C6C5E, 0x00616252, 0x0060624F,
        0x005B5D49, 0x005D604A, 0x003E412B, 0x00454636, 0x005D5E4E, 0x00606151, 0x00626353, 0x005E5F4F, 0x005B5C4C, 0x00606151, 0x004D4E3E, 0x00616C4B, 0x006D7A54, 0x006F7D53, 0x006D7C50, 0x0066724E, 0xFFE8EEDB, 0xFFFFFFFE, 0xFFFEFBFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x5FA1CAEA, 0x006EA2DE, 0x005A9EEB, 0x0053A1E8, 0x0057A0E4, 0x00629EEA, 0x006EA6F5, 0x005C97DC, 0x005699E8, 0x00569AE7, 0x00579BE6, 0x00579CE5, 0x00589EE4,
        0x00589FE3, 0x0059A0E3, 0x0059A0E3, 0x0065A0D8, 0x005998D5, 0x00589DE2, 0x00539AE4, 0x005094DD, 0x00629FE1, 0x005187BE, 0x00144474, 0x00104171, 0x00134873, 0x00114972, 0x00154977, 0x00224E77, 0x002D4F5F, 0x00375641, 0x00466532, 0x0058683F,
        0x005C6C43, 0x00617247, 0x00617247, 0x005E6F43, 0x005C6D41, 0x005C6D41, 0x005D6F40, 0x005E6941, 0x005C693B, 0x00576732, 0x00516027, 0x0049581F, 0x0045511D, 0x00434D1F, 0x00434B23, 0x003E4F2A, 0x003D4E29, 0x003B4C27, 0x00364722, 0x0031421D,
        0x002E3F1A, 0x002E3F1A, 0x002F401B, 0x0044481E, 0x00474D1C, 0x004A531A, 0x00495715, 0x004A5818, 0x004B591F, 0x00525C2D, 0x00575F37, 0x004C5D3B, 0x00425135, 0x00535F4B, 0x00545E51, 0x0060695F, 0x00697367, 0x00677164, 0x00596553, 0x005E6752,
        0x00505C40, 0x00687652, 0x006D7E52, 0x00697B4C, 0x006A7C4D, 0x006C7D52, 0x00506037, 0x00535856, 0x00606563, 0x00676C6A, 0x00707573, 0x00727775, 0x00696E6C, 0x005F6462, 0x00515654, 0x005F7148, 0x006F8257, 0x006D8055, 0x005F7148, 0x004D5C3C,
        0x00535F49, 0x006C746A, 0x006C716F, 0x00626A60, 0x0071796E, 0x006A7463, 0x005C6951, 0x004F5E3F, 0x0062744D, 0x00697C51, 0x006C8051, 0x006A794D, 0x0069784C, 0x00657448, 0x0059683C, 0x00445327, 0x00303F13, 0x0027360A, 0x0028370B, 0x00273A0F,
        0x002E4116, 0x0036491E, 0x00384B20, 0x0032451A, 0x00293C11, 0x00213409, 0x001D3005, 0x001B201E, 0x00192019, 0x00192116, 0x001B2514, 0x001B2810, 0x001C2A0E, 0x00243412, 0x002E3F1A, 0x00405027, 0x0044542B, 0x0047572E, 0x0046562D, 0x00425229,
        0x00415128, 0x0045552C, 0x00495930, 0x00616F46, 0x0067754C, 0x006D7B52, 0x00717F56, 0x00717F56, 0x006E7C53, 0x006C7A51, 0x006A784F, 0x0069774D, 0x0067754B, 0x00637147, 0x0065734A, 0x0067754C, 0x0047552C, 0x002B3812, 0x00424F29, 0x00474735,
        0x00555446, 0x005B594F, 0x00595651, 0x005E5B57, 0x00575652, 0x0051514B, 0x005C5D54, 0x0054554C, 0x00484940, 0x004F5047, 0x0053544B, 0x0047483F, 0x004F5047, 0x00525349, 0x0053544A, 0x005D5F53, 0x00606256, 0x00595B4F, 0x00575A4B, 0x00535647,
        0x00535647, 0x00525545, 0x003E4131, 0x00474838, 0x00585949, 0x005D5D4F, 0x00575749, 0x00575749, 0x0057574B, 0x00535347, 0x00525246, 0x004D6271, 0x00667B8A, 0x00516675, 0x00546978, 0x006F8493, 0x00647988, 0x003F5463, 0x00536877, 0x00557795,
        0x004A6883, 0x004F687C, 0x006B7D88, 0x00697275, 0x0053554F, 0x00585449, 0x005E5748, 0x00525343, 0x004F5040, 0x00545446, 0x0058574D, 0x0057554D, 0x00595651, 0x005B5854, 0x00575352, 0x0057544F, 0x00504E46, 0x0049483E, 0x004A4A3C, 0x004B4D3A,
        0x00464933, 0x00464A31, 0x0032371C, 0x00404034, 0x005E5E52, 0x005A5A4E, 0x0058584C, 0x005A5A4E, 0x0057574B, 0x005A5A4E, 0x004E4E42, 0x00606B4A, 0x006A7751, 0x006C7A50, 0x006E7D51, 0x0067734F, 0xFFE4EAD7, 0xFFFEFFFD, 0xFFFFFCFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x6EA9CBE9, 0x00659ED6, 0x0059A1EE, 0x005DA2EB, 0x005EA3E8, 0x005AA5F1, 0x005AA5F1, 0x00579DE0, 0x005C9CE3, 0x005D9DE4, 0x0060A0E7, 0x0060A0E7, 0x005F9FE6,
        0x005E9EE5, 0x005E9EE5, 0x005E9EE5, 0x0061A0E4, 0x00599ADE, 0x00589BE0, 0x005A9DE2, 0x00599ADE, 0x0064A3E0, 0x004D8AC2, 0x00094279, 0x001C4064, 0x00224567, 0x00193E60, 0x00113E64, 0x0017496D, 0x00204B5A, 0x00395440, 0x005D6939, 0x005A6D4C,
        0x00576B41, 0x005D703F, 0x00657244, 0x00626C48, 0x00626B4A, 0x005E6F44, 0x00566C38, 0x00576E38, 0x00596A37, 0x00515C2E, 0x004F572F, 0x00525535, 0x0052553B, 0x00464933, 0x00595E49, 0x006C6554, 0x00706958, 0x00666250, 0x005D5B49, 0x003A3A28,
        0x00424431, 0x004E523F, 0x003C402D, 0x003C441F, 0x003E481D, 0x00415017, 0x00465616, 0x00495919, 0x004F5B1F, 0x00545D2A, 0x00575E31, 0x00506435, 0x004D5A40, 0x008C918F, 0x009B9EA6, 0x00989CA1, 0x009DA5A4, 0x009DA5A5, 0x009BA2A5, 0x008F9689,
        0x00556145, 0x006A7B4F, 0x00647746, 0x00697A4F, 0x006B7955, 0x0066754F, 0x005C6D42, 0x007F8979, 0x00A9AEB1, 0x009DA4AD, 0x00A4B0B0, 0x00A6B3B1, 0x009CA4AB, 0x00A0A6A5, 0x00717C68, 0x005E6E4C, 0x006B7B59, 0x006E7D5D, 0x00556347, 0x005F6A56,
        0x006A7268, 0x007C8180, 0x00787B80, 0x006C6B75, 0x008D8F90, 0x00656B60, 0x00353F28, 0x00505D3D, 0x006B7955, 0x0065734F, 0x006B7955, 0x006D7A4E, 0x00697648, 0x00617141, 0x005D703F, 0x003D5025, 0x001C2D0B, 0x001D2B13, 0x001C2A18, 0x00232819,
        0x001E2217, 0x001C1D19, 0x001C1C1C, 0x001D1D1D, 0x001D1E1A, 0x001C2015, 0x00202516, 0x0017240C, 0x0017230D, 0x001C2615, 0x001C2616, 0x001B2515, 0x001B2514, 0x00192410, 0x001D2913, 0x00232B1A, 0x00202A1D, 0x002C3A34, 0x003C4F52, 0x00657984,
        0x006E8190, 0x00657887, 0x00617483, 0x00617265, 0x0030452C, 0x002C451F, 0x00384F29, 0x00344929, 0x0036492E, 0x00445736, 0x005B7246, 0x0067784D, 0x0067784C, 0x00647647, 0x00647647, 0x0064754A, 0x0044532D, 0x00293616, 0x003E4A2E, 0x003A3F2A,
        0x00515542, 0x005D614E, 0x005A5D4D, 0x005B5E4F, 0x005C5E52, 0x005B5C52, 0x005C5D53, 0x00545442, 0x00484836, 0x00484836, 0x004A4A38, 0x00424230, 0x0041412F, 0x00494937, 0x004D4D3B, 0x00555940, 0x00575945, 0x005A5A4E, 0x0057554D, 0x00524F4A,
        0x0054524A, 0x004F4F41, 0x0040422F, 0x00474739, 0x005A5A4E, 0x0057574B, 0x0059594D, 0x00525147, 0x0055544A, 0x00525048, 0x00514F47, 0x00465D4F, 0x005F8290, 0x00495A63, 0x0050525C, 0x007A8CAB, 0x00727C8D, 0x0056544C, 0x0057676D, 0x0064808B,
        0x0055737E, 0x0054747F, 0x004D6C75, 0x00697D82, 0x00555A5B, 0x005E534F, 0x0067544D, 0x00525244, 0x00515145, 0x00545349, 0x00545349, 0x00515046, 0x0057574B, 0x005B5C4C, 0x00555744, 0x004B4C38, 0x00474735, 0x00504F41, 0x00545247, 0x004C4A3F,
        0x004B4A3C, 0x00464634, 0x00363723, 0x0042432F, 0x005B5B4B, 0x00555549, 0x00585550, 0x0056534F, 0x00595854, 0x0055554F, 0x004D4E45, 0x00606D47, 0x006F7D54, 0x00738157, 0x00707D57, 0x00677252, 0xFFE7EDDA, 0xFFFFFFFC, 0xFFFFFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x6EA9CBE9, 0x00669ED9, 0x0059A1EE, 0x005DA2EB, 0x005FA4EA, 0x005AA5F1, 0x0059A5F1, 0x00579CE1, 0x005C9CE3, 0x005E9EE5, 0x0060A0E7, 0x0060A0E7, 0x005F9FE6,
        0x005E9EE5, 0x005E9EE5, 0x005E9EE5, 0x0064A3E7, 0x005E9FE4, 0x005A9DE2, 0x005B9EE3, 0x00599ADE, 0x005F9EDB, 0x004985C0, 0x000F487F, 0x000B3969, 0x000B3864, 0x000C3762, 0x0015426D, 0x001C4D6D, 0x001E454E, 0x00364C33, 0x005F6835, 0x00526445,
        0x00566942, 0x00617344, 0x0069764A, 0x00636C4B, 0x005D6546, 0x005D6D44, 0x00607544, 0x00576C3B, 0x00586839, 0x00555F35, 0x004D5330, 0x00686A4D, 0x004E503A, 0x00252714, 0x00565949, 0x00A09988, 0x00938C7B, 0x009B9785, 0x006E6C5A, 0x00262614,
        0x004A4C39, 0x00676B58, 0x00494D3A, 0x003C4223, 0x003F4722, 0x00424E1C, 0x00445316, 0x00465517, 0x004C5B1E, 0x00555F2A, 0x00596230, 0x004D6132, 0x00445137, 0x00969B99, 0x00C2C5CD, 0x00C2C7CA, 0x00B3B9B8, 0x00A9B1B0, 0x00B1B8BB, 0x00A2A99C,
        0x00546044, 0x0068794D, 0x006D804F, 0x006B7C51, 0x006B7955, 0x00677650, 0x00495A2F, 0x00808A7A, 0x00CFD4D7, 0x00D6DDE6, 0x00D4E0E0, 0x00D4E1DF, 0x00D2DAE1, 0x00C2C8C7, 0x006B7662, 0x00607348, 0x006B7E53, 0x006E8156, 0x005D6F48, 0x00617050,
        0x00748169, 0x007C8676, 0x00687065, 0x005B6359, 0x006B7565, 0x004F5C42, 0x002C3C1A, 0x0041532A, 0x0065784D, 0x006B7E53, 0x0064764D, 0x0064744B, 0x0065764B, 0x005F7045, 0x0056673C, 0x00475630, 0x003A462A, 0x0039422E, 0x00383F32, 0x00373C2D,
        0x001F2317, 0x00171913, 0x001C1D1B, 0x001A1B19, 0x001A1C16, 0x00272B1F, 0x0034392A, 0x003F4730, 0x00373E29, 0x00363C2B, 0x0034392A, 0x00353A2B, 0x00353B2A, 0x002F3621, 0x00303821, 0x003F4534, 0x003E463B, 0x006E7976, 0x008E9DA0, 0x009BACB5,
        0x0098A9B6, 0x0098A7B7, 0x0098A7B7, 0x0094A0A2, 0x006D7C74, 0x005F725D, 0x005C6D58, 0x00505E52, 0x00515F54, 0x00576553, 0x0056674C, 0x006A7B50, 0x006A7B4F, 0x0067784C, 0x0068794D, 0x0068794E, 0x00475531, 0x002B3818, 0x00404C30, 0x003F412E,
        0x00565845, 0x005E5F4F, 0x00565648, 0x0057574B, 0x005F5E54, 0x00615F57, 0x005F5D55, 0x004F4E40, 0x004D4C3E, 0x00504F41, 0x00545345, 0x00525143, 0x00525143, 0x00535244, 0x00525143, 0x00575945, 0x00555646, 0x0055544A, 0x00524F4A, 0x00524F4A,
        0x0057564C, 0x00535345, 0x00454734, 0x003E3E30, 0x00525244, 0x00515143, 0x0058584C, 0x0056564A, 0x005A594F, 0x0057564C, 0x0055544A, 0x0054655A, 0x006A8691, 0x00576669, 0x00505455, 0x00627086, 0x005B646E, 0x0058564C, 0x00667276, 0x006A8FB1,
        0x005B81A3, 0x005D87A4, 0x00628AA3, 0x006C8D9D, 0x00495E66, 0x00464E4E, 0x00555352, 0x00545448, 0x00505044, 0x00505044, 0x00505044, 0x00505044, 0x00545446, 0x00565747, 0x0050523F, 0x00585945, 0x00575745, 0x005C5B4D, 0x005B594E, 0x00545247,
        0x00555446, 0x004F4F3D, 0x003A3B27, 0x00464733, 0x005E5E4E, 0x0059594D, 0x0059574F, 0x0053504B, 0x00575751, 0x0054554C, 0x004E4F45, 0x005F6C46, 0x006D7B52, 0x00717F55, 0x006F7C56, 0x00667151, 0xFFE7EDDA, 0xFFFFFFFC, 0xFFFFFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x6EAACCEA, 0x00669ED9, 0x0059A1EF, 0x005BA2EC, 0x005FA4EA, 0x005BA5F3, 0x0059A4F2, 0x00579CE1, 0x005A9CE3, 0x005C9EE5, 0x005EA0E7, 0x005EA0E7, 0x005D9FE6,
        0x005C9EE5, 0x005B9DE4, 0x005C9EE5, 0x005F9EE2, 0x005B9CE1, 0x00589AE1, 0x00599EE4, 0x00589CDF, 0x005B9BDB, 0x005A96D1, 0x00427EB4, 0x003A77B9, 0x003C77B5, 0x00427BB3, 0x004D82B5, 0x00497B9F, 0x0038606C, 0x003C5541, 0x00566238, 0x0057684D,
        0x005D6E4C, 0x005F7045, 0x005F6A44, 0x00626A4C, 0x00676E53, 0x00616F4B, 0x00596D40, 0x0056663D, 0x0057643E, 0x00585F3E, 0x00505235, 0x00595843, 0x0093917F, 0x0029291B, 0x0057594D, 0x00BFB8A9, 0x00BFB8A9, 0x00C5C0B1, 0x005A5748, 0x003B3B2B,
        0x00909181, 0x00616454, 0x00232616, 0x00373C27, 0x003E4628, 0x00444E24, 0x00424F1B, 0x00435215, 0x0049581B, 0x00515F25, 0x0059642C, 0x00495B2C, 0x003E4A2E, 0x00939994, 0x00C0C3C8, 0x00B2B7BA, 0x009BA1A0, 0x009AA09F, 0x00B1B8BB, 0x009BA295,
        0x00505C40, 0x00627347, 0x006C7F4E, 0x00697A4F, 0x00677551, 0x00697852, 0x004E5F34, 0x007E8576, 0x00CCD0D1, 0x00C9CED7, 0x00BAC5C3, 0x00B9C4C2, 0x00C4CACF, 0x00BDC2C0, 0x00636C57, 0x00566B3A, 0x00617645, 0x006B814D, 0x00617645, 0x00495D30,
        0x0051633C, 0x00505F3F, 0x003A482C, 0x0037462A, 0x00425332, 0x003F512C, 0x002C4015, 0x00293E11, 0x003C5124, 0x0041552A, 0x0033471D, 0x00354625, 0x00364725, 0x002E3E1C, 0x002B3818, 0x003F482D, 0x00525742, 0x00515143, 0x00504F45, 0x00444A39,
        0x00232819, 0x0014180D, 0x001A1D14, 0x00191C13, 0x0022261B, 0x003C4132, 0x004C5241, 0x004E503C, 0x004C4E3B, 0x00545446, 0x00535347, 0x004D4D3F, 0x00474838, 0x003F412D, 0x00454930, 0x00414435, 0x0050564B, 0x00939C99, 0x00ABB6BA, 0x00A6B2BC,
        0x009DA8B6, 0x009AA5B3, 0x00A0A9B7, 0x00AEB4C7, 0x00A6AFB8, 0x00A4B1AF, 0x00A7B1B1, 0x00A0A8AF, 0x009FA6AF, 0x00959D9C, 0x006F7B6F, 0x006A7B50, 0x00697A4F, 0x00647549, 0x00647549, 0x0065764B, 0x00475531, 0x002E3A1C, 0x00455036, 0x00424333,
        0x00575848, 0x005E5E50, 0x00555549, 0x0056554B, 0x00605E56, 0x00636159, 0x005D5A55, 0x00626056, 0x0068665C, 0x006A685E, 0x006B695F, 0x006D6B61, 0x006C6A60, 0x0069675D, 0x006A685E, 0x005D5D51, 0x0059584E, 0x0055534B, 0x0054524A, 0x0056544C,
        0x0059594D, 0x00545545, 0x00484A36, 0x00454636, 0x00595A4A, 0x00575749, 0x005B5B4D, 0x00555549, 0x0056564A, 0x00545349, 0x0055544A, 0x004B564C, 0x005A6D70, 0x0057615B, 0x0051564D, 0x00505C60, 0x004B5150, 0x00514F44, 0x00595F5E, 0x005F7CA1,
        0x006282A6, 0x006487A8, 0x005F839B, 0x00557483, 0x00455B60, 0x0046544E, 0x0051594E, 0x00555549, 0x00515145, 0x00525244, 0x00535345, 0x00525343, 0x00545545, 0x00575848, 0x00555744, 0x00565743, 0x00575745, 0x005D5C4E, 0x005C5A4F, 0x0059574C,
        0x005E5D4F, 0x00545442, 0x003C3D29, 0x003F402C, 0x00585848, 0x0058584C, 0x005A594F, 0x00535248, 0x0056584C, 0x00505344, 0x00484B3B, 0x00606D47, 0x006B7950, 0x006F7D53, 0x006E7B55, 0x00667151, 0xFFE7EDDA, 0xFFFFFFFC, 0xFFFFFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x6BA8CBEC, 0x00649ED9, 0x0058A2F0, 0x005CA3ED, 0x005EA4EA, 0x005AA5F3, 0x0059A4F2, 0x00569DE1, 0x005B9CE6, 0x005C9DE7, 0x005E9FE9, 0x005E9FE9, 0x005D9EE8,
        0x005B9CE6, 0x005B9CE6, 0x005B9CE6, 0x0062A3E8, 0x005EA1E6, 0x00589DE3, 0x005BA0E6, 0x00589BE0, 0x005797D7, 0x0063A2DC, 0x0067A3D9, 0x005A9CDD, 0x0062A0E0, 0x00629CD7, 0x005489BB, 0x003B6A8A, 0x00284C56, 0x00274435, 0x00344A2D, 0x003E4D38,
        0x003E4D31, 0x0034421E, 0x0037401F, 0x0052583F, 0x006D725D, 0x00636F51, 0x004A5C35, 0x00596646, 0x00677153, 0x003C3F26, 0x005E5D49, 0x005A5546, 0x009E9A8F, 0x00727066, 0x0035362D, 0x00BFB7AA, 0x00CDC5B8, 0x00C5BFB2, 0x004C493B, 0x006D6C5E,
        0x00ACAC9E, 0x002F3223, 0x00202314, 0x00282A1E, 0x00323623, 0x003B4324, 0x00404A1F, 0x0042501A, 0x004B591F, 0x00556427, 0x005C6B2E, 0x00596C3B, 0x004E5A3E, 0x009A9E99, 0x00ABAEB3, 0x00979A9E, 0x00919895, 0x00999F9E, 0x00AEB3B6, 0x00969D90,
        0x00626E52, 0x00708155, 0x006E8150, 0x006D7E53, 0x00667450, 0x006C7B55, 0x00637449, 0x00848C7B, 0x00BFC3C4, 0x00B3B9C0, 0x00B1BCBA, 0x00B0BBB8, 0x00B2B8BD, 0x00B4B9B7, 0x0068715C, 0x0066774C, 0x006A7B4F, 0x00708253, 0x006B7E4D, 0x003E5021,
        0x003D4D24, 0x003B4925, 0x002E3B1B, 0x00293D1A, 0x002F441E, 0x002F441E, 0x00253A14, 0x001C300D, 0x001C300D, 0x001D300F, 0x0017290A, 0x00172A0F, 0x0017280D, 0x0017250D, 0x001C2510, 0x003E422F, 0x005A5A4C, 0x005D584F, 0x006B635C, 0x005D6050,
        0x00484B3B, 0x00303324, 0x002B2E1F, 0x002D3021, 0x00373A2B, 0x004C4F3F, 0x00565949, 0x00565442, 0x004F4C3D, 0x00585547, 0x00575348, 0x00504D3F, 0x00484634, 0x003D3C27, 0x0045462C, 0x0049493D, 0x0070736A, 0x00ACB2B1, 0x00AEB7BB, 0x00B2BBC5,
        0x00AFB7C4, 0x00A7ADBA, 0x00B1B4C2, 0x00A7ACC1, 0x00AEB6C3, 0x00AAB4BB, 0x00AFB6BF, 0x00ADB2C1, 0x00B0B5C4, 0x00A4AAAF, 0x006A756D, 0x0067774E, 0x0067784D, 0x00647549, 0x0065764A, 0x0064744B, 0x0043512D, 0x00273315, 0x003A452B, 0x003D3E2E,
        0x004D4E3E, 0x00555547, 0x00505044, 0x00515046, 0x0056554B, 0x0056544C, 0x00504D48, 0x004C4941, 0x00524F47, 0x004F4C44, 0x004D4A42, 0x00504D45, 0x004C4941, 0x00504D45, 0x00625F57, 0x0059574F, 0x0057554D, 0x0057564C, 0x0059584E, 0x005C5C50,
        0x005D5E4E, 0x005A5C49, 0x00535541, 0x00575946, 0x006A6B5B, 0x00676858, 0x00696A5A, 0x00606052, 0x00616153, 0x00646458, 0x006A6A5E, 0x00596156, 0x005A655B, 0x005D6455, 0x00646A59, 0x00656C5F, 0x0064685C, 0x0069695B, 0x0060625C, 0x00505C62,
        0x005B676D, 0x005A696C, 0x00596766, 0x0055615B, 0x005F6657, 0x00686752, 0x006E6B4F, 0x00606054, 0x005E5E50, 0x005F6050, 0x005E604D, 0x005A5C48, 0x00565844, 0x00585A47, 0x005B5C4C, 0x005C5D49, 0x00585846, 0x00575648, 0x00565449, 0x0057554A,
        0x005E5D4F, 0x00585846, 0x00474834, 0x0052533F, 0x00636353, 0x00626254, 0x00626256, 0x005E5E50, 0x00676B58, 0x00616750, 0x00595F46, 0x0064714B, 0x006D7B52, 0x006F7D53, 0x006F7C56, 0x00687353, 0xFFE8EEDB, 0xFFFFFFFC, 0xFFFFFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x6BA8CBEC, 0x00649DDA, 0x0058A1F1, 0x005AA2EF, 0x005EA3EC, 0x005AA5F3, 0x0058A5F4, 0x00579DE3, 0x00599DE6, 0x005B9FE8, 0x005CA0E9, 0x005CA0E9, 0x005B9FE8,
        0x00599DE6, 0x00599DE6, 0x00599DE6, 0x005FA0E5, 0x0060A2E9, 0x00579BE4, 0x00569BE4, 0x00589DE2, 0x005D9DDE, 0x005896D2, 0x004782BA, 0x004F7CA2, 0x005581A6, 0x00547D9E, 0x0041637A, 0x00253F46, 0x00182B22, 0x001E3019, 0x002A3A1B, 0x00566355,
        0x00435139, 0x00344022, 0x00394025, 0x00474B38, 0x004D5041, 0x00434D36, 0x003A4A28, 0x00525B41, 0x004F553E, 0x00575844, 0x00858071, 0x007D776A, 0x00716C63, 0x00A4A199, 0x00595651, 0x00999184, 0x00A1998C, 0x00A59F92, 0x007A7769, 0x008F8E80,
        0x00727264, 0x000E1102, 0x002F3223, 0x0021241B, 0x002A2F20, 0x00363C25, 0x003C4622, 0x00414C1E, 0x0046541E, 0x00505E24, 0x00576629, 0x005A6A3A, 0x004F593B, 0x00989C96, 0x00A6A7AB, 0x00989C9D, 0x00A3A8A6, 0x00A4ABA8, 0x00AAAFB0, 0x008E9588,
        0x005F6B4F, 0x00728357, 0x006D804F, 0x006D7E53, 0x006B7955, 0x006F7E58, 0x0064754A, 0x008C9281, 0x00C0C4C5, 0x00B4B7BF, 0x00B4BFBC, 0x00B3BEBB, 0x00B7BCBF, 0x00BCC2BD, 0x00737B64, 0x00687654, 0x006A7854, 0x0066754F, 0x00576640, 0x002B3915,
        0x002E3B1B, 0x002B371B, 0x001B250E, 0x002F4122, 0x00324427, 0x002C3C24, 0x00303F2A, 0x003C4A38, 0x00384535, 0x002E3B2B, 0x00323F31, 0x00384A33, 0x0031402B, 0x00333D2C, 0x0032392C, 0x00404137, 0x00514E46, 0x00655F58, 0x008B837C, 0x006F7263,
        0x00696C5D, 0x004E5141, 0x004C4F3F, 0x00575A4A, 0x00525545, 0x00595C4D, 0x0066695A, 0x00737061, 0x00595648, 0x00555146, 0x00514D42, 0x00524F41, 0x004F4E3A, 0x003D3D25, 0x00434528, 0x0046473D, 0x00797B75, 0x00B1B7B6, 0x00ADB5BC, 0x00B3B9C4,
        0x00B1B7C4, 0x00ABAFBA, 0x00B4B6C1, 0x00AFB7C4, 0x00B3BDC4, 0x00A8B4B6, 0x00ABB3BA, 0x00AAAFBE, 0x00AEB4BF, 0x00A4ACAB, 0x00687565, 0x0065754C, 0x0068784F, 0x00697A4F, 0x006C7D52, 0x0068784F, 0x00404E2C, 0x001C280C, 0x002B351E, 0x001E1F0B,
        0x00222210, 0x00272715, 0x002A2A1A, 0x0029281A, 0x0029271C, 0x0028261C, 0x0026241A, 0x00232117, 0x0026241A, 0x00222016, 0x00222016, 0x00232117, 0x001B190F, 0x0029271D, 0x004D4B41, 0x002D2D21, 0x002D2D1F, 0x002C2D1D, 0x002C2E1B, 0x002B2D19,
        0x00292B17, 0x00292B17, 0x002C2E1A, 0x00323420, 0x003E402D, 0x00373926, 0x003D3F2C, 0x00383929, 0x00393A2A, 0x0038382A, 0x003A3A2C, 0x003A3F30, 0x00333521, 0x00303421, 0x00363C29, 0x00393C23, 0x003C3E2A, 0x00434839, 0x003C3C2C, 0x0043462C,
        0x003D4027, 0x00363723, 0x0041412F, 0x00484634, 0x0048432E, 0x003F381F, 0x00453A1F, 0x00343329, 0x00333325, 0x00323420, 0x0032361D, 0x0034381F, 0x00333620, 0x00333522, 0x00343525, 0x003C3D29, 0x002E2E1C, 0x00242315, 0x00211F14, 0x00232116,
        0x00272618, 0x00282816, 0x00242511, 0x00252513, 0x002C2C1C, 0x002B2C1C, 0x00323323, 0x0040422F, 0x00636950, 0x006F7758, 0x00707958, 0x006A7751, 0x00707E55, 0x00707E54, 0x00707D57, 0x006A7555, 0xFFE8EEDB, 0xFFFEFFFB, 0xFFFEFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x67A6CBED, 0x00639FDB, 0x0057A2F1, 0x005AA2EF, 0x005DA5ED, 0x0058A5F4, 0x0058A5F4, 0x00559DE3, 0x00599CE7, 0x005B9EE9, 0x005DA0EB, 0x005C9FEA, 0x005B9EE9,
        0x00599CE7, 0x00589BE6, 0x00589BE6, 0x005597DE, 0x005BA0E6, 0x005398E1, 0x005196DF, 0x005CA1E7, 0x0063A5E6, 0x004584C0, 0x000D4A82, 0x002A4756, 0x00223F4E, 0x00263E4A, 0x0033413F, 0x00393E29, 0x00454422, 0x004F532A, 0x00515931, 0x00263129,
        0x00273321, 0x00465137, 0x006A6F5A, 0x0065675B, 0x00494A40, 0x00394130, 0x003A482C, 0x00373E29, 0x00292E19, 0x00686654, 0x00989384, 0x00978E84, 0x007F786F, 0x00A3A098, 0x0094928A, 0x007B7268, 0x007D746A, 0x00888277, 0x00A19D92, 0x0097958A,
        0x006C6C60, 0x004D4F43, 0x001D1F13, 0x0023261D, 0x0024281C, 0x002B311E, 0x00373F21, 0x003D4822, 0x00424D1F, 0x004B5923, 0x0055642B, 0x004A5A2A, 0x00404A2C, 0x00999B95, 0x00AFB0B4, 0x00A3A5A6, 0x00A9AFAA, 0x00AAAFAD, 0x00B5B9BA, 0x008A9184,
        0x00465236, 0x0067784C, 0x006D804F, 0x006B7C51, 0x00717F5B, 0x006B7A54, 0x004D5E33, 0x00747A67, 0x00C1C3C3, 0x00BDC1C6, 0x00BCC5C2, 0x00BBC5BF, 0x00C1C6C9, 0x00BEC2BD, 0x00575F48, 0x00636E4D, 0x00717C5B, 0x00687450, 0x00475231, 0x002E381A,
        0x004B533C, 0x00686E5D, 0x006E7266, 0x005D6953, 0x00677160, 0x00656E61, 0x00666D66, 0x006E746F, 0x00666C67, 0x005D635E, 0x00646B64, 0x00616C58, 0x005D6756, 0x006C7366, 0x0071746B, 0x0060605A, 0x0054514C, 0x005E5B53, 0x00747268, 0x00737169,
        0x00706F65, 0x0059594B, 0x00656656, 0x00767767, 0x00606052, 0x005F5E54, 0x0076746C, 0x006E6D5F, 0x00535146, 0x00535146, 0x00504E43, 0x004F4E40, 0x00494A36, 0x0036391F, 0x003F4425, 0x00444940, 0x00747A75, 0x00AFB7B7, 0x00B5BFC6, 0x00B1BAC4,
        0x00A8AEB9, 0x00AEB0BB, 0x00B2B3BD, 0x00A9B3BA, 0x00ADB9BB, 0x00A8B4B4, 0x00B0B8BF, 0x00B0B6C3, 0x00B2B9C2, 0x00A5B0A8, 0x006A7A63, 0x00677650, 0x0068784F, 0x0067784D, 0x0068794E, 0x0066754F, 0x0042502E, 0x00232F13, 0x00343E27, 0x00494C33,
        0x0043462D, 0x00444630, 0x00484935, 0x00474735, 0x00474735, 0x004A4A3A, 0x004B4B3B, 0x004C4C3A, 0x004A4A38, 0x00434331, 0x003F3F2D, 0x00353523, 0x00222210, 0x002E2E1C, 0x00565644, 0x00434630, 0x0042462D, 0x003E4328, 0x003C4124, 0x003A3F24,
        0x00373B22, 0x003D3F2B, 0x00484939, 0x003F412D, 0x00444632, 0x00383A26, 0x00424431, 0x00464835, 0x00484A37, 0x003F4030, 0x00383929, 0x00353B28, 0x003D3B1D, 0x003D422D, 0x00424C3C, 0x00424125, 0x00393D24, 0x00394434, 0x003B3D20, 0x00363C1D,
        0x0040452A, 0x00434531, 0x00444535, 0x004E4E42, 0x0049493D, 0x00474739, 0x00505141, 0x004A493F, 0x0048483A, 0x0041442E, 0x003E4326, 0x00444A2B, 0x00494E33, 0x00454734, 0x00414133, 0x0051523E, 0x00474735, 0x00414032, 0x00444237, 0x00464439,
        0x00454436, 0x00454533, 0x00454632, 0x00464634, 0x00454535, 0x003D3E2E, 0x003D3F2C, 0x00444731, 0x00666E4F, 0x006E7953, 0x006E7A50, 0x006E7B55, 0x00728057, 0x00707E54, 0x00717E58, 0x006B7656, 0xFFE9EFDC, 0xFFFEFFFB, 0xFFFDFCFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x6BA7CCEE, 0x00639EDC, 0x0057A2F1, 0x005AA2EF, 0x005DA4EE, 0x0059A6F5, 0x0056A5F4, 0x00559DE5, 0x00599DE8, 0x005A9EE9, 0x005CA0EB, 0x005B9FEA, 0x005A9EE9,
        0x00589CE7, 0x00579BE6, 0x00579BE6, 0x005C9EE5, 0x005DA1EA, 0x00579CE5, 0x00569BE4, 0x00599EE4, 0x005C9EDF, 0x004584C1, 0x0014508B, 0x00285576, 0x001B4667, 0x00163B55, 0x001E343A, 0x00292F1E, 0x003D3E1E, 0x004A4E2B, 0x003F4C2C, 0x0028312E,
        0x00364033, 0x004B543F, 0x004A4D3D, 0x0037382F, 0x00373731, 0x0031382B, 0x001C2911, 0x00282F1A, 0x002D2F1C, 0x002F2D1B, 0x00787162, 0x00978F82, 0x00958F84, 0x009F9A91, 0x0086847A, 0x00877E75, 0x00867D74, 0x00857E75, 0x00908B82, 0x008E8C82,
        0x009B9A90, 0x00A7A89E, 0x004A4B41, 0x0024281C, 0x001C2014, 0x001E2413, 0x002D351E, 0x003B4324, 0x00424D25, 0x00515D2D, 0x00626E3C, 0x00566734, 0x00424C2E, 0x009D9F99, 0x00BBBABE, 0x00AFB1B1, 0x00B4B8B3, 0x00B4BAB5, 0x00C6CBCA, 0x0092998C,
        0x00445034, 0x006B7C50, 0x00738655, 0x006F8055, 0x006D7B57, 0x005B6A44, 0x0044552A, 0x00898D7A, 0x00D1D3D3, 0x00C7CACF, 0x00BFC9C3, 0x00BDC7C1, 0x00C8CBCF, 0x00C9CDC8, 0x00626851, 0x005B6A3E, 0x006F7E52, 0x0067754B, 0x00404E25, 0x003D4827,
        0x005F684E, 0x009AA08F, 0x00C1C5BA, 0x00AEB5A0, 0x00B6BCAB, 0x00B7BBB0, 0x009FA09C, 0x007F807E, 0x00747571, 0x0073766D, 0x006A6E62, 0x005F644F, 0x00686B5B, 0x00808177, 0x007E7E78, 0x0053524E, 0x00474741, 0x0065675B, 0x00737667, 0x00736F6E,
        0x00706D68, 0x005A5A4E, 0x0068685A, 0x00737365, 0x0059594D, 0x00595651, 0x006C6867, 0x006B6B5F, 0x0059584E, 0x0067665C, 0x0066665A, 0x005E5F4F, 0x0051543E, 0x003C4124, 0x004C5332, 0x004A524B, 0x00828B88, 0x00ABB7B7, 0x00AFBBC1, 0x00B1BCC4,
        0x00A7B0BA, 0x00B3B8C1, 0x00B7BAC2, 0x00AFB8C1, 0x00AFBABE, 0x00AAB5B9, 0x00AEB4BF, 0x00ACB0C2, 0x00AFB5C2, 0x00A2ACA6, 0x006D7D66, 0x00687751, 0x0067774E, 0x0064744B, 0x0064744B, 0x0064734D, 0x00445131, 0x00293519, 0x003E4831, 0x0047492B,
        0x00414325, 0x00414326, 0x0045462C, 0x0044442C, 0x00484732, 0x004B4A35, 0x00494834, 0x00474B2E, 0x0045492C, 0x0043472A, 0x0043472A, 0x003C4023, 0x002F3316, 0x003D4124, 0x005F6346, 0x0059603D, 0x0058603B, 0x00555E36, 0x00576038, 0x005A613E,
        0x00585D42, 0x005C5E4B, 0x0069695D, 0x0051543E, 0x00545741, 0x00484B35, 0x00545642, 0x005B5D49, 0x00616350, 0x005C5E4B, 0x00595B48, 0x00414C32, 0x0058522D, 0x00575B48, 0x005B665E, 0x00666148, 0x0052553F, 0x00435544, 0x004B5029, 0x00535843,
        0x006E7161, 0x0076786C, 0x00595A50, 0x00565950, 0x0042483D, 0x00455040, 0x0052604E, 0x006B6961, 0x006D6E5E, 0x0060644B, 0x004F5635, 0x004D5433, 0x004F5437, 0x004C4E3A, 0x0049493B, 0x004C4D39, 0x004D4D3B, 0x00504F41, 0x00555348, 0x0057554A,
        0x00555446, 0x00525240, 0x0050513D, 0x00535341, 0x00525242, 0x0050523F, 0x004C4E3A, 0x004D5237, 0x006A7450, 0x006E7B4F, 0x006C7A4A, 0x006F7C56, 0x00717F56, 0x006E7C52, 0x00707D57, 0x006B7656, 0xFFE9EFDC, 0xFFFEFFFB, 0xFFFDFCFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x6BA7CCEE, 0x00639EDC, 0x0058A2F4, 0x0058A2F0, 0x005DA4EE, 0x0059A6F5, 0x0056A5F6, 0x00559DE5, 0x00599DE8, 0x005B9FEA, 0x005CA0EB, 0x005CA0EB, 0x005A9EE9,
        0x00589CE7, 0x00579BE6, 0x00579BE6, 0x005D9FE6, 0x00569AE3, 0x00579BE6, 0x005CA3ED, 0x00569CE2, 0x005697DB, 0x005F9EDB, 0x005793CE, 0x003C85C9, 0x00448BCE, 0x004584BE, 0x00305B7C, 0x00192F34, 0x001E2D1F, 0x00243625, 0x00122C1F, 0x001F2726,
        0x00404A3E, 0x00464E3D, 0x00212415, 0x0020201A, 0x005F5D5C, 0x00646A5F, 0x00232F19, 0x00272C17, 0x00232512, 0x0033311F, 0x004D4637, 0x008A8275, 0x00918B7E, 0x00928E83, 0x008D8B80, 0x00867D74, 0x00898077, 0x008B847B, 0x0079746B, 0x00838177,
        0x009D9C92, 0x00B0B1A7, 0x00A4A59B, 0x00333A2B, 0x0024291A, 0x001E2413, 0x002A311C, 0x00363D22, 0x003B4521, 0x004B552B, 0x005E693B, 0x00606F3C, 0x00394424, 0x00898C83, 0x00ACABAD, 0x00AEB0B0, 0x00BABEB9, 0x00B2B8B3, 0x00B9BEBD, 0x00858C7F,
        0x00404C30, 0x006A7B4F, 0x006F8251, 0x006F8055, 0x0063714D, 0x004B5A34, 0x00526338, 0x00868A77, 0x00C5C7C7, 0x00C0C3C8, 0x00CAD4CE, 0x00C6D0CA, 0x00BEC2C3, 0x00B7BBB5, 0x00555B42, 0x00657841, 0x00748750, 0x0072844F, 0x00536334, 0x004F5D34,
        0x00444E30, 0x005C624F, 0x007C8172, 0x00979B82, 0x00A1A390, 0x00B9B9AD, 0x00ACAAA2, 0x0086837E, 0x008C8B81, 0x00929383, 0x006F715D, 0x00676652, 0x008F8F7F, 0x00C3C1B7, 0x00B8B5B1, 0x00676662, 0x00484B42, 0x006A7162, 0x006F7864, 0x006D6869,
        0x006D6A66, 0x005A594F, 0x00626254, 0x0068685A, 0x0059584E, 0x00615E5A, 0x00696465, 0x006A6C60, 0x0053544A, 0x005E5F55, 0x005D5F53, 0x00565949, 0x00494F38, 0x0031391B, 0x00404928, 0x00636E66, 0x00A7B5B1, 0x00B4C1C3, 0x00A0AEB4, 0x00B1BEC6,
        0x00ABB6BE, 0x00AFB7BE, 0x00B4BAC1, 0x00B2BAC7, 0x00B1BAC3, 0x00B0B9C3, 0x00AFB3C5, 0x00ABACC6, 0x00AFB3C6, 0x009EA6A6, 0x006B7967, 0x0066754F, 0x00687751, 0x0068784F, 0x006A7A51, 0x00687751, 0x00435030, 0x00222D13, 0x00333D26, 0x003F4320,
        0x003D411E, 0x00404323, 0x00424426, 0x00434528, 0x00494A30, 0x004B4B33, 0x0043432B, 0x00424825, 0x00404623, 0x003F4522, 0x00414724, 0x003D4320, 0x00383E1B, 0x00484E2B, 0x00636946, 0x00626C41, 0x005E693B, 0x00596436, 0x005B6638, 0x005C653D,
        0x0052573A, 0x004D4F3C, 0x00545349, 0x004E5239, 0x00535640, 0x00454832, 0x004A4D37, 0x00494B37, 0x004E503C, 0x0050523F, 0x00565845, 0x003F4D31, 0x00524C23, 0x003D4332, 0x00404B49, 0x005A543D, 0x004F523C, 0x003D5341, 0x004B5225, 0x004D4A3B,
        0x005D5A4C, 0x00666257, 0x00524E43, 0x00595A4A, 0x00313823, 0x00313F23, 0x00475A39, 0x005A5850, 0x00656656, 0x005C6146, 0x00444B28, 0x0039401D, 0x003D4225, 0x00434531, 0x0049493B, 0x00484935, 0x004D4D3B, 0x004D4C3E, 0x004A483D, 0x004A483D,
        0x004C4B3D, 0x004C4C3A, 0x004B4C38, 0x004F4F3F, 0x00505040, 0x00515340, 0x004B4E38, 0x00484D30, 0x0066714B, 0x006E7B4D, 0x0071814C, 0x006E7B55, 0x006F7D54, 0x006C7A50, 0x006F7C56, 0x006B7656, 0xFFE9EFDC, 0xFFFEFFFB, 0xFFFEFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x4C97C1EB, 0x00629FE1, 0x0056A0F2, 0x005EA2ED, 0x0064A6ED, 0x005DA5F3, 0x005BA5F7, 0x00579BE6, 0x00589BE6, 0x005C9FEA, 0x00599DE8, 0x005DA1EC, 0x005CA0EB,
        0x00569DE7, 0x005AA1EB, 0x00579EE8, 0x00619FE5, 0x0063A3EA, 0x005D9EE8, 0x005697E1, 0x005A9AE1, 0x00629FE3, 0x00629DDB, 0x006097D4, 0x005D9CCF, 0x006396D5, 0x005993C7, 0x004B90C2, 0x003C73A6, 0x00274A4E, 0x00314B23, 0x0024381B, 0x001B1F13,
        0x00262919, 0x002C2E1A, 0x004A4D34, 0x002A2E15, 0x00454B34, 0x0047503B, 0x001E2A16, 0x00262913, 0x002E301C, 0x00323421, 0x00757666, 0x00848478, 0x0086857B, 0x008D8B83, 0x00908D88, 0x00938D82, 0x00888178, 0x008C877E, 0x00807B72, 0x007D7A72,
        0x009F9C94, 0x00AAA7A2, 0x00999691, 0x006A6860, 0x00252713, 0x00262C15, 0x0026281C, 0x002D3320, 0x003C4A20, 0x00445522, 0x00536439, 0x00626946, 0x00404724, 0x00464D2A, 0x004A5130, 0x00606746, 0x00717758, 0x00666C4D, 0x00646A4B, 0x006A715C,
        0x00464F34, 0x00798463, 0x00727F59, 0x0067754C, 0x00505C38, 0x00606B4B, 0x005A6348, 0x00686B5C, 0x00585B4B, 0x006F7360, 0x008F947F, 0x008B907B, 0x00696D5A, 0x00636656, 0x00525546, 0x0061704A, 0x006F7E58, 0x006E7E55, 0x0066744B, 0x004D5B31,
        0x0039461A, 0x00303D0F, 0x003C491B, 0x00576732, 0x0051622F, 0x004A5B2F, 0x0043572D, 0x003E5431, 0x00395133, 0x00314A30, 0x002A442C, 0x00314C2A, 0x00385030, 0x00405639, 0x00425038, 0x00454E3A, 0x00525546, 0x00636156, 0x007D786F, 0x0077746C,
        0x00726F67, 0x005A584E, 0x006A685D, 0x00666557, 0x005B5A4C, 0x00626252, 0x006F6F5D, 0x0067655A, 0x00545247, 0x00656359, 0x006D6B61, 0x005B594E, 0x00484836, 0x003A3D24, 0x00424629, 0x00687069, 0x00A9B1B0, 0x00ABB3BA, 0x00AAB2BF, 0x00B7BDC8,
        0x00BDC5C5, 0x00ABB4A7, 0x00979E89, 0x00889A6B, 0x00869470, 0x0096A488, 0x00AFBAAA, 0x00B9BDC8, 0x00A9A9C7, 0x00A2ADB1, 0x00667E5B, 0x0068754F, 0x006B7B52, 0x00697C51, 0x00667A50, 0x0062754E, 0x00424F2F, 0x00262D12, 0x00383B25, 0x003C3E21,
        0x003E4022, 0x00414424, 0x00434724, 0x00434724, 0x00424426, 0x0043442A, 0x0044442C, 0x00404526, 0x00444629, 0x0045442A, 0x00454128, 0x00424022, 0x003B3F1C, 0x00495329, 0x00637042, 0x005D6C40, 0x005B6B3C, 0x00586936, 0x005A6B38, 0x005B6A3E,
        0x004E5939, 0x00434A35, 0x0044483C, 0x004A5443, 0x00444B3C, 0x005A6739, 0x0055632D, 0x0042452F, 0x004D5040, 0x003E492F, 0x00485046, 0x004C5834, 0x004A5D26, 0x0047532F, 0x003E4433, 0x004D563C, 0x00424934, 0x00545D43, 0x004D6126, 0x00475630,
        0x00414B34, 0x00494F44, 0x00474C43, 0x0058614C, 0x00515D3F, 0x00596646, 0x00455135, 0x004B514C, 0x0069735C, 0x005C6C3D, 0x00475924, 0x0039461A, 0x00404627, 0x0045482F, 0x004B4D37, 0x004B4E38, 0x004C4E3A, 0x004D4F3B, 0x004C4E3B, 0x004B4D3A,
        0x004B4C3C, 0x004C4D3D, 0x004E4E40, 0x004D4C38, 0x0051503C, 0x004C4C3A, 0x00494B37, 0x004D533A, 0x00687353, 0x00697650, 0x0066774C, 0x006E7C52, 0x006F7E52, 0x00707F53, 0x00728056, 0x00677252, 0xFFE6ECD9, 0xFFFFFFFC, 0xFFFEFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x4C97C1EB, 0x0060A0E1, 0x0056A0F2, 0x005EA2ED, 0x0064A6ED, 0x005BA5F3, 0x005BA5F7, 0x00579BE6, 0x00589BE6, 0x005C9FEA, 0x00599DE8, 0x005CA0EB, 0x005A9EE9,
        0x00549BE5, 0x00579EE8, 0x00559CE6, 0x005B99DF, 0x005B9BE2, 0x005A9BE5, 0x005899E3, 0x005E9EE5, 0x00619EE2, 0x004C87C5, 0x003067A4, 0x0030699C, 0x003764A1, 0x00477EB1, 0x005798CB, 0x005388BB, 0x003A5A60, 0x003C5431, 0x0031442B, 0x00212519,
        0x00363929, 0x00363824, 0x005C5F46, 0x003B3E25, 0x00313720, 0x00303924, 0x001C2814, 0x00292C16, 0x00222410, 0x003C3E2B, 0x00828373, 0x008C8C80, 0x00919086, 0x0087857D, 0x007E7B76, 0x00969085, 0x00898279, 0x0089847B, 0x00807B72, 0x007A776F,
        0x00928F87, 0x00A09D98, 0x009F9C97, 0x007D7B73, 0x00353723, 0x002B311A, 0x00232519, 0x00242A17, 0x0038461C, 0x00475825, 0x005E6C42, 0x00616A42, 0x004B532E, 0x004B532E, 0x00424A25, 0x003D4421, 0x004B522F, 0x00444B2A, 0x004D5433, 0x00646D53,
        0x00636D4F, 0x006C7854, 0x0068764D, 0x00556339, 0x0058653F, 0x00465130, 0x003F4A2A, 0x00686E57, 0x004C5239, 0x004D5439, 0x00575E43, 0x00565D42, 0x004D5439, 0x00595F46, 0x00545A43, 0x0065764B, 0x006D7E53, 0x0075865B, 0x006A784F, 0x004F5C36,
        0x00353F1B, 0x00343E1A, 0x003F4925, 0x00455127, 0x003B461E, 0x002F3C16, 0x00263412, 0x001C2B0C, 0x00142308, 0x0012220A, 0x0015250E, 0x000F2209, 0x0016260E, 0x00121F09, 0x0018230F, 0x00424837, 0x00575A4B, 0x004E4E42, 0x00555348, 0x00626055,
        0x00636156, 0x00545247, 0x00666459, 0x005F5D52, 0x00514F44, 0x0057554A, 0x00656358, 0x006C6A5F, 0x0059574C, 0x006B695F, 0x00727066, 0x00605E53, 0x004B4B39, 0x003B3E25, 0x00424629, 0x0069706B, 0x00A6AEAE, 0x00A4ACB3, 0x00B1BAC4, 0x00B7BFC6,
        0x009BA59F, 0x007F8A76, 0x00768062, 0x00738459, 0x006E7C58, 0x0070805E, 0x008A9880, 0x00ABB2B5, 0x00B2B4CC, 0x009FAAAE, 0x00667E5C, 0x0068754F, 0x006B7B52, 0x00687B50, 0x0065794F, 0x0062754E, 0x00424F2F, 0x00262D12, 0x00383B25, 0x003B3D20,
        0x003D3F21, 0x003F4222, 0x003F4320, 0x00414522, 0x00434527, 0x0045462C, 0x0046462E, 0x00404427, 0x00434528, 0x00444329, 0x00454128, 0x00424022, 0x003B3F1C, 0x00485228, 0x005F6F40, 0x005C6B3F, 0x0059693A, 0x00556633, 0x00576737, 0x0058663C,
        0x00505B3B, 0x0049503B, 0x004E5246, 0x003E4838, 0x00464D3E, 0x00515E32, 0x0055632D, 0x00434630, 0x00484B3B, 0x00465137, 0x004B5349, 0x00525D3D, 0x00526330, 0x00434E2D, 0x003E4236, 0x004C533E, 0x003B4130, 0x00525944, 0x004F622B, 0x00485632,
        0x003F4932, 0x004E534A, 0x00474C43, 0x005C6551, 0x005F6B4D, 0x005C684A, 0x00465236, 0x004C524D, 0x0068725B, 0x005B6B3C, 0x00475924, 0x0039461A, 0x00404627, 0x0045482F, 0x004B4D37, 0x00484B35, 0x00494B37, 0x004B4D39, 0x004B4D39, 0x004A4C39,
        0x004A4C39, 0x004B4C3C, 0x004C4D3D, 0x004C4B37, 0x00504F3B, 0x004C4C3A, 0x004A4C38, 0x004E543B, 0x006A7555, 0x006B7852, 0x0068794E, 0x006D7B51, 0x006E7D51, 0x006F7E52, 0x00717F55, 0x00687353, 0xFFE7EDDA, 0xFFFFFFFC, 0xFFFFFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x4896C2EB, 0x005F9FE0, 0x0056A0F2, 0x005CA3ED, 0x0062A7ED, 0x005BA5F3, 0x005BA5F7, 0x00549BE5, 0x00589BE6, 0x005C9FEA, 0x00589CE7, 0x005B9FEA, 0x00599DE8,
        0x005299E3, 0x00559CE6, 0x005299E3, 0x0062A0E6, 0x005E9EE5, 0x005C9DE7, 0x00579BE4, 0x005B9DE4, 0x005F9EE2, 0x00417CBA, 0x000F4885, 0x00093B69, 0x00123872, 0x003B6C9E, 0x005794C6, 0x005584BA, 0x003B5A63, 0x003B5232, 0x00344530, 0x004A4E42,
        0x00696C5C, 0x004C4D39, 0x005D6047, 0x0045482F, 0x00333922, 0x00353E29, 0x00202B17, 0x00292B15, 0x0030311D, 0x004A4A38, 0x00777767, 0x008C8A7F, 0x009B998F, 0x0087847C, 0x00898580, 0x00908A7F, 0x00867F76, 0x0088837A, 0x0089847B, 0x00838078,
        0x008A877F, 0x009A9792, 0x00ABA8A3, 0x0094928A, 0x00464834, 0x00353822, 0x002D2F23, 0x00292D1A, 0x0037451B, 0x00435421, 0x005D6B41, 0x006A7449, 0x005A643A, 0x00555F35, 0x00535D33, 0x00454E26, 0x0059623A, 0x0048502B, 0x00545C37, 0x00566141,
        0x00636F4B, 0x006D7B52, 0x004D5B31, 0x00546337, 0x00505E34, 0x00606E45, 0x005F6C46, 0x0068724E, 0x00545E3A, 0x0057613D, 0x005A6440, 0x005B6541, 0x005D6743, 0x00626C48, 0x005B6541, 0x00657845, 0x00677A49, 0x00708155, 0x005F6C46, 0x00535E3E,
        0x003F452C, 0x00424732, 0x003E422F, 0x00474F31, 0x00434A2F, 0x00464D32, 0x004E5239, 0x004C4F39, 0x00434531, 0x003F412D, 0x00434532, 0x00414637, 0x003B4031, 0x00333829, 0x0036392A, 0x004B4E3F, 0x00575A4B, 0x004E4E40, 0x004E4E40, 0x00474735,
        0x00505040, 0x004F4F3F, 0x00676658, 0x00626055, 0x00555349, 0x005B5850, 0x006D6964, 0x00666459, 0x00555348, 0x0067655B, 0x00706E64, 0x005E5C51, 0x00494937, 0x00383B22, 0x003F4326, 0x00686E6D, 0x00AAB1B4, 0x00A8B0B7, 0x00B4BDC6, 0x00A9B3B3,
        0x00788577, 0x00687758, 0x00707E54, 0x006B7A54, 0x0072815B, 0x00687A51, 0x00687A55, 0x00848F85, 0x00B2B7C6, 0x009EAAAC, 0x00677D60, 0x0068754F, 0x006A7A51, 0x00677A4F, 0x0065794F, 0x0062754E, 0x00414E2E, 0x00262D12, 0x00393C26, 0x003A3C1F,
        0x003C3E20, 0x003C3F1F, 0x003B3F1C, 0x003E421F, 0x00454729, 0x0048492F, 0x0047472F, 0x00404528, 0x0041442A, 0x00434228, 0x00454128, 0x00414024, 0x003C401D, 0x00465228, 0x005D6C40, 0x005B693F, 0x0059693A, 0x00556535, 0x00576737, 0x0059673D,
        0x00515C3C, 0x004D5340, 0x0053574B, 0x00414A3D, 0x004B5245, 0x00546036, 0x0055622E, 0x00464834, 0x004E5142, 0x00454F38, 0x004D534E, 0x004F583D, 0x00526233, 0x0040492E, 0x0041433D, 0x00515647, 0x00494D42, 0x005A5F50, 0x004C5C2C, 0x00465430,
        0x003F4833, 0x0050554C, 0x004A4E48, 0x00636B5A, 0x00697559, 0x005F6B4D, 0x00465137, 0x004E544F, 0x0067715A, 0x005A6A3B, 0x00465823, 0x00384519, 0x003F4526, 0x00464930, 0x004B4D37, 0x00464733, 0x00484935, 0x004A4B37, 0x004A4B37, 0x004A4B37,
        0x00494A36, 0x00494A36, 0x004A4B37, 0x004A4934, 0x004F4E3A, 0x004B4C38, 0x00494B37, 0x004E543B, 0x006B7656, 0x006C7953, 0x006A7B50, 0x006E7C52, 0x006E7D51, 0x006E7D51, 0x00717F55, 0x00697454, 0xFFE9EFDC, 0xFFFFFFFC, 0xFFFFFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x4896C2EB, 0x005F9FE0, 0x0054A0F2, 0x005BA2EC, 0x0062A7ED, 0x005BA5F3, 0x0059A5F7, 0x00549BE5, 0x00579AE5, 0x005C9FEA, 0x00589CE7, 0x005B9FEA, 0x00599DE8,
        0x00539AE4, 0x00569DE7, 0x00539AE4, 0x005C9DE2, 0x00599BE2, 0x00599DE6, 0x00579BE4, 0x005A9CE3, 0x0063A2E6, 0x004784C2, 0x00124B88, 0x00154470, 0x001D4078, 0x004A78A8, 0x005C97C9, 0x00507EB4, 0x003C5962, 0x003D5336, 0x00394937, 0x0064665A,
        0x00878878, 0x00585945, 0x005C5C44, 0x004B4E35, 0x003B3E28, 0x00444B36, 0x0028331F, 0x002E301A, 0x00333420, 0x004A4A38, 0x00737363, 0x00817F74, 0x0086847A, 0x007B7870, 0x00898580, 0x00898378, 0x00847D74, 0x008A857C, 0x00959087, 0x008F8C84,
        0x0088857D, 0x0095928D, 0x00B2AFAA, 0x00A3A098, 0x004D4E3A, 0x00343721, 0x003C3C30, 0x00333724, 0x0039451B, 0x003C4D1A, 0x005A683E, 0x00667345, 0x00687547, 0x006C794B, 0x00727F53, 0x00515E32, 0x00626E44, 0x00566238, 0x0078845A, 0x00717E58,
        0x006D7B52, 0x00515F35, 0x0057663A, 0x004B5A2E, 0x00657448, 0x006E7D51, 0x006F7E52, 0x00718152, 0x006B7A4E, 0x00748258, 0x00748259, 0x00748259, 0x0078865C, 0x0077865A, 0x00708051, 0x006E844A, 0x006A7E49, 0x00647647, 0x00404C28, 0x004B543A,
        0x00555848, 0x00787970, 0x0083837D, 0x006C705D, 0x006E705D, 0x00727461, 0x00777765, 0x00777563, 0x00706C5A, 0x006B6453, 0x00696251, 0x006D6964, 0x005B5850, 0x006C6961, 0x006E6D63, 0x0056564A, 0x00505344, 0x00505343, 0x00434636, 0x0042442E,
        0x004C4D39, 0x0050503E, 0x006A695B, 0x0067655B, 0x005A5651, 0x005B5653, 0x006C6766, 0x00666459, 0x00545247, 0x0068665C, 0x00716F65, 0x00605E53, 0x004D4D3B, 0x003E4128, 0x0045492C, 0x00666B6C, 0x00AEB4B9, 0x00ACB4BB, 0x00B0BAC1, 0x0096A29C,
        0x00697962, 0x006B7D54, 0x00738550, 0x00717F5B, 0x0075845E, 0x00708455, 0x00657B4B, 0x00687762, 0x00A7AEB7, 0x009EAAAC, 0x006B8067, 0x00697650, 0x00697950, 0x0066794E, 0x0065794F, 0x0062754E, 0x00414E2E, 0x00262D12, 0x003A3D27, 0x00393B1E,
        0x003C3E20, 0x003C3F1F, 0x00393D1A, 0x003C401D, 0x00444628, 0x0047482E, 0x0045452D, 0x00414629, 0x0042452B, 0x0042422A, 0x0045432B, 0x00434226, 0x003D4222, 0x0047522A, 0x005C6B3F, 0x005C6A40, 0x005B6A3E, 0x00596939, 0x005B6B3B, 0x005B693F,
        0x00525D3D, 0x004C523F, 0x0052564B, 0x00485246, 0x004C544A, 0x00546239, 0x00505E2E, 0x00454838, 0x0055594D, 0x003D4834, 0x004E5552, 0x00464F3A, 0x003C4C23, 0x003E4732, 0x00444745, 0x00464C41, 0x00464A44, 0x004C5247, 0x0045562A, 0x00455331,
        0x004C5541, 0x00525650, 0x00494D48, 0x00616958, 0x00646F55, 0x00677357, 0x0049533C, 0x00525853, 0x0067715A, 0x00586839, 0x00465823, 0x00374418, 0x003F4526, 0x00464930, 0x004B4D37, 0x00444531, 0x00464733, 0x00484A34, 0x00494B35, 0x00474933,
        0x00464832, 0x00454731, 0x0045482F, 0x00464530, 0x004B4A35, 0x00494A36, 0x00474A34, 0x004D5439, 0x006A7554, 0x006C7A51, 0x006B7C50, 0x00707E54, 0x006F7E52, 0x006E7D51, 0x00717F55, 0x00687353, 0xFFE8EEDB, 0xFFFFFFFC, 0xFFFFFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x4593C1EA, 0x005D9FE0, 0x00539FF1, 0x0059A2EC, 0x0061A6EC, 0x005AA5F3, 0x0059A5F7, 0x00529BE5, 0x00579AE5, 0x005C9FEA, 0x00599DE8, 0x005CA0EB, 0x005B9FEA,
        0x00559CE6, 0x0059A0EA, 0x00569DE7, 0x005C9DE2, 0x005A9CE3, 0x005B9FE8, 0x00589DE6, 0x00599EE4, 0x0061A0E4, 0x00427FBD, 0x000D4683, 0x00154674, 0x00163B75, 0x00426FA2, 0x005792C4, 0x004F7DB3, 0x003D5A63, 0x00415636, 0x003F4E39, 0x005E6054,
        0x007D7E6E, 0x0062614D, 0x006B6B53, 0x005A5A42, 0x003A3D27, 0x00474E39, 0x0038412D, 0x003A3924, 0x002A2915, 0x0042402E, 0x00827F70, 0x0089857A, 0x007E7970, 0x007E7871, 0x00817B76, 0x00888277, 0x00857E75, 0x00858077, 0x008F8A81, 0x008B8880,
        0x00827F77, 0x00928F8A, 0x00B0ADA8, 0x0099968E, 0x00464733, 0x002E301A, 0x00444438, 0x00393B28, 0x003B471D, 0x00404F1C, 0x00647046, 0x006B7949, 0x006C7A4A, 0x006D7A4C, 0x00748153, 0x004B582C, 0x00576438, 0x004E5B2F, 0x007D895F, 0x00717F55,
        0x0067754B, 0x0046542A, 0x005B693F, 0x0057653B, 0x00718054, 0x00758556, 0x00718151, 0x00708050, 0x00728155, 0x0076845B, 0x0077835F, 0x00727E5A, 0x0075835A, 0x0077865A, 0x00798959, 0x0070854E, 0x0070844F, 0x0066774B, 0x00404C28, 0x004B543A,
        0x00555849, 0x008A8B82, 0x00B1B0AC, 0x00B1B5A2, 0x00B6BAA7, 0x00B5B7A4, 0x00A7A795, 0x00949280, 0x00817F6D, 0x00736F5D, 0x006B6453, 0x005C5853, 0x0059564E, 0x007F7C74, 0x00848379, 0x005E5E52, 0x00575A4B, 0x00535646, 0x00353828, 0x0041432D,
        0x00474834, 0x004C4C3A, 0x00676658, 0x0067655B, 0x005D5954, 0x00595451, 0x00686362, 0x0067655A, 0x00545247, 0x0066645A, 0x006F6D63, 0x005E5C51, 0x004C4C3A, 0x003E4128, 0x00464A2D, 0x00686D70, 0x00A9AFB4, 0x00A9B0B9, 0x00ADB8BC, 0x008B9791,
        0x0063735B, 0x006B7E53, 0x006C7F48, 0x00748260, 0x0063724C, 0x006A7F4E, 0x006A814D, 0x00617159, 0x009FA7AE, 0x009EAAAC, 0x006C8069, 0x00697650, 0x0068784F, 0x0065784D, 0x0065794F, 0x0062754E, 0x00404D2D, 0x00262D12, 0x003B3E28, 0x003A3C1F,
        0x003C3E20, 0x003C3F1F, 0x003A3E1B, 0x003C401D, 0x00414325, 0x0043442A, 0x0042422A, 0x0041482D, 0x0044472E, 0x0045452D, 0x0048462E, 0x0045462C, 0x003F4424, 0x0046542B, 0x005B6C41, 0x005D6B42, 0x005D6C40, 0x005B6B3C, 0x005C6C3D, 0x005B6940,
        0x00525C3E, 0x004E5443, 0x00565A4F, 0x00586157, 0x00676E67, 0x00525E3A, 0x00556234, 0x004B4E3F, 0x0054574E, 0x00535D4C, 0x006B7170, 0x005A634F, 0x00374620, 0x004B5440, 0x00505253, 0x0051554F, 0x006D706E, 0x005A5E58, 0x0046562D, 0x00424F2F,
        0x00505847, 0x00525651, 0x006A6D6B, 0x007A8172, 0x00535D46, 0x00606B51, 0x004B543F, 0x00585E59, 0x0068725B, 0x00576738, 0x00465823, 0x00364317, 0x003F4526, 0x00474A31, 0x004A4C36, 0x00474632, 0x00494834, 0x004A4934, 0x004A4A32, 0x0047472F,
        0x0044452B, 0x00424329, 0x00424427, 0x0043432B, 0x00494833, 0x00464832, 0x00454832, 0x004C5338, 0x006A7554, 0x006C7A51, 0x006A7B4F, 0x00717F55, 0x00718054, 0x006F7E52, 0x00717F55, 0x00677252, 0xFFE6ECD9, 0xFFFFFFFC, 0xFFFEFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x4593C1EA, 0x005D9FE0, 0x0051A0F1, 0x0059A2EC, 0x0060A6EC, 0x0059A4F2, 0x0056A5F6, 0x00529BE5, 0x00589BE6, 0x005C9FEA, 0x005A9EE9, 0x005DA1EC, 0x005CA0EB,
        0x00569DE7, 0x005AA1EB, 0x00589FE9, 0x0065A8ED, 0x005EA3E9, 0x005A9FE8, 0x00589DE6, 0x005A9FE5, 0x005E9FE3, 0x004483C0, 0x001B5793, 0x001B5186, 0x00174281, 0x0037699E, 0x00508FC2, 0x005282B6, 0x003E5B60, 0x0040522D, 0x0038452B, 0x00616155,
        0x00777767, 0x0064634F, 0x00626048, 0x0056563E, 0x0042442E, 0x00545944, 0x004A533F, 0x0033321D, 0x003B3A26, 0x00393725, 0x005C594A, 0x00827E73, 0x008A857C, 0x00868079, 0x0087817C, 0x00857F74, 0x00847D74, 0x007D786F, 0x00817C73, 0x00817E76,
        0x00838078, 0x0096938E, 0x00A8A5A0, 0x007A746D, 0x00474834, 0x0040422C, 0x0058564B, 0x003E402D, 0x00454F25, 0x00485724, 0x0069754B, 0x006C7C4D, 0x0069794A, 0x00677748, 0x006D7C50, 0x004F5E32, 0x0059673D, 0x004B592F, 0x00738157, 0x006E7D51,
        0x0069774D, 0x0058653F, 0x004F5C36, 0x0065724C, 0x006A784E, 0x00758556, 0x00697949, 0x0057683D, 0x005C6B45, 0x00596547, 0x00657056, 0x005F6A50, 0x00556143, 0x00586741, 0x00617247, 0x006D804F, 0x006B7D4E, 0x006E7F54, 0x00636F4B, 0x00626B50,
        0x0050563F, 0x00676A5A, 0x00878A7B, 0x007B8567, 0x008B9479, 0x00929B80, 0x00889177, 0x007A826B, 0x00747B66, 0x00717863, 0x006E7461, 0x00707566, 0x009CA192, 0x00BCC1B2, 0x00959889, 0x00555849, 0x00444738, 0x004C4C3E, 0x00505042, 0x00444432,
        0x00464636, 0x004B4B3B, 0x00656456, 0x0069675C, 0x00626056, 0x005C5951, 0x006B6762, 0x0069675C, 0x00555348, 0x0066645A, 0x006D6B61, 0x005B594E, 0x00474735, 0x00393C23, 0x00414528, 0x00696E6F, 0x00A5ABB0, 0x00ACB3BC, 0x00B2BCC3, 0x008B9693,
        0x005C6C55, 0x0065774E, 0x00758653, 0x0073815F, 0x006B7955, 0x006F8354, 0x005E7443, 0x00596853, 0x009CA3AC, 0x00A3AEB2, 0x00697D66, 0x00697650, 0x0067774E, 0x0064774C, 0x0065794F, 0x0061744D, 0x003F4C2C, 0x00262D12, 0x003D402A, 0x003B3D20,
        0x003B3D1F, 0x003C3F1F, 0x003B3F1C, 0x003B3F1C, 0x003C3E20, 0x003F4026, 0x0042422A, 0x0042492E, 0x00454930, 0x00484732, 0x00494833, 0x0046472D, 0x003F4526, 0x0047542E, 0x005C6D42, 0x005F6D44, 0x005D6B41, 0x005A6A3B, 0x0059693A, 0x0058663D,
        0x00535D3F, 0x00555B4A, 0x0061645B, 0x00B3BDB7, 0x00BDC3BE, 0x005F6D4B, 0x005A663C, 0x004D4F43, 0x00575B55, 0x009CA696, 0x00C9D0D3, 0x0068735F, 0x003D4E29, 0x004C5743, 0x004B4F50, 0x00767D76, 0x00D1D6D4, 0x008E958E, 0x0036481F, 0x00505C3E,
        0x004C5344, 0x00636664, 0x00C3C6C4, 0x00CCD3C6, 0x00535C47, 0x00505A43, 0x005A634F, 0x005E645F, 0x0069735C, 0x00586839, 0x00475924, 0x00364317, 0x003E4425, 0x00474A31, 0x004A4C36, 0x00494834, 0x004A4935, 0x004A4934, 0x00484830, 0x0045462C,
        0x00424427, 0x00404224, 0x00404323, 0x0042422A, 0x00474631, 0x00454731, 0x00454930, 0x004B5335, 0x00697551, 0x006C7A50, 0x006A7C4D, 0x00717F55, 0x00718054, 0x00718054, 0x00717F55, 0x00657050, 0xFFE3E9D6, 0xFFFEFFFB, 0xFFFEFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x4593C1EA, 0x005A9EDF, 0x0051A0F1, 0x0057A2EC, 0x0060A6EC, 0x0057A5F2, 0x0056A5F6, 0x004F9AE4, 0x00599CE7, 0x005DA0EB, 0x005A9EE9, 0x005DA1EC, 0x005CA0EB,
        0x00569DE7, 0x0059A0EA, 0x00579EE8, 0x0061A4E9, 0x005BA0E6, 0x00569BE4, 0x00569BE4, 0x005CA1E7, 0x0062A3E7, 0x005B9AD7, 0x004C88C4, 0x004888C2, 0x004C80C3, 0x00538CC4, 0x005092C7, 0x00497CAE, 0x003B595A, 0x0044572C, 0x00374221, 0x00323226,
        0x00454535, 0x00403F2B, 0x0035331B, 0x003C3A22, 0x00393B25, 0x004F543F, 0x005B6450, 0x00696651, 0x00807C69, 0x00787462, 0x00656051, 0x007A7469, 0x00908980, 0x008A827B, 0x008D8480, 0x00888277, 0x008E877E, 0x00878279, 0x00878279, 0x0088857D,
        0x00908D85, 0x009A9792, 0x00928F8A, 0x00655F58, 0x00686753, 0x0073755F, 0x00747267, 0x003F3F2D, 0x004D572D, 0x0055612F, 0x006B754B, 0x00647347, 0x006D7B51, 0x00748258, 0x006D7B51, 0x004F5D34, 0x0057653C, 0x00505D37, 0x0077845E, 0x00748258,
        0x0067754C, 0x00485430, 0x00566141, 0x00576241, 0x007B8862, 0x00728155, 0x00617141, 0x004B5937, 0x00505B41, 0x0048503F, 0x00676D62, 0x00646A5F, 0x0048503F, 0x00444F35, 0x004C5A38, 0x00707F59, 0x00677650, 0x006D7C56, 0x00727E5A, 0x00626D4C,
        0x004F5738, 0x0051593A, 0x00596142, 0x004E5F34, 0x00506037, 0x004A5C35, 0x003D4E2C, 0x00324425, 0x00304328, 0x0032472E, 0x00344831, 0x002B3E25, 0x0036462E, 0x002D3A24, 0x00202B17, 0x00404635, 0x005B5E4F, 0x00535347, 0x004E4C41, 0x00504E43,
        0x00504E43, 0x00555348, 0x006A685D, 0x0069675C, 0x00615F54, 0x0059574C, 0x0069675C, 0x006E6C61, 0x005B594E, 0x006D6B61, 0x00747268, 0x00615F54, 0x004D4D3B, 0x003D4027, 0x0045492C, 0x00686D6E, 0x00A6ACB1, 0x00AEB5BE, 0x00B5BEC7, 0x009AA4A4,
        0x00667363, 0x005B6A4A, 0x00707F53, 0x00697755, 0x006E7C58, 0x0063764B, 0x004A5E34, 0x006A7769, 0x00A4A9B8, 0x00A1ACB0, 0x00697E65, 0x00697650, 0x0066764D, 0x0063764B, 0x0064784E, 0x0061744D, 0x003F4C2C, 0x00252C11, 0x003D402A, 0x003D3F22,
        0x003A3C1E, 0x003A3D1D, 0x003B3F1C, 0x00393D1A, 0x0037391B, 0x003D3E24, 0x0046462E, 0x0040462D, 0x00454930, 0x00484A34, 0x004A4934, 0x0044472E, 0x003E4425, 0x0046532D, 0x005A6D42, 0x005D6B42, 0x005D6B41, 0x005A693D, 0x0059683C, 0x0058653F,
        0x00545D42, 0x00585E4D, 0x00666960, 0x00E1EAE7, 0x00D8DFDC, 0x00647151, 0x00546239, 0x004C5045, 0x00636762, 0x00AEB8AB, 0x00EDF3F8, 0x006F7A66, 0x00556740, 0x005E6955, 0x005A5F5E, 0x008D958B, 0x00F1F7F2, 0x00A5ADA3, 0x0046592E, 0x00626E52,
        0x00565D4E, 0x006F7171, 0x00D4D6D6, 0x00DAE0D5, 0x0057604B, 0x00525B46, 0x006A735F, 0x00636964, 0x006B755E, 0x00586839, 0x00485A25, 0x00364317, 0x003E4425, 0x00474A31, 0x00494B35, 0x004C4835, 0x004C4835, 0x004B4833, 0x0049482E, 0x00464529,
        0x00434424, 0x00424321, 0x00424321, 0x0043432B, 0x00484830, 0x00464832, 0x00464A31, 0x004C5436, 0x006A7652, 0x006D7B51, 0x006C7E4F, 0x006F7D53, 0x00707F53, 0x00718054, 0x00707E54, 0x00636E4E, 0xFFE1E7D4, 0xFFFDFEFA, 0xFFFFFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x4593C1EA, 0x005A9EDF, 0x0051A0F1, 0x0056A1EB, 0x0060A6EC, 0x0057A5F2, 0x0056A5F6, 0x004F9AE4, 0x005A9DE8, 0x005EA1EC, 0x005A9EE9, 0x005DA1EC, 0x005B9FEA,
        0x00549BE5, 0x00579EE8, 0x00559CE6, 0x005C9FE4, 0x005CA1E7, 0x005BA0E9, 0x0058A0E8, 0x00599FE5, 0x005B9CE0, 0x005E9DDA, 0x0064A0DC, 0x004F96D5, 0x0065A0E6, 0x005897D1, 0x003379AE, 0x00255A8C, 0x002E4B48, 0x004A5C2D, 0x003A451F, 0x00535347,
        0x005F5F4F, 0x006B6754, 0x005F5D45, 0x0058563E, 0x00343620, 0x003C412C, 0x006A705D, 0x006F6C57, 0x00676350, 0x00969280, 0x008E897A, 0x00696358, 0x00807970, 0x00958D86, 0x00928985, 0x00938D82, 0x00A09990, 0x009E9990, 0x009C978E, 0x0099968E,
        0x009D9A92, 0x0097948F, 0x0076736E, 0x0069635C, 0x008F8E7A, 0x00A3A28D, 0x00848277, 0x00363624, 0x004D572D, 0x005E6A38, 0x006F794F, 0x006E7C52, 0x006F7D54, 0x006A784F, 0x0056643B, 0x00424F29, 0x00515E38, 0x00495531, 0x00636F4B, 0x006F7D53,
        0x004C5933, 0x00566141, 0x00566042, 0x00555F41, 0x00626E4A, 0x0077855B, 0x005B6B3C, 0x004F5A40, 0x005B6450, 0x005C6158, 0x00919492, 0x00919492, 0x0061665D, 0x004C5541, 0x0049543A, 0x006C7959, 0x00667452, 0x00707E5C, 0x006E7A56, 0x0046532D,
        0x003D4820, 0x003E4A20, 0x00344016, 0x00384C17, 0x002E4410, 0x0022390C, 0x001E360C, 0x001C3612, 0x00173315, 0x00112E14, 0x000B2910, 0x0015300E, 0x00183010, 0x000E2407, 0x0017250D, 0x00404935, 0x00555849, 0x004B493E, 0x00514C43, 0x004B4840,
        0x004E4B43, 0x00555349, 0x0069675C, 0x00676658, 0x00605F51, 0x00595949, 0x006D6D5B, 0x006A685D, 0x0058564B, 0x006B695F, 0x00747268, 0x00626055, 0x004D4D3B, 0x003E4128, 0x0044482B, 0x00696F6E, 0x00A6ADB0, 0x00A8AFB8, 0x00B3BCC6, 0x00B1BABD,
        0x007D897D, 0x004B593D, 0x004C5933, 0x005A6844, 0x00505E3C, 0x00394B24, 0x00425331, 0x0096A199, 0x00B3B7C9, 0x0097A1A8, 0x006E836A, 0x00697650, 0x0066764D, 0x0062754A, 0x0064784E, 0x0061744D, 0x003E4B2B, 0x00252C11, 0x003E412B, 0x003E4023,
        0x00393B1D, 0x00383B1B, 0x003B3F1C, 0x00383C19, 0x00343618, 0x003D3E24, 0x004A4A32, 0x003F452C, 0x00454832, 0x00484A34, 0x004A4934, 0x0043462D, 0x003C4223, 0x0045522C, 0x005A6C43, 0x005B6842, 0x005D6B41, 0x005C6B3F, 0x005D6C40, 0x005B6842,
        0x00555E43, 0x00575C4D, 0x0064675E, 0x00848D8A, 0x00818786, 0x004C583A, 0x00536138, 0x00565950, 0x00606361, 0x005F695D, 0x008D9398, 0x0055614B, 0x004C5E35, 0x005E6A54, 0x005B605E, 0x005B6358, 0x00868D86, 0x005F675D, 0x0053673A, 0x00596549,
        0x005E6558, 0x005C5E5E, 0x00787A7B, 0x007E8479, 0x00424B37, 0x0057604B, 0x005F6756, 0x00666C67, 0x006C765F, 0x0059693A, 0x00485A25, 0x00364317, 0x003E4425, 0x00474A31, 0x00484A34, 0x004B4734, 0x004B4734, 0x004B4833, 0x0048472D, 0x00454527,
        0x00434424, 0x00434422, 0x00444622, 0x0044442C, 0x004A4A32, 0x00474A31, 0x00474B32, 0x004E5638, 0x006C7854, 0x006E7C52, 0x006D7F50, 0x006C7A50, 0x006F7E52, 0x00707F53, 0x00707E54, 0x00626D4D, 0xFFE0E6D3, 0xFFFEFFFB, 0xFFFFFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x4594C2EB, 0x005DA1E2, 0x0053A2F3, 0x005AA3ED, 0x0061A6EC, 0x005AA4F2, 0x005BA5F7, 0x00589CE7, 0x005A9EE9, 0x005B9FEA, 0x005DA1EC, 0x005CA0EB, 0x005B9FEA,
        0x005A9EE9, 0x005A9EE9, 0x005A9EE9, 0x0062A1EB, 0x005A9EE9, 0x00559EE8, 0x004F9BE3, 0x005AA2E8, 0x0067A4E6, 0x005587C3, 0x00406BA4, 0x003D6B94, 0x00466F8F, 0x00406272, 0x00344E4E, 0x00172C1D, 0x001A2D14, 0x00556748, 0x003F5330, 0x00616363,
        0x008A8985, 0x00949288, 0x008D8A7B, 0x006A6653, 0x003E3D29, 0x003A3A28, 0x00727461, 0x0087857D, 0x007B7A70, 0x00707064, 0x003F3E30, 0x00383826, 0x00605F4A, 0x007E7E66, 0x0082826A, 0x00807D79, 0x009D9A96, 0x00BFBBBA, 0x00ACA9A5, 0x009B9991,
        0x00858676, 0x0061644E, 0x003C4126, 0x00625F5A, 0x007E7D73, 0x00B1B1A3, 0x00464B36, 0x0042492E, 0x00656E4D, 0x00606D47, 0x006C7A51, 0x006B7C51, 0x006C7D51, 0x006E7E55, 0x00303C20, 0x00383F32, 0x00585E53, 0x004A543D, 0x0054623E, 0x00687652,
        0x00444E37, 0x007E8479, 0x006B7265, 0x00535E44, 0x005B6A44, 0x006D7D54, 0x00677551, 0x00516D6E, 0x00678A98, 0x005B7D94, 0x00638095, 0x00557892, 0x00527B94, 0x005E7D7C, 0x00586342, 0x0067784C, 0x006D7E53, 0x006B7A54, 0x004B5937, 0x005C684A,
        0x00535E44, 0x004C563F, 0x00313A25, 0x00474B32, 0x004E5338, 0x004E5338, 0x00505538, 0x00505637, 0x00454B2C, 0x003E4524, 0x003E4524, 0x00494B3F, 0x00585A4E, 0x00434539, 0x00454539, 0x004E4D43, 0x00545248, 0x004F4D43, 0x00535147, 0x00484A3E,
        0x004B4D41, 0x0054554B, 0x0064655B, 0x0068695F, 0x00595A51, 0x0057584F, 0x00696A61, 0x00656456, 0x005A594B, 0x006B6B5B, 0x00757563, 0x00646551, 0x004D4F39, 0x0042442E, 0x00464930, 0x006F7374, 0x00A9ACB0, 0x00B0B3BB, 0x00B7B9C3, 0x00B6BBC4,
        0x00B0B6BD, 0x00868F92, 0x004E5957, 0x0037453F, 0x0036453D, 0x005A6961, 0x00919D9D, 0x00A6AFB9, 0x00A8B1BB, 0x0099A7A3, 0x006D806D, 0x00687A51, 0x006A7C53, 0x00667851, 0x00697755, 0x00697656, 0x00424C2E, 0x00212A0F, 0x00394228, 0x003C441C,
        0x003B4020, 0x00353920, 0x00393C2C, 0x00393C33, 0x00757672, 0x00BDC0BE, 0x00C2C7C5, 0x0062654F, 0x003D3F29, 0x0042422A, 0x0048472B, 0x0048482A, 0x003C4121, 0x004A5430, 0x005E6D47, 0x005D694B, 0x00667452, 0x005B6B42, 0x005B6C41, 0x00576640,
        0x005B664C, 0x0061675C, 0x00535654, 0x00353B36, 0x00414C38, 0x00566742, 0x004D5F36, 0x00536242, 0x00626E58, 0x00414D37, 0x003F4C34, 0x004B5D36, 0x004F5C42, 0x004B5348, 0x00737B70, 0x0049553F, 0x00364525, 0x004B5B39, 0x00556445, 0x003E4F2A,
        0x006B7564, 0x00666B69, 0x0040493C, 0x0043542F, 0x00576B3E, 0x00455435, 0x005D655B, 0x007A7E7F, 0x00656D5C, 0x005F6D44, 0x00455525, 0x003D4822, 0x00454930, 0x004F4F3D, 0x00484838, 0x004E4C34, 0x0048462E, 0x0054513C, 0x003D3C28, 0x0043422D,
        0x00414228, 0x00393E1E, 0x003A401D, 0x003D4526, 0x00474F31, 0x004A5136, 0x004F583E, 0x004F583E, 0x00657155, 0x006F7C5C, 0x006A7854, 0x00707E5A, 0x00707F59, 0x0073835A, 0x0073825C, 0x00606C4E, 0xFFDEE6D5, 0xFFFDFFFE, 0xFFFDFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x4594C2EB, 0x005DA1E2, 0x0054A3F4, 0x005AA3ED, 0x0061A6EC, 0x005AA4F2, 0x005BA5F7, 0x00589CE7, 0x005A9EE9, 0x005CA0EB, 0x005DA1EC, 0x005CA0EB, 0x005B9FEA,
        0x005A9EE9, 0x005A9EE9, 0x005B9FEA, 0x005F9EE8, 0x00589CE7, 0x0057A0EA, 0x0054A0E8, 0x005BA3E9, 0x00609DDF, 0x00467AB6, 0x002F5C95, 0x00687284, 0x005B6471, 0x00444B4E, 0x00393D37, 0x002B2E1F, 0x00313522, 0x00616750, 0x00444C35, 0x003E423C,
        0x006C7065, 0x00929383, 0x007F7E6A, 0x00505038, 0x006B6B53, 0x007F826C, 0x00676C57, 0x005D5D51, 0x00818173, 0x00646456, 0x002D2D1D, 0x00323222, 0x00363624, 0x004A4836, 0x0062614D, 0x00646652, 0x00838572, 0x00939385, 0x00757569, 0x0067675B,
        0x004D4D3F, 0x00323421, 0x002C2E1A, 0x0054524A, 0x0098988C, 0x00656656, 0x002F341F, 0x006C7358, 0x0065704F, 0x006D7A54, 0x00738158, 0x00728358, 0x006F8054, 0x006A7A51, 0x00465236, 0x00697063, 0x0081877C, 0x005F6952, 0x0053613D, 0x006B7955,
        0x004F5942, 0x00979D92, 0x0092998C, 0x0059644A, 0x005B6A44, 0x0076865D, 0x005E6C48, 0x00577677, 0x00779BAB, 0x006F94AA, 0x007494AB, 0x006D92AC, 0x00719DB5, 0x00709291, 0x00586644, 0x00728357, 0x006D7E53, 0x00677551, 0x00434F31, 0x00545E47,
        0x004A5142, 0x00757B70, 0x0081867D, 0x0070725E, 0x00747662, 0x006F715D, 0x006B6D59, 0x006A6C58, 0x00636551, 0x00585A46, 0x0052553F, 0x00555848, 0x00707364, 0x00757869, 0x00767668, 0x005E5E52, 0x00555348, 0x00514F45, 0x004F4D43, 0x00525244,
        0x004C4C3E, 0x00545448, 0x006A6A5E, 0x00717066, 0x00605F55, 0x005E5C54, 0x0074726A, 0x006E6C61, 0x00565449, 0x00525143, 0x00565646, 0x0051513F, 0x00454632, 0x003D3F29, 0x00454731, 0x006A6E6F, 0x00AAADB1, 0x00B4B7BF, 0x00B3B5BF, 0x00AEB3BC,
        0x00B4BAC1, 0x00B3BCBF, 0x00ABB5B5, 0x00A9B6B4, 0x00A5B3AD, 0x00AAB8B2, 0x00B1BCC0, 0x00AEB6C3, 0x00B1B9C6, 0x009DAAA8, 0x00647665, 0x00687A51, 0x006A7C53, 0x00667851, 0x00687654, 0x00687555, 0x00424C2E, 0x00212A0F, 0x003A4329, 0x00394119,
        0x003A3F1F, 0x003C4027, 0x00393C2D, 0x00A5A89F, 0x00FFFFFC, 0x00FDFFFC, 0x00C3C8C6, 0x00585B45, 0x003F412B, 0x0044442C, 0x0046452B, 0x00464628, 0x003B4020, 0x0048522E, 0x005F6E48, 0x005B6947, 0x0064734D, 0x005F7045, 0x00637449, 0x00586642,
        0x00555F48, 0x0071766D, 0x005F6161, 0x002F362F, 0x003B4534, 0x00505E42, 0x004A583C, 0x006A7463, 0x006B7369, 0x00353D32, 0x00374130, 0x00495A35, 0x00515C48, 0x00585E59, 0x00757A79, 0x00576053, 0x0035422A, 0x00445236, 0x0057644A, 0x004C5645,
        0x00737876, 0x0066696D, 0x00363E34, 0x00394828, 0x0061734C, 0x005E6A54, 0x006F7473, 0x0085898A, 0x00555D4C, 0x0057653C, 0x00495929, 0x003B4620, 0x003F432A, 0x004B4B39, 0x00484838, 0x0046472D, 0x0052523A, 0x00454632, 0x00474735, 0x006A6A58,
        0x00383A26, 0x00252910, 0x00282D12, 0x00363C25, 0x00454A35, 0x00464B36, 0x00484F3A, 0x004E563F, 0x006C785C, 0x00758361, 0x006D7C56, 0x00707E5A, 0x00707F59, 0x0073835A, 0x0073825C, 0x00606C4E, 0xFFDEE6D5, 0xFFFCFFFD, 0xFFFDFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x4594C2EB, 0x005CA0E1, 0x0054A3F4, 0x005BA4EE, 0x0062A7ED, 0x005BA5F3, 0x005CA6F8, 0x00589CE7, 0x005B9FEA, 0x005CA0EB, 0x005DA1EC, 0x005DA1EC, 0x005B9FEA,
        0x005A9EE9, 0x005A9EE9, 0x005B9FEA, 0x005FA0EA, 0x005A9EE9, 0x0059A2EC, 0x0056A0E8, 0x005BA1E7, 0x005F9FE0, 0x00528AC5, 0x004A7BB3, 0x005282AC, 0x004D7B9D, 0x00466F86, 0x0046656E, 0x003B5349, 0x003B4B33, 0x00676F4A, 0x00555C2F, 0x00293323,
        0x00404935, 0x0051553C, 0x0043472A, 0x00555739, 0x00ADB293, 0x00BCC1A4, 0x006C755A, 0x004D523D, 0x00505540, 0x008C8E7B, 0x00757666, 0x00403F31, 0x00333224, 0x00363429, 0x00383429, 0x00414728, 0x00383D22, 0x00555842, 0x00838474, 0x008A8A7E,
        0x00515145, 0x003A3A2C, 0x005F6050, 0x00A9A99D, 0x00727264, 0x003A3E2B, 0x004D533A, 0x006E785A, 0x00697453, 0x00798660, 0x006C7C53, 0x0067784D, 0x00708155, 0x006D7D54, 0x004A563A, 0x00666D60, 0x006E7469, 0x0058624B, 0x005D6B47, 0x00697753,
        0x004D5740, 0x007B8176, 0x007B8275, 0x0049543A, 0x00596842, 0x0074845B, 0x005A6844, 0x004F7275, 0x00719AA9, 0x006F98AF, 0x007499AF, 0x006F97B3, 0x00709FBA, 0x00668C8C, 0x00465938, 0x0067784D, 0x006B7B52, 0x006F7D5B, 0x004B573B, 0x004D5642,
        0x00444A3F, 0x00858983, 0x00A7AAA8, 0x00B4B9A4, 0x00B1B5A2, 0x00A0A491, 0x00878A7A, 0x00797C6D, 0x006E7162, 0x00616357, 0x005B5D51, 0x00505742, 0x00727663, 0x008C907D, 0x008B8E7E, 0x00606052, 0x00535345, 0x00535146, 0x0049473C, 0x004C4D3D,
        0x004B4C3C, 0x00535444, 0x00626254, 0x00636355, 0x0057574B, 0x005A5A4E, 0x006F6F63, 0x006D6B61, 0x0058564C, 0x004D4B40, 0x004E4D3F, 0x00535343, 0x004C4C3A, 0x00424230, 0x004C4D39, 0x004B504F, 0x00989C9D, 0x00B6BABF, 0x00BABDC5, 0x00B4B9C2,
        0x00AEB4BB, 0x00AEB7BA, 0x00BAC4C4, 0x00B0BBBF, 0x00B1BDBD, 0x00AFBBBB, 0x00A9B2BB, 0x00A4ABBC, 0x00B0B7C6, 0x00A0ACAC, 0x00657668, 0x00697B52, 0x006A7C53, 0x00667851, 0x00687654, 0x00687555, 0x00424C2E, 0x00222B10, 0x003A4329, 0x00404722,
        0x003B4021, 0x0050533D, 0x00333627, 0x00888B82, 0x00BDBFB9, 0x00B9BDB8, 0x00565C57, 0x00464834, 0x0042442E, 0x0047472F, 0x0046452B, 0x00464628, 0x003A3F1F, 0x0046502C, 0x00606E4A, 0x005E6E45, 0x00607146, 0x005A6B3F, 0x00607047, 0x005C6949,
        0x006B7460, 0x00B5B9B3, 0x00939596, 0x00667060, 0x004C5842, 0x0057644C, 0x005C6656, 0x00A1A6A5, 0x00A3A6AA, 0x00606563, 0x00525B4E, 0x0052633E, 0x00535E4A, 0x007E8283, 0x00A9ACB1, 0x00828883, 0x00505B47, 0x00505D43, 0x00576549, 0x005C6061,
        0x00B0B1BB, 0x009EA0A8, 0x004D554A, 0x00445333, 0x00657553, 0x006D7766, 0x009B9EA2, 0x00B9BDBE, 0x00545D49, 0x004F5D34, 0x004C5C2C, 0x0038431B, 0x003C4027, 0x00474735, 0x00484836, 0x004D5237, 0x0050533D, 0x00383C29, 0x005D6051, 0x0092968A,
        0x00363A2E, 0x002F3425, 0x00323A29, 0x0055564C, 0x00505147, 0x0043443A, 0x00494E3F, 0x00565D48, 0x006B775B, 0x006B7955, 0x006C7D52, 0x006F7D59, 0x006F7E58, 0x0073835A, 0x0072815B, 0x005F6B4D, 0xFFDDE5D4, 0xFFFCFFFD, 0xFFFCFCFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x4593C1EA, 0x005CA0E1, 0x0054A3F4, 0x005BA4EE, 0x0063A8EE, 0x005BA5F3, 0x005CA6F8, 0x00589CE7, 0x005B9FEA, 0x005CA0EB, 0x005EA2ED, 0x005DA1EC, 0x005CA0EB,
        0x005B9FEA, 0x005B9FEA, 0x005CA0EB, 0x005FA3EC, 0x005B9FEA, 0x005BA2EC, 0x00569EE6, 0x00599EE4, 0x0063A3E4, 0x00639FDA, 0x0069A1D8, 0x00499EE2, 0x004C9CDB, 0x004C97CB, 0x004585A7, 0x0031636F, 0x00294B40, 0x00556A49, 0x005B6A3E, 0x004C5B40,
        0x00333F23, 0x00444D2C, 0x00464E29, 0x003A411C, 0x007C845F, 0x0098A180, 0x00505B3B, 0x00586042, 0x0040482A, 0x004C5239, 0x00888A76, 0x00959686, 0x00615F54, 0x00535048, 0x003D3934, 0x00393C26, 0x0030321F, 0x00525244, 0x00818076, 0x008B8981,
        0x00828177, 0x0096968A, 0x00B8B8AA, 0x005F6350, 0x00373C27, 0x00454B34, 0x005F684D, 0x006D7858, 0x007C8864, 0x00708057, 0x006E7F54, 0x006C7D52, 0x00728357, 0x006F7F56, 0x00445034, 0x00555C4F, 0x0051574C, 0x00434D36, 0x00596743, 0x00667450,
        0x00454F38, 0x0052584D, 0x004F5649, 0x003F4A30, 0x0065744E, 0x00718158, 0x0065734F, 0x005C8286, 0x0075A1B2, 0x00719DB5, 0x00789EB6, 0x006E9AB7, 0x006D9FBB, 0x006B9294, 0x005A6F4F, 0x006D7F56, 0x006C7E57, 0x00697A55, 0x004F5E3E, 0x004A583C,
        0x0058654D, 0x005F6A56, 0x005D6756, 0x00778065, 0x00828B70, 0x008A9379, 0x00848C75, 0x007F8671, 0x00787E6B, 0x006E7463, 0x006F7465, 0x00737C61, 0x0097A086, 0x00B2BAA3, 0x009FA48F, 0x00595D4A, 0x00464737, 0x004F4F41, 0x00535347, 0x00494B37,
        0x004C4E3A, 0x004E503D, 0x004E503D, 0x004E4F3F, 0x004F5040, 0x00535444, 0x0059594B, 0x0057554B, 0x00514F45, 0x004D4B40, 0x004E4D3F, 0x00555545, 0x004B4B3B, 0x0040402E, 0x004F4F3D, 0x003E443F, 0x00888D8C, 0x00AEB3B6, 0x00B5B8C0, 0x00B9BFC6,
        0x00B2B8BF, 0x00A8B1B5, 0x00B0B9BC, 0x00B1BAC3, 0x00AEB9BD, 0x00ACB7BB, 0x00ACB5BF, 0x00A7ADC0, 0x00A9B0C1, 0x0098A4A4, 0x00657668, 0x00677950, 0x006A7C53, 0x00667851, 0x00697755, 0x00697656, 0x00424C2E, 0x00212A0F, 0x00394228, 0x00414724,
        0x00373B1E, 0x0051543E, 0x0056594A, 0x0094978E, 0x00A8AAA4, 0x00A9ADA7, 0x00525952, 0x00323420, 0x0042432F, 0x004A4934, 0x0048472D, 0x0048472B, 0x003C4122, 0x00444D2C, 0x00606E4A, 0x00637546, 0x005F7142, 0x00596A3E, 0x005B6A44, 0x005F6B4D,
        0x00929A89, 0x00FDFFFC, 0x00BBBDBE, 0x00647151, 0x0052603E, 0x00616D51, 0x007C8376, 0x00F1F3F4, 0x00DDDEE2, 0x006A6F66, 0x00505944, 0x005D6E42, 0x005C674D, 0x00B6BAB5, 0x00FDFFFF, 0x00A3A89F, 0x00626D53, 0x004E5C3A, 0x0042502C, 0x006F6E77,
        0x00FBFAFF, 0x00D3D5D5, 0x0059624D, 0x0052613B, 0x0061704A, 0x006B745F, 0x00D2D5D3, 0x00FAFFFE, 0x00626B57, 0x004C5A30, 0x004B5B2B, 0x0037421A, 0x003F4429, 0x00484935, 0x00484836, 0x00424B30, 0x002D351E, 0x00393E2F, 0x00555A51, 0x005F635D,
        0x00414742, 0x00999F9A, 0x00777E77, 0x00878585, 0x00747271, 0x0052514D, 0x0045483F, 0x00535946, 0x006E7A5C, 0x006B7A54, 0x006D7E52, 0x006F7D59, 0x006F7E58, 0x0073835A, 0x0072815B, 0x005F6B4D, 0xFFDDE5D4, 0xFFFCFFFD, 0xFFFCFCFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x4192C0E9, 0x005CA0E1, 0x0054A3F4, 0x005CA5EF, 0x0064A9EF, 0x005CA6F4, 0x005DA7F9, 0x00599DE8, 0x005CA0EB, 0x005DA1EC, 0x005EA2ED, 0x005EA2ED, 0x005CA0EB,
        0x005B9FEA, 0x005B9FEA, 0x005CA0EB, 0x005CA1EA, 0x00599DE8, 0x005CA0EB, 0x005A9FE8, 0x005BA0E6, 0x005EA0E1, 0x005494CE, 0x00508EC4, 0x00568CC1, 0x00558EC2, 0x005A93C6, 0x005C90BE, 0x005682A1, 0x00486873, 0x005C7169, 0x00606C5A, 0x00495C3B,
        0x00586944, 0x005B6940, 0x00556137, 0x00596339, 0x00515C34, 0x0053603A, 0x0064724E, 0x0076835D, 0x006E7A56, 0x00586041, 0x003C4328, 0x00868874, 0x00BCBDAD, 0x00A2A095, 0x008D8B81, 0x0083807C, 0x00706C6B, 0x007B7776, 0x00AEAAA9, 0x00D6D3CE,
        0x00D1D0C6, 0x00919282, 0x00464834, 0x00363C25, 0x00494F36, 0x0050593E, 0x006C7757, 0x00778261, 0x0074835D, 0x00718158, 0x00738459, 0x0074855A, 0x006D7E52, 0x006A7A51, 0x00414D31, 0x004E5548, 0x00454B40, 0x00404A33, 0x0055633F, 0x00667450,
        0x00414B34, 0x00595F54, 0x004E5548, 0x00323D23, 0x005D6C46, 0x006E7E55, 0x00687652, 0x005C8287, 0x006F9AAD, 0x006994AF, 0x006C94AD, 0x006591B0, 0x006B9DB9, 0x006A9396, 0x005E7457, 0x006C7E57, 0x006F815A, 0x006F815A, 0x00687A53, 0x0051633A,
        0x007F9168, 0x005D6F46, 0x0055673E, 0x0073805A, 0x0076835D, 0x007A8662, 0x00747F5E, 0x00747F5E, 0x00758060, 0x00717B5D, 0x007C8668, 0x006B7957, 0x006D7858, 0x00677153, 0x005E674D, 0x004D533C, 0x00555946, 0x00515242, 0x00454537, 0x0052553C,
        0x004D5037, 0x00474933, 0x00444630, 0x00484935, 0x004E4F3B, 0x0050513D, 0x004D4D3B, 0x004C4A3F, 0x004E4C41, 0x004E4D3F, 0x004C4C3C, 0x00515141, 0x00484836, 0x003E3F2B, 0x0050513D, 0x00454D43, 0x00838984, 0x00AEB3B4, 0x00B0B4B9, 0x00B6BCC3,
        0x00B7BDC4, 0x00B0B9BD, 0x00B5BEC1, 0x00B2BBC5, 0x00ADB7BE, 0x00ABB6BA, 0x00ADB6C0, 0x00A9AFC2, 0x00A8AFBE, 0x0098A5A3, 0x006D7F6E, 0x0066784F, 0x00697B52, 0x00677952, 0x006B7957, 0x006A7757, 0x00434D2F, 0x0020290E, 0x00374026, 0x00414627,
        0x003D4026, 0x003F412D, 0x00434637, 0x00787C71, 0x0093968D, 0x005A5F56, 0x00151D12, 0x00232512, 0x0042432F, 0x004B4A35, 0x004B4931, 0x004B4A2E, 0x003F4425, 0x00444D2C, 0x00606E4A, 0x00617344, 0x00627347, 0x005F7045, 0x00576541, 0x00566147,
        0x00969E8D, 0x00FDFFF9, 0x00A0A3A1, 0x004F6132, 0x005A6C3D, 0x005B6A44, 0x006F7863, 0x00F8FCF6, 0x00D7DBD5, 0x0057604B, 0x0055633F, 0x005F723F, 0x005C6A48, 0x00AEB5A6, 0x00FDFFF8, 0x007B846F, 0x004D5B37, 0x00435429, 0x004D5E33, 0x006E716F,
        0x00FDFFFB, 0x00C8D0BF, 0x0052603E, 0x005F7142, 0x00647549, 0x00626E52, 0x00D3DACD, 0x00FBFFFF, 0x005F6853, 0x004B592F, 0x004C5D2A, 0x003A461C, 0x0044492E, 0x004A4B37, 0x00494A36, 0x0039412A, 0x004E5441, 0x00767D70, 0x003B3F39, 0x000E1311,
        0x00676C6A, 0x00FAFFFE, 0x00878E8B, 0x00858284, 0x00A9A6A8, 0x00A2A09F, 0x006C6F66, 0x00505645, 0x006A7658, 0x006F7F56, 0x00718354, 0x006F7D59, 0x006F7E58, 0x0073835A, 0x0072815B, 0x00606C4E, 0xFFDEE6D5, 0xFFFCFFFD, 0xFFFDFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x4192C0E9, 0x005B9FE0, 0x0054A3F4, 0x005DA6F0, 0x0064A9EF, 0x005DA7F5, 0x005DA7F9, 0x00599DE8, 0x005CA0EB, 0x005EA2ED, 0x005FA3EE, 0x005EA2ED, 0x005DA1EC,
        0x005CA0EB, 0x005CA0EB, 0x005DA1EC, 0x005FA4ED, 0x00599DE8, 0x005A9DE8, 0x005A9EE7, 0x0061A3EA, 0x005EA0E1, 0x003F84BD, 0x002A6EA3, 0x003773A1, 0x002B6B9B, 0x002A6C9D, 0x00316F98, 0x00477990, 0x004F7171, 0x006A7A62, 0x007D8059, 0x00718862,
        0x0074885E, 0x007D8E63, 0x00818E62, 0x00818E62, 0x0076845A, 0x0076855F, 0x00788964, 0x006E7C52, 0x00808E64, 0x007A8761, 0x006D7655, 0x00596143, 0x00575B42, 0x009A9C88, 0x00B5B7A4, 0x009B9894, 0x00A3A09C, 0x00A7A4A0, 0x00AFACA7, 0x00919185,
        0x005E604D, 0x003D4227, 0x003C4223, 0x00575F41, 0x00505A3C, 0x00495434, 0x00586342, 0x0052613B, 0x004C5C33, 0x0055683D, 0x004D6035, 0x004F6035, 0x00495A2E, 0x004E5E35, 0x00485438, 0x0062695C, 0x00565C51, 0x005D6750, 0x00566440, 0x006C7A56,
        0x0058624B, 0x00989E93, 0x00959C8F, 0x00586349, 0x0062714B, 0x0077875E, 0x0062704C, 0x00587C84, 0x006E99AE, 0x006E96B3, 0x007095B1, 0x006C95B6, 0x0071A3C1, 0x00648A8F, 0x004B5E43, 0x00687A51, 0x0071835A, 0x00798B62, 0x0074865D, 0x003F5128,
        0x00677950, 0x004A5C33, 0x0056693E, 0x00687652, 0x0066754F, 0x006A7953, 0x0065744E, 0x006F7F56, 0x0078885F, 0x006D7E53, 0x00708156, 0x006D7D54, 0x006F7E58, 0x006A7554, 0x005C6648, 0x00475036, 0x004F543F, 0x004B4F3C, 0x00505343, 0x004C5033,
        0x0045482E, 0x0045482E, 0x00484B31, 0x0042452C, 0x003E4128, 0x00484A34, 0x0053553F, 0x00505040, 0x00535343, 0x00535341, 0x0050513D, 0x00555741, 0x004D4F39, 0x0040432A, 0x004C4F35, 0x00424A3F, 0x007B827B, 0x00BAC0BF, 0x00BCC1C4, 0x00B1B7BE,
        0x00B0B6BD, 0x00ADB5BC, 0x00AFB8BC, 0x00A6AFB9, 0x00AAB5B9, 0x00A9B4B8, 0x00A7B0B9, 0x00A4ABBA, 0x00A7B0BA, 0x0095A39D, 0x00667964, 0x0066784F, 0x00697B52, 0x00677952, 0x006B7957, 0x006A7757, 0x00434D2F, 0x0020290E, 0x00374026, 0x00414528,
        0x00404329, 0x004C4E3A, 0x002A2D1E, 0x00303429, 0x007E8178, 0x00494F44, 0x00151E11, 0x00232512, 0x00454533, 0x004A4935, 0x004C4A32, 0x004B4A30, 0x00414528, 0x00464F2E, 0x00606E4C, 0x005E6F44, 0x0063734A, 0x005C6A46, 0x004D5A3A, 0x00404B31,
        0x006F7864, 0x00BCC3B4, 0x006F7669, 0x00677748, 0x00576737, 0x004E5C32, 0x0050593E, 0x00A2A798, 0x00A2A798, 0x00636D4F, 0x00606F43, 0x00455526, 0x004B5636, 0x00808673, 0x00C7CDBA, 0x006E785A, 0x004F5D34, 0x0039471E, 0x0045512D, 0x00595E4F,
        0x00B9BFAC, 0x008E977C, 0x0046532D, 0x0057663A, 0x005C6A41, 0x00535C41, 0x009AA08D, 0x00BCC1BF, 0x0049523D, 0x004E5D31, 0x0050612E, 0x003E4A20, 0x00474C2F, 0x004C4D39, 0x004C4D39, 0x00444835, 0x00787D6E, 0x00AAAEA3, 0x00939490, 0x00888B89,
        0x00878B86, 0x00A5ACA5, 0x00565D56, 0x00656362, 0x009B9998, 0x00AEADA9, 0x007E8277, 0x00555B48, 0x00657153, 0x006A7A51, 0x00708155, 0x00707E5A, 0x00707F59, 0x0073835A, 0x0073825C, 0x00606C4E, 0xFFDEE6D5, 0xFFFDFFFE, 0xFFFDFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x4191BFE8, 0x005B9FE0, 0x0054A3F4, 0x005DA6F0, 0x0065AAF0, 0x005DA7F5, 0x005DA7F9, 0x00599DE8, 0x005DA1EC, 0x005EA2ED, 0x005FA3EE, 0x005FA3EE, 0x005DA1EC,
        0x005CA0EB, 0x005CA0EB, 0x005DA1EC, 0x005FA7EF, 0x005B9FEA, 0x005B9EE9, 0x005A9BE5, 0x0062A2E9, 0x005DA1E2, 0x003D84BD, 0x00216A9E, 0x001C72A8, 0x001771A8, 0x001D77AE, 0x002277A5, 0x003D8099, 0x0049756F, 0x00617552, 0x00797F48, 0x006D855D,
        0x00798F65, 0x00778B5E, 0x00708051, 0x00808F63, 0x0078865D, 0x006D7E59, 0x00728361, 0x007A8A5B, 0x00718054, 0x00758359, 0x00738158, 0x007F8A64, 0x00737C5B, 0x00696F50, 0x00666C4D, 0x003A4021, 0x00393E21, 0x002D3217, 0x003C4027, 0x0044492E,
        0x004B5132, 0x004F5633, 0x00545D35, 0x004C5736, 0x00667150, 0x00717D59, 0x00586642, 0x0062714B, 0x0064764D, 0x005C6F44, 0x005D7045, 0x00607146, 0x005E6F43, 0x0057673E, 0x00727E62, 0x0091988B, 0x00757B70, 0x008B957E, 0x0055633F, 0x00677551,
        0x005F6952, 0x006E7469, 0x0062695C, 0x004F5A40, 0x0045542E, 0x005C6C43, 0x0052603C, 0x0050717A, 0x00648AA2, 0x00658BAB, 0x006A8CAA, 0x00648AAC, 0x006491B2, 0x0052767C, 0x00404F34, 0x005D7245, 0x00596D42, 0x00586B44, 0x00526342, 0x00314024,
        0x004D5A44, 0x0045513F, 0x00424D3D, 0x004C5941, 0x0048553B, 0x00465438, 0x003A4929, 0x004F603B, 0x0071835A, 0x0073865B, 0x00778B5E, 0x00728357, 0x006B7C51, 0x00677650, 0x00616C4B, 0x004C553A, 0x0050563F, 0x00474B38, 0x00484C39, 0x00444628,
        0x00424426, 0x004A4C2F, 0x004D4F32, 0x00404127, 0x0037381E, 0x0044452B, 0x0054553B, 0x004C4D39, 0x004D4F39, 0x004E503A, 0x004B4E35, 0x004D5036, 0x00474B2E, 0x003C4122, 0x00424728, 0x00434C3F, 0x006B7369, 0x00B7BEBB, 0x00BDC2C5, 0x00A5ABB2,
        0x00A2A8AF, 0x00A7AFB6, 0x00A9B2B6, 0x00A1AAB3, 0x00ACB8BA, 0x00AFBBBB, 0x00AFB9C0, 0x00AEB6C3, 0x00B0BAC1, 0x009CACA1, 0x006C8266, 0x00687A51, 0x006A7C53, 0x00667851, 0x00697755, 0x00687555, 0x00424C2E, 0x00212A0F, 0x00394228, 0x0043472A,
        0x0033361D, 0x0040422F, 0x002D2F23, 0x001B1F14, 0x00676B60, 0x00545B4E, 0x00182114, 0x002E2F1F, 0x004B4B39, 0x00474632, 0x00494631, 0x0046452B, 0x00404427, 0x00485031, 0x0062704E, 0x0062704C, 0x0062704E, 0x004F5C3C, 0x00485438, 0x003E492F,
        0x004A543D, 0x00707A63, 0x005B654E, 0x005B6547, 0x0044502C, 0x004E5A36, 0x004E573D, 0x00575C4D, 0x006A6F60, 0x0060694E, 0x004D5B31, 0x00565F45, 0x00555B4A, 0x005E6256, 0x007A806F, 0x00616A4F, 0x00535E3E, 0x00475036, 0x00484E3D, 0x004E5441,
        0x006D755E, 0x00535C41, 0x003C4628, 0x00434C31, 0x004B533C, 0x00494F3E, 0x0065695D, 0x00767B79, 0x003C4530, 0x0057663A, 0x0051632E, 0x003F4C20, 0x00474C2F, 0x004D4F39, 0x004E4F3B, 0x00545446, 0x0057564C, 0x0067685F, 0x00B3B3AD, 0x00DCDED8,
        0x00666960, 0x004A4E42, 0x00818677, 0x0054554C, 0x0057584F, 0x005B5C52, 0x004F5445, 0x004F5641, 0x006F7B5D, 0x006E7D57, 0x006F8054, 0x00717F5B, 0x0071805A, 0x0074845B, 0x0074835D, 0x00616D4F, 0xFFDFE7D6, 0xFFFDFFFE, 0xFFFDFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x4191BFE8, 0x005B9FE0, 0x0054A3F4, 0x005DA6F0, 0x0065AAF0, 0x005EA8F6, 0x005EA8FA, 0x00599DE8, 0x005DA1EC, 0x005EA2ED, 0x005FA3EE, 0x005FA3EE, 0x005EA2ED,
        0x005CA0EB, 0x005DA1EC, 0x005DA1EC, 0x0059A1E9, 0x005CA0EB, 0x0060A3EE, 0x005D9CE6, 0x005E9EE5, 0x005B9FE0, 0x003D87BF, 0x002470A4, 0x003A76A0, 0x003375A6, 0x003079B3, 0x002872AC, 0x003878A8, 0x003C6C84, 0x004F6C69, 0x006B7967, 0x00708A62,
        0x00697F55, 0x00798D60, 0x007A8B5F, 0x00748357, 0x006F7F56, 0x00778863, 0x00697A59, 0x006E8051, 0x00748455, 0x00738256, 0x00738256, 0x0078845A, 0x00747F57, 0x00858E66, 0x00757E56, 0x0074834A, 0x0067753F, 0x005D6939, 0x006B754A, 0x005F693F,
        0x00677147, 0x00737D52, 0x006C764B, 0x00616D49, 0x005E6A46, 0x00586642, 0x0055643E, 0x005F6F46, 0x004A5D32, 0x0054673C, 0x0054673C, 0x004F6035, 0x005C6D41, 0x0047572E, 0x00626E52, 0x006A7164, 0x00464C41, 0x00949E87, 0x00677551, 0x00687652,
        0x00768069, 0x00464C41, 0x003E4538, 0x00808B71, 0x0061704A, 0x0055653C, 0x0052603C, 0x0052707B, 0x00587C94, 0x00547798, 0x005A7998, 0x00507498, 0x00517C9D, 0x0050717A, 0x0055624A, 0x005A7040, 0x00586C41, 0x00576846, 0x004C5A42, 0x00495346,
        0x00626867, 0x007F8388, 0x006E707B, 0x00788076, 0x00798176, 0x00727C6B, 0x0054604A, 0x005B6A4B, 0x007A8A68, 0x0074865D, 0x006F8257, 0x00738758, 0x006F8054, 0x0076865D, 0x006E7C58, 0x004E583A, 0x004A5339, 0x00454A35, 0x00484C39, 0x00494C2C,
        0x00454828, 0x00484A2C, 0x00494B2D, 0x0045472A, 0x0045472A, 0x00494A30, 0x0046472D, 0x004A4D34, 0x00494C33, 0x004A4D33, 0x0043472A, 0x00414627, 0x00404525, 0x003C4121, 0x00464C29, 0x00475141, 0x005B6358, 0x00A7AEA9, 0x00B2B7BA, 0x00969CA3,
        0x0090969D, 0x00939BA2, 0x008E969D, 0x00909AA1, 0x0096A2A4, 0x009AA7A5, 0x00A0ABAF, 0x00A2ABB4, 0x00A3AEB2, 0x0093A496, 0x006B8164, 0x006A7C53, 0x006B7D54, 0x00657750, 0x00677553, 0x00667353, 0x00414B2D, 0x00222B10, 0x003B442A, 0x004A4D33,
        0x00454731, 0x0040422F, 0x0064665A, 0x0071756A, 0x0074786D, 0x0060675A, 0x001E2818, 0x00383929, 0x004F4F3D, 0x00454430, 0x0046432E, 0x00414026, 0x003E4225, 0x004A5233, 0x00647250, 0x006B7858, 0x00636F51, 0x00475337, 0x004F5A40, 0x00505A43,
        0x003E492F, 0x00445034, 0x005C684A, 0x004D5243, 0x005E6550, 0x00677056, 0x00555B48, 0x0044483D, 0x0052564B, 0x005B624D, 0x00667150, 0x00666960, 0x005F605C, 0x005A5B59, 0x004D5146, 0x00555C47, 0x00525A43, 0x0065695E, 0x00606060, 0x00545A47,
        0x004E5540, 0x00424934, 0x00515746, 0x00595D51, 0x005F615B, 0x00555751, 0x0052534F, 0x00595E5C, 0x00404A33, 0x005F6E42, 0x004F612C, 0x003D4A1E, 0x00474C2F, 0x004E503A, 0x0050523C, 0x004E4C42, 0x00545149, 0x00514E49, 0x006F6C67, 0x00737169,
        0x002B2D21, 0x00393C2C, 0x005E634E, 0x00434637, 0x00454839, 0x004E5142, 0x00474D3C, 0x004A523B, 0x00717D5F, 0x0072815B, 0x00708156, 0x00717F5B, 0x0071805A, 0x0075855C, 0x0074835D, 0x00616D4F, 0xFFDFE7D6, 0xFFFDFFFE, 0xFFFDFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x3189BDF2, 0x00659FDA, 0x0061A7EA, 0x005EABF4, 0x0057A7F4, 0x0058A6F2, 0x0063AAF4, 0x00579CE2, 0x005C9FEA, 0x0062A5F0, 0x005FA2ED, 0x005DA0EB, 0x005EA1EC,
        0x005B9EE9, 0x005C9FEA, 0x0060A3EE, 0x005FA0F1, 0x005B9DEC, 0x005FA3EE, 0x005DA3E9, 0x005EA5E5, 0x005EA2DD, 0x004686BC, 0x003372A5, 0x002D7CA5, 0x00327EAF, 0x003379B4, 0x003D7CB6, 0x004C81A6, 0x004D7A7D, 0x00587B59, 0x00719158, 0x0074895C,
        0x00728858, 0x00798D5E, 0x007A8F5E, 0x00748A56, 0x00748853, 0x00768A55, 0x00728651, 0x0074855A, 0x00738459, 0x00728358, 0x00718257, 0x00718257, 0x00738459, 0x0074855A, 0x0075865B, 0x0076875C, 0x00738459, 0x0076875C, 0x0077885D, 0x00728358,
        0x0074855A, 0x00798A5F, 0x0077885D, 0x007D8C66, 0x0076855F, 0x007A8963, 0x007A8963, 0x0073825C, 0x0075845E, 0x00778660, 0x0072815B, 0x007A8963, 0x0074835D, 0x006A7953, 0x0061704A, 0x005D6C46, 0x005D6C46, 0x0061704A, 0x0064734D, 0x006D7D54,
        0x00526239, 0x004D5D34, 0x005C6C43, 0x006E7E55, 0x005E6E45, 0x0056663D, 0x0053633A, 0x005D6D5C, 0x00586857, 0x00485847, 0x00738372, 0x00586857, 0x00465645, 0x00879786, 0x00637362, 0x00576B3E, 0x00576A3F, 0x005E6F4A, 0x00566546, 0x005E6A54,
        0x00818B7B, 0x00848C82, 0x006F766F, 0x0081857F, 0x008E938A, 0x005E6556, 0x00838C77, 0x00778365, 0x0074825E, 0x0076865D, 0x00718256, 0x0077875E, 0x0078885F, 0x0077875E, 0x0073835A, 0x006D7D54, 0x0068784F, 0x0065754C, 0x0064744B, 0x0062744D,
        0x0061734C, 0x0060724B, 0x005F714A, 0x005E7049, 0x005E7049, 0x005E7049, 0x005E7049, 0x0061724D, 0x0061724D, 0x0061724D, 0x0060714C, 0x0060714C, 0x005F704B, 0x005F704B, 0x005F704B, 0x00606F53, 0x00667559, 0x00758468, 0x007E8D71, 0x007B8A6E,
        0x0079886C, 0x007A896D, 0x00758468, 0x00778863, 0x00768762, 0x00758661, 0x00758661, 0x00768762, 0x00758661, 0x0072835E, 0x0070815C, 0x006E8156, 0x006F8257, 0x00697B54, 0x006A7A58, 0x006A795A, 0x00414E34, 0x001F2C14, 0x0037432D, 0x00404B2B,
        0x00394228, 0x00757A6B, 0x00ABAEA5, 0x008C8F86, 0x005E6354, 0x004E573D, 0x003F4A2A, 0x0045482F, 0x0040432A, 0x003E4127, 0x0042452B, 0x00404427, 0x003D4124, 0x004F5336, 0x006B7051, 0x006E8059, 0x00617051, 0x003F4A36, 0x005E665B, 0x004C5449,
        0x0036402F, 0x003B482E, 0x00627151, 0x00565F4A, 0x00868F7A, 0x005C6550, 0x004B543F, 0x00353E29, 0x0057604B, 0x0067705B, 0x00B4BDA8, 0x00B8BDBB, 0x00697167, 0x003A4630, 0x00465236, 0x005A6648, 0x008C947D, 0x00999E8F, 0x004F514B, 0x004E593F,
        0x00303C20, 0x005E6851, 0x007D8477, 0x00B1B3B3, 0x00686C67, 0x0057604C, 0x0035431F, 0x00505E42, 0x004C5A38, 0x005C6A40, 0x00525E2E, 0x00444E20, 0x004F502E, 0x004D4931, 0x004F4638, 0x00454A2D, 0x004B5033, 0x004F5437, 0x004A4F32, 0x00404528,
        0x00393E21, 0x003A3F22, 0x003E4326, 0x004F493E, 0x00534D40, 0x004F4C3D, 0x00454733, 0x00555E44, 0x006F7B5D, 0x0071815F, 0x006F815C, 0x0072805C, 0x0073825C, 0x0074845B, 0x00798862, 0x00657153, 0xFFDDE5D4, 0xFFFDFFFE, 0xFFFDFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x2684B4EE, 0x00699FDE, 0x0065A4E8, 0x005BA2EC, 0x0062ADF9, 0x0062AAF7, 0x0066AAF3, 0x005E9EE5, 0x005EA5EF, 0x0061A8F2, 0x0061A8F2, 0x0061A8F2, 0x0062A9F3,
        0x005EA5EF, 0x005CA3ED, 0x005AA1EB, 0x0064A7EC, 0x0060A5EA, 0x0061A8EB, 0x0059A1E1, 0x005CA5DF, 0x005EA4D9, 0x004789BA, 0x0030719E, 0x004F7E82, 0x0053828A, 0x0050808C, 0x00517F8A, 0x005D8688, 0x00638174, 0x00687E62, 0x0072805C, 0x0071855B,
        0x006E8257, 0x0072855A, 0x0075885D, 0x0073865B, 0x0078895E, 0x007B8C61, 0x0077885D, 0x0074855A, 0x00738459, 0x00738459, 0x00728358, 0x00738459, 0x00738459, 0x0075865B, 0x0075865B, 0x0075865B, 0x00738459, 0x0077885D, 0x0078895E, 0x0074855A,
        0x0076875C, 0x007A8B60, 0x0078895E, 0x0072815B, 0x0072815B, 0x0074835D, 0x0073825C, 0x006F7E58, 0x006F7E58, 0x0071805A, 0x0073825C, 0x0073825C, 0x0072815B, 0x0071805A, 0x00707F59, 0x00707F59, 0x00707F59, 0x00707F59, 0x00707F59, 0x0075845E,
        0x0071805A, 0x0071805A, 0x006A7953, 0x006E7D57, 0x0066754F, 0x0063724C, 0x00596842, 0x007F8E79, 0x00384732, 0x001D2C17, 0x0035442F, 0x000E1D08, 0x003C4B36, 0x009DAC97, 0x00586752, 0x005D7045, 0x00596B42, 0x005A6B46, 0x00495838, 0x00455238,
        0x005C6852, 0x00606A59, 0x00515B4B, 0x00575E4F, 0x00717968, 0x005E6752, 0x0069745A, 0x00606D4D, 0x00707E5A, 0x0073835A, 0x00728358, 0x006F7F56, 0x00718158, 0x0073835A, 0x0074845B, 0x0073835A, 0x00728259, 0x00728259, 0x0073835A, 0x0072845D,
        0x0071835C, 0x0070825B, 0x006F815A, 0x006E8059, 0x006E8059, 0x006E8059, 0x006E8059, 0x0071825D, 0x0071825D, 0x0071825D, 0x0071825D, 0x0071825D, 0x0071825D, 0x0071825D, 0x0071825D, 0x006C7D5C, 0x006D7E5D, 0x00738463, 0x00778867, 0x00738463,
        0x00738463, 0x00768766, 0x00738463, 0x0073845F, 0x0072835E, 0x0071825D, 0x0071825D, 0x0071825D, 0x0071825D, 0x006F805B, 0x006D7E59, 0x006A7D52, 0x006E8156, 0x006B7D56, 0x006C7C5A, 0x006D7C5D, 0x00425034, 0x001F2C14, 0x0039462E, 0x0047492B,
        0x003E3E26, 0x00575445, 0x00716C63, 0x00615C53, 0x004C493A, 0x00484830, 0x00454729, 0x00474A31, 0x0044472D, 0x003F4228, 0x00404329, 0x00414528, 0x003E4225, 0x004F5435, 0x006E7354, 0x006F8158, 0x006D7D5B, 0x0055624A, 0x00828C7B, 0x00909A8A,
        0x0067735D, 0x004E5D3E, 0x0071825D, 0x00616C52, 0x00717C62, 0x005D684E, 0x00606B51, 0x004F5A40, 0x006F7A60, 0x0079846A, 0x0097A288, 0x00CAD1CA, 0x0098A292, 0x0055624A, 0x00667353, 0x006C7959, 0x00737C61, 0x009FA592, 0x00868A7E, 0x00677355,
        0x00485535, 0x00707C60, 0x00959D8C, 0x00EBEFE9, 0x00A7ACA3, 0x006B755E, 0x00475630, 0x005C6B4C, 0x004F5D3B, 0x005D6B41, 0x00525F31, 0x003F481D, 0x00484C29, 0x004E4C34, 0x00504938, 0x004C4D39, 0x004F503C, 0x0051523E, 0x004F503C, 0x004A4B37,
        0x00464733, 0x00464733, 0x00464733, 0x00514C3D, 0x004E4C3A, 0x00484935, 0x00454930, 0x00566042, 0x006D7B59, 0x006B7C57, 0x00687B54, 0x0073815D, 0x0073825C, 0x00728259, 0x00778660, 0x00657153, 0xFFDDE5D4, 0xFFFDFFFE, 0xFFFCFCFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x67A6C4ED, 0x5099BCE8, 0x0085B1E0, 0x006091C1, 0x006FA4D6, 0x0070A4D3, 0x006E9FCD, 0x006F9CC8, 0x00629DCB, 0x00609BC9, 0x00649FCD, 0x00649FCD, 0x0069A4D2,
        0x0069A4D2, 0x006AA5D3, 0x0069A4D2, 0x0073A1DB, 0x006E9DDA, 0x006EA0DC, 0x00679BD7, 0x006B9DD7, 0x006898CC, 0x005987B6, 0x004D7AA5, 0x00788767, 0x00768A67, 0x00728B69, 0x006A8563, 0x00758D6D, 0x007A8C6F, 0x0082886F, 0x007D7F69, 0x00778B61,
        0x0073875D, 0x0072845D, 0x0072845D, 0x006F805B, 0x0072805E, 0x0073815F, 0x00717E5E, 0x0074855A, 0x0074855A, 0x0074855A, 0x0074855A, 0x0074855A, 0x0075865B, 0x0075865B, 0x0075865B, 0x0074855A, 0x0074855A, 0x0077885D, 0x00798A5F, 0x0076875C,
        0x0077885D, 0x00798A5F, 0x0077885D, 0x0073825C, 0x0076855F, 0x0073825C, 0x0071805A, 0x0073825C, 0x0071805A, 0x0072815B, 0x0075845E, 0x00707F59, 0x0071805A, 0x0074835D, 0x0075845E, 0x0075845E, 0x0074835D, 0x0071805A, 0x006F7E58, 0x006A7854,
        0x0074825E, 0x007A8864, 0x006F7D59, 0x00707E5A, 0x0073815D, 0x0075835F, 0x006D7B57, 0x0059664C, 0x005A674D, 0x0049563C, 0x004A573D, 0x0049563C, 0x00566349, 0x00707D63, 0x005C694F, 0x0076885F, 0x0074865F, 0x00758760, 0x00657651, 0x005A6A48,
        0x00657454, 0x006A795A, 0x00637253, 0x005D694B, 0x00727E60, 0x00758262, 0x00677553, 0x005F6D49, 0x0072815B, 0x006F7E58, 0x006F7F56, 0x00708057, 0x00718158, 0x0073835A, 0x0074845B, 0x0074845B, 0x0075855C, 0x0075855C, 0x0076865D, 0x0071835A,
        0x0071835A, 0x00708259, 0x006F8158, 0x006E8057, 0x006E8057, 0x006F8158, 0x006F8158, 0x0070825B, 0x0070825B, 0x0071835C, 0x0071835C, 0x0072845D, 0x0072845D, 0x0073855E, 0x0073855E, 0x0073845F, 0x0070815C, 0x0071825D, 0x0071825D, 0x006E7F5A,
        0x0070815C, 0x0072835E, 0x0071825D, 0x0073855E, 0x0072845D, 0x0070825B, 0x0070825B, 0x0071835C, 0x0071835C, 0x0071835C, 0x0070825B, 0x006A7D52, 0x006F8257, 0x006C7E57, 0x006D7E59, 0x006F7E5E, 0x00425132, 0x001D2B0F, 0x0039462C, 0x00504E30,
        0x004C482F, 0x004C4534, 0x004E4639, 0x004F473A, 0x004D4635, 0x004B472E, 0x004C4A2C, 0x0041462B, 0x00494E33, 0x00474C2F, 0x0042472A, 0x00404627, 0x003B4122, 0x004E5435, 0x00757C5B, 0x006D8154, 0x00758760, 0x00707F60, 0x0079866C, 0x00717E66,
        0x00667458, 0x00748462, 0x0081936C, 0x007A8767, 0x006A7757, 0x005B6848, 0x00697656, 0x00697656, 0x00798666, 0x006A7757, 0x00596646, 0x00828C7B, 0x007B8771, 0x00687758, 0x006C7A56, 0x00788662, 0x006F7A5A, 0x006D765C, 0x00818873, 0x0062704E,
        0x00485632, 0x00616F4B, 0x004A553B, 0x007C8374, 0x00767D6E, 0x00687456, 0x004E5F34, 0x00546141, 0x0045532F, 0x005F6D43, 0x00667347, 0x004D572C, 0x00474D2A, 0x004A4B31, 0x004C4B37, 0x004C493A, 0x004B4839, 0x004B4839, 0x004D4A3B, 0x004E4B3C,
        0x004E4B3C, 0x004C493A, 0x004A4738, 0x004E4D38, 0x004A4D34, 0x00484D32, 0x00474F31, 0x005C6746, 0x0073825C, 0x0072845B, 0x0073855C, 0x0073815D, 0x0073825C, 0x00718158, 0x0076855F, 0x00647052, 0xFFDDE5D4, 0xFFFDFFFE, 0xFFFBFBFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF8FFFF, 0xFBF3FEFF, 0x00BCCDD0, 0x005E7676, 0x006B8585, 0x006C8783, 0x006B827D, 0x00708681, 0x006A8784, 0x00678481, 0x006E8B88, 0x00688582, 0x006D8A87,
        0x006B8885, 0x006B8885, 0x006B8885, 0x006D8E7F, 0x0065877C, 0x00658983, 0x00648985, 0x006A8D89, 0x005D7F78, 0x005C7D6F, 0x00638372, 0x007B845C, 0x0074815B, 0x00788964, 0x0070815F, 0x00798A69, 0x00768363, 0x0083896A, 0x007E8364, 0x00768C5C,
        0x00758A5D, 0x0075895C, 0x0074865D, 0x0071835C, 0x0074825E, 0x00768462, 0x00768462, 0x0074855A, 0x0074855A, 0x0075865B, 0x0075865B, 0x0075865B, 0x0076875C, 0x0075865B, 0x0075865B, 0x0074855A, 0x0075865B, 0x0077885D, 0x00798A5F, 0x0077885D,
        0x0074855A, 0x00728358, 0x006F8055, 0x0076855F, 0x0075845E, 0x006F7E58, 0x00707F59, 0x0072815B, 0x0071805A, 0x0072815B, 0x006F7E58, 0x006F815A, 0x006E8059, 0x006D7F58, 0x006C7E57, 0x006B7D56, 0x006B7D56, 0x006A7C55, 0x006A7C55, 0x006A7A58,
        0x006B7B59, 0x006E7E5C, 0x006E7E5C, 0x0070805E, 0x0071815F, 0x0071815F, 0x006E7E5C, 0x006D7B59, 0x0073815F, 0x006B7957, 0x00677553, 0x0072805E, 0x006D7B59, 0x00657351, 0x00707E5C, 0x0070825B, 0x0073855E, 0x007A8C65, 0x00778962, 0x006F815A,
        0x0071835A, 0x0074865D, 0x00708259, 0x0075865B, 0x0076865D, 0x007D8D64, 0x00728259, 0x0074845B, 0x007A8A61, 0x0075845E, 0x0076855F, 0x0073835A, 0x0073835A, 0x0073835A, 0x0073835A, 0x0073835A, 0x00728259, 0x00728259, 0x00728259, 0x00708259,
        0x006F8158, 0x006F8158, 0x006E8057, 0x006D7F56, 0x006D7F56, 0x006E8057, 0x006E8057, 0x006F815A, 0x006F815A, 0x006F815A, 0x0070825B, 0x0070825B, 0x0070825B, 0x0070825B, 0x0071835C, 0x006F8158, 0x006D7F56, 0x006D7F56, 0x006E8057, 0x006F8158,
        0x00708259, 0x00708259, 0x006E8057, 0x0073855E, 0x0071835C, 0x006F815A, 0x006F815A, 0x0070825B, 0x0071835C, 0x0071835C, 0x0071835C, 0x006C7F54, 0x00708358, 0x006C7E55, 0x0070825B, 0x00778765, 0x004E5D3D, 0x00283718, 0x003F4E2F, 0x00454A2B,
        0x00474B2E, 0x0043452F, 0x0040412D, 0x00464733, 0x00484A34, 0x0043472A, 0x00414627, 0x00394123, 0x00434B2D, 0x003E4627, 0x003D4526, 0x0047502F, 0x00454E2D, 0x00505938, 0x006F7857, 0x006C8051, 0x0073865B, 0x00798B64, 0x00758464, 0x006E7D5D,
        0x00748462, 0x007C8E67, 0x005E7146, 0x006A7854, 0x005B6945, 0x004A5834, 0x004D5B37, 0x0065734F, 0x006A7854, 0x005F6D49, 0x00566440, 0x00647157, 0x00546344, 0x0072835E, 0x0083936A, 0x007A8A61, 0x006B7852, 0x005B6645, 0x00727C5E, 0x0052603C,
        0x004D5E33, 0x00697A4E, 0x00566440, 0x0078826B, 0x00747E67, 0x00677553, 0x004A5C2D, 0x005E6C48, 0x0053623C, 0x00697A4F, 0x00768559, 0x00616D43, 0x004A5430, 0x0043482B, 0x00494D34, 0x004B4A35, 0x004A4934, 0x004B4A35, 0x004D4C37, 0x00504F3A,
        0x0051503B, 0x00504F3A, 0x004E4D38, 0x00494E31, 0x004B5132, 0x0047502F, 0x00424C28, 0x0055623C, 0x006E7E55, 0x00728358, 0x00718459, 0x0075835F, 0x0074835D, 0x00718158, 0x0076855F, 0x00647052, 0xFFDDE5D4, 0xFFFDFFFE, 0xFFFCFCFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xF0FFFFED, 0xF7FFFFF1, 0x00C9D2B8, 0x00596646, 0x00778863, 0x007F916A, 0x006F7F56, 0x0064754A, 0x00646F49, 0x00646F49, 0x006E7953, 0x00646F49, 0x006D7852,
        0x0068734D, 0x0065704A, 0x0069744E, 0x00728A62, 0x00637A5A, 0x00576F57, 0x00506758, 0x00556D61, 0x00435849, 0x003E543C, 0x0040563A, 0x002D4435, 0x002C4138, 0x00405251, 0x003D4F50, 0x00506059, 0x004B5D46, 0x00697B52, 0x00768A55, 0x00708753,
        0x00738A56, 0x00748958, 0x00728657, 0x006F8354, 0x00708155, 0x00728357, 0x0076875C, 0x0074855A, 0x0074855A, 0x0074855A, 0x0075865B, 0x0075865B, 0x0075865B, 0x0075865B, 0x0075865B, 0x0074855A, 0x0075865B, 0x0076875C, 0x0077885D, 0x0076875C,
        0x00708156, 0x0066774C, 0x00607146, 0x005C6B45, 0x005B6A44, 0x00576640, 0x005B6A44, 0x00596842, 0x00576640, 0x005E6D47, 0x00576640, 0x00556740, 0x0054663F, 0x0052643D, 0x0052643D, 0x0052643D, 0x0054663F, 0x00566841, 0x00586A43, 0x00586846,
        0x00546442, 0x00526240, 0x00576745, 0x00596947, 0x00586846, 0x00546442, 0x00546442, 0x005D6C46, 0x0051603A, 0x00586741, 0x005D6C46, 0x005F6E48, 0x005E6D47, 0x0051603A, 0x00586741, 0x005B6C47, 0x0060724B, 0x006A7C55, 0x0073855C, 0x00718459,
        0x006E8156, 0x006E8255, 0x006A7E51, 0x00748657, 0x00708253, 0x00718256, 0x00738458, 0x00798A5F, 0x0075855C, 0x0078885F, 0x0074845B, 0x0073835A, 0x00728259, 0x00728259, 0x0073835A, 0x0073835A, 0x0074845B, 0x0074845B, 0x0074845B, 0x00708259,
        0x00708259, 0x006F8158, 0x006F8158, 0x006F8158, 0x006F8158, 0x006F8158, 0x00708259, 0x00708259, 0x00708259, 0x00708259, 0x00708259, 0x006F8158, 0x006F8158, 0x006F8158, 0x006F8158, 0x006D8055, 0x006C7F54, 0x006B7E53, 0x006C7F54, 0x006F8257,
        0x00708358, 0x006F8257, 0x006E8156, 0x006F8158, 0x006D7F56, 0x006B7D54, 0x006B7D54, 0x006C7E55, 0x006D7F56, 0x006D7F56, 0x006D7F56, 0x006F8257, 0x00708358, 0x006B7D54, 0x006E8059, 0x0073855E, 0x004C5D38, 0x0021310F, 0x002B3B19, 0x002A3B1A,
        0x0029391A, 0x0028371B, 0x0026351A, 0x0026351A, 0x0028371B, 0x0029391A, 0x00273817, 0x002E3919, 0x002E3919, 0x00273211, 0x00394423, 0x00626D4C, 0x00707C58, 0x006E7A56, 0x00788460, 0x00728657, 0x00768A5D, 0x0076895E, 0x007F916A, 0x008D9F78,
        0x00768861, 0x005F7148, 0x0053663B, 0x00627249, 0x006A7A51, 0x0066764D, 0x005C6C43, 0x00697950, 0x005F6F46, 0x005E6E45, 0x00697950, 0x00627151, 0x00677755, 0x00798B64, 0x007E8E65, 0x005B6C41, 0x00616F46, 0x0073805A, 0x0046532D, 0x0035441E,
        0x00617246, 0x00647746, 0x0046572C, 0x006B7858, 0x00636F53, 0x005D6C46, 0x00617443, 0x0054643B, 0x005C6C43, 0x006B7C51, 0x00738459, 0x00728057, 0x005F6B47, 0x00434E2E, 0x00374123, 0x003D4320, 0x003F4522, 0x00414724, 0x00424825, 0x00424825,
        0x00424825, 0x00434926, 0x00444A27, 0x003C4824, 0x003F4B27, 0x003B4723, 0x0036431D, 0x00515F36, 0x00728259, 0x0077885D, 0x0074855A, 0x00768460, 0x0075845E, 0x0073835A, 0x00778660, 0x00657153, 0xFFDDE5D4, 0xFFFDFFFE, 0xFFFDFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF9FFF7, 0xFFFBFFF5, 0x00CBD8C0, 0x00566746, 0x0080946A, 0x007F9368, 0x0052673A, 0x002C4114, 0x0039471E, 0x0036441B, 0x003A481F, 0x0028360D, 0x003C4A21,
        0x0039471E, 0x0035431A, 0x00404E25, 0x00697870, 0x005B6B6A, 0x00566673, 0x00677690, 0x0098A5C5, 0x009AA8C4, 0x009DAAC0, 0x0096A3B3, 0x0089A6B4, 0x00849EAF, 0x00869AB3, 0x005D6D84, 0x00596871, 0x0046594A, 0x00667E54, 0x00789459, 0x00778B60,
        0x00798D62, 0x00778B5E, 0x0074885B, 0x00748859, 0x00748657, 0x00748657, 0x00778A59, 0x00738459, 0x00738459, 0x00738459, 0x00738459, 0x0074855A, 0x0074855A, 0x0074855A, 0x0075865B, 0x0074855A, 0x0075865B, 0x0074855A, 0x0075865B, 0x0076875C,
        0x006C7D52, 0x005B6C41, 0x00506136, 0x00505F39, 0x0056653F, 0x0051603A, 0x0056653F, 0x0055643E, 0x004F5E38, 0x00586741, 0x0054633D, 0x004F613A, 0x004F613A, 0x0050623B, 0x0051633C, 0x0051633C, 0x0052643D, 0x0053653E, 0x0053653E, 0x004D5F38,
        0x0052643D, 0x0051633C, 0x0052643D, 0x004F613A, 0x00556740, 0x0053653E, 0x0052643D, 0x004D5F36, 0x0056683F, 0x00576940, 0x004C5E35, 0x0053653C, 0x0052643B, 0x00475930, 0x005A6C43, 0x004F613A, 0x00586A43, 0x0063754E, 0x0072845B, 0x0076885F,
        0x0073855C, 0x0074865D, 0x00718459, 0x006F8054, 0x00748559, 0x00708155, 0x00738458, 0x00738459, 0x00708156, 0x00798A5F, 0x006F8055, 0x0076865D, 0x0075855C, 0x0074845B, 0x0074845B, 0x0075855C, 0x0076865D, 0x0076865D, 0x0076865D, 0x006F8257,
        0x006E8156, 0x006E8156, 0x006D8055, 0x006D8055, 0x006E8156, 0x006E8156, 0x006F8257, 0x006F8158, 0x006E8057, 0x006E8057, 0x006D7F56, 0x006D7F56, 0x006C7E55, 0x006C7E55, 0x006C7E55, 0x00738459, 0x00728358, 0x006D7E53, 0x006B7C51, 0x006F8055,
        0x00708156, 0x006F8055, 0x00718257, 0x00708259, 0x006F8158, 0x006D7F56, 0x006D7F56, 0x006E8057, 0x006E8057, 0x006E8057, 0x006D7F56, 0x006F8257, 0x006E8156, 0x006C7E55, 0x006E8057, 0x00708259, 0x0051633C, 0x002A3C15, 0x00293B14, 0x00213917,
        0x001E3614, 0x001F3715, 0x00213917, 0x001C3412, 0x001C3412, 0x00203816, 0x00203816, 0x002B3615, 0x002B3615, 0x00283312, 0x003D4925, 0x0067734F, 0x0078855F, 0x0073805A, 0x0074815B, 0x0074885B, 0x0074885B, 0x0075885D, 0x007C8F64, 0x006E8057,
        0x005A6C43, 0x0055673E, 0x0061734C, 0x0058683F, 0x00607047, 0x0066764D, 0x005E6E45, 0x005E6E45, 0x0056663D, 0x005A6A41, 0x005E6E45, 0x0070815C, 0x00879873, 0x007C8E67, 0x00606F49, 0x005E6E45, 0x006B7950, 0x0056643A, 0x002E3C12, 0x006B7955,
        0x005C6D41, 0x003C4F1C, 0x003C4E1F, 0x006B7955, 0x006C7959, 0x00606F49, 0x00546637, 0x005B6C40, 0x00708156, 0x0074855A, 0x006F7F56, 0x00798862, 0x006F805B, 0x00465634, 0x00243412, 0x002E3A10, 0x00313D13, 0x00333F15, 0x00333F15, 0x00303C12,
        0x00303C12, 0x00323E14, 0x00354117, 0x00303E1A, 0x00303E1A, 0x002D3C16, 0x002E3D17, 0x004D5C36, 0x0074815B, 0x0077855C, 0x00717F56, 0x00768460, 0x0076855F, 0x0074845B, 0x00778660, 0x00636F51, 0xFFDBE3D2, 0xFFFDFFFE, 0xFFFDFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF8FEFF, 0xFFF8FEFD, 0x00C5CEC1, 0x00505D43, 0x007F8F6D, 0x007F916A, 0x0055693F, 0x002E441A, 0x00586A45, 0x0051633E, 0x004A5C37, 0x0031431E, 0x004C5E39,
        0x00435530, 0x0031431E, 0x003B4D28, 0x006A7E61, 0x00556855, 0x00576968, 0x008899A6, 0x00D2E3F8, 0x00D3E5F6, 0x00D3E5F0, 0x00CADDE2, 0x00D5E2FA, 0x00DAE7FD, 0x00CDD7E9, 0x0079828B, 0x00626D6A, 0x00485644, 0x00627554, 0x00667C52, 0x005D6A54,
        0x005B6951, 0x00536046, 0x00526142, 0x005A6A48, 0x005E6D47, 0x005B6C41, 0x005B6C41, 0x00738459, 0x00728358, 0x00728358, 0x00718257, 0x00718257, 0x00728358, 0x00738459, 0x0074855A, 0x00738459, 0x0075865B, 0x00728358, 0x0075865B, 0x0078895E,
        0x006A7B50, 0x00536439, 0x0045562B, 0x0031401A, 0x003D4C26, 0x002A3913, 0x002D3C16, 0x003C4B25, 0x0032411B, 0x002E3D17, 0x002C3B15, 0x002C3F18, 0x002D4019, 0x002F421B, 0x002F421B, 0x002F421B, 0x002E411A, 0x002C3F18, 0x002B3E17, 0x002D4116,
        0x0033471C, 0x00304419, 0x00304419, 0x00293D12, 0x0031451A, 0x002D4116, 0x002B3F14, 0x0030461C, 0x002B4117, 0x002D4319, 0x002A4016, 0x00263C12, 0x002C4218, 0x002C4218, 0x0023390F, 0x003A4C23, 0x004B5D34, 0x005B6D44, 0x006E8059, 0x0074865F,
        0x0070815C, 0x00748560, 0x00728260, 0x00707F59, 0x0074835D, 0x00718158, 0x0078885F, 0x00718257, 0x0076875B, 0x007B8C60, 0x0077885C, 0x0078885F, 0x0076865D, 0x0074845B, 0x0073835A, 0x0074845B, 0x0075855C, 0x0074845B, 0x0073835A, 0x00718459,
        0x00708358, 0x00708358, 0x00708358, 0x00708358, 0x00708358, 0x00718459, 0x00718459, 0x0071835A, 0x00708259, 0x00708259, 0x00708259, 0x00708259, 0x00708259, 0x00708259, 0x00708259, 0x0073835A, 0x0074845B, 0x006E7E55, 0x006C7C53, 0x00708057,
        0x00708057, 0x00708057, 0x0074845B, 0x0073855C, 0x0072845B, 0x0071835A, 0x0071835A, 0x0071835A, 0x0071835A, 0x006F8158, 0x006E8057, 0x006D8055, 0x006C7F54, 0x00708358, 0x00718459, 0x0074875C, 0x006B7E53, 0x00596C41, 0x00596C41, 0x005A6D4C,
        0x005A6E4B, 0x005B704A, 0x005C714B, 0x005C714B, 0x005A6F49, 0x00586C49, 0x00596C4B, 0x005C6A46, 0x00606E4A, 0x005F6D49, 0x0063724C, 0x00707F59, 0x0074835D, 0x00708057, 0x00728259, 0x0071835A, 0x00708259, 0x00708358, 0x0072855A, 0x0056693E,
        0x005E7047, 0x005B6C47, 0x0053643F, 0x0063724C, 0x0064734D, 0x00606F49, 0x00576640, 0x00576640, 0x005C6B45, 0x0061704A, 0x00606F49, 0x006C7E57, 0x0060714C, 0x00576843, 0x0054623E, 0x00677551, 0x0058653F, 0x003B491F, 0x00637147, 0x006A7856,
        0x0036471C, 0x003F521F, 0x00647746, 0x0065734F, 0x00536040, 0x0055633F, 0x005C6D42, 0x006E8051, 0x0078895D, 0x00728358, 0x006A7C55, 0x0073845F, 0x00758762, 0x00657752, 0x0052643F, 0x005A6943, 0x005B6A44, 0x005B6A44, 0x005B6A44, 0x005A6943,
        0x005A6943, 0x005B6A44, 0x005D6C46, 0x00586944, 0x00576843, 0x00576843, 0x00596743, 0x006A7854, 0x007C8864, 0x0077835F, 0x0076825E, 0x0074825E, 0x0075845E, 0x0074845B, 0x0076855F, 0x00606C4E, 0xFFD8E0CF, 0xFFFCFFFD, 0xFFFDFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFBFF, 0xFFFBF8FA, 0x00D3D1C6, 0x0063634B, 0x008D916E, 0x00838B62, 0x005D673D, 0x003C481E, 0x0064764F, 0x0060724B, 0x005B6D46, 0x00455730, 0x00687A53,
        0x00566841, 0x0032441D, 0x0033451E, 0x0079815C, 0x00535C42, 0x00565D58, 0x00989EA9, 0x00E3E9FC, 0x00CBD2E3, 0x00C1C8D1, 0x00C2C9CC, 0x00C8C6DA, 0x00E4E2EE, 0x00DBDCDA, 0x00767969, 0x005C6249, 0x0040492F, 0x004B543F, 0x003A4531, 0x005B625F,
        0x00515952, 0x00424A3F, 0x003E4837, 0x0048553D, 0x004C583C, 0x0043512F, 0x003F4D29, 0x00738459, 0x00728358, 0x00708156, 0x00708156, 0x00708156, 0x00718257, 0x00738459, 0x0074855A, 0x00738459, 0x0074855A, 0x00718257, 0x0074855A, 0x00798A5F,
        0x006A7B50, 0x004F6035, 0x003F5025, 0x004A5933, 0x0056653F, 0x0033421C, 0x0036451F, 0x00576640, 0x004A5933, 0x0035441E, 0x0031401A, 0x002D4019, 0x002E411A, 0x002F421B, 0x0030431C, 0x0031441D, 0x0030431C, 0x0030431C, 0x002F421B, 0x00324818,
        0x00324818, 0x002D4313, 0x00344A1A, 0x00304616, 0x00354B1B, 0x002F4515, 0x00304616, 0x002D451B, 0x0030481E, 0x002B4319, 0x002D451B, 0x002D451B, 0x00284016, 0x002B4319, 0x002A4218, 0x00314419, 0x00495C31, 0x005E7047, 0x0073855E, 0x00778765,
        0x00707F5F, 0x00738263, 0x00728162, 0x00798765, 0x00707E5C, 0x006E7C58, 0x00798862, 0x006E7F54, 0x0075865A, 0x006F8152, 0x00738556, 0x00728259, 0x00708057, 0x006F7F56, 0x006F7F56, 0x00728259, 0x0074845B, 0x0074845B, 0x0073835A, 0x006E8156,
        0x006E8156, 0x006D8055, 0x006D8055, 0x006D8055, 0x006E8156, 0x006F8257, 0x006F8257, 0x006E8156, 0x006E8156, 0x006E8156, 0x006F8257, 0x006F8257, 0x006F8257, 0x00708358, 0x00708358, 0x006E7D57, 0x0072815B, 0x006F7E58, 0x006F7E58, 0x0074835D,
        0x0073825C, 0x0071805A, 0x0075845E, 0x00708358, 0x00708358, 0x006F8257, 0x006F8257, 0x006F8257, 0x006E8156, 0x006C7F54, 0x006A7D52, 0x0071835A, 0x006D8055, 0x006F8257, 0x006A7D52, 0x006A7D52, 0x006F8257, 0x006F8257, 0x0074875C, 0x00778262,
        0x007B8665, 0x0075815D, 0x00727F59, 0x00798660, 0x00798561, 0x00747F5E, 0x00778262, 0x00748560, 0x00768762, 0x00748560, 0x0071835C, 0x0072845D, 0x0072845B, 0x00708259, 0x00708259, 0x0070825B, 0x007E9067, 0x0062744B, 0x00576A3F, 0x004D5F36,
        0x005F714A, 0x0050603E, 0x005C6B4B, 0x00505E3A, 0x00667450, 0x00687652, 0x0064724E, 0x0065734F, 0x00667450, 0x00606E4A, 0x00667450, 0x00657750, 0x0060714C, 0x00647452, 0x00707E5C, 0x00485634, 0x0035421C, 0x00606E45, 0x006B794F, 0x00334020,
        0x003E4F24, 0x00607340, 0x005A6D3C, 0x0055633F, 0x00717D5F, 0x0072805E, 0x006A7A51, 0x006D804F, 0x006A7B4F, 0x006A7A51, 0x006D7F58, 0x006A7C57, 0x00647651, 0x00667B55, 0x006D825B, 0x006D7D5E, 0x006B7B5C, 0x006A7A5B, 0x006A7A5B, 0x006C7C5D,
        0x006D7D5E, 0x006D7D5E, 0x006C7C5D, 0x006C7E59, 0x006D7E5C, 0x00748462, 0x00748462, 0x00768462, 0x00747F5E, 0x006D7857, 0x00768160, 0x0072805C, 0x0074835D, 0x0074845B, 0x0075845E, 0x005E6A4C, 0xFFD5DDCC, 0xFFFAFDFB, 0xFFFDFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xFFFBFBFB, 0x00C8CFBA, 0x0058693D, 0x00819267, 0x00808F70, 0x00566742, 0x00395117, 0x0062744D, 0x0034461F, 0x0064764F, 0x00394B24, 0x0053653E,
        0x00596B44, 0x0031431C, 0x00384A23, 0x00658054, 0x00617957, 0x00576958, 0x00A2AFAD, 0x00D7E1E8, 0x00CFD8E1, 0x00C5D0D4, 0x00B1BDBD, 0x00C9CDDF, 0x00D1D4E2, 0x00D3D5DD, 0x009B9D9D, 0x00757A71, 0x00757F6E, 0x0056664E, 0x0062755A, 0x009C9DA1,
        0x00787A7B, 0x00666868, 0x006C6F6D, 0x005D615B, 0x008D9289, 0x007F8679, 0x00373E31, 0x0076875B, 0x00708155, 0x006F8054, 0x00728357, 0x00728357, 0x00748559, 0x0075865A, 0x00738458, 0x0076875C, 0x00728358, 0x00738459, 0x00728259, 0x007A8A61,
        0x0064744B, 0x00505F39, 0x0041502A, 0x0034461F, 0x005E7049, 0x00556740, 0x00263811, 0x004A5C35, 0x00697B54, 0x00384A23, 0x002D3F18, 0x002E4519, 0x002B4216, 0x002A4115, 0x002F461A, 0x00334A1E, 0x0031481C, 0x002D4418, 0x002D4418, 0x002F4417,
        0x002E4316, 0x00314619, 0x00314619, 0x002E4316, 0x0033481B, 0x00364B1E, 0x00304518, 0x00314619, 0x002D4417, 0x002C4317, 0x002C4317, 0x002A4218, 0x00294117, 0x00274217, 0x00264018, 0x002F4319, 0x004F6339, 0x00576940, 0x0076885F, 0x0072845B,
        0x0075855C, 0x0073835A, 0x0077875E, 0x0076875C, 0x0075865B, 0x0074855A, 0x0074855A, 0x0074855A, 0x0074855A, 0x0075865B, 0x0076875C, 0x0076865D, 0x0076865D, 0x0074845B, 0x0073835A, 0x0073835A, 0x00728259, 0x00728259, 0x00728259, 0x00708057,
        0x00708057, 0x00718158, 0x00708057, 0x00708057, 0x006F7F56, 0x006E7E55, 0x006D7D54, 0x006C7E55, 0x006C7E55, 0x006D7F56, 0x006D7F56, 0x006E8057, 0x006F8158, 0x006F8158, 0x00708259, 0x00708057, 0x00708057, 0x00708057, 0x00708057, 0x00718158,
        0x00718158, 0x00718158, 0x00718158, 0x00728259, 0x00718158, 0x00718158, 0x00708057, 0x00708057, 0x00708057, 0x00708057, 0x00708057, 0x006E7D57, 0x0073835A, 0x00718158, 0x006D7F56, 0x0072855A, 0x00718459, 0x006D8255, 0x00708558, 0x00748153,
        0x00738354, 0x00718054, 0x006E7F54, 0x006B7D54, 0x0071845D, 0x006D7F5A, 0x006C7E59, 0x006A825A, 0x006E835C, 0x00697E57, 0x006E815A, 0x0073845F, 0x00738361, 0x00758262, 0x006D795B, 0x00758861, 0x006B7E57, 0x006A7D56, 0x00556841, 0x0053663F,
        0x00546740, 0x00546740, 0x004C5F38, 0x006B7E57, 0x004B5E37, 0x003F522B, 0x00566942, 0x00586B44, 0x003F522B, 0x00566942, 0x00556841, 0x003A4B29, 0x005D6E4C, 0x0052643F, 0x003E502B, 0x00576A43, 0x0055693F, 0x00586C42, 0x003D5126, 0x00333F21,
        0x006B7955, 0x004A5B30, 0x00606D47, 0x00768161, 0x006E7756, 0x00738052, 0x0065773C, 0x00606C50, 0x005E6B4B, 0x005B6945, 0x005A6B40, 0x005A6B3F, 0x005A6B3F, 0x0058693E, 0x0056663D, 0x005A6844, 0x00586642, 0x00596743, 0x005C6A46, 0x005D6B47,
        0x005B6945, 0x005D6B47, 0x00616F4B, 0x0063714F, 0x005C6A48, 0x006D7B59, 0x00667654, 0x00697A55, 0x0071835E, 0x006D8059, 0x00667A50, 0x006D7E59, 0x006E8059, 0x006E8057, 0x00728259, 0x00616E4E, 0xFFD1D8C3, 0xFFFFFFF9, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xFFFDFDFD, 0x00CAD1BC, 0x0056673B, 0x007B8C61, 0x00798869, 0x0050613C, 0x00324A10, 0x006C7E57, 0x0053653E, 0x0062744D, 0x00364821, 0x005E7049,
        0x0060724B, 0x002B3D16, 0x0033451E, 0x006E8461, 0x004F6249, 0x005C6A5F, 0x00AEB7BB, 0x00DAE0ED, 0x00C7CDDA, 0x00C9CFDA, 0x00CAD2D9, 0x00C9CFDC, 0x00DEE3EC, 0x00CFD2D6, 0x00909391, 0x00ADB3A8, 0x0076816D, 0x003D4D35, 0x006F8366, 0x00ABAFAA,
        0x008C908A, 0x005D6259, 0x00646A5F, 0x00818879, 0x00A8B09F, 0x00848D79, 0x00555E49, 0x005C6D41, 0x007A8B5F, 0x0078895D, 0x006C7D51, 0x0077885C, 0x0077885C, 0x006E7F53, 0x0076875B, 0x0077885D, 0x00738459, 0x0075865B, 0x0073835A, 0x007B8B62,
        0x0066764D, 0x0051603A, 0x0043522C, 0x00263811, 0x0043552E, 0x005F714A, 0x0041532C, 0x002B3D16, 0x00566841, 0x005C6E47, 0x00283A13, 0x0031481C, 0x0030471B, 0x002F461A, 0x0030471B, 0x0030471B, 0x002E4519, 0x002E4519, 0x002F461A, 0x00304419,
        0x00304419, 0x0031451A, 0x0031451A, 0x002F4318, 0x0031451A, 0x0032461B, 0x00304419, 0x0034491C, 0x0035491E, 0x002E4519, 0x002B4216, 0x002C441A, 0x002A4218, 0x00253F17, 0x00274119, 0x002F4319, 0x004F6339, 0x00576940, 0x0075875E, 0x0071835A,
        0x0075855C, 0x0073835A, 0x0076865D, 0x0075865B, 0x0074855A, 0x0074855A, 0x0074855A, 0x0074855A, 0x0074855A, 0x0074855A, 0x0075865B, 0x0074845B, 0x0073835A, 0x0073835A, 0x00728259, 0x00728259, 0x00728259, 0x00728259, 0x0073835A, 0x0073835A,
        0x0073835A, 0x0073835A, 0x0073835A, 0x00728259, 0x00718158, 0x00718158, 0x00708057, 0x006D7F56, 0x006D7F56, 0x006E8057, 0x006E8057, 0x006F8158, 0x00708259, 0x00708259, 0x00708259, 0x00708057, 0x00708057, 0x00708057, 0x00708057, 0x00718158,
        0x00718158, 0x00718158, 0x00718158, 0x00728259, 0x00718158, 0x00718158, 0x00708057, 0x00708057, 0x00708057, 0x00708057, 0x00708057, 0x006E7E55, 0x006D7D54, 0x006F7F56, 0x0071835A, 0x006F815A, 0x00667851, 0x0063764F, 0x006C7E59, 0x0076875C,
        0x0063734A, 0x006D7C56, 0x006B7C57, 0x007D8F68, 0x0064764D, 0x006F8257, 0x007C9063, 0x006D7F56, 0x006E8156, 0x006F8257, 0x006A7C53, 0x0070825B, 0x006E7C5A, 0x005F6B4D, 0x00697559, 0x00697A58, 0x0060714F, 0x00617250, 0x00485937, 0x004F603E,
        0x00536442, 0x004D5E3C, 0x00586947, 0x004A5B39, 0x00415230, 0x00465735, 0x00556644, 0x00495A38, 0x00374826, 0x00586947, 0x00586947, 0x00455536, 0x00556546, 0x00485938, 0x00455634, 0x00586A45, 0x00536540, 0x0053663F, 0x0051643D, 0x00536040,
        0x0053623C, 0x005D6E43, 0x00707C58, 0x00687156, 0x005B6247, 0x004E5931, 0x0062723D, 0x005C6544, 0x0058633D, 0x00535F35, 0x00505D2F, 0x004E5C2C, 0x004C5A2A, 0x00495628, 0x00465327, 0x00465228, 0x00445026, 0x00455127, 0x0048542A, 0x0049552B,
        0x0048542A, 0x004A562C, 0x004F5B31, 0x00515F35, 0x0057653C, 0x00586342, 0x005C684A, 0x00606C4E, 0x00637252, 0x00667752, 0x00687A51, 0x006B7C57, 0x006C7E57, 0x006C7E55, 0x00707F59, 0x005F6C4C, 0xFFD0D6C3, 0xFFFEFFFA, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFDFF, 0xFFFEFEFE, 0x00C9D0BB, 0x00596A3E, 0x00819267, 0x007F8E6F, 0x00566742, 0x00344C12, 0x0063754E, 0x0060724B, 0x00576942, 0x003B4D26, 0x005E7049,
        0x0061734C, 0x00364821, 0x003B4D26, 0x00727F65, 0x0047523E, 0x005E655E, 0x00A4A7AC, 0x00EAEAF6, 0x00E1E1ED, 0x00D4D5DF, 0x00D1D3DB, 0x00D2DBDE, 0x00D8DDDE, 0x00D1D6D4, 0x006B6E65, 0x00808576, 0x0067735D, 0x0057664A, 0x00455A3A, 0x004B5440,
        0x00656E5A, 0x00565F4A, 0x00505A43, 0x004F5A40, 0x00586448, 0x00546042, 0x006D7A5A, 0x004F6034, 0x0068794D, 0x007A8B5F, 0x0077885C, 0x00728357, 0x00748559, 0x0076875B, 0x00738458, 0x0077885D, 0x0074855A, 0x0076875C, 0x0074845B, 0x007C8C63,
        0x0067774E, 0x0053623C, 0x0045542E, 0x0050623B, 0x00253710, 0x004B5D36, 0x006A7C55, 0x002E4019, 0x0032441D, 0x00687A53, 0x004D5F38, 0x002D4116, 0x002E4217, 0x002F4318, 0x002E4217, 0x002C4015, 0x002E4217, 0x0031451A, 0x0032461B, 0x0032461B,
        0x0033471C, 0x0032461B, 0x0032461B, 0x0032461B, 0x0031451A, 0x00304419, 0x0032461B, 0x00304419, 0x0033471C, 0x002B4216, 0x00283E14, 0x002D4319, 0x002A421A, 0x00273F17, 0x00294119, 0x002E4218, 0x004F6339, 0x0056683F, 0x0075875E, 0x00708259,
        0x0074845B, 0x00728259, 0x0075855C, 0x00738459, 0x00738459, 0x00738459, 0x0074855A, 0x0074855A, 0x00738459, 0x00738459, 0x00738459, 0x00728259, 0x00718158, 0x00718158, 0x00718158, 0x00718158, 0x00728259, 0x0073835A, 0x0073835A, 0x0073835A,
        0x0073835A, 0x0073835A, 0x0073835A, 0x0073835A, 0x00728259, 0x00718158, 0x00708057, 0x006E8057, 0x006E8057, 0x006F8158, 0x006F8158, 0x00708259, 0x00708259, 0x0071835A, 0x0071835A, 0x00708057, 0x00708057, 0x00708057, 0x00708057, 0x00718158,
        0x00718158, 0x00718158, 0x00718158, 0x00728259, 0x00718158, 0x00718158, 0x00708057, 0x00708057, 0x00708057, 0x00708057, 0x00708057, 0x00718256, 0x006F8055, 0x006E7E55, 0x00657651, 0x00546442, 0x00495839, 0x0048573B, 0x004D5C40, 0x00526643,
        0x004C5F3E, 0x004A593D, 0x0049573B, 0x00617050, 0x0067774E, 0x00718151, 0x006F824B, 0x00778157, 0x00758256, 0x007A875B, 0x006E7D51, 0x00728259, 0x00647250, 0x00445236, 0x005D6A52, 0x003F4E33, 0x00445338, 0x004E5D42, 0x002F3E23, 0x00455439,
        0x0047563B, 0x002D3C21, 0x00505F44, 0x00526146, 0x00324126, 0x00707F64, 0x004A593E, 0x00314025, 0x004D5C41, 0x00324126, 0x0047563B, 0x0064715B, 0x0035432B, 0x00445338, 0x005E6D51, 0x00334324, 0x00425332, 0x005D6E4C, 0x00455732, 0x005A6846,
        0x004A5A31, 0x00607146, 0x005B6645, 0x00575F48, 0x00797E69, 0x00636B4C, 0x004D592F, 0x00586136, 0x00525C2E, 0x004C5824, 0x00495620, 0x0047541C, 0x0045521A, 0x00424E1A, 0x00404B19, 0x00434C19, 0x00434C19, 0x00444D1A, 0x00454E1B, 0x00464F1C,
        0x00464F1C, 0x0049521F, 0x004D5623, 0x00485620, 0x005B6638, 0x004C5332, 0x005A6049, 0x00595E49, 0x005C654B, 0x00646F4F, 0x00707D57, 0x00697957, 0x006A7C55, 0x006B7D54, 0x006F7E58, 0x005E6B4B, 0xFFCFD5C2, 0xFFFEFFFA, 0xFFFFFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xFFFEFEFE, 0x00C6CDB8, 0x0056673B, 0x007E8F64, 0x007A896A, 0x0051623D, 0x002D450B, 0x00354720, 0x00364821, 0x0031431C, 0x002D3F18, 0x00354720,
        0x003B4D26, 0x002E4019, 0x002D3F18, 0x006E7A5C, 0x006F7A60, 0x008E9685, 0x009BA097, 0x00DBDEDC, 0x00DEE1DF, 0x00D1D4D2, 0x00DCE0DB, 0x00D0DBD3, 0x00EDF6EC, 0x00CBD2C5, 0x006C7261, 0x00ABB39C, 0x00879377, 0x00637453, 0x007C8F6E, 0x00717F5D,
        0x00717F5D, 0x006A7856, 0x0074825E, 0x0073825C, 0x0073825C, 0x005D6D44, 0x00526239, 0x00516237, 0x00516237, 0x0067784D, 0x007A8B60, 0x0075865B, 0x00738459, 0x0077885D, 0x00738459, 0x0076875C, 0x0074855A, 0x0076875C, 0x0074845B, 0x007B8B62,
        0x0066764D, 0x0053623C, 0x0045542E, 0x0071835C, 0x004A5C35, 0x0031431C, 0x00556740, 0x005C6E47, 0x0030421B, 0x003B4D26, 0x006E8059, 0x003A4E24, 0x00384C22, 0x00384C22, 0x00364A20, 0x0033471D, 0x0035491F, 0x00374B21, 0x0033471D, 0x0033471D,
        0x0034481E, 0x0032461C, 0x0033471D, 0x0035491F, 0x0033471D, 0x0031451B, 0x0035491F, 0x0033471C, 0x0032461B, 0x0031451B, 0x002E441A, 0x002D421B, 0x002F441D, 0x002B431B, 0x00263D17, 0x002E4218, 0x004E6238, 0x0056683F, 0x0074865D, 0x00708259,
        0x0073835A, 0x00718158, 0x0074845B, 0x00718257, 0x00728358, 0x00738459, 0x00738459, 0x00738459, 0x00738459, 0x00728358, 0x00718257, 0x0073835A, 0x00728259, 0x00728259, 0x00718158, 0x00728259, 0x00728259, 0x0073835A, 0x0073835A, 0x00718158,
        0x00728259, 0x00728259, 0x00728259, 0x00718158, 0x00708057, 0x006F7F56, 0x006F7F56, 0x006E8057, 0x006E8057, 0x006F8158, 0x006F8158, 0x006F8158, 0x00708259, 0x00708259, 0x00708259, 0x00708057, 0x00708057, 0x00708057, 0x00708057, 0x00718158,
        0x00718158, 0x00718158, 0x00718158, 0x00728259, 0x00718158, 0x00718158, 0x00708057, 0x00708057, 0x00708057, 0x00708057, 0x00708057, 0x006E7F53, 0x00748559, 0x0075855C, 0x0061724D, 0x004E5D3E, 0x00525F45, 0x005C6953, 0x005A6652, 0x004A5E41,
        0x0055684D, 0x00495942, 0x00616E58, 0x0049573B, 0x0052613B, 0x00556633, 0x0076874E, 0x007A7E5A, 0x00767E55, 0x00778156, 0x00738052, 0x00758359, 0x005E6C4A, 0x00414E34, 0x0057624E, 0x00202C18, 0x003C4834, 0x004A5642, 0x001E2A16, 0x00424E3A,
        0x00434F3B, 0x00182410, 0x00525E4A, 0x0047533F, 0x0027331F, 0x00606C58, 0x00384430, 0x002C3824, 0x005F6B57, 0x0036422E, 0x00333F2B, 0x00636E5E, 0x002E3A28, 0x003D4935, 0x005A6850, 0x0026351A, 0x00425233, 0x00576847, 0x00495A38, 0x0052603C,
        0x0053633A, 0x00536439, 0x00505B3B, 0x00636956, 0x00929586, 0x00868C75, 0x0049532F, 0x005C6330, 0x00555D28, 0x004E571E, 0x004B5618, 0x00495416, 0x00465014, 0x00444D15, 0x00444C17, 0x00444B12, 0x00454C13, 0x00454C13, 0x00464D14, 0x00464D14,
        0x00474E15, 0x00495017, 0x004C531A, 0x004E5D1F, 0x00485321, 0x00464B2B, 0x007B7D6A, 0x00707062, 0x00555946, 0x004E5638, 0x0075805A, 0x00697957, 0x006A7B56, 0x006B7D56, 0x006F7E58, 0x005F6B4D, 0xFFD0D6C3, 0xFFFEFFFA, 0xFFFFFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFDFF, 0xFFFFFFFF, 0x00C7CEB9, 0x0057683C, 0x007B8C61, 0x00788768, 0x005F704B, 0x00486026, 0x00495B34, 0x00485A33, 0x00455730, 0x0050623B, 0x00475932,
        0x00475932, 0x004B5D36, 0x004B5D36, 0x006E7C53, 0x007A8761, 0x008F9A7A, 0x006E775C, 0x00858E74, 0x00818972, 0x007D856E, 0x00929B81, 0x00809079, 0x0078846E, 0x0078826B, 0x00676E53, 0x00777F61, 0x00687555, 0x00657553, 0x006E835D, 0x0075865B,
        0x00718257, 0x0078895E, 0x007F9064, 0x006E7F53, 0x00738458, 0x00728357, 0x00637448, 0x0056673C, 0x00516237, 0x004E5F34, 0x00637449, 0x007D8E63, 0x0078895E, 0x006D7E53, 0x0078895E, 0x0074855A, 0x00728358, 0x0075865B, 0x0073835A, 0x007A8A61,
        0x0064744B, 0x0051603A, 0x0045542E, 0x006F815A, 0x0072845D, 0x003E5029, 0x0031431C, 0x005D6F48, 0x004D5F38, 0x002D3F18, 0x00475932, 0x0063754C, 0x005D6F46, 0x005E7047, 0x00607249, 0x00607249, 0x0063754C, 0x0064764D, 0x005E7047, 0x005F724B,
        0x0060734C, 0x0060734C, 0x0061744D, 0x0062754E, 0x0062754E, 0x0062754E, 0x0063764F, 0x0062744B, 0x005C6E45, 0x0060744A, 0x0061744D, 0x005A6D46, 0x005A6F49, 0x005A6F49, 0x004A5F39, 0x002E4218, 0x004E6238, 0x0056683F, 0x0074865D, 0x006F8158,
        0x0073835A, 0x00708057, 0x0073835A, 0x00718257, 0x00718257, 0x00728358, 0x00738459, 0x00738459, 0x00728358, 0x00718257, 0x00718257, 0x0076865D, 0x0075855C, 0x0074845B, 0x0073835A, 0x00728259, 0x00728259, 0x00728259, 0x00728259, 0x00728259,
        0x00728259, 0x00728259, 0x00728259, 0x00728259, 0x00718158, 0x00708057, 0x006F7F56, 0x006D7F56, 0x006E8057, 0x006E8057, 0x006E8057, 0x006E8057, 0x006E8057, 0x006E8057, 0x006E8057, 0x00708057, 0x00708057, 0x00708057, 0x00708057, 0x00718158,
        0x00718158, 0x00718158, 0x00718158, 0x00728259, 0x00718158, 0x00718158, 0x00708057, 0x00708057, 0x00708057, 0x00708057, 0x00708057, 0x006E7F53, 0x00748559, 0x0075855C, 0x0061724D, 0x004D5C3D, 0x00515E44, 0x005B6852, 0x005A6652, 0x00617459,
        0x00607058, 0x0045553E, 0x0065725C, 0x005B6A4F, 0x00667555, 0x004F5F36, 0x00455728, 0x004C5136, 0x00555B3C, 0x005E6941, 0x00728155, 0x0079875D, 0x005D6B49, 0x0048523B, 0x004B5544, 0x001E281B, 0x00475144, 0x004F594C, 0x001A2417, 0x00465043,
        0x00495346, 0x001E281B, 0x00626C5F, 0x004A5447, 0x00263023, 0x005A6457, 0x002E382B, 0x002A3427, 0x00687265, 0x00323C2F, 0x00283225, 0x00646E62, 0x00283225, 0x003A4535, 0x005C6854, 0x00233217, 0x00425233, 0x00566746, 0x00495A38, 0x004E5C38,
        0x0058683F, 0x0054653A, 0x00545F3F, 0x00494F3C, 0x003A3D2E, 0x00434932, 0x0049532F, 0x005E672F, 0x00576027, 0x00505A1E, 0x004D581A, 0x004A5517, 0x00485216, 0x0049521A, 0x004C5320, 0x004B511A, 0x004C521B, 0x004C521B, 0x004C521B, 0x004C521B,
        0x004C521B, 0x004B511A, 0x004A5019, 0x00445316, 0x004C5725, 0x004F5434, 0x00595B48, 0x0048483C, 0x005F6350, 0x005C6446, 0x0067714D, 0x006B7A5A, 0x006C7D58, 0x006D7F58, 0x00717F5B, 0x00606C4E, 0xFFD1D7C6, 0xFFFFFFFC, 0xFFFFFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFCFF, 0xFFFFFFFF, 0x00C9D0BB, 0x005A6B3F, 0x007A8B60, 0x007A896A, 0x00788964, 0x00748C52, 0x007B8D66, 0x007B8D66, 0x00758760, 0x007C8E67, 0x007F916A,
        0x00778962, 0x00778962, 0x007F916A, 0x00829367, 0x00708155, 0x007B8C61, 0x00708156, 0x0077875E, 0x0071805A, 0x0072815B, 0x007C8B65, 0x006A7B5A, 0x006F7F5D, 0x00727D5C, 0x006C7652, 0x0069734F, 0x0077845E, 0x00788A63, 0x0072855E, 0x00788A5B,
        0x00748657, 0x00738556, 0x00728455, 0x006A7B4F, 0x00748559, 0x007B8C60, 0x0077885C, 0x0065754C, 0x0054643B, 0x004A5A31, 0x0055653C, 0x00697950, 0x0075855C, 0x0074845B, 0x00718158, 0x00728358, 0x00708156, 0x0074855A, 0x00728259, 0x0078885F,
        0x0063734A, 0x0051603A, 0x0045542E, 0x006D7F58, 0x00788A63, 0x00465831, 0x002F411A, 0x00576942, 0x0051633C, 0x002F411A, 0x00374922, 0x0061734A, 0x00576940, 0x005A6C43, 0x00607249, 0x005E7047, 0x00607249, 0x0062744B, 0x005D6F46, 0x00576944,
        0x00576944, 0x005A6C47, 0x00596B46, 0x00576944, 0x005B6D48, 0x005E704B, 0x00596B46, 0x005D6F46, 0x00596B42, 0x0060724B, 0x0062754E, 0x005C6E49, 0x0060724D, 0x005C714B, 0x004B5F3C, 0x002F4319, 0x004F6339, 0x0056683F, 0x0074865D, 0x006F8158,
        0x00728259, 0x00708057, 0x0073835A, 0x00718257, 0x00728358, 0x00728358, 0x00728358, 0x00728358, 0x00728358, 0x00728358, 0x00718257, 0x0077875E, 0x0076865D, 0x0075855C, 0x0074845B, 0x0073835A, 0x00728259, 0x00728259, 0x00728259, 0x0075855C,
        0x0075855C, 0x0075855C, 0x0075855C, 0x0074845B, 0x0073835A, 0x00728259, 0x00728259, 0x006D7F56, 0x006D7F56, 0x006D7F56, 0x006D7F56, 0x006D7F56, 0x006D7F56, 0x006D7F56, 0x006D7F56, 0x00708057, 0x00708057, 0x00708057, 0x00708057, 0x00718158,
        0x00718158, 0x00718158, 0x00718158, 0x00728259, 0x00718158, 0x00718158, 0x00708057, 0x00708057, 0x00708057, 0x00708057, 0x00708057, 0x00728357, 0x006F8055, 0x006E7E55, 0x00647550, 0x00536341, 0x00475637, 0x0047563A, 0x004B5A3E, 0x0048563A,
        0x004B593D, 0x004B5A3E, 0x0049583C, 0x00536449, 0x00506146, 0x00495941, 0x00182A13, 0x003A4339, 0x00414D3B, 0x00415031, 0x004D5F38, 0x0053633A, 0x00404E2C, 0x003D452E, 0x00434938, 0x00151F13, 0x00374135, 0x003E483C, 0x00151F13, 0x003A4438,
        0x003B4539, 0x001A2418, 0x004E584C, 0x003C463A, 0x001A2418, 0x00545E52, 0x00252F23, 0x001D271B, 0x005A6458, 0x00202A1E, 0x00293327, 0x00596258, 0x00182216, 0x002F3A2A, 0x0056634D, 0x00202F14, 0x003B4C2B, 0x00536442, 0x0040522D, 0x00515F3D,
        0x004B5B32, 0x0055663B, 0x0065704F, 0x007D856E, 0x006E735E, 0x004D5536, 0x004B572D, 0x005A642F, 0x00525D25, 0x004C581E, 0x004B571B, 0x004A561C, 0x004D5722, 0x00565F2D, 0x0061683B, 0x005E683A, 0x005E683A, 0x005E683A, 0x005E683A, 0x005E683A,
        0x005C6638, 0x00566032, 0x004F592B, 0x004C5925, 0x003D471C, 0x00515738, 0x006F745F, 0x00666A57, 0x00686F5A, 0x005E684A, 0x00737F5B, 0x006D7C5C, 0x006E7E5C, 0x006F805B, 0x0072805C, 0x00616D51, 0xFFD1D7C6, 0xFFFFFFFC, 0xFFFFFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xFFFFFFFF, 0x00C3CAB5, 0x0057683C, 0x0076875C, 0x00738263, 0x00758661, 0x00789056, 0x00758760, 0x0074865F, 0x0074865F, 0x006F815A, 0x007D8F68,
        0x00768861, 0x0071835C, 0x007B8D66, 0x007B8D66, 0x0070825B, 0x0076885F, 0x007F9267, 0x007C8F64, 0x0073855C, 0x00798B64, 0x007C8E67, 0x00798D62, 0x007B8E63, 0x007E8C62, 0x00828E64, 0x00808C62, 0x0078865C, 0x0072855A, 0x0072865B, 0x00718354,
        0x00728357, 0x00718256, 0x0075865A, 0x0078895E, 0x0078895E, 0x0073835A, 0x0073835A, 0x0076865D, 0x005C6C43, 0x0057673E, 0x0055653C, 0x004F5F36, 0x0065754C, 0x00798960, 0x006D7D54, 0x00718257, 0x00708156, 0x0074855A, 0x0073835A, 0x0078885F,
        0x0063734A, 0x0051603A, 0x0045542E, 0x006B7D56, 0x006C7E57, 0x0033451E, 0x0040522B, 0x006A7C55, 0x003B4D26, 0x002A3C15, 0x005E7049, 0x0061734C, 0x00556740, 0x00596B44, 0x0060724B, 0x00576942, 0x0052643D, 0x00596B44, 0x005A6C45, 0x00596A48,
        0x00576846, 0x005D6E4C, 0x005B6C4A, 0x00536442, 0x005D6E4C, 0x00637452, 0x00556644, 0x0054643B, 0x00596842, 0x00576942, 0x00566841, 0x00586A45, 0x005A6C47, 0x00526341, 0x00455936, 0x0030441A, 0x0050643A, 0x00576940, 0x0075875E, 0x00708259,
        0x0073835A, 0x00708057, 0x0073835A, 0x00728358, 0x00728358, 0x00728358, 0x00718257, 0x00718257, 0x00728358, 0x00728358, 0x00728358, 0x0075855C, 0x0074845B, 0x0073835A, 0x00728259, 0x00728259, 0x00728259, 0x00728259, 0x0073835A, 0x0075855C,
        0x0075855C, 0x0075855C, 0x0075855C, 0x0075855C, 0x0074845B, 0x0073835A, 0x00728259, 0x006F8158, 0x006F8158, 0x006E8057, 0x006E8057, 0x006E8057, 0x006E8057, 0x006E8057, 0x006E8057, 0x00708057, 0x00708057, 0x00708057, 0x00708057, 0x00718158,
        0x00718158, 0x00718158, 0x00718158, 0x00728259, 0x00718158, 0x00718158, 0x00708057, 0x00708057, 0x00708057, 0x00708057, 0x00708057, 0x006F7F56, 0x006E7E55, 0x006F7F56, 0x00708259, 0x006D7F58, 0x0063754E, 0x0060734C, 0x00697B56, 0x00565C3D,
        0x00303A16, 0x002A3913, 0x00374924, 0x004D6242, 0x003C4F3E, 0x00465753, 0x00213235, 0x0027373E, 0x003F504C, 0x00495D4A, 0x004B603F, 0x0052653E, 0x0043512F, 0x00333922, 0x00404131, 0x00192317, 0x001E281C, 0x00242E22, 0x00131D11, 0x00273125,
        0x00232D21, 0x00121C10, 0x00252F23, 0x001E281C, 0x00202A1E, 0x00283226, 0x00212B1F, 0x00162014, 0x00283226, 0x001E281C, 0x00172115, 0x00293327, 0x001D271A, 0x001B2715, 0x002E3B25, 0x00324125, 0x00435432, 0x00445730, 0x004D6137, 0x004E5B3B,
        0x0051603A, 0x0066774C, 0x005D6945, 0x0060694E, 0x0061684D, 0x00545F37, 0x0053632E, 0x00566031, 0x004D5826, 0x0044501C, 0x00424F19, 0x00424E1A, 0x00485224, 0x005A6239, 0x006D744F, 0x00667452, 0x00677553, 0x00667452, 0x00657351, 0x00667452,
        0x0062704E, 0x00566442, 0x004A5836, 0x003A4721, 0x003A4622, 0x00424C2E, 0x004D583E, 0x00576248, 0x00637155, 0x00667555, 0x006B7C57, 0x006F7E5E, 0x006F7F5D, 0x006F805B, 0x0072805E, 0x00616D51, 0xFFD0D5C6, 0xFFFEFFFD, 0xFFFFFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xFFFBFBFB, 0x00BCC3AE, 0x0058693D, 0x007E8F64, 0x00798869, 0x00778863, 0x00758D53, 0x00798B64, 0x0074865F, 0x0081936C, 0x0072845D, 0x007C8E67,
        0x007A8C65, 0x00788A63, 0x007B8D66, 0x00718366, 0x007E9071, 0x00708461, 0x00728761, 0x00748963, 0x00748963, 0x00748865, 0x006F8261, 0x006F8555, 0x00728455, 0x00788859, 0x00778156, 0x007F895E, 0x00738256, 0x00798A5F, 0x006E8257, 0x00748559,
        0x00738458, 0x00738459, 0x0074845B, 0x00718158, 0x006B7A54, 0x006D7C56, 0x0075835F, 0x007A8A61, 0x00728259, 0x005F6F46, 0x004F5F36, 0x004D5D34, 0x0056663D, 0x0068784F, 0x00798960, 0x00718257, 0x00718257, 0x0075865B, 0x0073835A, 0x00798960,
        0x0063734A, 0x0052613B, 0x0046552F, 0x006E8059, 0x00394B24, 0x002C3E17, 0x006A7C55, 0x0054663F, 0x00273912, 0x0054663F, 0x005F714A, 0x0032411B, 0x00283711, 0x0032411B, 0x003D4C26, 0x0035441E, 0x0033421C, 0x0042512B, 0x004C5B35, 0x00364725,
        0x00334422, 0x003C4D2B, 0x00384927, 0x002C3D1B, 0x00384927, 0x00415230, 0x002D3E1C, 0x00404F29, 0x004B5A34, 0x003F512A, 0x00364722, 0x003E502B, 0x003A4B29, 0x002A3B19, 0x00243815, 0x0030441A, 0x0050643A, 0x00576940, 0x0075875E, 0x00708259,
        0x0073835A, 0x00708057, 0x0073835A, 0x00738459, 0x00728358, 0x00728358, 0x00718257, 0x00718257, 0x00728358, 0x00728358, 0x00738459, 0x00728259, 0x00728259, 0x00718158, 0x00718158, 0x00718158, 0x00728259, 0x0073835A, 0x0073835A, 0x0073835A,
        0x0073835A, 0x0073835A, 0x0073835A, 0x0073835A, 0x00728259, 0x00718158, 0x00708057, 0x00708259, 0x00708259, 0x00708259, 0x006F8158, 0x006F8158, 0x006F8158, 0x006F8158, 0x006F8158, 0x00708057, 0x00708057, 0x00708057, 0x00708057, 0x00718158,
        0x00718158, 0x00718158, 0x00718158, 0x00728259, 0x00718158, 0x00718158, 0x00708057, 0x00708057, 0x00708057, 0x00708057, 0x00708057, 0x006F7E58, 0x0074845B, 0x00718158, 0x006C7E55, 0x00708358, 0x006E8156, 0x00697E51, 0x006D8255, 0x00434724,
        0x00343C14, 0x002C390D, 0x0036491E, 0x002D4222, 0x00405447, 0x0065777E, 0x005B6C7F, 0x00586C7D, 0x006C8287, 0x006E8577, 0x0051694B, 0x0050653F, 0x00465432, 0x002E341B, 0x004D4D3B, 0x00384235, 0x001D271A, 0x001E281B, 0x001C2619, 0x00212B1E,
        0x001B2518, 0x001B2518, 0x00192316, 0x001D271A, 0x00192316, 0x00202A1D, 0x001F291C, 0x001B2518, 0x002A3427, 0x001A2417, 0x001C2619, 0x001A2418, 0x001C2619, 0x001D2917, 0x00233119, 0x002D3D1E, 0x00495B36, 0x0052663C, 0x00455A2D, 0x00525E40,
        0x00495733, 0x00617247, 0x0076835D, 0x00707B5B, 0x005D6645, 0x00626F41, 0x005B6D32, 0x00566035, 0x004A5527, 0x003D4919, 0x00354111, 0x00323D0F, 0x00384218, 0x004E5532, 0x00646B4A, 0x0065765B, 0x0065765B, 0x0064755A, 0x00637459, 0x0064755A,
        0x005F7055, 0x004F6045, 0x00405136, 0x00344022, 0x00414D2F, 0x00394527, 0x003B4A2B, 0x00556445, 0x00617251, 0x0070815F, 0x006A7B59, 0x006F7E5F, 0x006F7F5D, 0x006F805B, 0x0072805E, 0x00606C50, 0xFFCFD4C5, 0xFFFDFEFC, 0xFFFEFCFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xFFFFFFFE, 0x00BAC1AC, 0x00586839, 0x0078875B, 0x00838E6E, 0x0066724E, 0x005E7138, 0x0063734A, 0x0064744B, 0x0063734A, 0x005F6F46, 0x005E6E45,
        0x0067774E, 0x00607047, 0x00627249, 0x00708565, 0x004F6247, 0x003D4D3B, 0x0069786A, 0x0049584A, 0x00354631, 0x0033462B, 0x003B502F, 0x00364236, 0x002E3A2E, 0x00354234, 0x00485442, 0x004A5840, 0x00455434, 0x00697C51, 0x00748859, 0x0076895E,
        0x0074865D, 0x0073855C, 0x00708259, 0x00758760, 0x006D7F58, 0x006F805B, 0x00768762, 0x0076855F, 0x00788761, 0x0074835D, 0x005E6D47, 0x0052613B, 0x004F5E38, 0x0054633D, 0x0072815B, 0x0074835D, 0x006F7E58, 0x0075845E, 0x006E7D57, 0x007B8A64,
        0x00677650, 0x00495832, 0x00485731, 0x00424D25, 0x00364119, 0x0065734A, 0x00627047, 0x002C3C13, 0x0047572E, 0x0063754C, 0x003D4F26, 0x002C4016, 0x00283C12, 0x00475B31, 0x00586B44, 0x0051643D, 0x0041532E, 0x005F714C, 0x00485A35, 0x003A4C27,
        0x004C5E39, 0x0051643D, 0x004F6339, 0x00394D23, 0x005A6E43, 0x005B7043, 0x00455A2D, 0x0055673E, 0x0055673E, 0x0042562C, 0x00455831, 0x00566B44, 0x00536842, 0x003A512B, 0x00233A14, 0x002C4016, 0x004D6137, 0x0054683E, 0x0072865C, 0x006A7E54,
        0x0070845A, 0x0072865C, 0x006F8359, 0x0070825B, 0x0071835C, 0x0071835C, 0x0072845D, 0x0072845D, 0x0073855E, 0x0073855E, 0x0073855E, 0x00758458, 0x00748357, 0x00748258, 0x00738157, 0x00738158, 0x00748259, 0x00748259, 0x0074815B, 0x00728259,
        0x00728259, 0x0073835A, 0x0074845B, 0x0074845B, 0x0073835A, 0x00728259, 0x00728259, 0x0074855A, 0x0074855A, 0x00738459, 0x00728358, 0x00728358, 0x00728358, 0x00738459, 0x00738459, 0x006B7D54, 0x006D7F56, 0x006E8057, 0x00708259, 0x0071835A,
        0x00708259, 0x006F8158, 0x006F8158, 0x006E8156, 0x006E8156, 0x006F8257, 0x00708358, 0x00708358, 0x006F8257, 0x006E8156, 0x006D8055, 0x0070825B, 0x0071835C, 0x0070825B, 0x006D7F58, 0x006E8059, 0x006F815A, 0x006D7F58, 0x00687A53, 0x00343E14,
        0x00374325, 0x00314129, 0x003C4C2D, 0x00627351, 0x005C6F5E, 0x006A859A, 0x006281B8, 0x006F8AAC, 0x006D8896, 0x002E4739, 0x00435A34, 0x004C6033, 0x003C4A28, 0x00262F22, 0x00474F4F, 0x00465141, 0x00434E3E, 0x003A4535, 0x00475242, 0x00333E2E,
        0x00354030, 0x00434E3E, 0x00323D2D, 0x00363F2B, 0x00414A36, 0x0038412D, 0x003F4631, 0x003F4631, 0x003F452E, 0x002C321B, 0x00232912, 0x00272916, 0x00282C19, 0x00242A13, 0x002C3219, 0x00343E20, 0x005F6A49, 0x004C5B35, 0x0051603A, 0x00495E3D,
        0x00465C39, 0x005B6E47, 0x006D7E53, 0x006A7749, 0x00657246, 0x005F6F46, 0x00546740, 0x00596338, 0x004F5B37, 0x00465540, 0x00495B54, 0x00879A9D, 0x009CACB3, 0x00A7B1BB, 0x00A7B0B9, 0x00ADBCBE, 0x00ADBCBE, 0x00ADBCBE, 0x00ACBBBD, 0x00AAB9BB,
        0x00A7B6B8, 0x00A4B3B5, 0x00A2B1B3, 0x00A4B2B8, 0x00A1AFB5, 0x00A4B2B8, 0x00A3B1B7, 0x00A8B6BC, 0x00AFBDC3, 0x00ACBAC0, 0x00AEBCC2, 0x00BCC3C6, 0x007F8C7C, 0x006D7F5A, 0x0076885F, 0x00616D4F, 0xFFCFD3C7, 0xFFFFFFFE, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBFAFC, 0xFFFDFEFC, 0x00BAC1AC, 0x005C6C3D, 0x007B8A5E, 0x007C8767, 0x004E5A36, 0x003A4D14, 0x00314118, 0x00304017, 0x00324219, 0x00314118, 0x0033431A,
        0x002D3D14, 0x002D3D14, 0x0035451C, 0x00667960, 0x0050604E, 0x00616F64, 0x00828D8A, 0x00838E8C, 0x007E8A84, 0x00758377, 0x00778776, 0x006E796F, 0x00869187, 0x00879086, 0x00555F52, 0x003B4832, 0x004A593A, 0x0073855E, 0x0073875A, 0x0071835C,
        0x00788A63, 0x00778962, 0x00798B64, 0x007C8E65, 0x007D8F66, 0x0075875E, 0x0074875C, 0x0074835D, 0x006C7B55, 0x007A8963, 0x00788761, 0x005A6943, 0x004D5C36, 0x00505F39, 0x004F5E38, 0x0073825C, 0x0071805A, 0x007B8A64, 0x00778660, 0x00707F59,
        0x0064734D, 0x00586741, 0x0035441E, 0x002B3B12, 0x0054643B, 0x005C6C43, 0x0033451C, 0x0036481F, 0x005D6F46, 0x0052663C, 0x002B3F15, 0x002B3F15, 0x002E4218, 0x00465932, 0x004F623B, 0x00475934, 0x003B4D28, 0x00586A45, 0x003B4C2A, 0x00283B14,
        0x00485B34, 0x003B4F25, 0x00394D23, 0x0040542A, 0x004B5F35, 0x0032461C, 0x0033471D, 0x00586A41, 0x0045572E, 0x002C3F18, 0x003E512A, 0x005B704A, 0x00556A44, 0x00384F29, 0x00253B18, 0x00364A20, 0x0051653B, 0x0055693F, 0x0074885E, 0x0071855B,
        0x00778B61, 0x0074885E, 0x006D8157, 0x0071835C, 0x0072845D, 0x0072845D, 0x0072845D, 0x0072845D, 0x0072845D, 0x0071835C, 0x0071835C, 0x0075865A, 0x0074855A, 0x0074855A, 0x00738459, 0x00728259, 0x00728259, 0x0072815B, 0x0073825C, 0x00728358,
        0x00728358, 0x00738459, 0x00738459, 0x00738459, 0x00738459, 0x00728358, 0x00728358, 0x00708358, 0x006F8257, 0x006F8257, 0x006F8257, 0x006F8257, 0x00708358, 0x00718459, 0x00718459, 0x00708259, 0x0071835A, 0x0071835A, 0x0071835A, 0x0071835A,
        0x00708259, 0x006F8158, 0x006E8057, 0x00708259, 0x00708259, 0x00708259, 0x00708259, 0x00708259, 0x006F8158, 0x006F8158, 0x006E8057, 0x0070825B, 0x0071835C, 0x0070825B, 0x006E8059, 0x006F815A, 0x0070825B, 0x006D7F58, 0x00687A53, 0x00464C35,
        0x00687265, 0x00717F74, 0x004D5D46, 0x00455837, 0x003F5640, 0x0063818C, 0x007399C3, 0x007192B3, 0x00708F9E, 0x00436051, 0x005A734D, 0x0065794C, 0x00586642, 0x002E3827, 0x00434A45, 0x00455040, 0x00333E2E, 0x00343F2F, 0x00404B3B, 0x00323D2D,
        0x003B4636, 0x00414C3C, 0x003D4838, 0x00414A36, 0x00404935, 0x003D4632, 0x00454C37, 0x003A412C, 0x00353A25, 0x00272D16, 0x00292F18, 0x002C2F1F, 0x00292D1A, 0x0030361F, 0x00232C12, 0x004A5436, 0x0053613F, 0x004B5935, 0x004C5B35, 0x004E6342,
        0x004B603A, 0x005A6A41, 0x00677448, 0x00657042, 0x00606D3F, 0x00606F43, 0x0057683D, 0x005C653A, 0x005D6945, 0x006C7963, 0x0075867D, 0x00B7C6C8, 0x00D4E2E8, 0x00DFE9F0, 0x00E4EDF1, 0x00DAE6E8, 0x00DBE7E9, 0x00DBE7E9, 0x00DDE9EB, 0x00DDE9EB,
        0x00DDE9EB, 0x00DCE8EA, 0x00DBE7E9, 0x00DDE8EC, 0x00D9E4E8, 0x00DDE8EC, 0x00DDE8EC, 0x00DFEAEE, 0x00E2EDF1, 0x00D9E4E8, 0x00D7E2E6, 0x00D9E0E3, 0x008F9C8C, 0x006E805B, 0x0072845B, 0x005F6B4D, 0xFFD0D4C8, 0xFFFFFFFE, 0xFFFFFDFD, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xFFFEFFFD, 0x00B9C0AB, 0x005B6B3C, 0x007C8B5F, 0x007C8767, 0x004A5632, 0x0032450C, 0x00526239, 0x0055653C, 0x0047572E, 0x0034441B, 0x0056663D,
        0x0048582F, 0x0034441B, 0x003E4E25, 0x00889782, 0x00505D4F, 0x005E6764, 0x00959BA2, 0x00D4D8E3, 0x00DEE2ED, 0x00D0D6DD, 0x00D3D9DE, 0x00DFE8E5, 0x00E2E9E6, 0x00C7CEC9, 0x00717A70, 0x00505A4A, 0x004E5B43, 0x00647250, 0x0064734D, 0x006C795F,
        0x005D6B4F, 0x00485738, 0x00556444, 0x00556641, 0x00586A43, 0x004F6138, 0x005B6E43, 0x0075855C, 0x0077875E, 0x0075855C, 0x0078885F, 0x00718158, 0x005E6E45, 0x0054643B, 0x00516138, 0x00586741, 0x00788761, 0x0073825C, 0x0071805A, 0x007E8D67,
        0x0065744E, 0x00505F39, 0x003F4E28, 0x00374922, 0x0061734C, 0x00455831, 0x0023360F, 0x004E633C, 0x005B7049, 0x002E461E, 0x00233B13, 0x002F4319, 0x002C3F18, 0x00384B24, 0x004B5E37, 0x0052643F, 0x00475934, 0x00576846, 0x00314220, 0x002C4015,
        0x0043572C, 0x004C6036, 0x0051643D, 0x00465932, 0x00566843, 0x00516240, 0x004B5C3A, 0x005A6C43, 0x00465831, 0x002E411A, 0x003E512A, 0x00576C46, 0x004F643E, 0x00384E2B, 0x00283E1B, 0x002A3D16, 0x004A5D36, 0x00576A43, 0x0073865F, 0x006D8059,
        0x0071845D, 0x0073865F, 0x00748760, 0x0071835C, 0x0071835C, 0x0072845D, 0x0072845D, 0x0071835C, 0x0070825B, 0x006F815A, 0x006E8059, 0x0075865B, 0x0074855A, 0x00738459, 0x00728259, 0x00728259, 0x0071805A, 0x0071805A, 0x0072815B, 0x006E8255,
        0x006E8255, 0x006F8356, 0x006F8356, 0x006F8356, 0x006F8356, 0x006E8255, 0x006E8255, 0x006E8156, 0x006E8156, 0x006E8156, 0x006E8156, 0x006F8257, 0x00708358, 0x00718459, 0x00718459, 0x0072845B, 0x0072845B, 0x0071835A, 0x0071835A, 0x00708259,
        0x006F8158, 0x006F8158, 0x006E8057, 0x0072845B, 0x0072845B, 0x0071835A, 0x00708259, 0x00708259, 0x00708259, 0x00708259, 0x00708259, 0x0070825B, 0x0070825B, 0x006F815A, 0x006E8059, 0x0070825B, 0x0072845D, 0x006E8059, 0x00687A53, 0x003C403B,
        0x00939A9D, 0x00AEBABE, 0x005B6C5F, 0x003C4E31, 0x004B644A, 0x00729696, 0x00749FBA, 0x007298BA, 0x00779BAB, 0x00527365, 0x00647E5A, 0x00667B4E, 0x00586741, 0x002E3821, 0x00444D40, 0x00353F2F, 0x00273121, 0x002D3727, 0x002C3626, 0x00283222,
        0x00333D2D, 0x002B3525, 0x00353F2F, 0x00353D2C, 0x00333B2A, 0x00363F2B, 0x00414734, 0x00363D28, 0x00373C27, 0x002C311C, 0x002B311A, 0x00252818, 0x002B2F1C, 0x00222914, 0x002C351B, 0x00343E20, 0x0052603E, 0x0045532F, 0x0055643E, 0x004A5F3E,
        0x004D5F38, 0x005D673C, 0x005C6637, 0x00535E30, 0x004E5B2F, 0x00556236, 0x00566035, 0x004C562B, 0x004E5938, 0x006A7660, 0x00717C72, 0x00B3BDBD, 0x00D7E0E3, 0x00D8E0E0, 0x00D4DCDC, 0x00DDE4E7, 0x00DCE3E6, 0x00DAE1E4, 0x00DAE1E4, 0x00DBE2E5,
        0x00DAE1E4, 0x00D8DFE2, 0x00D6DDE0, 0x00D9E1E1, 0x00D9E1E1, 0x00DFE7E7, 0x00DCE4E4, 0x00D9E1E1, 0x00DAE2E2, 0x00D8E0E0, 0x00DEE6E6, 0x00E4EBEE, 0x0094A191, 0x006E805B, 0x00708259, 0x005F6B4D, 0xFFD3D7CB, 0xFFFFFFFE, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xFFFFFFFE, 0x00B9C0AB, 0x005A6A3B, 0x007B8A5E, 0x007E8969, 0x00515D39, 0x003D5017, 0x0068784F, 0x005E6E45, 0x00607047, 0x003B4B22, 0x00617148,
        0x0065754C, 0x0048582F, 0x00405027, 0x007C8E71, 0x0051604B, 0x00515C54, 0x009FA5AA, 0x00DBDEEC, 0x00CFD2E1, 0x00C1C4D2, 0x00C8CCD7, 0x00CACFD0, 0x00DADFE0, 0x00C9CFCE, 0x00636964, 0x00474F44, 0x00434C38, 0x00445034, 0x003F4D2B, 0x006E736A,
        0x005B6057, 0x00363D30, 0x003E4635, 0x0039432C, 0x00475337, 0x003C4929, 0x00475533, 0x006E7E55, 0x0078885F, 0x006B7B52, 0x00708057, 0x007D8D64, 0x006F7F56, 0x005D6D44, 0x00526239, 0x00586741, 0x005E6D47, 0x006D7C56, 0x007F8E68, 0x00788761,
        0x005E6D47, 0x004F5E38, 0x003C4B25, 0x00445932, 0x00485D36, 0x002D421B, 0x002C441C, 0x00506840, 0x00415931, 0x001B350D, 0x0028421A, 0x00293C15, 0x002B3E17, 0x003A4D26, 0x0052643F, 0x00536540, 0x0040512F, 0x00586947, 0x004E5F3D, 0x00384C21,
        0x003F5328, 0x0055693F, 0x00566942, 0x0032441F, 0x004D5E3D, 0x005E6F4E, 0x00455536, 0x0051633C, 0x0054663F, 0x00455831, 0x003F512C, 0x00425731, 0x003E522F, 0x00334926, 0x00253B18, 0x002D4019, 0x004C5F38, 0x005E714A, 0x00748760, 0x006C7F58,
        0x006B7E57, 0x006F825B, 0x0073865F, 0x006F815A, 0x0070825B, 0x0071835C, 0x0071835C, 0x0071835C, 0x0070825B, 0x006F815A, 0x006E8059, 0x00718459, 0x00718459, 0x00708259, 0x006F8158, 0x006F815A, 0x006F815A, 0x006F815A, 0x006F805B, 0x006D8255,
        0x006E8356, 0x006E8356, 0x006E8356, 0x006E8356, 0x006E8356, 0x006E8356, 0x006D8255, 0x006F8158, 0x006F8158, 0x006F8158, 0x006F8158, 0x00708259, 0x00708259, 0x0071835A, 0x0072845B, 0x006F8158, 0x006F8158, 0x006F8158, 0x006F8158, 0x006F8158,
        0x006F8158, 0x00708259, 0x00708259, 0x0073855C, 0x0072845B, 0x0071835A, 0x00708259, 0x00708259, 0x00708259, 0x00708259, 0x0071835A, 0x0070825B, 0x0070825B, 0x006F815A, 0x006F815A, 0x0071835C, 0x0073855E, 0x006F815A, 0x00687A53, 0x00393E3C,
        0x00616C70, 0x0069767E, 0x0045544C, 0x004B5E43, 0x00556E54, 0x00769897, 0x00759FB6, 0x006B97B6, 0x00749BA9, 0x0046685D, 0x005B7654, 0x0062774A, 0x004F5F36, 0x002E3A1C, 0x004D5642, 0x003C4636, 0x003D4737, 0x00424C3C, 0x00353F2F, 0x00384232,
        0x00404A3A, 0x00313B2B, 0x003D4737, 0x00293120, 0x00363E2D, 0x00313928, 0x003C422F, 0x003B412E, 0x00424732, 0x00393E29, 0x00252A15, 0x002E3121, 0x00212716, 0x002E3520, 0x00262F15, 0x003C482C, 0x004C5939, 0x00586944, 0x00546540, 0x004A5D3C,
        0x0056643B, 0x0063693A, 0x00505829, 0x003D491F, 0x0030411C, 0x003C4B25, 0x00434F25, 0x00435128, 0x00424D2D, 0x0067705C, 0x0071796F, 0x00B5BBBA, 0x00D8DEDD, 0x00BCC3C0, 0x00A4ABA6, 0x009CA0A1, 0x00989C9D, 0x00949899, 0x00929697, 0x00929697,
        0x00919596, 0x008E9293, 0x008B8F90, 0x009BA09E, 0x009FA4A2, 0x00A8ADAB, 0x00A1A6A4, 0x00979C9A, 0x009DA2A0, 0x00ADB2B0, 0x00C5CAC8, 0x00DCE3E6, 0x008F9C8C, 0x006D7F5A, 0x0073855C, 0x00616D4F, 0xFFD2D6CA, 0xFFFFFFFE, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFBFD, 0xFFFEFFFD, 0x00BBC2AD, 0x005C6C3D, 0x007A895D, 0x007B8666, 0x004D5935, 0x003A4D14, 0x00596940, 0x00394920, 0x00607047, 0x00425229, 0x004B5B32,
        0x005C6C43, 0x0047572E, 0x00304017, 0x00687B54, 0x00677758, 0x005B6755, 0x00AFB6B3, 0x00DADEE3, 0x00C6C9D1, 0x00BEC2C7, 0x00B8BCC1, 0x00CACFD2, 0x00D2D7DA, 0x00CED2D3, 0x00818684, 0x0081867D, 0x00666E5D, 0x005C654B, 0x00677252, 0x00919296,
        0x00848687, 0x00545656, 0x004F534D, 0x005C6257, 0x00818879, 0x005D6652, 0x0037402B, 0x0074845B, 0x0073835A, 0x00728259, 0x0076865D, 0x006F7F56, 0x00718158, 0x0075855C, 0x00596940, 0x00596842, 0x004B5A34, 0x005E6D47, 0x0076855F, 0x00778660,
        0x006A7953, 0x004F5E38, 0x0042512B, 0x002A3F19, 0x00283D17, 0x0020350F, 0x00233A14, 0x002E451F, 0x00283F19, 0x001E3711, 0x00223B15, 0x002A3D16, 0x00283A15, 0x00293B16, 0x0032441F, 0x002F401E, 0x0020310F, 0x00344524, 0x00364726, 0x002B3E17,
        0x0032451E, 0x0032441F, 0x002E401B, 0x00253614, 0x00334423, 0x00334423, 0x00273718, 0x0030421B, 0x0041522D, 0x003B4D28, 0x0030421D, 0x002B3F1C, 0x002D411E, 0x002D4221, 0x00243918, 0x003A4C27, 0x004B5D38, 0x00596B46, 0x006E805B, 0x006F815C,
        0x0070825D, 0x006D7F5A, 0x006D7F5A, 0x006F815A, 0x0070825B, 0x0070825B, 0x0070825B, 0x0070825B, 0x0070825B, 0x006F815A, 0x006E8059, 0x006E8258, 0x006E8258, 0x006D8157, 0x006D8059, 0x006D8059, 0x006D7F5A, 0x006E805B, 0x006F815C, 0x006C8357,
        0x006C8357, 0x006C8357, 0x006C8357, 0x006C8357, 0x006C8357, 0x006C8357, 0x006C8357, 0x006E8258, 0x006E8258, 0x006E8258, 0x006D8157, 0x006E8258, 0x006E8258, 0x006F8359, 0x006F8359, 0x006D7F56, 0x006D7F56, 0x006D7F56, 0x006D7F56, 0x006E8057,
        0x006F8158, 0x00708259, 0x00708259, 0x0072845D, 0x0071835C, 0x0070825B, 0x006F815A, 0x006F815A, 0x006F815A, 0x0070825B, 0x0070825B, 0x006F815A, 0x0070825B, 0x006F815A, 0x006F815A, 0x0070825B, 0x0072845D, 0x006F815A, 0x006A7C55, 0x00434F3B,
        0x006E7D75, 0x00778584, 0x005C6A5F, 0x004B5941, 0x00314630, 0x00658286, 0x0081A6C2, 0x007099B0, 0x007398A0, 0x00385649, 0x005B7452, 0x006D8055, 0x00546239, 0x00313D1F, 0x004B543F, 0x003D4737, 0x00313B2B, 0x00364030, 0x00323C2C, 0x00313B2B,
        0x00353F2F, 0x00323C2C, 0x00323C2C, 0x00282F20, 0x003E4536, 0x00303827, 0x003A402F, 0x003B412E, 0x003F4330, 0x00393E29, 0x00232813, 0x0024291A, 0x001E2413, 0x00222815, 0x001C260F, 0x003A462A, 0x00485737, 0x004C5C3A, 0x0041522D, 0x00596B4C,
        0x00667046, 0x00666A39, 0x00454C21, 0x00314324, 0x00243D29, 0x00304830, 0x0035492C, 0x00324321, 0x002A371D, 0x00566050, 0x006A706B, 0x00B3B8B7, 0x00D6DCDB, 0x00B2B9B4, 0x008E9991, 0x00989C9D, 0x0095999A, 0x00929697, 0x00929697, 0x0095999A,
        0x0095999A, 0x00939798, 0x00919596, 0x00939896, 0x00959A98, 0x009CA19F, 0x00929795, 0x00898E8C, 0x00939896, 0x00AFB4B2, 0x00D1D6D4, 0x00DBE2E5, 0x008E9B8B, 0x006E805B, 0x0075875E, 0x00647052, 0xFFD2D6CA, 0xFFFEFFFD, 0xFFFFFEFE, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBFAFC, 0xFFFDFEFC, 0x00BBC2AD, 0x005C6C3D, 0x007A895D, 0x007A8565, 0x004E5A36, 0x003B4E15, 0x005F6F46, 0x00495930, 0x005E6E45, 0x004B5B32, 0x005F6F46,
        0x005C6C43, 0x0035451C, 0x00304017, 0x0063774C, 0x005D6E4C, 0x0053604A, 0x00A3ACA2, 0x00DCE2E1, 0x00CDD2D3, 0x00C8CECD, 0x00BDC4C1, 0x00CCCFD3, 0x00CDD0D4, 0x00D3D7D8, 0x00A8ABA9, 0x00B9BEB5, 0x00787E6B, 0x00535C41, 0x00636E4E, 0x00939492,
        0x008C8D89, 0x00686A64, 0x0062655C, 0x0073776B, 0x00959A8B, 0x007D8372, 0x005D6350, 0x00697A4F, 0x0075865B, 0x00798A5F, 0x0074855A, 0x006D7E53, 0x00728358, 0x00798A5F, 0x00728358, 0x00596842, 0x0053623C, 0x00505F39, 0x005A6943, 0x0074835D,
        0x007B8A64, 0x00576640, 0x0054633D, 0x004D5E39, 0x00576843, 0x00556742, 0x0052643F, 0x004E633D, 0x004F643E, 0x00536A44, 0x00526943, 0x005B6D48, 0x00556742, 0x004E603B, 0x00475836, 0x004D5E3C, 0x004D5E3D, 0x00455635, 0x00364726, 0x0039492A,
        0x0039492A, 0x003A4A2B, 0x00485938, 0x004E5F3D, 0x004B5C3A, 0x00465735, 0x004D5F3A, 0x0051623D, 0x00566742, 0x0050623D, 0x00526341, 0x00526643, 0x004F6241, 0x004F6443, 0x004A5F3E, 0x00495B36, 0x00546641, 0x0061734E, 0x006F815C, 0x006F815C,
        0x006E805B, 0x006B7D58, 0x006C7E59, 0x006E8059, 0x006D7F58, 0x006B7D56, 0x006A7C55, 0x00687A53, 0x00667851, 0x00657750, 0x0064764F, 0x0061774D, 0x0061774D, 0x0060754E, 0x0060754E, 0x0060754F, 0x0060754F, 0x00617650, 0x00617552, 0x005F774F,
        0x005F774F, 0x005E764E, 0x005E764E, 0x005E764E, 0x005E764E, 0x005F774F, 0x005F774F, 0x0061754B, 0x0060744A, 0x0060744A, 0x0060744A, 0x0061754B, 0x0062764C, 0x0063774D, 0x0064784E, 0x0065774E, 0x0066784F, 0x00677950, 0x00697B52, 0x006A7C53,
        0x006B7D54, 0x006C7E55, 0x006C7E55, 0x006E8059, 0x006E8059, 0x006E8059, 0x006E8059, 0x006E8059, 0x006E8059, 0x006E8059, 0x006F815A, 0x006F815A, 0x0070825B, 0x0070825B, 0x006E8059, 0x006E8059, 0x0070825B, 0x006F815A, 0x006C7E57, 0x005D6F48,
        0x0099A991, 0x00A6B7AA, 0x007E8A78, 0x005A664A, 0x003D4D35, 0x00708888, 0x008BABC2, 0x007E9FAE, 0x00789697, 0x00445B4C, 0x00647756, 0x0064744B, 0x0046522E, 0x00283418, 0x00424D39, 0x003A4132, 0x00171E0F, 0x001D2415, 0x0022291A, 0x001D2415,
        0x001F2617, 0x0022291A, 0x001C2314, 0x00171E0F, 0x0022291A, 0x001F2617, 0x00292F1E, 0x00262C1B, 0x002E321F, 0x002C301D, 0x00272B18, 0x0024291A, 0x00333A2B, 0x001F2814, 0x002E3821, 0x002F3D21, 0x00526142, 0x00485937, 0x005A6B49, 0x0065785D,
        0x006A734B, 0x005F6231, 0x003A451F, 0x00375445, 0x00345C5B, 0x003E6563, 0x003E6056, 0x002F452C, 0x00283826, 0x005F6A60, 0x00808584, 0x00C5C8CC, 0x00DEE3E4, 0x00B4BFBC, 0x008E9D95, 0x00949A99, 0x00929897, 0x00929897, 0x00939998, 0x00959B9A,
        0x00979D9C, 0x00959B9A, 0x00949A99, 0x00979E9B, 0x00939A97, 0x00959C99, 0x008D9491, 0x00888F8C, 0x00949B98, 0x00ADB4B1, 0x00CDD4D1, 0x00E1E8EB, 0x008E9B8B, 0x006A7C57, 0x0074865D, 0x00667254, 0xFFD5D9CD, 0xFFFFFFFE, 0xFFFFFEFE, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xFFFFFFFE, 0x00B9C0AB, 0x005A6A3B, 0x007A895D, 0x007D8868, 0x00515D39, 0x003D5017, 0x0066764D, 0x0068784F, 0x0055653C, 0x004E5E35, 0x006E7E55,
        0x005F6F46, 0x0037471E, 0x003D4D24, 0x00687757, 0x005A674D, 0x00868F82, 0x00A4A9A7, 0x00E1E5E6, 0x00C9CCD0, 0x00C1C6C5, 0x00C0C5C3, 0x00BEC2C3, 0x00DCE0E1, 0x00D0D2D2, 0x0070746F, 0x0084887D, 0x00616853, 0x00576143, 0x00616C4B, 0x00525B41,
        0x005B644A, 0x00596248, 0x00565F44, 0x00535C41, 0x00576045, 0x005F694B, 0x006D7759, 0x00495A2F, 0x0064754A, 0x00718257, 0x00718257, 0x0078895E, 0x00728358, 0x0068794E, 0x0075865B, 0x0074835D, 0x005B6A44, 0x0056653F, 0x005C6B45, 0x0056653F,
        0x00707F59, 0x0072815B, 0x005A6943, 0x00515F3D, 0x00576543, 0x0053613F, 0x0051613F, 0x004F5F3D, 0x004B5B39, 0x004F603E, 0x004F603E, 0x00455732, 0x004C5E39, 0x00556644, 0x00495A38, 0x004B5C3B, 0x004B5C3B, 0x002E3F1E, 0x00223213, 0x001F2B17,
        0x0014210B, 0x00253419, 0x00455536, 0x004B5C3A, 0x00485A35, 0x004C6036, 0x00506439, 0x004E5F3A, 0x004D5E39, 0x004A5B39, 0x00536442, 0x00526544, 0x004B5E3D, 0x004B603F, 0x004B6040, 0x00516240, 0x00586947, 0x00647553, 0x006C7D5B, 0x006A7B59,
        0x00667755, 0x00647553, 0x00697A58, 0x00677952, 0x0064764F, 0x0060724B, 0x005A6C45, 0x00556740, 0x0050623B, 0x004E6039, 0x004D5F38, 0x00465E34, 0x00455D35, 0x00445C34, 0x00435B33, 0x00425933, 0x00415832, 0x00415734, 0x00415734, 0x003E5634,
        0x003E5634, 0x003D5533, 0x003C5432, 0x003C5432, 0x003D5533, 0x003E5634, 0x003E5634, 0x0041562F, 0x0041562F, 0x00425730, 0x00435831, 0x00455A33, 0x00475C35, 0x00485D36, 0x00495E37, 0x0052643B, 0x0054663D, 0x00586A41, 0x005C6E45, 0x00607249,
        0x0064764D, 0x0066784F, 0x00677950, 0x006A7B56, 0x006A7B56, 0x006B7C57, 0x006C7D58, 0x006D7E59, 0x006D7E59, 0x006D7E59, 0x006D7E59, 0x006E8059, 0x0070825B, 0x0070825B, 0x006D7F58, 0x006C7E57, 0x006E8059, 0x006F815A, 0x006E8059, 0x00455526,
        0x003B4B2C, 0x00404E3C, 0x00444E37, 0x00656D48, 0x005B6945, 0x005F7668, 0x0066858E, 0x00789195, 0x00667C77, 0x0051624D, 0x00687555, 0x00505B35, 0x00404B2B, 0x002A371F, 0x00475444, 0x003E4536, 0x002E3526, 0x00394031, 0x00282F20, 0x00333A2B,
        0x00383F30, 0x00282F20, 0x00343B2C, 0x002E3526, 0x0022291A, 0x003A4132, 0x00393E2F, 0x002C3221, 0x00494C3C, 0x003E422F, 0x003C402D, 0x00363D30, 0x00303728, 0x002E3625, 0x00303C26, 0x00354228, 0x00435334, 0x005C6D4C, 0x006D7E5C, 0x0065785F,
        0x00677048, 0x005A5D30, 0x00364529, 0x00446A6A, 0x00467E91, 0x004F8498, 0x00477882, 0x00334F3C, 0x002D4337, 0x006C7977, 0x009CA0A5, 0x00D9DCE4, 0x00DEE4E9, 0x00ACBAB8, 0x0080928B, 0x008D9594, 0x008D9594, 0x008E9695, 0x008F9796, 0x008F9796,
        0x008F9796, 0x008E9695, 0x008D9594, 0x00929A9A, 0x008D9595, 0x008D9595, 0x00878F8F, 0x00838B8B, 0x008F9797, 0x00A9B1B1, 0x00C8D0D0, 0x00DEE5E8, 0x008D9A8A, 0x006A7C57, 0x0074865D, 0x00647052, 0xFFD3D7CB, 0xFFFFFFFE, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xFFFFFFFE, 0x00B7BEA9, 0x0059693A, 0x007B8A5E, 0x007C8767, 0x00495531, 0x0030430A, 0x0036461D, 0x003C4C23, 0x00293910, 0x0033431A, 0x0035451C,
        0x0036461D, 0x003D4D24, 0x002D3D14, 0x0069755F, 0x005F6959, 0x00BEC4BF, 0x009FA2A6, 0x00E8EAF2, 0x00E3E5ED, 0x00D8DBDF, 0x00D3D8D7, 0x00DCE1E0, 0x00F0F5F4, 0x00CED1CF, 0x00686C66, 0x008C9084, 0x00757D66, 0x00707B5B, 0x0076805C, 0x006D7D4E,
        0x006F7F50, 0x00707F53, 0x006D7C50, 0x00738157, 0x0068764C, 0x005B693F, 0x00505E35, 0x00536439, 0x00516237, 0x006B7C51, 0x0077885D, 0x00728358, 0x00728358, 0x00708156, 0x00738459, 0x0074835D, 0x0071805A, 0x0055643E, 0x0054633D, 0x0054633D,
        0x0064734D, 0x0076855F, 0x007A8963, 0x00798261, 0x007A8362, 0x00737E5D, 0x00778261, 0x00768462, 0x00748260, 0x00768664, 0x006E7E5C, 0x0072845F, 0x006B7C5A, 0x0070815F, 0x00657654, 0x00687958, 0x005F704F, 0x002D3D1E, 0x00203011, 0x001A2417,
        0x0015210F, 0x00212E18, 0x00455438, 0x00667755, 0x006E8258, 0x006B8053, 0x006C8252, 0x006B7C57, 0x006E7E5C, 0x00687957, 0x00617250, 0x004A5D3C, 0x00344726, 0x00304525, 0x002F4424, 0x00364725, 0x00354624, 0x003F502E, 0x004E5F3D, 0x005C6D4B,
        0x00627351, 0x00617250, 0x00627351, 0x0060724B, 0x005B6D46, 0x0054663F, 0x004B5D36, 0x0043552E, 0x003D4F28, 0x00394B24, 0x00374922, 0x002E461E, 0x002C441C, 0x002B431B, 0x00283F19, 0x00263D17, 0x00253B18, 0x00243A17, 0x00243A17, 0x00203717,
        0x00203717, 0x001F3616, 0x001E3515, 0x001E3515, 0x001F3616, 0x00203717, 0x00203717, 0x00263B14, 0x00273C15, 0x00283D16, 0x002A3F18, 0x002D421B, 0x0030451E, 0x00334821, 0x00344922, 0x003E5027, 0x0041532A, 0x00485A31, 0x00506239, 0x00576940,
        0x005D6F46, 0x0061734A, 0x0063754C, 0x00677853, 0x00687954, 0x006A7B56, 0x006B7C57, 0x006C7D58, 0x006C7D58, 0x006C7D58, 0x006C7D58, 0x006D7F58, 0x0070825B, 0x0070825B, 0x006C7E57, 0x006A7C55, 0x006C7E57, 0x006F815A, 0x0070825B, 0x004E5B2D,
        0x004C5939, 0x00646F5B, 0x004E573C, 0x005C653A, 0x004B582A, 0x00243B1F, 0x00163730, 0x005D6F6E, 0x00405045, 0x004D5A42, 0x00697251, 0x004E5532, 0x00586244, 0x00434F3B, 0x0059685A, 0x00232A1B, 0x0042493A, 0x00515849, 0x00212819, 0x00454C3D,
        0x00515849, 0x00272E1F, 0x004C5344, 0x00434A3D, 0x00252C1D, 0x00505748, 0x003B4031, 0x00232918, 0x00555848, 0x00363A27, 0x00272B18, 0x005B6255, 0x003A4132, 0x002B3322, 0x0057634D, 0x00354228, 0x005D6D4E, 0x006A7B5A, 0x006C7D5B, 0x0062745D,
        0x00676F4A, 0x005B5F35, 0x00394C33, 0x0049777F, 0x004888A7, 0x004B8AAC, 0x00417A90, 0x00426152, 0x00304840, 0x00647173, 0x0091969F, 0x00CDD1DC, 0x00D1D9E0, 0x00AABAB9, 0x00819890, 0x0085908E, 0x00879290, 0x00899492, 0x008B9694, 0x008B9694,
        0x008B9694, 0x008A9593, 0x008A9593, 0x00899295, 0x00858E91, 0x00889194, 0x00818A8D, 0x007C8588, 0x00899295, 0x00A7B0B3, 0x00CCD5D8, 0x00D9E0E3, 0x008E9B8B, 0x0070825D, 0x00778960, 0x00606C4E, 0xFFCACEC2, 0xFFFAFBF9, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCF5FF, 0xFFFFFFF8, 0x00BEC6A7, 0x0058673B, 0x00808E64, 0x0071805A, 0x00596940, 0x00425327, 0x003E5027, 0x003F5128, 0x00405229, 0x0042542B, 0x0041532A,
        0x003F5128, 0x0042542B, 0x0041532A, 0x00667157, 0x00656F58, 0x007B856E, 0x00929B86, 0x00A6AF9A, 0x00969F8A, 0x0099A38C, 0x00ADB7A0, 0x00A1A893, 0x009DA58E, 0x0097A085, 0x00667151, 0x005D6945, 0x0066744B, 0x00738256, 0x0078875B, 0x00768958,
        0x00728455, 0x00738556, 0x00738459, 0x00708057, 0x0073825C, 0x00707E5A, 0x0064724E, 0x00495930, 0x00526239, 0x004E5E35, 0x006B7B52, 0x0074845B, 0x0076865D, 0x0076865D, 0x00728259, 0x00718257, 0x00728358, 0x0074855A, 0x00596940, 0x00596940,
        0x00506037, 0x005F6E48, 0x00778660, 0x0071835C, 0x006F815A, 0x006E8059, 0x006E8059, 0x0070825B, 0x0071835C, 0x0072845D, 0x0071835C, 0x00737F5B, 0x00707F59, 0x00697B52, 0x00677B51, 0x00657851, 0x00566442, 0x00272F11, 0x0024280F, 0x001A200F,
        0x00262D18, 0x002D361B, 0x00374424, 0x00687652, 0x00697A55, 0x006A7A58, 0x006D7D5B, 0x00778058, 0x00596241, 0x00525F47, 0x004F5E50, 0x0044564F, 0x00354B49, 0x00233D3D, 0x00264242, 0x00173E4D, 0x000F2E37, 0x00213633, 0x00293725, 0x003B4527,
        0x003E4C28, 0x004C5F38, 0x005F774F, 0x00577149, 0x0053693F, 0x004C5D31, 0x00495525, 0x0043501C, 0x00394D18, 0x00314B15, 0x002B4914, 0x00273D20, 0x0021371B, 0x001B2F18, 0x00182B18, 0x0018291B, 0x0018281D, 0x0015241C, 0x0013211B, 0x001A1E19,
        0x001A1E19, 0x00191D18, 0x00191D18, 0x001A1E19, 0x001B1F1A, 0x001C201B, 0x001D211C, 0x001B2B1A, 0x001A2A18, 0x00192D16, 0x001D3317, 0x001E3715, 0x001F3A13, 0x00254116, 0x002B4A1D, 0x00384B12, 0x003A4D14, 0x003D5017, 0x00465922, 0x00536431,
        0x005F6D43, 0x00636F4B, 0x00636D4F, 0x00667752, 0x00687954, 0x00697A55, 0x006A7B56, 0x006A7B56, 0x006B7C57, 0x006D7E59, 0x006F805B, 0x006E8059, 0x006E8059, 0x006E8059, 0x006E8059, 0x006E8059, 0x006D7F58, 0x006D7F58, 0x006D7F58, 0x006E7F5A,
        0x006E7E5C, 0x00829270, 0x007D8D6B, 0x00697858, 0x003A4929, 0x00253415, 0x00334223, 0x003E4B31, 0x00384727, 0x00485A33, 0x005E7146, 0x004F603B, 0x00465339, 0x004A5346, 0x00888E89, 0x00192D1A, 0x00506451, 0x00687C69, 0x001B2F1C, 0x005C705D,
        0x00627663, 0x00172B18, 0x00627663, 0x005E6D58, 0x0023311F, 0x00768472, 0x00505E4C, 0x001C2919, 0x00818E7E, 0x00455244, 0x00233022, 0x00818C7C, 0x00374331, 0x00313E28, 0x0078876C, 0x004A5A3B, 0x006C7D5B, 0x00677954, 0x006B7E57, 0x00718260,
        0x005D6B47, 0x003B4C2B, 0x00304B3B, 0x00557F84, 0x00558A9F, 0x004B839C, 0x0052869D, 0x003E6C74, 0x00273F33, 0x00505945, 0x00727D7A, 0x009AA8AE, 0x00DDE5E5, 0x00B9C0BD, 0x00849094, 0x008A9390, 0x0087908D, 0x00848D8A, 0x00868F8C, 0x008A9390,
        0x008D9693, 0x008C9592, 0x008A9390, 0x0087988F, 0x0086978E, 0x00879790, 0x007F8D87, 0x00778581, 0x00879290, 0x00AAB5B3, 0x00CAD5D3, 0x00DEE4DF, 0x00848E7E, 0x00667556, 0x0075855C, 0x0062714B, 0xFFC5CDB6, 0xFFFDFEFA, 0xFFFEFAFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFAFF, 0xFFFFFFF9, 0x00B9C1A2, 0x00536236, 0x007D8B61, 0x00788761, 0x0077875E, 0x0076875B, 0x00778960, 0x00798B62, 0x00788A61, 0x0076885F, 0x0075875E,
        0x0076885F, 0x0075875E, 0x00708259, 0x00798763, 0x0073815F, 0x00717F5D, 0x006C7959, 0x0073815F, 0x006B7955, 0x006F7E58, 0x007A8A61, 0x00717A5F, 0x00677153, 0x00707B5B, 0x00687450, 0x006A7751, 0x006A784E, 0x00718054, 0x007B8A5E, 0x00738655,
        0x00738655, 0x00768859, 0x0076875B, 0x00728358, 0x0077875E, 0x00798960, 0x0074835D, 0x00617148, 0x004E5E35, 0x005A6A41, 0x0048582F, 0x006E7E55, 0x0077875E, 0x006F7F56, 0x0073835A, 0x0074855A, 0x00738459, 0x00798A5F, 0x0068784F, 0x0056663D,
        0x00506037, 0x005F6E48, 0x0076855F, 0x0072845D, 0x0070825B, 0x006F815A, 0x006F815A, 0x0070825B, 0x0071835C, 0x0071835C, 0x0071835C, 0x0076825E, 0x0072815B, 0x006B7D54, 0x006A7E54, 0x00657851, 0x00586644, 0x00384022, 0x00454930, 0x003D4332,
        0x00313823, 0x001F280D, 0x004A5737, 0x00697753, 0x0062734E, 0x00667654, 0x00687856, 0x006A7C53, 0x00556644, 0x003B4E39, 0x004E635B, 0x006F878D, 0x007D97A7, 0x007E9EB1, 0x007799B0, 0x006B90AC, 0x00638297, 0x0060757D, 0x003D4943, 0x00293120,
        0x00232E14, 0x00394829, 0x00526445, 0x00485D3C, 0x00455634, 0x003D4B29, 0x0038421E, 0x00343E1A, 0x002F3D19, 0x002C3D1B, 0x00283E1B, 0x0020331A, 0x001C2E17, 0x00172813, 0x00152514, 0x00162418, 0x0016231B, 0x0014201A, 0x00111C19, 0x00191C1A,
        0x00191C1A, 0x00181B19, 0x00181B19, 0x00181B19, 0x00191C1A, 0x001A1D1B, 0x001B1E1C, 0x00171E1B, 0x00141E18, 0x00162117, 0x001A2719, 0x00192A15, 0x001A2D14, 0x001D3316, 0x0023391C, 0x0037461A, 0x00384818, 0x003D4F1A, 0x00455821, 0x004D5F2A,
        0x00556536, 0x005C6943, 0x00616C4C, 0x0063744F, 0x00657651, 0x00677853, 0x00677853, 0x00687954, 0x00697A55, 0x006B7C57, 0x006D7E59, 0x006E8059, 0x006E8059, 0x006E8059, 0x006E8059, 0x006E8059, 0x006E8059, 0x006D7F58, 0x006D7F58, 0x006F815A,
        0x006B7D56, 0x00768762, 0x0070815C, 0x006B7B59, 0x00536341, 0x004C5C3A, 0x005B6A4A, 0x00627152, 0x004F5F3D, 0x00596B42, 0x0063764B, 0x004E6039, 0x004E5C40, 0x00717B6A, 0x00B0B8AE, 0x002D3621, 0x0077806B, 0x008D9681, 0x0019220D, 0x00606954,
        0x005E6752, 0x00222B16, 0x00707964, 0x005D6A54, 0x001F2C16, 0x0087947E, 0x0047533F, 0x00313D29, 0x00ABB7A5, 0x00505C4A, 0x00313D2B, 0x007E8A78, 0x002B3723, 0x0054614B, 0x00B6C5AA, 0x00506041, 0x006A7B59, 0x006F815C, 0x006B7E57, 0x005D6E4C,
        0x004C5A36, 0x00394A29, 0x003A5545, 0x005C868B, 0x005E93A8, 0x00558DA6, 0x005589A0, 0x00416F77, 0x00273F33, 0x004F5844, 0x00717C79, 0x0098A6AC, 0x00DDE5E5, 0x00BBC2BF, 0x00859195, 0x00879191, 0x00889292, 0x00889292, 0x00899393, 0x008A9494,
        0x008B9595, 0x008B9595, 0x008C9696, 0x00859490, 0x0081908C, 0x00808F8B, 0x007D8A88, 0x00788583, 0x00869090, 0x00ABB5B5, 0x00CED8D8, 0x00E0E7E0, 0x008F9988, 0x006F7E5E, 0x0075855C, 0x005E6C48, 0xFFC3CAB5, 0xFFFFFFFC, 0xFFFFFCFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFF9FF, 0xFFFFFFF7, 0x00BAC2A3, 0x0057663A, 0x007B895F, 0x0073825C, 0x00728259, 0x0078895D, 0x006C7E55, 0x00708259, 0x0071835A, 0x006F8158, 0x0072845B,
        0x00778960, 0x0076885F, 0x0073855C, 0x0078895D, 0x00798A5F, 0x0076865D, 0x00718158, 0x007C8D62, 0x007E9061, 0x00809360, 0x007D925B, 0x0084906C, 0x007B8862, 0x007F8C66, 0x007E8C63, 0x007A885E, 0x007B895F, 0x007A895D, 0x00768559, 0x00768958,
        0x00758857, 0x00758758, 0x00728455, 0x006E7F53, 0x006F8055, 0x00728358, 0x006F8055, 0x0076865D, 0x005E6E45, 0x004B5B32, 0x00596940, 0x0046562D, 0x006E7E55, 0x007E8E65, 0x006F7F56, 0x0074855A, 0x00728358, 0x007A8B60, 0x0075855C, 0x00516138,
        0x00506037, 0x005F6E48, 0x00788761, 0x0073855E, 0x0071835C, 0x0070825B, 0x0070825B, 0x0071835C, 0x0071835C, 0x0071835C, 0x0070825B, 0x0074805C, 0x006F7E58, 0x006C7E55, 0x006C8056, 0x00657851, 0x00566442, 0x003D4527, 0x00555940, 0x00656B5A,
        0x00282F1A, 0x00353E23, 0x00233010, 0x00576541, 0x0070815C, 0x006D7D5B, 0x00697957, 0x006C8357, 0x00556D4B, 0x002F4735, 0x00435D5D, 0x006F8C9B, 0x007090AD, 0x00688CB0, 0x00648BB2, 0x006B8FB3, 0x0064829F, 0x00617587, 0x004D595F, 0x00515853,
        0x00515849, 0x0045513B, 0x0035422A, 0x00324524, 0x002D3C1D, 0x00273317, 0x00252D16, 0x00272D1A, 0x00262E1D, 0x00222C1C, 0x001D2A1C, 0x001D2716, 0x001A2413, 0x00172111, 0x00161E13, 0x00161E14, 0x00161C17, 0x00151A18, 0x00131817, 0x00181A1A,
        0x00181A1A, 0x00171919, 0x00171919, 0x00171919, 0x00181A1A, 0x00191B1B, 0x001A1C1C, 0x0015141D, 0x0015151B, 0x0015191A, 0x00181D1B, 0x001A211A, 0x00192317, 0x001B2518, 0x001E2919, 0x00273016, 0x00293414, 0x00314014, 0x003B4D18, 0x0042551E,
        0x00485926, 0x00546238, 0x00606C48, 0x0062734E, 0x00647550, 0x00667752, 0x00677853, 0x00687954, 0x00697A55, 0x006B7C57, 0x006C7D58, 0x006E8059, 0x006E8059, 0x006E8059, 0x006E8059, 0x006E8059, 0x006E8059, 0x006E8059, 0x006E8059, 0x00718459,
        0x006C7E55, 0x00708259, 0x006A7C53, 0x0074865F, 0x006F815A, 0x0072835E, 0x007C8D68, 0x00758563, 0x006E7F5A, 0x0073865B, 0x0065794C, 0x004D5F36, 0x004C5B3B, 0x0096A38B, 0x00EAF4E3, 0x003E3F25, 0x00BDBEA4, 0x00DCDDC3, 0x0043442A, 0x00727359,
        0x00626349, 0x0033341A, 0x0067684E, 0x00495539, 0x00424E32, 0x008E9A7E, 0x00364127, 0x003B462C, 0x00959F88, 0x004B553E, 0x00444E37, 0x00535F4B, 0x0028351F, 0x004C5A42, 0x008D9C80, 0x003C4C2D, 0x006F805E, 0x006E805B, 0x00677954, 0x00526341,
        0x0045532F, 0x003A4B2A, 0x003B5646, 0x00578186, 0x005D92A7, 0x005B93AC, 0x005589A0, 0x0044727A, 0x00273F33, 0x004E5743, 0x00707B78, 0x0096A4AA, 0x00DDE5E5, 0x00BDC4C1, 0x00869296, 0x007D898D, 0x007F8B8F, 0x00828E92, 0x00879397, 0x008B979B,
        0x00899599, 0x00849094, 0x00808C90, 0x00808E8D, 0x00808E8D, 0x00818E90, 0x007E8A8C, 0x00768284, 0x00828B8E, 0x00AAB3B6, 0x00D0D9DC, 0x00B9C1B6, 0x00818C78, 0x00707F5F, 0x00778660, 0x005F6D4B, 0xFFC3CAB5, 0xFFFFFFFC, 0xFFFFFCFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFF9FF, 0xFFFFFFF9, 0x00BAC2A3, 0x0058673B, 0x007B895F, 0x0076855F, 0x0073835A, 0x0077885C, 0x00788A61, 0x00798B62, 0x007B8D64, 0x0076885F, 0x00788A61,
        0x007B8D64, 0x00788A61, 0x007D8F66, 0x00778B5E, 0x00778960, 0x00758661, 0x00778666, 0x00778765, 0x006E8057, 0x00708554, 0x00728953, 0x00728259, 0x00708057, 0x0073835A, 0x0077875E, 0x00738459, 0x0075865B, 0x00798A5F, 0x00718257, 0x00738556,
        0x00728455, 0x00728455, 0x00728357, 0x00728357, 0x00738458, 0x00738458, 0x00718256, 0x00718158, 0x007A8A61, 0x005C6C43, 0x004B5B32, 0x00596940, 0x0054643B, 0x006F7F56, 0x007B8B62, 0x00728358, 0x00718257, 0x00738459, 0x0077875E, 0x004E5E35,
        0x00526239, 0x005C6B45, 0x007C8B65, 0x0073855E, 0x0072845D, 0x0071835C, 0x0071835C, 0x0071835C, 0x0072845D, 0x0071835C, 0x006F815A, 0x00727E5A, 0x006F7E58, 0x006A7C53, 0x006B7F55, 0x00667952, 0x00596745, 0x003C4426, 0x004F533A, 0x002B3120,
        0x00353C27, 0x00343D22, 0x003C4929, 0x003E4C28, 0x0061724D, 0x006A7A58, 0x00657553, 0x0071835A, 0x005A6B4A, 0x00334633, 0x002F4441, 0x00637E88, 0x007D9DB0, 0x006E91AB, 0x007BA0BC, 0x006B90AA, 0x00658499, 0x00566D7C, 0x0039484B, 0x0038433B,
        0x003D4736, 0x00455036, 0x003D492B, 0x002E4017, 0x00273511, 0x00202C0E, 0x00242B16, 0x002A2E22, 0x00292E25, 0x00222620, 0x00171D18, 0x00202418, 0x001E2216, 0x001B1F14, 0x00191C13, 0x00191B15, 0x00191A18, 0x00191A18, 0x00191919, 0x00191B1B,
        0x00191B1B, 0x00181A1A, 0x00181A1A, 0x00181A1A, 0x00191B1B, 0x00191B1B, 0x00191B1B, 0x001C181E, 0x001C181D, 0x001E1B1D, 0x001F1F1F, 0x0021221E, 0x001F241B, 0x001E2419, 0x001E2419, 0x001F211B, 0x001F2514, 0x00273211, 0x00344414, 0x003C4F18,
        0x0041531E, 0x004E5E2F, 0x005C6A41, 0x0063744F, 0x00657651, 0x00687954, 0x00697A55, 0x006A7B56, 0x006B7C57, 0x006D7E59, 0x006E7F5A, 0x006D7F58, 0x006D7F58, 0x006D7F58, 0x006E8059, 0x006E8059, 0x006E8059, 0x006E8059, 0x006E8059, 0x00708457,
        0x006F8356, 0x00708358, 0x00677A4F, 0x00708259, 0x006D7F56, 0x006D7F56, 0x006C7E57, 0x0070815C, 0x00788A63, 0x00718459, 0x00586B40, 0x0054663D, 0x00425230, 0x00748266, 0x00BBC8B0, 0x003C3E20, 0x00838567, 0x00828466, 0x003D3F21, 0x00838567,
        0x00737557, 0x00434527, 0x004B4D2F, 0x003B4625, 0x00525D3C, 0x00C0CBAB, 0x00535E3E, 0x00374123, 0x00626C4E, 0x002B3419, 0x00565F44, 0x008C9A82, 0x00324028, 0x0037462B, 0x00617054, 0x003B4B2C, 0x006B7C5B, 0x00687957, 0x004F613C, 0x00495A38,
        0x004B5935, 0x00435433, 0x00375242, 0x004F797E, 0x00598EA3, 0x005991AA, 0x0054889F, 0x0045737B, 0x00253D31, 0x004E5743, 0x00717C79, 0x0094A2A8, 0x00DEE6E6, 0x00C0C7C4, 0x00869296, 0x00708087, 0x00708087, 0x0074848B, 0x007F8F96, 0x008999A0,
        0x0088989F, 0x007B8B92, 0x006E7E85, 0x00778389, 0x00818D93, 0x00869296, 0x00808B8F, 0x00758084, 0x00848D91, 0x00B0B9BC, 0x00CFD8DB, 0x00899382, 0x006E7B63, 0x006E7D5D, 0x00788761, 0x00606D4D, 0xFFC3C9B6, 0xFFFFFFFC, 0xFFFCFAFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFAFF, 0xFFFFFFF9, 0x00BBC3A4, 0x0059683C, 0x007D8B61, 0x007A8963, 0x006A7A51, 0x00607145, 0x0061734A, 0x005F7148, 0x00607249, 0x005C6E45, 0x005F7148,
        0x005F7148, 0x005A6C43, 0x0066784F, 0x0072835E, 0x00637253, 0x005B6751, 0x006C7666, 0x005D6757, 0x0038442E, 0x00303F20, 0x00394A25, 0x002F3C1C, 0x002F3D1B, 0x003A4826, 0x004B5935, 0x0054623E, 0x0051603A, 0x006D7D54, 0x0076865D, 0x00718158,
        0x0073835A, 0x0075855C, 0x0076875C, 0x0077885D, 0x0075865B, 0x0074855A, 0x0075865B, 0x0075855C, 0x0074845B, 0x0078885F, 0x0058683F, 0x0058683F, 0x0053633A, 0x00526239, 0x0074845B, 0x00738459, 0x0076875C, 0x00708156, 0x0074845B, 0x00506037,
        0x0056663D, 0x005A6943, 0x007E8D67, 0x0074865F, 0x0072845D, 0x0071835C, 0x0072845D, 0x0072845D, 0x0072845D, 0x0071835C, 0x0070825B, 0x0076825E, 0x0072815B, 0x006B7D54, 0x006A7E54, 0x006A7D56, 0x00616F4D, 0x003B4325, 0x003F432A, 0x003F4534,
        0x00414833, 0x00454E33, 0x00354222, 0x004B5935, 0x00374823, 0x00667654, 0x006F7F5D, 0x00717853, 0x00626A4B, 0x005E6851, 0x004E5B4D, 0x00889B92, 0x00ADC4BF, 0x006C8783, 0x0063817C, 0x00618383, 0x00658483, 0x005A746E, 0x00314538, 0x001D2C17,
        0x001C2B0C, 0x003D4C26, 0x00526036, 0x003D5019, 0x00374717, 0x002F3E18, 0x002C371D, 0x002C3423, 0x00292F24, 0x0025291E, 0x001F2318, 0x0026271D, 0x0024251B, 0x0022231A, 0x001F1F19, 0x001D1D17, 0x001D1C18, 0x001F1E1A, 0x00211F1E, 0x001A1D1B,
        0x001B1E1C, 0x001B1E1C, 0x001C1F1D, 0x001C1F1D, 0x001B1E1C, 0x001A1D1B, 0x001A1D1B, 0x00221F1B, 0x00221F1B, 0x0024211C, 0x0024251C, 0x0027291D, 0x00262B1C, 0x00242A19, 0x00212716, 0x00262527, 0x0023261D, 0x00283019, 0x0033401A, 0x003B4B1B,
        0x0040511E, 0x004B5B2B, 0x00576738, 0x0062734E, 0x00647550, 0x00677853, 0x00697A55, 0x006A7B56, 0x006B7C57, 0x006C7D58, 0x006E7F5A, 0x006D7F58, 0x006D7F58, 0x006D7F58, 0x006E8059, 0x006E8059, 0x006E8059, 0x006F815A, 0x006F815A, 0x00697D50,
        0x006E8255, 0x006F8257, 0x00687B50, 0x0071835A, 0x006D7F56, 0x00708259, 0x006D7F58, 0x0070815C, 0x006A7C55, 0x00586A41, 0x004A5C33, 0x005B6D46, 0x0042532E, 0x00404F2F, 0x00516041, 0x00576541, 0x004C5A36, 0x002A3814, 0x00465430, 0x006A7854,
        0x00485632, 0x003E4C28, 0x00586642, 0x004C5834, 0x00515D39, 0x007A8662, 0x00606B4A, 0x005A6544, 0x00515C3C, 0x00566141, 0x003C4727, 0x00324126, 0x0048573B, 0x00657458, 0x00546445, 0x005B6C4B, 0x00657655, 0x0050613F, 0x004C5D3B, 0x00435432,
        0x00576541, 0x004E5F3E, 0x00385343, 0x00517B80, 0x005B90A5, 0x00578FA8, 0x0053879E, 0x00437179, 0x0021392D, 0x004E5743, 0x00737E7B, 0x0093A1A7, 0x00DDE5E5, 0x00C2C9C6, 0x00859195, 0x00728289, 0x00718188, 0x0074848B, 0x007F8F96, 0x008999A0,
        0x008999A0, 0x007E8E95, 0x0073838A, 0x00758088, 0x00828D95, 0x00879399, 0x00808B8F, 0x00768185, 0x0090999C, 0x00BBC5C5, 0x00C4CECE, 0x007A876F, 0x006F7D61, 0x0071815F, 0x0075835F, 0x005D694B, 0xFFC1C7B6, 0xFFFFFFFC, 0xFFFFFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCF5FF, 0xFFFFFFF9, 0x00BAC2A3, 0x005E6D41, 0x00808E64, 0x0075845E, 0x004E5E35, 0x00314216, 0x0031431A, 0x002D3F16, 0x002E4017, 0x00293B12, 0x00304219,
        0x0033451C, 0x00273910, 0x0031431A, 0x006A7959, 0x0047543C, 0x00444C41, 0x008A8E8F, 0x00A6A9AD, 0x00919695, 0x00899187, 0x008F9989, 0x008D9787, 0x008E9887, 0x007B8672, 0x00536048, 0x004E5C40, 0x00435232, 0x006E7F5A, 0x0073845F, 0x00768462,
        0x007A8866, 0x007B8967, 0x007A8864, 0x00798862, 0x0075845E, 0x0076865D, 0x007C8C63, 0x007A8A61, 0x006D7D54, 0x0073835A, 0x007F8F66, 0x004F5F36, 0x0056663D, 0x0055653C, 0x0054643B, 0x00718257, 0x0076875C, 0x00738459, 0x0073835A, 0x0053633A,
        0x0058683F, 0x005B6A44, 0x007D8C66, 0x0073855E, 0x0072845D, 0x0072845D, 0x0072845D, 0x0073855E, 0x0073855E, 0x0072845D, 0x0070825B, 0x0074805C, 0x0074835D, 0x00708259, 0x0070845A, 0x006F825B, 0x005F6D4B, 0x002C3416, 0x00262A11, 0x00272D1C,
        0x00202712, 0x001E270C, 0x00364323, 0x00475531, 0x004C5D38, 0x00425230, 0x005A6A48, 0x007D845F, 0x00686D4D, 0x00697153, 0x005A6349, 0x0077846A, 0x007C8B6F, 0x002F4424, 0x002D4221, 0x00456046, 0x00486349, 0x00445A41, 0x002E4428, 0x002C3E1F,
        0x0031431C, 0x004E5F33, 0x00586B3A, 0x0048581B, 0x0043531E, 0x0037471E, 0x002A371D, 0x001F2918, 0x001C2316, 0x00212415, 0x00272818, 0x0027281E, 0x0027281E, 0x0026271D, 0x0023241A, 0x00202118, 0x00202118, 0x0023241B, 0x0026271E, 0x001B1F19,
        0x001C201A, 0x001F231D, 0x0020241E, 0x0020241E, 0x001E221C, 0x001B1F19, 0x001A1E18, 0x00212016, 0x00212016, 0x00202216, 0x00232617, 0x00252B1A, 0x00282E1D, 0x00232C18, 0x001E2713, 0x00222123, 0x0020211D, 0x00222718, 0x002A3319, 0x00323E1A,
        0x003A491D, 0x00475727, 0x00536431, 0x005E6F4A, 0x0061724D, 0x00647550, 0x00677853, 0x00687954, 0x00687954, 0x006A7B56, 0x006B7C57, 0x006C7E57, 0x006D7F58, 0x006D7F58, 0x006D7F58, 0x006E8059, 0x006F815A, 0x006F815A, 0x006F815A, 0x006B7E53,
        0x006F8158, 0x00708259, 0x006C7E55, 0x00758760, 0x006D7F58, 0x00748560, 0x00758661, 0x00728260, 0x00637351, 0x005D6D4B, 0x00556543, 0x00566644, 0x00516040, 0x00526141, 0x00546343, 0x005D724C, 0x00637852, 0x004E633D, 0x00617650, 0x0040552F,
        0x002C411B, 0x00485D37, 0x005B704A, 0x003A4824, 0x0043512D, 0x00616F4B, 0x00505E3C, 0x00687654, 0x005C6949, 0x005A6747, 0x00667353, 0x00627253, 0x00455536, 0x00667657, 0x006E7F5E, 0x00687958, 0x00536443, 0x00506140, 0x00475837, 0x00536442,
        0x0064724E, 0x00516241, 0x00344F3F, 0x004E787D, 0x00588DA2, 0x00558DA6, 0x005589A0, 0x00416F77, 0x001D3529, 0x004E5743, 0x00747F7C, 0x00909EA4, 0x00DCE4E4, 0x00C5CCC9, 0x00879397, 0x007D8D93, 0x007E8E94, 0x00809096, 0x00839399, 0x0086969C,
        0x0088989E, 0x0088989E, 0x0088989E, 0x00869199, 0x008A969C, 0x00899599, 0x00828E90, 0x00778383, 0x0097A2A0, 0x00BDC8C5, 0x00AAB5B2, 0x00707F60, 0x00707F5F, 0x00728260, 0x00748260, 0x005D694D, 0xFFC1C7B6, 0xFFFFFFFC, 0xFFFFFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFF9FF, 0xFFFFFFF8, 0x00B3BB9C, 0x005D6C40, 0x007F8D63, 0x0075845E, 0x004B5B32, 0x00314216, 0x0053653C, 0x004F6138, 0x00495B32, 0x003C4E25, 0x00495B32,
        0x00506239, 0x0036481F, 0x00304219, 0x006E8258, 0x00425135, 0x00414B3F, 0x00A4A8AD, 0x00DEE0EB, 0x00DCDEE8, 0x00D6DBDE, 0x00D8DFDC, 0x00DEE1E5, 0x00E9EEED, 0x00C0C6C1, 0x0061695E, 0x00515C48, 0x003C492F, 0x006A7959, 0x005B6B49, 0x005D684E,
        0x00586349, 0x00505C40, 0x00505C3E, 0x00596646, 0x005A6846, 0x00596745, 0x005E6C48, 0x00708057, 0x007B8B62, 0x0074845B, 0x006E7E55, 0x007B8B62, 0x0055653C, 0x005C6C43, 0x004A5A31, 0x00617247, 0x00697A4F, 0x0076875C, 0x0074845B, 0x0053633A,
        0x0055653C, 0x00606F49, 0x00788761, 0x0072845D, 0x0072845D, 0x0072845D, 0x0073855E, 0x0074865F, 0x0074865F, 0x0073855E, 0x0071835C, 0x00727E5A, 0x0073825C, 0x0072845B, 0x0073875D, 0x006D8059, 0x00586644, 0x00293113, 0x002A2E15, 0x00212716,
        0x00252C17, 0x00262F14, 0x003D4A2A, 0x0062704C, 0x0042532E, 0x0050603E, 0x00485836, 0x00677950, 0x006B7A54, 0x006C7A58, 0x00748161, 0x00798666, 0x00616F4D, 0x00475630, 0x00506037, 0x004C5D32, 0x00606F49, 0x006C7C5A, 0x00677758, 0x00687958,
        0x005D6F4A, 0x005F7247, 0x00506435, 0x0045521C, 0x003D4A1C, 0x002F3C1C, 0x001F2B19, 0x00162014, 0x00171C13, 0x001F2115, 0x00262517, 0x001F2619, 0x0021281B, 0x00232A1D, 0x0022291C, 0x001F2619, 0x001E2518, 0x0020271A, 0x00232A1D, 0x001A2114,
        0x001D2417, 0x0021281B, 0x00232A1D, 0x00232A1D, 0x001F2619, 0x001B2215, 0x00181F12, 0x001B1C13, 0x001A1D14, 0x001A1D14, 0x001D2116, 0x0021271C, 0x00262D20, 0x00212A1D, 0x001A2316, 0x001A1B17, 0x001A1B17, 0x001C1F16, 0x001D2213, 0x00222A13,
        0x002F3B17, 0x00425222, 0x0051642D, 0x005D6E49, 0x0060714C, 0x00647550, 0x00667752, 0x00687954, 0x00687954, 0x006A7B56, 0x006B7C57, 0x006C7E57, 0x006C7E57, 0x006D7F58, 0x006D7F58, 0x006E8059, 0x006F815A, 0x006F815A, 0x006F815A, 0x0072845D,
        0x0072845D, 0x006E7F5A, 0x006C7D58, 0x006D7D5B, 0x00546442, 0x0050603E, 0x004C5B3B, 0x00475636, 0x00495839, 0x00556445, 0x004D5B3F, 0x0039472B, 0x00404F30, 0x00455435, 0x00536242, 0x00364725, 0x004B5C3A, 0x004E5F3D, 0x00455634, 0x00354624,
        0x00445533, 0x00546543, 0x00364725, 0x00344621, 0x00566745, 0x00475836, 0x00354624, 0x00596A49, 0x003B4C2B, 0x002E3E1F, 0x002D3D1E, 0x004C5D3C, 0x005D6E4D, 0x00657655, 0x00647554, 0x00586948, 0x00546544, 0x00506140, 0x00556645, 0x00657654,
        0x00606E4A, 0x00455635, 0x002E4939, 0x00487277, 0x0054899E, 0x00548CA5, 0x005589A0, 0x003F6D75, 0x001B3327, 0x004D5642, 0x00737E7B, 0x008C9AA0, 0x00DAE2E2, 0x00C8CFCC, 0x008B979B, 0x008A969A, 0x008B979B, 0x008C989C, 0x008B979B, 0x00899599,
        0x008B979B, 0x008F9B9F, 0x0094A0A4, 0x0094A0A6, 0x008F9B9F, 0x008D9A9C, 0x00889494, 0x0073817D, 0x00919D97, 0x00B9C6BE, 0x009CAA9F, 0x00677755, 0x006F805B, 0x0071825D, 0x00758361, 0x00606B51, 0xFFC2C7B8, 0xFFFFFFFC, 0xFFFDFCFE, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFAFF, 0xFFFDFEF5, 0x00AEB697, 0x005C6B3F, 0x007F8D63, 0x0074835D, 0x004C5C33, 0x0038491D, 0x0062744B, 0x005F7148, 0x00576940, 0x00475930, 0x005A6C43,
        0x00687A51, 0x0046582F, 0x0032441B, 0x00677D4C, 0x004D5F3A, 0x00596454, 0x00B9BEC1, 0x00E1E2F0, 0x00CFD0DE, 0x00C0C4C9, 0x00BCC1C2, 0x00CBCADA, 0x00DCDCE8, 0x00C0C3C8, 0x00646967, 0x005B6457, 0x002C3822, 0x004E5C40, 0x00394828, 0x00616A55,
        0x004E5742, 0x00364029, 0x00354026, 0x00465236, 0x004A5638, 0x00424F2F, 0x003E4B2B, 0x00728259, 0x0073835A, 0x0078885F, 0x0078885F, 0x0073835A, 0x0078885F, 0x005A6A41, 0x00526239, 0x00506136, 0x0058693E, 0x0076875C, 0x0075855C, 0x00506037,
        0x00516138, 0x0065744E, 0x0074835D, 0x0072845D, 0x0071835C, 0x0072845D, 0x0073855E, 0x00758760, 0x00758760, 0x0074865F, 0x0072845D, 0x0077835F, 0x0072815B, 0x006E8057, 0x006D8157, 0x00667952, 0x00566442, 0x003B4325, 0x0050543B, 0x004D5342,
        0x002A311C, 0x00363F24, 0x004C5939, 0x00697753, 0x0061724D, 0x00455533, 0x0050603E, 0x00395629, 0x00668156, 0x005E7451, 0x00657758, 0x005D6D4E, 0x00465536, 0x00576543, 0x0047532F, 0x00464E1F, 0x0060693E, 0x00707A56, 0x006B7759, 0x006A795D,
        0x00607051, 0x005E6F4D, 0x00495B34, 0x00444B1E, 0x00373E1B, 0x00252E19, 0x001C251B, 0x001C2320, 0x00212422, 0x0027241F, 0x0029251A, 0x00192011, 0x001D2415, 0x0022291A, 0x00232A1B, 0x00202718, 0x001E2615, 0x00202817, 0x00222A19, 0x001A2112,
        0x001D2415, 0x00212819, 0x00242B1C, 0x00242B1C, 0x00202718, 0x001B2213, 0x00171E0F, 0x00171816, 0x00181917, 0x00161917, 0x00181B19, 0x001E241F, 0x00232924, 0x00212722, 0x0019201B, 0x001D2116, 0x0020221C, 0x001F201C, 0x001A1C16, 0x00191F0E,
        0x00263111, 0x003D4D1E, 0x004E6128, 0x005F704B, 0x0062734E, 0x00667752, 0x00697A55, 0x006A7B56, 0x006B7C57, 0x006C7D58, 0x006D7E59, 0x006C7E57, 0x006C7E57, 0x006D7F58, 0x006D7F58, 0x006E8059, 0x006F815A, 0x006F815A, 0x0070825B, 0x006E7F5A,
        0x006D7D5B, 0x006C7C5A, 0x006F7F5D, 0x006D7C5C, 0x00465535, 0x00364526, 0x002B3A1B, 0x003B4A2B, 0x004B593D, 0x0059664C, 0x0059664E, 0x004A573F, 0x00455238, 0x001F2E0F, 0x00202F0F, 0x00434C2B, 0x00444D2C, 0x005E6746, 0x004F5837, 0x00454E2D,
        0x0037401F, 0x003F4827, 0x003A4322, 0x00435734, 0x00495D3A, 0x005A6D4C, 0x00344726, 0x00384A2B, 0x00435536, 0x00384A2B, 0x002E4023, 0x003E4F2D, 0x00556644, 0x00556645, 0x006A7B5A, 0x00475837, 0x00556645, 0x00586948, 0x00657655, 0x00617250,
        0x004C5A36, 0x00354625, 0x002E4939, 0x004B757A, 0x00558A9F, 0x00568EA7, 0x0052869D, 0x003F6D75, 0x00193125, 0x004D5642, 0x00727D7A, 0x0089979D, 0x00D8E0E0, 0x00C9D0CD, 0x008E9A9E, 0x008D999B, 0x008D999B, 0x008C989A, 0x008B9799, 0x008B9799,
        0x008C989A, 0x008C989A, 0x008D999B, 0x00929EA2, 0x008B989A, 0x008F9D9C, 0x008C9997, 0x006F7D77, 0x008A988D, 0x00BBC9BD, 0x00A1AFA3, 0x006E7F5A, 0x00758661, 0x00748560, 0x00768363, 0x00626D53, 0xFFC2C7B8, 0xFFFFFFFC, 0xFFFDFCFE, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xFBFAFEF3, 0x00B2BC9E, 0x005B6940, 0x007B8862, 0x0075815D, 0x004D5B31, 0x00405020, 0x005B6D44, 0x0033451C, 0x005D6F46, 0x0044562D, 0x0035471E,
        0x00687A51, 0x004E6037, 0x00304219, 0x006E8559, 0x005A6E4B, 0x00667462, 0x00C0C9C6, 0x00D9E0E3, 0x00C3C9CE, 0x00BCC4C3, 0x00B4BEB8, 0x00CCD0D5, 0x00D6DBDE, 0x00C4CBC8, 0x00818A80, 0x00697563, 0x00637159, 0x004D5D3E, 0x00728362, 0x00A2A39A,
        0x0077786F, 0x00606158, 0x0063645B, 0x006A6B62, 0x0097988F, 0x0086877E, 0x003B3C33, 0x00728358, 0x0076875C, 0x0074855A, 0x00728358, 0x006F8055, 0x007A8B60, 0x007B8C61, 0x0054653A, 0x004E6039, 0x004C5E37, 0x005B6D46, 0x006E8059, 0x0051633C,
        0x00556740, 0x005C6E47, 0x00788A63, 0x0074875C, 0x0074865D, 0x0073855C, 0x0072845B, 0x0072845D, 0x0072845D, 0x0073845F, 0x00748560, 0x00788964, 0x006D7F58, 0x0072815B, 0x0074825E, 0x00757F61, 0x00586049, 0x00404536, 0x0066675D, 0x0066695A,
        0x003A402D, 0x00353E24, 0x004B5537, 0x00687654, 0x006E7C5A, 0x00627151, 0x00435233, 0x004D603F, 0x00435A2E, 0x002E431D, 0x00334030, 0x00465246, 0x004C5C44, 0x003F5035, 0x003A4739, 0x003B4D4E, 0x0034493A, 0x00354F2B, 0x00647D51, 0x005F7349,
        0x0063724C, 0x00626E44, 0x0052612E, 0x0038402F, 0x001A2112, 0x001A2015, 0x00262B22, 0x001C251B, 0x003A483C, 0x002B412F, 0x0027402C, 0x00384232, 0x00384232, 0x003B4534, 0x00363E2D, 0x00353E2A, 0x003E4431, 0x003B422D, 0x003A412C, 0x003C4834,
        0x00394531, 0x00303C28, 0x0036422E, 0x00384430, 0x002E3A26, 0x002D3925, 0x002D3925, 0x0024331E, 0x002B3A25, 0x002B3A25, 0x00263520, 0x002C3B26, 0x00364530, 0x00303F2A, 0x0021301B, 0x001B3821, 0x00203320, 0x0033372B, 0x0023241B, 0x00192117,
        0x00141F15, 0x002D3125, 0x00555244, 0x00626E50, 0x0063714F, 0x00606F49, 0x006E8057, 0x00697C51, 0x00697B52, 0x00677954, 0x006B7C5A, 0x00687A55, 0x006A7C57, 0x006C7E59, 0x006D7F5A, 0x006D7F5A, 0x006D7F5A, 0x006E805B, 0x006F815C, 0x00757F5B,
        0x00677954, 0x006A845C, 0x006B8353, 0x00718151, 0x0049532F, 0x00444D43, 0x004C5864, 0x0064696A, 0x0042504C, 0x003A4830, 0x007B7C55, 0x005B583C, 0x00414A47, 0x00344647, 0x002C3A2F, 0x00415150, 0x00445139, 0x005D6739, 0x00404A20, 0x00394833,
        0x00384847, 0x00414E50, 0x003E4A44, 0x003C484C, 0x0049573B, 0x005B653B, 0x004A513C, 0x00404C4C, 0x00405553, 0x003D524A, 0x00202F27, 0x003D5035, 0x004A5F3F, 0x00526643, 0x0060754E, 0x004E613A, 0x00485A35, 0x0071815F, 0x006F7E5E, 0x004A593D,
        0x0046562D, 0x003A4E23, 0x0029462F, 0x004B747D, 0x0050829E, 0x00558DA6, 0x00538A97, 0x00426F7A, 0x0020382E, 0x004D5544, 0x0076817E, 0x0086939B, 0x00DAE1E4, 0x00C8CECD, 0x00879399, 0x007D9095, 0x007C8F94, 0x007F9297, 0x00879A9F, 0x0090A3A8,
        0x008FA2A7, 0x0085989D, 0x007B8E93, 0x00829096, 0x007F8D93, 0x00869598, 0x007F8B8F, 0x00717E80, 0x00859193, 0x009AA6A8, 0x00919D9D, 0x006D7F5A, 0x006D7F5A, 0x006B7C57, 0x00758464, 0x00647157, 0xFFBEC5B6, 0xFFFAFEF9, 0xFFFCFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xFFFDFFF6, 0x00B1BB9D, 0x005A683F, 0x007D8A64, 0x00798561, 0x004E5C32, 0x003B4B1B, 0x0061734A, 0x0056683F, 0x00596B42, 0x0042542B, 0x0052643B,
        0x00697B52, 0x00405229, 0x00304219, 0x00677856, 0x00516045, 0x00646E62, 0x00BFC4C5, 0x00D4D6E0, 0x00C5C7D1, 0x00CCD0D5, 0x00C7CCCD, 0x00C9CAD4, 0x00D9DBE3, 0x00C9CCD0, 0x00707573, 0x0071796E, 0x00515B4A, 0x00505C46, 0x00626F55, 0x00989B8B,
        0x00898C7C, 0x006A6D5D, 0x00636656, 0x00747767, 0x00949787, 0x00838676, 0x005F6252, 0x005F7045, 0x0076875C, 0x0074855A, 0x006C7D52, 0x0074855A, 0x0078895E, 0x0075865B, 0x00718257, 0x005C6E47, 0x0051633C, 0x0050623B, 0x005B6D46, 0x0043552E,
        0x0052643D, 0x005E7049, 0x00798B64, 0x0074875C, 0x0074865D, 0x0073855C, 0x0073855C, 0x0073855E, 0x0073855E, 0x00748560, 0x00758661, 0x00768762, 0x00768762, 0x007B8A64, 0x0072815B, 0x00717C5B, 0x0060694E, 0x00444C35, 0x004E5441, 0x003F4534,
        0x003B422D, 0x00424B30, 0x004B5636, 0x00677551, 0x00788662, 0x00768664, 0x005F6F4D, 0x00435536, 0x004F653B, 0x00405431, 0x004D5A4C, 0x005E695F, 0x004A5A43, 0x001F2F17, 0x00344034, 0x0048595C, 0x00384C3F, 0x002E4624, 0x00637B53, 0x00637550,
        0x00667452, 0x0063704A, 0x004D5A2E, 0x00202718, 0x001B2116, 0x00171C13, 0x00191F1A, 0x00212823, 0x005B695E, 0x006A7C6F, 0x00728876, 0x0056685B, 0x00677A6B, 0x00748778, 0x00697A6C, 0x00647665, 0x006E7E6D, 0x00697967, 0x00637361, 0x006F7C6C,
        0x00657262, 0x00606D5D, 0x00697666, 0x00677464, 0x00637060, 0x006A7767, 0x00677464, 0x00617160, 0x00647463, 0x00637362, 0x00627261, 0x0060705F, 0x00596958, 0x00465645, 0x00334332, 0x005D7965, 0x006F7E70, 0x0060635A, 0x0023251F, 0x00161D18,
        0x000D1810, 0x0012170E, 0x00373229, 0x00515D3F, 0x00687555, 0x00677551, 0x0066784F, 0x0061734A, 0x00677952, 0x00687A55, 0x006B7C5A, 0x00697B56, 0x006B7D58, 0x006D7F5A, 0x006E805B, 0x006D7F5A, 0x006D7F5A, 0x006E805B, 0x006F815C, 0x0076815B,
        0x006A7C57, 0x00678159, 0x00698355, 0x0067784C, 0x003F4B2D, 0x004D5C58, 0x00516374, 0x00566578, 0x00465F69, 0x003B5347, 0x00697656, 0x00464F3A, 0x003A5056, 0x00405A6B, 0x003F565E, 0x003A525E, 0x0035493D, 0x00475837, 0x00354728, 0x00496058,
        0x004D6571, 0x00425969, 0x00465B63, 0x00364D55, 0x00435A44, 0x00384C29, 0x0038483D, 0x00526572, 0x003E5967, 0x003A575E, 0x003F565E, 0x00394C31, 0x00516646, 0x00526643, 0x00546942, 0x004D6039, 0x004E603B, 0x00657553, 0x00536242, 0x00415034,
        0x004A5A31, 0x0043572C, 0x002A4730, 0x0049727B, 0x004F819D, 0x00548CA5, 0x00528996, 0x00416E79, 0x0020382E, 0x004B5342, 0x0075807D, 0x0085929A, 0x00DAE1E4, 0x00CCD2D1, 0x008B979D, 0x0072858A, 0x00718489, 0x0075888D, 0x007F9297, 0x008A9DA2,
        0x008A9DA2, 0x007E9196, 0x0072858A, 0x0078868C, 0x007C8A90, 0x00859497, 0x00808C90, 0x00758284, 0x00849092, 0x0096A2A4, 0x00929E9E, 0x00728361, 0x00748661, 0x00748560, 0x00758563, 0x005D6A50, 0xFFB9C0B1, 0xFFFCFFFB, 0xFFFDFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFCFF, 0xF7F9FDF2, 0x00ADB799, 0x005A683F, 0x00818E68, 0x007F8B67, 0x00515F35, 0x00374717, 0x00586A41, 0x0064764D, 0x004C5E35, 0x003B4D24, 0x005A6C43,
        0x00596B42, 0x0036481F, 0x0032441B, 0x006D795D, 0x0057604C, 0x0062675E, 0x00BABCBD, 0x00E5E4ED, 0x00D8D7E0, 0x00D0D1D5, 0x00C7C9CA, 0x00D8D5DE, 0x00E5E3E9, 0x00C6C5C7, 0x005D5E5C, 0x0091948B, 0x005B6051, 0x005B6150, 0x00575E49, 0x0041482D,
        0x005F664B, 0x005F664B, 0x0052593E, 0x0051583D, 0x005E654A, 0x00656C51, 0x00747B60, 0x0058693E, 0x00627348, 0x0077885D, 0x00798A5F, 0x006E7F54, 0x00718257, 0x0077885D, 0x0077885D, 0x006E8059, 0x005D6F48, 0x004B5D36, 0x004D5F38, 0x00374922,
        0x0052643D, 0x005E7049, 0x00768861, 0x0074875C, 0x0073855C, 0x0073855C, 0x0073855C, 0x0074865F, 0x00758760, 0x00758661, 0x00768762, 0x00758563, 0x00758661, 0x00798B64, 0x0073835A, 0x0076865D, 0x00707C58, 0x004D5838, 0x003B4527, 0x00404732,
        0x00485137, 0x00525D3D, 0x005A6844, 0x006F7F56, 0x0071835A, 0x0073855E, 0x00748560, 0x005A694E, 0x00495B36, 0x004E5E3F, 0x00798278, 0x00616863, 0x00434F3D, 0x002A3622, 0x00575F58, 0x0078848A, 0x00536459, 0x002B4124, 0x00657956, 0x00657556,
        0x00657155, 0x005F694B, 0x0038421E, 0x00181E13, 0x00262A24, 0x00191F1A, 0x00131918, 0x000B1312, 0x00313D37, 0x003D4C44, 0x00455649, 0x0020392F, 0x002E473D, 0x003B5349, 0x002F473B, 0x002B4135, 0x00344B3D, 0x00334A3C, 0x00324639, 0x003C4A3F,
        0x002F3D32, 0x002F3D32, 0x0039473C, 0x00324035, 0x00313F34, 0x003C4A3F, 0x00334136, 0x00324237, 0x00304035, 0x002F3F34, 0x00324237, 0x0035453A, 0x0035453A, 0x00334338, 0x00334338, 0x00263D2E, 0x002F3A30, 0x00343531, 0x000C0F0D, 0x00071210,
        0x00121D1A, 0x00171B16, 0x00201A15, 0x00333E24, 0x00606C4E, 0x00667452, 0x0063754E, 0x00697B52, 0x006C7E57, 0x00697B56, 0x006B7C5A, 0x006B7D58, 0x006D7F5A, 0x006E805B, 0x006E805B, 0x006E805B, 0x006D7F5A, 0x006E805B, 0x006F815C, 0x00778058,
        0x006E8059, 0x00667F59, 0x006D855B, 0x00677A53, 0x0041533C, 0x00667C81, 0x0067859E, 0x00617BA3, 0x005C819B, 0x00416662, 0x00577056, 0x00344B3C, 0x00406171, 0x00597FA1, 0x006183A1, 0x005F809A, 0x003B5758, 0x00435A44, 0x00374F3D, 0x00547176,
        0x006F8DA8, 0x006786A5, 0x00718FAA, 0x004F6F7A, 0x00395946, 0x003E5A3D, 0x005E7674, 0x007894AC, 0x006085A1, 0x006A8EA6, 0x0054738C, 0x003B4E33, 0x00506545, 0x00546845, 0x00596E47, 0x004F623B, 0x003F512C, 0x004A5A38, 0x00475636, 0x00425135,
        0x0057673E, 0x004F6338, 0x002D4A33, 0x0049727B, 0x004F819D, 0x00538BA4, 0x00518895, 0x0044717C, 0x00223A30, 0x004A5241, 0x00747F7C, 0x00849199, 0x00D9E0E3, 0x00CDD3D2, 0x008D999F, 0x00718489, 0x00718489, 0x0075888D, 0x007F9297, 0x008A9DA2,
        0x008A9DA2, 0x007F9297, 0x0075888D, 0x007B898F, 0x00829096, 0x00879699, 0x00838F93, 0x007E8B8D, 0x00919D9F, 0x00ABB7B9, 0x00B7C3C3, 0x006E7E5F, 0x006D7E5D, 0x00768664, 0x00778765, 0x005B694D, 0xFFB5BDAC, 0xFFFBFFFA, 0xFFFDFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xFFFFFFF8, 0x00B1BB9D, 0x005C6A41, 0x00808D67, 0x007A8662, 0x004D5B31, 0x00314111, 0x002D3F16, 0x0036481F, 0x002A3C13, 0x0024360D, 0x0033451C,
        0x0034461D, 0x002C3E15, 0x00304219, 0x00707859, 0x00686E55, 0x00646855, 0x00AEAFA5, 0x00F3F3ED, 0x00E9E9E3, 0x00D7D8CF, 0x00DCDDD3, 0x00E6E6E0, 0x00EBECE3, 0x00C7C8BE, 0x00696C5D, 0x00B5B9A6, 0x00868C75, 0x007C8269, 0x00787F64, 0x007F8D64,
        0x0079875E, 0x00717F56, 0x006F7D54, 0x00738158, 0x0076845B, 0x0065734A, 0x004E5C33, 0x0054653A, 0x004E5F34, 0x006D7E53, 0x007B8C61, 0x00708156, 0x00738459, 0x0075865B, 0x00718257, 0x00788A63, 0x006A7C55, 0x0051633C, 0x004E6039, 0x00364821,
        0x00556740, 0x005D6F48, 0x0071835C, 0x0072855A, 0x0072845B, 0x0072845B, 0x0073855C, 0x0074865F, 0x0074865F, 0x00758661, 0x00768762, 0x00748564, 0x006F815C, 0x0074865D, 0x0072855A, 0x0076875B, 0x0076875C, 0x00627047, 0x0053603A, 0x00525A43,
        0x00545D42, 0x005E6C4A, 0x00707F59, 0x00798C61, 0x0072855A, 0x006E8258, 0x0075895F, 0x0079876F, 0x006C7D5B, 0x00425135, 0x00545C55, 0x004E5552, 0x00364131, 0x001C2816, 0x00787F7A, 0x00B4BEC5, 0x007D8B80, 0x00314429, 0x00596A49, 0x004A573D,
        0x004B5440, 0x00535A45, 0x002A3318, 0x00171913, 0x00222621, 0x001B2120, 0x00232D2D, 0x000F1919, 0x00172220, 0x00232D27, 0x002C372F, 0x00283A33, 0x00253730, 0x00293B34, 0x00283930, 0x00283930, 0x002C3D32, 0x002C3C31, 0x00314136, 0x002D3933,
        0x0026322C, 0x0027332D, 0x00313D37, 0x002C3832, 0x0028342E, 0x002F3B35, 0x0026322C, 0x002E3A34, 0x002B3731, 0x0028342E, 0x0025312B, 0x0024302A, 0x0028342E, 0x00323E38, 0x003C4842, 0x0028392E, 0x001C241D, 0x00202321, 0x00121718, 0x00202D2F,
        0x00253332, 0x00161B19, 0x00181310, 0x001A230E, 0x004E593F, 0x00546141, 0x005A6B46, 0x0070825B, 0x006E8059, 0x00677954, 0x006D7E5C, 0x006C7E59, 0x006E805B, 0x006F815C, 0x006F815C, 0x006E805B, 0x006D7F5A, 0x006E805B, 0x006E805B, 0x00788159,
        0x006F815A, 0x00687F59, 0x00688058, 0x00647855, 0x00405644, 0x00607F88, 0x006C93B3, 0x006F8FC0, 0x006F98B8, 0x00466E6C, 0x00516D4F, 0x0038533F, 0x004E7282, 0x006893BA, 0x006C93BA, 0x006F91AF, 0x00314E52, 0x003D5340, 0x00324836, 0x003D575D,
        0x006E8EAB, 0x006F93B7, 0x007699BB, 0x00628589, 0x0041634B, 0x002F4E2D, 0x003C5551, 0x007492AB, 0x006D93B3, 0x006E94B2, 0x006384A5, 0x003D5035, 0x00455A3A, 0x00516542, 0x005E734C, 0x0050633C, 0x003C4E29, 0x00485836, 0x004F5E3E, 0x00556448,
        0x00627249, 0x0052663B, 0x002D4A33, 0x004B747D, 0x005284A0, 0x00548CA5, 0x00528996, 0x004A7782, 0x00273F35, 0x004A5241, 0x00737E7B, 0x00828F97, 0x00D6DDE0, 0x00CDD3D2, 0x008C989E, 0x007C8F94, 0x007C8F94, 0x007E9196, 0x0084979C, 0x008A9DA2,
        0x008A9DA2, 0x0086999E, 0x00819499, 0x00849298, 0x0089979D, 0x00869598, 0x007D898D, 0x007A8789, 0x008C989A, 0x00ADB9BB, 0x00C8D4D4, 0x0085937B, 0x006E7D61, 0x00748462, 0x00778863, 0x005D6C4D, 0xFFB5BDAC, 0xFFFAFEF9, 0xFFFCFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xFFFFFFF8, 0x00B1BB9D, 0x005C6A41, 0x007D8A64, 0x00798561, 0x005C6A40, 0x004A5A2A, 0x004B5D34, 0x0045572E, 0x004B5D34, 0x0046582F, 0x003E5027,
        0x0041532A, 0x0042542B, 0x0046582F, 0x00717D53, 0x006E7953, 0x005D6645, 0x007A8263, 0x009EA688, 0x008C9476, 0x00878F71, 0x009DA586, 0x008D9577, 0x008C9476, 0x00868E6F, 0x00515A39, 0x007B8561, 0x006D7852, 0x006D7850, 0x007B865E, 0x00738256,
        0x00707F53, 0x0079885C, 0x007D8C60, 0x0077865A, 0x0077865A, 0x00758458, 0x00667549, 0x004D5E33, 0x004B5C31, 0x0054653A, 0x00637449, 0x00798A5F, 0x007D8E63, 0x006F8055, 0x0077885D, 0x00768861, 0x006F815A, 0x00566841, 0x0051633C, 0x00374922,
        0x00576942, 0x005D6F48, 0x0072845D, 0x00718459, 0x0071835A, 0x0072845B, 0x0073855C, 0x0073855E, 0x0074865F, 0x00748560, 0x00748560, 0x00718260, 0x0072845F, 0x00768A60, 0x00708457, 0x006D8154, 0x0077885C, 0x00798960, 0x00788761, 0x0078826B,
        0x00768266, 0x00768462, 0x00768861, 0x0073865B, 0x00768A5F, 0x0075895F, 0x0070845A, 0x006F7C64, 0x00798967, 0x00475539, 0x004C534C, 0x00515654, 0x003F4939, 0x00151F0E, 0x005F6560, 0x006F7679, 0x00505C50, 0x00253419, 0x00687859, 0x0064705A,
        0x006E7566, 0x0072796A, 0x003C4530, 0x00191A16, 0x001B201E, 0x00212929, 0x003B4749, 0x00182426, 0x000B1515, 0x00111815, 0x00171B16, 0x00141B14, 0x000E150E, 0x00151A11, 0x00171C13, 0x001B1F14, 0x001A1E13, 0x0013170C, 0x00181A0E, 0x00131B14,
        0x00141C15, 0x00111912, 0x00161E17, 0x00171F18, 0x00121A13, 0x00141C15, 0x00121A13, 0x000D1711, 0x00101A14, 0x00121C16, 0x00111B15, 0x00101A14, 0x00111B15, 0x00111B15, 0x00111B15, 0x000E1C11, 0x00171D18, 0x00191B1B, 0x001A2124, 0x00516364,
        0x00415352, 0x000D1312, 0x00201B1A, 0x00151D0C, 0x004F5843, 0x004E5A3E, 0x004B5B39, 0x00697A55, 0x00687A53, 0x00677954, 0x0070815F, 0x006D7F5A, 0x006E805B, 0x006F815C, 0x006F815C, 0x006E805B, 0x006E805B, 0x006E805B, 0x006F815C, 0x00788159,
        0x006D7E59, 0x006C805D, 0x00657A54, 0x006C7D5B, 0x00475F4D, 0x00567983, 0x00719EBF, 0x007693C0, 0x007AA2BB, 0x0054786C, 0x00657D55, 0x00506649, 0x0060818A, 0x00729BBC, 0x006D93B5, 0x007C9BB2, 0x00465D59, 0x00546347, 0x00465539, 0x00495E5B,
        0x00819DB5, 0x007A9BBC, 0x00769AB8, 0x00799490, 0x00476443, 0x00466038, 0x003E5448, 0x00768FA3, 0x007DA0BA, 0x00789CB4, 0x006E8CA7, 0x004C5F44, 0x00495E3E, 0x00556946, 0x00596E47, 0x004C5F38, 0x004A5C37, 0x00536341, 0x004A5939, 0x00657458,
        0x00607047, 0x00465A2F, 0x0027442D, 0x004B747D, 0x005385A1, 0x00558DA6, 0x00548B98, 0x004B7883, 0x00294137, 0x0048503F, 0x00727D7A, 0x00818E96, 0x00D4DBDE, 0x00CFD5D4, 0x008E9AA0, 0x0085989D, 0x0085989D, 0x0085989D, 0x0084979C, 0x0085989D,
        0x0085989D, 0x00879A9F, 0x00889BA0, 0x00829096, 0x0087959B, 0x00839295, 0x007B878B, 0x00778486, 0x00859193, 0x00A9B5B7, 0x00CDD9D9, 0x00BAC6B4, 0x007D8B73, 0x00728161, 0x00768861, 0x00606F4F, 0xFFB5BEAA, 0xFFFCFFFB, 0xFFFDFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBF9FF, 0xFFFDFFF6, 0x00ACB698, 0x0059673E, 0x007C8963, 0x007F8B67, 0x0076845A, 0x00748454, 0x0076885F, 0x006D7F56, 0x00788A61, 0x0076885F, 0x0073855C,
        0x00788A61, 0x0072845B, 0x00788A61, 0x007B885C, 0x00727F53, 0x006B784C, 0x00717E52, 0x00738054, 0x00667347, 0x00748155, 0x00808D61, 0x00778758, 0x00687849, 0x00778758, 0x006C7C4D, 0x00657546, 0x006E7E4F, 0x00788859, 0x00809061, 0x007B8D5E,
        0x0077895A, 0x00758758, 0x00728455, 0x007A8C5D, 0x00798B5C, 0x00798B5C, 0x00748657, 0x005F7045, 0x004D5E33, 0x00495A2F, 0x0055663B, 0x00697A4F, 0x0077885D, 0x0077885D, 0x0076875C, 0x0070825B, 0x006D7F58, 0x0052643D, 0x004E6039, 0x0033451E,
        0x00576942, 0x005E7049, 0x00798B64, 0x00718459, 0x0072845B, 0x0072845B, 0x0073855C, 0x0074865F, 0x0074865F, 0x00748560, 0x0073845F, 0x006E835D, 0x00748760, 0x0074885D, 0x006F8358, 0x00718459, 0x00788A61, 0x00768460, 0x00697656, 0x0069725D,
        0x00646F55, 0x00647353, 0x0070815C, 0x006D8157, 0x0071855B, 0x006F825B, 0x006B805A, 0x00707D63, 0x006B7C57, 0x00455435, 0x004D554B, 0x003C423D, 0x00242E1D, 0x001B2612, 0x005D645D, 0x006F7574, 0x004F5A4A, 0x00223112, 0x00647353, 0x00626E58,
        0x007B8275, 0x00787F72, 0x00313928, 0x00181A14, 0x001A201B, 0x0025302E, 0x00415150, 0x001A2A29, 0x000F1A18, 0x00171B16, 0x001D1D17, 0x00353426, 0x00343325, 0x00383527, 0x00322F20, 0x00332E1F, 0x00383422, 0x00322E1C, 0x00302C1A, 0x00282F20,
        0x00293021, 0x001F2617, 0x001D2415, 0x001E2516, 0x00192011, 0x001C2314, 0x001D2415, 0x001C2217, 0x001B2116, 0x00191F14, 0x001A2015, 0x001E2419, 0x0022281D, 0x0022281D, 0x001F251A, 0x001E271D, 0x00191D17, 0x00141917, 0x000C1818, 0x003C5250,
        0x00374C4A, 0x000C1512, 0x00231E1B, 0x001B2215, 0x00626A59, 0x005F6952, 0x004E5D3D, 0x00697A55, 0x00697A55, 0x006C7E59, 0x006D7F5A, 0x006C7E59, 0x006D7F5A, 0x006F815C, 0x006F815C, 0x006E805B, 0x006E805B, 0x006E805B, 0x006F815C, 0x0073805A,
        0x006C7C5A, 0x0070815F, 0x006A7C55, 0x0075835F, 0x004F644E, 0x0056797D, 0x007BAAC5, 0x007794B9, 0x007EA4B6, 0x005A7A69, 0x00708155, 0x00596947, 0x00678488, 0x007BA3BC, 0x00749CB5, 0x0085A1B2, 0x005B7067, 0x0054633D, 0x00414E28, 0x004F6055,
        0x008AA4B4, 0x007799B6, 0x007BA0BC, 0x00768D88, 0x003D5533, 0x0061754A, 0x00576A5B, 0x007A94A2, 0x0081A2B6, 0x007DA1B3, 0x006A899E, 0x0055684D, 0x004C6141, 0x005E724F, 0x005F744D, 0x004B5E37, 0x004A5C37, 0x00536341, 0x00505F3F, 0x00637256,
        0x0053633A, 0x00394D22, 0x00234029, 0x0048717A, 0x005284A0, 0x00578FA8, 0x00558C99, 0x0047747F, 0x00263E34, 0x00444C3B, 0x00707B78, 0x00808D95, 0x00D4DBDE, 0x00D4DAD9, 0x00929EA4, 0x00879A9F, 0x00879A9F, 0x00879A9F, 0x00879A9F, 0x0086999E,
        0x00879A9F, 0x00879A9F, 0x00889BA0, 0x0086949A, 0x0086949A, 0x00839295, 0x007C888C, 0x00778486, 0x00879395, 0x00B0BCBE, 0x00DAE6E6, 0x00D3DDD1, 0x007E8A76, 0x006F7E5E, 0x00768861, 0x00637351, 0xFFB7C0AB, 0xFFFDFFFC, 0xFFFDFCFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xFFFFFFF8, 0x00ADB799, 0x005C6A41, 0x007D8A64, 0x007D8965, 0x007A885E, 0x007F8F5F, 0x00788A61, 0x0073855C, 0x0075875E, 0x0076885F, 0x00778960,
        0x00788A61, 0x0073855C, 0x0074865D, 0x007A8963, 0x0073835A, 0x00798A5F, 0x007E8F63, 0x00809263, 0x007B8D5E, 0x0087986C, 0x007F9064, 0x00839867, 0x00778B5C, 0x007A8E5F, 0x00819566, 0x00728659, 0x00768A5D, 0x00788B60, 0x0072855A, 0x00798A5F,
        0x00798A5F, 0x0078895E, 0x006F8055, 0x0076875C, 0x00728358, 0x0075865B, 0x0076875C, 0x0077885D, 0x005C6D42, 0x00536439, 0x0056673C, 0x00526338, 0x0066774C, 0x007D8E63, 0x00728358, 0x0071835C, 0x006D7F58, 0x004F613A, 0x004D5F38, 0x0032441D,
        0x00576942, 0x005D6F48, 0x007B8D66, 0x0073865B, 0x0074865D, 0x0075875E, 0x0075875E, 0x00758760, 0x00758760, 0x00748560, 0x00748560, 0x0070865C, 0x006F855B, 0x006E8257, 0x00708459, 0x0070835C, 0x00627151, 0x004A583C, 0x0037442C, 0x00283221,
        0x002B3721, 0x00313F23, 0x004D5E3D, 0x0062744F, 0x006F815C, 0x00687C59, 0x006B7F5C, 0x00667556, 0x0073855C, 0x0050603E, 0x00646D60, 0x00788076, 0x005B6751, 0x005D6A52, 0x00A8B0A5, 0x00B6BDB6, 0x0088937F, 0x003E4E2C, 0x00566644, 0x003F4C34,
        0x006A7366, 0x007C8578, 0x00424C3C, 0x0014180D, 0x00131A13, 0x00182620, 0x00304340, 0x00122320, 0x000F1B15, 0x0022241E, 0x003F3D33, 0x0050533A, 0x0044472E, 0x003B3E25, 0x0033331B, 0x0036371D, 0x00403F25, 0x003D3C20, 0x003D3C20, 0x003F422C,
        0x003D402A, 0x00353822, 0x00343721, 0x0030331D, 0x002F321C, 0x00373A24, 0x00343721, 0x002A2E1B, 0x00232714, 0x001F2310, 0x00212512, 0x00272B18, 0x002F3320, 0x003F4330, 0x004D513E, 0x002C3528, 0x000F140B, 0x00181E19, 0x00162521, 0x002F4844,
        0x002E4842, 0x00111B15, 0x0015110C, 0x0021271C, 0x00666D5E, 0x00626B56, 0x00526142, 0x006F7F5D, 0x006C7D58, 0x006D7F5A, 0x006A7C57, 0x006A7C57, 0x006C7E59, 0x006E805B, 0x006F815C, 0x006E805B, 0x006E805B, 0x006F815C, 0x0070825D, 0x00728260,
        0x006D7E5D, 0x006D7E5C, 0x00717F56, 0x00737C54, 0x00415237, 0x00547370, 0x0078A4B5, 0x006D93B3, 0x00759DAF, 0x004C6D5E, 0x00606F49, 0x0048563A, 0x005B797E, 0x007AA3BC, 0x00709CB3, 0x0080A1B4, 0x005D766C, 0x004C5E37, 0x00394822, 0x00435A4C,
        0x007E9FAF, 0x007097B7, 0x0076A3C4, 0x006B8488, 0x00546D53, 0x00546C49, 0x0033483F, 0x007995A6, 0x00769DB3, 0x006D98A9, 0x007A9FB3, 0x004B5E43, 0x004E6343, 0x00667A57, 0x00627750, 0x004E613A, 0x004B5D38, 0x0050603E, 0x00556444, 0x0057664A,
        0x0048582F, 0x00394D22, 0x0027442D, 0x00466F78, 0x004F819D, 0x005991AA, 0x00538A97, 0x0045727D, 0x00263E34, 0x00424A39, 0x006F7A77, 0x00808D95, 0x00D4DBDE, 0x00D5DBDA, 0x00939FA5, 0x00819499, 0x00819499, 0x0084979C, 0x008A9DA2, 0x008FA2A7,
        0x008EA1A6, 0x00879A9F, 0x00819499, 0x0088969C, 0x00849298, 0x00829194, 0x007D898D, 0x00738082, 0x00808C8E, 0x00ACB8BA, 0x00D5E1E1, 0x00D2DAD3, 0x00788472, 0x00707F5F, 0x00788A61, 0x0063744F, 0xFFB7C0AB, 0xFFFDFFFC, 0xFFFAF9FF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xFFFFFFF8, 0x00ACB698, 0x005E6C43, 0x00808D67, 0x007C8864, 0x0078865C, 0x007D8D5D, 0x007A8C63, 0x00778960, 0x0073855C, 0x00798B62, 0x0075875E,
        0x0076885F, 0x007F9168, 0x00798B62, 0x00758465, 0x00788767, 0x007B8C67, 0x0075875E, 0x00778960, 0x006B7D54, 0x00687A51, 0x005D6F48, 0x00596E41, 0x00687C51, 0x00607449, 0x006D8157, 0x00798C65, 0x0072845F, 0x00738560, 0x00728361, 0x0074845B,
        0x006F7F56, 0x0077875E, 0x0075855C, 0x00798960, 0x0077875E, 0x007A8A61, 0x006F7F56, 0x00798A5F, 0x0076875C, 0x005C6D42, 0x004F6035, 0x00526338, 0x005C6D42, 0x00728358, 0x007A8B60, 0x00768861, 0x0070825B, 0x004F613A, 0x004F613A, 0x00354720,
        0x00596B44, 0x005B6D46, 0x00798B64, 0x0075885D, 0x0076885F, 0x00778960, 0x00778960, 0x00778962, 0x00768861, 0x00758661, 0x00748560, 0x00738A5E, 0x006E8559, 0x006A8155, 0x0070845A, 0x0061734E, 0x00374529, 0x001D2913, 0x001E2817, 0x00404A3A,
        0x00444F3B, 0x0027341A, 0x00203011, 0x003E4F2D, 0x00687C59, 0x006C7F5E, 0x006B7D5E, 0x00778464, 0x006C7D52, 0x00475531, 0x004D5445, 0x00555B50, 0x004F5942, 0x00566147, 0x00545B4E, 0x00383E33, 0x00404A33, 0x0035431F, 0x0063714D, 0x003F4C32,
        0x00646E5E, 0x00737C6F, 0x003E4838, 0x0026281C, 0x00212A20, 0x0026372E, 0x003D534E, 0x001F332E, 0x00101D15, 0x001B1E15, 0x004A463B, 0x00586343, 0x003B4324, 0x00293112, 0x00283110, 0x00343B1A, 0x00373E1D, 0x00313714, 0x00343A17, 0x002E3013,
        0x002A2C0F, 0x002C2E11, 0x00333518, 0x002D2F12, 0x00313316, 0x003E4023, 0x0036381B, 0x0031341B, 0x002D3017, 0x0030331A, 0x00363920, 0x0033361D, 0x0032351C, 0x00474A31, 0x0062654C, 0x003F4637, 0x001D2116, 0x00192019, 0x00182A23, 0x003F5B55,
        0x003B564D, 0x0017241C, 0x002B2820, 0x00242920, 0x00585F52, 0x004D5641, 0x00465438, 0x006B7B59, 0x00657651, 0x00697B56, 0x006B7D58, 0x00697B56, 0x006B7D58, 0x006D7F5A, 0x006E805B, 0x006E805B, 0x006E805B, 0x006F815C, 0x0070825D, 0x00718261,
        0x006C7E5F, 0x006D7D5B, 0x007D865E, 0x00767D52, 0x003D4C2D, 0x00638077, 0x0083ADB9, 0x006B96B7, 0x00709DB2, 0x0043685E, 0x00586948, 0x0041503B, 0x00597984, 0x0078A7C2, 0x00689AB0, 0x00739CB2, 0x0051726B, 0x00526845, 0x004B603A, 0x00456056,
        0x0078A0B3, 0x006A9ABE, 0x00689CC1, 0x005F7A88, 0x004D6A5A, 0x00506C4F, 0x002E4946, 0x007395AC, 0x0074A2BA, 0x0071A3B7, 0x005F8EA4, 0x004E6146, 0x00647959, 0x00748865, 0x00586D46, 0x004A5D36, 0x00546641, 0x00475735, 0x00364525, 0x004D5C40,
        0x0045552C, 0x0041552A, 0x00304D36, 0x00466F78, 0x004D7F9B, 0x005991AA, 0x00518895, 0x0047747F, 0x00273F35, 0x00424A39, 0x006F7A77, 0x007F8C94, 0x00D2D9DC, 0x00D3D9D8, 0x00919DA3, 0x0075888D, 0x0075888D, 0x007B8E93, 0x00889BA0, 0x0094A7AC,
        0x0092A5AA, 0x0083969B, 0x0075888D, 0x007A888E, 0x0079878D, 0x00819093, 0x00859195, 0x007C898B, 0x00869294, 0x00AEBABC, 0x00D3DFDF, 0x00D9E0DB, 0x007F8A7A, 0x00788768, 0x00798B62, 0x005E7049, 0xFFB4BEA7, 0xFFFDFFFC, 0xFFFDFCFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFBFF, 0xFFFFFFF9, 0x00ADB799, 0x005D6A44, 0x007D8A64, 0x00808D67, 0x00647347, 0x00637441, 0x005E7047, 0x00687A51, 0x00576940, 0x0066794E, 0x0065784D,
        0x00607447, 0x00596D40, 0x0063774A, 0x00718162, 0x0058664E, 0x00485442, 0x00646E62, 0x00505A4D, 0x003A4634, 0x00324126, 0x003A4A2B, 0x00313B2B, 0x00353F2E, 0x00313D27, 0x00495539, 0x00485634, 0x0057653C, 0x00728155, 0x00819162, 0x0076845B,
        0x00708050, 0x00768952, 0x00758555, 0x00828F69, 0x00727D5D, 0x0075825C, 0x0079895A, 0x0077875E, 0x007A8A61, 0x00798960, 0x0058683F, 0x004E5E35, 0x0058683F, 0x005A6A41, 0x0078885F, 0x00798C61, 0x0073855C, 0x004E6037, 0x004B5D34, 0x00384A23,
        0x0053653E, 0x0061724D, 0x007B8C67, 0x00758760, 0x00758760, 0x00758760, 0x00758760, 0x00758760, 0x0074865F, 0x0073855E, 0x0073855E, 0x00738462, 0x0070835C, 0x0071845D, 0x0069785C, 0x002E3929, 0x00242D23, 0x003B4735, 0x003B4A2E, 0x0058614D,
        0x004F5A46, 0x00384231, 0x00303A29, 0x0024311B, 0x0039482C, 0x006D815E, 0x006C815A, 0x0072865C, 0x00677954, 0x004B5C3B, 0x0049583C, 0x00425136, 0x002D3C20, 0x00223312, 0x002D3F1A, 0x003F5231, 0x00243716, 0x002D3E1C, 0x005B6A4A, 0x004A563A,
        0x006D745F, 0x007F8475, 0x00505147, 0x003B4025, 0x002B3420, 0x001F2A22, 0x002D3A3C, 0x000F1B21, 0x0012191C, 0x001D211C, 0x002A291F, 0x002B371B, 0x003D4C26, 0x00425425, 0x0036471C, 0x002B3719, 0x002D381E, 0x003F4D2B, 0x0055663A, 0x004E5D3D,
        0x00414E36, 0x001C2616, 0x00242822, 0x00222620, 0x00222718, 0x002C341D, 0x00364121, 0x00343623, 0x0030321E, 0x002F321C, 0x0030361F, 0x002F341F, 0x002D3020, 0x002C3025, 0x0030332A, 0x001B2319, 0x001D1F19, 0x001C1D1B, 0x0016211F, 0x00374C49,
        0x002B413C, 0x00152018, 0x002C2921, 0x00252C1F, 0x00545C4B, 0x00606A53, 0x004C5B3B, 0x006D7E59, 0x00687A53, 0x00687B54, 0x00687B54, 0x006C8055, 0x0071855A, 0x006E8258, 0x006E8258, 0x0072855E, 0x0070835C, 0x0070835C, 0x00738560, 0x00738263,
        0x006D7D5B, 0x006B7F55, 0x00738A5E, 0x00708761, 0x0051664D, 0x0067786D, 0x00879895, 0x007698A5, 0x0078979A, 0x00617D70, 0x005F7961, 0x003A543C, 0x006C8A7D, 0x007B9EA1, 0x007CA4B0, 0x008499A1, 0x00526A6A, 0x00526F60, 0x0042624A, 0x0042614C,
        0x0078928C, 0x008FA2AF, 0x008F9EB8, 0x00637F68, 0x0057735F, 0x004D6858, 0x00375046, 0x006E8783, 0x00869D9F, 0x0092AAB0, 0x00637A82, 0x005C6A46, 0x006E7F5A, 0x00748560, 0x005E6F4A, 0x00455732, 0x00495B36, 0x004F643E, 0x003C512B, 0x00435635,
        0x00505E3A, 0x00586642, 0x00354C36, 0x004C7272, 0x00578B9C, 0x00558DA6, 0x005187A0, 0x003F737F, 0x00213C33, 0x003D4737, 0x006F7776, 0x00818D93, 0x00D1D9D9, 0x00CFD8D5, 0x008FA0A3, 0x006E868C, 0x00657D83, 0x006C848A, 0x007A9298, 0x008AA2A8,
        0x008DA5AB, 0x007D959B, 0x00768E94, 0x0078889F, 0x00768697, 0x0082939C, 0x00839596, 0x007D8C88, 0x00889593, 0x00AEB8B8, 0x00D8E1E4, 0x00D4DAD9, 0x007E887B, 0x007B8A6B, 0x00788A61, 0x005D6F48, 0xFFB4BEA7, 0xFFFDFFFB, 0xFFFDFCFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBF7FF, 0xFFFFFFF9, 0x00ACB698, 0x005E6B45, 0x00818E68, 0x007F8C66, 0x004F5E32, 0x00394A17, 0x0031431A, 0x002F4118, 0x0032441B, 0x0035481D, 0x002C3F14,
        0x00304417, 0x00314518, 0x00364A1D, 0x006F805F, 0x00526146, 0x004E5A48, 0x00767F75, 0x00A2ABA1, 0x00ADB7AB, 0x00B0BCAA, 0x00AFBCA6, 0x00B3BBB1, 0x00B7C0B3, 0x00A3AD9C, 0x005C6550, 0x00394529, 0x004D5837, 0x007B8862, 0x00707E55, 0x007C8864,
        0x007D8D5E, 0x007E905B, 0x00778757, 0x00798660, 0x00798463, 0x007E8C63, 0x007C8D5A, 0x0078885F, 0x006C7C53, 0x007E8E65, 0x0078885F, 0x005B6B42, 0x0054643B, 0x0054643B, 0x0057673E, 0x00708358, 0x0072845B, 0x0055673E, 0x00506239, 0x00384A23,
        0x00556740, 0x0062734E, 0x007A8B66, 0x00758760, 0x00758760, 0x00758760, 0x00758760, 0x00758760, 0x0074865F, 0x0073855E, 0x0073855E, 0x0070815F, 0x0073865F, 0x006F825B, 0x0048573B, 0x00263121, 0x003E473D, 0x00414D3B, 0x0037462A, 0x004F5843,
        0x0046513D, 0x003B4534, 0x003B4632, 0x00293620, 0x0029381C, 0x00516443, 0x00687D57, 0x00687C52, 0x00778964, 0x00677758, 0x005F6D55, 0x0064725A, 0x0067765B, 0x00637453, 0x003D4F2A, 0x00536645, 0x00435635, 0x00556644, 0x00697858, 0x004D593D,
        0x00707762, 0x00727768, 0x0048493F, 0x00393D24, 0x002C3423, 0x00242F27, 0x00323F41, 0x00111D21, 0x00101818, 0x00171B16, 0x00222018, 0x00141B0C, 0x002F3922, 0x003B4828, 0x002A361A, 0x00141C0B, 0x00141B0E, 0x00323B26, 0x00556143, 0x006B7B59,
        0x004C593F, 0x00162010, 0x00191E15, 0x001E221C, 0x0022261B, 0x00232916, 0x00222A13, 0x00252812, 0x002A2D17, 0x002F331A, 0x002B311A, 0x00232714, 0x001A1E12, 0x001A1C16, 0x001E1F1D, 0x001A2218, 0x001C1E18, 0x001C1D1B, 0x0016211F, 0x00384D4A,
        0x002C423D, 0x00141F17, 0x0029261E, 0x002D3427, 0x005D6554, 0x00616B54, 0x004B5A3A, 0x006E7F5A, 0x00687A53, 0x006B7E57, 0x006B7E57, 0x006B7F54, 0x006C8055, 0x006C8056, 0x006F8359, 0x0070835C, 0x006B7E57, 0x006B7D58, 0x006E805B, 0x00768462,
        0x0076855F, 0x0072845B, 0x0073875D, 0x00718562, 0x004E6146, 0x003A4C3B, 0x002E3E33, 0x003C534E, 0x004C6355, 0x00566950, 0x00748363, 0x00556645, 0x00657A61, 0x00506A5C, 0x0044635C, 0x006A7872, 0x00556658, 0x0062775E, 0x00516848, 0x003C5235,
        0x00576A55, 0x006B7A72, 0x00717D7D, 0x00495B3E, 0x00586B50, 0x0060735A, 0x00465742, 0x00526453, 0x00536457, 0x005F7065, 0x0046574E, 0x0055633F, 0x0071825D, 0x0071825D, 0x005C6D48, 0x00566843, 0x004B5D38, 0x003D522C, 0x004E633D, 0x00485B3A,
        0x005F6D49, 0x00566440, 0x00374E38, 0x00527878, 0x00538798, 0x00548CA5, 0x004E849D, 0x00447884, 0x00254037, 0x00414B3B, 0x00717978, 0x00818D93, 0x00D1D9D9, 0x00D0D9D6, 0x0091A2A5, 0x0083969D, 0x0082959C, 0x008A9DA4, 0x008C9FA6, 0x008EA1A8,
        0x008EA1A8, 0x00899CA3, 0x008B9EA5, 0x008394A7, 0x0080919E, 0x008799A0, 0x00819394, 0x00798785, 0x00869391, 0x00ABB4B7, 0x00D4DADF, 0x00D1D8D3, 0x00778272, 0x00728162, 0x007A8C63, 0x00667851, 0xFFB7C1AA, 0xFFFCFFFA, 0xFFFBFBFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDF9FF, 0xFFFFFFF9, 0x00ABB597, 0x005C6943, 0x00818E68, 0x00818E68, 0x004C5B2F, 0x002F400D, 0x0051633A, 0x0052643B, 0x00475930, 0x00203308, 0x0035481D,
        0x00475B2E, 0x00304417, 0x0035491C, 0x006E7E5C, 0x004A583C, 0x00545E4E, 0x0099A099, 0x00EAEFEE, 0x00ECF1EF, 0x00EBF1EC, 0x00E8F0E6, 0x00E5EAE9, 0x00ECF1EF, 0x00C9D0C9, 0x005D6457, 0x005C6551, 0x00545D43, 0x00687254, 0x006F7A5A, 0x00656D56,
        0x005C6746, 0x00556339, 0x005E6C43, 0x00606B4A, 0x00606B4B, 0x005B6940, 0x005D6E3B, 0x0078885F, 0x00718158, 0x00728259, 0x0077875E, 0x00708057, 0x005B6B42, 0x004F5F36, 0x0054643B, 0x005C6F44, 0x0064764D, 0x0052643B, 0x004F6138, 0x0034461F,
        0x0053653E, 0x0061724D, 0x00778863, 0x00758760, 0x00758760, 0x00758760, 0x00758760, 0x00758760, 0x0074865F, 0x0074865F, 0x0073855E, 0x0070815F, 0x0073865F, 0x00687B54, 0x002A391D, 0x002B3626, 0x00565F55, 0x003C4836, 0x002E3D21, 0x00364029,
        0x002C3822, 0x00303B27, 0x00404B37, 0x0035422C, 0x0028371C, 0x003C4F2E, 0x006D815E, 0x0073875D, 0x006C7D5B, 0x005B6A4F, 0x006C7864, 0x006A7662, 0x0047553D, 0x005B6B4C, 0x00495B36, 0x00526544, 0x00455837, 0x00637452, 0x005E6D4D, 0x00354125,
        0x00676E59, 0x00676C5D, 0x00323329, 0x00212310, 0x001F2617, 0x00232D27, 0x003A4847, 0x001A2729, 0x00141C1C, 0x00151914, 0x001A1810, 0x001D201E, 0x0022271E, 0x001E2516, 0x00171D12, 0x00171B16, 0x001C1F1D, 0x00252923, 0x0030372A, 0x00647550,
        0x0048563A, 0x00151F0E, 0x00171C13, 0x001E221D, 0x00262822, 0x00282C21, 0x00272B1F, 0x00262A11, 0x00353920, 0x003E442B, 0x00383E27, 0x00262C1B, 0x00191D12, 0x00181917, 0x00191B1C, 0x00182016, 0x001A1B17, 0x001A1B19, 0x00172121, 0x003A4F4D,
        0x002D433E, 0x00121C16, 0x00242119, 0x00242B1E, 0x005A6251, 0x0059634C, 0x00465535, 0x006E7F5A, 0x00687A53, 0x006D8059, 0x006D8059, 0x006B7F55, 0x00697D53, 0x006C8056, 0x006E815A, 0x006D8059, 0x006B7D58, 0x006C7E59, 0x0070825D, 0x00717F55,
        0x00728057, 0x0075855C, 0x007A8864, 0x006E7E5C, 0x00465637, 0x00324126, 0x002D3B23, 0x00344425, 0x00475232, 0x0059603B, 0x0082855E, 0x00686D46, 0x005E6943, 0x00384727, 0x002E4323, 0x00555D46, 0x005E674C, 0x00747F5E, 0x0065724C, 0x00414F26,
        0x003B4822, 0x00404B2A, 0x00434D2F, 0x00485130, 0x00687150, 0x00777F60, 0x005C6747, 0x00475133, 0x00354123, 0x003C482A, 0x00333F23, 0x0054623E, 0x0070815C, 0x0070815C, 0x00677853, 0x004C5E39, 0x00455732, 0x00596E48, 0x00445933, 0x00485B3A,
        0x0063714D, 0x004B5935, 0x00304731, 0x004F7575, 0x004F8394, 0x00568EA7, 0x005288A1, 0x00447884, 0x00254037, 0x00424C3C, 0x00727A79, 0x007F8B91, 0x00CFD7D7, 0x00D2DBD8, 0x0093A4A7, 0x0091A3AA, 0x0090A2A9, 0x0094A6AD, 0x0092A4AB, 0x0094A6AD,
        0x009BADB4, 0x0099ABB2, 0x009CAEB5, 0x0096A8B3, 0x0090A3AA, 0x0090A3A6, 0x00819190, 0x00748281, 0x00889494, 0x00B4BDC1, 0x00DDE3E8, 0x00DEE7DD, 0x00818D7B, 0x00707F60, 0x00798B64, 0x00667752, 0xFFB4BEA7, 0xFFFCFFFA, 0xFFFDFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFAFF, 0xFFFFFFF9, 0x00ACB698, 0x005C6943, 0x007E8B65, 0x007E8B65, 0x00526135, 0x0040511E, 0x006A7A51, 0x00607047, 0x005D6D44, 0x0034451A, 0x005D6E43,
        0x006B7C50, 0x003C4D21, 0x003C4D21, 0x00677853, 0x004E5C40, 0x00545E4E, 0x00AAAFAD, 0x00E4E7EB, 0x00D0D3D8, 0x00C6C9CD, 0x00BABEBF, 0x00D3D6DB, 0x00DCDFE3, 0x00D6DBDA, 0x00686C66, 0x00474E41, 0x00414736, 0x00515843, 0x00424A33, 0x00575854,
        0x00535849, 0x003B442A, 0x003A4329, 0x00363C29, 0x003C422F, 0x003A4426, 0x004D5B32, 0x0073835A, 0x0076865D, 0x006C7C53, 0x0074845B, 0x007F8F66, 0x006E7E55, 0x005C6C43, 0x0053633A, 0x004E6136, 0x0051633A, 0x00485A31, 0x004E6037, 0x0034461F,
        0x0053653E, 0x0061724D, 0x00798A65, 0x0074865F, 0x0074865F, 0x00758760, 0x00758760, 0x00758760, 0x00758760, 0x0074865F, 0x0074865F, 0x00748563, 0x0073865F, 0x005C6F48, 0x00243317, 0x00424D3D, 0x005A6359, 0x002C3826, 0x00253418, 0x00303B21,
        0x0026331B, 0x002A3521, 0x003B4632, 0x003D4A34, 0x002F3E23, 0x00293B1C, 0x00617453, 0x0070845A, 0x00687958, 0x00414E38, 0x00465141, 0x00485245, 0x0025311D, 0x00516044, 0x00495B36, 0x005E7150, 0x00425534, 0x00637452, 0x00687757, 0x003E4A2E,
        0x006A715C, 0x006F7465, 0x0035362C, 0x0018180A, 0x00161C11, 0x001A241E, 0x002F3D3B, 0x00111F1E, 0x00121A19, 0x001A1E19, 0x0024211C, 0x001F2220, 0x001F231E, 0x001B1F19, 0x001C201A, 0x00242725, 0x00242725, 0x001E221D, 0x001C201A, 0x004C5C3A,
        0x003C4A2E, 0x0018230F, 0x001B2116, 0x001C201B, 0x00212220, 0x001F201E, 0x001F201C, 0x00242A11, 0x00363C23, 0x00444A31, 0x003B422D, 0x00292E1F, 0x001C1E18, 0x0017191A, 0x0018181E, 0x00171E17, 0x00181915, 0x00191919, 0x00182222, 0x003C514F,
        0x002E4340, 0x00101A14, 0x001F1B16, 0x00181F12, 0x00555D4C, 0x00566049, 0x00465535, 0x0070815C, 0x00687A53, 0x006C7F58, 0x006A7D56, 0x006C8056, 0x006A7E54, 0x006F825B, 0x006D8059, 0x006B7D58, 0x0071835E, 0x00738560, 0x00758762, 0x007C8B5F,
        0x00728056, 0x00717E58, 0x0077835F, 0x00717F5D, 0x005B6947, 0x00546240, 0x00586642, 0x00657147, 0x00687248, 0x00656A43, 0x007D8059, 0x006D7049, 0x00656D45, 0x004D5B31, 0x00516439, 0x004B5635, 0x005C6746, 0x006D7857, 0x006B7753, 0x005A6741,
        0x00515F35, 0x00556536, 0x00546434, 0x005B6142, 0x00717758, 0x00737B5C, 0x006C7554, 0x005A6642, 0x0056623E, 0x0056653F, 0x00505F39, 0x005C6A46, 0x00758661, 0x0072835E, 0x00556641, 0x004A5C37, 0x004F613C, 0x004D623C, 0x004A5F39, 0x003D502F,
        0x00515F3B, 0x003F4D29, 0x002A412B, 0x00486E6E, 0x00518596, 0x00578FA8, 0x005389A2, 0x003D717D, 0x001E3930, 0x003E4838, 0x00707877, 0x007C888E, 0x00CED6D6, 0x00D3DCD9, 0x0093A4A7, 0x008D9AA2, 0x008C99A1, 0x00919EA6, 0x00909DA5, 0x0097A4AC,
        0x00A3B0B8, 0x00A0ADB5, 0x009EABB3, 0x0099ADB2, 0x0099ACAF, 0x009AACAD, 0x00879698, 0x007B8988, 0x00969FA2, 0x00C1C8CB, 0x00E3EAED, 0x00D1DCCC, 0x0085927C, 0x00748363, 0x00788964, 0x0061714F, 0xFFABB49F, 0xFFFCFFFA, 0xFFFDFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDF9FF, 0xFFFDFFF7, 0x00AAB496, 0x005F6C46, 0x007F8C66, 0x0077845E, 0x004D5C30, 0x00425320, 0x00596940, 0x0034441B, 0x005A6A41, 0x0056673C, 0x0068794E,
        0x006B7C50, 0x00435428, 0x0036471B, 0x0072835E, 0x00657357, 0x005D6757, 0x00B6BBBA, 0x00D9DBE3, 0x00CDCED8, 0x00C9CBD3, 0x00B8BBC0, 0x00C3C5CD, 0x00D2D5DA, 0x00CFD3D4, 0x008B8F8A, 0x00797F74, 0x00666B5C, 0x00585E4B, 0x00636A55, 0x008C8992,
        0x00848583, 0x0054584C, 0x0055594D, 0x0083857F, 0x00A1A39D, 0x00707665, 0x004B5439, 0x00798960, 0x0075855C, 0x0076865D, 0x007A8A61, 0x00708057, 0x0074845B, 0x0078885F, 0x0054643B, 0x00506338, 0x0046582F, 0x0042542B, 0x0052643B, 0x003A4C25,
        0x00576942, 0x0063744F, 0x007E8F6A, 0x0074865F, 0x0074865F, 0x00758760, 0x00758760, 0x00758760, 0x00758760, 0x0074865F, 0x0074865F, 0x00758664, 0x00768962, 0x0052653E, 0x0027361A, 0x00556050, 0x00525B51, 0x0024301E, 0x00223115, 0x00313D21,
        0x002D3A20, 0x002B3721, 0x00333E2A, 0x003C4834, 0x0037452D, 0x00213316, 0x00556748, 0x006E8258, 0x006F805F, 0x00465240, 0x005D665C, 0x007A827B, 0x004F5A4A, 0x0057664A, 0x00455732, 0x00586B4A, 0x00415433, 0x00546543, 0x00627151, 0x00495539,
        0x00606752, 0x00616657, 0x00303127, 0x001A1810, 0x00191D17, 0x001C2522, 0x002E3C3A, 0x000E1C1A, 0x00101916, 0x00191D18, 0x0023201C, 0x00171D12, 0x001C2118, 0x001F231D, 0x00242920, 0x002A3025, 0x00262C21, 0x001E231A, 0x001C201A, 0x00313F23,
        0x002C3921, 0x0016200F, 0x00191F14, 0x00181C17, 0x001B1B1B, 0x00131214, 0x00111012, 0x001F240F, 0x002E331E, 0x00373E29, 0x00333926, 0x00272B1F, 0x001C201A, 0x00181A1B, 0x0016161C, 0x00161D16, 0x00181915, 0x00191919, 0x00182222, 0x003D5250,
        0x002F4441, 0x000F1913, 0x001D1914, 0x001B2215, 0x00596150, 0x005F6952, 0x004C5B3B, 0x0070815C, 0x00687A53, 0x006B7E57, 0x00687B54, 0x006A7D56, 0x006D8059, 0x00748760, 0x006C7E59, 0x006C7E59, 0x00758762, 0x006F805E, 0x006B7C5A, 0x00718257,
        0x0077875E, 0x007C8B65, 0x007A8864, 0x00788460, 0x007D8965, 0x007C8A61, 0x007A885E, 0x00809067, 0x00808D67, 0x00747F5E, 0x007F8768, 0x00778262, 0x007A8866, 0x0072845D, 0x007B9167, 0x00758665, 0x007A8A6B, 0x0078876B, 0x0077866A, 0x00748564,
        0x00738560, 0x00788C61, 0x00768C5C, 0x00748169, 0x007A876D, 0x00718065, 0x00768667, 0x00768766, 0x007C906D, 0x007B906A, 0x00748963, 0x00707E5A, 0x0072835E, 0x00596A45, 0x00556641, 0x004B5D38, 0x003E502B, 0x004E633D, 0x004A5F39, 0x00374A29,
        0x003F4D29, 0x00404E2A, 0x002E452F, 0x00436969, 0x00568A9B, 0x00538BA4, 0x004E849D, 0x003B6F7B, 0x001B362D, 0x003D4737, 0x00707877, 0x007C888E, 0x00CED6D6, 0x00D3DCD9, 0x0092A3A6, 0x00849199, 0x0089969E, 0x0093A0A8, 0x0094A1A9, 0x0098A5AD,
        0x009EABB3, 0x0099A6AE, 0x0096A3AB, 0x0099ADAE, 0x009BAFB0, 0x00A4B5B8, 0x009CA9AB, 0x00929EA0, 0x009DA7A7, 0x00A8B1AE, 0x00B0B7B4, 0x009BA892, 0x00758469, 0x00768565, 0x007B8C67, 0x00606F50, 0xFFA7B09C, 0xFFFBFFF9, 0xFFFDFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFBFF, 0xFBF9FCF3, 0x00A3AD8F, 0x005D6A44, 0x00828F69, 0x007A8761, 0x004D5C30, 0x0041521F, 0x006B7950, 0x00546239, 0x00637148, 0x00536137, 0x00616F45,
        0x00707F53, 0x004B5A2E, 0x003B4A1E, 0x006C7A58, 0x00576347, 0x00555C4F, 0x00AEB1AF, 0x00DEDEE4, 0x00D5D4DD, 0x00D5D5DB, 0x00D5D6DA, 0x00D1D4D9, 0x00E3E7E8, 0x00C0C5C3, 0x008D918B, 0x00A8AFA2, 0x00777D6A, 0x004B533C, 0x00838C72, 0x0099989A,
        0x008C8F86, 0x00616756, 0x00696E5F, 0x00A8AAA4, 0x00BEC0BA, 0x0084897A, 0x00545C45, 0x006A7A51, 0x0077875E, 0x007B8B62, 0x0075855C, 0x006C7C53, 0x0076865D, 0x007F8F66, 0x0065754C, 0x0054673C, 0x00405229, 0x003F5128, 0x0053653C, 0x00394B24,
        0x00556740, 0x005F704B, 0x00798A65, 0x0073855E, 0x0074865F, 0x0074865F, 0x00758760, 0x00758760, 0x00758760, 0x00758760, 0x00758760, 0x00718260, 0x007A8D66, 0x004F623B, 0x00233216, 0x00586353, 0x004A5349, 0x00273321, 0x00243317, 0x00273315,
        0x002D3B1F, 0x002A371F, 0x002E3925, 0x00394531, 0x003D4A34, 0x0026371C, 0x005B6D50, 0x0071855B, 0x00708061, 0x00606B5B, 0x00ACB3B0, 0x00D0D7D4, 0x007C867A, 0x00505F44, 0x0052643F, 0x004A5D3C, 0x00586B4A, 0x00566745, 0x00455434, 0x00475337,
        0x006D745F, 0x00656A5B, 0x00313228, 0x00181413, 0x001B1E1C, 0x00222B28, 0x00364541, 0x00162521, 0x00151E1B, 0x00181B19, 0x001D1A16, 0x001C2413, 0x001A2015, 0x00161A15, 0x001C201A, 0x00262D1E, 0x00232B1A, 0x00192013, 0x00151913, 0x001B2514,
        0x00232D1C, 0x001C2518, 0x001D2219, 0x00161A15, 0x001A1A1A, 0x00181719, 0x00201F23, 0x001E2014, 0x00252A1B, 0x002B3120, 0x002B3120, 0x00242B1E, 0x001E231A, 0x00191E1C, 0x00171B1C, 0x00171D18, 0x00181917, 0x0019181A, 0x00172023, 0x003C5051,
        0x002E4341, 0x000E1714, 0x001D1815, 0x001A2114, 0x00545C4B, 0x00656F58, 0x004C5B3B, 0x00687954, 0x00677952, 0x006B7E57, 0x006C7F58, 0x00677A53, 0x006D8059, 0x00768863, 0x006D7F5A, 0x0070825D, 0x00728361, 0x005D6E4C, 0x00576846, 0x004F603F,
        0x00576847, 0x00687957, 0x0073855E, 0x006F7E58, 0x0073835A, 0x00748259, 0x007A885F, 0x006E7F5D, 0x0070815F, 0x006C7C5D, 0x006F8162, 0x006D7F60, 0x00728465, 0x006D8360, 0x00718764, 0x00708565, 0x00708467, 0x006C8063, 0x006C7F64, 0x006D8164,
        0x006E8363, 0x006F8562, 0x006D845E, 0x0072826A, 0x0074856A, 0x006A7D62, 0x006C8063, 0x006D8464, 0x00708767, 0x00718967, 0x0067805E, 0x00788662, 0x005D6E49, 0x0053643F, 0x0050613C, 0x003D4F2A, 0x0041532E, 0x004F643E, 0x00435832, 0x00445736,
        0x0043512D, 0x0044522E, 0x00314832, 0x003D6363, 0x00568A9B, 0x00538BA4, 0x004F859E, 0x00407480, 0x001D382F, 0x003E4838, 0x00717978, 0x007B878D, 0x00CED6D6, 0x00D4DDDA, 0x0091A2A5, 0x00798B92, 0x00788A91, 0x0081939A, 0x00889AA1, 0x0092A4AB,
        0x0098AAB1, 0x008B9DA4, 0x0083959C, 0x00899C9F, 0x008D9DA3, 0x009FADB3, 0x00A7B3B7, 0x00A8B4B4, 0x00A6B0AA, 0x008C968A, 0x00727D6D, 0x0078876C, 0x006F7F60, 0x007B8B69, 0x007F8F6D, 0x00616F53, 0xFFA6AE9D, 0xFFFDFFFB, 0xFFFDFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFBFF, 0xFFFBFEF5, 0x00A2AC8E, 0x005B6842, 0x0083906A, 0x00808D67, 0x00505F33, 0x003F501D, 0x00647249, 0x006A784F, 0x00536138, 0x00324016, 0x00435127,
        0x0059683C, 0x0038471B, 0x0038471B, 0x00697656, 0x005F6A50, 0x007F8679, 0x00A6A9A7, 0x00ECEDF1, 0x00DDDEE2, 0x00D4D6D6, 0x00DDE0DE, 0x00D3D8D7, 0x00E2E7E5, 0x00CED5CE, 0x006B7265, 0x006A735F, 0x00525B41, 0x00596345, 0x00636E4E, 0x00575D4A,
        0x005E684A, 0x00586440, 0x00505B3B, 0x005F6651, 0x00606655, 0x00687157, 0x00798463, 0x004E5E35, 0x00697950, 0x0074845B, 0x0077875E, 0x007D8D64, 0x007C8C63, 0x0073835A, 0x0068784F, 0x00516439, 0x003D4F26, 0x0041532A, 0x0053653C, 0x00354720,
        0x0054663F, 0x005D6E49, 0x00758661, 0x0073855E, 0x0073855E, 0x0074865F, 0x00758760, 0x00758760, 0x00758760, 0x00758760, 0x00758760, 0x00728361, 0x00788B64, 0x0052653E, 0x00202F13, 0x00556050, 0x00464F45, 0x0025311F, 0x00243317, 0x00263313,
        0x002D3B1F, 0x0027341C, 0x002F3A26, 0x00394531, 0x00394630, 0x0022321A, 0x005A6B50, 0x0075895F, 0x006B7A5E, 0x00465044, 0x0074797A, 0x00888D8E, 0x00596258, 0x0044523A, 0x0071835E, 0x00465938, 0x00516443, 0x00536442, 0x00384727, 0x004A563A,
        0x00818873, 0x006C7162, 0x00303127, 0x001C1719, 0x001C1E1F, 0x001F2726, 0x002F3E3A, 0x0011211A, 0x00141E18, 0x001C1F1D, 0x00221E1D, 0x001A2015, 0x001C1F1D, 0x00191A1E, 0x00191B1C, 0x00191E15, 0x00141B0E, 0x00121610, 0x0018191D, 0x00171E17,
        0x00242C22, 0x00232B21, 0x0020251C, 0x00141812, 0x001A1B19, 0x001B1B1B, 0x00272628, 0x00191A16, 0x001C1E18, 0x0021251A, 0x0022291C, 0x001F2619, 0x00192116, 0x00171E17, 0x00171D18, 0x00181E19, 0x00191A18, 0x001A191B, 0x00172023, 0x003B4F50,
        0x002D4240, 0x000F1815, 0x001F1A17, 0x00171E11, 0x004E5645, 0x006E7861, 0x00516040, 0x00647550, 0x00687A53, 0x006B7E57, 0x006F825B, 0x006B7E57, 0x006D7F5A, 0x0072845F, 0x006C7E59, 0x00748563, 0x006C7D5B, 0x004D5E3D, 0x004E5F3E, 0x00566954,
        0x004A5F46, 0x00576C4B, 0x0073895F, 0x00778B60, 0x00798B62, 0x006F7E58, 0x00748260, 0x00778563, 0x00758563, 0x00758762, 0x00748963, 0x0070855F, 0x00738862, 0x00738764, 0x00748563, 0x0070825D, 0x006F815C, 0x00748661, 0x00758762, 0x00738462,
        0x00738463, 0x00718162, 0x006F7F60, 0x0073845F, 0x00778863, 0x00768762, 0x00738560, 0x00788A65, 0x00738764, 0x00788C69, 0x006E825F, 0x0065734F, 0x0052633E, 0x00596A45, 0x0032431E, 0x00465833, 0x005B6D48, 0x002A3F19, 0x00536842, 0x00495C3B,
        0x004F5D39, 0x0044522E, 0x00344B35, 0x00416767, 0x00548899, 0x00558DA6, 0x005389A2, 0x00427682, 0x001C372E, 0x003B4535, 0x006D7574, 0x0078848A, 0x00CDD5D5, 0x00D6DFDC, 0x0092A3A6, 0x0074878E, 0x006B7E85, 0x0070838A, 0x007D9097, 0x008EA1A8,
        0x0095A8AF, 0x008699A0, 0x007D9097, 0x007E9097, 0x007E8D96, 0x008F9BA5, 0x009EA9B1, 0x00ABB5B5, 0x00A9B4AA, 0x008A9682, 0x006B785E, 0x00778867, 0x00748564, 0x007C8C6A, 0x007D8C6C, 0x00606D53, 0xFFA5AD9C, 0xFFFDFFFB, 0xFFFDFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCF8FF, 0xFFFCFFF6, 0x00A7B193, 0x005D6A44, 0x00828F69, 0x007E8B65, 0x0049582C, 0x0031420F, 0x00344219, 0x0039471E, 0x00344219, 0x00344218, 0x002D3B11,
        0x00354418, 0x0029380C, 0x0039481C, 0x00677454, 0x00838D76, 0x00C2C9BC, 0x00969A95, 0x00E6E8E8, 0x00E1E3E3, 0x00D0D4CF, 0x00C9CEC5, 0x00D0D6D1, 0x00D9E1D7, 0x00CFD8CB, 0x00656D5C, 0x007F8972, 0x00646E50, 0x007B8665, 0x0077835F, 0x0076845B,
        0x00748454, 0x00768952, 0x006D7D4D, 0x007A8662, 0x00717C5C, 0x0066724E, 0x00505F33, 0x0056663D, 0x00516138, 0x00697950, 0x007B8B62, 0x0075855C, 0x0078885F, 0x0078885F, 0x00627249, 0x004F6237, 0x00405229, 0x00485A31, 0x00576940, 0x00374922,
        0x00596B44, 0x0063744F, 0x00798A65, 0x0073855E, 0x0073855E, 0x0074865F, 0x00758760, 0x00758760, 0x00758760, 0x00758760, 0x00758760, 0x00768765, 0x0073865F, 0x00556841, 0x00213014, 0x00545F4F, 0x00434C42, 0x001E2A18, 0x00223115, 0x00273414,
        0x00273617, 0x001E2B13, 0x00313D27, 0x003D4935, 0x00394630, 0x00213119, 0x005B6C51, 0x0073875D, 0x0069785C, 0x004E574D, 0x007D8285, 0x00808489, 0x00525A53, 0x00324028, 0x006A7C57, 0x00596C4B, 0x00324524, 0x004B5C3A, 0x00556444, 0x00667256,
        0x008C937E, 0x00676C5D, 0x00313228, 0x00191219, 0x001A1B1F, 0x001E2625, 0x002F3F38, 0x0011211A, 0x00121C16, 0x00171A18, 0x001A1615, 0x001A1E19, 0x0018181E, 0x00141120, 0x0016151F, 0x001A1D1B, 0x00171B15, 0x00151718, 0x001B1827, 0x00141918,
        0x001B201E, 0x00151B16, 0x00171B15, 0x00161A14, 0x0022231F, 0x001C1D1B, 0x00202020, 0x001C1B1D, 0x001D1D1D, 0x001F231D, 0x0023291E, 0x0021281B, 0x00192215, 0x00182015, 0x001C241A, 0x00191F1A, 0x001A1B19, 0x001A191B, 0x00172023, 0x00394D4E,
        0x002C413F, 0x000F1815, 0x00201B18, 0x001D2417, 0x00525A49, 0x007D8770, 0x005E6D4D, 0x00697A55, 0x006C7E57, 0x006A7D56, 0x006D8059, 0x0071835E, 0x006C7E59, 0x006C7E59, 0x006A7C57, 0x00768765, 0x00697A58, 0x00475837, 0x00526342, 0x003C5043,
        0x00445943, 0x005A7250, 0x006C8559, 0x00687F52, 0x0075895E, 0x0070805E, 0x00748364, 0x007D8461, 0x0078835D, 0x0076875C, 0x0073875A, 0x006F8356, 0x00708358, 0x00788761, 0x00788460, 0x0078865C, 0x00728155, 0x0079885C, 0x0077865A, 0x00738157,
        0x00788460, 0x00788363, 0x007B8469, 0x007A8656, 0x00798658, 0x007A8759, 0x00727F53, 0x007A885E, 0x00728057, 0x0074845B, 0x0065754C, 0x00515F3B, 0x005D6E49, 0x002C3D18, 0x004B5C37, 0x00596B46, 0x00334520, 0x004A5F39, 0x004A5F39, 0x003A4D2C,
        0x0052603C, 0x0042502C, 0x003E553F, 0x00507676, 0x00578B9C, 0x00568EA7, 0x0050869F, 0x00407480, 0x00163128, 0x00343E2E, 0x00676F6E, 0x00737F85, 0x00CCD4D4, 0x00D7E0DD, 0x0094A5A8, 0x0080989E, 0x007A9298, 0x00829AA0, 0x008BA3A9, 0x0095ADB3,
        0x0098B0B6, 0x0090A8AE, 0x0091A9AF, 0x0098A6B2, 0x008D9BA7, 0x00909CA8, 0x00939EA6, 0x0099A3A3, 0x009BA89A, 0x0087967B, 0x00728260, 0x00768765, 0x00728361, 0x00758563, 0x00798868, 0x005F6C52, 0xFFA3AB9A, 0xFFFDFFFB, 0xFFF9FBFB, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xFFFEFFF7, 0x009EA88A, 0x00606E45, 0x007F8C66, 0x00788460, 0x00657349, 0x004C5C2C, 0x004B5B32, 0x004D5D34, 0x004B5B32, 0x004B5B32, 0x004C5C33,
        0x0047572E, 0x0044542B, 0x004C5C33, 0x006C7D51, 0x00627348, 0x00899773, 0x007F8B6D, 0x008A967A, 0x008B977B, 0x00717D5F, 0x00879474, 0x006A7658, 0x00748062, 0x00869171, 0x006B7655, 0x005C6642, 0x00616C46, 0x007F8A62, 0x007E875F, 0x007B8E63,
        0x0075885D, 0x0074875C, 0x0077875E, 0x0076855F, 0x007B8862, 0x0077835F, 0x00697551, 0x004B5B32, 0x005D6D44, 0x004F5F36, 0x006E7E55, 0x007C8C63, 0x007B8B62, 0x007A8A61, 0x0067774E, 0x0050623B, 0x0042542D, 0x00475932, 0x0050623B, 0x003B4D26,
        0x00566841, 0x0061734C, 0x00768861, 0x0073855C, 0x0073855C, 0x0074865D, 0x0074865D, 0x0074865D, 0x0074865D, 0x0073855C, 0x0073855C, 0x00748661, 0x00778867, 0x00505F43, 0x00222F19, 0x004E5A46, 0x00485442, 0x00283420, 0x0024311B, 0x0029321E,
        0x002A371D, 0x00243314, 0x002F3C24, 0x003D473A, 0x00343E32, 0x00304028, 0x003F5330, 0x004F5D3B, 0x004B563C, 0x00545B4C, 0x00A4A9A0, 0x00BDC2B9, 0x006A7162, 0x00404B31, 0x004C5A38, 0x004C5A42, 0x00435237, 0x003F4D31, 0x00485738, 0x005D6B4F,
        0x006D7760, 0x00555D4C, 0x00282F20, 0x00211710, 0x001A1E18, 0x0022332F, 0x00384847, 0x00181F22, 0x001D1A1C, 0x001C1A19, 0x00191D18, 0x00181D1B, 0x00151B16, 0x00181F18, 0x001C2118, 0x00161B12, 0x00151914, 0x001B1B1B, 0x001C1B1D, 0x001B1B1B,
        0x00171816, 0x001A1C16, 0x001A1D14, 0x00161910, 0x00191C13, 0x001E201A, 0x001C1D19, 0x001B1B21, 0x001B1D1D, 0x00282D24, 0x001F2617, 0x001F2716, 0x00192011, 0x00191E15, 0x00191C1A, 0x00161A1B, 0x001B1D17, 0x001C2015, 0x0017231D, 0x003B4F50,
        0x00304445, 0x00111C14, 0x001E1E0C, 0x001F2213, 0x005C624F, 0x006B745A, 0x00556242, 0x0070815C, 0x00697C55, 0x006B8059, 0x00637852, 0x006D7A5A, 0x0071815F, 0x00748560, 0x0072845D, 0x006A7D56, 0x00516240, 0x00445637, 0x00607156, 0x00526348,
        0x004E6041, 0x00627351, 0x0071845D, 0x0074865D, 0x0073855C, 0x0070825B, 0x00788662, 0x0071835E, 0x0071835E, 0x0071835E, 0x0071835E, 0x0071835E, 0x0071835E, 0x0071835E, 0x0071835E, 0x00748563, 0x00748563, 0x00748563, 0x00748563, 0x00738462,
        0x00728361, 0x00718260, 0x0070815F, 0x006F805B, 0x00758661, 0x00768762, 0x00748560, 0x00798A65, 0x00748560, 0x00667752, 0x00495A35, 0x005E6F4A, 0x002C3D18, 0x0041532E, 0x005A6C47, 0x0030451F, 0x00425731, 0x005D744E, 0x001D340E, 0x00465E40,
        0x003F502E, 0x00303F19, 0x002F452C, 0x004B706C, 0x0055899A, 0x004A859F, 0x00558FAC, 0x003E7185, 0x001E3936, 0x00495346, 0x00697472, 0x0098A6AC, 0x00D5DCDF, 0x00CFD8DB, 0x0097A7B3, 0x008F9CA4, 0x0094A1A9, 0x008F9CA4, 0x0097A4AC, 0x009CA9B1,
        0x009CA9B1, 0x00A4B1B9, 0x00A1AEB6, 0x00A6AEB5, 0x009FAAA8, 0x00929F97, 0x008C9A94, 0x00909C9C, 0x0098A5A3, 0x00798C7B, 0x006C8466, 0x00728263, 0x00778867, 0x0073845F, 0x00788964, 0x00606F4F, 0xFFA4AD98, 0xFFFDFFF8, 0xFFF7FAF8, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xFFFEFFF7, 0x009DA789, 0x00606E45, 0x007D8A64, 0x007C8864, 0x007A885E, 0x00768656, 0x007E8E65, 0x007B8B62, 0x0077875E, 0x0078885F, 0x007C8C63,
        0x007C8C63, 0x007A8A61, 0x007B8B62, 0x007F925F, 0x006B7D4E, 0x00708156, 0x006B7A54, 0x0075845E, 0x00718158, 0x006D7E53, 0x00839468, 0x00778561, 0x006A7854, 0x006A7854, 0x006C7953, 0x00707E55, 0x00717C54, 0x0079855B, 0x0078845A, 0x00708457,
        0x00728659, 0x00778A5F, 0x0078895E, 0x0074845B, 0x0079875E, 0x007E8C63, 0x007C8963, 0x0066764D, 0x004A5A31, 0x005A6A41, 0x0054643B, 0x00708057, 0x0078885F, 0x007B8B62, 0x00627249, 0x0050623B, 0x0041532C, 0x00455730, 0x004E6039, 0x003A4C25,
        0x00556740, 0x0060724B, 0x00768861, 0x0073855C, 0x0073855C, 0x0074865D, 0x0074865D, 0x0074865D, 0x0074865D, 0x0073855C, 0x0073855C, 0x00738560, 0x00778867, 0x00505F43, 0x00212E18, 0x004C5844, 0x0044503E, 0x0024301C, 0x00212E18, 0x00222B17,
        0x00222F15, 0x00243314, 0x002F3C24, 0x00404A3D, 0x00424C40, 0x0044543C, 0x005A6E4B, 0x00556543, 0x00556445, 0x0045523A, 0x00626D59, 0x0068735F, 0x003D4A32, 0x003F4E2F, 0x005A6A48, 0x00404F34, 0x004B5A3F, 0x00505E42, 0x004D5C3D, 0x00566448,
        0x0058624B, 0x004B5342, 0x00495041, 0x003D342B, 0x0022271E, 0x001A2C25, 0x002F403D, 0x00141C1B, 0x001A1818, 0x00191814, 0x001A1E18, 0x001D231E, 0x00171E17, 0x001B2319, 0x00242A1F, 0x001F241B, 0x001B1C18, 0x001D1D1D, 0x001F1E22, 0x001C1B1D,
        0x001B1B1B, 0x001E1F1D, 0x001D1E1A, 0x00181A14, 0x001B1D17, 0x0020211D, 0x001E1F1B, 0x0017181C, 0x00161917, 0x00252B20, 0x0022291A, 0x00222A19, 0x001A2114, 0x001A1E18, 0x00212323, 0x00131718, 0x00191B15, 0x001B1E15, 0x0014201A, 0x00344849,
        0x002B3F40, 0x0019241C, 0x00333321, 0x00323526, 0x00646A57, 0x00646D53, 0x004D5A3A, 0x006B7C57, 0x00657851, 0x00697E58, 0x00677C56, 0x006B7858, 0x006F7F5D, 0x00687954, 0x006F815A, 0x005F724B, 0x004A5B39, 0x00506243, 0x0044553A, 0x00506146,
        0x005D6F50, 0x00768765, 0x00788B64, 0x0072845B, 0x0074865D, 0x0071835C, 0x00707E5A, 0x00738560, 0x00738560, 0x00738560, 0x00738560, 0x00738560, 0x00738560, 0x00738560, 0x00738560, 0x00738462, 0x00748563, 0x00748563, 0x00748563, 0x00738462,
        0x00738462, 0x00728361, 0x00718260, 0x0073845F, 0x00758661, 0x0072835E, 0x0073845F, 0x0073845F, 0x006C7D58, 0x00445530, 0x005D6E49, 0x002B3C17, 0x004A5B36, 0x005E704B, 0x002E401B, 0x00475C36, 0x005D724C, 0x00223913, 0x0049603A, 0x00496143,
        0x00293A18, 0x0035441E, 0x003D533A, 0x00365B57, 0x00518596, 0x005E99B3, 0x00457F9C, 0x0043768A, 0x002E4945, 0x00626C5F, 0x0085908D, 0x00AAB8BE, 0x00E0E7EA, 0x00D2DBDE, 0x0093A3AF, 0x008A9AA1, 0x0090A0A7, 0x0093A3AA, 0x0099A9B0, 0x009BABB2,
        0x0099A9B0, 0x009CACB3, 0x009AAAB1, 0x0099A3AD, 0x0097A4A6, 0x00919F9B, 0x0092A09C, 0x0097A3A3, 0x009DA8A5, 0x007B8B79, 0x00697E5D, 0x00768569, 0x007B8C6B, 0x00758661, 0x00788964, 0x005F6E4E, 0xFF9FA893, 0xFFFDFFF8, 0xFFFDFFFE, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFAFF, 0xFBFBFFF4, 0x00A0AA8C, 0x00647249, 0x007C8963, 0x00798561, 0x0078865C, 0x00819161, 0x0074845B, 0x0077875E, 0x007B8B62, 0x00798960, 0x0075855C,
        0x0075855C, 0x0078885F, 0x007A8A61, 0x00778A57, 0x007C8F5E, 0x0077895A, 0x0077885C, 0x00809165, 0x007D8F60, 0x00839663, 0x007D925B, 0x00809166, 0x00809166, 0x007E8F64, 0x007E8C62, 0x007A885E, 0x007A885E, 0x007C885E, 0x007B875D, 0x00728659,
        0x0074885B, 0x00778B5E, 0x0076875B, 0x00718256, 0x00758359, 0x0079875D, 0x0079875D, 0x0073835A, 0x0068784F, 0x0047572E, 0x0057673E, 0x00526239, 0x006F7F56, 0x007A8A61, 0x0064744B, 0x0050623B, 0x0041532C, 0x0044562F, 0x004C5E37, 0x00384A23,
        0x0054663F, 0x0060724B, 0x00768861, 0x0073855C, 0x0073855C, 0x0074865D, 0x0074865D, 0x0074865D, 0x0074865D, 0x0073855C, 0x0073855C, 0x0072845F, 0x00768766, 0x004E5D41, 0x001F2C16, 0x004B5743, 0x00414D3B, 0x00212D19, 0x00202D17, 0x0027301C,
        0x00263319, 0x002C3B1C, 0x00323F27, 0x003D473A, 0x00353F33, 0x002B3B23, 0x00495D3A, 0x00546442, 0x00556543, 0x00394927, 0x00394828, 0x003C4B2B, 0x00324220, 0x00465634, 0x00586846, 0x00314024, 0x00445337, 0x004E5D3E, 0x00475637, 0x00616F53,
        0x007F8972, 0x00646C5B, 0x0040473A, 0x004A4235, 0x00252B20, 0x00172A21, 0x0031423E, 0x0019221F, 0x001E1D19, 0x0021211B, 0x00282D24, 0x002A3228, 0x001B2318, 0x001B2215, 0x0021281B, 0x001E2118, 0x00171814, 0x00181719, 0x001E1A20, 0x00181719,
        0x001A191B, 0x001C1B1D, 0x00191919, 0x00161616, 0x00191A18, 0x001E1F1D, 0x001C1D1B, 0x001A1C1C, 0x00181C17, 0x00242B1E, 0x00232B1A, 0x00202817, 0x00161C11, 0x00161A15, 0x00222425, 0x001A1E1F, 0x001A1B17, 0x00191C13, 0x0018241E, 0x003F5255,
        0x00374A4D, 0x00222D25, 0x00383828, 0x00353829, 0x00616754, 0x005B644A, 0x00485535, 0x006A7B56, 0x00647651, 0x00647953, 0x00677C56, 0x00697656, 0x0071815F, 0x0072835E, 0x004D5F38, 0x004E613A, 0x00455634, 0x00213314, 0x004F6045, 0x0057684D,
        0x00617354, 0x00758664, 0x0072855E, 0x006C7E55, 0x0072845B, 0x0074865F, 0x00778561, 0x00748661, 0x00748661, 0x00748661, 0x00748661, 0x00748661, 0x00748661, 0x00748661, 0x00748661, 0x00738560, 0x00738560, 0x00738560, 0x00748661, 0x00748661,
        0x00738560, 0x00738560, 0x0072845F, 0x00788964, 0x006E7F5A, 0x006E7F5A, 0x00798A65, 0x00667752, 0x003F502B, 0x0061724D, 0x00344520, 0x00485934, 0x00657651, 0x002B3D18, 0x004A5C37, 0x005A6F49, 0x002A3F19, 0x00455C36, 0x0049603A, 0x0021391B,
        0x004B5C3A, 0x00485731, 0x00263C23, 0x00385D59, 0x00588C9D, 0x00538EA8, 0x004E88A5, 0x00487C8D, 0x0035504C, 0x00636E5E, 0x00838E8B, 0x009DABB1, 0x00DAE1E4, 0x00D2DBDE, 0x0094A5AE, 0x007F9299, 0x0080939A, 0x00889BA2, 0x008FA2A9, 0x0095A8AF,
        0x0096A9B0, 0x008DA0A7, 0x00899CA3, 0x008F9EAE, 0x00899BA2, 0x007F9190, 0x00879593, 0x008F9B9B, 0x009BA59F, 0x007D8975, 0x006A7A58, 0x00718064, 0x00768766, 0x00728260, 0x00778863, 0x00627152, 0xFF9FA893, 0xFFFCFFF8, 0xFFFDFFFE, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xFFFEFFF7, 0x009FA98B, 0x00627047, 0x007B8862, 0x007E8A66, 0x0078865C, 0x00819161, 0x00798960, 0x00798960, 0x007C8C63, 0x007D8D64, 0x007C8C63,
        0x007E8E65, 0x007D8D64, 0x0076865D, 0x0073865B, 0x007C8E65, 0x0073845F, 0x00758661, 0x00758760, 0x006D7F56, 0x00728657, 0x00687D4C, 0x00687B50, 0x006E8156, 0x00738459, 0x0076875C, 0x00738157, 0x007C8A60, 0x007B895F, 0x00778359, 0x0074875C,
        0x00708358, 0x006F8356, 0x00728357, 0x0075865A, 0x007A895D, 0x00788859, 0x00748455, 0x0077875E, 0x007D8D64, 0x005C6C43, 0x004B5B32, 0x0058683F, 0x00506037, 0x0076865D, 0x0068784F, 0x0052643D, 0x0042542D, 0x00455730, 0x004C5E37, 0x00384A23,
        0x0054663F, 0x0061734C, 0x00788A63, 0x0073855C, 0x0073855C, 0x0074865D, 0x0074865D, 0x0074865D, 0x0074865D, 0x0073855C, 0x0073855C, 0x0070825D, 0x00758665, 0x004D5C40, 0x001E2B15, 0x004C5844, 0x00414D3B, 0x00202C18, 0x00222F19, 0x001D2612,
        0x001E2B11, 0x00273617, 0x002F3C24, 0x00424C3F, 0x00323C30, 0x001F2F17, 0x004F6340, 0x00758661, 0x0072845D, 0x0054663D, 0x004F6237, 0x0052653A, 0x004B5D34, 0x0052643D, 0x0051623D, 0x00516142, 0x00425332, 0x004D5C3C, 0x00495838, 0x0048563A,
        0x005E6752, 0x00565E4D, 0x0040473A, 0x00423A2D, 0x00262D20, 0x00213529, 0x00394B44, 0x001A241E, 0x001A1A14, 0x001C1D14, 0x00242A1F, 0x002A3025, 0x001D2417, 0x001A2112, 0x00212617, 0x001E2217, 0x001A1C16, 0x001B1919, 0x001A161B, 0x00171717,
        0x00171618, 0x0017161A, 0x00161519, 0x00161519, 0x001A191B, 0x001B1C1A, 0x00191A16, 0x001A1E18, 0x001C2316, 0x00262E1D, 0x00242D18, 0x00222B17, 0x001C2316, 0x00181B19, 0x00222327, 0x001D2024, 0x001B1C18, 0x001A1D14, 0x001B2623, 0x0044575A,
        0x00394C4F, 0x001E2822, 0x00303020, 0x00393B2F, 0x006B7160, 0x00636B54, 0x004E5B3B, 0x006E7E5C, 0x00697B56, 0x00677C56, 0x006A7E5B, 0x00788565, 0x00687856, 0x00586944, 0x004E6039, 0x0050633C, 0x00445533, 0x0037492A, 0x004C5D42, 0x00506146,
        0x004C5E3F, 0x00627351, 0x0073865F, 0x00778960, 0x0075875E, 0x0070825B, 0x00778561, 0x0072845F, 0x0072845F, 0x0072845F, 0x0072845F, 0x0072845F, 0x0072845F, 0x0072845F, 0x0072845F, 0x0072845F, 0x0072845F, 0x00738560, 0x00738560, 0x00748661,
        0x00738560, 0x00738560, 0x00738560, 0x0073845F, 0x0073845F, 0x00768762, 0x00647550, 0x004E5F3A, 0x005D6E49, 0x002D3E19, 0x00475833, 0x005A6B46, 0x002F401B, 0x004A5C37, 0x00546641, 0x002F441E, 0x00415630, 0x004F6640, 0x00283F19, 0x00486042,
        0x00425331, 0x00374620, 0x002E442B, 0x003F6460, 0x00528697, 0x004F8AA4, 0x00508AA7, 0x00477B8C, 0x002D4844, 0x004D5848, 0x006A7572, 0x00808F92, 0x00CBD3D3, 0x00D0DADA, 0x0095A6AF, 0x00748A90, 0x006D8389, 0x00768C92, 0x007E949A, 0x008DA3A9,
        0x0093A9AF, 0x0081979D, 0x00788E94, 0x00788C9E, 0x007E929D, 0x007E9293, 0x00899998, 0x008E9A9C, 0x00979E9B, 0x007F8A76, 0x006E7C5A, 0x00718064, 0x00768667, 0x00728260, 0x00758563, 0x00637253, 0xFFA2AB97, 0xFFF9FEF5, 0xFFFBFDFD, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xFFFFFFF8, 0x009EA88A, 0x00606E45, 0x007C8963, 0x00808C68, 0x00627046, 0x005B6B3B, 0x0057673E, 0x005A6A41, 0x005E6E45, 0x005D6D44, 0x005A6A41,
        0x00596940, 0x005C6C43, 0x005E6E45, 0x00738165, 0x00556248, 0x0045513B, 0x0056614D, 0x00505B47, 0x0035422A, 0x002C3A1E, 0x00354425, 0x00263614, 0x00293917, 0x00374823, 0x0045532F, 0x0042512B, 0x0055633A, 0x006E7C52, 0x007B895F, 0x0074865F,
        0x0074865F, 0x0074865D, 0x0077885D, 0x00798A5F, 0x007A895D, 0x00768657, 0x00758556, 0x0076865D, 0x0077875E, 0x007C8C63, 0x0057673E, 0x0058683F, 0x00526239, 0x005C6C43, 0x005E6E45, 0x0053653E, 0x0043552E, 0x00465831, 0x004E6039, 0x003A4C25,
        0x00566841, 0x0063754E, 0x00798B64, 0x0073855C, 0x0073855C, 0x0074865D, 0x0074865D, 0x0074865D, 0x0074865D, 0x0073855C, 0x0073855C, 0x0070825D, 0x00758665, 0x004C5B3F, 0x001F2C16, 0x004F5B47, 0x00434F3D, 0x00212D19, 0x0026331D, 0x0029321E,
        0x0028351B, 0x00283718, 0x00313E26, 0x00495346, 0x00323C30, 0x001F2F17, 0x005E724F, 0x0072845F, 0x00798C65, 0x006D8157, 0x00708459, 0x0075895E, 0x0065794F, 0x00566942, 0x00475934, 0x00435432, 0x004E5F3D, 0x006C7C5A, 0x00637351, 0x004C5B3C,
        0x00565F4A, 0x004E5546, 0x002E3429, 0x00292114, 0x001B2215, 0x001E3128, 0x00354740, 0x00151E1B, 0x00171612, 0x00171711, 0x00191F14, 0x00191E15, 0x00171D12, 0x001F2317, 0x00272C1D, 0x002A2F20, 0x002D2E24, 0x00272721, 0x001B1918, 0x00181917,
        0x00181818, 0x0018171B, 0x001A181E, 0x001C1A20, 0x001B1A1C, 0x00171814, 0x0015180F, 0x0018210D, 0x00212A15, 0x00293519, 0x00273317, 0x0029331C, 0x00272E1F, 0x00161A15, 0x00131516, 0x0015181C, 0x00191A18, 0x001A1C16, 0x0016211E, 0x00374A4F,
        0x002A3D42, 0x00121C16, 0x0029281A, 0x002F3125, 0x00676D5C, 0x005F6750, 0x00424E30, 0x00566644, 0x00556742, 0x00556946, 0x00586C49, 0x005F6C4C, 0x0051613F, 0x00465732, 0x00495B34, 0x005A6D46, 0x00576846, 0x00465839, 0x0054654A, 0x004B5C41,
        0x00415334, 0x00576846, 0x0070835C, 0x00798B62, 0x0074865D, 0x006E8059, 0x00778561, 0x0071835E, 0x0071835E, 0x0071835E, 0x0071835E, 0x0071835E, 0x0071835E, 0x0071835E, 0x0071835E, 0x0071845D, 0x0072855E, 0x0072855E, 0x0073865F, 0x0073865F,
        0x0073865F, 0x0072855E, 0x0072855E, 0x0072835E, 0x00798A65, 0x006A7B56, 0x004A5B36, 0x005E6F4A, 0x003B4C27, 0x003D4E29, 0x005A6B46, 0x0030411C, 0x00475833, 0x005C6E49, 0x0031431E, 0x00445933, 0x00586D47, 0x00253C16, 0x0049603A, 0x00465E40,
        0x002B3C1A, 0x0036451F, 0x00455B42, 0x00476C68, 0x0055899A, 0x005994AE, 0x004C86A3, 0x0045798A, 0x0026423C, 0x00404C3A, 0x0065716B, 0x0078878A, 0x00CCD4D4, 0x00D1DBDB, 0x0092A4AB, 0x00748A90, 0x006C8288, 0x0073898F, 0x007A9096, 0x00899FA5,
        0x0090A6AC, 0x007F959B, 0x00778D93, 0x0074889A, 0x007D919C, 0x007C9192, 0x00859496, 0x00818C90, 0x00899190, 0x007F8979, 0x00758163, 0x00748368, 0x0079896A, 0x00788767, 0x00758563, 0x005F6D51, 0xFFA4AD99, 0xFFFCFFFA, 0xFFFDFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xFFFFFFF8, 0x009EA88A, 0x00627047, 0x007E8B65, 0x007C8864, 0x0048562C, 0x00334313, 0x0035451C, 0x0034441B, 0x00304017, 0x00304017, 0x0035451C,
        0x00324219, 0x00304017, 0x0036461D, 0x00727E68, 0x004B5544, 0x004A5346, 0x00828982, 0x009FA5A0, 0x0099A099, 0x0090988E, 0x009BA398, 0x009AA595, 0x009BA795, 0x008C9783, 0x0058654B, 0x00414E2E, 0x004D5B37, 0x00718158, 0x007D8B61, 0x00738262,
        0x00788767, 0x007A8A68, 0x007A8864, 0x0075845E, 0x00728057, 0x00738157, 0x007B8A5E, 0x00708057, 0x0076865D, 0x0076865D, 0x0077875E, 0x005C6C43, 0x00596940, 0x0056663D, 0x00415128, 0x0053653E, 0x0044562F, 0x00475932, 0x004F613A, 0x003B4D26,
        0x00576942, 0x0063754E, 0x00798B64, 0x0073855C, 0x0073855C, 0x0074865D, 0x0074865D, 0x0074865D, 0x0074865D, 0x0073855C, 0x0073855C, 0x0071835E, 0x00758665, 0x004A593D, 0x001F2C16, 0x00535F4B, 0x00465240, 0x00222E1A, 0x0028351F, 0x002A331F,
        0x0029361C, 0x00273617, 0x00323F27, 0x00475144, 0x00303A2E, 0x001F2F17, 0x00566A47, 0x0070815F, 0x00687957, 0x00485937, 0x00405130, 0x004B5C3B, 0x00495A38, 0x00475836, 0x00475836, 0x004F613C, 0x006B7E57, 0x006F805B, 0x00536341, 0x00556445,
        0x0077806B, 0x00676E5F, 0x002B3027, 0x001D140B, 0x0012170E, 0x00172922, 0x002E3F3C, 0x00141C1B, 0x001E1C1C, 0x001D1C18, 0x00181C16, 0x00131614, 0x00171814, 0x001D2116, 0x0024291A, 0x002B2E1E, 0x00303323, 0x002E2E20, 0x00202012, 0x00181C11,
        0x00171814, 0x00171618, 0x001B191F, 0x001C1B1F, 0x00161715, 0x00161910, 0x001D2213, 0x00525F3F, 0x005B6947, 0x005C6B45, 0x00596842, 0x005A6747, 0x00515A45, 0x00292E25, 0x00151816, 0x00111418, 0x00161715, 0x00191B15, 0x0015201E, 0x00394C51,
        0x002E4146, 0x00101916, 0x001E1D0F, 0x0023241A, 0x00616657, 0x00606851, 0x00455133, 0x004D5D3B, 0x004C5D3B, 0x004D613E, 0x004C5F3E, 0x004D5A3A, 0x0051613F, 0x0053643F, 0x0040522B, 0x005E714A, 0x006A7B59, 0x00445637, 0x0065765B, 0x00495A3F,
        0x004B5D3E, 0x00617250, 0x006D8059, 0x006F8158, 0x00708259, 0x0070825B, 0x00778561, 0x00738560, 0x00738560, 0x00738560, 0x00738560, 0x00738560, 0x00738560, 0x00738560, 0x00738560, 0x0071845D, 0x0071845D, 0x0072855E, 0x0072855E, 0x0072855E,
        0x0072855E, 0x0071845D, 0x0071845D, 0x00748560, 0x006A7B56, 0x00445530, 0x0062734E, 0x003A4B26, 0x00394A25, 0x005F704B, 0x00394A25, 0x00475833, 0x0060714C, 0x0030421D, 0x00445631, 0x00586D47, 0x00334822, 0x00415832, 0x004C633D, 0x0020381A,
        0x0040512F, 0x004A5933, 0x003E543B, 0x004B706C, 0x005A8E9F, 0x00518CA6, 0x00518BA8, 0x00477B8B, 0x0026423C, 0x003C4836, 0x006B7771, 0x007A898B, 0x00CFD7D6, 0x00D3DEDC, 0x008FA1A8, 0x007F9299, 0x007D9097, 0x0082959C, 0x0085989F, 0x008A9DA4,
        0x008DA0A7, 0x008699A0, 0x0084979E, 0x008598A7, 0x007D9097, 0x00667B79, 0x0069787A, 0x0069747C, 0x00788185, 0x00747F75, 0x0065755E, 0x006D7C61, 0x006D7C60, 0x00768565, 0x00788767, 0x005E6C50, 0xFFA3AB9A, 0xFFFCFFFA, 0xFFFDFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xFFFFFFF8, 0x009CA688, 0x00627047, 0x007E8B65, 0x007F8B67, 0x004D5B31, 0x00425222, 0x0063734A, 0x0054643B, 0x00415128, 0x004A5A31, 0x00627249,
        0x005B6B42, 0x00425229, 0x003A4A21, 0x0067765A, 0x0048553F, 0x004F5A4A, 0x00A4ACA5, 0x00D9E0DD, 0x00E4EAE9, 0x00E1E7E6, 0x00E2E8E7, 0x00E5E9EE, 0x00EAEFF2, 0x00BCC3BE, 0x00565F52, 0x0049553F, 0x00566244, 0x006A7953, 0x005A6A41, 0x00536046,
        0x00505D43, 0x0049573B, 0x004B5838, 0x00515F3D, 0x0054613B, 0x0055633A, 0x005C6A41, 0x0073835A, 0x0074845B, 0x0077875E, 0x00728259, 0x007D8D64, 0x004E5E35, 0x0064744B, 0x002E3E15, 0x0052643D, 0x0043552E, 0x00475932, 0x0050623B, 0x003B4D26,
        0x00566841, 0x0062744D, 0x00788A63, 0x0073855C, 0x0073855C, 0x0074865D, 0x0074865D, 0x0074865D, 0x0074865D, 0x0073855C, 0x0073855C, 0x00738560, 0x00768766, 0x0049583C, 0x001E2B15, 0x0054604C, 0x00475341, 0x00212D19, 0x0027341E, 0x0028311D,
        0x0027341A, 0x00283718, 0x0035422A, 0x003F493C, 0x00303A2E, 0x00293921, 0x00455936, 0x004C603D, 0x00455738, 0x0033432B, 0x003D4C37, 0x004C5B46, 0x003A4A32, 0x00394B2C, 0x00526643, 0x006E815A, 0x006E8258, 0x0062744D, 0x005D6E49, 0x005C6B4C,
        0x00646D58, 0x00596053, 0x00282D24, 0x00271C18, 0x00181C17, 0x00182926, 0x002D3C3E, 0x00141A1F, 0x001E1A1F, 0x001B1919, 0x00141715, 0x001C1A20, 0x00171618, 0x001A1C16, 0x00212415, 0x00262B16, 0x002E3219, 0x002E3219, 0x00262A11, 0x001E2413,
        0x001B1F14, 0x00181917, 0x0019181C, 0x00181719, 0x00161713, 0x00272C1D, 0x00444B36, 0x0074855A, 0x00738458, 0x006A7C4D, 0x006C7E4F, 0x00707F59, 0x00636F53, 0x002E3526, 0x00131711, 0x0013161B, 0x00191A18, 0x001A1B17, 0x0016211F, 0x003F5257,
        0x00384A51, 0x00141D1A, 0x00181709, 0x001A1B11, 0x0054594A, 0x005A614C, 0x00485438, 0x00475636, 0x00435432, 0x00455837, 0x00445736, 0x004C5939, 0x00475735, 0x00384924, 0x00455730, 0x004B5E37, 0x00495A38, 0x004C5E3F, 0x00334429, 0x003E4F34,
        0x004E6041, 0x006E7F5D, 0x00758861, 0x0071835A, 0x00708259, 0x006D7F58, 0x006F7D59, 0x0072845F, 0x0072845F, 0x0072845F, 0x0072845F, 0x0072845F, 0x0072845F, 0x0072845F, 0x0072845F, 0x0071855B, 0x0071855B, 0x0071855B, 0x0071855B, 0x0071855B,
        0x0070845A, 0x006F8359, 0x006F8359, 0x006E7F5A, 0x003F502B, 0x0061724D, 0x00374823, 0x0041522D, 0x005D6E49, 0x00364722, 0x0041522D, 0x00657651, 0x00263712, 0x004C5E39, 0x00536540, 0x00283D17, 0x00485D37, 0x00516842, 0x00243B15, 0x00496143,
        0x00475836, 0x00495832, 0x0040563D, 0x00426763, 0x004D8192, 0x004E89A3, 0x004F89A6, 0x004A7E8E, 0x00254239, 0x00374331, 0x006D7A72, 0x00758486, 0x00CBD3D2, 0x00D2DDDB, 0x0091A3AA, 0x0088989F, 0x008B9BA2, 0x008B9BA2, 0x008E9EA5, 0x008E9EA5,
        0x008D9DA4, 0x0090A0A7, 0x008F9FA6, 0x008A99A2, 0x00809293, 0x00687B78, 0x006A797B, 0x0069757F, 0x007A8690, 0x00818F8D, 0x0076897A, 0x0085937B, 0x006D7C60, 0x00707F5F, 0x00798868, 0x005F6D51, 0xFFA3AB9A, 0xFFFBFFF9, 0xFFFCFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xFFFCFFF5, 0x009DA789, 0x0067754C, 0x007F8C66, 0x007A8662, 0x0047552B, 0x00455525, 0x005E6E45, 0x00617148, 0x005B6B42, 0x005F6F46, 0x0067774E,
        0x004E5E35, 0x0035451C, 0x003B4B22, 0x006C7E59, 0x004A5A3B, 0x00505D47, 0x00BAC4B7, 0x00E8EFEA, 0x00D4DAD9, 0x00C8CDCE, 0x00C6CBCC, 0x00D0CFE3, 0x00E8E8F8, 0x00C4C7CF, 0x00585E59, 0x004E5848, 0x003F4B2F, 0x0045532F, 0x003C4C23, 0x00636F59,
        0x004E5A44, 0x0037442A, 0x003A462A, 0x004E5B3B, 0x00576241, 0x004E5A36, 0x004A5731, 0x007B8B62, 0x0077875E, 0x0076865D, 0x00718158, 0x00798960, 0x0068784F, 0x00526239, 0x003C4C23, 0x0050623B, 0x0042542D, 0x00465831, 0x004F613A, 0x003B4D26,
        0x00566841, 0x0061734C, 0x00768861, 0x0073855C, 0x0073855C, 0x0074865D, 0x0074865D, 0x0074865D, 0x0074865D, 0x0073855C, 0x0073855C, 0x00758762, 0x00778867, 0x0048573B, 0x001D2A14, 0x0055614D, 0x00475341, 0x001F2B17, 0x0025321C, 0x002C3521,
        0x0028351B, 0x002C3B1C, 0x0039462E, 0x003D473A, 0x003E483C, 0x00485840, 0x00526643, 0x004A5E3B, 0x00506146, 0x005D6A5A, 0x007E897F, 0x007C877D, 0x00435040, 0x003A4B30, 0x006D815E, 0x00667A50, 0x0062764C, 0x0062744B, 0x00697A55, 0x005B6A4B,
        0x00636C57, 0x00676E61, 0x00292E25, 0x00231715, 0x00141616, 0x00152426, 0x002B3A3D, 0x0012181F, 0x001B171D, 0x001B171C, 0x00181A1B, 0x001D1925, 0x0016141A, 0x001D1C18, 0x00292C1D, 0x002E341D, 0x0033381B, 0x0033391A, 0x00303617, 0x002B321D,
        0x00262A1E, 0x001E1F1B, 0x001A191B, 0x00181818, 0x001D1F19, 0x00424837, 0x00727B61, 0x00607342, 0x00586B38, 0x004B5F2A, 0x00576A37, 0x0066774B, 0x005F6C4C, 0x002C3423, 0x00151A11, 0x0014171C, 0x001E1E1E, 0x001D1E1A, 0x00111C1A, 0x0034464D,
        0x002F4148, 0x00141D1A, 0x001F1E10, 0x002F3026, 0x005D6253, 0x005F6651, 0x004E5A3E, 0x00475636, 0x00435432, 0x00495C3B, 0x004B5E3D, 0x004D5A3A, 0x0050603E, 0x0051623D, 0x0053653E, 0x00586B44, 0x00475836, 0x002F4122, 0x00394A2F, 0x003E4F34,
        0x004C5E3F, 0x006A7B59, 0x0072855E, 0x006E8057, 0x006F8158, 0x006F815A, 0x0075835F, 0x0070825D, 0x0070825D, 0x0070825D, 0x0070825D, 0x0070825D, 0x0070825D, 0x0070825D, 0x0070825D, 0x0071855B, 0x0071855B, 0x0071855B, 0x0071855B, 0x0070845A,
        0x006F8359, 0x006E8258, 0x006D8157, 0x004A5B36, 0x00667752, 0x00384924, 0x00455631, 0x005F704B, 0x003F502B, 0x0041522D, 0x0062734E, 0x002F401B, 0x00546540, 0x00556742, 0x002D3F1A, 0x004B603A, 0x00526741, 0x00213812, 0x004D643E, 0x003F5739,
        0x00536442, 0x005B6A44, 0x00445A41, 0x003D625E, 0x00528697, 0x005691AB, 0x004F89A6, 0x00477B8B, 0x00234037, 0x0036422E, 0x00707D75, 0x00748385, 0x00C9D1D0, 0x00D1DCDA, 0x0093A5AC, 0x0089969E, 0x008D9AA2, 0x0087949C, 0x008D9AA2, 0x00909DA5,
        0x008F9CA4, 0x0095A2AA, 0x00919EA6, 0x008B979B, 0x0093A19F, 0x00859692, 0x00808F91, 0x00727E8A, 0x008794A2, 0x00AEBFC2, 0x00C0D5CC, 0x00BCCAB2, 0x00849377, 0x00707F60, 0x00768565, 0x005E6B51, 0xFFA3AB9A, 0xFFFCFFFB, 0xFFFDFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xFFFFFFF8, 0x009DA789, 0x00637148, 0x00808D67, 0x007E8A66, 0x0047552B, 0x00465626, 0x005E6F44, 0x003E4F24, 0x005A6B40, 0x0054653A, 0x0055663B,
        0x00627348, 0x003D4E23, 0x003C4D22, 0x0072845B, 0x00566546, 0x00707A6A, 0x00BBC0BF, 0x00DDE0E5, 0x00E3E5ED, 0x00CED2D3, 0x00C0C5C4, 0x00D8D8DE, 0x00D5D6DA, 0x00D0D2D2, 0x00767A75, 0x006B7166, 0x006C7463, 0x00535C47, 0x00646E57, 0x008D908E,
        0x00737674, 0x00525553, 0x0050544F, 0x00737772, 0x00B5B9B3, 0x00B1B5AF, 0x00494D47, 0x005E6B45, 0x007B8862, 0x007A8761, 0x0072815B, 0x0075845E, 0x0063754E, 0x0052643D, 0x00354720, 0x0050633C, 0x00465932, 0x0043562F, 0x004E613A, 0x003A4D26,
        0x00546740, 0x00667952, 0x0073865F, 0x0072845B, 0x0073855C, 0x0075875E, 0x0076885F, 0x0075875E, 0x0074865D, 0x0074865D, 0x0076885F, 0x0074825E, 0x007E8F6A, 0x004C5B3B, 0x00232D1C, 0x00565D58, 0x004B524F, 0x00192715, 0x00233615, 0x00212E18,
        0x002D3C20, 0x002C3C1D, 0x003D4935, 0x00424B41, 0x003D463C, 0x0019280D, 0x0044582D, 0x004F6339, 0x004C5B3F, 0x00545D53, 0x00B3B8BB, 0x00CBCFD4, 0x00757D76, 0x0048563E, 0x006C7D5B, 0x006A7D56, 0x00637550, 0x006A7A58, 0x005D6B4F, 0x00404D35,
        0x007F8874, 0x006A7162, 0x002C3326, 0x001D1417, 0x00171A1E, 0x00243133, 0x003B4749, 0x00181C1D, 0x001C1915, 0x001B1E15, 0x001B2518, 0x00181C17, 0x00181C17, 0x001B1F1A, 0x002B2D27, 0x002A2E22, 0x00262B16, 0x00323A1C, 0x00333C1B, 0x0030371C,
        0x003A4029, 0x00282B1C, 0x001B1F14, 0x00171B0F, 0x0017210A, 0x0054623E, 0x006B7C50, 0x00525B41, 0x002C351A, 0x001C2707, 0x00242F0E, 0x00303A1C, 0x00333B24, 0x0023271B, 0x00171A11, 0x00151D12, 0x00171913, 0x001A1B19, 0x00192323, 0x00465A5B,
        0x003A4E4F, 0x000F1717, 0x00201A1B, 0x00262822, 0x0051574C, 0x005C6453, 0x004C5941, 0x00455438, 0x00455738, 0x00495D40, 0x00495D40, 0x0047563A, 0x004A593D, 0x004F5E43, 0x00516045, 0x004E5C44, 0x004C5A42, 0x0054624A, 0x005E6B55, 0x005A6954,
        0x0053634C, 0x005B6A4E, 0x006E7F5D, 0x007A8C65, 0x00778962, 0x00748560, 0x007A8864, 0x0075845E, 0x0076855F, 0x00788761, 0x00798862, 0x00788761, 0x00778660, 0x00778660, 0x00778660, 0x00798862, 0x007B8A64, 0x007A8963, 0x00778660, 0x00788761,
        0x0076855F, 0x00687751, 0x0056653F, 0x0063754E, 0x00384A23, 0x0041532C, 0x0060724B, 0x003E5029, 0x0041532C, 0x0063754E, 0x0031431C, 0x004D6137, 0x00596D43, 0x00293C15, 0x0051643D, 0x00576944, 0x00273914, 0x004C5D3B, 0x00516240, 0x0053643F,
        0x00667859, 0x00627352, 0x00475636, 0x00446459, 0x004C859A, 0x004C8CAB, 0x00548BA0, 0x004B798A, 0x001B4038, 0x00334633, 0x00767C77, 0x007A8184, 0x00C5D6D3, 0x00CFE0DD, 0x0097A0A9, 0x007F9198, 0x007C8E95, 0x0083959C, 0x008D9FA6, 0x0092A4AB,
        0x0093A5AC, 0x008D9FA6, 0x0081939A, 0x007E939B, 0x007D8F98, 0x0083959C, 0x007D9095, 0x00798A8D, 0x00808F91, 0x00B4C1C3, 0x00D0DEDD, 0x00A5B596, 0x007E8F6E, 0x00788866, 0x007C8C6A, 0x005D6B4F, 0xFF9FA796, 0xFFFAFEF8, 0xFFFDFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFCFF, 0xFFFCFFF5, 0x0098A284, 0x00606E45, 0x007E8B65, 0x007B8763, 0x0046542A, 0x00435323, 0x00606E44, 0x00505E34, 0x005D6B41, 0x004F5D33, 0x0057653B,
        0x00657349, 0x00414F25, 0x00414F25, 0x006D7F58, 0x004C5B3C, 0x00707A6A, 0x00C7CCCA, 0x00D8DBE0, 0x00D4D7DC, 0x00D6DADB, 0x00D6DBD9, 0x00D3D3D9, 0x00E7E8EC, 0x00C8CACA, 0x00727670, 0x00858C7D, 0x005F6853, 0x004E593F, 0x00727E62, 0x00A8AEA3,
        0x008E9489, 0x006E7469, 0x0072796C, 0x00899083, 0x00ACB3A4, 0x00B7BEAF, 0x008A9182, 0x004B5832, 0x0068754F, 0x00727F59, 0x00778660, 0x007D8C66, 0x005D6F48, 0x00556740, 0x00374922, 0x00546740, 0x003F522B, 0x004A5D36, 0x004B5E37, 0x00364922,
        0x005A6D46, 0x0061744D, 0x00788B64, 0x0074865D, 0x0075875E, 0x0076885F, 0x00778960, 0x0076885F, 0x0075875E, 0x0074865D, 0x0074865D, 0x0073815D, 0x00758760, 0x004C5B3B, 0x00293420, 0x00545B56, 0x004F5653, 0x0022301E, 0x001E3011, 0x00202D17,
        0x001D2C11, 0x002B3A1F, 0x003C4834, 0x004A5349, 0x00343E32, 0x00334226, 0x0055693E, 0x00768A60, 0x005B6A4E, 0x00545E52, 0x00999E9F, 0x00979C9F, 0x00565F55, 0x004A5840, 0x006B7C5A, 0x00667952, 0x00687A55, 0x00657553, 0x00576549, 0x00505D45,
        0x0077806C, 0x00555C4D, 0x001F2619, 0x001C1414, 0x00131817, 0x001C2A29, 0x00333F41, 0x0015191A, 0x001B1716, 0x001A1C16, 0x001A2319, 0x00212520, 0x001D211C, 0x001A1E19, 0x0023251F, 0x001F2317, 0x001F2310, 0x002C3219, 0x002F3718, 0x00262D12,
        0x002D321D, 0x00232519, 0x001B1E15, 0x001E2118, 0x001E2615, 0x004C583C, 0x005B6947, 0x002D3320, 0x001F2611, 0x00232C12, 0x00283117, 0x00252D16, 0x00212716, 0x00191C13, 0x001A1B17, 0x00171E17, 0x00191A16, 0x00191919, 0x00111B1B, 0x003C514F,
        0x00344946, 0x000D1613, 0x001D1914, 0x001B1E15, 0x005E6558, 0x0068715C, 0x00465438, 0x00425233, 0x00485B3A, 0x00445938, 0x003C5131, 0x00485839, 0x004A5A3B, 0x004E5E3F, 0x00505F43, 0x004E5D41, 0x004D5C41, 0x00536247, 0x005B6A4F, 0x0051614A,
        0x0047583D, 0x00455635, 0x004C5E39, 0x00506239, 0x004C5E35, 0x004B5D36, 0x00515F3B, 0x00576640, 0x00586741, 0x00596842, 0x00596842, 0x00576640, 0x00576640, 0x00586741, 0x005A6943, 0x00576640, 0x00576640, 0x0052613B, 0x004F5E38, 0x0054633D,
        0x005B6A44, 0x00596842, 0x0051603A, 0x003B4D26, 0x00455730, 0x0062744D, 0x00354720, 0x0040522B, 0x0064764F, 0x00384A23, 0x0042542D, 0x00596D43, 0x0030441A, 0x00546740, 0x004C5F38, 0x0031431E, 0x00546641, 0x00526341, 0x0030411F, 0x0053643F,
        0x00566849, 0x004B5C3B, 0x00394828, 0x003D5D52, 0x00478095, 0x004A8AA9, 0x005A91A6, 0x004A7889, 0x001A3F37, 0x00334633, 0x00757B76, 0x007A8184, 0x00C5D6D3, 0x00CEDFDC, 0x0097A0A9, 0x0074868D, 0x006F8188, 0x0076888F, 0x0084969D, 0x008EA0A7,
        0x0090A2A9, 0x0085979E, 0x0074868D, 0x00798B94, 0x007B8D96, 0x007F9198, 0x0079898F, 0x00778689, 0x00829193, 0x00B4C1C3, 0x00CCD8D8, 0x00829273, 0x006A7B5A, 0x006D7D5B, 0x00778765, 0x005F6D51, 0xFFA3AB9A, 0xFFFDFFFB, 0xFFFDFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xFFFEFFF7, 0x0098A284, 0x0065734A, 0x0083906A, 0x007D8965, 0x0049572D, 0x00415121, 0x007A845A, 0x00687248, 0x00596339, 0x004D572D, 0x00646E44,
        0x00697349, 0x00424C22, 0x003E481E, 0x006E7E5C, 0x00455238, 0x00596353, 0x00C0C6C1, 0x00E4E8E9, 0x00DADEDF, 0x00D4D9D7, 0x00D4DAD5, 0x00D7D8DC, 0x00E3E5E6, 0x00C0C4BF, 0x00697063, 0x00747D69, 0x00495539, 0x005D6B49, 0x0053613D, 0x005D6750,
        0x00545E47, 0x00465039, 0x00535E44, 0x005C674D, 0x00566246, 0x005B674B, 0x00566246, 0x00596640, 0x004A5731, 0x0075825C, 0x0073825C, 0x0074835D, 0x0061734C, 0x004E6039, 0x00354720, 0x004F613A, 0x0042542D, 0x00495B34, 0x0052643D, 0x003D4F28,
        0x00566841, 0x00687A53, 0x00758760, 0x0073855C, 0x0074865D, 0x0075875E, 0x0075875E, 0x0075875E, 0x0075875E, 0x0074865D, 0x0072845B, 0x0073815D, 0x007F916A, 0x00455533, 0x001E2A14, 0x005B635C, 0x0059605D, 0x00253222, 0x001A2C0F, 0x0027341E,
        0x00223018, 0x0028351F, 0x00424D3D, 0x00495347, 0x00253020, 0x00233216, 0x006A7E54, 0x0071855B, 0x00687859, 0x00566151, 0x00888F8C, 0x00909794, 0x004C564A, 0x003E4D32, 0x006E805B, 0x006C7F58, 0x00657752, 0x005C6C4A, 0x00576549, 0x006C7961,
        0x00838C78, 0x00676E5F, 0x00333A2D, 0x00211C13, 0x00121912, 0x00152420, 0x002C3838, 0x00131718, 0x001C1719, 0x001A1B19, 0x001B221D, 0x001A1E18, 0x00191D17, 0x00181C17, 0x001D1F19, 0x001A1E13, 0x001D2011, 0x00292E19, 0x002A3017, 0x002B3118,
        0x002C301D, 0x0027281E, 0x001D1E1A, 0x001F201C, 0x00181D14, 0x002B3223, 0x002E3723, 0x001F2318, 0x001C2015, 0x00262A1E, 0x0024281C, 0x001A1E13, 0x00151711, 0x00151614, 0x001C1B1D, 0x00181D1C, 0x001C1C1C, 0x001B1A1C, 0x000E171A, 0x00344947,
        0x002F463E, 0x000C170D, 0x001D1B10, 0x0025291D, 0x007B8372, 0x0088927B, 0x005B6A4B, 0x004D5E3C, 0x004F643E, 0x004F6542, 0x00536946, 0x00526341, 0x00526341, 0x00546543, 0x00556645, 0x00556645, 0x00556546, 0x0059694A, 0x005E6E4F, 0x00637459,
        0x005C6E4F, 0x00576944, 0x0055693F, 0x0056693E, 0x0055683D, 0x0056683F, 0x005C6B45, 0x005B6945, 0x005C6A46, 0x005B6945, 0x005A6844, 0x00596743, 0x00596743, 0x005B6945, 0x005D6B47, 0x00586642, 0x005D6B47, 0x005F6D49, 0x005E6C48, 0x005F6D49,
        0x005E6C48, 0x00566440, 0x004B5935, 0x0042542D, 0x0063754E, 0x00394B24, 0x00485A33, 0x00596B44, 0x003D4F28, 0x0053653E, 0x005D6F48, 0x003B4F25, 0x0045592F, 0x00566942, 0x003B4E27, 0x004A5C37, 0x00536540, 0x002B3C1A, 0x00495A38, 0x00596A45,
        0x00506243, 0x00465736, 0x00425131, 0x004C6C61, 0x004D869B, 0x004A8AA9, 0x005A91A6, 0x00497788, 0x001A3F37, 0x00314431, 0x00747A75, 0x007A8184, 0x00C6D7D4, 0x00CEDFDC, 0x0097A0A9, 0x00778990, 0x006F8188, 0x0073858C, 0x0081939A, 0x008D9FA6,
        0x0091A3AA, 0x008799A0, 0x0076888F, 0x00778992, 0x007C8F96, 0x0084969D, 0x007F8F95, 0x007C8B8E, 0x00919EA0, 0x00BECCCB, 0x00C8D4D4, 0x0079896A, 0x00718261, 0x00748462, 0x007A8A68, 0x00637155, 0xFFA4AC9B, 0xFFF9FDF7, 0xFFFBFDFE, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xFFFFFFF8, 0x0097A183, 0x0066744B, 0x00808D67, 0x0074805C, 0x00435127, 0x00374717, 0x00434C24, 0x003C451D, 0x00343D15, 0x00363F17, 0x00454E26,
        0x003C451D, 0x002F3810, 0x00434C24, 0x00758465, 0x005D6A50, 0x00596352, 0x00B2BAAF, 0x00EBF2EB, 0x00E2E9E2, 0x00D6DED4, 0x00DDE5DA, 0x00E1E4E2, 0x00F4F8F3, 0x00C4CABF, 0x006A735F, 0x00939E84, 0x00667452, 0x007C8C63, 0x006D7E53, 0x006F7D5B,
        0x0073815F, 0x006C7A56, 0x0072805C, 0x0074835D, 0x00697852, 0x006A7953, 0x006C7C53, 0x0055623C, 0x00606D47, 0x0053603A, 0x005F6E48, 0x0085946E, 0x005A6C45, 0x00586A43, 0x00374922, 0x0050623B, 0x003F512A, 0x004C5E37, 0x004F613A, 0x003C4E27,
        0x00576942, 0x00667851, 0x00768861, 0x0072845B, 0x0073855C, 0x0073855C, 0x0072845B, 0x0073855C, 0x0075875E, 0x0075875E, 0x0073855C, 0x00778561, 0x0073855E, 0x0062734E, 0x0025321A, 0x00636C62, 0x00838A87, 0x002B372B, 0x001D2D16, 0x00202D17,
        0x00323E2A, 0x00121D0D, 0x00606A5D, 0x00535D50, 0x00323E2A, 0x002D3D1E, 0x005A6D46, 0x006A7E53, 0x006C7D5C, 0x0065715D, 0x00A3ACA2, 0x00B6BFB5, 0x00606C5A, 0x00445337, 0x00788A65, 0x00586B44, 0x00475934, 0x00465634, 0x00465438, 0x005E6B53,
        0x006F7864, 0x006C7364, 0x003E4538, 0x00373426, 0x00232B20, 0x001E2F26, 0x00323F3D, 0x001A1E1F, 0x00211C1E, 0x001D1C1E, 0x001E2423, 0x001A2015, 0x001C2118, 0x0021251F, 0x00262723, 0x0023251F, 0x0025261C, 0x00282B1B, 0x00232714, 0x00262B16,
        0x00232617, 0x0026271E, 0x001B1C1A, 0x001D1D1D, 0x00141616, 0x00171A18, 0x00161A15, 0x001C1D1B, 0x00161715, 0x00191A18, 0x00181917, 0x00191919, 0x001D1C1E, 0x0019181C, 0x001B191F, 0x00121617, 0x00161519, 0x001E1D21, 0x00182124, 0x003E5350,
        0x00395147, 0x00202D1F, 0x003B3B2B, 0x002F3425, 0x00616A56, 0x00616D51, 0x00475735, 0x00475934, 0x00455A33, 0x00415931, 0x00455C36, 0x004D6039, 0x004C5F38, 0x004D5F3A, 0x004E603B, 0x004F613C, 0x004F603E, 0x00516240, 0x00536442, 0x004D5F40,
        0x004D613E, 0x004F623B, 0x004E6237, 0x004F6336, 0x00506338, 0x0051633A, 0x0053623C, 0x0052633E, 0x0052633E, 0x0053643F, 0x0051623D, 0x0050613C, 0x0050613C, 0x0051623D, 0x0053643F, 0x0052633E, 0x00556641, 0x00566742, 0x0053643F, 0x0052633E,
        0x0052633E, 0x004F603B, 0x004A5B36, 0x00596B44, 0x003F512A, 0x004A5C35, 0x005D6F48, 0x003C4E27, 0x0040522B, 0x005F714A, 0x00394B24, 0x004C6036, 0x005A6E44, 0x0031441D, 0x004B5E37, 0x00566843, 0x00354722, 0x004F603E, 0x00576846, 0x004F603B,
        0x00455738, 0x00415231, 0x00455434, 0x00507065, 0x004E879C, 0x004989A8, 0x00568DA2, 0x00487687, 0x001B4038, 0x00304330, 0x00737974, 0x007B8285, 0x00C6D7D4, 0x00CDDEDB, 0x0098A1AA, 0x0085979E, 0x007D8F96, 0x007D8F96, 0x0083959C, 0x00899BA2,
        0x008FA1A8, 0x008FA1A8, 0x008799A0, 0x0084979E, 0x0085979E, 0x0090A0A6, 0x0088999C, 0x00758486, 0x008D9B9A, 0x00B5C1C1, 0x00AAB7B5, 0x006F7F60, 0x00738463, 0x00748462, 0x00758563, 0x005F6D51, 0xFFA1A998, 0xFFF9FDF7, 0xFFFDFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFCFF, 0xFFFDFFF6, 0x00909A7C, 0x00647249, 0x00818E68, 0x007A8662, 0x005B693F, 0x00546434, 0x00515A32, 0x004E572F, 0x004B542C, 0x00505931, 0x00515A32,
        0x00454E26, 0x00464F27, 0x00576038, 0x007B8A6B, 0x007D8B6F, 0x006F7C64, 0x00929D89, 0x00A0AA99, 0x00939D8C, 0x0095A08C, 0x00A9B4A0, 0x00959C8D, 0x00939B8A, 0x00818A75, 0x00505B41, 0x007C8969, 0x005D6C46, 0x0078895E, 0x0075865A, 0x0076875C,
        0x00798A5F, 0x0077885D, 0x0077885C, 0x0075865A, 0x0075865A, 0x007A8C5D, 0x00788A5B, 0x006D7A54, 0x0053603A, 0x005B6842, 0x00586741, 0x006A7953, 0x005F714A, 0x00586A43, 0x0033451E, 0x00566841, 0x003C4E27, 0x0053653E, 0x00475932, 0x00394B24,
        0x005A6C45, 0x005B6D46, 0x00778962, 0x0075875E, 0x00778960, 0x0076885F, 0x0073855C, 0x0073855C, 0x0076885F, 0x0076885F, 0x0073855C, 0x007B8967, 0x00708259, 0x004F6138, 0x00324024, 0x00778175, 0x00A5ACA9, 0x00667167, 0x00162412, 0x0023301A,
        0x00162111, 0x003C453B, 0x008F988E, 0x006D7868, 0x004D5B43, 0x002D3E1D, 0x006A7C57, 0x0075895E, 0x006D7F5A, 0x004C5B40, 0x00475341, 0x00455040, 0x003D4A34, 0x00586948, 0x00647750, 0x00475A33, 0x00435530, 0x004B5B39, 0x0048563A, 0x005C6951,
        0x00646D59, 0x00666D5E, 0x00444B3E, 0x003F3C2D, 0x002A3326, 0x00213229, 0x00313F3B, 0x00181C1D, 0x001C1719, 0x00161616, 0x00191F1E, 0x001B2215, 0x00191F14, 0x001E221C, 0x0022231F, 0x0020211D, 0x00262620, 0x0026271D, 0x00222418, 0x001E2014,
        0x001C1D13, 0x0025251F, 0x001A1B19, 0x001D1C1E, 0x00151718, 0x00161819, 0x00161819, 0x001C1C1C, 0x00151416, 0x00151416, 0x00151418, 0x001B1A1E, 0x001D1C20, 0x00151418, 0x00151416, 0x00191D1E, 0x00161517, 0x001D1C20, 0x001B2525, 0x003B514C,
        0x002F473B, 0x001C2919, 0x0040402E, 0x00383D2E, 0x00515A46, 0x004F5B3F, 0x00465634, 0x00495B36, 0x00465B34, 0x00445C34, 0x00475E38, 0x00495C35, 0x00495C35, 0x00485B34, 0x00495B36, 0x004A5C37, 0x004C5E39, 0x004D5E3C, 0x004C5D3B, 0x00495C3B,
        0x004D613E, 0x0051643D, 0x0051653A, 0x00516439, 0x0051633A, 0x0050613C, 0x00505E3C, 0x00556543, 0x00566644, 0x00576745, 0x00576745, 0x00566644, 0x00546442, 0x00546442, 0x00556543, 0x00576745, 0x00586846, 0x00566644, 0x00526240, 0x0051613F,
        0x00546442, 0x00576745, 0x00596947, 0x0050623B, 0x00394B24, 0x005D6F48, 0x003E5029, 0x0043552E, 0x0062744D, 0x0034461F, 0x0043552E, 0x0054683E, 0x00384C22, 0x0052653E, 0x004E613A, 0x00364823, 0x004A5C37, 0x00566745, 0x00546543, 0x00697A55,
        0x005F7152, 0x00556645, 0x004C5B3B, 0x004C6C61, 0x004C859A, 0x004A8AA9, 0x00548BA0, 0x00497788, 0x001C4139, 0x00304330, 0x00727873, 0x007D8487, 0x00C7D8D5, 0x00CCDDDA, 0x0097A0A9, 0x008A9CA3, 0x0086989F, 0x008799A0, 0x008799A0, 0x0084969D,
        0x00889AA1, 0x008FA1A8, 0x0090A2A9, 0x0091A3AA, 0x00879A9F, 0x0093A3A9, 0x008F9EA1, 0x006F7C7E, 0x008B9797, 0x00B4C1BF, 0x00A0ABA9, 0x006A7A5B, 0x00748564, 0x00738361, 0x00738361, 0x005F6D51, 0xFFA2AA99, 0xFFFCFFFA, 0xFFFDFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFCFF, 0xFFFDFFF6, 0x008C9678, 0x0065734A, 0x0083906A, 0x0085916D, 0x007A885E, 0x007C8C5C, 0x00829067, 0x007D8B62, 0x0079875E, 0x007A885F, 0x007D8B62,
        0x007E8C63, 0x007D8B62, 0x0077855C, 0x00707F5F, 0x00758464, 0x006E7D5D, 0x00788768, 0x006C7B5C, 0x006B7A5B, 0x00758465, 0x007A896A, 0x00788466, 0x00687456, 0x007B8868, 0x00687654, 0x006C7B55, 0x00697950, 0x0085966B, 0x0074855A, 0x0076875C,
        0x0075865B, 0x00798A5E, 0x007C8D61, 0x00768859, 0x00738556, 0x00768859, 0x00768958, 0x007B8862, 0x00707D57, 0x0057643E, 0x005B6A44, 0x0051603A, 0x0041532C, 0x0062744D, 0x00354720, 0x004C5B35, 0x004E5D37, 0x00495832, 0x0052613B, 0x004C5B35,
        0x004D5C36, 0x00606F49, 0x0064734D, 0x0074865D, 0x007A8C63, 0x007B8D64, 0x00778960, 0x0075875E, 0x00778960, 0x00778960, 0x0073855C, 0x0072805E, 0x008A9C73, 0x005D7045, 0x00314020, 0x006B7568, 0x00A4ABA8, 0x00AAB4AE, 0x008A968A, 0x0066735D,
        0x00717B6E, 0x00A6ADAA, 0x00A1A8A3, 0x00919C8C, 0x00657458, 0x00415230, 0x0070815F, 0x006C8154, 0x0070835C, 0x00485839, 0x002B3921, 0x002A3721, 0x002F3E22, 0x00495A38, 0x00475A33, 0x00475A33, 0x004C5E39, 0x004D5D3B, 0x00475539, 0x006B7860,
        0x00737C68, 0x00545B4C, 0x002A3124, 0x002F2B20, 0x001F271C, 0x001A2A23, 0x002E3C38, 0x00171C1B, 0x001B1716, 0x00181915, 0x00222A23, 0x002E3526, 0x0020271A, 0x001B1F19, 0x00181917, 0x00161616, 0x001F1D1C, 0x0021201C, 0x0022221C, 0x0021201C,
        0x00201F1B, 0x00272622, 0x001C1D1B, 0x001E1E1E, 0x00151717, 0x00181A1A, 0x00191B1C, 0x001C1D1B, 0x001B1B1B, 0x001D1C1E, 0x0019181C, 0x0018171B, 0x0019181A, 0x001C1D1B, 0x002C2D29, 0x00202621, 0x00131412, 0x00161616, 0x0016211F, 0x00354B46,
        0x00284036, 0x00172416, 0x003D3C2E, 0x002F3327, 0x00555D4C, 0x005E6851, 0x004F5E3F, 0x00485937, 0x00475C36, 0x004C623F, 0x004E6441, 0x0051643D, 0x0052643F, 0x0051633E, 0x0051633E, 0x00536442, 0x00566745, 0x00556645, 0x00536443, 0x00546647,
        0x00566948, 0x00586A45, 0x00586B44, 0x00596B44, 0x00596A45, 0x00586747, 0x005A664A, 0x00576745, 0x00596947, 0x005A6A48, 0x005A6A48, 0x00586846, 0x00576745, 0x00566644, 0x00566644, 0x00576745, 0x005A6A48, 0x005C6C4A, 0x005B6B49, 0x00596947,
        0x00576745, 0x00556543, 0x00536341, 0x0053653E, 0x0064764F, 0x003B4D26, 0x0044562F, 0x005F714A, 0x0043552E, 0x00455730, 0x005D6F48, 0x00364A20, 0x00495D33, 0x005D7049, 0x00394C25, 0x00445631, 0x005C6E49, 0x00465735, 0x006C7D5B, 0x006C7D58,
        0x00667859, 0x005C6D4C, 0x00495838, 0x00446459, 0x004B8499, 0x004E8EAD, 0x00558CA1, 0x004B798A, 0x001F443C, 0x00314431, 0x00727873, 0x007E8588, 0x00C8D9D6, 0x00CBDCD9, 0x0097A0A9, 0x0086989F, 0x0086989F, 0x008B9DA4, 0x008D9FA6, 0x00899BA2,
        0x00899BA2, 0x008D9FA6, 0x008B9DA4, 0x008EA1A6, 0x0084949A, 0x008B9C9F, 0x008C9B9D, 0x006F7D7C, 0x008D9999, 0x00B7C2C0, 0x00A7B2B0, 0x006D7D5E, 0x00788968, 0x00768664, 0x00788866, 0x00616F53, 0xFFA1A998, 0xFFFBFFF9, 0xFFFAFCFD, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xFFFFFFF8, 0x008E987A, 0x0065734A, 0x007C8963, 0x007B8763, 0x0078865C, 0x007D8D5D, 0x0073855E, 0x007A8C65, 0x007F916A, 0x00788A63, 0x0073855E,
        0x0074865F, 0x00778962, 0x00798B64, 0x007D8D6B, 0x00798A65, 0x00768861, 0x00809269, 0x007C8F64, 0x0084976C, 0x0081936A, 0x00798B62, 0x007A8B5F, 0x008A9B6F, 0x0088996E, 0x00798A5F, 0x007B8B62, 0x007C8C63, 0x0073825C, 0x0075845E, 0x007D8D64,
        0x00798960, 0x0077885D, 0x0076875C, 0x00748559, 0x00708155, 0x00718256, 0x00788A5B, 0x0074815B, 0x00818E68, 0x006B7852, 0x004C5B35, 0x00576640, 0x0054663F, 0x003A4C25, 0x005B6D46, 0x0045542E, 0x0055643E, 0x0046552F, 0x0056653F, 0x00576640,
        0x00485731, 0x00596842, 0x0055643E, 0x0066784F, 0x0071835A, 0x00788A61, 0x0076885F, 0x0075875E, 0x00788A61, 0x00778960, 0x0074865D, 0x00758361, 0x006F8158, 0x0074885B, 0x00344422, 0x005F6A5A, 0x00A7AEAB, 0x00868F8C, 0x00828D85, 0x0097A48E,
        0x0098A197, 0x0084898A, 0x00A2A9A6, 0x009CA896, 0x00354625, 0x00556644, 0x007A8B6A, 0x00718659, 0x00697D53, 0x00566745, 0x00667559, 0x006C7B5F, 0x00364627, 0x00374924, 0x0050633C, 0x0042552E, 0x0042542F, 0x00425230, 0x00455337, 0x0078856D,
        0x0099A28E, 0x00686F60, 0x002C3326, 0x00231D18, 0x00191F1A, 0x001B2A26, 0x00323F3D, 0x001D2220, 0x00221F1A, 0x0022261B, 0x00313B2E, 0x00454D3C, 0x002D3427, 0x0022271E, 0x001C1D1B, 0x00181719, 0x001D1A1C, 0x001B1919, 0x001D1B1A, 0x001D1A1C,
        0x001D1A1C, 0x00201E1D, 0x001A1B17, 0x001F201C, 0x00171B16, 0x001E221D, 0x001C201B, 0x00171A11, 0x00181915, 0x001D1D1D, 0x0019181A, 0x00181719, 0x001C1D19, 0x002B2E25, 0x004B4F43, 0x00323A2F, 0x001E2118, 0x001B1C18, 0x001A2522, 0x003B514C,
        0x002E453D, 0x00121D15, 0x002C2A20, 0x002F3229, 0x005E6558, 0x00616A55, 0x0049573B, 0x004B5B3C, 0x00506342, 0x004D6241, 0x00465B3B, 0x0050613F, 0x00526341, 0x00526341, 0x00516241, 0x00526342, 0x00566647, 0x00556546, 0x00526243, 0x00536548,
        0x00516344, 0x00536443, 0x00556644, 0x00576745, 0x00576647, 0x0058654B, 0x005C6550, 0x00556644, 0x00566745, 0x00576846, 0x00566745, 0x00546543, 0x00536442, 0x00536442, 0x00546543, 0x00546543, 0x00556644, 0x00566745, 0x00566745, 0x00566745,
        0x00566745, 0x00576846, 0x00586947, 0x005F714A, 0x005B6D46, 0x004B5D36, 0x00596B44, 0x0040522B, 0x003B4D26, 0x0061734C, 0x00384A23, 0x004C6036, 0x0061754B, 0x0033461F, 0x00465932, 0x00566843, 0x004B5D38, 0x00748563, 0x006B7C5A, 0x006D7E59,
        0x00697B5C, 0x005F704F, 0x004B5A3A, 0x0045655A, 0x004E879C, 0x005191B0, 0x00548BA0, 0x004D7B8C, 0x0021463E, 0x00324532, 0x00727873, 0x007F8689, 0x00C8D9D6, 0x00CADBD8, 0x00969FA8, 0x007F9198, 0x007C8E95, 0x0083959C, 0x008D9FA6, 0x0090A2A9,
        0x0090A2A9, 0x008B9DA4, 0x00809299, 0x00839399, 0x00809096, 0x00829194, 0x00859294, 0x006F7B7B, 0x00828D8B, 0x009DA8A6, 0x009BA4A1, 0x0069795A, 0x00708160, 0x006E7E5C, 0x00748462, 0x005E6C50, 0xFF9EA695, 0xFFFCFFFA, 0xFFF8FAFB, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBF9FF, 0xFFFFFFF8, 0x00909A7C, 0x006A784F, 0x00808D67, 0x007B8763, 0x007B895F, 0x007F8F5F, 0x00788B64, 0x00788B64, 0x00798C65, 0x00798C65, 0x007C8F68,
        0x00798C65, 0x00758861, 0x007F926B, 0x00768762, 0x007A8C63, 0x00728659, 0x00728657, 0x006F8453, 0x006E8352, 0x00617546, 0x006A7E4F, 0x00677B46, 0x00617540, 0x00627544, 0x007D8F60, 0x006F8055, 0x00798862, 0x00778561, 0x007A8866, 0x00778660,
        0x00798862, 0x0074835D, 0x00718158, 0x00798960, 0x00798A5F, 0x00728358, 0x0077885D, 0x007D8A64, 0x0074815B, 0x007E8B65, 0x0074835D, 0x0054633D, 0x00576942, 0x0044562F, 0x004B5D36, 0x0056653F, 0x0042512B, 0x005C6B45, 0x0043522C, 0x00495832,
        0x005D6C46, 0x003D4C26, 0x00606F49, 0x0054663D, 0x0063754C, 0x00708259, 0x0073855C, 0x0074865D, 0x00778960, 0x00788A61, 0x0074865D, 0x00768462, 0x00798B62, 0x007D9164, 0x005A6A48, 0x00313D2B, 0x005B625F, 0x00A7AFAE, 0x0086908A, 0x00818E78,
        0x008D968C, 0x00ABB0B3, 0x008B9190, 0x003C4836, 0x00425332, 0x0071835E, 0x00758665, 0x006D8255, 0x0071855B, 0x00738560, 0x00708160, 0x005E6E4F, 0x00495A39, 0x00586A45, 0x0041552B, 0x004F623B, 0x00455732, 0x004A5A38, 0x0049573B, 0x0066735B,
        0x00969F8B, 0x00697061, 0x0021281B, 0x001C1414, 0x00151A18, 0x00182624, 0x002F3C3A, 0x00171D18, 0x0017150D, 0x00171C0D, 0x00293521, 0x002F3824, 0x00181F10, 0x00141910, 0x00171816, 0x0019181A, 0x001F1B20, 0x00191618, 0x001B1919, 0x001E1A20,
        0x001D191E, 0x001C1A1A, 0x00171814, 0x001E2118, 0x0013180F, 0x001A1F16, 0x0013180F, 0x001C2014, 0x001A1D14, 0x001D1E1A, 0x001B1B1B, 0x001A1B19, 0x00191C13, 0x00212617, 0x003E4431, 0x00273121, 0x0014180C, 0x0012140E, 0x00141F1C, 0x003C514E,
        0x00324843, 0x000E1812, 0x00191510, 0x002A2C26, 0x00696F64, 0x006A7261, 0x0048553D, 0x00465539, 0x00485A3B, 0x00465A3D, 0x004B5F42, 0x004C5D3C, 0x004E5F3E, 0x004F603F, 0x004E5E3F, 0x004F5F40, 0x00536246, 0x00526145, 0x004F5E42, 0x00506146,
        0x004D5F42, 0x004F5F40, 0x00536443, 0x00566546, 0x00546147, 0x00535F49, 0x0057604C, 0x00546544, 0x00546544, 0x00536443, 0x00526342, 0x00506140, 0x004F603F, 0x00506140, 0x00526342, 0x00546544, 0x00546544, 0x00546544, 0x00556645, 0x00546544,
        0x00536443, 0x00536443, 0x00546544, 0x0053653E, 0x0054663F, 0x00596B44, 0x003C4E27, 0x00485A33, 0x005A6C45, 0x003C4E27, 0x00475932, 0x0063774D, 0x002A3E14, 0x0052653E, 0x00586B44, 0x00394B26, 0x00546641, 0x00617250, 0x00596A48, 0x00596A45,
        0x00526445, 0x00495A39, 0x00394828, 0x0039594E, 0x00488196, 0x004F8FAE, 0x00548BA0, 0x004E7C8D, 0x00234840, 0x00324532, 0x00727873, 0x0080878A, 0x00C9DAD7, 0x00CADBD8, 0x00969FA8, 0x00778990, 0x00708289, 0x0074868D, 0x0083959C, 0x0090A2A9,
        0x0093A5AC, 0x008799A0, 0x0074868D, 0x0076868C, 0x007F9093, 0x007E8D90, 0x00869395, 0x007A8686, 0x00838E8C, 0x008F9A97, 0x0097A09D, 0x00748465, 0x00758665, 0x0070805E, 0x00778765, 0x005F6D51, 0xFFA1A998, 0xFFFDFFFB, 0xFFFDFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFCFF, 0xFFFFFFF7, 0x008E9979, 0x00637147, 0x007D8B62, 0x00818E68, 0x005C6B3F, 0x005B6C39, 0x005E6A40, 0x00626E44, 0x00636F45, 0x00616D43, 0x005D693F,
        0x005D693F, 0x00606C42, 0x00626E44, 0x006F7F5D, 0x004A593A, 0x0048553D, 0x005C6753, 0x0049553F, 0x00354327, 0x0032431E, 0x0036491E, 0x002B3820, 0x00313E24, 0x0039472B, 0x003A4727, 0x00475531, 0x0053603A, 0x0078865D, 0x007E8C62, 0x00768C5C,
        0x0074895C, 0x00708558, 0x007A8E61, 0x00778A5F, 0x006E7F54, 0x0076865D, 0x0077875E, 0x0075855C, 0x0078885F, 0x00708057, 0x007A8A61, 0x006B7B52, 0x0056663D, 0x005A6A41, 0x0047572E, 0x0051603A, 0x0055643E, 0x00404F29, 0x005E6D47, 0x00475630,
        0x004C5B35, 0x005D6C46, 0x003C4B25, 0x0066754F, 0x004A5933, 0x0064734D, 0x007F8E68, 0x0073825C, 0x007A8963, 0x0072815B, 0x00788761, 0x007A8B60, 0x0074845B, 0x007B8B62, 0x007C8C63, 0x005A6943, 0x0034431D, 0x0042502C, 0x006D7B57, 0x00778464,
        0x00748161, 0x0053613F, 0x00354321, 0x00465430, 0x006E7C58, 0x007C8A66, 0x0075835F, 0x00708057, 0x0073835A, 0x00798960, 0x00607047, 0x00495930, 0x005E6E45, 0x004A5A31, 0x0054643B, 0x0045542E, 0x00495733, 0x00495735, 0x004C583A, 0x005D684E,
        0x00727B66, 0x006D7564, 0x00343C2B, 0x00111614, 0x001A1F1E, 0x0025292A, 0x002E3135, 0x00141819, 0x001A1F1D, 0x00111811, 0x00171F14, 0x00181917, 0x00191A18, 0x001A1B19, 0x001A1B19, 0x00191A18, 0x00191A18, 0x00191A18, 0x00191A18, 0x00181818,
        0x00191919, 0x00191919, 0x001A1A1A, 0x001A1A1A, 0x00191919, 0x00181818, 0x00181818, 0x001A1B17, 0x001B1C18, 0x001C1D1B, 0x001B1C1A, 0x001A1A1A, 0x0019181A, 0x0019181A, 0x0019181A, 0x001A1817, 0x00141414, 0x0015161A, 0x0014171C, 0x0032393C,
        0x002F3939, 0x0015231F, 0x00091912, 0x001F251A, 0x00686F60, 0x007C8570, 0x005A684C, 0x00526141, 0x00576745, 0x00516240, 0x00516241, 0x00576745, 0x00556543, 0x00546442, 0x005B6B49, 0x005F6F4D, 0x005E6E4C, 0x005F6F4D, 0x005E6E4C, 0x005B6C47,
        0x005B6C47, 0x005A6B46, 0x00596A45, 0x00596A45, 0x00586944, 0x00576843, 0x00576843, 0x00576944, 0x00576944, 0x00576944, 0x00576944, 0x00576944, 0x00576944, 0x00576944, 0x00576944, 0x00596A45, 0x00596A45, 0x00596A45, 0x00586944, 0x00586944,
        0x00586944, 0x00586944, 0x00576843, 0x005B6D48, 0x00556742, 0x005A6C47, 0x005F714C, 0x0051633E, 0x003E502B, 0x00475934, 0x0060724D, 0x0031431E, 0x00536540, 0x00566843, 0x003B4D28, 0x00485A35, 0x005C6E49, 0x00546641, 0x00556742, 0x00566343,
        0x004B5D40, 0x00455435, 0x00485436, 0x0057746B, 0x004C8299, 0x00508DAF, 0x00558BA2, 0x00497789, 0x001D423A, 0x002F4130, 0x00717772, 0x0080868B, 0x00CBDBDA, 0x00CCDCDB, 0x00949DA7, 0x0074868D, 0x006F8188, 0x006D7F86, 0x007A8C93, 0x008A9CA3,
        0x008EA0A7, 0x0086989F, 0x007A8C93, 0x00798C93, 0x0075888F, 0x00809398, 0x00809096, 0x0079888B, 0x00879397, 0x00A6B3B5, 0x00C9D5D7, 0x007E8A78, 0x006C7A62, 0x00798868, 0x00778863, 0x005F6E4E, 0xFF9EA792, 0xFFFCFFFA, 0xFFFDFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF9F7FD, 0xFFFFFFF7, 0x008E9979, 0x0067754B, 0x00818F66, 0x0078855F, 0x00435226, 0x003C4D1A, 0x0039451B, 0x003C481E, 0x002B370D, 0x00313D13, 0x00414D23,
        0x00354117, 0x00323E14, 0x00414D23, 0x006E7C60, 0x0054604A, 0x00596255, 0x00929992, 0x00A7AEA7, 0x00A7AFA4, 0x00A4AF9B, 0x00AAB79F, 0x00A7B1A0, 0x00B0BBA7, 0x008C9882, 0x00465137, 0x004C583A, 0x00505B3B, 0x007B8763, 0x0078855F, 0x0075895E,
        0x007A8E63, 0x00778B60, 0x00788B60, 0x00798D60, 0x007A8B5F, 0x007B8C60, 0x0078895D, 0x0075855C, 0x0077875E, 0x0073835A, 0x007B8B62, 0x00798960, 0x005F6F46, 0x0056663D, 0x00607047, 0x0041502A, 0x004F5E38, 0x0061704A, 0x003A4923, 0x0063724C,
        0x0045542E, 0x004F5E38, 0x0062714B, 0x00374620, 0x00687751, 0x0054633D, 0x0064734D, 0x0086956F, 0x006D7C56, 0x007E8D67, 0x00778660, 0x0078895E, 0x00728259, 0x00728259, 0x007A8A61, 0x00778660, 0x0061704A, 0x00465430, 0x00364420, 0x00293616,
        0x00384525, 0x00485634, 0x005C6A48, 0x00768460, 0x0083916D, 0x007E8C68, 0x00778660, 0x0078885F, 0x006E7E55, 0x006A7A51, 0x004D5D34, 0x005E6E45, 0x0045552C, 0x00596940, 0x0046562D, 0x0053623C, 0x0053623C, 0x00495735, 0x005F6C4C, 0x00515D41,
        0x006B755E, 0x006F7864, 0x00212A16, 0x00151A18, 0x000E1312, 0x00292D2E, 0x003C3F43, 0x000E1115, 0x00121716, 0x00171D18, 0x00161E14, 0x00171A18, 0x00181B19, 0x00181B19, 0x00181B19, 0x00171A18, 0x00171A18, 0x00181B19, 0x00181B19, 0x00181A1A,
        0x00181A1A, 0x00181A1A, 0x00181A1A, 0x00171919, 0x00171919, 0x00171919, 0x00171919, 0x00161917, 0x00171A18, 0x00181A1A, 0x00181A1A, 0x0017191A, 0x0017191A, 0x0018191D, 0x00191A1E, 0x00171919, 0x00181C1D, 0x0015181C, 0x00212429, 0x0051555A,
        0x00384040, 0x000B1411, 0x00131E16, 0x00252C1F, 0x00697160, 0x0078826B, 0x00526142, 0x00485836, 0x004F5F3D, 0x004C5D3B, 0x004C5D3C, 0x004E5E3C, 0x00465634, 0x004E5E3C, 0x00566644, 0x004D5D3B, 0x004D5D3B, 0x00546442, 0x004D5D3B, 0x00546540,
        0x00546540, 0x0053643F, 0x0053643F, 0x0052633E, 0x0051623D, 0x0051623D, 0x0051623D, 0x0051633E, 0x0051633E, 0x0051633E, 0x0051633E, 0x0051633E, 0x0051633E, 0x0051633E, 0x0051633E, 0x00556641, 0x00556641, 0x00556641, 0x00546540, 0x00546540,
        0x00546540, 0x00546540, 0x00546540, 0x0050623D, 0x00566843, 0x00566843, 0x0052643F, 0x00536540, 0x00556742, 0x004B5D38, 0x003A4C27, 0x004C5E39, 0x00586A45, 0x003C4E29, 0x00485A35, 0x005E704B, 0x004D5F3A, 0x004E603B, 0x00546641, 0x0053613F,
        0x004A5C3F, 0x00475637, 0x00465234, 0x00516E65, 0x004B8198, 0x00528FB1, 0x00558BA2, 0x004D7B8D, 0x001F443C, 0x002F4130, 0x00707671, 0x0080868B, 0x00CADAD9, 0x00CBDBDA, 0x00959EA8, 0x0085979E, 0x0083959C, 0x0082949B, 0x008799A0, 0x008C9EA5,
        0x008EA0A7, 0x008B9DA4, 0x008799A0, 0x0080939A, 0x007A8D94, 0x0082959A, 0x007F8F95, 0x00768588, 0x00849094, 0x00A7B4B6, 0x00CEDADC, 0x0099A593, 0x006F7D65, 0x00707F5F, 0x00748560, 0x00606F4F, 0xFF9FA893, 0xFFF9FDF7, 0xFFFBFCFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFCFF, 0xFFFFFFF7, 0x008C9777, 0x0066744A, 0x0085936A, 0x0077845E, 0x003E4D21, 0x003D4E1B, 0x006B774D, 0x005F6B41, 0x00434F25, 0x004B572D, 0x005A663C,
        0x00424E24, 0x003A461C, 0x004C582E, 0x006B7761, 0x005D6757, 0x00686E69, 0x00BFC2C6, 0x00E9EBF3, 0x00EFF2F7, 0x00E5E9EA, 0x00EAEFED, 0x00E9F0E9, 0x00F1F8F1, 0x00AEB6AB, 0x00454C3D, 0x00545D49, 0x00515942, 0x006E775C, 0x00656F51, 0x00566647,
        0x004C5C3D, 0x00485737, 0x004C5D38, 0x0056653F, 0x00516237, 0x004E5F33, 0x00607041, 0x007B8B62, 0x0077875E, 0x0074845B, 0x00718158, 0x007D8D64, 0x007A8A61, 0x005D6D44, 0x0054643B, 0x005F6E48, 0x00404F29, 0x004A5933, 0x005C6B45, 0x0035441E,
        0x00687751, 0x0042512B, 0x004B5A34, 0x00606F49, 0x00374620, 0x0062714B, 0x0055643E, 0x0061704A, 0x007F8E68, 0x0076855F, 0x006E7D57, 0x00738459, 0x00798960, 0x007B8B62, 0x007A8A61, 0x007D8C66, 0x0082916B, 0x007C8A66, 0x0072805C, 0x00758361,
        0x00788664, 0x007C8A68, 0x0083916D, 0x0084926E, 0x007A8864, 0x0071805A, 0x0073825C, 0x007D8D64, 0x006B7B52, 0x0057673E, 0x00627249, 0x0048582F, 0x00697950, 0x0043532A, 0x0057673E, 0x0048582F, 0x004C5B35, 0x00586642, 0x00435030, 0x005E6A4C,
        0x00636E54, 0x004E5841, 0x00242E17, 0x00151B16, 0x00161B19, 0x001A1E1F, 0x001C1F23, 0x000C0F13, 0x001E2223, 0x00161B1A, 0x000F1510, 0x0014191A, 0x0014191A, 0x0014191A, 0x0014191A, 0x0014191A, 0x0014191A, 0x0014191A, 0x0014191A, 0x00121718,
        0x00111617, 0x00111617, 0x00101516, 0x00101516, 0x00101516, 0x00101516, 0x000F1415, 0x00111617, 0x00121718, 0x00131819, 0x00121718, 0x00111617, 0x00111617, 0x0012171A, 0x0013181B, 0x000A1212, 0x0012191C, 0x0014181D, 0x000C1015, 0x001C1F24,
        0x001C2021, 0x00151914, 0x00191D17, 0x00242C1B, 0x005F6854, 0x006F7A60, 0x00516040, 0x00485934, 0x004D5E39, 0x00485937, 0x00475836, 0x00495A35, 0x00546540, 0x0052633E, 0x004A5B36, 0x00546540, 0x0050613C, 0x00475833, 0x005E6F4A, 0x00576843,
        0x00566742, 0x00566742, 0x00566742, 0x00556641, 0x00546540, 0x00546540, 0x00546540, 0x00546641, 0x00546641, 0x00546641, 0x00546641, 0x00546641, 0x00546641, 0x00546641, 0x00546641, 0x00546540, 0x00546540, 0x00546540, 0x0053643F, 0x0053643F,
        0x0053643F, 0x0053643F, 0x0053643F, 0x0052643F, 0x0052643F, 0x0051633E, 0x0051633E, 0x00546641, 0x00536540, 0x004A5C37, 0x003F512C, 0x00556742, 0x00374924, 0x004C5E39, 0x00536540, 0x0051633E, 0x00697B56, 0x006F815C, 0x00738560, 0x00707E5C,
        0x00627457, 0x00596848, 0x00475434, 0x0047645B, 0x00457B92, 0x00518EAE, 0x005389A0, 0x00507E90, 0x0021463E, 0x002E402F, 0x006F7570, 0x0080868B, 0x00C9D9D8, 0x00CADAD9, 0x00969FA9, 0x008B9DA4, 0x008B9DA4, 0x008A9CA3, 0x0086989F, 0x0085979E,
        0x00889AA1, 0x008C9EA5, 0x0090A2A9, 0x00889BA2, 0x007F9299, 0x0085989D, 0x007F8F95, 0x00738285, 0x00808C90, 0x00A6B3B5, 0x00D1DDDF, 0x00DBE7D5, 0x008C9A82, 0x00778666, 0x00768762, 0x00606F4F, 0xFFA0A994, 0xFFFCFFFA, 0xFFFDFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xFFFFFFF7, 0x008A9575, 0x00647248, 0x0087956C, 0x00798660, 0x00455428, 0x004E5F2C, 0x006E7A50, 0x005C683E, 0x005C683E, 0x00647046, 0x005E6A40,
        0x00445026, 0x00364218, 0x0049552B, 0x006D7B5F, 0x0066715D, 0x00777E77, 0x00CED1D5, 0x00E8E9F3, 0x00E3E3EF, 0x00D3D5DD, 0x00D2D5DA, 0x00E5E8EC, 0x00EBEFF0, 0x00B1B6B4, 0x005B5F59, 0x005E6558, 0x00484E3D, 0x00414833, 0x00434B34, 0x00555D52,
        0x00454E41, 0x003B4535, 0x003A432E, 0x00455036, 0x00424C2E, 0x00374221, 0x004A5632, 0x00738459, 0x00728358, 0x007A8B60, 0x0074855A, 0x00718257, 0x007C8D62, 0x0076875C, 0x0065764B, 0x0053623C, 0x005C6B45, 0x003A4923, 0x0051603A, 0x005C6B45,
        0x0036451F, 0x0066754F, 0x003F4E28, 0x004F5E38, 0x00596842, 0x003B4A24, 0x0062714B, 0x004C5B35, 0x006D7C56, 0x0076855F, 0x007D8C66, 0x0077885D, 0x0078885F, 0x0076865D, 0x00728259, 0x0072815B, 0x00778660, 0x007C8A66, 0x007E8C68, 0x00849270,
        0x00859371, 0x00808E6A, 0x00798763, 0x00768460, 0x00788761, 0x007B8A64, 0x007E8D67, 0x006B7950, 0x005C6A41, 0x005B6940, 0x004E5C33, 0x00647249, 0x0048562D, 0x005E6C43, 0x0055633A, 0x004E5E35, 0x00596842, 0x0041502A, 0x005C6A48, 0x004C5939,
        0x00626E50, 0x00707C60, 0x004C583C, 0x0011190E, 0x00161D16, 0x00111614, 0x00111516, 0x0013161A, 0x00292C30, 0x00262A2B, 0x002D3231, 0x00272F2F, 0x00272F2F, 0x00272F2F, 0x00283030, 0x00293131, 0x00283030, 0x00272F2F, 0x00262E2E, 0x002A3232,
        0x002B3333, 0x002B3333, 0x002B3333, 0x002B3333, 0x00293131, 0x00283030, 0x00283030, 0x002B3235, 0x002B3235, 0x002B3333, 0x002A3232, 0x00283030, 0x00283030, 0x00293131, 0x002A3231, 0x00232F2F, 0x00283134, 0x00252C2F, 0x0015191E, 0x00151718,
        0x00212220, 0x0022231A, 0x0018170D, 0x003A432E, 0x00626D53, 0x00707C5E, 0x0063744F, 0x0064764F, 0x006A7C55, 0x00677954, 0x00687957, 0x00677551, 0x0072805C, 0x00616F4B, 0x00717F5B, 0x005B6945, 0x0043512D, 0x0063714D, 0x004F5D39, 0x005A6B46,
        0x005A6B46, 0x005A6B46, 0x00596A45, 0x00596A45, 0x00596A45, 0x00586944, 0x00586944, 0x00586A45, 0x00586A45, 0x00586A45, 0x00586A45, 0x00586A45, 0x00586A45, 0x00586A45, 0x00586A45, 0x005B6C47, 0x005B6C47, 0x005A6B46, 0x005A6B46, 0x005A6B46,
        0x005A6B46, 0x00596A45, 0x00596A45, 0x005C6E49, 0x00566843, 0x00566843, 0x005C6E49, 0x005A6C47, 0x00536540, 0x00546641, 0x005D6F4A, 0x002C3E19, 0x004B5D38, 0x004E603B, 0x00576944, 0x006D7F5A, 0x0070825D, 0x006E805B, 0x006B7D58, 0x00707E5C,
        0x00687A5B, 0x00667555, 0x00515E3E, 0x004C6A5F, 0x004C8398, 0x005491B1, 0x0052899E, 0x004E7C8E, 0x001F443C, 0x002B3D2C, 0x006D736E, 0x0080868B, 0x00C9D9D8, 0x00C9D9D8, 0x0097A0AA, 0x00889AA1, 0x00889AA1, 0x008B9DA4, 0x00889AA1, 0x00899BA2,
        0x008C9EA5, 0x00889AA1, 0x008799A0, 0x00889BA2, 0x007F9299, 0x0085989D, 0x00809096, 0x00728184, 0x007C888C, 0x00A2AFB1, 0x00D1DDDF, 0x00DCE8D6, 0x0084927A, 0x00748363, 0x00788964, 0x00627151, 0xFFA4AD98, 0xFFFDFFFB, 0xFFFDFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFBFF, 0xFFFFFFF7, 0x008C9777, 0x0067754B, 0x0087956C, 0x0075825C, 0x00415024, 0x004D5E2B, 0x00525E34, 0x004A562C, 0x00647046, 0x006B774D, 0x0069754B,
        0x0058643A, 0x0038441A, 0x004D592F, 0x00778863, 0x00707E62, 0x008C9686, 0x00D1D6D4, 0x00DBDEE2, 0x00D1D4D9, 0x00CACDD1, 0x00C4C8C9, 0x00CDCED8, 0x00DDDFE7, 0x00BFC2C6, 0x00909391, 0x0083887F, 0x00696E5F, 0x00585F4A, 0x00767E67, 0x008F9296,
        0x00767A7B, 0x00585B59, 0x004F534D, 0x00787C70, 0x00909685, 0x006C735E, 0x0050563F, 0x007D8E63, 0x0078895E, 0x0076875C, 0x007A8B60, 0x00738459, 0x00738459, 0x007F9065, 0x007D8E63, 0x0066754F, 0x0052613B, 0x0064734D, 0x003E4D27, 0x00586741,
        0x005A6943, 0x003D4C26, 0x005F6E48, 0x0041502A, 0x0056653F, 0x00576640, 0x003A4923, 0x00697852, 0x0053623C, 0x0072815B, 0x006F7E58, 0x0077885D, 0x0077875E, 0x0077875E, 0x0078885F, 0x00798862, 0x00778660, 0x00778561, 0x00778561, 0x00768460,
        0x007A8864, 0x007B8965, 0x00788662, 0x0076855F, 0x00788761, 0x0075855C, 0x006E7E55, 0x0058663D, 0x005E6C43, 0x00505E35, 0x005F6D44, 0x0056643B, 0x0057653C, 0x0058663D, 0x0058663D, 0x005D6D44, 0x0043532A, 0x005E6D47, 0x0054633D, 0x00677551,
        0x0072805E, 0x007C8A68, 0x00727F5F, 0x004A5541, 0x00182212, 0x000D150B, 0x001D2220, 0x00141819, 0x001C2021, 0x001D2122, 0x001A1F1E, 0x00151D1C, 0x00161E1D, 0x00171F1E, 0x00192120, 0x001A2221, 0x00192120, 0x00161E1D, 0x00131B1A, 0x00192120,
        0x001A2221, 0x001C2423, 0x001D2524, 0x001D2524, 0x001B2322, 0x00192120, 0x00171F1E, 0x00161E1E, 0x00161E1E, 0x00161E1E, 0x00171F1E, 0x0017201D, 0x0018211E, 0x0018221C, 0x0018221C, 0x00121D1B, 0x001C2725, 0x00131B1B, 0x00161B1A, 0x001E211F,
        0x0015160D, 0x001B1B0D, 0x003E402D, 0x00707B61, 0x007A8668, 0x007A8866, 0x0073855E, 0x0075875E, 0x0074865F, 0x0072845F, 0x00788967, 0x00778660, 0x0075845E, 0x00798862, 0x0071805A, 0x005D6C46, 0x004F5E38, 0x004A5933, 0x004B5A34, 0x0053643F,
        0x0053643F, 0x0053643F, 0x0053643F, 0x0053643F, 0x0052633E, 0x0052633E, 0x0052633E, 0x0052643F, 0x0052643F, 0x0052643F, 0x0052643F, 0x0052643F, 0x0052643F, 0x0052643F, 0x0052643F, 0x00546540, 0x00546540, 0x00546540, 0x00546540, 0x0053643F,
        0x0053643F, 0x0053643F, 0x0053643F, 0x004A5C37, 0x004E603B, 0x004E603B, 0x004B5D38, 0x004E603B, 0x00536540, 0x0050623D, 0x00465833, 0x004D5F3A, 0x005B6D48, 0x00586A45, 0x00657752, 0x00778964, 0x0072845F, 0x006E805B, 0x006F815C, 0x00687652,
        0x00617354, 0x0060704E, 0x00475533, 0x00466459, 0x004C8398, 0x005391AF, 0x00538A9F, 0x00497789, 0x001C4139, 0x00283A29, 0x006C726D, 0x0082888D, 0x00CADAD9, 0x00C7D7D6, 0x0097A0AA, 0x0082949B, 0x0081939A, 0x008B9DA4, 0x008FA1A8, 0x0097A9B0,
        0x009BADB4, 0x0086989F, 0x00798B92, 0x007F9299, 0x00778A91, 0x0082959A, 0x00839399, 0x00768588, 0x007D898D, 0x00A2AFB1, 0x00D2DEE0, 0x00C0CCBA, 0x00717F67, 0x006F7E5E, 0x00758661, 0x005C6B4B, 0xFFA3AC97, 0xFFFDFFFB, 0xFFFDFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBF9FF, 0xFFFFFFF7, 0x008C9777, 0x0069774D, 0x0086946B, 0x0076835D, 0x00435226, 0x004A5B28, 0x006A764C, 0x005D693F, 0x005F6B41, 0x005C683E, 0x007A865C,
        0x00748056, 0x003F4B21, 0x004D592F, 0x006A7C55, 0x00677657, 0x0098A291, 0x00CCD3CC, 0x00DDE2E1, 0x00DEE2E3, 0x00E8EDEB, 0x00E4EAE5, 0x00D9DAE4, 0x00E3E5ED, 0x00BABEBF, 0x00919590, 0x00838A7D, 0x005D6350, 0x00515A40, 0x00697257, 0x00A8ABA9,
        0x00898D88, 0x00626660, 0x0060635A, 0x008A8E82, 0x00A3A899, 0x00868979, 0x00626653, 0x00718256, 0x007C8D61, 0x00738458, 0x0075865A, 0x007B8C60, 0x0075865A, 0x00738458, 0x00708155, 0x007E8D67, 0x006E7D57, 0x004C5B35, 0x00606F49, 0x003E4D27,
        0x005D6C46, 0x004A5933, 0x004B5A34, 0x005A6943, 0x0041502A, 0x00586741, 0x0052613B, 0x0044532D, 0x0066754F, 0x004B5A34, 0x005A6943, 0x0057683D, 0x005D6D44, 0x005D6D44, 0x0057673E, 0x0053623C, 0x0056653F, 0x005A6844, 0x005C6A46, 0x00596743,
        0x0054623E, 0x0055633F, 0x005A6943, 0x005A6943, 0x00596940, 0x00596940, 0x0056663D, 0x005E6941, 0x00556038, 0x00636E46, 0x005A653D, 0x005A653D, 0x00667149, 0x00545F37, 0x005A653D, 0x00495930, 0x005F6F46, 0x004A5A31, 0x006D7D54, 0x00798960,
        0x007A8963, 0x00707F59, 0x006F7E58, 0x00738263, 0x003C492F, 0x00172110, 0x00131B10, 0x001A201B, 0x00282D2B, 0x001F2423, 0x00151A18, 0x001D241F, 0x001C231E, 0x001D241F, 0x001F2621, 0x00212823, 0x00202722, 0x001D241F, 0x0019201B, 0x0019211A,
        0x001B231C, 0x001D251E, 0x001E261F, 0x001E261F, 0x001D251E, 0x001B231C, 0x001A221B, 0x00171D1C, 0x00171E1B, 0x00171E1B, 0x001B221D, 0x001F2720, 0x00222C20, 0x00212B1F, 0x001F291C, 0x00182019, 0x001D231E, 0x00181E19, 0x0020241E, 0x00171D12,
        0x001C2211, 0x00464E37, 0x007D866B, 0x00788565, 0x00758361, 0x0072815B, 0x0073865B, 0x0076895E, 0x0072845B, 0x0070835C, 0x00758664, 0x0077845E, 0x007E8B65, 0x0073805A, 0x00525F39, 0x005E6B45, 0x0056633D, 0x0035421C, 0x005F6C46, 0x0053643F,
        0x0053643F, 0x0053643F, 0x0053643F, 0x0053643F, 0x0053643F, 0x0053643F, 0x0053643F, 0x0052643F, 0x0052643F, 0x0052643F, 0x0052643F, 0x0052643F, 0x0052643F, 0x0052643F, 0x0052643F, 0x0052633E, 0x0052633E, 0x0051623D, 0x0051623D, 0x0051623D,
        0x0051623D, 0x0050613C, 0x0050613C, 0x0050623D, 0x00576944, 0x00576944, 0x0050623D, 0x0052643F, 0x00596B46, 0x00556742, 0x00495B36, 0x00556742, 0x00445631, 0x006D7F5A, 0x00778964, 0x00677954, 0x0072845F, 0x006E805B, 0x00657752, 0x0072805C,
        0x00677A59, 0x005B6B49, 0x003A4826, 0x003D5B4E, 0x00477F92, 0x004D8BA9, 0x00558DA0, 0x00487688, 0x001A3F37, 0x00263827, 0x006B716C, 0x00848A8F, 0x00CADAD9, 0x00C6D6D5, 0x0098A1AB, 0x007A8C93, 0x0074868D, 0x007C8E95, 0x00809299, 0x0090A2A9,
        0x009DAFB6, 0x00899BA2, 0x007C8E95, 0x007A8D94, 0x0072858C, 0x00809398, 0x0086969C, 0x007B8A8D, 0x00808C90, 0x00A4B1B3, 0x00D5E1E3, 0x00BDC9B7, 0x00738169, 0x00758464, 0x00768762, 0x005A6949, 0xFFA0A994, 0xFFFCFFFA, 0xFFFDFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xFFFDFFF5, 0x008A9575, 0x0067754B, 0x0085936A, 0x007A8761, 0x0047562A, 0x00455623, 0x006D794F, 0x00616D43, 0x004A562C, 0x00445026, 0x00647046,
        0x00606C42, 0x003C481E, 0x0048542A, 0x006F7D61, 0x0079856F, 0x00BAC2B7, 0x00CED3D2, 0x00E0E3E7, 0x00D8DBDF, 0x00DFE4E3, 0x00DBE0DE, 0x00DFE2E7, 0x00E5E8EC, 0x00A9AEAC, 0x0082887D, 0x009BA392, 0x00576046, 0x00626D4D, 0x00606B4A, 0x005C674D,
        0x00586349, 0x00505B41, 0x00565F44, 0x005F684D, 0x00646B50, 0x00676F51, 0x00666E50, 0x004B5C30, 0x0067784C, 0x00798A5E, 0x00748559, 0x00748559, 0x0076875B, 0x006F8054, 0x00728357, 0x0072815B, 0x007C8B65, 0x0064734D, 0x004F5E38, 0x0065744E,
        0x0031401A, 0x0061704A, 0x00475630, 0x00475630, 0x005D6C46, 0x003E4D27, 0x00596842, 0x0053623C, 0x003F4E28, 0x00596842, 0x005C6B45, 0x0056673C, 0x0056663D, 0x0055653C, 0x0054643B, 0x00596842, 0x005F6E48, 0x005F6D49, 0x005A6844, 0x00616F4B,
        0x00586642, 0x00596842, 0x005F6E48, 0x005D6D44, 0x005B6B42, 0x005D6E43, 0x005D6E43, 0x0059643C, 0x005A653D, 0x00566139, 0x0057623A, 0x005D6840, 0x00535E36, 0x005B663E, 0x00525D35, 0x005F7045, 0x00516237, 0x00728358, 0x00718257, 0x00738459,
        0x0076875C, 0x007A8B60, 0x00708156, 0x0075875E, 0x0072835E, 0x004D5A40, 0x00182211, 0x000C1409, 0x00171E17, 0x00111712, 0x00111712, 0x00171D12, 0x00151B10, 0x00141A0F, 0x00151B10, 0x00171D12, 0x00181E13, 0x00161C11, 0x0013190E, 0x00141B0E,
        0x00151C0F, 0x00151C0F, 0x00161D10, 0x00171E11, 0x00171E11, 0x00171E11, 0x00171E11, 0x00151914, 0x00131711, 0x0013180F, 0x00181E13, 0x001E2518, 0x00212918, 0x001D2612, 0x0018210D, 0x00202216, 0x001B1C12, 0x0013170C, 0x001C2014, 0x001B2410,
        0x00424F35, 0x006E7E5C, 0x0074865F, 0x00768460, 0x0074835D, 0x0076875C, 0x00778B5E, 0x0075895C, 0x00718459, 0x006E815A, 0x006F815C, 0x007E8C63, 0x0078865D, 0x0056643B, 0x0058663D, 0x0049572E, 0x003E4C23, 0x005F6D44, 0x00546239, 0x00647550,
        0x00647550, 0x00647550, 0x00647550, 0x00647550, 0x00647550, 0x00647550, 0x00647550, 0x00647651, 0x00647651, 0x00647651, 0x00647651, 0x00647651, 0x00647651, 0x00647651, 0x00647651, 0x00677853, 0x00677853, 0x00667752, 0x00667752, 0x00667752,
        0x00667752, 0x00657651, 0x00657651, 0x00637550, 0x0062744F, 0x0062744F, 0x0061734E, 0x005E704B, 0x005C6E49, 0x005E704B, 0x00637550, 0x005D6F4A, 0x00798B66, 0x006C7E59, 0x006C7E59, 0x006E805B, 0x00667853, 0x00748661, 0x00748661, 0x006D7C56,
        0x00667958, 0x005D6D4B, 0x003F4D2B, 0x004B695C, 0x0050889B, 0x004B89A7, 0x00548C9F, 0x004C7A8C, 0x001D423A, 0x00263827, 0x006A706B, 0x00858B90, 0x00CADAD9, 0x00C6D6D5, 0x009AA3AD, 0x00809299, 0x00778990, 0x007B8D94, 0x00788A91, 0x0086989F,
        0x009AACB3, 0x0090A2A9, 0x008B9DA4, 0x0082959C, 0x00788B92, 0x0084979C, 0x008B9BA1, 0x007F8E91, 0x00818D91, 0x00A3B0B2, 0x00D4E0E2, 0x00B4C0AE, 0x006C7A62, 0x00738262, 0x00798A65, 0x00617050, 0xFFA3AC97, 0xFFF7FBF5, 0xFFFCFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xFBFBFFF3, 0x00899474, 0x0068764C, 0x0085936A, 0x007A8761, 0x00404F23, 0x00344512, 0x00323E14, 0x003B471D, 0x00333F15, 0x00364218, 0x0039451B,
        0x002F3B11, 0x0038441A, 0x004C582E, 0x006B7564, 0x00878F84, 0x00D5DAD9, 0x00D5D7DF, 0x00E7E7F3, 0x00D3D4DE, 0x00D2D5D9, 0x00D1D5D6, 0x00D5D8DC, 0x00E5EAE9, 0x00A9B0A9, 0x008B9285, 0x00C6D0B9, 0x005F694B, 0x00838F6B, 0x00748259, 0x0076875B,
        0x007B8C60, 0x0077865A, 0x0079875D, 0x007B875D, 0x00758157, 0x006A754D, 0x00515A32, 0x00596A3E, 0x004F6034, 0x006D7E52, 0x0075865A, 0x00738458, 0x007A8B5F, 0x006F8054, 0x0075865A, 0x00778660, 0x0071805A, 0x0082916B, 0x0065744E, 0x004E5D37,
        0x0064734D, 0x00384721, 0x00586741, 0x004B5A34, 0x004A5933, 0x0053623C, 0x0042512B, 0x0054633D, 0x00606F49, 0x0056653F, 0x00586741, 0x00596A3F, 0x0058683F, 0x005A6A41, 0x005C6C43, 0x00596842, 0x0054633D, 0x00576541, 0x005D6B47, 0x005B6945,
        0x00586741, 0x005A6943, 0x005B6A44, 0x0056663D, 0x00596940, 0x005C6D42, 0x00596A3F, 0x00606B43, 0x00545F37, 0x00616C44, 0x005A653D, 0x00556038, 0x005F6A42, 0x00505B33, 0x00626D45, 0x00516237, 0x006F8055, 0x0078895E, 0x00798A5F, 0x0075865B,
        0x00738458, 0x00718256, 0x007A8B5F, 0x00708455, 0x0075875E, 0x00758464, 0x004E5A44, 0x002F382B, 0x003E463C, 0x004A514A, 0x00404740, 0x004A4E42, 0x00474B3F, 0x0043473B, 0x0044483C, 0x00464A3E, 0x00484C40, 0x00484C40, 0x00464A3E, 0x00444A39,
        0x00434938, 0x00434938, 0x00434938, 0x00444A39, 0x00454B3A, 0x00474D3C, 0x00494F3E, 0x0043453F, 0x003F4239, 0x003E4237, 0x00424738, 0x00494F3E, 0x004A513C, 0x00434B34, 0x003C452B, 0x00444335, 0x00525143, 0x003E3E30, 0x003A402F, 0x0049533C,
        0x00687958, 0x00738861, 0x006F865A, 0x0073825C, 0x0075855C, 0x0078895D, 0x00748859, 0x00708457, 0x0072855A, 0x00768962, 0x00778964, 0x0076845B, 0x004D5B32, 0x005F6D44, 0x004B5930, 0x00414F26, 0x005C6A41, 0x00536138, 0x00637148, 0x00748560,
        0x00748560, 0x00748560, 0x00758661, 0x00758661, 0x00758661, 0x00758661, 0x00758661, 0x00748661, 0x00748661, 0x00748661, 0x00748661, 0x00748661, 0x00748661, 0x00748661, 0x00748661, 0x00758661, 0x00758661, 0x00748560, 0x00748560, 0x00748560,
        0x00748560, 0x0073845F, 0x0073845F, 0x00738560, 0x0072845F, 0x0072845F, 0x00738560, 0x00758762, 0x00758762, 0x00748661, 0x00748661, 0x0072845F, 0x0070825D, 0x0070825D, 0x006A7C57, 0x0070825D, 0x0071835E, 0x00677954, 0x006E805B, 0x00707F59,
        0x00697C5B, 0x005C6D48, 0x003D4B27, 0x004D6B5E, 0x0051899C, 0x004887A3, 0x00538B9E, 0x00517F91, 0x0020453D, 0x00273928, 0x006A706B, 0x00858B90, 0x00CADAD9, 0x00C7D7D6, 0x009CA5AF, 0x0093A5AC, 0x008EA0A7, 0x0095A7AE, 0x008B9DA4, 0x0091A3AA,
        0x00A0B2B9, 0x0096A8AF, 0x0095A7AE, 0x0090A3AA, 0x0081949B, 0x008A9DA2, 0x008E9EA4, 0x00808F92, 0x00808C90, 0x00A0ADAF, 0x00D2DEE0, 0x00C1CDBB, 0x00707E66, 0x006E7D5D, 0x0073845F, 0x00606F4F, 0xFFA4AD98, 0xFFF8FCF6, 0xFFFDFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFAFF, 0xFBFBFFF3, 0x00899474, 0x006C7A50, 0x00829067, 0x007E8B65, 0x00647347, 0x005E6F3C, 0x0058693E, 0x00596A3F, 0x005B6C41, 0x0054653A, 0x0055663B,
        0x00596A3F, 0x0056673C, 0x005F7045, 0x00728358, 0x00596842, 0x00869373, 0x00ADB99D, 0x0087917A, 0x006E795F, 0x00818C72, 0x008B977B, 0x00737E5D, 0x0075815D, 0x00737D59, 0x006A754F, 0x006C774F, 0x00697349, 0x007A8459, 0x007E895B, 0x00798862,
        0x00798960, 0x0075855C, 0x007A8A61, 0x00798A5F, 0x0078895E, 0x00798A5E, 0x0066774B, 0x0048592D, 0x005A6B3F, 0x004F6034, 0x006C7D51, 0x00798A5E, 0x0075865A, 0x0078895D, 0x0075865A, 0x00728455, 0x00748657, 0x00798B5C, 0x0077885C, 0x00617246,
        0x0057683D, 0x0055663B, 0x003E4F24, 0x0056663D, 0x004E5E35, 0x004F5F36, 0x0053633A, 0x004F5F36, 0x00516138, 0x0055653C, 0x0053633A, 0x0052613B, 0x0053623C, 0x0053623C, 0x0053623C, 0x0053623C, 0x0054633D, 0x0054633D, 0x0054633D, 0x0054633D,
        0x0055643E, 0x0055643E, 0x0056653F, 0x0056653F, 0x0055643E, 0x0055643E, 0x0054633D, 0x00526239, 0x0057673E, 0x005A6A41, 0x00526239, 0x0053633A, 0x0055653C, 0x005A6A41, 0x0057673E, 0x00748559, 0x00738458, 0x00738458, 0x00738458, 0x00738458,
        0x00748559, 0x00748559, 0x0075865A, 0x00728357, 0x00728357, 0x0074855A, 0x00788761, 0x00606C4E, 0x00838C77, 0x00848A7F, 0x00828680, 0x00A3A199, 0x009E9C94, 0x00A3A199, 0x009F9D95, 0x00A5A39B, 0x009D9B93, 0x009C9A92, 0x009A9890, 0x0099948B,
        0x00A5A097, 0x009C978E, 0x00928D84, 0x00B0ABA2, 0x00A09B92, 0x009B968D, 0x00A9A49B, 0x009E9C94, 0x00939189, 0x00A19F97, 0x0095938B, 0x009A9890, 0x00AFADA5, 0x0097958D, 0x008A8880, 0x00A2989E, 0x00A9A39E, 0x007C7D69, 0x0078825E, 0x00768559,
        0x00798D60, 0x006E8258, 0x0070855F, 0x0075885D, 0x00708358, 0x0076895E, 0x0076875C, 0x0078895E, 0x007D8B61, 0x0076845A, 0x007A885E, 0x004E5D37, 0x0055643E, 0x004B5A34, 0x00364821, 0x0052643D, 0x0054663F, 0x00586B44, 0x007F926B, 0x00748560,
        0x0073845F, 0x0072845D, 0x0072845D, 0x0072845B, 0x0072845B, 0x0072845B, 0x0072855A, 0x00758760, 0x0074865F, 0x0073855E, 0x0072845D, 0x0072845D, 0x0072845D, 0x0073855E, 0x0074865F, 0x0074865F, 0x0074865F, 0x0074865F, 0x0073855E, 0x0072845D,
        0x0071835C, 0x0070825B, 0x006F815A, 0x0073865F, 0x0073865F, 0x0072855E, 0x0072855E, 0x0071845D, 0x0070835C, 0x0070835C, 0x006F825B, 0x006F815C, 0x006F815C, 0x006F815C, 0x006F815C, 0x006E805B, 0x006E805B, 0x006E805B, 0x006E805B, 0x00697E5E,
        0x006B7A54, 0x00636D43, 0x00415334, 0x00416359, 0x005C8D95, 0x005C94A5, 0x0051899C, 0x004B7D89, 0x002C4C4B, 0x002E382B, 0x00767668, 0x00898D88, 0x00D4E0E4, 0x00BBCED5, 0x0090A4A9, 0x0091A0A9, 0x0093A2AB, 0x008F9EA7, 0x008C9BA4, 0x008D9CA5,
        0x008C9BA4, 0x008B9AA3, 0x00909FA8, 0x00869CA1, 0x0080999D, 0x007E979B, 0x00768F93, 0x0074888D, 0x007C8C92, 0x009DA7AE, 0x00CFD5DC, 0x00C9D3CD, 0x00849181, 0x006A795D, 0x00758760, 0x0063714D, 0xFFA3AC92, 0xFFFBFFF4, 0xFFFFFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xFFFFFFF7, 0x00879272, 0x006A784E, 0x007F8D64, 0x00828F69, 0x0079885C, 0x007C8D5A, 0x007F9065, 0x007C8D62, 0x007B8C61, 0x007C8D62, 0x007C8D62,
        0x007A8B60, 0x00798A5F, 0x00809166, 0x007A8C5D, 0x006F8054, 0x0072815B, 0x007B8965, 0x00778563, 0x0075835F, 0x00788761, 0x007C8C63, 0x007A8761, 0x00798660, 0x0076835D, 0x00717C54, 0x0077825A, 0x00778157, 0x00828C62, 0x00798358, 0x0075855C,
        0x0077875E, 0x0075865B, 0x0077885D, 0x0075865A, 0x0075865A, 0x007B8C60, 0x00708253, 0x0065764A, 0x0048592D, 0x005B6C40, 0x0056673B, 0x00738458, 0x007B8C60, 0x007C8D61, 0x00718256, 0x00768859, 0x00758758, 0x006F8152, 0x0078895D, 0x0076875B,
        0x00627348, 0x00596A3F, 0x0056673C, 0x0043532A, 0x004D5D34, 0x0048582F, 0x004B5B32, 0x005C6C43, 0x00596940, 0x00526239, 0x005F6F46, 0x00596842, 0x00596842, 0x00596842, 0x005A6943, 0x005A6943, 0x005A6943, 0x005A6943, 0x005A6943, 0x00596842,
        0x00596842, 0x005A6943, 0x005A6943, 0x005A6943, 0x005A6943, 0x00596842, 0x00596842, 0x0055653C, 0x0055653C, 0x004F5F36, 0x00596940, 0x00516138, 0x005B6B42, 0x005E6E45, 0x006D7D54, 0x0076875B, 0x0076875B, 0x0076875B, 0x0076875B, 0x0076875B,
        0x0076875B, 0x0076875B, 0x0076875B, 0x0077885C, 0x00798A5E, 0x00738459, 0x007F8E68, 0x00546141, 0x00727C65, 0x006E7568, 0x007B8077, 0x008A8A7E, 0x0089897D, 0x00838377, 0x007F7F73, 0x008B8B7F, 0x008E8E82, 0x009B9B8F, 0x00959589, 0x0089887A,
        0x00868577, 0x007D7C6E, 0x00888779, 0x007C7B6D, 0x008C8B7D, 0x007A796B, 0x008E8D7F, 0x008B8D81, 0x007F8175, 0x008E9084, 0x0085877B, 0x0086887C, 0x0094968A, 0x0085877B, 0x00828478, 0x00938B8B, 0x00908E84, 0x005F634A, 0x00707B55, 0x00758758,
        0x00708455, 0x00708459, 0x00738861, 0x0074865D, 0x0074865D, 0x00708259, 0x007D8D64, 0x00728259, 0x00748259, 0x007B8960, 0x004A582F, 0x00596842, 0x004C5B35, 0x0036451F, 0x005B6D46, 0x004F613A, 0x005F714A, 0x00748760, 0x0072855E, 0x00748560,
        0x0073845F, 0x0073855E, 0x0072845D, 0x0072845B, 0x0072845B, 0x0072845B, 0x0072855A, 0x0074865F, 0x0073855E, 0x0073855E, 0x0072845D, 0x0072845D, 0x0073855E, 0x0074865F, 0x0074865F, 0x00758760, 0x00758760, 0x0074865F, 0x0073855E, 0x0072845D,
        0x0071835C, 0x0070825B, 0x0070825B, 0x0073865F, 0x0073865F, 0x0072855E, 0x0072855E, 0x0071845D, 0x0070835C, 0x0070835C, 0x0070835C, 0x0070825D, 0x006F815C, 0x006F815C, 0x006F815C, 0x006F815C, 0x006F815C, 0x006E805B, 0x006E805B, 0x00697C5B,
        0x006D7A54, 0x00687248, 0x004A5C3D, 0x0044665F, 0x00548692, 0x00568FA5, 0x00518AA3, 0x004B7F8F, 0x002D5053, 0x002B392E, 0x0066675D, 0x00838886, 0x00D2DEE4, 0x00BCCFD4, 0x007B8E91, 0x006C838B, 0x006F868E, 0x006C838B, 0x006A8189, 0x006C838B,
        0x006A8189, 0x00687F87, 0x006C838B, 0x006F8388, 0x00677D82, 0x00647D81, 0x005D767A, 0x0061757A, 0x0074848A, 0x009CA6AD, 0x00CBD1D8, 0x00E4EEE8, 0x009DAA9C, 0x00728165, 0x00778962, 0x00616F4B, 0xFF9FA88E, 0xFFFEFFF8, 0xFFFFFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xFFFFFFF7, 0x00869171, 0x006F7D53, 0x007E8C63, 0x007E8B65, 0x007A895D, 0x007C8D5A, 0x0078895E, 0x00798A5F, 0x0076875C, 0x00798A5F, 0x0077885D,
        0x00798A5F, 0x007A8B60, 0x007C8D62, 0x007F9261, 0x00829465, 0x0077885D, 0x0076865D, 0x007D8D64, 0x00849569, 0x00809362, 0x007E915E, 0x0083936A, 0x00839168, 0x0086946B, 0x00849269, 0x00828D65, 0x0077825A, 0x00848D65, 0x007E875F, 0x0077885D,
        0x0077885D, 0x00748559, 0x0075865A, 0x00748657, 0x00748657, 0x007A8D5C, 0x00768958, 0x00748559, 0x0065764A, 0x0046572B, 0x0058693D, 0x0055663A, 0x00728357, 0x0077885C, 0x006E7F53, 0x00738556, 0x00788A5B, 0x00728455, 0x00748559, 0x00798A5E,
        0x006F8055, 0x00607146, 0x00536439, 0x0054643B, 0x00495930, 0x0055653C, 0x00617148, 0x00596940, 0x005C6C43, 0x0063734A, 0x00596940, 0x005E6E45, 0x005E6E45, 0x005E6E45, 0x005E6E45, 0x005F6F46, 0x005F6F46, 0x005F6F46, 0x005F6F46, 0x005F6E48,
        0x005E6D47, 0x005E6D47, 0x005E6D47, 0x005E6D47, 0x005E6D47, 0x005E6D47, 0x005F6E48, 0x0064744B, 0x0058683F, 0x005D6D44, 0x00516138, 0x005F6F46, 0x0057673E, 0x006C7C53, 0x0075855C, 0x00748559, 0x00748559, 0x0075865A, 0x0075865A, 0x0075865A,
        0x00738458, 0x00728357, 0x00718256, 0x0074855A, 0x00708156, 0x007B8C61, 0x0073835A, 0x00697755, 0x00818C72, 0x00A5AE9A, 0x00B7BEB1, 0x008F947F, 0x00969B86, 0x008F947F, 0x00808570, 0x00828772, 0x00828772, 0x00959A85, 0x008F947F, 0x008C8F79,
        0x00757862, 0x00747761, 0x00777A64, 0x006D705A, 0x006D705A, 0x00757862, 0x006E715B, 0x00676E59, 0x00646B56, 0x007F8671, 0x00838A75, 0x00868D78, 0x00888F7A, 0x00787F6A, 0x00757C67, 0x0089877C, 0x00898B78, 0x005B6344, 0x00748258, 0x00798B5C,
        0x006C8051, 0x006F8457, 0x0071855A, 0x006E8057, 0x006E8057, 0x0076885F, 0x006F7F56, 0x0078885F, 0x0078865D, 0x0055633A, 0x005B6940, 0x0044532D, 0x00394822, 0x00596842, 0x004B5D36, 0x00657750, 0x0072845D, 0x0073865F, 0x0071845D, 0x00748560,
        0x0073845F, 0x0073855E, 0x0072845D, 0x0072845B, 0x0072845B, 0x0072845B, 0x0073865B, 0x0073855E, 0x0073855E, 0x0072845D, 0x0072845D, 0x0073855E, 0x0074865F, 0x00758760, 0x00758760, 0x00768861, 0x00758760, 0x0073855E, 0x0072845D, 0x0071835C,
        0x0071835C, 0x0071835C, 0x0072845D, 0x0073865F, 0x0073865F, 0x0072855E, 0x0072855E, 0x0071845D, 0x0071845D, 0x0070835C, 0x0070835C, 0x0070825D, 0x0070825D, 0x0070825D, 0x0070825D, 0x0070825D, 0x006F815C, 0x006F815C, 0x006F815C, 0x006C805D,
        0x0075835A, 0x00717B51, 0x004C6043, 0x00466A64, 0x00558999, 0x005B95B2, 0x00518DAB, 0x00477F92, 0x00274C54, 0x00273834, 0x005D645D, 0x008F9495, 0x00D2DEE4, 0x00BECFD2, 0x006F7F7E, 0x00506D72, 0x00547176, 0x00526F74, 0x00516E73, 0x00547176,
        0x00537075, 0x004F6C71, 0x00506D72, 0x00596C6F, 0x0053686A, 0x00556C6E, 0x00506769, 0x00526769, 0x006C7D80, 0x0097A2A6, 0x00C2C8CD, 0x00EAF3F0, 0x00A9B6A8, 0x006F7E62, 0x0073845F, 0x005F6D49, 0xFF99A18A, 0xFFFFFFF9, 0xFFFFFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xFFFFFFF7, 0x007F8A6A, 0x006E7C52, 0x007E8C63, 0x007F8C66, 0x007B8A5E, 0x007F905D, 0x00808E64, 0x0089976D, 0x00829066, 0x00818F65, 0x007A885E,
        0x00808E64, 0x00829066, 0x00808E64, 0x007A8C65, 0x00768762, 0x00778765, 0x007F8E6E, 0x006B7B59, 0x0063754E, 0x0064774C, 0x005F7346, 0x0061704A, 0x005A6943, 0x0066734D, 0x00727F59, 0x007A8761, 0x0075805A, 0x00818C66, 0x007B8660, 0x007A8B5F,
        0x0077885C, 0x00738458, 0x00758758, 0x00758758, 0x00748756, 0x00758857, 0x00748756, 0x00798A5F, 0x007C8D62, 0x005D6E43, 0x004E5F34, 0x00596A3F, 0x0057683D, 0x0075865B, 0x0078895E, 0x00738556, 0x00728455, 0x00728455, 0x0075865A, 0x00738458,
        0x0077885D, 0x0077885D, 0x00607146, 0x0054643B, 0x0056663D, 0x0053633A, 0x0055653C, 0x005A6A41, 0x0055653C, 0x004E5E35, 0x00526239, 0x00516138, 0x00516138, 0x00526239, 0x00526239, 0x00526239, 0x00526239, 0x0053633A, 0x0053633A, 0x0055643E,
        0x0054633D, 0x0053623C, 0x0053623C, 0x0053623C, 0x0053623C, 0x0054633D, 0x0055643E, 0x00506037, 0x0057673E, 0x004A5A31, 0x0056663D, 0x004F5F36, 0x006D7D54, 0x006E7E55, 0x0077875E, 0x0076875C, 0x0076875C, 0x0077885D, 0x0078895E, 0x0077885D,
        0x0075865B, 0x00738459, 0x00728358, 0x006F7F56, 0x007C8C63, 0x006E7F54, 0x00708057, 0x00495733, 0x006E7A5C, 0x00717B64, 0x0077806C, 0x005F694B, 0x00677153, 0x006D7759, 0x005B6547, 0x00576143, 0x00535D3F, 0x00586244, 0x00545E40, 0x00545C3E,
        0x00646C4E, 0x00444C2E, 0x00424A2C, 0x006C7456, 0x005B6345, 0x0061694B, 0x00687052, 0x00656F51, 0x00566042, 0x005A6446, 0x00525C3E, 0x00576143, 0x005B6547, 0x00586244, 0x00545E40, 0x00696F58, 0x00737A5F, 0x005F6B47, 0x00748258, 0x0077895A,
        0x00728657, 0x0074885B, 0x0075885D, 0x00778960, 0x0073855C, 0x0073855C, 0x007C8C63, 0x006D7D54, 0x005B6940, 0x0059673E, 0x004F5D34, 0x0033421C, 0x00586741, 0x0053623C, 0x005C6E47, 0x006E8059, 0x00778962, 0x006E815A, 0x0072855E, 0x00748560,
        0x00748560, 0x0073855E, 0x0072845D, 0x0072845B, 0x0072845B, 0x0073855C, 0x0073865B, 0x0072845D, 0x0072845D, 0x0072845D, 0x0073855E, 0x0074865F, 0x00758760, 0x00758760, 0x00768861, 0x00768861, 0x00758760, 0x0073855E, 0x0071835C, 0x0071835C,
        0x0071835C, 0x0072845D, 0x0073855E, 0x0073865F, 0x0073865F, 0x0072855E, 0x0072855E, 0x0072855E, 0x0071845D, 0x0071845D, 0x0071845D, 0x0071835E, 0x0071835E, 0x0071835E, 0x0071835E, 0x0071835E, 0x0070825D, 0x0070825D, 0x0070825D, 0x00687C59,
        0x0076845A, 0x006C764C, 0x003D5134, 0x003D605C, 0x005A8C9E, 0x0067A0BF, 0x00528DAD, 0x004B8396, 0x00274E56, 0x00394B4A, 0x007A8380, 0x00B2BBBE, 0x00DAE6EC, 0x00C1CED0, 0x00697773, 0x004C6267, 0x0052686D, 0x0051676C, 0x0051676C, 0x00556B70,
        0x00546A6F, 0x004F656A, 0x004E6469, 0x0056676A, 0x004E6164, 0x004F6466, 0x00465B5D, 0x0043585A, 0x00627376, 0x0094A0A4, 0x00C2CBCF, 0x00E2EBE8, 0x00ACB9AB, 0x006A795E, 0x0072835E, 0x0062704E, 0xFF979F88, 0xFFFFFFF9, 0xFFFDF9FE, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xFFFEFFF6, 0x007C8767, 0x006F7D53, 0x00808E65, 0x007A8761, 0x00617044, 0x0060713E, 0x0057653B, 0x005E6C42, 0x0056643A, 0x005C6A40, 0x0058663C,
        0x005D6B41, 0x005C6A40, 0x0067754B, 0x0077846C, 0x005C6852, 0x00545E4D, 0x00697363, 0x004B5545, 0x003A4531, 0x0037442C, 0x002D3A20, 0x002B371B, 0x00364224, 0x00414D2F, 0x00404B2B, 0x00475231, 0x0054603C, 0x0077825C, 0x0079845E, 0x00798A5F,
        0x0076875C, 0x0077885D, 0x0078895D, 0x0077885C, 0x00738556, 0x00728455, 0x00788A5B, 0x0077885D, 0x0075865B, 0x007C8D62, 0x0057683D, 0x00526338, 0x0056673C, 0x00596A3F, 0x0074855A, 0x007A8C5D, 0x00708253, 0x006D7F50, 0x0076875B, 0x00728357,
        0x006E7F54, 0x0078895E, 0x0077885D, 0x0053633A, 0x0055653C, 0x00516138, 0x0053633A, 0x005B6B42, 0x00596940, 0x0057673E, 0x005D6D44, 0x00596940, 0x00596940, 0x00596940, 0x005A6A41, 0x005A6A41, 0x005A6A41, 0x005A6A41, 0x005B6B42, 0x005D6C46,
        0x005C6B45, 0x005B6A44, 0x005B6A44, 0x005B6A44, 0x005B6A44, 0x005C6B45, 0x005D6C46, 0x0058683F, 0x0053633A, 0x005C6C43, 0x00516138, 0x006C7C53, 0x006C7C53, 0x00798960, 0x006F7F56, 0x0075865B, 0x0076875C, 0x0077885D, 0x0077885D, 0x0077885D,
        0x0075865B, 0x00738459, 0x00728358, 0x00798763, 0x0072815B, 0x00728259, 0x0057683D, 0x0054643B, 0x004C5A36, 0x00546042, 0x003F4A30, 0x0057633F, 0x005A6642, 0x0076825E, 0x006F7B57, 0x0077835F, 0x007B8763, 0x00737F5B, 0x0075815D, 0x0073815F,
        0x00768462, 0x00505E3C, 0x00566442, 0x00465432, 0x005A6846, 0x004D5B39, 0x00505E3C, 0x0052603C, 0x004A5834, 0x0054623E, 0x00586642, 0x006C7A56, 0x006F7D59, 0x0065734F, 0x004C5A36, 0x004F5A3A, 0x004D5B39, 0x0052613B, 0x005F7045, 0x00647549,
        0x006A7B4F, 0x00617548, 0x00697C51, 0x00687A53, 0x006E8059, 0x006B7D56, 0x006A7953, 0x005D6C46, 0x0056633D, 0x00515E38, 0x003B4822, 0x00596842, 0x00586741, 0x005B6A44, 0x00768861, 0x0073855E, 0x0071835C, 0x0073865F, 0x0071845D, 0x00748560,
        0x00748560, 0x0073855E, 0x0073855E, 0x0072845B, 0x0072845B, 0x0073855C, 0x0073865B, 0x0072845D, 0x0072845D, 0x0073855E, 0x0074865F, 0x0074865F, 0x00758760, 0x00758760, 0x00758760, 0x00768861, 0x00758760, 0x0073855E, 0x0072845D, 0x0071835C,
        0x0072845D, 0x0073855E, 0x0074865F, 0x0073865F, 0x0073865F, 0x0072855E, 0x0072855E, 0x0072855E, 0x0072855E, 0x0072855E, 0x0072855E, 0x0072845F, 0x0072845F, 0x0072845F, 0x0072845F, 0x0072845F, 0x0071835E, 0x0071835E, 0x0071835E, 0x006B7F5C,
        0x0078865C, 0x006C764B, 0x003F5132, 0x0044645F, 0x006290A1, 0x007CB1CC, 0x006FA3C1, 0x00588A96, 0x00375C60, 0x00596A67, 0x00949D9A, 0x00C6D1D5, 0x00DEECF2, 0x00C0CECD, 0x0067746C, 0x004E595D, 0x00576266, 0x00596468, 0x00586367, 0x005C676B,
        0x005C676B, 0x00566165, 0x00545F63, 0x00566365, 0x00505F61, 0x00506465, 0x00485D5E, 0x00425758, 0x005A6C6D, 0x008D9A9C, 0x00C0CCCE, 0x00E4ECEB, 0x00B8C4B8, 0x00707F64, 0x00788866, 0x00687654, 0xFF989F8A, 0xFFFFFFFB, 0xFFFBF7FC, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xFFFEFFF6, 0x007E8969, 0x00707E54, 0x00839168, 0x0073805A, 0x003E4D21, 0x003C4D1A, 0x0039451B, 0x00344016, 0x002A360C, 0x0038441A, 0x003B471D,
        0x003A461C, 0x002B370D, 0x00475329, 0x006A7463, 0x004D5649, 0x004B5349, 0x00929893, 0x00AEB3B1, 0x00B8BDBB, 0x00B7BDB8, 0x00B1B8B1, 0x00A5AEA1, 0x00B1BBAB, 0x00959E8A, 0x00535D46, 0x00434D2F, 0x005B6645, 0x00808D67, 0x007F8A62, 0x007A8963,
        0x00788761, 0x007B8A64, 0x007B8B62, 0x007A8A61, 0x0075855C, 0x0074855A, 0x00839469, 0x0078885F, 0x007B8B62, 0x007C8C63, 0x00798960, 0x0053633A, 0x005B6B42, 0x00506037, 0x0058683F, 0x00708253, 0x007D8F60, 0x00768859, 0x0077885C, 0x007B8C60,
        0x00718257, 0x00708156, 0x00798A5F, 0x00708057, 0x005F6F46, 0x0064744B, 0x0068784F, 0x005B6B42, 0x005D6D44, 0x0065754C, 0x005C6C43, 0x005D6E43, 0x005E6F44, 0x005E6F44, 0x005E6F44, 0x005E6F44, 0x005E6F44, 0x005F7045, 0x005F7045, 0x005E6D47,
        0x005E6D47, 0x005E6D47, 0x005D6C46, 0x005D6C46, 0x005E6D47, 0x005E6D47, 0x005E6D47, 0x00627249, 0x00617148, 0x00596940, 0x00718158, 0x006E7E55, 0x007B8B62, 0x00718158, 0x0075855C, 0x0078885F, 0x0078885F, 0x00798960, 0x00798960, 0x0078885F,
        0x0077875E, 0x0076865D, 0x0075855C, 0x007B8967, 0x006A7854, 0x004F5F36, 0x0056673C, 0x0048592E, 0x0053633A, 0x0053613D, 0x00546240, 0x0053623C, 0x005B6A44, 0x007D8C66, 0x0075845E, 0x0076855F, 0x0075845E, 0x00707F59, 0x00788761, 0x0073845F,
        0x00566742, 0x005A6B46, 0x0050613C, 0x0031421D, 0x00556641, 0x004D5E39, 0x0052633E, 0x0051633C, 0x004A5C35, 0x0053653E, 0x005D6F48, 0x007A8C65, 0x007A8C65, 0x006F815A, 0x004B5D36, 0x0052643D, 0x003D4F28, 0x00465831, 0x004F613A, 0x0053623C,
        0x0056653F, 0x00485731, 0x0055643E, 0x004E6039, 0x00556740, 0x005A6C45, 0x004B5A34, 0x0053623C, 0x0053603A, 0x003B4822, 0x0053603A, 0x005F6E48, 0x0052613B, 0x007C8B65, 0x0072845D, 0x007A8C65, 0x0071835C, 0x0070835C, 0x0072855E, 0x00758661,
        0x00748560, 0x0074865F, 0x0073855E, 0x0073855C, 0x0073855C, 0x0073855C, 0x0073865B, 0x0072845D, 0x0073855E, 0x0074865F, 0x00758760, 0x00758760, 0x00758760, 0x00758760, 0x00758760, 0x00758760, 0x0074865F, 0x0073855E, 0x0072845D, 0x0072845D,
        0x0073855E, 0x0073855E, 0x0074865F, 0x0072855E, 0x0072855E, 0x0072855E, 0x0072855E, 0x0072855E, 0x0072855E, 0x0072855E, 0x0072855E, 0x00748661, 0x00738560, 0x00738560, 0x00738560, 0x00738560, 0x0072845F, 0x0072845F, 0x0072845F, 0x006E8664,
        0x0078885F, 0x00768055, 0x00596848, 0x005D786E, 0x00688F98, 0x0088B3C6, 0x008BB6CB, 0x005A8385, 0x0044605A, 0x00627065, 0x00818883, 0x00ACB5B9, 0x00D9E6EE, 0x00C2D0CF, 0x006D7870, 0x00676C6F, 0x0073787B, 0x00767B7E, 0x00757A7D, 0x00787D80,
        0x00787D80, 0x0074797C, 0x0072777A, 0x00697577, 0x00606D6F, 0x00596B6C, 0x004E6263, 0x00475B5C, 0x00516364, 0x007E8D8F, 0x00BECACC, 0x00E5EDEC, 0x00BECABE, 0x0074826A, 0x00778765, 0x00687555, 0xFF99A08B, 0xFFFFFFFB, 0xFFFFFBFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xFFFFFFF7, 0x007D8868, 0x006A784E, 0x00839168, 0x007A8761, 0x003F4E22, 0x004D5E2B, 0x006A764C, 0x00616D43, 0x00505C32, 0x00556137, 0x005F6B41,
        0x00616D43, 0x003A461C, 0x00546036, 0x005F6D55, 0x00434F3B, 0x004F594D, 0x00B9C0BB, 0x00E7ECED, 0x00EBF0F3, 0x00E4E9EC, 0x00E2E7EA, 0x00EEF1F5, 0x00EEF2F3, 0x00AEB4AF, 0x00515849, 0x003C462F, 0x00545F3F, 0x006B7852, 0x005C6A40, 0x005F6C4C,
        0x00525F3F, 0x004E5C3A, 0x004E5C3A, 0x0054623E, 0x0053613D, 0x004E5C38, 0x005D6C46, 0x007B8B62, 0x0075855C, 0x00798960, 0x0074845B, 0x0076865D, 0x00596940, 0x0065754C, 0x004D5D34, 0x005A6C3D, 0x00758758, 0x00758758, 0x00728357, 0x007A8B5F,
        0x0077885D, 0x0074855A, 0x0074855A, 0x00718158, 0x00798960, 0x00718158, 0x00718158, 0x007F8F66, 0x007A8A61, 0x00728259, 0x007F8F66, 0x00798A5F, 0x007A8B60, 0x007A8B60, 0x007A8B60, 0x007A8B60, 0x007A8B60, 0x007B8C61, 0x007B8C61, 0x00798862,
        0x007A8963, 0x007A8963, 0x007A8963, 0x007A8963, 0x007A8963, 0x007A8963, 0x00798862, 0x007C8C63, 0x0075855C, 0x007B8B62, 0x0078885F, 0x00819168, 0x0074845B, 0x0078885F, 0x0077875E, 0x00798960, 0x00798960, 0x00798960, 0x00798960, 0x00798960,
        0x00798960, 0x0078885F, 0x0078885F, 0x00707D5D, 0x005A6844, 0x0054643B, 0x004F6034, 0x005B6C40, 0x004A5B30, 0x005B6B42, 0x00576640, 0x00566841, 0x00687A53, 0x007B8D66, 0x0074865F, 0x00758760, 0x0070825B, 0x00778962, 0x0074865F, 0x0052643F,
        0x00546641, 0x00455732, 0x002F411C, 0x0050623D, 0x004D5F3A, 0x00364823, 0x00576944, 0x004E613A, 0x00475A33, 0x004F623B, 0x005C6F48, 0x00778A63, 0x0070835C, 0x006F825B, 0x0051643D, 0x0052663C, 0x0040532C, 0x00495B36, 0x0051613F, 0x0051613F,
        0x00556341, 0x004F5D3B, 0x00566442, 0x0053643F, 0x004E5F3A, 0x00485934, 0x005C6A46, 0x004D5B37, 0x003F4B27, 0x00586440, 0x0057633F, 0x004E5D37, 0x007D8C66, 0x006E7D57, 0x007B8D66, 0x0071835C, 0x0074865F, 0x0071845D, 0x0073865F, 0x00758661,
        0x00748560, 0x0074865F, 0x0073855E, 0x0073855C, 0x0073855C, 0x0073855C, 0x0074875C, 0x0073855E, 0x0074865F, 0x00758760, 0x00758760, 0x00758760, 0x00758760, 0x0074865F, 0x0074865F, 0x0074865F, 0x0074865F, 0x0074865F, 0x0073855E, 0x0073855E,
        0x0073855E, 0x0073855E, 0x0073855E, 0x0072855E, 0x0072855E, 0x0072855E, 0x0073865F, 0x0073865F, 0x0073865F, 0x0073865F, 0x0073865F, 0x00748661, 0x00748661, 0x00748661, 0x00748661, 0x00738560, 0x00738560, 0x00738560, 0x00738560, 0x00688362,
        0x0074865F, 0x007C865B, 0x006C7854, 0x006D8273, 0x00628081, 0x007899A2, 0x007997A2, 0x004F6F64, 0x00435645, 0x005F6756, 0x006D7067, 0x008E979A, 0x00D3E0E8, 0x00BDCACC, 0x0069766E, 0x00778181, 0x00858F8F, 0x008B9595, 0x00889292, 0x008A9494,
        0x008B9595, 0x00889292, 0x00879191, 0x00868D90, 0x006D797B, 0x00576668, 0x00495D5E, 0x0045595A, 0x00465859, 0x00707F81, 0x00BCC9CB, 0x00E1E9E8, 0x00BEC9BF, 0x0075836B, 0x00738361, 0x00667353, 0xFF99A08B, 0xFFFFFFFC, 0xFFFFFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xFFFDFFF5, 0x007E8969, 0x0069774D, 0x00849269, 0x007B8862, 0x0039481C, 0x0051622F, 0x00647046, 0x00636F45, 0x00515D33, 0x004B572D, 0x005F6B41,
        0x00717D53, 0x003D491F, 0x004A562C, 0x006B7C5B, 0x0047563A, 0x005F6B59, 0x00D2DBD1, 0x00E0E6E5, 0x00C6CBCE, 0x00BDC2C5, 0x00C2C6CB, 0x00D0CFDF, 0x00E6E7F1, 0x00BBBFC0, 0x0060655C, 0x00363F2A, 0x00344121, 0x004D5B32, 0x00526135, 0x00616D51,
        0x00465234, 0x00364224, 0x00364224, 0x00465333, 0x004A5737, 0x003F4D2B, 0x00485634, 0x0065754C, 0x005B6B42, 0x005E6E45, 0x00617148, 0x00697950, 0x0073835A, 0x0053633A, 0x005C6C43, 0x00556738, 0x00586A3B, 0x005A6C3D, 0x00627347, 0x005E6F43,
        0x0058693E, 0x00607146, 0x00637449, 0x00617148, 0x005B6B42, 0x00617148, 0x0068784F, 0x0065754C, 0x0065754C, 0x0065754C, 0x00617148, 0x0064754A, 0x0064754A, 0x0064754A, 0x0064754A, 0x0065764B, 0x0065764B, 0x0065764B, 0x0065764B, 0x0065744E,
        0x0066754F, 0x00677650, 0x00677650, 0x00677650, 0x00677650, 0x0066754F, 0x0065744E, 0x005E6E45, 0x0066764D, 0x0067774E, 0x0064744B, 0x005E6E45, 0x0065754C, 0x0067774E, 0x00697950, 0x00697950, 0x00697950, 0x00697950, 0x00697950, 0x00697950,
        0x006A7A51, 0x006A7A51, 0x006A7A51, 0x00576444, 0x00546240, 0x00516138, 0x0055663A, 0x00566839, 0x004F6034, 0x0058693E, 0x0053633A, 0x006F815A, 0x007D8F68, 0x0071835C, 0x006F815A, 0x007A8C65, 0x0073855E, 0x00788A63, 0x00566841, 0x004F643E,
        0x00536842, 0x00293E18, 0x00536842, 0x00536842, 0x00556A44, 0x004B603A, 0x004A5F39, 0x0053663F, 0x00495C35, 0x004F623B, 0x005C6F48, 0x00758861, 0x006D8059, 0x007B8E67, 0x006B7E57, 0x004A6036, 0x004D623B, 0x00576846, 0x005A6949, 0x00546344,
        0x005A6648, 0x005F6C4C, 0x005B6646, 0x00566742, 0x00596A45, 0x00576843, 0x0053613D, 0x00616F4B, 0x0065714D, 0x0055613D, 0x005B6743, 0x00778660, 0x007A8963, 0x007C8B65, 0x00697B54, 0x00758760, 0x0074865F, 0x006F825B, 0x00788B64, 0x00758661,
        0x00758661, 0x0074865F, 0x0073855E, 0x0073855C, 0x0073855C, 0x0073855C, 0x0074875C, 0x0074865F, 0x0074865F, 0x00758760, 0x00768861, 0x00758760, 0x00758760, 0x0074865F, 0x0073855E, 0x0073855E, 0x0074865F, 0x0074865F, 0x0074865F, 0x0074865F,
        0x0074865F, 0x0073855E, 0x0073855E, 0x0072855E, 0x0072855E, 0x0072855E, 0x0073865F, 0x0073865F, 0x0073865F, 0x0073865F, 0x0073865F, 0x00758762, 0x00758762, 0x00748661, 0x00748661, 0x00748661, 0x00748661, 0x00738560, 0x00738560, 0x00678365,
        0x00788B64, 0x00848E63, 0x0075805A, 0x00748774, 0x00637C78, 0x006A8387, 0x00586F71, 0x0048614D, 0x00414F37, 0x00686B55, 0x0075776B, 0x00929A9A, 0x00D7E4EC, 0x00B0BFC1, 0x005C6862, 0x005E706F, 0x006E807F, 0x00748685, 0x00718382, 0x00728483,
        0x00738584, 0x00718382, 0x00718382, 0x00777F7F, 0x00626C6C, 0x0052605F, 0x00516362, 0x00516664, 0x00465857, 0x005E6E6D, 0x00A6B4B3, 0x00E4ECEC, 0x00C3CEC4, 0x007C8A72, 0x00768565, 0x00687555, 0xFF9AA08D, 0xFFFFFFFC, 0xFFFFFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xF7FBFFF1, 0x007D8965, 0x0069784C, 0x00818F65, 0x0077855C, 0x003A4A1B, 0x00576934, 0x00505C32, 0x004D592F, 0x00647145, 0x00344115, 0x00697648,
        0x00546133, 0x00364315, 0x004E5C2C, 0x006C7B5F, 0x00505E46, 0x006F7A6A, 0x00CAD2CB, 0x00DBE2DF, 0x00D3D9D8, 0x00BBC1C0, 0x00C0C6C5, 0x00CCD1D4, 0x00DDE2E5, 0x00C6CBCC, 0x00747B78, 0x007D847F, 0x00616A60, 0x00586256, 0x007C8679, 0x00A2A4A4,
        0x006E716F, 0x005E625D, 0x0060645E, 0x005E6459, 0x009BA293, 0x00727A69, 0x00535B4A, 0x0056663D, 0x005E6E45, 0x0053633A, 0x005B6B42, 0x0054643B, 0x00607047, 0x006E7E55, 0x004A5A31, 0x0057653C, 0x005C6A41, 0x00616F46, 0x00606E45, 0x005C6A41,
        0x0059673E, 0x005A683F, 0x005D6B42, 0x005A6844, 0x00596743, 0x00586642, 0x005A6844, 0x005C6A46, 0x005D6B47, 0x005B6945, 0x00596743, 0x005C673F, 0x005D6840, 0x005F6A42, 0x00606B43, 0x00616C44, 0x00606B43, 0x005F6A42, 0x005E6941, 0x005C6B45,
        0x005C6B45, 0x005C6B45, 0x005C6B45, 0x005C6B45, 0x005C6B45, 0x005C6B45, 0x005C6B45, 0x00546239, 0x0057653C, 0x005B6940, 0x005C6943, 0x005D6A44, 0x005D6945, 0x005E6A46, 0x00606C48, 0x005F6B41, 0x00646F47, 0x00616C44, 0x005F6A42, 0x00636E48,
        0x00616C46, 0x005F6945, 0x00646E4A, 0x00707A56, 0x00525E3A, 0x005D6945, 0x00596541, 0x004E5C38, 0x005D6B47, 0x004F603B, 0x00758661, 0x00768962, 0x00768962, 0x006D8059, 0x0073865F, 0x006F825B, 0x00748760, 0x005B6E47, 0x004D6039, 0x005F714A,
        0x002D3E19, 0x00576646, 0x005C6B4C, 0x00536243, 0x00758464, 0x0051623D, 0x0052643D, 0x00596A3E, 0x004A5B30, 0x005A6943, 0x005A6846, 0x007E8C6A, 0x0072805C, 0x006E7D57, 0x007E8E65, 0x00717F5B, 0x0055633F, 0x0055633F, 0x0053613D, 0x0052603C,
        0x00576541, 0x00505E3A, 0x00566440, 0x0051623D, 0x00586944, 0x004F603B, 0x0051623D, 0x005D6E49, 0x00546540, 0x00576843, 0x00798A65, 0x00758760, 0x0074865F, 0x0074865F, 0x0074865F, 0x0074865F, 0x0073855E, 0x0073855E, 0x0073855E, 0x00748760,
        0x00748760, 0x00748760, 0x00748760, 0x0073865F, 0x0073865F, 0x0073865F, 0x0073865F, 0x0071845D, 0x0071845D, 0x0071845D, 0x0071845D, 0x0071845D, 0x0071845D, 0x0071845D, 0x0071845D, 0x0073865F, 0x0073865F, 0x0073865F, 0x0073865F, 0x0073865F,
        0x0073865F, 0x0073865F, 0x0073865F, 0x0072855A, 0x0076885F, 0x0071835A, 0x0075875E, 0x0074865F, 0x0071835C, 0x00788964, 0x00758661, 0x00768762, 0x00768762, 0x0073845F, 0x0073845F, 0x00778863, 0x0073845F, 0x006F805B, 0x00788964, 0x00758760,
        0x0074865F, 0x00687954, 0x006B7C57, 0x006E7E5C, 0x006E7E5C, 0x00677755, 0x004A5939, 0x00565B3E, 0x00494F38, 0x0060665B, 0x00686E6D, 0x008E999D, 0x00D3E1E7, 0x00ACBFC2, 0x0094A8A9, 0x0097A1AB, 0x009AA4AE, 0x009BA6AE, 0x009CA7AF, 0x00A4AFB7,
        0x00A7B2BA, 0x00A3AEB6, 0x00A6B1B9, 0x00A0AAB4, 0x00B4BFC7, 0x00A6B2B6, 0x00A5B2B4, 0x00ABB8BA, 0x00A4B1B3, 0x00AFBBC1, 0x00B8C4CA, 0x00E4E7EF, 0x00C9D0CB, 0x007B8771, 0x00778863, 0x00667752, 0xFF97A288, 0xFFFDFFF9, 0xFFFAFCFD, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xFBFDFFF3, 0x007E8968, 0x0069784C, 0x00808E64, 0x0076845B, 0x003A4A1B, 0x004E602B, 0x00636F45, 0x005E6A40, 0x005C683E, 0x003B481C, 0x00637044,
        0x00475426, 0x00384517, 0x00515E30, 0x006B7862, 0x00515C4C, 0x007A8379, 0x00D8DFDC, 0x00D9DEE1, 0x00CDD1D6, 0x00CACED3, 0x00D5D9DE, 0x00D7DADE, 0x00D4D8D9, 0x00BEC3C1, 0x009AA19A, 0x00969E93, 0x004D5747, 0x0057624E, 0x0095A18B, 0x00ACB1A8,
        0x007E8479, 0x00585F52, 0x00636A5B, 0x00848D79, 0x00ABB49F, 0x00646E57, 0x00454F38, 0x0056663D, 0x00495930, 0x00607047, 0x00516138, 0x00506037, 0x0056663D, 0x0056663D, 0x00495930, 0x004F5D34, 0x00536138, 0x0057653C, 0x0058663D, 0x0056643B,
        0x00546239, 0x0056643B, 0x0058663D, 0x00566440, 0x0055633F, 0x0055633F, 0x00566440, 0x00586642, 0x00586642, 0x00576541, 0x00566440, 0x00566139, 0x0057623A, 0x0059643C, 0x005A653D, 0x005A653D, 0x0059643C, 0x0058633B, 0x0057623A, 0x0053623C,
        0x0053623C, 0x0053623C, 0x0052613B, 0x0052613B, 0x0051603A, 0x0051603A, 0x0051603A, 0x00526037, 0x00546239, 0x0058663D, 0x00596640, 0x00596640, 0x00596640, 0x005A6642, 0x005B6743, 0x005E6A40, 0x005F6A42, 0x005E6941, 0x005E6941, 0x005D6842,
        0x005D6842, 0x005D6743, 0x005E6844, 0x004A5731, 0x0066734D, 0x005D6A44, 0x0048552F, 0x005F6E48, 0x0052613B, 0x006E8059, 0x00798B64, 0x006E8057, 0x0072845B, 0x007D8F66, 0x006F8158, 0x007A8C63, 0x005B6D44, 0x005A6C43, 0x00586A41, 0x0036481F,
        0x00566841, 0x00556543, 0x00556444, 0x007D8C6C, 0x0081916F, 0x00495B34, 0x0056683F, 0x004F6336, 0x0056693E, 0x004B5D34, 0x006A7C53, 0x00677952, 0x00778962, 0x0084966F, 0x00697B54, 0x00758760, 0x00788A63, 0x007B8D66, 0x007A8C65, 0x0074865F,
        0x00758760, 0x00788A63, 0x0073855E, 0x0073845F, 0x00798A65, 0x007B8C67, 0x00748560, 0x0071825D, 0x00778863, 0x007A8B66, 0x00778863, 0x0072845D, 0x0072845D, 0x0073855E, 0x0073855E, 0x0074865F, 0x00758760, 0x00768861, 0x00768861, 0x00748760,
        0x00748760, 0x00748760, 0x0073865F, 0x0073865F, 0x0073865F, 0x0073865F, 0x0072855E, 0x0072855E, 0x0072855E, 0x0072855E, 0x0072855E, 0x0072855E, 0x0072855E, 0x0072855E, 0x0072855E, 0x0073865F, 0x0073865F, 0x0073865F, 0x0073865F, 0x0073865F,
        0x0073865F, 0x0073865F, 0x0073865F, 0x0072855A, 0x0075875E, 0x0071835A, 0x0075875E, 0x00768861, 0x0073855E, 0x00788964, 0x00768762, 0x0073845F, 0x00758661, 0x0073845F, 0x00778863, 0x00768762, 0x00748560, 0x00798A65, 0x00768762, 0x0064764F,
        0x00556740, 0x0053643F, 0x00647550, 0x00556543, 0x005B6B49, 0x005A6A48, 0x00617050, 0x00545D3C, 0x004D563B, 0x00676F5E, 0x00737C72, 0x00959F99, 0x00D8E6E2, 0x00DAEAE3, 0x00D9EAE1, 0x00DDE8E6, 0x00E0EBE9, 0x00E0EBE9, 0x00E0EBE9, 0x00E1ECEA,
        0x00DFEAE8, 0x00E1ECEA, 0x00EBF6F4, 0x00E6EFF2, 0x00DDE7E7, 0x00E2EDEB, 0x00E6F1EE, 0x00E2EEE8, 0x00E2EEE8, 0x00E1ECE9, 0x00D9E4E1, 0x00E2E8E7, 0x00CAD4C8, 0x00859377, 0x007C8E67, 0x0064764F, 0xFF949F85, 0xFFFDFFF9, 0xFFF9FAFE, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFCFF, 0xFFFDFFF5, 0x00808B6A, 0x006D7B51, 0x0086946B, 0x007B8960, 0x00415024, 0x00495B26, 0x00778359, 0x0069754B, 0x0048542A, 0x00404D21, 0x00616E42,
        0x00455226, 0x003A4719, 0x00485527, 0x00687261, 0x004F584B, 0x006C736C, 0x00D1D6D5, 0x00E6E9ED, 0x00D7DADF, 0x00D0D3D8, 0x00D6D9DD, 0x00DCDCDC, 0x00F1F2EE, 0x00B8BBB2, 0x006C7162, 0x006B725D, 0x00586146, 0x00677252, 0x00545F3E, 0x005D6750,
        0x00646E57, 0x00566147, 0x00576347, 0x00566244, 0x00596646, 0x003B4927, 0x0053613F, 0x0075855C, 0x0077875E, 0x006E7E55, 0x007A8A61, 0x0058683F, 0x0058683F, 0x005A6A41, 0x0056663D, 0x0058663D, 0x005B6940, 0x005D6B42, 0x005E6C43, 0x005E6C43,
        0x005D6B42, 0x005E6C43, 0x005F6D44, 0x005D6C46, 0x005D6C46, 0x005D6C46, 0x005E6D47, 0x005F6E48, 0x005F6E48, 0x005E6D47, 0x005D6C46, 0x00636E46, 0x00636E46, 0x00646F47, 0x00657048, 0x00657048, 0x00646F47, 0x00626D45, 0x00626D45, 0x005E6E45,
        0x005E6E45, 0x005D6D44, 0x005C6C43, 0x005B6B42, 0x005B6B42, 0x005A6A41, 0x005A6A41, 0x005B693F, 0x005D6B41, 0x005F6D44, 0x00616F46, 0x00616E48, 0x00606D47, 0x00616E48, 0x00616D49, 0x00657147, 0x00636E46, 0x00657048, 0x0067724A, 0x0066714B,
        0x0068734D, 0x006A7450, 0x0068724E, 0x006B7950, 0x005B6940, 0x004F5D34, 0x00697950, 0x005E6E45, 0x00708259, 0x0075875E, 0x0075875E, 0x007B895F, 0x0077855B, 0x0079875D, 0x00839167, 0x00647248, 0x005A683E, 0x005D6B41, 0x0039471D, 0x005B6E43,
        0x0052643B, 0x005F704B, 0x007D8D6B, 0x007C8C6A, 0x0070815C, 0x00596B42, 0x0055683D, 0x005D6F46, 0x00516439, 0x00506437, 0x0055693C, 0x007A8E61, 0x0072845B, 0x006D7F58, 0x007A8B66, 0x006F8158, 0x00788A61, 0x006F8158, 0x0072845B, 0x00708259,
        0x00708259, 0x007D8F66, 0x0071835A, 0x0072835E, 0x006F805B, 0x0073845F, 0x00778863, 0x00758661, 0x00798A65, 0x007B8C67, 0x00748560, 0x0072845D, 0x0072845D, 0x0073855E, 0x0074865F, 0x00758760, 0x00758760, 0x00768861, 0x00768861, 0x00748760,
        0x0073865F, 0x0073865F, 0x0073865F, 0x0073865F, 0x0072855E, 0x0072855E, 0x0072855E, 0x0072855E, 0x0072855E, 0x0072855E, 0x0072855E, 0x0072855E, 0x0072855E, 0x0072855E, 0x0072855E, 0x0073865F, 0x0073865F, 0x0073865F, 0x0073865F, 0x0073865F,
        0x0073865F, 0x0073865F, 0x0073865F, 0x0072855A, 0x0075875E, 0x0072845B, 0x0075875E, 0x00758760, 0x0073855E, 0x00768762, 0x0073845F, 0x0072835E, 0x00768762, 0x00748560, 0x00798A65, 0x00748560, 0x0073845F, 0x00788964, 0x00687954, 0x005B6D46,
        0x004E6039, 0x005B6C47, 0x00576843, 0x0051613F, 0x005A6A48, 0x00536341, 0x004F5E3E, 0x005E6D47, 0x005A6846, 0x006D795D, 0x0078846E, 0x008D9884, 0x00AFB9A8, 0x00AEBAA6, 0x00A1AD99, 0x00A6B29E, 0x00A5B19D, 0x00A0AC98, 0x00A1AD99, 0x00A3AF9D,
        0x009CA896, 0x0097A391, 0x0099A593, 0x0099A397, 0x008B9589, 0x00929C90, 0x00899386, 0x008A9585, 0x00A3AF9D, 0x00A1AD9B, 0x009AA692, 0x009EA999, 0x0096A48C, 0x00768762, 0x00788B60, 0x00657750, 0xFF97A288, 0xFFFCFFFA, 0xFFFCFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFCFF, 0xFFFEFFF6, 0x00808B6A, 0x006B794F, 0x00818F66, 0x00727F59, 0x003B4A1E, 0x00384916, 0x0037451C, 0x003C4A21, 0x002B390F, 0x00334117, 0x003F4E22,
        0x00303F13, 0x00344317, 0x004D5D2E, 0x0068765A, 0x005A674F, 0x005C6655, 0x00B5BDB2, 0x00E9F0E9, 0x00E0E7E0, 0x00D3DBD1, 0x00DAE2D8, 0x00ECEDE4, 0x00EAEBE1, 0x00B3B6A6, 0x00797F68, 0x0071795B, 0x005D6842, 0x007C885E, 0x00788557, 0x00717F5B,
        0x00768460, 0x0071805A, 0x00788761, 0x006C7C53, 0x006D7E53, 0x006E7F54, 0x00809166, 0x0077875E, 0x0077875E, 0x007C8C63, 0x00728259, 0x0068784F, 0x0045552C, 0x00627249, 0x0055653C, 0x005E6C42, 0x005E6C42, 0x005E6C42, 0x005E6C42, 0x005E6C42,
        0x005D6B41, 0x005C6A40, 0x005B693F, 0x005B6B42, 0x005B6B42, 0x005B6B42, 0x005C6C43, 0x005C6C43, 0x005C6C43, 0x005B6B42, 0x005B6B42, 0x0059673D, 0x005A683E, 0x005A683E, 0x005A683E, 0x005A683E, 0x0059673D, 0x0058663C, 0x0057653B, 0x0057673E,
        0x0057673E, 0x0057673E, 0x0056663D, 0x0056663D, 0x0055653C, 0x0055653C, 0x0055653C, 0x0056673C, 0x0057683D, 0x0058693E, 0x005A6A41, 0x005A6A41, 0x00596842, 0x00596842, 0x00586741, 0x005B693F, 0x005D6B42, 0x005B6940, 0x005A683F, 0x005C6943,
        0x005B6842, 0x00596541, 0x00596541, 0x00637148, 0x005C6A41, 0x005A6A41, 0x004F5F36, 0x006D7F56, 0x0082946B, 0x007A8E64, 0x0074885E, 0x007C875F, 0x007D8860, 0x007D8860, 0x006C774F, 0x005E6941, 0x0069744C, 0x003B461E, 0x00515C34, 0x005D7144,
        0x005F7247, 0x00758760, 0x00788964, 0x0072835E, 0x007C8E67, 0x005E7146, 0x00576B3E, 0x00556740, 0x00405328, 0x005D7144, 0x004A5E2F, 0x00677B4E, 0x00778A5F, 0x0081936C, 0x00738361, 0x0074865D, 0x00798B62, 0x0075875E, 0x007C8E65, 0x00798B62,
        0x00708259, 0x007B8D64, 0x007A8C63, 0x007C8D68, 0x00788964, 0x0072835E, 0x00768762, 0x007B8C67, 0x00748560, 0x006F805B, 0x00778863, 0x00758760, 0x00758760, 0x00758760, 0x0074865F, 0x0074865F, 0x0074865F, 0x0074865F, 0x0074865F, 0x0073865F,
        0x0073865F, 0x0073865F, 0x0073865F, 0x0073865F, 0x0072855E, 0x0072855E, 0x0072855E, 0x0072855E, 0x0072855E, 0x0072855E, 0x0072855E, 0x0072855E, 0x0072855E, 0x0072855E, 0x0072855E, 0x0073865F, 0x0073865F, 0x0073865F, 0x0073865F, 0x0073865F,
        0x0073865F, 0x0073865F, 0x0073865F, 0x0074875C, 0x0075875E, 0x0074865D, 0x0075875E, 0x0074865F, 0x0072845D, 0x0073845F, 0x0072835E, 0x00768762, 0x00778863, 0x00788964, 0x00748560, 0x0073845F, 0x0073845F, 0x00667752, 0x00556641, 0x00556740,
        0x00556740, 0x005E6F4A, 0x0040512C, 0x00546442, 0x005F6F4D, 0x00697957, 0x005B6A4A, 0x00506239, 0x00667851, 0x007C8D68, 0x007F8C6C, 0x00818D6F, 0x00818A6F, 0x0080896E, 0x007A8368, 0x0073845F, 0x00788866, 0x0071815F, 0x006C7B5B, 0x00667556,
        0x00556445, 0x00475539, 0x003E4C30, 0x0037422E, 0x00545F4B, 0x005D6854, 0x00495440, 0x005D6953, 0x007B886E, 0x00718061, 0x006D7D5B, 0x00718064, 0x00768765, 0x0074875C, 0x007C9063, 0x006A7C55, 0xFF9BA58E, 0xFFFCFFFA, 0xFFFDFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xFFFFFFF8, 0x007F8A6A, 0x006A784F, 0x00818E68, 0x0076835D, 0x00505E34, 0x004C5D2A, 0x0048562D, 0x004F5D34, 0x004C5A31, 0x004E5C32, 0x0049572D,
        0x0048572B, 0x00455428, 0x005C6B3F, 0x006B7E53, 0x00677952, 0x00566644, 0x007F8D71, 0x0099A68C, 0x008D9A80, 0x008A987C, 0x0096A586, 0x00878A7A, 0x00989C89, 0x008B9178, 0x005B6345, 0x0056613B, 0x005F6C40, 0x007F8D5D, 0x007A8956, 0x007F9064,
        0x0078895D, 0x00708155, 0x007C8D61, 0x007A8C5D, 0x007D8F60, 0x007D8F60, 0x00728455, 0x00798960, 0x0074845B, 0x007A8A61, 0x0074845B, 0x00506037, 0x00617148, 0x00516138, 0x00617148, 0x005B693F, 0x005A683E, 0x005A683E, 0x005A683E, 0x005A683E,
        0x0059673D, 0x0058663C, 0x0057653B, 0x0054653A, 0x0055663B, 0x0056673C, 0x0056673C, 0x0055663B, 0x0055663B, 0x0054653A, 0x0054653A, 0x0059673D, 0x0059673D, 0x0059673D, 0x0059673D, 0x0059673D, 0x0058663C, 0x0057653B, 0x0057653B, 0x0056663D,
        0x0056663D, 0x0056663D, 0x0057673E, 0x0057673E, 0x0057673E, 0x0058683F, 0x0058683F, 0x0056673B, 0x0057683C, 0x0058693E, 0x00596A3F, 0x005A6A41, 0x00596940, 0x00586741, 0x00576640, 0x00556339, 0x005D6B42, 0x0059673E, 0x0058663D, 0x00616E48,
        0x005F6C46, 0x005A6642, 0x00616D49, 0x0053623C, 0x00586741, 0x005E6D47, 0x006D7F58, 0x0074865F, 0x0053663F, 0x00596C45, 0x0063764F, 0x005E6B45, 0x0064714B, 0x0065724C, 0x00525F39, 0x005A6741, 0x003E4B25, 0x00505D37, 0x006D7A54, 0x00566A3D,
        0x0073855C, 0x007A8C65, 0x00748462, 0x007A8A68, 0x00778962, 0x004C5E35, 0x00516538, 0x0051643D, 0x00485C32, 0x00384C22, 0x00586C42, 0x0054683E, 0x0050633C, 0x005B6D48, 0x00596A48, 0x005B6E47, 0x00576A43, 0x005B6E47, 0x006B7E57, 0x00798C65,
        0x00798C65, 0x0070835C, 0x006B7E57, 0x00748560, 0x00788964, 0x00748560, 0x0070815C, 0x00748560, 0x0073845F, 0x0070815C, 0x0072835E, 0x00758760, 0x00758760, 0x00758760, 0x00758760, 0x00758760, 0x0074865F, 0x0074865F, 0x0074865F, 0x00748760,
        0x00748760, 0x0073865F, 0x0073865F, 0x0073865F, 0x0073865F, 0x0072855E, 0x0072855E, 0x0073865F, 0x0073865F, 0x0073865F, 0x0073865F, 0x0073865F, 0x0073865F, 0x0073865F, 0x0073865F, 0x0073865F, 0x0073865F, 0x0073865F, 0x0073865F, 0x0073865F,
        0x0073865F, 0x0073865F, 0x0073865F, 0x0074875C, 0x0075875E, 0x00778960, 0x0076885F, 0x0074865F, 0x0074865F, 0x00748560, 0x00768762, 0x00758661, 0x00758661, 0x007A8B66, 0x006F805B, 0x00788964, 0x00768762, 0x0053643F, 0x004F603B, 0x0053653E,
        0x006C7E57, 0x005E6F4A, 0x0053643F, 0x00536341, 0x00576745, 0x00546442, 0x00526141, 0x0064784E, 0x007A8D66, 0x0081936C, 0x006A7B56, 0x006F7C5C, 0x00626E52, 0x00636D56, 0x0069705B, 0x005A6A41, 0x0068784F, 0x005E6D47, 0x004F5D39, 0x00445230,
        0x00424F2F, 0x004E5A3C, 0x00535F43, 0x003A442D, 0x00636C57, 0x005D6652, 0x0048503F, 0x00606954, 0x00798569, 0x00788662, 0x007C8C63, 0x00758664, 0x00788B64, 0x007C8F64, 0x007C8F64, 0x00667752, 0xFF97A08B, 0xFFFCFFFA, 0xFFFDFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xFFFFFFF8, 0x007E8969, 0x006C7A51, 0x0087946E, 0x0084906C, 0x0077855B, 0x00788858, 0x007F8E68, 0x00788761, 0x0077875E, 0x0075855C, 0x0074855A,
        0x007F9065, 0x0075865B, 0x007B8C60, 0x00748754, 0x00738655, 0x006E7F54, 0x007B8A64, 0x0074835D, 0x006E7D57, 0x00798960, 0x007E8F64, 0x00767F64, 0x00717B5D, 0x00788363, 0x006B7753, 0x00728057, 0x00748357, 0x007E8E5F, 0x007C8C5C, 0x00738556,
        0x007D8F60, 0x007A8C5D, 0x00788A5B, 0x00758758, 0x006F8054, 0x0075865A, 0x0075865A, 0x00819168, 0x006E7E55, 0x00708057, 0x00506037, 0x00526239, 0x0055653C, 0x0067774E, 0x00697950, 0x00748357, 0x00748357, 0x00748357, 0x00748357, 0x00758458,
        0x00768559, 0x00758458, 0x00758458, 0x00718256, 0x00728357, 0x00738458, 0x00738458, 0x00728357, 0x00718256, 0x00718256, 0x00718256, 0x006F8054, 0x006F8054, 0x006F8054, 0x006E7F53, 0x006E7F53, 0x006E7F53, 0x006E7F53, 0x006E7F53, 0x006B7C51,
        0x006C7D52, 0x006C7D52, 0x006D7E53, 0x006E7F54, 0x006F8055, 0x006F8055, 0x00708156, 0x006A7E51, 0x006A7E51, 0x006B7F52, 0x006D8055, 0x006E8156, 0x006E8057, 0x006C7E55, 0x006B7D54, 0x0067784D, 0x006F7F56, 0x006D7D54, 0x006B7B52, 0x00707F59,
        0x006D7C56, 0x00697753, 0x006F7D59, 0x006C7A58, 0x00717F5D, 0x0070805E, 0x006C7C5A, 0x00435432, 0x005E6F4D, 0x005E6F4D, 0x004F6340, 0x00627151, 0x00586747, 0x005A6949, 0x005A6949, 0x00435232, 0x00404F2F, 0x005C6B4B, 0x00475636, 0x007A8C63,
        0x007E9069, 0x0071815F, 0x00819070, 0x00718060, 0x00596947, 0x00556740, 0x0056683F, 0x0043572D, 0x005A6D46, 0x00485938, 0x0037462A, 0x00536246, 0x00607051, 0x005E6F4D, 0x005B6D48, 0x00667755, 0x005B6C4A, 0x0050613F, 0x00516240, 0x00657654,
        0x00778866, 0x00798A68, 0x00798A68, 0x00798A65, 0x0070815C, 0x00748560, 0x007C8D68, 0x00778863, 0x0072835E, 0x0071825D, 0x006F805B, 0x0073855E, 0x0073855E, 0x0074865F, 0x0074865F, 0x00758760, 0x00768861, 0x00778962, 0x00778962, 0x00748760,
        0x00748760, 0x00748760, 0x00748760, 0x0073865F, 0x0073865F, 0x0073865F, 0x0073865F, 0x00748760, 0x00748760, 0x00748760, 0x00748760, 0x00748760, 0x00748760, 0x00748760, 0x00748760, 0x0073865F, 0x0073865F, 0x0073865F, 0x0073865F, 0x0073865F,
        0x0073865F, 0x0073865F, 0x0073865F, 0x0074875C, 0x0074865D, 0x00798B62, 0x0075875E, 0x0071835C, 0x0072845D, 0x0072835E, 0x00788964, 0x0072835E, 0x0073845F, 0x00788964, 0x0070815C, 0x00798A65, 0x006F805B, 0x004B5C37, 0x00546540, 0x0061734C,
        0x0083956E, 0x0053643F, 0x0062734E, 0x004D5D3B, 0x00798967, 0x00748462, 0x00849373, 0x007B8C67, 0x007B8D68, 0x007D8E6C, 0x005D6E4D, 0x006F7E63, 0x00677563, 0x00606B61, 0x00626C66, 0x0052633E, 0x0060704E, 0x00526141, 0x00475539, 0x00435036,
        0x0045513B, 0x0055604C, 0x0058634F, 0x00596352, 0x00636D5D, 0x005D655A, 0x00586056, 0x00646D60, 0x00707D65, 0x00758464, 0x0075875E, 0x00758664, 0x00778964, 0x007A8C65, 0x00788A63, 0x00627151, 0xFF949D89, 0xFFFDFFF9, 0xFFFDFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFCFF, 0xFFFFFFF8, 0x007C8668, 0x006A784F, 0x00828F69, 0x007C8864, 0x007C8A61, 0x00819161, 0x007C8B65, 0x00788761, 0x007F8E68, 0x007C8C63, 0x0073835A,
        0x0078895E, 0x007C8D62, 0x00809166, 0x007C8F5E, 0x007C8D61, 0x00819168, 0x00879670, 0x0082906C, 0x00808F69, 0x0084946B, 0x00829368, 0x007F916A, 0x007D8F68, 0x00859770, 0x007E9069, 0x007F9168, 0x00788A61, 0x007D8F66, 0x00798B62, 0x007E8F63,
        0x00798A5E, 0x00718256, 0x006E7F54, 0x007B8C61, 0x0078885F, 0x007B8B62, 0x0076865D, 0x0077875E, 0x00708057, 0x00506037, 0x0057673E, 0x0055653C, 0x006B7B52, 0x007F8F66, 0x007E8E65, 0x00768559, 0x00768559, 0x00768559, 0x00768559, 0x00768559,
        0x00768559, 0x0077865A, 0x0077865A, 0x00738458, 0x00748559, 0x0076875B, 0x0075865A, 0x00738458, 0x00728357, 0x00728357, 0x00738458, 0x00748559, 0x00748559, 0x00748559, 0x00748559, 0x00748559, 0x0075865A, 0x0075865A, 0x0075865A, 0x00728358,
        0x00738459, 0x00738459, 0x0074855A, 0x0074855A, 0x0075865B, 0x0075865B, 0x0075865B, 0x0075895C, 0x0074885B, 0x0075895C, 0x00778B5E, 0x00798C61, 0x00798C61, 0x00778960, 0x0076885F, 0x00798A5F, 0x0076865D, 0x0078885F, 0x0073835A, 0x0066754F,
        0x005F6E48, 0x005E6C48, 0x005A6844, 0x00596849, 0x004E5D3E, 0x005E6D4E, 0x00485839, 0x00586849, 0x00415132, 0x002F4122, 0x00394B2C, 0x00415336, 0x0035472A, 0x004B5D40, 0x003D4F32, 0x0037492C, 0x005F7154, 0x00293B1E, 0x0046583B, 0x005B6C47,
        0x00667654, 0x004E5D3E, 0x004F5D41, 0x004C5A3E, 0x00485738, 0x004E5E3C, 0x005D6E49, 0x00748B5E, 0x00475B38, 0x0057674F, 0x00414E40, 0x00556254, 0x0041503B, 0x003A4C2D, 0x003A4F28, 0x00304223, 0x00405233, 0x00445637, 0x00536546, 0x00596B4C,
        0x00516344, 0x00566849, 0x00586A4B, 0x005F704B, 0x00546540, 0x00596A45, 0x006D7E59, 0x00778863, 0x0073845F, 0x0071825D, 0x00768762, 0x0073855E, 0x0073855E, 0x0074865F, 0x00758760, 0x00768861, 0x00768861, 0x00778962, 0x00778962, 0x00758861,
        0x00758861, 0x00758861, 0x00748760, 0x00748760, 0x00748760, 0x00748760, 0x00748760, 0x00748760, 0x00748760, 0x00748760, 0x00748760, 0x00748760, 0x00748760, 0x00748760, 0x00748760, 0x0073865F, 0x0073865F, 0x0073865F, 0x0073865F, 0x0073865F,
        0x0073865F, 0x0073865F, 0x0073865F, 0x0074875C, 0x0073855C, 0x00788A61, 0x006F8158, 0x00667851, 0x00667851, 0x00657651, 0x006E7F5A, 0x0071825D, 0x00768762, 0x0073845F, 0x00768762, 0x0070815C, 0x00546540, 0x00455631, 0x0051623D, 0x0054663F,
        0x0061734C, 0x00465732, 0x00556641, 0x00425230, 0x00647452, 0x005F6F4D, 0x006D7C5C, 0x006C785A, 0x005C6B4C, 0x00758566, 0x0055684F, 0x00647665, 0x005E6F6B, 0x0054646B, 0x005D6A78, 0x004D5B43, 0x00606E56, 0x00687460, 0x00838F7D, 0x0097A194,
        0x00959E94, 0x00979E99, 0x008E9590, 0x00818A80, 0x00808782, 0x00979C9D, 0x00A3A8AB, 0x00A3AAA7, 0x009CA699, 0x00839277, 0x00758664, 0x0077866A, 0x007A8A6B, 0x007B8A6A, 0x007B8A6A, 0x0068765A, 0xFF969F8B, 0xFFFDFFF9, 0xFFFAFEF9, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFCFF, 0xFFFDFFF7, 0x007D8769, 0x006D7A54, 0x00838F6B, 0x00798561, 0x007C8A61, 0x00829262, 0x0082916B, 0x00798862, 0x007B8A64, 0x007F8F66, 0x007C8C63,
        0x0078895E, 0x007F9065, 0x0077885D, 0x007A8B60, 0x007D8C66, 0x007B8967, 0x00778464, 0x007A8668, 0x00748161, 0x00697753, 0x006E7D57, 0x006B7F54, 0x00697D52, 0x006C8055, 0x0075895F, 0x007C8F68, 0x0071845D, 0x00798B66, 0x0072845F, 0x007B8C61,
        0x00728358, 0x00798A5F, 0x0078885F, 0x00788761, 0x0071805A, 0x007E8C68, 0x007C8A66, 0x006E7E55, 0x0053633A, 0x00596940, 0x0054643B, 0x0067774E, 0x007C8C63, 0x0073835A, 0x0075855C, 0x007B8A5E, 0x007B8A5E, 0x007B8A5E, 0x0079885C, 0x0077865A,
        0x00768559, 0x00768559, 0x0077865A, 0x00738556, 0x00758758, 0x00768859, 0x00768859, 0x00748657, 0x00728455, 0x00738556, 0x00738556, 0x00738458, 0x00738458, 0x00738458, 0x00738458, 0x00738458, 0x00748559, 0x0075865A, 0x0075865A, 0x0074855A,
        0x0074855A, 0x0074855A, 0x0074855A, 0x0074855A, 0x0074855A, 0x0074855A, 0x0074855A, 0x00718556, 0x00708457, 0x00718558, 0x0073875A, 0x0075885D, 0x0076895E, 0x0074865D, 0x0072845B, 0x00798A5F, 0x006E7E55, 0x0078885F, 0x0077875E, 0x005E6D47,
        0x005B6A44, 0x0062704C, 0x0055633F, 0x0038452B, 0x00455238, 0x003C492F, 0x0057664B, 0x00415035, 0x0037482D, 0x00627358, 0x0067785D, 0x003A4D34, 0x00374A31, 0x00485B42, 0x0055684F, 0x0050634A, 0x002B3E25, 0x0043563D, 0x00667960, 0x005B6A4A,
        0x003B4A2B, 0x0049563C, 0x00606D55, 0x0056634B, 0x00546147, 0x00546344, 0x007A8969, 0x00738A5D, 0x006B7F5C, 0x004D5B49, 0x00656F69, 0x0078817E, 0x00536052, 0x00344627, 0x003D5329, 0x00627358, 0x0055664B, 0x00304126, 0x00425338, 0x004C5D42,
        0x00324328, 0x0035462B, 0x002D3E23, 0x004B5C37, 0x00596A45, 0x0051623D, 0x0052633E, 0x0071825D, 0x007B8C67, 0x0070815C, 0x0072835E, 0x00768861, 0x00768861, 0x00768861, 0x00758760, 0x00758760, 0x00758760, 0x00758760, 0x0074865F, 0x00768962,
        0x00758861, 0x00758861, 0x00758861, 0x00758861, 0x00748760, 0x00748760, 0x00748760, 0x00748760, 0x00748760, 0x00748760, 0x00748760, 0x00748760, 0x00748760, 0x00748760, 0x00748760, 0x0073865F, 0x0073865F, 0x0073865F, 0x0073865F, 0x0073865F,
        0x0073865F, 0x0073865F, 0x0073865F, 0x0074875C, 0x0072845B, 0x0076885F, 0x00687A51, 0x005A6C45, 0x00576942, 0x00556641, 0x005E6F4A, 0x00748560, 0x007B8C67, 0x0070815C, 0x007B8C67, 0x00647550, 0x00384924, 0x003E4F2A, 0x00475833, 0x0042542D,
        0x0024360F, 0x00394A25, 0x003C4D28, 0x003D4D2B, 0x002E3E1C, 0x00495937, 0x006F7E5E, 0x00869276, 0x00556347, 0x00718065, 0x00556853, 0x006B7E75, 0x006E8186, 0x005F7083, 0x005F708B, 0x00515E4E, 0x00616E5E, 0x006B776B, 0x008E9991, 0x00A2ABA8,
        0x0098A0A0, 0x009BA2A5, 0x00999FA4, 0x00959E9B, 0x008F9699, 0x00A9AEB7, 0x00A2A6B1, 0x009FA5AA, 0x009DA8A0, 0x00778671, 0x00728465, 0x0076846C, 0x0078876C, 0x00768468, 0x00778569, 0x0067745A, 0xFF949C8B, 0xFFFDFFF8, 0xFFF7FBF5, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xFFFFFEFE, 0x00788664, 0x006E8450, 0x00788768, 0x0082906C, 0x0060703A, 0x00667045, 0x00626E44, 0x00677349, 0x00647046, 0x006B774D, 0x00677349,
        0x00667248, 0x00616D43, 0x00748056, 0x00778863, 0x00495838, 0x0049563C, 0x005B6751, 0x003F4C34, 0x00334223, 0x0030421B, 0x00384B20, 0x002B3820, 0x00303D23, 0x00374529, 0x00414E2E, 0x004C5A38, 0x005F6C46, 0x00818F66, 0x0076845A, 0x007F9261,
        0x00718453, 0x007B8D5E, 0x007A8B60, 0x0074845B, 0x007B8A64, 0x007A8864, 0x00768462, 0x00617148, 0x00516138, 0x00526239, 0x006B7C51, 0x007D8E63, 0x00798A5F, 0x00748559, 0x0076875B, 0x00768859, 0x00768859, 0x00768859, 0x00758758, 0x00758758,
        0x00748657, 0x00748657, 0x00748657, 0x0078875B, 0x0078875B, 0x0077865A, 0x0077865A, 0x0077865A, 0x0077865A, 0x00768559, 0x00768559, 0x00738458, 0x00728357, 0x00728357, 0x00728357, 0x00738458, 0x00748559, 0x00738458, 0x00728357, 0x0074855A,
        0x0075865B, 0x0075865B, 0x0076875C, 0x0076875C, 0x0075865B, 0x0075865B, 0x0074855A, 0x00728455, 0x00728455, 0x00748559, 0x0075865A, 0x0076875C, 0x0077885D, 0x0077885D, 0x0077875E, 0x00738955, 0x00768B5A, 0x006F8257, 0x0070825D, 0x00526243,
        0x0058694E, 0x0034432E, 0x00677563, 0x00536A3D, 0x005E6E57, 0x00737B7A, 0x00737F79, 0x002A3D22, 0x00415936, 0x0041553E, 0x004E5C58, 0x006D7A62, 0x006F7969, 0x00727972, 0x006D776B, 0x004D5A44, 0x0027391C, 0x0051614A, 0x00374434, 0x00677B58,
        0x005F7055, 0x005B685A, 0x007A847E, 0x0077827A, 0x003E4D38, 0x00667A57, 0x00748B5E, 0x00798C61, 0x0076885F, 0x00495838, 0x00515A4D, 0x00717576, 0x006B706F, 0x00596450, 0x0072835E, 0x00424F3F, 0x004A5A43, 0x00435439, 0x00293521, 0x00616B5F,
        0x00636B60, 0x0057644C, 0x0051613F, 0x0067706D, 0x0037423A, 0x00596454, 0x00455439, 0x00718060, 0x006B7B59, 0x0073845F, 0x0075835F, 0x0072845F, 0x0072845F, 0x0072845F, 0x00738560, 0x00738560, 0x00738560, 0x00738560, 0x00748661, 0x00778962,
        0x00768861, 0x00758760, 0x00758760, 0x0074865F, 0x00758760, 0x00758760, 0x00758760, 0x00768861, 0x00758760, 0x0074865F, 0x0073855E, 0x0073855E, 0x0074865F, 0x00758760, 0x00768861, 0x00778962, 0x00778962, 0x00768861, 0x00758760, 0x0073855E,
        0x0073855E, 0x0074865F, 0x00768861, 0x00768C62, 0x006D8359, 0x00768B65, 0x00586C49, 0x0057694A, 0x00435538, 0x005D6E53, 0x004C5C44, 0x007F8A69, 0x007C8668, 0x00758163, 0x00798765, 0x0063754C, 0x00485938, 0x00656F63, 0x005C5F67, 0x004F603B,
        0x001F2919, 0x00424647, 0x00626964, 0x0059684D, 0x00364928, 0x003C4939, 0x004D5358, 0x00726E7A, 0x00787965, 0x00788254, 0x00465539, 0x005B7282, 0x006C89BC, 0x006587BC, 0x006B92B9, 0x005E6C66, 0x00536449, 0x006D7C5D, 0x008A8E88, 0x00B0ADB6,
        0x00959798, 0x00A1A6A5, 0x009FA5AA, 0x00ABB7BD, 0x00A8B6B5, 0x009CAAA6, 0x00A1ABAB, 0x00A2A8AD, 0x009BA29F, 0x007B8872, 0x0073855E, 0x00798B62, 0x00778765, 0x00798869, 0x00798765, 0x00687652, 0xFF96A082, 0xFFFFFFF8, 0xFFFDFBFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xFFFFFEFE, 0x00798765, 0x00738955, 0x007B8A6B, 0x0073815D, 0x003F4F19, 0x00414B20, 0x0039451B, 0x00364218, 0x0039451B, 0x0039451B, 0x002D390F,
        0x0038441A, 0x003C481E, 0x004C582E, 0x00687758, 0x004F5C44, 0x00535D4D, 0x00889086, 0x009CA49A, 0x00A3AC9F, 0x00A1AC98, 0x00A0AD93, 0x0099A392, 0x00A5AF9E, 0x00727E68, 0x00444F35, 0x004A5638, 0x00576242, 0x007C8864, 0x0084906C, 0x007C8D61,
        0x00798A5E, 0x007C8D61, 0x00809166, 0x0078885F, 0x0075855C, 0x006F7E58, 0x005A6943, 0x0054643B, 0x0058683F, 0x0066764D, 0x0076875C, 0x007B8C61, 0x0075865A, 0x00748559, 0x00798A5E, 0x00768859, 0x00768859, 0x00768859, 0x00758758, 0x00758758,
        0x00748657, 0x00748657, 0x00748657, 0x0078875B, 0x0077865A, 0x0077865A, 0x0077865A, 0x0077865A, 0x00768559, 0x00768559, 0x00768559, 0x00748559, 0x00738458, 0x00738458, 0x00738458, 0x00748559, 0x0075865A, 0x00748559, 0x00738458, 0x0074855A,
        0x0074855A, 0x0075865B, 0x0075865B, 0x0075865B, 0x0075865B, 0x0074855A, 0x0074855A, 0x00718354, 0x00728455, 0x00738556, 0x0075865A, 0x0076875B, 0x0076875C, 0x0076875C, 0x0076875C, 0x006D834F, 0x00728756, 0x0082956A, 0x0071845D, 0x004A5B3A,
        0x00637257, 0x0062725B, 0x004E5D48, 0x003E5027, 0x0065715D, 0x006B726F, 0x00717972, 0x00304126, 0x00465B35, 0x005D7055, 0x005B695D, 0x003F4E2F, 0x004F5948, 0x00535A53, 0x00535C52, 0x004E5A46, 0x00344327, 0x0058694E, 0x00596853, 0x00546543,
        0x00425136, 0x00545F4F, 0x00676F68, 0x00687167, 0x0044503C, 0x005D6E4C, 0x007B9063, 0x00718459, 0x007B8D64, 0x00576646, 0x005B6555, 0x00767B7A, 0x006A6F6D, 0x00444F3B, 0x00445530, 0x00677760, 0x0056684B, 0x00415132, 0x0036432D, 0x00656F63,
        0x005E665C, 0x005A6551, 0x003C4B2B, 0x005A6A52, 0x005E6F54, 0x00647554, 0x004B5D38, 0x0074865F, 0x007B8D66, 0x00748560, 0x00717F5D, 0x00738560, 0x00738560, 0x00738560, 0x00738560, 0x00738560, 0x00748661, 0x00748661, 0x00748661, 0x00768861,
        0x00768861, 0x00758760, 0x00758760, 0x0074865F, 0x0074865F, 0x00758760, 0x00758760, 0x00768861, 0x00758760, 0x0074865F, 0x0073855E, 0x0073855E, 0x0074865F, 0x00758760, 0x00768861, 0x00758760, 0x00768861, 0x00768861, 0x00758760, 0x0074865F,
        0x0074865F, 0x00758760, 0x00768861, 0x00758B61, 0x0073895F, 0x00748962, 0x00566A47, 0x00536645, 0x004E6043, 0x0066775C, 0x00536449, 0x005F6F46, 0x00788662, 0x007A8866, 0x00778960, 0x006A7E51, 0x004C5E37, 0x005D6957, 0x00606665, 0x004B5B39,
        0x00404A3A, 0x00888C8D, 0x00A9B0AB, 0x007C8A72, 0x00314122, 0x00596658, 0x00A3A9AE, 0x008D969F, 0x004A5640, 0x00415324, 0x00516947, 0x00708D92, 0x007698BC, 0x0082A8CB, 0x007298B0, 0x0064756C, 0x00405639, 0x00475738, 0x00636B61, 0x009C9CA2,
        0x00909495, 0x00A0A9A6, 0x00A4ADB0, 0x00B3BFC5, 0x00A6B4B3, 0x0099A7A3, 0x00939D9D, 0x008E9499, 0x00939A97, 0x007E8A74, 0x00788A63, 0x00788A61, 0x00778765, 0x00798869, 0x007A8866, 0x00677551, 0xFF949E80, 0xFFFFFFF8, 0xFFFEFCFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFBFC, 0xFFFBF9F9, 0x00707E5C, 0x00708652, 0x00819071, 0x0073815D, 0x003B4B15, 0x00485227, 0x00677349, 0x00576339, 0x00424E24, 0x00475329, 0x005D693F,
        0x00546036, 0x00303C12, 0x00546036, 0x006A775F, 0x00545E4D, 0x00636A63, 0x00C4C8C9, 0x00EAEDF2, 0x00DEE1E5, 0x00D5DAD9, 0x00DDE3DE, 0x00ECF3EC, 0x00EFF7ED, 0x008F978C, 0x00464D3E, 0x0057604C, 0x005D664C, 0x00687156, 0x00656F51, 0x005A6747,
        0x005C6949, 0x00546240, 0x005B6947, 0x0054623E, 0x0056653F, 0x0066754F, 0x0063724C, 0x0055653C, 0x0067774E, 0x0078895E, 0x007C8D62, 0x0076875B, 0x00738458, 0x0075865A, 0x00798B5C, 0x00768859, 0x00768859, 0x00758758, 0x00758758, 0x00758758,
        0x00758758, 0x00748657, 0x00748657, 0x0077865A, 0x0077865A, 0x0077865A, 0x0077865A, 0x0077865A, 0x00768559, 0x00768559, 0x00768559, 0x0075865A, 0x00748559, 0x00738458, 0x00748559, 0x0075865A, 0x0075865A, 0x00748559, 0x00738458, 0x0074855A,
        0x0074855A, 0x0075865B, 0x0075865B, 0x0075865B, 0x0075865B, 0x0074855A, 0x0074855A, 0x00718354, 0x00728455, 0x00738556, 0x00748559, 0x0075865A, 0x0075865A, 0x0075865B, 0x0075865B, 0x00778A59, 0x00758857, 0x006A7E51, 0x0074865F, 0x007B8C6A,
        0x00404F33, 0x002D3B23, 0x0054644C, 0x00535F3B, 0x00434B3A, 0x00494D48, 0x004E5449, 0x003D4C2D, 0x006F8158, 0x00586947, 0x0058654F, 0x00707F59, 0x00424C35, 0x00444B44, 0x00505651, 0x00465141, 0x004A593D, 0x00718261, 0x00506243, 0x00606E4C,
        0x00626E52, 0x00454D3C, 0x004E5449, 0x004E5548, 0x00363F2A, 0x0072805E, 0x0074855A, 0x00788A61, 0x00809269, 0x00586846, 0x0046513D, 0x004E544F, 0x00474E47, 0x0047533D, 0x00667851, 0x00506243, 0x004C5D3B, 0x0070825D, 0x00435237, 0x004B5447,
        0x004C544A, 0x00404837, 0x00535F43, 0x004F653B, 0x00344B1F, 0x00455A2D, 0x006D8353, 0x0073875A, 0x006B7D54, 0x00748560, 0x00778563, 0x0073865F, 0x0073865F, 0x0073865F, 0x00748760, 0x00748760, 0x00748760, 0x00748760, 0x00748760, 0x00768861,
        0x00768861, 0x00758760, 0x0074865F, 0x0074865F, 0x0074865F, 0x00758760, 0x00758760, 0x00768861, 0x00758760, 0x0074865F, 0x0073855E, 0x0073855E, 0x0074865F, 0x00758760, 0x00768861, 0x0073855E, 0x00758760, 0x00768861, 0x00768861, 0x00758760,
        0x0074865F, 0x00758760, 0x00768861, 0x0072865C, 0x0073875D, 0x00798C65, 0x0070825D, 0x005E6F4D, 0x00506140, 0x00627253, 0x00657458, 0x00516236, 0x006A7953, 0x0073845F, 0x007B8D64, 0x00708455, 0x004B5D34, 0x004B5840, 0x00495049, 0x003C482A,
        0x002F3629, 0x005F6463, 0x006E746F, 0x00505C48, 0x001B2911, 0x00636D61, 0x00A6ACB1, 0x00859A9B, 0x003B5138, 0x00607A4A, 0x004D6C47, 0x005E807F, 0x0083ACC3, 0x007FA9C0, 0x0090BAC6, 0x0071847B, 0x00425A3C, 0x00364B2A, 0x005D665C, 0x00A5A9AE,
        0x00A1A6A7, 0x00AAB5B2, 0x00AFBBBD, 0x00B5C1C7, 0x00AEBABA, 0x00B8C6C2, 0x00B4BEBE, 0x00A0A6AB, 0x00A1A8A5, 0x00808C76, 0x006C7E57, 0x00778960, 0x00778765, 0x00798869, 0x007B8967, 0x00677551, 0xFF919B7D, 0xFFFFFFF8, 0xFFFEFCFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xFFFEFCFC, 0x006C7A58, 0x006C824E, 0x00829172, 0x0072805C, 0x0040501A, 0x005C663B, 0x006C784E, 0x00616D43, 0x00505C32, 0x005D693F, 0x007E8A60,
        0x00677349, 0x002B370D, 0x00546036, 0x00728162, 0x0054604A, 0x00727A70, 0x00D9DDDE, 0x00E3E5ED, 0x00C6C7D1, 0x00BBBEC3, 0x00CCCFD3, 0x00D2D5D9, 0x00E6EAEB, 0x00A5AAA8, 0x005F645B, 0x00565D50, 0x00414734, 0x00464E37, 0x00565F45, 0x00697063,
        0x004F5647, 0x00353D2C, 0x003E4733, 0x0047513A, 0x00535E44, 0x00576347, 0x00586448, 0x006D7E53, 0x0075865B, 0x007A8B60, 0x0078895D, 0x00748559, 0x0075865A, 0x00768859, 0x00768859, 0x00758758, 0x00758758, 0x00758758, 0x00758758, 0x00758758,
        0x00758758, 0x00758758, 0x00758758, 0x00778758, 0x00778758, 0x00778758, 0x00778758, 0x00768657, 0x00768657, 0x00768657, 0x00768657, 0x00748657, 0x00738556, 0x00728455, 0x00738556, 0x00748657, 0x00758758, 0x00748657, 0x00738556, 0x00738458,
        0x00748559, 0x00748559, 0x00748559, 0x00748559, 0x00748559, 0x00748559, 0x00738458, 0x00718453, 0x00728554, 0x00738556, 0x00748657, 0x0075865A, 0x0075865A, 0x0075865A, 0x0075865B, 0x00778A59, 0x006B7D4E, 0x00809165, 0x007D8F66, 0x00586944,
        0x00536443, 0x005E6D51, 0x0048573C, 0x0041492A, 0x00454936, 0x00393A31, 0x00494E3F, 0x004D5838, 0x00738459, 0x0078885F, 0x0073815F, 0x006A7C4D, 0x00465234, 0x00454A41, 0x00464C47, 0x00495343, 0x00516142, 0x006F815C, 0x0070835C, 0x0076825E,
        0x00616B4D, 0x004D543F, 0x00454B3A, 0x00454B3A, 0x00414932, 0x00667151, 0x0075825C, 0x0072835E, 0x006D7F58, 0x004A5A38, 0x00435038, 0x004A5247, 0x00394136, 0x00445139, 0x0072845D, 0x0071835E, 0x00798D63, 0x00667A50, 0x003C4B2C, 0x00505A4A,
        0x004C5148, 0x00474E3F, 0x003F4A30, 0x00506342, 0x00495D3A, 0x004F623B, 0x005F7348, 0x007A8D62, 0x0073855C, 0x0071835C, 0x007A8864, 0x0073865F, 0x00748760, 0x00748760, 0x00748760, 0x00748760, 0x00758861, 0x00758861, 0x00758861, 0x00768861,
        0x00758760, 0x00758760, 0x0074865F, 0x0074865F, 0x0074865F, 0x0074865F, 0x00758760, 0x00758760, 0x00758760, 0x0074865F, 0x0074865F, 0x0074865F, 0x0074865F, 0x00758760, 0x00758760, 0x0072845D, 0x0073855E, 0x00758760, 0x00768861, 0x00758760,
        0x00758760, 0x00758760, 0x00768861, 0x00788C62, 0x0071855B, 0x006E815A, 0x007C8E69, 0x00768863, 0x006A7B59, 0x00627351, 0x00576847, 0x006E8059, 0x00596848, 0x00515F43, 0x005E6D4D, 0x00576942, 0x00485634, 0x004D5642, 0x00484C46, 0x00414D31,
        0x002D3425, 0x0050544F, 0x00596059, 0x004A5443, 0x001E2B15, 0x005F695D, 0x00777C7D, 0x00738A82, 0x00647B5F, 0x00627A50, 0x00284728, 0x00507678, 0x0081ACC7, 0x007BAAC6, 0x0084B4C6, 0x00596E65, 0x00375133, 0x00344C2A, 0x005F6A60, 0x00A3A7AC,
        0x009BA3A2, 0x00A8B6B2, 0x00BCC9CB, 0x00B3BDC4, 0x00ACB8B8, 0x00C4CFCC, 0x00B9C1C1, 0x00999DA2, 0x009FA4A2, 0x00828E78, 0x0072815B, 0x0076885F, 0x00788866, 0x00798869, 0x007C8A68, 0x00667450, 0xFF8D9779, 0xFFFFFFF8, 0xFFFFFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xFFFEFCFC, 0x006E7C5A, 0x006D834F, 0x00819071, 0x006F7D59, 0x003D4D17, 0x005D673C, 0x004C582E, 0x004D592F, 0x00606C42, 0x0068744A, 0x00657147,
        0x00626E44, 0x00414D23, 0x00525E34, 0x007C8E67, 0x00617051, 0x00889281, 0x00DCE2DD, 0x00D2D6D7, 0x00CBCED2, 0x00C6CACB, 0x00CBD0CF, 0x00D1D3DB, 0x00DDE0E5, 0x00ADB2B1, 0x00848883, 0x00888F82, 0x00646A59, 0x00606851, 0x00848D73, 0x008E9090,
        0x006F7270, 0x005D615C, 0x00636761, 0x0072776E, 0x00838A7D, 0x005F6657, 0x00545B4C, 0x007E8F64, 0x00798A5F, 0x0075865A, 0x00748559, 0x0076875B, 0x0077895A, 0x00768859, 0x00768859, 0x00758758, 0x00758758, 0x00758758, 0x00758758, 0x00758758,
        0x00758758, 0x00758758, 0x00758758, 0x00778758, 0x00778758, 0x00778758, 0x00768657, 0x00768657, 0x00768657, 0x00768657, 0x00758556, 0x00738556, 0x00728455, 0x00718354, 0x00718354, 0x00738556, 0x00738556, 0x00728455, 0x00718354, 0x00738458,
        0x00738458, 0x00738458, 0x00748559, 0x00748559, 0x00738458, 0x00738458, 0x00738458, 0x00728554, 0x00728554, 0x00738655, 0x00748657, 0x00748657, 0x0075865A, 0x00748559, 0x00748559, 0x00758555, 0x00788859, 0x00728357, 0x006E7F54, 0x0052643D,
        0x00566644, 0x00475837, 0x005F704F, 0x004C5233, 0x008A8D77, 0x009B9E8E, 0x007D826D, 0x00475231, 0x004D5B31, 0x00556438, 0x0057673E, 0x004F602D, 0x00657050, 0x009EA598, 0x00A4A9A0, 0x0076816D, 0x004D5D3B, 0x004B5F35, 0x00506439, 0x005B6842,
        0x004F5A39, 0x00677056, 0x00939A85, 0x008E9580, 0x00586147, 0x004E5939, 0x00596541, 0x00566545, 0x0051623D, 0x004B5C37, 0x006D7B5F, 0x008C9686, 0x00697363, 0x00475539, 0x004A5C33, 0x00485B34, 0x00506538, 0x00465A2D, 0x005B6A4A, 0x009EA695,
        0x00979D92, 0x00777D6A, 0x003A4328, 0x00596559, 0x00505D4D, 0x005D6B53, 0x00465637, 0x006E7F5A, 0x00768861, 0x007A8C65, 0x0075845E, 0x0073875D, 0x0074885E, 0x0074885E, 0x0074885E, 0x0074885E, 0x0075895F, 0x0075895F, 0x0075895F, 0x00768861,
        0x00758760, 0x0074865F, 0x0074865F, 0x0074865F, 0x0074865F, 0x0074865F, 0x0074865F, 0x00758760, 0x00758760, 0x0074865F, 0x0074865F, 0x0074865F, 0x0074865F, 0x00758760, 0x00758760, 0x0071835C, 0x0073855E, 0x00758760, 0x00768861, 0x00758760,
        0x0074865F, 0x00758760, 0x00758760, 0x0073855C, 0x007A8C63, 0x0073855E, 0x00778962, 0x00788A63, 0x007F916A, 0x0071825D, 0x00546540, 0x005A6C4D, 0x0064715B, 0x00626D5D, 0x00545E4D, 0x004B563C, 0x00626D53, 0x0082867A, 0x00888987, 0x0060694F,
        0x00444A37, 0x006C7366, 0x00868D80, 0x0068735F, 0x0035402C, 0x007D877A, 0x009DA49F, 0x00728376, 0x00606D53, 0x0052603E, 0x00203421, 0x00557580, 0x0083AECF, 0x0079A6CB, 0x007EADCD, 0x00596F63, 0x00395131, 0x00314625, 0x005B6559, 0x009CA1A4,
        0x009BA3A2, 0x00ABB7B1, 0x00BECACA, 0x00B0BAC1, 0x00AFB9B9, 0x00C6D1CE, 0x00B6BEBE, 0x00909499, 0x00969B99, 0x00818A75, 0x0074835D, 0x0076885F, 0x00798967, 0x00788768, 0x007E8C6A, 0x00677551, 0xFF8B9577, 0xFFFFFFF8, 0xFFFFFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFBFC, 0xFFFEFCFC, 0x006F7D5B, 0x006D834F, 0x00808F70, 0x00717F5B, 0x003F4F19, 0x005B653A, 0x00647046, 0x005C683E, 0x005B673D, 0x00606C42, 0x00657147,
        0x00616D43, 0x00414D23, 0x00525E34, 0x006D7F56, 0x005B6A4A, 0x007B8672, 0x00D3DBD1, 0x00D2D7D5, 0x00D2D7D6, 0x00CBD0CE, 0x00C8CFC8, 0x00D2D5DA, 0x00E5E8EC, 0x00AEB3B1, 0x00969B92, 0x00949B8C, 0x00505841, 0x00525B40, 0x008A9575, 0x00999D97,
        0x00737771, 0x005B5F59, 0x00656A61, 0x0093998E, 0x00BCC2B7, 0x007A8174, 0x0052594C, 0x007D8E63, 0x0076875B, 0x00738458, 0x0076875B, 0x00798B5C, 0x0077895A, 0x00768958, 0x00788B5A, 0x00748657, 0x00748657, 0x00758758, 0x00758758, 0x00758758,
        0x00758758, 0x00768859, 0x00768859, 0x00778757, 0x00768656, 0x00768656, 0x00768656, 0x00768656, 0x00758555, 0x00758555, 0x00758555, 0x00728554, 0x00718453, 0x00708352, 0x00718453, 0x00728554, 0x00728554, 0x00728554, 0x00708352, 0x00728455,
        0x00728455, 0x00738556, 0x00738556, 0x00738556, 0x00738556, 0x00728455, 0x00728455, 0x00738653, 0x00738653, 0x00748756, 0x00748756, 0x00758758, 0x00758758, 0x00748657, 0x00748559, 0x00758555, 0x007A8A5B, 0x007A8A5B, 0x0074855A, 0x0054643B,
        0x0060724B, 0x004D5E39, 0x005E6F4A, 0x00626C48, 0x007E8667, 0x009FA68B, 0x007C856A, 0x005C6746, 0x0067774E, 0x0056673C, 0x00617247, 0x00637441, 0x005F6A49, 0x00929885, 0x009AA291, 0x006D795D, 0x005A6C45, 0x0065784D, 0x005D6F46, 0x005D6C46,
        0x00677551, 0x00738060, 0x00ABB79B, 0x00A2AD93, 0x00647054, 0x006C785A, 0x006A7757, 0x00617051, 0x00647550, 0x0060724B, 0x00728161, 0x008E9A84, 0x0078836F, 0x00627152, 0x005D6F46, 0x00657750, 0x00586B40, 0x005E7245, 0x005B6947, 0x00616A55,
        0x00616756, 0x00697258, 0x0065714D, 0x006A7370, 0x00404B41, 0x005E6A58, 0x0059684C, 0x00738361, 0x006E8059, 0x007A8C63, 0x00728259, 0x0073875D, 0x0073875D, 0x0073875D, 0x0074885E, 0x0074885E, 0x0074885E, 0x0074885E, 0x0074885E, 0x00758760,
        0x00758760, 0x0074865F, 0x0074865F, 0x0073855E, 0x0073855E, 0x0074865F, 0x0074865F, 0x00758760, 0x00758760, 0x0074865F, 0x0074865F, 0x0074865F, 0x0074865F, 0x00758760, 0x00758760, 0x0072845D, 0x0074865F, 0x00758760, 0x00758760, 0x0074865F,
        0x0074865F, 0x0074865F, 0x00758760, 0x0070825B, 0x007A8C63, 0x0076885F, 0x0074865D, 0x0072845B, 0x0076885F, 0x00798B62, 0x006F8158, 0x004D6242, 0x004E5D48, 0x00576155, 0x006D7669, 0x006D7661, 0x005E664F, 0x006A6D5E, 0x009D9C98, 0x0097A085,
        0x00737C62, 0x00939A85, 0x00BAC3AE, 0x00A3AC97, 0x0066725C, 0x00747E6D, 0x00656E61, 0x0063715F, 0x00434C32, 0x006E755A, 0x00536052, 0x00637D84, 0x0088ABC5, 0x0084AAC8, 0x008CB1CB, 0x0055665B, 0x00536A4A, 0x00617250, 0x0070786D, 0x00828387,
        0x00717674, 0x008D9791, 0x00B0BABA, 0x00A3ABB2, 0x00A2ACAC, 0x00AEB7B4, 0x009DA2A3, 0x007F8287, 0x00888B89, 0x007A836E, 0x0078855F, 0x00778960, 0x007A8A68, 0x00778667, 0x007F8D6B, 0x00687652, 0xFF8A9476, 0xFFFFFFF8, 0xFFFFFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xFFFFFFFF, 0x00717F5D, 0x006C824E, 0x007E8D6E, 0x0073815D, 0x003D4D17, 0x004C562B, 0x00748056, 0x00657147, 0x00445026, 0x004E5A30, 0x0069754B,
        0x0059653B, 0x00344016, 0x00556137, 0x006A795A, 0x007C8971, 0x0080887D, 0x00D8DDDB, 0x00E6E9ED, 0x00D7DBDC, 0x00CED3D1, 0x00CFD5D0, 0x00DFE3E4, 0x00E3E8E7, 0x00939A93, 0x009EA596, 0x00A8B19C, 0x00555F41, 0x005B6743, 0x0068754F, 0x00576347,
        0x005C684C, 0x00596549, 0x00586349, 0x005A654B, 0x00707A63, 0x0057614A, 0x00616A55, 0x0076875B, 0x00748559, 0x0075865A, 0x0077895A, 0x00788A5B, 0x00768958, 0x00768958, 0x00778A59, 0x00748657, 0x00748657, 0x00748657, 0x00758758, 0x00758758,
        0x00768859, 0x00768859, 0x00768859, 0x00768656, 0x00768656, 0x00768656, 0x00768656, 0x00768656, 0x00758555, 0x00758555, 0x00758555, 0x00738655, 0x00728554, 0x00718453, 0x00718453, 0x00738655, 0x00738655, 0x00728554, 0x00718453, 0x00728455,
        0x00728455, 0x00738556, 0x00738556, 0x00738556, 0x00738556, 0x00728455, 0x00728455, 0x00738653, 0x00748754, 0x00758855, 0x00758857, 0x00758857, 0x00758758, 0x00758758, 0x00748657, 0x00798959, 0x00738354, 0x00758556, 0x007A895D, 0x0065764B,
        0x0056663D, 0x004C5E37, 0x0051633C, 0x004E5C32, 0x003B4B22, 0x00202F09, 0x00303E1C, 0x00445432, 0x0052633E, 0x005D7147, 0x0051653A, 0x0059693A, 0x0045512D, 0x00313922, 0x002E371D, 0x00414F2B, 0x004F6034, 0x00506338, 0x005A6C45, 0x005B6F45,
        0x00475A33, 0x0040512F, 0x00314122, 0x002E3E1F, 0x00415034, 0x005A6A4B, 0x00748465, 0x00576549, 0x005B6B49, 0x0044562D, 0x002A3A18, 0x002D3A20, 0x002B3820, 0x00415030, 0x0056693E, 0x004E5E3C, 0x005B6D44, 0x0056673C, 0x0052603C, 0x00394228,
        0x001C240D, 0x00505B3B, 0x005A693D, 0x004B5A45, 0x004E5E46, 0x00536344, 0x00546641, 0x00798B62, 0x0073855C, 0x006D7F56, 0x0073825C, 0x0073875D, 0x0073875D, 0x0073875D, 0x0073875D, 0x0073875D, 0x0074885E, 0x0074885E, 0x0074885E, 0x00758760,
        0x00758760, 0x0074865F, 0x0073855E, 0x0073855E, 0x0073855E, 0x0074865F, 0x0074865F, 0x0074865F, 0x0074865F, 0x00758760, 0x00758760, 0x00758760, 0x00758760, 0x0074865F, 0x0074865F, 0x0074865F, 0x00758760, 0x00758760, 0x0074865F, 0x0073855E,
        0x0072845D, 0x0073855E, 0x00758760, 0x007C8E67, 0x0071835A, 0x006A7C53, 0x0071835A, 0x007C8F64, 0x0074875C, 0x00798D60, 0x00778B5E, 0x0051693F, 0x005A6F4F, 0x0051604B, 0x004A5640, 0x00586446, 0x00677251, 0x004D5439, 0x00464835, 0x007D8868,
        0x004C5737, 0x00364022, 0x00434C31, 0x00455036, 0x00434D36, 0x0045523A, 0x0026321C, 0x004D6350, 0x004D5B43, 0x006E755A, 0x0077806C, 0x0070817D, 0x006D8388, 0x008EA4A9, 0x0077898A, 0x00768478, 0x006D805F, 0x0073815F, 0x00868A7E, 0x009B989A,
        0x008A8B89, 0x00898D87, 0x00858B8A, 0x00899198, 0x0096A0A0, 0x00A4ADAA, 0x00A7ACAD, 0x009DA0A5, 0x009EA19F, 0x00858C77, 0x007B8862, 0x00798B62, 0x007B8B69, 0x00768566, 0x00808E6C, 0x006A7854, 0xFF8A9476, 0xFFFFFFF8, 0xFFFEFCFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFAFB, 0xFFFBF9F9, 0x006E7C5A, 0x006B814D, 0x007F8E6F, 0x00778561, 0x003B4B15, 0x003E481D, 0x0039451B, 0x003A461C, 0x002F3B11, 0x00374319, 0x003E4A20,
        0x003A461C, 0x00344016, 0x00525E34, 0x006C7763, 0x00B0B8AD, 0x00969B99, 0x00D3D6DB, 0x00E8E9F3, 0x00DADCE4, 0x00DBDEE2, 0x00DDE2E1, 0x00E3E8E7, 0x00F2F8F3, 0x00A9B1A6, 0x00C8D1BD, 0x00B7C3A7, 0x00596443, 0x00818F66, 0x00728155, 0x00778A59,
        0x007D8F60, 0x00748559, 0x007F9065, 0x0077875E, 0x0075835F, 0x00657351, 0x007C8A68, 0x0076875B, 0x0076875B, 0x00768859, 0x00748657, 0x00748657, 0x00758857, 0x00758857, 0x00748756, 0x00748657, 0x00748657, 0x00748657, 0x00758758, 0x00758758,
        0x00768859, 0x00768859, 0x00768859, 0x00768656, 0x00768656, 0x00768656, 0x00768656, 0x00758555, 0x00758555, 0x00758555, 0x00758555, 0x00748756, 0x00738655, 0x00728554, 0x00728554, 0x00748756, 0x00748756, 0x00738655, 0x00728554, 0x00718354,
        0x00728455, 0x00728455, 0x00738556, 0x00738556, 0x00728455, 0x00728455, 0x00718354, 0x00748754, 0x00748754, 0x00758855, 0x00768958, 0x00768958, 0x00758758, 0x00758758, 0x00748657, 0x00758353, 0x00768657, 0x00718152, 0x00707F53, 0x00819267,
        0x0066774C, 0x006F8158, 0x006F8158, 0x006C7F4E, 0x0055693A, 0x00576A3F, 0x00556740, 0x00617250, 0x00778866, 0x00738861, 0x00758B61, 0x007B885C, 0x00626D4C, 0x005C654A, 0x005F6A4A, 0x005E6F44, 0x006F8251, 0x007A8D62, 0x00768664, 0x00758B61,
        0x006E835C, 0x00516640, 0x004F6340, 0x004F6142, 0x00586A4B, 0x00748669, 0x006E8063, 0x0079866C, 0x007A8A68, 0x0065774E, 0x004C5D38, 0x00526044, 0x00505D45, 0x00647353, 0x0073865B, 0x00798869, 0x0073845F, 0x0075855C, 0x005E6C48, 0x005F684D,
        0x005C654B, 0x005E6B45, 0x006F7E4B, 0x006F855B, 0x00687F53, 0x005F7545, 0x00788E5D, 0x00748859, 0x00718558, 0x00778960, 0x0075845E, 0x0072865B, 0x0072865B, 0x0072865B, 0x0073875C, 0x0073875C, 0x0073875C, 0x0073875C, 0x0074885D, 0x00758760,
        0x00758760, 0x0074865F, 0x0073855E, 0x0073855E, 0x0073855E, 0x0073855E, 0x0074865F, 0x0074865F, 0x0074865F, 0x00758760, 0x00758760, 0x00758760, 0x00758760, 0x0074865F, 0x0074865F, 0x00758760, 0x00758760, 0x00758760, 0x0074865F, 0x0072845D,
        0x0072845D, 0x0073855E, 0x00758760, 0x0074835D, 0x0075855C, 0x00798960, 0x0074855A, 0x007E8F64, 0x00728357, 0x007B8C60, 0x00768859, 0x006B8A53, 0x0060784E, 0x0067795A, 0x00728362, 0x0068784F, 0x00647143, 0x00535D33, 0x005C6342, 0x005C6544,
        0x00535F3B, 0x004C5834, 0x00515C3B, 0x00465234, 0x004A553B, 0x004F5C42, 0x0049573B, 0x004A6A57, 0x00495F46, 0x00677355, 0x00616A4F, 0x005C6854, 0x004A5A48, 0x0059654F, 0x0063684D, 0x005C685C, 0x00657454, 0x007B8665, 0x00919185, 0x009A9597,
        0x00878584, 0x00888A84, 0x00818383, 0x00959DA4, 0x00A1A9A9, 0x009BA4A1, 0x009CA1A2, 0x009C9FA4, 0x009B9E9C, 0x00818873, 0x0078855F, 0x007A8C63, 0x007C8C6A, 0x00758465, 0x00808E6C, 0x006B7955, 0xFF8A9476, 0xFFFFFFF8, 0xFFFEFCFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBF8FF, 0xFBFCFFF4, 0x00727E5A, 0x00738354, 0x00808F63, 0x00717E58, 0x00536138, 0x004D5C30, 0x00495B34, 0x0044562F, 0x0043552E, 0x00465831, 0x00495B34,
        0x00485A33, 0x00495B34, 0x0053653E, 0x006F7A52, 0x007B865E, 0x0089946C, 0x00A5B088, 0x009FAA82, 0x0087926A, 0x0095A078, 0x00A0AB83, 0x00808B63, 0x0099A47C, 0x00B8C39B, 0x00B9C49C, 0x00A3AE86, 0x00667149, 0x0077825A, 0x007D8860, 0x007C8B5F,
        0x007C8B5F, 0x007C8B5F, 0x007B8A5E, 0x007B8A5E, 0x007B8A5E, 0x007B8A5E, 0x007A895D, 0x00788859, 0x00788859, 0x00788859, 0x00788858, 0x00788858, 0x00788858, 0x00788956, 0x00788956, 0x00758555, 0x00768656, 0x00778757, 0x00778757, 0x00778757,
        0x00778757, 0x00768656, 0x00758555, 0x00758458, 0x00788859, 0x00748455, 0x00728252, 0x00758555, 0x00758653, 0x00758752, 0x00758752, 0x00718151, 0x00798959, 0x00768656, 0x00708050, 0x00758555, 0x00778757, 0x00728252, 0x00738353, 0x00718152,
        0x00778758, 0x00778758, 0x00748455, 0x00788859, 0x00778758, 0x00708051, 0x00728253, 0x00728252, 0x00738353, 0x00748454, 0x00758556, 0x00768657, 0x00768559, 0x00768559, 0x00768559, 0x00768656, 0x00758555, 0x00758555, 0x00748454, 0x00748454,
        0x00758555, 0x00758555, 0x00768656, 0x00798A5E, 0x00798A5E, 0x0078895D, 0x0077885C, 0x0076875B, 0x0076875B, 0x0075865A, 0x0075865A, 0x00718459, 0x0072855A, 0x00788B60, 0x007C8C63, 0x0078885F, 0x0078865D, 0x0078855F, 0x0075825C, 0x006F8158,
        0x006A7C53, 0x0073855C, 0x007D8F66, 0x00798B62, 0x0075875E, 0x0075875E, 0x00708259, 0x0073835A, 0x00728259, 0x0075855C, 0x00778960, 0x0074865D, 0x0075895F, 0x00778B61, 0x006E8258, 0x006E8156, 0x00708358, 0x0072855A, 0x0074875C, 0x0075885D,
        0x0074875C, 0x0073865B, 0x0072855A, 0x0075885D, 0x0075885D, 0x0074875C, 0x0073865B, 0x0073865B, 0x0072855A, 0x00718459, 0x00718459, 0x0074875C, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0076895E,
        0x0076895E, 0x0074875C, 0x00718459, 0x0073865B, 0x0073865B, 0x006F8257, 0x0072855A, 0x00728358, 0x0075865B, 0x0078895E, 0x00738459, 0x006E7F54, 0x0078895E, 0x0076875C, 0x007A8B60, 0x0074895C, 0x0074895C, 0x0074885B, 0x0073865B, 0x0073855C,
        0x0074845B, 0x00778660, 0x007D8C66, 0x0073855C, 0x0074865D, 0x0075875E, 0x0076885F, 0x0075875E, 0x0074865D, 0x0072845B, 0x0071835A, 0x0076885F, 0x00778960, 0x00788A61, 0x00798B62, 0x00788A61, 0x0075875E, 0x0073855C, 0x0071835A, 0x00738462,
        0x00748563, 0x00768765, 0x00778A69, 0x00788B6A, 0x00738868, 0x006D8262, 0x00687D5D, 0x005E735D, 0x0060745D, 0x0072846D, 0x00506146, 0x0048563A, 0x003A4628, 0x004C5435, 0x004B5334, 0x00576345, 0x00707C60, 0x00848F75, 0x007D8672, 0x007D8475,
        0x00898E85, 0x008F938E, 0x007F8280, 0x00888D84, 0x0080867B, 0x008A9184, 0x00858D7C, 0x0079826D, 0x008D987E, 0x00818D6F, 0x007D896B, 0x00798D60, 0x00778962, 0x00798967, 0x007B8965, 0x006A7953, 0xFF889274, 0xFFFFFFF8, 0xFFFFFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFCFF, 0xFFFDFFF5, 0x006F7B57, 0x00708051, 0x00849367, 0x0083906A, 0x0078865D, 0x007D8C60, 0x00788A63, 0x00788A63, 0x007A8C65, 0x007B8D66, 0x00798B64,
        0x0074865F, 0x0073855E, 0x00778962, 0x00768159, 0x00747F57, 0x00707B53, 0x007D8860, 0x0079845C, 0x006F7A52, 0x007D8860, 0x007D8860, 0x00768159, 0x00737E56, 0x00727D55, 0x00737E56, 0x0079845C, 0x0068734B, 0x00818C64, 0x00818C64, 0x007A895D,
        0x007A895D, 0x007A895D, 0x0079885C, 0x0079885C, 0x0079885C, 0x0079885C, 0x0078875B, 0x0079895A, 0x0079895A, 0x00788859, 0x00778757, 0x00778757, 0x00768656, 0x00768754, 0x00758653, 0x00768656, 0x00768656, 0x00768656, 0x00778757, 0x00778757,
        0x00768656, 0x00768656, 0x00768656, 0x00718152, 0x00768657, 0x00778758, 0x00778757, 0x00768656, 0x00738451, 0x0070814E, 0x006D7E4B, 0x00778757, 0x00748454, 0x00758555, 0x00738353, 0x00738353, 0x00768656, 0x00738353, 0x00748454, 0x0079895A,
        0x00788859, 0x00738354, 0x006F7F50, 0x00738354, 0x00738354, 0x00738354, 0x007A8A5B, 0x00738353, 0x00738353, 0x00748454, 0x00768657, 0x00768657, 0x0077865A, 0x0077865A, 0x0077865A, 0x00768656, 0x00758555, 0x00758555, 0x00748454, 0x00748454,
        0x00758555, 0x00758555, 0x00768656, 0x0075865A, 0x00748559, 0x00748559, 0x00738458, 0x00738458, 0x00728357, 0x00728357, 0x00728357, 0x0076895E, 0x0073865B, 0x0073865B, 0x0076865D, 0x0074845B, 0x0075835A, 0x0073805A, 0x00717E58, 0x00788A61,
        0x0075875E, 0x0072845B, 0x0071835A, 0x0072845B, 0x0073855C, 0x0073855C, 0x0071835A, 0x0075855C, 0x00718158, 0x00728259, 0x0073855C, 0x0071835A, 0x0071855B, 0x0074885E, 0x006F8359, 0x00718459, 0x00718459, 0x0072855A, 0x0072855A, 0x0072855A,
        0x0073865B, 0x0073865B, 0x0073865B, 0x0074875C, 0x0074875C, 0x0073865B, 0x0073865B, 0x0072855A, 0x0072855A, 0x0072855A, 0x0072855A, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x00778960,
        0x0074865D, 0x0072845B, 0x0072845B, 0x0074865D, 0x0072845B, 0x0072845B, 0x00778960, 0x0076865D, 0x0074845B, 0x0076865D, 0x0076865D, 0x00798960, 0x00798960, 0x0074845B, 0x00718158, 0x00708558, 0x0074885D, 0x0075885D, 0x0075885D, 0x0076885F,
        0x0076865D, 0x0073825C, 0x0074835D, 0x0074865D, 0x0075875E, 0x0075875E, 0x0076885F, 0x0075875E, 0x0074865D, 0x0073855C, 0x0072845B, 0x0073855C, 0x0074865D, 0x0075875E, 0x0076885F, 0x0076885F, 0x0075875E, 0x0074865D, 0x0074865D, 0x0074865F,
        0x00738560, 0x00748661, 0x00778964, 0x00768A67, 0x00758966, 0x00708564, 0x006D8261, 0x00677C63, 0x00697F63, 0x0076896E, 0x00607255, 0x00627152, 0x00606D4D, 0x00677251, 0x00636C4B, 0x00606E4C, 0x00798666, 0x007D896D, 0x0069735C, 0x005D6652,
        0x005F6659, 0x0080867B, 0x00747970, 0x00737A6D, 0x00575E4F, 0x0058604F, 0x005A634E, 0x00556046, 0x00758163, 0x007A8767, 0x007C8A68, 0x00798D60, 0x00778962, 0x00798967, 0x007B8965, 0x006A7953, 0xFF889274, 0xFFFFFFF8, 0xFFFFFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFCFF, 0xFBFCFFF4, 0x006F7B57, 0x006E7E4F, 0x007F8E62, 0x00808D67, 0x0079875E, 0x00808F63, 0x00798B62, 0x00798B62, 0x007A8C63, 0x00798B62, 0x0076885F,
        0x0076885F, 0x00798B62, 0x00788A61, 0x00838E66, 0x00869169, 0x007A855D, 0x007E8961, 0x00848F67, 0x00828D65, 0x0088936B, 0x00848F67, 0x00848F67, 0x00818C64, 0x007C875F, 0x007E8961, 0x00838E66, 0x007E8961, 0x0087926A, 0x007B865E, 0x007A895D,
        0x007A895D, 0x007A895D, 0x007A895D, 0x0079885C, 0x0079885C, 0x0079885C, 0x0079885C, 0x0079895A, 0x0079895A, 0x00788859, 0x00778757, 0x00768656, 0x00758555, 0x00748552, 0x00748552, 0x00778757, 0x00768656, 0x00768656, 0x00768656, 0x00768656,
        0x00768656, 0x00768656, 0x00778757, 0x00768657, 0x00788859, 0x00788859, 0x00768657, 0x00778757, 0x00788858, 0x00778757, 0x00788858, 0x007A8A5B, 0x00708051, 0x00738354, 0x00728253, 0x00718152, 0x00758556, 0x00738354, 0x00758556, 0x00768657,
        0x00748455, 0x00738354, 0x00758556, 0x007A8A5B, 0x00758556, 0x00738354, 0x00728253, 0x00738353, 0x00748454, 0x00758555, 0x00768657, 0x00778758, 0x0078875B, 0x0078875B, 0x0078875B, 0x00768656, 0x00768656, 0x00758555, 0x00758555, 0x00758555,
        0x00758555, 0x00768656, 0x00768656, 0x00728357, 0x00728357, 0x00728357, 0x00728357, 0x00718256, 0x00718256, 0x00718256, 0x00718256, 0x0074875C, 0x006F8257, 0x006B7E53, 0x00728259, 0x00718158, 0x0075835A, 0x0073805A, 0x0074815B, 0x006E8057,
        0x0075875E, 0x0075875E, 0x00708259, 0x00708259, 0x00708259, 0x0071835A, 0x0076885F, 0x0076845B, 0x00728259, 0x0074845B, 0x0075855C, 0x0071835A, 0x0071835A, 0x0072865C, 0x0074885E, 0x0074875C, 0x0072855A, 0x00718459, 0x00708358, 0x00708358,
        0x00718459, 0x0073865B, 0x0074875C, 0x0073865B, 0x0073865B, 0x0072855A, 0x0072855A, 0x0072855A, 0x0072855A, 0x0072855A, 0x0072855A, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0075875E,
        0x0074865D, 0x0073855C, 0x0076885F, 0x00778960, 0x0073855C, 0x0072845B, 0x0075875E, 0x0078885F, 0x0073835A, 0x00728259, 0x00728259, 0x00798960, 0x0073835A, 0x0078885F, 0x0075855C, 0x0073875D, 0x00768A60, 0x0072845B, 0x00708259, 0x00708259,
        0x0075855C, 0x0074845B, 0x0077875E, 0x0075875E, 0x0075875E, 0x0075875E, 0x0075875E, 0x0075875E, 0x0074865D, 0x0074865D, 0x0074865D, 0x0072845B, 0x0073855C, 0x0073855C, 0x0073855C, 0x0074865D, 0x0074865D, 0x0074865D, 0x0074865D, 0x0074875C,
        0x0074875C, 0x0073875D, 0x0074885E, 0x00748962, 0x00758A63, 0x00758A63, 0x00738A64, 0x00748969, 0x00758A69, 0x007C8F6E, 0x00738462, 0x00778765, 0x00818F6B, 0x00818E68, 0x007E8B65, 0x0076855F, 0x0085936F, 0x007F8D6B, 0x00717D5F, 0x006B765C,
        0x00636C57, 0x00868F7B, 0x006B7362, 0x0079826D, 0x00565F4A, 0x004E5841, 0x00556145, 0x00515E3E, 0x006F7D5B, 0x007B8965, 0x007E8D67, 0x00798D60, 0x00778962, 0x00798967, 0x007B8965, 0x006A7953, 0xFF889274, 0xFFFFFFF8, 0xFFFFFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFAFF, 0xFBFCFFF4, 0x00737F5B, 0x00738354, 0x00819064, 0x00808D67, 0x0078865D, 0x007D8C60, 0x007A8A61, 0x007A8A61, 0x00798960, 0x007A8A61, 0x0078885F,
        0x007D8D64, 0x007F8F66, 0x00708057, 0x00748056, 0x00828E64, 0x007B875D, 0x007B875D, 0x00849066, 0x00818D63, 0x00808C62, 0x00808C62, 0x00808C62, 0x00818D63, 0x00808C62, 0x007E8A60, 0x007A865C, 0x00758157, 0x007B875D, 0x007C885E, 0x007D8C60,
        0x007D8C60, 0x007C8B5F, 0x007C8B5F, 0x007C8B5F, 0x007C8B5F, 0x007B8A5E, 0x007B8A5E, 0x0079895A, 0x0079895A, 0x00788859, 0x00788858, 0x00778757, 0x00768656, 0x00758653, 0x00758653, 0x00778757, 0x00778757, 0x00768656, 0x00758555, 0x00758555,
        0x00768656, 0x00778757, 0x00778757, 0x00758555, 0x00738353, 0x00738354, 0x00748455, 0x0079895A, 0x0079895A, 0x00728253, 0x00758458, 0x00768559, 0x00728155, 0x0077865A, 0x00768559, 0x0078875B, 0x007B8A5E, 0x00758458, 0x00758458, 0x007A895D,
        0x0078875B, 0x0078875B, 0x0077865A, 0x007B8A5E, 0x0077865A, 0x007E8D61, 0x0079885C, 0x00748454, 0x00758555, 0x00768656, 0x00778758, 0x00788859, 0x0078875B, 0x0078875B, 0x0078875B, 0x00778757, 0x00768656, 0x00768656, 0x00758555, 0x00758555,
        0x00768656, 0x00768656, 0x00778757, 0x00738458, 0x00738458, 0x00738458, 0x00748559, 0x00748559, 0x00748559, 0x0075865A, 0x0075865A, 0x0076895E, 0x0075885D, 0x0075885D, 0x007D8D64, 0x0075855C, 0x007A885F, 0x007B8862, 0x007F8C66, 0x00798960,
        0x007D8D64, 0x007F8F66, 0x007E8E65, 0x007C8C63, 0x007A8A61, 0x007A8A61, 0x007A8A61, 0x007C8A61, 0x007B8960, 0x007C8A61, 0x0076865D, 0x0076865D, 0x0074865D, 0x00708259, 0x0075875E, 0x0074875C, 0x0073865B, 0x0072855A, 0x00708358, 0x00708358,
        0x00718459, 0x0072855A, 0x0073865B, 0x0072855A, 0x0072855A, 0x0072855A, 0x0072855A, 0x0072855A, 0x0072855A, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0074875C, 0x0074875C, 0x0074875C, 0x0074875C, 0x0074875C, 0x0074865D,
        0x00798B62, 0x0074865D, 0x0075875E, 0x007A8C63, 0x00788A61, 0x00778960, 0x0076885F, 0x00798B62, 0x00798B62, 0x00788A61, 0x0072845B, 0x0075875E, 0x006E8057, 0x00798B62, 0x00798B62, 0x0072855E, 0x00768962, 0x0074865F, 0x00768861, 0x00708259,
        0x0076865D, 0x0075855C, 0x00798960, 0x0076885F, 0x0076885F, 0x0075875E, 0x0075875E, 0x0075875E, 0x0075875E, 0x0075875E, 0x0075875E, 0x0076885F, 0x0075875E, 0x0074865D, 0x0073855C, 0x0072845B, 0x0072845B, 0x0072845B, 0x0072845B, 0x0074885B,
        0x0074885B, 0x0074885B, 0x0074885D, 0x00768A5F, 0x00758C60, 0x00768C62, 0x00778D63, 0x007B9069, 0x007A8F68, 0x007E916A, 0x007F916A, 0x007B8D66, 0x0083936A, 0x007E8C63, 0x00818F66, 0x0086976C, 0x00829368, 0x007C8B65, 0x0075835F, 0x00778464,
        0x006E7A5C, 0x008A957B, 0x00707B61, 0x00768468, 0x006B7A5B, 0x00687757, 0x006E7E5C, 0x00667752, 0x0074865F, 0x0076885F, 0x0076885F, 0x00798D60, 0x00778962, 0x00798967, 0x007B8965, 0x006A7953, 0xFF889274, 0xFFFFFFF8, 0xFFFFFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFCFF, 0xFBFCFFF4, 0x00727E5A, 0x00728253, 0x007E8D61, 0x007F8C66, 0x007C8A61, 0x00829165, 0x0077875E, 0x0077875E, 0x0077875E, 0x007C8C63, 0x007A8A61,
        0x007C8C63, 0x0073835A, 0x004D5D34, 0x003B471D, 0x00404C22, 0x00404C22, 0x00414D23, 0x003F4B21, 0x003F4B21, 0x00445026, 0x00404C22, 0x003F4B21, 0x003D491F, 0x00434F25, 0x00445026, 0x004B572D, 0x00404C22, 0x00404C22, 0x004D592F, 0x007C8B5F,
        0x007C8B5F, 0x007C8B5F, 0x007C8B5F, 0x007C8B5F, 0x007B8A5E, 0x007B8A5E, 0x007B8A5E, 0x0079895A, 0x0079895A, 0x00788859, 0x00788858, 0x00788858, 0x00788858, 0x00788956, 0x00788956, 0x00778757, 0x00778757, 0x00768656, 0x00758555, 0x00758555,
        0x00768656, 0x00778757, 0x00778757, 0x007C8C5C, 0x00798959, 0x00788858, 0x00788859, 0x007A895D, 0x00637246, 0x00404E24, 0x003D4B21, 0x003F4E22, 0x00425125, 0x003D4C20, 0x003A491D, 0x003F4E22, 0x00435226, 0x00425125, 0x00445327, 0x00455428,
        0x00445327, 0x00455428, 0x003E4D21, 0x003E4D21, 0x00445327, 0x006D7C50, 0x007A895D, 0x00748454, 0x00758555, 0x00768656, 0x00778758, 0x00788859, 0x0078875B, 0x0079885C, 0x0078875B, 0x00778757, 0x00778757, 0x00768656, 0x00768656, 0x00768656,
        0x00768656, 0x00778757, 0x00778757, 0x00748559, 0x0075865A, 0x0075865A, 0x0075865A, 0x0076875B, 0x0076875B, 0x0077885C, 0x0077885C, 0x0073865B, 0x0073865B, 0x0072855A, 0x006F7F56, 0x004C5C33, 0x004A582F, 0x00495630, 0x00495630, 0x004E5E35,
        0x0045552C, 0x0044542B, 0x00495930, 0x004B5B32, 0x004C5C33, 0x004B5B32, 0x0047572E, 0x004D5830, 0x004A582F, 0x0049572E, 0x0048562D, 0x005E6E45, 0x0076865D, 0x0072845B, 0x0075875E, 0x0073865B, 0x0073865B, 0x0073865B, 0x0072855A, 0x0072855A,
        0x0072855A, 0x0072855A, 0x0072855A, 0x0072855A, 0x0072855A, 0x0072855A, 0x0072855A, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0074875C, 0x0074875C, 0x0074875C, 0x0075885D, 0x0075885D, 0x0073855E,
        0x00768861, 0x005A6C45, 0x00495B34, 0x004D5F38, 0x004D5F38, 0x004D5F38, 0x004B5D36, 0x004A5C35, 0x004F613A, 0x0050623B, 0x004D5F38, 0x004A5C35, 0x00495B34, 0x004B5D36, 0x00495B34, 0x00425331, 0x00485A35, 0x005B6C47, 0x00778962, 0x0073855E,
        0x0078885F, 0x0074855A, 0x0077885D, 0x0075875E, 0x0075875E, 0x0075875E, 0x0075875E, 0x0075875E, 0x0075875E, 0x0076885F, 0x0076885F, 0x00778960, 0x0076885F, 0x0075875E, 0x0073855C, 0x0073855C, 0x0073855C, 0x0074865D, 0x0075875E, 0x0077895A,
        0x0077895A, 0x00768A5D, 0x00778B5E, 0x00768A5F, 0x00778B60, 0x00788C61, 0x00768D61, 0x0063764B, 0x0055683D, 0x0056693E, 0x005E7047, 0x005B6B42, 0x005C6C43, 0x0057673E, 0x005D6D44, 0x00627347, 0x005D6E42, 0x00637449, 0x00607047, 0x005D6B47,
        0x00586644, 0x006A7757, 0x00657252, 0x005D6D4B, 0x007D8D6B, 0x0083946F, 0x00859671, 0x007E9069, 0x0081936A, 0x007B8D64, 0x007E9166, 0x00798D60, 0x00778962, 0x00798967, 0x007B8965, 0x006A7953, 0xFF889274, 0xFFFFFFF8, 0xFFFFFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFCFF, 0xF7F9FDF1, 0x00707C58, 0x00708051, 0x007A895D, 0x007A8761, 0x0077855C, 0x007B8A5E, 0x007D8B61, 0x007C8A60, 0x0078865C, 0x007D8B61, 0x007B895F,
        0x00808E64, 0x00748258, 0x003F4D23, 0x0039461A, 0x00364317, 0x003E4B1F, 0x00435024, 0x00414E22, 0x004B582C, 0x00515E32, 0x00424F23, 0x00455226, 0x003B481C, 0x003E4B1F, 0x00323F13, 0x00455226, 0x00323F13, 0x00334014, 0x00505D31, 0x0079885C,
        0x0079885C, 0x0079885C, 0x0079885C, 0x0079885C, 0x0078875B, 0x0078875B, 0x0078875B, 0x00788859, 0x00788859, 0x00788859, 0x00788858, 0x00788858, 0x00788858, 0x00788956, 0x00788956, 0x00778757, 0x00768656, 0x00768656, 0x00768656, 0x00768656,
        0x00768656, 0x00768656, 0x00778757, 0x00748552, 0x00738451, 0x00748454, 0x00778758, 0x007E8D61, 0x005B693F, 0x002B3910, 0x0033401A, 0x002D3B11, 0x0036441A, 0x00334117, 0x003B491F, 0x003B491F, 0x00334117, 0x0037451B, 0x002F3D13, 0x00303E14,
        0x00313F15, 0x003D4B21, 0x003A481E, 0x00303E14, 0x002B390F, 0x00637147, 0x007B895F, 0x00748454, 0x00758555, 0x00768656, 0x00778758, 0x00788859, 0x0078875B, 0x0078875B, 0x0078875B, 0x00788858, 0x00778757, 0x00778757, 0x00768656, 0x00768656,
        0x00778757, 0x00778757, 0x00788858, 0x00748559, 0x00748559, 0x0075865A, 0x0075865A, 0x0075865A, 0x0076875B, 0x0076875B, 0x0076875B, 0x007A8D62, 0x00798C61, 0x00798C61, 0x006C7C53, 0x0033431A, 0x00313F16, 0x0035421C, 0x002E3B15, 0x0035431A,
        0x0039471E, 0x003D4B22, 0x003C4A21, 0x0037451C, 0x00334118, 0x00334118, 0x0035431A, 0x0037421A, 0x0037421A, 0x002B3910, 0x00233108, 0x004C5A31, 0x0078885F, 0x0077875E, 0x0074845B, 0x0072855A, 0x0073865B, 0x0074875C, 0x0074875C, 0x0074875C,
        0x0074875C, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0074875C, 0x0074875C, 0x0075885D, 0x0075885D, 0x0076895E, 0x00778962,
        0x0073855E, 0x003D4F28, 0x001F310A, 0x00293B14, 0x00293B14, 0x00293B14, 0x002D3F18, 0x002C3E17, 0x00293B14, 0x00253710, 0x002A3C15, 0x0024360F, 0x0031431C, 0x00263811, 0x002B3D16, 0x00253615, 0x001F300F, 0x003C4C2A, 0x0071825D, 0x0070825B,
        0x0076865D, 0x0075865B, 0x0077885C, 0x0074865D, 0x0074865D, 0x0074865D, 0x0075875E, 0x0075875E, 0x0075875E, 0x0075875E, 0x0075875E, 0x0074865D, 0x0074865D, 0x0074865D, 0x0074865D, 0x0075875E, 0x0076885F, 0x00778960, 0x00788A61, 0x0077885C,
        0x00798A5E, 0x007A8B5F, 0x00788B60, 0x00788B60, 0x00768A60, 0x00778B61, 0x00788C62, 0x004B5C30, 0x002B3C11, 0x002A3B10, 0x002C3C13, 0x00293910, 0x002C3B15, 0x0032401C, 0x0034421E, 0x00304115, 0x002F4015, 0x00304116, 0x00314118, 0x00293812,
        0x002C3A16, 0x002A3814, 0x00293715, 0x00374823, 0x0072835E, 0x007B8C67, 0x00788A63, 0x0074865F, 0x00758760, 0x0073855E, 0x00798B62, 0x00798D60, 0x00778962, 0x00798967, 0x007B8965, 0x006A7953, 0xFF889274, 0xFFFFFFF8, 0xFFFFFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFAFF, 0xF4F7FBEF, 0x00727E5A, 0x00758556, 0x00808F63, 0x00818E68, 0x007C8A61, 0x007D8C60, 0x007C8A60, 0x007C8A60, 0x0077855B, 0x007A885E, 0x0077855B,
        0x00808E64, 0x0079875D, 0x003E4C22, 0x0049562A, 0x005F6C40, 0x005A673B, 0x00525F33, 0x00667347, 0x006F7C50, 0x005F6C40, 0x0059663A, 0x00505D31, 0x004C592D, 0x005B683C, 0x00475428, 0x0069764A, 0x00445125, 0x00354216, 0x004D5A2E, 0x0079885C,
        0x0079885C, 0x0078875B, 0x0078875B, 0x0078875B, 0x0078875B, 0x0077865A, 0x0077865A, 0x00778758, 0x00778758, 0x00778758, 0x00778757, 0x00768656, 0x00768656, 0x00758653, 0x00758653, 0x00768656, 0x00768656, 0x00768656, 0x00778757, 0x00778757,
        0x00768656, 0x00768656, 0x00768656, 0x00778954, 0x00778855, 0x00738353, 0x00748455, 0x007F8D63, 0x0059673E, 0x002D3A14, 0x004C5834, 0x004C5A31, 0x004F5D34, 0x00536138, 0x00707E55, 0x00637148, 0x004B5930, 0x0057653C, 0x00435128, 0x00556339,
        0x0049572D, 0x0058663C, 0x00606E44, 0x00505E34, 0x00334117, 0x00606E44, 0x0076845A, 0x00748454, 0x00758555, 0x00768656, 0x00778758, 0x00788859, 0x0078875B, 0x0078875B, 0x0078875B, 0x00788858, 0x00788858, 0x00778757, 0x00778757, 0x00778757,
        0x00778757, 0x00788858, 0x00788858, 0x0075865A, 0x0075865A, 0x0075865A, 0x0076875B, 0x0076875B, 0x0076875B, 0x0076875B, 0x0076875B, 0x0073865B, 0x0072855A, 0x00788B60, 0x006F7F56, 0x0033431A, 0x00435128, 0x0056633D, 0x004D5A34, 0x0049572E,
        0x00637148, 0x00627047, 0x00515F36, 0x004F5D34, 0x004A582F, 0x00435128, 0x004B5930, 0x005D6840, 0x00667149, 0x00566139, 0x00344219, 0x00515F36, 0x007D8D64, 0x0077875E, 0x00708057, 0x0073865B, 0x0073865B, 0x0074875C, 0x0075885D, 0x0075885D,
        0x0075885D, 0x0075885D, 0x0075885D, 0x0075885D, 0x0075885D, 0x0075885D, 0x0074875C, 0x0074875C, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0074875C, 0x0075885D, 0x0075885D, 0x0076895E, 0x0076895E, 0x007B8C67,
        0x00788964, 0x003E4F2A, 0x002D3E19, 0x00495A35, 0x00455631, 0x0041522D, 0x004C5D38, 0x0060724D, 0x0050623D, 0x0042542F, 0x004A5C37, 0x0031431E, 0x0040522D, 0x00243611, 0x00384A25, 0x004A5A3B, 0x00273718, 0x00394828, 0x00778863, 0x0070825B,
        0x0075855C, 0x0076875C, 0x0076875B, 0x0072845B, 0x0073855C, 0x0074865D, 0x0075875E, 0x0076885F, 0x0075875E, 0x0075875E, 0x0074865D, 0x0073855C, 0x0073855C, 0x0073855C, 0x0074865D, 0x0075875E, 0x0076885F, 0x00778960, 0x00788A61, 0x007A885E,
        0x00798A5F, 0x007A8A61, 0x007A8A61, 0x00778962, 0x00778962, 0x00788A65, 0x007A8C67, 0x00546036, 0x00344218, 0x0045532A, 0x0047542E, 0x00475531, 0x004D5B39, 0x005F6F4D, 0x00576646, 0x00425229, 0x004D5D34, 0x0037471E, 0x00485731, 0x004C5B35,
        0x0062704C, 0x0055633F, 0x0044522E, 0x00334520, 0x0071835E, 0x00798B66, 0x00788A65, 0x00738560, 0x00748661, 0x00778964, 0x00788A65, 0x00798D60, 0x00778962, 0x00798967, 0x007B8965, 0x006A7953, 0xFF889274, 0xFFFFFFF8, 0xFFFFFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFCFF, 0xF7FAFEF2, 0x00727E5A, 0x00738354, 0x007D8C60, 0x007F8C66, 0x007B8960, 0x007C8B5F, 0x0078865C, 0x007D8B61, 0x007C8A60, 0x00808E64, 0x007A885E,
        0x00818F65, 0x0079875D, 0x003A481E, 0x00475428, 0x007D8A5E, 0x005D6A3E, 0x003A471B, 0x00657246, 0x00616E42, 0x00384519, 0x004A572B, 0x006E7B4F, 0x005D6A3E, 0x005E6B3F, 0x003D4A1E, 0x00707D51, 0x004C592D, 0x00374418, 0x0049562A, 0x007A895D,
        0x007A895D, 0x007A895D, 0x007A895D, 0x007A895D, 0x0079885C, 0x0079885C, 0x0079885C, 0x00778758, 0x00778758, 0x00768657, 0x00758555, 0x00748454, 0x00738353, 0x00738451, 0x00728350, 0x00758555, 0x00768656, 0x00778757, 0x00778757, 0x00778757,
        0x00778757, 0x00768656, 0x00758555, 0x00778954, 0x007A8B58, 0x00748454, 0x00738256, 0x00808E64, 0x0058653F, 0x00313D19, 0x00606C48, 0x0067754C, 0x00505E35, 0x00425027, 0x005E6C43, 0x00445229, 0x00324017, 0x005F6D44, 0x00536138, 0x00657349,
        0x00435127, 0x0049572D, 0x005E6C42, 0x0058663C, 0x0036441A, 0x00647248, 0x007B895F, 0x00748454, 0x00748454, 0x00758555, 0x00778758, 0x00778758, 0x0078875B, 0x0078875B, 0x0078875B, 0x00788858, 0x00788858, 0x00778757, 0x00778757, 0x00778757,
        0x00778757, 0x00788858, 0x00788858, 0x0078895D, 0x0078895D, 0x0078895D, 0x0078895D, 0x0078895D, 0x0078895D, 0x0078895D, 0x0078895D, 0x00778A5F, 0x0072855A, 0x00798C61, 0x00708057, 0x0034441B, 0x00505E35, 0x006C7953, 0x00606D47, 0x00414F26,
        0x00606E45, 0x004D5B32, 0x0037451C, 0x00505E35, 0x005C6A41, 0x00515F36, 0x0058663D, 0x00485129, 0x00626D45, 0x005F6A42, 0x0037421A, 0x004B5930, 0x007B8960, 0x00798960, 0x0077875E, 0x0073865B, 0x0074875C, 0x0074875C, 0x0075885D, 0x0075885D,
        0x0076895E, 0x00778A5F, 0x00778A5F, 0x0076895E, 0x0076895E, 0x0075885D, 0x0075885D, 0x0074875C, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0074875C, 0x0075885D, 0x0076895E, 0x0076895E, 0x0076895E, 0x0073845F,
        0x00748560, 0x0040512C, 0x0040512C, 0x00677853, 0x00576843, 0x00465732, 0x0053643F, 0x004D5F3A, 0x003F512C, 0x003F512C, 0x005A6C47, 0x0040522D, 0x00556742, 0x00374924, 0x005D6F4A, 0x005F6E52, 0x0029391A, 0x00334222, 0x00778765, 0x0070825B,
        0x0075855C, 0x0075865A, 0x00748559, 0x0071835A, 0x0072845B, 0x0074865D, 0x0075875E, 0x0076885F, 0x0075875E, 0x0074865D, 0x0073855C, 0x0075875E, 0x0074865D, 0x0074865D, 0x0074865D, 0x0074865D, 0x0074865D, 0x0074865D, 0x0074865D, 0x007A885F,
        0x007B8862, 0x00798862, 0x00798862, 0x00768762, 0x00788964, 0x007B8B69, 0x007D8E6C, 0x004F5B31, 0x00404B23, 0x0066744B, 0x0063704A, 0x005B6947, 0x00525F3F, 0x005E6D4E, 0x00455435, 0x0042512B, 0x0064734D, 0x0045542E, 0x00606F49, 0x0062704C,
        0x0073815D, 0x0055633F, 0x002C3A16, 0x00354722, 0x006D7F5A, 0x00768863, 0x007C8D6B, 0x00768765, 0x00778866, 0x007A8B69, 0x00738462, 0x00798D60, 0x00778962, 0x00798967, 0x007B8965, 0x006A7953, 0xFF889274, 0xFFFFFFF8, 0xFFFFFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xF0F7F9ED, 0x006D7753, 0x00738354, 0x007C8B5F, 0x007A8A61, 0x007E8E65, 0x007D8E62, 0x00788D67, 0x00768A60, 0x00798A5E, 0x007F8F60, 0x007E8E5E,
        0x007A8C5D, 0x00768B5E, 0x00364D21, 0x004D582A, 0x007A8557, 0x005D683A, 0x00384315, 0x00697446, 0x00626D3F, 0x003A4517, 0x00384315, 0x005C6638, 0x00737D4F, 0x003C4618, 0x00353F11, 0x00646E40, 0x004D5729, 0x00323C0E, 0x004D5729, 0x00798B62,
        0x00758760, 0x00778660, 0x007A8963, 0x007A885E, 0x00798658, 0x007B8A57, 0x0080905B, 0x00768657, 0x00768657, 0x00768657, 0x00768657, 0x00768657, 0x00758556, 0x00748455, 0x00748455, 0x00738556, 0x00738556, 0x00748657, 0x00758758, 0x00758758,
        0x00768859, 0x00768859, 0x00768859, 0x006F8356, 0x00708358, 0x0075885D, 0x0071855B, 0x007B8F65, 0x00596C45, 0x002A3F19, 0x00566B45, 0x00677E51, 0x003A5124, 0x003A5124, 0x00647B4E, 0x003A5124, 0x00233A0D, 0x00374E21, 0x00667D50, 0x00445A30,
        0x00293E17, 0x004A5F39, 0x00516542, 0x00394E28, 0x002A3F18, 0x005E7548, 0x00758D5D, 0x00738A56, 0x00708656, 0x006F8158, 0x0071835C, 0x0075845E, 0x0076845A, 0x00758555, 0x00788B54, 0x00738655, 0x00738655, 0x00738655, 0x00738556, 0x00748657,
        0x00758758, 0x0075865A, 0x0076875B, 0x0077885C, 0x00798A5E, 0x007A8B5F, 0x00798A5E, 0x0078895D, 0x0076875B, 0x0077885C, 0x007A8B5F, 0x006E8160, 0x00778C66, 0x00798E67, 0x0070875B, 0x002E4518, 0x00485F32, 0x006B8255, 0x00465D31, 0x00364A1D,
        0x00627649, 0x004C6033, 0x00293D10, 0x00304417, 0x00627649, 0x00617548, 0x00384C1F, 0x0033451C, 0x0051633A, 0x00687B50, 0x002F4217, 0x004C5E35, 0x007F906B, 0x006F7E5E, 0x00778667, 0x0078895E, 0x007A8B60, 0x0077885D, 0x0077885D, 0x007B8C61,
        0x00798A5F, 0x0076875C, 0x0078895E, 0x00778960, 0x0076885F, 0x0075875E, 0x0074865D, 0x0074865D, 0x0074865D, 0x0075875E, 0x0075875E, 0x0079895A, 0x00738458, 0x00738459, 0x0074865D, 0x0074865F, 0x00738462, 0x00728564, 0x00738665, 0x0070855E,
        0x007C916B, 0x003A4E2B, 0x00354827, 0x00728663, 0x004C613B, 0x0022380E, 0x00536A3D, 0x00566E46, 0x002A421A, 0x00243C14, 0x00566E46, 0x00627A52, 0x002F471F, 0x00526A42, 0x005D754D, 0x00627250, 0x0023340F, 0x00364821, 0x0076885F, 0x00798B62,
        0x006D7E59, 0x00758464, 0x00748266, 0x00758760, 0x0074865F, 0x0073855E, 0x0072845D, 0x0073855E, 0x0073855E, 0x0073855E, 0x0073855E, 0x00758760, 0x00758760, 0x00768861, 0x00768861, 0x00778962, 0x00768861, 0x00768861, 0x00768861, 0x0078885F,
        0x00778660, 0x00788662, 0x00788866, 0x00788968, 0x00768765, 0x00768B65, 0x007B9069, 0x004F6334, 0x00344819, 0x00738758, 0x00627647, 0x00334718, 0x004D6132, 0x00708455, 0x00384C1D, 0x00354117, 0x00535F35, 0x007D895F, 0x004A562C, 0x004D592F,
        0x00626E44, 0x0068744A, 0x003F4B21, 0x002D4417, 0x006E845A, 0x00788C69, 0x00768869, 0x007D8F70, 0x00758966, 0x00768C62, 0x0072895C, 0x007A8D62, 0x007A8B66, 0x00788767, 0x007B8967, 0x006A7953, 0xFF869171, 0xFFFFFFF7, 0xFFFCFBFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xECF6F8EC, 0x006C7652, 0x00728253, 0x007B8A5E, 0x00788761, 0x007B8B62, 0x00798A5E, 0x00748661, 0x0073855C, 0x00768657, 0x0073824F, 0x006C7A4A,
        0x00637246, 0x005E7049, 0x002B3C1B, 0x00324322, 0x004C5D3C, 0x004A5B3A, 0x00394A29, 0x00506140, 0x004F603F, 0x003C4D2C, 0x002D3E1D, 0x00344226, 0x0049573B, 0x002C3A1E, 0x00344226, 0x00566448, 0x00404E32, 0x0029371B, 0x00334125, 0x00647549,
        0x0064754A, 0x0067784D, 0x006D7B51, 0x00728155, 0x00768355, 0x00758353, 0x0073824F, 0x00758556, 0x00758556, 0x00768657, 0x00768657, 0x00768657, 0x00768657, 0x00758556, 0x00758556, 0x00738556, 0x00738556, 0x00738556, 0x00738556, 0x00738556,
        0x00738556, 0x00728455, 0x00728455, 0x00768956, 0x006D804F, 0x00697D50, 0x005F7148, 0x0063744F, 0x00475837, 0x00213014, 0x00435236, 0x0049583C, 0x003F4E32, 0x00354428, 0x004D5C40, 0x003E4D31, 0x002A391D, 0x00243317, 0x004A593D, 0x002A3A1B,
        0x00334226, 0x00556448, 0x00455438, 0x002B3B1C, 0x00213210, 0x004E6238, 0x0063784B, 0x00647B44, 0x00667C48, 0x006B7F52, 0x00708057, 0x00718158, 0x00717F55, 0x00718151, 0x00728350, 0x00728554, 0x00728554, 0x00728554, 0x00738556, 0x00738556,
        0x00748559, 0x0075865A, 0x0075865A, 0x00748559, 0x0075865A, 0x0076875B, 0x0076875B, 0x0076875B, 0x0075865A, 0x00748559, 0x00738458, 0x0073875C, 0x0070845A, 0x006B7E57, 0x00637550, 0x002D3E1C, 0x00415230, 0x005D6E4C, 0x00495A38, 0x00374529,
        0x00566448, 0x004B593D, 0x00313F23, 0x00253317, 0x00404E32, 0x00435135, 0x00273519, 0x004F5E3E, 0x00546343, 0x00556444, 0x00253414, 0x00425230, 0x0072835E, 0x006C7E57, 0x00778960, 0x0075855C, 0x0078885F, 0x0076865D, 0x0076865D, 0x007A8A61,
        0x0078885F, 0x0074845B, 0x0077875E, 0x0076885F, 0x0076885F, 0x0075875E, 0x0074865D, 0x0074865D, 0x0074865D, 0x0074865D, 0x0074865D, 0x0077885C, 0x0075865A, 0x0075865B, 0x0075885D, 0x0076885F, 0x0073875D, 0x0070835C, 0x006E815A, 0x0070835C,
        0x006E7F5D, 0x00354526, 0x00324125, 0x00657459, 0x00556448, 0x00384928, 0x0050613F, 0x00495B3E, 0x002E4023, 0x001C2E11, 0x003A4C2F, 0x004C5E41, 0x00283A1D, 0x004A5C3F, 0x0057694C, 0x005F6C54, 0x002B391D, 0x00354424, 0x00697B54, 0x0076885F,
        0x0072845B, 0x00768861, 0x0071825D, 0x0072845D, 0x0072845D, 0x0072845D, 0x0072845D, 0x0073855E, 0x0073855E, 0x0073855E, 0x0073855E, 0x0074865F, 0x00758760, 0x00758760, 0x00768861, 0x00768861, 0x00768861, 0x00758760, 0x00758760, 0x00788761,
        0x00778660, 0x00768460, 0x00768762, 0x00758760, 0x0072865C, 0x0074885D, 0x00788C61, 0x00445333, 0x00303F1F, 0x00627151, 0x005A6949, 0x00425131, 0x004C5B3B, 0x005C6B4B, 0x00364525, 0x002F3B1D, 0x003C482A, 0x005C684A, 0x00354123, 0x003F4B2D,
        0x00546042, 0x005F6B4D, 0x00434F31, 0x0034481E, 0x006D8157, 0x00788A65, 0x00758664, 0x00798A68, 0x00748661, 0x00798D63, 0x00798D63, 0x00788B60, 0x00788964, 0x00778666, 0x007B8967, 0x006B7A54, 0xFF879272, 0xFFFFFFF7, 0xFFFEFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFBFF, 0xECF6F8EC, 0x006C7652, 0x00728253, 0x0079875D, 0x0075845E, 0x0076855F, 0x0074855A, 0x006C7D58, 0x006D7D54, 0x006B7949, 0x005C6935, 0x004F5B2B,
        0x00414C24, 0x003B472B, 0x0018230F, 0x00122514, 0x00172A19, 0x001F3221, 0x001A2D1C, 0x001B2E1D, 0x001D301F, 0x001D301F, 0x00102312, 0x0016261B, 0x001D2D22, 0x000F1F14, 0x0018281D, 0x0026362B, 0x001A2A1F, 0x0015251A, 0x0016261B, 0x00415121,
        0x00465626, 0x004C5C2C, 0x00546434, 0x00617142, 0x006C7C4D, 0x00738052, 0x00748153, 0x00748455, 0x00748455, 0x00758556, 0x00768657, 0x00768657, 0x00768657, 0x00768657, 0x00768657, 0x00738556, 0x00738556, 0x00728455, 0x00728455, 0x00718354,
        0x006F8152, 0x006E8051, 0x006D7F50, 0x00677A41, 0x00546730, 0x004E5E2E, 0x00425229, 0x00404D2D, 0x002D3A22, 0x00121C0B, 0x00212A1D, 0x00192019, 0x00282F28, 0x001C231C, 0x00252C25, 0x00282F28, 0x00232A23, 0x000F160F, 0x00222922, 0x00131D0D,
        0x001C2616, 0x002E3828, 0x00242E1D, 0x001F2B15, 0x00132105, 0x002E3E1C, 0x00455730, 0x00455B20, 0x004F652B, 0x005F723F, 0x006A7C4D, 0x006E7F53, 0x00728155, 0x00748357, 0x00758556, 0x00718453, 0x00718453, 0x00718354, 0x00728455, 0x00728357,
        0x00738458, 0x00738458, 0x0074855A, 0x00748559, 0x00738458, 0x00728357, 0x00738458, 0x00728357, 0x00708155, 0x006B7C50, 0x00647549, 0x00536A33, 0x00495D2E, 0x0042532E, 0x00414E34, 0x001C2713, 0x00222C1B, 0x00293420, 0x0024301A, 0x002A322B,
        0x00353D36, 0x00363E37, 0x002F3730, 0x001B231C, 0x00212922, 0x00272F28, 0x001B231C, 0x002E3A26, 0x002A3624, 0x00273222, 0x00111D0B, 0x002A3820, 0x004A5B39, 0x004C6232, 0x00566E38, 0x00697950, 0x006F7F56, 0x00708057, 0x00718158, 0x0076865D,
        0x0076865D, 0x0073835A, 0x0076865D, 0x0075875E, 0x0075875E, 0x0075875E, 0x0075875E, 0x0074865D, 0x0073855C, 0x0072845B, 0x0072845B, 0x0075855C, 0x0074855A, 0x00728358, 0x00708155, 0x006C8053, 0x00637748, 0x00576B3C, 0x00506435, 0x004B5B39,
        0x00435233, 0x00212E16, 0x0018230F, 0x002C3626, 0x002B3525, 0x00242E1E, 0x002A3423, 0x003A4535, 0x00364131, 0x001E2919, 0x001F2A1A, 0x002C3727, 0x000C1707, 0x00192414, 0x001C2717, 0x002D362C, 0x00182313, 0x0025331B, 0x00495A38, 0x00586C42,
        0x005A6F42, 0x00627848, 0x00697F4F, 0x006E8059, 0x006F815A, 0x0071835C, 0x0072845D, 0x0072845D, 0x0072845D, 0x0072845D, 0x0072845D, 0x0074865F, 0x0074865F, 0x00758760, 0x00758760, 0x00758760, 0x00758760, 0x0074865F, 0x0074865F, 0x0075835F,
        0x00717F5B, 0x006C7B55, 0x0066774C, 0x005C6D41, 0x00526637, 0x004F6433, 0x004E6332, 0x00313D2B, 0x001E2A18, 0x00323E2C, 0x002D3927, 0x00333F2D, 0x00394533, 0x00465240, 0x0044503E, 0x0025341F, 0x0022311C, 0x00374631, 0x0022311C, 0x002E3D28,
        0x00394833, 0x003E4D38, 0x002C3B26, 0x00253611, 0x004E6039, 0x005C6E45, 0x00607348, 0x00697C51, 0x006E8057, 0x00768861, 0x00758661, 0x0074875C, 0x00758661, 0x00768565, 0x007C8A68, 0x006C7B55, 0xFF879272, 0xFFFFFFF7, 0xFFFFFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFBFF, 0xF0F6F7ED, 0x006D7655, 0x00728155, 0x0079875D, 0x0074835D, 0x0074835D, 0x00708156, 0x00748260, 0x00717F56, 0x006A7646, 0x00515C2A, 0x00444E20,
        0x00363D1C, 0x002F3522, 0x001B2017, 0x00142216, 0x00152317, 0x00142216, 0x00111F13, 0x00132115, 0x00101E12, 0x00101E12, 0x00132115, 0x00111A17, 0x00121B18, 0x000D1613, 0x00151E1B, 0x0017201D, 0x00121B18, 0x00161F1C, 0x00161F1C, 0x00313F0F,
        0x00394815, 0x0041501D, 0x004D5C29, 0x005E6D3A, 0x00687849, 0x006F7E52, 0x00758359, 0x00738256, 0x00738256, 0x00748357, 0x00768559, 0x0077865A, 0x0077865A, 0x0077865A, 0x0077865A, 0x00748559, 0x00748559, 0x00738458, 0x00728357, 0x00708155,
        0x006D7E52, 0x006A7B4F, 0x00697A4E, 0x0066783B, 0x004D5E25, 0x00414F1F, 0x0033401A, 0x002B341A, 0x0022291A, 0x00141812, 0x00151816, 0x00111210, 0x00191A18, 0x00141513, 0x00191A18, 0x00171816, 0x0020211F, 0x00171816, 0x00171816, 0x001B1F1A,
        0x00131712, 0x00151914, 0x0013180F, 0x001E2518, 0x00141D08, 0x001F2B0F, 0x002E3C1A, 0x00374C0E, 0x0043561D, 0x00586B34, 0x00687946, 0x006B7B4C, 0x006B7C50, 0x006E7F54, 0x006D7E53, 0x00708253, 0x00708253, 0x00718354, 0x00728357, 0x00728357,
        0x00728357, 0x00728358, 0x00728358, 0x0077885D, 0x00728358, 0x00718257, 0x00708156, 0x006E7F54, 0x006B7C51, 0x00617247, 0x00536439, 0x00435A1C, 0x00374A17, 0x002B3917, 0x00293120, 0x00161A14, 0x00171B16, 0x0011150F, 0x00151B10, 0x00141B18,
        0x00111815, 0x00131A17, 0x001A211E, 0x00141B18, 0x00111815, 0x00131A17, 0x00121916, 0x000F180E, 0x00101811, 0x00141B16, 0x00101811, 0x001D2917, 0x002F401F, 0x00374D1C, 0x00425D1E, 0x00596940, 0x0063734A, 0x0068784F, 0x006B7B52, 0x00718158,
        0x0074845B, 0x0074845B, 0x0077875E, 0x0074865D, 0x0075875E, 0x0075875E, 0x0075875E, 0x0074865D, 0x0073855C, 0x0071835A, 0x00708259, 0x006E8059, 0x006F8158, 0x006D7E53, 0x00697A4E, 0x00637546, 0x00536633, 0x003F531E, 0x00364813, 0x00323E20,
        0x002B361C, 0x001E2713, 0x0013190E, 0x000F130D, 0x00121611, 0x00161A15, 0x00161A15, 0x00121610, 0x001E221C, 0x00141812, 0x00141812, 0x00232721, 0x00121610, 0x00181C16, 0x00141812, 0x0012171A, 0x00121914, 0x001C2717, 0x002E3D21, 0x003A4E24,
        0x00405626, 0x00526935, 0x0069814B, 0x006A7C55, 0x006C7E57, 0x006F815A, 0x0072845D, 0x0072845D, 0x0072845D, 0x0072845D, 0x0072845D, 0x0073855E, 0x0073855E, 0x0074865F, 0x0074865F, 0x0074865F, 0x0073855E, 0x0073855E, 0x0072845D, 0x00717F5D,
        0x006B7955, 0x0065764B, 0x005A6C3D, 0x004C5D2A, 0x0040511E, 0x003C4D1A, 0x00384916, 0x001F291C, 0x00121C0F, 0x00151F12, 0x000F190C, 0x001C2619, 0x00162013, 0x00151F12, 0x001F291C, 0x00192917, 0x0010200E, 0x001D2D1B, 0x00132311, 0x001B2B19,
        0x001D2D1B, 0x001C2C1A, 0x00122210, 0x00222F0F, 0x00364420, 0x00425328, 0x004D5F30, 0x00607243, 0x006E7F54, 0x0073815D, 0x006E7B5B, 0x0074875C, 0x00748560, 0x00768565, 0x007D8B69, 0x006D7C56, 0xFF869171, 0xFFFFFFF7, 0xFFFFFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFBFF, 0xF0F7F8EE, 0x006E7756, 0x00748357, 0x007B8960, 0x0074825E, 0x0074825E, 0x00708057, 0x0075805F, 0x006A724D, 0x005F673E, 0x00484C22, 0x003E411A,
        0x002F3218, 0x00282919, 0x001B1E15, 0x00232616, 0x00232616, 0x00202313, 0x00242717, 0x00272A1A, 0x00222515, 0x001D2010, 0x00242717, 0x0026251B, 0x0027261C, 0x00242319, 0x0025241A, 0x00232218, 0x001E1D13, 0x00222117, 0x00242319, 0x00323A11,
        0x00353E13, 0x00384414, 0x00475722, 0x005F6E3B, 0x00687848, 0x006A784E, 0x006F7E58, 0x00728155, 0x00738256, 0x00748357, 0x00768559, 0x0077865A, 0x0077865A, 0x0078875B, 0x0078875B, 0x0076875B, 0x0076875B, 0x0075865A, 0x00738458, 0x00708155,
        0x006D7E52, 0x006A7B4F, 0x0068794D, 0x00606D37, 0x00495422, 0x00374117, 0x002B3211, 0x00202510, 0x00202216, 0x001E1D19, 0x00191919, 0x00222018, 0x001D1B13, 0x001C1A12, 0x00222018, 0x0018160E, 0x001D1B13, 0x001E1C14, 0x001E1C14, 0x001D1C18,
        0x001D1C18, 0x001F1E1A, 0x00161511, 0x001A1B12, 0x001C1F0F, 0x00252B14, 0x00242B10, 0x00394612, 0x00404D19, 0x0053632E, 0x00677643, 0x00697949, 0x006A7B4F, 0x006D7E53, 0x006A7C53, 0x006F8152, 0x00708253, 0x00718256, 0x00728357, 0x00738458,
        0x00738459, 0x00728358, 0x00728358, 0x0077885D, 0x00718257, 0x006F8055, 0x006F8055, 0x006C7D52, 0x00697A4F, 0x005D6E43, 0x004A5B30, 0x003C4D14, 0x00334012, 0x00242C0E, 0x001D1F13, 0x00181615, 0x001D1B1B, 0x00191716, 0x0021211B, 0x00191E15,
        0x00141910, 0x0013180F, 0x00161B12, 0x001D2219, 0x00181D14, 0x0013180F, 0x00191E15, 0x00171B15, 0x001C201B, 0x00191B1B, 0x00161917, 0x00161D10, 0x00212D0F, 0x002D3F10, 0x00384E14, 0x0052613B, 0x00606F49, 0x00687751, 0x006B7A54, 0x0071805A,
        0x0075845E, 0x0075845E, 0x00778660, 0x0073855C, 0x0074865D, 0x0074865D, 0x0074865D, 0x0074865D, 0x0073855C, 0x0071835A, 0x00708259, 0x006B7D56, 0x006E8059, 0x006B7D54, 0x006A7B50, 0x00657546, 0x00506030, 0x003C4B18, 0x00364611, 0x00293015,
        0x00222811, 0x001E2112, 0x001A1B12, 0x00191716, 0x001C1A1A, 0x001C1A1A, 0x001A1817, 0x00161511, 0x001B1A16, 0x00181713, 0x00161511, 0x001D1C18, 0x00171612, 0x001F1E1A, 0x00191814, 0x0016161C, 0x00141616, 0x00131A0D, 0x001C270D, 0x002B3915,
        0x0037481C, 0x004C5F2E, 0x00697C49, 0x00677952, 0x006B7D56, 0x006F815A, 0x0072845D, 0x0072845D, 0x0071835C, 0x0072845D, 0x0072845D, 0x0073855E, 0x0073855E, 0x0074865F, 0x0074865F, 0x0073855E, 0x0073855E, 0x0072845D, 0x0071835C, 0x00738060,
        0x006D7C56, 0x0067784C, 0x005A6B38, 0x00465621, 0x003B4A17, 0x003B4717, 0x00374214, 0x00222B16, 0x001F2813, 0x00222B16, 0x001E2712, 0x0027301B, 0x001E2712, 0x0018210C, 0x001E2712, 0x001C2D12, 0x0017280D, 0x001D2E13, 0x0017280D, 0x00192A0F,
        0x001C2D12, 0x001F3015, 0x001C2D12, 0x002B3315, 0x002F3817, 0x0038431B, 0x00445125, 0x0059663A, 0x006E7951, 0x00747D5C, 0x00788062, 0x0075885D, 0x00758661, 0x00778666, 0x007E8C6A, 0x006C7B55, 0xFF848F6F, 0xFFFFFFF7, 0xFFFFFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFBFF, 0xF4F7F8EF, 0x006E7657, 0x00748258, 0x007C8A61, 0x00768460, 0x00768460, 0x00728259, 0x00797F60, 0x00686D4D, 0x006C6D4D, 0x005F5D3F, 0x00605D41,
        0x005E5B46, 0x00575745, 0x00555547, 0x0064614C, 0x005D5A45, 0x005C5944, 0x00625F4A, 0x005B5843, 0x005C5944, 0x005C5944, 0x005C5944, 0x005B5542, 0x00605A47, 0x005E5845, 0x005D5744, 0x005C5643, 0x0058523F, 0x0058523F, 0x005D5744, 0x0053543A,
        0x004C4F2F, 0x003D451C, 0x00424E1E, 0x005A6936, 0x00687848, 0x0068794E, 0x006C7B55, 0x00728056, 0x00738157, 0x00748258, 0x0076845A, 0x0077855B, 0x0077855B, 0x0077855B, 0x0077855B, 0x0076875C, 0x0076875C, 0x0076875C, 0x0074855A, 0x00718257,
        0x006E7F54, 0x006A7B50, 0x0068794E, 0x00626740, 0x005D613D, 0x0057593B, 0x0055583E, 0x00535440, 0x00525244, 0x0057564C, 0x0054524A, 0x00595745, 0x00565442, 0x0053513F, 0x00585644, 0x00565442, 0x0052503E, 0x00504E3C, 0x005A5846, 0x00515143,
        0x004E4E42, 0x00525147, 0x00504E46, 0x004C4B41, 0x0049493B, 0x004D4F3B, 0x004B4E38, 0x004A4E2A, 0x00424A21, 0x00515B30, 0x00667444, 0x006B7B4B, 0x006C7D51, 0x006F8257, 0x006F8158, 0x00708253, 0x00708155, 0x00728357, 0x00738458, 0x0074855A,
        0x0074855A, 0x0073835A, 0x00728259, 0x0075855C, 0x006E7E55, 0x006E7E55, 0x006E7E55, 0x006C7C53, 0x006B7B52, 0x00607047, 0x00495930, 0x003E481E, 0x00424926, 0x003F432A, 0x00404032, 0x00413E39, 0x0044413C, 0x00403D38, 0x003B3931, 0x00414532,
        0x00404431, 0x003F4330, 0x00383C29, 0x00414532, 0x003D412E, 0x00393D2A, 0x00424633, 0x003A3B31, 0x00414239, 0x003E3E38, 0x003D3E35, 0x003C3F30, 0x0041472E, 0x00404A26, 0x00424F23, 0x0054633D, 0x0064734D, 0x006E7D57, 0x006F7E58, 0x0073825C,
        0x0076855F, 0x0075845E, 0x00778660, 0x0073855C, 0x0073855C, 0x0074865D, 0x0074865D, 0x0073855C, 0x0073855C, 0x0072845B, 0x0072845B, 0x006E815A, 0x006F825B, 0x006D7F56, 0x006C7C53, 0x00657349, 0x004E5A30, 0x003E481D, 0x00444D22, 0x00383C23,
        0x00383A26, 0x003C3C2E, 0x003A3830, 0x003B3833, 0x003B3833, 0x0036332E, 0x0039372F, 0x00424036, 0x003D3B31, 0x003D3B31, 0x003C3A30, 0x003A382E, 0x0038362C, 0x003E3C32, 0x0037352B, 0x00323030, 0x0032312D, 0x00343728, 0x00383E27, 0x003D4526,
        0x00414B27, 0x004F5A34, 0x00626D47, 0x00677952, 0x006B7D56, 0x0070825B, 0x0072845D, 0x0072845D, 0x0071835C, 0x0072845D, 0x0073855E, 0x0073855E, 0x0074865F, 0x0074865F, 0x0074865F, 0x0073855E, 0x0072845D, 0x0071835C, 0x0071835C, 0x0073815F,
        0x006D7D54, 0x0069794A, 0x00596835, 0x00434F1D, 0x003E481A, 0x00474B27, 0x00464929, 0x003F462B, 0x003E452A, 0x003E452A, 0x00394025, 0x003D4429, 0x003C4328, 0x003D4429, 0x003D4429, 0x003A4426, 0x003C4628, 0x003F492B, 0x003F492B, 0x003C4628,
        0x003F492B, 0x00404A2C, 0x00414B2D, 0x00454A2F, 0x00414629, 0x00474D2E, 0x0049502F, 0x00565D3C, 0x00666C4D, 0x006D7255, 0x007E8368, 0x00778A5F, 0x00768762, 0x00778666, 0x007E8C6A, 0x006B7A54, 0xFF828D6D, 0xFFFDFFF5, 0xFFFFFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFAFF, 0xF0F6F7EE, 0x006D7556, 0x00748258, 0x007B8960, 0x00778563, 0x00778561, 0x0074845B, 0x006F7455, 0x005F6046, 0x00726D58, 0x006E6655, 0x00736B5A,
        0x0077715E, 0x0072725A, 0x0072755B, 0x006E6A57, 0x006C6855, 0x006A6653, 0x006C6855, 0x005E5A47, 0x00635F4C, 0x00635F4C, 0x00625E4B, 0x0066614C, 0x006C6752, 0x00746F5A, 0x0077725D, 0x0078735E, 0x0075705B, 0x00716C57, 0x00716C57, 0x00797066,
        0x0074705D, 0x005C5F3F, 0x004C552A, 0x005A6936, 0x006B7B4B, 0x006D7E53, 0x006C7E57, 0x00738157, 0x00748258, 0x00758359, 0x0076845A, 0x0077855B, 0x0077855B, 0x0077855B, 0x0077855B, 0x0075865B, 0x0075865B, 0x0075865B, 0x0074855A, 0x00728358,
        0x006E7F54, 0x006B7C51, 0x0068794E, 0x00625F4A, 0x006D6956, 0x006C6855, 0x006F6E5A, 0x0071705C, 0x00686955, 0x006E6E5C, 0x00727260, 0x00615D4A, 0x00615D4A, 0x005E5A47, 0x00605C49, 0x00625E4B, 0x005E5A47, 0x005C5845, 0x00615D4A, 0x0061644A,
        0x0061634D, 0x00656553, 0x006A695B, 0x006D6C5E, 0x006B6A5C, 0x006E6E5C, 0x00747561, 0x006A6752, 0x0054553B, 0x00565C39, 0x006B754B, 0x006D7D4E, 0x006B7D4E, 0x006B7E53, 0x006A7E54, 0x00708155, 0x00718256, 0x00738458, 0x0075865B, 0x0075865B,
        0x0075855C, 0x0074845B, 0x0073835A, 0x0075855C, 0x006E7E55, 0x006F7F56, 0x00708057, 0x006C7C53, 0x006E7E55, 0x0063734A, 0x004B5B32, 0x0060624C, 0x00686A54, 0x00686955, 0x0070705E, 0x006E6E5E, 0x006E6E5E, 0x00747464, 0x00686858, 0x00666551,
        0x0061604C, 0x00666551, 0x005A5945, 0x005E5D49, 0x005F5E4A, 0x00605F4B, 0x00656450, 0x00656351, 0x006E6C5A, 0x006B6957, 0x006F6D5B, 0x00706F5B, 0x0074735E, 0x006A6A52, 0x006F7056, 0x0053613D, 0x00667450, 0x00707E5A, 0x00717F5B, 0x0074825E,
        0x00768460, 0x0075835F, 0x00768460, 0x0073855C, 0x0073855C, 0x0073855C, 0x0073855C, 0x0073855C, 0x0074865D, 0x0074865D, 0x0075875E, 0x006F855B, 0x0071855B, 0x006C7E57, 0x006C7B55, 0x0066704C, 0x004E5534, 0x00464B2B, 0x00595C3C, 0x0062644E,
        0x00656652, 0x006A6A5A, 0x00666459, 0x0069675C, 0x006B6A5C, 0x00616151, 0x00626250, 0x00666452, 0x00605E4C, 0x00646250, 0x00646250, 0x0063614F, 0x00666452, 0x00676553, 0x00646250, 0x0068635A, 0x00686557, 0x006C6A58, 0x006B6B53, 0x0064654B,
        0x0063644A, 0x0063634B, 0x005F5E49, 0x00687A53, 0x006C7E57, 0x0071835C, 0x0072845D, 0x0072845D, 0x0071835C, 0x0073855E, 0x00758760, 0x0074865F, 0x0074865F, 0x0074865F, 0x0074865F, 0x0073855E, 0x0072845D, 0x0071835C, 0x0070825B, 0x006D7E59,
        0x006A7B50, 0x00687848, 0x005A6936, 0x00485223, 0x004F532F, 0x0065644A, 0x006D6655, 0x006D6E54, 0x006D6E54, 0x00696A50, 0x0067684E, 0x0066674D, 0x00616248, 0x00595A40, 0x00525339, 0x0059573F, 0x005C5A42, 0x005B5941, 0x00626048, 0x005E5C44,
        0x005F5D45, 0x005E5C44, 0x00626048, 0x00676A50, 0x0060634A, 0x0063654F, 0x005D5E4A, 0x0061624E, 0x00656751, 0x0060634A, 0x0076795F, 0x00778A5F, 0x00758661, 0x00768565, 0x007E8C6A, 0x006A7953, 0xFF808B6B, 0xFFFDFFF5, 0xFFFFFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFF8FF, 0xF0F5F6ED, 0x006C7455, 0x00738157, 0x007B8862, 0x00778563, 0x00788662, 0x0076855F, 0x00757A5B, 0x00636149, 0x00746D5E, 0x00695D53, 0x005E5248,
        0x00534C3B, 0x00454428, 0x003F4522, 0x004F5040, 0x005F6050, 0x005F6050, 0x00606151, 0x00575848, 0x00575848, 0x004D4E3E, 0x00505141, 0x0061624E, 0x005C5D49, 0x005A5B47, 0x00535440, 0x004B4C38, 0x004C4D39, 0x004D4E3A, 0x004D4E3A, 0x00625656,
        0x00736A60, 0x0065664C, 0x00515930, 0x005A6838, 0x006E7E4E, 0x00728357, 0x00708259, 0x00738157, 0x00748258, 0x00758359, 0x0076845A, 0x0076845A, 0x0077855B, 0x0076845A, 0x0076845A, 0x00738459, 0x0074855A, 0x0074855A, 0x0074855A, 0x00728358,
        0x006E7F54, 0x006B7C51, 0x0068794E, 0x00655C53, 0x0072695F, 0x00635D50, 0x005D5849, 0x00565541, 0x00484732, 0x004D5036, 0x00595C42, 0x00686150, 0x00625B4A, 0x00635C4B, 0x00615A49, 0x005B5443, 0x005E5746, 0x005F5847, 0x00554E3D, 0x005B5F3C,
        0x00717355, 0x006D6E54, 0x00555440, 0x004E4B3C, 0x00545142, 0x00555341, 0x00545240, 0x00797368, 0x005B5745, 0x0057593C, 0x006B744C, 0x00707F53, 0x006E8051, 0x006F8257, 0x006F8359, 0x00718256, 0x00728357, 0x0074855A, 0x0076875C, 0x0076875C,
        0x0076865D, 0x0074845B, 0x0074845B, 0x0078885F, 0x00718158, 0x00728259, 0x00718158, 0x006C7C53, 0x006E7E55, 0x0063734A, 0x004A5A31, 0x006F6B60, 0x006A6759, 0x005A5846, 0x00595843, 0x0053523D, 0x005A5944, 0x00797863, 0x00757460, 0x00706958,
        0x00625B4A, 0x006A6352, 0x005E5746, 0x005E5746, 0x00625B4A, 0x00665F4E, 0x00615A49, 0x0069654D, 0x006A664E, 0x005E5A41, 0x005B573E, 0x005D5941, 0x0068624F, 0x00676051, 0x007C7467, 0x004E5C38, 0x0062704C, 0x006E7C58, 0x006F7D59, 0x0072805C,
        0x0074825E, 0x0074825E, 0x0075835F, 0x0073855C, 0x0073855C, 0x0072845B, 0x0072845B, 0x0073855C, 0x0074865D, 0x0076885F, 0x0076885F, 0x006F865A, 0x006F8359, 0x006B7D56, 0x006E7A56, 0x00697152, 0x0054583B, 0x0056573D, 0x0072725A, 0x00686752,
        0x0062614D, 0x00605E4C, 0x005D5A4B, 0x006C695A, 0x00797864, 0x00686850, 0x00595B3E, 0x005C5B41, 0x0059583E, 0x0058573D, 0x00545339, 0x0058573D, 0x00605F45, 0x005E5D43, 0x00605F45, 0x0077705F, 0x006B6552, 0x0068634E, 0x0069654D, 0x006D6853,
        0x00797360, 0x00746D5E, 0x005B5248, 0x00697B54, 0x006D7F58, 0x0071835C, 0x0073855E, 0x0071835C, 0x0071835C, 0x0073855E, 0x00758760, 0x0074865F, 0x0074865F, 0x0074865F, 0x0074865F, 0x0073855E, 0x0072845D, 0x0071835C, 0x0070825B, 0x0072845D,
        0x006E7F54, 0x006A7A4A, 0x005B6735, 0x00465022, 0x00535434, 0x00706958, 0x007B7068, 0x005D5843, 0x005D5843, 0x005B5641, 0x00625D48, 0x006C6752, 0x006B6651, 0x0065604B, 0x005F5A45, 0x0063584A, 0x00615648, 0x00584D3F, 0x00625749, 0x00605547,
        0x00665B4D, 0x006A5F51, 0x00766B5D, 0x00636548, 0x005C5B46, 0x00636051, 0x00625E53, 0x006D695E, 0x006E6B5C, 0x005E5D48, 0x00737558, 0x00778A5F, 0x00758661, 0x00758464, 0x007D8B69, 0x006A7953, 0xFF808B6B, 0xFFFEFFF6, 0xFFFFFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xE8F1F5E9, 0x006A7652, 0x00768657, 0x007B895F, 0x007C8864, 0x00818D69, 0x0075835A, 0x00706F5A, 0x00645F4A, 0x00706551, 0x00675A44, 0x005E533D,
        0x00645E47, 0x00474A30, 0x002C3318, 0x003B391B, 0x00444023, 0x00514A2F, 0x00554B33, 0x00504831, 0x00524B37, 0x004E4A37, 0x003E3F2B, 0x00454723, 0x00353915, 0x002E3411, 0x00353A1A, 0x00303313, 0x004F4D2F, 0x005D563B, 0x0054492E, 0x00524F3A,
        0x006A694D, 0x005C603D, 0x005A6037, 0x0060693E, 0x006E764D, 0x00737A55, 0x007D8463, 0x007A885F, 0x00717F56, 0x0076845B, 0x0079875D, 0x00748258, 0x0077865A, 0x007B8A5E, 0x007A895D, 0x0071835C, 0x0073855C, 0x00708358, 0x0076875B, 0x006B7C50,
        0x007D8E63, 0x006D7A54, 0x00677251, 0x0061634F, 0x00686753, 0x0057523D, 0x00564E37, 0x005D523C, 0x00564C34, 0x0041381D, 0x0050472C, 0x00625B4A, 0x00595340, 0x00504A33, 0x004F482D, 0x00514D30, 0x00515034, 0x00474B32, 0x003F452E, 0x004E543D,
        0x0053573E, 0x004B4C32, 0x003E3C1E, 0x004B472A, 0x005B5439, 0x00514D31, 0x00565138, 0x0063625E, 0x00575844, 0x005B5C35, 0x00717246, 0x007C7D55, 0x007A805D, 0x007A8864, 0x0071855B, 0x00728357, 0x00728357, 0x00738458, 0x00738458, 0x00748559,
        0x00748559, 0x0075865A, 0x0075865A, 0x00748563, 0x00768762, 0x0072845B, 0x006D7E52, 0x007A895D, 0x006F7C50, 0x00646E44, 0x00576137, 0x00656456, 0x00565442, 0x004F4B33, 0x00555135, 0x00453E25, 0x0039321E, 0x00635C4D, 0x008F867C, 0x00898681,
        0x007E7C72, 0x00716C5D, 0x0066614C, 0x00615D45, 0x005F5E44, 0x00585B42, 0x00545741, 0x0054532D, 0x0043411E, 0x003F3A19, 0x00524C2F, 0x00524B30, 0x004B472F, 0x004B4833, 0x00686753, 0x00615942, 0x006D7255, 0x006B7F5C, 0x00687D56, 0x0076845B,
        0x007F895F, 0x00738158, 0x00697F55, 0x0072845B, 0x0072845B, 0x0072845B, 0x0072845B, 0x0072845B, 0x0071835A, 0x0071835A, 0x0071835A, 0x00747F59, 0x00707B5A, 0x00707E5C, 0x006F8257, 0x00627542, 0x00525F31, 0x0057593C, 0x00696358, 0x00525038,
        0x004F4C30, 0x00524F30, 0x004D4B29, 0x0041401E, 0x00454428, 0x0050513D, 0x00545545, 0x004A5339, 0x0050543B, 0x0053543A, 0x00545037, 0x00534F33, 0x00555134, 0x005A5839, 0x005C5D3D, 0x0052512B, 0x004A4825, 0x004C4726, 0x00524B30, 0x00565039,
        0x00595340, 0x00696657, 0x00525143, 0x0065744E, 0x0073835A, 0x00728357, 0x00778B5C, 0x00768A5D, 0x006F8358, 0x00768962, 0x0072845F, 0x00778A5F, 0x0076885F, 0x00708259, 0x0075875E, 0x0074865F, 0x006B7D56, 0x006E7F5A, 0x0071825D, 0x00778458,
        0x00757F54, 0x00707A4F, 0x00596139, 0x004D5030, 0x005D5A45, 0x00635B4E, 0x0061584F, 0x005A4B38, 0x005B5139, 0x00595739, 0x00454B28, 0x0049502D, 0x004E5336, 0x00575A44, 0x0051513F, 0x004E4D39, 0x004E4B36, 0x004E4930, 0x00524A2D, 0x00585033,
        0x005B5439, 0x00575239, 0x004F4B33, 0x0047502F, 0x004D5233, 0x0059553C, 0x00564D39, 0x005A513D, 0x006F6A55, 0x0062654B, 0x006C7456, 0x007F8C6C, 0x0074845B, 0x00778A59, 0x007B8E5B, 0x006F8055, 0xFF7D886E, 0xFFFAFEF9, 0xFFFDFCFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBF8FF, 0xE8F1F5EA, 0x006C7756, 0x00758359, 0x007B8960, 0x007D8867, 0x00778261, 0x005D6A44, 0x00585742, 0x00605B46, 0x00716854, 0x006B604A, 0x0060553F,
        0x0069644B, 0x005E5F45, 0x00353A1F, 0x00454D2F, 0x004A4E31, 0x00525135, 0x00535034, 0x004D492D, 0x004C492D, 0x004A4C2F, 0x0044482B, 0x003F3F1B, 0x003A3B19, 0x00363A17, 0x003C4121, 0x003D4223, 0x00565539, 0x005F5A41, 0x005D533B, 0x00636051,
        0x0073715F, 0x005D6047, 0x0044492A, 0x00383F1C, 0x00404823, 0x00424D27, 0x0058633D, 0x006D7B52, 0x007F8D64, 0x0077855C, 0x00738157, 0x0077855B, 0x00768559, 0x0079885C, 0x00758458, 0x00708259, 0x007B8E63, 0x006C7F54, 0x00728358, 0x007F8F66,
        0x00636F4B, 0x004C5638, 0x00475036, 0x006A6955, 0x0072715D, 0x00615C47, 0x005C563F, 0x00605B42, 0x005D583F, 0x004A462A, 0x00565235, 0x00515641, 0x004E5138, 0x00494B2E, 0x00444525, 0x003D3E1E, 0x003A3C1E, 0x0042472A, 0x004D5439, 0x00495238,
        0x0051573E, 0x0051543A, 0x00464529, 0x00524F33, 0x00625D44, 0x005E5841, 0x005F5942, 0x0067655A, 0x005E6147, 0x004E532C, 0x0042491E, 0x00474E29, 0x00515B37, 0x00617148, 0x00778C5B, 0x00728357, 0x00738458, 0x00738458, 0x00748559, 0x00748559,
        0x0075865A, 0x0075865A, 0x0075865A, 0x00748760, 0x0064764F, 0x0075875E, 0x00718158, 0x0059673E, 0x00444F29, 0x00484F2C, 0x00484F2E, 0x006B685A, 0x005B5947, 0x00585540, 0x00666249, 0x005C583F, 0x004F4C37, 0x006F6B59, 0x008F8C7E, 0x008F908C,
        0x007E7C74, 0x00656456, 0x00585441, 0x00504D38, 0x00494B35, 0x004B4D39, 0x004C533E, 0x00525333, 0x00494728, 0x00464426, 0x00595539, 0x005D583F, 0x005C5840, 0x0056533E, 0x006C6954, 0x006B624E, 0x004C5037, 0x003B4D2E, 0x004A5E3B, 0x0066724E,
        0x007A825D, 0x007B8862, 0x0071865F, 0x0073855C, 0x0073855C, 0x0073855C, 0x0072845B, 0x0072845B, 0x0072845B, 0x0072845B, 0x0071835A, 0x00768159, 0x007F8A69, 0x00606C4E, 0x00495937, 0x00435429, 0x00455028, 0x00606147, 0x006E685B, 0x0066614C,
        0x00534F37, 0x005D5B3D, 0x005D5B3C, 0x00404022, 0x0044452B, 0x00545642, 0x00454838, 0x004A543D, 0x00464E37, 0x00484B35, 0x004F4E39, 0x0051503B, 0x00484A34, 0x003C422B, 0x00363E27, 0x004C502D, 0x00474828, 0x004D4B2D, 0x00575438, 0x005D5941,
        0x005C5944, 0x006D6C58, 0x005A5846, 0x00475335, 0x00515E3E, 0x005F6D49, 0x00718459, 0x00718459, 0x0072865B, 0x007C9066, 0x00667952, 0x0072855A, 0x0074865D, 0x00708259, 0x0074865D, 0x0073855E, 0x006E8059, 0x0073845F, 0x0072835E, 0x006C7A51,
        0x0058643A, 0x00535F35, 0x004E572F, 0x00484E2B, 0x0057593C, 0x006D6A55, 0x0074705E, 0x006C604E, 0x005F5844, 0x0057563A, 0x004B5030, 0x004D5433, 0x0044492C, 0x00464832, 0x00494834, 0x00474B32, 0x004C4F35, 0x00525135, 0x00504E30, 0x004A4A2C,
        0x0048492F, 0x00464933, 0x00414833, 0x00505839, 0x00595C42, 0x00645F4A, 0x00605946, 0x00665E4D, 0x00797360, 0x005D5F49, 0x0061674E, 0x006A755B, 0x00727F5F, 0x007E8F64, 0x007B8D5E, 0x0068784F, 0xFF7A856B, 0xFFFCFFFB, 0xFFFDFCFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFBFF, 0xF0F4F7EE, 0x00687254, 0x00717F56, 0x00808C68, 0x007A8466, 0x005E674C, 0x003B4626, 0x00464531, 0x0068634E, 0x0068614D, 0x00686049, 0x007E775E,
        0x00777357, 0x005C5B3F, 0x00414326, 0x003F4B2F, 0x00434B2D, 0x004F5133, 0x005C5A3B, 0x00615C3B, 0x0064603D, 0x00696542, 0x00686741, 0x00554F2C, 0x004F4D2B, 0x003F4020, 0x0037391B, 0x00393D20, 0x0056573D, 0x00706E56, 0x00827D68, 0x00707064,
        0x00727266, 0x00545446, 0x00373A2A, 0x00282D18, 0x00293218, 0x00263111, 0x0037431F, 0x00515F36, 0x007F8D64, 0x0075835A, 0x00707E54, 0x007A885E, 0x00758458, 0x0079885C, 0x00768559, 0x0074885B, 0x0073875A, 0x00738459, 0x00798960, 0x006B7655,
        0x00485137, 0x00868C79, 0x009EA094, 0x0065614F, 0x00676350, 0x0063604B, 0x007D7B63, 0x00818268, 0x00616346, 0x003C3E20, 0x00464929, 0x003E472C, 0x003C4426, 0x00474B28, 0x005A5A36, 0x0062613B, 0x005D5B38, 0x00555432, 0x00525234, 0x004F5739,
        0x0052573C, 0x004E5338, 0x0047472F, 0x0054523A, 0x00716C57, 0x007E7763, 0x00827866, 0x00777662, 0x0053573A, 0x002F3915, 0x00192705, 0x00233115, 0x002F3D21, 0x003F4F26, 0x006C804B, 0x00738458, 0x00738458, 0x00738458, 0x00748559, 0x00748559,
        0x0075865A, 0x0075865A, 0x0076875B, 0x006F8257, 0x00788A61, 0x007A8963, 0x00505B3A, 0x003E4628, 0x003C4126, 0x00444731, 0x00565842, 0x006A6556, 0x00625E4C, 0x0065644F, 0x006E6F55, 0x0058593F, 0x003E4128, 0x004F513B, 0x00686955, 0x007E7F75,
        0x00727163, 0x00666452, 0x0066624A, 0x00615D44, 0x0054543C, 0x004F5239, 0x004F553E, 0x0053543A, 0x0047482E, 0x0044452B, 0x00595A40, 0x006D6C52, 0x0078765E, 0x006D6951, 0x0077735B, 0x00594E40, 0x00686A57, 0x00788871, 0x006A7B60, 0x00515A3F,
        0x00616748, 0x007A8564, 0x0070855F, 0x0074865D, 0x0073855C, 0x0073855C, 0x0073855C, 0x0073855C, 0x0072845B, 0x0072845B, 0x0072845B, 0x007C885E, 0x00616B4D, 0x0039422E, 0x00313C28, 0x00283418, 0x00283011, 0x0056563E, 0x00716C5D, 0x00736C59,
        0x00766F5B, 0x00747057, 0x00636246, 0x004A4C2F, 0x00404329, 0x00494C36, 0x00545642, 0x00585F44, 0x0054593E, 0x0055583F, 0x005C5B46, 0x00575945, 0x00494F3C, 0x00404A39, 0x00404E3C, 0x00414629, 0x0043482B, 0x00595D40, 0x0072755B, 0x0078795F,
        0x006B6A50, 0x006F6D55, 0x0059553D, 0x00777C6D, 0x006E7763, 0x004E5A3E, 0x005C6A48, 0x007B8D64, 0x0072855A, 0x006C8055, 0x00778B61, 0x00708358, 0x0074865D, 0x0071835A, 0x0073855C, 0x0073855E, 0x0072845D, 0x0073845F, 0x00697A55, 0x00445236,
        0x00384426, 0x0032401E, 0x00293511, 0x002A3410, 0x00515738, 0x007B7E64, 0x0072755C, 0x00807769, 0x00716D5A, 0x005E5E46, 0x004A4F32, 0x00474C2F, 0x00414326, 0x004B4931, 0x005E5743, 0x0066684A, 0x00696A4A, 0x00686745, 0x00616242, 0x005B5D3F,
        0x0052583F, 0x0046513D, 0x003C4A38, 0x0044492E, 0x00585742, 0x00736F5D, 0x00746D5E, 0x006C6457, 0x007E796A, 0x005E5C4A, 0x0051533F, 0x004B5245, 0x00626B56, 0x00788664, 0x007C8D62, 0x00697852, 0xFF7E896F, 0xFFFDFFF9, 0xFFFDFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFAFF, 0xF4F5F8EF, 0x00697355, 0x0076835D, 0x00859070, 0x00697258, 0x0049513A, 0x00495237, 0x00534F3C, 0x00625E4B, 0x00635E49, 0x0058523B, 0x00615C43,
        0x004B472A, 0x003A381A, 0x004A4829, 0x00434528, 0x004C4A2C, 0x005E5534, 0x006F603F, 0x007A6742, 0x007B673E, 0x00736237, 0x006C5C2E, 0x00766844, 0x006E6442, 0x005E5736, 0x00474526, 0x0037391C, 0x0042422A, 0x0055543F, 0x0064604D, 0x00414730,
        0x005E6151, 0x0056574D, 0x0053534D, 0x003B3D37, 0x002C3025, 0x00373F2E, 0x00505944, 0x004B5930, 0x006E7C53, 0x00748259, 0x00748258, 0x0077855B, 0x00748357, 0x0078875B, 0x0079885C, 0x00798B5C, 0x00728455, 0x00748559, 0x007C8963, 0x00576143,
        0x005A604D, 0x009FA097, 0x008D8B8A, 0x006B6352, 0x005C5544, 0x0044432F, 0x005D6047, 0x0063684B, 0x00464C2D, 0x00303613, 0x00424825, 0x004E4F2D, 0x00575532, 0x00676039, 0x0071653B, 0x00716137, 0x006B5930, 0x006D5B36, 0x0073603D, 0x00616240,
        0x00545737, 0x004D4F32, 0x00474933, 0x004C4B37, 0x00585442, 0x0057503F, 0x004B4431, 0x00676351, 0x00585A44, 0x00525C45, 0x00334030, 0x002F3C34, 0x00495346, 0x004B5739, 0x005A6739, 0x00738458, 0x00738458, 0x00748559, 0x00748559, 0x0075865A,
        0x0075865A, 0x0075865A, 0x0076875B, 0x00728756, 0x00738458, 0x005D6B47, 0x004A5338, 0x00565B46, 0x00444535, 0x00414032, 0x00504F41, 0x00655E4D, 0x005A5643, 0x005B5A45, 0x005F634A, 0x004D5237, 0x00393E23, 0x0046492F, 0x005E5F45, 0x0072694E,
        0x006B6044, 0x00665939, 0x00675935, 0x00685B35, 0x00675B37, 0x00685F3D, 0x006B6443, 0x0057553D, 0x00484830, 0x003F4229, 0x00494C33, 0x0054573E, 0x005E5F45, 0x00534F36, 0x00645F46, 0x0065594F, 0x006E6E62, 0x00AFBCAE, 0x00B4C1B1, 0x005B624D,
        0x004E5338, 0x007F8A6A, 0x00728761, 0x0074865D, 0x0074865D, 0x0074865D, 0x0074865D, 0x0073855C, 0x0073855C, 0x0073855C, 0x0073855C, 0x006E7B4F, 0x00555E43, 0x00646A5F, 0x00545B54, 0x00313829, 0x003F452E, 0x005A5944, 0x00605C4A, 0x005F5544,
        0x006E6754, 0x0066634E, 0x0052523A, 0x0046492F, 0x003B3F22, 0x00414326, 0x00595B3E, 0x00756F46, 0x0069623B, 0x00645B36, 0x006B6241, 0x00726C4F, 0x006D6C52, 0x005E6249, 0x00515942, 0x00484E35, 0x0041472E, 0x004C5239, 0x005E654A, 0x0062674C,
        0x00595B3E, 0x006B684C, 0x005B573B, 0x00A1A29E, 0x00BEC2B7, 0x007A806D, 0x00475335, 0x006B7955, 0x007C8E65, 0x0072855A, 0x00708358, 0x0073865B, 0x00778960, 0x0074865D, 0x0074865D, 0x00758760, 0x00758760, 0x006C7D58, 0x00546540, 0x00545C55,
        0x00586055, 0x00505A4A, 0x0039422D, 0x002F381E, 0x004B5138, 0x00747761, 0x00565844, 0x00535041, 0x0052503E, 0x00454632, 0x003D4027, 0x004A4B31, 0x00585539, 0x006A6245, 0x007C7054, 0x00756945, 0x00786B45, 0x00796C46, 0x00766A46, 0x0071694B,
        0x0069654C, 0x00595B48, 0x004B5041, 0x004C4B36, 0x00504F3B, 0x006D6859, 0x006A6459, 0x00575047, 0x007B756A, 0x00676455, 0x00595745, 0x004E514F, 0x005F6659, 0x006F7A60, 0x007E8C68, 0x006C7A56, 0xFF7D886E, 0xFFFDFFF8, 0xFFFDFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCF9FF, 0xF4F7FBF0, 0x006D7858, 0x0077845E, 0x00828D6D, 0x00616952, 0x00666D58, 0x00ADB69C, 0x00979381, 0x00605C49, 0x0063604B, 0x004E4D33, 0x003B381C,
        0x00393617, 0x00423D1C, 0x004E4825, 0x00695635, 0x00826C49, 0x009F8360, 0x00B7936D, 0x00C59F75, 0x00CDA67A, 0x00C9A373, 0x00C29D6B, 0x0090764E, 0x00826D47, 0x00766844, 0x00615A39, 0x004C4A2C, 0x0048472D, 0x004C4B36, 0x00484732, 0x00424928,
        0x0063684D, 0x005E6151, 0x00898A81, 0x00969493, 0x00868686, 0x008D8E8C, 0x00868783, 0x0068764D, 0x00606E45, 0x00738158, 0x007A885E, 0x00738157, 0x0078875B, 0x0078875B, 0x007A895D, 0x00788B5A, 0x00758758, 0x00707F53, 0x007C8963, 0x004A5338,
        0x005F6252, 0x00787872, 0x00686666, 0x00675C4E, 0x00625B4A, 0x003D3E2A, 0x003B3F26, 0x003A4224, 0x003A3F1F, 0x003E401C, 0x004F5029, 0x007A6742, 0x00856F46, 0x00977C50, 0x00AD8C5E, 0x00C09C6E, 0x00C59E72, 0x00B58F65, 0x00A37C55, 0x00796D43,
        0x00615832, 0x00524D2E, 0x00525038, 0x00535341, 0x00565442, 0x004F4B38, 0x003F3926, 0x005D544B, 0x005C5A4F, 0x008E9188, 0x008E9493, 0x007A8087, 0x0082878A, 0x006A7162, 0x00525B3A, 0x00738458, 0x00738458, 0x00738458, 0x00748559, 0x00748559,
        0x0075865A, 0x0075865A, 0x0075865A, 0x00768956, 0x00718256, 0x005D6847, 0x005A6049, 0x006D6E5E, 0x00717165, 0x007A786D, 0x005D594E, 0x0068624F, 0x004C4835, 0x003F412B, 0x00434932, 0x00474C31, 0x004C4E31, 0x005E5B3C, 0x00716C4D, 0x00785F33,
        0x00886E3F, 0x009B7E4B, 0x00A78853, 0x00A98A55, 0x00A28552, 0x00907745, 0x00816A3A, 0x00665D42, 0x00575035, 0x0048472B, 0x003E4127, 0x00373C21, 0x003B3C22, 0x0038341B, 0x005D583F, 0x0062554D, 0x00828078, 0x007B867C, 0x00707B71, 0x006A6F60,
        0x005A5D47, 0x00606B4B, 0x00768B64, 0x0074865D, 0x0074865D, 0x0074865D, 0x0074865D, 0x0074865D, 0x0073855C, 0x0073855C, 0x0073855C, 0x006A774B, 0x005E674D, 0x00B3B7B1, 0x00ADB2B1, 0x00797E75, 0x00737764, 0x005A5945, 0x0065614F, 0x00493F2E,
        0x00423C29, 0x00474330, 0x00494931, 0x00444629, 0x00524F30, 0x00645E3B, 0x00685F3A, 0x00765E28, 0x008C743E, 0x00A18654, 0x009D8355, 0x008C744A, 0x007D6B42, 0x00796C46, 0x00786F49, 0x0057573F, 0x0042452C, 0x00353B22, 0x00363F24, 0x003A4126,
        0x003F4326, 0x00636044, 0x005F583D, 0x005E5C5C, 0x00878983, 0x006F7465, 0x00535C42, 0x00616F4D, 0x006F7F56, 0x0072855A, 0x00718459, 0x0074875C, 0x00778960, 0x0074865D, 0x0076885F, 0x00788A63, 0x00778962, 0x00687954, 0x00445530, 0x0071737E,
        0x0084868E, 0x00878C8B, 0x00848882, 0x0072766B, 0x0056584C, 0x00656559, 0x0049493D, 0x003F3F2F, 0x00424230, 0x0041422E, 0x004C4C34, 0x00625E42, 0x00726747, 0x00766642, 0x007B6541, 0x0094744B, 0x009E7E55, 0x00A5855C, 0x00A1825B, 0x00967A57,
        0x00887256, 0x007D6B54, 0x00746552, 0x00605D48, 0x00534F3D, 0x00585547, 0x005A554C, 0x00534E45, 0x00757166, 0x00676154, 0x00777263, 0x0077797A, 0x006C7168, 0x00667059, 0x007F8D69, 0x00717F5B, 0xFF7A856B, 0xFFF8FEF3, 0xFFFDFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xECF6FBEC, 0x006B7655, 0x0076845B, 0x00828D6C, 0x00666F55, 0x00717962, 0x00C2CBB1, 0x00A5A18F, 0x00635F4D, 0x0061604B, 0x0047482E, 0x004A4A2C,
        0x00575533, 0x00544E2B, 0x00584F2A, 0x009A774F, 0x00B79167, 0x00CFA37A, 0x00D5A57B, 0x00D7A479, 0x00D7A577, 0x00D4A474, 0x00D2A370, 0x00CAA377, 0x00AE8C61, 0x00957C54, 0x00756540, 0x005A5332, 0x004F4B2E, 0x00504F35, 0x0046442C, 0x00454926,
        0x005F6445, 0x00545640, 0x008A897B, 0x00B7B5AD, 0x00D5D2CE, 0x00FAF8F7, 0x00F7F5F4, 0x00919F76, 0x006C7A51, 0x0079875E, 0x007C8A60, 0x00717F55, 0x0078875B, 0x00728155, 0x006E7D51, 0x00738556, 0x007A8A5B, 0x0077865A, 0x00798660, 0x0051593B,
        0x00A9AD9A, 0x00C7C5BD, 0x00999692, 0x00605448, 0x00666152, 0x004C4D39, 0x00474E33, 0x00414728, 0x00454422, 0x0058502B, 0x0071643E, 0x00A47E54, 0x00BB9468, 0x00D0A574, 0x00D3A471, 0x00CE9E6A, 0x00D09F6D, 0x00D4A474, 0x00D4A579, 0x00BA9B68,
        0x00988052, 0x0073633F, 0x00625B42, 0x00595844, 0x00575945, 0x005C5E48, 0x0056563E, 0x00665F56, 0x005A564B, 0x00929187, 0x00BFBDBD, 0x00DFDCE5, 0x00F6F4FA, 0x00B0B4A8, 0x00565D42, 0x00728357, 0x00728357, 0x00738458, 0x00738458, 0x00748559,
        0x00748559, 0x0075865A, 0x0075865A, 0x00798D58, 0x007B8A5E, 0x00545F3F, 0x00606550, 0x00BEBEB0, 0x00E9E7DC, 0x00BDBAAB, 0x00555341, 0x0067654D, 0x004A4934, 0x003E402C, 0x00414730, 0x004A4A32, 0x005A5437, 0x00706140, 0x0086734E, 0x00B18B55,
        0x00BF995F, 0x00CFA56A, 0x00D3A76A, 0x00D4A86B, 0x00D3A96E, 0x00C7A069, 0x00BB9460, 0x00907A57, 0x00756541, 0x0058522F, 0x00414424, 0x003B4122, 0x0044472D, 0x00413F27, 0x006B674F, 0x005E5149, 0x008E8C84, 0x00AEB8B2, 0x009DA8A0, 0x00686C60,
        0x005A5D47, 0x0075815D, 0x0072895D, 0x0074865D, 0x0074865D, 0x0074865D, 0x0074865D, 0x0074865D, 0x0073855C, 0x0073855C, 0x0073855C, 0x006B774D, 0x005A6349, 0x00B7BBB5, 0x00D1D6D4, 0x00BAC0B5, 0x00949885, 0x0051503C, 0x006C6758, 0x0058513D,
        0x00524D38, 0x004B4A35, 0x00494A30, 0x00524E31, 0x00685C38, 0x00816E43, 0x0092784A, 0x00C7A367, 0x00CCA66C, 0x00CFA66F, 0x00CDA36E, 0x00CBA36F, 0x00C29F6D, 0x00AC8C5B, 0x00937846, 0x0075684E, 0x00615A3F, 0x004F5134, 0x00474E33, 0x004B5237,
        0x0053563C, 0x00726E56, 0x00635B44, 0x008F8D8C, 0x00CCCDC4, 0x009AA08F, 0x004F583D, 0x00586440, 0x0076865D, 0x007D8E63, 0x00718257, 0x0072855A, 0x0074865D, 0x0073855C, 0x0076885F, 0x00778962, 0x00788A63, 0x006A7B56, 0x00455631, 0x009696A4,
        0x00BEBDC7, 0x00ACADB1, 0x00989995, 0x008A8B82, 0x005D5C52, 0x0068665C, 0x0057554B, 0x004D4C38, 0x004F4E3A, 0x0057543F, 0x006B654E, 0x00766B4D, 0x0086734E, 0x00967D51, 0x00AA8C5D, 0x00C79C71, 0x00CEA175, 0x00D1A478, 0x00CDA074, 0x00C59A6F,
        0x00B8916B, 0x00A98665, 0x009C7C5F, 0x0067624D, 0x005A5643, 0x00545142, 0x005A5A4E, 0x005D5D51, 0x006F6B60, 0x005C5748, 0x00A09988, 0x00D2D5D3, 0x009BA295, 0x00657155, 0x00778660, 0x00707E5A, 0xFF7F8A70, 0xFFF8FEF3, 0xFFFAFCFC, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFEFC, 0xD9EEF4E1, 0x006A784F, 0x00768559, 0x0085926C, 0x0070795E, 0x00596248, 0x006C755A, 0x00696253, 0x005D5B49, 0x0062644E, 0x0041462B, 0x004F5133,
        0x004D4B28, 0x00544B26, 0x007D704A, 0x00C09867, 0x00D0A776, 0x00D9AB7B, 0x00D8A777, 0x00D5A474, 0x00D1A171, 0x00CEA070, 0x00CEA574, 0x00DDAC7A, 0x00CDA271, 0x00B59466, 0x008D774D, 0x006E613B, 0x00524B2A, 0x004C482B, 0x0049462A, 0x00504B36,
        0x0077725D, 0x0065634B, 0x00525038, 0x0041402B, 0x0061604C, 0x009F9C8D, 0x00BDBCAE, 0x007A885F, 0x006C7A51, 0x0075835A, 0x0077855B, 0x0076845A, 0x007F8E62, 0x007C8B5F, 0x007B8A5E, 0x00748455, 0x007D8D5E, 0x007B8A5E, 0x00737E56, 0x00717A59,
        0x00EAEED5, 0x00FBFCEC, 0x0089887E, 0x00696053, 0x00676253, 0x00444632, 0x00454A2F, 0x00424525, 0x004E4825, 0x00796641, 0x00A38863, 0x00CCA071, 0x00D0A272, 0x00D3A36F, 0x00D2A169, 0x00D3A26A, 0x00D4A76E, 0x00D0A46F, 0x00C8A06C, 0x00D8A96C,
        0x00C69E6A, 0x009A8058, 0x00726749, 0x00515139, 0x00464933, 0x004A5136, 0x00474C2F, 0x00716D5B, 0x005A5840, 0x00555139, 0x005E5748, 0x00A69D99, 0x00F2EFE7, 0x00C4C9B4, 0x00697453, 0x00718256, 0x00718256, 0x00728357, 0x00728357, 0x00738458,
        0x00738458, 0x00748559, 0x00748559, 0x00738752, 0x007B8A5E, 0x00626A4B, 0x00575C47, 0x00919183, 0x00919181, 0x00666550, 0x00505137, 0x0064654B, 0x004E503A, 0x004B503B, 0x0050523E, 0x005A563E, 0x00716144, 0x008E724F, 0x00AD8963, 0x00C49C68,
        0x00CBA26B, 0x00D1A46B, 0x00CFA167, 0x00CFA068, 0x00D1A36D, 0x00CFA473, 0x00CCA273, 0x00BC996D, 0x009D8256, 0x0071653B, 0x004C4B25, 0x00424727, 0x004C5134, 0x0043422D, 0x0066624F, 0x00665A50, 0x006C6A62, 0x007A847E, 0x0089938D, 0x00717569,
        0x0063674E, 0x0077855C, 0x00708957, 0x0074865D, 0x0074865D, 0x0074865D, 0x0073855C, 0x0073855C, 0x0073855C, 0x0073855C, 0x0073855C, 0x00717C54, 0x00555E44, 0x006A7065, 0x00788076, 0x007A8172, 0x006D725D, 0x00545240, 0x00686255, 0x004F4B32,
        0x0054553B, 0x004D4E34, 0x004C4B2F, 0x00675E3D, 0x00867047, 0x00AA8656, 0x00CFA36E, 0x00C39C68, 0x00C79F6B, 0x00CC9F6C, 0x00CE9F6C, 0x00D1A26F, 0x00D1A471, 0x00C8A06C, 0x00BF9864, 0x008E7456, 0x00786649, 0x005B5538, 0x0044482B, 0x00444A31,
        0x0052553F, 0x006D6956, 0x00544C3B, 0x00898A80, 0x00ADB0A1, 0x007A8069, 0x00555F41, 0x0076825E, 0x0077875E, 0x006B7B52, 0x0078885F, 0x00718459, 0x0073855C, 0x0074865D, 0x0076885F, 0x0073855E, 0x00758760, 0x0070815C, 0x0053643F, 0x00C0C0C0,
        0x00FFFFFB, 0x00C1C2B8, 0x00767767, 0x0070705E, 0x00575642, 0x006E6A58, 0x005F5B49, 0x0055533B, 0x0059573F, 0x0067614A, 0x007B7055, 0x007E6B4A, 0x009A7E55, 0x00BA9765, 0x00D1AB71, 0x00CBA174, 0x00CCA071, 0x00CC9F6C, 0x00CFA06C, 0x00D3A471,
        0x00CFA473, 0x00C2996C, 0x00B58D63, 0x00767158, 0x0068634E, 0x00585846, 0x00565949, 0x00545748, 0x00696959, 0x005B5443, 0x00A29B88, 0x00EBF1E6, 0x00B2BBA7, 0x006E7B5B, 0x0075855C, 0x00687751, 0xFF7E896F, 0xFFFDFFF8, 0xFFFDFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFB, 0xD1EDF5DE, 0x0069774D, 0x006D7D4D, 0x00818F66, 0x00889274, 0x006E775C, 0x005B6547, 0x00665F50, 0x0063614F, 0x00696C56, 0x004A4F34, 0x00484D2D,
        0x0043411E, 0x006C633D, 0x00A79871, 0x00D3AE74, 0x00D4AE74, 0x00D0A770, 0x00D1A570, 0x00D3A875, 0x00CFA675, 0x00C9A373, 0x00CAA978, 0x00D69D69, 0x00DBAB77, 0x00D1AC7A, 0x00A98F61, 0x0086764C, 0x005C532E, 0x00524D2C, 0x00595637, 0x00594D47,
        0x00766D60, 0x006B6450, 0x00494729, 0x003D3C1A, 0x005D5E3C, 0x00676749, 0x00605F43, 0x00627047, 0x0079875E, 0x007D8B62, 0x0077855B, 0x0078865C, 0x00768559, 0x00758458, 0x00768559, 0x0079885C, 0x00738256, 0x00727F53, 0x007C885E, 0x00737D59,
        0x00787D60, 0x008E907C, 0x00808171, 0x00645C4F, 0x00696657, 0x00494B37, 0x00454A2F, 0x004C4D2D, 0x006A5E3A, 0x00957B56, 0x00B18D67, 0x00CFA473, 0x00CFA36E, 0x00CFA269, 0x00CFA368, 0x00CEA567, 0x00C9A468, 0x00C6A469, 0x00C2A46B, 0x00DC9F5F,
        0x00DBAA72, 0x00B8966B, 0x00847555, 0x0057563C, 0x00444A31, 0x004A5436, 0x004A5332, 0x00707254, 0x00595935, 0x004E4B25, 0x00595133, 0x007C735F, 0x0085816E, 0x006C7253, 0x00697A4E, 0x00718256, 0x00718256, 0x00718256, 0x00728357, 0x00728357,
        0x00738458, 0x00738458, 0x00748559, 0x00728350, 0x00758359, 0x007F8768, 0x005D5F4B, 0x00595949, 0x0051523E, 0x00414228, 0x00515232, 0x00696C52, 0x004E5239, 0x00474C37, 0x004C4E3A, 0x00605841, 0x00836D51, 0x00A7835F, 0x00C79B72, 0x00C8A174,
        0x00CAA272, 0x00CDA271, 0x00CEA16E, 0x00CB9D6D, 0x00C89C6D, 0x00C69C72, 0x00C79E77, 0x00CDA36E, 0x00B89765, 0x00917F50, 0x0059572E, 0x003F4522, 0x00474C2F, 0x003F412B, 0x00666452, 0x00675B4F, 0x0085847A, 0x009FA9A3, 0x009DA7A1, 0x00707468,
        0x0062674C, 0x007C8A60, 0x006D8751, 0x0074865D, 0x0074865D, 0x0073855C, 0x0073855C, 0x0073855C, 0x0073855C, 0x0072845B, 0x0072845B, 0x007D8862, 0x005C654B, 0x00636A5D, 0x0099A295, 0x00939C87, 0x0060664D, 0x005D5C48, 0x00686257, 0x00515133,
        0x0046482B, 0x00505437, 0x00646244, 0x00756945, 0x009F8458, 0x00CCA06B, 0x00DAA468, 0x00C19E73, 0x00C29C72, 0x00C69B70, 0x00CB9C70, 0x00CE9D6F, 0x00CD9F6F, 0x00CBA06F, 0x00CCA16E, 0x00B29170, 0x00987F5F, 0x006C6141, 0x0045472A, 0x0042482F,
        0x00585A46, 0x00747162, 0x005B5245, 0x00737365, 0x00808471, 0x00686E55, 0x00636C4B, 0x0077845E, 0x00798660, 0x0075845E, 0x0071805A, 0x0072855A, 0x0075875E, 0x00778960, 0x00778960, 0x006E8059, 0x0070825B, 0x00748560, 0x0060714C, 0x009C9F90,
        0x00F9FBE8, 0x00B9BDA4, 0x00686C4F, 0x00747658, 0x005F5F41, 0x006D6B4D, 0x00575438, 0x00524F33, 0x005B573E, 0x006D664D, 0x0085775B, 0x00927957, 0x00BA996C, 0x00CDA66F, 0x00CB9F60, 0x00C29E6E, 0x00C7A270, 0x00CBA36E, 0x00CEA36A, 0x00CEA36A,
        0x00CDA46D, 0x00CAA26D, 0x00C59E6A, 0x00888166, 0x006C6850, 0x005F604C, 0x005A5E4B, 0x00505343, 0x00737361, 0x0057513E, 0x006F6652, 0x00787F70, 0x00788369, 0x006A7854, 0x00829367, 0x00697950, 0xFF778268, 0xFFF9FEF5, 0xFFFDFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFBFD, 0xD1EBEEDE, 0x0068734B, 0x00738353, 0x007D8B61, 0x007A8864, 0x00778561, 0x0076855F, 0x00716C53, 0x006A5F57, 0x00695F58, 0x004B5138, 0x00424F2F,
        0x0042452B, 0x008C7958, 0x00C6A573, 0x00D2A572, 0x00D4A774, 0x00D5A875, 0x00D4A774, 0x00D3A673, 0x00D2A572, 0x00D3A673, 0x00D4A774, 0x00C6A46F, 0x00CDA771, 0x00D1A974, 0x00C7A06C, 0x00997B4C, 0x00826F49, 0x00524E31, 0x0051553C, 0x00544E3B,
        0x00756C62, 0x00625E4B, 0x00545D32, 0x00566133, 0x005C6445, 0x00606A4C, 0x006A7A51, 0x00728056, 0x0078865C, 0x00758359, 0x00728155, 0x00768559, 0x00788859, 0x00768657, 0x00768657, 0x0073825C, 0x00788761, 0x0072815B, 0x00748259, 0x00748258,
        0x006F7B51, 0x00657246, 0x00576438, 0x005F534D, 0x00767260, 0x00575C41, 0x00464E30, 0x004E4D33, 0x00807255, 0x00A2895D, 0x00C3A56A, 0x00CFA26F, 0x00CEA16E, 0x00CEA16E, 0x00CEA16E, 0x00CDA06D, 0x00CEA16E, 0x00CEA16E, 0x00CFA26F, 0x00C3A56E,
        0x00C9A26E, 0x00D3A576, 0x00B18E63, 0x00705D3A, 0x00514E32, 0x0051533F, 0x0049493B, 0x006F665D, 0x00605D48, 0x00565D3C, 0x0056623E, 0x0052603E, 0x005B6945, 0x00738052, 0x0074864B, 0x006F8251, 0x00708352, 0x00718453, 0x00728554, 0x00738655,
        0x00738655, 0x00738655, 0x00738655, 0x00758256, 0x0077865A, 0x00708156, 0x006D7F56, 0x005E6D47, 0x005A6440, 0x00545835, 0x00565737, 0x00616657, 0x004D5342, 0x00464B36, 0x004C4B31, 0x00756B49, 0x00937A4E, 0x00C7A06C, 0x00CD9F65, 0x00CBA06D,
        0x00CBA06D, 0x00CCA16E, 0x00CCA16E, 0x00CCA16E, 0x00CCA16E, 0x00CBA06D, 0x00CBA06D, 0x00D7A36E, 0x00CBA06D, 0x00A98E62, 0x00716541, 0x004A4B2B, 0x00454C31, 0x003A4329, 0x005B644A, 0x005C5B41, 0x006A6F50, 0x0068754F, 0x00607447, 0x00697E51,
        0x0074865D, 0x00748260, 0x00737D5F, 0x0076885F, 0x0075875E, 0x0074865D, 0x0073855C, 0x0072845B, 0x00708259, 0x006F8158, 0x006F8158, 0x006E8059, 0x00768460, 0x005F6B47, 0x0069764A, 0x006A7644, 0x004B5227, 0x0056543C, 0x006D655E, 0x004D5232,
        0x00494E2E, 0x00504F2D, 0x007F6C4B, 0x009A754F, 0x00C79A6E, 0x00C8A46E, 0x00BEA567, 0x00C99E6B, 0x00CA9F6C, 0x00CBA06D, 0x00CBA06D, 0x00C99E6B, 0x00C99E6B, 0x00CA9F6C, 0x00CBA06D, 0x00C8A26B, 0x00A78C5A, 0x00776C46, 0x004D5036, 0x00414A36,
        0x004C5142, 0x006A6759, 0x005D5244, 0x00556040, 0x005B6646, 0x005F6A49, 0x00707E5A, 0x00798862, 0x006E8156, 0x006F8257, 0x0075895C, 0x006E8057, 0x0071835A, 0x0073855C, 0x0072845B, 0x0071835A, 0x00708259, 0x006F8158, 0x00708259, 0x00626F4F,
        0x007B8862, 0x00657343, 0x00525F2B, 0x005A6435, 0x00585939, 0x006D6558, 0x00625654, 0x00615F54, 0x005E5D48, 0x0076704D, 0x008C7646, 0x00C39B66, 0x00CC9E68, 0x00C7A06C, 0x00BBA06E, 0x00CBA06D, 0x00CCA16E, 0x00CCA16E, 0x00CBA06D, 0x00CA9F6C,
        0x00CA9F6C, 0x00CCA16E, 0x00CDA26F, 0x00B99069, 0x00917C5C, 0x00626450, 0x00475248, 0x004E554E, 0x00747267, 0x0059553D, 0x00666344, 0x00607348, 0x006B7D56, 0x00768762, 0x007C8C63, 0x006C7D52, 0xFF7B8666, 0xFFFFFFF9, 0xFFFDF9FF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xCEE9ECDC, 0x0066714B, 0x00768656, 0x00808E64, 0x007C8A66, 0x00798763, 0x00778660, 0x00706D51, 0x00675E51, 0x00675E54, 0x004D5134, 0x004B5531,
        0x00555236, 0x00998360, 0x00CCA775, 0x00D3A673, 0x00D4A774, 0x00D4A774, 0x00D4A774, 0x00D3A673, 0x00D2A572, 0x00D3A673, 0x00D3A673, 0x00CBA470, 0x00CFA772, 0x00D5AB76, 0x00CFA773, 0x00A58457, 0x00887550, 0x005A5235, 0x0056563E, 0x00504B36,
        0x007B7366, 0x005F5C47, 0x00535D2F, 0x00616D3B, 0x00727D57, 0x00788460, 0x00728455, 0x00738157, 0x0079875D, 0x0078865C, 0x00768559, 0x0078875B, 0x00778758, 0x00758556, 0x00788859, 0x0078885F, 0x0078885F, 0x00738459, 0x00758359, 0x00707F53,
        0x006F7C50, 0x006F7C50, 0x00637042, 0x0061584E, 0x0073725E, 0x00505637, 0x00414728, 0x004D4A2E, 0x00887657, 0x00AE9164, 0x00D0AB71, 0x00D0A370, 0x00D0A370, 0x00CFA26F, 0x00CFA26F, 0x00CFA26F, 0x00CEA16E, 0x00CEA16E, 0x00CEA16E, 0x00C3A26A,
        0x00CEA46F, 0x00D4A676, 0x00B89268, 0x007D6B46, 0x00555134, 0x00494B35, 0x00474533, 0x00766D63, 0x0054523A, 0x00545D35, 0x00667549, 0x006D7D54, 0x00708057, 0x00717F4F, 0x006F8047, 0x006F8251, 0x00708352, 0x00718453, 0x00728554, 0x00738655,
        0x00738655, 0x00738655, 0x00738655, 0x00758254, 0x00758556, 0x00708155, 0x0072855A, 0x006E7E55, 0x00727D57, 0x00656946, 0x00575836, 0x00676858, 0x00555646, 0x004F503C, 0x00545037, 0x007C704E, 0x009A8155, 0x00CBA470, 0x00D0A169, 0x00CCA16E,
        0x00CBA06D, 0x00CBA06D, 0x00CBA06D, 0x00CBA06D, 0x00CBA06D, 0x00CBA06D, 0x00CCA16E, 0x00D5A16C, 0x00CCA16E, 0x00B79A6E, 0x0080724E, 0x00504C2F, 0x004B4E34, 0x00464C33, 0x00676D54, 0x005F5E42, 0x00626747, 0x0069774E, 0x006D8154, 0x006C8154,
        0x00697C51, 0x006E7D57, 0x00798463, 0x0075875E, 0x0075875E, 0x0074865D, 0x0073855C, 0x0072845B, 0x0071835A, 0x00708259, 0x006F8158, 0x00718459, 0x006F7E58, 0x0066734D, 0x0069764A, 0x0064713D, 0x004F5629, 0x00545339, 0x006E675E, 0x00545535,
        0x004A4D2D, 0x00524F30, 0x00867151, 0x00A67F59, 0x00CFA074, 0x00CBA36E, 0x00BB9F62, 0x00C99E6B, 0x00CA9F6C, 0x00CCA16E, 0x00CBA06D, 0x00CA9F6C, 0x00C99E6B, 0x00C99E6B, 0x00CA9F6C, 0x00D2A671, 0x00B49365, 0x0082724E, 0x0057563C, 0x004D5340,
        0x0054594A, 0x006D6A5B, 0x005B5244, 0x0064704C, 0x006E7A56, 0x0073805A, 0x0073825C, 0x0074845B, 0x0075885D, 0x00788B60, 0x0075885D, 0x0076885F, 0x0073855C, 0x0071835A, 0x0071835A, 0x0072845B, 0x0071835A, 0x0071835A, 0x0071835A, 0x006B7957,
        0x006E7C53, 0x00627040, 0x00525F2B, 0x004C5627, 0x00555634, 0x00766F5E, 0x00655B54, 0x00676358, 0x0064614C, 0x007E7553, 0x00967E50, 0x00CA9F6C, 0x00CF9F6B, 0x00C8A06C, 0x00C0A16E, 0x00CCA16E, 0x00CCA16E, 0x00CDA26F, 0x00CCA16E, 0x00CA9F6C,
        0x00CA9F6C, 0x00CBA06D, 0x00CCA16E, 0x00C59872, 0x009E8563, 0x00706D58, 0x00525B4E, 0x00545950, 0x00787567, 0x005F5B43, 0x006D6B4C, 0x0076885F, 0x00768762, 0x00768762, 0x007A8A61, 0x006B7C51, 0xFF7A8565, 0xFFFFFFF9, 0xFFFFFBFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xCAE6E9D9, 0x0067724C, 0x007A8A5B, 0x007F8D63, 0x00788662, 0x00778563, 0x0073825C, 0x006C6E50, 0x00665F4E, 0x006B6352, 0x004F502E, 0x00485028,
        0x005B5335, 0x00A38764, 0x00D5AC7B, 0x00D4A774, 0x00D3A673, 0x00D3A673, 0x00D3A673, 0x00D3A673, 0x00D3A673, 0x00D2A572, 0x00D2A572, 0x00D1A570, 0x00D1A570, 0x00D6A976, 0x00D3A97A, 0x00AD8B60, 0x008D7754, 0x00605438, 0x005B573E, 0x004E4A31,
        0x00756E5F, 0x005B5941, 0x00586234, 0x00626F3B, 0x00616D43, 0x0068774B, 0x006F854B, 0x00717F55, 0x0076845A, 0x0077855B, 0x0077865A, 0x0079885C, 0x00768657, 0x00758556, 0x007A8A5B, 0x0077885C, 0x0075865A, 0x00748559, 0x00778758, 0x00708051,
        0x006F7C4E, 0x00717E50, 0x00657244, 0x00615A4B, 0x006E6E56, 0x00464D2C, 0x003B411E, 0x004E482B, 0x00907959, 0x00BB9769, 0x00D6AC71, 0x00D1A471, 0x00D1A471, 0x00D1A471, 0x00D0A370, 0x00CFA26F, 0x00CFA26F, 0x00CEA16E, 0x00CDA06D, 0x00C7A367,
        0x00CFA36D, 0x00D2A272, 0x00BB966A, 0x008C7954, 0x005D5837, 0x0047462A, 0x0048442B, 0x006E6659, 0x0058573B, 0x00616C3E, 0x00667843, 0x00647746, 0x006E7F53, 0x00717F4F, 0x006D7E45, 0x00708352, 0x00708352, 0x00718453, 0x00728554, 0x00738655,
        0x00738655, 0x00738655, 0x00728554, 0x00788656, 0x00758555, 0x00708253, 0x006D8154, 0x00697A4F, 0x00707B53, 0x00676C45, 0x00545632, 0x006A6758, 0x005B5947, 0x0054533F, 0x0058533A, 0x00817252, 0x00A58960, 0x00D0A776, 0x00D3A16C, 0x00CCA16E,
        0x00CBA06D, 0x00CA9F6C, 0x00CA9F6C, 0x00CA9F6C, 0x00CA9F6C, 0x00CBA06D, 0x00CCA16E, 0x00D8A26D, 0x00CEA070, 0x00C0A075, 0x008B7855, 0x004D472A, 0x004A492F, 0x004A4A32, 0x0065644F, 0x00646446, 0x00656A4A, 0x0068754F, 0x006A7D52, 0x006B8053,
        0x0073875A, 0x0076875C, 0x00728057, 0x0074865D, 0x0074865D, 0x0073855C, 0x0073855C, 0x0072845B, 0x0071835A, 0x0071835A, 0x0071835A, 0x00718556, 0x006B7B52, 0x00707E55, 0x006F7C50, 0x0065723E, 0x00565E2F, 0x00535335, 0x006D6859, 0x00595538,
        0x00514F31, 0x00595435, 0x008D7858, 0x00AF8764, 0x00D3A276, 0x00CCA26D, 0x00C2A065, 0x00C99E6B, 0x00CBA06D, 0x00CCA16E, 0x00CCA16E, 0x00CA9F6C, 0x00C99E6B, 0x00C89D6A, 0x00C99E6B, 0x00D9A472, 0x00C3996E, 0x008D7754, 0x005E5A42, 0x00595C4C,
        0x005E6152, 0x00706D5E, 0x005C5443, 0x00627046, 0x006C7A50, 0x00728056, 0x006F8055, 0x00718257, 0x006E8057, 0x006C7E55, 0x0074865D, 0x00788A61, 0x0071835A, 0x006F8158, 0x0073855C, 0x0073855C, 0x0071835A, 0x006F8158, 0x006E8057, 0x0075835F,
        0x00707E54, 0x006C7B48, 0x00596632, 0x00495324, 0x00545632, 0x0076715C, 0x006B6258, 0x0070675A, 0x0068614D, 0x00817656, 0x009D8458, 0x00CDA271, 0x00D19F6B, 0x00CB9E6B, 0x00C4A16F, 0x00CCA16E, 0x00CDA26F, 0x00CDA26F, 0x00CCA16E, 0x00CA9F6C,
        0x00CA9F6C, 0x00CBA06D, 0x00CCA16E, 0x00CD9A72, 0x00A18360, 0x00736C53, 0x00555848, 0x00535647, 0x0073715F, 0x005B5A40, 0x006A6D4D, 0x00788A63, 0x00738361, 0x00728260, 0x00798862, 0x006D7E53, 0xFF7B8666, 0xFFFFFFF8, 0xFFFFFBFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xCEE7EADB, 0x0068734D, 0x00768657, 0x0079875E, 0x00798765, 0x007D8B69, 0x007B8965, 0x006B7251, 0x0065614E, 0x0069624E, 0x00484922, 0x0046481E,
        0x005D5232, 0x00A58764, 0x00D8AC7D, 0x00D5A875, 0x00D3A673, 0x00D2A572, 0x00D2A572, 0x00D3A673, 0x00D3A673, 0x00D2A572, 0x00D0A370, 0x00D4A56F, 0x00D1A26E, 0x00D3A575, 0x00D1A67B, 0x00AF8B65, 0x008C7353, 0x00625438, 0x005D563B, 0x00464426,
        0x00706A57, 0x005E5C44, 0x00525C2E, 0x00616E3A, 0x00727E54, 0x00708051, 0x00708749, 0x00707E54, 0x00738157, 0x00728056, 0x00748357, 0x0078875B, 0x00768657, 0x00758556, 0x007A8A5B, 0x00748756, 0x00758857, 0x00718453, 0x00768656, 0x00758556,
        0x00727F51, 0x00717E50, 0x00687547, 0x005C5845, 0x00686A4D, 0x003E4621, 0x003B3F1B, 0x0050482A, 0x00927959, 0x00BD9669, 0x00D5A66E, 0x00CFA26F, 0x00CFA26F, 0x00D0A370, 0x00D0A370, 0x00CFA26F, 0x00CFA26F, 0x00CEA16E, 0x00CDA06D, 0x00CCA567,
        0x00CFA068, 0x00D09F6F, 0x00BD976D, 0x008E7C57, 0x005E5A37, 0x004A4829, 0x004C4728, 0x0070695A, 0x00515133, 0x00535F2D, 0x00687B42, 0x006B7E4B, 0x006B7D4E, 0x00728050, 0x0074844E, 0x00708253, 0x00718354, 0x00728455, 0x00728455, 0x00738556,
        0x00738556, 0x00728455, 0x00728455, 0x00798757, 0x00718151, 0x006E8051, 0x006C8053, 0x0068794E, 0x006C784E, 0x00666B44, 0x00555831, 0x006D6655, 0x005A5644, 0x00544E3B, 0x00575037, 0x00837152, 0x00AF9069, 0x00D1A77A, 0x00D19E6C, 0x00CBA06D,
        0x00CBA06D, 0x00CBA06D, 0x00CBA06D, 0x00CBA06D, 0x00CBA06D, 0x00CBA06D, 0x00CBA06D, 0x00D8A16E, 0x00CC9E6E, 0x00C2A279, 0x008D7A59, 0x004C4427, 0x0049452D, 0x004B4833, 0x00625F4A, 0x00565638, 0x00666B4B, 0x00707C58, 0x006B7D54, 0x00667A4F,
        0x006C8053, 0x00738458, 0x00758458, 0x0073855C, 0x0073855C, 0x0073855C, 0x0072845B, 0x0072845B, 0x0072845B, 0x0072845B, 0x0072845B, 0x00718753, 0x00708156, 0x00717E58, 0x006D794F, 0x0064703E, 0x00545C2D, 0x00525333, 0x006D6957, 0x005D5538,
        0x00555134, 0x00615C3D, 0x00927D5E, 0x00B28A67, 0x00CF9D73, 0x00CFA06D, 0x00CAA46A, 0x00C99E6B, 0x00CBA06D, 0x00CCA16E, 0x00CCA16E, 0x00CA9F6C, 0x00C99E6B, 0x00C89D6A, 0x00C89D6A, 0x00D69E6F, 0x00C89C73, 0x00927959, 0x00635D46, 0x005D6050,
        0x005E6152, 0x006F6C5D, 0x005A5342, 0x00667647, 0x00748455, 0x0078875B, 0x00718257, 0x0075865B, 0x0071835A, 0x006B7D54, 0x00788A63, 0x0072845B, 0x006D7F56, 0x00708259, 0x0076885F, 0x0073855C, 0x006F8158, 0x006D7F56, 0x006B7D54, 0x0065744E,
        0x0068764C, 0x00687646, 0x0055612F, 0x004C5628, 0x00555733, 0x00716D54, 0x00756E5D, 0x00716357, 0x00675E4A, 0x007F7456, 0x009F865E, 0x00C99F72, 0x00D29D6B, 0x00CD9C6A, 0x00CAA26E, 0x00CDA26F, 0x00CDA26F, 0x00CDA26F, 0x00CCA16E, 0x00CA9F6C,
        0x00C99E6B, 0x00CA9F6C, 0x00CCA16E, 0x00D39F7A, 0x00A2825F, 0x0072674B, 0x00565743, 0x00545442, 0x0074705D, 0x005B5D40, 0x006A7150, 0x0071815F, 0x00718060, 0x00738262, 0x007A8963, 0x006D7E53, 0xFF7B8665, 0xFFFFFFF8, 0xFFFFFCFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xD1EAEDDE, 0x0066704C, 0x00728155, 0x007B8960, 0x007F8D6B, 0x00808D6D, 0x00778561, 0x00656C4B, 0x00615E49, 0x00675F48, 0x0046441B, 0x00494B1D,
        0x005F5635, 0x009F8364, 0x00D0A97D, 0x00D6A976, 0x00D4A774, 0x00D3A673, 0x00D3A673, 0x00D3A673, 0x00D3A673, 0x00D2A572, 0x00D0A370, 0x00D4A36B, 0x00D1A16D, 0x00D1A576, 0x00CEA57E, 0x00AC8C69, 0x00866E50, 0x00615336, 0x005B5336, 0x004F4D2E,
        0x006A6451, 0x0064614C, 0x00555D35, 0x00576136, 0x00636D49, 0x00606D47, 0x00758A53, 0x00748258, 0x00758359, 0x00728056, 0x00738256, 0x0079885C, 0x00788859, 0x00758556, 0x00768657, 0x00768956, 0x00798C59, 0x006F8251, 0x00738354, 0x00778758,
        0x00717E52, 0x006A764C, 0x00657147, 0x005E5845, 0x00646648, 0x003B421D, 0x00393F1C, 0x0050482B, 0x008C7456, 0x00BD976D, 0x00D7A973, 0x00CDA06D, 0x00CEA16E, 0x00CEA16E, 0x00CEA16E, 0x00CEA16E, 0x00CEA16E, 0x00CEA16E, 0x00CEA16E, 0x00CCA365,
        0x00CFA06A, 0x00D3A478, 0x00BD9973, 0x00807150, 0x0053512F, 0x004A4826, 0x004E4825, 0x00665F50, 0x005E5E40, 0x00596436, 0x00657545, 0x0066774C, 0x00677650, 0x00768357, 0x006F7E4B, 0x00718354, 0x00728455, 0x00728455, 0x00738556, 0x00738556,
        0x00738556, 0x00728455, 0x00728455, 0x00778456, 0x006D7D4E, 0x006F8054, 0x00718459, 0x00708156, 0x0069744C, 0x00636743, 0x00585A36, 0x006E6756, 0x005A5441, 0x00504A37, 0x00544C35, 0x007F6F52, 0x00AE936E, 0x00D0A67B, 0x00CF9F6F, 0x00CA9F6C,
        0x00CBA06D, 0x00CBA06D, 0x00CCA16E, 0x00CCA16E, 0x00CBA06D, 0x00CBA06D, 0x00CA9F6C, 0x00D3A16D, 0x00CCA071, 0x00C2A47B, 0x008B7A59, 0x004F462B, 0x004F4B33, 0x0055503B, 0x006F6A55, 0x00605F43, 0x00626649, 0x00646E50, 0x006B7B59, 0x0071835E,
        0x006E8057, 0x006D7E52, 0x00768657, 0x0072845B, 0x0072845B, 0x0072845B, 0x0072845B, 0x0072845B, 0x0072845B, 0x0072845B, 0x0072845B, 0x006D834F, 0x0078895E, 0x006F7B57, 0x006C7751, 0x00677146, 0x00535A2F, 0x00555537, 0x006B6754, 0x005E5338,
        0x00575035, 0x00605C3F, 0x008F7D5E, 0x00B38E6C, 0x00CE9E74, 0x00CE9F6C, 0x00CAA16A, 0x00C99E6B, 0x00CBA06D, 0x00CCA16E, 0x00CCA16E, 0x00CA9F6C, 0x00C99E6B, 0x00C99E6B, 0x00C99E6B, 0x00D59E73, 0x00C99D78, 0x0091795D, 0x00655E4A, 0x005C5F4F,
        0x00585B4B, 0x006D6B59, 0x005A5340, 0x00647248, 0x006D7B51, 0x0077855C, 0x0074845B, 0x00718158, 0x0074865D, 0x0076885F, 0x0072845B, 0x0071835A, 0x006D7F56, 0x0073855C, 0x00778960, 0x0071835A, 0x006E8057, 0x006F8158, 0x006F8158, 0x0072805C,
        0x006D7B52, 0x0069764A, 0x00566133, 0x004F572E, 0x00585937, 0x00716D54, 0x006D6853, 0x006B5B4E, 0x00615844, 0x007C7155, 0x00A18965, 0x00CAA076, 0x00D1A070, 0x00CE9D6B, 0x00CAA06B, 0x00CCA16E, 0x00CDA26F, 0x00CDA26F, 0x00CCA16E, 0x00CA9F6C,
        0x00CA9F6C, 0x00CBA06D, 0x00CCA16E, 0x00D1A381, 0x009A7E5C, 0x00696144, 0x0053523D, 0x0053523E, 0x00746F5A, 0x00595A40, 0x00666B4E, 0x00738262, 0x00778569, 0x007A896A, 0x007B8965, 0x00697A4F, 0xFF788362, 0xFFFFFFF7, 0xFFFFFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xD5E9EBDF, 0x00656F4B, 0x00758458, 0x00849269, 0x007B8868, 0x00657252, 0x00505E3A, 0x00505637, 0x00605A47, 0x00726751, 0x004D4B22, 0x00494C1E,
        0x00514E2F, 0x0088755A, 0x00C4A782, 0x00D6A976, 0x00D5A875, 0x00D4A774, 0x00D4A774, 0x00D4A774, 0x00D3A673, 0x00D2A572, 0x00D1A471, 0x00D2A46A, 0x00D2A671, 0x00D1A87B, 0x00C7A37D, 0x00A38768, 0x007A684B, 0x005D5234, 0x00574F31, 0x00575733,
        0x00696350, 0x005D5947, 0x00464A2D, 0x003E4326, 0x004D513E, 0x0049513A, 0x004F5F36, 0x00717F55, 0x0078865C, 0x0078865C, 0x00768559, 0x0079885C, 0x0079895A, 0x00758556, 0x00748455, 0x00748756, 0x00788B5A, 0x00738556, 0x00768559, 0x00717F55,
        0x005F6A42, 0x00515C36, 0x0047512D, 0x00635C49, 0x0067664A, 0x003A401D, 0x003A4120, 0x00454128, 0x0079684D, 0x00B0936C, 0x00D2AD7B, 0x00CFA26F, 0x00CFA26F, 0x00CEA16E, 0x00CDA06D, 0x00CDA06D, 0x00CEA16E, 0x00CEA16E, 0x00CFA26F, 0x00C99E65,
        0x00D2A370, 0x00D6A983, 0x00B29374, 0x006D6345, 0x00444727, 0x00454624, 0x004F4924, 0x0067604F, 0x005C5B41, 0x004A512E, 0x00475231, 0x00485339, 0x004F5A40, 0x006C7652, 0x007B885A, 0x00738458, 0x00738458, 0x00748559, 0x00748559, 0x00748559,
        0x00748559, 0x00738458, 0x00738458, 0x00778359, 0x00738157, 0x0073835A, 0x00667851, 0x00576541, 0x00434D29, 0x00474A2A, 0x00515133, 0x00706A57, 0x00595542, 0x004F4B38, 0x00524C35, 0x0074684C, 0x00A58D69, 0x00C5A277, 0x00CDA376, 0x00CBA06D,
        0x00CBA06D, 0x00CCA16E, 0x00CCA16E, 0x00CCA16E, 0x00CCA16E, 0x00CBA06D, 0x00CBA06D, 0x00CEA26D, 0x00CBA575, 0x00B99F77, 0x007F7050, 0x00494529, 0x004C4934, 0x00514D3A, 0x00746E5B, 0x0056533E, 0x0051533D, 0x004C543D, 0x00505D45, 0x00607051,
        0x0072835E, 0x0077885D, 0x00738354, 0x0072845B, 0x0072845B, 0x0072845B, 0x0072845B, 0x0072845B, 0x0072845B, 0x0072845B, 0x0072845B, 0x006E8352, 0x006F7E58, 0x00596345, 0x004E5637, 0x00474F2A, 0x00404420, 0x00555438, 0x006E6A58, 0x0060553A,
        0x00535034, 0x00575739, 0x00807557, 0x00AE906D, 0x00CDA178, 0x00CD9F6F, 0x00C49B64, 0x00C99E6B, 0x00CA9F6C, 0x00CBA06D, 0x00CA9F6C, 0x00C99E6B, 0x00C99E6B, 0x00CA9F6C, 0x00CBA06D, 0x00D0A279, 0x00BD9B77, 0x00837257, 0x005E5D49, 0x00565C4B,
        0x00545747, 0x006F6B59, 0x005B523E, 0x00505A3C, 0x004A5535, 0x005A6544, 0x0073815F, 0x0071805A, 0x006C7E57, 0x0076885F, 0x006D8055, 0x0076885F, 0x0072845B, 0x0076885F, 0x0076885F, 0x006F8158, 0x00708259, 0x0074865D, 0x00708259, 0x00677454,
        0x00525D3C, 0x004B5531, 0x00424A25, 0x003E4421, 0x00555537, 0x0078745B, 0x00615C47, 0x00645545, 0x005B5440, 0x00766F54, 0x009D8C6B, 0x00C5A179, 0x00D4A677, 0x00CFA06C, 0x00C69F68, 0x00CBA06D, 0x00CCA16E, 0x00CCA16E, 0x00CBA06D, 0x00CA9F6C,
        0x00CA9F6C, 0x00CCA16E, 0x00CDA26F, 0x00C29E80, 0x008A7559, 0x005F5C40, 0x0050523C, 0x00534F3C, 0x00716A59, 0x00575340, 0x0062654F, 0x006A785C, 0x00707D63, 0x00778569, 0x007D8B69, 0x006A7A51, 0xFF768160, 0xFFFCFFF4, 0xFFFEFCFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFBFF, 0xD5E9EBDF, 0x00646D4C, 0x0079875D, 0x0087946E, 0x006D7A5A, 0x004B5838, 0x003E4C2A, 0x004B5035, 0x00635C4D, 0x00776C56, 0x004D4B22, 0x0042491C,
        0x003A3E21, 0x00625A43, 0x00A59374, 0x00D6A976, 0x00D7AA77, 0x00D6A976, 0x00D5A875, 0x00D4A774, 0x00D2A572, 0x00D2A572, 0x00D2A572, 0x00D6AA6F, 0x00D5AB76, 0x00CBA67A, 0x00B59774, 0x008E795A, 0x006A5C3F, 0x00585032, 0x00575233, 0x004F4F2B,
        0x0076705D, 0x00615B50, 0x004F4F3D, 0x00666757, 0x009F9D9C, 0x009FA19B, 0x00666F5A, 0x00606E44, 0x00728056, 0x007B895F, 0x0077865A, 0x0077865A, 0x00788859, 0x00768657, 0x00768657, 0x00738556, 0x00728357, 0x0077885D, 0x0079875E, 0x00606C48,
        0x004C5534, 0x00474F30, 0x00394122, 0x00695C4E, 0x006A664E, 0x003C4122, 0x003B4325, 0x00373923, 0x005E533D, 0x0095815E, 0x00BBA173, 0x00D4A774, 0x00D3A673, 0x00D0A370, 0x00CEA16E, 0x00CDA06D, 0x00CDA06D, 0x00CEA16E, 0x00CFA26F, 0x00CCA26D,
        0x00D4A77B, 0x00C89D7C, 0x00967B61, 0x005C573E, 0x0042472A, 0x00474A2A, 0x00524E2B, 0x00726B5A, 0x004B4833, 0x0050533D, 0x007A7F70, 0x00979B95, 0x0080867B, 0x0062694E, 0x00768258, 0x00748559, 0x00748559, 0x0075865A, 0x0075865A, 0x0075865A,
        0x00748559, 0x00748559, 0x00738458, 0x0079845E, 0x00788460, 0x006D7B57, 0x0050603E, 0x00414E2E, 0x002F3719, 0x003B3C22, 0x004F4E34, 0x006D6956, 0x00575642, 0x004E503A, 0x004E4C34, 0x00645C3F, 0x00927E5B, 0x00B6986F, 0x00CBA67A, 0x00CEA370,
        0x00CDA26F, 0x00CCA16E, 0x00CBA06D, 0x00CBA06D, 0x00CCA16E, 0x00CDA26F, 0x00CEA370, 0x00CDA672, 0x00C6A577, 0x00A08C63, 0x00645B3A, 0x00444329, 0x004B4D37, 0x00474632, 0x00686753, 0x005E5A48, 0x007A7A6A, 0x00929788, 0x007C8675, 0x0057644E,
        0x00606F50, 0x007A8963, 0x007A895D, 0x0073855C, 0x0073855C, 0x0073855C, 0x0072845B, 0x0072845B, 0x0072845B, 0x0072845B, 0x0071835A, 0x00728657, 0x005A6846, 0x00485039, 0x00323821, 0x0024290C, 0x00343618, 0x0056543C, 0x00726C5F, 0x00645B40,
        0x00525135, 0x004C5033, 0x006B6647, 0x009E8662, 0x00C49E74, 0x00CCA372, 0x00C59F68, 0x00C99E6B, 0x00CA9F6C, 0x00CA9F6C, 0x00C99E6B, 0x00C89D6A, 0x00C99E6B, 0x00CCA16E, 0x00CEA370, 0x00C4A179, 0x00A68E6A, 0x006A6245, 0x004F543F, 0x004F5746,
        0x00535646, 0x00746E5B, 0x005F513E, 0x00696F5E, 0x00707663, 0x005B634C, 0x00667256, 0x00798765, 0x006E8059, 0x0073865B, 0x0074885B, 0x0076885F, 0x0072845B, 0x0075875E, 0x0075875E, 0x006F8158, 0x0073855C, 0x0072845B, 0x0064764D, 0x00404C30,
        0x00353E23, 0x00293113, 0x00272C0F, 0x002F3316, 0x004E4D33, 0x007A7560, 0x00635D4A, 0x005E5442, 0x0057523D, 0x006B6A50, 0x008E8363, 0x00B59873, 0x00CFA578, 0x00CDA36E, 0x00C6A167, 0x00CA9F6C, 0x00CBA06D, 0x00CBA06D, 0x00CBA06D, 0x00CBA06D,
        0x00CBA06D, 0x00CDA26F, 0x00CFA471, 0x00A89177, 0x00776E53, 0x00595D44, 0x00545944, 0x00575445, 0x00766D5F, 0x005E594A, 0x006B6C5C, 0x0069765C, 0x0066735B, 0x006E7B61, 0x007E8C6A, 0x006F7F56, 0xFF737E5D, 0xFFF7FBEF, 0xFFFEFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xD9EBEDE1, 0x00636C4B, 0x0076845A, 0x00828F69, 0x00657252, 0x004F5B3D, 0x00586644, 0x005D5F49, 0x00696053, 0x006F614E, 0x003F3F17, 0x003E481D,
        0x0031381D, 0x0044432E, 0x007C7457, 0x00D6A976, 0x00D7AA77, 0x00D8AB78, 0x00D6A976, 0x00D4A774, 0x00D2A572, 0x00D2A572, 0x00D3A673, 0x00D9B072, 0x00D6AE79, 0x00C3A275, 0x00A28864, 0x0079674A, 0x005A4F33, 0x00555031, 0x00585634, 0x00575831,
        0x00756E5D, 0x00665E57, 0x007B796E, 0x00827F7A, 0x008C8790, 0x009A989E, 0x00767C71, 0x004B592F, 0x0068764C, 0x0079875D, 0x00768559, 0x00738256, 0x00758556, 0x00778758, 0x0079895A, 0x00798A5E, 0x006E7F54, 0x0076865D, 0x0077845E, 0x00515C3B,
        0x00485032, 0x00596045, 0x0050573C, 0x006A5A4E, 0x006B6450, 0x003E4225, 0x003C452A, 0x002E331E, 0x0047422D, 0x007C6D4D, 0x00A18E63, 0x00D9AC79, 0x00D7AA77, 0x00D3A673, 0x00CFA26F, 0x00CDA06D, 0x00CDA06D, 0x00CEA16E, 0x00CFA26F, 0x00D2AA76,
        0x00D5AA7F, 0x00B28A6D, 0x0079614B, 0x0053503B, 0x00475036, 0x004A5031, 0x00565431, 0x006F6859, 0x00615D4B, 0x00727264, 0x00727371, 0x00909096, 0x00989B99, 0x005F644F, 0x0057623A, 0x0075865A, 0x0075865A, 0x0075865A, 0x0076875B, 0x0075865A,
        0x0075865A, 0x00748559, 0x00748559, 0x007A8362, 0x00737E5D, 0x005F6C4C, 0x00465536, 0x00505C40, 0x004C5239, 0x00505038, 0x00595641, 0x00676651, 0x00545640, 0x004E513B, 0x00484B32, 0x00565033, 0x00806F4E, 0x00A78D65, 0x00C8A67B, 0x00D1A673,
        0x00CFA471, 0x00CBA06D, 0x00C99E6B, 0x00C99E6B, 0x00CBA06D, 0x00CFA471, 0x00D1A673, 0x00CEA975, 0x00BDA172, 0x008A784F, 0x00514A29, 0x0047482E, 0x00565943, 0x00474834, 0x00636450, 0x00575144, 0x00807E73, 0x00BDC1B6, 0x00B0B8AD, 0x0064705E,
        0x00505D43, 0x00707E5A, 0x0078865C, 0x0074865D, 0x0073855C, 0x0073855C, 0x0073855C, 0x0072845B, 0x0072845B, 0x0071835A, 0x0071835A, 0x006A7E51, 0x00505D3D, 0x00626855, 0x00585C49, 0x0042462D, 0x00535538, 0x005F5B48, 0x006B655A, 0x00665F46,
        0x0054553B, 0x00464E30, 0x005A5839, 0x008B7754, 0x00B8966B, 0x00CDA574, 0x00D1AB75, 0x00C99E6B, 0x00C99E6B, 0x00C99E6B, 0x00C89D6A, 0x00C89D6A, 0x00C99E6B, 0x00CDA26F, 0x00CFA471, 0x00B59C72, 0x008F7F5B, 0x00535034, 0x00424934, 0x00495343,
        0x00535646, 0x0079725F, 0x0061523F, 0x00A2A59C, 0x00D0D4C9, 0x00808675, 0x004A543D, 0x00707D5D, 0x00778962, 0x00718558, 0x00738758, 0x0071835A, 0x006E8057, 0x0073855C, 0x0074865D, 0x0071835A, 0x0075875E, 0x006C7E55, 0x0053653C, 0x0068725B,
        0x006F785E, 0x00535940, 0x0040442B, 0x00464930, 0x00514E39, 0x00726C59, 0x00635D4A, 0x00615745, 0x0052513C, 0x00616248, 0x007E7658, 0x00A58B67, 0x00C7A074, 0x00CCA46F, 0x00C6A469, 0x00C99E6B, 0x00CA9F6C, 0x00CBA06D, 0x00CBA06D, 0x00CBA06D,
        0x00CCA16E, 0x00CEA370, 0x00CFA471, 0x008B7B64, 0x00615D45, 0x004D553E, 0x00505643, 0x00545142, 0x00736A5D, 0x0060574D, 0x00706E63, 0x007D8A72, 0x006C7862, 0x006B7860, 0x007E8C6A, 0x00708057, 0xFF707B5A, 0xFFF6FBEC, 0xFFFFFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFBFF, 0xD5EBEEDF, 0x00636E46, 0x00778653, 0x007C8A60, 0x00636C51, 0x0067705C, 0x00CCD3C4, 0x00AA9CA0, 0x00695F58, 0x00625D48, 0x004F4E2C, 0x0044451E,
        0x00474526, 0x00645F4A, 0x00635B4E, 0x007F7147, 0x00AB956B, 0x00D3B388, 0x00DEB588, 0x00E1B082, 0x00E5B683, 0x00E4B581, 0x00DAAE79, 0x00C7A784, 0x00BB9C7B, 0x009A8161, 0x00745F40, 0x005C4E32, 0x005B5035, 0x005A543D, 0x0058543C, 0x004F4E32,
        0x00747359, 0x005E5D49, 0x0089897B, 0x009D9D97, 0x00818181, 0x00919197, 0x0073757D, 0x0056663D, 0x0068784F, 0x0077875E, 0x0077885D, 0x0074855A, 0x0075865A, 0x0077885C, 0x0075865A, 0x00788B5A, 0x00748657, 0x007B8C61, 0x00737E5D, 0x00454D36,
        0x0084857B, 0x00C5C3C3, 0x00979399, 0x00625B4C, 0x0067614E, 0x003D3B1D, 0x0045461F, 0x00404218, 0x003D3F15, 0x00464720, 0x00696945, 0x00AA8E5F, 0x00C9A779, 0x00D9AF82, 0x00DAA77C, 0x00E0A97E, 0x00E1AC81, 0x00DCAB7F, 0x00D9AA7E, 0x00C2A487,
        0x00A58B6D, 0x00796347, 0x00615336, 0x005D5538, 0x004F4C30, 0x003F4228, 0x003B4025, 0x006A6352, 0x00605C49, 0x00909181, 0x00898B85, 0x00919394, 0x00949696, 0x00686B5B, 0x00586140, 0x0077885C, 0x0076875B, 0x0075865B, 0x0074855A, 0x0074855A,
        0x0074845B, 0x0075855C, 0x0076865D, 0x00718558, 0x007A8864, 0x0059604B, 0x00787B72, 0x0093928E, 0x009A9792, 0x00A3A196, 0x00545444, 0x0063684B, 0x00474B2E, 0x004D4F32, 0x00555236, 0x005A5338, 0x00554A2F, 0x0073644A, 0x00A08F75, 0x00C7A675,
        0x00D5AF7F, 0x00D4A879, 0x00D4A375, 0x00D2A173, 0x00D0A475, 0x00D5AF7F, 0x00CCAB7A, 0x00A79C7C, 0x007F7655, 0x005F5739, 0x00464324, 0x004A482A, 0x00434528, 0x003C3F25, 0x00666B50, 0x005D5849, 0x007E7C71, 0x00858781, 0x00919792, 0x00727B71,
        0x004D5943, 0x006B7955, 0x0079885C, 0x0074875C, 0x0074875C, 0x0074875C, 0x0074875C, 0x0074875C, 0x0074875C, 0x0074875C, 0x0074875C, 0x006C7959, 0x004D5342, 0x00888987, 0x00918E90, 0x00989590, 0x00959388, 0x00544F46, 0x006F6964, 0x0058573B,
        0x005A573B, 0x00585136, 0x00584C30, 0x0069563B, 0x008C7458, 0x00AE9176, 0x00BFA186, 0x00D7AE77, 0x00D8A975, 0x00D2A16F, 0x00CC9B6B, 0x00CFA074, 0x00D1AA7E, 0x00C5A57C, 0x00B79E74, 0x007E796A, 0x00565240, 0x00535139, 0x00504F33, 0x00504F35,
        0x00605D48, 0x00726C5F, 0x00564F46, 0x006C6B6F, 0x00787977, 0x007F8377, 0x00515B44, 0x00637050, 0x0072845D, 0x0076885F, 0x0075875E, 0x0074865F, 0x0072845D, 0x0073855E, 0x006F815A, 0x006D7F58, 0x00768861, 0x006B7D56, 0x004A5C35, 0x008A8D8B,
        0x009D9E9C, 0x007C7A79, 0x008D8A86, 0x008E8C82, 0x00494435, 0x00817B68, 0x006D6951, 0x00655941, 0x00695E48, 0x00635A46, 0x00635947, 0x007F6F58, 0x00A58D6F, 0x00C1A17D, 0x00CFAC81, 0x00D6A771, 0x00D6A672, 0x00D5A172, 0x00D29F74, 0x00D4A67D,
        0x00CEA983, 0x00B89D78, 0x009E8B66, 0x00636051, 0x005E5D49, 0x0053543A, 0x005B5D3F, 0x005C5E40, 0x00707157, 0x0051503C, 0x00787566, 0x00929992, 0x00636C5F, 0x00626F55, 0x0078895E, 0x006F824F, 0xFF727F59, 0xFFFEFFF8, 0xFFFBF5FF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xD5EBEEDF, 0x00616C44, 0x00748454, 0x00808E64, 0x00697559, 0x0057604C, 0x00909985, 0x00888277, 0x00625E4C, 0x00605F43, 0x003E401C, 0x00373A13,
        0x003E401C, 0x004D4D2F, 0x004B4A30, 0x003F3E22, 0x00635D40, 0x008E7E61, 0x00A99173, 0x00B79A7B, 0x00BEA07D, 0x00BDA07B, 0x00B69B76, 0x00A39065, 0x0088774C, 0x006D5D33, 0x0060532D, 0x005B522D, 0x00534D2A, 0x00494423, 0x00423F20, 0x0049472F,
        0x00706D58, 0x00595844, 0x00727264, 0x0083847A, 0x00737470, 0x00878A88, 0x00737877, 0x005C6C43, 0x006A7A51, 0x0076865D, 0x0075865B, 0x00728358, 0x00738458, 0x00748559, 0x00728357, 0x00738458, 0x0078895D, 0x00798A5F, 0x006F7C56, 0x004D5739,
        0x007C806D, 0x00ADAEA5, 0x008F8D8C, 0x00696154, 0x006A6451, 0x003A371B, 0x003F3E1C, 0x0043441D, 0x003E3F18, 0x00343410, 0x0042411F, 0x006C5E34, 0x0088764D, 0x00AE946C, 0x00BE9D76, 0x00BB9771, 0x00BD9973, 0x00BB9C75, 0x00B1946D, 0x008E7C57,
        0x0073633E, 0x005E502C, 0x004A411F, 0x003B3615, 0x00383919, 0x00383B1B, 0x0034391A, 0x0070685B, 0x00595543, 0x00797B68, 0x008B8F84, 0x00B1B4B2, 0x00B1B5AF, 0x00747A63, 0x00535E36, 0x00738458, 0x00738458, 0x00728357, 0x00728358, 0x00738459,
        0x0074845B, 0x0075855C, 0x0075855C, 0x00738758, 0x0072815B, 0x00646D53, 0x00BDC2B3, 0x00F9FAF1, 0x00B7B6AC, 0x007B7B6B, 0x00565743, 0x00656B4C, 0x003B4021, 0x003C3F1F, 0x003F3D1E, 0x00423D1C, 0x004C4321, 0x00605732, 0x00665A36, 0x009B8256,
        0x00B4976B, 0x00C4A176, 0x00CDA77D, 0x00CDA77D, 0x00C7A479, 0x00BB9E72, 0x00A58C60, 0x006E6740, 0x005C5730, 0x0048451F, 0x00363411, 0x003B3A18, 0x00393C1C, 0x003A3F1F, 0x005E6344, 0x0065614F, 0x00787769, 0x0075786F, 0x00838A83, 0x006F796C,
        0x00505D45, 0x006B7955, 0x0079885C, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x005D6B49, 0x00636A55, 0x00DDDFD9, 0x00FBFAF6, 0x00B9B9AD, 0x00838371, 0x00555244, 0x006B655E, 0x00555432,
        0x004C4B29, 0x004E4928, 0x005B5230, 0x00625634, 0x006C5B3A, 0x00867350, 0x00A38F6C, 0x00BA9968, 0x00C39F6F, 0x00CDA679, 0x00CDA57B, 0x00C7A37B, 0x00BD9D79, 0x00A78F6B, 0x00917F5A, 0x00504E36, 0x00535236, 0x004F5030, 0x004A4B29, 0x00434422,
        0x004B4B2D, 0x006F6D55, 0x00595542, 0x00646464, 0x006F706C, 0x00797E6F, 0x00545F45, 0x00667452, 0x0073855E, 0x00778960, 0x00778960, 0x0074865F, 0x0071835C, 0x0073855E, 0x0071835C, 0x006F815A, 0x00768861, 0x006C7E57, 0x0051633C, 0x0070746F,
        0x00B9BAB6, 0x00DFDFD9, 0x009C9A92, 0x00757368, 0x00625D4E, 0x00716C57, 0x005A563E, 0x0059502E, 0x00595133, 0x00564F34, 0x005D553E, 0x0070634D, 0x00837258, 0x00998165, 0x00AE9172, 0x00C0A273, 0x00C5A477, 0x00C8A67B, 0x00C5A47D, 0x00B89C79,
        0x00A08B6B, 0x00807555, 0x00686344, 0x005A5749, 0x005E5C4A, 0x0055553D, 0x00525339, 0x0054553B, 0x006F6F57, 0x005B5947, 0x00959284, 0x00C5CDC2, 0x007F8979, 0x0068755B, 0x007A8B60, 0x00708350, 0xFF6D7A54, 0xFFFDFFF7, 0xFFFFFAFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xD1EAEDDE, 0x00606D47, 0x00728252, 0x007F8D64, 0x00748062, 0x00525C45, 0x005F6952, 0x00636149, 0x00616046, 0x00747357, 0x00505131, 0x004F502E,
        0x00565834, 0x00494B27, 0x003F411D, 0x00454B34, 0x00464930, 0x00504B32, 0x0062573C, 0x00716347, 0x0077674A, 0x00756646, 0x00726343, 0x005C5827, 0x00534F1F, 0x004F4B1B, 0x00535023, 0x0057532A, 0x00504E26, 0x004A4820, 0x0048451F, 0x0054503D,
        0x00777360, 0x005F5D4B, 0x00616252, 0x00727565, 0x00737869, 0x00868D7E, 0x007F887B, 0x0066764D, 0x00708057, 0x0076865D, 0x0074855A, 0x00728358, 0x00738458, 0x00748559, 0x00728357, 0x0077885D, 0x0076875C, 0x00728358, 0x0078865C, 0x005E6A46,
        0x00565E40, 0x00626851, 0x005C604D, 0x00686053, 0x00746D5C, 0x004E4934, 0x004D4A2E, 0x004D4B2C, 0x00444223, 0x00373516, 0x00464426, 0x003C401C, 0x004B4B27, 0x00615D3A, 0x00766C4A, 0x00817553, 0x00817553, 0x00766C4A, 0x00685F3D, 0x00565229,
        0x004D4920, 0x004C4A22, 0x0042411B, 0x0043431F, 0x004B4C2A, 0x00454926, 0x00474A2A, 0x00776E65, 0x00595445, 0x00616350, 0x006E7364, 0x00878C83, 0x00848B7C, 0x00697152, 0x00647242, 0x00728455, 0x00728357, 0x00728357, 0x00738458, 0x00738459,
        0x0075865B, 0x0076865D, 0x0076865D, 0x00728756, 0x007A8B60, 0x00606B4B, 0x00818972, 0x00B2B6A3, 0x00878976, 0x00575943, 0x00585B41, 0x006A6D53, 0x00494D30, 0x004D4D2F, 0x00494826, 0x0043401A, 0x00433F16, 0x00534D22, 0x00544E21, 0x00655635,
        0x00756443, 0x00826D4D, 0x008E7757, 0x00957E5E, 0x008E7959, 0x007A6948, 0x005F502F, 0x004D4920, 0x0047451C, 0x003C3A12, 0x00393812, 0x003F411D, 0x003C401D, 0x00474B28, 0x00656A4A, 0x005E5A47, 0x00676755, 0x00636859, 0x00798275, 0x00768270,
        0x005E6C50, 0x00707F59, 0x0079885C, 0x0072855A, 0x0072855A, 0x0072855A, 0x0072855A, 0x0072855A, 0x0072855A, 0x0072855A, 0x0072855A, 0x00728259, 0x00626B50, 0x00ADB2A3, 0x00BEC1B2, 0x00757763, 0x004F5238, 0x00555440, 0x00716E60, 0x00464422,
        0x0043411E, 0x00474320, 0x00514D2A, 0x005B5530, 0x005C5631, 0x00615A33, 0x00655E37, 0x008C7751, 0x00957E58, 0x00A08864, 0x00A28967, 0x00927D5D, 0x00807151, 0x006A5F41, 0x004F472A, 0x00454626, 0x005E5F3D, 0x00484B24, 0x003E411A, 0x00393C15,
        0x00434422, 0x00737355, 0x005E5D41, 0x005F615B, 0x00696D62, 0x00757B68, 0x005C684A, 0x006D7B57, 0x0074865F, 0x0075875E, 0x0076885F, 0x0075875E, 0x0071835A, 0x0073855C, 0x0074865D, 0x0071835A, 0x0075875E, 0x006F8158, 0x005D6F46, 0x008A9085,
        0x00FAFEF3, 0x00DCDED2, 0x00666757, 0x00525240, 0x0055513E, 0x006B6651, 0x0058533E, 0x00484219, 0x0047411C, 0x00494627, 0x00575239, 0x00605A43, 0x00615640, 0x00685A44, 0x007B6B54, 0x00847A58, 0x00857957, 0x0087795C, 0x00887C60, 0x007F755D,
        0x0066644C, 0x004C4F39, 0x003A412C, 0x004F4C3E, 0x00575445, 0x0053523E, 0x00474631, 0x004B4A35, 0x00696854, 0x00545142, 0x00939082, 0x009AA494, 0x00677160, 0x00637155, 0x007E8F64, 0x00788B5A, 0xFF717E58, 0xFFF6FAEF, 0xFFF9F5FF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDF9FE, 0xCEE5EADB, 0x00626F49, 0x00738256, 0x00788761, 0x007A8668, 0x00616F53, 0x005E6D4E, 0x0058573B, 0x005B5A40, 0x00787560, 0x005C5845, 0x005E5B46,
        0x00646349, 0x004A4B2B, 0x0041431F, 0x004E4E30, 0x00413F20, 0x00353213, 0x003A3312, 0x00443B19, 0x004B411F, 0x00514521, 0x00534723, 0x00434A1F, 0x00444A21, 0x0043471E, 0x0042451E, 0x0050502C, 0x00605E3B, 0x005E5C3A, 0x00544F2E, 0x005F5A4B,
        0x00716D5B, 0x00565442, 0x004C4E3A, 0x005E644D, 0x00677056, 0x00717D61, 0x00717F63, 0x006F7F56, 0x0074845B, 0x0076865D, 0x0074855A, 0x00738459, 0x0076875B, 0x0076875B, 0x00748559, 0x0075845E, 0x00728259, 0x00718257, 0x00819064, 0x00758458,
        0x0058633B, 0x004D5733, 0x004C5534, 0x00696455, 0x00757061, 0x005A5643, 0x00605D48, 0x0066654B, 0x0055543A, 0x003D3C22, 0x00444327, 0x004B5237, 0x0051563B, 0x00474A30, 0x00494A30, 0x005C5B41, 0x0057563C, 0x004B4A30, 0x00535238, 0x00505431,
        0x004F5330, 0x00454828, 0x00353719, 0x00525135, 0x0066654B, 0x0058573D, 0x0068664E, 0x00786F66, 0x005A5546, 0x00595C46, 0x00646C55, 0x00656E5A, 0x00616B54, 0x006D7850, 0x0080905A, 0x00748657, 0x00748657, 0x00748559, 0x0075865A, 0x0075865A,
        0x0076875C, 0x0077885D, 0x0077885D, 0x00718753, 0x00849667, 0x0078855F, 0x005B6646, 0x005E654A, 0x0064694E, 0x00585D3E, 0x00545939, 0x006C6B5D, 0x00555243, 0x00514D3A, 0x00565239, 0x005C593A, 0x0047441E, 0x0047431A, 0x00545125, 0x005F5C40,
        0x005C583C, 0x00565236, 0x00575035, 0x00575035, 0x00545034, 0x00534F33, 0x004E4B2F, 0x004D4B29, 0x00464422, 0x003C391A, 0x00525032, 0x00616044, 0x00515238, 0x0054553B, 0x0067674F, 0x0064624A, 0x006C6E58, 0x00676E59, 0x0079856F, 0x007D8C71,
        0x006F7E5E, 0x0076855F, 0x0077865A, 0x0072855A, 0x0072855A, 0x0072855A, 0x0072855A, 0x0072855A, 0x0072855A, 0x0072855A, 0x0072855A, 0x0075865A, 0x00636E4D, 0x0069715A, 0x0070765F, 0x005E6445, 0x004E5431, 0x0056573D, 0x00696755, 0x004F482D,
        0x00534C31, 0x00534F33, 0x00565236, 0x005A583A, 0x005C5C3E, 0x00565737, 0x004C4F2F, 0x00585033, 0x0060583B, 0x00696144, 0x006A6348, 0x005D583F, 0x0057533A, 0x0055553D, 0x00484B32, 0x00444426, 0x005F6040, 0x00505131, 0x004F5030, 0x004F4F31,
        0x0059583C, 0x0079785C, 0x005B5A40, 0x00767B6C, 0x007B816E, 0x00858E74, 0x0072805E, 0x00788761, 0x0073855C, 0x006E8057, 0x006E8057, 0x0075875E, 0x00708259, 0x0073855C, 0x0076885F, 0x0074865D, 0x0074865D, 0x0071835A, 0x00697B52, 0x00737B6A,
        0x00CBD1BE, 0x00898F78, 0x0054583F, 0x00676A51, 0x0058563E, 0x006E6954, 0x00534E39, 0x00615A39, 0x005C5738, 0x00575336, 0x00575438, 0x0058573D, 0x00535238, 0x00535238, 0x005D5C42, 0x00605E46, 0x005E5B46, 0x005D5946, 0x005B5947, 0x005B5B4B,
        0x00595C4D, 0x00555C4F, 0x00525B4E, 0x004C493B, 0x00524F40, 0x005B5849, 0x00545240, 0x00595745, 0x00737061, 0x00545142, 0x00807D6F, 0x008E9A84, 0x0076836B, 0x00758465, 0x00798960, 0x006C7E4F, 0xFF717D59, 0xFFFFFFF8, 0xFFFFFBFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFBFD, 0xCAE3E8D9, 0x00626E4A, 0x00738459, 0x0073825C, 0x007A8969, 0x00738262, 0x0070815C, 0x0064654B, 0x00575642, 0x00757264, 0x006B655E, 0x006A655C,
        0x00696658, 0x004A4934, 0x00414326, 0x0036381B, 0x0037391C, 0x0037391C, 0x0036381A, 0x0037391B, 0x003B3B1D, 0x003F3D1F, 0x00403C1F, 0x004B5237, 0x0052593E, 0x004C5136, 0x003C3F25, 0x00424127, 0x0059553D, 0x0066624A, 0x00645E47, 0x006D6957,
        0x006C6855, 0x0052513C, 0x004A4F34, 0x005F6748, 0x00677251, 0x0065744E, 0x00667851, 0x00708057, 0x0073835A, 0x0073835A, 0x00728358, 0x00738459, 0x0075865A, 0x0076875B, 0x0075865A, 0x006F7D59, 0x0076865D, 0x0075865A, 0x00758556, 0x00768656,
        0x00768355, 0x006F7B51, 0x005E6941, 0x00605C49, 0x006F6B59, 0x006B6755, 0x00777360, 0x0074705D, 0x00535139, 0x00343319, 0x003B3A20, 0x00524F3A, 0x00565540, 0x0051503B, 0x0052513C, 0x005A5944, 0x0053503B, 0x004C4934, 0x005C5742, 0x005C5F49,
        0x00585B45, 0x00494A36, 0x00353622, 0x00504E3C, 0x00696455, 0x005C5649, 0x006B6558, 0x007A7167, 0x00524F3A, 0x004B5132, 0x00646F4F, 0x00667256, 0x00687555, 0x00758256, 0x0076884D, 0x00748657, 0x00758758, 0x00758758, 0x0075865A, 0x0075865A,
        0x0075865A, 0x0074855A, 0x0074855A, 0x00738857, 0x006D7F50, 0x007A885E, 0x0073805A, 0x0067714D, 0x00686F4C, 0x005B613E, 0x004F5532, 0x006F6865, 0x00655F5A, 0x00696259, 0x006D6657, 0x0066614C, 0x00413C23, 0x003E3A1D, 0x00524D2C, 0x00605F4A,
        0x00595B45, 0x00585A44, 0x00545741, 0x004A4D37, 0x00444630, 0x004B4D37, 0x005F5E49, 0x00615C47, 0x00514C37, 0x003B3724, 0x00534F3C, 0x006D6957, 0x00676253, 0x00646153, 0x00676456, 0x00525135, 0x005F6346, 0x005D6749, 0x00637253, 0x006C7D5C,
        0x006D7E59, 0x0074845B, 0x00768559, 0x0072855A, 0x0072855A, 0x0072855A, 0x0072855A, 0x0072855A, 0x0072855A, 0x0072855A, 0x0072855A, 0x00748657, 0x0076825E, 0x00616B4D, 0x005D6546, 0x005E673F, 0x004D552C, 0x00545638, 0x0072715D, 0x006A5F51,
        0x006A5F51, 0x00685F51, 0x00645D4C, 0x005A5644, 0x0054533F, 0x00585945, 0x00626450, 0x00535139, 0x00504E36, 0x00494931, 0x004C4F39, 0x004C4F39, 0x004A5039, 0x004D503A, 0x003D402A, 0x0045432B, 0x0053503B, 0x005B5744, 0x0064604E, 0x00605C4A,
        0x00676351, 0x006F6B59, 0x0054503E, 0x0059614A, 0x0060694E, 0x006E7959, 0x006B7A54, 0x0076875C, 0x0075885D, 0x0076885F, 0x00798B64, 0x0076885F, 0x0071835A, 0x0073855C, 0x00778960, 0x0075875E, 0x0073855C, 0x0073855C, 0x0071835A, 0x00677258,
        0x00677055, 0x005C6544, 0x00505734, 0x004D5232, 0x00555438, 0x006E6A52, 0x006F6956, 0x00675F4E, 0x0068614E, 0x005E5841, 0x00535034, 0x00545436, 0x00575A3A, 0x00515835, 0x004C5632, 0x0057503F, 0x00514C3D, 0x00494637, 0x00474638, 0x00555348,
        0x00656358, 0x0069675D, 0x00646258, 0x0052503E, 0x00545240, 0x006B6859, 0x006D6A5B, 0x00696657, 0x007B7869, 0x00585644, 0x006E6C5A, 0x006F7E5F, 0x00707E62, 0x007A8969, 0x00798960, 0x00697B4C, 0xFF717D59, 0xFFFDFFF5, 0xFFFCF9FF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xC6DFE6D7, 0x005C6A48, 0x0076865D, 0x0071825D, 0x00788767, 0x00758762, 0x0072865B, 0x006A6B51, 0x004E4D38, 0x00656253, 0x0069645B, 0x00676259,
        0x00625F51, 0x00484733, 0x00404028, 0x00474D3C, 0x00454F3E, 0x00445240, 0x00465645, 0x004F6150, 0x005B6B5A, 0x005D6858, 0x00576151, 0x00414532, 0x004E503D, 0x00535542, 0x004A4A38, 0x0044432F, 0x004C4B37, 0x00625E4B, 0x00716D5A, 0x0067654D,
        0x00605E46, 0x00545639, 0x00595F40, 0x006E7854, 0x00748259, 0x006F8055, 0x006F8356, 0x006E7E55, 0x006E7E55, 0x006F7F56, 0x00708156, 0x00718257, 0x00738458, 0x00748559, 0x00748559, 0x00768460, 0x0077875E, 0x0077885C, 0x00718152, 0x00748454,
        0x007B885A, 0x007B885C, 0x0069744C, 0x00605F45, 0x006E6C54, 0x00777360, 0x00827E6C, 0x00777360, 0x00595641, 0x0049482E, 0x0059583C, 0x006C614B, 0x00625A43, 0x00716D55, 0x00918F77, 0x00A0A088, 0x009D9B83, 0x0088826B, 0x006E664F, 0x00696A56,
        0x005E5F4B, 0x00575745, 0x004C493A, 0x00535042, 0x006D695E, 0x00777368, 0x00807970, 0x00726C59, 0x0058583A, 0x00586139, 0x00728057, 0x0072805C, 0x00778561, 0x007E8B5D, 0x00788950, 0x00738655, 0x00738655, 0x00748657, 0x00748657, 0x00748559,
        0x00738458, 0x00728357, 0x00728358, 0x00798D60, 0x00718256, 0x0078865C, 0x0078865C, 0x00768159, 0x0079815C, 0x006B724D, 0x00595F3C, 0x006A615D, 0x006F6760, 0x00827B72, 0x007D776A, 0x00655E4D, 0x0049442F, 0x0058543B, 0x005D5940, 0x006C6954,
        0x00656751, 0x0072755F, 0x007B836C, 0x007A826B, 0x0071745E, 0x00676953, 0x00706D58, 0x0063624E, 0x005B5A46, 0x00474533, 0x00524F40, 0x006F695C, 0x00787267, 0x00767065, 0x00696259, 0x005A5B3B, 0x006D7350, 0x006F7B57, 0x006A7C55, 0x006F8359,
        0x0072845B, 0x00738459, 0x00748258, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0073865B, 0x0076875B, 0x007D8A64, 0x0075805F, 0x007A855F, 0x00778156, 0x005A6338, 0x0056583A, 0x006D6B59, 0x00786C62,
        0x007A7164, 0x00776E61, 0x006A6255, 0x00605B4C, 0x00625D4E, 0x00646250, 0x00656351, 0x0065624D, 0x006F6E59, 0x006A6D57, 0x00696F58, 0x0060694F, 0x005A6047, 0x00676A51, 0x00696951, 0x00595641, 0x00565240, 0x00635D50, 0x00756F64, 0x00756E65,
        0x007C766B, 0x00706A5D, 0x005B5647, 0x006C7757, 0x00717C5B, 0x0078855F, 0x0075865B, 0x0077885C, 0x0072855A, 0x0072845B, 0x0072845D, 0x0076895E, 0x0073865B, 0x0074875C, 0x00778A5F, 0x0076895E, 0x0073865B, 0x0073865B, 0x0074875C, 0x007C8969,
        0x006E7A56, 0x00778359, 0x00677146, 0x004B532A, 0x00565737, 0x0068644C, 0x007F7867, 0x007F786F, 0x007D7568, 0x0068614D, 0x00545034, 0x00595735, 0x00656743, 0x00656D48, 0x00616C46, 0x00776D5C, 0x00857E6D, 0x008B8775, 0x007B7967, 0x006C695A,
        0x006D6657, 0x0073685A, 0x00776A5C, 0x0056563E, 0x00585742, 0x00706E5C, 0x00767364, 0x00696657, 0x00747260, 0x00605F4A, 0x0073735B, 0x007A8B66, 0x00768664, 0x00798967, 0x00798960, 0x006C7D51, 0xFF707C58, 0xFFFAFEF2, 0xFFFFFDFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFFFF, 0xC2DDE4D5, 0x005A6846, 0x00758760, 0x0072835E, 0x00748564, 0x0073865F, 0x006E8454, 0x006C724F, 0x004B5030, 0x004A4E31, 0x00474A30, 0x00464930,
        0x004A4D33, 0x00404427, 0x00404526, 0x00353523, 0x00282E1D, 0x001C2816, 0x001B2B19, 0x00273928, 0x00304231, 0x00303D2F, 0x002A3326, 0x002D2C17, 0x002D2C17, 0x0032311C, 0x003B3B23, 0x0042422A, 0x0045462C, 0x0048492F, 0x00494C32, 0x004A4A2C,
        0x004D4E2E, 0x00525633, 0x005F6742, 0x006C774F, 0x006F7D53, 0x006D7E52, 0x006C8051, 0x006D7D54, 0x006E7E55, 0x00708057, 0x00728358, 0x00738459, 0x00738458, 0x00748559, 0x0075865A, 0x00798862, 0x0075855C, 0x007A8B5F, 0x007A8A5B, 0x007F8F60,
        0x007B875D, 0x007C875F, 0x0079835F, 0x00646747, 0x00585A3D, 0x0053533B, 0x005B5A46, 0x00605F4B, 0x005A5A42, 0x00535538, 0x00575A3A, 0x0054492E, 0x003E371C, 0x004A492D, 0x006C7053, 0x007E8366, 0x00818366, 0x006A674B, 0x00423B20, 0x00423F23,
        0x004A472B, 0x005C583F, 0x00626048, 0x00615F47, 0x0064634E, 0x0062614D, 0x00585945, 0x00514E32, 0x005B5E37, 0x00697543, 0x00798959, 0x00728259, 0x0074835D, 0x00768357, 0x007B8B56, 0x00748756, 0x00758857, 0x00768958, 0x00768859, 0x00768859,
        0x0076875B, 0x0075865A, 0x0075865A, 0x006A7C53, 0x00798A5F, 0x0079875D, 0x00717F55, 0x00737F55, 0x007C855D, 0x00747B56, 0x00555B38, 0x005B5843, 0x005B5843, 0x00615F47, 0x00616046, 0x00616046, 0x005D5C40, 0x00676749, 0x0059593B, 0x00494229,
        0x00545037, 0x0075785E, 0x0092977C, 0x009A9F84, 0x0084876D, 0x005D5940, 0x00494229, 0x004F5336, 0x00585C3F, 0x00595A40, 0x005F5F47, 0x006C6C54, 0x0073705B, 0x00696552, 0x005B5744, 0x005D5F3B, 0x00707850, 0x0076845A, 0x00708457, 0x0074895C,
        0x00768A5D, 0x00738458, 0x0079875D, 0x0074875C, 0x0074875C, 0x0074875C, 0x0074875C, 0x0074875C, 0x0074875C, 0x0074875C, 0x0074875C, 0x0074855A, 0x00717D59, 0x00737E5D, 0x007A855D, 0x007C8759, 0x006C754A, 0x00585A3C, 0x00585546, 0x006A6550,
        0x00726D58, 0x00716A56, 0x0067614A, 0x0067614A, 0x006E6950, 0x00666148, 0x00555037, 0x00635B44, 0x00959179, 0x00A9AC92, 0x00A9AE93, 0x00868B70, 0x00585C3F, 0x00585539, 0x0060593E, 0x006E6E50, 0x006D6C52, 0x006F6B58, 0x007B7667, 0x00787265,
        0x00746F60, 0x005F5C47, 0x00535139, 0x00707D57, 0x0075835A, 0x0078875B, 0x00768859, 0x00758758, 0x00728659, 0x0074865D, 0x0074865F, 0x0075885D, 0x0075885D, 0x0075885D, 0x0076895E, 0x0076895E, 0x0074875C, 0x0073865B, 0x0073865B, 0x0074835D,
        0x00839167, 0x00788656, 0x006C7945, 0x00576132, 0x00535531, 0x0058543C, 0x00635C4D, 0x00616252, 0x00676652, 0x0068634A, 0x00696143, 0x006A6244, 0x005F5B3E, 0x004E4D31, 0x0043462D, 0x00767259, 0x00949379, 0x00A4A78D, 0x008C9176, 0x00606349,
        0x004E4A31, 0x005E543C, 0x00756750, 0x00686A4C, 0x0067694C, 0x006E6D58, 0x006C6A58, 0x005C5A48, 0x005E5D48, 0x00595B3E, 0x00747658, 0x00798C61, 0x00748560, 0x00778863, 0x007B8B62, 0x00738458, 0xFF717D59, 0xFFF6FBEC, 0xFFFCFBFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF9FBFB, 0xBEDBE2D3, 0x005A6747, 0x00758760, 0x0071815F, 0x00728361, 0x0073875D, 0x006F8554, 0x00818B5D, 0x00677045, 0x00596237, 0x004E562D, 0x004C542B,
        0x00515931, 0x004A522A, 0x004B522D, 0x00564B2D, 0x00544C2E, 0x004F4F31, 0x004A5233, 0x00485234, 0x00454C31, 0x0043462C, 0x00444329, 0x00464327, 0x00484529, 0x004A4A2C, 0x004E4E30, 0x004D5030, 0x004B512E, 0x004B512E, 0x004B522F, 0x00494A28,
        0x00555733, 0x00656945, 0x00717A52, 0x00717D53, 0x006E7D51, 0x00708155, 0x006C8051, 0x00708057, 0x00728259, 0x0074845B, 0x0076875C, 0x0077885D, 0x0076875B, 0x0076875B, 0x0077885C, 0x0071805A, 0x00798960, 0x00809165, 0x00758458, 0x007B8A5E,
        0x007C875F, 0x007F8965, 0x007E8667, 0x007B7F5B, 0x00696C4C, 0x00606147, 0x005E5D48, 0x005E5D49, 0x00606048, 0x00616345, 0x00606441, 0x006C6446, 0x00636041, 0x005E6343, 0x005A6544, 0x00545F3E, 0x00535C3B, 0x005B5E3E, 0x00605E3F, 0x00615C35,
        0x0069633E, 0x0063603A, 0x005C5C38, 0x005D5E3C, 0x00595C3C, 0x00575C3D, 0x005A5F40, 0x00676546, 0x00797D53, 0x007B8953, 0x007A8C57, 0x0077885D, 0x007C8B65, 0x00768258, 0x00818F5F, 0x00778A59, 0x00788B5A, 0x00798C5B, 0x007A8C5D, 0x007B8D5E,
        0x007B8D5E, 0x007B8C60, 0x007A8B5F, 0x007C8E67, 0x007B8A64, 0x00738157, 0x007B895F, 0x007C885E, 0x007C855D, 0x00838966, 0x00707555, 0x00545632, 0x005A5C38, 0x004F512D, 0x00484A26, 0x00555733, 0x004C4E2A, 0x004F512D, 0x00484C28, 0x00584D31,
        0x00524C2F, 0x00525234, 0x00474C2D, 0x00454A2B, 0x004F4F31, 0x00524C2F, 0x005C5135, 0x00465026, 0x004F572F, 0x00535B33, 0x00535733, 0x00525633, 0x00515232, 0x00515133, 0x00575537, 0x0070734C, 0x007C845B, 0x007D8C60, 0x00728657, 0x00758B5A,
        0x00768A5B, 0x0075865A, 0x00839167, 0x0075885D, 0x0075885D, 0x0075885D, 0x0075885D, 0x0075885D, 0x0075885D, 0x0075885D, 0x0075885D, 0x0073835A, 0x00788460, 0x007D8867, 0x0077825A, 0x007D8959, 0x007D865B, 0x00626447, 0x00585547, 0x00504E2F,
        0x004D4B2C, 0x004F4E2C, 0x00575533, 0x00585332, 0x00514B28, 0x004B4522, 0x004D4422, 0x004B3F27, 0x005A533A, 0x0049482C, 0x00484C2F, 0x004B5031, 0x00424021, 0x00494123, 0x004E4325, 0x00464922, 0x00565735, 0x004F4E32, 0x0053503B, 0x004E4B36,
        0x004D4B33, 0x00545436, 0x00676846, 0x0078865D, 0x007C8B5F, 0x007C8C5D, 0x00798C5B, 0x00758857, 0x0073875A, 0x00778960, 0x00758760, 0x0075885D, 0x0076895E, 0x0075885D, 0x0075885D, 0x0075885D, 0x0074875C, 0x0072855A, 0x0072855A, 0x0076875C,
        0x00778758, 0x007F8F5A, 0x007E8C56, 0x0065703E, 0x00666942, 0x005A563E, 0x00514A3B, 0x004A5037, 0x004A4E31, 0x004F4C2D, 0x005B5231, 0x005E5436, 0x00524830, 0x00494130, 0x004E483D, 0x00494B2D, 0x00454A2B, 0x00404829, 0x003F4A2A, 0x00424B2A,
        0x00464929, 0x004D4829, 0x00524729, 0x004B4F2C, 0x004C4E30, 0x004A4A32, 0x004D4C38, 0x004D4C38, 0x0056563E, 0x00606244, 0x00878B68, 0x00798D60, 0x00798B64, 0x007C8D68, 0x007A8A61, 0x00728358, 0xFF747F5E, 0xFFF9FEEF, 0xFFFFFEFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xBEDBDFD3, 0x00596444, 0x0077875E, 0x00788662, 0x00778765, 0x0071835C, 0x0073875A, 0x00718459, 0x00708358, 0x00708358, 0x006F8257, 0x006D8055,
        0x006B7E53, 0x006A7D52, 0x00697C51, 0x006B7C50, 0x006C7D51, 0x006D7E52, 0x006E7F53, 0x006E7F53, 0x006E7F53, 0x006E7F53, 0x006D7E52, 0x00697A4E, 0x0068794D, 0x0068794D, 0x0067784C, 0x0066774B, 0x0066774B, 0x0065764A, 0x0065764A, 0x0066764D,
        0x0066764D, 0x0066764D, 0x0067774E, 0x006A7A51, 0x006D7D54, 0x00708057, 0x00728259, 0x00728259, 0x00728259, 0x0073835A, 0x0073835A, 0x0074845B, 0x0074845B, 0x0075855C, 0x0075855C, 0x0077875E, 0x0078885F, 0x0078885F, 0x0078885F, 0x0078885F,
        0x00798960, 0x00798960, 0x00798960, 0x007B8B62, 0x007C8C63, 0x007C8C63, 0x007D8D64, 0x007E8E65, 0x007E8E65, 0x007F8F66, 0x007F8F66, 0x00829066, 0x00819064, 0x007F8E62, 0x007D8C60, 0x007B8B5C, 0x0079895A, 0x00778757, 0x00768656, 0x0083936A,
        0x00829269, 0x007F8F66, 0x007D8D64, 0x007B8B62, 0x007B8B62, 0x007C8C63, 0x007D8D64, 0x0076865D, 0x0076865D, 0x0076865D, 0x0076865D, 0x0076865D, 0x0076865D, 0x0076865D, 0x0076865D, 0x0076865D, 0x0077875E, 0x0077875E, 0x0077875E, 0x0078885F,
        0x0078885F, 0x0078885F, 0x00798960, 0x007A8B5F, 0x00798A5E, 0x0077885C, 0x0076875B, 0x0076875B, 0x0077885C, 0x00798A5E, 0x007A8B5F, 0x00778563, 0x00788664, 0x00788664, 0x00798765, 0x007A8866, 0x007A8866, 0x007B8967, 0x007B8967, 0x007C8B65,
        0x00798862, 0x0075845E, 0x0072815B, 0x0071805A, 0x0072815B, 0x0075845E, 0x00778660, 0x007B8965, 0x007B8965, 0x007A8864, 0x007A8864, 0x007A8864, 0x007A8864, 0x007A8864, 0x007A8864, 0x007C8B65, 0x007A8963, 0x00788761, 0x00778660, 0x00778660,
        0x00788761, 0x007A8963, 0x007C8B65, 0x00758760, 0x00758760, 0x00758760, 0x00758760, 0x00758760, 0x00758760, 0x00758760, 0x00758760, 0x00798F5F, 0x00788E5E, 0x00768C5C, 0x00768C5C, 0x00778D5D, 0x00758B5B, 0x00728858, 0x006F8555, 0x00707D5D,
        0x00717E5E, 0x00738060, 0x00738060, 0x00738060, 0x00717E5E, 0x006F7C5C, 0x006E7B5B, 0x00718047, 0x006D7B45, 0x006A7743, 0x006D7949, 0x00737E50, 0x00788258, 0x00778058, 0x00747D55, 0x00737B5C, 0x00707859, 0x006D7556, 0x006C7455, 0x006E7657,
        0x00737B5C, 0x00798162, 0x007D8566, 0x00778A59, 0x00778A59, 0x00768958, 0x00768958, 0x00768958, 0x00768958, 0x00758857, 0x00758857, 0x00778A59, 0x00778A59, 0x00778A59, 0x00778A59, 0x00778A59, 0x00778A59, 0x00778A59, 0x00778A59, 0x00778660,
        0x00788761, 0x007A8963, 0x007A8963, 0x00788761, 0x0074835D, 0x006F7E58, 0x006B7A54, 0x006C7554, 0x006D7655, 0x006F7857, 0x00717A59, 0x00717A59, 0x00717A59, 0x006F7857, 0x006F7857, 0x00656F41, 0x00677045, 0x006A7249, 0x006D754D, 0x006F7552,
        0x00707555, 0x00707457, 0x00707457, 0x006C785A, 0x00697557, 0x00667254, 0x00677355, 0x006C785A, 0x00737F61, 0x00798567, 0x007C886A, 0x0078895E, 0x007E8C68, 0x00768462, 0x007B8A64, 0x00748559, 0xFF73825C, 0xFFF3FCE7, 0xFFFDFFFC, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBFAFE, 0xBEDEE3D4, 0x005D6945, 0x00768859, 0x0074855A, 0x006F805B, 0x006D7F58, 0x00718459, 0x0072845D, 0x0072845D, 0x0072845D, 0x0072845D, 0x0072845D,
        0x0071835C, 0x0070825B, 0x0070825B, 0x00728259, 0x00718158, 0x00718158, 0x00708057, 0x006F7F56, 0x006E7E55, 0x006E7E55, 0x006D7D54, 0x006E7E55, 0x006D7D54, 0x006D7D54, 0x006C7C53, 0x006B7B52, 0x006B7B52, 0x006A7A51, 0x006A7A51, 0x006C7D52,
        0x006B7C51, 0x006B7C51, 0x006B7C51, 0x006C7D52, 0x006E7F54, 0x00708156, 0x00718257, 0x00718257, 0x00718257, 0x00728358, 0x00738459, 0x00738459, 0x0074855A, 0x0075865B, 0x0075865B, 0x0078895E, 0x0078895E, 0x0078895E, 0x00798A5F, 0x00798A5F,
        0x00798A5F, 0x00798A5F, 0x00798A5F, 0x007C8D62, 0x007C8D62, 0x007C8D62, 0x007C8D62, 0x007D8E63, 0x007D8E63, 0x007D8E63, 0x007D8E63, 0x007F8E62, 0x007F8E62, 0x007F8F60, 0x007F8F60, 0x007F8F5F, 0x007F8F5F, 0x007F8F5F, 0x007E8F5C, 0x007B8C61,
        0x007B8C61, 0x007D8E63, 0x007E8F64, 0x007F9065, 0x007E8F64, 0x007E8F64, 0x007D8E63, 0x007C8D62, 0x007C8D62, 0x007C8D62, 0x007C8D62, 0x007C8D62, 0x007C8D62, 0x007C8D62, 0x007C8D62, 0x0077885D, 0x0078895E, 0x0078895E, 0x0078895E, 0x0078895E,
        0x0078895E, 0x0078895E, 0x0078895E, 0x007C8B65, 0x007B8A64, 0x007A8963, 0x00798862, 0x00798862, 0x007A8963, 0x007B8A64, 0x007C8B65, 0x007C8B65, 0x007C8B65, 0x007D8C66, 0x007D8C66, 0x007E8D67, 0x007F8E68, 0x007F8E68, 0x007F8E68, 0x0084946B,
        0x0083936A, 0x00819168, 0x007F8F66, 0x007E8E65, 0x007F8F66, 0x00809067, 0x00819168, 0x007F8F66, 0x007F8F66, 0x007F8F66, 0x007E8E65, 0x007E8E65, 0x007E8E65, 0x007D8D64, 0x007D8D64, 0x007C8C63, 0x007B8B62, 0x007A8A61, 0x0078885F, 0x0078885F,
        0x0078885F, 0x0078885F, 0x00798960, 0x00788A61, 0x00788A61, 0x00788A61, 0x00798B62, 0x00798B62, 0x007A8C63, 0x007A8C63, 0x007A8C63, 0x00778B61, 0x00768A60, 0x0075895F, 0x00768A60, 0x00788C62, 0x00798D63, 0x00798D63, 0x00788C62, 0x0082916B,
        0x0082916B, 0x0083926C, 0x0083926C, 0x0082916B, 0x0081906A, 0x007F8E68, 0x007E8D67, 0x00818F5F, 0x007E8C5C, 0x007C895B, 0x007E8A60, 0x00838E66, 0x0086906C, 0x00858E6D, 0x00848D6C, 0x00828F69, 0x00828F69, 0x00818E68, 0x00808D67, 0x007F8C66,
        0x007D8A64, 0x007C8963, 0x007B8862, 0x0078885F, 0x0077875E, 0x0077875E, 0x0077875E, 0x0077875E, 0x0076865D, 0x0076865D, 0x0076865D, 0x0077875E, 0x0077875E, 0x0077875E, 0x0077875E, 0x0077875E, 0x0077875E, 0x0077875E, 0x0077875E, 0x0075855C,
        0x0077875E, 0x00798960, 0x007C8C63, 0x007D8D64, 0x007E8E65, 0x007D8D64, 0x007D8D64, 0x007F8C66, 0x00808D67, 0x00828F69, 0x0083906A, 0x0083906A, 0x00828F69, 0x00818E68, 0x00808D67, 0x007F8B5B, 0x00808B5D, 0x00828C61, 0x00858F65, 0x00868F67,
        0x00878E6B, 0x00868D6C, 0x00868D6C, 0x00818F6B, 0x007F8D69, 0x007D8B67, 0x007B8965, 0x007B8965, 0x007C8A66, 0x007E8C68, 0x007E8C68, 0x007C8C63, 0x00818F6D, 0x00798567, 0x007E8C6A, 0x0072815B, 0xFF727F5F, 0xFFF4FBEE, 0xFFFAFBFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFCFE, 0xC6E2E8D7, 0x00616F45, 0x00758954, 0x00798C5B, 0x0073855C, 0x0074865F, 0x0076885F, 0x0070815C, 0x0070815C, 0x0071825D, 0x0071825D, 0x0070815C,
        0x0070815C, 0x006F805B, 0x006F805B, 0x0071805A, 0x006F7E58, 0x006D7C56, 0x006B7A54, 0x006A7953, 0x006A7953, 0x006A7953, 0x006A7953, 0x006C7B55, 0x006C7B55, 0x006C7B55, 0x006C7B55, 0x006C7B55, 0x006C7B55, 0x006C7B55, 0x006C7B55, 0x006E7F53,
        0x006D7E52, 0x006C7D51, 0x006C7D51, 0x006C7D51, 0x006D7E52, 0x006E7F53, 0x006F8054, 0x00708155, 0x00708155, 0x00718256, 0x00728357, 0x00738458, 0x00748559, 0x0075865A, 0x0075865A, 0x0075865A, 0x0075865A, 0x0076875B, 0x0076875B, 0x0076875B,
        0x0076875B, 0x0077885C, 0x0077885C, 0x00798A5E, 0x00798A5E, 0x00798A5E, 0x00798A5E, 0x0078895D, 0x0078895D, 0x0078895D, 0x0078895D, 0x007A8A5B, 0x007B8B5C, 0x007B8B5C, 0x007C8C5C, 0x007D8D5D, 0x007E8F5C, 0x007E8F5C, 0x007E8F5C, 0x0078895D,
        0x00798A5E, 0x007A8B5F, 0x007B8C60, 0x007A8B5F, 0x00798A5E, 0x0077885C, 0x0076875B, 0x0077885C, 0x0077885C, 0x0077885C, 0x0077885C, 0x0077885C, 0x0077885C, 0x0077885C, 0x0077885C, 0x00798A5E, 0x00798A5E, 0x00798A5E, 0x00798A5E, 0x00798A5E,
        0x0078895D, 0x0078895D, 0x0078895D, 0x00798765, 0x00798765, 0x00788664, 0x00778563, 0x00778563, 0x00788664, 0x00798765, 0x00798765, 0x007C8C63, 0x007C8C63, 0x007C8C63, 0x007D8D64, 0x007D8D64, 0x007E8E65, 0x007E8E65, 0x007E8E65, 0x007B8C60,
        0x007B8C60, 0x007B8C60, 0x007B8C60, 0x007B8C60, 0x007B8C60, 0x007A8B5F, 0x007A8B5F, 0x007D8E63, 0x007D8E63, 0x007E8F64, 0x007E8F64, 0x007E8F64, 0x007D8E63, 0x007C8D62, 0x007B8C61, 0x00798A5E, 0x00798A5E, 0x00798A5E, 0x0078895D, 0x0078895D,
        0x0078895D, 0x0078895D, 0x0078895D, 0x0074885B, 0x0075895C, 0x0075895C, 0x00768A5D, 0x00778B5E, 0x00788C5F, 0x00788C5F, 0x00798D60, 0x00758563, 0x00748462, 0x00738361, 0x00748462, 0x00768664, 0x00788866, 0x007A8A68, 0x007A8A68, 0x007E8F63,
        0x007E8F63, 0x007D8E62, 0x007D8E62, 0x007C8D61, 0x007C8D61, 0x007B8C60, 0x007B8C60, 0x007A8B60, 0x00798A5F, 0x0078885F, 0x00798862, 0x007B8965, 0x007D8B69, 0x007D8A6A, 0x007C886A, 0x00778B5E, 0x00788C5F, 0x007B8F62, 0x007C9063, 0x007C9063,
        0x007A8E61, 0x00778B5E, 0x00768A5D, 0x00798763, 0x00798763, 0x00798763, 0x00798763, 0x00798763, 0x00788662, 0x00788662, 0x00788662, 0x00798763, 0x00798763, 0x00798763, 0x00798763, 0x00798763, 0x00798763, 0x00798763, 0x00798763, 0x00798A5E,
        0x00798A5E, 0x007A8B5F, 0x007B8C60, 0x007D8E62, 0x007F9064, 0x00809165, 0x00819266, 0x007D9162, 0x007E9263, 0x007F9364, 0x007F9364, 0x007F9364, 0x007E9263, 0x007D9162, 0x007C9061, 0x007C8D5A, 0x007D8D5D, 0x007E8E5F, 0x007F8E62, 0x00808E64,
        0x007F8D64, 0x007F8C66, 0x007E8B65, 0x007B8C61, 0x007C8D62, 0x007D8E63, 0x007E8F64, 0x007D8E63, 0x007D8E63, 0x007C8D62, 0x007C8D62, 0x007B8C61, 0x007F8D69, 0x00798666, 0x007F8C6C, 0x006E7C5A, 0xFF788369, 0xFFFDFFFB, 0xFFFDFCFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFDFF, 0xBADEE4D1, 0x00617044, 0x006F844D, 0x007E915E, 0x007B8D64, 0x00788866, 0x0071825D, 0x0077885D, 0x0077885D, 0x0077885D, 0x0076875C, 0x0075865B,
        0x0074855A, 0x00728358, 0x00718257, 0x00718256, 0x00708155, 0x006F8054, 0x006F8054, 0x006E7F53, 0x006E7F53, 0x006E7F53, 0x006F8054, 0x006C7D51, 0x006C7D51, 0x006C7D51, 0x006D7E52, 0x006D7E52, 0x006E7F53, 0x006E7F53, 0x006E7F53, 0x006D7E53,
        0x006D7E53, 0x006D7E53, 0x006D7E53, 0x006E7F54, 0x006F8055, 0x00708156, 0x00718257, 0x00738459, 0x00738459, 0x0074855A, 0x0075865B, 0x0076875C, 0x0077885D, 0x0078895E, 0x0078895E, 0x00798A5F, 0x00798A5F, 0x00798A5F, 0x007A8B60, 0x007A8B60,
        0x007A8B60, 0x007A8B60, 0x007A8B60, 0x007C8D62, 0x007C8D62, 0x007C8D62, 0x007C8D62, 0x007C8D62, 0x007C8D62, 0x007C8D62, 0x007C8D62, 0x007E8C62, 0x007E8C62, 0x007E8D61, 0x007E8D61, 0x007E8E5F, 0x007E8E5F, 0x007D8D5E, 0x007D8D5D, 0x007F9065,
        0x007E8F64, 0x007C8D62, 0x007B8C61, 0x007A8B60, 0x007B8C61, 0x007B8C61, 0x007C8D62, 0x007B8C61, 0x007B8C61, 0x007B8C61, 0x007B8C61, 0x007B8C61, 0x007B8C61, 0x007B8C61, 0x007B8C61, 0x007C8D62, 0x007C8D62, 0x007B8C61, 0x007B8C61, 0x007A8B60,
        0x007A8B60, 0x007A8B60, 0x007A8B60, 0x007A8864, 0x007A8864, 0x00798763, 0x00788662, 0x00788662, 0x00798763, 0x007A8864, 0x007A8864, 0x007C8C63, 0x007C8C63, 0x007C8C63, 0x007D8D64, 0x007D8D64, 0x007D8D64, 0x007E8E65, 0x007E8E65, 0x007B8C61,
        0x007C8D62, 0x007D8E63, 0x007E8F64, 0x007E8F64, 0x007E8F64, 0x007E8F64, 0x007D8E63, 0x007D8D64, 0x007E8E65, 0x007F8F66, 0x00809067, 0x007F8F66, 0x007E8E65, 0x007C8C63, 0x007B8B62, 0x007C8D62, 0x007C8D62, 0x007C8D62, 0x007C8D62, 0x007C8D62,
        0x007D8E63, 0x007D8E63, 0x007E8F64, 0x007A8B60, 0x007A8B60, 0x007A8B60, 0x007A8B60, 0x007B8C61, 0x007B8C61, 0x007C8D62, 0x007C8D62, 0x007B8D66, 0x007A8C65, 0x007A8C65, 0x007A8C65, 0x007A8C65, 0x007B8D66, 0x007B8D66, 0x007C8E67, 0x007E9061,
        0x007E9061, 0x007D8F60, 0x007D8F60, 0x007D8F60, 0x007D8F60, 0x007E9061, 0x007E9061, 0x007B8E63, 0x007B8E63, 0x007B8D64, 0x007B8D64, 0x007B8D66, 0x007C8D68, 0x007C8D68, 0x007C8D68, 0x007B9161, 0x007B9161, 0x007B9161, 0x007A9060, 0x00798F5F,
        0x00788E5E, 0x00778D5D, 0x00778D5D, 0x00798862, 0x00798862, 0x00788761, 0x00788761, 0x00788761, 0x00788761, 0x00778660, 0x00778660, 0x00788761, 0x00788761, 0x00788761, 0x00788761, 0x00788761, 0x00788761, 0x00788761, 0x00788761, 0x007B8C60,
        0x007B8C60, 0x007A8B5F, 0x007A8B5F, 0x007A8B5F, 0x007B8C60, 0x007B8C60, 0x007C8D61, 0x007A905F, 0x007A905F, 0x007B9160, 0x007B9160, 0x007A905F, 0x00798F5E, 0x00788E5D, 0x00788E5D, 0x007D9162, 0x007D9162, 0x007E9263, 0x007E9265, 0x007D9065,
        0x007C8F64, 0x007B8D64, 0x007B8D64, 0x007F9162, 0x00819364, 0x00839566, 0x00829465, 0x00809263, 0x007D8F60, 0x007C8E5F, 0x007B8D5E, 0x00778A57, 0x007E8F64, 0x007E8D67, 0x0084936D, 0x006D7D54, 0xFF7B8769, 0xFFFBFFF6, 0xFFF6F5FE, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xB7DAE0CF, 0x00626F49, 0x005D6F40, 0x0067774E, 0x006A7959, 0x006C7A5E, 0x00697859, 0x006B7C50, 0x006B7C50, 0x006B7C50, 0x006A7B4F, 0x00697A4E,
        0x0068794D, 0x0067784C, 0x0066774B, 0x00667849, 0x00667849, 0x00667849, 0x00667849, 0x00667849, 0x00657748, 0x00647647, 0x00637546, 0x00667849, 0x00667849, 0x00657748, 0x00647647, 0x00637546, 0x00627445, 0x00627445, 0x00617344, 0x00677553,
        0x00677553, 0x00677553, 0x00677553, 0x00677553, 0x00687654, 0x00697755, 0x00697755, 0x006C7A58, 0x006C7A58, 0x006D7B59, 0x006E7C5A, 0x006F7D5B, 0x006F7D5B, 0x00707E5C, 0x00707E5C, 0x00748260, 0x00748260, 0x00748260, 0x00748260, 0x00758361,
        0x00758361, 0x00758361, 0x00758361, 0x00758361, 0x00768462, 0x00768462, 0x00778563, 0x00778563, 0x00788664, 0x00788664, 0x00788664, 0x007A8564, 0x00798463, 0x00798463, 0x00788460, 0x00788460, 0x0077845E, 0x0077845E, 0x0077845E, 0x00758361,
        0x00758361, 0x00758361, 0x00768462, 0x00778563, 0x00788664, 0x00798765, 0x007A8866, 0x007B8967, 0x007B8967, 0x007B8967, 0x007B8967, 0x007B8967, 0x007B8967, 0x007B8967, 0x007B8967, 0x00788664, 0x00788664, 0x00778563, 0x00778563, 0x00768462,
        0x00768462, 0x00768462, 0x00768462, 0x007A8963, 0x007A8963, 0x00798862, 0x00798862, 0x00798862, 0x00798862, 0x007A8963, 0x007A8963, 0x007A8866, 0x007A8866, 0x007A8866, 0x007A8866, 0x007A8866, 0x007A8866, 0x007B8967, 0x007B8967, 0x007D8B67,
        0x007D8B67, 0x007D8B67, 0x007E8C68, 0x007E8C68, 0x007F8D69, 0x007F8D69, 0x00808E6A, 0x007B8967, 0x007C8A68, 0x007D8B69, 0x007E8C6A, 0x007E8C6A, 0x007D8B69, 0x007B8967, 0x007A8866, 0x007C8A66, 0x007B8965, 0x00798763, 0x00778561, 0x00778561,
        0x00788662, 0x00798763, 0x007A8864, 0x007B8965, 0x007A8864, 0x007A8864, 0x007A8864, 0x00798763, 0x00798763, 0x00798763, 0x00788662, 0x00798B62, 0x007A8C63, 0x007A8C63, 0x007A8C63, 0x00798B62, 0x00798B62, 0x00788A61, 0x00788A61, 0x007B8B62,
        0x007A8A61, 0x007A8A61, 0x007A8A61, 0x007B8B62, 0x007C8C63, 0x007E8E65, 0x007F8F66, 0x007A8D62, 0x007B8E63, 0x007B8E63, 0x007B8E63, 0x007A8D62, 0x007A8C63, 0x007A8C63, 0x007B8D64, 0x007E9268, 0x007E9268, 0x007D9167, 0x007D9167, 0x007C9066,
        0x007C9066, 0x007C9066, 0x007C9066, 0x007D8E63, 0x007D8E63, 0x007D8E63, 0x007D8E63, 0x007D8E63, 0x007C8D62, 0x007C8D62, 0x007C8D62, 0x007D8E63, 0x007D8E63, 0x007D8E63, 0x007D8E63, 0x007D8E63, 0x007D8E63, 0x007D8E63, 0x007D8E63, 0x007E8D67,
        0x007F8E68, 0x007F8E68, 0x0081906A, 0x0082916B, 0x0083926C, 0x0083926C, 0x0084936D, 0x0082966B, 0x0082966B, 0x0082966B, 0x0082966B, 0x0081956A, 0x0081956A, 0x00809469, 0x00809469, 0x0080926B, 0x0080926B, 0x0080926B, 0x007F916A, 0x007E9069,
        0x007D8E69, 0x007D8E69, 0x007C8D68, 0x00819168, 0x00829269, 0x0084946B, 0x0083936A, 0x00809067, 0x007E8E65, 0x007D8D64, 0x007E8E65, 0x007D915C, 0x007F9162, 0x007B8C61, 0x00798A5E, 0x00647647, 0xFF7E8D67, 0xFFFAFFF1, 0xFFFDFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBFAFE, 0xC6DEE2D7, 0x008F987E, 0x00838F71, 0x007E8772, 0x007D8777, 0x007D8874, 0x007D8A72, 0x00848F6E, 0x00848F6E, 0x0085906F, 0x04869170, 0x04869170,
        0x04869170, 0x04869170, 0x0085906F, 0x0084926E, 0x0084926E, 0x0084926E, 0x0084926E, 0x0082906C, 0x00808E6A, 0x007E8C68, 0x007D8B67, 0x007B8965, 0x007A8864, 0x00788662, 0x00768460, 0x0073815D, 0x00707E5A, 0x006E7C58, 0x006D7B57, 0x006C7560,
        0x006B745F, 0x006B745F, 0x006A735E, 0x0069725D, 0x0069725D, 0x0068715C, 0x0068715C, 0x006B745F, 0x006B745F, 0x006C7560, 0x006C7560, 0x006D7661, 0x006D7661, 0x006D7661, 0x006E7762, 0x006D7661, 0x006D7661, 0x006D7661, 0x006E7762, 0x006E7762,
        0x006E7762, 0x006E7762, 0x006E7762, 0x006E7762, 0x006E7762, 0x006F7863, 0x006F7863, 0x00707964, 0x00717A65, 0x00727B66, 0x00727B66, 0x00727865, 0x00727865, 0x00727964, 0x00727964, 0x00737B64, 0x00737B64, 0x00737B64, 0x00737C62, 0x006C7560,
        0x006E7762, 0x00707964, 0x00727B66, 0x00727B66, 0x00717A65, 0x006E7762, 0x006D7661, 0x006C7560, 0x006C7560, 0x006C7560, 0x006C7560, 0x006C7560, 0x006C7560, 0x006C7560, 0x006C7560, 0x006D7661, 0x006D7661, 0x006D7661, 0x006D7661, 0x006D7661,
        0x006C7560, 0x006C7560, 0x006C7560, 0x006F7B5D, 0x006F7B5D, 0x006F7B5D, 0x006F7B5D, 0x006F7B5D, 0x006F7B5D, 0x006F7B5D, 0x006F7B5D, 0x006D7760, 0x006D7760, 0x006D7760, 0x006D7760, 0x006D7760, 0x006D7760, 0x006D7760, 0x006D7760, 0x006D7760,
        0x006C765F, 0x006C765F, 0x006C765F, 0x006C765F, 0x006D7760, 0x006E7861, 0x006F7962, 0x006F7863, 0x00707964, 0x00707964, 0x00717A65, 0x00707964, 0x00707964, 0x006E7762, 0x006E7762, 0x006F7962, 0x006D7760, 0x0069735C, 0x00667059, 0x00646E57,
        0x00646E57, 0x00656F58, 0x00667059, 0x006A725B, 0x006A725B, 0x0069715A, 0x00687059, 0x00687059, 0x00676F58, 0x00666E57, 0x00666E57, 0x00626F4F, 0x00647151, 0x00667353, 0x00667353, 0x00667353, 0x00657252, 0x00647151, 0x00647151, 0x00626E52,
        0x00626E52, 0x00626E52, 0x00626E52, 0x00636F53, 0x00657155, 0x00667256, 0x00677357, 0x00647250, 0x00647250, 0x00657351, 0x00657351, 0x00647250, 0x00647250, 0x0063714F, 0x0063714F, 0x00616F53, 0x00637155, 0x00657357, 0x00677559, 0x00677559,
        0x00667458, 0x00647256, 0x00637155, 0x00677553, 0x00677553, 0x00677553, 0x00667452, 0x00667452, 0x00667452, 0x00667452, 0x00657351, 0x00667452, 0x00667452, 0x00667452, 0x00667452, 0x00667452, 0x00667452, 0x00667452, 0x00667452, 0x00647054,
        0x00657155, 0x00667256, 0x00687458, 0x00687458, 0x00697559, 0x00687458, 0x00687458, 0x00677559, 0x00677559, 0x00677559, 0x00677559, 0x00667458, 0x00667458, 0x00667458, 0x00667458, 0x0069765C, 0x0069765C, 0x0069765C, 0x0068755B, 0x00677559,
        0x00677559, 0x00667458, 0x00667458, 0x00697559, 0x00697559, 0x00687458, 0x00687458, 0x00677357, 0x00667256, 0x00677357, 0x00687458, 0x006A7B50, 0x006B7955, 0x006A7856, 0x006A7953, 0x0067784D, 0xFF8A9874, 0xFFF0F9E5, 0xFFFDFFFE, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xFFFFFFFE, 0xF4F6FAEF, 0xFFFDFFF9, 0xFFFDFFFF, 0xFFFBFFFF, 0xFFF9FFF7, 0xF4F8FFEF, 0xFFFDFFF6, 0xFFFEFFF7, 0xFFFEFFF7, 0xFFFFFFF8, 0xFFFFFFF8,
        0xFFFFFFF8, 0xFFFFFFF8, 0xFFFFFFF8, 0xFFFDFFF7, 0xFFFDFFF7, 0xFFFDFFF7, 0xFFFDFFF7, 0xFFFCFFF6, 0xFFFBFFF5, 0xFBFAFFF4, 0xFBFAFFF4, 0xFFFDFFF7, 0xFFFDFFF7, 0xFFFDFFF7, 0xFFFBFFF5, 0xFBF9FFF3, 0xF7F7FEF1, 0xF4F6FDF0, 0xF4F5FCEF, 0xFFFAFFF6,
        0xFFFAFFF6, 0xFFFAFFF6, 0xFFFAFFF6, 0xFFF9FEF5, 0xFBF8FDF4, 0xFBF8FDF4, 0xFBF7FCF3, 0xFFF9FEF5, 0xFFF9FEF5, 0xFFF9FEF5, 0xFFF9FEF5, 0xFFF9FEF5, 0xFFF9FEF5, 0xFFF9FEF5, 0xFFF9FEF5, 0xF7F5FAF1, 0xF7F5FAF1, 0xF7F5FAF1, 0xF7F6FBF2, 0xF7F6FBF2,
        0xF7F6FBF2, 0xF7F6FBF2, 0xF7F6FBF2, 0xF7F5FAF1, 0xF7F6FBF2, 0xF7F6FBF2, 0xF7F6FBF2, 0xFBF7FCF3, 0xFBF7FCF3, 0xFBF7FCF3, 0xFBF7FCF3, 0xFFF9FBF5, 0xFFF9FBF5, 0xFFF9FBF5, 0xFBF9FCF3, 0xFBFAFDF4, 0xFBFAFEF3, 0xFBFAFEF3, 0xFBFBFFF4, 0xF7F6FBF2,
        0xFBF7FCF3, 0xFFF9FEF5, 0xFFFAFFF6, 0xFBF8FDF4, 0xF7F5FAF1, 0xF0F1F6ED, 0xECEFF4EB, 0xE8EEF3EA, 0xE8EEF3EA, 0xE8EEF3EA, 0xE8EEF3EA, 0xE8EEF3EA, 0xE8EEF3EA, 0xE8EEF3EA, 0xE8EEF3EA, 0xE8EDF2E9, 0xE8EDF2E9, 0xE8EDF2E9, 0xE8EDF2E9, 0xE8EDF2E9,
        0xE8EDF2E9, 0xE8EEF3EA, 0xE8EEF3EA, 0xE4ECF2E7, 0xE4ECF2E7, 0xE4ECF2E7, 0xE4EDF3E8, 0xE4EDF3E8, 0xE4ECF2E7, 0xE4ECF2E7, 0xE4ECF2E7, 0xE4ECF2E7, 0xE4ECF2E7, 0xE1EBF1E6, 0xE1EBF1E6, 0xE1EBF1E6, 0xE1EBF1E6, 0xE1EBF1E6, 0xE1EBF1E6, 0xE1EBF1E6,
        0xE1EBF1E6, 0xE1EBF1E6, 0xE1EBF1E6, 0xE1EBF1E6, 0xE4ECF2E7, 0xE4ECF2E7, 0xE4ECF2E7, 0xE4ECF1E8, 0xE4ECF1E8, 0xE4ECF1E8, 0xE4ECF1E8, 0xE4ECF1E8, 0xE4ECF1E8, 0xE4EBF0E7, 0xE4EBF0E7, 0xE1EBF1E6, 0xE1EAF0E5, 0xD9E7EDE2, 0xD5E5EBE0, 0xD1E3E9DE,
        0xD1E2E8DD, 0xD1E3E9DE, 0xD1E3E9DE, 0xD1E5E9DE, 0xD1E5E9DE, 0xD1E5E9DE, 0xD1E4E8DD, 0xD1E4E8DD, 0xCEE3E7DC, 0xCEE3E7DC, 0xCEE3E7DC, 0xC6DEE5D8, 0xCAE0E7DA, 0xCEE1E8DB, 0xCEE1E8DB, 0xCAE0E7DA, 0xCADFE6D9, 0xCAE0E7DA, 0xCEE1E8DB, 0xD1E2E8DD,
        0xCEE1E7DC, 0xCEE1E7DC, 0xCEE1E7DC, 0xD1E2E8DD, 0xD1E2E8DD, 0xD1E3E9DE, 0xD1E3E9DE, 0xCEE5E7DB, 0xCEE5E7DB, 0xCEE5E7DB, 0xCEE6E8DC, 0xCAE6E9DA, 0xCAE6E9DA, 0xC6E5E8D8, 0xC6E4E7D7, 0xCAE0E4D9, 0xCEE2E6DB, 0xCEE3E7DC, 0xCEE3E7DC, 0xCAE1E5DA,
        0xC2DDE1D6, 0xBAD9DDD2, 0xB7D6DACF, 0xAFD4DBCC, 0xAFD4DBCC, 0xAFD4DBCC, 0xAFD4DBCC, 0xAFD4DBCC, 0xAFD3DACB, 0xAFD3DACB, 0xAFD3DACB, 0xAFD3DACB, 0xAFD3DACB, 0xAFD3DACB, 0xAFD3DACB, 0xAFD3DACB, 0xAFD3DACB, 0xAFD3DACB, 0xAFD3DACB, 0xABD2D9CA,
        0xABD2D9CA, 0xABD2D9CA, 0xABD2D9CA, 0xABD1D8C9, 0xA7D0D7C8, 0xA7D0D7C8, 0xA7CFD6C7, 0xB3D5D9CD, 0xB3D5D9CD, 0xB3D5D9CD, 0xB3D5D9CD, 0xB3D5D9CD, 0xB3D6DACE, 0xB3D6DACE, 0xB3D6DACE, 0xB3D5D9CD, 0xAFD4D8CC, 0xAFD4D8CC, 0xABD4D9CA, 0xABD3D8C9,
        0xA7D3D9C8, 0xABD4DAC9, 0xABD4DAC9, 0xB3D3DACD, 0xAFD1D8CB, 0xA7CED5C8, 0xA4CBD2C5, 0xA0C9D0C3, 0x9CC7CEC1, 0x98C6CDC0, 0x98C5CCBF, 0x91C5CDBC, 0x9CC8CFC2, 0xABCED4C9, 0x9CCCD5C1, 0x8DCAD6BA, 0xFFD2DCC5, 0xFFFBFFF7, 0xFFFAF9FF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF
};

static GUI_CONST_STORAGE GUI_BITMAP _bmPlatine_250x200 = {
  250,                                  // XSize
  200,                                  // YSize
  1000,                                 // BytesPerLine
  32,                                   // BitsPerPixel
  (unsigned char *)_acPlatine_250x200,  // Pointer to picture data
  NULL,                                 // Pointer to palette
  GUI_DRAW_BMP8888
};
#endif

/*********************************************************************
*
*       bmJLink_300x132
*/
#if (GUI_USE_ARGB)
static GUI_CONST_STORAGE unsigned long _acJLink_300x132[] = {
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFCFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFFFF, 0x00FDFFFE, 0x00FCFFFF, 0x00FCFFFF, 0x00F9FDFC, 0x00FEFFFD, 0x00F0F2EF, 0x04EAECE9, 0x00F7F9F6, 0x00FEFFFD,
        0x00FEFFFD, 0x00FBFDFA, 0x00FCFEFB, 0x00FBFBFB, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFCFC, 0x00FAFAFA, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFFFE, 0x00F9FBFA, 0x00FAFCFB, 0x00FCFFFF, 0x00FCFFFF, 0x00EFF5F3, 0x21DCE2E0, 0xC0919690, 0xEA7C817B, 0xFB747973, 0xD5878C86, 0x9BA2A7A1,
        0x69BBC0BA, 0x2ED7DCD6, 0x00F0F5EF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FCFEFD, 0x00FCFEFD, 0x00FEFFFF, 0x00FEFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F8FAF9, 0x00FEFFFF, 0x00FEFFFF, 0x00FCFFFF, 0x04E9EDEC, 0x58C1C7C5, 0xC98C9290, 0xFF626866, 0xFF5D625C, 0xFF5B605A, 0xFF5C615B, 0xFF5F645E, 0xFF60655F,
        0xFF616660, 0xFF6A6F69, 0xF7767B75, 0xC48D8F8E, 0x9BA2A4A3, 0x58C2C4C3, 0x19E0E2E1, 0x00F3F5F4, 0x00FCFEFD, 0x00FEFFFF, 0x00FEFFFF, 0x00F9F9F9, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFFFF, 0x00FDFFFE, 0x21DCDEDD, 0x9F9FA3A2, 0xFF686C6B, 0xFF525856, 0xFF595F5D, 0xFF656B69, 0xFF6A6F69, 0xFF6D726C, 0xFF6E736D, 0xFF6A6F69, 0xFF676C66,
        0xFF676C66, 0xFF676C66, 0xFF676C66, 0xFF666867, 0xFF686A69, 0xFF6B6D6C, 0xFF717372, 0xE67E807F, 0xB4969897, 0x79B2B4B3, 0x4FC6C8C7, 0x08E8E8E8, 0x00F1F1F1, 0x00FBFBFB, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFFFF, 0x00FDFDFD, 0x00FDFFFE, 0x00FEFFFF, 0x00FEFFFF,
        0x00FDFFFE, 0x00FEFFFF, 0x00FEFFFF, 0x00FBFDFC, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x1DDEE4E0, 0x92A5ABA7, 0xFF696F6B, 0xFF4F5551, 0xFF4C524E, 0xFF4B514D, 0xFF484E4A, 0xFF494F4B, 0xFF515753, 0xFF585E5A, 0xFF626864, 0xFF6A706C, 0xFF6D736F,
        0xFF6D736F, 0xFF6C726E, 0xFF6B716D, 0xFF6D746D, 0xFF6B726B, 0xFF686F68, 0xFF656C65, 0xFF636A63, 0xFF636A63, 0xFF646B64, 0xFF656C65, 0xFF6F7571, 0xE27F8581, 0xAB999F9B, 0x75B4BAB6, 0x3FCDD3CF, 0x11E3E9E5, 0x00F4FAF6, 0x00FBFFFD, 0x00FFFEFF,
        0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FEFCFD, 0x00FEFCFD, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FFFDFE, 0x00FEFCFD, 0x00FEFCFD, 0x00FFFDFE, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FEFEFE, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x00FCFAFB, 0x00FFFEFF, 0x00FEFEFE, 0x00FCFCFC, 0x00FBFDFC, 0x00F8FAF9, 0x00FEFFFF, 0x00FEFFFF,
        0x00FEFFFF, 0x00FCFEFD, 0x00FBFDFC, 0x00FEFFFF, 0x00FEFFFF, 0x2AD8DAD9, 0x96A3A5A4, 0xFF6A706C, 0xFF5F6561, 0xFF595F5B, 0xFF5D635F, 0xFF626864, 0xFF616763, 0xFF5E6460, 0xFF5D635F, 0xFF4D534F, 0xFF4A504C, 0xFF454B47, 0xFF424844, 0xFF444A46,
        0xFF4C524E, 0xFF585E5A, 0xFF606662, 0xFF656C65, 0xFF666D66, 0xFF686F68, 0xFF697069, 0xFF697069, 0xFF697069, 0xFF686F68, 0xFF686F68, 0xFF686E6A, 0xFF656B67, 0xFF626864, 0xFF626864, 0xFF686E6A, 0xF2777D79, 0xC98B918D, 0xB0989E9A, 0x5CBFC1C0,
        0x36D1D3D2, 0x04EAECEB, 0x00FDFFFE, 0x00FEFFFF, 0x00FEFFFF, 0x00FDFFFE, 0x00FBFDFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FEFEFE, 0x00FEFEFE, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FDFDFD, 0x00FCFCFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFAFB, 0x00FFFEFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FDFDFD, 0x00FEFFFF, 0x00FBFDFC, 0x00FEFFFF, 0x00FEFFFF,
        0x00F9FBFA, 0x00FEFFFF, 0x00FEFFFF, 0x0DE6E8E7, 0x82AEB0AF, 0xF7767877, 0xFF535554, 0xFF565C58, 0xFF5A605C, 0xFF5E6460, 0xFF5C625E, 0xFF585E5A, 0xFF575D59, 0xFF5A605C, 0xFF5D635F, 0xFF636965, 0xFF636965, 0xFF616763, 0xFF5C625E, 0xFF555B57,
        0xFF4C524E, 0xFF454B47, 0xFF424844, 0xFF464D46, 0xFF4A514A, 0xFF535A53, 0xFF5B625B, 0xFF626962, 0xFF676E67, 0xFF697069, 0xFF6A716A, 0xFF666C68, 0xFF676D69, 0xFF686E6A, 0xFF666C68, 0xFF636965, 0xFF616763, 0xFF626864, 0xFF646A66, 0xFF626866,
        0xFF6D7371, 0xE67E8482, 0xC08F9593, 0x96A4AAA8, 0x58C1C7C5, 0x19E0E6E4, 0x00F5FBF9, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FBFBFB, 0x00FBFBFB, 0x00FDFDFD, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FDFDFD, 0x00FEFEFE,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FDFDFD, 0x00FCFCFC, 0x00FCFCFC, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFE, 0x00FFFFFF, 0x00FCFCFC, 0x00FCFCFC, 0x00FEFFFF, 0x00FEFFFF, 0x00F4F8F7, 0x00F8FCFB, 0x00FBFFFE,
        0x00FAFEFD, 0x04E9EDEC, 0x71B6BAB9, 0xFB747877, 0xFF525655, 0xFF585C5B, 0xFF656968, 0xFF5F6561, 0xFF5F6561, 0xFF5E6460, 0xFF5B615D, 0xFF585E5A, 0xFF565C58, 0xFF585E5A, 0xFF5A605C, 0xFF515753, 0xFF565C58, 0xFF5E6460, 0xFF656B67, 0xFF686E6A,
        0xFF696F6B, 0xFF676D69, 0xFF656B67, 0xFF596059, 0xFF525952, 0xFF495049, 0xFF424942, 0xFF404740, 0xFF444B44, 0xFF4B524B, 0xFF505750, 0xFF606662, 0xFF636965, 0xFF666C68, 0xFF686E6A, 0xFF676D69, 0xFF666C68, 0xFF676D69, 0xFF676D69, 0xFF5E6963,
        0xFF606B65, 0xFF606B65, 0xFF5E6963, 0xFF5D6862, 0xFF626D67, 0xFF6D7872, 0xF776817B, 0xB0989A97, 0x82AEB0AD, 0x3FCED0CD, 0x04EBEDEA, 0x00FCFEFB, 0x00FEFFFD, 0x00FEFFFD, 0x00FEFFFD, 0x00FBFDFA, 0x00FCFEFB, 0x00FDFFFC, 0x00FDFFFC, 0x00FCFEFB,
        0x00FCFEFB, 0x00FCFEFB, 0x00FDFFFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFCFC, 0x00FAFAFA, 0x00FEFFFF, 0x00FAFCFB, 0x00F6FAF9, 0x00FCFFFF, 0x00FCFFFF, 0x00ECF0EF,
        0x71B5B9B8, 0xEE7A7E7D, 0xFF5A5E5D, 0xFF585C5B, 0xFF636766, 0xFF646867, 0xFF595D5C, 0xFF5D635F, 0xFF585E5A, 0xFF585E5A, 0xFF5E6460, 0xFF5F6561, 0xFF5A605C, 0xFF565C58, 0xFF575D59, 0xFF5B615D, 0xFF595F5B, 0xFF575D59, 0xFF575D59, 0xFF585E5A,
        0xFF5C625E, 0xFF5F6561, 0xFF626864, 0xFF676E67, 0xFF676E67, 0xFF666D66, 0xFF626962, 0xFF5C635C, 0xFF535A53, 0xFF4B524B, 0xFF464D46, 0xFF404642, 0xFF454B47, 0xFF4C524E, 0xFF545A56, 0xFF5B615D, 0xFF616763, 0xFF656B67, 0xFF686E6A, 0xFF65706A,
        0xFF65706A, 0xFF66716B, 0xFF65706A, 0xFF646F69, 0xFF626D67, 0xFF616C66, 0xFF606B65, 0xFF606561, 0xFF626763, 0xFF676C68, 0xFF717672, 0xD9838884, 0x9F9FA4A0, 0x64BCC1BD, 0x3BCFD4D0, 0x00F6F8F5, 0x00F9FBF8, 0x00FEFFFD, 0x00FEFFFD, 0x00FEFFFD,
        0x00FDFFFC, 0x00FCFEFB, 0x00FCFEFB, 0x00FEFEFE, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFFFE, 0x00FEFFFF, 0x00FEFFFF, 0x00F9FDFC, 0x00FCFFFF, 0x00F5FBF9, 0x79B2B8B6, 0xFF717574,
        0xFF585C5B, 0xFF525655, 0xFF616564, 0xFF676B6A, 0xFF606463, 0xFF5C605F, 0xFF5D6160, 0xFF5E6460, 0xFF585E5A, 0xFF565C58, 0xFF5B615D, 0xFF5C625E, 0xFF595F5B, 0xFF5C625E, 0xFF626864, 0xFF5A605C, 0xFF5A605C, 0xFF5A605C, 0xFF5B615D, 0xFF5B615D,
        0xFF5B615D, 0xFF5A605C, 0xFF585E5A, 0xFF585F58, 0xFF575E57, 0xFF575E57, 0xFF575E57, 0xFF585F58, 0xFF5A615A, 0xFF5C635C, 0xFF5D645D, 0xFF606662, 0xFF595F5B, 0xFF4E5450, 0xFF444A46, 0xFF3E4440, 0xFF3F4541, 0xFF424844, 0xFF454B47, 0xFF535D55,
        0xFF555F57, 0xFF5B655D, 0xFF616B63, 0xFF677169, 0xFF68726A, 0xFF667068, 0xFF646E66, 0xFF656C65, 0xFF656C65, 0xFF646B64, 0xFF626962, 0xFF5F665F, 0xFF5F665F, 0xFF626962, 0xFF656C65, 0xFF717670, 0xD9848983, 0x9BA3A8A2, 0x58C2C7C1, 0x26DBE0DA,
        0x00EDF2EC, 0x00F9FEF8, 0x00FCFFFB, 0x00FAFAFA, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFFFF, 0x00FCFEFD, 0x00FEFFFF, 0x00F7FBFA, 0x00FCFFFF, 0x00F3F9F7, 0xD9838987, 0xFF212725, 0xFF1B211F,
        0xFF2C3230, 0xFF3D4341, 0xFF414745, 0xFF444A48, 0xFF535957, 0xFF5E6462, 0xFF5B615F, 0xFF5A605C, 0xFF5C625E, 0xFF5F6561, 0xFF5F6561, 0xFF5C625E, 0xFF595F5B, 0xFF5A605C, 0xFF5C625E, 0xFF5C625E, 0xFF5C625E, 0xFF5B615D, 0xFF5C625E, 0xFF5C625E,
        0xFF5B615D, 0xFF595F5B, 0xFF565C58, 0xFF555C55, 0xFF575E57, 0xFF5A615A, 0xFF5D645D, 0xFF5E655E, 0xFF5D645D, 0xFF5C635C, 0xFF5B625B, 0xFF5D635F, 0xFF5E6460, 0xFF5F6561, 0xFF606662, 0xFF5D635F, 0xFF585E5A, 0xFF535955, 0xFF4F5551, 0xFF444B44,
        0xFF434A43, 0xFF424942, 0xFF454C45, 0xFF4B524B, 0xFF555C55, 0xFF5F665F, 0xFF666D66, 0xFF667068, 0xFF656F67, 0xFF626C64, 0xFF606A62, 0xFF5F6961, 0xFF5F6961, 0xFF616B63, 0xFF636D65, 0xFF626761, 0xFF5E635D, 0xFF595E58, 0xFF595E58, 0xFF636862,
        0xF2797E78, 0xBC939892, 0x92A6ABA5, 0x32D3D3D3, 0x19DFDFDF, 0x00F0F0F0, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFFFE, 0x00FDFFFE, 0x00FEFFFF, 0x00F7FBFA, 0x00FCFFFF, 0x96A4AAA8, 0xFF151B19, 0xFF0D1311, 0xFF131917,
        0xFF0F1513, 0xFF131917, 0xFF1B211F, 0xFF1D2321, 0xFF252B29, 0xFF333937, 0xFF3B413F, 0xFF464C48, 0xFF515753, 0xFF595F5B, 0xFF5B615D, 0xFF5C625E, 0xFF5F6561, 0xFF5F6561, 0xFF5B615D, 0xFF5E6460, 0xFF5C625E, 0xFF5B615D, 0xFF5B615D, 0xFF5B615D,
        0xFF5A605C, 0xFF585E5A, 0xFF565C58, 0xFF5B625B, 0xFF5C635C, 0xFF5F665F, 0xFF616861, 0xFF646B64, 0xFF656C65, 0xFF666D66, 0xFF666D66, 0xFF626864, 0xFF616763, 0xFF5F6561, 0xFF5E6460, 0xFF5D635F, 0xFF5D635F, 0xFF5C625E, 0xFF5C625E, 0xFF60655F,
        0xFF616660, 0xFF5F645E, 0xFF575C56, 0xFF4E534D, 0xFF464B45, 0xFF444943, 0xFF444943, 0xFF4A544C, 0xFF4E5850, 0xFF566058, 0xFF5E6860, 0xFF636D65, 0xFF646E66, 0xFF626C64, 0xFF606A62, 0xFF616660, 0xFF636862, 0xFF646963, 0xFF616660, 0xFF5E635D,
        0xFF5C615B, 0xFF5E635D, 0xFF60655F, 0xFF5E5E5E, 0xFF6C6C6C, 0xDE818181, 0xAB999999, 0x79B1B1B1, 0x43CBCBCB, 0x11E4E4E4, 0x00F5F5F5, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFD, 0x00FDFBFC, 0x00FCFDFF, 0x00FBFFFF, 0x00F3FBFD, 0xFF38413E, 0xFF0F160F, 0xFF0F140D, 0xFF0F1511,
        0xFF0E1410, 0xFF0E1410, 0xFF0E1410, 0xFF0E1410, 0xFF0E1410, 0xFF0E1410, 0xFF0F1511, 0xFF171D19, 0xFF202622, 0xFF2C322E, 0xFF363C38, 0xFF3E4440, 0xFF484E4A, 0xFF545A56, 0xFF5C625E, 0xFF5B605C, 0xFF5E635F, 0xFF606561, 0xFF606561, 0xFF5D625E,
        0xFF5C615D, 0xFF5F6460, 0xFF616662, 0xFF5F6561, 0xFF5F6561, 0xFF606662, 0xFF606662, 0xFF616763, 0xFF616763, 0xFF616763, 0xFF606662, 0xFF646A66, 0xFF646A66, 0xFF636965, 0xFF636965, 0xFF626864, 0xFF606662, 0xFF5E6460, 0xFF5D635F, 0xFF5C625E,
        0xFF5C625E, 0xFF5C625E, 0xFF5E6460, 0xFF606662, 0xFF616763, 0xFF606662, 0xFF5F6561, 0xFF505750, 0xFF4C534C, 0xFF464D46, 0xFF434A43, 0xFF454C45, 0xFF4C534C, 0xFF545B54, 0xFF596059, 0xFF616861, 0xFF616861, 0xFF626962, 0xFF616861, 0xFF616861,
        0xFF606760, 0xFF5F665F, 0xFF5E655E, 0xFF646B64, 0xFF646B64, 0xFF626962, 0xFF5F665F, 0xFF5F665F, 0xFF656C65, 0xFF6F766F, 0xF2777E77, 0xB4969C9A, 0x86ABB1AF, 0x47CAD0CE, 0x11E4EAE8, 0x00F3F9F7, 0x00FAFFFE, 0x00FBFFFF, 0x00FBFFFF, 0x00FBFDFC,
        0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FDFFFE, 0x00FEFFFF, 0x00FEFFFF, 0x00FCFEFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE,
        0x00FEFEFE, 0x00FDFDFD, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFBFB, 0x00FEFCFD, 0x00FEFFFF, 0x00FAFFFF, 0x21DBE3E5, 0xFF212A27, 0xFF0A110A, 0xFF10150E, 0xFF0E1410,
        0xFF0E1410, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0E1410, 0xFF0E1410, 0xFF080E0A, 0xFF0B110D, 0xFF0F1511, 0xFF121814, 0xFF151B17, 0xFF1C221E, 0xFF252B27, 0xFF2D332F, 0xFF3F4440, 0xFF474C48, 0xFF525753, 0xFF5A5F5B, 0xFF5E635F,
        0xFF616662, 0xFF636864, 0xFF656A66, 0xFF636965, 0xFF636965, 0xFF616763, 0xFF5F6561, 0xFF5D635F, 0xFF5C625E, 0xFF5C625E, 0xFF5D635F, 0xFF5B615D, 0xFF5D635F, 0xFF606662, 0xFF636965, 0xFF646A66, 0xFF646A66, 0xFF636965, 0xFF626864, 0xFF686E6A,
        0xFF666C68, 0xFF646A66, 0xFF626864, 0xFF626864, 0xFF606662, 0xFF5E6460, 0xFF5C625E, 0xFF656C65, 0xFF646B64, 0xFF626962, 0xFF5E655E, 0xFF585F58, 0xFF515851, 0xFF4B524B, 0xFF474E47, 0xFF414841, 0xFF464D46, 0xFF4D544D, 0xFF565D56, 0xFF5C635C,
        0xFF616861, 0xFF636A63, 0xFF636A63, 0xFF5E655E, 0xFF5F665F, 0xFF5F665F, 0xFF5D645D, 0xFF5C635C, 0xFF5D645D, 0xFF616861, 0xFF646B64, 0xFF616765, 0xFF626866, 0xFF666C6A, 0xFF6F7573, 0xDE818785, 0xA39DA3A1, 0x60BDC3C1, 0x36D2D8D6, 0x00F3F5F4,
        0x00F9FBFA, 0x00FDFFFE, 0x00FCFEFD, 0x00FCFEFD, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FDFDFD, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FEFEFE, 0x00FEFEFE, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFBFB, 0x00FFFEFF, 0x00FDFEFF, 0x00F9FEFF, 0x58C1C9CB, 0xFF0D1613, 0xFF091009, 0xFF0E130C, 0xFF0D130F,
        0xFF0C120E, 0xFF0C120E, 0xFF0B110D, 0xFF0B110D, 0xFF0C120E, 0xFF0C120E, 0xFF0D130F, 0xFF0D120E, 0xFF0D120E, 0xFF0C110D, 0xFF0A0F0B, 0xFF090E0A, 0xFF0B100C, 0xFF0F1410, 0xFF131814, 0xFF121713, 0xFF191E1A, 0xFF232824, 0xFF2F3430, 0xFF3B403C,
        0xFF474C48, 0xFF545955, 0xFF5C615D, 0xFF5D635F, 0xFF616763, 0xFF666C68, 0xFF676D69, 0xFF656B67, 0xFF626864, 0xFF606662, 0xFF606662, 0xFF636965, 0xFF626864, 0xFF616763, 0xFF5F6561, 0xFF5F6561, 0xFF616763, 0xFF636965, 0xFF646A66, 0xFF616763,
        0xFF606662, 0xFF606662, 0xFF616763, 0xFF636965, 0xFF656B67, 0xFF656B67, 0xFF656B67, 0xFF616861, 0xFF626962, 0xFF646B64, 0xFF656C65, 0xFF656C65, 0xFF646B64, 0xFF636A63, 0xFF616861, 0xFF606760, 0xFF596059, 0xFF4E554E, 0xFF454C45, 0xFF424942,
        0xFF444B44, 0xFF4A514A, 0xFF4E554E, 0xFF545B54, 0xFF585F58, 0xFF5E655E, 0xFF636A63, 0xFF646B64, 0xFF616861, 0xFF5E655E, 0xFF5B625B, 0xFF606664, 0xFF616765, 0xFF606664, 0xFF606664, 0xFF606664, 0xFF626866, 0xFF666C6A, 0xFF696F6D, 0xF2787A79,
        0xC98C8E8D, 0x8AA9ABAA, 0x4FC5C7C6, 0x1DDDDFDE, 0x00EFF1F0, 0x00FBFDFC, 0x00FEFFFF, 0x00FCFCFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FEFEFE, 0x00FDFDFD, 0x00FCFCFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFD, 0x00FFFEFF, 0x00FBFCFF, 0x00FBFFFF, 0x79B1B9BB, 0xFF08110E, 0xFF0E150E, 0xFF0B1009, 0xFF0B110D,
        0xFF0B110D, 0xFF0A100C, 0xFF0A100C, 0xFF0A100C, 0xFF0A100C, 0xFF0B110D, 0xFF0B110D, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0D120E, 0xFF0C110D, 0xFF0B100C, 0xFF101511, 0xFF0F1410, 0xFF0D120E, 0xFF0C110D, 0xFF0F1410,
        0xFF161B17, 0xFF1F2420, 0xFF262B27, 0xFF333935, 0xFF3C423E, 0xFF484E4A, 0xFF535955, 0xFF5A605C, 0xFF606662, 0xFF666C68, 0xFF6A706C, 0xFF636965, 0xFF626864, 0xFF616763, 0xFF5F6561, 0xFF5F6561, 0xFF606662, 0xFF636965, 0xFF666C68, 0xFF606662,
        0xFF606662, 0xFF606662, 0xFF616763, 0xFF636965, 0xFF656B67, 0xFF666C68, 0xFF666C68, 0xFF656C65, 0xFF646B64, 0xFF636A63, 0xFF626962, 0xFF636A63, 0xFF656C65, 0xFF676E67, 0xFF686F68, 0xFF626962, 0xFF646B64, 0xFF666D66, 0xFF656C65, 0xFF5F665F,
        0xFF575E57, 0xFF4E554E, 0xFF484F48, 0xFF474E47, 0xFF474E47, 0xFF474E47, 0xFF4B524B, 0xFF535A53, 0xFF5B625B, 0xFF626962, 0xFF666D66, 0xFF5D635F, 0xFF5E6460, 0xFF5F6561, 0xFF606662, 0xFF616763, 0xFF606662, 0xFF5F6561, 0xFF5E6460, 0xFF636864,
        0xFF626763, 0xFF606561, 0xFF606561, 0xFF6A6F6B, 0xE27F8480, 0xB0989D99, 0x8EA8ADA9, 0x32D4D6D3, 0x19E1E3E0, 0x00F2F4F1, 0x00FDFFFC, 0x00FEFFFD, 0x00FCFEFB, 0x00FDFFFC, 0x00FEFFFD, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF,
        0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FDFDFD, 0x00FCFCFC, 0x00FCFCFC, 0x00FDFDFD, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFD, 0x00FFFEFF, 0x00FBFCFF, 0x00FBFFFF, 0x96A3ABAD, 0xFF07100D, 0xFF111811, 0xFF080D06, 0xFF0A100C,
        0xFF0A100C, 0xFF090F0B, 0xFF090F0B, 0xFF090F0B, 0xFF090F0B, 0xFF0A100C, 0xFF0A100C, 0xFF090E0A, 0xFF090E0A, 0xFF0A0F0B, 0xFF0B100C, 0xFF0C110D, 0xFF0C110D, 0xFF0A0F0B, 0xFF090E0A, 0xFF0D120E, 0xFF0E130F, 0xFF0E130F, 0xFF0D120E, 0xFF0D120E,
        0xFF0D120E, 0xFF0E130F, 0xFF101511, 0xFF111713, 0xFF151B17, 0xFF1C221E, 0xFF232925, 0xFF2C322E, 0xFF373D39, 0xFF454B47, 0xFF4E5450, 0xFF5B615D, 0xFF5F6561, 0xFF656B67, 0xFF686E6A, 0xFF686E6A, 0xFF666C68, 0xFF646A66, 0xFF636965, 0xFF606662,
        0xFF606662, 0xFF606662, 0xFF5F6561, 0xFF5E6460, 0xFF5E6460, 0xFF5E6460, 0xFF5E6460, 0xFF5E655E, 0xFF5F665F, 0xFF616861, 0xFF626962, 0xFF636A63, 0xFF626962, 0xFF616861, 0xFF616861, 0xFF656C65, 0xFF656C65, 0xFF656C65, 0xFF656C65, 0xFF656C65,
        0xFF676E67, 0xFF686F68, 0xFF686F68, 0xFF676E67, 0xFF606760, 0xFF565D56, 0xFF4F564F, 0xFF4A514A, 0xFF474E47, 0xFF454C45, 0xFF434A43, 0xFF585E5A, 0xFF5A605C, 0xFF5D635F, 0xFF616763, 0xFF646A66, 0xFF646A66, 0xFF616763, 0xFF5F6561, 0xFF626763,
        0xFF656A66, 0xFF686D69, 0xFF686D69, 0xFF656A66, 0xFF616662, 0xFF606561, 0xFF626763, 0xFF616360, 0xFF70726F, 0xD1888A87, 0x9FA0A29F, 0x6DB8BAB7, 0x36D2D4D1, 0x00ECEEEB, 0x00FDFFFC, 0x00FDFFFE, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FCFEFD,
        0x00FDFFFE, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFF9F9, 0x00FEFCFD, 0x00FEFFFF, 0x00FBFFFF, 0xBC91999B, 0xFF010A07, 0xFF0E150E, 0xFF0A0F08, 0xFF0A100C,
        0xFF090F0B, 0xFF090F0B, 0xFF080E0A, 0xFF080E0A, 0xFF090F0B, 0xFF090F0B, 0xFF0A100C, 0xFF0D0F0C, 0xFF0C0E0B, 0xFF0B0D0A, 0xFF0A0C09, 0xFF0B0D0A, 0xFF0D0F0C, 0xFF0E100D, 0xFF0F110E, 0xFF0B100C, 0xFF0D120E, 0xFF0F1410, 0xFF111612, 0xFF111612,
        0xFF101511, 0xFF0E130F, 0xFF0D120E, 0xFF0F1511, 0xFF0F1511, 0xFF0E1410, 0xFF0D130F, 0xFF0E1410, 0xFF111713, 0xFF171D19, 0xFF1C221E, 0xFF292F2B, 0xFF323834, 0xFF3E4440, 0xFF4A504C, 0xFF535955, 0xFF5C625E, 0xFF636965, 0xFF686E6A, 0xFF626864,
        0xFF636965, 0xFF646A66, 0xFF646A66, 0xFF626864, 0xFF616763, 0xFF606662, 0xFF606662, 0xFF606760, 0xFF616861, 0xFF626962, 0xFF626962, 0xFF616861, 0xFF5F665F, 0xFF5D645D, 0xFF5C635C, 0xFF626962, 0xFF626962, 0xFF626962, 0xFF626962, 0xFF616861,
        0xFF606760, 0xFF606760, 0xFF5F665F, 0xFF606760, 0xFF626962, 0xFF636A63, 0xFF656C65, 0xFF646B64, 0xFF5F665F, 0xFF575E57, 0xFF515851, 0xFF454C45, 0xFF424942, 0xFF3F463F, 0xFF404740, 0xFF454C45, 0xFF4F564F, 0xFF596059, 0xFF5F665F, 0xFF616660,
        0xFF5F645E, 0xFF5F645E, 0xFF60655F, 0xFF60655F, 0xFF616660, 0xFF646963, 0xFF696E68, 0xFF686D67, 0xFF656A64, 0xFF616660, 0xFF5C615B, 0xFF5D625C, 0xFF686D67, 0xF2787D77, 0xD9858A84, 0x8AAAAEAD, 0x64BBBFBE, 0x2ED6DAD9, 0x00EDF1F0, 0x00FAFEFD,
        0x00FCFFFF, 0x00FCFFFF, 0x00FBFFFE, 0x00FEFEFE, 0x00FEFEFE, 0x00FEFEFE, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFF9F9, 0x00FEFCFD, 0x00FEFFFF, 0x00FBFFFF, 0xD9838B8D, 0xFF000704, 0xFF091009, 0xFF0D120B, 0xFF090F0B,
        0xFF090F0B, 0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF090F0B, 0xFF090F0B, 0xFF0C0E0B, 0xFF0B0D0A, 0xFF0B0D0A, 0xFF0A0C09, 0xFF090B08, 0xFF0A0C09, 0xFF0B0D0A, 0xFF0D0F0C, 0xFF080D09, 0xFF080D09, 0xFF090E0A, 0xFF0A0F0B, 0xFF0B100C,
        0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0B110D, 0xFF0C120E, 0xFF0E1410, 0xFF0F1511, 0xFF0F1511, 0xFF0E1410, 0xFF0E1410, 0xFF0E1410, 0xFF0A100C, 0xFF0E1410, 0xFF141A16, 0xFF191F1B, 0xFF212723, 0xFF2B312D, 0xFF363C38, 0xFF3F4541, 0xFF545A56,
        0xFF575D59, 0xFF5C625E, 0xFF5F6561, 0xFF606662, 0xFF616763, 0xFF626864, 0xFF636965, 0xFF626962, 0xFF616861, 0xFF606760, 0xFF5F665F, 0xFF5F665F, 0xFF606760, 0xFF626962, 0xFF636A63, 0xFF606760, 0xFF606760, 0xFF5F665F, 0xFF5F665F, 0xFF606760,
        0xFF616861, 0xFF626962, 0xFF636A63, 0xFF646B64, 0xFF646B64, 0xFF636A63, 0xFF646B64, 0xFF646B64, 0xFF646B64, 0xFF626962, 0xFF616861, 0xFF646B64, 0xFF606760, 0xFF596059, 0xFF505750, 0xFF474E47, 0xFF414841, 0xFF3D443D, 0xFF3B423B, 0xFF424942,
        0xFF474E47, 0xFF535A53, 0xFF616861, 0xFF686F68, 0xFF666D66, 0xFF616861, 0xFF5F665F, 0xFF616660, 0xFF636862, 0xFF646963, 0xFF646963, 0xFF646963, 0xFF646963, 0xFF646963, 0xFF656A64, 0xFF5B5F5E, 0xFF606463, 0xFF6B6F6E, 0xEE7A7E7D, 0xBC919594,
        0x82ADB1B0, 0x47CACECD, 0x21DCE0DF, 0x00FCFCFC, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFD, 0x00FFFDFE, 0x00FEFFFF, 0x00F9FEFF, 0xE67D8587, 0xFF000906, 0xFF080F08, 0xFF0E130C, 0xFF090F0B,
        0xFF090F0B, 0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF090F0B, 0xFF090F0B, 0xFF090B08, 0xFF0B0D0A, 0xFF0E100D, 0xFF0E100D, 0xFF0D0F0C, 0xFF0C0E0B, 0xFF0B0D0A, 0xFF0A0C09, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0B100C, 0xFF0B100C,
        0xFF0B100C, 0xFF0A0F0B, 0xFF090E0A, 0xFF090F0B, 0xFF090F0B, 0xFF090F0B, 0xFF0A100C, 0xFF0C120E, 0xFF0D130F, 0xFF0E1410, 0xFF0F1511, 0xFF0F1511, 0xFF111713, 0xFF111713, 0xFF0F1511, 0xFF0D130F, 0xFF0E1410, 0xFF141A16, 0xFF181E1A, 0xFF1D231F,
        0xFF252B27, 0xFF313733, 0xFF3E4440, 0xFF494F4B, 0xFF525854, 0xFF5A605C, 0xFF5E6460, 0xFF646B64, 0xFF646B64, 0xFF626962, 0xFF616861, 0xFF616861, 0xFF606760, 0xFF606760, 0xFF606760, 0xFF5E655E, 0xFF5E655E, 0xFF5E655E, 0xFF5E655E, 0xFF5E655E,
        0xFF5D645D, 0xFF5D645D, 0xFF5D645D, 0xFF616861, 0xFF616861, 0xFF616861, 0xFF626962, 0xFF626962, 0xFF636A63, 0xFF646B64, 0xFF646B64, 0xFF606760, 0xFF616861, 0xFF636A63, 0xFF626962, 0xFF5F665F, 0xFF5C635C, 0xFF5B625B, 0xFF5A615A, 0xFF505750,
        0xFF484F48, 0xFF424942, 0xFF434A43, 0xFF464D46, 0xFF4A514A, 0xFF4F564F, 0xFF545B54, 0xFF5E635D, 0xFF60655F, 0xFF636862, 0xFF666B65, 0xFF676C66, 0xFF656A64, 0xFF626761, 0xFF5F645E, 0xFF636766, 0xFF646867, 0xFF636766, 0xFF606463, 0xFF5F6362,
        0xFF626665, 0xFF686C6B, 0xFF6E7271, 0xCD8A8A8A, 0xA79B9B9B, 0x6DB8B8B8, 0x2AD7D7D7, 0x00EEEEEE, 0x00FAFAFA, 0x00FEFEFE, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FEFEFE, 0x00FEFEFE, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFD,
        0x00FFFFFD, 0x00FEFEFC, 0x00FEFFFA, 0x00FEFFFA, 0x00FFFFFB, 0x00FFFFFA, 0x00FFFFFA, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FEFEFE, 0x00FEFEFE,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFE, 0x00FFFFFF, 0x00F9FDFC, 0x00F9FFFF, 0xFF636C69, 0xFF030907, 0xFF0D1110, 0xFF0A0C0B, 0xFF080E0A,
        0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C,
        0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0D120E, 0xFF0D120E, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0F1410, 0xFF0F1410, 0xFF0F1410,
        0xFF111612, 0xFF131814, 0xFF161B17, 0xFF1B201C, 0xFF232824, 0xFF2D322E, 0xFF333834, 0xFF434945, 0xFF4A504C, 0xFF535955, 0xFF5B615D, 0xFF616763, 0xFF636965, 0xFF646A66, 0xFF656B67, 0xFF616763, 0xFF606662, 0xFF5F6561, 0xFF606662, 0xFF626864,
        0xFF626864, 0xFF616763, 0xFF5F6561, 0xFF616861, 0xFF626962, 0xFF626962, 0xFF616861, 0xFF616861, 0xFF616861, 0xFF646B64, 0xFF666D66, 0xFF656B67, 0xFF656B67, 0xFF666C68, 0xFF676D69, 0xFF676D69, 0xFF676D69, 0xFF666C68, 0xFF656B67, 0xFF676D69,
        0xFF656B67, 0xFF626864, 0xFF626864, 0xFF606662, 0xFF595F5B, 0xFF4C524E, 0xFF424844, 0xFF414743, 0xFF444A46, 0xFF4B514D, 0xFF525854, 0xFF585E5A, 0xFF5C625E, 0xFF5E6460, 0xFF5F6561, 0xFF5F6561, 0xFF616763, 0xFF636965, 0xFF646A66, 0xFF656B67,
        0xFF656B67, 0xFF646A66, 0xFF636965, 0xFF606A62, 0xFF606A62, 0xFF636A63, 0xFF666D66, 0xFF737872, 0xCD8B908A, 0x8EA8ADA7, 0x64BEC0BB, 0x19E0E2E1, 0x00EEF0EF, 0x00FCFEFD, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FDFFFE, 0x00FFFDFE,
        0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFDFE, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FEFEFE, 0x00FEFEFE, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE,
        0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FDFDFD, 0x00FCFCFC, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FDFDFD, 0x00FBFBFB, 0x00FBFBFB, 0x00FCFCFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFCFC, 0x00FCFCFC, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FCFCFC, 0x00FFFFFF,
        0x00FDFDFD, 0x00FBFBFB, 0x00FDFDFD, 0x00FFFFFD, 0x00FFFFFD, 0x00FFFFFB, 0x00FDFEF9, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFCFD, 0x00FFFFFF, 0x00FAFEFD, 0x00F9FFFF, 0xFF5E6764, 0xFF020806, 0xFF0C100F, 0xFF0B0D0C, 0xFF080E0A,
        0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C,
        0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0F1410,
        0xFF101511, 0xFF101511, 0xFF111612, 0xFF111612, 0xFF131814, 0xFF171C18, 0xFF1A1F1B, 0xFF171D19, 0xFF1C221E, 0xFF252B27, 0xFF303632, 0xFF3C423E, 0xFF474D49, 0xFF515753, 0xFF575D59, 0xFF5E6460, 0xFF5F6561, 0xFF616763, 0xFF636965, 0xFF636965,
        0xFF616763, 0xFF5E6460, 0xFF5B615D, 0xFF616861, 0xFF616861, 0xFF606760, 0xFF5E655E, 0xFF5E655E, 0xFF5F665F, 0xFF626962, 0xFF656C65, 0xFF5F6561, 0xFF5F6561, 0xFF606662, 0xFF626864, 0xFF656B67, 0xFF686E6A, 0xFF6B716D, 0xFF6D736F, 0xFF696F6B,
        0xFF686E6A, 0xFF686E6A, 0xFF686E6A, 0xFF696F6B, 0xFF686E6A, 0xFF666C68, 0xFF656B67, 0xFF616763, 0xFF5C625E, 0xFF545A56, 0xFF4C524E, 0xFF484E4A, 0xFF474D49, 0xFF484E4A, 0xFF4A504C, 0xFF555B57, 0xFF595F5B, 0xFF5E6460, 0xFF636965, 0xFF666C68,
        0xFF676D69, 0xFF666C68, 0xFF656B67, 0xFF616B63, 0xFF636D65, 0xFF666D66, 0xFF646B64, 0xFF616660, 0xFF5F645E, 0xFF616660, 0xFF666863, 0xFF6B6D6C, 0xE27F8180, 0xAB9A9C9B, 0x75B3B5B4, 0x47CACCCB, 0x11E3E5E4, 0x00F9FBFA, 0x00FEFFFF, 0x00FEFEFE,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FDFDFD, 0x00FCFCFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FDFDFD, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FEFDFF,
        0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFD, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FCFCFC, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFCFD, 0x00FFFFFF, 0x00FBFFFE, 0x00F6FFFC, 0xFF555E5B, 0xFF020806, 0xFF0C100F, 0xFF0C0E0D, 0xFF080E0A,
        0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C,
        0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0C110D,
        0xFF0E130F, 0xFF0F1410, 0xFF0F1410, 0xFF0F1410, 0xFF0E130F, 0xFF0F1410, 0xFF101511, 0xFF121814, 0xFF131915, 0xFF151B17, 0xFF171D19, 0xFF1A201C, 0xFF1F2521, 0xFF252B27, 0xFF292F2B, 0xFF3C423E, 0xFF414743, 0xFF4A504C, 0xFF535955, 0xFF5A605C,
        0xFF5F6561, 0xFF636965, 0xFF656B67, 0xFF616861, 0xFF616861, 0xFF616861, 0xFF606760, 0xFF5E655E, 0xFF5D645D, 0xFF5E655E, 0xFF5F665F, 0xFF636965, 0xFF636965, 0xFF656B67, 0xFF656B67, 0xFF656B67, 0xFF646A66, 0xFF646A66, 0xFF636965, 0xFF666C68,
        0xFF676D69, 0xFF686E6A, 0xFF676D69, 0xFF656B67, 0xFF666C68, 0xFF696F6B, 0xFF6C726E, 0xFF676D69, 0xFF686E6A, 0xFF696F6B, 0xFF686E6A, 0xFF666C68, 0xFF616763, 0xFF5C625E, 0xFF595F5B, 0xFF4C524E, 0xFF4C524E, 0xFF4C524E, 0xFF4E5450, 0xFF535955,
        0xFF5A605C, 0xFF616763, 0xFF656B67, 0xFF636D65, 0xFF656F67, 0xFF697069, 0xFF6A716A, 0xFF6A6F69, 0xFF6A6F69, 0xFF6B706A, 0xFF6E706B, 0xFF656766, 0xFF676968, 0xFF666867, 0xFF646665, 0xFF686A69, 0xF2777978, 0xC98B8D8C, 0xB0979998, 0x54C3C3C3,
        0x36D2D2D2, 0x04E9E9E9, 0x00FAFAFA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FAFAFA, 0x00FCFCFC, 0x00FEFEFE, 0x00FEFEFE, 0x00FCFCFC, 0x00FCFCFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF,
        0x00FFFEFF, 0x00FEFDFF, 0x00FEFDFF, 0x00FDFDFF, 0x00FEFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FEFEFE, 0x00FEFEFE, 0x00FEFEFE, 0x00FDFDFD, 0x00FDFDFD, 0x00FDFDFD, 0x00FEFEFE, 0x00FFFFFF, 0x00FEFEFE, 0x00FDFDFD, 0x00FEFEFE, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFCFD, 0x00FFFFFF, 0x00FCFFFF, 0x00F0F9F6, 0xFF49524F, 0xFF010705, 0xFF0B0F0E, 0xFF0D0F0E, 0xFF080E0A,
        0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C,
        0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF090E0A, 0xFF090E0A, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0B100C, 0xFF0B100C, 0xFF0A0F0B,
        0xFF0C110D, 0xFF0E130F, 0xFF0F1410, 0xFF101511, 0xFF101511, 0xFF101511, 0xFF111612, 0xFF0D130F, 0xFF0F1511, 0xFF111713, 0xFF131915, 0xFF131915, 0xFF151B17, 0xFF171D19, 0xFF191F1B, 0xFF141A16, 0xFF191F1B, 0xFF212723, 0xFF2A302C, 0xFF333935,
        0xFF3D433F, 0xFF464C48, 0xFF4C524E, 0xFF5C625E, 0xFF5F6561, 0xFF636965, 0xFF666C68, 0xFF666C68, 0xFF666C68, 0xFF656B67, 0xFF656B67, 0xFF5E6460, 0xFF606662, 0xFF636965, 0xFF656B67, 0xFF666C68, 0xFF666C68, 0xFF666C68, 0xFF666C68, 0xFF646A66,
        0xFF656B67, 0xFF666C68, 0xFF656B67, 0xFF646A66, 0xFF646A66, 0xFF666C68, 0xFF676D69, 0xFF636965, 0xFF646A66, 0xFF656B67, 0xFF666C68, 0xFF676D69, 0xFF676D69, 0xFF666C68, 0xFF666C68, 0xFF6B716D, 0xFF686E6A, 0xFF636965, 0xFF5D635F, 0xFF565C58,
        0xFF4F5551, 0xFF4A504C, 0xFF474D49, 0xFF4F5951, 0xFF535D55, 0xFF5C635C, 0xFF636A63, 0xFF696E68, 0xFF6C716B, 0xFF6C716B, 0xFF6E706B, 0xFF6A6F6B, 0xFF6D726E, 0xFF6F7470, 0xFF6D726E, 0xFF6A6F6B, 0xFF696E6A, 0xFF6B706C, 0xFF6D726E, 0xFF6A6C6B,
        0xFF727473, 0xE27F8180, 0xBC919392, 0x8EA7A9A8, 0x58C2C4C3, 0x1DDDDFDE, 0x00EEF0EF, 0x00FDFFFE, 0x00FDFFFE, 0x00FDFFFE, 0x00FDFFFE, 0x00FCFEFD, 0x00FCFEFD, 0x00FDFFFE, 0x00FEFFFF, 0x00FCFEFD, 0x00FCFEFD, 0x00FCFEFD, 0x00FDFFFE, 0x00FEFFFF,
        0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FEFEFE, 0x00FFFFFF, 0x00FCFCFC, 0x00FCFCFC, 0x00FDFDFD, 0x00FDFDFD, 0x00FEFEFE,
        0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FCFCFC, 0x00FDFDFD, 0x00FEFEFE, 0x00FFFEFF,
        0x00FDFCFF, 0x00FAF9FE, 0x00FDFDFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFCFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFE, 0x00FFFFFF, 0x00FCFFFF, 0x04E9F2EF, 0xFF3E4744, 0xFF020806, 0xFF0A0E0D, 0xFF0D0F0E, 0xFF080E0A,
        0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C,
        0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C,
        0xFF0B100C, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0A100C, 0xFF0D130F, 0xFF0F1511, 0xFF111713, 0xFF111713, 0xFF111713, 0xFF111713, 0xFF121814, 0xFF131915, 0xFF151B17, 0xFF161C18, 0xFF161C18, 0xFF171D19,
        0xFF181E1A, 0xFF1C221E, 0xFF1F2521, 0xFF2E3430, 0xFF343A36, 0xFF3E4440, 0xFF494F4B, 0xFF535955, 0xFF5C625E, 0xFF646A66, 0xFF696F6B, 0xFF666C68, 0xFF676D69, 0xFF686E6A, 0xFF676D69, 0xFF656B67, 0xFF656B67, 0xFF666C68, 0xFF686E6A, 0xFF636965,
        0xFF626864, 0xFF616763, 0xFF636965, 0xFF656B67, 0xFF676D69, 0xFF676D69, 0xFF666C68, 0xFF6C726E, 0xFF6A706C, 0xFF676D69, 0xFF656B67, 0xFF656B67, 0xFF676D69, 0xFF6A706C, 0xFF6C726E, 0xFF656B67, 0xFF656B67, 0xFF676D69, 0xFF676D69, 0xFF676D69,
        0xFF666C68, 0xFF646A66, 0xFF636965, 0xFF555F57, 0xFF505A52, 0xFF4C534C, 0xFF495049, 0xFF4C514B, 0xFF525751, 0xFF595E58, 0xFF5F615C, 0xFF6D726E, 0xFF6E736F, 0xFF6F7470, 0xFF707571, 0xFF6E736F, 0xFF6D726E, 0xFF6F7470, 0xFF717672, 0xFF747877,
        0xFF727675, 0xFF6E7271, 0xFF6A6E6D, 0xFF696D6C, 0xFF6F7372, 0xF2787C7B, 0xE2808483, 0xBC929493, 0x8AAAACAB, 0x43CCCECD, 0x08E8EAE9, 0x00F7F9F8, 0x00FDFFFE, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF,
        0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFEFE, 0x00FDFDFD, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FCFCFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFB, 0x00FBFBF9, 0x00FDFEF9, 0x00FFFFFB, 0x00FFFFFF, 0x00FEFEFE, 0x00FDFDFD, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x00FFFFFF, 0x00FCFFFF, 0x15E2EBE8, 0xFF333C39, 0xFF030907, 0xFF0A0E0D, 0xFF0D0F0E, 0xFF070D09,
        0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B,
        0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0C110D, 0xFF0C110D, 0xFF0B100C,
        0xFF0A0F0B, 0xFF090E0A, 0xFF090E0A, 0xFF0A0F0B, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF080E0A, 0xFF090F0B, 0xFF0A100C, 0xFF0B110D, 0xFF0B110D, 0xFF0E1410, 0xFF111713, 0xFF141A16, 0xFF161C18, 0xFF171D19, 0xFF191F1B, 0xFF191F1B, 0xFF181E1A,
        0xFF161C18, 0xFF141A16, 0xFF141A16, 0xFF0E1412, 0xFF111715, 0xFF161C1A, 0xFF1D2321, 0xFF262C2A, 0xFF313735, 0xFF3C4240, 0xFF434947, 0xFF535955, 0xFF595F5B, 0xFF616763, 0xFF676D69, 0xFF696F6B, 0xFF696F6B, 0xFF6B716D, 0xFF6C726E, 0xFF6E7470,
        0xFF6A706C, 0xFF666C68, 0xFF646A66, 0xFF656B67, 0xFF656B67, 0xFF636965, 0xFF626864, 0xFF5F6561, 0xFF616763, 0xFF646A66, 0xFF666C68, 0xFF676D69, 0xFF666C68, 0xFF656B67, 0xFF646A66, 0xFF6B716D, 0xFF6A706C, 0xFF686E6A, 0xFF676D69, 0xFF676D69,
        0xFF676D69, 0xFF686E6A, 0xFF696F6B, 0xFF636D65, 0xFF646E66, 0xFF666D66, 0xFF656C65, 0xFF616660, 0xFF5A5F59, 0xFF535852, 0xFF50524D, 0xFF4E534D, 0xFF515650, 0xFF595E58, 0xFF626761, 0xFF696E68, 0xFF6B706A, 0xFF6E736D, 0xFF717670, 0xFF6E7472,
        0xFF6E7472, 0xFF6D7371, 0xFF6C7270, 0xFF6A706E, 0xFF6A706E, 0xFF6B716F, 0xFF6C7270, 0xFF676B6A, 0xFF696D6C, 0xFF6B6F6E, 0xFF6F7372, 0xEA7C807F, 0xB4969A99, 0x6DB7BBBA, 0x3BCFD3D2, 0x04E9EDEC, 0x00EFF3F2, 0x00F9FDFC, 0x00FCFFFF, 0x00FCFFFF,
        0x00FCFFFF, 0x00FBFFFE, 0x00F9FDFC, 0x00FFFFFF, 0x00FEFEFE, 0x00FCFCFC, 0x00FCFCFC, 0x00FDFDFD, 0x00FEFEFE, 0x00FEFEFE, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F8F8F8, 0x00F9F9F9, 0x00F8F8F8, 0x00F5F5F5, 0x00F3F3F3, 0x00FAFAFA, 0x00FBFBFB, 0x00FCFCFC, 0x00FDFDFD, 0x00FEFEFE,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FBFBFB, 0x00FCFCFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFEF9,
        0x00FFFFFB, 0x00FBFCF7, 0x08EDEEE8, 0x2ADCDDD7, 0x32D8D9D3, 0x1DE4E5DD, 0x00F2F3EB, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FCFCFC, 0x00FEFEFE, 0x00FFFFFF, 0x00F9F9F9, 0x00FCFCFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x00FEFEFE, 0x00FCFFFF, 0x1DDDE6E3, 0xFF2C3532, 0xFF040A08, 0xFF090D0C, 0xFF0C0E0D, 0xFF070D09,
        0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B,
        0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0A0F0B,
        0xFF090E0A, 0xFF090E0A, 0xFF0A0F0B, 0xFF0C110D, 0xFF0E130F, 0xFF0E130F, 0xFF0D120E, 0xFF0B110D, 0xFF0B110D, 0xFF0B110D, 0xFF090F0B, 0xFF080E0A, 0xFF090F0B, 0xFF0C120E, 0xFF0E1410, 0xFF0A100C, 0xFF0B110D, 0xFF0E1410, 0xFF121814, 0xFF161C18,
        0xFF181E1A, 0xFF191F1B, 0xFF191F1B, 0xFF1C2220, 0xFF1B211F, 0xFF181E1C, 0xFF151B19, 0xFF121816, 0xFF131917, 0xFF151B19, 0xFF171D1B, 0xFF1E2420, 0xFF272D29, 0xFF363C38, 0xFF434945, 0xFF4F5551, 0xFF595F5B, 0xFF626864, 0xFF686E6A, 0xFF6A706C,
        0xFF6B716D, 0xFF6C726E, 0xFF6A706C, 0xFF686E6A, 0xFF666C68, 0xFF656B67, 0xFF646A66, 0xFF656B67, 0xFF656B67, 0xFF656B67, 0xFF656B67, 0xFF656B67, 0xFF656B67, 0xFF656B67, 0xFF656B67, 0xFF656B67, 0xFF676D69, 0xFF6B716D, 0xFF6D736F, 0xFF6D736F,
        0xFF6B716D, 0xFF676D69, 0xFF656B67, 0xFF616B63, 0xFF626C64, 0xFF666D66, 0xFF666D66, 0xFF676C66, 0xFF676C66, 0xFF686D67, 0xFF6A6C67, 0xFF646B64, 0xFF5C635C, 0xFF545B54, 0xFF505750, 0xFF4E554E, 0xFF4E554E, 0xFF525952, 0xFF575E57, 0xFF636C69,
        0xFF666F6C, 0xFF6C7572, 0xFF717A77, 0xFF747D7A, 0xFF737C79, 0xFF6F7875, 0xFF6C7572, 0xFF6B6F6E, 0xFF6E7271, 0xFF717574, 0xFF707473, 0xFF6E7271, 0xFF6D7170, 0xFF6D7170, 0xFF6F7372, 0xEA7B7F7E, 0xCD8A8E8D, 0x96A3A7A6, 0x64BCC0BF, 0x32D3D7D6,
        0x08E8ECEB, 0x00F9FDFC, 0x00FCFFFF, 0x00FCFCFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00EDEDED, 0x04E9E9E9, 0x11E4E4E4, 0x21DCDCDC, 0x2ED6D6D6, 0x2ED5D5D5, 0x2AD7D7D7, 0x21DBDBDB, 0x15E1E1E1, 0x08E7E7E7,
        0x00EDEDED, 0x00F1F1F1, 0x00F4F4F4, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F4F4F4, 0x04EAEAEA, 0x04EAEAEA, 0x00F2F2F2, 0x00FBFBFB, 0x00FEFEFE, 0x00FDFDFD, 0x00FFFFFA,
        0x00F2F3ED, 0x32DADBD3, 0x60C4C5BD, 0xFFB8B9B1, 0xFFBABCB1, 0x64C5C7BC, 0x4FCFD1C6, 0x36D2D2D2, 0x2AD8D8D8, 0x21DBDBDB, 0x2AD7D7D7, 0x3BCFCFCF, 0x47CACACA, 0x3FCECECE, 0x32D4D4D4, 0x00ECECEC, 0x00F2F2F2, 0x00FAFAFA, 0x00FEFEFE, 0x00FEFEFE,
        0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x00FEFEFE, 0x00FCFFFF, 0x26DAE3E0, 0xFF28312E, 0xFF050B09, 0xFF090D0C, 0xFF0B0D0C, 0xFF070D09,
        0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B,
        0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B,
        0xFF090E0A, 0xFF080D09, 0xFF090E0A, 0xFF0B100C, 0xFF0B100C, 0xFF090E0A, 0xFF070C08, 0xFF060C08, 0xFF070D09, 0xFF090F0B, 0xFF090F0B, 0xFF080E0A, 0xFF080E0A, 0xFF090F0B, 0xFF0A100C, 0xFF101612, 0xFF0E1410, 0xFF0C120E, 0xFF0D130F, 0xFF101612,
        0xFF141A16, 0xFF151B17, 0xFF161C18, 0xFF151B19, 0xFF171D1B, 0xFF181E1C, 0xFF181E1C, 0xFF171D1B, 0xFF151B19, 0xFF151B19, 0xFF161C1A, 0xFF141A16, 0xFF141A16, 0xFF131915, 0xFF131915, 0xFF171D19, 0xFF212723, 0xFF2F3531, 0xFF393F3B, 0xFF464C48,
        0xFF505652, 0xFF5C625E, 0xFF636965, 0xFF656B67, 0xFF656B67, 0xFF676D69, 0xFF6A706C, 0xFF656B67, 0xFF656B67, 0xFF656B67, 0xFF656B67, 0xFF646A66, 0xFF656B67, 0xFF656B67, 0xFF656B67, 0xFF646A66, 0xFF646A66, 0xFF646A66, 0xFF646A66, 0xFF656B67,
        0xFF676D69, 0xFF686E6A, 0xFF696F6B, 0xFF667068, 0xFF677169, 0xFF697069, 0xFF686F68, 0xFF676C66, 0xFF666B65, 0xFF656A64, 0xFF686A65, 0xFF666D66, 0xFF646B64, 0xFF656C65, 0xFF697069, 0xFF686F68, 0xFF616861, 0xFF5B625B, 0xFF5A615A, 0xFF4C5552,
        0xFF4B5451, 0xFF4C5552, 0xFF515A57, 0xFF5A6360, 0xFF646D6A, 0xFF6B7471, 0xFF6E7774, 0xFF767A79, 0xFF757978, 0xFF737776, 0xFF727675, 0xFF737776, 0xFF727675, 0xFF707473, 0xFF6E7271, 0xFF6A6E6D, 0xFF6A6E6D, 0xFF6A6E6D, 0xFF6A6E6D, 0xFF707473,
        0xE27F8382, 0xB8939796, 0x96A3A7A6, 0x43CBCBCB, 0x1DDDDDDD, 0x00F3F3F3, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x08EAEAE8, 0x15E3E3E1, 0xFFDFDFDD, 0xFFD5D5D3, 0xFFC9C9C7, 0xFFB8B9B4, 0xFFB3B4AF, 0xFFB3B4AF, 0xFFB5B6B1, 0x82B1B2AD,
        0x86B0B1AC, 0x79B5B6B1, 0x75B8B9B4, 0x64BFC0BB, 0x4FCACBC6, 0x3FD2D3CE, 0x19E4E5E0, 0x1DE2E3DE, 0x0DEAEBE6, 0x00EFF0EB, 0x00F5F6F1, 0x19E4E5DF, 0x82B2B3AD, 0x9FA5A6A0, 0x7DB7B8B0, 0x86B3B4AC, 0x5CC8CABF, 0x54CDCFC4, 0x2EDFE1D6, 0x36DBDDD2,
        0x69C3C5BA, 0x82B6B8AD, 0xFFB9BBAE, 0xFFBEC0B3, 0xFFC5C7B9, 0xFFC9CBBD, 0xFFC8CABC, 0xFFB7B8B0, 0xFFB0B2A7, 0xFFA9ABA0, 0xFFA1A597, 0xFFAEB2A4, 0xFFABB2A2, 0xFFAEB5A3, 0xB0A3AA98, 0x9BA6ACA2, 0x92AAB0A6, 0x71B8BDB6, 0x43CFD2CB, 0x26DDDFDA,
        0x0DE7E7E5, 0x00F3F3F1, 0x00FFFFFD, 0x00FCFCFC, 0x00FCFCFC, 0x00FDFDFF, 0x00FEFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FEFDFF, 0x00FFFDFF, 0x00FCFBFF, 0x00FCFFFF, 0x3FCDD6D5, 0xFF1B2421, 0xFF0A100E, 0xFF090E0A, 0xFF0B0D0A, 0xFF060C08,
        0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09,
        0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0E0D,
        0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0B0F0E, 0xFF0B100C, 0xFF0B100C, 0xFF0B100A, 0xFF0C110B, 0xFF0C110B, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0D120E,
        0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF131814, 0xFF131814, 0xFF151A16, 0xFF171C18, 0xFF191E1A, 0xFF1A1F1B, 0xFF1C211D, 0xFF1D221E, 0xFF161C18, 0xFF151B17, 0xFF141A16, 0xFF131915, 0xFF121814, 0xFF111713, 0xFF101612, 0xFF101612, 0xFF1B201C,
        0xFF1E231F, 0xFF242925, 0xFF2F3430, 0xFF3C413D, 0xFF4B504C, 0xFF575C58, 0xFF5E635F, 0xFF626864, 0xFF636965, 0xFF656B67, 0xFF676D69, 0xFF686E6A, 0xFF676D69, 0xFF676D69, 0xFF666C68, 0xFF636965, 0xFF636965, 0xFF636965, 0xFF646A66, 0xFF656B67,
        0xFF656B67, 0xFF656B67, 0xFF656B67, 0xFF656B67, 0xFF656B67, 0xFF666C68, 0xFF676D69, 0xFF686E6A, 0xFF696F6B, 0xFF6A706C, 0xFF696F6B, 0xFF686E6A, 0xFF686E6A, 0xFF676D69, 0xFF656B67, 0xFF636965, 0xFF626864, 0xFF646A66, 0xFF656B67, 0xFF6A706C,
        0xFF676D69, 0xFF616763, 0xFF5B615D, 0xFF545A56, 0xFF505652, 0xFF4C524E, 0xFF4B514D, 0xFF59625D, 0xFF5E6762, 0xFF676D69, 0xFF6E7470, 0xFF747975, 0xFF757A76, 0xFF747975, 0xFF757774, 0xFF717775, 0xFF717775, 0xFF727876, 0xFF717775, 0xFF6E7472,
        0xFF6B716F, 0xFF686E6C, 0xFF666C6A, 0xFF6B716F, 0xFF717775, 0xE2808684, 0xB0989E9C, 0x79B1B7B5, 0x47CAD0CE, 0x11E3E9E7, 0x00F5FBF9, 0x00FDFFFE, 0x00FDFFFE, 0x00FEFFFF, 0x00FCFEFD, 0x00FBFDFC, 0x00FEFFFF, 0x00FDFFFE, 0x00FBFDFC, 0x00FFFFFF,
        0x00FFFFFF, 0x00FEFEFE, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FCFCFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x0DE7E7E5, 0xFFDEDEDC, 0xFFD6D6D4, 0xFFC7C7C5, 0xFFB8B8B6, 0xFFA3A49E, 0xFFA3A49E, 0xFF9A9B95, 0xFF979892, 0xFF9B9C96,
        0xFF94958F, 0xFF8C8D87, 0xFF92938D, 0xFF9FA09A, 0xFFBBBCB6, 0xFFB2B3AD, 0xFFA2A39D, 0xFFA4A59F, 0x7DB5B6B0, 0x82B3B4AE, 0x64C0C1BB, 0x82B2B3AD, 0xFF9D9E98, 0xFF9C9D95, 0xFFADAEA6, 0xFFA5A69E, 0xFFA4A69B, 0xFFA6A89D, 0xFFAEB0A5, 0xFFA5A79C,
        0xFFA9ABA0, 0xFFACAEA3, 0xFFAEB0A3, 0xFFB5B7AA, 0xFFC1C3B5, 0xFFC4C6B8, 0xFFBBBDAF, 0xFFBCBEB3, 0xFFB6B8AD, 0xFFB8BAAF, 0xFFB6BAAC, 0xFFC7CBBD, 0xFFC4CBBB, 0xFFBCC3B3, 0xFFB2B9A9, 0xFFA6ACA0, 0xFF9FA599, 0xFF9CA298, 0xFFA4A79E, 0xFFA9ACA5,
        0x8EACADA7, 0x82B2B3AE, 0x6DBBBCB7, 0x36D6D7D2, 0x1DDFDFDD, 0x04EBEBE9, 0x00F6F6F4, 0x00FBFBFB, 0x00FDFDFD, 0x00FDFDFF, 0x00FEFEFF, 0x00FFFDFF, 0x00FFFFFF, 0x00FCFFFF, 0x4BC8D1D0, 0xFF141D1A, 0xFF050B09, 0xFF090E0A, 0xFF0C0E0B, 0xFF060C08,
        0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09,
        0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF090D0C,
        0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0B0F0E, 0xFF0B100C, 0xFF0B100C, 0xFF0B100A, 0xFF0C110B, 0xFF0C110B, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0D120E,
        0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0E130F, 0xFF0F1410, 0xFF101511, 0xFF111612, 0xFF131814, 0xFF141915, 0xFF151A16, 0xFF161B17, 0xFF161C18, 0xFF161C18, 0xFF161C18, 0xFF151B17, 0xFF151B17, 0xFF151B17, 0xFF151B17, 0xFF151B17, 0xFF121713,
        0xFF121713, 0xFF121713, 0xFF151A16, 0xFF191E1A, 0xFF1E231F, 0xFF232824, 0xFF262B27, 0xFF363C38, 0xFF3E4440, 0xFF4B514D, 0xFF595F5B, 0xFF636965, 0xFF686E6A, 0xFF696F6B, 0xFF686E6A, 0xFF696F6B, 0xFF696F6B, 0xFF676D69, 0xFF666C68, 0xFF646A66,
        0xFF646A66, 0xFF646A66, 0xFF656B67, 0xFF636965, 0xFF646A66, 0xFF656B67, 0xFF676D69, 0xFF686E6A, 0xFF696F6B, 0xFF696F6B, 0xFF696F6B, 0xFF6A706C, 0xFF6B716D, 0xFF6D736F, 0xFF6C726E, 0xFF6A706C, 0xFF676D69, 0xFF666C68, 0xFF656B67, 0xFF626864,
        0xFF626864, 0xFF616763, 0xFF616763, 0xFF616763, 0xFF616763, 0xFF626864, 0xFF626864, 0xFF535C57, 0xFF515A55, 0xFF505652, 0xFF4F5551, 0xFF545955, 0xFF5C615D, 0xFF646965, 0xFF6B6D6A, 0xFF747A78, 0xFF757B79, 0xFF767C7A, 0xFF777D7B, 0xFF777D7B,
        0xFF767C7A, 0xFF767C7A, 0xFF757B79, 0xFF727876, 0xFF6D7371, 0xFF6A706E, 0xFF696F6D, 0xFF696F6D, 0xFF6C7270, 0xF7757B79, 0xE67D8381, 0x96A3A5A4, 0x75B3B5B4, 0x32D4D6D5, 0x00EBEDEC, 0x00FCFEFD, 0x00FEFFFF, 0x00FDFFFE, 0x00FEFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FBFBFB, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x08E9E9E7, 0x1DE0E0DE, 0xFFD6D6D4, 0xFFC4C4C2, 0xFFB4B4B2, 0xFF93948C, 0xFF95968E, 0xFF808179, 0xFF75766E, 0xFF7F8078,
        0xFF75766E, 0xFF66675F, 0xFF6F7068, 0xFF797B70, 0xFFA3A59A, 0xFFA7A99E, 0xFF7A7C71, 0xFF76786D, 0xFFBBBDB2, 0xFFBEC0B5, 0xFF97998E, 0xFF9B9C96, 0xFF989991, 0xFF989991, 0xFFA9AAA2, 0xFFAFB1A6, 0xFFA4A69B, 0xFFAFB1A4, 0xFFA4A699, 0xFFA8AA9F,
        0xFFB4B6AB, 0xFFB6B8AD, 0xFFAAAC9F, 0xFFA5A79A, 0xFFAAAC9E, 0xFFADAFA1, 0xFFACAEA0, 0xFFB5B7AA, 0xFFA3A598, 0xFFA9AB9E, 0xFFA6AA9C, 0xFFB6BAAC, 0xFFB7BDAF, 0xFFB1B7A9, 0xFFAFB6A6, 0xFFA6AC9E, 0xFFA3A99B, 0xFFA5AB9F, 0xFFABAEA3, 0xFFAAADA2,
        0xFFA8AA9F, 0xFFA9ABA0, 0xFFAEB0A5, 0xFFA6A79F, 0xFFA9AAA2, 0x92ADAEA6, 0x82B2B3AD, 0x6DBCBDB7, 0x4BCCCDC7, 0x21DFE0DB, 0x04EDEEE9, 0x00F8F4F5, 0x00FCFCFC, 0x00FCFFFF, 0x58C1CAC7, 0xFF121B18, 0xFF050B07, 0xFF080D09, 0xFF0A0C09, 0xFF060C08,
        0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09,
        0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF090D0C,
        0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0B0F0E, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0B0F0E, 0xFF0B100C, 0xFF0B100C, 0xFF0B100A, 0xFF0C110B, 0xFF0C110B, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0D120E,
        0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0C110D, 0xFF0C110D, 0xFF0D120E, 0xFF0E130F, 0xFF0F1410, 0xFF101511, 0xFF101511, 0xFF101511, 0xFF131915, 0xFF131915, 0xFF141A16, 0xFF141A16, 0xFF151B17, 0xFF161C18, 0xFF161C18, 0xFF171D19, 0xFF171C18,
        0xFF151A16, 0xFF131814, 0xFF111612, 0xFF101511, 0xFF101511, 0xFF101511, 0xFF111612, 0xFF151B17, 0xFF161C18, 0xFF1A201C, 0xFF212723, 0xFF2C322E, 0xFF393F3B, 0xFF454B47, 0xFF4C524E, 0xFF595F5B, 0xFF5F6561, 0xFF656B67, 0xFF696F6B, 0xFF6A706C,
        0xFF6A706C, 0xFF6A706C, 0xFF6B716D, 0xFF676D69, 0xFF676D69, 0xFF656B67, 0xFF646A66, 0xFF636965, 0xFF646A66, 0xFF666C68, 0xFF686E6A, 0xFF6A706C, 0xFF6B716D, 0xFF6A706C, 0xFF696F6B, 0xFF676D69, 0xFF676D69, 0xFF686E6A, 0xFF6A706C, 0xFF656B67,
        0xFF656B67, 0xFF656B67, 0xFF646A66, 0xFF636965, 0xFF626864, 0xFF616763, 0xFF616763, 0xFF69726D, 0xFF69726D, 0xFF6A706C, 0xFF676D69, 0xFF636864, 0xFF5D625E, 0xFF585D59, 0xFF575956, 0xFF535957, 0xFF575D5B, 0xFF5E6462, 0xFF666C6A, 0xFF6E7472,
        0xFF747A78, 0xFF787E7C, 0xFF7A807E, 0xFF767C7A, 0xFF757B79, 0xFF757B79, 0xFF747A78, 0xFF707674, 0xFF6A706E, 0xFF666C6A, 0xFF666C6A, 0xFF636564, 0xFF616362, 0xFF676968, 0xFB737574, 0xD1888A89, 0x9F9FA1A0, 0x60BEC0BF, 0x0DE5E7E6, 0x00FDFDFD,
        0x00FFFFFF, 0x00FCFCFC, 0x00FFFFFF, 0x00FDFDFD, 0x00FBFBFB, 0x00FFFFFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00EEEEEC, 0x11E6E6E4, 0xFFDADAD8, 0xFFC8C8C6, 0xFFB8B8B6, 0xFFA0A297, 0xFFA3A59A, 0xFF8E9085, 0xFF7F8176, 0xFF828479,
        0xFF73756A, 0xFF606257, 0xFF66685D, 0xFF6F7164, 0xFF6D6F62, 0xFF5C5E51, 0xFF6A6C5F, 0xFFA2A497, 0xFFB9BBAE, 0xFF919386, 0xFF9EA093, 0xFF92938B, 0xFF909189, 0xFF8C8D85, 0xFF97998E, 0xFFA2A499, 0xFF9EA093, 0xFFC2C4B7, 0xFFAFB1A4, 0xFFA0A297,
        0xFFAEB0A5, 0xFFBBBDB2, 0xFFBDBFB2, 0xFFB0B2A5, 0xFF9FA193, 0xFF9C9E90, 0xFFA6A89A, 0xFFA0A294, 0xFF858779, 0xFF989A8D, 0xFF989C8E, 0xFF9A9E90, 0xFF979D8F, 0xFF93998B, 0xFF91978B, 0xFF979D8F, 0xFF9FA597, 0xFFAAB0A2, 0xFFB0B4A6, 0xFFA8AC9D,
        0xFF9EA092, 0xFF9C9E90, 0xFFA1A395, 0xFFA4A698, 0xFFA4A698, 0xFFA1A396, 0xFF9D9F92, 0xFF999B8E, 0xFF999B90, 0xFF9C9E93, 0xB49FA196, 0x71BCB8B5, 0x60C1C2BD, 0x43CCD1CB, 0x9F9FA8A3, 0xFF0A130E, 0xFF060C08, 0xFF0B100C, 0xFF0C0E0B, 0xFF060C08,
        0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09,
        0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF090D0C,
        0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0B0F0E, 0xFF0B100C, 0xFF0B100C, 0xFF0B100A, 0xFF0C110B, 0xFF0C110B, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0D120E,
        0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0F1410, 0xFF0F1410, 0xFF0F1410, 0xFF0E1410, 0xFF0E1410, 0xFF0F1511, 0xFF101612, 0xFF111713, 0xFF121814, 0xFF121814, 0xFF131915, 0xFF171C18,
        0xFF171C18, 0xFF161B17, 0xFF151A16, 0xFF141915, 0xFF141915, 0xFF141915, 0xFF141915, 0xFF0D130F, 0xFF0D130F, 0xFF0C120E, 0xFF0C120E, 0xFF101612, 0xFF151B17, 0xFF1A201C, 0xFF1D231F, 0xFF272D29, 0xFF303632, 0xFF3F4541, 0xFF4C524E, 0xFF565C58,
        0xFF5E6460, 0xFF666C68, 0xFF6B716D, 0xFF6A706C, 0xFF6B716D, 0xFF6D736F, 0xFF6D736F, 0xFF6C726E, 0xFF6B716D, 0xFF6B716D, 0xFF6B716D, 0xFF656B67, 0xFF676D69, 0xFF6A706C, 0xFF6B716D, 0xFF6A706C, 0xFF696F6B, 0xFF686E6A, 0xFF686E6A, 0xFF686E6A,
        0xFF696F6B, 0xFF6A706C, 0xFF6A706C, 0xFF696F6B, 0xFF686E6A, 0xFF676D69, 0xFF666C68, 0xFF5F6863, 0xFF626B66, 0xFF686E6A, 0xFF6C726E, 0xFF707571, 0xFF707571, 0xFF6F7470, 0xFF70726F, 0xFF6A706C, 0xFF656B67, 0xFF5C625E, 0xFF565C58, 0xFF535955,
        0xFF565C58, 0xFF5A605C, 0xFF5E6460, 0xFF6E7470, 0xFF727874, 0xFF797F7B, 0xFF7F8581, 0xFF808682, 0xFF7C827E, 0xFF767C78, 0xFF737975, 0xFF6B706C, 0xFF6D726E, 0xFF6C716D, 0xFF6A6F6B, 0xFF636864, 0xFF545955, 0xFF535854, 0xFF676C68, 0x21DBDBDB,
        0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FEFEFE, 0x00FBFBFB, 0x00FFFFFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F1F1EF, 0x08EAEAE8, 0x21DEDEDC, 0x43CECECC, 0xFFC0C0BE, 0xFFBDBFB4, 0xFFB6B8AD, 0xFFA1A398, 0xFF8D8F84, 0xFF828479,
        0xFF7A7C71, 0xFF76786D, 0xFF7A7C71, 0xFF848679, 0xFF8E9083, 0xFF828477, 0xFF747669, 0xFF8C8E81, 0xFFABADA0, 0xFF929487, 0xFF95978A, 0xFF94958D, 0xFF95968E, 0xFF97998E, 0xFF9A9C91, 0xFF96988B, 0xFF949689, 0xFFC5C7B9, 0xFFB5B7A9, 0xFFA1A398,
        0xFFAFB1A6, 0xFFC3C5BA, 0xFFCCCEC1, 0xFFBDBFB2, 0xFFA3A597, 0xFF9A9C8E, 0xFFA1A395, 0xFF898C7B, 0xFF6D705F, 0xFF888A7C, 0xFF898D7F, 0xFF818577, 0xFF848A7E, 0xFF8C9286, 0xFF8A9086, 0xFF878D81, 0xFF8F9589, 0xFF999F93, 0xFFA0A496, 0xFF9CA091,
        0xFF9A9D8C, 0xFF9DA08F, 0xFFA3A695, 0xFFA1A493, 0xFFA0A392, 0xFFA0A392, 0xFFA2A496, 0xFFA4A698, 0xFFA4A698, 0xFFA1A396, 0xFF9EA093, 0xFF96938C, 0xFF92938B, 0xFF9CA19A, 0xFF7E887F, 0xFF050F07, 0xFF080E0A, 0xFF0B100C, 0xFF0C0E0D, 0xFF060C08,
        0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09,
        0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF090D0C,
        0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0B0F0E, 0xFF0B100C, 0xFF0B100C, 0xFF0B100A, 0xFF0C110B, 0xFF0C110B, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0D120E,
        0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0C120E, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0E1410, 0xFF0E1410, 0xFF0F1511, 0xFF0F1511, 0xFF131814,
        0xFF131814, 0xFF131814, 0xFF121713, 0xFF121713, 0xFF111612, 0xFF111612, 0xFF111612, 0xFF111713, 0xFF121814, 0xFF131915, 0xFF131915, 0xFF131915, 0xFF111713, 0xFF0F1511, 0xFF0D130F, 0xFF0C120E, 0xFF101612, 0xFF151B17, 0xFF1B211D, 0xFF222824,
        0xFF2C322E, 0xFF383E3A, 0xFF414743, 0xFF545A56, 0xFF5B615D, 0xFF656B67, 0xFF6C726E, 0xFF6E7470, 0xFF6D736F, 0xFF6C726E, 0xFF6B716D, 0xFF6A706C, 0xFF696F6B, 0xFF676D69, 0xFF646A66, 0xFF626864, 0xFF636965, 0xFF656B67, 0xFF676D69, 0xFF646A66,
        0xFF656B67, 0xFF676D69, 0xFF686E6A, 0xFF696F6B, 0xFF686E6A, 0xFF666C68, 0xFF656B67, 0xFF5D6661, 0xFF5D6661, 0xFF5F6561, 0xFF606662, 0xFF626763, 0xFF646965, 0xFF666B67, 0xFF696B68, 0xFF646A66, 0xFF676D69, 0xFF6B716D, 0xFF6D736F, 0xFF6B716D,
        0xFF656B67, 0xFF5D635F, 0xFF595F5B, 0xFF555B57, 0xFF585E5A, 0xFF5C625E, 0xFF626864, 0xFF686E6A, 0xFF6F7571, 0xFF747A76, 0xFF767C78, 0xFF6F7470, 0xFF737874, 0xFF6D726E, 0xFF656A66, 0xFF5A5F5B, 0xFF4B504C, 0xFF494E4A, 0xFF525753, 0x8AA9A9A9,
        0x00FBFBFB, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F9F9F7, 0x00F5F5F3, 0x00EEEEEC, 0x15E4E4E2, 0x26DCDCDA, 0x47CECFC9, 0x5CC4C5BF, 0x79B7B8B2, 0x9FA5A6A0, 0xFF959690,
        0xFF94958F, 0xFF9A9B95, 0xFF9A9B95, 0xFF828479, 0xFF7D7F74, 0xFFA5A79C, 0xFFB6B8AD, 0xFF909287, 0xFF93958A, 0xFF989A8F, 0xFF97998E, 0xFF8A8C81, 0xFF898B80, 0xFF8F9186, 0xFF96988B, 0xFF8E9083, 0xFF8D8F81, 0xFFA8AA9C, 0xFF999B8D, 0xFF9EA095,
        0xFFA6A89D, 0xFFB2B4A9, 0xFFB3B5A8, 0xFFADAFA2, 0xFFAFB1A3, 0xFFB4B6A8, 0xFFB2B4A6, 0xFF929582, 0xFF848774, 0xFF959887, 0xFF848879, 0xFF75796B, 0xFF777D71, 0xFF7D8379, 0xFF70756E, 0xFF686D67, 0xFF6A6F68, 0xFF70766C, 0xFF7B7E73, 0xFF85897B,
        0xFF8D8F81, 0xFF929584, 0xFF959887, 0xFFA5A895, 0xFFA2A594, 0xFFA1A493, 0xFFA5A897, 0xFFABAD9F, 0xFFAEB0A2, 0xFFACAEA1, 0xFFAAAC9F, 0xFFA9A79B, 0xFF9C9E91, 0xFF9DA397, 0xFF7A8479, 0xFF0A140B, 0xFF091009, 0xFF080D09, 0xFF090B0A, 0xFF060C08,
        0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09,
        0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF080C0B,
        0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0B0F0E, 0xFF0B100C, 0xFF0B100C, 0xFF0B100A, 0xFF0C110B, 0xFF0C110B, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0D120E,
        0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0E1410, 0xFF0E1410, 0xFF0E1410, 0xFF0F1511, 0xFF0F1511, 0xFF0F1511, 0xFF0F1511, 0xFF0F1511, 0xFF121713,
        0xFF121713, 0xFF121713, 0xFF121713, 0xFF121713, 0xFF111612, 0xFF121713, 0xFF121713, 0xFF121814, 0xFF111713, 0xFF0E1410, 0xFF0D130F, 0xFF0D130F, 0xFF0E1410, 0xFF111713, 0xFF121814, 0xFF131915, 0xFF121814, 0xFF101612, 0xFF0C120E, 0xFF0A100C,
        0xFF0C120E, 0xFF101612, 0xFF141A16, 0xFF1A201C, 0xFF222824, 0xFF2E3430, 0xFF3B413D, 0xFF474D49, 0xFF535955, 0xFF5E6460, 0xFF656B67, 0xFF6A706C, 0xFF6A706C, 0xFF696F6B, 0xFF676D69, 0xFF666C68, 0xFF666C68, 0xFF686E6A, 0xFF6B716D, 0xFF656B67,
        0xFF666C68, 0xFF666C68, 0xFF656B67, 0xFF646A66, 0xFF616763, 0xFF5E6460, 0xFF5D635F, 0xFF616A65, 0xFF616A65, 0xFF646A66, 0xFF646A66, 0xFF646965, 0xFF626763, 0xFF606561, 0xFF616360, 0xFF636A63, 0xFF636A63, 0xFF636A63, 0xFF646B64, 0xFF656C65,
        0xFF686F68, 0xFF6B726B, 0xFF6C736C, 0xFF6F766F, 0xFF6D746D, 0xFF676E67, 0xFF606760, 0xFF5D645D, 0xFF5E655E, 0xFF5E655E, 0xFF5C635C, 0xFF6C716B, 0xFF686D67, 0xFF5B605A, 0xFF545953, 0xFF50554F, 0xFF515650, 0xFF5B605A, 0xFF5E635D, 0xC98C8C8C,
        0x00FCFCFC, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFD, 0x00FFFFFD, 0x00FEFEFC, 0x00FBFBF9, 0x00FAFAF8, 0x00F8F8F6, 0x00F4F4F2, 0x00F0F0EE, 0x11E5E5E3, 0x36D4D4D2,
        0x4FC8C8C6, 0x69BBBBB9, 0x86ADADAB, 0xA3A3A49E, 0xFF999A94, 0xFF92938D, 0xFF9E9F99, 0xFF969791, 0xFF898A84, 0xFF7F807A, 0xFF91928C, 0xFF898B80, 0xFF85877C, 0xFF87897C, 0xFF919386, 0xFF8F9183, 0xFF939587, 0xFF8F9183, 0xFF8F9183, 0xFF8C8E83,
        0xFF898B80, 0xFF8D8F84, 0xFF8C8E81, 0xFF919386, 0xFFAEB0A2, 0xFFC0C2B4, 0xFFB5B7A9, 0xFF838671, 0xFF9C9F8C, 0xFFA9AC9B, 0xFF8E9283, 0xFF8C8F84, 0xFF8F958B, 0xFF838881, 0xFF60655F, 0xFF4D5150, 0xFF4D5150, 0xFF525753, 0xFF60635C, 0xFF70736A,
        0xFF7F8174, 0xFF86887A, 0xFF888A7C, 0xFF838675, 0xFF8B8E7D, 0xFF97998B, 0xFFA5A799, 0xFFAEB0A3, 0xFFAFB1A4, 0xFFAAAC9F, 0xFFA5A79C, 0xFFADAB9E, 0xFFA6A89A, 0xFFA6AC9E, 0xFF7B8678, 0xFF09130A, 0xFF070E07, 0xFF070C08, 0xFF090B0A, 0xFF060C08,
        0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09,
        0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF080C0B,
        0xFF080C0B, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0B0F0E, 0xFF0B100C, 0xFF0B100C, 0xFF0B100A, 0xFF0C110B, 0xFF0C110B, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0D120E,
        0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0E1410, 0xFF0E1410, 0xFF0E1410, 0xFF0F1511, 0xFF0F1511, 0xFF101612, 0xFF101612, 0xFF101612, 0xFF0F1410,
        0xFF0F1410, 0xFF0F1410, 0xFF0F1410, 0xFF0F1410, 0xFF101511, 0xFF111612, 0xFF121713, 0xFF0F1511, 0xFF0F1511, 0xFF101612, 0xFF101612, 0xFF111713, 0xFF101612, 0xFF101612, 0xFF101612, 0xFF111713, 0xFF121814, 0xFF131915, 0xFF131915, 0xFF121814,
        0xFF111713, 0xFF101612, 0xFF101612, 0xFF0E1410, 0xFF0F1511, 0xFF111713, 0xFF121814, 0xFF161C18, 0xFF1F2521, 0xFF2A302C, 0xFF333935, 0xFF444A46, 0xFF4C524E, 0xFF585E5A, 0xFF616763, 0xFF676D69, 0xFF686E6A, 0xFF676D69, 0xFF676D69, 0xFF696F6B,
        0xFF696F6B, 0xFF6A706C, 0xFF6A706C, 0xFF696F6B, 0xFF686E6A, 0xFF666C68, 0xFF656B67, 0xFF616A65, 0xFF616A65, 0xFF626864, 0xFF626864, 0xFF646965, 0xFF656A66, 0xFF676C68, 0xFF6A6C69, 0xFF5D645D, 0xFF5E655E, 0xFF5E655E, 0xFF5F665F, 0xFF5F665F,
        0xFF5F665F, 0xFF5E655E, 0xFF5D645D, 0xFF636A63, 0xFF656C65, 0xFF656C65, 0xFF636A63, 0xFF636A63, 0xFF636A63, 0xFF5F665F, 0xFF5A615A, 0xFF4C534C, 0xFF454C45, 0xFF424942, 0xFF4C534C, 0xFF525952, 0xFF5A615A, 0xFF666D66, 0xFF5D645D, 0xC98C8C8C,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FDFDFD, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFD, 0x00FFFFFD, 0x00FFFFFD, 0x00FFFFFD, 0x00FFFFFD, 0x00FEFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FCFCFE, 0x00F3F3F5, 0x0DEAEBE6, 0x15E6E7E2, 0x3FD2D3CE, 0x54C7C8C3, 0x7DB3B4AF, 0x92AAABA6, 0xB09C9D98, 0xFF969792, 0xFF8C8E83, 0xFF8C8E83, 0xFF898B7E, 0xFF8C8E81, 0xFF888A7C, 0xFF8E9082, 0xFF808274, 0xFF999C8B, 0xFF8A8C81,
        0xFF77796E, 0xFF727469, 0xFF747669, 0xFF818376, 0xFFACAEA0, 0xFFCBCDBF, 0xFFC2C4B6, 0xFF6C6F5A, 0xFFA2A590, 0xFFA7AA99, 0xFF838778, 0xFF95988D, 0xFFA4AAA0, 0xFF91968F, 0xFF656A64, 0xFF63676A, 0xFF5F6364, 0xFF5B5F5E, 0xFF5B5D5A, 0xFF5E615A,
        0xFF66685D, 0xFF6C6E61, 0xFF6F7164, 0xFF7A7C6E, 0xFF7C7E70, 0xFF808274, 0xFF85877A, 0xFF8D8F82, 0xFF97998E, 0xFFA1A398, 0xFFA7A99E, 0xFFA4A293, 0xFFA9AC9B, 0xFFAFB5A7, 0xFF7D887A, 0xFF09130A, 0xFF060D06, 0xFF080D09, 0xFF090B0A, 0xFF060C08,
        0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09,
        0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF080C0B,
        0xFF080C0B, 0xFF080C0B, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0B0F0E, 0xFF0B100C, 0xFF0B100C, 0xFF0B100A, 0xFF0C110B, 0xFF0C110B, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0D120E,
        0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0F1410, 0xFF0F1410, 0xFF0F1410, 0xFF0F1410, 0xFF0F1410, 0xFF0F1410, 0xFF0F1410, 0xFF0F1410, 0xFF0C120E, 0xFF0C120E, 0xFF0D130F, 0xFF0D130F, 0xFF0E1410, 0xFF0F1511, 0xFF0F1511, 0xFF101612, 0xFF121713,
        0xFF121713, 0xFF101511, 0xFF101511, 0xFF0F1410, 0xFF101511, 0xFF111612, 0xFF121713, 0xFF101612, 0xFF101612, 0xFF101612, 0xFF101612, 0xFF101612, 0xFF101612, 0xFF101612, 0xFF101612, 0xFF101612, 0xFF101612, 0xFF0F1511, 0xFF0F1511, 0xFF101612,
        0xFF111713, 0xFF111713, 0xFF111713, 0xFF121814, 0xFF141A16, 0xFF141A16, 0xFF121814, 0xFF0F1511, 0xFF0E1410, 0xFF101612, 0xFF131915, 0xFF161C18, 0xFF1A201C, 0xFF212723, 0xFF2C322E, 0xFF393F3B, 0xFF494F4B, 0xFF585E5A, 0xFF616763, 0xFF6B716D,
        0xFF6A706C, 0xFF6A706C, 0xFF696F6B, 0xFF686E6A, 0xFF676D69, 0xFF656B67, 0xFF646A66, 0xFF616A65, 0xFF626B66, 0xFF656B67, 0xFF666C68, 0xFF666B67, 0xFF646965, 0xFF626763, 0xFF626461, 0xFF626962, 0xFF626962, 0xFF636A63, 0xFF636A63, 0xFF626962,
        0xFF616861, 0xFF5F665F, 0xFF5E655E, 0xFF5B625B, 0xFF5F665F, 0xFF616861, 0xFF606760, 0xFF606760, 0xFF606760, 0xFF5A615A, 0xFF525952, 0xFF484F48, 0xFF3B423B, 0xFF3A413A, 0xFF4F564F, 0xFF585F58, 0xFF606760, 0xFF6B726B, 0xFF5C635C, 0xBC929292,
        0x00FFFFFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FAFAFA, 0x00F8F8F8, 0x2ED5D5D5, 0xFF9F9F9F, 0xFF9C9E90, 0xFF9EA093, 0xFFA1A398, 0xFFA1A197, 0xFF9C9C90, 0xFF908E7F, 0xFF8C8B77, 0xFFA4A48C, 0xFF959885,
        0xFF838673, 0xFF7A7E6D, 0xFF787C6B, 0xFF828677, 0xFFA7AE9E, 0xFFC4CBBB, 0xFFBFC5B7, 0xFF645F4B, 0xFFBCBAA5, 0xFF94937F, 0xFF919280, 0xFF838776, 0xFF838A7A, 0xFF7A8372, 0xFF5A6555, 0xFF5B6157, 0xFF61665F, 0xFF686D69, 0xFF656968, 0xFF686C6B,
        0xFF6F7470, 0xFF696E67, 0xFF686E64, 0xFF6A6D62, 0xFF686B60, 0xFF676A5F, 0xFF6A6D64, 0xFF6D7067, 0xFF6F726B, 0xFF70736C, 0xFF72756E, 0xFF797F73, 0xFF80867A, 0xFF8F958B, 0xFF696E67, 0xFF0F140E, 0xFF070C08, 0xFF040905, 0xFF080C0B, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09,
        0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF030804, 0xFF080D09, 0xFF111612, 0xFF090E0A, 0xFF020703, 0xFF070C08, 0xFF090E0A, 0xFF0A140C, 0xFF040E06, 0xFF030D05, 0xFF030D05, 0xFF061008, 0xFF061008, 0xFF0B150D, 0xFF020C04, 0xFF0A0B05,
        0xFF0E0F09, 0xFF080904, 0xFF0F110C, 0xFF0C110D, 0xFF090D0C, 0xFF080E0C, 0xFF060C0A, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0C110D, 0xFF0C110D,
        0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C120E, 0xFF0C120E, 0xFF0C120E, 0xFF0C120E, 0xFF0C120E, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D120E, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0F1410, 0xFF101511, 0xFF101511, 0xFF101511, 0xFF101413,
        0xFF101413, 0xFF101413, 0xFF101413, 0xFF101413, 0xFF101413, 0xFF101413, 0xFF101413, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0F1511, 0xFF0F1511, 0xFF0F1511, 0xFF0F1511, 0xFF0F1511,
        0xFF0F1511, 0xFF0F1511, 0xFF0F1511, 0xFF121713, 0xFF121713, 0xFF121713, 0xFF131814, 0xFF131814, 0xFF131814, 0xFF131814, 0xFF141915, 0xFF121713, 0xFF111612, 0xFF101511, 0xFF101511, 0xFF131814, 0xFF181D19, 0xFF1E231F, 0xFF212622, 0xFF303531,
        0xFF3B403C, 0xFF4B504C, 0xFF565B57, 0xFF5D625E, 0xFF626763, 0xFF696E6A, 0xFF6E736F, 0xFF656A66, 0xFF656A66, 0xFF656A66, 0xFF636864, 0xFF616662, 0xFF606561, 0xFF626763, 0xFF636864, 0xFF5F6460, 0xFF696E6A, 0xFF616662, 0xFF5F6460, 0xFF666B67,
        0xFF646965, 0xFF636864, 0xFF606561, 0xFF60655F, 0xFF616660, 0xFF646963, 0xFF686D67, 0xFF656A64, 0xFF585D57, 0xFF464B45, 0xFF393E38, 0xFF4A4F49, 0xFF515650, 0xFF3A3F39, 0xFF585D57, 0xFF5A5F59, 0xFF60655F, 0xFF646963, 0xFF5C615B, 0xA79C9C9C,
        0x00FFFFFF, 0x00FFFFFF, 0x00FBFBFB, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x04E9E9E9, 0x58C1C1C1, 0xFFACAEA1, 0xFFB5B7AC, 0xFFBABBB3, 0xFFB3B3AB, 0xFFB2B2A8, 0xFFACAA9D, 0xFF9F9E8A, 0xFFAFAE99, 0xFFAEB1A0,
        0xFFA3A695, 0xFF999B8D, 0xFF8F9384, 0xFF898D7F, 0xFF949A8C, 0xFF969C8E, 0xFF787E72, 0xFF525041, 0xFF929383, 0xFF888979, 0xFF999C8B, 0xFF8C8F7E, 0xFF8B8F7E, 0xFF75796A, 0xFF4A5141, 0xFF4D5347, 0xFF42483E, 0xFF4C514B, 0xFF535854, 0xFF585D59,
        0xFF626761, 0xFF6A7066, 0xFF71776B, 0xFF6D7065, 0xFF65685D, 0xFF5B5E53, 0xFF55584F, 0xFF565950, 0xFF565950, 0xFF52554E, 0xFF4C4F48, 0xFF464C40, 0xFF4A5044, 0xFF5B6157, 0xFF464B44, 0xFF030802, 0xFF060B07, 0xFF060B07, 0xFF0A0E0D, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09,
        0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF0D120E, 0xFF090E0A, 0xFF070C08, 0xFF050A06, 0xFF040905, 0xFF0A0F0B, 0xFF0F1410, 0xFF080D09, 0xFF040A06, 0xFF0B110D, 0xFF090F0B, 0xFF080E0A, 0xFF050B07, 0xFF0A100C, 0xFF000602, 0xFF0D130F, 0xFF13140F,
        0xFF000100, 0xFF131510, 0xFF0C0E0B, 0xFF000501, 0xFF0A0E0D, 0xFF030907, 0xFF0C1210, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0C110D,
        0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0B110D, 0xFF0C120E, 0xFF0C120E, 0xFF0C120E, 0xFF0C120E, 0xFF0C120E, 0xFF0D130F, 0xFF0D130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0F1410, 0xFF0F1410, 0xFF101511, 0xFF101511, 0xFF101511, 0xFF111514,
        0xFF111514, 0xFF111514, 0xFF111514, 0xFF111514, 0xFF111514, 0xFF111514, 0xFF111514, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0F1511, 0xFF0F1511, 0xFF0F1511, 0xFF0F1511, 0xFF0F1511,
        0xFF0F1511, 0xFF0F1511, 0xFF0F1511, 0xFF111612, 0xFF111612, 0xFF111612, 0xFF121713, 0xFF121713, 0xFF121713, 0xFF121713, 0xFF121713, 0xFF161B17, 0xFF151A16, 0xFF131814, 0xFF111612, 0xFF111612, 0xFF111612, 0xFF131814, 0xFF131814, 0xFF111612,
        0xFF151A16, 0xFF1B201C, 0xFF212622, 0xFF272C28, 0xFF323733, 0xFF3E433F, 0xFF484D49, 0xFF585D59, 0xFF5D625E, 0xFF656A66, 0xFF6A6F6B, 0xFF6B706C, 0xFF696E6A, 0xFF666B67, 0xFF656A66, 0xFF686D69, 0xFF676C68, 0xFF636864, 0xFF646965, 0xFF616662,
        0xFF5B605C, 0xFF5E635F, 0xFF626763, 0xFF646963, 0xFF676C66, 0xFF666B65, 0xFF5C615B, 0xFF4E534D, 0xFF454A44, 0xFF454A44, 0xFF494E48, 0xFF545953, 0xFF595E58, 0xFF414640, 0xFF5C615B, 0xFF5B605A, 0xFF616660, 0xFF646963, 0xFF5C615B, 0x9F9F9F9F,
        0x00FFFFFF, 0x00FFFFFF, 0x00FBFBFB, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FCFCFC, 0x0DE6E6E6, 0x79BABCB1, 0xFFBDBEB6, 0xFFBFC0BA, 0xFFB1B0AB, 0xFFB4B4AC, 0xFFB6B4A8, 0xFFAEAC9D, 0xFFC9C8B4, 0xFFD7D8CA,
        0xFFC8CABC, 0xFFB5B7AA, 0xFFACAEA1, 0xFF9EA196, 0xFF93968B, 0xFF7A8074, 0xFF484E44, 0xFF393F35, 0xFF53564D, 0xFF64675C, 0xFF87897C, 0xFF87897B, 0xFFA0A191, 0xFF8E8F7D, 0xFF6A6957, 0xFF4A513F, 0xFF2E3426, 0xFF31372B, 0xFF383D36, 0xFF3A3F38,
        0xFF41473B, 0xFF515749, 0xFF5D6452, 0xFF686C5E, 0xFF65695B, 0xFF5F6257, 0xFF5A5D52, 0xFF5A5D54, 0xFF565950, 0xFF494C43, 0xFF3A3D36, 0xFF393F33, 0xFF343A2E, 0xFF3F453B, 0xFF343932, 0xFF040903, 0xFF090E0A, 0xFF060B07, 0xFF060A09, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09,
        0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF070C08, 0xFF050A06, 0xFF070C08, 0xFF080D09, 0xFF060B07, 0xFF060B07, 0xFF060B07, 0xFF070C08, 0xFF0A0E0D, 0xFF060A09, 0xFF080C0B, 0xFF0E1211, 0xFF0F1312, 0xFF030706, 0xFF020605, 0xFF0C100F, 0xFF070C08,
        0xFF0D120E, 0xFF0C110D, 0xFF010602, 0xFF0C110D, 0xFF0C100F, 0xFF0B0F0E, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0C110D,
        0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0B110D, 0xFF0B110D, 0xFF0B110D, 0xFF0C120E, 0xFF0C120E, 0xFF0C120E, 0xFF0C120E, 0xFF0D130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0F1410, 0xFF0F1410, 0xFF101511, 0xFF101511, 0xFF101511, 0xFF111514,
        0xFF111514, 0xFF111514, 0xFF111514, 0xFF111514, 0xFF111514, 0xFF111514, 0xFF111514, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0E1410, 0xFF0E1410, 0xFF0E1410, 0xFF0E1410, 0xFF0E1410,
        0xFF0E1410, 0xFF0E1410, 0xFF0E1410, 0xFF0F1410, 0xFF101511, 0xFF101511, 0xFF101511, 0xFF101511, 0xFF111612, 0xFF111612, 0xFF111612, 0xFF121713, 0xFF121713, 0xFF121713, 0xFF121713, 0xFF121713, 0xFF121713, 0xFF111612, 0xFF111612, 0xFF131814,
        0xFF131814, 0xFF131814, 0xFF131814, 0xFF131814, 0xFF151A16, 0xFF1A1F1B, 0xFF1E231F, 0xFF282D29, 0xFF2F3430, 0xFF383D39, 0xFF434844, 0xFF4D524E, 0xFF575C58, 0xFF5F6460, 0xFF636864, 0xFF636864, 0xFF5E635F, 0xFF646965, 0xFF666B67, 0xFF626763,
        0xFF616662, 0xFF626763, 0xFF656A66, 0xFF656A64, 0xFF626761, 0xFF585D57, 0xFF494E48, 0xFF3E433D, 0xFF40453F, 0xFF4C514B, 0xFF575C56, 0xFF585D57, 0xFF5C615B, 0xFF454A44, 0xFF5E635D, 0xFF5C615B, 0xFF626761, 0xFF636862, 0xFF5D625C, 0x92A5A5A5,
        0x00FFFFFF, 0x00FFFFFF, 0x00FCFCFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFCFC, 0x00FFFFFF, 0x00FFFFFF, 0x00F9F9F9, 0x19E7E8E0, 0x2ADDDED8, 0x2EDADBD6, 0x4BCCCBC7, 0x54CAC9C4, 0x64C7C4BB, 0xFFBFBDB0, 0xFFE3E2D0, 0xFFEFEFE3,
        0xFFD4D4C8, 0xFFB8BAAD, 0xFFB7B9AC, 0xFFAEB0A5, 0xFF979A8F, 0xFF7B7E75, 0xFF4D5047, 0xFF3A433E, 0xFF303632, 0xFF454A43, 0xFF606358, 0xFF717165, 0xFFA3A192, 0xFF9A9784, 0xFF817C69, 0xFF505541, 0xFF393D2C, 0xFF363A2C, 0xFF2F3227, 0xFF2F3227,
        0xFF2F3325, 0xFF373B2A, 0xFF3C412D, 0xFF44483A, 0xFF4F5345, 0xFF565A4C, 0xFF575A4F, 0xFF595C51, 0xFF5B5E55, 0xFF54574E, 0xFF494C43, 0xFF3B4135, 0xFF30362A, 0xFF33392F, 0xFF2C312A, 0xFF070C06, 0xFF090E0A, 0xFF060B07, 0xFF060A09, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09,
        0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF070C08, 0xFF070C08, 0xFF090E0A, 0xFF060B07, 0xFF090E0A, 0xFF0C110D, 0xFF0A0F0B, 0xFF0E130F, 0xFF0B0D0C, 0xFF0C0E0D, 0xFF0D0F0E, 0xFF020403, 0xFF0C0E0D, 0xFF060807, 0xFF141615, 0xFF141615, 0xFF0F1513,
        0xFF050B09, 0xFF070D0B, 0xFF0B110F, 0xFF0E1211, 0xFF030804, 0xFF0B100C, 0xFF090E0A, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0A0F0B, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C,
        0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0B110D, 0xFF0B110D, 0xFF0B110D, 0xFF0B110D, 0xFF0C120E, 0xFF0C120E, 0xFF0C120E, 0xFF0C120E, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0F1410, 0xFF0F1410, 0xFF101511, 0xFF101511, 0xFF101511, 0xFF111612,
        0xFF111612, 0xFF111612, 0xFF111612, 0xFF111612, 0xFF111612, 0xFF111612, 0xFF111612, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F,
        0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0F1410, 0xFF0F1410, 0xFF0F1410, 0xFF0F1410, 0xFF0F1410, 0xFF101511, 0xFF111612, 0xFF131814, 0xFF141915, 0xFF151A16, 0xFF151A16, 0xFF151A16, 0xFF141915,
        0xFF151A16, 0xFF171C18, 0xFF181D19, 0xFF181D19, 0xFF171C18, 0xFF151A16, 0xFF141915, 0xFF121713, 0xFF131814, 0xFF161B17, 0xFF1A1F1B, 0xFF202521, 0xFF2A2F2B, 0xFF343935, 0xFF3B403C, 0xFF4F5450, 0xFF525753, 0xFF5C615D, 0xFF5E635F, 0xFF636864,
        0xFF6A6F6B, 0xFF666B67, 0xFF646965, 0xFF5A5F59, 0xFF4F544E, 0xFF424741, 0xFF3D423C, 0xFF464B45, 0xFF525751, 0xFF585D57, 0xFF585D57, 0xFF525751, 0xFF565B55, 0xFF444943, 0xFF5D625C, 0xFF5B605A, 0xFF636862, 0xFF626761, 0xFF5E635D, 0x82ADADAD,
        0x00FEFEFE, 0x00FFFFFF, 0x00FCFCFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFCFC, 0x00FCFCFC, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFB, 0x00FDFEF9, 0x00FEFEFC, 0x00F8F6F7, 0x00F8F7F5, 0x08F0EDE8, 0x43D9D7CB, 0xFFE5E3D6, 0xFFE7E8DA,
        0xFFCACBBD, 0xFFB7B8AA, 0xFFBEC0B3, 0xFFB4B6A9, 0xFF989A8D, 0xFF82857A, 0xFF6A6D62, 0xFF545D58, 0xFF414841, 0xFF4F544D, 0xFF54574C, 0xFF6A6B5D, 0xFFA09F8D, 0xFF918E7B, 0xFF78735F, 0xFF5F644E, 0xFF585C4B, 0xFF4E5243, 0xFF34372C, 0xFF3E4136,
        0xFF424637, 0xFF454938, 0xFF474C36, 0xFF333728, 0xFF404435, 0xFF464A3C, 0xFF404436, 0xFF3C3F34, 0xFF43463B, 0xFF4B4E45, 0xFF4E5148, 0xFF41473B, 0xFF363C30, 0xFF33392F, 0xFF292E27, 0xFF090E08, 0xFF050A06, 0xFF050A06, 0xFF070B0A, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09,
        0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF070C08, 0xFF0C110D, 0xFF0D120E, 0xFF020703, 0xFF070C08, 0xFF0E130F, 0xFF040905, 0xFF050A06, 0xFF000203, 0xFF0D1112, 0xFF080C0D, 0xFF15191A, 0xFF090D0E, 0xFF020607, 0xFF000102, 0xFF75797A, 0xFF848E8F,
        0xFF01090B, 0xFF000807, 0xFF121818, 0xFF000402, 0xFF090E0A, 0xFF0A0C09, 0xFF0D0F0A, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0B100C, 0xFF0B100C,
        0xFF0B100C, 0xFF0B100C, 0xFF0C110D, 0xFF0B110D, 0xFF0B110D, 0xFF0B110D, 0xFF0B110D, 0xFF0C120E, 0xFF0C120E, 0xFF0C120E, 0xFF0C120E, 0xFF0D120E, 0xFF0D120E, 0xFF0E130F, 0xFF0E130F, 0xFF0F1410, 0xFF0F1410, 0xFF101511, 0xFF101511, 0xFF111612,
        0xFF111612, 0xFF111612, 0xFF111612, 0xFF111612, 0xFF111612, 0xFF111612, 0xFF111612, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F,
        0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF111612, 0xFF111612, 0xFF111612, 0xFF111612, 0xFF111612, 0xFF111612, 0xFF121713, 0xFF121713, 0xFF121713,
        0xFF121713, 0xFF121713, 0xFF131814, 0xFF151A16, 0xFF171C18, 0xFF191E1A, 0xFF191E1A, 0xFF1A1F1B, 0xFF1A1F1B, 0xFF1A1F1B, 0xFF191E1A, 0xFF171C18, 0xFF171C18, 0xFF181D19, 0xFF191E1A, 0xFF1C211D, 0xFF282D29, 0xFF323733, 0xFF3A3F3B, 0xFF464B47,
        0xFF4F5450, 0xFF535854, 0xFF5A5F5B, 0xFF444943, 0xFF40453F, 0xFF3D423C, 0xFF444943, 0xFF565B55, 0xFF626761, 0xFF5E635D, 0xFF535852, 0xFF4E534D, 0xFF525751, 0xFF454A44, 0xFF5E635D, 0xFF5D625C, 0xFF646963, 0xFF60655F, 0xFF5D625C, 0x71B5B5B5,
        0x00FEFEFE, 0x00FFFFFF, 0x00FCFCFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFB, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFDFF, 0x00FFFEFF, 0x00FFFEFB, 0x1DE7E4DD, 0xFFCBC9BD, 0xFFD6D5C1,
        0xFFCDCCBA, 0xFFD0D1BF, 0xFFD6D7C5, 0xFFC6C9B8, 0xFFAFB2A1, 0xFFA1A395, 0xFF999D8E, 0xFF80867A, 0xFF72756A, 0xFF787C6E, 0xFF66685A, 0xFF757867, 0xFF999A88, 0xFF80826D, 0xFF61604C, 0xFF595C49, 0xFF5F6251, 0xFF5C5E51, 0xFF3D3F34, 0xFF484A3F,
        0xFF434538, 0xFF3A3D2C, 0xFF333623, 0xFF35392A, 0xFF3D4132, 0xFF3E4233, 0xFF323628, 0xFF272B1D, 0xFF2A2D22, 0xFF34372C, 0xFF3C3F34, 0xFF464C40, 0xFF3F4539, 0xFF393F35, 0xFF292E27, 0xFF0C110B, 0xFF030804, 0xFF050A06, 0xFF070B0A, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09,
        0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF040905, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF000501, 0xFF020703, 0xFF292E2A, 0xFF4B5053, 0xFF0D1215, 0xFF050A0D, 0xFF060B0E, 0xFF94999C, 0xFF666B6E, 0xFF000104, 0xFF22272A, 0xFFC0CBCF,
        0xFF4A5557, 0xFF000304, 0xFF040D0C, 0xFF030907, 0xFF0F1410, 0xFF11130E, 0xFF0A0B06, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0B100C,
        0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0A100C, 0xFF0B110D, 0xFF0B110D, 0xFF0B110D, 0xFF0B110D, 0xFF0C120E, 0xFF0C120E, 0xFF0C120E, 0xFF0C110D, 0xFF0C110D, 0xFF0D120E, 0xFF0D120E, 0xFF0E130F, 0xFF0E130F, 0xFF0F1410, 0xFF0F1410, 0xFF10150F,
        0xFF10150F, 0xFF10150F, 0xFF10150F, 0xFF10150F, 0xFF10150F, 0xFF10150F, 0xFF10150F, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F,
        0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0F1410, 0xFF0E130F, 0xFF0E130F, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0E130F, 0xFF0F1410, 0xFF151A16,
        0xFF141915, 0xFF131814, 0xFF121713, 0xFF131814, 0xFF141915, 0xFF141915, 0xFF151A16, 0xFF161B17, 0xFF191E1A, 0xFF1B201C, 0xFF1C211D, 0xFF1C211D, 0xFF1C211D, 0xFF1C211D, 0xFF1D221E, 0xFF161B17, 0xFF1C211D, 0xFF1B201C, 0xFF1F2420, 0xFF1C211D,
        0xFF1A1F1B, 0xFF282D29, 0xFF2F3430, 0xFF363B35, 0xFF414640, 0xFF4D524C, 0xFF565B55, 0xFF5D625C, 0xFF616660, 0xFF5B605A, 0xFF515650, 0xFF50554F, 0xFF545953, 0xFF4A4F49, 0xFF616660, 0xFF5F645E, 0xFF676C66, 0xFF5F645E, 0xFF5D625C, 0x60BDBDBD,
        0x00FEFEFE, 0x00FFFFFF, 0x00FCFCFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFCFF, 0x00FDFBFF, 0x00FEFCFF, 0x00FFFEFD, 0x0DEEEBE6, 0x71C1BEB5, 0xFFCFCFB7,
        0xFFDBDBC3, 0xFFF2F1DC, 0xFFE6E8D2, 0xFFD7D9C3, 0xFFD5D8C3, 0xFFC9CCB7, 0xFFC2C5B0, 0xFFBCBBA7, 0xFFB1B39E, 0xFFB0B29D, 0xFF8F927D, 0xFF8C8F7A, 0xFF949983, 0xFF757A64, 0xFF464E39, 0xFF424534, 0xFF494B3E, 0xFF57594E, 0xFF4A4B45, 0xFF585953,
        0xFF4D4F44, 0xFF444639, 0xFF3C3F2E, 0xFF313526, 0xFF333728, 0xFF333728, 0xFF303425, 0xFF2C3022, 0xFF2C3022, 0xFF2E3126, 0xFF313429, 0xFF33392D, 0xFF33392D, 0xFF2E342A, 0xFF20251E, 0xFF0B100A, 0xFF030804, 0xFF070C08, 0xFF090D0C, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09,
        0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF0D120E, 0xFF010602, 0xFF050A06, 0xFF080D09, 0xFF0A0F0B, 0xFF010602, 0xFF0F1410, 0xFF505551, 0xFFF6FFFF, 0xFFBBC5C7, 0xFF313B3D, 0xFF0C1618, 0xFF687274, 0xFFD6E0E2, 0xFF646E70, 0xFF000305, 0xFF919EA4,
        0xFFA8B6B9, 0xFF0E191B, 0xFF020A0C, 0xFF0E1412, 0xFF040905, 0xFF11130E, 0xFF0B0C06, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0B100C,
        0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0A100C, 0xFF0A100C, 0xFF0B110D, 0xFF0B110D, 0xFF0B110D, 0xFF0B110D, 0xFF0B110D, 0xFF0C120E, 0xFF0B100C, 0xFF0B100C, 0xFF0C110D, 0xFF0C110D, 0xFF0D120E, 0xFF0D120E, 0xFF0E130F, 0xFF0E130F, 0xFF0F140E,
        0xFF0F140E, 0xFF0F140E, 0xFF0F140E, 0xFF0F140E, 0xFF0F140E, 0xFF0F140E, 0xFF0F140E, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F,
        0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0F1410, 0xFF0F1410, 0xFF0F1410, 0xFF0B100C, 0xFF0B100C, 0xFF0C110D, 0xFF0C110D, 0xFF0D120E, 0xFF0E130F, 0xFF0F1410, 0xFF0F1410, 0xFF0C110D,
        0xFF0D120E, 0xFF0F1410, 0xFF111612, 0xFF111612, 0xFF101511, 0xFF0F1410, 0xFF0F1410, 0xFF111612, 0xFF121713, 0xFF131814, 0xFF131814, 0xFF141915, 0xFF171C18, 0xFF1A1F1B, 0xFF1D221E, 0xFF1D221E, 0xFF222723, 0xFF1B201C, 0xFF1D221E, 0xFF161B17,
        0xFF232824, 0xFF3E433F, 0xFF292E2A, 0xFF3F443E, 0xFF4D524C, 0xFF5A5F59, 0xFF5D625C, 0xFF5D625C, 0xFF5D625C, 0xFF575C56, 0xFF50554F, 0xFF515650, 0xFF555A54, 0xFF4C514B, 0xFF626761, 0xFF5F645E, 0xFF676C66, 0xFF5F645E, 0xFF5F645E, 0x54C3C3C3,
        0x00FEFEFE, 0x00FFFFFF, 0x00FCFCFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FEFEFE, 0x00FBFBFB, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FEFEFF, 0x00FCFBFF, 0x00FFFDFF, 0x00FFFDFF, 0x00FFFDFE, 0x00FEFBF6, 0x3FD8D5CE, 0x75D0D0B4,
        0xFFE6E6CC, 0xFFFEFEE4, 0xFFE0E3C8, 0xFFD5D7BF, 0xFFE9EBD3, 0xFFDADEC7, 0xFFCBCFB8, 0xFFEAE6CB, 0xFFE4E2C9, 0xFFE6E6CC, 0xFFC3C5AD, 0xFFB2B7A0, 0xFFA6AE97, 0xFF838C77, 0xFF4B5741, 0xFF505245, 0xFF3C3E33, 0xFF3D3E38, 0xFF2C2D28, 0xFF363732,
        0xFF32332D, 0xFF43453A, 0xFF4E5043, 0xFF444837, 0xFF3E4233, 0xFF383C2D, 0xFF393D2E, 0xFF3A3E30, 0xFF383C2E, 0xFF32352A, 0xFF2E3126, 0xFF2A3024, 0xFF2E3428, 0xFF2A3026, 0xFF1D221B, 0xFF0C110B, 0xFF050A06, 0xFF080D09, 0xFF080C0B, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09,
        0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF050A06, 0xFF090E0A, 0xFF0C110D, 0xFF010602, 0xFF0A0F0B, 0xFF0B100C, 0xFF000300, 0xFF141915, 0xFF627071, 0xFFDBE9EA, 0xFFE9F7F8, 0xFF6D7B7C, 0xFF061415, 0xFF4C5A5B, 0xFFD5E3E4, 0xFF889697, 0xFFB2C1C6,
        0xFFE7F4FA, 0xFF2E393D, 0xFF000809, 0xFF070D0B, 0xFF090E0A, 0xFF0C0D08, 0xFF0E0F09, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B,
        0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0A100C, 0xFF0A100C, 0xFF0A100C, 0xFF0B110D, 0xFF0B110D, 0xFF0B110D, 0xFF0B110D, 0xFF0B110D, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0C110D, 0xFF0C110D, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0F140E,
        0xFF0F140E, 0xFF0F140E, 0xFF0F140E, 0xFF0F140E, 0xFF0F140E, 0xFF0F140E, 0xFF0F140E, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0E1410, 0xFF0E1410, 0xFF0E1410, 0xFF0E1410, 0xFF0E1410,
        0xFF0E1410, 0xFF0E1410, 0xFF0E1410, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0F1410, 0xFF0F1410, 0xFF0F1410, 0xFF0F1410, 0xFF0F1410, 0xFF0E130F, 0xFF0E130F, 0xFF0F1410, 0xFF0F1410, 0xFF0F1410, 0xFF0F1410, 0xFF0F1410, 0xFF0E130F, 0xFF0F1410,
        0xFF0F1410, 0xFF101511, 0xFF0F1410, 0xFF0E130F, 0xFF0D120E, 0xFF0D120E, 0xFF0E130F, 0xFF0E130F, 0xFF0F1410, 0xFF111612, 0xFF111612, 0xFF111612, 0xFF111612, 0xFF131814, 0xFF151A16, 0xFF121713, 0xFF1C211D, 0xFF181D19, 0xFF1B201C, 0xFF1C211D,
        0xFF474C48, 0xFF6C716D, 0xFF323733, 0xFF515650, 0xFF575C56, 0xFF5B605A, 0xFF5A5F59, 0xFF5C615B, 0xFF5E635D, 0xFF585D57, 0xFF4E534D, 0xFF4F544E, 0xFF525751, 0xFF494E48, 0xFF5F645E, 0xFF5C615B, 0xFF676C66, 0xFF5F645E, 0xFF616660, 0x4FC6C6C6,
        0x00FEFEFE, 0x00FFFFFF, 0x00FCFCFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFCFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x19E4E5E0,
        0x1DE2E3DD, 0x2EDCDDD5, 0x43D5D7CC, 0xFFD2D4C6, 0xFFD4D7C4, 0xFFD5D8C3, 0xFFD3D6C1, 0xFFF6F0CC, 0xFFFFF9D7, 0xFFFAF7D8, 0xFFD9D7BE, 0xFFBABCA7, 0xFFBABCAE, 0xFFAEB1A6, 0xFF8B9187, 0xFF7B7D72, 0xFF66685D, 0xFF4E5043, 0xFF3E4033, 0xFF313325,
        0xFF2E3022, 0xFF3F4133, 0xFF565948, 0xFF3B3E2B, 0xFF2F3221, 0xFF393B2D, 0xFF3C3E31, 0xFF4C4E43, 0xFF5D5E56, 0xFF44453F, 0xFF2E2F29, 0xFF343226, 0xFF36382D, 0xFF252B21, 0xFF1B251C, 0xFF050F07, 0xFF030907, 0xFF090D0E, 0xFF08090B, 0xFF0F110C,
        0xFF0A0C07, 0xFF040601, 0xFF080A07, 0xFF0D0F0C, 0xFF0B0D0C, 0xFF000100, 0xFF080A09, 0xFF090B0A, 0xFF0C0E0D, 0xFF060805, 0xFF0D0F0C, 0xFF060805, 0xFF10120D, 0xFF080A05, 0xFF070904, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF000606, 0xFF333939, 0xFFBDC3C3, 0xFFECF4F6, 0xFFABB3B5, 0xFF1A2426, 0xFF667072, 0xFFF4FEFF, 0xFFE4F2FD,
        0xFFA8B7BE, 0xFF111F20, 0xFF07120E, 0xFF09120D, 0xFF090F0B, 0xFF0A0E0D, 0xFF080A09, 0xFF080C0B, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D,
        0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF090F0B, 0xFF090F0B, 0xFF0A100C, 0xFF0A100C, 0xFF0A100C, 0xFF0A100C, 0xFF0B110D, 0xFF0B110D, 0xFF0D120C, 0xFF0D120C, 0xFF0D120C, 0xFF0D120C, 0xFF0D120C, 0xFF0E130D, 0xFF0E130D, 0xFF0E130D, 0xFF0F140E,
        0xFF0F140E, 0xFF0F140E, 0xFF0F140E, 0xFF10150F, 0xFF10150F, 0xFF10150F, 0xFF10150F, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0E150E, 0xFF0E150E, 0xFF0D140D, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E,
        0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F,
        0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0C120E, 0xFF0C120E, 0xFF0C120E, 0xFF0C120E, 0xFF0C120E, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF131716, 0xFF131716, 0xFF1A1E1D, 0xFF101511, 0xFF161B17,
        0xFF5E635D, 0xFF686D67, 0xFF393E38, 0xFF585F58, 0xFF555C55, 0xFF545B54, 0xFF565D56, 0xFF596059, 0xFF596059, 0xFF565D56, 0xFF525952, 0xFF535A53, 0xFF4C534C, 0xFF4A514A, 0xFF5B625B, 0xFF656C65, 0xFF5E655E, 0xFF5E655E, 0xFF616861, 0x2ED5D5D5,
        0x00FFFFFF, 0x00FFFFFF, 0x00FCFCFC, 0x00FFFFFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FCFCFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFCFA,
        0x00FBFCF7, 0x00F3F4EE, 0x19E6E7DF, 0x36DADCD1, 0x4BD3D5C7, 0xFFCED1C0, 0xFFC9CCBB, 0xFFF1EACD, 0xFFF9F4D7, 0xFFF7F3DA, 0xFFDAD8C3, 0xFFBCBDAB, 0xFFBABCAE, 0xFFB3B7A9, 0xFF989E92, 0xFF909285, 0xFF8D8F82, 0xFF8B8D7F, 0xFF838577, 0xFF6F7261,
        0xFF5B5E4D, 0xFF5C5F4E, 0xFF686B58, 0xFF515443, 0xFF414433, 0xFF414335, 0xFF343629, 0xFF33352A, 0xFF414338, 0xFF3B3C34, 0xFF383931, 0xFF363428, 0xFF36382B, 0xFF252B1F, 0xFF1F2920, 0xFF08120A, 0xFF030907, 0xFF060A0B, 0xFF0C0D0F, 0xFF040903,
        0xFF040903, 0xFF090E08, 0xFF040905, 0xFF000501, 0xFF080C0B, 0xFF0D1110, 0xFF070B0A, 0xFF080C0B, 0xFF090D0C, 0xFF070B0A, 0xFF0D120E, 0xFF050A06, 0xFF050A04, 0xFF050A04, 0xFF0E130D, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF070B0C, 0xFF020808, 0xFF151B1B, 0xFF7D8285, 0xFFF0F8FA, 0xFFCFD7DA, 0xFF7C8688, 0xFFA8B2B4, 0xFFDCEAF5,
        0xFF5F6E75, 0xFF000405, 0xFF010C08, 0xFF0A130E, 0xFF050A06, 0xFF060807, 0xFF0B0D0C, 0xFF080C0B, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D,
        0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF090F0B, 0xFF090F0B, 0xFF090F0B, 0xFF0A100C, 0xFF0A100C, 0xFF0A100C, 0xFF0A100C, 0xFF0B110D, 0xFF0C110B, 0xFF0D120C, 0xFF0D120C, 0xFF0D120C, 0xFF0D120C, 0xFF0D120C, 0xFF0E130D, 0xFF0E130D, 0xFF0F140E,
        0xFF0F140E, 0xFF0F140E, 0xFF0F140E, 0xFF0F140E, 0xFF10150F, 0xFF10150F, 0xFF10150F, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0E150E, 0xFF0E150E, 0xFF0D140D, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E,
        0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F,
        0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0C120E, 0xFF0C120E, 0xFF0C120E, 0xFF0C120E, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF101413, 0xFF0F1410, 0xFF0D120E,
        0xFF2F342E, 0xFF3E433D, 0xFF424741, 0xFF585F58, 0xFF5B625B, 0xFF5B625B, 0xFF596059, 0xFF596059, 0xFF596059, 0xFF555C55, 0xFF505750, 0xFF515851, 0xFF4A514A, 0xFF4C534C, 0xFF5C635C, 0xFF646B64, 0xFF5E655E, 0xFF5F665F, 0xFF656C65, 0x2AD8D8D8,
        0x00FFFFFF, 0x00FFFFFF, 0x00FCFCFC, 0x00FFFFFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFF,
        0x00FFFFFF, 0x00FFFFFD, 0x00FFFFFB, 0x00FDFEF8, 0x08EFF0E8, 0x3FD6D8CD, 0x6DC0C2B7, 0xFFE0DBC8, 0xFFEBE6D3, 0xFFEFECDB, 0xFFDEDDCB, 0xFFC4C5B5, 0xFFBCBFAE, 0xFFB4B8A7, 0xFFA0A495, 0xFF8D8F81, 0xFF898B7D, 0xFF8D907F, 0xFF989B8A, 0xFF9B9E8B,
        0xFF939683, 0xFF8C8F7C, 0xFF8B8E79, 0xFF7D806F, 0xFF6C6F5E, 0xFF66685A, 0xFF4E5042, 0xFF383A2D, 0xFF323429, 0xFF292B20, 0xFF2A2C21, 0xFF302E22, 0xFF2F3124, 0xFF20261A, 0xFF1F291E, 0xFF0A140C, 0xFF060C08, 0xFF030706, 0xFF090A0C, 0xFF0B100A,
        0xFF010600, 0xFF070C06, 0xFF090E0A, 0xFF080D09, 0xFF000403, 0xFF070B0A, 0xFF040807, 0xFF060A0B, 0xFF070B0A, 0xFF060A09, 0xFF020605, 0xFF050A06, 0xFF090E0A, 0xFF070C06, 0xFF010600, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF0A0E0D, 0xFF030708, 0xFF010707, 0xFF060C0C, 0xFF3C4446, 0xFFD0D8DA, 0xFFF1F9FC, 0xFFACB6B8, 0xFFB3C1CC,
        0xFFAFBCC4, 0xFF394446, 0xFF000400, 0xFF151B17, 0xFF0D120E, 0xFF0D0F0E, 0xFF0B0D0C, 0xFF080C0B, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D,
        0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF090F0B, 0xFF090F0B, 0xFF090F0B, 0xFF0A100C, 0xFF0A100C, 0xFF0A100C, 0xFF0A100C, 0xFF0A100C, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0D120C, 0xFF0D120C, 0xFF0D120C, 0xFF0D120C, 0xFF0E130D, 0xFF0E130D,
        0xFF0F140E, 0xFF0F140E, 0xFF0F140E, 0xFF0F140E, 0xFF10150F, 0xFF10150F, 0xFF10150F, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0E150E, 0xFF0E150E, 0xFF0D140D, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E,
        0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F,
        0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0C120E, 0xFF0C120E, 0xFF0C120E, 0xFF0C120E, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0A0E0D, 0xFF0F1312, 0xFF0E1211, 0xFF101511, 0xFF0E130F,
        0xFF111610, 0xFF232822, 0xFF525751, 0xFF5B625B, 0xFF5A615A, 0xFF575E57, 0xFF545B54, 0xFF565D56, 0xFF596059, 0xFF585F58, 0xFF535A53, 0xFF565D56, 0xFF4C534C, 0xFF525952, 0xFF5F665F, 0xFF646B64, 0xFF5F665F, 0xFF5F665F, 0xFF697069, 0x1DDDDDDD,
        0x00FFFFFF, 0x00FFFFFF, 0x00FCFCFC, 0x00FFFFFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF,
        0x00FEFEFF, 0x00FDFDFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFD, 0x00F1F2ED, 0x1DE1E2DD, 0x4BD4D0C7, 0x47D7D3CA, 0x47D6D3CA, 0xFFC9C9BD, 0xFFBBBCAE, 0xFFB9BBAD, 0xFFB9BCAB, 0xFFB0B4A3, 0xFFA4A796, 0xFF9FA291, 0xFF9C9F8E, 0xFF9B9E8D, 0xFF969986,
        0xFF909380, 0xFF91947F, 0xFF959883, 0xFF999B8D, 0xFF8C8E80, 0xFF8A8C7E, 0xFF7E8072, 0xFF696B5D, 0xFF57594C, 0xFF434538, 0xFF3C3E31, 0xFF3B392C, 0xFF37392C, 0xFF252B1F, 0xFF1D271C, 0xFF030D04, 0xFF080E0A, 0xFF060A09, 0xFF0A0C0B, 0xFF060B05,
        0xFF0A0F09, 0xFF0A0F09, 0xFF000300, 0xFF070C08, 0xFF0A0E0D, 0xFF0E1211, 0xFF060A09, 0xFF060A0B, 0xFF080C0D, 0xFF0D1110, 0xFF090D0C, 0xFF0C100F, 0xFF080D09, 0xFF090E0A, 0xFF060B07, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060A09, 0xFF070B0A, 0xFF191D1C, 0xFF050B0B, 0xFF000202, 0xFF363E40, 0xFFBDC5C7, 0xFFE8F0F2, 0xFFD0DCE8,
        0xFFEAF7FF, 0xFFA7B2B4, 0xFF19221F, 0xFF030905, 0xFF000501, 0xFF131514, 0xFF0A0A0A, 0xFF080C0B, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D,
        0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF090F0B, 0xFF090F0B, 0xFF090F0B, 0xFF090F0B, 0xFF0A100C, 0xFF0A100C, 0xFF0A100C, 0xFF0A100C, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0D120C, 0xFF0D120C, 0xFF0D120C, 0xFF0D120C, 0xFF0E130D,
        0xFF0E130D, 0xFF0E130D, 0xFF0F140E, 0xFF0F140E, 0xFF0F140E, 0xFF0F140E, 0xFF10150F, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0E150E, 0xFF0E150E, 0xFF0D140D, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E,
        0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F,
        0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0C120E, 0xFF0C120E, 0xFF0C120E, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0E1410, 0xFF0B0F0E, 0xFF151918, 0xFF111514, 0xFF0D120E, 0xFF121713,
        0xFF131812, 0xFF282D27, 0xFF585D57, 0xFF5D645D, 0xFF565D56, 0xFF505750, 0xFF525952, 0xFF575E57, 0xFF585F58, 0xFF545B54, 0xFF505750, 0xFF525952, 0xFF424942, 0xFF4D544D, 0xFF5A615A, 0xFF606760, 0xFF616861, 0xFF606760, 0xFF6E756E, 0x11E3E3E3,
        0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFF, 0x00FBFBFD, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFDFB, 0x00F7F6F2, 0x08ECEBE7, 0x2ADDDCD7, 0x58CCCCC2, 0x71C1C3B6, 0x86B7B9AB, 0xFFADAFA1, 0xFFB2B4A6, 0xFFABAD9F, 0xFFA2A496, 0xFF9B9E8D, 0xFF979A89,
        0xFF949786, 0xFF949784, 0xFF969986, 0xFF9D9F91, 0xFF939587, 0xFF97998B, 0xFF989A8C, 0xFF8F9183, 0xFF838675, 0xFF757867, 0xFF6E7160, 0xFF7B796C, 0xFF77796B, 0xFF5E6456, 0xFF3C463B, 0xFF030D04, 0xFF080F08, 0xFF090E0A, 0xFF0D0F0E, 0xFF060D06,
        0xFF030A03, 0xFF0A110A, 0xFF070D09, 0xFF040A06, 0xFF050B09, 0xFF030907, 0xFF050B09, 0xFF050B0B, 0xFF000303, 0xFF000202, 0xFF010705, 0xFF080E0C, 0xFF050B09, 0xFF060C08, 0xFF070D09, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF121411, 0xFF0D0F0E, 0xFF000403, 0xFF050908, 0xFF181E1E, 0xFF000202, 0xFF33383B, 0xFFBFC7C9, 0xFF95A1AD,
        0xFF9FA9B2, 0xFFBCC6C8, 0xFF28312E, 0xFF000300, 0xFF181A17, 0xFF060606, 0xFF121212, 0xFF080C0B, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D,
        0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF080E0A, 0xFF090F0B, 0xFF090F0B, 0xFF090F0B, 0xFF090F0B, 0xFF0A100C, 0xFF0A100C, 0xFF0A100C, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0D120C, 0xFF0D120C, 0xFF0D120C, 0xFF0D120C, 0xFF0E130D,
        0xFF0E130D, 0xFF0E130D, 0xFF0E130D, 0xFF0F140E, 0xFF0F140E, 0xFF0F140E, 0xFF0F140E, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0E150E, 0xFF0E150E, 0xFF0D140D, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E,
        0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F,
        0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0C120E, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0E1410, 0xFF0E1410, 0xFF0E1410, 0xFF0A0E0D, 0xFF121615, 0xFF0E1211, 0xFF090E0A, 0xFF0F1410,
        0xFF171C16, 0xFF333832, 0xFF545953, 0xFF5C635C, 0xFF555C55, 0xFF535A53, 0xFF5A615A, 0xFF5B625B, 0xFF545B54, 0xFF4E554E, 0xFF4F564F, 0xFF5D645D, 0xFF464D46, 0xFF545B54, 0xFF5F665F, 0xFF626962, 0xFF656C65, 0xFF606760, 0xFF707770, 0x04EAEAEA,
        0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FAFAFA,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FDFDFB, 0x00FEFEFC, 0x00FCFDF8, 0x00F7F8F2, 0x08EEEDE8, 0x21E4E4DC, 0x54CBCCC4, 0x71BEC0B5, 0x92AFB1A6, 0xA3A7A99E, 0xFFA7A99C,
        0xFFA4A699, 0xFF9B9D8F, 0xFF919385, 0xFF96988B, 0xFF909285, 0xFF949688, 0xFF97998B, 0xFF929584, 0xFF8D907D, 0xFF8A8D7A, 0xFF878A77, 0xFF9F9D8E, 0xFFA3A597, 0xFF949A8C, 0xFF657062, 0xFF09130A, 0xFF060D06, 0xFF070C08, 0xFF090B08, 0xFF0C130C,
        0xFF000600, 0xFF060D06, 0xFF040A06, 0xFF000200, 0xFF151B19, 0xFF060C0A, 0xFF010705, 0xFF282D30, 0xFF393E41, 0xFF2D3333, 0xFF121818, 0xFF000200, 0xFF090F0D, 0xFF080E0C, 0xFF000501, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF050704, 0xFF0C0E0B, 0xFF111312, 0xFF080C0B, 0xFF060A09, 0xFF0B1111, 0xFF000606, 0xFF434949, 0xFFC2CCD8,
        0xFF29333C, 0xFF162022, 0xFF060F0C, 0xFF000200, 0xFF121411, 0xFF0D0D0D, 0xFF080607, 0xFF080C0B, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D,
        0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF090F0B, 0xFF090F0B, 0xFF090F0B, 0xFF090F0B, 0xFF0A100C, 0xFF0B100A, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0D120C, 0xFF0D120C, 0xFF0D120C, 0xFF0E130D,
        0xFF0E130D, 0xFF0E130D, 0xFF0E130D, 0xFF0E130D, 0xFF0F140E, 0xFF0F140E, 0xFF0F140E, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0E150E, 0xFF0E150E, 0xFF0D140D, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E,
        0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F,
        0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0E1410, 0xFF0E1410, 0xFF0E1410, 0xFF0E1410, 0xFF0D1110, 0xFF0D1110, 0xFF0D1110, 0xFF0D120E, 0xFF0E130F,
        0xFF131812, 0xFF393E38, 0xFF565B55, 0xFF5B625B, 0xFF555C55, 0xFF555C55, 0xFF596059, 0xFF565D56, 0xFF515851, 0xFF525952, 0xFF5A615A, 0xFF596059, 0xFF414841, 0xFF585F58, 0xFF636A63, 0xFF636A63, 0xFF676E67, 0xFF606760, 0xFB747B74, 0x00F0F0F0,
        0x00FEFEFE, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFB,
        0x00FFFFFB, 0x00FEFEFC, 0x00FDFDFB, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFCFE, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFFFB, 0x00FEFFFA, 0x00F2F3ED, 0x2ADDDED8, 0x54C9CAC4,
        0x79B8B9B1, 0x96ABACA4, 0xABA1A29A, 0xFFA2A497, 0xFF9B9D90, 0xFF989A8C, 0xFF969988, 0xFF909380, 0xFF8E917E, 0xFF90937E, 0xFF8D907B, 0xFF8E8C7D, 0xFF939685, 0xFF959C8C, 0xFF6F7A6C, 0xFF071106, 0xFF040B04, 0xFF070C08, 0xFF080A07, 0xFF000901,
        0xFF0B150D, 0xFF030D05, 0xFF000702, 0xFF222B26, 0xFF747D7A, 0xFF2F3835, 0xFF000603, 0xFF565E60, 0xFF90989A, 0xFF868E90, 0xFF464F4E, 0xFF000302, 0xFF07100D, 0xFF09120F, 0xFF060F0C, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF0F0F0D, 0xFF060805, 0xFF0D0F0C, 0xFF131514, 0xFF040807, 0xFF090D0E, 0xFF333939, 0xFF707676, 0xFFDEE8F4,
        0xFF88929B, 0xFF050F11, 0xFF030907, 0xFF121713, 0xFF080806, 0xFF0D0B0C, 0xFF0F0D0E, 0xFF080C0B, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D,
        0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF090F0B, 0xFF090F0B, 0xFF090F0B, 0xFF090F0B, 0xFF090F0B, 0xFF0B100A, 0xFF0B100A, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0D120C, 0xFF0D120C,
        0xFF0E130D, 0xFF0E130D, 0xFF0E130D, 0xFF0E130D, 0xFF0F140E, 0xFF0F140E, 0xFF0F140E, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0E150E, 0xFF0E150E, 0xFF0D140D, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E,
        0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F,
        0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0E1410, 0xFF0E1410, 0xFF0E1410, 0xFF0E1410, 0xFF121615, 0xFF0E1211, 0xFF0F1312, 0xFF0F1410, 0xFF111612,
        0xFF141913, 0xFF3F443E, 0xFF5D625C, 0xFF5A615A, 0xFF575E57, 0xFF535A53, 0xFF515851, 0xFF525952, 0xFF535A53, 0xFF525952, 0xFF4F564F, 0xFF373E37, 0xFF262D26, 0xFF4F564F, 0xFF5F665F, 0xFF5E655E, 0xFF656C65, 0xFF626962, 0xE67E857E, 0x00F4F4F4,
        0x00FDFDFD, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFFF9,
        0x00FFFFFA, 0x00FFFFFB, 0x00FFFFFB, 0x00FCFCFA, 0x00FEFEFE, 0x00FEFEFE, 0x00FEFEFE, 0x00FAFFFB, 0x00FCFFFD, 0x00FEFFFF, 0x00FDFFFE, 0x00FEFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFDFF, 0x00FDFDFD, 0x00FFFFFD, 0x00FFFFFD, 0x00FFFFFD, 0x00FFFFFB,
        0x00FFFFFB, 0x00F1F2EC, 0x1DE3E4DE, 0x54CCCEC3, 0x79BDBFB2, 0x9FABAD9F, 0xBC9FA291, 0xFF969986, 0xFF949782, 0xFF949881, 0xFF8D917A, 0xFF9C9A8B, 0xFF949786, 0xFF949B8B, 0xFF707B6D, 0xFF050F04, 0xFF040B03, 0xFF0A0F09, 0xFF0A0C09, 0xFF020C04,
        0xFF050F07, 0xFF000901, 0xFF404944, 0xFFA2ABA6, 0xFFCFD8D5, 0xFF2E3734, 0xFF000300, 0xFF586062, 0xFF838B8D, 0xFF747C7E, 0xFF4C5554, 0xFF020B0A, 0xFF0A1312, 0xFF010A07, 0xFF050E0B, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF0B0C07, 0xFF0A0A08, 0xFF0D0F0C, 0xFF080A07, 0xFF0A0E0D, 0xFF0F1312, 0xFFB5B9BA, 0xFFF3F9F9, 0xFFE9F3FF,
        0xFF9EA7B0, 0xFF01090C, 0xFF050B09, 0xFF0D120E, 0xFF0A0A08, 0xFF0F0D0E, 0xFF0C0A0B, 0xFF080C0B, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D,
        0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF090F0B, 0xFF090F0B, 0xFF090F0B, 0xFF090F0B, 0xFF0B100A, 0xFF0B100A, 0xFF0B100A, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0D120C,
        0xFF0D120C, 0xFF0E130D, 0xFF0E130D, 0xFF0E130D, 0xFF0E130D, 0xFF0F140E, 0xFF0F140E, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0E150E, 0xFF0E150E, 0xFF0D140D, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E,
        0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F,
        0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0E1410, 0xFF0E1410, 0xFF0E1410, 0xFF0E1410, 0xFF0E1410, 0xFF111514, 0xFF111514, 0xFF0E1211, 0xFF0B100C, 0xFF121713,
        0xFF181D17, 0xFF454A44, 0xFF5F645E, 0xFF5B625B, 0xFF596059, 0xFF545B54, 0xFF525952, 0xFF575E57, 0xFF585F58, 0xFF444B44, 0xFF2B322B, 0xFF343B34, 0xFF282F28, 0xFF5A615A, 0xFF6A716A, 0xFF626962, 0xFF656C65, 0xFF626962, 0xDE828982, 0x00F7F7F7,
        0x00FDFDFD, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F7F7F7, 0x08E7E7E7, 0x2ED6D6D6, 0x54C3C3C3, 0x7DAFAFAF, 0x9F9F9F9F, 0xC4989C8D, 0xFF919586, 0xFF95998B, 0xFF777D71, 0xFF0E140A, 0xFF080D06, 0xFF091009, 0xFF060C08, 0xFF060702,
        0xFF060604, 0xFF3A3E3D, 0xFFD1D7D7, 0xFFC6D0D2, 0xFFBFCACE, 0xFF313E46, 0xFF00040B, 0xFF303634, 0xFF687170, 0xFF848E8F, 0xFF4C5759, 0xFF00080A, 0xFF000504, 0xFF050A06, 0xFF0B0C07, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF0D1009, 0xFF050A03, 0xFF0A0F0B, 0xFF090D0C, 0xFF050A0D, 0xFF000206, 0xFF8B9298, 0xFFEAF1F9, 0xFF657479,
        0xFF121D21, 0xFF07100F, 0xFF090E0A, 0xFF0A0C07, 0xFF080A07, 0xFF060A09, 0xFF080E0E, 0xFF070D0B, 0xFF070D0B, 0xFF080C0B, 0xFF090D0C, 0xFF090D0C, 0xFF0B0D0C, 0xFF0B0D0C, 0xFF0B0D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D,
        0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0C110D, 0xFF0C110D, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B,
        0xFF0C110B, 0xFF0C110B, 0xFF0D120C, 0xFF0D120C, 0xFF0E130D, 0xFF0E130D, 0xFF0F140E, 0xFF0E150D, 0xFF121911, 0xFF0F160E, 0xFF141B14, 0xFF0D140D, 0xFF060C08, 0xFF111713, 0xFF0B110D, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F,
        0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F,
        0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0F1410, 0xFF0F1410, 0xFF0F1410, 0xFF0F1410, 0xFF0F1410, 0xFF0D120E, 0xFF0F1410, 0xFF0B100C, 0xFF131812, 0xFF0E130D,
        0xFF1B201A, 0xFF4D524B, 0xFF60655E, 0xFF59635B, 0xFF4E5850, 0xFF566058, 0xFF5B625B, 0xFF575E57, 0xFF353A34, 0xFF30352F, 0xFF434842, 0xFF595E58, 0xFF353A34, 0xFF565B55, 0xFF646963, 0xFF5E635D, 0xFF666B65, 0xFF656A64, 0xE280857F, 0x00F9FEF8,
        0x00FBFFFA, 0x00FCFFFB, 0x00FCFFFD, 0x00FCFFFD, 0x00FCFFFD, 0x00FCFFFF, 0x00FCFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFCFC, 0x00FBFBFB, 0x00F8F8F8, 0x00F2F2F2, 0x00EBEBEB, 0x4BCFD2C7, 0x71BEC1B6, 0x82B4B7AE, 0xD5878C85, 0xFF0C110B, 0xFF050A06, 0xFF0B110D, 0xFF070D0B, 0xFF0D0E09,
        0xFF060805, 0xFF1D2120, 0xFF838989, 0xFFD1D9DC, 0xFFC3CED2, 0xFF222F35, 0xFF000B13, 0xFF5D6361, 0xFF525B5A, 0xFF8A9495, 0xFF384345, 0xFF000406, 0xFF091211, 0xFF0A0F0B, 0xFF0C0D08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF050801, 0xFF090E08, 0xFF040905, 0xFF0B0F0E, 0xFF23282B, 0xFF4A4F53, 0xFF42494F, 0xFFC8CFD7, 0xFF5F6E73,
        0xFF000307, 0xFF081110, 0xFF0A0F0B, 0xFF090B06, 0xFF070906, 0xFF060A09, 0xFF080E0E, 0xFF070D0B, 0xFF070D0B, 0xFF080C0B, 0xFF090D0C, 0xFF090D0C, 0xFF0B0D0C, 0xFF0B0D0C, 0xFF0B0D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D,
        0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B100C, 0xFF0C110D, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B,
        0xFF0C110B, 0xFF0C110B, 0xFF0D120C, 0xFF0D120C, 0xFF0E130D, 0xFF0E130D, 0xFF0E130D, 0xFF0C130B, 0xFF0E150D, 0xFF060D05, 0xFF141B14, 0xFF131A13, 0xFF131915, 0xFF0C120E, 0xFF0D130F, 0xFF0C130C, 0xFF0C130C, 0xFF0C130C, 0xFF0E150E, 0xFF101710,
        0xFF101710, 0xFF0E150E, 0xFF0C130C, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F,
        0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0F1410, 0xFF0F1410, 0xFF0F1410, 0xFF0F1410, 0xFF0D120E, 0xFF0E130F, 0xFF0C110D, 0xFF131812, 0xFF0F140E,
        0xFF1D221C, 0xFF4E534C, 0xFF61665F, 0xFF576159, 0xFF5B655D, 0xFF5F665F, 0xFF464D46, 0xFF343B34, 0xFF383D37, 0xFF484D47, 0xFF3D423C, 0xFF2F342E, 0xFF2F342E, 0xFF595E58, 0xFF636862, 0xFF636862, 0xFF666B65, 0xFF626761, 0xD1888D87, 0x00FCFEF9,
        0x00FDFFFA, 0x00FDFFFA, 0x00FDFFFC, 0x00FBFDFA, 0x00FBFDFC, 0x00FCFEFD, 0x00FEFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFFFB, 0x00FAFCF7, 0x00F4F6F1, 0x6DB8BDB9, 0xFF191D1C, 0xFF010504, 0xFF0A1010, 0xFF080E0E, 0xFF0A0A08,
        0xFF0C0E0B, 0xFF000302, 0xFF232929, 0xFF9AA2A4, 0xFFD2DCDE, 0xFF354044, 0xFF00060A, 0xFF474D4B, 0xFF889190, 0xFF545E5F, 0xFF5B6668, 0xFF010B0D, 0xFF081110, 0xFF060B07, 0xFF070803, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF0F110C, 0xFF0A0F09, 0xFF060B07, 0xFF000403, 0xFF424848, 0xFFF7FCFF, 0xFFB3BBBE, 0xFFA3AAB0, 0xFFB8C7CC,
        0xFF313C40, 0xFF000605, 0xFF0A0F0B, 0xFF0A0C07, 0xFF0C0E0B, 0xFF131716, 0xFF050B0B, 0xFF070D0B, 0xFF070D0B, 0xFF080C0B, 0xFF090D0C, 0xFF090D0C, 0xFF0B0D0C, 0xFF0B0D0C, 0xFF0B0D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D,
        0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B100C, 0xFF0C110D, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B,
        0xFF0C110B, 0xFF0C110B, 0xFF0D120C, 0xFF0D120C, 0xFF0E130D, 0xFF0E130D, 0xFF0E130D, 0xFF10170F, 0xFF0C130B, 0xFF070E07, 0xFF101710, 0xFF0B110D, 0xFF131915, 0xFF0B110D, 0xFF111715, 0xFF111811, 0xFF101710, 0xFF0F160F, 0xFF0E150E, 0xFF0D140D,
        0xFF0E150E, 0xFF101710, 0xFF111811, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F,
        0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0F1410, 0xFF0F1410, 0xFF0F1410, 0xFF0F1410, 0xFF0C110D, 0xFF0E130F, 0xFF0D120E, 0xFF121711, 0xFF0F140E,
        0xFF222721, 0xFF525750, 0xFF656A63, 0xFF576159, 0xFF5D645D, 0xFF444B44, 0xFF2B322B, 0xFF3F443E, 0xFF848983, 0xFFC2C4BF, 0xFFBBBDB8, 0xFFA4A9A3, 0xFF858A84, 0xFF757A74, 0xFF656A64, 0xFF656A64, 0xFF5C615B, 0xFF5B605A, 0xAB9BA09A, 0x00FFFFFB,
        0x00FFFFFB, 0x00FFFFFD, 0x00FFFFFD, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FCFCFC, 0x00FCFCFC, 0x00FCFCFC, 0x00FCFCFC, 0x00FCFCFC, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x3BCFD3D4, 0xFF1E2223, 0xFF000304, 0xFF0A1010, 0xFF070C0F, 0xFF050704,
        0xFF0F1110, 0xFF000100, 0xFF000505, 0xFF303636, 0xFFAAB2B4, 0xFF474F52, 0xFF000305, 0xFF3C4240, 0xFF5D6665, 0xFF4D5758, 0xFF444F51, 0xFF00090B, 0xFF000605, 0xFF121713, 0xFF040500, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070906, 0xFF060B07, 0xFF141817, 0xFF05090A, 0xFF101616, 0xFFA1A6A9, 0xFFF0F8FA, 0xFFD5DDE0, 0xFFDEEDF2,
        0xFFA7B2B6, 0xFF111A19, 0xFF060B07, 0xFF11130E, 0xFF060805, 0xFF080C0B, 0xFF090F0F, 0xFF070D0B, 0xFF070D0B, 0xFF080C0B, 0xFF090D0C, 0xFF090D0C, 0xFF0B0D0C, 0xFF0B0D0C, 0xFF0B0D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D,
        0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0E0D, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B100C, 0xFF0B100C, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0B100A,
        0xFF0B100A, 0xFF0C110B, 0xFF0C110B, 0xFF0D120C, 0xFF0D120C, 0xFF0E130D, 0xFF0E130D, 0xFF0D140D, 0xFF0A110A, 0xFF131A13, 0xFF0E1410, 0xFF0C120E, 0xFF101614, 0xFF101614, 0xFF0C1210, 0xFF0E1410, 0xFF121814, 0xFF151B17, 0xFF141A16, 0xFF0F1511,
        0xFF0B110D, 0xFF0C120E, 0xFF0E1410, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F,
        0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0D120E, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0F1410, 0xFF0F1410, 0xFF0F1410, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF10150F, 0xFF10150F,
        0xFF292E28, 0xFF575C55, 0xFF696E67, 0xFF616861, 0xFF5D645D, 0xFF404740, 0xFF595E58, 0xFF949993, 0xFFD5D7D2, 0xFFFEFFFB, 0xFFFEFFFB, 0xFFFBFDF8, 0xFFF4F6F1, 0xFFF0F2ED, 0xFFDCDED9, 0xFFC0C2BD, 0xFF878984, 0xFF656762, 0xFF939590, 0x00FFFFFD,
        0x00FFFFFD, 0x00FFFFFD, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFDFF, 0x00F8F9FD, 0x00F9FAFE, 0x43CCD0D3, 0xFF1F2324, 0xFF010506, 0xFF0A1010, 0xFF010707, 0xFF090D0C,
        0xFF070B0A, 0xFF080C0B, 0xFF0E1213, 0xFF000202, 0xFF444A4A, 0xFF2C3232, 0xFF000404, 0xFF2D3331, 0xFF626B6A, 0xFF858F90, 0xFF303B3D, 0xFF050F11, 0xFF030C0B, 0xFF040905, 0xFF0D0E09, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF0F1110, 0xFF020605, 0xFF020607, 0xFF0D1112, 0xFF000202, 0xFF272D2D, 0xFFC2CBCA, 0xFFBDC5C7, 0xFFAFBEC3,
        0xFFF7FFFF, 0xFF8D9695, 0xFF060B07, 0xFF070904, 0xFF10120F, 0xFF050908, 0xFF000606, 0xFF070D0B, 0xFF070D0B, 0xFF080C0B, 0xFF090D0C, 0xFF090D0C, 0xFF0B0D0C, 0xFF0B0D0C, 0xFF0B0D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D,
        0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0B100C, 0xFF0B100C, 0xFF0B100A, 0xFF0B100A, 0xFF0C110B, 0xFF0B100A,
        0xFF0B100A, 0xFF0C110B, 0xFF0C110B, 0xFF0D120C, 0xFF0D120C, 0xFF0E130D, 0xFF0E130D, 0xFF0A110A, 0xFF131A13, 0xFF131915, 0xFF000501, 0xFF080E0C, 0xFF000301, 0xFF0A100E, 0xFF0F1515, 0xFF0B110D, 0xFF0C120E, 0xFF0E1410, 0xFF0F1511, 0xFF0E1410,
        0xFF0E1410, 0xFF0E1410, 0xFF0E1410, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F,
        0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0F1410, 0xFF0F1410, 0xFF0E130F, 0xFF0E130F, 0xFF0E130D, 0xFF111610,
        0xFF2F342E, 0xFF5A5F58, 0xFF6A6F68, 0xFF656C65, 0xFF5C635C, 0xFF656A64, 0xFFBABFB9, 0xFFECEEE9, 0xFFF7F9F4, 0xFFF5F6F1, 0xFFEFF0EB, 0xFFF4F6F1, 0xFFF6F8F3, 0xFFF7F9F4, 0xFFF7F9F4, 0xFFFEFFFB, 0xFFF3F5F0, 0xFFD1D3CE, 0xFFCDCFCA, 0x00FBFAF8,
        0x00FCFBF9, 0x00FDFBFC, 0x00FFFDFE, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FFFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x21DBDFE0, 0xFF292D2C, 0xFF030706, 0xFF090F0B, 0xFF030905, 0xFF0A0E0F,
        0xFF030708, 0xFF0C100F, 0xFF0C100F, 0xFF040807, 0xFF010504, 0xFF0B0F0E, 0xFF060A09, 0xFF101614, 0xFF161F1E, 0xFF263031, 0xFF606B6D, 0xFF00090B, 0xFF050E0D, 0xFF080D09, 0xFF090A05, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF060709, 0xFF1E2223, 0xFF020607, 0xFF0D1112, 0xFF161C1C, 0xFF000202, 0xFF4C5554, 0xFFB9C2BF, 0xFF46555A,
        0xFF646F73, 0xFF9CA5A4, 0xFF191E1A, 0xFF000100, 0xFF060805, 0xFF0B0F0E, 0xFF101616, 0xFF070D0B, 0xFF070D0B, 0xFF080C0B, 0xFF090D0C, 0xFF090D0C, 0xFF0B0D0C, 0xFF0B0D0C, 0xFF0B0D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D,
        0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0F0B, 0xFF0B100C, 0xFF0B100A, 0xFF0B100A, 0xFF0B100A, 0xFF0B100A,
        0xFF0B100A, 0xFF0B100A, 0xFF0C110B, 0xFF0C110B, 0xFF0D120C, 0xFF0D120C, 0xFF0D120C, 0xFF0C120E, 0xFF0C120E, 0xFF020804, 0xFF494F4D, 0xFF898F8D, 0xFF737977, 0xFF454B4B, 0xFF151B1B, 0xFF080E0C, 0xFF060C0A, 0xFF060C0A, 0xFF0B110F, 0xFF131917,
        0xFF161C1A, 0xFF111715, 0xFF0C1210, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F,
        0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF101511, 0xFF0E130F, 0xFF0F1410, 0xFF0E130D, 0xFF131812,
        0xFF333832, 0xFF575C55, 0xFF61665F, 0xFF6E736D, 0xFF5C615B, 0xFF888D87, 0xFFF7F9F4, 0xFFFEFFFB, 0xFFF9FAF5, 0xFFF8F9F4, 0xFFF4F5F0, 0xFFF2F3EE, 0xFFF5F6F1, 0xFFF8F9F4, 0xFFF4F5F0, 0xFFF4F5F0, 0xFFF6F7F2, 0xFFF7F8F3, 0xFFFBFCF7, 0xFFFAFCF9,
        0x00FAFCFB, 0x00FBFDFC, 0x00FDFFFE, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F6F7FB, 0x00FEFFFF, 0x00FEFFFF, 0x0DE5E9E8, 0xFF303531, 0xFF010600, 0xFF060D05, 0xFF0A1109, 0xFF050B0B,
        0xFF040A0A, 0xFF060A0B, 0xFF010504, 0xFF111312, 0xFF030502, 0xFF060805, 0xFF080806, 0xFF373D3B, 0xFF6A7372, 0xFF788283, 0xFF5E696B, 0xFF061012, 0xFF050E0D, 0xFF060B07, 0xFF080904, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF121317, 0xFF969A9D, 0xFF85898C, 0xFF040809, 0xFF090F0F, 0xFF050B09, 0xFF000805, 0xFF6E7774, 0xFFA3B2B7,
        0xFF101B1F, 0xFF060F0E, 0xFF030804, 0xFF131510, 0xFF0C0E0B, 0xFF090D0C, 0xFF050B0B, 0xFF070D0B, 0xFF070D0B, 0xFF080C0B, 0xFF090D0C, 0xFF090D0C, 0xFF0B0D0C, 0xFF0B0D0C, 0xFF0B0D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D,
        0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0F0B, 0xFF0B100C, 0xFF0B100A, 0xFF0B100A, 0xFF0B100A, 0xFF0B100A,
        0xFF0B100A, 0xFF0B100A, 0xFF0C110B, 0xFF0C110B, 0xFF0D120C, 0xFF0D120C, 0xFF0D120C, 0xFF0B110D, 0xFF111713, 0xFF121814, 0xFFA3A9A7, 0xFFA3A9A7, 0xFFA2A8A8, 0xFFB9BFBF, 0xFFC3C9C9, 0xFFA1A7A5, 0xFF808684, 0xFF4D5351, 0xFF202624, 0xFF070D0B,
        0xFF030907, 0xFF090F0D, 0xFF101614, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F,
        0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF101511, 0xFF0D120E, 0xFF101511, 0xFF0E130D, 0xFF141913,
        0xFF333832, 0xFF4E534C, 0xFF51564F, 0xFF555A54, 0xFF444943, 0xFF868883, 0xFFF9FBF6, 0xFFF4F5F0, 0xFFF7F8F3, 0xFFFCFDF8, 0xFFFFFFFB, 0xFFFFFFFB, 0xFFF7F8F3, 0xFFF1F2ED, 0xFFF7F8F3, 0xFFF9FAF5, 0xFFF7F8F3, 0xFFF7F8F3, 0xFFF5F6F1, 0xFFF7FDFB,
        0xFFF5FBF9, 0xFFF4FAF8, 0xFFF4FAFA, 0x00F7FDFD, 0x00F9FEFF, 0x00FAFFFF, 0x00FBFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FBFBFB, 0x00FBFBFB, 0x00FFFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FDFFFE, 0x15E2E7E3, 0xFF333832, 0xFF020700, 0xFF020A00, 0xFF050D00, 0xFF070D0D,
        0xFF080E0E, 0xFF040809, 0xFF0B0F0E, 0xFF030502, 0xFF10100E, 0xFF070803, 0xFF0A0B06, 0xFF020806, 0xFF0D1615, 0xFF172122, 0xFF061113, 0xFF061012, 0xFF000302, 0xFF0D120E, 0xFF090A05, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF030409, 0xFF6C7073, 0xFFF0F4F7, 0xFF616566, 0xFF000303, 0xFF090F0D, 0xFF060F0A, 0xFF000601, 0xFFA1B0B5,
        0xFF788387, 0xFF000807, 0xFF0A0F0B, 0xFF0B0D08, 0xFF050704, 0xFF090D0C, 0xFF040A0A, 0xFF070D0B, 0xFF070D0B, 0xFF080C0B, 0xFF090D0C, 0xFF090D0C, 0xFF0B0D0C, 0xFF0B0D0C, 0xFF0B0D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D,
        0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0B100A, 0xFF0B100A, 0xFF0B100A, 0xFF0A0F09,
        0xFF0B100A, 0xFF0B100A, 0xFF0C110B, 0xFF0C110B, 0xFF0D120C, 0xFF0D120C, 0xFF0D120C, 0xFF0E1410, 0xFF0A100C, 0xFF0E1412, 0xFFB4BAB8, 0xFF2B3131, 0xFF000404, 0xFF1F2525, 0xFF33383B, 0xFF656B69, 0xFF868C8A, 0xFFB0B6B4, 0xFFC5CBC9, 0xFFB5BBB9,
        0xFF878D8B, 0xFF545A58, 0xFF333937, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F,
        0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0E130F, 0xFF0E130F, 0xFF0E130F, 0xFF0F1410, 0xFF0C110D, 0xFF101511, 0xFF0E130D, 0xFF151A14,
        0xFF323731, 0xFF474C45, 0xFF454A43, 0xFF484D47, 0xFF3C413B, 0xFF7C7E79, 0xFFEFF1EC, 0xFFF1F2ED, 0xFFF3F4EF, 0xFFE8E9E4, 0xFFF2F1ED, 0xFFF1F2ED, 0xFFFDFEF9, 0xFFFDFEF9, 0xFFF5F6F1, 0xFFECEDE8, 0xFFEDEEE9, 0xFFF5F6F1, 0xFFF3F4EF, 0xFFEFF8F5,
        0xFFEBF4F1, 0xFFE7F0EF, 0xFFE6EFEE, 0xFFE8F1F0, 0xFFECF4F6, 0xFFEEF6F8, 0x00EFF7F9, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFFFD, 0x00FEFFFD, 0x00FBFDFA, 0x08E8EDE9, 0xFF343935, 0xFF010602, 0xFF090F0B, 0xFF030905, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF040A06, 0xFF060C08, 0xFF040A06, 0xFF080E0A, 0xFF010703, 0xFF0A100C, 0xFF050B07, 0xFF000200, 0xFF000909, 0xFF071313, 0xFF869091, 0xFF60686A, 0xFF000206, 0xFF1A1D22, 0xFF95969B, 0xFF9A9BA0, 0xFFC0CBCD,
        0xFFCED9DB, 0xFF162021, 0xFF000807, 0xFF0D1311, 0xFF060A09, 0xFF0C0E0B, 0xFF090B06, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09,
        0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100A,
        0xFF0B100A, 0xFF0B100A, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0D140D, 0xFF0B110D, 0xFF161C1C, 0xFFB1B6B9, 0xFF272C2F, 0xFF0D1313, 0xFF101612, 0xFF0B120B, 0xFF060F00, 0xFF292F2B, 0xFF292D36, 0xFF31353E, 0xFF515659,
        0xFF858B89, 0xFFA2A7AA, 0xFFCACED7, 0xFFB8C3BF, 0xFF5E6867, 0xFF060E10, 0xFF293134, 0xFF202529, 0xFF101616, 0xFF121713, 0xFF0C110B, 0xFF0E1410, 0xFF0E1410, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0C120E, 0xFF0C120E, 0xFF0E130F,
        0xFF0E130F, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0E130F, 0xFF0E130F, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF11120D, 0xFF0E100B, 0xFF121411, 0xFF0C110D, 0xFF151E1B,
        0xFF323C3B, 0xFF3B4544, 0xFF323E3C, 0xFF2E394D, 0xFF222D2F, 0xFF747C6D, 0xFFF3F7E8, 0xFFCCCDC8, 0xFFE7E6EC, 0xFFEFEFF7, 0xFFECF0F3, 0xFFF5E4DD, 0xFFFFF7F0, 0xFFE2E3DB, 0xFFC7D1C8, 0xFFD8E3DB, 0xFFE4EBE3, 0xFFE6E5E0, 0xFFF2EBE5, 0xFFE6E6DA,
        0xFFE5EBE1, 0xFFF7FFFF, 0xFFE6FBFE, 0xFFB8D8E5, 0xFFAFD9F2, 0xFF98C8EC, 0xFF96C9F4, 0xFF94D4FA, 0x7DB0E3FF, 0x2AD8F5FF, 0x00F5FEFF, 0x00FFFEFF, 0x00FFFDFF, 0x00FEFDFF, 0x00F9FEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFDFF, 0x00FFFBFF,
        0x00FFF9FF, 0x00FFF9FF, 0x00FFF9FF, 0x00FFFFFD, 0x00FFFFFD, 0x00FFFEFD, 0x00FFFEFD, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FEFEFF, 0x00FEFFFF, 0x00FDFEFF, 0x00FDFEFF, 0x00FEFFFF, 0x00FDFFFE, 0x00FDFFFC, 0x00FCFEFB, 0x00F7FEFF,
        0x00FAFFFF, 0x00FBFFFF, 0x00FCFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FDFDFD, 0x00FCFCFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFFFD, 0x00FEFFFD, 0x00FAFCF9, 0x08E7ECE8, 0xFF353A36, 0xFF000300, 0xFF080E0A, 0xFF050B07, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF0C120E, 0xFF0C120E, 0xFF050B07, 0xFF050B07, 0xFF0B110D, 0xFF030905, 0xFF050B07, 0xFF5A605C, 0xFF384347, 0xFF061113, 0xFF000305, 0xFF0D1518, 0xFF000306, 0xFF191D20, 0xFFCECFD3, 0xFFE8E9ED, 0xFFD9E4E6,
        0xFFA2ADAF, 0xFF192324, 0xFF030C0B, 0xFF060C0A, 0xFF000201, 0xFF0F110E, 0xFF0A0C07, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09,
        0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100A,
        0xFF0B100A, 0xFF0B100A, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0D140D, 0xFF0B110D, 0xFF181E1E, 0xFFB0B5B8, 0xFF24292C, 0xFF0D1313, 0xFF101612, 0xFF0B120B, 0xFF030B00, 0xFF9AA09C, 0xFFD4D8E1, 0xFF8A8E99, 0xFF010609,
        0xFF0C1210, 0xFF03080B, 0xFF10141D, 0xFF4D5854, 0xFFBFC9C8, 0xFF272F31, 0xFF9EA6A9, 0xFFABB0B4, 0xFF1D2323, 0xFF0D120E, 0xFF0E130D, 0xFF0E1410, 0xFF0E1410, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0C120E, 0xFF0C120E, 0xFF0E130F,
        0xFF0E130F, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0E130F, 0xFF0E130F, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF11120D, 0xFF0E100B, 0xFF121411, 0xFF0D120E, 0xFF181E1C,
        0xFF333C39, 0xFF3A4541, 0xFF343E3D, 0xFF293543, 0xFF1A2829, 0xFF718278, 0xFFCFE4DF, 0xFF7F97A3, 0xFF8BAAC6, 0xFF98BFDE, 0xFF9FCCE9, 0xFFCCDEE0, 0xFFF0FFFF, 0xFFC5DFF0, 0xFF637E93, 0xFF5D7788, 0xFFBCCDD4, 0xFFF2FCF4, 0xFFE6E8DA, 0xFFFEFFF9,
        0xFFFEFFFA, 0xFFF4FBF4, 0xFFF8FFFF, 0xFFD8E8E8, 0xFFBACFD4, 0xFFB2C8D3, 0xFFABC3CF, 0xFF97B5D1, 0xFF8DB3D7, 0xD984B9E3, 0xC98CC7F3, 0x8EA8DDFF, 0x3BCFF0FF, 0x00F2FDF9, 0x00FFFFED, 0x00FFFDFF, 0x00FFFDFF, 0x00FFFDFC, 0x00FEFEFC, 0x00FEFFFB,
        0x00FBFFFA, 0x00F9FFFA, 0x00F8FFFA, 0x00FAFFFF, 0x00FAFFFF, 0x00FCFFFF, 0x00FEFFFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FFFCFE, 0x00FFFBFA, 0x00FFFDFB, 0x00FFFDFD, 0x00FFFEFF, 0x00FEFFFF, 0x00FCFFFF, 0x00F9FFFF, 0x00F9FFFF, 0x00FEFFFF,
        0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FDFEFF, 0x00FCFDFF, 0x00FDFDFD, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFFFD, 0x00FEFFFD, 0x00FDFFFC, 0x00ECF1ED, 0xFF404541, 0xFF000400, 0xFF080E0A, 0xFF060C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF000300, 0xFF010703, 0xFF080E0A, 0xFF030905, 0xFF0A100C, 0xFF030905, 0xFF1D231F, 0xFFBAC0BC, 0xFFD7E0E9, 0xFF959EA5, 0xFF3C4349, 0xFF000205, 0xFF090E11, 0xFF0D1313, 0xFF4C504F, 0xFFE3E8E4, 0xFF768183,
        0xFF040F11, 0xFF010B0C, 0xFF050E0D, 0xFF020806, 0xFF111514, 0xFF060805, 0xFF0A0C07, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09,
        0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100A,
        0xFF0B100A, 0xFF0B100A, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0D140D, 0xFF0B110D, 0xFF1D2323, 0xFFAFB4B7, 0xFF202528, 0xFF0D1313, 0xFF0F1511, 0xFF0B120B, 0xFF020D00, 0xFFA5AEAB, 0xFFDDE4EE, 0xFF9EA5AF, 0xFF2C3436,
        0xFFA7B0AD, 0xFF9AA2A4, 0xFF616872, 0xFF18231F, 0xFFA4AEAD, 0xFF3D4547, 0xFFC3CBCE, 0xFF909599, 0xFF171D1D, 0xFF050A06, 0xFF171C16, 0xFF0E1410, 0xFF0E1410, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0C120E, 0xFF0C120E, 0xFF0E130F,
        0xFF0E130F, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0E130F, 0xFF0E130F, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF10120D, 0xFF0E100B, 0xFF0F1410, 0xFF0D120E, 0xFF1B211D,
        0xFF363C38, 0xFF3B443F, 0xFF353E39, 0xFF373936, 0xFF141A10, 0xFF8E9C8F, 0xFFE6FCF9, 0xFF688C98, 0xFF6B9CBA, 0xFF7CBAE1, 0xFF74B9E2, 0xFF9AC8D5, 0xFFACDAF1, 0xFFADDBFD, 0xFF96C0E8, 0xFF87A9CC, 0xFFB6D0DF, 0xFFDFEEE9, 0xFFE8F1DE, 0xFFF6F8F7,
        0xFFFFFFFD, 0xFFF2F3ED, 0xFFFCFCF2, 0xFFFFFFF3, 0xFFFFFFEF, 0xFFFEFCE7, 0xFFEBE9D2, 0xFFE6E2BF, 0xFFD1D5BC, 0xFFAEBFB7, 0xFF8AABB4, 0xFF77A3BC, 0xEE79ADD4, 0xC48DC3F1, 0xA79CD4FF, 0x69BAE0EB, 0x43CCF1FA, 0x11E4FFFF, 0x00EDFFFF, 0x00F0FFFF,
        0x00F0FDFF, 0x00F4FDFF, 0x00F9FFFF, 0x00FFFDFA, 0x00FFFEFA, 0x00FFFDFC, 0x00FFFDFF, 0x00FDFEFF, 0x00FCFFFF, 0x00FBFFFF, 0x00F9FFFF, 0x00FAFDFF, 0x00FCFDFF, 0x00FDFEFF, 0x00FFFEFC, 0x00FFFDFA, 0x00FFFCF6, 0x00FFFCF2, 0x00FFFBF2, 0x00FFFCF8,
        0x00FFFBF8, 0x00FFFBF8, 0x00FDFCFA, 0x00FDFDFD, 0x00FEFEFE, 0x00FEFFFF, 0x00FEFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFEFB, 0x00FEFFFD, 0x00FEFFFD, 0x00F4F9F5, 0xFF505551, 0xFF010602, 0xFF080E0A, 0xFF060C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF0B110D, 0xFF020804, 0xFF0A100C, 0xFF060C08, 0xFF060C08, 0xFF050B07, 0xFF030905, 0xFF595F5B, 0xFFD1D7E3, 0xFFD5DCE6, 0xFFB9C0C8, 0xFF687073, 0xFF1C2124, 0xFF000200, 0xFF0A100C, 0xFF6E756E, 0xFF9CA7A9,
        0xFF131E20, 0xFF000A0B, 0xFF0F1817, 0xFF030907, 0xFF0F1312, 0xFF030502, 0xFF10120D, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09,
        0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100A,
        0xFF0B100A, 0xFF0B100A, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0C130C, 0xFF0B110D, 0xFF222828, 0xFFAEB3B6, 0xFF1B2023, 0xFF0D1313, 0xFF0F1511, 0xFF0C130C, 0xFF101B0B, 0xFFA3ACA9, 0xFFD5DCE6, 0xFFA1A8B2, 0xFF363E41,
        0xFFC3CCC9, 0xFFCFD7DA, 0xFF8B929C, 0xFF222D29, 0xFFA9B3B2, 0xFF2F3739, 0xFFB5BDC0, 0xFF868B8F, 0xFF1A2020, 0xFF121713, 0xFF0D120C, 0xFF0E1410, 0xFF0E1410, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0C120E, 0xFF0C120E, 0xFF0E130F,
        0xFF0E130F, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0E130F, 0xFF0E130F, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF10120D, 0xFF0C110B, 0xFF0F140E, 0xFF0E130D, 0xFF1F241E,
        0xFF393E37, 0xFF3D423B, 0xFF383F37, 0xFF3C3723, 0xFF2E2C17, 0xFFA0A28C, 0xFFE8F1E0, 0xFF94A59F, 0xFF8AA7AB, 0xFF7AA2AC, 0xFF8CBAC9, 0xFFAAD2DE, 0xFFAED5E6, 0xFFA7CDE2, 0xFF95B8CE, 0xFF89A4B5, 0xFFC7DAE1, 0xFFE6F1ED, 0xFFDFE5DB, 0xFFEBEBED,
        0xFFFDFDFD, 0xFFFFFEFB, 0xFFFBF7EE, 0xFFFCF5E3, 0xFFFEF7DD, 0xFFE1D9B5, 0xFFD9D0A9, 0xFFCBDBB4, 0xFFDDE4BA, 0xFFEFEBBE, 0xFFF0E4BE, 0xFFDAD2BD, 0xFFB4BCBE, 0xFF92ADC8, 0xFF7CA6D0, 0xFF7EB3E9, 0xFF82B6E8, 0xC48DBCE6, 0xA39EC6E9, 0x79B1D3EC,
        0x4BC7E4F2, 0x21DBF4FB, 0x0DE6FEFF, 0x00F3FCFF, 0x00F5FEFF, 0x00F8FFFF, 0x00FBFFFF, 0x00FEFFFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FFFFFF, 0x00FFFFFD, 0x00FFFFFB, 0x00FFFFFA, 0x00FFFFFA, 0x00FFFFF8, 0x00FFFEF8,
        0x00FFFEF8, 0x00FFFEFA, 0x00FFFEFB, 0x00FFFFFD, 0x00FFFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFFFC, 0x00FEFFFD, 0x00FEFFFD, 0x00F6FBF7, 0xFF5A5F5B, 0xFF000501, 0xFF070D09, 0xFF070D09, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF080E0A, 0xFF040A06, 0xFF070D09, 0xFF070D09, 0xFF020804, 0xFF0C120E, 0xFF000300, 0xFF020804, 0xFF515560, 0xFFD8DCE5, 0xFFB9BEC4, 0xFFADB2B6, 0xFF8B9395, 0xFF202928, 0xFF030C09, 0xFF040D0A, 0xFF9EA9AB,
        0xFF848F91, 0xFF000304, 0xFF060F0E, 0xFF090F0D, 0xFF000100, 0xFF0D0F0C, 0xFF0C0E09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09,
        0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100A,
        0xFF0B100A, 0xFF0B100A, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0B120B, 0xFF0B110D, 0xFF292F2F, 0xFFACB1B4, 0xFF151A1D, 0xFF0D1313, 0xFF0E1410, 0xFF0C130C, 0xFF000B00, 0xFFB0BAB9, 0xFFDDE5F0, 0xFF88909B, 0xFF434D4F,
        0xFFD4DEDD, 0xFFC8D2D4, 0xFF373F4A, 0xFF15201C, 0xFFAFB9B8, 0xFF1F2729, 0xFFA8B0B3, 0xFF999EA2, 0xFF0B1111, 0xFF181D19, 0xFF10150F, 0xFF0E1410, 0xFF0E1410, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0C120E, 0xFF0C120E, 0xFF0E130F,
        0xFF0E130F, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0E130F, 0xFF0E130F, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0C110B, 0xFF0E130D, 0xFF0F140D, 0xFF242720,
        0xFF3C3F36, 0xFF3E4136, 0xFF3C3F34, 0xFF2D3424, 0xFF272B1C, 0xFFB5B6A6, 0xFFEBEAD6, 0xFF94967E, 0xFFCBCFB6, 0xFFE0E8D1, 0xFFCCD5C2, 0xFFD1D9DC, 0xFFD1DAD7, 0xFFD7E3D5, 0xFFDEEBD7, 0xFFC0CCB6, 0xFFDAE0D2, 0xFFE4E4E2, 0xFFF2F0F5, 0xFFE7E7E5,
        0xFFEAE9E7, 0xFFFFFEFC, 0xFFFCFBF7, 0xFFFFFFF6, 0xFFFFFFED, 0xFFEBECCD, 0xFFF1F3CE, 0xFFE8E2C2, 0xFFD6D1B1, 0xFFC6C1A1, 0xFFC7C4A3, 0xFFD8D3B3, 0xFFE8E2C0, 0xFFECE4BF, 0xFFE8E0BB, 0xFFC0CFB0, 0xFFB6C8B0, 0xFFA9BFB3, 0xFFA0B9BE, 0xFF9BB9D1,
        0xFF99BCE4, 0xFF98BDF2, 0xB496BEFB, 0x92A5D7F8, 0x82AEDBFA, 0x60BEE3FD, 0x3BD0EDFF, 0x11E3F5FF, 0x00F3FCFF, 0x00FFFFFF, 0x00FFFEFB, 0x00FFFBF5, 0x00FFFEF9, 0x00FFFFFB, 0x00FCFFFF, 0x00F9FFFF, 0x00F6FFFF, 0x00F5FFFF, 0x00F4FFFF, 0x00FFFCF8,
        0x00FFFDF9, 0x00FFFDFA, 0x00FFFFFD, 0x00FFFFFD, 0x00FFFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFFFD, 0x00FDFFFC, 0x00FBFDFA, 0x00F2F7F3, 0xFF606561, 0xFF000200, 0xFF060C08, 0xFF080E0A, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF010703, 0xFF0D130F, 0xFF080E0A, 0xFF060C08, 0xFF000602, 0xFF0B110D, 0xFF0A100C, 0xFF060C08, 0xFF080C0F, 0xFF393C41, 0xFFAFB4B8, 0xFFC6CBCF, 0xFFB4BCBF, 0xFF98A0A3, 0xFF273133, 0xFF000305, 0xFF2A3537,
        0xFFBFCACC, 0xFF353F40, 0xFF07100F, 0xFF0E1412, 0xFF0B0F0E, 0xFF0D0F0C, 0xFF060803, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09,
        0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100A,
        0xFF0B100A, 0xFF0B100A, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0B120B, 0xFF0B110D, 0xFF2E3434, 0xFFABB0B3, 0xFF0F1417, 0xFF0E1414, 0xFF0D130F, 0xFF0C130C, 0xFF111D0F, 0xFFB2BCBB, 0xFFDBE3EE, 0xFF7E8691, 0xFF262F34,
        0xFF717B7A, 0xFF9AA3A8, 0xFF939BA6, 0xFF2C3733, 0xFFA6B0AF, 0xFF384042, 0xFFB1B9BC, 0xFF9A9FA3, 0xFF0A1010, 0xFF0D120E, 0xFF151A14, 0xFF0E1410, 0xFF0E1410, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0C120E, 0xFF0C120E, 0xFF0E130F,
        0xFF0E130F, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0E130F, 0xFF0E130F, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0C120E, 0xFF0C110B, 0xFF0D120C, 0xFF12150E, 0xFF272A1F,
        0xFF3E4033, 0xFF3E4033, 0xFF3E3F31, 0xFF29423C, 0xFF1D312F, 0xFFA6B2B0, 0xFFE8EBE2, 0xFFA3A38B, 0xFFB3B08F, 0xFFC4C19E, 0xFFDFDCBB, 0xFFF1E3DA, 0xFFF9ECD9, 0xFFF1E9C4, 0xFFDEDAAB, 0xFFB7B489, 0xFFDCD8BD, 0xFFD2CDC9, 0xFFCAC2CD, 0xFFE5E6E0,
        0xFFEEEFEA, 0xFFFEFFFF, 0xFFFCFDFF, 0xFFF3F8F4, 0xFFE5EBDD, 0xFFD7E1C8, 0xFFCDD8B8, 0xFFE1D0A2, 0xFFE6DDB4, 0xFFE5E6C6, 0xFFD8E2C9, 0xFFC8D2BA, 0xFFC0C4A9, 0xFFC7C3A0, 0xFFD2C6A0, 0xFFEEE2B8, 0xFFF1E8C1, 0xFFF2EEC9, 0xFFECEDCE, 0xFFDFE4CD,
        0xFFCEDAC6, 0xFFC3D1C2, 0xFFBDCDC0, 0xFFA2C3CC, 0xFFA0C0CB, 0xFF9DBFCB, 0xFF9BC0D0, 0x9F9FC5DA, 0x96A4CEE4, 0x86ABD7F0, 0x7DB0DEF8, 0x36D1F4FF, 0x26D9F8FF, 0x11E4FEFF, 0x00EEFFFF, 0x00F6FFFF, 0x00FBFFFF, 0x00FFFFFD, 0x00FFFEFB, 0x00FCFFFF,
        0x00FCFFFF, 0x00FCFDFF, 0x00FBFCFE, 0x00FCFCFC, 0x00FDFDFD, 0x00FEFEFE, 0x00FFFEFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFFFD, 0x00FDFFFC, 0x00FCFEFB, 0x00F6FBF7, 0xFF6C716D, 0xFF000400, 0xFF060C08, 0xFF070D09, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF030905, 0xFF090F0B, 0xFF000501, 0xFF040A06, 0xFF080E0A, 0xFF0E1410, 0xFF040A06, 0xFF020804, 0xFF0C0E0B, 0xFF000200, 0xFF242827, 0xFF8B9093, 0xFFC1C9CC, 0xFFD1DAE1, 0xFFBDC7D0, 0xFF2D3741, 0xFF3C4749,
        0xFFD2DDDF, 0xFF707A7B, 0xFF000504, 0xFF000200, 0xFF111514, 0xFF040603, 0xFF0C0E09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09,
        0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100A,
        0xFF0B100A, 0xFF0B100A, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0A110A, 0xFF0B110D, 0xFF333939, 0xFFAAAFB2, 0xFF0B1013, 0xFF0E1414, 0xFF0D130F, 0xFF0C130C, 0xFF101C10, 0xFFB6C0C1, 0xFFD9E1EC, 0xFF858D9A, 0xFF525B60,
        0xFF869091, 0xFF616A6F, 0xFF606873, 0xFF2E3935, 0xFFA7B1B0, 0xFF272F31, 0xFFA6AEB1, 0xFF9EA3A7, 0xFF191F1F, 0xFF0B100C, 0xFF080D07, 0xFF0E1410, 0xFF0E1410, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0C120E, 0xFF0C120E, 0xFF0E130F,
        0xFF0E130F, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0E130F, 0xFF0E130F, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0C120E, 0xFF0B120B, 0xFF0D120B, 0xFF12150C, 0xFF2A2C21,
        0xFF404133, 0xFF3F4030, 0xFF413F30, 0xFF273C2D, 0xFF33423D, 0xFFC2C7CA, 0xFFE2E3DD, 0xFF999B85, 0xFFCDCEAE, 0xFFEAEBCB, 0xFFD7DABF, 0xFFE3DAC9, 0xFFE7E0C6, 0xFFE9E3C1, 0xFFEDE8C2, 0xFFCCC8A5, 0xFFE6E4CF, 0xFFA7A49F, 0xFF67656A, 0xFF75786D,
        0xFF9EA19A, 0xFFF0F2EF, 0xFFFCFCFC, 0xFFEDEEE9, 0xFFEBEDDF, 0xFFEBEED3, 0xFFE8ECC9, 0xFFD2EBCE, 0xFFC5D8BA, 0xFFBBC6A6, 0xFFC4C5A3, 0xFFD9D5B2, 0xFFE9DFBC, 0xFFE9DCBC, 0xFFE2D4B7, 0xFFC2C495, 0xFFC4C395, 0xFFC8C59A, 0xFFD4CFA5, 0xFFE7DEB5,
        0xFFF7EBC5, 0xFFFFEFCB, 0xFFFFF1CD, 0xFFF7EECF, 0xFFEDE9CE, 0xFFDCDFCC, 0xFFC9D6CD, 0xFFB5CFD0, 0xFFA6CBD4, 0xFF9CCADA, 0xFF97CADF, 0xFF8FC1DC, 0xFF98C6E0, 0x8EA8CEE5, 0x6DB7D6E8, 0x4BC8DEEB, 0x21DBE8F1, 0x00EBF2F8, 0x00F6FAFD, 0x00F3FFFF,
        0x00F5FFFF, 0x00F7FFFF, 0x00F9FFFF, 0x00FCFFFF, 0x00FFFFFF, 0x00FFFCF9, 0x00FFFBF8, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFFFD, 0x00FEFFFD, 0x00FEFFFD, 0x00FCFFFD, 0xEE7A7F7B, 0xFF050A06, 0xFF080E0A, 0xFF050B07, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF080E0A, 0xFF090F0B, 0xFF080E0A, 0xFF080E0A, 0xFF010703, 0xFF060C08, 0xFF020804, 0xFF0F1511, 0xFF0B0E05, 0xFF070A03, 0xFF060B07, 0xFF030909, 0xFF42494F, 0xFF8F98A1, 0xFFC8D2DE, 0xFFCFD8E7, 0xFFC7D2D4,
        0xFFDDE8EA, 0xFF636D6E, 0xFF000605, 0xFF040A08, 0xFF090D0C, 0xFF0B0D0A, 0xFF0D0F0A, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09,
        0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100A,
        0xFF0B100A, 0xFF0B100A, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0A110A, 0xFF0B110D, 0xFF353B3B, 0xFFA9AEB1, 0xFF080D10, 0xFF0E1414, 0xFF0C120E, 0xFF0D140D, 0xFF0F1D10, 0xFFC5D1D1, 0xFFD4DEEA, 0xFF6B7581, 0xFF5F6A6E,
        0xFFD1DDDD, 0xFF949FA3, 0xFF98A2AE, 0xFF3C4743, 0xFF9FA9A8, 0xFF2B3335, 0xFF90989B, 0xFF5C6165, 0xFF0A1010, 0xFF0E130F, 0xFF0E130D, 0xFF0E1410, 0xFF0E1410, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0C120E, 0xFF0C120E, 0xFF0E130F,
        0xFF0E130F, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0E130F, 0xFF0E130F, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0C120E, 0xFF0B120B, 0xFF0D120B, 0xFF12150C, 0xFF2B2D20,
        0xFF414234, 0xFF403F2D, 0xFF41402E, 0xFF323112, 0xFF302B18, 0xFF948B84, 0xFFBAB2A7, 0xFF97937A, 0xFFB0B191, 0xFFC0C7A8, 0xFFDBE3CC, 0xFFEBF4DF, 0xFFEAF2DD, 0xFFE4E9D5, 0xFFD5D8C5, 0xFFA7A99B, 0xFFE7E9DE, 0xFFDFE4DD, 0xFFC9CECA, 0xFF858D7E,
        0xFF83897F, 0xFFEDEFEA, 0xFFFAF9F7, 0xFFEEEAE1, 0xFFF1EAD7, 0xFFD4CBAA, 0xFFDBD2A7, 0xFFDDCDB3, 0xFFEBDCBF, 0xFFF5E8C8, 0xFFEAE2BE, 0xFFD2D0AA, 0xFFBFC49C, 0xFFBDC89E, 0xFFC2D1A8, 0xFFE6DCD2, 0xFFEBE1D5, 0xFFEBE4D4, 0xFFE3DECA, 0xFFD6D4BB,
        0xFFCBCCAD, 0xFFC7CBA8, 0xFFC8CEA8, 0xFFDDD7B5, 0xFFE4DEBC, 0xFFF1E8C7, 0xFFFBF1CE, 0xFFFEF1CE, 0xFFF9EBC6, 0xFFF1E1BD, 0xFFEDDBB7, 0xFFD0D1C9, 0xFFC8CECA, 0xFFBBCACD, 0xFFAAC6D4, 0xFF9EC3DE, 0xB097C9EE, 0xB098D2FF, 0xA39DDBFF, 0x4FC6DBEC,
        0x3BD0E3F2, 0x19DFEFFC, 0x00F0FAFF, 0x00FBFFFF, 0x00FFFFFF, 0x00FFFEFB, 0x00FFFDF7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FBFFFE, 0xCD898D8C, 0xFF030706, 0xFF090F0D, 0xFF060C0A, 0xFF060C08,
        0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF0A0B0D, 0xFF0A0B0D, 0xFF0E0F11, 0xFF0C0D0F, 0xFF050608, 0xFF030406, 0xFF2F3032, 0xFF9D9EA0, 0xFFC8D3D9,
        0xFF5D686C, 0xFF1A2423, 0xFF060F0A, 0xFF060D06, 0xFF010602, 0xFF0C0E0D, 0xFF0A0B0D, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09,
        0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF040D08, 0xFF040D08, 0xFF070D09, 0xFF070D09, 0xFF080D09, 0xFF080D09, 0xFF090E0A, 0xFF0B0D0A, 0xFF0D1112, 0xFF050A04, 0xFF0A0F08, 0xFF090D0C, 0xFF080B14, 0xFF0D1019, 0xFF101413, 0xFF171D0F, 0xFF191F15,
        0xFF181D16, 0xFF10150E, 0xFF090E07, 0xFF0A0F08, 0xFF090E07, 0xFF090F05, 0xFF0E140A, 0xFF0A110A, 0xFF050B07, 0xFF444A4A, 0xFFA0A5A8, 0xFF04090C, 0xFF0E1414, 0xFF0C120E, 0xFF0A110A, 0xFF1B251C, 0xFFC3CCCB, 0xFFDEE5ED, 0xFF6F767C, 0xFF262F2E,
        0xFFBCC5C2, 0xFFCBD2D8, 0xFF9CA2B2, 0xFF313C38, 0xFF9BA5A4, 0xFF384042, 0xFFB9C1C4, 0xFF999EA2, 0xFF0F1515, 0xFF121713, 0xFF050A04, 0xFF0E1410, 0xFF0E1410, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0C120E, 0xFF0C120E, 0xFF0C1210,
        0xFF0E1211, 0xFF0A0E0D, 0xFF111514, 0xFF121413, 0xFF0A0C0B, 0xFF101010, 0xFF0F0F0F, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0E130F, 0xFF0E130F, 0xFF0A0F09, 0xFF13160F, 0xFF2C2F26,
        0xFF404237, 0xFF3E4033, 0xFF36382B, 0xFF323929, 0xFF23291F, 0xFFA1A39E, 0xFF95978C, 0xFF9A9E85, 0xFFE0E4C3, 0xFFDCDCC0, 0xFFD9D8C4, 0xFFE6DFD5, 0xFFE1DCC6, 0xFFE7E7C5, 0xFFE4E5C5, 0xFFC0C2AD, 0xFFEBECE4, 0xFFEDEEE9, 0xFFF3F6EF, 0xFFDCDBD6,
        0xFFA4A59F, 0xFFEBEEE7, 0xFFF2F8EE, 0xFFEBEFE0, 0xFFF3F5DF, 0xFFF2EED1, 0xFFF3EECE, 0xFFE1DEBD, 0xFFD3D0AF, 0xFFC7C4A3, 0xFFCAC7A6, 0xFFD7D4B3, 0xFFE1DEBD, 0xFFE1DEBD, 0xFFDBD8B7, 0xFFC6C3A2, 0xFFC4C1A0, 0xFFC7C4A3, 0xFFD4D1B0, 0xFFE6E3C2,
        0xFFF2EFCE, 0xFFF2EFCE, 0xFFEEEBCA, 0xFFE3E0BF, 0xFFDCD9B8, 0xFFD3D0AF, 0xFFCECBAA, 0xFFD1CEAD, 0xFFDBD8B7, 0xFFE8E5C4, 0xFFF0EDCC, 0xFFF9F6D5, 0xFFF6F3D2, 0xFFF2EFD0, 0xFFEBE8C9, 0xFFE3E0C1, 0xFFDAD6B9, 0xFFD3CFB2, 0xFFCECAAD, 0xFFAFC9C6,
        0xFFB3CDCE, 0x69BAD2DE, 0x58C1DAEE, 0x43CBE3FB, 0x2ED6F1FF, 0x19E0FBFF, 0x08E7FFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFFFF, 0x00FDFFFE, 0x00FEFFFF, 0x00FCFFFF, 0xB8949897, 0xFF0A0E0D, 0xFF080E0C, 0xFF060C0A, 0xFF060C08,
        0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF010506, 0xFF070B0C, 0xFF0A0E0F, 0xFF05090A, 0xFF0C1011, 0xFF070B0C, 0xFF2E3233, 0xFF585C5D, 0xFFA9B4BA,
        0xFF414C50, 0xFF000403, 0xFF000702, 0xFF020902, 0xFF101511, 0xFF090B0A, 0xFF08090B, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09,
        0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF040D08, 0xFF040D08, 0xFF070D09, 0xFF070D09, 0xFF080D09, 0xFF080D09, 0xFF090E0A, 0xFF0B0D0A, 0xFF0A0E0D, 0xFF060B05, 0xFF090E08, 0xFF080C0D, 0xFF090C13, 0xFF111516, 0xFF1D2414, 0xFF323C1A, 0xFF4E592E,
        0xFF545E39, 0xFF454E33, 0xFF272E1C, 0xFF11170D, 0xFF0B1009, 0xFF0C110B, 0xFF121710, 0xFF0A110A, 0xFF040A06, 0xFF464C4C, 0xFF9EA3A6, 0xFF04090C, 0xFF0D1313, 0xFF0D130F, 0xFF0A110A, 0xFF19231A, 0xFFC4CDCC, 0xFFD0D7DF, 0xFF697076, 0xFF293231,
        0xFF5A6360, 0xFF90979D, 0xFF525868, 0xFF3E4945, 0xFF99A3A2, 0xFF11191B, 0xFF646C6F, 0xFF63686C, 0xFF000505, 0xFF0C110D, 0xFF1E231D, 0xFF0E1410, 0xFF0E1410, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0C120E, 0xFF0C120E, 0xFF0E1412,
        0xFF131917, 0xFF080C0B, 0xFF0A0E0D, 0xFF171918, 0xFF131514, 0xFF121413, 0xFF141414, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0E130F, 0xFF0E130F, 0xFF0A0F09, 0xFF141710, 0xFF2D3027,
        0xFF404237, 0xFF3E4033, 0xFF36382B, 0xFF262D1D, 0xFF474D43, 0xFFF7F9F4, 0xFFDADCD1, 0xFF91957C, 0xFFA8AC8B, 0xFFC2C2A6, 0xFFEAE9D5, 0xFFF1EAE0, 0xFFECE7D1, 0xFFE5E5C3, 0xFFCFD0B0, 0xFFACAE99, 0xFFE6E7DF, 0xFFF2F3EE, 0xFFF7FAF3, 0xFFE6E5E0,
        0xFFB0B1AB, 0xFFF3F6EF, 0xFFF3F9EF, 0xFFE4E8D9, 0xFFE1E3CD, 0xFFD6D2B5, 0xFFD3CEAE, 0xFFDAD7B6, 0xFFE5E2C1, 0xFFEDEAC9, 0xFFE6E3C2, 0xFFD5D2B1, 0xFFC8C5A4, 0xFFC5C2A1, 0xFFC9C6A5, 0xFFE3E0BF, 0xFFE6E3C2, 0xFFE4E1C0, 0xFFDAD7B6, 0xFFCDCAA9,
        0xFFC7C4A3, 0xFFCCC9A8, 0xFFD3D0AF, 0xFFDEDBBA, 0xFFEAE7C6, 0xFFF6F3D2, 0xFFF8F5D4, 0xFFF0EDCC, 0xFFE3E0BF, 0xFFDBD8B7, 0xFFD8D5B4, 0xFFD0CDAC, 0xFFD5D2B1, 0xFFDEDBBC, 0xFFE8E5C6, 0xFFEFECCD, 0xFFF4F0D3, 0xFFF5F1D4, 0xFFF5F1D4, 0xFFDFECDB,
        0xFFDAE6DA, 0xFFD4DFDB, 0xFFCED9DD, 0xFFCED8E1, 0xFFD3E0E8, 0xFFDEEBF1, 0x0DE6F4F7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFFFF, 0x00FBFDFC, 0x00FDFFFE, 0x00FCFFFF, 0x92A5A9A8, 0xFF121615, 0xFF070D0B, 0xFF060C0A, 0xFF060C08,
        0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF0A0E0F, 0xFF0C1011, 0xFF000405, 0xFF05090A, 0xFF05090A, 0xFF060A0B, 0xFF828687, 0xFFC0C4C5, 0xFF535E64,
        0xFF6B767A, 0xFF050F0E, 0xFF050E09, 0xFF0D140D, 0xFF060B07, 0xFF0B0D0C, 0xFF090A0C, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09,
        0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF040D08, 0xFF040D08, 0xFF070D09, 0xFF070D09, 0xFF080D09, 0xFF080D09, 0xFF090E0A, 0xFF0B0D0A, 0xFF080C0B, 0xFF070C06, 0xFF090E0A, 0xFF070B0C, 0xFF0B0F12, 0xFF141A0E, 0xFF303A15, 0xFF606E31, 0xFF84973B,
        0xFF94A557, 0xFF839058, 0xFF4D5637, 0xFF1E2416, 0xFF0B100C, 0xFF0A0E0D, 0xFF0D1110, 0xFF0A110A, 0xFF030905, 0xFF494F4F, 0xFF9BA0A3, 0xFF03080B, 0xFF0C1212, 0xFF0D130F, 0xFF091009, 0xFF1F2920, 0xFFC9D2D1, 0xFFE2E9F1, 0xFF596066, 0xFF7F8887,
        0xFF8F9895, 0xFFD2D9DF, 0xFF888E9E, 0xFF404B47, 0xFF9EA8A7, 0xFF373F41, 0xFF747C7F, 0xFF4D5256, 0xFF0A1010, 0xFF161B17, 0xFF060B05, 0xFF0E1410, 0xFF0E1410, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0C120E, 0xFF0C120E, 0xFF080E0C,
        0xFF121816, 0xFF0F1513, 0xFF0D1110, 0xFF0C100F, 0xFF0B0D0C, 0xFF0F1110, 0xFF121413, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0E130F, 0xFF0E130F, 0xFF0A0F09, 0xFF151811, 0xFF2E3128,
        0xFF414338, 0xFF3E4033, 0xFF37392C, 0xFF2F3626, 0xFF43493F, 0xFFE0E2DD, 0xFFC1C3B8, 0xFF9B9F86, 0xFFD6DAB9, 0xFFE3E3C7, 0xFFD6D5C1, 0xFFEDE6DC, 0xFFE1DCC6, 0xFFE5E5C3, 0xFFE5E6C6, 0xFFC6C8B3, 0xFFEAEBE3, 0xFFECEDE8, 0xFFF8FBF4, 0xFFE6E5E0,
        0xFFB5B6B0, 0xFFF4F7F0, 0xFFF4FAF0, 0xFFEBEFE0, 0xFFEFF1DB, 0xFFEEEACD, 0xFFF0EBCB, 0xFFE0DDBC, 0xFFD3D0AF, 0xFFC9C6A5, 0xFFCECBAA, 0xFFDCD9B8, 0xFFE4E1C0, 0xFFDFDCBB, 0xFFD6D3B2, 0xFFC1BE9D, 0xFFC2BF9E, 0xFFC9C6A5, 0xFFD6D3B2, 0xFFE4E1C0,
        0xFFECE9C8, 0xFFEBE8C7, 0xFFE7E4C3, 0xFFDAD7B6, 0xFFD1CEAD, 0xFFC9C6A5, 0xFFCBC8A7, 0xFFD8D5B4, 0xFFE7E4C3, 0xFFF3F0CF, 0xFFF8F5D4, 0xFFF6F3D2, 0xFFEFECCB, 0xFFE6E3C4, 0xFFDDDABB, 0xFFD8D5B6, 0xFFD9D5B8, 0xFFDDD9BC, 0xFFE1DDC0, 0xFFEFECCD,
        0xFFF3EFD4, 0xFFF6F4DF, 0xFFF9F6E7, 0xFFF7F5E9, 0xFFF4F1E8, 0x11F0F0E4, 0x11EFEFE3, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFFFF, 0x00FBFDFC, 0x00FDFFFE, 0x00FCFFFF, 0x71B5B9B8, 0xFF181C1B, 0xFF030907, 0xFF070D0B, 0xFF060C08,
        0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF020607, 0xFF000102, 0xFF070B0C, 0xFF15191A, 0xFF030708, 0xFF000102, 0xFF5A5E5F, 0xFFD9DDDE, 0xFF2C373D,
        0xFF162125, 0xFF000807, 0xFF0A130E, 0xFF020902, 0xFF050A06, 0xFF060807, 0xFF0C0D0F, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09,
        0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF040D08, 0xFF040D08, 0xFF070D09, 0xFF070D09, 0xFF080D09, 0xFF080D09, 0xFF090E0A, 0xFF0B0D0A, 0xFF080D09, 0xFF080D09, 0xFF080C0B, 0xFF080C0F, 0xFF0C1011, 0xFF121A05, 0xFF3D4A15, 0xFF8B9C4C, 0xFFADC646,
        0xFFC0D669, 0xFFADBE70, 0xFF6D784D, 0xFF2F3626, 0xFF0F1410, 0xFF060A0D, 0xFF080C0F, 0xFF0A110A, 0xFF020804, 0xFF4D5353, 0xFF979C9F, 0xFF03080B, 0xFF0B1111, 0xFF0D130F, 0xFF080F08, 0xFF263027, 0xFFC7D0CF, 0xFFD9E0E8, 0xFF535A60, 0xFF848D8C,
        0xFF717A77, 0xFF656C72, 0xFF9197A7, 0xFF4D5854, 0xFF838D8C, 0xFF3A4244, 0xFFB6BEC1, 0xFF585D61, 0xFF010707, 0xFF101511, 0xFF0E130D, 0xFF0E1410, 0xFF0E1410, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0C120E, 0xFF0C120E, 0xFF0E1714,
        0xFF0A100E, 0xFF0E1412, 0xFF121816, 0xFF0D1110, 0xFF0E1211, 0xFF121413, 0xFF090B0A, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0F1410, 0xFF0E130F, 0xFF0B100A, 0xFF171A13, 0xFF30332A,
        0xFF414338, 0xFF3E4033, 0xFF37392C, 0xFF343B2B, 0xFF494F45, 0xFFE4E6E1, 0xFFC0C2B7, 0xFF8C9077, 0xFFAEB291, 0xFFCFCFB3, 0xFFEBEAD6, 0xFFF0E9DF, 0xFFEFEAD4, 0xFFE7E7C5, 0xFFC6C7A7, 0xFFAAAC97, 0xFFE7E8E0, 0xFFF4F5F0, 0xFFF2F5EE, 0xFFDBDAD5,
        0xFFB6B7B1, 0xFFF7FAF3, 0xFFF5FBF1, 0xFFE8ECDD, 0xFFE2E4CE, 0xFFD7D3B6, 0xFFD4CFAF, 0xFFDCD9B8, 0xFFE9E6C5, 0xFFF2EFCE, 0xFFEAE7C6, 0xFFD5D2B1, 0xFFC5C2A1, 0xFFC3C09F, 0xFFC8C5A4, 0xFFE0DDBC, 0xFFDFDCBB, 0xFFDAD7B6, 0xFFD0CDAC, 0xFFC6C3A2,
        0xFFC4C1A0, 0xFFCBC8A7, 0xFFD2CFAE, 0xFFE2DFBE, 0xFFEDEAC9, 0xFFF5F2D1, 0xFFF0EDCC, 0xFFE1DEBD, 0xFFD2CFAE, 0xFFCCC9A8, 0xFFCCC9A8, 0xFFD0CDAC, 0xFFDAD7B6, 0xFFE8E5C6, 0xFFF3F0D1, 0xFFF7F4D5, 0xFFF3EFD2, 0xFFEDE9CC, 0xFFE8E4C7, 0xFFDFD7B3,
        0xFFE1DBB9, 0xFFE7E0C3, 0xFFEDE6CC, 0xFFF3EDD7, 0xFFFAF3E1, 0x08FFFBE8, 0x00FFFEEE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFFFF, 0x00FCFEFD, 0x00FEFFFF, 0x00FCFFFF, 0x54C3C7C6, 0xFF1D2120, 0xFF000503, 0xFF090F0D, 0xFF060C08,
        0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF333939, 0xFF505656, 0xFF020808, 0xFF010707, 0xFF0D1313, 0xFF010707, 0xFF515757, 0xFFDCE2E2, 0xFF4C575D,
        0xFF000307, 0xFF06100F, 0xFF08110C, 0xFF040B04, 0xFF0B100C, 0xFF0C0E0D, 0xFF0A0B0D, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09,
        0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF040D08, 0xFF040D08, 0xFF070D09, 0xFF070D09, 0xFF080D09, 0xFF080D09, 0xFF090E0A, 0xFF0B0D0A, 0xFF0A0F0B, 0xFF080D09, 0xFF080C0B, 0xFF0A0E11, 0xFF0B0F10, 0xFF0B1300, 0xFF3D4A15, 0xFF98A959, 0xFFB7D14C,
        0xFFBFD566, 0xFFA6B767, 0xFF6A754A, 0xFF313826, 0xFF101413, 0xFF090D10, 0xFF0E1215, 0xFF091009, 0xFF010703, 0xFF515757, 0xFF92979A, 0xFF02070A, 0xFF0A1010, 0xFF0E1410, 0xFF060D06, 0xFF263027, 0xFFD2DBDA, 0xFFDEE5ED, 0xFF4F565C, 0xFF1E2726,
        0xFFB4BDBA, 0xFFC2C9CF, 0xFF696F7F, 0xFF46514D, 0xFF8D9796, 0xFF3D4547, 0xFFC8D0D3, 0xFF5A5F63, 0xFF121818, 0xFF151A16, 0xFF0D120C, 0xFF0E1410, 0xFF0E1410, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0C120E, 0xFF0C120E, 0xFF0C1512,
        0xFF07100D, 0xFF0A1310, 0xFF0F1513, 0xFF0D1311, 0xFF0D1110, 0xFF0D1110, 0xFF0E1211, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0F1410, 0xFF0D120E, 0xFF0B100A, 0xFF191C15, 0xFF32352C,
        0xFF414338, 0xFF3D3F32, 0xFF37392C, 0xFF242B1B, 0xFF5A6056, 0xFFEBEDE8, 0xFFB6B8AD, 0xFFA5A990, 0xFFDADEBD, 0xFFE1E1C5, 0xFFD6D5C1, 0xFFE9E2D8, 0xFFDED9C3, 0xFFE3E3C1, 0xFFE0E1C1, 0xFFCDCFBA, 0xFFEEEFE7, 0xFFF0F1EC, 0xFFFAFDF6, 0xFFDBDAD5,
        0xFFBCBDB7, 0xFFF6F9F2, 0xFFF1F7ED, 0xFFEAEEDF, 0xFFEBEDD7, 0xFFEBE7CA, 0xFFEFEACA, 0xFFDBD8B7, 0xFFCFCCAB, 0xFFC7C4A3, 0xFFCECBAA, 0xFFE0DDBC, 0xFFEAE7C6, 0xFFE6E3C2, 0xFFDDDAB9, 0xFFC0BD9C, 0xFFC2BF9E, 0xFFCBC8A7, 0xFFD9D6B5, 0xFFE6E3C2,
        0xFFEBE8C7, 0xFFE7E4C3, 0xFFE0DDBC, 0xFFCFCCAB, 0xFFC9C6A5, 0xFFC6C3A2, 0xFFCECBAA, 0xFFDDDAB9, 0xFFEBE8C7, 0xFFF1EECD, 0xFFF2EFCE, 0xFFE9E6C5, 0xFFE2DFBE, 0xFFD6D3B4, 0xFFCDCAAB, 0xFFCBC8A9, 0xFFD2CEB1, 0xFFDFDBBE, 0xFFE9E5C8, 0xFFF6F3D4,
        0xFFF5F1D4, 0xFFF4F0D3, 0xFFEFEDD4, 0x2AEEECD7, 0x26EDECDA, 0x19EFEDE0, 0x11F2F0E4, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFFFF, 0x00FDFFFE, 0x00FEFFFF, 0x00FBFFFE, 0x32D3D7D6, 0xFF282C2B, 0xFF000503, 0xFF090F0D, 0xFF070D09,
        0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF3E4444, 0xFFB8BEBE, 0xFF626868, 0xFF080E0E, 0xFF040A0A, 0xFF000202, 0xFF323838, 0xFFC7CDCD, 0xFF59646A,
        0xFF010C10, 0xFF06100F, 0xFF000601, 0xFF080F08, 0xFF050A06, 0xFF0F1110, 0xFF060709, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09,
        0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF040D08, 0xFF040D08, 0xFF070D09, 0xFF070D09, 0xFF080D09, 0xFF080D09, 0xFF090E0A, 0xFF0B0D0A, 0xFF0B0F0E, 0xFF080D07, 0xFF090E0A, 0xFF0A0E0F, 0xFF0B0F12, 0xFF070D01, 0xFF2D3712, 0xFF7A884B, 0xFFAFC45D,
        0xFFA9BC63, 0xFF869456, 0xFF505A37, 0xFF232A1A, 0xFF0B100C, 0xFF080C0D, 0xFF101417, 0xFF091009, 0xFF000602, 0xFF555B5B, 0xFF8E9396, 0xFF010609, 0xFF080E0E, 0xFF0E1410, 0xFF050C05, 0xFF252F26, 0xFFCAD3D2, 0xFFCAD1D9, 0xFF565D63, 0xFF313A39,
        0xFF616A67, 0xFF959CA2, 0xFF292F3F, 0xFF5A6561, 0xFF7E8887, 0xFF3A4244, 0xFFDDE5E8, 0xFF686D71, 0xFF040A0A, 0xFF0A0F0B, 0xFF111610, 0xFF0E1410, 0xFF0E1410, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0C120E, 0xFF0C120E, 0xFF0C1713,
        0xFF0F1815, 0xFF0C1512, 0xFF060F0C, 0xFF0D1311, 0xFF0D1311, 0xFF090D0C, 0xFF121615, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0F1410, 0xFF0D120E, 0xFF0B100A, 0xFF1B1E17, 0xFF34372E,
        0xFF424439, 0xFF3D3F32, 0xFF37392C, 0xFF252C1C, 0xFF62685E, 0xFFEEF0EB, 0xFFB3B5AA, 0xFF989C83, 0xFFB3B796, 0xFFC9C9AD, 0xFFECEBD7, 0xFFEDE6DC, 0xFFEDE8D2, 0xFFE5E5C3, 0xFFC1C2A2, 0xFFB1B39E, 0xFFEBECE4, 0xFFF7F8F3, 0xFFF3F6EF, 0xFFD3D2CD,
        0xFFBEBFB9, 0xFFF9FCF5, 0xFFF1F7ED, 0xFFE6EADB, 0xFFDBDDC7, 0xFFD1CDB0, 0xFFCFCAAA, 0xFFE2DFBE, 0xFFEDEAC9, 0xFFF3F0CF, 0xFFE9E6C5, 0xFFD4D1B0, 0xFFC5C2A1, 0xFFC5C2A1, 0xFFCBC8A7, 0xFFE2DFBE, 0xFFE0DDBC, 0xFFDAD7B6, 0xFFCFCCAB, 0xFFC6C3A2,
        0xFFC5C2A1, 0xFFCCC9A8, 0xFFD4D1B0, 0xFFE9E6C5, 0xFFEFECCB, 0xFFF0EDCC, 0xFFE7E4C3, 0xFFD7D4B3, 0xFFCBC8A7, 0xFFC9C6A5, 0xFFCBC8A7, 0xFFDBD8B7, 0xFFE3E0BF, 0xFFEDEACB, 0xFFF2EFD0, 0xFFEFECCD, 0xFFE7E3C6, 0xFFDFDBBE, 0xFFDAD6B9, 0xFFD5D5BD,
        0xFFDBDBC3, 0xFFE4E4CC, 0xFFEBEDD5, 0x19F2F4DF, 0x00F7F9EB, 0x00FBFCF4, 0x00FEFFFA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFFFF, 0x00FCFEFD, 0x00FEFFFF, 0x00FAFEFD, 0x11E3E7E6, 0xFF373B3A, 0xFF030907, 0xFF080E0C, 0xFF070D09,
        0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF000807, 0xFF697271, 0xFF8E9796, 0xFF040D0C, 0xFF091211, 0xFF0D1615, 0xFF000302, 0xFF606968, 0xFF919CA2,
        0xFF030E12, 0xFF050F0E, 0xFF0A130E, 0xFF070E07, 0xFF0F1410, 0xFF040605, 0xFF090A0C, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09,
        0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF040D08, 0xFF040D08, 0xFF070D09, 0xFF070D09, 0xFF080D09, 0xFF080D09, 0xFF090E0A, 0xFF0B0D0A, 0xFF090D0C, 0xFF090E08, 0xFF0A0F09, 0xFF080C0D, 0xFF0A0D14, 0xFF080C0D, 0xFF171E0E, 0xFF414B29, 0xFF829055,
        0xFF7A8654, 0xFF5C6641, 0xFF323A22, 0xFF161C10, 0xFF090E08, 0xFF060A09, 0xFF090D0C, 0xFF091009, 0xFF000501, 0xFF585E5E, 0xFF8B9093, 0xFF010609, 0xFF070D0D, 0xFF0E1410, 0xFF040B04, 0xFF2A342B, 0xFFCCD5D4, 0xFFDAE1E9, 0xFF495056, 0xFF899291,
        0xFF969F9C, 0xFFDCE3E9, 0xFF6E7484, 0xFF545F5B, 0xFF747E7D, 0xFF444C4E, 0xFFBCC4C7, 0xFF707579, 0xFF0C1212, 0xFF111612, 0xFF0E130D, 0xFF0E1410, 0xFF0E1410, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0C120E, 0xFF0C120E, 0xFF131E1A,
        0xFF15201C, 0xFF1F2825, 0xFF121B18, 0xFF080E0C, 0xFF101614, 0xFF0F1513, 0xFF0A0E0D, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0F1410, 0xFF0D120E, 0xFF0B100A, 0xFF1C1F18, 0xFF363930,
        0xFF424439, 0xFF3D3F32, 0xFF37392C, 0xFF242B1B, 0xFF696F65, 0xFFE9EBE6, 0xFFA7A99E, 0xFFA9AD94, 0xFFDADEBD, 0xFFDEDEC2, 0xFFDAD9C5, 0xFFE8E1D7, 0xFFE2DDC7, 0xFFE5E5C3, 0xFFD6D7B7, 0xFFCBCDB8, 0xFFEFF0E8, 0xFFF1F2ED, 0xFFF5F8F1, 0xFFCFCEC9,
        0xFFBFC0BA, 0xFFF8FBF4, 0xFFF1F7ED, 0xFFEDF1E2, 0xFFEBEDD7, 0xFFEBE7CA, 0xFFEFEACA, 0xFFD4D1B0, 0xFFCECBAA, 0xFFCECBAA, 0xFFD8D5B4, 0xFFE7E4C3, 0xFFECE9C8, 0xFFE4E1C0, 0xFFD9D6B5, 0xFFC2BF9E, 0xFFC3C09F, 0xFFC9C6A5, 0xFFD8D5B4, 0xFFE6E3C2,
        0xFFEAE7C6, 0xFFE3E0BF, 0xFFD9D6B5, 0xFFC7C4A3, 0xFFC6C3A2, 0xFFCAC7A6, 0xFFD2CFAE, 0xFFDFDCBB, 0xFFEAE7C6, 0xFFEFECCB, 0xFFF1EECD, 0xFFD5D2B1, 0xFFD3D0AF, 0xFFD0CDAE, 0xFFCDCAAB, 0xFFD0CDAE, 0xFFDAD6B9, 0xFFE9E5C8, 0xFFF4F0D3, 0xFFF2EDD9,
        0xFFF0EBD5, 0xFFEBE9D0, 0xFFE8E6CD, 0x36E8E7D2, 0x1DECEADD, 0x00F3F0EB, 0x00F6F5F3, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFFFF, 0x00FCFEFD, 0x00FEFFFF, 0x00F9FDFC, 0x00EFF3F2, 0xFF434746, 0xFF070D0B, 0xFF060C0A, 0xFF070D09,
        0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF030C0B, 0xFF050E0D, 0xFF141D1C, 0xFF000807, 0xFF848D8C, 0xFF8E9796, 0xFF4B5453, 0xFF121B1A, 0xFF96A1A7,
        0xFF4E595D, 0xFF000807, 0xFF000500, 0xFF080F08, 0xFF080D09, 0xFF070908, 0xFF0D0E10, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09,
        0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF040D08, 0xFF040D08, 0xFF070D09, 0xFF070D09, 0xFF080D09, 0xFF080D09, 0xFF090E0A, 0xFF0B0D0A, 0xFF060A0B, 0xFF0A0F09, 0xFF0B1009, 0xFF060A09, 0xFF0A0D16, 0xFF0B0E17, 0xFF070B0A, 0xFF141A0C, 0xFF252E13,
        0xFF2A321A, 0xFF1F2712, 0xFF0C1204, 0xFF070D03, 0xFF0B1009, 0xFF0B100A, 0xFF0B100C, 0xFF091009, 0xFF000501, 0xFF5A6060, 0xFF898E91, 0xFF010609, 0xFF070D0D, 0xFF0F1511, 0xFF040B04, 0xFF2F3930, 0xFFCDD6D5, 0xFFD4DBE3, 0xFF3B4248, 0xFF838C8B,
        0xFF68716E, 0xFF676E74, 0xFF808696, 0xFF56615D, 0xFF7A8483, 0xFF555D5F, 0xFFC5CDD0, 0xFFCACFD3, 0xFF333939, 0xFF020703, 0xFF111610, 0xFF0E1410, 0xFF0E1410, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0D130F, 0xFF0C120E, 0xFF0C120E, 0xFF6E7975,
        0xFF7F8A86, 0xFFA7B2AE, 0xFF78817E, 0xFF141D1A, 0xFF040A08, 0xFF181E1C, 0xFF0D1311, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0F1410, 0xFF0C110D, 0xFF0C110B, 0xFF1D2019, 0xFF363930,
        0xFF424439, 0xFF3C3E31, 0xFF37392C, 0xFF232A1A, 0xFF6E746A, 0xFFF3F5F0, 0xFFA5A79C, 0xFF8E9279, 0xFFB4B897, 0xFFD3D3B7, 0xFFEEEDD9, 0xFFF4EDE3, 0xFFEDE8D2, 0xFFE3E3C1, 0xFFC2C3A3, 0xFFB7B9A4, 0xFFEAEBE3, 0xFFF6F7F2, 0xFFF8FBF4, 0xFFCFCEC9,
        0xFFC3C4BE, 0xFFFAFDF6, 0xFFEFF5EB, 0xFFE6EADB, 0xFFDCDEC8, 0xFFD6D2B5, 0xFFD6D1B1, 0xFFE4E1C0, 0xFFEDEAC9, 0xFFF1EECD, 0xFFE6E3C2, 0xFFD4D1B0, 0xFFC7C4A3, 0xFFC9C6A5, 0xFFCFCCAB, 0xFFE3E0BF, 0xFFE1DEBD, 0xFFD9D6B5, 0xFFCBC8A7, 0xFFC0BD9C,
        0xFFC2BF9E, 0xFFD2CFAE, 0xFFE0DDBC, 0xFFEFECCB, 0xFFEDEAC9, 0xFFE8E5C4, 0xFFDEDBBA, 0xFFD4D1B0, 0xFFCCC9A8, 0xFFCAC7A6, 0xFFCBC8A7, 0xFFE3E0BF, 0xFFE9E6C5, 0xFFEFECCD, 0xFFEEEBCC, 0xFFE6E3C4, 0xFFDAD6B9, 0xFFD1CDB0, 0xFFCCC8AB, 0xFFD9CEBA,
        0xFFE0D8C1, 0xFFEEE7CB, 0xFFF9F2D6, 0xFFFDF7DF, 0x08FEF7E7, 0x00FBF4EC, 0x00F9F1EF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FDFDFD, 0x00FFFFFF, 0x00FEFFFF, 0x00F7F9F8, 0xFF606463, 0xFF030706, 0xFF040807, 0xFF070D0B,
        0xFF070D0B, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF070D0B, 0xFF060C0A, 0xFF060C0A, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF0C0F08, 0xFF050A04, 0xFF090E0A, 0xFF020605, 0xFF515659, 0xFFCACFD5, 0xFFCED5DD, 0xFFA7AEB6, 0xFF899297,
        0xFFAFB9BB, 0xFF1F292A, 0xFF000804, 0xFF05100A, 0xFF030F05, 0xFF061206, 0xFF000A00, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF070D0B, 0xFF080E0C, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08,
        0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0B0F0E,
        0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF091009, 0xFF010703, 0xFF5B6161, 0xFF878C8F, 0xFF010609, 0xFF0A1010, 0xFF0C120E, 0xFF020902, 0xFF323C3D, 0xFFC7D1D2, 0xFFD5DFE1, 0xFF414B4D, 0xFF2F383D,
        0xFFB0B9BE, 0xFFCBD4DB, 0xFF475057, 0xFF636D6C, 0xFF6D7778, 0xFF3C4447, 0xFFB4BBC1, 0xFF94999F, 0xFF292E31, 0xFF090D0C, 0xFF10150F, 0xFF0D120E, 0xFF0F1410, 0xFF0E130F, 0xFF0C110D, 0xFF0E130F, 0xFF101511, 0xFF0D120E, 0xFF080D09, 0xFF84898D,
        0xFF3D4246, 0xFF5B6063, 0xFF858A8D, 0xFF1D2225, 0xFF0C1212, 0xFF141A1A, 0xFF131919, 0xFF0D1611, 0xFF061008, 0xFF0D140C, 0xFF151D12, 0xFF0B1009, 0xFF0A0F0B, 0xFF15191A, 0xFF0E0F14, 0xFF0E100D, 0xFF141613, 0xFF0D0F0A, 0xFF1E211A, 0xFF3A3D34,
        0xFF404338, 0xFF393D2F, 0xFF333729, 0xFF1C2313, 0xFF81847B, 0xFFEEEFE9, 0xFFA9AC9B, 0xFFB7BA9D, 0xFFD3D4B4, 0xFFD6D4BD, 0xFFDDD9CD, 0xFFE0DACA, 0xFFE3DFC4, 0xFFE8E8C6, 0xFFD3D3B7, 0xFFC8C9BB, 0xFFF1F2ED, 0xFFF3F4EF, 0xFFF4F7EE, 0xFFC5C8BF,
        0xFFCCCDC7, 0xFFFDFEF6, 0xFFEEEEE4, 0xFFF1EFE0, 0xFFEFEDD6, 0xFFE9E6C7, 0xFFE5E2BF, 0xFFD3D0AF, 0xFFCDCAA9, 0xFFCCC9A8, 0xFFD7D4B3, 0xFFE6E3C2, 0xFFECE9C8, 0xFFE3E0BF, 0xFFD8D5B4, 0xFFBEBB9A, 0xFFC2BF9E, 0xFFCDCAA9, 0xFFDCD9B8, 0xFFE8E5C4,
        0xFFE7E4C3, 0xFFD9D6B5, 0xFFCBC8A7, 0xFFBFBC9B, 0xFFC5C2A1, 0xFFD2CFAE, 0xFFDFDCBB, 0xFFE9E6C5, 0xFFECE9C8, 0xFFEAE7C6, 0xFFE6E3C2, 0xFFD2CFAE, 0xFFCCC9A8, 0xFFC8C5A4, 0xFFCAC7A6, 0xFFD5D2B1, 0xFFE3E0BF, 0xFFEFECCB, 0xFFF4F1D0, 0xFFEAE6CB,
        0xFFE6E2C7, 0xFFDEDCC5, 0xFFDDDBC6, 0xFFE0DFCD, 0x26E9E7DA, 0x0DF4F2E6, 0x00F9F9EF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFCFC, 0x00FFFFFF, 0x00FDFFFE, 0x00FCFEFD, 0xFF6F7372, 0xFF050908, 0xFF050908, 0xFF070D0B,
        0xFF070D0B, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF070D0B, 0xFF070D0B, 0xFF060C0A, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF0B0E07, 0xFF030802, 0xFF070C08, 0xFF030706, 0xFF000508, 0xFF62676B, 0xFFC3CAD0, 0xFFA0A7AF, 0xFFB9C2C9,
        0xFFCFD8DD, 0xFF9BA5A7, 0xFF0F191A, 0xFF000905, 0xFF030E08, 0xFF08130B, 0xFF041006, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF060C0A, 0xFF050B09, 0xFF040A08, 0xFF040A08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08,
        0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D,
        0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF050C05, 0xFF000501, 0xFF5C6262, 0xFF8A8F92, 0xFF03080B, 0xFF090F0F, 0xFF0C120E, 0xFF060D06, 0xFF2E3839, 0xFFD5DFE0, 0xFFC4CED0, 0xFF414B4D, 0xFF666F74,
        0xFF5F686D, 0xFF515A61, 0xFF2C353C, 0xFF697372, 0xFF697374, 0xFF596164, 0xFFCDD4DA, 0xFF9CA1A7, 0xFF181D20, 0xFF0A0E0D, 0xFF0E130D, 0xFF0D120E, 0xFF0F1410, 0xFF0E130F, 0xFF0D120E, 0xFF0E130F, 0xFF0F1410, 0xFF0D120E, 0xFF090E0A, 0xFF5A5F63,
        0xFF888D91, 0xFF505558, 0xFF909598, 0xFF272C2F, 0xFF080E0E, 0xFF0F1515, 0xFF070D0D, 0xFF060F0C, 0xFF131C17, 0xFF0E150D, 0xFF0B120A, 0xFF0C110A, 0xFF0C110D, 0xFF121617, 0xFF0D0E12, 0xFF0E100D, 0xFF131512, 0xFF0D0F0A, 0xFF1E211A, 0xFF3A3D34,
        0xFF3F4237, 0xFF383C2E, 0xFF313527, 0xFF2A3121, 0xFF979A91, 0xFFF4F5EF, 0xFF9C9F8E, 0xFF9A9D80, 0xFFBFC0A0, 0xFFDEDCC5, 0xFFEDE9DD, 0xFFF2ECDC, 0xFFE5E1C6, 0xFFDADAB8, 0xFFBFBFA3, 0xFFC0C1B3, 0xFFF0F1EC, 0xFFF6F7F2, 0xFFF5F8EF, 0xFFC4C7BE,
        0xFFCBCCC6, 0xFFFBFCF4, 0xFFEFEFE5, 0xFFE8E6D7, 0xFFD7D5BE, 0xFFD3D0B1, 0xFFDAD7B4, 0xFFE7E4C3, 0xFFEEEBCA, 0xFFF1EECD, 0xFFE6E3C2, 0xFFD4D1B0, 0xFFCAC7A6, 0xFFCCC9A8, 0xFFD3D0AF, 0xFFEBE8C7, 0xFFE3E0BF, 0xFFD5D2B1, 0xFFC5C2A1, 0xFFBEBB9A,
        0xFFC6C3A2, 0xFFD9D6B5, 0xFFE9E6C5, 0xFFF4F1D0, 0xFFEBE8C7, 0xFFDEDBBA, 0xFFD2CFAE, 0xFFCCC9A8, 0xFFCCC9A8, 0xFFCFCCAB, 0xFFD1CEAD, 0xFFE9E6C5, 0xFFEDEAC9, 0xFFEFECCB, 0xFFEBE8C7, 0xFFE0DDBC, 0xFFD4D1B0, 0xFFCCC9A8, 0xFFC8C5A4, 0xFFDAD6BB,
        0xFFE5E1C6, 0xFFF2F0D9, 0xFFF9F7E2, 0xFFF7F6E4, 0xFFF5F3E6, 0x08F6F4E8, 0x00F6F6EC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFCFC, 0x00FFFFFF, 0x00FCFEFD, 0x00FEFFFF, 0xD9848887, 0xFF070B0A, 0xFF070B0A, 0xFF070D0B,
        0xFF070D0B, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060803, 0xFF0D120C, 0xFF0B100C, 0xFF131716, 0xFF000202, 0xFF090E11, 0xFF656D70, 0xFF8B9298, 0xFF1E252D,
        0xFF6D747C, 0xFFB6BDC3, 0xFF9FA7AA, 0xFF40484A, 0xFF000908, 0xFF000502, 0xFF0A1310, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF060C0A, 0xFF040A08, 0xFF030907, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08,
        0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D,
        0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF091009, 0xFF000501, 0xFF565C5C, 0xFF83888B, 0xFF000508, 0xFF090F0F, 0xFF0B110D, 0xFF050C05, 0xFF394344, 0xFFCAD4D5, 0xFFCBD5D7, 0xFF3A4446, 0xFF818A8F,
        0xFFA8B1B6, 0xFF868F96, 0xFF5C656C, 0xFF646E6D, 0xFF697374, 0xFF4C5457, 0xFFBAC1C7, 0xFFB8BDC3, 0xFF313639, 0xFF0C100F, 0xFF0E130D, 0xFF0D120E, 0xFF0E130F, 0xFF0E130F, 0xFF0D120E, 0xFF0D120E, 0xFF0F1410, 0xFF0D120E, 0xFF0A0F0B, 0xFF02070B,
        0xFF51565A, 0xFF909598, 0xFF5E6366, 0xFF0F1417, 0xFF080E0E, 0xFF202626, 0xFF101616, 0xFF101918, 0xFF0E1714, 0xFF020902, 0xFF0B120A, 0xFF191E17, 0xFF0D120E, 0xFF090D0E, 0xFF111216, 0xFF0D0F0C, 0xFF121411, 0xFF0D0F0A, 0xFF1E211A, 0xFF393C33,
        0xFF3E4136, 0xFF373B2D, 0xFF2F3325, 0xFF353C2C, 0xFFBEC1B8, 0xFFECEDE7, 0xFF949786, 0xFFBBBEA1, 0xFFD7D8B8, 0xFFD5D3BC, 0xFFDDD9CD, 0xFFE3DDCD, 0xFFE6E2C7, 0xFFEBEBC9, 0xFFD2D2B6, 0xFFD1D2C4, 0xFFF2F3EE, 0xFFF1F2ED, 0xFFEFF2E9, 0xFFC0C3BA,
        0xFFD5D6D0, 0xFFFCFDF5, 0xFFEBEBE1, 0xFFF1EFE0, 0xFFF6F4DD, 0xFFECE9CA, 0xFFDCD9B6, 0xFFD3D0AF, 0xFFCDCAA9, 0xFFCECBAA, 0xFFDBD8B7, 0xFFEAE7C6, 0xFFEDEAC9, 0xFFDFDCBB, 0xFFCFCCAB, 0xFFB6B392, 0xFFC1BE9D, 0xFFD3D0AF, 0xFFE4E1C0, 0xFFEAE7C6,
        0xFFE2DFBE, 0xFFD1CEAD, 0xFFC3C09F, 0xFFC0BD9C, 0xFFCAC7A6, 0xFFD9D6B5, 0xFFE6E3C2, 0xFFECE9C8, 0xFFEAE7C6, 0xFFE6E3C2, 0xFFE2DFBE, 0xFFCFCCAB, 0xFFC9C6A5, 0xFFC6C3A2, 0xFFCFCCAB, 0xFFDFDCBB, 0xFFEDEAC9, 0xFFF1EECD, 0xFFF0EDCC, 0xFFE3DFC4,
        0xFFDCD8BD, 0xFFD6D4BD, 0xFFDAD8C3, 0xFFE5E4D2, 0xFFF1EFE2, 0x04F8F6EA, 0x00F8F8EE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFCFC, 0x00FFFFFF, 0x00FCFEFD, 0x00FEFFFF, 0xAB999D9C, 0xFF0B0F0E, 0xFF080C0B, 0xFF070D0B,
        0xFF070D0B, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF070D0B, 0xFF080E0C, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF0B0D08, 0xFF0A0F0B, 0xFF000300, 0xFF000302, 0xFF0B1111, 0xFF000202, 0xFF1A2224, 0xFF81898B, 0xFF666D77,
        0xFF000109, 0xFF212830, 0xFFA4ABB1, 0xFFD3DAE0, 0xFF6D7578, 0xFF192123, 0xFF000204, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF070D0B, 0xFF080E0C, 0xFF080E0C, 0xFF070D0B, 0xFF070D0B, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08,
        0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C,
        0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF070E07, 0xFF010703, 0xFF585E5E, 0xFF82878A, 0xFF000508, 0xFF090F0F, 0xFF0A100C, 0xFF030A03, 0xFF364041, 0xFFCBD5D6, 0xFFD2DCDE, 0xFF374143, 0xFF6B7479,
        0xFFCCD5DA, 0xFFC5CED5, 0xFF677077, 0xFF697372, 0xFF5E6869, 0xFF4D5558, 0xFFAAB1B7, 0xFF5D6268, 0xFF000104, 0xFF0D1110, 0xFF0A0F09, 0xFF0D120E, 0xFF0E130F, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0E130F, 0xFF0D120E, 0xFF0C110D, 0xFF5A5F63,
        0xFF858A8E, 0xFFAAAFB2, 0xFF73787B, 0xFF12171A, 0xFF090F0F, 0xFF969C9C, 0xFF636969, 0xFF01090C, 0xFF060F0E, 0xFF161C18, 0xFF0D140D, 0xFF040903, 0xFF0E130F, 0xFF111514, 0xFF0F1012, 0xFF0D0F0C, 0xFF10120F, 0xFF0D0F0A, 0xFF1F221B, 0xFF383B32,
        0xFF3C3F34, 0xFF363A2C, 0xFF2C3022, 0xFF4A5141, 0xFFCDD0C7, 0xFFE9EAE4, 0xFF8F9281, 0xFF9B9E81, 0xFFBEBF9F, 0xFFE1DFC8, 0xFFE8E4D8, 0xFFF5EFDF, 0xFFE4E0C5, 0xFFD4D4B2, 0xFFB4B498, 0xFFC9CABC, 0xFFF3F4EF, 0xFFF4F5F0, 0xFFEEF1E8, 0xFFBCBFB6,
        0xFFD1D2CC, 0xFFF9FAF2, 0xFFF5F5EB, 0xFFEAE8D9, 0xFFD0CEB7, 0xFFCECBAC, 0xFFDEDBB8, 0xFFEAE7C6, 0xFFF1EECD, 0xFFF2EFCE, 0xFFE4E1C0, 0xFFD0CDAC, 0xFFC7C4A3, 0xFFCFCCAB, 0xFFDAD7B6, 0xFFEFECCB, 0xFFE2DFBE, 0xFFCECBAA, 0xFFC0BD9C, 0xFFBFBC9B,
        0xFFCCC9A8, 0xFFDFDCBB, 0xFFECE9C8, 0xFFEDEAC9, 0xFFE5E2C1, 0xFFD8D5B4, 0xFFCCC9A8, 0xFFC7C4A3, 0xFFCBC8A7, 0xFFD6D3B2, 0xFFDEDBBA, 0xFFEEEBCA, 0xFFEFECCB, 0xFFECE9C8, 0xFFE0DDBC, 0xFFD1CEAD, 0xFFC9C6A5, 0xFFCCC9A8, 0xFFD2CFAE, 0xFFDEDABF,
        0xFFEBE7CC, 0xFFF6F4DD, 0xFFF9F7E2, 0xFFF2F1DF, 0x1DEDEBDE, 0x15EFEDE1, 0x08F2F2E8, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFFFE, 0x00FEFFFF, 0x86ACB0AF, 0xFF131716, 0xFF070B0A, 0xFF060C0A,
        0xFF070D0B, 0xFF080E0C, 0xFF080E0C, 0xFF090F0D, 0xFF090F0D, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF060C0A, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08,
        0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF0A0C09, 0xFF040905, 0xFF0E130F, 0xFF070B0A, 0xFF0C1210, 0xFF090F0D, 0xFF000300, 0xFF1F2825, 0xFF9DA1AA,
        0xFF595E64, 0xFF000107, 0xFF171C22, 0xFF8C9197, 0xFFCED3D9, 0xFFA2A7AB, 0xFF1F2428, 0xFF090F0D, 0xFF070D0B, 0xFF050B09, 0xFF050B09, 0xFF060C0A, 0xFF080E0C, 0xFF090F0D, 0xFF080E0C, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08,
        0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C,
        0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF060D06, 0xFF020804, 0xFF565C5C, 0xFF7D8285, 0xFF000205, 0xFF090F0F, 0xFF0B110D, 0xFF050C05, 0xFF333D3E, 0xFFCAD4D5, 0xFFC4CED0, 0xFF353F41, 0xFF3E474C,
        0xFF5E676C, 0xFF9FA8AF, 0xFF202930, 0xFF6A7473, 0xFF626C6D, 0xFF656D70, 0xFFD8DFE5, 0xFF54595F, 0xFF090E11, 0xFF181C1B, 0xFF090E08, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0D120E, 0xFF0E130F, 0xFF63686C,
        0xFFD4D9DD, 0xFFB1B6B9, 0xFF181D20, 0xFF0B1013, 0xFF0D1313, 0xFF9FA5A5, 0xFFF3F9F9, 0xFF41484E, 0xFF030B0E, 0xFF0B110F, 0xFF121912, 0xFF040903, 0xFF0F140E, 0xFF111514, 0xFF0C0D0F, 0xFF0D0F0C, 0xFF0F110E, 0xFF0E100B, 0xFF20231C, 0xFF373A31,
        0xFF3C3F34, 0xFF35392B, 0xFF292D1F, 0xFF495040, 0xFFE9ECE3, 0xFFE2E3DD, 0xFF949786, 0xFFC5C8AB, 0xFFD4D5B5, 0xFFD2D0B9, 0xFFE4E0D4, 0xFFDDD7C7, 0xFFE5E1C6, 0xFFEFEFCD, 0xFFCBCBAF, 0xFFD8D9CB, 0xFFEFF0EB, 0xFFF1F2ED, 0xFFF3F6ED, 0xFFB9BCB3,
        0xFFE0E1DB, 0xFFF8F9F1, 0xFFEAEAE0, 0xFFF0EEDF, 0xFFF3F1DA, 0xFFEBE8C9, 0xFFD9D6B3, 0xFFCBC8A7, 0xFFC7C4A3, 0xFFC9C6A5, 0xFFD8D5B4, 0xFFE9E6C5, 0xFFEDEAC9, 0xFFE0DDBC, 0xFFD1CEAD, 0xFFB7B493, 0xFFC3C09F, 0xFFD5D2B1, 0xFFE3E0BF, 0xFFE6E3C2,
        0xFFDDDAB9, 0xFFCFCCAB, 0xFFC5C2A1, 0xFFC5C2A1, 0xFFD0CDAC, 0xFFDFDCBB, 0xFFEAE7C6, 0xFFECE9C8, 0xFFE6E3C2, 0xFFDEDBBA, 0xFFD8D5B4, 0xFFC9C6A5, 0xFFC9C6A5, 0xFFCECBAA, 0xFFDAD7B6, 0xFFE9E6C5, 0xFFEFECCB, 0xFFEAE7C6, 0xFFE2DFBE, 0xFFDCD8BD,
        0xFFD7D3B8, 0xFFD4D2BB, 0xFFDCDAC5, 0xFFE9E8D6, 0x0DF5F3E6, 0x00FAF8EC, 0x00F7F7ED, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFFFE, 0x00FEFFFF, 0x5CBFC3C2, 0xFF202423, 0xFF050908, 0xFF060C0A,
        0xFF070D0B, 0xFF080E0C, 0xFF090F0D, 0xFF090F0D, 0xFF090F0D, 0xFF090F0D, 0xFF090F0D, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09,
        0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF0A0C0B, 0xFF050908, 0xFF0A0E0D, 0xFF070C08, 0xFF000400, 0xFF070D09, 0xFF09120D, 0xFF000300, 0xFF33383B,
        0xFF9EA3A6, 0xFF202528, 0xFF000307, 0xFF070C10, 0xFF63686C, 0xFFB9BEC2, 0xFFB0B5B9, 0xFF0E1412, 0xFF0A100E, 0xFF060C0A, 0xFF040A08, 0xFF050B09, 0xFF070D0B, 0xFF070D0B, 0xFF060C0A, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08,
        0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C,
        0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0C130C, 0xFF010703, 0xFF535959, 0xFF84898C, 0xFF0D1215, 0xFF121818, 0xFF080E0A, 0xFF000500, 0xFF2D3738, 0xFFC9D3D4, 0xFFCFD9DB, 0xFF384244, 0xFF899297,
        0xFFA0A9AE, 0xFFC1CAD1, 0xFF454E55, 0xFF697372, 0xFF525C5D, 0xFF363E41, 0xFFB0B7BD, 0xFF383D43, 0xFF000508, 0xFF090D0C, 0xFF0D120C, 0xFF0D120E, 0xFF0C110D, 0xFF0D120E, 0xFF0E130F, 0xFF0D120E, 0xFF0C110D, 0xFF0D120E, 0xFF101511, 0xFF5F6468,
        0xFF2F3438, 0xFF969B9E, 0xFF616669, 0xFF080D10, 0xFF060C0C, 0xFF969C9C, 0xFFF7FDFD, 0xFFCCD3DD, 0xFF3D444A, 0xFF000505, 0xFF101612, 0xFF161B15, 0xFF0D120C, 0xFF0A0F0B, 0xFF121413, 0xFF0E100D, 0xFF0E100D, 0xFF0E100B, 0xFF22251E, 0xFF373A31,
        0xFF3B3E33, 0xFF35392B, 0xFF262A1C, 0xFF575E4E, 0xFFE2E5DC, 0xFFDCDDD7, 0xFF949786, 0xFFA2A588, 0xFFBDBE9E, 0xFFE0DEC7, 0xFFEEEADE, 0xFFEFE9D9, 0xFFE3DFC4, 0xFFD8D8B6, 0xFFADAD91, 0xFFD0D1C3, 0xFFEFF0EB, 0xFFF4F5F0, 0xFFF2F5EC, 0xFFB5B8AF,
        0xFFDCDDD7, 0xFFF4F5ED, 0xFFF3F3E9, 0xFFE8E6D7, 0xFFCDCBB4, 0xFFCDCAAB, 0xFFDCD9B6, 0xFFECE9C8, 0xFFF1EECD, 0xFFEFECCB, 0xFFDFDCBB, 0xFFCBC8A7, 0xFFC3C09F, 0xFFCCC9A8, 0xFFD9D6B5, 0xFFEDEAC9, 0xFFDEDBBA, 0xFFCCC9A8, 0xFFC1BE9D, 0xFFC4C1A0,
        0xFFD1CEAD, 0xFFDFDCBB, 0xFFE8E5C4, 0xFFE8E5C4, 0xFFDFDCBB, 0xFFD3D0AF, 0xFFC9C6A5, 0xFFC7C4A3, 0xFFCFCCAB, 0xFFDCD9B8, 0xFFE5E2C1, 0xFFEEEBCA, 0xFFEDEAC9, 0xFFE6E3C2, 0xFFD7D4B3, 0xFFCAC7A6, 0xFFC7C4A3, 0xFFD1CEAD, 0xFFDBD8B7, 0xFFEBE7CC,
        0xFFF1EDD2, 0xFFF4F2DB, 0xFFF0EED9, 0xFFE9E8D6, 0x26E9E7DA, 0x11F1EFE3, 0x00F8F8EE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFFFE, 0x00FEFFFF, 0x36D1D5D4, 0xFF323635, 0xFF030706, 0xFF060C0A,
        0xFF070D0B, 0xFF080E0C, 0xFF090F0D, 0xFF090F0D, 0xFF0A100E, 0xFF0A100E, 0xFF090F0D, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09,
        0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF080A09, 0xFF090D0C, 0xFF020605, 0xFF070C08, 0xFF070D09, 0xFF050C05, 0xFF0A140C, 0xFF030D05, 0xFF060A09,
        0xFF3A3E3D, 0xFF383C3B, 0xFF040809, 0xFF090D0E, 0xFF05090C, 0xFF282C2F, 0xFF4E5255, 0xFF0D1311, 0xFF0A100E, 0xFF060C0A, 0xFF050B09, 0xFF070D0B, 0xFF080E0C, 0xFF070D0B, 0xFF050B09, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08,
        0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C,
        0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0B120B, 0xFF000300, 0xFF505656, 0xFF83888B, 0xFF010609, 0xFF000202, 0xFF010703, 0xFF151C15, 0xFF8C9697, 0xFFCED8D9, 0xFFC2CCCE, 0xFF394345, 0xFF828B90,
        0xFFBCC5CA, 0xFF9FA8AF, 0xFF626B72, 0xFF6F7978, 0xFF5D6768, 0xFF636B6E, 0xFFD9E0E6, 0xFF4B5056, 0xFF04090C, 0xFF141817, 0xFF090E08, 0xFF0D120E, 0xFF0C110D, 0xFF0C110D, 0xFF0E130F, 0xFF0D120E, 0xFF0B100C, 0xFF0D120E, 0xFF111612, 0xFF9EA3A7,
        0xFF74797D, 0xFF42474A, 0xFF4D5255, 0xFF080D10, 0xFF0B1111, 0xFFA1A7A7, 0xFFF1F7F7, 0xFFF9FFFF, 0xFFB1B8C0, 0xFF363B3E, 0xFF040A08, 0xFF0B100A, 0xFF0B100A, 0xFF131814, 0xFF111310, 0xFF0E100D, 0xFF0E100D, 0xFF0F110C, 0xFF23261F, 0xFF373A31,
        0xFF3B3E33, 0xFF35392B, 0xFF25291B, 0xFF585F4F, 0xFFE9ECE3, 0xFFD1D2CC, 0xFF939685, 0xFFBFC2A5, 0xFFD9DABA, 0xFFDAD8C1, 0xFFDDD9CD, 0xFFE0DACA, 0xFFE3DFC4, 0xFFEAEAC8, 0xFFC0C0A4, 0xFFE1E2D4, 0xFFF1F2ED, 0xFFEFF0EB, 0xFFECEFE6, 0xFFB4B7AE,
        0xFFE7E8E2, 0xFFF4F5ED, 0xFFEDEDE3, 0xFFEFEDDE, 0xFFEBE9D2, 0xFFE7E4C5, 0xFFE0DDBA, 0xFFCDCAA9, 0xFFCCC9A8, 0xFFD0CDAC, 0xFFDEDBBA, 0xFFEBE8C7, 0xFFEBE8C7, 0xFFDBD8B7, 0xFFCBC8A7, 0xFFB6B392, 0xFFC7C4A3, 0xFFDDDAB9, 0xFFE8E5C4, 0xFFE5E2C1,
        0xFFD9D6B5, 0xFFCDCAA9, 0xFFC7C4A3, 0xFFC8C5A4, 0xFFD2CFAE, 0xFFE0DDBC, 0xFFEBE8C7, 0xFFECE9C8, 0xFFE4E1C0, 0xFFD6D3B2, 0xFFCDCAA9, 0xFFC5C2A1, 0xFFCAC7A6, 0xFFD6D3B2, 0xFFE3E0BF, 0xFFEDEAC9, 0xFFEEEBCA, 0xFFE9E6C5, 0xFFE3E0BF, 0xFFCFCBB0,
        0xFFD2CEB3, 0xFFD7D5BE, 0xFFE3E1CC, 0xFFEFEEDC, 0x04F9F7EA, 0x00FBF9ED, 0x00F9F9EF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FBFBFB, 0x00FDFFFE, 0x00FEFFFF, 0x1DDEE2E1, 0xFF3E4241, 0xFF020605, 0xFF060C0A,
        0xFF070D0B, 0xFF080E0C, 0xFF090F0D, 0xFF090F0D, 0xFF0A100E, 0xFF0A100E, 0xFF0A100E, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09,
        0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF0B0D0C, 0xFF070B0A, 0xFF060A09, 0xFF060B07, 0xFF091009, 0xFF0A110A, 0xFF040E05, 0xFF050F06, 0xFF020701,
        0xFF000400, 0xFF0C110D, 0xFF040905, 0xFF090D0C, 0xFF090D0E, 0xFF060A0B, 0xFF000304, 0xFF0A100E, 0xFF070D0B, 0xFF050B09, 0xFF060C0A, 0xFF090F0D, 0xFF0A100E, 0xFF090F0D, 0xFF070D0B, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08,
        0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C,
        0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF091009, 0xFF000501, 0xFF565C5C, 0xFF83888B, 0xFF000205, 0xFF161C1C, 0xFF6B716D, 0xFFCBD2CB, 0xFFC8D2D3, 0xFF5F696A, 0xFF919B9D, 0xFFA6B0B2, 0xFF2D363B,
        0xFF576065, 0xFF6C757C, 0xFF363F46, 0xFF6A7473, 0xFF5A6465, 0xFF343C3F, 0xFF878E94, 0xFF3F444A, 0xFF04090C, 0xFF0C100F, 0xFF0A0F09, 0xFF0D120E, 0xFF0C110D, 0xFF0C110D, 0xFF0E130F, 0xFF0D120E, 0xFF0B100C, 0xFF0D120E, 0xFF121713, 0xFF92979B,
        0xFF676C70, 0xFF909598, 0xFF565B5E, 0xFF111619, 0xFF0C1212, 0xFFA2A8A8, 0xFFF0F6F6, 0xFFEAF0FC, 0xFFECF3FD, 0xFF54595D, 0xFF010705, 0xFF121711, 0xFF090E08, 0xFF101511, 0xFF0A0C09, 0xFF0F110E, 0xFF0E100D, 0xFF10120D, 0xFF242720, 0xFF373A31,
        0xFF3B3E33, 0xFF35392B, 0xFF24281A, 0xFF606757, 0xFFEAEDE4, 0xFFD4D5CF, 0xFF8C8F7E, 0xFF9FA285, 0xFFC7C8A8, 0xFFE4E2CB, 0xFFEDE9DD, 0xFFF3EDDD, 0xFFE6E2C7, 0xFFDBDBB9, 0xFFACAC90, 0xFFD9DACC, 0xFFF1F2ED, 0xFFF2F3EE, 0xFFEDF0E7, 0xFFB4B7AE,
        0xFFE6E7E1, 0xFFF1F2EA, 0xFFEDEDE3, 0xFFE5E3D4, 0xFFD2D0B9, 0xFFD1CEAF, 0xFFD7D4B1, 0xFFECE9C8, 0xFFEFECCB, 0xFFEBE8C7, 0xFFDCD9B8, 0xFFCAC7A6, 0xFFC5C2A1, 0xFFCFCCAB, 0xFFDCD9B8, 0xFFEFECCB, 0xFFDBD8B7, 0xFFC4C1A0, 0xFFBAB796, 0xFFC2BF9E,
        0xFFD3D0AF, 0xFFE1DEBD, 0xFFE8E5C4, 0xFFEBE8C7, 0xFFDFDCBB, 0xFFCFCCAB, 0xFFC6C3A2, 0xFFC8C5A4, 0xFFD3D0AF, 0xFFE2DFBE, 0xFFEBE8C7, 0xFFF2EFCE, 0xFFEAE7C6, 0xFFDBD8B7, 0xFFCCC9A8, 0xFFC4C1A0, 0xFFC8C5A4, 0xFFD4D1B0, 0xFFDEDBBA, 0xFFF6F2D7,
        0xFFF6F2D7, 0xFFF1EFD8, 0xFFECEAD5, 0x2EE8E7D5, 0x21EAE8DB, 0x15F0EEE2, 0x04F3F3E9, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FDFDFD, 0x00FCFCFC, 0x00FFFFFF, 0x00EEEEEE, 0xD9848484, 0xFF020202, 0xFF0C100F,
        0xFF060A09, 0xFF030706, 0xFF070B0A, 0xFF0A0E0D, 0xFF080C0B, 0xFF070B0A, 0xFF090D0C, 0xFF0B110F, 0xFF080E0C, 0xFF080E0C, 0xFF0A100E, 0xFF090F0D, 0xFF060C0A, 0xFF070D0B, 0xFF0B110F, 0xFF050A06, 0xFF070C08, 0xFF080D09, 0xFF090E0A, 0xFF090E0A,
        0xFF090E0A, 0xFF090E0A, 0xFF0A0F0B, 0xFF080D09, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF080D09, 0xFF080D09, 0xFF070C08, 0xFF070C08, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF070D0B,
        0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08,
        0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090D0C,
        0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0D0F0C, 0xFF050908, 0xFF535756, 0xFF717576, 0xFF43484B, 0xFFC1C6CA, 0xFFD3DAE0, 0xFF6B7278, 0xFF101616, 0xFF070D0D, 0xFF090F0F, 0xFFA1A7A7, 0xFFA9AFAF,
        0xFF090F0F, 0xFF020808, 0xFF000303, 0xFF768081, 0xFF626C6E, 0xFF20272D, 0xFF555C62, 0xFF282D31, 0xFF101616, 0xFF080D09, 0xFF0E130C, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF6B6F70,
        0xFF4A4E4F, 0xFF161A1B, 0xFF131718, 0xFF111516, 0xFF060A0B, 0xFFB0B4B5, 0xFFFBFFFF, 0xFFA9ADAC, 0xFF3A3E3D, 0xFF060A09, 0xFF0E130F, 0xFF0E130F, 0xFF0D120C, 0xFF0D120C, 0xFF0F140E, 0xFF0E100B, 0xFF0D0F0A, 0xFF10130C, 0xFF282B22, 0xFF3A3D32,
        0xFF383C2E, 0xFF333729, 0xFF292D1E, 0xFF444B3B, 0xFFD9DCD3, 0xFFD8D9D3, 0xFF959887, 0xFFC9CCAF, 0xFFD5D6B6, 0xFFD9D7C0, 0xFFE2DED2, 0xFFDED9C3, 0xFFE5E1C4, 0xFFEDECCD, 0xFFC2C2AA, 0xFFE8E8E0, 0xFFECECEC, 0xFFF5F5F3, 0xFFEBEEE5, 0xFFB0B6AC,
        0xFFECEFE6, 0xFFF0F2E7, 0xFFEBEBDF, 0xFFEDECDA, 0xFFF2EED5, 0xFFE6E1C1, 0xFFDEDAB5, 0xFFCBC8A7, 0xFFCAC7A6, 0xFFD0CDAC, 0xFFDFDCBB, 0xFFECE9C8, 0xFFE9E6C5, 0xFFD5D2B1, 0xFFC3C09F, 0xFFBAB796, 0xFFC8C5A4, 0xFFDAD7B6, 0xFFE3E0BF, 0xFFDFDCBB,
        0xFFD2CFAE, 0xFFC6C3A2, 0xFFBEBB9A, 0xFFC9C6A5, 0xFFD6D3B2, 0xFFE7E4C3, 0xFFEFECCB, 0xFFEAE7C6, 0xFFDDDAB9, 0xFFD0CDAC, 0xFFC9C6A5, 0xFFC8C5A4, 0xFFCDCAA9, 0xFFD9D6B7, 0xFFE8E5C6, 0xFFF2EFD0, 0xFFEFEBCE, 0xFFE1DDC0, 0xFFD5D1B4, 0xFFCCC8AD,
        0xFFD2CEB5, 0xFFDBD9C4, 0xFFE9E6D3, 0x11F4F2E3, 0x00FBF9ED, 0x00FCF9F0, 0x00F9F9F1, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x43CCCCCC, 0xDE818181, 0xFF2F3332,
        0xFF222625, 0xFF121615, 0xFF080C0B, 0xFF040807, 0xFF050908, 0xFF090D0C, 0xFF0C100F, 0xFF090F0D, 0xFF060C0A, 0xFF060C0A, 0xFF090F0D, 0xFF090F0D, 0xFF060C0A, 0xFF050B09, 0xFF070D0B, 0xFF060B07, 0xFF070C08, 0xFF080D09, 0xFF080D09, 0xFF080D09,
        0xFF080D09, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF090E0A, 0xFF0A0F0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF070D0B,
        0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08,
        0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090D0C,
        0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090B08, 0xFF000100, 0xFF4D5150, 0xFF818586, 0xFF313639, 0xFF757A7E, 0xFF060D13, 0xFF01080E, 0xFF454A4D, 0xFFB8BDC0, 0xFF92979A, 0xFF050A0D, 0xFF9CA1A4,
        0xFFB3B8BB, 0xFF1A1F22, 0xFF010609, 0xFF747E7F, 0xFF505A5C, 0xFF5A6167, 0xFFC5CCD2, 0xFF44494D, 0xFF000606, 0xFF090E0A, 0xFF0F140D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF4C5051,
        0xFF797D7E, 0xFF9A9E9F, 0xFF4E5253, 0xFF010506, 0xFF181C1D, 0xFF939798, 0xFF6A6E6F, 0xFF161A19, 0xFF000302, 0xFF060A09, 0xFF101511, 0xFF0E130F, 0xFF0F140E, 0xFF0E130D, 0xFF111610, 0xFF0E100B, 0xFF0D0F0A, 0xFF10130C, 0xFF292C23, 0xFF3A3D32,
        0xFF383C2E, 0xFF333729, 0xFF292D1E, 0xFF3C4333, 0xFFD0D3CA, 0xFFD1D2CC, 0xFF888B7A, 0xFFA8AB8E, 0xFFBDBE9E, 0xFFDEDCC5, 0xFFF0ECE0, 0xFFEDE8D2, 0xFFE0DCBF, 0xFFD4D3B4, 0xFFAAAA92, 0xFFE1E1D9, 0xFFEFEFEF, 0xFFF6F6F4, 0xFFE1E4DB, 0xFFACB2A8,
        0xFFEBEEE5, 0xFFF6F8ED, 0xFFF0F0E4, 0xFFDEDDCB, 0xFFD1CDB4, 0xFFCFCAAA, 0xFFDCD8B3, 0xFFE9E6C5, 0xFFEAE7C6, 0xFFE5E2C1, 0xFFD7D4B3, 0xFFC9C6A5, 0xFFC8C5A4, 0xFFD6D3B2, 0xFFE5E2C1, 0xFFEAE7C6, 0xFFDBD8B7, 0xFFC8C5A4, 0xFFC0BD9C, 0xFFC7C4A3,
        0xFFD6D3B2, 0xFFE4E1C0, 0xFFECE9C8, 0xFFE1DEBD, 0xFFD6D3B2, 0xFFC8C5A4, 0xFFC4C1A0, 0xFFCCC9A8, 0xFFDAD7B6, 0xFFE7E4C3, 0xFFEEEBCA, 0xFFEFECCB, 0xFFE8E5C4, 0xFFDAD7B8, 0xFFC9C6A7, 0xFFC1BE9F, 0xFFC8C4A7, 0xFFDDD9BC, 0xFFEFEBCE, 0xFFF9F5DA,
        0xFFF3EFD6, 0xFFE9E7D2, 0xFFE4E1CE, 0x32E5E3D4, 0x1DEBE9DD, 0x08F3F0E7, 0x00F6F6EE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FFFFFF, 0x00FEFEFE, 0x04E9E9E9, 0x47CACECD,
        0x79B1B5B4, 0xD1888C8B, 0xFF5D6160, 0xFF3B3F3E, 0xFF232726, 0xFF111514, 0xFF050908, 0xFF060A09, 0xFF040807, 0xFF040807, 0xFF090D0C, 0xFF0C100F, 0xFF0B0F0E, 0xFF090D0C, 0xFF080C0B, 0xFF0C110D, 0xFF0B100C, 0xFF0A0F0B, 0xFF090E0A, 0xFF090E0A,
        0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF080D09, 0xFF090E0A, 0xFF090E0A, 0xFF080D09, 0xFF070C08, 0xFF070C08, 0xFF090E0A, 0xFF0B100C, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF070D0B,
        0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08,
        0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF090D0C,
        0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF141613, 0xFF010504, 0xFF3C403F, 0xFF8D9192, 0xFF000104, 0xFF000105, 0xFF3E454B, 0xFFA9B0B6, 0xFFD8DDE0, 0xFF8E9396, 0xFFA4A9AC, 0xFF989DA0, 0xFF101518,
        0xFF9A9FA2, 0xFFAEB3B6, 0xFF0F1417, 0xFF687273, 0xFF566062, 0xFF646B71, 0xFFD1D8DE, 0xFF3D4246, 0xFF030909, 0xFF090E0A, 0xFF0F140D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF000102,
        0xFF000405, 0xFF4C5051, 0xFF8F9394, 0xFF0F1314, 0xFF020607, 0xFF111516, 0xFF0A0E0F, 0xFF040807, 0xFF111514, 0xFF181C1B, 0xFF0C110D, 0xFF060B07, 0xFF090E08, 0xFF060B05, 0xFF0A0F09, 0xFF0E100B, 0xFF0D0F0A, 0xFF11140D, 0xFF292C23, 0xFF3A3D32,
        0xFF383C2E, 0xFF323628, 0xFF282C1D, 0xFF373E2E, 0xFFD8DBD2, 0xFFC6C7C1, 0xFF8A8D7C, 0xFFCACDB0, 0xFFDADBBB, 0xFFD9D7C0, 0xFFE2DED2, 0xFFE1DCC6, 0xFFE5E1C4, 0xFFE5E4C5, 0xFFBDBDA5, 0xFFE8E8E0, 0xFFECECEC, 0xFFF5F5F3, 0xFFE6E9E0, 0xFFB0B6AC,
        0xFFF3F6ED, 0xFFF4F6EB, 0xFFE9E9DD, 0xFFE7E6D4, 0xFFF1EDD4, 0xFFEBE6C6, 0xFFDEDAB5, 0xFFCECBAA, 0xFFCBC8A7, 0xFFCFCCAB, 0xFFDDDAB9, 0xFFEBE8C7, 0xFFE7E4C3, 0xFFD3D0AF, 0xFFBFBC9B, 0xFFB9B695, 0xFFCBC8A7, 0xFFDFDCBB, 0xFFE4E1C0, 0xFFD9D6B5,
        0xFFCAC7A6, 0xFFC2BF9E, 0xFFC0BD9C, 0xFFD7D4B3, 0xFFE2DFBE, 0xFFEDEAC9, 0xFFEEEBCA, 0xFFE4E1C0, 0xFFD5D2B1, 0xFFCAC7A6, 0xFFC5C2A1, 0xFFC8C5A4, 0xFFD1CEAD, 0xFFE0DDBE, 0xFFEDEACB, 0xFFF0EDCE, 0xFFE8E4C7, 0xFFD9D5B8, 0xFFCDC9AC, 0xFFCECAAF,
        0xFFD8D4BB, 0xFFE5E3CE, 0xFFF2EFDC, 0xFFF9F7E8, 0xFFFAF8EC, 0x00F7F4EB, 0x04F2F2EA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FEFEFE, 0x00FEFEFE, 0x00FBFBFB, 0x00FEFEFE, 0x00FFFFFF, 0x00FEFEFE, 0x00F8FAF9,
        0x00FEFFFF, 0x00FEFFFF, 0x00F4F6F5, 0x21DBDDDC, 0x75B3B5B4, 0xD9838584, 0xFF5F6160, 0xFF333736, 0xFF252928, 0xFF141817, 0xFF080C0B, 0xFF040807, 0xFF060A09, 0xFF080C0B, 0xFF090D0C, 0xFF060B07, 0xFF060B07, 0xFF070C08, 0xFF080D09, 0xFF080D09,
        0xFF080D09, 0xFF070C08, 0xFF050A06, 0xFF090E0A, 0xFF0A0F0B, 0xFF0B100C, 0xFF0A0F0B, 0xFF070C08, 0xFF060B07, 0xFF070C08, 0xFF090E0A, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF070D0B,
        0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08,
        0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF090D0C,
        0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF040603, 0xFF060A09, 0xFF4D5150, 0xFF797D7E, 0xFF1F2427, 0xFFA2A7AB, 0xFFD3DAE0, 0xFF959CA2, 0xFF282D30, 0xFF000104, 0xFF1B2023, 0xFFAAAFB2, 0xFF8A8F92,
        0xFF101518, 0xFF94999C, 0xFF686D70, 0xFF656F70, 0xFF616B6D, 0xFF4B5258, 0xFFA2A9AF, 0xFF22272B, 0xFF0D1313, 0xFF0B100C, 0xFF0B1009, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF121615,
        0xFF060A09, 0xFF2A2E2D, 0xFF383C3B, 0xFF040807, 0xFF111514, 0xFF151918, 0xFF050908, 0xFF111514, 0xFF121615, 0xFF0C100F, 0xFF080D09, 0xFF0D120E, 0xFF10150F, 0xFF0D120C, 0xFF111610, 0xFF0E100B, 0xFF0D0F0A, 0xFF11140D, 0xFF2A2D24, 0xFF3B3E33,
        0xFF383C2E, 0xFF323628, 0xFF272B1C, 0xFF484F3F, 0xFFD7DAD1, 0xFFC9CAC4, 0xFF909382, 0xFFA9AC8F, 0xFFC7C8A8, 0xFFE9E7D0, 0xFFEBE7DB, 0xFFECE7D1, 0xFFE2DEC1, 0xFFD3D2B3, 0xFFAEAE96, 0xFFE5E5DD, 0xFFF0F0F0, 0xFFF6F6F4, 0xFFDCDFD6, 0xFFB3B9AF,
        0xFFEBEEE5, 0xFFEEF0E5, 0xFFF2F2E6, 0xFFE2E1CF, 0xFFCECAB1, 0xFFCEC9A9, 0xFFDDD9B4, 0xFFE9E6C5, 0xFFEBE8C7, 0xFFE6E3C2, 0xFFD4D1B0, 0xFFC3C09F, 0xFFC1BE9D, 0xFFD2CFAE, 0xFFE3E0BF, 0xFFEBE8C7, 0xFFD5D2B1, 0xFFBDBA99, 0xFFB9B695, 0xFFCAC7A6,
        0xFFDEDBBA, 0xFFE9E6C5, 0xFFEBE8C7, 0xFFD8D5B4, 0xFFCDCAA9, 0xFFC3C09F, 0xFFC6C3A2, 0xFFD4D1B0, 0xFFE5E2C1, 0xFFEEEBCA, 0xFFF1EECD, 0xFFE6E3C2, 0xFFDCD9B8, 0xFFCECBAC, 0xFFC4C1A2, 0xFFC4C1A2, 0xFFD1CDB0, 0xFFE3DFC2, 0xFFF0ECCF, 0xFFF1EDD2,
        0xFFEDE9D0, 0xFFE5E3CE, 0xFFE1DECB, 0xFFE1DFD0, 0xFFEAE8DC, 0x04F6F3EA, 0x00FEFEF6, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFCFC, 0x00FEFFFF,
        0x00FEFFFF, 0x00FEFFFF, 0x00FBFDFC, 0x00FDFFFE, 0x00FEFFFF, 0x00FDFFFE, 0x00F6F8F7, 0x26D9DDDC, 0x6DB7BBBA, 0xDE828685, 0xFF4E5251, 0xFF2A2E2D, 0xFF151918, 0xFF0B0F0E, 0xFF060A09, 0xFF0C110D, 0xFF090E0A, 0xFF070C08, 0xFF060B07, 0xFF080D09,
        0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF070C08, 0xFF090E0A, 0xFF0B100C, 0xFF0B100C, 0xFF090E0A, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF070D0B,
        0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08,
        0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF090D0C,
        0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0E100D, 0xFF060A09, 0xFF373B3A, 0xFF7F8384, 0xFF393E41, 0xFF92979B, 0xFF3B4248, 0xFF000208, 0xFF23282C, 0xFF9EA3A7, 0xFF676C70, 0xFF12171B, 0xFFB4B9BD,
        0xFF868B8F, 0xFF0B1014, 0xFF080D11, 0xFF6E7879, 0xFF576163, 0xFF4F565C, 0xFFBAC1C7, 0xFF252A2E, 0xFF080E0E, 0xFF0C110D, 0xFF0A0F08, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0C110D, 0xFF0F1312,
        0xFF0A0E0D, 0xFF0C100F, 0xFF080C0B, 0xFF0A0E0D, 0xFF070B0A, 0xFF0C100F, 0xFF101413, 0xFF0E1211, 0xFF080C0B, 0xFF070B0A, 0xFF0E130F, 0xFF0F1410, 0xFF0A0F09, 0xFF0B100A, 0xFF0C110B, 0xFF0E100B, 0xFF0D0F0A, 0xFF12150E, 0xFF2A2D24, 0xFF3B3E33,
        0xFF383C2E, 0xFF313527, 0xFF262A1B, 0xFF444B3B, 0xFFEAEDE4, 0xFFC4C5BF, 0xFF959887, 0xFFCDD0B3, 0xFFDCDDBD, 0xFFD9D7C0, 0xFFE2DED2, 0xFFE1DCC6, 0xFFE5E1C4, 0xFFE0DFC0, 0xFFBCBCA4, 0xFFEAEAE2, 0xFFECECEC, 0xFFF5F5F3, 0xFFDFE2D9, 0xFFB2B8AE,
        0xFFF6F9F0, 0xFFF2F4E9, 0xFFE9E9DD, 0xFFE9E8D6, 0xFFEEEAD1, 0xFFEBE6C6, 0xFFDAD6B1, 0xFFCFCCAB, 0xFFCCC9A8, 0xFFD0CDAC, 0xFFDEDBBA, 0xFFECE9C8, 0xFFEAE7C6, 0xFFD5D2B1, 0xFFC1BE9D, 0xFFB8B594, 0xFFCDCAA9, 0xFFE2DFBE, 0xFFE5E2C1, 0xFFD6D3B2,
        0xFFC6C3A2, 0xFFC0BD9C, 0xFFC2BF9E, 0xFFD5D2B1, 0xFFE3E0BF, 0xFFF0EDCC, 0xFFECE9C8, 0xFFDCD9B8, 0xFFCBC8A7, 0xFFC4C1A0, 0xFFC5C2A1, 0xFFD1CEAD, 0xFFDBD8B7, 0xFFE9E6C7, 0xFFF0EDCE, 0xFFEDEACB, 0xFFE1DDC0, 0xFFD3CFB2, 0xFFCAC6A9, 0xFFD2CEB3,
        0xFFDCD8BF, 0xFFE9E7D2, 0xFFF5F2DF, 0xFFF9F7E8, 0x04F8F6EA, 0x04F5F2E9, 0x08F0F0E8, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00F9F9F9, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00F0F2F1, 0x3BD0D2D1, 0x9FA0A2A1, 0xFF6A6C6B, 0xFF454746, 0xFF1A1F1B, 0xFF141915, 0xFF0C110D, 0xFF070C08, 0xFF060B07,
        0xFF070C08, 0xFF080D09, 0xFF080D09, 0xFF070C08, 0xFF070C08, 0xFF080D09, 0xFF090E0A, 0xFF0A0F0B, 0xFF0B100C, 0xFF0B100C, 0xFF0A0F0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF070D0B,
        0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08,
        0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF090D0C,
        0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090B08, 0xFF040807, 0xFF222625, 0xFF9B9FA0, 0xFF73787B, 0xFF32373B, 0xFF02090F, 0xFF21282E, 0xFF9BA0A4, 0xFFE0E5E9, 0xFFE4E9ED, 0xFF757A7E, 0xFF1A1F23,
        0xFFB3B8BC, 0xFF878C90, 0xFF02070B, 0xFF677172, 0xFF535D5F, 0xFF585F65, 0xFFCBD2D8, 0xFF34393D, 0xFF040A0A, 0xFF080D09, 0xFF0F140D, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF070C08,
        0xFF151A16, 0xFF090E0A, 0xFF131814, 0xFF0E130F, 0xFF0B100C, 0xFF090E0A, 0xFF131814, 0xFF111514, 0xFF0C100F, 0xFF0A0E0D, 0xFF0E130F, 0xFF0A0F0B, 0xFF090E08, 0xFF111610, 0xFF0D120C, 0xFF0E100B, 0xFF0E100B, 0xFF12150E, 0xFF2B2E25, 0xFF3C3F34,
        0xFF383C2E, 0xFF313527, 0xFF262A1B, 0xFF545B4B, 0xFFE2E5DC, 0xFFBDBEB8, 0xFF959887, 0xFFAAAD90, 0xFFC6C7A7, 0xFFE7E5CE, 0xFFEDE9DD, 0xFFEAE5CF, 0xFFE2DEC1, 0xFFCECDAE, 0xFFAEAE96, 0xFFE8E8E0, 0xFFF0F0F0, 0xFFF6F6F4, 0xFFD4D7CE, 0xFFB7BDB3,
        0xFFEEF1E8, 0xFFEBEDE2, 0xFFF1F1E5, 0xFFE3E2D0, 0xFFCBC7AE, 0xFFCFCAAA, 0xFFDAD6B1, 0xFFEAE7C6, 0xFFECE9C8, 0xFFE7E4C3, 0xFFD6D3B2, 0xFFC4C1A0, 0xFFC3C09F, 0xFFD4D1B0, 0xFFE6E3C2, 0xFFE8E5C4, 0xFFD1CEAD, 0xFFBBB897, 0xFFBBB897, 0xFFCECBAA,
        0xFFE2DFBE, 0xFFE9E6C5, 0xFFE6E3C2, 0xFFCFCCAB, 0xFFCBC8A7, 0xFFC8C5A4, 0xFFCECBAA, 0xFFDAD7B6, 0xFFE8E5C4, 0xFFF1EECD, 0xFFF5F2D1, 0xFFE0DDBC, 0xFFD6D3B2, 0xFFC9C6A7, 0xFFC5C2A3, 0xFFCCC9AA, 0xFFDCD8BB, 0xFFEDE9CC, 0xFFF8F4D7, 0xFFEDE9CE,
        0xFFE9E5CC, 0xFFE2E0CB, 0xFFDFDCC9, 0xFFE2E0D1, 0x1DECEADE, 0x00F9F6ED, 0x00FFFFF7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FDFDFD, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FAFCFB, 0x00F0F2F1, 0x0DE6E8E7, 0x75B4B9B5, 0xB4969B97, 0xFF666B67, 0xFF393E3A, 0xFF191E1A,
        0xFF090E0A, 0xFF040905, 0xFF040905, 0xFF0D120E, 0xFF0A0F0B, 0xFF070C08, 0xFF070C08, 0xFF080D09, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF090E0A, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF070D0B,
        0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08,
        0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF090D0C,
        0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0F110E, 0xFF070B0A, 0xFF0B0F0E, 0xFF232728, 0xFF6B7073, 0xFF959A9E, 0xFFA1A8AE, 0xFF888F95, 0xFF72777D, 0xFF464B51, 0xFF60656B, 0xFF8B9096, 0xFF2B3036,
        0xFF0E1319, 0xFFBEC3C9, 0xFF585D63, 0xFF5E6869, 0xFF596365, 0xFF51585E, 0xFFADB4BA, 0xFF2F3438, 0xFF080E0E, 0xFF090E0A, 0xFF0E130C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0A0F0B,
        0xFF0E130F, 0xFF131814, 0xFF030804, 0xFF121713, 0xFF040905, 0xFF131814, 0xFF050A06, 0xFF0C100F, 0xFF0D1110, 0xFF090D0C, 0xFF0C110D, 0xFF0C110D, 0xFF0C110B, 0xFF121711, 0xFF070C06, 0xFF0E100B, 0xFF0E100B, 0xFF12150E, 0xFF2B2E25, 0xFF3C3F34,
        0xFF383C2E, 0xFF303426, 0xFF25291A, 0xFF5C6353, 0xFFEDF0E7, 0xFFB0B1AB, 0xFF979A89, 0xFFCDD0B3, 0xFFDFE0C0, 0xFFDFDDC6, 0xFFE1DDD1, 0xFFDDD8C2, 0xFFE5E1C4, 0xFFE0DFC0, 0xFFC3C3AB, 0xFFF1F1E9, 0xFFEEEEEE, 0xFFF5F5F3, 0xFFD7DAD1, 0xFFBDC3B9,
        0xFFF7FAF1, 0xFFE9EBE0, 0xFFE8E8DC, 0xFFEAE9D7, 0xFFEAE6CD, 0xFFEBE6C6, 0xFFDDD9B4, 0xFFCFCCAB, 0xFFCDCAA9, 0xFFD0CDAC, 0xFFDCD9B8, 0xFFE6E3C2, 0xFFE3E0BF, 0xFFD1CEAD, 0xFFC0BD9C, 0xFFBEBB9A, 0xFFD2CFAE, 0xFFE4E1C0, 0xFFE1DEBD, 0xFFCECBAA,
        0xFFBEBB9A, 0xFFBEBB9A, 0xFFC5C2A1, 0xFFE5E2C1, 0xFFE6E3C2, 0xFFE4E1C0, 0xFFDDDAB9, 0xFFD3D0AF, 0xFFCAC7A6, 0xFFC6C3A2, 0xFFC5C2A1, 0xFFDCD9B8, 0xFFE7E4C3, 0xFFF1EECF, 0xFFEFECCD, 0xFFE3E0C1, 0xFFD4D0B3, 0xFFCBC7AA, 0xFFC8C4A7, 0xFFDDD9BE,
        0xFFE4E0C7, 0xFFECEAD5, 0xFFF6F3E0, 0xFFFBF9EA, 0x00FAF8EC, 0x04F5F2E9, 0x0DEEEEE6, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FCFCFC, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FFFFFF,
        0x00FDFDFD, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FBFBFB, 0x00FCFCFC, 0x00FFFFFF, 0x00FDFFFE, 0x00FDFFFE, 0x00FBFDFC, 0x00FAFCFB, 0x00FDFFFE, 0x00FEFFFF, 0x00FEFFFF, 0x00FDFFFE, 0x00FCFFFD, 0x00FBFFFC, 0x00F0F5F1, 0x26DADFDB, 0x71B6BBB7,
        0xD1878C88, 0xFF595E5A, 0xFF3B403C, 0xFF151A16, 0xFF0F1410, 0xFF080D09, 0xFF050A06, 0xFF060B07, 0xFF070C08, 0xFF070C08, 0xFF050A06, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF070D0B,
        0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF060C08,
        0xFF060C08, 0xFF060C08, 0xFF060C08, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070D09, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF070C08, 0xFF090D0C,
        0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090B08, 0xFF0E1211, 0xFF060A09, 0xFF05090A, 0xFF03080B, 0xFF000408, 0xFF131A20, 0xFF3C4349, 0xFF6A6F75, 0xFF92979D, 0xFF91969C, 0xFF84898F, 0xFF686D73,
        0xFF30353B, 0xFF292E34, 0xFF1C2127, 0xFF667071, 0xFF576163, 0xFF50575D, 0xFFA2A9AF, 0xFF252A2E, 0xFF080E0E, 0xFF121713, 0xFF050A03, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0E130F,
        0xFF0F1410, 0xFF020703, 0xFF131814, 0xFF050A06, 0xFF111612, 0xFF090E0A, 0xFF0D120E, 0xFF090D0C, 0xFF101413, 0xFF0A0E0D, 0xFF0D120E, 0xFF0E130F, 0xFF090E08, 0xFF0E130D, 0xFF0C110B, 0xFF0E100B, 0xFF0E100B, 0xFF12150E, 0xFF2B2E25, 0xFF3C3F34,
        0xFF383C2E, 0xFF303426, 0xFF25291A, 0xFF676E5E, 0xFFEEF1E8, 0xFFAEAFA9, 0xFF8F9281, 0xFFAFB295, 0xFFC7C8A8, 0xFFE2E0C9, 0xFFEFEBDF, 0xFFEBE6D0, 0xFFE0DCBF, 0xFFC8C7A8, 0xFFACAC94, 0xFFEBEBE3, 0xFFF2F2F2, 0xFFF5F5F3, 0xFFCCCFC6, 0xFFBAC0B6,
        0xFFF7FAF1, 0xFFEEF0E5, 0xFFEBEBDF, 0xFFDAD9C7, 0xFFC9C5AC, 0xFFD5D0B0, 0xFFDDD9B4, 0xFFEDEAC9, 0xFFEDEAC9, 0xFFE5E2C1, 0xFFD4D1B0, 0xFFC3C09F, 0xFFC2BF9E, 0xFFD1CEAD, 0xFFE2DFBE, 0xFFE2DFBE, 0xFFCDCAA9, 0xFFBBB897, 0xFFC0BD9C, 0xFFD6D3B2,
        0xFFE7E4C3, 0xFFE5E2C1, 0xFFDBD8B7, 0xFFC7C4A3, 0xFFC5C2A1, 0xFFC6C3A2, 0xFFD2CFAE, 0xFFE2DFBE, 0xFFEDEAC9, 0xFFEEEBCA, 0xFFEAE7C6, 0xFFD3D0AF, 0xFFCCC9A8, 0xFFC6C3A4, 0xFFCBC8A9, 0xFFD9D6B7, 0xFFE8E4C7, 0xFFF1EDD0, 0xFFF3EFD2, 0xFFE4E0C5,
        0xFFE1DDC4, 0xFFDDDBC6, 0xFFE0DDCA, 0xFFE7E5D6, 0x11F1EFE3, 0x00FAF7EE, 0x00FDFDF5, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF,
        0x00FEFEFE, 0x00EFEFEF, 0x19DFDFDF, 0x79B1B3B2, 0xC48D8F8E, 0xFF5C5E5D, 0xFF343635, 0xFF191B1A, 0xFF0A0C0B, 0xFF050706, 0xFF060807, 0xFF060A09, 0xFF080C0B, 0xFF090D0C, 0xFF080C0B, 0xFF080C0B, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF080C0B, 0xFF050B09,
        0xFF080E0C, 0xFF090F0D, 0xFF070D0B, 0xFF040A08, 0xFF040A08, 0xFF080E0C, 0xFF0B110F, 0xFF090F0D, 0xFF090F0D, 0xFF070D0B, 0xFF050B09, 0xFF070D0B, 0xFF0B110F, 0xFF0A100E, 0xFF060C0A, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B,
        0xFF060C0A, 0xFF050B09, 0xFF040A08, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF090E0A,
        0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF070B0A, 0xFF080C0B, 0xFF090D0C, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF090D0C, 0xFF080C0B, 0xFF070B0A, 0xFF030708, 0xFF040809, 0xFF15191A, 0xFF404445, 0xFF75797A,
        0xFF949899, 0xFF95999A, 0xFF898D8E, 0xFFBABEBF, 0xFF404445, 0xFF5B5F60, 0xFFD6DADB, 0xFF414546, 0xFF000203, 0xFF0C1011, 0xFF0A0E0F, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0C110B,
        0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0C100F, 0xFF0C100F, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0C100F, 0xFF0C100F, 0xFF0E100D, 0xFF0C0E0B, 0xFF141611, 0xFF2D3029, 0xFF393C33,
        0xFF35382D, 0xFF303426, 0xFF222618, 0xFF6E736F, 0xFFF0F3EA, 0xFFABAE9D, 0xFF9FA38A, 0xFFD5D8BB, 0xFFD8D8BE, 0xFFD7D4C1, 0xFFE7E4D5, 0xFFE0E0BE, 0xFFE5E4C6, 0xFFD2D1BC, 0xFFC4C5B7, 0xFFE9EAE2, 0xFFF2F3ED, 0xFFF5F8F1, 0xFFC8CDC6, 0xFFC2C5BC,
        0xFFF6F8ED, 0xFFF1F3E5, 0xFFEBECDA, 0xFFEBEBD3, 0xFFEAE9CB, 0xFFE3E0BF, 0xFFE0DDBA, 0xFFC8C5A6, 0xFFC9C6A7, 0xFFD0CDAE, 0xFFDDDABB, 0xFFE7E4C5, 0xFFE1DEBF, 0xFFCDCAAB, 0xFFBBB899, 0xFFBFBC9B, 0xFFD4D1B0, 0xFFE6E3C2, 0xFFE1DEBD, 0xFFCBC8A7,
        0xFFBCB998, 0xFFC0BD9C, 0xFFCCC9A8, 0xFFE3E0BF, 0xFFE8E5C4, 0xFFE9E6C5, 0xFFDDDAB9, 0xFFCCC9A8, 0xFFC4C1A0, 0xFFC9C6A5, 0xFFD2CFAE, 0xFFE4E0BD, 0xFFECE8C5, 0xFFF0EBCB, 0xFFE8E3C3, 0xFFD9D4B6, 0xFFCDC8AA, 0xFFCCC7A9, 0xFFD0CBAE, 0xFFE1DDC4,
        0xFFECE7D1, 0xFFF7F5E0, 0xFFFAF7E6, 0x0DF5F3E6, 0x11F0EDE4, 0x08F2EFE8, 0x00F3F2ED, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFCFC, 0x00FDFDFD, 0x00FDFDFD, 0x00FDFDFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FEFEFE, 0x00F8F8F8, 0x00FEFFFF, 0x00FBFDFC, 0x00ECEEED, 0x32D4D6D5, 0x79B1B3B2, 0xD9838584, 0xFF525453, 0xFF323433, 0xFF1A1E1D, 0xFF080C0B, 0xFF000201, 0xFF060A09, 0xFF0C100F, 0xFF080C0B, 0xFF040807, 0xFF070B0A, 0xFF0B110F,
        0xFF060C0A, 0xFF030907, 0xFF060C0A, 0xFF0B110F, 0xFF0A100E, 0xFF040A08, 0xFF000503, 0xFF030907, 0xFF080E0C, 0xFF0A100E, 0xFF070D0B, 0xFF040A08, 0xFF050B09, 0xFF060C0A, 0xFF060C0A, 0xFF070D0B, 0xFF070D0B, 0xFF080E0C, 0xFF080E0C, 0xFF070D0B,
        0xFF060C0A, 0xFF060C0A, 0xFF050B09, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF060C0A, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF090E0A,
        0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF070B0A, 0xFF080C0B, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF090D0C, 0xFF080C0B, 0xFF070B0A, 0xFF080C0D, 0xFF0D1112, 0xFF0C1011, 0xFF020607, 0xFF000102,
        0xFF070B0C, 0xFF272B2C, 0xFF424647, 0xFF535758, 0xFF15191A, 0xFF303435, 0xFF939798, 0xFF383C3D, 0xFF0B0F10, 0xFF0C1011, 0xFF0A0E0F, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0C110B,
        0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0C110B, 0xFF0C100F, 0xFF0C100F, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0C100F, 0xFF0C100F, 0xFF0E100D, 0xFF0C0E0B, 0xFF131510, 0xFF2E312A, 0xFF3A3D34,
        0xFF36392E, 0xFF303426, 0xFF212517, 0xFF7C817D, 0xFFEBEEE5, 0xFFB4B7A6, 0xFF8F937A, 0xFFACAF92, 0xFFD5D5BB, 0xFFE5E2CF, 0xFFE5E2D3, 0xFFD9D9B7, 0xFFD6D5B7, 0xFFBEBDA8, 0xFFBABBAD, 0xFFE7E8E0, 0xFFF2F3ED, 0xFFF2F5EE, 0xFFC1C6BF, 0xFFC6C9C2,
        0xFFF8F9F1, 0xFFF0F2E4, 0xFFE3E4D2, 0xFFD8D8C0, 0xFFD4D3B5, 0xFFD3D0AF, 0xFFD6D3B0, 0xFFF0EDCE, 0xFFEEEBCC, 0xFFE4E1C2, 0xFFD1CEAF, 0xFFC0BD9E, 0xFFC1BE9F, 0xFFD5D2B3, 0xFFE8E5C6, 0xFFE0DDBC, 0xFFCCC9A8, 0xFFBBB897, 0xFFC0BD9C, 0xFFD6D3B2,
        0xFFE5E2C1, 0xFFE1DEBD, 0xFFD6D3B2, 0xFFC2BF9E, 0xFFC1BE9D, 0xFFC7C4A3, 0xFFD6D3B2, 0xFFE7E4C3, 0xFFEDEAC9, 0xFFE8E5C4, 0xFFDFDCBB, 0xFFCECAA7, 0xFFC8C4A1, 0xFFC6C1A1, 0xFFD2CDAD, 0xFFE5E0C2, 0xFFF3EED0, 0xFFF4EFD1, 0xFFF0EBCE, 0xFFE0DCC3,
        0xFFDDD8C2, 0xFFDAD8C3, 0xFFE2DFCE, 0x19EFEDE0, 0x00FAF7EE, 0x00FFFCF5, 0x00FDFCF7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FDFDFD,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F9FBFA, 0x00FDFFFE, 0x00FEFFFF, 0x00FEFFFF, 0x00FDFFFE, 0x00F8FAF9, 0x04E9EBEA, 0x26D9DBDA, 0x9F9FA3A2, 0xF7767A79, 0xFF474B4A, 0xFF272B2A, 0xFF101413, 0xFF000403, 0xFF000403, 0xFF090D0C, 0xFF080C0B,
        0xFF080C0B, 0xFF0A0E0D, 0xFF0C100F, 0xFF090D0C, 0xFF050908, 0xFF060A09, 0xFF0B0F0E, 0xFF070D0B, 0xFF080E0C, 0xFF070D0B, 0xFF060C0A, 0xFF070D0B, 0xFF090F0D, 0xFF0A100E, 0xFF090F0D, 0xFF070D0B, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C,
        0xFF070D0B, 0xFF060C0A, 0xFF060C0A, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF090E0A,
        0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF070B0A, 0xFF080C0B, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF090D0C, 0xFF080C0B, 0xFF070B0A, 0xFF080C0D, 0xFF0A0E0F, 0xFF0C1011, 0xFF0D1112, 0xFF0B0F10,
        0xFF080C0D, 0xFF05090A, 0xFF030708, 0xFF030708, 0xFF0A0E0F, 0xFF05090A, 0xFF15191A, 0xFF000102, 0xFF080C0D, 0xFF111516, 0xFF070B0C, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B100A,
        0xFF0B100A, 0xFF0B100A, 0xFF0B100A, 0xFF0B100A, 0xFF0B100A, 0xFF0B100A, 0xFF0B100A, 0xFF0C100F, 0xFF0C100F, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0C100F, 0xFF0C100F, 0xFF0E100D, 0xFF0B0D0A, 0xFF131510, 0xFF2E312A, 0xFF3B3E35,
        0xFF373A2F, 0xFF303426, 0xFF202416, 0xFF7B807C, 0xFFDEE1D8, 0xFF939685, 0xFFA5A990, 0xFFD5D8BB, 0xFFD1D1B7, 0xFFE0DDCA, 0xFFE5E2D3, 0xFFE8E8C6, 0xFFEAE9CB, 0xFFD3D2BD, 0xFFCCCDBF, 0xFFEBECE4, 0xFFEDEEE8, 0xFFEFF2EB, 0xFFC6CBC4, 0xFFCCCFC8,
        0xFFF5F6EE, 0xFFEAECDF, 0xFFE9EAD8, 0xFFECECD4, 0xFFECEBCF, 0xFFE3E0C1, 0xFFDBD8B5, 0xFFC9C6A7, 0xFFC8C5A6, 0xFFCECBAC, 0xFFDDDABB, 0xFFE8E5C6, 0xFFE2DFC0, 0xFFCAC7A8, 0xFFB5B293, 0xFFC7C4A3, 0xFFD8D5B4, 0xFFE5E2C1, 0xFFDDDAB9, 0xFFC8C5A4,
        0xFFBDBA99, 0xFFC5C2A1, 0xFFD3D0AF, 0xFFE4E1C0, 0xFFE6E3C2, 0xFFE2DFBE, 0xFFD5D2B1, 0xFFC6C3A2, 0xFFC4C1A0, 0xFFD0CDAC, 0xFFDEDBBA, 0xFFEFEBC8, 0xFFF1EDCA, 0xFFEEE9C9, 0xFFDFDABA, 0xFFCEC9AB, 0xFFC8C3A5, 0xFFCEC9AB, 0xFFD8D3B6, 0xFFEFEBD2,
        0xFFF3EED8, 0xFFF5F3DE, 0xFFF6F3E2, 0x11F3F1E4, 0x0DF2EFE6, 0x04F4F1EA, 0x00F5F4EF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE,
        0x00FDFDFD, 0x00FDFDFD, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFCFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FBFBFB, 0x00FEFFFF, 0x00FEFFFF, 0x04E9EBEA, 0x4FC6C8C7, 0xA39D9F9E, 0xFF707271, 0xFF414342, 0xFF1F2120, 0xFF121615,
        0xFF050908, 0xFF000100, 0xFF000403, 0xFF070B0A, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF080E0C, 0xFF090F0D, 0xFF0A100E, 0xFF0A100E, 0xFF090F0D, 0xFF080E0C, 0xFF060C0A, 0xFF040A08, 0xFF070D0B, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C,
        0xFF080E0C, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF090E0A,
        0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF080C0B, 0xFF080C0B, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF090D0C, 0xFF080C0B, 0xFF080C0B, 0xFF0D1110, 0xFF070B0A, 0xFF020605, 0xFF040807, 0xFF0B0F0E,
        0xFF0F1312, 0xFF0C100F, 0xFF080C0B, 0xFF040807, 0xFF111514, 0xFF070B0A, 0xFF080C0B, 0xFF101413, 0xFF0B0F0E, 0xFF0A0E0D, 0xFF0D1110, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0B100C,
        0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0C100F, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0C100F, 0xFF0D0F0C, 0xFF0B0D0A, 0xFF131510, 0xFF2E312A, 0xFF3C3F36,
        0xFF383B30, 0xFF2F3325, 0xFF1F2315, 0xFF484D49, 0xFF9C9F96, 0xFF959887, 0xFF9DA188, 0xFFAEB194, 0xFFD1D1B7, 0xFFE7E4D1, 0xFFE8E5D6, 0xFFDEDEBC, 0xFFD6D5B7, 0xFFB3B29D, 0xFFBABBAD, 0xFFECEDE5, 0xFFF9FAF4, 0xFFF2F5EE, 0xFFB9BEB7, 0xFFCED1CA,
        0xFFF8F9F1, 0xFFEEF0E3, 0xFFE1E2D2, 0xFFD2D1BC, 0xFFCDCCB0, 0xFFD2CFB0, 0xFFDBD8B7, 0xFFEDEACB, 0xFFEDEACB, 0xFFE5E2C3, 0xFFD1CEAF, 0xFFBFBC9D, 0xFFBFBC9D, 0xFFD2CFB0, 0xFFE6E3C4, 0xFFD2CFAE, 0xFFC3C09F, 0xFFB9B695, 0xFFC5C2A1, 0xFFDCD9B8,
        0xFFE8E5C4, 0xFFDFDCBB, 0xFFD0CDAC, 0xFFBEBB9A, 0xFFC2BF9E, 0xFFCDCAA9, 0xFFDEDBBA, 0xFFECE9C8, 0xFFECE9C8, 0xFFDFDCBB, 0xFFD3D0AF, 0xFFC9C5A2, 0xFFCAC6A3, 0xFFD0CBAB, 0xFFDED9B9, 0xFFEDE8CA, 0xFFF1ECCE, 0xFFEAE5C7, 0xFFE1DCBF, 0xFFD4D0B7,
        0xFFD9D4BE, 0xFFE0DEC9, 0xFFEAE7D6, 0x11F2F0E3, 0x00F9F6ED, 0x00FDFAF3, 0x00FDFCF7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FDFDFD, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FEFEFE, 0x00FDFDFD, 0x00FDFDFD, 0x00FBFBFB, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F7F9F8, 0x00FEFFFF, 0x00FEFFFF, 0x00FDFFFE, 0x00FEFFFF, 0x00FDFFFE, 0x11E3E5E4, 0x54C4C6C5, 0xCD898D8C,
        0xFF676B6A, 0xFF3B3F3E, 0xFF1C201F, 0xFF0C100F, 0xFF050908, 0xFF040807, 0xFF060A09, 0xFF000604, 0xFF040A08, 0xFF070D0B, 0xFF080E0C, 0xFF060C0A, 0xFF030907, 0xFF040A08, 0xFF050B09, 0xFF070D0B, 0xFF070D0B, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C,
        0xFF080E0C, 0xFF080E0C, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF090E0A,
        0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF080C0B, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF080C0B, 0xFF070B0A, 0xFF0C100F, 0xFF101413, 0xFF0F1312, 0xFF0A0E0D,
        0xFF060A09, 0xFF080C0B, 0xFF0B0F0E, 0xFF0F1312, 0xFF0E1211, 0xFF070B0A, 0xFF000403, 0xFF0E1211, 0xFF040807, 0xFF090D0C, 0xFF0C100F, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0B100C,
        0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0B100C, 0xFF0C100F, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0C100F, 0xFF0D0F0C, 0xFF0B0D0A, 0xFF141611, 0xFF2F322B, 0xFF3C3F36,
        0xFF373A2F, 0xFF2F3325, 0xFF1F2315, 0xFF979C98, 0xFFEAEDE4, 0xFF909382, 0xFFA5A990, 0xFFDADDC0, 0xFFD4D4BA, 0xFFE3E0CD, 0xFFE5E2D3, 0xFFE1E1BF, 0xFFEAE9CB, 0xFFCBCAB5, 0xFFCCCDBF, 0xFFECEDE5, 0xFFF4F5EF, 0xFFF4F7F0, 0xFFC2C7C0, 0xFFD5D7D2,
        0xFFF2F3ED, 0xFFE6E8DD, 0xFFE9EADA, 0xFFECEBD6, 0xFFEAE9CD, 0xFFE0DDBE, 0xFFD6D3B2, 0xFFC9C6A7, 0xFFC6C3A4, 0xFFCAC7A8, 0xFFD8D5B6, 0xFFE5E2C3, 0xFFE1DEBF, 0xFFCBC8A9, 0xFFB6B394, 0xFFCBC8A7, 0xFFDBD8B7, 0xFFE6E3C2, 0xFFDCD9B8, 0xFFC6C3A2,
        0xFFBCB998, 0xFFC6C3A2, 0xFFD6D3B2, 0xFFE7E4C3, 0xFFE4E1C0, 0xFFDBD8B7, 0xFFCDCAA9, 0xFFC2BF9E, 0xFFC5C2A1, 0xFFD4D1B0, 0xFFE2DFBE, 0xFFEFEBC8, 0xFFECE8C5, 0xFFE3DEBE, 0xFFD5D0B0, 0xFFCAC5A7, 0xFFCBC6A8, 0xFFDAD5B7, 0xFFE8E3C6, 0xFFF7F3DA,
        0xFFF4EFD9, 0xFFEEECD7, 0x2AEBE8D7, 0x21EBE9DC, 0x11EFECE3, 0x00F5F2EB, 0x00F7F6F1, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFCFC, 0x00FEFEFE, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FCFAFB, 0x00FDFBFC, 0x00FDFBFC, 0x00FBF9FA, 0x00FDFBFC, 0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FBFBFB, 0x00F8F8F8, 0x00FCFCFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FBFBFB, 0x00FEFFFF,
        0x00F6F8F7, 0x21DCDEDD, 0x6DB7B9B8, 0xCD898B8A, 0xFF595B5A, 0xFF303231, 0xFF171918, 0xFF0D1311, 0xFF050B09, 0xFF000604, 0xFF040A08, 0xFF0B110F, 0xFF0C1210, 0xFF080E0C, 0xFF040A08, 0xFF060C0A, 0xFF060C0A, 0xFF070D0B, 0xFF080E0C, 0xFF080E0C,
        0xFF080E0C, 0xFF080E0C, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF090E0A,
        0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF090E0A, 0xFF0A0F0B,
        0xFF0A0F0B, 0xFF090E0A, 0xFF080D09, 0xFF060B07, 0xFF0A0F0B, 0xFF111612, 0xFF040905, 0xFF0B100C, 0xFF0D120E, 0xFF121713, 0xFF000400, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0B0F0E,
        0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0C0E0B, 0xFF0B0D0A, 0xFF151712, 0xFF2F322B, 0xFF3B3E35,
        0xFF36392E, 0xFF2F3325, 0xFF212517, 0xFFBEC3BF, 0xFFECEFE6, 0xFFABAE9D, 0xFF969A81, 0xFFABAE91, 0xFFD0D0B6, 0xFFE5E2CF, 0xFFE4E1D2, 0xFFDBDBB9, 0xFFD8D7B9, 0xFFAFAE99, 0xFFBFC0B2, 0xFFEFF0E8, 0xFFFEFFF9, 0xFFF6F9F2, 0xFFB9BEB7, 0xFFD6D8D3,
        0xFFF5F6F0, 0xFFE9EBE0, 0xFFE1E2D4, 0xFFD2D1BD, 0xFFCAC8AF, 0xFFCFCBAE, 0xFFD6D3B4, 0xFFE9E6C7, 0xFFE7E4C5, 0xFFDDDABB, 0xFFCBC8A9, 0xFFBCB99A, 0xFFC0BD9E, 0xFFD5D2B3, 0xFFEAE7C8, 0xFFCECBAA, 0xFFC1BE9D, 0xFFBBB897, 0xFFC8C5A4, 0xFFDFDCBB,
        0xFFE7E4C3, 0xFFDBD8B7, 0xFFCAC7A6, 0xFFBEBB9A, 0xFFC5C2A1, 0xFFD2CFAE, 0xFFE1DEBD, 0xFFEAE7C6, 0xFFE6E3C2, 0xFFD9D6B5, 0xFFCDCAA9, 0xFFC3BF9C, 0xFFCBC7A4, 0xFFDAD5B5, 0xFFE9E4C4, 0xFFF2EDCF, 0xFFEEE9CB, 0xFFE2DDBF, 0xFFD7D2B5, 0xFFD1CDB4,
        0xFFDCD7C1, 0xFFE9E7D2, 0x15F5F2E1, 0x00FAF8EB, 0x00FCF9F0, 0x00FDFAF3, 0x00FDFCF7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF,
        0x00FEFEFE, 0x00FDFDFD, 0x00FFFFFF, 0x00FCFAFB, 0x00FDFBFC, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FFFDFE, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FEFEFE, 0x00FFFFFF, 0x00FBFDFC,
        0x00FDFFFE, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00F3F5F4, 0x36D2D4D3, 0x71B5B7B6, 0xEE7A807E, 0xFF565C5A, 0xFF292F2D, 0xFF0C1210, 0xFF020806, 0xFF030907, 0xFF070D0B, 0xFF0A100E, 0xFF050B09, 0xFF060C0A, 0xFF060C0A, 0xFF070D0B, 0xFF080E0C,
        0xFF080E0C, 0xFF070D0B, 0xFF070D0B, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF090E0A,
        0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF080C0B, 0xFF080C0B, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF090E0A, 0xFF0A0F0B,
        0xFF0A0F0B, 0xFF0B100C, 0xFF0B100C, 0xFF101511, 0xFF000501, 0xFF060B07, 0xFF0A0F0B, 0xFF0C110D, 0xFF010602, 0xFF070C08, 0xFF0B100C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D,
        0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0C0E0B, 0xFF0C0E0B, 0xFF161813, 0xFF2F322B, 0xFF393C33,
        0xFF34372C, 0xFF2F3325, 0xFF232719, 0xFFA7ACA8, 0xFFDBDED5, 0xFF888B7A, 0xFFA9AD94, 0xFFDCDFC2, 0xFFD6D6BC, 0xFFE1DECB, 0xFFE4E1D2, 0xFFE8E8C6, 0xFFE9E8CA, 0xFFC3C2AD, 0xFFD7D8CA, 0xFFF9FAF2, 0xFFFBFCF6, 0xFFF8FBF4, 0xFFCBD0C9, 0xFFDDDFDC,
        0xFFF2F3EE, 0xFFE4E5DD, 0xFFE8E9DB, 0xFFE6E5D1, 0xFFE3E1C8, 0xFFE0DCBF, 0xFFDBD8B9, 0xFFCAC7A8, 0xFFC8C5A6, 0xFFCCC9AA, 0xFFD8D5B6, 0xFFE1DEBF, 0xFFDAD7B8, 0xFFC2BFA0, 0xFFADAA8B, 0xFFD0CDAC, 0xFFDCD9B8, 0xFFE3E0BF, 0xFFD7D4B3, 0xFFC3C09F,
        0xFFBEBB9A, 0xFFCFCCAB, 0xFFE2DFBE, 0xFFF0EDCC, 0xFFE1DEBD, 0xFFCDCAA9, 0xFFC0BD9C, 0xFFC0BD9C, 0xFFCCC9A8, 0xFFDBD8B7, 0xFFE4E1C0, 0xFFF2EECB, 0xFFE6E2BF, 0xFFD5D0B0, 0xFFCAC5A5, 0xFFCBC6A8, 0xFFD6D1B3, 0xFFE4DFC1, 0xFFEEE9CC, 0xFFFAF6DD,
        0xFFF1ECD6, 0xFFE6E4CF, 0x36E5E2D1, 0x21EBE9DC, 0x08F3F0E7, 0x00F7F4ED, 0x00F6F5F0, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FBFBFB, 0x00FBFBFB, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF,
        0x00FDFDFD, 0x00FDFDFD, 0x00FFFFFF, 0x00FDFBFC, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FDFFFE,
        0x00FEFFFF, 0x00FEFFFF, 0x00F9FBFA, 0x00FDFFFE, 0x00FEFFFF, 0x00FEFFFF, 0x00F8FAF9, 0x00FBFFFF, 0x00ECF2F0, 0x36D2D8D6, 0x8AA9AFAD, 0xFB737977, 0xFF3E4442, 0xFF191F1D, 0xFF0A100E, 0xFF040A08, 0xFF050B09, 0xFF060C0A, 0xFF070D0B, 0xFF070D0B,
        0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080E0C, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF080D09, 0xFF090E0A,
        0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF080C0B, 0xFF080C0B, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090E0A, 0xFF070C08, 0xFF060B07, 0xFF070C08, 0xFF0A0F0B,
        0xFF0B100C, 0xFF0A0F0B, 0xFF080D09, 0xFF0C110D, 0xFF080D09, 0xFF0C110D, 0xFF070C08, 0xFF0A0F0B, 0xFF080D09, 0xFF0B100C, 0xFF0C110D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D,
        0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0C0E0B, 0xFF0C0E0B, 0xFF171914, 0xFF2F322B, 0xFF383B32,
        0xFF32352A, 0xFF2F3325, 0xFF25291B, 0xFFB2B7B3, 0xFFE5E8DF, 0xFFA1A493, 0xFF9CA087, 0xFFB0B396, 0xFFCACAB0, 0xFFE7E4D1, 0xFFE2DFD0, 0xFFDADAB8, 0xFFD2D1B3, 0xFFACAB96, 0xFFD0D1C3, 0xFFFDFEF6, 0xFFFFFFFA, 0xFFFDFFF9, 0xFFD3D8D1, 0xFFE1E3E0,
        0xFFF4F5F0, 0xFFE3E4DC, 0xFFDFE0D2, 0xFFD3D2BE, 0xFFCDCBB2, 0xFFCFCBAE, 0x7DD2CFB0, 0xFFE2DFC0, 0xFFE0DDBE, 0xFFD7D4B5, 0xFFC8C5A6, 0xFFBCB99A, 0xFFC1BE9F, 0xFFD6D3B4, 0xFFE8E5C6, 0xFFC6C3A2, 0xFFBCB998, 0xFFBAB796, 0xFFCCC9A8, 0xFFE4E1C0,
        0xFFEAE7C6, 0xFFD7D4B3, 0xFFC1BE9D, 0xFFBAB796, 0xFFCBC8A7, 0xFFE0DDBC, 0xFFECE9C8, 0xFFE9E6C5, 0xFFDBD8B7, 0xFFCDCAA9, 0xFFC5C2A1, 0xFFC4C09D, 0xFFD4D0AD, 0xFFE8E3C3, 0xFFF2EDCD, 0xFFEDE8CA, 0xFFE1DCBE, 0xFFD7D2B4, 0xFFD2CDB0, 0xFFD3CFB6,
        0xFFE2DDC7, 0xFFF1EFDA, 0xFFFAF7E6, 0x04F9F7EA, 0x00F8F5EC, 0x00FCF9F2, 0x00FEFDF8, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FFFFFF, 0x00FDFDFD, 0x08E7E7E7, 0x64BCBCBC, 0xB0979797, 0xFF5B5F5E, 0xFF313534, 0xFF111514, 0xFF090D0C, 0xFF050908,
        0xFF030706, 0xFF070B0A, 0xFF090D0C, 0xFF090F0D, 0xFF060C0A, 0xFF070D0B, 0xFF080E0C, 0xFF050B09, 0xFF080E0C, 0xFF0A100E, 0xFF060C0A, 0xFF0A100E, 0xFF070D0B, 0xFF060C0A, 0xFF090F0D, 0xFF0A100E, 0xFF070D0B, 0xFF070D0B, 0xFF0A100E, 0xFF070C08,
        0xFF090E0A, 0xFF090E0A, 0xFF070C08, 0xFF090E0A, 0xFF0C110D, 0xFF0B100C, 0xFF070C08, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C,
        0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D,
        0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0D120C, 0xFF0C110B, 0xFF111610, 0xFF2A2F29, 0xFF353A34,
        0xFF2E332D, 0xFF30352F, 0xFF333832, 0xFFB3B5AA, 0xFFF0F0E4, 0xFFB9BAAC, 0xFFA8A999, 0xFFE0DFCB, 0xFFE8E8D0, 0xFFE8E6CD, 0xFFE8E6CD, 0xFFE0E3C6, 0xFFE9EBD3, 0xFFB4B6A1, 0xFFD8D9CB, 0xFFE7E7DF, 0xFFF5F4F0, 0xFFFEFCFF, 0xFFE6E4E9, 0xFFE7E7E7,
        0xFFE9E9E9, 0xFFEDEDED, 0xFFFBFBFB, 0xFFFFFFFF, 0x00F7F7F7, 0x00F0F0F0, 0x11E3E3E3, 0x4BCFCFC7, 0x5CC9C9BF, 0x64C9CABC, 0xFFD3D2C0, 0xFFD6D5C0, 0xFFCDCBB2, 0xFFBDBCA0, 0xFFB2B193, 0xFFD3D0AD, 0xFFE0DDBC, 0xFFDFDCBB, 0xFFCBC6A6, 0xFFB9B496,
        0xFFC1BA9D, 0xFFD7D0B4, 0xFFE9E2C6, 0xFFF0E9CF, 0xFFDED7BB, 0xFFC9C2A6, 0xFFC0B99C, 0xFFC7C1A1, 0xFFD8D2B2, 0xFFE7E1BF, 0xFFEFE9C7, 0xFFEAE7C6, 0xFFDAD7B6, 0xFFC9C6A7, 0xFFC3C0A1, 0xFFCCC9AA, 0xFFDDD9BC, 0xFFEBE7CA, 0xFFF1EDD0, 0xFFEDEBD6,
        0xFFEAE7D4, 0xFFE5E4D2, 0x2AE6E4D7, 0x19ECE9E0, 0x00F2F1EC, 0x00F9F8F4, 0x00FDFCFA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F9F9F9, 0x00F8FCFB, 0x32D3D7D6, 0x96A4A8A7, 0xFB747877, 0xFF454948,
        0xFF222625, 0xFF0E1211, 0xFF020605, 0xFF050B09, 0xFF000402, 0xFF000604, 0xFF070D0B, 0xFF070D0B, 0xFF070D0B, 0xFF080E0C, 0xFF060C0A, 0xFF080E0C, 0xFF060C0A, 0xFF050B09, 0xFF040A08, 0xFF050B09, 0xFF070D0B, 0xFF070D0B, 0xFF080E0C, 0xFF090E0A,
        0xFF0B100C, 0xFF0B100C, 0xFF080D09, 0xFF070C08, 0xFF090E0A, 0xFF0A0F0B, 0xFF080D09, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C,
        0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D,
        0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0D120C, 0xFF0B100A, 0xFF121711, 0xFF2B302A, 0xFF383D37,
        0xFF313630, 0xFF323731, 0xFF343933, 0xFFCBCCC4, 0xFFF2F3EB, 0xFFC9CBC0, 0xFF797A6C, 0xFF7A7B6B, 0xFFB4B39F, 0xFFD2D1BC, 0xFFD0CFBA, 0xFFC4C6B0, 0xFFA7A994, 0xFF7D7E6E, 0xFFC8C9BB, 0xFFE9E9DF, 0xFFD9D8D3, 0xFFE0DFDD, 0xFFE5E3E4, 0xFFECECEC,
        0xFFF8F8F8, 0xFFF9F9F9, 0xFFFBFBFB, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FBFBFB, 0x00FFFFFA, 0x00FAF9F4, 0x21E5E5DB, 0x54D2D0C3, 0x7DC3C2B0, 0xFFC3C0AD, 0xFFCFCDB6, 0xFFDBD9C2, 0xFFC2C1A3, 0xFFBCB89B, 0xFFC0BD9E, 0xFFD5D2B3, 0xFFE7E2C4,
        0xFFE2DDBD, 0xFFCEC8A8, 0xFFBDB797, 0xFFBEB999, 0xFFD3CEAE, 0xFFEAE5C5, 0xFFF2EDCD, 0xFFE8E3C3, 0xFFD6D1B1, 0xFFC8C3A3, 0xFFC1BC9C, 0xFFCBC8A7, 0xFFDBD8B7, 0xFFEBE8C9, 0xFFEEEBCC, 0xFFE3E0C1, 0xFFD6D2B5, 0xFFCFCBAE, 0xFFCFCBAE, 0xFFDDDBC6,
        0xFFE7E4D1, 0xFFF2F1DF, 0x04F8F6E9, 0x00F8F5EC, 0x00F4F3EE, 0x00F5F4F0, 0x00F8F7F5, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FAFEFD, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x04EAEEED,
        0x54C4C8C7, 0xC48D9190, 0xFF5E6261, 0xFF323635, 0xFF1B1F1E, 0xFF0B0F0E, 0xFF070B0A, 0xFF030706, 0xFF020605, 0xFF070B0A, 0xFF0B0F0E, 0xFF040807, 0xFF070B0A, 0xFF090D0C, 0xFF0B0F0E, 0xFF0E1211, 0xFF0E1211, 0xFF090D0C, 0xFF040807, 0xFF040905,
        0xFF090E0A, 0xFF0B100C, 0xFF0A0F0B, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF080D09, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C,
        0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D,
        0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0C110B, 0xFF0B100A, 0xFF121711, 0xFF2E332D, 0xFF3B403A,
        0xFF343933, 0xFF353A34, 0xFF363B35, 0xFF8F918C, 0xFFC5C7C2, 0xFFEDEEE8, 0xFFC5C6BE, 0xFF808076, 0xFF6B6B5F, 0xFF8E8C7F, 0xFFBBB9AA, 0xFFC8C8BC, 0xFFD3D3C7, 0xFFBFBFB3, 0xFFD5D5CB, 0xFFE5E5DB, 0xFFE1E1D9, 0xFFE6E6DE, 0xFFD3D3CB, 0xFFDEDEDE,
        0xFFFBFBFB, 0xFFFFFFFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FAFAFA, 0x00FFFFFD, 0x00FFFFFB, 0x00FFFFF7, 0x08F3F0E7, 0x3BDEDCCF, 0x75C6C3B4, 0x9BB5B2A1, 0xFFD0CEB9, 0xFFDBD9C2, 0xFFD8D6BD, 0xFFC3BFA4, 0xFFB5B293,
        0xFFC2BD9D, 0xFFDAD6B3, 0xFFE9E5C0, 0xFFE1DDB7, 0xFFD3CFAA, 0xFFC4C09B, 0xFFC0BC99, 0xFFCBC6A6, 0xFFDDD8B8, 0xFFEBE6C8, 0xFFF2EDCF, 0xFFDFDCBB, 0xFFD1CEAD, 0xFFC4C1A2, 0xFFC6C3A4, 0xFFD5D2B3, 0xFFE6E2C5, 0xFFEFEBCE, 0xFFF0ECCF, 0xFFE3E1CC,
        0xFFDFDCC9, 0xFFDDDCCA, 0x2EE4E2D5, 0x11F0EDE4, 0x00F9F8F3, 0x00FEFDF9, 0x00FFFEFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFCFC, 0x00FCFCFC, 0x00FCFCFC, 0x00FDFDFD, 0x00FEFEFE, 0x00FFFFFF, 0x00FEFEFE, 0x00FDFDFD, 0x00FBFDFC, 0x00FEFFFF, 0x00FEFFFF, 0x00FCFEFD, 0x00FCFEFD,
        0x00FEFFFF, 0x00FEFFFF, 0x00F8FAF9, 0x2AD8DCDB, 0x7DB0B4B3, 0xE2808483, 0xFF545857, 0xFF2F3332, 0xFF161A19, 0xFF080C0B, 0xFF000302, 0xFF0D1110, 0xFF080C0B, 0xFF050908, 0xFF060A09, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF090D0C, 0xFF060B07,
        0xFF090E0A, 0xFF0C110D, 0xFF0B100C, 0xFF090E0A, 0xFF090E0A, 0xFF0B100C, 0xFF0D120E, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C,
        0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D,
        0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0B100A, 0xFF0A0F09, 0xFF131812, 0xFF30352F, 0xFF3E433D,
        0xFF383D37, 0xFF383D37, 0xFF383D37, 0xFF161A19, 0xFF3A3C39, 0xFF555754, 0xFF8C8E89, 0xFFBABBB6, 0xFFCECFC9, 0xFFEFEFE7, 0xFFFEFEF6, 0xFFEBECE7, 0xFFFAFBF6, 0xFFEEEFE9, 0xFFE2E3DB, 0xFFE5E6DE, 0xFFDCDED3, 0xFFD1D3C6, 0xFFB2B4A7, 0xFFD5D5D5,
        0x00F8F8F8, 0x00FFFFFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FDFDFD, 0x00FCFCFC, 0x00FBFBFB, 0x00FFFEFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFEFC, 0x00FFFFFB, 0x00FFFFFA, 0x00FCF9F2, 0x0DEFECE5, 0x64CAC8BB, 0x8ABAB8A9, 0xFFBDBCAA, 0xFFDAD8C1, 0xFFE7E6CA,
        0xFFD8D5B6, 0xFFC4C19E, 0xFFBEBC96, 0xFFCAC79E, 0xFFDBD7B1, 0xFFECE8C3, 0xFFEBE7C4, 0xFFDBD6B6, 0xFFC9C4A6, 0xFFC1BC9F, 0xFFC1BC9F, 0xFFD4D1B0, 0xFFE6E3C2, 0xFFF3F0D1, 0xFFEAE7C8, 0xFFD3D0B1, 0xFFC5C1A4, 0xFFCBC7AA, 0xFFD6D2B5, 0xFFE5E3CE,
        0xFFF1EEDB, 0xFFFBFAE8, 0x00FAF8EB, 0x0DF2EFE6, 0x0DECEBE6, 0x00F2F1ED, 0x00FBFAF8, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FCFCFC, 0x00FDFFFE, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FDFFFE,
        0x00FCFEFD, 0x00FAFCFB, 0x00F9FBFA, 0x00FCFFFF, 0x00FBFFFE, 0x00F9FDFC, 0x00ECF0EF, 0x3FCED2D1, 0x92A5A9A8, 0xFB747877, 0xFF4A4E4D, 0xFF202423, 0xFF111514, 0xFF050908, 0xFF050908, 0xFF060A09, 0xFF040807, 0xFF060A09, 0xFF0A0E0D, 0xFF090E0A,
        0xFF080D09, 0xFF080D09, 0xFF070C08, 0xFF060B07, 0xFF060B07, 0xFF080D09, 0xFF0B100C, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C,
        0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C,
        0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0F09, 0xFF090E08, 0xFF121711, 0xFF30352F, 0xFF40453F,
        0xFF3A3F39, 0xFF3A3F39, 0xFF393E38, 0xFF191F1D, 0xFF2A2E2D, 0xFF1D2120, 0xFF1D221E, 0xFF363835, 0xFF555754, 0xFF898987, 0xFFAFAFAD, 0xFFDDDDDD, 0xFFE3E3E3, 0xFFE4E4E2, 0xFFDCDDD8, 0xFFE1E2DA, 0xFFE0E2D7, 0xFFDFE1D4, 0xFFDEE0D3, 0x04EAEAEA,
        0x00FFFFFF, 0x00FEFEFE, 0x00FDFDFD, 0x00FFFFFF, 0x00FCFCFC, 0x00FDFDFD, 0x00FFFFFF, 0x00FDFEFF, 0x00FFFEFF, 0x00FEFDFF, 0x00F9F9FB, 0x00FAF8F9, 0x00FEFDFB, 0x00FFFEFD, 0x00FFFCFB, 0x00FCFBF6, 0x08F0F0E8, 0x47D5D5C9, 0x8ABBB9AA, 0x9BB7B6A2,
        0xFFCBC9B2, 0xFFDBDABE, 0xFFDFDEC0, 0xFFD7D4B1, 0xFFC7C4A1, 0xFFBBB897, 0xFFC4C1A0, 0xFFDAD7B8, 0xFFEBE7CA, 0xFFEAE6C9, 0xFFE3DFC4, 0xFFD4D1B0, 0xFFC7C4A3, 0xFFBFBC9D, 0xFFCAC7A8, 0xFFE2DFC0, 0xFFF2EED1, 0xFFF0ECCF, 0xFFE7E3C6, 0xFFDAD8C3,
        0xFFDAD7C4, 0xFFDEDDCB, 0x21EAE8DB, 0x00F8F5EC, 0x00FEFDF8, 0x00FEFDF9, 0x00FBFAF8, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFCFC, 0x00FBFBFB, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FBFBFB, 0x00FDFDFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FDFFFE, 0x00FCFEFD, 0x00FEFFFF, 0x00F9FBFA, 0x08E8EAE9, 0x58C2C4C3, 0xA79B9D9C, 0xFF666867, 0xFF3C3E3D, 0xFF1E201F, 0xFF0E100F, 0xFF090B0A, 0xFF0C0E0D, 0xFF090E0A,
        0xFF070C08, 0xFF080D09, 0xFF0C110D, 0xFF0E130F, 0xFF0B100C, 0xFF080D09, 0xFF070C08, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C,
        0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C,
        0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0F09, 0xFF090E08, 0xFF111610, 0xFF30352F, 0xFF40453F,
        0xFF3B403A, 0xFF3B403A, 0xFF3A3F39, 0xFF343A36, 0xFF1D231F, 0xFF282E2A, 0xFF2E332F, 0xFF3E4241, 0xFF454948, 0xFF2C2E2D, 0xFF313332, 0xFF5F6160, 0xFF848685, 0xFFB2B4B1, 0xFFC6C8C3, 0xFFDDE0D9, 0xFFF9FCF5, 0xFFEEF1E8, 0xFFECEFE6, 0x00FDFDFD,
        0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FEFEFE, 0x00FFFFFF, 0x00F9FAFE, 0x00FEFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFF, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFFFF, 0x00FDFDFB, 0x00FDFCF8, 0x00FDFCF7, 0x15EBEBE1,
        0x60CCCABE, 0x86BAB8AB, 0x8ABBB9AA, 0xFFD4CFB9, 0xFFE3DFC6, 0xFFEDE9D0, 0xFFE4E0C7, 0xFFCFCBB0, 0xFFC2BEA3, 0xFFC6C2A5, 0xFFD0CCAF, 0xFFE0DDBC, 0xFFE7E4C3, 0xFFE8E5C6, 0xFFDCD9BA, 0xFFCCC9AA, 0xFFC7C3A6, 0xFFD1CDB0, 0xFFDEDABD, 0xFFEDEBD6,
        0xFFF3F0DD, 0xFFF7F6E4, 0x11F3F1E4, 0x19ECE9E0, 0x0DEBEAE5, 0x00F5F4F0, 0x00FFFEFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FDFDFD, 0x00FBFBFB, 0x00FEFEFE, 0x00FFFFFF, 0x00FBFBFB, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FCFCFC, 0x00FDFDFD, 0x00FFFFFF, 0x00FDFFFE, 0x00FBFDFC, 0x00FDFFFE, 0x00FDFFFE, 0x00FCFEFD, 0x00FEFFFF, 0x00FEFFFF, 0x00F8FAF9, 0x00FEFFFF, 0x00FEFFFF, 0x00F4F6F5, 0x1DDEE0DF, 0x64BCBEBD, 0xC48D8F8E, 0xFF565857, 0xFF2F3130, 0xFF181D19,
        0xFF0D120E, 0xFF030804, 0xFF020703, 0xFF060B07, 0xFF090E0A, 0xFF090E0A, 0xFF0A0F0B, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C,
        0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C,
        0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0F09, 0xFF080D07, 0xFF10150F, 0xFF2E332D, 0xFF40453F,
        0xFF3B403A, 0xFF3A3F39, 0xFF3A3F39, 0xFF1C261D, 0xFF242E26, 0xFF424C44, 0xFF353C35, 0xFF323834, 0xFF3F4440, 0xFF272B2A, 0xFF2E3231, 0xFF4E504B, 0xFF42443F, 0xFF3B3D38, 0xFF454744, 0xFF7D7F7C, 0xFFDEE0DD, 0xFFF2F4F1, 0x00FEFFFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FEFEFE, 0x00FEFEFE, 0x00FEFFFF, 0x00FCFDFF, 0x00FBFCFE, 0x00FEFEFF, 0x00FFFFFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00F7F8FA, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFF, 0x00FEFCFD,
        0x00FFFEFF, 0x00FCF8F9, 0x04EEEAE9, 0x60CDC9BE, 0x7DBFBBAF, 0x92B7B4A5, 0xFFC5C2AF, 0xFFDDDBC4, 0xFFEAE9CD, 0xFFE1E0C2, 0xFFD3D2B3, 0xFFC3C09F, 0xFFC5C2A1, 0xFFCCC9AA, 0xFFDBD8B9, 0xFFE9E6C7, 0xFFEEEACD, 0xFFE7E3C6, 0xFFDFDBBE, 0xFFD8D6C1,
        0xFFDBD8C5, 0xFFE3E2D0, 0x15F0EEE1, 0x00FCF9F0, 0x00FFFFFA, 0x00FDFCF8, 0x00F9F8F6, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FCFCFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FCFCFC, 0x00FDFDFD, 0x00FDFDFD, 0x00FFFFFF, 0x00FDFDFD, 0x00FAFAFA, 0x00FFFFFF,
        0x00FFFFFF, 0x00FEFEFE, 0x00FDFDFD, 0x00FEFFFF, 0x00FCFEFD, 0x00FDFFFE, 0x00FEFFFF, 0x00FCFEFD, 0x00FDFFFE, 0x00FEFFFF, 0x00FDFFFE, 0x00F9FBFA, 0x00FEFFFF, 0x00FEFFFF, 0x00FBFDFC, 0x00FDFFFE, 0x00FEFFFF, 0x00EEF0EF, 0x2AD7D9D8, 0x9BA1A6A2,
        0xEA7C817D, 0xFF4C514D, 0xFF272C28, 0xFF121713, 0xFF080D09, 0xFF050A06, 0xFF060B07, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C,
        0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C,
        0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0B100A, 0xFF080D07, 0xFF0F140E, 0xFF2D322C, 0xFF3F443E,
        0xFF3A3F39, 0xFF3A3F39, 0xFF393E38, 0xFF313D31, 0xFF3D473E, 0xFF384239, 0xFF323C34, 0xFF3E453E, 0xFF393F3B, 0xFF2B302C, 0xFF333736, 0xFF4E514A, 0xFF545750, 0xFF454742, 0xFF373936, 0xFF70726F, 0xFFF8FAF9, 0x00FEFFFF, 0x00FCFDFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FEFEFE, 0x00FCFCFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFFFF, 0x00FDFFFE, 0x00FCFEFD, 0x00FEFFFF, 0x00FFFFFF, 0x00FCFCFC, 0x00FEFCFD, 0x00FFFEFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFDFF, 0x00FEFDFF, 0x00FFFEFF,
        0x00FFFDFF, 0x00FFFCFF, 0x00FBF8FF, 0x00FFFCFC, 0x00FDF8F4, 0x15EEEAE1, 0x54D6D3C4, 0x8AC0BEA9, 0x9FBAB89F, 0x92C6C5A6, 0xFFD4D4B2, 0xFFEDEAC9, 0xFFE9E6C5, 0xFFDEDBBC, 0xFFCECBAC, 0xFFC2BFA0, 0xFFC6C2A5, 0xFFD8D4B7, 0xFFE8E4C7, 0xFFF1EFDA,
        0xFFEFECD9, 0xFFECEBD9, 0x21EBE9DC, 0x19ECE9E0, 0x04F0EFEA, 0x00F8F7F3, 0x00FEFDFB, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FEFEFE, 0x00EEEEEE, 0x47CACACA, 0xA39D9D9D, 0xFF6F6F6F, 0xFF404040, 0xFF202020, 0xFF0C100F, 0xFF050908, 0xFF040807, 0xFF080C0B, 0xFF080C0B, 0xFF040807, 0xFF080C0B, 0xFF101413, 0xFF040807, 0xFF0A0E0D, 0xFF0B0F0E, 0xFF070B0A, 0xFF050908,
        0xFF070B0A, 0xFF090D0C, 0xFF070B0A, 0xFF0A0F0B, 0xFF060B07, 0xFF050A06, 0xFF080D09, 0xFF090E0A, 0xFF070C08, 0xFF080D09, 0xFF0B100C, 0xFF0A0E0D, 0xFF090D0C, 0xFF070B0A, 0xFF080C0B, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D,
        0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0C110D, 0xFF090E0A, 0xFF0F1410, 0xFF2D322E, 0xFF3E433F,
        0xFF393E3A, 0xFF393E3A, 0xFF3A3F3B, 0xFF3C413B, 0xFF383D37, 0xFF373C36, 0xFF3A3F39, 0xFF393E38, 0xFF363B35, 0xFF222721, 0xFF383D37, 0xFF494B46, 0xFF4F514C, 0xFF464843, 0xFF3D3F3C, 0xFFA4A6A3, 0x00FBFDFC, 0x00FCFEFD, 0x00FEFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00EFEFEF, 0x26DADADA, 0x47CACACA, 0x86B9B7AB, 0x79BFBDB1, 0xFFCCCABD, 0xFFDBDAC8, 0xFFE6E5D1, 0xFFE5E5CD, 0xFFDBDBC1, 0xFFD1D1B5, 0xFFD3D2BE,
        0xFFDCDBC7, 0xFFEAE9D7, 0x08F7F5E8, 0x00FEFBF2, 0x00FFFDF6, 0x00FDFAF5, 0x00FAF6F3, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FCFCFC, 0x00FDFDFD, 0x00F7F7F7, 0x19E0E0E0, 0x47C9C9C9, 0xC98C908F, 0xFF616564, 0xFF2F3332, 0xFF131716, 0xFF0C100F, 0xFF0A0E0D, 0xFF060A09, 0xFF040807, 0xFF0A0E0D, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0B0F0E,
        0xFF090D0C, 0xFF080C0B, 0xFF080C0B, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF070C08, 0xFF070C08, 0xFF080D09, 0xFF070C08, 0xFF060B07, 0xFF090D0C, 0xFF0A0E0D, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0A0E0D, 0xFF080C0B, 0xFF070B0A, 0xFF0A0E0D,
        0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0C110D, 0xFF090E0A, 0xFF0F1410, 0xFF2C312D, 0xFF3D423E,
        0xFF393E3A, 0xFF393E3A, 0xFF3A3F3B, 0xFF383D37, 0xFF343933, 0xFF3D423C, 0xFF3D423C, 0xFF313630, 0xFF3A3F39, 0xFF2D322C, 0xFF30352F, 0xFF4F514C, 0xFF4D4F4A, 0xFF30322D, 0xFF5C5E5B, 0x26DADCD9, 0x00FEFFFF, 0x00F8FAF9, 0x00FEFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FAFAFA, 0x00FDFAF3, 0x19EBE8DF, 0x4FD4D2C6, 0x71C4C2B5, 0x75C4C2B3, 0xFFCFCEBA, 0xFFDDDCC7, 0xFFE7E6D1, 0xFFF1F0DE,
        0xFFEEEDDB, 0x26EAE8D9, 0x21E9E7DB, 0x19ECE9E0, 0x04F4F1EA, 0x00FCF9F4, 0x00FFFEFB, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFCFC,
        0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F7F7F7, 0x00FCFFFF, 0x00F1F5F4, 0x2AD8DCDB, 0x79B1B5B4, 0xE27F8382, 0xFF4D5150, 0xFF262A29, 0xFF121615, 0xFF060A09, 0xFF040807, 0xFF070B0A, 0xFF0C100F, 0xFF0B0F0E,
        0xFF050908, 0xFF040807, 0xFF080C0B, 0xFF060B07, 0xFF0B100C, 0xFF0C110D, 0xFF080D09, 0xFF090E0A, 0xFF0D120E, 0xFF0D120E, 0xFF090E0A, 0xFF060A09, 0xFF070B0A, 0xFF0A0E0D, 0xFF0B0F0E, 0xFF0C100F, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF0A0E0D, 0xFF0A0E0D,
        0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0B100C, 0xFF090E0A, 0xFF0F1410, 0xFF2B302C, 0xFF3C413D,
        0xFF383D39, 0xFF393E3A, 0xFF393E3A, 0xFF3C413B, 0xFF363B35, 0xFF323731, 0xFF343933, 0xFF383D37, 0xFF3C413B, 0xFF262B25, 0xFF313630, 0xFF4B4D48, 0xFF333530, 0xFF50524D, 0x79B3B5B2, 0x00FEFFFD, 0x00FEFFFF, 0x00FDFFFE, 0x00FDFFFE, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FCFCFC, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFEFB, 0x00FFFEFB, 0x00FFFFFA, 0x00FEFBF4, 0x04F5F2E9, 0x2AE6E4D8, 0x4FD5D3C6, 0x64CAC8BB, 0x54D4D2C3,
        0xFFDDDBCE, 0x1DEBE9DD, 0x00F8F5EC, 0x00FFFCF5, 0x00FFFCF7, 0x00FAF6F3, 0x00F6F2EF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F9F9F9,
        0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00F9F9F9, 0x00FBFBFB, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFFFF, 0x00F9FBFA, 0x00F6F8F7, 0x00FBFDFC, 0x00FEFFFF, 0x00F1F3F2, 0x47CACCCB, 0x92A6A8A7, 0xFF717372, 0xFF484A49, 0xFF191B1A, 0xFF050706, 0xFF080A09,
        0xFF0E100F, 0xFF0C0E0D, 0xFF070908, 0xFF050A06, 0xFF080D09, 0xFF080D09, 0xFF070C08, 0xFF070C08, 0xFF0A0F0B, 0xFF0B100C, 0xFF090E0A, 0xFF070B0A, 0xFF070B0A, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF090D0C, 0xFF0A0E0D, 0xFF0C100F, 0xFF0A0E0D,
        0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0B100C, 0xFF090E0A, 0xFF0F1410, 0xFF292E2A, 0xFF3A3F3B,
        0xFF373C38, 0xFF383D39, 0xFF373C38, 0xFF363B35, 0xFF383D37, 0xFF393E38, 0xFF363B35, 0xFF313630, 0xFF353A34, 0xFF272C26, 0xFF393E38, 0xFF383A35, 0xFF4A4C47, 0x79B4B6B1, 0x00FAFCF9, 0x00FBFDFA, 0x00FEFFFF, 0x00FEFFFF, 0x00FCFEFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FDFDFD, 0x00FEFCFD, 0x00FFFDFE, 0x00FFFEFF, 0x00FFFFFD, 0x00FFFFFB, 0x00FFFEFA, 0x00FEFDF8, 0x00FDFCF7, 0x11EDEDE3,
        0x15EBEBE1, 0x15EAEAE2, 0x15EAEAE2, 0x04EFEEE9, 0x00F6F5F1, 0x00FDFCF8, 0x00FFFFFB, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FCFCFC, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FAFCFB, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FCFEFD, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00F7F9F8, 0x0DE6E8E7, 0x58C1C3C2, 0xC48D8F8E, 0xFF595B5A,
        0xFF313332, 0xFF141615, 0xFF050706, 0xFF060B07, 0xFF030804, 0xFF040905, 0xFF090E0A, 0xFF0A0F0B, 0xFF080D09, 0xFF060B07, 0xFF060B07, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF080C0B, 0xFF080C0B, 0xFF070B0A, 0xFF060A09, 0xFF060A09, 0xFF0A0E0D,
        0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0F1410, 0xFF282D29, 0xFF383D39,
        0xFF373C38, 0xFF383D39, 0xFF363B37, 0xFF3A3F39, 0xFF343933, 0xFF3B403A, 0xFF383D37, 0xFF2D322C, 0xFF393E38, 0xFF2D322C, 0xFF2A2F29, 0xFF555752, 0x82B0B2AD, 0x00FEFFFB, 0x00FEFFFD, 0x00F9FBF8, 0x00FEFFFF, 0x00FCFEFD, 0x00FDFFFE, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFDFF, 0x00FFFDFE, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFFFA,
        0x00FFFFFB, 0x00FFFEFA, 0x00FBFAF6, 0x00FAF9F5, 0x00F9F8F4, 0x00FAF9F7, 0x00FBFAF8, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FEFEFE, 0x00FDFDFD, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFCFC, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFCFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FBFBFB, 0x00F0F0F0,
        0x36D2D2D2, 0x9BA2A2A2, 0xEE797979, 0xFF3D423E, 0xFF292E2A, 0xFF131814, 0xFF070C08, 0xFF030804, 0xFF020703, 0xFF080D09, 0xFF0F1410, 0xFF060A09, 0xFF060A09, 0xFF070B0A, 0xFF090D0C, 0xFF0B0F0E, 0xFF0B0F0E, 0xFF090D0C, 0xFF070B0A, 0xFF0A0E0D,
        0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0E130F, 0xFF262B27, 0xFF363B37,
        0xFF363B37, 0xFF383D39, 0xFF353A36, 0xFF393E38, 0xFF393E38, 0xFF30352F, 0xFF333832, 0xFF3B403A, 0xFF2F342E, 0xFF1A1F19, 0xFF484D47, 0x71B8BAB5, 0x00FEFFFB, 0x00FEFFFB, 0x00FCFEFB, 0x00FEFFFD, 0x00FCFEFD, 0x00FCFEFD, 0x00FEFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FDFDFD, 0x00FCFCFC, 0x00FCFCFC, 0x00FBFBFB, 0x00FCFCFC, 0x00FCFCFC, 0x00FEFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFF, 0x00FDFDFF, 0x00FCFCFE, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFD, 0x00FFFFFD, 0x00FFFFFD, 0x00FEFEFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FEFEFE, 0x00FEFEFE, 0x00FDFDFD, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FDFDFD, 0x00FFFFFF,
        0x00FFFFFF, 0x00FBFBFB, 0x00F7F7F7, 0x21DBE0DC, 0x69BABFBB, 0xD1888D89, 0xFF5A5F5B, 0xFF363B37, 0xFF1C211D, 0xFF0B100C, 0xFF010602, 0xFF0E1211, 0xFF0A0E0D, 0xFF070B0A, 0xFF070B0A, 0xFF0B0F0E, 0xFF0D1110, 0xFF0C100F, 0xFF0A0E0D, 0xFF0A0E0D,
        0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0E130F, 0xFF252A26, 0xFF353A36,
        0xFF353A36, 0xFF373C38, 0xFF343935, 0xFF2C312B, 0xFF3D423C, 0xFF373C36, 0xFF343933, 0xFF373C36, 0xFF2E332D, 0xFF464B45, 0x75B4B9B3, 0x00FEFFFB, 0x00FEFFFB, 0x00F3F5F0, 0x00FEFFFD, 0x00FEFFFD, 0x00F6F8F7, 0x00FEFFFF, 0x00FEFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FBFBFB, 0x00FCFCFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF,
        0x00FEFDFF, 0x00FDFDFF, 0x00FDFDFF, 0x00FDFDFD, 0x00FEFEFE, 0x00FFFFFD, 0x00FFFFFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FEFEFE, 0x00FEFEFE, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FCFCFC, 0x00FFFFFF, 0x00FDFDFD, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF,
        0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFFFD, 0x00FCFFFD, 0x00FBFFFC, 0x00ECF1ED, 0x36D1D6D2, 0x8EA8ADA9, 0xFB737874, 0xFF4B504C, 0xFF212524, 0xFF171B1A, 0xFF0B0F0E, 0xFF040807, 0xFF050908, 0xFF080C0B, 0xFF080C0B, 0xFF080C0B, 0xFF0A0E0D,
        0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF090D0C, 0xFF0A0E0D, 0xFF0A0E0D, 0xFF0A0F0B, 0xFF0A0F0B, 0xFF0E130F, 0xFF242925, 0xFF343935,
        0xFF353A36, 0xFF373C38, 0xFF333834, 0xFF363B35, 0xFF30352F, 0xFF373C36, 0xFF30352F, 0xFF262B25, 0xFF626761, 0x75B5BAB4, 0x00FCFFFB, 0x00F9FBF6, 0x00FCFEF9, 0x00FEFFFB, 0x00FEFFFD, 0x00F8FAF7, 0x00FEFFFF, 0x00FDFFFE, 0x00FDFFFE, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFB, 0x00FFFFFD, 0x00FFFFFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x00FDFCFF, 0x00FFFEFF,
        0x00FFFEFF, 0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F7F7F7, 0x00F0F0F0, 0x60BEC2C1, 0xB4969A99, 0xFF626665, 0xFF383C3B, 0xFF1E2221, 0xFF0E1211, 0xFF080C0B, 0xFF090D0C, 0xFF090D0C,
        0xFF040807, 0xFF060A09, 0xFF0E1211, 0xFF0E1211, 0xFF070B0A, 0xFF070B0A, 0xFF0C100F, 0xFF050B09, 0xFF090F0D, 0xFF0B110F, 0xFF070D0B, 0xFF070D0B, 0xFF0A100E, 0xFF0A100E, 0xFF070D0B, 0xFF070C08, 0xFF090E0A, 0xFF0E130F, 0xFF212622, 0xFF343935,
        0xFF323733, 0xFF2F3430, 0xFF373C38, 0xFF333534, 0xFF3A3C3B, 0xFF2C2E2D, 0xFF2C2E2D, 0xFB737574, 0x21DBDDDC, 0x00FEFFFF, 0x00FAFCFB, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FCFCFC, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00F9F9F9, 0x00FCFFFF, 0x00FCFFFF, 0x00EFF3F2, 0x26D9DDDC, 0x71B6BAB9, 0xD1878B8A, 0xFF535756, 0xFF313534, 0xFF171B1A,
        0xFF0A0E0D, 0xFF010504, 0xFF050908, 0xFF0C100F, 0xFF0D1110, 0xFF0B0F0E, 0xFF090D0C, 0xFF0D1311, 0xFF090F0D, 0xFF030907, 0xFF020806, 0xFF070D0B, 0xFF0B110F, 0xFF0A100E, 0xFF060C0A, 0xFF0D120E, 0xFF000501, 0xFF0C110D, 0xFF262B27, 0xFF303531,
        0xFF333834, 0xFF363B37, 0xFF343935, 0xFF303231, 0xFF282A29, 0xFF3F4140, 0xB4959796, 0x00F0F2F1, 0x00FEFFFF, 0x00F9FBFA, 0x00FEFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FDFDFD, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F7FBFA, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00FCFFFF, 0x00F5F9F8, 0x11E3E7E6, 0x36D2D6D5, 0xA79CA09F,
        0xEA7C807F, 0xFF4C504F, 0xFF232726, 0xFF0C100F, 0xFF050908, 0xFF070B0A, 0xFF090D0C, 0xFF040807, 0xFF090D0C, 0xFF0E1211, 0xFF0F1312, 0xFF0A0E0D, 0xFF070B0A, 0xFF0B0F0E, 0xFF111514, 0xFF0C110D, 0xFF0A0F0B, 0xFF181D19, 0xFF272C28, 0xFF2D322E,
        0xFF313632, 0xFF343935, 0xFF2E332F, 0xFF262827, 0xFF585A59, 0x75B4B6B5, 0x00FAFCFB, 0x00FEFFFF, 0x00FCFEFD, 0x00FCFEFD, 0x00FCFEFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFFFE, 0x00FEFFFF, 0x00FCFEFD, 0x00FAFCFB, 0x00FCFEFD, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FDFFFE,
        0x00F5F7F6, 0x19E0E2E1, 0x5CBFC1C0, 0xB0979998, 0xFF6B6D6C, 0xFF3D3F3E, 0xFF1E201F, 0xFF0E1211, 0xFF060A09, 0xFF020605, 0xFF070B0A, 0xFF0C100F, 0xFF0B0F0E, 0xFF080C0B, 0xFF070B0A, 0xFF101511, 0xFF0A0F0B, 0xFF060B07, 0xFF1A1F1B, 0xFF353A36,
        0xFF313632, 0xFF242925, 0xFF2B302C, 0xFB747474, 0x2ED5D5D5, 0x00FFFFFF, 0x00FDFDFD, 0x00FCFCFC, 0x00FFFFFF, 0x00FAFAFA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FDFDFD, 0x00FEFEFE, 0x00FEFFFF, 0x00FDFFFE, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FDFFFE, 0x00FEFFFF, 0x00FEFFFF,
        0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00F2F4F3, 0x2ED5D7D6, 0x69BABCBB, 0xDE828685, 0xFF5E6261, 0xFF323635, 0xFF151918, 0xFF090D0C, 0xFF060A09, 0xFF070B0A, 0xFF090D0C, 0xFF080D09, 0xFF0B100C, 0xFF141915, 0xFF262B27, 0xFF2A2F2B,
        0xFF1F2420, 0xFF454A46, 0xC08F9490, 0x00F5F5F5, 0x00F6F6F6, 0x00FFFFFF, 0x00FFFFFF, 0x00F8F8F8, 0x00FFFFFF, 0x00FFFFFF, 0x00FBFBFB, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FFFFFF, 0x00FDFDFD, 0x00FCFCFC, 0x00FEFEFE, 0x00FFFFFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FCFCFC, 0x00FBFBFB, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FCFCFC, 0x00FEFFFF, 0x00ECEEED, 0x3FCED0CF, 0x8AAAACAB, 0xDE818382, 0xFF535554, 0xFF2A2C2B, 0xFF131514, 0xFF070C08, 0xFF060B07, 0xFF0B100C, 0xFF131814, 0xFF232824,
        0xFF545955, 0x7DAFB4B0, 0x00FCFFFD, 0x00FFFEFF, 0x00FFFEFF, 0x00FAF8F9, 0x00FBF9FA, 0x00FFFEFF, 0x00FCFAFB, 0x00F2F0F1, 0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFE, 0x00FEFEFE, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FCFCFC, 0x00F7F7F7,
        0x00FCFCFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFD, 0x00FCFCFC, 0x00FDFDFD, 0x00FEFEFE, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FBFDFC, 0x0DE6E8E7, 0x58C1C3C2, 0x96A3A5A4, 0xFF696E6A, 0xFF444945, 0xFF1D221E, 0xFF2A2F2B, 0xF2777C78,
        0x3BCFD4D0, 0x00FCFFFD, 0x00FCFFFD, 0x00FBF9FA, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FEFCFD, 0x00FFFEFF, 0x00FFFEFF, 0x00FBF9FA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FAFAFA, 0x00FBFBFB, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FAFCFB, 0x00FEFFFF, 0x00FEFFFF, 0x00FDFFFE, 0x00FEFFFF, 0x00FEFFFF, 0x00FEFFFF, 0x00FAFCFB, 0x00F9FEFA, 0x15E1E6E2, 0x69BABFBB, 0x60BEC3BF, 0x00EEF3EF,
        0x00FCFFFD, 0x00FAFFFB, 0x00FBFFFC, 0x00FEFCFD, 0x00FFFEFF, 0x00FFFDFE, 0x00FAF8F9, 0x00FFFEFF, 0x00FFFEFF, 0x00FEFCFD, 0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF
};

static GUI_CONST_STORAGE GUI_BITMAP _bmJLink_300x132 = {
  300,                                // XSize
  132,                                // YSize
  1200,                               // BytesPerLine
  32,                                 // BitsPerPixel
  (unsigned char *)_acJLink_300x132,  // Pointer to picture data
  NULL,                               // Pointer to palette
  GUI_DRAW_BMPM8888I
};
#else
static GUI_CONST_STORAGE unsigned long _acJLink_300x132[] = {
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFE, 0xFFFEFFFD, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFCFDF9, 0xFFFDFFFE, 0xFFEFF2F0, 0xFBE9ECEA, 0xFFF6F9F7, 0xFFFDFFFE,
        0xFFFDFFFE, 0xFFFAFDFB, 0xFFFBFEFC, 0xFFFBFBFB, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFAFAFA, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFFFD, 0xFFFAFBF9, 0xFFFBFCFA, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFF3F5EF, 0xDEE0E2DC, 0x3F909691, 0x157B817C, 0x04737974, 0x2A868C87, 0x64A1A7A2,
        0x96BAC0BB, 0xD1D6DCD7, 0xFFEFF5F0, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFDFEFC, 0xFFFDFEFC, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF9FAF8, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFC, 0xFBECEDE9, 0xA7C5C7C1, 0x3690928C, 0x00666862, 0x005C625D, 0x005A605B, 0x005B615C, 0x005E645F, 0x005F6560,
        0x00606661, 0x00696F6A, 0x08757B76, 0x3B8E8F8D, 0x64A3A4A2, 0xA7C3C4C2, 0xE6E1E2E0, 0xFFF4F5F3, 0xFFFDFEFC, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFF9F9F9, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFE, 0xFFFEFFFD, 0xDEDDDEDC, 0x60A2A39F, 0x006B6C68, 0x00565852, 0x005D5F59, 0x00696B65, 0x00696F6A, 0x006C726D, 0x006D736E, 0x00696F6A, 0x00666C67,
        0x00666C67, 0x00666C67, 0x00666C67, 0x00676866, 0x00696A68, 0x006C6D6B, 0x00727371, 0x197F807E, 0x4B979896, 0x86B3B4B2, 0xB0C7C8C6, 0xF7E8E8E8, 0xFFF1F1F1, 0xFFFBFBFB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFEFFFD, 0xFFFFFFFE, 0xFFFFFFFE,
        0xFFFEFFFD, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFCFDFB, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xE2E0E4DE, 0x6DA7ABA5, 0x006B6F69, 0x0051554F, 0x004E524C, 0x004D514B, 0x004A4E48, 0x004B4F49, 0x00535751, 0x005A5E58, 0x00646862, 0x006C706A, 0x006F736D,
        0x006F736D, 0x006E726C, 0x006D716B, 0x006D746D, 0x006B726B, 0x00686F68, 0x00656C65, 0x00636A63, 0x00636A63, 0x00646B64, 0x00656C65, 0x0071756F, 0x1D81857F, 0x549B9F99, 0x8AB6BAB4, 0xC0CFD3CD, 0xEEE5E9E3, 0xFFF6FAF4, 0xFFFDFFFB, 0xFFFFFEFF,
        0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFDFCFE, 0xFFFDFCFE, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFDFCFE, 0xFFFDFCFE, 0xFFFEFDFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFEFEFE, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xFFFBFAFC, 0xFFFFFEFF, 0xFFFEFEFE, 0xFFFCFCFC, 0xFFFCFDFB, 0xFFF9FAF8, 0xFFFFFFFE, 0xFFFFFFFE,
        0xFFFFFFFE, 0xFFFDFEFC, 0xFFFCFDFB, 0xFFFFFFFE, 0xFFFFFFFE, 0xD5D9DAD8, 0x69A4A5A3, 0x006C706A, 0x0061655F, 0x005B5F59, 0x005F635D, 0x00646862, 0x00636761, 0x0060645E, 0x005F635D, 0x004F534D, 0x004C504A, 0x00474B45, 0x00444842, 0x00464A44,
        0x004E524C, 0x005A5E58, 0x00626660, 0x00656C65, 0x00666D66, 0x00686F68, 0x00697069, 0x00697069, 0x00697069, 0x00686F68, 0x00686F68, 0x006A6E68, 0x00676B65, 0x00646862, 0x00646862, 0x006A6E68, 0x0D797D77, 0x368D918B, 0x4F9A9E98, 0xA3C0C1BF,
        0xC9D2D3D1, 0xFBEBECEA, 0xFFFEFFFD, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFEFFFD, 0xFFFCFDFB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFEFEFE, 0xFFFEFEFE, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFDFDFD, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBFAFC, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFFFFFE, 0xFFFCFDFB, 0xFFFFFFFE, 0xFFFFFFFE,
        0xFFFAFBF9, 0xFFFFFFFE, 0xFFFFFFFE, 0xF2E7E8E6, 0x7DAFB0AE, 0x08777876, 0x00545553, 0x00585C56, 0x005C605A, 0x0060645E, 0x005E625C, 0x005A5E58, 0x00595D57, 0x005C605A, 0x005F635D, 0x00656963, 0x00656963, 0x00636761, 0x005E625C, 0x00575B55,
        0x004E524C, 0x00474B45, 0x00444842, 0x00464D46, 0x004A514A, 0x00535A53, 0x005B625B, 0x00626962, 0x00676E67, 0x00697069, 0x006A716A, 0x00686C66, 0x00696D67, 0x006A6E68, 0x00686C66, 0x00656963, 0x00636761, 0x00646862, 0x00666A64, 0x00666862,
        0x0071736D, 0x1982847E, 0x3F93958F, 0x69A8AAA4, 0xA7C5C7C1, 0xE6E4E6E0, 0xFFF9FBF5, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFBFBFB, 0xFFFBFBFB, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFDFDFD, 0xFFFEFEFE,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFDFDFD, 0xFFFCFCFC, 0xFFFCFCFC, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFDFF, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFCFCFC, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFF7F8F4, 0xFFFBFCF8, 0xFFFEFFFB,
        0xFFFDFEFA, 0xFBECEDE9, 0x8EB9BAB6, 0x04777874, 0x00555652, 0x005B5C58, 0x00686965, 0x0061655F, 0x0061655F, 0x0060645E, 0x005D615B, 0x005A5E58, 0x00585C56, 0x005A5E58, 0x005C605A, 0x00535751, 0x00585C56, 0x0060645E, 0x00676B65, 0x006A6E68,
        0x006B6F69, 0x00696D67, 0x00676B65, 0x00596059, 0x00525952, 0x00495049, 0x00424942, 0x00404740, 0x00444B44, 0x004B524B, 0x00505750, 0x00626660, 0x00656963, 0x00686C66, 0x006A6E68, 0x00696D67, 0x00686C66, 0x00696D67, 0x00696D67, 0x0063695E,
        0x00656B60, 0x00656B60, 0x0063695E, 0x0062685D, 0x00676D62, 0x0072786D, 0x087B8176, 0x4F979A98, 0x7DADB0AE, 0xC0CDD0CE, 0xFBEAEDEB, 0xFFFBFEFC, 0xFFFDFFFE, 0xFFFDFFFE, 0xFFFDFFFE, 0xFFFAFDFB, 0xFFFBFEFC, 0xFFFCFFFD, 0xFFFCFFFD, 0xFFFBFEFC,
        0xFFFBFEFC, 0xFFFBFEFC, 0xFFFCFFFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFAFAFA, 0xFFFFFFFE, 0xFFFBFCFA, 0xFFF9FAF6, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFEFF0EC,
        0x8EB8B9B5, 0x117D7E7A, 0x005D5E5A, 0x005B5C58, 0x00666763, 0x00676864, 0x005C5D59, 0x005F635D, 0x005A5E58, 0x005A5E58, 0x0060645E, 0x0061655F, 0x005C605A, 0x00585C56, 0x00595D57, 0x005D615B, 0x005B5F59, 0x00595D57, 0x00595D57, 0x005A5E58,
        0x005E625C, 0x0061655F, 0x00646862, 0x00676E67, 0x00676E67, 0x00666D66, 0x00626962, 0x005C635C, 0x00535A53, 0x004B524B, 0x00464D46, 0x00424640, 0x00474B45, 0x004E524C, 0x00565A54, 0x005D615B, 0x00636761, 0x00676B65, 0x006A6E68, 0x006A7065,
        0x006A7065, 0x006B7166, 0x006A7065, 0x00696F64, 0x00676D62, 0x00666C61, 0x00656B60, 0x00616560, 0x00636762, 0x00686C67, 0x00727671, 0x26848883, 0x60A0A49F, 0x9BBDC1BC, 0xC4D0D4CF, 0xFFF5F8F6, 0xFFF8FBF9, 0xFFFDFFFE, 0xFFFDFFFE, 0xFFFDFFFE,
        0xFFFCFFFD, 0xFFFBFEFC, 0xFFFBFEFC, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFFFD, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFCFDF9, 0xFFFFFFFC, 0xFFF9FBF5, 0x86B6B8B2, 0x00747571,
        0x005B5C58, 0x00555652, 0x00646561, 0x006A6B67, 0x00636460, 0x005F605C, 0x0060615D, 0x0060645E, 0x005A5E58, 0x00585C56, 0x005D615B, 0x005E625C, 0x005B5F59, 0x005E625C, 0x00646862, 0x005C605A, 0x005C605A, 0x005C605A, 0x005D615B, 0x005D615B,
        0x005D615B, 0x005C605A, 0x005A5E58, 0x00585F58, 0x00575E57, 0x00575E57, 0x00575E57, 0x00585F58, 0x005A615A, 0x005C635C, 0x005D645D, 0x00626660, 0x005B5F59, 0x0050544E, 0x00464A44, 0x0040443E, 0x0041453F, 0x00444842, 0x00474B45, 0x00555D53,
        0x00575F55, 0x005D655B, 0x00636B61, 0x00697167, 0x006A7268, 0x00687066, 0x00666E64, 0x00656C65, 0x00656C65, 0x00646B64, 0x00626962, 0x005F665F, 0x005F665F, 0x00626962, 0x00656C65, 0x00707671, 0x26838984, 0x64A2A8A3, 0xA7C1C7C2, 0xD9DAE0DB,
        0xFFECF2ED, 0xFFF8FEF9, 0xFFFBFFFC, 0xFFFAFAFA, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFE, 0xFFFDFEFC, 0xFFFFFFFE, 0xFFFAFBF7, 0xFFFFFFFC, 0xFFF7F9F3, 0x26878983, 0x00252721, 0x001F211B,
        0x0030322C, 0x0041433D, 0x00454741, 0x00484A44, 0x00575953, 0x0062645E, 0x005F615B, 0x005C605A, 0x005E625C, 0x0061655F, 0x0061655F, 0x005E625C, 0x005B5F59, 0x005C605A, 0x005E625C, 0x005E625C, 0x005E625C, 0x005D615B, 0x005E625C, 0x005E625C,
        0x005D615B, 0x005B5F59, 0x00585C56, 0x00555C55, 0x00575E57, 0x005A615A, 0x005D645D, 0x005E655E, 0x005D645D, 0x005C635C, 0x005B625B, 0x005F635D, 0x0060645E, 0x0061655F, 0x00626660, 0x005F635D, 0x005A5E58, 0x00555953, 0x0051554F, 0x00444B44,
        0x00434A43, 0x00424942, 0x00454C45, 0x004B524B, 0x00555C55, 0x005F665F, 0x00666D66, 0x00687066, 0x00676F65, 0x00646C62, 0x00626A60, 0x0061695F, 0x0061695F, 0x00636B61, 0x00656D63, 0x00616762, 0x005D635E, 0x00585E59, 0x00585E59, 0x00626863,
        0x0D787E79, 0x43929893, 0x6DA5ABA6, 0xCDD3D3D3, 0xE6DFDFDF, 0xFFF0F0F0, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFFFD, 0xFFFEFFFD, 0xFFFFFFFE, 0xFFFAFBF7, 0xFFFFFFFC, 0x69A8AAA4, 0x00191B15, 0x0011130D, 0x00171913,
        0x0013150F, 0x00171913, 0x001F211B, 0x0021231D, 0x00292B25, 0x00373933, 0x003F413B, 0x00484C46, 0x00535751, 0x005B5F59, 0x005D615B, 0x005E625C, 0x0061655F, 0x0061655F, 0x005D615B, 0x0060645E, 0x005E625C, 0x005D615B, 0x005D615B, 0x005D615B,
        0x005C605A, 0x005A5E58, 0x00585C56, 0x005B625B, 0x005C635C, 0x005F665F, 0x00616861, 0x00646B64, 0x00656C65, 0x00666D66, 0x00666D66, 0x00646862, 0x00636761, 0x0061655F, 0x0060645E, 0x005F635D, 0x005F635D, 0x005E625C, 0x005E625C, 0x005F6560,
        0x00606661, 0x005E645F, 0x00565C57, 0x004D534E, 0x00454B46, 0x00434944, 0x00434944, 0x004C544A, 0x0050584E, 0x00586056, 0x0060685E, 0x00656D63, 0x00666E64, 0x00646C62, 0x00626A60, 0x00606661, 0x00626863, 0x00636964, 0x00606661, 0x005D635E,
        0x005B615C, 0x005D635E, 0x005F6560, 0x005E5E5E, 0x006C6C6C, 0x21818181, 0x54999999, 0x86B1B1B1, 0xBCCBCBCB, 0xEEE4E4E4, 0xFFF5F5F5, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFF, 0xFFFCFBFD, 0xFFFFFDFC, 0xFFFFFFFB, 0xFFFDFBF3, 0x003E4138, 0x000F160F, 0x000D140F, 0x0011150F,
        0x0010140E, 0x0010140E, 0x0010140E, 0x0010140E, 0x0010140E, 0x0010140E, 0x0011150F, 0x00191D17, 0x00222620, 0x002E322C, 0x00383C36, 0x0040443E, 0x004A4E48, 0x00565A54, 0x005E625C, 0x005C605B, 0x005F635E, 0x00616560, 0x00616560, 0x005E625D,
        0x005D615C, 0x0060645F, 0x00626661, 0x0061655F, 0x0061655F, 0x00626660, 0x00626660, 0x00636761, 0x00636761, 0x00636761, 0x00626660, 0x00666A64, 0x00666A64, 0x00656963, 0x00656963, 0x00646862, 0x00626660, 0x0060645E, 0x005F635D, 0x005E625C,
        0x005E625C, 0x005E625C, 0x0060645E, 0x00626660, 0x00636761, 0x00626660, 0x0061655F, 0x00505750, 0x004C534C, 0x00464D46, 0x00434A43, 0x00454C45, 0x004C534C, 0x00545B54, 0x00596059, 0x00616861, 0x00616861, 0x00626962, 0x00616861, 0x00616861,
        0x00606760, 0x005F665F, 0x005E655E, 0x00646B64, 0x00646B64, 0x00626962, 0x005F665F, 0x005F665F, 0x00656C65, 0x006F766F, 0x0D777E77, 0x4B9A9C96, 0x79AFB1AB, 0xB8CED0CA, 0xEEE8EAE4, 0xFFF7F9F3, 0xFFFEFFFA, 0xFFFFFFFB, 0xFFFFFFFB, 0xFFFCFDFB,
        0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFEFFFD, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFDFEFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE,
        0xFFFEFEFE, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBFBFF, 0xFFFDFCFE, 0xFFFFFFFE, 0xFFFFFFFA, 0xDEE5E3DB, 0x00272A21, 0x000A110A, 0x000E1510, 0x0010140E,
        0x0010140E, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x0010140E, 0x0010140E, 0x000A0E08, 0x000D110B, 0x0011150F, 0x00141812, 0x00171B15, 0x001E221C, 0x00272B25, 0x002F332D, 0x0040443F, 0x00484C47, 0x00535752, 0x005B5F5A, 0x005F635E,
        0x00626661, 0x00646863, 0x00666A65, 0x00656963, 0x00656963, 0x00636761, 0x0061655F, 0x005F635D, 0x005E625C, 0x005E625C, 0x005F635D, 0x005D615B, 0x005F635D, 0x00626660, 0x00656963, 0x00666A64, 0x00666A64, 0x00656963, 0x00646862, 0x006A6E68,
        0x00686C66, 0x00666A64, 0x00646862, 0x00646862, 0x00626660, 0x0060645E, 0x005E625C, 0x00656C65, 0x00646B64, 0x00626962, 0x005E655E, 0x00585F58, 0x00515851, 0x004B524B, 0x00474E47, 0x00414841, 0x00464D46, 0x004D544D, 0x00565D56, 0x005C635C,
        0x00616861, 0x00636A63, 0x00636A63, 0x005E655E, 0x005F665F, 0x005F665F, 0x005D645D, 0x005C635C, 0x005D645D, 0x00616861, 0x00646B64, 0x00656761, 0x00666862, 0x006A6C66, 0x0073756F, 0x21858781, 0x5CA1A39D, 0x9FC1C3BD, 0xC9D6D8D2, 0xFFF4F5F3,
        0xFFFAFBF9, 0xFFFEFFFD, 0xFFFDFEFC, 0xFFFDFEFC, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBFBFF, 0xFFFFFEFF, 0xFFFFFEFD, 0xFFFFFEF9, 0xA7CBC9C1, 0x0013160D, 0x00091009, 0x000C130E, 0x000F130D,
        0x000E120C, 0x000E120C, 0x000D110B, 0x000D110B, 0x000E120C, 0x000E120C, 0x000F130D, 0x000E120D, 0x000E120D, 0x000D110C, 0x000B0F0A, 0x000A0E09, 0x000C100B, 0x0010140F, 0x00141813, 0x00131712, 0x001A1E19, 0x00242823, 0x0030342F, 0x003C403B,
        0x00484C47, 0x00555954, 0x005D615C, 0x005F635D, 0x00636761, 0x00686C66, 0x00696D67, 0x00676B65, 0x00646862, 0x00626660, 0x00626660, 0x00656963, 0x00646862, 0x00636761, 0x0061655F, 0x0061655F, 0x00636761, 0x00656963, 0x00666A64, 0x00636761,
        0x00626660, 0x00626660, 0x00636761, 0x00656963, 0x00676B65, 0x00676B65, 0x00676B65, 0x00616861, 0x00626962, 0x00646B64, 0x00656C65, 0x00656C65, 0x00646B64, 0x00636A63, 0x00616861, 0x00606760, 0x00596059, 0x004E554E, 0x00454C45, 0x00424942,
        0x00444B44, 0x004A514A, 0x004E554E, 0x00545B54, 0x00585F58, 0x005E655E, 0x00636A63, 0x00646B64, 0x00616861, 0x005E655E, 0x005B625B, 0x00646660, 0x00656761, 0x00646660, 0x00646660, 0x00646660, 0x00666862, 0x006A6C66, 0x006D6F69, 0x0D797A78,
        0x368D8E8C, 0x75AAABA9, 0xB0C6C7C5, 0xE2DEDFDD, 0xFFF0F1EF, 0xFFFCFDFB, 0xFFFFFFFE, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFEFEFE, 0xFFFDFDFD, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFF, 0xFFFFFEFF, 0xFFFFFCFB, 0xFFFFFFFB, 0x86BBB9B1, 0x000E1108, 0x000E150E, 0x0009100B, 0x000D110B,
        0x000D110B, 0x000C100A, 0x000C100A, 0x000C100A, 0x000C100A, 0x000D110B, 0x000D110B, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000E120D, 0x000D110C, 0x000C100B, 0x00111510, 0x0010140F, 0x000E120D, 0x000D110C, 0x0010140F,
        0x00171B16, 0x0020241F, 0x00272B26, 0x00353933, 0x003E423C, 0x004A4E48, 0x00555953, 0x005C605A, 0x00626660, 0x00686C66, 0x006C706A, 0x00656963, 0x00646862, 0x00636761, 0x0061655F, 0x0061655F, 0x00626660, 0x00656963, 0x00686C66, 0x00626660,
        0x00626660, 0x00626660, 0x00636761, 0x00656963, 0x00676B65, 0x00686C66, 0x00686C66, 0x00656C65, 0x00646B64, 0x00636A63, 0x00626962, 0x00636A63, 0x00656C65, 0x00676E67, 0x00686F68, 0x00626962, 0x00646B64, 0x00666D66, 0x00656C65, 0x005F665F,
        0x00575E57, 0x004E554E, 0x00484F48, 0x00474E47, 0x00474E47, 0x00474E47, 0x004B524B, 0x00535A53, 0x005B625B, 0x00626962, 0x00666D66, 0x005F635D, 0x0060645E, 0x0061655F, 0x00626660, 0x00636761, 0x00626660, 0x0061655F, 0x0060645E, 0x00646863,
        0x00636762, 0x00616560, 0x00616560, 0x006B6F6A, 0x1D80847F, 0x4F999D98, 0x71A9ADA8, 0xCDD3D6D4, 0xE6E0E3E1, 0xFFF1F4F2, 0xFFFCFFFD, 0xFFFDFFFE, 0xFFFBFEFC, 0xFFFCFFFD, 0xFFFDFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE,
        0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFDFDFD, 0xFFFCFCFC, 0xFFFCFCFC, 0xFFFDFDFD, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFF, 0xFFFFFEFF, 0xFFFFFCFB, 0xFFFFFFFB, 0x69ADABA3, 0x000D1007, 0x00111811, 0x00060D08, 0x000C100A,
        0x000C100A, 0x000B0F09, 0x000B0F09, 0x000B0F09, 0x000B0F09, 0x000C100A, 0x000C100A, 0x000A0E09, 0x000A0E09, 0x000B0F0A, 0x000C100B, 0x000D110C, 0x000D110C, 0x000B0F0A, 0x000A0E09, 0x000E120D, 0x000F130E, 0x000F130E, 0x000E120D, 0x000E120D,
        0x000E120D, 0x000F130E, 0x00111510, 0x00131711, 0x00171B15, 0x001E221C, 0x00252923, 0x002E322C, 0x00393D37, 0x00474B45, 0x0050544E, 0x005D615B, 0x0061655F, 0x00676B65, 0x006A6E68, 0x006A6E68, 0x00686C66, 0x00666A64, 0x00656963, 0x00626660,
        0x00626660, 0x00626660, 0x0061655F, 0x0060645E, 0x0060645E, 0x0060645E, 0x0060645E, 0x005E655E, 0x005F665F, 0x00616861, 0x00626962, 0x00636A63, 0x00626962, 0x00616861, 0x00616861, 0x00656C65, 0x00656C65, 0x00656C65, 0x00656C65, 0x00656C65,
        0x00676E67, 0x00686F68, 0x00686F68, 0x00676E67, 0x00606760, 0x00565D56, 0x004F564F, 0x004A514A, 0x00474E47, 0x00454C45, 0x00434A43, 0x005A5E58, 0x005C605A, 0x005F635D, 0x00636761, 0x00666A64, 0x00666A64, 0x00636761, 0x0061655F, 0x00636762,
        0x00666A65, 0x00696D68, 0x00696D68, 0x00666A65, 0x00626661, 0x00616560, 0x00636762, 0x00606361, 0x006F7270, 0x2E878A88, 0x609FA2A0, 0x92B7BAB8, 0xC9D1D4D2, 0xFFEBEEEC, 0xFFFCFFFD, 0xFFFEFFFD, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFDFEFC,
        0xFFFEFFFD, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF9F9FF, 0xFFFDFCFE, 0xFFFFFFFE, 0xFFFFFFFB, 0x439B9991, 0x00070A01, 0x000E150E, 0x00080F0A, 0x000C100A,
        0x000B0F09, 0x000B0F09, 0x000A0E08, 0x000A0E08, 0x000B0F09, 0x000B0F09, 0x000C100A, 0x000C0F0D, 0x000B0E0C, 0x000A0D0B, 0x00090C0A, 0x000A0D0B, 0x000C0F0D, 0x000D100E, 0x000E110F, 0x000C100B, 0x000E120D, 0x0010140F, 0x00121611, 0x00121611,
        0x00111510, 0x000F130E, 0x000E120D, 0x0011150F, 0x0011150F, 0x0010140E, 0x000F130D, 0x0010140E, 0x00131711, 0x00191D17, 0x001E221C, 0x002B2F29, 0x00343832, 0x0040443E, 0x004C504A, 0x00555953, 0x005E625C, 0x00656963, 0x006A6E68, 0x00646862,
        0x00656963, 0x00666A64, 0x00666A64, 0x00646862, 0x00636761, 0x00626660, 0x00626660, 0x00606760, 0x00616861, 0x00626962, 0x00626962, 0x00616861, 0x005F665F, 0x005D645D, 0x005C635C, 0x00626962, 0x00626962, 0x00626962, 0x00626962, 0x00616861,
        0x00606760, 0x00606760, 0x005F665F, 0x00606760, 0x00626962, 0x00636A63, 0x00656C65, 0x00646B64, 0x005F665F, 0x00575E57, 0x00515851, 0x00454C45, 0x00424942, 0x003F463F, 0x00404740, 0x00454C45, 0x004F564F, 0x00596059, 0x005F665F, 0x00606661,
        0x005E645F, 0x005E645F, 0x005F6560, 0x005F6560, 0x00606661, 0x00636964, 0x00686E69, 0x00676D68, 0x00646A65, 0x00606661, 0x005B615C, 0x005C625D, 0x00676D68, 0x0D777D78, 0x26848A85, 0x75ADAEAA, 0x9BBEBFBB, 0xD1D9DAD6, 0xFFF0F1ED, 0xFFFDFEFA,
        0xFFFFFFFC, 0xFFFFFFFC, 0xFFFEFFFB, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF9F9FF, 0xFFFDFCFE, 0xFFFFFFFE, 0xFFFFFFFB, 0x268D8B83, 0x00040700, 0x00091009, 0x000B120D, 0x000B0F09,
        0x000B0F09, 0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000B0F09, 0x000B0F09, 0x000B0E0C, 0x000A0D0B, 0x000A0D0B, 0x00090C0A, 0x00080B09, 0x00090C0A, 0x000A0D0B, 0x000C0F0D, 0x00090D08, 0x00090D08, 0x000A0E09, 0x000B0F0A, 0x000C100B,
        0x000D110C, 0x000D110C, 0x000D110C, 0x000D110B, 0x000E120C, 0x0010140E, 0x0011150F, 0x0011150F, 0x0010140E, 0x0010140E, 0x0010140E, 0x000C100A, 0x0010140E, 0x00161A14, 0x001B1F19, 0x00232721, 0x002D312B, 0x00383C36, 0x0041453F, 0x00565A54,
        0x00595D57, 0x005E625C, 0x0061655F, 0x00626660, 0x00636761, 0x00646862, 0x00656963, 0x00626962, 0x00616861, 0x00606760, 0x005F665F, 0x005F665F, 0x00606760, 0x00626962, 0x00636A63, 0x00606760, 0x00606760, 0x005F665F, 0x005F665F, 0x00606760,
        0x00616861, 0x00626962, 0x00636A63, 0x00646B64, 0x00646B64, 0x00636A63, 0x00646B64, 0x00646B64, 0x00646B64, 0x00626962, 0x00616861, 0x00646B64, 0x00606760, 0x00596059, 0x00505750, 0x00474E47, 0x00414841, 0x003D443D, 0x003B423B, 0x00424942,
        0x00474E47, 0x00535A53, 0x00616861, 0x00686F68, 0x00666D66, 0x00616861, 0x005F665F, 0x00606661, 0x00626863, 0x00636964, 0x00636964, 0x00636964, 0x00636964, 0x00636964, 0x00646A65, 0x005E5F5B, 0x00636460, 0x006E6F6B, 0x117D7E7A, 0x43949591,
        0x7DB0B1AD, 0xB8CDCECA, 0xDEDFE0DC, 0xFFFCFCFC, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFF, 0xFFFEFDFF, 0xFFFFFFFE, 0xFFFFFEF9, 0x1987857D, 0x00060900, 0x00080F08, 0x000C130E, 0x000B0F09,
        0x000B0F09, 0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000B0F09, 0x000B0F09, 0x00080B09, 0x000A0D0B, 0x000D100E, 0x000D100E, 0x000C0F0D, 0x000B0E0C, 0x000A0D0B, 0x00090C0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000C100B, 0x000C100B,
        0x000C100B, 0x000B0F0A, 0x000A0E09, 0x000B0F09, 0x000B0F09, 0x000B0F09, 0x000C100A, 0x000E120C, 0x000F130D, 0x0010140E, 0x0011150F, 0x0011150F, 0x00131711, 0x00131711, 0x0011150F, 0x000F130D, 0x0010140E, 0x00161A14, 0x001A1E18, 0x001F231D,
        0x00272B25, 0x00333731, 0x0040443E, 0x004B4F49, 0x00545852, 0x005C605A, 0x0060645E, 0x00646B64, 0x00646B64, 0x00626962, 0x00616861, 0x00616861, 0x00606760, 0x00606760, 0x00606760, 0x005E655E, 0x005E655E, 0x005E655E, 0x005E655E, 0x005E655E,
        0x005D645D, 0x005D645D, 0x005D645D, 0x00616861, 0x00616861, 0x00616861, 0x00626962, 0x00626962, 0x00636A63, 0x00646B64, 0x00646B64, 0x00606760, 0x00616861, 0x00636A63, 0x00626962, 0x005F665F, 0x005C635C, 0x005B625B, 0x005A615A, 0x00505750,
        0x00484F48, 0x00424942, 0x00434A43, 0x00464D46, 0x004A514A, 0x004F564F, 0x00545B54, 0x005D635E, 0x005F6560, 0x00626863, 0x00656B66, 0x00666C67, 0x00646A65, 0x00616762, 0x005E645F, 0x00666763, 0x00676864, 0x00666763, 0x00636460, 0x0062635F,
        0x00656662, 0x006B6C68, 0x0071726E, 0x328A8A8A, 0x589B9B9B, 0x92B8B8B8, 0xD5D7D7D7, 0xFFEEEEEE, 0xFFFAFAFA, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFEFEFE, 0xFFFEFEFE, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFFFF,
        0xFFFDFFFF, 0xFFFCFEFE, 0xFFFAFFFE, 0xFFFAFFFE, 0xFFFBFFFF, 0xFFFAFFFF, 0xFFFAFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFEFEFE,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFDFF, 0xFFFFFFFF, 0xFFFCFDF9, 0xFFFFFFF9, 0x00696C63, 0x00070903, 0x0010110D, 0x000B0C0A, 0x000A0E08,
        0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09,
        0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000E120D, 0x000E120D, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x0010140F, 0x0010140F, 0x0010140F,
        0x00121611, 0x00141813, 0x00171B16, 0x001C201B, 0x00242823, 0x002E322D, 0x00343833, 0x00454943, 0x004C504A, 0x00555953, 0x005D615B, 0x00636761, 0x00656963, 0x00666A64, 0x00676B65, 0x00636761, 0x00626660, 0x0061655F, 0x00626660, 0x00646862,
        0x00646862, 0x00636761, 0x0061655F, 0x00616861, 0x00626962, 0x00626962, 0x00616861, 0x00616861, 0x00616861, 0x00646B64, 0x00666D66, 0x00676B65, 0x00676B65, 0x00686C66, 0x00696D67, 0x00696D67, 0x00696D67, 0x00686C66, 0x00676B65, 0x00696D67,
        0x00676B65, 0x00646862, 0x00646862, 0x00626660, 0x005B5F59, 0x004E524C, 0x00444842, 0x00434741, 0x00464A44, 0x004D514B, 0x00545852, 0x005A5E58, 0x005E625C, 0x0060645E, 0x0061655F, 0x0061655F, 0x00636761, 0x00656963, 0x00666A64, 0x00676B65,
        0x00676B65, 0x00666A64, 0x00656963, 0x00626A60, 0x00626A60, 0x00636A63, 0x00666D66, 0x00727873, 0x328A908B, 0x71A7ADA8, 0x9BBBC0BE, 0xE6E1E2E0, 0xFFEFF0EE, 0xFFFDFEFC, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFEFFFD, 0xFFFEFDFF,
        0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE,
        0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFDFDFD, 0xFFFCFCFC, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFDFDFD, 0xFFFBFBFB, 0xFFFBFBFB, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFCFCFC, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFCFCFC, 0xFFFFFFFF,
        0xFFFDFDFD, 0xFFFBFBFB, 0xFFFDFDFD, 0xFFFDFFFF, 0xFFFDFFFF, 0xFFFBFFFF, 0xFFF9FEFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFCFF, 0xFFFFFFFF, 0xFFFDFEFA, 0xFFFFFFF9, 0x0064675E, 0x00060802, 0x000F100C, 0x000C0D0B, 0x000A0E08,
        0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09,
        0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000E120D, 0x000E120D, 0x000E120D, 0x0010140F,
        0x00111510, 0x00111510, 0x00121611, 0x00121611, 0x00141813, 0x00181C17, 0x001B1F1A, 0x00191D17, 0x001E221C, 0x00272B25, 0x00323630, 0x003E423C, 0x00494D47, 0x00535751, 0x00595D57, 0x0060645E, 0x0061655F, 0x00636761, 0x00656963, 0x00656963,
        0x00636761, 0x0060645E, 0x005D615B, 0x00616861, 0x00616861, 0x00606760, 0x005E655E, 0x005E655E, 0x005F665F, 0x00626962, 0x00656C65, 0x0061655F, 0x0061655F, 0x00626660, 0x00646862, 0x00676B65, 0x006A6E68, 0x006D716B, 0x006F736D, 0x006B6F69,
        0x006A6E68, 0x006A6E68, 0x006A6E68, 0x006B6F69, 0x006A6E68, 0x00686C66, 0x00676B65, 0x00636761, 0x005E625C, 0x00565A54, 0x004E524C, 0x004A4E48, 0x00494D47, 0x004A4E48, 0x004C504A, 0x00575B55, 0x005B5F59, 0x0060645E, 0x00656963, 0x00686C66,
        0x00696D67, 0x00686C66, 0x00676B65, 0x00636B61, 0x00656D63, 0x00666D66, 0x00646B64, 0x00606661, 0x005E645F, 0x00606661, 0x00636866, 0x006C6D6B, 0x1D80817F, 0x549B9C9A, 0x8AB4B5B3, 0xB8CBCCCA, 0xEEE4E5E3, 0xFFFAFBF9, 0xFFFFFFFE, 0xFFFEFEFE,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFDFDFD, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFDFDFD, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFDFE,
        0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFDFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFCFCFC, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFCFF, 0xFFFFFFFF, 0xFFFEFFFB, 0xFFFCFFF6, 0x005B5E55, 0x00060802, 0x000F100C, 0x000D0E0C, 0x000A0E08,
        0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09,
        0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000C100B, 0x000C100B, 0x000C100B, 0x000C100B, 0x000D110C,
        0x000F130E, 0x0010140F, 0x0010140F, 0x0010140F, 0x000F130E, 0x0010140F, 0x00111510, 0x00141812, 0x00151913, 0x00171B15, 0x00191D17, 0x001C201A, 0x0021251F, 0x00272B25, 0x002B2F29, 0x003E423C, 0x00434741, 0x004C504A, 0x00555953, 0x005C605A,
        0x0061655F, 0x00656963, 0x00676B65, 0x00616861, 0x00616861, 0x00616861, 0x00606760, 0x005E655E, 0x005D645D, 0x005E655E, 0x005F665F, 0x00656963, 0x00656963, 0x00676B65, 0x00676B65, 0x00676B65, 0x00666A64, 0x00666A64, 0x00656963, 0x00686C66,
        0x00696D67, 0x006A6E68, 0x00696D67, 0x00676B65, 0x00686C66, 0x006B6F69, 0x006E726C, 0x00696D67, 0x006A6E68, 0x006B6F69, 0x006A6E68, 0x00686C66, 0x00636761, 0x005E625C, 0x005B5F59, 0x004E524C, 0x004E524C, 0x004E524C, 0x0050544E, 0x00555953,
        0x005C605A, 0x00636761, 0x00676B65, 0x00656D63, 0x00676F65, 0x00697069, 0x006A716A, 0x00696F6A, 0x00696F6A, 0x006A706B, 0x006B706E, 0x00666765, 0x00686967, 0x00676866, 0x00656664, 0x00696A68, 0x0D787977, 0x368C8D8B, 0x4F989997, 0xABC3C3C3,
        0xC9D2D2D2, 0xFBE9E9E9, 0xFFFAFAFA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFAFAFA, 0xFFFCFCFC, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFCFCFC, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF,
        0xFFFFFEFF, 0xFFFFFDFE, 0xFFFFFDFE, 0xFFFFFDFD, 0xFFFFFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFDFDFD, 0xFFFEFEFE, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFCFF, 0xFFFFFFFF, 0xFFFFFFFC, 0xFFF6F9F0, 0x004F5249, 0x00050701, 0x000E0F0B, 0x000E0F0D, 0x000A0E08,
        0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09,
        0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000A0E09, 0x000A0E09, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000C100B, 0x000C100B, 0x000B0F0A,
        0x000D110C, 0x000F130E, 0x0010140F, 0x00111510, 0x00111510, 0x00111510, 0x00121611, 0x000F130D, 0x0011150F, 0x00131711, 0x00151913, 0x00151913, 0x00171B15, 0x00191D17, 0x001B1F19, 0x00161A14, 0x001B1F19, 0x00232721, 0x002C302A, 0x00353933,
        0x003F433D, 0x00484C46, 0x004E524C, 0x005E625C, 0x0061655F, 0x00656963, 0x00686C66, 0x00686C66, 0x00686C66, 0x00676B65, 0x00676B65, 0x0060645E, 0x00626660, 0x00656963, 0x00676B65, 0x00686C66, 0x00686C66, 0x00686C66, 0x00686C66, 0x00666A64,
        0x00676B65, 0x00686C66, 0x00676B65, 0x00666A64, 0x00666A64, 0x00686C66, 0x00696D67, 0x00656963, 0x00666A64, 0x00676B65, 0x00686C66, 0x00696D67, 0x00696D67, 0x00686C66, 0x00686C66, 0x006D716B, 0x006A6E68, 0x00656963, 0x005F635D, 0x00585C56,
        0x0051554F, 0x004C504A, 0x00494D47, 0x0051594F, 0x00555D53, 0x005C635C, 0x00636A63, 0x00686E69, 0x006B716C, 0x006B716C, 0x006B706E, 0x006B6F6A, 0x006E726D, 0x0070746F, 0x006E726D, 0x006B6F6A, 0x006A6E69, 0x006C706B, 0x006E726D, 0x006B6C6A,
        0x00737472, 0x1D80817F, 0x43929391, 0x71A8A9A7, 0xA7C3C4C2, 0xE2DEDFDD, 0xFFEFF0EE, 0xFFFEFFFD, 0xFFFEFFFD, 0xFFFEFFFD, 0xFFFEFFFD, 0xFFFDFEFC, 0xFFFDFEFC, 0xFFFEFFFD, 0xFFFFFFFE, 0xFFFDFEFC, 0xFFFDFEFC, 0xFFFDFEFC, 0xFFFEFFFD, 0xFFFFFFFE,
        0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFCFCFC, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFEFEFE,
        0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFCFCFC, 0xFFFDFDFD, 0xFFFEFEFE, 0xFFFFFEFF,
        0xFFFFFCFD, 0xFFFEF9FA, 0xFFFFFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFDFF, 0xFFFFFFFF, 0xFFFFFFFC, 0xFBEFF2E9, 0x0044473E, 0x00060802, 0x000D0E0A, 0x000E0F0D, 0x000A0E08,
        0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09,
        0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000C100B, 0x000C100B, 0x000C100B, 0x000C100B, 0x000C100B, 0x000C100B,
        0x000C100B, 0x000B0F0A, 0x000B0F0A, 0x000C100B, 0x000C100B, 0x000C100B, 0x000C100B, 0x000C100A, 0x000F130D, 0x0011150F, 0x00131711, 0x00131711, 0x00131711, 0x00131711, 0x00141812, 0x00151913, 0x00171B15, 0x00181C16, 0x00181C16, 0x00191D17,
        0x001A1E18, 0x001E221C, 0x0021251F, 0x0030342E, 0x00363A34, 0x0040443E, 0x004B4F49, 0x00555953, 0x005E625C, 0x00666A64, 0x006B6F69, 0x00686C66, 0x00696D67, 0x006A6E68, 0x00696D67, 0x00676B65, 0x00676B65, 0x00686C66, 0x006A6E68, 0x00656963,
        0x00646862, 0x00636761, 0x00656963, 0x00676B65, 0x00696D67, 0x00696D67, 0x00686C66, 0x006E726C, 0x006C706A, 0x00696D67, 0x00676B65, 0x00676B65, 0x00696D67, 0x006C706A, 0x006E726C, 0x00676B65, 0x00676B65, 0x00696D67, 0x00696D67, 0x00696D67,
        0x00686C66, 0x00666A64, 0x00656963, 0x00575F55, 0x00525A50, 0x004C534C, 0x00495049, 0x004B514C, 0x00515752, 0x00585E59, 0x005C615F, 0x006E726D, 0x006F736E, 0x0070746F, 0x00717570, 0x006F736E, 0x006E726D, 0x0070746F, 0x00727671, 0x00777874,
        0x00757672, 0x0071726E, 0x006D6E6A, 0x006C6D69, 0x0072736F, 0x0D7B7C78, 0x1D838480, 0x43939492, 0x75ABACAA, 0xBCCDCECC, 0xF7E9EAE8, 0xFFF8F9F7, 0xFFFEFFFD, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE,
        0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFEFEFE, 0xFFFDFDFD, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFE,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBFDFD, 0xFFF9FBFB, 0xFFF9FEFD, 0xFFFBFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFDFDFD, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFC, 0xEAE8EBE2, 0x00393C33, 0x00070903, 0x000D0E0A, 0x000E0F0D, 0x00090D07,
        0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08,
        0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000B0F0A, 0x000B0F0A, 0x000C100B, 0x000C100B, 0x000C100B, 0x000C100B, 0x000D110C, 0x000D110C, 0x000C100B,
        0x000B0F0A, 0x000A0E09, 0x000A0E09, 0x000B0F0A, 0x000C100B, 0x000C100B, 0x000C100B, 0x000A0E08, 0x000B0F09, 0x000C100A, 0x000D110B, 0x000D110B, 0x0010140E, 0x00131711, 0x00161A14, 0x00181C16, 0x00191D17, 0x001B1F19, 0x001B1F19, 0x001A1E18,
        0x00181C16, 0x00161A14, 0x00161A14, 0x0012140E, 0x00151711, 0x001A1C16, 0x0021231D, 0x002A2C26, 0x00353731, 0x0040423C, 0x00474943, 0x00555953, 0x005B5F59, 0x00636761, 0x00696D67, 0x006B6F69, 0x006B6F69, 0x006D716B, 0x006E726C, 0x0070746E,
        0x006C706A, 0x00686C66, 0x00666A64, 0x00676B65, 0x00676B65, 0x00656963, 0x00646862, 0x0061655F, 0x00636761, 0x00666A64, 0x00686C66, 0x00696D67, 0x00686C66, 0x00676B65, 0x00666A64, 0x006D716B, 0x006C706A, 0x006A6E68, 0x00696D67, 0x00696D67,
        0x00696D67, 0x006A6E68, 0x006B6F69, 0x00656D63, 0x00666E64, 0x00666D66, 0x00656C65, 0x00606661, 0x00595F5A, 0x00525853, 0x004D5250, 0x004D534E, 0x00505651, 0x00585E59, 0x00616762, 0x00686E69, 0x006A706B, 0x006D736E, 0x00707671, 0x0072746E,
        0x0072746E, 0x0071736D, 0x0070726C, 0x006E706A, 0x006E706A, 0x006F716B, 0x0070726C, 0x006A6B67, 0x006C6D69, 0x006E6F6B, 0x0072736F, 0x157F807C, 0x4B999A96, 0x92BABBB7, 0xC4D2D3CF, 0xFBECEDE9, 0xFFF2F3EF, 0xFFFCFDF9, 0xFFFFFFFC, 0xFFFFFFFC,
        0xFFFFFFFC, 0xFFFEFFFB, 0xFFFCFDF9, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFCFCFC, 0xFFFCFCFC, 0xFFFDFDFD, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF8F8F8, 0xFFF9F9F9, 0xFFF8F8F8, 0xFFF5F5F5, 0xFFF3F3F3, 0xFFFAFAFA, 0xFFFBFBFB, 0xFFFCFCFC, 0xFFFDFDFD, 0xFFFEFEFE,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFBFBFB, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF9FEFD,
        0xFFFBFFFF, 0xFFF7FCFB, 0xF7E8EEED, 0xD5D7DDDC, 0xCDD3D9D8, 0xE2DDE5E4, 0xFFEBF3F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFCFCFC, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFF9F9F9, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xFFFEFEFE, 0xFFFFFFFC, 0xE2E3E6DD, 0x0032352C, 0x00080A04, 0x000C0D09, 0x000D0E0C, 0x00090D07,
        0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08,
        0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000C100B, 0x000C100B, 0x000C100B, 0x000B0F0A,
        0x000A0E09, 0x000A0E09, 0x000B0F0A, 0x000D110C, 0x000F130E, 0x000F130E, 0x000E120D, 0x000D110B, 0x000D110B, 0x000D110B, 0x000B0F09, 0x000A0E08, 0x000B0F09, 0x000E120C, 0x0010140E, 0x000C100A, 0x000D110B, 0x0010140E, 0x00141812, 0x00181C16,
        0x001A1E18, 0x001B1F19, 0x001B1F19, 0x0020221C, 0x001F211B, 0x001C1E18, 0x00191B15, 0x00161812, 0x00171913, 0x00191B15, 0x001B1D17, 0x0020241E, 0x00292D27, 0x00383C36, 0x00454943, 0x0051554F, 0x005B5F59, 0x00646862, 0x006A6E68, 0x006C706A,
        0x006D716B, 0x006E726C, 0x006C706A, 0x006A6E68, 0x00686C66, 0x00676B65, 0x00666A64, 0x00676B65, 0x00676B65, 0x00676B65, 0x00676B65, 0x00676B65, 0x00676B65, 0x00676B65, 0x00676B65, 0x00676B65, 0x00696D67, 0x006D716B, 0x006F736D, 0x006F736D,
        0x006D716B, 0x00696D67, 0x00676B65, 0x00636B61, 0x00646C62, 0x00666D66, 0x00666D66, 0x00666C67, 0x00666C67, 0x00676D68, 0x00676C6A, 0x00646B64, 0x005C635C, 0x00545B54, 0x00505750, 0x004E554E, 0x004E554E, 0x00525952, 0x00575E57, 0x00696C63,
        0x006C6F66, 0x0072756C, 0x00777A71, 0x007A7D74, 0x00797C73, 0x0075786F, 0x0072756C, 0x006E6F6B, 0x0071726E, 0x00747571, 0x00737470, 0x0071726E, 0x0070716D, 0x0070716D, 0x0072736F, 0x157E7F7B, 0x328D8E8A, 0x69A6A7A3, 0x9BBFC0BC, 0xCDD6D7D3,
        0xF7EBECE8, 0xFFFCFDF9, 0xFFFFFFFC, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFEDEDED, 0xFBE9E9E9, 0xEEE4E4E4, 0xDEDCDCDC, 0xD1D6D6D6, 0xD1D5D5D5, 0xD5D7D7D7, 0xDEDBDBDB, 0xEAE1E1E1, 0xF7E7E7E7,
        0xFFEDEDED, 0xFFF1F1F1, 0xFFF4F4F4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF4F4F4, 0xFBEAEAEA, 0xFBEAEAEA, 0xFFF2F2F2, 0xFFFBFBFB, 0xFFFEFEFE, 0xFFFDFDFD, 0xFFFAFFFF,
        0xFFEDF3F2, 0xCDD3DBDA, 0x9FBDC5C4, 0x00B1B9B8, 0x00B1BCBA, 0x9BBCC7C5, 0xB0C6D1CF, 0xC9D2D2D2, 0xD5D8D8D8, 0xDEDBDBDB, 0xD5D7D7D7, 0xC4CFCFCF, 0xB8CACACA, 0xC0CECECE, 0xCDD4D4D4, 0xFFECECEC, 0xFFF2F2F2, 0xFFFAFAFA, 0xFFFEFEFE, 0xFFFEFEFE,
        0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xFFFEFEFE, 0xFFFFFFFC, 0xD9E0E3DA, 0x002E3128, 0x00090B05, 0x000C0D09, 0x000C0D0B, 0x00090D07,
        0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08,
        0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A,
        0x000A0E09, 0x00090D08, 0x000A0E09, 0x000C100B, 0x000C100B, 0x000A0E09, 0x00080C07, 0x00080C06, 0x00090D07, 0x000B0F09, 0x000B0F09, 0x000A0E08, 0x000A0E08, 0x000B0F09, 0x000C100A, 0x00121610, 0x0010140E, 0x000E120C, 0x000F130D, 0x00121610,
        0x00161A14, 0x00171B15, 0x00181C16, 0x00191B15, 0x001B1D17, 0x001C1E18, 0x001C1E18, 0x001B1D17, 0x00191B15, 0x00191B15, 0x001A1C16, 0x00161A14, 0x00161A14, 0x00151913, 0x00151913, 0x00191D17, 0x00232721, 0x0031352F, 0x003B3F39, 0x00484C46,
        0x00525650, 0x005E625C, 0x00656963, 0x00676B65, 0x00676B65, 0x00696D67, 0x006C706A, 0x00676B65, 0x00676B65, 0x00676B65, 0x00676B65, 0x00666A64, 0x00676B65, 0x00676B65, 0x00676B65, 0x00666A64, 0x00666A64, 0x00666A64, 0x00666A64, 0x00676B65,
        0x00696D67, 0x006A6E68, 0x006B6F69, 0x00687066, 0x00697167, 0x00697069, 0x00686F68, 0x00666C67, 0x00656B66, 0x00646A65, 0x00656A68, 0x00666D66, 0x00646B64, 0x00656C65, 0x00697069, 0x00686F68, 0x00616861, 0x005B625B, 0x005A615A, 0x0052554C,
        0x0051544B, 0x0052554C, 0x00575A51, 0x0060635A, 0x006A6D64, 0x0071746B, 0x0074776E, 0x00797A76, 0x00787975, 0x00767773, 0x00757672, 0x00767773, 0x00757672, 0x00737470, 0x0071726E, 0x006D6E6A, 0x006D6E6A, 0x006D6E6A, 0x006D6E6A, 0x00737470,
        0x1D82837F, 0x47969793, 0x69A6A7A3, 0xBCCBCBCB, 0xE2DDDDDD, 0xFFF3F3F3, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xF7E8EAEA, 0xEAE1E3E3, 0x00DDDFDF, 0x00D3D5D5, 0x00C7C9C9, 0x00B4B9B8, 0x00AFB4B3, 0x00AFB4B3, 0x00B1B6B5, 0x7DADB2B1,
        0x79ACB1B0, 0x86B1B6B5, 0x8AB4B9B8, 0x9BBBC0BF, 0xB0C6CBCA, 0xC0CED3D2, 0xE6E0E5E4, 0xE2DEE3E2, 0xF2E6EBEA, 0xFFEBF0EF, 0xFFF1F6F5, 0xE6DFE5E4, 0x7DADB3B2, 0x60A0A6A5, 0x82B0B8B7, 0x79ACB4B3, 0xA3BFCAC8, 0xABC4CFCD, 0xD1D6E1DF, 0xC9D2DDDB,
        0x96BAC5C3, 0x7DADB8B6, 0x00AEBBB9, 0x00B3C0BE, 0x00B9C7C5, 0x00BDCBC9, 0x00BCCAC8, 0x00B0B8B7, 0x00A7B2B0, 0x00A0ABA9, 0x0097A5A1, 0x00A4B2AE, 0x00A2B2AB, 0x00A3B5AE, 0x4F98AAA3, 0x64A2ACA6, 0x6DA6B0AA, 0x8EB6BDB8, 0xBCCBD2CF, 0xD9DADFDD,
        0xF2E5E7E7, 0xFFF1F3F3, 0xFFFDFFFF, 0xFFFCFCFC, 0xFFFCFCFC, 0xFFFFFDFD, 0xFFFFFEFE, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFDFE, 0xFFFFFDFF, 0xFFFFFBFC, 0xFFFFFFFC, 0xC0D5D6CD, 0x0021241B, 0x000E100A, 0x000A0E09, 0x000A0D0B, 0x00080C06,
        0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08,
        0x00090D08, 0x00090D08, 0x00090D08, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000D0E0A,
        0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000D0E0A, 0x000D0E0A, 0x000E0F0B, 0x000C100B, 0x000C100B, 0x000A100B, 0x000B110C, 0x000B110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000E120D,
        0x000E120D, 0x000E120D, 0x000E120D, 0x00141813, 0x00141813, 0x00161A15, 0x00181C17, 0x001A1E19, 0x001B1F1A, 0x001D211C, 0x001E221D, 0x00181C16, 0x00171B15, 0x00161A14, 0x00151913, 0x00141812, 0x00131711, 0x00121610, 0x00121610, 0x001C201B,
        0x001F231E, 0x00252924, 0x0030342F, 0x003D413C, 0x004C504B, 0x00585C57, 0x005F635E, 0x00646862, 0x00656963, 0x00676B65, 0x00696D67, 0x006A6E68, 0x00696D67, 0x00696D67, 0x00686C66, 0x00656963, 0x00656963, 0x00656963, 0x00666A64, 0x00676B65,
        0x00676B65, 0x00676B65, 0x00676B65, 0x00676B65, 0x00676B65, 0x00686C66, 0x00696D67, 0x006A6E68, 0x006B6F69, 0x006C706A, 0x006B6F69, 0x006A6E68, 0x006A6E68, 0x00696D67, 0x00676B65, 0x00656963, 0x00646862, 0x00666A64, 0x00676B65, 0x006C706A,
        0x00696D67, 0x00636761, 0x005D615B, 0x00565A54, 0x00525650, 0x004E524C, 0x004D514B, 0x005D6259, 0x0062675E, 0x00696D67, 0x0070746E, 0x00757974, 0x00767A75, 0x00757974, 0x00747775, 0x00757771, 0x00757771, 0x00767872, 0x00757771, 0x0072746E,
        0x006F716B, 0x006C6E68, 0x006A6C66, 0x006F716B, 0x00757771, 0x1D848680, 0x4F9C9E98, 0x86B5B7B1, 0xB8CED0CA, 0xEEE7E9E3, 0xFFF9FBF5, 0xFFFEFFFD, 0xFFFEFFFD, 0xFFFFFFFE, 0xFFFDFEFC, 0xFFFCFDFB, 0xFFFFFFFE, 0xFFFEFFFD, 0xFFFCFDFB, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFEFEFE, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xF2E5E7E7, 0x00DCDEDE, 0x00D4D6D6, 0x00C5C7C7, 0x00B6B8B8, 0x009EA4A3, 0x009EA4A3, 0x00959B9A, 0x00929897, 0x00969C9B,
        0x008F9594, 0x00878D8C, 0x008D9392, 0x009AA09F, 0x00B6BCBB, 0x00ADB3B2, 0x009DA3A2, 0x009FA5A4, 0x82B0B6B5, 0x7DAEB4B3, 0x9BBBC1C0, 0x7DADB3B2, 0x00989E9D, 0x00959D9C, 0x00A6AEAD, 0x009EA6A5, 0x009BA6A4, 0x009DA8A6, 0x00A5B0AE, 0x009CA7A5,
        0x00A0ABA9, 0x00A3AEAC, 0x00A3B0AE, 0x00AAB7B5, 0x00B5C3C1, 0x00B8C6C4, 0x00AFBDBB, 0x00B3BEBC, 0x00ADB8B6, 0x00AFBAB8, 0x00ACBAB6, 0x00BDCBC7, 0x00BBCBC4, 0x00B3C3BC, 0x00A9B9B2, 0x00A0ACA6, 0x0099A59F, 0x0098A29C, 0x009EA7A4, 0x00A5ACA9,
        0x71A7ADAC, 0x7DAEB3B2, 0x92B7BCBB, 0xC9D2D7D6, 0xE2DDDFDF, 0xFBE9EBEB, 0xFFF4F6F6, 0xFFFBFBFB, 0xFFFDFDFD, 0xFFFFFDFD, 0xFFFFFEFE, 0xFFFFFDFF, 0xFFFFFFFF, 0xFFFFFFFC, 0xB4D0D1C8, 0x001A1D14, 0x00090B05, 0x000A0E09, 0x000B0E0C, 0x00080C06,
        0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08,
        0x00090D08, 0x00090D08, 0x00090D08, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000C0D09,
        0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000E0F0B, 0x000E0F0B, 0x000D0E0A, 0x000D0E0A, 0x000E0F0B, 0x000C100B, 0x000C100B, 0x000A100B, 0x000B110C, 0x000B110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000E120D,
        0x000E120D, 0x000E120D, 0x000E120D, 0x000F130E, 0x0010140F, 0x00111510, 0x00121611, 0x00141813, 0x00151914, 0x00161A15, 0x00171B16, 0x00181C16, 0x00181C16, 0x00181C16, 0x00171B15, 0x00171B15, 0x00171B15, 0x00171B15, 0x00171B15, 0x00131712,
        0x00131712, 0x00131712, 0x00161A15, 0x001A1E19, 0x001F231E, 0x00242823, 0x00272B26, 0x00383C36, 0x0040443E, 0x004D514B, 0x005B5F59, 0x00656963, 0x006A6E68, 0x006B6F69, 0x006A6E68, 0x006B6F69, 0x006B6F69, 0x00696D67, 0x00686C66, 0x00666A64,
        0x00666A64, 0x00666A64, 0x00676B65, 0x00656963, 0x00666A64, 0x00676B65, 0x00696D67, 0x006A6E68, 0x006B6F69, 0x006B6F69, 0x006B6F69, 0x006C706A, 0x006D716B, 0x006F736D, 0x006E726C, 0x006C706A, 0x00696D67, 0x00686C66, 0x00676B65, 0x00646862,
        0x00646862, 0x00636761, 0x00636761, 0x00636761, 0x00636761, 0x00646862, 0x00646862, 0x00575C53, 0x00555A51, 0x00525650, 0x0051554F, 0x00555954, 0x005D615C, 0x00656964, 0x006A6D6B, 0x00787A74, 0x00797B75, 0x007A7C76, 0x007B7D77, 0x007B7D77,
        0x007A7C76, 0x007A7C76, 0x00797B75, 0x00767872, 0x0071736D, 0x006E706A, 0x006D6F69, 0x006D6F69, 0x0070726C, 0x08797B75, 0x1981837D, 0x69A4A5A3, 0x8AB4B5B3, 0xCDD5D6D4, 0xFFECEDEB, 0xFFFDFEFC, 0xFFFFFFFE, 0xFFFEFFFD, 0xFFFFFFFE, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFBFBFB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xF7E7E9E9, 0xE2DEE0E0, 0x00D4D6D6, 0x00C2C4C4, 0x00B2B4B4, 0x008C9493, 0x008E9695, 0x00798180, 0x006E7675, 0x0078807F,
        0x006E7675, 0x005F6766, 0x0068706F, 0x00707B79, 0x009AA5A3, 0x009EA9A7, 0x00717C7A, 0x006D7876, 0x00B2BDBB, 0x00B5C0BE, 0x008E9997, 0x00969C9B, 0x00919998, 0x00919998, 0x00A2AAA9, 0x00A6B1AF, 0x009BA6A4, 0x00A4B1AF, 0x0099A6A4, 0x009FAAA8,
        0x00ABB6B4, 0x00ADB8B6, 0x009FACAA, 0x009AA7A5, 0x009EACAA, 0x00A1AFAD, 0x00A0AEAC, 0x00AAB7B5, 0x0098A5A3, 0x009EABA9, 0x009CAAA6, 0x00ACBAB6, 0x00AFBDB7, 0x00A9B7B1, 0x00A6B6AF, 0x009EACA6, 0x009BA9A3, 0x009FABA5, 0x00A3AEAB, 0x00A2ADAA,
        0x009FAAA8, 0x00A0ABA9, 0x00A5B0AE, 0x009FA7A6, 0x00A2AAA9, 0x6DA6AEAD, 0x7DADB3B2, 0x92B7BDBC, 0xB4C7CDCC, 0xDEDBE0DF, 0xFBE9EEED, 0xFFF5F4F8, 0xFFFCFCFC, 0xFFFFFFFC, 0xA7C7CAC1, 0x00181B12, 0x00070B05, 0x00090D08, 0x00090C0A, 0x00080C06,
        0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08,
        0x00090D08, 0x00090D08, 0x00090D08, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000C0D09,
        0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000E0F0B, 0x000D0E0A, 0x000D0E0A, 0x000E0F0B, 0x000C100B, 0x000C100B, 0x000A100B, 0x000B110C, 0x000B110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000E120D,
        0x000E120D, 0x000E120D, 0x000E120D, 0x000D110C, 0x000D110C, 0x000E120D, 0x000F130E, 0x0010140F, 0x00111510, 0x00111510, 0x00111510, 0x00151913, 0x00151913, 0x00161A14, 0x00161A14, 0x00171B15, 0x00181C16, 0x00181C16, 0x00191D17, 0x00181C17,
        0x00161A15, 0x00141813, 0x00121611, 0x00111510, 0x00111510, 0x00111510, 0x00121611, 0x00171B15, 0x00181C16, 0x001C201A, 0x00232721, 0x002E322C, 0x003B3F39, 0x00474B45, 0x004E524C, 0x005B5F59, 0x0061655F, 0x00676B65, 0x006B6F69, 0x006C706A,
        0x006C706A, 0x006C706A, 0x006D716B, 0x00696D67, 0x00696D67, 0x00676B65, 0x00666A64, 0x00656963, 0x00666A64, 0x00686C66, 0x006A6E68, 0x006C706A, 0x006D716B, 0x006C706A, 0x006B6F69, 0x00696D67, 0x00696D67, 0x006A6E68, 0x006C706A, 0x00676B65,
        0x00676B65, 0x00676B65, 0x00666A64, 0x00656963, 0x00646862, 0x00636761, 0x00636761, 0x006D7269, 0x006D7269, 0x006C706A, 0x00696D67, 0x00646863, 0x005E625D, 0x00595D58, 0x00565957, 0x00575953, 0x005B5D57, 0x0062645E, 0x006A6C66, 0x0072746E,
        0x00787A74, 0x007C7E78, 0x007E807A, 0x007A7C76, 0x00797B75, 0x00797B75, 0x00787A74, 0x00747670, 0x006E706A, 0x006A6C66, 0x006A6C66, 0x00646563, 0x00626361, 0x00686967, 0x04747573, 0x2E898A88, 0x60A0A19F, 0x9FBFC0BE, 0xF2E6E7E5, 0xFFFDFDFD,
        0xFFFFFFFF, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFBFBFB, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFECEEEE, 0xEEE4E6E6, 0x00D8DADA, 0x00C6C8C8, 0x00B6B8B8, 0x0097A2A0, 0x009AA5A3, 0x0085908E, 0x0076817F, 0x00798482,
        0x006A7573, 0x00576260, 0x005D6866, 0x0064716F, 0x00626F6D, 0x00515E5C, 0x005F6C6A, 0x0097A4A2, 0x00AEBBB9, 0x00869391, 0x0093A09E, 0x008B9392, 0x00899190, 0x00858D8C, 0x008E9997, 0x0099A4A2, 0x0093A09E, 0x00B7C4C2, 0x00A4B1AF, 0x0097A2A0,
        0x00A5B0AE, 0x00B2BDBB, 0x00B2BFBD, 0x00A5B2B0, 0x0093A19F, 0x00909E9C, 0x009AA8A6, 0x0094A2A0, 0x00798785, 0x008D9A98, 0x008E9C98, 0x00909E9A, 0x008F9D97, 0x008B9993, 0x008B9791, 0x008F9D97, 0x0097A59F, 0x00A2B0AA, 0x00A6B4B0, 0x009DACA8,
        0x0092A09E, 0x00909E9C, 0x0095A3A1, 0x0098A6A4, 0x0098A6A4, 0x0096A3A1, 0x00929F9D, 0x008E9B99, 0x00909B99, 0x00939E9C, 0x4B96A19F, 0x8EB5B8BC, 0x9FBDC2C1, 0xBCCBD1CC, 0x60A3A89F, 0x000E130A, 0x00080C06, 0x000C100B, 0x000B0E0C, 0x00080C06,
        0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08,
        0x00090D08, 0x00090D08, 0x00090D08, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000C0D09,
        0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000E0F0B, 0x000C100B, 0x000C100B, 0x000A100B, 0x000B110C, 0x000B110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000E120D,
        0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000F130E, 0x000F130E, 0x000F130E, 0x0010140F, 0x0010140F, 0x0010140F, 0x0010140E, 0x0010140E, 0x0011150F, 0x00121610, 0x00131711, 0x00141812, 0x00141812, 0x00151913, 0x00181C17,
        0x00181C17, 0x00171B16, 0x00161A15, 0x00151914, 0x00151914, 0x00151914, 0x00151914, 0x000F130D, 0x000F130D, 0x000E120C, 0x000E120C, 0x00121610, 0x00171B15, 0x001C201A, 0x001F231D, 0x00292D27, 0x00323630, 0x0041453F, 0x004E524C, 0x00585C56,
        0x0060645E, 0x00686C66, 0x006D716B, 0x006C706A, 0x006D716B, 0x006F736D, 0x006F736D, 0x006E726C, 0x006D716B, 0x006D716B, 0x006D716B, 0x00676B65, 0x00696D67, 0x006C706A, 0x006D716B, 0x006C706A, 0x006B6F69, 0x006A6E68, 0x006A6E68, 0x006A6E68,
        0x006B6F69, 0x006C706A, 0x006C706A, 0x006B6F69, 0x006A6E68, 0x00696D67, 0x00686C66, 0x0063685F, 0x00666B62, 0x006A6E68, 0x006E726C, 0x00717570, 0x00717570, 0x0070746F, 0x006F7270, 0x006C706A, 0x00676B65, 0x005E625C, 0x00585C56, 0x00555953,
        0x00585C56, 0x005C605A, 0x0060645E, 0x0070746E, 0x00747872, 0x007B7F79, 0x0081857F, 0x00828680, 0x007E827C, 0x00787C76, 0x00757973, 0x006C706B, 0x006E726D, 0x006D716C, 0x006B6F6A, 0x00646863, 0x00555954, 0x00545853, 0x00686C67, 0xDEDBDBDB,
        0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFBFBFB, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFEFF1F1, 0xF7E8EAEA, 0xDEDCDEDE, 0xBCCCCECE, 0x00BEC0C0, 0x00B4BFBD, 0x00ADB8B6, 0x0098A3A1, 0x00848F8D, 0x00798482,
        0x00717C7A, 0x006D7876, 0x00717C7A, 0x00798684, 0x0083908E, 0x00778482, 0x00697674, 0x00818E8C, 0x00A0ADAB, 0x00879492, 0x008A9795, 0x008D9594, 0x008E9695, 0x008E9997, 0x00919C9A, 0x008B9896, 0x00899694, 0x00B9C7C5, 0x00A9B7B5, 0x0098A3A1,
        0x00A6B1AF, 0x00BAC5C3, 0x00C1CECC, 0x00B2BFBD, 0x0097A5A3, 0x008E9C9A, 0x0095A3A1, 0x007B8C89, 0x005F706D, 0x007C8A88, 0x007F8D89, 0x00778581, 0x007E8A84, 0x0086928C, 0x0086908A, 0x00818D87, 0x0089958F, 0x00939F99, 0x0096A4A0, 0x0091A09C,
        0x008C9D9A, 0x008FA09D, 0x0095A6A3, 0x0093A4A1, 0x0092A3A0, 0x0092A3A0, 0x0096A4A2, 0x0098A6A4, 0x0098A6A4, 0x0096A3A1, 0x0093A09E, 0x008C9396, 0x008B9392, 0x009AA19C, 0x007F887E, 0x00070F05, 0x000A0E08, 0x000C100B, 0x000D0E0C, 0x00080C06,
        0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08,
        0x00090D08, 0x00090D08, 0x00090D08, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000C0D09,
        0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000E0F0B, 0x000C100B, 0x000C100B, 0x000A100B, 0x000B110C, 0x000B110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000E120D,
        0x000E120D, 0x000E120D, 0x000E120D, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000E120C, 0x000F130D, 0x000F130D, 0x000F130D, 0x0010140E, 0x0010140E, 0x0011150F, 0x0011150F, 0x00141813,
        0x00141813, 0x00141813, 0x00131712, 0x00131712, 0x00121611, 0x00121611, 0x00121611, 0x00131711, 0x00141812, 0x00151913, 0x00151913, 0x00151913, 0x00131711, 0x0011150F, 0x000F130D, 0x000E120C, 0x00121610, 0x00171B15, 0x001D211B, 0x00242822,
        0x002E322C, 0x003A3E38, 0x00434741, 0x00565A54, 0x005D615B, 0x00676B65, 0x006E726C, 0x0070746E, 0x006F736D, 0x006E726C, 0x006D716B, 0x006C706A, 0x006B6F69, 0x00696D67, 0x00666A64, 0x00646862, 0x00656963, 0x00676B65, 0x00696D67, 0x00666A64,
        0x00676B65, 0x00696D67, 0x006A6E68, 0x006B6F69, 0x006A6E68, 0x00686C66, 0x00676B65, 0x0061665D, 0x0061665D, 0x0061655F, 0x00626660, 0x00636762, 0x00656964, 0x00676B66, 0x00686B69, 0x00666A64, 0x00696D67, 0x006D716B, 0x006F736D, 0x006D716B,
        0x00676B65, 0x005F635D, 0x005B5F59, 0x00575B55, 0x005A5E58, 0x005E625C, 0x00646862, 0x006A6E68, 0x0071756F, 0x00767A74, 0x00787C76, 0x0070746F, 0x00747873, 0x006E726D, 0x00666A65, 0x005B5F5A, 0x004C504B, 0x004A4E49, 0x00535752, 0x75A9A9A9,
        0xFFFBFBFB, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF7F9F9, 0xFFF3F5F5, 0xFFECEEEE, 0xEAE2E4E4, 0xD9DADCDC, 0xB8C9CFCE, 0xA3BFC5C4, 0x86B2B8B7, 0x60A0A6A5, 0x00909695,
        0x008F9594, 0x00959B9A, 0x00959B9A, 0x00798482, 0x00747F7D, 0x009CA7A5, 0x00ADB8B6, 0x00879290, 0x008A9593, 0x008F9A98, 0x008E9997, 0x00818C8A, 0x00808B89, 0x0086918F, 0x008B9896, 0x0083908E, 0x00818F8D, 0x009CAAA8, 0x008D9B99, 0x0095A09E,
        0x009DA8A6, 0x00A9B4B2, 0x00A8B5B3, 0x00A2AFAD, 0x00A3B1AF, 0x00A8B6B4, 0x00A6B4B2, 0x00829592, 0x00748784, 0x00879895, 0x00798884, 0x006B7975, 0x00717D77, 0x0079837D, 0x006E7570, 0x00676D68, 0x00686F6A, 0x006C7670, 0x00737E7B, 0x007B8985,
        0x00818F8D, 0x00849592, 0x00879895, 0x0095A8A5, 0x0094A5A2, 0x0093A4A1, 0x0097A8A5, 0x009FADAB, 0x00A2B0AE, 0x00A1AEAC, 0x009FACAA, 0x009BA7A9, 0x00919E9C, 0x0097A39D, 0x0079847A, 0x000B140A, 0x00091009, 0x00090D08, 0x000A0B09, 0x00080C06,
        0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08,
        0x00090D08, 0x00090D08, 0x00090D08, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0C08,
        0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000E0F0B, 0x000C100B, 0x000C100B, 0x000A100B, 0x000B110C, 0x000B110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000E120D,
        0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000D110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x0010140E, 0x0010140E, 0x0010140E, 0x0011150F, 0x0011150F, 0x0011150F, 0x0011150F, 0x0011150F, 0x00131712,
        0x00131712, 0x00131712, 0x00131712, 0x00131712, 0x00121611, 0x00131712, 0x00131712, 0x00141812, 0x00131711, 0x0010140E, 0x000F130D, 0x000F130D, 0x0010140E, 0x00131711, 0x00141812, 0x00151913, 0x00141812, 0x00121610, 0x000E120C, 0x000C100A,
        0x000E120C, 0x00121610, 0x00161A14, 0x001C201A, 0x00242822, 0x0030342E, 0x003D413B, 0x00494D47, 0x00555953, 0x0060645E, 0x00676B65, 0x006C706A, 0x006C706A, 0x006B6F69, 0x00696D67, 0x00686C66, 0x00686C66, 0x006A6E68, 0x006D716B, 0x00676B65,
        0x00686C66, 0x00686C66, 0x00676B65, 0x00666A64, 0x00636761, 0x0060645E, 0x005F635D, 0x00656A61, 0x00656A61, 0x00666A64, 0x00666A64, 0x00656964, 0x00636762, 0x00616560, 0x00606361, 0x00636A63, 0x00636A63, 0x00636A63, 0x00646B64, 0x00656C65,
        0x00686F68, 0x006B726B, 0x006C736C, 0x006F766F, 0x006D746D, 0x00676E67, 0x00606760, 0x005D645D, 0x005E655E, 0x005E655E, 0x005C635C, 0x006B716C, 0x00676D68, 0x005A605B, 0x00535954, 0x004F5550, 0x00505651, 0x005A605B, 0x005D635E, 0x368C8C8C,
        0xFFFCFCFC, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFFFF, 0xFFFDFFFF, 0xFFFCFEFE, 0xFFF9FBFB, 0xFFF8FAFA, 0xFFF6F8F8, 0xFFF2F4F4, 0xFFEEF0F0, 0xEEE3E5E5, 0xC9D2D4D4,
        0xB0C6C8C8, 0x96B9BBBB, 0x79ABADAD, 0x5C9EA4A3, 0x00949A99, 0x008D9392, 0x00999F9E, 0x00919796, 0x00848A89, 0x007A807F, 0x008C9291, 0x00808B89, 0x007C8785, 0x007C8987, 0x00869391, 0x0083918F, 0x00879593, 0x0083918F, 0x0083918F, 0x00838E8C,
        0x00808B89, 0x00848F8D, 0x00818E8C, 0x00869391, 0x00A2B0AE, 0x00B4C2C0, 0x00A9B7B5, 0x00718683, 0x008C9F9C, 0x009BACA9, 0x0083928E, 0x00848F8C, 0x008B958F, 0x00818883, 0x005F6560, 0x0050514D, 0x0050514D, 0x00535752, 0x005C6360, 0x006A7370,
        0x0074817F, 0x007A8886, 0x007C8A88, 0x00758683, 0x007D8E8B, 0x008B9997, 0x0099A7A5, 0x00A3B0AE, 0x00A4B1AF, 0x009FACAA, 0x009CA7A5, 0x009EABAD, 0x009AA8A6, 0x009EACA6, 0x0078867B, 0x000A1309, 0x00070E07, 0x00080C07, 0x000A0B09, 0x00080C06,
        0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08,
        0x00090D08, 0x00090D08, 0x00090D08, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0C08,
        0x000B0C08, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000E0F0B, 0x000C100B, 0x000C100B, 0x000A100B, 0x000B110C, 0x000B110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000E120D,
        0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x0010140E, 0x0010140E, 0x0010140E, 0x0011150F, 0x0011150F, 0x00121610, 0x00121610, 0x00121610, 0x0010140F,
        0x0010140F, 0x0010140F, 0x0010140F, 0x0010140F, 0x00111510, 0x00121611, 0x00131712, 0x0011150F, 0x0011150F, 0x00121610, 0x00121610, 0x00131711, 0x00121610, 0x00121610, 0x00121610, 0x00131711, 0x00141812, 0x00151913, 0x00151913, 0x00141812,
        0x00131711, 0x00121610, 0x00121610, 0x0010140E, 0x0011150F, 0x00131711, 0x00141812, 0x00181C16, 0x0021251F, 0x002C302A, 0x00353933, 0x00464A44, 0x004E524C, 0x005A5E58, 0x00636761, 0x00696D67, 0x006A6E68, 0x00696D67, 0x00696D67, 0x006B6F69,
        0x006B6F69, 0x006C706A, 0x006C706A, 0x006B6F69, 0x006A6E68, 0x00686C66, 0x00676B65, 0x00656A61, 0x00656A61, 0x00646862, 0x00646862, 0x00656964, 0x00666A65, 0x00686C67, 0x00696C6A, 0x005D645D, 0x005E655E, 0x005E655E, 0x005F665F, 0x005F665F,
        0x005F665F, 0x005E655E, 0x005D645D, 0x00636A63, 0x00656C65, 0x00656C65, 0x00636A63, 0x00636A63, 0x00636A63, 0x005F665F, 0x005A615A, 0x004C534C, 0x00454C45, 0x00424942, 0x004C534C, 0x00525952, 0x005A615A, 0x00666D66, 0x005D645D, 0x368C8C8C,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFDFDFD, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFFFF, 0xFFFDFFFF, 0xFFFDFFFF, 0xFFFDFFFF, 0xFFFDFFFF, 0xFFFFFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFE, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFEFCFC, 0xFFF5F3F3, 0xF2E6EBEA, 0xEAE2E7E6, 0xC0CED3D2, 0xABC3C8C7, 0x82AFB4B3, 0x6DA6ABAA, 0x4F989D9C, 0x00929796, 0x00838E8C, 0x00838E8C, 0x007E8B89, 0x00818E8C, 0x007C8A88, 0x0082908E, 0x00748280, 0x008B9C99, 0x00818C8A,
        0x006E7977, 0x00697472, 0x00697674, 0x00768381, 0x00A0AEAC, 0x00BFCDCB, 0x00B6C4C2, 0x005A6F6C, 0x0090A5A2, 0x0099AAA7, 0x00788783, 0x008D9895, 0x00A0AAA4, 0x008F9691, 0x00646A65, 0x006A6763, 0x0064635F, 0x005E5F5B, 0x005A5D5B, 0x005A615E,
        0x005D6866, 0x00616E6C, 0x0064716F, 0x006E7C7A, 0x00707E7C, 0x00748280, 0x007A8785, 0x00828F8D, 0x008E9997, 0x0098A3A1, 0x009EA9A7, 0x0093A2A4, 0x009BACA9, 0x00A7B5AF, 0x007A887D, 0x000A1309, 0x00060D06, 0x00090D08, 0x000A0B09, 0x00080C06,
        0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08,
        0x00090D08, 0x00090D08, 0x00090D08, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0C08,
        0x000B0C08, 0x000B0C08, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000E0F0B, 0x000C100B, 0x000C100B, 0x000A100B, 0x000B110C, 0x000B110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000E120D,
        0x000E120D, 0x000E120D, 0x000E120D, 0x0010140F, 0x0010140F, 0x0010140F, 0x0010140F, 0x0010140F, 0x0010140F, 0x0010140F, 0x0010140F, 0x000E120C, 0x000E120C, 0x000F130D, 0x000F130D, 0x0010140E, 0x0011150F, 0x0011150F, 0x00121610, 0x00131712,
        0x00131712, 0x00111510, 0x00111510, 0x0010140F, 0x00111510, 0x00121611, 0x00131712, 0x00121610, 0x00121610, 0x00121610, 0x00121610, 0x00121610, 0x00121610, 0x00121610, 0x00121610, 0x00121610, 0x00121610, 0x0011150F, 0x0011150F, 0x00121610,
        0x00131711, 0x00131711, 0x00131711, 0x00141812, 0x00161A14, 0x00161A14, 0x00141812, 0x0011150F, 0x0010140E, 0x00121610, 0x00151913, 0x00181C16, 0x001C201A, 0x00232721, 0x002E322C, 0x003B3F39, 0x004B4F49, 0x005A5E58, 0x00636761, 0x006D716B,
        0x006C706A, 0x006C706A, 0x006B6F69, 0x006A6E68, 0x00696D67, 0x00676B65, 0x00666A64, 0x00656A61, 0x00666B62, 0x00676B65, 0x00686C66, 0x00676B66, 0x00656964, 0x00636762, 0x00616462, 0x00626962, 0x00626962, 0x00636A63, 0x00636A63, 0x00626962,
        0x00616861, 0x005F665F, 0x005E655E, 0x005B625B, 0x005F665F, 0x00616861, 0x00606760, 0x00606760, 0x00606760, 0x005A615A, 0x00525952, 0x00484F48, 0x003B423B, 0x003A413A, 0x004F564F, 0x00585F58, 0x00606760, 0x006B726B, 0x005C635C, 0x43929292,
        0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFAFAFA, 0xFFF8F8F8, 0xD1D5D5D5, 0x009F9F9F, 0x00909E9C, 0x0093A09E, 0x0098A3A1, 0x0097A1A1, 0x00909C9C, 0x007F8E90, 0x00778B8C, 0x008CA4A4, 0x00859895,
        0x00738683, 0x006D7E7A, 0x006B7C78, 0x00778682, 0x009EAEA7, 0x00BBCBC4, 0x00B7C5BF, 0x004B5F64, 0x00A5BABC, 0x007F9394, 0x00809291, 0x00768783, 0x007A8A83, 0x0072837A, 0x0055655A, 0x0057615B, 0x005F6661, 0x00696D68, 0x00686965, 0x006B6C68,
        0x0070746F, 0x00676E69, 0x00646E68, 0x00626D6A, 0x00606B68, 0x005F6A67, 0x00646D6A, 0x0067706D, 0x006B726F, 0x006C7370, 0x006E7572, 0x00737F79, 0x007A8680, 0x008B958F, 0x00676E69, 0x000E140F, 0x00080C07, 0x00050904, 0x000B0C08, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08,
        0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00040803, 0x00090D08, 0x00121611, 0x000A0E09, 0x00030702, 0x00080C07, 0x000A0E09, 0x000C140A, 0x00060E04, 0x00050D03, 0x00050D03, 0x00081006, 0x00081006, 0x000D150B, 0x00040C02, 0x00050B0A,
        0x00090F0E, 0x00040908, 0x000C110F, 0x000D110C, 0x000C0D09, 0x000C0E08, 0x000A0C06, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000C100B, 0x000C100B, 0x000C100B, 0x000D110C, 0x000D110C,
        0x000D110C, 0x000D110C, 0x000D110C, 0x000E120C, 0x000E120C, 0x000E120C, 0x000E120C, 0x000E120C, 0x000F130D, 0x000F130D, 0x000F130D, 0x000E120D, 0x000F130E, 0x000F130E, 0x000F130E, 0x0010140F, 0x00111510, 0x00111510, 0x00111510, 0x00131410,
        0x00131410, 0x00131410, 0x00131410, 0x00131410, 0x00131410, 0x00131410, 0x00131410, 0x000F160F, 0x000F160F, 0x000F160F, 0x000F160F, 0x000E150E, 0x000E150E, 0x000E150E, 0x000E150E, 0x0011150F, 0x0011150F, 0x0011150F, 0x0011150F, 0x0011150F,
        0x0011150F, 0x0011150F, 0x0011150F, 0x00131712, 0x00131712, 0x00131712, 0x00141813, 0x00141813, 0x00141813, 0x00141813, 0x00151914, 0x00131712, 0x00121611, 0x00111510, 0x00111510, 0x00141813, 0x00191D18, 0x001F231E, 0x00222621, 0x00313530,
        0x003C403B, 0x004C504B, 0x00575B56, 0x005E625D, 0x00636762, 0x006A6E69, 0x006F736E, 0x00666A65, 0x00666A65, 0x00666A65, 0x00646863, 0x00626661, 0x00616560, 0x00636762, 0x00646863, 0x0060645F, 0x006A6E69, 0x00626661, 0x0060645F, 0x00676B66,
        0x00656964, 0x00646863, 0x00616560, 0x005F6560, 0x00606661, 0x00636964, 0x00676D68, 0x00646A65, 0x00575D58, 0x00454B46, 0x00383E39, 0x00494F4A, 0x00505651, 0x00393F3A, 0x00575D58, 0x00595F5A, 0x005F6560, 0x00636964, 0x005B615C, 0x589C9C9C,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBFBFB, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFBE9E9E9, 0xA7C1C1C1, 0x00A1AEAC, 0x00ACB7B5, 0x00B3BBBA, 0x00ABB3B3, 0x00A8B2B2, 0x009DAAAC, 0x008A9E9F, 0x0099AEAF, 0x00A0B1AE,
        0x0095A6A3, 0x008D9B99, 0x0084938F, 0x007F8D89, 0x008C9A94, 0x008E9C96, 0x00727E78, 0x00415052, 0x00839392, 0x00798988, 0x008B9C99, 0x007E8F8C, 0x007E8F8B, 0x006A7975, 0x0041514A, 0x0047534D, 0x003E4842, 0x004B514C, 0x00545853, 0x00595D58,
        0x00616762, 0x0066706A, 0x006B7771, 0x0065706D, 0x005D6865, 0x00535E5B, 0x004F5855, 0x00505956, 0x00505956, 0x004E5552, 0x00484F4C, 0x00404C46, 0x0044504A, 0x0057615B, 0x00444B46, 0x00020803, 0x00070B06, 0x00070B06, 0x000D0E0A, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08,
        0x00090D08, 0x00090D08, 0x00090D08, 0x000E120D, 0x000A0E09, 0x00080C07, 0x00060A05, 0x00050904, 0x000B0F0A, 0x0010140F, 0x00090D08, 0x00060A04, 0x000D110B, 0x000B0F09, 0x000A0E08, 0x00070B05, 0x000C100A, 0x00020600, 0x000F130D, 0x000F1413,
        0x00000100, 0x00101513, 0x000B0E0C, 0x00010500, 0x000D0E0A, 0x00070903, 0x0010120C, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000C100B, 0x000C100B, 0x000C100B, 0x000C100B, 0x000D110C,
        0x000D110C, 0x000D110C, 0x000D110C, 0x000D110B, 0x000E120C, 0x000E120C, 0x000E120C, 0x000E120C, 0x000E120C, 0x000F130D, 0x000F130D, 0x000F130E, 0x000F130E, 0x000F130E, 0x0010140F, 0x0010140F, 0x00111510, 0x00111510, 0x00111510, 0x00141511,
        0x00141511, 0x00141511, 0x00141511, 0x00141511, 0x00141511, 0x00141511, 0x00141511, 0x000F160F, 0x000F160F, 0x000F160F, 0x000F160F, 0x000E150E, 0x000E150E, 0x000E150E, 0x000E150E, 0x0011150F, 0x0011150F, 0x0011150F, 0x0011150F, 0x0011150F,
        0x0011150F, 0x0011150F, 0x0011150F, 0x00121611, 0x00121611, 0x00121611, 0x00131712, 0x00131712, 0x00131712, 0x00131712, 0x00131712, 0x00171B16, 0x00161A15, 0x00141813, 0x00121611, 0x00121611, 0x00121611, 0x00141813, 0x00141813, 0x00121611,
        0x00161A15, 0x001C201B, 0x00222621, 0x00282C27, 0x00333732, 0x003F433E, 0x00494D48, 0x00595D58, 0x005E625D, 0x00666A65, 0x006B6F6A, 0x006C706B, 0x006A6E69, 0x00676B66, 0x00666A65, 0x00696D68, 0x00686C67, 0x00646863, 0x00656964, 0x00626661,
        0x005C605B, 0x005F635E, 0x00636762, 0x00636964, 0x00666C67, 0x00656B66, 0x005B615C, 0x004D534E, 0x00444A45, 0x00444A45, 0x00484E49, 0x00535954, 0x00585E59, 0x00404641, 0x005B615C, 0x005A605B, 0x00606661, 0x00636964, 0x005B615C, 0x609F9F9F,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBFBFB, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFCFCFC, 0xF2E6E6E6, 0x86B1BCBA, 0x00B6BEBD, 0x00BAC0BF, 0x00ABB0B1, 0x00ACB4B4, 0x00A8B4B6, 0x009DACAE, 0x00B4C8C9, 0x00CAD8D7,
        0x00BCCAC8, 0x00AAB7B5, 0x00A1AEAC, 0x0096A19E, 0x008B9693, 0x0074807A, 0x00444E48, 0x00353F39, 0x004D5653, 0x005C6764, 0x007C8987, 0x007B8987, 0x0091A1A0, 0x007D8F8E, 0x0057696A, 0x003F514A, 0x0026342E, 0x002B3731, 0x00363D38, 0x00383F3A,
        0x003B4741, 0x00495751, 0x0052645D, 0x005E6C68, 0x005B6965, 0x0057625F, 0x00525D5A, 0x00545D5A, 0x00505956, 0x00434C49, 0x00363D3A, 0x00333F39, 0x002E3A34, 0x003B453F, 0x00323934, 0x00030904, 0x000A0E09, 0x00070B06, 0x00090A06, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08,
        0x00090D08, 0x00090D08, 0x00090D08, 0x00080C07, 0x00060A05, 0x00080C07, 0x00090D08, 0x00070B06, 0x00070B06, 0x00070B06, 0x00080C07, 0x000D0E0A, 0x00090A06, 0x000B0C08, 0x0011120E, 0x0012130F, 0x00060703, 0x00050602, 0x000F100C, 0x00080C07,
        0x000E120D, 0x000D110C, 0x00020601, 0x000D110C, 0x000F100C, 0x000E0F0B, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000C100B, 0x000C100B, 0x000C100B, 0x000C100B, 0x000D110C,
        0x000D110C, 0x000D110C, 0x000D110C, 0x000D110B, 0x000D110B, 0x000D110B, 0x000E120C, 0x000E120C, 0x000E120C, 0x000E120C, 0x000F130D, 0x000F130E, 0x000F130E, 0x000F130E, 0x0010140F, 0x0010140F, 0x00111510, 0x00111510, 0x00111510, 0x00141511,
        0x00141511, 0x00141511, 0x00141511, 0x00141511, 0x00141511, 0x00141511, 0x00141511, 0x000F160F, 0x000F160F, 0x000F160F, 0x000F160F, 0x000E150E, 0x000E150E, 0x000E150E, 0x000E150E, 0x0010140E, 0x0010140E, 0x0010140E, 0x0010140E, 0x0010140E,
        0x0010140E, 0x0010140E, 0x0010140E, 0x0010140F, 0x00111510, 0x00111510, 0x00111510, 0x00111510, 0x00121611, 0x00121611, 0x00121611, 0x00131712, 0x00131712, 0x00131712, 0x00131712, 0x00131712, 0x00131712, 0x00121611, 0x00121611, 0x00141813,
        0x00141813, 0x00141813, 0x00141813, 0x00141813, 0x00161A15, 0x001B1F1A, 0x001F231E, 0x00292D28, 0x0030342F, 0x00393D38, 0x00444843, 0x004E524D, 0x00585C57, 0x0060645F, 0x00646863, 0x00646863, 0x005F635E, 0x00656964, 0x00676B66, 0x00636762,
        0x00626661, 0x00636762, 0x00666A65, 0x00646A65, 0x00616762, 0x00575D58, 0x00484E49, 0x003D433E, 0x003F4540, 0x004B514C, 0x00565C57, 0x00575D58, 0x005B615C, 0x00444A45, 0x005D635E, 0x005B615C, 0x00616762, 0x00626863, 0x005C625D, 0x6DA5A5A5,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF9F9F9, 0xE6E0E8E7, 0xD5D8DEDD, 0xD1D6DBDA, 0xB4C7CBCC, 0xABC4C9CA, 0x9BBBC4C7, 0x00B0BDBF, 0x00D0E2E3, 0x00E3EFEF,
        0x00C8D4D4, 0x00ADBAB8, 0x00ACB9B7, 0x00A5B0AE, 0x008F9A97, 0x00757E7B, 0x0047504D, 0x003E433A, 0x00323630, 0x00434A45, 0x00586360, 0x00657171, 0x0092A1A3, 0x0084979A, 0x00697C81, 0x00415550, 0x002C3D39, 0x002C3A36, 0x0027322F, 0x0027322F,
        0x0025332F, 0x002A3B37, 0x002D413C, 0x003A4844, 0x0045534F, 0x004C5A56, 0x004F5A57, 0x00515C59, 0x00555E5B, 0x004E5754, 0x00434C49, 0x0035413B, 0x002A3630, 0x002F3933, 0x002A312C, 0x00060C07, 0x000A0E09, 0x00070B06, 0x00090A06, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08,
        0x00090D08, 0x00090D08, 0x00090D08, 0x00080C07, 0x00080C07, 0x000A0E09, 0x00070B06, 0x000A0E09, 0x000D110C, 0x000B0F0A, 0x000F130E, 0x000C0D0B, 0x000D0E0C, 0x000E0F0D, 0x00030402, 0x000D0E0C, 0x00070806, 0x00151614, 0x00151614, 0x0013150F,
        0x00090B05, 0x000B0D07, 0x000F110B, 0x0011120E, 0x00040803, 0x000C100B, 0x000A0E09, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000B0F0A, 0x000C100B, 0x000C100B, 0x000C100B, 0x000C100B,
        0x000D110C, 0x000D110C, 0x000D110C, 0x000D110B, 0x000D110B, 0x000D110B, 0x000D110B, 0x000E120C, 0x000E120C, 0x000E120C, 0x000E120C, 0x000F130E, 0x000F130E, 0x000F130E, 0x0010140F, 0x0010140F, 0x00111510, 0x00111510, 0x00111510, 0x00121611,
        0x00121611, 0x00121611, 0x00121611, 0x00121611, 0x00121611, 0x00121611, 0x00121611, 0x000F160F, 0x000F160F, 0x000F160F, 0x000F160F, 0x000E150E, 0x000E150E, 0x000E150E, 0x000E150E, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D,
        0x000F130D, 0x000F130D, 0x000F130D, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x0010140F, 0x0010140F, 0x0010140F, 0x0010140F, 0x0010140F, 0x00111510, 0x00121611, 0x00141813, 0x00151914, 0x00161A15, 0x00161A15, 0x00161A15, 0x00151914,
        0x00161A15, 0x00181C17, 0x00191D18, 0x00191D18, 0x00181C17, 0x00161A15, 0x00151914, 0x00131712, 0x00141813, 0x00171B16, 0x001B1F1A, 0x00212520, 0x002B2F2A, 0x00353934, 0x003C403B, 0x0050544F, 0x00535752, 0x005D615C, 0x005F635E, 0x00646863,
        0x006B6F6A, 0x00676B66, 0x00656964, 0x00595F5A, 0x004E544F, 0x00414742, 0x003C423D, 0x00454B46, 0x00515752, 0x00575D58, 0x00575D58, 0x00515752, 0x00555B56, 0x00434944, 0x005C625D, 0x005A605B, 0x00626863, 0x00616762, 0x005D635E, 0x7DADADAD,
        0xFFFEFEFE, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFBFFFF, 0xFFF9FEFD, 0xFFFCFEFE, 0xFFF7F6F8, 0xFFF5F7F8, 0xF7E8EDF0, 0xBCCBD7D9, 0x00D6E3E5, 0x00DAE8E7,
        0x00BDCBCA, 0x00AAB8B7, 0x00B3C0BE, 0x00A9B6B4, 0x008D9A98, 0x007A8582, 0x00626D6A, 0x00585D54, 0x00414841, 0x004D544F, 0x004C5754, 0x005D6B6A, 0x008D9FA0, 0x007B8E91, 0x005F7378, 0x004E645F, 0x004B5C58, 0x0043524E, 0x002C3734, 0x0036413E,
        0x00374642, 0x00384945, 0x00364C47, 0x00283733, 0x00354440, 0x003C4A46, 0x00364440, 0x00343F3C, 0x003B4643, 0x00454E4B, 0x0048514E, 0x003B4741, 0x00303C36, 0x002F3933, 0x00272E29, 0x00080E09, 0x00060A05, 0x00060A05, 0x000A0B07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08,
        0x00090D08, 0x00090D08, 0x00090D08, 0x00080C07, 0x000D110C, 0x000E120D, 0x00030702, 0x00080C07, 0x000F130E, 0x00050904, 0x00060A05, 0x00030200, 0x0012110D, 0x000D0C08, 0x001A1915, 0x000E0D09, 0x00070602, 0x00020100, 0x007A7975, 0x008F8E84,
        0x000B0901, 0x00070800, 0x00181812, 0x00020400, 0x000A0E09, 0x00090C0A, 0x000A0F0D, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000C100B, 0x000C100B,
        0x000C100B, 0x000C100B, 0x000D110C, 0x000D110B, 0x000D110B, 0x000D110B, 0x000D110B, 0x000E120C, 0x000E120C, 0x000E120C, 0x000E120C, 0x000E120D, 0x000E120D, 0x000F130E, 0x000F130E, 0x0010140F, 0x0010140F, 0x00111510, 0x00111510, 0x00121611,
        0x00121611, 0x00121611, 0x00121611, 0x00121611, 0x00121611, 0x00121611, 0x00121611, 0x000F160F, 0x000F160F, 0x000F160F, 0x000F160F, 0x000E150E, 0x000E150E, 0x000E150E, 0x000E150E, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D,
        0x000F130D, 0x000F130D, 0x000F130D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x00121611, 0x00121611, 0x00121611, 0x00121611, 0x00121611, 0x00121611, 0x00131712, 0x00131712, 0x00131712,
        0x00131712, 0x00131712, 0x00141813, 0x00161A15, 0x00181C17, 0x001A1E19, 0x001A1E19, 0x001B1F1A, 0x001B1F1A, 0x001B1F1A, 0x001A1E19, 0x00181C17, 0x00181C17, 0x00191D18, 0x001A1E19, 0x001D211C, 0x00292D28, 0x00333732, 0x003B3F3A, 0x00474B46,
        0x0050544F, 0x00545853, 0x005B5F5A, 0x00434944, 0x003F4540, 0x003C423D, 0x00434944, 0x00555B56, 0x00616762, 0x005D635E, 0x00525853, 0x004D534E, 0x00515752, 0x00444A45, 0x005D635E, 0x005C625D, 0x00636964, 0x005F6560, 0x005C625D, 0x8EB5B5B5,
        0xFFFEFEFE, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBFDFD, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFDFF, 0xFFFFFEFF, 0xFFFBFEFF, 0xE2DDE4E7, 0x00BDC9CB, 0x00C1D5D6,
        0x00BACCCD, 0x00BFD1D0, 0x00C5D7D6, 0x00B8C9C6, 0x00A1B2AF, 0x0095A3A1, 0x008E9D99, 0x007A8680, 0x006A7572, 0x006E7C78, 0x005A6866, 0x00677875, 0x00889A99, 0x006D8280, 0x004C6061, 0x00495C59, 0x0051625F, 0x00515E5C, 0x00343F3D, 0x003F4A48,
        0x00384543, 0x002C3D3A, 0x00233633, 0x002A3935, 0x0032413D, 0x0033423E, 0x00283632, 0x001D2B27, 0x00222D2A, 0x002C3734, 0x00343F3C, 0x00404C46, 0x0039453F, 0x00353F39, 0x00272E29, 0x000B110C, 0x00040803, 0x00060A05, 0x000A0B07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08,
        0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00050904, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x00010500, 0x00030702, 0x002A2E29, 0x0053504B, 0x0015120D, 0x000D0A05, 0x000E0B06, 0x009C9994, 0x006E6B66, 0x00040100, 0x002A2722, 0x00CFCBC0,
        0x0057554A, 0x00040300, 0x000C0D04, 0x00070903, 0x0010140F, 0x000E1311, 0x00060B0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000C100B,
        0x000C100B, 0x000C100B, 0x000C100B, 0x000C100A, 0x000D110B, 0x000D110B, 0x000D110B, 0x000D110B, 0x000E120C, 0x000E120C, 0x000E120C, 0x000D110C, 0x000D110C, 0x000E120D, 0x000E120D, 0x000F130E, 0x000F130E, 0x0010140F, 0x0010140F, 0x000F1510,
        0x000F1510, 0x000F1510, 0x000F1510, 0x000F1510, 0x000F1510, 0x000F1510, 0x000F1510, 0x000F160F, 0x000F160F, 0x000F160F, 0x000F160F, 0x000E150E, 0x000E150E, 0x000E150E, 0x000E150E, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D,
        0x000F130D, 0x000F130D, 0x000F130D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x0010140F, 0x000F130E, 0x000F130E, 0x000E120D, 0x000E120D, 0x000E120D, 0x000F130E, 0x0010140F, 0x00161A15,
        0x00151914, 0x00141813, 0x00131712, 0x00141813, 0x00151914, 0x00151914, 0x00161A15, 0x00171B16, 0x001A1E19, 0x001C201B, 0x001D211C, 0x001D211C, 0x001D211C, 0x001D211C, 0x001E221D, 0x00171B16, 0x001D211C, 0x001C201B, 0x0020241F, 0x001D211C,
        0x001B1F1A, 0x00292D28, 0x0030342F, 0x00353B36, 0x00404641, 0x004C524D, 0x00555B56, 0x005C625D, 0x00606661, 0x005A605B, 0x00505651, 0x004F5550, 0x00535954, 0x00494F4A, 0x00606661, 0x005E645F, 0x00666C67, 0x005E645F, 0x005C625D, 0x9FBDBDBD,
        0xFFFEFEFE, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFCFD, 0xFFFFFBFD, 0xFFFFFCFE, 0xFFFDFEFF, 0xF2E6EBEE, 0x8EB5BEC1, 0x00B7CFCF,
        0x00C3DBDB, 0x00DCF1F2, 0x00D2E8E6, 0x00C3D9D7, 0x00C3D8D5, 0x00B7CCC9, 0x00B0C5C2, 0x00A7BBBC, 0x009EB3B1, 0x009DB2B0, 0x007D928F, 0x007A8F8C, 0x00839994, 0x00647A75, 0x00394E46, 0x00344542, 0x003E4B49, 0x004E5957, 0x00454B4A, 0x00535958,
        0x00444F4D, 0x00394644, 0x002E3F3C, 0x00263531, 0x00283733, 0x00283733, 0x00253430, 0x0022302C, 0x0022302C, 0x0026312E, 0x00293431, 0x002D3933, 0x002D3933, 0x002A342E, 0x001E2520, 0x000A100B, 0x00040803, 0x00080C07, 0x000C0D09, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08,
        0x00090D08, 0x00090D08, 0x00090D08, 0x000E120D, 0x00020601, 0x00060A05, 0x00090D08, 0x000B0F0A, 0x00020601, 0x0010140F, 0x00515550, 0x00FFFFF6, 0x00C7C5BB, 0x003D3B31, 0x0018160C, 0x00747268, 0x00E2E0D6, 0x00706E64, 0x00050300, 0x00A49E91,
        0x00B9B6A8, 0x001B190E, 0x000C0A02, 0x0012140E, 0x00050904, 0x000E1311, 0x00060C0B, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000C100B,
        0x000C100B, 0x000C100B, 0x000C100B, 0x000C100A, 0x000C100A, 0x000D110B, 0x000D110B, 0x000D110B, 0x000D110B, 0x000D110B, 0x000E120C, 0x000C100B, 0x000C100B, 0x000D110C, 0x000D110C, 0x000E120D, 0x000E120D, 0x000F130E, 0x000F130E, 0x000E140F,
        0x000E140F, 0x000E140F, 0x000E140F, 0x000E140F, 0x000E140F, 0x000E140F, 0x000E140F, 0x000F160F, 0x000F160F, 0x000F160F, 0x000F160F, 0x000E150E, 0x000E150E, 0x000E150E, 0x000E150E, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D,
        0x000F130D, 0x000F130D, 0x000F130D, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x0010140F, 0x0010140F, 0x0010140F, 0x000C100B, 0x000C100B, 0x000D110C, 0x000D110C, 0x000E120D, 0x000F130E, 0x0010140F, 0x0010140F, 0x000D110C,
        0x000E120D, 0x0010140F, 0x00121611, 0x00121611, 0x00111510, 0x0010140F, 0x0010140F, 0x00121611, 0x00131712, 0x00141813, 0x00141813, 0x00151914, 0x00181C17, 0x001B1F1A, 0x001E221D, 0x001E221D, 0x00232722, 0x001C201B, 0x001E221D, 0x00171B16,
        0x00242823, 0x003F433E, 0x002A2E29, 0x003E443F, 0x004C524D, 0x00595F5A, 0x005C625D, 0x005C625D, 0x005C625D, 0x00565C57, 0x004F5550, 0x00505651, 0x00545A55, 0x004B514C, 0x00616762, 0x005E645F, 0x00666C67, 0x005E645F, 0x005E645F, 0xABC3C3C3,
        0xFFFEFEFE, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFBFBFB, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFEFE, 0xFFFFFBFC, 0xFFFFFDFF, 0xFFFFFDFF, 0xFFFEFDFF, 0xFFF6FBFE, 0xC0CED5D8, 0x8AB4D0D0,
        0x00CCE6E6, 0x00E4FEFE, 0x00C8E3E0, 0x00BFD7D5, 0x00D3EBE9, 0x00C7DEDA, 0x00B8CFCB, 0x00CBE6EA, 0x00C9E2E4, 0x00CCE6E6, 0x00ADC5C3, 0x00A0B7B2, 0x0097AEA6, 0x00778C83, 0x0041574B, 0x00455250, 0x00333E3C, 0x00383E3D, 0x00282D2C, 0x00323736,
        0x002D3332, 0x003A4543, 0x0043504E, 0x00374844, 0x0033423E, 0x002D3C38, 0x002E3D39, 0x00303E3A, 0x002E3C38, 0x002A3532, 0x0026312E, 0x0024302A, 0x0028342E, 0x0026302A, 0x001B221D, 0x000B110C, 0x00060A05, 0x00090D08, 0x000B0C08, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08,
        0x00090D08, 0x00090D08, 0x00090D08, 0x00060A05, 0x000A0E09, 0x000D110C, 0x00020601, 0x000B0F0A, 0x000C100B, 0x00000300, 0x00151914, 0x00717062, 0x00EAE9DB, 0x00F8F7E9, 0x007C7B6D, 0x00151406, 0x005B5A4C, 0x00E4E3D5, 0x00979688, 0x00C6C1B2,
        0x00FAF4E7, 0x003D392E, 0x00090800, 0x000B0D07, 0x000A0E09, 0x00080D0C, 0x00090F0E, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A,
        0x000C100B, 0x000C100B, 0x000C100B, 0x000C100A, 0x000C100A, 0x000C100A, 0x000D110B, 0x000D110B, 0x000D110B, 0x000D110B, 0x000D110B, 0x000C100B, 0x000C100B, 0x000C100B, 0x000D110C, 0x000D110C, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E140F,
        0x000E140F, 0x000E140F, 0x000E140F, 0x000E140F, 0x000E140F, 0x000E140F, 0x000E140F, 0x000F160F, 0x000F160F, 0x000F160F, 0x000F160F, 0x000E150E, 0x000E150E, 0x000E150E, 0x000E150E, 0x0010140E, 0x0010140E, 0x0010140E, 0x0010140E, 0x0010140E,
        0x0010140E, 0x0010140E, 0x0010140E, 0x000F130E, 0x000F130E, 0x000F130E, 0x0010140F, 0x0010140F, 0x0010140F, 0x0010140F, 0x0010140F, 0x000F130E, 0x000F130E, 0x0010140F, 0x0010140F, 0x0010140F, 0x0010140F, 0x0010140F, 0x000F130E, 0x0010140F,
        0x0010140F, 0x00111510, 0x0010140F, 0x000F130E, 0x000E120D, 0x000E120D, 0x000F130E, 0x000F130E, 0x0010140F, 0x00121611, 0x00121611, 0x00121611, 0x00121611, 0x00141813, 0x00161A15, 0x00131712, 0x001D211C, 0x00191D18, 0x001C201B, 0x001D211C,
        0x00484C47, 0x006D716C, 0x00333732, 0x00505651, 0x00565C57, 0x005A605B, 0x00595F5A, 0x005B615C, 0x005D635E, 0x00575D58, 0x004D534E, 0x004E544F, 0x00515752, 0x00484E49, 0x005E645F, 0x005B615C, 0x00666C67, 0x005E645F, 0x00606661, 0xB0C6C6C6,
        0xFFFEFEFE, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xE6E0E5E4,
        0xE2DDE3E2, 0xD1D5DDDC, 0xBCCCD7D5, 0x00C6D4D2, 0x00C4D7D4, 0x00C3D8D5, 0x00C1D6D3, 0x00CCF0F6, 0x00D7F9FF, 0x00D8F7FA, 0x00BED7D9, 0x00A7BCBA, 0x00AEBCBA, 0x00A6B1AE, 0x0087918B, 0x00727D7B, 0x005D6866, 0x0043504E, 0x0033403E, 0x00253331,
        0x0022302E, 0x0033413F, 0x00485956, 0x002B3E3B, 0x0021322F, 0x002D3B39, 0x00313E3C, 0x00434E4C, 0x00565E5D, 0x003F4544, 0x00292F2E, 0x00263234, 0x002D3836, 0x00212B25, 0x001C251B, 0x00070F05, 0x00070903, 0x000E0D09, 0x000B0908, 0x000C110F,
        0x00070C0A, 0x00010604, 0x00070A08, 0x000C0F0D, 0x000C0D0B, 0x00000100, 0x00090A08, 0x000A0B09, 0x000D0E0C, 0x00050806, 0x000C0F0D, 0x00050806, 0x000D1210, 0x00050A08, 0x00040907, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00060600, 0x00393933, 0x00C3C3BD, 0x00F6F4EC, 0x00B5B3AB, 0x0026241A, 0x00727066, 0x00FFFEF4, 0x00FDF2E4,
        0x00BEB7A8, 0x00201F11, 0x000E1207, 0x000D1209, 0x000B0F09, 0x000D0E0A, 0x00090A08, 0x000B0C08, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A,
        0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000B0F09, 0x000B0F09, 0x000C100A, 0x000C100A, 0x000C100A, 0x000C100A, 0x000D110B, 0x000D110B, 0x000C120D, 0x000C120D, 0x000C120D, 0x000C120D, 0x000C120D, 0x000D130E, 0x000D130E, 0x000D130E, 0x000E140F,
        0x000E140F, 0x000E140F, 0x000E140F, 0x000F1510, 0x000F1510, 0x000F1510, 0x000F1510, 0x000F160F, 0x000F160F, 0x000F160F, 0x000F160F, 0x000F160F, 0x000E150E, 0x000E150E, 0x000D140D, 0x000E150E, 0x000E150E, 0x000E150E, 0x000E150E, 0x000E150E,
        0x000E150E, 0x000E150E, 0x000E150E, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E,
        0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000E120C, 0x000E120C, 0x000E120C, 0x000E120C, 0x000E120C, 0x000F130D, 0x000F130D, 0x000F130D, 0x00161713, 0x00161713, 0x001D1E1A, 0x00111510, 0x00171B16,
        0x005D635E, 0x00676D68, 0x00383E39, 0x00585F58, 0x00555C55, 0x00545B54, 0x00565D56, 0x00596059, 0x00596059, 0x00565D56, 0x00525952, 0x00535A53, 0x004C534C, 0x004A514A, 0x005B625B, 0x00656C65, 0x005E655E, 0x005E655E, 0x00616861, 0xD1D5D5D5,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFAFCFC,
        0xFFF7FCFB, 0xFFEEF4F3, 0xE6DFE7E6, 0xC9D1DCDA, 0xB4C7D5D3, 0x00C0D1CE, 0x00BBCCC9, 0x00CDEAF1, 0x00D7F4F9, 0x00DAF3F7, 0x00C3D8DA, 0x00ABBDBC, 0x00AEBCBA, 0x00A9B7B3, 0x00929E98, 0x00859290, 0x00828F8D, 0x007F8D8B, 0x00778583, 0x0061726F,
        0x004D5E5B, 0x004E5F5C, 0x00586B68, 0x00435451, 0x00334441, 0x00354341, 0x00293634, 0x002A3533, 0x00384341, 0x00343C3B, 0x00313938, 0x00283436, 0x002B3836, 0x001F2B25, 0x0020291F, 0x000A1208, 0x00070903, 0x000B0A06, 0x000F0D0C, 0x00030904,
        0x00030904, 0x00080E09, 0x00050904, 0x00010500, 0x000B0C08, 0x0010110D, 0x000A0B07, 0x000B0C08, 0x000C0D09, 0x000A0B07, 0x000E120D, 0x00060A05, 0x00040A05, 0x00040A05, 0x000D130E, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x000C0B07, 0x00080802, 0x001B1B15, 0x0085827D, 0x00FAF8F0, 0x00DAD7CF, 0x0088867C, 0x00B4B2A8, 0x00F5EADC,
        0x00756E5F, 0x00050400, 0x00080C01, 0x000E130A, 0x00060A05, 0x00070806, 0x000C0D0B, 0x000B0C08, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A,
        0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000B0F09, 0x000B0F09, 0x000B0F09, 0x000C100A, 0x000C100A, 0x000C100A, 0x000C100A, 0x000D110B, 0x000B110C, 0x000C120D, 0x000C120D, 0x000C120D, 0x000C120D, 0x000C120D, 0x000D130E, 0x000D130E, 0x000E140F,
        0x000E140F, 0x000E140F, 0x000E140F, 0x000E140F, 0x000F1510, 0x000F1510, 0x000F1510, 0x000F160F, 0x000F160F, 0x000F160F, 0x000F160F, 0x000F160F, 0x000E150E, 0x000E150E, 0x000D140D, 0x000E150E, 0x000E150E, 0x000E150E, 0x000E150E, 0x000E150E,
        0x000E150E, 0x000E150E, 0x000E150E, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E,
        0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000E120C, 0x000E120C, 0x000E120C, 0x000E120C, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000E0F0B, 0x000E0F0B, 0x00131410, 0x0010140F, 0x000E120D,
        0x002E342F, 0x003D433E, 0x00414742, 0x00585F58, 0x005B625B, 0x005B625B, 0x00596059, 0x00596059, 0x00596059, 0x00555C55, 0x00505750, 0x00515851, 0x004A514A, 0x004C534C, 0x005C635C, 0x00646B64, 0x005E655E, 0x005F665F, 0x00656C65, 0xD5D8D8D8,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFE,
        0xFFFFFFFF, 0xFFFDFFFF, 0xFFFBFFFF, 0xFFF8FEFD, 0xF7E8F0EF, 0xC0CDD8D6, 0x92B7C2C0, 0x00C8DBE0, 0x00D3E6EB, 0x00DBECEF, 0x00CBDDDE, 0x00B5C5C4, 0x00AEBFBC, 0x00A7B8B4, 0x0095A4A0, 0x00818F8D, 0x007D8B89, 0x007F908D, 0x008A9B98, 0x008B9E9B,
        0x00839693, 0x007C8F8C, 0x00798E8B, 0x006F807D, 0x005E6F6C, 0x005A6866, 0x0042504E, 0x002D3A38, 0x00293432, 0x00202B29, 0x00212C2A, 0x00222E30, 0x0024312F, 0x001A2620, 0x001E291F, 0x000C140A, 0x00080C06, 0x00060703, 0x000C0A09, 0x000A100B,
        0x00000601, 0x00060C07, 0x000A0E09, 0x00090D08, 0x00030400, 0x000A0B07, 0x00070804, 0x000B0A06, 0x000A0B07, 0x00090A06, 0x00050602, 0x00060A05, 0x000A0E09, 0x00060C07, 0x00000601, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x000D0E0A, 0x00080703, 0x00070701, 0x000C0C06, 0x0046443C, 0x00DAD8D0, 0x00FCF9F1, 0x00B8B6AC, 0x00CCC1B3,
        0x00C4BCAF, 0x00464439, 0x00000400, 0x00171B15, 0x000E120D, 0x000E0F0D, 0x000C0D0B, 0x000B0C08, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A,
        0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000B0F09, 0x000B0F09, 0x000B0F09, 0x000C100A, 0x000C100A, 0x000C100A, 0x000C100A, 0x000C100A, 0x000B110C, 0x000B110C, 0x000B110C, 0x000C120D, 0x000C120D, 0x000C120D, 0x000C120D, 0x000D130E, 0x000D130E,
        0x000E140F, 0x000E140F, 0x000E140F, 0x000E140F, 0x000F1510, 0x000F1510, 0x000F1510, 0x000F160F, 0x000F160F, 0x000F160F, 0x000F160F, 0x000F160F, 0x000E150E, 0x000E150E, 0x000D140D, 0x000E150E, 0x000E150E, 0x000E150E, 0x000E150E, 0x000E150E,
        0x000E150E, 0x000E150E, 0x000E150E, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E,
        0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000E120C, 0x000E120C, 0x000E120C, 0x000E120C, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000D0E0A, 0x0012130F, 0x0011120E, 0x00111510, 0x000F130E,
        0x00101611, 0x00222823, 0x00515752, 0x005B625B, 0x005A615A, 0x00575E57, 0x00545B54, 0x00565D56, 0x00596059, 0x00585F58, 0x00535A53, 0x00565D56, 0x004C534C, 0x00525952, 0x005F665F, 0x00646B64, 0x005F665F, 0x005F665F, 0x00697069, 0xE2DDDDDD,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF,
        0xFFFFFEFE, 0xFFFFFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFFFF, 0xFFEDF2F1, 0xE2DDE2E1, 0xB4C7D0D4, 0xB8CAD3D7, 0xB8CAD3D6, 0x00BDC9C9, 0x00AEBCBB, 0x00ADBBB9, 0x00ABBCB9, 0x00A3B4B0, 0x0096A7A4, 0x0091A29F, 0x008E9F9C, 0x008D9E9B, 0x00869996,
        0x00809390, 0x007F9491, 0x00839895, 0x008D9B99, 0x00808E8C, 0x007E8C8A, 0x0072807E, 0x005D6B69, 0x004C5957, 0x00384543, 0x00313E3C, 0x002C393B, 0x002C3937, 0x001F2B25, 0x001C271D, 0x00040D03, 0x000A0E08, 0x00090A06, 0x000B0C0A, 0x00050B06,
        0x00090F0A, 0x00090F0A, 0x00000300, 0x00080C07, 0x000D0E0A, 0x0011120E, 0x00090A06, 0x000B0A06, 0x000D0C08, 0x0010110D, 0x000C0D09, 0x000F100C, 0x00090D08, 0x000A0E09, 0x00070B06, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00090A06, 0x000A0B07, 0x001C1D19, 0x000B0B05, 0x00020200, 0x00403E36, 0x00C7C5BD, 0x00F2F0E8, 0x00E8DCD0,
        0x00FFF7EA, 0x00B4B2A7, 0x001F2219, 0x00050903, 0x00010500, 0x00141513, 0x000A0A0A, 0x000B0C08, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A,
        0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000B0F09, 0x000B0F09, 0x000B0F09, 0x000B0F09, 0x000C100A, 0x000C100A, 0x000C100A, 0x000C100A, 0x000B110C, 0x000B110C, 0x000B110C, 0x000B110C, 0x000C120D, 0x000C120D, 0x000C120D, 0x000C120D, 0x000D130E,
        0x000D130E, 0x000D130E, 0x000E140F, 0x000E140F, 0x000E140F, 0x000E140F, 0x000F1510, 0x000F160F, 0x000F160F, 0x000F160F, 0x000F160F, 0x000F160F, 0x000E150E, 0x000E150E, 0x000D140D, 0x000E150E, 0x000E150E, 0x000E150E, 0x000E150E, 0x000E150E,
        0x000E150E, 0x000E150E, 0x000E150E, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E,
        0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000E120C, 0x000E120C, 0x000E120C, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x0010140E, 0x000E0F0B, 0x00181915, 0x00141511, 0x000E120D, 0x00131712,
        0x00121813, 0x00272D28, 0x00575D58, 0x005D645D, 0x00565D56, 0x00505750, 0x00525952, 0x00575E57, 0x00585F58, 0x00545B54, 0x00505750, 0x00525952, 0x00424942, 0x004D544D, 0x005A615A, 0x00606760, 0x00616861, 0x00606760, 0x006E756E, 0xEEE3E3E3,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFE,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFE, 0xFFFDFBFB, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBFDFE, 0xFFF2F6F7, 0xF7E7EBEC, 0xD5D7DCDD, 0xA7C2CCCC, 0x8EB6C3C1, 0x79ABB9B7, 0x00A1AFAD, 0x00A6B4B2, 0x009FADAB, 0x0096A4A2, 0x008D9E9B, 0x00899A97,
        0x00869794, 0x00849794, 0x00869996, 0x00919F9D, 0x00879593, 0x008B9997, 0x008C9A98, 0x0083918F, 0x00758683, 0x00677875, 0x0060716E, 0x006C797B, 0x006B7977, 0x0056645E, 0x003B463C, 0x00040D03, 0x00080F08, 0x000A0E09, 0x000E0F0D, 0x00060D06,
        0x00030A03, 0x000A110A, 0x00090D07, 0x00060A04, 0x00090B05, 0x00070903, 0x00090B05, 0x000B0B05, 0x00030300, 0x00020200, 0x00050701, 0x000C0E08, 0x00090B05, 0x00080C06, 0x00090D07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00111412, 0x000E0F0D, 0x00030400, 0x00080905, 0x001E1E18, 0x00020200, 0x003B3833, 0x00C9C7BF, 0x00ADA195,
        0x00B2A99F, 0x00C8C6BC, 0x002E3128, 0x00000300, 0x00171A18, 0x00060606, 0x00121212, 0x000B0C08, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A,
        0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000A0E08, 0x000B0F09, 0x000B0F09, 0x000B0F09, 0x000B0F09, 0x000C100A, 0x000C100A, 0x000C100A, 0x000B110C, 0x000B110C, 0x000B110C, 0x000B110C, 0x000C120D, 0x000C120D, 0x000C120D, 0x000C120D, 0x000D130E,
        0x000D130E, 0x000D130E, 0x000D130E, 0x000E140F, 0x000E140F, 0x000E140F, 0x000E140F, 0x000F160F, 0x000F160F, 0x000F160F, 0x000F160F, 0x000F160F, 0x000E150E, 0x000E150E, 0x000D140D, 0x000E150E, 0x000E150E, 0x000E150E, 0x000E150E, 0x000E150E,
        0x000E150E, 0x000E150E, 0x000E150E, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E,
        0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000E120C, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x0010140E, 0x0010140E, 0x0010140E, 0x000D0E0A, 0x00151612, 0x0011120E, 0x000A0E09, 0x0010140F,
        0x00161C17, 0x00323833, 0x00535954, 0x005C635C, 0x00555C55, 0x00535A53, 0x005A615A, 0x005B625B, 0x00545B54, 0x004E554E, 0x004F564F, 0x005D645D, 0x00464D46, 0x00545B54, 0x005F665F, 0x00626962, 0x00656C65, 0x00606760, 0x00707770, 0xFBEAEAEA,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFAFAFA,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFE, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFBFDFD, 0xFFFCFEFE, 0xFFF8FDFC, 0xFFF2F8F7, 0xF7E8EDEE, 0xDEDCE4E4, 0xABC4CCCB, 0x8EB5C0BE, 0x6DA6B1AF, 0x5C9EA9A7, 0x009CA9A7,
        0x0099A6A4, 0x008F9D9B, 0x00859391, 0x008B9896, 0x00859290, 0x00889694, 0x008B9997, 0x00849592, 0x007D908D, 0x007A8D8A, 0x00778A87, 0x008E9D9F, 0x0097A5A3, 0x008C9A94, 0x00627065, 0x000A1309, 0x00060D06, 0x00080C07, 0x00080B09, 0x000C130C,
        0x00000600, 0x00060D06, 0x00060A04, 0x00000200, 0x00191B15, 0x000A0C06, 0x00050701, 0x00302D28, 0x00413E39, 0x0033332D, 0x00181812, 0x00000200, 0x000D0F09, 0x000C0E08, 0x00010500, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00040705, 0x000B0E0C, 0x00121311, 0x000B0C08, 0x00090A06, 0x0011110B, 0x00060600, 0x00494943, 0x00D8CCC2,
        0x003C3329, 0x00222016, 0x000C0F06, 0x00000200, 0x00111412, 0x000D0D0D, 0x00070608, 0x000B0C08, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A,
        0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000B0F09, 0x000B0F09, 0x000B0F09, 0x000B0F09, 0x000C100A, 0x000A100B, 0x000B110C, 0x000B110C, 0x000B110C, 0x000B110C, 0x000C120D, 0x000C120D, 0x000C120D, 0x000D130E,
        0x000D130E, 0x000D130E, 0x000D130E, 0x000D130E, 0x000E140F, 0x000E140F, 0x000E140F, 0x000F160F, 0x000F160F, 0x000F160F, 0x000F160F, 0x000F160F, 0x000E150E, 0x000E150E, 0x000D140D, 0x000E150E, 0x000E150E, 0x000E150E, 0x000E150E, 0x000E150E,
        0x000E150E, 0x000E150E, 0x000E150E, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E,
        0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x0010140E, 0x0010140E, 0x0010140E, 0x0010140E, 0x0010110D, 0x0010110D, 0x0010110D, 0x000E120D, 0x000F130E,
        0x00121813, 0x00383E39, 0x00555B56, 0x005B625B, 0x00555C55, 0x00555C55, 0x00596059, 0x00565D56, 0x00515851, 0x00525952, 0x005A615A, 0x00596059, 0x00414841, 0x00585F58, 0x00636A63, 0x00636A63, 0x00676E67, 0x00606760, 0x04747B74, 0xFFF0F0F0,
        0xFFFEFEFE, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBFFFF,
        0xFFFBFFFF, 0xFFFCFEFE, 0xFFFBFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFCFC, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFBFFFF, 0xFFFAFFFE, 0xFFEDF3F2, 0xD5D8DEDD, 0xABC4CAC9,
        0x86B1B9B8, 0x69A4ACAB, 0x549AA2A1, 0x0097A4A2, 0x00909D9B, 0x008C9A98, 0x00889996, 0x00809390, 0x007E918E, 0x007E9390, 0x007B908D, 0x007D8C8E, 0x00859693, 0x008C9C95, 0x006C7A6F, 0x00061107, 0x00040B04, 0x00080C07, 0x00070A08, 0x00010900,
        0x000D150B, 0x00050D03, 0x00020700, 0x00262B22, 0x007A7D74, 0x0035382F, 0x00030600, 0x00605E56, 0x009A9890, 0x00908E86, 0x004E4F46, 0x00020300, 0x000D1007, 0x000F1209, 0x000C0F06, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x000D0F0F, 0x00050806, 0x000C0F0D, 0x00141513, 0x00070804, 0x000E0D09, 0x00393933, 0x00767670, 0x00F4E8DE,
        0x009B9288, 0x00110F05, 0x00070903, 0x00131712, 0x00060808, 0x000C0B0D, 0x000E0D0F, 0x000B0C08, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A,
        0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000B0F09, 0x000B0F09, 0x000B0F09, 0x000B0F09, 0x000B0F09, 0x000A100B, 0x000A100B, 0x000B110C, 0x000B110C, 0x000B110C, 0x000B110C, 0x000B110C, 0x000C120D, 0x000C120D,
        0x000D130E, 0x000D130E, 0x000D130E, 0x000D130E, 0x000E140F, 0x000E140F, 0x000E140F, 0x000F160F, 0x000F160F, 0x000F160F, 0x000F160F, 0x000F160F, 0x000E150E, 0x000E150E, 0x000D140D, 0x000E150E, 0x000E150E, 0x000E150E, 0x000E150E, 0x000E150E,
        0x000E150E, 0x000E150E, 0x000E150E, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E,
        0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x0010140E, 0x0010140E, 0x0010140E, 0x0010140E, 0x00151612, 0x0011120E, 0x0012130F, 0x0010140F, 0x00121611,
        0x00131914, 0x003E443F, 0x005C625D, 0x005A615A, 0x00575E57, 0x00535A53, 0x00515851, 0x00525952, 0x00535A53, 0x00525952, 0x004F564F, 0x00373E37, 0x00262D26, 0x004F564F, 0x005F665F, 0x005E655E, 0x00656C65, 0x00626962, 0x197E857E, 0xFFF4F4F4,
        0xFFFDFDFD, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF9FFFE,
        0xFFFAFFFF, 0xFFFBFFFF, 0xFFFBFFFF, 0xFFFAFCFC, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFBFFFA, 0xFFFDFFFC, 0xFFFFFFFE, 0xFFFEFFFD, 0xFFFFFEFE, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFDFF, 0xFFFDFDFD, 0xFFFDFFFF, 0xFFFDFFFF, 0xFFFDFFFF, 0xFFFBFFFF,
        0xFFFBFFFF, 0xFFECF2F1, 0xE2DEE4E3, 0xABC3CECC, 0x86B2BFBD, 0x609FADAB, 0x4391A29F, 0x00869996, 0x00829794, 0x00819894, 0x007A918D, 0x008B9A9C, 0x00869794, 0x008B9B94, 0x006D7B70, 0x00040F05, 0x00030B04, 0x00090F0A, 0x00090C0A, 0x00040C02,
        0x00070F05, 0x00010900, 0x00444940, 0x00A6ABA2, 0x00D5D8CF, 0x0034372E, 0x00000300, 0x00626058, 0x008D8B83, 0x007E7C74, 0x0054554C, 0x000A0B02, 0x0012130A, 0x00070A01, 0x000B0E05, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00070C0B, 0x00080A0A, 0x000C0F0D, 0x00070A08, 0x000D0E0A, 0x0012130F, 0x00BAB9B5, 0x00F9F9F3, 0x00FFF3E9,
        0x00B0A79E, 0x000C0901, 0x00090B05, 0x000E120D, 0x00080A0A, 0x000E0D0F, 0x000B0A0C, 0x000B0C08, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A,
        0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000B0F09, 0x000B0F09, 0x000B0F09, 0x000B0F09, 0x000A100B, 0x000A100B, 0x000A100B, 0x000B110C, 0x000B110C, 0x000B110C, 0x000B110C, 0x000B110C, 0x000C120D,
        0x000C120D, 0x000D130E, 0x000D130E, 0x000D130E, 0x000D130E, 0x000E140F, 0x000E140F, 0x000F160F, 0x000F160F, 0x000F160F, 0x000F160F, 0x000F160F, 0x000E150E, 0x000E150E, 0x000D140D, 0x000E150E, 0x000E150E, 0x000E150E, 0x000E150E, 0x000E150E,
        0x000E150E, 0x000E150E, 0x000E150E, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E,
        0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130D, 0x000F130D, 0x000F130D, 0x0010140E, 0x0010140E, 0x0010140E, 0x0010140E, 0x0010140E, 0x00141511, 0x00141511, 0x0011120E, 0x000C100B, 0x00131712,
        0x00171D18, 0x00444A45, 0x005E645F, 0x005B625B, 0x00596059, 0x00545B54, 0x00525952, 0x00575E57, 0x00585F58, 0x00444B44, 0x002B322B, 0x00343B34, 0x00282F28, 0x005A615A, 0x006A716A, 0x00626962, 0x00656C65, 0x00626962, 0x21828982, 0xFFF7F7F7,
        0xFFFDFDFD, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF7F7F7, 0xF7E7E7E7, 0xD1D6D6D6, 0xABC3C3C3, 0x82AFAFAF, 0x609F9F9F, 0x3B8D9C98, 0x00869591, 0x008B9995, 0x00717D77, 0x000A140E, 0x00060D08, 0x00091009, 0x00080C06, 0x00020706,
        0x00040606, 0x003D3E3A, 0x00D7D7D1, 0x00D2D0C6, 0x00CECABF, 0x00463E31, 0x000B0400, 0x00343630, 0x00707168, 0x008F8E84, 0x0059574C, 0x000A0800, 0x00040500, 0x00060A05, 0x00070C0B, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00090D07, 0x00090D07, 0x00090D07, 0x0009100D, 0x00030A05, 0x000B0F0A, 0x000C0D09, 0x000D0A05, 0x00060200, 0x0098928B, 0x00F9F1EA, 0x00797465,
        0x00211D12, 0x000F1007, 0x000A0E09, 0x00070C0A, 0x00070A08, 0x00090A06, 0x000E0E08, 0x000B0D07, 0x000B0D07, 0x000B0C08, 0x000C0D09, 0x000C0D09, 0x000C0D0B, 0x000C0D0B, 0x000C0D0B, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A,
        0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000D110C, 0x000D110C, 0x000B110C, 0x000B110C, 0x000B110C, 0x000B110C,
        0x000B110C, 0x000B110C, 0x000C120D, 0x000C120D, 0x000D130E, 0x000D130E, 0x000E140F, 0x000D150E, 0x00111912, 0x000E160F, 0x00141B14, 0x000D140D, 0x00080C06, 0x00131711, 0x000D110B, 0x000F160F, 0x000F160F, 0x000F160F, 0x000F160F, 0x000F160F,
        0x000F160F, 0x000F160F, 0x000F160F, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E,
        0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x0010140F, 0x0010140F, 0x0010140F, 0x0010140F, 0x0010140F, 0x000E120D, 0x0010140F, 0x000C100B, 0x00121813, 0x000D130E,
        0x001A201B, 0x004B524D, 0x005E6560, 0x005B6359, 0x0050584E, 0x00586056, 0x005B625B, 0x00575E57, 0x00343A35, 0x002F3530, 0x00424843, 0x00585E59, 0x00343A35, 0x00555B56, 0x00636964, 0x005D635E, 0x00656B66, 0x00646A65, 0x1D7F8580, 0xFFF8FEF9,
        0xFFFAFFFB, 0xFFFBFFFC, 0xFFFDFFFC, 0xFFFDFFFC, 0xFFFDFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFBFBFB, 0xFFF8F8F8, 0xFFF2F2F2, 0xFFEBEBEB, 0xB4C7D2CF, 0x8EB6C1BE, 0x7DAEB7B4, 0x2A858C87, 0x000B110C, 0x00060A05, 0x000D110B, 0x000B0D07, 0x00090E0D,
        0x00050806, 0x0020211D, 0x00898983, 0x00DCD9D1, 0x00D2CEC3, 0x00352F22, 0x00130B00, 0x0061635D, 0x005A5B52, 0x0095948A, 0x00454338, 0x00060400, 0x00111209, 0x000B0F0A, 0x00080D0C, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00090D07, 0x00090D07, 0x00090D07, 0x00010805, 0x00080E09, 0x00050904, 0x000E0F0B, 0x002B2823, 0x00534F4A, 0x004F4942, 0x00D7CFC8, 0x00736E5F,
        0x00070300, 0x00101108, 0x000B0F0A, 0x00060B09, 0x00060907, 0x00090A06, 0x000E0E08, 0x000B0D07, 0x000B0D07, 0x000B0C08, 0x000C0D09, 0x000C0D09, 0x000C0D0B, 0x000C0D0B, 0x000C0D0B, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A,
        0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000C100B, 0x000D110C, 0x000B110C, 0x000B110C, 0x000B110C, 0x000B110C,
        0x000B110C, 0x000B110C, 0x000C120D, 0x000C120D, 0x000D130E, 0x000D130E, 0x000D130E, 0x000B130C, 0x000D150E, 0x00050D06, 0x00141B14, 0x00131A13, 0x00151913, 0x000E120C, 0x000F130D, 0x000C130C, 0x000C130C, 0x000C130C, 0x000E150E, 0x00101710,
        0x00101710, 0x000E150E, 0x000C130C, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E,
        0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x0010140F, 0x0010140F, 0x0010140F, 0x0010140F, 0x000E120D, 0x000F130E, 0x000D110C, 0x00121813, 0x000E140F,
        0x001C221D, 0x004C534E, 0x005F6661, 0x00596157, 0x005D655B, 0x005F665F, 0x00464D46, 0x00343B34, 0x00373D38, 0x00474D48, 0x003C423D, 0x002E342F, 0x002E342F, 0x00585E59, 0x00626863, 0x00626863, 0x00656B66, 0x00616762, 0x2E878D88, 0xFFF9FEFC,
        0xFFFAFFFD, 0xFFFAFFFD, 0xFFFCFFFD, 0xFFFAFDFB, 0xFFFCFDFB, 0xFFFDFEFC, 0xFFFFFFFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBFFFE, 0xFFF7FCFA, 0xFFF1F6F4, 0x92B9BDB8, 0x001C1D19, 0x00040501, 0x0010100A, 0x000E0E08, 0x00080A0A,
        0x000B0E0C, 0x00020300, 0x00292923, 0x00A4A29A, 0x00DEDCD2, 0x00444035, 0x000A0600, 0x004B4D47, 0x00909188, 0x005F5E54, 0x0068665B, 0x000D0B01, 0x00101108, 0x00070B06, 0x00030807, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00090D07, 0x00090D07, 0x00090D07, 0x000C110F, 0x00090F0A, 0x00070B06, 0x00030400, 0x00484842, 0x00FFFCF7, 0x00BEBBB3, 0x00B0AAA3, 0x00CCC7B8,
        0x00403C31, 0x00050600, 0x000B0F0A, 0x00070C0A, 0x000B0E0C, 0x00161713, 0x000B0B05, 0x000B0D07, 0x000B0D07, 0x000B0C08, 0x000C0D09, 0x000C0D09, 0x000C0D0B, 0x000C0D0B, 0x000C0D0B, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A,
        0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000C100B, 0x000D110C, 0x000B110C, 0x000B110C, 0x000B110C, 0x000B110C,
        0x000B110C, 0x000B110C, 0x000C120D, 0x000C120D, 0x000D130E, 0x000D130E, 0x000D130E, 0x000F1710, 0x000B130C, 0x00070E07, 0x00101710, 0x000D110B, 0x00151913, 0x000D110B, 0x00151711, 0x00111811, 0x00101710, 0x000F160F, 0x000E150E, 0x000D140D,
        0x000E150E, 0x00101710, 0x00111811, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E,
        0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x0010140F, 0x0010140F, 0x0010140F, 0x0010140F, 0x000D110C, 0x000F130E, 0x000E120D, 0x00111712, 0x000E140F,
        0x00212722, 0x00505752, 0x00636A65, 0x00596157, 0x005D645D, 0x00444B44, 0x002B322B, 0x003E443F, 0x00838984, 0x00BFC4C2, 0x00B8BDBB, 0x00A3A9A4, 0x00848A85, 0x00747A75, 0x00646A65, 0x00646A65, 0x005B615C, 0x005A605B, 0x549AA09B, 0xFFFBFFFF,
        0xFFFBFFFF, 0xFFFDFFFF, 0xFFFDFFFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFCFCFC, 0xFFFCFCFC, 0xFFFCFCFC, 0xFFFCFCFC, 0xFFFCFCFC, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xC4D4D3CF, 0x0023221E, 0x00040300, 0x0010100A, 0x000F0C07, 0x00040705,
        0x0010110F, 0x00000100, 0x00050500, 0x00363630, 0x00B4B2AA, 0x00524F47, 0x00050300, 0x0040423C, 0x0065665D, 0x0058574D, 0x00514F44, 0x000B0900, 0x00050600, 0x00131712, 0x00000504, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00090D07, 0x00090D07, 0x00090D07, 0x00060907, 0x00070B06, 0x00171814, 0x000A0905, 0x00161610, 0x00A9A6A1, 0x00FAF8F0, 0x00E0DDD5, 0x00F2EDDE,
        0x00B6B2A7, 0x00191A11, 0x00070B06, 0x000E1311, 0x00050806, 0x000B0C08, 0x000F0F09, 0x000B0D07, 0x000B0D07, 0x000B0C08, 0x000C0D09, 0x000C0D09, 0x000C0D0B, 0x000C0D0B, 0x000C0D0B, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A,
        0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000D0E0A, 0x000E0F0B, 0x000E0F0B, 0x000C100B, 0x000C100B, 0x000B110C, 0x000B110C, 0x000B110C, 0x000A100B,
        0x000A100B, 0x000B110C, 0x000B110C, 0x000C120D, 0x000C120D, 0x000D130E, 0x000D130E, 0x000D140D, 0x000A110A, 0x00131A13, 0x0010140E, 0x000E120C, 0x00141610, 0x00141610, 0x0010120C, 0x0010140E, 0x00141812, 0x00171B15, 0x00161A14, 0x0011150F,
        0x000D110B, 0x000E120C, 0x0010140E, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E,
        0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000E120D, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x0010140F, 0x0010140F, 0x0010140F, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F1510, 0x000F1510,
        0x00282E29, 0x00555C57, 0x00676E69, 0x00616861, 0x005D645D, 0x00404740, 0x00585E59, 0x00939994, 0x00D2D7D5, 0x00FBFFFE, 0x00FBFFFE, 0x00F8FDFB, 0x00F1F6F4, 0x00EDF2F0, 0x00D9DEDC, 0x00BDC2C0, 0x00848987, 0x00626765, 0x00909593, 0xFFFDFFFF,
        0xFFFDFFFF, 0xFFFDFFFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFC, 0xFFFDF9F8, 0xFFFEFAF9, 0xBCD3D0CC, 0x0024231F, 0x00060501, 0x0010100A, 0x00070701, 0x000C0D09,
        0x000A0B07, 0x000B0C08, 0x0013120E, 0x00020200, 0x004A4A44, 0x0032322C, 0x00040400, 0x0031332D, 0x006A6B62, 0x00908F85, 0x003D3B30, 0x00110F05, 0x000B0C03, 0x00050904, 0x00090E0D, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00090D07, 0x00090D07, 0x00090D07, 0x0010110F, 0x00050602, 0x00070602, 0x0012110D, 0x00020200, 0x002D2D27, 0x00CACBC2, 0x00C7C5BD, 0x00C3BEAF,
        0x00FFFFF7, 0x0095968D, 0x00070B06, 0x00040907, 0x000F1210, 0x00080905, 0x00060600, 0x000B0D07, 0x000B0D07, 0x000B0C08, 0x000C0D09, 0x000C0D09, 0x000C0D0B, 0x000C0D0B, 0x000C0D0B, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A,
        0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000C100B, 0x000C100B, 0x000A100B, 0x000A100B, 0x000B110C, 0x000A100B,
        0x000A100B, 0x000B110C, 0x000B110C, 0x000C120D, 0x000C120D, 0x000D130E, 0x000D130E, 0x000A110A, 0x00131A13, 0x00151913, 0x00010500, 0x000C0E08, 0x00010300, 0x000E100A, 0x0015150F, 0x000D110B, 0x000E120C, 0x0010140E, 0x0011150F, 0x0010140E,
        0x0010140E, 0x0010140E, 0x0010140E, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E,
        0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000E120D, 0x000E120D, 0x000E120D, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x0010140F, 0x0010140F, 0x000F130E, 0x000F130E, 0x000D130E, 0x00101611,
        0x002E342F, 0x00585F5A, 0x00686F6A, 0x00656C65, 0x005C635C, 0x00646A65, 0x00B9BFBA, 0x00E9EEEC, 0x00F4F9F7, 0x00F1F6F5, 0x00EBF0EF, 0x00F1F6F4, 0x00F3F8F6, 0x00F4F9F7, 0x00F4F9F7, 0x00FBFFFE, 0x00F0F5F3, 0x00CED3D1, 0x00CACFCD, 0xFFF8FAFB,
        0xFFF9FBFC, 0xFFFCFBFD, 0xFFFEFDFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xDEE0DFDB, 0x002C2D29, 0x00060703, 0x000B0F09, 0x00050903, 0x000F0E0A,
        0x00080703, 0x000F100C, 0x000F100C, 0x00070804, 0x00040501, 0x000E0F0B, 0x00090A06, 0x00141610, 0x001E1F16, 0x00313026, 0x006D6B60, 0x000B0900, 0x000D0E05, 0x00090D08, 0x00050A09, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090706, 0x0023221E, 0x00070602, 0x0012110D, 0x001C1C16, 0x00020200, 0x0054554C, 0x00BFC2B9, 0x005A5546,
        0x00736F64, 0x00A4A59C, 0x001A1E19, 0x00000100, 0x00050806, 0x000E0F0B, 0x00161610, 0x000B0D07, 0x000B0D07, 0x000B0C08, 0x000C0D09, 0x000C0D09, 0x000C0D0B, 0x000C0D0B, 0x000C0D0B, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A,
        0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000B0F0A, 0x000C100B, 0x000A100B, 0x000A100B, 0x000A100B, 0x000A100B,
        0x000A100B, 0x000A100B, 0x000B110C, 0x000B110C, 0x000C120D, 0x000C120D, 0x000C120D, 0x000E120C, 0x000E120C, 0x00040802, 0x004D4F49, 0x008D8F89, 0x00777973, 0x004B4B45, 0x001B1B15, 0x000C0E08, 0x000A0C06, 0x000A0C06, 0x000F110B, 0x00171913,
        0x001A1C16, 0x00151711, 0x0010120C, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E,
        0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x00111510, 0x000F130E, 0x0010140F, 0x000D130E, 0x00121813,
        0x00323833, 0x00555C57, 0x005F6661, 0x006D736E, 0x005B615C, 0x00878D88, 0x00F4F9F7, 0x00FBFFFE, 0x00F5FAF9, 0x00F4F9F8, 0x00F0F5F4, 0x00EEF3F2, 0x00F1F6F5, 0x00F4F9F8, 0x00F0F5F4, 0x00F0F5F4, 0x00F2F7F6, 0x00F3F8F7, 0x00F7FCFB, 0x00F9FCFA,
        0xFFFBFCFA, 0xFFFCFDFB, 0xFFFEFFFD, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBF7F6, 0xFFFFFFFE, 0xFFFFFFFE, 0xF2E8E9E5, 0x00313530, 0x00000601, 0x00050D06, 0x0009110A, 0x000B0B05,
        0x000A0A04, 0x000B0A06, 0x00040501, 0x00121311, 0x00020503, 0x00050806, 0x00060808, 0x003B3D37, 0x0072736A, 0x00838278, 0x006B695E, 0x00121006, 0x000D0E05, 0x00070B06, 0x00040908, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00090D07, 0x00090D07, 0x00090D07, 0x00171312, 0x009D9A96, 0x008C8985, 0x00090804, 0x000F0F09, 0x00090B05, 0x00050800, 0x0074776E, 0x00B7B2A3,
        0x001F1B10, 0x000E0F06, 0x00040803, 0x00101513, 0x000B0E0C, 0x000C0D09, 0x000B0B05, 0x000B0D07, 0x000B0D07, 0x000B0C08, 0x000C0D09, 0x000C0D09, 0x000C0D0B, 0x000C0D0B, 0x000C0D0B, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A,
        0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000B0F0A, 0x000C100B, 0x000A100B, 0x000A100B, 0x000A100B, 0x000A100B,
        0x000A100B, 0x000A100B, 0x000B110C, 0x000B110C, 0x000C120D, 0x000C120D, 0x000C120D, 0x000D110B, 0x00131711, 0x00141812, 0x00A7A9A3, 0x00A7A9A3, 0x00A8A8A2, 0x00BFBFB9, 0x00C9C9C3, 0x00A5A7A1, 0x00848680, 0x0051534D, 0x00242620, 0x000B0D07,
        0x00070903, 0x000D0F09, 0x00141610, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E,
        0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x00111510, 0x000E120D, 0x00111510, 0x000D130E, 0x00131914,
        0x00323833, 0x004C534E, 0x004F5651, 0x00545A55, 0x00434944, 0x00838886, 0x00F6FBF9, 0x00F0F5F4, 0x00F3F8F7, 0x00F8FDFC, 0x00FBFFFF, 0x00FBFFFF, 0x00F3F8F7, 0x00EDF2F1, 0x00F3F8F7, 0x00F5FAF9, 0x00F3F8F7, 0x00F3F8F7, 0x00F1F6F5, 0x00FBFDF7,
        0x00F9FBF5, 0x00F8FAF4, 0x00FAFAF4, 0xFFFDFDF7, 0xFFFFFEF9, 0xFFFFFFFA, 0xFFFFFFFB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBFBFB, 0xFFFBFBFB, 0xFFFFFFFF, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFEFFFD, 0xEAE3E7E2, 0x00323833, 0x00000702, 0x00000A02, 0x00000D05, 0x000D0D07,
        0x000E0E08, 0x00090804, 0x000E0F0B, 0x00020503, 0x000E1010, 0x00030807, 0x00060B0A, 0x00060802, 0x0015160D, 0x00222117, 0x00131106, 0x00121006, 0x00020300, 0x000E120D, 0x00050A09, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090403, 0x0073706C, 0x00F7F4F0, 0x00666561, 0x00030300, 0x000D0F09, 0x000A0F06, 0x00010600, 0x00B5B0A1,
        0x00878378, 0x00070800, 0x000B0F0A, 0x00080D0B, 0x00040705, 0x000C0D09, 0x000A0A04, 0x000B0D07, 0x000B0D07, 0x000B0C08, 0x000C0D09, 0x000C0D09, 0x000C0D0B, 0x000C0D0B, 0x000C0D0B, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A,
        0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000B0F0A, 0x000B0F0A, 0x000A100B, 0x000A100B, 0x000A100B, 0x00090F0A,
        0x000A100B, 0x000A100B, 0x000B110C, 0x000B110C, 0x000C120D, 0x000C120D, 0x000C120D, 0x0010140E, 0x000C100A, 0x0012140E, 0x00B8BAB4, 0x0031312B, 0x00040400, 0x0025251F, 0x003B3833, 0x00696B65, 0x008A8C86, 0x00B4B6B0, 0x00C9CBC5, 0x00B9BBB5,
        0x008B8D87, 0x00585A54, 0x00373933, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000D140D, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E,
        0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000F130E, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000F130E, 0x000F130E, 0x000F130E, 0x0010140F, 0x000D110C, 0x00111510, 0x000D130E, 0x00141A15,
        0x00313732, 0x00454C47, 0x00434A45, 0x00474D48, 0x003B413C, 0x00797E7C, 0x00ECF1EF, 0x00EDF2F1, 0x00EFF4F3, 0x00E4E9E8, 0x00EDF1F2, 0x00EDF2F1, 0x00F9FEFD, 0x00F9FEFD, 0x00F1F6F5, 0x00E8EDEC, 0x00E9EEED, 0x00F1F6F5, 0x00EFF4F3, 0x00F5F8EF,
        0x00F1F4EB, 0x00EFF0E7, 0x00EEEFE6, 0x00F0F1E8, 0x00F6F4EC, 0x00F8F6EE, 0xFFF9F7EF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFFFE, 0xFFFDFFFE, 0xFFFAFDFB, 0xF7E9EDE8, 0x00353934, 0x00020601, 0x000B0F09, 0x00050903, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00060A04, 0x00080C06, 0x00060A04, 0x000A0E08, 0x00030701, 0x000C100A, 0x00070B05, 0x00000200, 0x00090900, 0x00131307, 0x00919086, 0x006A6860, 0x00060200, 0x00221D1A, 0x009B9695, 0x00A09B9A, 0x00CDCBC0,
        0x00DBD9CE, 0x00212016, 0x00070800, 0x0011130D, 0x00090A06, 0x000B0E0C, 0x00060B09, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07,
        0x00090D07, 0x00090D07, 0x00090D07, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000C100B, 0x000C100B, 0x000C100B, 0x000A100B,
        0x000A100B, 0x000A100B, 0x000B110C, 0x000B110C, 0x000B110C, 0x000B110C, 0x000B110C, 0x000D140D, 0x000D110B, 0x001C1C16, 0x00B9B6B1, 0x002F2C27, 0x0013130D, 0x00121610, 0x000B120B, 0x00000F06, 0x002B2F29, 0x00362D29, 0x003E3531, 0x00595651,
        0x00898B85, 0x00AAA7A2, 0x00D7CECA, 0x00BFC3B8, 0x0067685E, 0x00100E06, 0x00343129, 0x00292520, 0x00161610, 0x00131712, 0x000B110C, 0x0010140E, 0x0010140E, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000E120C, 0x000E120C, 0x000F130E,
        0x000F130E, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000F130E, 0x000F130E, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000D1211, 0x000B100E, 0x00111412, 0x000D110C, 0x001B1E15,
        0x003B3C32, 0x0044453B, 0x003C3E32, 0x004D392E, 0x002F2D22, 0x006D7C74, 0x00E8F7F3, 0x00C8CDCC, 0x00ECE6E7, 0x00F7EFEF, 0x00F3F0EC, 0x00DDE4F5, 0x00F0F7FF, 0x00DBE3E2, 0x00C8D1C7, 0x00DBE3D8, 0x00E3EBE4, 0x00E0E5E6, 0x00E5EBF2, 0x00DAE6E6,
        0x00E1EBE5, 0x00FFFFF7, 0x00FEFBE6, 0x00E5D8B8, 0x00F2D9AF, 0x00ECC898, 0x00F4C996, 0x00FAD494, 0x82FFE3B0, 0xD5FFF5D8, 0xFFFFFEF5, 0xFFFFFEFF, 0xFFFFFDFF, 0xFFFFFDFE, 0xFFFFFEF9, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFDFF, 0xFFFFFBFF,
        0xFFFFF9FF, 0xFFFFF9FF, 0xFFFFF9FF, 0xFFFDFFFF, 0xFFFDFFFF, 0xFFFDFEFF, 0xFFFDFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFE, 0xFFFFFFFE, 0xFFFFFEFD, 0xFFFFFEFD, 0xFFFFFFFE, 0xFFFEFFFD, 0xFFFCFFFD, 0xFFFBFEFC, 0xFFFFFEF7,
        0xFFFFFFFA, 0xFFFFFFFB, 0xFFFFFFFC, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFDFDFD, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFFFE, 0xFFFDFFFE, 0xFFF9FCFA, 0xF7E8ECE7, 0x00363A35, 0x00000300, 0x000A0E08, 0x00070B05, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x000E120C, 0x000E120C, 0x00070B05, 0x00070B05, 0x000D110B, 0x00050903, 0x00070B05, 0x005C605A, 0x00474338, 0x00131106, 0x00050300, 0x0018150D, 0x00060300, 0x00201D19, 0x00D3CFCE, 0x00EDE9E8, 0x00E6E4D9,
        0x00AFADA2, 0x00242319, 0x000B0C03, 0x000A0C06, 0x00010200, 0x000E110F, 0x00070C0A, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07,
        0x00090D07, 0x00090D07, 0x00090D07, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000C100B, 0x000C100B, 0x000C100B, 0x000A100B,
        0x000A100B, 0x000A100B, 0x000B110C, 0x000B110C, 0x000B110C, 0x000B110C, 0x000B110C, 0x000D140D, 0x000D110B, 0x001E1E18, 0x00B8B5B0, 0x002C2924, 0x0013130D, 0x00121610, 0x000B120B, 0x00000B03, 0x009CA09A, 0x00E1D8D4, 0x00998E8A, 0x00090601,
        0x0010120C, 0x000B0803, 0x001D1410, 0x0054584D, 0x00C8C9BF, 0x00312F27, 0x00A9A69E, 0x00B4B0AB, 0x0023231D, 0x000E120D, 0x000D130E, 0x0010140E, 0x0010140E, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000E120C, 0x000E120C, 0x000F130E,
        0x000F130E, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000F130E, 0x000F130E, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000D1211, 0x000B100E, 0x00111412, 0x000E120D, 0x001C1E18,
        0x00393C33, 0x0041453A, 0x003D3E34, 0x00433529, 0x0029281A, 0x00788271, 0x00DFE4CF, 0x00A3977F, 0x00C6AA8B, 0x00DEBF98, 0x00E9CC9F, 0x00E0DECC, 0x00FFFFF0, 0x00F0DFC5, 0x00937E63, 0x0088775D, 0x00D4CDBC, 0x00F4FCF2, 0x00DAE8E6, 0x00F9FFFE,
        0x00FAFFFE, 0x00F4FBF4, 0x00FFFFF8, 0x00E8E8D8, 0x00D4CFBA, 0x00D3C8B2, 0x00CFC3AB, 0x00D1B597, 0x00D7B38D, 0x26E3B984, 0x36F3C78C, 0x71FFDDA8, 0xC4FFF0CF, 0xFFF9FDF2, 0xFFEDFFFF, 0xFFFFFDFF, 0xFFFFFDFF, 0xFFFCFDFF, 0xFFFCFEFE, 0xFFFBFFFE,
        0xFFFAFFFB, 0xFFFAFFF9, 0xFFFAFFF8, 0xFFFFFFFA, 0xFFFFFFFA, 0xFFFFFFFC, 0xFFFFFFFE, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFEFCFF, 0xFFFAFBFF, 0xFFFBFDFF, 0xFFFDFDFF, 0xFFFFFEFF, 0xFFFFFFFE, 0xFFFFFFFC, 0xFFFFFFF9, 0xFFFFFFF9, 0xFFFFFFFE,
        0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFEFD, 0xFFFFFDFC, 0xFFFDFDFD, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFFFE, 0xFFFDFFFE, 0xFFFCFFFD, 0xFFEDF1EC, 0x00414540, 0x00000400, 0x000A0E08, 0x00080C06, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00000300, 0x00030701, 0x000A0E08, 0x00050903, 0x000C100A, 0x00050903, 0x001F231D, 0x00BCC0BA, 0x00E9E0D7, 0x00A59E95, 0x0049433C, 0x00050200, 0x00110E09, 0x0013130D, 0x004F504C, 0x00E4E8E3, 0x00838176,
        0x00110F04, 0x000C0B01, 0x000D0E05, 0x00060802, 0x00141511, 0x00050806, 0x00070C0A, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07,
        0x00090D07, 0x00090D07, 0x00090D07, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000C100B, 0x000C100B, 0x000C100B, 0x000A100B,
        0x000A100B, 0x000A100B, 0x000B110C, 0x000B110C, 0x000B110C, 0x000B110C, 0x000B110C, 0x000D140D, 0x000D110B, 0x0023231D, 0x00B7B4AF, 0x00282520, 0x0013130D, 0x0011150F, 0x000B120B, 0x00000D02, 0x00ABAEA5, 0x00EEE4DD, 0x00AFA59E, 0x0036342C,
        0x00ADB0A7, 0x00A4A29A, 0x00726861, 0x001F2318, 0x00ADAEA4, 0x0047453D, 0x00CECBC3, 0x00999590, 0x001D1D17, 0x00060A05, 0x00161C17, 0x0010140E, 0x0010140E, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000E120C, 0x000E120C, 0x000F130E,
        0x000F130E, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000F130E, 0x000F130E, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000D1210, 0x000B100E, 0x0010140F, 0x000E120D, 0x001D211B,
        0x00383C36, 0x003F443B, 0x00393E35, 0x00363937, 0x00101A14, 0x008F9C8E, 0x00F9FCE6, 0x00988C68, 0x00BA9C6B, 0x00E1BA7C, 0x00E2B974, 0x00D5C89A, 0x00F1DAAC, 0x00FDDBAD, 0x00E8C096, 0x00CCA987, 0x00DFD0B6, 0x00E9EEDF, 0x00DEF1E8, 0x00F7F8F6,
        0x00FDFFFF, 0x00EDF3F2, 0x00F2FCFC, 0x00F3FFFF, 0x00EFFFFF, 0x00E7FCFE, 0x00D2E9EB, 0x00BFE2E6, 0x00BCD5D1, 0x00B7BFAE, 0x00B4AB8A, 0x00BCA377, 0x11D4AD79, 0x3BF1C38D, 0x58FFD49C, 0x96EBE0BA, 0xBCFAF1CC, 0xEEFFFFE4, 0xFFFFFFED, 0xFFFFFFF0,
        0xFFFFFDF0, 0xFFFFFDF4, 0xFFFFFFF9, 0xFFFAFDFF, 0xFFFAFEFF, 0xFFFCFDFF, 0xFFFFFDFF, 0xFFFFFEFD, 0xFFFFFFFC, 0xFFFFFFFB, 0xFFFFFFF9, 0xFFFFFDFA, 0xFFFFFDFC, 0xFFFFFEFD, 0xFFFCFEFF, 0xFFFAFDFF, 0xFFF6FCFF, 0xFFF2FCFF, 0xFFF2FBFF, 0xFFF8FCFF,
        0xFFF8FBFF, 0xFFF8FBFF, 0xFFFAFCFD, 0xFFFDFDFD, 0xFFFEFEFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBFEFC, 0xFFFDFFFE, 0xFFFDFFFE, 0xFFF5F9F4, 0x00515550, 0x00020601, 0x000A0E08, 0x00080C06, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x000D110B, 0x00040802, 0x000C100A, 0x00080C06, 0x00080C06, 0x00070B05, 0x00050903, 0x005B5F59, 0x00E3D7D1, 0x00E6DCD5, 0x00C8C0B9, 0x00737068, 0x0024211C, 0x00000200, 0x000C100A, 0x006E756E, 0x00A9A79C,
        0x00201E13, 0x000B0A00, 0x0017180F, 0x00070903, 0x0012130F, 0x00020503, 0x000D1210, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07,
        0x00090D07, 0x00090D07, 0x00090D07, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000C100B, 0x000C100B, 0x000C100B, 0x000A100B,
        0x000A100B, 0x000A100B, 0x000B110C, 0x000B110C, 0x000B110C, 0x000B110C, 0x000B110C, 0x000C130C, 0x000D110B, 0x00282822, 0x00B6B3AE, 0x0023201B, 0x0013130D, 0x0011150F, 0x000C130C, 0x000B1B10, 0x00A9ACA3, 0x00E6DCD5, 0x00B2A8A1, 0x00413E36,
        0x00C9CCC3, 0x00DAD7CF, 0x009C928B, 0x00292D22, 0x00B2B3A9, 0x0039372F, 0x00C0BDB5, 0x008F8B86, 0x0020201A, 0x00131712, 0x000C120D, 0x0010140E, 0x0010140E, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000E120C, 0x000E120C, 0x000F130E,
        0x000F130E, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000F130E, 0x000F130E, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000D1210, 0x000B110C, 0x000E140F, 0x000D130E, 0x001E241F,
        0x00373E39, 0x003B423D, 0x00373F38, 0x0023373C, 0x00172C2E, 0x008CA2A0, 0x00E0F1E8, 0x009FA594, 0x00ABA78A, 0x00ACA27A, 0x00C9BA8C, 0x00DED2AA, 0x00E6D5AE, 0x00E2CDA7, 0x00CEB895, 0x00B5A489, 0x00E1DAC7, 0x00EDF1E6, 0x00DBE5DF, 0x00EDEBEB,
        0x00FDFDFD, 0x00FBFEFF, 0x00EEF7FB, 0x00E3F5FC, 0x00DDF7FE, 0x00B5D9E1, 0x00A9D0D9, 0x00B4DBCB, 0x00BAE4DD, 0x00BEEBEF, 0x00BEE4F0, 0x00BDD2DA, 0x00BEBCB4, 0x00C8AD92, 0x00D0A67C, 0x00E9B37E, 0x00E8B682, 0x3BE6BC8D, 0x5CE9C69E, 0x86ECD3B1,
        0xB4F2E4C7, 0xDEFBF4DB, 0xF2FFFEE6, 0xFFFFFCF3, 0xFFFFFEF5, 0xFFFFFFF8, 0xFFFFFFFB, 0xFFFFFFFE, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFF, 0xFFFDFFFF, 0xFFFBFFFF, 0xFFFAFFFF, 0xFFFAFFFF, 0xFFF8FFFF, 0xFFF8FEFF,
        0xFFF8FEFF, 0xFFFAFEFF, 0xFFFBFEFF, 0xFFFDFFFF, 0xFFFFFFFF, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFFFD, 0xFFFDFFFE, 0xFFFDFFFE, 0xFFF7FBF6, 0x005B5F5A, 0x00010500, 0x00090D07, 0x00090D07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x000A0E08, 0x00060A04, 0x00090D07, 0x00090D07, 0x00040802, 0x000E120C, 0x00000300, 0x00040802, 0x00605551, 0x00E5DCD8, 0x00C4BEB9, 0x00B6B2AD, 0x0095938B, 0x00282920, 0x00090C03, 0x000A0D04, 0x00ABA99E,
        0x00918F84, 0x00040300, 0x000E0F06, 0x000D0F09, 0x00000100, 0x000C0F0D, 0x00090E0C, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07,
        0x00090D07, 0x00090D07, 0x00090D07, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000C100B, 0x000C100B, 0x000C100B, 0x000A100B,
        0x000A100B, 0x000A100B, 0x000B110C, 0x000B110C, 0x000B110C, 0x000B110C, 0x000B110C, 0x000B120B, 0x000D110B, 0x002F2F29, 0x00B4B1AC, 0x001D1A15, 0x0013130D, 0x0010140E, 0x000C130C, 0x00000B00, 0x00B9BAB0, 0x00F0E5DD, 0x009B9088, 0x004F4D43,
        0x00DDDED4, 0x00D4D2C8, 0x004A3F37, 0x001C2015, 0x00B8B9AF, 0x0029271F, 0x00B3B0A8, 0x00A29E99, 0x0011110B, 0x00191D18, 0x000F1510, 0x0010140E, 0x0010140E, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000E120C, 0x000E120C, 0x000F130E,
        0x000F130E, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000F130E, 0x000F130E, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000B110C, 0x000D130E, 0x000D140F, 0x00202724,
        0x00363F3C, 0x0036413E, 0x00343F3C, 0x0024342D, 0x001C2B27, 0x00A6B6B5, 0x00D6EAEB, 0x007E9694, 0x00B6CFCB, 0x00D1E8E0, 0x00C2D5CC, 0x00DCD9D1, 0x00D7DAD1, 0x00D5E3D7, 0x00D7EBDE, 0x00B6CCC0, 0x00D2E0DA, 0x00E2E4E4, 0x00F5F0F2, 0x00E5E7E7,
        0x00E7E9EA, 0x00FCFEFF, 0x00F7FBFC, 0x00F6FFFF, 0x00EDFFFF, 0x00CDECEB, 0x00CEF3F1, 0x00C2E2E8, 0x00B1D1D6, 0x00A1C1C6, 0x00A3C4C7, 0x00B3D3D8, 0x00C0E2E8, 0x00BFE4EC, 0x00BBE0E8, 0x00B0CFC0, 0x00B0C8B6, 0x00B3BFA9, 0x00BEB9A0, 0x00D1B99B,
        0x00E4BC99, 0x00F2BD98, 0x4BFBBE96, 0x6DF8D7A5, 0x7DFADBAE, 0x9FFDE3BE, 0xC4FFEDD0, 0xEEFFF5E3, 0xFFFFFCF3, 0xFFFFFFFF, 0xFFFBFEFF, 0xFFF5FBFF, 0xFFF9FEFF, 0xFFFBFFFF, 0xFFFFFFFC, 0xFFFFFFF9, 0xFFFFFFF6, 0xFFFFFFF5, 0xFFFFFFF4, 0xFFF8FCFF,
        0xFFF9FDFF, 0xFFFAFDFF, 0xFFFDFFFF, 0xFFFDFFFF, 0xFFFFFFFF, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFFFE, 0xFFFCFFFD, 0xFFFAFDFB, 0xFFF3F7F2, 0x00616560, 0x00000200, 0x00080C06, 0x000A0E08, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00030701, 0x000F130D, 0x000A0E08, 0x00080C06, 0x00020600, 0x000D110B, 0x000C100A, 0x00080C06, 0x000F0C08, 0x00413C39, 0x00B8B4AF, 0x00CFCBC6, 0x00BFBCB4, 0x00A3A098, 0x00333127, 0x00050300, 0x0037352A,
        0x00CCCABF, 0x00403F35, 0x000F1007, 0x0012140E, 0x000E0F0B, 0x000C0F0D, 0x00030806, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07,
        0x00090D07, 0x00090D07, 0x00090D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000C100B, 0x000C100B, 0x000C100B, 0x000A100B,
        0x000A100B, 0x000A100B, 0x000B110C, 0x000B110C, 0x000B110C, 0x000B110C, 0x000B110C, 0x000B120B, 0x000D110B, 0x0034342E, 0x00B3B0AB, 0x0017140F, 0x0014140E, 0x000F130D, 0x000C130C, 0x000F1D11, 0x00BBBCB2, 0x00EEE3DB, 0x0091867E, 0x00342F26,
        0x007A7B71, 0x00A8A39A, 0x00A69B93, 0x0033372C, 0x00AFB0A6, 0x00424038, 0x00BCB9B1, 0x00A39F9A, 0x0010100A, 0x000E120D, 0x00141A15, 0x0010140E, 0x0010140E, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000E120C, 0x000E120C, 0x000F130E,
        0x000F130E, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000F130E, 0x000F130E, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120C, 0x000B110C, 0x000C120D, 0x000E1512, 0x001F2A27,
        0x0033403E, 0x0033403E, 0x00313F3E, 0x003C4229, 0x002F311D, 0x00B0B2A6, 0x00E2EBE8, 0x008BA3A3, 0x008FB0B3, 0x009EC1C4, 0x00BBDCDF, 0x00DAE3F1, 0x00D9ECF9, 0x00C4E9F1, 0x00ABDADE, 0x0089B4B7, 0x00BDD8DC, 0x00C9CDD2, 0x00CDC2CA, 0x00E0E6E5,
        0x00EAEFEE, 0x00FFFFFE, 0x00FFFDFC, 0x00F4F8F3, 0x00DDEBE5, 0x00C8E1D7, 0x00B8D8CD, 0x00A2D0E1, 0x00B4DDE6, 0x00C6E6E5, 0x00C9E2D8, 0x00BAD2C8, 0x00A9C4C0, 0x00A0C3C7, 0x00A0C6D2, 0x00B8E2EE, 0x00C1E8F1, 0x00C9EEF2, 0x00CEEDEC, 0x00CDE4DF,
        0x00C6DACE, 0x00C2D1C3, 0x00C0CDBD, 0x00CCC3A2, 0x00CBC0A0, 0x00CBBF9D, 0x00D0C09B, 0x60DAC59F, 0x69E4CEA4, 0x79F0D7AB, 0x82F8DEB0, 0xC9FFF4D1, 0xD9FFF8D9, 0xEEFFFEE4, 0xFFFFFFEE, 0xFFFFFFF6, 0xFFFFFFFB, 0xFFFDFFFF, 0xFFFBFEFF, 0xFFFFFFFC,
        0xFFFFFFFC, 0xFFFFFDFC, 0xFFFEFCFB, 0xFFFCFCFC, 0xFFFDFDFD, 0xFFFEFEFE, 0xFFFCFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFFFE, 0xFFFCFFFD, 0xFFFBFEFC, 0xFFF7FBF6, 0x006D716C, 0x00000400, 0x00080C06, 0x00090D07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00050903, 0x000B0F09, 0x00010500, 0x00060A04, 0x000A0E08, 0x0010140E, 0x00060A04, 0x00040802, 0x000B0E0C, 0x00000200, 0x00272824, 0x0093908B, 0x00CCC9C1, 0x00E1DAD1, 0x00D0C7BD, 0x0041372D, 0x0049473C,
        0x00DFDDD2, 0x007B7A70, 0x00040500, 0x00000200, 0x00141511, 0x00030604, 0x00090E0C, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07,
        0x00090D07, 0x00090D07, 0x00090D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000C100B, 0x000C100B, 0x000C100B, 0x000A100B,
        0x000A100B, 0x000A100B, 0x000B110C, 0x000B110C, 0x000B110C, 0x000B110C, 0x000B110C, 0x000A110A, 0x000D110B, 0x00393933, 0x00B2AFAA, 0x0013100B, 0x0014140E, 0x000F130D, 0x000C130C, 0x00101C10, 0x00C1C0B6, 0x00ECE1D9, 0x009A8D85, 0x00605B52,
        0x00919086, 0x006F6A61, 0x00736860, 0x0035392E, 0x00B0B1A7, 0x00312F27, 0x00B1AEA6, 0x00A7A39E, 0x001F1F19, 0x000C100B, 0x00070D08, 0x0010140E, 0x0010140E, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000E120C, 0x000E120C, 0x000F130E,
        0x000F130E, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000F130E, 0x000F130E, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120C, 0x000B120B, 0x000B120D, 0x000C1512, 0x00212C2A,
        0x00334140, 0x0030403F, 0x00303F41, 0x002D3C27, 0x003D4233, 0x00CAC7C2, 0x00DDE3E2, 0x00859B99, 0x00AECECD, 0x00CBEBEA, 0x00BFDAD7, 0x00C9DAE3, 0x00C6E0E7, 0x00C1E3E9, 0x00C2E8ED, 0x00A5C8CC, 0x00CFE4E6, 0x009FA4A7, 0x006A6567, 0x006D7875,
        0x009AA19E, 0x00EFF2F0, 0x00FCFCFC, 0x00E9EEED, 0x00DFEDEB, 0x00D3EEEB, 0x00C9ECE8, 0x00CEEBD2, 0x00BAD8C5, 0x00A6C6BB, 0x00A3C5C4, 0x00B2D5D9, 0x00BCDFE9, 0x00BCDCE9, 0x00B7D4E2, 0x0095C4C2, 0x0095C3C4, 0x009AC5C8, 0x00A5CFD4, 0x00B5DEE7,
        0x00C5EBF7, 0x00CBEFFF, 0x00CDF1FF, 0x00CFEEF7, 0x00CEE9ED, 0x00CCDFDC, 0x00CDD6C9, 0x00D0CFB5, 0x00D4CBA6, 0x00DACA9C, 0x00DFCA97, 0x00DCC18F, 0x00E0C698, 0x71E5CEA8, 0x92E8D6B7, 0xB4EBDEC8, 0xDEF1E8DB, 0xFFF8F2EB, 0xFFFDFAF6, 0xFFFFFFF3,
        0xFFFFFFF5, 0xFFFFFFF7, 0xFFFFFFF9, 0xFFFFFFFC, 0xFFFFFFFF, 0xFFF9FCFF, 0xFFF8FBFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFFFE, 0xFFFDFFFE, 0xFFFDFFFE, 0xFFFDFFFC, 0x117B7F7A, 0x00060A05, 0x000A0E08, 0x00070B05, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x000A0E08, 0x000B0F09, 0x000A0E08, 0x000A0E08, 0x00030701, 0x00080C06, 0x00040802, 0x0011150F, 0x00050E0B, 0x00030A07, 0x00070B06, 0x00090903, 0x004F4942, 0x00A1988F, 0x00DED2C8, 0x00E7D8CF, 0x00D4D2C7,
        0x00EAE8DD, 0x006E6D63, 0x00050600, 0x00080A04, 0x000C0D09, 0x000A0D0B, 0x000A0F0D, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07,
        0x00090D07, 0x00090D07, 0x00090D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000C100B, 0x000C100B, 0x000C100B, 0x000A100B,
        0x000A100B, 0x000A100B, 0x000B110C, 0x000B110C, 0x000B110C, 0x000B110C, 0x000B110C, 0x000A110A, 0x000D110B, 0x003B3B35, 0x00B1AEA9, 0x00100D08, 0x0014140E, 0x000E120C, 0x000D140D, 0x00101D0F, 0x00D1D1C5, 0x00EADED4, 0x0081756B, 0x006E6A5F,
        0x00DDDDD1, 0x00A39F94, 0x00AEA298, 0x0043473C, 0x00A8A99F, 0x0035332B, 0x009B9890, 0x0065615C, 0x0010100A, 0x000F130E, 0x000D130E, 0x0010140E, 0x0010140E, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000E120C, 0x000E120C, 0x000F130E,
        0x000F130E, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000F130E, 0x000F130E, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120C, 0x000B120B, 0x000B120D, 0x000C1512, 0x00202D2B,
        0x00344241, 0x002D3F40, 0x002E4041, 0x00123132, 0x00182B30, 0x00848B94, 0x00A7B2BA, 0x007A9397, 0x0091B1B0, 0x00A8C7C0, 0x00CCE3DB, 0x00DFF4EB, 0x00DDF2EA, 0x00D5E9E4, 0x00C5D8D5, 0x009BA9A7, 0x00DEE9E7, 0x00DDE4DF, 0x00CACEC9, 0x007E8D85,
        0x007F8983, 0x00EAEFED, 0x00F7F9FA, 0x00E1EAEE, 0x00D7EAF1, 0x00AACBD4, 0x00A7D2DB, 0x00B3CDDD, 0x00BFDCEB, 0x00C8E8F5, 0x00BEE2EA, 0x00AAD0D2, 0x009CC4BF, 0x009EC8BD, 0x00A8D1C2, 0x00D2DCE6, 0x00D5E1EB, 0x00D4E4EB, 0x00CADEE3, 0x00BBD4D6,
        0x00ADCCCB, 0x00A8CBC7, 0x00A8CEC8, 0x00B5D7DD, 0x00BCDEE4, 0x00C7E8F1, 0x00CEF1FB, 0x00CEF1FE, 0x00C6EBF9, 0x00BDE1F1, 0x00B7DBED, 0x00C9D1D0, 0x00CACEC8, 0x00CDCABB, 0x00D4C6AA, 0x00DEC39E, 0x4FEEC997, 0x4FFFD298, 0x5CFFDB9D, 0xB0ECDBC6,
        0xC4F2E3D0, 0xE6FCEFDF, 0xFFFFFAF0, 0xFFFFFFFB, 0xFFFFFFFF, 0xFFFBFEFF, 0xFFF7FDFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFEFFFB, 0x328C8D89, 0x00060703, 0x000D0F09, 0x000A0C06, 0x00080C06,
        0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x000D0B0A, 0x000D0B0A, 0x00110F0E, 0x000F0D0C, 0x00080605, 0x00060403, 0x0032302F, 0x00A09E9D, 0x00D9D3C8,
        0x006C685D, 0x0023241A, 0x000A0F06, 0x00060D06, 0x00020601, 0x000D0E0C, 0x000D0B0A, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07,
        0x00090D07, 0x00090D07, 0x00090D07, 0x00080D04, 0x00080D04, 0x00090D07, 0x00090D07, 0x00090D08, 0x00090D08, 0x000A0E09, 0x000A0D0B, 0x0012110D, 0x00040A05, 0x00080F0A, 0x000C0D09, 0x00140B08, 0x0019100D, 0x00131410, 0x000F1D17, 0x00151F19,
        0x00161D18, 0x000E1510, 0x00070E09, 0x00080F0A, 0x00070E09, 0x00050F09, 0x000A140E, 0x000A110A, 0x00070B05, 0x004A4A44, 0x00A8A5A0, 0x000C0904, 0x0014140E, 0x000E120C, 0x000A110A, 0x001C251B, 0x00CBCCC3, 0x00EDE5DE, 0x007C766F, 0x002E2F26,
        0x00C2C5BC, 0x00D8D2CB, 0x00B2A29C, 0x00383C31, 0x00A4A59B, 0x00424038, 0x00C4C1B9, 0x00A29E99, 0x0015150F, 0x00131712, 0x00040A05, 0x0010140E, 0x0010140E, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000E120C, 0x000E120C, 0x0010120C,
        0x0011120E, 0x000D0E0A, 0x00141511, 0x00131412, 0x000B0C0A, 0x00101010, 0x000F0F0F, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000F130E, 0x000F130E, 0x00090F0A, 0x000F1613, 0x00262F2C,
        0x00374240, 0x0033403E, 0x002B3836, 0x00293932, 0x001F2923, 0x009EA3A1, 0x008C9795, 0x00859E9A, 0x00C3E4E0, 0x00C0DCDC, 0x00C4D8D9, 0x00D5DFE6, 0x00C6DCE1, 0x00C5E7E7, 0x00C5E5E4, 0x00ADC2C0, 0x00E4ECEB, 0x00E9EEED, 0x00EFF6F3, 0x00D6DBDC,
        0x009FA5A4, 0x00E7EEEB, 0x00EEF8F2, 0x00E0EFEB, 0x00DFF5F3, 0x00D1EEF2, 0x00CEEEF3, 0x00BDDEE1, 0x00AFD0D3, 0x00A3C4C7, 0x00A6C7CA, 0x00B3D4D7, 0x00BDDEE1, 0x00BDDEE1, 0x00B7D8DB, 0x00A2C3C6, 0x00A0C1C4, 0x00A3C4C7, 0x00B0D1D4, 0x00C2E3E6,
        0x00CEEFF2, 0x00CEEFF2, 0x00CAEBEE, 0x00BFE0E3, 0x00B8D9DC, 0x00AFD0D3, 0x00AACBCE, 0x00ADCED1, 0x00B7D8DB, 0x00C4E5E8, 0x00CCEDF0, 0x00D5F6F9, 0x00D2F3F6, 0x00D0EFF2, 0x00C9E8EB, 0x00C1E0E3, 0x00B9D6DA, 0x00B2CFD3, 0x00ADCACE, 0x00C6C9AF,
        0x00CECDB3, 0x96DED2BA, 0xA7EEDAC1, 0xBCFBE3CB, 0xD1FFF1D6, 0xE6FFFBE0, 0xF7FFFFE7, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFE, 0xFFFEFFFD, 0xFFFFFFFE, 0xFFFFFFFC, 0x47979894, 0x000D0E0A, 0x000C0E08, 0x000A0C06, 0x00080C06,
        0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00060501, 0x000C0B07, 0x000F0E0A, 0x000A0905, 0x0011100C, 0x000C0B07, 0x0033322E, 0x005D5C58, 0x00BAB4A9,
        0x00504C41, 0x00030400, 0x00020700, 0x00020902, 0x00111510, 0x000A0B09, 0x000B0908, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07,
        0x00090D07, 0x00090D07, 0x00090D07, 0x00080D04, 0x00080D04, 0x00090D07, 0x00090D07, 0x00090D08, 0x00090D08, 0x000A0E09, 0x000A0D0B, 0x000D0E0A, 0x00050B06, 0x00080E09, 0x000D0C08, 0x00130C09, 0x00161511, 0x0014241D, 0x001A3C32, 0x002E594E,
        0x00395E54, 0x00334E45, 0x001C2E27, 0x000D1711, 0x0009100B, 0x000B110C, 0x00101712, 0x000A110A, 0x00060A04, 0x004C4C46, 0x00A6A39E, 0x000C0904, 0x0013130D, 0x000F130D, 0x000A110A, 0x001A2319, 0x00CCCDC4, 0x00DFD7D0, 0x00767069, 0x00313229,
        0x0060635A, 0x009D9790, 0x00685852, 0x0045493E, 0x00A2A399, 0x001B1911, 0x006F6C64, 0x006C6863, 0x00050500, 0x000D110C, 0x001D231E, 0x0010140E, 0x0010140E, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000E120C, 0x000E120C, 0x0012140E,
        0x00171913, 0x000B0C08, 0x000D0E0A, 0x00181917, 0x00141513, 0x00131412, 0x00141414, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000F130E, 0x000F130E, 0x00090F0A, 0x00101714, 0x0027302D,
        0x00374240, 0x0033403E, 0x002B3836, 0x001D2D26, 0x00434D47, 0x00F4F9F7, 0x00D1DCDA, 0x007C9591, 0x008BACA8, 0x00A6C2C2, 0x00D5E9EA, 0x00E0EAF1, 0x00D1E7EC, 0x00C3E5E5, 0x00B0D0CF, 0x0099AEAC, 0x00DFE7E6, 0x00EEF3F2, 0x00F3FAF7, 0x00E0E5E6,
        0x00ABB1B0, 0x00EFF6F3, 0x00EFF9F3, 0x00D9E8E4, 0x00CDE3E1, 0x00B5D2D6, 0x00AECED3, 0x00B6D7DA, 0x00C1E2E5, 0x00C9EAED, 0x00C2E3E6, 0x00B1D2D5, 0x00A4C5C8, 0x00A1C2C5, 0x00A5C6C9, 0x00BFE0E3, 0x00C2E3E6, 0x00C0E1E4, 0x00B6D7DA, 0x00A9CACD,
        0x00A3C4C7, 0x00A8C9CC, 0x00AFD0D3, 0x00BADBDE, 0x00C6E7EA, 0x00D2F3F6, 0x00D4F5F8, 0x00CCEDF0, 0x00BFE0E3, 0x00B7D8DB, 0x00B4D5D8, 0x00ACCDD0, 0x00B1D2D5, 0x00BCDBDE, 0x00C6E5E8, 0x00CDECEF, 0x00D3F0F4, 0x00D4F1F5, 0x00D4F1F5, 0x00DBECDF,
        0x00DAE6DA, 0x00DBDFD4, 0x00DDD9CE, 0x00E1D8CE, 0x00E8E0D3, 0x00F1EBDE, 0xF2F7F4E6, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFE, 0xFFFCFDFB, 0xFFFEFFFD, 0xFFFFFFFC, 0x6DA8A9A5, 0x00151612, 0x000B0D07, 0x000A0C06, 0x00080C06,
        0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x000F0E0A, 0x0011100C, 0x00050400, 0x000A0905, 0x000A0905, 0x000B0A06, 0x00878682, 0x00C5C4C0, 0x00645E53,
        0x007A766B, 0x000E0F05, 0x00090E05, 0x000D140D, 0x00070B06, 0x000C0D0B, 0x000C0A09, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07,
        0x00090D07, 0x00090D07, 0x00090D07, 0x00080D04, 0x00080D04, 0x00090D07, 0x00090D07, 0x00090D08, 0x00090D08, 0x000A0E09, 0x000A0D0B, 0x000B0C08, 0x00060C07, 0x000A0E09, 0x000C0B07, 0x00120F0B, 0x000E1A14, 0x00153A30, 0x00316E60, 0x003B9784,
        0x0057A594, 0x00589083, 0x0037564D, 0x0016241E, 0x000C100B, 0x000D0E0A, 0x0010110D, 0x000A110A, 0x00050903, 0x004F4F49, 0x00A3A09B, 0x000B0803, 0x0012120C, 0x000F130D, 0x00091009, 0x0020291F, 0x00D1D2C9, 0x00F1E9E2, 0x00666059, 0x0087887F,
        0x0095988F, 0x00DFD9D2, 0x009E8E88, 0x00474B40, 0x00A7A89E, 0x00413F37, 0x007F7C74, 0x0056524D, 0x0010100A, 0x00171B16, 0x00050B06, 0x0010140E, 0x0010140E, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000E120C, 0x000E120C, 0x000C0E08,
        0x00161812, 0x0013150F, 0x0010110D, 0x000F100C, 0x000C0D0B, 0x0010110F, 0x00131412, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000F130E, 0x000F130E, 0x00090F0A, 0x00111815, 0x0028312E,
        0x00384341, 0x0033403E, 0x002C3937, 0x0026362F, 0x003F4943, 0x00DDE2E0, 0x00B8C3C1, 0x00869F9B, 0x00B9DAD6, 0x00C7E3E3, 0x00C1D5D6, 0x00DCE6ED, 0x00C6DCE1, 0x00C3E5E5, 0x00C6E6E5, 0x00B3C8C6, 0x00E3EBEA, 0x00E8EDEC, 0x00F4FBF8, 0x00E0E5E6,
        0x00B0B6B5, 0x00F0F7F4, 0x00F0FAF4, 0x00E0EFEB, 0x00DBF1EF, 0x00CDEAEE, 0x00CBEBF0, 0x00BCDDE0, 0x00AFD0D3, 0x00A5C6C9, 0x00AACBCE, 0x00B8D9DC, 0x00C0E1E4, 0x00BBDCDF, 0x00B2D3D6, 0x009DBEC1, 0x009EBFC2, 0x00A5C6C9, 0x00B2D3D6, 0x00C0E1E4,
        0x00C8E9EC, 0x00C7E8EB, 0x00C3E4E7, 0x00B6D7DA, 0x00ADCED1, 0x00A5C6C9, 0x00A7C8CB, 0x00B4D5D8, 0x00C3E4E7, 0x00CFF0F3, 0x00D4F5F8, 0x00D2F3F6, 0x00CBECEF, 0x00C4E3E6, 0x00BBDADD, 0x00B6D5D8, 0x00B8D5D9, 0x00BCD9DD, 0x00C0DDE1, 0x00CDECEF,
        0x00D4EFF3, 0x00DFF4F6, 0x00E7F6F9, 0x00E9F5F7, 0x00E8F1F4, 0xEEE4F0F0, 0xEEE3EFEF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFE, 0xFFFCFDFB, 0xFFFEFFFD, 0xFFFFFFFC, 0x8EB8B9B5, 0x001B1C18, 0x00070903, 0x000B0D07, 0x00080C06,
        0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00070602, 0x00020100, 0x000C0B07, 0x001A1915, 0x00080703, 0x00020100, 0x005F5E5A, 0x00DEDDD9, 0x003D372C,
        0x00252116, 0x00070800, 0x000E130A, 0x00020902, 0x00060A05, 0x00070806, 0x000F0D0C, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07,
        0x00090D07, 0x00090D07, 0x00090D07, 0x00080D04, 0x00080D04, 0x00090D07, 0x00090D07, 0x00090D08, 0x00090D08, 0x000A0E09, 0x000A0D0B, 0x00090D08, 0x00090D08, 0x000B0C08, 0x000F0C08, 0x0011100C, 0x00051A12, 0x00154A3D, 0x004C9C8B, 0x0046C6AD,
        0x0069D6C0, 0x0070BEAD, 0x004D786D, 0x0026362F, 0x0010140F, 0x000D0A06, 0x000F0C08, 0x000A110A, 0x00040802, 0x0053534D, 0x009F9C97, 0x000B0803, 0x0011110B, 0x000F130D, 0x00080F08, 0x00273026, 0x00CFD0C7, 0x00E8E0D9, 0x00605A53, 0x008C8D84,
        0x00777A71, 0x00726C65, 0x00A79791, 0x0054584D, 0x008C8D83, 0x0044423A, 0x00C1BEB6, 0x00615D58, 0x00070701, 0x00111510, 0x000D130E, 0x0010140E, 0x0010140E, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000E120C, 0x000E120C, 0x0014170E,
        0x000E100A, 0x0012140E, 0x00161812, 0x0010110D, 0x0011120E, 0x00131412, 0x000A0B09, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x0010140F, 0x000F130E, 0x000A100B, 0x00131A17, 0x002A3330,
        0x00384341, 0x0033403E, 0x002C3937, 0x002B3B34, 0x00454F49, 0x00E1E6E4, 0x00B7C2C0, 0x0077908C, 0x0091B2AE, 0x00B3CFCF, 0x00D6EAEB, 0x00DFE9F0, 0x00D4EAEF, 0x00C5E7E7, 0x00A7C7C6, 0x0097ACAA, 0x00E0E8E7, 0x00F0F5F4, 0x00EEF5F2, 0x00D5DADB,
        0x00B1B7B6, 0x00F3FAF7, 0x00F1FBF5, 0x00DDECE8, 0x00CEE4E2, 0x00B6D3D7, 0x00AFCFD4, 0x00B8D9DC, 0x00C5E6E9, 0x00CEEFF2, 0x00C6E7EA, 0x00B1D2D5, 0x00A1C2C5, 0x009FC0C3, 0x00A4C5C8, 0x00BCDDE0, 0x00BBDCDF, 0x00B6D7DA, 0x00ACCDD0, 0x00A2C3C6,
        0x00A0C1C4, 0x00A7C8CB, 0x00AECFD2, 0x00BEDFE2, 0x00C9EAED, 0x00D1F2F5, 0x00CCEDF0, 0x00BDDEE1, 0x00AECFD2, 0x00A8C9CC, 0x00A8C9CC, 0x00ACCDD0, 0x00B6D7DA, 0x00C6E5E8, 0x00D1F0F3, 0x00D5F4F7, 0x00D2EFF3, 0x00CCE9ED, 0x00C7E4E8, 0x00B3D7DF,
        0x00B9DBE1, 0x00C3E0E7, 0x00CCE6ED, 0x00D7EDF3, 0x00E1F3FA, 0xF7E8FBFF, 0xFFEEFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFE, 0xFFFDFEFC, 0xFFFFFFFE, 0xFFFFFFFC, 0xABC6C7C3, 0x0020211D, 0x00030500, 0x000D0F09, 0x00080C06,
        0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00393933, 0x00565650, 0x00080802, 0x00070701, 0x0013130D, 0x00070701, 0x00575751, 0x00E2E2DC, 0x005D574C,
        0x00070300, 0x000F1006, 0x000C1108, 0x00040B04, 0x000C100B, 0x000D0E0C, 0x000D0B0A, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07,
        0x00090D07, 0x00090D07, 0x00090D07, 0x00080D04, 0x00080D04, 0x00090D07, 0x00090D07, 0x00090D08, 0x00090D08, 0x000A0E09, 0x000A0D0B, 0x000B0F0A, 0x00090D08, 0x000B0C08, 0x00110E0A, 0x00100F0B, 0x0000130B, 0x00154A3D, 0x0059A998, 0x004CD1B7,
        0x0066D5BF, 0x0067B7A6, 0x004A756A, 0x00263831, 0x00131410, 0x00100D09, 0x0015120E, 0x00091009, 0x00030701, 0x00575751, 0x009A9792, 0x000A0702, 0x0010100A, 0x0010140E, 0x00060D06, 0x00273026, 0x00DADBD2, 0x00EDE5DE, 0x005C564F, 0x0026271E,
        0x00BABDB4, 0x00CFC9C2, 0x007F6F69, 0x004D5146, 0x0096978D, 0x0047453D, 0x00D3D0C8, 0x00635F5A, 0x00181812, 0x00161A15, 0x000C120D, 0x0010140E, 0x0010140E, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000E120C, 0x000E120C, 0x0012150C,
        0x000D1007, 0x0010130A, 0x0013150F, 0x0011130D, 0x0010110D, 0x0010110D, 0x0011120E, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x0010140F, 0x000E120D, 0x000A100B, 0x00151C19, 0x002C3532,
        0x00384341, 0x00323F3D, 0x002C3937, 0x001B2B24, 0x0056605A, 0x00E8EDEB, 0x00ADB8B6, 0x0090A9A5, 0x00BDDEDA, 0x00C5E1E1, 0x00C1D5D6, 0x00D8E2E9, 0x00C3D9DE, 0x00C1E3E3, 0x00C1E1E0, 0x00BACFCD, 0x00E7EFEE, 0x00ECF1F0, 0x00F6FDFA, 0x00D5DADB,
        0x00B7BDBC, 0x00F2F9F6, 0x00EDF7F1, 0x00DFEEEA, 0x00D7EDEB, 0x00CAE7EB, 0x00CAEAEF, 0x00B7D8DB, 0x00ABCCCF, 0x00A3C4C7, 0x00AACBCE, 0x00BCDDE0, 0x00C6E7EA, 0x00C2E3E6, 0x00B9DADD, 0x009CBDC0, 0x009EBFC2, 0x00A7C8CB, 0x00B5D6D9, 0x00C2E3E6,
        0x00C7E8EB, 0x00C3E4E7, 0x00BCDDE0, 0x00ABCCCF, 0x00A5C6C9, 0x00A2C3C6, 0x00AACBCE, 0x00B9DADD, 0x00C7E8EB, 0x00CDEEF1, 0x00CEEFF2, 0x00C5E6E9, 0x00BEDFE2, 0x00B4D3D6, 0x00ABCACD, 0x00A9C8CB, 0x00B1CED2, 0x00BEDBDF, 0x00C8E5E9, 0x00D4F3F6,
        0x00D4F1F5, 0x00D3F0F4, 0x00D4EDEF, 0xD5D7ECEE, 0xD9DAECED, 0xE6E0EDEF, 0xEEE4F0F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFE, 0xFFFEFFFD, 0xFFFFFFFE, 0xFFFEFFFB, 0xCDD6D7D3, 0x002B2C28, 0x00030500, 0x000D0F09, 0x00090D07,
        0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x0044443E, 0x00BEBEB8, 0x00686862, 0x000E0E08, 0x000A0A04, 0x00020200, 0x00383832, 0x00CDCDC7, 0x006A6459,
        0x00100C01, 0x000F1006, 0x00010600, 0x00080F08, 0x00060A05, 0x0010110F, 0x00090706, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07,
        0x00090D07, 0x00090D07, 0x00090D07, 0x00080D04, 0x00080D04, 0x00090D07, 0x00090D07, 0x00090D08, 0x00090D08, 0x000A0E09, 0x000A0D0B, 0x000E0F0B, 0x00070D08, 0x000A0E09, 0x000F0E0A, 0x00120F0B, 0x00010D07, 0x0012372D, 0x004B887A, 0x005DC4AF,
        0x0063BCA9, 0x00569486, 0x00375A50, 0x001A2A23, 0x000C100B, 0x000D0C08, 0x00171410, 0x00091009, 0x00020600, 0x005B5B55, 0x0096938E, 0x00090601, 0x000E0E08, 0x0010140E, 0x00050C05, 0x00262F25, 0x00D2D3CA, 0x00D9D1CA, 0x00635D56, 0x00393A31,
        0x00676A61, 0x00A29C95, 0x003F2F29, 0x0061655A, 0x0087887E, 0x0044423A, 0x00E8E5DD, 0x00716D68, 0x000A0A04, 0x000B0F0A, 0x00101611, 0x0010140E, 0x0010140E, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000E120C, 0x000E120C, 0x0013170C,
        0x0015180F, 0x0012150C, 0x000C0F06, 0x0011130D, 0x0011130D, 0x000C0D09, 0x00151612, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x0010140F, 0x000E120D, 0x000A100B, 0x00171E1B, 0x002E3734,
        0x00394442, 0x00323F3D, 0x002C3937, 0x001C2C25, 0x005E6862, 0x00EBF0EE, 0x00AAB5B3, 0x00839C98, 0x0096B7B3, 0x00ADC9C9, 0x00D7EBEC, 0x00DCE6ED, 0x00D2E8ED, 0x00C3E5E5, 0x00A2C2C1, 0x009EB3B1, 0x00E4ECEB, 0x00F3F8F7, 0x00EFF6F3, 0x00CDD2D3,
        0x00B9BFBE, 0x00F5FCF9, 0x00EDF7F1, 0x00DBEAE6, 0x00C7DDDB, 0x00B0CDD1, 0x00AACACF, 0x00BEDFE2, 0x00C9EAED, 0x00CFF0F3, 0x00C5E6E9, 0x00B0D1D4, 0x00A1C2C5, 0x00A1C2C5, 0x00A7C8CB, 0x00BEDFE2, 0x00BCDDE0, 0x00B6D7DA, 0x00ABCCCF, 0x00A2C3C6,
        0x00A1C2C5, 0x00A8C9CC, 0x00B0D1D4, 0x00C5E6E9, 0x00CBECEF, 0x00CCEDF0, 0x00C3E4E7, 0x00B3D4D7, 0x00A7C8CB, 0x00A5C6C9, 0x00A7C8CB, 0x00B7D8DB, 0x00BFE0E3, 0x00CBEAED, 0x00D0EFF2, 0x00CDECEF, 0x00C6E3E7, 0x00BEDBDF, 0x00B9D6DA, 0x00BDD5D5,
        0x00C3DBDB, 0x00CCE4E4, 0x00D5EDEB, 0xE6DFF4F2, 0xFFEBF9F7, 0xFFF4FCFB, 0xFFFAFFFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFE, 0xFFFDFEFC, 0xFFFFFFFE, 0xFFFDFEFA, 0xEEE6E7E3, 0x003A3B37, 0x00070903, 0x000C0E08, 0x00090D07,
        0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00070800, 0x00717269, 0x0096978E, 0x000C0D04, 0x00111209, 0x0015160D, 0x00020300, 0x00686960, 0x00A29C91,
        0x00120E03, 0x000E0F05, 0x000E130A, 0x00070E07, 0x0010140F, 0x00050604, 0x000C0A09, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07,
        0x00090D07, 0x00090D07, 0x00090D07, 0x00080D04, 0x00080D04, 0x00090D07, 0x00090D07, 0x00090D08, 0x00090D08, 0x000A0E09, 0x000A0D0B, 0x000C0D09, 0x00080E09, 0x00090F0A, 0x000D0C08, 0x00140D0A, 0x000D0C08, 0x000E1E17, 0x00294B41, 0x00559082,
        0x0054867A, 0x0041665C, 0x00223A32, 0x00101C16, 0x00080E09, 0x00090A06, 0x000C0D09, 0x00091009, 0x00010500, 0x005E5E58, 0x0093908B, 0x00090601, 0x000D0D07, 0x0010140E, 0x00040B04, 0x002B342A, 0x00D4D5CC, 0x00E9E1DA, 0x00565049, 0x00919289,
        0x009C9F96, 0x00E9E3DC, 0x0084746E, 0x005B5F54, 0x007D7E74, 0x004E4C44, 0x00C7C4BC, 0x00797570, 0x0012120C, 0x00121611, 0x000D130E, 0x0010140E, 0x0010140E, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000E120C, 0x000E120C, 0x001A1E13,
        0x001C2015, 0x0025281F, 0x00181B12, 0x000C0E08, 0x00141610, 0x0013150F, 0x000D0E0A, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x0010140F, 0x000E120D, 0x000A100B, 0x00181F1C, 0x00303936,
        0x00394442, 0x00323F3D, 0x002C3937, 0x001B2B24, 0x00656F69, 0x00E6EBE9, 0x009EA9A7, 0x0094ADA9, 0x00BDDEDA, 0x00C2DEDE, 0x00C5D9DA, 0x00D7E1E8, 0x00C7DDE2, 0x00C3E5E5, 0x00B7D7D6, 0x00B8CDCB, 0x00E8F0EF, 0x00EDF2F1, 0x00F1F8F5, 0x00C9CECF,
        0x00BAC0BF, 0x00F4FBF8, 0x00EDF7F1, 0x00E2F1ED, 0x00D7EDEB, 0x00CAE7EB, 0x00CAEAEF, 0x00B0D1D4, 0x00AACBCE, 0x00AACBCE, 0x00B4D5D8, 0x00C3E4E7, 0x00C8E9EC, 0x00C0E1E4, 0x00B5D6D9, 0x009EBFC2, 0x009FC0C3, 0x00A5C6C9, 0x00B4D5D8, 0x00C2E3E6,
        0x00C6E7EA, 0x00BFE0E3, 0x00B5D6D9, 0x00A3C4C7, 0x00A2C3C6, 0x00A6C7CA, 0x00AECFD2, 0x00BBDCDF, 0x00C6E7EA, 0x00CBECEF, 0x00CDEEF1, 0x00B1D2D5, 0x00AFD0D3, 0x00AECDD0, 0x00ABCACD, 0x00AECDD0, 0x00B9D6DA, 0x00C8E5E9, 0x00D3F0F4, 0x00D9EDF2,
        0x00D5EBF0, 0x00D0E9EB, 0x00CDE6E8, 0xC9D2E7E8, 0xE2DDEAEC, 0xFFEBF0F3, 0xFFF3F5F6, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFE, 0xFFFDFEFC, 0xFFFFFFFE, 0xFFFCFDF9, 0xFFF2F3EF, 0x00464743, 0x000B0D07, 0x000A0C06, 0x00090D07,
        0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x000B0C03, 0x000D0E05, 0x001C1D14, 0x00070800, 0x008C8D84, 0x0096978E, 0x0053544B, 0x001A1B12, 0x00A7A196,
        0x005D594E, 0x00070800, 0x00000500, 0x00080F08, 0x00090D08, 0x00080907, 0x00100E0D, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07,
        0x00090D07, 0x00090D07, 0x00090D07, 0x00080D04, 0x00080D04, 0x00090D07, 0x00090D07, 0x00090D08, 0x00090D08, 0x000A0E09, 0x000A0D0B, 0x000B0A06, 0x00090F0A, 0x0009100B, 0x00090A06, 0x00160D0A, 0x00170E0B, 0x000A0B07, 0x000C1A14, 0x00132E25,
        0x001A322A, 0x0012271F, 0x0004120C, 0x00030D07, 0x0009100B, 0x000A100B, 0x000C100B, 0x00091009, 0x00010500, 0x0060605A, 0x00918E89, 0x00090601, 0x000D0D07, 0x0011150F, 0x00040B04, 0x0030392F, 0x00D5D6CD, 0x00E3DBD4, 0x0048423B, 0x008B8C83,
        0x006E7168, 0x00746E67, 0x00968680, 0x005D6156, 0x0083847A, 0x005F5D55, 0x00D0CDC5, 0x00D3CFCA, 0x00393933, 0x00030702, 0x00101611, 0x0010140E, 0x0010140E, 0x000F130D, 0x000F130D, 0x000F130D, 0x000F130D, 0x000E120C, 0x000E120C, 0x0075796E,
        0x00868A7F, 0x00AEB2A7, 0x007E8178, 0x001A1D14, 0x00080A04, 0x001C1E18, 0x0011130D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x0010140F, 0x000D110C, 0x000B110C, 0x0019201D, 0x00303936,
        0x00394442, 0x00313E3C, 0x002C3937, 0x001A2A23, 0x006A746E, 0x00F0F5F3, 0x009CA7A5, 0x0079928E, 0x0097B8B4, 0x00B7D3D3, 0x00D9EDEE, 0x00E3EDF4, 0x00D2E8ED, 0x00C1E3E3, 0x00A3C3C2, 0x00A4B9B7, 0x00E3EBEA, 0x00F2F7F6, 0x00F4FBF8, 0x00C9CECF,
        0x00BEC4C3, 0x00F6FDFA, 0x00EBF5EF, 0x00DBEAE6, 0x00C8DEDC, 0x00B5D2D6, 0x00B1D1D6, 0x00C0E1E4, 0x00C9EAED, 0x00CDEEF1, 0x00C2E3E6, 0x00B0D1D4, 0x00A3C4C7, 0x00A5C6C9, 0x00ABCCCF, 0x00BFE0E3, 0x00BDDEE1, 0x00B5D6D9, 0x00A7C8CB, 0x009CBDC0,
        0x009EBFC2, 0x00AECFD2, 0x00BCDDE0, 0x00CBECEF, 0x00C9EAED, 0x00C4E5E8, 0x00BADBDE, 0x00B0D1D4, 0x00A8C9CC, 0x00A6C7CA, 0x00A7C8CB, 0x00BFE0E3, 0x00C5E6E9, 0x00CDECEF, 0x00CCEBEE, 0x00C4E3E6, 0x00B9D6DA, 0x00B0CDD1, 0x00ABC8CC, 0x00BACED9,
        0x00C1D8E0, 0x00CBE7EE, 0x00D6F2F9, 0x00DFF7FD, 0xF7E7F7FE, 0xFFECF4FB, 0xFFEFF1F9, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFE, 0xFFF8F9F7, 0x00636460, 0x00060703, 0x00070804, 0x000B0D07,
        0x000B0D07, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000B0D07, 0x000A0C06, 0x000A0C06, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080F0C, 0x00040A05, 0x000A0E09, 0x00050602, 0x00595651, 0x00D5CFCA, 0x00DDD5CE, 0x00B6AEA7, 0x00979289,
        0x00BBB9AF, 0x002A291F, 0x00040800, 0x000A1005, 0x00050F03, 0x00061206, 0x00000A00, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000B0D07, 0x000C0E08, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06,
        0x00080C06, 0x00080C06, 0x00080C06, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000E0F0B,
        0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x00091009, 0x00030701, 0x0061615B, 0x008F8C87, 0x00090601, 0x0010100A, 0x000E120C, 0x00020902, 0x003D3C32, 0x00D2D1C7, 0x00E1DFD5, 0x004D4B41, 0x003D382F,
        0x00BEB9B0, 0x00DBD4CB, 0x00575047, 0x006C6D63, 0x0078776D, 0x0047443C, 0x00C1BBB4, 0x009F9994, 0x00312E29, 0x000C0D09, 0x000F1510, 0x000E120D, 0x0010140F, 0x000F130E, 0x000D110C, 0x000F130E, 0x00111510, 0x000E120D, 0x00090D08, 0x008D8984,
        0x0046423D, 0x0063605B, 0x008D8A85, 0x0025221D, 0x0012120C, 0x001A1A14, 0x00191913, 0x0011160D, 0x00081006, 0x000C140D, 0x00121D15, 0x0009100B, 0x000B0F0A, 0x001A1915, 0x00140F0E, 0x000D100E, 0x00131614, 0x000A0F0D, 0x001A211E, 0x00343D3A,
        0x00384340, 0x002F3D39, 0x00293733, 0x0013231C, 0x007B8481, 0x00E9EFEE, 0x009BACA9, 0x009DBAB7, 0x00B4D4D3, 0x00BDD4D6, 0x00CDD9DD, 0x00CADAE0, 0x00C4DFE3, 0x00C6E8E8, 0x00B7D3D3, 0x00BBC9C8, 0x00EDF2F1, 0x00EFF4F3, 0x00EEF7F4, 0x00BFC8C5,
        0x00C7CDCC, 0x00F6FEFD, 0x00E4EEEE, 0x00E0EFF1, 0x00D6EDEF, 0x00C7E6E9, 0x00BFE2E5, 0x00AFD0D3, 0x00A9CACD, 0x00A8C9CC, 0x00B3D4D7, 0x00C2E3E6, 0x00C8E9EC, 0x00BFE0E3, 0x00B4D5D8, 0x009ABBBE, 0x009EBFC2, 0x00A9CACD, 0x00B8D9DC, 0x00C4E5E8,
        0x00C3E4E7, 0x00B5D6D9, 0x00A7C8CB, 0x009BBCBF, 0x00A1C2C5, 0x00AECFD2, 0x00BBDCDF, 0x00C5E6E9, 0x00C8E9EC, 0x00C6E7EA, 0x00C2E3E6, 0x00AECFD2, 0x00A8C9CC, 0x00A4C5C8, 0x00A6C7CA, 0x00B1D2D5, 0x00BFE0E3, 0x00CBECEF, 0x00D0F1F4, 0x00CBE6EA,
        0x00C7E2E6, 0x00C5DCDE, 0x00C6DBDD, 0x00CDDFE0, 0xD9DAE7E9, 0xF2E6F2F4, 0xFFEFF9F9, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFEFFFD, 0xFFFDFEFC, 0x0072736F, 0x00080905, 0x00080905, 0x000B0D07,
        0x000B0D07, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000B0D07, 0x000B0D07, 0x000A0C06, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00070E0B, 0x00020803, 0x00080C07, 0x00060703, 0x00080500, 0x006B6762, 0x00D0CAC3, 0x00AFA7A0, 0x00C9C2B9,
        0x00DDD8CF, 0x00A7A59B, 0x001A190F, 0x00050900, 0x00080E03, 0x000B1308, 0x00061004, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000A0C06, 0x00090B05, 0x00080A04, 0x00080A04, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06,
        0x00080C06, 0x00080C06, 0x00080C06, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000D0E0A,
        0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x00050C05, 0x00010500, 0x0062625C, 0x00928F8A, 0x000B0803, 0x000F0F09, 0x000E120C, 0x00060D06, 0x0039382E, 0x00E0DFD5, 0x00D0CEC4, 0x004D4B41, 0x00746F66,
        0x006D685F, 0x00615A51, 0x003C352C, 0x00727369, 0x00747369, 0x00646159, 0x00DAD4CD, 0x00A7A19C, 0x00201D18, 0x000D0E0A, 0x000D130E, 0x000E120D, 0x0010140F, 0x000F130E, 0x000E120D, 0x000F130E, 0x0010140F, 0x000E120D, 0x000A0E09, 0x00635F5A,
        0x00918D88, 0x00585550, 0x00989590, 0x002F2C27, 0x000E0E08, 0x0015150F, 0x000D0D07, 0x000C0F06, 0x00171C13, 0x000D150E, 0x000A120B, 0x000A110C, 0x000D110C, 0x00171612, 0x00120E0D, 0x000D100E, 0x00121513, 0x000A0F0D, 0x001A211E, 0x00343D3A,
        0x0037423F, 0x002E3C38, 0x00273531, 0x0021312A, 0x00919A97, 0x00EFF5F4, 0x008E9F9C, 0x00809D9A, 0x00A0C0BF, 0x00C5DCDE, 0x00DDE9ED, 0x00DCECF2, 0x00C6E1E5, 0x00B8DADA, 0x00A3BFBF, 0x00B3C1C0, 0x00ECF1F0, 0x00F2F7F6, 0x00EFF8F5, 0x00BEC7C4,
        0x00C6CCCB, 0x00F4FCFB, 0x00E5EFEF, 0x00D7E6E8, 0x00BED5D7, 0x00B1D0D3, 0x00B4D7DA, 0x00C3E4E7, 0x00CAEBEE, 0x00CDEEF1, 0x00C2E3E6, 0x00B0D1D4, 0x00A6C7CA, 0x00A8C9CC, 0x00AFD0D3, 0x00C7E8EB, 0x00BFE0E3, 0x00B1D2D5, 0x00A1C2C5, 0x009ABBBE,
        0x00A2C3C6, 0x00B5D6D9, 0x00C5E6E9, 0x00D0F1F4, 0x00C7E8EB, 0x00BADBDE, 0x00AECFD2, 0x00A8C9CC, 0x00A8C9CC, 0x00ABCCCF, 0x00ADCED1, 0x00C5E6E9, 0x00C9EAED, 0x00CBECEF, 0x00C7E8EB, 0x00BCDDE0, 0x00B0D1D4, 0x00A8C9CC, 0x00A4C5C8, 0x00BBD6DA,
        0x00C6E1E5, 0x00D9F0F2, 0x00E2F7F9, 0x00E4F6F7, 0x00E6F3F5, 0xF7E8F4F6, 0xFFECF6F6, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFDFEFC, 0xFFFFFFFE, 0x26878884, 0x000A0B07, 0x000A0B07, 0x000B0D07,
        0x000B0D07, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00030806, 0x000C120D, 0x000C100B, 0x00161713, 0x00020200, 0x00110E09, 0x00706D65, 0x0098928B, 0x002D251E,
        0x007C746D, 0x00C3BDB6, 0x00AAA79F, 0x004A4840, 0x00080900, 0x00020500, 0x0010130A, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000A0C06, 0x00080A04, 0x00070903, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06,
        0x00080C06, 0x00080C06, 0x00080C06, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000D0E0A,
        0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x00091009, 0x00010500, 0x005C5C56, 0x008B8883, 0x00080500, 0x000F0F09, 0x000D110B, 0x00050C05, 0x00444339, 0x00D5D4CA, 0x00D7D5CB, 0x0046443A, 0x008F8A81,
        0x00B6B1A8, 0x00968F86, 0x006C655C, 0x006D6E64, 0x00747369, 0x0057544C, 0x00C7C1BA, 0x00C3BDB8, 0x00393631, 0x000F100C, 0x000D130E, 0x000E120D, 0x000F130E, 0x000F130E, 0x000E120D, 0x000E120D, 0x0010140F, 0x000E120D, 0x000B0F0A, 0x000B0702,
        0x005A5651, 0x00989590, 0x0066635E, 0x0017140F, 0x000E0E08, 0x00262620, 0x00161610, 0x00181910, 0x0014170E, 0x00020902, 0x000A120B, 0x00171E19, 0x000E120D, 0x000E0D09, 0x00161211, 0x000C0F0D, 0x00111412, 0x000A0F0D, 0x001A211E, 0x00333C39,
        0x0036413E, 0x002D3B37, 0x0025332F, 0x002C3C35, 0x00B8C1BE, 0x00E7EDEC, 0x00869794, 0x00A1BEBB, 0x00B8D8D7, 0x00BCD3D5, 0x00CDD9DD, 0x00CDDDE3, 0x00C7E2E6, 0x00C9EBEB, 0x00B6D2D2, 0x00C4D2D1, 0x00EEF3F2, 0x00EDF2F1, 0x00E9F2EF, 0x00BAC3C0,
        0x00D0D6D5, 0x00F5FDFC, 0x00E1EBEB, 0x00E0EFF1, 0x00DDF4F6, 0x00CAE9EC, 0x00B6D9DC, 0x00AFD0D3, 0x00A9CACD, 0x00AACBCE, 0x00B7D8DB, 0x00C6E7EA, 0x00C9EAED, 0x00BBDCDF, 0x00ABCCCF, 0x0092B3B6, 0x009DBEC1, 0x00AFD0D3, 0x00C0E1E4, 0x00C6E7EA,
        0x00BEDFE2, 0x00ADCED1, 0x009FC0C3, 0x009CBDC0, 0x00A6C7CA, 0x00B5D6D9, 0x00C2E3E6, 0x00C8E9EC, 0x00C6E7EA, 0x00C2E3E6, 0x00BEDFE2, 0x00ABCCCF, 0x00A5C6C9, 0x00A2C3C6, 0x00ABCCCF, 0x00BBDCDF, 0x00C9EAED, 0x00CDEEF1, 0x00CCEDF0, 0x00C4DFE3,
        0x00BDD8DC, 0x00BDD4D6, 0x00C3D8DA, 0x00D2E4E5, 0x00E2EFF1, 0xFBEAF6F8, 0xFFEEF8F8, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFDFEFC, 0xFFFFFFFE, 0x549C9D99, 0x000E0F0B, 0x000B0C08, 0x000B0D07,
        0x000B0D07, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000B0D07, 0x000C0E08, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080D0B, 0x000B0F0A, 0x00000300, 0x00020300, 0x0011110B, 0x00020200, 0x0024221A, 0x008B8981, 0x00776D66,
        0x00090100, 0x00302821, 0x00B1ABA4, 0x00E0DAD3, 0x0078756D, 0x00232119, 0x00040200, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000B0D07, 0x000C0E08, 0x000C0E08, 0x000B0D07, 0x000B0D07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06,
        0x00080C06, 0x00080C06, 0x00080C06, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09,
        0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x00070E07, 0x00030701, 0x005E5E58, 0x008A8782, 0x00080500, 0x000F0F09, 0x000C100A, 0x00030A03, 0x00414036, 0x00D6D5CB, 0x00DEDCD2, 0x00434137, 0x0079746B,
        0x00DAD5CC, 0x00D5CEC5, 0x00777067, 0x00727369, 0x0069685E, 0x0058554D, 0x00B7B1AA, 0x0068625D, 0x00040100, 0x0010110D, 0x00090F0A, 0x000E120D, 0x000F130E, 0x000E120D, 0x000E120D, 0x000E120D, 0x000F130E, 0x000E120D, 0x000D110C, 0x00635F5A,
        0x008E8A85, 0x00B2AFAA, 0x007B7873, 0x001A1712, 0x000F0F09, 0x009C9C96, 0x00696963, 0x000C0901, 0x000E0F06, 0x00181C16, 0x000D140D, 0x00030904, 0x000F130E, 0x00141511, 0x0012100F, 0x000C0F0D, 0x000F1210, 0x000A0F0D, 0x001B221F, 0x00323B38,
        0x00343F3C, 0x002C3A36, 0x0022302C, 0x0041514A, 0x00C7D0CD, 0x00E4EAE9, 0x0081928F, 0x00819E9B, 0x009FBFBE, 0x00C8DFE1, 0x00D8E4E8, 0x00DFEFF5, 0x00C5E0E4, 0x00B2D4D4, 0x0098B4B4, 0x00BCCAC9, 0x00EFF4F3, 0x00F0F5F4, 0x00E8F1EE, 0x00B6BFBC,
        0x00CCD2D1, 0x00F2FAF9, 0x00EBF5F5, 0x00D9E8EA, 0x00B7CED0, 0x00ACCBCE, 0x00B8DBDE, 0x00C6E7EA, 0x00CDEEF1, 0x00CEEFF2, 0x00C0E1E4, 0x00ACCDD0, 0x00A3C4C7, 0x00ABCCCF, 0x00B6D7DA, 0x00CBECEF, 0x00BEDFE2, 0x00AACBCE, 0x009CBDC0, 0x009BBCBF,
        0x00A8C9CC, 0x00BBDCDF, 0x00C8E9EC, 0x00C9EAED, 0x00C1E2E5, 0x00B4D5D8, 0x00A8C9CC, 0x00A3C4C7, 0x00A7C8CB, 0x00B2D3D6, 0x00BADBDE, 0x00CAEBEE, 0x00CBECEF, 0x00C8E9EC, 0x00BCDDE0, 0x00ADCED1, 0x00A5C6C9, 0x00A8C9CC, 0x00AECFD2, 0x00BFDADE,
        0x00CCE7EB, 0x00DDF4F6, 0x00E2F7F9, 0x00DFF1F2, 0xE2DEEBED, 0xEAE1EDEF, 0xF7E8F2F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFFFD, 0xFFFFFFFE, 0x79AFB0AC, 0x00161713, 0x000A0B07, 0x000A0C06,
        0x000B0D07, 0x000C0E08, 0x000C0E08, 0x000D0F09, 0x000D0F09, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000A0C06, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07,
        0x00080C07, 0x00080C07, 0x00080C07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00090C0A, 0x00050904, 0x000F130E, 0x000A0B07, 0x0010120C, 0x000D0F09, 0x00000300, 0x0025281F, 0x00AAA19D,
        0x00645E59, 0x00070100, 0x00221C17, 0x0097918C, 0x00D9D3CE, 0x00ABA7A2, 0x0028241F, 0x000D0F09, 0x000B0D07, 0x00090B05, 0x00090B05, 0x000A0C06, 0x000C0E08, 0x000D0F09, 0x000C0E08, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06,
        0x00080C06, 0x00080C06, 0x00080C06, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09,
        0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x00060D06, 0x00040802, 0x005C5C56, 0x0085827D, 0x00050200, 0x000F0F09, 0x000D110B, 0x00050C05, 0x003E3D33, 0x00D5D4CA, 0x00D0CEC4, 0x00413F35, 0x004C473E,
        0x006C675E, 0x00AFA89F, 0x00302920, 0x0073746A, 0x006D6C62, 0x00706D65, 0x00E5DFD8, 0x005F5954, 0x00110E09, 0x001B1C18, 0x00080E09, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000E120D, 0x000F130E, 0x006C6863,
        0x00DDD9D4, 0x00B9B6B1, 0x00201D18, 0x0013100B, 0x0013130D, 0x00A5A59F, 0x00F9F9F3, 0x004E4841, 0x000E0B03, 0x000F110B, 0x00121912, 0x00030904, 0x000E140F, 0x00141511, 0x000F0D0C, 0x000C0F0D, 0x000E110F, 0x000B100E, 0x001C2320, 0x00313A37,
        0x00343F3C, 0x002B3935, 0x001F2D29, 0x00405049, 0x00E3ECE9, 0x00DDE3E2, 0x00869794, 0x00ABC8C5, 0x00B5D5D4, 0x00B9D0D2, 0x00D4E0E4, 0x00C7D7DD, 0x00C6E1E5, 0x00CDEFEF, 0x00AFCBCB, 0x00CBD9D8, 0x00EBF0EF, 0x00EDF2F1, 0x00EDF6F3, 0x00B3BCB9,
        0x00DBE1E0, 0x00F1F9F8, 0x00E0EAEA, 0x00DFEEF0, 0x00DAF1F3, 0x00C9E8EB, 0x00B3D6D9, 0x00A7C8CB, 0x00A3C4C7, 0x00A5C6C9, 0x00B4D5D8, 0x00C5E6E9, 0x00C9EAED, 0x00BCDDE0, 0x00ADCED1, 0x0093B4B7, 0x009FC0C3, 0x00B1D2D5, 0x00BFE0E3, 0x00C2E3E6,
        0x00B9DADD, 0x00ABCCCF, 0x00A1C2C5, 0x00A1C2C5, 0x00ACCDD0, 0x00BBDCDF, 0x00C6E7EA, 0x00C8E9EC, 0x00C2E3E6, 0x00BADBDE, 0x00B4D5D8, 0x00A5C6C9, 0x00A5C6C9, 0x00AACBCE, 0x00B6D7DA, 0x00C5E6E9, 0x00CBECEF, 0x00C6E7EA, 0x00BEDFE2, 0x00BDD8DC,
        0x00B8D3D7, 0x00BBD2D4, 0x00C5DADC, 0x00D6E8E9, 0xF2E6F3F5, 0xFFECF8FA, 0xFFEDF7F7, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFFFD, 0xFFFFFFFE, 0xA3C2C3BF, 0x00232420, 0x00080905, 0x000A0C06,
        0x000B0D07, 0x000C0E08, 0x000D0F09, 0x000D0F09, 0x000D0F09, 0x000D0F09, 0x000D0F09, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08,
        0x00090D08, 0x00090D08, 0x00090D08, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x000B0C0A, 0x00080905, 0x000D0E0A, 0x00080C07, 0x00000400, 0x00090D07, 0x000D1209, 0x00000300, 0x003B3833,
        0x00A6A39E, 0x00282520, 0x00070300, 0x00100C07, 0x006C6863, 0x00C2BEB9, 0x00B9B5B0, 0x0012140E, 0x000E100A, 0x000A0C06, 0x00080A04, 0x00090B05, 0x000B0D07, 0x000B0D07, 0x000A0C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06,
        0x00080C06, 0x00080C06, 0x00080C06, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09,
        0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C130C, 0x00030701, 0x00595953, 0x008C8984, 0x0015120D, 0x00181812, 0x000A0E08, 0x00000500, 0x0038372D, 0x00D4D3C9, 0x00DBD9CF, 0x00444238, 0x00979289,
        0x00AEA9A0, 0x00D1CAC1, 0x00554E45, 0x00727369, 0x005D5C52, 0x00413E36, 0x00BDB7B0, 0x00433D38, 0x00080500, 0x000C0D09, 0x000C120D, 0x000E120D, 0x000D110C, 0x000E120D, 0x000F130E, 0x000E120D, 0x000D110C, 0x000E120D, 0x00111510, 0x0068645F,
        0x0038342F, 0x009E9B96, 0x00696661, 0x00100D08, 0x000C0C06, 0x009C9C96, 0x00FDFDF7, 0x00DDD3CC, 0x004A443D, 0x00050500, 0x00121610, 0x00151B16, 0x000C120D, 0x000B0F0A, 0x00131412, 0x000D100E, 0x000D100E, 0x000B100E, 0x001E2522, 0x00313A37,
        0x00333E3B, 0x002B3935, 0x001C2A26, 0x004E5E57, 0x00DCE5E2, 0x00D7DDDC, 0x00869794, 0x0088A5A2, 0x009EBEBD, 0x00C7DEE0, 0x00DEEAEE, 0x00D9E9EF, 0x00C4DFE3, 0x00B6D8D8, 0x0091ADAD, 0x00C3D1D0, 0x00EBF0EF, 0x00F0F5F4, 0x00ECF5F2, 0x00AFB8B5,
        0x00D7DDDC, 0x00EDF5F4, 0x00E9F3F3, 0x00D7E6E8, 0x00B4CBCD, 0x00ABCACD, 0x00B6D9DC, 0x00C8E9EC, 0x00CDEEF1, 0x00CBECEF, 0x00BBDCDF, 0x00A7C8CB, 0x009FC0C3, 0x00A8C9CC, 0x00B5D6D9, 0x00C9EAED, 0x00BADBDE, 0x00A8C9CC, 0x009DBEC1, 0x00A0C1C4,
        0x00ADCED1, 0x00BBDCDF, 0x00C4E5E8, 0x00C4E5E8, 0x00BBDCDF, 0x00AFD0D3, 0x00A5C6C9, 0x00A3C4C7, 0x00ABCCCF, 0x00B8D9DC, 0x00C1E2E5, 0x00CAEBEE, 0x00C9EAED, 0x00C2E3E6, 0x00B3D4D7, 0x00A6C7CA, 0x00A3C4C7, 0x00ADCED1, 0x00B7D8DB, 0x00CCE7EB,
        0x00D2EDF1, 0x00DBF2F4, 0x00D9EEF0, 0x00D6E8E9, 0xD9DAE7E9, 0xEEE3EFF1, 0xFFEEF8F8, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFFFD, 0xFFFFFFFE, 0xC9D4D5D1, 0x00353632, 0x00060703, 0x000A0C06,
        0x000B0D07, 0x000C0E08, 0x000D0F09, 0x000D0F09, 0x000E100A, 0x000E100A, 0x000D0F09, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08,
        0x00090D08, 0x00090D08, 0x00090D08, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00090A08, 0x000C0D09, 0x00050602, 0x00080C07, 0x00090D07, 0x00050C05, 0x000C140A, 0x00050D03, 0x00090A06,
        0x003D3E3A, 0x003B3C38, 0x00090804, 0x000E0D09, 0x000C0905, 0x002F2C28, 0x0055524E, 0x0011130D, 0x000E100A, 0x000A0C06, 0x00090B05, 0x000B0D07, 0x000C0E08, 0x000B0D07, 0x00090B05, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06,
        0x00080C06, 0x00080C06, 0x00080C06, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09,
        0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000B120B, 0x00000300, 0x00565650, 0x008B8883, 0x00090601, 0x00020200, 0x00030701, 0x00151C15, 0x0097968C, 0x00D9D8CE, 0x00CECCC2, 0x00454339, 0x00908B82,
        0x00CAC5BC, 0x00AFA89F, 0x00726B62, 0x0078796F, 0x0068675D, 0x006E6B63, 0x00E6E0D9, 0x0056504B, 0x000C0904, 0x00171814, 0x00080E09, 0x000E120D, 0x000D110C, 0x000D110C, 0x000F130E, 0x000E120D, 0x000C100B, 0x000E120D, 0x00121611, 0x00A7A39E,
        0x007D7974, 0x004A4742, 0x0055524D, 0x00100D08, 0x0011110B, 0x00A7A7A1, 0x00F7F7F1, 0x00FFFFF9, 0x00C0B8B1, 0x003E3B36, 0x00080A04, 0x000A100B, 0x000A100B, 0x00141813, 0x00101311, 0x000D100E, 0x000D100E, 0x000C110F, 0x001F2623, 0x00313A37,
        0x00333E3B, 0x002B3935, 0x001B2925, 0x004F5F58, 0x00E3ECE9, 0x00CCD2D1, 0x00859693, 0x00A5C2BF, 0x00BADAD9, 0x00C1D8DA, 0x00CDD9DD, 0x00CADAE0, 0x00C4DFE3, 0x00C8EAEA, 0x00A4C0C0, 0x00D4E2E1, 0x00EDF2F1, 0x00EBF0EF, 0x00E6EFEC, 0x00AEB7B4,
        0x00E2E8E7, 0x00EDF5F4, 0x00E3EDED, 0x00DEEDEF, 0x00D2E9EB, 0x00C5E4E7, 0x00BADDE0, 0x00A9CACD, 0x00A8C9CC, 0x00ACCDD0, 0x00BADBDE, 0x00C7E8EB, 0x00C7E8EB, 0x00B7D8DB, 0x00A7C8CB, 0x0092B3B6, 0x00A3C4C7, 0x00B9DADD, 0x00C4E5E8, 0x00C1E2E5,
        0x00B5D6D9, 0x00A9CACD, 0x00A3C4C7, 0x00A4C5C8, 0x00AECFD2, 0x00BCDDE0, 0x00C7E8EB, 0x00C8E9EC, 0x00C0E1E4, 0x00B2D3D6, 0x00A9CACD, 0x00A1C2C5, 0x00A6C7CA, 0x00B2D3D6, 0x00BFE0E3, 0x00C9EAED, 0x00CAEBEE, 0x00C5E6E9, 0x00BFE0E3, 0x00B0CBCF,
        0x00B3CED2, 0x00BED5D7, 0x00CCE1E3, 0x00DCEEEF, 0xFBEAF7F9, 0xFFEDF9FB, 0xFFEFF9F9, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBFBFB, 0xFFFEFFFD, 0xFFFFFFFE, 0xE2E1E2DE, 0x0041423E, 0x00050602, 0x000A0C06,
        0x000B0D07, 0x000C0E08, 0x000D0F09, 0x000D0F09, 0x000E100A, 0x000E100A, 0x000E100A, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08,
        0x00090D08, 0x00090D08, 0x00090D08, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x000C0D0B, 0x000A0B07, 0x00090A06, 0x00070B06, 0x00091009, 0x000A110A, 0x00050E04, 0x00060F05, 0x00010702,
        0x00000400, 0x000D110C, 0x00050904, 0x000C0D09, 0x000E0D09, 0x000B0A06, 0x00040300, 0x000E100A, 0x000B0D07, 0x00090B05, 0x000A0C06, 0x000D0F09, 0x000E100A, 0x000D0F09, 0x000B0D07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06,
        0x00080C06, 0x00080C06, 0x00080C06, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09,
        0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x00091009, 0x00010500, 0x005C5C56, 0x008B8883, 0x00050200, 0x001C1C16, 0x006D716B, 0x00CBD2CB, 0x00D3D2C8, 0x006A695F, 0x009D9B91, 0x00B2B0A6, 0x003B362D,
        0x00656057, 0x007C756C, 0x00463F36, 0x0073746A, 0x0065645A, 0x003F3C34, 0x00948E87, 0x004A443F, 0x000C0904, 0x000F100C, 0x00090F0A, 0x000E120D, 0x000D110C, 0x000D110C, 0x000F130E, 0x000E120D, 0x000C100B, 0x000E120D, 0x00131712, 0x009B9792,
        0x00706C67, 0x00989590, 0x005E5B56, 0x00191611, 0x0012120C, 0x00A8A8A2, 0x00F6F6F0, 0x00FCF0EA, 0x00FDF3EC, 0x005D5954, 0x00050701, 0x00111712, 0x00080E09, 0x00111510, 0x00090C0A, 0x000E110F, 0x000D100E, 0x000D1210, 0x00202724, 0x00313A37,
        0x00333E3B, 0x002B3935, 0x001A2824, 0x00576760, 0x00E4EDEA, 0x00CFD5D4, 0x007E8F8C, 0x0085A29F, 0x00A8C8C7, 0x00CBE2E4, 0x00DDE9ED, 0x00DDEDF3, 0x00C7E2E6, 0x00B9DBDB, 0x0090ACAC, 0x00CCDAD9, 0x00EDF2F1, 0x00EEF3F2, 0x00E7F0ED, 0x00AEB7B4,
        0x00E1E7E6, 0x00EAF2F1, 0x00E3EDED, 0x00D4E3E5, 0x00B9D0D2, 0x00AFCED1, 0x00B1D4D7, 0x00C8E9EC, 0x00CBECEF, 0x00C7E8EB, 0x00B8D9DC, 0x00A6C7CA, 0x00A1C2C5, 0x00ABCCCF, 0x00B8D9DC, 0x00CBECEF, 0x00B7D8DB, 0x00A0C1C4, 0x0096B7BA, 0x009EBFC2,
        0x00AFD0D3, 0x00BDDEE1, 0x00C4E5E8, 0x00C7E8EB, 0x00BBDCDF, 0x00ABCCCF, 0x00A2C3C6, 0x00A4C5C8, 0x00AFD0D3, 0x00BEDFE2, 0x00C7E8EB, 0x00CEEFF2, 0x00C6E7EA, 0x00B7D8DB, 0x00A8C9CC, 0x00A0C1C4, 0x00A4C5C8, 0x00B0D1D4, 0x00BADBDE, 0x00D7F2F6,
        0x00D7F2F6, 0x00D8EFF1, 0x00D5EAEC, 0xD1D5E7E8, 0xDEDBE8EA, 0xEAE2EEF0, 0xFBE9F3F3, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFDFDFD, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFEEEEEE, 0x26848484, 0x00020202, 0x000F100C,
        0x00090A06, 0x00060703, 0x000A0B07, 0x000D0E0A, 0x000B0C08, 0x000A0B07, 0x000C0D09, 0x000F110B, 0x000C0E08, 0x000C0E08, 0x000E100A, 0x000D0F09, 0x000A0C06, 0x000B0D07, 0x000F110B, 0x00060A05, 0x00080C07, 0x00090D08, 0x000A0E09, 0x000A0E09,
        0x000A0E09, 0x000A0E09, 0x000B0F0A, 0x00090D08, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x00090D08, 0x00090D08, 0x00080C07, 0x00080C07, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0D07,
        0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06,
        0x00080C06, 0x00080C06, 0x00080C06, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000C0D09,
        0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0F0D, 0x00080905, 0x00565753, 0x00767571, 0x004B4843, 0x00CAC6C1, 0x00E0DAD3, 0x0078726B, 0x00161610, 0x000D0D07, 0x000F0F09, 0x00A7A7A1, 0x00AFAFA9,
        0x000F0F09, 0x00080802, 0x00030300, 0x00818076, 0x006E6C62, 0x002D2720, 0x00625C55, 0x00312D28, 0x00161610, 0x00090D08, 0x000C130E, 0x000D110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x00706F6B,
        0x004F4E4A, 0x001B1A16, 0x00181713, 0x00161511, 0x000B0A06, 0x00B5B4B0, 0x00FFFFFB, 0x00ACADA9, 0x003D3E3A, 0x00090A06, 0x000F130E, 0x000F130E, 0x000C120D, 0x000C120D, 0x000E140F, 0x000B100E, 0x000A0F0D, 0x000C1310, 0x00222B28, 0x00323D3A,
        0x002E3C38, 0x00293733, 0x001E2D29, 0x003B4B44, 0x00D3DCD9, 0x00D3D9D8, 0x00879895, 0x00AFCCC9, 0x00B6D6D5, 0x00C0D7D9, 0x00D2DEE2, 0x00C3D9DE, 0x00C4E1E5, 0x00CDECED, 0x00AAC2C2, 0x00E0E8E8, 0x00ECECEC, 0x00F3F5F5, 0x00E5EEEB, 0x00ACB6B0,
        0x00E6EFEC, 0x00E7F2F0, 0x00DFEBEB, 0x00DAECED, 0x00D5EEF2, 0x00C1E1E6, 0x00B5DADE, 0x00A7C8CB, 0x00A6C7CA, 0x00ACCDD0, 0x00BBDCDF, 0x00C8E9EC, 0x00C5E6E9, 0x00B1D2D5, 0x009FC0C3, 0x0096B7BA, 0x00A4C5C8, 0x00B6D7DA, 0x00BFE0E3, 0x00BBDCDF,
        0x00AECFD2, 0x00A2C3C6, 0x009ABBBE, 0x00A5C6C9, 0x00B2D3D6, 0x00C3E4E7, 0x00CBECEF, 0x00C6E7EA, 0x00B9DADD, 0x00ACCDD0, 0x00A5C6C9, 0x00A4C5C8, 0x00A9CACD, 0x00B7D6D9, 0x00C6E5E8, 0x00D0EFF2, 0x00CEEBEF, 0x00C0DDE1, 0x00B4D1D5, 0x00ADC8CC,
        0x00B5CED2, 0x00C4D9DB, 0x00D3E6E9, 0xEEE3F2F4, 0xFFEDF9FB, 0xFFF0F9FC, 0xFFF1F9F9, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xBCCCCCCC, 0x21818181, 0x0032332F,
        0x00252622, 0x00151612, 0x000B0C08, 0x00070804, 0x00080905, 0x000C0D09, 0x000F100C, 0x000D0F09, 0x000A0C06, 0x000A0C06, 0x000D0F09, 0x000D0F09, 0x000A0C06, 0x00090B05, 0x000B0D07, 0x00070B06, 0x00080C07, 0x00090D08, 0x00090D08, 0x00090D08,
        0x00090D08, 0x00080C07, 0x00080C07, 0x00080C07, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x000A0E09, 0x000B0F0A, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0D07,
        0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06,
        0x00080C06, 0x00080C06, 0x00080C06, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000C0D09,
        0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x00080B09, 0x00000100, 0x0050514D, 0x00868581, 0x00393631, 0x007E7A75, 0x00130D06, 0x000E0801, 0x004D4A45, 0x00C0BDB8, 0x009A9792, 0x000D0A05, 0x00A4A19C,
        0x00BBB8B3, 0x00221F1A, 0x00090601, 0x007F7E74, 0x005C5A50, 0x0067615A, 0x00D2CCC5, 0x004D4944, 0x00060600, 0x000A0E09, 0x000D140F, 0x000D110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x0051504C,
        0x007E7D79, 0x009F9E9A, 0x0053524E, 0x00060501, 0x001D1C18, 0x00989793, 0x006F6E6A, 0x00191A16, 0x00020300, 0x00090A06, 0x00111510, 0x000F130E, 0x000E140F, 0x000D130E, 0x00101611, 0x000B100E, 0x000A0F0D, 0x000C1310, 0x00232C29, 0x00323D3A,
        0x002E3C38, 0x00293733, 0x001E2D29, 0x0033433C, 0x00CAD3D0, 0x00CCD2D1, 0x007A8B88, 0x008EABA8, 0x009EBEBD, 0x00C5DCDE, 0x00E0ECF0, 0x00D2E8ED, 0x00BFDCE0, 0x00B4D3D4, 0x0092AAAA, 0x00D9E1E1, 0x00EFEFEF, 0x00F4F6F6, 0x00DBE4E1, 0x00A8B2AC,
        0x00E5EEEB, 0x00EDF8F6, 0x00E4F0F0, 0x00CBDDDE, 0x00B4CDD1, 0x00AACACF, 0x00B3D8DC, 0x00C5E6E9, 0x00C6E7EA, 0x00C1E2E5, 0x00B3D4D7, 0x00A5C6C9, 0x00A4C5C8, 0x00B2D3D6, 0x00C1E2E5, 0x00C6E7EA, 0x00B7D8DB, 0x00A4C5C8, 0x009CBDC0, 0x00A3C4C7,
        0x00B2D3D6, 0x00C0E1E4, 0x00C8E9EC, 0x00BDDEE1, 0x00B2D3D6, 0x00A4C5C8, 0x00A0C1C4, 0x00A8C9CC, 0x00B6D7DA, 0x00C3E4E7, 0x00CAEBEE, 0x00CBECEF, 0x00C4E5E8, 0x00B8D7DA, 0x00A7C6C9, 0x009FBEC1, 0x00A7C4C8, 0x00BCD9DD, 0x00CEEBEF, 0x00DAF5F9,
        0x00D6EFF3, 0x00D2E7E9, 0x00CEE1E4, 0xCDD4E3E5, 0xE2DDE9EB, 0xF7E7F0F3, 0xFFEEF6F6, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFEFEFE, 0xFBE9E9E9, 0xB8CDCECA,
        0x86B4B5B1, 0x2E8B8C88, 0x0060615D, 0x003E3F3B, 0x00262723, 0x00141511, 0x00080905, 0x00090A06, 0x00070804, 0x00070804, 0x000C0D09, 0x000F100C, 0x000E0F0B, 0x000C0D09, 0x000B0C08, 0x000D110C, 0x000C100B, 0x000B0F0A, 0x000A0E09, 0x000A0E09,
        0x000A0E09, 0x000A0E09, 0x000A0E09, 0x00090D08, 0x000A0E09, 0x000A0E09, 0x00090D08, 0x00080C07, 0x00080C07, 0x000A0E09, 0x000C100B, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0D07,
        0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06,
        0x00080C06, 0x00080C06, 0x00080C06, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x000C0D09,
        0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x00131614, 0x00040501, 0x003F403C, 0x0092918D, 0x00040100, 0x00050100, 0x004B453E, 0x00B6B0A9, 0x00E0DDD8, 0x0096938E, 0x00ACA9A4, 0x00A09D98, 0x00181510,
        0x00A29F9A, 0x00B6B3AE, 0x0017140F, 0x00737268, 0x00626056, 0x00716B64, 0x00DED8D1, 0x0046423D, 0x00090903, 0x000A0E09, 0x000D140F, 0x000D110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x00020100,
        0x00050400, 0x0051504C, 0x0094938F, 0x0014130F, 0x00070602, 0x00161511, 0x000F0E0A, 0x00070804, 0x00141511, 0x001B1C18, 0x000D110C, 0x00070B06, 0x00080E09, 0x00050B06, 0x00090F0A, 0x000B100E, 0x000A0F0D, 0x000D1411, 0x00232C29, 0x00323D3A,
        0x002E3C38, 0x00283632, 0x001D2C28, 0x002E3E37, 0x00D2DBD8, 0x00C1C7C6, 0x007C8D8A, 0x00B0CDCA, 0x00BBDBDA, 0x00C0D7D9, 0x00D2DEE2, 0x00C6DCE1, 0x00C4E1E5, 0x00C5E4E5, 0x00A5BDBD, 0x00E0E8E8, 0x00ECECEC, 0x00F3F5F5, 0x00E0E9E6, 0x00ACB6B0,
        0x00EDF6F3, 0x00EBF6F4, 0x00DDE9E9, 0x00D4E6E7, 0x00D4EDF1, 0x00C6E6EB, 0x00B5DADE, 0x00AACBCE, 0x00A7C8CB, 0x00ABCCCF, 0x00B9DADD, 0x00C7E8EB, 0x00C3E4E7, 0x00AFD0D3, 0x009BBCBF, 0x0095B6B9, 0x00A7C8CB, 0x00BBDCDF, 0x00C0E1E4, 0x00B5D6D9,
        0x00A6C7CA, 0x009EBFC2, 0x009CBDC0, 0x00B3D4D7, 0x00BEDFE2, 0x00C9EAED, 0x00CAEBEE, 0x00C0E1E4, 0x00B1D2D5, 0x00A6C7CA, 0x00A1C2C5, 0x00A4C5C8, 0x00ADCED1, 0x00BEDDE0, 0x00CBEAED, 0x00CEEDF0, 0x00C7E4E8, 0x00B8D5D9, 0x00ACC9CD, 0x00AFCACE,
        0x00BBD4D8, 0x00CEE3E5, 0x00DCEFF2, 0x00E8F7F9, 0x00ECF8FA, 0xFFEBF4F7, 0xFBEAF2F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFBFBFB, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFF9FAF8,
        0xFFFFFFFE, 0xFFFFFFFE, 0xFFF5F6F4, 0xDEDCDDDB, 0x8AB4B5B3, 0x26848583, 0x0060615F, 0x00363733, 0x00282925, 0x00171814, 0x000B0C08, 0x00070804, 0x00090A06, 0x000B0C08, 0x000C0D09, 0x00070B06, 0x00070B06, 0x00080C07, 0x00090D08, 0x00090D08,
        0x00090D08, 0x00080C07, 0x00060A05, 0x000A0E09, 0x000B0F0A, 0x000C100B, 0x000B0F0A, 0x00080C07, 0x00070B06, 0x00080C07, 0x000A0E09, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0D07,
        0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06,
        0x00080C06, 0x00080C06, 0x00080C06, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x000C0D09,
        0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x00030604, 0x00090A06, 0x0050514D, 0x007E7D79, 0x0027241F, 0x00ABA7A2, 0x00E0DAD3, 0x00A29C95, 0x00302D28, 0x00040100, 0x0023201B, 0x00B2AFAA, 0x00928F8A,
        0x00181510, 0x009C9994, 0x00706D68, 0x00706F65, 0x006D6B61, 0x0058524B, 0x00AFA9A2, 0x002B2722, 0x0013130D, 0x000C100B, 0x0009100B, 0x000D110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x00151612,
        0x00090A06, 0x002D2E2A, 0x003B3C38, 0x00070804, 0x00141511, 0x00181915, 0x00080905, 0x00141511, 0x00151612, 0x000F100C, 0x00090D08, 0x000E120D, 0x000F1510, 0x000C120D, 0x00101611, 0x000B100E, 0x000A0F0D, 0x000D1411, 0x00242D2A, 0x00333E3B,
        0x002E3C38, 0x00283632, 0x001C2B27, 0x003F4F48, 0x00D1DAD7, 0x00C4CAC9, 0x00829390, 0x008FACA9, 0x00A8C8C7, 0x00D0E7E9, 0x00DBE7EB, 0x00D1E7EC, 0x00C1DEE2, 0x00B3D2D3, 0x0096AEAE, 0x00DDE5E5, 0x00F0F0F0, 0x00F4F6F6, 0x00D6DFDC, 0x00AFB9B3,
        0x00E5EEEB, 0x00E5F0EE, 0x00E6F2F2, 0x00CFE1E2, 0x00B1CACE, 0x00A9C9CE, 0x00B4D9DD, 0x00C5E6E9, 0x00C7E8EB, 0x00C2E3E6, 0x00B0D1D4, 0x009FC0C3, 0x009DBEC1, 0x00AECFD2, 0x00BFE0E3, 0x00C7E8EB, 0x00B1D2D5, 0x0099BABD, 0x0095B6B9, 0x00A6C7CA,
        0x00BADBDE, 0x00C5E6E9, 0x00C7E8EB, 0x00B4D5D8, 0x00A9CACD, 0x009FC0C3, 0x00A2C3C6, 0x00B0D1D4, 0x00C1E2E5, 0x00CAEBEE, 0x00CDEEF1, 0x00C2E3E6, 0x00B8D9DC, 0x00ACCBCE, 0x00A2C1C4, 0x00A2C1C4, 0x00B0CDD1, 0x00C2DFE3, 0x00CFECF0, 0x00D2EDF1,
        0x00D0E9ED, 0x00CEE3E5, 0x00CBDEE1, 0x00D0DFE1, 0x00DCE8EA, 0xFBEAF3F6, 0xFFF6FEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFFFFFE,
        0xFFFFFFFE, 0xFFFFFFFE, 0xFFFCFDFB, 0xFFFEFFFD, 0xFFFFFFFE, 0xFFFEFFFD, 0xFFF7F8F6, 0xD9DCDDD9, 0x92BABBB7, 0x21858682, 0x0051524E, 0x002D2E2A, 0x00181915, 0x000E0F0B, 0x00090A06, 0x000D110C, 0x000A0E09, 0x00080C07, 0x00070B06, 0x00090D08,
        0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x00080C07, 0x000A0E09, 0x000C100B, 0x000C100B, 0x000A0E09, 0x00090D08, 0x00090D08, 0x00090D08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0D07,
        0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06,
        0x00080C06, 0x00080C06, 0x00080C06, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x000C0D09,
        0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000D100E, 0x00090A06, 0x003A3B37, 0x0084837F, 0x00413E39, 0x009B9792, 0x0048423B, 0x00080200, 0x002C2823, 0x00A7A39E, 0x00706C67, 0x001B1712, 0x00BDB9B4,
        0x008F8B86, 0x0014100B, 0x00110D08, 0x0079786E, 0x00636157, 0x005C564F, 0x00C7C1BA, 0x002E2A25, 0x000E0E08, 0x000D110C, 0x00080F0A, 0x000D110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x000D110C, 0x0012130F,
        0x000D0E0A, 0x000F100C, 0x000B0C08, 0x000D0E0A, 0x000A0B07, 0x000F100C, 0x00131410, 0x0011120E, 0x000B0C08, 0x000A0B07, 0x000F130E, 0x0010140F, 0x00090F0A, 0x000A100B, 0x000B110C, 0x000B100E, 0x000A0F0D, 0x000E1512, 0x00242D2A, 0x00333E3B,
        0x002E3C38, 0x00273531, 0x001B2A26, 0x003B4B44, 0x00E4EDEA, 0x00BFC5C4, 0x00879895, 0x00B3D0CD, 0x00BDDDDC, 0x00C0D7D9, 0x00D2DEE2, 0x00C6DCE1, 0x00C4E1E5, 0x00C0DFE0, 0x00A4BCBC, 0x00E2EAEA, 0x00ECECEC, 0x00F3F5F5, 0x00D9E2DF, 0x00AEB8B2,
        0x00F0F9F6, 0x00E9F4F2, 0x00DDE9E9, 0x00D6E8E9, 0x00D1EAEE, 0x00C6E6EB, 0x00B1D6DA, 0x00ABCCCF, 0x00A8C9CC, 0x00ACCDD0, 0x00BADBDE, 0x00C8E9EC, 0x00C6E7EA, 0x00B1D2D5, 0x009DBEC1, 0x0094B5B8, 0x00A9CACD, 0x00BEDFE2, 0x00C1E2E5, 0x00B2D3D6,
        0x00A2C3C6, 0x009CBDC0, 0x009EBFC2, 0x00B1D2D5, 0x00BFE0E3, 0x00CCEDF0, 0x00C8E9EC, 0x00B8D9DC, 0x00A7C8CB, 0x00A0C1C4, 0x00A1C2C5, 0x00ADCED1, 0x00B7D8DB, 0x00C7E6E9, 0x00CEEDF0, 0x00CBEAED, 0x00C0DDE1, 0x00B2CFD3, 0x00A9C6CA, 0x00B3CED2,
        0x00BFD8DC, 0x00D2E7E9, 0x00DFF2F5, 0x00E8F7F9, 0xFBEAF6F8, 0xFBE9F2F5, 0xF7E8F0F0, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFF9F9F9, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFF1F2F0, 0xC4D1D2D0, 0x60A1A2A0, 0x006B6C6A, 0x00464745, 0x001B1F1A, 0x00151914, 0x000D110C, 0x00080C07, 0x00070B06,
        0x00080C07, 0x00090D08, 0x00090D08, 0x00080C07, 0x00080C07, 0x00090D08, 0x000A0E09, 0x000B0F0A, 0x000C100B, 0x000C100B, 0x000B0F0A, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0D07,
        0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06,
        0x00080C06, 0x00080C06, 0x00080C06, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x000C0D09,
        0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x00080B09, 0x00070804, 0x00252622, 0x00A09F9B, 0x007B7873, 0x003B3732, 0x000F0902, 0x002E2821, 0x00A4A09B, 0x00E9E5E0, 0x00EDE9E4, 0x007E7A75, 0x00231F1A,
        0x00BCB8B3, 0x00908C87, 0x000B0702, 0x00727167, 0x005F5D53, 0x00655F58, 0x00D8D2CB, 0x003D3934, 0x000A0A04, 0x00090D08, 0x000D140F, 0x000C100B, 0x000C100B, 0x000C100B, 0x000C100B, 0x000C100B, 0x000C100B, 0x000C100B, 0x000C100B, 0x00080C07,
        0x00161A15, 0x000A0E09, 0x00141813, 0x000F130E, 0x000C100B, 0x000A0E09, 0x00141813, 0x00141511, 0x000F100C, 0x000D0E0A, 0x000F130E, 0x000B0F0A, 0x00080E09, 0x00101611, 0x000C120D, 0x000B100E, 0x000B100E, 0x000E1512, 0x00252E2B, 0x00343F3C,
        0x002E3C38, 0x00273531, 0x001B2A26, 0x004B5B54, 0x00DCE5E2, 0x00B8BEBD, 0x00879895, 0x0090ADAA, 0x00A7C7C6, 0x00CEE5E7, 0x00DDE9ED, 0x00CFE5EA, 0x00C1DEE2, 0x00AECDCE, 0x0096AEAE, 0x00E0E8E8, 0x00F0F0F0, 0x00F4F6F6, 0x00CED7D4, 0x00B3BDB7,
        0x00E8F1EE, 0x00E2EDEB, 0x00E5F1F1, 0x00D0E2E3, 0x00AEC7CB, 0x00AACACF, 0x00B1D6DA, 0x00C6E7EA, 0x00C8E9EC, 0x00C3E4E7, 0x00B2D3D6, 0x00A0C1C4, 0x009FC0C3, 0x00B0D1D4, 0x00C2E3E6, 0x00C4E5E8, 0x00ADCED1, 0x0097B8BB, 0x0097B8BB, 0x00AACBCE,
        0x00BEDFE2, 0x00C5E6E9, 0x00C2E3E6, 0x00ABCCCF, 0x00A7C8CB, 0x00A4C5C8, 0x00AACBCE, 0x00B6D7DA, 0x00C4E5E8, 0x00CDEEF1, 0x00D1F2F5, 0x00BCDDE0, 0x00B2D3D6, 0x00A7C6C9, 0x00A3C2C5, 0x00AAC9CC, 0x00BBD8DC, 0x00CCE9ED, 0x00D7F4F8, 0x00CEE9ED,
        0x00CCE5E9, 0x00CBE0E2, 0x00C9DCDF, 0x00D1E0E2, 0xE2DEEAEC, 0xFFEDF6F9, 0xFFF7FFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFBFCFA, 0xFFF1F2F0, 0xF2E7E8E6, 0x8AB5B9B4, 0x4B979B96, 0x00676B66, 0x003A3E39, 0x001A1E19,
        0x000A0E09, 0x00050904, 0x00050904, 0x000E120D, 0x000B0F0A, 0x00080C07, 0x00080C07, 0x00090D08, 0x000B0F0A, 0x000B0F0A, 0x000A0E09, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0D07,
        0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06,
        0x00080C06, 0x00080C06, 0x00080C06, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x000C0D09,
        0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000E110F, 0x000A0B07, 0x000E0F0B, 0x00282723, 0x0073706B, 0x009E9A95, 0x00AEA8A1, 0x00958F88, 0x007D7772, 0x00514B46, 0x006B6560, 0x0096908B, 0x0036302B,
        0x0019130E, 0x00C9C3BE, 0x00635D58, 0x0069685E, 0x00656359, 0x005E5851, 0x00BAB4AD, 0x0038342F, 0x000E0E08, 0x000A0E09, 0x000C130E, 0x000C100B, 0x000C100B, 0x000C100B, 0x000C100B, 0x000C100B, 0x000C100B, 0x000C100B, 0x000C100B, 0x000B0F0A,
        0x000F130E, 0x00141813, 0x00040803, 0x00131712, 0x00050904, 0x00141813, 0x00060A05, 0x000F100C, 0x0010110D, 0x000C0D09, 0x000D110C, 0x000D110C, 0x000B110C, 0x00111712, 0x00060C07, 0x000B100E, 0x000B100E, 0x000E1512, 0x00252E2B, 0x00343F3C,
        0x002E3C38, 0x00263430, 0x001A2925, 0x0053635C, 0x00E7F0ED, 0x00ABB1B0, 0x00899A97, 0x00B3D0CD, 0x00C0E0DF, 0x00C6DDDF, 0x00D1DDE1, 0x00C2D8DD, 0x00C4E1E5, 0x00C0DFE0, 0x00ABC3C3, 0x00E9F1F1, 0x00EEEEEE, 0x00F3F5F5, 0x00D1DAD7, 0x00B9C3BD,
        0x00F1FAF7, 0x00E0EBE9, 0x00DCE8E8, 0x00D7E9EA, 0x00CDE6EA, 0x00C6E6EB, 0x00B4D9DD, 0x00ABCCCF, 0x00A9CACD, 0x00ACCDD0, 0x00B8D9DC, 0x00C2E3E6, 0x00BFE0E3, 0x00ADCED1, 0x009CBDC0, 0x009ABBBE, 0x00AECFD2, 0x00C0E1E4, 0x00BDDEE1, 0x00AACBCE,
        0x009ABBBE, 0x009ABBBE, 0x00A1C2C5, 0x00C1E2E5, 0x00C2E3E6, 0x00C0E1E4, 0x00B9DADD, 0x00AFD0D3, 0x00A6C7CA, 0x00A2C3C6, 0x00A1C2C5, 0x00B8D9DC, 0x00C3E4E7, 0x00CFEEF1, 0x00CDECEF, 0x00C1E0E3, 0x00B3D0D4, 0x00AAC7CB, 0x00A7C4C8, 0x00BED9DD,
        0x00C7E0E4, 0x00D5EAEC, 0x00E0F3F6, 0x00EAF9FB, 0xFFECF8FA, 0xFBE9F2F5, 0xF2E6EEEE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFCFCFC, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFFFFFF,
        0xFFFDFDFD, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBFBFB, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFEFFFD, 0xFFFEFFFD, 0xFFFCFDFB, 0xFFFBFCFA, 0xFFFEFFFD, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFEFFFD, 0xFFFDFFFC, 0xFFFCFFFB, 0xFFF1F5F0, 0xD9DBDFDA, 0x8EB7BBB6,
        0x2E888C87, 0x005A5E59, 0x003C403B, 0x00161A15, 0x0010140F, 0x00090D08, 0x00060A05, 0x00070B06, 0x00080C07, 0x00080C07, 0x00060A05, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000B0D07,
        0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06, 0x00080C06,
        0x00080C06, 0x00080C06, 0x00080C06, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00090D07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x00080C07, 0x000C0D09,
        0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x00080B09, 0x0011120E, 0x00090A06, 0x000A0905, 0x000B0803, 0x00080400, 0x00201A13, 0x0049433C, 0x00756F6A, 0x009D9792, 0x009C9691, 0x008F8984, 0x00736D68,
        0x003B3530, 0x00342E29, 0x0027211C, 0x00717066, 0x00636157, 0x005D5750, 0x00AFA9A2, 0x002E2A25, 0x000E0E08, 0x00131712, 0x00030A05, 0x000C100B, 0x000C100B, 0x000C100B, 0x000C100B, 0x000C100B, 0x000C100B, 0x000C100B, 0x000C100B, 0x000F130E,
        0x0010140F, 0x00030702, 0x00141813, 0x00060A05, 0x00121611, 0x000A0E09, 0x000E120D, 0x000C0D09, 0x00131410, 0x000D0E0A, 0x000E120D, 0x000F130E, 0x00080E09, 0x000D130E, 0x000B110C, 0x000B100E, 0x000B100E, 0x000E1512, 0x00252E2B, 0x00343F3C,
        0x002E3C38, 0x00263430, 0x001A2925, 0x005E6E67, 0x00E8F1EE, 0x00A9AFAE, 0x0081928F, 0x0095B2AF, 0x00A8C8C7, 0x00C9E0E2, 0x00DFEBEF, 0x00D0E6EB, 0x00BFDCE0, 0x00A8C7C8, 0x0094ACAC, 0x00E3EBEB, 0x00F2F2F2, 0x00F3F5F5, 0x00C6CFCC, 0x00B6C0BA,
        0x00F1FAF7, 0x00E5F0EE, 0x00DFEBEB, 0x00C7D9DA, 0x00ACC5C9, 0x00B0D0D5, 0x00B4D9DD, 0x00C9EAED, 0x00C9EAED, 0x00C1E2E5, 0x00B0D1D4, 0x009FC0C3, 0x009EBFC2, 0x00ADCED1, 0x00BEDFE2, 0x00BEDFE2, 0x00A9CACD, 0x0097B8BB, 0x009CBDC0, 0x00B2D3D6,
        0x00C3E4E7, 0x00C1E2E5, 0x00B7D8DB, 0x00A3C4C7, 0x00A1C2C5, 0x00A2C3C6, 0x00AECFD2, 0x00BEDFE2, 0x00C9EAED, 0x00CAEBEE, 0x00C6E7EA, 0x00AFD0D3, 0x00A8C9CC, 0x00A4C3C6, 0x00A9C8CB, 0x00B7D6D9, 0x00C7E4E8, 0x00D0EDF1, 0x00D2EFF3, 0x00C5E0E4,
        0x00C4DDE1, 0x00C6DBDD, 0x00CADDE0, 0x00D6E5E7, 0xEEE3EFF1, 0xFFEEF7FA, 0xFFF5FDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFEFEFE, 0xFFEFEFEF, 0xE6DFDFDF, 0x86B2B3B1, 0x3B8E8F8D, 0x005D5E5C, 0x00353634, 0x001A1B19, 0x000B0C0A, 0x00060705, 0x00070806, 0x00090A06, 0x000B0C08, 0x000C0D09, 0x000B0C08, 0x000B0C08, 0x000D0E0A, 0x000D0E0A, 0x000B0C08, 0x00090B05,
        0x000C0E08, 0x000D0F09, 0x000B0D07, 0x00080A04, 0x00080A04, 0x000C0E08, 0x000F110B, 0x000D0F09, 0x000D0F09, 0x000B0D07, 0x00090B05, 0x000B0D07, 0x000F110B, 0x000E100A, 0x000A0C06, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07,
        0x000A0C06, 0x00090B05, 0x00080A04, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x000A0E09,
        0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0B07, 0x000B0C08, 0x000C0D09, 0x000E0F0B, 0x000E0F0B, 0x000C0D09, 0x000B0C08, 0x000A0B07, 0x00080703, 0x00090804, 0x001A1915, 0x00454440, 0x007A7975,
        0x00999894, 0x009A9995, 0x008E8D89, 0x00BFBEBA, 0x00454440, 0x00605F5B, 0x00DBDAD6, 0x00464541, 0x00030200, 0x0011100C, 0x000F0E0A, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000B110C,
        0x000B110C, 0x000B110C, 0x000B110C, 0x000B110C, 0x000B110C, 0x000B110C, 0x000B110C, 0x000F100C, 0x000F100C, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000F100C, 0x000F100C, 0x000D100E, 0x000B0E0C, 0x00111614, 0x0029302D, 0x00333C39,
        0x002D3835, 0x00263430, 0x00182622, 0x006F736E, 0x00EAF3F0, 0x009DAEAB, 0x008AA39F, 0x00BBD8D5, 0x00BED8D8, 0x00C1D4D7, 0x00D5E4E7, 0x00BEE0E0, 0x00C6E4E5, 0x00BCD1D2, 0x00B7C5C4, 0x00E2EAE9, 0x00EDF3F2, 0x00F1F8F5, 0x00C6CDC8, 0x00BCC5C2,
        0x00EDF8F6, 0x00E5F3F1, 0x00DAECEB, 0x00D3EBEB, 0x00CBE9EA, 0x00BFE0E3, 0x00BADDE0, 0x00A6C5C8, 0x00A7C6C9, 0x00AECDD0, 0x00BBDADD, 0x00C5E4E7, 0x00BFDEE1, 0x00ABCACD, 0x0099B8BB, 0x009BBCBF, 0x00B0D1D4, 0x00C2E3E6, 0x00BDDEE1, 0x00A7C8CB,
        0x0098B9BC, 0x009CBDC0, 0x00A8C9CC, 0x00BFE0E3, 0x00C4E5E8, 0x00C5E6E9, 0x00B9DADD, 0x00A8C9CC, 0x00A0C1C4, 0x00A5C6C9, 0x00AECFD2, 0x00BDE0E4, 0x00C5E8EC, 0x00CBEBF0, 0x00C3E3E8, 0x00B6D4D9, 0x00AAC8CD, 0x00A9C7CC, 0x00AECBD0, 0x00C4DDE1,
        0x00D1E7EC, 0x00E0F5F7, 0x00E6F7FA, 0xF2E6F3F5, 0xEEE4EDF0, 0xF7E8EFF2, 0xFFEDF2F3, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFEFEFE, 0xFFF8F8F8, 0xFFFFFFFE, 0xFFFCFDFB, 0xFFEDEEEC, 0xCDD5D6D4, 0x86B2B3B1, 0x26848583, 0x00535452, 0x00333432, 0x001D1E1A, 0x000B0C08, 0x00010200, 0x00090A06, 0x000F100C, 0x000B0C08, 0x00070804, 0x000A0B07, 0x000F110B,
        0x000A0C06, 0x00070903, 0x000A0C06, 0x000F110B, 0x000E100A, 0x00080A04, 0x00030500, 0x00070903, 0x000C0E08, 0x000E100A, 0x000B0D07, 0x00080A04, 0x00090B05, 0x000A0C06, 0x000A0C06, 0x000B0D07, 0x000B0D07, 0x000C0E08, 0x000C0E08, 0x000B0D07,
        0x000A0C06, 0x000A0C06, 0x00090B05, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x000A0C06, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x000A0E09,
        0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0B07, 0x000B0C08, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000C0D09, 0x000B0C08, 0x000A0B07, 0x000D0C08, 0x0012110D, 0x0011100C, 0x00070602, 0x00020100,
        0x000C0B07, 0x002C2B27, 0x00474642, 0x00585753, 0x001A1915, 0x00353430, 0x00989793, 0x003D3C38, 0x00100F0B, 0x0011100C, 0x000F0E0A, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000B110C,
        0x000B110C, 0x000B110C, 0x000B110C, 0x000B110C, 0x000B110C, 0x000B110C, 0x000B110C, 0x000F100C, 0x000F100C, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000F100C, 0x000F100C, 0x000D100E, 0x000B0E0C, 0x00101513, 0x002A312E, 0x00343D3A,
        0x002E3936, 0x00263430, 0x00172521, 0x007D817C, 0x00E5EEEB, 0x00A6B7B4, 0x007A938F, 0x0092AFAC, 0x00BBD5D5, 0x00CFE2E5, 0x00D3E2E5, 0x00B7D9D9, 0x00B7D5D6, 0x00A8BDBE, 0x00ADBBBA, 0x00E0E8E7, 0x00EDF3F2, 0x00EEF5F2, 0x00BFC6C1, 0x00C2C9C6,
        0x00F1F9F8, 0x00E4F2F0, 0x00D2E4E3, 0x00C0D8D8, 0x00B5D3D4, 0x00AFD0D3, 0x00B0D3D6, 0x00CEEDF0, 0x00CCEBEE, 0x00C2E1E4, 0x00AFCED1, 0x009EBDC0, 0x009FBEC1, 0x00B3D2D5, 0x00C6E5E8, 0x00BCDDE0, 0x00A8C9CC, 0x0097B8BB, 0x009CBDC0, 0x00B2D3D6,
        0x00C1E2E5, 0x00BDDEE1, 0x00B2D3D6, 0x009EBFC2, 0x009DBEC1, 0x00A3C4C7, 0x00B2D3D6, 0x00C3E4E7, 0x00C9EAED, 0x00C4E5E8, 0x00BBDCDF, 0x00A7CACE, 0x00A1C4C8, 0x00A1C1C6, 0x00ADCDD2, 0x00C2E0E5, 0x00D0EEF3, 0x00D1EFF4, 0x00CEEBF0, 0x00C3DCE0,
        0x00C2D8DD, 0x00C3D8DA, 0x00CEDFE2, 0xE6E0EDEF, 0xFFEEF7FA, 0xFFF5FCFF, 0xFFF7FCFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFDFDFD,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFAFBF9, 0xFFFEFFFD, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFEFFFD, 0xFFF9FAF8, 0xFBEAEBE9, 0xD9DADBD9, 0x60A2A39F, 0x08797A76, 0x004A4B47, 0x002A2B27, 0x00131410, 0x00030400, 0x00030400, 0x000C0D09, 0x000B0C08,
        0x000B0C08, 0x000D0E0A, 0x000F100C, 0x000C0D09, 0x00080905, 0x00090A06, 0x000E0F0B, 0x000B0D07, 0x000C0E08, 0x000B0D07, 0x000A0C06, 0x000B0D07, 0x000D0F09, 0x000E100A, 0x000D0F09, 0x000B0D07, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08,
        0x000B0D07, 0x000A0C06, 0x000A0C06, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x000A0E09,
        0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0B07, 0x000B0C08, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000C0D09, 0x000B0C08, 0x000A0B07, 0x000D0C08, 0x000F0E0A, 0x0011100C, 0x0012110D, 0x00100F0B,
        0x000D0C08, 0x000A0905, 0x00080703, 0x00080703, 0x000F0E0A, 0x000A0905, 0x001A1915, 0x00020100, 0x000D0C08, 0x00161511, 0x000C0B07, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000A100B,
        0x000A100B, 0x000A100B, 0x000A100B, 0x000A100B, 0x000A100B, 0x000A100B, 0x000A100B, 0x000F100C, 0x000F100C, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000F100C, 0x000F100C, 0x000D100E, 0x000A0D0B, 0x00101513, 0x002A312E, 0x00353E3B,
        0x002F3A37, 0x00263430, 0x00162420, 0x007C807B, 0x00D8E1DE, 0x00859693, 0x0090A9A5, 0x00BBD8D5, 0x00B7D1D1, 0x00CADDE0, 0x00D3E2E5, 0x00C6E8E8, 0x00CBE9EA, 0x00BDD2D3, 0x00BFCDCC, 0x00E4ECEB, 0x00E8EEED, 0x00EBF2EF, 0x00C4CBC6, 0x00C8CFCC,
        0x00EEF6F5, 0x00DFECEA, 0x00D8EAE9, 0x00D4ECEC, 0x00CFEBEC, 0x00C1E0E3, 0x00B5D8DB, 0x00A7C6C9, 0x00A6C5C8, 0x00ACCBCE, 0x00BBDADD, 0x00C6E5E8, 0x00C0DFE2, 0x00A8C7CA, 0x0093B2B5, 0x00A3C4C7, 0x00B4D5D8, 0x00C1E2E5, 0x00B9DADD, 0x00A4C5C8,
        0x0099BABD, 0x00A1C2C5, 0x00AFD0D3, 0x00C0E1E4, 0x00C2E3E6, 0x00BEDFE2, 0x00B1D2D5, 0x00A2C3C6, 0x00A0C1C4, 0x00ACCDD0, 0x00BADBDE, 0x00C8EBEF, 0x00CAEDF1, 0x00C9E9EE, 0x00BADADF, 0x00ABC9CE, 0x00A5C3C8, 0x00ABC9CE, 0x00B6D3D8, 0x00D2EBEF,
        0x00D8EEF3, 0x00DEF3F5, 0x00E2F3F6, 0xEEE4F1F3, 0xF2E6EFF2, 0xFBEAF1F4, 0xFFEFF4F5, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE,
        0xFFFDFDFD, 0xFFFDFDFD, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBFBFB, 0xFFFFFFFE, 0xFFFFFFFE, 0xFBEAEBE9, 0xB0C7C8C6, 0x5C9E9F9D, 0x00717270, 0x00424341, 0x0020211F, 0x00151612,
        0x00080905, 0x00000100, 0x00030400, 0x000A0B07, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000C0E08, 0x000D0F09, 0x000E100A, 0x000E100A, 0x000D0F09, 0x000C0E08, 0x000A0C06, 0x00080A04, 0x000B0D07, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08,
        0x000C0E08, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x000A0E09,
        0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000B0C08, 0x000B0C08, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000C0D09, 0x000B0C08, 0x000B0C08, 0x0010110D, 0x000A0B07, 0x00050602, 0x00070804, 0x000E0F0B,
        0x0012130F, 0x000F100C, 0x000B0C08, 0x00070804, 0x00141511, 0x000A0B07, 0x000B0C08, 0x00131410, 0x000E0F0B, 0x000D0E0A, 0x0010110D, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000C100B,
        0x000C100B, 0x000C100B, 0x000C100B, 0x000C100B, 0x000C100B, 0x000C100B, 0x000C100B, 0x000F100C, 0x000E0F0B, 0x000E0F0B, 0x000D0E0A, 0x000D0E0A, 0x000E0F0B, 0x000E0F0B, 0x000F100C, 0x000C0F0D, 0x000A0D0B, 0x00101513, 0x002A312E, 0x00363F3C,
        0x00303B38, 0x0025332F, 0x0015231F, 0x00494D48, 0x00969F9C, 0x00879895, 0x0088A19D, 0x0094B1AE, 0x00B7D1D1, 0x00D1E4E7, 0x00D6E5E8, 0x00BCDEDE, 0x00B7D5D6, 0x009DB2B3, 0x00ADBBBA, 0x00E5EDEC, 0x00F4FAF9, 0x00EEF5F2, 0x00B7BEB9, 0x00CAD1CE,
        0x00F1F9F8, 0x00E3F0EE, 0x00D2E2E1, 0x00BCD1D2, 0x00B0CCCD, 0x00B0CFD2, 0x00B7D8DB, 0x00CBEAED, 0x00CBEAED, 0x00C3E2E5, 0x00AFCED1, 0x009DBCBF, 0x009DBCBF, 0x00B0CFD2, 0x00C4E3E6, 0x00AECFD2, 0x009FC0C3, 0x0095B6B9, 0x00A1C2C5, 0x00B8D9DC,
        0x00C4E5E8, 0x00BBDCDF, 0x00ACCDD0, 0x009ABBBE, 0x009EBFC2, 0x00A9CACD, 0x00BADBDE, 0x00C8E9EC, 0x00C8E9EC, 0x00BBDCDF, 0x00AFD0D3, 0x00A2C5C9, 0x00A3C6CA, 0x00ABCBD0, 0x00B9D9DE, 0x00CAE8ED, 0x00CEECF1, 0x00C7E5EA, 0x00BFDCE1, 0x00B7D0D4,
        0x00BED4D9, 0x00C9DEE0, 0x00D6E7EA, 0xEEE3F0F2, 0xFFEDF6F9, 0xFFF3FAFD, 0xFFF7FCFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFEFEFE, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFBFBFB, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF8F9F7, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFEFFFD, 0xFFFFFFFE, 0xFFFEFFFD, 0xEEE4E5E3, 0xABC5C6C4, 0x328C8D89,
        0x006A6B67, 0x003E3F3B, 0x001F201C, 0x000F100C, 0x00080905, 0x00070804, 0x00090A06, 0x00040600, 0x00080A04, 0x000B0D07, 0x000C0E08, 0x000A0C06, 0x00070903, 0x00080A04, 0x00090B05, 0x000B0D07, 0x000B0D07, 0x000C0E08, 0x000C0E08, 0x000C0E08,
        0x000C0E08, 0x000C0E08, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x000A0E09,
        0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000B0C08, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000B0C08, 0x000A0B07, 0x000F100C, 0x00131410, 0x0012130F, 0x000D0E0A,
        0x00090A06, 0x000B0C08, 0x000E0F0B, 0x0012130F, 0x0011120E, 0x000A0B07, 0x00030400, 0x0011120E, 0x00070804, 0x000C0D09, 0x000F100C, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000C100B,
        0x000C100B, 0x000C100B, 0x000C100B, 0x000C100B, 0x000C100B, 0x000C100B, 0x000C100B, 0x000F100C, 0x000E0F0B, 0x000E0F0B, 0x000D0E0A, 0x000D0E0A, 0x000E0F0B, 0x000E0F0B, 0x000F100C, 0x000C0F0D, 0x000A0D0B, 0x00111614, 0x002B322F, 0x00363F3C,
        0x002F3A37, 0x0025332F, 0x0015231F, 0x00989C97, 0x00E4EDEA, 0x00829390, 0x0090A9A5, 0x00C0DDDA, 0x00BAD4D4, 0x00CDE0E3, 0x00D3E2E5, 0x00BFE1E1, 0x00CBE9EA, 0x00B5CACB, 0x00BFCDCC, 0x00E5EDEC, 0x00EFF5F4, 0x00F0F7F4, 0x00C0C7C2, 0x00D2D7D5,
        0x00EDF3F2, 0x00DDE8E6, 0x00DAEAE9, 0x00D6EBEC, 0x00CDE9EA, 0x00BEDDE0, 0x00B2D3D6, 0x00A7C6C9, 0x00A4C3C6, 0x00A8C7CA, 0x00B6D5D8, 0x00C3E2E5, 0x00BFDEE1, 0x00A9C8CB, 0x0094B3B6, 0x00A7C8CB, 0x00B7D8DB, 0x00C2E3E6, 0x00B8D9DC, 0x00A2C3C6,
        0x0098B9BC, 0x00A2C3C6, 0x00B2D3D6, 0x00C3E4E7, 0x00C0E1E4, 0x00B7D8DB, 0x00A9CACD, 0x009EBFC2, 0x00A1C2C5, 0x00B0D1D4, 0x00BEDFE2, 0x00C8EBEF, 0x00C5E8EC, 0x00BEDEE3, 0x00B0D0D5, 0x00A7C5CA, 0x00A8C6CB, 0x00B7D5DA, 0x00C6E3E8, 0x00DAF3F7,
        0x00D9EFF4, 0x00D7ECEE, 0xD5D7E8EB, 0xDEDCE9EB, 0xEEE3ECEF, 0xFFEBF2F5, 0xFFF1F6F7, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFEFEFE, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFBFAFC, 0xFFFCFBFD, 0xFFFCFBFD, 0xFFFAF9FB, 0xFFFCFBFD, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBFBFB, 0xFFF8F8F8, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBFBFB, 0xFFFFFFFE,
        0xFFF7F8F6, 0xDEDDDEDC, 0x92B8B9B7, 0x328A8B89, 0x005A5B59, 0x00313230, 0x00181917, 0x0011130D, 0x00090B05, 0x00040600, 0x00080A04, 0x000F110B, 0x0010120C, 0x000C0E08, 0x00080A04, 0x000A0C06, 0x000A0C06, 0x000B0D07, 0x000C0E08, 0x000C0E08,
        0x000C0E08, 0x000C0E08, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x000A0E09,
        0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x00090D08, 0x00090D08, 0x00090D08, 0x000A0E09, 0x000B0F0A,
        0x000B0F0A, 0x000A0E09, 0x00090D08, 0x00070B06, 0x000B0F0A, 0x00121611, 0x00050904, 0x000C100B, 0x000E120D, 0x00131712, 0x00000400, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000E0F0B,
        0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000E0F0B, 0x000E0F0B, 0x000B0E0C, 0x000A0D0B, 0x00121715, 0x002B322F, 0x00353E3B,
        0x002E3936, 0x0025332F, 0x00172521, 0x00BFC3BE, 0x00E6EFEC, 0x009DAEAB, 0x00819A96, 0x0091AEAB, 0x00B6D0D0, 0x00CFE2E5, 0x00D2E1E4, 0x00B9DBDB, 0x00B9D7D8, 0x0099AEAF, 0x00B2C0BF, 0x00E8F0EF, 0x00F9FFFE, 0x00F2F9F6, 0x00B7BEB9, 0x00D3D8D6,
        0x00F0F6F5, 0x00E0EBE9, 0x00D4E2E1, 0x00BDD1D2, 0x00AFC8CA, 0x00AECBCF, 0x00B4D3D6, 0x00C7E6E9, 0x00C5E4E7, 0x00BBDADD, 0x00A9C8CB, 0x009AB9BC, 0x009EBDC0, 0x00B3D2D5, 0x00C8E7EA, 0x00AACBCE, 0x009DBEC1, 0x0097B8BB, 0x00A4C5C8, 0x00BBDCDF,
        0x00C3E4E7, 0x00B7D8DB, 0x00A6C7CA, 0x009ABBBE, 0x00A1C2C5, 0x00AECFD2, 0x00BDDEE1, 0x00C6E7EA, 0x00C2E3E6, 0x00B5D6D9, 0x00A9CACD, 0x009CBFC3, 0x00A4C7CB, 0x00B5D5DA, 0x00C4E4E9, 0x00CFEDF2, 0x00CBE9EE, 0x00BFDDE2, 0x00B5D2D7, 0x00B4CDD1,
        0x00C1D7DC, 0x00D2E7E9, 0xEAE1F2F5, 0xFFEBF8FA, 0xFFF0F9FC, 0xFFF3FAFD, 0xFFF7FCFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFEFEFE, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFBFAFC, 0xFFFCFBFD, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFCFDFB,
        0xFFFEFFFD, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFF4F5F3, 0xC9D3D4D2, 0x8EB6B7B5, 0x117E807A, 0x005A5C56, 0x002D2F29, 0x0010120C, 0x00060802, 0x00070903, 0x000B0D07, 0x000E100A, 0x00090B05, 0x000A0C06, 0x000A0C06, 0x000B0D07, 0x000C0E08,
        0x000C0E08, 0x000B0D07, 0x000B0D07, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x000A0E09,
        0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000B0C08, 0x000B0C08, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000B0F0A, 0x000B0F0A, 0x000B0F0A, 0x000A0E09, 0x000B0F0A,
        0x000B0F0A, 0x000C100B, 0x000C100B, 0x00111510, 0x00010500, 0x00070B06, 0x000B0F0A, 0x000D110C, 0x00020601, 0x00080C07, 0x000C100B, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A,
        0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000E0F0B, 0x000E0F0B, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000E0F0B, 0x000E0F0B, 0x000B0E0C, 0x000B0E0C, 0x00131816, 0x002B322F, 0x00333C39,
        0x002C3734, 0x0025332F, 0x00192723, 0x00A8ACA7, 0x00D5DEDB, 0x007A8B88, 0x0094ADA9, 0x00C2DFDC, 0x00BCD6D6, 0x00CBDEE1, 0x00D2E1E4, 0x00C6E8E8, 0x00CAE8E9, 0x00ADC2C3, 0x00CAD8D7, 0x00F2FAF9, 0x00F6FCFB, 0x00F4FBF8, 0x00C9D0CB, 0x00DCDFDD,
        0x00EEF3F2, 0x00DDE5E4, 0x00DBE9E8, 0x00D1E5E6, 0x00C8E1E3, 0x00BFDCE0, 0x00B9D8DB, 0x00A8C7CA, 0x00A6C5C8, 0x00AAC9CC, 0x00B6D5D8, 0x00BFDEE1, 0x00B8D7DA, 0x00A0BFC2, 0x008BAAAD, 0x00ACCDD0, 0x00B8D9DC, 0x00BFE0E3, 0x00B3D4D7, 0x009FC0C3,
        0x009ABBBE, 0x00ABCCCF, 0x00BEDFE2, 0x00CCEDF0, 0x00BDDEE1, 0x00A9CACD, 0x009CBDC0, 0x009CBDC0, 0x00A8C9CC, 0x00B7D8DB, 0x00C0E1E4, 0x00CBEEF2, 0x00BFE2E6, 0x00B0D0D5, 0x00A5C5CA, 0x00A8C6CB, 0x00B3D1D6, 0x00C1DFE4, 0x00CCE9EE, 0x00DDF6FA,
        0x00D6ECF1, 0x00CFE4E6, 0xC9D1E2E5, 0xDEDCE9EB, 0xF7E7F0F3, 0xFFEDF4F7, 0xFFF0F5F6, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBFBFB, 0xFFFBFBFB, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFDFDFD, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFCFBFD, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFFFD,
        0xFFFFFFFE, 0xFFFFFFFE, 0xFFFAFBF9, 0xFFFEFFFD, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFF9FAF8, 0xFFFFFFFB, 0xFFF0F2EC, 0xC9D6D8D2, 0x75ADAFA9, 0x04777973, 0x0042443E, 0x001D1F19, 0x000E100A, 0x00080A04, 0x00090B05, 0x000A0C06, 0x000B0D07, 0x000B0D07,
        0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x000C0E08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x00090D08, 0x000A0E09,
        0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000B0C08, 0x000B0C08, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000A0E09, 0x00080C07, 0x00070B06, 0x00080C07, 0x000B0F0A,
        0x000C100B, 0x000B0F0A, 0x00090D08, 0x000D110C, 0x00090D08, 0x000D110C, 0x00080C07, 0x000B0F0A, 0x00090D08, 0x000C100B, 0x000D110C, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A,
        0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000E0F0B, 0x000E0F0B, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000E0F0B, 0x000E0F0B, 0x000B0E0C, 0x000B0E0C, 0x00141917, 0x002B322F, 0x00323B38,
        0x002A3532, 0x0025332F, 0x001B2925, 0x00B3B7B2, 0x00DFE8E5, 0x0093A4A1, 0x0087A09C, 0x0096B3B0, 0x00B0CACA, 0x00D1E4E7, 0x00D0DFE2, 0x00B8DADA, 0x00B3D1D2, 0x0096ABAC, 0x00C3D1D0, 0x00F6FEFD, 0x00FAFFFF, 0x00F9FFFD, 0x00D1D8D3, 0x00E0E3E1,
        0x00F0F5F4, 0x00DCE4E3, 0x00D2E0DF, 0x00BED2D3, 0x00B2CBCD, 0x00AECBCF, 0x82B0CFD2, 0x00C0DFE2, 0x00BEDDE0, 0x00B5D4D7, 0x00A6C5C8, 0x009AB9BC, 0x009FBEC1, 0x00B4D3D6, 0x00C6E5E8, 0x00A2C3C6, 0x0098B9BC, 0x0096B7BA, 0x00A8C9CC, 0x00C0E1E4,
        0x00C6E7EA, 0x00B3D4D7, 0x009DBEC1, 0x0096B7BA, 0x00A7C8CB, 0x00BCDDE0, 0x00C8E9EC, 0x00C5E6E9, 0x00B7D8DB, 0x00A9CACD, 0x00A1C2C5, 0x009DC0C4, 0x00ADD0D4, 0x00C3E3E8, 0x00CDEDF2, 0x00CAE8ED, 0x00BEDCE1, 0x00B4D2D7, 0x00B0CDD2, 0x00B6CFD3,
        0x00C7DDE2, 0x00DAEFF1, 0x00E6F7FA, 0xFBEAF7F9, 0xFFECF5F8, 0xFFF2F9FC, 0xFFF8FDFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFDFDFD, 0xF7E7E7E7, 0x9BBCBCBC, 0x4F979797, 0x005E5F5B, 0x00343531, 0x00141511, 0x000C0D09, 0x00080905,
        0x00060703, 0x000A0B07, 0x000C0D09, 0x000D0F09, 0x000A0C06, 0x000B0D07, 0x000C0E08, 0x00090B05, 0x000C0E08, 0x000E100A, 0x000A0C06, 0x000E100A, 0x000B0D07, 0x000A0C06, 0x000D0F09, 0x000E100A, 0x000B0D07, 0x000B0D07, 0x000E100A, 0x00080C07,
        0x000A0E09, 0x000A0E09, 0x00080C07, 0x000A0E09, 0x000D110C, 0x000C100B, 0x00080C07, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09,
        0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000D0E0A,
        0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000C120D, 0x000B110C, 0x00101611, 0x00292F2A, 0x00343A35,
        0x002D332E, 0x002F3530, 0x00323833, 0x00AAB5B3, 0x00E4F0F0, 0x00ACBAB9, 0x0099A9A8, 0x00CBDFE0, 0x00D0E8E8, 0x00CDE6E8, 0x00CDE6E8, 0x00C6E3E0, 0x00D3EBE9, 0x00A1B6B4, 0x00CBD9D8, 0x00DFE7E7, 0x00F0F4F5, 0x00FFFCFE, 0x00E9E4E6, 0x00E7E7E7,
        0x00E9E9E9, 0x00EDEDED, 0x00FBFBFB, 0x00FFFFFF, 0xFFF7F7F7, 0xFFF0F0F0, 0xEEE3E3E3, 0xB4C7CFCF, 0xA3BFC9C9, 0x9BBCCAC9, 0x00C0D2D3, 0x00C0D5D6, 0x00B2CBCD, 0x00A0BCBD, 0x0093B1B2, 0x00ADD0D3, 0x00BCDDE0, 0x00BBDCDF, 0x00A6C6CB, 0x0096B4B9,
        0x009DBAC1, 0x00B4D0D7, 0x00C6E2E9, 0x00CFE9F0, 0x00BBD7DE, 0x00A6C2C9, 0x009CB9C0, 0x00A1C1C7, 0x00B2D2D8, 0x00BFE1E7, 0x00C7E9EF, 0x00C6E7EA, 0x00B6D7DA, 0x00A7C6C9, 0x00A1C0C3, 0x00AAC9CC, 0x00BCD9DD, 0x00CAE7EB, 0x00D0EDF1, 0x00D6EBED,
        0x00D4E7EA, 0x00D2E4E5, 0xD5D7E4E6, 0xE6E0E9EC, 0xFFECF1F2, 0xFFF4F8F9, 0xFFFAFCFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF9F9F9, 0xFFFBFCF8, 0xCDD6D7D3, 0x69A7A8A4, 0x04777874, 0x00484945,
        0x00252622, 0x0011120E, 0x00050602, 0x00090B05, 0x00020400, 0x00040600, 0x000B0D07, 0x000B0D07, 0x000B0D07, 0x000C0E08, 0x000A0C06, 0x000C0E08, 0x000A0C06, 0x00090B05, 0x00080A04, 0x00090B05, 0x000B0D07, 0x000B0D07, 0x000C0E08, 0x000A0E09,
        0x000C100B, 0x000C100B, 0x00090D08, 0x00080C07, 0x000A0E09, 0x000B0F0A, 0x00090D08, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09,
        0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000D0E0A,
        0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000C120D, 0x000A100B, 0x00111712, 0x002A302B, 0x00373D38,
        0x00303631, 0x00313732, 0x00333934, 0x00C4CCCB, 0x00EBF3F2, 0x00C0CBC9, 0x006C7A79, 0x006B7B7A, 0x009FB3B4, 0x00BCD1D2, 0x00BACFD0, 0x00B0C6C4, 0x0094A9A7, 0x006E7E7D, 0x00BBC9C8, 0x00DFE9E9, 0x00D3D8D9, 0x00DDDFE0, 0x00E4E3E5, 0x00ECECEC,
        0x00F8F8F8, 0x00F9F9F9, 0x00FBFBFB, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBFBFB, 0xFFFAFFFF, 0xFFF4F9FA, 0xDEDBE5E5, 0xABC3D0D2, 0x82B0C2C3, 0x00ADC0C3, 0x00B6CDCF, 0x00C2D9DB, 0x00A3C1C2, 0x009BB8BC, 0x009EBDC0, 0x00B3D2D5, 0x00C4E2E7,
        0x00BDDDE2, 0x00A8C8CE, 0x0097B7BD, 0x0099B9BE, 0x00AECED3, 0x00C5E5EA, 0x00CDEDF2, 0x00C3E3E8, 0x00B1D1D6, 0x00A3C3C8, 0x009CBCC1, 0x00A7C8CB, 0x00B7D8DB, 0x00C9E8EB, 0x00CCEBEE, 0x00C1E0E3, 0x00B5D2D6, 0x00AECBCF, 0x00AECBCF, 0x00C6DBDD,
        0x00D1E4E7, 0x00DFF1F2, 0xFBE9F6F8, 0xFFECF5F8, 0xFFEEF3F4, 0xFFF0F4F5, 0xFFF5F7F8, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFEFA, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFBEDEEEA,
        0xABC7C8C4, 0x3B90918D, 0x0061625E, 0x00353632, 0x001E1F1B, 0x000E0F0B, 0x000A0B07, 0x00060703, 0x00050602, 0x000A0B07, 0x000E0F0B, 0x00070804, 0x000A0B07, 0x000C0D09, 0x000E0F0B, 0x0011120E, 0x0011120E, 0x000C0D09, 0x00070804, 0x00050904,
        0x000A0E09, 0x000C100B, 0x000B0F0A, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x00090D08, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09,
        0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000D0E0A,
        0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000B110C, 0x000A100B, 0x00111712, 0x002D332E, 0x003A403B,
        0x00333934, 0x00343A35, 0x00353B36, 0x008C918F, 0x00C2C7C5, 0x00E8EEED, 0x00BEC6C5, 0x00768080, 0x005F6B6B, 0x007F8C8E, 0x00AAB9BB, 0x00BCC8C8, 0x00C7D3D3, 0x00B3BFBF, 0x00CBD5D5, 0x00DBE5E5, 0x00D9E1E1, 0x00DEE6E6, 0x00CBD3D3, 0x00DEDEDE,
        0x00FBFBFB, 0x00FFFFFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFAFAFA, 0xFFFDFFFF, 0xFFFBFFFF, 0xFFF7FFFF, 0xF7E7F0F3, 0xC4CFDCDE, 0x8AB4C3C6, 0x64A1B2B5, 0x00B9CED0, 0x00C2D9DB, 0x00BDD6D8, 0x00A4BFC3, 0x0093B2B5,
        0x009DBDC2, 0x00B3D6DA, 0x00C0E5E9, 0x00B7DDE1, 0x00AACFD3, 0x009BC0C4, 0x0099BCC0, 0x00A6C6CB, 0x00B8D8DD, 0x00C8E6EB, 0x00CFEDF2, 0x00BBDCDF, 0x00ADCED1, 0x00A2C1C4, 0x00A4C3C6, 0x00B3D2D5, 0x00C5E2E6, 0x00CEEBEF, 0x00CFECF0, 0x00CCE1E3,
        0x00C9DCDF, 0x00CADCDD, 0xD1D5E2E4, 0xEEE4EDF0, 0xFFF3F8F9, 0xFFF9FDFE, 0xFFFCFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFCFCFC, 0xFFFCFCFC, 0xFFFDFDFD, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFDFDFD, 0xFFFCFDFB, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFDFEFC, 0xFFFDFEFC,
        0xFFFFFFFE, 0xFFFFFFFE, 0xFFF9FAF8, 0xD5DBDCD8, 0x82B3B4B0, 0x1D838480, 0x00575854, 0x0032332F, 0x00191A16, 0x000B0C08, 0x00020300, 0x0010110D, 0x000B0C08, 0x00080905, 0x00090A06, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000C0D09, 0x00070B06,
        0x000A0E09, 0x000D110C, 0x000C100B, 0x000A0E09, 0x000A0E09, 0x000C100B, 0x000E120D, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09,
        0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000D0E0A,
        0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000A100B, 0x00090F0A, 0x00121813, 0x002F3530, 0x003D433E,
        0x00373D38, 0x00373D38, 0x00373D38, 0x00191A16, 0x00393C3A, 0x00545755, 0x00898E8C, 0x00B6BBBA, 0x00C9CFCE, 0x00E7EFEF, 0x00F6FEFE, 0x00E7ECEB, 0x00F6FBFA, 0x00E9EFEE, 0x00DBE3E2, 0x00DEE6E5, 0x00D3DEDC, 0x00C6D3D1, 0x00A7B4B2, 0x00D5D5D5,
        0xFFF8F8F8, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFCFCFC, 0xFFFBFBFB, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFCFEFF, 0xFFFBFFFF, 0xFFFAFFFF, 0xFFF2F9FC, 0xF2E5ECEF, 0x9BBBC8CA, 0x75A9B8BA, 0x00AABCBD, 0x00C1D8DA, 0x00CAE6E7,
        0x00B6D5D8, 0x009EC1C4, 0x0096BCBE, 0x009EC7CA, 0x00B1D7DB, 0x00C3E8EC, 0x00C4E7EB, 0x00B6D6DB, 0x00A6C4C9, 0x009FBCC1, 0x009FBCC1, 0x00B0D1D4, 0x00C2E3E6, 0x00D1F0F3, 0x00C8E7EA, 0x00B1D0D3, 0x00A4C1C5, 0x00AAC7CB, 0x00B5D2D6, 0x00CEE3E5,
        0x00DBEEF1, 0x00E8FAFB, 0xFFEBF8FA, 0xF2E6EFF2, 0xF2E6EBEC, 0xFFEDF1F2, 0xFFF8FAFB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFCFCFC, 0xFFFEFFFD, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFEFFFD,
        0xFFFDFEFC, 0xFFFBFCFA, 0xFFFAFBF9, 0xFFFFFFFC, 0xFFFEFFFB, 0xFFFCFDF9, 0xFFEFF0EC, 0xC0D1D2CE, 0x6DA8A9A5, 0x04777874, 0x004D4E4A, 0x00232420, 0x00141511, 0x00080905, 0x00080905, 0x00090A06, 0x00070804, 0x00090A06, 0x000D0E0A, 0x000A0E09,
        0x00090D08, 0x00090D08, 0x00080C07, 0x00070B06, 0x00070B06, 0x00090D08, 0x000C100B, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09,
        0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09,
        0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x00090F0A, 0x00080E09, 0x00111712, 0x002F3530, 0x003F4540,
        0x00393F3A, 0x00393F3A, 0x00383E39, 0x001D1F19, 0x002D2E2A, 0x0020211D, 0x001E221D, 0x00353836, 0x00545755, 0x00878989, 0x00ADAFAF, 0x00DDDDDD, 0x00E3E3E3, 0x00E2E4E4, 0x00D8DDDC, 0x00DAE2E1, 0x00D7E2E0, 0x00D4E1DF, 0x00D3E0DE, 0xFBEAEAEA,
        0xFFFFFFFF, 0xFFFEFEFE, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFEFD, 0xFFFFFEFF, 0xFFFFFDFE, 0xFFFBF9F9, 0xFFF9F8FA, 0xFFFBFDFE, 0xFFFDFEFF, 0xFFFBFCFF, 0xFFF6FBFC, 0xF7E8F0F0, 0xB8C9D5D5, 0x75AAB9BB, 0x64A2B6B7,
        0x00B2C9CB, 0x00BEDADB, 0x00C0DEDF, 0x00B1D4D7, 0x00A1C4C7, 0x0097B8BB, 0x00A0C1C4, 0x00B8D7DA, 0x00CAE7EB, 0x00C9E6EA, 0x00C4DFE3, 0x00B0D1D4, 0x00A3C4C7, 0x009DBCBF, 0x00A8C7CA, 0x00C0DFE2, 0x00D1EEF2, 0x00CFECF0, 0x00C6E3E7, 0x00C3D8DA,
        0x00C4D7DA, 0x00CBDDDE, 0xDEDBE8EA, 0xFFECF5F8, 0xFFF8FDFE, 0xFFF9FDFE, 0xFFF8FAFB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFBFBFB, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBFBFB, 0xFFFDFDFD, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFEFFFD, 0xFFFDFEFC, 0xFFFFFFFE, 0xFFFAFBF9, 0xF7E9EAE8, 0xA7C3C4C2, 0x589C9D9B, 0x00676866, 0x003D3E3C, 0x001F201E, 0x000F100E, 0x000A0B09, 0x000D0E0C, 0x000A0E09,
        0x00080C07, 0x00090D08, 0x000D110C, 0x000F130E, 0x000C100B, 0x00090D08, 0x00080C07, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09,
        0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09,
        0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x00090F0A, 0x00080E09, 0x00101611, 0x002F3530, 0x003F4540,
        0x003A403B, 0x003A403B, 0x00393F3A, 0x00363A34, 0x001F231D, 0x002A2E28, 0x002F332E, 0x0041423E, 0x00484945, 0x002D2E2C, 0x00323331, 0x0060615F, 0x00858684, 0x00B1B4B2, 0x00C3C8C6, 0x00D9E0DD, 0x00F5FCF9, 0x00E8F1EE, 0x00E6EFEC, 0xFFFDFDFD,
        0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFEFAF9, 0xFFFFFFFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFFFFFF, 0xFFFBFDFD, 0xFFF8FCFD, 0xFFF7FCFD, 0xEAE1EBEB,
        0x9FBECACC, 0x79ABB8BA, 0x75AAB9BB, 0x00B9CFD4, 0x00C6DFE3, 0x00D0E9ED, 0x00C7E0E4, 0x00B0CBCF, 0x00A3BEC2, 0x00A5C2C6, 0x00AFCCD0, 0x00BCDDE0, 0x00C3E4E7, 0x00C6E5E8, 0x00BAD9DC, 0x00AAC9CC, 0x00A6C3C7, 0x00B0CDD1, 0x00BDDADE, 0x00D6EBED,
        0x00DDF0F3, 0x00E4F6F7, 0xEEE4F1F3, 0xE6E0E9EC, 0xF2E5EAEB, 0xFFF0F4F5, 0xFFFCFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFDFDFD, 0xFFFBFBFB, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFBFBFB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFCFCFC, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFEFFFD, 0xFFFCFDFB, 0xFFFEFFFD, 0xFFFEFFFD, 0xFFFDFEFC, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFF9FAF8, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFF5F6F4, 0xE2DFE0DE, 0x9BBDBEBC, 0x3B8E8F8D, 0x00575856, 0x0030312F, 0x00191D18,
        0x000E120D, 0x00040803, 0x00030702, 0x00070B06, 0x000A0E09, 0x000A0E09, 0x000B0F0A, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09,
        0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09,
        0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x00090F0A, 0x00070D08, 0x000F1510, 0x002D332E, 0x003F4540,
        0x003A403B, 0x00393F3A, 0x00393F3A, 0x001D261C, 0x00262E24, 0x00444C42, 0x00353C35, 0x00343832, 0x0040443F, 0x002A2B27, 0x0031322E, 0x004B504E, 0x003F4442, 0x00383D3B, 0x00444745, 0x007C7F7D, 0x00DDE0DE, 0x00F1F4F2, 0xFFFDFFFE, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFFFFFE, 0xFFFFFDFC, 0xFFFEFCFB, 0xFFFFFEFE, 0xFFFFFFFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFAF8F7, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFD, 0xFFFDFCFE,
        0xFFFFFEFF, 0xFFF9F8FC, 0xFBE9EAEE, 0x9FBEC9CD, 0x82AFBBBF, 0x6DA5B4B7, 0x00AFC2C5, 0x00C4DBDD, 0x00CDE9EA, 0x00C2E0E1, 0x00B3D2D3, 0x009FC0C3, 0x00A1C2C5, 0x00AAC9CC, 0x00B9D8DB, 0x00C7E6E9, 0x00CDEAEE, 0x00C6E3E7, 0x00BEDBDF, 0x00C1D6D8,
        0x00C5D8DB, 0x00D0E2E3, 0xEAE1EEF0, 0xFFF0F9FC, 0xFFFAFFFF, 0xFFF8FCFD, 0xFFF6F8F9, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFCFCFC, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFAFAFA, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFEFEFE, 0xFFFDFDFD, 0xFFFFFFFE, 0xFFFDFEFC, 0xFFFEFFFD, 0xFFFFFFFE, 0xFFFDFEFC, 0xFFFEFFFD, 0xFFFFFFFE, 0xFFFEFFFD, 0xFFFAFBF9, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFCFDFB, 0xFFFEFFFD, 0xFFFFFFFE, 0xFFEFF0EE, 0xD5D8D9D7, 0x64A2A6A1,
        0x157D817C, 0x004D514C, 0x00282C27, 0x00131712, 0x00090D08, 0x00060A05, 0x00070B06, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09,
        0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09,
        0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000A100B, 0x00070D08, 0x000E140F, 0x002C322D, 0x003E443F,
        0x00393F3A, 0x00393F3A, 0x00383E39, 0x00313D31, 0x003E473D, 0x00394238, 0x00343C32, 0x003E453E, 0x003B3F39, 0x002C302B, 0x00363733, 0x004A514E, 0x00505754, 0x00424745, 0x00363937, 0x006F7270, 0x00F9FAF8, 0xFFFFFFFE, 0xFFFFFDFC, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFC, 0xFFFEFFFD, 0xFFFDFEFC, 0xFFFFFFFE, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFDFCFE, 0xFFFFFEFF, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFDFE, 0xFFFFFDFE, 0xFFFFFEFF,
        0xFFFFFDFF, 0xFFFFFCFF, 0xFFFFF8FB, 0xFFFCFCFF, 0xFFF4F8FD, 0xEAE1EAEE, 0xABC4D3D6, 0x75A9BEC0, 0x609FB8BA, 0x6DA6C5C6, 0x00B2D4D4, 0x00C9EAED, 0x00C5E6E9, 0x00BCDBDE, 0x00ACCBCE, 0x00A0BFC2, 0x00A5C2C6, 0x00B7D4D8, 0x00C7E4E8, 0x00DAEFF1,
        0x00D9ECEF, 0x00D9EBEC, 0xDEDCE9EB, 0xE6E0E9EC, 0xFBEAEFF0, 0xFFF3F7F8, 0xFFFBFDFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFEFEFE, 0xFFEEEEEE, 0xB8CACACA, 0x5C9D9D9D, 0x006F6F6F, 0x00404040, 0x00202020, 0x000F100C, 0x00080905, 0x00070804, 0x000B0C08, 0x000B0C08, 0x00070804, 0x000B0C08, 0x00131410, 0x00070804, 0x000D0E0A, 0x000E0F0B, 0x000A0B07, 0x00080905,
        0x000A0B07, 0x000C0D09, 0x000A0B07, 0x000B0F0A, 0x00070B06, 0x00060A05, 0x00090D08, 0x000A0E09, 0x00080C07, 0x00090D08, 0x000C100B, 0x000D0E0A, 0x000C0D09, 0x000A0B07, 0x000B0C08, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A,
        0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000D110C, 0x000A0E09, 0x0010140F, 0x002E322D, 0x003F433E,
        0x003A3E39, 0x003A3E39, 0x003B3F3A, 0x003B413C, 0x00373D38, 0x00363C37, 0x00393F3A, 0x00383E39, 0x00353B36, 0x00212722, 0x00373D38, 0x00464B49, 0x004C514F, 0x00434846, 0x003C3F3D, 0x00A3A6A4, 0xFFFCFDFB, 0xFFFDFEFC, 0xFFFFFFFE, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFEFEFEF, 0xD9DADADA, 0xB8CACACA, 0x79ABB7B9, 0x86B1BDBF, 0x00BDCACC, 0x00C8DADB, 0x00D1E5E6, 0x00CDE5E5, 0x00C1DBDB, 0x00B5D1D1, 0x00BED2D3,
        0x00C7DBDC, 0x00D7E9EA, 0xF7E8F5F7, 0xFFF2FBFE, 0xFFF6FDFF, 0xFFF5FAFD, 0xFFF3F6FA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFDFDFD, 0xFFF7F7F7, 0xE6E0E0E0, 0xB8C9C9C9, 0x368F908C, 0x00646561, 0x0032332F, 0x00161713, 0x000F100C, 0x000D0E0A, 0x00090A06, 0x00070804, 0x000D0E0A, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000E0F0B,
        0x000C0D09, 0x000B0C08, 0x000B0C08, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x00080C07, 0x00080C07, 0x00090D08, 0x00080C07, 0x00070B06, 0x000C0D09, 0x000D0E0A, 0x000E0F0B, 0x000E0F0B, 0x000E0F0B, 0x000D0E0A, 0x000B0C08, 0x000A0B07, 0x000D0E0A,
        0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000D110C, 0x000A0E09, 0x0010140F, 0x002D312C, 0x003E423D,
        0x003A3E39, 0x003A3E39, 0x003B3F3A, 0x00373D38, 0x00333934, 0x003C423D, 0x003C423D, 0x00303631, 0x00393F3A, 0x002C322D, 0x002F3530, 0x004C514F, 0x004A4F4D, 0x002D3230, 0x005B5E5C, 0xD9D9DCDA, 0xFFFFFFFE, 0xFFF9FAF8, 0xFFFFFFFE, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFAFAFA, 0xFFF3FAFD, 0xE6DFE8EB, 0xB0C6D2D4, 0x8EB5C2C4, 0x8AB3C2C4, 0x00BACECF, 0x00C7DCDD, 0x00D1E6E7, 0x00DEF0F1,
        0x00DBEDEE, 0xD9D9E8EA, 0xDEDBE7E9, 0xE6E0E9EC, 0xFBEAF1F4, 0xFFF4F9FC, 0xFFFBFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFCFC,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF7F7F7, 0xFFFFFFFC, 0xFFF4F5F1, 0xD5DBDCD8, 0x86B4B5B1, 0x1D82837F, 0x0050514D, 0x00292A26, 0x00151612, 0x00090A06, 0x00070804, 0x000A0B07, 0x000F100C, 0x000E0F0B,
        0x00080905, 0x00070804, 0x000B0C08, 0x00070B06, 0x000C100B, 0x000D110C, 0x00090D08, 0x000A0E09, 0x000E120D, 0x000E120D, 0x000A0E09, 0x00090A06, 0x000A0B07, 0x000D0E0A, 0x000E0F0B, 0x000F100C, 0x000E0F0B, 0x000E0F0B, 0x000D0E0A, 0x000D0E0A,
        0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000C100B, 0x000A0E09, 0x0010140F, 0x002C302B, 0x003D413C,
        0x00393D38, 0x003A3E39, 0x003A3E39, 0x003B413C, 0x00353B36, 0x00313732, 0x00333934, 0x00373D38, 0x003B413C, 0x00252B26, 0x00303631, 0x00484D4B, 0x00303533, 0x004D5250, 0x86B2B5B3, 0xFFFDFFFE, 0xFFFFFFFE, 0xFFFEFFFD, 0xFFFEFFFD, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFCFCFC, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFBFEFF, 0xFFFBFEFF, 0xFFFAFFFF, 0xFFF4FBFE, 0xFBE9F2F5, 0xD5D8E4E6, 0xB0C6D3D5, 0x9BBBC8CA, 0xABC3D2D4,
        0x00CEDBDD, 0xE2DDE9EB, 0xFFECF5F8, 0xFFF5FCFF, 0xFFF7FCFF, 0xFFF3F6FA, 0xFFEFF2F6, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF9F9F9,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFF9F9F9, 0xFFFBFBFB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFE, 0xFFFAFBF9, 0xFFF7F8F6, 0xFFFCFDFB, 0xFFFFFFFE, 0xFFF2F3F1, 0xB8CBCCCA, 0x6DA7A8A6, 0x00727371, 0x00494A48, 0x001A1B19, 0x00060705, 0x00090A08,
        0x000F100E, 0x000D0E0C, 0x00080907, 0x00060A05, 0x00090D08, 0x00090D08, 0x00080C07, 0x00080C07, 0x000B0F0A, 0x000C100B, 0x000A0E09, 0x000A0B07, 0x000A0B07, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000C0D09, 0x000D0E0A, 0x000F100C, 0x000D0E0A,
        0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000C100B, 0x000A0E09, 0x0010140F, 0x002A2E29, 0x003B3F3A,
        0x00383C37, 0x00393D38, 0x00383C37, 0x00353B36, 0x00373D38, 0x00383E39, 0x00353B36, 0x00303631, 0x00343A35, 0x00262C27, 0x00383E39, 0x00353A38, 0x00474C4A, 0x86B1B6B4, 0xFFF9FCFA, 0xFFFAFDFB, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFDFEFC, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFDFDFD, 0xFFFDFCFE, 0xFFFEFDFF, 0xFFFFFEFF, 0xFFFDFFFF, 0xFFFBFFFF, 0xFFFAFEFF, 0xFFF8FDFE, 0xFFF7FCFD, 0xEEE3EDED,
        0xEAE1EBEB, 0xEAE2EAEA, 0xEAE2EAEA, 0xFBE9EEEF, 0xFFF1F5F6, 0xFFF8FCFD, 0xFFFBFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBFCFA, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFDFEFC, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFF8F9F7, 0xF2E7E8E6, 0xA7C2C3C1, 0x3B8E8F8D, 0x005A5B59,
        0x00323331, 0x00151614, 0x00060705, 0x00070B06, 0x00040803, 0x00050904, 0x000A0E09, 0x000B0F0A, 0x00090D08, 0x00070B06, 0x00070B06, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000B0C08, 0x000B0C08, 0x000A0B07, 0x00090A06, 0x00090A06, 0x000D0E0A,
        0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000B0F0A, 0x000B0F0A, 0x0010140F, 0x00292D28, 0x00393D38,
        0x00383C37, 0x00393D38, 0x00373B36, 0x00393F3A, 0x00333934, 0x003A403B, 0x00373D38, 0x002C322D, 0x00383E39, 0x002C322D, 0x00292F2A, 0x00525755, 0x7DADB2B0, 0xFFFBFFFE, 0xFFFDFFFE, 0xFFF8FBF9, 0xFFFFFFFE, 0xFFFDFEFC, 0xFFFEFFFD, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFDFF, 0xFFFEFDFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFAFFFF,
        0xFFFBFFFF, 0xFFFAFEFF, 0xFFF6FAFB, 0xFFF5F9FA, 0xFFF4F8F9, 0xFFF7F9FA, 0xFFF8FAFB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFEFEFE, 0xFFFDFDFD, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBFBFB, 0xFFF0F0F0,
        0xC9D2D2D2, 0x64A2A2A2, 0x11797979, 0x003E423D, 0x002A2E29, 0x00141813, 0x00080C07, 0x00040803, 0x00030702, 0x00090D08, 0x0010140F, 0x00090A06, 0x00090A06, 0x000A0B07, 0x000C0D09, 0x000E0F0B, 0x000E0F0B, 0x000C0D09, 0x000A0B07, 0x000D0E0A,
        0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000B0F0A, 0x000B0F0A, 0x000F130E, 0x00272B26, 0x00373B36,
        0x00373B36, 0x00393D38, 0x00363A35, 0x00383E39, 0x00383E39, 0x002F3530, 0x00323833, 0x003A403B, 0x002E342F, 0x00191F1A, 0x00474D48, 0x8EB5BAB8, 0xFFFBFFFE, 0xFFFBFFFE, 0xFFFBFEFC, 0xFFFDFFFE, 0xFFFDFEFC, 0xFFFDFEFC, 0xFFFFFFFE, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFDFDFD, 0xFFFCFCFC, 0xFFFCFCFC, 0xFFFBFBFB, 0xFFFCFCFC, 0xFFFCFCFC, 0xFFFFFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFE, 0xFFFFFDFD, 0xFFFEFCFC, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFFFF, 0xFFFDFFFF, 0xFFFDFFFF, 0xFFFCFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFDFDFD, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFBFBFB, 0xFFF7F7F7, 0xDEDCE0DB, 0x96BBBFBA, 0x2E898D88, 0x005B5F5A, 0x00373B36, 0x001D211C, 0x000C100B, 0x00020601, 0x0011120E, 0x000D0E0A, 0x000A0B07, 0x000A0B07, 0x000E0F0B, 0x0010110D, 0x000F100C, 0x000D0E0A, 0x000D0E0A,
        0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000B0F0A, 0x000B0F0A, 0x000F130E, 0x00262A25, 0x00363A35,
        0x00363A35, 0x00383C37, 0x00353934, 0x002B312C, 0x003C423D, 0x00363C37, 0x00333934, 0x00363C37, 0x002D332E, 0x00454B46, 0x8AB3B9B4, 0xFFFBFFFE, 0xFFFBFFFE, 0xFFF0F5F3, 0xFFFDFFFE, 0xFFFDFFFE, 0xFFF7F8F6, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBFBFB, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF,
        0xFFFFFDFE, 0xFFFFFDFD, 0xFFFFFDFD, 0xFFFDFDFD, 0xFFFEFEFE, 0xFFFDFFFF, 0xFFFDFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFEFEFE, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFFFC, 0xFFFDFFFC, 0xFFFCFFFB, 0xFFEDF1EC, 0xC9D2D6D1, 0x71A9ADA8, 0x04747873, 0x004C504B, 0x00242521, 0x001A1B17, 0x000E0F0B, 0x00070804, 0x00080905, 0x000B0C08, 0x000B0C08, 0x000B0C08, 0x000D0E0A,
        0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000D0E0A, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000C0D09, 0x000D0E0A, 0x000D0E0A, 0x000B0F0A, 0x000B0F0A, 0x000F130E, 0x00252924, 0x00353934,
        0x00363A35, 0x00383C37, 0x00343833, 0x00353B36, 0x002F3530, 0x00363C37, 0x002F3530, 0x00252B26, 0x00616762, 0x8AB4BAB5, 0xFFFBFFFC, 0xFFF6FBF9, 0xFFF9FEFC, 0xFFFBFFFE, 0xFFFDFFFE, 0xFFF7FAF8, 0xFFFFFFFE, 0xFFFEFFFD, 0xFFFEFFFD, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBFFFF, 0xFFFDFFFF, 0xFFFDFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xFFFFFCFD, 0xFFFFFEFF,
        0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF7F7F7, 0xFFF0F0F0, 0x9FC1C2BE, 0x4B999A96, 0x00656662, 0x003B3C38, 0x0021221E, 0x0011120E, 0x000B0C08, 0x000C0D09, 0x000C0D09,
        0x00070804, 0x00090A06, 0x0011120E, 0x0011120E, 0x000A0B07, 0x000A0B07, 0x000F100C, 0x00090B05, 0x000D0F09, 0x000F110B, 0x000B0D07, 0x000B0D07, 0x000E100A, 0x000E100A, 0x000B0D07, 0x00080C07, 0x000A0E09, 0x000F130E, 0x00222621, 0x00353934,
        0x00333732, 0x0030342F, 0x00383C37, 0x00343533, 0x003B3C3A, 0x002D2E2C, 0x002D2E2C, 0x04747573, 0xDEDCDDDB, 0xFFFFFFFE, 0xFFFBFCFA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFCFCFC, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFF9F9F9, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFF2F3EF, 0xD9DCDDD9, 0x8EB9BAB6, 0x2E8A8B87, 0x00565753, 0x00343531, 0x001A1B17,
        0x000D0E0A, 0x00040501, 0x00080905, 0x000F100C, 0x0010110D, 0x000E0F0B, 0x000C0D09, 0x0011130D, 0x000D0F09, 0x00070903, 0x00060802, 0x000B0D07, 0x000F110B, 0x000E100A, 0x000A0C06, 0x000E120D, 0x00010500, 0x000D110C, 0x00272B26, 0x00313530,
        0x00343833, 0x00373B36, 0x00353934, 0x00313230, 0x00292A28, 0x0040413F, 0x4B969795, 0xFFF1F2F0, 0xFFFFFFFE, 0xFFFAFBF9, 0xFFFFFFFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFDFDFD, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFAFBF7, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFFFFFFC, 0xFFF8F9F5, 0xEEE6E7E3, 0xC9D5D6D2, 0x589FA09C,
        0x157F807C, 0x004F504C, 0x00262723, 0x000F100C, 0x00080905, 0x000A0B07, 0x000C0D09, 0x00070804, 0x000C0D09, 0x0011120E, 0x0012130F, 0x000D0E0A, 0x000A0B07, 0x000E0F0B, 0x00141511, 0x000D110C, 0x000B0F0A, 0x00191D18, 0x00282C27, 0x002E322D,
        0x00323631, 0x00353934, 0x002F332E, 0x00272826, 0x00595A58, 0x8AB5B6B4, 0xFFFBFCFA, 0xFFFFFFFE, 0xFFFDFEFC, 0xFFFDFEFC, 0xFFFDFEFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFFFD, 0xFFFFFFFE, 0xFFFDFEFC, 0xFFFBFCFA, 0xFFFDFEFC, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFEFFFD,
        0xFFF6F7F5, 0xE6E1E2E0, 0xA3C0C1BF, 0x4F989997, 0x006C6D6B, 0x003E3F3D, 0x001F201E, 0x0011120E, 0x00090A06, 0x00050602, 0x000A0B07, 0x000F100C, 0x000E0F0B, 0x000B0C08, 0x000A0B07, 0x00111510, 0x000B0F0A, 0x00070B06, 0x001B1F1A, 0x00363A35,
        0x00323631, 0x00252924, 0x002C302B, 0x04747474, 0xD1D5D5D5, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFAFAFA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFDFDFD, 0xFFFEFEFE, 0xFFFFFFFE, 0xFFFEFFFD, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFEFFFD, 0xFFFFFFFE, 0xFFFFFFFE,
        0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFF3F4F2, 0xD1D6D7D5, 0x96BBBCBA, 0x21858682, 0x0061625E, 0x00353632, 0x00181915, 0x000C0D09, 0x00090A06, 0x000A0B07, 0x000C0D09, 0x00090D08, 0x000C100B, 0x00151914, 0x00272B26, 0x002B2F2A,
        0x0020241F, 0x00464A45, 0x3F90948F, 0xFFF5F5F5, 0xFFF6F6F6, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF8F8F8, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBFBFB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFCFCFC, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFCFCFC, 0xFFFBFBFB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFCFC, 0xFFFFFFFE, 0xFFEDEEEC, 0xC0CFD0CE, 0x75ABACAA, 0x21828381, 0x00545553, 0x002B2C2A, 0x00141513, 0x00080C07, 0x00070B06, 0x000C100B, 0x00141813, 0x00242823,
        0x00555954, 0x82B0B4AF, 0xFFFDFFFC, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFF9F8FA, 0xFFFAF9FB, 0xFFFFFEFF, 0xFFFBFAFC, 0xFFF1F0F2, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFCFCFC, 0xFFF7F7F7,
        0xFFFCFCFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFDFDFD, 0xFFFCFCFC, 0xFFFDFDFD, 0xFFFEFEFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFCFDFB, 0xF2E7E8E6, 0xA7C2C3C1, 0x69A4A5A3, 0x006A6E69, 0x00454944, 0x001E221D, 0x002B2F2A, 0x0D787C77,
        0xC4D0D4CF, 0xFFFDFFFC, 0xFFFDFFFC, 0xFFFAF9FB, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFDFCFE, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFAF9FB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFAFAFA, 0xFFFBFBFB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBFCFA, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFEFFFD, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFFFFFE, 0xFFFBFCFA, 0xFFFAFEF9, 0xEAE2E6E1, 0x96BBBFBA, 0x9FBFC3BE, 0xFFEFF3EE,
        0xFFFDFFFC, 0xFFFBFFFA, 0xFFFCFFFB, 0xFFFDFCFE, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFF9F8FA, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFDFCFE, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF
};

static GUI_CONST_STORAGE GUI_BITMAP _bmJLink_300x132 = {
  300,                                // XSize
  132,                                // YSize
  1200,                               // BytesPerLine
  32,                                 // BitsPerPixel
  (unsigned char *)_acJLink_300x132,  // Pointer to picture data
  NULL,                               // Pointer to palette
  GUI_DRAW_BMP8888
};
#endif

/*********************************************************************
*
*       _bmJTrace_300x164
*/
#if (GUI_USE_ARGB)
static GUI_CONST_STORAGE unsigned long _acJTrace_300x164[] = {
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF718583, 0xFF93A7A5, 0xFF94A8A6, 0xFF8DA19F, 0xFF8A9E9C, 0xFF839795, 0xFF8A9E9C, 0xFF889C9A, 0xFF708D89, 0xFF718E8A, 0xFF73908C, 0xFF76938F, 0xFF799692,
        0xFF799692, 0xFF779490, 0xFF74918D, 0xFF6B8883, 0xFF6D8A85, 0xFF6E8B86, 0xFF6C8984, 0xFF698681, 0xFF688580, 0xFF6A8782, 0xFF6D8A85, 0xFF6B8A85, 0xFF698883, 0xFF6C8B86, 0xFF688782, 0xFF65847F, 0xFF65847F, 0xFF5F7E79, 0xFF61807B, 0xFF57746F,
        0xFF587570, 0xFF55726D, 0xFF54716C, 0xFF5A7772, 0xFF5E7B76, 0xFF5D7A75, 0xFF5C7974, 0xFF637873, 0xFF647974, 0xFF596E69, 0xFF586C6A, 0xFF566A68, 0xFF546866, 0xFF566A69, 0xFF485C5B, 0xFF525E5E, 0xFF535F5F, 0xFF566262, 0xFF586464, 0xFF566262,
        0xFF535F5F, 0xFF556161, 0xFF596565, 0xFF5C6C6B, 0xFF5B6B6A, 0xFF556564, 0xFF4E5C5C, 0xFF515F5F, 0xFF566464, 0xFF505C5C, 0xFF455151, 0xFF5A7570, 0xFF57726D, 0xFF516C67, 0xFF58736E, 0xFF5A7570, 0xFF506B66, 0xFF4B6661, 0xFF46615C, 0xFF425D58,
        0xFF425D58, 0xFF415C57, 0xFF3F5A55, 0xFF3E5954, 0xFF405B56, 0xFF415C57, 0xFF405B56, 0xFF35504B, 0xFF3A5550, 0xFF3F5A55, 0xFF3D5853, 0xFF36514C, 0xFF314C47, 0xFF324D48, 0xFF344F4A, 0xFF3A4A49, 0xFF384847, 0xFF384847, 0xFF3C4C4B, 0xFF3F4F4E,
        0xFF3F4F4E, 0xFF3E4E4D, 0xFF3D4D4C, 0xFF38534E, 0xFF3C5752, 0xFF3D5853, 0xFF3F5A55, 0xFF425D58, 0xFF3E5954, 0xFF39544F, 0xFF3A5550, 0xFF3D514F, 0xFF3D514F, 0xFF3D514F, 0xFF3B4F4D, 0xFF374B49, 0xFF334745, 0xFF304442, 0xFF2F4341, 0xFF354846,
        0xFF334644, 0xFF314442, 0xFF304341, 0xFF314442, 0xFF324543, 0xFF334644, 0xFF334644, 0xFF2E4240, 0xFF2C403E, 0xFF2B3F3D, 0xFF2C403E, 0xFF2C403E, 0xFF293D3B, 0xFF273B39, 0xFF283C3A, 0xFF2E413F, 0xFF293C3A, 0xFF253836, 0xFF263937, 0xFF273A38,
        0xFF263937, 0xFF243735, 0xFF233634, 0xFF2B3F3D, 0xFF283C3A, 0xFF233735, 0xFF213533, 0xFF203432, 0xFF1F3331, 0xFF1D312F, 0xFF1B2F2D, 0xFF212A29, 0xFF212A29, 0xFF212A29, 0xFF222B2A, 0xFF222B2A, 0xFF222B2A, 0xFF222B2A, 0xFF232C2B, 0xFF2A3332,
        0xFF212A29, 0xFF1C2524, 0xFF1F2827, 0xFF212A29, 0xFF1E2726, 0xFF1B2423, 0xFF1B2423, 0xFF1F2928, 0xFF1E2827, 0xFF1D2726, 0xFF1C2625, 0xFF1D2726, 0xFF1C2625, 0xFF1B2524, 0xFF192322, 0xFF17201F, 0xFF161F1E, 0xFF151E1D, 0xFF161F1E, 0xFF182120,
        0xFF192221, 0xFF1A2322, 0xFF1A2322, 0xFF222B2A, 0xFF202928, 0xFF1E2726, 0xFF1B2423, 0xFF1A2322, 0xFF1A2322, 0xFF1B2423, 0xFF1B2423, 0xFF1A2322, 0xFF1A2322, 0xFF1A2322, 0xFF1A2322, 0xFF1A2322, 0xFF1B2423, 0xFF1B2423, 0xFF1B2423, 0xFF1A2322,
        0xFF1B2423, 0xFF1C2524, 0xFF1D2625, 0xFF1E2726, 0xFF1F2827, 0xFF1F2827, 0xFF1F2827, 0xFF242A28, 0xFF232927, 0xFF202624, 0xFF1E2422, 0xFF1E2422, 0xFF1E2422, 0xFF1F2523, 0xFF202624, 0xFF222B2A, 0xFF212A29, 0xFF212A29, 0xFF202928, 0xFF202928,
        0xFF212A29, 0xFF212A29, 0xFF222B2A, 0xFF252F2E, 0xFF26302F, 0xFF273130, 0xFF293332, 0xFF2B3534, 0xFF2D3736, 0xFF2E3837, 0xFF2F3938, 0xFF364041, 0xFF2F393A, 0xFF354141, 0xFF404C4C, 0xFF394747, 0xFF2F3D3D, 0xFF2D3B3B, 0xFF2F3F3E, 0xFF2D383A,
        0xFF2D383A, 0xFF2C3739, 0xFF2D383A, 0xFF2E393B, 0xFF2F3A3C, 0xFF2F3A3C, 0xFF2E393B, 0xFF334343, 0xFF2E3C3D, 0xFF303E3F, 0xFF313F40, 0xFF384345, 0xFF313C3E, 0xFF232D2F, 0xFF323C3E, 0xFF1F2E2B, 0xFF1E2D2A, 0xFF1D2C29, 0xFF1E2D2A, 0xFF1F2E2B,
        0xFF202F2C, 0xFF202F2C, 0xFF202F2C, 0xFF1B2A27, 0xFF1C2B28, 0xFF1D2C29, 0xFF1F2E2B, 0xFF21302D, 0xFF21302D, 0xFF202F2C, 0xFF1F2E2B, 0xFF27302F, 0xFF262F2E, 0xFF242D2C, 0xFF232C2B, 0xFF232C2B, 0xFF252E2D, 0xFF27302F, 0xFF293231, 0xFF2E3638,
        0xFF30383A, 0xFF2F3739, 0xFF2A3234, 0xFF283032, 0xFF2B3335, 0xFF2E3638, 0xFF2F3739, 0xFF333D3C, 0xFF353F3E, 0xFF353F3E, 0xFF313B3A, 0xFF313B3A, 0xFF374140, 0xFF394342, 0xFF394342, 0xFF515D59, 0xFF9BA6A2, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FCFEFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF777D7B, 0xFF5C706E, 0xFF607472, 0xFF405452, 0xFF203432, 0xFF213533, 0xFF283C3A, 0xFF293D3B, 0xFF263A38, 0xFF344245, 0xFF334144, 0xFF324043, 0xFF334144, 0xFF344245,
        0xFF334144, 0xFF2F3D40, 0xFF2B393C, 0xFF354343, 0xFF374545, 0xFF384646, 0xFF364444, 0xFF324040, 0xFF303E3E, 0xFF324040, 0xFF354343, 0xFF384848, 0xFF334343, 0xFF384848, 0xFF3A4A4A, 0xFF415151, 0xFF475757, 0xFF415151, 0xFF3F4F4F, 0xFF3D4B4B,
        0xFF3C4A4A, 0xFF3B4949, 0xFF3D4B4B, 0xFF435151, 0xFF485656, 0xFF495757, 0xFF485656, 0xFF525C5B, 0xFF505A59, 0xFF4A5453, 0xFF6A7475, 0xFF7A8485, 0xFF697374, 0xFF667072, 0xFF5D6769, 0xFF616F6F, 0xFF637171, 0xFF687676, 0xFF6C7A7A, 0xFF6C7A7A,
        0xFF6A7878, 0xFF6E7C7C, 0xFF738181, 0xFF738785, 0xFF6D817F, 0xFF687C7A, 0xFF647775, 0xFF6A7D7B, 0xFF697978, 0xFF516160, 0xFF415150, 0xFF555F60, 0xFF535D5E, 0xFF465051, 0xFF454F50, 0xFF485253, 0xFF444E4F, 0xFF454F50, 0xFF434D4E, 0xFF444E4F,
        0xFF434D4E, 0xFF424C4D, 0xFF404A4B, 0xFF404A4B, 0xFF414B4C, 0xFF434D4E, 0xFF434D4E, 0xFF3A4445, 0xFF3D4748, 0xFF3F494A, 0xFF3E4849, 0xFF3B4546, 0xFF394344, 0xFF3A4445, 0xFF3D4748, 0xFF414548, 0xFF3C4043, 0xFF393D40, 0xFF3A3E41, 0xFF3D4144,
        0xFF3E4245, 0xFF3E4245, 0xFF3D4144, 0xFF394344, 0xFF374142, 0xFF353F40, 0xFF374142, 0xFF384243, 0xFF353F40, 0xFF313B3C, 0xFF313B3C, 0xFF2E3E3D, 0xFF2E3E3D, 0xFF2F3F3E, 0xFF2F3F3E, 0xFF2E3E3D, 0xFF2D3D3C, 0xFF2C3C3B, 0xFF2B3B3A, 0xFF263434,
        0xFF243232, 0xFF223030, 0xFF223030, 0xFF243232, 0xFF263434, 0xFF273535, 0xFF273535, 0xFF263937, 0xFF253836, 0xFF243735, 0xFF253836, 0xFF253836, 0xFF243735, 0xFF223533, 0xFF213432, 0xFF273535, 0xFF233131, 0xFF1F2D2D, 0xFF1D2B2B, 0xFF1D2B2B,
        0xFF1D2B2B, 0xFF1D2B2B, 0xFF1E2C2C, 0xFF192C2A, 0xFF172A28, 0xFF152826, 0xFF162927, 0xFF182B29, 0xFF192C2A, 0xFF172A28, 0xFF142725, 0xFF1B2423, 0xFF1B2423, 0xFF1C2524, 0xFF1C2524, 0xFF1C2524, 0xFF1C2524, 0xFF1C2524, 0xFF1C2524, 0xFF1C2524,
        0xFF141D1C, 0xFF111A19, 0xFF161F1E, 0xFF182120, 0xFF141D1C, 0xFF141D1C, 0xFF192221, 0xFF192825, 0xFF172623, 0xFF162522, 0xFF162522, 0xFF172623, 0xFF172623, 0xFF152421, 0xFF142320, 0xFF1C2524, 0xFF1C2524, 0xFF1A2322, 0xFF1A2322, 0xFF1A2322,
        0xFF1A2322, 0xFF1A2322, 0xFF1A2322, 0xFF1A2322, 0xFF182120, 0xFF161F1E, 0xFF141D1C, 0xFF131C1B, 0xFF131C1B, 0xFF131C1B, 0xFF131C1B, 0xFF121B1A, 0xFF121B1A, 0xFF131C1B, 0xFF131C1B, 0xFF131C1B, 0xFF131C1B, 0xFF131C1B, 0xFF141D1C, 0xFF121B1A,
        0xFF121B1A, 0xFF131C1B, 0xFF141D1C, 0xFF151E1D, 0xFF161F1E, 0xFF17201F, 0xFF17201F, 0xFF1C2220, 0xFF1B211F, 0xFF191F1D, 0xFF181E1C, 0xFF171D1B, 0xFF171D1B, 0xFF181E1C, 0xFF191F1D, 0xFF182120, 0xFF182120, 0xFF17201F, 0xFF17201F, 0xFF17201F,
        0xFF182120, 0xFF182120, 0xFF192221, 0xFF1B2524, 0xFF1B2524, 0xFF1C2625, 0xFF1D2726, 0xFF1E2827, 0xFF1F2928, 0xFF1F2928, 0xFF1F2928, 0xFF0F1D1E, 0xFF2A3839, 0xFF566465, 0xFF748484, 0xFF788888, 0xFF728484, 0xFF708282, 0xFF6E8080, 0xFF6E7D80,
        0xFF6E7D80, 0xFF6F7E81, 0xFF707F82, 0xFF728184, 0xFF738285, 0xFF748386, 0xFF748386, 0xFF6E8082, 0xFF6F8183, 0xFF788A8C, 0xFF768588, 0xFF738285, 0xFF4F5D60, 0xFF132124, 0xFF0C1A1D, 0xFF1F2022, 0xFF1E1F21, 0xFF1C1D1F, 0xFF1B1C1E, 0xFF1B1C1E,
        0xFF1C1D1F, 0xFF1C1D1F, 0xFF1C1D1F, 0xFF161719, 0xFF141517, 0xFF121315, 0xFF131416, 0xFF151618, 0xFF18191B, 0xFF191A1C, 0xFF191A1C, 0xFF131C1B, 0xFF111A19, 0xFF0F1817, 0xFF0D1615, 0xFF0D1615, 0xFF0E1716, 0xFF101918, 0xFF121B1A, 0xFF111A19,
        0xFF151E1D, 0xFF141D1C, 0xFF0E1716, 0xFF091211, 0xFF07100F, 0xFF07100F, 0xFF060F0E, 0xFF010B0A, 0xFF091312, 0xFF0A1413, 0xFF091312, 0xFF0A1413, 0xFF0F1918, 0xFF172120, 0xFF182221, 0xFF23342E, 0xFF333F3B, 0xFFB2BDB9, 0x00FBFFFF,
  0x00FFFFFF, 0x00FCFCFC, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF3A4541, 0xFF6A7475, 0xFF0B1516, 0xFF000304, 0xFF091314, 0xFF182223, 0xFF313B3C, 0xFF394344, 0xFF384243, 0xFF383D43, 0xFF373C42, 0xFF353A40, 0xFF34393F, 0xFF33383E,
        0xFF32373D, 0xFF2F343A, 0xFF2D3238, 0xFF272C2F, 0xFF292E31, 0xFF2A2F32, 0xFF282D30, 0xFF24292C, 0xFF212629, 0xFF22272A, 0xFF23282B, 0xFF20282B, 0xFF1A2225, 0xFF1B2326, 0xFF1A2225, 0xFF1C2427, 0xFF1D2528, 0xFF141C1F, 0xFF121A1D, 0xFF181D20,
        0xFF14191C, 0xFF12171A, 0xFF13181B, 0xFF151A1D, 0xFF161B1E, 0xFF161B1E, 0xFF14191C, 0xFF0A1413, 0xFF0A1413, 0xFF0E1817, 0xFF454F50, 0xFF707A7B, 0xFF6D7778, 0xFF6F797B, 0xFF707A7C, 0xFF677576, 0xFF677576, 0xFF687677, 0xFF697778, 0xFF657374,
        0xFF616F70, 0xFF616F70, 0xFF657374, 0xFF607673, 0xFF607472, 0xFF657977, 0xFF667A78, 0xFF687B79, 0xFF586B69, 0xFF3D4D4C, 0xFF374746, 0xFF000105, 0xFF131418, 0xFF121317, 0xFF0E0F13, 0xFF0D0E12, 0xFF0B0C10, 0xFF0F1014, 0xFF101115, 0xFF111216,
        0xFF101115, 0xFF0E0F13, 0xFF0E0F13, 0xFF0F1014, 0xFF101115, 0xFF111216, 0xFF131418, 0xFF111216, 0xFF101115, 0xFF0E0F13, 0xFF0E0F13, 0xFF0F1014, 0xFF111216, 0xFF111216, 0xFF121317, 0xFF111619, 0xFF0F1417, 0xFF0C1114, 0xFF0C1114, 0xFF0E1316,
        0xFF0F1417, 0xFF0D1215, 0xFF0A0F12, 0xFF090A0E, 0xFF0C0D11, 0xFF111216, 0xFF131418, 0xFF101115, 0xFF0E0F13, 0xFF0E0F13, 0xFF0E0F13, 0xFF091113, 0xFF091113, 0xFF081012, 0xFF081012, 0xFF091113, 0xFF091113, 0xFF0A1214, 0xFF0A1214, 0xFF0E1316,
        0xFF0B1013, 0xFF090E11, 0xFF080D10, 0xFF0A0F12, 0xFF0B1013, 0xFF0B1013, 0xFF0A0F12, 0xFF030D0E, 0xFF020C0D, 0xFF020C0D, 0xFF020C0D, 0xFF040E0F, 0xFF050F10, 0xFF030D0E, 0xFF010B0C, 0xFF070C0F, 0xFF0B1013, 0xFF0E1316, 0xFF0E1316, 0xFF0C1114,
        0xFF0B1013, 0xFF0E1316, 0xFF101518, 0xFF091314, 0xFF061011, 0xFF020C0D, 0xFF020C0D, 0xFF040E0F, 0xFF061011, 0xFF050F10, 0xFF040E0F, 0xFF050F0E, 0xFF050F0E, 0xFF050F0E, 0xFF050F0E, 0xFF050F0E, 0xFF050F0E, 0xFF050F0E, 0xFF050F0E, 0xFF06100F,
        0xFF010B0A, 0xFF020C0B, 0xFF06100F, 0xFF050F0E, 0xFF000A09, 0xFF030D0C, 0xFF0C1615, 0xFF02120F, 0xFF01110E, 0xFF00100D, 0xFF00100D, 0xFF02120F, 0xFF02120F, 0xFF01110E, 0xFF00100D, 0xFF040E0D, 0xFF040E0D, 0xFF030D0C, 0xFF020C0B, 0xFF010B0A,
        0xFF010B0A, 0xFF010B0A, 0xFF010B0A, 0xFF040E0D, 0xFF040E0D, 0xFF030D0C, 0xFF030D0C, 0xFF020C0B, 0xFF010B0A, 0xFF010B0A, 0xFF000A09, 0xFF010B0A, 0xFF010B0A, 0xFF010B0A, 0xFF010B0A, 0xFF020C0B, 0xFF020C0B, 0xFF020C0B, 0xFF020C0B, 0xFF030D0C,
        0xFF030D0C, 0xFF030D0C, 0xFF030D0C, 0xFF030D0C, 0xFF040E0D, 0xFF040E0D, 0xFF050F0E, 0xFF030C09, 0xFF040D0A, 0xFF040D0A, 0xFF040D0A, 0xFF040D0A, 0xFF040D0A, 0xFF040D0A, 0xFF040D0A, 0xFF020C0B, 0xFF020C0B, 0xFF010B0A, 0xFF010B0A, 0xFF020C0B,
        0xFF020C0B, 0xFF030D0C, 0xFF040E0D, 0xFF000908, 0xFF000908, 0xFF010B0A, 0xFF020C0B, 0xFF020C0B, 0xFF020C0B, 0xFF020C0B, 0xFF020C0B, 0xFF0A1819, 0xFF192728, 0xFF334343, 0xFF485858, 0xFF495B5B, 0xFF485A5A, 0xFF485C5B, 0xFF475B5A, 0xFF4E5D60,
        0xFF4E5D60, 0xFF4E5D60, 0xFF4D5C5F, 0xFF4D5C5F, 0xFF4C5B5E, 0xFF4C5B5E, 0xFF4C5B5E, 0xFF4C6061, 0xFF495D5E, 0xFF4A5C5E, 0xFF47595B, 0xFF546366, 0xFF415053, 0xFF0C1A1D, 0xFF041215, 0xFF0D0E10, 0xFF0D0E10, 0xFF0D0E10, 0xFF0D0E10, 0xFF0D0E10,
        0xFF0E0F11, 0xFF0F1012, 0xFF101113, 0xFF141517, 0xFF121315, 0xFF101113, 0xFF111214, 0xFF131416, 0xFF161719, 0xFF17181A, 0xFF17181A, 0xFF151F1E, 0xFF151F1E, 0xFF151F1E, 0xFF151F1E, 0xFF151F1E, 0xFF16201F, 0xFF182221, 0xFF182221, 0xFF121C1B,
        0xFF1A2423, 0xFF212B2A, 0xFF232D2C, 0xFF242E2D, 0xFF283231, 0xFF2A3433, 0xFF2A3433, 0xFF2E3837, 0xFF323C3B, 0xFF303A39, 0xFF2E3837, 0xFF26302F, 0xFF192322, 0xFF101A19, 0xFF06100F, 0xFF21322C, 0xFF283732, 0xFF636E6A, 0x00F9FFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFE, 0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF374641, 0xFF3D4142, 0xFF060A0B, 0xFF25292A, 0xFF4C5051, 0xFF626667, 0xFF818586, 0xFF848889, 0xFF868A8B, 0xFF828E8E, 0xFF828E8E, 0xFF828E8E, 0xFF828E8E, 0xFF828E8E,
        0xFF828E8E, 0xFF828E8E, 0xFF838F8F, 0xFF7E8D8A, 0xFF7F8E8B, 0xFF808F8C, 0xFF7F8E8B, 0xFF7D8C89, 0xFF7B8A87, 0xFF7A8986, 0xFF7A8986, 0xFF798989, 0xFF798989, 0xFF7F8F8F, 0xFF7A8A8A, 0xFF768686, 0xFF768686, 0xFF728282, 0xFF768686, 0xFF7A8888,
        0xFF738181, 0xFF707E7E, 0xFF707E7E, 0xFF6D7B7B, 0xFF697777, 0xFF677575, 0xFF627070, 0xFF617872, 0xFF5D746E, 0xFF4B625C, 0xFF5B716E, 0xFF738986, 0xFF6E8481, 0xFF677D7B, 0xFF647A78, 0xFF6E7E7E, 0xFF6C7C7C, 0xFF6B7B7B, 0xFF697979, 0xFF657575,
        0xFF5E6E6E, 0xFF5C6C6C, 0xFF5D6D6D, 0xFF637775, 0xFF5D716F, 0xFF627573, 0xFF627573, 0xFF647473, 0xFF4E5E5D, 0xFF283837, 0xFF253333, 0xFF313D3D, 0xFF5B6767, 0xFF707C7C, 0xFF727E7E, 0xFF6D7979, 0xFF657171, 0xFF667272, 0xFF677373, 0xFF657171,
        0xFF636F6F, 0xFF626E6E, 0xFF636F6F, 0xFF647070, 0xFF636F6F, 0xFF647070, 0xFF667272, 0xFF687474, 0xFF647070, 0xFF606C6C, 0xFF606C6C, 0xFF626E6E, 0xFF647070, 0xFF636F6F, 0xFF616D6D, 0xFF5E7471, 0xFF5E7471, 0xFF5D7370, 0xFF5B716E, 0xFF596F6C,
        0xFF556B68, 0xFF4E6461, 0xFF475D5A, 0xFF4C5858, 0xFF576363, 0xFF626E6E, 0xFF5E6A6A, 0xFF515D5D, 0xFF4B5757, 0xFF4B5757, 0xFF495555, 0xFF4A4F52, 0xFF484D50, 0xFF454A4D, 0xFF43484B, 0xFF43484B, 0xFF43484B, 0xFF43484B, 0xFF42474A, 0xFF414548,
        0xFF3F4346, 0xFF3D4144, 0xFF3B3F42, 0xFF393D40, 0xFF373B3E, 0xFF363A3D, 0xFF34383B, 0xFF292E31, 0xFF292E31, 0xFF282D30, 0xFF282D30, 0xFF2B3033, 0xFF2E3336, 0xFF2C3134, 0xFF282D30, 0xFF2A2E31, 0xFF35393C, 0xFF3F4346, 0xFF424649, 0xFF404447,
        0xFF3F4346, 0xFF404447, 0xFF414548, 0xFF373C3F, 0xFF313639, 0xFF2A2F32, 0xFF252A2D, 0xFF24292C, 0xFF252A2D, 0xFF272C2F, 0xFF272C2F, 0xFF232D2C, 0xFF232D2C, 0xFF232D2C, 0xFF222C2B, 0xFF222C2B, 0xFF222C2B, 0xFF222C2B, 0xFF222C2B, 0xFF242E2D,
        0xFF232D2C, 0xFF252F2E, 0xFF273130, 0xFF232D2C, 0xFF1E2827, 0xFF222C2B, 0xFF2A3433, 0xFF1D2C29, 0xFF1C2B28, 0xFF1B2A27, 0xFF1C2B28, 0xFF1D2C29, 0xFF1E2D2A, 0xFF1D2C29, 0xFF1C2B28, 0xFF232D2C, 0xFF232D2C, 0xFF242E2D, 0xFF232D2C, 0xFF222C2B,
        0xFF232D2C, 0xFF242E2D, 0xFF252F2E, 0xFF242E2D, 0xFF242E2D, 0xFF242E2D, 0xFF252F2E, 0xFF242E2D, 0xFF242E2D, 0xFF232D2C, 0xFF232D2C, 0xFF242E2D, 0xFF242E2D, 0xFF242E2D, 0xFF242E2D, 0xFF242E2D, 0xFF252F2E, 0xFF252F2E, 0xFF252F2E, 0xFF293332,
        0xFF283231, 0xFF283231, 0xFF273130, 0xFF273130, 0xFF273130, 0xFF283231, 0xFF283231, 0xFF28312E, 0xFF28312E, 0xFF29322F, 0xFF2A3330, 0xFF2A3330, 0xFF2A3330, 0xFF29322F, 0xFF29322F, 0xFF2D3736, 0xFF2D3736, 0xFF2D3736, 0xFF2D3736, 0xFF2E3837,
        0xFF2E3837, 0xFF2F3938, 0xFF303A39, 0xFF2F3B39, 0xFF303C3A, 0xFF333F3D, 0xFF35413F, 0xFF364240, 0xFF374341, 0xFF374341, 0xFF364240, 0xFF333F3F, 0xFF283636, 0xFF313F3F, 0xFF485656, 0xFF536362, 0xFF536362, 0xFF4E615F, 0xFF4B5E5C, 0xFF526262,
        0xFF526262, 0xFF536363, 0xFF536363, 0xFF526262, 0xFF516161, 0xFF526262, 0xFF536363, 0xFF556767, 0xFF5B6D6D, 0xFF607070, 0xFF5C6C6C, 0xFF667475, 0xFF5C6A6B, 0xFF404E4F, 0xFF485355, 0xFF445451, 0xFF455552, 0xFF475754, 0xFF485855, 0xFF4A5A57,
        0xFF4C5C59, 0xFF4E5E5B, 0xFF50605D, 0xFF4F5F5C, 0xFF50605D, 0xFF52625F, 0xFF546461, 0xFF556562, 0xFF556562, 0xFF546461, 0xFF536360, 0xFF576160, 0xFF586261, 0xFF596362, 0xFF5C6665, 0xFF5E6867, 0xFF5F6968, 0xFF606A69, 0xFF616B6A, 0xFF5E6965,
        0xFF646F6B, 0xFF697470, 0xFF6A7571, 0xFF6A7571, 0xFF6B7672, 0xFF6B7672, 0xFF6A7571, 0xFF697372, 0xFF66706F, 0xFF656F6E, 0xFF626C6B, 0xFF5C6665, 0xFF495352, 0xFF283231, 0xFF101A19, 0xFF0A1914, 0xFF37433F, 0xFF424B48, 0x00E9EFED,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFDFE, 0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF495A54, 0xFF323B3A, 0xFF394241, 0xFF4F5857, 0xFF47504F, 0xFF47504F, 0xFF5E6766, 0xFF5F6867, 0xFF606968, 0xFF5C6D67, 0xFF5F706A, 0xFF64756F, 0xFF667771, 0xFF677872,
        0xFF687973, 0xFF6A7B75, 0xFF6D7E78, 0xFF657874, 0xFF667975, 0xFF687B77, 0xFF6B7E7A, 0xFF6F827E, 0xFF718480, 0xFF728581, 0xFF728581, 0xFF697D7C, 0xFF6B7F7E, 0xFF738786, 0xFF708483, 0xFF6D8180, 0xFF6F8382, 0xFF6F8382, 0xFF768A89, 0xFF7A8D8B,
        0xFF738684, 0xFF758886, 0xFF7B8E8C, 0xFF7A8D8B, 0xFF798C8A, 0xFF798C8A, 0xFF758886, 0xFF748984, 0xFF768B86, 0xFF607570, 0xFF556967, 0xFF647876, 0xFF6A7E7C, 0xFF667A79, 0xFF697D7C, 0xFF6D7F81, 0xFF6B7D7F, 0xFF6C7E80, 0xFF708284, 0xFF728486,
        0xFF718385, 0xFF718385, 0xFF728486, 0xFF728281, 0xFF6C7C7B, 0xFF6E7E7D, 0xFF697777, 0xFF697777, 0xFF546060, 0xFF253131, 0xFF1B2727, 0xFF3F5351, 0xFF637775, 0xFF718583, 0xFF748886, 0xFF758987, 0xFF708482, 0xFF738785, 0xFF788C8A, 0xFF788C8A,
        0xFF758987, 0xFF748886, 0xFF768A88, 0xFF758987, 0xFF728684, 0xFF728684, 0xFF748886, 0xFF647876, 0xFF637775, 0xFF617573, 0xFF617573, 0xFF607472, 0xFF607472, 0xFF5F7371, 0xFF5E7270, 0xFF5E7471, 0xFF617774, 0xFF637976, 0xFF627875, 0xFF627875,
        0xFF637976, 0xFF607673, 0xFF5B716E, 0xFF627674, 0xFF677B79, 0xFF687C7A, 0xFF5D716F, 0xFF506462, 0xFF546866, 0xFF596D6B, 0xFF546866, 0xFF626E6E, 0xFF606C6C, 0xFF5E6A6A, 0xFF5D6969, 0xFF5D6969, 0xFF5D6969, 0xFF5B6767, 0xFF5A6666, 0xFF596364,
        0xFF596364, 0xFF596364, 0xFF576162, 0xFF535D5E, 0xFF505A5B, 0xFF4E5859, 0xFF4E5859, 0xFF4B5556, 0xFF4B5556, 0xFF4A5455, 0xFF495354, 0xFF4C5657, 0xFF4F595A, 0xFF4D5758, 0xFF485253, 0xFF3F494A, 0xFF485253, 0xFF4E5859, 0xFF4F595A, 0xFF4E5859,
        0xFF4F595A, 0xFF4F595A, 0xFF4E5859, 0xFF4F595A, 0xFF4C5657, 0xFF495354, 0xFF475152, 0xFF475152, 0xFF495354, 0xFF4C5657, 0xFF4E5859, 0xFF495352, 0xFF495352, 0xFF495352, 0xFF485251, 0xFF485251, 0xFF485251, 0xFF475150, 0xFF475150, 0xFF424C4B,
        0xFF434D4C, 0xFF444E4D, 0xFF434D4C, 0xFF414B4A, 0xFF3F4948, 0xFF414B4A, 0xFF434D4C, 0xFF444E4D, 0xFF424C4B, 0xFF414B4A, 0xFF424C4B, 0xFF434D4C, 0xFF434D4C, 0xFF424C4B, 0xFF414B4A, 0xFF404A49, 0xFF414B4A, 0xFF424C4B, 0xFF414B4A, 0xFF404A49,
        0xFF404A49, 0xFF424C4B, 0xFF434D4C, 0xFF434D4C, 0xFF434D4C, 0xFF424C4B, 0xFF424C4B, 0xFF424C4B, 0xFF424C4B, 0xFF424C4B, 0xFF424C4B, 0xFF424C4B, 0xFF424C4B, 0xFF424C4B, 0xFF424C4B, 0xFF434D4C, 0xFF434D4C, 0xFF434D4C, 0xFF434D4C, 0xFF46504F,
        0xFF46504F, 0xFF454F4E, 0xFF444E4D, 0xFF444E4D, 0xFF454F4E, 0xFF46504F, 0xFF46504F, 0xFF444F4B, 0xFF434E4A, 0xFF434E4A, 0xFF424D49, 0xFF424D49, 0xFF424D49, 0xFF434E4A, 0xFF434E4A, 0xFF444E4D, 0xFF444E4D, 0xFF444E4D, 0xFF444E4D, 0xFF454F4E,
        0xFF46504F, 0xFF475150, 0xFF485251, 0xFF495553, 0xFF495553, 0xFF4A5654, 0xFF495553, 0xFF485452, 0xFF45514F, 0xFF434F4D, 0xFF414D4B, 0xFF38433F, 0xFF293430, 0xFF25312D, 0xFF2F3B37, 0xFF34433E, 0xFF3A4944, 0xFF3F4E49, 0xFF3F4E49, 0xFF414D4B,
        0xFF434F4D, 0xFF45514F, 0xFF45514F, 0xFF44504E, 0xFF44504E, 0xFF465250, 0xFF475351, 0xFF445350, 0xFF485754, 0xFF455451, 0xFF41504D, 0xFF434F4D, 0xFF374341, 0xFF343E3D, 0xFF485251, 0xFF435652, 0xFF445753, 0xFF455854, 0xFF445753, 0xFF425551,
        0xFF40534F, 0xFF40534F, 0xFF415450, 0xFF415450, 0xFF425551, 0xFF425551, 0xFF415450, 0xFF3F524E, 0xFF3C4F4B, 0xFF394C48, 0xFF384B47, 0xFF404A49, 0xFF3F4948, 0xFF3F4948, 0xFF3D4746, 0xFF3C4645, 0xFF3B4544, 0xFF3A4443, 0xFF3A4443, 0xFF36413D,
        0xFF38433F, 0xFF36413D, 0xFF323D39, 0xFF303B37, 0xFF323D39, 0xFF323D39, 0xFF303B37, 0xFF2B3534, 0xFF252F2E, 0xFF283231, 0xFF252F2E, 0xFF323C3B, 0xFF3E4847, 0xFF2A3433, 0xFF1A2423, 0xFF1E2A26, 0xFF313C38, 0xFF303634, 0x00D6DAD9,
  0x00FFFFFF, 0x00FEFCFD, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF50615B, 0xFF1E2D2A, 0xFF23322F, 0xFF42514E, 0xFF3E4D4A, 0xFF0E1D1A, 0xFF000A07, 0xFF03120F, 0xFF03120F, 0xFF010B03, 0xFF071109, 0xFF0E1810, 0xFF121C14, 0xFF131D15,
        0xFF141E16, 0xFF172119, 0xFF1A241C, 0xFF192322, 0xFF182221, 0xFF192322, 0xFF1C2625, 0xFF1F2928, 0xFF212B2A, 0xFF202A29, 0xFF1E2827, 0xFF1E292B, 0xFF1C2729, 0xFF212C2E, 0xFF1F2A2C, 0xFF202B2D, 0xFF232E30, 0xFF212C2E, 0xFF253032, 0xFF2C3637,
        0xFF252F30, 0xFF2A3435, 0xFF323C3D, 0xFF323C3D, 0xFF333D3E, 0xFF343E3F, 0xFF303A3B, 0xFF35393A, 0xFF3C4041, 0xFF2C3031, 0xFF1D2124, 0xFF292D30, 0xFF313538, 0xFF303338, 0xFF3E4146, 0xFF384A4C, 0xFF354749, 0xFF354749, 0xFF394B4D, 0xFF3C4E50,
        0xFF3C4E50, 0xFF3B4D4F, 0xFF3B4D4F, 0xFF394747, 0xFF3A4848, 0xFF3D4B4B, 0xFF313D3D, 0xFF313D3D, 0xFF2B3536, 0xFF101A1B, 0xFF151F20, 0xFF0E1E1D, 0xFF233332, 0xFF263635, 0xFF2D3D3C, 0xFF374746, 0xFF334342, 0xFF30403F, 0xFF334342, 0xFF324241,
        0xFF2F3F3E, 0xFF2E3E3D, 0xFF30403F, 0xFF2E3E3D, 0xFF2A3A39, 0xFF283837, 0xFF2A3A39, 0xFF253534, 0xFF283837, 0xFF2B3B3A, 0xFF2A3A39, 0xFF283837, 0xFF253534, 0xFF253534, 0xFF263635, 0xFF2A3435, 0xFF2C3637, 0xFF2A3435, 0xFF252F30, 0xFF242E2F,
        0xFF283233, 0xFF293334, 0xFF283233, 0xFF223231, 0xFF233332, 0xFF243433, 0xFF1F2F2E, 0xFF213130, 0xFF2E3E3D, 0xFF30403F, 0xFF213130, 0xFF1C322F, 0xFF1B312E, 0xFF1C322F, 0xFF1E3431, 0xFF203633, 0xFF213734, 0xFF1F3532, 0xFF1D3330, 0xFF192D2B,
        0xFF1B2F2D, 0xFF1D312F, 0xFF1C302E, 0xFF192D2B, 0xFF172B29, 0xFF162A28, 0xFF162A28, 0xFF162625, 0xFF162625, 0xFF152524, 0xFF132322, 0xFF162625, 0xFF192928, 0xFF162625, 0xFF10201F, 0xFF132725, 0xFF142826, 0xFF122624, 0xFF0D211F, 0xFF0C201E,
        0xFF0F2321, 0xFF112523, 0xFF0F2321, 0xFF112120, 0xFF122221, 0xFF132322, 0xFF132322, 0xFF132322, 0xFF112120, 0xFF10201F, 0xFF10201F, 0xFF111D1B, 0xFF101C1A, 0xFF101C1A, 0xFF101C1A, 0xFF0F1B19, 0xFF0F1B19, 0xFF0E1A18, 0xFF0E1A18, 0xFF0C1816,
        0xFF0C1816, 0xFF0C1816, 0xFF0C1816, 0xFF0D1917, 0xFF0E1A18, 0xFF0E1A18, 0xFF0C1816, 0xFF0D1615, 0xFF0C1514, 0xFF0A1312, 0xFF0A1312, 0xFF0B1413, 0xFF0A1312, 0xFF091211, 0xFF081110, 0xFF0A1614, 0xFF0B1715, 0xFF0A1614, 0xFF081412, 0xFF061210,
        0xFF05110F, 0xFF061210, 0xFF071311, 0xFF0B1715, 0xFF0A1614, 0xFF081412, 0xFF061210, 0xFF05110F, 0xFF061210, 0xFF071311, 0xFF081412, 0xFF071311, 0xFF071311, 0xFF071311, 0xFF071311, 0xFF071311, 0xFF081412, 0xFF081412, 0xFF081412, 0xFF071311,
        0xFF071311, 0xFF061210, 0xFF061210, 0xFF061210, 0xFF071311, 0xFF081412, 0xFF091513, 0xFF0B1612, 0xFF091410, 0xFF07120E, 0xFF05100C, 0xFF05100C, 0xFF05100C, 0xFF06110D, 0xFF07120E, 0xFF030F0D, 0xFF030F0D, 0xFF030F0D, 0xFF030F0D, 0xFF04100E,
        0xFF05110F, 0xFF061210, 0xFF071311, 0xFF030F0D, 0xFF04100E, 0xFF05110F, 0xFF05110F, 0xFF05110F, 0xFF030F0D, 0xFF010D0B, 0xFF000C0A, 0xFF010A05, 0xFF0C1510, 0xFF18231D, 0xFF18231D, 0xFF131E18, 0xFF15221B, 0xFF19261F, 0xFF14211A, 0xFF17221E,
        0xFF192420, 0xFF192420, 0xFF18231F, 0xFF15201C, 0xFF131E1A, 0xFF141F1B, 0xFF15201C, 0xFF16221E, 0xFF15211D, 0xFF14201C, 0xFF232E2A, 0xFF27322E, 0xFF0E1915, 0xFF000805, 0xFF030C09, 0xFF010506, 0xFF040809, 0xFF060A0B, 0xFF060A0B, 0xFF040809,
        0xFF030708, 0xFF030708, 0xFF040809, 0xFF020607, 0xFF020607, 0xFF020607, 0xFF010506, 0xFF000304, 0xFF000405, 0xFF020607, 0xFF05090A, 0xFF000A08, 0xFF000907, 0xFF000806, 0xFF000705, 0xFF000604, 0xFF000604, 0xFF000604, 0xFF000604, 0xFF000804,
        0xFF000804, 0xFF000501, 0xFF000400, 0xFF000602, 0xFF020E0A, 0xFF081410, 0xFF091511, 0xFF000504, 0xFF000403, 0xFF030D0C, 0xFF000605, 0xFF0E1817, 0xFF2B3534, 0xFF212B2A, 0xFF16201F, 0xFF1D2824, 0xFF383E3C, 0xFF313534, 0x00D6D6D6,
  0x00FFFFFF, 0x00FFFFFD, 0x00FEFEFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF677774, 0xFF313439, 0xFF283033, 0xFF2F393B, 0xFF576264, 0xFF0D1919, 0xFF1A2320, 0xFF191E18, 0xFF0C0F08, 0xFF15201C, 0xFF000500, 0xFF060C08, 0xFF151918, 0xFF000102,
        0xFF0A1010, 0xFF040D0A, 0xFF000B05, 0xFF111619, 0xFF050A0D, 0xFF000202, 0xFF080E0E, 0xFF000202, 0xFF0C1212, 0xFF101518, 0xFF000205, 0xFF020B06, 0xFF000906, 0xFF000204, 0xFF060E10, 0xFF000204, 0xFF000507, 0xFF000107, 0xFF00030B, 0xFF000807,
        0xFF000306, 0xFF000009, 0xFF00000B, 0xFF090915, 0xFF000009, 0xFF000104, 0xFF000605, 0xFF050507, 0xFF000004, 0xFF030305, 0xFF050505, 0xFF060702, 0xFF0E0F07, 0xFF0F1103, 0xFF131603, 0xFF0E110A, 0xFF0E100B, 0xFF0D0F0C, 0xFF0E100D, 0xFF0F110C,
        0xFF0E1108, 0xFF0D1102, 0xFF0C1000, 0xFF0A0A02, 0xFF14170E, 0xFF060B04, 0xFF0B150D, 0xFF040E06, 0xFF273128, 0xFF373F32, 0xFF070D00, 0xFF000709, 0xFF000102, 0xFF0A0907, 0xFF030000, 0xFF060500, 0xFF080D06, 0xFF000400, 0xFF01120A, 0xFF0B0B0D,
        0xFF030406, 0xFF000104, 0xFF060B0E, 0xFF02070B, 0xFF02050C, 0xFF13111E, 0xFF2B2936, 0xFF1F2517, 0xFF1D1F1C, 0xFF080808, 0xFF010100, 0xFF040300, 0xFF000002, 0xFF040507, 0xFF000200, 0xFF060000, 0xFF0A0501, 0xFF030000, 0xFF030000, 0xFF0D0804,
        0xFF060000, 0xFF080202, 0xFF030002, 0xFF000009, 0xFF060B04, 0xFF000600, 0xFF000107, 0xFF050A1E, 0xFF020816, 0xFF000304, 0xFF0B0C07, 0xFF041000, 0xFF121E08, 0xFF0B1009, 0xFF000007, 0xFF07080A, 0xFF101707, 0xFF000900, 0xFF080F19, 0xFF0B1103,
        0xFF060D06, 0xFF0B1111, 0xFF000502, 0xFF000400, 0xFF0A1409, 0xFF000204, 0xFF000513, 0xFF050600, 0xFF000100, 0xFF0C0D11, 0xFF000007, 0xFF000009, 0xFF09060D, 0xFF040002, 0xFF070001, 0xFF020202, 0xFF010009, 0xFF05000A, 0xFF0F060B, 0xFF070002,
        0xFF060009, 0xFF09010E, 0xFF030000, 0xFF030500, 0xFF000005, 0xFF01000B, 0xFF080204, 0xFF060000, 0xFF07000A, 0xFF000009, 0xFF000200, 0xFF030000, 0xFF040000, 0xFF0B0700, 0xFF141108, 0xFF080904, 0xFF101211, 0xFF282826, 0xFF1B1C16, 0xFF000901,
        0xFF050C05, 0xFF151A16, 0xFF232522, 0xFF363A39, 0xFF181E1E, 0xFF000402, 0xFF000C0C, 0xFF040D0C, 0xFF0A1312, 0xFF111A19, 0xFF09120F, 0xFF17201D, 0xFF242D28, 0xFF151E19, 0xFF1C2520, 0xFF1C1E1B, 0xFF070906, 0xFF131512, 0xFF0A0C09, 0xFF0B0D0A,
        0xFF1A1C19, 0xFF0B0D0A, 0xFF0D0F0C, 0xFF100D18, 0xFF07060C, 0xFF0B0B0D, 0xFF171916, 0xFF181A15, 0xFF090E08, 0xFF040B04, 0xFF0D140D, 0xFF13131B, 0xFF000004, 0xFF111312, 0xFF0D0D0B, 0xFF000100, 0xFF0E0E0C, 0xFF0B090A, 0xFF010002, 0xFF0B0C0E,
        0xFF000002, 0xFF0B0C0E, 0xFF090A0C, 0xFF060709, 0xFF0F1012, 0xFF060709, 0xFF030406, 0xFF0A0B0D, 0xFF000002, 0xFF000002, 0xFF07080A, 0xFF0D0E10, 0xFF060709, 0xFF030406, 0xFF060709, 0xFF0C170F, 0xFF0E1810, 0xFF101A12, 0xFF060F0A, 0xFF191F1B,
        0xFF2D3331, 0xFF232726, 0xFF272B2C, 0xFF2C3532, 0xFF222B28, 0xFF202A22, 0xFF131D14, 0xFF071204, 0xFF1E2919, 0xFF374231, 0xFF303C28, 0xFF0D1307, 0xFF12180C, 0xFF1B2115, 0xFF12180C, 0xFF0B1105, 0xFF0E1108, 0xFF090C03, 0xFF0D1007, 0xFF0D0D0B,
        0xFF0C0C0A, 0xFF0B0B09, 0xFF0A0A08, 0xFF0A0A08, 0xFF0A0A08, 0xFF0A0A08, 0xFF0A0A08, 0xFF11120D, 0xFF0C0D05, 0xFF12150A, 0xFF212818, 0xFF242D1C, 0xFF162111, 0xFF0A160A, 0xFF09160C, 0xFF161511, 0xFF171812, 0xFF0A1006, 0xFF050D00, 0xFF202819,
        0xFF080E02, 0xFF191C13, 0xFF1B1A15, 0xFF05150B, 0xFF0F1911, 0xFF1A1C19, 0xFF0C0809, 0xFF363437, 0xFF161C1C, 0xFF01100D, 0xFF112822, 0xFF0F0E14, 0xFF141519, 0xFF161A19, 0xFF0D130F, 0xFF141915, 0xFF121212, 0xFF191418, 0xFF180E16, 0xFF0F1515,
        0xFF101A12, 0xFF0F1A16, 0xFF1D282A, 0xFF111C14, 0xFF0B1701, 0xFF242C21, 0xFF545665, 0xFF222625, 0xFF030706, 0xFF080E0C, 0xFF07100D, 0xFF222D29, 0xFF4E5A56, 0xFF222E2A, 0xFF0F1E19, 0xFF12211C, 0xFF333F3B, 0xFF353E3B, 0x00D7DDDB,
  0x00FFFFFF, 0x00FFFFFD, 0x00FCFCFA, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF5A6A67, 0xFF2E3336, 0xFF232B2E, 0xFF374045, 0xFF545F61, 0xFF192521, 0xFF626D5F, 0xFF8E967F, 0xFF9DA284, 0xFFC3CCB7, 0xFF7B836E, 0xFFABAF9E, 0xFFA5A99A, 0xFF7A7E6F,
        0xFFB0B4A3, 0xFF969E89, 0xFF858E79, 0xFFC4CAAE, 0xFF6B7253, 0xFF69704F, 0xFF99A07E, 0xFF727957, 0xFF919877, 0xFFA5AC8D, 0xFF656B4F, 0xFF8E9873, 0xFF77815E, 0xFF656F4C, 0xFF929C7A, 0xFF565F40, 0xFF434C2F, 0xFF6E775C, 0xFF3B4429, 0xFF2E3718,
        0xFF60664A, 0xFF40452E, 0xFF6D705B, 0xFF666954, 0xFF2D321B, 0xFF545A3E, 0xFF4D5637, 0xFF33381A, 0xFF575B40, 0xFF40442B, 0xFF373B24, 0xFF4F523D, 0xFF3F422D, 0xFF1D200B, 0xFF0B0E00, 0xFF111306, 0xFF0A0C01, 0xFF0F100A, 0xFF0D0E09, 0xFF050601,
        0xFF070800, 0xFF0B0D00, 0xFF0B0D00, 0xFF12130D, 0xFF000100, 0xFF10130C, 0xFF070D03, 0xFF0F1509, 0xFF161D0D, 0xFF181D09, 0xFF333621, 0xFF3B4429, 0xFF525740, 0xFF4C513D, 0xFF494B3D, 0xFF4D5045, 0xFF51564F, 0xFF6B756D, 0xFF59645C, 0xFF6D7665,
        0xFF5F6855, 0xFF5B6753, 0xFF6B7761, 0xFF5A664E, 0xFF586549, 0xFF6D785A, 0xFF7A8565, 0xFF99A68A, 0xFFA1AD99, 0xFF707968, 0xFF6D7560, 0xFF7A816F, 0xFF6F7768, 0xFF899484, 0xFF6A7660, 0xFF5B6B51, 0xFF687760, 0xFF5F6E57, 0xFF61705B, 0xFF7A8974,
        0xFF71806B, 0xFF73826F, 0xFF788774, 0xFF707966, 0xFF848F6D, 0xFF7C8B62, 0xFF768568, 0xFF7E8C7B, 0xFF707D6C, 0xFF6C7A61, 0xFF99A68A, 0xFF727D6C, 0xFF8B9481, 0xFF8B9187, 0xFF767A7B, 0xFF858B81, 0xFF848E6C, 0xFF727D5D, 0xFF818B80, 0xFF838A78,
        0xFF959D92, 0xFF828C83, 0xFF707B6B, 0xFF7B886C, 0xFF768463, 0xFF5A644B, 0xFF7F8676, 0xFF697F5B, 0xFF526746, 0xFF879E81, 0xFF576E52, 0xFF52664B, 0xFF798C70, 0xFF465639, 0xFF5C6D4D, 0xFF64785C, 0xFF5B6E58, 0xFF738471, 0xFF66755E, 0xFF5B6952,
        0xFF6C7967, 0xFF606F5A, 0xFF59694E, 0xFF728B63, 0xFF5E715B, 0xFF606F5C, 0xFF718061, 0xFF6B7A5B, 0xFF697766, 0xFF5E715E, 0xFF5F7755, 0xFF6E7E63, 0xFF809075, 0xFF83967A, 0xFF82947A, 0xFF758970, 0xFF73876E, 0xFF879E84, 0xFF95AC92, 0xFF889977,
        0xFF909F7E, 0xFFA2AD8D, 0xFFAAB396, 0xFFBFCAAC, 0xFFA8B79A, 0xFF8B9E80, 0xFF8A9F80, 0xFF7C8B6E, 0xFF8D9C7F, 0xFFA1B093, 0xFFA2B192, 0xFFB1C0A1, 0xFFBCCBAC, 0xFFB0BF9E, 0xFFB9C8A7, 0xFF9BB18D, 0xFF8AA07C, 0xFF92A884, 0xFF8BA17D, 0xFF8BA17D,
        0xFF97AD89, 0xFF8DA37F, 0xFF8FA581, 0xFF8EA27F, 0xFF798F6B, 0xFF889E77, 0xFF9BB288, 0xFF8BA47A, 0xFF839C72, 0xFF8BA67B, 0xFF8AA57A, 0xFF83977B, 0xFF728766, 0xFF829671, 0xFF7C916A, 0xFF788A62, 0xFF879973, 0xFF7C8D69, 0xFF6F7E5D, 0xFF98AC89,
        0xFF657956, 0xFF8DA17E, 0xFF98AC89, 0xFF687C59, 0xFF7D916E, 0xFF90A481, 0xFF667A57, 0xFF7E926F, 0xFF718562, 0xFF6C805D, 0xFF798D6A, 0xFF869A77, 0xFF879B78, 0xFF819572, 0xFF7F9370, 0xFFA1AE92, 0xFF8D9A7E, 0xFF98A587, 0xFF9EAB8D, 0xFFA0AF8E,
        0xFFA6B594, 0xFF9AAA86, 0xFF96A682, 0xFFA9B4A3, 0xFFA4AF9E, 0xFFA4B09C, 0xFF99A591, 0xFF89957F, 0xFF8D9983, 0xFF919D85, 0xFF838F77, 0xFF80856E, 0xFF5E634C, 0xFF3C412B, 0xFF1F240E, 0xFF131804, 0xFF111602, 0xFF0C1100, 0xFF101403, 0xFF0F1106,
        0xFF0F1106, 0xFF0E1005, 0xFF0E1005, 0xFF0E1005, 0xFF0F1106, 0xFF101207, 0xFF111308, 0xFF0E0F00, 0xFF161803, 0xFF2E3219, 0xFF505738, 0xFF66704E, 0xFF6E7959, 0xFF727D5F, 0xFF768367, 0xFF888C71, 0xFF8D9475, 0xFF909B79, 0xFF8D9B77, 0xFF9AA982,
        0xFF8F9B75, 0xFF9DA784, 0xFFA4AA88, 0xFF979F78, 0xFFA0A481, 0xFFAAA788, 0xFFAAA389, 0xFFC3BCA2, 0xFF9C9B7F, 0xFF9DA483, 0xFFABB694, 0xFF929978, 0xFF909A77, 0xFF919D77, 0xFF8A9972, 0xFF97A37D, 0xFF99A37E, 0xFFA1A785, 0xFF9D9E7F, 0xFF979D7B,
        0xFF979F76, 0xFF98A27D, 0xFFA4AD8E, 0xFF9AA57B, 0xFF98A56D, 0xFF9DA679, 0xFFABAE9B, 0xFF222423, 0xFF060807, 0xFF080C0B, 0xFF080E0C, 0xFF1F2825, 0xFF48534F, 0xFF26312D, 0xFF15211D, 0xFF1E2D28, 0xFF27332F, 0xFF313A37, 0x00DEE4E2,
  0x00FFFFFF, 0x00FFFFFD, 0x00FBFBF9, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF596966, 0xFF2D3333, 0xFF161E21, 0xFF313A41, 0xFF4A5559, 0xFF141F17, 0xFF788267, 0xFFA4AE7C, 0xFFA2AB6C, 0xFF9BA35A, 0xFF9AA15D, 0xFF919759, 0xFF979D5F, 0xFF969C5C,
        0xFF9AA15B, 0xFF969E55, 0xFF929A51, 0xFF919759, 0xFF969C5C, 0xFF8A914D, 0xFF99A158, 0xFF9BA35A, 0xFF949B57, 0xFF919757, 0xFFA2A86A, 0xFF969D57, 0xFF9BA35A, 0xFF899148, 0xFF9EA55F, 0xFF9FA565, 0xFF959B5D, 0xFF9DA365, 0xFF979D5D, 0xFF97A061,
        0xFF9DA667, 0xFF868E52, 0xFF98A064, 0xFF90985C, 0xFF8C9458, 0xFF838C4D, 0xFF788142, 0xFF838C4D, 0xFF767E42, 0xFF6E763D, 0xFF7D8451, 0xFF747950, 0xFF535734, 0xFF3B3E1F, 0xFF202306, 0xFF171903, 0xFF1A1C07, 0xFF0F1002, 0xFF111107, 0xFF1A1A12,
        0xFF0E0E06, 0xFF101006, 0xFF141408, 0xFF11120C, 0xFF12130B, 0xFF17190E, 0xFF141505, 0xFF101200, 0xFF202008, 0xFF37371B, 0xFF57573B, 0xFF879258, 0xFFB0BD87, 0xFFBCCA97, 0xFFB1BE90, 0xFFA6B483, 0xFF8B9862, 0xFF8B9759, 0xFF97A260, 0xFF97A375,
        0xFF889466, 0xFF839160, 0xFF909E69, 0xFF89985D, 0xFF819150, 0xFF859951, 0xFF82974A, 0xFF829156, 0xFF85945B, 0xFF818F5A, 0xFF879564, 0xFF8F9C6E, 0xFF82905F, 0xFF8A9863, 0xFF8D9C65, 0xFF7F9763, 0xFF869D65, 0xFF82975C, 0xFF7E935A, 0xFF879F6B,
        0xFF809866, 0xFF809761, 0xFF849960, 0xFF828D55, 0xFF859356, 0xFF7E924B, 0xFF83974F, 0xFF8FA260, 0xFF82945A, 0xFF788C57, 0xFFA6BD87, 0xFFE8EEE2, 0xFFF6FDED, 0xFFF7FDF3, 0xFFEBF1ED, 0xFFECF5E0, 0xFFA3AF81, 0xFF96A272, 0xFFE3ECCF, 0xFFF9FDEC,
        0xFFFCFFF6, 0xFFF3FEF0, 0xFFE7F5DB, 0xFFBBCB9C, 0xFF93A166, 0xFF818B4F, 0xFF939764, 0xFF7A9352, 0xFF829B5A, 0xFF869F5E, 0xFF809957, 0xFF829B59, 0xFF849D5B, 0xFF869F5D, 0xFF7F9856, 0xFF85A157, 0xFF79944D, 0xFF88A360, 0xFF839B5D, 0xFF7C9456,
        0xFF849A59, 0xFF7F9650, 0xFF8AA159, 0xFF82A058, 0xFF829B5A, 0xFF8B9E5C, 0xFF8B9D51, 0xFF839951, 0xFF8CA36B, 0xFF859D61, 0xFF7B9647, 0xFF869C5C, 0xFF81995F, 0xFF7A935C, 0xFF7E975E, 0xFF81995B, 0xFF7C9252, 0xFF7A9254, 0xFF7B975D, 0xFF778C49,
        0xFF798C4A, 0xFF7F8D4F, 0xFF788446, 0xFF869254, 0xFF809051, 0xFF768B4A, 0xFF7A904F, 0xFF7D8D4F, 0xFF829254, 0xFF859556, 0xFF788849, 0xFF7A8A49, 0xFF80904F, 0xFF788847, 0xFF80914D, 0xFF7E9551, 0xFF748B47, 0xFF758C48, 0xFF728945, 0xFF718844,
        0xFF788F4B, 0xFF758C48, 0xFF768D49, 0xFF6F8739, 0xFF7E9648, 0xFF748C3E, 0xFF6C8436, 0xFF7C9448, 0xFF7B9347, 0xFF70883C, 0xFF778F43, 0xFF6B7F38, 0xFF71823B, 0xFF798B3F, 0xFF738635, 0xFF728333, 0xFF788939, 0xFF758338, 0xFF738038, 0xFF78883D,
        0xFF728237, 0xFF7D8D42, 0xFF7C8C41, 0xFF7A8A3F, 0xFF809045, 0xFF7B8B40, 0xFF7C8C41, 0xFF829247, 0xFF7C8C41, 0xFF79893E, 0xFF7C8C41, 0xFF7D8D42, 0xFF7B8B40, 0xFF79893E, 0xFF79893E, 0xFF888E52, 0xFF777D41, 0xFF7E8748, 0xFF7C8848, 0xFF73823F,
        0xFF71853E, 0xFF6C823A, 0xFF6E853D, 0xFF717E39, 0xFF707C3A, 0xFF727E3C, 0xFF6F7B3B, 0xFF667234, 0xFF616C30, 0xFF5A6529, 0xFF4E591F, 0xFF4F512C, 0xFF4A4C27, 0xFF414220, 0xFF282C0B, 0xFF181B00, 0xFF171A00, 0xFF151800, 0xFF151900, 0xFF111401,
        0xFF111401, 0xFF101300, 0xFF0F1200, 0xFF101300, 0xFF111401, 0xFF121502, 0xFF131603, 0xFF181700, 0xFF201F00, 0xFF30320B, 0xFF45481B, 0xFF4F5623, 0xFF525925, 0xFF525B26, 0xFF535B28, 0xFF586025, 0xFF566024, 0xFF5E6C2E, 0xFF5F6F2E, 0xFF667534,
        0xFF697533, 0xFF6A7330, 0xFF717835, 0xFF85831E, 0xFFACA845, 0xFFBBB557, 0xFFBBB257, 0xFFC3B960, 0xFFAAA446, 0xFFA09D3C, 0xFF6B6C06, 0xFF6F772A, 0xFF697426, 0xFF6C7A2D, 0xFF637426, 0xFF6A7B2D, 0xFF6E7A30, 0xFF747C33, 0xFF6D732B, 0xFF72732D,
        0xFF717427, 0xFF6F7327, 0xFF747634, 0xFF6D7426, 0xFF7B8628, 0xFF7E8734, 0xFF7D8248, 0xFF101010, 0xFF000000, 0xFF080A09, 0xFF080A09, 0xFF0A0E0D, 0xFF222826, 0xFF121B18, 0xFF0E1714, 0xFF1A2622, 0x0046514D, 0x00999F9D, 0x00F0F4F3,
  0x00FFFFFF, 0x00FFFFFD, 0x00FBFBF9, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF5D6D6A, 0xFF313A35, 0xFF121C1E, 0xFF303942, 0xFF545F65, 0xFF172116, 0xFF6B764C, 0xFF88924B, 0xFF6D781C, 0xFF747517, 0xFF81832F, 0xFF9B9D52, 0xFFA7AB5F, 0xFF9A9F4B,
        0xFF7A7F23, 0xFF6E7214, 0xFF75751B, 0xFF767731, 0xFF96984D, 0xFF868938, 0xFF6E721D, 0xFF6A6E19, 0xFF707322, 0xFF7C7E33, 0xFF8F904A, 0xFF898C3D, 0xFF676B17, 0xFF7E822B, 0xFF6A6E1A, 0xFF6E7025, 0xFF999A56, 0xFF8A8C43, 0xFF777A2B, 0xFF696B20,
        0xFF6D7127, 0xFF71772F, 0xFF8C944B, 0xFF7C843B, 0xFF5F651D, 0xFF5F6319, 0xFF5C5E13, 0xFF515614, 0xFF818644, 0xFF767B3B, 0xFF4F5318, 0xFF4D5019, 0xFF454718, 0xFF35360E, 0xFF454521, 0xFF282907, 0xFF181900, 0xFF212109, 0xFF141301, 0xFF0F0D00,
        0xFF151307, 0xFF181609, 0xFF121003, 0xFF0D0E06, 0xFF16160A, 0xFF161302, 0xFF2A250F, 0xFF1B1700, 0xFF383516, 0xFF4F4E2F, 0xFF6A6B4B, 0xFF88926F, 0xFFB7C2A0, 0xFFD3E4C0, 0xFFD8EEC0, 0xFFDCF1B6, 0xFFC3D687, 0xFF8D9B3A, 0xFF5F6B00, 0xFF636923,
        0xFF656823, 0xFF61641F, 0xFF63661F, 0xFF696D23, 0xFF5B6316, 0xFF596513, 0xFF556411, 0xFF5E6611, 0xFF606A11, 0xFF636F1B, 0xFF4F5A15, 0xFF535E1A, 0xFF616C1D, 0xFF5A640E, 0xFF5D6416, 0xFF565F1C, 0xFF5D6013, 0xFF676711, 0xFF616310, 0xFF545B15,
        0xFF5D6623, 0xFF767D30, 0xFF7E7E28, 0xFF81823C, 0xFF757A37, 0xFF6B762A, 0xFF667113, 0xFF626808, 0xFF646919, 0xFF626D2B, 0xFF7E9253, 0xFFFBFEF7, 0xFFFEFFF6, 0xFFF9FFF9, 0xFFFBFFFF, 0xFFF9FFF1, 0xFF7B885D, 0xFF828B60, 0xFFEFF5DB, 0xFFFEFFFA,
        0xFFF7FCF8, 0xFFF9FFFB, 0xFFF8FFEC, 0xFF9EAF79, 0xFF5E6A22, 0xFF64681B, 0xFF605E14, 0xFF6A6720, 0xFF656319, 0xFF6C691C, 0xFF706D1C, 0xFF717020, 0xFF5F6215, 0xFF5D6319, 0xFF5E651F, 0xFF60670A, 0xFF707717, 0xFF666F14, 0xFF5F6819, 0xFF5E6517,
        0xFF62680E, 0xFF6E7213, 0xFF66660C, 0xFF586314, 0xFF646718, 0xFF746F14, 0xFF73710C, 0xFF61640B, 0xFF595F19, 0xFF616518, 0xFF717610, 0xFF686B14, 0xFF5D6417, 0xFF5E6823, 0xFF62681E, 0xFF62650A, 0xFF666606, 0xFF63660D, 0xFF5A6314, 0xFF5B6617,
        0xFF5D6617, 0xFF676C1C, 0xFF5E5F11, 0xFF656618, 0xFF626516, 0xFF606717, 0xFF636C19, 0xFF65681B, 0xFF656819, 0xFF66691A, 0xFF5E6112, 0xFF656817, 0xFF6B6E1D, 0xFF626612, 0xFF646814, 0xFF6D6A1B, 0xFF6B6819, 0xFF696617, 0xFF696617, 0xFF686516,
        0xFF686516, 0xFF6B6819, 0xFF6A6718, 0xFF6C6B11, 0xFF6C6A13, 0xFF6D6814, 0xFF6B6615, 0xFF6B6517, 0xFF6E6918, 0xFF6C6716, 0xFF686110, 0xFF8D8228, 0xFF928529, 0xFF938626, 0xFF938723, 0xFF928421, 0xFF90821F, 0xFF938324, 0xFF968529, 0xFF908221,
        0xFF948625, 0xFF988A29, 0xFF948625, 0xFF9B8D2C, 0xFF9C8E2D, 0xFF948625, 0xFF9D8F2E, 0xFF9C8E2D, 0xFF9D8F2E, 0xFF9D8F2E, 0xFF9A8C2B, 0xFF978928, 0xFF978928, 0xFF998B2A, 0xFF9A8C2B, 0xFF998C30, 0xFF94892F, 0xFF847A21, 0xFF656108, 0xFF62620C,
        0xFF676B16, 0xFF5E6611, 0xFF616A17, 0xFF606509, 0xFF5D6208, 0xFF595D06, 0xFF575B07, 0xFF555809, 0xFF4F5108, 0xFF484903, 0xFF434400, 0xFF3A3307, 0xFF393508, 0xFF3C370F, 0xFF35310C, 0xFF2A2704, 0xFF262304, 0xFF1E1D01, 0xFF151400, 0xFF151402,
        0xFF141301, 0xFF121100, 0xFF111000, 0xFF111000, 0xFF131200, 0xFF141301, 0xFF151402, 0xFF211A00, 0xFF282200, 0xFF312E03, 0xFF3A3805, 0xFF414007, 0xFF454509, 0xFF474709, 0xFF47490A, 0xFF514F05, 0xFF4B4B01, 0xFF54560D, 0xFF55590D, 0xFF575A0D,
        0xFF605F0F, 0xFF5E5907, 0xFF766E1B, 0xFFB5A246, 0xFFAB9740, 0xFF937E2D, 0xFF887526, 0xFF968132, 0xFFAB9643, 0xFFB9A44B, 0xFF937E21, 0xFF665F04, 0xFF5F5B00, 0xFF686810, 0xFF60630C, 0xFF61630F, 0xFF63620F, 0xFF6A6813, 0xFF67620E, 0xFF655E0D,
        0xFF6A6309, 0xFF6A620D, 0xFF6B6414, 0xFF666209, 0xFF70730A, 0xFF727919, 0xFF828840, 0xFF131112, 0xFF010000, 0xFF080808, 0xFF0B0D0C, 0xFF000302, 0xFF000604, 0xFF000503, 0xFF020B08, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFD, 0x00FCFCFA, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF5D6D6A, 0xFF303A32, 0xFF101A1B, 0xFF263039, 0xFF596269, 0xFF121D0F, 0xFF555E31, 0xFF858E3D, 0xFF747C15, 0xFF746929, 0xFFA89E6D, 0xFFFEF6D2, 0xFFE2DEB9, 0xFFF4F2BF,
        0xFFD8D494, 0xFF756D2E, 0xFF7D7137, 0xFFCEC894, 0xFFE5E0A6, 0xFFE1DE9B, 0xFFA4A258, 0xFF656319, 0xFF74712E, 0xFFC4BF85, 0xFFF0EAB6, 0xFFE4DEA2, 0xFF9B9750, 0xFF6A6718, 0xFF858138, 0xFFCCC68C, 0xFFE8E1AD, 0xFFE2DCA2, 0xFF9B9652, 0xFF766D28,
        0xFF7D7836, 0xFFC0BE83, 0xFFDBDCA3, 0xFFE6E7AE, 0xFF9D9B60, 0xFF66611F, 0xFF6D641F, 0xFFBDBC90, 0xFFD6D5A5, 0xFFCECE9A, 0xFF92935A, 0xFF45460D, 0xFF53531F, 0xFF9E9D6F, 0xFFA5A478, 0xFF5F6034, 0xFF35360E, 0xFF1B1B00, 0xFF171600, 0xFF3A3821,
        0xFF393720, 0xFF28260F, 0xFF242209, 0xFF121100, 0xFF201E09, 0xFF251F05, 0xFF150E00, 0xFF464020, 0xFF777455, 0xFF55583B, 0xFF545A40, 0xFF747E8A, 0xFFBCCAD5, 0xFFD6E7F1, 0xFFDFF3F4, 0xFFD0E2D4, 0xFFD7E5C4, 0xFFE5EDBA, 0xFF9EA363, 0xFF726B1B,
        0xFF726718, 0xFF77691E, 0xFF6E5D15, 0xFF695A17, 0xFF675D20, 0xFF88834C, 0xFF8F8F5D, 0xFF6B6119, 0xFF696410, 0xFF6C6A1D, 0xFF908F59, 0xFF8A8855, 0xFF605E14, 0xFF635E0D, 0xFF7B702E, 0xFF8C8859, 0xFF857A38, 0xFF746311, 0xFF6C5E13, 0xFF7C773F,
        0xFFA7A574, 0xFFC9C180, 0xFFCABA65, 0xFFBBB06E, 0xFFD8D2A2, 0xFFD4D39B, 0xFFACA84F, 0xFF7C6F09, 0xFF6F5C0F, 0xFF675E23, 0xFF7C8147, 0xFFF8F7F3, 0xFFFFFFFA, 0xFFF2F8F6, 0xFFF7FEFF, 0xFFEDF7EF, 0xFF8F9A7C, 0xFFA3A78E, 0xFFD8D7D2, 0xFFFBFAFF,
        0xFFFBFEFF, 0xFFEFF8FD, 0xFFF3FFEF, 0xFFA1AE80, 0xFF585F1B, 0xFF656215, 0xFF74691A, 0xFF786324, 0xFF67500C, 0xFF7C6519, 0xFF70590B, 0xFF6D5A0D, 0xFF6F621C, 0xFF7F763B, 0xFF9B9863, 0xFF7A6D27, 0xFF655F0B, 0xFF636013, 0xFF8E8854, 0xFF968F5B,
        0xFF70681D, 0xFF655A08, 0xFF73621D, 0xFF939161, 0xFF93864F, 0xFF755F14, 0xFF655300, 0xFF7D7431, 0xFF938C60, 0xFF8A814A, 0xFF695D07, 0xFF6E5F0E, 0xFF8A8243, 0xFF93915E, 0xFF756F31, 0xFF6C5C05, 0xFF7B6706, 0xFF756611, 0xFF6E6723, 0xFF6D671D,
        0xFF61590E, 0xFF6D5F16, 0xFF6E5C12, 0xFF76641A, 0xFF736115, 0xFF706314, 0xFF6E6313, 0xFF715F15, 0xFF736117, 0xFF756319, 0xFF736115, 0xFF79671B, 0xFF7B6A1C, 0xFF726113, 0xFF705F11, 0xFF735C10, 0xFF786115, 0xFF755E12, 0xFF776014, 0xFF765F13,
        0xFF735C10, 0xFF755E12, 0xFF755E12, 0xFF7A6315, 0xFF6D550B, 0xFF725A14, 0xFF826625, 0xFF7A5C1C, 0xFF694C0A, 0xFF7C5E18, 0xFFA2843B, 0xFFDDB959, 0xFFCCA645, 0xFFC19B36, 0xFFC9A43A, 0xFFCAA33A, 0xFFC69F38, 0xFFCBA13D, 0xFFC59B39, 0xFFCEA43E,
        0xFFC79D37, 0xFFD2A842, 0xFFD2A842, 0xFFC89E38, 0xFFCBA13B, 0xFFCEA43E, 0xFFC99F39, 0xFFCDA33D, 0xFFCBA13B, 0xFFC89E38, 0xFFC89E38, 0xFFCCA23C, 0xFFD0A640, 0xFFCFA53F, 0xFFCBA13B, 0xFFC8A233, 0xFFD6B146, 0xFFC7A63F, 0xFF967818, 0xFF765D03,
        0xFF6C5603, 0xFF665306, 0xFF6D5910, 0xFF6C5E15, 0xFF675912, 0xFF5F500D, 0xFF5B4B0F, 0xFF5A4911, 0xFF544210, 0xFF4C3A0A, 0xFF4A380A, 0xFF51400C, 0xFF443200, 0xFF3E2D00, 0xFF3F3007, 0xFF372904, 0xFF2C2101, 0xFF291E02, 0xFF231900, 0xFF1D1604,
        0xFF1C1503, 0xFF1B1402, 0xFF1A1301, 0xFF1B1402, 0xFF1D1604, 0xFF1F1806, 0xFF211A08, 0xFF2E2305, 0xFF362C09, 0xFF40340A, 0xFF453B08, 0xFF494007, 0xFF4E450A, 0xFF51480D, 0xFF52480D, 0xFF5A4601, 0xFF5E4B07, 0xFF665412, 0xFF675513, 0xFF63500B,
        0xFF685207, 0xFF785D0E, 0xFFAA8B38, 0xFFA38444, 0xFF83662C, 0xFF886E3B, 0xFF9E8857, 0xFF917A48, 0xFF856731, 0xFF957334, 0xFFC7A25E, 0xFF846815, 0xFF6D5203, 0xFF735B11, 0xFF6B5712, 0xFF685511, 0xFF695411, 0xFF715A14, 0xFF705810, 0xFF695611,
        0xFF6F5C0F, 0xFF6D570E, 0xFF6C5714, 0xFF6B5B0E, 0xFF6A6507, 0xFF6F721B, 0xFFA0A565, 0xFF312F30, 0xFF050505, 0xFF070707, 0xFF161817, 0xFF101413, 0xFF080E0C, 0xFF08110E, 0xFF0E1714, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFD, 0x00FEFEFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF667673, 0xFF2E3931, 0xFF0B1715, 0xFF172228, 0xFF4F585D, 0xFF0E1609, 0xFF484F25, 0xFF7B8137, 0xFF73781A, 0xFF635317, 0xFFB6A97F, 0xFFC8BDA7, 0xFF251F05, 0xFF918E61,
        0xFFD5D192, 0xFF584E11, 0xFF9F8E5A, 0xFFC9C19D, 0xFF322B00, 0xFF88834C, 0xFFCAC687, 0xFF625E1F, 0xFF908B54, 0xFFD5CEA2, 0xFF251D00, 0xFF85794F, 0xFFDED59C, 0xFF6A631F, 0xFF90874C, 0xFFCABE96, 0xFF312402, 0xFF786C42, 0xFFDFD69D, 0xFF675611,
        0xFF8E8246, 0xFFE2DAAC, 0xFF2D2904, 0xFF605C37, 0xFFE6DEB0, 0xFF94884C, 0xFF7F6E29, 0xFFCCC4AF, 0xFF2A2306, 0xFF666139, 0xFFCECA9A, 0xFF484510, 0xFF625F2A, 0xFFD3CF9F, 0xFF1B1700, 0xFF3B3906, 0xFF979564, 0xFF252200, 0xFF3C3812, 0xFF96926D,
        0xFF161200, 0xFF221E00, 0xFF96936A, 0xFF363312, 0xFF282401, 0xFF2E2601, 0xFF635935, 0xFFBBB595, 0xFF73755D, 0xFF626D5D, 0xFFA4B4A9, 0xFFB2CED1, 0xFFD1EDF1, 0xFFDDF5FF, 0xFFE8FEFF, 0xFFE5F6FE, 0xFFDEE8EA, 0xFFDFE4DE, 0xFFEBEDE2, 0xFF91833A,
        0xFF645208, 0xFF71590F, 0xFF745914, 0xFF725920, 0xFF948159, 0xFFEEE2CC, 0xFFFAF1E8, 0xFF9F885F, 0xFF695A1F, 0xFF9A905D, 0xFFFFFAE9, 0xFFF0E7D8, 0xFF7F7446, 0xFF63541D, 0xFFC6AE8C, 0xFFFFFFF4, 0xFFD2CAA3, 0xFF6D5D21, 0xFF716531, 0xFFD2D2B8,
        0xFFFEFFF1, 0xFFBCBA93, 0xFF5F5110, 0xFF857A42, 0xFFE7DFC8, 0xFFF8F2DA, 0xFFCBBE78, 0xFF987B1F, 0xFF7F5A14, 0xFF6D5624, 0xFF87814D, 0xFFF8F2F4, 0xFFFFFFFB, 0xFFFBFFFF, 0xFFF8FFFF, 0xFFEDF8F4, 0xFFB5BFA6, 0xFFC9CBB5, 0xFFDCD7D4, 0xFFFFFEFF,
        0xFFFCFEFF, 0xFFE5EDF8, 0xFFE9F6ED, 0xFF9EAA84, 0xFF595A1E, 0xFF695C16, 0xFF746019, 0xFF6E5B21, 0xFF7D6A28, 0xFF735D14, 0xFF755F14, 0xFF705F19, 0xFF786F38, 0xFFDEDFB7, 0xFFF7FBE0, 0xFFC5B895, 0xFF5F5D24, 0xFF808251, 0xFFE8E9DB, 0xFFF8F6E9,
        0xFF979565, 0xFF5F5821, 0xFFA99977, 0xFFF6FCFA, 0xFFF0E9D6, 0xFF938151, 0xFF685C22, 0xFFB8B798, 0xFFFCFEF9, 0xFFDBD9C4, 0xFF6E652C, 0xFF72662C, 0xFFDEDBBA, 0xFFFEFFF1, 0xFFAEAD8E, 0xFF857736, 0xFF806E1C, 0xFF766829, 0xFF908D64, 0xFF8C7E3D,
        0xFF6B5919, 0xFF695111, 0xFF6B4F0E, 0xFF745715, 0xFF765915, 0xFF795E17, 0xFF745C14, 0xFF775A18, 0xFF795C1A, 0xFF785B19, 0xFF735612, 0xFF715410, 0xFF72550F, 0xFF745711, 0xFF785B15, 0xFF6E580F, 0xFF735D14, 0xFF735D14, 0xFF745E15, 0xFF755F16,
        0xFF715B12, 0xFF705A11, 0xFF715B12, 0xFF6B5710, 0xFF745F1C, 0xFF6E561A, 0xFF6B5018, 0xFF72551B, 0xFF725414, 0xFF997B33, 0xFFDFBF72, 0xFFDEB654, 0xFFC69F38, 0xFFC1982E, 0xFFC89F33, 0xFFC39B2C, 0xFFC2982C, 0xFFC89D34, 0xFFC49931, 0xFFC2962B,
        0xFFC99D32, 0xFFC89C31, 0xFFC5992E, 0xFFC99D32, 0xFFC79B30, 0xFFC2962B, 0xFFCA9E33, 0xFFCA9E33, 0xFFCA9E33, 0xFFC89C31, 0xFFC69A2F, 0xFFC5992E, 0xFFC5992E, 0xFFC4982D, 0xFFC3972C, 0xFFCE9C23, 0xFFDCA934, 0xFFE7B447, 0xFFD0A03A, 0xFFA5761A,
        0xFF926510, 0xFF936819, 0xFF946A1E, 0xFF906D19, 0xFF8E6B19, 0xFF8B6719, 0xFF8A661C, 0xFF88631D, 0xFF815B1A, 0xFF775112, 0xFF714B0D, 0xFF5B3E00, 0xFF5E4100, 0xFF5E4203, 0xFF5A4109, 0xFF513B0A, 0xFF513C11, 0xFF4B3712, 0xFF342200, 0xFF2E2307,
        0xFF2D2206, 0xFF2D2206, 0xFF2D2206, 0xFF2F2408, 0xFF33280C, 0xFF362B0F, 0xFF392E12, 0xFF42320E, 0xFF483910, 0xFF524113, 0xFF574713, 0xFF5B4A12, 0xFF5B4B10, 0xFF5C4911, 0xFF5C4911, 0xFF684A0A, 0xFF6A4D11, 0xFF694F14, 0xFF6D5018, 0xFF6D4F13,
        0xFF6F4C0A, 0xFF936C21, 0xFFBD9341, 0xFF7D5F2B, 0xFFA48C5C, 0xFFEAD9AE, 0xFFFFFFDA, 0xFFEEE0B9, 0xFFBDA778, 0xFF84632D, 0xFFAA8245, 0xFFAE843A, 0xFF7D5510, 0xFF755115, 0xFF6D4C17, 0xFF6D4D1A, 0xFF6E4F19, 0xFF73531A, 0xFF725013, 0xFF70551D,
        0xFF725615, 0xFF69490E, 0xFF6A4B15, 0xFF6C5414, 0xFF645C09, 0xFF696A1E, 0xFFAAAF79, 0xFF383A39, 0xFF010302, 0xFF050908, 0xFF1B211F, 0xFF1A2320, 0xFF131E1A, 0xFF1C2723, 0xFF27332F, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFD, 0x00FFFFFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF6C7C79, 0xFF313E34, 0xFF0D1917, 0xFF1A2529, 0xFF505A5C, 0xFF1F271C, 0xFF606443, 0xFF7A7B41, 0xFF6E6C21, 0xFF78681A, 0xFFA69964, 0xFFE1DBB9, 0xFF42421C, 0xFF909457,
        0xFFD6D787, 0xFF635A0B, 0xFF988742, 0xFFD7CEAD, 0xFF4E471B, 0xFF9C975F, 0xFFC2BE7E, 0xFF534F0F, 0xFF837E46, 0xFFE4DDB1, 0xFF564D2C, 0xFF92865C, 0xFFC5BC81, 0xFF655E16, 0xFF8E8647, 0xFFE2D6AE, 0xFF605333, 0xFF968B5E, 0xFFF2EAAC, 0xFF77630D,
        0xFF8D7C37, 0xFFDFD5A4, 0xFF67613F, 0xFF797351, 0xFFDAD09F, 0xFFAB9A55, 0xFFAE9A44, 0xFFDCD1BD, 0xFF655C3F, 0xFF8D855E, 0xFFC0BA88, 0xFF4A450E, 0xFF6C6632, 0xFFD9D2A4, 0xFF615A30, 0xFF646229, 0xFFB6B37C, 0xFF343100, 0xFF545020, 0xFFC8C494,
        0xFF5A5724, 0xFF68652E, 0xFFBBB980, 0xFF332F00, 0xFF4C4614, 0xFF5C5123, 0xFFB3AA81, 0xFFA6A384, 0xFF4A5342, 0xFF7A8E8C, 0xFFC1DCE3, 0xFF93C1B6, 0xFF9DC6BE, 0xFFCFEFEA, 0xFFECFFFF, 0xFFF2FFFF, 0xFFCEDCDD, 0xFF97A3A3, 0xFFD7E1E2, 0xFFDBD196,
        0xFF766925, 0xFF6A580E, 0xFF6D570E, 0xFF645015, 0xFF867756, 0xFFEFE8E2, 0xFFF8F6FF, 0xFFB3A082, 0xFF574E17, 0xFF8E8B60, 0xFFF3F2EE, 0xFFEBEAE8, 0xFF8B885F, 0xFF5A501D, 0xFFD3BFA7, 0xFFF6FFFF, 0xFFE3E2C4, 0xFF71662E, 0xFF625E2F, 0xFFC2CABB,
        0xFFF1FFFB, 0xFFB6BB9B, 0xFF5F5718, 0xFF787B46, 0xFFEBF0EA, 0xFFF4F9F5, 0xFFC9C389, 0xFF977C23, 0xFF7B5411, 0xFF694F1E, 0xFF7B7940, 0xFFFDF3FB, 0xFFFFFDFE, 0xFFF9FEFF, 0xFFF3FEFF, 0xFFE2F0E3, 0xFF9CA882, 0xFF8D926A, 0xFFD9D4C0, 0xFFFFFFFD,
        0xFFF5F8FD, 0xFFEAF3F8, 0xFFF1FEF4, 0xFF98A27F, 0xFF5A581F, 0xFF71601B, 0xFF6D500C, 0xFF615219, 0xFF6E5F1E, 0xFF6F5D15, 0xFF6D5B11, 0xFF605213, 0xFF6C673D, 0xFFDDE2CC, 0xFFF7FFFC, 0xFFBDB69A, 0xFF53561F, 0xFF7F885B, 0xFFE3ECE7, 0xFFECF2EE,
        0xFF8E9569, 0xFF595822, 0xFFADA187, 0xFFF0FFFF, 0xFFECEEE1, 0xFF7C7445, 0xFF5D5A23, 0xFFB4C1A7, 0xFFDDEFF3, 0xFFC5D0C2, 0xFF6A6A34, 0xFF6E6635, 0xFFD6D9C4, 0xFFF7FFFF, 0xFFC0C5B1, 0xFFC1B77A, 0xFFC6B567, 0xFFA79E63, 0xFFC1C4A5, 0xFF8A8043,
        0xFF65591C, 0xFF685618, 0xFF6B5315, 0xFF6F5614, 0xFF705714, 0xFF745C16, 0xFF6B550C, 0xFF725615, 0xFF745817, 0xFF765A19, 0xFF755A17, 0xFF725714, 0xFF715611, 0xFF735813, 0xFF755A15, 0xFF6C5811, 0xFF705C15, 0xFF715D16, 0xFF6F5B14, 0xFF725E17,
        0xFF705C15, 0xFF6C5811, 0xFF705C15, 0xFF6A5A0F, 0xFF67540F, 0xFF6E5918, 0xFF695113, 0xFF6B500D, 0xFF9E8336, 0xFFD2B75E, 0xFFD7BD5C, 0xFFD4AB42, 0xFFC9A036, 0xFFCCA433, 0xFFCCA331, 0xFFC49B27, 0xFFC89D2C, 0xFFCEA235, 0xFFCEA237, 0xFFCEA030,
        0xFFCB9D2D, 0xFFCEA030, 0xFFD0A232, 0xFFCA9C2C, 0xFFCA9C2C, 0xFFCEA030, 0xFFC99B2B, 0xFFCB9D2D, 0xFFCEA030, 0xFFD1A333, 0xFFD1A333, 0xFFCEA030, 0xFFCB9D2D, 0xFFCEA030, 0xFFD3A535, 0xFFD8A62D, 0xFFD3A02A, 0xFFDCA934, 0xFFE6B445, 0xFFE3B045,
        0xFFE0AC46, 0xFFDCAB45, 0xFFD2A03D, 0xFFD8A933, 0xFFD8A935, 0xFFD9AA38, 0xFFD8A839, 0xFFD3A335, 0xFFCE9D34, 0xFFC6952E, 0xFFBD8C25, 0xFFAA8324, 0xFFA78227, 0xFF9C7822, 0xFF8D6C1D, 0xFF7A5C14, 0xFF765A19, 0xFF765C21, 0xFF5E450D, 0xFF49380A,
        0xFF483709, 0xFF473608, 0xFF463507, 0xFF483709, 0xFF4B3A0C, 0xFF4E3D0F, 0xFF503F11, 0xFF4F3E0A, 0xFF53420A, 0xFF5C490E, 0xFF655112, 0xFF695413, 0xFF6B5315, 0xFF6B5317, 0xFF6C5119, 0xFF6D5612, 0xFF695111, 0xFF644E14, 0xFF695018, 0xFF6E5115,
        0xFF6D4C07, 0xFFA37E2E, 0xFFAA822B, 0xFF866D4E, 0xFFD8C7AD, 0xFFFFF9E6, 0xFFFCFDED, 0xFFFFFCE9, 0xFFFAECD1, 0xFF977B56, 0xFF89643A, 0xFFBA913F, 0xFF795308, 0xFF6E4C0C, 0xFF6B4E14, 0xFF6D521B, 0xFF6E531B, 0xFF745819, 0xFF735815, 0xFF715317,
        0xFF755615, 0xFF755113, 0xFF74511B, 0xFF705714, 0xFF685E07, 0xFF606312, 0xFF939D61, 0xFF2E3432, 0xFF000200, 0xFF0A1310, 0xFF212C28, 0xFF232F2B, 0xFF1E2D28, 0xFF273631, 0xFF30413B, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFD, 0x00FDFDFB, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF637370, 0xFF2D3A30, 0xFF0C1814, 0xFF1F2A2C, 0xFF495355, 0xFF232821, 0xFF6E7156, 0xFF797747, 0xFF6C682B, 0xFF6D5C16, 0xFF95875A, 0xFFE5DDC8, 0xFFE6E4CB, 0xFFEAEDB8,
        0xFF98984E, 0xFF6B611A, 0xFF614E13, 0xFFBAB58B, 0xFFE0DDAA, 0xFFC0BF7E, 0xFF67671D, 0xFF5A5A10, 0xFF676625, 0xFFABA875, 0xFFE4DFB5, 0xFFC1B988, 0xFF6D6824, 0xFF615F0C, 0xFF59550C, 0xFFB8B07F, 0xFFE1D8B1, 0xFFC8C18B, 0xFF97934A, 0xFFBFAC45,
        0xFF71600E, 0xFFABA068, 0xFFE8E0BB, 0xFFE3DBB6, 0xFFC7BC84, 0xFF9C8B39, 0xFFAA9730, 0xFFBFB699, 0xFFEBE3BF, 0xFFD3CD9D, 0xFF757038, 0xFF4A450B, 0xFF57511D, 0xFF8D865A, 0xFFEFE7C2, 0xFFCECC91, 0xFF65632A, 0xFF535019, 0xFF4E4B14, 0xFF817F46,
        0xFFD1CF92, 0xFFD1D08F, 0xFF6F6F29, 0xFF4D480E, 0xFF564D14, 0xFF776B37, 0xFFE1D8AD, 0xFF737254, 0xFF7A867C, 0xFFAFC8CF, 0xFFC9ECFF, 0xFF89BCD1, 0xFF76A3B6, 0xFF91B2C3, 0xFFBAD2DE, 0xFFD2E3EB, 0xFFE2EFF7, 0xFFB2C1C6, 0xFF889A9E, 0xFFF3EFBF,
        0xFF7D773B, 0xFF655C11, 0xFF675A0B, 0xFF615614, 0xFF74704A, 0xFFDDE0D9, 0xFFF3FAFF, 0xFFA79E75, 0xFF555813, 0xFF7A854B, 0xFFF5FFF2, 0xFFE7F1E6, 0xFF6C773F, 0xFF515312, 0xFFCAC09D, 0xFFF5FFFE, 0xFFE6E2BD, 0xFF706221, 0xFF635C25, 0xFFBFC7B2,
        0xFFF7FFFA, 0xFFBFC49C, 0xFF564D02, 0xFF74874F, 0xFFE0F2F2, 0xFFEDFCFF, 0xFFCDD09B, 0xFF958027, 0xFF77550E, 0xFF6B5621, 0xFF707533, 0xFFF8ECF8, 0xFFFFFCFF, 0xFFEAF0F0, 0xFFDBE7E5, 0xFFE1F1D7, 0xFF8A9962, 0xFF54591F, 0xFFE3E0B7, 0xFFFFFFF3,
        0xFFFAFFF9, 0xFFEEFAF8, 0xFFEFFDEE, 0xFF96A179, 0xFF57551A, 0xFF705B18, 0xFF765510, 0xFF6C591F, 0xFF6E5918, 0xFF6F570F, 0xFF775F17, 0xFF68551A, 0xFF6C623F, 0xFFDDDFD4, 0xFFF4FDFF, 0xFFB4AB80, 0xFF545710, 0xFF757F42, 0xFFE2EBDA, 0xFFE6EFDE,
        0xFF8E965A, 0xFF83833D, 0xFFE8DBB1, 0xFFDCF1F2, 0xFFE7EBD0, 0xFF766E2F, 0xFF58580E, 0xFFB4C39C, 0xFFD6EBE2, 0xFFC2CFB3, 0xFF666922, 0xFF73692E, 0xFFCCCFB4, 0xFFF6FFFB, 0xFFBBBFA4, 0xFF958842, 0xFF7B660B, 0xFF695C16, 0xFFA3A37D, 0xFF717131,
        0xFF585414, 0xFF645B1A, 0xFF675815, 0xFF675611, 0xFF6A5810, 0xFF736318, 0xFF695B10, 0xFF6F5A17, 0xFF6C5714, 0xFF6A5611, 0xFF6C5813, 0xFF6D5912, 0xFF6C5811, 0xFF6C5811, 0xFF68540B, 0xFF6E530E, 0xFF6F540F, 0xFF725712, 0xFF6D520D, 0xFF715611,
        0xFF725712, 0xFF6B500B, 0xFF735813, 0xFF6B5603, 0xFF705B0C, 0xFF6D520B, 0xFF72550F, 0xFFA08033, 0xFFD6B75C, 0xFFE0C157, 0xFFC8A935, 0xFFDCAC40, 0xFFD8A83A, 0xFFDAAB39, 0xFFD4A32D, 0xFFD3A22C, 0xFFDCA936, 0xFFD8A434, 0xFFD4A032, 0xFFD5A02C,
        0xFFDCA733, 0xFFD8A32F, 0xFFD7A22E, 0xFFDEA935, 0xFFDBA632, 0xFFDAA531, 0xFFDEA935, 0xFFDBA632, 0xFFD6A12D, 0xFFD5A02C, 0xFFDBA632, 0xFFDEA935, 0xFFDDA834, 0xFFDAA531, 0xFFDAA531, 0xFFD7AA33, 0xFFDEB13A, 0xFFDAAC34, 0xFFD7A931, 0xFFDAAC32,
        0xFFD7A62D, 0xFFD6A62A, 0xFFDDAD31, 0xFFE2A731, 0xFFE1A630, 0xFFE3A832, 0xFFE3A834, 0xFFE0A432, 0xFFE4A836, 0xFFE4A838, 0xFFDCA030, 0xFFC69D29, 0xFFBA9020, 0xFFB28C21, 0xFFA98524, 0xFF7C5A00, 0xFF5F4000, 0xFF816418, 0xFFA58940, 0xFF5E4A0B,
        0xFF5C4809, 0xFF594506, 0xFF574304, 0xFF574304, 0xFF584405, 0xFF5A4607, 0xFF5C4809, 0xFF5F4D0B, 0xFF5E4D08, 0xFF625008, 0xFF68540B, 0xFF6C560B, 0xFF6B530D, 0xFF6D520F, 0xFF715315, 0xFF65570E, 0xFF635610, 0xFF665819, 0xFF69591D, 0xFF665311,
        0xFF634C00, 0xFFAB9039, 0xFF9D7C1F, 0xFF7F7337, 0xFFE6E3AC, 0xFFFFFFD6, 0xFFF2FFD5, 0xFFFAFFD7, 0xFFF8F9C0, 0xFF948544, 0xFF8F752B, 0xFFB99B39, 0xFF745900, 0xFF6C5507, 0xFF6C5813, 0xFF685614, 0xFF63520D, 0xFF645409, 0xFF675606, 0xFF6E510D,
        0xFF745409, 0xFF78530D, 0xFF724F0F, 0xFF6B5104, 0xFF6D6503, 0xFF6B7117, 0xFF97A45F, 0xFF323B38, 0xFF000400, 0xFF131E1A, 0xFF2C3834, 0xFF35443F, 0xFF364741, 0xFF30433D, 0xFF2C3F39, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FCFCFC, 0x00FFFDFE, 0x00FEFCFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF556660, 0xFF374138, 0xFF0D1814, 0xFF192428, 0xFF4D565B, 0xFF18221A, 0xFF4D5433, 0xFF727A3F, 0xFF646B1E, 0xFF725C20, 0xFF6F591D, 0xFF766024, 0xFF7A6428, 0xFF735D21,
        0xFF6E581C, 0xFF6F591D, 0xFF6D571B, 0xFF5D5316, 0xFF61571C, 0xFF61541D, 0xFF60531E, 0xFF675A25, 0xFF695A23, 0xFF67581F, 0xFF6A5B20, 0xFF615115, 0xFF66561A, 0xFF69591D, 0xFF68581C, 0xFF67571B, 0xFF68581C, 0xFF67571B, 0xFF645418, 0xFFA68332,
        0xFFCCA756, 0xFF785504, 0xFF765E14, 0xFF5C520D, 0xFFCCC47B, 0xFFA89841, 0xFF705700, 0xFFB7974A, 0xFF998129, 0xFF655400, 0xFF584701, 0xFF655323, 0xFF634D1C, 0xFF644F00, 0xFFAD982B, 0xFF725907, 0xFF625008, 0xFF5E5419, 0xFF5F5B2C, 0xFF4F4A20,
        0xFF4D4215, 0xFF5B4A1C, 0xFF584412, 0xFF614D18, 0xFF64501D, 0xFFA69162, 0xFFE5D5B1, 0xFF746F59, 0xFF98A59C, 0xFFD8F7FC, 0xFFAAD4E2, 0xFF6F9EB0, 0xFFA2DAE9, 0xFFA3E1EC, 0xFF7CB2BC, 0xFF80A6B1, 0xFFDCF7FF, 0xFFB5CED3, 0xFF789497, 0xFFDCE4BF,
        0xFFA09F5E, 0xFF5D5002, 0xFF6C5C29, 0xFF67614B, 0xFF898F85, 0xFFCAD4D3, 0xFFD5DEE7, 0xFF899E97, 0xFF616D61, 0xFF949B89, 0xFFDCE7D7, 0xFFC2D6CD, 0xFF768B86, 0xFF5A6A5F, 0xFFB3BCAB, 0xFFD6E8DC, 0xFFB6C8BC, 0xFF5B6D61, 0xFF5F7165, 0xFFADBFB3,
        0xFFCEE0D4, 0xFF9AACA0, 0xFF5E7064, 0xFF7C887E, 0xFFCDDBCE, 0xFFBBD0C1, 0xFF9CB097, 0xFF777441, 0xFF715609, 0xFF5F4B02, 0xFF787B44, 0xFFE8EFDF, 0xFFFFFCFF, 0xFFEDF1F0, 0xFFB8CDC6, 0xFFD8E7E0, 0xFF817F56, 0xFF5F5317, 0xFFD3C8AA, 0xFFFFFDF1,
        0xFFFFFEFB, 0xFFF2F3F5, 0xFFFCFFF6, 0xFF999D78, 0xFF5D571B, 0xFF75611A, 0xFF6F5109, 0xFF6E5D05, 0xFF675410, 0xFF6D4C2D, 0xFF735316, 0xFF62571F, 0xFF657874, 0xFFA0BEBE, 0xFFCEE2EB, 0xFF929E92, 0xFF55684C, 0xFF698169, 0xFFC0D8D8, 0xFFC5DDDD,
        0xFF869E86, 0xFF8C9F83, 0xFF939F93, 0xFFCCDBE0, 0xFFBCCFBB, 0xFF6D835C, 0xFF52664A, 0xFF9BABAA, 0xFFDBE9F2, 0xFFB0C1B7, 0xFF7B916D, 0xFF547655, 0xFFB3C9B2, 0xFFCADCE0, 0xFF7E9AA5, 0xFF405F3F, 0xFF768739, 0xFFCBD195, 0xFF8C918A, 0xFF6F6025,
        0xFF6B5C21, 0xFF66571C, 0xFF65561B, 0xFF66581B, 0xFF695B1E, 0xFF6B5D1E, 0xFF6D5F20, 0xFF6C5B16, 0xFF6B5A15, 0xFF6A5914, 0xFF6A5914, 0xFF6B5A15, 0xFF6B5A15, 0xFF695813, 0xFF675611, 0xFF685712, 0xFF695813, 0xFF6B5A15, 0xFF6D5C17, 0xFF6C5B16,
        0xFF6B5A15, 0xFF685712, 0xFF665510, 0xFF6E5C1C, 0xFF675410, 0xFF664E06, 0xFFA08536, 0xFFD5B560, 0xFFD4AF54, 0xFFD4AC4C, 0xFFD5A948, 0xFFD7AB54, 0xFFDFB24B, 0xFFCD9E28, 0xFFCE9A22, 0xFFE2A938, 0xFFD89E32, 0xFFD29925, 0xFFD8A123, 0xFFD1A027,
        0xFFD6A32D, 0xFFE1AC38, 0xFFD49C2B, 0xFFD6A12F, 0xFFD6A32E, 0xFFD2A42C, 0xFFCFA62A, 0xFFCC9D2B, 0xFFE4AA3F, 0xFFD5922A, 0xFFE2A234, 0xFFD9A72E, 0xFFD6AA31, 0xFFD4A136, 0xFFDDA342, 0xFFDCB02D, 0xFFDCAF32, 0xFFE1B43F, 0xFFC49627, 0xFFB5891E,
        0xFFBA8E21, 0xFFBA8F1F, 0xFFB98E1B, 0xFFBB9526, 0xFFB69023, 0xFFBC9629, 0xFFB28C21, 0xFFB58E27, 0xFFB28B26, 0xFFBE9634, 0xFFB68E2C, 0xFFC2932B, 0xFFB88C23, 0xFFA37A11, 0xFFAD8A26, 0xFFB8993C, 0xFF8E741F, 0xFF594200, 0xFF816A24, 0xFFB49C3A,
        0xFF705508, 0xFF60420C, 0xFF664718, 0xFF61470A, 0xFF69520C, 0xFF64500B, 0xFF5E480C, 0xFF654F13, 0xFF695111, 0xFF6A4F08, 0xFF765B0E, 0xFF674A00, 0xFF75580A, 0xFF7B600F, 0xFF765C08, 0xFF695F00, 0xFF695B03, 0xFF6A530D, 0xFF6D5016, 0xFF7C5C21,
        0xFF6B4A03, 0xFF9E802A, 0xFFAA8E2B, 0xFF766C3B, 0xFFB8B487, 0xFFFFFFDF, 0xFFFFFFE3, 0xFFFFFFE2, 0xFFD5CDA8, 0xFF806D42, 0xFF917647, 0xFFAF9443, 0xFF745A0D, 0xFF665007, 0xFF68540F, 0xFF65500F, 0xFF6C5414, 0xFF6B4F10, 0xFF6F5111, 0xFF674E0C,
        0xFF675108, 0xFF6E5713, 0xFF6B4E12, 0xFF6E500A, 0xFF695500, 0xFF6E701C, 0xFFA5B574, 0xFF3B3B3B, 0xFF020605, 0xFF16211D, 0xFF364643, 0xFF344743, 0xFF3F5250, 0xFF3F4F4E, 0xFF334342, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FDFDFD, 0x00FFFEFF, 0x00FEFCFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF5E6F69, 0xFF414B42, 0xFF0F1A16, 0xFF162125, 0xFF475055, 0xFF141E16, 0xFF4C5332, 0xFF6E763B, 0xFF5E6518, 0xFF766026, 0xFF6F591F, 0xFF6E581E, 0xFF6F591F, 0xFF6C561C,
        0xFF6C561C, 0xFF6E581E, 0xFF6B551B, 0xFF6A5B20, 0xFF67581D, 0xFF65561D, 0xFF6B5A24, 0xFF71602A, 0xFF6D5A22, 0xFF69561C, 0xFF6F5C22, 0xFF6A571C, 0xFF6F5C21, 0xFF715E23, 0xFF6E5B20, 0xFF6C591E, 0xFF6D5A1F, 0xFF6D5A1F, 0xFF6B581D, 0xFF635400,
        0xFFA28D34, 0xFFBFA956, 0xFF6F5C18, 0xFF6A5F29, 0xFFBCB07C, 0xFF806A30, 0xFF694805, 0xFFA48B3B, 0xFFD6C166, 0xFF948125, 0xFF5F4C00, 0xFF755F2D, 0xFF6E572B, 0xFF624D0E, 0xFFA4903A, 0xFFB3903E, 0xFF6A530F, 0xFF6C632C, 0xFF6B6332, 0xFF5A4D18,
        0xFF69561E, 0xFF62511B, 0xFF5B4F1B, 0xFF68571F, 0xFF685721, 0xFFB4A272, 0xFFCFC19C, 0xFF6A6851, 0xFF8B9891, 0xFFA2BFC7, 0xFF93BACB, 0xFF98C5DA, 0xFFC3FBFF, 0xFFB5F0FE, 0xFF78AEBA, 0xFF9AC0CD, 0xFFCFE7F3, 0xFF94ADB2, 0xFF6A8689, 0xFFC2CAA3,
        0xFFACAB6A, 0xFF5D530B, 0xFF68592E, 0xFF777367, 0xFF58615C, 0xFF576361, 0xFF5E696F, 0xFF6D7F89, 0xFF687377, 0xFF5C6262, 0xFF5B6365, 0xFF63747C, 0xFF6C808B, 0xFF56656C, 0xFF5E6464, 0xFF4F6067, 0xFF697A81, 0xFF6E7F86, 0xFF55666D, 0xFF4A5B62,
        0xFF4A5B62, 0xFF76878E, 0xFF7C8D94, 0xFF5D666B, 0xFF485355, 0xFF516365, 0xFF71847E, 0xFF484323, 0xFF6C4F15, 0xFF836E37, 0xFF717550, 0xFFF8FFFD, 0xFFFEFDFF, 0xFFE3EAF0, 0xFFB8CDCE, 0xFFDCEEEE, 0xFF8A8B69, 0xFF68612B, 0xFFD2CCB2, 0xFFFFFFFA,
        0xFFF7F8FC, 0xFFE8EFF5, 0xFFCFDAD2, 0xFF9EA584, 0xFF5F5D24, 0xFF5A4B06, 0xFF78601A, 0xFF61530A, 0xFF7C6D2C, 0xFF7D6036, 0xFF75550C, 0xFF6C5D22, 0xFF59625F, 0xFF475C57, 0xFF3D4645, 0xFF515D59, 0xFF556364, 0xFF506369, 0xFF4D6569, 0xFF496165,
        0xFF475A60, 0xFF627071, 0xFF4F5B57, 0xFF45545B, 0xFF465656, 0xFF576764, 0xFF576766, 0xFF445358, 0xFF45535C, 0xFF415053, 0xFF5A6A67, 0xFF5D7370, 0xFF5D695F, 0xFF475152, 0xFF40525C, 0xFF5F6E5B, 0xFF92914E, 0xFFB4AB60, 0xFF726A3B, 0xFF7A5F12,
        0xFF795E11, 0xFF785D10, 0xFF795E0F, 0xFF7C6112, 0xFF7F6413, 0xFF806514, 0xFF806514, 0xFF81620F, 0xFF80610E, 0xFF80610E, 0xFF80610E, 0xFF81620F, 0xFF80610E, 0xFF7F600D, 0xFF7E5F0C, 0xFF7F600D, 0xFF80610E, 0xFF81620F, 0xFF826310, 0xFF826310,
        0xFF80610E, 0xFF7F600D, 0xFF7D5E0B, 0xFF745B09, 0xFF7A600C, 0xFF997E27, 0xFFCFB055, 0xFFDEBC5B, 0xFFD4AE49, 0xFFD9B249, 0xFFD9B046, 0xFFD9B056, 0xFFCCA23C, 0xFFCEA533, 0xFFD0A232, 0xFFDBAC42, 0xFFD8A842, 0xFFDAAB41, 0xFFD0A330, 0xFFD69C31,
        0xFFDFA237, 0xFFD29226, 0xFFE39F32, 0xFFEBA738, 0xFFE19F2F, 0xFFE1A231, 0xFFD99B2A, 0xFFD9A828, 0xFFDBA124, 0xFFDB9D22, 0xFFE0A52D, 0xFFCEA028, 0xFFD1A52C, 0xFFDBA92E, 0xFFE9AD31, 0xFFDEA728, 0xFFE1A92E, 0xFFE0A932, 0xFFE0A935, 0xFFD49F2F,
        0xFFCF9A2A, 0xFFD5A030, 0xFFDBA737, 0xFFCF9E27, 0xFFD4A32C, 0xFFD7A630, 0xFFD9A734, 0xFFD5A332, 0xFFD6A435, 0xFFD8A538, 0xFFCE9B2E, 0xFFBF9026, 0xFFBF9024, 0xFFBD9327, 0xFFA57F14, 0xFFB6932D, 0xFFB39233, 0xFF9B7B24, 0xFF674800, 0xFF826813,
        0xFFB79E4C, 0xFF7B6319, 0xFF5F4707, 0xFF624C13, 0xFF67521B, 0xFF574409, 0xFF665714, 0xFF6A511B, 0xFF644F0E, 0xFF665609, 0xFF655D08, 0xFFAFAA59, 0xFFB7B471, 0xFFC9C595, 0xFFC9C39F, 0xFFC0B982, 0xFFA0955D, 0xFF7E6E33, 0xFF624D0E, 0xFF6C540E,
        0xFF6D5107, 0xFF816212, 0xFFC0A14F, 0xFF86722D, 0xFF8A773F, 0xFFBFB18A, 0xFFDFD1B7, 0xFFBEAE94, 0xFF8B754E, 0xFF83642D, 0xFFB08D49, 0xFF937829, 0xFF6E5407, 0xFF6A540B, 0xFF685310, 0xFF665110, 0xFF6E5616, 0xFF6B4F10, 0xFF6B4D0D, 0xFF6D5412,
        0xFF665007, 0xFF6A530F, 0xFF6B4E12, 0xFF72540E, 0xFF6B5700, 0xFF646612, 0xFF8A9A59, 0xFF282828, 0xFF000100, 0xFF1A2521, 0xFF3C4C49, 0xFF374A46, 0xFF3D504E, 0xFF3F4F4E, 0xFF354544, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FDFDFD, 0x00FFFEFF, 0x00FFFDFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF677872, 0xFF3B453C, 0xFF091410, 0xFF131E22, 0xFF495257, 0xFF1C261E, 0xFF555C3B, 0xFF71793E, 0xFF5D6417, 0xFF715B21, 0xFF6D571D, 0xFF6D571D, 0xFF6E581E, 0xFF6E581E,
        0xFF725C22, 0xFF755F25, 0xFF725C22, 0xFF6C591F, 0xFF6F5C22, 0xFF6D581F, 0xFF6C571E, 0xFF705B22, 0xFF705A21, 0xFF6A541B, 0xFF665017, 0xFF6B551B, 0xFF715B21, 0xFF755F25, 0xFF715B21, 0xFF6C561C, 0xFF6C561C, 0xFF6F591F, 0xFF715B21, 0xFF808135,
        0xFFA59D52, 0xFFA49753, 0xFFB4A772, 0xFFC5BE94, 0xFFB3A985, 0xFF645127, 0xFFA98B59, 0xFFB8A45D, 0xFF8D7923, 0xFFAD983B, 0xFFA08939, 0xFF61460F, 0xFF6F562D, 0xFF715B2A, 0xFF665318, 0xFFBF9543, 0xFF9D7F37, 0xFF6E5C1E, 0xFF655519, 0xFF705B1A,
        0xFF6B5313, 0xFF675419, 0xFF655B26, 0xFF64551A, 0xFF5F5017, 0xFFAEA16D, 0xFFE1D8B1, 0xFF64644C, 0xFF919E97, 0xFF9FBAC3, 0xFF80A3B6, 0xFF7CA5BB, 0xFFADE0F3, 0xFF9CD5E6, 0xFF95C6D7, 0xFFBDDEEF, 0xFF9AB2BE, 0xFF688084, 0xFF385553, 0xFF949C73,
        0xFF949454, 0xFF534A09, 0xFF554A2A, 0xFF61615F, 0xFF0C1719, 0xFF000908, 0xFF223232, 0xFF596C7B, 0xFF49565E, 0xFF0A0F13, 0xFF00040B, 0xFF364854, 0xFF6A808E, 0xFF404E59, 0xFF00070D, 0xFF00111B, 0xFF465761, 0xFF73848E, 0xFF283943, 0xFF000913,
        0xFF061721, 0xFF63747E, 0xFF6C7D87, 0xFF20272F, 0xFF000209, 0xFF26373F, 0xFF5E7070, 0xFF423E23, 0xFFA08653, 0xFFC7B682, 0xFF7E8563, 0xFFDEEFE5, 0xFFFBFFFF, 0xFFDDEAE1, 0xFFA7C3B5, 0xFFDAF2E5, 0xFF98A174, 0xFF787A39, 0xFFDAE1B8, 0xFFF9FDFC,
        0xFFF9FFFF, 0xFFDDE9F5, 0xFFCADAD7, 0xFFA8B599, 0xFF515620, 0xFF675E1D, 0xFF63500C, 0xFF85763D, 0xFFBFB06D, 0xFFD1B47A, 0xFFC4A249, 0xFFA08C4D, 0xFF4F4F51, 0xFF101A12, 0xFF050500, 0xFF2A3932, 0xFF677485, 0xFF4A5B6F, 0xFF00110F, 0xFF000E0C,
        0xFF304155, 0xFF5C697A, 0xFF32413A, 0xFF000F0E, 0xFF010F18, 0xFF4A5768, 0xFF62707D, 0xFF202F32, 0xFF000908, 0xFF07161B, 0xFF536070, 0xFF4F5D77, 0xFF212629, 0xFF05090A, 0xFF252F3B, 0xFF606159, 0xFF826E2F, 0xFFA7882B, 0xFFAA9033, 0xFFBD9331,
        0xFFBE9430, 0xFFC19733, 0xFFC49A36, 0xFFC79D37, 0xFFC89E38, 0xFFC79E35, 0xFFC59C33, 0xFFC79A33, 0xFFC89B34, 0xFFC99C35, 0xFFC89B34, 0xFFC89B34, 0xFFC79A33, 0xFFC69932, 0xFFC79A33, 0xFFC2952E, 0xFFC3962F, 0xFFC3962F, 0xFFC49730, 0xFFC3962F,
        0xFFC3962F, 0xFFC2952E, 0xFFC1942D, 0xFFB99630, 0xFFB5922C, 0xFFCEAB43, 0xFFE1BC52, 0xFFCEA93E, 0xFFCDA73A, 0xFFDDB748, 0xFFD7B240, 0xFFD3A744, 0xFFCBA239, 0xFFDDB74A, 0xFFD0AB43, 0xFFB99535, 0xFFA9862A, 0xFFB69534, 0xFFB69631, 0xFFBFAA51,
        0xFFBFA649, 0xFFC9AB48, 0xFFC2A136, 0xFFAB8719, 0xFFBD992B, 0xFFB49226, 0xFFB9962E, 0xFFAF962F, 0xFFA98D20, 0xFFBB9F34, 0xFFBBA447, 0xFFB8A85A, 0xFFBFB05F, 0xFFC2AA46, 0xFFB4941D, 0xFFE8AA33, 0xFFF3B53E, 0xFFE0A42A, 0xFFDEA22A, 0xFFDEA32D,
        0xFFE0A432, 0xFFE2A83A, 0xFFD99F33, 0xFFDEA82A, 0xFFDFA92D, 0xFFD59F23, 0xFFE0AA30, 0xFFDEA82E, 0xFFDDA62F, 0xFFD39C25, 0xFFBF8813, 0xFFA87C0F, 0xFFB18518, 0xFFC69B2B, 0xFFB68C1C, 0xFFAD8315, 0xFFB38A20, 0xFFC19733, 0xFF9B710F, 0xFF644400,
        0xFF8F741B, 0xFFAC9535, 0xFF7D6714, 0xFF5E480E, 0xFF614C1F, 0xFF6C5B25, 0xFF5F520C, 0xFF6D550F, 0xFF5E4C0A, 0xFF918D4D, 0xFFCAD498, 0xFFB9CB99, 0xFFC6D9B9, 0xFFE6F8EA, 0xFFE8F8F7, 0xFFFDFCFA, 0xFFFAF7E8, 0xFFE2DEB9, 0xFFA49B62, 0xFF6A5914,
        0xFF624A00, 0xFF6D4F07, 0xFFA4833E, 0xFFBD9E44, 0xFF92742C, 0xFF806132, 0xFF826547, 0xFF775839, 0xFF8C6838, 0xFFB68E46, 0xFFBD9236, 0xFF765B0E, 0xFF694F04, 0xFF6C550F, 0xFF66510E, 0xFF65500F, 0xFF6E5616, 0xFF6B4F0E, 0xFF6B4E0C, 0xFF6F5614,
        0xFF665007, 0xFF67500C, 0xFF6A4D11, 0xFF745610, 0xFF6E5A03, 0xFF626410, 0xFF7C8C4B, 0xFF1C1C1C, 0xFF000100, 0xFF1A2521, 0xFF3E4E4B, 0xFF394C48, 0xFF3D504E, 0xFF3F4F4E, 0xFF344443, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FDFDFD, 0x00FFFEFF, 0x00FFFDFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF677872, 0xFF384239, 0xFF05100C, 0xFF131E22, 0xFF4A5358, 0xFF1F2921, 0xFF5B6241, 0xFF757D42, 0xFF62691C, 0xFF6E581E, 0xFF6F591F, 0xFF6F591F, 0xFF6D571D, 0xFF6B551B,
        0xFF6D571D, 0xFF6F591F, 0xFF6E581E, 0xFF725B25, 0xFF715B22, 0xFF6F561D, 0xFF6E561A, 0xFF73591E, 0xFF765B23, 0xFF755A23, 0xFF755723, 0xFF735A21, 0xFF71581F, 0xFF6F561D, 0xFF70571E, 0xFF745B22, 0xFF755C23, 0xFF725920, 0xFF6E551C, 0xFFD4D3A3,
        0xFFE8E0AF, 0xFFCCBE8D, 0xFFAFA37B, 0xFF858261, 0xFF828363, 0xFFB4AF87, 0xFFBBAD7C, 0xFF67541C, 0xFF5E4A05, 0xFF76600D, 0xFFBFA455, 0xFFBCA061, 0xFFAD9263, 0xFFB5A375, 0xFFA99C68, 0xFFB99847, 0xFFE4C06A, 0xFFAD8830, 0xFF765504, 0xFF6E5713,
        0xFF66561B, 0xFF6E5D27, 0xFF705B26, 0xFF74652A, 0xFF63571D, 0xFF8C804C, 0xFFFBF4C8, 0xFF5D6043, 0xFF7B877D, 0xFFB7CBD2, 0xFF87A2B3, 0xFFA0C1D4, 0xFF8FBACB, 0xFFA0D1E2, 0xFFA6D1E2, 0xFFB5D2E2, 0xFFA3B5BF, 0xFF596D6C, 0xFF314B42, 0xFF8B9266,
        0xFF717133, 0xFF534A11, 0xFF5A523D, 0xFF5B5E67, 0xFF071820, 0xFF001211, 0xFF2B3B3A, 0xFF6B8387, 0xFF465454, 0xFF07100B, 0xFF081410, 0xFF3E5356, 0xFF62797F, 0xFF36484A, 0xFF0A150F, 0xFF0C1E20, 0xFF607274, 0xFF849698, 0xFF324446, 0xFF041618,
        0xFF182A2C, 0xFF627476, 0xFF697B7D, 0xFF252E2B, 0xFF08130F, 0xFF273A38, 0xFF596D64, 0xFF474421, 0xFF836B2B, 0xFF8C7D3A, 0xFF67713D, 0xFFEEFFFD, 0xFFE8EFF5, 0xFFB8C4BA, 0xFF99B4A5, 0xFFDAF1E9, 0xFF7C8661, 0xFF575D21, 0xFFDCE8C4, 0xFFFBFFFD,
        0xFFE7F0F5, 0xFFD0DEE7, 0xFFD4E7E3, 0xFFAEBDA0, 0xFF525923, 0xFF615918, 0xFF82702E, 0xFFDEC38C, 0xFFE8D185, 0xFFD7B669, 0xFFCAA53D, 0xFFA8914E, 0xFF525159, 0xFF151F1E, 0xFF0F1104, 0xFF293D34, 0xFF637579, 0xFF44575D, 0xFF000F08, 0xFF00140D,
        0xFF34474D, 0xFF596B6F, 0xFF31453C, 0xFF04150D, 0xFF081818, 0xFF39484D, 0xFF4D5C5F, 0xFF21322C, 0xFF03140C, 0xFF0B1B18, 0xFF4E5D62, 0xFF586D80, 0xFF1C2628, 0xFF00070F, 0xFF1F2A3E, 0xFF5E5E56, 0xFF7E661E, 0xFFA2790D, 0xFFBC9329, 0xFFC79728,
        0xFFCB9B2C, 0xFFCE9F2D, 0xFFCFA02E, 0xFFD0A12D, 0xFFD0A12D, 0xFFCD9E2A, 0xFFC99A24, 0xFFCE9C2B, 0xFFCF9D2C, 0xFFD19F2E, 0xFFD19F2E, 0xFFCF9D2C, 0xFFCE9C2B, 0xFFCF9D2C, 0xFFD09E2D, 0xFFD19F2E, 0xFFD19F2E, 0xFFD09E2D, 0xFFD09E2D, 0xFFD09E2D,
        0xFFCF9D2C, 0xFFCF9D2C, 0xFFCF9D2C, 0xFFCD9F30, 0xFFC69829, 0xFFCEA235, 0xFFD0A638, 0xFFCCA236, 0xFFD7AE42, 0xFFD8B245, 0xFFC9A338, 0xFFD3A132, 0xFFE3B448, 0xFFD4AA44, 0xFF9C791B, 0xFF6B4D00, 0xFF715702, 0xFFA18730, 0xFFAE973A, 0xFFAFB792,
        0xFFA0A678, 0xFFB8B77E, 0xFFACA763, 0xFF6A6019, 0xFF685B15, 0xFF5A4C0D, 0xFF594D11, 0xFF5C4B03, 0xFF847323, 0xFFA89C4A, 0xFFA49F65, 0xFFA9AA88, 0xFFACAC86, 0xFFAAA059, 0xFF96831C, 0xFFBA851B, 0xFFDBA739, 0xFFE2AF3A, 0xFFD4A32C, 0xFFD5A42D,
        0xFFCD9E2C, 0xFFD4A33A, 0xFFD3A23C, 0xFFCDA02D, 0xFFD4A734, 0xFFD3A633, 0xFFD2A532, 0xFFCC9F2C, 0xFFB38515, 0xFFBF9121, 0xFFB88A1A, 0xFFB18A25, 0xFFA98219, 0xFFAE8519, 0xFFC49C2D, 0xFFB98F1F, 0xFFB68A1D, 0xFFB3871C, 0xFFC99A32, 0xFF99751F,
        0xFF674600, 0xFF886A08, 0xFFBCA146, 0xFF745A10, 0xFF685012, 0xFF685314, 0xFF6B5814, 0xFF544500, 0xFFA59C5B, 0xFFDADBBC, 0xFF79867F, 0xFF64797A, 0xFFBBD3D5, 0xFFE7FCFD, 0xFFF1FFFF, 0xFFDEE6FD, 0xFFE7EDFB, 0xFFF8FDF7, 0xFFFBFADE, 0xFFAFA778,
        0xFF695719, 0xFF6D520B, 0xFF7A5A11, 0xFFB28F33, 0xFFC2A150, 0xFFBB9C5B, 0xFFAD8E57, 0xFFAA8A51, 0xFFB89551, 0xFFB18B38, 0xFF835B00, 0xFF6B4F05, 0xFF694E07, 0xFF6B5410, 0xFF634E0D, 0xFF655011, 0xFF6D5515, 0xFF6B4F0E, 0xFF6E510F, 0xFF6B5210,
        0xFF665007, 0xFF69520E, 0xFF6C4F13, 0xFF71530D, 0xFF6F5B04, 0xFF6C6E1A, 0xFF829251, 0xFF1E1E1E, 0xFF000100, 0xFF18231F, 0xFF3A4A47, 0xFF384B47, 0xFF3D504E, 0xFF3F4F4E, 0xFF314140, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FCFCFC, 0x00FFFDFE, 0x00FEFCFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF61726C, 0xFF414B42, 0xFF0A1511, 0xFF152024, 0xFF454E53, 0xFF162018, 0xFF545B3A, 0xFF71793E, 0xFF646B1E, 0xFF735D24, 0xFF725C23, 0xFF6E581F, 0xFF6C561D, 0xFF6E581F,
        0xFF705A21, 0xFF715B22, 0xFF725C23, 0xFF6E551F, 0xFF684F16, 0xFF6C5215, 0xFF775E1C, 0xFF795D1C, 0xFF725519, 0xFF6F521A, 0xFF735320, 0xFF70571E, 0xFF71581F, 0xFF725920, 0xFF725920, 0xFF70571E, 0xFF6E551C, 0xFF6D541B, 0xFF6D541B, 0xFF746D39,
        0xFF6D5D29, 0xFF7A6633, 0xFFA99C72, 0xFF7A7B5C, 0xFFA5AD96, 0xFFABB499, 0xFF838766, 0xFF78653B, 0xFF6D5821, 0xFF573E00, 0xFF745912, 0xFFAD9150, 0xFFCEB87E, 0xFFBFAE76, 0xFFB4A86C, 0xFFB4A457, 0xFFD9B253, 0xFFF2BA4B, 0xFFA97915, 0xFF604800,
        0xFF625825, 0xFF6A582A, 0xFF6F4B17, 0xFF66551D, 0xFF67581F, 0xFF6C612B, 0xFFF3EDBD, 0xFFAEAF8D, 0xFF646B59, 0xFF7E8D8A, 0xFF76878E, 0xFFBFD7E3, 0xFFA8CAD6, 0xFF95BDC9, 0xFF97BBC9, 0xFF5E747F, 0xFF536162, 0xFF48584B, 0xFF2A4029, 0xFF818258,
        0xFF4C480B, 0xFF413801, 0xFF4B4633, 0xFF686E7A, 0xFF60727C, 0xFF5B6F6E, 0xFF5D6D6A, 0xFF6A8388, 0xFF687878, 0xFF5E6965, 0xFF53625F, 0xFF576F73, 0xFF708990, 0xFF627579, 0xFF616D69, 0xFF56696D, 0xFF6D8084, 0xFF76898D, 0xFF73868A, 0xFF677A7E,
        0xFF697C80, 0xFF75888C, 0xFF7C8F93, 0xFF7C8783, 0xFF6C7876, 0xFF617373, 0xFF687973, 0xFF57512D, 0xFF886B27, 0xFF806D21, 0xFF6B6F32, 0xFFE8F3F5, 0xFFAFAEB6, 0xFF96988D, 0xFFACBCAF, 0xFFEEFAFA, 0xFF81806B, 0xFF58592D, 0xFFE6EED6, 0xFFEDF0DB,
        0xFFCFD5C9, 0xFFB6C1BB, 0xFFE4F2E1, 0xFFA4B088, 0xFF4C4D11, 0xFF7E712D, 0xFFDBC47E, 0xFFF4CC86, 0xFFE5C36A, 0xFFCCA24C, 0xFFB48B1F, 0xFF8B7333, 0xFF5C606C, 0xFF4B5E64, 0xFF475351, 0xFF485C65, 0xFF5C716C, 0xFF4A5F5A, 0xFF384950, 0xFF3D4E55,
        0xFF3E534E, 0xFF506560, 0xFF3C5059, 0xFF303F46, 0xFF3D4D4C, 0xFF50615B, 0xFF5A6A67, 0xFF415055, 0xFF34434A, 0xFF384848, 0xFF576862, 0xFF4B6B66, 0xFF304540, 0xFF304250, 0xFF3B4F67, 0xFF646B5B, 0xFF8B7623, 0xFFAC8214, 0xFFB98B2D, 0xFFCC9E2E,
        0xFFD1A333, 0xFFD3A633, 0xFFD2A532, 0xFFD1A42F, 0xFFD1A42F, 0xFFCFA22D, 0xFFCA9D26, 0xFFCFA22F, 0xFFD0A330, 0xFFD2A532, 0xFFD2A532, 0xFFD1A431, 0xFFD0A330, 0xFFD0A330, 0xFFD1A431, 0xFFCFA22F, 0xFFCEA12E, 0xFFCDA02D, 0xFFCDA02D, 0xFFCC9F2C,
        0xFFCDA02D, 0xFFCDA02D, 0xFFCDA02D, 0xFFD19D30, 0xFFD09D30, 0xFFD19E33, 0xFFD5A43B, 0xFFDEB14A, 0xFFDEB34D, 0xFFD2A844, 0xFFD0A644, 0xFFE3AF39, 0xFFDBAB3F, 0xFFA37A1E, 0xFF704F00, 0xFF644A00, 0xFF867024, 0xFFAC9748, 0xFF7D6819, 0xFF4F4B25,
        0xFF4D4717, 0xFF615519, 0xFF9D8C46, 0xFF8B752A, 0xFF5F4700, 0xFF6F5614, 0xFF6B5315, 0xFF73570D, 0xFFA78B37, 0xFF907723, 0xFF5E4C0E, 0xFF473B15, 0xFF54461F, 0xFF7E681F, 0xFFB4932A, 0xFF7A5C02, 0xFF987B1D, 0xFFB49834, 0xFFB59D33, 0xFFBAA238,
        0xFF9B831F, 0xFF796003, 0xFF7C6309, 0xFF7F640B, 0xFF80650C, 0xFF876C13, 0xFF866B12, 0xFF9F842B, 0xFF7D6207, 0xFF9D8227, 0xFF957A1F, 0xFFA48834, 0xFF927720, 0xFF9D8024, 0xFF9D7F1D, 0xFFB89634, 0xFFAF8B2A, 0xFFA37D1C, 0xFFB28B2C, 0xFFBC993B,
        0xFF94721B, 0xFF745100, 0xFF866514, 0xFFB69840, 0xFF745902, 0xFF775B11, 0xFF6C4E10, 0xFF887F34, 0xFFE8E4B7, 0xFF878984, 0xFF8B93A6, 0xFFCDDCEF, 0xFFE4F2FD, 0xFFE2EBF2, 0xFFAEB2BB, 0xFFD0DFF2, 0xFFC3D2E5, 0xFFBBC7D7, 0xFFE3E8EE, 0xFFF3F0E1,
        0xFF93865A, 0xFF654E08, 0xFF735800, 0xFF694C00, 0xFF896E1F, 0xFFA38A38, 0xFFA7913B, 0xFFA48C36, 0xFF8C721D, 0xFF6F5300, 0xFF6B4C00, 0xFF6D500A, 0xFF6B500B, 0xFF6A5310, 0xFF665112, 0xFF695415, 0xFF6C5414, 0xFF6A4F0C, 0xFF6F520E, 0xFF6A510F,
        0xFF685209, 0xFF6B5410, 0xFF6D5014, 0xFF6E500A, 0xFF6F5B04, 0xFF70721E, 0xFF7F8F4E, 0xFF1B1B1B, 0xFF000403, 0xFF1A2521, 0xFF394946, 0xFF374A46, 0xFF3B4E4C, 0xFF40504F, 0xFF314140, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FCFCFC, 0x00FFFDFE, 0x00FEFCFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF5B6C66, 0xFF404A41, 0xFF07120E, 0xFF152024, 0xFF434C51, 0xFF151F17, 0xFF545B3A, 0xFF6D753A, 0xFF60671A, 0xFF725C23, 0xFF705A21, 0xFF6C561D, 0xFF6F5920, 0xFF745E25,
        0xFF725C23, 0xFF705A21, 0xFF735D24, 0xFF705623, 0xFF735A21, 0xFF755D1D, 0xFF705812, 0xFF6B530D, 0xFF725617, 0xFF785B23, 0xFF755725, 0xFF735D23, 0xFF705A20, 0xFF6E581E, 0xFF6F591F, 0xFF705A20, 0xFF6D571D, 0xFF6A541A, 0xFF6A541A, 0xFF635A0D,
        0xFF6C5A0E, 0xFF664E08, 0xFF62511D, 0xFFB6B394, 0xFF777E6E, 0xFF717B70, 0xFFC4CABC, 0xFFA79162, 0xFF604A18, 0xFF766027, 0xFF644C10, 0xFF786226, 0xFFE0CC8F, 0xFFC2B370, 0xFFA49751, 0xFFA7A05C, 0xFFB29339, 0xFFF2BD4D, 0xFFF8C457, 0xFFAA8B38,
        0xFF746428, 0xFF6E5924, 0xFF7C5923, 0xFF6B5621, 0xFF715E26, 0xFF574A13, 0xFFA9A26C, 0xFFFFFCD1, 0xFF797C5D, 0xFF3F4334, 0xFF626961, 0xFF616F70, 0xFF546C6E, 0xFF86A3A9, 0xFF96AFB6, 0xFF485659, 0xFF464C42, 0xFF374025, 0xFF5B6B44, 0xFF615932,
        0xFF473E03, 0xFF473C04, 0xFF48432F, 0xFF6D747E, 0xFF8FA1AB, 0xFF90A2A2, 0xFF8D9B9B, 0xFF6A8495, 0xFF7B8B98, 0xFF848E97, 0xFF76848F, 0xFF778C9D, 0xFF98B1C5, 0xFF8C9FAE, 0xFF828F97, 0xFF8192A2, 0xFF6E7F8F, 0xFF748595, 0xFF9BACBC, 0xFF96A7B7,
        0xFF95A6B6, 0xFF93A4B4, 0xFF95A6B6, 0xFF839099, 0xFF85929B, 0xFF788897, 0xFF677278, 0xFF55472C, 0xFFB38B4D, 0xFFB39348, 0xFF645C1B, 0xFF8A8F66, 0xFF938C62, 0xFFBDB779, 0xFF8E965A, 0xFF90926D, 0xFF726B35, 0xFF605A10, 0xFF858A52, 0xFFB9B683,
        0xFFA9A780, 0xFF858869, 0xFFAAB08C, 0xFF72723C, 0xFF766C25, 0xFFDAC377, 0xFFF3D285, 0xFFE5B960, 0xFFD6AF50, 0xFFD1A453, 0xFFC49A34, 0xFF8E7938, 0xFF606972, 0xFF637C83, 0xFF6A7B82, 0xFF5B6E8C, 0xFF61767B, 0xFF5C6E70, 0xFF616D85, 0xFF626E86,
        0xFF4A5C5E, 0xFF54696E, 0xFF4B5E7C, 0xFF5B677D, 0xFF5B6974, 0xFF6E7D80, 0xFF84939A, 0xFF6B788B, 0xFF5F6B83, 0xFF5D6A7A, 0xFF5F6E73, 0xFF476B5F, 0xFF536D62, 0xFF6B818C, 0xFF5C758B, 0xFF5C6955, 0xFF7A6814, 0xFFA67B15, 0xFFBA8736, 0xFFC19729,
        0xFFC79D2F, 0xFFC99F31, 0xFFC69C2C, 0xFFC69C2C, 0xFFC89F2D, 0xFFC79E2C, 0xFFC39A28, 0xFFC89D2C, 0xFFC99E2D, 0xFFCA9F2E, 0xFFCBA02F, 0xFFCA9F2E, 0xFFCA9F2E, 0xFFCA9F2E, 0xFFCA9F2E, 0xFFCEA332, 0xFFCDA231, 0xFFCDA231, 0xFFCCA130, 0xFFCCA130,
        0xFFCCA130, 0xFFCCA130, 0xFFCCA130, 0xFFD29F32, 0xFFD5A237, 0xFFCE9B32, 0xFFCF9E37, 0xFFD4A43E, 0xFFC79937, 0xFFC89A3A, 0xFFDEB050, 0xFFDAAF3F, 0xFFA47D20, 0xFF8B6D25, 0xFF725D24, 0xFF867A40, 0xFF887E41, 0xFFB7AB6E, 0xFF76682B, 0xFF7B7841,
        0xFF6E682C, 0xFF827833, 0xFF9B8B40, 0xFFAE9D4F, 0xFF79671B, 0xFF816F25, 0xFF86752F, 0xFF786E29, 0xFFAFA158, 0xFF91833C, 0xFF867D42, 0xFF706F43, 0xFF797B4C, 0xFF7C7433, 0xFFC6B45E, 0xFF736C28, 0xFF867F39, 0xFF757128, 0xFF848036, 0xFF8C883E,
        0xFFA3A156, 0xFF77742D, 0xFF86833C, 0xFF797339, 0xFF736D33, 0xFF6D672B, 0xFF767034, 0xFFA49E60, 0xFF898345, 0xFFA19C5C, 0xFF858040, 0xFFA6975C, 0xFF897B3E, 0xFFB6A764, 0xFF7A6C23, 0xFFB4A459, 0xFF948238, 0xFFB3A159, 0xFF97833C, 0xFFB5A351,
        0xFFB9A25F, 0xFF977E48, 0xFF7E6427, 0xFFB79E4C, 0xFFA89134, 0xFF745A06, 0xFF6D500C, 0xFFC7C49B, 0xFFD6D5C3, 0xFFB0B1B6, 0xFFEDF4FE, 0xFFF7FFFF, 0xFFF9FFFD, 0xFFA6A9AE, 0xFF787588, 0xFFADC0BE, 0xFFE8FAFF, 0xFFC5D4E9, 0xFF858DA4, 0xFFCECFD4,
        0xFFD4CBAE, 0xFF6B5A15, 0xFF6F5800, 0xFF725A1C, 0xFF644D07, 0xFF644F00, 0xFF5F4E00, 0xFF625200, 0xFF635100, 0xFF604D01, 0xFF725D1A, 0xFF6F520E, 0xFF6C500F, 0xFF6A5212, 0xFF6B5617, 0xFF6D5819, 0xFF6C5414, 0xFF6A4F0C, 0xFF6B4E0A, 0xFF6C5311,
        0xFF685209, 0xFF6A530F, 0xFF6D5014, 0xFF6D4F09, 0xFF6F5B04, 0xFF6C6E1A, 0xFF6B7B3A, 0xFF0F0F0F, 0xFF000403, 0xFF1D2824, 0xFF3C4C49, 0xFF374A46, 0xFF394C4A, 0xFF415150, 0xFF344443, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FDFDFD, 0x00FFFEFF, 0x00FFFDFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF5D6E68, 0xFF3B453C, 0xFF010C08, 0xFF131E22, 0xFF464F54, 0xFF1D271F, 0xFF5E6544, 0xFF71793E, 0xFF5F6619, 0xFF715A24, 0xFF705923, 0xFF6D5620, 0xFF6E5721, 0xFF6F5822,
        0xFF6D5620, 0xFF765F29, 0xFF8A733D, 0xFFAE9763, 0xFFAC965C, 0xFFA28D4A, 0xFF8B752A, 0xFF735D12, 0xFF725917, 0xFF775C24, 0xFF745826, 0xFF6D5A1F, 0xFF736025, 0xFF79662B, 0xFF776429, 0xFF6E5B20, 0xFF6A571C, 0xFF736025, 0xFF806D32, 0xFF6C641B,
        0xFF6D5B0F, 0xFF755B11, 0xFF6A5418, 0xFF635B2D, 0xFF8B8C6C, 0xFFA2A586, 0xFF6D6C4D, 0xFF68510D, 0xFF675213, 0xFF66511A, 0xFF62501E, 0xFF6B5A26, 0xFF76662B, 0xFF5C4B06, 0xFF604E04, 0xFF5C4B15, 0xFF79621F, 0xFFAE8E39, 0xFFE6BC58, 0xFFDDB049,
        0xFFA0791C, 0xFF785C13, 0xFF715E24, 0xFF735728, 0xFF745D29, 0xFF63521A, 0xFF746931, 0xFFE1DBA7, 0xFFBDBA8F, 0xFF656243, 0xFF525039, 0xFF4E5148, 0xFF3B4841, 0xFF425654, 0xFF687877, 0xFF474C45, 0xFF3A3924, 0xFF53552D, 0xFF626D35, 0xFF433512,
        0xFF4D3E05, 0xFF584910, 0xFF6A644A, 0xFF626A6D, 0xFF42535A, 0xFF364646, 0xFF576163, 0xFF6D8796, 0xFF5C6E78, 0xFF303D43, 0xFF29383F, 0xFF506775, 0xFF7B96A7, 0xFF5B717E, 0xFF2F3C42, 0xFF2A3D4B, 0xFF556876, 0xFF7F92A0, 0xFF617482, 0xFF2E414F,
        0xFF3A4D5B, 0xFF708391, 0xFF879AA8, 0xFF48565F, 0xFF324049, 0xFF435360, 0xFF6D7577, 0xFF5A4625, 0xFFA27226, 0xFFB1852C, 0xFF6A5502, 0xFF454300, 0xFF8D7D28, 0xFFE8D971, 0xFF777310, 0xFF574E0B, 0xFFA79447, 0xFFBCAE4D, 0xFF918E3F, 0xFFA29244,
        0xFF695E1C, 0xFF615A23, 0xFF58511A, 0xFF6F651E, 0xFFD5BF6C, 0xFFFAD47F, 0xFFE1B15F, 0xFFE6C65F, 0xFFBB9C41, 0xFFAB8242, 0xFFBB933C, 0xFF9B8846, 0xFF535C5B, 0xFF304548, 0xFF29363F, 0xFF3C5569, 0xFF637986, 0xFF55666E, 0xFF313F4A, 0xFF34424D,
        0xFF43545C, 0xFF586E7B, 0xFF3C5569, 0xFF334249, 0xFF2F3D48, 0xFF5A6778, 0xFF798696, 0xFF526168, 0xFF39484D, 0xFF38464F, 0xFF515E6E, 0xFF59717D, 0xFF3F504A, 0xFF364947, 0xFF394F5C, 0xFF5D6558, 0xFF856C2A, 0xFFAA771B, 0xFFB9812A, 0xFFC59529,
        0xFFCD9D31, 0xFFD0A034, 0xFFCC9C2E, 0xFFCC9C2E, 0xFFCF9F30, 0xFFCF9F30, 0xFFCA9A2B, 0xFFCC9F2C, 0xFFCC9F2C, 0xFFCC9F2C, 0xFFCDA02D, 0xFFCEA12E, 0xFFCFA22F, 0xFFCEA12E, 0xFFCDA02D, 0xFFCC9F2C, 0xFFCC9F2C, 0xFFCC9F2C, 0xFFCC9F2C, 0xFFCC9F2C,
        0xFFCC9F2C, 0xFFCB9E2B, 0xFFCB9E2B, 0xFFC89B28, 0xFFD0A330, 0xFFCD9F2F, 0xFFD3A535, 0xFFD4A637, 0xFFD4A539, 0xFFDFB046, 0xFFDBAC42, 0xFF90791B, 0xFF75631B, 0xFFCAC196, 0xFF9D9E7F, 0xFFC3CAA9, 0xFF7F8760, 0xFFC4C9A0, 0xFF999A72, 0xFFB9C5B1,
        0xFF909984, 0xFFC1C6B0, 0xFF92947E, 0xFFB7B9A1, 0xFF9FA388, 0xFFA2A88C, 0xFFA7B093, 0xFF9BAC82, 0xFFB6BE97, 0xFF989C77, 0xFFB1B896, 0xFF91A582, 0xFFB2C8A4, 0xFF80906B, 0xFFC4CAA6, 0xFF848960, 0xFFC0C59C, 0xFF8C9169, 0xFFB4BA94, 0xFF848B62,
        0xFFC2CBA0, 0xFF889162, 0xFFADB785, 0xFF979B7A, 0xFFAFB392, 0xFF9DA17E, 0xFFA2A681, 0xFFA9AE86, 0xFF979C73, 0xFFA7AD81, 0xFF8F9569, 0xFFC3C3A1, 0xFF90906C, 0xFFC0C197, 0xFF797B4C, 0xFFBEC290, 0xFF808354, 0xFFC2C598, 0xFF808358, 0xFFC3C69B,
        0xFF7F815A, 0xFFC6C49E, 0xFF8F885E, 0xFFC5BC85, 0xFF9B8E4A, 0xFF645708, 0xFF837421, 0xFFEEF4CE, 0xFFB7BBAD, 0xFFE2E5EE, 0xFFF3FAFF, 0xFFECF8EE, 0xFFC0CFBC, 0xFFA3B3A8, 0xFF8C999F, 0xFF708171, 0xFFC8D8D5, 0xFFC3D3E0, 0xFF647182, 0xFF898E92,
        0xFFFFFBE2, 0xFF8E7F46, 0xFF644E02, 0xFF695017, 0xFF684E11, 0xFF6F5613, 0xFF6A5208, 0xFF664E04, 0xFF705A11, 0xFF6C5714, 0xFF624E0F, 0xFF6D4F0F, 0xFF6C5011, 0xFF6B5315, 0xFF6A5418, 0xFF685314, 0xFF695111, 0xFF6C510C, 0xFF6A4D07, 0xFF6C5311,
        0xFF675108, 0xFF68510D, 0xFF6C4F13, 0xFF6C4E08, 0xFF6F5B04, 0xFF6C6E1A, 0xFF627231, 0xFF0A0A0A, 0xFF000302, 0xFF1B2622, 0xFF3A4A47, 0xFF364945, 0xFF384B49, 0xFF415150, 0xFF334342, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF60716B, 0xFF444E45, 0xFF040F0B, 0xFF121D21, 0xFF454E53, 0xFF202A22, 0xFF656C4B, 0xFF777F44, 0xFF646B1E, 0xFF715A24, 0xFF735C26, 0xFF715A24, 0xFF705923, 0xFF705923,
        0xFF7B642E, 0xFFA28B55, 0xFFD1BA84, 0xFFBDA873, 0xFFB09C61, 0xFFBBA863, 0xFFC7B468, 0xFFA9964A, 0xFF7C6521, 0xFF6D541B, 0xFF765C29, 0xFF726226, 0xFFB1A165, 0xFFE3D397, 0xFFCBBB7F, 0xFF88783C, 0xFF706024, 0xFFA9995D, 0xFFF3E3A7, 0xFFDFDBAC,
        0xFF93844D, 0xFF6A5013, 0xFF6A5013, 0xFF6B5C21, 0xFF9B965C, 0xFF8D8949, 0xFF716A22, 0xFFB39B43, 0xFF7A6419, 0xFF6D5A20, 0xFF7E7043, 0xFF66582B, 0xFF5F4F1B, 0xFF675314, 0xFF735C16, 0xFF745527, 0xFF988A5D, 0xFFD4CB92, 0xFFD3B55B, 0xFFEEB33B,
        0xFFFBBD44, 0xFF9C7D22, 0xFF616123, 0xFF75562A, 0xFF6D5322, 0xFF6B5820, 0xFF504408, 0xFF675E25, 0xFFEAE4B2, 0xFFD9D1AA, 0xFF76704E, 0xFF4C4A3B, 0xFF333B2C, 0xFF34443A, 0xFF3D4840, 0xFF3E3E32, 0xFF494427, 0xFF60602E, 0xFF414707, 0xFF534020,
        0xFF57440C, 0xFF5F4F13, 0xFF9E977A, 0xFF797F7F, 0xFF0B1A1D, 0xFF000405, 0xFF33383C, 0xFF6E8C8C, 0xFF516661, 0xFF000B00, 0xFF000800, 0xFF355250, 0xFF648284, 0xFF3B5450, 0xFF000600, 0xFF000F0C, 0xFF4D6360, 0xFF7E9491, 0xFF1F3532, 0xFF000704,
        0xFF0B211E, 0xFF5B716E, 0xFF687E7B, 0xFF11261F, 0xFF000600, 0xFF1D302C, 0xFF7F8778, 0xFF735E29, 0xFF9F6A0A, 0xFFBF901E, 0xFFAC9224, 0xFF655611, 0xFF775913, 0xFFCCAE54, 0xFF746210, 0xFF5F4719, 0xFFA98853, 0xFFB69953, 0xFF8C7C48, 0xFF7E6805,
        0xFF645200, 0xFF63550E, 0xFF766722, 0xFFD6C374, 0xFFF1D278, 0xFFE0B358, 0xFFF3BA63, 0xFFC3AE43, 0xFFB39D47, 0xFFB49060, 0xFFCBA459, 0xFFA59250, 0xFF41473D, 0xFF051610, 0xFF03080E, 0xFF223F3A, 0xFF708691, 0xFF506168, 0xFF000700, 0xFF000F00,
        0xFF42535A, 0xFF647A85, 0xFF2C4944, 0xFF000900, 0xFF061616, 0xFF4E5B6E, 0xFF627079, 0xFF223522, 0xFF000E00, 0xFF091A10, 0xFF5C6979, 0xFF5F6891, 0xFF1F2427, 0xFF000800, 0xFF17292D, 0xFF5E5E5C, 0xFF886636, 0xFFAE751C, 0xFFCC8E2B, 0xFFD09A2E,
        0xFFD8A236, 0xFFDBA539, 0xFFD7A134, 0xFFD59F32, 0xFFD8A333, 0xFFD7A232, 0xFFD29D2D, 0xFFD3A02A, 0xFFD29F29, 0xFFD29F29, 0xFFD4A12B, 0xFFD6A32D, 0xFFD6A32D, 0xFFD5A22C, 0xFFD3A02A, 0xFFD4A12B, 0xFFD4A12B, 0xFFD5A22C, 0xFFD5A22C, 0xFFD5A22C,
        0xFFD4A12B, 0xFFD3A02A, 0xFFD3A02A, 0xFFCEA62C, 0xFFD4AC32, 0xFFD0A62D, 0xFFCFA52D, 0xFFCC9F28, 0xFFDCAD37, 0xFFDDAE38, 0xFFA67401, 0xFF5A5504, 0xFF54531D, 0xFFD0D8C1, 0xFF627369, 0xFFB7CFC1, 0xFF6B836B, 0xFFB2C4AA, 0xFF738168, 0xFFB0C19D,
        0xFF748362, 0xFFABB89A, 0xFF7F896E, 0xFFA9B398, 0xFF859475, 0xFF9FB390, 0xFF819A73, 0xFF8B9980, 0xFF9FA28F, 0xFF968F7F, 0xFFB1AF9A, 0xFF757E5F, 0xFFAFBE9F, 0xFF707966, 0xFFBBBCB6, 0xFF73795F, 0xFFBEC3AD, 0xFF686F5F, 0xFFBEC4B6, 0xFF676F60,
        0xFFB4BDA8, 0xFF6E7B5D, 0xFFAFBD9A, 0xFF7B806A, 0xFFAAAF99, 0xFF8D927B, 0xFF999F85, 0xFF898F73, 0xFF8E9576, 0xFF989F7E, 0xFF848B6A, 0xFF9CA594, 0xFF77806B, 0xFFA0AD93, 0xFF6F7C60, 0xFFA8B798, 0xFF5D6D50, 0xFFAABAA0, 0xFF63755D, 0xFFA0B5AE,
        0xFF637865, 0xFFA2B295, 0xFF707B5D, 0xFFB4B79C, 0xFF8C8A61, 0xFF5B5811, 0xFF979735, 0xFFDAE8AA, 0xFF878F77, 0xFFDFE2F1, 0xFFDCE1F7, 0xFF94A2A3, 0xFF93AB95, 0xFFB1D2B3, 0xFFA4C6AD, 0xFF94A18D, 0xFF728277, 0xFF7C8E8E, 0xFF839295, 0xFF707974,
        0xFFFCFAE3, 0xFFA39669, 0xFF685417, 0xFF705317, 0xFF6D4E15, 0xFF6E4D18, 0xFF735121, 0xFF6D4D1A, 0xFF684B13, 0xFF6C5414, 0xFF695510, 0xFF6A4C0C, 0xFF6C5011, 0xFF6A5214, 0xFF665014, 0xFF604B0C, 0xFF664E0E, 0xFF705510, 0xFF6C4F09, 0xFF69500E,
        0xFF665007, 0xFF67500C, 0xFF6B4E12, 0xFF6A4C06, 0xFF705C05, 0xFF727420, 0xFF697938, 0xFF0F0F0F, 0xFF000403, 0xFF15201C, 0xFF334340, 0xFF344743, 0xFF394C4A, 0xFF415150, 0xFF2F3F3E, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FEFFFF, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF67736F, 0xFF42534B, 0xFF020B0A, 0xFF17171F, 0xFF454A4D, 0xFF1B291A, 0xFF4A5C36, 0xFF6F773C, 0xFF716E29, 0xFF71581F, 0xFF6D5924, 0xFF6E5D29, 0xFF6A541A, 0xFF755613,
        0xFFA6863B, 0xFFC8B068, 0xFF867935, 0xFF7F6729, 0xFF917955, 0xFF857134, 0xFF907142, 0xFFC6A062, 0xFFC29F45, 0xFF705415, 0xFF685D0D, 0xFF948453, 0xFFF2F1DF, 0xFFF6FFFF, 0xFFFEFFF6, 0xFFC3B988, 0xFF8C7C41, 0xFFF6EFD2, 0xFFF8FFFF, 0xFFF7FFF1,
        0xFFE3DFC2, 0xFF6E5423, 0xFF6C4E08, 0xFFAA8F57, 0xFFFFEDC5, 0xFFC3B27A, 0xFFA49739, 0xFFA1882C, 0xFFBBA446, 0xFFD7C36D, 0xFFF2DFA4, 0xFF857252, 0xFF67593F, 0xFF574E23, 0xFF656121, 0xFF7E7049, 0xFFF5E1DA, 0xFFFFF7FF, 0xFFF0E2DF, 0xFFC9AF65,
        0xFFE6B334, 0xFFF4B943, 0xFFAF7C2D, 0xFF634706, 0xFF745916, 0xFF715816, 0xFF5B440E, 0xFF614B22, 0xFF54471D, 0xFFA5A068, 0xFFD4D58F, 0xFFA59A64, 0xFF6C6134, 0xFF49441A, 0xFF4C4C1A, 0xFF5D5E1C, 0xFF5D5910, 0xFF504505, 0xFF5D4916, 0xFF5B470C,
        0xFF5A3A14, 0xFF73551F, 0xFF8B7E4A, 0xFF60625D, 0xFF1C2F2D, 0xFF071E0C, 0xFF3A494E, 0xFF6C8484, 0xFF516566, 0xFF132124, 0xFF111A1F, 0xFF475055, 0xFF6D7B7E, 0xFF425657, 0xFF071F1F, 0xFF152E2B, 0xFF4F636A, 0xFF748491, 0xFF3F4E53, 0xFF04150F,
        0xFF1E3229, 0xFF63787B, 0xFF576D7B, 0xFF282C2B, 0xFF182130, 0xFF31394E, 0xFF7C7C74, 0xFF6C5C21, 0xFF9D7B17, 0xFFBA911D, 0xFFCCA130, 0xFF6A6523, 0xFF5A510E, 0xFF594C06, 0xFF604F09, 0xFF66540C, 0xFF65520D, 0xFF604D08, 0xFF685511, 0xFF77621F,
        0xFF5F5518, 0xFF79743C, 0xFFD2C37E, 0xFFF4D078, 0xFFE5B853, 0xFFE3BC57, 0xFFC9AC50, 0xFFB09A51, 0xFFCAA554, 0xFFD7A550, 0xFFD7A95E, 0xFF917A48, 0xFF585742, 0xFF16221E, 0xFF021416, 0xFF294143, 0xFF5A6E6D, 0xFF4A5954, 0xFF15201A, 0xFF111C16,
        0xFF384744, 0xFF5D6F71, 0xFF30444B, 0xFF23261B, 0xFF202528, 0xFF4B5763, 0xFF596C73, 0xFF223634, 0xFF0E2219, 0xFF1A2527, 0xFF505662, 0xFF4B6274, 0xFF152E2A, 0xFF162B22, 0xFF1F292A, 0xFF686554, 0xFF807032, 0xFF977923, 0xFFAE8237, 0xFFB8952D,
        0xFFC6A035, 0xFFC59B2F, 0xFFCA9C2D, 0xFFD19D2D, 0xFFD5A02C, 0xFFDEA733, 0xFFD49B27, 0xFFD4A12C, 0xFFD4A12C, 0xFFD4A12C, 0xFFD4A12C, 0xFFD3A02B, 0xFFD3A02B, 0xFFD3A02B, 0xFFD3A02B, 0xFFD3A02A, 0xFFD4A12B, 0xFFD5A22C, 0xFFD4A12B, 0xFFD3A02A,
        0xFFD29F29, 0xFFD3A02A, 0xFFD5A22C, 0xFFD5A22C, 0xFFD79F2E, 0xFFDC9E31, 0xFFDA9C2B, 0xFFDBA731, 0xFFC79E2C, 0xFF917313, 0xFF6E5707, 0xFF594812, 0xFF5B6059, 0xFF5E7676, 0xFF5D7263, 0xFF67746D, 0xFF5C7067, 0xFF5E7064, 0xFF5C606B, 0xFF62736D,
        0xFF62736D, 0xFF61726C, 0xFF60716B, 0xFF5E6F69, 0xFF5B6C66, 0xFF5A6B65, 0xFF596A64, 0xFF5F6E69, 0xFF596863, 0xFF5E6D68, 0xFF5C6B66, 0xFF5C6B66, 0xFF606F6A, 0xFF606F6A, 0xFF6B7A75, 0xFF5E6F69, 0xFF60716B, 0xFF5C6D67, 0xFF60706D, 0xFF586865,
        0xFF5D6D6C, 0xFF5C6C6B, 0xFF617170, 0xFF586867, 0xFF637372, 0xFF637372, 0xFF5D6D6A, 0xFF60706D, 0xFF64756F, 0xFF61726C, 0xFF5E6F69, 0xFF5F706A, 0xFF60716B, 0xFF637370, 0xFF5D6D6A, 0xFF637370, 0xFF5E6E6D, 0xFF60706F, 0xFF586867, 0xFF63757F,
        0xFF596767, 0xFF6B766E, 0xFF606E6E, 0xFF687B81, 0xFF51605B, 0xFF3F411C, 0xFFADA566, 0xFFCFDC96, 0xFF57613E, 0xFFCAD4D6, 0xFFBFCBD7, 0xFF9AAAA9, 0xFFBECEC1, 0xFFE4F1E8, 0xFFE7F1F3, 0xFFA9C4B5, 0xFF859C96, 0xFF91A4A8, 0xFF9BA6AC, 0xFF6B6D68,
        0xFFFFFFE2, 0xFFADA26A, 0xFF64560D, 0xFF705100, 0xFF735608, 0xFF6C5007, 0xFF6C540E, 0xFF715913, 0xFF644C04, 0xFF6B5106, 0xFF6D5205, 0xFF645013, 0xFF6C5313, 0xFF725312, 0xFF73520F, 0xFF6F4E09, 0xFF6E500A, 0xFF70520A, 0xFF6B4E02, 0xFF6D520F,
        0xFF6B5309, 0xFF68510B, 0xFF6E5014, 0xFF704F0C, 0xFF6E5803, 0xFF706F1D, 0xFF687639, 0xFF090D0C, 0xFF000503, 0xFF17231F, 0xFF364643, 0xFF374A46, 0xFF3B4B4A, 0xFF414F4F, 0xFF384444, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FEFFFF, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF6C7874, 0xFF3D4E46, 0xFF040D0C, 0xFF17171F, 0xFF42474A, 0xFF202E1F, 0xFF566842, 0xFF747C41, 0xFF6D6A25, 0xFF796028, 0xFF715D28, 0xFF6E5D27, 0xFF735D21, 0xFF82641E,
        0xFFBD9C55, 0xFF967D3D, 0xFF8B7C43, 0xFFC9B88C, 0xFFD6C7A8, 0xFFCFC497, 0xFFA48F62, 0xFF9A7A3F, 0xFFBB9749, 0xFF947630, 0xFF6D5A0E, 0xFFA39066, 0xFFFFFCEE, 0xFFF9FCFF, 0xFFFFFFF6, 0xFFCEBF94, 0xFF96814C, 0xFFFFF7DF, 0xFFFEFEFF, 0xFFF6FCF0,
        0xFFF5F0B8, 0xFFB9A753, 0xFF664B04, 0xFFA8945F, 0xFFDFD8A2, 0xFF797643, 0xFFB7B28A, 0xFFB6A762, 0xFF887A3D, 0xFF7D7141, 0xFFAB9F85, 0xFF9E948A, 0xFF8E8A81, 0xFF73725E, 0xFF747758, 0xFFE0DBC5, 0xFFFFF2ED, 0xFFFFF3FF, 0xFFFBFFFF, 0xFFDFE5B9,
        0xFFD2BB5D, 0xFFE8B548, 0xFFE3A747, 0xFF8F742D, 0xFF5B4300, 0xFF624B05, 0xFF6C571E, 0xFF615024, 0xFF55491F, 0xFF54511A, 0xFF4D4F0D, 0xFF7E7031, 0xFF6E612C, 0xFF685D2F, 0xFF5B5523, 0xFF514B0D, 0xFF574E0B, 0xFF5D4D11, 0xFF5B4611, 0xFF5B480E,
        0xFF553911, 0xFF72551D, 0xFF988753, 0xFF69655A, 0xFF76827E, 0xFF778B7F, 0xFF78868F, 0xFF7C8D94, 0xFF707F86, 0xFF738088, 0xFF7B858E, 0xFF6D7780, 0xFF76838B, 0xFF707F86, 0xFF687980, 0xFF5E777B, 0xFF758B96, 0xFF7F919F, 0xFF83949C, 0xFF899B9B,
        0xFF788C8A, 0xFF748B91, 0xFF677E90, 0xFF868E91, 0xFF79888B, 0xFF7A8A89, 0xFF6A6E5D, 0xFF4A380A, 0xFF9C7729, 0xFFB98A22, 0xFFD8A731, 0xFF6D6220, 0xFF625511, 0xFF62530E, 0xFF65540F, 0xFF695612, 0xFF685511, 0xFF62500E, 0xFF695715, 0xFF564A0E,
        0xFF847534, 0xFFD1BB72, 0xFFE8C46E, 0xFFE2B452, 0xFFE5B853, 0xFFCDAA4C, 0xFFB69E48, 0xFFCBAF5C, 0xFFD1AC54, 0xFFD4A650, 0xFFBF974F, 0xFF745F32, 0xFF5B5D48, 0xFF606C68, 0xFF647376, 0xFF586B7C, 0xFF617180, 0xFF505A66, 0xFF454D58, 0xFF4A525D,
        0xFF464F5E, 0xFF566578, 0xFF42546A, 0xFF485251, 0xFF546169, 0xFF4E606E, 0xFF5A707B, 0xFF445E5F, 0xFF4F6563, 0xFF42535A, 0xFF596575, 0xFF535C6D, 0xFF546360, 0xFF465550, 0xFF5B626A, 0xFF616159, 0xFF352A00, 0xFF584302, 0xFF5E3E05, 0xFF674E00,
        0xFF715400, 0xFF8C6A08, 0xFFBC952C, 0xFFDBAF42, 0xFFD6A637, 0xFFD19F2E, 0xFFD4A22F, 0xFFD1A230, 0xFFD1A230, 0xFFD1A230, 0xFFD1A230, 0xFFD0A12F, 0xFFD0A12F, 0xFFD0A12F, 0xFFD0A12F, 0xFFCD9E2A, 0xFFCE9F2B, 0xFFCFA02C, 0xFFCE9F2B, 0xFFCD9E2A,
        0xFFCC9D29, 0xFFCE9F2B, 0xFFCFA02C, 0xFFCDA029, 0xFFD2A031, 0xFFD8A236, 0xFFD59D2E, 0xFFD2A12B, 0xFFBD9826, 0xFF876B0A, 0xFF5D4700, 0xFF4B3A02, 0xFF434841, 0xFF465E5E, 0xFF455A4B, 0xFF49564F, 0xFF3F534A, 0xFF4C5E52, 0xFF545863, 0xFF485953,
        0xFF475852, 0xFF475852, 0xFF475852, 0xFF475852, 0xFF465751, 0xFF455650, 0xFF43544E, 0xFF45544F, 0xFF41504B, 0xFF465550, 0xFF465550, 0xFF475651, 0xFF495853, 0xFF465550, 0xFF4D5C57, 0xFF495A54, 0xFF4B5C56, 0xFF465751, 0xFF4A5A57, 0xFF445451,
        0xFF495958, 0xFF475756, 0xFF4A5A59, 0xFF425251, 0xFF4A5A59, 0xFF4A5A59, 0xFF455552, 0xFF475754, 0xFF495A54, 0xFF465751, 0xFF465751, 0xFF475852, 0xFF475852, 0xFF495A54, 0xFF435350, 0xFF495956, 0xFF475756, 0xFF4C5C5B, 0xFF485857, 0xFF465862,
        0xFF3E4C4C, 0xFF4C574F, 0xFF445252, 0xFF4E6167, 0xFF3F4E49, 0xFF353712, 0xFF958D4E, 0xFFD0DB99, 0xFF5D6647, 0xFF919A9F, 0xFFBFCCDD, 0xFFCADCE0, 0xFFDFF5E9, 0xFFE7FBF0, 0xFFC6D9D7, 0xFF8AA49B, 0xFFB9CFCD, 0xFF81949A, 0xFF646F75, 0xFF6A6C67,
        0xFFF5F2D3, 0xFFC1B87F, 0xFF584C02, 0xFF6A540B, 0xFF6A530D, 0xFF66510E, 0xFF665112, 0xFF655011, 0xFF6A5214, 0xFF6B5212, 0xFF694D0C, 0xFF685014, 0xFF6A5216, 0xFF6C5215, 0xFF6C5215, 0xFF6D5316, 0xFF6D5414, 0xFF6B5210, 0xFF654C0A, 0xFF6D520F,
        0xFF6A5208, 0xFF67500A, 0xFF6E5014, 0xFF704F0C, 0xFF6F5904, 0xFF6F6E1C, 0xFF647235, 0xFF090D0C, 0xFF000604, 0xFF192521, 0xFF374744, 0xFF384B47, 0xFF3B4B4A, 0xFF3E4C4C, 0xFF333F3F, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FEFFFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF67736F, 0xFF374840, 0xFF07100F, 0xFF17171F, 0xFF3D4245, 0xFF263425, 0xFF61734D, 0xFF767E43, 0xFF696621, 0xFF755B28, 0xFF6F5B28, 0xFF6B5A22, 0xFF735E1D, 0xFF9C7E38,
        0xFFBF9E5B, 0xFF836833, 0xFFC4B28A, 0xFFFFFDEB, 0xFFFFFFE8, 0xFFFFFFEF, 0xFFDAD5AD, 0xFF948045, 0xFFAD8B4C, 0xFFB69546, 0xFF6F4E0B, 0xFF79622E, 0xFFD4C9AB, 0xFFFFFDF1, 0xFFE9E2C8, 0xFF95814C, 0xFF6C5416, 0xFFC4B58E, 0xFFFAF3E9, 0xFFEEE9CC,
        0xFF90863F, 0xFF9D8A2B, 0xFFAB9740, 0xFF72651F, 0xFF58561D, 0xFFA1A37C, 0xFFB9BBAE, 0xFF87844D, 0xFF605C36, 0xFF615E4D, 0xFF8D8C88, 0xFFBABCB9, 0xFFB4BBB4, 0xFF99A39B, 0xFF98A39D, 0xFFF5FBF9, 0xFFFFFEFA, 0xFFFDFDFF, 0xFFEEFFFF, 0xFFEBFFF8,
        0xFFD0CE91, 0xFFD0A644, 0xFFE1A334, 0xFF8E7437, 0xFF755D1D, 0xFF8A763B, 0xFFB09F73, 0xFFA3957A, 0xFF978F78, 0xFFAEAB8A, 0xFF9FA172, 0xFF8F7C2F, 0xFF665311, 0xFF52410B, 0xFF5F4F1C, 0xFF60511A, 0xFF5B4B10, 0xFF634E15, 0xFF624913, 0xFF544710,
        0xFF5E4618, 0xFF604309, 0xFF917842, 0xFF4C3D26, 0xFF34362B, 0xFF2B382F, 0xFF2B3944, 0xFF2A3234, 0xFF1D2728, 0xFF232F2F, 0xFF2B3939, 0xFF1E2C2C, 0xFF232F2F, 0xFF1F292A, 0xFF2C3436, 0xFF233332, 0xFF1D2B2E, 0xFF252E35, 0xFF343C3E, 0xFF363F3A,
        0xFF202B25, 0xFF212F30, 0xFF2B3942, 0xFF252E35, 0xFF2E413D, 0xFF182F1D, 0xFF2B3419, 0xFF3C2803, 0xFF966A2D, 0xFFD39D3D, 0xFFD59D24, 0xFF705D19, 0xFF614E0A, 0xFF655210, 0xFF675412, 0xFF655313, 0xFF6C5A1A, 0xFF6A5818, 0xFF614F11, 0xFF746E32,
        0xFFD5BD73, 0xFFF3C668, 0xFFECB34A, 0xFFEBB84F, 0xFFCCA444, 0xFFB79C43, 0xFFC3AD57, 0xFFDDB657, 0xFFD8B051, 0xFFC19741, 0xFF866724, 0xFF3F3208, 0xFF313220, 0xFF414A45, 0xFF253131, 0xFF273633, 0xFF1E2A26, 0xFF25302A, 0xFF29332B, 0xFF2D3631,
        0xFF1D2927, 0xFF1C2C2C, 0xFF1C2E32, 0xFF292F2D, 0xFF2D3739, 0xFF25323A, 0xFF1B2A2F, 0xFF314140, 0xFF22312E, 0xFF293335, 0xFF272E38, 0xFF2D2C34, 0xFF353A36, 0xFF2E3432, 0xFF2A2E37, 0xFF242422, 0xFF3A3515, 0xFF3B2F00, 0xFF5A4518, 0xFF8D7D32,
        0xFF998735, 0xFFBEA34A, 0xFFD4B250, 0xFFD1AB40, 0xFFCCA232, 0xFFCCA12E, 0xFFCFA431, 0xFFD1A230, 0xFFD1A230, 0xFFD1A230, 0xFFD1A230, 0xFFD1A230, 0xFFD0A12F, 0xFFD0A12F, 0xFFD0A12F, 0xFFD1A431, 0xFFD2A532, 0xFFD1A431, 0xFFD0A330, 0xFFCFA22F,
        0xFFD0A330, 0xFFD1A431, 0xFFD3A633, 0xFFCAA12B, 0xFFCEA031, 0xFFD3A035, 0xFFD09C2C, 0xFFD3A42E, 0xFFD6AF3A, 0xFFBB9B34, 0xFF9D8229, 0xFF6E5A1D, 0xFF4A4D42, 0xFF445A58, 0xFF475B4F, 0xFF4E5A56, 0xFF465A51, 0xFF4F6357, 0xFF535A62, 0xFF4C5D57,
        0xFF4B5C56, 0xFF4A5B55, 0xFF4A5B55, 0xFF4B5C56, 0xFF4B5C56, 0xFF495A54, 0xFF475852, 0xFF475651, 0xFF44534E, 0xFF4A5954, 0xFF4C5B56, 0xFF4D5C57, 0xFF4D5C57, 0xFF485752, 0xFF4A5954, 0xFF4A5B55, 0xFF4A5B55, 0xFF465751, 0xFF495956, 0xFF465653,
        0xFF4A5A59, 0xFF475756, 0xFF485857, 0xFF495958, 0xFF4E5E5D, 0xFF4D5D5C, 0xFF4B5B58, 0xFF4D5D5A, 0xFF4B5C56, 0xFF495A54, 0xFF4C5D57, 0xFF4C5D55, 0xFF4B5C56, 0xFF4E5F59, 0xFF455650, 0xFF495956, 0xFF475754, 0xFF4A5A59, 0xFF475756, 0xFF455761,
        0xFF435151, 0xFF4B564E, 0xFF435151, 0xFF495C62, 0xFF3C4B46, 0xFF363813, 0xFF7A7233, 0xFFD6DC9C, 0xFF757A5A, 0xFF6C7477, 0xFF637081, 0xFF889CA5, 0xFFE5FFFB, 0xFFBED9D2, 0xFF859F9C, 0xFFB0C6C4, 0xFFC4D7DB, 0xFF5F7077, 0xFF374246, 0xFFADB0A5,
        0xFFFFFFE1, 0xFF938A51, 0xFF675B13, 0xFF6B5B1D, 0xFF554507, 0xFF665414, 0xFF725D1E, 0xFF5F4707, 0xFF664A09, 0xFF6B4D0D, 0xFF795C1A, 0xFF906F28, 0xFF80631F, 0xFF6C5414, 0xFF624D0E, 0xFF654D0D, 0xFF664F0C, 0xFF6A5310, 0xFF725D1C, 0xFF735815,
        0xFF6D550B, 0xFF67500A, 0xFF6C4E12, 0xFF6E4D0A, 0xFF705A05, 0xFF71701E, 0xFF637134, 0xFF080C0B, 0xFF000604, 0xFF1A2622, 0xFF394946, 0xFF394C48, 0xFF3C4C4B, 0xFF3C4A4A, 0xFF2F3B3B, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FEFFFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF5C6864, 0xFF34453D, 0xFF081110, 0xFF15151D, 0xFF3B4043, 0xFF283627, 0xFF61734D, 0xFF737B40, 0xFF696621, 0xFF785E2D, 0xFF75612E, 0xFF6F5F24, 0xFF6C5714, 0xFFAF9149,
        0xFFA58344, 0xFF8F7246, 0xFFF8E4C9, 0xFFFEFDFB, 0xFFF7FFEB, 0xFFF7FFFF, 0xFFFFFFDF, 0xFFA99A61, 0xFF997843, 0xFFC29E4A, 0xFF7C5216, 0xFF6E560C, 0xFF736328, 0xFF887D4F, 0xFF786C32, 0xFF695405, 0xFF6F5602, 0xFF756220, 0xFF807548, 0xFF6F6424,
        0xFF695719, 0xFF70590B, 0xFFAE9833, 0xFFA28F33, 0xFF64551C, 0xFFA89F78, 0xFF888656, 0xFF484B12, 0xFF7B7B59, 0xFFDBDBD1, 0xFFFFFFFB, 0xFFEDF4E4, 0xFFE6F0D8, 0xFFDBE6D6, 0xFFCAD5CF, 0xFFDDE7F0, 0xFFE2E7E3, 0xFFCAD5CF, 0xFFAFC8C5, 0xFFB8C7B4,
        0xFFCBBA82, 0xFFCFA64A, 0xFFCBA030, 0xFF927A32, 0xFF8B772E, 0xFFC5B673, 0xFF897F4C, 0xFF8F8967, 0xFF696349, 0xFF7E7B58, 0xFF818354, 0xFFA68E36, 0xFF8F7828, 0xFF6B5210, 0xFF644B13, 0xFF6B521C, 0xFF6A511B, 0xFF684F17, 0xFF60450E, 0xFF534812,
        0xFF54400B, 0xFF7B5E24, 0xFF8D6E37, 0xFF3F2701, 0xFF372F1A, 0xFF272D23, 0xFF051218, 0xFF0A0B00, 0xFF0A0E00, 0xFF0B1605, 0xFF1B2A17, 0xFF081704, 0xFF010C00, 0xFF0C1001, 0xFF141507, 0xFF191B0E, 0xFF0E0C00, 0xFF130F03, 0xFF120A00, 0xFF201907,
        0xFF272413, 0xFF0F0D01, 0xFF040500, 0xFF0C1617, 0xFF172822, 0xFF243626, 0xFF313616, 0xFF4A3403, 0xFF9A6B23, 0xFFC58B28, 0xFFDDA22E, 0xFF775C19, 0xFF6E5513, 0xFF675211, 0xFF685417, 0xFF6B591B, 0xFF614F11, 0xFF614D0E, 0xFF826D2C, 0xFFD0BE74,
        0xFFECC56A, 0xFFECB247, 0xFFF0B648, 0xFFD4A845, 0xFFB5963B, 0xFFCEAF54, 0xFFD7B353, 0xFFE2B54E, 0xFFC59D3D, 0xFF8F6C1A, 0xFF533E00, 0xFF342B02, 0xFF272611, 0xFF3B4137, 0xFF272D29, 0xFF182008, 0xFF050D00, 0xFF121701, 0xFF060E00, 0xFF0A1100,
        0xFF071201, 0xFF041002, 0xFF101C10, 0xFF1C1C14, 0xFF090A02, 0xFF090B00, 0xFF070900, 0xFF1F2211, 0xFF0B0C00, 0xFF151307, 0xFF0D0A03, 0xFF090900, 0xFF1E2015, 0xFF11170D, 0xFF222721, 0xFF242618, 0xFF3E3A17, 0xFF443806, 0xFF9A8953, 0xFFCBBA74,
        0xFFA58F43, 0xFFBC9C45, 0xFFDDB651, 0xFFDAAE41, 0xFFD3A631, 0xFFCEA12A, 0xFFCCA22A, 0xFFD5A22D, 0xFFD4A12C, 0xFFD4A12C, 0xFFD4A12C, 0xFFD4A12C, 0xFFD3A02B, 0xFFD3A02B, 0xFFD3A02B, 0xFFD3A02B, 0xFFD3A02B, 0xFFD29F2A, 0xFFD19E29, 0xFFD19E29,
        0xFFD19E29, 0xFFD3A02B, 0xFFD4A12C, 0xFFD1A52C, 0xFFD2A331, 0xFFD5A133, 0xFFD49F2B, 0xFFD6A427, 0xFFDBAE31, 0xFFD2A937, 0xFFC29B34, 0xFF7D6827, 0xFF47483A, 0xFF3F5352, 0xFF4A5B51, 0xFF505C5A, 0xFF465953, 0xFF495D51, 0xFF4A5157, 0xFF4D5E56,
        0xFF4A5B53, 0xFF485951, 0xFF485951, 0xFF495A52, 0xFF485951, 0xFF45564E, 0xFF41524A, 0xFF45544D, 0xFF44534C, 0xFF46554E, 0xFF485750, 0xFF4B5A53, 0xFF4C5B54, 0xFF495851, 0xFF485750, 0xFF4F605A, 0xFF4C5D57, 0xFF495A54, 0xFF4A5A57, 0xFF4B5B58,
        0xFF4D5D5C, 0xFF4B5B5A, 0xFF495958, 0xFF495958, 0xFF4C5C5B, 0xFF4B5B5A, 0xFF4B5B58, 0xFF4D5D5A, 0xFF485953, 0xFF485953, 0xFF4F605A, 0xFF495A52, 0xFF495A52, 0xFF51625C, 0xFF475852, 0xFF4B5C56, 0xFF475754, 0xFF455552, 0xFF42524F, 0xFF495B65,
        0xFF4D5B5B, 0xFF525D55, 0xFF4C5A5A, 0xFF4B5E64, 0xFF3B4A45, 0xFF32340F, 0xFF585011, 0xFFCDCF84, 0xFFA8AB7C, 0xFF42483E, 0xFF28353E, 0xFF4F636C, 0xFF8DA6AA, 0xFF89A5A6, 0xFF9FBBBF, 0xFF909FA2, 0xFF536265, 0xFF425051, 0xFF5B645F, 0xFFD8DBC8,
        0xFFF1EDC7, 0xFF73692E, 0xFF645711, 0xFF615018, 0xFF624E11, 0xFF664E06, 0xFF664900, 0xFF85630C, 0xFFB59236, 0xFFBA973B, 0xFFD6B156, 0xFFEBBF5E, 0xFFDFBC62, 0xFFC9AF5A, 0xFFA9913B, 0xFF8B7017, 0xFF785801, 0xFF7A5E0B, 0xFF977D30, 0xFF7A5F1C,
        0xFF70580E, 0xFF67500A, 0xFF6B4D11, 0xFF6C4B08, 0xFF715B06, 0xFF757422, 0xFF657336, 0xFF070B0A, 0xFF000604, 0xFF192521, 0xFF384845, 0xFF394C48, 0xFF3D4D4C, 0xFF3E4C4C, 0xFF2F3B3B, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FEFFFF, 0x00FFFDFE, 0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF5C6864, 0xFF36473F, 0xFF081110, 0xFF14141C, 0xFF3B4043, 0xFF263425, 0xFF596B45, 0xFF6D753A, 0xFF6C6924, 0xFF775D2C, 0xFF735F2C, 0xFF706025, 0xFF6B5613, 0xFFAA8C44,
        0xFFAA8849, 0xFF96794D, 0xFFE4D0B5, 0xFFFCFAFB, 0xFFFBFFEF, 0xFFF6FFFF, 0xFFE8ECC9, 0xFFA09158, 0xFF9A7946, 0xFFB48F3E, 0xFF81571B, 0xFF745A0F, 0xFF6C5714, 0xFF624F14, 0xFF685511, 0xFF6E5805, 0xFF745A06, 0xFF715A14, 0xFF614E16, 0xFF6B5A0A,
        0xFF755D1F, 0xFF6E5115, 0xFF725303, 0xFFB6933F, 0xFFB4934C, 0xFF866B26, 0xFF9C8A38, 0xFF8F8C5F, 0xFFF3EFD6, 0xFFFFFDFA, 0xFFFAF5F1, 0xFFF5F6E4, 0xFFE7EBD0, 0xFFB7BCA8, 0xFFD5D8D1, 0xFFE2E7EB, 0xFFA3AEA8, 0xFF7F9086, 0xFF7D8A78, 0xFF938153,
        0xFFD5A251, 0xFFDBAD4D, 0xFFB29E3F, 0xFF988026, 0xFFB5A246, 0xFF6E640B, 0xFF5E5A11, 0xFF43420A, 0xFF434212, 0xFF4B4B15, 0xFF484808, 0xFF92781B, 0xFF9C8128, 0xFF765909, 0xFF614200, 0xFF6F5017, 0xFF6E501A, 0xFF674C15, 0xFF674E15, 0xFF574C16,
        0xFF473700, 0xFF6A5216, 0xFF987942, 0xFF563802, 0xFF523E19, 0xFFBBB6A3, 0xFF878F82, 0xFF322715, 0xFF140D00, 0xFF41402B, 0xFFBDC1AA, 0xFF8F937C, 0xFF161500, 0xFF201906, 0xFF574C3A, 0xFFA6A797, 0xFF5D5C47, 0xFF191500, 0xFF171100, 0xFF8E8870,
        0xFFC9C4B0, 0xFF5A5945, 0xFF141601, 0xFF1B1E0D, 0xFF949992, 0xFFB2B8AC, 0xFF6B6640, 0xFF6D5308, 0xFFBE932B, 0xFFDBA83B, 0xFFD19D37, 0xFF6E4F0C, 0xFF745817, 0xFF6A5214, 0xFF645013, 0xFF6C5A1C, 0xFF644F0C, 0xFF816923, 0xFFD8BC73, 0xFFF1C969,
        0xFFE2B14A, 0xFFE8B245, 0xFFD4A740, 0xFFBB9D43, 0xFFC5AB54, 0xFFD9B14F, 0xFFEAB447, 0xFFDCB04D, 0xFF98731A, 0xFF5B4100, 0xFF453500, 0xFF473C0F, 0xFF423D1D, 0xFF898873, 0xFFB7BAA9, 0xFF383019, 0xFF1F1903, 0xFF23200D, 0xFF6F6D5E, 0xFF8B8B7F,
        0xFF282A1F, 0xFF050700, 0xFF2E2F27, 0xFF797F73, 0xFF626655, 0xFF151900, 0xFF101300, 0xFF2F2F15, 0xFF85856D, 0xFF52513D, 0xFF161501, 0xFF212504, 0xFF42462B, 0xFF929A82, 0xFF6D7453, 0xFF2B2D05, 0xFF342D00, 0xFF857331, 0xFFC1AA5A, 0xFF997F34,
        0xFF9E7E2B, 0xFFC59A3B, 0xFFDEAB40, 0xFFDEA935, 0xFFDCA830, 0xFFD1A32B, 0xFFCFA52D, 0xFFDAA32E, 0xFFD9A22D, 0xFFD9A22D, 0xFFD9A22D, 0xFFD9A22D, 0xFFD8A12C, 0xFFD8A12C, 0xFFD8A12C, 0xFFD6A22C, 0xFFD6A22C, 0xFFD5A12B, 0xFFD4A02A, 0xFFD5A12B,
        0xFFD5A12B, 0xFFD6A22C, 0xFFD6A22C, 0xFFD5A529, 0xFFD4A12E, 0xFFD49F2F, 0xFFDAA12D, 0xFFDAA324, 0xFFD7A11D, 0xFFD59D22, 0xFFD29A29, 0xFF806820, 0xFF494836, 0xFF465957, 0xFF516157, 0xFF515C5E, 0xFF455854, 0xFF4B6154, 0xFF535C61, 0xFF506159,
        0xFF4D5E56, 0xFF4B5C54, 0xFF4B5C54, 0xFF4C5D55, 0xFF4C5D55, 0xFF485951, 0xFF45564E, 0xFF4B5A53, 0xFF4A5952, 0xFF485750, 0xFF495851, 0xFF4C5B54, 0xFF4F5E57, 0xFF52615A, 0xFF516059, 0xFF4E5F59, 0xFF4A5B55, 0xFF475852, 0xFF465653, 0xFF495956,
        0xFF495958, 0xFF495958, 0xFF465655, 0xFF4A5A59, 0xFF4C5C5B, 0xFF4B5B5A, 0xFF4D5D5A, 0xFF4F5F5C, 0xFF4A5B55, 0xFF4B5C56, 0xFF566761, 0xFF4E5F57, 0xFF4C5D55, 0xFF576860, 0xFF4D5E58, 0xFF53645E, 0xFF52635D, 0xFF4F5F5C, 0xFF4C5C59, 0xFF42545E,
        0xFF4A5858, 0xFF4D5850, 0xFF4A5858, 0xFF495C62, 0xFF3A4944, 0xFF32340F, 0xFF494102, 0xFF898730, 0xFFCDCC8C, 0xFF707453, 0xFF303A2F, 0xFF475555, 0xFF65797A, 0xFF475C61, 0xFF677B84, 0xFF37403D, 0xFF2A332E, 0xFF5D655A, 0xFFBCC1AD, 0xFFFFFFE3,
        0xFFA6A070, 0xFF53470B, 0xFF675611, 0xFF62470F, 0xFF997A37, 0xFF9C7822, 0xFF896000, 0xFFC99F29, 0xFFFDD154, 0xFFD7AE30, 0xFFCBA425, 0xFFE2AE38, 0xFFD4AA3A, 0xFFC5A538, 0xFFC8A736, 0xFFEAC048, 0xFFF2C34F, 0xFFCBA239, 0xFFB29134, 0xFF765B18,
        0xFF6D550B, 0xFF67500A, 0xFF6B4D11, 0xFF6C4B08, 0xFF715B06, 0xFF737220, 0xFF5F6D30, 0xFF060A09, 0xFF000503, 0xFF17231F, 0xFF354542, 0xFF384B47, 0xFF3F4F4E, 0xFF414F4F, 0xFF323E3E, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FEFFFF, 0x00FDFBFC, 0x00FEFCFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF65716D, 0xFF3A4B43, 0xFF07100F, 0xFF13131B, 0xFF3C4144, 0xFF233122, 0xFF52643E, 0xFF697136, 0xFF6D6A25, 0xFF735926, 0xFF6D5926, 0xFF6B5A22, 0xFF715C1B, 0xFF91732D,
        0xFFC6A562, 0xFF927742, 0xFFA4926A, 0xFFFAF1E0, 0xFFFFFFE9, 0xFFEFF0E0, 0xFFADA882, 0xFF947F46, 0xFFBF9D60, 0xFFA07E34, 0xFF725010, 0xFF765921, 0xFF6E531E, 0xFF6F5620, 0xFF71581F, 0xFF725617, 0xFF785C1D, 0xFF6D521B, 0xFF6D5223, 0xFF715611,
        0xFF6D550D, 0xFF795F24, 0xFF6B491B, 0xFF764E11, 0xFFAE8930, 0xFFBC9D43, 0xFFC7AC69, 0xFFF6EFDF, 0xFFFFFDF8, 0xFFFDF4F7, 0xFFFFFCFF, 0xFFFDF8F5, 0xFFFFFFF8, 0xFFF4F3EE, 0xFFE9E8E6, 0xFFFBFCF4, 0xFFC7D1C9, 0xFF76877F, 0xFF62664D, 0xFF926B2A,
        0xFFEFAA43, 0xFFD39E36, 0xFF958530, 0xFFC9A865, 0xFFB49C56, 0xFFAE9F5C, 0xFF8E8751, 0xFFA5A17B, 0xFF999474, 0xFFACA77F, 0xFFA6A06E, 0xFFB79941, 0xFFBE9F44, 0xFF97781D, 0xFF765301, 0xFF755411, 0xFF6E4F16, 0xFF644911, 0xFF6A5516, 0xFF5D4C14,
        0xFF574803, 0xFF4D3900, 0xFF8A6C36, 0xFF93752F, 0xFF634810, 0xFFCFC09F, 0xFFF6F6D0, 0xFF6C5A42, 0xFF46361D, 0xFF5F5137, 0xFFFFF3D9, 0xFFE4D8BE, 0xFF3F3117, 0xFF403017, 0xFF98866E, 0xFFEAEFD8, 0xFF828767, 0xFF383A13, 0xFF4A4A24, 0xFFC3C2A3,
        0xFFF9F9DF, 0xFF7B8062, 0xFF212806, 0xFF544A26, 0xFFE8DEC3, 0xFFF7E9CE, 0xFFC1A86F, 0xFFD0AD49, 0xFFE2B83F, 0xFFD2A93D, 0xFF926E16, 0xFF694C0A, 0xFF644B09, 0xFF6B5617, 0xFF5F4D0F, 0xFF5C4905, 0xFF90782E, 0xFFD2B160, 0xFFF4D07A, 0xFFE6B244,
        0xFFE8B548, 0xFFCBA038, 0xFFB79436, 0xFFC5AA4F, 0xFFD8B75A, 0xFFE1B24A, 0xFFEAAE3C, 0xFFA9832E, 0xFF6B4E02, 0xFF4E3B00, 0xFF52430A, 0xFF4D3D0A, 0xFF46390C, 0xFF867E59, 0xFFAFAE8F, 0xFF55410E, 0xFF6C5C2B, 0xFF574C1F, 0xFFABA37C, 0xFFCFCBA6,
        0xFF4A4623, 0xFF292502, 0xFF4B4521, 0xFFB2BFAE, 0xFF818E72, 0xFF252D04, 0xFF2D3307, 0xFF767851, 0xFFC7C8A8, 0xFF7B7F5E, 0xFF3C421E, 0xFF555519, 0xFF7F8054, 0xFFD9DBB3, 0xFF7D824A, 0xFF8D8D4D, 0xFF9F9655, 0xFFC8B568, 0xFFA8902C, 0xFF84640F,
        0xFFCBA447, 0xFFF1C059, 0xFFE6AE3D, 0xFFE4AB37, 0xFFE1AE3B, 0xFFCFA535, 0xFFCCA73C, 0xFFD0A233, 0xFFD0A233, 0xFFD0A233, 0xFFD0A233, 0xFFD0A233, 0xFFCFA132, 0xFFCFA132, 0xFFCFA132, 0xFFCFA434, 0xFFCFA434, 0xFFCEA333, 0xFFCFA434, 0xFFD0A535,
        0xFFD0A535, 0xFFCFA434, 0xFFCEA333, 0xFFD2A935, 0xFFD2A63B, 0xFFCF9F39, 0xFFDAA73A, 0xFFE1AD35, 0xFFD8A122, 0xFFD3991E, 0xFFD49625, 0xFF846A1F, 0xFF4B4934, 0xFF475756, 0xFF4C5B54, 0xFF4E575C, 0xFF455957, 0xFF485E51, 0xFF4F5A5C, 0xFF4A5B51,
        0xFF495A50, 0xFF48594F, 0xFF495A50, 0xFF4B5C52, 0xFF4B5C52, 0xFF4A5B51, 0xFF48594F, 0xFF4A5A50, 0xFF4C5C52, 0xFF46564C, 0xFF46564C, 0xFF49594F, 0xFF4D5D53, 0xFF54645A, 0xFF536359, 0xFF52635D, 0xFF4D5E58, 0xFF4B5C56, 0xFF475754, 0xFF4C5C59,
        0xFF4B5B5A, 0xFF4C5C5B, 0xFF4A5A59, 0xFF455554, 0xFF485857, 0xFF475756, 0xFF485855, 0xFF495956, 0xFF465751, 0xFF495A54, 0xFF53645E, 0xFF516258, 0xFF4A5B51, 0xFF53645C, 0xFF45564E, 0xFF4B5C56, 0xFF4E5F59, 0xFF4A5B55, 0xFF495956, 0xFF475963,
        0xFF4C5A5A, 0xFF4C574F, 0xFF495757, 0xFF4A5D63, 0xFF3E4D48, 0xFF3A3C17, 0xFF51490A, 0xFF4F4500, 0xFFA09A50, 0xFFAEAB74, 0xFF62643F, 0xFF363B24, 0xFF4B5344, 0xFF222922, 0xFF171D1B, 0xFF262715, 0xFF585A44, 0xFFC8CBAC, 0xFFFDFED6, 0xFFB8B484,
        0xFF5D521A, 0xFF675515, 0xFF624B08, 0xFF725219, 0xFF8F6A24, 0xFFC29A3B, 0xFFDFB23F, 0xFFFACB49, 0xFFEDC13A, 0xFFE5BD39, 0xFFDDB836, 0xFFE2B33D, 0xFFE2BD4B, 0xFFD7B844, 0xFFD6AE32, 0xFFFFC942, 0xFFFFC947, 0xFFB9891B, 0xFF7D5C00, 0xFF6C510E,
        0xFF674F05, 0xFF664F09, 0xFF6E5014, 0xFF6D4C09, 0xFF705A05, 0xFF6F6E1C, 0xFF556326, 0xFF050908, 0xFF000503, 0xFF16221E, 0xFF334340, 0xFF384B47, 0xFF425251, 0xFF455353, 0xFF354141, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FEFFFF, 0x00FDFBFC, 0x00FEFCFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF65716D, 0xFF3D4E46, 0xFF07100F, 0xFF13131B, 0xFF3B4043, 0xFF212F20, 0xFF53653F, 0xFF6A7237, 0xFF6A6722, 0xFF785F27, 0xFF725E29, 0xFF6C5B25, 0xFF755F23, 0xFF795B15,
        0xFFB6954E, 0xFFA68D4D, 0xFF8E7F46, 0xFF9B8961, 0xFFA29278, 0xFF998D65, 0xFF856F46, 0xFF9E7D47, 0xFFBF9B51, 0xFF856626, 0xFF6A560F, 0xFF6F5324, 0xFF674B19, 0xFF745620, 0xFF6F541C, 0xFF694E17, 0xFF745924, 0xFF634715, 0xFF745526, 0xFF714F0F,
        0xFF775E1C, 0xFF644F18, 0xFF775F33, 0xFF775821, 0xFF6D5107, 0xFFC5B077, 0xFFF9E8D4, 0xFFFFFCFF, 0xFFFFFEFF, 0xFFFDFCFA, 0xFFFEFEFE, 0xFFFBFAFF, 0xFFFCFBFF, 0xFFFDFFFC, 0xFFDDE0D7, 0xFFC1C6B2, 0xFFCCD4C7, 0xFF798581, 0xFF545B49, 0xFF7E6427,
        0xFFF4BB54, 0xFFE2AC40, 0xFFC4A248, 0xFFCBA16F, 0xFFA5844F, 0xFF927E49, 0xFF867A50, 0xFF7D7659, 0xFF5A5339, 0xFF857B58, 0xFF807346, 0xFF9A7D2D, 0xFFBB9C41, 0xFFC3A33E, 0xFF947113, 0xFF6B4A00, 0xFF6A4D0B, 0xFF6A5214, 0xFF604E0C, 0xFF624C12,
        0xFF635308, 0xFF63541B, 0xFF59420C, 0xFF967A27, 0xFF9B7E32, 0xFF7C6634, 0xFFA49C5D, 0xFFB7A672, 0xFFB29E6B, 0xFF8B7543, 0xFFA68F5D, 0xFF87703E, 0xFF5F4917, 0xFF8E7A47, 0xFFA99864, 0xFF675F30, 0xFF8A8148, 0xFFBBB16C, 0xFFC8BA79, 0xFFA4955E,
        0xFF8A7C4B, 0xFF726932, 0xFF706A2C, 0xFFA08953, 0xFFA28C52, 0xFFC2A55F, 0xFFD3AC4F, 0xFFF0C052, 0xFFC69829, 0xFF806104, 0xFF635204, 0xFF725714, 0xFF644F0E, 0xFF6B5919, 0xFF675817, 0xFF7D6922, 0xFFC9AD5A, 0xFFF1CA6D, 0xFFDAAD48, 0xFFE3B345,
        0xFFCDA43B, 0xFFB59331, 0xFFCDAB49, 0xFFD5AB45, 0xFFDCAD43, 0xFFE7B850, 0xFFAC7F18, 0xFF6C510C, 0xFF5B470A, 0xFF594C15, 0xFF605118, 0xFF5A4609, 0xFF533D01, 0xFF574A15, 0xFF534F22, 0xFF7E620E, 0xFFB39C4C, 0xFF827227, 0xFF534904, 0xFF5C5413,
        0xFF4A4203, 0xFF534A09, 0xFF4E4301, 0xFF4B4C22, 0xFF5D5C23, 0xFF7B752B, 0xFF93873B, 0xFF94833E, 0xFF615115, 0xFF6B5D20, 0xFF9A914E, 0xFF746514, 0xFF94854C, 0xFF695F2C, 0xFF53500D, 0xFFE1DE97, 0xFFEDE3A6, 0xFFB3A25A, 0xFF80700E, 0xFFD7B85D,
        0xFFE1BB5A, 0xFFDBAC42, 0xFFE3AD40, 0xFFE7B346, 0xFFBE952C, 0xFF927517, 0xFF88721C, 0xFF8B7112, 0xFF8B7112, 0xFF8B7112, 0xFF8B7112, 0xFF8A7011, 0xFF8A7011, 0xFF8A7011, 0xFF8A7011, 0xFF866D10, 0xFF866D10, 0xFF866D10, 0xFF886F12, 0xFF897013,
        0xFF897013, 0xFF866D10, 0xFF846B0E, 0xFF7E6403, 0xFF8A6C12, 0xFF8D6D18, 0xFFAB882C, 0xFFD0A940, 0xFFD6AB3A, 0xFFD4A233, 0xFFD59E37, 0xFF896E1D, 0xFF544F39, 0xFF4D5C59, 0xFF4E5A56, 0xFF525B62, 0xFF4C605E, 0xFF486050, 0xFF4C5858, 0xFF4B5C52,
        0xFF4B5C52, 0xFF4C5D53, 0xFF4C5D53, 0xFF4D5E54, 0xFF4D5E54, 0xFF4C5D53, 0xFF4B5C52, 0xFF4B5B51, 0xFF506056, 0xFF4A5A50, 0xFF4A5A50, 0xFF4B5B51, 0xFF4D5D53, 0xFF56665C, 0xFF536359, 0xFF51625C, 0xFF4B5C56, 0xFF4A5B55, 0xFF435350, 0xFF495956,
        0xFF465655, 0xFF4B5B5A, 0xFF495958, 0xFF495958, 0xFF4D5D5C, 0xFF4C5C5B, 0xFF4A5A57, 0xFF4A5A57, 0xFF495A54, 0xFF4D5E58, 0xFF566761, 0xFF57685E, 0xFF4D5E54, 0xFF55665C, 0xFF46574F, 0xFF4D5E56, 0xFF50615B, 0xFF485953, 0xFF465751, 0xFF4A5C66,
        0xFF4C5A5A, 0xFF4A554D, 0xFF455353, 0xFF475A60, 0xFF3C4B46, 0xFF373914, 0xFF554D0E, 0xFF4F3C00, 0xFF5F500B, 0xFF938A47, 0xFFB3AF70, 0xFF83824C, 0xFF4A471C, 0xFF4E4A27, 0xFF50492C, 0xFF7C7855, 0xFFD3D0A7, 0xFFFFFDCD, 0xFFB9B47A, 0xFF685E20,
        0xFF5C4A08, 0xFF6B5210, 0xFF6E5010, 0xFF684A14, 0xFF5E3D00, 0xFF9D751D, 0xFFDAB044, 0xFFDDB440, 0xFF835E00, 0xFF836300, 0xFF83690A, 0xFF86670C, 0xFF7D650B, 0xFF7A6401, 0xFF997100, 0xFFE4A929, 0xFFF4B33B, 0xFF9C6E10, 0xFF5C4100, 0xFF6A4F0C,
        0xFF654D03, 0xFF68510B, 0xFF705216, 0xFF6D4C09, 0xFF705A05, 0xFF706F1D, 0xFF546225, 0xFF050908, 0xFF000503, 0xFF17231F, 0xFF334340, 0xFF394C48, 0xFF445453, 0xFF475555, 0xFF354141, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FEFFFF, 0x00FFFDFE, 0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF5C6864, 0xFF3F5048, 0xFF07100F, 0xFF12121A, 0xFF3B4043, 0xFF212F20, 0xFF566842, 0xFF6D753A, 0xFF66631E, 0xFF725920, 0xFF725E29, 0xFF6A5925, 0xFF6F591F, 0xFF70510E,
        0xFF907025, 0xFFD5BD75, 0xFFB4A763, 0xFFA38A52, 0xFF9E8566, 0xFF927D46, 0xFFC1A276, 0xFFD6AF78, 0xFF9F7B27, 0xFF63460C, 0xFF695D11, 0xFF6B501B, 0xFF745A1F, 0xFF624704, 0xFF755A17, 0xFF705717, 0xFF674F13, 0xFF71571C, 0xFF6E5117, 0xFF795509,
        0xFF694D1E, 0xFF75633B, 0xFF625717, 0xFF6A5911, 0xFF99865C, 0xFFF7ECE8, 0xFFFEFEFF, 0xFFF7FBFF, 0xFFF6FDF6, 0xFFFBFFF1, 0xFFF0F8EB, 0xFFF9FFFF, 0xFFF1F9FC, 0xFFF3FDF4, 0xFFA5B298, 0xFF5F674F, 0xFF808476, 0xFF737C7B, 0xFF819187, 0xFF757342,
        0xFFD3B052, 0xFFE3AD40, 0xFFE3AD49, 0xFFA87F30, 0xFF634400, 0xFF564600, 0xFF565004, 0xFF4F4E0D, 0xFF4A4607, 0xFF554D05, 0xFF594E00, 0xFF644600, 0xFF7B5C02, 0xFFBFA036, 0xFFCBAA41, 0xFF93731C, 0xFF765913, 0xFF715C1B, 0xFF6B5C19, 0xFF684B11,
        0xFF67570A, 0xFF5E511A, 0xFF604C19, 0xFF644B00, 0xFF9F7F2A, 0xFFA58C4C, 0xFF564900, 0xFF5E5000, 0xFF6E5A03, 0xFF977D28, 0xFFBFA04D, 0xFF8B6C19, 0xFF9A802B, 0xFFB8A44D, 0xFF584A00, 0xFF917128, 0xFFC4A44D, 0xFFD4B251, 0xFFE9C567, 0xFFEBC572,
        0xFFE7C176, 0xFFE9C976, 0xFFEED175, 0xFFD6B776, 0xFFC1A34D, 0xFFD3AF41, 0xFFE4B33C, 0xFFD99F31, 0xFFAE7D21, 0xFFA08639, 0xFF5D5910, 0xFF654E0B, 0xFF705D1B, 0xFF605012, 0xFF776825, 0xFFCEBB6F, 0xFFEECE77, 0xFFD6AB45, 0xFFE7B346, 0xFFC9A43A,
        0xFFB59735, 0xFFC3A746, 0xFFDCB34A, 0xFFECB340, 0xFFEEB13E, 0xFFB58821, 0xFF6D4F00, 0xFF5D480F, 0xFF625521, 0xFF5D5320, 0xFF5F5017, 0xFF6B520F, 0xFF6E530E, 0xFF5C4C0E, 0xFF5E5922, 0xFFB08B3D, 0xFFA7873E, 0xFFA18846, 0xFF605014, 0xFF4F420B,
        0xFF5E511A, 0xFF5B4A12, 0xFF5E4B11, 0xFF6A530D, 0xFFA88E37, 0xFFBFA036, 0xFFC29C31, 0xFFB28826, 0xFFB4882F, 0xFFBB9437, 0xFF9A7816, 0xFFAE8F34, 0xFF816323, 0xFF624D16, 0xFF595109, 0xFF6F6B24, 0xFFE5DEA8, 0xFFBCB275, 0xFFBDB35A, 0xFFDEC467,
        0xFFDBB958, 0xFFDCB149, 0xFFE4B54B, 0xFFC19630, 0xFF836409, 0xFF675606, 0xFF5C540C, 0xFF605805, 0xFF605805, 0xFF605805, 0xFF5F5704, 0xFF5F5704, 0xFF5F5704, 0xFF5F5704, 0xFF5E5603, 0xFF5D5605, 0xFF5D5605, 0xFF5E5706, 0xFF605908, 0xFF615A09,
        0xFF615A09, 0xFF5E5706, 0xFF5B5403, 0xFF615501, 0xFF67570C, 0xFF564300, 0xFF695307, 0xFF977C21, 0xFFA88622, 0xFFA8811C, 0xFFA97A1E, 0xFF765A07, 0xFF4F4931, 0xFF51605D, 0xFF4E5A56, 0xFF4E575E, 0xFF475B59, 0xFF455D4D, 0xFF4F5B5B, 0xFF495A50,
        0xFF495A50, 0xFF495A50, 0xFF495A50, 0xFF47584E, 0xFF46574D, 0xFF45564C, 0xFF45564C, 0xFF44544A, 0xFF4B5B51, 0xFF47574D, 0xFF47574D, 0xFF46564C, 0xFF46564C, 0xFF4E5E54, 0xFF4A5A50, 0xFF4F605A, 0xFF495A54, 0xFF485953, 0xFF3F4F4C, 0xFF465653,
        0xFF425251, 0xFF485857, 0xFF475756, 0xFF485857, 0xFF4E5E5D, 0xFF4C5C5B, 0xFF485855, 0xFF485855, 0xFF485953, 0xFF4B5C56, 0xFF53645E, 0xFF53645A, 0xFF495A50, 0xFF55665C, 0xFF495A52, 0xFF52635B, 0xFF56675F, 0xFF4B5C56, 0xFF475852, 0xFF475963,
        0xFF485656, 0xFF48534B, 0xFF445252, 0xFF475A60, 0xFF3A4944, 0xFF333510, 0xFF564E0F, 0xFF67501A, 0xFF4F3B00, 0xFF5B4F07, 0xFF665E13, 0xFFB9B369, 0xFFDFD695, 0xFFD7CA93, 0xFFE5D5A2, 0xFFEEE6B8, 0xFFCBC490, 0xFF807A3C, 0xFF5A530D, 0xFF584C04,
        0xFF68540D, 0xFF6B4E0C, 0xFF6D4B0C, 0xFF634D1B, 0xFF6E5515, 0xFF684700, 0xFF856101, 0xFFD4B251, 0xFF83650F, 0xFF65510A, 0xFF5F4F14, 0xFF5C4A08, 0xFF524705, 0xFF635608, 0xFF9C7A16, 0xFFEDB23E, 0xFFE9AA40, 0xFF90661A, 0xFF625020, 0xFF6F5411,
        0xFF695107, 0xFF6A530D, 0xFF705216, 0xFF6C4B08, 0xFF725C07, 0xFF747321, 0xFF5A682B, 0xFF050908, 0xFF000604, 0xFF182420, 0xFF344441, 0xFF3A4D49, 0xFF455554, 0xFF475555, 0xFF344040, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FEFFFF, 0x00FFFDFE, 0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF525E5A, 0xFF3A4A40, 0xFF070D0B, 0xFF0F1015, 0xFF3E4245, 0xFF243225, 0xFF475733, 0xFF6D753C, 0xFF686321, 0xFF715A26, 0xFF725B27, 0xFF6F5822, 0xFF745E25, 0xFF745E24,
        0xFF695317, 0xFF7D6829, 0xFFAD9857, 0xFFCDB471, 0xFFD5BC79, 0xFFCFB674, 0xFFAD9452, 0xFF826929, 0xFF695010, 0xFF695010, 0xFF72581B, 0xFF6F5113, 0xFF6B4E12, 0xFF6F541C, 0xFF70571F, 0xFF6A4F1A, 0xFF745620, 0xFF6A4B15, 0xFF75541E, 0xFF6E510B,
        0xFF745916, 0xFF7B6229, 0xFF6C572A, 0xFF98896A, 0xFFF0E9D9, 0xFFFFFFFB, 0xFFF9FDFF, 0xFFF4FFF8, 0xFFF9FFFD, 0xFFFAFFFC, 0xFFFEFFFD, 0xFFF6F7F1, 0xFFFFFFF4, 0xFFF2F7E3, 0xFF757D65, 0xFF313328, 0xFF556157, 0xFF626D71, 0xFF554D58, 0xFF6F4D34,
        0xFFC0903B, 0xFFC99B21, 0xFFE0B739, 0xFF776614, 0xFF97804C, 0xFFA8946F, 0xFF9A9C6B, 0xFF9BB284, 0xFF89A084, 0xFFADB190, 0xFFA7985F, 0xFFB09B58, 0xFFB89541, 0xFFA27510, 0xFFC09232, 0xFFD4AF61, 0xFFE0C37F, 0xFFE2C277, 0xFFD8B45C, 0xFF88671A,
        0xFF755812, 0xFF665110, 0xFF615115, 0xFF5E4C0E, 0xFF68510B, 0xFF967929, 0xFFCBA952, 0xFF746021, 0xFF554305, 0xFF7D5F16, 0xFFD3A044, 0xFFD1A043, 0xFFA48A3F, 0xFF584F0E, 0xFF877E3D, 0xFFCBBA74, 0xFFD3B35E, 0xFFDFAF4B, 0xFFEAB54D, 0xFFEAB855,
        0xFFE9B85C, 0xFFF0B75D, 0xFFF6B559, 0xFFF2C06B, 0xFFEEB95B, 0xFFEEBB52, 0xFFD2AB44, 0xFF81690F, 0xFFAF9F52, 0xFFCFBE78, 0xFF6C5511, 0xFF665004, 0xFF544B10, 0xFF6F6A32, 0xFFD5BF6A, 0xFFFAC555, 0xFFEAA734, 0xFFECB950, 0xFFC0A441, 0xFFBF9B4F,
        0xFFC1983C, 0xFFE6B445, 0xFFE8B341, 0xFFE7B64F, 0xFFA78629, 0xFF695B02, 0xFF575700, 0xFF6E501C, 0xFF624913, 0xFF635015, 0xFF615410, 0xFF594D03, 0xFF65570C, 0xFF63520A, 0xFF7A6923, 0xFF9A7C3C, 0xFF6F5F21, 0xFFC5B8AF, 0xFF7F784E, 0xFF5B5100,
        0xFF69521C, 0xFF6F5A25, 0xFF6F6118, 0xFFBAA169, 0xFFB49C54, 0xFFA78C33, 0xFFBC9E3E, 0xFFB08F30, 0xFF9F7E1F, 0xFFAC8F29, 0xFFA88E1F, 0xFF9F8629, 0xFF877624, 0xFF514C08, 0xFF5D6126, 0xFFC3C387, 0xFFD8CF82, 0xFFCEB656, 0xFFDFBD51, 0xFFE1A834,
        0xFFE1AC3C, 0xFFDFB046, 0xFFD7AE45, 0xFFC29B32, 0xFFB1881C, 0xFFC09125, 0xFFC49124, 0xFFB48C1D, 0xFFBC9327, 0xFFB79027, 0xFFAF8A22, 0xFFB38C25, 0xFFB68D24, 0xFFB68A21, 0xFFBA8B21, 0xFFC08B23, 0xFFBC8822, 0xFFBC8A27, 0xFFB88A2A, 0xFFB68B2F,
        0xFFB68E36, 0xFF99751F, 0xFF6B4800, 0xFF70540B, 0xFF6D5014, 0xFF6A4E1C, 0xFF644914, 0xFF604906, 0xFF624E05, 0xFF5F4D05, 0xFF554200, 0xFF3F360F, 0xFF3D4746, 0xFF435C58, 0xFF495A4A, 0xFF50585A, 0xFF425553, 0xFF455D4D, 0xFF485951, 0xFF475555,
        0xFF465454, 0xFF465454, 0xFF455353, 0xFF465454, 0xFF465454, 0xFF475555, 0xFF475555, 0xFF445350, 0xFF41504D, 0xFF41504D, 0xFF455451, 0xFF475653, 0xFF455451, 0xFF445350, 0xFF455451, 0xFF44554D, 0xFF45564E, 0xFF45564E, 0xFF44554D, 0xFF42534B,
        0xFF42534B, 0xFF44554D, 0xFF46574F, 0xFF495956, 0xFF495956, 0xFF4A5A57, 0xFF4B5C56, 0xFF4B5C56, 0xFF4A5B53, 0xFF475850, 0xFF45564E, 0xFF4B5A55, 0xFF4A5954, 0xFF4C5B56, 0xFF4C5B56, 0xFF4A5954, 0xFF495853, 0xFF495853, 0xFF475651, 0xFF475850,
        0xFF4A5750, 0xFF475351, 0xFF465558, 0xFF425655, 0xFF374433, 0xFF3E390F, 0xFF5C460A, 0xFF5E4912, 0xFF5D480F, 0xFF59460C, 0xFF56460B, 0xFF5C4E11, 0xFF665C1E, 0xFF736A29, 0xFF776F2E, 0xFF675314, 0xFF5F4B0C, 0xFF5C4706, 0xFF604B0A, 0xFF68510E,
        0xFF69520F, 0xFF68510E, 0xFF6A510E, 0xFF684909, 0xFF6C5505, 0xFF6A530F, 0xFF634200, 0xFFD4AD48, 0xFFD0A441, 0xFF896310, 0xFF977D1E, 0xFFCCAE4E, 0xFFC19C44, 0xFFC9A141, 0xFFC79E2C, 0xFFD2AA3B, 0xFF9C7822, 0xFF7D5F17, 0xFF6D560A, 0xFF6D4F13,
        0xFF6A530D, 0xFF65510C, 0xFF684E11, 0xFF694C06, 0xFF6D5902, 0xFF726D19, 0xFF636C2B, 0xFF0E0C0D, 0xFF030504, 0xFF18211E, 0xFF354441, 0xFF3C4C49, 0xFF455554, 0xFF4A5858, 0xFF3B4949, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FEFFFF, 0x00FFFDFE, 0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF525E5A, 0xFF3E4E44, 0xFF090F0D, 0xFF0F1015, 0xFF3C4043, 0xFF233124, 0xFF485834, 0xFF6E763D, 0xFF6A6523, 0xFF79622E, 0xFF7A632F, 0xFF725B25, 0xFF705A21, 0xFF745E24,
        0xFF705A1E, 0xFF6A5516, 0xFF6E591A, 0xFF6C5313, 0xFF6F5616, 0xFF6E5515, 0xFF6B5114, 0xFF6A5013, 0xFF6D5318, 0xFF6F551A, 0xFF6F551A, 0xFF74561A, 0xFF75581E, 0xFF6F541C, 0xFF6E531B, 0xFF6C5119, 0xFF644911, 0xFF806329, 0xFF67480F, 0xFF7D601E,
        0xFF72581B, 0xFF695321, 0xFF92825E, 0xFFF0E5CF, 0xFFFFFFF3, 0xFFF3EFEC, 0xFFFFFFFF, 0xFFF5FCF4, 0xFFF9FFF9, 0xFFFCFFFD, 0xFFF3F5F2, 0xFFFEFFFB, 0xFFF8FBF2, 0xFFB0B6A8, 0xFF3B4431, 0xFF2E3428, 0xFF585755, 0xFF4E4438, 0xFF544318, 0xFF917838,
        0xFF80590E, 0xFFCE9834, 0xFFECB22C, 0xFFB1913E, 0xFFB19444, 0xFF99863A, 0xFF8E884C, 0xFF79795D, 0xFF57534A, 0xFF8C7D6A, 0xFF967E50, 0xFF868C4E, 0xFF8C8535, 0xFFAD9437, 0xFFB7933B, 0xFFE1BB74, 0xFFE1B77B, 0xFFE8B671, 0xFFE1A656, 0xFFE6C168,
        0xFFDBB761, 0xFF967629, 0xFF694F05, 0xFF6C540E, 0xFF594100, 0xFF725710, 0xFFD1B56B, 0xFF715F21, 0xFF5C4C0E, 0xFF705209, 0xFFE4AF51, 0xFFC89333, 0xFF836412, 0xFF8C7A32, 0xFFD0BD71, 0xFFDCBC67, 0xFFD4A847, 0xFFE0AB3B, 0xFFE5AD3C, 0xFFE7B449,
        0xFFEABD58, 0xFFE8B653, 0xFFEFB853, 0xFFDCB14B, 0xFFF4C55B, 0xFFD1A236, 0xFF99730E, 0xFF9E8832, 0xFF9F8F44, 0xFF69550E, 0xFF684C03, 0xFF685105, 0xFF786521, 0xFFCCBA70, 0xFFE1C061, 0xFFE1AC3A, 0xFFF2B445, 0xFFCC9C36, 0xFFBE9D40, 0xFFC29B40,
        0xFFE5BC53, 0xFFE1B441, 0xFFE6B648, 0xFFA77A1D, 0xFF6F4E00, 0xFF5E4C02, 0xFF5F570E, 0xFF7B6A3E, 0xFF9C8E5D, 0xFF8A7F47, 0xFF92884A, 0xFFAEA560, 0xFF8B7F35, 0xFF5F4D03, 0xFF7D6A1E, 0xFFAE8D3C, 0xFF978638, 0xFF665E49, 0xFF6C6A39, 0xFF868326,
        0xFF9D8A48, 0xFF907C37, 0xFFA99942, 0xFFB5A142, 0xFFAC952D, 0xFFAB9120, 0xFFA78B1E, 0xFFC2A346, 0xFF7B5C0A, 0xFF9C8130, 0xFF9C842E, 0xFFAC9641, 0xFFBEAE61, 0xFF5A5211, 0xFF5F5D20, 0xFFB0A867, 0xFFB3A14F, 0xFFD5B550, 0xFFCFA634, 0xFFDFAA36,
        0xFFE2AF3C, 0xFFDAA839, 0xFFD9AB3C, 0xFFD9AB3C, 0xFFD7A93A, 0xFFD8A839, 0xFFD2A031, 0xFFD7A22E, 0xFFD6A32E, 0xFFD5A332, 0xFFD5A536, 0xFFD6A839, 0xFFD2A337, 0xFFD1A135, 0xFFD6A63A, 0xFFD5A330, 0xFFD3A130, 0xFFD4A435, 0xFFD2A337, 0xFFCEA237,
        0xFFD1A63E, 0xFFC39933, 0xFFA27B14, 0xFF654F04, 0xFF624D0E, 0xFF68511D, 0xFF6A5520, 0xFF614F0F, 0xFF5A4B06, 0xFF574908, 0xFF55460B, 0xFF3F3309, 0xFF3E4744, 0xFF455E5B, 0xFF455548, 0xFF495153, 0xFF3D504E, 0xFF415949, 0xFF43524B, 0xFF455451,
        0xFF455451, 0xFF445350, 0xFF445350, 0xFF455451, 0xFF455451, 0xFF465552, 0xFF475653, 0xFF45544F, 0xFF42514C, 0xFF42514C, 0xFF465550, 0xFF485752, 0xFF465550, 0xFF45544F, 0xFF465550, 0xFF475850, 0xFF485951, 0xFF495A52, 0xFF495A52, 0xFF485951,
        0xFF485951, 0xFF495A52, 0xFF4B5C54, 0xFF4A5A57, 0xFF475754, 0xFF445451, 0xFF43544E, 0xFF455650, 0xFF475850, 0xFF485951, 0xFF485951, 0xFF465550, 0xFF485752, 0xFF485752, 0xFF485752, 0xFF495853, 0xFF485752, 0xFF45544F, 0xFF45544F, 0xFF475850,
        0xFF49564F, 0xFF465250, 0xFF455457, 0xFF415554, 0xFF374433, 0xFF3F3A10, 0xFF5D470B, 0xFF604A10, 0xFF614B11, 0xFF5F4B10, 0xFF5A480A, 0xFF574709, 0xFF584A09, 0xFF594B0A, 0xFF554A08, 0xFF645011, 0xFF5F4C0A, 0xFF5E4908, 0xFF634E0D, 0xFF69520F,
        0xFF69520F, 0xFF684F0C, 0xFF684F0C, 0xFF715212, 0xFF675002, 0xFF634B0B, 0xFF6C4A0A, 0xFFAD8523, 0xFFB38724, 0xFFBC973F, 0xFFB9A03B, 0xFFCBA542, 0xFFEEC66E, 0xFFD5AC50, 0xFF916A01, 0xFFB18E2A, 0xFF785708, 0xFF624501, 0xFF715B0F, 0xFF6F5115,
        0xFF6D5610, 0xFF68540F, 0xFF6C5215, 0xFF6C4F09, 0xFF6F5B04, 0xFF736E1A, 0xFF656E2D, 0xFF0D0B0C, 0xFF020403, 0xFF17201D, 0xFF33423F, 0xFF384845, 0xFF3E4E4D, 0xFF414F4F, 0xFF313F3F, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FEFFFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF5E6A66, 0xFF415147, 0xFF0B110F, 0xFF101116, 0xFF393D40, 0xFF212F22, 0xFF495935, 0xFF6E763D, 0xFF6B6624, 0xFF765F29, 0xFF79622C, 0xFF77602A, 0xFF715B22, 0xFF725C22,
        0xFF755F25, 0xFF705A1E, 0xFF665014, 0xFF72581D, 0xFF73591E, 0xFF72581D, 0xFF6F541C, 0xFF70551D, 0xFF71561F, 0xFF6F541D, 0xFF6B5019, 0xFF6C4F17, 0xFF71541C, 0xFF6D5318, 0xFF6B5116, 0xFF705619, 0xFF775D20, 0xFFC9B173, 0xFF907838, 0xFF74551E,
        0xFF634C1A, 0xFF9E8E6A, 0xFFF0E8D1, 0xFFFFFFF3, 0xFFFFFFFA, 0xFFFFFEFD, 0xFFFDF7F7, 0xFFFBFCF4, 0xFFFDFEF8, 0xFFF7F9F6, 0xFFFAFCFB, 0xFFFCFFFF, 0xFFCAD1CA, 0xFF464D45, 0xFF262E21, 0xFF37372F, 0xFF483628, 0xFF593C10, 0xFF8F782A, 0xFF695703,
        0xFF886E24, 0xFFD0A24D, 0xFFD89928, 0xFFC59541, 0xFF937213, 0xFF5A4B00, 0xFF6A5E16, 0xFF9B8B4F, 0xFFB59E58, 0xFFBDA452, 0xFFBAA14D, 0xFFC1A96B, 0xFFC1A252, 0xFFE7BF5F, 0xFFFACE6B, 0xFFF5C970, 0xFFFCD079, 0xFFEABA56, 0xFFE3B140, 0xFFE5B94C,
        0xFFE9BD52, 0xFFD2A741, 0xFF9B7216, 0xFF704B00, 0xFF654701, 0xFF654D0F, 0xFF5C4710, 0xFF614F11, 0xFF69591D, 0xFF705307, 0xFFDAA444, 0xFFCC932C, 0xFFB48B2F, 0xFFD2B45E, 0xFFD2AF55, 0xFFD9A83F, 0xFFECB443, 0xFFE5AA34, 0xFFDCA530, 0xFFDEB245,
        0xFFCEAB47, 0xFFBE9B3F, 0xFFC6A146, 0xFFCEB149, 0xFFC3A239, 0xFFC4A240, 0xFFB69A46, 0xFF93823C, 0xFF625413, 0xFF5C4803, 0xFF654800, 0xFF7A5C13, 0xFFD5B562, 0xFFE7BF5F, 0xFFDEAE42, 0xFFE5B03E, 0xFFD4A134, 0xFFC19534, 0xFFB9953D, 0xFFDAB044,
        0xFFDCB341, 0xFFCEA533, 0xFFB18A23, 0xFF724D00, 0xFF6D4F11, 0xFF624C12, 0xFF615115, 0xFFB0B08C, 0xFFAEAC83, 0xFF9F9D6C, 0xFFBDBB82, 0xFFECE8A8, 0xFFAEA45D, 0xFF5C4900, 0xFF7F6614, 0xFFBF9934, 0xFF9E8B2F, 0xFF7A7653, 0xFF636A34, 0xFFB7BA61,
        0xFF8B7D32, 0xFF655000, 0xFF9B8321, 0xFFB5994F, 0xFFF1D581, 0xFFAC8F31, 0xFF8C6E14, 0xFFBDA052, 0xFFA68E46, 0xFF8A772B, 0xFF5A4B00, 0xFF6D5914, 0xFF5F4D0B, 0xFF5E5013, 0xFF514606, 0xFF806E22, 0xFFC6A94D, 0xFFDAB040, 0xFFD9A72C, 0xFFD4A22F,
        0xFFE3B03A, 0xFFDBA72F, 0xFFDBA72F, 0xFFD9A430, 0xFFDAA636, 0xFFD3A334, 0xFFD2A532, 0xFFD9A12A, 0xFFDBA32C, 0xFFDBA42F, 0xFFD8A12D, 0xFFD19C28, 0xFFD29D2B, 0xFFD4A030, 0xFFD29E2E, 0xFFCC9F22, 0xFFCB9E21, 0xFFD1A427, 0xFFD2A528, 0xFFD2A128,
        0xFFDAA930, 0xFFDEAC33, 0xFFD3A128, 0xFF88741E, 0xFF6D5910, 0xFF604A10, 0xFF645015, 0xFF665412, 0xFF61520D, 0xFF5E500F, 0xFF5D4E15, 0xFF423200, 0xFF3D423C, 0xFF445A58, 0xFF405045, 0xFF424C4D, 0xFF3B4E4A, 0xFF425949, 0xFF45514F, 0xFF43524B,
        0xFF43524B, 0xFF43524B, 0xFF43524B, 0xFF44534C, 0xFF45544D, 0xFF46554E, 0xFF47564F, 0xFF48584E, 0xFF45554B, 0xFF44544A, 0xFF48584E, 0xFF49594F, 0xFF47574D, 0xFF45554B, 0xFF46564C, 0xFF485951, 0xFF495A52, 0xFF4B5C54, 0xFF4C5D55, 0xFF4C5D55,
        0xFF4C5D55, 0xFF4D5E56, 0xFF4E5F57, 0xFF4E5E5B, 0xFF495956, 0xFF455552, 0xFF455650, 0xFF495A54, 0xFF4E5F57, 0xFF4F6058, 0xFF4F6058, 0xFF495853, 0xFF4F5E59, 0xFF4D5C57, 0xFF4C5B56, 0xFF51605B, 0xFF4D5C57, 0xFF475651, 0xFF4A5954, 0xFF45564E,
        0xFF48554E, 0xFF44504E, 0xFF445356, 0xFF405453, 0xFF374433, 0xFF403B11, 0xFF5F490D, 0xFF61470A, 0xFF654B0E, 0xFF674F11, 0xFF65500F, 0xFF65500F, 0xFF64510D, 0xFF624F0B, 0xFF5C4B06, 0xFF624D0C, 0xFF614C0B, 0xFF624D0C, 0xFF67500D, 0xFF6A5310,
        0xFF69500D, 0xFF684F0C, 0xFF674E0B, 0xFF6C4C0F, 0xFF644C02, 0xFF6C5418, 0xFF6D4B0E, 0xFFAC8425, 0xFFA47815, 0xFF8B660B, 0xFFB39B2D, 0xFFAC8216, 0xFFAF8227, 0xFF744A00, 0xFF9A751A, 0xFFA5822E, 0xFF634401, 0xFF6B5210, 0xFF695405, 0xFF6C4E12,
        0xFF6B540E, 0xFF67530E, 0xFF6D5316, 0xFF6F520C, 0xFF715D06, 0xFF76711D, 0xFF6A7332, 0xFF0F0D0E, 0xFF040605, 0xFF19221F, 0xFF354441, 0xFF394946, 0xFF3E4E4D, 0xFF404E4E, 0xFF2F3D3D, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FEFFFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF616D69, 0xFF415147, 0xFF0B110F, 0xFF0F1015, 0xFF363A3D, 0xFF212F22, 0xFF4B5B37, 0xFF6C743B, 0xFF6A6523, 0xFF7B642E, 0xFF78612B, 0xFF7A632D, 0xFF7A642B, 0xFF725C23,
        0xFF705A21, 0xFF755F26, 0xFF755F26, 0xFF6F541C, 0xFF745922, 0xFF765B24, 0xFF725720, 0xFF6E531E, 0xFF6E531E, 0xFF6F5321, 0xFF6E5220, 0xFF71561F, 0xFF6F541C, 0xFF6F5218, 0xFF6E5213, 0xFF684F0F, 0xFF796121, 0xFFBAA667, 0xFF7C6A2C, 0xFF654A1F,
        0xFFA79370, 0xFFEDE2CC, 0xFFFFFFF6, 0xFFFEFFFF, 0xFFF5F5F7, 0xFFFFFCFC, 0xFFFFF7F6, 0xFFFFFEF8, 0xFFFEFBF6, 0xFFFFFFFD, 0xFFFCFFFF, 0xFFCFD5D3, 0xFF494F4B, 0xFF252C25, 0xFF2B322A, 0xFF493627, 0xFF543F10, 0xFF866F2C, 0xFF978449, 0xFFB5AB6E,
        0xFFA1984D, 0xFF746115, 0xFFB39154, 0xFFC29542, 0xFF7F6616, 0xFF6D6429, 0xFFAEA473, 0xFFBCA65A, 0xFFBFA42F, 0xFFC9B143, 0xFFD1BA74, 0xFFD5B87C, 0xFFCBAE62, 0xFFB99A3F, 0xFFC6A643, 0xFFBE9D3C, 0xFFC7A541, 0xFFC0A12D, 0xFFDBBA37, 0xFFF9C74E,
        0xFFE0AC32, 0xFFF0B942, 0xFFE8B343, 0xFF9D6F0F, 0xFF704C00, 0xFF6A5118, 0xFF59481A, 0xFF604E10, 0xFF5C4C10, 0xFF79590E, 0xFFE2AB47, 0xFFDEA033, 0xFFD7A640, 0xFFDEB450, 0xFFD5A539, 0xFFE5AA34, 0xFFEDB440, 0xFFE7B343, 0xFFC0952D, 0xFF715100,
        0xFF6B550C, 0xFFA39358, 0xFF938652, 0xFF7F762B, 0xFF7E7029, 0xFFA18F51, 0xFF988C5A, 0xFF8E895F, 0xFF676333, 0xFF695A19, 0xFF745B0B, 0xFFD6B065, 0xFFF4C664, 0xFFE1AA36, 0xFFECB342, 0xFFD6A338, 0xFFB38A21, 0xFFCBA747, 0xFFBB9445, 0xFFCA9930,
        0xFFB08515, 0xFFBE982D, 0xFF917013, 0xFF6B4D07, 0xFF684D16, 0xFF654E18, 0xFF5E480C, 0xFFA5B392, 0xFF6B714B, 0xFF7F8150, 0xFF7B7A44, 0xFF7E7D44, 0xFF817938, 0xFF624F02, 0xFF80620C, 0xFFC99D30, 0xFF917A1C, 0xFF908E68, 0xFF626E3E, 0xFFD0D690,
        0xFF7F732B, 0xFF937C16, 0xFFAF8E2F, 0xFFC2A85D, 0xFFE9D17B, 0xFF9C8327, 0xFFA89038, 0xFFB6A054, 0xFF9C8B43, 0xFF817829, 0xFF5F5B02, 0xFF5E4B13, 0xFF5A470D, 0xFF614D0E, 0xFF7C661D, 0xFFBD9F49, 0xFFDFB64D, 0xFFD9A52F, 0xFFE9AE2E, 0xFFDAAC3C,
        0xFFE4B33A, 0xFFDDA828, 0xFFE0A92B, 0xFFDEA733, 0xFFD6A338, 0xFFC79B30, 0xFFCCA637, 0xFFCFA63D, 0xFFC79C33, 0xFFCD9D2F, 0xFFD7A333, 0xFFD6A12D, 0xFFD69F2A, 0xFFD69F2A, 0xFFD29B26, 0xFFCEA229, 0xFFCB9F24, 0xFFCFA225, 0xFFD3A124, 0xFFD29C1E,
        0xFFD7A021, 0xFFE0A726, 0xFFE1A624, 0xFFBDA23B, 0xFF93751B, 0xFF715408, 0xFF6D5108, 0xFF715808, 0xFF735A08, 0xFF6D550B, 0xFF674E0C, 0xFF584407, 0xFF43453A, 0xFF415554, 0xFF3E4E44, 0xFF424C4D, 0xFF3B4E48, 0xFF405747, 0xFF465051, 0xFF435349,
        0xFF435349, 0xFF435349, 0xFF44544A, 0xFF45554B, 0xFF46564C, 0xFF47574D, 0xFF48584E, 0xFF49594E, 0xFF46564B, 0xFF46564B, 0xFF49594E, 0xFF4A5A4F, 0xFF47574C, 0xFF46564B, 0xFF46564B, 0xFF475850, 0xFF485951, 0xFF495A52, 0xFF4B5C54, 0xFF4B5C54,
        0xFF4B5C54, 0xFF4A5B53, 0xFF4A5B53, 0xFF475754, 0xFF455552, 0xFF435350, 0xFF465751, 0xFF4C5D57, 0xFF4F6058, 0xFF4D5E56, 0xFF4A5B53, 0xFF475651, 0xFF505F5A, 0xFF4C5B56, 0xFF4A5954, 0xFF505F5A, 0xFF4B5A55, 0xFF42514C, 0xFF475651, 0xFF44554D,
        0xFF46534C, 0xFF424E4C, 0xFF425154, 0xFF3F5352, 0xFF364332, 0xFF413C12, 0xFF604A0E, 0xFF654908, 0xFF684C0B, 0xFF694D0C, 0xFF664D0A, 0xFF664D0A, 0xFF684F0C, 0xFF68500A, 0xFF654D07, 0xFF614C0B, 0xFF624D0C, 0xFF664F0C, 0xFF68510E, 0xFF69500D,
        0xFF684F0C, 0xFF674E0B, 0xFF674F09, 0xFF6B4B0E, 0xFF6F570F, 0xFF6B521A, 0xFF644109, 0xFFB2892D, 0xFF9E720F, 0xFF644000, 0xFFB09929, 0xFFECBE4F, 0xFF94650B, 0xFF744900, 0xFF8E6918, 0xFFAE8E43, 0xFF65480C, 0xFF695113, 0xFF645104, 0xFF6A4C10,
        0xFF6A530D, 0xFF66520D, 0xFF6C5215, 0xFF6F520C, 0xFF6F5B04, 0xFF726D19, 0xFF6A7332, 0xFF0E0C0D, 0xFF040605, 0xFF18211E, 0xFF344340, 0xFF3A4A47, 0xFF40504F, 0xFF445252, 0xFF344242, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FEFFFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF55615D, 0xFF3E4E44, 0xFF090F0D, 0xFF0F1015, 0xFF34383B, 0xFF223023, 0xFF4E5E3A, 0xFF6A7239, 0xFF686321, 0xFF7B652C, 0xFF705A21, 0xFF735D24, 0xFF7B642E, 0xFF765F29,
        0xFF725B25, 0xFF735C26, 0xFF6F5822, 0xFF70551E, 0xFF70551E, 0xFF6E531C, 0xFF6C511C, 0xFF6F541F, 0xFF725624, 0xFF715523, 0xFF6C501E, 0xFF6C551F, 0xFF6D521A, 0xFF6D5014, 0xFF745618, 0xFF6B5212, 0xFF6B551B, 0xFF695A21, 0xFF5D521C, 0xFFA48F72,
        0xFFF8E9D4, 0xFFFFFFF3, 0xFFF1F5F4, 0xFFFBFFFF, 0xFFFEFFFF, 0xFFFFFCFB, 0xFFFFF9F7, 0xFFFFF8F2, 0xFFFBF8F3, 0xFFFEFFFB, 0xFFDCE2E0, 0xFF505954, 0xFF232A23, 0xFF23261D, 0xFF414338, 0xFF523002, 0xFF6E5D25, 0xFF878354, 0xFFB0AF93, 0xFFBFBC9D,
        0xFF858553, 0xFF3F481B, 0xFF35432C, 0xFF735926, 0xFF948C5D, 0xFF959980, 0xFFC4C1B8, 0xFFBFAE90, 0xFFA68D4D, 0xFFAD9565, 0xFFC3AFA8, 0xFFA6AD7A, 0xFF82884C, 0xFF4F520B, 0xFF585208, 0xFFA49049, 0xFFAC8C41, 0xFFC6A149, 0xFFD7AF4D, 0xFFE5B33A,
        0xFFEBB338, 0xFFE4A929, 0xFFEDB135, 0xFFE7B144, 0xFF997016, 0xFF5C3F00, 0xFF6B5621, 0xFF685417, 0xFF594709, 0xFF77570A, 0xFFDCA540, 0xFFDD9E2D, 0xFFDAA639, 0xFFDCAA3B, 0xFFE9AE38, 0xFFE6AE37, 0xFFD9A738, 0xFFB18B2A, 0xFF735809, 0xFF524206,
        0xFF8B835E, 0xFFCECDBB, 0xFFC7CCC6, 0xFFCBD1B5, 0xFFA6A98E, 0xFFD2D1BD, 0xFFC2C4B6, 0xFFDDE4D4, 0xFF9EA27F, 0xFF675B1E, 0xFFD7BE6A, 0xFFE9B964, 0xFFE2B041, 0xFFF1BA43, 0xFFC38E28, 0xFFAA7D20, 0xFFBE9D36, 0xFFA28122, 0xFF8C661B, 0xFFBF8D2E,
        0xFFB48825, 0xFFAE8A29, 0xFF8B6D17, 0xFF614805, 0xFF6D5519, 0xFF9F8A4B, 0xFFA8944D, 0xFF8C9B74, 0xFF808356, 0xFF8C8652, 0xFF716E37, 0xFF969660, 0xFF858346, 0xFF554300, 0xFF85660C, 0xFFCC9C30, 0xFF957B24, 0xFFD3D0AF, 0xFF9CA785, 0xFFBBBE91,
        0xFFA49657, 0xFFA08720, 0xFFC39D4C, 0xFF9A9021, 0xFF9B911E, 0xFFACA032, 0xFF8F8222, 0xFF91853B, 0xFF898143, 0xFFD0CF8F, 0xFF5F621B, 0xFF665521, 0xFF5D480F, 0xFF664E08, 0xFFB89845, 0xFFE4BC5A, 0xFFD7A536, 0xFFDDA52C, 0xFFE1A425, 0xFFE1B246,
        0xFFDBAA31, 0xFFD7A625, 0xFFDCAA2F, 0xFFD2A138, 0xFFAE8325, 0xFF89680B, 0xFF836704, 0xFF7D6C1A, 0xFF7C640C, 0xFF997817, 0xFFC59A31, 0xFFD8A635, 0xFFD5A12B, 0xFFD69F28, 0xFFD9A22B, 0xFFD1A635, 0xFFCEA12E, 0xFFD3A12E, 0xFFD8A32F, 0xFFD9A12A,
        0xFFDDA127, 0xFFE2A429, 0xFFE6A62B, 0xFFDFB53C, 0xFFCA9E31, 0xFFB98C2E, 0xFFB5882D, 0xFFB28826, 0xFFB18A25, 0xFFB0892C, 0xFFAD8530, 0xFF7E631C, 0xFF504D3E, 0xFF435555, 0xFF405149, 0xFF455151, 0xFF3B4F46, 0xFF3B5041, 0xFF444B51, 0xFF44544A,
        0xFF44544A, 0xFF44544A, 0xFF44544A, 0xFF45554B, 0xFF47574D, 0xFF48584E, 0xFF49594F, 0xFF49594E, 0xFF46564B, 0xFF45554A, 0xFF48584D, 0xFF49594E, 0xFF47574C, 0xFF46564B, 0xFF46564B, 0xFF475850, 0xFF475850, 0xFF485951, 0xFF495A52, 0xFF495A52,
        0xFF495A52, 0xFF475850, 0xFF46574F, 0xFF465653, 0xFF445451, 0xFF445451, 0xFF485953, 0xFF4E5F59, 0xFF4F6058, 0xFF4C5D55, 0xFF485951, 0xFF495853, 0xFF52615C, 0xFF4D5C57, 0xFF4A5954, 0xFF505F5A, 0xFF495853, 0xFF404F4A, 0xFF45544F, 0xFF42534B,
        0xFF44514A, 0xFF414D4B, 0xFF415053, 0xFF3E5251, 0xFF364332, 0xFF413C12, 0xFF604A0E, 0xFF674B0A, 0xFF684C0B, 0xFF674C09, 0xFF644906, 0xFF634805, 0xFF674A06, 0xFF684B07, 0xFF664905, 0xFF634C09, 0xFF654E0B, 0xFF664F0C, 0xFF674E0B, 0xFF664D0A,
        0xFF674F09, 0xFF694E09, 0xFF694E09, 0xFF68480D, 0xFF725A12, 0xFF61480F, 0xFF68450B, 0xFFA1791A, 0xFFAB7F1C, 0xFF9F7A21, 0xFFAC942A, 0xFFA1780F, 0xFFC69944, 0xFF805507, 0xFF946E1D, 0xFF9A792C, 0xFF725418, 0xFF685012, 0xFF6A580C, 0xFF6E5014,
        0xFF6E5711, 0xFF68540F, 0xFF6C5215, 0xFF70530D, 0xFF6D5902, 0xFF6F6A16, 0xFF6A7332, 0xFF0D0B0C, 0xFF010302, 0xFF141D1A, 0xFF2F3E3B, 0xFF354542, 0xFF3D4D4C, 0xFF425050, 0xFF334141, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FEFFFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF54605C, 0xFF3D4D43, 0xFF080E0C, 0xFF101116, 0xFF33373A, 0xFF243225, 0xFF52623E, 0xFF697138, 0xFF676220, 0xFF745E25, 0xFF6F5920, 0xFF725C23, 0xFF755E28, 0xFF735C28,
        0xFF755E2A, 0xFF765F2D, 0xFF6F5826, 0xFF72571F, 0xFF71561E, 0xFF6E531C, 0xFF6B5019, 0xFF6D521D, 0xFF705520, 0xFF6E531E, 0xFF6A4E1C, 0xFF604B12, 0xFF6C531A, 0xFF6E5115, 0xFF735517, 0xFF72551D, 0xFF654F1D, 0xFF5E5125, 0xFFA8A079, 0xFFF2E9DA,
        0xFFFFFCF0, 0xFFFDFEF8, 0xFFF8FCFD, 0xFFF6FBFE, 0xFFFCFDFF, 0xFFFFFFFD, 0xFFFAF5F1, 0xFFFFFEFA, 0xFFFCFDF7, 0xFFD8DFD8, 0xFF535E56, 0xFF1E281F, 0xFF2A2E20, 0xFF33301F, 0xFF3D3623, 0xFF4F3400, 0xFF7C724F, 0xFFA6AEA3, 0xFF93A08E, 0xFFAEB6A1,
        0xFF80857E, 0xFF475B5A, 0xFF42685D, 0xFF4C4A4D, 0xFF899182, 0xFFA9B7A0, 0xFF9BA396, 0xFFA5A196, 0xFFBDB199, 0xFF9D906D, 0xFF9F966F, 0xFFB1905B, 0xFF997F44, 0xFFA08D4B, 0xFFCBB270, 0xFFEAC785, 0xFFEEC27B, 0xFFDCAF5C, 0xFFAA7F23, 0xFFA8790F,
        0xFFE4B040, 0xFFECB439, 0xFFDA9F1F, 0xFFEAB239, 0xFFE2B148, 0xFFA07B23, 0xFF684800, 0xFF624A0A, 0xFF645011, 0xFF755508, 0xFFCE9933, 0xFFD79928, 0xFFE3AF3F, 0xFFEAB744, 0xFFE1A427, 0xFFD9A02F, 0xFFC69B35, 0xFF866A17, 0xFF524000, 0xFFA39669,
        0xFFEAE4CA, 0xFFC0C2BD, 0xFFB3BDC6, 0xFFD3E0D9, 0xFFBDC6C1, 0xFFE4E9E5, 0xFFB0B6B2, 0xFFA5ADA0, 0xFFA0A07C, 0xFF9F8C47, 0xFFE6C365, 0xFFE2A844, 0xFFE8B740, 0xFFC1992A, 0xFF957022, 0xFFA37F35, 0xFFB1912E, 0xFF927411, 0xFF73520B, 0xFFB68632,
        0xFFB78C30, 0xFFAC882A, 0xFF8D6F19, 0xFF705810, 0xFFB9A463, 0xFF9F8D45, 0xFF7C6D1C, 0xFF565924, 0xFF8B8345, 0xFF897735, 0xFF73672B, 0xFFAFAE78, 0xFF98975F, 0xFF74661D, 0xFF82640A, 0xFFCA9B33, 0xFF947A26, 0xFF676242, 0xFF4A5336, 0xFF6D6C4E,
        0xFF89793E, 0xFFB39B31, 0xFF9D7633, 0xFFC4C18A, 0xFF989359, 0xFFC8C288, 0xFF9F9768, 0xFFABA281, 0xFF847F61, 0xFFD2D4AD, 0xFF565D2A, 0xFF5D4C14, 0xFF5C4706, 0xFFA88B3B, 0xFFD5AE4F, 0xFFC6972B, 0xFFDBA632, 0xFFE3AB34, 0xFFDCA12B, 0xFFD9A63D,
        0xFFCFA02A, 0xFFDDB134, 0xFFC9A130, 0xFFA5832C, 0xFF725511, 0xFF604E04, 0xFF5D5100, 0xFF564C04, 0xFF806F1F, 0xFFB29539, 0xFFCBA43B, 0xFFCF9F30, 0xFFD29F2A, 0xFFD3A02A, 0xFFCF9E27, 0xFFCB9C30, 0xFFC9992B, 0xFFCC9A2B, 0xFFD5A030, 0xFFDBA332,
        0xFFDFA331, 0xFFE1A431, 0xFFE2A331, 0xFFD6A11D, 0xFFD49D26, 0xFFD49E34, 0xFFD49F35, 0xFFD19F2E, 0xFFD1A22E, 0xFFD4A435, 0xFFCFA038, 0xFF937422, 0xFF554E3B, 0xFF435255, 0xFF41524C, 0xFF465552, 0xFF3D5347, 0xFF3C5142, 0xFF494D56, 0xFF44534C,
        0xFF44534C, 0xFF44534C, 0xFF44534C, 0xFF45544D, 0xFF46554E, 0xFF47564F, 0xFF485750, 0xFF47574D, 0xFF44544A, 0xFF44544A, 0xFF47574D, 0xFF48584E, 0xFF46564C, 0xFF45554B, 0xFF45554B, 0xFF495A52, 0xFF485951, 0xFF475850, 0xFF485951, 0xFF485951,
        0xFF485951, 0xFF475850, 0xFF46574F, 0xFF4C5C59, 0xFF485855, 0xFF465653, 0xFF475852, 0xFF4B5C56, 0xFF4D5E56, 0xFF4C5D55, 0xFF4A5B53, 0xFF4C5B56, 0xFF51605B, 0xFF4E5D58, 0xFF4C5B56, 0xFF4F5E59, 0xFF495853, 0xFF42514C, 0xFF45544F, 0xFF405149,
        0xFF435049, 0xFF404C4A, 0xFF415053, 0xFF3E5251, 0xFF364332, 0xFF403B11, 0xFF5F490D, 0xFF634B0B, 0xFF654D0D, 0xFF674E0C, 0xFF674B0A, 0xFF684D0A, 0xFF6B4E0C, 0xFF6B4E0C, 0xFF6A4B08, 0xFF644D0A, 0xFF654E0B, 0xFF664D0A, 0xFF644B08, 0xFF644C06,
        0xFF684D08, 0xFF694E09, 0xFF694E07, 0xFF68480F, 0xFF654D03, 0xFF644C0C, 0xFF654401, 0xFF976F0D, 0xFFCEA241, 0xFFAD8637, 0xFF7B6005, 0xFF654800, 0xFF967224, 0xFFC79E4F, 0xFFB68D31, 0xFF9F7720, 0xFF6D4A06, 0xFF63490C, 0xFF604E06, 0xFF6C4E12,
        0xFF6D5610, 0xFF67530E, 0xFF6D5316, 0xFF735610, 0xFF715D06, 0xFF75701C, 0xFF747D3C, 0xFF121011, 0xFF050706, 0xFF17201D, 0xFF303F3C, 0xFF344441, 0xFF3C4C4B, 0xFF414F4F, 0xFF324040, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FEFFFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF596561, 0xFF3D4D43, 0xFF090F0D, 0xFF101116, 0xFF323639, 0xFF263427, 0xFF576743, 0xFF6B733A, 0xFF6A6523, 0xFF766026, 0xFF776127, 0xFF776128, 0xFF735C26, 0xFF6D5622,
        0xFF6E5725, 0xFF705828, 0xFF6E5626, 0xFF71571C, 0xFF755B20, 0xFF755B20, 0xFF70551D, 0xFF6C5119, 0xFF6C511A, 0xFF6D521B, 0xFF6C511A, 0xFF665319, 0xFF6D5519, 0xFF705317, 0xFF6B4B10, 0xFF6A4C18, 0xFF675128, 0xFF978C6E, 0xFFEAE5CF, 0xFFFCFDF8,
        0xFFFEFFFB, 0xFFF9FBF8, 0xFFFBFDFA, 0xFFFEFFFD, 0xFFF6F8F5, 0xFFF6F7F2, 0xFFFFFFFB, 0xFFFBFCF7, 0xFFE0E5DF, 0xFF535E56, 0xFF1C2A1D, 0xFF1E2918, 0xFF30321C, 0xFF362C11, 0xFF443317, 0xFF898258, 0xFFB9B6A3, 0xFF8A9086, 0xFFA8B5A3, 0xFF6F7D70,
        0xFF434D56, 0xFF4A5865, 0xFF536567, 0xFF4F6477, 0xFF425158, 0xFF8D9888, 0xFFA4AF8F, 0xFF8B9678, 0xFFAAAF91, 0xFFB9B880, 0xFFBBB55D, 0xFFFCB468, 0xFFECB565, 0xFFF4CB7B, 0xFFD0AD5C, 0xFFA47E2D, 0xFFD7AD57, 0xFFBA9837, 0xFFAA912A, 0xFF8D630D,
        0xFFA67A19, 0xFFE2B03F, 0xFFF6C042, 0xFFE0AB29, 0xFFEBB73D, 0xFFDEAE3F, 0xFF9C7007, 0xFF6C510E, 0xFF5E4908, 0xFF79590C, 0xFFDCA843, 0xFFDAA130, 0xFFDAA839, 0xFFE2B13B, 0xFFE9AE2E, 0xFFE8A93F, 0xFFB98B2B, 0xFF816615, 0xFF604F0A, 0xFF908045,
        0xFFC7BA8D, 0xFFB0AB8D, 0xFF929685, 0xFFA2A997, 0xFF999C87, 0xFF92917C, 0xFF787762, 0xFF848364, 0xFF9C8F58, 0xFFC29E48, 0xFFDEA83C, 0xFFF4B140, 0xFFCBA329, 0xFF998628, 0xFFC7B585, 0xFFCBAD77, 0xFFA2821F, 0xFFA58820, 0xFF6C510A, 0xFFBD903B,
        0xFFAD8526, 0xFFB4902F, 0xFF88690E, 0xFF83671D, 0xFFDCC784, 0xFF80722B, 0xFF4F4300, 0xFF5C5008, 0xFFAF9548, 0xFFD3B263, 0xFFA18A46, 0xFF99945D, 0xFFA1A46F, 0xFFBBB16A, 0xFF836912, 0xFFCA9E31, 0xFFA18931, 0xFF807B51, 0xFF4C5230, 0xFF787258,
        0xFF9D8B4B, 0xFFB29B27, 0xFF956F30, 0xFF7B7E83, 0xFF686A67, 0xFF85857D, 0xFF726F68, 0xFF86837E, 0xFF66665A, 0xFF878C6C, 0xFF4A5626, 0xFF5A4B0A, 0xFFA69045, 0xFFC9A74D, 0xFFC79C33, 0xFFC79522, 0xFFD9A430, 0xFFDAA636, 0xFFDDA93C, 0xFFDDA43D,
        0xFFC69723, 0xFFD4B038, 0xFFA18726, 0xFF7E6C2C, 0xFF5E5325, 0xFF6B6833, 0xFF5B5E19, 0xFF836A16, 0xFFBD9E44, 0xFFE3B955, 0xFFDBA93A, 0xFFCF9A26, 0xFFD5A22C, 0xFFD5A833, 0xFFCCA331, 0xFFD0A12F, 0xFFCE9F2D, 0xFFD09E2B, 0xFFD5A22F, 0xFFD9A432,
        0xFFDAA32F, 0xFFD9A22E, 0xFFD8A12D, 0xFFD7A220, 0xFFD29B26, 0xFFD09A30, 0xFFCF9B2E, 0xFFCC9B25, 0xFFCB9E21, 0xFFCB9D25, 0xFFC59825, 0xFF947018, 0xFF544933, 0xFF425053, 0xFF3F4F4C, 0xFF455451, 0xFF3F5548, 0xFF415345, 0xFF4C4E5A, 0xFF43524F,
        0xFF43524F, 0xFF42514E, 0xFF42514E, 0xFF43524F, 0xFF445350, 0xFF445350, 0xFF455451, 0xFF44534E, 0xFF41504B, 0xFF41504B, 0xFF45544F, 0xFF465550, 0xFF45544F, 0xFF44534E, 0xFF45544F, 0xFF475850, 0xFF46574F, 0xFF44554D, 0xFF45564E, 0xFF46574F,
        0xFF475850, 0xFF475850, 0xFF46574F, 0xFF475754, 0xFF445451, 0xFF41514E, 0xFF40514B, 0xFF41524C, 0xFF42534B, 0xFF42534B, 0xFF41524A, 0xFF45544F, 0xFF465550, 0xFF465550, 0xFF45544F, 0xFF45544F, 0xFF42514C, 0xFF3F4E49, 0xFF3E4D48, 0xFF3F5048,
        0xFF424F48, 0xFF404C4A, 0xFF415053, 0xFF3E5251, 0xFF354231, 0xFF3F3A10, 0xFF5D470B, 0xFF5F4B0C, 0xFF634E0F, 0xFF644F10, 0xFF644C0C, 0xFF644B0B, 0xFF674B0A, 0xFF664808, 0xFF624404, 0xFF634C09, 0xFF664D0A, 0xFF654C09, 0xFF624906, 0xFF654A05,
        0xFF684D08, 0xFF694C06, 0xFF674A04, 0xFF6F4F16, 0xFF624B00, 0xFF654E08, 0xFF694900, 0xFFAA831A, 0xFFD3A648, 0xFF7D5510, 0xFF5F4100, 0xFF5A4D00, 0xFF684D06, 0xFF8F6916, 0xFFE2B64D, 0xFF9D7009, 0xFF633C00, 0xFF745817, 0xFF62510C, 0xFF694B0F,
        0xFF6B540E, 0xFF64500B, 0xFF6B5114, 0xFF735610, 0xFF725E07, 0xFF77721E, 0xFF798241, 0xFF131112, 0xFF070908, 0xFF19221F, 0xFF32413E, 0xFF354542, 0xFF3C4C4B, 0xFF404E4E, 0xFF313F3F, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FEFFFF, 0x00FEFCFD, 0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF54605C, 0xFF3F4F45, 0xFF0A100E, 0xFF111217, 0xFF323639, 0xFF273528, 0xFF596945, 0xFF6D753C, 0xFF6D6826, 0xFF745E24, 0xFF705A20, 0xFF6E581F, 0xFF715A24, 0xFF715A26,
        0xFF6C5523, 0xFF695121, 0xFF6A5224, 0xFF6C5215, 0xFF6E5417, 0xFF6D5316, 0xFF6B5116, 0xFF6B5116, 0xFF6C5119, 0xFF6B5018, 0xFF684D15, 0xFF69561B, 0xFF5D470B, 0xFF6C4E10, 0xFF68480F, 0xFF715223, 0xFF826B49, 0xFFDDD2BE, 0xFFF2EEE2, 0xFFF6FEFF,
        0xFFF8FEFE, 0xFFF9FEFA, 0xFFF9FAF5, 0xFFFAFBF5, 0xFFFEFFFA, 0xFFFCFFFB, 0xFFFCFFFD, 0xFFE6EBE7, 0xFF4C5550, 0xFF1E2B22, 0xFF1D2E1E, 0xFF2E3A24, 0xFF3E3E22, 0xFF2E2101, 0xFF7A6341, 0xFFBCC6CF, 0xFF8C8F7A, 0xFFB6B694, 0xFF7C7E71, 0xFF475556,
        0xFF4F5F5F, 0xFF56575C, 0xFF615063, 0xFF4C6F69, 0xFF52626F, 0xFF4A4A54, 0xFF8C917D, 0xFFA4B396, 0xFF909B8B, 0xFFC7C0AD, 0xFFC3AE81, 0xFF9C8829, 0xFFB0AA4C, 0xFFC3C86E, 0xFF5C5E0A, 0xFF8B7E2F, 0xFFBFA858, 0xFF6A5400, 0xFF72620A, 0xFFC19B54,
        0xFF8D6510, 0xFFA47911, 0xFFE4B33A, 0xFFEBB632, 0xFFDEA822, 0xFFE8B132, 0xFFECB439, 0xFF896C28, 0xFF5E4704, 0xFF7C5C0F, 0xFFD3A13E, 0xFFD39B2C, 0xFFE1B343, 0xFFDDAE3A, 0xFFEAB130, 0xFFE39E36, 0xFFC39132, 0xFF80610F, 0xFF695307, 0xFF624C00,
        0xFF5A4600, 0xFF5C5014, 0xFF373401, 0xFF1D1E00, 0xFF242100, 0xFF2E2700, 0xFF2E2600, 0xFF6D612D, 0xFFB89D56, 0xFFD39F3C, 0xFFE59D25, 0xFFF2AB2F, 0xFFC09E22, 0xFFA9A655, 0xFF989375, 0xFF9E875D, 0xFFB2902E, 0xFF917508, 0xFF6F5812, 0xFFBB9437,
        0xFFAF8926, 0xFFBB9830, 0xFF876607, 0xFF6B4E00, 0xFF77621F, 0xFF574908, 0xFF5D540F, 0xFF826915, 0xFFD6B158, 0xFFD7A951, 0xFF8A6C23, 0xFF99925C, 0xFF707441, 0xFF645D19, 0xFF826A14, 0xFFCDA533, 0xFFB9A244, 0xFF8C8652, 0xFF61633B, 0xFF857C5D,
        0xFFAD9B53, 0xFFB19D20, 0xFF8F6D2D, 0xFF697C5C, 0xFF536440, 0xFF626E4A, 0xFF51573B, 0xFF6D705D, 0xFF585D47, 0xFF656F4A, 0xFF43511E, 0xFF998842, 0xFFCCB663, 0xFFC5A341, 0xFFC19625, 0xFFCF9C26, 0xFFDAA732, 0xFFD5A539, 0xFFD8A842, 0xFFDDA23C,
        0xFFB78816, 0xFFC9A938, 0xFFA29139, 0xFFADA672, 0xFFAAA98A, 0xFFA8AF86, 0xFF646F35, 0xFFD2A846, 0xFFE4B34C, 0xFFE6AE3F, 0xFFDCA028, 0xFFD79D22, 0xFFD19E28, 0xFFC89F2D, 0xFFC6A234, 0xFFCD9C23, 0xFFCF9E25, 0xFFD09F26, 0xFFD2A128, 0xFFD7A42E,
        0xFFD8A52F, 0xFFD8A52F, 0xFFD9A630, 0xFFD6A62A, 0xFFCF9D2E, 0xFFCE9D36, 0xFFCFA036, 0xFFC89E28, 0xFFC79E22, 0xFFC9A127, 0xFFC9A22D, 0xFF926E10, 0xFF564A30, 0xFF455356, 0xFF3D4D4A, 0xFF404F4C, 0xFF3D5346, 0xFF3E5042, 0xFF474956, 0xFF425050,
        0xFF425050, 0xFF414F4F, 0xFF414F4F, 0xFF414F4F, 0xFF414F4F, 0xFF425050, 0xFF425050, 0xFF42514E, 0xFF3F4E4B, 0xFF404F4C, 0xFF43524F, 0xFF455451, 0xFF445350, 0xFF43524F, 0xFF445350, 0xFF43544C, 0xFF42534B, 0xFF405149, 0xFF41524A, 0xFF43544C,
        0xFF45564E, 0xFF45564E, 0xFF45564E, 0xFF465653, 0xFF455552, 0xFF435350, 0xFF42534D, 0xFF41524C, 0xFF405149, 0xFF3F5048, 0xFF3D4E46, 0xFF44534E, 0xFF43524D, 0xFF44534E, 0xFF45544F, 0xFF43524D, 0xFF42514C, 0xFF42514C, 0xFF404F4A, 0xFF3E4F47,
        0xFF424F48, 0xFF404C4A, 0xFF415053, 0xFF3F5352, 0xFF354231, 0xFF3E390F, 0xFF5C460A, 0xFF5D4B0D, 0xFF604E10, 0xFF614D10, 0xFF614C0D, 0xFF654B0E, 0xFF694D0E, 0xFF6A4E0F, 0xFF694B0D, 0xFF634A07, 0xFF654C09, 0xFF644B08, 0xFF624A04, 0xFF654A05,
        0xFF684D06, 0xFF694C06, 0xFF654802, 0xFF684711, 0xFF685103, 0xFF5E4800, 0xFF917220, 0xFFCEA83B, 0xFFB78A2C, 0xFF6D4406, 0xFF6F4F12, 0xFF58530F, 0xFF5C4905, 0xFF755000, 0xFFD0A12F, 0xFFCF9D2C, 0xFF805300, 0xFF674A08, 0xFF63510F, 0xFF6D4F13,
        0xFF6E5711, 0xFF65510C, 0xFF694F12, 0xFF6F520C, 0xFF6B5700, 0xFF6F6A16, 0xFF727B3A, 0xFF0D0B0C, 0xFF010302, 0xFF141D1A, 0xFF2F3E3B, 0xFF32423F, 0xFF384847, 0xFF3C4A4A, 0xFF2C3A3A, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FEFFFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF5A6662, 0xFF455548, 0xFF050B09, 0xFF13131B, 0xFF34373C, 0xFF243127, 0xFF495934, 0xFF697135, 0xFF6B671D, 0xFF725C23, 0xFF715B22, 0xFF705A21, 0xFF6F5920, 0xFF6F5920,
        0xFF6D571E, 0xFF6B551C, 0xFF69531A, 0xFF755024, 0xFF72511B, 0xFF6C510E, 0xFF6B5411, 0xFF664D14, 0xFF6F541F, 0xFF6C4D16, 0xFF705111, 0xFF7E4C19, 0xFF9B7949, 0xFFA69567, 0xFFAAA273, 0xFF928960, 0xFF90856F, 0xFFE8E2E6, 0xFFFCFCFF, 0xFFF8FCEE,
        0xFFF7F9EB, 0xFFFAFCEE, 0xFFFAFBEB, 0xFFFEFFF2, 0xFFFAFFF2, 0xFFFBFFF8, 0xFFD8E2D9, 0xFF465E5E, 0xFF1D2A19, 0xFF302C13, 0xFF473119, 0xFF483108, 0xFF695A21, 0xFF968F63, 0xFFB5B3A7, 0xFF899182, 0xFFAAB2A5, 0xFF727C73, 0xFF424D47, 0xFF546060,
        0xFF515C62, 0xFF44515A, 0xFF55636E, 0xFF4A5C5E, 0xFF556469, 0xFF56636B, 0xFF475258, 0xFF7F8887, 0xFFAAB2A7, 0xFF98A08B, 0xFFBDC3A7, 0xFF8A9559, 0xFFA8A77B, 0xFFB3A372, 0xFF897224, 0xFFB19738, 0xFF664D00, 0xFF584608, 0xFF5B4E17, 0xFF7C5707,
        0xFFB9994C, 0xFF8A6D1D, 0xFF977013, 0xFFE1AD3F, 0xFFE9AD33, 0xFFE5AB2E, 0xFFE4B036, 0xFFB58D1E, 0xFF62450B, 0xFF705715, 0xFFD5AB3D, 0xFFD49B27, 0xFFE2AD39, 0xFFE2B035, 0xFFE6AB37, 0xFFD8A731, 0xFFC89D2A, 0xFF815816, 0xFF6C4B22, 0xFF5D4807,
        0xFF68551A, 0xFF544116, 0xFF5B4C0B, 0xFF5B4714, 0xFF5D480F, 0xFF584000, 0xFF694D03, 0xFFB09039, 0xFFC7A342, 0xFFC19A31, 0xFFC79F30, 0xFFC5992C, 0xFFC19B38, 0xFFB39441, 0xFFA58D45, 0xFFA88F4C, 0xFFAD8F46, 0xFF9A7620, 0xFF7D5500, 0xFFB48D30,
        0xFFAD882D, 0xFFAE8A34, 0xFF8F6C1B, 0xFF6D4D02, 0xFF654603, 0xFF694D0C, 0xFF86692D, 0xFFD7B35F, 0xFFBFA14B, 0xFF7B6710, 0xFF615200, 0xFF9A8B38, 0xFF7A6418, 0xFF86651E, 0xFFAC8542, 0xFFA9912D, 0xFFC7AE51, 0xFF977B27, 0xFFCCAD5B, 0xFF82620F,
        0xFF8B680C, 0xFFC2A138, 0xFFAD8C1B, 0xFFA6A370, 0xFF847E4C, 0xFF999361, 0xFF888250, 0xFF9D9564, 0xFF726A39, 0xFFA29867, 0xFF857B4A, 0xFFE0A73E, 0xFFCE952C, 0xFFC99027, 0xFFD69D32, 0xFFDEA53A, 0xFFD69E31, 0xFFCD9528, 0xFFCC9427, 0xFFD29F32,
        0xFFB98A22, 0xFFC89B3E, 0xFFB38B36, 0xFF835F11, 0xFF7F6214, 0xFF99802E, 0xFFB49E4B, 0xFFDAA136, 0xFFD9A035, 0xFFC99025, 0xFFC68D22, 0xFFBB8217, 0xFFB77E13, 0xFFCD9429, 0xFFCD9429, 0xFFCFA537, 0xFFD0A232, 0xFFD3A22B, 0xFFD6A326, 0xFFD9A327,
        0xFFDBA72F, 0xFFD9A633, 0xFFD19E31, 0xFFCEA124, 0xFFC99724, 0xFFCD972B, 0xFFD19C2C, 0xFFCB9A21, 0xFFC59E1D, 0xFFC6A72A, 0xFFC5AA35, 0xFF85712C, 0xFF4B4C3C, 0xFF3D514F, 0xFF42564A, 0xFF445350, 0xFF395046, 0xFF405946, 0xFF404E4F, 0xFF41504B,
        0xFF41504B, 0xFF42514C, 0xFF43524D, 0xFF44534E, 0xFF44534E, 0xFF41504B, 0xFF3F4E49, 0xFF3F4E47, 0xFF404F48, 0xFF415049, 0xFF42514A, 0xFF43524B, 0xFF43524B, 0xFF42514A, 0xFF42514A, 0xFF455650, 0xFF43544E, 0xFF40514B, 0xFF3F504A, 0xFF41524C,
        0xFF43544E, 0xFF455650, 0xFF465751, 0xFF425252, 0xFF415151, 0xFF405050, 0xFF405050, 0xFF405050, 0xFF405050, 0xFF415151, 0xFF425252, 0xFF3F4E47, 0xFF415049, 0xFF42514A, 0xFF43524D, 0xFF41504B, 0xFF404F4C, 0xFF404F4C, 0xFF404F4C, 0xFF3D4C51,
        0xFF404C48, 0xFF414E44, 0xFF3C4C49, 0xFF394C52, 0xFF36413D, 0xFF3D3717, 0xFF5D470D, 0xFF624909, 0xFF634A0A, 0xFF644B0B, 0xFF664D0B, 0xFF664D0B, 0xFF664D0B, 0xFF654C09, 0xFF654C09, 0xFF654A07, 0xFF664B08, 0xFF674C09, 0xFF684D0A, 0xFF684D0A,
        0xFF684D0A, 0xFF684D0A, 0xFF674C09, 0xFF695000, 0xFF614600, 0xFF9D782A, 0xFFB98E49, 0xFF976C1D, 0xFFB18D2D, 0xFF684800, 0xFF66460B, 0xFF584600, 0xFF7C6203, 0xFFD9B03A, 0xFFEFBC3D, 0xFFE6AE3D, 0xFFCB9940, 0xFF775108, 0xFF684D0A, 0xFF6D4F0F,
        0xFF685209, 0xFF67530E, 0xFF6A5013, 0xFF6D500A, 0xFF6D5902, 0xFF6D6816, 0xFF6D7637, 0xFF100E0F, 0xFF010302, 0xFF17201D, 0xFF2E3D3A, 0xFF31413E, 0xFF3A4A49, 0xFF394747, 0xFF2C3A3A, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FEFFFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF586460, 0xFF425245, 0xFF080E0C, 0xFF16161E, 0xFF34373C, 0xFF222F25, 0xFF465631, 0xFF646C30, 0xFF656117, 0xFF735D24, 0xFF725C23, 0xFF705A21, 0xFF6F5920, 0xFF6F5920,
        0xFF6D571E, 0xFF6A541B, 0xFF685219, 0xFF714F22, 0xFF73541D, 0xFF6D5411, 0xFF6E5714, 0xFF695017, 0xFF6C511C, 0xFF684910, 0xFF775817, 0xFFC4944A, 0xFFE0C373, 0xFFCFC379, 0xFFA59F6B, 0xFF877C68, 0xFFA69895, 0xFFFEFAEF, 0xFFF5FDE5, 0xFFF9FFF5,
        0xFFF4F7EC, 0xFFFEFFF6, 0xFFFCFFF4, 0xFFF9FCF1, 0xFFFCFFF8, 0xFFD0D7D0, 0xFF48524A, 0xFF12250F, 0xFF212723, 0xFF433A31, 0xFF40310A, 0xFF685723, 0xFF897B58, 0xFFC0BBA5, 0xFF92967F, 0xFF9FA79A, 0xFF768075, 0xFF4F5950, 0xFF46514B, 0xFF465250,
        0xFF4C5A5B, 0xFF56656A, 0xFF4B5A5F, 0xFF536567, 0xFF46585A, 0xFF4F5E63, 0xFF5A656B, 0xFF4F595A, 0xFF768078, 0xFFAEB6A7, 0xFF9DA590, 0xFFBFC1B4, 0xFFB6B980, 0xFF9D9743, 0xFFA5924D, 0xFF664A0B, 0xFF684E01, 0xFF6A5504, 0xFF5E490A, 0xFF6C4C01,
        0xFF84681F, 0xFFAB9144, 0xFF967319, 0xFFB27F14, 0xFFEBAF37, 0xFFE0A62B, 0xFFEAB43A, 0xFFB68E1F, 0xFF62450B, 0xFF715816, 0xFFD6AC3E, 0xFFD49B27, 0xFFE2AD39, 0xFFE2B035, 0xFFE6AB37, 0xFFD5A02C, 0xFFC49627, 0xFF8D6913, 0xFF5B4300, 0xFF5D4E00,
        0xFF594E00, 0xFF605012, 0xFF624E11, 0xFF646019, 0xFF7F792F, 0xFF827B2D, 0xFF867B29, 0xFFB2A44C, 0xFFBBAA4E, 0xFFBAA748, 0xFFC0AC4B, 0xFFC0A74B, 0xFFBDA953, 0xFFAFA359, 0xFFA59D5C, 0xFFA7A262, 0xFFADA260, 0xFFA29046, 0xFF8E7729, 0xFFBFAC4E,
        0xFFB9A84C, 0xFFBAAA52, 0xFF9E8F3C, 0xFF827727, 0xFF81782B, 0xFF8E863D, 0xFFAEA75F, 0xFFB7A64E, 0xFFA79C43, 0xFF837F26, 0xFF787B22, 0xFFAAAC58, 0xFF969140, 0xFF9F9146, 0xFFAB974E, 0xFF79711C, 0xFFA09847, 0xFFA2964C, 0xFFCEC17B, 0xFF8A7C35,
        0xFF8B7C2B, 0xFFBCAD52, 0xFFC5B754, 0xFFB4B965, 0xFFA0A551, 0xFFA5A955, 0xFF9B9F4B, 0xFF999B48, 0xFF868835, 0xFFAFB15E, 0xFFABAA58, 0xFFBEB058, 0xFFBBAD54, 0xFFBBAD54, 0xFFBEB057, 0xFFBEB155, 0xFFB9AC50, 0xFFB7AA4C, 0xFFB9AC4E, 0xFFBFA73D,
        0xFFAF9832, 0xFFC6AF52, 0xFFC3AF59, 0xFFAB9A48, 0xFFAA9E4C, 0xFFAFA752, 0xFFBAB35B, 0xFFB9AC50, 0xFFB9AC50, 0xFFB0A347, 0xFFB5A84C, 0xFFB4A74B, 0xFFB3A64A, 0xFFBEB155, 0xFFB3A64A, 0xFF9D8227, 0xFF987716, 0xFFBC962B, 0xFFE0B13D, 0xFFD6A228,
        0xFFD39B20, 0xFFE0A52D, 0xFFDEA32D, 0xFFC79826, 0xFFCF9C33, 0xFFD19B3B, 0xFFD09837, 0xFFCE9B32, 0xFFBF922B, 0xFFA1781C, 0xFF85600F, 0xFF614E0C, 0xFF404436, 0xFF3D5253, 0xFF3D5148, 0xFF414F4F, 0xFF394E45, 0xFF3D5442, 0xFF434D4E, 0xFF45544F,
        0xFF44534E, 0xFF44534E, 0xFF45544F, 0xFF465550, 0xFF465550, 0xFF44534E, 0xFF43524D, 0xFF44534C, 0xFF44534C, 0xFF45544D, 0xFF45544D, 0xFF46554E, 0xFF46554E, 0xFF45544D, 0xFF45544D, 0xFF42534D, 0xFF40514B, 0xFF3F504A, 0xFF40514B, 0xFF41524C,
        0xFF43544E, 0xFF43544E, 0xFF43544E, 0xFF435352, 0xFF425251, 0xFF425251, 0xFF425251, 0xFF425251, 0xFF425251, 0xFF425251, 0xFF435352, 0xFF42514A, 0xFF43524B, 0xFF44534C, 0xFF43524D, 0xFF42514C, 0xFF41504D, 0xFF41504D, 0xFF42514E, 0xFF3F4E53,
        0xFF424E4A, 0xFF435046, 0xFF3E4E4B, 0xFF3C4F55, 0xFF38433F, 0xFF3E3818, 0xFF5E480E, 0xFF624909, 0xFF634A0A, 0xFF644B0B, 0xFF664D0B, 0xFF664D0B, 0xFF664D0B, 0xFF654C09, 0xFF654C09, 0xFF674C09, 0xFF674C09, 0xFF684D0A, 0xFF694E0B, 0xFF694E0B,
        0xFF694E0B, 0xFF684D0A, 0xFF684D0A, 0xFF654A07, 0xFF896C20, 0xFFAB873B, 0xFF896215, 0xFF6B4400, 0xFFC6A732, 0xFFB59A27, 0xFFA58B2E, 0xFFA8892E, 0xFFCCA838, 0xFFE4B531, 0xFFC58D04, 0xFFEAB035, 0xFFB17C14, 0xFFC8A041, 0xFF7C5D02, 0xFF6B4D0D,
        0xFF675108, 0xFF66520D, 0xFF694F12, 0xFF6B4E08, 0xFF6E5A03, 0xFF736E1C, 0xFF767F40, 0xFF100E0F, 0xFF000201, 0xFF141D1A, 0xFF293835, 0xFF2D3D3A, 0xFF384847, 0xFF3A4848, 0xFF2E3C3C, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FEFFFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF54605C, 0xFF455548, 0xFF080E0C, 0xFF111119, 0xFF313439, 0xFF2B382E, 0xFF576742, 0xFF6E763A, 0xFF635F15, 0xFF745B22, 0xFF735A21, 0xFF71581F, 0xFF71581F, 0xFF71581F,
        0xFF70571E, 0xFF6D541B, 0xFF6B5219, 0xFF6F4D1F, 0xFF73561E, 0xFF6D5412, 0xFF6D5515, 0xFF6A5119, 0xFF6B4D19, 0xFF6A4A0F, 0xFF8F6E29, 0xFFDEB34B, 0xFFB89B3F, 0xFF958A48, 0xFF80795D, 0xFF5F5355, 0xFF92858C, 0xFFFFFFF0, 0xFFFCFFDF, 0xFFF1F8F1,
        0xFFFBFFFB, 0xFFF1F8F1, 0xFFF2F8F4, 0xFFFBFFFD, 0xFFBEC4C0, 0xFF555B57, 0xFF191F1B, 0xFF232E0E, 0xFF2C2C22, 0xFF3B3127, 0xFF5F522F, 0xFF827749, 0xFFC3BEA1, 0xFF939587, 0xFF9AA595, 0xFF7C867E, 0xFF49544C, 0xFF39443E, 0xFF4F5C55, 0xFF495853,
        0xFF3B4B48, 0xFF435654, 0xFF495C5A, 0xFF405453, 0xFF506465, 0xFF546668, 0xFF49575A, 0xFF4A5557, 0xFF56605F, 0xFF79837B, 0xFFB1B9AE, 0xFF97998E, 0xFFB8B99A, 0xFFA6A46B, 0xFF686015, 0xFF7C6A20, 0xFFAE9350, 0xFFB09249, 0xFFAD8D38, 0xFF6C510C,
        0xFF584400, 0xFF7C661D, 0xFFA78732, 0xFFC99831, 0xFFE7AA37, 0xFFEFB33B, 0xFFE0A831, 0xFFB78F20, 0xFF63460C, 0xFF715816, 0xFFD6AC3E, 0xFFD59C28, 0xFFE3AE3A, 0xFFE2B035, 0xFFE6AB37, 0xFFE0A629, 0xFFC49231, 0xFF846312, 0xFF726217, 0xFF9B9352,
        0xFF888037, 0xFF675A0C, 0xFF624D0E, 0xFF8C8E78, 0xFFBBBAA5, 0xFFC0BFAB, 0xFFBAB9A5, 0xFFCCC9B6, 0xFFCBC8B7, 0xFFD2CCBC, 0xFFD5CFBF, 0xFFD9D1AA, 0xFFD4D0AB, 0xFFCACEAD, 0xFFC3CCAD, 0xFFC5D0B2, 0xFFCBD1B5, 0xFFC8C7A9, 0xFFC0BB9D, 0xFFD2CDB7,
        0xFFCFCDB8, 0xFFD6D4BF, 0xFFC7C6B2, 0xFFB7BAA7, 0xFFB8BCAB, 0xFFC0C4B5, 0xFFD4DBCB, 0xFFC7C1AB, 0xFFC6C4AD, 0xFFC1C6AF, 0xFFB6C0A8, 0xFFC3CCB7, 0xFFB2B6A5, 0xFFC8C4B8, 0xFFCAC1B8, 0xFFBCC0AF, 0xFFB6B8AB, 0xFFB8B9B3, 0xFFD7D6D2, 0xFFC0BFBA,
        0xFFC5C3B7, 0xFFCAC9B4, 0xFFCECDB1, 0xFFC5C8B5, 0xFFCBCEBB, 0xFFCBCCBA, 0xFFD7D8C6, 0xFFC6C5B3, 0xFFC2C1AF, 0xFFCFCCBB, 0xFFCFCCBB, 0xFFBFC7BC, 0xFFC7CFC4, 0xFFCAD2C5, 0xFFC4CCBF, 0xFFC0C8B9, 0xFFC2CABB, 0xFFC4CCBD, 0xFFC3CCBB, 0xFFD2CDAF,
        0xFFC7C2A5, 0xFFD3CEB8, 0xFFD4CFBC, 0xFFCBC8B7, 0xFFCECBBA, 0xFFC8C7B3, 0xFFC9C8B3, 0xFFC5CDC0, 0xFFC9D1C4, 0xFFC4CCBF, 0xFFCAD2C5, 0xFFCBD3C6, 0xFFCAD2C5, 0xFFCCD4C7, 0xFFBAC2B5, 0xFF514803, 0xFF5C4C00, 0xFF81680C, 0xFFB9942A, 0xFFDFAD3A,
        0xFFE1A62E, 0xFFD99B20, 0xFFDC9A20, 0xFFC89B26, 0xFFCC9C30, 0xFFCF9B36, 0xFFCD9832, 0xFFD19E35, 0xFFCF9D3C, 0xFFA77627, 0xFF714101, 0xFF413100, 0xFF3B403A, 0xFF475E64, 0xFF3F524C, 0xFF414C4E, 0xFF3D4E48, 0xFF405141, 0xFF464A4B, 0xFF404F4A,
        0xFF3F4E49, 0xFF3E4D48, 0xFF3F4E49, 0xFF404F4A, 0xFF404F4A, 0xFF3E4D48, 0xFF3D4C47, 0xFF404F48, 0xFF404F48, 0xFF404F48, 0xFF404F48, 0xFF404F48, 0xFF404F48, 0xFF415049, 0xFF415049, 0xFF3E4F49, 0xFF3E4F49, 0xFF3E4F49, 0xFF40514B, 0xFF42534D,
        0xFF42534D, 0xFF40514B, 0xFF3F504A, 0xFF40514B, 0xFF41524C, 0xFF41524C, 0xFF41524C, 0xFF41524C, 0xFF41524C, 0xFF41524C, 0xFF40514B, 0xFF43524B, 0xFF44534C, 0xFF44534C, 0xFF42514C, 0xFF404F4A, 0xFF3F4E4B, 0xFF404F4C, 0xFF41504D, 0xFF3D4C51,
        0xFF404C48, 0xFF424F45, 0xFF3E4E4B, 0xFF3B4E54, 0xFF37423E, 0xFF3C3616, 0xFF5A440A, 0xFF624909, 0xFF634A0A, 0xFF654C0C, 0xFF664D0B, 0xFF664D0B, 0xFF664D0B, 0xFF654C09, 0xFF654C09, 0xFF694E0B, 0xFF694E0B, 0xFF6A4F0C, 0xFF6A4F0C, 0xFF6A4F0C,
        0xFF694E0B, 0xFF684D0A, 0xFF684D0A, 0xFF5F420A, 0xFF745711, 0xFF76560D, 0xFF613C00, 0xFF6F4D00, 0xFFDABC4C, 0xFFCAB041, 0xFF967C1F, 0xFF95761C, 0xFFBB9530, 0xFFEAC04A, 0xFFEAB943, 0xFFD09D34, 0xFF7C5100, 0xFF9F8025, 0xFFAE9538, 0xFF6B4D0D,
        0xFF665007, 0xFF634F0A, 0xFF674D10, 0xFF6B4E08, 0xFF6E5A03, 0xFF726D1B, 0xFF747D3E, 0xFF0E0C0D, 0xFF000100, 0xFF131C19, 0xFF283734, 0xFF2C3C39, 0xFF374746, 0xFF394747, 0xFF2D3B3B, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FEFFFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF515D59, 0xFF445447, 0xFF090F0D, 0xFF0F0F17, 0xFF2D3035, 0xFF2E3B31, 0xFF5F6F4A, 0xFF727A3E, 0xFF635F15, 0xFF71581F, 0xFF70571E, 0xFF70571E, 0xFF70571E, 0xFF71581F,
        0xFF71581F, 0xFF6F561D, 0xFF6D541B, 0xFF6C4C1B, 0xFF72551B, 0xFF6A5310, 0xFF685012, 0xFF684F17, 0xFF694B15, 0xFF704E0F, 0xFFAA883E, 0xFFD7AD35, 0xFF725205, 0xFF5B4B31, 0xFF7A7174, 0xFF595055, 0xFF776D6B, 0xFFE9E4E1, 0xFFFFFFFF, 0xFFF8FEFA,
        0xFFF8FEFC, 0xFFF9FFFF, 0xFFF8FFFF, 0xFFC1CBCA, 0xFF4F5855, 0xFF171C18, 0xFF2A2C27, 0xFF34321B, 0xFF272403, 0xFF2F280C, 0xFF6A6152, 0xFFC1BCA6, 0xFF8B916D, 0xFFA0AC96, 0xFF748189, 0xFF3B4743, 0xFF495551, 0xFF4D5C57, 0xFF475852, 0xFF41544E,
        0xFF42554F, 0xFF445952, 0xFF41564F, 0xFF415959, 0xFF475C5D, 0xFF4B5F60, 0xFF4D5F61, 0xFF586669, 0xFF4C5759, 0xFF4D5758, 0xFF838C8B, 0xFFAEB29B, 0xFF999A95, 0xFFC3C1B4, 0xFF97965E, 0xFFB0AA56, 0xFFB8A354, 0xFFF0CE86, 0xFFBC9343, 0xFFD2BB77,
        0xFF7E6B27, 0xFF5B4803, 0xFF91721F, 0xFFDDAC46, 0xFFE8AC3A, 0xFFE1A42F, 0xFFE4AB38, 0xFFB89021, 0xFF64470D, 0xFF725917, 0xFFD7AD3F, 0xFFD59C28, 0xFFE3AE3A, 0xFFE2B035, 0xFFE6AB37, 0xFFE4A91D, 0xFFC79340, 0xFF76551F, 0xFFB4A57E, 0xFFFFFEEF,
        0xFFE6DFB3, 0xFF6F6414, 0xFF624F0A, 0xFF788057, 0xFFA1A982, 0xFF9FA782, 0xFF98A27F, 0xFFA0AB89, 0xFF9BA686, 0xFFA1AE90, 0xFFA1AE90, 0xFFA8AF8D, 0xFFA4AE8B, 0xFF9DAD88, 0xFF9AAF88, 0xFF9AB08A, 0xFF9EAF8D, 0xFFA0AD8F, 0xFFA1AB90, 0xFFA0AE8B,
        0xFF9DAB88, 0xFFA2B08D, 0xFF9DAD89, 0xFF98AA84, 0xFF97A983, 0xFF8DA27B, 0xFF90A57E, 0xFF9BA37E, 0xFF949E79, 0xFF91A17A, 0xFF8D9F79, 0xFF91A27E, 0xFF889874, 0xFF9AA384, 0xFF92997A, 0xFF9DAF85, 0xFF869870, 0xFF889975, 0xFF8FA07E, 0xFF91A17D,
        0xFF91A17A, 0xFF8A9B6E, 0xFF8D9F6D, 0xFF8D9A7C, 0xFF99A486, 0xFF929D7F, 0xFF9FAA8C, 0xFF97A083, 0xFF9AA386, 0xFF999F83, 0xFF969C80, 0xFF93A17E, 0xFF97A582, 0xFF96A481, 0xFF919F7B, 0xFF92A07C, 0xFF98A780, 0xFF98A780, 0xFF93A27B, 0xFF9AAA85,
        0xFF96A682, 0xFF9AA885, 0xFF95A380, 0xFF96A17F, 0xFF9DA784, 0xFF9BA580, 0xFFA0AA85, 0xFF9CAA86, 0xFFA1AF8B, 0xFF9FAD89, 0xFF9FAD89, 0xFF9BA985, 0xFF9BA985, 0xFF9DAB87, 0xFF8C9A76, 0xFF4E4D17, 0xFF5A5214, 0xFF5F4E00, 0xFF7F6005, 0xFFBE952C,
        0xFFE1AF40, 0xFFDBA332, 0xFFD49B28, 0xFFCAA324, 0xFFC59B23, 0xFFCC9F2A, 0xFFCEA026, 0xFFCE9E20, 0xFFDCAB32, 0xFFD19D37, 0xFFA67018, 0xFF473801, 0xFF3D433F, 0xFF455C64, 0xFF2C3F3B, 0xFF263032, 0xFF31403B, 0xFF435140, 0xFF4A4B4D, 0xFF41504B,
        0xFF404F4A, 0xFF404F4A, 0xFF41504B, 0xFF41504B, 0xFF41504B, 0xFF3E4D48, 0xFF3D4C47, 0xFF415049, 0xFF415049, 0xFF404F48, 0xFF404F48, 0xFF415049, 0xFF42514A, 0xFF44534C, 0xFF45544D, 0xFF43544E, 0xFF43544E, 0xFF44554F, 0xFF455650, 0xFF475852,
        0xFF475852, 0xFF465751, 0xFF44554F, 0xFF44544A, 0xFF45554B, 0xFF46564C, 0xFF47574D, 0xFF47574D, 0xFF46564C, 0xFF45554B, 0xFF44544A, 0xFF44534C, 0xFF44534C, 0xFF43524B, 0xFF41504B, 0xFF3F4E49, 0xFF3E4D4A, 0xFF3F4E4B, 0xFF404F4C, 0xFF3F4E53,
        0xFF424E4A, 0xFF445147, 0xFF41514E, 0xFF3F5258, 0xFF3A4541, 0xFF3E3818, 0xFF5B450B, 0xFF624909, 0xFF634A0A, 0xFF654C0C, 0xFF664D0B, 0xFF674E0C, 0xFF664D0B, 0xFF664D0A, 0xFF654C09, 0xFF694E0B, 0xFF694E0B, 0xFF694E0B, 0xFF694E0B, 0xFF694E0B,
        0xFF684D0A, 0xFF684D0A, 0xFF674C09, 0xFF6E5119, 0xFF694C06, 0xFF624600, 0xFF644505, 0xFF6F4E0B, 0xFF9C7D2D, 0xFF88681D, 0xFF604007, 0xFF584000, 0xFF644A00, 0xFF8F7320, 0xFFA28333, 0xFF765714, 0xFF644C0C, 0xFF554400, 0xFF786F22, 0xFF6E5010,
        0xFF665007, 0xFF624E09, 0xFF664C0F, 0xFF6C4F09, 0xFF6E5A03, 0xFF6D6816, 0xFF6B7435, 0xFF0C0A0B, 0xFF000100, 0xFF151E1B, 0xFF2C3B38, 0xFF30403D, 0xFF374746, 0xFF364444, 0xFF283636, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FEFFFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF4F5B57, 0xFF3D4D40, 0xFF0B110F, 0xFF15151D, 0xFF2C2F34, 0xFF243127, 0xFF50603B, 0xFF697135, 0xFF635F15, 0xFF72571F, 0xFF71561E, 0xFF70551D, 0xFF70551D, 0xFF71561E,
        0xFF70551D, 0xFF6E531B, 0xFF6C5119, 0xFF684C1A, 0xFF6F551A, 0xFF6A5212, 0xFF674F13, 0xFF6A4F1A, 0xFF694A14, 0xFF714E0C, 0xFFB99645, 0xFFD2A435, 0xFF72510A, 0xFF42341A, 0xFF494641, 0xFF39362D, 0xFF4C453B, 0xFFA6A0A2, 0xFFFBF7FF, 0xFFFEFFFA,
        0xFFF0F6F2, 0xFFE5F0EC, 0xFFA6B2B0, 0xFF3D4746, 0xFF171E17, 0xFF2A2B23, 0xFF323024, 0xFF3A3112, 0xFF352F0B, 0xFF5C563C, 0xFFB7B4AB, 0xFF888C7E, 0xFFA0AD91, 0xFF818F80, 0xFF3F4D58, 0xFF515F5F, 0xFF455353, 0xFF3C4C4B, 0xFF3A4D4B, 0xFF3C504E,
        0xFF3C524F, 0xFF4B6460, 0xFF68817D, 0xFF7E979C, 0xFF738C91, 0xFF455D61, 0xFF4C5F63, 0xFF556769, 0xFF4C5A5D, 0xFF586369, 0xFF444F55, 0xFF898A8E, 0xFFA1A79B, 0xFF9BA290, 0xFFCBCDB8, 0xFFA7A471, 0xFFB0A54C, 0xFFC5A955, 0xFF9D753A, 0xFFBFA55A,
        0xFF937F36, 0xFF5B4800, 0xFF7D5D06, 0xFFDEAD46, 0xFFECB03E, 0xFFE4A734, 0xFFE1AA36, 0xFFB89021, 0xFF64470D, 0xFF725917, 0xFFD7AD3F, 0xFFD59C28, 0xFFE2AD39, 0xFFE2B035, 0xFFE6AB37, 0xFFDEA31B, 0xFFC69141, 0xFF7A5923, 0xFFB1A37C, 0xFFFDF7E9,
        0xFFD9D6AD, 0xFF625C12, 0xFF544409, 0xFFBABAB8, 0xFFD5D5D3, 0xFFCBCDCA, 0xFFCBD0CC, 0xFFCED4D0, 0xFFC5CEC9, 0xFFC9D4CC, 0xFFC6D3CA, 0xFFCFD3D6, 0xFFCCD2D0, 0xFFC9D4CE, 0xFFC8D5CC, 0xFFC7D4CB, 0xFFC6D2CE, 0xFFC8D2D3, 0xFFCDD5D8, 0xFFC6D1CD,
        0xFFC3CECA, 0xFFC5D0CC, 0xFFC5D0CA, 0xFFC8D5CC, 0xFFCCD9D0, 0xFFC7D4CA, 0xFFC6D3C9, 0xFFC8CDC7, 0xFFC3CAC3, 0xFFC3CDC5, 0xFFC5D0CA, 0xFFC8D3CD, 0xFFC0C9C4, 0xFFC5CBC7, 0xFFC1C6C2, 0xFFBDC9BF, 0xFFBEC9C1, 0xFFCBD6D0, 0xFFC4CFCB, 0xFFC3D0C9,
        0xFFB9C6BD, 0xFFBFCCC2, 0xFFC7D5C8, 0xFFC2D3C3, 0xFFC6D4C5, 0xFFC0CEBF, 0xFFBDCBBC, 0xFFC7D3C5, 0xFFC5D1C3, 0xFFC7D2C4, 0xFFC3CEC0, 0xFFC9CAC4, 0xFFC6C7C1, 0xFFC3C4BC, 0xFFC2C3BB, 0xFFC6C8BD, 0xFFCACCC1, 0xFFC9CBC0, 0xFFC5C7BA, 0xFFB7C5C5,
        0xFFBCCBC8, 0xFFC1CDC9, 0xFFBEC9C3, 0xFFC4CBC4, 0xFFC7CCC6, 0xFFC3C8C2, 0xFFCACCC7, 0xFFC8C9C1, 0xFFCCCDC5, 0xFFCBCCC4, 0xFFCACBC3, 0xFFC8C9C1, 0xFFCCCDC5, 0xFFCECFC7, 0xFFC0C1B9, 0xFF524E21, 0xFF544A15, 0xFF584906, 0xFF604900, 0xFF7E5F04,
        0xFFB59332, 0xFFD0AD49, 0xFFC39D3A, 0xFFC7A430, 0xFFC5A131, 0xFFCAA533, 0xFFCCA62B, 0xFFCCA51C, 0xFFD9AB21, 0xFFDCA92A, 0xFFD09925, 0xFF655218, 0xFF41433E, 0xFF3A4E55, 0xFF172724, 0xFF081214, 0xFF23322D, 0xFF435444, 0xFF4E4F53, 0xFF41504B,
        0xFF41504B, 0xFF42514C, 0xFF43524D, 0xFF43524D, 0xFF42514C, 0xFF3F4E49, 0xFF3C4B46, 0xFF415049, 0xFF415049, 0xFF415049, 0xFF415049, 0xFF43524B, 0xFF44534C, 0xFF46554E, 0xFF47564F, 0xFF43544E, 0xFF42534D, 0xFF41524C, 0xFF42534D, 0xFF43544E,
        0xFF455650, 0xFF44554F, 0xFF44554F, 0xFF425248, 0xFF435349, 0xFF44544A, 0xFF45554B, 0xFF45554B, 0xFF44544A, 0xFF435349, 0xFF425248, 0xFF45544D, 0xFF45544D, 0xFF44534C, 0xFF42514C, 0xFF404F4A, 0xFF3F4E4B, 0xFF404F4C, 0xFF42514E, 0xFF3D4C51,
        0xFF3F4B47, 0xFF414E44, 0xFF3F4F4C, 0xFF3E5157, 0xFF394440, 0xFF3C3616, 0xFF584208, 0xFF634A0A, 0xFF644B0B, 0xFF654C0C, 0xFF664D0B, 0xFF674E0C, 0xFF674E0C, 0xFF664D0A, 0xFF654C09, 0xFF694E0B, 0xFF684D0A, 0xFF684D0A, 0xFF674C09, 0xFF674C09,
        0xFF674C09, 0xFF674C09, 0xFF674C09, 0xFF5E400A, 0xFF5F4500, 0xFF6D5606, 0xFF6E5610, 0xFF604500, 0xFF694E00, 0xFF715404, 0xFF705110, 0xFF705B18, 0xFF654F04, 0xFF685302, 0xFF5D4A00, 0xFF6B5615, 0xFF6B5718, 0xFF74661F, 0xFF564A00, 0xFF6B4D0D,
        0xFF665007, 0xFF64500B, 0xFF674D10, 0xFF6A4D07, 0xFF6C5801, 0xFF716C1A, 0xFF747D3E, 0xFF0F0D0E, 0xFF010302, 0xFF18211E, 0xFF2F3E3B, 0xFF31413E, 0xFF374746, 0xFF354343, 0xFF263434, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FEFFFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF4F5B57, 0xFF3D4D40, 0xFF0A100E, 0xFF12121A, 0xFF2B2E33, 0xFF263329, 0xFF53633E, 0xFF6A7236, 0xFF656117, 0xFF745A1F, 0xFF73591E, 0xFF71571C, 0xFF70561B, 0xFF6F551A,
        0xFF6D5318, 0xFF6B5116, 0xFF684E13, 0xFF684F19, 0xFF6E561A, 0xFF6B5617, 0xFF695319, 0xFF6D511F, 0xFF6B4A14, 0xFF6F4A04, 0xFFB8943E, 0xFFCF9D3C, 0xFF7E5F0D, 0xFF393300, 0xFF424623, 0xFF757865, 0xFF726E63, 0xFF49453C, 0xFF757269, 0xFFA5A698,
        0xFFA5A89D, 0xFF79837B, 0xFF333E38, 0xFF111B13, 0xFF292B20, 0xFF37321F, 0xFF3C311B, 0xFF5B5322, 0xFF817964, 0xFFBAB7B0, 0xFF8C9381, 0xFFAAB8A1, 0xFF7D8A83, 0xFF3C474B, 0xFF4C5856, 0xFF455555, 0xFF6A7C7E, 0xFF5C6E70, 0xFF3D5054, 0xFF667B80,
        0xFF9BB2B8, 0xFFABC4C9, 0xFFBCD5DC, 0xFFABC5D6, 0xFFB6D0DD, 0xFF7D969D, 0xFF465E62, 0xFF435758, 0xFF4D5F61, 0xFF57666B, 0xFF505D65, 0xFF424858, 0xFF838C87, 0xFFA7B39D, 0xFF8E967E, 0xFFB8B89E, 0xFFC3B78F, 0xFFC0AC6F, 0xFFB3994F, 0xFFBE9C43,
        0xFFBB9F4B, 0xFF8F7520, 0xFF916E10, 0xFFD5A235, 0xFFE2A731, 0xFFEBB03A, 0xFFDDA836, 0xFFB89021, 0xFF64470D, 0xFF725917, 0xFFD7AD3F, 0xFFD59C28, 0xFFE2AD39, 0xFFE1AF34, 0xFFE5AA36, 0xFFDEA62F, 0xFFBF9133, 0xFF7F6310, 0xFF61580B, 0xFF7B7B3D,
        0xFF6C6D31, 0xFF5A5720, 0xFF5B512E, 0xFF727473, 0xFF828483, 0xFF7B807C, 0xFF838882, 0xFF838A82, 0xFF7C867B, 0xFF7C8779, 0xFF7D897B, 0xFF838A7A, 0xFF818A77, 0xFF818A75, 0xFF828C74, 0xFF7E8A72, 0xFF7B8771, 0xFF7B8874, 0xFF7F8C78, 0xFF7E8A80,
        0xFF7F8B81, 0xFF7F8B81, 0xFF7B877D, 0xFF79837A, 0xFF7A8479, 0xFF798378, 0xFF798378, 0xFF767D75, 0xFF787F77, 0xFF787F77, 0xFF7A8179, 0xFF767D75, 0xFF6E766B, 0xFF6D756A, 0xFF757D72, 0xFF73787B, 0xFF787D80, 0xFF798183, 0xFF70787A, 0xFF788283,
        0xFF707A7B, 0xFF758082, 0xFF717C7E, 0xFF6C7D75, 0xFF6D7E76, 0xFF718079, 0xFF687770, 0xFF74837C, 0xFF6C7972, 0xFF6F7C75, 0xFF6C7972, 0xFF797F7B, 0xFF777D79, 0xFF767D76, 0xFF777E77, 0xFF787F77, 0xFF777E76, 0xFF787F77, 0xFF7A8277, 0xFF727D79,
        0xFF76817B, 0xFF768278, 0xFF768173, 0xFF7C8477, 0xFF7A8076, 0xFF737871, 0xFF757774, 0xFF7C837C, 0xFF7A817A, 0xFF7A817A, 0xFF7C837C, 0xFF7D847D, 0xFF828982, 0xFF7F867F, 0xFF727972, 0xFF534F2C, 0xFF595228, 0xFF5D5019, 0xFF5B4A04, 0xFF634E00,
        0xFF7D6712, 0xFF917B25, 0xFF917D27, 0xFF917319, 0xFF987926, 0xFF947421, 0xFF9E7C1B, 0xFFC19C29, 0xFFD6A92A, 0xFFD49E20, 0xFFD49921, 0xFF785F1D, 0xFF444235, 0xFF435555, 0xFF314137, 0xFF232D2E, 0xFF31443E, 0xFF435849, 0xFF454D50, 0xFF41504B,
        0xFF41504B, 0xFF41504B, 0xFF42514C, 0xFF42514C, 0xFF41504B, 0xFF3E4D48, 0xFF3B4A45, 0xFF3E4D46, 0xFF3F4E47, 0xFF3F4E47, 0xFF404F48, 0xFF415049, 0xFF43524B, 0xFF44534C, 0xFF45544D, 0xFF41524C, 0xFF3F504A, 0xFF3D4E48, 0xFF3D4E48, 0xFF3F504A,
        0xFF41524C, 0xFF42534D, 0xFF42534D, 0xFF445148, 0xFF445148, 0xFF445148, 0xFF455249, 0xFF455249, 0xFF445148, 0xFF445148, 0xFF445148, 0xFF44534C, 0xFF45544D, 0xFF45544D, 0xFF43524D, 0xFF41504B, 0xFF404F4C, 0xFF41504D, 0xFF42514E, 0xFF3C4B50,
        0xFF3E4A46, 0xFF404D43, 0xFF3F4F4C, 0xFF3E5157, 0xFF3A4541, 0xFF3D3717, 0xFF594309, 0xFF634A0A, 0xFF644B0B, 0xFF654C0C, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF664D0A, 0xFF664D0A, 0xFF684D0A, 0xFF684D0A, 0xFF674C09, 0xFF664B08, 0xFF664B08,
        0xFF664B08, 0xFF674C09, 0xFF674C09, 0xFF68491B, 0xFF6E560C, 0xFFAF9C40, 0xFFCDBC62, 0xFFC9B95A, 0xFFC6B444, 0xFFCDB940, 0xFFD0B947, 0xFFCFB554, 0xFFCAB349, 0xFFD4C050, 0xFFC6B146, 0xFFD8C161, 0xFFC8AE51, 0xFFD6B953, 0xFF9E8010, 0xFF654707,
        0xFF654F06, 0xFF67530E, 0xFF684E11, 0xFF674A04, 0xFF6C5801, 0xFF7A7523, 0xFF879051, 0xFF141213, 0xFF040605, 0xFF18211E, 0xFF2D3C39, 0xFF2E3E3B, 0xFF364645, 0xFF364444, 0xFF283636, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FEFFFF, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF505C58, 0xFF425245, 0xFF090F0D, 0xFF0D0D15, 0xFF2B2E33, 0xFF2F3C32, 0xFF5C6C47, 0xFF6E763A, 0xFF646016, 0xFF73561C, 0xFF72551B, 0xFF71541A, 0xFF705319, 0xFF705319,
        0xFF6F5218, 0xFF6D5016, 0xFF6B4E14, 0xFF675118, 0xFF6A5418, 0xFF6B5617, 0xFF69531A, 0xFF6D511F, 0xFF6D4C16, 0xFF6F4B01, 0xFFB69138, 0xFFCC9833, 0xFF745505, 0xFF635D2D, 0xFF949983, 0xFFDEE1D6, 0xFFDEDCCF, 0xFF666350, 0xFF41402B, 0xFF59543E,
        0xFF515240, 0xFF43493B, 0xFF374136, 0xFF2E3426, 0xFF34331F, 0xFF31270C, 0xFF433312, 0xFF8C8453, 0xFFB9B4A0, 0xFF8B8D8A, 0xFFAAB6A8, 0xFF697A68, 0xFF3D4C49, 0xFF50595E, 0xFF48514E, 0xFF445656, 0xFF738788, 0xFF798C92, 0xFF6D8188, 0xFF92A8B5,
        0xFFADC4D2, 0xFFA5BFD0, 0xFFB6CFE3, 0xFFA9C3DE, 0xFFADC8DD, 0xFFB5CFDC, 0xFF7C959A, 0xFF475C5D, 0xFF4A5E5F, 0xFF4E6062, 0xFF57666B, 0xFF4E5D5A, 0xFF454F58, 0xFF7D8786, 0xFFA2AC93, 0xFF868A73, 0xFFBFB8AE, 0xFFBBB090, 0xFFB5AD62, 0xFFC69D34,
        0xFFA88521, 0xFFB79635, 0xFFAB841B, 0xFFD7A431, 0xFFEBB136, 0xFFD7A029, 0xFFDFAD3A, 0xFFB89021, 0xFF64470D, 0xFF725917, 0xFFD6AC3E, 0xFFD49B27, 0xFFE1AC38, 0xFFE0AE33, 0xFFE4A935, 0xFFDEA93F, 0xFFBC9128, 0xFF856A11, 0xFF594C00, 0xFF4D4F00,
        0xFF494D10, 0xFF5F5E42, 0xFF696655, 0xFF39463C, 0xFF424F45, 0xFF404D43, 0xFF47544A, 0xFF455248, 0xFF47544A, 0xFF445147, 0xFF48554B, 0xFF485253, 0xFF485052, 0xFF485052, 0xFF495153, 0xFF485052, 0xFF454F4E, 0xFF44504C, 0xFF44534E, 0xFF3E4F3F,
        0xFF435444, 0xFF445545, 0xFF445245, 0xFF414E44, 0xFF424E44, 0xFF434E46, 0xFF404B43, 0xFF445449, 0xFF404D43, 0xFF38443A, 0xFF3E4A40, 0xFF404C40, 0xFF465246, 0xFF3F4D3E, 0xFF3F5040, 0xFF3C4542, 0xFF3F4845, 0xFF414C48, 0xFF39443E, 0xFF404C48,
        0xFF3B4A47, 0xFF40504F, 0xFF3F4F4F, 0xFF3D4C51, 0xFF3C4B50, 0xFF3C4B50, 0xFF3A474D, 0xFF3B484E, 0xFF3A454B, 0xFF3B464C, 0xFF404B51, 0xFF3C4B46, 0xFF3D4C45, 0xFF3E4D46, 0xFF3D4C45, 0xFF3B4B41, 0xFF3B4B41, 0xFF3D4D42, 0xFF3E4E43, 0xFF424E44,
        0xFF434F43, 0xFF424F3E, 0xFF43503C, 0xFF465341, 0xFF445042, 0xFF434E46, 0xFF46514D, 0xFF415147, 0xFF3D4D43, 0xFF405046, 0xFF425248, 0xFF405046, 0xFF425248, 0xFF405046, 0xFF39493F, 0xFF535641, 0xFF575638, 0xFF545023, 0xFF584E11, 0xFF60520B,
        0xFF5B4A00, 0xFF554400, 0xFF5B4A00, 0xFF614A00, 0xFF644C06, 0xFF5D4404, 0xFF654700, 0xFF926F13, 0xFFC5992C, 0xFFD7A22E, 0xFFD59825, 0xFF816116, 0xFF473F2A, 0xFF4D5A53, 0xFF4C5A4D, 0xFF46504F, 0xFF3F564E, 0xFF3A5546, 0xFF3D4C51, 0xFF45544F,
        0xFF43524D, 0xFF41504B, 0xFF404F4A, 0xFF41504B, 0xFF404F4A, 0xFF3F4E49, 0xFF3E4D48, 0xFF3F4E47, 0xFF404F48, 0xFF415049, 0xFF415049, 0xFF42514A, 0xFF42514A, 0xFF42514A, 0xFF42514A, 0xFF42534D, 0xFF41524C, 0xFF3F504A, 0xFF3F504A, 0xFF41524C,
        0xFF43544E, 0xFF43544E, 0xFF43544E, 0xFF46524E, 0xFF45514D, 0xFF45514D, 0xFF45514D, 0xFF45514D, 0xFF45514D, 0xFF45514D, 0xFF46524E, 0xFF3E4D46, 0xFF3F4E47, 0xFF404F48, 0xFF3F4E49, 0xFF3E4D48, 0xFF3D4C49, 0xFF3D4C49, 0xFF3E4D4A, 0xFF404F54,
        0xFF424E4A, 0xFF435046, 0xFF42524F, 0xFF42555B, 0xFF3F4A46, 0xFF433D1D, 0xFF5F490F, 0xFF634A0A, 0xFF644B0B, 0xFF664D0D, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF664D0A, 0xFF664D0A, 0xFF684D0A, 0xFF684D0A, 0xFF674C09, 0xFF674C09, 0xFF674C09,
        0xFF684D0A, 0xFF694E0B, 0xFF694E0B, 0xFF654715, 0xFF695401, 0xFFD5C860, 0xFFFFFE98, 0xFFFFFF96, 0xFFF3E76D, 0xFFEBDA56, 0xFFEFDA5A, 0xFFF5DE76, 0xFFFAE975, 0xFFFCF175, 0xFFF2E66E, 0xFFFDEB7D, 0xFFFFFD94, 0xFFF9D567, 0xFFA07701, 0xFF644606,
        0xFF634D04, 0xFF65510C, 0xFF684E11, 0xFF694C06, 0xFF6D5902, 0xFF797422, 0xFF838C4D, 0xFF131112, 0xFF030504, 0xFF18211E, 0xFF2D3C39, 0xFF2F3F3C, 0xFF364645, 0xFF364444, 0xFF283636, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FEFFFF, 0x00FFFDFE, 0x00FEFCFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF515D59, 0xFF405043, 0xFF0A100E, 0xFF101018, 0xFF2D3035, 0xFF2D3A30, 0xFF54643F, 0xFF656D31, 0xFF605C12, 0xFF6E5117, 0xFF6E5117, 0xFF6E5117, 0xFF6F5218, 0xFF705319,
        0xFF71541A, 0xFF705319, 0xFF6F5218, 0xFF69531A, 0xFF685216, 0xFF675316, 0xFF654F16, 0xFF6A4E1E, 0xFF6E4D17, 0xFF734D04, 0xFFBB9439, 0xFFD5A42E, 0xFF9E7C3D, 0xFFE7D9D8, 0xFFFCFDFF, 0xFFCCD0D1, 0xFFC1C0AC, 0xFF706D5A, 0xFF403B37, 0xFF4F4A2D,
        0xFF595740, 0xFF4D5140, 0xFF3D4635, 0xFF333726, 0xFF322E15, 0xFF4C3D1C, 0xFF9D8760, 0xFFB8AF8E, 0xFF979775, 0xFFA9B0A0, 0xFF727F85, 0xFF3A4C4E, 0xFF475846, 0xFF3D493D, 0xFF454753, 0xFF394D4C, 0xFF405455, 0xFF60757A, 0xFF6D838E, 0xFF758C9C,
        0xFF96AFC5, 0xFFABC3DD, 0xFFA9C3DE, 0xFFA8C2E3, 0xFFABC5E0, 0xFFAFC9DA, 0xFFADC6CB, 0xFF7C9494, 0xFF4B5F5E, 0xFF4C5E60, 0xFF516365, 0xFF52635B, 0xFF4C5D57, 0xFF4D585E, 0xFF818793, 0xFFB0B3AC, 0xFF959A72, 0xFFB7B88C, 0xFFBAB699, 0xFFCB9C2A,
        0xFFB88E22, 0xFFB28D23, 0xFFC59B2D, 0xFFEBB73F, 0xFFE2A82B, 0xFFE8B238, 0xFFD9AA36, 0xFFB89021, 0xFF64470D, 0xFF725917, 0xFFD6AC3E, 0xFFD49B27, 0xFFE1AC38, 0xFFE0AE33, 0xFFE4A935, 0xFFD9A83F, 0xFFBE9627, 0xFF8F6E29, 0xFF9E895C, 0xFF868044,
        0xFF706E47, 0xFF494843, 0xFF484A3C, 0xFF435D52, 0xFF465D55, 0xFF405751, 0xFF425654, 0xFF405254, 0xFF495B5F, 0xFF435259, 0xFF46545D, 0xFF44554F, 0xFF445350, 0xFF465252, 0xFF465153, 0xFF465252, 0xFF43544E, 0xFF42564B, 0xFF415848, 0xFF445950,
        0xFF465B52, 0xFF435650, 0xFF425551, 0xFF455554, 0xFF49575A, 0xFF4A575D, 0xFF455258, 0xFF3E5452, 0xFF425655, 0xFF435555, 0xFF475756, 0xFF3B4B48, 0xFF40534F, 0xFF3C514A, 0xFF3E554D, 0xFF435049, 0xFF435047, 0xFF49594E, 0xFF435549, 0xFF465A4F,
        0xFF445950, 0xFF415852, 0xFF445A57, 0xFF44554D, 0xFF45564E, 0xFF42534B, 0xFF4E5D56, 0xFF45544D, 0xFF4C5952, 0xFF424F48, 0xFF47544D, 0xFF44504E, 0xFF465250, 0xFF45514F, 0xFF424E4A, 0xFF434F4B, 0xFF46534C, 0xFF45524B, 0xFF424F48, 0xFF434F4D,
        0xFF44514A, 0xFF435348, 0xFF445447, 0xFF3F5145, 0xFF394C48, 0xFF3D5054, 0xFF43565D, 0xFF3D4945, 0xFF3C4844, 0xFF434F4B, 0xFF434F4B, 0xFF3C4844, 0xFF3D4945, 0xFF414D49, 0xFF44504C, 0xFF404C42, 0xFF3D4432, 0xFF63633F, 0xFF8C854F, 0xFF90833F,
        0xFF8E7E31, 0xFF8C7A28, 0xFF796713, 0xFF807019, 0xFF7B691F, 0xFF756021, 0xFF614805, 0xFF664600, 0xFFA87F15, 0xFFE0AB39, 0xFFDC9F2C, 0xFF8B6615, 0xFF44391D, 0xFF3D493D, 0xFF3D4A39, 0xFF3D4945, 0xFF314B42, 0xFF2B4A3B, 0xFF3F5258, 0xFF3F4E49,
        0xFF3C4B46, 0xFF384742, 0xFF364540, 0xFF364540, 0xFF374641, 0xFF384742, 0xFF384742, 0xFF384740, 0xFF394841, 0xFF394841, 0xFF3A4942, 0xFF394841, 0xFF384740, 0xFF37463F, 0xFF36453E, 0xFF384943, 0xFF364741, 0xFF354640, 0xFF364741, 0xFF384943,
        0xFF394A44, 0xFF394A44, 0xFF394A44, 0xFF3A4644, 0xFF394543, 0xFF384442, 0xFF384442, 0xFF384442, 0xFF384442, 0xFF394543, 0xFF3A4644, 0xFF36453E, 0xFF384740, 0xFF394841, 0xFF394843, 0xFF384742, 0xFF374643, 0xFF374643, 0xFF374643, 0xFF36454A,
        0xFF37433F, 0xFF38453B, 0xFF374744, 0xFF384B51, 0xFF36413D, 0xFF3A3414, 0xFF564006, 0xFF634A0A, 0xFF644B0B, 0xFF664D0D, 0xFF674E0C, 0xFF684F0D, 0xFF674E0C, 0xFF674E0B, 0xFF664D0A, 0xFF694E0B, 0xFF684D0A, 0xFF684D0A, 0xFF684D0A, 0xFF684D0A,
        0xFF694E0B, 0xFF6A4F0C, 0xFF6B500D, 0xFF6A4E0F, 0xFF635000, 0xFFD3C95A, 0xFFFEF492, 0xFFFFF79B, 0xFFFDED8B, 0xFFFDE87F, 0xFFFFEC88, 0xFFF5EB94, 0xFFFEFA97, 0xFFF2F486, 0xFFF1F386, 0xFFF6F08E, 0xFFFFEE95, 0xFFF6D77C, 0xFF9A7210, 0xFF674909,
        0xFF624C03, 0xFF614D08, 0xFF674D10, 0xFF6D500A, 0xFF6F5B04, 0xFF6F6A18, 0xFF6E7738, 0xFF0E0C0D, 0xFF000201, 0xFF18211E, 0xFF2F3E3B, 0xFF32423F, 0xFF384847, 0xFF344242, 0xFF253333, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FDFFFE, 0x00FFFEFF, 0x00FFFDFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF505C58, 0xFF3A4942, 0xFF070F04, 0xFF0F1110, 0xFF333542, 0xFF222E2A, 0xFF4E5F33, 0xFF66702B, 0xFF66611D, 0xFF6B5919, 0xFF695612, 0xFF66520D, 0xFF6A530F, 0xFF6B5411,
        0xFF675213, 0xFF665215, 0xFF655315, 0xFF6F5021, 0xFF6B5627, 0xFF645119, 0xFF6D5412, 0xFF665311, 0xFF64551A, 0xFF6C4D0A, 0xFFC38A33, 0xFFBF9F32, 0xFFC5AF88, 0xFFE9E4FA, 0xFFE5EAFF, 0xFFEBEFEE, 0xFFACAEA1, 0xFF4B5049, 0xFF384440, 0xFF39544F,
        0xFF40554E, 0xFF374140, 0xFF2C2836, 0xFF393600, 0xFF423512, 0xFF4F3E24, 0xFFCAB69E, 0xFF949B8B, 0xFFADB5A6, 0xFF7D887A, 0xFF424E44, 0xFF43524B, 0xFF465653, 0xFF374A48, 0xFF3B4F4E, 0xFF445350, 0xFF414F4F, 0xFF485A5A, 0xFF728589, 0xFF6A8189,
        0xFF6E8690, 0xFF9FB9C6, 0xFFABC7D5, 0xFFAFCADB, 0xFFACC9DB, 0xFFA5C4D6, 0xFFA9C9D8, 0xFFAECCD7, 0xFF819CA3, 0xFF40585A, 0xFF516564, 0xFF4D5B5C, 0xFF4C5A5B, 0xFF526262, 0xFF41504D, 0xFF818C86, 0xFFA5AB9F, 0xFF9C9D8B, 0xFFB5B39C, 0xFF8A8146,
        0xFF987E27, 0xFFD3A42E, 0xFFE7AC2C, 0xFFE9AD35, 0xFFE0A839, 0xFFDBA632, 0xFFE6B034, 0xFFB78D29, 0xFF65490A, 0xFF79611B, 0xFFD3A842, 0xFFD49A2C, 0xFFDEAB2E, 0xFFE2B22C, 0xFFE6AD3C, 0xFFDCA830, 0xFFC0952C, 0xFF998641, 0xFF9BA281, 0xFFA7B2A1,
        0xFFBCB9A8, 0xFF424238, 0xFF48555B, 0xFF475A58, 0xFF475A58, 0xFF475A58, 0xFF495956, 0xFF485855, 0xFF465751, 0xFF465550, 0xFF485752, 0xFF4A5654, 0xFF485452, 0xFF4D5957, 0xFF4C5A5A, 0xFF465454, 0xFF445453, 0xFF475757, 0xFF455555, 0xFF495959,
        0xFF495959, 0xFF485858, 0xFF475757, 0xFF465656, 0xFF455555, 0xFF455555, 0xFF455555, 0xFF4A5A59, 0xFF435352, 0xFF3F4F4E, 0xFF425251, 0xFF435352, 0xFF425251, 0xFF435352, 0xFF465655, 0xFF455554, 0xFF465655, 0xFF465655, 0xFF455554, 0xFF485857,
        0xFF4C5C5B, 0xFF4C5C5B, 0xFF495958, 0xFF455552, 0xFF445451, 0xFF42524F, 0xFF41514E, 0xFF41514E, 0xFF42524F, 0xFF435350, 0xFF445451, 0xFF425251, 0xFF40504F, 0xFF40504F, 0xFF40504F, 0xFF425251, 0xFF435352, 0xFF435352, 0xFF425251, 0xFF3C4C49,
        0xFF3D4D4A, 0xFF40504D, 0xFF41514E, 0xFF41514E, 0xFF3F4F4C, 0xFF3C4C49, 0xFF3B4B48, 0xFF3C4D47, 0xFF3E4F49, 0xFF40514B, 0xFF41524C, 0xFF40514B, 0xFF40514B, 0xFF41524C, 0xFF42534D, 0xFF394747, 0xFF27331F, 0xFF969C9A, 0xFFA3AC97, 0xFF8D9876,
        0xFFB0AF83, 0xFF9A8728, 0xFFA68117, 0xFFAC8921, 0xFFA68621, 0xFFA98F30, 0xFF80670D, 0xFF674800, 0xFFA77C1D, 0xFFD59E37, 0xFFDC9E31, 0xFFA5750F, 0xFF492A00, 0xFF524E4D, 0xFF1E2D30, 0xFF47594D, 0xFF283322, 0xFF545744, 0xFF272911, 0xFF464B44,
        0xFF292F25, 0xFF485041, 0xFF232C19, 0xFF394531, 0xFF303C28, 0xFF323E2A, 0xFF323F2D, 0xFF3D4133, 0xFF2B2F20, 0xFF343B29, 0xFF3B4431, 0xFF27331F, 0xFF3B4837, 0xFF222E20, 0xFF3F4D40, 0xFF1B2F23, 0xFF3F4B47, 0xFF272C2F, 0xFF40413C, 0xFF323522,
        0xFF3E472C, 0xFF263319, 0xFF3D4C39, 0xFF283227, 0xFF424C41, 0xFF273126, 0xFF394338, 0xFF2A3429, 0xFF333D32, 0xFF333D32, 0xFF2E382D, 0xFF393833, 0xFF34332E, 0xFF41403B, 0xFF33332B, 0xFF3B3B33, 0xFF2C2C22, 0xFF4A4A40, 0xFF28281E, 0xFF525246,
        0xFF2B2609, 0xFF4F4C43, 0xFF1B2A13, 0xFF4A514A, 0xFF3D3204, 0xFF493208, 0xFF584614, 0xFF604E0C, 0xFF634E0D, 0xFF674E0E, 0xFF694D0C, 0xFF6A4C0C, 0xFF684C0B, 0xFF654C09, 0xFF634C08, 0xFF6B500D, 0xFF694E0B, 0xFF674C09, 0xFF664B08, 0xFF674C09,
        0xFF674C09, 0xFF664B08, 0xFF654A07, 0xFF62440E, 0xFF6F5309, 0xFFD7BE5A, 0xFFFFF081, 0xFFFFE97E, 0xFFFAE27E, 0xFFF9E582, 0xFFFEEF8A, 0xFFF4E56E, 0xFFF7E776, 0xFFF8E57C, 0xFFF8E678, 0xFFFEED77, 0xFFFFF27C, 0xFFEED970, 0xFF8C7518, 0xFF664A0B,
        0xFF604906, 0xFF65530B, 0xFF665007, 0xFF6A4F08, 0xFF715D16, 0xFF726E25, 0xFF748034, 0xFF0E0C0D, 0xFF000100, 0xFF17201D, 0xFF293835, 0xFF2D3D3A, 0xFF384847, 0xFF324040, 0xFF263434, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FDFFFE, 0x00FFFEFF, 0x00FFFDFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF505C58, 0xFF3E4D46, 0xFF091106, 0xFF0D0F0E, 0xFF2C2E3B, 0xFF1D2925, 0xFF4D5E32, 0xFF646E29, 0xFF615C18, 0xFF6C561C, 0xFF6C5418, 0xFF6D5318, 0xFF6E5419, 0xFF6D5318,
        0xFF6A4F17, 0xFF6C531B, 0xFF6D5620, 0xFF6E4E15, 0xFF6B541E, 0xFF695319, 0xFF6E5211, 0xFF675115, 0xFF665720, 0xFF6B4C0B, 0xFFC58D38, 0xFFC4A435, 0xFF7F6123, 0xFF978571, 0xFF929288, 0xFF9DAB9E, 0xFF94A499, 0xFF687160, 0xFF3C3F20, 0xFF3B3F3E,
        0xFF5B5964, 0xFF595343, 0xFF4D3D2E, 0xFF50410A, 0xFF4A3905, 0xFF97873C, 0xFFB09D63, 0xFFB3BEB0, 0xFF7C867B, 0xFF4B574D, 0xFF44514A, 0xFF374744, 0xFF344646, 0xFF627677, 0xFF8FA4A7, 0xFF53625F, 0xFF3A4A49, 0xFF445658, 0xFF56696D, 0xFF6C8087,
        0xFF6B838D, 0xFF6C8792, 0xFF99B3C0, 0xFF9FBACD, 0xFFA9C4D7, 0xFFACCBDF, 0xFFA9C8DA, 0xFFADCAD8, 0xFFACC7D0, 0xFF8DA2A7, 0xFF4B5F60, 0xFF505F62, 0xFF48575A, 0xFF3F5153, 0xFF516161, 0xFF434F4B, 0xFF8B9388, 0xFFABAE9D, 0xFF81806C, 0xFF544B14,
        0xFF7A600C, 0xFFCFA02C, 0xFFE1A628, 0xFFDFA22D, 0xFFE2AA3D, 0xFFE2AD3B, 0xFFE2AC32, 0xFFB58D2D, 0xFF644A0F, 0xFF745B18, 0xFFD0A642, 0xFFDAA034, 0xFFE6B036, 0xFFE2B02D, 0xFFE4A637, 0xFFD9A52F, 0xFFB78C24, 0xFF927F3A, 0xFF8B9573, 0xFF939E8D,
        0xFFAFAE9C, 0xFF3E3E34, 0xFF48575A, 0xFF485B59, 0xFF4C5F5D, 0xFF4A5D5B, 0xFF465653, 0xFF475754, 0xFF4D5C57, 0xFF4C5B56, 0xFF45544F, 0xFF4D5957, 0xFF495553, 0xFF475351, 0xFF455353, 0xFF455353, 0xFF485857, 0xFF4B5B5B, 0xFF485858, 0xFF465656,
        0xFF465656, 0xFF465656, 0xFF465656, 0xFF455555, 0xFF455555, 0xFF455555, 0xFF445454, 0xFF465655, 0xFF425251, 0xFF40504F, 0xFF415150, 0xFF435352, 0xFF445453, 0xFF455554, 0xFF465655, 0xFF445453, 0xFF465655, 0xFF465655, 0xFF455554, 0xFF465655,
        0xFF495958, 0xFF485857, 0xFF445453, 0xFF435350, 0xFF42524F, 0xFF41514E, 0xFF41514E, 0xFF41514E, 0xFF42524F, 0xFF435350, 0xFF435350, 0xFF455554, 0xFF435352, 0xFF425251, 0xFF425251, 0xFF435352, 0xFF435352, 0xFF415150, 0xFF3F4F4E, 0xFF40504D,
        0xFF41514E, 0xFF42524F, 0xFF435350, 0xFF435350, 0xFF41514E, 0xFF3F4F4C, 0xFF3E4E4B, 0xFF3D4E48, 0xFF3E4F49, 0xFF40514B, 0xFF41524C, 0xFF41524C, 0xFF41524C, 0xFF41524C, 0xFF41524C, 0xFF3C4A4A, 0xFF26321E, 0xFF8E9492, 0xFF97A08B, 0xFF818C6A,
        0xFFA7A67A, 0xFF968324, 0xFFA68117, 0xFF9D7B10, 0xFFA3831C, 0xFFA08625, 0xFFAA9031, 0xFF816001, 0xFFAE821F, 0xFFE0AA3E, 0xFFD69829, 0xFFCB952B, 0xFF805D1D, 0xFFA1967A, 0xFF515941, 0xFF8C9779, 0xFF4D513A, 0xFF83856D, 0xFF4D532F, 0xFF828769,
        0xFF595E40, 0xFF808567, 0xFF5B6145, 0xFF767C60, 0xFF596243, 0xFF6A7354, 0xFF6C7654, 0xFF5D6244, 0xFF7C8065, 0xFF51553A, 0xFF787C61, 0xFF595E40, 0xFF868B6D, 0xFF515636, 0xFF898F6D, 0xFF495726, 0xFF828A63, 0xFF534F32, 0xFF9E9576, 0xFF857750,
        0xFFA4976B, 0xFF7B6F49, 0xFF9D9274, 0xFF887D45, 0xFF998E56, 0xFF897E46, 0xFF938850, 0xFF8A7F47, 0xFF897E46, 0xFF8A7F47, 0xFF837840, 0xFF918C54, 0xFF827D45, 0xFF989359, 0xFF7A753B, 0xFF9A9659, 0xFF7E7A3D, 0xFF9E9A5D, 0xFF767233, 0xFF969C76,
        0xFF817E47, 0xFF848158, 0xFF49521B, 0xFF9F9F79, 0xFFA69448, 0xFFBFA15B, 0xFF9A8238, 0xFF5F4D0B, 0xFF624D0C, 0xFF664D0D, 0xFF694D0C, 0xFF6A4C0C, 0xFF694D0C, 0xFF664D0A, 0xFF644D09, 0xFF6A4F0C, 0xFF694E0B, 0xFF674C09, 0xFF664B08, 0xFF674C09,
        0xFF674C09, 0xFF664B08, 0xFF654A07, 0xFF6C4210, 0xFF714A00, 0xFFD4AE3F, 0xFFFAD44F, 0xFFF1CC40, 0xFFF1CD43, 0xFFF1D045, 0xFFF4D546, 0xFFF7CC42, 0xFFF7CB48, 0xFFF2C546, 0xFFF4C543, 0xFFF9CB42, 0xFFFFD24A, 0xFFFFCE53, 0xFFAC780A, 0xFF65490A,
        0xFF614A07, 0xFF64520A, 0xFF665007, 0xFF6B5009, 0xFF6F5B14, 0xFF716D24, 0xFF7B873B, 0xFF151314, 0xFF000100, 0xFF161F1C, 0xFF293835, 0xFF2B3B38, 0xFF364645, 0xFF313F3F, 0xFF253333, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FDFFFE, 0x00FFFEFF, 0x00FFFDFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF4E5A56, 0xFF3D4C45, 0xFF070F04, 0xFF0A0C0B, 0xFF2B2D3A, 0xFF293531, 0xFF5F7044, 0xFF6B7530, 0xFF5C5713, 0xFF71541A, 0xFF6E501A, 0xFF71531F, 0xFF6F511D, 0xFF6F511B,
        0xFF6A4C16, 0xFF6C4E1A, 0xFF6F5021, 0xFF72510A, 0xFF654C0C, 0xFF684E11, 0xFF745213, 0xFF6D521A, 0xFF5F4E1A, 0xFF5F4101, 0xFFC9933D, 0xFFD3AF41, 0xFF744E00, 0xFF9D8044, 0xFF817F4F, 0xFF6A7D61, 0xFF64796A, 0xFF5F6444, 0xFF3F3500, 0xFF3D2C12,
        0xFF402723, 0xFF463100, 0xFF77602A, 0xFF5E450F, 0xFF816A38, 0xFFA49536, 0xFF534500, 0xFF5F6A62, 0xFF444F49, 0xFF455451, 0xFF3C4C4B, 0xFF3E5052, 0xFF62777C, 0xFF7A9197, 0xFF92A9B1, 0xFF879595, 0xFF5B6B6A, 0xFF3D4F51, 0xFF36494D, 0xFF576B72,
        0xFF6E858D, 0xFF7F97A1, 0xFF758D99, 0xFF9FB8CC, 0xFFB3CEE1, 0xFFABC7DC, 0xFF9EBACF, 0xFFA5C0D1, 0xFF99B1BD, 0xFF91A5AC, 0xFF506168, 0xFF3E5157, 0xFF506369, 0xFF42575C, 0xFF475B5C, 0xFF465655, 0xFF4A554D, 0xFF656B5D, 0xFF383B28, 0xFF4D4310,
        0xFF7D6213, 0xFFDFAF41, 0xFFEBAF35, 0xFFE2A433, 0xFFE4AB40, 0xFFE4AF3F, 0xFFDEA730, 0xFFB79031, 0xFF654A13, 0xFF6E5714, 0xFFCFA342, 0xFFDB9E34, 0xFFE6AE35, 0xFFE5AE2D, 0xFFE8A83A, 0xFFDBA632, 0xFFB78C26, 0xFF998843, 0xFF8A9374, 0xFF899484,
        0xFFACAB99, 0xFF3F3F33, 0xFF4E5D60, 0xFF4B5E5C, 0xFF4C5F5D, 0xFF4F5F5E, 0xFF4D5D5A, 0xFF4E5D5A, 0xFF505F5A, 0xFF4C5B56, 0xFF46524E, 0xFF485452, 0xFF4B5755, 0xFF4A5654, 0xFF445252, 0xFF465454, 0xFF485857, 0xFF475757, 0xFF455555, 0xFF425251,
        0xFF435352, 0xFF445453, 0xFF455554, 0xFF455554, 0xFF455554, 0xFF445453, 0xFF445453, 0xFF425251, 0xFF425251, 0xFF415150, 0xFF415150, 0xFF435352, 0xFF455554, 0xFF465655, 0xFF455554, 0xFF445453, 0xFF465655, 0xFF455554, 0xFF445453, 0xFF445453,
        0xFF465655, 0xFF445453, 0xFF40504F, 0xFF40504D, 0xFF40504D, 0xFF41514E, 0xFF41514E, 0xFF41514E, 0xFF42524F, 0xFF42524F, 0xFF435350, 0xFF475756, 0xFF445453, 0xFF435352, 0xFF445453, 0xFF465655, 0xFF455554, 0xFF425251, 0xFF3E4E4D, 0xFF42524F,
        0xFF42524F, 0xFF435350, 0xFF435350, 0xFF435350, 0xFF42524F, 0xFF41514E, 0xFF40504D, 0xFF3E4F49, 0xFF3E4F49, 0xFF3F504A, 0xFF40514B, 0xFF42534D, 0xFF42534D, 0xFF41524C, 0xFF40514B, 0xFF3E4C4C, 0xFF27331F, 0xFF8C9290, 0xFF929B86, 0xFF7E8967,
        0xFFA6A579, 0xFF978425, 0xFFAB861C, 0xFFA8881B, 0xFF97780E, 0xFFA48721, 0xFFA38522, 0xFFAA8721, 0xFFB98D22, 0xFFD6A131, 0xFFE0A330, 0xFFE0A032, 0xFFC2943E, 0xFFA18D48, 0xFF534F0F, 0xFF807E4B, 0xFF6D6848, 0xFFA8A989, 0xFF5E6836, 0xFF7D7F40,
        0xFF57581F, 0xFF6F713F, 0xFF6F7046, 0xFF878961, 0xFF7A7D52, 0xFF767A48, 0xFF646B37, 0xFF585C2A, 0xFF9B9D6E, 0xFF747249, 0xFF87835D, 0xFF635E34, 0xFF7B7545, 0xFF615C24, 0xFFA8A264, 0xFF72783A, 0xFF82814B, 0xFF786C3A, 0xFFBCA76E, 0xFFA0843B,
        0xFFC6A554, 0xFFB3903F, 0xFFC19B52, 0xFFB48F41, 0xFFBA9547, 0xFFBA9547, 0xFFBE994B, 0xFFC19C4E, 0xFFBE994B, 0xFFC8A355, 0xFFC39E50, 0xFFBF9D53, 0xFFB49346, 0xFFC6A558, 0xFFAD8C3F, 0xFFCAA95A, 0xFFB99849, 0xFFCAA958, 0xFFAE8D3C, 0xFFB39F6A,
        0xFFB2954F, 0xFFAD8F5B, 0xFF6E6225, 0xFF7D6F4C, 0xFF705B1A, 0xFF988050, 0xFFB1A678, 0xFF5F4D0B, 0xFF614C0B, 0xFF664D0D, 0xFF694D0C, 0xFF6B4D0D, 0xFF694D0C, 0xFF664D0A, 0xFF654E0A, 0xFF6A4F0C, 0xFF684D0A, 0xFF674C09, 0xFF664B08, 0xFF674C09,
        0xFF674C09, 0xFF674C09, 0xFF664B08, 0xFF5E4E12, 0xFF665600, 0xFFD2C14D, 0xFFF9E562, 0xFFF6DE60, 0xFFFFE877, 0xFFFFE67F, 0xFFFEE580, 0xFFF9DC5C, 0xFFFEDE65, 0xFFFDDC69, 0xFFFFE46B, 0xFFFFE062, 0xFFFDDB5D, 0xFFFBD664, 0xFFA47D18, 0xFF644809,
        0xFF634C09, 0xFF635109, 0xFF654F06, 0xFF6C510A, 0xFF6C5811, 0xFF6F6B22, 0xFF869246, 0xFF1F1D1E, 0xFF000100, 0xFF141D1A, 0xFF2B3A37, 0xFF293936, 0xFF334342, 0xFF313F3F, 0xFF243232, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FCFEFD, 0x00FFFEFF, 0x00FFFDFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF4D5955, 0xFF3F4E47, 0xFF0D150A, 0xFF121413, 0xFF2D2F3C, 0xFF283430, 0xFF5F7044, 0xFF66702B, 0xFF57520E, 0xFF735411, 0xFF6B4B10, 0xFF6E4E1B, 0xFF694B17, 0xFF6E5117,
        0xFF6C4E10, 0xFF6C4C13, 0xFF6E4D1A, 0xFF6E5105, 0xFF6F5614, 0xFF725519, 0xFF6C480A, 0xFF694A14, 0xFF685422, 0xFF6B4D0D, 0xFFC38F39, 0xFFC89D35, 0xFF86630F, 0xFF9F8E49, 0xFF717236, 0xFF858E61, 0xFF6F7051, 0xFF504518, 0xFF5C4800, 0xFF61490B,
        0xFF4E3500, 0xFF776205, 0xFFA9904D, 0xFF614A14, 0xFFA38E73, 0xFF958B50, 0xFF403809, 0xFF394543, 0xFF475351, 0xFF556364, 0xFF394B4D, 0xFF34474D, 0xFF748891, 0xFF90A6B1, 0xFF68808C, 0xFF8F9F9E, 0xFF5E6E6E, 0xFF435557, 0xFF415458, 0xFF51646A,
        0xFF586C73, 0xFF8B9FA8, 0xFF9CB3BB, 0xFF7B92A2, 0xFF96B0C1, 0xFFAEC9DC, 0xFFB0CBDE, 0xFF95AFC0, 0xFF455B69, 0xFF4A5C66, 0xFF4A5960, 0xFF4E6269, 0xFF42565D, 0xFF50676D, 0xFF3E5358, 0xFF495B5B, 0xFF49564F, 0xFF485043, 0xFF404435, 0xFF524719,
        0xFF7E6218, 0xFFDEAD44, 0xFFE8AC34, 0xFFE5A738, 0xFFE2A940, 0xFFDFA93C, 0xFFE1AA33, 0xFFB48D2E, 0xFF61460E, 0xFF6E5513, 0xFFD6AA49, 0xFFD89A33, 0xFFE1A932, 0xFFE5AE2F, 0xFFEDAA41, 0xFFDBA835, 0xFFBC922E, 0xFFAA9856, 0xFF959E7F, 0xFF8A9786,
        0xFFADAC9A, 0xFF3B3D30, 0xFF485858, 0xFF546463, 0xFF4D5D5C, 0xFF4E5E5D, 0xFF4F5E5B, 0xFF41504D, 0xFF2F3E39, 0xFF283430, 0xFF26322E, 0xFF313D3B, 0xFF414D4B, 0xFF4B5755, 0xFF4A5858, 0xFF4A5858, 0xFF485857, 0xFF455555, 0xFF445454, 0xFF425251,
        0xFF435352, 0xFF445453, 0xFF465655, 0xFF465655, 0xFF455554, 0xFF445453, 0xFF435352, 0xFF40504F, 0xFF435352, 0xFF435352, 0xFF415150, 0xFF425251, 0xFF465655, 0xFF465655, 0xFF425251, 0xFF455554, 0xFF465655, 0xFF445453, 0xFF425251, 0xFF425251,
        0xFF455554, 0xFF445453, 0xFF40504F, 0xFF40504D, 0xFF40504D, 0xFF41514E, 0xFF42524F, 0xFF435350, 0xFF435350, 0xFF435350, 0xFF435350, 0xFF455554, 0xFF435352, 0xFF435352, 0xFF455554, 0xFF485857, 0xFF485857, 0xFF455554, 0xFF415150, 0xFF41514E,
        0xFF41514E, 0xFF41514E, 0xFF40504D, 0xFF40504D, 0xFF40504D, 0xFF40504D, 0xFF40504D, 0xFF41524C, 0xFF40514B, 0xFF3E4F49, 0xFF3F504A, 0xFF41524C, 0xFF42534D, 0xFF42534D, 0xFF41524C, 0xFF404E4E, 0xFF2B3723, 0xFF8F9593, 0xFF949D88, 0xFF808B69,
        0xFFA5A478, 0xFF917E1F, 0xFFA58016, 0xFFA28116, 0xFFC9AA3E, 0xFF927309, 0xFFA5841B, 0xFFB48E23, 0xFFCEA030, 0xFFD29B26, 0xFFD99E26, 0xFFE49D29, 0xFFCD9830, 0xFFA28623, 0xFF796C10, 0xFF6C621D, 0xFFA79E77, 0xFFA6A77F, 0xFFACB77D, 0xFF91943D,
        0xFF87883C, 0xFF898848, 0xFFA3A26C, 0xFFA09E6D, 0xFFA1A16B, 0xFF6B6C30, 0xFF4A4C0A, 0xFF555716, 0xFF8A8B51, 0xFFAFAC79, 0xFFA59F6F, 0xFF9F9562, 0xFF85793C, 0xFF8A7E32, 0xFF9F933F, 0xFFB6B890, 0xFF8C8A63, 0xFFACA173, 0xFF988342, 0xFF9F8224,
        0xFFBA9723, 0xFFC59B22, 0xFFC79921, 0xFFC19421, 0xFFBC8F1C, 0xFFC19421, 0xFFBB8E1B, 0xFFC29522, 0xFFBC8F1C, 0xFFCB9E2B, 0xFFC99C29, 0xFFBE881B, 0xFFC69023, 0xFFC89225, 0xFFC48F1F, 0xFFC69121, 0xFFCF9A28, 0xFFCA9523, 0xFFC38E1C, 0xFFC59123,
        0xFFCE9417, 0xFFC6922D, 0xFF8B720E, 0xFF514103, 0xFF554C00, 0xFF4E4E1C, 0xFF7A8C62, 0xFF5E4C0A, 0xFF614C0B, 0xFF664D0D, 0xFF694D0C, 0xFF6B4D0D, 0xFF6A4E0D, 0xFF674E0B, 0xFF664F0B, 0xFF6A4F0C, 0xFF684D0A, 0xFF664B08, 0xFF664B08, 0xFF674C09,
        0xFF674C09, 0xFF674C09, 0xFF664B08, 0xFF68491D, 0xFF6B4D04, 0xFFD6B647, 0xFFF9D751, 0xFFF5CE45, 0xFFFFD754, 0xFFFDD456, 0xFFFFD658, 0xFFF6D54A, 0xFFFAD755, 0xFFF6D355, 0xFFFFDA58, 0xFFFCD84E, 0xFFF8D249, 0xFFF4CC50, 0xFF9A7000, 0xFF634708,
        0xFF644D0A, 0xFF625008, 0xFF644E05, 0xFF6E530C, 0xFF69550E, 0xFF6D6920, 0xFF8E9A4E, 0xFF292728, 0xFF000100, 0xFF141D1A, 0xFF2D3C39, 0xFF293936, 0xFF324241, 0xFF313F3F, 0xFF243232, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FCFEFD, 0x00FFFEFF, 0x00FFFDFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF4B5753, 0xFF33423B, 0xFF081005, 0xFF121413, 0xFF252734, 0xFF1A2622, 0xFF55663A, 0xFF656F2A, 0xFF66611D, 0xFF795C0E, 0xFF6A4E0D, 0xFF6C521F, 0xFF604B16, 0xFF695614,
        0xFF69520C, 0xFF6B4F0E, 0xFF6F4E18, 0xFF65500D, 0xFF635015, 0xFF674C14, 0xFF765216, 0xFF765720, 0xFF644F1A, 0xFF6C4D0C, 0xFFBB882F, 0xFFD7A544, 0xFF79601E, 0xFFAEB182, 0xFF737B48, 0xFF544913, 0xFF58390A, 0xFF624314, 0xFF624E13, 0xFF574709,
        0xFF776B19, 0xFFA99F40, 0xFFC7B583, 0xFF94884C, 0xFF9A8E7E, 0xFF746F59, 0xFF242601, 0xFF333E3A, 0xFF4D5957, 0xFF3B4949, 0xFF4E5D60, 0xFF4A5D63, 0xFF30444D, 0xFF647A85, 0xFF97AFBB, 0xFF697B7B, 0xFF3E5050, 0xFF445658, 0xFF506266, 0xFF7E9195,
        0xFF788B91, 0xFF566970, 0xFF697C83, 0xFF758B98, 0xFF778E9C, 0xFFA1BBCA, 0xFFA8C2D3, 0xFF859CAC, 0xFF3F5260, 0xFF465761, 0xFF47545C, 0xFF455960, 0xFF4C6067, 0xFF42595F, 0xFF485D62, 0xFF425454, 0xFF515E57, 0xFF343C2F, 0xFF2C3021, 0xFF463A10,
        0xFF765A11, 0xFFD9A83F, 0xFFE1A42F, 0xFFE7A938, 0xFFE1A83D, 0xFFDBA636, 0xFFE4AE34, 0xFFB68B23, 0xFF634505, 0xFF6E530C, 0xFFDCAF4A, 0xFFDA9D33, 0xFFE2AB36, 0xFFE6B034, 0xFFE6A740, 0xFFBE8A1C, 0xFFAA7F20, 0xFFA29050, 0xFF919C7E, 0xFF899685,
        0xFFB0B29D, 0xFF424436, 0xFF4D5F5F, 0xFF526261, 0xFF4D5D5C, 0xFF4E5C5C, 0xFF455451, 0xFF2B3A37, 0xFF17231F, 0xFF131F1B, 0xFF131F1B, 0xFF172321, 0xFF2B3735, 0xFF404C4A, 0xFF495757, 0xFF4E5C5C, 0xFF4D5D5C, 0xFF4B5B5B, 0xFF485858, 0xFF435352,
        0xFF445453, 0xFF465655, 0xFF465655, 0xFF465655, 0xFF455554, 0xFF435352, 0xFF425251, 0xFF40504F, 0xFF445453, 0xFF455554, 0xFF425251, 0xFF415150, 0xFF445453, 0xFF445453, 0xFF40504F, 0xFF455554, 0xFF455554, 0xFF435352, 0xFF40504F, 0xFF415150,
        0xFF455554, 0xFF465655, 0xFF445453, 0xFF41514E, 0xFF42524F, 0xFF435350, 0xFF445451, 0xFF445451, 0xFF455552, 0xFF455552, 0xFF455552, 0xFF455554, 0xFF445453, 0xFF445453, 0xFF465655, 0xFF485857, 0xFF485857, 0xFF445453, 0xFF415150, 0xFF42524F,
        0xFF42524F, 0xFF41514E, 0xFF41514E, 0xFF40504D, 0xFF40504D, 0xFF41514E, 0xFF41514E, 0xFF44554F, 0xFF41524C, 0xFF3E4F49, 0xFF3D4E48, 0xFF3F504A, 0xFF41524C, 0xFF42534D, 0xFF43544E, 0xFF414F4F, 0xFF2D3925, 0xFF8F9593, 0xFF939C87, 0xFF7D8866,
        0xFF9F9E72, 0xFF877415, 0xFF9B760C, 0xFF9B7C12, 0xFFBB9A31, 0xFFCFAD42, 0xFF936E03, 0xFFAC8212, 0xFFDBA936, 0xFFD9A22B, 0xFFDDA328, 0xFFDA941B, 0xFFCC982B, 0xFF826602, 0xFF948729, 0xFF7C722D, 0xFF595029, 0xFF676741, 0xFF5D682E, 0xFF6A6D16,
        0xFF636418, 0xFF5F5E1E, 0xFF61602A, 0xFF615F2E, 0xFF676731, 0xFF535418, 0xFF515311, 0xFF575C1A, 0xFF55561C, 0xFF73713E, 0xFF655F2F, 0xFF6A602D, 0xFF6F6326, 0xFF6E6216, 0xFF6E620E, 0xFF4D5735, 0xFF525737, 0xFF67653F, 0xFF51480D, 0xFF7B6B13,
        0xFFB0952A, 0xFFB38F1D, 0xFFC8A02F, 0xFFBC972C, 0xFFB99429, 0xFFB79227, 0xFFB38E23, 0xFFB48F24, 0xFFAF8A1F, 0xFFB89328, 0xFFB89328, 0xFFB5851F, 0xFFBF8F29, 0xFFC09129, 0xFFBC8D25, 0xFFBB8C24, 0xFFC5962C, 0xFFBE8F25, 0xFFBB8C22, 0xFFC28E20,
        0xFFBD8306, 0xFFC38D29, 0xFF9C801C, 0xFF604E0E, 0xFF5A4D00, 0xFF4C4914, 0xFF627245, 0xFF5E4C0A, 0xFF614C0B, 0xFF664D0D, 0xFF694D0C, 0xFF6B4D0D, 0xFF6A4E0D, 0xFF674E0B, 0xFF664F0B, 0xFF694E0B, 0xFF684D0A, 0xFF664B08, 0xFF664B08, 0xFF674C09,
        0xFF684D0A, 0xFF674C09, 0xFF674C09, 0xFF62440E, 0xFF6A4D00, 0xFFD5B843, 0xFFFFDE56, 0xFFFFDB4F, 0xFFFFDB55, 0xFFFDD552, 0xFFFFD955, 0xFFFADB4D, 0xFFFEDE57, 0xFFF6D554, 0xFFFAD852, 0xFFFEDD50, 0xFFFFDB4F, 0xFFF6D051, 0xFF9E7604, 0xFF634708,
        0xFF654E0B, 0xFF625008, 0xFF634D04, 0xFF6E530C, 0xFF68540D, 0xFF6B671E, 0xFF909C50, 0xFF2C2A2B, 0xFF010302, 0xFF141D1A, 0xFF2E3D3A, 0xFF2A3A37, 0xFF324241, 0xFF324040, 0xFF263434, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FCFEFD, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF4A5652, 0xFF3C4B44, 0xFF0B1308, 0xFF121413, 0xFF252734, 0xFF1F2B27, 0xFF596A3E, 0xFF5F6924, 0xFF5D5814, 0xFF6C5505, 0xFF644E12, 0xFF726135, 0xFF64572A, 0xFF685E20,
        0xFF63570F, 0xFF685314, 0xFF725624, 0xFF776C3F, 0xFF6F663B, 0xFF5D4917, 0xFF6D4B0E, 0xFF705013, 0xFF624C12, 0xFF86651E, 0xFFC38D33, 0xFFCF9B36, 0xFF7F6A33, 0xFF959B81, 0xFF4F512A, 0xFF5F4508, 0xFF794B10, 0xFF6B491B, 0xFF8A7E56, 0xFF989373,
        0xFFC2BF88, 0xFF807B41, 0xFFCEC4A9, 0xFFBEBD7A, 0xFF9A9679, 0xFF6A6A4E, 0xFF323A09, 0xFF2A342B, 0xFF424C44, 0xFF4A554F, 0xFF3E4A48, 0xFF3D4D4D, 0xFF384A4E, 0xFF46595F, 0xFF7E9299, 0xFF98AAAC, 0xFF5F7173, 0xFF3D4F51, 0xFF4B5D5F, 0xFF74868A,
        0xFF809296, 0xFF5F7175, 0xFF47595D, 0xFF73888D, 0xFF738A90, 0xFF748C96, 0xFF556D79, 0xFF516975, 0xFF4F616B, 0xFF526168, 0xFF475258, 0xFF405359, 0xFF42555B, 0xFF4F6469, 0xFF4C6061, 0xFF3D4D4C, 0xFF38433B, 0xFF202618, 0xFF191C09, 0xFF403508,
        0xFF755910, 0xFFE1B047, 0xFFE3A72F, 0xFFE8AB38, 0xFFE4AC3D, 0xFFDCA733, 0xFFDFA92B, 0xFFCC9926, 0xFF7F5B11, 0xFF735404, 0xFFD3A43A, 0xFFD79A2F, 0xFFE5B03C, 0xFFDEB038, 0xFFD9A23E, 0xFFB78417, 0xFFAC8123, 0xFF9D8D4F, 0xFF899378, 0xFF818F7E,
        0xFFA8AA95, 0xFF3D4132, 0xFF475A58, 0xFF4B5B5A, 0xFF515F5F, 0xFF505E5E, 0xFF384744, 0xFF1C2826, 0xFF16221E, 0xFF1D2824, 0xFF1A2521, 0xFF15211F, 0xFF1D2927, 0xFF323E3C, 0xFF445252, 0xFF4C5A5A, 0xFF4E5E5D, 0xFF4D5D5D, 0xFF455555, 0xFF445451,
        0xFF455552, 0xFF455552, 0xFF465653, 0xFF455552, 0xFF455552, 0xFF435350, 0xFF435350, 0xFF415150, 0xFF445453, 0xFF445453, 0xFF435352, 0xFF415150, 0xFF425251, 0xFF425251, 0xFF415150, 0xFF445453, 0xFF445453, 0xFF425251, 0xFF40504F, 0xFF425251,
        0xFF465655, 0xFF475756, 0xFF455554, 0xFF435350, 0xFF435350, 0xFF435350, 0xFF445451, 0xFF445451, 0xFF455552, 0xFF455552, 0xFF465653, 0xFF475756, 0xFF475756, 0xFF465655, 0xFF465655, 0xFF455554, 0xFF445453, 0xFF415150, 0xFF3F4F4E, 0xFF455552,
        0xFF445451, 0xFF445451, 0xFF435350, 0xFF435350, 0xFF435350, 0xFF435350, 0xFF435350, 0xFF44554F, 0xFF42534D, 0xFF3F504A, 0xFF3D4E48, 0xFF3E4F49, 0xFF40514B, 0xFF42534D, 0xFF44554F, 0xFF414F4F, 0xFF2D3925, 0xFF8C9290, 0xFF8E9782, 0xFF7C8765,
        0xFFA1A074, 0xFF8E7B1C, 0xFFA68117, 0xFF8C6C09, 0xFFA88622, 0xFFC19C34, 0xFFD3A93D, 0xFFA07001, 0xFFC3901B, 0xFFDBA42D, 0xFFD0981F, 0xFFDFA126, 0xFFC89C3B, 0xFFBBA85B, 0xFF88853E, 0xFFBAB981, 0xFF797552, 0xFFAAAB8B, 0xFF687142, 0xFF969859,
        0xFF797A41, 0xFF9C9E6C, 0xFF7D7E54, 0xFFA2A47C, 0xFF7F8257, 0xFF939765, 0xFF919864, 0xFF969D6A, 0xFF949A6A, 0xFF8C8F64, 0xFF9D9E76, 0xFF7A794D, 0xFFAEAC7B, 0xFF828047, 0xFFA2A161, 0xFF5E6E41, 0xFF9BA781, 0xFF6E7853, 0xFF9EA476, 0xFF817F42,
        0xFFD6CC85, 0xFFB9A463, 0xFFCEB179, 0xFFC1A969, 0xFFCFB777, 0xFFC2AA6A, 0xFFCEB676, 0xFFC6AE6E, 0xFFCCB474, 0xFFC8B070, 0xFFCBB373, 0xFFC1AF6F, 0xFFC0AE6E, 0xFFCCBA78, 0xFFB3A15F, 0xFFCBBA75, 0xFFBDAC67, 0xFFC9B873, 0xFFB7A660, 0xFFD2BE89,
        0xFFBFA25C, 0xFFD8B983, 0xFFB7A867, 0xFFB29F75, 0xFF7F611B, 0xFF9C7B45, 0xFF8F7842, 0xFF5F4D0B, 0xFF614C0B, 0xFF664D0D, 0xFF694D0C, 0xFF6B4D0D, 0xFF694D0C, 0xFF664D0A, 0xFF654E0A, 0xFF694E0B, 0xFF674C09, 0xFF664B08, 0xFF664B08, 0xFF674C09,
        0xFF684D0A, 0xFF684D0A, 0xFF674C09, 0xFF584D00, 0xFF675B00, 0xFFC7B93F, 0xFFF6E464, 0xFFFFEA72, 0xFFFDE577, 0xFFF5DE74, 0xFFF8E177, 0xFFF4DB5C, 0xFFFAE069, 0xFFF8DD6A, 0xFFFADD67, 0xFFFFE365, 0xFFFADD5F, 0xFFE8C857, 0xFF9E7C18, 0xFF644809,
        0xFF654E0B, 0xFF625008, 0xFF634D04, 0xFF6E530C, 0xFF69550E, 0xFF6B671E, 0xFF8B974B, 0xFF292728, 0xFF010302, 0xFF141D1A, 0xFF2C3B38, 0xFF2C3C39, 0xFF344443, 0xFF324040, 0xFF283636, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FCFEFD, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF485450, 0xFF3C4B44, 0xFF070F04, 0xFF0E100F, 0xFF292B38, 0xFF2C3834, 0xFF66774B, 0xFF636D28, 0xFF5C5713, 0xFF6A5913, 0xFF7E7041, 0xFFB4AA8F, 0xFFB4B093, 0xFFB9B888,
        0xFFB0AB71, 0xFFBCAE7D, 0xFFD2BB99, 0xFFB0B19F, 0xFFC4C6AE, 0xFF887C52, 0xFF664A0B, 0xFF684B07, 0xFF8D7835, 0xFFDAB96C, 0xFFE2AB4E, 0xFFC99B2B, 0xFF846628, 0xFFA89A80, 0xFF6A5C35, 0xFF5E4503, 0xFF6A4A0D, 0xFF65522A, 0xFFB0B096, 0xFF939882,
        0xFFAFB0A0, 0xFF4B4A2B, 0xFFBDB4A2, 0xFF8C884B, 0xFF6B6438, 0xFF56531E, 0xFF6A673A, 0xFF999D8E, 0xFF383E30, 0xFF424A3D, 0xFF485249, 0xFF35423B, 0xFF41504D, 0xFF3E4E4D, 0xFF455757, 0xFF576B6C, 0xFF465A5B, 0xFF445658, 0xFF7D8F91, 0xFF667578,
        0xFF475659, 0xFF4C5B5E, 0xFF404E51, 0xFF566A69, 0xFF768B8C, 0xFF738B8F, 0xFF536C71, 0xFF4A6167, 0xFF485B61, 0xFF4A585B, 0xFF535E60, 0xFF4D5C5F, 0xFF526164, 0xFF3A4C4E, 0xFF506060, 0xFF3C4844, 0xFF272F24, 0xFF8F9281, 0xFF54533F, 0xFF514619,
        0xFF6F5309, 0xFFDDAD3F, 0xFFE4A82C, 0xFFE4A830, 0xFFE1AA36, 0xFFE0AC32, 0xFFDDA824, 0xFFE4A929, 0xFFAF832C, 0xFF8A670D, 0xFFCB9B2C, 0xFFD2982A, 0xFFE1B23E, 0xFFCAA12B, 0xFFC19031, 0xFFBB881F, 0xFFB58C30, 0xFF9C8C50, 0xFF849175, 0xFF818F7E,
        0xFFA5A893, 0xFF3F4334, 0xFF475C57, 0xFF4E5C5C, 0xFF526060, 0xFF4E5C5C, 0xFF34403E, 0xFF14201E, 0xFF141F1B, 0xFF1B2622, 0xFF141F1B, 0xFF1B2725, 0xFF182422, 0xFF2E3A38, 0xFF435151, 0xFF465454, 0xFF485857, 0xFF495959, 0xFF405050, 0xFF42524F,
        0xFF435350, 0xFF435350, 0xFF435350, 0xFF445451, 0xFF445451, 0xFF445451, 0xFF445451, 0xFF41514F, 0xFF42524F, 0xFF425251, 0xFF435352, 0xFF425251, 0xFF40504F, 0xFF425251, 0xFF445453, 0xFF415150, 0xFF435352, 0xFF425251, 0xFF415150, 0xFF435352,
        0xFF465655, 0xFF445453, 0xFF425250, 0xFF435350, 0xFF435350, 0xFF435350, 0xFF42524F, 0xFF42524F, 0xFF435350, 0xFF445451, 0xFF455552, 0xFF465655, 0xFF475756, 0xFF475756, 0xFF455554, 0xFF435352, 0xFF415150, 0xFF40504F, 0xFF40504F, 0xFF445451,
        0xFF445451, 0xFF445451, 0xFF445451, 0xFF435350, 0xFF435350, 0xFF435350, 0xFF42524F, 0xFF42534D, 0xFF42534D, 0xFF41524C, 0xFF40514B, 0xFF3F504A, 0xFF40514B, 0xFF41524C, 0xFF43544E, 0xFF404E4E, 0xFF2B3723, 0xFF888E8C, 0xFF8A937E, 0xFF7C8765,
        0xFFA6A579, 0xFF968324, 0xFFB38E24, 0xFF957417, 0xFF6C4A00, 0xFFA67F1A, 0xFFD6AA3F, 0xFFCC9A2B, 0xFFCA9521, 0xFFD89F2B, 0xFFD9A12A, 0xFFD4A124, 0xFFCAA958, 0xFFCDC499, 0xFF6B7552, 0xFFAFBB97, 0xFF666A51, 0xFFCCCEB6, 0xFF6A704E, 0xFFAEB395,
        0xFF717658, 0xFFADB294, 0xFF72785C, 0xFFABB195, 0xFF757E5F, 0xFF99A283, 0xFF848E6C, 0xFF879071, 0xFF939C7F, 0xFF868F72, 0xFF9BA487, 0xFF727B5C, 0xFFA1AA8B, 0xFF747E5C, 0xFFAEB895, 0xFF707851, 0xFFA8B192, 0xFF606A51, 0xFFB0BB9D, 0xFF636943,
        0xFFC4C59D, 0xFFACA183, 0xFFC6B5A1, 0xFFACA56E, 0xFFCCC58E, 0xFFABA46D, 0xFFC7C089, 0xFFB3AC75, 0xFFC4BD86, 0xFFB3AC75, 0xFFBBB47D, 0xFFB5B27D, 0xFFB4B17A, 0xFFC1BE87, 0xFFA7A46D, 0xFFBFBD84, 0xFFAAA86F, 0xFFC3C186, 0xFFACAA6F, 0xFFC1C9A4,
        0xFFABA871, 0xFFCCC79D, 0xFFA3A76C, 0xFFE4DCAE, 0xFFC1A34D, 0xFFDBAE5D, 0xFF936B14, 0xFF5F4D0B, 0xFF624D0C, 0xFF664D0D, 0xFF694D0C, 0xFF6A4C0C, 0xFF694D0C, 0xFF664D0A, 0xFF644D09, 0xFF684D0A, 0xFF674C09, 0xFF664B08, 0xFF664B08, 0xFF674C09,
        0xFF684D0A, 0xFF684D0A, 0xFF674C09, 0xFF6E4912, 0xFF6F4B01, 0xFF9C781A, 0xFFAE871E, 0xFFB28C21, 0xFFAE8A1C, 0xFFAB8B14, 0xFFAC8F0F, 0xFFB08817, 0xFFB48B21, 0xFFB58B25, 0xFFB0851C, 0xFFB68D1B, 0xFFB48919, 0xFFAF821D, 0xFF8C5D05, 0xFF65490A,
        0xFF644D0A, 0xFF625008, 0xFF644E05, 0xFF6D520B, 0xFF6A560F, 0xFF6B671E, 0xFF849044, 0xFF211F20, 0xFF000201, 0xFF121B18, 0xFF293835, 0xFF2C3C39, 0xFF354544, 0xFF313F3F, 0xFF293737, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FCFEFD, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF485450, 0xFF394841, 0xFF081005, 0xFF141615, 0xFF292B38, 0xFF24302C, 0xFF5A6B3F, 0xFF5D6722, 0xFF625D19, 0xFF5A4B10, 0xFF827757, 0xFFCFCBC0, 0xFFD1D3C6, 0xFFC6CCAA,
        0xFFAAAC7D, 0xFFB0A682, 0xFFC7B29D, 0xFFB6C0BF, 0xFFC3CBC0, 0xFF7B7551, 0xFF6A5111, 0xFF694D03, 0xFF7B671E, 0xFFCDAC5B, 0xFFCD9534, 0xFFCFA829, 0xFF7B5409, 0xFF977550, 0xFF6D5223, 0xFF624F0B, 0xFF5D5115, 0xFF514E2B, 0xFFC0C8B3, 0xFF556139,
        0xFF68696B, 0xFF66644B, 0xFFAFA58C, 0xFF5F5524, 0xFF544619, 0xFF7E742F, 0xFFD3C1AB, 0xFF8C8F7C, 0xFF7E836F, 0xFF414838, 0xFF3A4233, 0xFF465045, 0xFF3E4B42, 0xFF475651, 0xFF3B4A45, 0xFF384C4D, 0xFF3D5152, 0xFF405254, 0xFF556769, 0xFF445356,
        0xFF4C5A5D, 0xFF445255, 0xFF536164, 0xFF475C55, 0xFF506663, 0xFF6D8585, 0xFF8CA4A6, 0xFF697E81, 0xFF455759, 0xFF475555, 0xFF4D5756, 0xFF4B595A, 0xFF435152, 0xFF556565, 0xFF243330, 0xFF2F3A34, 0xFF8E9488, 0xFF7C7D6B, 0xFF918F78, 0xFF7D7244,
        0xFF755A0D, 0xFFDBAB3C, 0xFFE9AE30, 0xFFE4A82E, 0xFFDDA631, 0xFFE1AE31, 0xFFDEAA22, 0xFFE9A921, 0xFFD2A446, 0xFFA78021, 0xFFD3A22C, 0xFFD9A02F, 0xFFDEB13C, 0xFFB18E18, 0xFFA4791B, 0xFFC28F26, 0xFFC0973B, 0xFF9E8E52, 0xFF849175, 0xFF829081,
        0xFFA2A590, 0xFF3C402F, 0xFF425752, 0xFF515F5F, 0xFF4F5D5D, 0xFF4F5B5B, 0xFF3C4846, 0xFF1F2B29, 0xFF1A2521, 0xFF1F2A26, 0xFF1A2521, 0xFF182422, 0xFF14201E, 0xFF2F3B39, 0xFF475555, 0xFF435151, 0xFF425251, 0xFF495959, 0xFF425252, 0xFF40504D,
        0xFF40504D, 0xFF40504D, 0xFF41514E, 0xFF42524F, 0xFF435351, 0xFF455553, 0xFF455654, 0xFF3E504E, 0xFF3D4D4C, 0xFF425251, 0xFF475756, 0xFF445453, 0xFF40504F, 0xFF425251, 0xFF475756, 0xFF3F4F4E, 0xFF415150, 0xFF435352, 0xFF425251, 0xFF435352,
        0xFF445453, 0xFF445453, 0xFF425251, 0xFF415150, 0xFF40504F, 0xFF435351, 0xFF435351, 0xFF41514E, 0xFF42524F, 0xFF435350, 0xFF445451, 0xFF435352, 0xFF445453, 0xFF455554, 0xFF445453, 0xFF415150, 0xFF40504F, 0xFF425251, 0xFF445453, 0xFF41514E,
        0xFF41514E, 0xFF41514E, 0xFF42524F, 0xFF41514E, 0xFF41514E, 0xFF40504D, 0xFF40504D, 0xFF40514B, 0xFF42534D, 0xFF43544E, 0xFF42534D, 0xFF41524C, 0xFF40514B, 0xFF40514B, 0xFF41524C, 0xFF404E4E, 0xFF2B3723, 0xFF868C8A, 0xFF868F7A, 0xFF788361,
        0xFFA09F73, 0xFF907D1E, 0xFFAC871D, 0xFFCAAB51, 0xFF937014, 0xFF774D00, 0xFFBB8C24, 0xFFD9A537, 0xFFD9A130, 0xFFDAA12D, 0xFFDAA12D, 0xFFC69A1D, 0xFF836825, 0xFF9F9E8C, 0xFF5F7066, 0xFF94A793, 0xFF616D57, 0xFF919481, 0xFF656751, 0xFF999E97,
        0xFF5A6056, 0xFF9DA596, 0xFF626B58, 0xFF8D9985, 0xFF6B7763, 0xFF8D9985, 0xFF687563, 0xFF838B7C, 0xFF788372, 0xFF7D8975, 0xFF778470, 0xFF7C8B76, 0xFF7E8F7D, 0xFF687A6A, 0xFF8B9D8F, 0xFF716E5F, 0xFF9F9F97, 0xFF676A63, 0xFF949C8D, 0xFF677254,
        0xFFA2A884, 0xFF807F61, 0xFFB2AA95, 0xFF807B67, 0xFFABA692, 0xFF7B7662, 0xFFA19C88, 0xFF85806C, 0xFFA29D89, 0xFF8C8773, 0xFF999480, 0xFF968979, 0xFFA29585, 0xFFA69989, 0xFF9D907F, 0xFF9D907F, 0xFF968976, 0xFFAEA18E, 0xFFA09380, 0xFFAEB0A5,
        0xFF868164, 0xFFACA89C, 0xFF7B8469, 0xFFAAA798, 0xFF7B6229, 0xFF7F561E, 0xFF633D00, 0xFF604E0C, 0xFF634E0D, 0xFF674E0E, 0xFF694D0C, 0xFF6A4C0C, 0xFF684C0B, 0xFF654C09, 0xFF634C08, 0xFF684D0A, 0xFF674C09, 0xFF664B08, 0xFF664B08, 0xFF674C09,
        0xFF684D0A, 0xFF684D0A, 0xFF684D0A, 0xFF664D24, 0xFF5D440E, 0xFF694D0E, 0xFF614307, 0xFF61430F, 0xFF5C4010, 0xFF59440B, 0xFF544202, 0xFF564A02, 0xFF594B0C, 0xFF5E4E13, 0xFF564608, 0xFF5B4A04, 0xFF594802, 0xFF5F4C12, 0xFF5D461A, 0xFF664A0B,
        0xFF644D0A, 0xFF635109, 0xFF654F06, 0xFF6D520B, 0xFF6B5710, 0xFF6B671E, 0xFF7F8B3F, 0xFF1A1819, 0xFF000100, 0xFF111A17, 0xFF263532, 0xFF2C3C39, 0xFF354544, 0xFF2F3D3D, 0xFF283636, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x00FFFBFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF4D5854, 0xFF34453B, 0xFF091406, 0xFF141613, 0xFF2A2C38, 0xFF222D29, 0xFF69784D, 0xFF69702C, 0xFF5F5814, 0xFF6A4B00, 0xFF6D6340, 0xFFA3ACBD, 0xFFB7C1DC, 0xFF8F8A76,
        0xFF534504, 0xFF665925, 0xFFBFB5AC, 0xFFCBCCBC, 0xFF8F8763, 0xFF614E16, 0xFF6D5519, 0xFF684D16, 0xFF7B5B1E, 0xFFAA802E, 0xFFCC9833, 0xFFDFA92D, 0xFF7D6407, 0xFFC1C080, 0xFF978E55, 0xFF664301, 0xFF774804, 0xFF654917, 0xFF8C886B, 0xFF68705B,
        0xFF9D966A, 0xFF928C5C, 0xFF8E8D6F, 0xFF645127, 0xFFA47523, 0xFFFBCD6F, 0xFFD0B86E, 0xFF898876, 0xFF7B7C74, 0xFF868E83, 0xFF39492E, 0xFF364D31, 0xFF435851, 0xFF3D5253, 0xFF39504A, 0xFF485448, 0xFF46534A, 0xFF4E5A58, 0xFF404E4F, 0xFF46555A,
        0xFF47595D, 0xFF475B5C, 0xFF445857, 0xFF445859, 0xFF46595D, 0xFF52656C, 0xFF81929C, 0xFF5B6C76, 0xFF4B5962, 0xFF47565D, 0xFF445358, 0xFF425855, 0xFF505D54, 0xFF35393A, 0xFF393D49, 0xFF828E8E, 0xFF73866A, 0xFF7A8565, 0xFFA9A79A, 0xFFAC822E,
        0xFF9A6F13, 0xFFCDA23A, 0xFFD0A330, 0xFFC8961D, 0xFFE2AF32, 0xFFDEA62B, 0xFFE4AA2F, 0xFFDAA615, 0xFFEEB63B, 0xFFB78524, 0xFFCDA140, 0xFFC9971C, 0xFFEAB430, 0xFFAF8E31, 0xFF9B9970, 0xFFE9B939, 0xFFD4A531, 0xFFAC903C, 0xFF95966E, 0xFF899685,
        0xFFA2A698, 0xFF3E4033, 0xFF505B55, 0xFF506060, 0xFF4E5E5E, 0xFF4E5E5E, 0xFF485858, 0xFF304040, 0xFF152525, 0xFF102020, 0xFF1B2B2B, 0xFF0F1F1E, 0xFF20302F, 0xFF3C4C4B, 0xFF435352, 0xFF445453, 0xFF485857, 0xFF435352, 0xFF465655, 0xFF43524F,
        0xFF3F4E4B, 0xFF455451, 0xFF3E4E4B, 0xFF465654, 0xFF485957, 0xFF475756, 0xFF435352, 0xFF445552, 0xFF475756, 0xFF475756, 0xFF40504F, 0xFF445453, 0xFF455452, 0xFF41504E, 0xFF424E4C, 0xFF455451, 0xFF43524F, 0xFF445350, 0xFF435351, 0xFF41514F,
        0xFF415250, 0xFF435352, 0xFF435352, 0xFF445453, 0xFF445453, 0xFF455554, 0xFF455654, 0xFF3E504E, 0xFF3D4D4C, 0xFF425251, 0xFF475756, 0xFF455554, 0xFF445453, 0xFF435352, 0xFF435352, 0xFF435352, 0xFF445453, 0xFF455554, 0xFF455554, 0xFF3E4E4D,
        0xFF415150, 0xFF445453, 0xFF435352, 0xFF40504F, 0xFF3F4F4E, 0xFF435352, 0xFF485857, 0xFF41514E, 0xFF435350, 0xFF445451, 0xFF41514E, 0xFF3D4D4A, 0xFF3C4C49, 0xFF3F4F4C, 0xFF42524F, 0xFF415049, 0xFF1F3121, 0xFF8B9E88, 0xFF83927D, 0xFF7D806B,
        0xFFA2946D, 0xFF977E2C, 0xFFA48413, 0xFFBF9817, 0xFFD1A739, 0xFF9A6D10, 0xFFB58727, 0xFFD6A735, 0xFFD5A127, 0xFFDBA02C, 0xFFDB9C35, 0xFF9A6C14, 0xFF414433, 0xFF3D4F5D, 0xFF404B47, 0xFF364945, 0xFF3B4949, 0xFF424844, 0xFF3C504E, 0xFF3C4B48,
        0xFF3F4E4B, 0xFF41504D, 0xFF41504D, 0xFF404F4C, 0xFF3F4E4B, 0xFF3F4E4B, 0xFF3F4E4B, 0xFF3E4E4D, 0xFF3E4E4D, 0xFF3F4F4E, 0xFF3E4E4D, 0xFF3E4E4D, 0xFF3E4E4D, 0xFF3E4E4D, 0xFF3F4F4E, 0xFF3F4F4C, 0xFF3F4F4C, 0xFF3E4E4B, 0xFF3D4D4C, 0xFF3C4C4B,
        0xFF3B4B4B, 0xFF3C4C4C, 0xFF3C4C4C, 0xFF384A4A, 0xFF3C4E4E, 0xFF3E5050, 0xFF3D4D4C, 0xFF3C4C4B, 0xFF3F4E4B, 0xFF3E4D4A, 0xFF3B4A47, 0xFF3D4E48, 0xFF40514B, 0xFF3B4B48, 0xFF3B4B48, 0xFF40504D, 0xFF3F4F4E, 0xFF3F4F4E, 0xFF415150, 0xFF3A4D4B,
        0xFF3D4B4C, 0xFF3F4E49, 0xFF3C5045, 0xFF3D5351, 0xFF3B444B, 0xFF403823, 0xFF5D490C, 0xFF664B08, 0xFF664B08, 0xFF654A07, 0xFF674C09, 0xFF694E0B, 0xFF6A4F0C, 0xFF684D0A, 0xFF674C09, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C,
        0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF654C0A, 0xFF664D0B, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF664D0B, 0xFF654C0A, 0xFF644B09, 0xFF654C0A, 0xFF664D0B, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF664D0B, 0xFF654C0A, 0xFF684C03,
        0xFF654E0B, 0xFF614B0F, 0xFF684F0F, 0xFF6D5108, 0xFF6A5506, 0xFF716B1D, 0xFF7C873B, 0xFF181617, 0xFF000100, 0xFF19221F, 0xFF273633, 0xFF2D3D3A, 0xFF2F3F3E, 0xFF313F3F, 0xFF2B3939, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFEFF, 0x00FFFCFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF4C5753, 0xFF3C4D43, 0xFF081207, 0xFF101211, 0xFF2C2E3A, 0xFF2C3731, 0xFF718154, 0xFF6E7531, 0xFF5F5812, 0xFF6C5006, 0xFF514B27, 0xFF454F5B, 0xFF434E64, 0xFF64624D,
        0xFF514807, 0xFF5F5321, 0xFFACA39A, 0xFFA8AE94, 0xFF7E7C4C, 0xFF4E3F00, 0xFF725B17, 0xFF6A5111, 0xFF745310, 0xFFBF913C, 0xFFC58F25, 0xFFCF9B2D, 0xFF8E7521, 0xFFEBE8B1, 0xFFAAA271, 0xFF604108, 0xFF795114, 0xFF70592D, 0xFFC8C8AE, 0xFF8A8F67,
        0xFF514A16, 0xFF767137, 0xFF555718, 0xFF615500, 0xFFE5C04E, 0xFFCDA738, 0xFF746309, 0xFF807F60, 0xFF888A63, 0xFF80856F, 0xFF798184, 0xFF374343, 0xFF3E5237, 0xFF405641, 0xFF43545E, 0xFF465349, 0xFF505F58, 0xFF3D4D4C, 0xFF677679, 0xFF96A7AE,
        0xFF6C7F85, 0xFF506568, 0xFF3D5253, 0xFF495D5E, 0xFF4A5D61, 0xFF485B62, 0xFF566771, 0xFF3E4F59, 0xFF44525B, 0xFF48575C, 0xFF465558, 0xFF535F5D, 0xFF333740, 0xFF363739, 0xFF8E9687, 0xFF6C7A6B, 0xFF6B776B, 0xFFB4B691, 0xFF8B853B, 0xFFB48831,
        0xFFB38628, 0xFFCCA037, 0xFFCE9F2D, 0xFFC9971E, 0xFFD9A327, 0xFFDCA429, 0xFFE2A82D, 0xFFE3AD29, 0xFFE6AD3C, 0xFFB68125, 0xFFD1A345, 0xFFCE9A24, 0xFFE6AC31, 0xFFAD8C3B, 0xFFB1B092, 0xFFAA8922, 0xFFC29E3D, 0xFFBEA85D, 0xFF9A9F77, 0xFF86937F,
        0xFFA2A695, 0xFF3A3C2F, 0xFF525C5B, 0xFF4D5D5D, 0xFF4B5B5B, 0xFF4B5B5B, 0xFF495959, 0xFF394949, 0xFF233333, 0xFF192929, 0xFF1B2B2B, 0xFF253534, 0xFF324241, 0xFF485857, 0xFF4A5A59, 0xFF485857, 0xFF485857, 0xFF40504F, 0xFF40504F, 0xFF485754,
        0xFF455451, 0xFF465552, 0xFF3D4D4A, 0xFF435351, 0xFF495958, 0xFF465655, 0xFF435352, 0xFF40504F, 0xFF3E4E4D, 0xFF455554, 0xFF465655, 0xFF40504F, 0xFF435352, 0xFF435250, 0xFF3C4846, 0xFF455451, 0xFF42514E, 0xFF404F4C, 0xFF445452, 0xFF465654,
        0xFF435352, 0xFF415150, 0xFF445453, 0xFF485857, 0xFF495958, 0xFF475756, 0xFF435352, 0xFF445552, 0xFF475756, 0xFF475756, 0xFF40504F, 0xFF455554, 0xFF465655, 0xFF465655, 0xFF465655, 0xFF465655, 0xFF475756, 0xFF485857, 0xFF495958, 0xFF425251,
        0xFF425251, 0xFF435352, 0xFF435352, 0xFF445453, 0xFF445453, 0xFF455554, 0xFF455554, 0xFF435350, 0xFF455552, 0xFF465653, 0xFF445451, 0xFF42524F, 0xFF41514E, 0xFF42524F, 0xFF445451, 0xFF3D4D43, 0xFF1E3022, 0xFF889987, 0xFF818E7C, 0xFF80856E,
        0xFFA19C74, 0xFF887A39, 0xFF84721E, 0xFFA78612, 0xFFBC9725, 0xFFC89D2C, 0xFFCA9923, 0xFFCC991A, 0xFFD6A324, 0xFFD09C26, 0xFFD4A033, 0xFF996B13, 0xFF404332, 0xFF3D4F5D, 0xFF414C48, 0xFF364943, 0xFF394747, 0xFF3E4440, 0xFF364A48, 0xFF374641,
        0xFF3A4944, 0xFF3D4C47, 0xFF3F4E49, 0xFF3F4E49, 0xFF3C4B46, 0xFF3A4944, 0xFF384742, 0xFF3A4A47, 0xFF3A4A47, 0xFF3A4A47, 0xFF3B4B48, 0xFF3C4C49, 0xFF3B4B48, 0xFF394946, 0xFF374744, 0xFF384845, 0xFF384845, 0xFF394946, 0xFF3A4A49, 0xFF394948,
        0xFF384848, 0xFF374747, 0xFF364646, 0xFF344646, 0xFF384A4A, 0xFF394B4B, 0xFF394948, 0xFF384847, 0xFF3A4946, 0xFF394845, 0xFF374643, 0xFF354640, 0xFF364741, 0xFF34453F, 0xFF364643, 0xFF3B4B48, 0xFF384847, 0xFF354544, 0xFF394948, 0xFF374A48,
        0xFF384647, 0xFF364540, 0xFF32463B, 0xFF324846, 0xFF323B42, 0xFF3A321D, 0xFF584407, 0xFF664B08, 0xFF664B08, 0xFF664B08, 0xFF674C09, 0xFF694E0B, 0xFF6A4F0C, 0xFF684D0A, 0xFF674C09, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C,
        0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF654C0A, 0xFF664D0B, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF664D0B, 0xFF654C0A, 0xFF664D0B, 0xFF664D0B, 0xFF674E0C, 0xFF684F0D, 0xFF684F0D, 0xFF684F0D, 0xFF674E0C, 0xFF664D0B, 0xFF6C5007,
        0xFF67500D, 0xFF634D11, 0xFF695010, 0xFF6D5108, 0xFF6B5607, 0xFF726C1E, 0xFF7F8A3E, 0xFF1E1C1D, 0xFF000100, 0xFF141D1A, 0xFF22312E, 0xFF2B3B38, 0xFF2F3F3E, 0xFF2F3D3D, 0xFF283636, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FEFEFE, 0x00FFFEFF, 0x00FFFDFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF4C5753, 0xFF405149, 0xFF061005, 0xFF0E100F, 0xFF2F313D, 0xFF303B35, 0xFF708053, 0xFF6F7630, 0xFF625B13, 0xFF6F5B16, 0xFF54502B, 0xFF475256, 0xFF47505F, 0xFF76765E,
        0xFF534D0F, 0xFF655E2A, 0xFFBDB7AB, 0xFFCDDAC0, 0xFF848559, 0xFF594E0E, 0xFF75601D, 0xFF705317, 0xFF785213, 0xFFBB8835, 0xFFD99B34, 0xFFDAA641, 0xFF775A0A, 0xFF766D34, 0xFF756937, 0xFF644A0F, 0xFF6D4D10, 0xFF675523, 0xFFA2A47F, 0xFF74713A,
        0xFF635C18, 0xFFA9A367, 0xFF8B8855, 0xFF615201, 0xFFDDC241, 0xFFCBB136, 0xFF5C5000, 0xFF5B5E19, 0xFF8B8C62, 0xFF8B897A, 0xFF7E7F77, 0xFF81877D, 0xFF374138, 0xFF3F4C43, 0xFF45544D, 0xFF4C5D53, 0xFF3D4E48, 0xFF415353, 0xFF6F8288, 0xFFA4B8BF,
        0xFF99B0B6, 0xFF6D8589, 0xFF476064, 0xFF485C5B, 0xFF45595A, 0xFF42535A, 0xFF405159, 0xFF4C5A63, 0xFF4D5C61, 0xFF435353, 0xFF5D6B6B, 0xFF414550, 0xFF2E2F34, 0xFF929688, 0xFF717C5E, 0xFF748068, 0xFF9D9D83, 0xFFA4934E, 0xFFA98D10, 0xFFB08325,
        0xFFBD902B, 0xFFC09125, 0xFFCA9825, 0xFFD3A128, 0xFFD9A327, 0xFFE1A92E, 0xFFE4AA2F, 0xFFE5AC38, 0xFFE6A93F, 0xFFBA8127, 0xFFD19F3E, 0xFFD49B27, 0xFFE8AD37, 0xFFAE8C42, 0xFFA6A590, 0xFF897B3A, 0xFFB9A661, 0xFFBCB076, 0xFF8D9570, 0xFF7E8C73,
        0xFFA6A994, 0xFF383A2D, 0xFF4C5759, 0xFF4C5C5C, 0xFF495959, 0xFF485858, 0xFF4B5B5B, 0xFF4B5B5B, 0xFF465656, 0xFF435353, 0xFF435353, 0xFF425251, 0xFF465655, 0xFF4D5D5C, 0xFF485857, 0xFF435352, 0xFF435352, 0xFF40504F, 0xFF425251, 0xFF475653,
        0xFF3D4C49, 0xFF3E4D4A, 0xFF425250, 0xFF455553, 0xFF455554, 0xFF455554, 0xFF455554, 0xFF415150, 0xFF445453, 0xFF495958, 0xFF3E4E4D, 0xFF3D4D4C, 0xFF415150, 0xFF465553, 0xFF44504E, 0xFF404F4C, 0xFF43524F, 0xFF43524F, 0xFF445452, 0xFF465655,
        0xFF445453, 0xFF425251, 0xFF445453, 0xFF485857, 0xFF495958, 0xFF465655, 0xFF435352, 0xFF40504F, 0xFF3E4E4D, 0xFF455554, 0xFF465655, 0xFF40504F, 0xFF425251, 0xFF445453, 0xFF455554, 0xFF445453, 0xFF445453, 0xFF465655, 0xFF485857, 0xFF465655,
        0xFF435352, 0xFF415150, 0xFF445453, 0xFF485857, 0xFF495958, 0xFF475756, 0xFF435352, 0xFF435350, 0xFF445451, 0xFF455552, 0xFF465653, 0xFF455552, 0xFF445451, 0xFF435350, 0xFF435350, 0xFF415144, 0xFF29392F, 0xFF86938A, 0xFF97A291, 0xFF767F60,
        0xFF8E956B, 0xFFB5B88B, 0xFF8F8D64, 0xFF7C680F, 0xFFAD9126, 0xFFC69F1E, 0xFFD9A619, 0xFFE2A820, 0xFFD59B1D, 0xFFD8A52F, 0xFFC89B26, 0xFF94670C, 0xFF3D402D, 0xFF3C4E5A, 0xFF414C48, 0xFF374A44, 0xFF394747, 0xFF3D443D, 0xFF344944, 0xFF3C4C42,
        0xFF3D4D43, 0xFF3F4F45, 0xFF415147, 0xFF425248, 0xFF415147, 0xFF3E4E44, 0xFF3C4C42, 0xFF3C4D45, 0xFF3A4B43, 0xFF394A42, 0xFF3B4C44, 0xFF3E4F47, 0xFF3E4F47, 0xFF3A4B43, 0xFF35463E, 0xFF41524C, 0xFF3E4E4B, 0xFF394946, 0xFF354542, 0xFF334342,
        0xFF344443, 0xFF364646, 0xFF384848, 0xFF324444, 0xFF354747, 0xFF364947, 0xFF364645, 0xFF354542, 0xFF374643, 0xFF394845, 0xFF394843, 0xFF3B4C44, 0xFF3A4B45, 0xFF384943, 0xFF3A4B45, 0xFF3D4D4A, 0xFF384845, 0xFF314140, 0xFF354544, 0xFF394C4A,
        0xFF3A4849, 0xFF384742, 0xFF34483D, 0xFF344A48, 0xFF333C43, 0xFF3A321D, 0xFF594508, 0xFF644B09, 0xFF644B09, 0xFF644B09, 0xFF654C0A, 0xFF674E0C, 0xFF674E0C, 0xFF664D0B, 0xFF654C0A, 0xFF664D0B, 0xFF664D0B, 0xFF664D0B, 0xFF664D0B, 0xFF664D0B,
        0xFF664D0B, 0xFF664D0B, 0xFF664D0B, 0xFF664D0B, 0xFF664D0B, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF664D0B, 0xFF664D0B, 0xFF674E0C, 0xFF684F0D, 0xFF684F0D, 0xFF684F0D, 0xFF684F0D, 0xFF684F0D, 0xFF684F0D, 0xFF674E0C, 0xFF6C5007,
        0xFF67500D, 0xFF654F13, 0xFF695010, 0xFF6F530A, 0xFF6D5809, 0xFF736D1F, 0xFF859044, 0xFF242223, 0xFF000100, 0xFF101916, 0xFF1F2E2B, 0xFF293936, 0xFF2E3E3D, 0xFF2E3C3C, 0xFF233131, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FEFEFE, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF4C5753, 0xFF3D4E46, 0xFF040E03, 0xFF0F1110, 0xFF2F313D, 0xFF29342C, 0xFF617140, 0xFF6D752A, 0xFF696319, 0xFF655313, 0xFF7D784E, 0xFFA5ADA2, 0xFF9BA1A1, 0xFFB4B395,
        0xFF918C4C, 0xFF938C56, 0xFFA8A18E, 0xFFA6B1A3, 0xFF999977, 0xFF766932, 0xFF6A5418, 0xFF74571F, 0xFF8F692A, 0xFFC3903D, 0xFFC3861C, 0xFFEAB648, 0xFFAB8729, 0xFF634C00, 0xFF68540D, 0xFF73560A, 0xFF785909, 0xFF675106, 0xFF564D0A, 0xFF5D4D11,
        0xFF756C1F, 0xFFB2AB77, 0xFFC3B3A6, 0xFF988259, 0xFF988318, 0xFFEADB76, 0xFFE4D6B1, 0xFF868849, 0xFF57562A, 0xFF8F8D67, 0xFF918F66, 0xFF7F7F67, 0xFF828187, 0xFF363B41, 0xFF434E3D, 0xFF42564D, 0xFF4C5F5B, 0xFF415556, 0xFF3A4F54, 0xFF768D95,
        0xFFA5BCC4, 0xFF90A9AE, 0xFF829EA2, 0xFF3E5050, 0xFF4E6062, 0xFF425458, 0xFF47565D, 0xFF49585D, 0xFF4A5859, 0xFF576663, 0xFF24332C, 0xFF2A3143, 0xFF9CA495, 0xFF6D7453, 0xFF6B7059, 0xFFA7A888, 0xFFA4964F, 0xFFC29A29, 0xFFD99C1D, 0xFFB2831B,
        0xFFB8871E, 0xFFB78516, 0xFFCB9825, 0xFFE1AD35, 0xFFE3AD33, 0xFFE4AC31, 0xFFE0A92B, 0xFFE1A533, 0xFFEBAE43, 0xFFBE8723, 0xFFCA9930, 0xFFD3991E, 0xFFEAAC33, 0xFFAE8938, 0xFF6D6A47, 0xFF93936D, 0xFFC2BA8B, 0xFFAFAB7C, 0xFF848F6D, 0xFF828F75,
        0xFFA7A993, 0xFF393B2E, 0xFF445255, 0xFF4D5D5D, 0xFF4B5B5B, 0xFF485858, 0xFF485858, 0xFF4C5C5C, 0xFF4F5F5F, 0xFF4F5F5F, 0xFF4D5D5D, 0xFF495958, 0xFF495958, 0xFF485857, 0xFF445453, 0xFF415150, 0xFF415150, 0xFF425251, 0xFF445453, 0xFF43524F,
        0xFF41504D, 0xFF41504D, 0xFF41514E, 0xFF425250, 0xFF415150, 0xFF445453, 0xFF475756, 0xFF465655, 0xFF45534F, 0xFF435348, 0xFF435348, 0xFF41524D, 0xFF445453, 0xFF445452, 0xFF495553, 0xFF3B4A47, 0xFF42514E, 0xFF41504D, 0xFF445452, 0xFF445453,
        0xFF445453, 0xFF445453, 0xFF455554, 0xFF455554, 0xFF455554, 0xFF455554, 0xFF455554, 0xFF415150, 0xFF445453, 0xFF495958, 0xFF3E4E4D, 0xFF3D4D4C, 0xFF415150, 0xFF445453, 0xFF455554, 0xFF435352, 0xFF425251, 0xFF445453, 0xFF465655, 0xFF465655,
        0xFF445453, 0xFF425251, 0xFF445453, 0xFF485857, 0xFF495958, 0xFF465655, 0xFF435352, 0xFF40504F, 0xFF415150, 0xFF435352, 0xFF455554, 0xFF465655, 0xFF455554, 0xFF425251, 0xFF3F4F4E, 0xFF344533, 0xFF25322B, 0xFF8E9897, 0xFF848C7D, 0xFF727E5A,
        0xFF8D9A6E, 0xFF8A9779, 0xFF95A193, 0xFFA6A473, 0xFF948834, 0xFFCBAB30, 0xFFD8A518, 0xFFDD9D17, 0xFFDC9B23, 0xFFD29925, 0xFFD09F26, 0xFF906308, 0xFF3A3D2A, 0xFF394B57, 0xFF404B45, 0xFF384B45, 0xFF3B4A47, 0xFF404740, 0xFF374C47, 0xFF445449,
        0xFF425247, 0xFF405045, 0xFF415146, 0xFF445449, 0xFF46564B, 0xFF47574C, 0xFF48584D, 0xFF47574C, 0xFF445449, 0xFF415146, 0xFF445449, 0xFF48584D, 0xFF4A5A4F, 0xFF46564B, 0xFF415146, 0xFF44554F, 0xFF3F504A, 0xFF384943, 0xFF334340, 0xFF32423F,
        0xFF344443, 0xFF384847, 0xFF3A4A49, 0xFF354846, 0xFF364947, 0xFF364947, 0xFF374744, 0xFF364643, 0xFF3A4944, 0xFF3F4E49, 0xFF43524D, 0xFF42514A, 0xFF42514A, 0xFF42514C, 0xFF3E4D48, 0xFF42514C, 0xFF43524F, 0xFF394845, 0xFF3A4946, 0xFF354846,
        0xFF374546, 0xFF394843, 0xFF35493E, 0xFF354B49, 0xFF323B42, 0xFF372F1A, 0xFF533F02, 0xFF634A08, 0xFF644B09, 0xFF644B09, 0xFF654C0A, 0xFF664D0B, 0xFF674E0C, 0xFF664D0B, 0xFF664D0B, 0xFF664D0B, 0xFF664D0B, 0xFF664D0B, 0xFF664D0B, 0xFF664D0B,
        0xFF664D0B, 0xFF664D0B, 0xFF664D0B, 0xFF664D0B, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF664D0B, 0xFF684F0D, 0xFF684F0D, 0xFF684F0D, 0xFF684F0D, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF684C03,
        0xFF644D0A, 0xFF654F13, 0xFF695010, 0xFF70540B, 0xFF6E590A, 0xFF736D1F, 0xFF8A9549, 0xFF242223, 0xFF000100, 0xFF111A17, 0xFF22312E, 0xFF2B3B38, 0xFF2E3E3D, 0xFF2D3B3B, 0xFF223030, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FEFEFE, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF4E5955, 0xFF40514B, 0xFF050F06, 0xFF101211, 0xFF2E313A, 0xFF243026, 0xFF586935, 0xFF6F772A, 0xFF6F691B, 0xFF6E5C1C, 0xFF897F4C, 0xFFB8B799, 0xFFABA895, 0xFFB4AF85,
        0xFF988F4A, 0xFF9E9459, 0xFFA09275, 0xFF82846F, 0xFFC1BA8E, 0xFF897737, 0xFF6A540B, 0xFF755D17, 0xFF76590D, 0xFFC19937, 0xFFB98811, 0xFFDEAB2E, 0xFFECBF4A, 0xFFDCB74D, 0xFFDBB957, 0xFFD3B14F, 0xFFCEAB45, 0xFFAA8A23, 0xFF937610, 0xFF694B02,
        0xFF9E8D48, 0xFFE5DBA8, 0xFFA4926A, 0xFF98834E, 0xFF6F601F, 0xFFB2A982, 0xFFBFB5B6, 0xFFDFDAD4, 0xFFC7C88E, 0xFF90923E, 0xFF9D9B62, 0xFF88826C, 0xFF807E72, 0xFF84877E, 0xFF2F3535, 0xFF3A4B43, 0xFF415450, 0xFF405252, 0xFF43565C, 0xFF3D5158,
        0xFF6E858B, 0xFF9DB6BB, 0xFF647D81, 0xFF425454, 0xFF455759, 0xFF4A595C, 0xFF404F54, 0xFF505E5F, 0xFF485754, 0xFF2F3F35, 0xFF334036, 0xFF849992, 0xFF616F55, 0xFF787C5B, 0xFFA29E7B, 0xFF9D9155, 0xFFB69933, 0xFFE8B037, 0xFFEB9A27, 0xFFCA9626,
        0xFFBC8818, 0xFFC28F1C, 0xFFD7A22E, 0xFFE6AF3A, 0xFFE5AF35, 0xFFDEA729, 0xFFDEA728, 0xFFE0A62B, 0xFFE9B13A, 0xFFB8881A, 0xFFC99E2E, 0xFFD59F21, 0xFFE6A724, 0xFFBB9028, 0xFF5C510F, 0xFF7B7C5A, 0xFFA79E73, 0xFF918D60, 0xFF838E6E, 0xFF86947B,
        0xFF9EA28B, 0xFF3C4032, 0xFF475659, 0xFF485858, 0xFF4A5A5A, 0xFF4A5A5A, 0xFF495959, 0xFF495959, 0xFF495959, 0xFF465656, 0xFF425252, 0xFF415150, 0xFF435352, 0xFF455554, 0xFF485857, 0xFF475756, 0xFF445453, 0xFF425251, 0xFF40504F, 0xFF43524F,
        0xFF485754, 0xFF42514E, 0xFF3C4E4A, 0xFF43524F, 0xFF425251, 0xFF455554, 0xFF475756, 0xFF465656, 0xFF40504E, 0xFF40504D, 0xFF41514F, 0xFF425250, 0xFF445452, 0xFF44544F, 0xFF4E5D56, 0xFF45544D, 0xFF4C5952, 0xFF424F49, 0xFF465551, 0xFF435352,
        0xFF455554, 0xFF465655, 0xFF445453, 0xFF425251, 0xFF415150, 0xFF445453, 0xFF475756, 0xFF465655, 0xFF45534F, 0xFF435348, 0xFF435348, 0xFF41524D, 0xFF455554, 0xFF485857, 0xFF485857, 0xFF465655, 0xFF445453, 0xFF455554, 0xFF475756, 0xFF445453,
        0xFF445453, 0xFF445453, 0xFF455554, 0xFF455554, 0xFF455554, 0xFF455554, 0xFF455554, 0xFF415150, 0xFF415150, 0xFF435352, 0xFF455554, 0xFF475756, 0xFF465655, 0xFF435352, 0xFF40504F, 0xFF435444, 0xFF24302C, 0xFF828B8A, 0xFF939B8C, 0xFF77815C,
        0xFF839062, 0xFF919F7C, 0xFF798474, 0xFF8C947F, 0xFF4C4A11, 0xFF998320, 0xFFD2A62D, 0xFFD99B24, 0xFFDC9626, 0xFFDD9A27, 0xFFD4961D, 0xFF92650A, 0xFF3B3E2B, 0xFF394B55, 0xFF404B45, 0xFF374B42, 0xFF3C4B48, 0xFF424941, 0xFF3A4F48, 0xFF445449,
        0xFF425247, 0xFF415146, 0xFF425247, 0xFF45554A, 0xFF48584D, 0xFF4A5A4F, 0xFF4B5B50, 0xFF45554A, 0xFF415146, 0xFF3E4E43, 0xFF405045, 0xFF45554A, 0xFF47574C, 0xFF46564B, 0xFF435348, 0xFF3A4B43, 0xFF384943, 0xFF364741, 0xFF354640, 0xFF364643,
        0xFF374744, 0xFF364645, 0xFF354544, 0xFF374A48, 0xFF374A48, 0xFF364945, 0xFF374744, 0xFF374842, 0xFF3C4B46, 0xFF44534E, 0xFF4B5A53, 0xFF3F4E47, 0xFF42514A, 0xFF44534C, 0xFF3B4A45, 0xFF41504B, 0xFF495853, 0xFF3F4E4B, 0xFF3D4C49, 0xFF364947,
        0xFF384647, 0xFF394843, 0xFF364A3F, 0xFF354B49, 0xFF333C43, 0xFF372F1A, 0xFF544003, 0xFF624B08, 0xFF634C09, 0xFF644D0A, 0xFF654E0B, 0xFF654E0B, 0xFF654E0B, 0xFF654E0B, 0xFF654E0B, 0xFF664D0B, 0xFF664D0B, 0xFF664D0B, 0xFF664D0B, 0xFF664D0B,
        0xFF664D0B, 0xFF664D0B, 0xFF664D0B, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF664D0B, 0xFF664D0B, 0xFF664D0B, 0xFF654C0A, 0xFF654C0A, 0xFF674B02,
        0xFF634C09, 0xFF665014, 0xFF695010, 0xFF70540B, 0xFF6D5809, 0xFF6F691B, 0xFF8A9549, 0xFF222021, 0xFF000100, 0xFF151E1B, 0xFF293835, 0xFF2D3D3A, 0xFF2E3E3D, 0xFF2E3C3C, 0xFF223030, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FDFDFD, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF525D59, 0xFF485855, 0xFF09130A, 0xFF111312, 0xFF31343D, 0xFF2B372D, 0xFF5E6F39, 0xFF788132, 0xFF716C1B, 0xFF806926, 0xFF7E6C2C, 0xFF8D7F4E, 0xFF85764D, 0xFF8D7E47,
        0xFF6C5C0F, 0xFF7F6C28, 0xFF9F885E, 0xFF8F8550, 0xFFC8B76F, 0xFFAD953B, 0xFF776003, 0xFF766410, 0xFF675500, 0xFFA08720, 0xFFC6A62B, 0xFFC9A022, 0xFFE6BD41, 0xFFE0B640, 0xFFECC455, 0xFFECC75D, 0xFFE1BC51, 0xFFDFB642, 0xFFD7AB32, 0xFF865D01,
        0xFFA18C4D, 0xFFF8EFAA, 0xFFD5CA60, 0xFF998B2A, 0xFFB2A988, 0xFFCDCAD1, 0xFFB0ABAF, 0xFF9E9BA2, 0xFFC1C19D, 0xFF8F9044, 0xFFA39F55, 0xFF989163, 0xFF95907D, 0xFF6A6B63, 0xFF7A8179, 0xFF36463B, 0xFF3F4E47, 0xFF40504F, 0xFF4A595C, 0xFF42535A,
        0xFF4B5E64, 0xFF516669, 0xFF3A4F50, 0xFF4B5B5B, 0xFF3A4A4A, 0xFF49585B, 0xFF4D5B5C, 0xFF4A5858, 0xFF37463F, 0xFF313F32, 0xFF839180, 0xFF68845B, 0xFF6A725A, 0xFFA49777, 0xFF97833C, 0xFFCAB04F, 0xFFBA9433, 0xFFAD7516, 0xFFF8B34B, 0xFFDFAB31,
        0xFFCD961F, 0xFFD8A12C, 0xFFE2AB37, 0xFFE1AA35, 0xFFE2AA33, 0xFFDCA527, 0xFFE3AC2D, 0xFFE5AC29, 0xFFE3AD33, 0xFFAC8317, 0xFFBF9C36, 0xFFD9A832, 0xFFEAA722, 0xFFDFA72E, 0xFFA08421, 0xFF7D703B, 0xFF927F45, 0xFF7D7340, 0xFF828B6C, 0xFF889782,
        0xFF9CA18D, 0xFF3C4032, 0xFF475756, 0xFF485858, 0xFF495959, 0xFF4A5A5A, 0xFF495959, 0xFF485858, 0xFF485858, 0xFF495959, 0xFF495959, 0xFF435352, 0xFF455554, 0xFF435352, 0xFF485857, 0xFF485857, 0xFF435352, 0xFF435352, 0xFF415150, 0xFF43524F,
        0xFF43524F, 0xFF3F4E4B, 0xFF41514E, 0xFF445453, 0xFF455554, 0xFF465655, 0xFF475756, 0xFF445454, 0xFF42524F, 0xFF42524F, 0xFF41514F, 0xFF415150, 0xFF425250, 0xFF42524F, 0xFF41514E, 0xFF41514E, 0xFF42524F, 0xFF435350, 0xFF445452, 0xFF465655,
        0xFF465655, 0xFF455554, 0xFF435352, 0xFF425251, 0xFF425251, 0xFF455554, 0xFF475756, 0xFF465656, 0xFF40504E, 0xFF40504D, 0xFF41514F, 0xFF425250, 0xFF445453, 0xFF465655, 0xFF465655, 0xFF445453, 0xFF435352, 0xFF445453, 0xFF455554, 0xFF435352,
        0xFF455554, 0xFF465655, 0xFF445453, 0xFF425251, 0xFF415150, 0xFF445453, 0xFF475756, 0xFF465655, 0xFF455554, 0xFF455554, 0xFF465655, 0xFF485857, 0xFF485857, 0xFF455554, 0xFF435352, 0xFF404D43, 0xFF202B25, 0xFF88928A, 0xFF7A8372, 0xFF7D8462,
        0xFF919465, 0xFF646430, 0xFF7B7946, 0xFF86866A, 0xFF575121, 0xFF5F4B02, 0xFFA38022, 0xFFD09F38, 0xFFD19429, 0xFFDD9C28, 0xFFDD981F, 0xFF976A0D, 0xFF3E412C, 0xFF3B4D57, 0xFF404B43, 0xFF364A41, 0xFF3A4944, 0xFF414840, 0xFF394E47, 0xFF435349,
        0xFF435349, 0xFF45554B, 0xFF47574D, 0xFF49594F, 0xFF4A5A50, 0xFF4A5A50, 0xFF49594F, 0xFF47544B, 0xFF455249, 0xFF435047, 0xFF435047, 0xFF455249, 0xFF47544B, 0xFF49564D, 0xFF49564D, 0xFF43544C, 0xFF3F5048, 0xFF3A4B45, 0xFF374842, 0xFF374842,
        0xFF364643, 0xFF344441, 0xFF334340, 0xFF364945, 0xFF354844, 0xFF354844, 0xFF364741, 0xFF364741, 0xFF394843, 0xFF42514A, 0xFF4B5A53, 0xFF47544B, 0xFF48554C, 0xFF49564F, 0xFF3D4A43, 0xFF424E4A, 0xFF4A5652, 0xFF3D4945, 0xFF3B4745, 0xFF394C4A,
        0xFF394748, 0xFF384742, 0xFF33473C, 0xFF334947, 0xFF323B42, 0xFF3A321D, 0xFF584407, 0xFF624A0A, 0xFF634B0B, 0xFF654D0D, 0xFF654D0D, 0xFF654D0D, 0xFF644C0C, 0xFF654D0D, 0xFF664E0E, 0xFF664D0B, 0xFF664D0B, 0xFF664D0B, 0xFF664D0B, 0xFF664D0B,
        0xFF664D0B, 0xFF664D0B, 0xFF664D0B, 0xFF684F0D, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF684F0D, 0xFF684F0D, 0xFF674E0C, 0xFF664D0B, 0xFF654C0A, 0xFF654C0A, 0xFF654C0A, 0xFF654C0A, 0xFF654C0A, 0xFF694D04,
        0xFF654E0B, 0xFF685216, 0xFF684F0F, 0xFF6E5209, 0xFF6A5506, 0xFF6A6416, 0xFF8A9549, 0xFF252324, 0xFF000100, 0xFF17201D, 0xFF2B3A37, 0xFF2E3E3B, 0xFF2E3E3D, 0xFF2F3D3D, 0xFF233131, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FDFDFD, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF56615D, 0xFF495956, 0xFF0A140C, 0xFF141615, 0xFF373A43, 0xFF364236, 0xFF687942, 0xFF818A39, 0xFF736E1A, 0xFF81631D, 0xFF84681E, 0xFF826722, 0xFF745A1F, 0xFF816825,
        0xFF7B630D, 0xFF7C6112, 0xFF7A5923, 0xFF725A10, 0xFF9B7D25, 0xFFC8A746, 0xFF7F640B, 0xFF867530, 0xFF968A4E, 0xFF8F803B, 0xFFA48F3C, 0xFFB59F30, 0xFFC5A737, 0xFFAC881A, 0xFF8B6802, 0xFF876A0C, 0xFF9A7B1E, 0xFFCCA33A, 0xFFDEA937, 0xFFD09E3B,
        0xFF665200, 0xFF8D882C, 0xFFF3E383, 0xFFB6A35F, 0xFFAEAB98, 0xFFBBC4BF, 0xFF979C86, 0xFF4C543F, 0xFF9C9C94, 0xFFE4DEC4, 0xFFAFAA6A, 0xFF8F8C47, 0xFF8D8A67, 0xFF7B7F6E, 0xFF6E7B61, 0xFF687365, 0xFF3D493F, 0xFF414C48, 0xFF4C5858, 0xFF404E51,
        0xFF4D5C5F, 0xFF445656, 0xFF425553, 0xFF435352, 0xFF495959, 0xFF425252, 0xFF556364, 0xFF21302D, 0xFF38453C, 0xFF808E7F, 0xFF72816E, 0xFF5D6D48, 0xFFBAB28D, 0xFFB69757, 0xFFB68E2C, 0xFFCCA53E, 0xFF97731F, 0xFF633F00, 0xFFAD8B32, 0xFFE6AF2E,
        0xFFDCA527, 0xFFE1A932, 0xFFE3AA37, 0xFFE0A734, 0xFFE0A831, 0xFFE0A92B, 0xFFE5AE2F, 0xFFE7AC2C, 0xFFE0AD38, 0xFF9F7C1E, 0xFF927726, 0xFFC09232, 0xFFEAA327, 0xFFF4AF2D, 0xFFE0AF38, 0xFFB69641, 0xFFA98635, 0xFF948043, 0xFF8B9273, 0xFF869787,
        0xFFA2AB9A, 0xFF383E30, 0xFF42534D, 0xFF4A5A5A, 0xFF475757, 0xFF455555, 0xFF445454, 0xFF435353, 0xFF435353, 0xFF455555, 0xFF485858, 0xFF495958, 0xFF495958, 0xFF425251, 0xFF455554, 0xFF445453, 0xFF3E4E4D, 0xFF445453, 0xFF435352, 0xFF404F4C,
        0xFF41504D, 0xFF455452, 0xFF465653, 0xFF415150, 0xFF425351, 0xFF425251, 0xFF435351, 0xFF435351, 0xFF42524F, 0xFF435350, 0xFF415150, 0xFF435351, 0xFF425250, 0xFF41514E, 0xFF41514E, 0xFF41514E, 0xFF42524F, 0xFF435350, 0xFF445451, 0xFF485857,
        0xFF475756, 0xFF435352, 0xFF425251, 0xFF435352, 0xFF455554, 0xFF465655, 0xFF475756, 0xFF445454, 0xFF42524F, 0xFF42524F, 0xFF41514F, 0xFF415150, 0xFF415150, 0xFF41514F, 0xFF41514F, 0xFF41514F, 0xFF425251, 0xFF435352, 0xFF445453, 0xFF465655,
        0xFF465655, 0xFF455554, 0xFF435352, 0xFF425251, 0xFF425251, 0xFF455554, 0xFF475756, 0xFF475757, 0xFF465656, 0xFF455555, 0xFF455555, 0xFF455555, 0xFF455555, 0xFF455555, 0xFF445454, 0xFF3D4746, 0xFF263129, 0xFF8B9789, 0xFF909986, 0xFF7D8063,
        0xFF8A8251, 0xFF857528, 0xFF816D0C, 0xFF88712D, 0xFF6B5313, 0xFF594101, 0xFF684E03, 0xFFA28424, 0xFFD1A737, 0xFFC5911B, 0xFFDB9F27, 0xFF96690C, 0xFF3E412C, 0xFF3B4D57, 0xFF3F4A42, 0xFF35493E, 0xFF394843, 0xFF40473F, 0xFF384D46, 0xFF42514C,
        0xFF44534E, 0xFF465550, 0xFF475651, 0xFF475651, 0xFF465550, 0xFF45544F, 0xFF44534E, 0xFF4B5851, 0xFF4B5851, 0xFF4B5851, 0xFF4A5750, 0xFF48554E, 0xFF48554E, 0xFF4B5851, 0xFF4D5A53, 0xFF4D5E56, 0xFF45564E, 0xFF3C4D45, 0xFF364741, 0xFF354640,
        0xFF354542, 0xFF354542, 0xFF354542, 0xFF354844, 0xFF344743, 0xFF354844, 0xFF384943, 0xFF364741, 0xFF37463F, 0xFF3D4C45, 0xFF45544D, 0xFF4C5950, 0xFF455249, 0xFF48554C, 0xFF414E47, 0xFF45524B, 0xFF47534F, 0xFF35413D, 0xFF394541, 0xFF344745,
        0xFF344243, 0xFF33423D, 0xFF2F4338, 0xFF2F4543, 0xFF2F383F, 0xFF372F1A, 0xFF554104, 0xFF5F4A09, 0xFF614C0B, 0xFF634E0D, 0xFF634E0D, 0xFF624D0C, 0xFF624D0C, 0xFF634E0D, 0xFF644F0E, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C,
        0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF684F0D, 0xFF684F0D, 0xFF674E0C, 0xFF664D0B, 0xFF664D0B, 0xFF674E0C, 0xFF684F0D, 0xFF684F0D, 0xFF69500E, 0xFF684F0D, 0xFF674E0C, 0xFF664D0B, 0xFF654C0A, 0xFF654C0A, 0xFF654C0A, 0xFF664D0B, 0xFF6A4E05,
        0xFF654E0B, 0xFF675115, 0xFF654C0C, 0xFF6D5108, 0xFF6A5506, 0xFF6B6517, 0xFF919C50, 0xFF2E2C2D, 0xFF000100, 0xFF131C19, 0xFF283734, 0xFF2C3C39, 0xFF2D3D3C, 0xFF2F3D3D, 0xFF212F2F, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FDFDFD, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF596460, 0xFF445451, 0xFF09130B, 0xFF181A19, 0xFF3C3F48, 0xFF3C483C, 0xFF6C7D46, 0xFF868F3C, 0xFF736E1A, 0xFF8D6B23, 0xFF886513, 0xFF896517, 0xFF896622, 0xFF87671C,
        0xFF83650B, 0xFF886813, 0xFF865E20, 0xFF89651B, 0xFF906813, 0xFFCEA64F, 0xFFB99853, 0xFFE0CDA5, 0xFFFFF3E0, 0xFFFAEED6, 0xFFFEEECC, 0xFFD2CB70, 0xFFB4A142, 0xFFCDB052, 0xFF91761F, 0xFF715B0F, 0xFF745D11, 0xFF8E660E, 0xFFE1AC46, 0xFFDDA546,
        0xFF6E5F00, 0xFF89861F, 0xFFD1B399, 0xFFFAD8E6, 0xFFC7C8B8, 0xFFB2C89A, 0xFF424F23, 0xFF788267, 0xFFB5B8A7, 0xFFB0A9A3, 0xFFF4E8D8, 0xFF7A753E, 0xFF424900, 0xFF828F59, 0xFF7B877D, 0xFF81897A, 0xFF6C7467, 0xFF333D35, 0xFF3E4847, 0xFF4E5A5A,
        0xFF485656, 0xFF465552, 0xFF475754, 0xFF455554, 0xFF4C5C5C, 0xFF485657, 0xFF2C3A3A, 0xFF303C38, 0xFF7E8B81, 0xFF707E6D, 0xFF6A7763, 0xFFAEABA4, 0xFF9D894A, 0xFFC3982F, 0xFFD19733, 0xFFC89641, 0xFF7D610E, 0xFF584B00, 0xFF6D6524, 0xFFE5AC27,
        0xFFE7AD2F, 0xFFE1A62E, 0xFFDFA632, 0xFFE1A835, 0xFFE1A932, 0xFFE1AA2C, 0xFFDCA524, 0xFFE4A82C, 0xFFE5B247, 0xFF977828, 0xFF5C4406, 0xFF986A1C, 0xFFE19525, 0xFFEC9D1A, 0xFFE5A927, 0xFFC99D32, 0xFFAE8221, 0xFFA68E48, 0xFF8D9274, 0xFF7C8E82,
        0xFFA4AFA1, 0xFF373F30, 0xFF48584E, 0xFF455555, 0xFF425252, 0xFF435353, 0xFF485858, 0xFF4A5A5A, 0xFF475757, 0xFF455555, 0xFF465656, 0xFF465655, 0xFF485857, 0xFF425251, 0xFF475756, 0xFF455554, 0xFF3C4C4B, 0xFF40504F, 0xFF425251, 0xFF465654,
        0xFF41514F, 0xFF435352, 0xFF435352, 0xFF425251, 0xFF4A5A59, 0xFF465655, 0xFF41504E, 0xFF41514E, 0xFF42514F, 0xFF40504E, 0xFF40504F, 0xFF40504D, 0xFF40504D, 0xFF41514E, 0xFF41514E, 0xFF41514E, 0xFF41524E, 0xFF41514E, 0xFF425250, 0xFF455554,
        0xFF435352, 0xFF445453, 0xFF435352, 0xFF425251, 0xFF425251, 0xFF435352, 0xFF435352, 0xFF435351, 0xFF42524F, 0xFF435350, 0xFF415150, 0xFF435351, 0xFF435351, 0xFF41514F, 0xFF41514F, 0xFF41514D, 0xFF435351, 0xFF465654, 0xFF485857, 0xFF495958,
        0xFF475756, 0xFF435352, 0xFF425251, 0xFF435352, 0xFF455554, 0xFF465655, 0xFF475756, 0xFF465656, 0xFF455555, 0xFF435353, 0xFF425252, 0xFF415151, 0xFF425252, 0xFF425252, 0xFF435353, 0xFF3F494B, 0xFF29362C, 0xFF899885, 0xFF838C77, 0xFF7E7C63,
        0xFFA2925F, 0xFFA78D2E, 0xFFB99815, 0xFFC1962E, 0xFFAF8936, 0xFF725617, 0xFF5C4905, 0xFF5C4B00, 0xFF9E8511, 0xFFD2AA30, 0xFFC5921C, 0xFF936609, 0xFF3B3E29, 0xFF394C53, 0xFF3F4A42, 0xFF364A3F, 0xFF3A4944, 0xFF40483D, 0xFF384D46, 0xFF3F4E4B,
        0xFF404F4C, 0xFF3F4E4B, 0xFF3D4C49, 0xFF3B4A47, 0xFF3A4946, 0xFF3C4B48, 0xFF3D4C49, 0xFF3D4945, 0xFF404C48, 0xFF424E4A, 0xFF404C48, 0xFF3B4743, 0xFF394541, 0xFF3B4743, 0xFF3E4A46, 0xFF405147, 0xFF3B4C44, 0xFF36473F, 0xFF35463E, 0xFF364741,
        0xFF374842, 0xFF364643, 0xFF344441, 0xFF354844, 0xFF354844, 0xFF384B45, 0xFF3B4C46, 0xFF384941, 0xFF36453E, 0xFF3A4942, 0xFF415147, 0xFF424F46, 0xFF36433A, 0xFF3D4A41, 0xFF3F4C45, 0xFF46534C, 0xFF44514A, 0xFF333F3B, 0xFF3D4945, 0xFF344745,
        0xFF374546, 0xFF394843, 0xFF364A3F, 0xFF374D4B, 0xFF353E45, 0xFF3A321D, 0xFF574306, 0xFF5F4A09, 0xFF614C0B, 0xFF634E0D, 0xFF634E0D, 0xFF624D0C, 0xFF624D0C, 0xFF634E0D, 0xFF644F0E, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF674E0C,
        0xFF674E0C, 0xFF674E0C, 0xFF674E0C, 0xFF69500E, 0xFF684F0D, 0xFF674E0C, 0xFF664D0B, 0xFF664D0B, 0xFF674E0C, 0xFF684F0D, 0xFF69500E, 0xFF6B5210, 0xFF6A510F, 0xFF684F0D, 0xFF674E0C, 0xFF664D0B, 0xFF664D0B, 0xFF664D0B, 0xFF674E0C, 0xFF684C03,
        0xFF624B08, 0xFF654F13, 0xFF634A0A, 0xFF6C5007, 0xFF6D5809, 0xFF6F691B, 0xFF9AA559, 0xFF363435, 0xFF000201, 0xFF0F1815, 0xFF23322F, 0xFF293936, 0xFF2D3D3C, 0xFF2F3D3D, 0xFF202E2E, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FCFFFD, 0x00FCFEFB, 0x00FFFFFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF58635D, 0xFF596756, 0xFF141F19, 0xFF0D141A, 0xFF33363D, 0xFF494A42, 0xFF7C7E56, 0xFF878B3F, 0xFF6E7613, 0xFF896E27, 0xFF856A23, 0xFF886D26, 0xFF896E27, 0xFF836821,
        0xFF836821, 0xFF856A23, 0xFF81661F, 0xFF866311, 0xFF8C7518, 0xFFCFC57D, 0xFFFFFBE0, 0xFFFFFFF8, 0xFFFEFFF3, 0xFFFDFDF3, 0xFFFFFAFF, 0xFFF8FFF6, 0xFFEAF0C2, 0xFFB5AE56, 0xFFD1BA4A, 0xFF9E7A08, 0xFF876100, 0xFF815E00, 0xFFCFB046, 0xFFD7C34A,
        0xFF675500, 0xFFA1986D, 0xFFC0BEAF, 0xFF9FA696, 0xFFBBC1A7, 0xFF515532, 0xFF7B7C54, 0xFFC0B3BA, 0xFFC1BBBF, 0xFF9E988C, 0xFFBFB182, 0xFFC1A858, 0xFF5F4000, 0xFF6E551C, 0xFFAD9E81, 0xFF7D8075, 0xFF787B70, 0xFF7A7D74, 0xFF3B4039, 0xFF3D4840,
        0xFF43544C, 0xFF41584E, 0xFF3F594E, 0xFF3C5C67, 0xFF43565A, 0xFF333A33, 0xFF33392F, 0xFF8E988F, 0xFF6F7B6D, 0xFF6A725B, 0xFFA7A888, 0xFF9D832E, 0xFF8C7B2B, 0xFF8D8232, 0xFFD9CD77, 0xFFBEA74A, 0xFF977A1E, 0xFF644A00, 0xFF7A6419, 0xFFDCAB32,
        0xFFDEAA32, 0xFFE2AA31, 0xFFE5A931, 0xFFE3A72F, 0xFFE1A62E, 0xFFDFA72C, 0xFFDFA92D, 0xFFE2A121, 0xFFE8AD37, 0xFFA57918, 0xFF5C4400, 0xFF5A5012, 0xFF7E6C24, 0xFFC59938, 0xFFE7A52D, 0xFFC99E2B, 0xFFA68C1E, 0xFF969140, 0xFF87936F, 0xFF7F9387,
        0xFFA2B4A4, 0xFF383E30, 0xFF585252, 0xFF465656, 0xFF485858, 0xFF485858, 0xFF465656, 0xFF475757, 0xFF4A5A5A, 0xFF4A5A5A, 0xFF485858, 0xFF445453, 0xFF485857, 0xFF495958, 0xFF3C4C4B, 0xFF3F4F4E, 0xFF445453, 0xFF445453, 0xFF425251, 0xFF475756,
        0xFF425251, 0xFF475756, 0xFF455554, 0xFF3C4C4B, 0xFF40504F, 0xFF425251, 0xFF465654, 0xFF41514F, 0xFF435452, 0xFF4B5B5A, 0xFF435352, 0xFF41514E, 0xFF40504D, 0xFF41514E, 0xFF42524F, 0xFF44544F, 0xFF42524D, 0xFF40504B, 0xFF3F504D, 0xFF445453,
        0xFF435352, 0xFF425251, 0xFF445453, 0xFF465655, 0xFF465655, 0xFF445453, 0xFF415150, 0xFF41514F, 0xFF41514E, 0xFF40504E, 0xFF40504F, 0xFF40504E, 0xFF40504D, 0xFF40504D, 0xFF40504D, 0xFF41524C, 0xFF40514C, 0xFF3E4F4B, 0xFF40504F, 0xFF425251,
        0xFF435352, 0xFF445453, 0xFF435352, 0xFF425251, 0xFF425251, 0xFF435352, 0xFF445453, 0xFF465656, 0xFF455555, 0xFF455555, 0xFF465656, 0xFF475757, 0xFF475757, 0xFF455555, 0xFF435353, 0xFF46474C, 0xFF2F3A2A, 0xFF83957F, 0xFF7D8D83, 0xFF747C6D,
        0xFF999661, 0xFF95842A, 0xFFA1841C, 0xFFB28C1F, 0xFFBD922A, 0xFFB58930, 0xFF745409, 0xFF544300, 0xFF625204, 0xFF977E17, 0xFFCAA62B, 0xFF906D11, 0xFF453921, 0xFF39423F, 0xFF404B3A, 0xFF3D4641, 0xFF334549, 0xFF374E48, 0xFF3E4C4C, 0xFF3C4B44,
        0xFF3F4E47, 0xFF3A4942, 0xFF3C4B44, 0xFF3B4A43, 0xFF394841, 0xFF3D4C45, 0xFF3A4942, 0xFF3C4B48, 0xFF3A4946, 0xFF394845, 0xFF3C4B48, 0xFF3B4A47, 0xFF384744, 0xFF374643, 0xFF384744, 0xFF394845, 0xFF364542, 0xFF374643, 0xFF374643, 0xFF344340,
        0xFF364542, 0xFF374643, 0xFF31403D, 0xFF374643, 0xFF344340, 0xFF344340, 0xFF374641, 0xFF394843, 0xFF394841, 0xFF394841, 0xFF3C4B44, 0xFF374641, 0xFF35443F, 0xFF364540, 0xFF374641, 0xFF374641, 0xFF384742, 0xFF384742, 0xFF35443F, 0xFF35444B,
        0xFF3B4743, 0xFF38453B, 0xFF34453F, 0xFF304445, 0xFF2A352D, 0xFF39330F, 0xFF5A4506, 0xFF5B4708, 0xFF5D4809, 0xFF5F4A0B, 0xFF614C0D, 0xFF654D0F, 0xFF654D0F, 0xFF664C0F, 0xFF664C0F, 0xFF614D04, 0xFF634F08, 0xFF6A530F, 0xFF604903, 0xFF654D05,
        0xFF644A00, 0xFF694E07, 0xFF6B500B, 0xFF684D08, 0xFF634602, 0xFF6D4F0F, 0xFF68480B, 0xFF68480D, 0xFF6F5111, 0xFF654802, 0xFF6B5104, 0xFF684E04, 0xFF664B04, 0xFF674E0B, 0xFF634A08, 0xFF604808, 0xFF654E0B, 0xFF624B07, 0xFF614D08, 0xFF665112,
        0xFF664816, 0xFF6A471D, 0xFF704C1A, 0xFF6A4F0A, 0xFF6B5F0D, 0xFF6B6C1C, 0xFF9BA358, 0xFF424335, 0xFF000300, 0xFF0E1C0F, 0xFF273B2F, 0xFF1F362C, 0xFF2A3F3A, 0xFF2C3C3C, 0xFF263133, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FBFFFC, 0x00FBFBF9, 0x00FEFEFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF636E68, 0xFF5E6C5B, 0xFF061109, 0xFF11191C, 0xFF3B3E45, 0xFF41423A, 0xFF74764F, 0xFF7F853D, 0xFF6F7619, 0xFF896A1A, 0xFF876818, 0xFF8B6C1C, 0xFF8D6E1E, 0xFF896A1A,
        0xFF8A6B1B, 0xFF8B6C1C, 0xFF876818, 0xFF8B6929, 0xFF9A8244, 0xFFF4E7BA, 0xFFFFFFEC, 0xFFE4E4E2, 0xFFF2F4F3, 0xFFFAF8FD, 0xFFFFFBFF, 0xFFFBFFFF, 0xFFF6F5E3, 0xFFF0E5AF, 0xFFA79145, 0xFFC9AA57, 0xFF916C1C, 0xFF7E5908, 0xFFC3A04C, 0xFFDCB93B,
        0xFF896A0D, 0xFF725D28, 0xFFBFB59C, 0xFFCAC8B9, 0xFF737663, 0xFF878876, 0xFFCAC9B7, 0xFFA9AC81, 0xFF898E70, 0xFF262A13, 0xFF68603B, 0xFFC0AD6B, 0xFF6E5302, 0xFF6D5608, 0xFFA1904B, 0xFF848969, 0xFF797F63, 0xFF737864, 0xFF787E72, 0xFF38413C,
        0xFF3D4B4B, 0xFF445658, 0xFF405357, 0xFF425A5E, 0xFF344340, 0xFF34372E, 0xFF919386, 0xFF6D7162, 0xFF7B7E63, 0xFFA7A074, 0xFF938048, 0xFF9F811F, 0xFF826F20, 0xFF938852, 0xFFB2A47D, 0xFFB39C72, 0xFF8E6F39, 0xFF583A00, 0xFF846B28, 0xFFDCAB32,
        0xFFDEAA32, 0xFFE2AA33, 0xFFE5A931, 0xFFE5A931, 0xFFE4AA2F, 0xFFE0A82D, 0xFFDBA529, 0xFFE9AA29, 0xFFE7AD32, 0xFFCF9E35, 0xFF7E5E09, 0xFF5A4800, 0xFF675408, 0xFF916F15, 0xFFC19027, 0xFFDDAA3D, 0xFFC6A438, 0xFFB1A251, 0xFF9A9B73, 0xFF889382,
        0xFFA0AD9B, 0xFF363E2F, 0xFF565658, 0xFF4A5A5A, 0xFF4A5A5A, 0xFF485858, 0xFF445454, 0xFF435353, 0xFF475757, 0xFF495959, 0xFF485858, 0xFF445453, 0xFF455554, 0xFF475756, 0xFF445453, 0xFF445453, 0xFF425251, 0xFF435353, 0xFF445453, 0xFF485857,
        0xFF495958, 0xFF3C4C4B, 0xFF3F4F4E, 0xFF445453, 0xFF445453, 0xFF425251, 0xFF475756, 0xFF425251, 0xFF475756, 0xFF455554, 0xFF3C4C4B, 0xFF40504F, 0xFF41514E, 0xFF435350, 0xFF435350, 0xFF44554F, 0xFF42534D, 0xFF3F504A, 0xFF3E4F4A, 0xFF445452,
        0xFF435352, 0xFF435352, 0xFF455554, 0xFF485857, 0xFF485857, 0xFF445453, 0xFF415150, 0xFF42524F, 0xFF425250, 0xFF3E4E4D, 0xFF435351, 0xFF40504D, 0xFF40504D, 0xFF41514E, 0xFF41514E, 0xFF44554F, 0xFF41524C, 0xFF3F4F4A, 0xFF3E4F4C, 0xFF445452,
        0xFF435352, 0xFF425251, 0xFF435352, 0xFF455554, 0xFF455554, 0xFF435352, 0xFF415150, 0xFF475757, 0xFF465656, 0xFF465656, 0xFF465656, 0xFF475757, 0xFF475757, 0xFF465656, 0xFF465656, 0xFF41444D, 0xFF2C3729, 0xFF82917A, 0xFF808D7C, 0xFF7A7C64,
        0xFFA29659, 0xFF9A8023, 0xFFA07B11, 0xFFB68A11, 0xFFC2931D, 0xFFC09127, 0xFFB18C33, 0xFF70590D, 0xFF5A4A00, 0xFF634D00, 0xFF9F8322, 0xFF90751C, 0xFF4D4634, 0xFF3E494B, 0xFF3B4837, 0xFF3B4441, 0xFF36484C, 0xFF394E49, 0xFF3F4B49, 0xFF45544D,
        0xFF46554E, 0xFF404F48, 0xFF42514A, 0xFF415049, 0xFF3E4D46, 0xFF42514A, 0xFF404F48, 0xFF42514C, 0xFF42514C, 0xFF43524D, 0xFF45544F, 0xFF44534E, 0xFF404F4A, 0xFF3E4D48, 0xFF404F4A, 0xFF485752, 0xFF43524D, 0xFF45544F, 0xFF465550, 0xFF43524D,
        0xFF42514C, 0xFF41504B, 0xFF3D4C47, 0xFF3F4E4B, 0xFF3B4A47, 0xFF394845, 0xFF3C4B46, 0xFF3D4C47, 0xFF3C4B44, 0xFF3C4B44, 0xFF3D4C45, 0xFF3D4C47, 0xFF3E4D48, 0xFF3D4C47, 0xFF3D4C47, 0xFF3E4D48, 0xFF3B4A45, 0xFF374641, 0xFF35443F, 0xFF36454C,
        0xFF3C4844, 0xFF38453B, 0xFF354640, 0xFF314546, 0xFF2C372F, 0xFF3B3511, 0xFF5B4607, 0xFF5B4708, 0xFF5E490A, 0xFF604B0C, 0xFF614C0D, 0xFF654D0F, 0xFF654D0F, 0xFF664C0F, 0xFF664C0F, 0xFF654D11, 0xFF685014, 0xFF695017, 0xFF614B11, 0xFF6A5418,
        0xFF665014, 0xFF6A5619, 0xFF5C480D, 0xFF644E14, 0xFF684F17, 0xFF654A15, 0xFF6C4E1C, 0xFF684A18, 0xFF654A13, 0xFF6E561A, 0xFF644F10, 0xFF6A4C06, 0xFF684B07, 0xFF6B4E0C, 0xFF6F5111, 0xFF6B4F0E, 0xFF674B0A, 0xFF69500E, 0xFF6B5210, 0xFF6F580A,
        0xFF6A4D0B, 0xFF6C490F, 0xFF6F4C0C, 0xFF684D00, 0xFF675901, 0xFF696712, 0xFF979B4E, 0xFF312E43, 0xFF000012, 0xFF111B27, 0xFF293740, 0xFF233539, 0xFF304040, 0xFF2A3936, 0xFF232D2C, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FEFFFD, 0x00FDFDFB, 0x00FEFDFB, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF636E68, 0xFF6E7B6A, 0xFF040E06, 0xFF14191C, 0xFF3E4245, 0xFF42433B, 0xFF767A59, 0xFF7E8245, 0xFF6B7224, 0xFF917424, 0xFF8D7020, 0xFF8B6E1E, 0xFF896C1C, 0xFF856818,
        0xFF866919, 0xFF886B1B, 0xFF866919, 0xFF785D26, 0xFFB9A37C, 0xFFFFFEE4, 0xFFD8D6BF, 0xFFB3B5A8, 0xFFF8F9FD, 0xFFFFFEFF, 0xFFFEFDF9, 0xFFF3F4E6, 0xFFFEFEDC, 0xFFFFFFC6, 0xFF9B9243, 0xFFB8A853, 0xFF9F872F, 0xFF745600, 0xFFCDAE51, 0xFFF4C750,
        0xFF7F5800, 0xFF684A0C, 0xFF8A7A56, 0xFFAEA892, 0xFFB7B5A8, 0xFF97978F, 0xFFC9C8C4, 0xFFA3A576, 0xFF343A0E, 0xFF68703F, 0xFF99995B, 0xFF756A1A, 0xFF564000, 0xFF82661C, 0xFF997B3F, 0xFF424A11, 0xFF848A5A, 0xFF8C926E, 0xFF6D725E, 0xFF7A817A,
        0xFF2A3137, 0xFF4C545F, 0xFF3C4451, 0xFF35413D, 0xFF262D25, 0xFF9EA294, 0xFF6E7263, 0xFF72765F, 0xFF9F9A70, 0xFFA18845, 0xFFB28832, 0xFFC09E32, 0xFF8B7623, 0xFF8A834F, 0xFF4C4727, 0xFF574829, 0xFF664D24, 0xFF5A3B05, 0xFF836529, 0xFFDDAC35,
        0xFFE0AC36, 0xFFE2AA33, 0xFFE2A62E, 0xFFE4A830, 0xFFE6AC31, 0xFFE1AA2C, 0xFFDBA527, 0xFFE2A727, 0xFFE1A628, 0xFFE5AC38, 0xFFC99D34, 0xFF7C5D03, 0xFF584300, 0xFF917D27, 0xFF98832A, 0xFFB88A2C, 0xFFBB9737, 0xFFA99347, 0xFF978E65, 0xFF8B8D77,
        0xFF9CA68E, 0xFF333E2E, 0xFF4F5457, 0xFF495959, 0xFF4A5A5A, 0xFF485858, 0xFF445454, 0xFF435353, 0xFF465656, 0xFF475757, 0xFF455555, 0xFF475756, 0xFF425251, 0xFF3D4D4C, 0xFF435352, 0xFF415150, 0xFF435353, 0xFF475756, 0xFF445453, 0xFF455554,
        0xFF475756, 0xFF445453, 0xFF445453, 0xFF425251, 0xFF435353, 0xFF445453, 0xFF485857, 0xFF495958, 0xFF3C4C4B, 0xFF3F4F4E, 0xFF445453, 0xFF445453, 0xFF41514F, 0xFF42524F, 0xFF42524F, 0xFF42534D, 0xFF42534D, 0xFF41524C, 0xFF465752, 0xFF435451,
        0xFF445453, 0xFF435352, 0xFF465655, 0xFF455554, 0xFF445453, 0xFF415150, 0xFF3F4F4E, 0xFF445452, 0xFF455553, 0xFF435351, 0xFF435351, 0xFF435350, 0xFF435350, 0xFF435350, 0xFF435350, 0xFF44554F, 0xFF42534D, 0xFF3F504A, 0xFF3E4F4A, 0xFF445452,
        0xFF435352, 0xFF415150, 0xFF435352, 0xFF475756, 0xFF485857, 0xFF445453, 0xFF40504F, 0xFF475756, 0xFF475756, 0xFF475756, 0xFF465655, 0xFF465655, 0xFF465655, 0xFF475756, 0xFF485857, 0xFF41434F, 0xFF293426, 0xFF829076, 0xFF869078, 0xFF83835F,
        0xFFADA05C, 0xFFAB8D33, 0xFFB48B2F, 0xFFBC9111, 0xFFB98808, 0xFFCE9C23, 0xFFC09427, 0xFFAE8C32, 0xFF695206, 0xFF574000, 0xFF5E4704, 0xFF6D5807, 0xFF3E3E32, 0xFF394C52, 0xFF36483C, 0xFF3A4644, 0xFF384A4E, 0xFF384B45, 0xFF3D4643, 0xFF47564F,
        0xFF485750, 0xFF42514A, 0xFF43524B, 0xFF42514A, 0xFF3F4E47, 0xFF43524B, 0xFF415049, 0xFF425248, 0xFF425248, 0xFF45554B, 0xFF48584E, 0xFF46564C, 0xFF415147, 0xFF3F4F45, 0xFF425248, 0xFF405046, 0xFF39493F, 0xFF3C4C42, 0xFF435349, 0xFF415147,
        0xFF3F4F45, 0xFF3F4F45, 0xFF3E4E44, 0xFF3C4B48, 0xFF374643, 0xFF374643, 0xFF3C4B46, 0xFF42514C, 0xFF44534C, 0xFF44534C, 0xFF45544D, 0xFF3F4E49, 0xFF44534E, 0xFF42514C, 0xFF42514C, 0xFF465550, 0xFF404F4A, 0xFF364540, 0xFF364540, 0xFF37464D,
        0xFF3C4844, 0xFF38453B, 0xFF354640, 0xFF334748, 0xFF2E3931, 0xFF3B3511, 0xFF5A4506, 0xFF5C4809, 0xFF5E490A, 0xFF604B0C, 0xFF624D0E, 0xFF654D0F, 0xFF654D0F, 0xFF664C0F, 0xFF654B0E, 0xFF644C04, 0xFF614900, 0xFF5E4B00, 0xFF806E22, 0xFF908237,
        0xFF595005, 0xFF625B0D, 0xFF7E7928, 0xFF5F530B, 0xFF675813, 0xFF604D0B, 0xFF5D4904, 0xFF624E05, 0xFF5E4C02, 0xFF5A4C01, 0xFF584C04, 0xFF6B5001, 0xFF674C00, 0xFF654B00, 0xFF6E5407, 0xFF6A5307, 0xFF634C00, 0xFF6E570B, 0xFF624C01, 0xFF614B00,
        0xFF664C00, 0xFF72540E, 0xFF71540E, 0xFF654F04, 0xFF605506, 0xFF6B691E, 0xFF989E58, 0xFF1B201C, 0xFF000200, 0xFF141F1B, 0xFF253431, 0xFF273535, 0xFF364143, 0xFF2D363B, 0xFF252A30, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00F7F9F4, 0x00F5F6F1, 0x00EFEEEA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF656F67, 0xFF5C6757, 0xFF151C14, 0xFF1E2422, 0xFF3C3E3D, 0xFF393A32, 0xFF62654A, 0xFF6D7041, 0xFF656B2D, 0xFF816D32, 0xFF7E6A2F, 0xFF7C682D, 0xFF7B672C, 0xFF7C682D,
        0xFF7E6A2F, 0xFF816D32, 0xFF816D32, 0xFF7A6B32, 0xFFC7B899, 0xFFFDF5DE, 0xFFB4B68F, 0xFFABB18F, 0xFFF6F9F0, 0xFFF0F2E7, 0xFFDCE1C1, 0xFFCAC99B, 0xFFD3D39D, 0xFFD0D593, 0xFF9FA357, 0xFF888536, 0xFFA29747, 0xFF7B6615, 0xFFCBB15D, 0xFFE3C167,
        0xFF8C6F21, 0xFF796528, 0xFF82774A, 0xFF878262, 0xFFBABAA2, 0xFF969785, 0xFF6B6C5C, 0xFF6B562B, 0xFF443403, 0xFFB4AA6D, 0xFF8E863D, 0xFFA59643, 0xFF816816, 0xFF805E17, 0xFFAD8548, 0xFFADAD65, 0xFFA7A769, 0xFF8A8C5D, 0xFF7F8061, 0xFF7B7C6A,
        0xFF6B6861, 0xFF1D1814, 0xFF3B3633, 0xFF121607, 0xFF6F7364, 0xFF757C6C, 0xFF6B7665, 0xFF9CA68B, 0xFFA19A66, 0xFFC29F4B, 0xFFBE851A, 0xFFC1962D, 0xFFDBC467, 0xFFD4D589, 0xFF565F1C, 0xFF3E3D00, 0xFF5D4C04, 0xFF634600, 0xFF83621D, 0xFFDCAB35,
        0xFFE1AC38, 0xFFE2A935, 0xFFE0A32E, 0xFFDFA329, 0xFFE3A92C, 0xFFE3AC2D, 0xFFDFAA2A, 0xFFE4AA2C, 0xFFE1A523, 0xFFDF9F21, 0xFFECB139, 0xFFBF9328, 0xFF9D8326, 0xFFDED182, 0xFFD7D389, 0xFFB49B59, 0xFFC4B067, 0xFFAA9E61, 0xFF9B9573, 0xFF939580,
        0xFFA1A992, 0xFF343F2F, 0xFF444F51, 0xFF455555, 0xFF485858, 0xFF495959, 0xFF475757, 0xFF475757, 0xFF475757, 0xFF455555, 0xFF647171, 0xFFD9DCDC, 0xFFF5F6F6, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5,
        0xFFF4F5F4, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F4, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5,
        0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F4, 0xFFF4F5F5, 0xFFF4F5F5,
        0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF4F5F5, 0xFFF5F6F6, 0xFFDDE0DF, 0xFF687574, 0xFF465655, 0xFF465655, 0xFF465655, 0xFF465655, 0xFF475756, 0xFF454753, 0xFF273126, 0xFF7F8D76, 0xFF808C76, 0xFF717954,
        0xFF939154, 0xFF988A41, 0xFFAD9652, 0xFFAE8C20, 0xFFBC9422, 0xFFBE9018, 0xFFC3921C, 0xFFB98D22, 0xFFAA852C, 0xFF7A5C13, 0xFF5E4203, 0xFF504300, 0xFF2E332C, 0xFF344A55, 0xFF364A3F, 0xFF3C4848, 0xFF36484C, 0xFF354940, 0xFF3F4843, 0xFF44534C,
        0xFF46554E, 0xFF415049, 0xFF43524B, 0xFF43524B, 0xFF404F48, 0xFF44534C, 0xFF415049, 0xFF3F4F45, 0xFF405046, 0xFF435349, 0xFF45554B, 0xFF425248, 0xFF3D4D43, 0xFF3E4E44, 0xFF415147, 0xFF4A5A50, 0xFF415147, 0xFF435349, 0xFF49594F, 0xFF44544A,
        0xFF3D4D43, 0xFF3B4B41, 0xFF3A4A40, 0xFF3F4E4B, 0xFF394845, 0xFF374643, 0xFF3C4B46, 0xFF41504B, 0xFF42514A, 0xFF415049, 0xFF415049, 0xFF364540, 0xFF3F4E49, 0xFF3D4C47, 0xFF3C4B46, 0xFF44534E, 0xFF3D4C47, 0xFF31403B, 0xFF33423D, 0xFF36454C,
        0xFF3A4642, 0xFF364339, 0xFF34453F, 0xFF334748, 0xFF2E3931, 0xFF3B3511, 0xFF584304, 0xFF5C4809, 0xFF5E490A, 0xFF604B0C, 0xFF624D0E, 0xFF654D0F, 0xFF654D0F, 0xFF654B0E, 0xFF654B0E, 0xFF6F540F, 0xFF614A00, 0xFF614F00, 0xFFB0A354, 0xFFCDC57D,
        0xFF5F5C17, 0xFF707227, 0xFFCCD17F, 0xFF6F6F29, 0xFF534E0C, 0xFF665B1B, 0xFF5C4D08, 0xFF5B4E00, 0xFF7A7221, 0xFF77732A, 0xFF7C7B3B, 0xFF8E7B41, 0xFF86763A, 0xFF5E4E10, 0xFF675719, 0xFF8B7D3E, 0xFF77682D, 0xFF75692F, 0xFF7C6F38, 0xFF6D6117,
        0xFF5D4E0B, 0xFF504004, 0xFF4D3E07, 0xFF594F1C, 0xFF676535, 0xFF676E42, 0xFF778258, 0xFF303936, 0xFF1B2421, 0xFF212A27, 0xFF1E2724, 0xFF1D2623, 0xFF2D3331, 0xFF272B2A, 0xFF242625, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0xFFCBCCC7, 0xFFD4D3CF, 0xFFDAD6D3, 0xFFCBCAC6, 0xFFD3D4CF, 0xFFBCC3BC, 0xFFB8C2BA, 0xFFB4BCAF, 0xFFADB3A9, 0xFFB0B5AF, 0xFFB7B9B6, 0xFFAAADA4, 0xFFBCBFAA, 0xFFCBCFAE, 0xFFD1D6AD, 0xFFCCC6AC, 0xFFCBC5AB, 0xFFC9C3A9, 0xFFC9C3A9, 0xFFCBC5AB,
        0xFFCAC4AA, 0xFFC7C1A7, 0xFFC6C0A6, 0xFFC8C099, 0xFFDBD4C2, 0xFFEEEBDC, 0xFFE0E4C1, 0xFFE8EECC, 0xFFF0F4E6, 0xFFD1D5C7, 0xFFC2CAA5, 0xFFD5D0B2, 0xFFC8C7A9, 0xFFCBCEB3, 0xFFC3C7AC, 0xFFBCBCA2, 0xFFD2CBB1, 0xFFCCBAA4, 0xFFD8BEAD, 0xFFCFC592,
        0xFF9C9561, 0xFF72703D, 0xFF686A3B, 0xFF5F6139, 0xFF4E5029, 0xFF92946C, 0xFF7D8053, 0xFF9A7525, 0xFFBA9648, 0xFF72510A, 0xFF624503, 0xFF6C510C, 0xFFAB8E3E, 0xFF9C7A18, 0xFFCAA231, 0xFFD5C77C, 0xFF847A35, 0xFF938A51, 0xFF928A5C, 0xFF6A5E38,
        0xFF51421B, 0xFF67542A, 0xFF382100, 0xFF1B1800, 0xFF1D1C00, 0xFF585E44, 0xFF8E9D86, 0xFF92A184, 0xFFB1AE77, 0xFFAB8930, 0xFFD19825, 0xFFD6A33A, 0xFF987B1F, 0xFFA5A356, 0xFFD8E299, 0xFFBDC06F, 0xFF9E9037, 0xFF8C7316, 0xFF816303, 0xFFD4A22F,
        0xFFDCA735, 0xFFE2A935, 0xFFE2A530, 0xFFDFA329, 0xFFE0A629, 0xFFE1AA29, 0xFFE0AB29, 0xFFE0A628, 0xFFE4A728, 0xFFE6A523, 0xFFE09F1D, 0xFFDEA62D, 0xFFBD982E, 0xFF988833, 0xFF75722B, 0xFF918F66, 0xFFAAAA76, 0xFF939667, 0xFF888A72, 0xFF898D7E,
        0xFF9BA48F, 0xFF344130, 0xFF475353, 0xFF435353, 0xFF475757, 0xFF485858, 0xFF475757, 0xFF465656, 0xFF475757, 0xFF455555, 0xFFD8DBDB, 0xFFD3D7D7, 0xFF818C8B, 0xFF7A8685, 0xFF7E8988, 0xFF778382, 0xFF7A8585, 0xFF778383, 0xFF7D8888, 0xFF7C8887,
        0xFF798483, 0xFF808B8B, 0xFF7B8686, 0xFF7A8686, 0xFF7A8686, 0xFF7C8887, 0xFF798483, 0xFF758180, 0xFF7A8584, 0xFF788383, 0xFF7A8585, 0xFF7C8887, 0xFF7A8685, 0xFF7A8685, 0xFF7D8888, 0xFF788381, 0xFF7A8583, 0xFF7A8683, 0xFF7B8785, 0xFF788382,
        0xFF7B8686, 0xFF7C8786, 0xFF7B8686, 0xFF798483, 0xFF788383, 0xFF778382, 0xFF798484, 0xFF727D7D, 0xFF768181, 0xFF798483, 0xFF798482, 0xFF788381, 0xFF788381, 0xFF778381, 0xFF778381, 0xFF77837F, 0xFF798581, 0xFF7A8681, 0xFF7A8783, 0xFF7A8583,
        0xFF7B8686, 0xFF7B8686, 0xFF7A8584, 0xFF788383, 0xFF798483, 0xFF7B8686, 0xFF7D8888, 0xFF7C8786, 0xFFD2D6D6, 0xFFD8DCDB, 0xFF465655, 0xFF465655, 0xFF455554, 0xFF435352, 0xFF425251, 0xFF464950, 0xFF273128, 0xFF82907F, 0xFF829383, 0xFF68785D,
        0xFF808E5B, 0xFF88915A, 0xFFA6A77F, 0xFF796415, 0xFFA2872E, 0xFFB59028, 0xFFBB8E19, 0xFFBC8913, 0xFFC89627, 0xFFB4892A, 0xFF9E7A26, 0xFF675A0C, 0xFF383D36, 0xFF344A55, 0xFF34483D, 0xFF3B4747, 0xFF334549, 0xFF34483F, 0xFF424B46, 0xFF404F48,
        0xFF44534C, 0xFF415049, 0xFF45544D, 0xFF46554E, 0xFF43524B, 0xFF46554E, 0xFF43524B, 0xFF425248, 0xFF425248, 0xFF435349, 0xFF44544A, 0xFF415147, 0xFF3E4E44, 0xFF405046, 0xFF45554B, 0xFF4B5B51, 0xFF425248, 0xFF435349, 0xFF49594F, 0xFF45554B,
        0xFF3D4D43, 0xFF3B4B41, 0xFF3A4A40, 0xFF3E4D4A, 0xFF384744, 0xFF364542, 0xFF3B4A45, 0xFF404F4A, 0xFF415049, 0xFF404F48, 0xFF404F48, 0xFF384742, 0xFF41504B, 0xFF3E4D48, 0xFF3E4D48, 0xFF465550, 0xFF3F4E49, 0xFF33423D, 0xFF364540, 0xFF334249,
        0xFF384440, 0xFF344137, 0xFF33443E, 0xFF324647, 0xFF2D3830, 0xFF39330F, 0xFF574203, 0xFF5D490A, 0xFF5F4A0B, 0xFF604B0C, 0xFF624D0E, 0xFF644C0E, 0xFF644C0E, 0xFF654B0E, 0xFF644A0D, 0xFF603E1B, 0xFF6A4E1F, 0xFF5D4610, 0xFF8F7F4C, 0xFFC8BC96,
        0xFF938E6E, 0xFFA8A97F, 0xFFC1C892, 0xFFB9BB96, 0xFF767153, 0xFFA19376, 0xFFA39169, 0xFF897945, 0xFFBEB57E, 0xFFD6D2AC, 0xFFC7C7AF, 0xFFA8A089, 0xFF9C967C, 0xFF5F5A3C, 0xFF817D5A, 0xFFEAE7C6, 0xFFDEDABD, 0xFFC3C1AA, 0xFFC4C1AE, 0xFFC2C195,
        0xFFB1AF86, 0xFF8F8F6B, 0xFF616449, 0xFF4F5344, 0xFF4A554D, 0xFF475754, 0xFF485C5A, 0xFF525D63, 0xFF4F595A, 0xFF5E695B, 0xFF697255, 0xFF7F865A, 0xFF9CA669, 0xFFA2AC63, 0xFFA3AF5D, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0xFF8E8F8A, 0xFFB4B3AF, 0xFFEBE7E4, 0xFFEDE9E6, 0xFFEDEEE9, 0xFFEFF4EE, 0xFFF9FFFB, 0xFFF8FEF2, 0xFFFCFFF8, 0xFFF4F7F0, 0xFFF6F9F2, 0xFFF8FBF2, 0xFFFFFFF3, 0xFFFFFFED, 0xFFF6FAE3, 0xFFFCFFFB, 0xFFFAFFF9, 0xFFF0F5EF, 0xFFEAEFE9, 0xFFEAEFE9,
        0xFFE8EDE7, 0xFFE5EAE4, 0xFFE5EAE4, 0xFFF9F6ED, 0xFFEFEEEC, 0xFFF2F1EF, 0xFFF9FBEE, 0xFFF6FAEC, 0xFFE8EAE5, 0xFFC5C8C1, 0xFF9AA191, 0xFFB3AE9A, 0xFFAFADA1, 0xFFA3A5A2, 0xFFA6AAAB, 0xFFAAAEAD, 0xFFB4B2B3, 0xFFB9AFB8, 0xFF9E90A1, 0xFF7F836A,
        0xFF4F5332, 0xFF4C5129, 0xFF585D34, 0xFF858760, 0xFF8B8D65, 0xFF505421, 0xFF4F5318, 0xFF906C18, 0xFFEEC76A, 0xFFDFB851, 0xFFBD9732, 0xFFA38223, 0xFF90700D, 0xFFB28F1B, 0xFFC79F1B, 0xFFA1822F, 0xFF654A00, 0xFF99813B, 0xFF947F3C, 0xFF604705,
        0xFF7D5F16, 0xFFB48E3B, 0xFF9E7219, 0xFF796425, 0xFF4B3A02, 0xFF4E4A1B, 0xFF8E9671, 0xFF89956F, 0xFF545219, 0xFFB5953E, 0xFFCC9425, 0xFFDBA636, 0xFF84600A, 0xFF62571F, 0xFFA3A276, 0xFF7F7844, 0xFF87732A, 0xFFAE9433, 0xFFAD8F1F, 0xFFC99726,
        0xFFD29D2B, 0xFFDEA532, 0xFFE5A833, 0xFFE4A82E, 0xFFE0A628, 0xFFDDA625, 0xFFDDA824, 0xFFDFA327, 0xFFE2A62A, 0xFFE3A627, 0xFFE1A21F, 0xFFE9AA29, 0xFFDEAB38, 0xFFB69536, 0xFF5E4B00, 0xFF6C6D43, 0xFF8C915B, 0xFF828A5B, 0xFF7D856D, 0xFF7E8979,
        0xFF95A28E, 0xFF323F2E, 0xFF495354, 0xFF475757, 0xFF485858, 0xFF475757, 0xFF435353, 0xFF435353, 0xFF465656, 0xFF475757, 0xFFF8F9F9, 0xFF798483, 0xFF455554, 0xFF40504F, 0xFF3F4F4E, 0xFF3E4E4D, 0xFF435353, 0xFF425252, 0xFF465655, 0xFF495958,
        0xFF445453, 0xFF495958, 0xFF40504F, 0xFF435353, 0xFF405050, 0xFF485857, 0xFF475756, 0xFF425251, 0xFF4C5C5B, 0xFF455554, 0xFF445454, 0xFF445454, 0xFF475756, 0xFF435352, 0xFF445453, 0xFF435350, 0xFF455552, 0xFF465653, 0xFF455653, 0xFF425251,
        0xFF445453, 0xFF455554, 0xFF445453, 0xFF425251, 0xFF425251, 0xFF415150, 0xFF425252, 0xFF415151, 0xFF3E4E4D, 0xFF3F4F4F, 0xFF435352, 0xFF40504F, 0xFF3F4F4E, 0xFF435352, 0xFF485857, 0xFF41514E, 0xFF435350, 0xFF445451, 0xFF455653, 0xFF41514F,
        0xFF455554, 0xFF465655, 0xFF445453, 0xFF40504F, 0xFF40504F, 0xFF445453, 0xFF495958, 0xFF42524F, 0xFF798482, 0xFFF8F9F9, 0xFF465653, 0xFF475754, 0xFF455552, 0xFF41514E, 0xFF3E4E4B, 0xFF3F4847, 0xFF243026, 0xFF829083, 0xFF809086, 0xFF5F7159,
        0xFF718353, 0xFF6F7B49, 0xFF818867, 0xFF504002, 0xFF76641C, 0xFFA78D36, 0xFFB48E23, 0xFFCB971F, 0xFFB78107, 0xFFBD8D1E, 0xFFB58E29, 0xFF836E1D, 0xFF414135, 0xFF33464C, 0xFF35473B, 0xFF3C4846, 0xFF34464A, 0xFF334640, 0xFF3D4643, 0xFF394841,
        0xFF3D4C45, 0xFF3B4A43, 0xFF3F4E47, 0xFF404F48, 0xFF3E4D46, 0xFF42514A, 0xFF3F4E47, 0xFF41504B, 0xFF3F4E49, 0xFF3F4E49, 0xFF404F4A, 0xFF3E4D48, 0xFF3C4B46, 0xFF3F4E49, 0xFF43524D, 0xFF45544F, 0xFF3E4D48, 0xFF41504B, 0xFF485752, 0xFF465550,
        0xFF43524D, 0xFF44534E, 0xFF42514C, 0xFF3B4A47, 0xFF364542, 0xFF354441, 0xFF3A4944, 0xFF3F4E49, 0xFF415049, 0xFF415049, 0xFF42514A, 0xFF3C4B46, 0xFF41504B, 0xFF3F4E49, 0xFF3E4D48, 0xFF42514C, 0xFF3E4D48, 0xFF35443F, 0xFF374641, 0xFF314047,
        0xFF36423E, 0xFF344137, 0xFF32433D, 0xFF314546, 0xFF2C372F, 0xFF38320E, 0xFF564102, 0xFF5D490A, 0xFF5F4A0B, 0xFF604B0C, 0xFF624D0E, 0xFF644C0E, 0xFF644C0E, 0xFF644A0D, 0xFF644A0D, 0xFF674721, 0xFF6D521D, 0xFF594506, 0xFF67581F, 0xFF7D714B,
        0xFF4E4828, 0xFF6B6940, 0xFF676A31, 0xFF646C47, 0xFF444325, 0xFF6D6244, 0xFF837246, 0xFF6F6122, 0xFF7F793B, 0xFF8A8B63, 0xFF767A69, 0xFF535432, 0xFF5C5E37, 0xFF4F5527, 0xFF59602D, 0xFF808855, 0xFF878E62, 0xFF959B77, 0xFF767D5E, 0xFF696A58,
        0xFF6D705B, 0xFF666D5B, 0xFF4B554D, 0xFF414E56, 0xFF405165, 0xFF40556A, 0xFF3A5163, 0xFF435353, 0xFF46524E, 0xFF586051, 0xFF6A6E53, 0xFF85865C, 0xFFA0A167, 0xFFA2A862, 0xFFA3AA5C, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0xFF63645E, 0xFF7E7B76, 0xFFBFBAB6, 0xFFCDCAC5, 0xFFD3D4CE, 0xFFDFE4DD, 0xFFEDF4EC, 0xFFEDF0E5, 0xFFDEE1D6, 0xFFD2D5CA, 0xFFD9DCD3, 0xFFDFE2D9, 0xFFEFF2E9, 0xFFE2E5DA, 0xFFD4D7CC, 0xFFD2D9D2, 0xFFCFD6CF, 0xFFC5CCC5, 0xFFC0C7C0, 0xFFC3CAC3,
        0xFFC3CAC3, 0xFFC1C8C1, 0xFFC3CAC3, 0xFFDDE0E5, 0xFFE3E8E2, 0xFFE4E7E0, 0xFFD8DAD7, 0xFFD0D2CF, 0xFFD0D4C6, 0xFFADB3A5, 0xFF707571, 0xFF919179, 0xFF9D9F91, 0xFF8A8C87, 0xFF868D86, 0xFF808879, 0xFF818A79, 0xFF7A7F78, 0xFF75767A, 0xFF554F37,
        0xFF676242, 0xFF74704B, 0xFF797454, 0xFF827C64, 0xFFBBB79E, 0xFF82825C, 0xFF9CA06E, 0xFFB4A770, 0xFFB19C49, 0xFFD1B343, 0xFFE2C24B, 0xFFC7A93B, 0xFFB0902D, 0xFFC79E35, 0xFFC69423, 0xFFBE912C, 0xFFA67B1C, 0xFF916C13, 0xFFB7953E, 0xFFC7A249,
        0xFFC29530, 0xFFDAA231, 0xFFD3951C, 0xFFCEA040, 0xFFA47B21, 0xFF644800, 0xFF655B1E, 0xFF5E5D27, 0xFF493E00, 0xFF9F7C22, 0xFFCF972A, 0xFFCD9327, 0xFFAA822D, 0xFF534008, 0xFF4B3F17, 0xFF544317, 0xFF5C4303, 0xFF755604, 0xFFB49538, 0xFFC69425,
        0xFFCA9525, 0xFFD49B28, 0xFFE2A530, 0xFFE5A92F, 0xFFE0A628, 0xFFDCA622, 0xFFDCA821, 0xFFE5A42C, 0xFFE0A52D, 0xFFD9A329, 0xFFE6AB2D, 0xFFE2A11F, 0xFFDE9E23, 0xFFE0AB41, 0xFF8B6004, 0xFF7B6823, 0xFF92863C, 0xFF8B8748, 0xFF838464, 0xFF808775,
        0xFF96A48D, 0xFF303C2E, 0xFF454F51, 0xFF4A5A5A, 0xFF4A5A5A, 0xFF475757, 0xFF435353, 0xFF425252, 0xFF455555, 0xFF475757, 0xFFF4F5F5, 0xFF778382, 0xFF445453, 0xFF465655, 0xFF435353, 0xFF465656, 0xFF475757, 0xFF465656, 0xFF425251, 0xFF465655,
        0xFF40504F, 0xFF3F4F4E, 0xFF3E4E4D, 0xFF435353, 0xFF425252, 0xFF465655, 0xFF495958, 0xFF445453, 0xFF495958, 0xFF40504F, 0xFF435353, 0xFF405050, 0xFF485857, 0xFF475756, 0xFF435352, 0xFF435350, 0xFF445451, 0xFF455552, 0xFF485855, 0xFF435351,
        0xFF425251, 0xFF435352, 0xFF455554, 0xFF465655, 0xFF455554, 0xFF425251, 0xFF3D4D4D, 0xFF405050, 0xFF415151, 0xFF40504F, 0xFF465655, 0xFF445453, 0xFF445453, 0xFF455554, 0xFF455554, 0xFF435350, 0xFF455552, 0xFF465653, 0xFF455653, 0xFF425251,
        0xFF445453, 0xFF455554, 0xFF445453, 0xFF425251, 0xFF425251, 0xFF435352, 0xFF455554, 0xFF465653, 0xFF798482, 0xFFF4F5F5, 0xFF455552, 0xFF475754, 0xFF465653, 0xFF41514E, 0xFF3D4D4A, 0xFF384742, 0xFF233026, 0xFF859088, 0xFF899089, 0xFF75795E,
        0xFF93945A, 0xFF858141, 0xFF807645, 0xFF513D00, 0xFF67550B, 0xFF745F0E, 0xFFAB8A29, 0xFFBA8C1D, 0xFFC4911C, 0xFFB18518, 0xFFAE8B25, 0xFF82670E, 0xFF403927, 0xFF374244, 0xFF3B4837, 0xFF3D4643, 0xFF35474B, 0xFF344944, 0xFF364240, 0xFF35443D,
        0xFF384740, 0xFF34433C, 0xFF37463F, 0xFF384740, 0xFF37463F, 0xFF3C4B44, 0xFF3A4942, 0xFF3B4949, 0xFF394747, 0xFF394747, 0xFF3B4949, 0xFF3A4848, 0xFF384646, 0xFF394747, 0xFF3D4B4B, 0xFF3B4949, 0xFF364444, 0xFF374545, 0xFF394747, 0xFF364444,
        0xFF354343, 0xFF344242, 0xFF303E3E, 0xFF3D4C49, 0xFF394845, 0xFF374643, 0xFF384742, 0xFF394843, 0xFF384740, 0xFF36453E, 0xFF37463F, 0xFF34433E, 0xFF34433E, 0xFF32413C, 0xFF32413C, 0xFF34433E, 0xFF33423D, 0xFF31403B, 0xFF303F3A, 0xFF2F3E45,
        0xFF36423E, 0xFF354238, 0xFF34453F, 0xFF314546, 0xFF2C372F, 0xFF39330F, 0xFF584304, 0xFF5D490A, 0xFF5F4A0B, 0xFF614C0D, 0xFF624D0E, 0xFF644C0E, 0xFF644C0E, 0xFF644A0D, 0xFF644A0D, 0xFF6A5015, 0xFF5D4700, 0xFF5D4E00, 0xFF8E8132, 0xFF988C52,
        0xFF3C3103, 0xFF686427, 0xFFA8A95B, 0xFF5E6B33, 0xFF434715, 0xFF564E1D, 0xFF594A07, 0xFF514600, 0xFF66640D, 0xFF7E8448, 0xFF98A280, 0xFF858353, 0xFF585822, 0xFF3C3E00, 0xFF3A3D00, 0xFF525714, 0xFF616528, 0xFF686C39, 0xFF262900, 0xFF181204,
        0xFF0B0800, 0xFF030600, 0xFF0B120A, 0xFF313B45, 0xFF424E66, 0xFF36465D, 0xFF182939, 0xFF324450, 0xFF25313D, 0xFF222430, 0xFF211B25, 0xFF271C22, 0xFF2F2325, 0xFF291F1D, 0xFF261D18, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0xFF686963, 0xFF817E79, 0xFFBCB7B3, 0xFFCDCAC5, 0xFFD3D2CD, 0xFFDCE1DA, 0xFFE2E9E1, 0xFFE6E8DD, 0xFFD0D2C7, 0xFFDCDFD4, 0xFFDFE2D7, 0xFFC8CBC2, 0xFFD8DBD4, 0xFFE0E2DD, 0xFFECEEEB, 0xFFD1D4C3, 0xFFD4D7C6, 0xFFD2D5C4, 0xFFD3D6C5, 0xFFD6D9C8,
        0xFFD0D3C2, 0xFFC4C7B6, 0xFFC1C4B3, 0xFFD0DAD9, 0xFFD8E4CC, 0xFFDDE6CB, 0xFFD9DCD3, 0xFFD8DBD0, 0xFFC9D0AF, 0xFF9BA485, 0xFF777E76, 0xFF9B9D92, 0xFF9FA19E, 0xFFA2A3A7, 0xFF8C918D, 0xFF939C89, 0xFF7E8A72, 0xFF778473, 0xFF818C86, 0xFF6E5B3A,
        0xFF75633D, 0xFF66542E, 0xFF4C3B21, 0xFF392B20, 0xFF5E554E, 0xFF7D7A67, 0xFF808463, 0xFFC0D19A, 0xFF85853B, 0xFF76620B, 0xFF886718, 0xFF805D1B, 0xFF5D3800, 0xFF895D04, 0xFFD3A02D, 0xFFD5A02C, 0xFFBB881B, 0xFFB3861F, 0xFFA47A16, 0xFFB88822,
        0xFFCB9322, 0xFFE5A122, 0xFFECA319, 0xFFDA981E, 0xFFDCA02E, 0xFFAD7F1F, 0xFF9C8134, 0xFF786926, 0xFF67530A, 0xFFBA9135, 0xFFCF9227, 0xFFC48925, 0xFFB0872F, 0xFF877126, 0xFF60530D, 0xFF5D4B03, 0xFF684C03, 0xFF654200, 0xFF775315, 0xFFCA972A,
        0xFFC69023, 0xFFCB9221, 0xFFDB9E29, 0xFFE3A72D, 0xFFE1A827, 0xFFDDA721, 0xFFDEAA22, 0xFFE4A22A, 0xFFDBA22E, 0xFFDDAA37, 0xFFD59F25, 0xFFE9A624, 0xFFE79E1E, 0xFFDA9726, 0xFFDBA03A, 0xFFC4993B, 0xFFC4A342, 0xFFAB984B, 0xFF948D63, 0xFF898D76,
        0xFF9EAA94, 0xFF333F31, 0xFF464F54, 0xFF495959, 0xFF4B5B5B, 0xFF4A5A5A, 0xFF465656, 0xFF445454, 0xFF455555, 0xFF445454, 0xFFF4F5F5, 0xFF798484, 0xFF435352, 0xFFB7BDBD, 0xFFB9BFBF, 0xFF475757, 0xFF475757, 0xFF465656, 0xFF808A8A, 0xFFD4D7D7,
        0xFF667372, 0xFF435353, 0xFF465656, 0xFF475757, 0xFF465656, 0xFF425251, 0xFF455554, 0xFF40504F, 0xFF3F4F4E, 0xFF3E4E4D, 0xFF435353, 0xFF425252, 0xFF465655, 0xFF475756, 0xFF435352, 0xFF40504F, 0xFF415150, 0xFF435352, 0xFF455554, 0xFF455554,
        0xFF445453, 0xFF3F4F4E, 0xFF3E4E4D, 0xFF425251, 0xFF425251, 0xFF415150, 0xFF445453, 0xFF435251, 0xFF415151, 0xFF3E4E4E, 0xFF455554, 0xFF485857, 0xFF495958, 0xFF475756, 0xFF435352, 0xFF435350, 0xFF445451, 0xFF455552, 0xFF485855, 0xFF435351,
        0xFF425251, 0xFF435352, 0xFF455554, 0xFF465655, 0xFF455554, 0xFF425251, 0xFF40504F, 0xFF495956, 0xFF7C8785, 0xFFF4F5F5, 0xFF445451, 0xFF475754, 0xFF475754, 0xFF42524F, 0xFF3D4D4A, 0xFF364A41, 0xFF223227, 0xFF848E86, 0xFF8A8A82, 0xFF857D59,
        0xFFAEA059, 0xFF9A8333, 0xFF816422, 0xFF947920, 0xFF846E19, 0xFF745F0C, 0xFF84660C, 0xFFB18721, 0xFFB6871D, 0xFFAA841F, 0xFF8B7114, 0xFF805D01, 0xFF43371F, 0xFF3F4845, 0xFF404B3A, 0xFF39423D, 0xFF324448, 0xFF374E48, 0xFF3A4848, 0xFF394841,
        0xFF3B4A43, 0xFF35443D, 0xFF36453E, 0xFF37463F, 0xFF36453E, 0xFF3D4C45, 0xFF3C4B44, 0xFF3A4849, 0xFF374546, 0xFF384647, 0xFF3B494A, 0xFF3B494A, 0xFF394748, 0xFF384647, 0xFF3A4849, 0xFF3C4A4B, 0xFF394748, 0xFF3A4849, 0xFF3A4849, 0xFF374546,
        0xFF394748, 0xFF3A4849, 0xFF344243, 0xFF364542, 0xFF32413E, 0xFF31403D, 0xFF34433E, 0xFF35443F, 0xFF33423B, 0xFF33423B, 0xFF35443D, 0xFF34433E, 0xFF31403B, 0xFF303F3A, 0xFF31403B, 0xFF32413C, 0xFF35443F, 0xFF374641, 0xFF364540, 0xFF2F3E45,
        0xFF37433F, 0xFF37443A, 0xFF354640, 0xFF324647, 0xFF2C372F, 0xFF3A3410, 0xFF594405, 0xFF5E4A0B, 0xFF5F4A0B, 0xFF614C0D, 0xFF624D0E, 0xFF644C0E, 0xFF644C0E, 0xFF644A0D, 0xFF63490C, 0xFF5F4610, 0xFF69550E, 0xFF594900, 0xFF827129, 0xFFCDBD8C,
        0xFFA1936E, 0xFFB1A774, 0xFFC9C27C, 0xFF9FAD7C, 0xFF65673F, 0xFFA2966E, 0xFF9E8E52, 0xFF7C701C, 0xFFAAA553, 0xFFC9CD9A, 0xFFE9F2DD, 0xFFB3A48F, 0xFF817358, 0xFF534925, 0xFF7A7146, 0xFFD3CCA0, 0xFFC2BA96, 0xFFB1A78E, 0xFFAEA594, 0xFF837363,
        0xFF796F54, 0xFF69684A, 0xFF505340, 0xFF474C4F, 0xFF414858, 0xFF4F5965, 0xFF525D63, 0xFF445C4E, 0xFF405141, 0xFF4F5644, 0xFF69674E, 0xFF837D5B, 0xFF968F65, 0xFF989562, 0xFF97945D, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0xFF6B6660, 0xFF8B8881, 0xFFAFAFA7, 0xFFB9BAB2, 0xFFD1D2CA, 0xFFD2D5CC, 0xFFC8CEC4, 0xFFDBE1D5, 0xFFDBE1D5, 0xFFD3D9CD, 0xFFCCD2C8, 0xFFC8CEC4, 0xFFD6DBD4, 0xFFCED3CC, 0xFFD0D5CE, 0xFFD6DCD0, 0xFFE8EEE2, 0xFFE6ECE0, 0xFFD6DCD0, 0xFFD0D6CA,
        0xFFE0E6DA, 0xFFDFE5D9, 0xFFB2B8AC, 0xFFCDD5C8, 0xFFDAE2D5, 0xFFD9E1D4, 0xFFD7DDCF, 0xFFCAD0C2, 0xFFC6CABB, 0xFF8D9182, 0xFF7F8374, 0xFFA2A499, 0xFF93958A, 0xFF9B9D92, 0xFF83857A, 0xFF828479, 0xFF898B80, 0xFF828479, 0xFF818378, 0xFF6E5335,
        0xFF82611E, 0xFF855900, 0xFF754500, 0xFF865508, 0xFFA67415, 0xFFBB800A, 0xFFD99413, 0xFFD9912D, 0xFFECB131, 0xFFDDAD25, 0xFFDBA938, 0xFFC5942B, 0xFFAA8614, 0xFF685600, 0xFF635E1E, 0xFFC8A353, 0xFFD89932, 0xFF995A00, 0xFF553800, 0xFF918548,
        0xFFD0AE64, 0xFFB3781C, 0xFFE29E3D, 0xFFE4A32F, 0xFFD39422, 0xFFE5A939, 0xFFEBB548, 0xFFE0AB41, 0xFFC49128, 0xFFC5912B, 0xFFCC9731, 0xFFB48720, 0xFFA47719, 0xFFB38937, 0xFF98722B, 0xFF694707, 0xFF604405, 0xFF674F11, 0xFF5D4807, 0xFF7E5211,
        0xFFB79333, 0xFFAF8928, 0xFFBE8C29, 0xFFDFA128, 0xFFEBA32B, 0xFFE69F2D, 0xFFE3A419, 0xFFD6AB29, 0xFFDBA624, 0xFFDCA123, 0xFFDCA531, 0xFFC6972D, 0xFFD5A93E, 0xFFE7B03B, 0xFFDE9D1B, 0xFFD1983E, 0xFFAF7B15, 0xFF977939, 0xFF848671, 0xFF829380,
        0xFF9CA298, 0xFF39373C, 0xFF474C48, 0xFF495758, 0xFF485657, 0xFF475555, 0xFF465454, 0xFF455353, 0xFF445350, 0xFF455451, 0xFFF4F5F5, 0xFF798484, 0xFF495958, 0xFFC6CBCB, 0xFFFFFFFF, 0xFFA2AAAA, 0xFF445454, 0xFF425252, 0xFF848E8E, 0xFFFFFFFF,
        0xFFDDDFDF, 0xFF4F5E5E, 0xFF455555, 0xFF475757, 0xFF465656, 0xFF415150, 0xFF445453, 0xFF465655, 0xFF435353, 0xFF465656, 0xFF475757, 0xFF465656, 0xFF425150, 0xFF475656, 0xFF445453, 0xFF415150, 0xFF415150, 0xFF435352, 0xFF455554, 0xFF475756,
        0xFF455554, 0xFF40504F, 0xFF3E4E4D, 0xFF425251, 0xFF425251, 0xFF40504F, 0xFF445453, 0xFF42504E, 0xFF425150, 0xFF425251, 0xFF445453, 0xFF485857, 0xFF495958, 0xFF465655, 0xFF435352, 0xFF40504F, 0xFF415150, 0xFF435352, 0xFF455554, 0xFF455554,
        0xFF445453, 0xFF3F4F4E, 0xFF3E4E4D, 0xFF425251, 0xFF425251, 0xFF415150, 0xFF445453, 0xFF41504D, 0xFF77817F, 0xFFF4F5F5, 0xFF445350, 0xFF465552, 0xFF485754, 0xFF41504D, 0xFF42514E, 0xFF414550, 0xFF292F23, 0xFF898F8D, 0xFF839275, 0xFF78855A,
        0xFFACA674, 0xFFB69933, 0xFFC9942A, 0xFFC39B1F, 0xFFC1971F, 0xFFAF8517, 0xFF88650B, 0xFF6B550A, 0xFF6E5B16, 0xFF846B1B, 0xFF9D7B22, 0xFF7D5301, 0xFF413829, 0xFF3A4E4F, 0xFF324736, 0xFF3A403E, 0xFF3A4348, 0xFF35443D, 0xFF3B4841, 0xFF364741,
        0xFF374842, 0xFF384943, 0xFF374842, 0xFF354640, 0xFF354640, 0xFF364741, 0xFF374842, 0xFF3A4644, 0xFF3C4846, 0xFF3D4947, 0xFF3D4947, 0xFF3B4745, 0xFF3B4745, 0xFF3C4846, 0xFF3E4A48, 0xFF384744, 0xFF384744, 0xFF384744, 0xFF384744, 0xFF374643,
        0xFF374643, 0xFF374643, 0xFF384744, 0xFF394541, 0xFF37433F, 0xFF35413D, 0xFF35413D, 0xFF36423E, 0xFF37433F, 0xFF36423E, 0xFF35413D, 0xFF34433C, 0xFF32413A, 0xFF32413A, 0xFF33423D, 0xFF35443F, 0xFF364542, 0xFF344340, 0xFF33423F, 0xFF334642,
        0xFF374343, 0xFF39423D, 0xFF364339, 0xFF354545, 0xFF31383E, 0xFF363219, 0xFF534504, 0xFF5C4708, 0xFF5E490A, 0xFF604B0C, 0xFF614C0D, 0xFF604B0C, 0xFF5F4A0B, 0xFF604B0C, 0xFF614C0D, 0xFF674A06, 0xFF5E4E1D, 0xFF4D4214, 0xFF685920, 0xFF7D7147,
        0xFF4E4C3F, 0xFF66664E, 0xFF6F692F, 0xFF787759, 0xFF3E3A2F, 0xFF676149, 0xFF7A712E, 0xFF7B691F, 0xFF938058, 0xFF918B7B, 0xFF76826A, 0xFF565030, 0xFF6C6644, 0xFF4D4723, 0xFF615C36, 0xFF9A9470, 0xFFA59F7F, 0xFF928B71, 0xFF8E866F, 0xFF8C8773,
        0xFF807D6C, 0xFF78786C, 0xFF585D56, 0xFF4A5453, 0xFF445358, 0xFF465960, 0xFF445863, 0xFF414757, 0xFF474A51, 0xFF6A6A62, 0xFF6C664C, 0xFF938C60, 0xFF9E985E, 0xFFA9A460, 0xFFB0AC63, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0xFF645F59, 0xFF89867F, 0xFFBDBDB5, 0xFFC1C2BA, 0xFFD0D1C9, 0xFFD6D9D0, 0xFFD4DAD0, 0xFFC7CDC1, 0xFFC6CCC0, 0xFFBAC0B4, 0xFFC1C7BD, 0xFFD0D6CC, 0xFFD2D8CE, 0xFFC7CCC5, 0xFFE1E6DF, 0xFFECF2E6, 0xFFD7DDD1, 0xFFCED4C8, 0xFFD4DACE, 0xFFCFD5C9,
        0xFFCCD2C6, 0xFFD5DBCF, 0xFFD5DBCF, 0xFFD4DCCF, 0xFFD2DACD, 0xFFCED6C9, 0xFFD1D7C9, 0xFFC3C9BB, 0xFFB7BBAC, 0xFF7B7F70, 0xFF75796A, 0xFFA2A499, 0xFF9A9C91, 0xFF96988D, 0xFF84867B, 0xFF8D8F84, 0xFF7D7F74, 0xFF65675C, 0xFF67695E, 0xFF55522F,
        0xFF625918, 0xFF6E5B0C, 0xFF5E4608, 0xFF563F0D, 0xFF6F571B, 0xFF8A681E, 0xFF9D6F22, 0xFFA27522, 0xFF9B7814, 0xFFA18017, 0xFFA57D1D, 0xFFCEA239, 0xFFD5AD34, 0xFFAD8D28, 0xFF614A07, 0xFF736000, 0xFFC69C2C, 0xFFD8A238, 0xFF8C6313, 0xFF745819,
        0xFFA68840, 0xFFAC7D21, 0xFFB57C11, 0xFFDCA234, 0xFFD8A029, 0xFFD7A123, 0xFFD29E24, 0xFFE0AB39, 0xFFDAA437, 0xFFD9A22E, 0xFFDCA42B, 0xFFC49124, 0xFFBA8A24, 0xFFA07318, 0xFFB38C3D, 0xFF94722B, 0xFF674C09, 0xFF5E4906, 0xFF66530F, 0xFF6D5112,
        0xFFB29E45, 0xFF8F7B24, 0xFF98791F, 0xFFAE7F0B, 0xFFE6A934, 0xFFDB9D2C, 0xFFE6B127, 0xFFD7A729, 0xFFDCA524, 0xFFE0A324, 0xFFE1AA35, 0xFFAA7E15, 0xFF8F6B0A, 0xFFAB841F, 0xFFBF9328, 0xFFA47519, 0xFFA87D14, 0xFFAA8F4A, 0xFF8D9073, 0xFF7F8F72,
        0xFF9A9F89, 0xFF383733, 0xFF3F493E, 0xFF485657, 0xFF465455, 0xFF445252, 0xFF425050, 0xFF425050, 0xFF43524F, 0xFF455451, 0xFFF4F5F5, 0xFF7B8686, 0xFF465554, 0xFF596565, 0xFFE7E9E9, 0xFFFEFEFE, 0xFF75807E, 0xFF455451, 0xFF445453, 0xFFB0B7B6,
        0xFFFFFFFF, 0xFFB9BFBF, 0xFF455555, 0xFF445454, 0xFF425252, 0xFF445453, 0xFF435352, 0xFF445454, 0xFF425252, 0xFF455555, 0xFF475757, 0xFF455554, 0xFF425050, 0xFF41504F, 0xFF42514F, 0xFF415250, 0xFF495A59, 0xFF465655, 0xFF465655, 0xFF485857,
        0xFF465655, 0xFF40504F, 0xFF3E4E4D, 0xFF425251, 0xFF415150, 0xFF40504F, 0xFF445453, 0xFF445250, 0xFF445453, 0xFF445453, 0xFF455554, 0xFF455554, 0xFF455554, 0xFF455554, 0xFF455554, 0xFF415150, 0xFF415150, 0xFF435352, 0xFF455554, 0xFF475756,
        0xFF455554, 0xFF40504F, 0xFF3E4E4D, 0xFF425251, 0xFF425251, 0xFF40504F, 0xFF445453, 0xFF445350, 0xFF788381, 0xFFF4F5F5, 0xFF43524F, 0xFF445350, 0xFF455451, 0xFF3D4C49, 0xFF3D4C49, 0xFF3F484F, 0xFF293122, 0xFF888C8B, 0xFF818E74, 0xFF75815B,
        0xFF9E9C6C, 0xFF9E8727, 0xFFAF841E, 0xFFA78517, 0xFFB99120, 0xFFB68A1F, 0xFFA68222, 0xFF7E6515, 0xFF554300, 0xFF644E03, 0xFF85681A, 0xFF9A701E, 0xFF483F30, 0xFF364A4B, 0xFF344938, 0xFF404644, 0xFF3E474C, 0xFF35443D, 0xFF36433C, 0xFF34453F,
        0xFF354640, 0xFF364741, 0xFF364741, 0xFF354640, 0xFF354640, 0xFF374842, 0xFF394A44, 0xFF394543, 0xFF3B4745, 0xFF3C4846, 0xFF3B4745, 0xFF3A4644, 0xFF394543, 0xFF3A4644, 0xFF3B4745, 0xFF394845, 0xFF394845, 0xFF394845, 0xFF394845, 0xFF3A4946,
        0xFF3A4946, 0xFF394845, 0xFF394845, 0xFF384440, 0xFF35413D, 0xFF333F3B, 0xFF333F3B, 0xFF34403C, 0xFF35413D, 0xFF35413D, 0xFF34403C, 0xFF36453E, 0xFF35443D, 0xFF34433C, 0xFF35443F, 0xFF364540, 0xFF364542, 0xFF354441, 0xFF33423F, 0xFF324541,
        0xFF364242, 0xFF37403B, 0xFF334036, 0xFF324242, 0xFF2D343A, 0xFF322E15, 0xFF504201, 0xFF5E490A, 0xFF5F4A0B, 0xFF604B0C, 0xFF604B0C, 0xFF604B0C, 0xFF604B0C, 0xFF614C0D, 0xFF614C0D, 0xFF674C14, 0xFF604917, 0xFF624100, 0xFFA47616, 0xFFB58A2E,
        0xFF463300, 0xFF636420, 0xFFC8CC6B, 0xFF595F17, 0xFF464C02, 0xFF4F500C, 0xFF564C0E, 0xFF574400, 0xFF564A00, 0xFF6A6F1F, 0xFF687B39, 0xFF656533, 0xFF4E4E1A, 0xFF3C3C06, 0xFF383802, 0xFF4C4C18, 0xFF6F6E3E, 0xFF68673B, 0xFF232100, 0xFF0E0900,
        0xFF0C0900, 0xFF030300, 0xFF010600, 0xFF303A39, 0xFF3A494E, 0xFF263940, 0xFF20343F, 0xFF385251, 0xFF344A48, 0xFF061512, 0xFF111A17, 0xFF181A19, 0xFF201F24, 0xFF1E1D25, 0xFF0C0C16, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0xFF625D57, 0xFF88857E, 0xFFB7B7AF, 0xFFB8B9B1, 0xFFD1D2CA, 0xFFDDE0D7, 0xFFD2D8CE, 0xFFD1D7C9, 0xFFCED4C6, 0xFFBBC1B5, 0xFFB5BBAF, 0xFFC3C9BF, 0xFFCDD3C9, 0xFFC8CEC4, 0xFFDDE2DB, 0xFFDBE1D5, 0xFFC2C8BC, 0xFFC2C8BC, 0xFFCCD2C6, 0xFFC9CFC3,
        0xFFC3C9BD, 0xFFC3C9BD, 0xFFC8CEC2, 0xFFDDE5D8, 0xFFCDD5C8, 0xFFC7CDC1, 0xFFD3D9CB, 0xFFDCE0D2, 0xFFDADECF, 0xFF8F9384, 0xFF76786A, 0xFF9B9D92, 0xFF95978C, 0xFF888A7F, 0xFF7C7E73, 0xFF888A7F, 0xFF77796E, 0xFF66685D, 0xFF63655A, 0xFF424730,
        0xFF4B4B17, 0xFF524A0B, 0xFF3E3104, 0xFF2D2403, 0xFF393005, 0xFF483701, 0xFF4D3200, 0xFF564C11, 0xFF574E13, 0xFF594D13, 0xFF6D550F, 0xFF775100, 0xFFC2961B, 0xFFD6AB43, 0xFF906727, 0xFF595305, 0xFF7A6812, 0xFFCA9F40, 0xFFD09B35, 0xFF744B00,
        0xFF6B5200, 0xFF967D21, 0xFFA88217, 0xFFB3871A, 0xFFB58718, 0xFFB38410, 0xFFB9870C, 0xFFD5A01E, 0xFFE3AA27, 0xFFE9AD2B, 0xFFE7AA2B, 0xFFDFA730, 0xFFBF8A18, 0xFFBD8C25, 0xFFA2771B, 0xFFAD8A38, 0xFF8E7228, 0xFF654F06, 0xFF594700, 0xFF5A4E14,
        0xFFD8D689, 0xFFA2A35F, 0xFF90843C, 0xFFB9982D, 0xFFCB9928, 0xFFE3AE3E, 0xFFD6A821, 0xFFDEA62F, 0xFFE3A629, 0xFFDB9B1D, 0xFFE4AC35, 0xFFA47B12, 0xFF614700, 0xFF6B590D, 0xFF84732B, 0xFFBE9F45, 0xFFB19228, 0xFFA49047, 0xFF8E906B, 0xFF889770,
        0xFFA3A88A, 0xFF3A3C31, 0xFF414F40, 0xFF475556, 0xFF455354, 0xFF414F4F, 0xFF3F4D4D, 0xFF3F4D4D, 0xFF41504D, 0xFF455451, 0xFFF4F5F5, 0xFF7A8584, 0xFF435251, 0xFF425050, 0xFF727D7D, 0xFFFBFCFC, 0xFFE8EAE9, 0xFF596764, 0xFF455554, 0xFF4C5A59,
        0xFFD1D5D5, 0xFFFFFFFF, 0xFF919A98, 0xFF455451, 0xFF455451, 0xFF445453, 0xFF495958, 0xFF465656, 0xFF445454, 0xFF455555, 0xFF445454, 0xFF405050, 0xFF445352, 0xFF414F4F, 0xFF43524F, 0xFF3C4C4B, 0xFF475756, 0xFF435353, 0xFF455555, 0xFF455555,
        0xFF445454, 0xFF405050, 0xFF3E4E4D, 0xFF415150, 0xFF415150, 0xFF40504F, 0xFF445453, 0xFF42504F, 0xFF445453, 0xFF465554, 0xFF435352, 0xFF425250, 0xFF425250, 0xFF445352, 0xFF475756, 0xFF465655, 0xFF455554, 0xFF455554, 0xFF465655, 0xFF485857,
        0xFF465655, 0xFF40504F, 0xFF3E4E4D, 0xFF425251, 0xFF415150, 0xFF40504F, 0xFF445453, 0xFF445350, 0xFF788381, 0xFFF4F5F5, 0xFF42514E, 0xFF42514E, 0xFF43524F, 0xFF3C4B48, 0xFF3C4B48, 0xFF344243, 0xFF212C1B, 0xFF858786, 0xFF858D78, 0xFF788363,
        0xFF91936C, 0xFF80751B, 0xFF8D7312, 0xFF977E21, 0xFFB3902C, 0xFFB98D22, 0xFFC1962E, 0xFF9B7C22, 0xFF584200, 0xFF584300, 0xFF755E1B, 0xFF9A701E, 0xFF443B2C, 0xFF334748, 0xFF304534, 0xFF353B39, 0xFF374045, 0xFF34433C, 0xFF303D36, 0xFF2F403A,
        0xFF30413B, 0xFF31423C, 0xFF31423C, 0xFF31423C, 0xFF33443E, 0xFF354640, 0xFF374842, 0xFF3D4947, 0xFF3D4947, 0xFF3E4A48, 0xFF3E4A48, 0xFF3D4947, 0xFF3C4846, 0xFF3D4947, 0xFF3D4947, 0xFF374643, 0xFF364542, 0xFF364542, 0xFF384744, 0xFF3A4946,
        0xFF3B4A47, 0xFF394845, 0xFF374643, 0xFF3B4743, 0xFF394541, 0xFF37433F, 0xFF36423E, 0xFF36423E, 0xFF37433F, 0xFF384440, 0xFF384440, 0xFF37463F, 0xFF36453E, 0xFF34433C, 0xFF34433E, 0xFF34433E, 0xFF33423F, 0xFF32413E, 0xFF31403D, 0xFF334642,
        0xFF364242, 0xFF38413C, 0xFF344137, 0xFF324242, 0xFF2E353B, 0xFF343017, 0xFF524403, 0xFF604B0C, 0xFF604B0C, 0xFF5F4A0B, 0xFF604B0C, 0xFF614C0D, 0xFF624D0E, 0xFF624D0E, 0xFF624D0E, 0xFF654806, 0xFF584614, 0xFF564612, 0xFF947F3C, 0xFFC0AC77,
        0xFF8A8568, 0xFFA7A984, 0xFFCDC884, 0xFFAAAB83, 0xFF5F5D44, 0xFF9D9976, 0xFFA49B58, 0xFF86742A, 0xFFB3A16F, 0xFFD4D1B0, 0xFFC8D6B2, 0xFF9F9F7D, 0xFF6E6E4A, 0xFF4C4C26, 0xFF666640, 0xFFB5B591, 0xFFBFBE9F, 0xFFA6A589, 0xFF87856E, 0xFF6D6854,
        0xFF656251, 0xFF626256, 0xFF484D46, 0xFF444E4D, 0xFF404F54, 0xFF3E5158, 0xFF3A4E59, 0xFF444F51, 0xFF434C47, 0xFF515544, 0xFF5A593A, 0xFF807D50, 0xFF8D8A53, 0xFF908E51, 0xFF8C8B4B, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0xFF68635D, 0xFF8E8B84, 0xFFB1B1A9, 0xFFA6A79F, 0xFFBABBB3, 0xFFCBCEC5, 0xFFC8CEC4, 0xFFC2C8BA, 0xFFBFC5B7, 0xFFC7CDBF, 0xFFCCD2C6, 0xFFCED4C8, 0xFFD1D7CD, 0xFFC6CCC2, 0xFFBFC5BB, 0xFFD9DFD3, 0xFFCFD5C9, 0xFFD6DCD0, 0xFFD6DCD0, 0xFFD5DBCF,
        0xFFD8DED2, 0xFFC9CFC3, 0xFFBEC4B8, 0xFFC0C6BA, 0xFFB8BEB2, 0xFFB3B9AD, 0xFFC4C8BA, 0xFFDEE2D4, 0xFFF6F8EA, 0xFFAFB1A3, 0xFF848678, 0xFF9FA196, 0xFF95978C, 0xFF898B80, 0xFF7E8075, 0xFF7C7E73, 0xFF6F7166, 0xFF6E7065, 0xFF5B5D52, 0xFF564D44,
        0xFF6A5C39, 0xFF695322, 0xFF543E15, 0xFF514219, 0xFF5C5215, 0xFF6A580E, 0xFF7B5E18, 0xFF9DA47A, 0xFFA7AD93, 0xFFADAE9C, 0xFFBEB386, 0xFF755C02, 0xFF8E6900, 0xFFC79B42, 0xFFD2A772, 0xFF7C7D6D, 0xFFB3AF8C, 0xFFBA9D4F, 0xFFD7A029, 0xFFD8AA30,
        0xFF827517, 0xFF514B00, 0xFF927D2C, 0xFF98791C, 0xFFC39F47, 0xFFCA9F43, 0xFFC69425, 0xFFDBA51E, 0xFFE6AC1C, 0xFFE5A422, 0xFFE19E2B, 0xFFE2A322, 0xFFE4A82C, 0xFFC18C1A, 0xFFB78820, 0xFF9B7417, 0xFFAB8D37, 0xFF8C7525, 0xFF5B4800, 0xFF5B4C15,
        0xFF757736, 0xFF9DA677, 0xFFC2BF8C, 0xFFA98F32, 0xFFD9A639, 0xFFDEA439, 0xFFE1AE2F, 0xFFE1A230, 0xFFE8A72F, 0xFFE7A72B, 0xFFE4AD36, 0xFF966F08, 0xFF7C6A18, 0xFFB4AF75, 0xFFBAC092, 0xFFCAB566, 0xFFBEA646, 0xFFAF9E59, 0xFF90916F, 0xFF808C66,
        0xFF969A7F, 0xFF363833, 0xFF48594F, 0xFF485657, 0xFF455354, 0xFF425050, 0xFF3F4D4D, 0xFF3F4D4D, 0xFF41504D, 0xFF445350, 0xFFF4F5F5, 0xFF778382, 0xFF3F4E4D, 0xFF3F4D4D, 0xFF3F4D4D, 0xFF969E9D, 0xFFFFFFFF, 0xFFC6CBCA, 0xFF445453, 0xFF435251,
        0xFF596565, 0xFFEBECEC, 0xFFF9FAF9, 0xFF6F7A78, 0xFF475653, 0xFF455554, 0xFF465554, 0xFF465454, 0xFF455353, 0xFF445350, 0xFF455451, 0xFF445350, 0xFF435151, 0xFF445252, 0xFF3F4E4B, 0xFF455353, 0xFF3C4A4A, 0xFF415151, 0xFF425252, 0xFF415151,
        0xFF415151, 0xFF3F4F4F, 0xFF3E4E4D, 0xFF415150, 0xFF40504F, 0xFF3F4F4E, 0xFF445352, 0xFF455352, 0xFF435351, 0xFF455452, 0xFF435250, 0xFF43524F, 0xFF455452, 0xFF435351, 0xFF465655, 0xFF475757, 0xFF465656, 0xFF455555, 0xFF455555, 0xFF455555,
        0xFF445454, 0xFF405050, 0xFF3E4E4D, 0xFF415150, 0xFF415150, 0xFF40504F, 0xFF445453, 0xFF43524F, 0xFF778280, 0xFFF4F5F5, 0xFF42514E, 0xFF43524F, 0xFF455451, 0xFF3E4D4A, 0xFF3F4E4B, 0xFF354846, 0xFF25321E, 0xFF898B8A, 0xFF8A8E7D, 0xFF768065,
        0xFF7C8260, 0xFF575502, 0xFF615000, 0xFF725F12, 0xFFA28124, 0xFFC19226, 0xFFCB9B2D, 0xFFA68221, 0xFF654C00, 0xFF594604, 0xFF78632A, 0xFF916715, 0xFF433A2B, 0xFF324647, 0xFF192E1D, 0xFF0E1412, 0xFF1F282D, 0xFF35443D, 0xFF3A4740, 0xFF34433E,
        0xFF34433E, 0xFF34433E, 0xFF34433E, 0xFF35443F, 0xFF374641, 0xFF394843, 0xFF3B4A45, 0xFF3D4945, 0xFF3E4A46, 0xFF3E4A46, 0xFF3F4B47, 0xFF3F4B47, 0xFF3F4B47, 0xFF3F4B47, 0xFF404C48, 0xFF3D4945, 0xFF3C4844, 0xFF3C4844, 0xFF3F4B47, 0xFF424E4A,
        0xFF434F4B, 0xFF404C48, 0xFF3D4945, 0xFF3D4945, 0xFF3B4743, 0xFF394541, 0xFF384440, 0xFF384440, 0xFF394541, 0xFF3A4642, 0xFF3B4743, 0xFF3E4B44, 0xFF3D4A43, 0xFF3B4841, 0xFF3A4642, 0xFF384440, 0xFF374341, 0xFF374341, 0xFF364240, 0xFF2F423E,
        0xFF344040, 0xFF38413C, 0xFF354238, 0xFF334343, 0xFF2F363C, 0xFF343017, 0xFF524403, 0xFF5F4A0B, 0xFF5E490A, 0xFF5F4A0B, 0xFF614C0D, 0xFF644F10, 0xFF655011, 0xFF644F10, 0xFF624D0E, 0xFF684A02, 0xFF5B4D1C, 0xFF4A4619, 0xFF655E2A, 0xFF7A7452,
        0xFF52554E, 0xFF6E6F5D, 0xFF766C37, 0xFF75735A, 0xFF484242, 0xFF67604E, 0xFF797129, 0xFF726113, 0xFF836F4E, 0xFF99928A, 0xFF84907C, 0xFF4D5035, 0xFF595C3F, 0xFF4B4E2F, 0xFF575A3B, 0xFFA3A689, 0xFF9B9D85, 0xFF9A9B89, 0xFF9B9C8C, 0xFF999480,
        0xFF8B8877, 0xFF89897D, 0xFF646962, 0xFF434D4C, 0xFF37464B, 0xFF3C4F56, 0xFF374B56, 0xFF454857, 0xFF45464B, 0xFF726E63, 0xFF716B49, 0xFFA49A65, 0xFFAFA75F, 0xFFB6B05C, 0xFFB8B459, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0xFF6B6660, 0xFF94918A, 0xFFC0C0B8, 0xFFB4B5AD, 0xFFB7B8B0, 0xFFC6C9C0, 0xFFDBE1D7, 0xFFD2D9C9, 0xFFBCC3B3, 0xFFBFC5B7, 0xFFC8CEC0, 0xFFC9CFC3, 0xFFC5CBBF, 0xFFC7CDC3, 0xFFD3D9CF, 0xFFCFD5C9, 0xFFC7CDC1, 0xFFD2D8CC, 0xFFD9DFD3, 0xFFDBE1D5,
        0xFFD6DCD0, 0xFFBEC4B8, 0xFFB1B7AB, 0xFFCBD1C5, 0xFFCED4C8, 0xFFC7CABF, 0xFFC3C7B9, 0xFFC3C5B8, 0xFFD2D4C6, 0xFF9D9F91, 0xFF7A7B6D, 0xFF9A9C91, 0xFF93958A, 0xFF8C8E83, 0xFF898B80, 0xFF818378, 0xFF65675C, 0xFF626459, 0xFF54564B, 0xFF524535,
        0xFF68542F, 0xFF674C1D, 0xFF583C15, 0xFF584316, 0xFF685916, 0xFF816A1E, 0xFF93722B, 0xFF848A66, 0xFFB1B9AE, 0xFFC0C6C6, 0xFFA3A283, 0xFF988C40, 0xFF947C24, 0xFFAD904E, 0xFFAF916B, 0xFF43463B, 0xFF5D614A, 0xFF8A773C, 0xFFC39736, 0xFFAD8627,
        0xFF696123, 0xFF4B4F2C, 0xFF645C38, 0xFFB9A479, 0xFFB9A05E, 0xFF967518, 0xFF986C00, 0xFFC99120, 0xFFE5A736, 0xFFE8A733, 0xFFE5A52A, 0xFFE6A320, 0xFFE0A120, 0xFFE5AB30, 0xFFC59320, 0xFFB3881F, 0xFF9E7A1A, 0xFFAD8F37, 0xFF8D711E, 0xFF614506,
        0xFF494309, 0xFFB4BA9E, 0xFF888363, 0xFF957522, 0xFFD0952D, 0xFFEBA840, 0xFFDFA32B, 0xFFE8A733, 0xFFD79922, 0xFFD79C24, 0xFFE0AD38, 0xFFAE871E, 0xFFB49E4B, 0xFFC7C28A, 0xFF899168, 0xFF74601B, 0xFF917921, 0xFFA39056, 0xFF8A8C74, 0xFF818E72,
        0xFF9FA291, 0xFF37383C, 0xFF3B4B4A, 0xFF485657, 0xFF475556, 0xFF445252, 0xFF425050, 0xFF414F4F, 0xFF41504D, 0xFF41504D, 0xFFF4F5F5, 0xFF778382, 0xFF404F4E, 0xFF404E4E, 0xFF3F4D4D, 0xFF42514E, 0xFFC5CAC9, 0xFFFFFFFF, 0xFF9AA2A2, 0xFF3F4E4D,
        0xFF3F4D4D, 0xFF778181, 0xFFFFFFFF, 0xFFE3E6E5, 0xFF586563, 0xFF435352, 0xFF435251, 0xFF425050, 0xFF425050, 0xFF43524F, 0xFF455451, 0xFF455451, 0xFF3E4D4C, 0xFF455353, 0xFF3F4E4B, 0xFF475351, 0xFF424F4D, 0xFF455555, 0xFF465656, 0xFF455555,
        0xFF425251, 0xFF3F4F4E, 0xFF3E4E4D, 0xFF415150, 0xFF3F4F4E, 0xFF3F4F4E, 0xFF445352, 0xFF485654, 0xFF445350, 0xFF465552, 0xFF43524F, 0xFF43524F, 0xFF455451, 0xFF404F4D, 0xFF435351, 0xFF465656, 0xFF455555, 0xFF435353, 0xFF425252, 0xFF415151,
        0xFF415151, 0xFF3F4F4F, 0xFF3E4E4D, 0xFF415150, 0xFF40504F, 0xFF3F4F4E, 0xFF445453, 0xFF455451, 0xFF788381, 0xFFF4F5F5, 0xFF43524F, 0xFF43524F, 0xFF465552, 0xFF404F4C, 0xFF41504D, 0xFF394C48, 0xFF273420, 0xFF878B8A, 0xFF878E7C, 0xFF747E65,
        0xFF777C5C, 0xFF514E00, 0xFF5A4A00, 0xFF513F00, 0xFF8B690F, 0xFFC69628, 0xFFCF9A28, 0xFFAA7F17, 0xFF6E5000, 0xFF543D00, 0xFF796425, 0xFF926816, 0xFF443B2C, 0xFF364A4B, 0xFF1F3423, 0xFF0E1412, 0xFF1C252A, 0xFF35443D, 0xFF3B4841, 0xFF35443F,
        0xFF35443F, 0xFF34433E, 0xFF34433E, 0xFF35443F, 0xFF35443F, 0xFF364540, 0xFF364540, 0xFF3B4743, 0xFF3B4743, 0xFF3B4743, 0xFF3C4844, 0xFF3E4A46, 0xFF3F4B47, 0xFF3F4B47, 0xFF3F4B47, 0xFF3B4743, 0xFF3A4642, 0xFF3A4642, 0xFF3D4945, 0xFF404C48,
        0xFF414D49, 0xFF3E4A46, 0xFF3B4743, 0xFF3B4743, 0xFF3A4642, 0xFF394541, 0xFF384440, 0xFF37433F, 0xFF384440, 0xFF394541, 0xFF3A4642, 0xFF3B4841, 0xFF3A4740, 0xFF39463F, 0xFF37433F, 0xFF35413D, 0xFF34403E, 0xFF34403E, 0xFF34403E, 0xFF2C3F3B,
        0xFF333F3F, 0xFF38413C, 0xFF354238, 0xFF324242, 0xFF2C3339, 0xFF312D14, 0xFF4E4000, 0xFF5B4607, 0xFF5C4708, 0xFF5E490A, 0xFF624D0E, 0xFF665112, 0xFF675213, 0xFF655011, 0xFF634E0F, 0xFF674D10, 0xFF604B16, 0xFF5F4200, 0xFF9D7213, 0xFFB08830,
        0xFF463600, 0xFF5D5E1C, 0xFFAFB352, 0xFF5A601A, 0xFF404600, 0xFF4E4F0D, 0xFF514807, 0xFF574500, 0xFF5D5100, 0xFF676B1E, 0xFF687B3B, 0xFF6D7345, 0xFF525828, 0xFF353C09, 0xFF333A06, 0xFF454C19, 0xFF343A0E, 0xFF484D25, 0xFF1B1F00, 0xFF0C0700,
        0xFF090600, 0xFF060600, 0xFF040902, 0xFF283231, 0xFF324146, 0xFF25383F, 0xFF1D313C, 0xFF304653, 0xFF2D3F4B, 0xFF08121C, 0xFF0F131C, 0xFF16151D, 0xFF1C1922, 0xFF1C1826, 0xFF0E0C1A, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0xFF69645E, 0xFF8C8982, 0xFFBABAB2, 0xFFC2C3BB, 0xFFCFD0C8, 0xFFCFD2C9, 0xFFD1D7CD, 0xFFC8CFBF, 0xFFC5CCBC, 0xFFC5CCBC, 0xFFBEC4B6, 0xFFC6CCBE, 0xFFBEC4B8, 0xFFBAC0B4, 0xFFC9CFC3, 0xFFC6CCC0, 0xFFC8CEC2, 0xFFD1D7CB, 0xFFD5DBCF, 0xFFD7DDD1,
        0xFFD5DBCF, 0xFFC5CBBF, 0xFFB8BEB2, 0xFFCDD0C5, 0xFFCED1C6, 0xFFC0C3B8, 0xFFC5C7BA, 0xFFB6B8AB, 0xFFB0B2A4, 0xFF898A7C, 0xFF717264, 0xFF8B8D82, 0xFF8B8D82, 0xFF84867B, 0xFF87897E, 0xFF898B80, 0xFF5F6156, 0xFF54564B, 0xFF5C5E53, 0xFF4D4E2E,
        0xFF4E4818, 0xFF4C3C0B, 0xFF42300C, 0xFF352700, 0xFF3E3300, 0xFF4C3700, 0xFF3E1B00, 0xFF5C5737, 0xFFB0B1AC, 0xFF9AA2A4, 0xFF5B6243, 0xFF838347, 0xFFC1B77A, 0xFFBAAD81, 0xFFC0B796, 0xFF8C9252, 0xFF868952, 0xFFAAA06F, 0xFFBAA377, 0xFF786547,
        0xFF4A463B, 0xFF494F4D, 0xFF656764, 0xFFD7C6CE, 0xFFE5D5A1, 0xFFE2CC5C, 0xFFC39F27, 0xFF976109, 0xFFB77930, 0xFFE9AE4A, 0xFFDDAA1D, 0xFFE6A525, 0xFFE1A221, 0xFFE5AB2D, 0xFFDEAA30, 0xFFBA8B19, 0xFFB88D24, 0xFF9D7516, 0xFFAE8930, 0xFF926417,
        0xFF5A4B0A, 0xFF4D513A, 0xFF46402A, 0xFF957221, 0xFFE8A83C, 0xFFDD942B, 0xFFE4A32B, 0xFFE3A52A, 0xFFE0A831, 0xFFCB9825, 0xFFD3A335, 0xFFD1A53C, 0xFFB59339, 0xFF907E34, 0xFF8A8448, 0xFF9B7D35, 0xFFA28329, 0xFF9E8950, 0xFF858873, 0xFF84937C,
        0xFFA2A698, 0xFF36363E, 0xFF3A4849, 0xFF475556, 0xFF475556, 0xFF455353, 0xFF435151, 0xFF425050, 0xFF404F4C, 0xFF3F4E4B, 0xFFF4F5F4, 0xFF788483, 0xFF475756, 0xFF828C8C, 0xFF414F4F, 0xFF41504D, 0xFF54615E, 0xFFE7E8E8, 0xFFFDFDFD, 0xFF717D7C,
        0xFF404E4E, 0xFF3F4D4D, 0xFF9FA6A5, 0xFFFFFFFF, 0xFFC4C9C8, 0xFF415150, 0xFF3F4E4D, 0xFF3F4D4D, 0xFF3F4D4D, 0xFF41504D, 0xFF455451, 0xFF475654, 0xFF455353, 0xFF465454, 0xFF3F4E4B, 0xFF3A4443, 0xFF475252, 0xFF465555, 0xFF455555, 0xFF425251,
        0xFF40504E, 0xFF40504E, 0xFF3F4F4E, 0xFF40504F, 0xFF3F4F4E, 0xFF3E4E4D, 0xFF445352, 0xFF465552, 0xFF42514E, 0xFF455451, 0xFF42514E, 0xFF42514E, 0xFF445350, 0xFF3F4E4B, 0xFF41504D, 0xFF465656, 0xFF455555, 0xFF455555, 0xFF465656, 0xFF455555,
        0xFF425251, 0xFF3F4F4E, 0xFF3E4E4D, 0xFF415150, 0xFF3F4F4E, 0xFF3F4F4E, 0xFF445453, 0xFF485754, 0xFF7A8583, 0xFFF4F5F5, 0xFF43524F, 0xFF43524F, 0xFF455451, 0xFF3F4E4B, 0xFF3F4E4B, 0xFF384845, 0xFF23301C, 0xFF848A88, 0xFF86927C, 0xFF788569,
        0xFF8B8F6E, 0xFF776F1E, 0xFF886E17, 0xFF7C6812, 0xFF957110, 0xFFC38F1F, 0xFFCC9521, 0xFFBE8F25, 0xFF947316, 0xFF664A00, 0xFF725706, 0xFF966C1A, 0xFF3E3526, 0xFF364A4B, 0xFF374C3B, 0xFF353B39, 0xFF343D42, 0xFF37463F, 0xFF37443D, 0xFF384440,
        0xFF37433F, 0xFF37433F, 0xFF37433F, 0xFF37433F, 0xFF37433F, 0xFF36423E, 0xFF35413D, 0xFF3B4841, 0xFF3B4841, 0xFF3B4841, 0xFF3C4942, 0xFF3E4B44, 0xFF3F4C45, 0xFF3F4C45, 0xFF3F4C45, 0xFF3A453F, 0xFF39443E, 0xFF39443E, 0xFF3A453F, 0xFF3D4842,
        0xFF3D4842, 0xFF3B4640, 0xFF39443E, 0xFF3A4642, 0xFF3A4642, 0xFF3A4642, 0xFF394541, 0xFF384440, 0xFF384440, 0xFF394541, 0xFF3A4642, 0xFF39443E, 0xFF38433D, 0xFF37423C, 0xFF35403C, 0xFF333E3A, 0xFF333D3C, 0xFF343E3D, 0xFF353F3E, 0xFF30433F,
        0xFF374343, 0xFF3C4540, 0xFF38453B, 0xFF344444, 0xFF2D343A, 0xFF312D14, 0xFF4F4100, 0xFF5A4506, 0xFF5C4708, 0xFF5F4A0B, 0xFF624D0E, 0xFF655011, 0xFF665112, 0xFF655011, 0xFF644F10, 0xFF664B08, 0xFF5C4813, 0xFF584407, 0xFF997C2C, 0xFFC9AE69,
        0xFF8A8156, 0xFF9C9E6C, 0xFFC4C371, 0xFFA1A46F, 0xFF5B5C30, 0xFF92905F, 0xFF99904B, 0xFF89772B, 0xFFB3A463, 0xFFCACA96, 0xFFC1D1A0, 0xFFA6A77F, 0xFF707147, 0xFF3E3F13, 0xFF686A3B, 0xFFA9AA7E, 0xFF9D9E76, 0xFFA0A07E, 0xFF807F60, 0xFF66614D,
        0xFF5C5948, 0xFF57574B, 0xFF3E433C, 0xFF3B4544, 0xFF3A494E, 0xFF3A4D54, 0xFF374B56, 0xFF3E4E4E, 0xFF45514D, 0xFF474F42, 0xFF51533E, 0xFF686749, 0xFF726F4C, 0xFF787650, 0xFF75734C, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0xFF67625C, 0xFF85827B, 0xFFABABA3, 0xFFB4B5AD, 0xFFCFD0C8, 0xFFCCCFC6, 0xFFB7BDB3, 0xFFC1C8B8, 0xFFCAD1C1, 0xFFCED5C5, 0xFFB9C0B0, 0xFFBBC1B3, 0xFFBAC0B2, 0xFFB9BFB3, 0xFFBEC4B8, 0xFFC1C7BB, 0xFFCDD3C7, 0xFFCBD1C5, 0xFFC1C7BB, 0xFFBFC5B9,
        0xFFC8CEC2, 0xFFD1D7CB, 0xFFCBD1C5, 0xFFD3D6CB, 0xFFC9CCC1, 0xFFB1B4A9, 0xFFCCCEC1, 0xFFC7C9BC, 0xFFB9BAAC, 0xFF949587, 0xFF797A6C, 0xFF85877C, 0xFF87897E, 0xFF83857A, 0xFF7E8075, 0xFF818378, 0xFF606257, 0xFF54564B, 0xFF5A5C51, 0xFF485132,
        0xFF48491D, 0xFF4A3F12, 0xFF4C3C1A, 0xFF4B3F15, 0xFF5C5014, 0xFF6C5418, 0xFF573206, 0xFF5B4F35, 0xFFA8A4A1, 0xFF9DA6A3, 0xFF5D6B48, 0xFF484F1C, 0xFF7B7752, 0xFF595538, 0xFF777952, 0xFF7F792D, 0xFF726926, 0xFF938E64, 0xFF81877B, 0xFF404A4C,
        0xFF484C4D, 0xFF4A504C, 0xFF404E4E, 0xFF817A81, 0xFF787041, 0xFF9C8B23, 0xFFE4C550, 0xFFCEA040, 0xFFA06C18, 0xFFB3821B, 0xFFD7AC2A, 0xFFE0A227, 0xFFE9AC2F, 0xFFDCA224, 0xFFD9A224, 0xFFDEAA32, 0xFFBB891A, 0xFFBA8923, 0xFFA87718, 0xFFBD8926,
        0xFF826F22, 0xFF7F856B, 0xFF6C6C54, 0xFF947824, 0xFFDA9F2B, 0xFFE5A231, 0xFFDDA326, 0xFFE0A525, 0xFFDDAA34, 0xFFAD8317, 0xFFA57811, 0xFFD8A73E, 0xFFD3A23B, 0xFF916909, 0xFFA88A32, 0xFFAC8125, 0xFFA57E15, 0xFF9D8642, 0xFF898F73, 0xFF869A7E,
        0xFF99A28F, 0xFF303231, 0xFF435047, 0xFF455354, 0xFF445253, 0xFF435151, 0xFF425050, 0xFF404E4E, 0xFF3F4E4B, 0xFF3F4E4B, 0xFFF4F5F4, 0xFF788483, 0xFF425251, 0xFFE0E3E3, 0xFF7D8887, 0xFF404F4C, 0xFF3F4E4B, 0xFF707B79, 0xFFFDFDFD, 0xFFE7E9E9,
        0xFF576565, 0xFF414F4F, 0xFF43524F, 0xFFC5C9C9, 0xFFFFFFFF, 0xFF9CA4A4, 0xFF41504F, 0xFF404E4E, 0xFF3F4D4D, 0xFF41504D, 0xFF445351, 0xFF465654, 0xFF455353, 0xFF3F4D4D, 0xFF455451, 0xFF454E4B, 0xFF424E4D, 0xFF435351, 0xFF435352, 0xFF42524F,
        0xFF41514F, 0xFF404F4D, 0xFF42514F, 0xFF40504E, 0xFF3F4F4E, 0xFF3E4E4D, 0xFF445353, 0xFF42514E, 0xFF3E4D4A, 0xFF42514E, 0xFF404F4C, 0xFF42514E, 0xFF455451, 0xFF41504D, 0xFF42514E, 0xFF475757, 0xFF465656, 0xFF465656, 0xFF465656, 0xFF425252,
        0xFF3F4F4E, 0xFF3F4F4E, 0xFF3F4F4E, 0xFF40504F, 0xFF3F4F4E, 0xFF3E4E4D, 0xFF445453, 0xFF465552, 0xFF798381, 0xFFF4F5F5, 0xFF42514E, 0xFF42514E, 0xFF445350, 0xFF3F4E4B, 0xFF404F4C, 0xFF3E4C4C, 0xFF25321E, 0xFF87908B, 0xFF88987E, 0xFF788766,
        0xFF989872, 0xFF907E28, 0xFFA47B23, 0xFFA88F2B, 0xFFB58C22, 0xFFC58E1A, 0xFFC58C1B, 0xFFC8972E, 0xFFBD9734, 0xFF967412, 0xFF856200, 0xFF9A701E, 0xFF3B3223, 0xFF2C4041, 0xFF334837, 0xFF3B413F, 0xFF3A4348, 0xFF37463F, 0xFF36433C, 0xFF394541,
        0xFF384440, 0xFF384440, 0xFF394541, 0xFF3A4642, 0xFF394541, 0xFF37433F, 0xFF34403C, 0xFF38453E, 0xFF36433C, 0xFF36433C, 0xFF36433C, 0xFF38453E, 0xFF39463F, 0xFF38453E, 0xFF37443D, 0xFF38433D, 0xFF37423C, 0xFF37423C, 0xFF38433D, 0xFF38433D,
        0xFF38433D, 0xFF38433D, 0xFF37423C, 0xFF333F3B, 0xFF34403C, 0xFF36423E, 0xFF35413D, 0xFF34403C, 0xFF323E3A, 0xFF333F3B, 0xFF333F3B, 0xFF37423C, 0xFF37423C, 0xFF37423C, 0xFF35403C, 0xFF333E3A, 0xFF343E3D, 0xFF36403F, 0xFF384241, 0xFF2E413D,
        0xFF354141, 0xFF39423D, 0xFF354238, 0xFF314141, 0xFF2C3339, 0xFF332F16, 0xFF524403, 0xFF5C4708, 0xFF5E490A, 0xFF604B0C, 0xFF614C0D, 0xFF624D0E, 0xFF634E0F, 0xFF644F10, 0xFF655011, 0xFF684B00, 0xFF5B4E1A, 0xFF464316, 0xFF595623, 0xFF73704F,
        0xFF525751, 0xFF6B6C5C, 0xFF645A25, 0xFF6F6D56, 0xFF4B4547, 0xFF615A4A, 0xFF7C742B, 0xFF7E6D1D, 0xFF897455, 0xFFA19996, 0xFF879281, 0xFF595637, 0xFF63603F, 0xFF4C4926, 0xFF615E3B, 0xFFA9A685, 0xFFC1BDA0, 0xFFB3AF96, 0xFF9E9985, 0xFFA7A28E,
        0xFF949180, 0xFF929286, 0xFF666B64, 0xFF374140, 0xFF2B3A3F, 0xFF3C4F56, 0xFF384C57, 0xFF3A3E41, 0xFF43443C, 0xFF807C61, 0xFF7E7846, 0xFFAFA75E, 0xFFB8B156, 0xFFC2BE53, 0xFFCBC958, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0xFF645F59, 0xFF8E8B84, 0xFFB9B9B1, 0xFFB1B2AA, 0xFFC6C7BF, 0xFFD8DBD2, 0xFFD7DDD3, 0xFFE6EDDB, 0xFFD9E0D0, 0xFFE1E8D8, 0xFFD1D8C8, 0xFFC2C8BA, 0xFFB9BFB1, 0xFFC5CBBF, 0xFFCAD0C4, 0xFFB9BFB3, 0xFFC8CEC2, 0xFFC7CDC1, 0xFFC6CCC0, 0xFFBFC5B9,
        0xFFBCC2B6, 0xFFCCD2C6, 0xFFD3D9CD, 0xFFD4D7CC, 0xFFC6C9BE, 0xFFA2A499, 0xFFBFC1B4, 0xFFBEBEB2, 0xFFAFB0A2, 0xFF919284, 0xFF727063, 0xFF7B7D72, 0xFF7F8176, 0xFF87897E, 0xFF7B7D72, 0xFF77796E, 0xFF6C6E63, 0xFF64665B, 0xFF515348, 0xFF424436,
        0xFF564F32, 0xFF55431F, 0xFF503A15, 0xFF58450D, 0xFF6E5C08, 0xFF8F7117, 0xFF9B6F24, 0xFF7D705D, 0xFFA2A19F, 0xFFBDC8C0, 0xFF61714C, 0xFF596237, 0xFFADAB94, 0xFFB0AE97, 0xFF9DA375, 0xFF635025, 0xFF594111, 0xFFBEBA97, 0xFFBFDDD1, 0xFF4F726B,
        0xFF4D5B44, 0xFF4A523A, 0xFF22362A, 0xFF313419, 0xFF5C572D, 0xFF4B3900, 0xFF8D7008, 0xFFE7C446, 0xFFC49D1C, 0xFF916900, 0xFFBF9831, 0xFFE0A42C, 0xFFDDA127, 0xFFE1A72A, 0xFFE1AA2B, 0xFFE0A82F, 0xFFDDA534, 0xFFC38C25, 0xFFBA831F, 0xFFAF7A08,
        0xFF988830, 0xFF5D6B4A, 0xFFBBC4A7, 0xFFC8B358, 0xFFD09D20, 0xFFE7AC34, 0xFFD8A726, 0xFFDAA41E, 0xFFDCB037, 0xFFC29D33, 0xFF8B6100, 0xFFA16C02, 0xFFECB040, 0xFFD6A131, 0xFFC7992A, 0xFFD29F34, 0xFFC89B24, 0xFFAA9343, 0xFF808863, 0xFF78906C,
        0xFF96A389, 0xFF33362B, 0xFF404D39, 0xFF435152, 0xFF425051, 0xFF414F4F, 0xFF404E4E, 0xFF3F4D4D, 0xFF3E4D4A, 0xFF3F4E4B, 0xFFF4F5F4, 0xFF778382, 0xFF3C4C4B, 0xFFE6E8E8, 0xFFEDEEEE, 0xFF576462, 0xFF3F4E4B, 0xFF3E4D4A, 0xFF9CA4A4, 0xFFFFFFFF,
        0xFFC8CCCC, 0xFF455453, 0xFF404F4C, 0xFF515F5C, 0xFFE4E6E6, 0xFFFFFFFF, 0xFFD1D5D5, 0xFFD0D3D3, 0xFFCFD2D2, 0xFFCFD2D2, 0xFFCFD2D2, 0xFFCFD3D2, 0xFFCED2D2, 0xFFCDD1D1, 0xFFCED2D1, 0xFFCFD1D0, 0xFFCFD2D2, 0xFFCFD2D2, 0xFFCED2D2, 0xFFCFD3D3,
        0xFFD0D4D3, 0xFFCFD3D3, 0xFFCFD3D3, 0xFFCFD3D2, 0xFFCED2D2, 0xFFCED2D2, 0xFFCDD1D1, 0xFFCFD2D1, 0xFFCFD3D1, 0xFFCFD2D1, 0xFFCED2D1, 0xFFCED2D1, 0xFFCFD3D2, 0xFFCFD3D2, 0xFFCFD3D2, 0xFFD0D4D4, 0xFFD0D4D4, 0xFFD0D4D4, 0xFFD0D4D4, 0xFFCED2D2,
        0xFFCED2D2, 0xFFCED2D2, 0xFFCED2D2, 0xFFCED2D2, 0xFFCFD3D3, 0xFFCBCFCF, 0xFF758180, 0xFF42514E, 0xFF76817F, 0xFFF4F5F5, 0xFF404F4C, 0xFF42514E, 0xFF455451, 0xFF41504D, 0xFF42514E, 0xFF3C4749, 0xFF222F1D, 0xFF85908A, 0xFF84967C, 0xFF71835D,
        0xFF949269, 0xFF927821, 0xFFA37015, 0xFF937507, 0xFFBF9423, 0xFFD29B26, 0xFFC48B1A, 0xFFBB8A23, 0xFFC09932, 0xFFBD972C, 0xFFAB8210, 0xFF8E6412, 0xFF3A3122, 0xFF2B3F40, 0xFF293E2D, 0xFF2E3432, 0xFF2D363B, 0xFF293831, 0xFF26332C, 0xFF2A3632,
        0xFF2A3632, 0xFF2B3733, 0xFF2C3834, 0xFF2E3A36, 0xFF2D3935, 0xFF2A3632, 0xFF27332F, 0xFF2E3B34, 0xFF2C3932, 0xFF2A3730, 0xFF2A3730, 0xFF2B3831, 0xFF2B3831, 0xFF2A3730, 0xFF28352E, 0xFF29342E, 0xFF29342E, 0xFF29342E, 0xFF28332D, 0xFF27322C,
        0xFF27322C, 0xFF28332D, 0xFF28332D, 0xFF26322E, 0xFF283430, 0xFF2A3632, 0xFF2A3632, 0xFF283430, 0xFF27332F, 0xFF26322E, 0xFF27332F, 0xFF28332D, 0xFF29342E, 0xFF29342E, 0xFF27322E, 0xFF26312D, 0xFF273130, 0xFF2A3433, 0xFF2D3736, 0xFF223531,
        0xFF293535, 0xFF2D3631, 0xFF29362C, 0xFF273737, 0xFF252C32, 0xFF2F2B12, 0xFF514302, 0xFF604B0C, 0xFF604B0C, 0xFF614C0D, 0xFF604B0C, 0xFF5F4A0B, 0xFF604B0C, 0xFF634E0F, 0xFF655011, 0xFF674B0A, 0xFF5C4710, 0xFF5A4100, 0xFF967116, 0xFFA48431,
        0xFF3A2D00, 0xFF4D4E12, 0xFF94973C, 0xFF545917, 0xFF323600, 0xFF4D4D11, 0xFF4D4400, 0xFF564400, 0xFF625409, 0xFF5C5E1C, 0xFF5E7034, 0xFF7C7946, 0xFF514E19, 0xFF413E07, 0xFF3A3700, 0xFF4D4A15, 0xFF989464, 0xFF8D895C, 0xFF353008, 0xFF130E00,
        0xFF050200, 0xFF040400, 0xFF060B04, 0xFF293332, 0xFF36454A, 0xFF283B42, 0xFF142833, 0xFF354755, 0xFF303E49, 0xFF111A1F, 0xFF121315, 0xFF191816, 0xFF1F1B18, 0xFF211D1A, 0xFF181411, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0xFF65625B, 0xFF8A8780, 0xFFAEAEA6, 0xFFA7A79F, 0xFFC0C1B9, 0xFFCACBC3, 0xFFB8B9B1, 0xFFD9E1D4, 0xFFD8E0D3, 0xFFDAE2D5, 0xFFCFD7CA, 0xFFBCC4B7, 0xFFD4DCCF, 0xFFD1D9CC, 0xFFBBC3B6, 0xFFB0B4A6, 0xFFB5B9AB, 0xFFC2C6B8, 0xFFC2C6B8, 0xFFC1C5B7,
        0xFFC0C4B6, 0xFFB8BCAE, 0xFFBFC3B5, 0xFFC1C7BB, 0xFFBCC2B6, 0xFFB5BBAF, 0xFFBEC2B4, 0xFFB7BBAD, 0xFFB4B6A8, 0xFF818375, 0xFF76786A, 0xFF848780, 0xFF8B8E87, 0xFF91948D, 0xFF7A7D76, 0xFF777A73, 0xFF71746D, 0xFF6A6D66, 0xFF646760, 0xFF414137,
        0xFF534D2D, 0xFF514219, 0xFF413213, 0xFF302708, 0xFF353101, 0xFF423600, 0xFF6B5420, 0xFF6C794E, 0xFFA0A496, 0xFF9B998D, 0xFF79774E, 0xFF86854D, 0xFF7F7B55, 0xFF504D2E, 0xFF9C9D71, 0xFF4E5100, 0xFF595A14, 0xFF85865A, 0xFF8D8F7A, 0xFF5D5D55,
        0xFF4D4D43, 0xFF52543F, 0xFF373819, 0xFF3A5005, 0xFF525A1F, 0xFF4A3E0A, 0xFF614A00, 0xFF896F00, 0xFFECD65C, 0xFF998725, 0xFF65540F, 0xFFB8962B, 0xFFD7AD37, 0xFFE4A929, 0xFFE7A217, 0xFFE9A119, 0xFFE7A628, 0xFFDCA737, 0xFFB98E28, 0xFFB1891A,
        0xFFAE8921, 0xFFB29134, 0xFFA07E27, 0xFFA98227, 0xFFD6A23C, 0xFFE2A42D, 0xFFE19C1B, 0xFFE1A626, 0xFFD79E1D, 0xFFE1AB2D, 0xFFCE9D26, 0xFF875E00, 0xFF8D6E11, 0xFFA89143, 0xFF9A8742, 0xFFB1871B, 0xFFAD9529, 0xFF9A9539, 0xFF8B8F54, 0xFF8D8C77,
        0xFFA3A1A2, 0xFF2C322E, 0xFF3F5040, 0xFF445350, 0xFF42514E, 0xFF404F4C, 0xFF3F4E4B, 0xFF3F4E4B, 0xFF404F4C, 0xFF3F4E4B, 0xFFF4F5F4, 0xFF77817F, 0xFF43524F, 0xFFDFE1E1, 0xFFFFFFFF, 0xFFCCD0CF, 0xFF44524F, 0xFF404F4C, 0xFF445454, 0xFFC3C8C8,
        0xFFFFFFFF, 0xFF9DA5A4, 0xFF3F4E4C, 0xFF3F4E4B, 0xFF6B7775, 0xFFEAECEC, 0xFFFAFAFA, 0xFFFAFAFA, 0xFFFAFAFA, 0xFFFAFAFA, 0xFFFAFAFA, 0xFFFAFAFA, 0xFFFAFAFA, 0xFFFAFAFA, 0xFFFAFAFA, 0xFFFAFAFA, 0xFFFAFAFA, 0xFFFAFAFA, 0xFFFAFAFA, 0xFFFAFAFA,
        0xFFFAFAFA, 0xFFFAFAFA, 0xFFFAFAFA, 0xFFFAFAFA, 0xFFFAFAFA, 0xFFFAFAFA, 0xFFFAFAFA, 0xFFFAFAFA, 0xFFFAFAFA, 0xFFFAFAFA, 0xFFFAFAFA, 0xFFFAFAFA, 0xFFFAFAFA, 0xFFFAFAFA, 0xFFFAFAFA, 0xFFFAFAFA, 0xFFFAFAFA, 0xFFFAFAFA, 0xFFFAFAFA, 0xFFFAFAFA,
        0xFFFAFAFA, 0xFFFAFAFA, 0xFFFAFAFA, 0xFFFAFAFA, 0xFFFAFAFA, 0xFFF7F8F8, 0xFF848E8D, 0xFF41504B, 0xFF798380, 0xFFF4F5F4, 0xFF3F4E49, 0xFF3F4E49, 0xFF43524D, 0xFF44534E, 0xFF43524D, 0xFF404B43, 0xFF1F2B27, 0xFF828E8C, 0xFF879186, 0xFF787758,
        0xFFA69859, 0xFFA08627, 0xFFAA8614, 0xFF92892C, 0xFF988616, 0xFFB89A14, 0xFFCA9C12, 0xFFC08608, 0xFFC98C19, 0xFFD49B2A, 0xFFCA9521, 0xFFB37804, 0xFF6B4E0C, 0xFF6A6853, 0xFF313325, 0xFF616A4D, 0xFF213510, 0xFF617250, 0xFF2D3413, 0xFF5E6251,
        0xFF2D3619, 0xFF4D5C33, 0xFF374722, 0xFF4C5A40, 0xFF384430, 0xFF454D36, 0xFF42482C, 0xFF474B30, 0xFF464E37, 0xFF374231, 0xFF44543A, 0xFF2D411E, 0xFF4F6340, 0xFF293821, 0xFF4A564A, 0xFF252D15, 0xFF5B6856, 0xFF1E2E21, 0xFF5D6556, 0xFF373520,
        0xFF655F45, 0xFF3D3F29, 0xFF545F4E, 0xFF3F3C19, 0xFF5B5835, 0xFF43401D, 0xFF595633, 0xFF45421F, 0xFF55522F, 0xFF4D4A27, 0xFF4E4B28, 0xFF53502D, 0xFF484522, 0xFF5B5835, 0xFF393613, 0xFF63603D, 0xFF3A3714, 0xFF5F5C39, 0xFF3A3714, 0xFF5B5B4F,
        0xFF3B3C2A, 0xFF57594C, 0xFF2D2825, 0xFF6E5D4D, 0xFF70531B, 0xFFC99C49, 0xFF905801, 0xFF5B4909, 0xFF5C4A0A, 0xFF5F4D0D, 0xFF634F10, 0xFF624E0F, 0xFF614C0D, 0xFF624D0E, 0xFF655011, 0xFF63500E, 0xFF614F11, 0xFF4D3C04, 0xFF90834F, 0xFFB5AB78,
        0xFF615B29, 0xFF8D8A55, 0xFFC8C78F, 0xFF8E8647, 0xFF4B5324, 0xFF777E52, 0xFF7B702E, 0xFF715F0D, 0xFFAFA864, 0xFFC3C59D, 0xFFDCDCC4, 0xFFB8B395, 0xFF777252, 0xFF4A4822, 0xFF595933, 0xFFB6B893, 0xFFA1A584, 0xFFA1A487, 0xFF80846B, 0xFF565842,
        0xFF534F34, 0xFF4C452B, 0xFF373828, 0xFF323E3C, 0xFF3B4F56, 0xFF394C53, 0xFF394448, 0xFF3C4A57, 0xFF3C4648, 0xFF434739, 0xFF484728, 0xFF646031, 0xFF807942, 0xFF80773C, 0xFF827A3C, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0xFF67645D, 0xFF86837C, 0xFFAFAFA7, 0xFFBABAB2, 0xFFC1C2BA, 0xFFBDBEB6, 0xFFC7C8C0, 0xFFD1D9CC, 0xFFCCD4C7, 0xFFDAE2D5, 0xFFE0E8DB, 0xFFC5CDC0, 0xFFBFC7BA, 0xFFBEC6B9, 0xFFC5CDC0, 0xFFC6CABC, 0xFFC6CABC, 0xFFBFC3B5, 0xFFBFC3B5, 0xFFC0C4B6,
        0xFFC2C6B8, 0xFFCCD0C2, 0xFFD0D4C6, 0xFFBFC5B9, 0xFFBFC5B9, 0xFFBDC3B7, 0xFFBFC3B5, 0xFFB4B8AA, 0xFFBCBEB0, 0xFF8C8E80, 0xFF6F7163, 0xFF82857E, 0xFF8F928B, 0xFF92958E, 0xFF7A7D76, 0xFF7C7F78, 0xFF747770, 0xFF696C65, 0xFF696C65, 0xFF414135,
        0xFF484220, 0xFF43340B, 0xFF38290A, 0xFF2E2506, 0xFF343000, 0xFF372A00, 0xFF503905, 0xFF575D2F, 0xFF52503B, 0xFF5D5644, 0xFF777042, 0xFF7D773B, 0xFF544C1E, 0xFF473F1A, 0xFFA9A575, 0xFF9C9B6B, 0xFF7F8056, 0xFF8B8F6E, 0xFF81866F, 0xFF313828,
        0xFF525A4D, 0xFF464E43, 0xFF434B40, 0xFFC0C9B8, 0xFFB8B991, 0xFF706621, 0xFF644D00, 0xFF705007, 0xFFF0D28C, 0xFFA5903F, 0xFF7E7111, 0xFFC5AB4C, 0xFFBE9931, 0xFFD7A32D, 0xFFEDAC2C, 0xFFE9A31F, 0xFFE7A626, 0xFFE4AC35, 0xFFD6A232, 0xFFBD901D,
        0xFFB0861A, 0xFFAD8726, 0xFF9C771C, 0xFFA67D21, 0xFFCA9933, 0xFFD29926, 0xFFD19115, 0xFFE0A42A, 0xFFDFA327, 0xFFD7A021, 0xFFDCAA2D, 0xFFC19623, 0xFF8B6804, 0xFF624600, 0xFF6C540A, 0xFF8C6925, 0xFFB5A35B, 0xFF9C9B58, 0xFF888B5E, 0xFF898876,
        0xFF9A9695, 0xFF2E3430, 0xFF445447, 0xFF43524F, 0xFF42514E, 0xFF404F4C, 0xFF404F4C, 0xFF41504D, 0xFF41504D, 0xFF404F4C, 0xFFF4F5F4, 0xFF747F7D, 0xFF41504D, 0xFFDFE1E1, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFA3AAA9, 0xFF435251, 0xFF455554, 0xFF546261,
        0xFFE3E5E5, 0xFFFDFDFD, 0xFF6E7978, 0xFF3F4E4C, 0xFF3F4E4B, 0xFF485857, 0xFF445352, 0xFF455453, 0xFF475655, 0xFF475552, 0xFF475552, 0xFF465451, 0xFF4A5A59, 0xFF475553, 0xFF42504D, 0xFF485653, 0xFF495958, 0xFF475756, 0xFF495959, 0xFF404F4F,
        0xFF465554, 0xFF495957, 0xFF495956, 0xFF485855, 0xFF485855, 0xFF465552, 0xFF475654, 0xFF475651, 0xFF4A5954, 0xFF4B5A55, 0xFF495853, 0xFF485752, 0xFF495853, 0xFF485752, 0xFF46544F, 0xFF495958, 0xFF4A5A59, 0xFF4C5C5A, 0xFF4E5F5E, 0xFF455453,
        0xFF455453, 0xFF475655, 0xFF455453, 0xFF445352, 0xFF465554, 0xFF465554, 0xFF3E4E4D, 0xFF41504B, 0xFF7A8481, 0xFFF4F5F5, 0xFF404F4A, 0xFF404F4A, 0xFF42514C, 0xFF42514C, 0xFF404F4A, 0xFF3C4741, 0xFF26322E, 0xFF85908A, 0xFF899182, 0xFF707354,
        0xFF918A54, 0xFFA39346, 0xFFA48F34, 0xFFBFB97F, 0xFF9F903F, 0xFFA8891D, 0xFFC69820, 0xFFCA9219, 0xFFCE901F, 0xFFD09724, 0xFFCA931E, 0xFFD59B1E, 0xFF9B7D27, 0xFFAAA470, 0xFF666338, 0xFF918F66, 0xFF5D6744, 0xFF8A9877, 0xFF595D3A, 0xFF888967,
        0xFF5B6135, 0xFF7E8854, 0xFF6F7B4D, 0xFF828C69, 0xFF606748, 0xFF6E724F, 0xFF75784B, 0xFF6F6C43, 0xFF8E8B6A, 0xFF626248, 0xFF777A5B, 0xFF5F643B, 0xFF878A5B, 0xFF68663F, 0xFF9D987A, 0xFF665A28, 0xFF7A7649, 0xFF62633B, 0xFFA19C72, 0xFF948655,
        0xFFB0A06C, 0xFF867E4D, 0xFF9E9F73, 0xFF918A56, 0xFFA39C68, 0xFF958E5A, 0xFFA49D69, 0xFF968F5B, 0xFF9F9864, 0xFF99925E, 0xFF97905C, 0xFF9D9662, 0xFF98915D, 0xFFA29B67, 0xFF908955, 0xFFA69F6B, 0xFF928B57, 0xFFA69F6B, 0xFF8E8753, 0xFFAFA26E,
        0xFF8E854A, 0xFF969360, 0xFF605D3E, 0xFF8F8C6B, 0xFF5F5D24, 0xFF98914B, 0xFFBEB373, 0xFF614F0F, 0xFF5E4C0C, 0xFF5D4B0B, 0xFF614D0E, 0xFF614D0E, 0xFF5F4A0B, 0xFF604B0C, 0xFF655011, 0xFF604D0B, 0xFF5F4D0F, 0xFF57460E, 0xFF665924, 0xFF897F4C,
        0xFF746E3A, 0xFF88854E, 0xFF9B9A61, 0xFF9E8E81, 0xFF3E3E3E, 0xFF797A74, 0xFF9D8D69, 0xFF947E42, 0xFF827942, 0xFFA2A47F, 0xFF8E8E76, 0xFF575330, 0xFF726E49, 0xFF514F28, 0xFF606139, 0xFFA5A780, 0xFFACB08D, 0xFFA2A586, 0xFFA1A58A, 0xFF9EA08A,
        0xFFA5A186, 0xFF9D967C, 0xFF6D6E5E, 0xFF414D4B, 0xFF30444B, 0xFF2F4249, 0xFF3A4549, 0xFF2C3C49, 0xFF4A5656, 0xFF6B735C, 0xFF828556, 0xFF9A9B57, 0xFFADAC59, 0xFFB4B356, 0xFFBBBB59, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0xFF615E57, 0xFF8D8A83, 0xFFC2C2BA, 0xFFBEBEB6, 0xFFBFC0B8, 0xFFCACBC3, 0xFFDBDCD4, 0xFFD1D7CB, 0xFFCAD0C4, 0xFFC0C6BA, 0xFFB3B9AD, 0xFFB1B7AB, 0xFFC9CFC3, 0xFFC3C9BD, 0xFFB1B7AB, 0xFFB5B9AB, 0xFFC7CBBD, 0xFFC7CBBD, 0xFFC4C8BA, 0xFFBDC1B3,
        0xFFC0C4B6, 0xFFD4D8CA, 0xFFD1D5C7, 0xFFB5BBAF, 0xFFB7BDB1, 0xFFB6BCB0, 0xFFADB1A3, 0xFFA1A597, 0xFFB6B8AA, 0xFF8B8D7F, 0xFF616355, 0xFF7E8178, 0xFF8D9087, 0xFF8F9289, 0xFF767970, 0xFF7B7E75, 0xFF71746B, 0xFF666960, 0xFF6B6E65, 0xFF434134,
        0xFF403814, 0xFF3A2B00, 0xFF372706, 0xFF2F2607, 0xFF3C3606, 0xFF3B2E00, 0xFF483100, 0xFF47410D, 0xFF443919, 0xFF524324, 0xFF5D4D19, 0xFF5C4D0C, 0xFF52430A, 0xFF5D4F1E, 0xFF7C703C, 0xFF777161, 0xFF7A7863, 0xFFA9A98D, 0xFFA1A686, 0xFF424B2E,
        0xFF4E5A46, 0xFF404B43, 0xFF586464, 0xFFC7C7C9, 0xFFB2AE89, 0xFFA69A50, 0xFF9F883A, 0xFF7F5F22, 0xFFB3945D, 0xFF8A7526, 0xFFC5BA50, 0xFFA18F43, 0xFF7E640F, 0xFFB98F2B, 0xFFEDB544, 0xFFE2A226, 0xFFE1A020, 0xFFE6A726, 0xFFDEA325, 0xFFDEA730,
        0xFFC08E1D, 0xFFB4881F, 0xFFB18725, 0xFFB78D2B, 0xFFC69630, 0xFFC99527, 0xFFD59C28, 0xFFC7891A, 0xFFD99C27, 0xFFDDA325, 0xFFDCA725, 0xFFD7A72B, 0xFFC59B2D, 0xFF8A6608, 0xFF553500, 0xFF776245, 0xFFB0A682, 0xFF878961, 0xFF757B59, 0xFF858374,
        0xFF95928D, 0xFF2B322B, 0xFF324438, 0xFF3F4E4B, 0xFF3F4E4B, 0xFF3E4D4A, 0xFF3F4E4B, 0xFF404F4C, 0xFF404F4C, 0xFF3F4E4B, 0xFFF4F4F4, 0xFF737E7C, 0xFF404F4C, 0xFFDFE1E1, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFA2AAA9, 0xFF455555, 0xFF475757, 0xFF556363,
        0xFFE3E5E5, 0xFFFDFDFD, 0xFF727E7E, 0xFF41514F, 0xFF404F4D, 0xFF495754, 0xFF4D5B58, 0xFF4A5855, 0xFF475555, 0xFF485653, 0xFF495754, 0xFF495754, 0xFF4D5B5A, 0xFF455452, 0xFF44524F, 0xFF4B5856, 0xFF4C5B5A, 0xFF4B5A59, 0xFF4C5B5B, 0xFF4B5A5A,
        0xFF485756, 0xFF495858, 0xFF4D5C5B, 0xFF4A5958, 0xFF495857, 0xFF4C5A59, 0xFF4B5957, 0xFF4A5853, 0xFF4D5B56, 0xFF4E5C58, 0xFF4C5A56, 0xFF4B5955, 0xFF4D5B56, 0xFF4B5954, 0xFF485651, 0xFF4C5B58, 0xFF4C5B59, 0xFF4D5C59, 0xFF4E5E5C, 0xFF485754,
        0xFF485756, 0xFF485756, 0xFF475655, 0xFF475655, 0xFF495857, 0xFF485857, 0xFF40504F, 0xFF404F4A, 0xFF7A8481, 0xFFF4F5F5, 0xFF42514C, 0xFF41504B, 0xFF42514C, 0xFF41504B, 0xFF3E4D48, 0xFF3B4745, 0xFF293430, 0xFF838D84, 0xFF8C9582, 0xFF7B8062,
        0xFF8C8E66, 0xFFA6A674, 0xFF8D8C54, 0xFF999782, 0xFF685B2E, 0xFF715605, 0xFFB58A21, 0xFFD39C28, 0xFFD39824, 0xFFD19622, 0xFFCE9521, 0xFFD4991B, 0xFFA6871B, 0xFF8F842B, 0xFF6E6113, 0xFF796D31, 0xFF7E7C55, 0xFF868A67, 0xFF7C7D53, 0xFF7E7941,
        0xFF5D5C1C, 0xFF707233, 0xFF8A8D56, 0xFF9FA074, 0xFF8C8B5F, 0xFF77743D, 0xFF6A6724, 0xFF756B2E, 0xFF9E9463, 0xFF908760, 0xFF7F764D, 0xFF7F763F, 0xFF6C6121, 0xFF766424, 0xFF9B854B, 0xFF83894D, 0xFF798347, 0xFF828848, 0xFF898031, 0xFFA0821F,
        0xFFB98E1B, 0xFFB2840C, 0xFFC0931C, 0xFFAD8321, 0xFFB28826, 0xFFB38927, 0xFFB98F2D, 0xFFB68C2A, 0xFFB98F2D, 0xFFB88E2C, 0xFFB68C2A, 0xFFB58B29, 0xFFB68C2A, 0xFFB98F2D, 0xFFB58B29, 0xFFB68C2A, 0xFFB88E2C, 0xFFBB912F, 0xFFAF8523, 0xFFC59026,
        0xFFB48714, 0xFFB08B23, 0xFF6F5204, 0xFF5D4B09, 0xFF484100, 0xFF5F5F1F, 0xFFA3A576, 0xFF655313, 0xFF5E4C0C, 0xFF5B4909, 0xFF604C0D, 0xFF604C0D, 0xFF5D4809, 0xFF5E490A, 0xFF644F10, 0xFF665110, 0xFF5E4A0D, 0xFF5F4C12, 0xFF493A03, 0xFF3E3200,
        0xFF3C3500, 0xFF514C12, 0xFF3F3D02, 0xFF4A4203, 0xFF384011, 0xFF434B1A, 0xFF504600, 0xFF554400, 0xFF514B00, 0xFF606431, 0xFF7B7D58, 0xFF7F7B55, 0xFF4D4A21, 0xFF39380C, 0xFF36370B, 0xFF4B4E23, 0xFF333712, 0xFF4F5233, 0xFF33381A, 0xFF121400,
        0xFF151100, 0xFF1D1600, 0xFF1D1E0E, 0xFF293533, 0xFF33474E, 0xFF24373E, 0xFF162125, 0xFF2F414D, 0xFF2C3B42, 0xFF192123, 0xFF11100E, 0xFF19110F, 0xFF221618, 0xFF291923, 0xFF291926, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0xFF5B5851, 0xFF8A8780, 0xFFBFBFB7, 0xFFAFAFA7, 0xFFAEAFA7, 0xFFC0C1B9, 0xFFD4D5CD, 0xFFBBC1B5, 0xFFC9CFC3, 0xFFD7DDD1, 0xFFCED4C8, 0xFFBFC5B9, 0xFFBDC3B7, 0xFFB2B8AC, 0xFFACB2A6, 0xFFA7AB9D, 0xFFBFC3B5, 0xFFD1D5C7, 0xFFCCD0C2, 0xFFBDC1B3,
        0xFFBFC3B5, 0xFFC4C8BA, 0xFFB7BBAD, 0xFFADB3A7, 0xFFB6BCB0, 0xFFBCC2B6, 0xFFB9BDAF, 0xFFB7BBAD, 0xFFC5C7B9, 0xFF939587, 0xFF6A6C5E, 0xFF81847B, 0xFF898C83, 0xFF8D9087, 0xFF797C73, 0xFF797C73, 0xFF72756C, 0xFF6C6F66, 0xFF6D7067, 0xFF444233,
        0xFF3E3611, 0xFF3C2E01, 0xFF3A2A09, 0xFF2E2305, 0xFF3D3608, 0xFF453600, 0xFF513706, 0xFF4E3F06, 0xFF4F3D0F, 0xFF543D11, 0xFF553C04, 0xFF584100, 0xFF5B4603, 0xFF665318, 0xFF54430D, 0xFF3E3313, 0xFF3D330F, 0xFF544B22, 0xFF4F481C, 0xFF322F06,
        0xFF282909, 0xFF252912, 0xFF252C1C, 0xFF545126, 0xFF5E5625, 0xFF5D4A10, 0xFF8B732B, 0xFFAF9541, 0xFF9A822A, 0xFF95832D, 0xFF7B6C19, 0xFF594F12, 0xFF534000, 0xFF7E5F0D, 0xFFBB8D2F, 0xFFE0A638, 0xFFE9A92E, 0xFFE2A21E, 0xFFE5A319, 0xFFE6A82D,
        0xFFD29721, 0xFFC59121, 0xFFBA8B21, 0xFFB88B24, 0xFFBD9128, 0xFFC29329, 0xFFCE9E32, 0xFFDB9D36, 0xFFCF9323, 0xFFD89C20, 0xFFE0A722, 0xFFDBA624, 0xFFE1B039, 0xFFC1962E, 0xFF805800, 0xFF716348, 0xFF979473, 0xFF7C845F, 0xFF808766, 0xFF8F8E79,
        0xFF9C9A8E, 0xFF343B33, 0xFF3C4D45, 0xFF3B4A47, 0xFF3B4A47, 0xFF3B4A47, 0xFF3C4B48, 0xFF3E4D4A, 0xFF3E4D4A, 0xFF3C4B48, 0xFFF3F4F4, 0xFF727D7B, 0xFF404F4C, 0xFFDFE1E1, 0xFFFFFFFF, 0xFFCDD1D1, 0xFF475656, 0xFF455555, 0xFF455555, 0xFFC4C9C9,
        0xFFFFFFFF, 0xFF9EA6A5, 0xFF445454, 0xFF425251, 0xFF6F7B79, 0xFFECEDED, 0xFFFCFCFC, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFDFDFD,
        0xFFFDFDFD, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFDFDFD,
        0xFFFDFDFD, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFDFDFD, 0xFFFAFAFA, 0xFF889291, 0xFF404F4A, 0xFF7A8481, 0xFFF4F5F5, 0xFF42514C, 0xFF41504B, 0xFF43524D, 0xFF41504B, 0xFF3E4D48, 0xFF3F4A4C, 0xFF26312D, 0xFF858D80, 0xFF89917A, 0xFF7A8366,
        0xFF838D6B, 0xFF949F7D, 0xFF677554, 0xFF868A7B, 0xFF5C5731, 0xFF6A540B, 0xFFB8912A, 0xFFD3A02A, 0xFFD0961B, 0xFFCD921C, 0xFFD09724, 0xFFD19421, 0xFFB29223, 0xFF7E7005, 0xFF9C8A28, 0xFF7E681F, 0xFF989061, 0xFF7F8152, 0xFFA0A064, 0xFF8C843C,
        0xFF7F792F, 0xFF76732C, 0xFF89874A, 0xFF807C4C, 0xFF8B8551, 0xFF817938, 0xFF827B2A, 0xFF7B7529, 0xFF928A4C, 0xFF8A8251, 0xFF908653, 0xFF928747, 0xFF918134, 0xFF8B7625, 0xFF957C2C, 0xFF858A74, 0xFF6B735B, 0xFF757B59, 0xFF615620, 0xFF906D1C,
        0xFFB67F1B, 0xFFD09026, 0xFFCF8F25, 0xFFBD850E, 0xFFBA820B, 0xFFC28A13, 0xFFC28A13, 0xFFC48C15, 0xFFC48C15, 0xFFC68E17, 0xFFC68E17, 0xFFC18912, 0xFFC28A13, 0xFFC89019, 0xFFC28A13, 0xFFC08811, 0xFFC58D16, 0xFFC58D16, 0xFFBD850E, 0xFFC28413,
        0xFFC78F14, 0xFFC6931E, 0xFF9F7418, 0xFF5A3900, 0xFF5A4800, 0xFF4F460D, 0xFF847E5C, 0xFF655313, 0xFF5D4B0B, 0xFF594707, 0xFF604C0D, 0xFF614D0E, 0xFF5D4809, 0xFF5E490A, 0xFF634E0F, 0xFF624A0A, 0xFF5C4708, 0xFF624E13, 0xFF604F17, 0xFF4E410A,
        0xFF655C21, 0xFFADA769, 0xFF969252, 0xFFA1985F, 0xFF858C63, 0xFF979D77, 0xFF92864A, 0xFF7A661F, 0xFFA79E67, 0xFFC4C4A8, 0xFFE0DED2, 0xFFA6A378, 0xFF6B683B, 0xFF4C4B1D, 0xFF5A5C2D, 0xFFACAF84, 0xFF878B66, 0xFF878A6B, 0xFF65694E, 0xFF4F513B,
        0xFF545035, 0xFF544D33, 0xFF3B3C2C, 0xFF2D3937, 0xFF30444B, 0xFF31444B, 0xFF374246, 0xFF3C4F60, 0xFF3C4D55, 0xFF313C36, 0xFF404330, 0xFF595839, 0xFF66623D, 0xFF69643C, 0xFF5E5931, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0xFF65625B, 0xFF86837C, 0xFFB3B3AB, 0xFFB7B7AF, 0xFFAEAFA7, 0xFFAAABA3, 0xFFC1C2BA, 0xFFBDC3B7, 0xFFBAC0B4, 0xFFC0C6BA, 0xFFBFC5B9, 0xFFCAD0C4, 0xFFD5DBCF, 0xFFCAD0C4, 0xFFB9BFB3, 0xFFC1C5B7, 0xFFB5B9AB, 0xFFC5C9BB, 0xFFBEC2B4, 0xFFB5B9AB,
        0xFFC0C4B6, 0xFFBDC1B3, 0xFFBCC0B2, 0xFFAFB5A9, 0xFFB0B6AA, 0xFFABB1A5, 0xFFA8AC9E, 0xFFB4B8AA, 0xFFBBBDAF, 0xFF909284, 0xFF7D7F71, 0xFF8D9085, 0xFF878A7F, 0xFF909388, 0xFF818479, 0xFF7A7D72, 0xFF787B70, 0xFF787B70, 0xFF707368, 0xFF43402D,
        0xFF3F360D, 0xFF402F01, 0xFF3C2C0A, 0xFF2C2101, 0xFF3A3204, 0xFF423200, 0xFF4E3304, 0xFF523E03, 0xFF533D04, 0xFF563B06, 0xFF5F420A, 0xFF664B08, 0xFF5E4800, 0xFF5E4B07, 0xFF58440F, 0xFF5E5007, 0xFF564500, 0xFF594405, 0xFF523C02, 0xFF543F0A,
        0xFF675523, 0xFF5C5020, 0xFF3E3605, 0xFF514B00, 0xFF524308, 0xFF614B1A, 0xFF523800, 0xFF6B5300, 0xFF685300, 0xFF614D04, 0xFF523E0B, 0xFF50450D, 0xFF65551A, 0xFF583F00, 0xFF755103, 0xFFC29436, 0xFFE1A93A, 0xFFDDA021, 0xFFE3A31B, 0xFFDD9C1E,
        0xFFE2A429, 0xFFDCA129, 0xFFBF8A18, 0xFFB18112, 0xFFBA8B21, 0xFFBC9027, 0xFFBD9029, 0xFFD59C35, 0xFFC89021, 0xFFCA9015, 0xFFD59C19, 0xFFDCA524, 0xFFDBA52B, 0xFFD4A233, 0xFFCF9E38, 0xFF766426, 0xFF625E21, 0xFF5C632F, 0xFF7D855C, 0xFF8B8E71,
        0xFF9C9D8D, 0xFF2D352A, 0xFF32433D, 0xFF3C4B48, 0xFF3B4A47, 0xFF3B4A47, 0xFF3D4C49, 0xFF3E4D4A, 0xFF3E4D4A, 0xFF3C4B48, 0xFFF3F4F4, 0xFF737E7C, 0xFF404F4C, 0xFFE6E8E8, 0xFFEDEEEE, 0xFF5B6868, 0xFF455353, 0xFF445350, 0xFF9DA4A3, 0xFFFFFFFF,
        0xFFC8CDCC, 0xFF4D5C5B, 0xFF465656, 0xFF566464, 0xFFE4E7E6, 0xFFFFFFFF, 0xFFCED1D1, 0xFFCACECE, 0xFFCACFCE, 0xFFCCD0CF, 0xFFCBCFCF, 0xFFCBCFCE, 0xFFCCD0CF, 0xFFCCD0D0, 0xFFCBD0CF, 0xFFCCD1D0, 0xFFCDD1D1, 0xFFCCD0D0, 0xFFCDD1D1, 0xFFCDD1D1,
        0xFFCCD0D0, 0xFFCBCFCF, 0xFFCDD1D1, 0xFFCDD2D2, 0xFFCED2D2, 0xFFCDD2D1, 0xFFCCD0D0, 0xFFCCD0D0, 0xFFCDD2D1, 0xFFCCD1D0, 0xFFCCD0D0, 0xFFCCD0CF, 0xFFCCD0D0, 0xFFCBCFCF, 0xFFCCD0CF, 0xFFCBD0CF, 0xFFCBD0CF, 0xFFCCD0CF, 0xFFCCD0CF, 0xFFCCD1CF,
        0xFFCCD0CF, 0xFFCBCFCE, 0xFFCBCFCF, 0xFFCBCFCF, 0xFFCBCFCF, 0xFFC8CDCC, 0xFF717D7D, 0xFF3F4E49, 0xFF798380, 0xFFF4F5F4, 0xFF404F4A, 0xFF41504B, 0xFF43524D, 0xFF43524D, 0xFF41504B, 0xFF404B4D, 0xFF232E2A, 0xFF90988B, 0xFF8E967F, 0xFF7E876A,
        0xFF8A9472, 0xFF8E9977, 0xFF677554, 0xFF596138, 0xFF4C4912, 0xFF665402, 0xFFB69126, 0xFFCE9C21, 0xFFCC9516, 0xFFCF941C, 0xFFD09726, 0xFFCF9331, 0xFF9B7D1D, 0xFF918424, 0xFF9A892D, 0xFF746017, 0xFF554E1A, 0xFF686B34, 0xFF73742E, 0xFF68611D,
        0xFF5B5710, 0xFF696825, 0xFF616027, 0xFF615F2F, 0xFF545020, 0xFF615B1D, 0xFF605A0C, 0xFF5D6417, 0xFF5B5F22, 0xFF666837, 0xFF535523, 0xFF686828, 0xFF646114, 0xFF625A09, 0xFF6C5F10, 0xFF7B692B, 0xFF6C6328, 0xFF4D4F10, 0xFF686820, 0xFF736711,
        0xFFA99536, 0xFFA49031, 0xFFA7933A, 0xFFAD9032, 0xFFAB8E30, 0xFFAC8F31, 0xFFAC8F31, 0xFFA88B2D, 0xFFAA8D2F, 0xFFA98C2E, 0xFFAD9032, 0xFFAB8E30, 0xFFA78A2C, 0xFFB6993B, 0xFFA48729, 0xFFAE9133, 0xFFA6892B, 0xFFAF9234, 0xFFA38628, 0xFFA79042,
        0xFFA28D30, 0xFFAA953A, 0xFF9B853C, 0xFF7A6627, 0xFF584A09, 0xFF50450D, 0xFF807652, 0xFF604E0E, 0xFF5A4808, 0xFF594707, 0xFF604C0D, 0xFF624E0F, 0xFF5F4A0B, 0xFF5F4A0B, 0xFF624D0E, 0xFF684F0D, 0xFF674F11, 0xFF5F4B10, 0xFF5C490F, 0xFF4F4005,
        0xFF5D5315, 0xFF898140, 0xFF6D6824, 0xFF8B7C67, 0xFF404139, 0xFF747668, 0xFF847649, 0xFF7F6A27, 0xFF7D7537, 0xFF979971, 0xFF88886C, 0xFF575427, 0xFF6C693C, 0xFF4D4C1E, 0xFF5D5F30, 0xFFA4A67E, 0xFFAAAE8B, 0xFF9FA285, 0xFF93977E, 0xFF9C9E88,
        0xFFA29E83, 0xFF9A9379, 0xFF6B6C5C, 0xFF414D4B, 0xFF33474E, 0xFF34474E, 0xFF404B4F, 0xFF30444B, 0xFF4C5D57, 0xFF637054, 0xFF8A935E, 0xFFAEB266, 0xFFBCBF64, 0xFFC4C963, 0xFFC1C75D, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0xFF5F5C55, 0xFF8D8A83, 0xFFC4C4BC, 0xFFC4C4BC, 0xFFC3C4BC, 0xFFBBBCB4, 0xFFB7B8B0, 0xFFA9ACA1, 0xFFBFC2B7, 0xFFD8DBD0, 0xFFC6C9BE, 0xFFB4B7AC, 0xFFB5B8AD, 0xFFBABDB2, 0xFFB5B8AD, 0xFFBFC3B5, 0xFFA7AB9D, 0xFFC5C9BB, 0xFFCACEC0, 0xFFBBBFB1,
        0xFFB9BDAF, 0xFFACB0A2, 0xFFB5B9AB, 0xFFA9AFA3, 0xFFB1B7AB, 0xFFB5BBAF, 0xFFBBBFB1, 0xFFC9CDBF, 0xFFBCBEB0, 0xFF828476, 0xFF6F7163, 0xFF909388, 0xFF808378, 0xFF8C8F84, 0xFF82857A, 0xFF7A7D72, 0xFF7A7D72, 0xFF7A7D72, 0xFF696C61, 0xFF413F2A,
        0xFF3E350A, 0xFF403000, 0xFF412F0B, 0xFF352808, 0xFF40380A, 0xFF453400, 0xFF4D3205, 0xFF4F3A01, 0xFF584304, 0xFF5D4308, 0xFF5F440F, 0xFF5C4303, 0xFF5F4C00, 0xFF564500, 0xFF5B4B1A, 0xFF524800, 0xFF5A4900, 0xFF674D03, 0xFF6B4A05, 0xFF5C3900,
        0xFF9E7F3C, 0xFF79611B, 0xFF584400, 0xFF594709, 0xFF5B470C, 0xFF583F07, 0xFF664B13, 0xFF64470B, 0xFF5E450C, 0xFF5D4614, 0xFF604B20, 0xFF5A4A0C, 0xFF624F14, 0xFF553F05, 0xFF5F4101, 0xFF825D0F, 0xFFB6892B, 0xFFE7B240, 0xFFE7AD2F, 0xFFE2A025,
        0xFFE6A62A, 0xFFE1A529, 0xFFD39B20, 0xFFC4901A, 0xFFB88617, 0xFFB5861C, 0xFFBC8F2A, 0xFFC19123, 0xFFCC9A29, 0xFFD7A02B, 0xFFD29A1F, 0xFFD3991E, 0xFFCB931C, 0xFFD49C2B, 0xFFDFA93C, 0xFFBF9A2F, 0xFF7A650A, 0xFF666524, 0xFF848B61, 0xFF8E9277,
        0xFFA3A796, 0xFF2C3729, 0xFF324238, 0xFF3F4E4B, 0xFF3E4D4A, 0xFF3E4D4A, 0xFF3E4D4A, 0xFF3F4E4B, 0xFF404F4C, 0xFF3E4D4A, 0xFFF4F4F4, 0xFF727D7B, 0xFF41504D, 0xFFE0E2E2, 0xFF7B8686, 0xFF414F4F, 0xFF425050, 0xFF737E7C, 0xFFFDFDFD, 0xFFE7E9E9,
        0xFF576665, 0xFF465554, 0xFF485656, 0xFFC6CACA, 0xFFFFFFFF, 0xFF9DA4A3, 0xFF445351, 0xFF42514E, 0xFF3C4C4B, 0xFF3F4F4E, 0xFF3F504D, 0xFF40504D, 0xFF435252, 0xFF425252, 0xFF445453, 0xFF435352, 0xFF445454, 0xFF425252, 0xFF455555, 0xFF475656,
        0xFF435352, 0xFF425251, 0xFF445453, 0xFF485857, 0xFF495958, 0xFF465655, 0xFF435352, 0xFF425251, 0xFF445453, 0xFF415150, 0xFF3F4F4E, 0xFF445452, 0xFF455553, 0xFF435351, 0xFF435351, 0xFF435350, 0xFF435350, 0xFF435350, 0xFF435350, 0xFF44554F,
        0xFF42534D, 0xFF3F504A, 0xFF3E4F4A, 0xFF3F4F4E, 0xFF3B4B4A, 0xFF3B4B4A, 0xFF3E4E4D, 0xFF3F4E49, 0xFF78837F, 0xFFF4F5F4, 0xFF404F4A, 0xFF41504B, 0xFF43524D, 0xFF44534E, 0xFF42514C, 0xFF3F4B49, 0xFF1E2925, 0xFF89938A, 0xFF8A9380, 0xFF797E60,
        0xFF85875F, 0xFF727240, 0xFF4D4C14, 0xFF474E0A, 0xFF4E4A01, 0xFF6D5701, 0xFFBB952A, 0xFFD09E23, 0xFFD59E1D, 0xFFD49C23, 0xFFCD9826, 0xFFC08A30, 0xFF6F550A, 0xFFC6C181, 0xFFA9A163, 0xFFC9C089, 0xFF68693D, 0xFFB4BA8A, 0xFF75763C, 0xFFB2B181,
        0xFF787B46, 0xFF9DA46E, 0xFF747C4D, 0xFFA8AE8C, 0xFF797D5C, 0xFFA1A276, 0xFF8B8C52, 0xFF828D53, 0xFF97A375, 0xFF818D69, 0xFF95A17B, 0xFF687442, 0xFF9AA468, 0xFF71753A, 0xFFB0AF77, 0xFF897B56, 0xFFA9A985, 0xFF636F4B, 0xFFA8B589, 0xFF6F723B,
        0xFFCAC488, 0xFFA69D66, 0xFFBCB684, 0xFFB7A865, 0xFFC5B673, 0xFFB6A764, 0xFFC2B370, 0xFFB3A461, 0xFFC0B16E, 0xFFB6A764, 0xFFC1B26F, 0xFFBCAD6A, 0xFFB7A865, 0xFFC9BA77, 0xFFB1A25F, 0xFFC4B572, 0xFFB1A25F, 0xFFC4B572, 0xFFB2A360, 0xFFC8C487,
        0xFFA5A359, 0xFFCBC47C, 0xFFA89C60, 0xFFC6B378, 0xFF9A8438, 0xFFB09546, 0xFF9B7C39, 0xFF5A4808, 0xFF594707, 0xFF5B4909, 0xFF604C0D, 0xFF624E0F, 0xFF614C0D, 0xFF604B0C, 0xFF614C0D, 0xFF634A08, 0xFF604707, 0xFF624C10, 0xFF5E4A0F, 0xFF5C4C10,
        0xFF534806, 0xFF4B4300, 0xFF494300, 0xFF4E4701, 0xFF38410A, 0xFF3E470E, 0xFF4A4200, 0xFF594A00, 0xFF565300, 0xFF6B722C, 0xFF6C7139, 0xFF6D6A3F, 0xFF504D22, 0xFF3B3A0E, 0xFF34350B, 0xFF43451E, 0xFF393C1D, 0xFF555741, 0xFF3E412E, 0xFF101200,
        0xFF110D00, 0xFF171000, 0xFF171808, 0xFF25312F, 0xFF2F434A, 0xFF20333A, 0xFF131E22, 0xFF33444E, 0xFF2C393F, 0xFF1E2221, 0xFF1C1910, 0xFF261E13, 0xFF2C1F16, 0xFF2D1E19, 0xFF2B1B1B, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0xFF535049, 0xFF918E87, 0xFFCDCDC5, 0xFFB8B8B0, 0xFFBFC0B8, 0xFFC9CAC2, 0xFFB8B9B1, 0xFFA8ABA0, 0xFFAAADA2, 0xFFC8CBC0, 0xFFC3C6BB, 0xFFADB0A5, 0xFFA2A59A, 0xFFB9BCB1, 0xFFCDD0C5, 0xFFC2C6B8, 0xFFB2B6A8, 0xFFC8CCBE, 0xFFC0C4B6, 0xFFA5A99B,
        0xFFA8AC9E, 0xFFB0B4A6, 0xFFB8BCAE, 0xFFABB1A5, 0xFFB2B8AC, 0xFFB7BDB1, 0xFFB6BAAC, 0xFFC0C4B6, 0xFFB6B8AA, 0xFF828476, 0xFF6A6C5E, 0xFF8A8D82, 0xFF7A7D72, 0xFF83867B, 0xFF7D8075, 0xFF7B7E73, 0xFF797C71, 0xFF72756A, 0xFF5E6156, 0xFF424027,
        0xFF423A0C, 0xFF443200, 0xFF42300C, 0xFF392C0C, 0xFF453A0D, 0xFF473503, 0xFF54370B, 0xFF533F0A, 0xFF4E3B00, 0xFF543F00, 0xFF5C4513, 0xFF5A460B, 0xFF847826, 0xFF736921, 0xFF74683E, 0xFF747043, 0xFF7C703C, 0xFF685312, 0xFF6E4E03, 0xFF724F00,
        0xFFAB8739, 0xFF7C5E16, 0xFF624704, 0xFF614714, 0xFF5C4205, 0xFF6A510E, 0xFF5F4604, 0xFF5A3F08, 0xFF705625, 0xFF57420B, 0xFF624E11, 0xFF684E04, 0xFF5D4402, 0xFF644E14, 0xFF624B15, 0xFF553900, 0xFF7C580A, 0xFFBC9230, 0xFFCEA031, 0xFFE3A22A,
        0xFFDE9E23, 0xFFD99C1D, 0xFFE0A525, 0xFFD8A123, 0xFFBF8A16, 0xFFB38015, 0xFFBB8A24, 0xFFBB941D, 0xFFBA901A, 0xFFD09E2B, 0xFFDBA332, 0xFFD59C2B, 0xFFC18817, 0xFFC9901F, 0xFFD49C2B, 0xFFF0B028, 0xFFB78D21, 0xFF938443, 0xFF8F9071, 0xFF828979,
        0xFF949A8C, 0xFF283323, 0xFF3E4F3F, 0xFF404F4C, 0xFF3F4E4B, 0xFF3D4C49, 0xFF3D4C49, 0xFF3E4D4A, 0xFF3E4D4A, 0xFF3D4C49, 0xFFF4F4F4, 0xFF727C7A, 0xFF44524F, 0xFF808A88, 0xFF3E4E4D, 0xFF3F4D4D, 0xFF525E5E, 0xFFE6E8E8, 0xFFFDFDFD, 0xFF778280,
        0xFF435352, 0xFF435251, 0xFF9FA6A6, 0xFFFFFFFF, 0xFFC3C7C6, 0xFF465453, 0xFF435351, 0xFF445351, 0xFF485552, 0xFF455453, 0xFF3E504D, 0xFF435251, 0xFF455451, 0xFF465552, 0xFF485857, 0xFF4E5E5D, 0xFF4B5B5B, 0xFF455555, 0xFF455555, 0xFF445454,
        0xFF415150, 0xFF4F5E5D, 0xFF505F5E, 0xFF455554, 0xFF455554, 0xFF455554, 0xFF455554, 0xFF415150, 0xFF475655, 0xFF4A5959, 0xFF435251, 0xFF3D4D4D, 0xFF415150, 0xFF41514F, 0xFF445451, 0xFF475754, 0xFF485855, 0xFF485855, 0xFF435350, 0xFF42534D,
        0xFF42534D, 0xFF455550, 0xFF4A5B57, 0xFF445553, 0xFF3B4B4A, 0xFF3B4B4A, 0xFF3E4E4D, 0xFF3F4E49, 0xFF798380, 0xFFF4F5F5, 0xFF42514C, 0xFF42514C, 0xFF43524D, 0xFF42514C, 0xFF3F4E49, 0xFF3F4A44, 0xFF1F2B27, 0xFF808B85, 0xFF8E9687, 0xFF808364,
        0xFF98915B, 0xFF867629, 0xFF6C5700, 0xFF545509, 0xFF5A4E04, 0xFF6F4F02, 0xFFB8892D, 0xFFC89321, 0xFFCF9819, 0xFFCD9A1D, 0xFFCB9823, 0xFFC79B38, 0xFF5A4A0C, 0xFFAAAF8F, 0xFF777B62, 0xFFB2B799, 0xFF5E6B4F, 0xFFA9B398, 0xFF646546, 0xFFAAB29D,
        0xFF6B7658, 0xFFA1B18D, 0xFF6B7C5C, 0xFF9EAD98, 0xFF697665, 0xFF939C87, 0xFF778061, 0xFF92967B, 0xFF8B9280, 0xFF838E80, 0xFF899684, 0xFF79886B, 0xFF92A07F, 0xFF767F64, 0xFF9DA190, 0xFF6F7358, 0xFFA6B69C, 0xFF617B60, 0xFF9EB893, 0xFF5F6B39,
        0xFFBFBC85, 0xFF9C905E, 0xFFC8BB8F, 0xFF9E926A, 0xFFC0B48C, 0xFFA1956D, 0xFFBDB189, 0xFFA2966E, 0xFFBCB088, 0xFFA89C74, 0xFFB9AD85, 0xFFAEA27A, 0xFFAEA27A, 0xFFB7AB83, 0xFFA89C74, 0xFFB7AB83, 0xFFA79B73, 0xFFBBAF87, 0xFFA69A72, 0xFFC7BE95,
        0xFF9F9766, 0xFFD3CCA0, 0xFFA4997B, 0xFFCFBD97, 0xFFA68F4C, 0xFFB49342, 0xFF7B5405, 0xFF574505, 0xFF5A4808, 0xFF5D4B0B, 0xFF5E4A0B, 0xFF5F4B0C, 0xFF614C0D, 0xFF614C0D, 0xFF604B0C, 0xFF684C0B, 0xFF614808, 0xFF634B0D, 0xFF624E11, 0xFF5C4C0E,
        0xFF625511, 0xFF70681F, 0xFF756F21, 0xFF857E38, 0xFF6C7540, 0xFF6F7746, 0xFF7A702B, 0xFF645102, 0xFF999152, 0xFFCBCDA8, 0xFFD3D2C0, 0xFFA6A37A, 0xFF716E45, 0xFF58562D, 0xFF5C5C36, 0xFFA6A787, 0xFF7E8068, 0xFF898A78, 0xFF6F7164, 0xFF444630,
        0xFF49452A, 0xFF4B442A, 0xFF393A2A, 0xFF323E3C, 0xFF384C53, 0xFF384B52, 0xFF3B464A, 0xFF454F6A, 0xFF3B4054, 0xFF333238, 0xFF39322C, 0xFF4E4232, 0xFF5D4E3B, 0xFF594834, 0xFF584735, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0xFF636059, 0xFF96938C, 0xFFC2C2BA, 0xFFAEAEA6, 0xFFADAEA6, 0xFFB9BAB2, 0xFFC3C4BC, 0xFFC0C3B8, 0xFFBDC0B5, 0xFFC5C8BD, 0xFFA8ABA0, 0xFF9B9E93, 0xFFABAEA3, 0xFFBABDB2, 0xFFAAADA2, 0xFFA6AA9C, 0xFFAFB3A5, 0xFFC7CBBD, 0xFFC0C4B6, 0xFFACB0A2,
        0xFFBDC1B3, 0xFFC9CDBF, 0xFFB5B9AB, 0xFFA2A89C, 0xFFA9AFA3, 0xFFB2B8AC, 0xFFABAFA1, 0xFFB3B7A9, 0xFFB8BAAC, 0xFF909284, 0xFF707264, 0xFF888C7E, 0xFF7B7F71, 0xFF818577, 0xFF7B7F71, 0xFF828678, 0xFF7C8072, 0xFF6D7163, 0xFF5A5E50, 0xFF48442B,
        0xFF4B4013, 0xFF473602, 0xFF402E08, 0xFF322505, 0xFF3C3104, 0xFF412F00, 0xFF54370B, 0xFF4B3907, 0xFF524000, 0xFF5B490B, 0xFF574014, 0xFF68551D, 0xFFDED685, 0xFFCFC880, 0xFFA19974, 0xFF68676C, 0xFFA9A290, 0xFFA69663, 0xFFA68C3F, 0xFFAC8D33,
        0xFF81610E, 0xFF624300, 0xFF5F410B, 0xFF644807, 0xFF634805, 0xFF644902, 0xFF5C4400, 0xFF685105, 0xFF513E00, 0xFF655404, 0xFF5B4A00, 0xFF694800, 0xFF6B4F06, 0xFF634A11, 0xFF553F0D, 0xFF5E4711, 0xFF624501, 0xFF795500, 0xFFC09934, 0xFFD99A26,
        0xFFE0A229, 0xFFDDA021, 0xFFDCA01C, 0xFFE0A726, 0xFFD8A12A, 0xFFC49022, 0xFFB5811B, 0xFFAC8A0C, 0xFFBD961D, 0xFFC49626, 0xFFC48E24, 0xFFD89D35, 0xFFCB9126, 0xFFC18719, 0xFFC18A16, 0xFFEF9D0B, 0xFFCE9428, 0xFF9D824A, 0xFF878471, 0xFF828781,
        0xFF939D94, 0xFF253221, 0xFF3A4934, 0xFF3E4D4A, 0xFF3C4B48, 0xFF3A4946, 0xFF394845, 0xFF394845, 0xFF3A4946, 0xFF394845, 0xFFF3F4F4, 0xFF727C7A, 0xFF3D4C49, 0xFF404F4C, 0xFF3E4E4D, 0xFF414E4D, 0xFFC4C8C8, 0xFFFFFFFF, 0xFF9CA4A2, 0xFF465552,
        0xFF40504F, 0xFF778281, 0xFFFFFFFF, 0xFFE2E4E4, 0xFF52605E, 0xFF475653, 0xFF7C8684, 0xFFD6DAD9, 0xFFE3E5E4, 0xFFE5E7E6, 0xFFB4BBBA, 0xFF4A5A57, 0xFF64716E, 0xFFD7DBDA, 0xFFDADDDD, 0xFFD9DCDC, 0xFFDADDDD, 0xFFA7ADAD, 0xFF445350, 0xFF697572,
        0xFFCDD1D0, 0xFFE9EBEB, 0xFFE8EAEA, 0xFFCCD0D0, 0xFF6A7775, 0xFF445352, 0xFF4B5B5A, 0xFFA6AEAD, 0xFFE5E7E7, 0xFFE9EAEA, 0xFFE1E3E3, 0xFF939C9C, 0xFF40504F, 0xFF52605E, 0xFFCFD3D2, 0xFFDADDDC, 0xFFD8DCDB, 0xFFD9DCDB, 0xFFB3B9B8, 0xFF465651,
        0xFFB7BEBB, 0xFFDBDEDD, 0xFFD9DCDB, 0xFFDADDDC, 0xFFC0C5C5, 0xFF5B6867, 0xFF3F4F4E, 0xFF3F4E49, 0xFF798380, 0xFFF4F5F5, 0xFF44534E, 0xFF43524D, 0xFF43524D, 0xFF404F4A, 0xFF3C4B46, 0xFF3E4941, 0xFF27332F, 0xFF7C8886, 0xFF8C968B, 0xFF7D7C5D,
        0xFFA09253, 0xFFA78D2E, 0xFFA78311, 0xFF564E05, 0xFF5B4806, 0xFF724C0B, 0xFFC18E3D, 0xFFD29930, 0xFFD1991E, 0xFFCC991C, 0xFFC99821, 0xFFB69023, 0xFF544A0F, 0xFF657167, 0xFF4A5656, 0xFF69766C, 0xFF4C5E52, 0xFF67746B, 0xFF595A54, 0xFF707C7A,
        0xFF3F4F42, 0xFF6F856E, 0xFF48604A, 0xFF62776E, 0xFF4F6260, 0xFF677671, 0xFF4B584E, 0xFF74716A, 0xFF585858, 0xFF666A6D, 0xFF656E6B, 0xFF5A665A, 0xFF606C5E, 0xFF5F6865, 0xFF6C707B, 0xFF665448, 0xFF7A7770, 0xFF45544F, 0xFF6E817B, 0xFF505D53,
        0xFF7B7E77, 0xFF535456, 0xFF7F818D, 0xFF535D55, 0xFF828C84, 0xFF566058, 0xFF7D877F, 0xFF576159, 0xFF78827A, 0xFF5C665E, 0xFF707A72, 0xFF656F67, 0xFF6B756D, 0xFF69736B, 0xFF677169, 0xFF6D776F, 0xFF656F67, 0xFF757F77, 0xFF626C64, 0xFF837F7E,
        0xFF5D6059, 0xFF767E81, 0xFF5B6378, 0xFF7F838F, 0xFF44432E, 0xFF383002, 0xFF544315, 0xFF564404, 0xFF5C4A0A, 0xFF5F4D0D, 0xFF5D490A, 0xFF5C4809, 0xFF614C0D, 0xFF624D0E, 0xFF5F4A0B, 0xFF644606, 0xFF6C5011, 0xFF60480A, 0xFF614B0F, 0xFF513F00,
        0xFF796C26, 0xFFBBB267, 0xFF918A3A, 0xFFB4A494, 0xFF585954, 0xFF92938B, 0xFFA3936F, 0xFF8F7940, 0xFF8E8453, 0xFF9F9F83, 0xFF979586, 0xFF5A5630, 0xFF6D6943, 0xFF595731, 0xFF696945, 0xFFB1B293, 0xFFB5B7A1, 0xFFB5B6A8, 0xFFACADA5, 0xFFA8AA94,
        0xFFACA88D, 0xFFA0997F, 0xFF717262, 0xFF475351, 0xFF384C53, 0xFF374A51, 0xFF424D51, 0xFF333E44, 0xFF4C534C, 0xFF76795E, 0xFF969360, 0xFFB3AD63, 0xFFC5BE64, 0xFFC8C260, 0xFFD6D16B, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0xFF5C584F, 0xFF838077, 0xFFADADA3, 0xFFB4B6AB, 0xFFB7B9AE, 0xFFACAFA4, 0xFFC6CCC0, 0xFFCDD1C0, 0xFFBBBFB0, 0xFFAAAE9F, 0xFFBBBFB0, 0xFFA3A799, 0xFFBCC0B2, 0xFFB0B3A8, 0xFFB5B8AD, 0xFF9EA294, 0xFFB9BDAF, 0xFFC9CDBF, 0xFFBBBFB1, 0xFFCBCFC1,
        0xFFDBDFD1, 0xFFBFC3B5, 0xFFB2B6A8, 0xFFB3B9AF, 0xFFB3B9AF, 0xFFA5AB9F, 0xFFABAEA3, 0xFFABAEA3, 0xFFA7A99C, 0xFF86887B, 0xFF6B6D60, 0xFF7B7E77, 0xFF7D8079, 0xFF777A73, 0xFF878A83, 0xFF878A83, 0xFF767972, 0xFF6F726B, 0xFF575A53, 0xFF3D402F,
        0xFF544600, 0xFF563B00, 0xFF4A3518, 0xFF2E2305, 0xFF372902, 0xFF433510, 0xFF403E0D, 0xFF463F00, 0xFF4F4102, 0xFF5A4408, 0xFF5A4200, 0xFF776415, 0xFF8F8435, 0xFF727131, 0xFF626538, 0xFF42522B, 0xFF889359, 0xFF5C5E13, 0xFF5D5409, 0xFF665311,
        0xFF624911, 0xFF5F460E, 0xFF594103, 0xFF674805, 0xFF634604, 0xFF634805, 0xFF5B4200, 0xFF5C4404, 0xFF594405, 0xFF5A4609, 0xFF5A480A, 0xFF604200, 0xFF654802, 0xFF5E4100, 0xFF5B4200, 0xFF604906, 0xFF584302, 0xFF604C0D, 0xFFB8A465, 0xFFA5913A,
        0xFFBA8C2C, 0xFFECA12E, 0xFFF19E1C, 0xFFE6A118, 0xFFDCA224, 0xFFD39B2A, 0xFFC1871C, 0xFFBE8200, 0xFFC89520, 0xFFD7AB4A, 0xFF966A11, 0xFF925F03, 0xFFDAA33C, 0xFFD29B27, 0xFFB7830B, 0xFFBA9216, 0xFFCDA239, 0xFFA58A47, 0xFF7F8060, 0xFF818F78,
        0xFF989D86, 0xFF363A2C, 0xFF3B464C, 0xFF3F4E49, 0xFF3D4C47, 0xFF3D4C47, 0xFF3E4D48, 0xFF3C4B46, 0xFF3A4944, 0xFF3B4A45, 0xFFF4F5F4, 0xFF707B79, 0xFF394845, 0xFF3E4D4A, 0xFF3C4C4B, 0xFF949C9B, 0xFFFFFFFF, 0xFFC4C9C8, 0xFF42514E, 0xFF42514E,
        0xFF576564, 0xFFEBECEC, 0xFFF9F9F9, 0xFF697474, 0xFF3F4E4B, 0xFF465451, 0xFFEAECEB, 0xFFCBCFCF, 0xFF4F5D5B, 0xFF6C7877, 0xFFCED2D2, 0xFF5D6A68, 0xFF7E8886, 0xFFFFFFFF, 0xFF858F8E, 0xFF697574, 0xFF687373, 0xFF596565, 0xFF53615E, 0xFFEBECEC,
        0xFFD3D6D5, 0xFF626E6D, 0xFF616E6C, 0xFFCED2D1, 0xFFADB3B2, 0xFF435351, 0xFF9FA8A7, 0xFFFEFEFE, 0xFF848E8E, 0xFF576565, 0xFF969F9F, 0xFFE2E4E4, 0xFF4C5A5A, 0xFF606D6D, 0xFFFFFFFF, 0xFF929B9B, 0xFF657271, 0xFF697575, 0xFF616F6E, 0xFF495956,
        0xFFF8F8F8, 0xFFB9BFBD, 0xFF667472, 0xFF6E7A79, 0xFFF8F8F8, 0xFFB7BCBC, 0xFF404E4F, 0xFF3B4C46, 0xFF77837F, 0xFFF4F5F5, 0xFF43544E, 0xFF41524C, 0xFF3F504A, 0xFF3C4D47, 0xFF394A44, 0xFF3B414D, 0xFF283217, 0xFF848F7E, 0xFF889685, 0xFF74806C,
        0xFF979467, 0xFFAA9321, 0xFFBD8D11, 0xFFAD8128, 0xFF7C5A01, 0xFF6B5100, 0xFFB08F26, 0xFFCD9C25, 0xFFCC9214, 0xFFD0951D, 0xFFC99623, 0xFF966B0C, 0xFF403523, 0xFF2D4047, 0xFF2D3E34, 0xFF343935, 0xFF313A39, 0xFF2F3C33, 0xFF333D3C, 0xFF2E3A36,
        0xFF323E3A, 0xFF34403C, 0xFF333F3B, 0xFF323E3A, 0xFF34403C, 0xFF34403C, 0xFF333F3B, 0xFF2E3D38, 0xFF31403B, 0xFF34433E, 0xFF35443F, 0xFF34433E, 0xFF32413C, 0xFF32413C, 0xFF32413C, 0xFF333F3B, 0xFF323E3A, 0xFF313D39, 0xFF2F3B39, 0xFF2E3A38,
        0xFF2E3A3A, 0xFF2F3B3B, 0xFF303C3C, 0xFF30403F, 0xFF30403F, 0xFF2F3F3E, 0xFF2E3E3B, 0xFF2E3E3B, 0xFF2F403A, 0xFF2F403A, 0xFF2E3F39, 0xFF2E3837, 0xFF36403F, 0xFF333D3C, 0xFF303A39, 0xFF313B3A, 0xFF303A39, 0xFF2E3837, 0xFF2D3736, 0xFF293B2D,
        0xFF2C3637, 0xFF303539, 0xFF2D3834, 0xFF2D3C37, 0xFF2A3330, 0xFF332D15, 0xFF51400A, 0xFF594405, 0xFF5B4607, 0xFF5D4809, 0xFF5E490A, 0xFF5E490A, 0xFF5E490A, 0xFF5E490A, 0xFF5F4A0B, 0xFF634E0F, 0xFF614C0D, 0xFF60480A, 0xFF624D0E, 0xFF5E4C0C,
        0xFF534806, 0xFF615C18, 0xFF434300, 0xFF584D1F, 0xFF2F310A, 0xFF414314, 0xFF5E4B07, 0xFF604600, 0xFF594F12, 0xFF646A3C, 0xFF5C643B, 0xFF51562D, 0xFF4D4A21, 0xFF423A13, 0xFF3A370E, 0xFF394118, 0xFF374521, 0xFF53593D, 0xFF36341F, 0xFF1C1E09,
        0xFF1B1700, 0xFF1A1200, 0xFF1A1B0D, 0xFF2A3636, 0xFF2F434C, 0xFF1E3138, 0xFF19242A, 0xFF29404E, 0xFF2C3E48, 0xFF1D282C, 0xFF161C1A, 0xFF21221A, 0xFF2D2B1F, 0xFF2F2D21, 0xFF262418, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0xFF5A564D, 0xFF827F76, 0xFFABABA1, 0xFFB4B6AB, 0xFFC4C6BB, 0xFFB9BCB1, 0xFFBDC3B7, 0xFFADB1A2, 0xFFA3A798, 0xFFA5A99A, 0xFFBDC1B2, 0xFFA9AD9F, 0xFFADB1A3, 0xFF9DA095, 0xFFA7AA9F, 0xFFA5A89D, 0xFFBABDB2, 0xFFC5C8BD, 0xFFB1B4A9, 0xFFBBBEB3,
        0xFFD5D8CD, 0xFFC3C6BB, 0xFFB0B3A8, 0xFFB1B7AD, 0xFFA8AEA4, 0xFFAAB0A4, 0xFFB4B7AC, 0xFFA7AA9F, 0xFFA7A99C, 0xFF8B8D80, 0xFF686A5D, 0xFF7A7D76, 0xFF7C7F78, 0xFF7C7F78, 0xFF858881, 0xFF80837C, 0xFF767972, 0xFF6F726B, 0xFF575A53, 0xFF4B4239,
        0xFF674E0B, 0xFF623B00, 0xFF593815, 0xFF4D370E, 0xFF4E3705, 0xFF493103, 0xFF463700, 0xFF453600, 0xFF5C4815, 0xFF573B0B, 0xFF61440C, 0xFFAB9250, 0xFFBFAD6D, 0xFF948C5B, 0xFFA19D7A, 0xFF8D867E, 0xFFB5AB92, 0xFF7B6D3C, 0xFF624E0F, 0xFF583D00,
        0xFF705007, 0xFF987823, 0xFFAC8B2C, 0xFFB2861B, 0xFFB3891D, 0xFFB58B1F, 0xFFB0871B, 0xFFAF8718, 0xFFB38D1E, 0xFFAF891A, 0xFFAF891A, 0xFFB88A1B, 0xFFAF8316, 0xFFB18518, 0xFFB48A1E, 0xFFAC8319, 0xFFAB841B, 0xFF9D7810, 0xFFA68119, 0xFF615201,
        0xFF7A5A0F, 0xFFC18B35, 0xFFE4A534, 0xFFE1A523, 0xFFD89E21, 0xFFDEA82E, 0xFFD39D21, 0xFFCF9518, 0xFFCD9622, 0xFFDEAD47, 0xFF9A6E17, 0xFF653F00, 0xFF9D751D, 0xFFC89B34, 0xFFD19F2E, 0xFFB88B0C, 0xFFB7881C, 0xFFA1843E, 0xFF868767, 0xFF869582,
        0xFF9EA68F, 0xFF363A2C, 0xFF434C51, 0xFF3E4D48, 0xFF3B4A45, 0xFF3B4A45, 0xFF3C4B46, 0xFF3B4A45, 0xFF394843, 0xFF3A4944, 0xFFF4F4F4, 0xFF747F7C, 0xFF3B4A47, 0xFF3E4D4A, 0xFF6D7978, 0xFFFCFCFC, 0xFFE6E8E8, 0xFF53605E, 0xFF3F4E4B, 0xFF455451,
        0xFFD1D5D5, 0xFFFFFFFF, 0xFF909999, 0xFF3F4D4C, 0xFF3D4C4A, 0xFF445350, 0xFFBABEBD, 0xFFF2F3F3, 0xFFC6CBCA, 0xFF9BA3A3, 0xFF6C7877, 0xFF41514E, 0xFF7E8886, 0xFFFEFEFE, 0xFFBFC4C4, 0xFFB1B7B6, 0xFFB0B6B6, 0xFF646F6F, 0xFF788381, 0xFFFFFFFF,
        0xFF6F7C7A, 0xFF425050, 0xFF5F6C6A, 0xFF707B79, 0xFF697673, 0xFF445351, 0xFFE6E8E8, 0xFFC0C5C5, 0xFF455554, 0xFF4F5E5D, 0xFF677373, 0xFF717D7D, 0xFF4C5B5B, 0xFF626F6E, 0xFFFBFBFB, 0xFFC8CDCC, 0xFFB3B9B9, 0xFFB3BAB9, 0xFF788483, 0xFF465653,
        0xFFF0F1F1, 0xFFC4C9C8, 0xFF818C8B, 0xFF879091, 0xFFEDEEEE, 0xFF939A9B, 0xFF3D4B4C, 0xFF3C4D47, 0xFF77837F, 0xFFF4F5F4, 0xFF3F504A, 0xFF3D4E48, 0xFF3E4F49, 0xFF3D4E48, 0xFF3B4C46, 0xFF38414A, 0xFF222F13, 0xFF808B7D, 0xFF879385, 0xFF737C69,
        0xFF989163, 0xFFAC911E, 0xFFC29013, 0xFFC2932B, 0xFFB18B26, 0xFF987612, 0xFFB38D20, 0xFFCB9822, 0xFFD09619, 0xFFCE961D, 0xFFC79421, 0xFF916607, 0xFF3D3220, 0xFF2C3F46, 0xFF2F4036, 0xFF363B37, 0xFF333C3B, 0xFF2F3C33, 0xFF323C3B, 0xFF313D39,
        0xFF34403C, 0xFF36423E, 0xFF333F3B, 0xFF323E3A, 0xFF34403C, 0xFF34403C, 0xFF333F3B, 0xFF31403B, 0xFF32413C, 0xFF34433E, 0xFF34433E, 0xFF33423D, 0xFF32413C, 0xFF31403B, 0xFF31403B, 0xFF313D39, 0xFF333F3B, 0xFF34403C, 0xFF35413F, 0xFF35413F,
        0xFF354141, 0xFF344040, 0xFF333F3F, 0xFF30403F, 0xFF30403F, 0xFF30403F, 0xFF30403D, 0xFF2F3F3C, 0xFF2F403A, 0xFF2F403A, 0xFF2F403A, 0xFF333D3C, 0xFF36403F, 0xFF323C3B, 0xFF323C3B, 0xFF36403F, 0xFF353F3E, 0xFF333D3C, 0xFF36403F, 0xFF2C3E30,
        0xFF2F393A, 0xFF33383C, 0xFF2E3935, 0xFF2E3D38, 0xFF2A3330, 0xFF322C14, 0xFF503F09, 0xFF594405, 0xFF5B4607, 0xFF5D4809, 0xFF5E490A, 0xFF5E490A, 0xFF5E490A, 0xFF5E490A, 0xFF5F4A0B, 0xFF5A4506, 0xFF6A5516, 0xFF644C0E, 0xFF5C4708, 0xFF614F0F,
        0xFF5D5210, 0xFF746F2B, 0xFF6A6A24, 0xFF6B6519, 0xFF5E6724, 0xFF636825, 0xFF736210, 0xFF725905, 0xFF867D3C, 0xFFCCD2A6, 0xFFD2DAB5, 0xFFA9AE85, 0xFF747148, 0xFF484019, 0xFF47441B, 0xFFA5AD84, 0xFF82906C, 0xFF81876B, 0xFF63614C, 0xFF444631,
        0xFF403C23, 0xFF3B331C, 0xFF2F3022, 0xFF323E3E, 0xFF374B54, 0xFF32454C, 0xFF364147, 0xFF455054, 0xFF3B4443, 0xFF2E2F27, 0xFF3C3626, 0xFF50462D, 0xFF53482A, 0xFF554828, 0xFF534926, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0xFF545047, 0xFF7F7C73, 0xFFA5A59B, 0xFFB0B2A7, 0xFFC8CABF, 0xFFBABDB2, 0xFFA7ADA1, 0xFFBFC3B4, 0xFFB3B7A8, 0xFFACB0A1, 0xFFA9AD9F, 0xFF9A9E90, 0xFF9EA196, 0xFFA5A89D, 0xFFBCBFB4, 0xFFB3B6AB, 0xFFA0A398, 0xFFA4A79C, 0xFFA5A89D, 0xFFA8ABA0,
        0xFFAEB1A6, 0xFFA4A79C, 0xFFA0A398, 0xFFA9AFA5, 0xFFA1A79D, 0xFFACB2A6, 0xFFB4B7AC, 0xFFA8ABA0, 0xFFABADA0, 0xFF898B7E, 0xFF6B6D60, 0xFF767970, 0xFF7F8279, 0xFF8B8E85, 0xFF898C83, 0xFF7A7D74, 0xFF777A71, 0xFF74776E, 0xFF60635A, 0xFF4A332D,
        0xFF7A520D, 0xFF7A4500, 0xFF764711, 0xFF795315, 0xFF764C04, 0xFF623600, 0xFF5F3D00, 0xFF624501, 0xFF664404, 0xFF684203, 0xFF6E4801, 0xFF8B671B, 0xFF8B6F25, 0xFF705B1A, 0xFF76652F, 0xFF593A1D, 0xFF795A2E, 0xFF694B0B, 0xFF705101, 0xFF785500,
        0xFFAC831A, 0xFFE5B73F, 0xFFFDCC49, 0xFFFFC942, 0xFFFDC740, 0xFFFDC83C, 0xFFFEC739, 0xFFF9C231, 0xFFFFCB35, 0xFFFAC42C, 0xFFFFC830, 0xFFFFC83B, 0xFFFAC134, 0xFFFFC83B, 0xFFFFCE43, 0xFFF4BF35, 0xFFFFCA40, 0xFFF5C139, 0xFFDCA820, 0xFF6E5D03,
        0xFF4F3900, 0xFF72551B, 0xFFB99233, 0xFFDCAA31, 0xFFD59921, 0xFFDBA124, 0xFFD7A717, 0xFFD39F29, 0xFFDA9F2B, 0xFFD89F36, 0xFF966C16, 0xFF543F00, 0xFF594A07, 0xFF927622, 0xFFC39832, 0xFFCA9416, 0xFFB27E11, 0xFFA0823A, 0xFF818564, 0xFF829381,
        0xFFA3AC99, 0xFF323427, 0xFF384042, 0xFF3D4C47, 0xFF394843, 0xFF384742, 0xFF394843, 0xFF394843, 0xFF384742, 0xFF384742, 0xFFF3F4F4, 0xFF747F7C, 0xFF3A4946, 0xFF505D5B, 0xFFE6E8E8, 0xFFFDFEFE, 0xFF6A7472, 0xFF3F4E4B, 0xFF3F4E4A, 0xFFABB2B0,
        0xFFFFFFFF, 0xFFB7BDBC, 0xFF445452, 0xFF3F4F4D, 0xFF3C4B4A, 0xFF404F4C, 0xFF4B5250, 0xFF7C8583, 0xFFACB3B2, 0xFFD9DCDC, 0xFFF6F7F7, 0xFF858F8D, 0xFF75807E, 0xFFFEFEFE, 0xFFAAB1B0, 0xFF979F9F, 0xFF969E9E, 0xFF5A6666, 0xFF7D8886, 0xFFFFFFFF,
        0xFF697676, 0xFF3B4B4B, 0xFFAEB5B4, 0xFFDBDEDD, 0xFFEDEFEF, 0xFF53615F, 0xFFE6E8E7, 0xFFBAC0C0, 0xFF425251, 0xFF788383, 0xFFCDD1D1, 0xFFF5F6F5, 0xFF919B9B, 0xFF556464, 0xFFFBFBFB, 0xFFB7BEBD, 0xFF9CA5A4, 0xFF9AA3A2, 0xFF697674, 0xFF465451,
        0xFFEAECEC, 0xFFE5E8E7, 0xFFC7CCCB, 0xFFDDE0E0, 0xFFF4F4F4, 0xFF636E6E, 0xFF3A4848, 0xFF3A4B45, 0xFF76827E, 0xFFF4F5F4, 0xFF3E4F49, 0xFF3C4D47, 0xFF3D4E48, 0xFF3D4E48, 0xFF3B4C46, 0xFF374548, 0xFF233215, 0xFF878E86, 0xFF939A92, 0xFF7D8170,
        0xFF9C9163, 0xFFAC8F1A, 0xFFC38D11, 0xFFCC9A1D, 0xFFD3A62F, 0xFFBE9523, 0xFFC19323, 0xFFC8941E, 0xFFCF971E, 0xFFCF9821, 0xFFC3921C, 0xFF8C6102, 0xFF3A2F1D, 0xFF2B3E45, 0xFF2F4036, 0xFF373C38, 0xFF343D3C, 0xFF303D34, 0xFF323C3B, 0xFF34403C,
        0xFF37433F, 0xFF37433F, 0xFF34403C, 0xFF323E3A, 0xFF34403C, 0xFF35413D, 0xFF34403C, 0xFF34433E, 0xFF34433E, 0xFF33423D, 0xFF33423D, 0xFF33423D, 0xFF33423D, 0xFF32413C, 0xFF31403B, 0xFF303C38, 0xFF313D39, 0xFF313D39, 0xFF323E3C, 0xFF323E3C,
        0xFF323E3E, 0xFF313D3D, 0xFF303C3C, 0xFF30403F, 0xFF314140, 0xFF324241, 0xFF32423F, 0xFF2F3F3C, 0xFF2E3F39, 0xFF30413B, 0xFF33443E, 0xFF36403F, 0xFF36403F, 0xFF333D3C, 0xFF323C3B, 0xFF353F3E, 0xFF333D3C, 0xFF2D3736, 0xFF2E3837, 0xFF304234,
        0xFF323C3D, 0xFF353A3E, 0xFF303B37, 0xFF2E3D38, 0xFF29322F, 0xFF302A12, 0xFF4E3D07, 0xFF594405, 0xFF5B4607, 0xFF5D4809, 0xFF5E490A, 0xFF5E490A, 0xFF5E490A, 0xFF5E490A, 0xFF5F4A0B, 0xFF655011, 0xFF574203, 0xFF60480A, 0xFF644F10, 0xFF5A4808,
        0xFF776C2A, 0xFFB4AF6B, 0xFF979751, 0xFF8B7E6E, 0xFF45453B, 0xFF7D7B64, 0xFFA28E5C, 0xFF9A7E3D, 0xFF958B50, 0xFF969C6C, 0xFFA2AB7C, 0xFF5A5F36, 0xFF6F6C43, 0xFF585029, 0xFF615E35, 0xFFB3BB92, 0xFFACBA96, 0xFFABB195, 0xFFA8A691, 0xFFA3A590,
        0xFFA8A48B, 0xFFA49C85, 0xFF77786A, 0xFF445050, 0xFF2E424B, 0xFF35484F, 0xFF4A555B, 0xFF384347, 0xFF4C564E, 0xFF6A6E55, 0xFF959466, 0xFFB0AD68, 0xFFB2B059, 0xFFBDBB57, 0xFFC7C95E, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0xFF545047, 0xFF7D7A71, 0xFF9C9C92, 0xFFAAACA1, 0xFFC7C9BE, 0xFFC4C7BC, 0xFFB8BEB2, 0xFFA8AC9D, 0xFFADB1A2, 0xFFB5B9AB, 0xFFABAFA1, 0xFFA9ACA1, 0xFFACAFA4, 0xFFB0B3AA, 0xFFB8BBB2, 0xFFC0C3B8, 0xFF9B9E93, 0xFFA6A99E, 0xFFC1C4B9, 0xFFBDC0B5,
        0xFFABAEA3, 0xFF9DA095, 0xFF9EA196, 0xFFABB1A7, 0xFFACB2A8, 0xFFAEB4A8, 0xFFADB0A5, 0xFFAFB2A7, 0xFFAFB1A4, 0xFF7F8174, 0xFF6E7063, 0xFF7A7B73, 0xFF81827A, 0xFF8E8F87, 0xFF84857D, 0xFF74756D, 0xFF76776F, 0xFF6C6D65, 0xFF53544C, 0xFF654131,
        0xFFA5711E, 0xFFAB6A06, 0xFFA16414, 0xFFA87519, 0xFFBA811A, 0xFFBB7F1F, 0xFFC4902A, 0xFFD6A336, 0xFFCC982A, 0xFFE1AC3C, 0xFFE6B13D, 0xFFD7A42E, 0xFFD6A932, 0xFFD6AE3D, 0xFFCEAC3E, 0xFFD4A734, 0xFFDCB23C, 0xFFD5AF32, 0xFFDEB937, 0xFFDDB32D,
        0xFFF0BF36, 0xFFFEC434, 0xFFFFC02A, 0xFFFABF33, 0xFFF6BC2D, 0xFFF8BB2C, 0xFFFDC12D, 0xFFFABD26, 0xFFFFC227, 0xFFF8BA1B, 0xFFFFC122, 0xFFFFC32C, 0xFFFFC32C, 0xFFFABF27, 0xFFFEC22C, 0xFFF4B822, 0xFFFCC02A, 0xFFFEC22C, 0xFFFABE28, 0xFFCBA72D,
        0xFF7C5F11, 0xFF4F3900, 0xFF7D6306, 0xFFCD9E2C, 0xFFD9992B, 0xFFD59921, 0xFFCFA110, 0xFFD4A231, 0xFFE7AA35, 0xFFDC9D2C, 0xFF986F17, 0xFF52480D, 0xFF4D4F1D, 0xFF928541, 0xFFB7933D, 0xFFD59C28, 0xFFC28D23, 0xFFA78B41, 0xFF7D835F, 0xFF7B8D7D,
        0xFFA3AC9B, 0xFF383A2D, 0xFF3F4543, 0xFF3D4C47, 0xFF394843, 0xFF364540, 0xFF384742, 0xFF394843, 0xFF384742, 0xFF384742, 0xFFF3F4F4, 0xFF707B79, 0xFF364542, 0xFFC2C7C6, 0xFFFFFFFF, 0xFF9CA3A3, 0xFF384745, 0xFF3A4845, 0xFF7D8785, 0xFFFFFFFF,
        0xFFDCDFDF, 0xFF4E5D5B, 0xFF425150, 0xFF435250, 0xFF435250, 0xFF43524F, 0xFFC3C7C6, 0xFF9EA6A5, 0xFF3D4D4D, 0xFF51605F, 0xFFF4F5F5, 0xFFAFB6B5, 0xFF75817F, 0xFFFFFFFF, 0xFF737F7E, 0xFF546362, 0xFF576464, 0xFF536060, 0xFF566361, 0xFFF4F5F5,
        0xFFC5CAC9, 0xFF546262, 0xFF505F5E, 0xFFC3C8C7, 0xFFFFFFFF, 0xFF4E5C59, 0xFFA5ACAB, 0xFFFAFAFA, 0xFF737F7E, 0xFF4B5A59, 0xFF838E8D, 0xFFFFFFFF, 0xFF9AA2A2, 0xFF5A6867, 0xFFFFFFFF, 0xFF889292, 0xFF596766, 0xFF566461, 0xFF53605D, 0xFF495752,
        0xFFF7F7F7, 0xFFA7AEAB, 0xFF43524D, 0xFF535F5D, 0xFFFFFFFF, 0xFF9FA6A6, 0xFF3B4949, 0xFF364643, 0xFF75807E, 0xFFF4F5F4, 0xFF3E4E4B, 0xFF3D4D4A, 0xFF3D4D4A, 0xFF3C4C49, 0xFF394946, 0xFF354545, 0xFF1F2D13, 0xFF828884, 0xFF90958F, 0xFF7D7E6E,
        0xFF9B8F5F, 0xFFAE901E, 0xFFC7941F, 0xFFC99A10, 0xFFC99815, 0xFFCA971A, 0xFFD39C27, 0xFFCC931F, 0xFFCD951E, 0xFFD09C24, 0xFFC2911A, 0xFF8B6001, 0xFF392E1C, 0xFF2A3D44, 0xFF2F4036, 0xFF373C38, 0xFF343D3C, 0xFF2F3C33, 0xFF313B3A, 0xFF35423B,
        0xFF38453E, 0xFF38453E, 0xFF34413A, 0xFF334039, 0xFF36433C, 0xFF38453E, 0xFF37443D, 0xFF39463F, 0xFF37443D, 0xFF35423B, 0xFF35423B, 0xFF37443D, 0xFF38453E, 0xFF37443D, 0xFF36433C, 0xFF3B4743, 0xFF384440, 0xFF34403C, 0xFF313D3B, 0xFF303C3A,
        0xFF313D3D, 0xFF323E3E, 0xFF323E3E, 0xFF313F3F, 0xFF324040, 0xFF334141, 0xFF32413E, 0xFF303F3C, 0xFF2F3E39, 0xFF34433E, 0xFF3B4A45, 0xFF3C4743, 0xFF3E4945, 0xFF3E4945, 0xFF394440, 0xFF3D4844, 0xFF3D4844, 0xFF303B37, 0xFF2B3632, 0xFF304234,
        0xFF323C3D, 0xFF353A3E, 0xFF2F3A36, 0xFF2D3C37, 0xFF29322F, 0xFF302A12, 0xFF4D3C06, 0xFF594405, 0xFF5B4607, 0xFF5D4809, 0xFF5E490A, 0xFF5E490A, 0xFF5E490A, 0xFF5E490A, 0xFF5F4A0B, 0xFF5E490A, 0xFF644F10, 0xFF634B0D, 0xFF5D4809, 0xFF564404,
        0xFF4D4200, 0xFF5F5A16, 0xFF444400, 0xFF4B4217, 0xFF313512, 0xFF45481D, 0xFF574505, 0xFF604800, 0xFF554D0F, 0xFF4F5728, 0xFF586136, 0xFF4E532A, 0xFF47441B, 0xFF403811, 0xFF3D3A11, 0xFF323A11, 0xFF3D4B27, 0xFF61674B, 0xFF413F2A, 0xFF242611,
        0xFF201C03, 0xFF271F08, 0xFF212214, 0xFF1D2929, 0xFF22363F, 0xFF21343B, 0xFF1F2A30, 0xFF32495B, 0xFF2D3E4E, 0xFF1A242D, 0xFF161C1C, 0xFF252620, 0xFF2E2B22, 0xFF2E2C20, 0xFF252317, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0xFF534F46, 0xFF7B786F, 0xFF939389, 0xFF9C9E93, 0xFFAAACA1, 0xFFABAEA3, 0xFFB1B7AB, 0xFFB0B4A6, 0xFFAFB3A5, 0xFFB3B7A9, 0xFFA7AA9F, 0xFFABAEA3, 0xFFADB0A7, 0xFFAFB2A9, 0xFFB1B4AB, 0xFFA2A59C, 0xFF8F9289, 0xFF94978E, 0xFF999C93, 0xFF92958C,
        0xFF999C93, 0xFFA3A69D, 0xFFA5A89F, 0xFF9DA399, 0xFF9CA298, 0xFF9BA195, 0xFF9C9F94, 0xFFA5A89D, 0xFFAAAC9F, 0xFF7C7E71, 0xFF6D6F62, 0xFF75776C, 0xFF76786D, 0xFF7D7F74, 0xFF77796E, 0xFF727469, 0xFF787A6F, 0xFF67695E, 0xFF46483D, 0xFF6D4424,
        0xFFB57B17, 0xFFBF7700, 0xFFAA6601, 0xFFAB6E00, 0xFFCE8B09, 0xFFE0991D, 0xFFE6A724, 0xFFF6AF1F, 0xFFFCB623, 0xFFFFBC23, 0xFFFFBD24, 0xFFFFC42C, 0xFFFFC730, 0xFFFCC32B, 0xFFFFC930, 0xFFFFCA22, 0xFFF9C823, 0xFFF3C625, 0xFFFDD02F, 0xFFFCCC2C,
        0xFFFFCA2A, 0xFFFFC022, 0xFFFFBF24, 0xFFFDBB27, 0xFFFDBB26, 0xFFFEBB26, 0xFFFFBC25, 0xFFFFBE25, 0xFFFFC229, 0xFFFFC126, 0xFFFFBF24, 0xFFFDBF20, 0xFFFFC324, 0xFFF8B81A, 0xFFFDBD1F, 0xFFFCBC1C, 0xFFFEBE1E, 0xFFFBBB1B, 0xFFFFC020, 0xFFFFC62D,
        0xFFD4A531, 0xFF826104, 0xFF6C4A00, 0xFFBC891E, 0xFFE2A13B, 0xFFDA9C2D, 0xFFD19F1C, 0xFFCA9A2B, 0xFFDA9C23, 0xFFE3A129, 0xFFAA7C1C, 0xFF4F4404, 0xFF575E2B, 0xFFB6B179, 0xFFAC964D, 0xFFB78522, 0xFFBB8D2D, 0xFFA68F43, 0xFF888F66, 0xFF839683,
        0xFF98A091, 0xFF313328, 0xFF3A4340, 0xFF3E4D48, 0xFF394843, 0xFF374641, 0xFF394843, 0xFF3A4944, 0xFF394843, 0xFF394843, 0xFFF3F4F4, 0xFF6F7A77, 0xFF364542, 0xFFB3B9B8, 0xFFB8BDBD, 0xFF3F4E4C, 0xFF3C4B49, 0xFF3C4B48, 0xFF7C8684, 0xFFD2D6D5,
        0xFF64716F, 0xFF425250, 0xFF425250, 0xFF425251, 0xFF3F4E4C, 0xFF3B4A47, 0xFF8B9391, 0xFFE9EBEB, 0xFFDEE1E1, 0xFFE0E3E3, 0xFFD3D7D7, 0xFF5B6868, 0xFF667270, 0xFFEBEDED, 0xFFEAECEC, 0xFFE9EBEB, 0xFFEAEBEB, 0xFFCCD0D0, 0xFF40504D, 0xFF747F7E,
        0xFFE1E4E4, 0xFFEAECEC, 0xFFE3E5E5, 0xFFC5C9C9, 0xFFE1E4E3, 0xFF515F5C, 0xFF4C5A57, 0xFFBDC2C1, 0xFFF2F3F2, 0xFFE4E6E6, 0xFFD6D9D9, 0xFFD4D7D7, 0xFF98A0A0, 0xFF4F5E5D, 0xFFE4E6E6, 0xFFECEEED, 0xFFEAEBEB, 0xFFEAEBEB, 0xFFD6DAD9, 0xFF53615C,
        0xFFDDE0DF, 0xFF979F9D, 0xFF42514C, 0xFF42514C, 0xFFDDDFDF, 0xFFA8AEAE, 0xFF3D4B4B, 0xFF384845, 0xFF75807E, 0xFFF4F5F4, 0xFF3B4B48, 0xFF3A4A47, 0xFF3C4C49, 0xFF3C4C49, 0xFF3A4A47, 0xFF354747, 0xFF223017, 0xFF888C8B, 0xFF969B95, 0xFF818471,
        0xFF999363, 0xFFA58E28, 0xFFBA8E2B, 0xFFCA9E17, 0xFFC79711, 0xFFCE9514, 0xFFD69A20, 0xFFD29721, 0xFFCC9520, 0xFFCE9B25, 0xFFC7951C, 0xFF8E6304, 0xFF3B301E, 0xFF2A3D44, 0xFF2D3E34, 0xFF353A36, 0xFF323B3A, 0xFF2E3B32, 0xFF313B3A, 0xFF34413A,
        0xFF37443D, 0xFF38453E, 0xFF35423B, 0xFF35423B, 0xFF38453E, 0xFF3B4841, 0xFF3A4740, 0xFF3A4740, 0xFF38453E, 0xFF37443D, 0xFF37443D, 0xFF39463F, 0xFF3B4841, 0xFF3C4942, 0xFF3B4841, 0xFF3C4844, 0xFF384440, 0xFF323E3A, 0xFF2F3B39, 0xFF2E3A38,
        0xFF303C3C, 0xFF313D3D, 0xFF313D3D, 0xFF313F3F, 0xFF303E3E, 0xFF313F3F, 0xFF31403D, 0xFF2F3E3B, 0xFF2F3E39, 0xFF384742, 0xFF42514C, 0xFF3C4743, 0xFF414C48, 0xFF424D49, 0xFF37423E, 0xFF3D4844, 0xFF46514D, 0xFF38433F, 0xFF303B37, 0xFF2D3F31,
        0xFF303A3B, 0xFF33383C, 0xFF2E3935, 0xFF2C3B36, 0xFF28312E, 0xFF2F2911, 0xFF4D3C06, 0xFF594405, 0xFF5B4607, 0xFF5D4809, 0xFF5E490A, 0xFF5E490A, 0xFF5E490A, 0xFF5E490A, 0xFF5F4A0B, 0xFF554001, 0xFF614C0D, 0xFF5F4709, 0xFF5D4809, 0xFF604E0E,
        0xFF5D5210, 0xFF827D39, 0xFF85853F, 0xFF948B62, 0xFF676D49, 0xFF7C7F54, 0xFF7C6D2C, 0xFF7B631B, 0xFF9B9355, 0xFFBCC495, 0xFFB9C49A, 0xFF9BA077, 0xFF625F36, 0xFF484019, 0xFF4E4B22, 0xFF919970, 0xFF72805C, 0xFF81876B, 0xFF5E5C47, 0xFF41432E,
        0xFF38341B, 0xFF3A321B, 0xFF323325, 0xFF2F3B3B, 0xFF364A53, 0xFF34474E, 0xFF313C42, 0xFF375150, 0xFF344846, 0xFF202D23, 0xFF212A17, 0xFF3F432A, 0xFF58583C, 0xFF575839, 0xFF444525, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0xFF545047, 0xFF827F76, 0xFF9C9C92, 0xFFA1A398, 0xFFA3A59A, 0xFFAAADA2, 0xFFBFC5B9, 0xFFC0C4B6, 0xFFABAFA1, 0xFFA7AA9F, 0xFFA9ACA1, 0xFFADB0A7, 0xFFADB0A7, 0xFFACAFA6, 0xFFB2B5AE, 0xFF9FA299, 0xFFA1A49B, 0xFFA9ACA3, 0xFFA5A89F, 0xFF9A9D94,
        0xFF9C9F96, 0xFFA6A9A0, 0xFFACAFA6, 0xFF979D93, 0xFF8F958B, 0xFF92988C, 0xFF9A9D92, 0xFF999C91, 0xFFA1A396, 0xFF7E8073, 0xFF65675A, 0xFF737369, 0xFF7C7C72, 0xFF818177, 0xFF7E7E74, 0xFF76766C, 0xFF727268, 0xFF68685E, 0xFF54544A, 0xFF6E491C,
        0xFFB77F10, 0xFFC37C00, 0xFFAC6700, 0xFFA96800, 0xFFCF8700, 0xFFE1950E, 0xFFDC9807, 0xFFF2A010, 0xFFFFAE17, 0xFFFEB012, 0xFFF9AF12, 0xFFFFBB24, 0xFFFFC027, 0xFFFDBB1E, 0xFFFFC01A, 0xFFFEC629, 0xFFFCC32B, 0xFFFAC22D, 0xFFFEC72E, 0xFFF8C221,
        0xFFF7BE19, 0xFFF8BC1E, 0xFFFFC32E, 0xFFFFCA31, 0xFFFFCA32, 0xFFFFC22C, 0xFFFBBA28, 0xFFFFC436, 0xFFF8BA31, 0xFFFFC941, 0xFFF9BF39, 0xFFF9C031, 0xFFFFCA39, 0xFFFFCA3A, 0xFFFFCB39, 0xFFFDC12F, 0xFFFFC733, 0xFFFEC02D, 0xFFFEC12B, 0xFFFFBE28,
        0xFFFFC938, 0xFFC89B1A, 0xFF906400, 0xFFC08C29, 0xFFDFA43C, 0xFFD89A29, 0xFFDE9F2B, 0xFFD8A530, 0xFFDB9D22, 0xFFE7A328, 0xFFD29E31, 0xFF896F1B, 0xFF5B5517, 0xFF88834B, 0xFF675D22, 0xFFA5853A, 0xFFA0802D, 0xFF8E7C34, 0xFF858C62, 0xFF8F9E8B,
        0xFF9FA395, 0xFF33342C, 0xFF3E4A48, 0xFF3E4D48, 0xFF3A4944, 0xFF384742, 0xFF3A4944, 0xFF3A4944, 0xFF384742, 0xFF394843, 0xFFF3F4F4, 0xFF707B79, 0xFF3A4946, 0xFF404F4C, 0xFF404F4C, 0xFF404F4C, 0xFF41504D, 0xFF41504D, 0xFF404F4C, 0xFF41504D,
        0xFF445350, 0xFF404F4D, 0xFF41514F, 0xFF425150, 0xFF3B4B49, 0xFF3A4946, 0xFF414F4D, 0xFF4C5B5B, 0xFF677473, 0xFF637171, 0xFF455555, 0xFF3E4E4D, 0xFF415150, 0xFF4B5956, 0xFF4F5D5B, 0xFF4B5A58, 0xFF495856, 0xFF485554, 0xFF41514F, 0xFF415150,
        0xFF4B5A59, 0xFF667271, 0xFF5F6C6C, 0xFF3E4D4D, 0xFF556260, 0xFF445351, 0xFF435250, 0xFF465553, 0xFF596764, 0xFF697573, 0xFF4E5D5C, 0xFF4B5959, 0xFF505E5E, 0xFF465554, 0xFF4A5A59, 0xFF4E5D5B, 0xFF4D5B58, 0xFF505E5A, 0xFF4F5D58, 0xFF43524D,
        0xFF4D5B56, 0xFF485652, 0xFF41504B, 0xFF3E4D48, 0xFF4B5B5A, 0xFF485754, 0xFF3D4C49, 0xFF3C4C4B, 0xFF747F7F, 0xFFF4F4F4, 0xFF374746, 0xFF364645, 0xFF394948, 0xFF3C4C4B, 0xFF3D4D4C, 0xFF314045, 0xFF1C2A13, 0xFF7F8384, 0xFF8B9089, 0xFF747C65,
        0xFF8C8D61, 0xFF948930, 0xFFA4843B, 0xFFBE9A28, 0xFFD2A42C, 0xFFD79D1F, 0xFFCD9013, 0xFFD59A22, 0xFFCD9923, 0xFFCA9721, 0xFFCC9820, 0xFF916607, 0xFF3D3220, 0xFF2A3D44, 0xFF2C3D33, 0xFF333834, 0xFF313A39, 0xFF2E3B32, 0xFF313B3A, 0xFF334037,
        0xFF36433A, 0xFF38453C, 0xFF36433A, 0xFF36433A, 0xFF3A473E, 0xFF3C4940, 0xFF3C4940, 0xFF3B463E, 0xFF3A453D, 0xFF39443C, 0xFF3A453D, 0xFF3C473F, 0xFF3D4840, 0xFF3F4A42, 0xFF3F4A42, 0xFF36423E, 0xFF323E3A, 0xFF2F3B37, 0xFF2E3A38, 0xFF2F3B39,
        0xFF2F3B3B, 0xFF2E3A3A, 0xFF2D3939, 0xFF344040, 0xFF313D3D, 0xFF313D3D, 0xFF333F3D, 0xFF323E3C, 0xFF323E3A, 0xFF3B4743, 0xFF47534F, 0xFF404B45, 0xFF414C46, 0xFF414C46, 0xFF333E38, 0xFF38433D, 0xFF434E48, 0xFF35403A, 0xFF313C36, 0xFF2C3E30,
        0xFF2F393A, 0xFF32373B, 0xFF2D3834, 0xFF2C3B36, 0xFF28312E, 0xFF302A12, 0xFF4E3D07, 0xFF594405, 0xFF5B4607, 0xFF5D4809, 0xFF5E490A, 0xFF5E490A, 0xFF5E490A, 0xFF5E490A, 0xFF5F4A0B, 0xFF695415, 0xFF5D4809, 0xFF5C4406, 0xFF5F4A0B, 0xFF564404,
        0xFF5B500E, 0xFF8F8A46, 0xFF8B8B45, 0xFF968A7E, 0xFF43443C, 0xFF787A65, 0xFF978555, 0xFF927A3A, 0xFF9F995F, 0xFF9DA677, 0xFF9EAA7C, 0xFF585D34, 0xFF625F36, 0xFF4E461F, 0xFF5C5930, 0xFFB5BD94, 0xFF9FAD89, 0xFF9FA589, 0xFFA09E89, 0xFFA1A38E,
        0xFFA4A087, 0xFF9E967F, 0xFF747567, 0xFF445050, 0xFF30444D, 0xFF32454C, 0xFF424D53, 0xFF2C3838, 0xFF4B554A, 0xFF6B7052, 0xFF92925E, 0xFFB8B66C, 0xFFCDCC70, 0xFFDAD871, 0xFFD8DA6C, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0xFF555148, 0xFF8A877E, 0xFFA7A79D, 0xFFA5A79C, 0xFFA3A59A, 0xFFB0B3A8, 0xFFC1C7BB, 0xFFC4C7BC, 0xFFA4A79C, 0xFF979A8F, 0xFF9FA299, 0xFF9DA097, 0xFFA4A79E, 0xFFA1A49D, 0xFFA7AAA3, 0xFFABAEA7, 0xFFA2A59E, 0xFF9A9D96, 0xFFA0A39C, 0xFFA1A49D,
        0xFF969992, 0xFF9EA19A, 0xFFB7BAB3, 0xFFABB1A7, 0xFFAAB0A6, 0xFFA7ADA1, 0xFFABAEA3, 0xFFA1A499, 0xFF9B9D90, 0xFF787A6D, 0xFF5B5D50, 0xFF737369, 0xFF838379, 0xFF818177, 0xFF7C7C72, 0xFF6E6E64, 0xFF5A5A50, 0xFF56564C, 0xFF505046, 0xFF6D4C1F,
        0xFFB17E13, 0xFFBA7402, 0xFFA56005, 0xFFA76600, 0xFFD0860B, 0xFFE59719, 0xFFE9A419, 0xFFF6A522, 0xFFF8AD1E, 0xFFFFB822, 0xFFFFBB28, 0xFFFAB92B, 0xFFFEBC30, 0xFFFFC52E, 0xFFFFC420, 0xFFFBC433, 0xFFFAC134, 0xFFFBC036, 0xFFFEC230, 0xFFFBC026,
        0xFFFDC52A, 0xFFFFC83B, 0xFFFBC143, 0xFFBE941C, 0xFFAF8612, 0xFFAA8417, 0xFFB28F2B, 0xFFC0A147, 0xFF8A6D1D, 0xFF9D8339, 0xFF927A34, 0xFFA08538, 0xFFAB8E42, 0xFFB09345, 0xFFAA8B3B, 0xFF9D7C2B, 0xFFA4812F, 0xFF9A7524, 0xFFA5812D, 0xFFEFBA4A,
        0xFFFAC833, 0xFFEEBF29, 0xFFB8881C, 0xFFBF9036, 0xFFD5A332, 0xFFD69B1B, 0xFFE69B26, 0xFFD7A125, 0xFFD99E1E, 0xFFD99A19, 0xFFE5A92F, 0xFFD0A43B, 0xFF8C7321, 0xFF81793A, 0xFF515321, 0xFFBDB07B, 0xFF9F8C48, 0xFF8A8039, 0xFF7C8455, 0xFF859179,
        0xFFA1A396, 0xFF30332C, 0xFF3A4A49, 0xFF3C4B46, 0xFF394843, 0xFF394843, 0xFF3A4944, 0xFF394843, 0xFF374641, 0xFF384742, 0xFFF8F9F8, 0xFF727C7A, 0xFF3D4C49, 0xFF3F4E4B, 0xFF3E4D4A, 0xFF3F4E4B, 0xFF404F4C, 0xFF404F4C, 0xFF3F4E4B, 0xFF3D4C49,
        0xFF404F4C, 0xFF3F4E4C, 0xFF404F4C, 0xFF42514E, 0xFF3E4D4B, 0xFF3A4946, 0xFF42504D, 0xFF455554, 0xFF425251, 0xFF3D4D4D, 0xFF405050, 0xFF415151, 0xFF425150, 0xFF43524F, 0xFF42504E, 0xFF435351, 0xFF404F4D, 0xFF414E4B, 0xFF3F4F4D, 0xFF40504F,
        0xFF445453, 0xFF42504E, 0xFF425150, 0xFF41514F, 0xFF425250, 0xFF455552, 0xFF475654, 0xFF465655, 0xFF435352, 0xFF425251, 0xFF445453, 0xFF415150, 0xFF40504F, 0xFF455452, 0xFF435250, 0xFF42514F, 0xFF404F4A, 0xFF43524D, 0xFF44534F, 0xFF42514D,
        0xFF41504C, 0xFF43524D, 0xFF41504B, 0xFF3E4D48, 0xFF42524F, 0xFF3E4E4B, 0xFF3B4B48, 0xFF3B4B4A, 0xFF758180, 0xFFF8F9F9, 0xFF394948, 0xFF374746, 0xFF394948, 0xFF3B4B4A, 0xFF3A4A49, 0xFF3D4B54, 0xFF26321E, 0xFF828685, 0xFF878F82, 0xFF718063,
        0xFF909B71, 0xFF9B9B53, 0xFFAA9868, 0xFF8A7117, 0xFFBB912B, 0xFFD8A029, 0xFFCD9011, 0xFFD1991E, 0xFFCC9923, 0xFFCA9620, 0xFFCD951E, 0xFF916607, 0xFF3D3220, 0xFF2B3E45, 0xFF2D3E34, 0xFF343935, 0xFF313A39, 0xFF2E3B32, 0xFF313B3A, 0xFF334037,
        0xFF37443B, 0xFF39463D, 0xFF37443B, 0xFF37443B, 0xFF3A473E, 0xFF3B483F, 0xFF3A473E, 0xFF39443C, 0xFF3A453D, 0xFF3B463E, 0xFF3B463E, 0xFF3B463E, 0xFF3C473F, 0xFF3D4840, 0xFF3E4941, 0xFF3C4844, 0xFF384440, 0xFF333F3B, 0xFF323E3C, 0xFF333F3D,
        0xFF344040, 0xFF333F3F, 0xFF313D3D, 0xFF354141, 0xFF313D3D, 0xFF313D3D, 0xFF34403E, 0xFF34403E, 0xFF323E3A, 0xFF3A4642, 0xFF45514D, 0xFF47524C, 0xFF424D47, 0xFF45504A, 0xFF3B4640, 0xFF3E4943, 0xFF424D47, 0xFF323D37, 0xFF343F39, 0xFF2E4032,
        0xFF303A3B, 0xFF33383C, 0xFF2E3935, 0xFF2D3C37, 0xFF29322F, 0xFF302A12, 0xFF4E3D07, 0xFF594405, 0xFF5B4607, 0xFF5D4809, 0xFF5E490A, 0xFF5E490A, 0xFF5E490A, 0xFF5E490A, 0xFF5F4A0B, 0xFF584304, 0xFF665112, 0xFF624A0C, 0xFF604B0C, 0xFF625010,
        0xFF483D00, 0xFF4B4602, 0xFF3C3C00, 0xFF3B3900, 0xFF384404, 0xFF3F4807, 0xFF514400, 0xFF614C00, 0xFF524C10, 0xFF5C643B, 0xFF687351, 0xFF575C33, 0xFF524F26, 0xFF413912, 0xFF3B380F, 0xFF2F370E, 0xFF3A4824, 0xFF5D6347, 0xFF45432E, 0xFF252712,
        0xFF27230A, 0xFF2A220B, 0xFF242517, 0xFF283434, 0xFF2B3F48, 0xFF24373E, 0xFF273238, 0xFF2F3652, 0xFF2E3349, 0xFF242230, 0xFF292025, 0xFF362827, 0xFF3A2B24, 0xFF402F25, 0xFF3B2D22, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0xFF4D4940, 0xFF9B988F, 0xFFCACAC0, 0xFFC8CABF, 0xFFBFC1B6, 0xFFBFC2B7, 0xFFBBC1B5, 0xFFA0A398, 0xFF8E9186, 0xFF8E9186, 0xFFA3A69D, 0xFFA1A49B, 0xFFB1B4AD, 0xFFA6A9A2, 0xFF9FA29B, 0xFFC9CCC5, 0xFFB4B7B0, 0xFF9A9D96, 0xFF9EA19A, 0xFFA6A9A2,
        0xFF90938C, 0xFF888B84, 0xFF9C9F98, 0xFFA5ABA1, 0xFFB7BDB3, 0xFFACB2A6, 0xFFADB0A5, 0xFFA8ABA0, 0xFF96988B, 0xFF6E7063, 0xFF5C5E51, 0xFF5E5E52, 0xFF69695D, 0xFF5C5C50, 0xFF606054, 0xFF5F5F53, 0xFF4B4B3F, 0xFF47473B, 0xFF414135, 0xFF634720,
        0xFFAC7C1A, 0xFFB6730B, 0xFFA46015, 0xFFA8660F, 0xFFC57B10, 0xFFD58816, 0xFFE19C1A, 0xFFEAA323, 0xFFEEAC20, 0xFFEDB11B, 0xFFEFB724, 0xFFF3BE34, 0xFFF3BE34, 0xFFF4C12A, 0xFFF9C521, 0xFFF5C825, 0xFFF2BF25, 0xFFF5BC27, 0xFFF9BC26, 0xFFF3B81E,
        0xFFF3BB2A, 0xFFE3AD31, 0xFFB8841E, 0xFF5C4F09, 0xFF433800, 0xFF504913, 0xFF7D7A51, 0xFF92957A, 0xFF2D3325, 0xFF37403B, 0xFF323E3E, 0xFF373B3C, 0xFF3F4042, 0xFF3D3F3E, 0xFF3A3937, 0xFF3F3C37, 0xFF413C36, 0xFF2D2620, 0xFF484139, 0xFFD9B46D,
        0xFFEAC338, 0xFFFED136, 0xFFC2912A, 0xFFAC822E, 0xFFCDA52B, 0xFFE0A718, 0xFFEA971D, 0xFFD9A01F, 0xFFDAA11E, 0xFFD49814, 0xFFD0910E, 0xFFCC9119, 0xFFB6902F, 0xFFB8AE69, 0xFF677041, 0xFF8C8C66, 0xFF7B7136, 0xFF8C8743, 0xFF7F8756, 0xFF838B73,
        0xFFA8A99B, 0xFF30332C, 0xFF334545, 0xFF3A4944, 0xFF394843, 0xFF384742, 0xFF394843, 0xFF374641, 0xFF35443F, 0xFF374641, 0xFFD6D9D8, 0xFFD1D4D4, 0xFF7A8482, 0xFF77817F, 0xFF77817F, 0xFF788280, 0xFF798381, 0xFF798381, 0xFF788280, 0xFF76817F,
        0xFF788381, 0xFF7B8683, 0xFF7A8583, 0xFF7A8583, 0xFF798381, 0xFF76807E, 0xFF77807F, 0xFF7C8786, 0xFF7B8686, 0xFF7D8888, 0xFF7D8786, 0xFF7B8686, 0xFF7A8685, 0xFF7B8684, 0xFF7B8584, 0xFF7D8888, 0xFF7E8887, 0xFF7A8381, 0xFF757F7E, 0xFF7A8584,
        0xFF7D8888, 0xFF7D8786, 0xFF7D8888, 0xFF7D8888, 0xFF7E8988, 0xFF7E8988, 0xFF7E8988, 0xFF7E8988, 0xFF7E8988, 0xFF7B8686, 0xFF7A8685, 0xFF788383, 0xFF7A8584, 0xFF7A8685, 0xFF7D8785, 0xFF7D8785, 0xFF7A8682, 0xFF7D8785, 0xFF7D8886, 0xFF7B8684,
        0xFF7B8683, 0xFF7C8783, 0xFF7C8783, 0xFF7B8683, 0xFF7C8786, 0xFF788381, 0xFF76807E, 0xFF778181, 0xFFD1D5D5, 0xFFD7DADA, 0xFF3E4E4D, 0xFF3D4D4C, 0xFF3B4B4A, 0xFF384847, 0xFF344443, 0xFF35414D, 0xFF242F1E, 0xFF888A89, 0xFF8D9586, 0xFF6F8264,
        0xFF82926B, 0xFF7B8445, 0xFF7D7254, 0xFF513D00, 0xFF8D680F, 0xFFD09827, 0xFFD59A1A, 0xFFCF981A, 0xFFCA9721, 0xFFCE9A24, 0xFFC98E18, 0xFF906506, 0xFF3C311F, 0xFF2B3E45, 0xFF2E3F35, 0xFF353A36, 0xFF323B3A, 0xFF2F3C33, 0xFF313B3A, 0xFF344138,
        0xFF38453C, 0xFF3A473E, 0xFF38453C, 0xFF38453C, 0xFF39463D, 0xFF3A473E, 0xFF38453C, 0xFF39443C, 0xFF3A453D, 0xFF3B463E, 0xFF3B463E, 0xFF3A453D, 0xFF39443C, 0xFF3B463E, 0xFF3C473F, 0xFF424E4A, 0xFF3B4743, 0xFF323E3A, 0xFF2D3937, 0xFF2D3937,
        0xFF2F3B3B, 0xFF303C3C, 0xFF303C3C, 0xFF354141, 0xFF323E3E, 0xFF323E3E, 0xFF364240, 0xFF35413F, 0xFF323E3A, 0xFF37433F, 0xFF414D49, 0xFF3D4842, 0xFF35403A, 0xFF3E4943, 0xFF3E4943, 0xFF424D47, 0xFF3F4A44, 0xFF2B3630, 0xFF333E38, 0xFF304234,
        0xFF323C3D, 0xFF353A3E, 0xFF2F3A36, 0xFF2E3D38, 0xFF29322F, 0xFF302A12, 0xFF4E3D07, 0xFF594405, 0xFF5B4607, 0xFF5D4809, 0xFF5E490A, 0xFF5E490A, 0xFF5E490A, 0xFF5E490A, 0xFF5F4A0B, 0xFF5F4A0B, 0xFF594405, 0xFF61490B, 0xFF624D0E, 0xFF554303,
        0xFF615614, 0xFF97924E, 0xFF878741, 0xFF9A9367, 0xFF6B734E, 0xFF878D61, 0xFF7C6E2F, 0xFF634C06, 0xFF837D43, 0xFFB4BD92, 0xFFBECAA4, 0xFFBCC198, 0xFF706D44, 0xFF443C15, 0xFF49461D, 0xFF899168, 0xFF788662, 0xFF868C70, 0xFF585641, 0xFF272914,
        0xFF2A260D, 0xFF2B230C, 0xFF242517, 0xFF2C3838, 0xFF31454E, 0xFF293C43, 0xFF303B41, 0xFF334D4C, 0xFF2F4341, 0xFF1E2B22, 0xFF1F2817, 0xFF363924, 0xFF47472F, 0xFF46462E, 0xFF393921, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0xFF59544E, 0xFFA09B95, 0xFFB8B5AE, 0xFFB6B3AC, 0xFFACACA4, 0xFF94948C, 0xFF9D9D95, 0xFF8F8F87, 0xFF7D7D75, 0xFF7F7C75, 0xFF84817C, 0xFF8D8885, 0xFF9B9693, 0xFF969090, 0xFF8C8384, 0xFFA4A095, 0xFF8F8B80, 0xFF817D72, 0xFF837F74, 0xFF8A867B,
        0xFF928E83, 0xFF8E8A7F, 0xFF817D72, 0xFF959887, 0xFF8D8B7E, 0xFF8C857B, 0xFF948985, 0xFF9B908E, 0xFF7F7979, 0xFF606060, 0xFF6C7270, 0xFF888981, 0xFF85877C, 0xFF7B7B6F, 0xFF817F73, 0xFF837F74, 0xFF77736A, 0xFF7D7572, 0xFF877F7D, 0xFF5A523B,
        0xFF483E25, 0xFF3D2F12, 0xFF372403, 0xFF412B02, 0xFF573C0D, 0xFF5D410F, 0xFF61430D, 0xFF694F05, 0xFF70560C, 0xFF6E5409, 0xFF72570A, 0xFF755A0B, 0xFF785B0B, 0xFF7D6010, 0xFF745805, 0xFF6B6413, 0xFF835E10, 0xFF9B6914, 0xFF9F7A08, 0xFFA57E00,
        0xFFB0750F, 0xFFA06919, 0xFF816A0D, 0xFF9C5A00, 0xFF845C05, 0xFFC1B58B, 0xFFB2BAAD, 0xFF66745B, 0xFF3E4A36, 0xFF30372F, 0xFF394328, 0xFF3A4036, 0xFF3E433C, 0xFF383D37, 0xFF3C3E3B, 0xFF3D3F3A, 0xFF464741, 0xFF2B2D22, 0xFF4B4D3F, 0xFF81854A,
        0xFFD8CBC2, 0xFFFFECBD, 0xFFB29628, 0xFF996F0D, 0xFF9F6A1C, 0xFFC6922F, 0xFFC69225, 0xFFD6A51A, 0xFFD9A327, 0xFFCE952A, 0xFFC48D28, 0xFFBA8A24, 0xFFB99534, 0xFFC3A955, 0xFFC0AC67, 0xFFB28655, 0xFFB2884E, 0xFFB59C64, 0xFF8B9067, 0xFF839474,
        0xFFA6AC92, 0xFF2E3022, 0xFF354042, 0xFF3F4B47, 0xFF3D4945, 0xFF3B4743, 0xFF3A4642, 0xFF394541, 0xFF384440, 0xFF36423E, 0xFF56605D, 0xFFD4D7D6, 0xFFF2F3F3, 0xFFF3F4F3, 0xFFF3F4F4, 0xFFF2F3F3, 0xFFF2F3F3, 0xFFF4F5F5, 0xFFF2F3F3, 0xFFF4F5F5,
        0xFFF3F4F4, 0xFFF2F3F3, 0xFFF3F4F4, 0xFFF5F5F5, 0xFFF2F4F3, 0xFFF5F5F5, 0xFFF2F3F3, 0xFFF4F5F5, 0xFFF5F6F6, 0xFFF4F5F5, 0xFFF1F2F2, 0xFFF1F2F2, 0xFFF5F5F5, 0xFFF4F5F5, 0xFFF1F3F2, 0xFFF1F2F2, 0xFFF4F5F5, 0xFFF5F6F6, 0xFFF2F3F3, 0xFFF1F2F2,
        0xFFF2F3F3, 0xFFF4F5F5, 0xFFF5F6F6, 0xFFF1F2F2, 0xFFF2F3F3, 0xFFF2F3F3, 0xFFF5F6F5, 0xFFF3F4F4, 0xFFF1F2F2, 0xFFF3F4F4, 0xFFF5F6F5, 0xFFF3F4F4, 0xFFF2F3F3, 0xFFF3F4F4, 0xFFF5F6F6, 0xFFF5F5F5, 0xFFF2F3F3, 0xFFF1F2F2, 0xFFF4F5F5, 0xFFF5F6F6,
        0xFFF3F4F4, 0xFFF0F1F1, 0xFFF2F3F3, 0xFFF5F6F5, 0xFFF3F4F4, 0xFFF2F3F3, 0xFFF0F2F1, 0xFFF2F3F3, 0xFFD0D4D4, 0xFF5A6766, 0xFF3B4949, 0xFF3B4949, 0xFF3B4949, 0xFF3B4949, 0xFF3A4848, 0xFF2A453C, 0xFF1C2D1A, 0xFF86858A, 0xFF929383, 0xFF787C63,
        0xFF838476, 0xFF7C7855, 0xFF847162, 0xFF484212, 0xFF503E00, 0xFF8D6A10, 0xFFCC9828, 0xFFD4981C, 0xFFCF9317, 0xFFD09925, 0xFFC39325, 0xFF956510, 0xFF402C11, 0xFF363C38, 0xFF333F3B, 0xFF2D383E, 0xFF2A3E3C, 0xFF2B402F, 0xFF323834, 0xFF333E38,
        0xFF333E38, 0xFF333E38, 0xFF343F39, 0xFF36413B, 0xFF37423C, 0xFF36413B, 0xFF35403A, 0xFF353F3E, 0xFF313B3A, 0xFF313B3A, 0xFF353F3E, 0xFF353F3E, 0xFF323C3B, 0xFF313B3A, 0xFF343E3D, 0xFF2F3938, 0xFF343E3D, 0xFF2E3837, 0xFF313B3A, 0xFF2D3736,
        0xFF333D3C, 0xFF2F3938, 0xFF333D3C, 0xFF323D37, 0xFF343F39, 0xFF323D37, 0xFF323D37, 0xFF35403A, 0xFF37423C, 0xFF35403A, 0xFF323D37, 0xFF343E3F, 0xFF333D3E, 0xFF353F40, 0xFF343E3F, 0xFF313B3C, 0xFF313B3C, 0xFF333D3E, 0xFF333D3E, 0xFF313D3B,
        0xFF343D3A, 0xFF303936, 0xFF2C3A3A, 0xFF2A3D3B, 0xFF283325, 0xFF36300C, 0xFF56410C, 0xFF5A4506, 0xFF5B4607, 0xFF5D4809, 0xFF5E490A, 0xFF5F4A0B, 0xFF5E490A, 0xFF5E490A, 0xFF5D4809, 0xFF5B4A05, 0xFF614F0D, 0xFF5B4C0B, 0xFF59490B, 0xFF504004,
        0xFF66571C, 0xFF96874E, 0xFF92834A, 0xFF86867C, 0xFF4E483C, 0xFF83745D, 0xFF927F55, 0xFF8E7E43, 0xFF8E854A, 0xFF9F9E72, 0xFF979A7F, 0xFF625B3F, 0xFF4E462F, 0xFF3F3A24, 0xFF616041, 0xFFAFB585, 0xFFB6BE8B, 0xFFAEB78C, 0xFFB3BD9B, 0xFFAFA48E,
        0xFFA6A8A5, 0xFF9BA695, 0xFF757962, 0xFF454754, 0xFF31415A, 0xFF364A51, 0xFF464F58, 0xFF3A4853, 0xFF394543, 0xFF5F6950, 0xFFA1A777, 0xFFC4C583, 0xFFCAC478, 0xFFD0C575, 0xFFD5C879, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0xFF67625C, 0xFF5C5751, 0xFF4B4841, 0xFF4A4740, 0xFF55554D, 0xFF42423A, 0xFF303028, 0xFF36362E, 0xFF2B2B23, 0xFF31302B, 0xFF34312C, 0xFF302C29, 0xFF3A3532, 0xFF3C3734, 0xFF383232, 0xFF39352C, 0xFF353128, 0xFF37332A, 0xFF3A362D, 0xFF3B372E,
        0xFF3E3A31, 0xFF3E3A31, 0xFF37332A, 0xFF373828, 0xFF383629, 0xFF473F34, 0xFF4B3E36, 0xFF45362F, 0xFF3A2F29, 0xFF342D25, 0xFF3D3A31, 0xFF655243, 0xFF6B594B, 0xFF625447, 0xFF574F42, 0xFF554F41, 0xFF4F493B, 0xFF4E483A, 0xFF534D3F, 0xFF2E2611,
        0xFF191100, 0xFF1F1500, 0xFF2C1E01, 0xFF2E2000, 0xFF352300, 0xFF3F2C02, 0xFF4E3B10, 0xFF4B3E0A, 0xFF50430F, 0xFF4E410C, 0xFF50410A, 0xFF55460D, 0xFF57480D, 0xFF59490E, 0xFF615115, 0xFF5D4F00, 0xFFAF8813, 0xFFEBB72D, 0xFFF2C335, 0xFFF1BB34,
        0xFFF9B432, 0xFFF6BA36, 0xFFE3C237, 0xFFFFC234, 0xFFE8C13E, 0xFFDED087, 0xFFB2B394, 0xFF797F63, 0xFF474B3D, 0xFF404048, 0xFF3C413A, 0xFF394038, 0xFF3C433C, 0xFF3B413D, 0xFF3C403F, 0xFF383D39, 0xFF3D3F3A, 0xFF2A2D24, 0xFF5D6153, 0xFF7A824F,
        0xFFBAB1AA, 0xFFFBE9C1, 0xFFA9922C, 0xFF997515, 0xFFB68536, 0xFFC3912E, 0xFFD09E2F, 0xFFD5A223, 0xFFD5A02E, 0xFFCD932F, 0xFFCE9732, 0xFFC8952A, 0xFFA97F11, 0xFF9F7C16, 0xFFA9882B, 0xFFA27A32, 0xFFA37C2F, 0xFFA28B48, 0xFF83885F, 0xFF87977D,
        0xFFAAAE9D, 0xFF31322C, 0xFF364049, 0xFF3E4A46, 0xFF3C4844, 0xFF3A4642, 0xFF394541, 0xFF394541, 0xFF394541, 0xFF384440, 0xFF394541, 0xFF3B4A47, 0xFF525F5D, 0xFF4F5C5A, 0xFF4D5B58, 0xFF55625F, 0xFF53615E, 0xFF4B5957, 0xFF54615F, 0xFF4A5855,
        0xFF4E5C59, 0xFF566462, 0xFF53605F, 0xFF505E5E, 0xFF596767, 0xFF4F5D5B, 0xFF54605D, 0xFF485654, 0xFF4D5B5A, 0xFF4F5E5D, 0xFF677372, 0xFF667373, 0xFF4C5B59, 0xFF435250, 0xFF586563, 0xFF65706E, 0xFF4D5C59, 0xFF4A5753, 0xFF515F5D, 0xFF68706D,
        0xFF586461, 0xFF52605F, 0xFF495857, 0xFF677371, 0xFF606D6B, 0xFF576462, 0xFF505E5C, 0xFF505F5D, 0xFF6D7A79, 0xFF5B6969, 0xFF4B5A59, 0xFF546261, 0xFF566464, 0xFF596767, 0xFF4D5A5A, 0xFF4B5A57, 0xFF5A6868, 0xFF6B7877, 0xFF505F5E, 0xFF485756,
        0xFF566462, 0xFF6D7A78, 0xFF586765, 0xFF4C5B59, 0xFF556461, 0xFF5A6865, 0xFF65716F, 0xFF515F5D, 0xFF586562, 0xFF3E4D4B, 0xFF3C4B4A, 0xFF3C4A4A, 0xFF3C4A4A, 0xFF3C4A4A, 0xFF3B4949, 0xFF304741, 0xFF1E2D1A, 0xFF868789, 0xFF949881, 0xFF828A63,
        0xFF9B9B75, 0xFF867D38, 0xFF695017, 0xFF4D4811, 0xFF564B0B, 0xFF614800, 0xFF886201, 0xFFC39325, 0xFFD49D29, 0xFFC89019, 0xFFCD951E, 0xFF966914, 0xFF3F2F15, 0xFF333C39, 0xFF313D39, 0xFF2D383E, 0xFF283C3A, 0xFF2A3F2E, 0xFF333937, 0xFF323D37,
        0xFF313C36, 0xFF313C36, 0xFF333E38, 0xFF36413B, 0xFF37423C, 0xFF36413B, 0xFF343F39, 0xFF37423E, 0xFF38433F, 0xFF38433F, 0xFF37423E, 0xFF35403C, 0xFF343F3B, 0xFF333E3A, 0xFF323D39, 0xFF36413D, 0xFF333E3A, 0xFF333E3A, 0xFF3A4541, 0xFF313C38,
        0xFF37423E, 0xFF38433F, 0xFF37423E, 0xFF3B4640, 0xFF323D37, 0xFF313C36, 0xFF37423C, 0xFF37423C, 0xFF333E38, 0xFF36413B, 0xFF3C4741, 0xFF36403F, 0xFF384241, 0xFF374140, 0xFF353F3E, 0xFF36403F, 0xFF343E3D, 0xFF333D3C, 0xFF353F3E, 0xFF303C3A,
        0xFF343D3A, 0xFF2F3835, 0xFF2C3A3A, 0xFF2A3D3B, 0xFF273224, 0xFF36300C, 0xFF55400B, 0xFF5A4506, 0xFF5B4607, 0xFF5D4809, 0xFF5E490A, 0xFF5E490A, 0xFF5E490A, 0xFF5D4809, 0xFF5D4809, 0xFF614E0C, 0xFF5D4A08, 0xFF594506, 0xFF604E10, 0xFF5D4A0F,
        0xFF59490D, 0xFF5B4B10, 0xFF473700, 0xFF46441D, 0xFF302801, 0xFF534217, 0xFF554007, 0xFF564400, 0xFF4F4300, 0xFF464205, 0xFF3C3B0B, 0xFF352201, 0xFF38270B, 0xFF433319, 0xFF382B09, 0xFF544C1D, 0xFF9D9A67, 0xFF9F9C73, 0xFF3F3A1C, 0xFF41321B,
        0xFF2B2A25, 0xFF2C3019, 0xFF302D0E, 0xFF292428, 0xFF2B3743, 0xFF2C3F3B, 0xFF313A37, 0xFF2A3247, 0xFF212934, 0xFF292F2B, 0xFF373A25, 0xFF3A391B, 0xFF3A3515, 0xFF3F3416, 0xFF382A0F, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00D6D1CD, 0x00CBC8C3, 0x00D0CDC8, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF0C0D07, 0xFF080903, 0xFF010200, 0xFF0B0A05, 0xFF0A0904, 0xFF030000, 0xFF090502, 0xFF0F0B08, 0xFF120D0A, 0xFF0A0700, 0xFF0F0C03, 0xFF141108, 0xFF110E05, 0xFF0B0800,
        0xFF0E0B02, 0xFF120F06, 0xFF100D04, 0xFF161407, 0xFF1B1507, 0xFF352B1F, 0xFF403023, 0xFF301D0F, 0xFF281506, 0xFF281705, 0xFF2A1907, 0xFF401A05, 0xFF41200D, 0xFF2D1709, 0xFF140800, 0xFF110B00, 0xFF131000, 0xFF0F0A00, 0xFF120C00, 0xFF1E1905,
        0xFF161100, 0xFF241F09, 0xFF28230D, 0xFF211D04, 0xFF2A260B, 0xFF39351A, 0xFF423E23, 0xFF43452F, 0xFF484A34, 0xFF4F4F37, 0xFF4D4D33, 0xFF525236, 0xFF4E4D2F, 0xFF414022, 0xFF5E5D3E, 0xFFD5BD77, 0xFFF3CB4F, 0xFFFBC61E, 0xFFFBC01E, 0xFFFFB82C,
        0xFFFFB722, 0xFFFFC721, 0xFFF9C825, 0xFFFBBD1E, 0xFFF1CB3A, 0xFFE0CE78, 0xFFA39F79, 0xFF818260, 0xFF4D4B3C, 0xFF39383E, 0xFF373E36, 0xFF364139, 0xFF353E39, 0xFF384140, 0xFF384140, 0xFF393F3F, 0xFF393F3B, 0xFF232821, 0xFF5E645A, 0xFF697452,
        0xFFA2A19C, 0xFFF4EAC7, 0xFF8B7B24, 0xFF83650B, 0xFFB99040, 0xFFA57918, 0xFFAD7F0F, 0xFFD49925, 0xFFDDA337, 0xFFD19630, 0xFFC38A21, 0xFFBF8A1A, 0xFFC2911B, 0xFFC99A26, 0xFFCA9C2C, 0xFFBC9933, 0xFFB6932F, 0xFFA08A3E, 0xFF7F845B, 0xFF899786,
        0xFFA7AAA1, 0xFF313131, 0xFF37414D, 0xFF3D4945, 0xFF3B4743, 0xFF384440, 0xFF37433F, 0xFF384440, 0xFF394541, 0xFF394541, 0xFFBBBFBD, 0xFF737E7C, 0xFFC0C5C4, 0xFF969F9D, 0xFF8A9391, 0xFFB4BAB9, 0xFFC2C7C6, 0xFF5C6966, 0xFFDFE2E1, 0xFF717C7A,
        0xFFA4ABAA, 0xFFB0B7B5, 0xFFB0B6B5, 0xFF6F7B78, 0xFFE2E5E4, 0xFF606C6A, 0xFFC8CCCB, 0xFF4C5A57, 0xFF404F4C, 0xFF99A1A0, 0xFFCED2D1, 0xFFC2C6C6, 0xFF6E7A77, 0xFF6D7976, 0xFFD0D3D3, 0xFFC4C9C9, 0xFFA7AFAE, 0xFF576361, 0xFFD3D6D6, 0xFFCBCECD,
        0xFFDADDDD, 0xFF828D8D, 0xFF7D8887, 0xFFDDE0E0, 0xFFC7CBCB, 0xFFD9DCDB, 0xFF556360, 0xFFB5BBBA, 0xFFC3C8C7, 0xFFCFD3D3, 0xFF687574, 0xFFAEB5B5, 0xFFD7DADA, 0xFFA6ADAD, 0xFF404F4D, 0xFF65716D, 0xFFD6D9D9, 0xFFD0D4D4, 0xFFAEB4B4, 0xFF566463,
        0xFFCACECE, 0xFFCACFCE, 0xFFD3D7D6, 0xFF616F6C, 0xFFA9B0AF, 0xFFCED2D1, 0xFFDBDEDD, 0xFFCCD0CF, 0xFFD3D7D5, 0xFFCACFCD, 0xFF3E4D4B, 0xFF3E4C4C, 0xFF3D4B4B, 0xFF3C4A4A, 0xFF3C4A4A, 0xFF384847, 0xFF202D1C, 0xFF848A8A, 0xFF8C957A, 0xFF788152,
        0xFFA2A266, 0xFFAD9B2F, 0xFFA27C11, 0xFF5F5510, 0xFF564C07, 0xFF4F4100, 0xFF5F4900, 0xFF8C6E16, 0xFFBD922C, 0xFFCE9925, 0xFFCA8E12, 0xFF8D6817, 0xFF3C3018, 0xFF2E3935, 0xFF2F3C35, 0xFF2F383F, 0xFF2A3D3B, 0xFF2E4030, 0xFF373D3B, 0xFF3C4741,
        0xFF3A453F, 0xFF38433D, 0xFF3A453F, 0xFF3D4842, 0xFF3D4842, 0xFF3B4640, 0xFF38433D, 0xFF3B463E, 0xFF424D45, 0xFF444F47, 0xFF3F4A42, 0xFF3C473F, 0xFF3D4840, 0xFF3C473F, 0xFF39443C, 0xFF414C44, 0xFF37423A, 0xFF3A453D, 0xFF48534B, 0xFF3A453D,
        0xFF3F4A42, 0xFF424D45, 0xFF3F4A42, 0xFF3C4741, 0xFF313C36, 0xFF35403A, 0xFF404B45, 0xFF3F4A44, 0xFF3F4A44, 0xFF46514B, 0xFF48534D, 0xFF3E4943, 0xFF434E48, 0xFF3F4A44, 0xFF3D4842, 0xFF424D47, 0xFF3E4943, 0xFF37423C, 0xFF3A453F, 0xFF2F3B39,
        0xFF333C39, 0xFF2F3835, 0xFF2B3939, 0xFF293C3A, 0xFF263123, 0xFF352F0B, 0xFF543F0A, 0xFF5A4506, 0xFF5B4607, 0xFF5C4708, 0xFF5E490A, 0xFF5E490A, 0xFF5E490A, 0xFF5D4809, 0xFF5D4809, 0xFF604808, 0xFF614909, 0xFF5E490A, 0xFF604B0C, 0xFF604A0E,
        0xFF5D490E, 0xFF5D490E, 0xFF59450A, 0xFF554D0C, 0xFF56470E, 0xFF5F4A15, 0xFF5B4307, 0xFF5C4500, 0xFF5E4C02, 0xFF544605, 0xFF4B420B, 0xFF583D10, 0xFF402400, 0xFF432906, 0xFF432C03, 0xFF342000, 0xFF5A4816, 0xFF604E26, 0xFF291500, 0xFF261500,
        0xFF110900, 0xFF1B1600, 0xFF342600, 0xFF261607, 0xFF22241F, 0xFF1F2D16, 0xFF2D321C, 0xFF161C2C, 0xFF09101A, 0xFF0D1215, 0xFF0F1410, 0xFF131512, 0xFF1F1F1F, 0xFF27242B, 0xFF1D1725, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFB, 0x00FFFBF8, 0x00FFFFFB, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF0E0F0A, 0xFF0D0F0A, 0xFF000200, 0xFF070803, 0xFF090A05, 0xFF060500, 0xFF0E0D08, 0xFF14110C, 0xFF14110C, 0xFF16130A, 0xFF19160D, 0xFF18150C, 0xFF120F06, 0xFF0F0C03,
        0xFF120F06, 0xFF141108, 0xFF100D04, 0xFF130F03, 0xFF0E0800, 0xFF291D0F, 0xFF453525, 0xFF45301D, 0xFF3E2811, 0xFF40260D, 0xFF3F250A, 0xFF503017, 0xFF412611, 0xFF241407, 0xFF0D0600, 0xFF12110C, 0xFF1D1D13, 0xFF1E1B0A, 0xFF26210B, 0xFF3B3825,
        0xFF2C2B19, 0xFF2F3020, 0xFF242716, 0xFF111506, 0xFF1C2214, 0xFF2D3528, 0xFF30382B, 0xFF323836, 0xFF363C38, 0xFF3F4541, 0xFF3E433D, 0xFF434841, 0xFF3D4037, 0xFF272A1F, 0xFF5E6156, 0xFFC8B08E, 0xFFF3CE88, 0xFFFAC841, 0xFFF9BC16, 0xFFFFB522,
        0xFFFFBC32, 0xFFFFC639, 0xFFEAB130, 0xFFE9B23E, 0xFFD6B448, 0xFFE1D196, 0xFF9D977F, 0xFF7E7F5F, 0xFF4E523B, 0xFF2A3429, 0xFF384C30, 0xFF333E38, 0xFF2F3A36, 0xFF36403F, 0xFF323A3C, 0xFF353D3F, 0xFF393F3D, 0xFF232A23, 0xFF5C6459, 0xFF707C66,
        0xFF9FA299, 0xFFCDCAAB, 0xFF5B510A, 0xFF543F00, 0xFF937223, 0xFF9F781D, 0xFFC89F33, 0xFFE09C21, 0xFFE5A32B, 0xFFDFA02E, 0xFFD29624, 0xFFC18918, 0xFFBE8B18, 0xFFC18F20, 0xFFB28214, 0xFFAF8E1A, 0xFFAE8C20, 0xFF978135, 0xFF7F835E, 0xFF8D9B8C,
        0xFFA6A9A0, 0xFF2E2F2A, 0xFF333E42, 0xFF3D4945, 0xFF3B4743, 0xFF384440, 0xFF36423E, 0xFF37433F, 0xFF384440, 0xFF394541, 0xFFABB0AF, 0xFFBFC4C3, 0xFFD2D5D4, 0xFFC6CBCA, 0xFFCACECD, 0xFF848E8C, 0xFFBFC3C2, 0xFFB6BCBB, 0xFFE3E5E5, 0xFFACB2B1,
        0xFFD6D9D9, 0xFF747F7D, 0xFFD8DBDA, 0xFFAFB5B4, 0xFFE8EAE9, 0xFFB6BCBB, 0xFFC9CECD, 0xFF3A4845, 0xFF3F4C49, 0xFFA8AEAE, 0xFFD6DAD9, 0xFFB3B9B8, 0xFF717D7B, 0xFFC1C6C5, 0xFFD4D8D7, 0xFFA6AEAD, 0xFFE1E3E3, 0xFFAFB5B4, 0xFFCFD3D3, 0xFF42504E,
        0xFFC8CDCD, 0xFFA6ADAD, 0xFFD0D4D4, 0xFF818B8B, 0xFF52605E, 0xFFECEDED, 0xFF86908E, 0xFFF4F5F4, 0xFFA1A9A8, 0xFFD8DCDC, 0xFFA5ACAC, 0xFFC9CECE, 0xFFAAB1B1, 0xFF475554, 0xFF3F4E4D, 0xFFBBC0BF, 0xFFBCC2C0, 0xFF475755, 0xFF6D7978, 0xFFA2AAAA,
        0xFFCFD3D3, 0xFF404F4E, 0xFFBCC1C1, 0xFFB6BDBB, 0xFFC0C6C5, 0xFF969F9D, 0xFF909998, 0xFFCDD1D0, 0xFF5F6E69, 0xFFF0F2F1, 0xFF42524D, 0xFF3F4E4C, 0xFF3C4A4A, 0xFF3B4949, 0xFF3A4848, 0xFF374343, 0xFF202B1B, 0xFF858D8F, 0xFF8C9882, 0xFF6D784D,
        0xFF8F894D, 0xFFB1941C, 0xFFCF9D1A, 0xFF9F8224, 0xFF6C5601, 0xFF534600, 0xFF4F4507, 0xFF524501, 0xFF856B17, 0xFFBF952F, 0xFFC99623, 0xFF826517, 0xFF3A341E, 0xFF2B3A37, 0xFF2F3C35, 0xFF32393F, 0xFF2B3B38, 0xFF2C3E30, 0xFF363B3E, 0xFF3E4943,
        0xFF3C4741, 0xFF39443E, 0xFF3A453F, 0xFF3C4741, 0xFF3C4741, 0xFF38433D, 0xFF343F39, 0xFF364238, 0xFF3C483E, 0xFF404C42, 0xFF3E4A40, 0xFF3C483E, 0xFF3C483E, 0xFF3C483E, 0xFF3A463C, 0xFF3E4A40, 0xFF354137, 0xFF364238, 0xFF445046, 0xFF39453B,
        0xFF3B473D, 0xFF3A463C, 0xFF3A463C, 0xFF3A453F, 0xFF333E38, 0xFF39443E, 0xFF3F4A44, 0xFF3C4741, 0xFF424D47, 0xFF434E48, 0xFF36413B, 0xFF3C473F, 0xFF434E46, 0xFF3F4A42, 0xFF3D4840, 0xFF444F47, 0xFF3C473F, 0xFF2F3A32, 0xFF323D35, 0xFF2E3A38,
        0xFF323B38, 0xFF2E3734, 0xFF2B3939, 0xFF293C3A, 0xFF263123, 0xFF342E0A, 0xFF533E09, 0xFF5A4506, 0xFF5B4607, 0xFF5C4708, 0xFF5D4809, 0xFF5E490A, 0xFF5E490A, 0xFF5D4809, 0xFF5C4708, 0xFF5A4003, 0xFF62480B, 0xFF63490C, 0xFF5A4204, 0xFF5C4406,
        0xFF5E480C, 0xFF5C460A, 0xFF604A0E, 0xFF5B4D04, 0xFF5C480B, 0xFF5C4211, 0xFF604210, 0xFF583B01, 0xFF5D4505, 0xFF5B460D, 0xFF503E0C, 0xFF573E05, 0xFF533906, 0xFF4C3406, 0xFF463000, 0xFF453000, 0xFF78652D, 0xFF6C572C, 0xFF331E00, 0xFF3B2404,
        0xFF2D1E0B, 0xFF463507, 0xFF6A4E0D, 0xFF391D00, 0xFF252003, 0xFF2A3308, 0xFF4E5028, 0xFF1E2221, 0xFF000200, 0xFF010801, 0xFF0D140C, 0xFF161F1A, 0xFF1C2426, 0xFF20272F, 0xFF121A25, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFD, 0x00FFFEFC, 0x00FFFFFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF171916, 0xFF101511, 0xFF000300, 0xFF030500, 0xFF060803, 0xFF060701, 0xFF0E0F07, 0xFF111109, 0xFF101008, 0xFF131009, 0xFF16130C, 0xFF16130C, 0xFF14110A, 0xFF16130C,
        0xFF18150E, 0xFF15120B, 0xFF100D06, 0xFF1A130B, 0xFF120C00, 0xFF190F03, 0xFF312614, 0xFF44331F, 0xFF49351C, 0xFF4E3419, 0xFF513619, 0xFF362C13, 0xFF221B09, 0xFF0E0B04, 0xFF08090B, 0xFF121619, 0xFF1C1E1D, 0xFF24231E, 0xFF312F23, 0xFF717262,
        0xFF4E4F3F, 0xFF434538, 0xFF343A2E, 0xFF1A241B, 0xFF18231D, 0xFF24302C, 0xFF2B3A37, 0xFF35393C, 0xFF373B3C, 0xFF3D4142, 0xFF3C3E3D, 0xFF434542, 0xFF40423D, 0xFF2F302A, 0xFF7F807A, 0xFF8D8171, 0xFFE9D0CB, 0xFFFED885, 0xFFF4C116, 0xFFFFC524,
        0xFFF9BD45, 0xFFC28E2B, 0xFF7C4B00, 0xFF6E4600, 0xFF654D07, 0xFF9E9373, 0xFF908D84, 0xFF80826A, 0xFF575B4A, 0xFF2B3A35, 0xFF28442E, 0xFF333E36, 0xFF323D37, 0xFF3A4342, 0xFF2F3739, 0xFF333939, 0xFF3A403C, 0xFF293028, 0xFF555B51, 0xFF6E735F,
        0xFF8B8F78, 0xFF959475, 0xFF514C14, 0xFF4F4100, 0xFF604900, 0xFF826313, 0xFFC3A241, 0xFFE49E1C, 0xFFDB9712, 0xFFD89513, 0xFFE0A227, 0xFFD19827, 0xFFAB7711, 0xFF9E7113, 0xFFA57C20, 0xFF997709, 0xFF9E7B15, 0xFF8E782F, 0xFF7A7E5D, 0xFF8B998C,
        0xFFA6AA9C, 0xFF313325, 0xFF39463D, 0xFF3E4A46, 0xFF3B4743, 0xFF37433F, 0xFF35413D, 0xFF35413D, 0xFF36423E, 0xFF37433F, 0xFF67706D, 0xFFFFFFFF, 0xFFA8AFAD, 0xFFC7CBCA, 0xFFEFF0F0, 0xFF42504C, 0xFF939B99, 0xFFFFFFFF, 0xFF8F9795, 0xFFECEDED,
        0xFFCCD0CF, 0xFF3D4C49, 0xFFC2C7C6, 0xFFF3F4F4, 0xFF929B99, 0xFFFFFFFF, 0xFF99A1A0, 0xFF6E7977, 0xFF5F6D6B, 0xFF808A8A, 0xFF9CA5A4, 0xFFCDD0D0, 0xFFCDD1D0, 0xFFBAC0BF, 0xFFD4D8D7, 0xFF8E9796, 0xFFA1A9A8, 0xFF8C9594, 0xFFE0E3E2, 0xFF737D7C,
        0xFFDADDDD, 0xFF9DA6A5, 0xFFBFC4C4, 0xFFAEB5B5, 0xFF818B8A, 0xFFEDEFEE, 0xFF7C8684, 0xFFF2F3F3, 0xFF949D9B, 0xFFA8AFAE, 0xFF717D7D, 0xFFCFD3D3, 0xFF8A9493, 0xFF64716F, 0xFF6D7877, 0xFFABB1B0, 0xFFCED2D1, 0xFF6B7776, 0xFF97A0A0, 0xFF969F9F,
        0xFFDDE0E0, 0xFF596766, 0xFFCDD1D1, 0xFFA8AFAF, 0xFFC2C7C6, 0xFF8A9493, 0xFF8D9797, 0xFFC6CBCB, 0xFF556361, 0xFFF2F4F3, 0xFF445451, 0xFF40504F, 0xFF3B4949, 0xFF394747, 0xFF384646, 0xFF3A4445, 0xFF232E1D, 0xFF838B8E, 0xFF8F9B8D, 0xFF778065,
        0xFF8D8255, 0xFFA58317, 0xFFD59A1C, 0xFFD39E2A, 0xFFA17711, 0xFF6C5301, 0xFF504706, 0xFF4C460A, 0xFF574D08, 0xFF7B6510, 0xFFA08222, 0xFF715D14, 0xFF373826, 0xFF2C3F3D, 0xFF303D36, 0xFF353A3E, 0xFF2A3936, 0xFF283A2C, 0xFF303539, 0xFF3C4741,
        0xFF3B4640, 0xFF39443E, 0xFF3A453F, 0xFF3C4741, 0xFF3C4741, 0xFF3A453F, 0xFF37423C, 0xFF39453B, 0xFF3A463C, 0xFF3E4A40, 0xFF414D43, 0xFF3E4A40, 0xFF39453B, 0xFF39453B, 0xFF3D493F, 0xFF3E4A40, 0xFF3A463C, 0xFF354137, 0xFF414D43, 0xFF3C483E,
        0xFF3B473D, 0xFF354137, 0xFF3B473D, 0xFF3C4741, 0xFF323D37, 0xFF36413B, 0xFF3C4741, 0xFF3D4842, 0xFF444F49, 0xFF434E48, 0xFF323D37, 0xFF3B463E, 0xFF434E46, 0xFF404B43, 0xFF3E4941, 0xFF444F47, 0xFF3B463E, 0xFF2C372F, 0xFF2D3830, 0xFF2D3937,
        0xFF313A37, 0xFF2E3734, 0xFF2B3939, 0xFF293C3A, 0xFF263123, 0xFF342E0A, 0xFF533E09, 0xFF594405, 0xFF5A4506, 0xFF5C4708, 0xFF5D4809, 0xFF5E490A, 0xFF5D4809, 0xFF5D4809, 0xFF5C4708, 0xFF644A0D, 0xFF5F4508, 0xFF5C4205, 0xFF5B4305, 0xFF624A0C,
        0xFF634E0D, 0xFF5D4807, 0xFF5D4807, 0xFF5C4B00, 0xFF5D4904, 0xFF5D440C, 0xFF684D16, 0xFF644A0D, 0xFF5F4805, 0xFF5B4607, 0xFF58430C, 0xFF554601, 0xFF524407, 0xFF52430A, 0xFF504106, 0xFF4F4200, 0xFF807231, 0xFF71642F, 0xFF44350C, 0xFF483108,
        0xFF3C2708, 0xFF5E4608, 0xFF946E1B, 0xFF522B00, 0xFF362900, 0xFF383E02, 0xFF646730, 0xFF333532, 0xFF000200, 0xFF030801, 0xFF0D170E, 0xFF172319, 0xFF17241D, 0xFF192823, 0xFF0C1C19, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FDFBFC, 0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF1E201F, 0xFF131917, 0xFF020605, 0xFF040905, 0xFF050A04, 0xFF040700, 0xFF090C03, 0xFF0D0F04, 0xFF0E1005, 0xFF0C0C04, 0xFF0F0F07, 0xFF0E0E06, 0xFF0F0F07, 0xFF111109,
        0xFF101008, 0xFF0D0D05, 0xFF0E0E06, 0xFF17100A, 0xFF1B140C, 0xFF0F0B00, 0xFF110E00, 0xFF201906, 0xFF291D07, 0xFF2C1A04, 0xFF2B1700, 0xFF141801, 0xFF0F1000, 0xFF0A0A02, 0xFF101010, 0xFF0F0E14, 0xFF0E0F13, 0xFF1B1F20, 0xFF212622, 0xFF575848,
        0xFF424436, 0xFF3B3F31, 0xFF2A3026, 0xFF192019, 0xFF202926, 0xFF293332, 0xFF2A3636, 0xFF363A39, 0xFF383C3B, 0xFF3F413E, 0xFF3E403B, 0xFF43443E, 0xFF404139, 0xFF303028, 0xFF8B8B81, 0xFF6F735A, 0xFFBDB4C5, 0xFFD7BF81, 0xFFEAC117, 0xFFFDCB2E,
        0xFFBF982F, 0xFF715703, 0xFF5A3B02, 0xFF5D4507, 0xFF483D00, 0xFF504C29, 0xFF484536, 0xFF3A361B, 0xFF2C2A1E, 0xFF353B47, 0xFF384B4F, 0xFF383F37, 0xFF343B34, 0xFF363C38, 0xFF313534, 0xFF373C38, 0xFF3D3F3A, 0xFF25281F, 0xFF2E3224, 0xFF2D2910,
        0xFF464519, 0xFF4E4A24, 0xFF474316, 0xFF5C550F, 0xFF524300, 0xFF644C0C, 0xFF806715, 0xFFCC941D, 0xFFE4A929, 0xFFE1A31C, 0xFFDB9C19, 0xFFD89C2A, 0xFFBC8727, 0xFFA47A24, 0xFFA07D2B, 0xFFA17E1A, 0xFFA48022, 0xFF98813E, 0xFF7E8162, 0xFF879586,
        0xFFA5A998, 0xFF313421, 0xFF3A483B, 0xFF3C4844, 0xFF394541, 0xFF36423E, 0xFF34403C, 0xFF34403C, 0xFF35413D, 0xFF35413D, 0xFF3A4541, 0xFFB9BEBD, 0xFF64706D, 0xFF858E8D, 0xFFA2A9A7, 0xFF3C4B46, 0xFF586561, 0xFFBFC4C3, 0xFF4E5B57, 0xFFABB2B0,
        0xFF7F8987, 0xFF3A4945, 0xFF7B8683, 0xFFB4BAB8, 0xFF4F5D58, 0xFFBFC3C2, 0xFF5C6966, 0xFFA0A7A6, 0xFF7F8A89, 0xFF808A8A, 0xFFC4C9C9, 0xFFBCC1C1, 0xFF7F8A88, 0xFF5A6865, 0xFFBEC3C2, 0xFFC4C8C7, 0xFF8C9593, 0xFF485654, 0xFFACB2B1, 0xFFB8BDBC,
        0xFFE5E7E7, 0xFF99A1A1, 0xFF637070, 0xFFC1C6C6, 0xFFB7BDBD, 0xFFF3F4F4, 0xFF556360, 0xFF949C9A, 0xFFC7CBCA, 0xFFB7BDBC, 0xFF576463, 0xFF9DA5A4, 0xFF6D7976, 0xFF899290, 0xFF98A09F, 0xFF53605E, 0xFFBFC5C4, 0xFFD0D4D4, 0xFF949D9D, 0xFF475656,
        0xFFABB2B2, 0xFFCDD1D1, 0xFFB6BCBC, 0xFF556463, 0xFF99A2A1, 0xFF74807F, 0xFF748080, 0xFF99A2A1, 0xFF4E5D5A, 0xFFB5BBBA, 0xFF465653, 0xFF40514F, 0xFF3A4848, 0xFF384646, 0xFF374545, 0xFF394344, 0xFF283120, 0xFF83888C, 0xFF8D978E, 0xFF7E8575,
        0xFF94896B, 0xFFA68423, 0xFFCF9720, 0xFFD69515, 0xFFCC9521, 0xFFA27C1B, 0xFF6E590A, 0xFF534904, 0xFF504706, 0xFF544701, 0xFF5C4B03, 0xFF554802, 0xFF2E3525, 0xFF2B3F3E, 0xFF303D36, 0xFF363B3E, 0xFF2D3937, 0xFF2D3D32, 0xFF33383E, 0xFF38433D,
        0xFF38433D, 0xFF39443E, 0xFF3B4640, 0xFF3D4842, 0xFF3E4943, 0xFF3E4943, 0xFF3D4842, 0xFF3F4A42, 0xFF3E4941, 0xFF404B43, 0xFF434E46, 0xFF404B43, 0xFF39443C, 0xFF38433B, 0xFF3D4840, 0xFF414C44, 0xFF404B43, 0xFF39443C, 0xFF414C44, 0xFF404B43,
        0xFF414C44, 0xFF38433B, 0xFF3E4941, 0xFF3F4A44, 0xFF2E3933, 0xFF2F3A34, 0xFF3C4741, 0xFF414C46, 0xFF434E48, 0xFF45504A, 0xFF424D47, 0xFF3C4741, 0xFF424D47, 0xFF414C46, 0xFF404B45, 0xFF434E48, 0xFF3B4640, 0xFF2F3A34, 0xFF2F3A34, 0xFF2D3937,
        0xFF313A37, 0xFF2F3835, 0xFF2C3A3A, 0xFF2A3D3B, 0xFF263123, 0xFF342E0A, 0xFF523D08, 0xFF594405, 0xFF5A4506, 0xFF5C4708, 0xFF5D4809, 0xFF5D4809, 0xFF5D4809, 0xFF5C4708, 0xFF5C4708, 0xFF5F470B, 0xFF5E460A, 0xFF655011, 0xFF695415, 0xFF5C4706,
        0xFF523F00, 0xFF5A4703, 0xFF604D09, 0xFF5D4E00, 0xFF5F4E00, 0xFF574500, 0xFF554300, 0xFF5F4E00, 0xFF564600, 0xFF554400, 0xFF5F4C07, 0xFF534B03, 0xFF4C4204, 0xFF4D4308, 0xFF504706, 0xFF4D4500, 0xFF4A4200, 0xFF3D3100, 0xFF4F4110, 0xFF5B4410,
        0xFF4E390E, 0xFF795C10, 0xFFC3932F, 0xFF704100, 0xFF4D3A00, 0xFF454705, 0xFF707437, 0xFF323139, 0xFF000005, 0xFF080B10, 0xFF0E1316, 0xFF121C1B, 0xFF162220, 0xFF1B2A25, 0xFF0A1B15, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FDFBFC, 0x00FFFEFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF171918, 0xFF161C1C, 0xFF030907, 0xFF040A06, 0xFF070C06, 0xFF030801, 0xFF0A0D02, 0xFF0D1103, 0xFF0F1305, 0xFF0F0E09, 0xFF0F0E09, 0xFF0D0C07, 0xFF11100B, 0xFF15140F,
        0xFF0F0E09, 0xFF090803, 0xFF0E0D08, 0xFF160C0A, 0xFF1C1713, 0xFF12120A, 0xFF0E1106, 0xFF131507, 0xFF161405, 0xFF1B1404, 0xFF160D00, 0xFF1F1A04, 0xFF2A210F, 0xFF281A0F, 0xFF291A13, 0xFF1A0F0D, 0xFF1A1917, 0xFF2E3935, 0xFF1D2E28, 0xFF303322,
        0xFF393B2D, 0xFF3F4134, 0xFF272A1F, 0xFF181B12, 0xFF262823, 0xFF2B302C, 0xFF252A26, 0xFF323B36, 0xFF37403B, 0xFF3B453D, 0xFF3F463E, 0xFF3D453A, 0xFF373D33, 0xFF272D21, 0xFF82887C, 0xFF707756, 0xFFADAEB2, 0xFFE7D992, 0xFFFBD33E, 0xFFCB9729,
        0xFF6A4D00, 0xFF433B00, 0xFF665716, 0xFF5B470A, 0xFF524800, 0xFF48410D, 0xFF3F330B, 0xFF3B2B00, 0xFF3F2E10, 0xFF2F2428, 0xFF232426, 0xFF272A1F, 0xFF22251C, 0xFF252821, 0xFF262722, 0xFF2A2B25, 0xFF2B2B23, 0xFF19190D, 0xFF121303, 0xFF372703,
        0xFF433B00, 0xFF534D1B, 0xFF534E28, 0xFF575313, 0xFF4D4305, 0xFF574513, 0xFF5B4806, 0xFF896A18, 0xFFB8932B, 0xFFD5A326, 0xFFDCA01C, 0xFFE09F27, 0xFFDCA33A, 0xFFC79A3C, 0xFFA58224, 0xFF9B7716, 0xFF9D781D, 0xFF9F8844, 0xFF848867, 0xFF8A9889,
        0xFFABAFA1, 0xFF2E3025, 0xFF2F3B39, 0xFF384440, 0xFF36423E, 0xFF34403C, 0xFF34403C, 0xFF34403C, 0xFF35413D, 0xFF34403C, 0xFF34403C, 0xFF344340, 0xFF374642, 0xFF364541, 0xFF374643, 0xFF384743, 0xFF394743, 0xFF384742, 0xFF374643, 0xFF384743,
        0xFF354441, 0xFF32413E, 0xFF364543, 0xFF3D4B48, 0xFF3E4C48, 0xFF3C4B48, 0xFF3A4946, 0xFF414F4D, 0xFF425251, 0xFF415150, 0xFF465555, 0xFF455454, 0xFF3E4E4D, 0xFF41514F, 0xFF455451, 0xFF42504D, 0xFF404F4C, 0xFF515E5C, 0xFFC6CAC9, 0xFFA7ACAC,
        0xFFD8DBDB, 0xFF697675, 0xFF85908F, 0xFFC4C9C8, 0xFFB2B9B9, 0xFFC5CACA, 0xFF475654, 0xFF41504E, 0xFF485755, 0xFF455554, 0xFF3B4B4B, 0xFF3F4F4D, 0xFF42514E, 0xFF445350, 0xFF3F4E4C, 0xFF404F4D, 0xFF425251, 0xFF465655, 0xFF415150, 0xFF445453,
        0xFF435251, 0xFF465656, 0xFF3E4E4E, 0xFF455554, 0xFF485857, 0xFF495958, 0xFF465655, 0xFF435351, 0xFF42514E, 0xFF435350, 0xFF445451, 0xFF3F4E4C, 0xFF3B4948, 0xFF384646, 0xFF384646, 0xFF303B3F, 0xFF232C1B, 0xFF878B8A, 0xFF939B8E, 0xFF7A816F,
        0xFF8C8364, 0xFF977E19, 0xFFBA8D10, 0xFFD19115, 0xFFD59A24, 0xFFC99B2C, 0xFF9A7816, 0xFF674F00, 0xFF5A4800, 0xFF584A0B, 0xFF463A00, 0xFF3E3700, 0xFF273224, 0xFF2B4041, 0xFF2E3B32, 0xFF35393C, 0xFF2E3A38, 0xFF314136, 0xFF373B44, 0xFF313C36,
        0xFF323D37, 0xFF343F39, 0xFF36413B, 0xFF36413B, 0xFF36413B, 0xFF37423C, 0xFF38433D, 0xFF37423E, 0xFF37423E, 0xFF37423E, 0xFF38433F, 0xFF36413D, 0xFF333E3A, 0xFF323D39, 0xFF333E3A, 0xFF38433F, 0xFF35403C, 0xFF303B37, 0xFF38433F, 0xFF333E3A,
        0xFF3A4541, 0xFF36413D, 0xFF35403C, 0xFF39443E, 0xFF2F3A34, 0xFF2E3933, 0xFF343F39, 0xFF35403A, 0xFF323D37, 0xFF333E38, 0xFF38433D, 0xFF333D3C, 0xFF353F3E, 0xFF36403F, 0xFF353F3E, 0xFF343E3D, 0xFF303A39, 0xFF2D3736, 0xFF2C3635, 0xFF2D3937,
        0xFF323B38, 0xFF2F3835, 0xFF2C3A3A, 0xFF2B3E3C, 0xFF273224, 0xFF342E0A, 0xFF523D08, 0xFF594405, 0xFF5A4506, 0xFF5B4607, 0xFF5D4809, 0xFF5D4809, 0xFF5D4809, 0xFF5C4708, 0xFF5C4708, 0xFF5A4609, 0xFF594508, 0xFF5B4708, 0xFF594705, 0xFF4D3C00,
        0xFF6A5B16, 0xFFA4964F, 0xFFAFA15A, 0xFFB4AA55, 0xFFB7AB5F, 0xFFBAB069, 0xFFA89F54, 0xFFB3AB56, 0xFFAFA850, 0xFFB3A858, 0xFFAEA15B, 0xFFAFA162, 0xFFBAAB72, 0xFFA89962, 0xFF625214, 0xFF4D3F00, 0xFF584A00, 0xFF574503, 0xFF57430E, 0xFF554104,
        0xFF503B04, 0xFF86670D, 0xFFDFAB3D, 0xFF824E00, 0xFF5E4606, 0xFF545710, 0xFF85894C, 0xFF333438, 0xFF000004, 0xFF06090E, 0xFF0C0F14, 0xFF161B1E, 0xFF1A2322, 0xFF202B27, 0xFF0E1913, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFF, 0x00FFFFFF, 0x00FBFBFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF1F2324, 0xFF161F1E, 0xFF010707, 0xFF000604, 0xFF040B04, 0xFF060B04, 0xFF0D1307, 0xFF111507, 0xFF111506, 0xFF100F0A, 0xFF0F0E09, 0xFF12110C, 0xFF201F1A, 0xFF2A2924,
        0xFF1E1D18, 0xFF0F0E09, 0xFF0E0D08, 0xFF180E0D, 0xFF14100D, 0xFF0C0F08, 0xFF101A11, 0xFF111B10, 0xFF0F1509, 0xFF191B10, 0xFF1A170E, 0xFF2F1D07, 0xFF432A16, 0xFF3C1D0B, 0xFF321507, 0xFF210B00, 0xFF2B271C, 0xFF46564B, 0xFF223E32, 0xFF121504,
        0xFF232517, 0xFF383A2C, 0xFF323226, 0xFF25251B, 0xFF2B2B23, 0xFF35352D, 0xFF413E39, 0xFF31423C, 0xFF364540, 0xFF35443D, 0xFF37473D, 0xFF364339, 0xFF313F32, 0xFF293529, 0xFF869284, 0xFF72755A, 0xFF9DA095, 0xFFD6CB79, 0xFFC69723, 0xFF7E410B,
        0xFF5A3E0F, 0xFF5A5A02, 0xFF544A03, 0xFF5C450F, 0xFF594B04, 0xFF57490C, 0xFF4C3800, 0xFF664B00, 0xFF81642A, 0xFF321C04, 0xFF120800, 0xFF050700, 0xFF0A0A00, 0xFF14140C, 0xFF13130B, 0xFF0B0801, 0xFF0A0800, 0xFF171405, 0xFF211E0D, 0xFF3D2600,
        0xFF4B3E00, 0xFF8C834A, 0xFF9B936F, 0xFF7F7D40, 0xFF6D672B, 0xFF807349, 0xFF9A8B52, 0xFFA79F6E, 0xFF9E883C, 0xFFBE9625, 0xFFE2A928, 0xFFCD8C0E, 0xFFBA7C0B, 0xFFC08D20, 0xFFAD871A, 0xFFA07D19, 0xFF9B7717, 0xFF9F8842, 0xFF7F835E, 0xFF859384,
        0xFFB0B3AA, 0xFF343434, 0xFF343E48, 0xFF35413D, 0xFF34403C, 0xFF333F3B, 0xFF333F3B, 0xFF34403C, 0xFF35413D, 0xFF35413D, 0xFF34403C, 0xFF32413E, 0xFF394845, 0xFF384743, 0xFF364540, 0xFF384742, 0xFF394843, 0xFF384742, 0xFF374742, 0xFF3A4945,
        0xFF324240, 0xFF384846, 0xFF3C4C4A, 0xFF3F4E4C, 0xFF41504F, 0xFF3F4E4C, 0xFF3A4946, 0xFF42504D, 0xFF455554, 0xFF425251, 0xFF3D4D4D, 0xFF405050, 0xFF415151, 0xFF40504E, 0xFF455451, 0xFF3E4D4A, 0xFF414F4D, 0xFF435351, 0xFF404F4D, 0xFF414E4B,
        0xFF3F4F4D, 0xFF40504F, 0xFF445453, 0xFF42504E, 0xFF425150, 0xFF41514F, 0xFF425250, 0xFF445451, 0xFF43524F, 0xFF425150, 0xFF415151, 0xFF3E4E4D, 0xFF41514F, 0xFF455451, 0xFF42514E, 0xFF41504D, 0xFF435250, 0xFF425251, 0xFF40504F, 0xFF445453,
        0xFF42504E, 0xFF425150, 0xFF425251, 0xFF445453, 0xFF485857, 0xFF485857, 0xFF445451, 0xFF41504C, 0xFF3F4E49, 0xFF3F4E49, 0xFF43524D, 0xFF404F4C, 0xFF3B4949, 0xFF394747, 0xFF384646, 0xFF39434C, 0xFF1D2915, 0xFF848983, 0xFF979F8A, 0xFF7B8069,
        0xFF8A855F, 0xFF9B8919, 0xFFBB9A0F, 0xFFCF9623, 0xFFC7901B, 0xFFCB9823, 0xFFC69B2B, 0xFF9F7B1A, 0xFF6B5000, 0xFF544100, 0xFF564710, 0xFF393500, 0xFF2A382B, 0xFF2E4646, 0xFF2D3A31, 0xFF313538, 0xFF2A3531, 0xFF2C3C31, 0xFF32363F, 0xFF35403A,
        0xFF37423C, 0xFF38433D, 0xFF38433D, 0xFF35403A, 0xFF343F39, 0xFF35403A, 0xFF36413B, 0xFF313B3A, 0xFF353F3E, 0xFF353F3E, 0xFF313B3A, 0xFF323C3B, 0xFF353F3E, 0xFF353F3E, 0xFF313B3A, 0xFF323C3B, 0xFF2B3534, 0xFF2B3534, 0xFF343E3D, 0xFF2A3433,
        0xFF36403F, 0xFF384241, 0xFF303A39, 0xFF2C3731, 0xFF323D37, 0xFF343F39, 0xFF313C36, 0xFF303B35, 0xFF303B35, 0xFF313C36, 0xFF333E38, 0xFF2F393A, 0xFF2F393A, 0xFF303A3B, 0xFF303A3B, 0xFF2D3738, 0xFF2D3738, 0xFF2F393A, 0xFF2F393A, 0xFF2D3937,
        0xFF323B38, 0xFF2F3835, 0xFF2D3B3B, 0xFF2B3E3C, 0xFF273224, 0xFF342E0A, 0xFF523D08, 0xFF594405, 0xFF5A4506, 0xFF5B4607, 0xFF5D4809, 0xFF5D4809, 0xFF5D4809, 0xFF5C4708, 0xFF5C4708, 0xFF5D4A0F, 0xFF5B490B, 0xFF554605, 0xFF584906, 0xFF5D4E09,
        0xFF8A7E36, 0xFFAEA258, 0xFF82762C, 0xFF84783C, 0xFF8B8048, 0xFFB5AC75, 0xFFA8A264, 0xFFACA85E, 0xFFB0AA5E, 0xFFB3AA67, 0xFF8D804B, 0xFF877239, 0xFF8A7540, 0xFFBFA977, 0xFF9D874D, 0xFF5D4700, 0xFF4F3900, 0xFF634A08, 0xFF553A03, 0xFF574601,
        0xFF503C00, 0xFF876607, 0xFFE5AE39, 0xFF824C00, 0xFF5E4703, 0xFF52550E, 0xFF83894D, 0xFF4A4E40, 0xFF000300, 0xFF030801, 0xFF0C110B, 0xFF191E18, 0xFF1C231C, 0xFF232A22, 0xFF121A0F, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFF, 0x00FEFCFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF171916, 0xFF181E1A, 0xFF020806, 0xFF060A09, 0xFF030804, 0xFF040902, 0xFF131709, 0xFF111602, 0xFF161B04, 0xFF181914, 0xFF11110F, 0xFF151316, 0xFF2A232A, 0xFF3A2F35,
        0xFF392B2A, 0xFF2B1B0E, 0xFF52412D, 0xFF444D2E, 0xFF1C1D0B, 0xFF2B2622, 0xFF1C1C1C, 0xFF292F2B, 0xFF373D2F, 0xFF3F3722, 0xFF321C04, 0xFF332603, 0xFF392A0D, 0xFF382507, 0xFF302102, 0xFF1A1207, 0xFF2B2D3A, 0xFF4C5864, 0xFF294036, 0xFF1F2100,
        0xFF504613, 0xFF5B4514, 0xFF493510, 0xFF393220, 0xFF30383A, 0xFF34424D, 0xFF34424F, 0xFF3B454E, 0xFF374246, 0xFF3C4749, 0xFF374341, 0xFF3A453F, 0xFF364038, 0xFF2F362F, 0xFF80857F, 0xFF6B5F51, 0xFFA79776, 0xFFAD9857, 0xFF6A5101, 0xFF563E02,
        0xFFAA9A79, 0xFFA5A67A, 0xFF87974C, 0xFF817D3D, 0xFF797545, 0xFF989872, 0xFF908782, 0xFFAE9251, 0xFFC9A130, 0xFFC1A07F, 0xFF585953, 0xFF686B70, 0xFF5B5351, 0xFF503200, 0xFFA36F09, 0xFF654304, 0xFF424B36, 0xFF4B634D, 0xFF575B44, 0xFF706315,
        0xFF9A8D58, 0xFF897F64, 0xFFA29D87, 0xFF999A78, 0xFF7C8252, 0xFFA4A778, 0xFF989A73, 0xFF809E9E, 0xFF646C23, 0xFF7D6800, 0xFFCEA333, 0xFFDA9F1F, 0xFFB16F01, 0xFF8B5E09, 0xFF858219, 0xFF917222, 0xFF9C7A23, 0xFF998236, 0xFF827F56, 0xFF8E9284,
        0xFFAAAAA2, 0xFF2D3029, 0xFF293C36, 0xFF323F38, 0xFF34413A, 0xFF35423B, 0xFF34433E, 0xFF33423D, 0xFF334340, 0xFF364643, 0xFF384845, 0xFF384845, 0xFF3C4B47, 0xFF394843, 0xFF374641, 0xFF394843, 0xFF3A4944, 0xFF394843, 0xFF394843, 0xFF3A4944,
        0xFF384744, 0xFF3B4A48, 0xFF42514F, 0xFF455352, 0xFF435351, 0xFF42514E, 0xFF3A4946, 0xFF3B4846, 0xFF425251, 0xFF415150, 0xFF445453, 0xFF435251, 0xFF415151, 0xFF3E4D4D, 0xFF435250, 0xFF3D4C4A, 0xFF404E4D, 0xFF445453, 0xFF455452, 0xFF3F4C49,
        0xFF394846, 0xFF3F4F4E, 0xFF435352, 0xFF445250, 0xFF445453, 0xFF445453, 0xFF445453, 0xFF425250, 0xFF40504E, 0xFF3D4D4D, 0xFF405050, 0xFF415151, 0xFF40504E, 0xFF455451, 0xFF42514E, 0xFF42514E, 0xFF43514E, 0xFF404F4D, 0xFF40504E, 0xFF445453,
        0xFF445250, 0xFF445453, 0xFF445453, 0xFF455554, 0xFF445453, 0xFF435350, 0xFF43524E, 0xFF43524D, 0xFF404F4A, 0xFF404F4A, 0xFF41504B, 0xFF3A4945, 0xFF3A4A46, 0xFF3C4C49, 0xFF3C4C49, 0xFF3D443C, 0xFF1A2521, 0xFF808F8C, 0xFF87948A, 0xFF767B5B,
        0xFF968A4E, 0xFF96781E, 0xFFB2891D, 0xFFCBA128, 0xFFCA981D, 0xFFC78D10, 0xFFD4981E, 0xFFC89426, 0xFF9E7D1E, 0xFF5E5304, 0xFF464806, 0xFF3B3400, 0xFF222D1D, 0xFF2B3B48, 0xFF343642, 0xFF333230, 0xFF353E39, 0xFF2B3C36, 0xFF2A372E, 0xFF303B37,
        0xFF323D39, 0xFF2E3935, 0xFF36413D, 0xFF323D39, 0xFF37423E, 0xFF313C38, 0xFF313C38, 0xFF333F3B, 0xFF323E3A, 0xFF313D39, 0xFF303C38, 0xFF2F3B37, 0xFF2E3A36, 0xFF2E3A36, 0xFF2E3A36, 0xFF2F3B39, 0xFF2E3A38, 0xFF2F3B39, 0xFF313D3B, 0xFF2F3B39,
        0xFF2B3735, 0xFF2C3836, 0xFF303C3A, 0xFF2E3A36, 0xFF2E3A36, 0xFF2F3B37, 0xFF2F3B37, 0xFF2F3B37, 0xFF303C38, 0xFF303C38, 0xFF303C38, 0xFF2C3D37, 0xFF2D3E38, 0xFF2D3D3A, 0xFF2D3D3A, 0xFF2C3C39, 0xFF2B3B3A, 0xFF2A3A39, 0xFF293938, 0xFF2A3B33,
        0xFF2A372E, 0xFF313A35, 0xFF303C3C, 0xFF2A3A3A, 0xFF283428, 0xFF323210, 0xFF4E410D, 0xFF554303, 0xFF594707, 0xFF5C4A0A, 0xFF5B4708, 0xFF594506, 0xFF5C4708, 0xFF5D4809, 0xFF5C4708, 0xFF594803, 0xFF5F4D0D, 0xFF564406, 0xFF574803, 0xFF5A4C01,
        0xFFB0A45C, 0xFF80713A, 0xFF9D8B63, 0xFF908359, 0xFF5D521A, 0xFFC2BB6D, 0xFFFCF79B, 0xFFF7F296, 0xFFFFF8AA, 0xFFB1A66E, 0xFF5F5228, 0xFF99844F, 0xFF927D50, 0xFF80713A, 0xFF9C9044, 0xFF4E4300, 0xFF564309, 0xFF5C4513, 0xFF5C4408, 0xFF5B3E04,
        0xFF4A3E0C, 0xFF8D6A04, 0xFFEAAD30, 0xFF825409, 0xFF5F470B, 0xFF645C09, 0xFF7F8942, 0xFF464543, 0xFF000100, 0xFF090F0B, 0xFF111C18, 0xFF121E1A, 0xFF1F2B29, 0xFF212B2A, 0xFF151E1D, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFF, 0x00FEFCFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF181A17, 0xFF141A16, 0xFF000402, 0xFF050908, 0xFF0A0F0B, 0xFF0A0F08, 0xFF171B0D, 0xFF1C210D, 0xFF151A03, 0xFF221E00, 0xFF2D2800, 0xFF231E00, 0xFF302A10, 0xFF3E371D,
        0xFF3E3518, 0xFF3B310E, 0xFF574B25, 0xFF584827, 0xFF351F07, 0xFF3D2817, 0xFF211505, 0xFF272B10, 0xFF414C24, 0xFF4D511E, 0xFF3D3600, 0xFF342F09, 0xFF3E3818, 0xFF302A06, 0xFF211D00, 0xFF191B06, 0xFF4B525A, 0xFF7D8B94, 0xFF4E655B, 0xFF101100,
        0xFF44391D, 0xFF654E2C, 0xFF4D3918, 0xFF2D2711, 0xFF2F352B, 0xFF394440, 0xFF363F3E, 0xFF394738, 0xFF334336, 0xFF303F38, 0xFF3C4743, 0xFF424645, 0xFF35302D, 0xFF1D1008, 0xFF412F25, 0xFF4A3F00, 0xFF7D6B2B, 0xFF675004, 0xFF5C4200, 0xFF9F8A47,
        0xFFD9CBB0, 0xFF9F9F97, 0xFF939F95, 0xFF8A8D56, 0xFF9B9877, 0xFF9C9A8D, 0xFF7A737B, 0xFFE7D691, 0xFFDCC249, 0xFFC5AF86, 0xFF8B9491, 0xFF8F9F7B, 0xFF74725D, 0xFF785909, 0xFFF0BE3B, 0xFFBA9441, 0xFF95907D, 0xFF8F9788, 0xFF969070, 0xFFB5B56F,
        0xFFBDBD87, 0xFF858968, 0xFF8C917A, 0xFF9BA38B, 0xFF818A6B, 0xFF929978, 0xFF8D9371, 0xFF4D5941, 0xFF65651D, 0xFF554200, 0xFF876122, 0xFFD59E37, 0xFFDCA213, 0xFF9C7405, 0xFF5F521E, 0xFF968540, 0xFF937C2E, 0xFF927F3A, 0xFF83815A, 0xFF8C9082,
        0xFFA5A29B, 0xFF292C23, 0xFF2D403A, 0xFF313E37, 0xFF334039, 0xFF36433C, 0xFF35443F, 0xFF35443F, 0xFF344441, 0xFF354542, 0xFF364643, 0xFF344441, 0xFF3E4D48, 0xFF3A4944, 0xFF384742, 0xFF3A4944, 0xFF3A4944, 0xFF384742, 0xFF394843, 0xFF3B4A45,
        0xFF384744, 0xFF3B4A48, 0xFF445352, 0xFF485654, 0xFF445350, 0xFF455452, 0xFF41504D, 0xFF414E4B, 0xFF3F4F4D, 0xFF40504F, 0xFF445453, 0xFF42504E, 0xFF425150, 0xFF41514F, 0xFF425250, 0xFF3E4E4B, 0xFF3F4E4C, 0xFF404F4C, 0xFF455452, 0xFF3D4B49,
        0xFF3B4B49, 0xFF41504E, 0xFF42514F, 0xFF42514F, 0xFF415150, 0xFF425150, 0xFF3F4D4C, 0xFF41514F, 0xFF415150, 0xFF445453, 0xFF435251, 0xFF415151, 0xFF3E4D4D, 0xFF435250, 0xFF41504E, 0xFF435250, 0xFF465553, 0xFF41514E, 0xFF435250, 0xFF465655,
        0xFF435251, 0xFF435352, 0xFF455453, 0xFF415150, 0xFF41514F, 0xFF404F4B, 0xFF43524D, 0xFF44534E, 0xFF42514C, 0xFF404F4A, 0xFF3E4D49, 0xFF3A4A46, 0xFF394946, 0xFF384845, 0xFF354542, 0xFF3F4541, 0xFF1F2A26, 0xFF76837C, 0xFF849082, 0xFF777E5F,
        0xFF97915F, 0xFF907E36, 0xFF7B610C, 0xFFAF891E, 0xFFCB9C2A, 0xFFD69C21, 0xFFD59517, 0xFFD1971C, 0xFFCEA235, 0xFF9A7D21, 0xFF615000, 0xFF3E3400, 0xFF2C3524, 0xFF2C3D47, 0xFF2E303C, 0xFF32312F, 0xFF303934, 0xFF263731, 0xFF344138, 0xFF2A3632,
        0xFF2D3935, 0xFF2C3834, 0xFF333F3B, 0xFF303C38, 0xFF36423E, 0xFF323E3A, 0xFF333F3B, 0xFF37433F, 0xFF36423E, 0xFF34403C, 0xFF323E3A, 0xFF303C38, 0xFF2F3B37, 0xFF2F3B37, 0xFF2E3A36, 0xFF2F3B39, 0xFF2E3A38, 0xFF2F3B39, 0xFF323E3C, 0xFF323E3C,
        0xFF2F3B39, 0xFF303C3A, 0xFF323E3C, 0xFF333F3B, 0xFF323E3A, 0xFF323E3A, 0xFF323E3A, 0xFF313D39, 0xFF313D39, 0xFF303C38, 0xFF303C38, 0xFF2E3D38, 0xFF2E3D38, 0xFF2D3C37, 0xFF2D3C39, 0xFF2D3C39, 0xFF2D3B3B, 0xFF2D3B3B, 0xFF2D3B3B, 0xFF2D3E38,
        0xFF2F3A34, 0xFF323B38, 0xFF2E393B, 0xFF2A393C, 0xFF263228, 0xFF2E2D0E, 0xFF4C3E0D, 0xFF554303, 0xFF594707, 0xFF5B4909, 0xFF5A4607, 0xFF5A4607, 0xFF5C4708, 0xFF5D4809, 0xFF5C4708, 0xFF544300, 0xFF564404, 0xFF564406, 0xFF564702, 0xFF4E4000,
        0xFFACA058, 0xFF7C6D36, 0xFF7B6941, 0xFFCCC47C, 0xFFC0B772, 0xFF544A0C, 0xFFA99F62, 0xFFE8DEA1, 0xFF83793B, 0xFF5B520D, 0xFFA59D55, 0xFFBDA775, 0xFF735E33, 0xFF786932, 0xFFA19549, 0xFF4D4200, 0xFF5D4A10, 0xFF533C08, 0xFF5E460A, 0xFF5F4208,
        0xFF4D410F, 0xFF8E6B05, 0xFFE9AC2D, 0xFF805205, 0xFF5E460A, 0xFF635B08, 0xFF7F8942, 0xFF3A3937, 0xFF000100, 0xFF080E0A, 0xFF16211D, 0xFF192521, 0xFF25312F, 0xFF26302F, 0xFF182120, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFF, 0x00FFFDFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF1A1C19, 0xFF161C18, 0xFF010703, 0xFF080C0B, 0xFF141817, 0xFF10150F, 0xFF282B20, 0xFF525645, 0xFF4F5440, 0xFF706E35, 0xFF767443, 0xFF61613B, 0xFF646543, 0xFF494D2A,
        0xFF2B2C04, 0xFF262400, 0xFF2E2900, 0xFF342307, 0xFF341F04, 0xFF412B14, 0xFF2C200A, 0xFF5B5F46, 0xFFA2AD8B, 0xFFBDC49A, 0xFFB7B585, 0xFFA9AA8A, 0xFF8F9378, 0xFF5D6744, 0xFF313E12, 0xFF192308, 0xFF444E4F, 0xFF717F88, 0xFF435956, 0xFF2F3028,
        0xFF443C25, 0xFF3C2A06, 0xFF2E1C00, 0xFF2A2306, 0xFF232414, 0xFF1E1F17, 0xFF1D1A15, 0xFF172109, 0xFF172413, 0xFF0C1814, 0xFF161E20, 0xFF181713, 0xFF231103, 0xFF3A1B00, 0xFF61370F, 0xFF6A4900, 0xFF643E00, 0xFF663E00, 0xFFAE8C20, 0xFFE5CD63,
        0xFFCBBC79, 0xFF827F60, 0xFF959988, 0xFF8C8C72, 0xFF989375, 0xFF86816D, 0xFF615B5B, 0xFFB5AA72, 0xFF867616, 0xFF8B8257, 0xFFA2B3A0, 0xFF889F6B, 0xFF504B2D, 0xFF775200, 0xFFFDC733, 0xFF996E08, 0xFF483719, 0xFF3B3619, 0xFF463905, 0xFF484A0B,
        0xFF565A1D, 0xFF5F6330, 0xFF6A6E4B, 0xFF72745E, 0xFF696752, 0xFF736E4E, 0xFF666235, 0xFF7B6E42, 0xFF695B1A, 0xFF4E4200, 0xFF57400C, 0xFF8D5D1D, 0xFFD99E2A, 0xFFCEA531, 0xFF7C6B35, 0xFF9C9C66, 0xFF8F8643, 0xFF8D8546, 0xFF80845F, 0xFF8D9183,
        0xFFAAA79E, 0xFF2C2F26, 0xFF293C36, 0xFF313E37, 0xFF334039, 0xFF36433C, 0xFF364540, 0xFF364540, 0xFF354542, 0xFF354542, 0xFF354542, 0xFF354441, 0xFF3C4B46, 0xFF394843, 0xFF394843, 0xFF3A4944, 0xFF394843, 0xFF374641, 0xFF384742, 0xFF3C4B46,
        0xFF384744, 0xFF394845, 0xFF435251, 0xFF465552, 0xFF42514E, 0xFF455451, 0xFF445350, 0xFF3F4C49, 0xFF384745, 0xFF3F4F4E, 0xFF445453, 0xFF445250, 0xFF445453, 0xFF445453, 0xFF425250, 0xFF3D4C4A, 0xFF3B4A49, 0xFF3E4C4B, 0xFF40504E, 0xFF40504F,
        0xFF415151, 0xFF425150, 0xFF43524F, 0xFF42504E, 0xFF435351, 0xFF404F4D, 0xFF414E4B, 0xFF3F4F4D, 0xFF40504F, 0xFF445453, 0xFF42504E, 0xFF425150, 0xFF41514F, 0xFF425250, 0xFF455552, 0xFF475654, 0xFF465655, 0xFF435352, 0xFF425251, 0xFF445453,
        0xFF415150, 0xFF40504F, 0xFF445351, 0xFF3D4C4A, 0xFF3E4D4B, 0xFF3E4E49, 0xFF41504B, 0xFF42514D, 0xFF3F4E4B, 0xFF3B4A47, 0xFF394945, 0xFF394946, 0xFF374744, 0xFF354542, 0xFF344441, 0xFF363F3E, 0xFF242D2A, 0xFF778379, 0xFF899684, 0xFF6F7A5C,
        0xFF898E66, 0xFF9F9C6F, 0xFF888250, 0xFF755906, 0xFFAA8324, 0xFFD19D2D, 0xFFD29516, 0xFFC58702, 0xFFD49A1D, 0xFFC7992A, 0xFFA17A15, 0xFF443700, 0xFF2B3220, 0xFF313F48, 0xFF2F333C, 0xFF2A2B26, 0xFF253028, 0xFF23342E, 0xFF323D35, 0xFF263731,
        0xFF293A34, 0xFF2A3B35, 0xFF2F403A, 0xFF2D3E38, 0xFF30413B, 0xFF2F403A, 0xFF31423C, 0xFF394541, 0xFF384440, 0xFF37433F, 0xFF35413D, 0xFF34403C, 0xFF333F3B, 0xFF333F3B, 0xFF333F3B, 0xFF303C3A, 0xFF2F3B39, 0xFF303C3A, 0xFF34403E, 0xFF364240,
        0xFF35413F, 0xFF34403E, 0xFF364240, 0xFF35413D, 0xFF35413D, 0xFF35413D, 0xFF35413D, 0xFF35413D, 0xFF35413D, 0xFF35413D, 0xFF34403C, 0xFF33423B, 0xFF32413C, 0xFF303F3A, 0xFF2E3D38, 0xFF2D3C39, 0xFF2D3C39, 0xFF2D3B3B, 0xFF2E3C3C, 0xFF2C3C39,
        0xFF313C38, 0xFF333939, 0xFF2C373B, 0xFF2C3B40, 0xFF27322A, 0xFF2D290C, 0xFF4B3D10, 0xFF564404, 0xFF594707, 0xFF5B4909, 0xFF5A4607, 0xFF5A4607, 0xFF5D4809, 0xFF5D4809, 0xFF5C4708, 0xFF594803, 0xFF5C4A0A, 0xFF5E4C0E, 0xFF5B4C07, 0xFF4F4100,
        0xFFA89C54, 0xFF786932, 0xFF6F5D35, 0xFF65591C, 0xFFB1A568, 0xFFDBD090, 0xFF726727, 0xFF5B5010, 0xFF756A2A, 0xFFC5B97C, 0xFFA69A5D, 0xFF604A19, 0xFF6F5930, 0xFF81713D, 0xFFA1954B, 0xFF4D4100, 0xFF5A470C, 0xFF5B4410, 0xFF563E02, 0xFF5D4004,
        0xFF4A3E0C, 0xFF8B6802, 0xFFE7AA2B, 0xFF805205, 0xFF60480A, 0xFF675F0C, 0xFF848E47, 0xFF3C3B39, 0xFF000100, 0xFF0A100C, 0xFF16211D, 0xFF182420, 0xFF212D2B, 0xFF252F2E, 0xFF1A2322, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFF, 0x00FFFDFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF1C1E1B, 0xFF1C231C, 0xFF010703, 0xFF000403, 0xFF0A0E0D, 0xFF0F1410, 0xFF575A51, 0xFFD9DDCF, 0xFFFCFFEF, 0xFFF9FFEC, 0xFFFBFFF8, 0xFFF6FFFB, 0xFFDBE7DD, 0xFF788269,
        0xFF333712, 0xFF353207, 0xFF443B12, 0xFF39350F, 0xFF403710, 0xFF3D2E0F, 0xFF352A16, 0xFF9B9A95, 0xFFFCFFFF, 0xFFFFFEFF, 0xFFFFF7FB, 0xFFF9FFF0, 0xFFEEFAEC, 0xFFB6CAAF, 0xFF4B6037, 0xFF273416, 0xFF7D8381, 0xFFA7B1BB, 0xFF718387, 0xFF51553C,
        0xFF87825C, 0xFF5A4A16, 0xFF433400, 0xFF3C3406, 0xFF171300, 0xFF1C1503, 0xFF190B00, 0xFF110A00, 0xFF110E07, 0xFF060A0D, 0xFF0C0F16, 0xFF0B0801, 0xFF2F1F00, 0xFF69490E, 0xFF976E1F, 0xFFBE8F1D, 0xFFD5A536, 0xFFECBF40, 0xFFF1CA3E, 0xFFB99B23,
        0xFF927B37, 0xFF65573C, 0xFF565042, 0xFF4D403A, 0xFF5D4D12, 0xFF675513, 0xFF68593A, 0xFF5F512A, 0xFF443000, 0xFF6B613D, 0xFF89996C, 0xFF7D876E, 0xFF2E1B00, 0xFF8C5A03, 0xFFFEC446, 0xFF996D16, 0xFF412F00, 0xFF403A06, 0xFF4B450B, 0xFF4B410E,
        0xFF3F3900, 0xFF514B0D, 0xFF3A3201, 0xFF291A00, 0xFF190600, 0xFF3D2700, 0xFF7F6824, 0xFFAD913E, 0xFF523700, 0xFF53470A, 0xFF59500B, 0xFF5B3800, 0xFFB78026, 0xFFD0A534, 0xFF846F1C, 0xFF768254, 0xFF5E5E22, 0xFF67652C, 0xFF727957, 0xFF8C9284,
        0xFFABA89F, 0xFF2B2C24, 0xFF293C36, 0xFF334039, 0xFF34413A, 0xFF36433C, 0xFF364540, 0xFF364540, 0xFF354542, 0xFF354542, 0xFF354542, 0xFF364542, 0xFF3A4944, 0xFF394843, 0xFF384742, 0xFF394843, 0xFF374641, 0xFF35443F, 0xFF374641, 0xFF3B4A45,
        0xFF374643, 0xFF354441, 0xFF425250, 0xFF41504D, 0xFF3D4C49, 0xFF42514E, 0xFF41504E, 0xFF3E4C49, 0xFF3A4946, 0xFF404F4C, 0xFF435251, 0xFF42514F, 0xFF435352, 0xFF435250, 0xFF3D4C49, 0xFF41504D, 0xFF40504D, 0xFF40504F, 0xFF445453, 0xFF435251,
        0xFF415151, 0xFF40504F, 0xFF41504E, 0xFF414F4E, 0xFF445453, 0xFF455452, 0xFF3F4C49, 0xFF384745, 0xFF3F4F4E, 0xFF445453, 0xFF445250, 0xFF445453, 0xFF445453, 0xFF455554, 0xFF455554, 0xFF455554, 0xFF455554, 0xFF455554, 0xFF415150, 0xFF40504F,
        0xFF3D4D4D, 0xFF3F4F4E, 0xFF3F4F4E, 0xFF394845, 0xFF364542, 0xFF384844, 0xFF3B4A47, 0xFF3C4B48, 0xFF384744, 0xFF364542, 0xFF384744, 0xFF394845, 0xFF364542, 0xFF364542, 0xFF394845, 0xFF3D4548, 0xFF242D2A, 0xFF778274, 0xFF8F9B85, 0xFF717E62,
        0xFF798466, 0xFF868F72, 0xFF7A8368, 0xFF998D59, 0xFF95813C, 0xFFAB8928, 0xFFCD9C25, 0xFFCE9512, 0xFFCB900E, 0xFFCD951E, 0xFFC18D1F, 0xFF5C4911, 0xFF252813, 0xFF27363D, 0xFF1F262C, 0xFF0D1009, 0xFF111E15, 0xFF263731, 0xFF2E3830, 0xFF2A3B35,
        0xFF2C3D37, 0xFF2E3F39, 0xFF30413B, 0xFF2D3E38, 0xFF2E3F39, 0xFF2E3F39, 0xFF30413B, 0xFF38433D, 0xFF38433D, 0xFF38433D, 0xFF38433D, 0xFF38433D, 0xFF39443E, 0xFF3A453F, 0xFF3A453F, 0xFF343F3B, 0xFF323D39, 0xFF333E3A, 0xFF37423E, 0xFF3A4541,
        0xFF394440, 0xFF38433F, 0xFF38433F, 0xFF35403A, 0xFF35403A, 0xFF36413B, 0xFF37423C, 0xFF39443E, 0xFF3A453F, 0xFF3B4640, 0xFF3B4640, 0xFF3B4841, 0xFF3A4740, 0xFF37433F, 0xFF35413D, 0xFF323E3A, 0xFF303C3A, 0xFF2F3B39, 0xFF2E3A38, 0xFF293737,
        0xFF313B3A, 0xFF32373A, 0xFF2C353C, 0xFF303D45, 0xFF29322D, 0xFF2B270C, 0xFF4D3C10, 0xFF564404, 0xFF594707, 0xFF5A4808, 0xFF5A4607, 0xFF5A4607, 0xFF5D4809, 0xFF5D4809, 0xFF5B4607, 0xFF564500, 0xFF5B4909, 0xFF544204, 0xFF514200, 0xFF534500,
        0xFFAB9F57, 0xFF8A7B44, 0xFF9B8961, 0xFF9C8F58, 0xFF52460C, 0xFF988C4F, 0xFFDBD090, 0xFFA79C5C, 0xFFC0B477, 0xFF8A7E44, 0xFF52450E, 0xFF957E52, 0xFF957F58, 0xFF82723F, 0xFFA3974F, 0xFF554900, 0xFF503D02, 0xFF57400C, 0xFF5B4305, 0xFF5C3F03,
        0xFF493D09, 0xFF8A6700, 0xFFE8AB2C, 0xFF825407, 0xFF624A0C, 0xFF68600B, 0xFF848E45, 0xFF3D3C3A, 0xFF020401, 0xFF0B110D, 0xFF16211D, 0xFF17231F, 0xFF1C2826, 0xFF212B2A, 0xFF17201F, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF1F211E, 0xFF181F17, 0xFF040B04, 0xFF0C100F, 0xFF161A19, 0xFF0D120E, 0xFF4C4F48, 0xFFD9DCD1, 0xFFF5F9EB, 0xFFF1FAFF, 0xFFF8FFFF, 0xFFE5EEFD, 0xFFB4BAB6, 0xFF666745,
        0xFF4C4004, 0xFF533800, 0xFF643F00, 0xFF5A4000, 0xFF5F3F00, 0xFF613E00, 0xFF583D10, 0xFF9B8F81, 0xFFE0DEE9, 0xFFF2F0FF, 0xFFFFFAFF, 0xFFE9F2F1, 0xFFC3D3D3, 0xFF96AD9D, 0xFF3B4D25, 0xFF292D08, 0xFF6D665C, 0xFF69686E, 0xFF414C50, 0xFF4B4B31,
        0xFF867D54, 0xFF7B6B30, 0xFFAEA15D, 0xFF8C8747, 0xFF343200, 0xFF534916, 0xFF3A2800, 0xFF3B2607, 0xFF2D1E09, 0xFF211C16, 0xFF222224, 0xFF1F1E19, 0xFF3C3219, 0xFF503C0A, 0xFF644906, 0xFF886126, 0xFF856115, 0xFF97780E, 0xFFB59E2A, 0xFF95802D,
        0xFF716340, 0xFF403B28, 0xFF2A2F11, 0xFF341B00, 0xFF815F00, 0xFFA27C1B, 0xFF81642C, 0xFF4A350A, 0xFF422B0B, 0xFF817251, 0xFF8F946B, 0xFF87847B, 0xFF482900, 0xFFA16C0C, 0xFFB07C26, 0xFF76552C, 0xFF494015, 0xFF505422, 0xFF4B4E3D, 0xFF5D553E,
        0xFF5A552F, 0xFF565423, 0xFF383309, 0xFF756A4C, 0xFF857252, 0xFF8F7A45, 0xFFDAC479, 0xFFC9AF3E, 0xFF7A5819, 0xFF5E4912, 0xFF4E4504, 0xFF604A01, 0xFFB68C1C, 0xFFC9A022, 0xFF866715, 0xFF686F3C, 0xFF4A4607, 0xFF5D5B20, 0xFF7A8160, 0xFF959D92,
        0xFFAAA7A0, 0xFF26271F, 0xFF33443C, 0xFF35423B, 0xFF35423B, 0xFF36433C, 0xFF34433E, 0xFF35443F, 0xFF354542, 0xFF364643, 0xFF364643, 0xFF3C4846, 0xFF3F4B47, 0xFF3D4945, 0xFF3B4743, 0xFF3A4642, 0xFF394541, 0xFF384440, 0xFF36423E, 0xFF34403C,
        0xFF374643, 0xFF354441, 0xFF384745, 0xFF3A4947, 0xFF3C4B47, 0xFF3E4D49, 0xFF3E4D48, 0xFF3D4C48, 0xFF3C4B48, 0xFF414F4B, 0xFF41504D, 0xFF414F4D, 0xFF404F4D, 0xFF41514E, 0xFF404F4C, 0xFF414F4B, 0xFF3F4F4D, 0xFF40504F, 0xFF445453, 0xFF42504E,
        0xFF425150, 0xFF415150, 0xFF3E4E4B, 0xFF3F4E4C, 0xFF404F4C, 0xFF455452, 0xFF3C4946, 0xFF394845, 0xFF404F4C, 0xFF435251, 0xFF425150, 0xFF445453, 0xFF465554, 0xFF435352, 0xFF425250, 0xFF425250, 0xFF445352, 0xFF475756, 0xFF445453, 0xFF425251,
        0xFF40504F, 0xFF425252, 0xFF374747, 0xFF394845, 0xFF364542, 0xFF354441, 0xFF354441, 0xFF32413E, 0xFF31403D, 0xFF354441, 0xFF3A4946, 0xFF3A4946, 0xFF364542, 0xFF374643, 0xFF3B4A47, 0xFF3E4649, 0xFF232C29, 0xFF7D887A, 0xFF929E88, 0xFF79866A,
        0xFF7E896B, 0xFF757E61, 0xFF757E63, 0xFF9AA180, 0xFF636130, 0xFF615002, 0xFFA4831A, 0xFFC89C23, 0xFFBE8A10, 0xFFC08C16, 0xFFC38E1E, 0xFF6C5418, 0xFF2B2B13, 0xFF2B393C, 0xFF262E30, 0xFF191E17, 0xFF1C2920, 0xFF2E3F39, 0xFF313733, 0xFF2C3D37,
        0xFF2E3F39, 0xFF30413B, 0xFF2F403A, 0xFF2F403A, 0xFF2D3E38, 0xFF2F403A, 0xFF30413B, 0xFF37423C, 0xFF37423C, 0xFF37423C, 0xFF37423C, 0xFF38433D, 0xFF38433D, 0xFF39443E, 0xFF3A453F, 0xFF36413D, 0xFF333E3A, 0xFF333E3A, 0xFF37423E, 0xFF394440,
        0xFF37423E, 0xFF36413D, 0xFF37423E, 0xFF343F39, 0xFF343F39, 0xFF35403A, 0xFF36413B, 0xFF37423C, 0xFF39443E, 0xFF3A453F, 0xFF3A453F, 0xFF3B4640, 0xFF3A453F, 0xFF39443E, 0xFF38433F, 0xFF35403C, 0xFF333E3A, 0xFF313B3A, 0xFF2F3938, 0xFF293835,
        0xFF313A39, 0xFF323838, 0xFF2D363D, 0xFF303D45, 0xFF29322D, 0xFF2B250B, 0xFF4A390D, 0xFF574505, 0xFF594707, 0xFF5A4808, 0xFF5A4607, 0xFF5A4607, 0xFF5E490A, 0xFF5E490A, 0xFF5B4607, 0xFF5A4904, 0xFF5F4D0D, 0xFF5B490B, 0xFF564702, 0xFF534500,
        0xFFA49850, 0xFF7F7039, 0xFF827048, 0xFFD9CD83, 0xFFC0B273, 0xFF5A4A19, 0xFF79673F, 0xFFBFAD85, 0xFF6F5F2E, 0xFF5C4E0F, 0xFFB0A45A, 0xFFC5AE84, 0xFF7F6944, 0xFF7D6D3C, 0xFFA3974F, 0xFF4F4300, 0xFF5D4A0F, 0xFF563F09, 0xFF5F4707, 0xFF614506,
        0xFF4D410D, 0xFF8E6B03, 0xFFEAAE2C, 0xFF835507, 0xFF604808, 0xFF635B06, 0xFF7C863D, 0xFF302F2D, 0xFF000100, 0xFF090F0B, 0xFF192420, 0xFF1D2925, 0xFF1F2B29, 0xFF222C2B, 0xFF161F1E, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFDFF, 0x00FFFEFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF212320, 0xFF1D241C, 0xFF040B04, 0xFF0D120E, 0xFF171B1A, 0xFF0C100F, 0xFF454742, 0xFFCED1CA, 0xFFDCDFD4, 0xFFE8E9E3, 0xFFE9EAEE, 0xFFBBBBBD, 0xFF93917A, 0xFF9F9356,
        0xFFCCAF53, 0xFFD5A43E, 0xFFDC9D3C, 0xFFE5A43E, 0xFFE4A234, 0xFFEDAF42, 0xFFC69944, 0xFFAA9566, 0xFFACAD9D, 0xFFC1CAC7, 0xFFEAF4F3, 0xFFC3CCD1, 0xFFBFCED3, 0xFFA7B9A9, 0xFF4A511E, 0xFF433400, 0xFF7D6445, 0xFF564939, 0xFF2B2F20, 0xFF5D5445,
        0xFF524123, 0xFF4F3B08, 0xFFA89958, 0xFF8F8B42, 0xFF7B792C, 0xFFD5CF7B, 0xFF9E8F34, 0xFF3D2D00, 0xFF342900, 0xFF2B270C, 0xFF2C2F24, 0xFF272B2A, 0xFF2D2E29, 0xFF282117, 0xFF2E2212, 0xFF241733, 0xFF493E44, 0xFF8E8969, 0xFFB6B684, 0xFFA8A88C,
        0xFFCFD4D0, 0xFFBECBBA, 0xFF859B6A, 0xFF624800, 0xFFDAAF2F, 0xFFD79C42, 0xFF663D00, 0xFF4D3B00, 0xFF413302, 0xFF948764, 0xFFBAB3A9, 0xFF878471, 0xFF91732D, 0xFFC3962F, 0xFF63420C, 0xFF3E2F32, 0xFF323323, 0xFF343C24, 0xFF282E4E, 0xFF373D3B,
        0xFF3C463D, 0xFF596454, 0xFF798473, 0xFFD4DBC9, 0xFFD8DAC4, 0xFFDEDBB8, 0xFFEFEBBC, 0xFFFFEDB3, 0xFFE8CB6D, 0xFFB29327, 0xFF624E07, 0xFF58430C, 0xFFA78720, 0xFFD8B130, 0xFFA97F1D, 0xFF6C5F1B, 0xFF635004, 0xFF776C2C, 0xFF767D5B, 0xFF89958B,
        0xFFADAEA8, 0xFF2B2C24, 0xFF304036, 0xFF37443D, 0xFF36433C, 0xFF35423B, 0xFF34433E, 0xFF35443F, 0xFF354542, 0xFF364643, 0xFF364643, 0xFF3E4A48, 0xFF3E4A46, 0xFF3C4844, 0xFF3A4642, 0xFF394541, 0xFF394541, 0xFF394541, 0xFF384440, 0xFF37433F,
        0xFF3A4946, 0xFF3B4A47, 0xFF3A4A47, 0xFF374744, 0xFF3C4B46, 0xFF3E4D49, 0xFF3E4D49, 0xFF3B4B48, 0xFF3B4B4A, 0xFF3F4C4A, 0xFF404F4D, 0xFF414F4E, 0xFF445352, 0xFF445453, 0xFF445351, 0xFF3F4C49, 0xFF384745, 0xFF3F4F4E, 0xFF445453, 0xFF445251,
        0xFF435352, 0xFF3E4E4C, 0xFF3B4A48, 0xFF3B4A48, 0xFF3E4C4A, 0xFF42524F, 0xFF414F4B, 0xFF3B4A48, 0xFF424C49, 0xFF43514E, 0xFF425150, 0xFF40504F, 0xFF455452, 0xFF435250, 0xFF43524F, 0xFF455452, 0xFF435351, 0xFF465655, 0xFF445454, 0xFF425251,
        0xFF415150, 0xFF405050, 0xFF3B4948, 0xFF3C4846, 0xFF384442, 0xFF2C3836, 0xFF24302E, 0xFF182422, 0xFF1D2927, 0xFF263230, 0xFF303C3A, 0xFF374341, 0xFF394543, 0xFF3A4644, 0xFF3B4745, 0xFF37403F, 0xFF1F2825, 0xFF7B877D, 0xFF808D7B, 0xFF6D785A,
        0xFF7B8058, 0xFF636033, 0xFF5A5422, 0xFF7C8B6A, 0xFF515729, 0xFF4B4200, 0xFF8C7317, 0xFFC09A2F, 0xFFB98B1B, 0xFFC08E1D, 0xFFC69224, 0xFF6A4C0E, 0xFF343015, 0xFF313F40, 0xFF323C3B, 0xFF3A4237, 0xFF2F3F35, 0xFF2E3D38, 0xFF303632, 0xFF2E3A36,
        0xFF2D3935, 0xFF313D39, 0xFF2E3A36, 0xFF313D39, 0xFF2F3B37, 0xFF323E3A, 0xFF323E3A, 0xFF354038, 0xFF343F37, 0xFF333E36, 0xFF323D35, 0xFF323D35, 0xFF323D35, 0xFF323D35, 0xFF323D35, 0xFF333E38, 0xFF303B35, 0xFF2F3A34, 0xFF323D37, 0xFF333E38,
        0xFF313C36, 0xFF313C36, 0xFF333E38, 0xFF313C34, 0xFF313C34, 0xFF313C34, 0xFF313C34, 0xFF313C34, 0xFF313C34, 0xFF313C34, 0xFF313C34, 0xFF313B33, 0xFF323C34, 0xFF333C37, 0xFF343D38, 0xFF343D3A, 0xFF333C39, 0xFF323B38, 0xFF313A39, 0xFF2D3935,
        0xFF303934, 0xFF323635, 0xFF2D3538, 0xFF303B3F, 0xFF283229, 0xFF2C2709, 0xFF4A3808, 0xFF574505, 0xFF594707, 0xFF594707, 0xFF594506, 0xFF5B4708, 0xFF5E490A, 0xFF5E490A, 0xFF5A4506, 0xFF574601, 0xFF594707, 0xFF604E10, 0xFF5D4E09, 0xFF524400,
        0xFFA89C54, 0xFF82733C, 0xFF5D4B23, 0xFF60501D, 0xFFB8A874, 0xFFE0D19A, 0xFF84753C, 0xFF52430A, 0xFF6C5D26, 0xFFCDBD89, 0xFF9F8F5C, 0xFF674F29, 0xFF634C2C, 0xFF7C6B3D, 0xFFA49751, 0xFF504400, 0xFF5D4A0F, 0xFF5A430D, 0xFF594101, 0xFF604405,
        0xFF4C410B, 0xFF8D6A02, 0xFFEAAE2C, 0xFF845608, 0xFF624A0A, 0xFF645D05, 0xFF7C873B, 0xFF2E2D2B, 0xFF000100, 0xFF050B07, 0xFF16211D, 0xFF1C2824, 0xFF1E2A28, 0xFF242E2D, 0xFF182120, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFDFF, 0x00FFFEFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF232522, 0xFF1E261B, 0xFF000600, 0xFF0A0F0B, 0xFF141819, 0xFF101415, 0xFF4D4F4C, 0xFFD0D2CD, 0xFFD8DBD2, 0xFFC6C9B4, 0xFFBEC3BC, 0xFF8F9591, 0xFF81876D, 0xFF868243,
        0xFF937E23, 0xFFA07519, 0xFFAB711F, 0xFFA47422, 0xFFA36E0E, 0xFFA7740B, 0xFF906B13, 0xFF92834C, 0xFFA2A48C, 0xFFA0AD9C, 0xFFB3C0AE, 0xFFB8C1BE, 0xFFC1CDC9, 0xFFBCC7A9, 0xFF8D843F, 0xFF83600C, 0xFF794D0C, 0xFF452E00, 0xFF464614, 0xFF544423,
        0xFF4F3910, 0xFF4E3401, 0xFF57440C, 0xFF423B04, 0xFF969255, 0xFFFFF8B0, 0xFFBFAE60, 0xFF514E1B, 0xFF404115, 0xFF131800, 0xFF141D0C, 0xFF161F1A, 0xFF11191C, 0xFF10151B, 0xFF151821, 0xFF05110F, 0xFF4F595A, 0xFFD8E2D9, 0xFFF0F7E5, 0xFF979D93,
        0xFFBDC0C7, 0xFFC7CCC6, 0xFF899571, 0xFF5E4900, 0xFFE9BB31, 0xFFCF8D36, 0xFF572E00, 0xFF514901, 0xFF3F3D0A, 0xFF5B5932, 0xFF545351, 0xFF48513C, 0xFF7F6B30, 0xFF8C6E18, 0xFF281F00, 0xFF111828, 0xFF19221F, 0xFF1C2311, 0xFF151D34, 0xFF0E1B14,
        0xFF081414, 0xFF455457, 0xFFB9C9C8, 0xFFD8E8DD, 0xFF889583, 0xFFBCC4B5, 0xFFBABDB2, 0xFFCFCDD8, 0xFFD0B752, 0xFFF2C834, 0xFFA48127, 0xFF543D11, 0xFF83662C, 0xFFC99F3D, 0xFFCF9E1E, 0xFFA57D25, 0xFFA27B20, 0xFFA08A41, 0xFF787E5A, 0xFF85928B,
        0xFFB4B9B5, 0xFF31342B, 0xFF2D3A30, 0xFF37443D, 0xFF36433C, 0xFF36433C, 0xFF35443F, 0xFF374641, 0xFF364643, 0xFF354542, 0xFF344441, 0xFF3A4644, 0xFF3C4844, 0xFF3B4743, 0xFF384440, 0xFF37433F, 0xFF384440, 0xFF394541, 0xFF394541, 0xFF394541,
        0xFF3C4B48, 0xFF3F4E4B, 0xFF3B4B48, 0xFF394947, 0xFF3F4D49, 0xFF42514C, 0xFF41504C, 0xFF3E4D4B, 0xFF415150, 0xFF455351, 0xFF41514D, 0xFF43524F, 0xFF41504D, 0xFF445351, 0xFF43514F, 0xFF3B4946, 0xFF3B4A47, 0xFF404F4C, 0xFF42514F, 0xFF42514F,
        0xFF404F4D, 0xFF3D4C49, 0xFF43524F, 0xFF3F4E4B, 0xFF3F4E4D, 0xFF435352, 0xFF414F4D, 0xFF425250, 0xFF434F4D, 0xFF3F4F4D, 0xFF3D4D4D, 0xFF3F4F4E, 0xFF3F4E4E, 0xFF42524F, 0xFF43524F, 0xFF455451, 0xFF404F4D, 0xFF435352, 0xFF465655, 0xFF455554,
        0xFF415150, 0xFF3A4847, 0xFF374342, 0xFF384442, 0xFF2E3A38, 0xFF1A2624, 0xFF101C1A, 0xFF091513, 0xFF0B1715, 0xFF101C1A, 0xFF1D2927, 0xFF2E3A38, 0xFF3A4644, 0xFF3B4745, 0xFF374341, 0xFF3F4541, 0xFF1F2A26, 0xFF78857E, 0xFF838F81, 0xFF7B8263,
        0xFF9A9462, 0xFF827028, 0xFF654B00, 0xFF646C3B, 0xFF515017, 0xFF4B3D00, 0xFF826710, 0xFFC29A38, 0xFFC19125, 0xFFC28E20, 0xFFC38E1E, 0xFF6D4B0C, 0xFF312C0F, 0xFF293535, 0xFF242F2B, 0xFF313C2E, 0xFF25372B, 0xFF273631, 0xFF2F3430, 0xFF2E3734,
        0xFF2B3431, 0xFF2E3734, 0xFF2A3330, 0xFF2F3835, 0xFF2B3431, 0xFF2F3835, 0xFF2D3633, 0xFF2F3931, 0xFF2E3830, 0xFF2D372F, 0xFF2C362E, 0xFF2B352D, 0xFF2B352D, 0xFF2B352D, 0xFF2B352D, 0xFF2D3631, 0xFF2B342F, 0xFF2A332E, 0xFF2B342F, 0xFF2B342F,
        0xFF2A332E, 0xFF2B342F, 0xFF2F3833, 0xFF2A342C, 0xFF2A342C, 0xFF2A342C, 0xFF29332B, 0xFF29332B, 0xFF29332B, 0xFF28322A, 0xFF28322A, 0xFF293029, 0xFF2A312A, 0xFF2B322B, 0xFF2D332F, 0xFF2E3430, 0xFF2F3533, 0xFF303634, 0xFF303634, 0xFF2D3A31,
        0xFF2C332B, 0xFF2D322C, 0xFF2B3433, 0xFF2B3737, 0xFF2B3326, 0xFF332F0C, 0xFF503F0B, 0xFF574505, 0xFF594707, 0xFF594707, 0xFF594506, 0xFF5B4708, 0xFF5F4A0B, 0xFF5E490A, 0xFF5A4506, 0xFF5E4D08, 0xFF594707, 0xFF5C4A0C, 0xFF544500, 0xFF4C3E00,
        0xFFA89C54, 0xFF8C7D46, 0xFF64522A, 0xFF6E584A, 0xFF5F4D29, 0xFF847733, 0xFFE4DA81, 0xFF91872E, 0xFFD4C783, 0xFF806E4A, 0xFF5B4537, 0xFF644C28, 0xFF796243, 0xFF79683C, 0xFFA79A54, 0xFF594D03, 0xFF4E3B00, 0xFF553F06, 0xFF5B4401, 0xFF5E4203,
        0xFF4A3F09, 0xFF8A6800, 0xFFE9AD29, 0xFF845706, 0xFF644D0A, 0xFF686109, 0xFF818C40, 0xFF343331, 0xFF010300, 0xFF030905, 0xFF131E1A, 0xFF1A2622, 0xFF1C2826, 0xFF242E2D, 0xFF1A2322, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFDFF, 0x00FFFEFF, 0x00FCFCFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF242623, 0xFF1E261B, 0xFF000600, 0xFF0E130F, 0xFF171B1C, 0xFF141819, 0xFF4E504F, 0xFFCACCC7, 0xFFCFD2CB, 0xFFACBEA8, 0xFF8EA19D, 0xFF738A92, 0xFF95ACA4, 0xFF74845D,
        0xFF3F4004, 0xFF4A3500, 0xFF59330C, 0xFF383812, 0xFF4A440A, 0xFF463800, 0xFF453700, 0xFF918C6C, 0xFFCBCDC8, 0xFFAFB3B4, 0xFFA8A9A4, 0xFFACB4A5, 0xFFBFCABA, 0xFFC2C89C, 0xFFCEBA63, 0xFFEFC056, 0xFFCB963A, 0xFF6A4D00, 0xFF434400, 0xFF513F00,
        0xFF5A4300, 0xFF654B0E, 0xFF4A3604, 0xFF4B411E, 0xFFADA68A, 0xFFFFF6D7, 0xFFFFF0CC, 0xFFE7F0DD, 0xFFA7B09D, 0xFF283322, 0xFF0F1C0B, 0xFF19271A, 0xFF132019, 0xFF14201E, 0xFF0F191A, 0xFF18260D, 0xFF161C1A, 0xFF6F7072, 0xFF9E9D8B, 0xFF7D7954,
        0xFFC1BA8C, 0xFFD3CD8F, 0xFFA5A04C, 0xFF554200, 0xFFE6BD31, 0xFFC78926, 0xFF644000, 0xFF454507, 0xFF7D8465, 0xFFA6B189, 0xFF87907F, 0xFF8BA09B, 0xFF93886C, 0xFF5D4A10, 0xFF192300, 0xFF072027, 0xFF1A2922, 0xFF1F2604, 0xFF19220F, 0xFF161A01,
        0xFF141A10, 0xFF272C30, 0xFF98A1A0, 0xFFB6C1B1, 0xFF616A57, 0xFFADB0A9, 0xFF7C7984, 0xFF8A9AA9, 0xFF705609, 0xFFEFB440, 0xFFC49638, 0xFF513F00, 0xFF564011, 0xFF87591B, 0xFFD49D1E, 0xFFCA912A, 0xFFAD7812, 0xFF977A2A, 0xFF727651, 0xFF84938E,
        0xFFADB3B1, 0xFF292C23, 0xFF364236, 0xFF36433C, 0xFF36433C, 0xFF36433C, 0xFF364540, 0xFF384742, 0xFF374744, 0xFF354542, 0xFF32423F, 0xFF384241, 0xFF3C4846, 0xFF3B4743, 0xFF384440, 0xFF36423E, 0xFF37433F, 0xFF384440, 0xFF394541, 0xFF394541,
        0xFF3C4B48, 0xFF3F4E4B, 0xFF364643, 0xFF364745, 0xFF3E4C48, 0xFF41504B, 0xFF404F4A, 0xFF3D4C49, 0xFF445352, 0xFF465552, 0xFF42514E, 0xFF455451, 0xFF42514E, 0xFF42514E, 0xFF445350, 0xFF3C4B48, 0xFF3A4845, 0xFF3F4C49, 0xFF404E4D, 0xFF3E4E4D,
        0xFF42514F, 0xFF41504E, 0xFF43524E, 0xFF43524F, 0xFF3F4F4D, 0xFF445352, 0xFF455352, 0xFF3E4E4B, 0xFF414F4D, 0xFF425251, 0xFF40504F, 0xFF425252, 0xFF384848, 0xFF3F4F4D, 0xFF42514E, 0xFF445350, 0xFF3F4E4C, 0xFF404F4D, 0xFF425251, 0xFF425251,
        0xFF40504E, 0xFF3B4847, 0xFF364240, 0xFF34403E, 0xFF25312F, 0xFF111D1B, 0xFF0A1614, 0xFF14201E, 0xFF0F1B19, 0xFF0B1715, 0xFF131F1D, 0xFF283432, 0xFF3B4745, 0xFF3C4846, 0xFF34403E, 0xFF3B423A, 0xFF202B27, 0xFF798885, 0xFF859288, 0xFF717656,
        0xFF9B8F53, 0xFFAD8F35, 0xFFA37A0E, 0xFF595515, 0xFF564D08, 0xFF574100, 0xFF8B6711, 0xFFC79939, 0xFFBC871D, 0xFFC08817, 0xFFCC9320, 0xFF775414, 0xFF2F280B, 0xFF2B3735, 0xFF26322E, 0xFF283426, 0xFF1F3125, 0xFF2A3934, 0xFF2A2F2B, 0xFF303634,
        0xFF2C3230, 0xFF2E3432, 0xFF272D2B, 0xFF2D3331, 0xFF272D2B, 0xFF2A302E, 0xFF262C2A, 0xFF273129, 0xFF273129, 0xFF263028, 0xFF263028, 0xFF263028, 0xFF263028, 0xFF273129, 0xFF273129, 0xFF27302B, 0xFF252E29, 0xFF252E29, 0xFF262F2A, 0xFF252E29,
        0xFF242D28, 0xFF27302B, 0xFF2C3530, 0xFF222C24, 0xFF222C24, 0xFF222C24, 0xFF232D25, 0xFF232D25, 0xFF232D25, 0xFF242E26, 0xFF242E26, 0xFF252C25, 0xFF252C25, 0xFF252C25, 0xFF252B27, 0xFF262C28, 0xFF282E2A, 0xFF2A302E, 0xFF2B312F, 0xFF2A382B,
        0xFF262E21, 0xFF282E24, 0xFF27302B, 0xFF283430, 0xFF2D3526, 0xFF3C3711, 0xFF584711, 0xFF584606, 0xFF594707, 0xFF594707, 0xFF594506, 0xFF5B4708, 0xFF5F4A0B, 0xFF5E490A, 0xFF5A4506, 0xFF564500, 0xFF5A4808, 0xFF5A480A, 0xFF574803, 0xFF574900,
        0xFFA59951, 0xFF83743D, 0xFF6D5B33, 0xFF6A553A, 0xFF68552D, 0xFF635318, 0xFF9D8F46, 0xFFC8BA71, 0xFF99894E, 0xFF5D4A22, 0xFF725D42, 0xFF715935, 0xFF675031, 0xFF827146, 0xFF9E914D, 0xFF463A00, 0xFF604D12, 0xFF5E480F, 0xFF5A4300, 0xFF624605,
        0xFF4C410B, 0xFF8B6900, 0xFFE8AC28, 0xFF835605, 0xFF624B08, 0xFF676008, 0xFF808B3F, 0xFF302F2D, 0xFF000200, 0xFF040A06, 0xFF17221E, 0xFF202C28, 0xFF202C2A, 0xFF252F2E, 0xFF182120, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFFFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF252722, 0xFF232615, 0xFF050800, 0xFF090F0F, 0xFF13181E, 0xFF151A1D, 0xFF545A50, 0xFF7C8065, 0xFF7D7F58, 0xFF7F8368, 0xFF707054, 0xFF6F6C4D, 0xFF89815D, 0xFF81724B,
        0xFF594419, 0xFF53380B, 0xFF604415, 0xFF563C01, 0xFF594007, 0xFF4E3904, 0xFF5A4A17, 0xFF817648, 0xFF8C875D, 0xFF828059, 0xFF848460, 0xFF83835F, 0xFF878B68, 0xFF83845C, 0xFF6E5F26, 0xFF795404, 0xFFE1AB49, 0xFFD8A13A, 0xFF744000, 0xFF524206,
        0xFF5C4011, 0xFF5A3A0B, 0xFF513900, 0xFF473D08, 0xFFA8A88E, 0xFFDDDAD5, 0xFFB2ADA7, 0xFFD9E5DB, 0xFFB5C1B7, 0xFF263027, 0xFF111B12, 0xFF242B23, 0xFF1E251D, 0xFF20251E, 0xFF20251E, 0xFF192028, 0xFF14160B, 0xFF252000, 0xFF948949, 0xFFABA051,
        0xFFB8B05F, 0xFFDDDB90, 0xFF91944F, 0xFF634A00, 0xFFECB443, 0xFFC9951B, 0xFF5C3B00, 0xFF573E28, 0xFFCFDACC, 0xFFE0FBEC, 0xFFEFEEF3, 0xFFD0CFCA, 0xFFE5E8E1, 0xFF989F97, 0xFF28342A, 0xFF162218, 0xFF202A21, 0xFF1C231B, 0xFF1A1F18, 0xFF192813,
        0xFF1B2316, 0xFF161711, 0xFF2B2517, 0xFF342E0C, 0xFF484212, 0xFF6E6836, 0xFF524C1C, 0xFF453907, 0xFF6A4900, 0xFFECB852, 0xFFBC8F2A, 0xFF5B4500, 0xFF525212, 0xFF4D5407, 0xFF606201, 0xFF8D7020, 0xFF9B862B, 0xFF756A1A, 0xFF7B764E, 0xFF949488,
        0xFFB7BFB4, 0xFF212D23, 0xFF303C38, 0xFF35403C, 0xFF38433F, 0xFF37423E, 0xFF374341, 0xFF3B4745, 0xFF384646, 0xFF354343, 0xFF394747, 0xFF354441, 0xFF394744, 0xFF3B4743, 0xFF37433F, 0xFF35413D, 0xFF35413D, 0xFF36423E, 0xFF37433F, 0xFF36423E,
        0xFF384744, 0xFF394845, 0xFF364643, 0xFF384847, 0xFF3A4745, 0xFF404F4A, 0xFF3F4E48, 0xFF3A4946, 0xFF465655, 0xFF41504D, 0xFF3D4C49, 0xFF42514E, 0xFF404F4C, 0xFF42514E, 0xFF455451, 0xFF3F4E4B, 0xFF3E4C49, 0xFF3E4E4C, 0xFF3D4D4D, 0xFF3F4F4E,
        0xFF3F4E4E, 0xFF42524F, 0xFF43524F, 0xFF43524E, 0xFF3F4E4C, 0xFF445352, 0xFF40504E, 0xFF3B4A47, 0xFF414F4D, 0xFF425251, 0xFF415150, 0xFF425252, 0xFF415151, 0xFF3E4E4D, 0xFF41514F, 0xFF455451, 0xFF42514E, 0xFF41504D, 0xFF435250, 0xFF425251,
        0xFF40504E, 0xFF3D4948, 0xFF394342, 0xFF343E3D, 0xFF1F2928, 0xFF111B1A, 0xFF141E1D, 0xFF131416, 0xFF18191B, 0xFF151B1B, 0xFF0D1615, 0xFF202C2A, 0xFF384744, 0xFF364643, 0xFF334642, 0xFF354344, 0xFF1F2B1D, 0xFF7F868C, 0xFF869284, 0xFF6F7A5C,
        0xFF979066, 0xFF9C801C, 0xFFC29021, 0xFFA48028, 0xFF6B4F00, 0xFF503B00, 0xFF81680E, 0xFFC69F38, 0xFFBA8B17, 0xFFBF901A, 0xFFBA901A, 0xFF986F03, 0xFF644F00, 0xFF877B55, 0xFF4F423A, 0xFF76736E, 0xFF394634, 0xFF6E7B61, 0xFF423E23, 0xFF6F7760,
        0xFF3B4523, 0xFF757D56, 0xFF434B26, 0xFF666C50, 0xFF474B34, 0xFF656A4A, 0xFF484F25, 0xFF646442, 0xFF525635, 0xFF55593E, 0xFF5F674F, 0xFF4A523B, 0xFF5C5F4A, 0xFF4D4F37, 0xFF6C6A53, 0xFF434422, 0xFF6D7657, 0xFF364123, 0xFF747A54, 0xFF535121,
        0xFF7D7A47, 0xFF494C1D, 0xFF6C7850, 0xFF4E5028, 0xFF747555, 0xFF555927, 0xFF6F6F53, 0xFF555926, 0xFF6D6E4F, 0xFF5A5D30, 0xFF5C5E39, 0xFF61633C, 0xFF585A33, 0xFF686A43, 0xFF50522B, 0xFF6A6C45, 0xFF4F512A, 0xFF72744D, 0xFF50522B, 0xFF79714C,
        0xFF48491F, 0xFF707B5D, 0xFF2D382A, 0xFF707058, 0xFF715F1F, 0xFFC59E43, 0xFF9D6504, 0xFF564500, 0xFF554400, 0xFF584604, 0xFF574304, 0xFF5A4607, 0xFF5F490D, 0xFF594309, 0xFF5B450B, 0xFF554100, 0xFF5C460A, 0xFF59440B, 0xFF564406, 0xFF544300,
        0xFFA79952, 0xFF7B6C31, 0xFF776638, 0xFF776F24, 0xFF897D4B, 0xFF68592E, 0xFFA0934F, 0xFFBBAF5B, 0xFFA08E4C, 0xFF6C5631, 0xFF70563F, 0xFF73583B, 0xFF6D5338, 0xFF837045, 0xFFA3974D, 0xFF534800, 0xFF544504, 0xFF5B460D, 0xFF5B4605, 0xFF5E400A,
        0xFF504608, 0xFF8D6A04, 0xFFE9AB32, 0xFF855900, 0xFF644D09, 0xFF665D0E, 0xFF737F2B, 0xFF21231E, 0xFF000100, 0xFF090E0A, 0xFF191F1D, 0xFF1D2623, 0xFF1F2928, 0xFF222E2E, 0xFF111D1D, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFFFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF232520, 0xFF232517, 0xFF070A03, 0xFF0A1010, 0xFF151A1E, 0xFF151B19, 0xFF454938, 0xFF4C502D, 0xFF393C0D, 0xFF423F16, 0xFF403B11, 0xFF3E3709, 0xFF4C4010, 0xFF50400D,
        0xFF4C3702, 0xFF543D07, 0xFF5C430B, 0xFF583F06, 0xFF5E480F, 0xFF56410A, 0xFF4D3C08, 0xFF4E400F, 0xFF453A0C, 0xFF3D3409, 0xFF443D11, 0xFF40390F, 0xFF3A3610, 0xFF423F14, 0xFF4B3C03, 0xFF593700, 0xFF95670F, 0xFFDBAE51, 0xFF8C6309, 0xFF4A3A00,
        0xFF5A3E0F, 0xFF624213, 0xFF5D4509, 0xFF4A400B, 0xFF95957B, 0xFFABA8A3, 0xFF716C66, 0xFFCFDBD1, 0xFFAAB6AC, 0xFF1E281F, 0xFF0F1910, 0xFF232A22, 0xFF1D241C, 0xFF222720, 0xFF242922, 0xFF161E13, 0xFF252813, 0xFF1D1E00, 0xFF4F4D12, 0xFF615D16,
        0xFF56570B, 0xFF5D6117, 0xFF4C530D, 0xFF634B00, 0xFFE5B23C, 0xFFC69715, 0xFF5B3D00, 0xFF594220, 0xFFD9E3CB, 0xFFD8F4DD, 0xFFC6C7BF, 0xFF797873, 0xFFD1D4CD, 0xFFB9C0B8, 0xFF333F35, 0xFF0F1B11, 0xFF19231A, 0xFF232A22, 0xFF282D26, 0xFF242716,
        0xFF1A1803, 0xFF2E2504, 0xFF493D03, 0xFF786914, 0xFF796B0A, 0xFF6E5D01, 0xFF655300, 0xFF4F4D10, 0xFF665000, 0xFFE8BF56, 0xFFDCB558, 0xFF9A864B, 0xFF928E6B, 0xFF8B8865, 0xFF9E9769, 0xFF6C540C, 0xFF83731E, 0xFF7D752C, 0xFF87855F, 0xFF94968B,
        0xFFA2AA9F, 0xFF323C31, 0xFF2A352F, 0xFF35403C, 0xFF3B4642, 0xFF3A4541, 0xFF364240, 0xFF374341, 0xFF354343, 0xFF344242, 0xFF384646, 0xFF374643, 0xFF384643, 0xFF394642, 0xFF36423E, 0xFF34403C, 0xFF34403C, 0xFF35413D, 0xFF35413D, 0xFF34403C,
        0xFF33423F, 0xFF354441, 0xFF354442, 0xFF384948, 0xFF3C4947, 0xFF3C4A46, 0xFF3F4E48, 0xFF344341, 0xFF394947, 0xFF3A4946, 0xFF3A4945, 0xFF3D4C48, 0xFF3E4D48, 0xFF3F4E49, 0xFF42504C, 0xFF3D4C49, 0xFF404E4C, 0xFF425251, 0xFF40504F, 0xFF425252,
        0xFF384848, 0xFF3F4F4D, 0xFF42514E, 0xFF43524F, 0xFF3D4C49, 0xFF42514E, 0xFF3E4D4B, 0xFF3A4946, 0xFF42504D, 0xFF455554, 0xFF415150, 0xFF3C4C4C, 0xFF405050, 0xFF415151, 0xFF40504E, 0xFF455451, 0xFF42514E, 0xFF42514E, 0xFF445350, 0xFF42504F,
        0xFF42514E, 0xFF3C4947, 0xFF384241, 0xFF353F3E, 0xFF232D2C, 0xFF101A19, 0xFF0A1413, 0xFF18191B, 0xFF121617, 0xFF101616, 0xFF121B1A, 0xFF273331, 0xFF394845, 0xFF344441, 0xFF324541, 0xFF324041, 0xFF1F2B1D, 0xFF7B8386, 0xFF8E9989, 0xFF7B8467,
        0xFF9B9468, 0xFF9C801C, 0xFFB28011, 0xFFB89132, 0xFF94751B, 0xFF705802, 0xFF84670B, 0xFFC29930, 0xFFB88613, 0xFFBE8B16, 0xFFC09320, 0xFFA57918, 0xFF866D1D, 0xFF998852, 0xFF66532B, 0xFF7F7749, 0xFF5B662A, 0xFF828E4E, 0xFF545217, 0xFF6B6C4D,
        0xFF575931, 0xFF6C703E, 0xFF626337, 0xFF8C8C68, 0xFF555533, 0xFF64623B, 0xFF6A6A38, 0xFF67652C, 0xFF8A8953, 0xFF676938, 0xFF636639, 0xFF55582B, 0xFF797B4A, 0xFF5D5C26, 0xFF89874E, 0xFF5E5729, 0xFF6B6940, 0xFF50502A, 0xFF7E774B, 0xFF927E49,
        0xFFAA8F57, 0xFF907B46, 0xFFA69869, 0xFF988541, 0xFFAB975C, 0xFF958339, 0xFFA6915A, 0xFF99873B, 0xFFA48F58, 0xFF98863E, 0xFF9B8748, 0xFFA28F4D, 0xFFA18E4C, 0xFFA89553, 0xFFA18E4C, 0xFFA4914F, 0xFF978442, 0xFFA69351, 0xFF917E3C, 0xFFA98B4D,
        0xFF95823E, 0xFF948B52, 0xFF53502D, 0xFF817F59, 0xFF53490B, 0xFF846D21, 0xFFC4A25A, 0xFF5E4C0C, 0xFF564404, 0xFF584606, 0xFF574304, 0xFF574304, 0xFF5C4708, 0xFF594405, 0xFF5B4607, 0xFF5E4A05, 0xFF5E480C, 0xFF564108, 0xFF544204, 0xFF524100,
        0xFFA89A53, 0xFF8B7C41, 0xFF968557, 0xFFBEB66B, 0xFFC4B886, 0xFF796A3F, 0xFF998C48, 0xFFBFB35F, 0xFF9C8A48, 0xFF5D4722, 0xFF755B44, 0xFF72573A, 0xFF6D5338, 0xFF826F44, 0xFFA2964C, 0xFF534800, 0xFF544504, 0xFF5B460D, 0xFF5B4605, 0xFF5C3E08,
        0xFF4F4507, 0xFF8B6802, 0xFFE9AB32, 0xFF855900, 0xFF654E0A, 0xFF675E0F, 0xFF75812D, 0xFF262823, 0xFF000100, 0xFF090E0A, 0xFF181E1C, 0xFF1C2522, 0xFF1F2928, 0xFF232F2F, 0xFF142020, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFDFE, 0x00FFFEFF, 0x00FFFFFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF1F211C, 0xFF23261B, 0xFF070C06, 0xFF050A0D, 0xFF0F1417, 0xFF161B14, 0xFF4A4E33, 0xFF4C4F20, 0xFF3C4003, 0xFF4F4309, 0xFF55460B, 0xFF514207, 0xFF504005, 0xFF534005,
        0xFF574308, 0xFF5B4509, 0xFF564004, 0xFF543D07, 0xFF5A430D, 0xFF58430C, 0xFF56410C, 0xFF56420D, 0xFF54400D, 0xFF52410D, 0xFF584713, 0xFF594717, 0xFF4F4313, 0xFF504414, 0xFF53420A, 0xFF5A4100, 0xFF725207, 0xFFE1C478, 0xFF80661B, 0xFF4D3D00,
        0xFF593D0D, 0xFF5E3E0D, 0xFF584002, 0xFF473E07, 0xFF939379, 0xFFA29F98, 0xFF6C6761, 0xFFD0DCD2, 0xFFA8B4AA, 0xFF1F2920, 0xFF121C13, 0xFF252C24, 0xFF1B221A, 0xFF20251E, 0xFF252A23, 0xFF191D0E, 0xFF292E1A, 0xFF41452C, 0xFF727655, 0xFFA6AB82,
        0xFFBCC292, 0xFF949C69, 0xFF7B844F, 0xFF9E8C2A, 0xFFEBBC46, 0xFFD0A928, 0xFF957D1D, 0xFF907B4C, 0xFFE6EFD0, 0xFFCEE9CA, 0xFFB9BEA8, 0xFF666560, 0xFFCDD0C9, 0xFFBDC4BC, 0xFF253127, 0xFF101C12, 0xFF1F2920, 0xFF20271F, 0xFF171C15, 0xFF26252A,
        0xFF201D18, 0xFF534C32, 0xFF9B925B, 0xFFF4E99A, 0xFFF4E798, 0xFFF0E1A0, 0xFFEAD8AA, 0xFFA8AF85, 0xFF6E641F, 0xFF9E832A, 0xFF957828, 0xFF6D5D2C, 0xFF615B45, 0xFF5C5545, 0xFF776A57, 0xFFBCAC6E, 0xFF837C2E, 0xFF969552, 0xFF898A6A, 0xFF969992,
        0xFFA7AEA6, 0xFF2C3427, 0xFF363D36, 0xFF35403C, 0xFF3A4541, 0xFF38433F, 0xFF35413F, 0xFF3B4745, 0xFF3C4A4A, 0xFF384646, 0xFF354343, 0xFF384744, 0xFF374643, 0xFF374441, 0xFF34403C, 0xFF34403C, 0xFF34403C, 0xFF35413D, 0xFF34403C, 0xFF34403C,
        0xFF344340, 0xFF364542, 0xFF344340, 0xFF344444, 0xFF354543, 0xFF384643, 0xFF384643, 0xFF374643, 0xFF374643, 0xFF354441, 0xFF32413E, 0xFF364543, 0xFF3D4B48, 0xFF3E4C48, 0xFF3C4B48, 0xFF3A4946, 0xFF414F4D, 0xFF425251, 0xFF415150, 0xFF425252,
        0xFF415151, 0xFF3E4E4D, 0xFF41514F, 0xFF455451, 0xFF3D4C49, 0xFF404F4C, 0xFF3E4D4A, 0xFF394845, 0xFF3B4846, 0xFF425251, 0xFF3C4C4A, 0xFF3F4E4D, 0xFF435251, 0xFF415151, 0xFF3E4D4D, 0xFF435250, 0xFF41504E, 0xFF435250, 0xFF455554, 0xFF3B4B4B,
        0xFF3F4E4C, 0xFF3A4644, 0xFF35403F, 0xFF384241, 0xFF2D3736, 0xFF182221, 0xFF0B1514, 0xFF0F1515, 0xFF0A1010, 0xFF0E1716, 0xFF1C2625, 0xFF2F3B39, 0xFF364542, 0xFF32423F, 0xFF374744, 0xFF3C4A4B, 0xFF243120, 0xFF737B7E, 0xFF859080, 0xFF727B5E,
        0xFF948D61, 0xFFA1841E, 0xFFB58312, 0xFFB4881D, 0xFFB18E2A, 0xFF937618, 0xFF906E0D, 0xFFC79B30, 0xFFBF8A18, 0xFFC08B19, 0xFFC69326, 0xFFB28329, 0xFF866812, 0xFF927B2F, 0xFF71550C, 0xFF523F00, 0xFF595D09, 0xFF888E46, 0xFF4F4811, 0xFF52510E,
        0xFF464400, 0xFF4F4E00, 0xFF787725, 0xFF7D7930, 0xFF4B4602, 0xFF575105, 0xFF474300, 0xFF4A4000, 0xFF847C3B, 0xFF686228, 0xFF57541F, 0xFF47450C, 0xFF494404, 0xFF413B00, 0xFF8A8231, 0xFF716227, 0xFF584F14, 0xFF544C0E, 0xFF584600, 0xFF9B791F,
        0xFFA47911, 0xFFA77D11, 0xFFA27B12, 0xFFA67B15, 0xFFA6791E, 0xFFA3790D, 0xFFA47620, 0xFFA57B0D, 0xFF9F7119, 0xFFA1760D, 0xFFA77A1C, 0xFFA17512, 0xFFA37714, 0xFFA27613, 0xFFAC801D, 0xFF9D710E, 0xFFA57916, 0xFFA47815, 0xFFA07411, 0xFFAD7518,
        0xFFA47712, 0xFF8F7015, 0xFF584503, 0xFF564D14, 0xFF403C00, 0xFF5B561C, 0xFFAEA57A, 0xFF67541A, 0xFF564308, 0xFF574409, 0xFF584407, 0xFF554200, 0xFF594401, 0xFF594401, 0xFF5A4601, 0xFF584400, 0xFF594307, 0xFF574209, 0xFF594709, 0xFF524100,
        0xFF9C8E47, 0xFF796A2F, 0xFF877648, 0xFF9E964B, 0xFF8F8351, 0xFF635429, 0xFFA19450, 0xFFBEB25E, 0xFFA59351, 0xFF6D5732, 0xFF6A5039, 0xFF72573A, 0xFF6C5237, 0xFF816E43, 0xFFA1954B, 0xFF524700, 0xFF544504, 0xFF5A450C, 0xFF5A4504, 0xFF5B3D07,
        0xFF4D4305, 0xFF8A6701, 0xFFE9AB32, 0xFF865A01, 0xFF654E0A, 0xFF685F10, 0xFF76822E, 0xFF2B2D28, 0xFF010300, 0xFF090E0A, 0xFF171D1B, 0xFF1B2421, 0xFF1E2827, 0xFF232F2F, 0xFF152121, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFDFE, 0x00FFFEFF, 0x00FFFFFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF1B1D18, 0xFF171A13, 0xFF060A09, 0xFF090E11, 0xFF111717, 0xFF181E10, 0xFF4E522F, 0xFF4E5217, 0xFF4A4C01, 0xFF563E00, 0xFF563E00, 0xFF563E00, 0xFF584206, 0xFF5A440A,
        0xFF59430A, 0xFF574108, 0xFF553F06, 0xFF58430E, 0xFF543F08, 0xFF533E07, 0xFF553F06, 0xFF594007, 0xFF5B4008, 0xFF593F04, 0xFF563C01, 0xFF583F07, 0xFF58410B, 0xFF57420B, 0xFF564309, 0xFF5F4B10, 0xFF6E5C1E, 0xFFC6B678, 0xFF6E6023, 0xFF4E3E00,
        0xFF573B0B, 0xFF5B3B0A, 0xFF573F01, 0xFF4A410A, 0xFF99997D, 0xFFA09D96, 0xFF716D64, 0xFFD2DED4, 0xFFA7B3A9, 0xFF1F2920, 0xFF151F16, 0xFF242B23, 0xFF181F17, 0xFF1E231C, 0xFF222720, 0xFF1B1C17, 0xFF595B58, 0xFFD4D8D7, 0xFFE5EBE9, 0xFFBDC4BD,
        0xFFE1E9DC, 0xFFEAF2DD, 0xFFEFF5DB, 0xFFE4D578, 0xFFFBD269, 0xFFF2D160, 0xFFEEDB7F, 0xFFDDC899, 0xFFF9FDE2, 0xFFC1D9BF, 0xFFB2B7A1, 0xFF73726D, 0xFFCCCFC8, 0xFFB7BEB6, 0xFF1C281E, 0xFF162218, 0xFF1F2920, 0xFF222921, 0xFF222720, 0xFF1A2622,
        0xFF192420, 0xFF4E554D, 0xFFB5BAA4, 0xFFE3EACB, 0xFFA8B098, 0xFFDAE0DC, 0xFFD7DDED, 0xFFDBE4DF, 0xFF64603D, 0xFF513F00, 0xFF513F00, 0xFF4B3E07, 0xFF3F3A12, 0xFF3C340D, 0xFF55481B, 0xFF968B55, 0xFF4B4803, 0xFF76773B, 0xFF74785D, 0xFF919690,
        0xFFB4BBB3, 0xFF1D2518, 0xFF383F37, 0xFF38433F, 0xFF333E3A, 0xFF222D29, 0xFF1A2624, 0xFF283432, 0xFF374545, 0xFF394747, 0xFF354343, 0xFF384744, 0xFF364542, 0xFF364441, 0xFF34413D, 0xFF333F3B, 0xFF34403C, 0xFF35413D, 0xFF35413D, 0xFF34403C,
        0xFF32413E, 0xFF374643, 0xFF364542, 0xFF374747, 0xFF364645, 0xFF394846, 0xFF364542, 0xFF2F413D, 0xFF394947, 0xFF30403E, 0xFF384846, 0xFF3C4C4A, 0xFF3F4E4C, 0xFF41504F, 0xFF3F4E4C, 0xFF3A4946, 0xFF42504D, 0xFF455554, 0xFF425251, 0xFF3D4D4D,
        0xFF40504F, 0xFF40504F, 0xFF40504E, 0xFF455451, 0xFF3E4D4A, 0xFF414F4D, 0xFF435351, 0xFF404F4D, 0xFF414E4B, 0xFF3C4B49, 0xFF354443, 0xFF3A4947, 0xFF414F4D, 0xFF425150, 0xFF41514F, 0xFF425250, 0xFF445451, 0xFF43524F, 0xFF425150, 0xFF415151,
        0xFF3D4D4C, 0xFF384543, 0xFF35413F, 0xFF3A4644, 0xFF384442, 0xFF2B3735, 0xFF1D2927, 0xFF121B1A, 0xFF151E1D, 0xFF202A29, 0xFF2E3A38, 0xFF374341, 0xFF344340, 0xFF344340, 0xFF3A4A47, 0xFF323E3E, 0xFF1F2A1A, 0xFF7E8387, 0xFF909B8B, 0xFF788162,
        0xFF978E63, 0xFF9D801A, 0xFFB17D0D, 0xFFAA7805, 0xFFB88F23, 0xFFA5831F, 0xFF9B7510, 0xFFCA9A2C, 0xFFC58C1B, 0xFFC58B1D, 0xFFC89329, 0xFF9B6E11, 0xFF937419, 0xFF9C8126, 0xFF9A761E, 0xFF886C18, 0xFF8A823A, 0xFF948F65, 0xFF847363, 0xFFA39767,
        0xFF8B8146, 0xFF817837, 0xFFA99D60, 0xFF988B56, 0xFF938352, 0xFFA89960, 0xFF857834, 0xFF887938, 0xFF9E8F54, 0xFFA69A66, 0xFF9C9261, 0xFFA99F6C, 0xFF83793B, 0xFF85792D, 0xFF9D913D, 0xFFA89682, 0xFF8D806D, 0xFF8C8165, 0xFF644E1C, 0xFF9A7324,
        0xFFA6720D, 0xFFC08817, 0xFFB5820D, 0xFFC28B16, 0xFFBC8417, 0xFFC79115, 0xFFBB821B, 0xFFC38D0F, 0xFFB67D14, 0xFFC18B11, 0xFFC99120, 0xFFBF8814, 0xFFBA830F, 0xFFB57E0A, 0xFFBE8713, 0xFFB37C08, 0xFFC38C18, 0xFFC08915, 0xFFC58E1A, 0xFFC18218,
        0xFFC08914, 0xFFB68C1C, 0xFF8D6F17, 0xFF534100, 0xFF504B07, 0xFF464612, 0xFF676849, 0xFF6A5621, 0xFF55420A, 0xFF57440A, 0xFF5B470A, 0xFF544100, 0xFF574300, 0xFF594500, 0xFF594500, 0xFF594500, 0xFF594307, 0xFF564108, 0xFF574507, 0xFF514000,
        0xFF9E9049, 0xFF7C6D32, 0xFF8A794B, 0xFFAAA257, 0xFFA69A68, 0xFF67582D, 0xFF938642, 0xFFBCB05C, 0xFFA2904E, 0xFF644E29, 0xFF765C45, 0xFF72573A, 0xFF6B5136, 0xFF806D42, 0xFFA0944A, 0xFF524700, 0xFF544504, 0xFF59440B, 0xFF584302, 0xFF5C3E08,
        0xFF4D4305, 0xFF8A6701, 0xFFE9AB32, 0xFF865A01, 0xFF634C08, 0xFF665D0E, 0xFF76822E, 0xFF2D2F2A, 0xFF020400, 0xFF090E0A, 0xFF171D1B, 0xFF1B2421, 0xFF1C2625, 0xFF212D2D, 0xFF142020, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFDFE, 0x00FFFEFF, 0x00FFFFFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF191B16, 0xFF101511, 0xFF070B0C, 0xFF0E1316, 0xFF151B19, 0xFF191D0C, 0xFF50522A, 0xFF4A4A0A, 0xFF494800, 0xFF62450D, 0xFF5B3E06, 0xFF5A3C06, 0xFF5A3F0A, 0xFF5A400D,
        0xFF593F0E, 0xFF573F0F, 0xFF5B4315, 0xFF57430E, 0xFF58430C, 0xFF5C460D, 0xFF5E450C, 0xFF5B4106, 0xFF5C3F03, 0xFF604204, 0xFF604204, 0xFF5B3E04, 0xFF5A3D01, 0xFF5B3E04, 0xFF5A440A, 0xFF4F3E08, 0xFF493E08, 0xFF635723, 0xFF493E08, 0xFF514103,
        0xFF593D0D, 0xFF5F3F0C, 0xFF5B4303, 0xFF4D440B, 0xFF9B9B7F, 0xFF96938A, 0xFF6E6A61, 0xFFD1DDD3, 0xFFA4B0A6, 0xFF1D271E, 0xFF141E15, 0xFF232A22, 0xFF181F17, 0xFF1F241D, 0xFF222720, 0xFF0F100B, 0xFF4E504F, 0xFFD9DDDE, 0xFFD2D8D8, 0xFF777D79,
        0xFF91978B, 0xFFAFB29F, 0xFFC2C4AE, 0xFFC1B465, 0xFFCEA959, 0xFFCCB158, 0xFFCCBC6F, 0xFFBDA77E, 0xFFF0EEE1, 0xFFC4D8CC, 0xFFACB2A4, 0xFF74736E, 0xFFC7CAC3, 0xFFADB4AC, 0xFF1C281E, 0xFF18241A, 0xFF162017, 0xFF181F17, 0xFF1E231C, 0xFF1C2C0F,
        0xFF101D0B, 0xFF3D4840, 0xFFB2BDB5, 0xFFC8D6C7, 0xFF657767, 0xFF99B0A8, 0xFF98B1B5, 0xFFB8BEBA, 0xFF5A593A, 0xFF493F01, 0xFF534700, 0xFF5B510C, 0xFF52490E, 0xFF4D4305, 0xFF574A04, 0xFF463A08, 0xFF413D00, 0xFF5D5E25, 0xFF70745B, 0xFF8A8F8B,
        0xFFB2B9B1, 0xFF272F22, 0xFF3B423A, 0xFF36413D, 0xFF323D39, 0xFF232E2A, 0xFF1C2826, 0xFF2B3735, 0xFF384646, 0xFF394747, 0xFF344242, 0xFF374643, 0xFF364542, 0xFF354441, 0xFF364440, 0xFF35443F, 0xFF33423D, 0xFF334340, 0xFF364643, 0xFF384845,
        0xFF384845, 0xFF364643, 0xFF384845, 0xFF364947, 0xFF344745, 0xFF394B49, 0xFF364642, 0xFF3B4B47, 0xFF3B4B48, 0xFF3B4B47, 0xFF3C4B49, 0xFF42514F, 0xFF455352, 0xFF435351, 0xFF42514E, 0xFF3A4946, 0xFF3B4846, 0xFF425251, 0xFF415150, 0xFF435352,
        0xFF3E4D4B, 0xFF394947, 0xFF3A4948, 0xFF42514F, 0xFF3D4C4A, 0xFF404E4D, 0xFF445453, 0xFF445351, 0xFF3C4946, 0xFF354441, 0xFF33423F, 0xFF374643, 0xFF3E4C4A, 0xFF445453, 0xFF445453, 0xFF445453, 0xFF425250, 0xFF40504E, 0xFF3D4D4D, 0xFF404F4F,
        0xFF3D4B4A, 0xFF384442, 0xFF35413F, 0xFF374341, 0xFF3C4846, 0xFF394543, 0xFF313D3B, 0xFF2D3937, 0xFF303C3A, 0xFF364240, 0xFF3B4745, 0xFF3A4644, 0xFF354441, 0xFF374643, 0xFF374643, 0xFF354141, 0xFF1E2918, 0xFF84898C, 0xFF959E8D, 0xFF7B8263,
        0xFF9E9668, 0xFFA2851F, 0xFFBA8714, 0xFFB7830D, 0xFFB98D20, 0xFFA9861E, 0xFF9C750E, 0xFFC39023, 0xFFBF8613, 0xFFC48A1C, 0xFFC79228, 0xFF926812, 0xFF755A01, 0xFF8C7319, 0xFF95731A, 0xFF7A6009, 0xFF5A540A, 0xFF625D33, 0xFF604E42, 0xFF5F5524,
        0xFF5E551C, 0xFF564D0C, 0xFF5E5416, 0xFF70632F, 0xFF716332, 0xFF716229, 0xFF5E500F, 0xFF65571A, 0xFF62551E, 0xFF6C622D, 0xFF524B17, 0xFF655E28, 0xFF645C1D, 0xFF71681D, 0xFF655A0A, 0xFF69613C, 0xFF585834, 0xFF51532C, 0xFF4D4610, 0xFF785E13,
        0xFFA77F20, 0xFFB2831B, 0xFFA87C11, 0xFFB08213, 0xFFB0801C, 0xFFB58813, 0xFFA97819, 0xFFB1840D, 0xFFAC7C1A, 0xFFB08310, 0xFFB2831B, 0xFFBE8F23, 0xFFB08115, 0xFFAE7F13, 0xFFA97A0E, 0xFFAE7F13, 0xFFB08115, 0xFFB48519, 0xFFB18216, 0xFFAE7E1C,
        0xFFAC8111, 0xFFAA8618, 0xFF95761C, 0xFF5E4700, 0xFF524400, 0xFF493F04, 0xFF7F7551, 0xFF66521D, 0xFF523F07, 0xFF57440A, 0xFF5C480B, 0xFF564301, 0xFF584400, 0xFF594500, 0xFF594500, 0xFF5A4601, 0xFF5B4509, 0xFF554007, 0xFF534103, 0xFF514000,
        0xFFA59750, 0xFF837439, 0xFF8C7B4D, 0xFFA59D52, 0xFFAEA270, 0xFF7A6B40, 0xFF9F924E, 0xFFC0B460, 0xFFA49250, 0xFF624C27, 0xFF725841, 0xFF72573A, 0xFF6B5136, 0xFF7F6C41, 0xFF9F9349, 0xFF524700, 0xFF544504, 0xFF59440B, 0xFF584302, 0xFF5E400A,
        0xFF4E4406, 0xFF8A6701, 0xFFEAAC33, 0xFF865A01, 0xFF624B07, 0xFF645B0C, 0xFF75812D, 0xFF2D2F2A, 0xFF000200, 0xFF070C08, 0xFF171D1B, 0xFF1B2421, 0xFF1B2524, 0xFF1F2B2B, 0xFF111D1D, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFDFE, 0x00FFFEFF, 0x00FFFFFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF191B16, 0xFF161B17, 0xFF060C0C, 0xFF090E12, 0xFF111514, 0xFF181A0C, 0xFF595933, 0xFF525111, 0xFF514E00, 0xFF5D3F0D, 0xFF583C0A, 0xFF5A3E0C, 0xFF563A0A, 0xFF583C0D,
        0xFF5D4215, 0xFF5C4114, 0xFF5E4316, 0xFF554009, 0xFF56410A, 0xFF59430A, 0xFF5B420A, 0xFF593E06, 0xFF593F04, 0xFF5E4105, 0xFF5F4206, 0xFF5A3F07, 0xFF5C3E02, 0xFF5F4000, 0xFF61470C, 0xFF53420C, 0xFF534715, 0xFF473704, 0xFF59460E, 0xFF554605,
        0xFF5B3F0D, 0xFF5F3F0C, 0xFF584000, 0xFF483F06, 0xFF98997A, 0xFF8E8B82, 0xFF716D62, 0xFFD9E5DB, 0xFFABB7AD, 0xFF232D24, 0xFF182219, 0xFF262D25, 0xFF1D241C, 0xFF242922, 0xFF222720, 0xFF1F271A, 0xFF171E0E, 0xFF3A3E27, 0xFF3C3F20, 0xFF2E2E08,
        0xFF4C4A21, 0xFF484719, 0xFF4D4B1B, 0xFF534904, 0xFF623F05, 0xFF5F4805, 0xFF554B06, 0xFF614B26, 0xFFD7CFCC, 0xFFD2E5E3, 0xFFB9C1B6, 0xFF72716C, 0xFFC9CCC5, 0xFFA6ADA5, 0xFF212D23, 0xFF1C281E, 0xFF1F2920, 0xFF20271F, 0xFF1C211A, 0xFF25271C,
        0xFF151314, 0xFF282128, 0xFF5E5654, 0xFF76715D, 0xFF3A3C17, 0xFF495026, 0xFF4A5229, 0xFF61633B, 0xFF48460D, 0xFF534B00, 0xFF564B00, 0xFF564702, 0xFF514106, 0xFF58480D, 0xFF594707, 0xFF52400E, 0xFF544B0A, 0xFF636128, 0xFF787860, 0xFF929491,
        0xFFB5BCB5, 0xFF1F271A, 0xFF343B33, 0xFF303B37, 0xFF38433F, 0xFF3A4541, 0xFF394543, 0xFF404C4A, 0xFF3F4D4D, 0xFF384646, 0xFF334141, 0xFF374643, 0xFF374643, 0xFF374643, 0xFF374643, 0xFF374642, 0xFF354440, 0xFF344441, 0xFF354542, 0xFF364643,
        0xFF344441, 0xFF374744, 0xFF394946, 0xFF384B49, 0xFF394C4A, 0xFF3A4D4A, 0xFF384845, 0xFF3D4B48, 0xFF3F4E4A, 0xFF3F4E4B, 0xFF3E4D4B, 0xFF445352, 0xFF485654, 0xFF43524F, 0xFF435250, 0xFF3E4D4A, 0xFF3D4B48, 0xFF3C4B49, 0xFF3B4B49, 0xFF3C4B49,
        0xFF384744, 0xFF354441, 0xFF374643, 0xFF3A4946, 0xFF3A4A47, 0xFF3C4B49, 0xFF3D4C49, 0xFF3D4C49, 0xFF374643, 0xFF344441, 0xFF33423F, 0xFF354441, 0xFF374643, 0xFF3C4C4A, 0xFF41504F, 0xFF3F4D4C, 0xFF41514F, 0xFF415150, 0xFF425251, 0xFF3C4B49,
        0xFF364643, 0xFF364542, 0xFF33423F, 0xFF31403D, 0xFF364542, 0xFF394845, 0xFF364542, 0xFF3A4946, 0xFF394845, 0xFF344340, 0xFF344340, 0xFF32413E, 0xFF344340, 0xFF3B4745, 0xFF35413F, 0xFF374142, 0xFF1E2716, 0xFF7B7F82, 0xFF8C9584, 0xFF737A59,
        0xFF9B9063, 0xFFA88821, 0xFFBD8816, 0xFFC58E1A, 0xFFB78B20, 0xFFAB8520, 0xFFA07912, 0xFFC28F22, 0xFFBC8511, 0xFFC08B19, 0xFFC29224, 0xFF977128, 0xFF493500, 0xFF907F37, 0xFF9D873B, 0xFF827320, 0xFF535702, 0xFF828744, 0xFF665E2D, 0xFF626321,
        0xFF585A0F, 0xFF787A27, 0xFF626313, 0xFF8F8C47, 0xFF767330, 0xFF747227, 0xFF666411, 0xFF6A6428, 0xFF747033, 0xFF7F7F43, 0xFF6D6E32, 0xFF5B5D1E, 0xFF6D6E2C, 0xFF64611C, 0xFF716D24, 0xFF838929, 0xFF75832C, 0xFF43580B, 0xFF687428, 0xFF66610F,
        0xFFB09C45, 0xFFA58A33, 0xFFB39743, 0xFFA68B34, 0xFFB49749, 0xFFA2882B, 0xFFA78941, 0xFFA3892A, 0xFFB09249, 0xFFA1862B, 0xFFA48835, 0xFFB49844, 0xFFAB8F3B, 0xFFAD913D, 0xFFA28632, 0xFFB09440, 0xFFA68A36, 0xFFAD913D, 0xFFA18531, 0xFFB09649,
        0xFF9E862C, 0xFFAE9940, 0xFFA38B43, 0xFF846D2A, 0xFF614900, 0xFF614903, 0xFF886D38, 0xFF5E4B11, 0xFF523F04, 0xFF58450A, 0xFF5C480B, 0xFF574402, 0xFF5A4502, 0xFF594401, 0xFF594500, 0xFF554100, 0xFF594307, 0xFF574209, 0xFF564406, 0xFF524100,
        0xFFA1934C, 0xFF78692E, 0xFF7A693B, 0xFF686015, 0xFF716533, 0xFF625328, 0xFF9D904C, 0xFFB5A955, 0xFFA59351, 0xFF6B5530, 0xFF6B513A, 0xFF72573A, 0xFF6B5136, 0xFF7E6B40, 0xFF9F9349, 0xFF524700, 0xFF554605, 0xFF59440B, 0xFF574201, 0xFF5F410B,
        0xFF4D4305, 0xFF896600, 0xFFEAAC33, 0xFF865A01, 0xFF614A06, 0xFF635A0B, 0xFF74802C, 0xFF2D2F2A, 0xFF000100, 0xFF060B07, 0xFF181E1C, 0xFF1C2522, 0xFF1B2524, 0xFF1D2929, 0xFF0F1B1B, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFDFE, 0x00FFFEFF, 0x00FFFFFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF1A1C17, 0xFF141A16, 0xFF020808, 0xFF070C10, 0xFF131718, 0xFF181A0D, 0xFF5E5B3A, 0xFF524D13, 0xFF4B4400, 0xFF59420C, 0xFF57400A, 0xFF5D4610, 0xFF5B420C, 0xFF5A410B,
        0xFF5D420D, 0xFF583D08, 0xFF5C3E0A, 0xFF584209, 0xFF574108, 0xFF553F06, 0xFF553F06, 0xFF59430A, 0xFF59430A, 0xFF564007, 0xFF564007, 0xFF56410C, 0xFF5E4107, 0xFF624000, 0xFF5E4002, 0xFF56410C, 0xFF5F4D1B, 0xFF57400A, 0xFF614305, 0xFF514201,
        0xFF593D0B, 0xFF60400D, 0xFF573F00, 0xFF453C01, 0xFF959677, 0xFF868478, 0xFF726E63, 0xFFDCE8DE, 0xFFADB9AF, 0xFF242E25, 0xFF162017, 0xFF252C24, 0xFF212820, 0xFF282D26, 0xFF21261F, 0xFF111E15, 0xFF18210E, 0xFF232300, 0xFF403903, 0xFF483D00,
        0xFF4A4000, 0xFF453C00, 0xFF4E4608, 0xFF524904, 0xFF5D3F0B, 0xFF56460A, 0xFF4A4300, 0xFF59431A, 0xFFDBD2CD, 0xFFD3E3E2, 0xFFA5B09F, 0xFF72716C, 0xFFD7DAD3, 0xFFB0B7AF, 0xFF2F3B31, 0xFF162218, 0xFF19231A, 0xFF232A22, 0xFF222720, 0xFF1B1D29,
        0xFF1D1A25, 0xFF211718, 0xFF261600, 0xFF4B3504, 0xFF523B00, 0xFF573F00, 0xFF5B3F00, 0xFF4C4205, 0xFF4D4300, 0xFF594E00, 0xFF5A4901, 0xFF584206, 0xFF533906, 0xFF5C450F, 0xFF553F03, 0xFF604510, 0xFF4B3800, 0xFF675D20, 0xFF767257, 0xFF979893,
        0xFFAEB5AE, 0xFF232D22, 0xFF35403A, 0xFF323D39, 0xFF394440, 0xFF3B4642, 0xFF394543, 0xFF3B4745, 0xFF384646, 0xFF364444, 0xFF394747, 0xFF394845, 0xFF3A4946, 0xFF3A4946, 0xFF394845, 0xFF384744, 0xFF364541, 0xFF354542, 0xFF354542, 0xFF354542,
        0xFF354441, 0xFF394946, 0xFF374744, 0xFF374645, 0xFF344644, 0xFF314340, 0xFF344341, 0xFF3B4947, 0xFF3D4C49, 0xFF3B4A46, 0xFF3B4A48, 0xFF3F4D4D, 0xFF404F4D, 0xFF3A4847, 0xFF3C4B4A, 0xFF374643, 0xFF364542, 0xFF364542, 0xFF384744, 0xFF384744,
        0xFF374643, 0xFF364542, 0xFF364542, 0xFF384744, 0xFF394845, 0xFF394846, 0xFF3A4946, 0xFF3A4946, 0xFF384744, 0xFF364542, 0xFF364542, 0xFF344340, 0xFF354441, 0xFF374644, 0xFF3A4946, 0xFF3C4A47, 0xFF3B4A48, 0xFF3A4A48, 0xFF394845, 0xFF364542,
        0xFF33423F, 0xFF364542, 0xFF364542, 0xFF32413E, 0xFF344340, 0xFF374643, 0xFF344340, 0xFF344441, 0xFF32423F, 0xFF2D3D3A, 0xFF32413E, 0xFF31403D, 0xFF333F3D, 0xFF3B4745, 0xFF364240, 0xFF384241, 0xFF2A3322, 0xFF7C8081, 0xFF979E8C, 0xFF7A7F5F,
        0xFF968B5D, 0xFFB6962F, 0xFFC38E1A, 0xFFC28C20, 0xFFB48722, 0xFFA98525, 0xFFA27C19, 0xFFC49627, 0xFFBD8814, 0xFFBE8B16, 0xFFC39525, 0xFF896B23, 0xFF504407, 0xFFAAA774, 0xFF9B9365, 0xFFBFBE88, 0xFF6F7D3F, 0xFF97A266, 0xFF706C3C, 0xFFA5AB8F,
        0xFF666C46, 0xFFA0A879, 0xFF697046, 0xFF9B9F7E, 0xFF727556, 0xFFA1A37E, 0xFF818455, 0xFF909368, 0xFF7B8153, 0xFF88925E, 0xFF808D55, 0xFF77844C, 0xFF8E9864, 0xFF767C4E, 0xFFA5A77F, 0xFF797535, 0xFF9AA16D, 0xFF697952, 0xFFA9B897, 0xFF6D6F4A,
        0xFFB8AC84, 0xFF9C8A64, 0xFFBFAF8B, 0xFFA3975A, 0xFFC7BA85, 0xFF9F954E, 0xFFBCAE7D, 0xFFA1974F, 0xFFBEB17D, 0xFFA39856, 0xFFB4A86E, 0xFFACA064, 0xFFB2A66A, 0xFFB2A66A, 0xFFAEA266, 0xFFB6AA6E, 0xFFADA165, 0xFFB4A86C, 0xFFA79B5F, 0xFFC4BB82,
        0xFFA19A56, 0xFFC0B879, 0xFFA99B6C, 0xFFC1AD7B, 0xFFA68B46, 0xFFBC9A50, 0xFFB18846, 0xFF584606, 0xFF544202, 0xFF5A4808, 0xFF5A4607, 0xFF574304, 0xFF5A4506, 0xFF574203, 0xFF5A4506, 0xFF5A4601, 0xFF5A4408, 0xFF564108, 0xFF564406, 0xFF514000,
        0xFF9F914A, 0xFF7D6E33, 0xFF8A794B, 0xFFB4AC61, 0xFFB8AC7A, 0xFF796A3F, 0xFF938642, 0xFFB9AD59, 0xFFA18F4D, 0xFF5E4823, 0xFF7F654E, 0xFF73583B, 0xFF6B5136, 0xFF7E6B40, 0xFF9F9349, 0xFF534800, 0xFF554605, 0xFF5A450C, 0xFF574201, 0xFF5E400A,
        0xFF4C4204, 0xFF886500, 0xFFEAAC33, 0xFF865A01, 0xFF614A06, 0xFF635A0B, 0xFF76822E, 0xFF2F312C, 0xFF000100, 0xFF060B07, 0xFF181E1C, 0xFF1D2623, 0xFF1C2625, 0xFF1E2A2A, 0xFF111D1D, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFDFE, 0x00FFFEFF, 0x00FFFFFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF1B1D18, 0xFF171E17, 0xFF030909, 0xFF0A0F13, 0xFF15191C, 0xFF16180D, 0xFF615E3F, 0xFF59521C, 0xFF534A07, 0xFF57470B, 0xFF513E03, 0xFF574306, 0xFF594307, 0xFF5A4204,
        0xFF583F00, 0xFF563A00, 0xFF624404, 0xFF594007, 0xFF5E450C, 0xFF5A440B, 0xFF58410B, 0xFF59440D, 0xFF56420D, 0xFF54400D, 0xFF574612, 0xFF534310, 0xFF593E06, 0xFF623F00, 0xFF614104, 0xFF563F09, 0xFF523D08, 0xFF65460D, 0xFF674000, 0xFF554605,
        0xFF5C400E, 0xFF62430D, 0xFF594101, 0xFF4A4106, 0xFFA2A384, 0xFF9A988C, 0xFF918D82, 0xFFCFDBD1, 0xFFA1ADA3, 0xFF182219, 0xFF0C160D, 0xFF1E251D, 0xFF1F261E, 0xFF282D26, 0xFF1F241D, 0xFF101E27, 0xFF202720, 0xFF302B0B, 0xFF63541B, 0xFF69550E,
        0xFF634F08, 0xFF6A5818, 0xFF5C4D12, 0xFF504A00, 0xFF5C410C, 0xFF56470C, 0xFF444000, 0xFF554011, 0xFFD8CEC4, 0xFFBECFC7, 0xFF7C8870, 0xFF565550, 0xFFBCBFB8, 0xFF929991, 0xFF232F25, 0xFF0B170D, 0xFF131D14, 0xFF20271F, 0xFF1D221B, 0xFF142A1E,
        0xFF162510, 0xFF2A2C04, 0xFF665E15, 0xFFBBA744, 0xFFE0BF56, 0xFFD4A94B, 0xFFB8873A, 0xFF74602B, 0xFF614E13, 0xFF554200, 0xFF584304, 0xFF61460F, 0xFF5A3C06, 0xFF644A0D, 0xFF5A4400, 0xFF604007, 0xFF584000, 0xFF534508, 0xFF453E22, 0xFF51504C,
        0xFF585F58, 0xFF253127, 0xFF29362F, 0xFF343F3B, 0xFF37423E, 0xFF37423E, 0xFF384442, 0xFF3D4947, 0xFF3A4848, 0xFF394747, 0xFF3D4B4B, 0xFF3B4A47, 0xFF3D4C49, 0xFF3D4C49, 0xFF3B4A47, 0xFF374643, 0xFF344340, 0xFF344340, 0xFF344441, 0xFF374744,
        0xFF384744, 0xFF394845, 0xFF394845, 0xFF3A4947, 0xFF394946, 0xFF3A4946, 0xFF3B4A47, 0xFF384646, 0xFF3C4A49, 0xFF3B4948, 0xFF3F4D4D, 0xFF384646, 0xFF3B4949, 0xFF384646, 0xFF3B4949, 0xFF3B4A47, 0xFF3B4A47, 0xFF3B4A47, 0xFF3B4A47, 0xFF3A4946,
        0xFF394845, 0xFF374643, 0xFF364542, 0xFF394845, 0xFF3A4946, 0xFF3B4A47, 0xFF3B4A47, 0xFF3B4A47, 0xFF3A4946, 0xFF394845, 0xFF384744, 0xFF344340, 0xFF354441, 0xFF364542, 0xFF374643, 0xFF374643, 0xFF374643, 0xFF364542, 0xFF354441, 0xFF354441,
        0xFF32413E, 0xFF374643, 0xFF3B4A47, 0xFF374643, 0xFF374643, 0xFF384744, 0xFF33423F, 0xFF2F423E, 0xFF30433F, 0xFF344441, 0xFF3D4D4A, 0xFF374643, 0xFF313D3B, 0xFF3B4745, 0xFF394543, 0xFF333D3C, 0xFF222B18, 0xFF4E5253, 0xFF5E6553, 0xFF393E1E,
        0xFF5B5022, 0xFFAA8A23, 0xFFC8931F, 0xFFC18A23, 0xFFB58928, 0xFFA78428, 0xFF9C7817, 0xFFC29629, 0xFFBB8813, 0xFFB88913, 0xFFC29925, 0xFF8C7525, 0xFF423E00, 0xFFA1A584, 0xFF6D6E60, 0xFFA8AC9E, 0xFF5C6E58, 0xFFA1AD99, 0xFF686055, 0xFFA2AE9A,
        0xFF556246, 0xFFA2B08C, 0xFF5E6947, 0xFFA4AE96, 0xFF636C57, 0xFF9DA689, 0xFF67714C, 0xFF9EA593, 0xFF6D7961, 0xFF8D9C7B, 0xFF768B64, 0xFF7F936E, 0xFF86947A, 0xFF6C7767, 0xFF969B94, 0xFF7E6D51, 0xFFABA693, 0xFF616B60, 0xFFA5B1A3, 0xFF5B5F46,
        0xFFBAB58F, 0xFF989165, 0xFFBDB68A, 0xFF8D8757, 0xFFC3BB94, 0xFF918C54, 0xFFC0B894, 0xFF908B53, 0xFFB5AD89, 0xFF97915D, 0xFFBBB488, 0xFF9E9769, 0xFFB0A97B, 0xFFA8A173, 0xFFACA577, 0xFFA69F71, 0xFFA49D6F, 0xFFABA476, 0xFFA19A6C, 0xFFB1AF7F,
        0xFF989862, 0xFFB5B484, 0xFF8A8366, 0xFFCDBD9B, 0xFF977D40, 0xFF9B7628, 0xFF724300, 0xFF554400, 0xFF564500, 0xFF5B4907, 0xFF584405, 0xFF574304, 0xFF5A4408, 0xFF553F05, 0xFF5A440A, 0xFF5D4904, 0xFF594307, 0xFF564108, 0xFF594709, 0xFF503F00,
        0xFF9B8D46, 0xFF7E6F34, 0xFF958456, 0xFFB2AA5F, 0xFF9F9361, 0xFF6E5F34, 0xFF988B47, 0xFFB5A955, 0xFFA08E4C, 0xFF624C27, 0xFF715740, 0xFF74593C, 0xFF6B5136, 0xFF7E6B40, 0xFF9F9349, 0xFF534800, 0xFF564706, 0xFF5A450C, 0xFF574201, 0xFF5D3F09,
        0xFF4A4002, 0xFF876400, 0xFFEAAC33, 0xFF875B02, 0xFF614A06, 0xFF645B0C, 0xFF77832F, 0xFF31332E, 0xFF000200, 0xFF060B07, 0xFF181E1C, 0xFF1E2724, 0xFF1D2726, 0xFF202C2C, 0xFF131F1F, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFFFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF1A1C17, 0xFF151A16, 0xFF080B10, 0xFF091009, 0xFF0F140E, 0xFF1C1C1A, 0xFF5C5E2F, 0xFF595C05, 0xFF504C05, 0xFF534808, 0xFF4D4200, 0xFF564905, 0xFF5A4904, 0xFF675412,
        0xFF574203, 0xFF5A3F07, 0xFF5E430E, 0xFF5C4400, 0xFF5B4605, 0xFF544107, 0xFF50400D, 0xFF564613, 0xFF57430E, 0xFF553F03, 0xFF5B4202, 0xFF5B4008, 0xFF5B4008, 0xFF5B4008, 0xFF5B4008, 0xFF5C4109, 0xFF5C4109, 0xFF5C4109, 0xFF5C4109, 0xFF594100,
        0xFF594000, 0xFF5D440E, 0xFF574112, 0xFF543F14, 0xFFAA976D, 0xFFFFF0C2, 0xFFDBCB9A, 0xFF827F60, 0xFF575C48, 0xFF17221A, 0xFF171E17, 0xFF17140B, 0xFF1B170C, 0xFF141913, 0xFF101F22, 0xFF22240E, 0xFF3A422B, 0xFF84917D, 0xFFBFCEBB, 0xFFAAB3A0,
        0xFFCECCB5, 0xFFE2D4B9, 0xFFD3BB9F, 0xFFD49627, 0xFFDD9B2A, 0xFFDC9D29, 0xFFB17D10, 0xFF997515, 0xFFAD8E3B, 0xFFAA8735, 0xFF9C741F, 0xFF9C6B0E, 0xFF916E08, 0xFF705C13, 0xFF251D12, 0xFF151628, 0xFF14201C, 0xFF071A00, 0xFF102005, 0xFF191917,
        0xFF17120C, 0xFF62584C, 0xFFA29584, 0xFFFFFCE8, 0xFFE2E1CD, 0xFFEAF6E2, 0xFFE6F9E6, 0xFFDEEAD2, 0xFF5E653B, 0xFF484204, 0xFF594604, 0xFF654B10, 0xFF553A03, 0xFF574105, 0xFF5A4903, 0xFF584E09, 0xFF534506, 0xFF4B3A04, 0xFF412C00, 0xFF2E1E00,
        0xFF211800, 0xFF201F03, 0xFF262A11, 0xFF2E3231, 0xFF303433, 0xFF333736, 0xFF343A3A, 0xFF343A3A, 0xFF323B3A, 0xFF323A3C, 0xFF323A3C, 0xFF353C35, 0xFF383E3E, 0xFF343B41, 0xFF303741, 0xFF2E3740, 0xFF323B40, 0xFF323C3B, 0xFF2E3B32, 0xFF2E3A3B,
        0xFF303B3D, 0xFF323D3F, 0xFF323C3D, 0xFF323C3D, 0xFF323C3B, 0xFF363F3E, 0xFF384140, 0xFF333937, 0xFF383C3B, 0xFF333534, 0xFF38383A, 0xFF363638, 0xFF323433, 0xFF393D3C, 0xFF303634, 0xFF2E393B, 0xFF2E383A, 0xFF2D3537, 0xFF3A3E41, 0xFF303437,
        0xFF35383F, 0xFF2D313C, 0xFF383E4A, 0xFF29363F, 0xFF324049, 0xFF303F46, 0xFF233539, 0xFF344646, 0xFF273B39, 0xFF273E38, 0xFF253C34, 0xFF2C353C, 0xFF2E373C, 0xFF333B3E, 0xFF34393C, 0xFF323A3C, 0xFF313C3E, 0xFF2E3B41, 0xFF303F46, 0xFF303C3A,
        0xFF303C3C, 0xFF2F3D3E, 0xFF2A383B, 0xFF2D3C43, 0xFF2B3C44, 0xFF2B3C46, 0xFF2A3B45, 0xFF2F383F, 0xFF323B42, 0xFF323B42, 0xFF2F383D, 0xFF2E373C, 0xFF303A3C, 0xFF303A3C, 0xFF2B3537, 0xFF232426, 0xFF1F200E, 0xFF1A1900, 0xFF372E00, 0xFF634E0B,
        0xFF694800, 0xFFB58A24, 0xFFC2911A, 0xFFBD8D1E, 0xFFB98923, 0xFFAD8023, 0xFFA37819, 0xFFBF942B, 0xFFB28415, 0xFFBB881D, 0xFFCA932F, 0xFF876412, 0xFF43372B, 0xFF4C575B, 0xFF485949, 0xFF4C5854, 0xFF47595D, 0xFF4B5C56, 0xFF484C4B, 0xFF5C6769,
        0xFF455052, 0xFF556161, 0xFF4A5656, 0xFF4D5957, 0xFF485452, 0xFF505C5A, 0xFF495551, 0xFF5B6566, 0xFF4F595A, 0xFF505A5B, 0xFF4F595A, 0xFF4E5859, 0xFF4D5758, 0xFF4A5455, 0xFF566061, 0xFF4F5A56, 0xFF535E5A, 0xFF4B5652, 0xFF515C58, 0xFF45504C,
        0xFF57625E, 0xFF4B5652, 0xFF5C6763, 0xFF485253, 0xFF576162, 0xFF4B5556, 0xFF5C6665, 0xFF495352, 0xFF596460, 0xFF4B5652, 0xFF58635F, 0xFF45504C, 0xFF545F5B, 0xFF535E5A, 0xFF545F5B, 0xFF535E5A, 0xFF535E5A, 0xFF5A6561, 0xFF505B57, 0xFF596861,
        0xFF51595C, 0xFF5D6266, 0xFF4E5953, 0xFF586963, 0xFF3A4646, 0xFF3A3927, 0xFF4A3D10, 0xFF5A480A, 0xFF5A480A, 0xFF5A4609, 0xFF594506, 0xFF594506, 0xFF5A4504, 0xFF5B4605, 0xFF5B4605, 0xFF584304, 0xFF5B450B, 0xFF5D480F, 0xFF534100, 0xFF534200,
        0xFF9D8F48, 0xFF75662F, 0xFF77653D, 0xFF9A944A, 0xFF8B8539, 0xFF5F531F, 0xFF918450, 0xFFB1A752, 0xFF9E9045, 0xFF634F2C, 0xFF6F573B, 0xFF71553D, 0xFF6C523B, 0xFF7C693F, 0xFF9D904A, 0xFF554A00, 0xFF574807, 0xFF58430A, 0xFF574201, 0xFF5F4304,
        0xFF4A4005, 0xFF845E0B, 0xFFECAF30, 0xFF865A03, 0xFF60480A, 0xFF676200, 0xFF6F7839, 0xFF2A2A20, 0xFF000300, 0xFF080E0C, 0xFF162021, 0xFF1B2628, 0xFF1A2626, 0xFF212B2A, 0xFF151E1B, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFFFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF1A1C17, 0xFF131814, 0xFF05080D, 0xFF091009, 0xFF131812, 0xFF1F1F1D, 0xFF616334, 0xFF61640D, 0xFF56520B, 0xFF484222, 0xFF433B17, 0xFF4C4016, 0xFF504211, 0xFF57460E,
        0xFF574209, 0xFF5C430A, 0xFF5A3F08, 0xFF594405, 0xFF56450F, 0xFF483C14, 0xFF373111, 0xFF383114, 0xFF473B15, 0xFF52400E, 0xFF584209, 0xFF5B4008, 0xFF5B4008, 0xFF5B4008, 0xFF5B4008, 0xFF5C4109, 0xFF5C4109, 0xFF5C4109, 0xFF5C4109, 0xFF61490D,
        0xFF5F460D, 0xFF583F07, 0xFF564007, 0xFF4A3500, 0xFF8A7830, 0xFFE3D282, 0xFFBEB058, 0xFF3E3804, 0xFF282C07, 0xFF0D1A06, 0xFF131F13, 0xFF191F15, 0xFF1B1E17, 0xFF151E1B, 0xFF101F22, 0xFF111610, 0xFF616A65, 0xFFEDFEF6, 0xFFE6FAEF, 0xFF95A797,
        0xFFC8D2BA, 0xFFD2D3B4, 0xFFCAC4A2, 0xFFD1AB40, 0xFFE1B037, 0xFFDDA216, 0xFFF5B724, 0xFFFBC12F, 0xFFEBB223, 0xFFFBBA2C, 0xFFFAB222, 0xFFFFB828, 0xFFF6B525, 0xFFBA8B23, 0xFF4A2C08, 0xFF231419, 0xFF1F1F21, 0xFF151C14, 0xFF151E1B, 0xFF0A1F16,
        0xFF0D1D13, 0xFF495146, 0xFFB9BCB3, 0xFFDFE2DB, 0xFF8C9793, 0xFFC1D5D6, 0xFFB1CACE, 0xFFD1DCBE, 0xFF65683B, 0xFF4D4709, 0xFF503D00, 0xFF5A4003, 0xFF5C4109, 0xFF5F490D, 0xFF564500, 0xFF4F4200, 0xFF4D3D00, 0xFF4C3901, 0xFF493405, 0xFF402E08,
        0xFF332606, 0xFF272205, 0xFF212105, 0xFF151C0C, 0xFF141B0B, 0xFF13190B, 0xFF121A0B, 0xFF121A0D, 0xFF101B0D, 0xFF0F1A0C, 0xFF0E180D, 0xFF0E140A, 0xFF0F140D, 0xFF121912, 0xFF121814, 0xFF0D1611, 0xFF0D170F, 0xFF101A11, 0xFF101C10, 0xFF142016,
        0xFF111D11, 0xFF0D190D, 0xFF0E1A0E, 0xFF121D0F, 0xFF131E10, 0xFF121A0B, 0xFF0E1607, 0xFF141A18, 0xFF151B17, 0xFF111612, 0xFF171916, 0xFF161815, 0xFF111612, 0xFF171D19, 0xFF101614, 0xFF0B180E, 0xFF0F1B11, 0xFF0E180D, 0xFF171F14, 0xFF10180D,
        0xFF121A0F, 0xFF131A12, 0xFF0E150E, 0xFF121C04, 0xFF111902, 0xFF151C0A, 0xFF0C1303, 0xFF14170C, 0xFF13160D, 0xFF10110B, 0xFF10110C, 0xFF0F1A12, 0xFF0F1910, 0xFF101A0F, 0xFF0C160B, 0xFF0B150A, 0xFF0A160A, 0xFF0A170D, 0xFF0B1B11, 0xFF15190B,
        0xFF14180A, 0xFF161A0C, 0xFF14180A, 0xFF121608, 0xFF14170C, 0xFF0E1106, 0xFF0E1106, 0xFF0C160D, 0xFF0F1910, 0xFF101A11, 0xFF101A11, 0xFF131D12, 0xFF131D12, 0xFF101B0D, 0xFF0E190B, 0xFF191911, 0xFF141600, 0xFF282900, 0xFF726D33, 0xFF907F3A,
        0xFF7F600E, 0xFFBC9129, 0xFFC08E15, 0xFFBA8F1E, 0xFFBA8E25, 0xFFAE7D1E, 0xFF996903, 0xFFC09021, 0xFFC3921C, 0xFFC18C1C, 0xFFC38D23, 0xFF7D5A08, 0xFF312519, 0xFF313C40, 0xFF2D3E2E, 0xFF303C38, 0xFF283A3E, 0xFF2F403A, 0xFF323635, 0xFF313D3D,
        0xFF232F2F, 0xFF313D3D, 0xFF2B3735, 0xFF313D3B, 0xFF27332F, 0xFF2D3935, 0xFF25312D, 0xFF313B3A, 0xFF283231, 0xFF2B3534, 0xFF2A3433, 0xFF2A3433, 0xFF2A3433, 0xFF273130, 0xFF2E3837, 0xFF2B3632, 0xFF2E3935, 0xFF293430, 0xFF2C3733, 0xFF242F2B,
        0xFF2C3733, 0xFF1F2A26, 0xFF293430, 0xFF222C2D, 0xFF2C3637, 0xFF222C2D, 0xFF2F3938, 0xFF242E2D, 0xFF2F3A36, 0xFF222D29, 0xFF2A3531, 0xFF25302C, 0xFF2B3632, 0xFF26312D, 0xFF27322E, 0xFF26312D, 0xFF232E2A, 0xFF2A3531, 0xFF26312D, 0xFF23322B,
        0xFF1D2528, 0xFF24292D, 0xFF1D2822, 0xFF293A34, 0xFF1F2B2B, 0xFF2B2A18, 0xFF473A0D, 0xFF584608, 0xFF584608, 0xFF594709, 0xFF5A4607, 0xFF5A4607, 0xFF5B4605, 0xFF5A4504, 0xFF594403, 0xFF564102, 0xFF5A440A, 0xFF5C470E, 0xFF513F00, 0xFF4D3C00,
        0xFF9A8C45, 0xFF83743D, 0xFF938159, 0xFFC9C27A, 0xFFB1AB5F, 0xFF766A38, 0xFF998C58, 0xFFB1A752, 0xFF9C8E45, 0xFF614D2A, 0xFF6E563C, 0xFF71553D, 0xFF6B513A, 0xFF7B683E, 0xFF9C8F49, 0xFF544900, 0xFF564706, 0xFF58430A, 0xFF584302, 0xFF5F4304,
        0xFF4A4005, 0xFF845E0B, 0xFFECAF30, 0xFF875B04, 0xFF60480A, 0xFF676200, 0xFF70793A, 0xFF2B2B21, 0xFF000200, 0xFF060C0A, 0xFF131D1E, 0xFF1A2527, 0xFF1D2929, 0xFF252F2E, 0xFF17201D, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFFFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF1A1C17, 0xFF161B17, 0xFF03060B, 0xFF091009, 0xFF141913, 0xFF1F1F1D, 0xFF666839, 0xFF6D7019, 0xFF615D16, 0xFF080808, 0xFF040400, 0xFF201B08, 0xFF483F1E, 0xFF4F4015,
        0xFF594512, 0xFF5B450C, 0xFF573D02, 0xFF58450B, 0xFF524519, 0xFF312F18, 0xFF040A00, 0xFF000400, 0xFF2A2813, 0xFF50421B, 0xFF56420D, 0xFF5B4008, 0xFF5B4008, 0xFF5B4008, 0xFF5B4008, 0xFF5C4109, 0xFF5C4109, 0xFF5C4109, 0xFF5C4109, 0xFF5B420C,
        0xFF5F460E, 0xFF553C04, 0xFF61490D, 0xFF574000, 0xFF7B6418, 0xFFC4AE59, 0xFFA8933A, 0xFF453700, 0xFF312E01, 0xFF1C260E, 0xFF101D14, 0xFF131C19, 0xFF151716, 0xFF1D1C1A, 0xFF191718, 0xFF1C1620, 0xFF34343C, 0xFF899193, 0xFF75847D, 0xFF475846,
        0xFF7F8C72, 0xFF6E7553, 0xFF5E6039, 0xFF594B0C, 0xFF624000, 0xFFB27A01, 0xFFFBB424, 0xFFFEB31C, 0xFFFEB01C, 0xFFFFB927, 0xFFFFB727, 0xFFFFB218, 0xFFFFB321, 0xFFBF8519, 0xFF4E2500, 0xFF230F08, 0xFF231F1E, 0xFF1B2423, 0xFF131E24, 0xFF061D17,
        0xFF14281F, 0xFF343E35, 0xFFACB0A2, 0xFFB4B6A8, 0xFF6F7666, 0xFFA3B09F, 0xFF7A8B7B, 0xFF969A75, 0xFF504F1F, 0xFF564C0F, 0xFF5C4907, 0xFF5A4204, 0xFF5D4509, 0xFF5D4809, 0xFF554301, 0xFF564402, 0xFF554104, 0xFF513B02, 0xFF4B3504, 0xFF473108,
        0xFF3F2F0B, 0xFF362B0B, 0xFF2E2808, 0xFF5C604F, 0xFF585C4B, 0xFF515848, 0xFF4E5545, 0xFF4D5345, 0xFF4C5445, 0xFF4C5445, 0xFF4B5346, 0xFF565752, 0xFF4D4F4A, 0xFF4F5249, 0xFF4E5146, 0xFF494F43, 0xFF474D43, 0xFF4B524B, 0xFF4A514A, 0xFF475244,
        0xFF475244, 0xFF4A5245, 0xFF4A5243, 0xFF4C5244, 0xFF4C5343, 0xFF4C5343, 0xFF4E5243, 0xFF454C45, 0xFF495048, 0xFF4D554A, 0xFF4A5247, 0xFF454D42, 0xFF3F473C, 0xFF464D45, 0xFF464D46, 0xFF454F44, 0xFF465045, 0xFF3E483D, 0xFF41493C, 0xFF444C3D,
        0xFF3C4234, 0xFF484F3F, 0xFF464D3D, 0xFF454A36, 0xFF3E4231, 0xFF474A39, 0xFF48483C, 0xFF47443B, 0xFF4E4B44, 0xFF48433F, 0xFF48433F, 0xFF444C3F, 0xFF434B3E, 0xFF444C3F, 0xFF3F4A3C, 0xFF3F4A3C, 0xFF3D4838, 0xFF414C3C, 0xFF404B3B, 0xFF43463F,
        0xFF43443C, 0xFF46483D, 0xFF46483B, 0xFF424335, 0xFF4A4B3B, 0xFF444331, 0xFF474634, 0xFF41493A, 0xFF424A3B, 0xFF3C4435, 0xFF3B4433, 0xFF414A39, 0xFF414A39, 0xFF424B38, 0xFF49523F, 0xFF211C08, 0xFF1F1E00, 0xFF636536, 0xFFDADA9E, 0xFFCABD77,
        0xFF91761F, 0xFFBB9125, 0xFFBD8A0D, 0xFFBC9423, 0xFFB0841B, 0xFFBB8A24, 0xFFB27E10, 0xFFC89218, 0xFFC48D0E, 0xFFC28A11, 0xFFC48C1D, 0xFF8A6715, 0xFF392D21, 0xFF323D41, 0xFF304131, 0xFF323E3A, 0xFF293B3F, 0xFF2E3F39, 0xFF373B3A, 0xFF323E3E,
        0xFF2F3B3B, 0xFF34403E, 0xFF313D3B, 0xFF3A4642, 0xFF2D3935, 0xFF334039, 0xFF2D3A33, 0xFF333E3A, 0xFF2F3A36, 0xFF303B37, 0xFF2F3A36, 0xFF303B37, 0xFF323D39, 0xFF2D3834, 0xFF2D3834, 0xFF2F3A36, 0xFF2F3A36, 0xFF2E3935, 0xFF2D3834, 0xFF2F3A36,
        0xFF323D39, 0xFF2B3632, 0xFF303B37, 0xFF2A3435, 0xFF2E3839, 0xFF283233, 0xFF2D3736, 0xFF2D3736, 0xFF313C38, 0xFF293430, 0xFF2D3834, 0xFF303B37, 0xFF2D3834, 0xFF28332F, 0xFF2B3632, 0xFF2B3632, 0xFF28332F, 0xFF293430, 0xFF293430, 0xFF293831,
        0xFF293134, 0xFF2A2F33, 0xFF28332D, 0xFF293A34, 0xFF253131, 0xFF2B2A18, 0xFF4A3D10, 0xFF544406, 0xFF574507, 0xFF594709, 0xFF5A4808, 0xFF5B4708, 0xFF5A4705, 0xFF594403, 0xFF584302, 0xFF574203, 0xFF584208, 0xFF5C470E, 0xFF5A4806, 0xFF564500,
        0xFF9A8C45, 0xFF7A6B34, 0xFF806E46, 0xFF989149, 0xFF8C863C, 0xFF605422, 0xFF8F8150, 0xFFB0A651, 0xFF9E9047, 0xFF624E2B, 0xFF6D553B, 0xFF71553D, 0xFF6B513A, 0xFF7A673D, 0xFF9A8D47, 0xFF524700, 0xFF554605, 0xFF574209, 0xFF584302, 0xFF5E4203,
        0xFF4A4005, 0xFF845E0B, 0xFFEBAE2F, 0xFF875B04, 0xFF60480A, 0xFF666100, 0xFF717A3B, 0xFF2E2E24, 0xFF000200, 0xFF060C0A, 0xFF121C1D, 0xFF192426, 0xFF1F2B2B, 0xFF252F2E, 0xFF161F1C, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FEFEFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF1B1D18, 0xFF1C211D, 0xFF05080D, 0xFF080F08, 0xFF111610, 0xFF1A1A18, 0xFF67693A, 0xFF757821, 0xFF6B6720, 0xFF060E10, 0xFF000200, 0xFF0E0F00, 0xFF4A4527, 0xFF4A3F12,
        0xFF54430F, 0xFF5C460D, 0xFF61470C, 0xFF524204, 0xFF50491B, 0xFF2D321C, 0xFF000500, 0xFF000502, 0xFF222713, 0xFF50491F, 0xFF514008, 0xFF5B4008, 0xFF5B4008, 0xFF5B4008, 0xFF5B4008, 0xFF5C4109, 0xFF5C4109, 0xFF5C4109, 0xFF5C4109, 0xFF564004,
        0xFF5E450C, 0xFF573E06, 0xFF624712, 0xFF563B06, 0xFF543602, 0xFF6B4B18, 0xFF543401, 0xFF4A3500, 0xFF362A04, 0xFF171904, 0xFF080D06, 0xFF151610, 0xFF1B0F01, 0xFF3A2009, 0xFF310E00, 0xFF3A1B09, 0xFF290E00, 0xFF251100, 0xFF302301, 0xFF352C01,
        0xFF4B4312, 0xFF443903, 0xFF443800, 0xFF4F4400, 0xFF957810, 0xFFE7B12A, 0xFFF9B81C, 0xFFFBBC21, 0xFFEAB124, 0xFFBF8809, 0xFFBF8710, 0xFFC88D25, 0xFFBD8B2C, 0xFF89641E, 0xFF382000, 0xFF1D1403, 0xFF181A0D, 0xFF121D0D, 0xFF091708, 0xFF141414,
        0xFF1C1711, 0xFF190D00, 0xFF58492C, 0xFF705D32, 0xFF706025, 0xFF877A36, 0xFF786F24, 0xFFA2A070, 0xFF59531F, 0xFF4C4004, 0xFF554102, 0xFF564100, 0xFF5C4406, 0xFF584304, 0xFF574304, 0xFF574201, 0xFF584304, 0xFF553F05, 0xFF4A3500, 0xFF432E00,
        0xFF402D03, 0xFF3C2D06, 0xFF372904, 0xFF45453B, 0xFF3F3F37, 0xFF36372F, 0xFF31322A, 0xFF30332C, 0xFF30332C, 0xFF30322D, 0xFF2D322C, 0xFF343031, 0xFF2E2A27, 0xFF313127, 0xFF313224, 0xFF353729, 0xFF323429, 0xFF33342F, 0xFF292B2A, 0xFF2B302A,
        0xFF2F312C, 0xFF30322D, 0xFF2F322B, 0xFF2E2F29, 0xFF2D2E26, 0xFF2E2E26, 0xFF2F2F27, 0xFF2F3325, 0xFF2D3424, 0xFF303926, 0xFF27331F, 0xFF2E3A26, 0xFF2E3724, 0xFF2E3525, 0xFF2E3224, 0xFF353732, 0xFF30352F, 0xFF303531, 0xFF293029, 0xFF3A3F38,
        0xFF33362B, 0xFF36382A, 0xFF303121, 0xFF313038, 0xFF313237, 0xFF313236, 0xFF333738, 0xFF2D3331, 0xFF313831, 0xFF263027, 0xFF263027, 0xFF31322C, 0xFF30332C, 0xFF323731, 0xFF313630, 0xFF313630, 0xFF2C2F26, 0xFF35372A, 0xFF303123, 0xFF2C3437,
        0xFF2A3234, 0xFF2B3433, 0xFF2E3430, 0xFF272E27, 0xFF333B30, 0xFF2F3529, 0xFF303628, 0xFF303129, 0xFF35362E, 0xFF35372C, 0xFF383A2F, 0xFF3E4035, 0xFF3B3D30, 0xFF3C3E31, 0xFF484A3D, 0xFF2D2705, 0xFF2A2700, 0xFF6A6A38, 0xFFC7C588, 0xFFA2964A,
        0xFF8F7516, 0xFFBD9222, 0xFFC69014, 0xFFA38115, 0xFF956E07, 0xFFC79731, 0xFFC89323, 0xFFC38C0D, 0xFFC78E09, 0xFFD19719, 0xFFC08811, 0xFF825F0D, 0xFF362A1E, 0xFF2E393D, 0xFF2E3F2F, 0xFF313D39, 0xFF2B3D41, 0xFF2C3D37, 0xFF333736, 0xFF353F3E,
        0xFF394342, 0xFF333E3A, 0xFF303B37, 0xFF3B4640, 0xFF303B35, 0xFF39443E, 0xFF37423A, 0xFF36413D, 0xFF36413D, 0xFF35403C, 0xFF323D39, 0xFF343F3B, 0xFF37423E, 0xFF323D39, 0xFF2D3834, 0xFF37433F, 0xFF34403C, 0xFF2F3B37, 0xFF27332F, 0xFF2E3A36,
        0xFF2C3834, 0xFF2A3632, 0xFF2C3834, 0xFF2D3738, 0xFF2D3738, 0xFF2C3637, 0xFF2A3433, 0xFF303A39, 0xFF2D3834, 0xFF2E3935, 0xFF343F3B, 0xFF323D37, 0xFF2F3A34, 0xFF2F3A34, 0xFF2F3A34, 0xFF313C36, 0xFF323D37, 0xFF2C3731, 0xFF29342E, 0xFF25342D,
        0xFF2C3437, 0xFF2B3034, 0xFF303B35, 0xFF263731, 0xFF263232, 0xFF232210, 0xFF433609, 0xFF544406, 0xFF554507, 0xFF584608, 0xFF5A4808, 0xFF5A4808, 0xFF5A4705, 0xFF594604, 0xFF584503, 0xFF5A4506, 0xFF564006, 0xFF554007, 0xFF564402, 0xFF554400,
        0xFF998B44, 0xFF7C6D36, 0xFF827048, 0xFFB3AC64, 0xFFA8A258, 0xFF726634, 0xFF918352, 0xFFA99E4C, 0xFF96883F, 0xFF5E4A29, 0xFF6F573D, 0xFF71553D, 0xFF6B513A, 0xFF79663C, 0xFF998C46, 0xFF514600, 0xFF544504, 0xFF574209, 0xFF584302, 0xFF5E4203,
        0xFF4B4106, 0xFF835D0A, 0xFFEBAE2F, 0xFF885C05, 0xFF5F4709, 0xFF656000, 0xFF747D3E, 0xFF2E2E24, 0xFF000200, 0xFF0A100E, 0xFF141E1F, 0xFF172224, 0xFF1D2929, 0xFF212B2A, 0xFF121B18, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FEFEFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF1B1D18, 0xFF1E231F, 0xFF06090E, 0xFF091009, 0xFF121711, 0xFF1A1A18, 0xFF666839, 0xFF73761F, 0xFF68641D, 0xFF212D15, 0xFF0E1700, 0xFF232702, 0xFF575525, 0xFF504410,
        0xFF544109, 0xFF553C04, 0xFF543901, 0xFF544600, 0xFF4E4710, 0xFF363D1B, 0xFF13220F, 0xFF091805, 0xFF2B3211, 0xFF4F4814, 0xFF544502, 0xFF5B4008, 0xFF5B4008, 0xFF5B4008, 0xFF5B4008, 0xFF5C4109, 0xFF5C4109, 0xFF5C4109, 0xFF5C4109, 0xFF5A4109,
        0xFF5C430B, 0xFF5A4109, 0xFF5D420B, 0xFF624712, 0xFF5F4311, 0xFF593D0E, 0xFF583C0D, 0xFF4F3907, 0xFF46370C, 0xFF2D2705, 0xFF252104, 0xFF342C08, 0xFF4A3100, 0xFF946418, 0xFFA5650B, 0xFFA4690F, 0xFFAF781B, 0xFFA2700F, 0xFFA97E18, 0xFFA57C13,
        0xFF9C750C, 0xFFA17A13, 0xFFA9821B, 0xFFB58815, 0xFFE9B02D, 0xFFFDB626, 0xFFFFBC2D, 0xFFD39F25, 0xFF7C5B00, 0xFF584200, 0xFF554106, 0xFF503D00, 0xFF493802, 0xFF3B2C01, 0xFF2F2600, 0xFF37310F, 0xFF2A2507, 0xFF252002, 0xFF211D00, 0xFF221100,
        0xFF2F1B03, 0xFF3C2304, 0xFF664A1B, 0xFF826426, 0xFF8F6F22, 0xFF896A10, 0xFF987A18, 0xFF9B9156, 0xFF695D23, 0xFF534308, 0xFF5E4A0D, 0xFF5D4807, 0xFF5C4704, 0xFF594604, 0xFF5D490C, 0xFF573F00, 0xFF5A4506, 0xFF5A4609, 0xFF544107, 0xFF4F3B06,
        0xFF493705, 0xFF433101, 0xFF3C2A00, 0xFF403B27, 0xFF393420, 0xFF322D1A, 0xFF2C2916, 0xFF2C2918, 0xFF2C2B19, 0xFF2C2B19, 0xFF2C2A1B, 0xFF312C18, 0xFF302C13, 0xFF322F10, 0xFF242400, 0xFF282804, 0xFF252606, 0xFF2E2E14, 0xFF2E2D18, 0xFF282917,
        0xFF272816, 0xFF262715, 0xFF292814, 0xFF2A2915, 0xFF2C2A15, 0xFF2B2914, 0xFF2A2813, 0xFF302F11, 0xFF292C0F, 0xFF272D13, 0xFF141E05, 0xFF252F16, 0xFF282E14, 0xFF292C0F, 0xFF39381A, 0xFF36311B, 0xFF22210C, 0xFF333623, 0xFF2A2E1D, 0xFF232613,
        0xFF21200B, 0xFF322E13, 0xFF393215, 0xFF33240F, 0xFF3E321C, 0xFF2C2610, 0xFF28260F, 0xFF282C15, 0xFF222C14, 0xFF212F18, 0xFF20321A, 0xFF2E2714, 0xFF2A2714, 0xFF2A2B1B, 0xFF262918, 0xFF282917, 0xFF24220B, 0xFF383115, 0xFF352A0C, 0xFF2C2C10,
        0xFF2D2D11, 0xFF303014, 0xFF2D3013, 0xFF292C0D, 0xFF2F3213, 0xFF2D3212, 0xFF222707, 0xFF2A2914, 0xFF2C2B16, 0xFF2A2914, 0xFF2B2B13, 0xFF2E2E16, 0xFF303016, 0xFF38381E, 0xFF49492F, 0xFF3C3406, 0xFF3D3506, 0xFF47420B, 0xFF615813, 0xFF655500,
        0xFF9B7E16, 0xFFBC911E, 0xFFC59119, 0xFF9D811D, 0xFF674600, 0xFF926B0E, 0xFFBA8E25, 0xFFCB9A21, 0xFFC28D09, 0xFFC89110, 0xFFC88E13, 0xFF7F5C0A, 0xFF3B2F23, 0xFF323D41, 0xFF2F4030, 0xFF323E3A, 0xFF324448, 0xFF2F403A, 0xFF343837, 0xFF3A4541,
        0xFF414C48, 0xFF3A453F, 0xFF38433D, 0xFF434E46, 0xFF3D4840, 0xFF465149, 0xFF475349, 0xFF3E4943, 0xFF404B45, 0xFF3D4842, 0xFF3B4640, 0xFF3D4842, 0xFF414C46, 0xFF414C46, 0xFF3A453F, 0xFF394541, 0xFF35413D, 0xFF2D3935, 0xFF27332F, 0xFF2E3A36,
        0xFF2B3733, 0xFF2D3935, 0xFF2D3935, 0xFF2E3839, 0xFF2F393A, 0xFF303A3B, 0xFF2B3534, 0xFF303A39, 0xFF2B3632, 0xFF343F3B, 0xFF3F4A46, 0xFF3A453F, 0xFF37423C, 0xFF3A453F, 0xFF333E38, 0xFF36413B, 0xFF3E4943, 0xFF323D37, 0xFF2B3630, 0xFF24332C,
        0xFF2C3437, 0xFF282D31, 0xFF2D3832, 0xFF22332D, 0xFF293535, 0xFF2A2917, 0xFF4D4013, 0xFF554507, 0xFF554507, 0xFF564608, 0xFF594707, 0xFF594707, 0xFF594705, 0xFF5A4705, 0xFF594604, 0xFF5B4607, 0xFF5A440A, 0xFF5A450C, 0xFF594705, 0xFF503F00,
        0xFF94863F, 0xFF83743D, 0xFF8D7B53, 0xFF99924C, 0xFF9B944C, 0xFF6E6232, 0xFF8F8150, 0xFFA89D4B, 0xFF958740, 0xFF5E4A29, 0xFF715941, 0xFF71553D, 0xFF6A5039, 0xFF79663C, 0xFF998C46, 0xFF514600, 0xFF544504, 0xFF574209, 0xFF574201, 0xFF5D4102,
        0xFF4B4106, 0xFF825C09, 0xFFEAAD2E, 0xFF885C05, 0xFF5E4608, 0xFF645F00, 0xFF767F40, 0xFF2E2E24, 0xFF000200, 0xFF0B110F, 0xFF172122, 0xFF172224, 0xFF1B2727, 0xFF1F2928, 0xFF121B18, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FDFDFB, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF1C1E19, 0xFF191E1A, 0xFF06090E, 0xFF0B120B, 0xFF141913, 0xFF1D1D1B, 0xFF636536, 0xFF696C15, 0xFF5C5811, 0xFF656937, 0xFF50531E, 0xFF4B4A11, 0xFF544B10, 0xFF4A3C00,
        0xFF544005, 0xFF5F440D, 0xFF5E400C, 0xFF564200, 0xFF514508, 0xFF545422, 0xFF585F36, 0xFF545B32, 0xFF4E4D1D, 0xFF51450B, 0xFF574200, 0xFF5B4008, 0xFF5B4008, 0xFF5B4008, 0xFF5B4008, 0xFF5C4109, 0xFF5C4109, 0xFF5C4109, 0xFF5C4109, 0xFF5D3E10,
        0xFF5B3F0D, 0xFF5B4209, 0xFF523D00, 0xFF5A4702, 0xFF5D4C04, 0xFF4F3E00, 0xFF544600, 0xFF4D3D00, 0xFF514207, 0xFF4E430B, 0xFF453E07, 0xFF3F3500, 0xFF5F4600, 0xFFBF8D1E, 0xFFF9B633, 0xFFFBB41E, 0xFFFCB71E, 0xFFFEBC20, 0xFFFFBE22, 0xFFFABB1E,
        0xFFFDC027, 0xFFF8BC28, 0xFFF1B725, 0xFFFFB62F, 0xFFFFAA21, 0xFFFFB12D, 0xFFE8A233, 0xFF8C5F0E, 0xFF4B3402, 0xFF4C3D1E, 0xFF52432C, 0xFF4F4C00, 0xFF473E05, 0xFF493D0D, 0xFF65561B, 0xFF796729, 0xFF573F11, 0xFF4F330C, 0xFF4F3206, 0xFF463B05,
        0xFF493C08, 0xFF493804, 0xFF533E09, 0xFF513800, 0xFF4E3200, 0xFF614200, 0xFF604200, 0xFF5F4E09, 0xFF625012, 0xFF503D03, 0xFF594709, 0xFF564300, 0xFF554200, 0xFF574402, 0xFF5C480D, 0xFF604808, 0xFF594604, 0xFF514201, 0xFF4E4001, 0xFF4F4104,
        0xFF524207, 0xFF503D05, 0xFF4C3702, 0xFF4D4312, 0xFF473F10, 0xFF433B0C, 0xFF41390A, 0xFF40390B, 0xFF413A0C, 0xFF3F3B0E, 0xFF3E3A0D, 0xFF3E3B08, 0xFF585520, 0xFF727037, 0xFF66652C, 0xFF605F26, 0xFF404107, 0xFF3C3C06, 0xFF42420E, 0xFF3C3E0D,
        0xFF3E400F, 0xFF414010, 0xFF3F3F0D, 0xFF3D3B0A, 0xFF3C3A07, 0xFF3E3C09, 0xFF423F0C, 0xFF3D3705, 0xFF545327, 0xFF707451, 0xFF5D6347, 0xFF5A6044, 0xFF3B3F1C, 0xFF3F3E12, 0xFF77713F, 0xFF978D4F, 0xFF49440A, 0xFF3C3C08, 0xFF454819, 0xFF3A3C0D,
        0xFF413D0D, 0xFF746A35, 0xFF9D8E57, 0xFF7C6610, 0xFF826D1A, 0xFF69570B, 0xFF473A00, 0xFF443B00, 0xFF363300, 0xFF39380C, 0xFF393A12, 0xFF473B09, 0xFF413908, 0xFF3F3C0F, 0xFF3B3A0E, 0xFF403E0E, 0xFF443A05, 0xFF6A5A1E, 0xFF6D5817, 0xFF755F0C,
        0xFF7B6414, 0xFF7B681B, 0xFF6F5E16, 0xFF6C5F19, 0xFF5F5615, 0xFF5F591B, 0xFF3D3700, 0xFF3C3C0A, 0xFF3B3B09, 0xFF3C3C0A, 0xFF3C3C08, 0xFF3B3B07, 0xFF3C3C06, 0xFF41410B, 0xFF464610, 0xFF493D09, 0xFF50430F, 0xFF483800, 0xFF594300, 0xFF886C0B,
        0xFFBC9627, 0xFFBD8E1C, 0xFFCA9626, 0xFFA0882E, 0xFF614800, 0xFF634A00, 0xFF8C6E16, 0xFFC3A135, 0xFFC0961D, 0xFFC28C10, 0xFFCC9016, 0xFF7E5B09, 0xFF3D3125, 0xFF303B3F, 0xFF2B3C2C, 0xFF2C3834, 0xFF2F4145, 0xFF2C3D37, 0xFF323635, 0xFF333E38,
        0xFF3C4741, 0xFF3C473F, 0xFF3C473F, 0xFF444F47, 0xFF424E44, 0xFF455147, 0xFF465248, 0xFF39443C, 0xFF3C473F, 0xFF38433B, 0xFF39443C, 0xFF3D4840, 0xFF404B43, 0xFF444F47, 0xFF3E4941, 0xFF2E3D38, 0xFF303F3A, 0xFF25342F, 0xFF263530, 0xFF2A3934,
        0xFF2A3934, 0xFF293833, 0xFF2A3934, 0xFF2B3536, 0xFF2D3738, 0xFF2D3738, 0xFF2B3534, 0xFF2C3635, 0xFF293430, 0xFF333E3A, 0xFF434E4A, 0xFF424D45, 0xFF3B463E, 0xFF3F4A42, 0xFF313C34, 0xFF323D35, 0xFF3F4A42, 0xFF2F3A32, 0xFF2A352D, 0xFF293831,
        0xFF2F373A, 0xFF2B3034, 0xFF2A352F, 0xFF21322C, 0xFF253131, 0xFF282715, 0xFF483B0E, 0xFF544607, 0xFF544607, 0xFF564608, 0xFF574807, 0xFF5A4808, 0xFF5A4806, 0xFF5A4806, 0xFF5B4806, 0xFF5C4708, 0xFF5D470D, 0xFF5C470E, 0xFF5E4C0A, 0xFF534200,
        0xFF93853E, 0xFF796A33, 0xFF76643C, 0xFF67601A, 0xFF777028, 0xFF5A4E1E, 0xFF887A4B, 0xFFABA050, 0xFF9C8E47, 0xFF614C2D, 0xFF715941, 0xFF71553D, 0xFF6A5039, 0xFF7A673D, 0xFF998C46, 0xFF514600, 0xFF544504, 0xFF574209, 0xFF574201, 0xFF5D4102,
        0xFF4B4106, 0xFF825C09, 0xFFEAAD2E, 0xFF895D06, 0xFF5D4507, 0xFF635E00, 0xFF798243, 0xFF35352B, 0xFF000200, 0xFF080E0C, 0xFF172122, 0xFF172224, 0xFF1C2828, 0xFF1F2928, 0xFF151E1B, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFF, 0x00FEFCFD, 0x00FDFDFB, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF1C1E19, 0xFF141915, 0xFF06090E, 0xFF0A110A, 0xFF121711, 0xFF1E1E1C, 0xFF5F6132, 0xFF5E610A, 0xFF524E07, 0xFF565016, 0xFF534A0F, 0xFF54480B, 0xFF504100, 0xFF544001,
        0xFF584004, 0xFF61430D, 0xFF5B3B08, 0xFF5D4306, 0xFF533F04, 0xFF4A3E04, 0xFF4C450E, 0xFF4F4812, 0xFF4C3F0A, 0xFF523D06, 0xFF60450E, 0xFF5B4008, 0xFF5B4008, 0xFF5B4008, 0xFF5B4008, 0xFF5C4109, 0xFF5C4109, 0xFF5C4109, 0xFF5C4109, 0xFF5F3E11,
        0xFF614311, 0xFF644B12, 0xFF594506, 0xFF534401, 0xFF564D0A, 0xFF524D0D, 0xFF4C480B, 0xFF564C0F, 0xFF3F3300, 0xFF4E4208, 0xFF56511A, 0xFF55541E, 0xFF726823, 0xFF88690E, 0xFFAE7B0E, 0xFFFDBA2E, 0xFFF9B526, 0xFFFEBB26, 0xFFFAB71E, 0xFFF9B71B,
        0xFFFEBD23, 0xFFF7B722, 0xFFF9BB28, 0xFFF8B318, 0xFFFFC32E, 0xFFE9AD2B, 0xFF835800, 0xFF513A00, 0xFF574C0A, 0xFF4D3F00, 0xFF594500, 0xFF5B4A00, 0xFF4E3900, 0xFF503905, 0xFF745E12, 0xFF836D1A, 0xFF584002, 0xFF543A09, 0xFF563F09, 0xFF493E00,
        0xFF4A3E01, 0xFF4E3F06, 0xFF4F4009, 0xFF56420D, 0xFF604C11, 0xFFA68E4E, 0xFF8A712E, 0xFF947E32, 0xFF9E8744, 0xFF5E4910, 0xFF554207, 0xFF554400, 0xFF564600, 0xFF574400, 0xFF523D04, 0xFF5B4806, 0xFF564402, 0xFF4E4400, 0xFF4E4703, 0xFF514A06,
        0xFF524707, 0xFF534103, 0xFF523E01, 0xFF504002, 0xFF4F3F03, 0xFF4E4003, 0xFF4E4003, 0xFF4C4004, 0xFF4C4004, 0xFF4B3F05, 0xFF4A4005, 0xFF4B450B, 0xFF78713A, 0xFFA59E68, 0xFFA19B67, 0xFF938D59, 0xFF504D16, 0xFF3A3800, 0xFF47450A, 0xFF4B4B0B,
        0xFF464606, 0xFF404100, 0xFF403F00, 0xFF434200, 0xFF464300, 0xFF444100, 0xFF413E00, 0xFF463C00, 0xFF5D571D, 0xFF838158, 0xFF7E8164, 0xFF86896C, 0xFF545229, 0xFF4D470D, 0xFF9D934C, 0xFFC7B75F, 0xFF857A28, 0xFF403C00, 0xFF474707, 0xFF4B490E,
        0xFF4A4106, 0xFF6D5B1D, 0xFF907838, 0xFF9C8D3C, 0xFF948436, 0xFF8D7F34, 0xFF534500, 0xFF514201, 0xFF4C3C00, 0xFF4D3E05, 0xFF4F4007, 0xFF544100, 0xFF4E3E00, 0xFF4C4308, 0xFF464006, 0xFF4F4709, 0xFF584702, 0xFF8B7124, 0xFF927321, 0xFF98710A,
        0xFFA17D1C, 0xFFA28328, 0xFF907524, 0xFF9A863F, 0xFF857738, 0xFF958C55, 0xFF605925, 0xFF4A480B, 0xFF3F3E00, 0xFF3C3B00, 0xFF3D3C00, 0xFF3F3E00, 0xFF484706, 0xFF4D4C09, 0xFF464502, 0xFF4C410B, 0xFF4F3E08, 0xFF604405, 0xFF916E1C, 0xFFBA9127,
        0xFFC49722, 0xFFC2901F, 0xFFBE8A24, 0xFF5E4D00, 0xFF71621D, 0xFF706427, 0xFF554802, 0xFF867317, 0xFFB49425, 0xFFCD9C25, 0xFFC3870F, 0xFF7E5B09, 0xFF3D3125, 0xFF303B3F, 0xFF2F4030, 0xFF2E3A36, 0xFF2F4145, 0xFF2A3B35, 0xFF373B3A, 0xFF333D35,
        0xFF39433B, 0xFF3F4941, 0xFF3D473E, 0xFF3F4940, 0xFF404A3F, 0xFF3C463B, 0xFF3D473C, 0xFF364238, 0xFF3A463C, 0xFF364238, 0xFF39453B, 0xFF3C483E, 0xFF3B473D, 0xFF414D43, 0xFF3D493F, 0xFF384742, 0xFF3C4B46, 0xFF273631, 0xFF2D3C37, 0xFF25342F,
        0xFF293833, 0xFF24332E, 0xFF273631, 0xFF2D3738, 0xFF2F393A, 0xFF2A3435, 0xFF2F3938, 0xFF2C3635, 0xFF2E3935, 0xFF313C38, 0xFF424D49, 0xFF434E46, 0xFF38433B, 0xFF414C44, 0xFF333E36, 0xFF343F37, 0xFF3F4A42, 0xFF2B362E, 0xFF2B362E, 0xFF26352E,
        0xFF2C3437, 0xFF2E3337, 0xFF29342E, 0xFF273832, 0xFF263232, 0xFF2B2A18, 0xFF46390C, 0xFF534506, 0xFF544607, 0xFF564809, 0xFF594A09, 0xFF5A4B0A, 0xFF5B4907, 0xFF5A4806, 0xFF594705, 0xFF5D4809, 0xFF594309, 0xFF523D04, 0xFF544200, 0xFF503F00,
        0xFF958740, 0xFF7B6C35, 0xFF6D5B33, 0xFFB6AF6B, 0xFFB3AC64, 0xFF786B3E, 0xFF8D7F50, 0xFFA69B4B, 0xFF978843, 0xFF5F4A2B, 0xFF705741, 0xFF70543C, 0xFF6A5039, 0xFF7A673D, 0xFF9B8E48, 0xFF534800, 0xFF554605, 0xFF574209, 0xFF564100, 0xFF5C4001,
        0xFF4B4106, 0xFF815B08, 0xFFE9AC2D, 0xFF8A5E07, 0xFF5D4507, 0xFF625D00, 0xFF7A8344, 0xFF4A4A40, 0xFF000200, 0xFF040A08, 0xFF131D1E, 0xFF172224, 0xFF1E2A2A, 0xFF1F2928, 0xFF17201D, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFF, 0x00FEFCFD, 0x00FDFDFB, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF1C1E19, 0xFF131814, 0xFF070A0F, 0xFF070E07, 0xFF0E130D, 0xFF1B1B19, 0xFF5B5D2E, 0xFF585B04, 0xFF4F4B04, 0xFF52410D, 0xFF523E09, 0xFF59440B, 0xFF543E02, 0xFF63490C,
        0xFF593C02, 0xFF5C3D06, 0xFF5A3B05, 0xFF5A3A0B, 0xFF5C410C, 0xFF553F06, 0xFF503D02, 0xFF574409, 0xFF5B440E, 0xFF5A3E0E, 0xFF5E3D12, 0xFF5B4008, 0xFF5B4008, 0xFF5B4008, 0xFF5B4008, 0xFF5C4109, 0xFF5C4109, 0xFF5C4109, 0xFF5C4109, 0xFF5F3E08,
        0xFF5E3F08, 0xFF594007, 0xFF523F07, 0xFF4F4112, 0xFF726A46, 0xFF9F9981, 0xFF9F9C8B, 0xFFB9AF7E, 0xFF685A2B, 0xFF685B31, 0xFF858062, 0xFFA7AB94, 0xFFC4C4A8, 0xFF746337, 0xFF593900, 0xFFA46808, 0xFFCD8F2A, 0xFFD79729, 0xFFD79620, 0xFFDA9A1E,
        0xFFD19115, 0xFFD29419, 0xFFDEA228, 0xFFD6AD37, 0xFFBD972C, 0xFF7C5E08, 0xFF524204, 0xFF474515, 0xFF494613, 0xFF5A4903, 0xFF5E3E00, 0xFF5D4000, 0xFF5E3C16, 0xFF6C4C23, 0xFF927C30, 0xFF988B2D, 0xFF5E550A, 0xFF4C4809, 0xFF444500, 0xFF59430A,
        0xFF4F3802, 0xFF59440D, 0xFF543F06, 0xFF584405, 0xFF543E00, 0xFF9E8835, 0xFF80670D, 0xFF7F6614, 0xFF977E3B, 0xFF543E05, 0xFF534005, 0xFF5A4903, 0xFF574700, 0xFF594803, 0xFF5A450C, 0xFF534000, 0xFF5A4D09, 0xFF685F1A, 0xFF736F28, 0xFF706C25,
        0xFF625914, 0xFF564704, 0xFF523F00, 0xFF503B02, 0xFF513C03, 0xFF533E07, 0xFF534008, 0xFF534008, 0xFF503F09, 0xFF4F3E08, 0xFF4E3D07, 0xFF4B4013, 0xFF60542C, 0xFF6F6240, 0xFF605539, 0xFF62573B, 0xFF3E3612, 0xFF585127, 0xFF918A5C, 0xFF747235,
        0xFF5D5B1E, 0xFF4C4A0D, 0xFF5C5819, 0xFF7C7839, 0xFF8E8949, 0xFF827D3D, 0xFF6E6929, 0xFF766714, 0xFF494000, 0xFF363306, 0xFF3C3C20, 0xFF7A7A5E, 0xFF656235, 0xFF625916, 0xFFBAAB58, 0xFF776207, 0xFFBBAA58, 0xFF857D35, 0xFF6B672A, 0xFF605923,
        0xFF4D3D09, 0xFF563D07, 0xFF6A4B14, 0xFF90927C, 0xFF7D7D63, 0xFF928F70, 0xFF4D451E, 0xFF4B4012, 0xFF51420B, 0xFF503D02, 0xFF523E00, 0xFF583F00, 0xFF523E03, 0xFF52430C, 0xFF4A3E0A, 0xFF514209, 0xFF554000, 0xFF8C6C23, 0xFF936C1D, 0xFF987019,
        0xFFA17C2B, 0xFF9B7A33, 0xFF7D642B, 0xFF8A794D, 0xFF6C6342, 0xFF84826B, 0xFF434432, 0xFF454008, 0xFF4D4810, 0xFF6E6931, 0xFF8D884E, 0xFF969157, 0xFF928E51, 0xFF777336, 0xFF4E4A0D, 0xFF524711, 0xFF523D06, 0xFF7B5919, 0xFFBB8F38, 0xFFC39122,
        0xFFBC8A11, 0xFFC29021, 0xFF865400, 0xFF86762B, 0xFFB9AF72, 0xFFC6C38E, 0xFF88884C, 0xFF7B7322, 0xFF8F760F, 0xFFC09320, 0xFFCA8D18, 0xFF755200, 0xFF33271B, 0xFF283337, 0xFF2F4030, 0xFF2E3A36, 0xFF2A3C40, 0xFF23342E, 0xFF343837, 0xFF3B453D,
        0xFF3B453D, 0xFF444E45, 0xFF3C463D, 0xFF384237, 0xFF3C463B, 0xFF343E33, 0xFF354032, 0xFF3B473D, 0xFF3F4B41, 0xFF3A463C, 0xFF3F4B41, 0xFF3E4A40, 0xFF39453B, 0xFF3F4B41, 0xFF3B473D, 0xFF3B4A45, 0xFF41504B, 0xFF25342F, 0xFF2E3D38, 0xFF21302B,
        0xFF2C3B36, 0xFF293833, 0xFF303F3A, 0xFF2F393A, 0xFF303A3B, 0xFF273132, 0xFF323C3B, 0xFF2E3837, 0xFF323D39, 0xFF2F3A36, 0xFF3E4945, 0xFF3D4840, 0xFF303B33, 0xFF3E4941, 0xFF38433B, 0xFF3A453D, 0xFF3F4A42, 0xFF28332B, 0xFF2C372F, 0xFF27362F,
        0xFF293134, 0xFF2C3135, 0xFF232E28, 0xFF263731, 0xFF243030, 0xFF2D2C1A, 0xFF473A0D, 0xFF524405, 0xFF544607, 0xFF584A0B, 0xFF5B4C0B, 0xFF5C4D0C, 0xFF5B4C09, 0xFF5A4806, 0xFF584604, 0xFF5A4506, 0xFF5D470D, 0xFF5A450C, 0xFF594705, 0xFF4F3E00,
        0xFF93853E, 0xFF7D6E37, 0xFF715F37, 0xFFB0A965, 0xFFAAA35B, 0xFF6B5E31, 0xFF807243, 0xFFA19646, 0xFF988944, 0xFF5F4A2B, 0xFF6F5640, 0xFF70543C, 0xFF6A5039, 0xFF7B683E, 0xFF9C8F49, 0xFF544900, 0xFF554605, 0xFF574209, 0xFF564100, 0xFF5C4001,
        0xFF4B4106, 0xFF815B08, 0xFFE9AC2D, 0xFF8A5E07, 0xFF5C4406, 0xFF625D00, 0xFF7B8445, 0xFF5D5D53, 0xFF020500, 0xFF010705, 0xFF101A1B, 0xFF162123, 0xFF1E2A2A, 0xFF1D2726, 0xFF151E1B, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFFFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF1B1D18, 0xFF111610, 0xFF070D01, 0xFF060C08, 0xFF13161F, 0xFF13140F, 0xFF54552D, 0xFF565212, 0xFF534C08, 0xFF56450D, 0xFF564309, 0xFF544003, 0xFF543F00, 0xFF5B4303,
        0xFF574201, 0xFF523F00, 0xFF584606, 0xFF5A3F07, 0xFF60460B, 0xFF5E4409, 0xFF5C4408, 0xFF594307, 0xFF543F00, 0xFF544001, 0xFF5C4809, 0xFF584304, 0xFF5A4408, 0xFF573E05, 0xFF594008, 0xFF60450D, 0xFF5D4308, 0xFF5C3F03, 0xFF5D4102, 0xFF56410C,
        0xFF5D420B, 0xFF62410B, 0xFF5E3A06, 0xFF5A3D13, 0xFF625535, 0xFFA6A996, 0xFFA3B3A8, 0xFF99A58D, 0xFF8E8355, 0xFF8D7C48, 0xFF8C9077, 0xFF899D9C, 0xFF8B968E, 0xFF55502A, 0xFF473B00, 0xFF4D4104, 0xFF554100, 0xFF80610F, 0xFF866611, 0xFF533800,
        0xFF4A3E0C, 0xFF9EA289, 0xFFDBE8DE, 0xFFD9E4D6, 0xFFCAE0CB, 0xFF75702E, 0xFF6C4F00, 0xFF83781C, 0xFF525005, 0xFF594705, 0xFF5A470F, 0xFF5C3900, 0xFF524417, 0xFFC9C7B0, 0xFFEDEAD7, 0xFFE9E8D6, 0xFFDDE0CB, 0xFF5F5A32, 0xFF553D00, 0xFF4B4100,
        0xFF59490B, 0xFF574209, 0xFF593E07, 0xFF604007, 0xFF5E3B00, 0xFF987325, 0xFF724E00, 0xFF745515, 0xFF7E6A23, 0xFF564900, 0xFF574700, 0xFF5F4600, 0xFF664B04, 0xFF504203, 0xFF56551C, 0xFF665A10, 0xFF5B5521, 0xFFD2D4BF, 0xFFE2E8E8, 0xFFE6EAEB,
        0xFFCCC7B1, 0xFF50410A, 0xFF5E4700, 0xFF544005, 0xFF544005, 0xFF554106, 0xFF554106, 0xFF544005, 0xFF544005, 0xFF533F04, 0xFF533F04, 0xFF4A4406, 0xFF4D440B, 0xFF3F3400, 0xFF3B3100, 0xFF433E00, 0xFF444207, 0xFF53542C, 0xFFAFB19B, 0xFF8B9596,
        0xFF8B916F, 0xFF868B51, 0xFF8C996E, 0xFF8C9E90, 0xFFB0B49D, 0xFFEACE7B, 0xFFEAB227, 0xFFD7B645, 0xFFA17E2A, 0xFF513200, 0xFF55410C, 0xFF7B7332, 0xFF535911, 0xFF4A551D, 0xFF8D9975, 0xFF57553C, 0xFFB0AC91, 0xFF8B8867, 0xFF918D67, 0xFF565223,
        0xFF49430F, 0xFF474107, 0xFF4C4308, 0xFF4A390E, 0xFF4B3A0F, 0xFF4C3B0D, 0xFF4D3D0C, 0xFF4F3F0B, 0xFF50410A, 0xFF514209, 0xFF524308, 0xFF524308, 0xFF544003, 0xFF584206, 0xFF534008, 0xFF4F420D, 0xFF504002, 0xFF907023, 0xFF986D0F, 0xFF9D6F1A,
        0xFFA0771F, 0xFFA4842D, 0xFF675004, 0xFF413100, 0xFF463F08, 0xFF878747, 0xFF4F550B, 0xFF4C4205, 0xFF414000, 0xFF93957F, 0xFFF9FDFF, 0xFFF9FFFF, 0xFFFBFFFF, 0xFFB5B1A5, 0xFF413E07, 0xFF473B09, 0xFF513E00, 0xFF7F5F0A, 0xFFBC9530, 0xFFBA932C,
        0xFFAC8D30, 0xFF6F5E0E, 0xFF4E4500, 0xFFC9D7C6, 0xFFF0FFFA, 0xFFE4F3F6, 0xFFF8FFF8, 0xFF9FA074, 0xFF4B3C00, 0xFF96750A, 0xFFBD901B, 0xFF956719, 0xFF443410, 0xFF30383A, 0xFF313B44, 0xFF2E3A3A, 0xFF253938, 0xFF2C3C3B, 0xFF353A34, 0xFF2F3938,
        0xFF2D3736, 0xFF313B3A, 0xFF303A39, 0xFF2D3736, 0xFF2E3837, 0xFF2E3837, 0xFF303A39, 0xFF2F3A36, 0xFF313C38, 0xFF2C3635, 0xFF2E3837, 0xFF2E3837, 0xFF2B3536, 0xFF2F393A, 0xFF2C3637, 0xFF2C3635, 0xFF283231, 0xFF2B3534, 0xFF2C3635, 0xFF2A3433,
        0xFF2D3736, 0xFF313B3A, 0xFF2C3635, 0xFF28332F, 0xFF2D3834, 0xFF2D3834, 0xFF303B35, 0xFF2A352F, 0xFF29342C, 0xFF2B362E, 0xFF343F37, 0xFF303B33, 0xFF2D3830, 0xFF2D3830, 0xFF2F3A34, 0xFF2F3A34, 0xFF2F3A36, 0xFF2F3A36, 0xFF2E3935, 0xFF2A3A37,
        0xFF2B3632, 0xFF2A3330, 0xFF253032, 0xFF253535, 0xFF243024, 0xFF2C2C0A, 0xFF4C3F0B, 0xFF524910, 0xFF4A4100, 0xFF5B4F05, 0xFF5C4C00, 0xFF5A4A00, 0xFF604F09, 0xFF554207, 0xFF55440E, 0xFF604C19, 0xFF534008, 0xFF5A4806, 0xFF544500, 0xFF4E4000,
        0xFF96863B, 0xFF7E693C, 0xFF6E5541, 0xFF71573E, 0xFF69512D, 0xFF5D4813, 0xFF917D3E, 0xFFAF9B5C, 0xFF99844F, 0xFF654D29, 0xFF6E543B, 0xFF74543D, 0xFF6D5235, 0xFF766037, 0xFF9C8B53, 0xFF524600, 0xFF584B00, 0xFF513F00, 0xFF594500, 0xFF504211,
        0xFF5B3900, 0xFFA06500, 0xFFEAAC35, 0xFF8C5A05, 0xFF583D0E, 0xFF5D5C1C, 0xFF79891E, 0xFF4F4E4A, 0xFF040601, 0xFF060C08, 0xFF141F1B, 0xFF15211D, 0xFF1A2624, 0xFF222C2D, 0xFF161E20, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFDFE, 0x00FFFDFE, 0x00FFFFFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF1B1D18, 0xFF0D1110, 0xFF020800, 0xFF070D09, 0xFF171A21, 0xFF262721, 0xFF68693F, 0xFF53520F, 0xFF4A4600, 0xFF533E0F, 0xFF56400F, 0xFF57400C, 0xFF594008, 0xFF5E430B,
        0xFF5B4106, 0xFF553D00, 0xFF594103, 0xFF563D04, 0xFF563D04, 0xFF4F3600, 0xFF543B03, 0xFF5C430D, 0xFF5D440E, 0xFF5B410E, 0xFF543A07, 0xFF5B3F0F, 0xFF5D4111, 0xFF583C0C, 0xFF593D0B, 0xFF5D420D, 0xFF5B4009, 0xFF5E4409, 0xFF5D4306, 0xFF58410D,
        0xFF5C410A, 0xFF5E3E05, 0xFF5D3C07, 0xFF563E12, 0xFF42381D, 0xFF6A7066, 0xFF6D7D7D, 0xFF424F35, 0xFF483D0F, 0xFF483709, 0xFF4C513D, 0xFF6C7F85, 0xFF515C56, 0xFF423E18, 0xFF4A4003, 0xFF4F3F03, 0xFF533E00, 0xFF7D621B, 0xFF8E7229, 0xFF543F00,
        0xFF4D430E, 0xFF969871, 0xFFC6D3B7, 0xFFD4D1B2, 0xFFC8D1B2, 0xFFD2BF7D, 0xFFF0C567, 0xFFCCB565, 0xFF746627, 0xFF5A4101, 0xFF543B03, 0xFF5E4700, 0xFF504D1A, 0xFFA6AB95, 0xFFC5C9BB, 0xFFCAD0C4, 0xFFBECAB4, 0xFF565A28, 0xFF4F4100, 0xFF5F471B,
        0xFF523B09, 0xFF544005, 0xFF594707, 0xFF4B3F03, 0xFF4F4814, 0xFFACA882, 0xFF807D5E, 0xFF8A794D, 0xFFC2B982, 0xFF4C4705, 0xFF554700, 0xFF583D00, 0xFF5F4200, 0xFF553F03, 0xFF92864A, 0xFFB6B171, 0xFF8B8B59, 0xFFBDC8A8, 0xFFB8C9B6, 0xFFC3D2BF,
        0xFFA9AF8D, 0xFF504913, 0xFF514200, 0xFF544005, 0xFF544005, 0xFF554106, 0xFF554106, 0xFF554106, 0xFF544005, 0xFF533F04, 0xFF533F04, 0xFF4F3C01, 0xFF513C07, 0xFF543F0A, 0xFF533E05, 0xFF4D3D00, 0xFF4F450A, 0xFF595228, 0xFF8F8971, 0xFF8D8D83,
        0xFFAEA989, 0xFF949061, 0xFFA5A986, 0xFF899484, 0xFF858569, 0xFFE6C87F, 0xFFE0A934, 0xFFD6B34B, 0xFFAD893B, 0xFF5F3F0C, 0xFF513B0A, 0xFF4F450A, 0xFF3D3D00, 0xFF565C2E, 0xFF656B4F, 0xFF464A2F, 0xFF929578, 0xFF4B4B27, 0xFF4F4A20, 0xFF463E0D,
        0xFF51440D, 0xFF4F3F03, 0xFF524000, 0xFF4C3B0D, 0xFF4C3B0D, 0xFF4D3D0C, 0xFF4E3E0A, 0xFF4F4009, 0xFF504106, 0xFF514306, 0xFF524407, 0xFF514207, 0xFF533F02, 0xFF574105, 0xFF534008, 0xFF4F420D, 0xFF514103, 0xFF917222, 0xFF9B7012, 0xFF9A6D12,
        0xFFA0791C, 0xFFA68B32, 0xFF594700, 0xFF4C4109, 0xFF7D7743, 0xFFA19F62, 0xFF4D4D05, 0xFF554509, 0xFF3F3600, 0xFFA29C90, 0xFFDAD9E9, 0xFFB1B7B7, 0xFFB2B4AF, 0xFF949188, 0xFF626039, 0xFF6B5D50, 0xFF6D5941, 0xFF80643D, 0xFFA1804B, 0xFF9C7B45,
        0xFF9C8251, 0xFF7D6B45, 0xFF6F6448, 0xFFA4A691, 0xFFC4C7C0, 0xFFAEB3B9, 0xFFBABFC2, 0xFF838570, 0xFF403507, 0xFF624A0C, 0xFF8E6D28, 0xFF88631C, 0xFF3F3615, 0xFF2E393B, 0xFF2D383E, 0xFF2F3938, 0xFF283B39, 0xFF293C3A, 0xFF303632, 0xFF2F3A36,
        0xFF323D39, 0xFF303B37, 0xFF313C38, 0xFF323D39, 0xFF313C38, 0xFF323D39, 0xFF333E3A, 0xFF35403A, 0xFF37423C, 0xFF323D37, 0xFF323D39, 0xFF313C38, 0xFF2D3736, 0xFF313B3A, 0xFF2F3938, 0xFF343F3B, 0xFF2E3935, 0xFF2E3935, 0xFF303B37, 0xFF2D3834,
        0xFF2F3A36, 0xFF303B37, 0xFF2D3834, 0xFF303B37, 0xFF2A3531, 0xFF2B3632, 0xFF2D3832, 0xFF313C36, 0xFF2F3A32, 0xFF313C34, 0xFF303B33, 0xFF2F3A32, 0xFF303B33, 0xFF2E3931, 0xFF2C3731, 0xFF2E3933, 0xFF2D3834, 0xFF2A3531, 0xFF293430, 0xFF2A3A37,
        0xFF2B3632, 0xFF2A3330, 0xFF263133, 0xFF263636, 0xFF243024, 0xFF2D2D0B, 0xFF4D400C, 0xFF524609, 0xFF5C4E11, 0xFF534305, 0xFF574505, 0xFF645011, 0xFF5B4806, 0xFF554301, 0xFF61500B, 0xFF4F3C00, 0xFF6A5619, 0xFF564207, 0xFF564404, 0xFF5B4A05,
        0xFF897A39, 0xFF7E6C3C, 0xFF5F4A2B, 0xFF685038, 0xFF6B5434, 0xFF644F22, 0xFF8B7744, 0xFF9D8956, 0xFF907B4E, 0xFF675030, 0xFF6C543C, 0xFF6C4B3C, 0xFF715638, 0xFF715F2D, 0xFF938848, 0xFF4E4400, 0xFF524204, 0xFF59430A, 0xFF5E430E, 0xFF543E00,
        0xFF886501, 0xFFD3A02A, 0xFFCA952B, 0xFF714A00, 0xFF614E0A, 0xFF57580C, 0xFF858E35, 0xFF3D3C38, 0xFF000100, 0xFF030905, 0xFF141F1B, 0xFF182420, 0xFF1C2826, 0xFF202A2B, 0xFF141C1E, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FFFFFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF1B1D18, 0xFF111216, 0xFF040902, 0xFF0B100C, 0xFF0D1015, 0xFF202318, 0xFF686B3C, 0xFF4F5108, 0xFF504E04, 0xFF584014, 0xFF5C4114, 0xFF583D0E, 0xFF573B0B, 0xFF5C3E0C,
        0xFF5C3E08, 0xFF5D3E07, 0xFF5E3F08, 0xFF584209, 0xFF60470F, 0xFF5C430D, 0xFF5B410E, 0xFF583C0A, 0xFF573B0B, 0xFF624315, 0xFF604113, 0xFF5E3D10, 0xFF5E3F11, 0xFF5C3D0E, 0xFF5F410F, 0xFF5D420D, 0xFF563B06, 0xFF5B4009, 0xFF553C04, 0xFF573D0C,
        0xFF5D3F09, 0xFF5D3F03, 0xFF5E4109, 0xFF574217, 0xFF332D17, 0xFF4A5254, 0xFF596A7A, 0xFF212C0E, 0xFF3D3507, 0xFF372A00, 0xFF31372B, 0xFF6A7C86, 0xFF313A35, 0xFF3D3913, 0xFF494103, 0xFF5B4209, 0xFF564007, 0xFF78632C, 0xFF8E7942, 0xFF4A3A00,
        0xFF473E03, 0xFF919259, 0xFFC8D19C, 0xFFC4C486, 0xFF929C6A, 0xFF8C7934, 0xFF986E18, 0xFF826F2B, 0xFF615B21, 0xFF4C3D00, 0xFF544504, 0xFF534000, 0xFF484A19, 0xFF8A9972, 0xFFBDC9A1, 0xFFBDCCA3, 0xFF94A67E, 0xFF464F1A, 0xFF504300, 0xFF5C3B0E,
        0xFF60400D, 0xFF5D4004, 0xFF594200, 0xFF504002, 0xFF49420E, 0xFFBFBF99, 0xFFB2B799, 0xFFBFC0A0, 0xFFC3C69B, 0xFF43440B, 0xFF4B4103, 0xFF664E10, 0xFF604206, 0xFF5B3E02, 0xFF685314, 0xFF625D25, 0xFF4F541E, 0xFF94A672, 0xFFB3CE9B, 0xFFBFD7A5,
        0xFF84925D, 0xFF494812, 0xFF493E08, 0xFF554106, 0xFF554106, 0xFF554106, 0xFF554106, 0xFF554106, 0xFF544005, 0xFF544005, 0xFF533F04, 0xFF5C3D04, 0xFF5A3A07, 0xFF5F3F0E, 0xFF5D4008, 0xFF543C00, 0xFF523F04, 0xFF55481B, 0xFF7A6F4F, 0xFF8F886B,
        0xFF807150, 0xFF746544, 0xFF756F55, 0xFF5B5F48, 0xFF63613B, 0xFFA28947, 0xFFB5862E, 0xFFC4A241, 0xFF775609, 0xFF583903, 0xFF594111, 0xFF51420B, 0xFF474009, 0xFF504C26, 0xFF716F58, 0xFF666E47, 0xFF848961, 0xFF4B4A1E, 0xFF6A6434, 0xFF60531E,
        0xFF554207, 0xFF553D00, 0xFF583C00, 0xFF4F3F0B, 0xFF4F3F0B, 0xFF4F4009, 0xFF504108, 0xFF514207, 0xFF514306, 0xFF524405, 0xFF524403, 0xFF54450A, 0xFF564205, 0xFF594307, 0xFF544109, 0xFF4F420B, 0xFF4F3F01, 0xFF907121, 0xFF9B7012, 0xFFA27615,
        0xFF9E7C1B, 0xFF97832C, 0xFF7E742F, 0xFF3F3907, 0xFF8B8758, 0xFF7D773D, 0xFF463E00, 0xFF5A4500, 0xFF6D5D0F, 0xFFA79C7E, 0xFF979388, 0xFFAAB188, 0xFFDEE7B8, 0xFFD4DAB6, 0xFFD4DBA8, 0xFFD8D5AC, 0xFFDAD3A5, 0xFFDAD19A, 0xFFE4DA9C, 0xFFDDD493,
        0xFFE9E1A3, 0xFFDED9A1, 0xFFDFDEA8, 0xFFE2DDA6, 0xFFFFFED9, 0xFFD4D3BF, 0xFF9A9B8B, 0xFF626548, 0xFF48451A, 0xFF493F0C, 0xFF514410, 0xFF604B0C, 0xFF2E3018, 0xFF2C3B40, 0xFF2D363B, 0xFF313733, 0xFF2A3B35, 0xFF273A38, 0xFF2C3635, 0xFF37423A,
        0xFF404B43, 0xFF364139, 0xFF38433B, 0xFF3E4941, 0xFF38433B, 0xFF3A453D, 0xFF3A453D, 0xFF3A463C, 0xFF3F4B41, 0xFF3D4840, 0xFF3F4A42, 0xFF3D4840, 0xFF37423C, 0xFF3A453F, 0xFF38433D, 0xFF404B43, 0xFF38433B, 0xFF37423A, 0xFF3C473F, 0xFF3E4941,
        0xFF3D4840, 0xFF3E4941, 0xFF3D4840, 0xFF3F4A46, 0xFF2D3834, 0xFF313C38, 0xFF2D3832, 0xFF38433D, 0xFF354038, 0xFF404B43, 0xFF38433B, 0xFF354038, 0xFF3D4840, 0xFF39443C, 0xFF343F39, 0xFF39443E, 0xFF394440, 0xFF313C38, 0xFF2F3A36, 0xFF2A3A37,
        0xFF2B3632, 0xFF2B3431, 0xFF273234, 0xFF273737, 0xFF253125, 0xFF2D2D0B, 0xFF4D400C, 0xFF523E00, 0xFF634F14, 0xFF513E06, 0xFF604D15, 0xFF614E14, 0xFF503E00, 0xFF635308, 0xFF5C4C00, 0xFF5F4F00, 0xFF564400, 0xFF604B14, 0xFF5C4815, 0xFF4B3A02,
        0xFF827237, 0xFF7D6D39, 0xFF67562A, 0xFF5F4C2C, 0xFF655231, 0xFF5C4A22, 0xFF69582D, 0xFF6B5A2F, 0xFF6E5C34, 0xFF5E4B2A, 0xFF635030, 0xFF674637, 0xFF624A24, 0xFF716626, 0xFF8F8B42, 0xFF4F4709, 0xFF534312, 0xFF583C0D, 0xFF5B3A07, 0xFF7F5900,
        0xFFD4AC32, 0xFFC29820, 0xFF754D00, 0xFF583D00, 0xFF5F5702, 0xFF5D5D05, 0xFF77772F, 0xFF32312D, 0xFF000100, 0xFF030905, 0xFF131E1A, 0xFF1A2622, 0xFF1D2927, 0xFF1F292A, 0xFF161E20, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FFFFFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF1A1C17, 0xFF0F0F17, 0xFF040601, 0xFF0D120E, 0xFF090D10, 0xFF22251A, 0xFF676D3D, 0xFF4A4D08, 0xFF4B4C06, 0xFF553C03, 0xFF594008, 0xFF543D07, 0xFF583E0B, 0xFF5C400E,
        0xFF5E3E0D, 0xFF613D0B, 0xFF5D3804, 0xFF57400A, 0xFF57400A, 0xFF543D07, 0xFF594008, 0xFF583F07, 0xFF583D05, 0xFF5E430B, 0xFF593E06, 0xFF664911, 0xFF5F4208, 0xFF5C3F05, 0xFF5C4207, 0xFF60460B, 0xFF573E06, 0xFF604613, 0xFF5D4312, 0xFF593D0D,
        0xFF5F420A, 0xFF5F4206, 0xFF573F03, 0xFF4D3C0E, 0xFF342F19, 0xFF4C5457, 0xFF617085, 0xFF1E2503, 0xFF473F10, 0xFF3D310B, 0xFF2D322B, 0xFF67757E, 0xFF3D423B, 0xFF4A451D, 0xFF4B4507, 0xFF553400, 0xFF563E0E, 0xFF807148, 0xFF9F9369, 0xFF514511,
        0xFF423800, 0xFF82823A, 0xFFBEC67D, 0xFFB3C36A, 0xFF6F8942, 0xFF3D3A00, 0xFF543800, 0xFF7A7637, 0xFF7D8850, 0xFF4B4F05, 0xFF4B5204, 0xFF594018, 0xFF3F3C0F, 0xFF748244, 0xFFBECE76, 0xFFB4C46B, 0xFF6B7C36, 0xFF40400A, 0xFF594311, 0xFF504609,
        0xFF594A11, 0xFF54400B, 0xFF56410C, 0xFF5C4813, 0xFF564A0E, 0xFFD5D28F, 0xFFDCDD97, 0xFFD5DDB6, 0xFFA8AF7C, 0xFF505213, 0xFF4E4607, 0xFF544409, 0xFF503B02, 0xFF584407, 0xFF544200, 0xFF493E06, 0xFF373700, 0xFF859348, 0xFFB3CC79, 0xFFB6CF7E,
        0xFF65722A, 0xFF3C3A00, 0xFF4D400C, 0xFF554104, 0xFF554104, 0xFF564205, 0xFF564205, 0xFF554104, 0xFF554104, 0xFF544003, 0xFF544003, 0xFF5D4004, 0xFF5C3D07, 0xFF593908, 0xFF593C04, 0xFF5B4401, 0xFF4C3D00, 0xFF4D420A, 0xFF797045, 0xFF766F3B,
        0xFF251700, 0xFF2A1B00, 0xFF231C00, 0xFF1D2100, 0xFF3A3C0A, 0xFF413300, 0xFF654606, 0xFFB49332, 0xFF755403, 0xFF593A00, 0xFF553B00, 0xFF594506, 0xFF504205, 0xFF5B4F1F, 0xFFA29573, 0xFF565419, 0xFF514C12, 0xFF5D541B, 0xFF91854B, 0xFF5D4D12,
        0xFF493500, 0xFF5A4408, 0xFF594105, 0xFF524207, 0xFF524207, 0xFF524206, 0xFF524204, 0xFF534305, 0xFF534403, 0xFF534401, 0xFF534401, 0xFF544609, 0xFF544001, 0xFF584304, 0xFF554208, 0xFF50430C, 0xFF504100, 0xFF937424, 0xFFA07516, 0xFF96700B,
        0xFF937714, 0xFF928630, 0xFF9F9E5E, 0xFF3C3B0F, 0xFF6C673D, 0xFF5A4D16, 0xFF564500, 0xFFA98E27, 0xFFA18B36, 0xFF968362, 0xFF8F8470, 0xFFD1D79B, 0xFFD4E09E, 0xFFB1B990, 0xFFC2CDA3, 0xFFCDCDB3, 0xFFD0D3B8, 0xFFCACFB1, 0xFFCBD0B2, 0xFFC6CBAB,
        0xFFCFD5B3, 0xFFC9CDAC, 0xFFCCCDAD, 0xFFC1C088, 0xFFCECCA3, 0xFFBAB69B, 0xFF928E73, 0xFF57542B, 0xFF434009, 0xFF474105, 0xFF494007, 0xFF3A2D00, 0xFF1E2712, 0xFF2B3E44, 0xFF2F393B, 0xFF323730, 0xFF293831, 0xFF263937, 0xFF2E3A3A, 0xFF38443A,
        0xFF455147, 0xFF333F35, 0xFF374339, 0xFF404C42, 0xFF354137, 0xFF38443A, 0xFF374339, 0xFF2F3B2F, 0xFF384438, 0xFF3B473B, 0xFF414D43, 0xFF3F4B41, 0xFF38433B, 0xFF39443C, 0xFF364139, 0xFF38443A, 0xFF2E3A30, 0xFF2D392F, 0xFF354137, 0xFF39453B,
        0xFF354137, 0xFF333F35, 0xFF344036, 0xFF3C4743, 0xFF293430, 0xFF35403C, 0xFF2D3832, 0xFF38433D, 0xFF303B33, 0xFF424D45, 0xFF38433B, 0xFF2F3A32, 0xFF3D4840, 0xFF3A453D, 0xFF343F39, 0xFF3C4741, 0xFF3B4642, 0xFF2F3A36, 0xFF2C3733, 0xFF293936,
        0xFF2B3632, 0xFF2B3431, 0xFF273234, 0xFF283838, 0xFF263226, 0xFF2E2E0C, 0xFF4E410D, 0xFF5F470B, 0xFF564102, 0xFF655311, 0xFF5B4E08, 0xFF564C04, 0xFF5D5409, 0xFF564A00, 0xFF63570B, 0xFF635701, 0xFF5A4C03, 0xFF4F3F03, 0xFF5D4F12, 0xFF4F4200,
        0xFF837630, 0xFFB7A96C, 0xFF978754, 0xFF9B8D5C, 0xFF9D8F60, 0xFF928455, 0xFF8F8154, 0xFF85774A, 0xFF918354, 0xFF928455, 0xFF988A59, 0xFF9A7D5F, 0xFF9A8653, 0xFF989048, 0xFF7C7934, 0xFF483F08, 0xFF5C4815, 0xFF5D4000, 0xFF825E08, 0xFFD5A235,
        0xFFC2962B, 0xFF765100, 0xFF523700, 0xFF5F500D, 0xFF4F4D00, 0xFF5B5B05, 0xFF828036, 0xFF363531, 0xFF000200, 0xFF050B07, 0xFF101B17, 0xFF182420, 0xFF1C2826, 0xFF1F292A, 0xFF1A2224, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FFFFFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF1A1C17, 0xFF1A1823, 0xFF070906, 0xFF0C110D, 0xFF0A0F12, 0xFF242A20, 0xFF666D43, 0xFF4B5113, 0xFF43440A, 0xFF594700, 0xFF5B4A04, 0xFF524204, 0xFF53440B, 0xFF54400D,
        0xFF563C09, 0xFF62410E, 0xFF623D09, 0xFF553E08, 0xFF57400A, 0xFF59430A, 0xFF5A4609, 0xFF594506, 0xFF574503, 0xFF584702, 0xFF574600, 0xFF5A4702, 0xFF534100, 0xFF584600, 0xFF574300, 0xFF604B08, 0xFF5A4206, 0xFF5C420F, 0xFF593E11, 0xFF5B3F0D,
        0xFF5D420A, 0xFF604609, 0xFF543E02, 0xFF4D3D0C, 0xFF4A4327, 0xFF62635D, 0xFF6F767E, 0xFF686A42, 0xFF7E7240, 0xFF685C32, 0xFF535A4A, 0xFF6B7574, 0xFF5F5E4C, 0xFF50481A, 0xFF464002, 0xFF623F07, 0xFF553E12, 0xFF68603C, 0xFF7E7953, 0xFF453905,
        0xFF413700, 0xFF949046, 0xFFE1E89B, 0xFFCEDF83, 0xFF809A53, 0xFF484500, 0xFF7E6111, 0xFF8B874A, 0xFF707B43, 0xFF3E4200, 0xFF434A00, 0xFF624012, 0xFF453907, 0xFF8C924A, 0xFFEAF191, 0xFFDBE17D, 0xFF838B38, 0xFF544907, 0xFF573500, 0xFF474300,
        0xFF463E00, 0xFF544500, 0xFF5E4B09, 0xFF4B3C00, 0xFF4B4400, 0xFFC9CD83, 0xFFD0DC90, 0xFFC6C78D, 0xFFA2A25C, 0xFF443E00, 0xFF564E05, 0xFF51470C, 0xFF524813, 0xFF423C02, 0xFF443F00, 0xFF563E00, 0xFF524600, 0xFFAAAE59, 0xFFD1E083, 0xFFCADB7F,
        0xFF7A7F2D, 0xFF463B00, 0xFF5A430D, 0xFF564205, 0xFF564205, 0xFF564205, 0xFF564205, 0xFF564205, 0xFF554104, 0xFF554104, 0xFF544003, 0xFF514000, 0xFF533F04, 0xFF533E07, 0xFF544003, 0xFF594802, 0xFF483F00, 0xFF474200, 0xFF67622A, 0xFF74712A,
        0xFF403601, 0xFF3A2E08, 0xFF413C16, 0xFF3C3F0A, 0xFF414604, 0xFF4A4406, 0xFF55440E, 0xFF856705, 0xFFAA8C32, 0xFF7C5F0F, 0xFF674C00, 0xFF644D00, 0xFF5B4800, 0xFF857230, 0xFF9F8C54, 0xFF665100, 0xFF685206, 0xFF8C792D, 0xFF95823D, 0xFF554303,
        0xFF4F3F03, 0xFF56470E, 0xFF473801, 0xFF534305, 0xFF534305, 0xFF534403, 0xFF534403, 0xFF534403, 0xFF534403, 0xFF534401, 0xFF534401, 0xFF504205, 0xFF513D00, 0xFF564102, 0xFF554208, 0xFF51450B, 0xFF534403, 0xFF997A28, 0xFFA97E1F, 0xFF8E7010,
        0xFF907D1F, 0xFF857E2D, 0xFF636526, 0xFF424319, 0xFF605B31, 0xFF90814A, 0xFFA7934C, 0xFF8F6E27, 0xFF5F420A, 0xFF40251A, 0xFF6C5A58, 0xFFA09E77, 0xFF858B65, 0xFF8A918A, 0xFF8B9493, 0xFF858A86, 0xFF858B8B, 0xFF788284, 0xFF788389, 0xFF758086,
        0xFF798385, 0xFF767C7A, 0xFF747973, 0xFF768075, 0xFF818682, 0xFF807C7B, 0xFF7B6F63, 0xFF544420, 0xFF4E3B03, 0xFF5B4605, 0xFF543F00, 0xFF3E3100, 0xFF212A15, 0xFF2D4044, 0xFF303A3C, 0xFF33392F, 0xFF2A3A30, 0xFF263937, 0xFF2D3937, 0xFF38443A,
        0xFF455147, 0xFF344036, 0xFF38443A, 0xFF414D43, 0xFF38443A, 0xFF3B473D, 0xFF3A463C, 0xFF303C30, 0xFF384438, 0xFF3A463A, 0xFF3E4A40, 0xFF3C483E, 0xFF364139, 0xFF39443C, 0xFF364139, 0xFF414D43, 0xFF3A463C, 0xFF374339, 0xFF3E4A40, 0xFF414D43,
        0xFF39453B, 0xFF323E34, 0xFF323E34, 0xFF3A4541, 0xFF293430, 0xFF38433F, 0xFF333E38, 0xFF3E4943, 0xFF343F37, 0xFF424D45, 0xFF343F37, 0xFF2D3830, 0xFF3C473F, 0xFF3A453D, 0xFF35403A, 0xFF3B4640, 0xFF38433F, 0xFF2A3531, 0xFF25302C, 0xFF283835,
        0xFF2A3531, 0xFF2B3431, 0xFF283335, 0xFF283838, 0xFF263226, 0xFF2E2E0C, 0xFF4E410D, 0xFF5E450C, 0xFF5F4C08, 0xFF4E4600, 0xFF535300, 0xFF83862D, 0xFFB3B667, 0xFFCBCA8A, 0xFFD3CE97, 0xFFC6C585, 0xFFB0AC6F, 0xFF777232, 0xFF605A10, 0xFF474100,
        0xFF504800, 0xFF837632, 0xFF948450, 0xFF958B46, 0xFF948949, 0xFF928649, 0xFF92864C, 0xFF887C42, 0xFF8E8245, 0xFF8F8444, 0xFF8F8540, 0xFF978249, 0xFF988948, 0xFF7B712C, 0xFF62581A, 0xFF504005, 0xFF594100, 0xFF7F5F00, 0xFFCEAD2C, 0xFFCC9737,
        0xFF6F4000, 0xFF5B3A07, 0xFF59450A, 0xFF544A05, 0xFF524B07, 0xFF5E5A10, 0xFF87872F, 0xFF3B3A36, 0xFF020400, 0xFF050B07, 0xFF0F1A16, 0xFF17231F, 0xFF1A2624, 0xFF1E2829, 0xFF1A2224, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FFFFFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF191B16, 0xFF0E0B16, 0xFF040402, 0xFF0E130F, 0xFF151A1E, 0xFF1C231C, 0xFF535A38, 0xFF575D2D, 0xFF505328, 0xFF655C1B, 0xFF6A6126, 0xFF645F28, 0xFF625F2C, 0xFF544C1B,
        0xFF4C3A08, 0xFF5E430C, 0xFF624209, 0xFF5C4109, 0xFF5C430A, 0xFF5B460D, 0xFF4B3B00, 0xFF52460C, 0xFF665D24, 0xFF666026, 0xFF706B31, 0xFF686228, 0xFF645E20, 0xFF6A611E, 0xFF4F4200, 0xFF574601, 0xFF5A4408, 0xFF5B3F0D, 0xFF5C4011, 0xFF5B420C,
        0xFF5C410A, 0xFF60480C, 0xFF584206, 0xFF58440F, 0xFF5E5127, 0xFF686145, 0xFF6F6A57, 0xFF726B3F, 0xFF6A5B22, 0xFF625721, 0xFF646949, 0xFF6A6F5B, 0xFF766D4C, 0xFF594915, 0xFF4F4708, 0xFF623F07, 0xFF4B390B, 0xFF4A4621, 0xFF59562D, 0xFF4E4208,
        0xFF584702, 0xFF9B9652, 0xFFD9E2A3, 0xFFD8D993, 0xFF939D68, 0xFF6E5B19, 0xFFA3782A, 0xFF655116, 0xFF4B440E, 0xFF524300, 0xFF5A4C03, 0xFF624000, 0xFF4F4307, 0xFFA4A36D, 0xFFE9E4AA, 0xFFDDD898, 0xFFB3B06D, 0xFF9D8E3B, 0xFF9F7C16, 0xFFAC7A19,
        0xFFAE7B10, 0xFFAD7903, 0xFFAA7900, 0xFF987000, 0xFF8B7112, 0xFFCABE74, 0xFFD9D79C, 0xFFE5E19A, 0xFFBCB463, 0xFF645500, 0xFF6E6017, 0xFF7B743E, 0xFF636236, 0xFF7C7E4C, 0xFF6F6F31, 0xFF634401, 0xFF6E5A13, 0xFFBEBA70, 0xFFD0D78A, 0xFFCBD388,
        0xFF9B9B55, 0xFF564608, 0xFF5B4008, 0xFF564203, 0xFF564203, 0xFF574304, 0xFF574304, 0xFF574304, 0xFF564203, 0xFF554102, 0xFF554102, 0xFF4D4700, 0xFF4D4202, 0xFF52430A, 0xFF503E00, 0xFF514100, 0xFF4C3F00, 0xFF4D4400, 0xFF4D4401, 0xFF595403,
        0xFF514604, 0xFF4F3F0B, 0xFF52450E, 0xFF494000, 0xFF474000, 0xFF524904, 0xFF4D3E03, 0xFF553700, 0xFF947620, 0xFFA2842E, 0xFFA78C33, 0xFFA78C31, 0xFF998026, 0xFFA18635, 0xFF94782E, 0xFFA48228, 0xFFA08127, 0xFF957925, 0xFF6B5104, 0xFF4F3B00,
        0xFF574807, 0xFF504408, 0xFF50460B, 0xFF554303, 0xFF554303, 0xFF554303, 0xFF554303, 0xFF554303, 0xFF554305, 0xFF554305, 0xFF554305, 0xFF554708, 0xFF544100, 0xFF584302, 0xFF564308, 0xFF51450B, 0xFF514200, 0xFF977826, 0xFFA97E1F, 0xFF93812B,
        0xFF9E923E, 0xFF6A671A, 0xFF3C3E00, 0xFF403F11, 0xFF4D4419, 0xFF5F4E16, 0xFF6E5A13, 0xFF5D4004, 0xFF533B00, 0xFF3E290E, 0xFF756556, 0xFF95936A, 0xFF808567, 0xFF92979A, 0xFF747D82, 0xFF707E65, 0xFF687561, 0xFF5C6D5D, 0xFF65756A, 0xFF66766C,
        0xFF647467, 0xFF657260, 0xFF616F58, 0xFF6E8686, 0xFF818F90, 0xFF75726D, 0xFF766549, 0xFF765C1F, 0xFF8C6C15, 0xFF9A7816, 0xFF8E6C08, 0xFF685312, 0xFF34371A, 0xFF2E3E3E, 0xFF2E383A, 0xFF333A32, 0xFF2C3D35, 0xFF283B37, 0xFF2C3731, 0xFF37423A,
        0xFF404B43, 0xFF37423A, 0xFF3B463E, 0xFF424D45, 0xFF3C473F, 0xFF3F4A42, 0xFF3F4A42, 0xFF3A463C, 0xFF3E4A40, 0xFF3A453D, 0xFF3B463E, 0xFF38433B, 0xFF343F39, 0xFF39443E, 0xFF38433D, 0xFF3D4840, 0xFF3A453D, 0xFF38433B, 0xFF3C473F, 0xFF404B43,
        0xFF39443C, 0xFF323D35, 0xFF333E36, 0xFF434E4A, 0xFF303B37, 0xFF343F3B, 0xFF313C36, 0xFF3D4842, 0xFF3A453D, 0xFF434E46, 0xFF3A453D, 0xFF364139, 0xFF404B43, 0xFF3F4A42, 0xFF3B4640, 0xFF3E4943, 0xFF394440, 0xFF2D3834, 0xFF28332F, 0xFF273734,
        0xFF2A3531, 0xFF2B3431, 0xFF283335, 0xFF283838, 0xFF263226, 0xFF2E2E0C, 0xFF4E410D, 0xFF584604, 0xFF4D4300, 0xFF707324, 0xFFB9C674, 0xFFBECE85, 0xFF9BA872, 0xFFA9B08E, 0xFFC0C2AC, 0xFFCAD4B9, 0xFFD8DFC0, 0xFFDEE3BA, 0xFF9FA162, 0xFF666513,
        0xFF5D5703, 0xFF544500, 0xFF56430B, 0xFF524600, 0xFF4F4300, 0xFF514400, 0xFF584B05, 0xFF534600, 0xFF544701, 0xFF534700, 0xFF514500, 0xFF544600, 0xFF554603, 0xFF463400, 0xFF543F00, 0xFF5C4200, 0xFF826402, 0xFFC7A42E, 0xFFC49F1A, 0xFF744800,
        0xFF5C3609, 0xFF5C3E18, 0xFF574306, 0xFF534200, 0xFF57460E, 0xFF625D19, 0xFF78801D, 0xFF3E3D39, 0xFF000100, 0xFF010703, 0xFF111C18, 0xFF182420, 0xFF1A2624, 0xFF1D2728, 0xFF171F21, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FFFFFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF191B16, 0xFF16131C, 0xFF0B0B09, 0xFF0B100C, 0xFF12171D, 0xFF111717, 0xFF616952, 0xFFBDC2A2, 0xFFDCDEC6, 0xFFEDE7CD, 0xFFF1EFD6, 0xFFE4E6CE, 0xFFCACEB3, 0xFF8E906B,
        0xFF585021, 0xFF564308, 0xFF553C00, 0xFF5E4201, 0xFF573D00, 0xFF56410A, 0xFF524415, 0xFF99916C, 0xFFE9E8CA, 0xFFECF0D9, 0xFFF1F6E2, 0xFFF2F5E2, 0xFFE8EBCE, 0xFFCDCBA2, 0xFF6A632D, 0xFF534409, 0xFF594309, 0xFF543904, 0xFF5E3E0D, 0xFF5A430D,
        0xFF5A440B, 0xFF5E450C, 0xFF594105, 0xFF594309, 0xFF55420A, 0xFF54430F, 0xFF6A5A27, 0xFF6A5C2D, 0xFF4E3A00, 0xFF524403, 0xFF575928, 0xFF56572F, 0xFF685827, 0xFF533F04, 0xFF4B3F02, 0xFF593A01, 0xFF574918, 0xFF616238, 0xFF7A7744, 0xFF9D8B4B,
        0xFF907C37, 0xFF958E57, 0xFF97A278, 0xFF8D8C60, 0xFF848E6B, 0xFF806C31, 0xFFAD802F, 0xFF5A4100, 0xFF56470C, 0xFF654D07, 0xFF574000, 0xFF644A00, 0xFF4E4713, 0xFF8E8F70, 0xFF969277, 0xFF8B8666, 0xFF9F9D6D, 0xFFB1A656, 0xFFCEB042, 0xFFDEA439,
        0xFFECAE3D, 0xFFE6A429, 0xFFEDAC2C, 0xFFF6BB45, 0xFFD4A646, 0xFFA78940, 0xFFA48F56, 0xFFA5A162, 0xFF93873D, 0xFF917B2F, 0xFF847031, 0xFF857C53, 0xFF464324, 0xFF8B8960, 0xFF817C44, 0xFF9A7F3C, 0xFF8B783D, 0xFF959161, 0xFF848B62, 0xFF8D956E,
        0xFF919268, 0xFF524614, 0xFF503D05, 0xFF564203, 0xFF574304, 0xFF574304, 0xFF574304, 0xFF574304, 0xFF564203, 0xFF564203, 0xFF554102, 0xFF4F4900, 0xFF504503, 0xFF554208, 0xFF594105, 0xFF6F550B, 0xFF735809, 0xFF72570A, 0xFF694E09, 0xFF6D5A0B,
        0xFF6A5409, 0xFF71560F, 0xFF6D4F09, 0xFF6F4E07, 0xFF74560E, 0xFF6F520C, 0xFF705510, 0xFF775815, 0xFF6C4C03, 0xFF7A5B0B, 0xFF7D5F07, 0xFF795A00, 0xFF7A5B01, 0xFF785501, 0xFF825F0D, 0xFF7D5B02, 0xFF7F5F08, 0xFF785A04, 0xFF684E00, 0xFF725B0D,
        0xFF705A0F, 0xFF645208, 0xFF604E06, 0xFF534101, 0xFF534101, 0xFF544204, 0xFF544204, 0xFF544106, 0xFF554207, 0xFF554207, 0xFF554208, 0xFF584A0B, 0xFF574402, 0xFF5A4504, 0xFF574409, 0xFF51450B, 0xFF504100, 0xFF967724, 0xFFA97F1D, 0xFF979048,
        0xFF88843A, 0xFF4A4800, 0xFF454404, 0xFF47410F, 0xFF554B1A, 0xFF483A00, 0xFF564600, 0xFF524000, 0xFF514400, 0xFF372B00, 0xFF6F663B, 0xFF9B9B65, 0xFF898E6E, 0xFF858B89, 0xFF8C988E, 0xFF778574, 0xFF687669, 0xFF657269, 0xFF78837F, 0xFF77827E,
        0xFF6C7972, 0xFF6E7B72, 0xFF6A786B, 0xFF678877, 0xFF6E8070, 0xFF6E6A4F, 0xFF85703B, 0xFF9B7922, 0xFFAE8614, 0xFFB28A0E, 0xFFBD9519, 0xFF8E6A1E, 0xFF423A15, 0xFF2C3836, 0xFF2C3739, 0xFF303B35, 0xFF2A3D37, 0xFF2B3E38, 0xFF313830, 0xFF303B37,
        0xFF333E3A, 0xFF313C38, 0xFF323D39, 0xFF343F3B, 0xFF333E3A, 0xFF343F3B, 0xFF35403C, 0xFF35403A, 0xFF37423C, 0xFF313C36, 0xFF323D39, 0xFF303B37, 0xFF2C3635, 0xFF313B3A, 0xFF2F3938, 0xFF333E3A, 0xFF35403C, 0xFF323D39, 0xFF343F3B, 0xFF394440,
        0xFF37423E, 0xFF333E3A, 0xFF343F3B, 0xFF3A4541, 0xFF2E3935, 0xFF293430, 0xFF29342E, 0xFF2F3A34, 0xFF323D35, 0xFF364139, 0xFF364139, 0xFF333E36, 0xFF354038, 0xFF354038, 0xFF343F39, 0xFF343F39, 0xFF313C38, 0xFF2B3632, 0xFF28332F, 0xFF263633,
        0xFF293430, 0xFF2A3330, 0xFF273234, 0xFF283838, 0xFF263226, 0xFF2E2E0C, 0xFF4D400C, 0xFF4C4400, 0xFF75752D, 0xFFC4CD94, 0xFFA9BB93, 0xFF97AF97, 0xFFC2D8CC, 0xFFCEDEDD, 0xFFF7FFFF, 0xFFDCF4F6, 0xFFC5D8DE, 0xFFD0DCDC, 0xFFDFE4D0, 0xFFB1B082,
        0xFF605819, 0xFF514200, 0xFF5D490A, 0xFF574503, 0xFF544300, 0xFF544300, 0xFF574601, 0xFF564500, 0xFF574601, 0xFF584702, 0xFF574503, 0xFF473E00, 0xFF57470C, 0xFF5A3F07, 0xFF5E3C00, 0xFF855F00, 0xFFC49C2A, 0xFFD1A63E, 0xFF784A00, 0xFF563701,
        0xFF674E15, 0xFF4F3C00, 0xFF5F4802, 0xFF6C4E0E, 0xFF573F01, 0xFF5B550B, 0xFF7D8A2E, 0xFF484743, 0xFF000100, 0xFF000602, 0xFF15201C, 0xFF1A2622, 0xFF1B2725, 0xFF1E2829, 0xFF161E20, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FFFFFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF191B16, 0xFF15121B, 0xFF0A0B06, 0xFF080D09, 0xFF171B24, 0xFF0D1216, 0xFF5C6353, 0xFFD3D8C1, 0xFFF2F4E7, 0xFFFFFBFF, 0xFFFFFEFF, 0xFFEEF3F6, 0xFFC7D1C8, 0xFF7B8469,
        0xFF414012, 0xFF4C4101, 0xFF5B4900, 0xFF624500, 0xFF5D4404, 0xFF584411, 0xFF483A17, 0xFF948F7C, 0xFFF0F0EE, 0xFFE3EAF4, 0xFFDBE4F5, 0xFFEDF4FF, 0xFFF3F8FC, 0xFFDBDCCC, 0xFF5F5B36, 0xFF463A08, 0xFF5B460F, 0xFF5B3D07, 0xFF674611, 0xFF574209,
        0xFF5A440B, 0xFF5E450D, 0xFF5C430A, 0xFF5F4508, 0xFF533A00, 0xFF533C00, 0xFF856E28, 0xFF796533, 0xFF533C00, 0xFF514300, 0xFF45460A, 0xFF43420C, 0xFF574209, 0xFF584000, 0xFF524405, 0xFF543A00, 0xFF4F4510, 0xFF464817, 0xFF4D480E, 0xFF8D7932,
        0xFF8F7633, 0xFF8D8556, 0xFF7E8B71, 0xFF7A8769, 0xFF93AA8E, 0xFF7C763A, 0xFF8E6F15, 0xFF4A3D00, 0xFF524F0A, 0xFF574600, 0xFF574600, 0xFF523D06, 0xFF4A4724, 0xFF939C89, 0xFF888B76, 0xFF76795C, 0xFF7A7F57, 0xFF5D5514, 0xFF5F4400, 0xFF4F4A00,
        0xFF574700, 0xFF614400, 0xFF6F4700, 0xFF734B06, 0xFF826021, 0xFF786228, 0xFF93874D, 0xFF676535, 0xFF55490D, 0xFF5D4603, 0xFF523C0A, 0xFF55472C, 0xFF2D2613, 0xFF443B1A, 0xFF4D3F0E, 0xFF4B3600, 0xFF554917, 0xFF6A6A4E, 0xFF6D786A, 0xFF7C887E,
        0xFF8B907A, 0xFF4C4822, 0xFF4D410F, 0xFF574304, 0xFF574304, 0xFF574304, 0xFF574304, 0xFF574304, 0xFF574304, 0xFF564203, 0xFF554102, 0xFF4A4100, 0xFF534403, 0xFF573E06, 0xFF714E14, 0xFFA9823D, 0xFFAB8135, 0xFF9F7529, 0xFFA47934, 0xFF9E7E33,
        0xFF9F7B2D, 0xFF987122, 0xFFA5772A, 0xFFAC7A31, 0xFFA4722B, 0xFFA1722C, 0xFF9E772C, 0xFF9A7946, 0xFF95753A, 0xFF95742D, 0xFF9C7B2A, 0xFF9A7723, 0xFF9A7622, 0xFFA37D2A, 0xFF997320, 0xFF937724, 0xFF927623, 0xFF967A27, 0xFF987E2A, 0xFF977D29,
        0xFF937924, 0xFFA48A35, 0xFF8A701B, 0xFF524002, 0xFF524002, 0xFF534103, 0xFF534005, 0xFF544107, 0xFF544107, 0xFF55420A, 0xFF55420A, 0xFF524405, 0xFF513E00, 0xFF564100, 0xFF564308, 0xFF53470B, 0xFF544502, 0xFF9C7D2A, 0xFFB18725, 0xFF868648,
        0xFF585714, 0xFF454500, 0xFF4A4606, 0xFF504611, 0xFF514410, 0xFF554708, 0xFF4E4000, 0xFF4D3F12, 0xFF473E00, 0xFF382F00, 0xFF675F38, 0xFF8D8B65, 0xFF838379, 0xFF606372, 0xFF707A7B, 0xFF819282, 0xFF6E7C6F, 0xFF6C766E, 0xFF818783, 0xFF7A807E,
        0xFF67706B, 0xFF6A7770, 0xFF66766C, 0xFF58766E, 0xFF6E7B72, 0xFF7F7664, 0xFF846539, 0xFF8E6418, 0xFFB68924, 0xFFB98D22, 0xFFB88C23, 0xFF976A15, 0xFF423308, 0xFF2A332E, 0xFF2C3739, 0xFF2E3A36, 0xFF283D36, 0xFF30413B, 0xFF373D33, 0xFF313B3A,
        0xFF2F3938, 0xFF323C3B, 0xFF303A39, 0xFF2C3635, 0xFF2C3635, 0xFF2C3635, 0xFF2D3736, 0xFF2E3935, 0xFF313C38, 0xFF2F3938, 0xFF323C3B, 0xFF323C3B, 0xFF2E3839, 0xFF303A3B, 0xFF2B3536, 0xFF2E3837, 0xFF313B3A, 0xFF2D3736, 0xFF2A3433, 0xFF2F3938,
        0xFF2F3938, 0xFF2B3534, 0xFF2D3736, 0xFF293430, 0xFF2C3733, 0xFF2C3733, 0xFF323D37, 0xFF303B35, 0xFF2E3931, 0xFF2B362E, 0xFF303B33, 0xFF2C372F, 0xFF29342C, 0xFF29342C, 0xFF2C3731, 0xFF2B3630, 0xFF2B3632, 0xFF2C3733, 0xFF2A3531, 0xFF253532,
        0xFF28332F, 0xFF2A3330, 0xFF273234, 0xFF283838, 0xFF263226, 0xFF2D2D0B, 0xFF4D400C, 0xFF555400, 0xFFB7BD75, 0xFF8F9D79, 0xFF7D9391, 0xFFBED9EC, 0xFFD4EEFF, 0xFFDBF1FF, 0xFFE2F5FF, 0xFFE2FFFF, 0xFFC2DBFA, 0xFFB6C4E7, 0xFF999EB4, 0xFFB1ADA4,
        0xFF9C9066, 0xFF584608, 0xFF564200, 0xFF5A450E, 0xFF5A460B, 0xFF5A4609, 0xFF594604, 0xFF584503, 0xFF584407, 0xFF584409, 0xFF57420B, 0xFF534A13, 0xFF4F3D0D, 0xFF5D3C06, 0xFF7E5600, 0xFFDAAE35, 0xFFC89A22, 0xFF6C3B00, 0xFF67310B, 0xFF554217,
        0xFF524700, 0xFF584C00, 0xFF5E4300, 0xFF5E3700, 0xFF6A4E05, 0xFF5A5501, 0xFF758431, 0xFF565551, 0xFF020400, 0xFF010703, 0xFF17221E, 0xFF1B2723, 0xFF1B2725, 0xFF212B2C, 0xFF182022, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFF, 0x00FEFCFD, 0x00FEFEFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF1C1E19, 0xFF0A1507, 0xFF020800, 0xFF151611, 0xFF100C0D, 0xFF1D1C1A, 0xFF676C65, 0xFFB8C4B6, 0xFFB7C8B5, 0xFFDAEDDA, 0xFFCEDCDF, 0xFFBEC1D0, 0xFF94918C, 0xFF9F9564,
        0xFFB4A049, 0xFFCFAF4C, 0xFFC49C3C, 0xFFC4AD50, 0xFFB6A434, 0xFFD3C74D, 0xFFAFA543, 0xFFA9A36F, 0xFFBDBDB3, 0xFFA6ADB3, 0xFF9EACAF, 0xFFCEDAD6, 0xFFC4D7C4, 0xFFB2C7A0, 0xFF565D27, 0xFF4D3F02, 0xFF5F4304, 0xFF5A3B00, 0xFF5E4203, 0xFF574308,
        0xFF694312, 0xFF643C08, 0xFF5A4703, 0xFF524403, 0xFF4F3A0D, 0xFF50401C, 0xFF57582C, 0xFF4A5449, 0xFF34432E, 0xFF9CAC88, 0xFF949963, 0xFF514A06, 0xFF594C08, 0xFF8C8648, 0xFF878753, 0xFF848450, 0xFF757039, 0xFF433A00, 0xFF63591E, 0xFFA69F68,
        0xFF8C8954, 0xFF64622F, 0xFF4E4C19, 0xFF464610, 0xFF7F6A29, 0xFFA47621, 0xFFB2780C, 0xFFB57E00, 0xFFC08A0C, 0xFFB77C08, 0xFFBE7911, 0xFFAE7B20, 0xFFB18021, 0xFFB88B26, 0xFFAF8519, 0xFFA97E0E, 0xFFB28414, 0xFFB88617, 0xFFB78115, 0xFFB18600,
        0xFF866000, 0xFF5A3A00, 0xFF5A4518, 0xFF484814, 0xFF404400, 0xFFA08214, 0xFFFFB428, 0xFF805D00, 0xFF5D4800, 0xFF4D4709, 0xFF504D18, 0xFF4E410A, 0xFF533E00, 0xFF594101, 0xFF503D00, 0xFF544202, 0xFF544202, 0xFF534101, 0xFF534101, 0xFF534101,
        0xFF534101, 0xFF544202, 0xFF544202, 0xFF574402, 0xFF574402, 0xFF564301, 0xFF564301, 0xFF564301, 0xFF564301, 0xFF554200, 0xFF554200, 0xFF534700, 0xFF554007, 0xFF5B4106, 0xFF70520C, 0xFF99793C, 0xFF6A4D25, 0xFF664D2F, 0xFF63512B, 0xFF6A4D3F,
        0xFF6F5240, 0xFF684E37, 0xFF684E33, 0xFF5F472B, 0xFF6A5236, 0xFF685036, 0xFF5F4931, 0xFF5D4D2C, 0xFF5E4D33, 0xFF61512F, 0xFF655429, 0xFF5D4A2A, 0xFF6A5245, 0xFF634D38, 0xFF604D22, 0xFF574E25, 0xFF594628, 0xFF573E20, 0xFF5D4722, 0xFF615130,
        0xFF604C34, 0xFF765532, 0xFFA0783D, 0xFF554303, 0xFF554303, 0xFF554303, 0xFF554303, 0xFF554303, 0xFF554303, 0xFF554303, 0xFF554303, 0xFF564210, 0xFF56420F, 0xFF514008, 0xFF504002, 0xFF534401, 0xFF4D4100, 0xFF8E8334, 0xFF9E9343, 0xFF6B641E,
        0xFF524B05, 0xFF4D4400, 0xFF564807, 0xFF534101, 0xFF554104, 0xFF594307, 0xFF553D01, 0xFF644608, 0xFF423A00, 0xFF323309, 0xFF625932, 0xFF9B915C, 0xFF7D826B, 0xFF3B4C54, 0xFF536662, 0xFF869485, 0xFF748273, 0xFF6C7A6B, 0xFF808E81, 0xFF7A887B,
        0xFF748177, 0xFF6E7B71, 0xFF6F7C72, 0xFF747B8D, 0xFF71746B, 0xFF756F4F, 0xFF7E6B40, 0xFF69500D, 0xFF8C7211, 0xFF967B12, 0xFF886E0F, 0xFF5F551A, 0xFF39391D, 0xFF2C3930, 0xFF2E4034, 0xFF2F3A32, 0xFF32373B, 0xFF2E353B, 0xFF2B3A35, 0xFF323D39,
        0xFF323D39, 0xFF313C38, 0xFF303B37, 0xFF2E3935, 0xFF2E3935, 0xFF303B37, 0xFF323D39, 0xFF303C38, 0xFF2F3B37, 0xFF2E3A36, 0xFF2D3935, 0xFF2D3935, 0xFF2D3935, 0xFF2D3935, 0xFF2E3A36, 0xFF2F3B39, 0xFF2D3937, 0xFF2C3836, 0xFF2C3836, 0xFF2E3A38,
        0xFF2E3A38, 0xFF2C3836, 0xFF2A3634, 0xFF2E3A3A, 0xFF2D3939, 0xFF2D3939, 0xFF2C3838, 0xFF2C3838, 0xFF2C3838, 0xFF2C3838, 0xFF2D3939, 0xFF2B3630, 0xFF2D3832, 0xFF2D3832, 0xFF2A3531, 0xFF2A3531, 0xFF2C3635, 0xFF2C3635, 0xFF2A3433, 0xFF293231,
        0xFF2C353C, 0xFF283335, 0xFF2B362E, 0xFF29332B, 0xFF2D2F2C, 0xFF26290E, 0xFF3C4005, 0xFF7E8B2F, 0xFF9AA77B, 0xFF626D69, 0xFFC4C9CC, 0xFFFAFCFB, 0xFFECF3FB, 0xFF7A919F, 0xFF7096A1, 0xFFD2E7EC, 0xFFE6FEFE, 0xFFC2DAE4, 0xFF8397B0, 0xFF717E86,
        0xFFA4A583, 0xFF6E6522, 0xFF574400, 0xFF5B4000, 0xFF5D4404, 0xFF554009, 0xFF574610, 0xFF54450A, 0xFF56460A, 0xFF5F4C11, 0xFF513C03, 0xFF604200, 0xFF5B3A04, 0xFF815C0E, 0xFFD6A92C, 0xFFCB951B, 0xFF764200, 0xFF563B03, 0xFF4C4E00, 0xFF5B4401,
        0xFF564301, 0xFF544504, 0xFF584304, 0xFF604405, 0xFF5C4803, 0xFF5D5A09, 0xFF76842C, 0xFF4E4E44, 0xFF010400, 0xFF000602, 0xFF151F1E, 0xFF1C2828, 0xFF1E2A28, 0xFF25302C, 0xFF19221D, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FEFEFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF1C1E19, 0xFF111B10, 0xFF060B05, 0xFF111111, 0xFF131114, 0xFF1D1B1C, 0xFF686B64, 0xFFBFCABA, 0xFFB2C1AA, 0xFFC8DCD0, 0xFFA8B7BC, 0xFF969FB0, 0xFF868A8B, 0xFF98966F,
        0xFF988F44, 0xFFA28E38, 0xFF957B27, 0xFFAD9161, 0xFF947D39, 0xFF9F8C3D, 0xFF877A36, 0xFF948969, 0xFFA7A49F, 0xFF979D9D, 0xFF9BA89F, 0xFFB5BDBF, 0xFFB0BEB1, 0xFFA5B495, 0xFF575A2D, 0xFF50400C, 0xFF5F400A, 0xFF593A03, 0xFF5E430C, 0xFF5C4A00,
        0xFF654111, 0xFF67401F, 0xFF52400E, 0xFF493F01, 0xFF4C3C08, 0xFF3F3414, 0xFF303521, 0xFF323426, 0xFF5C5C5A, 0xFFCAC6C3, 0xFFB8AD8D, 0xFF8B7833, 0xFFA38F46, 0xFFB9AB84, 0xFF9D9593, 0xFFABAE8F, 0xFF797248, 0xFF534312, 0xFF877B4B, 0xFFC2C097,
        0xFFB3B890, 0xFF787548, 0xFF473C06, 0xFF7A6718, 0xFF967413, 0xFFBD8B18, 0xFFE7B137, 0xFFDAA835, 0xFFC8A02F, 0xFFCDA631, 0xFFD9B135, 0xFFEBA728, 0xFFEAA929, 0xFFDB9F1D, 0xFFE3AA27, 0xFFE9B02F, 0xFFD99F21, 0xFFDD9F26, 0xFFE9A832, 0xFFE4B744,
        0xFFD5B046, 0xFF836205, 0xFF4A2E00, 0xFF574B17, 0xFF454111, 0xFF664700, 0xFFEEAB29, 0xFFAB7A11, 0xFF856208, 0xFF594700, 0xFF483A00, 0xFF4E3C00, 0xFF5A4202, 0xFF5A4504, 0xFF554303, 0xFF554303, 0xFF544202, 0xFF544202, 0xFF534101, 0xFF534101,
        0xFF544202, 0xFF544202, 0xFF554303, 0xFF564301, 0xFF564301, 0xFF564301, 0xFF564301, 0xFF564301, 0xFF554200, 0xFF554200, 0xFF554200, 0xFF4F4200, 0xFF554007, 0xFF533900, 0xFF70520C, 0xFFA28342, 0xFF715428, 0xFF88704E, 0xFFAE9D72, 0xFF6D5B37,
        0xFF605128, 0xFF8F8154, 0xFFB1A470, 0xFFADA368, 0xFFA69C5E, 0xFF847D39, 0xFF9B9450, 0xFF827735, 0xFFAA9E62, 0xFFA3975A, 0xFFB4A665, 0xFF988855, 0xFF644F34, 0xFF715C3F, 0xFFA08E5C, 0xFFA4A15E, 0xFFA2965C, 0xFFA8955D, 0xFFA19153, 0xFFA29761,
        0xFF6E5F38, 0xFF785B2F, 0xFF9E7837, 0xFF554303, 0xFF554303, 0xFF554303, 0xFF554303, 0xFF554303, 0xFF554303, 0xFF554303, 0xFF554303, 0xFF55420A, 0xFF554208, 0xFF5B480D, 0xFF554507, 0xFF524300, 0xFF5B4E08, 0xFF948840, 0xFF85792F, 0xFF57500A,
        0xFF4D4400, 0xFF4F4500, 0xFF584A09, 0xFF574505, 0xFF574306, 0xFF5A4408, 0xFF584004, 0xFF5C4001, 0xFF474303, 0xFF52552A, 0xFF7F764D, 0xFFA0975E, 0xFF898E6E, 0xFF3A4646, 0xFF5A6859, 0xFF7B8783, 0xFF616D69, 0xFF5A6664, 0xFF798583, 0xFF7F8B89,
        0xFF737F7F, 0xFF647070, 0xFF677373, 0xFF5D707E, 0xFF74837C, 0xFF727763, 0xFF736D57, 0xFF62542F, 0xFF5E4F0E, 0xFF4C3B00, 0xFF554305, 0xFF3F3300, 0xFF272611, 0xFF283430, 0xFF2F4038, 0xFF313C34, 0xFF353A3E, 0xFF31383E, 0xFF2C3B36, 0xFF313C38,
        0xFF323D39, 0xFF313C38, 0xFF303B37, 0xFF2E3935, 0xFF2D3834, 0xFF2E3935, 0xFF2F3A36, 0xFF2E3A36, 0xFF2E3A36, 0xFF2D3935, 0xFF2D3935, 0xFF2C3834, 0xFF2C3834, 0xFF2C3834, 0xFF2C3834, 0xFF2E3A38, 0xFF2C3836, 0xFF2B3735, 0xFF2B3735, 0xFF2C3836,
        0xFF2D3937, 0xFF2C3836, 0xFF2B3735, 0xFF2F3B39, 0xFF2D3937, 0xFF2C3836, 0xFF2A3634, 0xFF2A3634, 0xFF2A3634, 0xFF2C3836, 0xFF2C3836, 0xFF2A352F, 0xFF2C3731, 0xFF2D3832, 0xFF2A3531, 0xFF2A3531, 0xFF2C3635, 0xFF2C3635, 0xFF293332, 0xFF29322F,
        0xFF2B343B, 0xFF263133, 0xFF29342C, 0xFF27302B, 0xFF2E302D, 0xFF282A12, 0xFF3F4308, 0xFF8D9947, 0xFF5C6844, 0xFF838E90, 0xFFF2FBFF, 0xFFFBFFFF, 0xFFCFD9E3, 0xFF627C8B, 0xFF5D858F, 0xFF768F94, 0xFFADC9CA, 0xFF9DB8C3, 0xFF71859E, 0xFF5D6A73,
        0xFF8B8F6E, 0xFF918948, 0xFF524100, 0xFF5C4A08, 0xFF564300, 0xFF5B4400, 0xFF604705, 0xFF5B4106, 0xFF4F3802, 0xFF4E3D07, 0xFF5B4E17, 0xFF573E05, 0xFF825F00, 0xFFD9A92D, 0xFFC99734, 0xFF704A01, 0xFF553D00, 0xFF5A4601, 0xFF563D07, 0xFF5B4401,
        0xFF574402, 0xFF544504, 0xFF584304, 0xFF604405, 0xFF5C4803, 0xFF5D5A09, 0xFF76842C, 0xFF505046, 0xFF000200, 0xFF000400, 0xFF182221, 0xFF202C2C, 0xFF1F2B29, 0xFF222D29, 0xFF17201B, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FEFEFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF1A1C17, 0xFF0C130C, 0xFF060A09, 0xFF100F14, 0xFF18151C, 0xFF18181A, 0xFF62655E, 0xFFBFC8B5, 0xFFB4C1A5, 0xFF869689, 0xFF6A787B, 0xFF808998, 0xFF889190, 0xFF6F7253,
        0xFF413F04, 0xFF494000, 0xFF514201, 0xFF4E3403, 0xFF594307, 0xFF4D3900, 0xFF4A390D, 0xFF93857C, 0xFF7F7687, 0xFF3B3C50, 0xFF4E5663, 0xFF8F8F8F, 0xFFA8AEA2, 0xFFAFB89D, 0xFF626039, 0xFF513C0F, 0xFF604011, 0xFF5E3E0F, 0xFF5A400F, 0xFF4E3A00,
        0xFF61420B, 0xFF5C3F15, 0xFF4A3D13, 0xFF80774C, 0xFF918360, 0xFF514B35, 0xFF364437, 0xFF352E24, 0xFF48403D, 0xFF6C5E53, 0xFF60491F, 0xFF664800, 0xFF674C07, 0xFF584831, 0xFF857F97, 0xFF646B63, 0xFF51452B, 0xFF51380F, 0xFF594A23, 0xFF3C4228,
        0xFF39472E, 0xFF443F15, 0xFF6C5011, 0xFF9E6D20, 0xFFCE973C, 0xFFDFA541, 0xFFA67018, 0xFF693E00, 0xFF64470B, 0xFF574600, 0xFF5E5200, 0xFF624D00, 0xFF645102, 0xFF5A4D00, 0xFF524600, 0xFF554B04, 0xFF60530F, 0xFF5E4E10, 0xFF503E00, 0xFF865906,
        0xFFCAA53A, 0xFFD5B232, 0xFF846000, 0xFF533900, 0xFF514223, 0xFF6C4F15, 0xFFCE992F, 0xFFFAC052, 0xFFD5A84A, 0xFF8E7125, 0xFF5D4603, 0xFF5B4200, 0xFF614603, 0xFF574201, 0xFF534305, 0xFF564404, 0xFF554303, 0xFF554303, 0xFF544202, 0xFF544202,
        0xFF554303, 0xFF554303, 0xFF564404, 0xFF564301, 0xFF564301, 0xFF564301, 0xFF554200, 0xFF554200, 0xFF554200, 0xFF554200, 0xFF554200, 0xFF544703, 0xFF5A450C, 0xFF573D02, 0xFF75570F, 0xFF9B7D37, 0xFF634717, 0xFF846D43, 0xFFA99966, 0xFF938A53,
        0xFF8E884E, 0xFF918B4D, 0xFFA5A05C, 0xFFA9A75A, 0xFFC1C169, 0xFFD8DD79, 0xFFEBF089, 0xFFEAE486, 0xFFA39B46, 0xFFB1A752, 0xFFC6BC67, 0xFF79692B, 0xFF5B4722, 0xFF826E4B, 0xFFB09C67, 0xFF8A8D36, 0xFFD4CD7F, 0xFFD8CB7D, 0xFFA59B46, 0xFFC2BC72,
        0xFF837743, 0xFF7A612B, 0xFF9A7830, 0xFF544202, 0xFF544202, 0xFF544202, 0xFF544202, 0xFF544202, 0xFF544202, 0xFF544202, 0xFF544202, 0xFF564203, 0xFF564404, 0xFF5A4808, 0xFF554303, 0xFF524302, 0xFF594A09, 0xFF7A6C2B, 0xFF5D4F0E, 0xFF494000,
        0xFF4D4300, 0xFF534602, 0xFF564706, 0xFF574505, 0xFF584407, 0xFF594307, 0xFF5A4408, 0xFF54390C, 0xFF444212, 0xFF969A7F, 0xFFC3B99E, 0xFF9C9163, 0xFF8A8871, 0xFF595C61, 0xFF74766B, 0xFF828E82, 0xFF707C70, 0xFF6A766A, 0xFF818D83, 0xFF89958B,
        0xFF7C877F, 0xFF6C776F, 0xFF727D75, 0xFF6A8074, 0xFF6D7F67, 0xFF636C51, 0xFF848170, 0xFF9C9279, 0xFF8E8252, 0xFF584C12, 0xFF4C3C09, 0xFF352800, 0xFF29271A, 0xFF303B3D, 0xFF2F3F3C, 0xFF2B3632, 0xFF31363A, 0xFF30383B, 0xFF2A3A30, 0xFF303B37,
        0xFF313C38, 0xFF313C38, 0xFF303B37, 0xFF2E3935, 0xFF2D3834, 0xFF2E3935, 0xFF2F3A36, 0xFF2E3A36, 0xFF2E3A36, 0xFF2E3A36, 0xFF2E3A36, 0xFF2E3A36, 0xFF2D3935, 0xFF2D3935, 0xFF2C3834, 0xFF2C3836, 0xFF2B3735, 0xFF2B3735, 0xFF2B3735, 0xFF2D3937,
        0xFF2E3A38, 0xFF2E3A38, 0xFF2D3937, 0xFF303D36, 0xFF2F3C35, 0xFF2C3932, 0xFF2A3730, 0xFF2A3730, 0xFF2B3831, 0xFF2D3A33, 0xFF2E3B34, 0xFF2B3630, 0xFF2E3933, 0xFF2D3832, 0xFF2B3632, 0xFF2A3531, 0xFF2C3635, 0xFF2B3534, 0xFF283231, 0xFF2A3330,
        0xFF2B3439, 0xFF253032, 0xFF27322C, 0xFF27302B, 0xFF2E302F, 0xFF292B15, 0xFF41440D, 0xFF7E8746, 0xFF5D674F, 0xFF828F98, 0xFFC4D0DC, 0xFFE2ECF5, 0xFFB0BDCD, 0xFF819CAD, 0xFF92BDC4, 0xFF87A3AE, 0xFF78979C, 0xFF75919F, 0xFF677C97, 0xFF4E5A66,
        0xFF696E4E, 0xFF9D995A, 0xFF554804, 0xFF584E13, 0xFF524100, 0xFF624B00, 0xFF634300, 0xFF5E3B03, 0xFF634513, 0xFF544107, 0xFF4B4100, 0xFF825D0F, 0xFFD1AB2C, 0xFFBD9512, 0xFF744F01, 0xFF5A3F10, 0xFF4F4200, 0xFF533E00, 0xFF613D19, 0xFF5B4401,
        0xFF574402, 0xFF554605, 0xFF584304, 0xFF604405, 0xFF5C4803, 0xFF5D5A09, 0xFF76842C, 0xFF5B5B51, 0xFF020500, 0xFF000501, 0xFF172120, 0xFF1F2B2B, 0xFF1F2B29, 0xFF232E2A, 0xFF1D2621, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FEFEFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF181A15, 0xFF090E0A, 0xFF05090A, 0xFF101018, 0xFF17171F, 0xFF1A1A1C, 0xFF5B5D52, 0xFF949A80, 0xFF89916C, 0xFF555940, 0xFF525449, 0xFF858583, 0xFF96968C, 0xFF6D6D4B,
        0xFF46400C, 0xFF53490E, 0xFF564710, 0xFF5A4506, 0xFF5D4904, 0xFF574402, 0xFF564318, 0xFF7C6A5C, 0xFF6B5E65, 0xFF4C4650, 0xFF535458, 0xFF584E42, 0xFF797760, 0xFF8D8E6C, 0xFF5C5529, 0xFF523C0B, 0xFF60400F, 0xFF60400D, 0xFF5C430D, 0xFF5F4816,
        0xFF614504, 0xFF534000, 0xFF524A1B, 0xFFBDB09D, 0xFFE1CDC4, 0xFF686254, 0xFF273D26, 0xFF4C4944, 0xFF29270E, 0xFF69622B, 0xFF958233, 0xFF9D8030, 0xFF8C733A, 0xFF403A24, 0xFF434C51, 0xFF283734, 0xFF5A4E34, 0xFF967748, 0xFF948156, 0xFF525C41,
        0xFF506347, 0xFF5E5721, 0xFFA47E2D, 0xFFC28711, 0xFFE2A83A, 0xFFA97311, 0xFF6C3F00, 0xFF5C3F00, 0xFF594D05, 0xFF5A5907, 0xFF838A2D, 0xFFB5B96F, 0xFFB0B66E, 0xFFC7CD83, 0xFFA9AF65, 0xFF73772A, 0xFF61600E, 0xFF575100, 0xFF594F00, 0xFF603800,
        0xFF7F5C00, 0xFFD0A928, 0xFFDFAF2F, 0xFF875F0A, 0xFF4D360A, 0xFF5C4614, 0xFF7E5908, 0xFF8D5F00, 0xFF86620E, 0xFF654E08, 0xFF4F3B00, 0xFF5A4101, 0xFF614603, 0xFF594401, 0xFF574709, 0xFF574402, 0xFF574402, 0xFF564301, 0xFF564301, 0xFF564301,
        0xFF564301, 0xFF574402, 0xFF574402, 0xFF564301, 0xFF564301, 0xFF564301, 0xFF554200, 0xFF554200, 0xFF554200, 0xFF554200, 0xFF554200, 0xFF4E4100, 0xFF513C05, 0xFF533900, 0xFF76580F, 0xFF9B7D35, 0xFF785D28, 0xFFA48D61, 0xFFA79861, 0xFF6C5E2F,
        0xFF7E7041, 0xFF706432, 0xFF867B45, 0xFFA29A5C, 0xFFCBC47C, 0xFFCCC776, 0xFF918F38, 0xFFDFD484, 0xFFA89C52, 0xFFBEB268, 0xFFC8BA6F, 0xFF86763B, 0xFF6A5333, 0xFF7B6444, 0xFFB7A370, 0xFFA5A55F, 0xFF958B4D, 0xFF9B8C49, 0xFFB3A859, 0xFFBCB571,
        0xFF6D612F, 0xFF715A26, 0xFF9C7935, 0xFF544202, 0xFF544202, 0xFF544202, 0xFF544202, 0xFF544202, 0xFF544202, 0xFF544202, 0xFF544202, 0xFF584500, 0xFF594601, 0xFF544300, 0xFF513F00, 0xFF5B490B, 0xFF4F3F03, 0xFF55450A, 0xFF4B3B00, 0xFF4E4100,
        0xFF564905, 0xFF554603, 0xFF504100, 0xFF554303, 0xFF574306, 0xFF564205, 0xFF5A4408, 0xFF5A4212, 0xFF3B3B09, 0xFFA3AA8B, 0xFFC1BC9C, 0xFF8A814A, 0xFF6D6946, 0xFF626054, 0xFF6D684B, 0xFF666B64, 0xFF676C65, 0xFF666B64, 0xFF656A64, 0xFF666B65,
        0xFF626763, 0xFF5A5F5B, 0xFF646965, 0xFF666C48, 0xFF585E30, 0xFF5E5C35, 0xFF80745A, 0xFF736244, 0xFF645119, 0xFF574300, 0xFF543F00, 0xFF352605, 0xFF2D2A23, 0xFF313B44, 0xFF273737, 0xFF1E2925, 0xFF2A2F33, 0xFF323A3D, 0xFF304035, 0xFF303B37,
        0xFF303B37, 0xFF303B37, 0xFF2E3935, 0xFF2D3834, 0xFF2E3935, 0xFF303B37, 0xFF323D39, 0xFF313C36, 0xFF323D37, 0xFF323D37, 0xFF333E38, 0xFF333E38, 0xFF323D37, 0xFF313C36, 0xFF303B35, 0xFF2C3733, 0xFF2D3834, 0xFF2F3A36, 0xFF303B37, 0xFF323D39,
        0xFF323D39, 0xFF313C38, 0xFF303B37, 0xFF323E34, 0xFF323E34, 0xFF303C32, 0xFF303C32, 0xFF303C32, 0xFF313D33, 0xFF333F35, 0xFF344036, 0xFF313C36, 0xFF323D37, 0xFF2F3A34, 0xFF2B3632, 0xFF293430, 0xFF2B3534, 0xFF2B3534, 0xFF293332, 0xFF2B342F,
        0xFF2B3439, 0xFF253032, 0xFF28332D, 0xFF27302D, 0xFF2E2F31, 0xFF282A15, 0xFF3F420D, 0xFF666836, 0xFF4E5647, 0xFF7E8C99, 0xFF8494A3, 0xFF8F9DAA, 0xFF5E6F83, 0xFF7998AA, 0xFFADDADF, 0xFF8AAAB9, 0xFF42636A, 0xFF7995A3, 0xFF8296B1, 0xFF5C6874,
        0xFF5F6446, 0xFF88864B, 0xFF4C4405, 0xFF58430E, 0xFF574400, 0xFF5C4900, 0xFF5C4300, 0xFF5C410C, 0xFF5E400C, 0xFF624700, 0xFF775F00, 0xFFDDA22A, 0xFFBD962F, 0xFF675200, 0xFF483C00, 0xFF594604, 0xFF5A4005, 0xFF5B410E, 0xFF4F3B09, 0xFF5C4502,
        0xFF574402, 0xFF554605, 0xFF584304, 0xFF604405, 0xFF5B4702, 0xFF5D5A09, 0xFF75832B, 0xFF57574D, 0xFF010400, 0xFF020804, 0xFF151F1E, 0xFF1A2626, 0xFF1D2927, 0xFF212C28, 0xFF1E2722, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FEFEFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF161813, 0xFF0F1410, 0xFF03070A, 0xFF090C13, 0xFF0C0F16, 0xFF191B1A, 0xFF515441, 0xFF4A4E29, 0xFF36390A, 0xFF504118, 0xFF4B3C1B, 0xFF514329, 0xFF504122, 0xFF504116,
        0xFF544109, 0xFF5C460C, 0xFF563B03, 0xFF58410D, 0xFF543C00, 0xFF563E00, 0xFF503A00, 0xFF554311, 0xFF504316, 0xFF4F4C19, 0xFF47480E, 0xFF4B3B19, 0xFF483F16, 0xFF524F1C, 0xFF52470F, 0xFF5E450C, 0xFF5C3C01, 0xFF583C00, 0xFF5D4A08, 0xFF604615,
        0xFF553C00, 0xFF5E4D08, 0xFF463D12, 0xFF6B5A40, 0xFFA68B70, 0xFF5D5137, 0xFF324333, 0xFF3A4139, 0xFF20260A, 0xFF616229, 0xFF68600F, 0xFF5D4B00, 0xFF55470A, 0xFF21250C, 0xFF586C6D, 0xFF2F4337, 0xFF494019, 0xFF6A4A0D, 0xFF634D11, 0xFF474D1F,
        0xFF778456, 0xFF82752F, 0xFFA47A18, 0xFFE3AA1F, 0xFFBF8A22, 0xFF653800, 0xFF604005, 0xFF554300, 0xFF75701E, 0xFFDBDE97, 0xFFF3FBC2, 0xFFEDF4E4, 0xFFF9FFEC, 0xFFF5F9E0, 0xFFF5F6D4, 0xFFFDF7C7, 0xFFD9CB8C, 0xFF927C30, 0xFF5E4400, 0xFF6A4C16,
        0xFF5B3E00, 0xFF7D5300, 0xFFD79F26, 0xFFCE9E30, 0xFF624900, 0xFF514105, 0xFF593F02, 0xFF4F3700, 0xFF4E3F00, 0xFF4D440D, 0xFF4F440E, 0xFF574306, 0xFF594000, 0xFF554000, 0xFF514201, 0xFF574402, 0xFF574402, 0xFF574402, 0xFF564301, 0xFF564301,
        0xFF574402, 0xFF574402, 0xFF574402, 0xFF564301, 0xFF564301, 0xFF564301, 0xFF564301, 0xFF554200, 0xFF554200, 0xFF554200, 0xFF554200, 0xFF544701, 0xFF58430A, 0xFF543A00, 0xFF705209, 0xFF93752F, 0xFF6C5020, 0xFF816A40, 0xFF6C5C29, 0xFF61452F,
        0xFF6A4E38, 0xFF775B45, 0xFF70563D, 0xFF63492E, 0xFF7A6343, 0xFF735C3A, 0xFF634D28, 0xFF68572C, 0xFF83714B, 0xFF766339, 0xFF83713F, 0xFF806A43, 0xFF674E3A, 0xFF664E36, 0xFF7D6738, 0xFF6E6642, 0xFF665539, 0xFF644D2B, 0xFF776535, 0xFF706336,
        0xFF675736, 0xFF7B5F37, 0xFF946E2F, 0xFF544202, 0xFF544202, 0xFF544202, 0xFF544202, 0xFF544202, 0xFF544202, 0xFF544202, 0xFF544202, 0xFF584600, 0xFF574500, 0xFF554200, 0xFF554301, 0xFF5F4D0F, 0xFF534005, 0xFF514008, 0xFF53420A, 0xFF574503,
        0xFF5E4C0A, 0xFF574503, 0xFF503E00, 0xFF554303, 0xFF594508, 0xFF554104, 0xFF564205, 0xFF543D00, 0xFF484B06, 0xFF9EAA78, 0xFFACAA77, 0xFF9C964C, 0xFF5A581F, 0xFF5F5B36, 0xFF6E652E, 0xFF585B2C, 0xFF5B5E2F, 0xFF5E6134, 0xFF54572A, 0xFF575A2F,
        0xFF5A5D32, 0xFF55582D, 0xFF5D5F37, 0xFF59543E, 0xFF4A4726, 0xFF625D40, 0xFF867A6C, 0xFF605039, 0xFF473704, 0xFF523F00, 0xFF4E3900, 0xFF352605, 0xFF2C2922, 0xFF2D3740, 0xFF213131, 0xFF17221E, 0xFF24292D, 0xFF2D3538, 0xFF2C3C31, 0xFF2F3A36,
        0xFF2F3A36, 0xFF2E3935, 0xFF2C3733, 0xFF2C3733, 0xFF2E3935, 0xFF323D39, 0xFF36413D, 0xFF343F39, 0xFF343F39, 0xFF35403A, 0xFF36413B, 0xFF36413B, 0xFF36413B, 0xFF35403A, 0xFF343F39, 0xFF2B3632, 0xFF2E3935, 0xFF323D39, 0xFF35403C, 0xFF35403C,
        0xFF343F3B, 0xFF333E3A, 0xFF313C38, 0xFF323E34, 0xFF333F35, 0xFF344036, 0xFF354137, 0xFF364238, 0xFF374339, 0xFF38443A, 0xFF38443A, 0xFF36413B, 0xFF35403A, 0xFF313C36, 0xFF2A3531, 0xFF28332F, 0xFF2A3433, 0xFF2B3534, 0xFF2A3433, 0xFF2B342F,
        0xFF2C3638, 0xFF273234, 0xFF2B3630, 0xFF2A3332, 0xFF2F3034, 0xFF262715, 0xFF3A3C0B, 0xFF676334, 0xFF363A2B, 0xFF6E7C85, 0xFF657783, 0xFF8999A8, 0xFF5A6C84, 0xFF536F84, 0xFF9ECACD, 0xFF6A8B9C, 0xFF52737C, 0xFF97B1C0, 0xFF7888A1, 0xFF4E5760,
        0xFF5B5E41, 0xFF77753C, 0xFF474103, 0xFF5F3B07, 0xFF5F450A, 0xFF514103, 0xFF54480E, 0xFF52430C, 0xFF4F3400, 0xFF846100, 0xFFCFA628, 0xFFC99118, 0xFF6C4400, 0xFF53410F, 0xFF534C06, 0xFF4D4100, 0xFF5C430D, 0xFF5C420F, 0xFF5B4900, 0xFF5C4502,
        0xFF584503, 0xFF554605, 0xFF584304, 0xFF604405, 0xFF5B4702, 0xFF5C5908, 0xFF75832B, 0xFF46463C, 0xFF000200, 0xFF030905, 0xFF172120, 0xFF1B2727, 0xFF1E2A28, 0xFF1E2925, 0xFF1A231E, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FEFEFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF151712, 0xFF10120D, 0xFF05090A, 0xFF0C1117, 0xFF11161A, 0xFF232822, 0xFF63664B, 0xFF4A4C1B, 0xFF434305, 0xFF583F09, 0xFF58410F, 0xFF523A0C, 0xFF543C0C, 0xFF644B13,
        0xFF5A3C00, 0xFF5B3800, 0xFF734D0E, 0xFF7C6320, 0xFF81661F, 0xFF7A5F12, 0xFF654B00, 0xFF604800, 0xFF503D00, 0xFF504300, 0xFF534A05, 0xFF5F4514, 0xFF4F3E08, 0xFF4B4106, 0xFF524206, 0xFF5E4105, 0xFF583900, 0xFF573C00, 0xFF5C4A02, 0xFF5D4401,
        0xFF563C00, 0xFF58451B, 0xFF4B3C1B, 0xFF6B5315, 0xFFBC9A43, 0xFF644F1A, 0xFF33323A, 0xFF2C382A, 0xFF343E36, 0xFFBABCAF, 0xFFB3AC7E, 0xFF5D5002, 0xFF4D4300, 0xFF3B3D25, 0xFF4A5569, 0xFF3C4F33, 0xFF453F0D, 0xFF5F4100, 0xFF5A3E00, 0xFF4F4402,
        0xFF8F8A4A, 0xFFA38E3D, 0xFFA27A18, 0xFFE2A928, 0xFFA0701C, 0xFF4F2C04, 0xFF513D18, 0xFF897F41, 0xFFD7D48D, 0xFFE1E2BA, 0xFFA6A7A2, 0xFFADD0D4, 0xFFC2E4E6, 0xFFCAE9E4, 0xFFE0FBEC, 0xFFCEE3C2, 0xFFD1DEA8, 0xFFE4EAA0, 0xFF939541, 0xFF534504,
        0xFF5E4E13, 0xFF5F3A00, 0xFFAD740B, 0xFFE2AE34, 0xFF6F5400, 0xFF574900, 0xFF54400B, 0xFF54480B, 0xFF48430C, 0xFF484616, 0xFF524815, 0xFF554106, 0xFF594000, 0xFF5B4702, 0xFF574807, 0xFF584300, 0xFF584300, 0xFF574200, 0xFF574200, 0xFF574200,
        0xFF574200, 0xFF584300, 0xFF584300, 0xFF574402, 0xFF574402, 0xFF574402, 0xFF564301, 0xFF564301, 0xFF564301, 0xFF564301, 0xFF554200, 0xFF4E4200, 0xFF5C480D, 0xFF573D02, 0xFF72540C, 0xFFA28342, 0xFF6D5026, 0xFF6C5334, 0xFF64522A, 0xFF795650,
        0xFF6E4C43, 0xFF654435, 0xFF765743, 0xFF6B4C38, 0xFF6B4E3C, 0xFF785B4D, 0xFF73554A, 0xFF654E3E, 0xFF6D5549, 0xFF5D4730, 0xFF644D2B, 0xFF654D31, 0xFF705647, 0xFF70563F, 0xFF604A19, 0xFF5B4C37, 0xFF70584E, 0xFF67493E, 0xFF6A5037, 0xFF614D34,
        0xFF695240, 0xFF765431, 0xFFA77E40, 0xFF554303, 0xFF554303, 0xFF554303, 0xFF554303, 0xFF554303, 0xFF554303, 0xFF554303, 0xFF554303, 0xFF5A4601, 0xFF564200, 0xFF5C4905, 0xFF584503, 0xFF554305, 0xFF58450A, 0xFF5C490F, 0xFF55450A, 0xFF584503,
        0xFF5C4907, 0xFF584503, 0xFF544001, 0xFF5A4607, 0xFF5B490B, 0xFF554305, 0xFF534103, 0xFF5B3B0A, 0xFF4C4614, 0xFF848B6C, 0xFF858062, 0xFF7F7540, 0xFF3D3812, 0xFF58513E, 0xFF5F512A, 0xFF4A3E40, 0xFF413537, 0xFF473B3D, 0xFF413539, 0xFF473B3F,
        0xFF463940, 0xFF3E3138, 0xFF42353C, 0xFF514D4E, 0xFF3C3930, 0xFF3B3B33, 0xFF5C575B, 0xFF555148, 0xFF4B4521, 0xFF544A15, 0xFF4A3E0E, 0xFF3A2D03, 0xFF2C2A1D, 0xFF2D383A, 0xFF293936, 0xFF232E2A, 0xFF2A2F33, 0xFF2B3336, 0xFF27372D, 0xFF2E3935,
        0xFF2D3834, 0xFF2C3733, 0xFF2B3632, 0xFF2B3632, 0xFF2D3834, 0xFF313C38, 0xFF35403C, 0xFF333E36, 0xFF333E36, 0xFF343F37, 0xFF343F37, 0xFF343F37, 0xFF343F37, 0xFF343F37, 0xFF343F37, 0xFF2A352F, 0xFF2D3832, 0xFF323D37, 0xFF343F39, 0xFF35403A,
        0xFF333E38, 0xFF313C36, 0xFF303B35, 0xFF303B33, 0xFF313C34, 0xFF333E36, 0xFF354038, 0xFF37423A, 0xFF37423A, 0xFF37423A, 0xFF37423A, 0xFF36413B, 0xFF35403A, 0xFF303B35, 0xFF2A3531, 0xFF27322E, 0xFF2A3433, 0xFF2B3534, 0xFF2A3433, 0xFF2A342C,
        0xFF2C3638, 0xFF283335, 0xFF2D3832, 0xFF2C3534, 0xFF2F3035, 0xFF242517, 0xFF363809, 0xFF655927, 0xFF45442F, 0xFF414D4B, 0xFF3D4F53, 0xFF90A0AD, 0xFF96A6C0, 0xFF809BAE, 0xFF97BFBF, 0xFF608192, 0xFF75939B, 0xFF748A97, 0xFF444F65, 0xFF3F4249,
        0xFF605F40, 0xFF726D33, 0xFF4B4305, 0xFF63400A, 0xFF593E09, 0xFF53420C, 0xFF50440A, 0xFF4A3C00, 0xFF82670E, 0xFFCAA034, 0xFFC4901A, 0xFF684B00, 0xFF5A3700, 0xFF5F3D00, 0xFF584407, 0xFF4C460A, 0xFF4C470D, 0xFF574400, 0xFF6A4A00, 0xFF5D4603,
        0xFF584503, 0xFF554605, 0xFF584304, 0xFF604405, 0xFF5B4702, 0xFF5C5908, 0xFF74822A, 0xFF4D4D43, 0xFF000200, 0xFF040A06, 0xFF182221, 0xFF1E2A2A, 0xFF24302E, 0xFF25302C, 0xFF252E29, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FEFEFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF131510, 0xFF0E0F07, 0xFF080D09, 0xFF0C1114, 0xFF0D1517, 0xFF12180C, 0xFF515532, 0xFF48470E, 0xFF4A4800, 0xFF564006, 0xFF533D03, 0xFF533D03, 0xFF553F05, 0xFF583E01,
        0xFF5F4000, 0xFF805918, 0xFFA57938, 0xFF927531, 0xFF8F7129, 0xFFA58538, 0xFFA07F2E, 0xFF836318, 0xFF5F4200, 0xFF5A410B, 0xFF563E10, 0xFF5D3C09, 0xFF5C430D, 0xFF55440E, 0xFF523D06, 0xFF573802, 0xFF5C3903, 0xFF5D4306, 0xFF5A4806, 0xFF583D00,
        0xFF644911, 0xFF58411F, 0xFF4F3B18, 0xFF725300, 0xFFD6AB29, 0xFF6F4B00, 0xFF3E2C38, 0xFF282B20, 0xFF585C5D, 0xFFB4B4B4, 0xFF9D9477, 0xFF897735, 0xFF91803A, 0xFF7D734F, 0xFF635F60, 0xFF5B6B47, 0xFF76703C, 0xFF997E37, 0xFFA07D2C, 0xFF977828,
        0xFFA58B3E, 0xFF9D8030, 0xFF85620E, 0xFFD79A0C, 0xFFB5891E, 0xFF5B4500, 0xFF605C13, 0xFFC4C874, 0xFFCACE82, 0xFF828864, 0xFF9EA6A8, 0xFFBBC8D8, 0xFFF0FFFF, 0xFFD6E6F5, 0xFFDDEBF4, 0xFFF4FFFE, 0xFFCED5C3, 0xFFC8CCA9, 0xFFDCDEAF, 0xFF717423,
        0xFF4A4606, 0xFF5A3E00, 0xFFA97416, 0xFFE7B23E, 0xFF684C00, 0xFF544400, 0xFF514008, 0xFF564100, 0xFF4E3E03, 0xFF4E430D, 0xFF56450D, 0xFF5A4101, 0xFF5C3F00, 0xFF574300, 0xFF4F4402, 0xFF584300, 0xFF584300, 0xFF574200, 0xFF574200, 0xFF574200,
        0xFF574200, 0xFF584300, 0xFF584300, 0xFF584503, 0xFF574402, 0xFF574402, 0xFF574402, 0xFF574402, 0xFF564301, 0xFF564301, 0xFF564301, 0xFF504400, 0xFF5B470A, 0xFF553B00, 0xFF6A4B08, 0xFF9D7D42, 0xFF6C4E2C, 0xFF6B503B, 0xFF725E43, 0xFF65493E,
        0xFF755A45, 0xFF715935, 0xFFA28E5C, 0xFF8C7843, 0xFF63511F, 0xFF725F37, 0xFF5C4827, 0xFF624C34, 0xFF654F3A, 0xFF705B3C, 0xFF857045, 0xFF78623B, 0xFF664C35, 0xFF6C5432, 0xFF9D894E, 0xFF918055, 0xFF684E33, 0xFF6F4E3B, 0xFF775941, 0xFF644B35,
        0xFF745745, 0xFF8B653E, 0xFF996D26, 0xFF564404, 0xFF564404, 0xFF564404, 0xFF564404, 0xFF564404, 0xFF564404, 0xFF564404, 0xFF564404, 0xFF5A4504, 0xFF594403, 0xFF604B0A, 0xFF544001, 0xFF513D00, 0xFF594709, 0xFF574507, 0xFF544204, 0xFF574201,
        0xFF5A4504, 0xFF594403, 0xFF574304, 0xFF594506, 0xFF584608, 0xFF554305, 0xFF544204, 0xFF663C14, 0xFF473912, 0xFF636452, 0xFF716A58, 0xFF696037, 0xFF453F27, 0xFF473E37, 0xFF291B00, 0xFF2B1807, 0xFF281504, 0xFF392615, 0xFF2B1809, 0xFF271405,
        0xFF261305, 0xFF241103, 0xFF2A1709, 0xFF403523, 0xFF494323, 0xFF484526, 0xFF56543F, 0xFF545033, 0xFF423C08, 0xFF483E00, 0xFF4E3F08, 0xFF443804, 0xFF2B2A15, 0xFF293531, 0xFF2D3E36, 0xFF2E3931, 0xFF33383C, 0xFF30373D, 0xFF2C3B36, 0xFF2B3632,
        0xFF2B3632, 0xFF2B3632, 0xFF2B3632, 0xFF2A3531, 0xFF2A3531, 0xFF2C3733, 0xFF2E3935, 0xFF2E3830, 0xFF2E3830, 0xFF2E3830, 0xFF2E3830, 0xFF2F3931, 0xFF2F3931, 0xFF303A32, 0xFF303A32, 0xFF29322D, 0xFF2B342F, 0xFF2F3833, 0xFF303934, 0xFF303934,
        0xFF2E3732, 0xFF2E3732, 0xFF2F3833, 0xFF2E3732, 0xFF2F3833, 0xFF2F3833, 0xFF303934, 0xFF313A35, 0xFF313A35, 0xFF313A35, 0xFF313A35, 0xFF303B35, 0xFF303B35, 0xFF2D3832, 0xFF293430, 0xFF28332F, 0xFF2A3433, 0xFF2B3534, 0xFF293332, 0xFF29332A,
        0xFF2B3536, 0xFF273234, 0xFF2D3832, 0xFF2D3537, 0xFF303038, 0xFF242418, 0xFF36370B, 0xFF534006, 0xFF4D4725, 0xFF2A3525, 0xFF263731, 0xFF414F58, 0xFF75839E, 0xFF94AABF, 0xFF6E9290, 0xFF59788A, 0xFF58737A, 0xFF344650, 0xFF3F4457, 0xFF59575A,
        0xFF6D6946, 0xFF5E581C, 0xFF483E00, 0xFF554509, 0xFF54400B, 0xFF5D440C, 0xFF624700, 0xFF7E6100, 0xFFC09E30, 0xFFC59A3B, 0xFF764700, 0xFF4F4507, 0xFF5A4601, 0xFF5C3E00, 0xFF5D410F, 0xFF4D3E07, 0xFF4C4400, 0xFF605002, 0xFF533600, 0xFF5D4603,
        0xFF584503, 0xFF564706, 0xFF584304, 0xFF604405, 0xFF5B4702, 0xFF5C5908, 0xFF74822A, 0xFF68685E, 0xFF000300, 0xFF010703, 0xFF16201F, 0xFF202C2C, 0xFF24302E, 0xFF242F2B, 0xFF2C3530, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FEFEFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF131510, 0xFF0F1106, 0xFF0B100A, 0xFF0B1111, 0xFF151E1D, 0xFF131A0A, 0xFF4E532B, 0xFF4E4F0D, 0xFF484500, 0xFF594810, 0xFF4F3F04, 0xFF57470B, 0xFF524002, 0xFF523A00,
        0xFF86682A, 0xFFA27C3D, 0xFF79500E, 0xFF734A34, 0xFF724929, 0xFF6C4413, 0xFF7E5716, 0xFFAC883C, 0xFF85631B, 0xFF533600, 0xFF61440C, 0xFF633D10, 0xFF5C3D0E, 0xFF533D0C, 0xFF5A4214, 0xFF603F14, 0xFF603B0E, 0xFF5F4311, 0xFF57460E, 0xFF5F3F0E,
        0xFF5F4208, 0xFF513D0A, 0xFF56410C, 0xFF6C4600, 0xFFE7B12B, 0xFF7E5000, 0xFF2D140D, 0xFF25201D, 0xFF211E15, 0xFF47452C, 0xFF4D4215, 0xFF5F4910, 0xFF5F4208, 0xFF695220, 0xFF5A4B20, 0xFF47512C, 0xFF625E2E, 0xFF705715, 0xFF754C00, 0xFF805300,
        0xFF784F00, 0xFF6F4B01, 0xFF644600, 0xFFECA533, 0xFFA57513, 0xFF4E3C00, 0xFFA5A55D, 0xFFBDC383, 0xFF757A52, 0xFF8D9693, 0xFFBCC7E3, 0xFFE4E4DC, 0xFFF5F7F4, 0xFFF3F8FB, 0xFFE7F0F5, 0xFFF8FFFF, 0xFFE7F1E8, 0xFFB6BFAA, 0xFFB0B99A, 0xFFAFBA5C,
        0xFF434A00, 0xFF5A4800, 0xFFA27115, 0xFFE0AB43, 0xFF6A4900, 0xFF554400, 0xFF534307, 0xFF633E00, 0xFF614300, 0xFF5E4505, 0xFF5E4203, 0xFF603F00, 0xFF5E3D00, 0xFF574300, 0xFF514807, 0xFF584300, 0xFF574200, 0xFF574200, 0xFF564100, 0xFF564100,
        0xFF574200, 0xFF574200, 0xFF584300, 0xFF584503, 0xFF584503, 0xFF584503, 0xFF574402, 0xFF574402, 0xFF574402, 0xFF574402, 0xFF574402, 0xFF4E4300, 0xFF544001, 0xFF5C4207, 0xFF6F500F, 0xFF997842, 0xFF705135, 0xFF6E5246, 0xFF685340, 0xFF65533B,
        0xFF62532C, 0xFF817538, 0xFFC4BD6C, 0xFFC7C267, 0xFFB2AE57, 0xFFAAA456, 0xFFA19A54, 0xFF9A875C, 0xFF715E36, 0xFF5D4819, 0xFFAD9A5F, 0xFF9F8957, 0xFF5A421E, 0xFF816A3E, 0xFFBEA966, 0xFFBEB067, 0xFF715724, 0xFF72502B, 0xFF704F2E, 0xFF6E5338,
        0xFF684A32, 0xFF855C2E, 0xFFA47525, 0xFF564404, 0xFF564404, 0xFF564404, 0xFF564404, 0xFF564404, 0xFF564404, 0xFF564404, 0xFF564404, 0xFF594307, 0xFF5F490D, 0xFF5D4809, 0xFF513D00, 0xFF5B4708, 0xFF5E4A0B, 0xFF4E3C00, 0xFF5A4806, 0xFF5B4303,
        0xFF5B4303, 0xFF594403, 0xFF584304, 0xFF544001, 0xFF534103, 0xFF554305, 0xFF564406, 0xFF653B00, 0xFF524600, 0xFF424613, 0xFF413F0E, 0xFF413E00, 0xFF46470E, 0xFF403E18, 0xFF443E02, 0xFF493800, 0xFF5A4903, 0xFF7A6924, 0xFF5C4B06, 0xFF483700,
        0xFF4F3D00, 0xFF5E4C0A, 0xFF695715, 0xFF6D5517, 0xFF715F13, 0xFF5F5405, 0xFF675C1A, 0xFF716720, 0xFF6D5F06, 0xFF766305, 0xFF7A600C, 0xFF594F14, 0xFF323216, 0xFF253229, 0xFF283A2E, 0xFF29342C, 0xFF2B3034, 0xFF272E34, 0xFF25342F, 0xFF28332F,
        0xFF2A3531, 0xFF2B3632, 0xFF2B3632, 0xFF293430, 0xFF28332F, 0xFF27322E, 0xFF28332F, 0xFF29332B, 0xFF29332B, 0xFF28322A, 0xFF28322A, 0xFF28322A, 0xFF29332B, 0xFF2B352D, 0xFF2C362E, 0xFF27302B, 0xFF29322D, 0xFF2B342F, 0xFF2B342F, 0xFF2A332E,
        0xFF29322D, 0xFF2B342F, 0xFF2C3530, 0xFF2C3532, 0xFF2B3431, 0xFF2B3431, 0xFF2A3330, 0xFF2A3330, 0xFF2A3330, 0xFF2B3431, 0xFF2B3431, 0xFF29342E, 0xFF2B3630, 0xFF2B3630, 0xFF28332F, 0xFF28332F, 0xFF2A3433, 0xFF2A3433, 0xFF283231, 0xFF29332A,
        0xFF2A3435, 0xFF263133, 0xFF2C3733, 0xFF2D3537, 0xFF31313B, 0xFF26261A, 0xFF38390F, 0xFF4D3800, 0xFF534A21, 0xFF313922, 0xFF263629, 0xFF29363E, 0xFF4A5670, 0xFF3B5063, 0xFF476764, 0xFF819EAE, 0xFF526B70, 0xFF324148, 0xFF606271, 0xFF635D5D,
        0xFF635B36, 0xFF453B00, 0xFF4F4306, 0xFF404503, 0xFF5B5018, 0xFF5E3E01, 0xFF805300, 0xFFD1A427, 0xFFC2981F, 0xFF6A4400, 0xFF5E3813, 0xFF564B00, 0xFF483F04, 0xFF54491C, 0xFF523E0C, 0xFF614000, 0xFF6D4D00, 0xFF564200, 0xFF584E19, 0xFF5D4603,
        0xFF584503, 0xFF564706, 0xFF584304, 0xFF604405, 0xFF5A4601, 0xFF5B5807, 0xFF74822A, 0xFF76766C, 0xFF000300, 0xFF000200, 0xFF151F1E, 0xFF202C2C, 0xFF1D2927, 0xFF17221E, 0xFF202924, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FEFEFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF161813, 0xFF0B0D0C, 0xFF070A01, 0xFF0E130F, 0xFF14171E, 0xFF1A1D16, 0xFF5F6237, 0xFF4F500E, 0xFF4A4704, 0xFF5C420F, 0xFF57400A, 0xFF563A0B, 0xFF5C350A, 0xFF7F5312,
        0xFFA27D24, 0xFF644D07, 0xFF66563C, 0xFFA68B5C, 0xFFC4B679, 0xFFABA767, 0xFF6B6130, 0xFF685022, 0xFFA28043, 0xFF765311, 0xFF5F3C04, 0xFF5D420A, 0xFF5C4109, 0xFF5B4008, 0xFF5A3F07, 0xFF5A3F07, 0xFF5C4109, 0xFF5E430B, 0xFF60450D, 0xFF614611,
        0xFF5A430D, 0xFF58430E, 0xFF553F05, 0xFF624300, 0xFFB28836, 0xFFDBA140, 0xFF874400, 0xFF383000, 0xFF413900, 0xFF4A4200, 0xFF4F4700, 0xFF4E4600, 0xFF4D4500, 0xFF4E4600, 0xFF504800, 0xFF5B4907, 0xFF544502, 0xFF4D4500, 0xFF524C00, 0xFF524B00,
        0xFF514100, 0xFF695307, 0xFF92772A, 0xFFE9BD40, 0xFF83600E, 0xFF614F11, 0xFFCBC77E, 0xFF7B7948, 0xFF615E65, 0xFFACB6CF, 0xFF7C9697, 0xFF4B726D, 0xFF698A93, 0xFF90A8C0, 0xFF74869C, 0xFFD4E5ED, 0xFFF4FFFF, 0xFFAFBCC2, 0xFF788492, 0xFFAAA969,
        0xFF625C10, 0xFF5D4600, 0xFF9E720F, 0xFFD8A235, 0xFF744300, 0xFF543F12, 0xFF3E4B00, 0xFF584304, 0xFF564102, 0xFF574203, 0xFF584304, 0xFF574203, 0xFF554001, 0xFF564102, 0xFF594405, 0xFF5E3F06, 0xFF5E3F06, 0xFF5E3F06, 0xFF5E3F06, 0xFF5E3F06,
        0xFF5E3F06, 0xFF5E3F06, 0xFF5E3F06, 0xFF5D4102, 0xFF5E4203, 0xFF5E4203, 0xFF5F4304, 0xFF5F4304, 0xFF5E4203, 0xFF5E4203, 0xFF5D4102, 0xFF504300, 0xFF5A4800, 0xFF5C4100, 0xFF6E4D08, 0xFF9C7C41, 0xFF61491B, 0xFF6A5A39, 0xFF635940, 0xFF63533C,
        0xFF645633, 0xFF80753F, 0xFFBEB86E, 0xFFD3CF76, 0xFFEBEA8D, 0xFFD3D275, 0xFFAFAE52, 0xFFB5A874, 0xFF655824, 0xFF5F521D, 0xFF988C52, 0xFFA09457, 0xFF968B4B, 0xFF807533, 0xFFB5AB66, 0xFF9D8E57, 0xFF6C592F, 0xFF664E36, 0xFF6A4C41, 0xFF6D4D3E,
        0xFF745332, 0xFF785922, 0xFF977931, 0xFF544605, 0xFF534504, 0xFF534504, 0xFF524403, 0xFF524403, 0xFF524403, 0xFF524403, 0xFF534504, 0xFF594405, 0xFF594405, 0xFF594405, 0xFF594405, 0xFF594405, 0xFF594405, 0xFF594405, 0xFF594405, 0xFF5A4202,
        0xFF5A4202, 0xFF5A4204, 0xFF5A4204, 0xFF5A4204, 0xFF5A4206, 0xFF5A4206, 0xFF5A4206, 0xFF504106, 0xFF504106, 0xFF504106, 0xFF4F4005, 0xFF4E3F04, 0xFF4D3E03, 0xFF4C3D02, 0xFF4B3C01, 0xFF584407, 0xFF4E3B00, 0xFF897830, 0xFF635411, 0xFF463700,
        0xFF908146, 0xFF72641D, 0xFF645804, 0xFF775D12, 0xFF755B10, 0xFF73590E, 0xFF755B10, 0xFF785E13, 0xFF795F14, 0xFF765C11, 0xFF745A0F, 0xFF453B17, 0xFF201C00, 0xFF343827, 0xFF16231A, 0xFF293936, 0xFF14201E, 0xFF373C36, 0xFF1F2018, 0xFF2D3828,
        0xFF1B2316, 0xFF363B34, 0xFF1C1E19, 0xFF2D2F2C, 0xFF161D16, 0xFF2F3B31, 0xFF17241A, 0xFF2E352D, 0xFF1B241F, 0xFF2A3531, 0xFF202E21, 0xFF1F2C18, 0xFF232C17, 0xFF282A1D, 0xFF2F2E29, 0xFF1F271A, 0xFF262E21, 0xFF1A2215, 0xFF242C21, 0xFF1F271C,
        0xFF2F362E, 0xFF141B13, 0xFF343B33, 0xFF131C19, 0xFF2E3734, 0xFF101916, 0xFF2D3633, 0xFF1A2320, 0xFF29322F, 0xFF1C2522, 0xFF27302D, 0xFF1A261A, 0xFF253125, 0xFF1F2B1F, 0xFF243024, 0xFF222E22, 0xFF1B271B, 0xFF232F23, 0xFF1A261A, 0xFF2A2E20,
        0xFF1E261B, 0xFF202F2A, 0xFF1A2A27, 0xFF25322B, 0xFF25291B, 0xFF282009, 0xFF4A3B1E, 0xFF3C3904, 0xFF444211, 0xFF43411B, 0xFF37371D, 0xFF2D2F24, 0xFF292D2E, 0xFF212733, 0xFF161D2D, 0xFF141E28, 0xFF171E31, 0xFF31384B, 0xFF5F6460, 0xFF5D5F37,
        0xFF444100, 0xFF4D4100, 0xFF544100, 0xFF5B4410, 0xFF534000, 0xFF7A5800, 0xFFD99937, 0xFFCE9029, 0xFF684F00, 0xFF494300, 0xFF524707, 0xFF544005, 0xFF554104, 0xFF574306, 0xFF584407, 0xFF594506, 0xFF594506, 0xFF594604, 0xFF584503, 0xFF564300,
        0xFF534401, 0xFF544605, 0xFF584405, 0xFF5E4505, 0xFF5A4601, 0xFF585504, 0xFF707D25, 0xFF50534C, 0xFF050801, 0xFF000300, 0xFF131915, 0xFF2D3633, 0xFF212C28, 0xFF212D2B, 0xFF212D2D, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FEFEFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF151712, 0xFF101211, 0xFF090C03, 0xFF0B100C, 0xFF13161D, 0xFF1C1F18, 0xFF64673C, 0xFF535412, 0xFF4B4805, 0xFF583D08, 0xFF58410F, 0xFF5E450F, 0xFF583600, 0xFF8D621E,
        0xFF8D6327, 0xFF644914, 0xFF9D8F5E, 0xFFFFFA9F, 0xFFFFFF9E, 0xFFF7FB9D, 0xFFBAB571, 0xFF715B22, 0xFF8D6B2B, 0xFF84611F, 0xFF583500, 0xFF5B4008, 0xFF5B4008, 0xFF5C4109, 0xFF5C4109, 0xFF5C4109, 0xFF5D420A, 0xFF5D420A, 0xFF5D420A, 0xFF5A3D05,
        0xFF5A410B, 0xFF56410A, 0xFF5C470E, 0xFF573B00, 0xFF6F4A00, 0xFFBB8930, 0xFFE5AD4C, 0xFFCB8F21, 0xFFD09426, 0xFFD69A2C, 0xFFD79B2D, 0xFFD5992B, 0xFFD39729, 0xFFD4982A, 0xFFD69A2C, 0xFFD19527, 0xFFD1992A, 0xFFCE9B28, 0xFFCD9A27, 0xFFD39E2A,
        0xFFDBA22F, 0xFFE4A533, 0xFFEDA737, 0xFF93650D, 0xFF583900, 0xFF796B24, 0xFFCECA8D, 0xFF46421F, 0xFF5A5B56, 0xFFB3C0C8, 0xFF617D7E, 0xFF5D8585, 0xFF85A7B3, 0xFF839FB4, 0xFF576E80, 0xFF82959C, 0xFFBBCDCF, 0xFF91A0A7, 0xFF5D6979, 0xFF82804D,
        0xFF605B19, 0xFF4E3A00, 0xFFA78023, 0xFFEBBB3D, 0xFF794C00, 0xFF5C3F03, 0xFF4E4607, 0xFF5C4100, 0xFF5C4100, 0xFF5B4000, 0xFF5E4300, 0xFF614601, 0xFF5B4000, 0xFF583D00, 0xFF5E4300, 0xFF544000, 0xFF544000, 0xFF544000, 0xFF544000, 0xFF544000,
        0xFF544000, 0xFF544000, 0xFF544000, 0xFF534100, 0xFF534100, 0xFF544200, 0xFF544200, 0xFF544200, 0xFF544200, 0xFF534100, 0xFF534100, 0xFF524405, 0xFF574304, 0xFF593D00, 0xFF714F10, 0xFFA38048, 0xFF6C4D21, 0xFF6F5637, 0xFF655138, 0xFF705449,
        0xFF69503C, 0xFF6A5333, 0xFF836E41, 0xFF7A6935, 0xFF897844, 0xFF7C6C3B, 0xFF6D5C30, 0xFF796240, 0xFF5A441F, 0xFF634D26, 0xFF887249, 0xFF877245, 0xFF877243, 0xFF735F2D, 0xFF8B7744, 0xFF826E39, 0xFF675026, 0xFF694D35, 0xFF6E4E3F, 0xFF6E4B38,
        0xFF72502D, 0xFF7E5D27, 0xFF9C7B36, 0xFF554700, 0xFF544600, 0xFF544600, 0xFF544600, 0xFF544600, 0xFF554700, 0xFF564801, 0xFF564801, 0xFF544600, 0xFF544600, 0xFF544600, 0xFF544600, 0xFF544600, 0xFF544600, 0xFF544600, 0xFF544600, 0xFF534500,
        0xFF534500, 0xFF534500, 0xFF534500, 0xFF534500, 0xFF534400, 0xFF534400, 0xFF534400, 0xFF534403, 0xFF544504, 0xFF534403, 0xFF534403, 0xFF524302, 0xFF514201, 0xFF4F4000, 0xFF4F4000, 0xFF4E3C00, 0xFF493A03, 0xFF81754F, 0xFF8A7F6D, 0xFF50433A,
        0xFF8E826A, 0xFF5C5119, 0xFF625A05, 0xFF524707, 0xFF4F4404, 0xFF4D4202, 0xFF4D4202, 0xFF4F4404, 0xFF504505, 0xFF4F4404, 0xFF4D4202, 0xFF4C3D06, 0xFF352D00, 0xFF6E6F47, 0xFF495331, 0xFF616C4E, 0xFF434A2B, 0xFF6A6B49, 0xFF383610, 0xFF646F4F,
        0xFF3C4526, 0xFF5F6444, 0xFF414222, 0xFF6E6F4F, 0xFF434424, 0xFF606644, 0xFF404725, 0xFF5B594A, 0xFF4F503E, 0xFF565B45, 0xFF474F3A, 0xFF4C5341, 0xFF4D5140, 0xFF4B4C3A, 0xFF56543F, 0xFF4B4D35, 0xFF5F6149, 0xFF3B3D25, 0xFF666852, 0xFF33351F,
        0xFF656752, 0xFF363823, 0xFF646651, 0xFF3C4123, 0xFF5F6446, 0xFF3B4022, 0xFF63684A, 0xFF3C4123, 0xFF575C3E, 0xFF43482A, 0xFF585D3F, 0xFF484C35, 0xFF52563F, 0xFF4B4F38, 0xFF50543D, 0xFF4F533C, 0xFF494D36, 0xFF50543D, 0xFF474B34, 0xFF60603C,
        0xFF393D1C, 0xFF555E41, 0xFF2D371C, 0xFF666D4E, 0xFF32300A, 0xFF43380B, 0xFF493705, 0xFF463D04, 0xFF453E0A, 0xFF423B0F, 0xFF3B3714, 0xFF323019, 0xFF28281C, 0xFF21221D, 0xFF1C1E1D, 0xFF202014, 0xFF2B271B, 0xFF504533, 0xFF60502F, 0xFF54400D,
        0xFF4F3D00, 0xFF534307, 0xFF53420A, 0xFF563E00, 0xFF7F5F08, 0xFFCFA736, 0xFFC0962A, 0xFF744A00, 0xFF5C3800, 0xFF584407, 0xFF4A4600, 0xFF544003, 0xFF554104, 0xFF564205, 0xFF584405, 0xFF584405, 0xFF584503, 0xFF584503, 0xFF584503, 0xFF584501,
        0xFF544502, 0xFF564807, 0xFF594506, 0xFF604707, 0xFF5C4803, 0xFF5B5807, 0xFF738028, 0xFF43463F, 0xFF000200, 0xFF050A04, 0xFF141A16, 0xFF222B28, 0xFF222D29, 0xFF364240, 0xFF3D4949, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FEFEFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF151712, 0xFF0C0E0D, 0xFF0B0E05, 0xFF0F1410, 0xFF13161D, 0xFF141710, 0xFF585B30, 0xFF4D4E0C, 0xFF4C4906, 0xFF573D02, 0xFF543D0B, 0xFF5C460A, 0xFF543700, 0xFF9C742F,
        0xFF815331, 0xFF6E4E28, 0xFFDFD28E, 0xFFFFFD80, 0xFFECF371, 0xFFFDFF95, 0xFFDFDC8D, 0xFF735B1D, 0xFF7F5C1A, 0xFF987531, 0xFF604005, 0xFF5A3F07, 0xFF5B4008, 0xFF5D420A, 0xFF5E430B, 0xFF5E430B, 0xFF5D420A, 0xFF5C4109, 0xFF5B4008, 0xFF604108,
        0xFF5D420A, 0xFF4D3801, 0xFF59440D, 0xFF5B4509, 0xFF513400, 0xFF744F01, 0xFFA37927, 0xFFBC821E, 0xFFBD831F, 0xFFBE8420, 0xFFBD831F, 0xFFBB811D, 0xFFBA801C, 0xFFBB811D, 0xFFBC821E, 0xFFB58517, 0xFFB7891A, 0xFFB48919, 0xFFAD8114, 0xFFAC8013,
        0xFFB4831A, 0xFFB07B13, 0xFFA26804, 0xFF5E3402, 0xFF533B00, 0xFF837A2D, 0xFFB9B681, 0xFF3E3925, 0xFF63645E, 0xFF889895, 0xFF688684, 0xFF7AA0AB, 0xFFBADEEE, 0xFFA5C4D6, 0xFF76929E, 0xFF6E868A, 0xFFB8CBCF, 0xFF98A6AF, 0xFF535E70, 0xFF646140,
        0xFF696528, 0xFF4D3D01, 0xFF755407, 0xFFE4BB3B, 0xFFC69B19, 0xFF764D00, 0xFF593600, 0xFF514103, 0xFF564608, 0xFF524204, 0xFF4E3E00, 0xFF504002, 0xFF504002, 0xFF504002, 0xFF574709, 0xFF4F4507, 0xFF4F4507, 0xFF4F4507, 0xFF4F4507, 0xFF4F4507,
        0xFF4F4507, 0xFF4F4507, 0xFF4F4507, 0xFF4E4607, 0xFF4E4607, 0xFF4E4607, 0xFF4E4607, 0xFF4E4607, 0xFF4E4607, 0xFF4E4607, 0xFF4E4607, 0xFF504915, 0xFF50430E, 0xFF503B04, 0xFF6A4C16, 0xFF9B7B48, 0xFF715226, 0xFF705633, 0xFF654E2F, 0xFF735249,
        0xFF715142, 0xFF674B35, 0xFF6D5237, 0xFF5C4227, 0xFF654A35, 0xFF61473A, 0xFF63483F, 0xFF674C3B, 0xFF644936, 0xFF644936, 0xFF694E39, 0xFF5C4229, 0xFF674D32, 0xFF5F452A, 0xFF654C2E, 0xFF655420, 0xFF614B22, 0xFF654E2F, 0xFF6B503B, 0xFF6A4C34,
        0xFF6C4E2A, 0xFF7A5F2A, 0xFF8C702F, 0xFF4F4104, 0xFF4F4104, 0xFF4F4104, 0xFF504205, 0xFF514306, 0xFF524407, 0xFF534508, 0xFF544609, 0xFF504809, 0xFF504809, 0xFF504809, 0xFF504809, 0xFF504809, 0xFF504809, 0xFF504809, 0xFF504809, 0xFF4D4A07,
        0xFF4D4A07, 0xFF4D4909, 0xFF4D4909, 0xFF4D490A, 0xFF4D490A, 0xFF4D490A, 0xFF4D490C, 0xFF52430C, 0xFF53440D, 0xFF53440D, 0xFF52430C, 0xFF51420B, 0xFF50410A, 0xFF4F4009, 0xFF4E3F08, 0xFF4D400C, 0xFF46400C, 0xFF696D3B, 0xFF4E5833, 0xFF2A3316,
        0xFF555936, 0xFF56551D, 0xFF908C43, 0xFF584A0B, 0xFF554708, 0xFF514304, 0xFF504203, 0xFF514304, 0xFF524405, 0xFF524405, 0xFF514304, 0xFF504108, 0xFF453B06, 0xFF6B6939, 0xFF4B4E23, 0xFF575E32, 0xFF4C4F20, 0xFF787643, 0xFF605B24, 0xFF556445,
        0xFF384420, 0xFF656C42, 0xFF595C2D, 0xFF7D7D4B, 0xFF4A4919, 0xFF626337, 0xFF515228, 0xFF5F5B2E, 0xFF64652C, 0xFF6B7133, 0xFF525C2A, 0xFF515A3B, 0xFF51573D, 0xFF575E34, 0xFF6C7639, 0xFF626039, 0xFF65633D, 0xFF55532D, 0xFF65633D, 0xFF4B4825,
        0xFF63603D, 0xFF4D4A29, 0xFF686544, 0xFF4C4D21, 0xFF66673B, 0xFF4C4D21, 0xFF68693D, 0xFF4C4D21, 0xFF5D5E32, 0xFF505125, 0xFF606135, 0xFF575733, 0xFF5E5E3A, 0xFF575733, 0xFF5A5A36, 0xFF5A5A36, 0xFF575733, 0xFF5D5D39, 0xFF545430, 0xFF6D6A3D,
        0xFF525027, 0xFF5D623A, 0xFF494F29, 0xFF6E7048, 0xFF504C1D, 0xFF504410, 0xFF524109, 0xFF49420B, 0xFF423B05, 0xFF3E3808, 0xFF3D3A0F, 0xFF383512, 0xFF2B2B0F, 0xFF23250D, 0xFF252813, 0xFF2F2D16, 0xFF38320E, 0xFF4F3F0E, 0xFF493200, 0xFF4E3203,
        0xFF5D441B, 0xFF544318, 0xFF4E4316, 0xFF755B00, 0xFFD1A43F, 0xFFBA9024, 0xFF685400, 0xFF574400, 0xFF5E390F, 0xFF5A390C, 0xFF54510E, 0xFF544003, 0xFF554104, 0xFF564203, 0xFF574304, 0xFF574304, 0xFF584503, 0xFF574402, 0xFF574402, 0xFF584501,
        0xFF544502, 0xFF554706, 0xFF594506, 0xFF604707, 0xFF5B4702, 0xFF5B5807, 0xFF738028, 0xFF494C45, 0xFF000200, 0xFF010600, 0xFF0F1511, 0xFF1D2623, 0xFF26312D, 0xFF333F3D, 0xFF263232, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FEFEFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF131510, 0xFF090B0A, 0xFF0B0E05, 0xFF0E130F, 0xFF13161D, 0xFF13160F, 0xFF55582D, 0xFF4E4F0D, 0xFF4C4906, 0xFF5B4200, 0xFF513B02, 0xFF554104, 0xFF593E00, 0xFF9C7534,
        0xFF875B2E, 0xFF6B4A1F, 0xFFD3BE87, 0xFFFCF98E, 0xFFF4F889, 0xFFF4FA9A, 0xFFC2B978, 0xFF694D1B, 0xFF825F1F, 0xFF93702C, 0xFF563800, 0xFF5A3F07, 0xFF5B4008, 0xFF5D420A, 0xFF5E430B, 0xFF5E430B, 0xFF5D420A, 0xFF5C4109, 0xFF5B4008, 0xFF614305,
        0xFF62480D, 0xFF574108, 0xFF56430B, 0xFF57440A, 0xFF533D01, 0xFF573E00, 0xFF553800, 0xFF523E00, 0xFF513D00, 0xFF513D00, 0xFF513D00, 0xFF513D00, 0xFF513D00, 0xFF523E00, 0xFF523E00, 0xFF4B3F00, 0xFF493D00, 0xFF4B4200, 0xFF4D4300, 0xFF4A4000,
        0xFF4E4000, 0xFF514103, 0xFF493900, 0xFF634016, 0xFF473300, 0xFF7E782C, 0xFFB3B174, 0xFF363018, 0xFF6A686D, 0xFF808F96, 0xFF698883, 0xFF9BBED2, 0xFFAACDE0, 0xFF9BBDC9, 0xFFA2C3CA, 0xFF91ADB0, 0xFF96ABAE, 0xFF717F88, 0xFF5A6674, 0xFF65634C,
        0xFF4B4A0A, 0xFF4D4104, 0xFF503500, 0xFF896600, 0xFFDDB438, 0xFFC4932D, 0xFF774300, 0xFF543D00, 0xFF5A4300, 0xFF604903, 0xFF5F4802, 0xFF5B4400, 0xFF5B4400, 0xFF5A4300, 0xFF543D00, 0xFF5C3F00, 0xFF5C3F00, 0xFF5C3F00, 0xFF5C3F00, 0xFF5C3F00,
        0xFF5C3F00, 0xFF5C3F00, 0xFF5C3F00, 0xFF5C4100, 0xFF5B4000, 0xFF5A3F00, 0xFF5A3F00, 0xFF5A3F00, 0xFF5A3F00, 0xFF5B4000, 0xFF5C4100, 0xFF594200, 0xFF5B4000, 0xFF603E00, 0xFF785106, 0xFFAC8037, 0xFF9E712D, 0xFFAB7F3E, 0xFFA47B3D, 0xFFA47726,
        0xFFA67C28, 0xFFA27822, 0xFFA77E26, 0xFF9F7720, 0xFFA47B2B, 0xFF9D762B, 0xFFA07832, 0xFF9E7831, 0xFFA17B34, 0xFF9A742D, 0xFF9E782F, 0xFF967025, 0xFF9D772C, 0xFF997426, 0xFF987325, 0xFF937225, 0xFF97752E, 0xFF967032, 0xFF997239, 0xFF9A7237,
        0xFF9A712F, 0xFFA17A2B, 0xFF9F7A22, 0xFF5F4000, 0xFF5F4000, 0xFF5E3F00, 0xFF5F4000, 0xFF5F4000, 0xFF604101, 0xFF614202, 0xFF624303, 0xFF5F4101, 0xFF5F4101, 0xFF5F4101, 0xFF5F4101, 0xFF5F4101, 0xFF5F4101, 0xFF5F4101, 0xFF5F4101, 0xFF5C4500,
        0xFF5C4500, 0xFF5C4501, 0xFF5C4501, 0xFF5C4502, 0xFF5C4502, 0xFF5C4404, 0xFF5C4404, 0xFF5E3E01, 0xFF5E3E01, 0xFF5E3E01, 0xFF5E3E01, 0xFF5E3E01, 0xFF5D3D00, 0xFF5C3C00, 0xFF5B3B00, 0xFF5C3900, 0xFF5C4400, 0xFF8B7F33, 0xFF65651D, 0xFF434104,
        0xFF645A1C, 0xFF766219, 0xFF947922, 0xFF56480B, 0xFF544609, 0xFF514306, 0xFF504205, 0xFF514306, 0xFF524407, 0xFF524407, 0xFF514306, 0xFF603C00, 0xFF5D3D00, 0xFF533900, 0xFF513B01, 0xFF4E3A00, 0xFF5B4605, 0xFF745C16, 0xFF94782F, 0xFF726E49,
        0xFF696035, 0xFF82753E, 0xFF8A7637, 0xFF7D6622, 0xFF543B00, 0xFF553B00, 0xFF553A05, 0xFF5B3C00, 0xFF6F5602, 0xFF857619, 0xFF796D23, 0xFF786A3D, 0xFF746641, 0xFF7A7130, 0xFF888224, 0xFF654C09, 0xFF513800, 0xFF5B4200, 0xFF482F00, 0xFF5B4200,
        0xFF4F3600, 0xFF563D00, 0xFF543B00, 0xFF503800, 0xFF563E02, 0xFF523A00, 0xFF503800, 0xFF594105, 0xFF4F3700, 0xFF553D01, 0xFF533B00, 0xFF553E00, 0xFF594200, 0xFF523B00, 0xFF543D00, 0xFF563F00, 0xFF553E00, 0xFF594200, 0xFF523B00, 0xFF553A02,
        0xFF583F07, 0xFF503903, 0xFF533E09, 0xFF513A04, 0xFF64470D, 0xFF573500, 0xFF613B00, 0xFF614500, 0xFF583D00, 0xFF543C00, 0xFF574000, 0xFF543F00, 0xFF4C3800, 0xFF493600, 0xFF4A3A00, 0xFF3F420D, 0xFF424500, 0xFF484A00, 0xFF504D00, 0xFF5C5214,
        0xFF53440B, 0xFF4B3D00, 0xFF695A05, 0xFFC9A52A, 0xFFC29432, 0xFF734D02, 0xFF483900, 0xFF544B08, 0xFF574105, 0xFF5A410B, 0xFF503E0E, 0xFF554104, 0xFF554102, 0xFF564203, 0xFF564203, 0xFF574402, 0xFF574402, 0xFF574400, 0xFF574400, 0xFF554200,
        0xFF514200, 0xFF524403, 0xFF564203, 0xFF5D4404, 0xFF594500, 0xFF595605, 0xFF727F27, 0xFF53564F, 0xFF020500, 0xFF060B05, 0xFF171D19, 0xFF313A37, 0xFF3C4743, 0xFF35413F, 0xFF162222, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFF, 0x00FFFDFE, 0x00FFFFFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF12140F, 0xFF0E100F, 0xFF0B0E05, 0xFF070C08, 0xFF13161D, 0xFF1E211A, 0xFF62653A, 0xFF575816, 0xFF4B4805, 0xFF5C4400, 0xFF554200, 0xFF523F07, 0xFF5B3F10, 0xFF876123,
        0xFF9A7326, 0xFF6A4809, 0xFF876A4A, 0xFFEBDAA6, 0xFFF8EFAE, 0xFFD4CB90, 0xFF85724A, 0xFF704E29, 0xFF9B743B, 0xFF85641F, 0xFF583F00, 0xFF5C4109, 0xFF5C4109, 0xFF5D420A, 0xFF5D420A, 0xFF5C4109, 0xFF5C4109, 0xFF5C4109, 0xFF5C4109, 0xFF5A3E00,
        0xFF5B4104, 0xFF594309, 0xFF59440B, 0xFF55420A, 0xFF534008, 0xFF5A470D, 0xFF59450A, 0xFF54450E, 0xFF53440D, 0xFF53440D, 0xFF53440D, 0xFF54450E, 0xFF55460F, 0xFF55460F, 0xFF54450E, 0xFF5C4D0A, 0xFF584606, 0xFF5B490B, 0xFF5D4A0F, 0xFF55420A,
        0xFF52410B, 0xFF574612, 0xFF574714, 0xFF4F3700, 0xFF53430F, 0xFF6B6624, 0xFFB3B164, 0xFF50491D, 0xFF3C3644, 0xFF4E5871, 0xFF557371, 0xFF7995AA, 0xFF5E7E8D, 0xFF577982, 0xFF6F9095, 0xFF708D91, 0xFF475C61, 0xFF2E3C3F, 0xFF485053, 0xFF5D5C40,
        0xFF464700, 0xFF463C00, 0xFF583F16, 0xFF503100, 0xFF946D12, 0xFFDBAA43, 0xFFCD971D, 0xFF625413, 0xFF4D3F00, 0xFF4C3E00, 0xFF564807, 0xFF524403, 0xFF524403, 0xFF554706, 0xFF504201, 0xFF564406, 0xFF564406, 0xFF564406, 0xFF564406, 0xFF564406,
        0xFF564406, 0xFF564406, 0xFF564406, 0xFF564406, 0xFF554305, 0xFF544204, 0xFF544204, 0xFF544204, 0xFF544204, 0xFF554305, 0xFF564406, 0xFF4D4A05, 0xFF514A04, 0xFF4F4500, 0xFF544300, 0xFF6E5A11, 0xFF69530A, 0xFF705A11, 0xFF69520C, 0xFF6C540E,
        0xFF705810, 0xFF6B5309, 0xFF6D570B, 0xFF6B5607, 0xFF6C590C, 0xFF675408, 0xFF6A560D, 0xFF695614, 0xFF6C5915, 0xFF675410, 0xFF715E1A, 0xFF705D19, 0xFF705D19, 0xFF6C5914, 0xFF6B5813, 0xFF65570C, 0xFF6C5E15, 0xFF63540F, 0xFF655613, 0xFF675813,
        0xFF67560E, 0xFF6A5A0C, 0xFF5F5000, 0xFF504407, 0xFF504407, 0xFF4F4306, 0xFF4F4306, 0xFF4F4306, 0xFF4F4306, 0xFF504407, 0xFF504407, 0xFF554207, 0xFF554207, 0xFF554207, 0xFF554207, 0xFF554207, 0xFF554207, 0xFF554207, 0xFF554207, 0xFF514606,
        0xFF514606, 0xFF514606, 0xFF514508, 0xFF514508, 0xFF514508, 0xFF514509, 0xFF514509, 0xFF52460A, 0xFF52460A, 0xFF52460A, 0xFF52460A, 0xFF52460A, 0xFF514509, 0xFF504408, 0xFF4F4307, 0xFF563D00, 0xFF4D3D02, 0xFF706C3F, 0xFF7A7D62, 0xFF40412F,
        0xFF7E795B, 0xFF7D7135, 0xFF695701, 0xFF5B4700, 0xFF5A4600, 0xFF5A4600, 0xFF5A4600, 0xFF5B4700, 0xFF5B4700, 0xFF5B4700, 0xFF5A4600, 0xFF56450F, 0xFF584912, 0xFF4A3D08, 0xFF594E16, 0xFF4C4207, 0xFF4C4204, 0xFF473C00, 0xFF655814, 0xFF57613F,
        0xFF50552C, 0xFF5C5C28, 0xFF666223, 0xFF5F5615, 0xFF514508, 0xFF50430C, 0xFF4F430F, 0xFF513C0F, 0xFF544607, 0xFF635F18, 0xFF5E5C29, 0xFF5E5C45, 0xFF5E5B4A, 0xFF616238, 0xFF5D6019, 0xFF584F0C, 0xFF4E4502, 0xFF4C4302, 0xFF4D4403, 0xFF4E4504,
        0xFF514709, 0xFF4D4305, 0xFF4C4204, 0xFF504609, 0xFF4F4508, 0xFF52480B, 0xFF483E01, 0xFF574D10, 0xFF493F02, 0xFF53490C, 0xFF4E4407, 0xFF514604, 0xFF534806, 0xFF4F4402, 0xFF4F4402, 0xFF514604, 0xFF524705, 0xFF544907, 0xFF4F4402, 0xFF4E410C,
        0xFF4D420C, 0xFF554B16, 0xFF433904, 0xFF514610, 0xFF51420B, 0xFF59490E, 0xFF5B480D, 0xFF554A0A, 0xFF524707, 0xFF4E4504, 0xFF4E4607, 0xFF4F4708, 0xFF4D4808, 0xFF4B4707, 0xFF4B4708, 0xFF584618, 0xFF4C4308, 0xFF434201, 0xFF515115, 0xFF4F4716,
        0xFF4D3600, 0xFF876212, 0xFFCBA037, 0xFFC69628, 0xFF6D4E00, 0xFF594405, 0xFF5C4513, 0xFF523F05, 0xFF4E4A03, 0xFF4C4200, 0xFF5B3C06, 0xFF564203, 0xFF564203, 0xFF564203, 0xFF564301, 0xFF564301, 0xFF564300, 0xFF574400, 0xFF574400, 0xFF554200,
        0xFF514200, 0xFF524403, 0xFF564203, 0xFF5D4404, 0xFF594500, 0xFF5A5706, 0xFF738028, 0xFF3D4039, 0xFF000200, 0xFF020701, 0xFF0A100C, 0xFF1F2825, 0xFF25302C, 0xFF1E2A28, 0xFF0C1818, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFFFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF11130E, 0xFF0E100F, 0xFF0F1209, 0xFF0A0F0B, 0xFF15181F, 0xFF1A1D16, 0xFF5A5D32, 0xFF525311, 0xFF474401, 0xFF5A3F00, 0xFF594602, 0xFF503E0C, 0xFF513712, 0xFF69470A,
        0xFFA68224, 0xFF866518, 0xFF6F4C2E, 0xFF785843, 0xFF876F49, 0xFF755F36, 0xFF634427, 0xFF89613E, 0xFF966D35, 0xFF614400, 0xFF584405, 0xFF5E430B, 0xFF5D420A, 0xFF5C4109, 0xFF5B4008, 0xFF5B4008, 0xFF5B4008, 0xFF5C4109, 0xFF5C4109, 0xFF594000,
        0xFF5A4204, 0xFF563E02, 0xFF564006, 0xFF5C470E, 0xFF58450D, 0xFF514008, 0xFF54430D, 0xFF5B450B, 0xFF5A440A, 0xFF5A440A, 0xFF5A440A, 0xFF5A440A, 0xFF5A440A, 0xFF594309, 0xFF584208, 0xFF5C4301, 0xFF5E4203, 0xFF5F4103, 0xFF5E3E03, 0xFF5B3D01,
        0xFF5C4207, 0xFF5A4408, 0xFF554104, 0xFF5C4A00, 0xFF473908, 0xFF483F08, 0xFFBDB962, 0xFF827938, 0xFF1F1514, 0xFF565A73, 0xFF96B0B1, 0xFF5B7282, 0xFF4F6A75, 0xFF517075, 0xFF58777C, 0xFF627D86, 0xFF32434A, 0xFF2A3330, 0xFF45493B, 0xFF4B4920,
        0xFF484500, 0xFF4C3D00, 0xFF5A431A, 0xFF543C0E, 0xFF5B3D01, 0xFF8C640F, 0xFFDEB330, 0xFFA88936, 0xFF7B5C09, 0xFF674800, 0xFF705100, 0xFF6C4D00, 0xFF664700, 0xFF6A4B00, 0xFF6E4F00, 0xFF624E00, 0xFF624E00, 0xFF624E00, 0xFF624E00, 0xFF624E00,
        0xFF624E00, 0xFF624E00, 0xFF624E00, 0xFF634D00, 0xFF634D00, 0xFF634D00, 0xFF624C00, 0xFF624C00, 0xFF634D00, 0xFF634D00, 0xFF634D00, 0xFF554E00, 0xFF5A5102, 0xFF5D5202, 0xFF584C00, 0xFF635401, 0xFF625100, 0xFF655500, 0xFF604E00, 0xFF604B0A,
        0xFF634E0D, 0xFF614E0A, 0xFF614E09, 0xFF5D4C04, 0xFF5F4F04, 0xFF5E4E01, 0xFF605305, 0xFF5F4F02, 0xFF645407, 0xFF5E4E01, 0xFF5D4D00, 0xFF5C4C00, 0xFF5D4D02, 0xFF5D4D02, 0xFF5E4E03, 0xFF594D00, 0xFF635703, 0xFF5C5000, 0xFF5C5200, 0xFF5E5400,
        0xFF5B5100, 0xFF5E5400, 0xFF594F00, 0xFF5B5300, 0xFF5B5300, 0xFF5A5200, 0xFF595100, 0xFF595100, 0xFF595100, 0xFF5A5200, 0xFF5A5200, 0xFF645200, 0xFF645200, 0xFF645200, 0xFF645200, 0xFF645200, 0xFF645200, 0xFF645200, 0xFF645200, 0xFF625300,
        0xFF625300, 0xFF625300, 0xFF625300, 0xFF625300, 0xFF625300, 0xFF625302, 0xFF625302, 0xFF5D5502, 0xFF5E5603, 0xFF5E5603, 0xFF5E5603, 0xFF5E5603, 0xFF5D5502, 0xFF5C5401, 0xFF5C5401, 0xFF684D08, 0xFF5A4600, 0xFF554B04, 0xFF4E4A0D, 0xFF363300,
        0xFF5C531A, 0xFF625109, 0xFF5F4B00, 0xFF554E0A, 0xFF564F0B, 0xFF564F0B, 0xFF57500C, 0xFF57500C, 0xFF57500C, 0xFF57500C, 0xFF564F0B, 0xFF584C02, 0xFF504400, 0xFF5C5004, 0xFF5D5203, 0xFF5C4F01, 0xFF544700, 0xFF5B4F00, 0xFF564A00, 0xFF444607,
        0xFF3D3C00, 0xFF4F4B01, 0xFF4C4400, 0xFF5D5100, 0xFF524600, 0xFF5E5304, 0xFF554900, 0xFF5E4C04, 0xFF524600, 0xFF544F00, 0xFF484500, 0xFF433F02, 0xFF433E06, 0xFF504C05, 0xFF4D4900, 0xFF564A00, 0xFF594D03, 0xFF534700, 0xFF5E520A, 0xFF504400,
        0xFF5A4D07, 0xFF524500, 0xFF574A04, 0xFF554A00, 0xFF5D5200, 0xFF574C00, 0xFF5E5301, 0xFF534800, 0xFF584D00, 0xFF564B00, 0xFF5D5200, 0xFF574900, 0xFF5A4C01, 0xFF584A00, 0xFF574900, 0xFF584A00, 0xFF5A4C01, 0xFF5B4D02, 0xFF584A00, 0xFF5E5304,
        0xFF534800, 0xFF5F5405, 0xFF514600, 0xFF655809, 0xFF514400, 0xFF5C4F00, 0xFF5D4D00, 0xFF5A4C05, 0xFF5A4E06, 0xFF584C04, 0xFF544800, 0xFF544800, 0xFF544A02, 0xFF544B00, 0xFF514800, 0xFF6D4000, 0xFF664100, 0xFF5C4100, 0xFF5C4502, 0xFF5D4200,
        0xFF8A6112, 0xFFCB9332, 0xFFD49221, 0xFF734700, 0xFF524100, 0xFF504400, 0xFF5D3F09, 0xFF5D4112, 0xFF4E480E, 0xFF4A4300, 0xFF634600, 0xFF574304, 0xFF574402, 0xFF564301, 0xFF564301, 0xFF564300, 0xFF564300, 0xFF574400, 0xFF574400, 0xFF584501,
        0xFF544502, 0xFF544605, 0xFF584405, 0xFF5F4606, 0xFF5C4803, 0xFF5D5A09, 0xFF77842C, 0xFF494C45, 0xFF010400, 0xFF080D07, 0xFF080E0A, 0xFF121B18, 0xFF15201C, 0xFF172321, 0xFF192525, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFFFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF10120D, 0xFF080A09, 0xFF10130A, 0xFF0F1410, 0xFF15181F, 0xFF10130C, 0xFF4A4D22, 0xFF4A4B09, 0xFF484502, 0xFF593C00, 0xFF554009, 0xFF534312, 0xFF4E3806, 0xFF5E3C00,
        0xFF8C651A, 0xFF99732C, 0xFF8C692F, 0xFF784C29, 0xFF735018, 0xFF735517, 0xFF8F6A36, 0xFFA17643, 0xFF81591B, 0xFF583C00, 0xFF554511, 0xFF5E430B, 0xFF5D420A, 0xFF5D420A, 0xFF5C4109, 0xFF5B4008, 0xFF5B4008, 0xFF5C4109, 0xFF5C4109, 0xFF584304,
        0xFF61490B, 0xFF5E4608, 0xFF573F01, 0xFF5B4307, 0xFF5B450B, 0xFF58430C, 0xFF5B4810, 0xFF57440A, 0xFF57440A, 0xFF57440A, 0xFF564309, 0xFF564309, 0xFF554208, 0xFF564309, 0xFF57440A, 0xFF534308, 0xFF5B480D, 0xFF5C460C, 0xFF5A4206, 0xFF5A4408,
        0xFF594A09, 0xFF534A07, 0xFF4C4703, 0xFF584701, 0xFF564817, 0xFF413503, 0xFF887C30, 0xFFC5B86A, 0xFF554921, 0xFF1D1F1A, 0xFF60737A, 0xFF4A5B63, 0xFF41565B, 0xFF3E5A5D, 0xFF66838B, 0xFF6B818F, 0xFF233038, 0xFF24271C, 0xFF444325, 0xFF444209,
        0xFF372D00, 0xFF564006, 0xFF56410C, 0xFF4B3C00, 0xFF4E3D09, 0xFF583B03, 0xFF876704, 0xFFFAB23A, 0xFFEEA62E, 0xFFE8A028, 0xFFEEA62E, 0xFFF3AB33, 0xFFEFA72F, 0xFFEDA52D, 0xFFF1A931, 0xFFE5A72E, 0xFFE5A72E, 0xFFE5A72E, 0xFFE5A72E, 0xFFE5A72E,
        0xFFE5A72E, 0xFFE5A72E, 0xFFE5A72E, 0xFFE6A52F, 0xFFE6A52F, 0xFFE6A52F, 0xFFE7A630, 0xFFE7A630, 0xFFE6A52F, 0xFFE6A52F, 0xFFE6A52F, 0xFFDFA22F, 0xFFDDA12F, 0xFFDEA532, 0xFFDCA332, 0xFFDDA431, 0xFFDCA32F, 0xFFDDA22C, 0xFFDDA22C, 0xFFDCA71D,
        0xFFDCA61F, 0xFFDDA721, 0xFFDDA922, 0xFFD8A41D, 0xFFD5A31C, 0xFFD5A41B, 0xFFD4A318, 0xFFD7A214, 0xFFDDA81A, 0xFFDDA81A, 0xFFDAA519, 0xFFDCA71B, 0xFFDEA91F, 0xFFD9A41A, 0xFFDBA61C, 0xFFD79B2B, 0xFFDDA12F, 0xFFD79F28, 0xFFD8A027, 0xFFDAA42A,
        0xFFD7A029, 0xFFD6A12D, 0xFFD9A432, 0xFFD8A12A, 0xFFD8A12A, 0xFFD7A029, 0xFFD7A029, 0xFFD7A029, 0xFFD8A12A, 0xFFD8A12A, 0xFFD9A22B, 0xFFD99E28, 0xFFD99E28, 0xFFD99E28, 0xFFD99E28, 0xFFD99E28, 0xFFD99E28, 0xFFD99E28, 0xFFD99E28, 0xFFD99F24,
        0xFFD99F24, 0xFFD99E26, 0xFFD99E26, 0xFFD99E28, 0xFFD99E28, 0xFFD99E2A, 0xFFD99E2A, 0xFFD49D26, 0xFFD59E27, 0xFFD59E27, 0xFFD59E27, 0xFFD59E27, 0xFFD59E27, 0xFFD49D26, 0xFFD39C25, 0xFFD49231, 0xFFDA9F2B, 0xFFD8A41D, 0xFFC7980C, 0xFFD7A826,
        0xFFCC9A21, 0xFFCE9722, 0xFFDDA22A, 0xFFD89925, 0xFFD79824, 0xFFD79824, 0xFFD79824, 0xFFD69723, 0xFFD69723, 0xFFD79824, 0xFFD79824, 0xFFD69F2B, 0xFFCD9622, 0xFFD49B27, 0xFFCF9420, 0xFFCE911E, 0xFFCF921F, 0xFFDA9E2C, 0xFFD09424, 0xFFCA9724,
        0xFFCA9521, 0xFFD89F2C, 0xFFD59825, 0xFFD89B26, 0xFFD19620, 0xFFD69D29, 0xFFCE9722, 0xFFD2961C, 0xFFD1971A, 0xFFD6A121, 0xFFCF9C1D, 0xFFCE9B1E, 0xFFC99317, 0xFFD59B1E, 0xFFD99E1E, 0xFFD09422, 0xFFD29624, 0xFFD49828, 0xFFD09424, 0xFFD4982A,
        0xFFCE9224, 0xFFD19429, 0xFFD19429, 0xFFCE961B, 0xFFD49C21, 0xFFCF971C, 0xFFD79F24, 0xFFC99116, 0xFFD39B20, 0xFFCF971C, 0xFFD79F24, 0xFFD49625, 0xFFD79928, 0xFFD89A29, 0xFFD69827, 0xFFD59726, 0xFFD89A29, 0xFFD89A29, 0xFFD79928, 0xFFD69E27,
        0xFFD39B22, 0xFFD3991E, 0xFFDEA228, 0xFFDCA026, 0xFFD79D22, 0xFFD29A21, 0xFFD59D26, 0xFFD89A29, 0xFFD99B2A, 0xFFD89A29, 0xFFD49625, 0xFFD29423, 0xFFD39524, 0xFFD39524, 0xFFD19322, 0xFFC69616, 0xFFCB9A21, 0xFFCD9E2A, 0xFFC69820, 0xFFC59A1A,
        0xFFD6AE2B, 0xFFC0971B, 0xFF7C5300, 0xFF514000, 0xFF564704, 0xFF544001, 0xFF5B4008, 0xFF563B06, 0xFF523E03, 0xFF5E4D08, 0xFF4D3B00, 0xFF584503, 0xFF584503, 0xFF574402, 0xFF564300, 0xFF564300, 0xFF564300, 0xFF574400, 0xFF584500, 0xFF594602,
        0xFF554603, 0xFF554706, 0xFF584405, 0xFF604707, 0xFF5D4904, 0xFF5E5B0A, 0xFF78852D, 0xFF5B5E57, 0xFF000200, 0xFF040903, 0xFF0C120E, 0xFF19221F, 0xFF212C28, 0xFF273331, 0xFF273333, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FFFEFF, 0x00FFFEFF, 0x00FFFFFD, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF10120D, 0xFF050706, 0xFF0D1007, 0xFF0A0F0B, 0xFF13161D, 0xFF12150E, 0xFF505328, 0xFF535412, 0xFF4F4C09, 0xFF5D3F09, 0xFF50390F, 0xFF584B17, 0xFF554400, 0xFF5D3E00,
        0xFF5D3100, 0xFF855D1F, 0xFF9A7B20, 0xFFB58543, 0xFFA17B26, 0xFF98761C, 0xFF9A7326, 0xFF794C08, 0xFF5E3600, 0xFF614808, 0xFF4A3D10, 0xFF5D420A, 0xFF5D420A, 0xFF5D420A, 0xFF5D420A, 0xFF5C4109, 0xFF5C4109, 0xFF5B4008, 0xFF5B4008, 0xFF5A4609,
        0xFF554001, 0xFF5B4303, 0xFF5D4404, 0xFF583F00, 0xFF5C4207, 0xFF5C460C, 0xFF59420C, 0xFF5E4505, 0xFF5E4505, 0xFF5E4505, 0xFF5D4404, 0xFF5C4303, 0xFF5D4404, 0xFF604707, 0xFF624909, 0xFF534808, 0xFF5C4D0C, 0xFF5E4B09, 0xFF5D4805, 0xFF5D4A05,
        0xFF554900, 0xFF4C4900, 0xFF505200, 0xFF59460C, 0xFF4F420E, 0xFF574918, 0xFF3B2B00, 0xFFBBAA58, 0xFFC6BF71, 0xFF3B411F, 0xFF051522, 0xFF1F2D30, 0xFF263A39, 0xFF1D3736, 0xFF57727B, 0xFF5A6D7E, 0xFF263039, 0xFF333424, 0xFF433F19, 0xFF393600,
        0xFF57470B, 0xFF583C0D, 0xFF513C00, 0xFF534C00, 0xFF4B4207, 0xFF54401D, 0xFF5A4101, 0xFF7E5200, 0xFF9F7312, 0xFFA57918, 0xFF986C0B, 0xFF9C700F, 0xFFA07413, 0xFF9B6F0E, 0xFF9D7110, 0xFF9D6F11, 0xFF9D6F11, 0xFF9D6F11, 0xFF9D6F11, 0xFF9D6F11,
        0xFF9D6F11, 0xFF9D6F11, 0xFF9D6F11, 0xFF9E6D11, 0xFF9E6D11, 0xFF9F6E12, 0xFFA06F13, 0xFFA06F13, 0xFF9F6E12, 0xFF9E6D11, 0xFF9E6D11, 0xFFAA7410, 0xFF9F6B08, 0xFF9E6D0E, 0xFF9E7113, 0xFF9B7011, 0xFF9A700E, 0xFF996F0B, 0xFFA0750F, 0xFF9C750C,
        0xFF956E09, 0xFF956D0D, 0xFF956E13, 0xFF946C14, 0xFF957018, 0xFF967119, 0xFF906B12, 0xFF9E721B, 0xFF986B16, 0xFF966914, 0xFF936613, 0xFF9A6D1A, 0xFF9B6E1D, 0xFF936517, 0xFF9A6C1E, 0xFF9C6D11, 0xFF9B6D0D, 0xFF976A05, 0xFF976C03, 0xFF9C7309,
        0xFF986E08, 0xFF926A0A, 0xFF986F13, 0xFF976A0F, 0xFF976A0F, 0xFF976A0F, 0xFF976A0F, 0xFF986B10, 0xFF996C11, 0xFF9A6D12, 0xFF9B6E13, 0xFF946D10, 0xFF946D10, 0xFF946D10, 0xFF946D10, 0xFF946D10, 0xFF946D10, 0xFF946D10, 0xFF946D10, 0xFF956D0E,
        0xFF956D0E, 0xFF956D0E, 0xFF956C10, 0xFF956C10, 0xFF956C12, 0xFF956C12, 0xFF956C12, 0xFF9A6C14, 0xFF9A6C14, 0xFF9A6C14, 0xFF9B6D15, 0xFF9B6D15, 0xFF9A6C14, 0xFF9A6C14, 0xFF996B13, 0xFF996D14, 0xFF966E0F, 0xFF8E6805, 0xFF9F7A1F, 0xFF896310,
        0xFF946E19, 0xFF916D0C, 0xFF906E00, 0xFF95720E, 0xFF94710D, 0xFF926F0B, 0xFF906D09, 0xFF8F6C08, 0xFF8F6C08, 0xFF906D09, 0xFF926F0B, 0xFF906809, 0xFF9C7113, 0xFF956709, 0xFFA47316, 0xFF9A670C, 0xFFA57219, 0xFF96640F, 0xFF9B6B17, 0xFF9D700B,
        0xFFA27210, 0xFF956206, 0xFFA67016, 0xFF945E06, 0xFFA67519, 0xFF996C0E, 0xFF9C7112, 0xFFA16E1F, 0xFF9E6D20, 0xFF9A6A1E, 0xFF976A17, 0xFFA37619, 0xFF9B6A0D, 0xFFA06813, 0xFFA4691D, 0xFFA2730B, 0xFFA5750F, 0xFF9F6F09, 0xFF9F6F09, 0xFFA47410,
        0xFFA3730F, 0xFFA0700E, 0xFF9F6F0D, 0xFFA47614, 0xFF9D6F0D, 0xFFA37513, 0xFF9B6D0B, 0xFFA47614, 0xFFA07210, 0xFFA57715, 0xFFA27412, 0xFFA87710, 0xFFAB7A13, 0xFFAF7E17, 0xFFAC7B14, 0xFFAA7912, 0xFFAD7C15, 0xFFAD7C15, 0xFFAD7C15, 0xFFA97D1A,
        0xFFB0831E, 0xFFB2821C, 0xFFB5841E, 0xFFB07F19, 0xFFB1811D, 0xFFAF8320, 0xFFAC8220, 0xFFB18115, 0xFFB18115, 0xFFB18115, 0xFFAF7E15, 0xFFAD7C13, 0xFFAD7913, 0xFFAE7A14, 0xFFAF7B15, 0xFFAA8118, 0xFFA57314, 0xFFB2721B, 0xFFB67517, 0xFFB78019,
        0xFF9D7617, 0xFF644F0C, 0xFF4B3F17, 0xFF475100, 0xFF544107, 0xFF674520, 0xFF533D0C, 0xFF574100, 0xFF664500, 0xFF533700, 0xFF534D1D, 0xFF594604, 0xFF584503, 0xFF574402, 0xFF564300, 0xFF564300, 0xFF574400, 0xFF574400, 0xFF584500, 0xFF574400,
        0xFF534401, 0xFF534504, 0xFF564203, 0xFF5E4505, 0xFF5B4702, 0xFF5D5A09, 0xFF77842C, 0xFF666962, 0xFF000200, 0xFF050A04, 0xFF161C18, 0xFF222B28, 0xFF2A3531, 0xFF2F3B39, 0xFF273333, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
  0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FEFEFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF11120A, 0xFF0D1313, 0xFF0C1013, 0xFF0F1015, 0xFF131210, 0xFF1B1A06, 0xFF605F31, 0xFF4E5007, 0xFF4A4D00, 0xFF564203, 0xFF564203, 0xFF564203, 0xFF564203, 0xFF554102,
        0xFF544001, 0xFF544001, 0xFF533F00, 0xFF62500E, 0xFF604E0C, 0xFF5C4A08, 0xFF584604, 0xFF554301, 0xFF544200, 0xFF544200, 0xFF554301, 0xFF58450A, 0xFF574409, 0xFF564308, 0xFF564308, 0xFF564308, 0xFF574409, 0xFF58450A, 0xFF59460B, 0xFF544408,
        0xFF554509, 0xFF56460A, 0xFF554509, 0xFF544408, 0xFF544408, 0xFF56460A, 0xFF58480C, 0xFF534500, 0xFF584A00, 0xFF5B4D02, 0xFF5A4C01, 0xFF594B00, 0xFF5A4C01, 0xFF5B4D02, 0xFF5A4C01, 0xFF614E0C, 0xFF624F0D, 0xFF63500E, 0xFF624F0D, 0xFF614E0C,
        0xFF614E0C, 0xFF614E0C, 0xFF624F0D, 0xFF575603, 0xFF525005, 0xFF4D490A, 0xFF4B460F, 0xFF645F28, 0xFFAAA667, 0xFFB6B469, 0xFF63620F, 0xFF312B1D, 0xFF261F19, 0xFF2C2328, 0xFF4B414A, 0xFF5A5154, 0xFF433D2D, 0xFF33310B, 0xFF41410D, 0xFF4D3E05,
        0xFF52430A, 0xFF54450C, 0xFF514207, 0xFF504106, 0xFF524407, 0xFF524407, 0xFF4F4104, 0xFF473F00, 0xFF473F00, 0xFF473F00, 0xFF473F00, 0xFF473F00, 0xFF473F00, 0xFF473F00, 0xFF473F00, 0xFF483F06, 0xFF483F06, 0xFF473E05, 0xFF473E05, 0xFF473E05,
        0xFF483F06, 0xFF494007, 0xFF494007, 0xFF484001, 0xFF484001, 0xFF473F00, 0xFF463E00, 0xFF453D00, 0xFF473F00, 0xFF4A4203, 0xFF4C4405, 0xFF494309, 0xFF484208, 0xFF474107, 0xFF484208, 0xFF494309, 0xFF4A440A, 0xFF494309, 0xFF484208, 0xFF4C4302,
        0xFF4B4201, 0xFF4B4201, 0xFF4B4201, 0xFF4A4100, 0xFF494000, 0xFF494000, 0xFF494000, 0xFF474400, 0xFF484500, 0xFF484500, 0xFF484500, 0xFF484500, 0xFF494600, 0xFF494600, 0xFF494600, 0xFF4A4700, 0xFF4A4700, 0xFF4A4700, 0xFF4A4700, 0xFF4A4700,
        0xFF494600, 0xFF494600, 0xFF494600, 0xFF484400, 0xFF484400, 0xFF494500, 0xFF494500, 0xFF4A4600, 0xFF4B4700, 0xFF4B4700, 0xFF4C4800, 0xFF494505, 0xFF484404, 0xFF484404, 0xFF474303, 0xFF484404, 0xFF494505, 0xFF4A4606, 0xFF4B4707, 0xFF4D4808,
        0xFF4D4808, 0xFF4D4808, 0xFF4D4808, 0xFF4D4808, 0xFF4D4808, 0xFF4D4808, 0xFF4D4808, 0xFF4A4606, 0xFF4A4606, 0xFF4A4606, 0xFF4A4606, 0xFF4A4606, 0xFF4A4606, 0xFF4A4606, 0xFF4A4606, 0xFF484303, 0xFF484303, 0xFF484303, 0xFF494404, 0xFF494404,
        0xFF494404, 0xFF494404, 0xFF4A4505, 0xFF494505, 0xFF484404, 0xFF484404, 0xFF474303, 0xFF464202, 0xFF454101, 0xFF454101, 0xFF444000, 0xFF464202, 0xFF464202, 0xFF464202, 0xFF464202, 0xFF464202, 0xFF464202, 0xFF464202, 0xFF464202, 0xFF494305,
        0xFF4A4406, 0xFF4A4406, 0xFF4A4505, 0xFF4A4505, 0xFF4A4503, 0xFF4A4503, 0xFF494402, 0xFF494505, 0xFF494505, 0xFF494505, 0xFF494505, 0xFF494505, 0xFF494505, 0xFF494505, 0xFF494505, 0xFF4A4606, 0xFF4A4606, 0xFF494505, 0xFF494505, 0xFF484404,
        0xFF484404, 0xFF484404, 0xFF484404, 0xFF4C4403, 0xFF4C4403, 0xFF4E4605, 0xFF4F4706, 0xFF4F4706, 0xFF4F4706, 0xFF4F4706, 0xFF4E4605, 0xFF4B4A00, 0xFF4F4E00, 0xFF4F4E00, 0xFF514F04, 0xFF555309, 0xFF514E07, 0xFF4E4B06, 0xFF565310, 0xFF55510A,
        0xFF55510A, 0xFF56520B, 0xFF58540D, 0xFF5A560F, 0xFF5A560F, 0xFF57530C, 0xFF545009, 0xFF564E0D, 0xFF564E0D, 0xFF534C08, 0xFF4E4701, 0xFF4E4700, 0xFF514B01, 0xFF514B01, 0xFF4D4700, 0xFF4C4406, 0xFF4C4406, 0xFF4D4507, 0xFF4E4608, 0xFF4F4709,
        0xFF50480A, 0xFF51490B, 0xFF524A0C, 0xFF56480B, 0xFF56480B, 0xFF55470A, 0xFF544609, 0xFF544609, 0xFF55470A, 0xFF56480B, 0xFF56480B, 0xFF534305, 0xFF534305, 0xFF534305, 0xFF534305, 0xFF544406, 0xFF544406, 0xFF554507, 0xFF564608, 0xFF58480D,
        0xFF534307, 0xFF564608, 0xFF584908, 0xFF5B4C07, 0xFF554700, 0xFF5E5005, 0xFF87792E, 0xFF484F47, 0xFF000400, 0xFF0C1510, 0xFF28312E, 0xFF263031, 0xFF283234, 0xFF2E393D, 0xFF29343A, 0xFF2B3534, 0x00C4CDCC, 0x00FCFFFF, 0x00FAFBFD,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF101109, 0xFF090D0C, 0xFF0D1112, 0xFF0B0C10, 0xFF0E0E0E, 0xFF1E1C0D, 0xFF65653F, 0xFF5C5D21, 0xFF505207, 0xFF544605, 0xFF534504, 0xFF534504, 0xFF534504, 0xFF534504,
        0xFF544605, 0xFF554706, 0xFF564807, 0xFF524707, 0xFF524707, 0xFF514606, 0xFF514606, 0xFF504505, 0xFF514606, 0xFF514606, 0xFF514606, 0xFF504600, 0xFF514700, 0xFF514700, 0xFF514700, 0xFF524801, 0xFF524801, 0xFF524801, 0xFF524801, 0xFF524A02,
        0xFF534B03, 0xFF544C04, 0xFF534B03, 0xFF524A02, 0xFF514901, 0xFF524A02, 0xFF544C04, 0xFF574D0F, 0xFF53490B, 0xFF52480A, 0xFF53490B, 0xFF4F4507, 0xFF504608, 0xFF605618, 0xFF756B2D, 0xFF6C682B, 0xFF6E6A2D, 0xFF6F6B2E, 0xFF6E6A2D, 0xFF6C682B,
        0xFF6A6629, 0xFF696528, 0xFF696528, 0xFF707326, 0xFF72742B, 0xFF737333, 0xFF74733A, 0xFF6D6C33, 0xFF777737, 0xFF94964D, 0xFF9B9E51, 0xFF656936, 0xFF4D5023, 0xFF353712, 0xFF2C2D0B, 0xFF31330C, 0xFF3B3F0C, 0xFF3E4501, 0xFF3B4400, 0xFF4A4300,
        0xFF4C4501, 0xFF4D4600, 0xFF4B4400, 0xFF4B4400, 0xFF4D4600, 0xFF4E4700, 0xFF4E4800, 0xFF4C4A00, 0xFF4C4A00, 0xFF4C4A00, 0xFF4C4A00, 0xFF4C4A00, 0xFF4D4B01, 0xFF4D4B01, 0xFF4D4B01, 0xFF4B4B00, 0xFF4B4B00, 0xFF4C4C00, 0xFF4C4C00, 0xFF4B4B00,
        0xFF4A4A00, 0xFF494900, 0xFF494900, 0xFF504E04, 0xFF514F05, 0xFF514F05, 0xFF4F4D03, 0xFF4C4A00, 0xFF4B4900, 0xFF4B4900, 0xFF4C4A00, 0xFF4A4D00, 0xFF494C00, 0xFF484B00, 0xFF494C00, 0xFF4A4D00, 0xFF4B4E00, 0xFF4A4D00, 0xFF494C00, 0xFF4E4B08,
        0xFF4E4B08, 0xFF4D4A07, 0xFF4D4A07, 0xFF4C4906, 0xFF4C4906, 0xFF4B4805, 0xFF4B4805, 0xFF484903, 0xFF484903, 0xFF484903, 0xFF484903, 0xFF484903, 0xFF484903, 0xFF484903, 0xFF484903, 0xFF4C4D07, 0xFF4C4D07, 0xFF4C4D07, 0xFF4C4D07, 0xFF4C4D07,
        0xFF4B4C06, 0xFF4B4C06, 0xFF4B4C06, 0xFF4B4B01, 0xFF4B4B01, 0xFF4B4B01, 0xFF4B4B01, 0xFF4C4C02, 0xFF4C4C02, 0xFF4C4C02, 0xFF4C4C02, 0xFF494A08, 0xFF494A08, 0xFF4A4B09, 0xFF4A4B09, 0xFF4B4C0A, 0xFF4B4C0A, 0xFF4B4C0A, 0xFF4B4C0A, 0xFF4B4A09,
        0xFF4B4A09, 0xFF4B4A09, 0xFF4B4A09, 0xFF4B4A09, 0xFF4B4A09, 0xFF4B4A09, 0xFF4B4A09, 0xFF494A08, 0xFF494A08, 0xFF494A08, 0xFF494A08, 0xFF494A08, 0xFF494A08, 0xFF494A08, 0xFF494A08, 0xFF484706, 0xFF484706, 0xFF484706, 0xFF494807, 0xFF494807,
        0xFF494807, 0xFF494807, 0xFF494807, 0xFF484907, 0xFF484907, 0xFF484907, 0xFF474806, 0xFF474806, 0xFF464705, 0xFF464705, 0xFF454604, 0xFF494B00, 0xFF494B00, 0xFF494B00, 0xFF494B00, 0xFF494B00, 0xFF494B00, 0xFF494B00, 0xFF494B00, 0xFF4A4A02,
        0xFF4B4B03, 0xFF4D4D05, 0xFF4E4E04, 0xFF4E4E04, 0xFF4D4E02, 0xFF4B4C00, 0xFF4A4B00, 0xFF4A4B09, 0xFF4A4B09, 0xFF4A4B09, 0xFF4A4B09, 0xFF4A4B09, 0xFF4A4B09, 0xFF4A4B09, 0xFF4A4B09, 0xFF494A08, 0xFF494A08, 0xFF494A08, 0xFF484907, 0xFF484907,
        0xFF474806, 0xFF474806, 0xFF474806, 0xFF4C4906, 0xFF4C4906, 0xFF4D4A07, 0xFF4D4A07, 0xFF4E4B08, 0xFF4E4B08, 0xFF4E4B08, 0xFF4E4B08, 0xFF565713, 0xFF595A18, 0xFF585917, 0xFF595A18, 0xFF5C5C1C, 0xFF5C5C1E, 0xFF5B5B1D, 0xFF5C5C1E, 0xFF5B591C,
        0xFF5B591C, 0xFF5C5A1D, 0xFF5F5D20, 0xFF626023, 0xFF636124, 0xFF626023, 0xFF605E21, 0xFF625F1A, 0xFF5F5C17, 0xFF595611, 0xFF54510C, 0xFF524F0A, 0xFF524F0A, 0xFF504D08, 0xFF4D4A05, 0xFF514E07, 0xFF504D06, 0xFF504D06, 0xFF4F4C05, 0xFF4F4C05,
        0xFF4E4B04, 0xFF4E4B04, 0xFF4D4A03, 0xFF4F4800, 0xFF4F4800, 0xFF4E4700, 0xFF4E4700, 0xFF4E4700, 0xFF4E4700, 0xFF4F4800, 0xFF4F4800, 0xFF4F4700, 0xFF4F4700, 0xFF4F4700, 0xFF4F4700, 0xFF504800, 0xFF514900, 0xFF524A01, 0xFF534B02, 0xFF564D08,
        0xFF514901, 0xFF504800, 0xFF504800, 0xFF564F00, 0xFF585100, 0xFF66600C, 0xFF8D8731, 0xFF464D45, 0xFF0A110A, 0xFF1A231E, 0xFF343D3A, 0xFF384243, 0xFF384244, 0xFF3C474B, 0xFF323D43, 0xFF36403F, 0x00959E9D, 0x00CACECF, 0x00FEFFFF,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFEFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF0F1008, 0xFF060B05, 0xFF0D1110, 0xFF0A0B0F, 0xFF121214, 0xFF1E1E16, 0xFF55553D, 0xFF54552B, 0xFF474A15, 0xFF4C4818, 0xFF4B4717, 0xFF484414, 0xFF464212, 0xFF454111,
        0xFF454111, 0xFF464212, 0xFF474313, 0xFF45440E, 0xFF45440E, 0xFF46450F, 0xFF46450F, 0xFF46450F, 0xFF45440E, 0xFF44430D, 0xFF44430D, 0xFF46460A, 0xFF47470B, 0xFF49490D, 0xFF4A4A0E, 0xFF4A4A0E, 0xFF49490D, 0xFF48480C, 0xFF48480C, 0xFF47480C,
        0xFF48490D, 0xFF48490D, 0xFF48490D, 0xFF47480C, 0xFF46470B, 0xFF47480C, 0xFF48490D, 0xFF4D4C20, 0xFF48471B, 0xFF47461A, 0xFF4A491D, 0xFF49481C, 0xFF4B4A1E, 0xFF5B5A2E, 0xFF6E6D41, 0xFF606C3A, 0xFF626E3C, 0xFF65713F, 0xFF65713F, 0xFF64703E,
        0xFF626E3C, 0xFF616D3B, 0xFF606C3A, 0xFF727353, 0xFF6B6C4C, 0xFF616242, 0xFF646546, 0xFF646546, 0xFF5A5B3B, 0xFF535434, 0xFF4B4C2C, 0xFF393F1B, 0xFF2F3513, 0xFF262B0B, 0xFF202507, 0xFF282D0D, 0xFF434923, 0xFF4B5226, 0xFF38400F, 0xFF43460D,
        0xFF40430A, 0xFF3F4209, 0xFF41450A, 0xFF42460B, 0xFF43470A, 0xFF44480B, 0xFF464A0D, 0xFF44480B, 0xFF44480B, 0xFF45490C, 0xFF45490C, 0xFF45490C, 0xFF464A0D, 0xFF464A0D, 0xFF464A0D, 0xFF42460B, 0xFF45490E, 0xFF484C11, 0xFF494D12, 0xFF494D12,
        0xFF484C11, 0xFF494D12, 0xFF4B4F14, 0xFF43470A, 0xFF45490C, 0xFF474B0E, 0xFF474B0E, 0xFF45490C, 0xFF43470A, 0xFF424609, 0xFF424609, 0xFF43470C, 0xFF42460B, 0xFF41450A, 0xFF42460B, 0xFF43470C, 0xFF43470C, 0xFF43470C, 0xFF41450A, 0xFF3B3F0D,
        0xFF3B3F0D, 0xFF3A3E0C, 0xFF3A3E0C, 0xFF393D0B, 0xFF393D0B, 0xFF383C0A, 0xFF383C0A, 0xFF3A3E0B, 0xFF3A3E0B, 0xFF3A3E0B, 0xFF3A3E0B, 0xFF3A3E0B, 0xFF3A3E0B, 0xFF393D0A, 0xFF393D0A, 0xFF3E420F, 0xFF3E420F, 0xFF3E420F, 0xFF3D410E, 0xFF3D410E,
        0xFF3D410E, 0xFF3D410E, 0xFF3D410E, 0xFF434611, 0xFF434611, 0xFF434611, 0xFF434611, 0xFF424510, 0xFF424510, 0xFF424510, 0xFF424510, 0xFF424905, 0xFF434A06, 0xFF444B07, 0xFF454C08, 0xFF464D09, 0xFF454C08, 0xFF444B07, 0xFF434A06, 0xFF444906,
        0xFF444906, 0xFF444906, 0xFF444906, 0xFF444906, 0xFF444906, 0xFF444906, 0xFF444906, 0xFF434A06, 0xFF434A06, 0xFF434A06, 0xFF434A06, 0xFF434A06, 0xFF434A06, 0xFF434A06, 0xFF434A06, 0xFF434805, 0xFF434805, 0xFF434805, 0xFF444906, 0xFF444906,
        0xFF444906, 0xFF444906, 0xFF444906, 0xFF434A06, 0xFF434A06, 0xFF434A06, 0xFF434A06, 0xFF434A06, 0xFF424905, 0xFF424905, 0xFF424905, 0xFF42480A, 0xFF42480A, 0xFF42480A, 0xFF42480A, 0xFF42480A, 0xFF42480A, 0xFF42480A, 0xFF42480A, 0xFF41450A,
        0xFF42460B, 0xFF44480D, 0xFF464A0D, 0xFF464A0D, 0xFF44480B, 0xFF424707, 0xFF414606, 0xFF3D4400, 0xFF3D4400, 0xFF3D4400, 0xFF3D4400, 0xFF3D4400, 0xFF3D4400, 0xFF3D4400, 0xFF3D4400, 0xFF404703, 0xFF404703, 0xFF404703, 0xFF404703, 0xFF3F4602,
        0xFF3F4602, 0xFF3F4602, 0xFF3F4602, 0xFF424802, 0xFF424802, 0xFF424802, 0xFF424802, 0xFF424802, 0xFF434903, 0xFF434903, 0xFF444A04, 0xFF46491C, 0xFF484B1E, 0xFF4C4F22, 0xFF4E5122, 0xFF4C501E, 0xFF4D511F, 0xFF505421, 0xFF4F5320, 0xFF515322,
        0xFF505221, 0xFF505221, 0xFF515322, 0xFF525423, 0xFF525423, 0xFF525423, 0xFF515322, 0xFF4E5311, 0xFF4C5111, 0xFF4A4F0F, 0xFF494D10, 0xFF474A11, 0xFF444710, 0xFF454811, 0xFF484B16, 0xFF44480D, 0xFF44480D, 0xFF44480D, 0xFF44480D, 0xFF44480D,
        0xFF44480D, 0xFF44480D, 0xFF44480D, 0xFF45490C, 0xFF45490C, 0xFF44480B, 0xFF44480B, 0xFF44480B, 0xFF44480B, 0xFF45490C, 0xFF45490C, 0xFF47480C, 0xFF47480C, 0xFF47480C, 0xFF48490D, 0xFF494A0E, 0xFF4A4B0F, 0xFF4B4C10, 0xFF4C4D11, 0xFF505117,
        0xFF4E4F13, 0xFF4D4E12, 0xFF4B4D0E, 0xFF505210, 0xFF535611, 0xFF5C5F1A, 0xFF767932, 0xFF353C35, 0xFF111811, 0xFF131C17, 0xFF1D2623, 0xFF1D2625, 0xFF192324, 0xFF252F31, 0xFF1A2328, 0xFF1B2524, 0xFF363F3E, 0xFF494D4E, 0xFFBDBEC0,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFB, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF0E0F07, 0xFF090E07, 0xFF0C110D, 0xFF08090B, 0xFF0F1014, 0xFF060604, 0xFF141408, 0xFF111300, 0xFF0C0F00, 0xFF0F1201, 0xFF0E1100, 0xFF0C0F00, 0xFF0A0D00, 0xFF090C00,
        0xFF080B00, 0xFF090C00, 0xFF090C00, 0xFF091000, 0xFF091000, 0xFF091000, 0xFF091000, 0xFF080F00, 0xFF070E00, 0xFF070E00, 0xFF060D00, 0xFF070B00, 0xFF070B00, 0xFF080C00, 0xFF090D00, 0xFF090D00, 0xFF090D00, 0xFF090D00, 0xFF080C00, 0xFF050A00,
        0xFF060B00, 0xFF070C00, 0xFF080D00, 0xFF080D00, 0xFF080D00, 0xFF080D00, 0xFF080D00, 0xFF080D00, 0xFF060B00, 0xFF040900, 0xFF050A00, 0xFF0C1100, 0xFF131801, 0xFF141902, 0xFF111600, 0xFF132203, 0xFF152405, 0xFF172607, 0xFF192809, 0xFF192809,
        0xFF192809, 0xFF192809, 0xFF1A290A, 0xFF11130E, 0xFF1F2219, 0xFF292D1E, 0xFF222615, 0xFF171B0A, 0xFF1B1F10, 0xFF1A1D14, 0xFF040601, 0xFF050B07, 0xFF030905, 0xFF050C05, 0xFF040B04, 0xFF010801, 0xFF050B07, 0xFF050B09, 0xFF000301, 0xFF030B00,
        0xFF000700, 0xFF000700, 0xFF030B00, 0xFF050D00, 0xFF030B00, 0xFF010900, 0xFF030B00, 0xFF060A00, 0xFF060A00, 0xFF060A00, 0xFF070B00, 0xFF070B00, 0xFF080C00, 0xFF080C00, 0xFF080C00, 0xFF010300, 0xFF040600, 0xFF060800, 0xFF060800, 0xFF040600,
        0xFF040600, 0xFF060800, 0xFF090B00, 0xFF030700, 0xFF050900, 0xFF080C00, 0xFF080C00, 0xFF070B00, 0xFF050900, 0xFF040800, 0xFF040800, 0xFF050700, 0xFF040600, 0xFF030500, 0xFF040600, 0xFF050700, 0xFF050700, 0xFF040600, 0xFF030500, 0xFF030A00,
        0xFF020900, 0xFF020900, 0xFF010800, 0xFF010800, 0xFF000700, 0xFF000700, 0xFF000700, 0xFF040800, 0xFF040800, 0xFF050900, 0xFF050900, 0xFF050900, 0xFF060A00, 0xFF060A00, 0xFF060A00, 0xFF050900, 0xFF050900, 0xFF050900, 0xFF050900, 0xFF040800,
        0xFF040800, 0xFF040800, 0xFF040800, 0xFF050800, 0xFF050800, 0xFF050800, 0xFF050800, 0xFF050800, 0xFF050800, 0xFF050800, 0xFF050800, 0xFF050800, 0xFF050800, 0xFF060900, 0xFF060900, 0xFF060900, 0xFF060900, 0xFF060900, 0xFF050800, 0xFF070800,
        0xFF070800, 0xFF070800, 0xFF070800, 0xFF070800, 0xFF070800, 0xFF070800, 0xFF070800, 0xFF060900, 0xFF060900, 0xFF060900, 0xFF060900, 0xFF060900, 0xFF060900, 0xFF060900, 0xFF060900, 0xFF080901, 0xFF080901, 0xFF080901, 0xFF080901, 0xFF080901,
        0xFF090A02, 0xFF090A02, 0xFF090A02, 0xFF070A01, 0xFF070A01, 0xFF070A01, 0xFF080B02, 0xFF080B02, 0xFF080B02, 0xFF090C03, 0xFF090C03, 0xFF060B00, 0xFF060B00, 0xFF060B00, 0xFF060B00, 0xFF060B00, 0xFF060B00, 0xFF060B00, 0xFF060B00, 0xFF040700,
        0xFF050800, 0xFF060900, 0xFF070A00, 0xFF070B00, 0xFF060A00, 0xFF050900, 0xFF040800, 0xFF060900, 0xFF060900, 0xFF060900, 0xFF060900, 0xFF060900, 0xFF060900, 0xFF060900, 0xFF060900, 0xFF060900, 0xFF060900, 0xFF060900, 0xFF060900, 0xFF050800,
        0xFF050800, 0xFF050800, 0xFF050800, 0xFF090B00, 0xFF080A00, 0xFF080A00, 0xFF070900, 0xFF070900, 0xFF080A00, 0xFF090B00, 0xFF090B00, 0xFF0A0F00, 0xFF090E00, 0xFF151B01, 0xFF1D2307, 0xFF181F00, 0xFF161D00, 0xFF1C2400, 0xFF1E2601, 0xFF1E2503,
        0xFF1D2402, 0xFF1C2301, 0xFF1C2301, 0xFF1C2301, 0xFF1D2402, 0xFF1E2503, 0xFF1E2503, 0xFF1E2600, 0xFF1E2500, 0xFF1F2600, 0xFF1C2200, 0xFF131900, 0xFF090E00, 0xFF080C00, 0xFF0E1200, 0xFF090C00, 0xFF090C00, 0xFF090C00, 0xFF0A0D00, 0xFF0A0D00,
        0xFF0A0D00, 0xFF0A0D00, 0xFF0A0D00, 0xFF0B1000, 0xFF0B1000, 0xFF0B1000, 0xFF0B1000, 0xFF0B1000, 0xFF0B1000, 0xFF0B1000, 0xFF0B1000, 0xFF0A0F00, 0xFF0A0F00, 0xFF0A0F00, 0xFF0B1000, 0xFF0C1100, 0xFF0E1300, 0xFF0F1400, 0xFF101500, 0xFF131802,
        0xFF171C06, 0xFF181D06, 0xFF171D03, 0xFF181E04, 0xFF111700, 0xFF0D1300, 0xFF151C00, 0xFF121814, 0xFF0C120E, 0xFF131917, 0xFF141D1C, 0xFF131C1B, 0xFF0B1315, 0xFF162021, 0xFF081214, 0xFF08130F, 0xFF0A1310, 0xFF000302, 0xFF828483,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FDFDFB, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF0E0F07, 0xFF090F05, 0xFF0E130D, 0xFF0F1312, 0xFF191A1E, 0xFF121315, 0xFF141613, 0xFF191C11, 0xFF161A0B, 0xFF000300, 0xFF000400, 0xFF000400, 0xFF000602, 0xFF010703,
        0xFF020804, 0xFF030905, 0xFF030905, 0xFF020B06, 0xFF020B06, 0xFF020B06, 0xFF030C07, 0xFF040D08, 0xFF050E09, 0xFF060F0A, 0xFF060F0A, 0xFF080D10, 0xFF070C0F, 0xFF070C0F, 0xFF070C0F, 0xFF070C0F, 0xFF080D10, 0xFF080D10, 0xFF090E11, 0xFF070F11,
        0xFF070F11, 0xFF091113, 0xFF0A1214, 0xFF0C1416, 0xFF0C1416, 0xFF0C1416, 0xFF0C1416, 0xFF0F190E, 0xFF0C160B, 0xFF0C160B, 0xFF172116, 0xFF2B352A, 0xFF3B453A, 0xFF3C463B, 0xFF364035, 0xFF3F4537, 0xFF404638, 0xFF414739, 0xFF414739, 0xFF414739,
        0xFF43493B, 0xFF454B3D, 0xFF484E40, 0xFF525B48, 0xFF454F36, 0xFF414C2C, 0xFF44502A, 0xFF44502A, 0xFF3B4626, 0xFF202A11, 0xFF060F00, 0xFF0D140D, 0xFF10170F, 0xFF10180D, 0xFF121A0D, 0xFF121A0F, 0xFF0C130C, 0xFF0E1412, 0xFF1B2023, 0xFF151E23,
        0xFF131C21, 0xFF151F21, 0xFF1B2527, 0xFF1C2627, 0xFF162021, 0xFF121C1D, 0xFF111B1C, 0xFF13171A, 0xFF13171A, 0xFF13171A, 0xFF13171A, 0xFF14181B, 0xFF14181B, 0xFF14181B, 0xFF14181B, 0xFF262B24, 0xFF272C25, 0xFF262B24, 0xFF222720, 0xFF1D221B,
        0xFF1B2019, 0xFF1C211A, 0xFF1F241D, 0xFF1F2326, 0xFF1F2326, 0xFF1F2326, 0xFF1D2124, 0xFF1A1E21, 0xFF181C1F, 0xFF171B1E, 0xFF171B1E, 0xFF191E17, 0xFF181D16, 0xFF171C15, 0xFF171C15, 0xFF171C15, 0xFF181D16, 0xFF161B14, 0xFF151A13, 0xFF161F1C,
        0xFF161F1C, 0xFF161F1C, 0xFF151E1B, 0xFF151E1B, 0xFF141D1A, 0xFF141D1A, 0xFF141D1A, 0xFF131917, 0xFF141A18, 0xFF151B19, 0xFF161C1A, 0xFF171D1B, 0xFF181E1C, 0xFF181E1C, 0xFF191F1D, 0xFF141A18, 0xFF141A18, 0xFF141A18, 0xFF131917, 0xFF131917,
        0xFF131917, 0xFF131917, 0xFF131917, 0xFF161A19, 0xFF161A19, 0xFF171B1A, 0xFF171B1A, 0xFF181C1B, 0xFF181C1B, 0xFF191D1C, 0xFF191D1C, 0xFF161D15, 0xFF161D15, 0xFF151C14, 0xFF141B13, 0xFF141B13, 0xFF151C14, 0xFF151C14, 0xFF161D15, 0xFF161B14,
        0xFF161B14, 0xFF161B14, 0xFF161B14, 0xFF161B14, 0xFF161B14, 0xFF161B14, 0xFF161B14, 0xFF151C14, 0xFF151C14, 0xFF151C14, 0xFF151C14, 0xFF151C14, 0xFF151C14, 0xFF151C14, 0xFF151C14, 0xFF171C15, 0xFF171C15, 0xFF171C15, 0xFF181D16, 0xFF181D16,
        0xFF181D16, 0xFF181D16, 0xFF191E17, 0xFF161D15, 0xFF171E16, 0xFF171E16, 0xFF171E16, 0xFF181F17, 0xFF181F17, 0xFF192018, 0xFF192018, 0xFF191E21, 0xFF191E21, 0xFF191E21, 0xFF191E21, 0xFF191E21, 0xFF191E21, 0xFF191E21, 0xFF191E21, 0xFF181B20,
        0xFF181B20, 0xFF181B20, 0xFF181C1F, 0xFF181C1F, 0xFF181C1D, 0xFF181C1D, 0xFF181C1D, 0xFF1A2119, 0xFF1A2119, 0xFF1A2119, 0xFF1A2119, 0xFF1A2119, 0xFF1A2119, 0xFF1A2119, 0xFF1A2119, 0xFF192018, 0xFF1A2119, 0xFF1A2119, 0xFF1A2119, 0xFF1A2119,
        0xFF1A2119, 0xFF1A2119, 0xFF1A2119, 0xFF1D2319, 0xFF1D2319, 0xFF1C2218, 0xFF1B2117, 0xFF1B2117, 0xFF1B2117, 0xFF1C2218, 0xFF1D2319, 0xFF1A211A, 0xFF141B13, 0xFF1F271C, 0xFF31392C, 0xFF313A29, 0xFF2D3623, 0xFF303924, 0xFF323C24, 0xFF2E3724,
        0xFF2F3825, 0xFF303926, 0xFF313A27, 0xFF333C29, 0xFF363F2C, 0xFF39422F, 0xFF3B4431, 0xFF39412C, 0xFF363D2B, 0xFF353C2C, 0xFF343A2E, 0xFF2C312A, 0xFF202521, 0xFF181C1B, 0xFF171B1C, 0xFF1C1F24, 0xFF1C1F24, 0xFF1B1E23, 0xFF191C21, 0xFF181B20,
        0xFF16191E, 0xFF15181D, 0xFF14171C, 0xFF11191B, 0xFF11191B, 0xFF121A1C, 0xFF121A1C, 0xFF121A1C, 0xFF121A1C, 0xFF11191B, 0xFF11191B, 0xFF11191B, 0xFF10181A, 0xFF11191B, 0xFF11191B, 0xFF121A1C, 0xFF131B1D, 0xFF141C1E, 0xFF151D1F, 0xFF161E20,
        0xFF1C2426, 0xFF1D2527, 0xFF1E2628, 0xFF1C2524, 0xFF141D1C, 0xFF0C1514, 0xFF0A1312, 0xFF0E1211, 0xFF151918, 0xFF171D1D, 0xFF171D1D, 0xFF1B2121, 0xFF121B1A, 0xFF1B2423, 0xFF0A1214, 0xFF040F0B, 0xFF121B18, 0xFF030706, 0xFF7F8180,
  0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FEFEFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF0F1008, 0xFF080E04, 0xFF0F140E, 0xFF151918, 0xFF15191A, 0xFF181C1D, 0xFF232726, 0xFF2B312D, 0xFF222922, 0xFF212B20, 0xFF222C21, 0xFF242E23, 0xFF252F24, 0xFF263025,
        0xFF263025, 0xFF273126, 0xFF273126, 0xFF2B3439, 0xFF2B3439, 0xFF2A3338, 0xFF2A3338, 0xFF2A3338, 0xFF2B3439, 0xFF2B3439, 0xFF2C353A, 0xFF313737, 0xFF303636, 0xFF303636, 0xFF2F3535, 0xFF2F3535, 0xFF303636, 0xFF323838, 0xFF333939, 0xFF2F3837,
        0xFF2F3837, 0xFF303938, 0xFF323B3A, 0xFF343D3C, 0xFF353E3D, 0xFF343D3C, 0xFF333C3B, 0xFF38413E, 0xFF2F3835, 0xFF303936, 0xFF3E4744, 0xFF48514E, 0xFF47504D, 0xFF444D4A, 0xFF454E4B, 0xFF484944, 0xFF484944, 0xFF474843, 0xFF464742, 0xFF454641,
        0xFF464742, 0xFF484944, 0xFF4B4C47, 0xFF3C463D, 0xFF3D483A, 0xFF414D39, 0xFF4A573D, 0xFF4E5B41, 0xFF313D29, 0xFF0E190B, 0xFF101A11, 0xFF282E2A, 0xFF333935, 0xFF353C35, 0xFF363D35, 0xFF3E453D, 0xFF3D443D, 0xFF3B413F, 0xFF434949, 0xFF3B4547,
        0xFF3C4647, 0xFF3F494A, 0xFF434D4E, 0xFF424C4B, 0xFF3E4847, 0xFF38433F, 0xFF36413D, 0xFF343A3A, 0xFF343A3A, 0xFF343A3A, 0xFF343A3A, 0xFF343A3A, 0xFF343A3A, 0xFF343A3A, 0xFF343A3A, 0xFF404642, 0xFF3F4541, 0xFF3D433F, 0xFF393F3B, 0xFF353B37,
        0xFF343A36, 0xFF363C38, 0xFF383E3A, 0xFF323838, 0xFF323838, 0xFF323838, 0xFF303636, 0xFF2E3434, 0xFF2D3333, 0xFF2E3434, 0xFF2F3535, 0xFF2F3531, 0xFF2D332F, 0xFF2C322E, 0xFF2C322E, 0xFF2D332F, 0xFF2D332F, 0xFF2C322E, 0xFF2A302C, 0xFF252F26,
        0xFF252F26, 0xFF252F26, 0xFF242E25, 0xFF242E25, 0xFF232D24, 0xFF232D24, 0xFF232D24, 0xFF212B22, 0xFF212B22, 0xFF222C23, 0xFF232D24, 0xFF242E25, 0xFF252F26, 0xFF252F26, 0xFF263027, 0xFF222C23, 0xFF222C23, 0xFF222C23, 0xFF222C23, 0xFF212B22,
        0xFF212B22, 0xFF212B22, 0xFF212B22, 0xFF1E251D, 0xFF1E251D, 0xFF1F261E, 0xFF20271F, 0xFF212820, 0xFF212820, 0xFF222921, 0xFF222921, 0xFF222933, 0xFF212832, 0xFF202731, 0xFF1F2630, 0xFF1E252F, 0xFF1F2630, 0xFF202731, 0xFF212832, 0xFF20242F,
        0xFF20242F, 0xFF20242F, 0xFF20242F, 0xFF20242F, 0xFF20242F, 0xFF20242F, 0xFF20242F, 0xFF1E252F, 0xFF1E252F, 0xFF1E252F, 0xFF1E252F, 0xFF1E252F, 0xFF1E252F, 0xFF1E252F, 0xFF1E252F, 0xFF20242F, 0xFF20242F, 0xFF20242F, 0xFF20242F, 0xFF212530,
        0xFF212530, 0xFF212530, 0xFF212530, 0xFF1E252F, 0xFF1E252F, 0xFF1F2630, 0xFF1F2630, 0xFF1F2630, 0xFF202731, 0xFF202731, 0xFF202731, 0xFF1F2827, 0xFF1F2827, 0xFF1F2827, 0xFF1F2827, 0xFF1F2827, 0xFF1F2827, 0xFF1F2827, 0xFF1F2827, 0xFF212629,
        0xFF212629, 0xFF1F2525, 0xFF1F2525, 0xFF1F2523, 0xFF1F2523, 0xFF212725, 0xFF212723, 0xFF1D242E, 0xFF1D242E, 0xFF1D242E, 0xFF1D242E, 0xFF1D242E, 0xFF1D242E, 0xFF1D242E, 0xFF1D242E, 0xFF1E252F, 0xFF1E252F, 0xFF1F2630, 0xFF1F2630, 0xFF1F2630,
        0xFF1F2630, 0xFF202731, 0xFF202731, 0xFF232730, 0xFF232730, 0xFF22262F, 0xFF22262F, 0xFF22262F, 0xFF22262F, 0xFF22262F, 0xFF22262F, 0xFF1C2522, 0xFF131C19, 0xFF19221F, 0xFF2C3530, 0xFF364038, 0xFF333D35, 0xFF2F3930, 0xFF2D372E, 0xFF2F3931,
        0xFF303A32, 0xFF313B33, 0xFF313B33, 0xFF323C34, 0xFF343E36, 0xFF374139, 0xFF39433B, 0xFF444A48, 0xFF3E4442, 0xFF3B4141, 0xFF3F4447, 0xFF43484C, 0xFF3F444A, 0xFF343841, 0xFF2B2F38, 0xFF2B3131, 0xFF2B3131, 0xFF2A3030, 0xFF2A3030, 0xFF292F2F,
        0xFF282E2E, 0xFF272D2D, 0xFF272D2D, 0xFF282E2E, 0xFF282E2E, 0xFF282E2E, 0xFF292F2F, 0xFF292F2F, 0xFF282E2E, 0xFF282E2E, 0xFF282E2E, 0xFF27302F, 0xFF27302F, 0xFF27302F, 0xFF27302F, 0xFF27302F, 0xFF283130, 0xFF293231, 0xFF2A3332, 0xFF283130,
        0xFF2C3534, 0xFF293231, 0xFF2B3433, 0xFF2C3534, 0xFF293231, 0xFF2C3534, 0xFF2A3332, 0xFF2C3031, 0xFF2E3233, 0xFF272B2C, 0xFF1F2525, 0xFF272D2D, 0xFF242A2A, 0xFF262F2E, 0xFF131C19, 0xFF010C06, 0xFF111A15, 0xFF010602, 0xFF727471,
  0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FEFEFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF11120A, 0xFF0C110A, 0xFF10150F, 0xFF1B201C, 0xFF121816, 0xFF141A18, 0xFF19221F, 0xFF1F2823, 0xFF17201B, 0xFF1B2318, 0xFF1B2318, 0xFF1A2217, 0xFF1A2217, 0xFF192116,
        0xFF192116, 0xFF192116, 0xFF192116, 0xFF182120, 0xFF192221, 0xFF192221, 0xFF182120, 0xFF182120, 0xFF17201F, 0xFF151E1D, 0xFF151E1D, 0xFF171C15, 0xFF171C15, 0xFF171C15, 0xFF171C15, 0xFF171C15, 0xFF181D16, 0xFF181D16, 0xFF181D16, 0xFF151C14,
        0xFF151C14, 0xFF161D15, 0xFF171E16, 0xFF192018, 0xFF192018, 0xFF161D15, 0xFF141B13, 0xFF141A18, 0xFF111715, 0xFF161C1A, 0xFF252B29, 0xFF2D3331, 0xFF2B312F, 0xFF292F2D, 0xFF2D3331, 0xFF2D3130, 0xFF2E3231, 0xFF2D3130, 0xFF2C302F, 0xFF2B2F2E,
        0xFF2B2F2E, 0xFF2D3130, 0xFF2F3332, 0xFF2B2F3A, 0xFF2D313A, 0xFF2E3337, 0xFF2C3134, 0xFF313639, 0xFF24292D, 0xFF0A0E17, 0xFF0B0F1A, 0xFF090C11, 0xFF111419, 0xFF13161B, 0xFF0E1116, 0xFF0F1217, 0xFF13171A, 0xFF111516, 0xFF0C100F, 0xFF0C1510,
        0xFF0F1813, 0xFF111A15, 0xFF101914, 0xFF0F1911, 0xFF0F1911, 0xFF0D170E, 0xFF0A140B, 0xFF0F160E, 0xFF0E150D, 0xFF0E150D, 0xFF0D140C, 0xFF0D140C, 0xFF0C130B, 0xFF0C130B, 0xFF0C130B, 0xFF090E12, 0xFF090E12, 0xFF080D11, 0xFF070C10, 0xFF060B0F,
        0xFF070C10, 0xFF0A0F13, 0xFF0B1014, 0xFF0B120A, 0xFF0B120A, 0xFF0C130B, 0xFF0C130B, 0xFF0B120A, 0xFF0C130B, 0xFF0E150D, 0xFF0F160E, 0xFF0E1116, 0xFF0D1015, 0xFF0B0E13, 0xFF0B0E13, 0xFF0C0F14, 0xFF0C0F14, 0xFF0A0D12, 0xFF090C11, 0xFF0F1209,
        0xFF0F1209, 0xFF0E1108, 0xFF0E1108, 0xFF0D1007, 0xFF0D1007, 0xFF0D1007, 0xFF0C0F06, 0xFF091008, 0xFF091008, 0xFF091008, 0xFF0A1109, 0xFF0A1109, 0xFF0A1109, 0xFF0A1109, 0xFF0A1109, 0xFF091008, 0xFF091008, 0xFF091008, 0xFF091008, 0xFF080F07,
        0xFF080F07, 0xFF080F07, 0xFF080F07, 0xFF090E08, 0xFF090E08, 0xFF090E08, 0xFF0A0F09, 0xFF0A0F09, 0xFF0A0F09, 0xFF0B100A, 0xFF0B100A, 0xFF0E1609, 0xFF0D1508, 0xFF0D1508, 0xFF0C1407, 0xFF0C1407, 0xFF0C1407, 0xFF0C1407, 0xFF0C1407, 0xFF0B1105,
        0xFF0B1105, 0xFF0B1105, 0xFF0B1105, 0xFF0B1105, 0xFF0B1105, 0xFF0B1105, 0xFF0B1105, 0xFF0A1205, 0xFF0A1205, 0xFF0A1205, 0xFF0A1205, 0xFF0A1205, 0xFF0A1205, 0xFF0A1205, 0xFF0A1205, 0xFF0A1004, 0xFF0A1004, 0xFF0B1105, 0xFF0B1105, 0xFF0B1105,
        0xFF0B1105, 0xFF0C1206, 0xFF0C1206, 0xFF0A1205, 0xFF0A1205, 0xFF0A1205, 0xFF0A1205, 0xFF0A1205, 0xFF0B1306, 0xFF0B1306, 0xFF0B1306, 0xFF091008, 0xFF091008, 0xFF091008, 0xFF091008, 0xFF091008, 0xFF091008, 0xFF091008, 0xFF091008, 0xFF0A0F0B,
        0xFF090E08, 0xFF090E08, 0xFF080D07, 0xFF080D06, 0xFF090E07, 0xFF090F05, 0xFF0A1006, 0xFF0A1205, 0xFF0A1205, 0xFF0A1205, 0xFF0A1205, 0xFF0A1205, 0xFF0A1205, 0xFF0A1205, 0xFF0A1205, 0xFF070F02, 0xFF070F02, 0xFF070F02, 0xFF081003, 0xFF081003,
        0xFF091104, 0xFF091104, 0xFF091104, 0xFF091102, 0xFF0A1203, 0xFF0A1203, 0xFF0A1203, 0xFF091102, 0xFF081001, 0xFF081001, 0xFF070F00, 0xFF0B110D, 0xFF0B110D, 0xFF0E1412, 0xFF171D1B, 0xFF212727, 0xFF212629, 0xFF1B2023, 0xFF191E21, 0xFF1B2325,
        0xFF1C2426, 0xFF1D2527, 0xFF1C2426, 0xFF1A2224, 0xFF1A2224, 0xFF1B2325, 0xFF1C2426, 0xFF1E2326, 0xFF1C2124, 0xFF191E21, 0xFF1B2023, 0xFF1F2427, 0xFF1E2326, 0xFF151B1B, 0xFF0B1111, 0xFF080F08, 0xFF080F08, 0xFF080F08, 0xFF091009, 0xFF091009,
        0xFF091009, 0xFF0A110A, 0xFF0A110A, 0xFF0D120B, 0xFF0E130C, 0xFF0E130C, 0xFF0F140D, 0xFF0F140D, 0xFF0E130C, 0xFF0E130C, 0xFF0D120B, 0xFF0D140D, 0xFF0D140D, 0xFF0C130C, 0xFF0C130C, 0xFF0C130C, 0xFF0C130C, 0xFF0D140D, 0xFF0D140D, 0xFF0B120A,
        0xFF0C130C, 0xFF070E07, 0xFF0C130C, 0xFF0D130F, 0xFF0C120E, 0xFF131917, 0xFF131917, 0xFF16171B, 0xFF17181C, 0xFF191D20, 0xFF141819, 0xFF1B2121, 0xFF181E1C, 0xFF1B211F, 0xFF0A1310, 0xFF0D1812, 0xFF101914, 0xFF000400, 0xFF696B68,
  0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FEFEFC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF12130B, 0xFF0D120B, 0xFF0D120C, 0xFF1F2420, 0xFF141A16, 0xFF0C1510, 0xFF030C07, 0xFF040F09, 0xFF05100A, 0xFF0E1213, 0xFF0D1112, 0xFF0C1011, 0xFF0B0F10, 0xFF0C1011,
        0xFF0D1112, 0xFF0F1314, 0xFF101415, 0xFF0C1208, 0xFF0E140A, 0xFF11170D, 0xFF13190F, 0xFF151B11, 0xFF151B11, 0xFF141A10, 0xFF13190F, 0xFF161614, 0xFF171715, 0xFF181816, 0xFF191917, 0xFF1A1A18, 0xFF191917, 0xFF181816, 0xFF181816, 0xFF1A1C19,
        0xFF191B18, 0xFF191B18, 0xFF1A1C19, 0xFF1B1D1A, 0xFF1A1C19, 0xFF171916, 0xFF141613, 0xFF161815, 0xFF151714, 0xFF141613, 0xFF131512, 0xFF161815, 0xFF191B18, 0xFF161815, 0xFF121411, 0xFF101916, 0xFF111A17, 0xFF131C19, 0xFF131C19, 0xFF131C19,
        0xFF131C19, 0xFF141D1A, 0xFF161F1C, 0xFF171D1B, 0xFF101614, 0xFF1C2220, 0xFF191F1D, 0xFF101614, 0xFF121816, 0xFF0B110F, 0xFF070D0B, 0xFF0A0D06, 0xFF11130E, 0xFF1C1E1B, 0xFF1E201D, 0xFF1A1C17, 0xFF212419, 0xFF252918, 0xFF1C210D, 0xFF1E2223,
        0xFF222627, 0xFF232726, 0xFF1E2221, 0xFF1D221E, 0xFF202521, 0xFF222723, 0xFF20251F, 0xFF222824, 0xFF222824, 0xFF212723, 0xFF202622, 0xFF1F2521, 0xFF1E2420, 0xFF1E2420, 0xFF1D231F, 0xFF1F271C, 0xFF1E261B, 0xFF1D251A, 0xFF1D251A, 0xFF1E261B,
        0xFF1E261B, 0xFF1F271C, 0xFF1F271C, 0xFF1E2420, 0xFF1F2521, 0xFF1F2521, 0xFF1E2420, 0xFF1C221E, 0xFF1B211D, 0xFF1B211D, 0xFF1C221E, 0xFF1B2117, 0xFF1A2016, 0xFF181E14, 0xFF181E14, 0xFF191F15, 0xFF181E14, 0xFF171D13, 0xFF161C12, 0xFF18171C,
        0xFF18171C, 0xFF18171C, 0xFF17161B, 0xFF17161B, 0xFF16151A, 0xFF16151A, 0xFF16151A, 0xFF0F1418, 0xFF0E1317, 0xFF0E1317, 0xFF0D1216, 0xFF0D1216, 0xFF0C1115, 0xFF0C1115, 0xFF0B1014, 0xFF0B1014, 0xFF0B1014, 0xFF0B1014, 0xFF0B1014, 0xFF0A0F13,
        0xFF0A0F13, 0xFF0A0F13, 0xFF0A0F13, 0xFF0A0D14, 0xFF0A0D14, 0xFF0A0D14, 0xFF0A0D14, 0xFF0A0D14, 0xFF0A0D14, 0xFF090C13, 0xFF090C13, 0xFF070D0D, 0xFF070D0D, 0xFF080E0E, 0xFF080E0E, 0xFF080E0E, 0xFF070D0D, 0xFF050B0B, 0xFF040A0A, 0xFF080C0D,
        0xFF080C0D, 0xFF080C0D, 0xFF080C0D, 0xFF080C0D, 0xFF080C0D, 0xFF080C0D, 0xFF080C0D, 0xFF070D0D, 0xFF070D0D, 0xFF070D0D, 0xFF070D0D, 0xFF070D0D, 0xFF070D0D, 0xFF070D0D, 0xFF070D0D, 0xFF060A0B, 0xFF060A0B, 0xFF070B0C, 0xFF070B0C, 0xFF070B0C,
        0xFF070B0C, 0xFF070B0C, 0xFF080C0D, 0xFF060C0C, 0xFF060C0C, 0xFF060C0C, 0xFF060C0C, 0xFF060C0C, 0xFF060C0C, 0xFF060C0C, 0xFF060C0C, 0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF080E0A, 0xFF080C0B,
        0xFF080C0B, 0xFF090E0A, 0xFF090E0A, 0xFF090E0A, 0xFF090E08, 0xFF080D07, 0xFF080D07, 0xFF080E0E, 0xFF080E0E, 0xFF080E0E, 0xFF080E0E, 0xFF080E0E, 0xFF080E0E, 0xFF080E0E, 0xFF080E0E, 0xFF070D0D, 0xFF080E0E, 0xFF080E0E, 0xFF080E0E, 0xFF090F0F,
        0xFF090F0F, 0xFF0A1010, 0xFF0A1010, 0xFF0A100E, 0xFF0A100E, 0xFF0B110F, 0xFF0B110F, 0xFF0A100E, 0xFF090F0D, 0xFF080E0C, 0xFF070D0B, 0xFF000600, 0xFF0E150E, 0xFF111713, 0xFF0D1311, 0xFF111619, 0xFF12171B, 0xFF10151B, 0xFF12161F, 0xFF0D1518,
        0xFF0E1619, 0xFF10181B, 0xFF0F171A, 0xFF0E1619, 0xFF0E1619, 0xFF10181B, 0xFF11191C, 0xFF111715, 0xFF151B19, 0xFF171D19, 0xFF141B14, 0xFF121912, 0xFF121911, 0xFF0F170C, 0xFF0B1308, 0xFF0F1513, 0xFF0E1412, 0xFF0E1412, 0xFF0D1311, 0xFF0C1210,
        0xFF0B110F, 0xFF0B110F, 0xFF0A100E, 0xFF0E0E0C, 0xFF0E0E0C, 0xFF0F0F0D, 0xFF10100E, 0xFF10100E, 0xFF0F0F0D, 0xFF0E0E0C, 0xFF0E0E0C, 0xFF0C0E0D, 0xFF0B0D0C, 0xFF0A0C0B, 0xFF0A0C0B, 0xFF090B0A, 0xFF090B0A, 0xFF0A0C0B, 0xFF0A0C0B, 0xFF070906,
        0xFF090B08, 0xFF050704, 0xFF0C0E0D, 0xFF0B0C0E, 0xFF040507, 0xFF08090D, 0xFF05060A, 0xFF06070C, 0xFF07080C, 0xFF0C1013, 0xFF030708, 0xFF040A0A, 0xFF060C0A, 0xFF131915, 0xFF121B16, 0xFF121D17, 0xFF0F1813, 0xFF060B07, 0xFF727471,
  0x00FFFFFF, 0x00FEFEFE, 0x00FFFFFF, 0x00FAF9FE, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF030500, 0xFF171A0F, 0xFF1E2116, 0xFF262920, 0xFF21241B, 0xFF22251C, 0xFF2B2E27, 0xFF2D3029, 0xFF33362F, 0xFF2C362E, 0xFF2C362E, 0xFF2D372F, 0xFF2E3830, 0xFF303A32,
        0xFF343E36, 0xFF374139, 0xFF39433B, 0xFF3A4244, 0xFF3A4244, 0xFF3A4244, 0xFF3B4443, 0xFF3D4645, 0xFF404948, 0xFF434C49, 0xFF454E4B, 0xFF404F4C, 0xFF42514E, 0xFF43524F, 0xFF42514E, 0xFF41504D, 0xFF404F4C, 0xFF404F4C, 0xFF404F4C, 0xFF414D4B,
        0xFF3F4B49, 0xFF3D4947, 0xFF3B4745, 0xFF394543, 0xFF35413F, 0xFF303C3A, 0xFF2C3836, 0xFF2B322B, 0xFF2A312A, 0xFF1C231C, 0xFF0F160F, 0xFF121912, 0xFF1B221B, 0xFF1D241D, 0xFF1C231C, 0xFF1A211A, 0xFF1C231C, 0xFF1E251E, 0xFF1F261F, 0xFF1E251E,
        0xFF1E251E, 0xFF1E251E, 0xFF1E251E, 0xFF1A201C, 0xFF1E2420, 0xFF1F2521, 0xFF1C231C, 0xFF171E17, 0xFF121912, 0xFF0D140C, 0xFF080F07, 0xFF0F1817, 0xFF262F2E, 0xFF343D3C, 0xFF363F3E, 0xFF394241, 0xFF3B4443, 0xFF3A4342, 0xFF3B4443, 0xFF3A4644,
        0xFF394543, 0xFF384442, 0xFF384442, 0xFF384442, 0xFF394543, 0xFF3A4644, 0xFF3A4644, 0xFF3F4B4B, 0xFF3D4949, 0xFF394545, 0xFF354141, 0xFF333F3F, 0xFF313D3D, 0xFF303C3C, 0xFF303C3C, 0xFF333F3F, 0xFF303C3C, 0xFF313D3D, 0xFF354141, 0xFF364242,
        0xFF323E3E, 0xFF303C3C, 0xFF313D3D, 0xFF343E3F, 0xFF323C3D, 0xFF303A3B, 0xFF2E3839, 0xFF2E3839, 0xFF2C3637, 0xFF2A3435, 0xFF283233, 0xFF223030, 0xFF233131, 0xFF233131, 0xFF233131, 0xFF233131, 0xFF233131, 0xFF233131, 0xFF233131, 0xFF232D2C,
        0xFF222C2B, 0xFF202A29, 0xFF202A29, 0xFF202A29, 0xFF1F2928, 0xFF1E2827, 0xFF1C2625, 0xFF1B2526, 0xFF192324, 0xFF172122, 0xFF162021, 0xFF162021, 0xFF151F20, 0xFF131D1E, 0xFF111B1C, 0xFF0C1A1A, 0xFF0D1B1B, 0xFF0F1D1D, 0xFF0F1D1D, 0xFF0E1C1C,
        0xFF0E1C1C, 0xFF0F1D1D, 0xFF101E1E, 0xFF111B1C, 0xFF101A1B, 0xFF0F191A, 0xFF0E1819, 0xFF0E1819, 0xFF0E1819, 0xFF0F191A, 0xFF101A1B, 0xFF0F1918, 0xFF101A19, 0xFF101A19, 0xFF111B1A, 0xFF121C1B, 0xFF121C1B, 0xFF111B1A, 0xFF111B1A, 0xFF121B1A,
        0xFF121B1A, 0xFF121B1A, 0xFF121B1A, 0xFF111A19, 0xFF111A19, 0xFF111A19, 0xFF111A19, 0xFF101A19, 0xFF101A19, 0xFF101A19, 0xFF101A19, 0xFF0F1918, 0xFF0F1918, 0xFF0F1918, 0xFF0F1918, 0xFF101A19, 0xFF101A19, 0xFF101A19, 0xFF101A19, 0xFF0F1918,
        0xFF0F1918, 0xFF0F1918, 0xFF0F1918, 0xFF0F1918, 0xFF101A19, 0xFF101A19, 0xFF101A19, 0xFF101A19, 0xFF0F1918, 0xFF0E1817, 0xFF0E1817, 0xFF0D1517, 0xFF0D1517, 0xFF0D1517, 0xFF0D1517, 0xFF0D1517, 0xFF0D1517, 0xFF0D1517, 0xFF0D1517, 0xFF0D1716,
        0xFF0F1918, 0xFF101A19, 0xFF101A19, 0xFF0F1918, 0xFF0E1817, 0xFF0E1817, 0xFF0F1918, 0xFF0F1918, 0xFF0F1918, 0xFF101A19, 0xFF101A19, 0xFF101A19, 0xFF101A19, 0xFF101A19, 0xFF111B1A, 0xFF131D1C, 0xFF121C1B, 0xFF111B1A, 0xFF121C1B, 0xFF141E1D,
        0xFF151F1E, 0xFF151F1E, 0xFF131D1C, 0xFF111B1A, 0xFF121C1B, 0xFF131D1C, 0xFF141E1D, 0xFF141E1D, 0xFF131D1C, 0xFF121C1B, 0xFF111B1A, 0xFF0E1712, 0xFF121B16, 0xFF161F1A, 0xFF17201B, 0xFF151E19, 0xFF151E19, 0xFF17201B, 0xFF19221D, 0xFF171D13,
        0xFF1B2117, 0xFF1C2218, 0xFF191F15, 0xFF181E14, 0xFF1A2016, 0xFF181E14, 0xFF13190F, 0xFF11191B, 0xFF0C1416, 0xFF0D1517, 0xFF151D1F, 0xFF161E20, 0xFF121A1C, 0xFF131B1D, 0xFF192123, 0xFF192525, 0xFF192525, 0xFF1A2626, 0xFF1B2727, 0xFF1C2828,
        0xFF1C2828, 0xFF1C2828, 0xFF1C2828, 0xFF1C2A1D, 0xFF1C2A1D, 0xFF1D2B1E, 0xFF1D2B1E, 0xFF1D2B1E, 0xFF1C2A1D, 0xFF1B291C, 0xFF1A281B, 0xFF222C23, 0xFF242E25, 0xFF242E25, 0xFF222C23, 0xFF1E281F, 0xFF1B251C, 0xFF1C261D, 0xFF1E281F, 0xFF1C2723,
        0xFF1A2521, 0xFF17221E, 0xFF15201C, 0xFF15201C, 0xFF16211D, 0xFF15201C, 0xFF15201C, 0xFF191F1D, 0xFF161C1A, 0xFF141A18, 0xFF151B19, 0xFF181E1C, 0xFF1C2220, 0xFF1F2523, 0xFF202624, 0xFF1A1F19, 0xFF171C16, 0xFF070C06, 0x00BCC1BB,
  0x00FFFFFF, 0x00FDFDFD, 0x00FFFFFF, 0x00FFFEFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF14160B, 0xFF0E100F, 0xFF1A1C1B, 0xFF2B2C2E, 0xFF28292B, 0xFF26272B, 0xFF2F3034, 0xFF303136, 0xFF313237, 0xFF292E32, 0xFF292E32, 0xFF272C30, 0xFF262B2F, 0xFF262B2F,
        0xFF262B2F, 0xFF272C30, 0xFF282D31, 0xFF282E2E, 0xFF292F2F, 0xFF2A3030, 0xFF2B312F, 0xFF2A302E, 0xFF292F2B, 0xFF282E2A, 0xFF272D29, 0xFF262F2C, 0xFF27302D, 0xFF28312E, 0xFF27302D, 0xFF242D2A, 0xFF222B28, 0xFF222B28, 0xFF222B28, 0xFF1C2522,
        0xFF1C2522, 0xFF1B2421, 0xFF1B2421, 0xFF1A2320, 0xFF19221F, 0xFF161F1C, 0xFF141D1A, 0xFF151A14, 0xFF1B201A, 0xFF1A1F19, 0xFF181D17, 0xFF212620, 0xFF272C26, 0xFF272C26, 0xFF262B25, 0xFF222721, 0xFF232822, 0xFF252A24, 0xFF242923, 0xFF232822,
        0xFF212620, 0xFF20251F, 0xFF20251F, 0xFF252A26, 0xFF242925, 0xFF212622, 0xFF1E231D, 0xFF1D221C, 0xFF1B201A, 0xFF131811, 0xFF0A0F08, 0xFF060C0A, 0xFF0E1412, 0xFF0D1311, 0xFF0C1210, 0xFF131917, 0xFF181E1C, 0xFF181E1C, 0xFF181E1C, 0xFF131C19,
        0xFF121B18, 0xFF121B18, 0xFF111A17, 0xFF111A17, 0xFF101916, 0xFF101916, 0xFF111A17, 0xFF101916, 0xFF101916, 0xFF0F1815, 0xFF0E1714, 0xFF0D1613, 0xFF0D1613, 0xFF0E1714, 0xFF0F1815, 0xFF131C19, 0xFF111A17, 0xFF111A17, 0xFF131C19, 0xFF141D1A,
        0xFF121B18, 0xFF101916, 0xFF0F1815, 0xFF0F1513, 0xFF0E1412, 0xFF0E1412, 0xFF0E1412, 0xFF101614, 0xFF101614, 0xFF101614, 0xFF0F1513, 0xFF101914, 0xFF101914, 0xFF101914, 0xFF111A15, 0xFF121B16, 0xFF121B16, 0xFF131C17, 0xFF131C17, 0xFF141B14,
        0xFF131A13, 0xFF121912, 0xFF121912, 0xFF141B14, 0xFF141B14, 0xFF131A13, 0xFF121912, 0xFF131915, 0xFF121814, 0xFF111713, 0xFF111713, 0xFF121814, 0xFF121814, 0xFF111713, 0xFF101612, 0xFF0F1813, 0xFF0E1712, 0xFF0D1611, 0xFF0B140F, 0xFF09120D,
        0xFF0B140F, 0xFF0E1712, 0xFF111A15, 0xFF101612, 0xFF101612, 0xFF0F1511, 0xFF0F1511, 0xFF0F1511, 0xFF0F1511, 0xFF0F1511, 0xFF0F1511, 0xFF101710, 0xFF111811, 0xFF111811, 0xFF121912, 0xFF121912, 0xFF121912, 0xFF121912, 0xFF111811, 0xFF121711,
        0xFF121711, 0xFF121711, 0xFF121711, 0xFF111610, 0xFF111610, 0xFF111610, 0xFF111610, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0F160F, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0C130C, 0xFF0C130C, 0xFF0C130C, 0xFF0D140D, 0xFF0D140D,
        0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF101710, 0xFF111811, 0xFF111811, 0xFF111811, 0xFF111811, 0xFF101710, 0xFF101710, 0xFF0F160F, 0xFF0F1410, 0xFF0F1410, 0xFF0F1410, 0xFF0F1410, 0xFF0F1410, 0xFF0F1410, 0xFF0F1410, 0xFF0F1410, 0xFF0C130C,
        0xFF0D140D, 0xFF0E150E, 0xFF0E150E, 0xFF0E150E, 0xFF0D140D, 0xFF0D140D, 0xFF0D140D, 0xFF101710, 0xFF101710, 0xFF101710, 0xFF101710, 0xFF101710, 0xFF111811, 0xFF111811, 0xFF111811, 0xFF131A13, 0xFF121912, 0xFF121912, 0xFF121912, 0xFF131A13,
        0xFF131A13, 0xFF131A13, 0xFF131A13, 0xFF151C15, 0xFF161D16, 0xFF171E17, 0xFF171E17, 0xFF171E17, 0xFF171E17, 0xFF161D16, 0xFF151C15, 0xFF111610, 0xFF191E18, 0xFF232822, 0xFF292E28, 0xFF2B302A, 0xFF292E28, 0xFF272C26, 0xFF252A24, 0xFF292A2E,
        0xFF2A2B2F, 0xFF2B2C30, 0xFF2A2B2F, 0xFF292A2E, 0xFF2A2B2F, 0xFF292A2E, 0xFF26272B, 0xFF1C211D, 0xFF1D221E, 0xFF232824, 0xFF2B302C, 0xFF282D29, 0xFF1D221E, 0xFF131814, 0xFF101511, 0xFF131C19, 0xFF131C19, 0xFF141D1A, 0xFF151E1B, 0xFF151E1B,
        0xFF151E1B, 0xFF151E1B, 0xFF151E1B, 0xFF171F21, 0xFF171F21, 0xFF182022, 0xFF182022, 0xFF171F21, 0xFF161E20, 0xFF141C1E, 0xFF131B1D, 0xFF161B1E, 0xFF181D20, 0xFF191E21, 0xFF181D20, 0xFF161B1E, 0xFF151A1D, 0xFF171C1F, 0xFF181D20, 0xFF18211C,
        0xFF161F1A, 0xFF151E19, 0xFF151E19, 0xFF161F1A, 0xFF17201B, 0xFF17201B, 0xFF161F1A, 0xFF191F1B, 0xFF191F1B, 0xFF1A201C, 0xFF1B211D, 0xFF1C221E, 0xFF1D231F, 0xFF1D231F, 0xFF1C221E, 0xFF171C16, 0xFF000200, 0xFF676C66, 0x00F9FEF8,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FEFDFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0xFF8F9186, 0xFF191B18, 0xFF070906, 0xFF050704, 0xFF020403, 0xFF000100, 0xFF030504, 0xFF030406, 0xFF020305, 0xFF000103, 0xFF000103, 0xFF000103, 0xFF000103, 0xFF010204,
        0xFF010204, 0xFF020305, 0xFF020305, 0xFF000200, 0xFF010300, 0xFF030502, 0xFF050702, 0xFF060803, 0xFF060803, 0xFF050801, 0xFF040700, 0xFF080A05, 0xFF0A0C07, 0xFF0B0D08, 0xFF0C0E09, 0xFF0C0E09, 0xFF0B0D08, 0xFF0C0E09, 0xFF0D0F0A, 0xFF0F100B,
        0xFF10110C, 0xFF12130E, 0xFF141510, 0xFF151611, 0xFF171813, 0xFF181914, 0xFF1A1B16, 0xFF191A15, 0xFF1F201B, 0xFF21221D, 0xFF23241F, 0xFF2A2B26, 0xFF2A2B26, 0xFF272823, 0xFF282924, 0xFF2A2B26, 0xFF2C2D28, 0xFF2E2F2A, 0xFF2D2E29, 0xFF2C2D28,
        0xFF2A2B26, 0xFF292A25, 0xFF292A25, 0xFF282826, 0xFF272725, 0xFF252523, 0xFF242520, 0xFF272823, 0xFF292A25, 0xFF262721, 0xFF1F201A, 0xFF1D1F1A, 0xFF1F211C, 0xFF1D1F1A, 0xFF1F211C, 0xFF272924, 0xFF272924, 0xFF20221D, 0xFF1C1E19, 0xFF23241F,
        0xFF23241F, 0xFF22231E, 0xFF22231E, 0xFF20211C, 0xFF1F201B, 0xFF1E1F1A, 0xFF1D1E19, 0xFF202119, 0xFF21221A, 0xFF202119, 0xFF1D1E16, 0xFF191A12, 0xFF16170F, 0xFF16170F, 0xFF16170F, 0xFF181911, 0xFF181911, 0xFF171810, 0xFF15160E, 0xFF16170F,
        0xFF171810, 0xFF15160E, 0xFF12130B, 0xFF15150D, 0xFF14140C, 0xFF14140C, 0xFF14140C, 0xFF16160E, 0xFF17170F, 0xFF16160E, 0xFF15150D, 0xFF16130C, 0xFF15120B, 0xFF15120B, 0xFF15120B, 0xFF15120B, 0xFF15120B, 0xFF15120B, 0xFF15120B, 0xFF121208,
        0xFF111107, 0xFF101006, 0xFF111107, 0xFF121208, 0xFF121208, 0xFF121208, 0xFF101006, 0xFF12120A, 0xFF111109, 0xFF111109, 0xFF111109, 0xFF13130B, 0xFF13130B, 0xFF13130B, 0xFF12120A, 0xFF131009, 0xFF131009, 0xFF131009, 0xFF131009, 0xFF14110A,
        0xFF17140D, 0xFF1C1912, 0xFF201D16, 0xFF101008, 0xFF101008, 0xFF101008, 0xFF101008, 0xFF101008, 0xFF101008, 0xFF0F0F07, 0xFF0F0F07, 0xFF101006, 0xFF101006, 0xFF101006, 0xFF111107, 0xFF101006, 0xFF101006, 0xFF0F0F05, 0xFF0F0F05, 0xFF100D04,
        0xFF100D04, 0xFF100D04, 0xFF100D04, 0xFF100D04, 0xFF0F0C03, 0xFF0F0C03, 0xFF0F0C03, 0xFF101006, 0xFF101006, 0xFF101006, 0xFF0F0F05, 0xFF0F0F05, 0xFF0F0F05, 0xFF0F0F05, 0xFF0E0E04, 0xFF101006, 0xFF101006, 0xFF101006, 0xFF101006, 0xFF101006,
        0xFF101006, 0xFF101006, 0xFF101006, 0xFF121208, 0xFF131309, 0xFF131309, 0xFF131309, 0xFF131309, 0xFF121208, 0xFF111107, 0xFF111107, 0xFF120F08, 0xFF120F08, 0xFF120F08, 0xFF120F08, 0xFF120F08, 0xFF120F08, 0xFF120F08, 0xFF120F08, 0xFF14140A,
        0xFF131309, 0xFF14140A, 0xFF14140A, 0xFF15150B, 0xFF15150B, 0xFF14140A, 0xFF131309, 0xFF101006, 0xFF101006, 0xFF111107, 0xFF111107, 0xFF111107, 0xFF111107, 0xFF121208, 0xFF121208, 0xFF121208, 0xFF121208, 0xFF131309, 0xFF121208, 0xFF111107,
        0xFF111107, 0xFF111107, 0xFF121208, 0xFF101006, 0xFF101006, 0xFF101006, 0xFF111107, 0xFF111107, 0xFF101006, 0xFF101006, 0xFF101006, 0xFF100D06, 0xFF100D06, 0xFF0E0B04, 0xFF0C0902, 0xFF090600, 0xFF070400, 0xFF060300, 0xFF050200, 0xFF030102,
        0xFF020001, 0xFF020001, 0xFF030102, 0xFF030102, 0xFF020001, 0xFF020001, 0xFF030102, 0xFF030000, 0xFF040100, 0xFF070400, 0xFF0B0801, 0xFF0D0A03, 0xFF0D0A03, 0xFF0C0902, 0xFF0D0A03, 0xFF080901, 0xFF090A02, 0xFF090A02, 0xFF0A0B03, 0xFF0A0B03,
        0xFF0A0B03, 0xFF090A02, 0xFF090A02, 0xFF0B0B0B, 0xFF0B0B0B, 0xFF0B0B0B, 0xFF0B0B0B, 0xFF0A0A0A, 0xFF090909, 0xFF070707, 0xFF060606, 0xFF010204, 0xFF020305, 0xFF020305, 0xFF020305, 0xFF020305, 0xFF020305, 0xFF020305, 0xFF030406, 0xFF000500,
        0xFF000400, 0xFF000300, 0xFF000300, 0xFF000400, 0xFF000500, 0xFF000400, 0xFF000300, 0xFF000300, 0xFF000300, 0xFF000500, 0xFF000500, 0xFF000600, 0xFF000600, 0xFF000700, 0xFF000700, 0xFF191E18, 0x0060655F, 0x00E6EBE5, 0x00FCFFFB,
  0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF
};

static GUI_CONST_STORAGE GUI_BITMAP _bmJTrace_300x164 = {
  300,                                 // XSize
  164,                                 // YSize
  1200,                                // BytesPerLine
  32,                                  // BitsPerPixel
  (unsigned char *)_acJTrace_300x164,  // Pointer to picture data
  NULL,                                // Pointer to palette
  GUI_DRAW_BMPM8888I
};
#else
static GUI_CONST_STORAGE unsigned long _acJTrace_300x164[] = {
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00838571, 0x00A5A793, 0x00A6A894, 0x009FA18D, 0x009C9E8A, 0x00959783, 0x009C9E8A, 0x009A9C88, 0x00898D70, 0x008A8E71, 0x008C9073, 0x008F9376, 0x00929679,
        0x00929679, 0x00909477, 0x008D9174, 0x0083886B, 0x00858A6D, 0x00868B6E, 0x0084896C, 0x00818669, 0x00808568, 0x0082876A, 0x00858A6D, 0x00858A6B, 0x00838869, 0x00868B6C, 0x00828768, 0x007F8465, 0x007F8465, 0x00797E5F, 0x007B8061, 0x006F7457,
        0x00707558, 0x006D7255, 0x006C7154, 0x0072775A, 0x00767B5E, 0x00757A5D, 0x0074795C, 0x00737863, 0x00747964, 0x00696E59, 0x006A6C58, 0x00686A56, 0x00666854, 0x00696A56, 0x005B5C48, 0x005E5E52, 0x005F5F53, 0x00626256, 0x00646458, 0x00626256,
        0x005F5F53, 0x00616155, 0x00656559, 0x006B6C5C, 0x006A6B5B, 0x00646555, 0x005C5C4E, 0x005F5F51, 0x00646456, 0x005C5C50, 0x00515145, 0x0070755A, 0x006D7257, 0x00676C51, 0x006E7358, 0x0070755A, 0x00666B50, 0x0061664B, 0x005C6146, 0x00585D42,
        0x00585D42, 0x00575C41, 0x00555A3F, 0x0054593E, 0x00565B40, 0x00575C41, 0x00565B40, 0x004B5035, 0x0050553A, 0x00555A3F, 0x0053583D, 0x004C5136, 0x00474C31, 0x00484D32, 0x004A4F34, 0x00494A3A, 0x00474838, 0x00474838, 0x004B4C3C, 0x004E4F3F,
        0x004E4F3F, 0x004D4E3E, 0x004C4D3D, 0x004E5338, 0x0052573C, 0x0053583D, 0x00555A3F, 0x00585D42, 0x0054593E, 0x004F5439, 0x0050553A, 0x004F513D, 0x004F513D, 0x004F513D, 0x004D4F3B, 0x00494B37, 0x00454733, 0x00424430, 0x0041432F, 0x00464835,
        0x00444633, 0x00424431, 0x00414330, 0x00424431, 0x00434532, 0x00444633, 0x00444633, 0x0040422E, 0x003E402C, 0x003D3F2B, 0x003E402C, 0x003E402C, 0x003B3D29, 0x00393B27, 0x003A3C28, 0x003F412E, 0x003A3C29, 0x00363825, 0x00373926, 0x00383A27,
        0x00373926, 0x00353724, 0x00343623, 0x003D3F2B, 0x003A3C28, 0x00353723, 0x00333521, 0x00323420, 0x0031331F, 0x002F311D, 0x002D2F1B, 0x00292A21, 0x00292A21, 0x00292A21, 0x002A2B22, 0x002A2B22, 0x002A2B22, 0x002A2B22, 0x002B2C23, 0x0032332A,
        0x00292A21, 0x0024251C, 0x0027281F, 0x00292A21, 0x0026271E, 0x0023241B, 0x0023241B, 0x0028291F, 0x0027281E, 0x0026271D, 0x0025261C, 0x0026271D, 0x0025261C, 0x0024251B, 0x00222319, 0x001F2017, 0x001E1F16, 0x001D1E15, 0x001E1F16, 0x00202118,
        0x00212219, 0x0022231A, 0x0022231A, 0x002A2B22, 0x00282920, 0x0026271E, 0x0023241B, 0x0022231A, 0x0022231A, 0x0023241B, 0x0023241B, 0x0022231A, 0x0022231A, 0x0022231A, 0x0022231A, 0x0022231A, 0x0023241B, 0x0023241B, 0x0023241B, 0x0022231A,
        0x0023241B, 0x0024251C, 0x0025261D, 0x0026271E, 0x0027281F, 0x0027281F, 0x0027281F, 0x00282A24, 0x00272923, 0x00242620, 0x0022241E, 0x0022241E, 0x0022241E, 0x0023251F, 0x00242620, 0x002A2B22, 0x00292A21, 0x00292A21, 0x00282920, 0x00282920,
        0x00292A21, 0x00292A21, 0x002A2B22, 0x002E2F25, 0x002F3026, 0x00303127, 0x00323329, 0x0034352B, 0x0036372D, 0x0037382E, 0x0038392F, 0x00414036, 0x003A392F, 0x00414135, 0x004C4C40, 0x00474739, 0x003D3D2F, 0x003B3B2D, 0x003E3F2F, 0x003A382D,
        0x003A382D, 0x0039372C, 0x003A382D, 0x003B392E, 0x003C3A2F, 0x003C3A2F, 0x003B392E, 0x00434333, 0x003D3C2E, 0x003F3E30, 0x00403F31, 0x00454338, 0x003E3C31, 0x002F2D23, 0x003E3C32, 0x002B2E1F, 0x002A2D1E, 0x00292C1D, 0x002A2D1E, 0x002B2E1F,
        0x002C2F20, 0x002C2F20, 0x002C2F20, 0x00272A1B, 0x00282B1C, 0x00292C1D, 0x002B2E1F, 0x002D3021, 0x002D3021, 0x002C2F20, 0x002B2E1F, 0x002F3027, 0x002E2F26, 0x002C2D24, 0x002B2C23, 0x002B2C23, 0x002D2E25, 0x002F3027, 0x00313229, 0x0038362E,
        0x003A3830, 0x0039372F, 0x0034322A, 0x00323028, 0x0035332B, 0x0038362E, 0x0039372F, 0x003C3D33, 0x003E3F35, 0x003E3F35, 0x003A3B31, 0x003A3B31, 0x00404137, 0x00424339, 0x00424339, 0x00595D51, 0x00A2A69B, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFDFEFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x007B7D77, 0x006E705C, 0x00727460, 0x00525440, 0x00323420, 0x00333521, 0x003A3C28, 0x003B3D29, 0x00383A26, 0x00454234, 0x00444133, 0x00434032, 0x00444133, 0x00454234,
        0x00444133, 0x00403D2F, 0x003C392B, 0x00434335, 0x00454537, 0x00464638, 0x00444436, 0x00404032, 0x003E3E30, 0x00404032, 0x00434335, 0x00484838, 0x00434333, 0x00484838, 0x004A4A3A, 0x00515141, 0x00575747, 0x00515141, 0x004F4F3F, 0x004B4B3D,
        0x004A4A3C, 0x0049493B, 0x004B4B3D, 0x00515143, 0x00565648, 0x00575749, 0x00565648, 0x005B5C52, 0x00595A50, 0x0053544A, 0x0075746A, 0x0085847A, 0x00747369, 0x00727066, 0x0069675D, 0x006F6F61, 0x00717163, 0x00767668, 0x007A7A6C, 0x007A7A6C,
        0x0078786A, 0x007C7C6E, 0x00818173, 0x00858773, 0x007F816D, 0x007A7C68, 0x00757764, 0x007B7D6A, 0x00787969, 0x00606151, 0x00505141, 0x00605F55, 0x005E5D53, 0x00515046, 0x00504F45, 0x00535248, 0x004F4E44, 0x00504F45, 0x004E4D43, 0x004F4E44,
        0x004E4D43, 0x004D4C42, 0x004B4A40, 0x004B4A40, 0x004C4B41, 0x004E4D43, 0x004E4D43, 0x0045443A, 0x0048473D, 0x004A493F, 0x0049483E, 0x0046453B, 0x00444339, 0x0045443A, 0x0048473D, 0x00484541, 0x0043403C, 0x00403D39, 0x00413E3A, 0x0044413D,
        0x0045423E, 0x0045423E, 0x0044413D, 0x00444339, 0x00424137, 0x00403F35, 0x00424137, 0x00434238, 0x00403F35, 0x003C3B31, 0x003C3B31, 0x003D3E2E, 0x003D3E2E, 0x003E3F2F, 0x003E3F2F, 0x003D3E2E, 0x003C3D2D, 0x003B3C2C, 0x003A3B2B, 0x00343426,
        0x00323224, 0x00303022, 0x00303022, 0x00323224, 0x00343426, 0x00353527, 0x00353527, 0x00373926, 0x00363825, 0x00353724, 0x00363825, 0x00363825, 0x00353724, 0x00333522, 0x00323421, 0x00353527, 0x00313123, 0x002D2D1F, 0x002B2B1D, 0x002B2B1D,
        0x002B2B1D, 0x002B2B1D, 0x002C2C1E, 0x002A2C19, 0x00282A17, 0x00262815, 0x00272916, 0x00292B18, 0x002A2C19, 0x00282A17, 0x00252714, 0x0023241B, 0x0023241B, 0x0024251C, 0x0024251C, 0x0024251C, 0x0024251C, 0x0024251C, 0x0024251C, 0x0024251C,
        0x001C1D14, 0x00191A11, 0x001E1F16, 0x00202118, 0x001C1D14, 0x001C1D14, 0x00212219, 0x00252819, 0x00232617, 0x00222516, 0x00222516, 0x00232617, 0x00232617, 0x00212415, 0x00202314, 0x0024251C, 0x0024251C, 0x0022231A, 0x0022231A, 0x0022231A,
        0x0022231A, 0x0022231A, 0x0022231A, 0x0022231A, 0x00202118, 0x001E1F16, 0x001C1D14, 0x001B1C13, 0x001B1C13, 0x001B1C13, 0x001B1C13, 0x001A1B12, 0x001A1B12, 0x001B1C13, 0x001B1C13, 0x001B1C13, 0x001B1C13, 0x001B1C13, 0x001C1D14, 0x001A1B12,
        0x001A1B12, 0x001B1C13, 0x001C1D14, 0x001D1E15, 0x001E1F16, 0x001F2017, 0x001F2017, 0x0020221C, 0x001F211B, 0x001D1F19, 0x001C1E18, 0x001B1D17, 0x001B1D17, 0x001C1E18, 0x001D1F19, 0x00202118, 0x00202118, 0x001F2017, 0x001F2017, 0x001F2017,
        0x00202118, 0x00202118, 0x00212219, 0x0024251B, 0x0024251B, 0x0025261C, 0x0026271D, 0x0027281E, 0x0028291F, 0x0028291F, 0x0028291F, 0x001E1D0F, 0x0039382A, 0x00656456, 0x00848474, 0x00888878, 0x00848472, 0x00828270, 0x0080806E, 0x00807D6E,
        0x00807D6E, 0x00817E6F, 0x00827F70, 0x00848172, 0x00858273, 0x00868374, 0x00868374, 0x0082806E, 0x0083816F, 0x008C8A78, 0x00888576, 0x00858273, 0x00605D4F, 0x00242113, 0x001D1A0C, 0x0022201F, 0x00211F1E, 0x001F1D1C, 0x001E1C1B, 0x001E1C1B,
        0x001F1D1C, 0x001F1D1C, 0x001F1D1C, 0x00191716, 0x00171514, 0x00151312, 0x00161413, 0x00181615, 0x001B1918, 0x001C1A19, 0x001C1A19, 0x001B1C13, 0x00191A11, 0x0017180F, 0x0015160D, 0x0015160D, 0x0016170E, 0x00181910, 0x001A1B12, 0x00191A11,
        0x001D1E15, 0x001C1D14, 0x0016170E, 0x00111209, 0x000F1007, 0x000F1007, 0x000E0F06, 0x000A0B01, 0x00121309, 0x0013140A, 0x00121309, 0x0013140A, 0x0018190F, 0x00202117, 0x00212218, 0x002E3423, 0x003B3F33, 0x00B9BDB2, 0xFFFFFFFB,
  0xFFFFFFFF, 0xFFFCFCFC, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x0041453A, 0x0075746A, 0x0016150B, 0x00040300, 0x00141309, 0x00232218, 0x003C3B31, 0x00444339, 0x00434238, 0x00433D38, 0x00423C37, 0x00403A35, 0x003F3934, 0x003E3833,
        0x003D3732, 0x003A342F, 0x0038322D, 0x002F2C27, 0x00312E29, 0x00322F2A, 0x00302D28, 0x002C2924, 0x00292621, 0x002A2722, 0x002B2823, 0x002B2820, 0x0025221A, 0x0026231B, 0x0025221A, 0x0027241C, 0x0028251D, 0x001F1C14, 0x001D1A12, 0x00201D18,
        0x001C1914, 0x001A1712, 0x001B1813, 0x001D1A15, 0x001E1B16, 0x001E1B16, 0x001C1914, 0x0013140A, 0x0013140A, 0x0017180E, 0x00504F45, 0x007B7A70, 0x0078776D, 0x007B796F, 0x007C7A70, 0x00767567, 0x00767567, 0x00777668, 0x00787769, 0x00747365,
        0x00706F61, 0x00706F61, 0x00747365, 0x00737660, 0x00727460, 0x00777965, 0x00787A66, 0x00797B68, 0x00696B58, 0x004C4D3D, 0x00464737, 0x00050100, 0x00181413, 0x00171312, 0x00130F0E, 0x00120E0D, 0x00100C0B, 0x0014100F, 0x00151110, 0x00161211,
        0x00151110, 0x00130F0E, 0x00130F0E, 0x0014100F, 0x00151110, 0x00161211, 0x00181413, 0x00161211, 0x00151110, 0x00130F0E, 0x00130F0E, 0x0014100F, 0x00161211, 0x00161211, 0x00171312, 0x00191611, 0x0017140F, 0x0014110C, 0x0014110C, 0x0016130E,
        0x0017140F, 0x0015120D, 0x00120F0A, 0x000E0A09, 0x00110D0C, 0x00161211, 0x00181413, 0x00151110, 0x00130F0E, 0x00130F0E, 0x00130F0E, 0x00131109, 0x00131109, 0x00121008, 0x00121008, 0x00131109, 0x00131109, 0x0014120A, 0x0014120A, 0x0016130E,
        0x0013100B, 0x00110E09, 0x00100D08, 0x00120F0A, 0x0013100B, 0x0013100B, 0x00120F0A, 0x000E0D03, 0x000D0C02, 0x000D0C02, 0x000D0C02, 0x000F0E04, 0x00100F05, 0x000E0D03, 0x000C0B01, 0x000F0C07, 0x0013100B, 0x0016130E, 0x0016130E, 0x0014110C,
        0x0013100B, 0x0016130E, 0x00181510, 0x00141309, 0x00111006, 0x000D0C02, 0x000D0C02, 0x000F0E04, 0x00111006, 0x00100F05, 0x000F0E04, 0x000E0F05, 0x000E0F05, 0x000E0F05, 0x000E0F05, 0x000E0F05, 0x000E0F05, 0x000E0F05, 0x000E0F05, 0x000F1006,
        0x000A0B01, 0x000B0C02, 0x000F1006, 0x000E0F05, 0x00090A00, 0x000C0D03, 0x0015160C, 0x000F1202, 0x000E1101, 0x000D1000, 0x000D1000, 0x000F1202, 0x000F1202, 0x000E1101, 0x000D1000, 0x000D0E04, 0x000D0E04, 0x000C0D03, 0x000B0C02, 0x000A0B01,
        0x000A0B01, 0x000A0B01, 0x000A0B01, 0x000D0E04, 0x000D0E04, 0x000C0D03, 0x000C0D03, 0x000B0C02, 0x000A0B01, 0x000A0B01, 0x00090A00, 0x000A0B01, 0x000A0B01, 0x000A0B01, 0x000A0B01, 0x000B0C02, 0x000B0C02, 0x000B0C02, 0x000B0C02, 0x000C0D03,
        0x000C0D03, 0x000C0D03, 0x000C0D03, 0x000C0D03, 0x000D0E04, 0x000D0E04, 0x000E0F05, 0x00090C03, 0x000A0D04, 0x000A0D04, 0x000A0D04, 0x000A0D04, 0x000A0D04, 0x000A0D04, 0x000A0D04, 0x000B0C02, 0x000B0C02, 0x000A0B01, 0x000A0B01, 0x000B0C02,
        0x000B0C02, 0x000C0D03, 0x000D0E04, 0x00080900, 0x00080900, 0x000A0B01, 0x000B0C02, 0x000B0C02, 0x000B0C02, 0x000B0C02, 0x000B0C02, 0x0019180A, 0x00282719, 0x00434333, 0x00585848, 0x005B5B49, 0x005A5A48, 0x005B5C48, 0x005A5B47, 0x00605D4E,
        0x00605D4E, 0x00605D4E, 0x005F5C4D, 0x005F5C4D, 0x005E5B4C, 0x005E5B4C, 0x005E5B4C, 0x0061604C, 0x005E5D49, 0x005E5C4A, 0x005B5947, 0x00666354, 0x00535041, 0x001D1A0C, 0x00151204, 0x00100E0D, 0x00100E0D, 0x00100E0D, 0x00100E0D, 0x00100E0D,
        0x00110F0E, 0x0012100F, 0x00131110, 0x00171514, 0x00151312, 0x00131110, 0x00141211, 0x00161413, 0x00191716, 0x001A1817, 0x001A1817, 0x001E1F15, 0x001E1F15, 0x001E1F15, 0x001E1F15, 0x001E1F15, 0x001F2016, 0x00212218, 0x00212218, 0x001B1C12,
        0x0023241A, 0x002A2B21, 0x002C2D23, 0x002D2E24, 0x00313228, 0x0033342A, 0x0033342A, 0x0037382E, 0x003B3C32, 0x00393A30, 0x0037382E, 0x002F3026, 0x00222319, 0x00191A10, 0x000F1006, 0x002C3221, 0x00323728, 0x006A6E63, 0xFFFFFFF9,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFDFF, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00414637, 0x0042413D, 0x000B0A06, 0x002A2925, 0x0051504C, 0x00676662, 0x00868581, 0x00898884, 0x008B8A86, 0x008E8E82, 0x008E8E82, 0x008E8E82, 0x008E8E82, 0x008E8E82,
        0x008E8E82, 0x008E8E82, 0x008F8F83, 0x008A8D7E, 0x008B8E7F, 0x008C8F80, 0x008B8E7F, 0x00898C7D, 0x00878A7B, 0x0086897A, 0x0086897A, 0x00898979, 0x00898979, 0x008F8F7F, 0x008A8A7A, 0x00868676, 0x00868676, 0x00828272, 0x00868676, 0x0088887A,
        0x00818173, 0x007E7E70, 0x007E7E70, 0x007B7B6D, 0x00777769, 0x00757567, 0x00707062, 0x00727861, 0x006E745D, 0x005C624B, 0x006E715B, 0x00868973, 0x0081846E, 0x007B7D67, 0x00787A64, 0x007E7E6E, 0x007C7C6C, 0x007B7B6B, 0x00797969, 0x00757565,
        0x006E6E5E, 0x006C6C5C, 0x006D6D5D, 0x00757763, 0x006F715D, 0x00737562, 0x00737562, 0x00737464, 0x005D5E4E, 0x00373828, 0x00333325, 0x003D3D31, 0x0067675B, 0x007C7C70, 0x007E7E72, 0x0079796D, 0x00717165, 0x00727266, 0x00737367, 0x00717165,
        0x006F6F63, 0x006E6E62, 0x006F6F63, 0x00707064, 0x006F6F63, 0x00707064, 0x00727266, 0x00747468, 0x00707064, 0x006C6C60, 0x006C6C60, 0x006E6E62, 0x00707064, 0x006F6F63, 0x006D6D61, 0x0071745E, 0x0071745E, 0x0070735D, 0x006E715B, 0x006C6F59,
        0x00686B55, 0x0061644E, 0x005A5D47, 0x0058584C, 0x00636357, 0x006E6E62, 0x006A6A5E, 0x005D5D51, 0x0057574B, 0x0057574B, 0x00555549, 0x00524F4A, 0x00504D48, 0x004D4A45, 0x004B4843, 0x004B4843, 0x004B4843, 0x004B4843, 0x004A4742, 0x00484541,
        0x0046433F, 0x0044413D, 0x00423F3B, 0x00403D39, 0x003E3B37, 0x003D3A36, 0x003B3834, 0x00312E29, 0x00312E29, 0x00302D28, 0x00302D28, 0x0033302B, 0x0036332E, 0x0034312C, 0x00302D28, 0x00312E2A, 0x003C3935, 0x0046433F, 0x00494642, 0x00474440,
        0x0046433F, 0x00474440, 0x00484541, 0x003F3C37, 0x00393631, 0x00322F2A, 0x002D2A25, 0x002C2924, 0x002D2A25, 0x002F2C27, 0x002F2C27, 0x002C2D23, 0x002C2D23, 0x002C2D23, 0x002B2C22, 0x002B2C22, 0x002B2C22, 0x002B2C22, 0x002B2C22, 0x002D2E24,
        0x002C2D23, 0x002E2F25, 0x00303127, 0x002C2D23, 0x0027281E, 0x002B2C22, 0x0033342A, 0x00292C1D, 0x00282B1C, 0x00272A1B, 0x00282B1C, 0x00292C1D, 0x002A2D1E, 0x00292C1D, 0x00282B1C, 0x002C2D23, 0x002C2D23, 0x002D2E24, 0x002C2D23, 0x002B2C22,
        0x002C2D23, 0x002D2E24, 0x002E2F25, 0x002D2E24, 0x002D2E24, 0x002D2E24, 0x002E2F25, 0x002D2E24, 0x002D2E24, 0x002C2D23, 0x002C2D23, 0x002D2E24, 0x002D2E24, 0x002D2E24, 0x002D2E24, 0x002D2E24, 0x002E2F25, 0x002E2F25, 0x002E2F25, 0x00323329,
        0x00313228, 0x00313228, 0x00303127, 0x00303127, 0x00303127, 0x00313228, 0x00313228, 0x002E3128, 0x002E3128, 0x002F3229, 0x0030332A, 0x0030332A, 0x0030332A, 0x002F3229, 0x002F3229, 0x0036372D, 0x0036372D, 0x0036372D, 0x0036372D, 0x0037382E,
        0x0037382E, 0x0038392F, 0x00393A30, 0x00393B2F, 0x003A3C30, 0x003D3F33, 0x003F4135, 0x00404236, 0x00414337, 0x00414337, 0x00404236, 0x003F3F33, 0x00363628, 0x003F3F31, 0x00565648, 0x00626353, 0x00626353, 0x005F614E, 0x005C5E4B, 0x00626252,
        0x00626252, 0x00636353, 0x00636353, 0x00626252, 0x00616151, 0x00626252, 0x00636353, 0x00676755, 0x006D6D5B, 0x00707060, 0x006C6C5C, 0x00757466, 0x006B6A5C, 0x004F4E40, 0x00555348, 0x00515444, 0x00525545, 0x00545747, 0x00555848, 0x00575A4A,
        0x00595C4C, 0x005B5E4E, 0x005D6050, 0x005C5F4F, 0x005D6050, 0x005F6252, 0x00616454, 0x00626555, 0x00626555, 0x00616454, 0x00606353, 0x00606157, 0x00616258, 0x00626359, 0x0065665C, 0x0067685E, 0x0068695F, 0x00696A60, 0x006A6B61, 0x0065695E,
        0x006B6F64, 0x00707469, 0x0071756A, 0x0071756A, 0x0072766B, 0x0072766B, 0x0071756A, 0x00727369, 0x006F7066, 0x006E6F65, 0x006B6C62, 0x0065665C, 0x00525349, 0x00313228, 0x00191A10, 0x0014190A, 0x003F4337, 0x00484B42, 0xFFEDEFE9,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFEFDFF, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00545A49, 0x003A3B32, 0x00414239, 0x0057584F, 0x004F5047, 0x004F5047, 0x0066675E, 0x0067685F, 0x00686960, 0x00676D5C, 0x006A705F, 0x006F7564, 0x00717766, 0x00727867,
        0x00737968, 0x00757B6A, 0x00787E6D, 0x00747865, 0x00757966, 0x00777B68, 0x007A7E6B, 0x007E826F, 0x00808471, 0x00818572, 0x00818572, 0x007C7D69, 0x007E7F6B, 0x00868773, 0x00838470, 0x0080816D, 0x0082836F, 0x0082836F, 0x00898A76, 0x008B8D7A,
        0x00848673, 0x00868875, 0x008C8E7B, 0x008B8D7A, 0x008A8C79, 0x008A8C79, 0x00868875, 0x00848974, 0x00868B76, 0x00707560, 0x00676955, 0x00767864, 0x007C7E6A, 0x00797A66, 0x007C7D69, 0x00817F6D, 0x007F7D6B, 0x00807E6C, 0x00848270, 0x00868472,
        0x00858371, 0x00858371, 0x00868472, 0x00818272, 0x007B7C6C, 0x007D7E6E, 0x00777769, 0x00777769, 0x00606054, 0x00313125, 0x0027271B, 0x0051533F, 0x00757763, 0x00838571, 0x00868874, 0x00878975, 0x00828470, 0x00858773, 0x008A8C78, 0x008A8C78,
        0x00878975, 0x00868874, 0x00888A76, 0x00878975, 0x00848672, 0x00848672, 0x00868874, 0x00767864, 0x00757763, 0x00737561, 0x00737561, 0x00727460, 0x00727460, 0x0071735F, 0x0070725E, 0x0071745E, 0x00747761, 0x00767963, 0x00757862, 0x00757862,
        0x00767963, 0x00737660, 0x006E715B, 0x00747662, 0x00797B67, 0x007A7C68, 0x006F715D, 0x00626450, 0x00666854, 0x006B6D59, 0x00666854, 0x006E6E62, 0x006C6C60, 0x006A6A5E, 0x0069695D, 0x0069695D, 0x0069695D, 0x0067675B, 0x0066665A, 0x00646359,
        0x00646359, 0x00646359, 0x00626157, 0x005E5D53, 0x005B5A50, 0x0059584E, 0x0059584E, 0x0056554B, 0x0056554B, 0x0055544A, 0x00545349, 0x0057564C, 0x005A594F, 0x0058574D, 0x00535248, 0x004A493F, 0x00535248, 0x0059584E, 0x005A594F, 0x0059584E,
        0x005A594F, 0x005A594F, 0x0059584E, 0x005A594F, 0x0057564C, 0x00545349, 0x00525147, 0x00525147, 0x00545349, 0x0057564C, 0x0059584E, 0x00525349, 0x00525349, 0x00525349, 0x00515248, 0x00515248, 0x00515248, 0x00505147, 0x00505147, 0x004B4C42,
        0x004C4D43, 0x004D4E44, 0x004C4D43, 0x004A4B41, 0x0048493F, 0x004A4B41, 0x004C4D43, 0x004D4E44, 0x004B4C42, 0x004A4B41, 0x004B4C42, 0x004C4D43, 0x004C4D43, 0x004B4C42, 0x004A4B41, 0x00494A40, 0x004A4B41, 0x004B4C42, 0x004A4B41, 0x00494A40,
        0x00494A40, 0x004B4C42, 0x004C4D43, 0x004C4D43, 0x004C4D43, 0x004B4C42, 0x004B4C42, 0x004B4C42, 0x004B4C42, 0x004B4C42, 0x004B4C42, 0x004B4C42, 0x004B4C42, 0x004B4C42, 0x004B4C42, 0x004C4D43, 0x004C4D43, 0x004C4D43, 0x004C4D43, 0x004F5046,
        0x004F5046, 0x004E4F45, 0x004D4E44, 0x004D4E44, 0x004E4F45, 0x004F5046, 0x004F5046, 0x004B4F44, 0x004A4E43, 0x004A4E43, 0x00494D42, 0x00494D42, 0x00494D42, 0x004A4E43, 0x004A4E43, 0x004D4E44, 0x004D4E44, 0x004D4E44, 0x004D4E44, 0x004E4F45,
        0x004F5046, 0x00505147, 0x00515248, 0x00535549, 0x00535549, 0x0054564A, 0x00535549, 0x00525448, 0x004F5145, 0x004D4F43, 0x004B4D41, 0x003F4338, 0x00303429, 0x002D3125, 0x00373B2F, 0x003E4334, 0x0044493A, 0x00494E3F, 0x00494E3F, 0x004B4D41,
        0x004D4F43, 0x004F5145, 0x004F5145, 0x004E5044, 0x004E5044, 0x00505246, 0x00515347, 0x00505344, 0x00545748, 0x00515445, 0x004D5041, 0x004D4F43, 0x00414337, 0x003D3E34, 0x00515248, 0x00525643, 0x00535744, 0x00545845, 0x00535744, 0x00515542,
        0x004F5340, 0x004F5340, 0x00505441, 0x00505441, 0x00515542, 0x00515542, 0x00505441, 0x004E523F, 0x004B4F3C, 0x00484C39, 0x00474B38, 0x00494A40, 0x0048493F, 0x0048493F, 0x0046473D, 0x0045463C, 0x0044453B, 0x0043443A, 0x0043443A, 0x003D4136,
        0x003F4338, 0x003D4136, 0x00393D32, 0x00373B30, 0x00393D32, 0x00393D32, 0x00373B30, 0x0034352B, 0x002E2F25, 0x00313228, 0x002E2F25, 0x003B3C32, 0x0047483E, 0x0033342A, 0x0023241A, 0x00262A1E, 0x00383C31, 0x00343630, 0xFFD9DAD6,
  0xFFFFFFFF, 0xFFFDFCFE, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x005B6150, 0x002A2D1E, 0x002F3223, 0x004E5142, 0x004A4D3E, 0x001A1D0E, 0x00070A00, 0x000F1203, 0x000F1203, 0x00030B01, 0x00091107, 0x0010180E, 0x00141C12, 0x00151D13,
        0x00161E14, 0x00192117, 0x001C241A, 0x00222319, 0x00212218, 0x00222319, 0x0025261C, 0x0028291F, 0x002A2B21, 0x00292A20, 0x0027281E, 0x002B291E, 0x0029271C, 0x002E2C21, 0x002C2A1F, 0x002D2B20, 0x00302E23, 0x002E2C21, 0x00323025, 0x0037362C,
        0x00302F25, 0x0035342A, 0x003D3C32, 0x003D3C32, 0x003E3D33, 0x003F3E34, 0x003B3A30, 0x003A3935, 0x0041403C, 0x0031302C, 0x0024211D, 0x00302D29, 0x00383531, 0x00383330, 0x0046413E, 0x004C4A38, 0x00494735, 0x00494735, 0x004D4B39, 0x00504E3C,
        0x00504E3C, 0x004F4D3B, 0x004F4D3B, 0x00474739, 0x0048483A, 0x004B4B3D, 0x003D3D31, 0x003D3D31, 0x0036352B, 0x001B1A10, 0x00201F15, 0x001D1E0E, 0x00323323, 0x00353626, 0x003C3D2D, 0x00464737, 0x00424333, 0x003F4030, 0x00424333, 0x00414232,
        0x003E3F2F, 0x003D3E2E, 0x003F4030, 0x003D3E2E, 0x00393A2A, 0x00373828, 0x00393A2A, 0x00343525, 0x00373828, 0x003A3B2B, 0x00393A2A, 0x00373828, 0x00343525, 0x00343525, 0x00353626, 0x0035342A, 0x0037362C, 0x0035342A, 0x00302F25, 0x002F2E24,
        0x00333228, 0x00343329, 0x00333228, 0x00313222, 0x00323323, 0x00333424, 0x002E2F1F, 0x00303121, 0x003D3E2E, 0x003F4030, 0x00303121, 0x002F321C, 0x002E311B, 0x002F321C, 0x0031341E, 0x00333620, 0x00343721, 0x0032351F, 0x0030331D, 0x002B2D19,
        0x002D2F1B, 0x002F311D, 0x002E301C, 0x002B2D19, 0x00292B17, 0x00282A16, 0x00282A16, 0x00252616, 0x00252616, 0x00242515, 0x00222313, 0x00252616, 0x00282919, 0x00252616, 0x001F2010, 0x00252713, 0x00262814, 0x00242612, 0x001F210D, 0x001E200C,
        0x0021230F, 0x00232511, 0x0021230F, 0x00202111, 0x00212212, 0x00222313, 0x00222313, 0x00222313, 0x00202111, 0x001F2010, 0x001F2010, 0x001B1D11, 0x001A1C10, 0x001A1C10, 0x001A1C10, 0x00191B0F, 0x00191B0F, 0x00181A0E, 0x00181A0E, 0x0016180C,
        0x0016180C, 0x0016180C, 0x0016180C, 0x0017190D, 0x00181A0E, 0x00181A0E, 0x0016180C, 0x0015160D, 0x0014150C, 0x0012130A, 0x0012130A, 0x0013140B, 0x0012130A, 0x00111209, 0x00101108, 0x0014160A, 0x0015170B, 0x0014160A, 0x00121408, 0x00101206,
        0x000F1105, 0x00101206, 0x00111307, 0x0015170B, 0x0014160A, 0x00121408, 0x00101206, 0x000F1105, 0x00101206, 0x00111307, 0x00121408, 0x00111307, 0x00111307, 0x00111307, 0x00111307, 0x00111307, 0x00121408, 0x00121408, 0x00121408, 0x00111307,
        0x00111307, 0x00101206, 0x00101206, 0x00101206, 0x00111307, 0x00121408, 0x00131509, 0x0012160B, 0x00101409, 0x000E1207, 0x000C1005, 0x000C1005, 0x000C1005, 0x000D1106, 0x000E1207, 0x000D0F03, 0x000D0F03, 0x000D0F03, 0x000D0F03, 0x000E1004,
        0x000F1105, 0x00101206, 0x00111307, 0x000D0F03, 0x000E1004, 0x000F1105, 0x000F1105, 0x000F1105, 0x000D0F03, 0x000B0D01, 0x000A0C00, 0x00050A01, 0x0010150C, 0x001D2318, 0x001D2318, 0x00181E13, 0x001B2215, 0x001F2619, 0x001A2114, 0x001E2217,
        0x00202419, 0x00202419, 0x001F2318, 0x001C2015, 0x001A1E13, 0x001B1F14, 0x001C2015, 0x001E2216, 0x001D2115, 0x001C2014, 0x002A2E23, 0x002E3227, 0x0015190E, 0x00050800, 0x00090C03, 0x00060501, 0x00090804, 0x000B0A06, 0x000B0A06, 0x00090804,
        0x00080703, 0x00080703, 0x00090804, 0x00070602, 0x00070602, 0x00070602, 0x00060501, 0x00040300, 0x00050400, 0x00070602, 0x000A0905, 0x00080A00, 0x00070900, 0x00060800, 0x00050700, 0x00040600, 0x00040600, 0x00040600, 0x00040600, 0x00040800,
        0x00040800, 0x00010500, 0x00000400, 0x00020600, 0x000A0E02, 0x00101408, 0x00111509, 0x00040500, 0x00030400, 0x000C0D03, 0x00050600, 0x0017180E, 0x0034352B, 0x002A2B21, 0x001F2016, 0x0024281D, 0x003C3E38, 0x00343531, 0xFFD6D6D6,
  0xFFFFFFFF, 0xFFFDFFFF, 0xFFFCFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00747767, 0x00393431, 0x00333028, 0x003B392F, 0x00646257, 0x0019190D, 0x0020231A, 0x00181E19, 0x00080F0C, 0x001C2015, 0x00000500, 0x00080C06, 0x00181915, 0x00020100,
        0x0010100A, 0x000A0D04, 0x00050B00, 0x00191611, 0x000D0A05, 0x00020200, 0x000E0E08, 0x00020200, 0x0012120C, 0x00181510, 0x00050200, 0x00060B02, 0x00060900, 0x00040200, 0x00100E06, 0x00040200, 0x00070500, 0x00070100, 0x000B0300, 0x00070800,
        0x00060300, 0x00090000, 0x000B0000, 0x00150909, 0x00090000, 0x00040100, 0x00050600, 0x00070505, 0x00040000, 0x00050303, 0x00050505, 0x00020706, 0x00070F0E, 0x0003110F, 0x00031613, 0x000A110E, 0x000B100E, 0x000C0F0D, 0x000D100E, 0x000C110F,
        0x0008110E, 0x0002110D, 0x0000100C, 0x00020A0A, 0x000E1714, 0x00040B06, 0x000D150B, 0x00060E04, 0x00283127, 0x00323F37, 0x00000D07, 0x00090700, 0x00020100, 0x0007090A, 0x00000003, 0x00000506, 0x00060D08, 0x00000400, 0x000A1201, 0x000D0B0B,
        0x00060403, 0x00040100, 0x000E0B06, 0x000B0702, 0x000C0502, 0x001E1113, 0x0036292B, 0x0017251F, 0x001C1F1D, 0x00080808, 0x00000101, 0x00000304, 0x00020000, 0x00070504, 0x00000200, 0x00000006, 0x0001050A, 0x00000003, 0x00000003, 0x0004080D,
        0x00000006, 0x00020208, 0x00020003, 0x00090000, 0x00040B06, 0x00000600, 0x00070100, 0x001E0A05, 0x00160802, 0x00040300, 0x00070C0B, 0x00001004, 0x00081E12, 0x0009100B, 0x00070000, 0x000A0807, 0x00071710, 0x00000900, 0x00190F08, 0x0003110B,
        0x00060D06, 0x0011110B, 0x00020500, 0x00000400, 0x0009140A, 0x00040200, 0x00130500, 0x00000605, 0x00000100, 0x00110D0C, 0x00070000, 0x00090000, 0x000D0609, 0x00020004, 0x00010007, 0x00020202, 0x00090001, 0x000A0005, 0x000B060F, 0x00020007,
        0x00090006, 0x000E0109, 0x00000003, 0x00000503, 0x00050000, 0x000B0001, 0x00040208, 0x00000006, 0x000A0007, 0x00090000, 0x00000200, 0x00000003, 0x00000004, 0x0000070B, 0x00081114, 0x00040908, 0x00111210, 0x00262828, 0x00161C1B, 0x00010900,
        0x00050C05, 0x00161A15, 0x00222523, 0x00393A36, 0x001E1E18, 0x00020400, 0x000C0C00, 0x000C0D04, 0x0012130A, 0x00191A11, 0x000F1209, 0x001D2017, 0x00282D24, 0x00191E15, 0x0020251C, 0x001B1E1C, 0x00060907, 0x00121513, 0x00090C0A, 0x000A0D0B,
        0x00191C1A, 0x000A0D0B, 0x000C0F0D, 0x00180D10, 0x000C0607, 0x000D0B0B, 0x00161917, 0x00151A18, 0x00080E09, 0x00040B04, 0x000D140D, 0x001B1313, 0x00040000, 0x00121311, 0x000B0D0D, 0x00000100, 0x000C0E0E, 0x000A090B, 0x00020001, 0x000E0C0B,
        0x00020000, 0x000E0C0B, 0x000C0A09, 0x00090706, 0x0012100F, 0x00090706, 0x00060403, 0x000D0B0A, 0x00020000, 0x00020000, 0x000A0807, 0x00100E0D, 0x00090706, 0x00060403, 0x00090706, 0x000F170C, 0x0010180E, 0x00121A10, 0x000A0F06, 0x001B1F19,
        0x0031332D, 0x00262723, 0x002C2B27, 0x0032352C, 0x00282B22, 0x00222A20, 0x00141D13, 0x00041207, 0x0019291E, 0x00314237, 0x00283C30, 0x0007130D, 0x000C1812, 0x0015211B, 0x000C1812, 0x0005110B, 0x0008110E, 0x00030C09, 0x0007100D, 0x000B0D0D,
        0x000A0C0C, 0x00090B0B, 0x00080A0A, 0x00080A0A, 0x00080A0A, 0x00080A0A, 0x00080A0A, 0x000D1211, 0x00050D0C, 0x000A1512, 0x00182821, 0x001C2D24, 0x00112116, 0x000A160A, 0x000C1609, 0x00111516, 0x00121817, 0x0006100A, 0x00000D05, 0x00192820,
        0x00020E08, 0x00131C19, 0x00151A1B, 0x000B1505, 0x0011190F, 0x00191C1A, 0x0009080C, 0x00373436, 0x001C1C16, 0x000D1001, 0x00222811, 0x00140E0F, 0x00191514, 0x00191A16, 0x000F130D, 0x00151914, 0x00121212, 0x00181419, 0x00160E18, 0x0015150F,
        0x00121A10, 0x00161A0F, 0x002A281D, 0x00141C11, 0x0001170B, 0x00212C24, 0x00655654, 0x00252622, 0x00060703, 0x000C0E08, 0x000D1007, 0x00292D22, 0x00565A4E, 0x002A2E22, 0x00191E0F, 0x001C2112, 0x003B3F33, 0x003B3E35, 0xFFDBDDD7,
  0xFFFFFFFF, 0xFFFDFFFF, 0xFFFAFCFC, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00676A5A, 0x0036332E, 0x002E2B23, 0x00454037, 0x00615F54, 0x00212519, 0x005F6D62, 0x007F968E, 0x0084A29D, 0x00B7CCC3, 0x006E837B, 0x009EAFAB, 0x009AA9A5, 0x006F7E7A,
        0x00A3B4B0, 0x00899E96, 0x00798E85, 0x00AECAC4, 0x0053726B, 0x004F7069, 0x007EA099, 0x00577972, 0x00779891, 0x008DACA5, 0x004F6B65, 0x0073988E, 0x005E8177, 0x004C6F65, 0x007A9C92, 0x00405F56, 0x002F4C43, 0x005C776E, 0x0029443B, 0x0018372E,
        0x004A6660, 0x002E4540, 0x005B706D, 0x00546966, 0x001B322D, 0x003E5A54, 0x0037564D, 0x001A3833, 0x00405B57, 0x002B4440, 0x00243B37, 0x003D524F, 0x002D423F, 0x000B201D, 0x00000E0B, 0x00061311, 0x00010C0A, 0x000A100F, 0x00090E0D, 0x00010605,
        0x00000807, 0x00000D0B, 0x00000D0B, 0x000D1312, 0x00000100, 0x000C1310, 0x00030D07, 0x0009150F, 0x000D1D16, 0x00091D18, 0x00213633, 0x0029443B, 0x00405752, 0x003D514C, 0x003D4B49, 0x0045504D, 0x004F5651, 0x006D756B, 0x005C6459, 0x0065766D,
        0x0055685F, 0x0053675B, 0x0061776B, 0x004E665A, 0x00496558, 0x005A786D, 0x0065857A, 0x008AA699, 0x0099ADA1, 0x00687970, 0x0060756D, 0x006F817A, 0x0068776F, 0x00849489, 0x0060766A, 0x00516B5B, 0x00607768, 0x00576E5F, 0x005B7061, 0x0074897A,
        0x006B8071, 0x006F8273, 0x00748778, 0x00667970, 0x006D8F84, 0x00628B7C, 0x00688576, 0x007B8C7E, 0x006C7D70, 0x00617A6C, 0x008AA699, 0x006C7D72, 0x0081948B, 0x0087918B, 0x007B7A76, 0x00818B85, 0x006C8E84, 0x005D7D72, 0x00808B81, 0x00788A83,
        0x00929D95, 0x00838C82, 0x006B7B70, 0x006C887B, 0x00638476, 0x004B645A, 0x0076867F, 0x005B7F69, 0x00466752, 0x00819E87, 0x00526E57, 0x004B6652, 0x00708C79, 0x00395646, 0x004D6D5C, 0x005C7864, 0x00586E5B, 0x00718473, 0x005E7566, 0x0052695B,
        0x0067796C, 0x005A6F60, 0x004E6959, 0x00638B72, 0x005B715E, 0x005C6F60, 0x00618071, 0x005B7A6B, 0x00667769, 0x005E715E, 0x0055775F, 0x00637E6E, 0x00759080, 0x007A9683, 0x007A9482, 0x00708975, 0x006E8773, 0x00849E87, 0x0092AC95, 0x00779988,
        0x007E9F90, 0x008DADA2, 0x0096B3AA, 0x00ACCABF, 0x009AB7A8, 0x00809E8B, 0x00809F8A, 0x006E8B7C, 0x007F9C8D, 0x0093B0A1, 0x0092B1A2, 0x00A1C0B1, 0x00ACCBBC, 0x009EBFB0, 0x00A7C8B9, 0x008DB19B, 0x007CA08A, 0x0084A892, 0x007DA18B, 0x007DA18B,
        0x0089AD97, 0x007FA38D, 0x0081A58F, 0x007FA28E, 0x006B8F79, 0x00779E88, 0x0088B29B, 0x007AA48B, 0x00729C83, 0x007BA68B, 0x007AA58A, 0x007B9783, 0x00668772, 0x00719682, 0x006A917C, 0x00628A78, 0x00739987, 0x00698D7C, 0x005D7E6F, 0x0089AC98,
        0x00567965, 0x007EA18D, 0x0089AC98, 0x00597C68, 0x006E917D, 0x0081A490, 0x00577A66, 0x006F927E, 0x00628571, 0x005D806C, 0x006A8D79, 0x00779A86, 0x00789B87, 0x00729581, 0x0070937F, 0x0092AEA1, 0x007E9A8D, 0x0087A598, 0x008DAB9E, 0x008EAFA0,
        0x0094B5A6, 0x0086AA9A, 0x0082A696, 0x00A3B4A9, 0x009EAFA4, 0x009CB0A4, 0x0091A599, 0x007F9589, 0x0083998D, 0x00859D91, 0x00778F83, 0x006E8580, 0x004C635E, 0x002B413C, 0x000E241F, 0x00041813, 0x00021611, 0x0000110C, 0x00031410, 0x0006110F,
        0x0006110F, 0x0005100E, 0x0005100E, 0x0005100E, 0x0006110F, 0x00071210, 0x00081311, 0x00000F0E, 0x00031816, 0x0019322E, 0x00385750, 0x004E7066, 0x0059796E, 0x005F7D72, 0x00678376, 0x00718C88, 0x0075948D, 0x00799B90, 0x00779B8D, 0x0082A99A,
        0x00759B8F, 0x0084A79D, 0x0088AAA4, 0x00789F97, 0x0081A4A0, 0x0088A7AA, 0x0089A3AA, 0x00A2BCC3, 0x007F9B9C, 0x0083A49D, 0x0094B6AB, 0x00789992, 0x00779A90, 0x00779D91, 0x0072998A, 0x007DA397, 0x007EA399, 0x0085A7A1, 0x007F9E9D, 0x007B9D97,
        0x00769F97, 0x007DA298, 0x008EADA4, 0x007BA59A, 0x006DA598, 0x0079A69D, 0x009BAEAB, 0x00232422, 0x00070806, 0x000B0C08, 0x000C0E08, 0x0025281F, 0x004F5348, 0x002D3126, 0x001D2115, 0x00282D1E, 0x002F3327, 0x00373A31, 0xFFE2E4DE,
  0xFFFFFFFF, 0xFFFDFFFF, 0xFFF9FBFB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00666959, 0x0033332D, 0x00211E16, 0x00413A31, 0x0059554A, 0x00171F14, 0x00678278, 0x007CAEA4, 0x006CABA2, 0x005AA39B, 0x005DA19A, 0x00599791, 0x005F9D97, 0x005C9C96,
        0x005BA19A, 0x00559E96, 0x00519A92, 0x00599791, 0x005C9C96, 0x004D918A, 0x0058A199, 0x005AA39B, 0x00579B94, 0x00579791, 0x006AA8A2, 0x00579D96, 0x005AA39B, 0x00489189, 0x005FA59E, 0x0065A59F, 0x005D9B95, 0x0065A39D, 0x005D9D97, 0x0061A097,
        0x0067A69D, 0x00528E86, 0x0064A098, 0x005C9890, 0x0058948C, 0x004D8C83, 0x00428178, 0x004D8C83, 0x00427E76, 0x003D766E, 0x0051847D, 0x00507974, 0x00345753, 0x001F3E3B, 0x00062320, 0x00031917, 0x00071C1A, 0x0002100F, 0x00071111, 0x00121A1A,
        0x00060E0E, 0x00061010, 0x00081414, 0x000C1211, 0x000B1312, 0x000E1917, 0x00051514, 0x00001210, 0x00082020, 0x001B3737, 0x003B5757, 0x00589287, 0x0087BDB0, 0x0097CABC, 0x0090BEB1, 0x0083B4A6, 0x0062988B, 0x0059978B, 0x0060A297, 0x0075A397,
        0x00669488, 0x00609183, 0x00699E90, 0x005D9889, 0x00509181, 0x00519985, 0x004A9782, 0x00569182, 0x005B9485, 0x005A8F81, 0x00649587, 0x006E9C8F, 0x005F9082, 0x0063988A, 0x00659C8D, 0x0063977F, 0x00659D86, 0x005C9782, 0x005A937E, 0x006B9F87,
        0x00669880, 0x00619780, 0x00609984, 0x00558D82, 0x00569385, 0x004B927E, 0x004F9783, 0x0060A28F, 0x005A9482, 0x00578C78, 0x0087BDA6, 0x00E2EEE8, 0x00EDFDF6, 0x00F3FDF7, 0x00EDF1EB, 0x00E0F5EC, 0x0081AFA3, 0x0072A296, 0x00CFECE3, 0x00ECFDF9,
        0x00F6FFFC, 0x00F0FEF3, 0x00DBF5E7, 0x009CCBBB, 0x0066A193, 0x004F8B81, 0x00649793, 0x0052937A, 0x005A9B82, 0x005E9F86, 0x00579980, 0x00599B82, 0x005B9D84, 0x005D9F86, 0x0056987F, 0x0057A185, 0x004D9479, 0x0060A388, 0x005D9B83, 0x0056947C,
        0x00599A84, 0x0050967F, 0x0059A18A, 0x0058A082, 0x005A9B82, 0x005C9E8B, 0x00519D8B, 0x00519983, 0x006BA38C, 0x00619D85, 0x0047967B, 0x005C9C86, 0x005F9981, 0x005C937A, 0x005E977E, 0x005B9981, 0x0052927C, 0x0054927A, 0x005D977B, 0x00498C77,
        0x004A8C79, 0x004F8D7F, 0x00468478, 0x00549286, 0x00519080, 0x004A8B76, 0x004F907A, 0x004F8D7D, 0x00549282, 0x00569585, 0x00498878, 0x00498A7A, 0x004F9080, 0x00478878, 0x004D9180, 0x0051957E, 0x00478B74, 0x00488C75, 0x00458972, 0x00448871,
        0x004B8F78, 0x00488C75, 0x00498D76, 0x0039876F, 0x0048967E, 0x003E8C74, 0x0036846C, 0x0048947C, 0x0047937B, 0x003C8870, 0x00438F77, 0x00387F6B, 0x003B8271, 0x003F8B79, 0x00358673, 0x00338372, 0x00398978, 0x00388375, 0x00388073, 0x003D8878,
        0x00378272, 0x00428D7D, 0x00418C7C, 0x003F8A7A, 0x00459080, 0x00408B7B, 0x00418C7C, 0x00479282, 0x00418C7C, 0x003E8979, 0x00418C7C, 0x00428D7D, 0x00408B7B, 0x003E8979, 0x003E8979, 0x00528E88, 0x00417D77, 0x0048877E, 0x0048887C, 0x003F8273,
        0x003E8571, 0x003A826C, 0x003D856E, 0x00397E71, 0x003A7C70, 0x003C7E72, 0x003B7B6F, 0x00347266, 0x00306C61, 0x0029655A, 0x001F594E, 0x002C514F, 0x00274C4A, 0x00204241, 0x000B2C28, 0x00001B18, 0x00001A17, 0x00001815, 0x00001915, 0x00011411,
        0x00011411, 0x00001310, 0x0000120F, 0x00001310, 0x00011411, 0x00021512, 0x00031613, 0x00001718, 0x00001F20, 0x000B3230, 0x001B4845, 0x0023564F, 0x00255952, 0x00265B52, 0x00285B53, 0x00256058, 0x00246056, 0x002E6C5E, 0x002E6F5F, 0x00347566,
        0x00337569, 0x0030736A, 0x00357871, 0x001E8385, 0x0045A8AC, 0x0057B5BB, 0x0057B2BB, 0x0060B9C3, 0x0046A4AA, 0x003C9DA0, 0x00066C6B, 0x002A776F, 0x00267469, 0x002D7A6C, 0x00267463, 0x002D7B6A, 0x00307A6E, 0x00337C74, 0x002B736D, 0x002D7372,
        0x00277471, 0x0027736F, 0x00347674, 0x0026746D, 0x0028867B, 0x0034877E, 0x0048827D, 0x00101010, 0x00000000, 0x00090A08, 0x00090A08, 0x000D0E0A, 0x00262822, 0x00181B12, 0x0014170E, 0x0022261A, 0xFF4D5146, 0xFF9D9F99, 0xFFF3F4F0,
  0xFFFFFFFF, 0xFFFDFFFF, 0xFFF9FBFB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x006A6D5D, 0x00353A31, 0x001E1C12, 0x00423930, 0x00655F54, 0x00162117, 0x004C766B, 0x004B9288, 0x001C786D, 0x00177574, 0x002F8381, 0x00529D9B, 0x005FABA7, 0x004B9F9A,
        0x00237F7A, 0x0014726E, 0x001B7575, 0x00317776, 0x004D9896, 0x00388986, 0x001D726E, 0x00196E6A, 0x00227370, 0x00337E7C, 0x004A908F, 0x003D8C89, 0x00176B67, 0x002B827E, 0x001A6E6A, 0x0025706E, 0x00569A99, 0x00438C8A, 0x002B7A77, 0x00206B69,
        0x0027716D, 0x002F7771, 0x004B948C, 0x003B847C, 0x001D655F, 0x0019635F, 0x00135E5C, 0x00145651, 0x00448681, 0x003B7B76, 0x0018534F, 0x0019504D, 0x00184745, 0x000E3635, 0x00214545, 0x00072928, 0x00001918, 0x00092121, 0x00011314, 0x00000D0F,
        0x00071315, 0x00091618, 0x00031012, 0x00060E0D, 0x000A1616, 0x00021316, 0x000F252A, 0x0000171B, 0x00163538, 0x002F4E4F, 0x004B6B6A, 0x006F9288, 0x00A0C2B7, 0x00C0E4D3, 0x00C0EED8, 0x00B6F1DC, 0x0087D6C3, 0x003A9B8D, 0x00006B5F, 0x00236963,
        0x00236865, 0x001F6461, 0x001F6663, 0x00236D69, 0x0016635B, 0x00136559, 0x00116455, 0x0011665E, 0x00116A60, 0x001B6F63, 0x00155A4F, 0x001A5E53, 0x001D6C61, 0x000E645A, 0x0016645D, 0x001C5F56, 0x0013605D, 0x00116767, 0x00106361, 0x00155B54,
        0x0023665D, 0x00307D76, 0x00287E7E, 0x003C8281, 0x00377A75, 0x002A766B, 0x00137166, 0x00086862, 0x00196964, 0x002B6D62, 0x0053927E, 0x00F7FEFB, 0x00F6FFFE, 0x00F9FFF9, 0x00FFFFFB, 0x00F1FFF9, 0x005D887B, 0x00608B82, 0x00DBF5EF, 0x00FAFFFE,
        0x00F8FCF7, 0x00FBFFF9, 0x00ECFFF8, 0x0079AF9E, 0x00226A5E, 0x001B6864, 0x00145E60, 0x0020676A, 0x00196365, 0x001C696C, 0x001C6D70, 0x00207071, 0x0015625F, 0x0019635D, 0x001F655E, 0x000A6760, 0x00177770, 0x00146F66, 0x0019685F, 0x0017655E,
        0x000E6862, 0x0013726E, 0x000C6666, 0x00146358, 0x00186764, 0x00146F74, 0x000C7173, 0x000B6461, 0x00195F59, 0x00186561, 0x00107671, 0x00146B68, 0x0017645D, 0x0023685E, 0x001E6862, 0x000A6562, 0x00066666, 0x000D6663, 0x0014635A, 0x0017665B,
        0x0017665D, 0x001C6C67, 0x00115F5E, 0x00186665, 0x00166562, 0x00176760, 0x00196C63, 0x001B6865, 0x00196865, 0x001A6966, 0x0012615E, 0x00176865, 0x001D6E6B, 0x00126662, 0x00146864, 0x001B6A6D, 0x0019686B, 0x00176669, 0x00176669, 0x00166568,
        0x00166568, 0x0019686B, 0x0018676A, 0x00116B6C, 0x00136A6C, 0x0014686D, 0x0015666B, 0x0017656B, 0x0018696E, 0x0016676C, 0x00106168, 0x0028828D, 0x00298592, 0x00268693, 0x00238793, 0x00218492, 0x001F8290, 0x00248393, 0x00298596, 0x00218290,
        0x00258694, 0x00298A98, 0x00258694, 0x002C8D9B, 0x002D8E9C, 0x00258694, 0x002E8F9D, 0x002D8E9C, 0x002E8F9D, 0x002E8F9D, 0x002B8C9A, 0x00288997, 0x00288997, 0x002A8B99, 0x002B8C9A, 0x00308C99, 0x002F8994, 0x00217A84, 0x00086165, 0x000C6262,
        0x00166B67, 0x0011665E, 0x00176A61, 0x00096560, 0x0008625D, 0x00065D59, 0x00075B57, 0x00095855, 0x0008514F, 0x00034948, 0x00004443, 0x0007333A, 0x00083539, 0x000F373C, 0x000C3135, 0x0004272A, 0x00042326, 0x00011D1E, 0x00001415, 0x00021415,
        0x00011314, 0x00001112, 0x00001011, 0x00001011, 0x00001213, 0x00011314, 0x00021415, 0x00001A21, 0x00002228, 0x00032E31, 0x0005383A, 0x00074041, 0x00094545, 0x00094747, 0x000A4947, 0x00054F51, 0x00014B4B, 0x000D5654, 0x000D5955, 0x000D5A57,
        0x000F5F60, 0x0007595E, 0x001B6E76, 0x0046A2B5, 0x004097AB, 0x002D7E93, 0x00267588, 0x00328196, 0x004396AB, 0x004BA4B9, 0x00217E93, 0x00045F66, 0x00005B5F, 0x00106868, 0x000C6360, 0x000F6361, 0x000F6263, 0x0013686A, 0x000E6267, 0x000D5E65,
        0x0009636A, 0x000D626A, 0x0014646B, 0x00096266, 0x000A7370, 0x00197972, 0x00408882, 0x00121113, 0x00000001, 0x00080808, 0x000C0D0B, 0x00020300, 0x00040600, 0x00030500, 0x00080B02, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFDFFFF, 0xFFFAFCFC, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x006A6D5D, 0x00323A30, 0x001B1A10, 0x00393026, 0x00696259, 0x000F1D12, 0x00315E55, 0x003D8E85, 0x00157C74, 0x00296974, 0x006D9EA8, 0x00D2F6FE, 0x00B9DEE2, 0x00BFF2F4,
        0x0094D4D8, 0x002E6D75, 0x0037717D, 0x0094C8CE, 0x00A6E0E5, 0x009BDEE1, 0x0058A2A4, 0x00196365, 0x002E7174, 0x0085BFC4, 0x00B6EAF0, 0x00A2DEE4, 0x0050979B, 0x0018676A, 0x00388185, 0x008CC6CC, 0x00ADE1E8, 0x00A2DCE2, 0x0052969B, 0x00286D76,
        0x0036787D, 0x0083BEC0, 0x00A3DCDB, 0x00AEE7E6, 0x00609B9D, 0x001F6166, 0x001F646D, 0x0090BCBD, 0x00A5D5D6, 0x009ACECE, 0x005A9392, 0x000D4645, 0x001F5353, 0x006F9D9E, 0x0078A4A5, 0x0034605F, 0x000E3635, 0x00001B1B, 0x00001617, 0x0021383A,
        0x00203739, 0x000F2628, 0x00092224, 0x00001112, 0x00091E20, 0x00051F25, 0x00000E15, 0x00204046, 0x00557477, 0x003B5855, 0x00405A54, 0x008A7E74, 0x00D5CABC, 0x00F1E7D6, 0x00F4F3DF, 0x00D4E2D0, 0x00C4E5D7, 0x00BAEDE5, 0x0063A39E, 0x001B6B72,
        0x00186772, 0x001E6977, 0x00155D6E, 0x00175A69, 0x00205D67, 0x004C8388, 0x005D8F8F, 0x0019616B, 0x00106469, 0x001D6A6C, 0x00598F90, 0x0055888A, 0x00145E60, 0x000D5E63, 0x002E707B, 0x0059888C, 0x00387A85, 0x00116374, 0x00135E6C, 0x003F777C,
        0x0074A5A7, 0x0080C1C9, 0x0065BACA, 0x006EB0BB, 0x00A2D2D8, 0x009BD3D4, 0x004FA8AC, 0x00096F7C, 0x000F5C6F, 0x00235E67, 0x0047817C, 0x00F3F7F8, 0x00FAFFFF, 0x00F6F8F2, 0x00FFFEF7, 0x00EFF7ED, 0x007C9A8F, 0x008EA7A3, 0x00D2D7D8, 0x00FFFAFB,
        0x00FFFEFB, 0x00FDF8EF, 0x00EFFFF3, 0x0080AEA1, 0x001B5F58, 0x00156265, 0x001A6974, 0x00246378, 0x000C5067, 0x0019657C, 0x000B5970, 0x000D5A6D, 0x001C626F, 0x003B767F, 0x0063989B, 0x00276D7A, 0x000B5F65, 0x00136063, 0x0054888E, 0x005B8F96,
        0x001D6870, 0x00085A65, 0x001D6273, 0x00619193, 0x004F8693, 0x00145F75, 0x00005365, 0x0031747D, 0x00608C93, 0x004A818A, 0x00075D69, 0x000E5F6E, 0x0043828A, 0x005E9193, 0x00316F75, 0x00055C6C, 0x0006677B, 0x00116675, 0x0023676E, 0x001D676D,
        0x000E5961, 0x00165F6D, 0x00125C6E, 0x001A6476, 0x00156173, 0x00146370, 0x0013636E, 0x00155F71, 0x00176173, 0x00196375, 0x00156173, 0x001B6779, 0x001C6A7B, 0x00136172, 0x00115F70, 0x00105C73, 0x00156178, 0x00125E75, 0x00146077, 0x00135F76,
        0x00105C73, 0x00125E75, 0x00125E75, 0x0015637A, 0x000B556D, 0x00145A72, 0x00256682, 0x001C5C7A, 0x000A4C69, 0x00185E7C, 0x003B84A2, 0x0059B9DD, 0x0045A6CC, 0x00369BC1, 0x003AA4C9, 0x003AA3CA, 0x00389FC6, 0x003DA1CB, 0x00399BC5, 0x003EA4CE,
        0x00379DC7, 0x0042A8D2, 0x0042A8D2, 0x00389EC8, 0x003BA1CB, 0x003EA4CE, 0x00399FC9, 0x003DA3CD, 0x003BA1CB, 0x00389EC8, 0x00389EC8, 0x003CA2CC, 0x0040A6D0, 0x003FA5CF, 0x003BA1CB, 0x0033A2C8, 0x0046B1D6, 0x003FA6C7, 0x00187896, 0x00035D76,
        0x0003566C, 0x00065366, 0x0010596D, 0x00155E6C, 0x00125967, 0x000D505F, 0x000F4B5B, 0x0011495A, 0x00104254, 0x000A3A4C, 0x000A384A, 0x000C4051, 0x00003244, 0x00002D3E, 0x0007303F, 0x00042937, 0x0001212C, 0x00021E29, 0x00001923, 0x0004161D,
        0x0003151C, 0x0002141B, 0x0001131A, 0x0002141B, 0x0004161D, 0x0006181F, 0x00081A21, 0x0005232E, 0x00092C36, 0x000A3440, 0x00083B45, 0x00074049, 0x000A454E, 0x000D4851, 0x000D4852, 0x0001465A, 0x00074B5E, 0x00125466, 0x00135567, 0x000B5063,
        0x00075268, 0x000E5D78, 0x00388BAA, 0x004484A3, 0x002C6683, 0x003B6E88, 0x0057889E, 0x00487A91, 0x00316785, 0x00347395, 0x005EA2C7, 0x00156884, 0x0003526D, 0x00115B73, 0x0012576B, 0x00115568, 0x00115469, 0x00145A71, 0x00105870, 0x00115669,
        0x000F5C6F, 0x000E576D, 0x0014576C, 0x000E5B6B, 0x0007656A, 0x001B726F, 0x0065A5A0, 0x00302F31, 0x00050505, 0x00070707, 0x00171816, 0x00131410, 0x000C0E08, 0x000E1108, 0x0014170E, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFDFFFF, 0xFFFCFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00737666, 0x0031392E, 0x0015170B, 0x00282217, 0x005D584F, 0x0009160E, 0x00254F48, 0x0037817B, 0x001A7873, 0x00175363, 0x007FA9B6, 0x00A7BDC8, 0x00051F25, 0x00618E91,
        0x0092D1D5, 0x00114E58, 0x005A8E9F, 0x009DC1C9, 0x00002B32, 0x004C8388, 0x0087C6CA, 0x001F5E62, 0x00548B90, 0x00A2CED5, 0x00001D25, 0x004F7985, 0x009CD5DE, 0x001F636A, 0x004C8790, 0x0096BECA, 0x00022431, 0x00426C78, 0x009DD6DF, 0x00115667,
        0x0046828E, 0x00ACDAE2, 0x0004292D, 0x00375C60, 0x00B0DEE6, 0x004C8894, 0x00296E7F, 0x00AFC4CC, 0x0006232A, 0x00396166, 0x009ACACE, 0x00104548, 0x002A5F62, 0x009FCFD3, 0x0000171B, 0x0006393B, 0x00649597, 0x00002225, 0x0012383C, 0x006D9296,
        0x00001216, 0x00001E22, 0x006A9396, 0x00123336, 0x00012428, 0x0001262E, 0x00355963, 0x0095B5BB, 0x005D7573, 0x005D6D62, 0x00A9B4A4, 0x00D1CEB2, 0x00F1EDD1, 0x00FFF5DD, 0x00FFFEE8, 0x00FEF6E5, 0x00EAE8DE, 0x00DEE4DF, 0x00E2EDEB, 0x003A8391,
        0x00085264, 0x000F5971, 0x00145974, 0x00205972, 0x00598194, 0x00CCE2EE, 0x00E8F1FA, 0x005F889F, 0x001F5A69, 0x005D909A, 0x00E9FAFF, 0x00D8E7F0, 0x0046747F, 0x001D5463, 0x008CAEC6, 0x00F4FFFF, 0x00A3CAD2, 0x00215D6D, 0x00316571, 0x00B8D2D2,
        0x00F1FFFE, 0x0093BABC, 0x0010515F, 0x00427A85, 0x00C8DFE7, 0x00DAF2F8, 0x0078BECB, 0x001F7B98, 0x00145A7F, 0x0024566D, 0x004D8187, 0x00F4F2F8, 0x00FBFFFF, 0x00FFFFFB, 0x00FFFFF8, 0x00F4F8ED, 0x00A6BFB5, 0x00B5CBC9, 0x00D4D7DC, 0x00FFFEFF,
        0x00FFFEFC, 0x00F8EDE5, 0x00EDF6E9, 0x0084AA9E, 0x001E5A59, 0x00165C69, 0x00196074, 0x00215B6E, 0x00286A7D, 0x00145D73, 0x00145F75, 0x00195F70, 0x00386F78, 0x00B7DFDE, 0x00E0FBF7, 0x0095B8C5, 0x00245D5F, 0x00518280, 0x00DBE9E8, 0x00E9F6F8,
        0x00659597, 0x0021585F, 0x007799A9, 0x00FAFCF6, 0x00D6E9F0, 0x00518193, 0x00225C68, 0x0098B7B8, 0x00F9FEFC, 0x00C4D9DB, 0x002C656E, 0x002C6672, 0x00BADBDE, 0x00F1FFFE, 0x008EADAE, 0x00367785, 0x001C6E80, 0x00296876, 0x00648D90, 0x003D7E8C,
        0x0019596B, 0x00115169, 0x000E4F6B, 0x00155774, 0x00155976, 0x00175E79, 0x00145C74, 0x00185A77, 0x001A5C79, 0x00195B78, 0x00125673, 0x00105471, 0x000F5572, 0x00115774, 0x00155B78, 0x000F586E, 0x00145D73, 0x00145D73, 0x00155E74, 0x00165F75,
        0x00125B71, 0x00115A70, 0x00125B71, 0x0010576B, 0x001C5F74, 0x001A566E, 0x0018506B, 0x001B5572, 0x00145472, 0x00337B99, 0x0072BFDF, 0x0054B6DE, 0x00389FC6, 0x002E98C1, 0x00339FC8, 0x002C9BC3, 0x002C98C2, 0x00349DC8, 0x003199C4, 0x002B96C2,
        0x00329DC9, 0x00319CC8, 0x002E99C5, 0x00329DC9, 0x00309BC7, 0x002B96C2, 0x00339ECA, 0x00339ECA, 0x00339ECA, 0x00319CC8, 0x002F9AC6, 0x002E99C5, 0x002E99C5, 0x002D98C4, 0x002C97C3, 0x00239CCE, 0x0034A9DC, 0x0047B4E7, 0x003AA0D0, 0x001A76A5,
        0x00106592, 0x00196893, 0x001E6A94, 0x00196D90, 0x00196B8E, 0x0019678B, 0x001C668A, 0x001D6388, 0x001A5B81, 0x00125177, 0x000D4B71, 0x00003E5B, 0x0000415E, 0x0003425E, 0x0009415A, 0x000A3B51, 0x00113C51, 0x0012374B, 0x00002234, 0x0007232E,
        0x0006222D, 0x0006222D, 0x0006222D, 0x0008242F, 0x000C2833, 0x000F2B36, 0x00122E39, 0x000E3242, 0x00103948, 0x00134152, 0x00134757, 0x00124A5B, 0x00104B5B, 0x0011495C, 0x0011495C, 0x000A4A68, 0x00114D6A, 0x00144F69, 0x0018506D, 0x00134F6D,
        0x000A4C6F, 0x00216C93, 0x004193BD, 0x002B5F7D, 0x005C8CA4, 0x00AED9EA, 0x00DAFFFF, 0x00B9E0EE, 0x0078A7BD, 0x002D6384, 0x004582AA, 0x003A84AE, 0x0010557D, 0x00155175, 0x00174C6D, 0x001A4D6D, 0x00194F6E, 0x001A5373, 0x00135072, 0x001D5570,
        0x00155672, 0x000E4969, 0x00154B6A, 0x0014546C, 0x00095C64, 0x001E6A69, 0x0079AFAA, 0x00393A38, 0x00020301, 0x00080905, 0x001F211B, 0x0020231A, 0x001A1E13, 0x0023271C, 0x002F3327, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFDFFFF, 0xFFFDFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00797C6C, 0x00343E31, 0x0017190D, 0x0029251A, 0x005C5A50, 0x001C271F, 0x00436460, 0x00417B7A, 0x00216C6E, 0x001A6878, 0x006499A6, 0x00B9DBE1, 0x001C4242, 0x00579490,
        0x0087D7D6, 0x000B5A63, 0x00428798, 0x00ADCED7, 0x001B474E, 0x005F979C, 0x007EBEC2, 0x000F4F53, 0x00467E83, 0x00B1DDE4, 0x002C4D56, 0x005C8692, 0x0081BCC5, 0x00165E65, 0x0047868E, 0x00AED6E2, 0x00335360, 0x005E8B96, 0x00ACEAF2, 0x000D6377,
        0x00377C8D, 0x00A4D5DF, 0x003F6167, 0x00517379, 0x009FD0DA, 0x00559AAB, 0x00449AAE, 0x00BDD1DC, 0x003F5C65, 0x005E858D, 0x0088BAC0, 0x000E454A, 0x0032666C, 0x00A4D2D9, 0x00305A61, 0x00296264, 0x007CB3B6, 0x00003134, 0x00205054, 0x0094C4C8,
        0x0024575A, 0x002E6568, 0x0080B9BB, 0x00002F33, 0x0014464C, 0x0023515C, 0x0081AAB3, 0x0084A3A6, 0x0042534A, 0x008C8E7A, 0x00E3DCC1, 0x00B6C193, 0x00BEC69D, 0x00EAEFCF, 0x00FFFFEC, 0x00FFFFF2, 0x00DDDCCE, 0x00A3A397, 0x00E2E1D7, 0x0096D1DB,
        0x00256976, 0x000E586A, 0x000E576D, 0x00155064, 0x00567786, 0x00E2E8EF, 0x00FFF6F8, 0x0082A0B3, 0x00174E57, 0x00608B8E, 0x00EEF2F3, 0x00E8EAEB, 0x005F888B, 0x001D505A, 0x00A7BFD3, 0x00FFFFF6, 0x00C4E2E3, 0x002E6671, 0x002F5E62, 0x00BBCAC2,
        0x00FBFFF1, 0x009BBBB6, 0x0018575F, 0x00467B78, 0x00EAF0EB, 0x00F5F9F4, 0x0089C3C9, 0x00237C97, 0x0011547B, 0x001E4F69, 0x0040797B, 0x00FBF3FD, 0x00FEFDFF, 0x00FFFEF9, 0x00FFFEF3, 0x00E3F0E2, 0x0082A89C, 0x006A928D, 0x00C0D4D9, 0x00FDFFFF,
        0x00FDF8F5, 0x00F8F3EA, 0x00F4FEF1, 0x007FA298, 0x001F585A, 0x001B6071, 0x000C506D, 0x00195261, 0x001E5F6E, 0x00155D6F, 0x00115B6D, 0x00135260, 0x003D676C, 0x00CCE2DD, 0x00FCFFF7, 0x009AB6BD, 0x001F5653, 0x005B887F, 0x00E7ECE3, 0x00EEF2EC,
        0x0069958E, 0x00225859, 0x0087A1AD, 0x00FFFFF0, 0x00E1EEEC, 0x0045747C, 0x00235A5D, 0x00A7C1B4, 0x00F3EFDD, 0x00C2D0C5, 0x00346A6A, 0x0035666E, 0x00C4D9D6, 0x00FFFFF7, 0x00B1C5C0, 0x007AB7C1, 0x0067B5C6, 0x00639EA7, 0x00A5C4C1, 0x0043808A,
        0x001C5965, 0x00185668, 0x0015536B, 0x0014566F, 0x00145770, 0x00165C74, 0x000C556B, 0x00155672, 0x00175874, 0x00195A76, 0x00175A75, 0x00145772, 0x00115671, 0x00135873, 0x00155A75, 0x0011586C, 0x00155C70, 0x00165D71, 0x00145B6F, 0x00175E72,
        0x00155C70, 0x0011586C, 0x00155C70, 0x000F5A6A, 0x000F5467, 0x0018596E, 0x00135169, 0x000D506B, 0x0036839E, 0x005EB7D2, 0x005CBDD7, 0x0042ABD4, 0x0036A0C9, 0x0033A4CC, 0x0031A3CC, 0x00279BC4, 0x002C9DC8, 0x0035A2CE, 0x0037A2CE, 0x0030A0CE,
        0x002D9DCB, 0x0030A0CE, 0x0032A2D0, 0x002C9CCA, 0x002C9CCA, 0x0030A0CE, 0x002B9BC9, 0x002D9DCB, 0x0030A0CE, 0x0033A3D1, 0x0033A3D1, 0x0030A0CE, 0x002D9DCB, 0x0030A0CE, 0x0035A5D3, 0x002DA6D8, 0x002AA0D3, 0x0034A9DC, 0x0045B4E6, 0x0045B0E3,
        0x0046ACE0, 0x0045ABDC, 0x003DA0D2, 0x0033A9D8, 0x0035A9D8, 0x0038AAD9, 0x0039A8D8, 0x0035A3D3, 0x00349DCE, 0x002E95C6, 0x00258CBD, 0x002483AA, 0x002782A7, 0x0022789C, 0x001D6C8D, 0x00145C7A, 0x00195A76, 0x00215C76, 0x000D455E, 0x000A3849,
        0x00093748, 0x00083647, 0x00073546, 0x00093748, 0x000C3A4B, 0x000F3D4E, 0x00113F50, 0x000A3E4F, 0x000A4253, 0x000E495C, 0x00125165, 0x00135469, 0x0015536B, 0x0017536B, 0x0019516C, 0x0012566D, 0x00115169, 0x00144E64, 0x00185069, 0x0015516E,
        0x00074C6D, 0x002E7EA3, 0x002B82AA, 0x004E6D86, 0x00ADC7D8, 0x00E6F9FF, 0x00EDFDFC, 0x00E9FCFF, 0x00D1ECFA, 0x00567B97, 0x003A6489, 0x003F91BA, 0x00085379, 0x000C4C6E, 0x00144E6B, 0x001B526D, 0x001B536E, 0x00195874, 0x00155873, 0x00175371,
        0x00155675, 0x00135175, 0x001B5174, 0x00145770, 0x00075E68, 0x00126360, 0x00619D93, 0x0032342E, 0x00000200, 0x0010130A, 0x00282C21, 0x002B2F23, 0x00282D1E, 0x00313627, 0x003B4130, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFDFFFF, 0xFFFBFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00707363, 0x00303A2D, 0x0014180C, 0x002C2A1F, 0x00555349, 0x00212823, 0x0056716E, 0x00477779, 0x002B686C, 0x00165C6D, 0x005A8795, 0x00C8DDE5, 0x00CBE4E6, 0x00B8EDEA,
        0x004E9898, 0x001A616B, 0x00134E61, 0x008BB5BA, 0x00AADDE0, 0x007EBFC0, 0x001D6767, 0x00105A5A, 0x00256667, 0x0075A8AB, 0x00B5DFE4, 0x0088B9C1, 0x0024686D, 0x000C5F61, 0x000C5559, 0x007FB0B8, 0x00B1D8E1, 0x008BC1C8, 0x004A9397, 0x0045ACBF,
        0x000E6071, 0x0068A0AB, 0x00BBE0E8, 0x00B6DBE3, 0x0084BCC7, 0x00398B9C, 0x003097AA, 0x0099B6BF, 0x00BFE3EB, 0x009DCDD3, 0x00387075, 0x000B454A, 0x001D5157, 0x005A868D, 0x00C2E7EF, 0x0091CCCE, 0x002A6365, 0x00195053, 0x00144B4E, 0x00467F81,
        0x0092CFD1, 0x008FD0D1, 0x00296F6F, 0x000E484D, 0x00144D56, 0x00376B77, 0x00ADD8E1, 0x00547273, 0x007C867A, 0x00CFC8AF, 0x00FFECC9, 0x00D1BC89, 0x00B6A376, 0x00C3B291, 0x00DED2BA, 0x00EBE3D2, 0x00F7EFE2, 0x00C6C1B2, 0x009E9A88, 0x00BFEFF3,
        0x003B777D, 0x00115C65, 0x000B5A67, 0x00145661, 0x004A7074, 0x00D9E0DD, 0x00FFFAF3, 0x00759EA7, 0x00135855, 0x004B857A, 0x00F2FFF5, 0x00E6F1E7, 0x003F776C, 0x00125351, 0x009DC0CA, 0x00FEFFF5, 0x00BDE2E6, 0x00216270, 0x00255C63, 0x00B2C7BF,
        0x00FAFFF7, 0x009CC4BF, 0x00024D56, 0x004F8774, 0x00F2F2E0, 0x00FFFCED, 0x009BD0CD, 0x00278095, 0x000E5577, 0x0021566B, 0x00337570, 0x00F8ECF8, 0x00FFFCFF, 0x00F0F0EA, 0x00E5E7DB, 0x00D7F1E1, 0x0062998A, 0x001F5954, 0x00B7E0E3, 0x00F3FFFF,
        0x00F9FFFA, 0x00F8FAEE, 0x00EEFDEF, 0x0079A196, 0x001A5557, 0x00185B70, 0x00105576, 0x001F596C, 0x0018596E, 0x000F576F, 0x00175F77, 0x001A5568, 0x003F626C, 0x00D4DFDD, 0x00FFFDF4, 0x0080ABB4, 0x00105754, 0x00427F75, 0x00DAEBE2, 0x00DEEFE6,
        0x005A968E, 0x003D8383, 0x00B1DBE8, 0x00F2F1DC, 0x00D0EBE7, 0x002F6E76, 0x000E5858, 0x009CC3B4, 0x00E2EBD6, 0x00B3CFC2, 0x00226966, 0x002E6973, 0x00B4CFCC, 0x00FBFFF6, 0x00A4BFBB, 0x00428895, 0x000B667B, 0x00165C69, 0x007DA3A3, 0x00317171,
        0x00145458, 0x001A5B64, 0x00155867, 0x00115667, 0x0010586A, 0x00186373, 0x00105B69, 0x00175A6F, 0x0014576C, 0x0011566A, 0x0013586C, 0x0012596D, 0x0011586C, 0x0011586C, 0x000B5468, 0x000E536E, 0x000F546F, 0x00125772, 0x000D526D, 0x00115671,
        0x00125772, 0x000B506B, 0x00135873, 0x0003566B, 0x000C5B70, 0x000B526D, 0x000F5572, 0x003380A0, 0x005CB7D6, 0x0057C1E0, 0x0035A9C8, 0x0040ACDC, 0x003AA8D8, 0x0039ABDA, 0x002DA3D4, 0x002CA2D3, 0x0036A9DC, 0x0034A4D8, 0x0032A0D4, 0x002CA0D5,
        0x0033A7DC, 0x002FA3D8, 0x002EA2D7, 0x0035A9DE, 0x0032A6DB, 0x0031A5DA, 0x0035A9DE, 0x0032A6DB, 0x002DA1D6, 0x002CA0D5, 0x0032A6DB, 0x0035A9DE, 0x0034A8DD, 0x0031A5DA, 0x0031A5DA, 0x0033AAD7, 0x003AB1DE, 0x0034ACDA, 0x0031A9D7, 0x0032ACDA,
        0x002DA6D7, 0x002AA6D6, 0x0031ADDD, 0x0031A7E2, 0x0030A6E1, 0x0032A8E3, 0x0034A8E3, 0x0032A4E0, 0x0036A8E4, 0x0038A8E4, 0x0030A0DC, 0x00299DC6, 0x002090BA, 0x00218CB2, 0x002485A9, 0x00005A7C, 0x0000405F, 0x00186481, 0x004089A5, 0x000B4A5E,
        0x0009485C, 0x00064559, 0x00044357, 0x00044357, 0x00054458, 0x0007465A, 0x0009485C, 0x000B4D5F, 0x00084D5E, 0x00085062, 0x000B5468, 0x000B566C, 0x000D536B, 0x000F526D, 0x00155371, 0x000E5765, 0x00105663, 0x00195866, 0x001D5969, 0x00115366,
        0x00004C63, 0x003990AB, 0x001F7C9D, 0x0037737F, 0x00ACE3E6, 0x00D6FFFF, 0x00D5FFF2, 0x00D7FFFA, 0x00C0F9F8, 0x00448594, 0x002B758F, 0x00399BB9, 0x00005974, 0x0007556C, 0x0013586C, 0x00145668, 0x000D5263, 0x00095464, 0x00065667, 0x000D516E,
        0x00095474, 0x000D5378, 0x000F4F72, 0x0004516B, 0x0003656D, 0x0017716B, 0x005FA497, 0x00383B32, 0x00000400, 0x001A1E13, 0x0034382C, 0x003F4435, 0x00414736, 0x003D4330, 0x00393F2C, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFCFCFC, 0xFFFEFDFF, 0xFFFDFCFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00606655, 0x00384137, 0x0014180D, 0x00282419, 0x005B564D, 0x001A2218, 0x0033544D, 0x003F7A72, 0x001E6B64, 0x00205C72, 0x001D596F, 0x00246076, 0x0028647A, 0x00215D73,
        0x001C586E, 0x001D596F, 0x001B576D, 0x0016535D, 0x001C5761, 0x001D5461, 0x001E5360, 0x00255A67, 0x00235A69, 0x001F5867, 0x00205B6A, 0x00155161, 0x001A5666, 0x001D5969, 0x001C5868, 0x001B5767, 0x001C5868, 0x001B5767, 0x00185464, 0x003283A6,
        0x0056A7CC, 0x00045578, 0x00145E76, 0x000D525C, 0x007BC4CC, 0x004198A8, 0x00005770, 0x004A97B7, 0x00298199, 0x00005465, 0x00014758, 0x00235365, 0x001C4D63, 0x00004F64, 0x002B98AD, 0x00075972, 0x00085062, 0x0019545E, 0x002C5B5F, 0x00204A4F,
        0x0015424D, 0x001C4A5B, 0x00124458, 0x00184D61, 0x001D5064, 0x006291A6, 0x00B1D5E5, 0x00596F74, 0x009CA598, 0x00FCF7D8, 0x00E2D4AA, 0x00B09E6F, 0x00E9DAA2, 0x00ECE1A3, 0x00BCB27C, 0x00B1A680, 0x00FFF7DC, 0x00D3CEB5, 0x00979478, 0x00BFE4DC,
        0x005E9FA0, 0x0002505D, 0x00295C6C, 0x004B6167, 0x00858F89, 0x00D3D4CA, 0x00E7DED5, 0x00979E89, 0x00616D61, 0x00899B94, 0x00D7E7DC, 0x00CDD6C2, 0x00868B76, 0x005F6A5A, 0x00ABBCB3, 0x00DCE8D6, 0x00BCC8B6, 0x00616D5B, 0x0065715F, 0x00B3BFAD,
        0x00D4E0CE, 0x00A0AC9A, 0x0064705E, 0x007E887C, 0x00CEDBCD, 0x00C1D0BB, 0x0097B09C, 0x00417477, 0x00095671, 0x00024B5F, 0x00447B78, 0x00DFEFE8, 0x00FFFCFF, 0x00F0F1ED, 0x00C6CDB8, 0x00E0E7D8, 0x00567F81, 0x0017535F, 0x00AAC8D3, 0x00F1FDFF,
        0x00FBFEFF, 0x00F5F3F2, 0x00F6FFFC, 0x00789D99, 0x001B575D, 0x001A6175, 0x0009516F, 0x00055D6E, 0x00105467, 0x002D4C6D, 0x00165373, 0x001F5762, 0x00747865, 0x00BEBEA0, 0x00EBE2CE, 0x00929E92, 0x004C6855, 0x00698169, 0x00D8D8C0, 0x00DDDDC5,
        0x00869E86, 0x00839F8C, 0x00939F93, 0x00E0DBCC, 0x00BBCFBC, 0x005C836D, 0x004A6652, 0x00AAAB9B, 0x00F2E9DB, 0x00B7C1B0, 0x006D917B, 0x00557654, 0x00B2C9B3, 0x00E0DCCA, 0x00A59A7E, 0x003F5F40, 0x00398776, 0x0095D1CB, 0x008A918C, 0x0025606F,
        0x00215C6B, 0x001C5766, 0x001B5665, 0x001B5866, 0x001E5B69, 0x001E5D6B, 0x00205F6D, 0x00165B6C, 0x00155A6B, 0x0014596A, 0x0014596A, 0x00155A6B, 0x00155A6B, 0x00135869, 0x00115667, 0x00125768, 0x00135869, 0x00155A6B, 0x00175C6D, 0x00165B6C,
        0x00155A6B, 0x00125768, 0x00105566, 0x001C5C6E, 0x00105467, 0x00064E66, 0x003685A0, 0x0060B5D5, 0x0054AFD4, 0x004CACD4, 0x0048A9D5, 0x0054ABD7, 0x004BB2DF, 0x00289ECD, 0x00229ACE, 0x0038A9E2, 0x00329ED8, 0x002599D2, 0x0023A1D8, 0x0027A0D1,
        0x002DA3D6, 0x0038ACE1, 0x002B9CD4, 0x002FA1D6, 0x002EA3D6, 0x002CA4D2, 0x002AA6CF, 0x002B9DCC, 0x003FAAE4, 0x002A92D5, 0x0034A2E2, 0x002EA7D9, 0x0031AAD6, 0x0036A1D4, 0x0042A3DD, 0x002DB0DC, 0x0032AFDC, 0x003FB4E1, 0x002796C4, 0x001E89B5,
        0x00218EBA, 0x001F8FBA, 0x001B8EB9, 0x002695BB, 0x002390B6, 0x002996BC, 0x00218CB2, 0x00278EB5, 0x00268BB2, 0x003496BE, 0x002C8EB6, 0x002B93C2, 0x00238CB8, 0x00117AA3, 0x00268AAD, 0x003C99B8, 0x001F748E, 0x00004259, 0x00246A81, 0x003A9CB4,
        0x00085570, 0x000C4260, 0x00184766, 0x000A4761, 0x000C5269, 0x000B5064, 0x000C485E, 0x00134F65, 0x00115169, 0x00084F6A, 0x000E5B76, 0x00004A67, 0x000A5875, 0x000F607B, 0x00085C76, 0x00005F69, 0x00035B69, 0x000D536A, 0x0016506D, 0x00215C7C,
        0x00034A6B, 0x002A809E, 0x002B8EAA, 0x003B6C76, 0x0087B4B8, 0x00DFFFFF, 0x00E3FFFF, 0x00E2FFFF, 0x00A8CDD5, 0x00426D80, 0x00477691, 0x004394AF, 0x000D5A74, 0x00075066, 0x000F5468, 0x000F5065, 0x0014546C, 0x00104F6B, 0x0011516F, 0x000C4E67,
        0x00085167, 0x0013576E, 0x00124E6B, 0x000A506E, 0x00005569, 0x001C706E, 0x0074B5A5, 0x003B3B3B, 0x00050602, 0x001D2116, 0x00434636, 0x00434734, 0x0050523F, 0x004E4F3F, 0x00424333, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFEFF, 0xFFFDFCFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00696F5E, 0x00424B41, 0x00161A0F, 0x00252116, 0x00555047, 0x00161E14, 0x0032534C, 0x003B766E, 0x0018655E, 0x00266076, 0x001F596F, 0x001E586E, 0x001F596F, 0x001C566C,
        0x001C566C, 0x001E586E, 0x001B556B, 0x00205B6A, 0x001D5867, 0x001D5665, 0x00245A6B, 0x002A6071, 0x00225A6D, 0x001C5669, 0x00225C6F, 0x001C576A, 0x00215C6F, 0x00235E71, 0x00205B6E, 0x001E596C, 0x001F5A6D, 0x001F5A6D, 0x001D586B, 0x00005463,
        0x00348DA2, 0x0056A9BF, 0x00185C6F, 0x00295F6A, 0x007CB0BC, 0x00306A80, 0x00054869, 0x003B8BA4, 0x0066C1D6, 0x00258194, 0x00004C5F, 0x002D5F75, 0x002B576E, 0x000E4D62, 0x003A90A4, 0x003E90B3, 0x000F536A, 0x002C636C, 0x0032636B, 0x00184D5A,
        0x001E5669, 0x001B5162, 0x001B4F5B, 0x001F5768, 0x00215768, 0x0072A2B4, 0x009CC1CF, 0x0051686A, 0x0091988B, 0x00C7BFA2, 0x00CBBA93, 0x00DAC598, 0x00FFFBC3, 0x00FEF0B5, 0x00BAAE78, 0x00CDC09A, 0x00F3E7CF, 0x00B2AD94, 0x0089866A, 0x00A3CAC2,
        0x006AABAC, 0x000B535D, 0x002E5968, 0x00677377, 0x005C6158, 0x00616357, 0x006F695E, 0x00897F6D, 0x00777368, 0x0062625C, 0x0065635B, 0x007C7463, 0x008B806C, 0x006C6556, 0x0064645E, 0x0067604F, 0x00817A69, 0x00867F6E, 0x006D6655, 0x00625B4A,
        0x00625B4A, 0x008E8776, 0x00948D7C, 0x006B665D, 0x00555348, 0x00656351, 0x007E8471, 0x00234348, 0x00154F6C, 0x00376E83, 0x00507571, 0x00FDFFF8, 0x00FFFDFE, 0x00F0EAE3, 0x00CECDB8, 0x00EEEEDC, 0x00698B8A, 0x002B6168, 0x00B2CCD2, 0x00FAFFFF,
        0x00FCF8F7, 0x00F5EFE8, 0x00D2DACF, 0x0084A59E, 0x00245D5F, 0x00064B5A, 0x001A6078, 0x000A5361, 0x002C6D7C, 0x0036607D, 0x000C5575, 0x00225D6C, 0x005F6259, 0x00575C47, 0x0045463D, 0x00595D51, 0x00646355, 0x00696350, 0x0069654D, 0x00656149,
        0x00605A47, 0x00717062, 0x00575B4F, 0x005B5445, 0x00565646, 0x00646757, 0x00666757, 0x00585344, 0x005C5345, 0x00535041, 0x00676A5A, 0x0070735D, 0x005F695D, 0x00525147, 0x005C5240, 0x005B6E5F, 0x004E9192, 0x0060ABB4, 0x003B6A72, 0x00125F7A,
        0x00115E79, 0x00105D78, 0x000F5E79, 0x0012617C, 0x0013647F, 0x00146580, 0x00146580, 0x000F6281, 0x000E6180, 0x000E6180, 0x000E6180, 0x000F6281, 0x000E6180, 0x000D607F, 0x000C5F7E, 0x000D607F, 0x000E6180, 0x000F6281, 0x00106382, 0x00106382,
        0x000E6180, 0x000D607F, 0x000B5E7D, 0x00095B74, 0x000C607A, 0x00277E99, 0x0055B0CF, 0x005BBCDE, 0x0049AED4, 0x0049B2D9, 0x0046B0D9, 0x0056B0D9, 0x003CA2CC, 0x0033A5CE, 0x0032A2D0, 0x0042ACDB, 0x0042A8D8, 0x0041ABDA, 0x0030A3D0, 0x00319CD6,
        0x0037A2DF, 0x002692D2, 0x00329FE3, 0x0038A7EB, 0x002F9FE1, 0x0031A2E1, 0x002A9BD9, 0x0028A8D9, 0x0024A1DB, 0x00229DDB, 0x002DA5E0, 0x0028A0CE, 0x002CA5D1, 0x002EA9DB, 0x0031ADE9, 0x0028A7DE, 0x002EA9E1, 0x0032A9E0, 0x0035A9E0, 0x002F9FD4,
        0x002A9ACF, 0x0030A0D5, 0x0037A7DB, 0x00279ECF, 0x002CA3D4, 0x0030A6D7, 0x0034A7D9, 0x0032A3D5, 0x0035A4D6, 0x0038A5D8, 0x002E9BCE, 0x002690BF, 0x002490BF, 0x002793BD, 0x00147FA5, 0x002D93B6, 0x003392B3, 0x00247B9B, 0x00004867, 0x00136882,
        0x004C9EB7, 0x0019637B, 0x0007475F, 0x00134C62, 0x001B5267, 0x00094457, 0x00145766, 0x001B516A, 0x000E4F64, 0x00095666, 0x00085D65, 0x0059AAAF, 0x0071B4B7, 0x0095C5C9, 0x009FC3C9, 0x0082B9C0, 0x005D95A0, 0x00336E7E, 0x000E4D62, 0x000E546C,
        0x0007516D, 0x00126281, 0x004FA1C0, 0x002D7286, 0x003F778A, 0x008AB1BF, 0x00B7D1DF, 0x0094AEBE, 0x004E758B, 0x002D6483, 0x00498DB0, 0x00297893, 0x0007546E, 0x000B546A, 0x00105368, 0x00105166, 0x0016566E, 0x00104F6B, 0x000D4D6B, 0x0012546D,
        0x00075066, 0x000F536A, 0x00124E6B, 0x000E5472, 0x0000576B, 0x00126664, 0x00599A8A, 0x00282828, 0x00000100, 0x0021251A, 0x00494C3C, 0x00464A37, 0x004E503D, 0x004E4F3F, 0x00444535, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00727867, 0x003C453B, 0x00101409, 0x00221E13, 0x00575249, 0x001E261C, 0x003B5C55, 0x003E7971, 0x0017645D, 0x00215B71, 0x001D576D, 0x001D576D, 0x001E586E, 0x001E586E,
        0x00225C72, 0x00255F75, 0x00225C72, 0x001F596C, 0x00225C6F, 0x001F586D, 0x001E576C, 0x00225B70, 0x00215A70, 0x001B546A, 0x00175066, 0x001B556B, 0x00215B71, 0x00255F75, 0x00215B71, 0x001C566C, 0x001C566C, 0x001F596F, 0x00215B71, 0x00358180,
        0x00529DA5, 0x005397A4, 0x0072A7B4, 0x0094BEC5, 0x0085A9B3, 0x00275164, 0x00598BA9, 0x005DA4B8, 0x0023798D, 0x003B98AD, 0x003989A0, 0x000F4661, 0x002D566F, 0x002A5B71, 0x00185366, 0x004395BF, 0x00377F9D, 0x001E5C6E, 0x00195565, 0x001A5B70,
        0x0013536B, 0x00195467, 0x00265B65, 0x001A5564, 0x0017505F, 0x006DA1AE, 0x00B1D8E1, 0x004C6464, 0x00979E91, 0x00C3BA9F, 0x00B6A380, 0x00BBA57C, 0x00F3E0AD, 0x00E6D59C, 0x00D7C695, 0x00EFDEBD, 0x00BEB29A, 0x00848068, 0x00535538, 0x00739C94,
        0x00549494, 0x00094A53, 0x002A4A55, 0x005F6161, 0x0019170C, 0x00080900, 0x00323222, 0x007B6C59, 0x005E5649, 0x00130F0A, 0x000B0400, 0x00544836, 0x008E806A, 0x00594E40, 0x000D0700, 0x001B1100, 0x00615746, 0x008E8473, 0x00433928, 0x00130900,
        0x00211706, 0x007E7463, 0x00877D6C, 0x002F2720, 0x00090200, 0x003F3726, 0x0070705E, 0x00233E42, 0x005386A0, 0x0082B6C7, 0x0063857E, 0x00E5EFDE, 0x00FFFFFB, 0x00E1EADD, 0x00B5C3A7, 0x00E5F2DA, 0x0074A198, 0x00397A78, 0x00B8E1DA, 0x00FCFDF9,
        0x00FFFFF9, 0x00F5E9DD, 0x00D7DACA, 0x0099B5A8, 0x00205651, 0x001D5E67, 0x000C5063, 0x003D7685, 0x006DB0BF, 0x007AB4D1, 0x0049A2C4, 0x004D8CA0, 0x00514F4F, 0x00121A10, 0x00000505, 0x0032392A, 0x00857467, 0x006F5B4A, 0x000F1100, 0x000C0E00,
        0x00554130, 0x007A695C, 0x003A4132, 0x000E0F00, 0x00180F01, 0x0068574A, 0x007D7062, 0x00322F20, 0x00080900, 0x001B1607, 0x00706053, 0x00775D4F, 0x00292621, 0x000A0905, 0x003B2F25, 0x00596160, 0x002F6E82, 0x002B88A7, 0x003390AA, 0x003193BD,
        0x003094BE, 0x003397C1, 0x00369AC4, 0x00379DC7, 0x00389EC8, 0x00359EC7, 0x00339CC5, 0x00339AC7, 0x00349BC8, 0x00359CC9, 0x00349BC8, 0x00349BC8, 0x00339AC7, 0x003299C6, 0x00339AC7, 0x002E95C2, 0x002F96C3, 0x002F96C3, 0x003097C4, 0x002F96C3,
        0x002F96C3, 0x002E95C2, 0x002D94C1, 0x003096B9, 0x002C92B5, 0x0043ABCE, 0x0052BCE1, 0x003EA9CE, 0x003AA7CD, 0x0048B7DD, 0x0040B2D7, 0x0044A7D3, 0x0039A2CB, 0x004AB7DD, 0x0043ABD0, 0x003595B9, 0x002A86A9, 0x003495B6, 0x003196B6, 0x0051AABF,
        0x0049A6BF, 0x0048ABC9, 0x0036A1C2, 0x001987AB, 0x002B99BD, 0x002692B4, 0x002E96B9, 0x002F96AF, 0x00208DA9, 0x00349FBB, 0x0047A4BB, 0x005AA8B8, 0x005FB0BF, 0x0046AAC2, 0x001D94B4, 0x0033AAE8, 0x003EB5F3, 0x002AA4E0, 0x002AA2DE, 0x002DA3DE,
        0x0032A4E0, 0x003AA8E2, 0x00339FD9, 0x002AA8DE, 0x002DA9DF, 0x00239FD5, 0x0030AAE0, 0x002EA8DE, 0x002FA6DD, 0x00259CD3, 0x001388BF, 0x000F7CA8, 0x001885B1, 0x002B9BC6, 0x001C8CB6, 0x001583AD, 0x00208AB3, 0x003397C1, 0x000F719B, 0x00004464,
        0x001B748F, 0x003595AC, 0x0014677D, 0x000E485E, 0x001F4C61, 0x00255B6C, 0x000C525F, 0x000F556D, 0x000A4C5E, 0x004D8D91, 0x0098D4CA, 0x0099CBB9, 0x00B9D9C6, 0x00EAF8E6, 0x00F7F8E8, 0x00FAFCFD, 0x00E8F7FA, 0x00B9DEE2, 0x00629BA4, 0x0014596A,
        0x00004A62, 0x00074F6D, 0x003E83A4, 0x00449EBD, 0x002C7492, 0x00326180, 0x00476582, 0x00395877, 0x0038688C, 0x00468EB6, 0x003692BD, 0x000E5B76, 0x00044F69, 0x000F556C, 0x000E5166, 0x000F5065, 0x0016566E, 0x000E4F6B, 0x000C4E6B, 0x0014566F,
        0x00075066, 0x000C5067, 0x00114D6A, 0x00105674, 0x00035A6E, 0x00106462, 0x004B8C7C, 0x001C1C1C, 0x00000100, 0x0021251A, 0x004B4E3E, 0x00484C39, 0x004E503D, 0x004E4F3F, 0x00434434, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00727867, 0x00394238, 0x000C1005, 0x00221E13, 0x0058534A, 0x0021291F, 0x0041625B, 0x00427D75, 0x001C6962, 0x001E586E, 0x001F596F, 0x001F596F, 0x001D576D, 0x001B556B,
        0x001D576D, 0x001F596F, 0x001E586E, 0x00255B72, 0x00225B71, 0x001D566F, 0x001A566E, 0x001E5973, 0x00235B76, 0x00235A75, 0x00235775, 0x00215A73, 0x001F5871, 0x001D566F, 0x001E5770, 0x00225B74, 0x00235C75, 0x00205972, 0x001C556E, 0x00A3D3D4,
        0x00AFE0E8, 0x008DBECC, 0x007BA3AF, 0x00618285, 0x00638382, 0x0087AFB4, 0x007CADBB, 0x001C5467, 0x00054A5E, 0x000D6076, 0x0055A4BF, 0x0061A0BC, 0x006392AD, 0x0075A3B5, 0x00689CA9, 0x004798B9, 0x006AC0E4, 0x003088AD, 0x00045576, 0x0013576E,
        0x001B5666, 0x00275D6E, 0x00265B70, 0x002A6574, 0x001D5763, 0x004C808C, 0x00C8F4FB, 0x0043605D, 0x007D877B, 0x00D2CBB7, 0x00B3A287, 0x00D4C1A0, 0x00CBBA8F, 0x00E2D1A0, 0x00E2D1A6, 0x00E2D2B5, 0x00BFB5A3, 0x006C6D59, 0x00424B31, 0x0066928B,
        0x00337171, 0x00114A53, 0x003D525A, 0x00675E5B, 0x00201807, 0x00111200, 0x003A3B2B, 0x0087836B, 0x00545446, 0x000B1007, 0x00101408, 0x0056533E, 0x007F7962, 0x004A4836, 0x000F150A, 0x00201E0C, 0x00747260, 0x00989684, 0x00464432, 0x00181604,
        0x002C2A18, 0x00767462, 0x007D7B69, 0x002B2E25, 0x000F1308, 0x00383A27, 0x00646D59, 0x00214447, 0x002B6B83, 0x003A7D8C, 0x003D7167, 0x00FDFFEE, 0x00F5EFE8, 0x00BAC4B8, 0x00A5B499, 0x00E9F1DA, 0x0061867C, 0x00215D57, 0x00C4E8DC, 0x00FDFFFB,
        0x00F5F0E7, 0x00E7DED0, 0x00E3E7D4, 0x00A0BDAE, 0x00235952, 0x00185961, 0x002E7082, 0x008CC3DE, 0x0085D1E8, 0x0069B6D7, 0x003DA5CA, 0x004E91A8, 0x00595152, 0x001E1F15, 0x0004110F, 0x00343D29, 0x00797563, 0x005D5744, 0x00080F00, 0x000D1400,
        0x004D4734, 0x006F6B59, 0x003C4531, 0x000D1504, 0x00181808, 0x004D4839, 0x005F5C4D, 0x002C3221, 0x000C1403, 0x00181B0B, 0x00625D4E, 0x00806D58, 0x0028261C, 0x000F0700, 0x003E2A1F, 0x00565E5E, 0x001E667E, 0x000D79A2, 0x002993BC, 0x002897C7,
        0x002C9BCB, 0x002D9FCE, 0x002EA0CF, 0x002DA1D0, 0x002DA1D0, 0x002A9ECD, 0x00249AC9, 0x002B9CCE, 0x002C9DCF, 0x002E9FD1, 0x002E9FD1, 0x002C9DCF, 0x002B9CCE, 0x002C9DCF, 0x002D9ED0, 0x002E9FD1, 0x002E9FD1, 0x002D9ED0, 0x002D9ED0, 0x002D9ED0,
        0x002C9DCF, 0x002C9DCF, 0x002C9DCF, 0x00309FCD, 0x002998C6, 0x0035A2CE, 0x0038A6D0, 0x0036A2CC, 0x0042AED7, 0x0045B2D8, 0x0038A3C9, 0x0032A1D3, 0x0048B4E3, 0x0044AAD4, 0x001B799C, 0x00004D6B, 0x00025771, 0x003087A1, 0x003A97AE, 0x0092B7AF,
        0x0078A6A0, 0x007EB7B8, 0x0063A7AC, 0x0019606A, 0x00155B68, 0x000D4C5A, 0x00114D59, 0x00034B5C, 0x00237384, 0x004A9CA8, 0x00659FA4, 0x0088AAA9, 0x0086ACAC, 0x0059A0AA, 0x001C8396, 0x001B85BA, 0x0039A7DB, 0x003AAFE2, 0x002CA3D4, 0x002DA4D5,
        0x002C9ECD, 0x003AA3D4, 0x003CA2D3, 0x002DA0CD, 0x0034A7D4, 0x0033A6D3, 0x0032A5D2, 0x002C9FCC, 0x001585B3, 0x002191BF, 0x001A8AB8, 0x00258AB1, 0x001982A9, 0x001985AE, 0x002D9CC4, 0x001F8FB9, 0x001D8AB6, 0x001C87B3, 0x00329AC9, 0x001F7599,
        0x00004667, 0x00086A88, 0x0046A1BC, 0x00105A74, 0x00125068, 0x00145368, 0x0014586B, 0x00004554, 0x005B9CA5, 0x00BCDBDA, 0x007F8679, 0x007A7964, 0x00D5D3BB, 0x00FDFCE7, 0x00FFFFF1, 0x00FDE6DE, 0x00FBEDE7, 0x00F7FDF8, 0x00DEFAFB, 0x0078A7AF,
        0x00195769, 0x000B526D, 0x00115A7A, 0x00338FB2, 0x0050A1C2, 0x005B9CBB, 0x00578EAD, 0x00518AAA, 0x005195B8, 0x00388BB1, 0x00005B83, 0x00054F6B, 0x00074E69, 0x0010546B, 0x000D4E63, 0x00115065, 0x0015556D, 0x000E4F6B, 0x000F516E, 0x0010526B,
        0x00075066, 0x000E5269, 0x00134F6C, 0x000D5371, 0x00045B6F, 0x001A6E6C, 0x00519282, 0x001E1E1E, 0x00000100, 0x001F2318, 0x00474A3A, 0x00474B38, 0x004E503D, 0x004E4F3F, 0x00404131, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFCFCFC, 0xFFFEFDFF, 0xFFFDFCFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x006C7261, 0x00424B41, 0x0011150A, 0x00242015, 0x00534E45, 0x00182016, 0x003A5B54, 0x003E7971, 0x001E6B64, 0x00245D73, 0x00235C72, 0x001F586E, 0x001D566C, 0x001F586E,
        0x00215A70, 0x00225B71, 0x00235C72, 0x001F556E, 0x00164F68, 0x0015526C, 0x001C5E77, 0x001C5D79, 0x00195572, 0x001A526F, 0x00205373, 0x001E5770, 0x001F5871, 0x00205972, 0x00205972, 0x001E5770, 0x001C556E, 0x001B546D, 0x001B546D, 0x00396D74,
        0x00295D6D, 0x0033667A, 0x00729CA9, 0x005C7B7A, 0x0096ADA5, 0x0099B4AB, 0x00668783, 0x003B6578, 0x0021586D, 0x00003E57, 0x00125974, 0x005091AD, 0x007EB8CE, 0x0076AEBF, 0x006CA8B4, 0x0057A4B4, 0x0053B2D9, 0x004BBAF2, 0x001579A9, 0x00004860,
        0x00255862, 0x002A586A, 0x00174B6F, 0x001D5566, 0x001F5867, 0x002B616C, 0x00BDEDF3, 0x008DAFAE, 0x00596B64, 0x008A8D7E, 0x008E8776, 0x00E3D7BF, 0x00D6CAA8, 0x00C9BD95, 0x00C9BB97, 0x007F745E, 0x00626153, 0x004B5848, 0x0029402A, 0x00588281,
        0x000B484C, 0x00013841, 0x0033464B, 0x007A6E68, 0x007C7260, 0x006E6F5B, 0x006A6D5D, 0x0088836A, 0x00787868, 0x0065695E, 0x005F6253, 0x00736F57, 0x00908970, 0x00797562, 0x00696D61, 0x006D6956, 0x0084806D, 0x008D8976, 0x008A8673, 0x007E7A67,
        0x00807C69, 0x008C8875, 0x00938F7C, 0x0083877C, 0x0076786C, 0x00737361, 0x00737968, 0x002D5157, 0x00276B88, 0x00216D80, 0x00326F6B, 0x00F5F3E8, 0x00B6AEAF, 0x008D9896, 0x00AFBCAC, 0x00FAFAEE, 0x006B8081, 0x002D5958, 0x00D6EEE6, 0x00DBF0ED,
        0x00C9D5CF, 0x00BBC1B6, 0x00E1F2E4, 0x0088B0A4, 0x00114D4C, 0x002D717E, 0x007EC4DB, 0x0086CCF4, 0x006AC3E5, 0x004CA2CC, 0x001F8BB4, 0x0033738B, 0x006C605C, 0x00645E4B, 0x00515347, 0x00655C48, 0x006C715C, 0x005A5F4A, 0x00504938, 0x00554E3D,
        0x004E533E, 0x00606550, 0x0059503C, 0x00463F30, 0x004C4D3D, 0x005B6150, 0x00676A5A, 0x00555041, 0x004A4334, 0x00484838, 0x00626857, 0x00666B4B, 0x00404530, 0x00504230, 0x00674F3B, 0x005B6B64, 0x0023768B, 0x001482AC, 0x002D8BB9, 0x002E9ECC,
        0x0033A3D1, 0x0033A6D3, 0x0032A5D2, 0x002FA4D1, 0x002FA4D1, 0x002DA2CF, 0x00269DCA, 0x002FA2CF, 0x0030A3D0, 0x0032A5D2, 0x0032A5D2, 0x0031A4D1, 0x0030A3D0, 0x0030A3D0, 0x0031A4D1, 0x002FA2CF, 0x002EA1CE, 0x002DA0CD, 0x002DA0CD, 0x002C9FCC,
        0x002DA0CD, 0x002DA0CD, 0x002DA0CD, 0x00309DD1, 0x00309DD0, 0x00339ED1, 0x003BA4D5, 0x004AB1DE, 0x004DB3DE, 0x0044A8D2, 0x0044A6D0, 0x0039AFE3, 0x003FABDB, 0x001E7AA3, 0x00004F70, 0x00004A64, 0x00247086, 0x004897AC, 0x0019687D, 0x00254B4F,
        0x0017474D, 0x00195561, 0x00468C9D, 0x002A758B, 0x0000475F, 0x0014566F, 0x0015536B, 0x000D5773, 0x00378BA7, 0x00237790, 0x000E4C5E, 0x00153B47, 0x001F4654, 0x001F687E, 0x002A93B4, 0x00025C7A, 0x001D7B98, 0x003498B4, 0x00339DB5, 0x0038A2BA,
        0x001F839B, 0x00036079, 0x0009637C, 0x000B647F, 0x000C6580, 0x00136C87, 0x00126B86, 0x002B849F, 0x0007627D, 0x0027829D, 0x001F7A95, 0x003488A4, 0x00207792, 0x0024809D, 0x001D7F9D, 0x003496B8, 0x002A8BAF, 0x001C7DA3, 0x002C8BB2, 0x003B99BC,
        0x001B7294, 0x00005174, 0x00146586, 0x004098B6, 0x00025974, 0x00115B77, 0x00104E6C, 0x00347F88, 0x00B7E4E8, 0x00848987, 0x00A6938B, 0x00EFDCCD, 0x00FDF2E4, 0x00F2EBE2, 0x00BBB2AE, 0x00F2DFD0, 0x00E5D2C3, 0x00D7C7BB, 0x00EEE8E3, 0x00E1F0F3,
        0x005A8693, 0x00084E65, 0x00005873, 0x00004C69, 0x001F6E89, 0x00388AA3, 0x003B91A7, 0x00368CA4, 0x001D728C, 0x0000536F, 0x00004C6B, 0x000A506D, 0x000B506B, 0x0010536A, 0x00125166, 0x00155469, 0x0014546C, 0x000C4F6A, 0x000E526F, 0x000F516A,
        0x00095268, 0x0010546B, 0x0014506D, 0x000A506E, 0x00045B6F, 0x001E7270, 0x004E8F7F, 0x001B1B1B, 0x00030400, 0x0021251A, 0x00464939, 0x00464A37, 0x004C4E3B, 0x004F5040, 0x00404131, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFCFCFC, 0xFFFEFDFF, 0xFFFDFCFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00666C5B, 0x00414A40, 0x000E1207, 0x00242015, 0x00514C43, 0x00171F15, 0x003A5B54, 0x003A756D, 0x001A6760, 0x00235C72, 0x00215A70, 0x001D566C, 0x0020596F, 0x00255E74,
        0x00235C72, 0x00215A70, 0x00245D73, 0x00235670, 0x00215A73, 0x001D5D75, 0x00125870, 0x000D536B, 0x00175672, 0x00235B78, 0x00255775, 0x00235D73, 0x00205A70, 0x001E586E, 0x001F596F, 0x00205A70, 0x001D576D, 0x001A546A, 0x001A546A, 0x000D5A63,
        0x000E5A6C, 0x00084E66, 0x001D5162, 0x0094B3B6, 0x006E7E77, 0x00707B71, 0x00BCCAC4, 0x006291A7, 0x00184A60, 0x00276076, 0x00104C64, 0x00266278, 0x008FCCE0, 0x0070B3C2, 0x005197A4, 0x005CA0A7, 0x003993B2, 0x004DBDF2, 0x0057C4F8, 0x00388BAA,
        0x00286474, 0x0024596E, 0x0023597C, 0x0021566B, 0x00265E71, 0x00134A57, 0x006CA2A9, 0x00D1FCFF, 0x005D7C79, 0x0034433F, 0x00616962, 0x00706F61, 0x006E6C54, 0x00A9A386, 0x00B6AF96, 0x00595648, 0x00424C46, 0x00254037, 0x00446B5B, 0x00325961,
        0x00033E47, 0x00043C47, 0x002F4348, 0x007E746D, 0x00ABA18F, 0x00A2A290, 0x009B9B8D, 0x0095846A, 0x00988B7B, 0x00978E84, 0x008F8476, 0x009D8C77, 0x00C5B198, 0x00AE9F8C, 0x00978F82, 0x00A29281, 0x008F7F6E, 0x00958574, 0x00BCAC9B, 0x00B7A796,
        0x00B6A695, 0x00B4A493, 0x00B6A695, 0x00999083, 0x009B9285, 0x00978878, 0x00787267, 0x002C4755, 0x004D8BB3, 0x004893B3, 0x001B5C64, 0x00668F8A, 0x00628C93, 0x0079B7BD, 0x005A968E, 0x006D9290, 0x00356B72, 0x00105A60, 0x00528A85, 0x0083B6B9,
        0x0080A7A9, 0x00698885, 0x008CB0AA, 0x003C7272, 0x00256C76, 0x0077C3DA, 0x0085D2F3, 0x0060B9E5, 0x0050AFD6, 0x0053A4D1, 0x00349AC4, 0x0038798E, 0x00726960, 0x00837C63, 0x00827B6A, 0x008C6E5B, 0x007B7661, 0x00706E5C, 0x00856D61, 0x00866E62,
        0x005E5C4A, 0x006E6954, 0x007C5E4B, 0x007D675B, 0x0074695B, 0x00807D6E, 0x009A9384, 0x008B786B, 0x00836B5F, 0x007A6A5D, 0x00736E5F, 0x005F6B47, 0x00626D53, 0x008C816B, 0x008B755C, 0x0055695C, 0x0014687A, 0x00157BA6, 0x003687BA, 0x002997C1,
        0x002F9DC7, 0x00319FC9, 0x002C9CC6, 0x002C9CC6, 0x002D9FC8, 0x002C9EC7, 0x00289AC3, 0x002C9DC8, 0x002D9EC9, 0x002E9FCA, 0x002FA0CB, 0x002E9FCA, 0x002E9FCA, 0x002E9FCA, 0x002E9FCA, 0x0032A3CE, 0x0031A2CD, 0x0031A2CD, 0x0030A1CC, 0x0030A1CC,
        0x0030A1CC, 0x0030A1CC, 0x0030A1CC, 0x00329FD2, 0x0037A2D5, 0x00329BCE, 0x00379ECF, 0x003EA4D4, 0x003799C7, 0x003A9AC8, 0x0050B0DE, 0x003FAFDA, 0x00207DA4, 0x00256D8B, 0x00245D72, 0x00407A86, 0x00417E88, 0x006EABB7, 0x002B6876, 0x0041787B,
        0x002C686E, 0x00337882, 0x00408B9B, 0x004F9DAE, 0x001B6779, 0x00256F81, 0x002F7586, 0x00296E78, 0x0058A1AF, 0x003C8391, 0x00427D86, 0x00436F70, 0x004C7B79, 0x0033747C, 0x005EB4C6, 0x00286C73, 0x00397F86, 0x00287175, 0x00368084, 0x003E888C,
        0x0056A1A3, 0x002D7477, 0x003C8386, 0x00397379, 0x00336D73, 0x002B676D, 0x00347076, 0x00609EA4, 0x00458389, 0x005C9CA1, 0x00408085, 0x005C97A6, 0x003E7B89, 0x0064A7B6, 0x00236C7A, 0x0059A4B4, 0x00388294, 0x0059A1B3, 0x003C8397, 0x0051A3B5,
        0x005FA2B9, 0x00487E97, 0x0027647E, 0x004C9EB7, 0x003491A8, 0x00065A74, 0x000C506D, 0x009BC4C7, 0x00C3D5D6, 0x00B6B1B0, 0x00FEF4ED, 0x00FFFFF7, 0x00FDFFF9, 0x00AEA9A6, 0x00887578, 0x00BEC0AD, 0x00FFFAE8, 0x00E9D4C5, 0x00A48D85, 0x00D4CFCE,
        0x00AECBD4, 0x00155A6B, 0x0000586F, 0x001C5A72, 0x00074D64, 0x00004F64, 0x00004E5F, 0x00005262, 0x00005163, 0x00014D60, 0x001A5D72, 0x000E526F, 0x000F506C, 0x0012526A, 0x0017566B, 0x0019586D, 0x0014546C, 0x000C4F6A, 0x000A4E6B, 0x0011536C,
        0x00095268, 0x000F536A, 0x0014506D, 0x00094F6D, 0x00045B6F, 0x001A6E6C, 0x003A7B6B, 0x000F0F0F, 0x00030400, 0x0024281D, 0x00494C3C, 0x00464A37, 0x004A4C39, 0x00505141, 0x00434434, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00686E5D, 0x003C453B, 0x00080C01, 0x00221E13, 0x00544F46, 0x001F271D, 0x0044655E, 0x003E7971, 0x0019665F, 0x00245A71, 0x00235970, 0x0020566D, 0x0021576E, 0x0022586F,
        0x0020566D, 0x00295F76, 0x003D738A, 0x006397AE, 0x005C96AC, 0x004A8DA2, 0x002A758B, 0x00125D73, 0x00175972, 0x00245C77, 0x00265874, 0x001F5A6D, 0x00256073, 0x002B6679, 0x00296477, 0x00205B6E, 0x001C576A, 0x00256073, 0x00326D80, 0x001B646C,
        0x000F5B6D, 0x00115B75, 0x0018546A, 0x002D5B63, 0x006C8C8B, 0x0086A5A2, 0x004D6C6D, 0x000D5168, 0x00135267, 0x001A5166, 0x001E5062, 0x00265A6B, 0x002B6676, 0x00064B5C, 0x00044E60, 0x00154B5C, 0x001F6279, 0x00398EAE, 0x0058BCE6, 0x0049B0DD,
        0x001C79A0, 0x00135C78, 0x00245E71, 0x00285773, 0x00295D74, 0x001A5263, 0x00316974, 0x00A7DBE1, 0x008FBABD, 0x00436265, 0x00395052, 0x0048514E, 0x0041483B, 0x00545642, 0x00777868, 0x00454C47, 0x0024393A, 0x002D5553, 0x00356D62, 0x00123543,
        0x00053E4D, 0x00104958, 0x004A646A, 0x006D6A62, 0x005A5342, 0x00464636, 0x00636157, 0x0096876D, 0x00786E5C, 0x00433D30, 0x003F3829, 0x00756750, 0x00A7967B, 0x007E715B, 0x00423C2F, 0x004B3D2A, 0x00766855, 0x00A0927F, 0x00827461, 0x004F412E,
        0x005B4D3A, 0x00918370, 0x00A89A87, 0x005F5648, 0x00494032, 0x00605343, 0x0077756D, 0x0025465A, 0x002672A2, 0x002C85B1, 0x0002556A, 0x00004345, 0x00287D8D, 0x0071D9E8, 0x00107377, 0x000B4E57, 0x004794A7, 0x004DAEBC, 0x003F8E91, 0x004492A2,
        0x001C5E69, 0x00235A61, 0x001A5158, 0x001E656F, 0x006CBFD5, 0x007FD4FA, 0x005FB1E1, 0x005FC6E6, 0x00419CBB, 0x004282AB, 0x003C93BB, 0x0046889B, 0x005B5C53, 0x00484530, 0x003F3629, 0x0069553C, 0x00867963, 0x006E6655, 0x004A3F31, 0x004D4234,
        0x005C5443, 0x007B6E58, 0x0069553C, 0x00494233, 0x00483D2F, 0x0078675A, 0x00968679, 0x00686152, 0x004D4839, 0x004F4638, 0x006E5E51, 0x007D7159, 0x004A503F, 0x00474936, 0x005C4F39, 0x0058655D, 0x002A6C85, 0x001B77AA, 0x002A81B9, 0x002995C5,
        0x00319DCD, 0x0034A0D0, 0x002E9CCC, 0x002E9CCC, 0x00309FCF, 0x00309FCF, 0x002B9ACA, 0x002C9FCC, 0x002C9FCC, 0x002C9FCC, 0x002DA0CD, 0x002EA1CE, 0x002FA2CF, 0x002EA1CE, 0x002DA0CD, 0x002C9FCC, 0x002C9FCC, 0x002C9FCC, 0x002C9FCC, 0x002C9FCC,
        0x002C9FCC, 0x002B9ECB, 0x002B9ECB, 0x00289BC8, 0x0030A3D0, 0x002F9FCD, 0x0035A5D3, 0x0037A6D4, 0x0039A5D4, 0x0046B0DF, 0x0042ACDB, 0x001B7990, 0x001B6375, 0x0096C1CA, 0x007F9E9D, 0x00A9CAC3, 0x0060877F, 0x00A0C9C4, 0x00729A99, 0x00B1C5B9,
        0x00849990, 0x00B0C6C1, 0x007E9492, 0x00A1B9B7, 0x0088A39F, 0x008CA8A2, 0x0093B0A7, 0x0082AC9B, 0x0097BEB6, 0x00779C98, 0x0096B8B1, 0x0082A591, 0x00A4C8B2, 0x006B9080, 0x00A6CAC4, 0x00608984, 0x009CC5C0, 0x0069918C, 0x0094BAB4, 0x00628B84,
        0x00A0CBC2, 0x00629188, 0x0085B7AD, 0x007A9B97, 0x0092B3AF, 0x007EA19D, 0x0081A6A2, 0x0086AEA9, 0x00739C97, 0x0081ADA7, 0x0069958F, 0x00A1C3C3, 0x006C9090, 0x0097C1C0, 0x004C7B79, 0x0090C2BE, 0x00548380, 0x0098C5C2, 0x00588380, 0x009BC6C3,
        0x005A817F, 0x009EC4C6, 0x005E888F, 0x0085BCC5, 0x004A8E9B, 0x00085764, 0x00217483, 0x00CEF4EE, 0x00ADBBB7, 0x00EEE5E2, 0x00FFFAF3, 0x00EEF8EC, 0x00BCCFC0, 0x00A8B3A3, 0x009F998C, 0x00718170, 0x00D5D8C8, 0x00E0D3C3, 0x00827164, 0x00928E89,
        0x00E2FBFF, 0x00467F8E, 0x00024E64, 0x00175069, 0x00114E68, 0x0013566F, 0x0008526A, 0x00044E66, 0x00115A70, 0x0014576C, 0x000F4E62, 0x000F4F6D, 0x0011506C, 0x0015536B, 0x0018546A, 0x00145368, 0x00115169, 0x000C516C, 0x00074D6A, 0x0011536C,
        0x00085167, 0x000D5168, 0x00134F6C, 0x00084E6C, 0x00045B6F, 0x001A6E6C, 0x00317262, 0x000A0A0A, 0x00020300, 0x0022261B, 0x00474A3A, 0x00454936, 0x00494B38, 0x00505141, 0x00424333, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x006B7160, 0x00454E44, 0x000B0F04, 0x00211D12, 0x00534E45, 0x00222A20, 0x004B6C65, 0x00447F77, 0x001E6B64, 0x00245A71, 0x00265C73, 0x00245A71, 0x00235970, 0x00235970,
        0x002E647B, 0x00558BA2, 0x0084BAD1, 0x0073A8BD, 0x00619CB0, 0x0063A8BB, 0x0068B4C7, 0x004A96A9, 0x0021657C, 0x001B546D, 0x00295C76, 0x00266272, 0x0065A1B1, 0x0097D3E3, 0x007FBBCB, 0x003C7888, 0x00246070, 0x005D99A9, 0x00A7E3F3, 0x00ACDBDF,
        0x004D8493, 0x0013506A, 0x0013506A, 0x00215C6B, 0x005C969B, 0x0049898D, 0x00226A71, 0x00439BB3, 0x0019647A, 0x00205A6D, 0x0043707E, 0x002B5866, 0x001B4F5F, 0x00145367, 0x00165C73, 0x00275574, 0x005D8A98, 0x0092CBD4, 0x005BB5D3, 0x003BB3EE,
        0x0044BDFB, 0x00227D9C, 0x00236161, 0x002A5675, 0x0022536D, 0x0020586B, 0x00084450, 0x00255E67, 0x00B2E4EA, 0x00AAD1D9, 0x004E7076, 0x003B4A4C, 0x002C3B33, 0x003A4434, 0x0040483D, 0x00323E3E, 0x00274449, 0x002E6060, 0x00074741, 0x00204053,
        0x000C4457, 0x00134F5F, 0x007A979E, 0x007F7F79, 0x001D1A0B, 0x00050400, 0x003C3833, 0x008C8C6E, 0x00616651, 0x00000B00, 0x00000800, 0x00505235, 0x00848264, 0x0050543B, 0x00000600, 0x000C0F00, 0x0060634D, 0x0091947E, 0x0032351F, 0x00040700,
        0x001E210B, 0x006E715B, 0x007B7E68, 0x001F2611, 0x00000600, 0x002C301D, 0x0078877F, 0x00295E73, 0x000A6A9F, 0x001E90BF, 0x002492AC, 0x00115665, 0x00135977, 0x0054AECC, 0x00106274, 0x0019475F, 0x005388A9, 0x005399B6, 0x00487C8C, 0x0005687E,
        0x00005264, 0x000E5563, 0x00226776, 0x0074C3D6, 0x0078D2F1, 0x0058B3E0, 0x0063BAF3, 0x0043AEC3, 0x00479DB3, 0x006090B4, 0x0059A4CB, 0x005092A5, 0x003D4741, 0x00101605, 0x000E0803, 0x003A3F22, 0x00918670, 0x00686150, 0x00000700, 0x00000F00,
        0x005A5342, 0x00857A64, 0x0044492C, 0x00000900, 0x00161606, 0x006E5B4E, 0x00797062, 0x00223522, 0x00000E00, 0x00101A09, 0x0079695C, 0x0091685F, 0x0027241F, 0x00000800, 0x002D2917, 0x005C5E5E, 0x00366688, 0x001C75AE, 0x002B8ECC, 0x002E9AD0,
        0x0036A2D8, 0x0039A5DB, 0x0034A1D7, 0x00329FD5, 0x0033A3D8, 0x0032A2D7, 0x002D9DD2, 0x002AA0D3, 0x00299FD2, 0x00299FD2, 0x002BA1D4, 0x002DA3D6, 0x002DA3D6, 0x002CA2D5, 0x002AA0D3, 0x002BA1D4, 0x002BA1D4, 0x002CA2D5, 0x002CA2D5, 0x002CA2D5,
        0x002BA1D4, 0x002AA0D3, 0x002AA0D3, 0x002CA6CE, 0x0032ACD4, 0x002DA6D0, 0x002DA5CF, 0x00289FCC, 0x0037ADDC, 0x0038AEDD, 0x000174A6, 0x0004555A, 0x001D5354, 0x00C1D8D0, 0x00697362, 0x00C1CFB7, 0x006B836B, 0x00AAC4B2, 0x00688173, 0x009DC1B0,
        0x00628374, 0x009AB8AB, 0x006E897F, 0x0098B3A9, 0x00759485, 0x0090B39F, 0x00739A81, 0x0080998B, 0x008FA29F, 0x007F8F96, 0x009AAFB1, 0x005F7E75, 0x009FBEAF, 0x00667970, 0x00B6BCBB, 0x005F7973, 0x00ADC3BE, 0x005F6F68, 0x00B6C4BE, 0x00606F67,
        0x00A8BDB4, 0x005D7B6E, 0x009ABDAF, 0x006A807B, 0x0099AFAA, 0x007B928D, 0x00859F99, 0x00738F89, 0x0076958E, 0x007E9F98, 0x006A8B84, 0x0094A59C, 0x006B8077, 0x0093ADA0, 0x00607C6F, 0x0098B7A8, 0x00506D5D, 0x00A0BAAA, 0x005D7563, 0x00AEB5A0,
        0x00657863, 0x0095B2A2, 0x005D7B70, 0x009CB7B4, 0x00618A8C, 0x0011585B, 0x00359797, 0x00AAE8DA, 0x00778F87, 0x00F1E2DF, 0x00F7E1DC, 0x00A3A294, 0x0095AB93, 0x00B3D2B1, 0x00ADC6A4, 0x008DA194, 0x00778272, 0x008E8E7C, 0x00959283, 0x00747970,
        0x00E3FAFC, 0x006996A3, 0x00175468, 0x00175370, 0x00154E6D, 0x00184D6E, 0x00215173, 0x001A4D6D, 0x00134B68, 0x0014546C, 0x00105569, 0x000C4C6A, 0x0011506C, 0x0014526A, 0x00145066, 0x000C4B60, 0x000E4E66, 0x00105570, 0x00094F6C, 0x000E5069,
        0x00075066, 0x000C5067, 0x00124E6B, 0x00064C6A, 0x00055C70, 0x00207472, 0x00387969, 0x000F0F0F, 0x00030400, 0x001C2015, 0x00404333, 0x00434734, 0x004A4C39, 0x00505141, 0x003E3F2F, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFE, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x006F7367, 0x004B5342, 0x000A0B02, 0x001F1717, 0x004D4A45, 0x001A291B, 0x00365C4A, 0x003C776F, 0x00296E71, 0x001F5871, 0x0024596D, 0x00295D6E, 0x001A546A, 0x00135675,
        0x003B86A6, 0x0068B0C8, 0x00357986, 0x0029677F, 0x00557991, 0x00347185, 0x00427190, 0x0062A0C6, 0x00459FC2, 0x00155470, 0x000D5D68, 0x00538494, 0x00DFF1F2, 0x00FFFFF6, 0x00F6FFFE, 0x0088B9C3, 0x00417C8C, 0x00D2EFF6, 0x00FFFFF8, 0x00F1FFF7,
        0x00C2DFE3, 0x0023546E, 0x00084E6C, 0x00578FAA, 0x00C5EDFF, 0x007AB2C3, 0x003997A4, 0x002C88A1, 0x0046A4BB, 0x006DC3D7, 0x00A4DFF2, 0x00527285, 0x003F5967, 0x00234E57, 0x00216165, 0x0049707E, 0x00DAE1F5, 0x00FFF7FF, 0x00DFE2F0, 0x0065AFC9,
        0x0034B3E6, 0x0043B9F4, 0x002D7CAF, 0x00064763, 0x00165974, 0x00165871, 0x000E445B, 0x00224B61, 0x001D4754, 0x0068A0A5, 0x008FD5D4, 0x00649AA5, 0x0034616C, 0x001A4449, 0x001A4C4C, 0x001C5E5D, 0x0010595D, 0x00054550, 0x0016495D, 0x000C475B,
        0x00143A5A, 0x001F5573, 0x004A7E8B, 0x005D6260, 0x002D2F1C, 0x000C1E07, 0x004E493A, 0x0084846C, 0x00666551, 0x00242113, 0x001F1A11, 0x00555047, 0x007E7B6D, 0x00575642, 0x001F1F07, 0x002B2E15, 0x006A634F, 0x00918474, 0x00534E3F, 0x000F1504,
        0x0029321E, 0x007B7863, 0x007B6D57, 0x002B2C28, 0x00302118, 0x004E3931, 0x00747C7C, 0x00215C6C, 0x00177B9D, 0x001D91BA, 0x0030A1CC, 0x0023656A, 0x000E515A, 0x00064C59, 0x00094F60, 0x000C5466, 0x000D5265, 0x00084D60, 0x00115568, 0x001F6277,
        0x0018555F, 0x003C7479, 0x007EC3D2, 0x0078D0F4, 0x0053B8E5, 0x0057BCE3, 0x0050ACC9, 0x00519AB0, 0x0054A5CA, 0x0050A5D7, 0x005EA9D7, 0x00487A91, 0x00425758, 0x001E2216, 0x00161402, 0x00434129, 0x006D6E5A, 0x0054594A, 0x001A2015, 0x00161C11,
        0x00444738, 0x00716F5D, 0x004B4430, 0x001B2623, 0x00282520, 0x0063574B, 0x00736C59, 0x00343622, 0x0019220E, 0x0027251A, 0x00625650, 0x0074624B, 0x002A2E15, 0x00222B16, 0x002A291F, 0x00546568, 0x00327080, 0x00237997, 0x003782AE, 0x002D95B8,
        0x0035A0C6, 0x002F9BC5, 0x002D9CCA, 0x002D9DD1, 0x002CA0D5, 0x0033A7DE, 0x00279BD4, 0x002CA1D4, 0x002CA1D4, 0x002CA1D4, 0x002CA1D4, 0x002BA0D3, 0x002BA0D3, 0x002BA0D3, 0x002BA0D3, 0x002AA0D3, 0x002BA1D4, 0x002CA2D5, 0x002BA1D4, 0x002AA0D3,
        0x00299FD2, 0x002AA0D3, 0x002CA2D5, 0x002CA2D5, 0x002E9FD7, 0x00319EDC, 0x002B9CDA, 0x0031A7DB, 0x002C9EC7, 0x00137391, 0x0007576E, 0x00124859, 0x0059605B, 0x0076765E, 0x0063725D, 0x006D7467, 0x0067705C, 0x0064705E, 0x006B605C, 0x006D7362,
        0x006D7362, 0x006C7261, 0x006B7160, 0x00696F5E, 0x00666C5B, 0x00656B5A, 0x00646A59, 0x00696E5F, 0x00636859, 0x00686D5E, 0x00666B5C, 0x00666B5C, 0x006A6F60, 0x006A6F60, 0x00757A6B, 0x00696F5E, 0x006B7160, 0x00676D5C, 0x006D7060, 0x00656858,
        0x006C6D5D, 0x006B6C5C, 0x00707161, 0x00676858, 0x00727363, 0x00727363, 0x006A6D5D, 0x006D7060, 0x006F7564, 0x006C7261, 0x00696F5E, 0x006A705F, 0x006B7160, 0x00707363, 0x006A6D5D, 0x00707363, 0x006D6E5E, 0x006F7060, 0x00676858, 0x007F7563,
        0x00676759, 0x006E766B, 0x006E6E60, 0x00817B68, 0x005B6051, 0x001C413F, 0x0066A5AD, 0x0096DCCF, 0x003E6157, 0x00D6D4CA, 0x00D7CBBF, 0x00A9AA9A, 0x00C1CEBE, 0x00E8F1E4, 0x00F3F1E7, 0x00B5C4A9, 0x00969C85, 0x00A8A491, 0x00ACA69B, 0x00686D6B,
        0x00E2FFFF, 0x006AA2AD, 0x000D5664, 0x00005170, 0x00085673, 0x0007506C, 0x000E546C, 0x00135971, 0x00044C64, 0x0006516B, 0x0005526D, 0x00135064, 0x0013536C, 0x00125372, 0x000F5273, 0x00094E6F, 0x000A506E, 0x000A5270, 0x00024E6B, 0x000F526D,
        0x0009536B, 0x000B5168, 0x0014506E, 0x000C4F70, 0x0003586E, 0x001D6F70, 0x00397668, 0x000C0D09, 0x00030500, 0x001F2317, 0x00434636, 0x00464A37, 0x004A4B3B, 0x004F4F41, 0x00444438, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFE, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x0074786C, 0x00464E3D, 0x000C0D04, 0x001F1717, 0x004A4742, 0x001F2E20, 0x00426856, 0x00417C74, 0x00256A6D, 0x00286079, 0x00285D71, 0x00275D6E, 0x00215D73, 0x001E6482,
        0x00559CBD, 0x003D7D96, 0x00437C8B, 0x008CB8C9, 0x00A8C7D6, 0x0097C4CF, 0x00628FA4, 0x003F7A9A, 0x004997BB, 0x00307694, 0x000E5A6D, 0x006690A3, 0x00EEFCFF, 0x00FFFCF9, 0x00F6FFFF, 0x0094BFCE, 0x004C8196, 0x00DFF7FF, 0x00FFFEFE, 0x00F0FCF6,
        0x00B8F0F5, 0x0053A7B9, 0x00044B66, 0x005F94A8, 0x00A2D8DF, 0x00437679, 0x008AB2B7, 0x0062A7B6, 0x003D7A88, 0x0041717D, 0x00859FAB, 0x008A949E, 0x00818A8E, 0x005E7273, 0x00587774, 0x00C5DBE0, 0x00EDF2FF, 0x00FFF3FF, 0x00FFFFFB, 0x00B9E5DF,
        0x005DBBD2, 0x0048B5E8, 0x0047A7E3, 0x002D748F, 0x0000435B, 0x00054B62, 0x001E576C, 0x00245061, 0x001F4955, 0x001A5154, 0x000D4F4D, 0x0031707E, 0x002C616E, 0x002F5D68, 0x0023555B, 0x000D4B51, 0x000B4E57, 0x00114D5D, 0x0011465B, 0x000E485B,
        0x00113955, 0x001D5572, 0x00538798, 0x005A6569, 0x007E8276, 0x007F8B77, 0x008F8678, 0x00948D7C, 0x00867F70, 0x00888073, 0x008E857B, 0x0080776D, 0x008B8376, 0x00867F70, 0x00807968, 0x007B775E, 0x00968B75, 0x009F917F, 0x009C9483, 0x009B9B89,
        0x008A8C78, 0x00918B74, 0x00907E67, 0x00918E86, 0x008B8879, 0x00898A7A, 0x005D6E6A, 0x000A384A, 0x0029779C, 0x00228AB9, 0x0031A7D8, 0x0020626D, 0x00115562, 0x000E5362, 0x000F5465, 0x00125669, 0x00115568, 0x000E5062, 0x00155769, 0x000E4A56,
        0x00347584, 0x0072BBD1, 0x006EC4E8, 0x0052B4E2, 0x0053B8E5, 0x004CAACD, 0x00489EB6, 0x005CAFCB, 0x0054ACD1, 0x0050A6D4, 0x004F97BF, 0x00325F74, 0x00485D5B, 0x00686C60, 0x00767364, 0x007C6B58, 0x00807161, 0x00665A50, 0x00584D45, 0x005D524A,
        0x005E4F46, 0x00786556, 0x006A5442, 0x00515248, 0x00696154, 0x006E604E, 0x007B705A, 0x005F5E44, 0x0063654F, 0x005A5342, 0x00756559, 0x006D5C53, 0x00606354, 0x00505546, 0x006A625B, 0x00596161, 0x00002A35, 0x00024358, 0x00053E5E, 0x00004E67,
        0x00005471, 0x00086A8C, 0x002C95BC, 0x0042AFDB, 0x0037A6D6, 0x002E9FD1, 0x002FA2D4, 0x0030A2D1, 0x0030A2D1, 0x0030A2D1, 0x0030A2D1, 0x002FA1D0, 0x002FA1D0, 0x002FA1D0, 0x002FA1D0, 0x002A9ECD, 0x002B9FCE, 0x002CA0CF, 0x002B9FCE, 0x002A9ECD,
        0x00299DCC, 0x002B9FCE, 0x002CA0CF, 0x0029A0CD, 0x0031A0D2, 0x0036A2D8, 0x002E9DD5, 0x002BA1D2, 0x002698BD, 0x000A6B87, 0x0000475D, 0x00023A4B, 0x00414843, 0x005E5E46, 0x004B5A45, 0x004F5649, 0x004A533F, 0x00525E4C, 0x00635854, 0x00535948,
        0x00525847, 0x00525847, 0x00525847, 0x00525847, 0x00515746, 0x00505645, 0x004E5443, 0x004F5445, 0x004B5041, 0x00505546, 0x00505546, 0x00515647, 0x00535849, 0x00505546, 0x00575C4D, 0x00545A49, 0x00565C4B, 0x00515746, 0x00575A4A, 0x00515444,
        0x00585949, 0x00565747, 0x00595A4A, 0x00515242, 0x00595A4A, 0x00595A4A, 0x00525545, 0x00545747, 0x00545A49, 0x00515746, 0x00515746, 0x00525847, 0x00525847, 0x00545A49, 0x00505343, 0x00565949, 0x00565747, 0x005B5C4C, 0x00575848, 0x00625846,
        0x004C4C3E, 0x004F574C, 0x00525244, 0x0067614E, 0x00494E3F, 0x00123735, 0x004E8D95, 0x0099DBD0, 0x0047665D, 0x009F9A91, 0x00DDCCBF, 0x00E0DCCA, 0x00E9F5DF, 0x00F0FBE7, 0x00D7D9C6, 0x009BA48A, 0x00CDCFB9, 0x009A9481, 0x00756F64, 0x00676C6A,
        0x00D3F2F5, 0x007FB8C1, 0x00024C58, 0x000B546A, 0x000D536A, 0x000E5166, 0x00125166, 0x00115065, 0x0014526A, 0x0012526B, 0x000C4D69, 0x00145068, 0x0016526A, 0x0015526C, 0x0015526C, 0x0016536D, 0x0014546D, 0x0010526B, 0x000A4C65, 0x000F526D,
        0x0008526A, 0x000A5067, 0x0014506E, 0x000C4F70, 0x0004596F, 0x001C6E6F, 0x00357264, 0x000C0D09, 0x00040600, 0x00212519, 0x00444737, 0x00474B38, 0x004A4B3B, 0x004C4C3E, 0x003F3F33, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFE, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x006F7367, 0x00404837, 0x000F1007, 0x001F1717, 0x0045423D, 0x00253426, 0x004D7361, 0x00437E76, 0x00216669, 0x00285B75, 0x00285B6F, 0x00225A6B, 0x001D5E73, 0x00387E9C,
        0x005B9EBF, 0x00336883, 0x008AB2C4, 0x00EBFDFF, 0x00E8FFFF, 0x00EFFFFF, 0x00ADD5DA, 0x00458094, 0x004C8BAD, 0x004695B6, 0x000B4E6F, 0x002E6279, 0x00ABC9D4, 0x00F1FDFF, 0x00C8E2E9, 0x004C8195, 0x0016546C, 0x008EB5C4, 0x00E9F3FA, 0x00CCE9EE,
        0x003F8690, 0x002B8A9D, 0x004097AB, 0x001F6572, 0x001D5658, 0x007CA3A1, 0x00AEBBB9, 0x004D8487, 0x00365C60, 0x004D5E61, 0x00888C8D, 0x00B9BCBA, 0x00B4BBB4, 0x009BA399, 0x009DA398, 0x00F9FBF5, 0x00FAFEFF, 0x00FFFDFD, 0x00FFFFEE, 0x00F8FFEB,
        0x0091CED0, 0x0044A6D0, 0x0034A3E1, 0x0037748E, 0x001D5D75, 0x003B768A, 0x00739FB0, 0x007A95A3, 0x00788F97, 0x008AABAE, 0x0072A19F, 0x002F7C8F, 0x00115366, 0x000B4152, 0x001C4F5F, 0x001A5160, 0x00104B5B, 0x00154E63, 0x00134962, 0x00104754,
        0x0018465E, 0x00094360, 0x00427891, 0x00263D4C, 0x002B3634, 0x002F382B, 0x0044392B, 0x0034322A, 0x0028271D, 0x002F2F23, 0x0039392B, 0x002C2C1E, 0x002F2F23, 0x002A291F, 0x0036342C, 0x00323323, 0x002E2B1D, 0x00352E25, 0x003E3C34, 0x003A3F36,
        0x00252B20, 0x00302F21, 0x0042392B, 0x00352E25, 0x003D412E, 0x001D2F18, 0x0019342B, 0x0003283C, 0x002D6A96, 0x003D9DD3, 0x00249DD5, 0x00195D70, 0x000A4E61, 0x00105265, 0x00125467, 0x00135365, 0x001A5A6C, 0x0018586A, 0x00114F61, 0x00326E74,
        0x0073BDD5, 0x0068C6F3, 0x004AB3EC, 0x004FB8EB, 0x0044A4CC, 0x00439CB7, 0x0057ADC3, 0x0057B6DD, 0x0051B0D8, 0x004197C1, 0x00246786, 0x0008323F, 0x00203231, 0x00454A41, 0x00313125, 0x00333627, 0x00262A1E, 0x002A3025, 0x002B3329, 0x0031362D,
        0x0027291D, 0x002C2C1C, 0x00322E1C, 0x002D2F29, 0x0039372D, 0x003A3225, 0x002F2A1B, 0x00404131, 0x002E3122, 0x00353329, 0x00382E27, 0x00342C2D, 0x00363A35, 0x0032342E, 0x00372E2A, 0x00222424, 0x0015353A, 0x00002F3B, 0x0018455A, 0x00327D8D,
        0x00358799, 0x004AA3BE, 0x0050B2D4, 0x0040ABD1, 0x0032A2CC, 0x002EA1CC, 0x0031A4CF, 0x0030A2D1, 0x0030A2D1, 0x0030A2D1, 0x0030A2D1, 0x0030A2D1, 0x002FA1D0, 0x002FA1D0, 0x002FA1D0, 0x0031A4D1, 0x0032A5D2, 0x0031A4D1, 0x0030A3D0, 0x002FA2CF,
        0x0030A3D0, 0x0031A4D1, 0x0033A6D3, 0x002BA1CA, 0x0031A0CE, 0x0035A0D3, 0x002C9CD0, 0x002EA4D3, 0x003AAFD6, 0x00349BBB, 0x0029829D, 0x001D5A6E, 0x00424D4A, 0x00585A44, 0x004F5B47, 0x00565A4E, 0x00515A46, 0x0057634F, 0x00625A53, 0x00575D4C,
        0x00565C4B, 0x00555B4A, 0x00555B4A, 0x00565C4B, 0x00565C4B, 0x00545A49, 0x00525847, 0x00515647, 0x004E5344, 0x0054594A, 0x00565B4C, 0x00575C4D, 0x00575C4D, 0x00525748, 0x0054594A, 0x00555B4A, 0x00555B4A, 0x00515746, 0x00565949, 0x00535646,
        0x00595A4A, 0x00565747, 0x00575848, 0x00585949, 0x005D5E4E, 0x005C5D4D, 0x00585B4B, 0x005A5D4D, 0x00565C4B, 0x00545A49, 0x00575D4C, 0x00555D4C, 0x00565C4B, 0x00595F4E, 0x00505645, 0x00565949, 0x00545747, 0x00595A4A, 0x00565747, 0x00615745,
        0x00515143, 0x004E564B, 0x00515143, 0x00625C49, 0x00464B3C, 0x00133836, 0x0033727A, 0x009CDCD6, 0x005A7A75, 0x0077746C, 0x00817063, 0x00A59C88, 0x00FBFFE5, 0x00D2D9BE, 0x009C9F85, 0x00C4C6B0, 0x00DBD7C4, 0x0077705F, 0x00464237, 0x00A5B0AD,
        0x00E1FFFF, 0x00518A93, 0x00135B67, 0x001D5B6B, 0x00074555, 0x00145466, 0x001E5D72, 0x0007475F, 0x00094A66, 0x000D4D6B, 0x001A5C79, 0x00286F90, 0x001F6380, 0x0014546C, 0x000E4D62, 0x000D4D65, 0x000C4F66, 0x0010536A, 0x001C5D72, 0x00155873,
        0x000B556D, 0x000A5067, 0x00124E6C, 0x000A4D6E, 0x00055A70, 0x001E7071, 0x00347163, 0x000B0C08, 0x00040600, 0x0022261A, 0x00464939, 0x00484C39, 0x004B4C3C, 0x004A4A3C, 0x003B3B2F, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFE, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x0064685C, 0x003D4534, 0x00101108, 0x001D1515, 0x0043403B, 0x00273628, 0x004D7361, 0x00407B73, 0x00216669, 0x002D5E78, 0x002E6175, 0x00245F6F, 0x0014576C, 0x004991AF,
        0x004483A5, 0x0046728F, 0x00C9E4F8, 0x00FBFDFE, 0x00EBFFF7, 0x00FFFFF7, 0x00DFFFFF, 0x00619AA9, 0x00437899, 0x004A9EC2, 0x0016527C, 0x000C566E, 0x00286373, 0x004F7D88, 0x00326C78, 0x00055469, 0x0002566F, 0x00206275, 0x00487580, 0x0024646F,
        0x00195769, 0x000B5970, 0x003398AE, 0x00338FA2, 0x001C5564, 0x00789FA8, 0x00568688, 0x00124B48, 0x00597B7B, 0x00D1DBDB, 0x00FBFFFF, 0x00E4F4ED, 0x00D8F0E6, 0x00D6E6DB, 0x00CFD5CA, 0x00F0E7DD, 0x00E3E7E2, 0x00CFD5CA, 0x00C5C8AF, 0x00B4C7B8,
        0x0082BACB, 0x004AA6CF, 0x0030A0CB, 0x00327A92, 0x002E778B, 0x0073B6C5, 0x004C7F89, 0x0067898F, 0x00496369, 0x00587B7E, 0x00548381, 0x00368EA6, 0x0028788F, 0x0010526B, 0x00134B64, 0x001C526B, 0x001B516A, 0x00174F68, 0x000E4560, 0x00124853,
        0x000B4054, 0x00245E7B, 0x00376E8D, 0x0001273F, 0x001A2F37, 0x00232D27, 0x00181205, 0x00000B0A, 0x00000E0A, 0x0005160B, 0x00172A1B, 0x00041708, 0x00000C01, 0x0001100C, 0x00071514, 0x000E1B19, 0x00000C0E, 0x00030F13, 0x00000A12, 0x00071920,
        0x00132427, 0x00010D0F, 0x00000504, 0x0017160C, 0x00222817, 0x00263624, 0x00163631, 0x0003344A, 0x00236B9A, 0x00288BC5, 0x002EA2DD, 0x00195C77, 0x0013556E, 0x00115267, 0x00175468, 0x001B596B, 0x00114F61, 0x000E4D61, 0x002C6D82, 0x0074BED0,
        0x006AC5EC, 0x0047B2EC, 0x0048B6F0, 0x0045A8D4, 0x003B96B5, 0x0054AFCE, 0x0053B3D7, 0x004EB5E2, 0x003D9DC5, 0x001A6C8F, 0x00003E53, 0x00022B34, 0x00112627, 0x0037413B, 0x00292D27, 0x00082018, 0x00000D05, 0x00011712, 0x00000E06, 0x0000110A,
        0x00011207, 0x00021004, 0x00101C10, 0x00141C1C, 0x00020A09, 0x00000B09, 0x00000907, 0x0011221F, 0x00000C0B, 0x00071315, 0x00030A0D, 0x00000909, 0x0015201E, 0x000D1711, 0x00212722, 0x00182624, 0x00173A3E, 0x00063844, 0x0053899A, 0x0074BACB,
        0x00438FA5, 0x00459CBC, 0x0051B6DD, 0x0041AEDA, 0x0031A6D3, 0x002AA1CE, 0x002AA2CC, 0x002DA2D5, 0x002CA1D4, 0x002CA1D4, 0x002CA1D4, 0x002CA1D4, 0x002BA0D3, 0x002BA0D3, 0x002BA0D3, 0x002BA0D3, 0x002BA0D3, 0x002A9FD2, 0x00299ED1, 0x00299ED1,
        0x00299ED1, 0x002BA0D3, 0x002CA1D4, 0x002CA5D1, 0x0031A3D2, 0x0033A1D5, 0x002B9FD4, 0x0027A4D6, 0x0031AEDB, 0x0037A9D2, 0x00349BC2, 0x0027687D, 0x003A4847, 0x0052533F, 0x00515B4A, 0x005A5C50, 0x00535946, 0x00515D49, 0x0057514A, 0x00565E4D,
        0x00535B4A, 0x00515948, 0x00515948, 0x00525A49, 0x00515948, 0x004E5645, 0x004A5241, 0x004D5445, 0x004C5344, 0x004E5546, 0x00505748, 0x00535A4B, 0x00545B4C, 0x00515849, 0x00505748, 0x005A604F, 0x00575D4C, 0x00545A49, 0x00575A4A, 0x00585B4B,
        0x005C5D4D, 0x005A5B4B, 0x00585949, 0x00585949, 0x005B5C4C, 0x005A5B4B, 0x00585B4B, 0x005A5D4D, 0x00535948, 0x00535948, 0x005A604F, 0x00525A49, 0x00525A49, 0x005C6251, 0x00525847, 0x00565C4B, 0x00545747, 0x00525545, 0x004F5242, 0x00655B49,
        0x005B5B4D, 0x00555D52, 0x005A5A4C, 0x00645E4B, 0x00454A3B, 0x000F3432, 0x00115058, 0x0084CFCD, 0x007CABA8, 0x003E4842, 0x003E3528, 0x006C634F, 0x00AAA68D, 0x00A6A589, 0x00BFBB9F, 0x00A29F90, 0x00656253, 0x00515042, 0x005F645B, 0x00C8DBD8,
        0x00C7EDF1, 0x002E6973, 0x00115764, 0x00185061, 0x00114E62, 0x00064E66, 0x00004966, 0x000C6385, 0x003692B5, 0x003B97BA, 0x0056B1D6, 0x005EBFEB, 0x0062BCDF, 0x005AAFC9, 0x003B91A9, 0x0017708B, 0x00015878, 0x000B5E7A, 0x00307D97, 0x001C5F7A,
        0x000E5870, 0x000A5067, 0x00114D6B, 0x00084B6C, 0x00065B71, 0x00227475, 0x00367365, 0x000A0B07, 0x00040600, 0x00212519, 0x00454838, 0x00484C39, 0x004C4D3D, 0x004C4C3E, 0x003B3B2F, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFE, 0xFFFEFDFF, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x0064685C, 0x003F4736, 0x00101108, 0x001C1414, 0x0043403B, 0x00253426, 0x00456B59, 0x003A756D, 0x0024696C, 0x002C5D77, 0x002C5F73, 0x00256070, 0x0013566B, 0x00448CAA,
        0x004988AA, 0x004D7996, 0x00B5D0E4, 0x00FBFAFC, 0x00EFFFFB, 0x00FFFFF6, 0x00C9ECE8, 0x005891A0, 0x0046799A, 0x003E8FB4, 0x001B5781, 0x000F5A74, 0x0014576C, 0x00144F62, 0x00115568, 0x0005586E, 0x00065A74, 0x00145A71, 0x00164E61, 0x000A5A6B,
        0x001F5D75, 0x0015516E, 0x00035372, 0x003F93B6, 0x004C93B4, 0x00266B86, 0x00388A9C, 0x005F8C8F, 0x00D6EFF3, 0x00FAFDFF, 0x00F1F5FA, 0x00E4F6F5, 0x00D0EBE7, 0x00A8BCB7, 0x00D1D8D5, 0x00EBE7E2, 0x00A8AEA3, 0x0086907F, 0x00788A7D, 0x00538193,
        0x0051A2D5, 0x004DADDB, 0x003F9EB2, 0x00268098, 0x0046A2B5, 0x000B646E, 0x00115A5E, 0x000A4243, 0x00124243, 0x00154B4B, 0x00084848, 0x001B7892, 0x0028819C, 0x00095976, 0x00004261, 0x0017506F, 0x001A506E, 0x00154C67, 0x00154E67, 0x00164C57,
        0x00003747, 0x0016526A, 0x00427998, 0x00023856, 0x00193E52, 0x00A3B6BB, 0x00828F87, 0x00152732, 0x00000D14, 0x002B4041, 0x00AAC1BD, 0x007C938F, 0x00001516, 0x00061920, 0x003A4C57, 0x0097A7A6, 0x00475C5D, 0x00001519, 0x00001117, 0x0070888E,
        0x00B0C4C9, 0x0045595A, 0x00011614, 0x000D1E1B, 0x00929994, 0x00ACB8B2, 0x0040666B, 0x0008536D, 0x002B93BE, 0x003BA8DB, 0x00379DD1, 0x000C4F6E, 0x00175874, 0x0014526A, 0x00135064, 0x001C5A6C, 0x000C4F64, 0x00236981, 0x0073BCD8, 0x0069C9F1,
        0x004AB1E2, 0x0045B2E8, 0x0040A7D4, 0x00439DBB, 0x0054ABC5, 0x004FB1D9, 0x0047B4EA, 0x004DB0DC, 0x001A7398, 0x0000415B, 0x00003545, 0x000F3C47, 0x001D3D42, 0x00738889, 0x00A9BAB7, 0x00193038, 0x0003191F, 0x000D2023, 0x005E6D6F, 0x007F8B8B,
        0x001F2A28, 0x00000705, 0x00272F2E, 0x00737F79, 0x00556662, 0x00001915, 0x00001310, 0x00152F2F, 0x006D8585, 0x003D5152, 0x00011516, 0x00042521, 0x002B4642, 0x00829A92, 0x0053746D, 0x00052D2B, 0x00002D34, 0x00317385, 0x005AAAC1, 0x00347F99,
        0x002B7E9E, 0x003B9AC5, 0x0040ABDE, 0x0035A9DE, 0x0030A8DC, 0x002BA3D1, 0x002DA5CF, 0x002EA3DA, 0x002DA2D9, 0x002DA2D9, 0x002DA2D9, 0x002DA2D9, 0x002CA1D8, 0x002CA1D8, 0x002CA1D8, 0x002CA2D6, 0x002CA2D6, 0x002BA1D5, 0x002AA0D4, 0x002BA1D5,
        0x002BA1D5, 0x002CA2D6, 0x002CA2D6, 0x0029A5D5, 0x002EA1D4, 0x002F9FD4, 0x002DA1DA, 0x0024A3DA, 0x001DA1D7, 0x00229DD5, 0x00299AD2, 0x00206880, 0x00364849, 0x00575946, 0x00576151, 0x005E5C51, 0x00545845, 0x0054614B, 0x00615C53, 0x00596150,
        0x00565E4D, 0x00545C4B, 0x00545C4B, 0x00555D4C, 0x00555D4C, 0x00515948, 0x004E5645, 0x00535A4B, 0x0052594A, 0x00505748, 0x00515849, 0x00545B4C, 0x00575E4F, 0x005A6152, 0x00596051, 0x00595F4E, 0x00555B4A, 0x00525847, 0x00535646, 0x00565949,
        0x00585949, 0x00585949, 0x00555646, 0x00595A4A, 0x005B5C4C, 0x005A5B4B, 0x005A5D4D, 0x005C5F4F, 0x00555B4A, 0x00565C4B, 0x00616756, 0x00575F4E, 0x00555D4C, 0x00606857, 0x00585E4D, 0x005E6453, 0x005D6352, 0x005C5F4F, 0x00595C4C, 0x005E5442,
        0x0058584A, 0x0050584D, 0x0058584A, 0x00625C49, 0x0044493A, 0x000F3432, 0x00024149, 0x00308789, 0x008CCCCD, 0x00537470, 0x002F3A30, 0x00555547, 0x007A7965, 0x00615C47, 0x00847B67, 0x003D4037, 0x002E332A, 0x005A655D, 0x00ADC1BC, 0x00E3FFFF,
        0x0070A0A6, 0x000B4753, 0x00115667, 0x000F4762, 0x00377A99, 0x0022789C, 0x00006089, 0x00299FC9, 0x0054D1FD, 0x0030AED7, 0x0025A4CB, 0x0038AEE2, 0x003AAAD4, 0x0038A5C5, 0x0036A7C8, 0x0048C0EA, 0x004FC3F2, 0x0039A2CB, 0x003491B2, 0x00185B76,
        0x000B556D, 0x000A5067, 0x00114D6B, 0x00084B6C, 0x00065B71, 0x00207273, 0x00306D5F, 0x00090A06, 0x00030500, 0x001F2317, 0x00424535, 0x00474B38, 0x004E4F3F, 0x004F4F41, 0x003E3E32, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFE, 0xFFFCFBFD, 0xFFFDFCFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x006D7165, 0x00434B3A, 0x000F1007, 0x001B1313, 0x0044413C, 0x00223123, 0x003E6452, 0x00367169, 0x00256A6D, 0x00265973, 0x0026596D, 0x00225A6B, 0x001B5C71, 0x002D7391,
        0x0062A5C6, 0x00427792, 0x006A92A4, 0x00E0F1FA, 0x00E9FFFF, 0x00E0F0EF, 0x0082A8AD, 0x00467F94, 0x00609DBF, 0x00347EA0, 0x00105072, 0x00215976, 0x001E536E, 0x0020566F, 0x001F5871, 0x00175672, 0x001D5C78, 0x001B526D, 0x0023526D, 0x00115671,
        0x000D556D, 0x00245F79, 0x001B496B, 0x00114E76, 0x003089AE, 0x00439DBC, 0x0069ACC7, 0x00DFEFF6, 0x00F8FDFF, 0x00F7F4FD, 0x00FFFCFF, 0x00F5F8FD, 0x00F8FFFF, 0x00EEF3F4, 0x00E6E8E9, 0x00F4FCFB, 0x00C9D1C7, 0x007F8776, 0x004D6662, 0x002A6B92,
        0x0043AAEF, 0x00369ED3, 0x00308595, 0x0065A8C9, 0x00569CB4, 0x005C9FAE, 0x0051878E, 0x007BA1A5, 0x00749499, 0x007FA7AC, 0x006EA0A6, 0x004199B7, 0x00449FBE, 0x001D7897, 0x00015376, 0x00115475, 0x00164F6E, 0x00114964, 0x0016556A, 0x00144C5D,
        0x00034857, 0x0000394D, 0x00366C8A, 0x002F7593, 0x00104863, 0x009FC0CF, 0x00D0F6F6, 0x00425A6C, 0x001D3646, 0x0037515F, 0x00D9F3FF, 0x00BED8E4, 0x0017313F, 0x00173040, 0x006E8698, 0x00D8EFEA, 0x00678782, 0x00133A38, 0x00244A4A, 0x00A3C2C3,
        0x00DFF9F9, 0x0062807B, 0x00062821, 0x00264A54, 0x00C3DEE8, 0x00CEE9F7, 0x006FA8C1, 0x0049ADD0, 0x003FB8E2, 0x003DA9D2, 0x00166E92, 0x000A4C69, 0x00094B64, 0x0017566B, 0x000F4D5F, 0x0005495C, 0x002E7890, 0x0060B1D2, 0x007AD0F4, 0x0044B2E6,
        0x0048B5E8, 0x0038A0CB, 0x003694B7, 0x004FAAC5, 0x005AB7D8, 0x004AB2E1, 0x003CAEEA, 0x002E83A9, 0x00024E6B, 0x00003B4E, 0x000A4352, 0x000A3D4D, 0x000C3946, 0x00597E86, 0x008FAEAF, 0x000E4155, 0x002B5C6C, 0x001F4C57, 0x007CA3AB, 0x00A6CBCF,
        0x0023464A, 0x00022529, 0x0021454B, 0x00AEBFB2, 0x00728E81, 0x00042D25, 0x0007332D, 0x00517876, 0x00A8C8C7, 0x005E7F7B, 0x001E423C, 0x00195555, 0x0054807F, 0x00B3DBD9, 0x004A827D, 0x004D8D8D, 0x0055969F, 0x0068B5C8, 0x002C90A8, 0x000F6484,
        0x0047A4CB, 0x0059C0F1, 0x003DAEE6, 0x0037ABE4, 0x003BAEE1, 0x0035A5CF, 0x003CA7CC, 0x0033A2D0, 0x0033A2D0, 0x0033A2D0, 0x0033A2D0, 0x0033A2D0, 0x0032A1CF, 0x0032A1CF, 0x0032A1CF, 0x0034A4CF, 0x0034A4CF, 0x0033A3CE, 0x0034A4CF, 0x0035A5D0,
        0x0035A5D0, 0x0034A4CF, 0x0033A3CE, 0x0035A9D2, 0x003BA6D2, 0x00399FCF, 0x003AA7DA, 0x0035ADE1, 0x0022A1D8, 0x001E99D3, 0x002596D4, 0x001F6A84, 0x0034494B, 0x00565747, 0x00545B4C, 0x005C574E, 0x00575945, 0x00515E48, 0x005C5A4F, 0x00515B4A,
        0x00505A49, 0x004F5948, 0x00505A49, 0x00525C4B, 0x00525C4B, 0x00515B4A, 0x004F5948, 0x00505A4A, 0x00525C4C, 0x004C5646, 0x004C5646, 0x004F5949, 0x00535D4D, 0x005A6454, 0x00596353, 0x005D6352, 0x00585E4D, 0x00565C4B, 0x00545747, 0x00595C4C,
        0x005A5B4B, 0x005B5C4C, 0x00595A4A, 0x00545545, 0x00575848, 0x00565747, 0x00555848, 0x00565949, 0x00515746, 0x00545A49, 0x005E6453, 0x00586251, 0x00515B4A, 0x005C6453, 0x004E5645, 0x00565C4B, 0x00595F4E, 0x00555B4A, 0x00565949, 0x00635947,
        0x005A5A4C, 0x004F574C, 0x00575749, 0x00635D4A, 0x00484D3E, 0x00173C3A, 0x000A4951, 0x0000454F, 0x00509AA0, 0x0074ABAE, 0x003F6462, 0x00243B36, 0x0044534B, 0x00222922, 0x001B1D17, 0x00152726, 0x00445A58, 0x00ACCBC8, 0x00D6FEFD, 0x0084B4B8,
        0x001A525D, 0x00155567, 0x00084B62, 0x00195272, 0x00246A8F, 0x003B9AC2, 0x003FB2DF, 0x0049CBFA, 0x003AC1ED, 0x0039BDE5, 0x0036B8DD, 0x003DB3E2, 0x004BBDE2, 0x0044B8D7, 0x0032AED6, 0x0042C9FF, 0x0047C9FF, 0x001B89B9, 0x00005C7D, 0x000E516C,
        0x00054F67, 0x00094F66, 0x0014506E, 0x00094C6D, 0x00055A70, 0x001C6E6F, 0x00266355, 0x00080905, 0x00030500, 0x001E2216, 0x00404333, 0x00474B38, 0x00515242, 0x00535345, 0x00414135, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFE, 0xFFFCFBFD, 0xFFFDFCFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x006D7165, 0x00464E3D, 0x000F1007, 0x001B1313, 0x0043403B, 0x00202F21, 0x003F6553, 0x0037726A, 0x0022676A, 0x00275F78, 0x00295E72, 0x00255B6C, 0x00235F75, 0x00155B79,
        0x004E95B6, 0x004D8DA6, 0x00467F8E, 0x0061899B, 0x007892A2, 0x00658D99, 0x00466F85, 0x00477D9E, 0x00519BBF, 0x00266685, 0x000F566A, 0x0024536F, 0x00194B67, 0x00205674, 0x001C546F, 0x00174E69, 0x00245974, 0x00154763, 0x00265574, 0x000F4F71,
        0x001C5E77, 0x00184F64, 0x00335F77, 0x00215877, 0x0007516D, 0x0077B0C5, 0x00D4E8F9, 0x00FFFCFF, 0x00FFFEFF, 0x00FAFCFD, 0x00FEFEFE, 0x00FFFAFB, 0x00FFFBFC, 0x00FCFFFD, 0x00D7E0DD, 0x00B2C6C1, 0x00C7D4CC, 0x00818579, 0x00495B54, 0x0027647E,
        0x0054BBF4, 0x0040ACE2, 0x0048A2C4, 0x006FA1CB, 0x004F84A5, 0x00497E92, 0x00507A86, 0x0059767D, 0x0039535A, 0x00587B85, 0x00467380, 0x002D7D9A, 0x00419CBB, 0x003EA3C3, 0x00137194, 0x00004A6B, 0x000B4D6A, 0x0014526A, 0x000C4E60, 0x00124C62,
        0x00085363, 0x001B5463, 0x000C4259, 0x00277A96, 0x00327E9B, 0x0034667C, 0x005D9CA4, 0x0072A6B7, 0x006B9EB2, 0x0043758B, 0x005D8FA6, 0x003E7087, 0x0017495F, 0x00477A8E, 0x006498A9, 0x00305F67, 0x0048818A, 0x006CB1BB, 0x0079BAC8, 0x005E95A4,
        0x004B7C8A, 0x00326972, 0x002C6A70, 0x005389A0, 0x00528CA2, 0x005FA5C2, 0x004FACD3, 0x0052C0F0, 0x002998C6, 0x00046180, 0x00045263, 0x00145772, 0x000E4F64, 0x0019596B, 0x00175867, 0x0022697D, 0x005AADC9, 0x006DCAF1, 0x0048ADDA, 0x0045B3E3,
        0x003BA4CD, 0x003193B5, 0x0049ABCD, 0x0045ABD5, 0x0043ADDC, 0x0050B8E7, 0x00187FAC, 0x000C516C, 0x000A475B, 0x00154C59, 0x00185160, 0x0009465A, 0x00013D53, 0x00154A57, 0x00224F53, 0x000E627E, 0x004C9CB3, 0x00277282, 0x00044953, 0x0013545C,
        0x0003424A, 0x00094A53, 0x0001434E, 0x00224C4B, 0x00235C5D, 0x002B757B, 0x003B8793, 0x003E8394, 0x00155161, 0x00205D6B, 0x004E919A, 0x00146574, 0x004C8594, 0x002C5F69, 0x000D5053, 0x0097DEE1, 0x00A6E3ED, 0x005AA2B3, 0x000E7080, 0x005DB8D7,
        0x005ABBE1, 0x0042ACDB, 0x0040ADE3, 0x0046B3E7, 0x002C95BE, 0x00177592, 0x001C7288, 0x0012718B, 0x0012718B, 0x0012718B, 0x0012718B, 0x0011708A, 0x0011708A, 0x0011708A, 0x0011708A, 0x00106D86, 0x00106D86, 0x00106D86, 0x00126F88, 0x00137089,
        0x00137089, 0x00106D86, 0x000E6B84, 0x0003647E, 0x00126C8A, 0x00186D8D, 0x002C88AB, 0x0040A9D0, 0x003AABD6, 0x0033A2D4, 0x00379ED5, 0x001D6E89, 0x00394F54, 0x00595C4D, 0x00565A4E, 0x00625B52, 0x005E604C, 0x00506048, 0x0058584C, 0x00525C4B,
        0x00525C4B, 0x00535D4C, 0x00535D4C, 0x00545E4D, 0x00545E4D, 0x00535D4C, 0x00525C4B, 0x00515B4B, 0x00566050, 0x00505A4A, 0x00505A4A, 0x00515B4B, 0x00535D4D, 0x005C6656, 0x00596353, 0x005C6251, 0x00565C4B, 0x00555B4A, 0x00505343, 0x00565949,
        0x00555646, 0x005A5B4B, 0x00585949, 0x00585949, 0x005C5D4D, 0x005B5C4C, 0x00575A4A, 0x00575A4A, 0x00545A49, 0x00585E4D, 0x00616756, 0x005E6857, 0x00545E4D, 0x005C6655, 0x004F5746, 0x00565E4D, 0x005B6150, 0x00535948, 0x00515746, 0x00665C4A,
        0x005A5A4C, 0x004D554A, 0x00535345, 0x00605A47, 0x00464B3C, 0x00143937, 0x000E4D55, 0x00003C4F, 0x000B505F, 0x00478A93, 0x0070AFB3, 0x004C8283, 0x001C474A, 0x00274A4E, 0x002C4950, 0x0055787C, 0x00A7D0D3, 0x00CDFDFF, 0x007AB4B9, 0x00205E68,
        0x00084A5C, 0x0010526B, 0x0010506E, 0x00144A68, 0x00003D5E, 0x001D759D, 0x0044B0DA, 0x0040B4DD, 0x00005E83, 0x00006383, 0x000A6983, 0x000C6786, 0x000B657D, 0x0001647A, 0x00007199, 0x0029A9E4, 0x003BB3F4, 0x00106E9C, 0x0000415C, 0x000C4F6A,
        0x00034D65, 0x000B5168, 0x00165270, 0x00094C6D, 0x00055A70, 0x001D6F70, 0x00256254, 0x00080905, 0x00030500, 0x001F2317, 0x00404333, 0x00484C39, 0x00535444, 0x00555547, 0x00414135, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFE, 0xFFFEFDFF, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x0064685C, 0x0048503F, 0x000F1007, 0x001A1212, 0x0043403B, 0x00202F21, 0x00426856, 0x003A756D, 0x001E6366, 0x00205972, 0x00295E72, 0x0025596A, 0x001F596F, 0x000E5170,
        0x00257090, 0x0075BDD5, 0x0063A7B4, 0x00528AA3, 0x0066859E, 0x00467D92, 0x0076A2C1, 0x0078AFD6, 0x00277B9F, 0x000C4663, 0x00115D69, 0x001B506B, 0x001F5A74, 0x00044762, 0x00175A75, 0x00175770, 0x00134F67, 0x001C5771, 0x0017516E, 0x00095579,
        0x001E4D69, 0x003B6375, 0x00175762, 0x0011596A, 0x005C8699, 0x00E8ECF7, 0x00FFFEFE, 0x00FFFBF7, 0x00F6FDF6, 0x00F1FFFB, 0x00EBF8F0, 0x00FFFFF9, 0x00FCF9F1, 0x00F4FDF3, 0x0098B2A5, 0x004F675F, 0x00768480, 0x007B7C73, 0x00879181, 0x00427375,
        0x0052B0D3, 0x0040ADE3, 0x0049ADE3, 0x00307FA8, 0x00004463, 0x00004656, 0x00045056, 0x000D4E4F, 0x0007464A, 0x00054D55, 0x00004E59, 0x00004664, 0x00025C7B, 0x0036A0BF, 0x0041AACB, 0x001C7393, 0x00135976, 0x001B5C71, 0x00195C6B, 0x00114B68,
        0x000A5767, 0x001A515E, 0x00194C60, 0x00004B64, 0x002A7F9F, 0x004C8CA5, 0x00004956, 0x0000505E, 0x00035A6E, 0x00287D97, 0x004DA0BF, 0x00196C8B, 0x002B809A, 0x004DA4B8, 0x00004A58, 0x00287191, 0x004DA4C4, 0x0051B2D4, 0x0067C5E9, 0x0072C5EB,
        0x0076C1E7, 0x0076C9E9, 0x0075D1EE, 0x0076B7D6, 0x004DA3C1, 0x0041AFD3, 0x003CB3E4, 0x00319FD9, 0x00217DAE, 0x003986A0, 0x0010595D, 0x000B4E65, 0x001B5D70, 0x00125060, 0x00256877, 0x006FBBCE, 0x0077CEEE, 0x0045ABD6, 0x0046B3E7, 0x003AA4C9,
        0x003597B5, 0x0046A7C3, 0x004AB3DC, 0x0040B3EC, 0x003EB1EE, 0x002188B5, 0x00004F6D, 0x000F485D, 0x00215562, 0x0020535D, 0x0017505F, 0x000F526B, 0x000E536E, 0x000E4C5C, 0x0022595E, 0x003D8BB0, 0x003E87A7, 0x004688A1, 0x00145060, 0x000B424F,
        0x001A515E, 0x00124A5B, 0x00114B5E, 0x000D536A, 0x00378EA8, 0x0036A0BF, 0x00319CC2, 0x002688B2, 0x002F88B4, 0x003794BB, 0x0016789A, 0x00348FAE, 0x00236381, 0x00164D62, 0x00095159, 0x00246B6F, 0x00A8DEE5, 0x0075B2BC, 0x005AB3BD, 0x0067C4DE,
        0x0058B9DB, 0x0049B1DC, 0x004BB5E4, 0x003096C1, 0x00096483, 0x00065667, 0x000C545C, 0x00055860, 0x00055860, 0x00055860, 0x0004575F, 0x0004575F, 0x0004575F, 0x0004575F, 0x0003565E, 0x0005565D, 0x0005565D, 0x0006575E, 0x00085960, 0x00095A61,
        0x00095A61, 0x0006575E, 0x0003545B, 0x00015561, 0x000C5767, 0x00004356, 0x00075369, 0x00217C97, 0x002286A8, 0x001C81A8, 0x001E7AA9, 0x00075A76, 0x0031494F, 0x005D6051, 0x00565A4E, 0x005E574E, 0x00595B47, 0x004D5D45, 0x005B5B4F, 0x00505A49,
        0x00505A49, 0x00505A49, 0x00505A49, 0x004E5847, 0x004D5746, 0x004C5645, 0x004C5645, 0x004A5444, 0x00515B4B, 0x004D5747, 0x004D5747, 0x004C5646, 0x004C5646, 0x00545E4E, 0x00505A4A, 0x005A604F, 0x00545A49, 0x00535948, 0x004C4F3F, 0x00535646,
        0x00515242, 0x00575848, 0x00565747, 0x00575848, 0x005D5E4E, 0x005B5C4C, 0x00555848, 0x00555848, 0x00535948, 0x00565C4B, 0x005E6453, 0x005A6453, 0x00505A49, 0x005C6655, 0x00525A49, 0x005B6352, 0x005F6756, 0x00565C4B, 0x00525847, 0x00635947,
        0x00565648, 0x004B5348, 0x00525244, 0x00605A47, 0x0044493A, 0x00103533, 0x000F4E56, 0x001A5067, 0x00003B4F, 0x00074F5B, 0x00135E66, 0x0069B3B9, 0x0095D6DF, 0x0093CAD7, 0x00A2D5E5, 0x00B8E6EE, 0x0090C4CB, 0x003C7A80, 0x000D535A, 0x00044C58,
        0x000D5468, 0x000C4E6B, 0x000C4B6D, 0x001B4D63, 0x0015556E, 0x00004768, 0x00016185, 0x0051B2D4, 0x000F6583, 0x000A5165, 0x00144F5F, 0x00084A5C, 0x00054752, 0x00085663, 0x00167A9C, 0x003EB2ED, 0x0040AAE9, 0x001A6690, 0x00205062, 0x0011546F,
        0x00075169, 0x000D536A, 0x00165270, 0x00084B6C, 0x00075C72, 0x00217374, 0x002B685A, 0x00080905, 0x00040600, 0x00202418, 0x00414434, 0x00494D3A, 0x00545545, 0x00555547, 0x00404034, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFE, 0xFFFEFDFF, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x005A5E52, 0x00404A3A, 0x000B0D07, 0x0015100F, 0x0045423E, 0x00253224, 0x00335747, 0x003C756D, 0x00216368, 0x00265A71, 0x00275B72, 0x0022586F, 0x00255E74, 0x00245E74,
        0x00175369, 0x0029687D, 0x005798AD, 0x0071B4CD, 0x0079BCD5, 0x0074B6CF, 0x005294AD, 0x00296982, 0x00105069, 0x00105069, 0x001B5872, 0x0013516F, 0x00124E6B, 0x001C546F, 0x001F5770, 0x001A4F6A, 0x00205674, 0x00154B6A, 0x001E5475, 0x000B516E,
        0x00165974, 0x0029627B, 0x002A576C, 0x006A8998, 0x00D9E9F0, 0x00FBFFFF, 0x00FFFDF9, 0x00F8FFF4, 0x00FDFFF9, 0x00FCFFFA, 0x00FDFFFE, 0x00F1F7F6, 0x00F4FFFF, 0x00E3F7F2, 0x00657D75, 0x00283331, 0x00576155, 0x00716D62, 0x00584D55, 0x00344D6F,
        0x003B90C0, 0x00219BC9, 0x0039B7E0, 0x00146677, 0x004C8097, 0x006F94A8, 0x006B9C9A, 0x0084B29B, 0x0084A089, 0x0090B1AD, 0x005F98A7, 0x00589BB0, 0x004195B8, 0x001075A2, 0x003292C0, 0x0061AFD4, 0x007FC3E0, 0x0077C2E2, 0x005CB4D8, 0x001A6788,
        0x00125875, 0x00105166, 0x00155161, 0x000E4C5E, 0x000B5168, 0x00297996, 0x0052A9CB, 0x00216074, 0x00054355, 0x00165F7D, 0x0044A0D3, 0x0043A0D1, 0x003F8AA4, 0x000E4F58, 0x003D7E87, 0x0074BACB, 0x005EB3D3, 0x004BAFDF, 0x004DB5EA, 0x0055B8EA,
        0x005CB8E9, 0x005DB7F0, 0x0059B5F6, 0x006BC0F2, 0x005BB9EE, 0x0052BBEE, 0x0044ABD2, 0x000F6981, 0x00529FAF, 0x0078BECF, 0x0011556C, 0x00045066, 0x00104B54, 0x00326A6F, 0x006ABFD5, 0x0055C5FA, 0x0034A7EA, 0x0050B9EC, 0x0041A4C0, 0x004F9BBF,
        0x003C98C1, 0x0045B4E6, 0x0041B3E8, 0x004FB6E7, 0x002986A7, 0x00025B69, 0x00005757, 0x001C506E, 0x00134962, 0x00155063, 0x00105461, 0x00034D59, 0x000C5765, 0x000A5263, 0x0023697A, 0x003C7C9A, 0x00215F6F, 0x00AFB8C5, 0x004E787F, 0x0000515B,
        0x001C5269, 0x00255A6F, 0x0018616F, 0x0069A1BA, 0x00549CB4, 0x00338CA7, 0x003E9EBC, 0x00308FB0, 0x001F7E9F, 0x00298FAC, 0x001F8EA8, 0x0029869F, 0x00247687, 0x00084C51, 0x0026615D, 0x0087C3C3, 0x0082CFD8, 0x0056B6CE, 0x0051BDDF, 0x0034A8E1,
        0x003CACE1, 0x0046B0DF, 0x0045AED7, 0x00329BC2, 0x001C88B1, 0x002591C0, 0x002491C4, 0x001D8CB4, 0x002793BC, 0x002790B7, 0x00228AAF, 0x00258CB3, 0x00248DB6, 0x00218AB6, 0x00218BBA, 0x00238BC0, 0x002288BC, 0x00278ABC, 0x002A8AB8, 0x002F8BB6,
        0x00368EB6, 0x001F7599, 0x0000486B, 0x000B5470, 0x0014506D, 0x001C4E6A, 0x00144964, 0x00064960, 0x00054E62, 0x00054D5F, 0x00004255, 0x000F363F, 0x0046473D, 0x00585C43, 0x004A5A49, 0x005A5850, 0x00535542, 0x004D5D45, 0x00515948, 0x00555547,
        0x00545446, 0x00545446, 0x00535345, 0x00545446, 0x00545446, 0x00555547, 0x00555547, 0x00505344, 0x004D5041, 0x004D5041, 0x00515445, 0x00535647, 0x00515445, 0x00505344, 0x00515445, 0x004D5544, 0x004E5645, 0x004E5645, 0x004D5544, 0x004B5342,
        0x004B5342, 0x004D5544, 0x004F5746, 0x00565949, 0x00565949, 0x00575A4A, 0x00565C4B, 0x00565C4B, 0x00535B4A, 0x00505847, 0x004E5645, 0x00555A4B, 0x0054594A, 0x00565B4C, 0x00565B4C, 0x0054594A, 0x00535849, 0x00535849, 0x00515647, 0x00505847,
        0x0050574A, 0x00515347, 0x00585546, 0x00555642, 0x00334437, 0x000F393E, 0x000A465C, 0x0012495E, 0x000F485D, 0x000C4659, 0x000B4656, 0x00114E5C, 0x001E5C66, 0x00296A73, 0x002E6F77, 0x00145367, 0x000C4B5F, 0x0006475C, 0x000A4B60, 0x000E5168,
        0x000F5269, 0x000E5168, 0x000E516A, 0x00094968, 0x0005556C, 0x000F536A, 0x00004263, 0x0048ADD4, 0x0041A4D0, 0x00106389, 0x001E7D97, 0x004EAECC, 0x00449CC1, 0x0041A1C9, 0x002C9EC7, 0x003BAAD2, 0x0022789C, 0x00175F7D, 0x000A566D, 0x00134F6D,
        0x000D536A, 0x000C5165, 0x00114E68, 0x00064C69, 0x0002596D, 0x00196D72, 0x002B6C63, 0x000D0C0E, 0x00040503, 0x001E2118, 0x00414435, 0x00494C3C, 0x00545545, 0x0058584A, 0x0049493B, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFE, 0xFFFEFDFF, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x005A5E52, 0x00444E3E, 0x000D0F09, 0x0015100F, 0x0043403C, 0x00243123, 0x00345848, 0x003D766E, 0x0023656A, 0x002E6279, 0x002F637A, 0x00255B72, 0x00215A70, 0x00245E74,
        0x001E5A70, 0x0016556A, 0x001A596E, 0x0013536C, 0x0016566F, 0x0015556E, 0x0014516B, 0x0013506A, 0x0018536D, 0x001A556F, 0x001A556F, 0x001A5674, 0x001E5875, 0x001C546F, 0x001B536E, 0x0019516C, 0x00114964, 0x00296380, 0x000F4867, 0x001E607D,
        0x001B5872, 0x00215369, 0x005E8292, 0x00CFE5F0, 0x00F3FFFF, 0x00ECEFF3, 0x00FFFFFF, 0x00F4FCF5, 0x00F9FFF9, 0x00FDFFFC, 0x00F2F5F3, 0x00FBFFFE, 0x00F2FBF8, 0x00A8B6B0, 0x0031443B, 0x0028342E, 0x00555758, 0x0038444E, 0x00184354, 0x00387891,
        0x000E5980, 0x003498CE, 0x002CB2EC, 0x003E91B1, 0x004494B1, 0x003A8699, 0x004C888E, 0x005D7979, 0x004A5357, 0x006A7D8C, 0x00507E96, 0x004E8C86, 0x0035858C, 0x003794AD, 0x003B93B7, 0x0074BBE1, 0x007BB7E1, 0x0071B6E8, 0x0056A6E1, 0x0068C1E6,
        0x0061B7DB, 0x00297696, 0x00054F69, 0x000E546C, 0x00004159, 0x00105772, 0x006BB5D1, 0x00215F71, 0x000E4C5C, 0x00095270, 0x0051AFE4, 0x003393C8, 0x00126483, 0x00327A8C, 0x0071BDD0, 0x0067BCDC, 0x0047A8D4, 0x003BABE0, 0x003CADE5, 0x0049B4E7,
        0x0058BDEA, 0x0053B6E8, 0x0053B8EF, 0x004BB1DC, 0x005BC5F4, 0x0036A2D1, 0x000E7399, 0x0032889E, 0x00448F9F, 0x000E5569, 0x00034C68, 0x00055168, 0x00216578, 0x0070BACC, 0x0061C0E1, 0x003AACE1, 0x0045B4F2, 0x00369CCC, 0x00409DBE, 0x00409BC2,
        0x0053BCE5, 0x0041B4E1, 0x0048B6E6, 0x001D7AA7, 0x00004E6F, 0x00024C5E, 0x000E575F, 0x003E6A7B, 0x005D8E9C, 0x00477F8A, 0x004A8892, 0x0060A5AE, 0x00357F8B, 0x00034D5F, 0x001E6A7D, 0x003C8DAE, 0x00388697, 0x00495E66, 0x00396A6C, 0x00268386,
        0x00488A9D, 0x00377C90, 0x004299A9, 0x0042A1B5, 0x002D95AC, 0x002091AB, 0x001E8BA7, 0x0046A3C2, 0x000A5C7B, 0x0030819C, 0x002E849C, 0x004196AC, 0x0061AEBE, 0x0011525A, 0x00205D5F, 0x0067A8B0, 0x004FA1B3, 0x0050B5D5, 0x0034A6CF, 0x0036AADF,
        0x003CAFE2, 0x0039A8DA, 0x003CABD9, 0x003CABD9, 0x003AA9D7, 0x0039A8D8, 0x0031A0D2, 0x002EA2D7, 0x002EA3D6, 0x0032A3D5, 0x0036A5D5, 0x0039A8D6, 0x0037A3D2, 0x0035A1D1, 0x003AA6D6, 0x0030A3D5, 0x0030A1D3, 0x0035A4D4, 0x0037A3D2, 0x0037A2CE,
        0x003EA6D1, 0x003399C3, 0x00147BA2, 0x00044F65, 0x000E4D62, 0x001D5168, 0x0020556A, 0x000F4F61, 0x00064B5A, 0x00084957, 0x000B4655, 0x0009333F, 0x0044473E, 0x005B5E45, 0x00485545, 0x00535149, 0x004E503D, 0x00495941, 0x004B5243, 0x00515445,
        0x00515445, 0x00505344, 0x00505344, 0x00515445, 0x00515445, 0x00525546, 0x00535647, 0x004F5445, 0x004C5142, 0x004C5142, 0x00505546, 0x00525748, 0x00505546, 0x004F5445, 0x00505546, 0x00505847, 0x00515948, 0x00525A49, 0x00525A49, 0x00515948,
        0x00515948, 0x00525A49, 0x00545C4B, 0x00575A4A, 0x00545747, 0x00515444, 0x004E5443, 0x00505645, 0x00505847, 0x00515948, 0x00515948, 0x00505546, 0x00525748, 0x00525748, 0x00525748, 0x00535849, 0x00525748, 0x004F5445, 0x004F5445, 0x00505847,
        0x004F5649, 0x00505246, 0x00575445, 0x00545541, 0x00334437, 0x00103A3F, 0x000B475D, 0x00104A60, 0x00114B61, 0x00104B5F, 0x000A485A, 0x00094757, 0x00094A58, 0x000A4B59, 0x00084A55, 0x00115064, 0x000A4C5F, 0x0008495E, 0x000D4E63, 0x000F5269,
        0x000F5269, 0x000C4F68, 0x000C4F68, 0x00125271, 0x00025067, 0x000B4B63, 0x000A4A6C, 0x002385AD, 0x002487B3, 0x003F97BC, 0x003BA0B9, 0x0042A5CB, 0x006EC6EE, 0x0050ACD5, 0x00016A91, 0x002A8EB1, 0x00085778, 0x00014562, 0x000F5B71, 0x0015516F,
        0x0010566D, 0x000F5468, 0x0015526C, 0x00094F6C, 0x00045B6F, 0x001A6E73, 0x002D6E65, 0x000C0B0D, 0x00030402, 0x001D2017, 0x003F4233, 0x00454838, 0x004D4E3E, 0x004F4F41, 0x003F3F31, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFE, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00666A5E, 0x00475141, 0x000F110B, 0x00161110, 0x00403D39, 0x00222F21, 0x00355949, 0x003D766E, 0x0024666B, 0x00295F76, 0x002C6279, 0x002A6077, 0x00225B71, 0x00225C72,
        0x00255F75, 0x001E5A70, 0x00145066, 0x001D5872, 0x001E5973, 0x001D5872, 0x001C546F, 0x001D5570, 0x001F5671, 0x001D546F, 0x0019506B, 0x00174F6C, 0x001C5471, 0x0018536D, 0x0016516B, 0x00195670, 0x00205D77, 0x0073B1C9, 0x00387890, 0x001E5574,
        0x001A4C63, 0x006A8E9E, 0x00D1E8F0, 0x00F3FFFF, 0x00FAFFFF, 0x00FDFEFF, 0x00F7F7FD, 0x00F4FCFB, 0x00F8FEFD, 0x00F6F9F7, 0x00FBFCFA, 0x00FFFFFC, 0x00CAD1CA, 0x00454D46, 0x00212E26, 0x002F3737, 0x00283648, 0x00103C59, 0x002A788F, 0x00035769,
        0x00246E88, 0x004DA2D0, 0x002899D8, 0x004195C5, 0x00137293, 0x00004B5A, 0x00165E6A, 0x004F8B9B, 0x00589EB5, 0x0052A4BD, 0x004DA1BA, 0x006BA9C1, 0x0052A2C1, 0x005FBFE7, 0x006BCEFA, 0x0070C9F5, 0x0079D0FC, 0x0056BAEA, 0x0040B1E3, 0x004CB9E5,
        0x0052BDE9, 0x0041A7D2, 0x0016729B, 0x00004B70, 0x00014765, 0x000F4D65, 0x0010475C, 0x00114F61, 0x001D5969, 0x00075370, 0x0044A4DA, 0x002C93CC, 0x002F8BB4, 0x005EB4D2, 0x0055AFD2, 0x003FA8D9, 0x0043B4EC, 0x0034AAE5, 0x0030A5DC, 0x0045B2DE,
        0x0047ABCE, 0x003F9BBE, 0x0046A1C6, 0x0049B1CE, 0x0039A2C3, 0x0040A2C4, 0x00469AB6, 0x003C8293, 0x00135462, 0x0003485C, 0x00004865, 0x00135C7A, 0x0062B5D5, 0x005FBFE7, 0x0042AEDE, 0x003EB0E5, 0x0034A1D4, 0x003495C1, 0x003D95B9, 0x0044B0DA,
        0x0041B3DC, 0x0033A5CE, 0x00238AB1, 0x00004D72, 0x00114F6D, 0x00124C62, 0x00155161, 0x008CB0B0, 0x0083ACAE, 0x006C9D9F, 0x0082BBBD, 0x00A8E8EC, 0x005DA4AE, 0x0000495C, 0x0014667F, 0x003499BF, 0x002F8B9E, 0x0053767A, 0x00346A63, 0x0061BAB7,
        0x00327D8B, 0x00005065, 0x0021839B, 0x004F99B5, 0x0081D5F1, 0x00318FAC, 0x00146E8C, 0x0052A0BD, 0x00468EA6, 0x002B778A, 0x00004B5A, 0x0014596D, 0x000B4D5F, 0x0013505E, 0x00064651, 0x00226E80, 0x004DA9C6, 0x0040B0DA, 0x002CA7D9, 0x002FA2D4,
        0x003AB0E3, 0x002FA7DB, 0x002FA7DB, 0x0030A4D9, 0x0036A6DA, 0x0034A3D3, 0x0032A5D2, 0x002AA1D9, 0x002CA3DB, 0x002FA4DB, 0x002DA1D8, 0x00289CD1, 0x002B9DD2, 0x0030A0D4, 0x002E9ED2, 0x00229FCC, 0x00219ECB, 0x0027A4D1, 0x0028A5D2, 0x0028A1D2,
        0x0030A9DA, 0x0033ACDE, 0x0028A1D3, 0x001E7488, 0x0010596D, 0x00104A60, 0x00155064, 0x00125466, 0x000D5261, 0x000F505E, 0x00154E5D, 0x00003242, 0x003C423D, 0x00585A44, 0x00455040, 0x004D4C42, 0x004A4E3B, 0x00495942, 0x004F5145, 0x004B5243,
        0x004B5243, 0x004B5243, 0x004B5243, 0x004C5344, 0x004D5445, 0x004E5546, 0x004F5647, 0x004E5848, 0x004B5545, 0x004A5444, 0x004E5848, 0x004F5949, 0x004D5747, 0x004B5545, 0x004C5646, 0x00515948, 0x00525A49, 0x00545C4B, 0x00555D4C, 0x00555D4C,
        0x00555D4C, 0x00565E4D, 0x00575F4E, 0x005B5E4E, 0x00565949, 0x00525545, 0x00505645, 0x00545A49, 0x00575F4E, 0x0058604F, 0x0058604F, 0x00535849, 0x00595E4F, 0x00575C4D, 0x00565B4C, 0x005B6051, 0x00575C4D, 0x00515647, 0x0054594A, 0x004E5645,
        0x004E5548, 0x004E5044, 0x00565344, 0x00535440, 0x00334437, 0x00113B40, 0x000D495F, 0x000A4761, 0x000E4B65, 0x00114F67, 0x000F5065, 0x000F5065, 0x000D5164, 0x000B4F62, 0x00064B5C, 0x000C4D62, 0x000B4C61, 0x000C4D62, 0x000D5067, 0x0010536A,
        0x000D5069, 0x000C4F68, 0x000B4E67, 0x000F4C6C, 0x00024C64, 0x0018546C, 0x000E4B6D, 0x002584AC, 0x001578A4, 0x000B668B, 0x002D9BB3, 0x001682AC, 0x002782AF, 0x00004A74, 0x001A759A, 0x002E82A5, 0x00014463, 0x0010526B, 0x00055469, 0x00124E6C,
        0x000E546B, 0x000E5367, 0x0016536D, 0x000C526F, 0x00065D71, 0x001D7176, 0x0032736A, 0x000E0D0F, 0x00050604, 0x001F2219, 0x00414435, 0x00464939, 0x004D4E3E, 0x004E4E40, 0x003D3D2F, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFE, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00696D61, 0x00475141, 0x000F110B, 0x0015100F, 0x003D3A36, 0x00222F21, 0x00375B4B, 0x003B746C, 0x0023656A, 0x002E647B, 0x002B6178, 0x002D637A, 0x002B647A, 0x00235C72,
        0x00215A70, 0x00265F75, 0x00265F75, 0x001C546F, 0x00225974, 0x00245B76, 0x00205772, 0x001E536E, 0x001E536E, 0x0021536F, 0x0020526E, 0x001F5671, 0x001C546F, 0x0018526F, 0x0013526E, 0x000F4F68, 0x00216179, 0x0067A6BA, 0x002C6A7C, 0x001F4A65,
        0x007093A7, 0x00CCE2ED, 0x00F6FFFF, 0x00FFFFFE, 0x00F7F5F5, 0x00FCFCFF, 0x00F6F7FF, 0x00F8FEFF, 0x00F6FBFE, 0x00FDFFFF, 0x00FFFFFC, 0x00D3D5CF, 0x004B4F49, 0x00252C25, 0x002A322B, 0x00273649, 0x00103F54, 0x002C6F86, 0x00498497, 0x006EABB5,
        0x004D98A1, 0x00156174, 0x005491B3, 0x004295C2, 0x0016667F, 0x0029646D, 0x0073A4AE, 0x005AA6BC, 0x002FA4BF, 0x0043B1C9, 0x0074BAD1, 0x007CB8D5, 0x0062AECB, 0x003F9AB9, 0x0043A6C6, 0x003C9DBE, 0x0041A5C7, 0x002DA1C0, 0x0037BADB, 0x004EC7F9,
        0x0032ACE0, 0x0042B9F0, 0x0043B3E8, 0x000F6F9D, 0x00004C70, 0x0018516A, 0x001A4859, 0x00104E60, 0x00104C5C, 0x000E5979, 0x0047ABE2, 0x0033A0DE, 0x0040A6D7, 0x0050B4DE, 0x0039A5D5, 0x0034AAE5, 0x0040B4ED, 0x0043B3E7, 0x002D95C0, 0x00005171,
        0x000C556B, 0x005893A3, 0x00528693, 0x002B767F, 0x0029707E, 0x00518FA1, 0x005A8C98, 0x005F898E, 0x00336367, 0x00195A69, 0x000B5B74, 0x0065B0D6, 0x0064C6F4, 0x0036AAE1, 0x0042B3EC, 0x0038A3D6, 0x00218AB3, 0x0047A7CB, 0x004594BB, 0x003099CA,
        0x001585B0, 0x002D98BE, 0x00137091, 0x00074D6B, 0x00164D68, 0x00184E65, 0x000C485E, 0x0092B3A5, 0x004B716B, 0x0050817F, 0x00447A7B, 0x00447D7E, 0x00387981, 0x00024F62, 0x000C6280, 0x00309DC9, 0x001C7A91, 0x00688E90, 0x003E6E62, 0x0090D6D0,
        0x002B737F, 0x00167C93, 0x002F8EAF, 0x005DA8C2, 0x007BD1E9, 0x0027839C, 0x003890A8, 0x0054A0B6, 0x00438B9C, 0x00297881, 0x00025B5F, 0x00134B5E, 0x000D475A, 0x000E4D61, 0x001D667C, 0x00499FBD, 0x004DB6DF, 0x002FA5D9, 0x002EAEE9, 0x003CACDA,
        0x003AB3E4, 0x0028A8DD, 0x002BA9E0, 0x0033A7DE, 0x0038A3D6, 0x00309BC7, 0x0037A6CC, 0x003DA6CF, 0x00339CC7, 0x002F9DCD, 0x0033A3D7, 0x002DA1D6, 0x002A9FD6, 0x002A9FD6, 0x00269BD2, 0x0029A2CE, 0x00249FCB, 0x0025A2CF, 0x0024A1D3, 0x001E9CD2,
        0x0021A0D7, 0x0026A7E0, 0x0024A6E1, 0x003BA2BD, 0x001B7593, 0x00085471, 0x0008516D, 0x00085871, 0x00085A73, 0x000B556D, 0x000C4E67, 0x00074458, 0x003A4543, 0x00545541, 0x00444E3E, 0x004D4C42, 0x00484E3B, 0x00475740, 0x00515046, 0x00495343,
        0x00495343, 0x00495343, 0x004A5444, 0x004B5545, 0x004C5646, 0x004D5747, 0x004E5848, 0x004E5949, 0x004B5646, 0x004B5646, 0x004E5949, 0x004F5A4A, 0x004C5747, 0x004B5646, 0x004B5646, 0x00505847, 0x00515948, 0x00525A49, 0x00545C4B, 0x00545C4B,
        0x00545C4B, 0x00535B4A, 0x00535B4A, 0x00545747, 0x00525545, 0x00505343, 0x00515746, 0x00575D4C, 0x0058604F, 0x00565E4D, 0x00535B4A, 0x00515647, 0x005A5F50, 0x00565B4C, 0x0054594A, 0x005A5F50, 0x00555A4B, 0x004C5142, 0x00515647, 0x004D5544,
        0x004C5346, 0x004C4E42, 0x00545142, 0x0052533F, 0x00324336, 0x00123C41, 0x000E4A60, 0x00084965, 0x000B4C68, 0x000C4D69, 0x000A4D66, 0x000A4D66, 0x000C4F68, 0x000A5068, 0x00074D65, 0x000B4C61, 0x000C4D62, 0x000C4F66, 0x000E5168, 0x000D5069,
        0x000C4F68, 0x000B4E67, 0x00094F67, 0x000E4B6B, 0x000F576F, 0x001A526B, 0x00094164, 0x002D89B2, 0x000F729E, 0x00004064, 0x002999B0, 0x004FBEEC, 0x000B6594, 0x00004974, 0x0018698E, 0x00438EAE, 0x000C4865, 0x00135169, 0x00045164, 0x00104C6A,
        0x000D536A, 0x000D5266, 0x0015526C, 0x000C526F, 0x00045B6F, 0x00196D72, 0x0032736A, 0x000D0C0E, 0x00050604, 0x001E2118, 0x00404334, 0x00474A3A, 0x004F5040, 0x00525244, 0x00424234, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFE, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x005D6155, 0x00444E3E, 0x000D0F09, 0x0015100F, 0x003B3834, 0x00233022, 0x003A5E4E, 0x0039726A, 0x00216368, 0x002C657B, 0x00215A70, 0x00245D73, 0x002E647B, 0x00295F76,
        0x00255B72, 0x00265C73, 0x0022586F, 0x001E5570, 0x001E5570, 0x001C536E, 0x001C516C, 0x001F546F, 0x00245672, 0x00235571, 0x001E506C, 0x001F556C, 0x001A526D, 0x0014506D, 0x00185674, 0x0012526B, 0x001B556B, 0x00215A69, 0x001C525D, 0x00728FA4,
        0x00D4E9F8, 0x00F3FFFF, 0x00F4F5F1, 0x00FFFFFB, 0x00FFFFFE, 0x00FBFCFF, 0x00F7F9FF, 0x00F2F8FF, 0x00F3F8FB, 0x00FBFFFE, 0x00E0E2DC, 0x00545950, 0x00232A23, 0x001D2623, 0x00384341, 0x00023052, 0x00255D6E, 0x00548387, 0x0093AFB0, 0x009DBCBF,
        0x00538585, 0x001B483F, 0x002C4335, 0x00265973, 0x005D8C94, 0x00809995, 0x00B8C1C4, 0x0090AEBF, 0x004D8DA6, 0x006595AD, 0x00A8AFC3, 0x007AADA6, 0x004C8882, 0x000B524F, 0x00085258, 0x004990A4, 0x00418CAC, 0x0049A1C6, 0x004DAFD7, 0x003AB3E5,
        0x0038B3EB, 0x0029A9E4, 0x0035B1ED, 0x0044B1E7, 0x00167099, 0x00003F5C, 0x0021566B, 0x00175468, 0x00094759, 0x000A5777, 0x0040A5DC, 0x002D9EDD, 0x0039A6DA, 0x003BAADC, 0x0038AEE9, 0x0037AEE6, 0x0038A7D9, 0x002A8BB1, 0x00095873, 0x00064252,
        0x005E838B, 0x00BBCDCE, 0x00C6CCC7, 0x00B5D1CB, 0x008EA9A6, 0x00BDD1D2, 0x00B6C4C2, 0x00D4E4DD, 0x007FA29E, 0x001E5B67, 0x006ABED7, 0x0064B9E9, 0x0041B0E2, 0x0043BAF1, 0x00288EC3, 0x00207DAA, 0x00369DBE, 0x002281A2, 0x001B668C, 0x002E8DBF,
        0x002588B4, 0x00298AAE, 0x00176D8B, 0x00054861, 0x0019556D, 0x004B8A9F, 0x004D94A8, 0x00749B8C, 0x00568380, 0x0052868C, 0x00376E71, 0x00609696, 0x00468385, 0x00004355, 0x000C6685, 0x00309CCC, 0x00247B95, 0x00AFD0D3, 0x0085A79C, 0x0091BEBB,
        0x005796A4, 0x002087A0, 0x004C9DC3, 0x0021909A, 0x001E919B, 0x0032A0AC, 0x0022828F, 0x003B8591, 0x00438189, 0x008FCFD0, 0x001B625F, 0x00215566, 0x000F485D, 0x00084E66, 0x004598B8, 0x005ABCE4, 0x0036A5D7, 0x002CA5DD, 0x0025A4E1, 0x0046B2E1,
        0x0031AADB, 0x0025A6D7, 0x002FAADC, 0x0038A1D2, 0x002583AE, 0x000B6889, 0x00046783, 0x001A6C7D, 0x000C647C, 0x00177899, 0x00319AC5, 0x0035A6D8, 0x002BA1D5, 0x00289FD6, 0x002BA2D9, 0x0035A6D1, 0x002EA1CE, 0x002EA1D3, 0x002FA3D8, 0x002AA1D9,
        0x0027A1DD, 0x0029A4E2, 0x002BA6E6, 0x003CB5DF, 0x00319ECA, 0x002E8CB9, 0x002D88B5, 0x002688B2, 0x00258AB1, 0x002C89B0, 0x003085AD, 0x001C637E, 0x003E4D50, 0x00555543, 0x00495140, 0x00515145, 0x00464F3B, 0x0041503B, 0x00514B44, 0x004A5444,
        0x004A5444, 0x004A5444, 0x004A5444, 0x004B5545, 0x004D5747, 0x004E5848, 0x004F5949, 0x004E5949, 0x004B5646, 0x004A5545, 0x004D5848, 0x004E5949, 0x004C5747, 0x004B5646, 0x004B5646, 0x00505847, 0x00505847, 0x00515948, 0x00525A49, 0x00525A49,
        0x00525A49, 0x00505847, 0x004F5746, 0x00535646, 0x00515444, 0x00515444, 0x00535948, 0x00595F4E, 0x0058604F, 0x00555D4C, 0x00515948, 0x00535849, 0x005C6152, 0x00575C4D, 0x0054594A, 0x005A5F50, 0x00535849, 0x004A4F40, 0x004F5445, 0x004B5342,
        0x004A5144, 0x004B4D41, 0x00535041, 0x0051523E, 0x00324336, 0x00123C41, 0x000E4A60, 0x000A4B67, 0x000B4C68, 0x00094C67, 0x00064964, 0x00054863, 0x00064A67, 0x00074B68, 0x00054966, 0x00094C63, 0x000B4E65, 0x000C4F66, 0x000B4E67, 0x000A4D66,
        0x00094F67, 0x00094E69, 0x00094E69, 0x000D4868, 0x00125A72, 0x000F4861, 0x000B4568, 0x001A79A1, 0x001C7FAB, 0x00217A9F, 0x002A94AC, 0x000F78A1, 0x004499C6, 0x00075580, 0x001D6E94, 0x002C799A, 0x00185472, 0x00125068, 0x000C586A, 0x0014506E,
        0x0011576E, 0x000F5468, 0x0015526C, 0x000D5370, 0x0002596D, 0x00166A6F, 0x0032736A, 0x000C0B0D, 0x00020301, 0x001A1D14, 0x003B3E2F, 0x00424535, 0x004C4D3D, 0x00505042, 0x00414133, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFE, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x005C6054, 0x00434D3D, 0x000C0E08, 0x00161110, 0x003A3733, 0x00253224, 0x003E6252, 0x00387169, 0x00206267, 0x00255E74, 0x0020596F, 0x00235C72, 0x00285E75, 0x00285C73,
        0x002A5E75, 0x002D5F76, 0x0026586F, 0x001F5772, 0x001E5671, 0x001C536E, 0x0019506B, 0x001D526D, 0x00205570, 0x001E536E, 0x001C4E6A, 0x00124B60, 0x001A536C, 0x0015516E, 0x00175573, 0x001D5572, 0x001D4F65, 0x0025515E, 0x0079A0A8, 0x00DAE9F2,
        0x00F0FCFF, 0x00F8FEFD, 0x00FDFCF8, 0x00FEFBF6, 0x00FFFDFC, 0x00FDFFFF, 0x00F1F5FA, 0x00FAFEFF, 0x00F7FDFC, 0x00D8DFD8, 0x00565E53, 0x001F281E, 0x00202E2A, 0x001F3033, 0x0023363D, 0x0000344F, 0x004F727C, 0x00A3AEA6, 0x008EA093, 0x00A1B6AE,
        0x007E8580, 0x005A5B47, 0x005D6842, 0x004D4A4C, 0x00829189, 0x00A0B7A9, 0x0096A39B, 0x0096A1A5, 0x0099B1BD, 0x006D909D, 0x006F969F, 0x005B90B1, 0x00447F99, 0x004B8DA0, 0x0070B2CB, 0x0085C7EA, 0x007BC2EE, 0x005CAFDC, 0x00237FAA, 0x000F79A8,
        0x0040B0E4, 0x0039B4EC, 0x001F9FDA, 0x0039B2EA, 0x0048B1E2, 0x00237BA0, 0x00004868, 0x000A4A62, 0x00115064, 0x00085575, 0x003399CE, 0x002899D7, 0x003FAFE3, 0x0044B7EA, 0x0027A4E1, 0x002FA0D9, 0x00359BC6, 0x00176A86, 0x00004052, 0x006996A3,
        0x00CAE4EA, 0x00BDC2C0, 0x00C6BDB3, 0x00D9E0D3, 0x00C1C6BD, 0x00E5E9E4, 0x00B2B6B0, 0x00A0ADA5, 0x007CA0A0, 0x00478C9F, 0x0065C3E6, 0x0044A8E2, 0x0040B7E8, 0x002A99C1, 0x00227095, 0x00357FA3, 0x002E91B1, 0x00117492, 0x000B5273, 0x003286B6,
        0x00308CB7, 0x002A88AC, 0x00196F8D, 0x00105870, 0x0063A4B9, 0x00458D9F, 0x001C6D7C, 0x00245956, 0x0045838B, 0x00357789, 0x002B6773, 0x0078AEAF, 0x005F9798, 0x001D6674, 0x000A6482, 0x00339BCA, 0x00267A94, 0x00426267, 0x0036534A, 0x004E6C6D,
        0x003E7989, 0x00319BB3, 0x0033769D, 0x008AC1C4, 0x00599398, 0x0088C2C8, 0x0068979F, 0x0081A2AB, 0x00617F84, 0x00ADD4D2, 0x002A5D56, 0x00144C5D, 0x0006475C, 0x003B8BA8, 0x004FAED5, 0x002B97C6, 0x0032A6DB, 0x0034ABE3, 0x002BA1DC, 0x003DA6D9,
        0x002AA0CF, 0x0034B1DD, 0x0030A1C9, 0x002C83A5, 0x00115572, 0x00044E60, 0x0000515D, 0x00044C56, 0x001F6F80, 0x003995B2, 0x003BA4CB, 0x00309FCF, 0x002A9FD2, 0x002AA0D3, 0x00279ECF, 0x00309CCB, 0x002B99C9, 0x002B9ACC, 0x0030A0D5, 0x0032A3DB,
        0x0031A3DF, 0x0031A4E1, 0x0031A3E2, 0x001DA1D6, 0x00269DD4, 0x00349ED4, 0x00359FD4, 0x002E9FD1, 0x002EA2D1, 0x0035A4D4, 0x0038A0CF, 0x00227493, 0x003B4E55, 0x00555243, 0x004C5241, 0x00525546, 0x0047533D, 0x0042513C, 0x00564D49, 0x004C5344,
        0x004C5344, 0x004C5344, 0x004C5344, 0x004D5445, 0x004E5546, 0x004F5647, 0x00505748, 0x004D5747, 0x004A5444, 0x004A5444, 0x004D5747, 0x004E5848, 0x004C5646, 0x004B5545, 0x004B5545, 0x00525A49, 0x00515948, 0x00505847, 0x00515948, 0x00515948,
        0x00515948, 0x00505847, 0x004F5746, 0x00595C4C, 0x00555848, 0x00535646, 0x00525847, 0x00565C4B, 0x00565E4D, 0x00555D4C, 0x00535B4A, 0x00565B4C, 0x005B6051, 0x00585D4E, 0x00565B4C, 0x00595E4F, 0x00535849, 0x004C5142, 0x004F5445, 0x00495140,
        0x00495043, 0x004A4C40, 0x00535041, 0x0051523E, 0x00324336, 0x00113B40, 0x000D495F, 0x000B4B63, 0x000D4D65, 0x000C4E67, 0x000A4B67, 0x000A4D68, 0x000C4E6B, 0x000C4E6B, 0x00084B6A, 0x000A4D64, 0x000B4E65, 0x000A4D66, 0x00084B64, 0x00064C64,
        0x00084D68, 0x00094E69, 0x00074E69, 0x000F4868, 0x00034D65, 0x000C4C64, 0x00014465, 0x000D6F97, 0x0041A2CE, 0x003786AD, 0x0005607B, 0x00004865, 0x00247296, 0x004F9EC7, 0x00318DB6, 0x0020779F, 0x00064A6D, 0x000C4963, 0x00064E60, 0x00124E6C,
        0x0010566D, 0x000E5367, 0x0016536D, 0x00105673, 0x00065D71, 0x001C7075, 0x003C7D74, 0x00111012, 0x00060705, 0x001D2017, 0x003C3F30, 0x00414434, 0x004B4C3C, 0x004F4F41, 0x00404032, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFE, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00616559, 0x00434D3D, 0x000D0F09, 0x00161110, 0x00393632, 0x00273426, 0x00436757, 0x003A736B, 0x0023656A, 0x00266076, 0x00276177, 0x00286177, 0x00265C73, 0x0022566D,
        0x0025576E, 0x00285870, 0x0026566E, 0x001C5771, 0x00205B75, 0x00205B75, 0x001D5570, 0x0019516C, 0x001A516C, 0x001B526D, 0x001A516C, 0x00195366, 0x0019556D, 0x00175370, 0x00104B6B, 0x00184C6A, 0x00285167, 0x006E8C97, 0x00CFE5EA, 0x00F8FDFC,
        0x00FBFFFE, 0x00F8FBF9, 0x00FAFDFB, 0x00FDFFFE, 0x00F5F8F6, 0x00F2F7F6, 0x00FBFFFF, 0x00F7FCFB, 0x00DFE5E0, 0x00565E53, 0x001D2A1C, 0x0018291E, 0x001C3230, 0x00112C36, 0x00173344, 0x00588289, 0x00A3B6B9, 0x0086908A, 0x00A3B5A8, 0x00707D6F,
        0x00564D43, 0x0065584A, 0x00676553, 0x0077644F, 0x00585142, 0x0088988D, 0x008FAFA4, 0x0078968B, 0x0091AFAA, 0x0080B8B9, 0x005DB5BB, 0x0068B4FC, 0x0065B5EC, 0x007BCBF4, 0x005CADD0, 0x002D7EA4, 0x0057ADD7, 0x003798BA, 0x002A91AA, 0x000D638D,
        0x00197AA6, 0x003FB0E2, 0x0042C0F6, 0x0029ABE0, 0x003DB7EB, 0x003FAEDE, 0x0007709C, 0x000E516C, 0x0008495E, 0x000C5979, 0x0043A8DC, 0x0030A1DA, 0x0039A8DA, 0x003BB1E2, 0x002EAEE9, 0x003FA9E8, 0x002B8BB9, 0x00156681, 0x000A4F60, 0x00458090,
        0x008DBAC7, 0x008DABB0, 0x00859692, 0x0097A9A2, 0x00879C99, 0x007C9192, 0x00627778, 0x00648384, 0x00588F9C, 0x00489EC2, 0x003CA8DE, 0x0040B1F4, 0x0029A3CB, 0x00288699, 0x0085B5C7, 0x0077ADCB, 0x001F82A2, 0x002088A5, 0x000A516C, 0x003B90BD,
        0x002685AD, 0x002F90B4, 0x000E6988, 0x001D6783, 0x0084C7DC, 0x002B7280, 0x0000434F, 0x0008505C, 0x004895AF, 0x0063B2D3, 0x00468AA1, 0x005D9499, 0x006FA4A1, 0x006AB1BB, 0x00126983, 0x00319ECA, 0x003189A1, 0x00517B80, 0x0030524C, 0x00587278,
        0x004B8B9D, 0x00279BB2, 0x00306F95, 0x00837E7B, 0x00676A68, 0x007D8585, 0x00686F72, 0x007E8386, 0x005A6666, 0x006C8C87, 0x0026564A, 0x000A4B5A, 0x004590A6, 0x004DA7C9, 0x00339CC7, 0x002295C7, 0x0030A4D9, 0x0036A6DA, 0x003CA9DD, 0x003DA4DD,
        0x002397C6, 0x0038B0D4, 0x002687A1, 0x002C6C7E, 0x0025535E, 0x0033686B, 0x00195E5B, 0x00166A83, 0x00449EBD, 0x0055B9E3, 0x003AA9DB, 0x00269ACF, 0x002CA2D5, 0x0033A8D5, 0x0031A3CC, 0x002FA1D0, 0x002D9FCE, 0x002B9ED0, 0x002FA2D5, 0x0032A4D9,
        0x002FA3DA, 0x002EA2D9, 0x002DA1D8, 0x0020A2D7, 0x00269BD2, 0x00309AD0, 0x002E9BCF, 0x00259BCC, 0x00219ECB, 0x00259DCB, 0x002598C5, 0x00187094, 0x00334954, 0x00535042, 0x004C4F3F, 0x00515445, 0x0048553F, 0x00455341, 0x005A4E4C, 0x004F5243,
        0x004F5243, 0x004E5142, 0x004E5142, 0x004F5243, 0x00505344, 0x00505344, 0x00515445, 0x004E5344, 0x004B5041, 0x004B5041, 0x004F5445, 0x00505546, 0x004F5445, 0x004E5344, 0x004F5445, 0x00505847, 0x004F5746, 0x004D5544, 0x004E5645, 0x004F5746,
        0x00505847, 0x00505847, 0x004F5746, 0x00545747, 0x00515444, 0x004E5141, 0x004B5140, 0x004C5241, 0x004B5342, 0x004B5342, 0x004A5241, 0x004F5445, 0x00505546, 0x00505546, 0x004F5445, 0x004F5445, 0x004C5142, 0x00494E3F, 0x00484D3E, 0x0048503F,
        0x00484F42, 0x004A4C40, 0x00535041, 0x0051523E, 0x00314235, 0x00103A3F, 0x000B475D, 0x000C4B5F, 0x000F4E63, 0x00104F64, 0x000C4C64, 0x000B4B64, 0x000A4B67, 0x00084866, 0x00044462, 0x00094C63, 0x000A4D66, 0x00094C65, 0x00064962, 0x00054A65,
        0x00084D68, 0x00064C69, 0x00044A67, 0x00164F6F, 0x00004B62, 0x00084E65, 0x00004969, 0x001A83AA, 0x0048A6D3, 0x0010557D, 0x0000415F, 0x00004D5A, 0x00064D68, 0x0016698F, 0x004DB6E2, 0x0009709D, 0x00003C63, 0x00175874, 0x000C5162, 0x000F4B69,
        0x000E546B, 0x000B5064, 0x0014516B, 0x00105673, 0x00075E72, 0x001E7277, 0x00418279, 0x00121113, 0x00080907, 0x001F2219, 0x003E4132, 0x00424535, 0x004B4C3C, 0x004E4E40, 0x003F3F31, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFE, 0xFFFDFCFE, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x005C6054, 0x00454F3F, 0x000E100A, 0x00171211, 0x00393632, 0x00283527, 0x00456959, 0x003C756D, 0x0026686D, 0x00245E74, 0x00205A70, 0x001F586E, 0x00245A71, 0x00265A71,
        0x0023556C, 0x00215169, 0x0024526A, 0x0015526C, 0x0017546E, 0x0016536D, 0x0016516B, 0x0016516B, 0x0019516C, 0x0018506B, 0x00154D68, 0x001B5669, 0x000B475D, 0x00104E6C, 0x000F4868, 0x00235271, 0x00496B82, 0x00BED2DD, 0x00E2EEF2, 0x00FFFEF6,
        0x00FEFEF8, 0x00FAFEF9, 0x00F5FAF9, 0x00F5FBFA, 0x00FAFFFE, 0x00FBFFFC, 0x00FDFFFC, 0x00E7EBE6, 0x0050554C, 0x00222B1E, 0x001E2E1D, 0x00243A2E, 0x00223E3E, 0x0001212E, 0x0041637A, 0x00CFC6BC, 0x007A8F8C, 0x0094B6B6, 0x00717E7C, 0x00565547,
        0x005F5F4F, 0x005C5756, 0x00635061, 0x00696F4C, 0x006F6252, 0x00544A4A, 0x007D918C, 0x0096B3A4, 0x008B9B90, 0x00ADC0C7, 0x0081AEC3, 0x0029889C, 0x004CAAB0, 0x006EC8C3, 0x000A5E5C, 0x002F7E8B, 0x0058A8BF, 0x0000546A, 0x000A6272, 0x00549BC1,
        0x0010658D, 0x001179A4, 0x003AB3E4, 0x0032B6EB, 0x0022A8DE, 0x0032B1E8, 0x0039B4EC, 0x00286C89, 0x0004475E, 0x000F5C7C, 0x003EA1D3, 0x002C9BD3, 0x0043B3E1, 0x003AAEDD, 0x0030B1EA, 0x00369EE3, 0x003291C3, 0x000F6180, 0x00075369, 0x00004C62,
        0x0000465A, 0x0014505C, 0x00013437, 0x00001E1D, 0x00002124, 0x0000272E, 0x0000262E, 0x002D616D, 0x00569DB8, 0x003C9FD3, 0x00259DE5, 0x002FABF2, 0x00229EC0, 0x0055A6A9, 0x00759398, 0x005D879E, 0x002E90B2, 0x00087591, 0x0012586F, 0x003794BB,
        0x002689AF, 0x003098BB, 0x00076687, 0x00004E6B, 0x001F6277, 0x00084957, 0x000F545D, 0x00156982, 0x0058B1D6, 0x0051A9D7, 0x00236C8A, 0x005C9299, 0x00417470, 0x00195D64, 0x00146A82, 0x0033A5CD, 0x0044A2B9, 0x0052868C, 0x003B6361, 0x005D7C85,
        0x00539BAD, 0x00209DB1, 0x002D6D8F, 0x005C7C69, 0x00406453, 0x004A6E62, 0x003B5751, 0x005D706D, 0x00475D58, 0x004A6F65, 0x001E5143, 0x00428899, 0x0063B6CC, 0x0041A3C5, 0x002596C1, 0x00269CCF, 0x0032A7DA, 0x0039A5D5, 0x0042A8D8, 0x003CA2DD,
        0x001688B7, 0x0038A9C9, 0x003991A2, 0x0072A6AD, 0x008AA9AA, 0x0086AFA8, 0x00356F64, 0x0046A8D2, 0x004CB3E4, 0x003FAEE6, 0x0028A0DC, 0x00229DD7, 0x00289ED1, 0x002D9FC8, 0x0034A2C6, 0x00239CCD, 0x00259ECF, 0x00269FD0, 0x0028A1D2, 0x002EA4D7,
        0x002FA5D8, 0x002FA5D8, 0x0030A6D9, 0x002AA6D6, 0x002E9DCF, 0x00369DCE, 0x0036A0CF, 0x00289EC8, 0x00229EC7, 0x0027A1C9, 0x002DA2C9, 0x00106E92, 0x00304A56, 0x00565345, 0x004A4D3D, 0x004C4F40, 0x0046533D, 0x0042503E, 0x00564947, 0x00505042,
        0x00505042, 0x004F4F41, 0x004F4F41, 0x004F4F41, 0x004F4F41, 0x00505042, 0x00505042, 0x004E5142, 0x004B4E3F, 0x004C4F40, 0x004F5243, 0x00515445, 0x00505344, 0x004F5243, 0x00505344, 0x004C5443, 0x004B5342, 0x00495140, 0x004A5241, 0x004C5443,
        0x004E5645, 0x004E5645, 0x004E5645, 0x00535646, 0x00525545, 0x00505343, 0x004D5342, 0x004C5241, 0x00495140, 0x0048503F, 0x00464E3D, 0x004E5344, 0x004D5243, 0x004E5344, 0x004F5445, 0x004D5243, 0x004C5142, 0x004C5142, 0x004A4F40, 0x00474F3E,
        0x00484F42, 0x004A4C40, 0x00535041, 0x0052533F, 0x00314235, 0x000F393E, 0x000A465C, 0x000D4B5D, 0x00104E60, 0x00104D61, 0x000D4C61, 0x000E4B65, 0x000E4D69, 0x000F4E6A, 0x000D4B69, 0x00074A63, 0x00094C65, 0x00084B64, 0x00044A62, 0x00054A65,
        0x00064D68, 0x00064C69, 0x00024865, 0x00114768, 0x00035168, 0x0000485E, 0x00207291, 0x003BA8CE, 0x002C8AB7, 0x0006446D, 0x00124F6F, 0x000F5358, 0x0005495C, 0x00005075, 0x002FA1D0, 0x002C9DCF, 0x00005380, 0x00084A67, 0x000F5163, 0x00134F6D,
        0x0011576E, 0x000C5165, 0x00124F69, 0x000C526F, 0x0000576B, 0x00166A6F, 0x003A7B72, 0x000C0B0D, 0x00020301, 0x001A1D14, 0x003B3E2F, 0x003F4232, 0x00474838, 0x004A4A3C, 0x003A3A2C, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFE, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x0062665A, 0x00485545, 0x00090B05, 0x001B1313, 0x003C3734, 0x00273124, 0x00345949, 0x00357169, 0x001D676B, 0x00235C72, 0x00225B71, 0x00215A70, 0x0020596F, 0x0020596F,
        0x001E576D, 0x001C556B, 0x001A5369, 0x00245075, 0x001B5172, 0x000E516C, 0x0011546B, 0x00144D66, 0x001F546F, 0x00164D6C, 0x00115170, 0x00194C7E, 0x0049799B, 0x006795A6, 0x0073A2AA, 0x00608992, 0x006F8590, 0x00E6E2E8, 0x00FFFCFC, 0x00EEFCF8,
        0x00EBF9F7, 0x00EEFCFA, 0x00EBFBFA, 0x00F2FFFE, 0x00F2FFFA, 0x00F8FFFB, 0x00D9E2D8, 0x005E5E46, 0x00192A1D, 0x00132C30, 0x00193147, 0x00083148, 0x00215A69, 0x00638F96, 0x00A7B3B5, 0x00829189, 0x00A5B2AA, 0x00737C72, 0x00474D42, 0x00606054,
        0x00625C51, 0x005A5144, 0x006E6355, 0x005E5C4A, 0x00696455, 0x006B6356, 0x00585247, 0x0087887F, 0x00A7B2AA, 0x008BA098, 0x00A7C3BD, 0x0059958A, 0x007BA7A8, 0x0072A3B3, 0x00247289, 0x003897B1, 0x00004D66, 0x00084658, 0x00174E5B, 0x0007577C,
        0x004C99B9, 0x001D6D8A, 0x00137097, 0x003FADE1, 0x0033ADE9, 0x002EABE5, 0x0036B0E4, 0x001E8DB5, 0x000B4562, 0x00155770, 0x003DABD5, 0x00279BD4, 0x0039ADE2, 0x0035B0E2, 0x0037ABE6, 0x0031A7D8, 0x002A9DC8, 0x00165881, 0x00224B6C, 0x0007485D,
        0x001A5568, 0x00164154, 0x000B4C5B, 0x0014475B, 0x000F485D, 0x00004058, 0x00034D69, 0x003990B0, 0x0042A3C7, 0x00319AC1, 0x00309FC7, 0x002C99C5, 0x00389BC1, 0x004194B3, 0x00458DA5, 0x004C8FA8, 0x00468FAD, 0x0020769A, 0x0000557D, 0x00308DB4,
        0x002D88AD, 0x00348AAE, 0x001B6C8F, 0x00024D6D, 0x00034665, 0x000C4D69, 0x002D6986, 0x005FB3D7, 0x004BA1BF, 0x0010677B, 0x00005261, 0x00388B9A, 0x0018647A, 0x001E6586, 0x004285AC, 0x002D91A9, 0x0051AEC7, 0x00277B97, 0x005BADCC, 0x000F6282,
        0x000C688B, 0x0038A1C2, 0x001B8CAD, 0x0070A3A6, 0x004C7E84, 0x00619399, 0x00508288, 0x0064959D, 0x00396A72, 0x006798A2, 0x004A7B85, 0x003EA7E0, 0x002C95CE, 0x002790C9, 0x00329DD6, 0x003AA5DE, 0x00319ED6, 0x002895CD, 0x002794CC, 0x00329FD2,
        0x00228AB9, 0x003E9BC8, 0x00368BB3, 0x00115F83, 0x0014627F, 0x002E8099, 0x004B9EB4, 0x0036A1DA, 0x0035A0D9, 0x002590C9, 0x00228DC6, 0x001782BB, 0x00137EB7, 0x002994CD, 0x002994CD, 0x0037A5CF, 0x0032A2D0, 0x002BA2D3, 0x0026A3D6, 0x0027A3D9,
        0x002FA7DB, 0x0033A6D9, 0x00319ED1, 0x0024A1CE, 0x002497C9, 0x002B97CD, 0x002C9CD1, 0x00219ACB, 0x001D9EC5, 0x002AA7C6, 0x0035AAC5, 0x002C7185, 0x003C4C4B, 0x004F513D, 0x004A5642, 0x00505344, 0x00465039, 0x00465940, 0x004F4E40, 0x004B5041,
        0x004B5041, 0x004C5142, 0x004D5243, 0x004E5344, 0x004E5344, 0x004B5041, 0x00494E3F, 0x00474E3F, 0x00484F40, 0x00495041, 0x004A5142, 0x004B5243, 0x004B5243, 0x004A5142, 0x004A5142, 0x00505645, 0x004E5443, 0x004B5140, 0x004A503F, 0x004C5241,
        0x004E5443, 0x00505645, 0x00515746, 0x00525242, 0x00515141, 0x00505040, 0x00505040, 0x00505040, 0x00505040, 0x00515141, 0x00525242, 0x00474E3F, 0x00495041, 0x004A5142, 0x004D5243, 0x004B5041, 0x004C4F40, 0x004C4F40, 0x004C4F40, 0x00514C3D,
        0x00484C40, 0x00444E41, 0x00494C3C, 0x00524C39, 0x003D4136, 0x0017373D, 0x000D475D, 0x00094962, 0x000A4A63, 0x000B4B64, 0x000B4D66, 0x000B4D66, 0x000B4D66, 0x00094C65, 0x00094C65, 0x00074A65, 0x00084B66, 0x00094C67, 0x000A4D68, 0x000A4D68,
        0x000A4D68, 0x000A4D68, 0x00094C67, 0x00005069, 0x00004661, 0x002A789D, 0x00498EB9, 0x001D6C97, 0x002D8DB1, 0x00004868, 0x000B4666, 0x00004658, 0x0003627C, 0x003AB0D9, 0x003DBCEF, 0x003DAEE6, 0x004099CB, 0x00085177, 0x000A4D68, 0x000F4F6D,
        0x00095268, 0x000E5367, 0x0013506A, 0x000A506D, 0x0002596D, 0x0016686D, 0x0037766D, 0x000F0E10, 0x00020301, 0x001D2017, 0x003A3D2E, 0x003E4131, 0x00494A3A, 0x00474739, 0x003A3A2C, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFE, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00606458, 0x00455242, 0x000C0E08, 0x001E1616, 0x003C3734, 0x00252F22, 0x00315646, 0x00306C64, 0x00176165, 0x00245D73, 0x00235C72, 0x00215A70, 0x0020596F, 0x0020596F,
        0x001E576D, 0x001B546A, 0x00195268, 0x00224F71, 0x001D5473, 0x0011546D, 0x0014576E, 0x00175069, 0x001C516C, 0x00104968, 0x00175877, 0x004A94C4, 0x0073C3E0, 0x0079C3CF, 0x006B9FA5, 0x00687C87, 0x009598A6, 0x00EFFAFE, 0x00E5FDF5, 0x00F5FFF9,
        0x00ECF7F4, 0x00F6FFFE, 0x00F4FFFC, 0x00F1FCF9, 0x00F8FFFC, 0x00D0D7D0, 0x004A5248, 0x000F2512, 0x00232721, 0x00313A43, 0x000A3140, 0x00235768, 0x00587B89, 0x00A5BBC0, 0x007F9692, 0x009AA79F, 0x00758076, 0x0050594F, 0x004B5146, 0x00505246,
        0x005B5A4C, 0x006A6556, 0x005F5A4B, 0x00676553, 0x005A5846, 0x00635E4F, 0x006B655A, 0x005A594F, 0x00788076, 0x00A7B6AE, 0x0090A59D, 0x00B4C1BF, 0x0080B9B6, 0x0043979D, 0x004D92A5, 0x000B4A66, 0x00014E68, 0x0004556A, 0x000A495E, 0x00014C6C,
        0x001F6884, 0x004491AB, 0x00197396, 0x00147FB2, 0x0037AFEB, 0x002BA6E0, 0x003AB4EA, 0x001F8EB6, 0x000B4562, 0x00165871, 0x003EACD6, 0x00279BD4, 0x0039ADE2, 0x0035B0E2, 0x0037ABE6, 0x002CA0D5, 0x002796C4, 0x0013698D, 0x0000435B, 0x00004E5D,
        0x00004E59, 0x00125060, 0x00114E62, 0x00196064, 0x002F797F, 0x002D7B82, 0x00297B86, 0x004CA4B2, 0x004EAABB, 0x0048A7BA, 0x004BACC0, 0x004BA7C0, 0x0053A9BD, 0x0059A3AF, 0x005C9DA5, 0x0062A2A7, 0x0060A2AD, 0x004690A2, 0x0029778E, 0x004EACBF,
        0x004CA8B9, 0x0052AABA, 0x003C8F9E, 0x00277782, 0x002B7881, 0x003D868E, 0x005FA7AE, 0x004EA6B7, 0x00439CA7, 0x00267F83, 0x00227B78, 0x0058ACAA, 0x00409196, 0x0046919F, 0x004E97AB, 0x001C7179, 0x004798A0, 0x004C96A2, 0x007BC1CE, 0x00357C8A,
        0x002B7C8B, 0x0052ADBC, 0x0054B7C5, 0x0065B9B4, 0x0051A5A0, 0x0055A9A5, 0x004B9F9B, 0x00489B99, 0x00358886, 0x005EB1AF, 0x0058AAAB, 0x0058B0BE, 0x0054ADBB, 0x0054ADBB, 0x0057B0BE, 0x0055B1BE, 0x0050ACB9, 0x004CAAB7, 0x004EACB9, 0x003DA7BF,
        0x003298AF, 0x0052AFC6, 0x0059AFC3, 0x00489AAB, 0x004C9EAA, 0x0052A7AF, 0x005BB3BA, 0x0050ACB9, 0x0050ACB9, 0x0047A3B0, 0x004CA8B5, 0x004BA7B4, 0x004AA6B3, 0x0055B1BE, 0x004AA6B3, 0x0027829D, 0x00167798, 0x002B96BC, 0x003DB1E0, 0x0028A2D6,
        0x00209BD3, 0x002DA5E0, 0x002DA3DE, 0x002698C7, 0x00339CCF, 0x003B9BD1, 0x003798D0, 0x00329BCE, 0x002B92BF, 0x001C78A1, 0x000F6085, 0x000C4E61, 0x00364440, 0x0053523D, 0x0048513D, 0x004F4F41, 0x00454E39, 0x0042543D, 0x004E4D43, 0x004F5445,
        0x004E5344, 0x004E5344, 0x004F5445, 0x00505546, 0x00505546, 0x004E5344, 0x004D5243, 0x004C5344, 0x004C5344, 0x004D5445, 0x004D5445, 0x004E5546, 0x004E5546, 0x004D5445, 0x004D5445, 0x004D5342, 0x004B5140, 0x004A503F, 0x004B5140, 0x004C5241,
        0x004E5443, 0x004E5443, 0x004E5443, 0x00525343, 0x00515242, 0x00515242, 0x00515242, 0x00515242, 0x00515242, 0x00515242, 0x00525343, 0x004A5142, 0x004B5243, 0x004C5344, 0x004D5243, 0x004C5142, 0x004D5041, 0x004D5041, 0x004E5142, 0x00534E3F,
        0x004A4E42, 0x00465043, 0x004B4E3E, 0x00554F3C, 0x003F4338, 0x0018383E, 0x000E485E, 0x00094962, 0x000A4A63, 0x000B4B64, 0x000B4D66, 0x000B4D66, 0x000B4D66, 0x00094C65, 0x00094C65, 0x00094C67, 0x00094C67, 0x000A4D68, 0x000B4E69, 0x000B4E69,
        0x000B4E69, 0x000A4D68, 0x000A4D68, 0x00074A65, 0x00206C89, 0x003B87AB, 0x00156289, 0x0000446B, 0x0032A7C6, 0x00279AB5, 0x002E8BA5, 0x002E89A8, 0x0038A8CC, 0x0031B5E4, 0x00048DC5, 0x0035B0EA, 0x00147CB1, 0x0041A0C8, 0x00025D7C, 0x000D4D6B,
        0x00085167, 0x000D5266, 0x00124F69, 0x00084E6B, 0x00035A6E, 0x001C6E73, 0x00407F76, 0x000F0E10, 0x00010200, 0x001A1D14, 0x00353829, 0x003A3D2D, 0x00474838, 0x0048483A, 0x003C3C2E, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFE, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x005C6054, 0x00485545, 0x000C0E08, 0x00191111, 0x00393431, 0x002E382B, 0x00426757, 0x003A766E, 0x00155F63, 0x00225B74, 0x00215A73, 0x001F5871, 0x001F5871, 0x001F5871,
        0x001E5770, 0x001B546D, 0x0019526B, 0x001F4D6F, 0x001E5673, 0x0012546D, 0x0015556D, 0x0019516A, 0x00194D6B, 0x000F4A6A, 0x00296E8F, 0x004BB3DE, 0x003F9BB8, 0x00488A95, 0x005D7980, 0x0055535F, 0x008C8592, 0x00F0FFFF, 0x00DFFFFC, 0x00F1F8F1,
        0x00FBFFFB, 0x00F1F8F1, 0x00F4F8F2, 0x00FDFFFB, 0x00C0C4BE, 0x00575B55, 0x001B1F19, 0x000E2E23, 0x00222C2C, 0x0027313B, 0x002F525F, 0x00497782, 0x00A1BEC3, 0x00879593, 0x0095A59A, 0x007E867C, 0x004C5449, 0x003E4439, 0x00555C4F, 0x00535849,
        0x00484B3B, 0x00545643, 0x005A5C49, 0x00535440, 0x00656450, 0x00686654, 0x005A5749, 0x0057554A, 0x005F6056, 0x007B8379, 0x00AEB9B1, 0x008E9997, 0x009AB9B8, 0x006BA4A6, 0x00156068, 0x00206A7C, 0x005093AE, 0x004992B0, 0x00388DAD, 0x000C516C,
        0x00004458, 0x001D667C, 0x003287A7, 0x003198C9, 0x0037AAE7, 0x003BB3EF, 0x0031A8E0, 0x00208FB7, 0x000C4663, 0x00165871, 0x003EACD6, 0x00289CD5, 0x003AAEE3, 0x0035B0E2, 0x0037ABE6, 0x0029A6E0, 0x003192C4, 0x00126384, 0x00176272, 0x0052939B,
        0x00378088, 0x000C5A67, 0x000E4D62, 0x00788E8C, 0x00A5BABB, 0x00ABBFC0, 0x00A5B9BA, 0x00B6C9CC, 0x00B7C8CB, 0x00BCCCD2, 0x00BFCFD5, 0x00AAD1D9, 0x00ABD0D4, 0x00ADCECA, 0x00ADCCC3, 0x00B2D0C5, 0x00B5D1CB, 0x00A9C7C8, 0x009DBBC0, 0x00B7CDD2,
        0x00B8CDCF, 0x00BFD4D6, 0x00B2C6C7, 0x00A7BAB7, 0x00ABBCB8, 0x00B5C4C0, 0x00CBDBD4, 0x00ABC1C7, 0x00ADC4C6, 0x00AFC6C1, 0x00A8C0B6, 0x00B7CCC3, 0x00A5B6B2, 0x00B8C4C8, 0x00B8C1CA, 0x00AFC0BC, 0x00ABB8B6, 0x00B3B9B8, 0x00D2D6D7, 0x00BABFC0,
        0x00B7C3C5, 0x00B4C9CA, 0x00B1CDCE, 0x00B5C8C5, 0x00BBCECB, 0x00BACCCB, 0x00C6D8D7, 0x00B3C5C6, 0x00AFC1C2, 0x00BBCCCF, 0x00BBCCCF, 0x00BCC7BF, 0x00C4CFC7, 0x00C5D2CA, 0x00BFCCC4, 0x00B9C8C0, 0x00BBCAC2, 0x00BDCCC4, 0x00BBCCC3, 0x00AFCDD2,
        0x00A5C2C7, 0x00B8CED3, 0x00BCCFD4, 0x00B7C8CB, 0x00BACBCE, 0x00B3C7C8, 0x00B3C8C9, 0x00C0CDC5, 0x00C4D1C9, 0x00BFCCC4, 0x00C5D2CA, 0x00C6D3CB, 0x00C5D2CA, 0x00C7D4CC, 0x00B5C2BA, 0x00034851, 0x00004C5C, 0x000C6881, 0x002A94B9, 0x003AADDF,
        0x002EA6E1, 0x00209BD9, 0x00209ADC, 0x00269BC8, 0x00309CCC, 0x00369BCF, 0x003298CD, 0x00359ED1, 0x003C9DCF, 0x002776A7, 0x00014171, 0x00003141, 0x003A403B, 0x00645E47, 0x004C523F, 0x004E4C41, 0x00484E3D, 0x00415140, 0x004B4A46, 0x004A4F40,
        0x00494E3F, 0x00484D3E, 0x00494E3F, 0x004A4F40, 0x004A4F40, 0x00484D3E, 0x00474C3D, 0x00484F40, 0x00484F40, 0x00484F40, 0x00484F40, 0x00484F40, 0x00484F40, 0x00495041, 0x00495041, 0x00494F3E, 0x00494F3E, 0x00494F3E, 0x004B5140, 0x004D5342,
        0x004D5342, 0x004B5140, 0x004A503F, 0x004B5140, 0x004C5241, 0x004C5241, 0x004C5241, 0x004C5241, 0x004C5241, 0x004C5241, 0x004B5140, 0x004B5243, 0x004C5344, 0x004C5344, 0x004C5142, 0x004A4F40, 0x004B4E3F, 0x004C4F40, 0x004D5041, 0x00514C3D,
        0x00484C40, 0x00454F42, 0x004B4E3E, 0x00544E3B, 0x003E4237, 0x0016363C, 0x000A445A, 0x00094962, 0x000A4A63, 0x000C4C65, 0x000B4D66, 0x000B4D66, 0x000B4D66, 0x00094C65, 0x00094C65, 0x000B4E69, 0x000B4E69, 0x000C4F6A, 0x000C4F6A, 0x000C4F6A,
        0x000B4E69, 0x000A4D68, 0x000A4D68, 0x000A425F, 0x00115774, 0x000D5676, 0x00003C61, 0x00004D6F, 0x004CBCDA, 0x0041B0CA, 0x001F7C96, 0x001C7695, 0x003095BB, 0x004AC0EA, 0x0043B9EA, 0x00349DD0, 0x0000517C, 0x0025809F, 0x003895AE, 0x000D4D6B,
        0x00075066, 0x000A4F63, 0x00104D67, 0x00084E6B, 0x00035A6E, 0x001B6D72, 0x003E7D74, 0x000D0C0E, 0x00000100, 0x00191C13, 0x00343728, 0x00393C2C, 0x00464737, 0x00474739, 0x003B3B2D, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFE, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00595D51, 0x00475444, 0x000D0F09, 0x00170F0F, 0x0035302D, 0x00313B2E, 0x004A6F5F, 0x003E7A72, 0x00155F63, 0x001F5871, 0x001E5770, 0x001E5770, 0x001E5770, 0x001F5871,
        0x001F5871, 0x001D566F, 0x001B546D, 0x001B4C6C, 0x001B5572, 0x0010536A, 0x00125068, 0x00174F68, 0x00154B69, 0x000F4E70, 0x003E88AA, 0x0035ADD7, 0x00055272, 0x00314B5B, 0x0074717A, 0x00555059, 0x006B6D77, 0x00E1E4E9, 0x00FFFFFF, 0x00FAFEF8,
        0x00FCFEF8, 0x00FFFFF9, 0x00FFFFF8, 0x00CACBC1, 0x0055584F, 0x00181C17, 0x00272C2A, 0x001B3234, 0x00032427, 0x000C282F, 0x0052616A, 0x00A6BCC1, 0x006D918B, 0x0096ACA0, 0x00898174, 0x0043473B, 0x00515549, 0x00575C4D, 0x00525847, 0x004E5441,
        0x004F5542, 0x00525944, 0x004F5641, 0x00595941, 0x005D5C47, 0x00605F4B, 0x00615F4D, 0x00696658, 0x0059574C, 0x0058574D, 0x008B8C83, 0x009BB2AE, 0x00959A99, 0x00B4C1C3, 0x005E9697, 0x0056AAB0, 0x0054A3B8, 0x0086CEF0, 0x004393BC, 0x0077BBD2,
        0x00276B7E, 0x0003485B, 0x001F7291, 0x0046ACDD, 0x003AACE8, 0x002FA4E1, 0x0038ABE4, 0x002190B8, 0x000D4764, 0x00175972, 0x003FADD7, 0x00289CD5, 0x003AAEE3, 0x0035B0E2, 0x0037ABE6, 0x001DA9E4, 0x004093C7, 0x001F5576, 0x007EA5B4, 0x00EFFEFF,
        0x00B3DFE6, 0x0014646F, 0x000A4F62, 0x00578078, 0x0082A9A1, 0x0082A79F, 0x007FA298, 0x0089ABA0, 0x0086A69B, 0x0090AEA1, 0x0090AEA1, 0x008DAFA8, 0x008BAEA4, 0x0088AD9D, 0x0088AF9A, 0x008AB09A, 0x008DAF9E, 0x008FADA0, 0x0090ABA1, 0x008BAEA0,
        0x0088AB9D, 0x008DB0A2, 0x0089AD9D, 0x0084AA98, 0x0083A997, 0x007BA28D, 0x007EA590, 0x007EA39B, 0x00799E94, 0x007AA191, 0x00799F8D, 0x007EA291, 0x00749888, 0x0084A39A, 0x007A9992, 0x0085AF9D, 0x00709886, 0x00759988, 0x007EA08F, 0x007DA191,
        0x007AA191, 0x006E9B8A, 0x006D9F8D, 0x007C9A8D, 0x0086A499, 0x007F9D92, 0x008CAA9F, 0x0083A097, 0x0086A39A, 0x00839F99, 0x00809C96, 0x007EA193, 0x0082A597, 0x0081A496, 0x007B9F91, 0x007CA092, 0x0080A798, 0x0080A798, 0x007BA293, 0x0085AA9A,
        0x0082A696, 0x0085A89A, 0x0080A395, 0x007FA196, 0x0084A79D, 0x0080A59B, 0x0085AAA0, 0x0086AA9C, 0x008BAFA1, 0x0089AD9F, 0x0089AD9F, 0x0085A99B, 0x0085A99B, 0x0087AB9D, 0x00769A8C, 0x00174D4E, 0x0014525A, 0x00004E5F, 0x0005607F, 0x002C95BE,
        0x0040AFE1, 0x0032A3DB, 0x00289BD4, 0x0024A3CA, 0x00239BC5, 0x002A9FCC, 0x0026A0CE, 0x00209ECE, 0x0032ABDC, 0x00379DD1, 0x001870A6, 0x00013847, 0x003F433D, 0x00645C45, 0x003B3F2C, 0x00323026, 0x003B4031, 0x00405143, 0x004D4B4A, 0x004B5041,
        0x004A4F40, 0x004A4F40, 0x004B5041, 0x004B5041, 0x004B5041, 0x00484D3E, 0x00474C3D, 0x00495041, 0x00495041, 0x00484F40, 0x00484F40, 0x00495041, 0x004A5142, 0x004C5344, 0x004D5445, 0x004E5443, 0x004E5443, 0x004F5544, 0x00505645, 0x00525847,
        0x00525847, 0x00515746, 0x004F5544, 0x004A5444, 0x004B5545, 0x004C5646, 0x004D5747, 0x004D5747, 0x004C5646, 0x004B5545, 0x004A5444, 0x004C5344, 0x004C5344, 0x004B5243, 0x004B5041, 0x00494E3F, 0x004A4D3E, 0x004B4E3F, 0x004C4F40, 0x00534E3F,
        0x004A4E42, 0x00475144, 0x004E5141, 0x0058523F, 0x0041453A, 0x0018383E, 0x000B455B, 0x00094962, 0x000A4A63, 0x000C4C65, 0x000B4D66, 0x000C4E67, 0x000B4D66, 0x000A4D66, 0x00094C65, 0x000B4E69, 0x000B4E69, 0x000B4E69, 0x000B4E69, 0x000B4E69,
        0x000A4D68, 0x000A4D68, 0x00094C67, 0x0019516E, 0x00064C69, 0x00004662, 0x00054564, 0x000B4E6F, 0x002D7D9C, 0x001D6888, 0x00074060, 0x00004058, 0x00004A64, 0x0020738F, 0x003383A2, 0x00145776, 0x000C4C64, 0x00004455, 0x00226F78, 0x0010506E,
        0x00075066, 0x00094E62, 0x000F4C66, 0x00094F6C, 0x00035A6E, 0x0016686D, 0x0035746B, 0x000B0A0C, 0x00000100, 0x001B1E15, 0x00383B2C, 0x003D4030, 0x00464737, 0x00444436, 0x00363628, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFE, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00575B4F, 0x00404D3D, 0x000F110B, 0x001D1515, 0x00342F2C, 0x00273124, 0x003B6050, 0x00357169, 0x00155F63, 0x001F5772, 0x001E5671, 0x001D5570, 0x001D5570, 0x001E5671,
        0x001D5570, 0x001B536E, 0x0019516C, 0x001A4C68, 0x001A556F, 0x0012526A, 0x00134F67, 0x001A4F6A, 0x00144A69, 0x000C4E71, 0x004596B9, 0x0035A4D2, 0x000A5172, 0x001A3442, 0x00414649, 0x002D3639, 0x003B454C, 0x00A2A0A6, 0x00FFF7FB, 0x00FAFFFE,
        0x00F2F6F0, 0x00ECF0E5, 0x00B0B2A6, 0x0046473D, 0x00171E17, 0x00232B2A, 0x00243032, 0x0012313A, 0x000B2F35, 0x003C565C, 0x00ABB4B7, 0x007E8C88, 0x0091ADA0, 0x00808F81, 0x00584D3F, 0x005F5F51, 0x00535345, 0x004B4C3C, 0x004B4D3A, 0x004E503C,
        0x004F523C, 0x0060644B, 0x007D8168, 0x009C977E, 0x00918C73, 0x00615D45, 0x00635F4C, 0x00696755, 0x005D5A4C, 0x00696358, 0x00554F44, 0x008E8A89, 0x009BA7A1, 0x0090A29B, 0x00B8CDCB, 0x0071A4A7, 0x004CA5B0, 0x0055A9C5, 0x003A759D, 0x005AA5BF,
        0x00367F93, 0x0000485B, 0x00065D7D, 0x0046ADDE, 0x003EB0EC, 0x0034A7E4, 0x0036AAE1, 0x002190B8, 0x000D4764, 0x00175972, 0x003FADD7, 0x00289CD5, 0x0039ADE2, 0x0035B0E2, 0x0037ABE6, 0x001BA3DE, 0x004191C6, 0x0023597A, 0x007CA3B1, 0x00E9F7FD,
        0x00ADD6D9, 0x00125C62, 0x00094454, 0x00B8BABA, 0x00D3D5D5, 0x00CACDCB, 0x00CCD0CB, 0x00D0D4CE, 0x00C9CEC5, 0x00CCD4C9, 0x00CAD3C6, 0x00D6D3CF, 0x00D0D2CC, 0x00CED4C9, 0x00CCD5C8, 0x00CBD4C7, 0x00CED2C6, 0x00D3D2C8, 0x00D8D5CD, 0x00CDD1C6,
        0x00CACEC3, 0x00CCD0C5, 0x00CAD0C5, 0x00CCD5C8, 0x00D0D9CC, 0x00CAD4C7, 0x00C9D3C6, 0x00C7CDC8, 0x00C3CAC3, 0x00C5CDC3, 0x00CAD0C5, 0x00CDD3C8, 0x00C4C9C0, 0x00C7CBC5, 0x00C2C6C1, 0x00BFC9BD, 0x00C1C9BE, 0x00D0D6CB, 0x00CBCFC4, 0x00C9D0C3,
        0x00BDC6B9, 0x00C2CCBF, 0x00C8D5C7, 0x00C3D3C2, 0x00C5D4C6, 0x00BFCEC0, 0x00BCCBBD, 0x00C5D3C7, 0x00C3D1C5, 0x00C4D2C7, 0x00C0CEC3, 0x00C4CAC9, 0x00C1C7C6, 0x00BCC4C3, 0x00BBC3C2, 0x00BDC8C6, 0x00C1CCCA, 0x00C0CBC9, 0x00BAC7C5, 0x00C5C5B7,
        0x00C8CBBC, 0x00C9CDC1, 0x00C3C9BE, 0x00C4CBC4, 0x00C6CCC7, 0x00C2C8C3, 0x00C7CCCA, 0x00C1C9C8, 0x00C5CDCC, 0x00C4CCCB, 0x00C3CBCA, 0x00C1C9C8, 0x00C5CDCC, 0x00C7CFCE, 0x00B9C1C0, 0x00214E52, 0x00154A54, 0x00064958, 0x00004960, 0x00045F7E,
        0x003293B5, 0x0049ADD0, 0x003A9DC3, 0x0030A4C7, 0x0031A1C5, 0x0033A5CA, 0x002BA6CC, 0x001CA5CC, 0x0021ABD9, 0x002AA9DC, 0x002599D0, 0x00185265, 0x003E4341, 0x00554E3A, 0x00242717, 0x00141208, 0x002D3223, 0x00445443, 0x00534F4E, 0x004B5041,
        0x004B5041, 0x004C5142, 0x004D5243, 0x004D5243, 0x004C5142, 0x00494E3F, 0x00464B3C, 0x00495041, 0x00495041, 0x00495041, 0x00495041, 0x004B5243, 0x004C5344, 0x004E5546, 0x004F5647, 0x004E5443, 0x004D5342, 0x004C5241, 0x004D5342, 0x004E5443,
        0x00505645, 0x004F5544, 0x004F5544, 0x00485242, 0x00495343, 0x004A5444, 0x004B5545, 0x004B5545, 0x004A5444, 0x00495343, 0x00485242, 0x004D5445, 0x004D5445, 0x004C5344, 0x004C5142, 0x004A4F40, 0x004B4E3F, 0x004C4F40, 0x004E5142, 0x00514C3D,
        0x00474B3F, 0x00444E41, 0x004C4F3F, 0x0057513E, 0x00404439, 0x0016363C, 0x00084258, 0x000A4A63, 0x000B4B64, 0x000C4C65, 0x000B4D66, 0x000C4E67, 0x000C4E67, 0x000A4D66, 0x00094C65, 0x000B4E69, 0x000A4D68, 0x000A4D68, 0x00094C67, 0x00094C67,
        0x00094C67, 0x00094C67, 0x00094C67, 0x000A405E, 0x0000455F, 0x0006566D, 0x0010566E, 0x00004560, 0x00004E69, 0x00045471, 0x00105170, 0x00185B70, 0x00044F65, 0x00025368, 0x00004A5D, 0x0015566B, 0x0018576B, 0x001F6674, 0x00004A56, 0x000D4D6B,
        0x00075066, 0x000B5064, 0x00104D67, 0x00074D6A, 0x0001586C, 0x001A6C71, 0x003E7D74, 0x000E0D0F, 0x00020301, 0x001E2118, 0x003B3E2F, 0x003E4131, 0x00464737, 0x00434335, 0x00343426, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFE, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00575B4F, 0x00404D3D, 0x000E100A, 0x001A1212, 0x00332E2B, 0x00293326, 0x003E6353, 0x0036726A, 0x00176165, 0x001F5A74, 0x001E5973, 0x001C5771, 0x001B5670, 0x001A556F,
        0x0018536D, 0x0016516B, 0x00134E68, 0x00194F68, 0x001A566E, 0x0017566B, 0x00195369, 0x001F516D, 0x00144A6B, 0x00044A6F, 0x003E94B8, 0x003C9DCF, 0x000D5F7E, 0x00003339, 0x00234642, 0x00657875, 0x00636E72, 0x003C4549, 0x00697275, 0x0098A6A5,
        0x009DA8A5, 0x007B8379, 0x00383E33, 0x00131B11, 0x00202B29, 0x001F3237, 0x001B313C, 0x0022535B, 0x00647981, 0x00B0B7BA, 0x0081938C, 0x00A1B8AA, 0x00838A7D, 0x004B473C, 0x0056584C, 0x00555545, 0x007E7C6A, 0x00706E5C, 0x0054503D, 0x00807B66,
        0x00B8B29B, 0x00C9C4AB, 0x00DCD5BC, 0x00D6C5AB, 0x00DDD0B6, 0x009D967D, 0x00625E46, 0x00585743, 0x00615F4D, 0x006B6657, 0x00655D50, 0x00584842, 0x00878C83, 0x009DB3A7, 0x007E968E, 0x009EB8B8, 0x008FB7C3, 0x006FACC0, 0x004F99B3, 0x00439CBE,
        0x004B9FBB, 0x0020758F, 0x00106E91, 0x0035A2D5, 0x0031A7E2, 0x003AB0EB, 0x0036A8DD, 0x002190B8, 0x000D4764, 0x00175972, 0x003FADD7, 0x00289CD5, 0x0039ADE2, 0x0034AFE1, 0x0036AAE5, 0x002FA6DE, 0x003391BF, 0x0010637F, 0x000B5861, 0x003D7B7B,
        0x00316D6C, 0x0020575A, 0x002E515B, 0x00737472, 0x00838482, 0x007C807B, 0x00828883, 0x00828A83, 0x007B867C, 0x0079877C, 0x007B897D, 0x007A8A83, 0x00778A81, 0x00758A81, 0x00748C82, 0x00728A7E, 0x0071877B, 0x0074887B, 0x00788C7F, 0x00808A7E,
        0x00818B7F, 0x00818B7F, 0x007D877B, 0x007A8379, 0x0079847A, 0x00788379, 0x00788379, 0x00757D76, 0x00777F78, 0x00777F78, 0x0079817A, 0x00757D76, 0x006B766E, 0x006A756D, 0x00727D75, 0x007B7873, 0x00807D78, 0x00838179, 0x007A7870, 0x00838278,
        0x007B7A70, 0x00828075, 0x007E7C71, 0x00757D6C, 0x00767E6D, 0x00798071, 0x00707768, 0x007C8374, 0x0072796C, 0x00757C6F, 0x0072796C, 0x007B7F79, 0x00797D77, 0x00767D76, 0x00777E77, 0x00777F78, 0x00767E77, 0x00777F78, 0x0077827A, 0x00797D72,
        0x007B8176, 0x00788276, 0x00738176, 0x0077847C, 0x0076807A, 0x00717873, 0x00747775, 0x007C837C, 0x007A817A, 0x007A817A, 0x007C837C, 0x007D847D, 0x00828982, 0x007F867F, 0x00727972, 0x002C4F53, 0x00285259, 0x0019505D, 0x00044A5B, 0x00004E63,
        0x0012677D, 0x00257B91, 0x00277D91, 0x00197391, 0x00267998, 0x00217494, 0x001B7C9E, 0x00299CC1, 0x002AA9D6, 0x00209ED4, 0x002199D4, 0x001D5F78, 0x00354244, 0x00555543, 0x00374131, 0x002E2D23, 0x003E4431, 0x00495843, 0x00504D45, 0x004B5041,
        0x004B5041, 0x004B5041, 0x004C5142, 0x004C5142, 0x004B5041, 0x00484D3E, 0x00454A3B, 0x00464D3E, 0x00474E3F, 0x00474E3F, 0x00484F40, 0x00495041, 0x004B5243, 0x004C5344, 0x004D5445, 0x004C5241, 0x004A503F, 0x00484E3D, 0x00484E3D, 0x004A503F,
        0x004C5241, 0x004D5342, 0x004D5342, 0x00485144, 0x00485144, 0x00485144, 0x00495245, 0x00495245, 0x00485144, 0x00485144, 0x00485144, 0x004C5344, 0x004D5445, 0x004D5445, 0x004D5243, 0x004B5041, 0x004C4F40, 0x004D5041, 0x004E5142, 0x00504B3C,
        0x00464A3E, 0x00434D40, 0x004C4F3F, 0x0057513E, 0x0041453A, 0x0017373D, 0x00094359, 0x000A4A63, 0x000B4B64, 0x000C4C65, 0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000A4D66, 0x000A4D66, 0x000A4D68, 0x000A4D68, 0x00094C67, 0x00084B66, 0x00084B66,
        0x00084B66, 0x00094C67, 0x00094C67, 0x001B4968, 0x000C566E, 0x00409CAF, 0x0062BCCD, 0x005AB9C9, 0x0044B4C6, 0x0040B9CD, 0x0047B9D0, 0x0054B5CF, 0x0049B3CA, 0x0050C0D4, 0x0046B1C6, 0x0061C1D8, 0x0051AEC8, 0x0053B9D6, 0x0010809E, 0x00074765,
        0x00064F65, 0x000E5367, 0x00114E68, 0x00044A67, 0x0001586C, 0x0023757A, 0x00519087, 0x00131214, 0x00050604, 0x001E2118, 0x00393C2D, 0x003B3E2E, 0x00454636, 0x00444436, 0x00363628, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFE, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00585C50, 0x00455242, 0x000D0F09, 0x00150D0D, 0x00332E2B, 0x00323C2F, 0x00476C5C, 0x003A766E, 0x00166064, 0x001C5673, 0x001B5572, 0x001A5471, 0x00195370, 0x00195370,
        0x0018526F, 0x0016506D, 0x00144E6B, 0x00185167, 0x0018546A, 0x0017566B, 0x001A5369, 0x001F516D, 0x00164C6D, 0x00014B6F, 0x003891B6, 0x003398CC, 0x00055574, 0x002D5D63, 0x00839994, 0x00D6E1DE, 0x00CFDCDE, 0x00506366, 0x002B4041, 0x003E5459,
        0x00405251, 0x003B4943, 0x00364137, 0x0026342E, 0x001F3334, 0x000C2731, 0x00123343, 0x0053848C, 0x00A0B4B9, 0x008A8D8B, 0x00A8B6AA, 0x00687A69, 0x00494C3D, 0x005E5950, 0x004E5148, 0x00565644, 0x00888773, 0x00928C79, 0x0088816D, 0x00B5A892,
        0x00D2C4AD, 0x00D0BFA5, 0x00E3CFB6, 0x00DEC3A9, 0x00DDC8AD, 0x00DCCFB5, 0x009A957C, 0x005D5C47, 0x005F5E4A, 0x0062604E, 0x006B6657, 0x005A5D4E, 0x00584F45, 0x0086877D, 0x0093ACA2, 0x00738A86, 0x00AEB8BF, 0x0090B0BB, 0x0062ADB5, 0x00349DC6,
        0x002185A8, 0x003596B7, 0x001B84AB, 0x0031A4D7, 0x0036B1EB, 0x0029A0D7, 0x003AADDF, 0x002190B8, 0x000D4764, 0x00175972, 0x003EACD6, 0x00279BD4, 0x0038ACE1, 0x0033AEE0, 0x0035A9E4, 0x003FA9DE, 0x002891BC, 0x00116A85, 0x00004C59, 0x00004F4D,
        0x00104D49, 0x00425E5F, 0x00556669, 0x003C4639, 0x00454F42, 0x00434D40, 0x004A5447, 0x00485245, 0x004A5447, 0x00475144, 0x004B5548, 0x00535248, 0x00525048, 0x00525048, 0x00535149, 0x00525048, 0x004E4F45, 0x004C5044, 0x004E5344, 0x003F4F3E,
        0x00445443, 0x00455544, 0x00455244, 0x00444E41, 0x00444E42, 0x00464E43, 0x00434B40, 0x00495444, 0x00434D40, 0x003A4438, 0x00404A3E, 0x00404C40, 0x00465246, 0x003E4D3F, 0x0040503F, 0x0042453C, 0x0045483F, 0x00484C41, 0x003E4439, 0x00484C40,
        0x00474A3B, 0x004F5040, 0x004F4F3F, 0x00514C3D, 0x00504B3C, 0x00504B3C, 0x004D473A, 0x004E483B, 0x004B453A, 0x004C463B, 0x00514B40, 0x00464B3C, 0x00454C3D, 0x00464D3E, 0x00454C3D, 0x00414B3B, 0x00414B3B, 0x00424D3D, 0x00434E3E, 0x00444E42,
        0x00434F43, 0x003E4F42, 0x003C5043, 0x00415346, 0x00425044, 0x00464E43, 0x004D5146, 0x00475141, 0x00434D3D, 0x00465040, 0x00485242, 0x00465040, 0x00485242, 0x00465040, 0x003F4939, 0x00415653, 0x00385657, 0x00235054, 0x00114E58, 0x000B5260,
        0x00004A5B, 0x00004455, 0x00004A5B, 0x00004A61, 0x00064C64, 0x0004445D, 0x00004765, 0x00136F92, 0x002C99C5, 0x002EA2D7, 0x002598D5, 0x00166181, 0x002A3F47, 0x00535A4D, 0x004D5A4C, 0x004F5046, 0x004E563F, 0x0046553A, 0x00514C3D, 0x004F5445,
        0x004D5243, 0x004B5041, 0x004A4F40, 0x004B5041, 0x004A4F40, 0x00494E3F, 0x00484D3E, 0x00474E3F, 0x00484F40, 0x00495041, 0x00495041, 0x004A5142, 0x004A5142, 0x004A5142, 0x004A5142, 0x004D5342, 0x004C5241, 0x004A503F, 0x004A503F, 0x004C5241,
        0x004E5443, 0x004E5443, 0x004E5443, 0x004E5246, 0x004D5145, 0x004D5145, 0x004D5145, 0x004D5145, 0x004D5145, 0x004D5145, 0x004E5246, 0x00464D3E, 0x00474E3F, 0x00484F40, 0x00494E3F, 0x00484D3E, 0x00494C3D, 0x00494C3D, 0x004A4D3E, 0x00544F40,
        0x004A4E42, 0x00465043, 0x004F5242, 0x005B5542, 0x00464A3F, 0x001D3D43, 0x000F495F, 0x000A4A63, 0x000B4B64, 0x000D4D66, 0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000A4D66, 0x000A4D66, 0x000A4D68, 0x000A4D68, 0x00094C67, 0x00094C67, 0x00094C67,
        0x000A4D68, 0x000B4E69, 0x000B4E69, 0x00154765, 0x00015469, 0x0060C8D5, 0x0098FEFF, 0x0096FFFF, 0x006DE7F3, 0x0056DAEB, 0x005ADAEF, 0x0076DEF5, 0x0075E9FA, 0x0075F1FC, 0x006EE6F2, 0x007DEBFD, 0x0094FDFF, 0x0067D5F9, 0x000177A0, 0x00064664,
        0x00044D63, 0x000C5165, 0x00114E68, 0x00064C69, 0x0002596D, 0x00227479, 0x004D8C83, 0x00121113, 0x00040503, 0x001E2118, 0x00393C2D, 0x003C3F2F, 0x00454636, 0x00444436, 0x00363628, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFE, 0xFFFEFDFF, 0xFFFDFCFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00595D51, 0x00435040, 0x000E100A, 0x00181010, 0x0035302D, 0x00303A2D, 0x003F6454, 0x00316D65, 0x00125C60, 0x0017516E, 0x0017516E, 0x0017516E, 0x0018526F, 0x00195370,
        0x001A5471, 0x00195370, 0x0018526F, 0x001A5369, 0x00165268, 0x00165367, 0x00164F65, 0x001E4E6A, 0x00174D6E, 0x00044D73, 0x003994BB, 0x002EA4D5, 0x003D7C9E, 0x00D8D9E7, 0x00FFFDFC, 0x00D1D0CC, 0x00ACC0C1, 0x005A6D70, 0x00373B40, 0x002D4A4F,
        0x00405759, 0x0040514D, 0x0035463D, 0x00263733, 0x00152E32, 0x001C3D4C, 0x0060879D, 0x008EAFB8, 0x00759797, 0x00A0B0A9, 0x00857F72, 0x004E4C3A, 0x00465847, 0x003D493D, 0x00534745, 0x004C4D39, 0x00555440, 0x007A7560, 0x008E836D, 0x009C8C75,
        0x00C5AF96, 0x00DDC3AB, 0x00DEC3A9, 0x00E3C2A8, 0x00E0C5AB, 0x00DAC9AF, 0x00CBC6AD, 0x0094947C, 0x005E5F4B, 0x00605E4C, 0x00656351, 0x005B6352, 0x00575D4C, 0x005E584D, 0x00938781, 0x00ACB3B0, 0x00729A95, 0x008CB8B7, 0x0099B6BA, 0x002A9CCB,
        0x00228EB8, 0x00238DB2, 0x002D9BC5, 0x003FB7EB, 0x002BA8E2, 0x0038B2E8, 0x0036AAD9, 0x002190B8, 0x000D4764, 0x00175972, 0x003EACD6, 0x00279BD4, 0x0038ACE1, 0x0033AEE0, 0x0035A9E4, 0x003FA8D9, 0x002796BE, 0x00296E8F, 0x005C899E, 0x00448086,
        0x00476E70, 0x00434849, 0x003C4A48, 0x00525D43, 0x00555D46, 0x00515740, 0x00545642, 0x00545240, 0x005F5B49, 0x00595243, 0x005D5446, 0x004F5544, 0x00505344, 0x00525246, 0x00535146, 0x00525246, 0x004E5443, 0x004B5642, 0x00485841, 0x00505944,
        0x00525B46, 0x00505643, 0x00515542, 0x00545545, 0x005A5749, 0x005D574A, 0x00585245, 0x0052543E, 0x00555642, 0x00555543, 0x00565747, 0x00484B3B, 0x004F5340, 0x004A513C, 0x004D553E, 0x00495043, 0x00475043, 0x004E5949, 0x00495543, 0x004F5A46,
        0x00505944, 0x00525841, 0x00575A44, 0x004D5544, 0x004E5645, 0x004B5342, 0x00565D4E, 0x004D5445, 0x0052594C, 0x00484F42, 0x004D5447, 0x004E5044, 0x00505246, 0x004F5145, 0x004A4E42, 0x004B4F43, 0x004C5346, 0x004B5245, 0x00484F42, 0x004D4F43,
        0x004A5144, 0x00485343, 0x00475444, 0x0045513F, 0x00484C39, 0x0054503D, 0x005D5643, 0x0045493D, 0x0044483C, 0x004B4F43, 0x004B4F43, 0x0044483C, 0x0045493D, 0x00494D41, 0x004C5044, 0x00424C40, 0x0032443D, 0x003F6363, 0x004F858C, 0x003F8390,
        0x00317E8E, 0x00287A8C, 0x00136779, 0x00197080, 0x001F697B, 0x00216075, 0x00054861, 0x00004666, 0x00157FA8, 0x0039ABE0, 0x002C9FDC, 0x0015668B, 0x001D3944, 0x003D493D, 0x00394A3D, 0x0045493D, 0x00424B31, 0x003B4A2B, 0x0058523F, 0x00494E3F,
        0x00464B3C, 0x00424738, 0x00404536, 0x00404536, 0x00414637, 0x00424738, 0x00424738, 0x00404738, 0x00414839, 0x00414839, 0x0042493A, 0x00414839, 0x00404738, 0x003F4637, 0x003E4536, 0x00434938, 0x00414736, 0x00404635, 0x00414736, 0x00434938,
        0x00444A39, 0x00444A39, 0x00444A39, 0x0044463A, 0x00434539, 0x00424438, 0x00424438, 0x00424438, 0x00424438, 0x00434539, 0x0044463A, 0x003E4536, 0x00404738, 0x00414839, 0x00434839, 0x00424738, 0x00434637, 0x00434637, 0x00434637, 0x004A4536,
        0x003F4337, 0x003B4538, 0x00444737, 0x00514B38, 0x003D4136, 0x0014343A, 0x00064056, 0x000A4A63, 0x000B4B64, 0x000D4D66, 0x000C4E67, 0x000D4F68, 0x000C4E67, 0x000B4E67, 0x000A4D66, 0x000B4E69, 0x000A4D68, 0x000A4D68, 0x000A4D68, 0x000A4D68,
        0x000B4E69, 0x000C4F6A, 0x000D506B, 0x000F4E6A, 0x00005063, 0x005AC9D3, 0x0092F4FE, 0x009BF7FF, 0x008BEDFD, 0x007FE8FD, 0x0088ECFF, 0x0094EBF5, 0x0097FAFE, 0x0086F4F2, 0x0086F3F1, 0x008EF0F6, 0x0095EEFF, 0x007CD7F6, 0x0010729A, 0x00094967,
        0x00034C62, 0x00084D61, 0x00104D67, 0x000A506D, 0x00045B6F, 0x00186A6F, 0x0038776E, 0x000D0C0E, 0x00010200, 0x001E2118, 0x003B3E2F, 0x003F4232, 0x00474838, 0x00424234, 0x00333325, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFEFFFD, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00585C50, 0x0042493A, 0x00040F07, 0x0010110F, 0x00423533, 0x002A2E22, 0x00335F4E, 0x002B7066, 0x001D6166, 0x0019596B, 0x00125669, 0x000D5266, 0x000F536A, 0x0011546B,
        0x00135267, 0x00155266, 0x00155365, 0x0021506F, 0x0027566B, 0x00195164, 0x0012546D, 0x00115366, 0x001A5564, 0x000A4D6C, 0x00338AC3, 0x00329FBF, 0x0088AFC5, 0x00FAE4E9, 0x00FFEAE5, 0x00EEEFEB, 0x00A1AEAC, 0x0049504B, 0x00404438, 0x004F5439,
        0x004E5540, 0x00404137, 0x0036282C, 0x00003639, 0x00123542, 0x00243E4F, 0x009EB6CA, 0x008B9B94, 0x00A6B5AD, 0x007A887D, 0x00444E42, 0x004B5243, 0x00535646, 0x00484A37, 0x004E4F3B, 0x00505344, 0x004F4F41, 0x005A5A48, 0x00898572, 0x0089816A,
        0x0090866E, 0x00C6B99F, 0x00D5C7AB, 0x00DBCAAF, 0x00DBC9AC, 0x00D6C4A5, 0x00D8C9A9, 0x00D7CCAE, 0x00A39C81, 0x005A5840, 0x00646551, 0x005C5B4D, 0x005B5A4C, 0x00626252, 0x004D5041, 0x00868C81, 0x009FABA5, 0x008B9D9C, 0x009CB3B5, 0x0046818A,
        0x00277E98, 0x002EA4D3, 0x002CACE7, 0x0035ADE9, 0x0039A8E0, 0x0032A6DB, 0x0034B0E6, 0x00298DB7, 0x000A4965, 0x001B6179, 0x0042A8D3, 0x002C9AD4, 0x002EABDE, 0x002CB2E2, 0x003CADE6, 0x0030A8DC, 0x002C95C0, 0x00418699, 0x0081A29B, 0x00A1B2A7,
        0x00A8B9BC, 0x00384242, 0x005B5548, 0x00585A47, 0x00585A47, 0x00585A47, 0x00565949, 0x00555848, 0x00515746, 0x00505546, 0x00525748, 0x0054564A, 0x00525448, 0x0057594D, 0x005A5A4C, 0x00545446, 0x00535444, 0x00575747, 0x00555545, 0x00595949,
        0x00595949, 0x00585848, 0x00575747, 0x00565646, 0x00555545, 0x00555545, 0x00555545, 0x00595A4A, 0x00525343, 0x004E4F3F, 0x00515242, 0x00525343, 0x00515242, 0x00525343, 0x00555646, 0x00545545, 0x00555646, 0x00555646, 0x00545545, 0x00575848,
        0x005B5C4C, 0x005B5C4C, 0x00585949, 0x00525545, 0x00515444, 0x004F5242, 0x004E5141, 0x004E5141, 0x004F5242, 0x00505343, 0x00515444, 0x00515242, 0x004F5040, 0x004F5040, 0x004F5040, 0x00515242, 0x00525343, 0x00525343, 0x00515242, 0x00494C3C,
        0x004A4D3D, 0x004D5040, 0x004E5141, 0x004E5141, 0x004C4F3F, 0x00494C3C, 0x00484B3B, 0x00474D3C, 0x00494F3E, 0x004B5140, 0x004C5241, 0x004B5140, 0x004B5140, 0x004C5241, 0x004D5342, 0x00474739, 0x001F3327, 0x009A9C96, 0x0097ACA3, 0x0076988D,
        0x0083AFB0, 0x0028879A, 0x001781A6, 0x002189AC, 0x002186A6, 0x00308FA9, 0x000D6780, 0x00004867, 0x001D7CA7, 0x00379ED5, 0x00319EDC, 0x000F75A5, 0x00002A49, 0x004D4E52, 0x00302D1E, 0x004D5947, 0x00223328, 0x00445754, 0x00112927, 0x00444B46,
        0x00252F29, 0x00415048, 0x00192C23, 0x00314539, 0x00283C30, 0x002A3E32, 0x002D3F32, 0x0033413D, 0x00202F2B, 0x00293B34, 0x0031443B, 0x001F3327, 0x0037483B, 0x00202E22, 0x00404D3F, 0x00232F1B, 0x00474B3F, 0x002F2C27, 0x003C4140, 0x00223532,
        0x002C473E, 0x00193326, 0x00394C3D, 0x00273228, 0x00414C42, 0x00263127, 0x00384339, 0x0029342A, 0x00323D33, 0x00323D33, 0x002D382E, 0x00333839, 0x002E3334, 0x003B4041, 0x002B3333, 0x00333B3B, 0x00222C2C, 0x00404A4A, 0x001E2828, 0x00465252,
        0x0009262B, 0x00434C4F, 0x00132A1B, 0x004A514A, 0x0004323D, 0x00083249, 0x00144658, 0x000C4E60, 0x000D4E63, 0x000E4E67, 0x000C4D69, 0x000C4C6A, 0x000B4C68, 0x00094C65, 0x00084C63, 0x000D506B, 0x000B4E69, 0x00094C67, 0x00084B66, 0x00094C67,
        0x00094C67, 0x00084B66, 0x00074A65, 0x000E4462, 0x0009536F, 0x005ABED7, 0x0081F0FF, 0x007EE9FF, 0x007EE2FA, 0x0082E5F9, 0x008AEFFE, 0x006EE5F4, 0x0076E7F7, 0x007CE5F8, 0x0078E6F8, 0x0077EDFE, 0x007CF2FF, 0x0070D9EE, 0x0018758C, 0x000B4A66,
        0x00064960, 0x000B5365, 0x00075066, 0x00084F6A, 0x00165D71, 0x00256E72, 0x00348074, 0x000D0C0E, 0x00000100, 0x001D2017, 0x00353829, 0x003A3D2D, 0x00474838, 0x00404032, 0x00343426, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFEFFFD, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00585C50, 0x00464D3E, 0x00061109, 0x000E0F0D, 0x003B2E2C, 0x0025291D, 0x00325E4D, 0x00296E64, 0x00185C61, 0x001C566C, 0x0018546C, 0x0018536D, 0x0019546E, 0x0018536D,
        0x00174F6A, 0x001B536C, 0x0020566D, 0x00154E6E, 0x001E546B, 0x00195369, 0x0011526E, 0x00155167, 0x00205766, 0x000B4C6B, 0x00388DC5, 0x0035A4C4, 0x0023617F, 0x00718597, 0x00889292, 0x009EAB9D, 0x0099A494, 0x00607168, 0x00203F3C, 0x003E3F3B,
        0x0064595B, 0x00435359, 0x002E3D4D, 0x000A4150, 0x0005394A, 0x003C8797, 0x00639DB0, 0x00B0BEB3, 0x007B867C, 0x004D574B, 0x004A5144, 0x00444737, 0x00464634, 0x00777662, 0x00A7A48F, 0x005F6253, 0x00494A3A, 0x00585644, 0x006D6956, 0x0087806C,
        0x008D836B, 0x0092876C, 0x00C0B399, 0x00CDBA9F, 0x00D7C4A9, 0x00DFCBAC, 0x00DAC8A9, 0x00D8CAAD, 0x00D0C7AC, 0x00A7A28D, 0x00605F4B, 0x00625F50, 0x005A5748, 0x0053513F, 0x00616151, 0x004B4F43, 0x0088938B, 0x009DAEAB, 0x006C8081, 0x00144B54,
        0x000C607A, 0x002CA0CF, 0x0028A6E1, 0x002DA2DF, 0x003DAAE2, 0x003BADE2, 0x0032ACE2, 0x002D8DB5, 0x000F4A64, 0x00185B74, 0x0042A6D0, 0x0034A0DA, 0x0036B0E6, 0x002DB0E2, 0x0037A6E4, 0x002FA5D9, 0x00248CB7, 0x003A7F92, 0x0073958B, 0x008D9E93,
        0x009CAEAF, 0x00343E3E, 0x005A5748, 0x00595B48, 0x005D5F4C, 0x005B5D4A, 0x00535646, 0x00545747, 0x00575C4D, 0x00565B4C, 0x004F5445, 0x0057594D, 0x00535549, 0x00515347, 0x00535345, 0x00535345, 0x00575848, 0x005B5B4B, 0x00585848, 0x00565646,
        0x00565646, 0x00565646, 0x00565646, 0x00555545, 0x00555545, 0x00555545, 0x00545444, 0x00555646, 0x00515242, 0x004F5040, 0x00505141, 0x00525343, 0x00535444, 0x00545545, 0x00555646, 0x00535444, 0x00555646, 0x00555646, 0x00545545, 0x00555646,
        0x00585949, 0x00575848, 0x00535444, 0x00505343, 0x004F5242, 0x004E5141, 0x004E5141, 0x004E5141, 0x004F5242, 0x00505343, 0x00505343, 0x00545545, 0x00525343, 0x00515242, 0x00515242, 0x00525343, 0x00525343, 0x00505141, 0x004E4F3F, 0x004D5040,
        0x004E5141, 0x004F5242, 0x00505343, 0x00505343, 0x004E5141, 0x004C4F3F, 0x004B4E3E, 0x00484E3D, 0x00494F3E, 0x004B5140, 0x004C5241, 0x004C5241, 0x004C5241, 0x004C5241, 0x004C5241, 0x004A4A3C, 0x001E3226, 0x0092948E, 0x008BA097, 0x006A8C81,
        0x007AA6A7, 0x00248396, 0x001781A6, 0x00107B9D, 0x001C83A3, 0x002586A0, 0x003190AA, 0x00016081, 0x001F82AE, 0x003EAAE0, 0x002998D6, 0x002B95CB, 0x001D5D80, 0x007A96A1, 0x00415951, 0x0079978C, 0x003A514D, 0x006D8583, 0x002F534D, 0x00698782,
        0x00405E59, 0x00678580, 0x0045615B, 0x00607C76, 0x00436259, 0x0054736A, 0x0054766C, 0x0044625D, 0x0065807C, 0x003A5551, 0x00617C78, 0x00405E59, 0x006D8B86, 0x00365651, 0x006D8F89, 0x00265749, 0x00638A82, 0x00324F53, 0x0076959E, 0x00507785,
        0x006B97A4, 0x00496F7B, 0x0074929D, 0x00457D88, 0x00568E99, 0x00467E89, 0x00508893, 0x00477F8A, 0x00467E89, 0x00477F8A, 0x00407883, 0x00548C91, 0x00457D82, 0x00599398, 0x003B757A, 0x0059969A, 0x003D7A7E, 0x005D9A9E, 0x00337276, 0x00769C96,
        0x00477E81, 0x00588184, 0x001B5249, 0x00799F9F, 0x004894A6, 0x005BA1BF, 0x0038829A, 0x000B4D5F, 0x000C4D62, 0x000D4D66, 0x000C4D69, 0x000C4C6A, 0x000C4D69, 0x000A4D66, 0x00094D64, 0x000C4F6A, 0x000B4E69, 0x00094C67, 0x00084B66, 0x00094C67,
        0x00094C67, 0x00084B66, 0x00074A65, 0x0010426C, 0x00004A71, 0x003FAED4, 0x004FD4FA, 0x0040CCF1, 0x0043CDF1, 0x0045D0F1, 0x0046D5F4, 0x0042CCF7, 0x0048CBF7, 0x0046C5F2, 0x0043C5F4, 0x0042CBF9, 0x004AD2FF, 0x0053CEFF, 0x000A78AC, 0x000A4965,
        0x00074A61, 0x000A5264, 0x00075066, 0x0009506B, 0x00145B6F, 0x00246D71, 0x003B877B, 0x00141315, 0x00000100, 0x001C1F16, 0x00353829, 0x00383B2B, 0x00454636, 0x003F3F31, 0x00333325, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFEFFFD, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00565A4E, 0x00454C3D, 0x00040F07, 0x000B0C0A, 0x003A2D2B, 0x00313529, 0x0044705F, 0x0030756B, 0x0013575C, 0x001A5471, 0x001A506E, 0x001F5371, 0x001D516F, 0x001B516F,
        0x00164C6A, 0x001A4E6C, 0x0021506F, 0x000A5172, 0x000C4C65, 0x00114E68, 0x00135274, 0x001A526D, 0x001A4E5F, 0x0001415F, 0x003D93C9, 0x0041AFD3, 0x00004E74, 0x0044809D, 0x004F7F81, 0x00617D6A, 0x006A7964, 0x0044645F, 0x0000353F, 0x00122C3D,
        0x00232740, 0x00003146, 0x002A6077, 0x000F455E, 0x00386A81, 0x003695A4, 0x00004553, 0x00626A5F, 0x00494F44, 0x00515445, 0x004B4C3C, 0x0052503E, 0x007C7762, 0x0097917A, 0x00B1A992, 0x00959587, 0x006A6B5B, 0x00514F3D, 0x004D4936, 0x00726B57,
        0x008D856E, 0x00A1977F, 0x00998D75, 0x00CCB89F, 0x00E1CEB3, 0x00DCC7AB, 0x00CFBA9E, 0x00D1C0A5, 0x00BDB199, 0x00ACA591, 0x00686150, 0x0057513E, 0x00696350, 0x005C5742, 0x005C5B47, 0x00555646, 0x004D554A, 0x005D6B65, 0x00283B38, 0x0010434D,
        0x0013627D, 0x0041AFDF, 0x0035AFEB, 0x0033A4E2, 0x0040ABE4, 0x003FAFE4, 0x0030A7DE, 0x003190B7, 0x00134A65, 0x0014576E, 0x0042A3CF, 0x00349EDB, 0x0035AEE6, 0x002DAEE5, 0x003AA8E8, 0x0032A6DB, 0x00268CB7, 0x00438899, 0x0074938A, 0x00849489,
        0x0099ABAC, 0x00333F3F, 0x00605D4E, 0x005C5E4B, 0x005D5F4C, 0x005E5F4F, 0x005A5D4D, 0x005A5D4E, 0x005A5F50, 0x00565B4C, 0x004E5246, 0x00525448, 0x0055574B, 0x0054564A, 0x00525244, 0x00545446, 0x00575848, 0x00575747, 0x00555545, 0x00515242,
        0x00525343, 0x00535444, 0x00545545, 0x00545545, 0x00545545, 0x00535444, 0x00535444, 0x00515242, 0x00515242, 0x00505141, 0x00505141, 0x00525343, 0x00545545, 0x00555646, 0x00545545, 0x00535444, 0x00555646, 0x00545545, 0x00535444, 0x00535444,
        0x00555646, 0x00535444, 0x004F5040, 0x004D5040, 0x004D5040, 0x004E5141, 0x004E5141, 0x004E5141, 0x004F5242, 0x004F5242, 0x00505343, 0x00565747, 0x00535444, 0x00525343, 0x00535444, 0x00555646, 0x00545545, 0x00515242, 0x004D4E3E, 0x004F5242,
        0x004F5242, 0x00505343, 0x00505343, 0x00505343, 0x004F5242, 0x004E5141, 0x004D5040, 0x00494F3E, 0x00494F3E, 0x004A503F, 0x004B5140, 0x004D5342, 0x004D5342, 0x004C5241, 0x004B5140, 0x004C4C3E, 0x001F3327, 0x0090928C, 0x00869B92, 0x0067897E,
        0x0079A5A6, 0x00258497, 0x001C86AB, 0x001B88A8, 0x000E7897, 0x002187A4, 0x002285A3, 0x002187AA, 0x00228DB9, 0x0031A1D6, 0x0030A3E0, 0x0032A0E0, 0x003E94C2, 0x00488DA1, 0x000F4F53, 0x004B7E80, 0x0048686D, 0x0089A9A8, 0x0036685E, 0x00407F7D,
        0x001F5857, 0x003F716F, 0x0046706F, 0x00618987, 0x00527D7A, 0x00487A76, 0x00376B64, 0x002A5C58, 0x006E9D9B, 0x00497274, 0x005D8387, 0x00345E63, 0x0045757B, 0x00245C61, 0x0064A2A8, 0x003A7872, 0x004B8182, 0x003A6C78, 0x006EA7BC, 0x003B84A0,
        0x0054A5C6, 0x003F90B3, 0x00529BC1, 0x00418FB4, 0x004795BA, 0x004795BA, 0x004B99BE, 0x004E9CC1, 0x004B99BE, 0x0055A3C8, 0x00509EC3, 0x00539DBF, 0x004693B4, 0x0058A5C6, 0x003F8CAD, 0x005AA9CA, 0x004998B9, 0x0058A9CA, 0x003C8DAE, 0x006A9FB3,
        0x004F95B2, 0x005B8FAD, 0x0025626E, 0x004C6F7D, 0x001A5B70, 0x00508098, 0x0078A6B1, 0x000B4D5F, 0x000B4C61, 0x000D4D66, 0x000C4D69, 0x000D4D6B, 0x000C4D69, 0x000A4D66, 0x000A4E65, 0x000C4F6A, 0x000A4D68, 0x00094C67, 0x00084B66, 0x00094C67,
        0x00094C67, 0x00094C67, 0x00084B66, 0x00124E5E, 0x00005666, 0x004DC1D2, 0x0062E5F9, 0x0060DEF6, 0x0077E8FF, 0x007FE6FF, 0x0080E5FE, 0x005CDCF9, 0x0065DEFE, 0x0069DCFD, 0x006BE4FF, 0x0062E0FF, 0x005DDBFD, 0x0064D6FB, 0x00187DA4, 0x00094864,
        0x00094C63, 0x00095163, 0x00064F65, 0x000A516C, 0x0011586C, 0x00226B6F, 0x00469286, 0x001E1D1F, 0x00000100, 0x001A1D14, 0x00373A2B, 0x00363929, 0x00424333, 0x003F3F31, 0x00323224, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFDFEFC, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x0055594D, 0x00474E3F, 0x000A150D, 0x00131412, 0x003C2F2D, 0x00303428, 0x0044705F, 0x002B7066, 0x000E5257, 0x00115473, 0x00104B6B, 0x001B4E6E, 0x00174B69, 0x0017516E,
        0x00104E6C, 0x00134C6C, 0x001A4D6E, 0x0005516E, 0x0014566F, 0x00195572, 0x000A486C, 0x00144A69, 0x00225468, 0x000D4D6B, 0x00398FC3, 0x00359DC8, 0x000F6386, 0x00498E9F, 0x00367271, 0x00618E85, 0x0051706F, 0x00184550, 0x0000485C, 0x000B4961,
        0x0000354E, 0x00056277, 0x004D90A9, 0x00144A61, 0x00738EA3, 0x00508B95, 0x00093840, 0x00434539, 0x00515347, 0x00646355, 0x004D4B39, 0x004D4734, 0x00918874, 0x00B1A690, 0x008C8068, 0x009E9F8F, 0x006E6E5E, 0x00575543, 0x00585441, 0x006A6451,
        0x00736C58, 0x00A89F8B, 0x00BBB39C, 0x00A2927B, 0x00C1B096, 0x00DCC9AE, 0x00DECBB0, 0x00C0AF95, 0x00695B45, 0x00665C4A, 0x0060594A, 0x0069624E, 0x005D5642, 0x006D6750, 0x0058533E, 0x005B5B49, 0x004F5649, 0x00435048, 0x00354440, 0x00194752,
        0x0018627E, 0x0044ADDE, 0x0034ACE8, 0x0038A7E5, 0x0040A9E2, 0x003CA9DF, 0x0033AAE1, 0x002E8DB4, 0x000E4661, 0x0013556E, 0x0049AAD6, 0x00339AD8, 0x0032A9E1, 0x002FAEE5, 0x0041AAED, 0x0035A8DB, 0x002E92BC, 0x005698AA, 0x007F9E95, 0x0086978A,
        0x009AACAD, 0x00303D3B, 0x00585848, 0x00636454, 0x005C5D4D, 0x005D5E4E, 0x005B5E4F, 0x004D5041, 0x00393E2F, 0x00303428, 0x002E3226, 0x003B3D31, 0x004B4D41, 0x0055574B, 0x0058584A, 0x0058584A, 0x00575848, 0x00555545, 0x00545444, 0x00515242,
        0x00525343, 0x00535444, 0x00555646, 0x00555646, 0x00545545, 0x00535444, 0x00525343, 0x004F5040, 0x00525343, 0x00525343, 0x00505141, 0x00515242, 0x00555646, 0x00555646, 0x00515242, 0x00545545, 0x00555646, 0x00535444, 0x00515242, 0x00515242,
        0x00545545, 0x00535444, 0x004F5040, 0x004D5040, 0x004D5040, 0x004E5141, 0x004F5242, 0x00505343, 0x00505343, 0x00505343, 0x00505343, 0x00545545, 0x00525343, 0x00525343, 0x00545545, 0x00575848, 0x00575848, 0x00545545, 0x00505141, 0x004E5141,
        0x004E5141, 0x004E5141, 0x004D5040, 0x004D5040, 0x004D5040, 0x004D5040, 0x004D5040, 0x004C5241, 0x004B5140, 0x00494F3E, 0x004A503F, 0x004C5241, 0x004D5342, 0x004D5342, 0x004C5241, 0x004E4E40, 0x0023372B, 0x0093958F, 0x00889D94, 0x00698B80,
        0x0078A4A5, 0x001F7E91, 0x001680A5, 0x001681A2, 0x003EAAC9, 0x00097392, 0x001B84A5, 0x00238EB4, 0x0030A0CE, 0x00269BD2, 0x00269ED9, 0x00299DE4, 0x003098CD, 0x002386A2, 0x00106C79, 0x001D626C, 0x00779EA7, 0x007FA7A6, 0x007DB7AC, 0x003D9491,
        0x003C8887, 0x00488889, 0x006CA2A3, 0x006D9EA0, 0x006BA1A1, 0x00306C6B, 0x000A4C4A, 0x00165755, 0x00518B8A, 0x0079ACAF, 0x006F9FA5, 0x0062959F, 0x003C7985, 0x00327E8A, 0x003F939F, 0x0090B8B6, 0x00638A8C, 0x0073A1AC, 0x00428398, 0x0024829F,
        0x002397BA, 0x00229BC5, 0x002199C7, 0x002194C1, 0x001C8FBC, 0x002194C1, 0x001B8EBB, 0x002295C2, 0x001C8FBC, 0x002B9ECB, 0x00299CC9, 0x001B88BE, 0x002390C6, 0x002592C8, 0x001F8FC4, 0x002191C6, 0x00289ACF, 0x002395CA, 0x001C8EC3, 0x002391C5,
        0x001794CE, 0x002D92C6, 0x000E728B, 0x00034151, 0x00004C55, 0x001C4E4E, 0x00628C7A, 0x000A4C5E, 0x000B4C61, 0x000D4D66, 0x000C4D69, 0x000D4D6B, 0x000D4E6A, 0x000B4E67, 0x000B4F66, 0x000C4F6A, 0x000A4D68, 0x00084B66, 0x00084B66, 0x00094C67,
        0x00094C67, 0x00094C67, 0x00084B66, 0x001D4968, 0x00044D6B, 0x0047B6D6, 0x0051D7F9, 0x0045CEF5, 0x0054D7FF, 0x0056D4FD, 0x0058D6FF, 0x004AD5F6, 0x0055D7FA, 0x0055D3F6, 0x0058DAFF, 0x004ED8FC, 0x0049D2F8, 0x0050CCF4, 0x0000709A, 0x00084763,
        0x000A4D64, 0x00085062, 0x00054E64, 0x000C536E, 0x000E5569, 0x0020696D, 0x004E9A8E, 0x00282729, 0x00000100, 0x001A1D14, 0x00393C2D, 0x00363929, 0x00414232, 0x003F3F31, 0x00323224, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFDFEFC, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x0053574B, 0x003B4233, 0x00051008, 0x00131412, 0x00342725, 0x0022261A, 0x003A6655, 0x002A6F65, 0x001D6166, 0x000E5C79, 0x000D4E6A, 0x001F526C, 0x00164B60, 0x00145669,
        0x000C5269, 0x000E4F6B, 0x00184E6F, 0x000D5065, 0x00155063, 0x00144C67, 0x00165276, 0x00205776, 0x001A4F64, 0x000C4D6C, 0x002F88BB, 0x0044A5D7, 0x001E6079, 0x0082B1AE, 0x00487B73, 0x00134954, 0x000A3958, 0x00144362, 0x00134E62, 0x00094757,
        0x00196B77, 0x00409FA9, 0x0083B5C7, 0x004C8894, 0x007E8E9A, 0x00596F74, 0x00012624, 0x003A3E33, 0x0057594D, 0x0049493B, 0x00605D4E, 0x00635D4A, 0x004D4430, 0x00857A64, 0x00BBAF97, 0x007B7B69, 0x0050503E, 0x00585644, 0x00666250, 0x0095917E,
        0x00918B78, 0x00706956, 0x00837C69, 0x00988B75, 0x009C8E77, 0x00CABBA1, 0x00D3C2A8, 0x00AC9C85, 0x0060523F, 0x00615746, 0x005C5447, 0x00605945, 0x0067604C, 0x005F5942, 0x00625D48, 0x00545442, 0x00575E51, 0x002F3C34, 0x0021302C, 0x00103A46,
        0x00115A76, 0x003FA8D9, 0x002FA4E1, 0x0038A9E7, 0x003DA8E1, 0x0036A6DB, 0x0034AEE4, 0x00238BB6, 0x00054563, 0x000C536E, 0x004AAFDC, 0x00339DDA, 0x0036ABE2, 0x0034B0E6, 0x0040A7E6, 0x001C8ABE, 0x00207FAA, 0x005090A2, 0x007E9C91, 0x00859689,
        0x009DB2B0, 0x00364442, 0x005F5F4D, 0x00616252, 0x005C5D4D, 0x005C5C4E, 0x00515445, 0x00373A2B, 0x001F2317, 0x001B1F13, 0x001B1F13, 0x00212317, 0x0035372B, 0x004A4C40, 0x00575749, 0x005C5C4E, 0x005C5D4D, 0x005B5B4B, 0x00585848, 0x00525343,
        0x00535444, 0x00555646, 0x00555646, 0x00555646, 0x00545545, 0x00525343, 0x00515242, 0x004F5040, 0x00535444, 0x00545545, 0x00515242, 0x00505141, 0x00535444, 0x00535444, 0x004F5040, 0x00545545, 0x00545545, 0x00525343, 0x004F5040, 0x00505141,
        0x00545545, 0x00555646, 0x00535444, 0x004E5141, 0x004F5242, 0x00505343, 0x00515444, 0x00515444, 0x00525545, 0x00525545, 0x00525545, 0x00545545, 0x00535444, 0x00535444, 0x00555646, 0x00575848, 0x00575848, 0x00535444, 0x00505141, 0x004F5242,
        0x004F5242, 0x004E5141, 0x004E5141, 0x004D5040, 0x004D5040, 0x004E5141, 0x004E5141, 0x004F5544, 0x004C5241, 0x00494F3E, 0x00484E3D, 0x004A503F, 0x004C5241, 0x004D5342, 0x004E5443, 0x004F4F41, 0x0025392D, 0x0093958F, 0x00879C93, 0x0066887D,
        0x00729E9F, 0x00157487, 0x000C769B, 0x00127C9B, 0x00319ABB, 0x0042ADCF, 0x00036E93, 0x001282AC, 0x0036A9DB, 0x002BA2D9, 0x0028A3DD, 0x001B94DA, 0x002B98CC, 0x00026682, 0x00298794, 0x002D727C, 0x00295059, 0x00416767, 0x002E685D, 0x00166D6A,
        0x00186463, 0x001E5E5F, 0x002A6061, 0x002E5F61, 0x00316767, 0x00185453, 0x00115351, 0x001A5C57, 0x001C5655, 0x003E7173, 0x002F5F65, 0x002D606A, 0x0026636F, 0x0016626E, 0x000E626E, 0x0035574D, 0x00375752, 0x003F6567, 0x000D4851, 0x00136B7B,
        0x002A95B0, 0x001D8FB3, 0x002FA0C8, 0x002C97BC, 0x002994B9, 0x002792B7, 0x00238EB3, 0x00248FB4, 0x001F8AAF, 0x002893B8, 0x002893B8, 0x001F85B5, 0x00298FBF, 0x002991C0, 0x00258DBC, 0x00248CBB, 0x002C96C5, 0x00258FBE, 0x00228CBB, 0x00208EC2,
        0x000683BD, 0x00298DC3, 0x001C809C, 0x000E4E60, 0x00004D5A, 0x0014494C, 0x00457262, 0x000A4C5E, 0x000B4C61, 0x000D4D66, 0x000C4D69, 0x000D4D6B, 0x000D4E6A, 0x000B4E67, 0x000B4F66, 0x000B4E69, 0x000A4D68, 0x00084B66, 0x00084B66, 0x00094C67,
        0x000A4D68, 0x00094C67, 0x00094C67, 0x000E4462, 0x00004D6A, 0x0043B8D5, 0x0056DEFF, 0x004FDBFF, 0x0055DBFF, 0x0052D5FD, 0x0055D9FF, 0x004DDBFA, 0x0057DEFE, 0x0054D5F6, 0x0052D8FA, 0x0050DDFE, 0x004FDBFF, 0x0051D0F6, 0x0004769E, 0x00084763,
        0x000B4E65, 0x00085062, 0x00044D63, 0x000C536E, 0x000D5468, 0x001E676B, 0x00509C90, 0x002B2A2C, 0x00020301, 0x001A1D14, 0x003A3D2E, 0x00373A2A, 0x00414232, 0x00404032, 0x00343426, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFDFEFC, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x0052564A, 0x00444B3C, 0x0008130B, 0x00131412, 0x00342725, 0x00272B1F, 0x003E6A59, 0x0024695F, 0x0014585D, 0x0005556C, 0x00124E64, 0x00356172, 0x002A5764, 0x00205E68,
        0x000F5763, 0x00145368, 0x00245672, 0x003F6C77, 0x003B666F, 0x0017495D, 0x000E4B6D, 0x00135070, 0x00124C62, 0x001E6586, 0x00338DC3, 0x00369BCF, 0x00336A7F, 0x00819B95, 0x002A514F, 0x0008455F, 0x00104B79, 0x001B496B, 0x00567E8A, 0x00739398,
        0x0088BFC2, 0x00417B80, 0x00A9C4CE, 0x007ABDBE, 0x0079969A, 0x004E6A6A, 0x00093A32, 0x002B342A, 0x00444C42, 0x004F554A, 0x00484A3E, 0x004D4D3D, 0x004E4A38, 0x005F5946, 0x0099927E, 0x00ACAA98, 0x0073715F, 0x00514F3D, 0x005F5D4B, 0x008A8674,
        0x00969280, 0x0075715F, 0x005D5947, 0x008D8873, 0x00908A73, 0x00968C74, 0x00796D55, 0x00756951, 0x006B614F, 0x00686152, 0x00585247, 0x00595340, 0x005B5542, 0x0069644F, 0x0061604C, 0x004C4D3D, 0x003B4338, 0x00182620, 0x00091C19, 0x00083540,
        0x00105975, 0x0047B0E1, 0x002FA7E3, 0x0038ABE8, 0x003DACE4, 0x0033A7DC, 0x002BA9DF, 0x002699CC, 0x00115B7F, 0x00045473, 0x003AA4D3, 0x002F9AD7, 0x003CB0E5, 0x0038B0DE, 0x003EA2D9, 0x001784B7, 0x002381AC, 0x004F8D9D, 0x00789389, 0x007E8F81,
        0x0095AAA8, 0x0032413D, 0x00585A47, 0x005A5B4B, 0x005F5F51, 0x005E5E50, 0x00444738, 0x0026281C, 0x001E2216, 0x0024281D, 0x0021251A, 0x001F2115, 0x0027291D, 0x003C3E32, 0x00525244, 0x005A5A4C, 0x005D5E4E, 0x005D5D4D, 0x00555545, 0x00515444,
        0x00525545, 0x00525545, 0x00535646, 0x00525545, 0x00525545, 0x00505343, 0x00505343, 0x00505141, 0x00535444, 0x00535444, 0x00525343, 0x00505141, 0x00515242, 0x00515242, 0x00505141, 0x00535444, 0x00535444, 0x00515242, 0x004F5040, 0x00515242,
        0x00555646, 0x00565747, 0x00545545, 0x00505343, 0x00505343, 0x00505343, 0x00515444, 0x00515444, 0x00525545, 0x00525545, 0x00535646, 0x00565747, 0x00565747, 0x00555646, 0x00555646, 0x00545545, 0x00535444, 0x00505141, 0x004E4F3F, 0x00525545,
        0x00515444, 0x00515444, 0x00505343, 0x00505343, 0x00505343, 0x00505343, 0x00505343, 0x004F5544, 0x004D5342, 0x004A503F, 0x00484E3D, 0x00494F3E, 0x004B5140, 0x004D5342, 0x004F5544, 0x004F4F41, 0x0025392D, 0x0090928C, 0x0082978E, 0x0065877C,
        0x0074A0A1, 0x001C7B8E, 0x001781A6, 0x00096C8C, 0x002286A8, 0x00349CC1, 0x003DA9D3, 0x000170A0, 0x001B90C3, 0x002DA4DB, 0x001F98D0, 0x0026A1DF, 0x003B9CC8, 0x005BA8BB, 0x003E8588, 0x0081B9BA, 0x00527579, 0x008BABAA, 0x00427168, 0x00599896,
        0x00417A79, 0x006C9E9C, 0x00547E7D, 0x007CA4A2, 0x0057827F, 0x00659793, 0x00649891, 0x006A9D96, 0x006A9A94, 0x00648F8C, 0x00769E9D, 0x004D797A, 0x007BACAE, 0x00478082, 0x0061A1A2, 0x00416E5E, 0x0081A79B, 0x0053786E, 0x0076A49E, 0x00427F81,
        0x0085CCD6, 0x0063A4B9, 0x0079B1CE, 0x0069A9C1, 0x0077B7CF, 0x006AAAC2, 0x0076B6CE, 0x006EAEC6, 0x0074B4CC, 0x0070B0C8, 0x0073B3CB, 0x006FAFC1, 0x006EAEC0, 0x0078BACC, 0x005FA1B3, 0x0075BACB, 0x0067ACBD, 0x0073B8C9, 0x0060A6B7, 0x0089BED2,
        0x005CA2BF, 0x0083B9D8, 0x0067A8B7, 0x00759FB2, 0x001B617F, 0x00457B9C, 0x0042788F, 0x000B4D5F, 0x000B4C61, 0x000D4D66, 0x000C4D69, 0x000D4D6B, 0x000C4D69, 0x000A4D66, 0x000A4E65, 0x000B4E69, 0x00094C67, 0x00084B66, 0x00084B66, 0x00094C67,
        0x000A4D68, 0x000A4D68, 0x00094C67, 0x00004D58, 0x00005B67, 0x003FB9C7, 0x0064E4F6, 0x0072EAFF, 0x0077E5FD, 0x0074DEF5, 0x0077E1F8, 0x005CDBF4, 0x0069E0FA, 0x006ADDF8, 0x0067DDFA, 0x0065E3FF, 0x005FDDFA, 0x0057C8E8, 0x00187C9E, 0x00094864,
        0x000B4E65, 0x00085062, 0x00044D63, 0x000C536E, 0x000E5569, 0x001E676B, 0x004B978B, 0x00282729, 0x00020301, 0x001A1D14, 0x00383B2C, 0x00393C2C, 0x00434434, 0x00404032, 0x00363628, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFDFEFC, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00505448, 0x00444B3C, 0x00040F07, 0x000F100E, 0x00382B29, 0x0034382C, 0x004B7766, 0x00286D63, 0x0013575C, 0x0013596A, 0x0041707E, 0x008FAAB4, 0x0093B0B4, 0x0088B8B9,
        0x0071ABB0, 0x007DAEBC, 0x0099BBD2, 0x009FB1B0, 0x00AEC6C4, 0x00527C88, 0x000B4A66, 0x00074B68, 0x0035788D, 0x006CB9DA, 0x004EABE2, 0x002B9BC9, 0x00286684, 0x00809AA8, 0x00355C6A, 0x0003455E, 0x000D4A6A, 0x002A5265, 0x0096B0B0, 0x00829893,
        0x00A0B0AF, 0x002B4A4B, 0x00A2B4BD, 0x004B888C, 0x0038646B, 0x001E5356, 0x003A676A, 0x008E9D99, 0x00303E38, 0x003D4A42, 0x00495248, 0x003B4235, 0x004D5041, 0x004D4E3E, 0x00575745, 0x006C6B57, 0x005B5A46, 0x00585644, 0x00918F7D, 0x00787566,
        0x00595647, 0x005E5B4C, 0x00514E40, 0x00696A56, 0x008C8B76, 0x008F8B73, 0x00716C53, 0x0067614A, 0x00615B48, 0x005B584A, 0x00605E53, 0x005F5C4D, 0x00646152, 0x004E4C3A, 0x00606050, 0x0044483C, 0x00242F27, 0x0081928F, 0x003F5354, 0x00194651,
        0x0009536F, 0x003FADDD, 0x002CA8E4, 0x0030A8E4, 0x0036AAE1, 0x0032ACE0, 0x0024A8DD, 0x0029A9E4, 0x002C83AF, 0x000D678A, 0x002C9BCB, 0x002A98D2, 0x003EB2E1, 0x002BA1CA, 0x003190C1, 0x001F88BB, 0x00308CB5, 0x00508C9C, 0x00759184, 0x007E8F81,
        0x0093A8A5, 0x0034433F, 0x00575C47, 0x005C5C4E, 0x00606052, 0x005C5C4E, 0x003E4034, 0x001E2014, 0x001B1F14, 0x0022261B, 0x001B1F14, 0x0025271B, 0x00222418, 0x00383A2E, 0x00515143, 0x00545446, 0x00575848, 0x00595949, 0x00505040, 0x004F5242,
        0x00505343, 0x00505343, 0x00505343, 0x00515444, 0x00515444, 0x00515444, 0x00515444, 0x004F5141, 0x004F5242, 0x00515242, 0x00525343, 0x00515242, 0x004F5040, 0x00515242, 0x00535444, 0x00505141, 0x00525343, 0x00515242, 0x00505141, 0x00525343,
        0x00555646, 0x00535444, 0x00505242, 0x00505343, 0x00505343, 0x00505343, 0x004F5242, 0x004F5242, 0x00505343, 0x00515444, 0x00525545, 0x00555646, 0x00565747, 0x00565747, 0x00545545, 0x00525343, 0x00505141, 0x004F5040, 0x004F5040, 0x00515444,
        0x00515444, 0x00515444, 0x00515444, 0x00505343, 0x00505343, 0x00505343, 0x004F5242, 0x004D5342, 0x004D5342, 0x004C5241, 0x004B5140, 0x004A503F, 0x004B5140, 0x004C5241, 0x004E5443, 0x004E4E40, 0x0023372B, 0x008C8E88, 0x007E938A, 0x0065877C,
        0x0079A5A6, 0x00248396, 0x00248EB3, 0x00177495, 0x00004A6C, 0x001A7FA6, 0x003FAAD6, 0x002B9ACC, 0x002195CA, 0x002B9FD8, 0x002AA1D9, 0x0024A1D4, 0x0058A9CA, 0x0099C4CD, 0x0052756B, 0x0097BBAF, 0x00516A66, 0x00B6CECC, 0x004E706A, 0x0095B3AE,
        0x00587671, 0x0094B2AD, 0x005C7872, 0x0095B1AB, 0x005F7E75, 0x0083A299, 0x006C8E84, 0x00719087, 0x007F9C93, 0x00728F86, 0x0087A49B, 0x005C7B72, 0x008BAAA1, 0x005C7E74, 0x0095B8AE, 0x00517870, 0x0092B1A8, 0x00516A60, 0x009DBBB0, 0x00436963,
        0x009DC5C4, 0x0083A1AC, 0x00A1B5C6, 0x006EA5AC, 0x008EC5CC, 0x006DA4AB, 0x0089C0C7, 0x0075ACB3, 0x0086BDC4, 0x0075ACB3, 0x007DB4BB, 0x007DB2B5, 0x007AB1B4, 0x0087BEC1, 0x006DA4A7, 0x0084BDBF, 0x006FA8AA, 0x0086C1C3, 0x006FAAAC, 0x00A4C9C1,
        0x0071A8AB, 0x009DC7CC, 0x006CA7A3, 0x00AEDCE4, 0x004DA3C1, 0x005DAEDB, 0x00146B93, 0x000B4D5F, 0x000C4D62, 0x000D4D66, 0x000C4D69, 0x000C4C6A, 0x000C4D69, 0x000A4D66, 0x00094D64, 0x000A4D68, 0x00094C67, 0x00084B66, 0x00084B66, 0x00094C67,
        0x000A4D68, 0x000A4D68, 0x00094C67, 0x0012496E, 0x00014B6F, 0x001A789C, 0x001E87AE, 0x00218CB2, 0x001C8AAE, 0x00148BAB, 0x000F8FAC, 0x001788B0, 0x00218BB4, 0x00258BB5, 0x001C85B0, 0x001B8DB6, 0x001989B4, 0x001D82AF, 0x00055D8C, 0x000A4965,
        0x000A4D64, 0x00085062, 0x00054E64, 0x000B526D, 0x000F566A, 0x001E676B, 0x00449084, 0x00201F21, 0x00010200, 0x00181B12, 0x00353829, 0x00393C2C, 0x00444535, 0x003F3F31, 0x00373729, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFDFEFC, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00505448, 0x00414839, 0x00051008, 0x00151614, 0x00382B29, 0x002C3024, 0x003F6B5A, 0x0022675D, 0x00195D62, 0x00104B5A, 0x00577782, 0x00C0CBCF, 0x00C6D3D1, 0x00AACCC6,
        0x007DACAA, 0x0082A6B0, 0x009DB2C7, 0x00BFC0B6, 0x00C0CBC3, 0x0051757B, 0x0011516A, 0x00034D69, 0x001E677B, 0x005BACCD, 0x003495CD, 0x0029A8CF, 0x0009547B, 0x00507597, 0x0023526D, 0x000B4F62, 0x0015515D, 0x002B4E51, 0x00B3C8C0, 0x00396155,
        0x006B6968, 0x004B6466, 0x008CA5AF, 0x0024555F, 0x00194654, 0x002F747E, 0x00ABC1D3, 0x007C8F8C, 0x006F837E, 0x00384841, 0x0033423A, 0x00455046, 0x00424B3E, 0x00515647, 0x00454A3B, 0x004D4C38, 0x0052513D, 0x00545240, 0x00696755, 0x00565344,
        0x005D5A4C, 0x00555244, 0x00646153, 0x00555C47, 0x00636650, 0x0085856D, 0x00A6A48C, 0x00817E69, 0x00595745, 0x00555547, 0x0056574D, 0x005A594B, 0x00525143, 0x00656555, 0x00303324, 0x00343A2F, 0x0088948E, 0x006B7D7C, 0x00788F91, 0x0044727D,
        0x000D5A75, 0x003CABDB, 0x0030AEE9, 0x002EA8E4, 0x0031A6DD, 0x0031AEE1, 0x0022AADE, 0x0021A9E9, 0x0046A4D2, 0x002180A7, 0x002CA2D3, 0x002FA0D9, 0x003CB1DE, 0x00188EB1, 0x001B79A4, 0x00268FC2, 0x003B97C0, 0x00528E9E, 0x00759184, 0x00819082,
        0x0090A5A2, 0x002F403C, 0x00525742, 0x005F5F51, 0x005D5D4F, 0x005B5B4F, 0x0046483C, 0x00292B1F, 0x0021251A, 0x00262A1F, 0x0021251A, 0x00222418, 0x001E2014, 0x00393B2F, 0x00555547, 0x00515143, 0x00515242, 0x00595949, 0x00525242, 0x004D5040,
        0x004D5040, 0x004D5040, 0x004E5141, 0x004F5242, 0x00515343, 0x00535545, 0x00545645, 0x004E503E, 0x004C4D3D, 0x00515242, 0x00565747, 0x00535444, 0x004F5040, 0x00515242, 0x00565747, 0x004E4F3F, 0x00505141, 0x00525343, 0x00515242, 0x00525343,
        0x00535444, 0x00535444, 0x00515242, 0x00505141, 0x004F5040, 0x00515343, 0x00515343, 0x004E5141, 0x004F5242, 0x00505343, 0x00515444, 0x00525343, 0x00535444, 0x00545545, 0x00535444, 0x00505141, 0x004F5040, 0x00515242, 0x00535444, 0x004E5141,
        0x004E5141, 0x004E5141, 0x004F5242, 0x004E5141, 0x004E5141, 0x004D5040, 0x004D5040, 0x004B5140, 0x004D5342, 0x004E5443, 0x004D5342, 0x004C5241, 0x004B5140, 0x004B5140, 0x004C5241, 0x004E4E40, 0x0023372B, 0x008A8C86, 0x007A8F86, 0x00618378,
        0x00739FA0, 0x001E7D90, 0x001D87AC, 0x0051ABCA, 0x00147093, 0x00004D77, 0x00248CBB, 0x0037A5D9, 0x0030A1D9, 0x002DA1DA, 0x002DA1DA, 0x001D9AC6, 0x00256883, 0x008C9E9F, 0x0066705F, 0x0093A794, 0x00576D61, 0x00819491, 0x00516765, 0x00979E99,
        0x0056605A, 0x0096A59D, 0x00586B62, 0x0085998D, 0x0063776B, 0x0085998D, 0x00637568, 0x007C8B83, 0x00728378, 0x0075897D, 0x00708477, 0x00768B7C, 0x007D8F7E, 0x006A7A68, 0x008F9D8B, 0x005F6E71, 0x00979F9F, 0x00636A67, 0x008D9C94, 0x00547267,
        0x0084A8A2, 0x00617F80, 0x0095AAB2, 0x00677B80, 0x0092A6AB, 0x0062767B, 0x00889CA1, 0x006C8085, 0x00899DA2, 0x0073878C, 0x00809499, 0x00798996, 0x008595A2, 0x008999A6, 0x007F909D, 0x007F909D, 0x00768996, 0x008EA1AE, 0x008093A0, 0x00A5B0AE,
        0x00648186, 0x009CA8AC, 0x0069847B, 0x0098A7AA, 0x0029627B, 0x001E567F, 0x00003D63, 0x000C4E60, 0x000D4E63, 0x000E4E67, 0x000C4D69, 0x000C4C6A, 0x000B4C68, 0x00094C65, 0x00084C63, 0x000A4D68, 0x00094C67, 0x00084B66, 0x00084B66, 0x00094C67,
        0x000A4D68, 0x000A4D68, 0x000A4D68, 0x00244D66, 0x000E445D, 0x000E4D69, 0x00074361, 0x000F4361, 0x0010405C, 0x000B4459, 0x00024254, 0x00024A56, 0x000C4B59, 0x00134E5E, 0x00084656, 0x00044A5B, 0x00024859, 0x00124C5F, 0x001A465D, 0x000B4A66,
        0x000A4D64, 0x00095163, 0x00064F65, 0x000B526D, 0x0010576B, 0x001E676B, 0x003F8B7F, 0x0019181A, 0x00000100, 0x00171A11, 0x00323526, 0x00393C2C, 0x00444535, 0x003D3D2F, 0x00363628, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xFFFCFBFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x0054584D, 0x003B4534, 0x00061409, 0x00131614, 0x00382C2A, 0x00292D22, 0x004D7869, 0x002C7069, 0x0014585F, 0x00004B6A, 0x0040636D, 0x00BDACA3, 0x00DCC1B7, 0x00768A8F,
        0x00044553, 0x00255966, 0x00ACB5BF, 0x00BCCCCB, 0x0063878F, 0x00164E61, 0x0019556D, 0x00164D68, 0x001E5B7B, 0x002E80AA, 0x003398CC, 0x002DA9DF, 0x0007647D, 0x0080C0C1, 0x00558E97, 0x00014366, 0x00044877, 0x00174965, 0x006B888C, 0x005B7068,
        0x006A969D, 0x005C8C92, 0x006F8D8E, 0x00275164, 0x002375A4, 0x006FCDFB, 0x006EB8D0, 0x00768889, 0x00747C7B, 0x00838E86, 0x002E4939, 0x00314D36, 0x00515843, 0x0053523D, 0x004A5039, 0x00485448, 0x004A5346, 0x00585A4E, 0x004F4E40, 0x005A5546,
        0x005D5947, 0x005C5B47, 0x00575844, 0x00595844, 0x005D5946, 0x006C6552, 0x009C9281, 0x00766C5B, 0x0062594B, 0x005D5647, 0x00585344, 0x00555842, 0x00545D50, 0x003A3935, 0x00493D39, 0x008E8E82, 0x006A8673, 0x0065857A, 0x009AA7A9, 0x002E82AC,
        0x00136F9A, 0x003AA2CD, 0x0030A3D0, 0x001D96C8, 0x0032AFE2, 0x002BA6DE, 0x002FAAE4, 0x0015A6DA, 0x003BB6EE, 0x002485B7, 0x0040A1CD, 0x001C97C9, 0x0030B4EA, 0x00318EAF, 0x0070999B, 0x0039B9E9, 0x0031A5D4, 0x003C90AC, 0x006E9695, 0x00859689,
        0x0098A6A2, 0x0033403E, 0x00555B50, 0x00606050, 0x005E5E4E, 0x005E5E4E, 0x00585848, 0x00404030, 0x00252515, 0x00202010, 0x002B2B1B, 0x001E1F0F, 0x002F3020, 0x004B4C3C, 0x00525343, 0x00535444, 0x00575848, 0x00525343, 0x00555646, 0x004F5243,
        0x004B4E3F, 0x00515445, 0x004B4E3E, 0x00545646, 0x00575948, 0x00565747, 0x00525343, 0x00525544, 0x00565747, 0x00565747, 0x004F5040, 0x00535444, 0x00525445, 0x004E5041, 0x004C4E42, 0x00515445, 0x004F5243, 0x00505344, 0x00515343, 0x004F5141,
        0x00505241, 0x00525343, 0x00525343, 0x00535444, 0x00535444, 0x00545545, 0x00545645, 0x004E503E, 0x004C4D3D, 0x00515242, 0x00565747, 0x00545545, 0x00535444, 0x00525343, 0x00525343, 0x00525343, 0x00535444, 0x00545545, 0x00545545, 0x004D4E3E,
        0x00505141, 0x00535444, 0x00525343, 0x004F5040, 0x004E4F3F, 0x00525343, 0x00575848, 0x004E5141, 0x00505343, 0x00515444, 0x004E5141, 0x004A4D3D, 0x00494C3C, 0x004C4F3F, 0x004F5242, 0x00495041, 0x0021311F, 0x00889E8B, 0x007D9283, 0x006B807D,
        0x006D94A2, 0x002C7E97, 0x001384A4, 0x001798BF, 0x0039A7D1, 0x00106D9A, 0x002787B5, 0x0035A7D6, 0x0027A1D5, 0x002CA0DB, 0x00359CDB, 0x00146C9A, 0x00334441, 0x005D4F3D, 0x00474B40, 0x00454936, 0x0049493B, 0x00444842, 0x004E503C, 0x00484B3C,
        0x004B4E3F, 0x004D5041, 0x004D5041, 0x004C4F40, 0x004B4E3F, 0x004B4E3F, 0x004B4E3F, 0x004D4E3E, 0x004D4E3E, 0x004E4F3F, 0x004D4E3E, 0x004D4E3E, 0x004D4E3E, 0x004D4E3E, 0x004E4F3F, 0x004C4F3F, 0x004C4F3F, 0x004B4E3E, 0x004C4D3D, 0x004B4C3C,
        0x004B4B3B, 0x004C4C3C, 0x004C4C3C, 0x004A4A38, 0x004E4E3C, 0x0050503E, 0x004C4D3D, 0x004B4C3C, 0x004B4E3F, 0x004A4D3E, 0x00474A3B, 0x00484E3D, 0x004B5140, 0x00484B3B, 0x00484B3B, 0x004D5040, 0x004E4F3F, 0x004E4F3F, 0x00505141, 0x004B4D3A,
        0x004C4B3D, 0x00494E3F, 0x0045503C, 0x0051533D, 0x004B443B, 0x00233840, 0x000C495D, 0x00084B66, 0x00084B66, 0x00074A65, 0x00094C67, 0x000B4E69, 0x000C4F6A, 0x000A4D68, 0x00094C67, 0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000C4E67,
        0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000A4C65, 0x000B4D66, 0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000B4D66, 0x000A4C65, 0x00094B64, 0x000A4C65, 0x000B4D66, 0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000B4D66, 0x000A4C65, 0x00034C68,
        0x000B4E65, 0x000F4B61, 0x000F4F68, 0x0008516D, 0x0006556A, 0x001D6B71, 0x003B877C, 0x00171618, 0x00000100, 0x001F2219, 0x00333627, 0x003A3D2D, 0x003E3F2F, 0x003F3F31, 0x0039392B, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFEFF, 0xFFFDFCFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x0053574C, 0x00434D3C, 0x00071208, 0x00111210, 0x003A2E2C, 0x0031372C, 0x00548171, 0x0031756E, 0x0012585F, 0x0006506C, 0x00274B51, 0x005B4F45, 0x00644E43, 0x004D6264,
        0x00074851, 0x0021535F, 0x009AA3AC, 0x0094AEA8, 0x004C7C7E, 0x00003F4E, 0x00175B72, 0x0011516A, 0x00105374, 0x003C91BF, 0x00258FC5, 0x002D9BCF, 0x0021758E, 0x00B1E8EB, 0x0071A2AA, 0x00084160, 0x00145179, 0x002D5970, 0x00AEC8C8, 0x00678F8A,
        0x00164A51, 0x00377176, 0x00185755, 0x00005561, 0x004EC0E5, 0x0038A7CD, 0x00096374, 0x00607F80, 0x00638A88, 0x006F8580, 0x00848179, 0x00434337, 0x0037523E, 0x00415640, 0x005E5443, 0x00495346, 0x00585F50, 0x004C4D3D, 0x00797667, 0x00AEA796,
        0x00857F6C, 0x00686550, 0x0053523D, 0x005E5D49, 0x00615D4A, 0x00625B48, 0x00716756, 0x00594F3E, 0x005B5244, 0x005C5748, 0x00585546, 0x005D5F53, 0x00403733, 0x00393736, 0x0087968E, 0x006B7A6C, 0x006B776B, 0x0091B6B4, 0x003B858B, 0x003188B4,
        0x002886B3, 0x0037A0CC, 0x002D9FCE, 0x001E97C9, 0x0027A3D9, 0x0029A4DC, 0x002DA8E2, 0x0029ADE3, 0x003CADE6, 0x002581B6, 0x0045A3D1, 0x00249ACE, 0x0031ACE6, 0x003B8CAD, 0x0092B0B1, 0x002289AA, 0x003D9EC2, 0x005DA8BE, 0x00779F9A, 0x007F9386,
        0x0095A6A2, 0x002F3C3A, 0x005B5C52, 0x005D5D4D, 0x005B5B4B, 0x005B5B4B, 0x00595949, 0x00494939, 0x00333323, 0x00292919, 0x002B2B1B, 0x00343525, 0x00414232, 0x00575848, 0x00595A4A, 0x00575848, 0x00575848, 0x004F5040, 0x004F5040, 0x00545748,
        0x00515445, 0x00525546, 0x004A4D3D, 0x00515343, 0x00585949, 0x00555646, 0x00525343, 0x004F5040, 0x004D4E3E, 0x00545545, 0x00555646, 0x004F5040, 0x00525343, 0x00505243, 0x0046483C, 0x00515445, 0x004E5142, 0x004C4F40, 0x00525444, 0x00545646,
        0x00525343, 0x00505141, 0x00535444, 0x00575848, 0x00585949, 0x00565747, 0x00525343, 0x00525544, 0x00565747, 0x00565747, 0x004F5040, 0x00545545, 0x00555646, 0x00555646, 0x00555646, 0x00555646, 0x00565747, 0x00575848, 0x00585949, 0x00515242,
        0x00515242, 0x00525343, 0x00525343, 0x00535444, 0x00535444, 0x00545545, 0x00545545, 0x00505343, 0x00525545, 0x00535646, 0x00515444, 0x004F5242, 0x004E5141, 0x004F5242, 0x00515444, 0x00434D3D, 0x0022301E, 0x00879988, 0x007C8E81, 0x006E8580,
        0x00749CA1, 0x00397A88, 0x001E7284, 0x001286A7, 0x002597BC, 0x002C9DC8, 0x002399CA, 0x001A99CC, 0x0024A3D6, 0x00269CD0, 0x0033A0D4, 0x00136B99, 0x00324340, 0x005D4F3D, 0x00484C41, 0x00434936, 0x00474739, 0x0040443E, 0x00484A36, 0x00414637,
        0x0044493A, 0x00474C3D, 0x00494E3F, 0x00494E3F, 0x00464B3C, 0x0044493A, 0x00424738, 0x00474A3A, 0x00474A3A, 0x00474A3A, 0x00484B3B, 0x00494C3C, 0x00484B3B, 0x00464939, 0x00444737, 0x00454838, 0x00454838, 0x00464939, 0x00494A3A, 0x00484939,
        0x00484838, 0x00474737, 0x00464636, 0x00464634, 0x004A4A38, 0x004B4B39, 0x00484939, 0x00474838, 0x0046493A, 0x00454839, 0x00434637, 0x00404635, 0x00414736, 0x003F4534, 0x00434636, 0x00484B3B, 0x00474838, 0x00444535, 0x00484939, 0x00484A37,
        0x00474638, 0x00404536, 0x003B4632, 0x00464832, 0x00423B32, 0x001D323A, 0x00074458, 0x00084B66, 0x00084B66, 0x00084B66, 0x00094C67, 0x000B4E69, 0x000C4F6A, 0x000A4D68, 0x00094C67, 0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000C4E67,
        0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000A4C65, 0x000B4D66, 0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000B4D66, 0x000A4C65, 0x000B4D66, 0x000B4D66, 0x000C4E67, 0x000D4F68, 0x000D4F68, 0x000D4F68, 0x000C4E67, 0x000B4D66, 0x0007506C,
        0x000D5067, 0x00114D63, 0x00105069, 0x0008516D, 0x0007566B, 0x001E6C72, 0x003E8A7F, 0x001D1C1E, 0x00000100, 0x001A1D14, 0x002E3122, 0x00383B2B, 0x003E3F2F, 0x003D3D2F, 0x00363628, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x0053574C, 0x00495140, 0x00051006, 0x000F100E, 0x003D312F, 0x00353B30, 0x00538070, 0x0030766F, 0x00135B62, 0x00165B6F, 0x002B5054, 0x00565247, 0x005F5047, 0x005E7676,
        0x000F4D53, 0x002A5E65, 0x00ABB7BD, 0x00C0DACD, 0x00598584, 0x000E4E59, 0x001D6075, 0x00175370, 0x00135278, 0x003588BB, 0x00349BD9, 0x0041A6DA, 0x000A5A77, 0x00346D76, 0x00376975, 0x000F4A64, 0x00104D6D, 0x00235567, 0x007FA4A2, 0x003A7174,
        0x00185C63, 0x0067A3A9, 0x0055888B, 0x00015261, 0x0041C2DD, 0x0036B1CB, 0x0000505C, 0x00195E5B, 0x00628C8B, 0x007A898B, 0x00777F7E, 0x007D8781, 0x00384137, 0x00434C3F, 0x004D5445, 0x00535D4C, 0x00484E3D, 0x00535341, 0x0088826F, 0x00BFB8A4,
        0x00B6B099, 0x0089856D, 0x00646047, 0x005B5C48, 0x005A5945, 0x005A5342, 0x00595140, 0x00635A4C, 0x00615C4D, 0x00535343, 0x006B6B5D, 0x00504541, 0x00342F2E, 0x00889692, 0x005E7C71, 0x00688074, 0x00839D9D, 0x004E93A4, 0x00108DA9, 0x002583B0,
        0x002B90BD, 0x002591C0, 0x002598CA, 0x0028A1D3, 0x0027A3D9, 0x002EA9E1, 0x002FAAE4, 0x0038ACE5, 0x003FA9E6, 0x002781BA, 0x003E9FD1, 0x00279BD4, 0x0037ADE8, 0x00428CAE, 0x0090A5A6, 0x003A7B89, 0x0061A6B9, 0x0076B0BC, 0x0070958D, 0x00738C7E,
        0x0094A9A6, 0x002D3A38, 0x0059574C, 0x005C5C4C, 0x00595949, 0x00585848, 0x005B5B4B, 0x005B5B4B, 0x00565646, 0x00535343, 0x00535343, 0x00515242, 0x00555646, 0x005C5D4D, 0x00575848, 0x00525343, 0x00525343, 0x004F5040, 0x00515242, 0x00535647,
        0x00494C3D, 0x004A4D3E, 0x00505242, 0x00535545, 0x00545545, 0x00545545, 0x00545545, 0x00505141, 0x00535444, 0x00585949, 0x004D4E3E, 0x004C4D3D, 0x00505141, 0x00535546, 0x004E5044, 0x004C4F40, 0x004F5243, 0x004F5243, 0x00525444, 0x00555646,
        0x00535444, 0x00515242, 0x00535444, 0x00575848, 0x00585949, 0x00555646, 0x00525343, 0x004F5040, 0x004D4E3E, 0x00545545, 0x00555646, 0x004F5040, 0x00515242, 0x00535444, 0x00545545, 0x00535444, 0x00535444, 0x00555646, 0x00575848, 0x00555646,
        0x00525343, 0x00505141, 0x00535444, 0x00575848, 0x00585949, 0x00565747, 0x00525343, 0x00505343, 0x00515444, 0x00525545, 0x00535646, 0x00525545, 0x00515444, 0x00505343, 0x00505343, 0x00445141, 0x002F3929, 0x008A9386, 0x0091A297, 0x00607F76,
        0x006B958E, 0x008BB8B5, 0x00648D8F, 0x000F687C, 0x002691AD, 0x001E9FC6, 0x0019A6D9, 0x0020A8E2, 0x001D9BD5, 0x002FA5D8, 0x00269BC8, 0x000C6794, 0x002D403D, 0x005A4E3C, 0x00484C41, 0x00444A37, 0x00474739, 0x003D443D, 0x00444934, 0x00424C3C,
        0x00434D3D, 0x00454F3F, 0x00475141, 0x00485242, 0x00475141, 0x00444E3E, 0x00424C3C, 0x00454D3C, 0x00434B3A, 0x00424A39, 0x00444C3B, 0x00474F3E, 0x00474F3E, 0x00434B3A, 0x003E4635, 0x004C5241, 0x004B4E3E, 0x00464939, 0x00424535, 0x00424333,
        0x00434434, 0x00464636, 0x00484838, 0x00444432, 0x00474735, 0x00474936, 0x00454636, 0x00424535, 0x00434637, 0x00454839, 0x00434839, 0x00444C3B, 0x00454B3A, 0x00434938, 0x00454B3A, 0x004A4D3D, 0x00454838, 0x00404131, 0x00444535, 0x004A4C39,
        0x0049483A, 0x00424738, 0x003D4834, 0x00484A34, 0x00433C33, 0x001D323A, 0x00084559, 0x00094B64, 0x00094B64, 0x00094B64, 0x000A4C65, 0x000C4E67, 0x000C4E67, 0x000B4D66, 0x000A4C65, 0x000B4D66, 0x000B4D66, 0x000B4D66, 0x000B4D66, 0x000B4D66,
        0x000B4D66, 0x000B4D66, 0x000B4D66, 0x000B4D66, 0x000B4D66, 0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000B4D66, 0x000B4D66, 0x000C4E67, 0x000D4F68, 0x000D4F68, 0x000D4F68, 0x000D4F68, 0x000D4F68, 0x000D4F68, 0x000C4E67, 0x0007506C,
        0x000D5067, 0x00134F65, 0x00105069, 0x000A536F, 0x0009586D, 0x001F6D73, 0x00449085, 0x00232224, 0x00000100, 0x00161910, 0x002B2E1F, 0x00363929, 0x003D3E2E, 0x003C3C2E, 0x00313123, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x0053574C, 0x00464E3D, 0x00030E04, 0x0010110F, 0x003D312F, 0x002C3429, 0x00407161, 0x002A756D, 0x00196369, 0x00135365, 0x004E787D, 0x00A2ADA5, 0x00A1A19B, 0x0095B3B4,
        0x004C8C91, 0x00568C93, 0x008EA1A8, 0x00A3B1A6, 0x00779999, 0x00326976, 0x0018546A, 0x001F5774, 0x002A698F, 0x003D90C3, 0x001C86C3, 0x0048B6EA, 0x002987AB, 0x00004C63, 0x000D5468, 0x000A5673, 0x00095978, 0x00065167, 0x000A4D56, 0x00114D5D,
        0x001F6C75, 0x0077ABB2, 0x00A6B3C3, 0x00598298, 0x00188398, 0x0076DBEA, 0x00B1D6E4, 0x00498886, 0x002A5657, 0x00678D8F, 0x00668F91, 0x00677F7F, 0x00878182, 0x00413B36, 0x003D4E43, 0x004D5642, 0x005B5F4C, 0x00565541, 0x00544F3A, 0x00958D76,
        0x00C4BCA5, 0x00AEA990, 0x00A29E82, 0x0050503E, 0x0062604E, 0x00585442, 0x005D5647, 0x005D5849, 0x0059584A, 0x00636657, 0x002C3324, 0x0043312A, 0x0095A49C, 0x0053746D, 0x0059706B, 0x0088A8A7, 0x004F96A4, 0x00299AC2, 0x001D9CD9, 0x001B83B2,
        0x001E87B8, 0x001685B7, 0x002598CB, 0x0035ADE1, 0x0033ADE3, 0x0031ACE4, 0x002BA9E0, 0x0033A5E1, 0x0043AEEB, 0x002387BE, 0x003099CA, 0x001E99D3, 0x0033ACEA, 0x003889AE, 0x00476A6D, 0x006D9393, 0x008BBAC2, 0x007CABAF, 0x006D8F84, 0x00758F82,
        0x0093A9A7, 0x002E3B39, 0x00555244, 0x005D5D4D, 0x005B5B4B, 0x00585848, 0x00585848, 0x005C5C4C, 0x005F5F4F, 0x005F5F4F, 0x005D5D4D, 0x00585949, 0x00585949, 0x00575848, 0x00535444, 0x00505141, 0x00505141, 0x00515242, 0x00535444, 0x004F5243,
        0x004D5041, 0x004D5041, 0x004E5141, 0x00505242, 0x00505141, 0x00535444, 0x00565747, 0x00555646, 0x004F5345, 0x00485343, 0x00485343, 0x004D5241, 0x00535444, 0x00525444, 0x00535549, 0x00474A3B, 0x004E5142, 0x004D5041, 0x00525444, 0x00535444,
        0x00535444, 0x00535444, 0x00545545, 0x00545545, 0x00545545, 0x00545545, 0x00545545, 0x00505141, 0x00535444, 0x00585949, 0x004D4E3E, 0x004C4D3D, 0x00505141, 0x00535444, 0x00545545, 0x00525343, 0x00515242, 0x00535444, 0x00555646, 0x00555646,
        0x00535444, 0x00515242, 0x00535444, 0x00575848, 0x00585949, 0x00555646, 0x00525343, 0x004F5040, 0x00505141, 0x00525343, 0x00545545, 0x00555646, 0x00545545, 0x00515242, 0x004E4F3F, 0x00334534, 0x002B3225, 0x0097988E, 0x007D8C84, 0x005A7E72,
        0x006E9A8D, 0x0079978A, 0x0093A195, 0x0073A4A6, 0x00348894, 0x0030ABCB, 0x0018A5D8, 0x00179DDD, 0x00239BDC, 0x002599D2, 0x00269FD0, 0x00086390, 0x002A3D3A, 0x00574B39, 0x00454B40, 0x00454B38, 0x00474A3B, 0x00404740, 0x00474C37, 0x00495444,
        0x00475242, 0x00455040, 0x00465141, 0x00495444, 0x004B5646, 0x004C5747, 0x004D5848, 0x004C5747, 0x00495444, 0x00465141, 0x00495444, 0x004D5848, 0x004F5A4A, 0x004B5646, 0x00465141, 0x004F5544, 0x004A503F, 0x00434938, 0x00404333, 0x003F4232,
        0x00434434, 0x00474838, 0x00494A3A, 0x00464835, 0x00474936, 0x00474936, 0x00444737, 0x00434636, 0x0044493A, 0x00494E3F, 0x004D5243, 0x004A5142, 0x004A5142, 0x004C5142, 0x00484D3E, 0x004C5142, 0x004F5243, 0x00454839, 0x0046493A, 0x00464835,
        0x00464537, 0x00434839, 0x003E4935, 0x00494B35, 0x00423B32, 0x001A2F37, 0x00023F53, 0x00084A63, 0x00094B64, 0x00094B64, 0x000A4C65, 0x000B4D66, 0x000C4E67, 0x000B4D66, 0x000B4D66, 0x000B4D66, 0x000B4D66, 0x000B4D66, 0x000B4D66, 0x000B4D66,
        0x000B4D66, 0x000B4D66, 0x000B4D66, 0x000B4D66, 0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000B4D66, 0x000D4F68, 0x000D4F68, 0x000D4F68, 0x000D4F68, 0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000C4E67, 0x00034C68,
        0x000A4D64, 0x00134F65, 0x00105069, 0x000B5470, 0x000A596E, 0x001F6D73, 0x0049958A, 0x00232224, 0x00000100, 0x00171A11, 0x002E3122, 0x00383B2B, 0x003D3E2E, 0x003B3B2D, 0x00303022, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x0055594E, 0x004B5140, 0x00060F05, 0x00111210, 0x003A312E, 0x00263024, 0x00356958, 0x002A776F, 0x001B696F, 0x001C5C6E, 0x004C7F89, 0x0099B7B8, 0x0095A8AB, 0x0085AFB4,
        0x004A8F98, 0x0059949E, 0x007592A0, 0x006F8482, 0x008EBAC1, 0x00377789, 0x000B546A, 0x00175D75, 0x000D5976, 0x003799C1, 0x001188B9, 0x002EABDE, 0x004ABFEC, 0x004DB7DC, 0x0057B9DB, 0x004FB1D3, 0x0045ABCE, 0x00238AAA, 0x00107693, 0x00024B69,
        0x00488D9E, 0x00A8DBE5, 0x006A92A4, 0x004E8398, 0x001F606F, 0x0082A9B2, 0x00B6B5BF, 0x00D4DADF, 0x008EC8C7, 0x003E9290, 0x00629B9D, 0x006C8288, 0x00727E80, 0x007E8784, 0x0035352F, 0x00434B3A, 0x00505441, 0x00525240, 0x005C5643, 0x0058513D,
        0x008B856E, 0x00BBB69D, 0x00817D64, 0x00545442, 0x00595745, 0x005C594A, 0x00544F40, 0x005F5E50, 0x00545748, 0x00353F2F, 0x00364033, 0x00929984, 0x00556F61, 0x005B7C78, 0x007B9EA2, 0x0055919D, 0x003399B6, 0x0037B0E8, 0x00279AEB, 0x002696CA,
        0x001888BC, 0x001C8FC2, 0x002EA2D7, 0x003AAFE6, 0x0035AFE5, 0x0029A7DE, 0x0028A7DE, 0x002BA6E0, 0x003AB1E9, 0x001A88B8, 0x002E9EC9, 0x00219FD5, 0x0024A7E6, 0x002890BB, 0x000F515C, 0x005A7C7B, 0x00739EA7, 0x00608D91, 0x006E8E83, 0x007B9486,
        0x008BA29E, 0x0032403C, 0x00595647, 0x00585848, 0x005A5A4A, 0x005A5A4A, 0x00595949, 0x00595949, 0x00595949, 0x00565646, 0x00525242, 0x00505141, 0x00525343, 0x00545545, 0x00575848, 0x00565747, 0x00535444, 0x00515242, 0x004F5040, 0x004F5243,
        0x00545748, 0x004E5142, 0x004A4E3C, 0x004F5243, 0x00515242, 0x00545545, 0x00565747, 0x00565646, 0x004E5040, 0x004D5040, 0x004F5141, 0x00505242, 0x00525444, 0x004F5444, 0x00565D4E, 0x004D5445, 0x0052594C, 0x00494F42, 0x00515546, 0x00525343,
        0x00545545, 0x00555646, 0x00535444, 0x00515242, 0x00505141, 0x00535444, 0x00565747, 0x00555646, 0x004F5345, 0x00485343, 0x00485343, 0x004D5241, 0x00545545, 0x00575848, 0x00575848, 0x00555646, 0x00535444, 0x00545545, 0x00565747, 0x00535444,
        0x00535444, 0x00535444, 0x00545545, 0x00545545, 0x00545545, 0x00545545, 0x00545545, 0x00505141, 0x00505141, 0x00525343, 0x00545545, 0x00565747, 0x00555646, 0x00525343, 0x004F5040, 0x00445443, 0x002C3024, 0x008A8B82, 0x008C9B93, 0x005C8177,
        0x00629083, 0x007C9F91, 0x00748479, 0x007F948C, 0x00114A4C, 0x00208399, 0x002DA6D2, 0x00249BD9, 0x002696DC, 0x00279ADD, 0x001D96D4, 0x000A6592, 0x002B3E3B, 0x00554B39, 0x00454B40, 0x00424B37, 0x00484B3C, 0x00414942, 0x00484F3A, 0x00495444,
        0x00475242, 0x00465141, 0x00475242, 0x004A5545, 0x004D5848, 0x004F5A4A, 0x00505B4B, 0x004A5545, 0x00465141, 0x00434E3E, 0x00455040, 0x004A5545, 0x004C5747, 0x004B5646, 0x00485343, 0x00434B3A, 0x00434938, 0x00414736, 0x00404635, 0x00434636,
        0x00444737, 0x00454636, 0x00444535, 0x00484A37, 0x00484A37, 0x00454936, 0x00444737, 0x00424837, 0x00464B3C, 0x004E5344, 0x00535A4B, 0x00474E3F, 0x004A5142, 0x004C5344, 0x00454A3B, 0x004B5041, 0x00535849, 0x004B4E3F, 0x00494C3D, 0x00474936,
        0x00474638, 0x00434839, 0x003F4A36, 0x00494B35, 0x00433C33, 0x001A2F37, 0x00034054, 0x00084B62, 0x00094C63, 0x000A4D64, 0x000B4E65, 0x000B4E65, 0x000B4E65, 0x000B4E65, 0x000B4E65, 0x000B4D66, 0x000B4D66, 0x000B4D66, 0x000B4D66, 0x000B4D66,
        0x000B4D66, 0x000B4D66, 0x000B4D66, 0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000B4D66, 0x000B4D66, 0x000B4D66, 0x000A4C65, 0x000A4C65, 0x00024B67,
        0x00094C63, 0x00145066, 0x00105069, 0x000B5470, 0x0009586D, 0x001B696F, 0x0049958A, 0x00212022, 0x00000100, 0x001B1E15, 0x00353829, 0x003A3D2D, 0x003D3E2E, 0x003C3C2E, 0x00303022, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00595D52, 0x00555848, 0x000A1309, 0x00121311, 0x003D3431, 0x002D372B, 0x00396F5E, 0x00328178, 0x001B6C71, 0x00266980, 0x002C6C7E, 0x004E7F8D, 0x004D7685, 0x00477E8D,
        0x000F5C6C, 0x00286C7F, 0x005E889F, 0x0050858F, 0x006FB7C8, 0x003B95AD, 0x00036077, 0x00106476, 0x00005567, 0x002087A0, 0x002BA6C6, 0x0022A0C9, 0x0041BDE6, 0x0040B6E0, 0x0055C4EC, 0x005DC7EC, 0x0051BCE1, 0x0042B6DF, 0x0032ABD7, 0x00015D86,
        0x004D8CA1, 0x00AAEFF8, 0x0060CAD5, 0x002A8B99, 0x0088A9B2, 0x00D1CACD, 0x00AFABB0, 0x00A29B9E, 0x009DC1C1, 0x0044908F, 0x00559FA3, 0x00639198, 0x007D9095, 0x00636B6A, 0x0079817A, 0x003B4636, 0x00474E3F, 0x004F5040, 0x005C594A, 0x005A5342,
        0x00645E4B, 0x00696651, 0x00504F3A, 0x005B5B4B, 0x004A4A3A, 0x005B5849, 0x005C5B4D, 0x0058584A, 0x003F4637, 0x00323F31, 0x00809183, 0x005B8468, 0x005A726A, 0x007797A4, 0x003C8397, 0x004FB0CA, 0x003394BA, 0x001675AD, 0x004BB3F8, 0x0031ABDF,
        0x001F96CD, 0x002CA1D8, 0x0037ABE2, 0x0035AAE1, 0x0033AAE2, 0x0027A5DC, 0x002DACE3, 0x0029ACE5, 0x0033ADE3, 0x001783AC, 0x00369CBF, 0x0032A8D9, 0x0022A7EA, 0x002EA7DF, 0x002184A0, 0x003B707D, 0x00457F92, 0x0040737D, 0x006C8B82, 0x00829788,
        0x008DA19C, 0x0032403C, 0x00565747, 0x00585848, 0x00595949, 0x005A5A4A, 0x00595949, 0x00585848, 0x00585848, 0x00595949, 0x00595949, 0x00525343, 0x00545545, 0x00525343, 0x00575848, 0x00575848, 0x00525343, 0x00525343, 0x00505141, 0x004F5243,
        0x004F5243, 0x004B4E3F, 0x004E5141, 0x00535444, 0x00545545, 0x00555646, 0x00565747, 0x00545444, 0x004F5242, 0x004F5242, 0x004F5141, 0x00505141, 0x00505242, 0x004F5242, 0x004E5141, 0x004E5141, 0x004F5242, 0x00505343, 0x00525444, 0x00555646,
        0x00555646, 0x00545545, 0x00525343, 0x00515242, 0x00515242, 0x00545545, 0x00565747, 0x00565646, 0x004E5040, 0x004D5040, 0x004F5141, 0x00505242, 0x00535444, 0x00555646, 0x00555646, 0x00535444, 0x00525343, 0x00535444, 0x00545545, 0x00525343,
        0x00545545, 0x00555646, 0x00535444, 0x00515242, 0x00505141, 0x00535444, 0x00565747, 0x00555646, 0x00545545, 0x00545545, 0x00555646, 0x00575848, 0x00575848, 0x00545545, 0x00525343, 0x00434D40, 0x00252B20, 0x008A9288, 0x0072837A, 0x0062847D,
        0x00659491, 0x00306464, 0x0046797B, 0x006A8686, 0x00215157, 0x00024B5F, 0x002280A3, 0x00389FD0, 0x002994D1, 0x00289CDD, 0x001F98DD, 0x000D6A97, 0x002C413E, 0x00574D3B, 0x00434B40, 0x00414A36, 0x0044493A, 0x00404841, 0x00474E39, 0x00495343,
        0x00495343, 0x004B5545, 0x004D5747, 0x004F5949, 0x00505A4A, 0x00505A4A, 0x004F5949, 0x004B5447, 0x00495245, 0x00475043, 0x00475043, 0x00495245, 0x004B5447, 0x004D5649, 0x004D5649, 0x004C5443, 0x0048503F, 0x00454B3A, 0x00424837, 0x00424837,
        0x00434636, 0x00414434, 0x00404333, 0x00454936, 0x00444835, 0x00444835, 0x00414736, 0x00414736, 0x00434839, 0x004A5142, 0x00535A4B, 0x004B5447, 0x004C5548, 0x004F5649, 0x00434A3D, 0x004A4E42, 0x0052564A, 0x0045493D, 0x0045473B, 0x004A4C39,
        0x00484739, 0x00424738, 0x003C4733, 0x00474933, 0x00423B32, 0x001D323A, 0x00074458, 0x000A4A62, 0x000B4B63, 0x000D4D65, 0x000D4D65, 0x000D4D65, 0x000C4C64, 0x000D4D65, 0x000E4E66, 0x000B4D66, 0x000B4D66, 0x000B4D66, 0x000B4D66, 0x000B4D66,
        0x000B4D66, 0x000B4D66, 0x000B4D66, 0x000D4F68, 0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000D4F68, 0x000D4F68, 0x000C4E67, 0x000B4D66, 0x000A4C65, 0x000A4C65, 0x000A4C65, 0x000A4C65, 0x000A4C65, 0x00044D69,
        0x000B4E65, 0x00165268, 0x000F4F68, 0x0009526E, 0x0006556A, 0x0016646A, 0x0049958A, 0x00242325, 0x00000100, 0x001D2017, 0x00373A2B, 0x003B3E2E, 0x003D3E2E, 0x003D3D2F, 0x00313123, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x005D6156, 0x00565949, 0x000C140A, 0x00151614, 0x00433A37, 0x00364236, 0x00427968, 0x00398A81, 0x001A6E73, 0x001D6381, 0x001E6884, 0x00226782, 0x001F5A74, 0x00256881,
        0x000D637B, 0x0012617C, 0x0023597A, 0x00105A72, 0x00257D9B, 0x0046A7C8, 0x000B647F, 0x00307586, 0x004E8A96, 0x003B808F, 0x003C8FA4, 0x00309FB5, 0x0037A7C5, 0x001A88AC, 0x0002688B, 0x000C6A87, 0x001E7B9A, 0x003AA3CC, 0x0037A9DE, 0x003B9ED0,
        0x00005266, 0x002C888D, 0x0083E3F3, 0x005FA3B6, 0x0098ABAE, 0x00BFC4BB, 0x00869C97, 0x003F544C, 0x00949C9C, 0x00C4DEE4, 0x006AAAAF, 0x00478C8F, 0x00678A8D, 0x006E7F7B, 0x00617B6E, 0x00657368, 0x003F493D, 0x00484C41, 0x0058584C, 0x00514E40,
        0x005F5C4D, 0x00565644, 0x00535542, 0x00525343, 0x00595949, 0x00525242, 0x00646355, 0x002D3021, 0x003C4538, 0x007F8E80, 0x006E8172, 0x00486D5D, 0x008DB2BA, 0x005797B6, 0x002C8EB6, 0x003EA5CC, 0x001F7397, 0x00003F63, 0x00328BAD, 0x002EAFE6,
        0x0027A5DC, 0x0032A9E1, 0x0037AAE3, 0x0034A7E0, 0x0031A8E0, 0x002BA9E0, 0x002FAEE5, 0x002CACE7, 0x0038ADE0, 0x001E7C9F, 0x00267792, 0x003292C0, 0x0027A3EA, 0x002DAFF4, 0x0038AFE0, 0x004196B6, 0x003586A9, 0x00438094, 0x0073928B, 0x00879786,
        0x009AABA2, 0x00303E38, 0x004D5342, 0x005A5A4A, 0x00575747, 0x00555545, 0x00545444, 0x00535343, 0x00535343, 0x00555545, 0x00585848, 0x00585949, 0x00585949, 0x00515242, 0x00545545, 0x00535444, 0x004D4E3E, 0x00535444, 0x00525343, 0x004C4F40,
        0x004D5041, 0x00525445, 0x00535646, 0x00505141, 0x00515342, 0x00515242, 0x00515343, 0x00515343, 0x004F5242, 0x00505343, 0x00505141, 0x00515343, 0x00505242, 0x004E5141, 0x004E5141, 0x004E5141, 0x004F5242, 0x00505343, 0x00515444, 0x00575848,
        0x00565747, 0x00525343, 0x00515242, 0x00525343, 0x00545545, 0x00555646, 0x00565747, 0x00545444, 0x004F5242, 0x004F5242, 0x004F5141, 0x00505141, 0x00505141, 0x004F5141, 0x004F5141, 0x004F5141, 0x00515242, 0x00525343, 0x00535444, 0x00555646,
        0x00555646, 0x00545545, 0x00525343, 0x00515242, 0x00515242, 0x00545545, 0x00565747, 0x00575747, 0x00565646, 0x00555545, 0x00555545, 0x00555545, 0x00555545, 0x00555545, 0x00545444, 0x0046473D, 0x00293126, 0x0089978B, 0x00869990, 0x0063807D,
        0x0051828A, 0x00287585, 0x000C6D81, 0x002D7188, 0x0013536B, 0x00014159, 0x00034E68, 0x002484A2, 0x0037A7D1, 0x001B91C5, 0x00279FDB, 0x000C6996, 0x002C413E, 0x00574D3B, 0x00424A3F, 0x003E4935, 0x00434839, 0x003F4740, 0x00464D38, 0x004C5142,
        0x004E5344, 0x00505546, 0x00515647, 0x00515647, 0x00505546, 0x004F5445, 0x004E5344, 0x0051584B, 0x0051584B, 0x0051584B, 0x0050574A, 0x004E5548, 0x004E5548, 0x0051584B, 0x00535A4D, 0x00565E4D, 0x004E5645, 0x00454D3C, 0x00414736, 0x00404635,
        0x00424535, 0x00424535, 0x00424535, 0x00444835, 0x00434734, 0x00444835, 0x00434938, 0x00414736, 0x003F4637, 0x00454C3D, 0x004D5445, 0x0050594C, 0x00495245, 0x004C5548, 0x00474E41, 0x004B5245, 0x004F5347, 0x003D4135, 0x00414539, 0x00454734,
        0x00434234, 0x003D4233, 0x0038432F, 0x0043452F, 0x003F382F, 0x001A2F37, 0x00044155, 0x00094A5F, 0x000B4C61, 0x000D4E63, 0x000D4E63, 0x000C4D62, 0x000C4D62, 0x000D4E63, 0x000E4F64, 0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000C4E67,
        0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000D4F68, 0x000D4F68, 0x000C4E67, 0x000B4D66, 0x000B4D66, 0x000C4E67, 0x000D4F68, 0x000D4F68, 0x000E5069, 0x000D4F68, 0x000C4E67, 0x000B4D66, 0x000A4C65, 0x000A4C65, 0x000A4C65, 0x000B4D66, 0x00054E6A,
        0x000B4E65, 0x00155167, 0x000C4C65, 0x0008516D, 0x0006556A, 0x0017656B, 0x00509C91, 0x002D2C2E, 0x00000100, 0x00191C13, 0x00343728, 0x00393C2C, 0x003C3D2D, 0x003D3D2F, 0x002F2F21, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00606459, 0x00515444, 0x000B1309, 0x00191A18, 0x00483F3C, 0x003C483C, 0x00467D6C, 0x003C8F86, 0x001A6E73, 0x00236B8D, 0x00136588, 0x00176589, 0x00226689, 0x001C6787,
        0x000B6583, 0x00136888, 0x00205E86, 0x001B6589, 0x00136890, 0x004FA6CE, 0x005398B9, 0x00A5CDE0, 0x00E0F3FF, 0x00D6EEFA, 0x00CCEEFE, 0x0070CBD2, 0x0042A1B4, 0x0052B0CD, 0x001F7691, 0x000F5B71, 0x00115D74, 0x000E668E, 0x0046ACE1, 0x0046A5DD,
        0x00005F6E, 0x001F8689, 0x0099B3D1, 0x00E6D8FA, 0x00B8C8C7, 0x009AC8B2, 0x00234F42, 0x00678278, 0x00A7B8B5, 0x00A3A9B0, 0x00D8E8F4, 0x003E757A, 0x00004942, 0x00598F82, 0x007D877B, 0x007A8981, 0x0067746C, 0x00353D33, 0x0047483E, 0x005A5A4E,
        0x00565648, 0x00525546, 0x00545747, 0x00545545, 0x005C5C4C, 0x00575648, 0x003A3A2C, 0x00383C30, 0x00818B7E, 0x006D7E70, 0x0063776A, 0x00A4ABAE, 0x004A899D, 0x002F98C3, 0x003397D1, 0x004196C8, 0x000E617D, 0x00004B58, 0x0024656D, 0x0027ACE5,
        0x002FADE7, 0x002EA6E1, 0x0032A6DF, 0x0035A8E1, 0x0032A9E1, 0x002CAAE1, 0x0024A5DC, 0x002CA8E4, 0x0047B2E5, 0x00287897, 0x0006445C, 0x001C6A98, 0x002595E1, 0x001A9DEC, 0x0027A9E5, 0x00329DC9, 0x002182AE, 0x00488EA6, 0x0074928D, 0x00828E7C,
        0x00A1AFA4, 0x00303F37, 0x004E5848, 0x00555545, 0x00525242, 0x00535343, 0x00585848, 0x005A5A4A, 0x00575747, 0x00555545, 0x00565646, 0x00555646, 0x00575848, 0x00515242, 0x00565747, 0x00545545, 0x004B4C3C, 0x004F5040, 0x00515242, 0x00545646,
        0x004F5141, 0x00525343, 0x00525343, 0x00515242, 0x00595A4A, 0x00555646, 0x004E5041, 0x004E5141, 0x004F5142, 0x004E5040, 0x004F5040, 0x004D5040, 0x004D5040, 0x004E5141, 0x004E5141, 0x004E5141, 0x004E5241, 0x004E5141, 0x00505242, 0x00545545,
        0x00525343, 0x00535444, 0x00525343, 0x00515242, 0x00515242, 0x00525343, 0x00525343, 0x00515343, 0x004F5242, 0x00505343, 0x00505141, 0x00515343, 0x00515343, 0x004F5141, 0x004F5141, 0x004D5141, 0x00515343, 0x00545646, 0x00575848, 0x00585949,
        0x00565747, 0x00525343, 0x00515242, 0x00525343, 0x00545545, 0x00555646, 0x00565747, 0x00565646, 0x00555545, 0x00535343, 0x00525242, 0x00515141, 0x00525242, 0x00525242, 0x00535343, 0x004B493F, 0x002C3629, 0x00859889, 0x00778C83, 0x00637C7E,
        0x005F92A2, 0x002E8DA7, 0x001598B9, 0x002E96C1, 0x003689AF, 0x00175672, 0x0005495C, 0x00004B5C, 0x0011859E, 0x0030AAD2, 0x001C92C5, 0x00096693, 0x00293E3B, 0x00534C39, 0x00424A3F, 0x003F4A36, 0x0044493A, 0x003D4840, 0x00464D38, 0x004B4E3F,
        0x004C4F40, 0x004B4E3F, 0x00494C3D, 0x00474A3B, 0x0046493A, 0x00484B3C, 0x00494C3D, 0x0045493D, 0x00484C40, 0x004A4E42, 0x00484C40, 0x0043473B, 0x00414539, 0x0043473B, 0x00464A3E, 0x00475140, 0x00444C3B, 0x003F4736, 0x003E4635, 0x00414736,
        0x00424837, 0x00434636, 0x00414434, 0x00444835, 0x00444835, 0x00454B38, 0x00464C3B, 0x00414938, 0x003E4536, 0x0042493A, 0x00475141, 0x00464F42, 0x003A4336, 0x00414A3D, 0x00454C3F, 0x004C5346, 0x004A5144, 0x003B3F33, 0x0045493D, 0x00454734,
        0x00464537, 0x00434839, 0x003F4A36, 0x004B4D37, 0x00453E35, 0x001D323A, 0x00064357, 0x00094A5F, 0x000B4C61, 0x000D4E63, 0x000D4E63, 0x000C4D62, 0x000C4D62, 0x000D4E63, 0x000E4F64, 0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000C4E67,
        0x000C4E67, 0x000C4E67, 0x000C4E67, 0x000E5069, 0x000D4F68, 0x000C4E67, 0x000B4D66, 0x000B4D66, 0x000C4E67, 0x000D4F68, 0x000E5069, 0x0010526B, 0x000F516A, 0x000D4F68, 0x000C4E67, 0x000B4D66, 0x000B4D66, 0x000B4D66, 0x000C4E67, 0x00034C68,
        0x00084B62, 0x00134F65, 0x000A4A63, 0x0007506C, 0x0009586D, 0x001B696F, 0x0059A59A, 0x00353436, 0x00010200, 0x0015180F, 0x002F3223, 0x00363929, 0x003C3D2D, 0x003D3D2F, 0x002E2E20, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFDFFFC, 0xFFFBFEFC, 0xFFFDFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x005D6358, 0x00566759, 0x00191F14, 0x001A140D, 0x003D3633, 0x00424A49, 0x00567E7C, 0x003F8B87, 0x0013766E, 0x00276E89, 0x00236A85, 0x00266D88, 0x00276E89, 0x00216883,
        0x00216883, 0x00236A85, 0x001F6681, 0x00116386, 0x0018758C, 0x007DC5CF, 0x00E0FBFF, 0x00F8FFFF, 0x00F3FFFE, 0x00F3FDFD, 0x00FFFAFF, 0x00F6FFF8, 0x00C2F0EA, 0x0056AEB5, 0x004ABAD1, 0x00087A9E, 0x00006187, 0x00005E81, 0x0046B0CF, 0x004AC3D7,
        0x00005567, 0x006D98A1, 0x00AFBEC0, 0x0096A69F, 0x00A7C1BB, 0x00325551, 0x00547C7B, 0x00BAB3C0, 0x00BFBBC1, 0x008C989E, 0x0082B1BF, 0x0058A8C1, 0x0000405F, 0x001C556E, 0x00819EAD, 0x0075807D, 0x00707B78, 0x00747D7A, 0x0039403B, 0x0040483D,
        0x004C5443, 0x004E5841, 0x004E593F, 0x00675C3C, 0x005A5643, 0x00333A33, 0x002F3933, 0x008F988E, 0x006D7B6F, 0x005B726A, 0x0088A8A7, 0x002E839D, 0x002B7B8C, 0x0032828D, 0x0077CDD9, 0x004AA7BE, 0x001E7A97, 0x00004A64, 0x0019647A, 0x0032ABDC,
        0x0032AADE, 0x0031AAE2, 0x0031A9E5, 0x002FA7E3, 0x002EA6E1, 0x002CA7DF, 0x002DA9DF, 0x0021A1E2, 0x0037ADE8, 0x001879A5, 0x0000445C, 0x0012505A, 0x00246C7E, 0x003899C5, 0x002DA5E7, 0x002B9EC9, 0x001E8CA6, 0x00409196, 0x006F9387, 0x0087937F,
        0x00A4B4A2, 0x00303E38, 0x00525258, 0x00565646, 0x00585848, 0x00585848, 0x00565646, 0x00575747, 0x005A5A4A, 0x005A5A4A, 0x00585848, 0x00535444, 0x00575848, 0x00585949, 0x004B4C3C, 0x004E4F3F, 0x00535444, 0x00535444, 0x00515242, 0x00565747,
        0x00515242, 0x00565747, 0x00545545, 0x004B4C3C, 0x004F5040, 0x00515242, 0x00545646, 0x004F5141, 0x00525443, 0x005A5B4B, 0x00525343, 0x004E5141, 0x004D5040, 0x004E5141, 0x004F5242, 0x004F5444, 0x004D5242, 0x004B5040, 0x004D503F, 0x00535444,
        0x00525343, 0x00515242, 0x00535444, 0x00555646, 0x00555646, 0x00535444, 0x00505141, 0x004F5141, 0x004E5141, 0x004E5040, 0x004F5040, 0x004E5040, 0x004D5040, 0x004D5040, 0x004D5040, 0x004C5241, 0x004C5140, 0x004B4F3E, 0x004F5040, 0x00515242,
        0x00525343, 0x00535444, 0x00525343, 0x00515242, 0x00515242, 0x00525343, 0x00535444, 0x00565646, 0x00555545, 0x00555545, 0x00565646, 0x00575747, 0x00575747, 0x00555545, 0x00535343, 0x004C4746, 0x002A3A2F, 0x007F9583, 0x00838D7D, 0x006D7C74,
        0x00619699, 0x002A8495, 0x001C84A1, 0x001F8CB2, 0x002A92BD, 0x003089B5, 0x00095474, 0x00004354, 0x00045262, 0x00177E97, 0x002BA6CA, 0x00116D90, 0x00213945, 0x003F4239, 0x003A4B40, 0x0041463D, 0x00494533, 0x00484E37, 0x004C4C3E, 0x00444B3C,
        0x00474E3F, 0x0042493A, 0x00444B3C, 0x00434A3B, 0x00414839, 0x00454C3D, 0x0042493A, 0x00484B3C, 0x0046493A, 0x00454839, 0x00484B3C, 0x00474A3B, 0x00444738, 0x00434637, 0x00444738, 0x00454839, 0x00424536, 0x00434637, 0x00434637, 0x00404334,
        0x00424536, 0x00434637, 0x003D4031, 0x00434637, 0x00404334, 0x00404334, 0x00414637, 0x00434839, 0x00414839, 0x00414839, 0x00444B3C, 0x00414637, 0x003F4435, 0x00404536, 0x00414637, 0x00414637, 0x00424738, 0x00424738, 0x003F4435, 0x004B4435,
        0x0043473B, 0x003B4538, 0x003F4534, 0x00454430, 0x002D352A, 0x000F3339, 0x0006455A, 0x0008475B, 0x0009485D, 0x000B4A5F, 0x000D4C61, 0x000F4D65, 0x000F4D65, 0x000F4C66, 0x000F4C66, 0x00044D61, 0x00084F63, 0x000F536A, 0x00034960, 0x00054D65,
        0x00004A64, 0x00074E69, 0x000B506B, 0x00084D68, 0x00024663, 0x000F4F6D, 0x000B4868, 0x000D4868, 0x0011516F, 0x00024865, 0x0004516B, 0x00044E68, 0x00044B66, 0x000B4E67, 0x00084A63, 0x00084860, 0x000B4E65, 0x00074B62, 0x00084D61, 0x00125166,
        0x00164866, 0x001D476A, 0x001A4C70, 0x000A4F6A, 0x000D5F6B, 0x001C6C6B, 0x0058A39B, 0x00354342, 0x00000300, 0x000F1C0E, 0x002F3B27, 0x002C361F, 0x003A3F2A, 0x003C3C2C, 0x00333126, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFCFFFB, 0xFFF9FBFB, 0xFFFCFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00686E63, 0x005B6C5E, 0x00091106, 0x001C1911, 0x00453E3B, 0x003A4241, 0x004F7674, 0x003D857F, 0x0019766F, 0x001A6A89, 0x00186887, 0x001C6C8B, 0x001E6E8D, 0x001A6A89,
        0x001B6B8A, 0x001C6C8B, 0x00186887, 0x0029698B, 0x0044829A, 0x00BAE7F4, 0x00ECFFFF, 0x00E2E4E4, 0x00F3F4F2, 0x00FDF8FA, 0x00FFFBFF, 0x00FFFFFB, 0x00E3F5F6, 0x00AFE5F0, 0x004591A7, 0x0057AAC9, 0x001C6C91, 0x0008597E, 0x004CA0C3, 0x003BB9DC,
        0x000D6A89, 0x00285D72, 0x009CB5BF, 0x00B9C8CA, 0x00637673, 0x00768887, 0x00B7C9CA, 0x0081ACA9, 0x00708E89, 0x00132A26, 0x003B6068, 0x006BADC0, 0x0002536E, 0x0008566D, 0x004B90A1, 0x00698984, 0x00637F79, 0x00647873, 0x00727E78, 0x003C4138,
        0x004B4B3D, 0x00585644, 0x00575340, 0x005E5A42, 0x00404334, 0x002E3734, 0x00869391, 0x0062716D, 0x00637E7B, 0x0074A0A7, 0x00488093, 0x001F819F, 0x00206F82, 0x00528893, 0x007DA4B2, 0x00729CB3, 0x00396F8E, 0x00003A58, 0x00286B84, 0x0032ABDC,
        0x0032AADE, 0x0033AAE2, 0x0031A9E5, 0x0031A9E5, 0x002FAAE4, 0x002DA8E0, 0x0029A5DB, 0x0029AAE9, 0x0032ADE7, 0x00359ECF, 0x00095E7E, 0x0000485A, 0x00085467, 0x00156F91, 0x002790C1, 0x003DAADD, 0x0038A4C6, 0x0051A2B1, 0x00739B9A, 0x00829388,
        0x009BADA0, 0x002F3E36, 0x00585656, 0x005A5A4A, 0x005A5A4A, 0x00585848, 0x00545444, 0x00535343, 0x00575747, 0x00595949, 0x00585848, 0x00535444, 0x00545545, 0x00565747, 0x00535444, 0x00535444, 0x00515242, 0x00535343, 0x00535444, 0x00575848,
        0x00585949, 0x004B4C3C, 0x004E4F3F, 0x00535444, 0x00535444, 0x00515242, 0x00565747, 0x00515242, 0x00565747, 0x00545545, 0x004B4C3C, 0x004F5040, 0x004E5141, 0x00505343, 0x00505343, 0x004F5544, 0x004D5342, 0x004A503F, 0x004A4F3E, 0x00525444,
        0x00525343, 0x00525343, 0x00545545, 0x00575848, 0x00575848, 0x00535444, 0x00505141, 0x004F5242, 0x00505242, 0x004D4E3E, 0x00515343, 0x004D5040, 0x004D5040, 0x004E5141, 0x004E5141, 0x004F5544, 0x004C5241, 0x004A4F3F, 0x004C4F3E, 0x00525444,
        0x00525343, 0x00515242, 0x00525343, 0x00545545, 0x00545545, 0x00525343, 0x00505141, 0x00575747, 0x00565646, 0x00565646, 0x00565646, 0x00575747, 0x00575747, 0x00565646, 0x00565646, 0x004D4441, 0x0029372C, 0x007A9182, 0x007C8D80, 0x00647C7A,
        0x005996A2, 0x0023809A, 0x00117BA0, 0x00118AB6, 0x001D93C2, 0x002791C0, 0x00338CB1, 0x000D5970, 0x00004A5A, 0x00004D63, 0x0022839F, 0x001C7590, 0x0034464D, 0x004B493E, 0x0037483B, 0x0041443B, 0x004C4836, 0x00494E39, 0x00494B3F, 0x004D5445,
        0x004E5546, 0x00484F40, 0x004A5142, 0x00495041, 0x00464D3E, 0x004A5142, 0x00484F40, 0x004C5142, 0x004C5142, 0x004D5243, 0x004F5445, 0x004E5344, 0x004A4F40, 0x00484D3E, 0x004A4F40, 0x00525748, 0x004D5243, 0x004F5445, 0x00505546, 0x004D5243,
        0x004C5142, 0x004B5041, 0x00474C3D, 0x004B4E3F, 0x00474A3B, 0x00454839, 0x00464B3C, 0x00474C3D, 0x00444B3C, 0x00444B3C, 0x00454C3D, 0x00474C3D, 0x00484D3E, 0x00474C3D, 0x00474C3D, 0x00484D3E, 0x00454A3B, 0x00414637, 0x003F4435, 0x004C4536,
        0x0044483C, 0x003B4538, 0x00404635, 0x00464531, 0x002F372C, 0x0011353B, 0x0007465B, 0x0008475B, 0x000A495E, 0x000C4B60, 0x000D4C61, 0x000F4D65, 0x000F4D65, 0x000F4C66, 0x000F4C66, 0x00114D65, 0x00145068, 0x00175069, 0x00114B61, 0x0018546A,
        0x00145066, 0x0019566A, 0x000D485C, 0x00144E64, 0x00174F68, 0x00154A65, 0x001C4E6C, 0x00184A68, 0x00134A65, 0x001A566E, 0x00104F64, 0x00064C6A, 0x00074B68, 0x000C4E6B, 0x0011516F, 0x000E4F6B, 0x000A4B67, 0x000E5069, 0x0010526B, 0x000A586F,
        0x000B4D6A, 0x000F496C, 0x000C4C6F, 0x00004D68, 0x00015967, 0x00126769, 0x004E9B97, 0x00432E31, 0x00120000, 0x00271B11, 0x00403729, 0x00393523, 0x00404030, 0x0036392A, 0x002C2D23, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFDFFFE, 0xFFFBFDFD, 0xFFFBFDFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00686E63, 0x006A7B6E, 0x00060E04, 0x001C1914, 0x0045423E, 0x003B4342, 0x00597A76, 0x0045827E, 0x0024726B, 0x00247491, 0x0020708D, 0x001E6E8B, 0x001C6C89, 0x00186885,
        0x00196986, 0x001B6B88, 0x00196986, 0x00265D78, 0x007CA3B9, 0x00E4FEFF, 0x00BFD6D8, 0x00A8B5B3, 0x00FDF9F8, 0x00FFFEFF, 0x00F9FDFE, 0x00E6F4F3, 0x00DCFEFE, 0x00C6FFFF, 0x0043929B, 0x0053A8B8, 0x002F879F, 0x00005674, 0x0051AECD, 0x0050C7F4,
        0x0000587F, 0x000C4A68, 0x00567A8A, 0x0092A8AE, 0x00A8B5B7, 0x008F9797, 0x00C4C8C9, 0x0076A5A3, 0x000E3A34, 0x003F7068, 0x005B9999, 0x001A6A75, 0x00004056, 0x001C6682, 0x003F7B99, 0x00114A42, 0x005A8A84, 0x006E928C, 0x005E726D, 0x007A817A,
        0x0037312A, 0x005F544C, 0x0051443C, 0x003D4135, 0x00252D26, 0x0094A29E, 0x0063726E, 0x005F7672, 0x00709A9F, 0x004588A1, 0x003288B2, 0x00329EC0, 0x0023768B, 0x004F838A, 0x0027474C, 0x00294857, 0x00244D66, 0x00053B5A, 0x00296583, 0x0035ACDD,
        0x0036ACE0, 0x0033AAE2, 0x002EA6E2, 0x0030A8E4, 0x0031ACE6, 0x002CAAE1, 0x0027A5DB, 0x0027A7E2, 0x0028A6E1, 0x0038ACE5, 0x00349DC9, 0x00035D7C, 0x00004358, 0x00277D91, 0x002A8398, 0x002C8AB8, 0x003797BB, 0x004793A9, 0x00658E97, 0x00778D8B,
        0x008EA69C, 0x002E3E33, 0x0057544F, 0x00595949, 0x005A5A4A, 0x00585848, 0x00545444, 0x00535343, 0x00565646, 0x00575747, 0x00555545, 0x00565747, 0x00515242, 0x004C4D3D, 0x00525343, 0x00505141, 0x00535343, 0x00565747, 0x00535444, 0x00545545,
        0x00565747, 0x00535444, 0x00535444, 0x00515242, 0x00535343, 0x00535444, 0x00575848, 0x00585949, 0x004B4C3C, 0x004E4F3F, 0x00535444, 0x00535444, 0x004F5141, 0x004F5242, 0x004F5242, 0x004D5342, 0x004D5342, 0x004C5241, 0x00525746, 0x00515443,
        0x00535444, 0x00525343, 0x00555646, 0x00545545, 0x00535444, 0x00505141, 0x004E4F3F, 0x00525444, 0x00535545, 0x00515343, 0x00515343, 0x00505343, 0x00505343, 0x00505343, 0x00505343, 0x004F5544, 0x004D5342, 0x004A503F, 0x004A4F3E, 0x00525444,
        0x00525343, 0x00505141, 0x00525343, 0x00565747, 0x00575848, 0x00535444, 0x004F5040, 0x00565747, 0x00565747, 0x00565747, 0x00555646, 0x00555646, 0x00555646, 0x00565747, 0x00575848, 0x004F4341, 0x00263429, 0x00769082, 0x00789086, 0x005F8383,
        0x005CA0AD, 0x00338DAB, 0x002F8BB4, 0x001191BC, 0x000888B9, 0x00239CCE, 0x002794C0, 0x00328CAE, 0x00065269, 0x00004057, 0x0004475E, 0x0007586D, 0x00323E3E, 0x00524C39, 0x003C4836, 0x0044463A, 0x004E4A38, 0x00454B38, 0x0043463D, 0x004F5647,
        0x00505748, 0x004A5142, 0x004B5243, 0x004A5142, 0x00474E3F, 0x004B5243, 0x00495041, 0x00485242, 0x00485242, 0x004B5545, 0x004E5848, 0x004C5646, 0x00475141, 0x00454F3F, 0x00485242, 0x00465040, 0x003F4939, 0x00424C3C, 0x00495343, 0x00475141,
        0x00454F3F, 0x00454F3F, 0x00444E3E, 0x00484B3C, 0x00434637, 0x00434637, 0x00464B3C, 0x004C5142, 0x004C5344, 0x004C5344, 0x004D5445, 0x00494E3F, 0x004E5344, 0x004C5142, 0x004C5142, 0x00505546, 0x004A4F40, 0x00404536, 0x00404536, 0x004D4637,
        0x0044483C, 0x003B4538, 0x00404635, 0x00484733, 0x0031392E, 0x0011353B, 0x0006455A, 0x0009485C, 0x000A495E, 0x000C4B60, 0x000E4D62, 0x000F4D65, 0x000F4D65, 0x000F4C66, 0x000E4B65, 0x00044C64, 0x00004961, 0x00004B5E, 0x00226E80, 0x00378290,
        0x00055059, 0x000D5B62, 0x0028797E, 0x000B535F, 0x00135867, 0x000B4D60, 0x0004495D, 0x00054E62, 0x00024C5E, 0x00014C5A, 0x00044C58, 0x0001506B, 0x00004C67, 0x00004B65, 0x0007546E, 0x0007536A, 0x00004C63, 0x000B576E, 0x00014C62, 0x00004B61,
        0x00004C66, 0x000E5472, 0x000E5471, 0x00044F65, 0x00065560, 0x001E696B, 0x00589E98, 0x001C201B, 0x00000200, 0x001B1F14, 0x00313425, 0x00353527, 0x00434136, 0x003B362D, 0x00302A25, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFF4F9F7, 0xFFF1F6F5, 0xFFEAEEEF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00676F65, 0x0057675C, 0x00141C15, 0x0022241E, 0x003D3E3C, 0x00323A39, 0x004A6562, 0x0041706D, 0x002D6B65, 0x00326D81, 0x002F6A7E, 0x002D687C, 0x002C677B, 0x002D687C,
        0x002F6A7E, 0x00326D81, 0x00326D81, 0x00326B7A, 0x0099B8C7, 0x00DEF5FD, 0x008FB6B4, 0x008FB1AB, 0x00F0F9F6, 0x00E7F2F0, 0x00C1E1DC, 0x009BC9CA, 0x009DD3D3, 0x0093D5D0, 0x0057A39F, 0x00368588, 0x004797A2, 0x0015667B, 0x005DB1CB, 0x0067C1E3,
        0x00216F8C, 0x00286579, 0x004A7782, 0x00628287, 0x00A2BABA, 0x00859796, 0x005C6C6B, 0x002B566B, 0x00033444, 0x006DAAB4, 0x003D868E, 0x004396A5, 0x00166881, 0x00175E80, 0x004885AD, 0x0065ADAD, 0x0069A7A7, 0x005D8C8A, 0x0061807F, 0x006A7C7B,
        0x0061686B, 0x0014181D, 0x0033363B, 0x00071612, 0x0064736F, 0x006C7C75, 0x0065766B, 0x008BA69C, 0x00669AA1, 0x004B9FC2, 0x001A85BE, 0x002D96C1, 0x0067C4DB, 0x0089D5D4, 0x001C5F56, 0x00003D3E, 0x00044C5D, 0x00004663, 0x001D6283, 0x0035ABDC,
        0x0038ACE1, 0x0035A9E2, 0x002EA3E0, 0x0029A3DF, 0x002CA9E3, 0x002DACE3, 0x002AAADF, 0x002CAAE4, 0x0023A5E1, 0x00219FDF, 0x0039B1EC, 0x002893BF, 0x0026839D, 0x0082D1DE, 0x0089D3D7, 0x00599BB4, 0x0067B0C4, 0x00619EAA, 0x0073959B, 0x00809593,
        0x0092A9A1, 0x002F3F34, 0x00514F44, 0x00555545, 0x00585848, 0x00595949, 0x00575747, 0x00575747, 0x00575747, 0x00555545, 0x00717164, 0x00DCDCD9, 0x00F6F6F5, 0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4,
        0x00F4F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F4F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4,
        0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F4F5F4, 0x00F5F5F4, 0x00F5F5F4,
        0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F5F5F4, 0x00F6F6F5, 0x00DFE0DD, 0x00747568, 0x00555646, 0x00555646, 0x00555646, 0x00555646, 0x00565747, 0x00534745, 0x00263127, 0x00768D7F, 0x00768C80, 0x00547971,
        0x00549193, 0x00418A98, 0x005296AD, 0x00208CAE, 0x002294BC, 0x001890BE, 0x001C92C3, 0x00228DB9, 0x002C85AA, 0x00135C7A, 0x0003425E, 0x00004350, 0x002C332E, 0x00554A34, 0x003F4A36, 0x0048483C, 0x004C4836, 0x00404935, 0x0043483F, 0x004C5344,
        0x004E5546, 0x00495041, 0x004B5243, 0x004B5243, 0x00484F40, 0x004C5344, 0x00495041, 0x00454F3F, 0x00465040, 0x00495343, 0x004B5545, 0x00485242, 0x00434D3D, 0x00444E3E, 0x00475141, 0x00505A4A, 0x00475141, 0x00495343, 0x004F5949, 0x004A5444,
        0x00434D3D, 0x00414B3B, 0x00404A3A, 0x004B4E3F, 0x00454839, 0x00434637, 0x00464B3C, 0x004B5041, 0x004A5142, 0x00495041, 0x00495041, 0x00404536, 0x00494E3F, 0x00474C3D, 0x00464B3C, 0x004E5344, 0x00474C3D, 0x003B4031, 0x003D4233, 0x004C4536,
        0x0042463A, 0x00394336, 0x003F4534, 0x00484733, 0x0031392E, 0x0011353B, 0x00044358, 0x0009485C, 0x000A495E, 0x000C4B60, 0x000E4D62, 0x000F4D65, 0x000F4D65, 0x000E4B65, 0x000E4B65, 0x000F546F, 0x00004A61, 0x00004F61, 0x0054A3B0, 0x007DC5CD,
        0x00175C5F, 0x00277270, 0x007FD1CC, 0x00296F6F, 0x000C4E53, 0x001B5B66, 0x00084D5C, 0x00004E5B, 0x0021727A, 0x002A7377, 0x003B7B7C, 0x00417B8E, 0x003A7686, 0x00104E5E, 0x00195767, 0x003E7D8B, 0x002D6877, 0x002F6975, 0x00386F7C, 0x0017616D,
        0x000B4E5D, 0x00044050, 0x00073E4D, 0x001C4F59, 0x00356567, 0x00426E67, 0x00588277, 0x00363930, 0x0021241B, 0x00272A21, 0x0024271E, 0x0023261D, 0x0031332D, 0x002A2B27, 0x00252624, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0x00C7CCCB, 0x00CFD3D4, 0x00D3D6DA, 0x00C6CACB, 0x00CFD4D3, 0x00BCC3BC, 0x00BAC2B8, 0x00AFBCB4, 0x00A9B3AD, 0x00AFB5B0, 0x00B6B9B7, 0x00A4ADAA, 0x00AABFBC, 0x00AECFCB, 0x00ADD6D1, 0x00ACC6CC, 0x00ABC5CB, 0x00A9C3C9, 0x00A9C3C9, 0x00ABC5CB,
        0x00AAC4CA, 0x00A7C1C7, 0x00A6C0C6, 0x0099C0C8, 0x00C2D4DB, 0x00DCEBEE, 0x00C1E4E0, 0x00CCEEE8, 0x00E6F4F0, 0x00C7D5D1, 0x00A5CAC2, 0x00B2D0D5, 0x00A9C7C8, 0x00B3CECB, 0x00ACC7C3, 0x00A2BCBC, 0x00B1CBD2, 0x00A4BACC, 0x00ADBED8, 0x0092C5CF,
        0x0061959C, 0x003D7072, 0x003B6A68, 0x0039615F, 0x0029504E, 0x006C9492, 0x0053807D, 0x0025759A, 0x004896BA, 0x000A5172, 0x00034562, 0x000C516C, 0x003E8EAB, 0x00187A9C, 0x0031A2CA, 0x007CC7D5, 0x00357A84, 0x00518A93, 0x005C8A92, 0x00385E6A,
        0x001B4251, 0x002A5467, 0x00002138, 0x0000181B, 0x00001C1D, 0x00445E58, 0x00869D8E, 0x0084A192, 0x0077AEB1, 0x003089AB, 0x002598D1, 0x003AA3D6, 0x001F7B98, 0x0056A3A5, 0x0099E2D8, 0x006FC0BD, 0x0037909E, 0x0016738C, 0x00036381, 0x002FA2D4,
        0x0035A7DC, 0x0035A9E2, 0x0030A5E2, 0x0029A3DF, 0x0029A6E0, 0x0029AAE1, 0x0029ABE0, 0x0028A6E0, 0x0028A7E4, 0x0023A5E6, 0x001D9FE0, 0x002DA6DE, 0x002E98BD, 0x00338898, 0x002B7275, 0x00668F91, 0x0076AAAA, 0x00679693, 0x00728A88, 0x007E8D89,
        0x008FA49B, 0x00304134, 0x00535347, 0x00535343, 0x00575747, 0x00585848, 0x00575747, 0x00565646, 0x00575747, 0x00555545, 0x00DBDBD8, 0x00D7D7D3, 0x008B8C81, 0x0085867A, 0x0088897E, 0x00828377, 0x0085857A, 0x00838377, 0x0088887D, 0x0087887C,
        0x00838479, 0x008B8B80, 0x0086867B, 0x0086867A, 0x0086867A, 0x0087887C, 0x00838479, 0x00808175, 0x0084857A, 0x00838378, 0x0085857A, 0x0087887C, 0x0085867A, 0x0085867A, 0x0088887D, 0x00818378, 0x0083857A, 0x0083867A, 0x0085877B, 0x00828378,
        0x0086867B, 0x0086877C, 0x0086867B, 0x00838479, 0x00838378, 0x00828377, 0x00848479, 0x007D7D72, 0x00818176, 0x00838479, 0x00828479, 0x00818378, 0x00818378, 0x00818377, 0x00818377, 0x007F8377, 0x00818579, 0x0081867A, 0x0083877A, 0x0083857A,
        0x0086867B, 0x0086867B, 0x0084857A, 0x00838378, 0x00838479, 0x0086867B, 0x0088887D, 0x0086877C, 0x00D6D6D2, 0x00DBDCD8, 0x00555646, 0x00555646, 0x00545545, 0x00525343, 0x00515242, 0x00504946, 0x00283127, 0x007F9082, 0x00839382, 0x005D7868,
        0x005B8E80, 0x005A9188, 0x007FA7A6, 0x00156479, 0x002E87A2, 0x002890B5, 0x00198EBB, 0x001389BC, 0x002796C8, 0x002A89B4, 0x00267A9E, 0x000C5A67, 0x00363D38, 0x00554A34, 0x003D4834, 0x0047473B, 0x00494533, 0x003F4834, 0x00464B42, 0x00484F40,
        0x004C5344, 0x00495041, 0x004D5445, 0x004E5546, 0x004B5243, 0x004E5546, 0x004B5243, 0x00485242, 0x00485242, 0x00495343, 0x004A5444, 0x00475141, 0x00444E3E, 0x00465040, 0x004B5545, 0x00515B4B, 0x00485242, 0x00495343, 0x004F5949, 0x004B5545,
        0x00434D3D, 0x00414B3B, 0x00404A3A, 0x004A4D3E, 0x00444738, 0x00424536, 0x00454A3B, 0x004A4F40, 0x00495041, 0x00484F40, 0x00484F40, 0x00424738, 0x004B5041, 0x00484D3E, 0x00484D3E, 0x00505546, 0x00494E3F, 0x003D4233, 0x00404536, 0x00494233,
        0x00404438, 0x00374134, 0x003E4433, 0x00474632, 0x0030382D, 0x000F3339, 0x00034257, 0x000A495D, 0x000B4A5F, 0x000C4B60, 0x000E4D62, 0x000E4C64, 0x000E4C64, 0x000E4B65, 0x000D4A64, 0x001B3E60, 0x001F4E6A, 0x0010465D, 0x004C7F8F, 0x0096BCC8,
        0x006E8E93, 0x007FA9A8, 0x0092C8C1, 0x0096BBB9, 0x00537176, 0x007693A1, 0x006991A3, 0x00457989, 0x007EB5BE, 0x00ACD2D6, 0x00AFC7C7, 0x0089A0A8, 0x007C969C, 0x003C5A5F, 0x005A7D81, 0x00C6E7EA, 0x00BDDADE, 0x00AAC1C3, 0x00AEC1C4, 0x0095C1C2,
        0x0086AFB1, 0x006B8F8F, 0x00496461, 0x0044534F, 0x004D554A, 0x00545747, 0x005A5C48, 0x00635D52, 0x005A594F, 0x005B695E, 0x00557269, 0x005A867F, 0x0069A69C, 0x0063ACA2, 0x005DAFA3, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0x008A8F8E, 0x00AFB3B4, 0x00E4E7EB, 0x00E6E9ED, 0x00E9EEED, 0x00EEF4EF, 0x00FBFFF9, 0x00F2FEF8, 0x00F8FFFC, 0x00F0F7F4, 0x00F2F9F6, 0x00F2FBF8, 0x00F3FFFF, 0x00EDFFFF, 0x00E3FAF6, 0x00FBFFFC, 0x00F9FFFA, 0x00EFF5F0, 0x00E9EFEA, 0x00E9EFEA,
        0x00E7EDE8, 0x00E4EAE5, 0x00E4EAE5, 0x00EDF6F9, 0x00ECEEEF, 0x00EFF1F2, 0x00EEFBF9, 0x00ECFAF6, 0x00E5EAE8, 0x00C1C8C5, 0x0091A19A, 0x009AAEB3, 0x00A1ADAF, 0x00A2A5A3, 0x00ABAAA6, 0x00ADAEAA, 0x00B3B2B4, 0x00B8AFB9, 0x00A1909E, 0x006A837F,
        0x0032534F, 0x0029514C, 0x00345D58, 0x00608785, 0x00658D8B, 0x00215450, 0x0018534F, 0x00186C90, 0x006AC7EE, 0x0051B8DF, 0x003297BD, 0x002382A3, 0x000D7090, 0x001B8FB2, 0x001B9FC7, 0x002F82A1, 0x00004A65, 0x003B8199, 0x003C7F94, 0x00054760,
        0x00165F7D, 0x003B8EB4, 0x0019729E, 0x00256479, 0x00023A4B, 0x001B4A4E, 0x0071968E, 0x006F9589, 0x00195254, 0x003E95B5, 0x002594CC, 0x0036A6DB, 0x000A6084, 0x001F5762, 0x0076A2A3, 0x0044787F, 0x002A7387, 0x003394AE, 0x001F8FAD, 0x002697C9,
        0x002B9DD2, 0x0032A5DE, 0x0033A8E5, 0x002EA8E4, 0x0028A6E0, 0x0025A6DD, 0x0024A8DD, 0x0027A3DF, 0x002AA6E2, 0x0027A6E3, 0x001FA2E1, 0x0029AAE9, 0x0038ABDE, 0x003695B6, 0x00004B5E, 0x00436D6C, 0x005B918C, 0x005B8A82, 0x006D857D, 0x0079897E,
        0x008EA295, 0x002E3F32, 0x00545349, 0x00575747, 0x00585848, 0x00575747, 0x00535343, 0x00535343, 0x00565646, 0x00575747, 0x00F9F9F8, 0x00838479, 0x00545545, 0x004F5040, 0x004E4F3F, 0x004D4E3E, 0x00535343, 0x00525242, 0x00555646, 0x00585949,
        0x00535444, 0x00585949, 0x004F5040, 0x00535343, 0x00505040, 0x00575848, 0x00565747, 0x00515242, 0x005B5C4C, 0x00545545, 0x00545444, 0x00545444, 0x00565747, 0x00525343, 0x00535444, 0x00505343, 0x00525545, 0x00535646, 0x00535645, 0x00515242,
        0x00535444, 0x00545545, 0x00535444, 0x00515242, 0x00515242, 0x00505141, 0x00525242, 0x00515141, 0x004D4E3E, 0x004F4F3F, 0x00525343, 0x004F5040, 0x004E4F3F, 0x00525343, 0x00575848, 0x004E5141, 0x00505343, 0x00515444, 0x00535645, 0x004F5141,
        0x00545545, 0x00555646, 0x00535444, 0x004F5040, 0x004F5040, 0x00535444, 0x00585949, 0x004F5242, 0x00828479, 0x00F9F9F8, 0x00535646, 0x00545747, 0x00525545, 0x004E5141, 0x004B4E3E, 0x0047483F, 0x00263024, 0x00839082, 0x00869080, 0x0059715F,
        0x00538371, 0x00497B6F, 0x00678881, 0x00024050, 0x001C6476, 0x00368DA7, 0x00238EB4, 0x001F97CB, 0x000781B7, 0x001E8DBD, 0x00298EB5, 0x001D6E83, 0x00354141, 0x004C4633, 0x003B4735, 0x0046483C, 0x004A4634, 0x00404633, 0x0043463D, 0x00414839,
        0x00454C3D, 0x00434A3B, 0x00474E3F, 0x00484F40, 0x00464D3E, 0x004A5142, 0x00474E3F, 0x004B5041, 0x00494E3F, 0x00494E3F, 0x004A4F40, 0x00484D3E, 0x00464B3C, 0x00494E3F, 0x004D5243, 0x004F5445, 0x00484D3E, 0x004B5041, 0x00525748, 0x00505546,
        0x004D5243, 0x004E5344, 0x004C5142, 0x00474A3B, 0x00424536, 0x00414435, 0x0044493A, 0x00494E3F, 0x00495041, 0x00495041, 0x004A5142, 0x00464B3C, 0x004B5041, 0x00494E3F, 0x00484D3E, 0x004C5142, 0x00484D3E, 0x003F4435, 0x00414637, 0x00474031,
        0x003E4236, 0x00374134, 0x003D4332, 0x00464531, 0x002F372C, 0x000E3238, 0x00024156, 0x000A495D, 0x000B4A5F, 0x000C4B60, 0x000E4D62, 0x000E4C64, 0x000E4C64, 0x000D4A64, 0x000D4A64, 0x00214767, 0x001D526D, 0x00064559, 0x001F5867, 0x004B717D,
        0x0028484E, 0x0040696B, 0x00316A67, 0x00476C64, 0x00254344, 0x0044626D, 0x00467283, 0x0022616F, 0x003B797F, 0x00638B8A, 0x00697A76, 0x00325453, 0x00375E5C, 0x0027554F, 0x002D6059, 0x00558880, 0x00628E87, 0x00779B95, 0x005E7D76, 0x00586A69,
        0x005B706D, 0x005B6D66, 0x004D554B, 0x00564E41, 0x00655140, 0x006A5540, 0x0063513A, 0x00535343, 0x004E5246, 0x00516058, 0x00536E6A, 0x005C8685, 0x0067A1A0, 0x0062A8A2, 0x005CAAA3, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0x005E6463, 0x00767B7E, 0x00B6BABF, 0x00C5CACD, 0x00CED4D3, 0x00DDE4DF, 0x00ECF4ED, 0x00E5F0ED, 0x00D6E1DE, 0x00CAD5D2, 0x00D3DCD9, 0x00D9E2DF, 0x00E9F2EF, 0x00DAE5E2, 0x00CCD7D4, 0x00D2D9D2, 0x00CFD6CF, 0x00C5CCC5, 0x00C0C7C0, 0x00C3CAC3,
        0x00C3CAC3, 0x00C1C8C1, 0x00C3CAC3, 0x00E5E0DD, 0x00E2E8E3, 0x00E0E7E4, 0x00D7DAD8, 0x00CFD2D0, 0x00C6D4D0, 0x00A5B3AD, 0x00717570, 0x00799191, 0x00919F9D, 0x00878C8A, 0x00868D86, 0x00798880, 0x00798A81, 0x00787F7A, 0x007A7675, 0x00374F55,
        0x00426267, 0x004B7074, 0x00547479, 0x00647C82, 0x009EB7BB, 0x005C8282, 0x006EA09C, 0x0070A7B4, 0x00499CB1, 0x0043B3D1, 0x004BC2E2, 0x003BA9C7, 0x002D90B0, 0x00359EC7, 0x002394C6, 0x002C91BE, 0x001C7BA6, 0x00136C91, 0x003E95B7, 0x0049A2C7,
        0x003095C2, 0x0031A2DA, 0x001C95D3, 0x0040A0CE, 0x00217BA4, 0x00004864, 0x001E5B65, 0x00275D5E, 0x00003E49, 0x00227C9F, 0x002A97CF, 0x002793CD, 0x002D82AA, 0x00084053, 0x00173F4B, 0x00174354, 0x0003435C, 0x00045675, 0x003895B4, 0x002594C6,
        0x002595CA, 0x00289BD4, 0x0030A5E2, 0x002FA9E5, 0x0028A6E0, 0x0022A6DC, 0x0021A8DC, 0x002CA4E5, 0x002DA5E0, 0x0029A3D9, 0x002DABE6, 0x001FA1E2, 0x00239EDE, 0x0041ABE0, 0x0004608B, 0x0023687B, 0x003C8692, 0x0048878B, 0x00648483, 0x00758780,
        0x008DA496, 0x002E3C30, 0x00514F45, 0x005A5A4A, 0x005A5A4A, 0x00575747, 0x00535343, 0x00525242, 0x00555545, 0x00575747, 0x00F5F5F4, 0x00828377, 0x00535444, 0x00555646, 0x00535343, 0x00565646, 0x00575747, 0x00565646, 0x00515242, 0x00555646,
        0x004F5040, 0x004E4F3F, 0x004D4E3E, 0x00535343, 0x00525242, 0x00555646, 0x00585949, 0x00535444, 0x00585949, 0x004F5040, 0x00535343, 0x00505040, 0x00575848, 0x00565747, 0x00525343, 0x00505343, 0x00515444, 0x00525545, 0x00555848, 0x00515343,
        0x00515242, 0x00525343, 0x00545545, 0x00555646, 0x00545545, 0x00515242, 0x004D4D3D, 0x00505040, 0x00515141, 0x004F5040, 0x00555646, 0x00535444, 0x00535444, 0x00545545, 0x00545545, 0x00505343, 0x00525545, 0x00535646, 0x00535645, 0x00515242,
        0x00535444, 0x00545545, 0x00535444, 0x00515242, 0x00515242, 0x00525343, 0x00545545, 0x00535646, 0x00828479, 0x00F5F5F4, 0x00525545, 0x00545747, 0x00535646, 0x004E5141, 0x004A4D3D, 0x00424738, 0x00263023, 0x00889085, 0x00899089, 0x005E7975,
        0x005A9493, 0x00418185, 0x00457680, 0x00003D51, 0x000B5567, 0x000E5F74, 0x00298AAB, 0x001D8CBA, 0x001C91C4, 0x001885B1, 0x00258BAE, 0x000E6782, 0x00273940, 0x00444237, 0x0037483B, 0x0043463D, 0x004B4735, 0x00444934, 0x00404236, 0x003D4435,
        0x00404738, 0x003C4334, 0x003F4637, 0x00404738, 0x003F4637, 0x00444B3C, 0x0042493A, 0x0049493B, 0x00474739, 0x00474739, 0x0049493B, 0x0048483A, 0x00464638, 0x00474739, 0x004B4B3D, 0x0049493B, 0x00444436, 0x00454537, 0x00474739, 0x00444436,
        0x00434335, 0x00424234, 0x003E3E30, 0x00494C3D, 0x00454839, 0x00434637, 0x00424738, 0x00434839, 0x00404738, 0x003E4536, 0x003F4637, 0x003E4334, 0x003E4334, 0x003C4132, 0x003C4132, 0x003E4334, 0x003D4233, 0x003B4031, 0x003A3F30, 0x00453E2F,
        0x003E4236, 0x00384235, 0x003F4534, 0x00464531, 0x002F372C, 0x000F3339, 0x00044358, 0x000A495D, 0x000B4A5F, 0x000D4C61, 0x000E4D62, 0x000E4C64, 0x000E4C64, 0x000D4A64, 0x000D4A64, 0x0015506A, 0x0000475D, 0x00004E5D, 0x0032818E, 0x00528C98,
        0x0003313C, 0x00276468, 0x005BA9A8, 0x00336B5E, 0x00154743, 0x001D4E56, 0x00074A59, 0x00004651, 0x000D6466, 0x0048847E, 0x0080A298, 0x00538385, 0x00225858, 0x00003E3C, 0x00003D3A, 0x00145752, 0x00286561, 0x00396C68, 0x00002926, 0x00041218,
        0x0000080B, 0x00000603, 0x000A120B, 0x00453B31, 0x00664E42, 0x005D4636, 0x00392918, 0x00504432, 0x003D3125, 0x00302422, 0x00251B21, 0x00221C27, 0x0025232F, 0x001D1F29, 0x00181D26, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0x00636968, 0x00797E81, 0x00B3B7BC, 0x00C5CACD, 0x00CDD2D3, 0x00DAE1DC, 0x00E1E9E2, 0x00DDE8E6, 0x00C7D2D0, 0x00D4DFDC, 0x00D7E2DF, 0x00C2CBC8, 0x00D4DBD8, 0x00DDE2E0, 0x00EBEEEC, 0x00C3D4D1, 0x00C6D7D4, 0x00C4D5D2, 0x00C5D6D3, 0x00C8D9D6,
        0x00C2D3D0, 0x00B6C7C4, 0x00B3C4C1, 0x00D9DAD0, 0x00CCE4D8, 0x00CBE6DD, 0x00D3DCD9, 0x00D0DBD8, 0x00AFD0C9, 0x0085A49B, 0x00767E77, 0x00929D9B, 0x009EA19F, 0x00A7A3A2, 0x008D918C, 0x00899C93, 0x00728A7E, 0x00738477, 0x00868C81, 0x003A5B6E,
        0x003D6375, 0x002E5466, 0x00213B4C, 0x00202B39, 0x004E555E, 0x00677A7D, 0x00638480, 0x009AD1C0, 0x003B8585, 0x000B6276, 0x00186788, 0x001B5D80, 0x0000385D, 0x00045D89, 0x002DA0D3, 0x002CA0D5, 0x001B88BB, 0x001F86B3, 0x00167AA4, 0x002288B8,
        0x002293CB, 0x0022A1E5, 0x0019A3EC, 0x001E98DA, 0x002EA0DC, 0x001F7FAD, 0x0034819C, 0x00266978, 0x000A5367, 0x003591BA, 0x002792CF, 0x002589C4, 0x002F87B0, 0x00267187, 0x000D5360, 0x00034B5D, 0x00034C68, 0x00004265, 0x00155377, 0x002A97CA,
        0x002390C6, 0x002192CB, 0x00299EDB, 0x002DA7E3, 0x0027A8E1, 0x0021A7DD, 0x0022AADE, 0x002AA2E4, 0x002EA2DB, 0x0037AADD, 0x00259FD5, 0x0024A6E9, 0x001E9EE7, 0x002697DA, 0x003AA0DB, 0x003B99C4, 0x0042A3C4, 0x004B98AB, 0x00638D94, 0x00768D89,
        0x0094AA9E, 0x00313F33, 0x00544F46, 0x00595949, 0x005B5B4B, 0x005A5A4A, 0x00565646, 0x00545444, 0x00555545, 0x00545444, 0x00F5F5F4, 0x00848479, 0x00525343, 0x00BDBDB7, 0x00BFBFB9, 0x00575747, 0x00575747, 0x00565646, 0x008A8A80, 0x00D7D7D4,
        0x00727366, 0x00535343, 0x00565646, 0x00575747, 0x00565646, 0x00515242, 0x00545545, 0x004F5040, 0x004E4F3F, 0x004D4E3E, 0x00535343, 0x00525242, 0x00555646, 0x00565747, 0x00525343, 0x004F5040, 0x00505141, 0x00525343, 0x00545545, 0x00545545,
        0x00535444, 0x004E4F3F, 0x004D4E3E, 0x00515242, 0x00515242, 0x00505141, 0x00535444, 0x00515243, 0x00515141, 0x004E4E3E, 0x00545545, 0x00575848, 0x00585949, 0x00565747, 0x00525343, 0x00505343, 0x00515444, 0x00525545, 0x00555848, 0x00515343,
        0x00515242, 0x00525343, 0x00545545, 0x00555646, 0x00545545, 0x00515242, 0x004F5040, 0x00565949, 0x0085877C, 0x00F5F5F4, 0x00515444, 0x00545747, 0x00545747, 0x004F5242, 0x004A4D3D, 0x00414A36, 0x00273222, 0x00868E84, 0x00828A8A, 0x00597D85,
        0x0059A0AE, 0x0033839A, 0x00226481, 0x00207994, 0x00196E84, 0x000C5F74, 0x000C6684, 0x002187B1, 0x001D87B6, 0x001F84AA, 0x0014718B, 0x00015D80, 0x001F3743, 0x0045483F, 0x003A4B40, 0x003D4239, 0x00484432, 0x00484E37, 0x0048483A, 0x00414839,
        0x00434A3B, 0x003D4435, 0x003E4536, 0x003F4637, 0x003E4536, 0x00454C3D, 0x00444B3C, 0x0049483A, 0x00464537, 0x00474638, 0x004A493B, 0x004A493B, 0x00484739, 0x00474638, 0x0049483A, 0x004B4A3C, 0x00484739, 0x0049483A, 0x0049483A, 0x00464537,
        0x00484739, 0x0049483A, 0x00434234, 0x00424536, 0x003E4132, 0x003D4031, 0x003E4334, 0x003F4435, 0x003B4233, 0x003B4233, 0x003D4435, 0x003E4334, 0x003B4031, 0x003A3F30, 0x003B4031, 0x003C4132, 0x003F4435, 0x00414637, 0x00404536, 0x00453E2F,
        0x003F4337, 0x003A4437, 0x00404635, 0x00474632, 0x002F372C, 0x0010343A, 0x00054459, 0x000B4A5E, 0x000B4A5F, 0x000D4C61, 0x000E4D62, 0x000E4C64, 0x000E4C64, 0x000D4A64, 0x000C4963, 0x0010465F, 0x000E5569, 0x00004959, 0x00297182, 0x008CBDCD,
        0x006E93A1, 0x0074A7B1, 0x007CC2C9, 0x007CAD9F, 0x003F6765, 0x006E96A2, 0x00528E9E, 0x001C707C, 0x0053A5AA, 0x009ACDC9, 0x00DDF2E9, 0x008FA4B3, 0x00587381, 0x00254953, 0x0046717A, 0x00A0CCD3, 0x0096BAC2, 0x008EA7B1, 0x0094A5AE, 0x00637383,
        0x00546F79, 0x004A6869, 0x00405350, 0x004F4C47, 0x00584841, 0x0065594F, 0x00635D52, 0x004E5C44, 0x00415140, 0x0044564F, 0x004E6769, 0x005B7D83, 0x00658F96, 0x00629598, 0x005D9497, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0x0060666B, 0x0081888B, 0x00A7AFAF, 0x00B2BAB9, 0x00CAD2D1, 0x00CCD5D2, 0x00C4CEC8, 0x00D5E1DB, 0x00D5E1DB, 0x00CDD9D3, 0x00C8D2CC, 0x00C4CEC8, 0x00D4DBD6, 0x00CCD3CE, 0x00CED5D0, 0x00D0DCD6, 0x00E2EEE8, 0x00E0ECE6, 0x00D0DCD6, 0x00CAD6D0,
        0x00DAE6E0, 0x00D9E5DF, 0x00ACB8B2, 0x00C8D5CD, 0x00D5E2DA, 0x00D4E1D9, 0x00CFDDD7, 0x00C2D0CA, 0x00BBCAC6, 0x0082918D, 0x0074837F, 0x0099A4A2, 0x008A9593, 0x00929D9B, 0x007A8583, 0x00798482, 0x00808B89, 0x00798482, 0x00788381, 0x0035536E,
        0x001E6182, 0x00005985, 0x00004575, 0x00085586, 0x001574A6, 0x000A80BB, 0x001394D9, 0x002D91D9, 0x0031B1EC, 0x0025ADDD, 0x0038A9DB, 0x002B94C5, 0x001486AA, 0x00005668, 0x001E5E63, 0x0053A3C8, 0x003299D8, 0x00005A99, 0x00003855, 0x00488591,
        0x0064AED0, 0x001C78B3, 0x003D9EE2, 0x002FA3E4, 0x002294D3, 0x0039A9E5, 0x0048B5EB, 0x0041ABE0, 0x002891C4, 0x002B91C5, 0x003197CC, 0x002087B4, 0x001977A4, 0x003789B3, 0x002B7298, 0x00074769, 0x00054460, 0x00114F67, 0x0007485D, 0x0011527E,
        0x003393B7, 0x002889AF, 0x00298CBE, 0x0028A1DF, 0x002BA3EB, 0x002D9FE6, 0x0019A4E3, 0x0029ABD6, 0x0024A6DB, 0x0023A1DC, 0x0031A5DC, 0x002D97C6, 0x003EA9D5, 0x003BB0E7, 0x001B9DDE, 0x003E98D1, 0x00157BAF, 0x00397997, 0x00718684, 0x00809382,
        0x0098A29C, 0x003C3739, 0x00484C47, 0x00585749, 0x00575648, 0x00555547, 0x00545446, 0x00535345, 0x00505344, 0x00515445, 0x00F5F5F4, 0x00848479, 0x00585949, 0x00CBCBC6, 0x00FFFFFF, 0x00AAAAA2, 0x00545444, 0x00525242, 0x008E8E84, 0x00FFFFFF,
        0x00DFDFDD, 0x005E5E4F, 0x00555545, 0x00575747, 0x00565646, 0x00505141, 0x00535444, 0x00555646, 0x00535343, 0x00565646, 0x00575747, 0x00565646, 0x00505142, 0x00565647, 0x00535444, 0x00505141, 0x00505141, 0x00525343, 0x00545545, 0x00565747,
        0x00545545, 0x004F5040, 0x004D4E3E, 0x00515242, 0x00515242, 0x004F5040, 0x00535444, 0x004E5042, 0x00505142, 0x00515242, 0x00535444, 0x00575848, 0x00585949, 0x00555646, 0x00525343, 0x004F5040, 0x00505141, 0x00525343, 0x00545545, 0x00545545,
        0x00535444, 0x004E4F3F, 0x004D4E3E, 0x00515242, 0x00515242, 0x00505141, 0x00535444, 0x004D5041, 0x007F8177, 0x00F5F5F4, 0x00505344, 0x00525546, 0x00545748, 0x004D5041, 0x004E5142, 0x00504541, 0x00232F29, 0x008D8F89, 0x00759283, 0x005A8578,
        0x0074A6AC, 0x003399B6, 0x002A94C9, 0x001F9BC3, 0x001F97C1, 0x001785AF, 0x000B6588, 0x000A556B, 0x00165B6E, 0x001B6B84, 0x00227B9D, 0x0001537D, 0x00293841, 0x004F4E3A, 0x00364732, 0x003E403A, 0x0048433A, 0x003D4435, 0x0041483B, 0x00414736,
        0x00424837, 0x00434938, 0x00424837, 0x00404635, 0x00404635, 0x00414736, 0x00424837, 0x0044463A, 0x0046483C, 0x0047493D, 0x0047493D, 0x0045473B, 0x0045473B, 0x0046483C, 0x00484A3E, 0x00444738, 0x00444738, 0x00444738, 0x00444738, 0x00434637,
        0x00434637, 0x00434637, 0x00444738, 0x00414539, 0x003F4337, 0x003D4135, 0x003D4135, 0x003E4236, 0x003F4337, 0x003E4236, 0x003D4135, 0x003C4334, 0x003A4132, 0x003A4132, 0x003D4233, 0x003F4435, 0x00424536, 0x00404334, 0x003F4233, 0x00424633,
        0x00434337, 0x003D4239, 0x00394336, 0x00454535, 0x003E3831, 0x00193236, 0x00044553, 0x0008475C, 0x000A495E, 0x000C4B60, 0x000D4C61, 0x000C4B60, 0x000B4A5F, 0x000C4B60, 0x000D4C61, 0x00064A67, 0x001D4E5E, 0x0014424D, 0x00205968, 0x0047717D,
        0x003F4C4E, 0x004E6666, 0x002F696F, 0x00597778, 0x002F3A3E, 0x00496167, 0x002E717A, 0x001F697B, 0x00588093, 0x007B8B91, 0x006A8276, 0x00305056, 0x0044666C, 0x0023474D, 0x00365C61, 0x0070949A, 0x007F9FA5, 0x00718B92, 0x006F868E, 0x0073878C,
        0x006C7D80, 0x006C7878, 0x00565D58, 0x0053544A, 0x00585344, 0x00605946, 0x00635844, 0x00574741, 0x00514A47, 0x00626A6A, 0x004C666C, 0x00608C93, 0x005E989E, 0x0060A4A9, 0x0063ACB0, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0x00595F64, 0x007F8689, 0x00B5BDBD, 0x00BAC2C1, 0x00C9D1D0, 0x00D0D9D6, 0x00D0DAD4, 0x00C1CDC7, 0x00C0CCC6, 0x00B4C0BA, 0x00BDC7C1, 0x00CCD6D0, 0x00CED8D2, 0x00C5CCC7, 0x00DFE6E1, 0x00E6F2EC, 0x00D1DDD7, 0x00C8D4CE, 0x00CEDAD4, 0x00C9D5CF,
        0x00C6D2CC, 0x00CFDBD5, 0x00CFDBD5, 0x00CFDCD4, 0x00CDDAD2, 0x00C9D6CE, 0x00C9D7D1, 0x00BBC9C3, 0x00ACBBB7, 0x00707F7B, 0x006A7975, 0x0099A4A2, 0x00919C9A, 0x008D9896, 0x007B8684, 0x00848F8D, 0x00747F7D, 0x005C6765, 0x005E6967, 0x002F5255,
        0x00185962, 0x000C5B6E, 0x0008465E, 0x000D3F56, 0x001B576F, 0x001E688A, 0x00226F9D, 0x002275A2, 0x0014789B, 0x001780A1, 0x001D7DA5, 0x0039A2CE, 0x0034ADD5, 0x00288DAD, 0x00074A61, 0x00006073, 0x002C9CC6, 0x0038A2D8, 0x0013638C, 0x00195874,
        0x004088A6, 0x00217DAC, 0x00117CB5, 0x0034A2DC, 0x0029A0D8, 0x0023A1D7, 0x00249ED2, 0x0039ABE0, 0x0037A4DA, 0x002EA2D9, 0x002BA4DC, 0x002491C4, 0x00248ABA, 0x001873A0, 0x003D8CB3, 0x002B7294, 0x00094C67, 0x0006495E, 0x000F5366, 0x0012516D,
        0x00459EB2, 0x00247B8F, 0x001F7998, 0x000B7FAE, 0x0034A9E6, 0x002C9DDB, 0x0027B1E6, 0x0029A7D7, 0x0024A5DC, 0x0024A3E0, 0x0035AAE1, 0x00157EAA, 0x000A6B8F, 0x001F84AB, 0x002893BF, 0x001975A4, 0x00147DA8, 0x004A8FAA, 0x0073908D, 0x00728F7F,
        0x00899F9A, 0x00333738, 0x003E493F, 0x00575648, 0x00555446, 0x00525244, 0x00505042, 0x00505042, 0x004F5243, 0x00515445, 0x00F5F5F4, 0x0086867B, 0x00545546, 0x00656559, 0x00E9E9E7, 0x00FEFEFE, 0x007E8075, 0x00515445, 0x00535444, 0x00B6B7B0,
        0x00FFFFFF, 0x00BFBFB9, 0x00555545, 0x00545444, 0x00525242, 0x00535444, 0x00525343, 0x00545444, 0x00525242, 0x00555545, 0x00575747, 0x00545545, 0x00505042, 0x004F5041, 0x004F5142, 0x00505241, 0x00595A49, 0x00555646, 0x00555646, 0x00575848,
        0x00555646, 0x004F5040, 0x004D4E3E, 0x00515242, 0x00505141, 0x004F5040, 0x00535444, 0x00505244, 0x00535444, 0x00535444, 0x00545545, 0x00545545, 0x00545545, 0x00545545, 0x00545545, 0x00505141, 0x00505141, 0x00525343, 0x00545545, 0x00565747,
        0x00545545, 0x004F5040, 0x004D4E3E, 0x00515242, 0x00515242, 0x004F5040, 0x00535444, 0x00505344, 0x00818378, 0x00F5F5F4, 0x004F5243, 0x00505344, 0x00515445, 0x00494C3D, 0x00494C3D, 0x004F483F, 0x00223129, 0x008B8C88, 0x00748E81, 0x005B8175,
        0x006C9C9E, 0x0027879E, 0x001E84AF, 0x001785A7, 0x002091B9, 0x001F8AB6, 0x002282A6, 0x0015657E, 0x00004355, 0x00034E64, 0x001A6885, 0x001E709A, 0x00303F48, 0x004B4A36, 0x00384934, 0x00444640, 0x004C473E, 0x003D4435, 0x003C4336, 0x003F4534,
        0x00404635, 0x00414736, 0x00414736, 0x00404635, 0x00404635, 0x00424837, 0x00444A39, 0x00434539, 0x0045473B, 0x0046483C, 0x0045473B, 0x0044463A, 0x00434539, 0x0044463A, 0x0045473B, 0x00454839, 0x00454839, 0x00454839, 0x00454839, 0x0046493A,
        0x0046493A, 0x00454839, 0x00454839, 0x00404438, 0x003D4135, 0x003B3F33, 0x003B3F33, 0x003C4034, 0x003D4135, 0x003D4135, 0x003C4034, 0x003E4536, 0x003D4435, 0x003C4334, 0x003F4435, 0x00404536, 0x00424536, 0x00414435, 0x003F4233, 0x00414532,
        0x00424236, 0x003B4037, 0x00364033, 0x00424232, 0x003A342D, 0x00152E32, 0x00014250, 0x000A495E, 0x000B4A5F, 0x000C4B60, 0x000C4B60, 0x000C4B60, 0x000C4B60, 0x000D4C61, 0x000D4C61, 0x00144C67, 0x00174960, 0x00004162, 0x001676A4, 0x002E8AB5,
        0x00003346, 0x00206463, 0x006BCCC8, 0x00175F59, 0x00024C46, 0x000C504F, 0x000E4C56, 0x00004457, 0x00004A56, 0x001F6F6A, 0x00397B68, 0x00336565, 0x001A4E4E, 0x00063C3C, 0x00023838, 0x00184C4C, 0x003E6E6F, 0x003B6768, 0x00002123, 0x0000090E,
        0x0000090C, 0x00000303, 0x00000601, 0x00393A30, 0x004E493A, 0x00403926, 0x003F3420, 0x00515238, 0x00484A34, 0x00121506, 0x00171A11, 0x00191A18, 0x00241F20, 0x00251D1E, 0x00160C0C, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0x00575D62, 0x007E8588, 0x00AFB7B7, 0x00B1B9B8, 0x00CAD2D1, 0x00D7E0DD, 0x00CED8D2, 0x00C9D7D1, 0x00C6D4CE, 0x00B5C1BB, 0x00AFBBB5, 0x00BFC9C3, 0x00C9D3CD, 0x00C4CEC8, 0x00DBE2DD, 0x00D5E1DB, 0x00BCC8C2, 0x00BCC8C2, 0x00C6D2CC, 0x00C3CFC9,
        0x00BDC9C3, 0x00BDC9C3, 0x00C2CEC8, 0x00D8E5DD, 0x00C8D5CD, 0x00C1CDC7, 0x00CBD9D3, 0x00D2E0DC, 0x00CFDEDA, 0x0084938F, 0x006A7876, 0x00929D9B, 0x008C9795, 0x007F8A88, 0x00737E7C, 0x007F8A88, 0x006E7977, 0x005D6866, 0x005A6563, 0x00304742,
        0x00174B4B, 0x000B4A52, 0x0004313E, 0x0003242D, 0x00053039, 0x00013748, 0x0000324D, 0x00114C56, 0x00134E57, 0x00134D59, 0x000F556D, 0x00005177, 0x001B96C2, 0x0043ABD6, 0x00276790, 0x00055359, 0x0012687A, 0x00409FCA, 0x00359BD0, 0x00004B74,
        0x0000526B, 0x00217D96, 0x001782A8, 0x001A87B3, 0x001887B5, 0x001084B3, 0x000C87B9, 0x001EA0D5, 0x0027AAE3, 0x002BADE9, 0x002BAAE7, 0x0030A7DF, 0x00188ABF, 0x00258CBD, 0x001B77A2, 0x00388AAD, 0x0028728E, 0x00064F65, 0x00004759, 0x00144E5A,
        0x0089D6D8, 0x005FA3A2, 0x003C8490, 0x002D98B9, 0x002899CB, 0x003EAEE3, 0x0021A8D6, 0x002FA6DE, 0x0029A6E3, 0x001D9BDB, 0x0035ACE4, 0x00127BA4, 0x00004761, 0x000D596B, 0x002B7384, 0x00459FBE, 0x002892B1, 0x004790A4, 0x006B908E, 0x00709788,
        0x008AA8A3, 0x00313C3A, 0x00404F41, 0x00565547, 0x00545345, 0x004F4F41, 0x004D4D3F, 0x004D4D3F, 0x004D5041, 0x00515445, 0x00F5F5F4, 0x0084857A, 0x00515243, 0x00505042, 0x007D7D72, 0x00FCFCFB, 0x00E9EAE8, 0x00646759, 0x00545545, 0x00595A4C,
        0x00D5D5D1, 0x00FFFFFF, 0x00989A91, 0x00515445, 0x00515445, 0x00535444, 0x00585949, 0x00565646, 0x00545444, 0x00555545, 0x00545444, 0x00505040, 0x00525344, 0x004F4F41, 0x004F5243, 0x004B4C3C, 0x00565747, 0x00535343, 0x00555545, 0x00555545,
        0x00545444, 0x00505040, 0x004D4E3E, 0x00505141, 0x00505141, 0x004F5040, 0x00535444, 0x004F5042, 0x00535444, 0x00545546, 0x00525343, 0x00505242, 0x00505242, 0x00525344, 0x00565747, 0x00555646, 0x00545545, 0x00545545, 0x00555646, 0x00575848,
        0x00555646, 0x004F5040, 0x004D4E3E, 0x00515242, 0x00505141, 0x004F5040, 0x00535444, 0x00505344, 0x00818378, 0x00F5F5F4, 0x004E5142, 0x004E5142, 0x004F5243, 0x00484B3C, 0x00484B3C, 0x00434234, 0x001B2C21, 0x00868785, 0x00788D85, 0x00638378,
        0x006C9391, 0x001B7580, 0x0012738D, 0x00217E97, 0x002C90B3, 0x00228DB9, 0x002E96C1, 0x00227C9B, 0x00004258, 0x00004358, 0x001B5E75, 0x001E709A, 0x002C3B44, 0x00484733, 0x00344530, 0x00393B35, 0x00454037, 0x003C4334, 0x00363D30, 0x003A402F,
        0x003B4130, 0x003C4231, 0x003C4231, 0x003C4231, 0x003E4433, 0x00404635, 0x00424837, 0x0047493D, 0x0047493D, 0x00484A3E, 0x00484A3E, 0x0047493D, 0x0046483C, 0x0047493D, 0x0047493D, 0x00434637, 0x00424536, 0x00424536, 0x00444738, 0x0046493A,
        0x00474A3B, 0x00454839, 0x00434637, 0x0043473B, 0x00414539, 0x003F4337, 0x003E4236, 0x003E4236, 0x003F4337, 0x00404438, 0x00404438, 0x003F4637, 0x003E4536, 0x003C4334, 0x003E4334, 0x003E4334, 0x003F4233, 0x003E4132, 0x003D4031, 0x00424633,
        0x00424236, 0x003C4138, 0x00374134, 0x00424232, 0x003B352E, 0x00173034, 0x00034452, 0x000C4B60, 0x000C4B60, 0x000B4A5F, 0x000C4B60, 0x000D4C61, 0x000E4D62, 0x000E4D62, 0x000E4D62, 0x00064865, 0x00144658, 0x00124656, 0x003C7F94, 0x0077ACC0,
        0x0068858A, 0x0084A9A7, 0x0084C8CD, 0x0083ABAA, 0x00445D5F, 0x0076999D, 0x00589BA4, 0x002A7486, 0x006FA1B3, 0x00B0D1D4, 0x00B2D6C8, 0x007D9F9F, 0x004A6E6E, 0x00264C4C, 0x00406666, 0x0091B5B5, 0x009FBEBF, 0x0089A5A6, 0x006E8587, 0x0054686D,
        0x00516265, 0x00566262, 0x00464D48, 0x004D4E44, 0x00544F40, 0x0058513E, 0x00594E3A, 0x00514F44, 0x00474C43, 0x00445551, 0x003A595A, 0x00507D80, 0x00538A8D, 0x00518E90, 0x004B8B8C, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0x005D6368, 0x00848B8E, 0x00A9B1B1, 0x009FA7A6, 0x00B3BBBA, 0x00C5CECB, 0x00C4CEC8, 0x00BAC8C2, 0x00B7C5BF, 0x00BFCDC7, 0x00C6D2CC, 0x00C8D4CE, 0x00CDD7D1, 0x00C2CCC6, 0x00BBC5BF, 0x00D3DFD9, 0x00C9D5CF, 0x00D0DCD6, 0x00D0DCD6, 0x00CFDBD5,
        0x00D2DED8, 0x00C3CFC9, 0x00B8C4BE, 0x00BAC6C0, 0x00B2BEB8, 0x00ADB9B3, 0x00BAC8C4, 0x00D4E2DE, 0x00EAF8F6, 0x00A3B1AF, 0x00788684, 0x0096A19F, 0x008C9795, 0x00808B89, 0x0075807E, 0x00737E7C, 0x0066716F, 0x0065706E, 0x00525D5B, 0x00444D56,
        0x00395C6A, 0x00225369, 0x00153E54, 0x00194251, 0x0015525C, 0x000E586A, 0x00185E7B, 0x007AA49D, 0x0093ADA7, 0x009CAEAD, 0x0086B3BE, 0x00025C75, 0x0000698E, 0x00429BC7, 0x0072A7D2, 0x006D7D7C, 0x008CAFB3, 0x004F9DBA, 0x0029A0D7, 0x0030AAD8,
        0x00177582, 0x00004B51, 0x002C7D92, 0x001C7998, 0x00479FC3, 0x00439FCA, 0x002594C6, 0x001EA5DB, 0x001CACE6, 0x0022A4E5, 0x002B9EE1, 0x0022A3E2, 0x002CA8E4, 0x001A8CC1, 0x002088B7, 0x0017749B, 0x00378DAB, 0x0025758C, 0x0000485B, 0x00154C5B,
        0x00367775, 0x0077A69D, 0x008CBFC2, 0x00328FA9, 0x0039A6D9, 0x0039A4DE, 0x002FAEE1, 0x0030A2E1, 0x002FA7E8, 0x002BA7E7, 0x0036ADE4, 0x00086F96, 0x00186A7C, 0x0075AFB4, 0x0092C0BA, 0x0066B5CA, 0x0046A6BE, 0x00599EAF, 0x006F9190, 0x00668C80,
        0x007F9A96, 0x00333836, 0x004F5948, 0x00575648, 0x00545345, 0x00505042, 0x004D4D3F, 0x004D4D3F, 0x004D5041, 0x00505344, 0x00F5F5F4, 0x00828377, 0x004D4E3F, 0x004D4D3F, 0x004D4D3F, 0x009D9E96, 0x00FFFFFF, 0x00CACBC6, 0x00535444, 0x00515243,
        0x00656559, 0x00ECECEB, 0x00F9FAF9, 0x00787A6F, 0x00535647, 0x00545545, 0x00545546, 0x00545446, 0x00535345, 0x00505344, 0x00515445, 0x00505344, 0x00515143, 0x00525244, 0x004B4E3F, 0x00535345, 0x004A4A3C, 0x00515141, 0x00525242, 0x00515141,
        0x00515141, 0x004F4F3F, 0x004D4E3E, 0x00505141, 0x004F5040, 0x004E4F3F, 0x00525344, 0x00525345, 0x00515343, 0x00525445, 0x00505243, 0x004F5243, 0x00525445, 0x00515343, 0x00555646, 0x00575747, 0x00565646, 0x00555545, 0x00555545, 0x00555545,
        0x00545444, 0x00505040, 0x004D4E3E, 0x00505141, 0x00505141, 0x004F5040, 0x00535444, 0x004F5243, 0x00808277, 0x00F5F5F4, 0x004E5142, 0x004F5243, 0x00515445, 0x004A4D3E, 0x004B4E3F, 0x00464835, 0x001E3225, 0x008A8B89, 0x007D8E8A, 0x00658076,
        0x0060827C, 0x00025557, 0x00005061, 0x00125F72, 0x002481A2, 0x002692C1, 0x002D9BCB, 0x002182A6, 0x00004C65, 0x00044659, 0x002A6378, 0x00156791, 0x002B3A43, 0x00474632, 0x001D2E19, 0x0012140E, 0x002D281F, 0x003D4435, 0x0040473A, 0x003E4334,
        0x003E4334, 0x003E4334, 0x003E4334, 0x003F4435, 0x00414637, 0x00434839, 0x00454A3B, 0x0045493D, 0x00464A3E, 0x00464A3E, 0x00474B3F, 0x00474B3F, 0x00474B3F, 0x00474B3F, 0x00484C40, 0x0045493D, 0x0044483C, 0x0044483C, 0x00474B3F, 0x004A4E42,
        0x004B4F43, 0x00484C40, 0x0045493D, 0x0045493D, 0x0043473B, 0x00414539, 0x00404438, 0x00404438, 0x00414539, 0x0042463A, 0x0043473B, 0x00444B3E, 0x00434A3D, 0x0041483B, 0x0042463A, 0x00404438, 0x00414337, 0x00414337, 0x00404236, 0x003E422F,
        0x00404034, 0x003C4138, 0x00384235, 0x00434333, 0x003C362F, 0x00173034, 0x00034452, 0x000B4A5F, 0x000A495E, 0x000B4A5F, 0x000D4C61, 0x00104F64, 0x00115065, 0x00104F64, 0x000E4D62, 0x00024A68, 0x001C4D5B, 0x0019464A, 0x002A5E65, 0x0052747A,
        0x004E5552, 0x005D6F6E, 0x00376C76, 0x005A7375, 0x00424248, 0x004E6067, 0x00297179, 0x00136172, 0x004E6F83, 0x008A9299, 0x007C9084, 0x0035504D, 0x003F5C59, 0x002F4E4B, 0x003B5A57, 0x0089A6A3, 0x00859D9B, 0x00899B9A, 0x008C9C9B, 0x00809499,
        0x0077888B, 0x007D8989, 0x00626964, 0x004C4D43, 0x004B4637, 0x00564F3C, 0x00564B37, 0x00574845, 0x004B4645, 0x00636E72, 0x00496B71, 0x00659AA4, 0x005FA7AF, 0x005CB0B6, 0x0059B4B8, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0x0060666B, 0x008A9194, 0x00B8C0C0, 0x00ADB5B4, 0x00B0B8B7, 0x00C0C9C6, 0x00D7E1DB, 0x00C9D9D2, 0x00B3C3BC, 0x00B7C5BF, 0x00C0CEC8, 0x00C3CFC9, 0x00BFCBC5, 0x00C3CDC7, 0x00CFD9D3, 0x00C9D5CF, 0x00C1CDC7, 0x00CCD8D2, 0x00D3DFD9, 0x00D5E1DB,
        0x00D0DCD6, 0x00B8C4BE, 0x00ABB7B1, 0x00C5D1CB, 0x00C8D4CE, 0x00BFCAC7, 0x00B9C7C3, 0x00B8C5C3, 0x00C6D4D2, 0x00919F9D, 0x006D7B7A, 0x00919C9A, 0x008A9593, 0x00838E8C, 0x00808B89, 0x00788381, 0x005C6765, 0x00596462, 0x004B5654, 0x00354552,
        0x002F5468, 0x001D4C67, 0x00153C58, 0x00164358, 0x00165968, 0x001E6A81, 0x002B7293, 0x00668A84, 0x00AEB9B1, 0x00C6C6C0, 0x0083A2A3, 0x00408C98, 0x00247C94, 0x004E90AD, 0x006B91AF, 0x003B4643, 0x004A615D, 0x003C778A, 0x003697C3, 0x002786AD,
        0x00236169, 0x002C4F4B, 0x00385C64, 0x0079A4B9, 0x005EA0B9, 0x00187596, 0x00006C98, 0x002091C9, 0x0036A7E5, 0x0033A7E8, 0x002AA5E5, 0x0020A3E6, 0x0020A1E0, 0x0030ABE5, 0x002093C5, 0x001F88B3, 0x001A7A9E, 0x00378FAD, 0x001E718D, 0x00064561,
        0x00094349, 0x009EBAB4, 0x00638388, 0x00227595, 0x002D95D0, 0x0040A8EB, 0x002BA3DF, 0x0033A7E8, 0x002299D7, 0x00249CD7, 0x0038ADE0, 0x001E87AE, 0x004B9EB4, 0x008AC2C7, 0x00689189, 0x001B6074, 0x00217991, 0x005690A3, 0x00748C8A, 0x00728E81,
        0x0091A29F, 0x003C3837, 0x004A4B3B, 0x00575648, 0x00565547, 0x00525244, 0x00505042, 0x004F4F41, 0x004D5041, 0x004D5041, 0x00F5F5F4, 0x00828377, 0x004E4F40, 0x004E4E40, 0x004D4D3F, 0x004E5142, 0x00C9CAC5, 0x00FFFFFF, 0x00A2A29A, 0x004D4E3F,
        0x004D4D3F, 0x00818177, 0x00FFFFFF, 0x00E5E6E3, 0x00636558, 0x00525343, 0x00515243, 0x00505042, 0x00505042, 0x004F5243, 0x00515445, 0x00515445, 0x004C4D3E, 0x00535345, 0x004B4E3F, 0x00515347, 0x004D4F42, 0x00555545, 0x00565646, 0x00555545,
        0x00515242, 0x004E4F3F, 0x004D4E3E, 0x00505141, 0x004E4F3F, 0x004E4F3F, 0x00525344, 0x00545648, 0x00505344, 0x00525546, 0x004F5243, 0x004F5243, 0x00515445, 0x004D4F40, 0x00515343, 0x00565646, 0x00555545, 0x00535343, 0x00525242, 0x00515141,
        0x00515141, 0x004F4F3F, 0x004D4E3E, 0x00505141, 0x004F5040, 0x004E4F3F, 0x00535444, 0x00515445, 0x00818378, 0x00F5F5F4, 0x004F5243, 0x004F5243, 0x00525546, 0x004C4F40, 0x004D5041, 0x00484C39, 0x00203427, 0x008A8B87, 0x007C8E87, 0x00657E74,
        0x005C7C77, 0x00004E51, 0x00004A5A, 0x00003F51, 0x000F698B, 0x002896C6, 0x00289ACF, 0x00177FAA, 0x0000506E, 0x00003D54, 0x00256479, 0x00166892, 0x002C3B44, 0x004B4A36, 0x0023341F, 0x0012140E, 0x002A251C, 0x003D4435, 0x0041483B, 0x003F4435,
        0x003F4435, 0x003E4334, 0x003E4334, 0x003F4435, 0x003F4435, 0x00404536, 0x00404536, 0x0043473B, 0x0043473B, 0x0043473B, 0x0044483C, 0x00464A3E, 0x00474B3F, 0x00474B3F, 0x00474B3F, 0x0043473B, 0x0042463A, 0x0042463A, 0x0045493D, 0x00484C40,
        0x00494D41, 0x00464A3E, 0x0043473B, 0x0043473B, 0x0042463A, 0x00414539, 0x00404438, 0x003F4337, 0x00404438, 0x00414539, 0x0042463A, 0x0041483B, 0x0040473A, 0x003F4639, 0x003F4337, 0x003D4135, 0x003E4034, 0x003E4034, 0x003E4034, 0x003B3F2C,
        0x003F3F33, 0x003C4138, 0x00384235, 0x00424232, 0x0039332C, 0x00142D31, 0x0000404E, 0x0007465B, 0x0008475C, 0x000A495E, 0x000E4D62, 0x00125166, 0x00135267, 0x00115065, 0x000F4E63, 0x00104D67, 0x00164B60, 0x0000425F, 0x0013729D, 0x003088B0,
        0x00003646, 0x001C5E5D, 0x0052B3AF, 0x001A605A, 0x00004640, 0x000D4F4E, 0x00074851, 0x00004557, 0x0000515D, 0x001E6B67, 0x003B7B68, 0x0045736D, 0x00285852, 0x00093C35, 0x00063A33, 0x00194C45, 0x000E3A34, 0x00254D48, 0x00001F1B, 0x0000070C,
        0x00000609, 0x00000606, 0x00020904, 0x00313228, 0x00464132, 0x003F3825, 0x003C311D, 0x00534630, 0x004B3F2D, 0x001C1208, 0x001C130F, 0x001D1516, 0x0022191C, 0x0026181C, 0x001A0C0E, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0x005E6469, 0x0082898C, 0x00B2BABA, 0x00BBC3C2, 0x00C8D0CF, 0x00C9D2CF, 0x00CDD7D1, 0x00BFCFC8, 0x00BCCCC5, 0x00BCCCC5, 0x00B6C4BE, 0x00BECCC6, 0x00B8C4BE, 0x00B4C0BA, 0x00C3CFC9, 0x00C0CCC6, 0x00C2CEC8, 0x00CBD7D1, 0x00CFDBD5, 0x00D1DDD7,
        0x00CFDBD5, 0x00BFCBC5, 0x00B2BEB8, 0x00C5D0CD, 0x00C6D1CE, 0x00B8C3C0, 0x00BAC7C5, 0x00ABB8B6, 0x00A4B2B0, 0x007C8A89, 0x00647271, 0x00828D8B, 0x00828D8B, 0x007B8684, 0x007E8987, 0x00808B89, 0x0056615F, 0x004B5654, 0x00535E5C, 0x002E4E4D,
        0x0018484E, 0x000B3C4C, 0x000C3042, 0x00002735, 0x0000333E, 0x0000374C, 0x00001B3E, 0x0037575C, 0x00ACB1B0, 0x00A4A29A, 0x0043625B, 0x00478383, 0x007AB7C1, 0x0081ADBA, 0x0096B7C0, 0x0052928C, 0x00528986, 0x006FA0AA, 0x0077A3BA, 0x00476578,
        0x003B464A, 0x004D4F49, 0x00646765, 0x00CEC6D7, 0x00A1D5E5, 0x005CCCE2, 0x00279FC3, 0x00096197, 0x003079B7, 0x004AAEE9, 0x001DAADD, 0x0025A5E6, 0x0021A2E1, 0x002DABE5, 0x0030AADE, 0x00198BBA, 0x00248DB8, 0x0016759D, 0x003089AE, 0x00176492,
        0x000A4B5A, 0x003A514D, 0x002A4046, 0x00217295, 0x003CA8E8, 0x002B94DD, 0x002BA3E4, 0x002AA5E3, 0x0031A8E0, 0x002598CB, 0x0035A3D3, 0x003CA5D1, 0x003993B5, 0x00347E90, 0x0048848A, 0x00357D9B, 0x002983A2, 0x0050899E, 0x00738885, 0x007C9384,
        0x0098A6A2, 0x003E3636, 0x0049483A, 0x00565547, 0x00565547, 0x00535345, 0x00515143, 0x00505042, 0x004C4F40, 0x004B4E3F, 0x00F4F5F4, 0x00838478, 0x00565747, 0x008C8C82, 0x004F4F41, 0x004D5041, 0x005E6154, 0x00E8E8E7, 0x00FDFDFD, 0x007C7D71,
        0x004E4E40, 0x004D4D3F, 0x00A5A69F, 0x00FFFFFF, 0x00C8C9C4, 0x00505141, 0x004D4E3F, 0x004D4D3F, 0x004D4D3F, 0x004D5041, 0x00515445, 0x00545647, 0x00535345, 0x00545446, 0x004B4E3F, 0x0043443A, 0x00525247, 0x00555546, 0x00555545, 0x00515242,
        0x004E5040, 0x004E5040, 0x004E4F3F, 0x004F5040, 0x004E4F3F, 0x004D4E3E, 0x00525344, 0x00525546, 0x004E5142, 0x00515445, 0x004E5142, 0x004E5142, 0x00505344, 0x004B4E3F, 0x004D5041, 0x00565646, 0x00555545, 0x00555545, 0x00565646, 0x00555545,
        0x00515242, 0x004E4F3F, 0x004D4E3E, 0x00505141, 0x004E4F3F, 0x004E4F3F, 0x00535444, 0x00545748, 0x0083857A, 0x00F5F5F4, 0x004F5243, 0x004F5243, 0x00515445, 0x004B4E3F, 0x004B4E3F, 0x00454838, 0x001C3023, 0x00888A84, 0x007C9286, 0x00698578,
        0x006E8F8B, 0x001E6F77, 0x00176E88, 0x0012687C, 0x00107195, 0x001F8FC3, 0x002195CC, 0x00258FBE, 0x00167394, 0x00004A66, 0x00065772, 0x001A6C96, 0x0026353E, 0x004B4A36, 0x003B4C37, 0x00393B35, 0x00423D34, 0x003F4637, 0x003D4437, 0x00404438,
        0x003F4337, 0x003F4337, 0x003F4337, 0x003F4337, 0x003F4337, 0x003E4236, 0x003D4135, 0x0041483B, 0x0041483B, 0x0041483B, 0x0042493C, 0x00444B3E, 0x00454C3F, 0x00454C3F, 0x00454C3F, 0x003F453A, 0x003E4439, 0x003E4439, 0x003F453A, 0x0042483D,
        0x0042483D, 0x0040463B, 0x003E4439, 0x0042463A, 0x0042463A, 0x0042463A, 0x00414539, 0x00404438, 0x00404438, 0x00414539, 0x0042463A, 0x003E4439, 0x003D4338, 0x003C4237, 0x003C4035, 0x003A3E33, 0x003C3D33, 0x003D3E34, 0x003E3F35, 0x003F4330,
        0x00434337, 0x0040453C, 0x003B4538, 0x00444434, 0x003A342D, 0x00142D31, 0x0000414F, 0x0006455A, 0x0008475C, 0x000B4A5F, 0x000E4D62, 0x00115065, 0x00125166, 0x00115065, 0x00104F64, 0x00084B66, 0x0013485C, 0x00074458, 0x002C7C99, 0x0069AEC9,
        0x0056818A, 0x006C9E9C, 0x0071C3C4, 0x006FA4A1, 0x00305C5B, 0x005F9092, 0x004B9099, 0x002B7789, 0x0063A4B3, 0x0096CACA, 0x00A0D1C1, 0x007FA7A6, 0x00477170, 0x00133F3E, 0x003B6A68, 0x007EAAA9, 0x00769E9D, 0x007EA0A0, 0x00607F80, 0x004D6166,
        0x0048595C, 0x004B5757, 0x003C433E, 0x0044453B, 0x004E493A, 0x00544D3A, 0x00564B37, 0x004E4E3E, 0x004D5145, 0x00424F47, 0x003E5351, 0x00496768, 0x004C6F72, 0x00507678, 0x004C7375, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0x005C6267, 0x007B8285, 0x00A3ABAB, 0x00ADB5B4, 0x00C8D0CF, 0x00C6CFCC, 0x00B3BDB7, 0x00B8C8C1, 0x00C1D1CA, 0x00C5D5CE, 0x00B0C0B9, 0x00B3C1BB, 0x00B2C0BA, 0x00B3BFB9, 0x00B8C4BE, 0x00BBC7C1, 0x00C7D3CD, 0x00C5D1CB, 0x00BBC7C1, 0x00B9C5BF,
        0x00C2CEC8, 0x00CBD7D1, 0x00C5D1CB, 0x00CBD6D3, 0x00C1CCC9, 0x00A9B4B1, 0x00C1CECC, 0x00BCC9C7, 0x00ACBAB9, 0x00879594, 0x006C7A79, 0x007C8785, 0x007E8987, 0x007A8583, 0x0075807E, 0x00788381, 0x00576260, 0x004B5654, 0x00515C5A, 0x00325148,
        0x001D4948, 0x00123F4A, 0x001A3C4C, 0x00153F4B, 0x0014505C, 0x0018546C, 0x00063257, 0x00354F5B, 0x00A1A4A8, 0x00A3A69D, 0x00486B5D, 0x001C4F48, 0x0052777B, 0x00385559, 0x00527977, 0x002D797F, 0x00266972, 0x00648E93, 0x007B8781, 0x004C4A40,
        0x004D4C48, 0x004C504A, 0x004E4E40, 0x00817A81, 0x00417078, 0x00238B9C, 0x0050C5E4, 0x0040A0CE, 0x00186CA0, 0x001B82B3, 0x002AACD7, 0x0027A2E0, 0x002FACE9, 0x0024A2DC, 0x0024A2D9, 0x0032AADE, 0x001A89BB, 0x002389BA, 0x001877A8, 0x002689BD,
        0x00226F82, 0x006B857F, 0x00546C6C, 0x00247894, 0x002B9FDA, 0x0031A2E5, 0x0026A3DD, 0x0025A5E0, 0x0034AADD, 0x001783AD, 0x001178A5, 0x003EA7D8, 0x003BA2D3, 0x00096991, 0x00328AA8, 0x002581AC, 0x00157EA5, 0x0042869D, 0x00738F89, 0x007E9A86,
        0x008FA299, 0x00313230, 0x00475043, 0x00545345, 0x00535244, 0x00515143, 0x00505042, 0x004E4E40, 0x004B4E3F, 0x004B4E3F, 0x00F4F5F4, 0x00838478, 0x00515242, 0x00E3E3E0, 0x0087887D, 0x004C4F40, 0x004B4E3F, 0x00797B70, 0x00FDFDFD, 0x00E9E9E7,
        0x00656557, 0x004F4F41, 0x004F5243, 0x00C9C9C5, 0x00FFFFFF, 0x00A4A49C, 0x004F5041, 0x004E4E40, 0x004D4D3F, 0x004D5041, 0x00515344, 0x00545646, 0x00535345, 0x004D4D3F, 0x00515445, 0x004B4E45, 0x004D4E42, 0x00515343, 0x00525343, 0x004F5242,
        0x004F5141, 0x004D4F40, 0x004F5142, 0x004E5040, 0x004E4F3F, 0x004D4E3E, 0x00535344, 0x004E5142, 0x004A4D3E, 0x004E5142, 0x004C4F40, 0x004E5142, 0x00515445, 0x004D5041, 0x004E5142, 0x00575747, 0x00565646, 0x00565646, 0x00565646, 0x00525242,
        0x004E4F3F, 0x004E4F3F, 0x004E4F3F, 0x004F5040, 0x004E4F3F, 0x004D4E3E, 0x00535444, 0x00525546, 0x00818379, 0x00F5F5F4, 0x004E5142, 0x004E5142, 0x00505344, 0x004B4E3F, 0x004C4F40, 0x004C4C3E, 0x001E3225, 0x008B9087, 0x007E9888, 0x00668778,
        0x00729898, 0x00287E90, 0x00237BA4, 0x002B8FA8, 0x00228CB5, 0x001A8EC5, 0x001B8CC5, 0x002E97C8, 0x003497BD, 0x00127496, 0x00006285, 0x001E709A, 0x0023323B, 0x0041402C, 0x00374833, 0x003F413B, 0x0048433A, 0x003F4637, 0x003C4336, 0x00414539,
        0x00404438, 0x00404438, 0x00414539, 0x0042463A, 0x00414539, 0x003F4337, 0x003C4034, 0x003E4538, 0x003C4336, 0x003C4336, 0x003C4336, 0x003E4538, 0x003F4639, 0x003E4538, 0x003D4437, 0x003D4338, 0x003C4237, 0x003C4237, 0x003D4338, 0x003D4338,
        0x003D4338, 0x003D4338, 0x003C4237, 0x003B3F33, 0x003C4034, 0x003E4236, 0x003D4135, 0x003C4034, 0x003A3E32, 0x003B3F33, 0x003B3F33, 0x003C4237, 0x003C4237, 0x003C4237, 0x003C4035, 0x003A3E33, 0x003D3E34, 0x003F4036, 0x00414238, 0x003D412E,
        0x00414135, 0x003D4239, 0x00384235, 0x00414131, 0x0039332C, 0x00162F33, 0x00034452, 0x0008475C, 0x000A495E, 0x000C4B60, 0x000D4C61, 0x000E4D62, 0x000F4E63, 0x00104F64, 0x00115065, 0x00004B68, 0x001A4E5B, 0x00164346, 0x00235659, 0x004F7073,
        0x00515752, 0x005C6C6B, 0x00255A64, 0x00566D6F, 0x0047454B, 0x004A5A61, 0x002B747C, 0x001D6D7E, 0x00557489, 0x009699A1, 0x00819287, 0x00375659, 0x003F6063, 0x0026494C, 0x003B5E61, 0x0085A6A9, 0x00A0BDC1, 0x0096AFB3, 0x0085999E, 0x008EA2A7,
        0x00809194, 0x00869292, 0x00646B66, 0x00404137, 0x003F3A2B, 0x00564F3C, 0x00574C38, 0x00413E3A, 0x003C4443, 0x00617C80, 0x0046787E, 0x005EA7AF, 0x0056B1B8, 0x0053BEC2, 0x0058C9CB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0x00595F64, 0x00848B8E, 0x00B1B9B9, 0x00AAB2B1, 0x00BFC7C6, 0x00D2DBD8, 0x00D3DDD7, 0x00DBEDE6, 0x00D0E0D9, 0x00D8E8E1, 0x00C8D8D1, 0x00BAC8C2, 0x00B1BFB9, 0x00BFCBC5, 0x00C4D0CA, 0x00B3BFB9, 0x00C2CEC8, 0x00C1CDC7, 0x00C0CCC6, 0x00B9C5BF,
        0x00B6C2BC, 0x00C6D2CC, 0x00CDD9D3, 0x00CCD7D4, 0x00BEC9C6, 0x0099A4A2, 0x00B4C1BF, 0x00B2BEBE, 0x00A2B0AF, 0x00849291, 0x00637072, 0x00727D7B, 0x0076817F, 0x007E8987, 0x00727D7B, 0x006E7977, 0x00636E6C, 0x005B6664, 0x00485351, 0x00364442,
        0x00324F56, 0x001F4355, 0x00153A50, 0x000D4558, 0x00085C6E, 0x0017718F, 0x00246F9B, 0x005D707D, 0x009FA1A2, 0x00C0C8BD, 0x004C7161, 0x00376259, 0x0094ABAD, 0x0097AEB0, 0x0075A39D, 0x00255063, 0x00114159, 0x0097BABE, 0x00D1DDBF, 0x006B724F,
        0x00445B4D, 0x003A524A, 0x002A3622, 0x00193431, 0x002D575C, 0x0000394B, 0x0008708D, 0x0046C4E7, 0x001C9DC4, 0x00006991, 0x003198BF, 0x002CA4E0, 0x0027A1DD, 0x002AA7E1, 0x002BAAE1, 0x002FA8E0, 0x0034A5DD, 0x00258CC3, 0x001F83BA, 0x00087AAF,
        0x00308898, 0x004A6B5D, 0x00A7C4BB, 0x0058B3C8, 0x00209DD0, 0x0034ACE7, 0x0026A7D8, 0x001EA4DA, 0x0037B0DC, 0x00339DC2, 0x0000618B, 0x00026CA1, 0x0040B0EC, 0x0031A1D6, 0x002A99C7, 0x00349FD2, 0x00249BC8, 0x004393AA, 0x00638880, 0x006C9078,
        0x0089A396, 0x002B3633, 0x00394D40, 0x00525143, 0x00515042, 0x004F4F41, 0x004E4E40, 0x004D4D3F, 0x004A4D3E, 0x004B4E3F, 0x00F4F5F4, 0x00828377, 0x004B4C3C, 0x00E8E8E6, 0x00EEEEED, 0x00626457, 0x004B4E3F, 0x004A4D3E, 0x00A4A49C, 0x00FFFFFF,
        0x00CCCCC8, 0x00535445, 0x004C4F40, 0x005C5F51, 0x00E6E6E4, 0x00FFFFFF, 0x00D5D5D1, 0x00D3D3D0, 0x00D2D2CF, 0x00D2D2CF, 0x00D2D2CF, 0x00D2D3CF, 0x00D2D2CE, 0x00D1D1CD, 0x00D1D2CE, 0x00D0D1CF, 0x00D2D2CF, 0x00D2D2CF, 0x00D2D2CE, 0x00D3D3CF,
        0x00D3D4D0, 0x00D3D3CF, 0x00D3D3CF, 0x00D2D3CF, 0x00D2D2CE, 0x00D2D2CE, 0x00D1D1CD, 0x00D1D2CF, 0x00D1D3CF, 0x00D1D2CF, 0x00D1D2CE, 0x00D1D2CE, 0x00D2D3CF, 0x00D2D3CF, 0x00D2D3CF, 0x00D4D4D0, 0x00D4D4D0, 0x00D4D4D0, 0x00D4D4D0, 0x00D2D2CE,
        0x00D2D2CE, 0x00D2D2CE, 0x00D2D2CE, 0x00D2D2CE, 0x00D3D3CF, 0x00CFCFCB, 0x00808175, 0x004E5142, 0x007F8176, 0x00F5F5F4, 0x004C4F40, 0x004E5142, 0x00515445, 0x004D5041, 0x004E5142, 0x0049473C, 0x001D2F22, 0x008A9085, 0x007C9684, 0x005D8371,
        0x00699294, 0x00217892, 0x001570A3, 0x00077593, 0x002394BF, 0x00269BD2, 0x001A8BC4, 0x00238ABB, 0x003299C0, 0x002C97BD, 0x001082AB, 0x0012648E, 0x0022313A, 0x00403F2B, 0x002D3E29, 0x0032342E, 0x003B362D, 0x00313829, 0x002C3326, 0x0032362A,
        0x0032362A, 0x0033372B, 0x0034382C, 0x00363A2E, 0x0035392D, 0x0032362A, 0x002F3327, 0x00343B2E, 0x0032392C, 0x0030372A, 0x0030372A, 0x0031382B, 0x0031382B, 0x0030372A, 0x002E3528, 0x002E3429, 0x002E3429, 0x002E3429, 0x002D3328, 0x002C3227,
        0x002C3227, 0x002D3328, 0x002D3328, 0x002E3226, 0x00303428, 0x0032362A, 0x0032362A, 0x00303428, 0x002F3327, 0x002E3226, 0x002F3327, 0x002D3328, 0x002E3429, 0x002E3429, 0x002E3227, 0x002D3126, 0x00303127, 0x0033342A, 0x0036372D, 0x00313522,
        0x00353529, 0x0031362D, 0x002C3629, 0x00373727, 0x00322C25, 0x00122B2F, 0x00024351, 0x000C4B60, 0x000C4B60, 0x000D4C61, 0x000C4B60, 0x000B4A5F, 0x000C4B60, 0x000F4E63, 0x00115065, 0x000A4B67, 0x0010475C, 0x0000415A, 0x00167196, 0x003184A4,
        0x00002D3A, 0x00124E4D, 0x003C9794, 0x00175954, 0x00003632, 0x00114D4D, 0x0000444D, 0x00004456, 0x00095462, 0x001C5E5C, 0x0034705E, 0x0046797C, 0x00194E51, 0x00073E41, 0x0000373A, 0x00154A4D, 0x00649498, 0x005C898D, 0x00083035, 0x00000E13,
        0x00000205, 0x00000404, 0x00040B06, 0x00323329, 0x004A4536, 0x00423B28, 0x00332814, 0x00554735, 0x00493E30, 0x001F1A11, 0x00151312, 0x00161819, 0x00181B1F, 0x001A1D21, 0x00111418, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0x005B6265, 0x0080878A, 0x00A6AEAE, 0x009FA7A7, 0x00B9C1C0, 0x00C3CBCA, 0x00B1B9B8, 0x00D4E1D9, 0x00D3E0D8, 0x00D5E2DA, 0x00CAD7CF, 0x00B7C4BC, 0x00CFDCD4, 0x00CCD9D1, 0x00B6C3BB, 0x00A6B4B0, 0x00ABB9B5, 0x00B8C6C2, 0x00B8C6C2, 0x00B7C5C1,
        0x00B6C4C0, 0x00AEBCB8, 0x00B5C3BF, 0x00BBC7C1, 0x00B6C2BC, 0x00AFBBB5, 0x00B4C2BE, 0x00ADBBB7, 0x00A8B6B4, 0x00758381, 0x006A7876, 0x00808784, 0x00878E8B, 0x008D9491, 0x00767D7A, 0x00737A77, 0x006D7471, 0x00666D6A, 0x00606764, 0x00374141,
        0x002D4D53, 0x00194251, 0x00133241, 0x00082730, 0x00013135, 0x00003642, 0x0020546B, 0x004E796C, 0x0096A4A0, 0x008D999B, 0x004E7779, 0x004D8586, 0x00557B7F, 0x002E4D50, 0x00719D9C, 0x0000514E, 0x00145A59, 0x005A8685, 0x007A8F8D, 0x00555D5D,
        0x00434D4D, 0x003F5452, 0x00193837, 0x0005503A, 0x001F5A52, 0x000A3E4A, 0x00004A61, 0x00006F89, 0x005CD6EC, 0x00258799, 0x000F5465, 0x002B96B8, 0x0037ADD7, 0x0029A9E4, 0x0017A2E7, 0x0019A1E9, 0x0028A6E7, 0x0037A7DC, 0x00288EB9, 0x001A89B1,
        0x002189AE, 0x003491B2, 0x00277EA0, 0x002782A9, 0x003CA2D6, 0x002DA4E2, 0x001B9CE1, 0x0026A6E1, 0x001D9ED7, 0x002DABE1, 0x00269DCE, 0x00005E87, 0x00116E8D, 0x004391A8, 0x0042879A, 0x001B87B1, 0x002995AD, 0x0039959A, 0x00548F8B, 0x00778C8D,
        0x00A2A1A3, 0x002E322C, 0x0040503F, 0x00505344, 0x004E5142, 0x004C4F40, 0x004B4E3F, 0x004B4E3F, 0x004C4F40, 0x004B4E3F, 0x00F4F5F4, 0x007F8177, 0x004F5243, 0x00E1E1DF, 0x00FFFFFF, 0x00CFD0CC, 0x004F5244, 0x004C4F40, 0x00545444, 0x00C8C8C3,
        0x00FFFFFF, 0x00A4A59D, 0x004C4E3F, 0x004B4E3F, 0x0075776B, 0x00ECECEA, 0x00FAFAFA, 0x00FAFAFA, 0x00FAFAFA, 0x00FAFAFA, 0x00FAFAFA, 0x00FAFAFA, 0x00FAFAFA, 0x00FAFAFA, 0x00FAFAFA, 0x00FAFAFA, 0x00FAFAFA, 0x00FAFAFA, 0x00FAFAFA, 0x00FAFAFA,
        0x00FAFAFA, 0x00FAFAFA, 0x00FAFAFA, 0x00FAFAFA, 0x00FAFAFA, 0x00FAFAFA, 0x00FAFAFA, 0x00FAFAFA, 0x00FAFAFA, 0x00FAFAFA, 0x00FAFAFA, 0x00FAFAFA, 0x00FAFAFA, 0x00FAFAFA, 0x00FAFAFA, 0x00FAFAFA, 0x00FAFAFA, 0x00FAFAFA, 0x00FAFAFA, 0x00FAFAFA,
        0x00FAFAFA, 0x00FAFAFA, 0x00FAFAFA, 0x00FAFAFA, 0x00FAFAFA, 0x00F8F8F7, 0x008D8E84, 0x004B5041, 0x00808379, 0x00F4F5F4, 0x00494E3F, 0x00494E3F, 0x004D5243, 0x004E5344, 0x004D5243, 0x00434B40, 0x00272B1F, 0x008C8E82, 0x00869187, 0x00587778,
        0x005998A6, 0x002786A0, 0x001486AA, 0x002C8992, 0x00168698, 0x00149AB8, 0x00129CCA, 0x000886C0, 0x00198CC9, 0x002A9BD4, 0x002195CA, 0x000478B3, 0x000C4E6B, 0x0053686A, 0x00253331, 0x004D6A61, 0x00103521, 0x00507261, 0x0013342D, 0x0051625E,
        0x0019362D, 0x00335C4D, 0x00224737, 0x00405A4C, 0x00304438, 0x00364D45, 0x002C4842, 0x00304B47, 0x00374E46, 0x00314237, 0x003A5444, 0x001E412D, 0x0040634F, 0x00213829, 0x004A564A, 0x00152D25, 0x0056685B, 0x00212E1E, 0x0056655D, 0x00203537,
        0x00455F65, 0x00293F3D, 0x004E5F54, 0x00193C3F, 0x0035585B, 0x001D4043, 0x00335659, 0x001F4245, 0x002F5255, 0x00274A4D, 0x00284B4E, 0x002D5053, 0x00224548, 0x0035585B, 0x00133639, 0x003D6063, 0x0014373A, 0x00395C5F, 0x0014373A, 0x004F5B5B,
        0x002A3C3B, 0x004C5957, 0x0025282D, 0x004D5D6E, 0x001B5370, 0x00499CC9, 0x00015890, 0x0009495B, 0x000A4A5C, 0x000D4D5F, 0x00104F63, 0x000F4E62, 0x000D4C61, 0x000E4D62, 0x00115065, 0x000E5063, 0x00114F61, 0x00043C4D, 0x004F8390, 0x0078ABB5,
        0x00295B61, 0x00558A8D, 0x008FC7C8, 0x0047868E, 0x0024534B, 0x00527E77, 0x002E707B, 0x000D5F71, 0x0064A8AF, 0x009DC5C3, 0x00C4DCDC, 0x0095B3B8, 0x00527277, 0x0022484A, 0x00335959, 0x0093B8B6, 0x0084A5A1, 0x0087A4A1, 0x006B8480, 0x00425856,
        0x00344F53, 0x002B454C, 0x00283837, 0x003C3E32, 0x00564F3B, 0x00534C39, 0x00484439, 0x00574A3C, 0x0048463C, 0x00394743, 0x00284748, 0x00316064, 0x00427980, 0x003C7780, 0x003C7A82, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0x005D6467, 0x007C8386, 0x00A7AFAF, 0x00B2BABA, 0x00BAC2C1, 0x00B6BEBD, 0x00C0C8C7, 0x00CCD9D1, 0x00C7D4CC, 0x00D5E2DA, 0x00DBE8E0, 0x00C0CDC5, 0x00BAC7BF, 0x00B9C6BE, 0x00C0CDC5, 0x00BCCAC6, 0x00BCCAC6, 0x00B5C3BF, 0x00B5C3BF, 0x00B6C4C0,
        0x00B8C6C2, 0x00C2D0CC, 0x00C6D4D0, 0x00B9C5BF, 0x00B9C5BF, 0x00B7C3BD, 0x00B5C3BF, 0x00AAB8B4, 0x00B0BEBC, 0x00808E8C, 0x0063716F, 0x007E8582, 0x008B928F, 0x008E9592, 0x00767D7A, 0x00787F7C, 0x00707774, 0x00656C69, 0x00656C69, 0x00354141,
        0x00204248, 0x000B3443, 0x000A2938, 0x0006252E, 0x00003034, 0x00002A37, 0x00053950, 0x002F5D57, 0x003B5052, 0x0044565D, 0x00427077, 0x003B777D, 0x001E4C54, 0x001A3F47, 0x0075A5A9, 0x006B9B9C, 0x0056807F, 0x006E8F8B, 0x006F8681, 0x00283831,
        0x004D5A52, 0x00434E46, 0x00404B43, 0x00B8C9C0, 0x0091B9B8, 0x00216670, 0x00004D64, 0x00075070, 0x008CD2F0, 0x003F90A5, 0x0011717E, 0x004CABC5, 0x003199BE, 0x002DA3D7, 0x002CACED, 0x001FA3E9, 0x0026A6E7, 0x0035ACE4, 0x0032A2D6, 0x001D90BD,
        0x001A86B0, 0x002687AD, 0x001C779C, 0x00217DA6, 0x003399CA, 0x002699D2, 0x001591D1, 0x002AA4E0, 0x0027A3DF, 0x0021A0D7, 0x002DAADC, 0x002396C1, 0x0004688B, 0x00004662, 0x000A546C, 0x0025698C, 0x005BA3B5, 0x00589B9C, 0x005E8B88, 0x00768889,
        0x0095969A, 0x0030342E, 0x00475444, 0x004F5243, 0x004E5142, 0x004C4F40, 0x004C4F40, 0x004D5041, 0x004D5041, 0x004C4F40, 0x00F4F5F4, 0x007D7F74, 0x004D5041, 0x00E1E1DF, 0x00FFFFFF, 0x00FFFFFF, 0x00A9AAA3, 0x00515243, 0x00545545, 0x00616254,
        0x00E5E5E3, 0x00FDFDFD, 0x0078796E, 0x004C4E3F, 0x004B4E3F, 0x00575848, 0x00525344, 0x00535445, 0x00555647, 0x00525547, 0x00525547, 0x00515446, 0x00595A4A, 0x00535547, 0x004D5042, 0x00535648, 0x00585949, 0x00565747, 0x00595949, 0x004F4F40,
        0x00545546, 0x00575949, 0x00565949, 0x00555848, 0x00555848, 0x00525546, 0x00545647, 0x00515647, 0x0054594A, 0x00555A4B, 0x00535849, 0x00525748, 0x00535849, 0x00525748, 0x004F5446, 0x00585949, 0x00595A4A, 0x005A5C4C, 0x005E5F4E, 0x00535445,
        0x00535445, 0x00555647, 0x00535445, 0x00525344, 0x00545546, 0x00545546, 0x004D4E3E, 0x004B5041, 0x0081847A, 0x00F5F5F4, 0x004A4F40, 0x004A4F40, 0x004C5142, 0x004C5142, 0x004A4F40, 0x0041473C, 0x002E3226, 0x008A9085, 0x00829189, 0x00547370,
        0x00548A91, 0x004693A3, 0x00348FA4, 0x007FB9BF, 0x003F909F, 0x001D89A8, 0x002098C6, 0x001992CA, 0x001F90CE, 0x002497D0, 0x001E93CA, 0x001E9BD5, 0x00277D9B, 0x0070A4AA, 0x00386366, 0x00668F91, 0x0044675D, 0x0077988A, 0x003A5D59, 0x00678988,
        0x0035615B, 0x0054887E, 0x004D7B6F, 0x00698C82, 0x00486760, 0x004F726E, 0x004B7875, 0x00436C6F, 0x006A8B8E, 0x00486262, 0x005B7A77, 0x003B645F, 0x005B8A87, 0x003F6668, 0x007A989D, 0x00285A66, 0x0049767A, 0x003B6362, 0x00729CA1, 0x00558694,
        0x006CA0B0, 0x004D7E86, 0x00739F9E, 0x00568A91, 0x00689CA3, 0x005A8E95, 0x00699DA4, 0x005B8F96, 0x0064989F, 0x005E9299, 0x005C9097, 0x0062969D, 0x005D9198, 0x00679BA2, 0x00558990, 0x006B9FA6, 0x00578B92, 0x006B9FA6, 0x0053878E, 0x006EA2AF,
        0x004A858E, 0x00609396, 0x003E5D60, 0x006B8C8F, 0x00245D5F, 0x004B9198, 0x0073B3BE, 0x000F4F61, 0x000C4C5E, 0x000B4B5D, 0x000E4D61, 0x000E4D61, 0x000B4A5F, 0x000C4B60, 0x00115065, 0x000B4D60, 0x000F4D5F, 0x000E4657, 0x00245966, 0x004C7F89,
        0x003A6E74, 0x004E8588, 0x00619A9B, 0x00818E9E, 0x003E3E3E, 0x00747A79, 0x00698D9D, 0x00427E94, 0x00427982, 0x007FA4A2, 0x00768E8E, 0x00305357, 0x00496E72, 0x00284F51, 0x00396160, 0x0080A7A5, 0x008DB0AC, 0x0086A5A2, 0x008AA5A1, 0x008AA09E,
        0x0086A1A5, 0x007C969D, 0x005E6E6D, 0x004B4D41, 0x004B4430, 0x0049422F, 0x0049453A, 0x00493C2C, 0x0056564A, 0x005C736B, 0x00568582, 0x00579B9A, 0x0059ACAD, 0x0056B3B4, 0x0059BBBB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0x00575E61, 0x00838A8D, 0x00BAC2C2, 0x00B6BEBE, 0x00B8C0BF, 0x00C3CBCA, 0x00D4DCDB, 0x00CBD7D1, 0x00C4D0CA, 0x00BAC6C0, 0x00ADB9B3, 0x00ABB7B1, 0x00C3CFC9, 0x00BDC9C3, 0x00ABB7B1, 0x00ABB9B5, 0x00BDCBC7, 0x00BDCBC7, 0x00BAC8C4, 0x00B3C1BD,
        0x00B6C4C0, 0x00CAD8D4, 0x00C7D5D1, 0x00AFBBB5, 0x00B1BDB7, 0x00B0BCB6, 0x00A3B1AD, 0x0097A5A1, 0x00AAB8B6, 0x007F8D8B, 0x00556361, 0x0078817E, 0x0087908D, 0x0089928F, 0x00707976, 0x00757E7B, 0x006B7471, 0x00606966, 0x00656E6B, 0x00344143,
        0x00143840, 0x00002B3A, 0x00062737, 0x0007262F, 0x0006363C, 0x00002E3B, 0x00003148, 0x000D4147, 0x00193944, 0x00244352, 0x00194D5D, 0x000C4D5C, 0x000A4352, 0x001E4F5D, 0x003C707C, 0x00617177, 0x0063787A, 0x008DA9A9, 0x0086A6A1, 0x002E4B42,
        0x00465A4E, 0x00434B40, 0x00646458, 0x00C9C7C7, 0x0089AEB2, 0x00509AA6, 0x003A889F, 0x00225F7F, 0x005D94B3, 0x0026758A, 0x0050BAC5, 0x00438FA1, 0x000F647E, 0x002B8FB9, 0x0044B5ED, 0x0026A2E2, 0x0020A0E1, 0x0026A7E6, 0x0025A3DE, 0x0030A7DE,
        0x001D8EC0, 0x001F88B4, 0x002587B1, 0x002B8DB7, 0x003096C6, 0x002795C9, 0x00289CD5, 0x001A89C7, 0x00279CD9, 0x0025A3DD, 0x0025A7DC, 0x002BA7D7, 0x002D9BC5, 0x0008668A, 0x00003555, 0x00456277, 0x0082A6B0, 0x00618987, 0x00597B75, 0x00748385,
        0x008D9295, 0x002B322B, 0x00384432, 0x004B4E3F, 0x004B4E3F, 0x004A4D3E, 0x004B4E3F, 0x004C4F40, 0x004C4F40, 0x004B4E3F, 0x00F4F4F4, 0x007C7E73, 0x004C4F40, 0x00E1E1DF, 0x00FFFFFF, 0x00FFFFFF, 0x00A9AAA2, 0x00555545, 0x00575747, 0x00636355,
        0x00E5E5E3, 0x00FDFDFD, 0x007E7E72, 0x004F5141, 0x004D4F40, 0x00545749, 0x00585B4D, 0x0055584A, 0x00555547, 0x00535648, 0x00545749, 0x00545749, 0x005A5B4D, 0x00525445, 0x004F5244, 0x0056584B, 0x005A5B4C, 0x00595A4B, 0x005B5B4C, 0x005A5A4B,
        0x00565748, 0x00585849, 0x005B5C4D, 0x0058594A, 0x00575849, 0x00595A4C, 0x0057594B, 0x0053584A, 0x00565B4D, 0x00585C4E, 0x00565A4C, 0x0055594B, 0x00565B4D, 0x0054594B, 0x00515648, 0x00585B4C, 0x00595B4C, 0x00595C4D, 0x005C5E4E, 0x00545748,
        0x00565748, 0x00565748, 0x00555647, 0x00555647, 0x00575849, 0x00575848, 0x004F5040, 0x004A4F40, 0x0081847A, 0x00F5F5F4, 0x004C5142, 0x004B5041, 0x004C5142, 0x004B5041, 0x00484D3E, 0x0045473B, 0x00303429, 0x00848D83, 0x0082958C, 0x0062807B,
        0x00668E8C, 0x0074A6A6, 0x00548C8D, 0x00829799, 0x002E5B68, 0x00055671, 0x00218AB5, 0x00289CD3, 0x002498D3, 0x002296D1, 0x002195CE, 0x001B99D4, 0x001B87A6, 0x002B848F, 0x0013616E, 0x00316D79, 0x00557C7E, 0x00678A86, 0x00537D7C, 0x0041797E,
        0x001C5C5D, 0x00337270, 0x00568D8A, 0x0074A09F, 0x005F8B8C, 0x003D7477, 0x0024676A, 0x002E6B75, 0x0063949E, 0x00608790, 0x004D767F, 0x003F767F, 0x0021616C, 0x00246476, 0x004B859B, 0x004D8983, 0x00478379, 0x00488882, 0x00318089, 0x001F82A0,
        0x001B8EB9, 0x000C84B2, 0x001C93C0, 0x002183AD, 0x002688B2, 0x002789B3, 0x002D8FB9, 0x002A8CB6, 0x002D8FB9, 0x002C8EB8, 0x002A8CB6, 0x00298BB5, 0x002A8CB6, 0x002D8FB9, 0x00298BB5, 0x002A8CB6, 0x002C8EB8, 0x002F91BB, 0x002385AF, 0x002690C5,
        0x001487B4, 0x00238BB0, 0x0004526F, 0x00094B5D, 0x00004148, 0x001F5F5F, 0x0076A5A3, 0x00135365, 0x000C4C5E, 0x0009495B, 0x000D4C60, 0x000D4C60, 0x0009485D, 0x000A495E, 0x00104F64, 0x00105166, 0x000D4A5E, 0x00124C5F, 0x00033A49, 0x0000323E,
        0x0000353C, 0x00124C51, 0x00023D3F, 0x0003424A, 0x00114038, 0x001A4B43, 0x00004650, 0x00004455, 0x00004B51, 0x00316460, 0x00587D7B, 0x00557B7F, 0x00214A4D, 0x000C3839, 0x000B3736, 0x00234E4B, 0x00123733, 0x0033524F, 0x001A3833, 0x00001412,
        0x00001115, 0x0000161D, 0x000E1E1D, 0x00333529, 0x004E4733, 0x003E3724, 0x00252116, 0x004D412F, 0x00423B2C, 0x00232119, 0x000E1011, 0x000F1119, 0x00181622, 0x00231929, 0x00261929, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0x0051585B, 0x0080878A, 0x00B7BFBF, 0x00A7AFAF, 0x00A7AFAE, 0x00B9C1C0, 0x00CDD5D4, 0x00B5C1BB, 0x00C3CFC9, 0x00D1DDD7, 0x00C8D4CE, 0x00B9C5BF, 0x00B7C3BD, 0x00ACB8B2, 0x00A6B2AC, 0x009DABA7, 0x00B5C3BF, 0x00C7D5D1, 0x00C2D0CC, 0x00B3C1BD,
        0x00B5C3BF, 0x00BAC8C4, 0x00ADBBB7, 0x00A7B3AD, 0x00B0BCB6, 0x00B6C2BC, 0x00AFBDB9, 0x00ADBBB7, 0x00B9C7C5, 0x00879593, 0x005E6C6A, 0x007B8481, 0x00838C89, 0x0087908D, 0x00737C79, 0x00737C79, 0x006C7572, 0x00666F6C, 0x0067706D, 0x00334244,
        0x0011363E, 0x00012E3C, 0x00092A3A, 0x0005232E, 0x0008363D, 0x00003645, 0x00063751, 0x00063F4E, 0x000F3D4F, 0x00113D54, 0x00043C55, 0x00004158, 0x0003465B, 0x00185366, 0x000D4354, 0x0013333E, 0x000F333D, 0x00224B54, 0x001C484F, 0x00062F32,
        0x00092928, 0x00122925, 0x001C2C25, 0x00265154, 0x0025565E, 0x00104A5D, 0x002B738B, 0x004195AF, 0x002A829A, 0x002D8395, 0x00196C7B, 0x00124F59, 0x00004053, 0x000D5F7E, 0x002F8DBB, 0x0038A6E0, 0x002EA9E9, 0x001EA2E2, 0x0019A3E5, 0x002DA8E6,
        0x002197D2, 0x002191C5, 0x00218BBA, 0x00248BB8, 0x002891BD, 0x002993C2, 0x00329ECE, 0x00369DDB, 0x002393CF, 0x00209CD8, 0x0022A7E0, 0x0024A6DB, 0x0039B0E1, 0x002E96C1, 0x00005880, 0x00486371, 0x00739497, 0x005F847C, 0x00668780, 0x00798E8F,
        0x008E9A9C, 0x00333B34, 0x00454D3C, 0x00474A3B, 0x00474A3B, 0x00474A3B, 0x00484B3C, 0x004A4D3E, 0x004A4D3E, 0x00484B3C, 0x00F4F4F3, 0x007B7D72, 0x004C4F40, 0x00E1E1DF, 0x00FFFFFF, 0x00D1D1CD, 0x00565647, 0x00555545, 0x00555545, 0x00C9C9C4,
        0x00FFFFFF, 0x00A5A69E, 0x00545444, 0x00515242, 0x00797B6F, 0x00EDEDEC, 0x00FCFCFC, 0x00FDFDFD, 0x00FDFDFD, 0x00FDFDFD, 0x00FDFDFD, 0x00FDFDFD, 0x00FDFDFD, 0x00FDFDFD, 0x00FDFDFD, 0x00FDFDFD, 0x00FDFDFD, 0x00FDFDFD, 0x00FDFDFD, 0x00FDFDFD,
        0x00FDFDFD, 0x00FDFDFD, 0x00FDFDFD, 0x00FDFDFD, 0x00FDFDFD, 0x00FDFDFD, 0x00FDFDFD, 0x00FDFDFD, 0x00FDFDFD, 0x00FDFDFD, 0x00FDFDFD, 0x00FDFDFD, 0x00FDFDFD, 0x00FDFDFD, 0x00FDFDFD, 0x00FDFDFD, 0x00FDFDFD, 0x00FDFDFD, 0x00FDFDFD, 0x00FDFDFD,
        0x00FDFDFD, 0x00FDFDFD, 0x00FDFDFD, 0x00FDFDFD, 0x00FDFDFD, 0x00FAFAFA, 0x00919288, 0x004A4F40, 0x0081847A, 0x00F5F5F4, 0x004C5142, 0x004B5041, 0x004D5243, 0x004B5041, 0x00484D3E, 0x004C4A3F, 0x002D3126, 0x00808D85, 0x007A9189, 0x0066837A,
        0x006B8D83, 0x007D9F94, 0x00547567, 0x007B8A86, 0x0031575C, 0x000B546A, 0x002A91B8, 0x002AA0D3, 0x001B96D0, 0x001C92CD, 0x002497D0, 0x002194D1, 0x002392B2, 0x0005707E, 0x00288A9C, 0x001F687E, 0x00619098, 0x0052817F, 0x0064A0A0, 0x003C848C,
        0x002F797F, 0x002C7376, 0x004A8789, 0x004C7C80, 0x0051858B, 0x00387981, 0x002A7B82, 0x0029757B, 0x004C8A92, 0x0051828A, 0x00538690, 0x00478792, 0x00348191, 0x0025768B, 0x002C7C95, 0x00748A85, 0x005B736B, 0x00597B75, 0x00205661, 0x001C6D90,
        0x001B7FB6, 0x002690D0, 0x00258FCF, 0x000E85BD, 0x000B82BA, 0x00138AC2, 0x00138AC2, 0x00158CC4, 0x00158CC4, 0x00178EC6, 0x00178EC6, 0x001289C1, 0x00138AC2, 0x001990C8, 0x00138AC2, 0x001188C0, 0x00168DC5, 0x00168DC5, 0x000E85BD, 0x001384C2,
        0x00148FC7, 0x001E93C6, 0x0018749F, 0x0000395A, 0x0000485A, 0x000D464F, 0x005C7E84, 0x00135365, 0x000B4B5D, 0x00074759, 0x000D4C60, 0x000E4D61, 0x0009485D, 0x000A495E, 0x000F4E63, 0x000A4A62, 0x0008475C, 0x00134E62, 0x00174F60, 0x000A414E,
        0x00215C65, 0x0069A7AD, 0x00529296, 0x005F98A1, 0x00638C85, 0x00779D97, 0x004A8692, 0x001F667A, 0x00679EA7, 0x00A8C4C4, 0x00D2DEE0, 0x0078A3A6, 0x003B686B, 0x001D4B4C, 0x002D5C5A, 0x0084AFAC, 0x00668B87, 0x006B8A87, 0x004E6965, 0x003B514F,
        0x00355054, 0x00334D54, 0x002C3C3B, 0x0037392D, 0x004B4430, 0x004B4431, 0x00464237, 0x00604F3C, 0x00554D3C, 0x00363C31, 0x00304340, 0x00395859, 0x003D6266, 0x003C6469, 0x0031595E, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0x005B6265, 0x007C8386, 0x00ABB3B3, 0x00AFB7B7, 0x00A7AFAE, 0x00A3ABAA, 0x00BAC2C1, 0x00B7C3BD, 0x00B4C0BA, 0x00BAC6C0, 0x00B9C5BF, 0x00C4D0CA, 0x00CFDBD5, 0x00C4D0CA, 0x00B3BFB9, 0x00B7C5C1, 0x00ABB9B5, 0x00BBC9C5, 0x00B4C2BE, 0x00ABB9B5,
        0x00B6C4C0, 0x00B3C1BD, 0x00B2C0BC, 0x00A9B5AF, 0x00AAB6B0, 0x00A5B1AB, 0x009EACA8, 0x00AAB8B4, 0x00AFBDBB, 0x00849290, 0x00717F7D, 0x0085908D, 0x007F8A87, 0x00889390, 0x00798481, 0x00727D7A, 0x00707B78, 0x00707B78, 0x00687370, 0x002D4043,
        0x000D363F, 0x00012F40, 0x000A2C3C, 0x0001212C, 0x0004323A, 0x00003242, 0x0004334E, 0x00033E52, 0x00043D53, 0x00063B56, 0x000A425F, 0x00084B66, 0x0000485E, 0x00074B5E, 0x000F4458, 0x0007505E, 0x00004556, 0x00054459, 0x00023C52, 0x000A3F54,
        0x00235567, 0x0020505C, 0x0005363E, 0x00004B51, 0x00084352, 0x001A4B61, 0x00003852, 0x0000536B, 0x00005368, 0x00044D61, 0x000B3E52, 0x000D4550, 0x001A5565, 0x00003F58, 0x00035175, 0x003694C2, 0x003AA9E1, 0x0021A0DD, 0x001BA3E3, 0x001E9CDD,
        0x0029A4E2, 0x0029A1DC, 0x00188ABF, 0x001281B1, 0x00218BBA, 0x002790BC, 0x002990BD, 0x00359CD5, 0x002190C8, 0x001590CA, 0x00199CD5, 0x0024A5DC, 0x002BA5DB, 0x0033A2D4, 0x00389ECF, 0x00266476, 0x00215E62, 0x002F635C, 0x005C857D, 0x00718E8B,
        0x008D9D9C, 0x002A352D, 0x003D4332, 0x00484B3C, 0x00474A3B, 0x00474A3B, 0x00494C3D, 0x004A4D3E, 0x004A4D3E, 0x00484B3C, 0x00F4F4F3, 0x007C7E73, 0x004C4F40, 0x00E8E8E6, 0x00EEEEED, 0x0068685B, 0x00535345, 0x00505344, 0x00A3A49D, 0x00FFFFFF,
        0x00CCCDC8, 0x005B5C4D, 0x00565646, 0x00646456, 0x00E6E7E4, 0x00FFFFFF, 0x00D1D1CE, 0x00CECECA, 0x00CECFCA, 0x00CFD0CC, 0x00CFCFCB, 0x00CECFCB, 0x00CFD0CC, 0x00D0D0CC, 0x00CFD0CB, 0x00D0D1CC, 0x00D1D1CD, 0x00D0D0CC, 0x00D1D1CD, 0x00D1D1CD,
        0x00D0D0CC, 0x00CFCFCB, 0x00D1D1CD, 0x00D2D2CD, 0x00D2D2CE, 0x00D1D2CD, 0x00D0D0CC, 0x00D0D0CC, 0x00D1D2CD, 0x00D0D1CC, 0x00D0D0CC, 0x00CFD0CC, 0x00D0D0CC, 0x00CFCFCB, 0x00CFD0CC, 0x00CFD0CB, 0x00CFD0CB, 0x00CFD0CC, 0x00CFD0CC, 0x00CFD1CC,
        0x00CFD0CC, 0x00CECFCB, 0x00CFCFCB, 0x00CFCFCB, 0x00CFCFCB, 0x00CCCDC8, 0x007D7D71, 0x00494E3F, 0x00808379, 0x00F4F5F4, 0x004A4F40, 0x004B5041, 0x004D5243, 0x004D5243, 0x004B5041, 0x004D4B40, 0x002A2E23, 0x008B9890, 0x007F968E, 0x006A877E,
        0x0072948A, 0x0077998E, 0x00547567, 0x00386159, 0x0012494C, 0x00025466, 0x002691B6, 0x00219CCE, 0x001695CC, 0x001C94CF, 0x002697D0, 0x003193CF, 0x001D7D9B, 0x00248491, 0x002D899A, 0x00176074, 0x001A4E55, 0x00346B68, 0x002E7473, 0x001D6168,
        0x0010575B, 0x00256869, 0x00276061, 0x002F5F61, 0x00205054, 0x001D5B61, 0x000C5A60, 0x0017645D, 0x00225F5B, 0x00376866, 0x00235553, 0x00286868, 0x00146164, 0x00095A62, 0x00105F6C, 0x002B697B, 0x0028636C, 0x00104F4D, 0x00206868, 0x00116773,
        0x003695A9, 0x003190A4, 0x003A93A7, 0x003290AD, 0x00308EAB, 0x00318FAC, 0x00318FAC, 0x002D8BA8, 0x002F8DAA, 0x002E8CA9, 0x003290AD, 0x00308EAB, 0x002C8AA7, 0x003B99B6, 0x002987A4, 0x003391AE, 0x002B89A6, 0x003492AF, 0x002886A3, 0x004290A7,
        0x00308DA2, 0x003A95AA, 0x003C859B, 0x0027667A, 0x00094A58, 0x000D4550, 0x00527680, 0x000E4E60, 0x0008485A, 0x00074759, 0x000D4C60, 0x000F4E62, 0x000B4A5F, 0x000B4A5F, 0x000E4D62, 0x000D4F68, 0x00114F67, 0x00104B5F, 0x000F495C, 0x0005404F,
        0x0015535D, 0x00408189, 0x0024686D, 0x00677C8B, 0x00394140, 0x00687674, 0x00497684, 0x00276A7F, 0x0037757D, 0x00719997, 0x006C8888, 0x00275457, 0x003C696C, 0x001E4C4D, 0x00305F5D, 0x007EA6A4, 0x008BAEAA, 0x0085A29F, 0x007E9793, 0x00889E9C,
        0x00839EA2, 0x0079939A, 0x005C6C6B, 0x004B4D41, 0x004E4733, 0x004E4734, 0x004F4B40, 0x004B4430, 0x00575D4C, 0x00547063, 0x005E938A, 0x0066B2AE, 0x0064BFBC, 0x0063C9C4, 0x005DC7C1, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0x00555C5F, 0x00838A8D, 0x00BCC4C4, 0x00BCC4C4, 0x00BCC4C3, 0x00B4BCBB, 0x00B0B8B7, 0x00A1ACA9, 0x00B7C2BF, 0x00D0DBD8, 0x00BEC9C6, 0x00ACB7B4, 0x00ADB8B5, 0x00B2BDBA, 0x00ADB8B5, 0x00B5C3BF, 0x009DABA7, 0x00BBC9C5, 0x00C0CECA, 0x00B1BFBB,
        0x00AFBDB9, 0x00A2B0AC, 0x00ABB9B5, 0x00A3AFA9, 0x00ABB7B1, 0x00AFBBB5, 0x00B1BFBB, 0x00BFCDC9, 0x00B0BEBC, 0x00768482, 0x0063716F, 0x00889390, 0x00788380, 0x00848F8C, 0x007A8582, 0x00727D7A, 0x00727D7A, 0x00727D7A, 0x00616C69, 0x002A3F41,
        0x000A353E, 0x00003040, 0x000B2F41, 0x00082835, 0x000A3840, 0x00003445, 0x0005324D, 0x00013A4F, 0x00044358, 0x0008435D, 0x000F445F, 0x0003435C, 0x00004C5F, 0x00004556, 0x001A4B5B, 0x00004852, 0x0000495A, 0x00034D67, 0x00054A6B, 0x0000395C,
        0x003C7F9E, 0x001B6179, 0x00004458, 0x00094759, 0x000C475B, 0x00073F58, 0x00134B66, 0x000B4764, 0x000C455E, 0x0014465D, 0x00204B60, 0x000C4A5A, 0x00144F62, 0x00053F55, 0x0001415F, 0x000F5D82, 0x002B89B6, 0x0040B2E7, 0x002FADE7, 0x0025A0E2,
        0x002AA6E6, 0x0029A5E1, 0x00209BD3, 0x001A90C4, 0x001786B8, 0x001C86B5, 0x002A8FBC, 0x002391C1, 0x00299ACC, 0x002BA0D7, 0x001F9AD2, 0x001E99D3, 0x001C93CB, 0x002B9CD4, 0x003CA9DF, 0x002F9ABF, 0x000A657A, 0x00246566, 0x00618B84, 0x0077928E,
        0x0096A7A3, 0x0029372C, 0x00384232, 0x004B4E3F, 0x004A4D3E, 0x004A4D3E, 0x004A4D3E, 0x004B4E3F, 0x004C4F40, 0x004A4D3E, 0x00F4F4F4, 0x007B7D72, 0x004D5041, 0x00E2E2E0, 0x0086867B, 0x004F4F41, 0x00505042, 0x007C7E73, 0x00FDFDFD, 0x00E9E9E7,
        0x00656657, 0x00545546, 0x00565648, 0x00CACAC6, 0x00FFFFFF, 0x00A3A49D, 0x00515344, 0x004E5142, 0x004B4C3C, 0x004E4F3F, 0x004D503F, 0x004D5040, 0x00525243, 0x00525242, 0x00535444, 0x00525343, 0x00545444, 0x00525242, 0x00555545, 0x00565647,
        0x00525343, 0x00515242, 0x00535444, 0x00575848, 0x00585949, 0x00555646, 0x00525343, 0x00515242, 0x00535444, 0x00505141, 0x004E4F3F, 0x00525444, 0x00535545, 0x00515343, 0x00515343, 0x00505343, 0x00505343, 0x00505343, 0x00505343, 0x004F5544,
        0x004D5342, 0x004A503F, 0x004A4F3E, 0x004E4F3F, 0x004A4B3B, 0x004A4B3B, 0x004D4E3E, 0x00494E3F, 0x007F8378, 0x00F4F5F4, 0x004A4F40, 0x004B5041, 0x004D5243, 0x004E5344, 0x004C5142, 0x00494B3F, 0x0025291E, 0x008A9389, 0x0080938A, 0x00607E79,
        0x005F8785, 0x00407272, 0x00144C4D, 0x000A4E47, 0x00014A4E, 0x0001576D, 0x002A95BB, 0x00239ED0, 0x001D9ED5, 0x00239CD4, 0x002698CD, 0x00308AC0, 0x000A556F, 0x0081C1C6, 0x0063A1A9, 0x0089C0C9, 0x003D6968, 0x008ABAB4, 0x003C7675, 0x0081B1B2,
        0x00467B78, 0x006EA49D, 0x004D7C74, 0x008CAEA8, 0x005C7D79, 0x0076A2A1, 0x00528C8B, 0x00538D82, 0x0075A397, 0x00698D81, 0x007BA195, 0x00427468, 0x0068A49A, 0x003A7571, 0x0077AFB0, 0x00567B89, 0x0085A9A9, 0x004B6F63, 0x0089B5A8, 0x003B726F,
        0x0088C4CA, 0x00669DA6, 0x0084B6BC, 0x0065A8B7, 0x0073B6C5, 0x0064A7B6, 0x0070B3C2, 0x0061A4B3, 0x006EB1C0, 0x0064A7B6, 0x006FB2C1, 0x006AADBC, 0x0065A8B7, 0x0077BAC9, 0x005FA2B1, 0x0072B5C4, 0x005FA2B1, 0x0072B5C4, 0x0060A3B2, 0x0087C4C8,
        0x0059A3A5, 0x007CC4CB, 0x00609CA8, 0x0078B3C6, 0x0038849A, 0x004695B0, 0x00397C9B, 0x0008485A, 0x00074759, 0x0009495B, 0x000D4C60, 0x000F4E62, 0x000D4C61, 0x000C4B60, 0x000D4C61, 0x00084A63, 0x00074760, 0x00104C62, 0x000F4A5E, 0x00104C5C,
        0x00064853, 0x0000434B, 0x00004349, 0x0001474E, 0x000A4138, 0x000E473E, 0x0000424A, 0x00004A59, 0x00005356, 0x002C726B, 0x0039716C, 0x003F6A6D, 0x00224D50, 0x000E3A3B, 0x000B3534, 0x001E4543, 0x001D3C39, 0x00415755, 0x002E413E, 0x00001210,
        0x00000D11, 0x00001017, 0x00081817, 0x002F3125, 0x004A432F, 0x003A3320, 0x00221E13, 0x004E4433, 0x003F392C, 0x0021221E, 0x0010191C, 0x00131E26, 0x00161F2C, 0x00191E2D, 0x001B1B2B, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0x00495053, 0x00878E91, 0x00C5CDCD, 0x00B0B8B8, 0x00B8C0BF, 0x00C2CAC9, 0x00B1B9B8, 0x00A0ABA8, 0x00A2ADAA, 0x00C0CBC8, 0x00BBC6C3, 0x00A5B0AD, 0x009AA5A2, 0x00B1BCB9, 0x00C5D0CD, 0x00B8C6C2, 0x00A8B6B2, 0x00BECCC8, 0x00B6C4C0, 0x009BA9A5,
        0x009EACA8, 0x00A6B4B0, 0x00AEBCB8, 0x00A5B1AB, 0x00ACB8B2, 0x00B1BDB7, 0x00ACBAB6, 0x00B6C4C0, 0x00AAB8B6, 0x00768482, 0x005E6C6A, 0x00828D8A, 0x00727D7A, 0x007B8683, 0x0075807D, 0x00737E7B, 0x00717C79, 0x006A7572, 0x0056615E, 0x00274042,
        0x000C3A42, 0x00003244, 0x000C3042, 0x000C2C39, 0x000D3A45, 0x00033547, 0x000B3754, 0x000A3F53, 0x00003B4E, 0x00003F54, 0x0013455C, 0x000B465A, 0x00267884, 0x00216973, 0x003E6874, 0x00437074, 0x003C707C, 0x00125368, 0x00034E6E, 0x00004F72,
        0x003987AB, 0x00165E7C, 0x00044762, 0x00144761, 0x0005425C, 0x000E516A, 0x0004465F, 0x00083F5A, 0x00255670, 0x000B4257, 0x00114E62, 0x00044E68, 0x0002445D, 0x00144E64, 0x00154B62, 0x00003955, 0x000A587C, 0x003092BC, 0x0031A0CE, 0x002AA2E3,
        0x00239EDE, 0x001D9CD9, 0x0025A5E0, 0x0023A1D8, 0x00168ABF, 0x001580B3, 0x00248ABB, 0x001D94BB, 0x001A90BA, 0x002B9ED0, 0x0032A3DB, 0x002B9CD5, 0x001788C1, 0x001F90C9, 0x002B9CD4, 0x0028B0F0, 0x00218DB7, 0x00438493, 0x0071908F, 0x00798982,
        0x008C9A94, 0x00233328, 0x003F4F3E, 0x004C4F40, 0x004B4E3F, 0x00494C3D, 0x00494C3D, 0x004A4D3E, 0x004A4D3E, 0x00494C3D, 0x00F4F4F4, 0x007A7C72, 0x004F5244, 0x00888A80, 0x004D4E3E, 0x004D4D3F, 0x005E5E52, 0x00E8E8E6, 0x00FDFDFD, 0x00808277,
        0x00525343, 0x00515243, 0x00A6A69F, 0x00FFFFFF, 0x00C6C7C3, 0x00535446, 0x00515343, 0x00515344, 0x00525548, 0x00535445, 0x004D503E, 0x00515243, 0x00515445, 0x00525546, 0x00575848, 0x005D5E4E, 0x005B5B4B, 0x00555545, 0x00555545, 0x00545444,
        0x00505141, 0x005D5E4F, 0x005E5F50, 0x00545545, 0x00545545, 0x00545545, 0x00545545, 0x00505141, 0x00555647, 0x0059594A, 0x00515243, 0x004D4D3D, 0x00505141, 0x004F5141, 0x00515444, 0x00545747, 0x00555848, 0x00555848, 0x00505343, 0x004D5342,
        0x004D5342, 0x00505545, 0x00575B4A, 0x00535544, 0x004A4B3B, 0x004A4B3B, 0x004D4E3E, 0x00494E3F, 0x00808379, 0x00F5F5F4, 0x004C5142, 0x004C5142, 0x004D5243, 0x004C5142, 0x00494E3F, 0x00444A3F, 0x00272B1F, 0x00858B80, 0x0087968E, 0x00648380,
        0x005B9198, 0x00297686, 0x0000576C, 0x00095554, 0x00044E5A, 0x00024F6F, 0x002D89B8, 0x002193C8, 0x001998CF, 0x001D9ACD, 0x002398CB, 0x00389BC7, 0x000C4A5A, 0x008FAFAA, 0x00627B77, 0x0099B7B2, 0x004F6B5E, 0x0098B3A9, 0x00466564, 0x009DB2AA,
        0x0058766B, 0x008DB1A1, 0x005C7C6B, 0x0098AD9E, 0x00657669, 0x00879C93, 0x00618077, 0x007B9692, 0x0080928B, 0x00808E83, 0x00849689, 0x006B8879, 0x007FA092, 0x00647F76, 0x0090A19D, 0x0058736F, 0x009CB6A6, 0x00607B61, 0x0093B89E, 0x00396B5F,
        0x0085BCBF, 0x005E909C, 0x008FBBC8, 0x006A929E, 0x008CB4C0, 0x006D95A1, 0x0089B1BD, 0x006E96A2, 0x0088B0BC, 0x00749CA8, 0x0085ADB9, 0x007AA2AE, 0x007AA2AE, 0x0083ABB7, 0x00749CA8, 0x0083ABB7, 0x00739BA7, 0x0087AFBB, 0x00729AA6, 0x0095BEC7,
        0x0066979F, 0x00A0CCD3, 0x007B99A4, 0x0097BDCF, 0x004C8FA6, 0x004293B4, 0x0005547B, 0x00054557, 0x0008485A, 0x000B4B5D, 0x000B4A5E, 0x000C4B5F, 0x000D4C61, 0x000D4C61, 0x000C4B60, 0x000B4C68, 0x00084861, 0x000D4B63, 0x00114E62, 0x000E4C5C,
        0x00115562, 0x001F6870, 0x00216F75, 0x00387E85, 0x0040756C, 0x0046776F, 0x002B707A, 0x00025164, 0x00529199, 0x00A8CDCB, 0x00C0D2D3, 0x007AA3A6, 0x00456E71, 0x002D5658, 0x00365C5C, 0x0087A7A6, 0x0068807E, 0x00788A89, 0x0064716F, 0x00304644,
        0x002A4549, 0x002A444B, 0x002A3A39, 0x003C3E32, 0x00534C38, 0x00524B38, 0x004A463B, 0x006A4F45, 0x0054403B, 0x00383233, 0x002C3239, 0x0032424E, 0x003B4E5D, 0x00344859, 0x00354758, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0x00596063, 0x008C9396, 0x00BAC2C2, 0x00A6AEAE, 0x00A6AEAD, 0x00B2BAB9, 0x00BCC4C3, 0x00B8C3C0, 0x00B5C0BD, 0x00BDC8C5, 0x00A0ABA8, 0x00939E9B, 0x00A3AEAB, 0x00B2BDBA, 0x00A2ADAA, 0x009CAAA6, 0x00A5B3AF, 0x00BDCBC7, 0x00B6C4C0, 0x00A2B0AC,
        0x00B3C1BD, 0x00BFCDC9, 0x00ABB9B5, 0x009CA8A2, 0x00A3AFA9, 0x00ACB8B2, 0x00A1AFAB, 0x00A9B7B3, 0x00ACBAB8, 0x00849290, 0x00647270, 0x007E8C88, 0x00717F7B, 0x00778581, 0x00717F7B, 0x00788682, 0x0072807C, 0x0063716D, 0x00505E5A, 0x002B4448,
        0x0013404B, 0x00023647, 0x00082E40, 0x00052532, 0x0004313C, 0x00002F41, 0x000B3754, 0x0007394B, 0x00004052, 0x000B495B, 0x00144057, 0x001D5568, 0x0085D6DE, 0x0080C8CF, 0x007499A1, 0x006C6768, 0x0090A2A9, 0x006396A6, 0x003F8CA6, 0x00338DAC,
        0x000E6181, 0x00004362, 0x000B415F, 0x00074864, 0x00054863, 0x00024964, 0x0000445C, 0x00055168, 0x00003E51, 0x00045465, 0x00004A5B, 0x00004869, 0x00064F6B, 0x00114A63, 0x000D3F55, 0x0011475E, 0x00014562, 0x00005579, 0x003499C0, 0x00269AD9,
        0x0029A2E0, 0x0021A0DD, 0x001CA0DC, 0x0026A7E0, 0x002AA1D8, 0x002290C4, 0x001B81B5, 0x000C8AAC, 0x001D96BD, 0x002696C4, 0x00248EC4, 0x00359DD8, 0x002691CB, 0x001987C1, 0x00168AC1, 0x000B9DEF, 0x002894CE, 0x004A829D, 0x00718487, 0x00818782,
        0x00949D93, 0x00213225, 0x0034493A, 0x004A4D3E, 0x00484B3C, 0x0046493A, 0x00454839, 0x00454839, 0x0046493A, 0x00454839, 0x00F4F4F3, 0x007A7C72, 0x00494C3D, 0x004C4F40, 0x004D4E3E, 0x004D4E41, 0x00C8C8C4, 0x00FFFFFF, 0x00A2A49C, 0x00525546,
        0x004F5040, 0x00818277, 0x00FFFFFF, 0x00E4E4E2, 0x005E6052, 0x00535647, 0x0084867C, 0x00D9DAD6, 0x00E4E5E3, 0x00E6E7E5, 0x00BABBB4, 0x00575A4A, 0x006E7164, 0x00DADBD7, 0x00DDDDDA, 0x00DCDCD9, 0x00DDDDDA, 0x00ADADA7, 0x00505344, 0x00727569,
        0x00D0D1CD, 0x00EBEBE9, 0x00EAEAE8, 0x00D0D0CC, 0x0075776A, 0x00525344, 0x005A5B4B, 0x00ADAEA6, 0x00E7E7E5, 0x00EAEAE9, 0x00E3E3E1, 0x009C9C93, 0x004F5040, 0x005E6052, 0x00D2D3CF, 0x00DCDDDA, 0x00DBDCD8, 0x00DBDCD9, 0x00B8B9B3, 0x00515646,
        0x00BBBEB7, 0x00DDDEDB, 0x00DBDCD9, 0x00DCDDDA, 0x00C5C5C0, 0x0067685B, 0x004E4F3F, 0x00494E3F, 0x00808379, 0x00F5F5F4, 0x004E5344, 0x004D5243, 0x004D5243, 0x004A4F40, 0x00464B3C, 0x0041493E, 0x002F3327, 0x0086887C, 0x008B968C, 0x005D7C7D,
        0x005392A0, 0x002E8DA7, 0x001183A7, 0x00054E56, 0x0006485B, 0x000B4C72, 0x003D8EC1, 0x003099D2, 0x001E99D1, 0x001C99CC, 0x002198C9, 0x002390B6, 0x000F4A54, 0x00677165, 0x0056564A, 0x006C7669, 0x00525E4C, 0x006B7467, 0x00545A59, 0x007A7C70,
        0x00424F3F, 0x006E856F, 0x004A6048, 0x006E7762, 0x0060624F, 0x00717667, 0x004E584B, 0x006A7174, 0x00585858, 0x006D6A66, 0x006B6E65, 0x005A665A, 0x005E6C60, 0x0065685F, 0x007B706C, 0x00485466, 0x0070777A, 0x004F5445, 0x007B816E, 0x00535D50,
        0x00777E7B, 0x00565453, 0x008D817F, 0x00555D53, 0x00848C82, 0x00586056, 0x007F877D, 0x00596157, 0x007A8278, 0x005E665C, 0x00727A70, 0x00676F65, 0x006D756B, 0x006B7369, 0x00697167, 0x006F776D, 0x00676F65, 0x00777F75, 0x00646C62, 0x007E7F83,
        0x0059605D, 0x00817E76, 0x0078635B, 0x008F837F, 0x002E4344, 0x00023038, 0x00154354, 0x00044456, 0x000A4A5C, 0x000D4D5F, 0x000A495D, 0x0009485C, 0x000D4C61, 0x000E4D62, 0x000B4A5F, 0x00064664, 0x0011506C, 0x000A4860, 0x000F4B61, 0x00003F51,
        0x00266C79, 0x0067B2BB, 0x003A8A91, 0x0094A4B4, 0x00545958, 0x008B9392, 0x006F93A3, 0x0040798F, 0x0053848E, 0x00839F9F, 0x00869597, 0x0030565A, 0x0043696D, 0x00315759, 0x00456969, 0x0093B2B1, 0x00A1B7B5, 0x00A8B6B5, 0x00A5ADAC, 0x0094AAA8,
        0x008DA8AC, 0x007F99A0, 0x00627271, 0x00515347, 0x00534C38, 0x00514A37, 0x00514D42, 0x00443E33, 0x004C534C, 0x005E7976, 0x00609396, 0x0063ADB3, 0x0064BEC5, 0x0060C2C8, 0x006BD1D6, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0x004F585C, 0x00778083, 0x00A3ADAD, 0x00ABB6B4, 0x00AEB9B7, 0x00A4AFAC, 0x00C0CCC6, 0x00C0D1CD, 0x00B0BFBB, 0x009FAEAA, 0x00B0BFBB, 0x0099A7A3, 0x00B2C0BC, 0x00A8B3B0, 0x00ADB8B5, 0x0094A29E, 0x00AFBDB9, 0x00BFCDC9, 0x00B1BFBB, 0x00C1CFCB,
        0x00D1DFDB, 0x00B5C3BF, 0x00A8B6B2, 0x00AFB9B3, 0x00AFB9B3, 0x009FABA5, 0x00A3AEAB, 0x00A3AEAB, 0x009CA9A7, 0x007B8886, 0x00606D6B, 0x00777E7B, 0x0079807D, 0x00737A77, 0x00838A87, 0x00838A87, 0x00727976, 0x006B726F, 0x00535A57, 0x002F403D,
        0x00004654, 0x00003B56, 0x0018354A, 0x0005232E, 0x00022937, 0x00103543, 0x000D3E40, 0x00003F46, 0x0002414F, 0x0008445A, 0x0000425A, 0x00156477, 0x0035848F, 0x00317172, 0x00386562, 0x002B5242, 0x00599388, 0x00135E5C, 0x0009545D, 0x00115366,
        0x00114962, 0x000E465F, 0x00034159, 0x00054867, 0x00044663, 0x00054863, 0x0000425B, 0x0004445C, 0x00054459, 0x0009465A, 0x000A485A, 0x00004260, 0x00024865, 0x0000415E, 0x0000425B, 0x00064960, 0x00024358, 0x000D4C60, 0x0065A4B8, 0x003A91A5,
        0x002C8CBA, 0x002EA1EC, 0x001C9EF1, 0x0018A1E6, 0x0024A2DC, 0x002A9BD3, 0x001C87C1, 0x000082BE, 0x002095C8, 0x004AABD7, 0x00116A96, 0x00035F92, 0x003CA3DA, 0x00279BD2, 0x000B83B7, 0x001692BA, 0x0039A2CD, 0x00478AA5, 0x0060807F, 0x00788F81,
        0x00869D98, 0x002C3A36, 0x004C463B, 0x00494E3F, 0x00474C3D, 0x00474C3D, 0x00484D3E, 0x00464B3C, 0x0044493A, 0x00454A3B, 0x00F4F5F4, 0x00797B70, 0x00454839, 0x004A4D3E, 0x004B4C3C, 0x009B9C94, 0x00FFFFFF, 0x00C8C9C4, 0x004E5142, 0x004E5142,
        0x00646557, 0x00ECECEB, 0x00F9F9F9, 0x00747469, 0x004B4E3F, 0x00515446, 0x00EBECEA, 0x00CFCFCB, 0x005B5D4F, 0x0077786C, 0x00D2D2CE, 0x00686A5D, 0x0086887E, 0x00FFFFFF, 0x008E8F85, 0x00747569, 0x00737368, 0x00656559, 0x005E6153, 0x00ECECEB,
        0x00D5D6D3, 0x006D6E62, 0x006C6E61, 0x00D1D2CE, 0x00B2B3AD, 0x00515343, 0x00A7A89F, 0x00FEFEFE, 0x008E8E84, 0x00656557, 0x009F9F96, 0x00E4E4E2, 0x005A5A4C, 0x006D6D60, 0x00FFFFFF, 0x009B9B92, 0x00717265, 0x00757569, 0x006E6F61, 0x00565949,
        0x00F8F8F8, 0x00BDBFB9, 0x00727466, 0x00797A6E, 0x00F8F8F8, 0x00BCBCB7, 0x004F4E40, 0x00464C3B, 0x007F8377, 0x00F5F5F4, 0x004E5443, 0x004C5241, 0x004A503F, 0x00474D3C, 0x00444A39, 0x004D413B, 0x00173228, 0x007E8F84, 0x00859688, 0x006C8074,
        0x00679497, 0x002193AA, 0x00118DBD, 0x002881AD, 0x00015A7C, 0x0000516B, 0x00268FB0, 0x00259CCD, 0x001492CC, 0x001D95D0, 0x002396C9, 0x000C6B96, 0x00233540, 0x0047402D, 0x00343E2D, 0x00353934, 0x00393A31, 0x00333C2F, 0x003C3D33, 0x00363A2E,
        0x003A3E32, 0x003C4034, 0x003B3F33, 0x003A3E32, 0x003C4034, 0x003C4034, 0x003B3F33, 0x00383D2E, 0x003B4031, 0x003E4334, 0x003F4435, 0x003E4334, 0x003C4132, 0x003C4132, 0x003C4132, 0x003B3F33, 0x003A3E32, 0x00393D31, 0x00393B2F, 0x00383A2E,
        0x003A3A2E, 0x003B3B2F, 0x003C3C30, 0x003F4030, 0x003F4030, 0x003E3F2F, 0x003B3E2E, 0x003B3E2E, 0x003A402F, 0x003A402F, 0x00393F2E, 0x0037382E, 0x003F4036, 0x003C3D33, 0x00393A30, 0x003A3B31, 0x00393A30, 0x0037382E, 0x0036372D, 0x002D3B29,
        0x0037362C, 0x00393530, 0x0034382D, 0x00373C2D, 0x0030332A, 0x00152D33, 0x000A4051, 0x00054459, 0x0007465B, 0x0009485D, 0x000A495E, 0x000A495E, 0x000A495E, 0x000A495E, 0x000B4A5F, 0x000F4E63, 0x000D4C61, 0x000A4860, 0x000E4D62, 0x000C4C5E,
        0x00064853, 0x00185C61, 0x00004343, 0x001F4D58, 0x000A312F, 0x00144341, 0x00074B5E, 0x00004660, 0x00124F59, 0x003C6A64, 0x003B645C, 0x002D5651, 0x00214A4D, 0x00133A42, 0x000E373A, 0x00184139, 0x00214537, 0x003D5953, 0x001F3436, 0x00091E1C,
        0x0000171B, 0x0000121A, 0x000D1B1A, 0x0036362A, 0x004C432F, 0x0038311E, 0x002A2419, 0x004E4029, 0x00483E2C, 0x002C281D, 0x001A1C16, 0x001A2221, 0x001F2B2D, 0x00212D2F, 0x00182426, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0x004D565A, 0x00767F82, 0x00A1ABAB, 0x00ABB6B4, 0x00BBC6C4, 0x00B1BCB9, 0x00B7C3BD, 0x00A2B1AD, 0x0098A7A3, 0x009AA9A5, 0x00B2C1BD, 0x009FADA9, 0x00A3B1AD, 0x0095A09D, 0x009FAAA7, 0x009DA8A5, 0x00B2BDBA, 0x00BDC8C5, 0x00A9B4B1, 0x00B3BEBB,
        0x00CDD8D5, 0x00BBC6C3, 0x00A8B3B0, 0x00ADB7B1, 0x00A4AEA8, 0x00A4B0AA, 0x00ACB7B4, 0x009FAAA7, 0x009CA9A7, 0x00808D8B, 0x005D6A68, 0x00767D7A, 0x00787F7C, 0x00787F7C, 0x00818885, 0x007C8380, 0x00727976, 0x006B726F, 0x00535A57, 0x0039424B,
        0x000B4E67, 0x00003B62, 0x00153859, 0x000E374D, 0x0005374E, 0x00033149, 0x00003746, 0x00003645, 0x0015485C, 0x000B3B57, 0x000C4461, 0x005092AB, 0x006DADBF, 0x005B8C94, 0x007A9DA1, 0x007E868D, 0x0092ABB5, 0x003C6D7B, 0x000F4E62, 0x00003D58,
        0x00075070, 0x00237898, 0x002C8BAC, 0x001B86B2, 0x001D89B3, 0x001F8BB5, 0x001B87B0, 0x001887AF, 0x001E8DB3, 0x001A89AF, 0x001A89AF, 0x001B8AB8, 0x001683AF, 0x001885B1, 0x001E8AB4, 0x001983AC, 0x001B84AB, 0x0010789D, 0x001981A6, 0x00015261,
        0x000F5A7A, 0x00358BC1, 0x0034A5E4, 0x0023A5E1, 0x00219ED8, 0x002EA8DE, 0x00219DD3, 0x001895CF, 0x002296CD, 0x0047ADDE, 0x00176E9A, 0x00003F65, 0x001D759D, 0x00349BC8, 0x002E9FD1, 0x000C8BB8, 0x001C88B7, 0x003E84A1, 0x00678786, 0x00829586,
        0x008FA69E, 0x002C3A36, 0x00514C43, 0x00484D3E, 0x00454A3B, 0x00454A3B, 0x00464B3C, 0x00454A3B, 0x00434839, 0x0044493A, 0x00F4F4F4, 0x007C7F74, 0x00474A3B, 0x004A4D3E, 0x0078796D, 0x00FCFCFC, 0x00E8E8E6, 0x005E6053, 0x004B4E3F, 0x00515445,
        0x00D5D5D1, 0x00FFFFFF, 0x00999990, 0x004C4D3F, 0x004A4C3D, 0x00505344, 0x00BDBEBA, 0x00F3F3F2, 0x00CACBC6, 0x00A3A39B, 0x0077786C, 0x004E5141, 0x0086887E, 0x00FEFEFE, 0x00C4C4BF, 0x00B6B7B1, 0x00B6B6B0, 0x006F6F64, 0x00818378, 0x00FFFFFF,
        0x007A7C6F, 0x00505042, 0x006A6C5F, 0x00797B70, 0x00737669, 0x00515344, 0x00E8E8E6, 0x00C5C5C0, 0x00545545, 0x005D5E4F, 0x00737367, 0x007D7D71, 0x005B5B4C, 0x006E6F62, 0x00FBFBFB, 0x00CCCDC8, 0x00B9B9B3, 0x00B9BAB3, 0x00838478, 0x00535646,
        0x00F1F1F0, 0x00C8C9C4, 0x008B8C81, 0x00919087, 0x00EEEEED, 0x009B9A93, 0x004C4B3D, 0x00474D3C, 0x007F8377, 0x00F4F5F4, 0x004A503F, 0x00484E3D, 0x00494F3E, 0x00484E3D, 0x00464C3B, 0x004A4138, 0x00132F22, 0x007D8B80, 0x00859387, 0x00697C73,
        0x00639198, 0x001E91AC, 0x001390C2, 0x002B93C2, 0x00268BB1, 0x00127698, 0x00208DB3, 0x002298CB, 0x001996D0, 0x001D96CE, 0x002194C7, 0x00076691, 0x0020323D, 0x00463F2C, 0x0036402F, 0x00373B36, 0x003B3C33, 0x00333C2F, 0x003B3C32, 0x00393D31,
        0x003C4034, 0x003E4236, 0x003B3F33, 0x003A3E32, 0x003C4034, 0x003C4034, 0x003B3F33, 0x003B4031, 0x003C4132, 0x003E4334, 0x003E4334, 0x003D4233, 0x003C4132, 0x003B4031, 0x003B4031, 0x00393D31, 0x003B3F33, 0x003C4034, 0x003F4135, 0x003F4135,
        0x00414135, 0x00404034, 0x003F3F33, 0x003F4030, 0x003F4030, 0x003F4030, 0x003D4030, 0x003C3F2F, 0x003A402F, 0x003A402F, 0x003A402F, 0x003C3D33, 0x003F4036, 0x003B3C32, 0x003B3C32, 0x003F4036, 0x003E3F35, 0x003C3D33, 0x003F4036, 0x00303E2C,
        0x003A392F, 0x003C3833, 0x0035392E, 0x00383D2E, 0x0030332A, 0x00142C32, 0x00093F50, 0x00054459, 0x0007465B, 0x0009485D, 0x000A495E, 0x000A495E, 0x000A495E, 0x000A495E, 0x000B4A5F, 0x0006455A, 0x0016556A, 0x000E4C64, 0x0008475C, 0x000F4F61,
        0x0010525D, 0x002B6F74, 0x00246A6A, 0x0019656B, 0x0024675E, 0x00256863, 0x00106273, 0x00055972, 0x003C7D86, 0x00A6D2CC, 0x00B5DAD2, 0x0085AEA9, 0x00487174, 0x00194048, 0x001B4447, 0x0084ADA5, 0x006C9082, 0x006B8781, 0x004C6163, 0x00314644,
        0x00233C40, 0x001C333B, 0x0022302F, 0x003E3E32, 0x00544B37, 0x004C4532, 0x00474136, 0x00545045, 0x0043443B, 0x00272F2E, 0x0026363C, 0x002D4650, 0x002A4853, 0x00284855, 0x00264953, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0x00475054, 0x00737C7F, 0x009BA5A5, 0x00A7B2B0, 0x00BFCAC8, 0x00B2BDBA, 0x00A1ADA7, 0x00B4C3BF, 0x00A8B7B3, 0x00A1B0AC, 0x009FADA9, 0x00909E9A, 0x0096A19E, 0x009DA8A5, 0x00B4BFBC, 0x00ABB6B3, 0x0098A3A0, 0x009CA7A4, 0x009DA8A5, 0x00A0ABA8,
        0x00A6B1AE, 0x009CA7A4, 0x0098A3A0, 0x00A5AFA9, 0x009DA7A1, 0x00A6B2AC, 0x00ACB7B4, 0x00A0ABA8, 0x00A0ADAB, 0x007E8B89, 0x00606D6B, 0x00707976, 0x0079827F, 0x00858E8B, 0x00838C89, 0x00747D7A, 0x00717A77, 0x006E7774, 0x005A6360, 0x002D334A,
        0x000D527A, 0x0000457A, 0x00114776, 0x00155379, 0x00044C76, 0x00003662, 0x00003D5F, 0x00014562, 0x00044466, 0x00034268, 0x0001486E, 0x001B678B, 0x00256F8B, 0x001A5B70, 0x002F6576, 0x001D3A59, 0x002E5A79, 0x000B4B69, 0x00015170, 0x00005578,
        0x001A83AC, 0x003FB7E5, 0x0049CCFD, 0x0042C9FF, 0x0040C7FD, 0x003CC8FD, 0x0039C7FE, 0x0031C2F9, 0x0035CBFF, 0x002CC4FA, 0x0030C8FF, 0x003BC8FF, 0x0034C1FA, 0x003BC8FF, 0x0043CEFF, 0x0035BFF4, 0x0040CAFF, 0x0039C1F5, 0x0020A8DC, 0x00035D6E,
        0x0000394F, 0x001B5572, 0x003392B9, 0x0031AADC, 0x002199D5, 0x0024A1DB, 0x0017A7D7, 0x00299FD3, 0x002B9FDA, 0x00369FD8, 0x00166C96, 0x00003F54, 0x00074A59, 0x00227692, 0x003298C3, 0x001694CA, 0x00117EB2, 0x003A82A0, 0x00648581, 0x00819382,
        0x0099ACA3, 0x00273432, 0x00424038, 0x00474C3D, 0x00434839, 0x00424738, 0x00434839, 0x00434839, 0x00424738, 0x00424738, 0x00F4F4F3, 0x007C7F74, 0x0046493A, 0x005B5D50, 0x00E8E8E6, 0x00FEFEFD, 0x0072746A, 0x004B4E3F, 0x004A4E3F, 0x00B0B2AB,
        0x00FFFFFF, 0x00BCBDB7, 0x00525444, 0x004D4F3F, 0x004A4B3C, 0x004C4F40, 0x0050524B, 0x0083857C, 0x00B2B3AC, 0x00DCDCD9, 0x00F7F7F6, 0x008D8F85, 0x007E8075, 0x00FEFEFE, 0x00B0B1AA, 0x009F9F97, 0x009E9E96, 0x0066665A, 0x0086887D, 0x00FFFFFF,
        0x00767669, 0x004B4B3B, 0x00B4B5AE, 0x00DDDEDB, 0x00EFEFED, 0x005F6153, 0x00E7E8E6, 0x00C0C0BA, 0x00515242, 0x00838378, 0x00D1D1CD, 0x00F5F6F5, 0x009B9B91, 0x00646455, 0x00FBFBFB, 0x00BDBEB7, 0x00A4A59C, 0x00A2A39A, 0x00747669, 0x00515446,
        0x00ECECEA, 0x00E7E8E5, 0x00CBCCC7, 0x00E0E0DD, 0x00F4F4F4, 0x006E6E63, 0x0048483A, 0x00454B3A, 0x007E8276, 0x00F4F5F4, 0x00494F3E, 0x00474D3C, 0x00484E3D, 0x00484E3D, 0x00464C3B, 0x00484537, 0x00153223, 0x00868E87, 0x00929A93, 0x0070817D,
        0x0063919C, 0x001A8FAC, 0x00118DC3, 0x001D9ACC, 0x002FA6D3, 0x002395BE, 0x002393C1, 0x001E94C8, 0x001E97CF, 0x002198CF, 0x001C92C3, 0x0002618C, 0x001D2F3A, 0x00453E2B, 0x0036402F, 0x00383C37, 0x003C3D34, 0x00343D30, 0x003B3C32, 0x003C4034,
        0x003F4337, 0x003F4337, 0x003C4034, 0x003A3E32, 0x003C4034, 0x003D4135, 0x003C4034, 0x003E4334, 0x003E4334, 0x003D4233, 0x003D4233, 0x003D4233, 0x003D4233, 0x003C4132, 0x003B4031, 0x00383C30, 0x00393D31, 0x00393D31, 0x003C3E32, 0x003C3E32,
        0x003E3E32, 0x003D3D31, 0x003C3C30, 0x003F4030, 0x00404131, 0x00414232, 0x003F4232, 0x003C3F2F, 0x00393F2E, 0x003B4130, 0x003E4433, 0x003F4036, 0x003F4036, 0x003C3D33, 0x003B3C32, 0x003E3F35, 0x003C3D33, 0x0036372D, 0x0037382E, 0x00344230,
        0x003D3C32, 0x003E3A35, 0x00373B30, 0x00383D2E, 0x002F3229, 0x00122A30, 0x00073D4E, 0x00054459, 0x0007465B, 0x0009485D, 0x000A495E, 0x000A495E, 0x000A495E, 0x000A495E, 0x000B4A5F, 0x00115065, 0x00034257, 0x000A4860, 0x00104F64, 0x0008485A,
        0x002A6C77, 0x006BAFB4, 0x00519797, 0x006E7E8B, 0x003B4545, 0x00647B7D, 0x005C8EA2, 0x003D7E9A, 0x00508B95, 0x006C9C96, 0x007CABA2, 0x00365F5A, 0x00436C6F, 0x00295058, 0x00355E61, 0x0092BBB3, 0x0096BAAC, 0x0095B1AB, 0x0091A6A8, 0x0090A5A3,
        0x008BA4A8, 0x00859CA4, 0x006A7877, 0x00505044, 0x004B422E, 0x004F4835, 0x005B554A, 0x00474338, 0x004E564C, 0x00556E6A, 0x00669495, 0x0068ADB0, 0x0059B0B2, 0x0057BBBD, 0x005EC9C7, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0x00475054, 0x00717A7D, 0x00929C9C, 0x00A1ACAA, 0x00BEC9C7, 0x00BCC7C4, 0x00B2BEB8, 0x009DACA8, 0x00A2B1AD, 0x00ABB9B5, 0x00A1AFAB, 0x00A1ACA9, 0x00A4AFAC, 0x00AAB3B0, 0x00B2BBB8, 0x00B8C3C0, 0x00939E9B, 0x009EA9A6, 0x00B9C4C1, 0x00B5C0BD,
        0x00A3AEAB, 0x0095A09D, 0x0096A19E, 0x00A7B1AB, 0x00A8B2AC, 0x00A8B4AE, 0x00A5B0AD, 0x00A7B2AF, 0x00A4B1AF, 0x0074817F, 0x0063706E, 0x00737B7A, 0x007A8281, 0x00878F8E, 0x007D8584, 0x006D7574, 0x006F7776, 0x00656D6C, 0x004C5453, 0x00314165,
        0x001E71A5, 0x00066AAB, 0x001464A1, 0x001975A8, 0x001A81BA, 0x001F7FBB, 0x002A90C4, 0x0036A3D6, 0x002A98CC, 0x003CACE1, 0x003DB1E6, 0x002EA4D7, 0x0032A9D6, 0x003DAED6, 0x003EACCE, 0x0034A7D4, 0x003CB2DC, 0x0032AFD5, 0x0037B9DE, 0x002DB3DD,
        0x0036BFF0, 0x0034C4FE, 0x002AC0FF, 0x0033BFFA, 0x002DBCF6, 0x002CBBF8, 0x002DC1FD, 0x0026BDFA, 0x0027C2FF, 0x001BBAF8, 0x0022C1FF, 0x002CC3FF, 0x002CC3FF, 0x0027BFFA, 0x002CC2FE, 0x0022B8F4, 0x002AC0FC, 0x002CC2FE, 0x0028BEFA, 0x002DA7CB,
        0x00115F7C, 0x0000394F, 0x0006637D, 0x002C9ECD, 0x002B99D9, 0x002199D5, 0x0010A1CF, 0x0031A2D4, 0x0035AAE7, 0x002C9DDC, 0x00176F98, 0x000D4852, 0x001D4F4D, 0x00418592, 0x003D93B7, 0x00289CD5, 0x00238DC2, 0x00418BA7, 0x005F837D, 0x007D8D7B,
        0x009BACA3, 0x002D3A38, 0x0043453F, 0x00474C3D, 0x00434839, 0x00404536, 0x00424738, 0x00434839, 0x00424738, 0x00424738, 0x00F4F4F3, 0x00797B70, 0x00424536, 0x00C6C7C2, 0x00FFFFFF, 0x00A3A39C, 0x00454738, 0x0045483A, 0x0085877D, 0x00FFFFFF,
        0x00DFDFDC, 0x005B5D4E, 0x00505142, 0x00505243, 0x00505243, 0x004F5243, 0x00C6C7C3, 0x00A5A69E, 0x004D4D3D, 0x005F6051, 0x00F5F5F4, 0x00B5B6AF, 0x007F8175, 0x00FFFFFF, 0x007E7F73, 0x00626354, 0x00646457, 0x00606053, 0x00616356, 0x00F5F5F4,
        0x00C9CAC5, 0x00626254, 0x005E5F50, 0x00C7C8C3, 0x00FFFFFF, 0x00595C4E, 0x00ABACA5, 0x00FAFAFA, 0x007E7F73, 0x00595A4B, 0x008D8E83, 0x00FFFFFF, 0x00A2A29A, 0x0067685A, 0x00FFFFFF, 0x00929288, 0x00666759, 0x00616456, 0x005D6053, 0x00525749,
        0x00F7F7F7, 0x00ABAEA7, 0x004D5243, 0x005D5F53, 0x00FFFFFF, 0x00A6A69F, 0x0049493B, 0x00434636, 0x007E8075, 0x00F4F5F4, 0x004B4E3E, 0x004A4D3D, 0x004A4D3D, 0x00494C3C, 0x00464939, 0x00454535, 0x00132D1F, 0x00848882, 0x008F9590, 0x006E7E7D,
        0x005F8F9B, 0x001E90AE, 0x001F94C7, 0x00109AC9, 0x001598C9, 0x001A97CA, 0x00279CD3, 0x001F93CC, 0x001E95CD, 0x00249CD0, 0x001A91C2, 0x0001608B, 0x001C2E39, 0x00443D2A, 0x0036402F, 0x00383C37, 0x003C3D34, 0x00333C2F, 0x003A3B31, 0x003B4235,
        0x003E4538, 0x003E4538, 0x003A4134, 0x00394033, 0x003C4336, 0x003E4538, 0x003D4437, 0x003F4639, 0x003D4437, 0x003B4235, 0x003B4235, 0x003D4437, 0x003E4538, 0x003D4437, 0x003C4336, 0x0043473B, 0x00404438, 0x003C4034, 0x003B3D31, 0x003A3C30,
        0x003D3D31, 0x003E3E32, 0x003E3E32, 0x003F3F31, 0x00404032, 0x00414133, 0x003E4132, 0x003C3F30, 0x00393E2F, 0x003E4334, 0x00454A3B, 0x0043473C, 0x0045493E, 0x0045493E, 0x00404439, 0x0044483D, 0x0044483D, 0x00373B30, 0x0032362B, 0x00344230,
        0x003D3C32, 0x003E3A35, 0x00363A2F, 0x00373C2D, 0x002F3229, 0x00122A30, 0x00063C4D, 0x00054459, 0x0007465B, 0x0009485D, 0x000A495E, 0x000A495E, 0x000A495E, 0x000A495E, 0x000B4A5F, 0x000A495E, 0x00104F64, 0x000D4B63, 0x0009485D, 0x00044456,
        0x0000424D, 0x00165A5F, 0x00004444, 0x0017424B, 0x00123531, 0x001D4845, 0x00054557, 0x00004860, 0x000F4D55, 0x0028574F, 0x00366158, 0x002A534E, 0x001B4447, 0x00113840, 0x00113A3D, 0x00113A32, 0x00274B3D, 0x004B6761, 0x002A3F41, 0x00112624,
        0x00031C20, 0x00081F27, 0x00142221, 0x0029291D, 0x003F3622, 0x003B3421, 0x00302A1F, 0x005B4932, 0x004E3E2D, 0x002D241A, 0x001C1C16, 0x00202625, 0x00222B2E, 0x00202C2E, 0x00172325, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0x00464F53, 0x006F787B, 0x00899393, 0x00939E9C, 0x00A1ACAA, 0x00A3AEAB, 0x00ABB7B1, 0x00A6B4B0, 0x00A5B3AF, 0x00A9B7B3, 0x009FAAA7, 0x00A3AEAB, 0x00A7B0AD, 0x00A9B2AF, 0x00ABB4B1, 0x009CA5A2, 0x0089928F, 0x008E9794, 0x00939C99, 0x008C9592,
        0x00939C99, 0x009DA6A3, 0x009FA8A5, 0x0099A39D, 0x0098A29C, 0x0095A19B, 0x00949F9C, 0x009DA8A5, 0x009FACAA, 0x00717E7C, 0x00626F6D, 0x006C7775, 0x006D7876, 0x00747F7D, 0x006E7977, 0x00697472, 0x006F7A78, 0x005E6967, 0x003D4846, 0x0024446D,
        0x00177BB5, 0x000077BF, 0x000166AA, 0x00006EAB, 0x00098BCE, 0x001D99E0, 0x0024A7E6, 0x001FAFF6, 0x0023B6FC, 0x0023BCFF, 0x0024BDFF, 0x002CC4FF, 0x0030C7FF, 0x002BC3FC, 0x0030C9FF, 0x0022CAFF, 0x0023C8F9, 0x0025C6F3, 0x002FD0FD, 0x002CCCFC,
        0x002ACAFF, 0x0022C0FF, 0x0024BFFF, 0x0027BBFD, 0x0026BBFD, 0x0026BBFE, 0x0025BCFF, 0x0025BEFF, 0x0029C2FF, 0x0026C1FF, 0x0024BFFF, 0x0020BFFD, 0x0024C3FF, 0x001AB8F8, 0x001FBDFD, 0x001CBCFC, 0x001EBEFE, 0x001BBBFB, 0x0020C0FF, 0x002DC6FF,
        0x0031A5D4, 0x00046182, 0x00004A6C, 0x001E89BC, 0x003BA1E2, 0x002D9CDA, 0x001C9FD1, 0x002B9ACA, 0x00239CDA, 0x0029A1E3, 0x001C7CAA, 0x0004444F, 0x002B5E57, 0x0079B1B6, 0x004D96AC, 0x002285B7, 0x002D8DBB, 0x00438FA6, 0x00668F88, 0x00839683,
        0x0091A098, 0x00283331, 0x0040433A, 0x00484D3E, 0x00434839, 0x00414637, 0x00434839, 0x0044493A, 0x00434839, 0x00434839, 0x00F4F4F3, 0x00777A6F, 0x00424536, 0x00B8B9B3, 0x00BDBDB8, 0x004C4E3F, 0x00494B3C, 0x00484B3C, 0x0084867C, 0x00D5D6D2,
        0x006F7164, 0x00505242, 0x00505242, 0x00515242, 0x004C4E3F, 0x00474A3B, 0x0091938B, 0x00EBEBE9, 0x00E1E1DE, 0x00E3E3E0, 0x00D7D7D3, 0x0068685B, 0x00707266, 0x00EDEDEB, 0x00ECECEA, 0x00EBEBE9, 0x00EBEBEA, 0x00D0D0CC, 0x004D5040, 0x007E7F74,
        0x00E4E4E1, 0x00ECECEA, 0x00E5E5E3, 0x00C9C9C5, 0x00E3E4E1, 0x005C5F51, 0x00575A4C, 0x00C1C2BD, 0x00F2F3F2, 0x00E6E6E4, 0x00D9D9D6, 0x00D7D7D4, 0x00A0A098, 0x005D5E4F, 0x00E6E6E4, 0x00EDEEEC, 0x00EBEBEA, 0x00EBEBEA, 0x00D9DAD6, 0x005C6153,
        0x00DFE0DD, 0x009D9F97, 0x004C5142, 0x004C5142, 0x00DFDFDD, 0x00AEAEA8, 0x004B4B3D, 0x00454838, 0x007E8075, 0x00F4F5F4, 0x00484B3B, 0x00474A3A, 0x00494C3C, 0x00494C3C, 0x00474A3A, 0x00474735, 0x00173022, 0x008B8C88, 0x00959B96, 0x00718481,
        0x00639399, 0x00288EA5, 0x002B8EBA, 0x00179ECA, 0x001197C7, 0x001495CE, 0x00209AD6, 0x002197D2, 0x002095CC, 0x00259BCE, 0x001C95C7, 0x0004638E, 0x001E303B, 0x00443D2A, 0x00343E2D, 0x00363A35, 0x003A3B32, 0x00323B2E, 0x003A3B31, 0x003A4134,
        0x003D4437, 0x003E4538, 0x003B4235, 0x003B4235, 0x003E4538, 0x0041483B, 0x0040473A, 0x0040473A, 0x003E4538, 0x003D4437, 0x003D4437, 0x003F4639, 0x0041483B, 0x0042493C, 0x0041483B, 0x0044483C, 0x00404438, 0x003A3E32, 0x00393B2F, 0x00383A2E,
        0x003C3C30, 0x003D3D31, 0x003D3D31, 0x003F3F31, 0x003E3E30, 0x003F3F31, 0x003D4031, 0x003B3E2F, 0x00393E2F, 0x00424738, 0x004C5142, 0x0043473C, 0x00484C41, 0x00494D42, 0x003E4237, 0x0044483D, 0x004D5146, 0x003F4338, 0x00373B30, 0x00313F2D,
        0x003B3A30, 0x003C3833, 0x0035392E, 0x00363B2C, 0x002E3128, 0x0011292F, 0x00063C4D, 0x00054459, 0x0007465B, 0x0009485D, 0x000A495E, 0x000A495E, 0x000A495E, 0x000A495E, 0x000B4A5F, 0x00014055, 0x000D4C61, 0x0009475F, 0x0009485D, 0x000E4E60,
        0x0010525D, 0x00397D82, 0x003F8585, 0x00628B94, 0x00496D67, 0x00547F7C, 0x002C6D7C, 0x001B637B, 0x0055939B, 0x0095C4BC, 0x009AC4B9, 0x0077A09B, 0x00365F62, 0x00194048, 0x00224B4E, 0x00709991, 0x005C8072, 0x006B8781, 0x00475C5E, 0x002E4341,
        0x001B3438, 0x001B323A, 0x00253332, 0x003B3B2F, 0x00534A36, 0x004E4734, 0x00423C31, 0x00505137, 0x00464834, 0x00232D20, 0x00172A21, 0x002A433F, 0x003C5858, 0x00395857, 0x00254544, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0x00475054, 0x00767F82, 0x00929C9C, 0x0098A3A1, 0x009AA5A3, 0x00A2ADAA, 0x00B9C5BF, 0x00B6C4C0, 0x00A1AFAB, 0x009FAAA7, 0x00A1ACA9, 0x00A7B0AD, 0x00A7B0AD, 0x00A6AFAC, 0x00AEB5B2, 0x0099A29F, 0x009BA4A1, 0x00A3ACA9, 0x009FA8A5, 0x00949D9A,
        0x00969F9C, 0x00A0A9A6, 0x00A6AFAC, 0x00939D97, 0x008B958F, 0x008C9892, 0x00929D9A, 0x00919C99, 0x0096A3A1, 0x0073807E, 0x005A6765, 0x00697373, 0x00727C7C, 0x00778181, 0x00747E7E, 0x006C7676, 0x00687272, 0x005E6868, 0x004A5454, 0x001C496E,
        0x00107FB7, 0x00007CC3, 0x000067AC, 0x000068A9, 0x000087CF, 0x000E95E1, 0x000798DC, 0x0010A0F2, 0x0017AEFF, 0x0012B0FE, 0x0012AFF9, 0x0024BBFF, 0x0027C0FF, 0x001EBBFD, 0x001AC0FF, 0x0029C6FE, 0x002BC3FC, 0x002DC2FA, 0x002EC7FE, 0x0021C2F8,
        0x0019BEF7, 0x001EBCF8, 0x002EC3FF, 0x0031CAFF, 0x0032CAFF, 0x002CC2FF, 0x0028BAFB, 0x0036C4FF, 0x0031BAF8, 0x0041C9FF, 0x0039BFF9, 0x0031C0F9, 0x0039CAFF, 0x003ACAFF, 0x0039CBFF, 0x002FC1FD, 0x0033C7FF, 0x002DC0FE, 0x002BC1FE, 0x0028BEFF,
        0x0038C9FF, 0x001A9BC8, 0x00006490, 0x00298CC0, 0x003CA4DF, 0x00299AD8, 0x002B9FDE, 0x0030A5D8, 0x00229DDB, 0x0028A3E7, 0x00319ED2, 0x001B6F89, 0x0017555B, 0x004B8388, 0x00225D67, 0x003A85A5, 0x002D80A0, 0x00347C8E, 0x00628C85, 0x008B9E8F,
        0x0095A39F, 0x002C3433, 0x00484A3E, 0x00484D3E, 0x0044493A, 0x00424738, 0x0044493A, 0x0044493A, 0x00424738, 0x00434839, 0x00F4F4F3, 0x00797B70, 0x0046493A, 0x004C4F40, 0x004C4F40, 0x004C4F40, 0x004D5041, 0x004D5041, 0x004C4F40, 0x004D5041,
        0x00505344, 0x004D4F40, 0x004F5141, 0x00505142, 0x00494B3B, 0x0046493A, 0x004D4F41, 0x005B5B4C, 0x00737467, 0x00717163, 0x00555545, 0x004D4E3E, 0x00505141, 0x0056594B, 0x005B5D4F, 0x00585A4B, 0x00565849, 0x00545548, 0x004F5141, 0x00505141,
        0x00595A4B, 0x00717266, 0x006C6C5F, 0x004D4D3E, 0x00606255, 0x00515344, 0x00505243, 0x00535546, 0x00646759, 0x00737569, 0x005C5D4E, 0x0059594B, 0x005E5E50, 0x00545546, 0x00595A4A, 0x005B5D4E, 0x00585B4D, 0x005A5E50, 0x00585D4F, 0x004D5243,
        0x00565B4D, 0x00525648, 0x004B5041, 0x00484D3E, 0x005A5B4B, 0x00545748, 0x00494C3D, 0x004B4C3C, 0x007F7F74, 0x00F4F4F4, 0x00464737, 0x00454636, 0x00484939, 0x004B4C3C, 0x004C4D3D, 0x00454031, 0x00132A1C, 0x0084837F, 0x0089908B, 0x00657C74,
        0x00618D8C, 0x00308994, 0x003B84A4, 0x00289ABE, 0x002CA4D2, 0x001F9DD7, 0x001390CD, 0x00229AD5, 0x002399CD, 0x002197CA, 0x002098CC, 0x00076691, 0x0020323D, 0x00443D2A, 0x00333D2C, 0x00343833, 0x00393A31, 0x00323B2E, 0x003A3B31, 0x00374033,
        0x003A4336, 0x003C4538, 0x003A4336, 0x003A4336, 0x003E473A, 0x0040493C, 0x0040493C, 0x003E463B, 0x003D453A, 0x003C4439, 0x003D453A, 0x003F473C, 0x0040483D, 0x00424A3F, 0x00424A3F, 0x003E4236, 0x003A3E32, 0x00373B2F, 0x00383A2E, 0x00393B2F,
        0x003B3B2F, 0x003A3A2E, 0x0039392D, 0x00404034, 0x003D3D31, 0x003D3D31, 0x003D3F33, 0x003C3E32, 0x003A3E32, 0x0043473B, 0x004F5347, 0x00454B40, 0x00464C41, 0x00464C41, 0x00383E33, 0x003D4338, 0x00484E43, 0x003A4035, 0x00363C31, 0x00303E2C,
        0x003A392F, 0x003B3732, 0x0034382D, 0x00363B2C, 0x002E3128, 0x00122A30, 0x00073D4E, 0x00054459, 0x0007465B, 0x0009485D, 0x000A495E, 0x000A495E, 0x000A495E, 0x000A495E, 0x000B4A5F, 0x00155469, 0x0009485D, 0x0006445C, 0x000B4A5F, 0x00044456,
        0x000E505B, 0x00468A8F, 0x00458B8B, 0x007E8A96, 0x003C4443, 0x00657A78, 0x00558597, 0x003A7A92, 0x005F999F, 0x0077A69D, 0x007CAA9E, 0x00345D58, 0x00365F62, 0x001F464E, 0x0030595C, 0x0094BDB5, 0x0089AD9F, 0x0089A59F, 0x00899EA0, 0x008EA3A1,
        0x0087A0A4, 0x007F969E, 0x00677574, 0x00505044, 0x004D4430, 0x004C4532, 0x00534D42, 0x0038382C, 0x004A554B, 0x0052706B, 0x005E9292, 0x006CB6B8, 0x0070CCCD, 0x0071D8DA, 0x006CDAD8, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0x00485155, 0x007E878A, 0x009DA7A7, 0x009CA7A5, 0x009AA5A3, 0x00A8B3B0, 0x00BBC7C1, 0x00BCC7C4, 0x009CA7A4, 0x008F9A97, 0x0099A29F, 0x0097A09D, 0x009EA7A4, 0x009DA4A1, 0x00A3AAA7, 0x00A7AEAB, 0x009EA5A2, 0x00969D9A, 0x009CA3A0, 0x009DA4A1,
        0x00929996, 0x009AA19E, 0x00B3BAB7, 0x00A7B1AB, 0x00A6B0AA, 0x00A1ADA7, 0x00A3AEAB, 0x0099A4A1, 0x00909D9B, 0x006D7A78, 0x00505D5B, 0x00697373, 0x00798383, 0x00778181, 0x00727C7C, 0x00646E6E, 0x00505A5A, 0x004C5656, 0x00465050, 0x001F4C6D,
        0x00137EB1, 0x000274BA, 0x000560A5, 0x000066A7, 0x000B86D0, 0x001997E5, 0x0019A4E9, 0x0022A5F6, 0x001EADF8, 0x0022B8FF, 0x0028BBFF, 0x002BB9FA, 0x0030BCFE, 0x002EC5FF, 0x0020C4FF, 0x0033C4FB, 0x0034C1FA, 0x0036C0FB, 0x0030C2FE, 0x0026C0FB,
        0x002AC5FD, 0x003BC8FF, 0x0043C1FB, 0x001C94BE, 0x001286AF, 0x001784AA, 0x002B8FB2, 0x0047A1C0, 0x001D6D8A, 0x0039839D, 0x00347A92, 0x003885A0, 0x00428EAB, 0x004593B0, 0x003B8BAA, 0x002B7C9D, 0x002F81A4, 0x0024759A, 0x002D81A5, 0x004ABAEF,
        0x0033C8FA, 0x0029BFEE, 0x001C88B8, 0x003690BF, 0x0032A3D5, 0x001B9BD6, 0x00269BE6, 0x0025A1D7, 0x001E9ED9, 0x00199AD9, 0x002FA9E5, 0x003BA4D0, 0x0021738C, 0x003A7981, 0x00215351, 0x007BB0BD, 0x00488C9F, 0x0039808A, 0x0055847C, 0x00799185,
        0x0096A3A1, 0x002C3330, 0x00494A3A, 0x00464B3C, 0x00434839, 0x00434839, 0x0044493A, 0x00434839, 0x00414637, 0x00424738, 0x00F8F9F8, 0x007A7C72, 0x00494C3D, 0x004B4E3F, 0x004A4D3E, 0x004B4E3F, 0x004C4F40, 0x004C4F40, 0x004B4E3F, 0x00494C3D,
        0x004C4F40, 0x004C4E3F, 0x004C4F40, 0x004E5142, 0x004B4D3E, 0x0046493A, 0x004D5042, 0x00545545, 0x00515242, 0x004D4D3D, 0x00505040, 0x00515141, 0x00505142, 0x004F5243, 0x004E5042, 0x00515343, 0x004D4F40, 0x004B4E41, 0x004D4F3F, 0x004F5040,
        0x00535444, 0x004E5042, 0x00505142, 0x004F5141, 0x00505242, 0x00525545, 0x00545647, 0x00555646, 0x00525343, 0x00515242, 0x00535444, 0x00505141, 0x004F5040, 0x00525445, 0x00505243, 0x004F5142, 0x004A4F40, 0x004D5243, 0x004F5344, 0x004D5142,
        0x004C5041, 0x004D5243, 0x004B5041, 0x00484D3E, 0x004F5242, 0x004B4E3E, 0x00484B3B, 0x004A4B3B, 0x00808175, 0x00F9F9F8, 0x00484939, 0x00464737, 0x00484939, 0x004A4B3B, 0x00494A3A, 0x00544B3D, 0x001E3226, 0x00858682, 0x00828F87, 0x00638071,
        0x00719B90, 0x00539B9B, 0x006898AA, 0x0017718A, 0x002B91BB, 0x0029A0D8, 0x001190CD, 0x001E99D1, 0x002399CC, 0x002096CA, 0x001E95CD, 0x00076691, 0x0020323D, 0x00453E2B, 0x00343E2D, 0x00353934, 0x00393A31, 0x00323B2E, 0x003A3B31, 0x00374033,
        0x003B4437, 0x003D4639, 0x003B4437, 0x003B4437, 0x003E473A, 0x003F483B, 0x003E473A, 0x003C4439, 0x003D453A, 0x003E463B, 0x003E463B, 0x003E463B, 0x003F473C, 0x0040483D, 0x0041493E, 0x0044483C, 0x00404438, 0x003B3F33, 0x003C3E32, 0x003D3F33,
        0x00404034, 0x003F3F33, 0x003D3D31, 0x00414135, 0x003D3D31, 0x003D3D31, 0x003E4034, 0x003E4034, 0x003A3E32, 0x0042463A, 0x004D5145, 0x004C5247, 0x00474D42, 0x004A5045, 0x0040463B, 0x0043493E, 0x00474D42, 0x00373D32, 0x00393F34, 0x0032402E,
        0x003B3A30, 0x003C3833, 0x0035392E, 0x00373C2D, 0x002F3229, 0x00122A30, 0x00073D4E, 0x00054459, 0x0007465B, 0x0009485D, 0x000A495E, 0x000A495E, 0x000A495E, 0x000A495E, 0x000B4A5F, 0x00044358, 0x00125166, 0x000C4A62, 0x000C4B60, 0x00105062,
        0x00003D48, 0x0002464B, 0x00003C3C, 0x0000393B, 0x00044438, 0x0007483F, 0x00004451, 0x00004C61, 0x00104C52, 0x003B645C, 0x00517368, 0x00335C57, 0x00264F52, 0x00123941, 0x000F383B, 0x000E372F, 0x0024483A, 0x0047635D, 0x002E4345, 0x00122725,
        0x000A2327, 0x000B222A, 0x00172524, 0x00343428, 0x00483F2B, 0x003E3724, 0x00383227, 0x0052362F, 0x0049332E, 0x00302224, 0x00252029, 0x00272836, 0x00242B3A, 0x00252F40, 0x00222D3B, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0x0040494D, 0x008F989B, 0x00C0CACA, 0x00BFCAC8, 0x00B6C1BF, 0x00B7C2BF, 0x00B5C1BB, 0x0098A3A0, 0x0086918E, 0x0086918E, 0x009DA6A3, 0x009BA4A1, 0x00ADB4B1, 0x00A2A9A6, 0x009BA29F, 0x00C5CCC9, 0x00B0B7B4, 0x00969D9A, 0x009AA19E, 0x00A2A9A6,
        0x008C9390, 0x00848B88, 0x00989F9C, 0x00A1ABA5, 0x00B3BDB7, 0x00A6B2AC, 0x00A5B0AD, 0x00A0ABA8, 0x008B9896, 0x0063706E, 0x00515E5C, 0x00525E5E, 0x005D6969, 0x00505C5C, 0x00546060, 0x00535F5F, 0x003F4B4B, 0x003B4747, 0x00354141, 0x00204763,
        0x001A7CAC, 0x000B73B6, 0x001560A4, 0x000F66A8, 0x00107BC5, 0x001688D5, 0x001A9CE1, 0x0023A3EA, 0x0020ACEE, 0x001BB1ED, 0x0024B7EF, 0x0034BEF3, 0x0034BEF3, 0x002AC1F4, 0x0021C5F9, 0x0025C8F5, 0x0025BFF2, 0x0027BCF5, 0x0026BCF9, 0x001EB8F3,
        0x002ABBF3, 0x0031ADE3, 0x001E84B8, 0x00094F5C, 0x00003843, 0x00134950, 0x00517A7D, 0x007A9592, 0x0025332D, 0x003B4037, 0x003E3E32, 0x003C3B37, 0x0042403F, 0x003E3F3D, 0x0037393A, 0x00373C3F, 0x00363C41, 0x0020262D, 0x00394148, 0x006DB4D9,
        0x0038C3EA, 0x0036D1FE, 0x002A91C2, 0x002E82AC, 0x002BA5CD, 0x0018A7E0, 0x001D97EA, 0x001FA0D9, 0x001EA1DA, 0x001498D4, 0x000E91D0, 0x001991CC, 0x002F90B6, 0x0069AEB8, 0x00417067, 0x00668C8C, 0x0036717B, 0x0043878C, 0x0056877F, 0x00738B83,
        0x009BA9A8, 0x002C3330, 0x00454533, 0x0044493A, 0x00434839, 0x00424738, 0x00434839, 0x00414637, 0x003F4435, 0x00414637, 0x00D8D9D6, 0x00D4D4D1, 0x0082847A, 0x007F8177, 0x007F8177, 0x00808278, 0x00818379, 0x00818379, 0x00808278, 0x007F8176,
        0x00818378, 0x0083867B, 0x0083857A, 0x0083857A, 0x00818379, 0x007E8076, 0x007F8077, 0x0086877C, 0x0086867B, 0x0088887D, 0x0086877D, 0x0086867B, 0x0085867A, 0x0084867B, 0x0084857B, 0x0088887D, 0x0087887E, 0x0081837A, 0x007E7F75, 0x0084857A,
        0x0088887D, 0x0086877D, 0x0088887D, 0x0088887D, 0x0088897E, 0x0088897E, 0x0088897E, 0x0088897E, 0x0088897E, 0x0086867B, 0x0085867A, 0x00838378, 0x0084857A, 0x0085867A, 0x0085877D, 0x0085877D, 0x0082867A, 0x0085877D, 0x0086887D, 0x0084867B,
        0x0083867B, 0x0083877C, 0x0083877C, 0x0083867B, 0x0086877C, 0x00818378, 0x007E8076, 0x00818177, 0x00D5D5D1, 0x00DADAD7, 0x004D4E3E, 0x004C4D3D, 0x004A4B3B, 0x00474838, 0x00434434, 0x004D4135, 0x001E2F24, 0x00898A88, 0x0086958D, 0x0064826F,
        0x006B9282, 0x0045847B, 0x0054727D, 0x00003D51, 0x000F688D, 0x002798D0, 0x001A9AD5, 0x001A98CF, 0x002197CA, 0x00249ACE, 0x00188EC9, 0x00066590, 0x001F313C, 0x00453E2B, 0x00353F2E, 0x00363A35, 0x003A3B32, 0x00333C2F, 0x003A3B31, 0x00384134,
        0x003C4538, 0x003E473A, 0x003C4538, 0x003C4538, 0x003D4639, 0x003E473A, 0x003C4538, 0x003C4439, 0x003D453A, 0x003E463B, 0x003E463B, 0x003D453A, 0x003C4439, 0x003E463B, 0x003F473C, 0x004A4E42, 0x0043473B, 0x003A3E32, 0x0037392D, 0x0037392D,
        0x003B3B2F, 0x003C3C30, 0x003C3C30, 0x00414135, 0x003E3E32, 0x003E3E32, 0x00404236, 0x003F4135, 0x003A3E32, 0x003F4337, 0x00494D41, 0x0042483D, 0x003A4035, 0x0043493E, 0x0043493E, 0x00474D42, 0x00444A3F, 0x0030362B, 0x00383E33, 0x00344230,
        0x003D3C32, 0x003E3A35, 0x00363A2F, 0x00383D2E, 0x002F3229, 0x00122A30, 0x00073D4E, 0x00054459, 0x0007465B, 0x0009485D, 0x000A495E, 0x000A495E, 0x000A495E, 0x000A495E, 0x000B4A5F, 0x000B4A5F, 0x00054459, 0x000B4961, 0x000E4D62, 0x00034355,
        0x00145661, 0x004E9297, 0x00418787, 0x0067939A, 0x004E736B, 0x00618D87, 0x002F6E7C, 0x00064C63, 0x00437D83, 0x0092BDB4, 0x00A4CABE, 0x0098C1BC, 0x00446D70, 0x00153C44, 0x001D4649, 0x00689189, 0x00628678, 0x00708C86, 0x00415658, 0x00142927,
        0x000D262A, 0x000C232B, 0x00172524, 0x0038382C, 0x004E4531, 0x00433C29, 0x00413B30, 0x004C4D33, 0x0041432F, 0x00222B1E, 0x0017281F, 0x00243936, 0x002F4747, 0x002E4646, 0x00213939, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0x004E5459, 0x00959BA0, 0x00AEB5B8, 0x00ACB3B6, 0x00A4ACAC, 0x008C9494, 0x00959D9D, 0x00878F8F, 0x00757D7D, 0x00757C7F, 0x007C8184, 0x0085888D, 0x0093969B, 0x00909096, 0x0084838C, 0x0095A0A4, 0x00808B8F, 0x00727D81, 0x00747F83, 0x007B868A,
        0x00838E92, 0x007F8A8E, 0x00727D81, 0x00879895, 0x007E8B8D, 0x007B858C, 0x00858994, 0x008E909B, 0x0079797F, 0x00606060, 0x0070726C, 0x00818988, 0x007C8785, 0x006F7B7B, 0x00737F81, 0x00747F83, 0x006A7377, 0x0072757D, 0x007D7F87, 0x003B525A,
        0x00253E48, 0x00122F3D, 0x00032437, 0x00022B41, 0x000D3C57, 0x000F415D, 0x000D4361, 0x00054F69, 0x000C5670, 0x0009546E, 0x000A5772, 0x000B5A75, 0x000B5B78, 0x0010607D, 0x00055874, 0x0013646B, 0x00105E83, 0x0014699B, 0x00087A9F, 0x00007EA5,
        0x000F75B0, 0x001969A0, 0x000D6A81, 0x00005A9C, 0x00055C84, 0x008BB5C1, 0x00ADBAB2, 0x005B7466, 0x00364A3E, 0x002F3730, 0x00284339, 0x0036403A, 0x003C433E, 0x00373D38, 0x003B3E3C, 0x003A3F3D, 0x00414746, 0x00222D2B, 0x003F4D4B, 0x004A8581,
        0x00C2CBD8, 0x00BDECFF, 0x002896B2, 0x000D6F99, 0x001C6A9F, 0x002F92C6, 0x002592C6, 0x001AA5D6, 0x0027A3D9, 0x002A95CE, 0x00288DC4, 0x00248ABA, 0x003495B9, 0x0055A9C3, 0x0067ACC0, 0x005586B2, 0x004E88B2, 0x00649CB5, 0x0067908B, 0x00749483,
        0x0092ACA6, 0x0022302E, 0x00424035, 0x00474B3F, 0x0045493D, 0x0043473B, 0x0042463A, 0x00414539, 0x00404438, 0x003E4236, 0x005D6056, 0x00D6D7D4, 0x00F3F3F2, 0x00F3F4F3, 0x00F4F4F3, 0x00F3F3F2, 0x00F3F3F2, 0x00F5F5F4, 0x00F3F3F2, 0x00F5F5F4,
        0x00F4F4F3, 0x00F3F3F2, 0x00F4F4F3, 0x00F5F5F5, 0x00F3F4F2, 0x00F5F5F5, 0x00F3F3F2, 0x00F5F5F4, 0x00F6F6F5, 0x00F5F5F4, 0x00F2F2F1, 0x00F2F2F1, 0x00F5F5F5, 0x00F5F5F4, 0x00F2F3F1, 0x00F2F2F1, 0x00F5F5F4, 0x00F6F6F5, 0x00F3F3F2, 0x00F2F2F1,
        0x00F3F3F2, 0x00F5F5F4, 0x00F6F6F5, 0x00F2F2F1, 0x00F3F3F2, 0x00F3F3F2, 0x00F5F6F5, 0x00F4F4F3, 0x00F2F2F1, 0x00F4F4F3, 0x00F5F6F5, 0x00F4F4F3, 0x00F3F3F2, 0x00F4F4F3, 0x00F6F6F5, 0x00F5F5F5, 0x00F3F3F2, 0x00F2F2F1, 0x00F5F5F4, 0x00F6F6F5,
        0x00F4F4F3, 0x00F1F1F0, 0x00F3F3F2, 0x00F5F6F5, 0x00F4F4F3, 0x00F3F3F2, 0x00F1F2F0, 0x00F3F3F2, 0x00D4D4D0, 0x0066675A, 0x0049493B, 0x0049493B, 0x0049493B, 0x0049493B, 0x0048483A, 0x003C452A, 0x001A2D1C, 0x008A8586, 0x00839392, 0x00637C78,
        0x00768483, 0x0055787C, 0x00627184, 0x00124248, 0x00003E50, 0x00106A8D, 0x002898CC, 0x001C98D4, 0x001793CF, 0x002599D0, 0x002593C3, 0x00106595, 0x00112C40, 0x00383C36, 0x003B3F33, 0x003E382D, 0x003C3E2A, 0x002F402B, 0x00343832, 0x00383E33,
        0x00383E33, 0x00383E33, 0x00393F34, 0x003B4136, 0x003C4237, 0x003B4136, 0x003A4035, 0x003E3F35, 0x003A3B31, 0x003A3B31, 0x003E3F35, 0x003E3F35, 0x003B3C32, 0x003A3B31, 0x003D3E34, 0x0038392F, 0x003D3E34, 0x0037382E, 0x003A3B31, 0x0036372D,
        0x003C3D33, 0x0038392F, 0x003C3D33, 0x00373D32, 0x00393F34, 0x00373D32, 0x00373D32, 0x003A4035, 0x003C4237, 0x003A4035, 0x00373D32, 0x003F3E34, 0x003E3D33, 0x00403F35, 0x003F3E34, 0x003C3B31, 0x003C3B31, 0x003E3D33, 0x003E3D33, 0x003B3D31,
        0x003A3D34, 0x00363930, 0x003A3A2C, 0x003B3D2A, 0x00253328, 0x000C3036, 0x000C4156, 0x0006455A, 0x0007465B, 0x0009485D, 0x000A495E, 0x000B4A5F, 0x000A495E, 0x000A495E, 0x0009485D, 0x00054A5B, 0x000D4F61, 0x000B4C5B, 0x000B4959, 0x00044050,
        0x001C5766, 0x004E8796, 0x004A8392, 0x007C8686, 0x003C484E, 0x005D7483, 0x00557F92, 0x00437E8E, 0x004A858E, 0x00729E9F, 0x007F9A97, 0x003F5B62, 0x002F464E, 0x00243A3F, 0x00416061, 0x0085B5AF, 0x008BBEB6, 0x008CB7AE, 0x009BBDB3, 0x008EA4AF,
        0x00A5A8A6, 0x0095A69B, 0x00627975, 0x00544745, 0x005A4131, 0x00514A36, 0x00584F46, 0x0053483A, 0x00434539, 0x0050695F, 0x0077A7A1, 0x0083C5C4, 0x0078C4CA, 0x0075C5D0, 0x0079C8D5, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0x005C6267, 0x0051575C, 0x0041484B, 0x0040474A, 0x004D5555, 0x003A4242, 0x00283030, 0x002E3636, 0x00232B2B, 0x002B3031, 0x002C3134, 0x00292C30, 0x0032353A, 0x0034373C, 0x00323238, 0x002C3539, 0x00283135, 0x002A3337, 0x002D363A, 0x002E373B,
        0x00313A3E, 0x00313A3E, 0x002A3337, 0x00283837, 0x00293638, 0x00343F47, 0x00363E4B, 0x002F3645, 0x00292F3A, 0x00252D34, 0x00313A3D, 0x00435265, 0x004B596B, 0x00475462, 0x00424F57, 0x00414F55, 0x003B494F, 0x003A484E, 0x003F4D53, 0x0011262E,
        0x00001119, 0x0000151F, 0x00011E2C, 0x0000202E, 0x00002335, 0x00022C3F, 0x00103B4E, 0x000A3E4B, 0x000F4350, 0x000C414E, 0x000A4150, 0x000D4655, 0x000D4857, 0x000E4959, 0x00155161, 0x00004F5D, 0x001388AF, 0x002DB7EB, 0x0035C3F2, 0x0034BBF1,
        0x0032B4F9, 0x0036BAF6, 0x0037C2E3, 0x0034C2FF, 0x003EC1E8, 0x0087D0DE, 0x0094B3B2, 0x00637F79, 0x003D4B47, 0x00484040, 0x003A413C, 0x00384039, 0x003C433C, 0x003D413B, 0x003F403C, 0x00393D38, 0x003A3F3D, 0x00242D2A, 0x0053615D, 0x004F827A,
        0x00AAB1BA, 0x00C1E9FB, 0x002C92A9, 0x00157599, 0x003685B6, 0x002E91C3, 0x002F9ED0, 0x0023A2D5, 0x002EA0D5, 0x002F93CD, 0x003297CE, 0x002A95C8, 0x00117FA9, 0x00167C9F, 0x002B88A9, 0x00327AA2, 0x002F7CA3, 0x00488BA2, 0x005F8883, 0x007D9787,
        0x009DAEAA, 0x002C3231, 0x00494036, 0x00464A3E, 0x0044483C, 0x0042463A, 0x00414539, 0x00414539, 0x00414539, 0x00404438, 0x00414539, 0x00474A3B, 0x005D5F52, 0x005A5C4F, 0x00585B4D, 0x005F6255, 0x005E6153, 0x0057594B, 0x005F6154, 0x0055584A,
        0x00595C4E, 0x00626456, 0x005F6053, 0x005E5E50, 0x00676759, 0x005B5D4F, 0x005D6054, 0x00545648, 0x005A5B4D, 0x005D5E4F, 0x00727367, 0x00737366, 0x00595B4C, 0x00505243, 0x00636558, 0x006E7065, 0x00595C4D, 0x0053574A, 0x005D5F51, 0x006D7068,
        0x00616458, 0x005F6052, 0x00575849, 0x00717367, 0x006B6D60, 0x00626457, 0x005C5E50, 0x005D5F50, 0x00797A6D, 0x0069695B, 0x00595A4B, 0x00616254, 0x00646456, 0x00676759, 0x005A5A4D, 0x00575A4B, 0x0068685A, 0x0077786B, 0x005E5F50, 0x00565748,
        0x00626456, 0x00787A6D, 0x00656758, 0x00595B4C, 0x00616455, 0x0065685A, 0x006F7165, 0x005D5F51, 0x00626558, 0x004B4D3E, 0x004A4B3C, 0x004A4A3C, 0x004A4A3C, 0x004A4A3C, 0x0049493B, 0x00414730, 0x001A2D1E, 0x00898786, 0x00819894, 0x00638A82,
        0x00759B9B, 0x00387D86, 0x00175069, 0x0011484D, 0x000B4B56, 0x00004861, 0x00016288, 0x002593C3, 0x00299DD4, 0x001990C8, 0x001E95CD, 0x00146996, 0x00152F3F, 0x00393C33, 0x00393D31, 0x003E382D, 0x003A3C28, 0x002E3F2A, 0x00373933, 0x00373D32,
        0x00363C31, 0x00363C31, 0x00383E33, 0x003B4136, 0x003C4237, 0x003B4136, 0x00393F34, 0x003E4237, 0x003F4338, 0x003F4338, 0x003E4237, 0x003C4035, 0x003B3F34, 0x003A3E33, 0x00393D32, 0x003D4136, 0x003A3E33, 0x003A3E33, 0x0041453A, 0x00383C31,
        0x003E4237, 0x003F4338, 0x003E4237, 0x0040463B, 0x00373D32, 0x00363C31, 0x003C4237, 0x003C4237, 0x00383E33, 0x003B4136, 0x0041473C, 0x003F4036, 0x00414238, 0x00404137, 0x003E3F35, 0x003F4036, 0x003D3E34, 0x003C3D33, 0x003E3F35, 0x003A3C30,
        0x003A3D34, 0x0035382F, 0x003A3A2C, 0x003B3D2A, 0x00243227, 0x000C3036, 0x000B4055, 0x0006455A, 0x0007465B, 0x0009485D, 0x000A495E, 0x000A495E, 0x000A495E, 0x0009485D, 0x0009485D, 0x000C4E61, 0x00084A5D, 0x00064559, 0x00104E60, 0x000F4A5D,
        0x000D4959, 0x00104B5B, 0x00003747, 0x001D4446, 0x00012830, 0x00174253, 0x00074055, 0x00004456, 0x0000434F, 0x00054246, 0x000B3B3C, 0x00012235, 0x000B2738, 0x00193343, 0x00092B38, 0x001D4C54, 0x00679A9D, 0x00739C9F, 0x001C3A3F, 0x001B3241,
        0x00252A2B, 0x0019302C, 0x000E2D30, 0x00282429, 0x0043372B, 0x003B3F2C, 0x00373A31, 0x0047322A, 0x00342921, 0x002B2F29, 0x00253A37, 0x001B393A, 0x0015353A, 0x0016343F, 0x000F2A38, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFCDD1D6, 0xFFC3C8CB, 0xFFC8CDD0, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00070D0C, 0x00030908, 0x00000201, 0x00050A0B, 0x0004090A, 0x00000003, 0x00020509, 0x00080B0F, 0x000A0D12, 0x0000070A, 0x00030C0F, 0x00081114, 0x00050E11, 0x0000080B,
        0x00020B0E, 0x00060F12, 0x00040D10, 0x00071416, 0x0007151B, 0x001F2B35, 0x00233040, 0x000F1D30, 0x00061528, 0x00051728, 0x0007192A, 0x00051A40, 0x000D2041, 0x0009172D, 0x00000814, 0x00000B11, 0x00001013, 0x00000A0F, 0x00000C12, 0x0005191E,
        0x00001116, 0x00091F24, 0x000D2328, 0x00041D21, 0x000B262A, 0x001A3539, 0x00233E42, 0x002F4543, 0x00344A48, 0x00374F4F, 0x00334D4D, 0x00365252, 0x002F4D4E, 0x00224041, 0x003E5D5E, 0x0077BDD5, 0x004FCBF3, 0x001EC6FB, 0x001EC0FB, 0x002CB8FF,
        0x0022B7FF, 0x0021C7FF, 0x0025C8F9, 0x001EBDFB, 0x003ACBF1, 0x0078CEE0, 0x00799FA3, 0x00608281, 0x003C4B4D, 0x003E3839, 0x00363E37, 0x00394136, 0x00393E35, 0x00404138, 0x00404138, 0x003F3F39, 0x003B3F39, 0x00212823, 0x005A645E, 0x00527469,
        0x009CA1A2, 0x00C7EAF4, 0x00247B8B, 0x000B6583, 0x004090B9, 0x001879A5, 0x000F7FAD, 0x002599D4, 0x0037A3DD, 0x003096D1, 0x00218AC3, 0x001A8ABF, 0x001B91C2, 0x00269AC9, 0x002C9CCA, 0x003399BC, 0x002F93B6, 0x003E8AA0, 0x005B847F, 0x00869789,
        0x00A1AAA7, 0x00313131, 0x004D4137, 0x0045493D, 0x0043473B, 0x00404438, 0x003F4337, 0x00404438, 0x00414539, 0x00414539, 0x00BDBFBB, 0x007C7E73, 0x00C4C5C0, 0x009D9F96, 0x0091938A, 0x00B9BAB4, 0x00C6C7C2, 0x0066695C, 0x00E1E2DF, 0x007A7C71,
        0x00AAABA4, 0x00B5B7B0, 0x00B5B6B0, 0x00787B6F, 0x00E4E5E2, 0x006A6C60, 0x00CBCCC8, 0x00575A4C, 0x004C4F40, 0x00A0A199, 0x00D1D2CE, 0x00C6C6C2, 0x00777A6E, 0x0076796D, 0x00D3D3D0, 0x00C9C9C4, 0x00AEAFA7, 0x00616357, 0x00D6D6D3, 0x00CDCECB,
        0x00DDDDDA, 0x008D8D82, 0x0087887D, 0x00E0E0DD, 0x00CBCBC7, 0x00DBDCD9, 0x00606355, 0x00BABBB5, 0x00C7C8C3, 0x00D3D3CF, 0x00747568, 0x00B5B5AE, 0x00DADAD7, 0x00ADADA6, 0x004D4F40, 0x006D7165, 0x00D9D9D6, 0x00D4D4D0, 0x00B4B4AE, 0x00636456,
        0x00CECECA, 0x00CECFCA, 0x00D6D7D3, 0x006C6F61, 0x00AFB0A9, 0x00D1D2CE, 0x00DDDEDB, 0x00CFD0CC, 0x00D5D7D3, 0x00CDCFCA, 0x004B4D3E, 0x004C4C3E, 0x004B4B3D, 0x004A4A3C, 0x004A4A3C, 0x00474838, 0x001C2D20, 0x008A8A84, 0x007A958C, 0x00528178,
        0x0066A2A2, 0x002F9BAD, 0x00117CA2, 0x0010555F, 0x00074C56, 0x0000414F, 0x0000495F, 0x00166E8C, 0x002C92BD, 0x002599CE, 0x00128ECA, 0x0017688D, 0x0018303C, 0x0035392E, 0x00353C2F, 0x003F382F, 0x003B3D2A, 0x0030402E, 0x003B3D37, 0x0041473C,
        0x003F453A, 0x003D4338, 0x003F453A, 0x0042483D, 0x0042483D, 0x0040463B, 0x003D4338, 0x003E463B, 0x00454D42, 0x00474F44, 0x00424A3F, 0x003F473C, 0x0040483D, 0x003F473C, 0x003C4439, 0x00444C41, 0x003A4237, 0x003D453A, 0x004B5348, 0x003D453A,
        0x00424A3F, 0x00454D42, 0x00424A3F, 0x0041473C, 0x00363C31, 0x003A4035, 0x00454B40, 0x00444A3F, 0x00444A3F, 0x004B5146, 0x004D5348, 0x0043493E, 0x00484E43, 0x00444A3F, 0x0042483D, 0x00474D42, 0x0043493E, 0x003C4237, 0x003F453A, 0x00393B2F,
        0x00393C33, 0x0035382F, 0x0039392B, 0x003A3C29, 0x00233126, 0x000B2F35, 0x000A3F54, 0x0006455A, 0x0007465B, 0x0008475C, 0x000A495E, 0x000A495E, 0x000A495E, 0x0009485D, 0x0009485D, 0x00084860, 0x00094961, 0x000A495E, 0x000C4B60, 0x000E4A60,
        0x000E495D, 0x000E495D, 0x000A4559, 0x000C4D55, 0x000E4756, 0x00154A5F, 0x0007435B, 0x0000455C, 0x00024C5E, 0x00054654, 0x000B424B, 0x00103D58, 0x00002440, 0x00062943, 0x00032C43, 0x00002034, 0x0016485A, 0x00264E60, 0x00001529, 0x00001526,
        0x00000911, 0x0000161B, 0x00002634, 0x00071626, 0x001F2422, 0x00162D1F, 0x001C322D, 0x002C1C16, 0x001A1009, 0x0015120D, 0x0010140F, 0x00121513, 0x001F1F1F, 0x002B2427, 0x0025171D, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFBFEFF, 0xFFF8FBFF, 0xFFFBFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x000A0F0E, 0x000A0F0D, 0x00000200, 0x00030807, 0x00050A09, 0x00000506, 0x00080D0E, 0x000C1114, 0x000C1114, 0x000A1316, 0x000D1619, 0x000C1518, 0x00060F12, 0x00030C0F,
        0x00060F12, 0x00081114, 0x00040D10, 0x00030F13, 0x0000080E, 0x000F1D29, 0x00253545, 0x001D3045, 0x0011283E, 0x000D2640, 0x000A253F, 0x00173050, 0x00112641, 0x00071424, 0x0000060D, 0x000C1112, 0x00131D1D, 0x000A1B1E, 0x000B2126, 0x0025383B,
        0x00192B2C, 0x0020302F, 0x00162724, 0x00061511, 0x0014221C, 0x0028352D, 0x002B3830, 0x00363832, 0x00383C36, 0x0041453F, 0x003D433E, 0x00414843, 0x0037403D, 0x001F2A27, 0x0056615E, 0x008EB0C8, 0x0088CEF3, 0x0041C8FA, 0x0016BCF9, 0x0022B5FF,
        0x0032BCFF, 0x0039C6FF, 0x0030B1EA, 0x003EB2E9, 0x0048B4D6, 0x0096D1E1, 0x007F979D, 0x005F7F7E, 0x003B524E, 0x0029342A, 0x00304C38, 0x00383E33, 0x00363A2F, 0x003F4036, 0x003C3A32, 0x003F3D35, 0x003D3F39, 0x00232A23, 0x0059645C, 0x00667C70,
        0x0099A29F, 0x00ABCACD, 0x000A515B, 0x00003F54, 0x00237293, 0x001D789F, 0x00339FC8, 0x00219CE0, 0x002BA3E5, 0x002EA0DF, 0x002496D2, 0x001889C1, 0x00188BBE, 0x00208FC1, 0x001482B2, 0x001A8EAF, 0x00208CAE, 0x00358197, 0x005E837F, 0x008C9B8D,
        0x00A0A9A6, 0x002A2F2E, 0x00423E33, 0x0045493D, 0x0043473B, 0x00404438, 0x003E4236, 0x003F4337, 0x00404438, 0x00414539, 0x00AFB0AB, 0x00C3C4BF, 0x00D4D5D2, 0x00CACBC6, 0x00CDCECA, 0x008C8E84, 0x00C2C3BF, 0x00BBBCB6, 0x00E5E5E3, 0x00B1B2AC,
        0x00D9D9D6, 0x007D7F74, 0x00DADBD8, 0x00B4B5AF, 0x00E9EAE8, 0x00BBBCB6, 0x00CDCEC9, 0x0045483A, 0x00494C3F, 0x00AEAEA8, 0x00D9DAD6, 0x00B8B9B3, 0x007B7D71, 0x00C5C6C1, 0x00D7D8D4, 0x00ADAEA6, 0x00E3E3E1, 0x00B4B5AF, 0x00D3D3CF, 0x004E5042,
        0x00CDCDC8, 0x00ADADA6, 0x00D4D4D0, 0x008B8B81, 0x005E6052, 0x00EDEDEC, 0x008E9086, 0x00F4F5F4, 0x00A8A9A1, 0x00DCDCD8, 0x00ACACA5, 0x00CECEC9, 0x00B1B1AA, 0x00545547, 0x004D4E3F, 0x00BFC0BB, 0x00C0C2BC, 0x00555747, 0x0078796D, 0x00AAAAA2,
        0x00D3D3CF, 0x004E4F40, 0x00C1C1BC, 0x00BBBDB6, 0x00C5C6C0, 0x009D9F96, 0x00989990, 0x00D0D1CD, 0x00696E5F, 0x00F1F2F0, 0x004D5242, 0x004C4E3F, 0x004A4A3C, 0x0049493B, 0x0048483A, 0x00434337, 0x001B2B20, 0x008F8D85, 0x0082988C, 0x004D786D,
        0x004D898F, 0x001C94B1, 0x001A9DCF, 0x0024829F, 0x0001566C, 0x00004653, 0x0007454F, 0x00014552, 0x00176B85, 0x002F95BF, 0x002396C9, 0x00176582, 0x001E343A, 0x00373A2B, 0x00353C2F, 0x003F3932, 0x00383B2B, 0x00303E2C, 0x003E3B36, 0x0043493E,
        0x0041473C, 0x003E4439, 0x003F453A, 0x0041473C, 0x0041473C, 0x003D4338, 0x00393F34, 0x00384236, 0x003E483C, 0x00424C40, 0x00404A3E, 0x003E483C, 0x003E483C, 0x003E483C, 0x003C463A, 0x00404A3E, 0x00374135, 0x00384236, 0x00465044, 0x003B4539,
        0x003D473B, 0x003C463A, 0x003C463A, 0x003F453A, 0x00383E33, 0x003E4439, 0x00444A3F, 0x0041473C, 0x00474D42, 0x00484E43, 0x003B4136, 0x003F473C, 0x00464E43, 0x00424A3F, 0x0040483D, 0x00474F44, 0x003F473C, 0x00323A2F, 0x00353D32, 0x00383A2E,
        0x00383B32, 0x0034372E, 0x0039392B, 0x003A3C29, 0x00233126, 0x000A2E34, 0x00093E53, 0x0006455A, 0x0007465B, 0x0008475C, 0x0009485D, 0x000A495E, 0x000A495E, 0x0009485D, 0x0008475C, 0x0003405A, 0x000B4862, 0x000C4963, 0x0004425A, 0x0006445C,
        0x000C485E, 0x000A465C, 0x000E4A60, 0x00044D5B, 0x000B485C, 0x0011425C, 0x00104260, 0x00013B58, 0x0005455D, 0x000D465B, 0x000C3E50, 0x00053E57, 0x00063953, 0x0006344C, 0x00003046, 0x00003045, 0x002D6578, 0x002C576C, 0x00001E33, 0x0004243B,
        0x000B1E2D, 0x00073546, 0x000D4E6A, 0x00001D39, 0x00032025, 0x0008332A, 0x0028504E, 0x0021221E, 0x00000200, 0x00010801, 0x000C140D, 0x001A1F16, 0x0026241C, 0x002F2720, 0x00251A12, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFDFEFF, 0xFFFCFEFF, 0xFFFDFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00161917, 0x00111510, 0x00000300, 0x00000503, 0x00030806, 0x00010706, 0x00070F0E, 0x00091111, 0x00081010, 0x00091013, 0x000C1316, 0x000C1316, 0x000A1114, 0x000C1316,
        0x000E1518, 0x000B1215, 0x00060D10, 0x000B131A, 0x00000C12, 0x00030F19, 0x00142631, 0x001F3344, 0x001C3549, 0x0019344E, 0x00193651, 0x00132C36, 0x00091B22, 0x00040B0E, 0x000B0908, 0x00191612, 0x001D1E1C, 0x001E2324, 0x00232F31, 0x00627271,
        0x003F4F4E, 0x00384543, 0x002E3A34, 0x001B241A, 0x001D2318, 0x002C3024, 0x00373A2B, 0x003C3935, 0x003C3B37, 0x0042413D, 0x003D3E3C, 0x00424543, 0x003D4240, 0x002A302F, 0x007A807F, 0x0071818D, 0x00CBD0E9, 0x0085D8FE, 0x0016C1F4, 0x0024C5FF,
        0x0045BDF9, 0x002B8EC2, 0x00004B7C, 0x0000466E, 0x00074D65, 0x0073939E, 0x00848D90, 0x006A8280, 0x004A5B57, 0x00353A2B, 0x002E4428, 0x00363E33, 0x00373D32, 0x0042433A, 0x0039372F, 0x00393933, 0x003C403A, 0x00283029, 0x00515B55, 0x005F736E,
        0x00788F8B, 0x00759495, 0x00144C51, 0x0000414F, 0x00004960, 0x00136382, 0x0041A2C3, 0x001C9EE4, 0x001297DB, 0x001395D8, 0x0027A2E0, 0x002798D1, 0x001177AB, 0x0013719E, 0x00207CA5, 0x00097799, 0x00157B9E, 0x002F788E, 0x005D7E7A, 0x008C998B,
        0x009CAAA6, 0x00253331, 0x003D4639, 0x00464A3E, 0x0043473B, 0x003F4337, 0x003D4135, 0x003D4135, 0x003E4236, 0x003F4337, 0x006D7067, 0x00FFFFFF, 0x00ADAFA8, 0x00CACBC7, 0x00F0F0EF, 0x004C5042, 0x00999B93, 0x00FFFFFF, 0x0095978F, 0x00EDEDEC,
        0x00CFD0CC, 0x00494C3D, 0x00C6C7C2, 0x00F4F4F3, 0x00999B92, 0x00FFFFFF, 0x00A0A199, 0x0077796E, 0x006B6D5F, 0x008A8A80, 0x00A4A59C, 0x00D0D0CD, 0x00D0D1CD, 0x00BFC0BA, 0x00D7D8D4, 0x0096978E, 0x00A8A9A1, 0x0094958C, 0x00E2E3E0, 0x007C7D73,
        0x00DDDDDA, 0x00A5A69D, 0x00C4C4BF, 0x00B5B5AE, 0x008A8B81, 0x00EEEFED, 0x0084867C, 0x00F3F3F2, 0x009B9D94, 0x00AEAFA8, 0x007D7D71, 0x00D3D3CF, 0x0093948A, 0x006F7164, 0x0077786D, 0x00B0B1AB, 0x00D1D2CE, 0x0076776B, 0x00A0A097, 0x009F9F96,
        0x00E0E0DD, 0x00666759, 0x00D1D1CD, 0x00AFAFA8, 0x00C6C7C2, 0x0093948A, 0x0097978D, 0x00CBCBC6, 0x00616355, 0x00F3F4F2, 0x00515444, 0x004F5040, 0x0049493B, 0x00474739, 0x00464638, 0x0045443A, 0x001D2E23, 0x008E8B83, 0x008D9B8F, 0x00658077,
        0x0055828D, 0x001783A5, 0x001C9AD5, 0x002A9ED3, 0x001177A1, 0x0001536C, 0x00064750, 0x000A464C, 0x00084D57, 0x0010657B, 0x002282A0, 0x00145D71, 0x00263837, 0x003D3F2C, 0x00363D30, 0x003E3A35, 0x0036392A, 0x002C3A28, 0x00393530, 0x0041473C,
        0x0040463B, 0x003E4439, 0x003F453A, 0x0041473C, 0x0041473C, 0x003F453A, 0x003C4237, 0x003B4539, 0x003C463A, 0x00404A3E, 0x00434D41, 0x00404A3E, 0x003B4539, 0x003B4539, 0x003F493D, 0x00404A3E, 0x003C463A, 0x00374135, 0x00434D41, 0x003E483C,
        0x003D473B, 0x00374135, 0x003D473B, 0x0041473C, 0x00373D32, 0x003B4136, 0x0041473C, 0x0042483D, 0x00494F44, 0x00484E43, 0x00373D32, 0x003E463B, 0x00464E43, 0x00434B40, 0x0041493E, 0x00474F44, 0x003E463B, 0x002F372C, 0x0030382D, 0x0037392D,
        0x00373A31, 0x0034372E, 0x0039392B, 0x003A3C29, 0x00233126, 0x000A2E34, 0x00093E53, 0x00054459, 0x0006455A, 0x0008475C, 0x0009485D, 0x000A495E, 0x0009485D, 0x0009485D, 0x0008475C, 0x000D4A64, 0x0008455F, 0x0005425C, 0x0005435B, 0x000C4A62,
        0x000D4E63, 0x0007485D, 0x0007485D, 0x00004B5C, 0x0004495D, 0x000C445D, 0x00164D68, 0x000D4A64, 0x0005485F, 0x0007465B, 0x000C4358, 0x00014655, 0x00074452, 0x000A4352, 0x00064150, 0x0000424F, 0x00317280, 0x002F6471, 0x000C3544, 0x00083148,
        0x0008273C, 0x0008465E, 0x001B6E94, 0x00002B52, 0x00002936, 0x00023E38, 0x00306764, 0x00323533, 0x00000200, 0x00010803, 0x000E170D, 0x00192317, 0x001D2417, 0x00232819, 0x00191C0C, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFCFBFD, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x001F201E, 0x00171913, 0x00050602, 0x00050904, 0x00040A05, 0x00000704, 0x00030C09, 0x00040F0D, 0x0005100E, 0x00040C0C, 0x00070F0F, 0x00060E0E, 0x00070F0F, 0x00091111,
        0x00081010, 0x00050D0D, 0x00060E0E, 0x000A1017, 0x000C141B, 0x00000B0F, 0x00000E11, 0x00061920, 0x00071D29, 0x00041A2C, 0x0000172B, 0x00011814, 0x0000100F, 0x00020A0A, 0x00101010, 0x00140E0F, 0x00130F0E, 0x00201F1B, 0x00222621, 0x00485857,
        0x00364442, 0x00313F3B, 0x0026302A, 0x00192019, 0x00262920, 0x00323329, 0x0036362A, 0x00393A36, 0x003B3C38, 0x003E413F, 0x003B403E, 0x003E4443, 0x00394140, 0x00283030, 0x00818B8B, 0x005A736F, 0x00C5B4BD, 0x0081BFD7, 0x0017C1EA, 0x002ECBFD,
        0x002F98BF, 0x00035771, 0x00023B5A, 0x0007455D, 0x00003D48, 0x00294C50, 0x00364548, 0x001B363A, 0x001E2A2C, 0x00473B35, 0x004F4B38, 0x00373F38, 0x00343B34, 0x00383C36, 0x00343531, 0x00383C37, 0x003A3F3D, 0x001F2825, 0x0024322E, 0x0010292D,
        0x00194546, 0x00244A4E, 0x00164347, 0x000F555C, 0x00004352, 0x000C4C64, 0x00156780, 0x001D94CC, 0x0029A9E4, 0x001CA3E1, 0x00199CDB, 0x002A9CD8, 0x002787BC, 0x00247AA4, 0x002B7DA0, 0x001A7EA1, 0x002280A4, 0x003E8198, 0x0062817E, 0x00869587,
        0x0098A9A5, 0x00213431, 0x003B483A, 0x0044483C, 0x00414539, 0x003E4236, 0x003C4034, 0x003C4034, 0x003D4135, 0x003D4135, 0x0041453A, 0x00BDBEB9, 0x006D7064, 0x008D8E85, 0x00A7A9A2, 0x00464B3C, 0x00616558, 0x00C3C4BF, 0x00575B4E, 0x00B0B2AB,
        0x0087897F, 0x0045493A, 0x0083867B, 0x00B8BAB4, 0x00585D4F, 0x00C2C3BF, 0x0066695C, 0x00A6A7A0, 0x00898A7F, 0x008A8A80, 0x00C9C9C4, 0x00C1C1BC, 0x00888A7F, 0x0065685A, 0x00C2C3BE, 0x00C7C8C4, 0x0093958C, 0x00545648, 0x00B1B2AC, 0x00BCBDB8,
        0x00E7E7E5, 0x00A1A199, 0x00707063, 0x00C6C6C1, 0x00BDBDB7, 0x00F4F4F3, 0x00606355, 0x009A9C94, 0x00CACBC7, 0x00BCBDB7, 0x00636457, 0x00A4A59D, 0x0076796D, 0x00909289, 0x009FA098, 0x005E6053, 0x00C4C5BF, 0x00D4D4D0, 0x009D9D94, 0x00565647,
        0x00B2B2AB, 0x00D1D1CD, 0x00BCBCB6, 0x00636455, 0x00A1A299, 0x007F8074, 0x00808074, 0x00A1A299, 0x005A5D4E, 0x00BABBB5, 0x00535646, 0x004F5140, 0x0048483A, 0x00464638, 0x00454537, 0x00444339, 0x00203128, 0x008C8883, 0x008E978D, 0x0075857E,
        0x006B8994, 0x002384A6, 0x002097CF, 0x001595D6, 0x002195CC, 0x001B7CA2, 0x000A596E, 0x00044953, 0x00064750, 0x00014754, 0x00034B5C, 0x00024855, 0x0025352E, 0x003E3F2B, 0x00363D30, 0x003E3B36, 0x0037392D, 0x00323D2D, 0x003E3833, 0x003D4338,
        0x003D4338, 0x003E4439, 0x0040463B, 0x0042483D, 0x0043493E, 0x0043493E, 0x0042483D, 0x00424A3F, 0x0041493E, 0x00434B40, 0x00464E43, 0x00434B40, 0x003C4439, 0x003B4338, 0x0040483D, 0x00444C41, 0x00434B40, 0x003C4439, 0x00444C41, 0x00434B40,
        0x00444C41, 0x003B4338, 0x0041493E, 0x00444A3F, 0x0033392E, 0x00343A2F, 0x0041473C, 0x00464C41, 0x00484E43, 0x004A5045, 0x00474D42, 0x0041473C, 0x00474D42, 0x00464C41, 0x00454B40, 0x00484E43, 0x0040463B, 0x00343A2F, 0x00343A2F, 0x0037392D,
        0x00373A31, 0x0035382F, 0x003A3A2C, 0x003B3D2A, 0x00233126, 0x000A2E34, 0x00083D52, 0x00054459, 0x0006455A, 0x0008475C, 0x0009485D, 0x0009485D, 0x0009485D, 0x0008475C, 0x0008475C, 0x000B475F, 0x000A465E, 0x00115065, 0x00155469, 0x0006475C,
        0x00003F52, 0x0003475A, 0x00094D60, 0x00004E5D, 0x00004E5F, 0x00004557, 0x00004355, 0x00004E5F, 0x00004656, 0x00004455, 0x00074C5F, 0x00034B53, 0x0004424C, 0x0008434D, 0x00064750, 0x0000454D, 0x0000424A, 0x0000313D, 0x0010414F, 0x0010445B,
        0x000E394E, 0x00105C79, 0x002F93C3, 0x00004170, 0x00003A4D, 0x00054745, 0x00377470, 0x00393132, 0x00050000, 0x00100B08, 0x0016130E, 0x001B1C12, 0x00202216, 0x00252A1B, 0x00151B0A, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFCFBFD, 0xFFFFFEFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00181917, 0x001C1C16, 0x00070903, 0x00060A04, 0x00060C07, 0x00010803, 0x00020D0A, 0x0003110D, 0x0005130F, 0x00090E0F, 0x00090E0F, 0x00070C0D, 0x000B1011, 0x000F1415,
        0x00090E0F, 0x00030809, 0x00080D0E, 0x000A0C16, 0x0013171C, 0x000A1212, 0x0006110E, 0x00071513, 0x00051416, 0x0004141B, 0x00000D16, 0x00041A1F, 0x000F212A, 0x000F1A28, 0x00131A29, 0x000D0F1A, 0x0017191A, 0x0035392E, 0x00282E1D, 0x00223330,
        0x002D3B39, 0x0034413F, 0x001F2A27, 0x00121B18, 0x00232826, 0x002C302B, 0x00262A25, 0x00363B32, 0x003B4037, 0x003D453B, 0x003E463F, 0x003A453D, 0x00333D37, 0x00212D27, 0x007C8882, 0x00567770, 0x00B2AEAD, 0x0092D9E7, 0x003ED3FB, 0x002997CB,
        0x00004D6A, 0x00003B43, 0x00165766, 0x000A475B, 0x00004852, 0x000D4148, 0x000B333F, 0x00002B3B, 0x00102E3F, 0x0028242F, 0x00262423, 0x001F2A27, 0x001C2522, 0x00212825, 0x00222726, 0x00252B2A, 0x00232B2B, 0x000D1919, 0x00031312, 0x00032737,
        0x00003B43, 0x001B4D53, 0x00284E53, 0x00135357, 0x0005434D, 0x00134557, 0x0006485B, 0x00186A89, 0x002B93B8, 0x0026A3D5, 0x001CA0DC, 0x00279FE0, 0x003AA3DC, 0x003C9AC7, 0x002482A5, 0x0016779B, 0x001D789D, 0x0044889F, 0x00678884, 0x0089988A,
        0x00A1AFAB, 0x0025302E, 0x00393B2F, 0x00404438, 0x003E4236, 0x003C4034, 0x003C4034, 0x003C4034, 0x003D4135, 0x003C4034, 0x003C4034, 0x00404334, 0x00424637, 0x00414536, 0x00434637, 0x00434738, 0x00434739, 0x00424738, 0x00434637, 0x00434738,
        0x00414435, 0x003E4132, 0x00434536, 0x00484B3D, 0x00484C3E, 0x00484B3C, 0x0046493A, 0x004D4F41, 0x00515242, 0x00505141, 0x00555546, 0x00545445, 0x004D4E3E, 0x004F5141, 0x00515445, 0x004D5042, 0x004C4F40, 0x005C5E51, 0x00C9CAC6, 0x00ACACA7,
        0x00DBDBD8, 0x00757669, 0x008F9085, 0x00C8C9C4, 0x00B9B9B2, 0x00CACAC5, 0x00545647, 0x004E5041, 0x00555748, 0x00545545, 0x004B4B3B, 0x004D4F3F, 0x004E5142, 0x00505344, 0x004C4E3F, 0x004D4F40, 0x00515242, 0x00555646, 0x00505141, 0x00535444,
        0x00515243, 0x00565646, 0x004E4E3E, 0x00545545, 0x00575848, 0x00585949, 0x00555646, 0x00515343, 0x004E5142, 0x00505343, 0x00515444, 0x004C4E3F, 0x0048493B, 0x00464638, 0x00464638, 0x003F3B30, 0x001B2C23, 0x008A8B87, 0x008E9B93, 0x006F817A,
        0x0064838C, 0x00197E97, 0x00108DBA, 0x001591D1, 0x00249AD5, 0x002C9BC9, 0x0016789A, 0x00004F67, 0x0000485A, 0x000B4A58, 0x00003A46, 0x0000373E, 0x00243227, 0x0041402B, 0x00323B2E, 0x003C3935, 0x00383A2E, 0x00364131, 0x00443B37, 0x00363C31,
        0x00373D32, 0x00393F34, 0x003B4136, 0x003B4136, 0x003B4136, 0x003C4237, 0x003D4338, 0x003E4237, 0x003E4237, 0x003E4237, 0x003F4338, 0x003D4136, 0x003A3E33, 0x00393D32, 0x003A3E33, 0x003F4338, 0x003C4035, 0x00373B30, 0x003F4338, 0x003A3E33,
        0x0041453A, 0x003D4136, 0x003C4035, 0x003E4439, 0x00343A2F, 0x0033392E, 0x00393F34, 0x003A4035, 0x00373D32, 0x00383E33, 0x003D4338, 0x003C3D33, 0x003E3F35, 0x003F4036, 0x003E3F35, 0x003D3E34, 0x00393A30, 0x0036372D, 0x0035362C, 0x0037392D,
        0x00383B32, 0x0035382F, 0x003A3A2C, 0x003C3E2B, 0x00243227, 0x000A2E34, 0x00083D52, 0x00054459, 0x0006455A, 0x0007465B, 0x0009485D, 0x0009485D, 0x0009485D, 0x0008475C, 0x0008475C, 0x0009465A, 0x00084559, 0x0008475B, 0x00054759, 0x00003C4D,
        0x00165B6A, 0x004F96A4, 0x005AA1AF, 0x0055AAB4, 0x005FABB7, 0x0069B0BA, 0x00549FA8, 0x0056ABB3, 0x0050A8AF, 0x0058A8B3, 0x005BA1AE, 0x0062A1AF, 0x0072ABBA, 0x006299A8, 0x00145262, 0x00003F4D, 0x00004A58, 0x00034557, 0x000E4357, 0x00044155,
        0x00043B50, 0x000D6786, 0x003DABDF, 0x00004E82, 0x0006465E, 0x00105754, 0x004C8985, 0x00383433, 0x00040000, 0x000E0906, 0x00140F0C, 0x001E1B16, 0x0022231A, 0x00272B20, 0x0013190E, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFDFBFB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x0024231F, 0x001E1F16, 0x00070701, 0x00040600, 0x00040B04, 0x00040B06, 0x0007130D, 0x00071511, 0x00061511, 0x000A0F10, 0x00090E0F, 0x000C1112, 0x001A1F20, 0x0024292A,
        0x00181D1E, 0x00090E0F, 0x00080D0E, 0x000D0E18, 0x000D1014, 0x00080F0C, 0x00111A10, 0x00101B11, 0x0009150F, 0x00101B19, 0x000E171A, 0x00071D2F, 0x00162A43, 0x000B1D3C, 0x00071532, 0x00000B21, 0x001C272B, 0x004B5646, 0x00323E22, 0x00041512,
        0x00172523, 0x002C3A38, 0x00263232, 0x001B2525, 0x00232B2B, 0x002D3535, 0x00393E41, 0x003C4231, 0x00404536, 0x003D4435, 0x003D4737, 0x00394336, 0x00323F31, 0x00293529, 0x00849286, 0x005A7572, 0x0095A09D, 0x0079CBD6, 0x002397C6, 0x000B417E,
        0x000F3E5A, 0x00025A5A, 0x00034A54, 0x000F455C, 0x00044B59, 0x000C4957, 0x0000384C, 0x00004B66, 0x002A6481, 0x00041C32, 0x00000812, 0x00000705, 0x00000A0A, 0x000C1414, 0x000B1313, 0x0001080B, 0x0000080A, 0x00051417, 0x000D1E21, 0x0000263D,
        0x00003E4B, 0x004A838C, 0x006F939B, 0x00407D7F, 0x002B676D, 0x00497380, 0x00528B9A, 0x006E9FA7, 0x003C889E, 0x002596BE, 0x0028A9E2, 0x000E8CCD, 0x000B7CBA, 0x00208DC0, 0x001A87AD, 0x00197DA0, 0x0017779B, 0x0042889F, 0x005E837F, 0x00849385,
        0x00AAB3B0, 0x00343434, 0x00483E34, 0x003D4135, 0x003C4034, 0x003B3F33, 0x003B3F33, 0x003C4034, 0x003D4135, 0x003D4135, 0x003C4034, 0x003E4132, 0x00454839, 0x00434738, 0x00404536, 0x00424738, 0x00434839, 0x00424738, 0x00424737, 0x0045493A,
        0x00404232, 0x00464838, 0x004A4C3C, 0x004C4E3F, 0x004F5041, 0x004C4E3F, 0x0046493A, 0x004D5042, 0x00545545, 0x00515242, 0x004D4D3D, 0x00505040, 0x00515141, 0x004E5040, 0x00515445, 0x004A4D3E, 0x004D4F41, 0x00515343, 0x004D4F40, 0x004B4E41,
        0x004D4F3F, 0x004F5040, 0x00535444, 0x004E5042, 0x00505142, 0x004F5141, 0x00505242, 0x00515444, 0x004F5243, 0x00505142, 0x00515141, 0x004D4E3E, 0x004F5141, 0x00515445, 0x004E5142, 0x004D5041, 0x00505243, 0x00515242, 0x004F5040, 0x00535444,
        0x004E5042, 0x00505142, 0x00515242, 0x00535444, 0x00575848, 0x00575848, 0x00515444, 0x004C5041, 0x00494E3F, 0x00494E3F, 0x004D5243, 0x004C4F40, 0x0049493B, 0x00474739, 0x00464638, 0x004C4339, 0x0015291D, 0x00838984, 0x008A9F97, 0x0069807B,
        0x005F858A, 0x0019899B, 0x000F9ABB, 0x002396CF, 0x001B90C7, 0x002398CB, 0x002B9BC6, 0x001A7B9F, 0x0000506B, 0x00004154, 0x00104756, 0x00003539, 0x002B382A, 0x0046462E, 0x00313A2D, 0x00383531, 0x0031352A, 0x00313C2C, 0x003F3632, 0x003A4035,
        0x003C4237, 0x003D4338, 0x003D4338, 0x003A4035, 0x00393F34, 0x003A4035, 0x003B4136, 0x003A3B31, 0x003E3F35, 0x003E3F35, 0x003A3B31, 0x003B3C32, 0x003E3F35, 0x003E3F35, 0x003A3B31, 0x003B3C32, 0x0034352B, 0x0034352B, 0x003D3E34, 0x0033342A,
        0x003F4036, 0x00414238, 0x00393A30, 0x0031372C, 0x00373D32, 0x00393F34, 0x00363C31, 0x00353B30, 0x00353B30, 0x00363C31, 0x00383E33, 0x003A392F, 0x003A392F, 0x003B3A30, 0x003B3A30, 0x0038372D, 0x0038372D, 0x003A392F, 0x003A392F, 0x0037392D,
        0x00383B32, 0x0035382F, 0x003B3B2D, 0x003C3E2B, 0x00243227, 0x000A2E34, 0x00083D52, 0x00054459, 0x0006455A, 0x0007465B, 0x0009485D, 0x0009485D, 0x0009485D, 0x0008475C, 0x0008475C, 0x000F4A5D, 0x000B495B, 0x00054655, 0x00064958, 0x00094E5D,
        0x00367E8A, 0x0058A2AE, 0x002C7682, 0x003C7884, 0x0048808B, 0x0075ACB5, 0x0064A2A8, 0x005EA8AC, 0x005EAAB0, 0x0067AAB3, 0x004B808D, 0x00397287, 0x0040758A, 0x0077A9BF, 0x004D879D, 0x0000475D, 0x0000394F, 0x00084A63, 0x00033A55, 0x00014657,
        0x00003C50, 0x00076687, 0x0039AEE5, 0x00004C82, 0x0003475E, 0x000E5552, 0x004D8983, 0x00404E4A, 0x00000300, 0x00010803, 0x000B110C, 0x00181E19, 0x001C231C, 0x00222A23, 0x000F1A12, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFEFF, 0xFFFFFCFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00161917, 0x001A1E18, 0x00060802, 0x00090A06, 0x00040803, 0x00020904, 0x00091713, 0x00021611, 0x00041B16, 0x00141918, 0x000F1111, 0x00161315, 0x002A232A, 0x00352F3A,
        0x002A2B39, 0x000E1B2B, 0x002D4152, 0x002E4D44, 0x000B1D1C, 0x0022262B, 0x001C1C1C, 0x002B2F29, 0x002F3D37, 0x0022373F, 0x00041C32, 0x00032633, 0x000D2A39, 0x00072538, 0x00022130, 0x0007121A, 0x003A2D2B, 0x0064584C, 0x00364029, 0x0000211F,
        0x00134650, 0x0014455B, 0x00103549, 0x00203239, 0x003A3830, 0x004D4234, 0x004F4234, 0x004E453B, 0x00464237, 0x0049473C, 0x00414337, 0x003F453A, 0x00384036, 0x002F362F, 0x007F8580, 0x00515F6B, 0x007697A7, 0x005798AD, 0x0001516A, 0x00023E56,
        0x00799AAA, 0x007AA6A5, 0x004C9787, 0x003D7D81, 0x00457579, 0x00729898, 0x00828790, 0x005192AE, 0x0030A1C9, 0x007FA0C1, 0x00535958, 0x00706B68, 0x0051535B, 0x00003250, 0x00096FA3, 0x00044365, 0x00364B42, 0x004D634B, 0x00445B57, 0x00156370,
        0x00588D9A, 0x00647F89, 0x00879DA2, 0x00789A99, 0x0052827C, 0x0078A7A4, 0x00739A98, 0x009E9E80, 0x00236C64, 0x0000687D, 0x0033A3CE, 0x001F9FDA, 0x00016FB1, 0x00095E8B, 0x00198285, 0x00227291, 0x00237A9C, 0x00368299, 0x00567F82, 0x0084928E,
        0x00A2AAAA, 0x0029302D, 0x00363C29, 0x00383F32, 0x003A4134, 0x003B4235, 0x003E4334, 0x003D4233, 0x00404333, 0x00434636, 0x00454838, 0x00454838, 0x00474B3C, 0x00434839, 0x00414637, 0x00434839, 0x0044493A, 0x00434839, 0x00434839, 0x0044493A,
        0x00444738, 0x00484A3B, 0x004F5142, 0x00525345, 0x00515343, 0x004E5142, 0x0046493A, 0x0046483B, 0x00515242, 0x00505141, 0x00535444, 0x00515243, 0x00515141, 0x004D4D3E, 0x00505243, 0x004A4C3D, 0x004D4E40, 0x00535444, 0x00525445, 0x00494C3F,
        0x00464839, 0x004E4F3F, 0x00525343, 0x00505244, 0x00535444, 0x00535444, 0x00535444, 0x00505242, 0x004E5040, 0x004D4D3D, 0x00505040, 0x00515141, 0x004E5040, 0x00515445, 0x004E5142, 0x004E5142, 0x004E5143, 0x004D4F40, 0x004E5040, 0x00535444,
        0x00505244, 0x00535444, 0x00535444, 0x00545545, 0x00535444, 0x00505343, 0x004E5243, 0x004D5243, 0x004A4F40, 0x004A4F40, 0x004B5041, 0x0045493A, 0x00464A3A, 0x00494C3C, 0x00494C3C, 0x003C443D, 0x0021251A, 0x008C8F80, 0x008A9487, 0x005B7B76,
        0x004E8A96, 0x001E7896, 0x001D89B2, 0x0028A1CB, 0x001D98CA, 0x00108DC7, 0x001E98D4, 0x002694C8, 0x001E7D9E, 0x0004535E, 0x00064846, 0x0000343B, 0x001D2D22, 0x00483B2B, 0x00423634, 0x00303233, 0x00393E35, 0x00363C2B, 0x002E372A, 0x00373B30,
        0x00393D32, 0x0035392E, 0x003D4136, 0x00393D32, 0x003E4237, 0x00383C31, 0x00383C31, 0x003B3F33, 0x003A3E32, 0x00393D31, 0x00383C30, 0x00373B2F, 0x00363A2E, 0x00363A2E, 0x00363A2E, 0x00393B2F, 0x00383A2E, 0x00393B2F, 0x003B3D31, 0x00393B2F,
        0x0035372B, 0x0036382C, 0x003A3C30, 0x00363A2E, 0x00363A2E, 0x00373B2F, 0x00373B2F, 0x00373B2F, 0x00383C30, 0x00383C30, 0x00383C30, 0x00373D2C, 0x00383E2D, 0x003A3D2D, 0x003A3D2D, 0x00393C2C, 0x003A3B2B, 0x00393A2A, 0x00383929, 0x00333B2A,
        0x002E372A, 0x00353A31, 0x003C3C30, 0x003A3A2A, 0x00283428, 0x00103232, 0x000D414E, 0x00034355, 0x00074759, 0x000A4A5C, 0x0008475B, 0x00064559, 0x0008475C, 0x0009485D, 0x0008475C, 0x00034859, 0x000D4D5F, 0x00064456, 0x00034857, 0x00014C5A,
        0x005CA4B0, 0x003A7180, 0x00638B9D, 0x00598390, 0x001A525D, 0x006DBBC2, 0x009BF7FC, 0x0096F2F7, 0x00AAF8FF, 0x006EA6B1, 0x0028525F, 0x004F8499, 0x00507D92, 0x003A7180, 0x0044909C, 0x0000434E, 0x00094356, 0x0013455C, 0x0008445C, 0x00043E5B,
        0x000C3E4A, 0x00046A8D, 0x0030ADEA, 0x00095482, 0x000B475F, 0x00095C64, 0x0042897F, 0x00434546, 0x00000100, 0x000B0F09, 0x00181C11, 0x001A1E12, 0x00292B1F, 0x002A2B21, 0x001D1E15, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFEFF, 0xFFFFFCFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00171A18, 0x00161A14, 0x00020400, 0x00080905, 0x000B0F0A, 0x00080F0A, 0x000D1B17, 0x000D211C, 0x00031A15, 0x00001E22, 0x0000282D, 0x00001E23, 0x00102A30, 0x001D373E,
        0x0018353E, 0x000E313B, 0x00254B57, 0x00274858, 0x00071F35, 0x0017283D, 0x00051521, 0x00102B27, 0x00244C41, 0x001E514D, 0x0000363D, 0x00092F34, 0x0018383E, 0x00062A30, 0x00001D21, 0x00061B19, 0x005A524B, 0x00948B7D, 0x005B654E, 0x00001110,
        0x001D3944, 0x002C4E65, 0x0018394D, 0x0011272D, 0x002B352F, 0x00404439, 0x003E3F36, 0x00384739, 0x00364333, 0x00383F30, 0x0043473C, 0x00454642, 0x002D3035, 0x0008101D, 0x00252F41, 0x00003F4A, 0x002B6B7D, 0x00045067, 0x0000425C, 0x00478A9F,
        0x00B0CBD9, 0x00979F9F, 0x00959F93, 0x00568D8A, 0x0077989B, 0x008D9A9C, 0x007B737A, 0x0091D6E7, 0x0049C2DC, 0x0086AFC5, 0x0091948B, 0x007B9F8F, 0x005D7274, 0x00095978, 0x003BBEF0, 0x004194BA, 0x007D9095, 0x0088978F, 0x00709096, 0x006FB5B5,
        0x0087BDBD, 0x00688985, 0x007A918C, 0x008BA39B, 0x006B8A81, 0x00789992, 0x0071938D, 0x0041594D, 0x001D6565, 0x00004255, 0x00226187, 0x00379ED5, 0x0013A2DC, 0x0005749C, 0x001E525F, 0x00408596, 0x002E7C93, 0x003A7F92, 0x005A8183, 0x0082908C,
        0x009BA2A5, 0x00232C29, 0x003A402D, 0x00373E31, 0x00394033, 0x003C4336, 0x003F4435, 0x003F4435, 0x00414434, 0x00424535, 0x00434636, 0x00414434, 0x00484D3E, 0x0044493A, 0x00424738, 0x0044493A, 0x0044493A, 0x00424738, 0x00434839, 0x00454A3B,
        0x00444738, 0x00484A3B, 0x00525344, 0x00545648, 0x00505344, 0x00525445, 0x004D5041, 0x004B4E41, 0x004D4F3F, 0x004F5040, 0x00535444, 0x004E5042, 0x00505142, 0x004F5141, 0x00505242, 0x004B4E3E, 0x004C4E3F, 0x004C4F40, 0x00525445, 0x00494B3D,
        0x00494B3B, 0x004E5041, 0x004F5142, 0x004F5142, 0x00505141, 0x00505142, 0x004C4D3F, 0x004F5141, 0x00505141, 0x00535444, 0x00515243, 0x00515141, 0x004D4D3E, 0x00505243, 0x004E5041, 0x00505243, 0x00535546, 0x004E5141, 0x00505243, 0x00555646,
        0x00515243, 0x00525343, 0x00535445, 0x00505141, 0x004F5141, 0x004B4F40, 0x004D5243, 0x004E5344, 0x004C5142, 0x004A4F40, 0x00494D3E, 0x00464A3A, 0x00464939, 0x00454838, 0x00424535, 0x0041453F, 0x00262A1F, 0x007C8376, 0x00829084, 0x005F7E77,
        0x005F9197, 0x00367E90, 0x000C617B, 0x001E89AF, 0x002A9CCB, 0x00219CD6, 0x001795D5, 0x001C97D1, 0x0035A2CE, 0x00217D9A, 0x00005061, 0x0000343E, 0x0024352C, 0x00473D2C, 0x003C302E, 0x002F3132, 0x00343930, 0x00313726, 0x00384134, 0x0032362A,
        0x0035392D, 0x0034382C, 0x003B3F33, 0x00383C30, 0x003E4236, 0x003A3E32, 0x003B3F33, 0x003F4337, 0x003E4236, 0x003C4034, 0x003A3E32, 0x00383C30, 0x00373B2F, 0x00373B2F, 0x00363A2E, 0x00393B2F, 0x00383A2E, 0x00393B2F, 0x003C3E32, 0x003C3E32,
        0x00393B2F, 0x003A3C30, 0x003C3E32, 0x003B3F33, 0x003A3E32, 0x003A3E32, 0x003A3E32, 0x00393D31, 0x00393D31, 0x00383C30, 0x00383C30, 0x00383D2E, 0x00383D2E, 0x00373C2D, 0x00393C2D, 0x00393C2D, 0x003B3B2D, 0x003B3B2D, 0x003B3B2D, 0x00383E2D,
        0x00343A2F, 0x00383B32, 0x003B392E, 0x003C392A, 0x00283226, 0x000E2D2E, 0x000D3E4C, 0x00034355, 0x00074759, 0x0009495B, 0x0007465A, 0x0007465A, 0x0008475C, 0x0009485D, 0x0008475C, 0x00004354, 0x00044456, 0x00064456, 0x00024756, 0x0000404E,
        0x0058A0AC, 0x00366D7C, 0x0041697B, 0x007CC4CC, 0x0072B7C0, 0x000C4A54, 0x00629FA9, 0x00A1DEE8, 0x003B7983, 0x000D525B, 0x00559DA5, 0x0075A7BD, 0x00335E73, 0x00326978, 0x004995A1, 0x0000424D, 0x00104A5D, 0x00083C53, 0x000A465E, 0x0008425F,
        0x000F414D, 0x00056B8E, 0x002DACE9, 0x00055280, 0x000A465E, 0x00085B63, 0x0042897F, 0x0037393A, 0x00000100, 0x000A0E08, 0x001D2116, 0x00212519, 0x002F3125, 0x002F3026, 0x00202118, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFEFF, 0xFFFFFDFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00191C1A, 0x00181C16, 0x00030701, 0x000B0C08, 0x00171814, 0x000F1510, 0x00202B28, 0x00455652, 0x0040544F, 0x00356E70, 0x00437476, 0x003B6161, 0x00436564, 0x002A4D49,
        0x00042C2B, 0x00002426, 0x0000292E, 0x00072334, 0x00041F34, 0x00142B41, 0x000A202C, 0x00465F5B, 0x008BADA2, 0x009AC4BD, 0x0085B5B7, 0x008AAAA9, 0x0078938F, 0x0044675D, 0x00123E31, 0x00082319, 0x004F4E44, 0x00887F71, 0x00565943, 0x0028302F,
        0x00253C44, 0x00062A3C, 0x00001C2E, 0x0006232A, 0x00142423, 0x00171F1E, 0x00151A1D, 0x00092117, 0x00132417, 0x0014180C, 0x00201E16, 0x00131718, 0x00031123, 0x00001B3A, 0x000F3761, 0x0000496A, 0x00003E64, 0x00003E66, 0x00208CAE, 0x0063CDE5,
        0x0079BCCB, 0x00607F82, 0x00889995, 0x00728C8C, 0x00759398, 0x006D8186, 0x005B5B61, 0x0072AAB5, 0x00167686, 0x0057828B, 0x00A0B3A2, 0x006B9F88, 0x002D4B50, 0x00005277, 0x0033C7FD, 0x00086E99, 0x00193748, 0x0019363B, 0x00053946, 0x000B4A48,
        0x001D5A56, 0x0030635F, 0x004B6E6A, 0x005E7472, 0x00526769, 0x004E6E73, 0x00356266, 0x00426E7B, 0x001A5B69, 0x0000424E, 0x000C4057, 0x001D5D8D, 0x002A9ED9, 0x0031A5CE, 0x00356B7C, 0x00669C9C, 0x0043868F, 0x0046858D, 0x005F8480, 0x0083918D,
        0x009EA7AA, 0x00262F2C, 0x00363C29, 0x00373E31, 0x00394033, 0x003C4336, 0x00404536, 0x00404536, 0x00424535, 0x00424535, 0x00424535, 0x00414435, 0x00464B3C, 0x00434839, 0x00434839, 0x0044493A, 0x00434839, 0x00414637, 0x00424738, 0x00464B3C,
        0x00444738, 0x00454839, 0x00515243, 0x00525546, 0x004E5142, 0x00515445, 0x00505344, 0x00494C3F, 0x00454738, 0x004E4F3F, 0x00535444, 0x00505244, 0x00535444, 0x00535444, 0x00505242, 0x004A4C3D, 0x00494A3B, 0x004B4C3E, 0x004E5040, 0x004F5040,
        0x00515141, 0x00505142, 0x004F5243, 0x004E5042, 0x00515343, 0x004D4F40, 0x004B4E41, 0x004D4F3F, 0x004F5040, 0x00535444, 0x004E5042, 0x00505142, 0x004F5141, 0x00505242, 0x00525545, 0x00545647, 0x00555646, 0x00525343, 0x00515242, 0x00535444,
        0x00505141, 0x004F5040, 0x00515344, 0x004A4C3D, 0x004B4D3E, 0x00494E3E, 0x004B5041, 0x004D5142, 0x004B4E3F, 0x00474A3B, 0x00454939, 0x00464939, 0x00444737, 0x00424535, 0x00414434, 0x003E3F36, 0x002A2D24, 0x00798377, 0x00849689, 0x005C7A6F,
        0x00668E89, 0x006F9C9F, 0x00508288, 0x00065975, 0x002483AA, 0x002D9DD1, 0x001695D2, 0x000287C5, 0x001D9AD4, 0x002A99C7, 0x00157AA1, 0x00003744, 0x0020322B, 0x00483F31, 0x003C332F, 0x00262B2A, 0x00283025, 0x002E3423, 0x00353D32, 0x00313726,
        0x00343A29, 0x00353B2A, 0x003A402F, 0x00383E2D, 0x003B4130, 0x003A402F, 0x003C4231, 0x00414539, 0x00404438, 0x003F4337, 0x003D4135, 0x003C4034, 0x003B3F33, 0x003B3F33, 0x003B3F33, 0x003A3C30, 0x00393B2F, 0x003A3C30, 0x003E4034, 0x00404236,
        0x003F4135, 0x003E4034, 0x00404236, 0x003D4135, 0x003D4135, 0x003D4135, 0x003D4135, 0x003D4135, 0x003D4135, 0x003D4135, 0x003C4034, 0x003B4233, 0x003C4132, 0x003A3F30, 0x00383D2E, 0x00393C2D, 0x00393C2D, 0x003B3B2D, 0x003C3C2E, 0x00393C2C,
        0x00383C31, 0x00393933, 0x003B372C, 0x00403B2C, 0x002A3227, 0x000C292D, 0x00103D4B, 0x00044456, 0x00074759, 0x0009495B, 0x0007465A, 0x0007465A, 0x0009485D, 0x0009485D, 0x0008475C, 0x00034859, 0x000A4A5C, 0x000E4C5E, 0x00074C5B, 0x0000414F,
        0x00549CA8, 0x00326978, 0x00355D6F, 0x001C5965, 0x0068A5B1, 0x0090D0DB, 0x00276772, 0x0010505B, 0x002A6A75, 0x007CB9C5, 0x005D9AA6, 0x00194A60, 0x0030596F, 0x003D7181, 0x004B95A1, 0x0000414D, 0x000C475A, 0x0010445B, 0x00023E56, 0x0004405D,
        0x000C3E4A, 0x0002688B, 0x002BAAE7, 0x00055280, 0x000A4860, 0x000C5F67, 0x00478E84, 0x00393B3C, 0x00000100, 0x000C100A, 0x001D2116, 0x00202418, 0x002B2D21, 0x002E2F25, 0x0022231A, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFEFF, 0xFFFFFDFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x001B1E1C, 0x001C231C, 0x00030701, 0x00030400, 0x000D0E0A, 0x0010140F, 0x00515A57, 0x00CFDDD9, 0x00EFFFFC, 0x00ECFFF9, 0x00F8FFFB, 0x00FBFFF6, 0x00DDE7DB, 0x00698278,
        0x00123733, 0x00073235, 0x00123B44, 0x000F3539, 0x00103740, 0x000F2E3D, 0x00162A35, 0x00959A9B, 0x00FFFFFC, 0x00FFFEFF, 0x00FBF7FF, 0x00F0FFF9, 0x00ECFAEE, 0x00AFCAB6, 0x0037604B, 0x00163427, 0x0081837D, 0x00BBB1A7, 0x00878371, 0x003C5551,
        0x005C8287, 0x00164A5A, 0x00003443, 0x0006343C, 0x00001317, 0x0003151C, 0x00000B19, 0x00000A11, 0x00070E11, 0x000D0A06, 0x00160F0C, 0x0001080B, 0x00001F2F, 0x000E4969, 0x001F6E97, 0x001D8FBE, 0x0036A5D5, 0x0040BFEC, 0x003ECAF1, 0x00239BB9,
        0x00377B92, 0x003C5765, 0x00425056, 0x003A404D, 0x00124D5D, 0x00135567, 0x003A5968, 0x002A515F, 0x00003044, 0x003D616B, 0x006C9989, 0x006E877D, 0x00001B2E, 0x00035A8C, 0x0046C4FE, 0x00166D99, 0x00002F41, 0x00063A40, 0x000B454B, 0x000E414B,
        0x0000393F, 0x000D4B51, 0x0001323A, 0x00001A29, 0x00000619, 0x0000273D, 0x0024687F, 0x003E91AD, 0x00003752, 0x000A4753, 0x000B5059, 0x0000385B, 0x002680B7, 0x0034A5D0, 0x001C6F84, 0x00548276, 0x00225E5E, 0x002C6567, 0x00577972, 0x0084928C,
        0x009FA8AB, 0x00242C2B, 0x00363C29, 0x00394033, 0x003A4134, 0x003C4336, 0x00404536, 0x00404536, 0x00424535, 0x00424535, 0x00424535, 0x00424536, 0x0044493A, 0x00434839, 0x00424738, 0x00434839, 0x00414637, 0x003F4435, 0x00414637, 0x00454A3B,
        0x00434637, 0x00414435, 0x00505242, 0x004D5041, 0x00494C3D, 0x004E5142, 0x004E5041, 0x00494C3E, 0x0046493A, 0x004C4F40, 0x00515243, 0x004F5142, 0x00525343, 0x00505243, 0x00494C3D, 0x004D5041, 0x004D5040, 0x004F5040, 0x00535444, 0x00515243,
        0x00515141, 0x004F5040, 0x004E5041, 0x004E4F41, 0x00535444, 0x00525445, 0x00494C3F, 0x00454738, 0x004E4F3F, 0x00535444, 0x00505244, 0x00535444, 0x00535444, 0x00545545, 0x00545545, 0x00545545, 0x00545545, 0x00545545, 0x00505141, 0x004F5040,
        0x004D4D3D, 0x004E4F3F, 0x004E4F3F, 0x00454839, 0x00424536, 0x00444838, 0x00474A3B, 0x00484B3C, 0x00444738, 0x00424536, 0x00444738, 0x00454839, 0x00424536, 0x00424536, 0x00454839, 0x0048453D, 0x002A2D24, 0x00748277, 0x00859B8F, 0x00627E71,
        0x00668479, 0x00728F86, 0x0068837A, 0x00598D99, 0x003C8195, 0x002889AB, 0x00259CCD, 0x001295CE, 0x000E90CB, 0x001E95CD, 0x001F8DC1, 0x0011495C, 0x00132825, 0x003D3627, 0x002C261F, 0x0009100D, 0x00151E11, 0x00313726, 0x0030382E, 0x00353B2A,
        0x00373D2C, 0x00393F2E, 0x003B4130, 0x00383E2D, 0x00393F2E, 0x00393F2E, 0x003B4130, 0x003D4338, 0x003D4338, 0x003D4338, 0x003D4338, 0x003D4338, 0x003E4439, 0x003F453A, 0x003F453A, 0x003B3F34, 0x00393D32, 0x003A3E33, 0x003E4237, 0x0041453A,
        0x00404439, 0x003F4338, 0x003F4338, 0x003A4035, 0x003A4035, 0x003B4136, 0x003C4237, 0x003E4439, 0x003F453A, 0x0040463B, 0x0040463B, 0x0041483B, 0x0040473A, 0x003F4337, 0x003D4135, 0x003A3E32, 0x003A3C30, 0x00393B2F, 0x00383A2E, 0x00373729,
        0x003A3B31, 0x003A3732, 0x003C352C, 0x00453D30, 0x002D3229, 0x000C272B, 0x00103C4D, 0x00044456, 0x00074759, 0x0008485A, 0x0007465A, 0x0007465A, 0x0009485D, 0x0009485D, 0x0007465B, 0x00004556, 0x0009495B, 0x00044254, 0x00004251, 0x00004553,
        0x00579FAB, 0x00447B8A, 0x0061899B, 0x00588F9C, 0x000C4652, 0x004F8C98, 0x0090D0DB, 0x005C9CA7, 0x0077B4C0, 0x00447E8A, 0x000E4552, 0x00527E95, 0x00587F95, 0x003F7282, 0x004F97A3, 0x00004955, 0x00023D50, 0x000C4057, 0x0005435B, 0x00033F5C,
        0x00093D49, 0x0000678A, 0x002CABE8, 0x00075482, 0x000C4A62, 0x000B6068, 0x00458E84, 0x003A3C3D, 0x00010402, 0x000D110B, 0x001D2116, 0x001F2317, 0x0026281C, 0x002A2B21, 0x001F2017, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x001E211F, 0x00171F18, 0x00040B04, 0x000F100C, 0x00191A16, 0x000E120D, 0x00484F4C, 0x00D1DCD9, 0x00EBF9F5, 0x00FFFAF1, 0x00FFFFF8, 0x00FDEEE5, 0x00B6BAB4, 0x00456766,
        0x0004404C, 0x00003853, 0x00003F64, 0x0000405A, 0x00003F5F, 0x00003E61, 0x00103D58, 0x00818F9B, 0x00E9DEE0, 0x00FFF0F2, 0x00FFFAFF, 0x00F1F2E9, 0x00D3D3C3, 0x009DAD96, 0x00254D3B, 0x00082D29, 0x005C666D, 0x006E6869, 0x00504C41, 0x00314B4B,
        0x00547D86, 0x00306B7B, 0x005DA1AE, 0x0047878C, 0x00003234, 0x00164953, 0x0000283A, 0x0007263B, 0x00091E2D, 0x00161C21, 0x00242222, 0x00191E1F, 0x0019323C, 0x000A3C50, 0x00064964, 0x00266188, 0x00156185, 0x000E7897, 0x002A9EB5, 0x002D8095,
        0x00406371, 0x00283B40, 0x00112F2A, 0x00001B34, 0x00005F81, 0x001B7CA2, 0x002C6481, 0x000A354A, 0x000B2B42, 0x00517281, 0x006B948F, 0x007B8487, 0x00002948, 0x000C6CA1, 0x00267CB0, 0x002C5576, 0x00154049, 0x00225450, 0x003D4E4B, 0x003E555D,
        0x002F555A, 0x00235456, 0x00093338, 0x004C6A75, 0x00527285, 0x00457A8F, 0x0079C4DA, 0x003EAFC9, 0x0019587A, 0x0012495E, 0x0004454E, 0x00014A60, 0x001C8CB6, 0x0022A0C9, 0x00156786, 0x003C6F68, 0x0007464A, 0x00205B5D, 0x0060817A, 0x00929D95,
        0x00A0A7AA, 0x001F2726, 0x003C4433, 0x003B4235, 0x003B4235, 0x003C4336, 0x003E4334, 0x003F4435, 0x00424535, 0x00434636, 0x00434636, 0x0046483C, 0x00474B3F, 0x0045493D, 0x0043473B, 0x0042463A, 0x00414539, 0x00404438, 0x003E4236, 0x003C4034,
        0x00434637, 0x00414435, 0x00454738, 0x0047493A, 0x00474B3C, 0x00494D3E, 0x00484D3E, 0x00484C3D, 0x00484B3C, 0x004B4F41, 0x004D5041, 0x004D4F41, 0x004D4F40, 0x004E5141, 0x004C4F40, 0x004B4F41, 0x004D4F3F, 0x004F5040, 0x00535444, 0x004E5042,
        0x00505142, 0x00505141, 0x004B4E3E, 0x004C4E3F, 0x004C4F40, 0x00525445, 0x0046493C, 0x00454839, 0x004C4F40, 0x00515243, 0x00505142, 0x00535444, 0x00545546, 0x00525343, 0x00505242, 0x00505242, 0x00525344, 0x00565747, 0x00535444, 0x00515242,
        0x004F5040, 0x00525242, 0x00474737, 0x00454839, 0x00424536, 0x00414435, 0x00414435, 0x003E4132, 0x003D4031, 0x00414435, 0x0046493A, 0x0046493A, 0x00424536, 0x00434637, 0x00474A3B, 0x0049463E, 0x00292C23, 0x007A887D, 0x00889E92, 0x006A8679,
        0x006B897E, 0x00617E75, 0x00637E75, 0x0080A19A, 0x00306163, 0x00025061, 0x001A83A4, 0x00239CC8, 0x00108ABE, 0x00168CC0, 0x001E8EC3, 0x0018546C, 0x00132B2B, 0x003C392B, 0x00302E26, 0x00171E19, 0x0020291C, 0x00393F2E, 0x00333731, 0x00373D2C,
        0x00393F2E, 0x003B4130, 0x003A402F, 0x003A402F, 0x00383E2D, 0x003A402F, 0x003B4130, 0x003C4237, 0x003C4237, 0x003C4237, 0x003C4237, 0x003D4338, 0x003D4338, 0x003E4439, 0x003F453A, 0x003D4136, 0x003A3E33, 0x003A3E33, 0x003E4237, 0x00404439,
        0x003E4237, 0x003D4136, 0x003E4237, 0x00393F34, 0x00393F34, 0x003A4035, 0x003B4136, 0x003C4237, 0x003E4439, 0x003F453A, 0x003F453A, 0x0040463B, 0x003F453A, 0x003E4439, 0x003F4338, 0x003C4035, 0x003A3E33, 0x003A3B31, 0x0038392F, 0x00353829,
        0x00393A31, 0x00383832, 0x003D362D, 0x00453D30, 0x002D3229, 0x000B252B, 0x000D394A, 0x00054557, 0x00074759, 0x0008485A, 0x0007465A, 0x0007465A, 0x000A495E, 0x000A495E, 0x0007465B, 0x0004495A, 0x000D4D5F, 0x000B495B, 0x00024756, 0x00004553,
        0x005098A4, 0x0039707F, 0x00487082, 0x0083CDD9, 0x0073B2C0, 0x00194A5A, 0x003F6779, 0x0085ADBF, 0x002E5F6F, 0x000F4E5C, 0x005AA4B0, 0x0084AEC5, 0x0044697F, 0x003C6D7D, 0x004F97A3, 0x0000434F, 0x000F4A5D, 0x00093F56, 0x0007475F, 0x00064561,
        0x000D414D, 0x00036B8E, 0x002CAEEA, 0x00075583, 0x00084860, 0x00065B63, 0x003D867C, 0x002D2F30, 0x00000100, 0x000B0F09, 0x00202419, 0x0025291D, 0x00292B1F, 0x002B2C22, 0x001E1F16, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFDFF, 0xFFFFFEFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00202321, 0x001C241D, 0x00040B04, 0x000E120D, 0x001A1B17, 0x000F100C, 0x00424745, 0x00CAD1CE, 0x00D4DFDC, 0x00E3E9E8, 0x00EEEAE9, 0x00BDBBBB, 0x007A9193, 0x0056939F,
        0x0053AFCC, 0x003EA4D5, 0x003C9DDC, 0x003EA4E5, 0x0034A2E4, 0x0042AFED, 0x004499C6, 0x006695AA, 0x009DADAC, 0x00C7CAC1, 0x00F3F4EA, 0x00D1CCC3, 0x00D3CEBF, 0x00A9B9A7, 0x001E514A, 0x00003443, 0x0045647D, 0x00394956, 0x00202F2B, 0x0045545D,
        0x00234152, 0x00083B4F, 0x005899A8, 0x00428B8F, 0x002C797B, 0x007BCFD5, 0x00348F9E, 0x00002D3D, 0x00002934, 0x000C272B, 0x00242F2C, 0x002A2B27, 0x00292E2D, 0x00172128, 0x0012222E, 0x00331724, 0x00443E49, 0x0069898E, 0x0084B6B6, 0x008CA8A8,
        0x00D0D4CF, 0x00BACBBE, 0x006A9B85, 0x00004862, 0x002FAFDA, 0x00429CD7, 0x00003D66, 0x00003B4D, 0x00023341, 0x00648794, 0x00A9B3BA, 0x00718487, 0x002D7391, 0x002F96C3, 0x000C4263, 0x00322F3E, 0x00233332, 0x00243C34, 0x004E2E28, 0x003B3D37,
        0x003D463C, 0x00546459, 0x00738479, 0x00C9DBD4, 0x00C4DAD8, 0x00B8DBDE, 0x00BCEBEF, 0x00B3EDFF, 0x006DCBE8, 0x002793B2, 0x00074E62, 0x000C4358, 0x002087A7, 0x0030B1D8, 0x001D7FA9, 0x001B5F6C, 0x00045063, 0x002C6C77, 0x005B7D76, 0x008B9589,
        0x00A8AEAD, 0x00242C2B, 0x00364030, 0x003D4437, 0x003C4336, 0x003B4235, 0x003E4334, 0x003F4435, 0x00424535, 0x00434636, 0x00434636, 0x00484A3E, 0x00464A3E, 0x0044483C, 0x0042463A, 0x00414539, 0x00414539, 0x00414539, 0x00404438, 0x003F4337,
        0x0046493A, 0x00474A3B, 0x00474A3A, 0x00444737, 0x00464B3C, 0x00494D3E, 0x00494D3E, 0x00484B3B, 0x004A4B3B, 0x004A4C3F, 0x004D4F40, 0x004E4F41, 0x00525344, 0x00535444, 0x00515344, 0x00494C3F, 0x00454738, 0x004E4F3F, 0x00535444, 0x00515244,
        0x00525343, 0x004C4E3E, 0x00484A3B, 0x00484A3B, 0x004A4C3E, 0x004F5242, 0x004B4F41, 0x00484A3B, 0x00494C42, 0x004E5143, 0x00505142, 0x004F5040, 0x00525445, 0x00505243, 0x004F5243, 0x00525445, 0x00515343, 0x00555646, 0x00545444, 0x00515242,
        0x00505141, 0x00505040, 0x0048493B, 0x0046483C, 0x00424438, 0x0036382C, 0x002E3024, 0x00222418, 0x0027291D, 0x00303226, 0x003A3C30, 0x00414337, 0x00434539, 0x0044463A, 0x0045473B, 0x003F4037, 0x0025281F, 0x007D877B, 0x007B8D80, 0x005A786D,
        0x0058807B, 0x00336063, 0x0022545A, 0x006A8B7C, 0x00295751, 0x0000424B, 0x0017738C, 0x002F9AC0, 0x001B8BB9, 0x001D8EC0, 0x002492C6, 0x000E4C6A, 0x00153034, 0x00403F31, 0x003B3C32, 0x0037423A, 0x00353F2F, 0x00383D2E, 0x00323630, 0x00363A2E,
        0x0035392D, 0x00393D31, 0x00363A2E, 0x00393D31, 0x00373B2F, 0x003A3E32, 0x003A3E32, 0x00384035, 0x00373F34, 0x00363E33, 0x00353D32, 0x00353D32, 0x00353D32, 0x00353D32, 0x00353D32, 0x00383E33, 0x00353B30, 0x00343A2F, 0x00373D32, 0x00383E33,
        0x00363C31, 0x00363C31, 0x00383E33, 0x00343C31, 0x00343C31, 0x00343C31, 0x00343C31, 0x00343C31, 0x00343C31, 0x00343C31, 0x00343C31, 0x00333B31, 0x00343C32, 0x00373C33, 0x00383D34, 0x003A3D34, 0x00393C33, 0x00383B32, 0x00393A31, 0x0035392D,
        0x00343930, 0x00353632, 0x0038352D, 0x003F3B30, 0x00293228, 0x0009272C, 0x0008384A, 0x00054557, 0x00074759, 0x00074759, 0x00064559, 0x0008475B, 0x000A495E, 0x000A495E, 0x0006455A, 0x00014657, 0x00074759, 0x00104E60, 0x00094E5D, 0x00004452,
        0x00549CA8, 0x003C7382, 0x00234B5D, 0x001D5060, 0x0074A8B8, 0x009AD1E0, 0x003C7584, 0x000A4352, 0x00265D6C, 0x0089BDCD, 0x005C8F9F, 0x00294F67, 0x002C4C63, 0x003D6B7C, 0x005197A4, 0x00004450, 0x000F4A5D, 0x000D435A, 0x00014159, 0x00054460,
        0x000B414C, 0x00026A8D, 0x002CAEEA, 0x00085684, 0x000A4A62, 0x00055D64, 0x003B877C, 0x002B2D2E, 0x00000100, 0x00070B05, 0x001D2116, 0x0024281C, 0x00282A1E, 0x002D2E24, 0x00202118, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFDFF, 0xFFFFFEFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00222523, 0x001B261E, 0x00000600, 0x000B0F0A, 0x00191814, 0x00151410, 0x004C4F4D, 0x00CDD2D0, 0x00D2DBD8, 0x00B4C9C6, 0x00BCC3BE, 0x0091958F, 0x006D8781, 0x00438286,
        0x00237E93, 0x001975A0, 0x001F71AB, 0x002274A4, 0x000E6EA3, 0x000B74A7, 0x00136B90, 0x004C8392, 0x008CA4A2, 0x009CADA0, 0x00AEC0B3, 0x00BEC1B8, 0x00C9CDC1, 0x00A9C7BC, 0x003F848D, 0x000C6083, 0x000C4D79, 0x00002E45, 0x00144646, 0x00234454,
        0x0010394F, 0x0001344E, 0x000C4457, 0x00043B42, 0x00559296, 0x00B0F8FF, 0x0060AEBF, 0x001B4E51, 0x00154140, 0x00001813, 0x000C1D14, 0x001A1F16, 0x001C1911, 0x001B1510, 0x00211815, 0x000F1105, 0x005A594F, 0x00D9E2D8, 0x00E5F7F0, 0x00939D97,
        0x00C7C0BD, 0x00C6CCC7, 0x00719589, 0x0000495E, 0x0031BBE9, 0x00368DCF, 0x00002E57, 0x00014951, 0x000A3D3F, 0x0032595B, 0x00515354, 0x003C5148, 0x00306B7F, 0x00186E8C, 0x00001F28, 0x00281811, 0x001F2219, 0x0011231C, 0x00341D15, 0x00141B0E,
        0x00141408, 0x00575445, 0x00C8C9B9, 0x00DDE8D8, 0x00839588, 0x00B5C4BC, 0x00B2BDBA, 0x00D8CDCF, 0x0052B7D0, 0x0034C8F2, 0x002781A4, 0x00113D54, 0x002C6683, 0x003D9FC9, 0x001E9ECF, 0x00257DA5, 0x00207BA2, 0x00418AA0, 0x005A7E78, 0x008B9285,
        0x00B5B9B4, 0x002B3431, 0x00303A2D, 0x003D4437, 0x003C4336, 0x003C4336, 0x003F4435, 0x00414637, 0x00434636, 0x00424535, 0x00414434, 0x0044463A, 0x0044483C, 0x0043473B, 0x00404438, 0x003F4337, 0x00404438, 0x00414539, 0x00414539, 0x00414539,
        0x00484B3C, 0x004B4E3F, 0x00484B3B, 0x00474939, 0x00494D3F, 0x004C5142, 0x004C5041, 0x004B4D3E, 0x00505141, 0x00515345, 0x004D5141, 0x004F5243, 0x004D5041, 0x00515344, 0x004F5143, 0x0046493B, 0x00474A3B, 0x004C4F40, 0x004F5142, 0x004F5142,
        0x004D4F40, 0x00494C3D, 0x004F5243, 0x004B4E3F, 0x004D4E3F, 0x00525343, 0x004D4F41, 0x00505242, 0x004D4F43, 0x004D4F3F, 0x004D4D3D, 0x004E4F3F, 0x004E4E3F, 0x004F5242, 0x004F5243, 0x00515445, 0x004D4F40, 0x00525343, 0x00555646, 0x00545545,
        0x00505141, 0x0047483A, 0x00424337, 0x00424438, 0x00383A2E, 0x0024261A, 0x001A1C10, 0x00131509, 0x0015170B, 0x001A1C10, 0x0027291D, 0x00383A2E, 0x0044463A, 0x0045473B, 0x00414337, 0x0041453F, 0x00262A1F, 0x007E8578, 0x00818F83, 0x0063827B,
        0x0062949A, 0x00287082, 0x00004B65, 0x003B6C64, 0x00175051, 0x00003D4B, 0x00106782, 0x00389AC2, 0x002591C1, 0x00208EC2, 0x001E8EC3, 0x000C4B6D, 0x000F2C31, 0x00353529, 0x002B2F24, 0x002E3C31, 0x002B3725, 0x00313627, 0x0030342F, 0x0034372E,
        0x0031342B, 0x0034372E, 0x0030332A, 0x0035382F, 0x0031342B, 0x0035382F, 0x0033362D, 0x0031392F, 0x0030382E, 0x002F372D, 0x002E362C, 0x002D352B, 0x002D352B, 0x002D352B, 0x002D352B, 0x0031362D, 0x002F342B, 0x002E332A, 0x002F342B, 0x002F342B,
        0x002E332A, 0x002F342B, 0x0033382F, 0x002C342A, 0x002C342A, 0x002C342A, 0x002B3329, 0x002B3329, 0x002B3329, 0x002A3228, 0x002A3228, 0x00293029, 0x002A312A, 0x002B322B, 0x002F332D, 0x0030342E, 0x0033352F, 0x00343630, 0x00343630, 0x00313A2D,
        0x002B332C, 0x002C322D, 0x0033342B, 0x0037372B, 0x0026332B, 0x000C2F33, 0x000B3F50, 0x00054557, 0x00074759, 0x00074759, 0x00064559, 0x0008475B, 0x000B4A5F, 0x000A495E, 0x0006455A, 0x00084D5E, 0x00074759, 0x000C4A5C, 0x00004554, 0x00003E4C,
        0x00549CA8, 0x00467D8C, 0x002A5264, 0x004A586E, 0x00294D5F, 0x00337784, 0x0081DAE4, 0x002E8791, 0x0083C7D4, 0x004A6E80, 0x0037455B, 0x00284C64, 0x00436279, 0x003C6879, 0x00549AA7, 0x00034D59, 0x00003B4E, 0x00063F55, 0x0001445B, 0x0003425E,
        0x00093F4A, 0x0000688A, 0x0029ADE9, 0x00065784, 0x000A4D64, 0x00096168, 0x00408C81, 0x00313334, 0x00000301, 0x00050903, 0x001A1E13, 0x0022261A, 0x0026281C, 0x002D2E24, 0x0022231A, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFDFF, 0xFFFFFEFF, 0xFFFCFCFC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00232624, 0x001B261E, 0x00000600, 0x000F130E, 0x001C1B17, 0x00191814, 0x004F504E, 0x00C7CCCA, 0x00CBD2CF, 0x00A8BEAC, 0x009DA18E, 0x00928A73, 0x00A4AC95, 0x005D8474,
        0x0004403F, 0x0000354A, 0x000C3359, 0x00123838, 0x000A444A, 0x00003846, 0x00003745, 0x006C8C91, 0x00C8CDCB, 0x00B4B3AF, 0x00A4A9A8, 0x00A5B4AC, 0x00BACABF, 0x009CC8C2, 0x0063BACE, 0x0056C0EF, 0x003A96CB, 0x00004D6A, 0x00004443, 0x00003F51,
        0x0000435A, 0x000E4B65, 0x0004364A, 0x001E414B, 0x008AA6AD, 0x00D7F6FF, 0x00CCF0FF, 0x00DDF0E7, 0x009DB0A7, 0x00223328, 0x000B1C0F, 0x001A2719, 0x00192013, 0x001E2014, 0x001A190F, 0x000D2618, 0x001A1C16, 0x0072706F, 0x008B9D9E, 0x0054797D,
        0x008CBAC1, 0x008FCDD3, 0x004CA0A5, 0x00004255, 0x0031BDE6, 0x002689C7, 0x00004064, 0x00074545, 0x0065847D, 0x0089B1A6, 0x007F9087, 0x009BA08B, 0x006C8893, 0x00104A5D, 0x00002319, 0x00272007, 0x0022291A, 0x0004261F, 0x000F2219, 0x00011A16,
        0x00101A14, 0x00302C27, 0x00A0A198, 0x00B1C1B6, 0x00576A61, 0x00A9B0AD, 0x0084797C, 0x00A99A8A, 0x00095670, 0x0040B4EF, 0x003896C4, 0x00003F51, 0x00114056, 0x001B5987, 0x001E9DD4, 0x002A91CA, 0x001278AD, 0x002A7A97, 0x00517672, 0x008E9384,
        0x00B1B3AD, 0x00232C29, 0x00364236, 0x003C4336, 0x003C4336, 0x003C4336, 0x00404536, 0x00424738, 0x00444737, 0x00424535, 0x003F4232, 0x00414238, 0x0046483C, 0x0043473B, 0x00404438, 0x003E4236, 0x003F4337, 0x00404438, 0x00414539, 0x00414539,
        0x00484B3C, 0x004B4E3F, 0x00434636, 0x00454736, 0x00484C3E, 0x004B5041, 0x004A4F40, 0x00494C3D, 0x00525344, 0x00525546, 0x004E5142, 0x00515445, 0x004E5142, 0x004E5142, 0x00505344, 0x00484B3C, 0x0045483A, 0x00494C3F, 0x004D4E40, 0x004D4E3E,
        0x004F5142, 0x004E5041, 0x004E5243, 0x004F5243, 0x004D4F3F, 0x00525344, 0x00525345, 0x004B4E3E, 0x004D4F41, 0x00515242, 0x004F5040, 0x00525242, 0x00484838, 0x004D4F3F, 0x004E5142, 0x00505344, 0x004C4E3F, 0x004D4F40, 0x00515242, 0x00515242,
        0x004E5040, 0x0047483B, 0x00404236, 0x003E4034, 0x002F3125, 0x001B1D11, 0x0014160A, 0x001E2014, 0x00191B0F, 0x0015170B, 0x001D1F13, 0x00323428, 0x0045473B, 0x0046483C, 0x003E4034, 0x003A423B, 0x00272B20, 0x00858879, 0x00889285, 0x00567671,
        0x00538F9B, 0x00358FAD, 0x000E7AA3, 0x00155559, 0x00084D56, 0x00004157, 0x0011678B, 0x003999C7, 0x001D87BC, 0x001788C0, 0x002093CC, 0x00145477, 0x000B282F, 0x0035372B, 0x002E3226, 0x00263428, 0x0025311F, 0x0034392A, 0x002B2F2A, 0x00343630,
        0x0030322C, 0x0032342E, 0x002B2D27, 0x0031332D, 0x002B2D27, 0x002E302A, 0x002A2C26, 0x00293127, 0x00293127, 0x00283026, 0x00283026, 0x00283026, 0x00283026, 0x00293127, 0x00293127, 0x002B3027, 0x00292E25, 0x00292E25, 0x002A2F26, 0x00292E25,
        0x00282D24, 0x002B3027, 0x0030352C, 0x00242C22, 0x00242C22, 0x00242C22, 0x00252D23, 0x00252D23, 0x00252D23, 0x00262E24, 0x00262E24, 0x00252C25, 0x00252C25, 0x00252C25, 0x00272B25, 0x00282C26, 0x002A2E28, 0x002E302A, 0x002F312B, 0x002B382A,
        0x00212E26, 0x00242E28, 0x002B3027, 0x00303428, 0x0026352D, 0x0011373C, 0x00114758, 0x00064658, 0x00074759, 0x00074759, 0x00064559, 0x0008475B, 0x000B4A5F, 0x000A495E, 0x0006455A, 0x00004556, 0x0008485A, 0x000A485A, 0x00034857, 0x00004957,
        0x005199A5, 0x003D7483, 0x00335B6D, 0x003A556A, 0x002D5568, 0x00185363, 0x00468F9D, 0x0071BAC8, 0x004E8999, 0x00224A5D, 0x00425D72, 0x00355971, 0x00315067, 0x00467182, 0x004D919E, 0x00003A46, 0x00124D60, 0x000F485E, 0x0000435A, 0x00054662,
        0x000B414C, 0x0000698B, 0x0028ACE8, 0x00055683, 0x00084B62, 0x00086067, 0x003F8B80, 0x002D2F30, 0x00000200, 0x00060A04, 0x001E2217, 0x00282C20, 0x002A2C20, 0x002E2F25, 0x00202118, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFDFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00222725, 0x00152623, 0x00000805, 0x000F0F09, 0x001E1813, 0x001D1A15, 0x00505A54, 0x0065807C, 0x00587F7D, 0x0068837F, 0x00547070, 0x004D6C6F, 0x005D8189, 0x004B7281,
        0x00194459, 0x000B3853, 0x00154460, 0x00013C56, 0x00074059, 0x0004394E, 0x00174A5A, 0x00487681, 0x005D878C, 0x00598082, 0x00608484, 0x005F8383, 0x00688B87, 0x005C8483, 0x00265F6E, 0x00045479, 0x0049ABE1, 0x003AA1D8, 0x00004074, 0x00064252,
        0x0011405C, 0x000B3A5A, 0x00003951, 0x00083D47, 0x008EA8A8, 0x00D5DADD, 0x00A7ADB2, 0x00DBE5D9, 0x00B7C1B5, 0x00273026, 0x00121B11, 0x00232B24, 0x001D251E, 0x001E2520, 0x001E2520, 0x00282019, 0x000B1614, 0x00002025, 0x00498994, 0x0051A0AB,
        0x005FB0B8, 0x0090DBDD, 0x004F9491, 0x00004A63, 0x0043B4EC, 0x001B95C9, 0x00003B5C, 0x00283E57, 0x00CCDACF, 0x00ECFBE0, 0x00F3EEEF, 0x00CACFD0, 0x00E1E8E5, 0x00979F98, 0x002A3428, 0x00182216, 0x00212A20, 0x001B231C, 0x00181F1A, 0x00132819,
        0x0016231B, 0x00111716, 0x0017252B, 0x000C2E34, 0x00124248, 0x0036686E, 0x001C4C52, 0x00073945, 0x0000496A, 0x0052B8EC, 0x002A8FBC, 0x0000455B, 0x00125252, 0x0007544D, 0x00016260, 0x0020708D, 0x002B869B, 0x001A6A75, 0x004E767B, 0x00889494,
        0x00B4BFB7, 0x00232D21, 0x00383C30, 0x003C4035, 0x003F4338, 0x003E4237, 0x00414337, 0x0045473B, 0x00464638, 0x00434335, 0x00474739, 0x00414435, 0x00444739, 0x0043473B, 0x003F4337, 0x003D4135, 0x003D4135, 0x003E4236, 0x003F4337, 0x003E4236,
        0x00444738, 0x00454839, 0x00434636, 0x00474838, 0x0045473A, 0x004A4F40, 0x00484E3F, 0x0046493A, 0x00555646, 0x004D5041, 0x00494C3D, 0x004E5142, 0x004C4F40, 0x004E5142, 0x00515445, 0x004B4E3F, 0x00494C3E, 0x004C4E3E, 0x004D4D3D, 0x004E4F3F,
        0x004E4E3F, 0x004F5242, 0x004F5243, 0x004E5243, 0x004C4E3F, 0x00525344, 0x004E5040, 0x00474A3B, 0x004D4F41, 0x00515242, 0x00505141, 0x00525242, 0x00515141, 0x004D4E3E, 0x004F5141, 0x00515445, 0x004E5142, 0x004D5041, 0x00505243, 0x00515242,
        0x004E5040, 0x0048493D, 0x00424339, 0x003D3E34, 0x0028291F, 0x001A1B11, 0x001D1E14, 0x00161413, 0x001B1918, 0x001B1B15, 0x0015160D, 0x002A2C20, 0x00444738, 0x00434636, 0x00424633, 0x00444335, 0x001D2B1F, 0x008C867F, 0x00849286, 0x005C7A6F,
        0x00669097, 0x001C809C, 0x002190C2, 0x002880A4, 0x00004F6B, 0x00003B50, 0x000E6881, 0x00389FC6, 0x00178BBA, 0x001A90BF, 0x001A90BA, 0x00036F98, 0x00004F64, 0x00557B87, 0x003A424F, 0x006E7376, 0x00344639, 0x00617B6E, 0x00233E42, 0x0060776F,
        0x0023453B, 0x00567D75, 0x00264B43, 0x00506C66, 0x00344B47, 0x004A6A65, 0x00254F48, 0x00426464, 0x00355652, 0x003E5955, 0x004F675F, 0x003B524A, 0x004A5F5C, 0x00374F4D, 0x00536A6C, 0x00224443, 0x0057766D, 0x00234136, 0x00547A74, 0x00215153,
        0x00477A7D, 0x001D4C49, 0x0050786C, 0x0028504E, 0x00557574, 0x00275955, 0x00536F6F, 0x00265955, 0x004F6E6D, 0x00305D5A, 0x00395E5C, 0x003C6361, 0x00335A58, 0x00436A68, 0x002B5250, 0x00456C6A, 0x002A514F, 0x004D7472, 0x002B5250, 0x004C7179,
        0x001F4948, 0x005D7B70, 0x002A382D, 0x00587070, 0x001F5F71, 0x00439EC5, 0x0004659D, 0x00004556, 0x00004455, 0x00044658, 0x00044357, 0x0007465A, 0x000D495F, 0x00094359, 0x000B455B, 0x00004155, 0x000A465C, 0x000B4459, 0x00064456, 0x00004354,
        0x005299A7, 0x00316C7B, 0x00386677, 0x00246F77, 0x004B7D89, 0x002E5968, 0x004F93A0, 0x005BAFBB, 0x004C8EA0, 0x0031566C, 0x003F5670, 0x003B5873, 0x0038536D, 0x00457083, 0x004D97A3, 0x00004853, 0x00044554, 0x000D465B, 0x0005465B, 0x000A405E,
        0x00084650, 0x00046A8D, 0x0032ABE9, 0x00005985, 0x00094D64, 0x000E5D66, 0x002B7F73, 0x001E2321, 0x00000100, 0x000A0E09, 0x001D1F19, 0x0023261D, 0x0028291F, 0x002E2E22, 0x001D1D11, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFDFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00202523, 0x00172523, 0x00030A07, 0x0010100A, 0x001E1A15, 0x00191B15, 0x00384945, 0x002D504C, 0x000D3C39, 0x00163F42, 0x00113B40, 0x0009373E, 0x0010404C, 0x000D4050,
        0x0002374C, 0x00073D54, 0x000B435C, 0x00063F58, 0x000F485E, 0x000A4156, 0x00083C4D, 0x000F404E, 0x000C3A45, 0x0009343D, 0x00113D44, 0x000F3940, 0x0010363A, 0x00143F42, 0x00033C4B, 0x00003759, 0x000F6795, 0x0051AEDB, 0x0009638C, 0x00003A4A,
        0x000F3E5A, 0x00134262, 0x0009455D, 0x000B404A, 0x007B9595, 0x00A3A8AB, 0x00666C71, 0x00D1DBCF, 0x00ACB6AA, 0x001F281E, 0x0010190F, 0x00222A23, 0x001C241D, 0x00202722, 0x00222924, 0x00131E16, 0x00132825, 0x00001E1D, 0x00124D4F, 0x00165D61,
        0x000B5756, 0x0017615D, 0x000D534C, 0x00004B63, 0x003CB2E5, 0x001597C6, 0x00003D5B, 0x00204259, 0x00CBE3D9, 0x00DDF4D8, 0x00BFC7C6, 0x00737879, 0x00CDD4D1, 0x00B8C0B9, 0x00353F33, 0x00111B0F, 0x001A2319, 0x00222A23, 0x00262D28, 0x00162724,
        0x0003181A, 0x0004252E, 0x00033D49, 0x00146978, 0x000A6B79, 0x00015D6E, 0x00005365, 0x00104D4F, 0x00005066, 0x0056BFE8, 0x0058B5DC, 0x004B869A, 0x006B8E92, 0x0065888B, 0x0069979E, 0x000C546C, 0x001E7383, 0x002C757D, 0x005F8587, 0x008B9694,
        0x009FAAA2, 0x00313C32, 0x002F352A, 0x003C4035, 0x0042463B, 0x0041453A, 0x00404236, 0x00414337, 0x00434335, 0x00424234, 0x00464638, 0x00434637, 0x00434638, 0x00424639, 0x003E4236, 0x003C4034, 0x003C4034, 0x003D4135, 0x003D4135, 0x003C4034,
        0x003F4233, 0x00414435, 0x00424435, 0x00484938, 0x0047493C, 0x00464A3C, 0x00484E3F, 0x00414334, 0x00474939, 0x0046493A, 0x0045493A, 0x00484C3D, 0x00484D3E, 0x00494E3F, 0x004C5042, 0x00494C3D, 0x004C4E40, 0x00515242, 0x004F5040, 0x00525242,
        0x00484838, 0x004D4F3F, 0x004E5142, 0x004F5243, 0x00494C3D, 0x004E5142, 0x004B4D3E, 0x0046493A, 0x004D5042, 0x00545545, 0x00505141, 0x004C4C3C, 0x00505040, 0x00515141, 0x004E5040, 0x00515445, 0x004E5142, 0x004E5142, 0x00505344, 0x004F5042,
        0x004E5142, 0x0047493C, 0x00414238, 0x003E3F35, 0x002C2D23, 0x00191A10, 0x0013140A, 0x001B1918, 0x00171612, 0x00161610, 0x001A1B12, 0x00313327, 0x00454839, 0x00414434, 0x00414532, 0x00414032, 0x001D2B1F, 0x0086837B, 0x0089998E, 0x0067847B,
        0x0068949B, 0x001C809C, 0x001180B2, 0x003291B8, 0x001B7594, 0x00025870, 0x000B6784, 0x003099C2, 0x001386B8, 0x00168BBE, 0x002093C0, 0x001879A5, 0x001D6D86, 0x00528899, 0x002B5366, 0x0049777F, 0x002A665B, 0x004E8E82, 0x00175254, 0x004D6C6B,
        0x00315957, 0x003E706C, 0x00376362, 0x00688C8C, 0x00335555, 0x003B6264, 0x00386A6A, 0x002C6567, 0x0053898A, 0x00386967, 0x00396663, 0x002B5855, 0x004A7B79, 0x00265C5D, 0x004E8789, 0x0029575E, 0x0040696B, 0x002A5050, 0x004B777E, 0x00497E92,
        0x00578FAA, 0x00467B90, 0x006998A6, 0x00418598, 0x005C97AB, 0x00398395, 0x005A91A6, 0x003B8799, 0x00588FA4, 0x003E8698, 0x0048879B, 0x004D8FA2, 0x004C8EA1, 0x005395A8, 0x004C8EA1, 0x004F91A4, 0x00428497, 0x005193A6, 0x003C7E91, 0x004D8BA9,
        0x003E8295, 0x00528B94, 0x002D5053, 0x00597F81, 0x000B4953, 0x00216D84, 0x005AA2C4, 0x000C4C5E, 0x00044456, 0x00064658, 0x00044357, 0x00044357, 0x0008475C, 0x00054459, 0x0007465B, 0x00054A5E, 0x000C485E, 0x00084156, 0x00044254, 0x00004152,
        0x00539AA8, 0x00417C8B, 0x00578596, 0x006BB6BE, 0x0086B8C4, 0x003F6A79, 0x00488C99, 0x005FB3BF, 0x00488A9C, 0x0022475D, 0x00445B75, 0x003A5772, 0x0038536D, 0x00446F82, 0x004C96A2, 0x00004853, 0x00044554, 0x000D465B, 0x0005465B, 0x00083E5C,
        0x0007454F, 0x0002688B, 0x0032ABE9, 0x00005985, 0x000A4E65, 0x000F5E67, 0x002D8175, 0x00232826, 0x00000100, 0x000A0E09, 0x001C1E18, 0x0022251C, 0x0028291F, 0x002F2F23, 0x00202014, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFEFDFF, 0xFFFFFEFF, 0xFFFDFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x001C211F, 0x001B2623, 0x00060C07, 0x000D0A05, 0x0017140F, 0x00141B16, 0x00334E4A, 0x00204F4C, 0x0003403C, 0x0009434F, 0x000B4655, 0x00074251, 0x00054050, 0x00054053,
        0x00084357, 0x0009455B, 0x00044056, 0x00073D54, 0x000D435A, 0x000C4358, 0x000C4156, 0x000D4256, 0x000D4054, 0x000D4152, 0x00134758, 0x00174759, 0x0013434F, 0x00144450, 0x000A4253, 0x0000415A, 0x00075272, 0x0078C4E1, 0x001B6680, 0x00003D4D,
        0x000D3D59, 0x000D3E5E, 0x00024058, 0x00073E47, 0x00799393, 0x00989FA2, 0x0061676C, 0x00D2DCD0, 0x00AAB4A8, 0x0020291F, 0x00131C12, 0x00242C25, 0x001A221B, 0x001E2520, 0x00232A25, 0x000E1D19, 0x001A2E29, 0x002C4541, 0x00557672, 0x0082ABA6,
        0x0092C2BC, 0x00699C94, 0x004F847B, 0x002A8C9E, 0x0046BCEB, 0x0028A9D0, 0x001D7D95, 0x004C7B90, 0x00D0EFE6, 0x00CAE9CE, 0x00A8BEB9, 0x00606566, 0x00C9D0CD, 0x00BCC4BD, 0x00273125, 0x00121C10, 0x0020291F, 0x001F2720, 0x00151C17, 0x002A2526,
        0x00181D20, 0x00324C53, 0x005B929B, 0x009AE9F4, 0x0098E7F4, 0x00A0E1F0, 0x00AAD8EA, 0x0085AFA8, 0x001F646E, 0x002A839E, 0x00287895, 0x002C5D6D, 0x00455B61, 0x0045555C, 0x00576A77, 0x006EACBC, 0x002E7C83, 0x00529596, 0x006A8A89, 0x00929996,
        0x00A6AEA7, 0x0027342C, 0x00363D36, 0x003C4035, 0x0041453A, 0x003F4338, 0x003F4135, 0x0045473B, 0x004A4A3C, 0x00464638, 0x00434335, 0x00444738, 0x00434637, 0x00414437, 0x003C4034, 0x003C4034, 0x003C4034, 0x003D4135, 0x003C4034, 0x003C4034,
        0x00404334, 0x00424536, 0x00404334, 0x00444434, 0x00434535, 0x00434638, 0x00434638, 0x00434637, 0x00434637, 0x00414435, 0x003E4132, 0x00434536, 0x00484B3D, 0x00484C3E, 0x00484B3C, 0x0046493A, 0x004D4F41, 0x00515242, 0x00505141, 0x00525242,
        0x00515141, 0x004D4E3E, 0x004F5141, 0x00515445, 0x00494C3D, 0x004C4F40, 0x004A4D3E, 0x00454839, 0x0046483B, 0x00515242, 0x004A4C3C, 0x004D4E3F, 0x00515243, 0x00515141, 0x004D4D3E, 0x00505243, 0x004E5041, 0x00505243, 0x00545545, 0x004B4B3B,
        0x004C4E3F, 0x0044463A, 0x003F4035, 0x00414238, 0x0036372D, 0x00212218, 0x0014150B, 0x0015150F, 0x0010100A, 0x0016170E, 0x0025261C, 0x00393B2F, 0x00424536, 0x003F4232, 0x00444737, 0x004B4A3C, 0x00203124, 0x007E7B73, 0x00809085, 0x005E7B72,
        0x00618D94, 0x001E84A1, 0x001283B5, 0x001D88B4, 0x002A8EB1, 0x00187693, 0x000D6E90, 0x00309BC7, 0x00188ABF, 0x00198BC0, 0x002693C6, 0x002983B2, 0x00126886, 0x002F7B92, 0x000C5571, 0x00003F52, 0x00095D59, 0x00468E88, 0x0011484F, 0x000E5152,
        0x00004446, 0x00004E4F, 0x00257778, 0x0030797D, 0x0002464B, 0x00055157, 0x00004347, 0x0000404A, 0x003B7C84, 0x00286268, 0x001F5457, 0x000C4547, 0x00044449, 0x00003B41, 0x0031828A, 0x00276271, 0x00144F58, 0x000E4C54, 0x00004658, 0x001F799B,
        0x001179A4, 0x00117DA7, 0x00127BA2, 0x00157BA6, 0x001E79A6, 0x000D79A3, 0x002076A4, 0x000D7BA5, 0x0019719F, 0x000D76A1, 0x001C7AA7, 0x001275A1, 0x001477A3, 0x001376A2, 0x001D80AC, 0x000E719D, 0x001679A5, 0x001578A4, 0x001174A0, 0x001875AD,
        0x001277A4, 0x0015708F, 0x00034558, 0x00144D56, 0x00003C40, 0x001C565B, 0x007AA5AE, 0x001A5467, 0x00084356, 0x00094457, 0x00074458, 0x00004255, 0x00014459, 0x00014459, 0x0001465A, 0x00004458, 0x00074359, 0x00094257, 0x00094759, 0x00004152,
        0x00478E9C, 0x002F6A79, 0x00487687, 0x004B969E, 0x0051838F, 0x00295463, 0x005094A1, 0x005EB2BE, 0x005193A5, 0x0032576D, 0x0039506A, 0x003A5772, 0x0037526C, 0x00436E81, 0x004B95A1, 0x00004752, 0x00044554, 0x000C455A, 0x0004455A, 0x00073D5B,
        0x0005434D, 0x0001678A, 0x0032ABE9, 0x00015A86, 0x000A4E65, 0x00105F68, 0x002E8276, 0x00282D2B, 0x00000301, 0x000A0E09, 0x001B1D17, 0x0021241B, 0x0027281E, 0x002F2F23, 0x00212115, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFEFDFF, 0xFFFFFEFF, 0xFFFDFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00181D1B, 0x00131A17, 0x00090A06, 0x00110E09, 0x00171711, 0x00101E18, 0x002F524E, 0x0017524E, 0x00014C4A, 0x00003E56, 0x00003E56, 0x00003E56, 0x00064258, 0x000A445A,
        0x000A4359, 0x00084157, 0x00063F55, 0x000E4358, 0x00083F54, 0x00073E53, 0x00063F55, 0x00074059, 0x0008405B, 0x00043F59, 0x00013C56, 0x00073F58, 0x000B4158, 0x000B4257, 0x00094356, 0x00104B5F, 0x001E5C6E, 0x0078B6C6, 0x0023606E, 0x00003E4E,
        0x000B3B57, 0x000A3B5B, 0x00013F57, 0x000A414A, 0x007D9999, 0x00969DA0, 0x00646D71, 0x00D4DED2, 0x00A9B3A7, 0x0020291F, 0x00161F15, 0x00232B24, 0x00171F18, 0x001C231E, 0x00202722, 0x00171C1B, 0x00585B59, 0x00D7D8D4, 0x00E9EBE5, 0x00BDC4BD,
        0x00DCE9E1, 0x00DDF2EA, 0x00DBF5EF, 0x0078D5E4, 0x0069D2FB, 0x0060D1F2, 0x007FDBEE, 0x0099C8DD, 0x00E2FDF9, 0x00BFD9C1, 0x00A1B7B2, 0x006D7273, 0x00C8CFCC, 0x00B6BEB7, 0x001E281C, 0x00182216, 0x0020291F, 0x00212922, 0x00202722, 0x0022261A,
        0x00202419, 0x004D554E, 0x00A4BAB5, 0x00CBEAE3, 0x0098B0A8, 0x00DCE0DA, 0x00EDDDD7, 0x00DFE4DB, 0x003D6064, 0x00003F51, 0x00003F51, 0x00073E4B, 0x00123A3F, 0x000D343C, 0x001B4855, 0x00558B96, 0x0003484B, 0x003B7776, 0x005D7874, 0x00909691,
        0x00B3BBB4, 0x0018251D, 0x00373F38, 0x003F4338, 0x003A3E33, 0x00292D22, 0x0024261A, 0x00323428, 0x00454537, 0x00474739, 0x00434335, 0x00444738, 0x00424536, 0x00414436, 0x003D4134, 0x003B3F33, 0x003C4034, 0x003D4135, 0x003D4135, 0x003C4034,
        0x003E4132, 0x00434637, 0x00424536, 0x00474737, 0x00454636, 0x00464839, 0x00424536, 0x003D412F, 0x00474939, 0x003E4030, 0x00464838, 0x004A4C3C, 0x004C4E3F, 0x004F5041, 0x004C4E3F, 0x0046493A, 0x004D5042, 0x00545545, 0x00515242, 0x004D4D3D,
        0x004F5040, 0x004F5040, 0x004E5040, 0x00515445, 0x004A4D3E, 0x004D4F41, 0x00515343, 0x004D4F40, 0x004B4E41, 0x00494B3C, 0x00434435, 0x0047493A, 0x004D4F41, 0x00505142, 0x004F5141, 0x00505242, 0x00515444, 0x004F5243, 0x00505142, 0x00515141,
        0x004C4D3D, 0x00434538, 0x003F4135, 0x0044463A, 0x00424438, 0x0035372B, 0x0027291D, 0x001A1B12, 0x001D1E15, 0x00292A20, 0x00383A2E, 0x00414337, 0x00404334, 0x00404334, 0x00474A3A, 0x003E3E32, 0x001A2A1F, 0x0087837E, 0x008B9B90, 0x00628178,
        0x00638E97, 0x001A809D, 0x000D7DB1, 0x000578AA, 0x00238FB8, 0x001F83A5, 0x0010759B, 0x002C9ACA, 0x001B8CC5, 0x001D8BC5, 0x002993C8, 0x00116E9B, 0x00197493, 0x0026819C, 0x001E769A, 0x00186C88, 0x003A828A, 0x00658F94, 0x00637384, 0x006797A3,
        0x0046818B, 0x00377881, 0x00609DA9, 0x00568B98, 0x00528393, 0x006099A8, 0x00347885, 0x00387988, 0x00548F9E, 0x00669AA6, 0x0061929C, 0x006C9FA9, 0x003B7983, 0x002D7985, 0x003D919D, 0x008296A8, 0x006D808D, 0x0065818C, 0x001C4E64, 0x0024739A,
        0x000D72A6, 0x001788C0, 0x000D82B5, 0x00168BC2, 0x001784BC, 0x001591C7, 0x001B82BB, 0x000F8DC3, 0x00147DB6, 0x00118BC1, 0x002091C9, 0x001488BF, 0x000F83BA, 0x000A7EB5, 0x001387BE, 0x00087CB3, 0x00188CC3, 0x001589C0, 0x001A8EC5, 0x001882C1,
        0x001489C0, 0x001C8CB6, 0x00176F8D, 0x00004153, 0x00074B50, 0x00124646, 0x00496867, 0x0021566A, 0x000A4255, 0x000A4457, 0x000A475B, 0x00004154, 0x00004357, 0x00004559, 0x00004559, 0x00004559, 0x00074359, 0x00084156, 0x00074557, 0x00004051,
        0x0049909E, 0x00326D7C, 0x004B798A, 0x0057A2AA, 0x00689AA6, 0x002D5867, 0x00428693, 0x005CB0BC, 0x004E90A2, 0x00294E64, 0x00455C76, 0x003A5772, 0x0036516B, 0x00426D80, 0x004A94A0, 0x00004752, 0x00044554, 0x000B4459, 0x00024358, 0x00083E5C,
        0x0005434D, 0x0001678A, 0x0032ABE9, 0x00015A86, 0x00084C63, 0x000E5D66, 0x002E8276, 0x002A2F2D, 0x00000402, 0x000A0E09, 0x001B1D17, 0x0021241B, 0x0025261C, 0x002D2D21, 0x00202014, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFEFDFF, 0xFFFFFEFF, 0xFFFDFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00161B19, 0x00111510, 0x000C0B07, 0x0016130E, 0x00191B15, 0x000C1D19, 0x002A5250, 0x000A4A4A, 0x00004849, 0x000D4562, 0x00063E5B, 0x00063C5A, 0x000A3F5A, 0x000D405A,
        0x000E3F59, 0x000F3F57, 0x0015435B, 0x000E4357, 0x000C4358, 0x000D465C, 0x000C455E, 0x0006415B, 0x00033F5C, 0x00044260, 0x00044260, 0x00043E5B, 0x00013D5A, 0x00043E5B, 0x000A445A, 0x00083E4F, 0x00083E49, 0x00235763, 0x00083E49, 0x00034151,
        0x000D3D59, 0x000C3F5F, 0x0003435B, 0x000B444D, 0x007F9B9B, 0x008A9396, 0x00616A6E, 0x00D3DDD1, 0x00A6B0A4, 0x001E271D, 0x00151E14, 0x00222A23, 0x00171F18, 0x001D241F, 0x00202722, 0x000B100F, 0x004F504E, 0x00DEDDD9, 0x00D8D8D2, 0x00797D77,
        0x008B9791, 0x009FB2AF, 0x00AEC4C2, 0x0065B4C1, 0x0059A9CE, 0x0058B1CC, 0x006FBCCC, 0x007EA7BD, 0x00E1EEF0, 0x00CCD8C4, 0x00A4B2AC, 0x006E7374, 0x00C3CAC7, 0x00ACB4AD, 0x001E281C, 0x001A2418, 0x00172016, 0x00171F18, 0x001C231E, 0x000F2C1C,
        0x000B1D10, 0x0040483D, 0x00B5BDB2, 0x00C7D6C8, 0x00677765, 0x00A8B099, 0x00B5B198, 0x00BABEB8, 0x003A595A, 0x00013F49, 0x00004753, 0x000C515B, 0x000E4952, 0x0005434D, 0x00044A57, 0x00083A46, 0x00003D41, 0x00255E5D, 0x005B7470, 0x008B8F8A,
        0x00B1B9B2, 0x00222F27, 0x003A423B, 0x003D4136, 0x00393D32, 0x002A2E23, 0x0026281C, 0x0035372B, 0x00464638, 0x00474739, 0x00424234, 0x00434637, 0x00424536, 0x00414435, 0x00404436, 0x003F4435, 0x003D4233, 0x00404333, 0x00434636, 0x00454838,
        0x00454838, 0x00434636, 0x00454838, 0x00474936, 0x00454734, 0x00494B39, 0x00424636, 0x00474B3B, 0x00484B3B, 0x00474B3B, 0x00494B3C, 0x004F5142, 0x00525345, 0x00515343, 0x004E5142, 0x0046493A, 0x0046483B, 0x00515242, 0x00505141, 0x00525343,
        0x004B4D3E, 0x00474939, 0x0048493A, 0x004F5142, 0x004A4C3D, 0x004D4E40, 0x00535444, 0x00515344, 0x0046493C, 0x00414435, 0x003F4233, 0x00434637, 0x004A4C3E, 0x00535444, 0x00535444, 0x00535444, 0x00505242, 0x004E5040, 0x004D4D3D, 0x004F4F40,
        0x004A4B3D, 0x00424438, 0x003F4135, 0x00414337, 0x0046483C, 0x00434539, 0x003B3D31, 0x0037392D, 0x003A3C30, 0x00404236, 0x0045473B, 0x0044463A, 0x00414435, 0x00434637, 0x00434637, 0x00414135, 0x0018291E, 0x008C8984, 0x008D9E95, 0x0063827B,
        0x0068969E, 0x001F85A2, 0x001487BA, 0x000D83B7, 0x00208DB9, 0x001E86A9, 0x000E759C, 0x002390C3, 0x001386BF, 0x001C8AC4, 0x002892C7, 0x00126892, 0x00015A75, 0x0019738C, 0x001A7395, 0x0009607A, 0x000A545A, 0x00335D62, 0x00424E60, 0x0024555F,
        0x001C555E, 0x000C4D56, 0x0016545E, 0x002F6370, 0x00326371, 0x00296271, 0x000F505E, 0x001A5765, 0x001E5562, 0x002D626C, 0x00174B52, 0x00285E65, 0x001D5C64, 0x001D6871, 0x000A5A65, 0x003C6169, 0x00345858, 0x002C5351, 0x0010464D, 0x00135E78,
        0x00207FA7, 0x001B83B2, 0x00117CA8, 0x001382B0, 0x001C80B0, 0x001388B5, 0x001978A9, 0x000D84B1, 0x001A7CAC, 0x001083B0, 0x001B83B2, 0x00238FBE, 0x001581B0, 0x00137FAE, 0x000E7AA9, 0x00137FAE, 0x001581B0, 0x001985B4, 0x001682B1, 0x001C7EAE,
        0x001181AC, 0x001886AA, 0x001C7695, 0x0000475E, 0x00004452, 0x00043F49, 0x0051757F, 0x001D5266, 0x00073F52, 0x000A4457, 0x000B485C, 0x00014356, 0x00004458, 0x00004559, 0x00004559, 0x0001465A, 0x0009455B, 0x00074055, 0x00034153, 0x00004051,
        0x005097A5, 0x00397483, 0x004D7B8C, 0x00529DA5, 0x0070A2AE, 0x00406B7A, 0x004E929F, 0x0060B4C0, 0x005092A4, 0x00274C62, 0x00415872, 0x003A5772, 0x0036516B, 0x00416C7F, 0x0049939F, 0x00004752, 0x00044554, 0x000B4459, 0x00024358, 0x000A405E,
        0x0006444E, 0x0001678A, 0x0033ACEA, 0x00015A86, 0x00074B62, 0x000C5B64, 0x002D8175, 0x002A2F2D, 0x00000200, 0x00080C07, 0x001B1D17, 0x0021241B, 0x0024251B, 0x002B2B1F, 0x001D1D11, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFEFDFF, 0xFFFFFEFF, 0xFFFDFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00161B19, 0x00171B16, 0x000C0C06, 0x00120E09, 0x00141511, 0x000C1A18, 0x00335959, 0x00115152, 0x00004E51, 0x000D3F5D, 0x000A3C58, 0x000C3E5A, 0x000A3A56, 0x000D3C58,
        0x0015425D, 0x0014415C, 0x0016435E, 0x00094055, 0x000A4156, 0x000A4359, 0x000A425B, 0x00063E59, 0x00043F59, 0x0005415E, 0x0006425F, 0x00073F5A, 0x00023E5C, 0x0000405F, 0x000C4761, 0x000C4253, 0x00154753, 0x00043747, 0x000E4659, 0x00054655,
        0x000D3F5B, 0x000C3F5F, 0x00004058, 0x00063F48, 0x007A9998, 0x00828B8E, 0x00626D71, 0x00DBE5D9, 0x00ADB7AB, 0x00242D23, 0x00192218, 0x00252D26, 0x001C241D, 0x00222924, 0x00202722, 0x001A271F, 0x000E1E17, 0x00273E3A, 0x00203F3C, 0x00082E2E,
        0x00214A4C, 0x00194748, 0x001B4B4D, 0x00044953, 0x00053F62, 0x0005485F, 0x00064B55, 0x00264B61, 0x00CCCFD7, 0x00E3E5D2, 0x00B6C1B9, 0x006C7172, 0x00C5CCC9, 0x00A5ADA6, 0x00232D21, 0x001E281C, 0x0020291F, 0x001F2720, 0x001A211C, 0x001C2725,
        0x00141315, 0x00282128, 0x0054565E, 0x005D7176, 0x00173C3A, 0x00265049, 0x0029524A, 0x003B6361, 0x000D4648, 0x00004B53, 0x00004B56, 0x00024756, 0x00064151, 0x000D4858, 0x00074759, 0x000E4052, 0x000A4B54, 0x00286163, 0x00607878, 0x00919492,
        0x00B5BCB5, 0x001A271F, 0x00333B34, 0x00373B30, 0x003F4338, 0x0041453A, 0x00434539, 0x004A4C40, 0x004D4D3F, 0x00464638, 0x00414133, 0x00434637, 0x00434637, 0x00434637, 0x00434637, 0x00424637, 0x00404435, 0x00414434, 0x00424535, 0x00434636,
        0x00414434, 0x00444737, 0x00464939, 0x00494B38, 0x004A4C39, 0x004A4D3A, 0x00454838, 0x00484B3D, 0x004A4E3F, 0x004B4E3F, 0x004B4D3E, 0x00525344, 0x00545648, 0x004F5243, 0x00505243, 0x004A4D3E, 0x00484B3D, 0x00494B3C, 0x00494B3B, 0x00494B3C,
        0x00444738, 0x00414435, 0x00434637, 0x0046493A, 0x00474A3A, 0x00494B3C, 0x00494C3D, 0x00494C3D, 0x00434637, 0x00414434, 0x003F4233, 0x00414435, 0x00434637, 0x004A4C3C, 0x004F5041, 0x004C4D3F, 0x004F5141, 0x00505141, 0x00515242, 0x00494B3C,
        0x00434636, 0x00424536, 0x003F4233, 0x003D4031, 0x00424536, 0x00454839, 0x00424536, 0x0046493A, 0x00454839, 0x00404334, 0x00404334, 0x003E4132, 0x00404334, 0x0045473B, 0x003F4135, 0x00424137, 0x0016271E, 0x00827F7B, 0x0084958C, 0x00597A73,
        0x0063909B, 0x002188A8, 0x001688BD, 0x001A8EC5, 0x00208BB7, 0x002085AB, 0x001279A0, 0x00228FC2, 0x001185BC, 0x00198BC0, 0x002492C2, 0x00287197, 0x00003549, 0x00377F90, 0x003B879D, 0x00207382, 0x00025753, 0x00448782, 0x002D5E66, 0x00216362,
        0x000F5A58, 0x00277A78, 0x00136362, 0x00478C8F, 0x00307376, 0x00277274, 0x00116466, 0x0028646A, 0x00337074, 0x00437F7F, 0x00326E6D, 0x001E5D5B, 0x002C6E6D, 0x001C6164, 0x00246D71, 0x00298983, 0x002C8375, 0x000B5843, 0x00287468, 0x000F6166,
        0x00459CB0, 0x00338AA5, 0x004397B3, 0x00348BA6, 0x004997B4, 0x002B88A2, 0x004189A7, 0x002A89A3, 0x004992B0, 0x002B86A1, 0x003588A4, 0x004498B4, 0x003B8FAB, 0x003D91AD, 0x003286A2, 0x004094B0, 0x00368AA6, 0x003D91AD, 0x003185A1, 0x004996B0,
        0x002C869E, 0x004099AE, 0x00438BA3, 0x002A6D84, 0x00004961, 0x00034961, 0x00386D88, 0x00114B5E, 0x00043F52, 0x000A4558, 0x000B485C, 0x00024457, 0x0002455A, 0x00014459, 0x00004559, 0x00004155, 0x00074359, 0x00094257, 0x00064456, 0x00004152,
        0x004C93A1, 0x002E6978, 0x003B697A, 0x00156068, 0x00336571, 0x00285362, 0x004C909D, 0x0055A9B5, 0x005193A5, 0x0030556B, 0x003A516B, 0x003A5772, 0x0036516B, 0x00406B7E, 0x0049939F, 0x00004752, 0x00054655, 0x000B4459, 0x00014257, 0x000B415F,
        0x0005434D, 0x00006689, 0x0033ACEA, 0x00015A86, 0x00064A61, 0x000B5A63, 0x002C8074, 0x002A2F2D, 0x00000100, 0x00070B06, 0x001C1E18, 0x0022251C, 0x0024251B, 0x0029291D, 0x001B1B0F, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFEFDFF, 0xFFFFFEFF, 0xFFFDFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00171C1A, 0x00161A14, 0x00080802, 0x00100C07, 0x00181713, 0x000D1A18, 0x003A5B5E, 0x00134D52, 0x0000444B, 0x000C4259, 0x000A4057, 0x0010465D, 0x000C425B, 0x000B415A,
        0x000D425D, 0x00083D58, 0x000A3E5C, 0x00094258, 0x00084157, 0x00063F55, 0x00063F55, 0x000A4359, 0x000A4359, 0x00074056, 0x00074056, 0x000C4156, 0x0007415E, 0x00004062, 0x0002405E, 0x000C4156, 0x001B4D5F, 0x000A4057, 0x00054361, 0x00014251,
        0x000B3D59, 0x000D4060, 0x00003F57, 0x00013C45, 0x00779695, 0x00788486, 0x00636E72, 0x00DEE8DC, 0x00AFB9AD, 0x00252E24, 0x00172016, 0x00242C25, 0x00202821, 0x00262D28, 0x001F2621, 0x00151E11, 0x000E2118, 0x00002323, 0x00033940, 0x00003D48,
        0x0000404A, 0x00003C45, 0x0008464E, 0x00044952, 0x000B3F5D, 0x000A4656, 0x0000434A, 0x001A4359, 0x00CDD2DB, 0x00E2E3D3, 0x009FB0A5, 0x006C7172, 0x00D3DAD7, 0x00AFB7B0, 0x00313B2F, 0x00182216, 0x001A2319, 0x00222A23, 0x00202722, 0x00291D1B,
        0x00251A1D, 0x00181721, 0x00001626, 0x0004354B, 0x00003B52, 0x00003F57, 0x00003F5B, 0x0005424C, 0x0000434D, 0x00004E59, 0x0001495A, 0x00064258, 0x00063953, 0x000F455C, 0x00033F55, 0x00104560, 0x0000384B, 0x00205D67, 0x00577276, 0x00939897,
        0x00AEB5AE, 0x00222D23, 0x003A4035, 0x00393D32, 0x00404439, 0x0042463B, 0x00434539, 0x0045473B, 0x00464638, 0x00444436, 0x00474739, 0x00454839, 0x0046493A, 0x0046493A, 0x00454839, 0x00444738, 0x00414536, 0x00424535, 0x00424535, 0x00424535,
        0x00414435, 0x00464939, 0x00444737, 0x00454637, 0x00444634, 0x00404331, 0x00414334, 0x0047493B, 0x00494C3D, 0x00464A3B, 0x00484A3B, 0x004D4D3F, 0x004D4F40, 0x0047483A, 0x004A4B3C, 0x00434637, 0x00424536, 0x00424536, 0x00444738, 0x00444738,
        0x00434637, 0x00424536, 0x00424536, 0x00444738, 0x00454839, 0x00464839, 0x0046493A, 0x0046493A, 0x00444738, 0x00424536, 0x00424536, 0x00404334, 0x00414435, 0x00444637, 0x0046493A, 0x00474A3C, 0x00484A3B, 0x00484A3A, 0x00454839, 0x00424536,
        0x003F4233, 0x00424536, 0x00424536, 0x003E4132, 0x00404334, 0x00434637, 0x00404334, 0x00414434, 0x003F4232, 0x003A3D2D, 0x003E4132, 0x003D4031, 0x003D3F33, 0x0045473B, 0x00404236, 0x00414238, 0x0022332A, 0x0081807C, 0x008C9E97, 0x005F7F7A,
        0x005D8B96, 0x002F96B6, 0x001A8EC3, 0x00208CC2, 0x002287B4, 0x002585A9, 0x00197CA2, 0x002796C4, 0x001488BD, 0x00168BBE, 0x002595C3, 0x00236B89, 0x00074450, 0x0074A7AA, 0x0065939B, 0x0088BEBF, 0x003F7D6F, 0x0066A297, 0x003C6C70, 0x008FABA5,
        0x00466C66, 0x0079A8A0, 0x00467069, 0x007E9F9B, 0x00567572, 0x007EA3A1, 0x00558481, 0x00689390, 0x0053817B, 0x005E9288, 0x00558D80, 0x004C8477, 0x0064988E, 0x004E7C76, 0x007FA7A5, 0x00357579, 0x006DA19A, 0x00527969, 0x0097B8A9, 0x004A6F6D,
        0x0084ACB8, 0x00648A9C, 0x008BAFBF, 0x005A97A3, 0x0085BAC7, 0x004E959F, 0x007DAEBC, 0x004F97A1, 0x007DB1BE, 0x005698A3, 0x006EA8B4, 0x0064A0AC, 0x006AA6B2, 0x006AA6B2, 0x0066A2AE, 0x006EAAB6, 0x0065A1AD, 0x006CA8B4, 0x005F9BA7, 0x0082BBC4,
        0x00569AA1, 0x0079B8C0, 0x006C9BA9, 0x007BADC1, 0x00468BA6, 0x00509ABC, 0x004688B1, 0x00064658, 0x00024254, 0x0008485A, 0x0007465A, 0x00044357, 0x0006455A, 0x00034257, 0x0006455A, 0x0001465A, 0x0008445A, 0x00084156, 0x00064456, 0x00004051,
        0x004A919F, 0x00336E7D, 0x004B798A, 0x0061ACB4, 0x007AACB8, 0x003F6A79, 0x00428693, 0x0059ADB9, 0x004D8FA1, 0x0023485E, 0x004E657F, 0x003B5873, 0x0036516B, 0x00406B7E, 0x0049939F, 0x00004853, 0x00054655, 0x000C455A, 0x00014257, 0x000A405E,
        0x0004424C, 0x00006588, 0x0033ACEA, 0x00015A86, 0x00064A61, 0x000B5A63, 0x002E8276, 0x002C312F, 0x00000100, 0x00070B06, 0x001C1E18, 0x0023261D, 0x0025261C, 0x002A2A1E, 0x001D1D11, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFEFDFF, 0xFFFFFEFF, 0xFFFDFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00181D1B, 0x00171E17, 0x00090903, 0x00130F0A, 0x001C1915, 0x000D1816, 0x003F5E61, 0x001C5259, 0x00074A53, 0x000B4757, 0x00033E51, 0x00064357, 0x00074359, 0x0004425A,
        0x00003F58, 0x00003A56, 0x00044462, 0x00074059, 0x000C455E, 0x000B445A, 0x000B4158, 0x000D4459, 0x000D4256, 0x000D4054, 0x00124657, 0x00104353, 0x00063E59, 0x00003F62, 0x00044161, 0x00093F56, 0x00083D52, 0x000D4665, 0x00004067, 0x00054655,
        0x000E405C, 0x000D4362, 0x00014159, 0x0006414A, 0x0084A3A2, 0x008C989A, 0x00828D91, 0x00D1DBCF, 0x00A3ADA1, 0x00192218, 0x000D160C, 0x001D251E, 0x001E261F, 0x00262D28, 0x001D241F, 0x00271E10, 0x00202720, 0x000B2B30, 0x001B5463, 0x000E5569,
        0x00084F63, 0x0018586A, 0x00124D5C, 0x00004A50, 0x000C415C, 0x000C4756, 0x00004044, 0x00114055, 0x00C4CED8, 0x00C7CFBE, 0x0070887C, 0x00505556, 0x00B8BFBC, 0x00919992, 0x00252F23, 0x000D170B, 0x00141D13, 0x001F2720, 0x001B221D, 0x001E2A14,
        0x00102516, 0x00042C2A, 0x00155E66, 0x0044A7BB, 0x0056BFE0, 0x004BA9D4, 0x003A87B8, 0x002B6074, 0x00134E61, 0x00004255, 0x00044358, 0x000F4661, 0x00063C5A, 0x000D4A64, 0x0000445A, 0x00074060, 0x00004058, 0x00084553, 0x00223E45, 0x004C5051,
        0x00585F58, 0x00273125, 0x002F3629, 0x003B3F34, 0x003E4237, 0x003E4237, 0x00424438, 0x0047493D, 0x0048483A, 0x00474739, 0x004B4B3D, 0x00474A3B, 0x00494C3D, 0x00494C3D, 0x00474A3B, 0x00434637, 0x00404334, 0x00404334, 0x00414434, 0x00444737,
        0x00444738, 0x00454839, 0x00454839, 0x0047493A, 0x00464939, 0x0046493A, 0x00474A3B, 0x00464638, 0x00494A3C, 0x0048493B, 0x004D4D3F, 0x00464638, 0x0049493B, 0x00464638, 0x0049493B, 0x00474A3B, 0x00474A3B, 0x00474A3B, 0x00474A3B, 0x0046493A,
        0x00454839, 0x00434637, 0x00424536, 0x00454839, 0x0046493A, 0x00474A3B, 0x00474A3B, 0x00474A3B, 0x0046493A, 0x00454839, 0x00444738, 0x00404334, 0x00414435, 0x00424536, 0x00434637, 0x00434637, 0x00434637, 0x00424536, 0x00414435, 0x00414435,
        0x003E4132, 0x00434637, 0x00474A3B, 0x00434637, 0x00434637, 0x00444738, 0x003F4233, 0x003E422F, 0x003F4330, 0x00414434, 0x004A4D3D, 0x00434637, 0x003B3D31, 0x0045473B, 0x00434539, 0x003C3D33, 0x00182B22, 0x0053524E, 0x0053655E, 0x001E3E39,
        0x0022505B, 0x00238AAA, 0x001F93C8, 0x00238AC1, 0x002889B5, 0x002884A7, 0x0017789C, 0x002996C2, 0x001388BB, 0x001389B8, 0x002599C2, 0x0025758C, 0x00003E42, 0x0084A5A1, 0x00606E6D, 0x009EACA8, 0x00586E5C, 0x0099ADA1, 0x00556068, 0x009AAEA2,
        0x00466255, 0x008CB0A2, 0x0047695E, 0x0096AEA4, 0x00576C63, 0x0089A69D, 0x004C7167, 0x0093A59E, 0x0061796D, 0x007B9C8D, 0x00648B76, 0x006E937F, 0x007A9486, 0x0067776C, 0x00949B96, 0x00516D7E, 0x0093A6AB, 0x00606B61, 0x00A3B1A5, 0x00465F5B,
        0x008FB5BA, 0x00659198, 0x008AB6BD, 0x0057878D, 0x0094BBC3, 0x00548C91, 0x0094B8C0, 0x00538B90, 0x0089ADB5, 0x005D9197, 0x0088B4BB, 0x0069979E, 0x007BA9B0, 0x0073A1A8, 0x0077A5AC, 0x00719FA6, 0x006F9DA4, 0x0076A4AB, 0x006C9AA1, 0x007FAFB1,
        0x00629898, 0x0084B4B5, 0x0066838A, 0x009BBDCD, 0x00407D97, 0x0028769B, 0x00004372, 0x00004455, 0x00004556, 0x0007495B, 0x00054458, 0x00044357, 0x0008445A, 0x00053F55, 0x000A445A, 0x0004495D, 0x00074359, 0x00084156, 0x00094759, 0x00003F50,
        0x00468D9B, 0x00346F7E, 0x00568495, 0x005FAAB2, 0x0061939F, 0x00345F6E, 0x00478B98, 0x0055A9B5, 0x004C8EA0, 0x00274C62, 0x00405771, 0x003C5974, 0x0036516B, 0x00406B7E, 0x0049939F, 0x00004853, 0x00064756, 0x000C455A, 0x00014257, 0x00093F5D,
        0x0002404A, 0x00006487, 0x0033ACEA, 0x00025B87, 0x00064A61, 0x000C5B64, 0x002F8377, 0x002E3331, 0x00000200, 0x00070B06, 0x001C1E18, 0x0024271E, 0x0026271D, 0x002C2C20, 0x001F1F13, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFDFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00171C1A, 0x00161A15, 0x00100B08, 0x00091009, 0x000E140F, 0x001A1C1C, 0x002F5E5C, 0x00055C59, 0x00054C50, 0x00084853, 0x0000424D, 0x00054956, 0x0004495A, 0x00125467,
        0x00034257, 0x00073F5A, 0x000E435E, 0x0000445C, 0x0005465B, 0x00074154, 0x000D4050, 0x00134656, 0x000E4357, 0x00033F55, 0x0002425B, 0x0008405B, 0x0008405B, 0x0008405B, 0x0008405B, 0x0009415C, 0x0009415C, 0x0009415C, 0x0009415C, 0x00004159,
        0x00004059, 0x000E445D, 0x00124157, 0x00143F54, 0x006D97AA, 0x00C2F0FF, 0x009ACBDB, 0x00607F82, 0x00485C57, 0x001A2217, 0x00171E17, 0x000B1417, 0x000C171B, 0x00131914, 0x00221F10, 0x000E2422, 0x002B423A, 0x007D9184, 0x00BBCEBF, 0x00A0B3AA,
        0x00B5CCCE, 0x00B9D4E2, 0x009FBBD3, 0x002796D4, 0x002A9BDD, 0x00299DDC, 0x00107DB1, 0x00157599, 0x003B8EAD, 0x003587AA, 0x001F749C, 0x000E6B9C, 0x00086E91, 0x00135C70, 0x00121D25, 0x00281615, 0x001C2014, 0x00001A07, 0x00052010, 0x00171919,
        0x000C1217, 0x004C5862, 0x008495A2, 0x00E8FCFF, 0x00CDE1E2, 0x00E2F6EA, 0x00E6F9E6, 0x00D2EADE, 0x003B655E, 0x00044248, 0x00044659, 0x00104B65, 0x00033A55, 0x00054157, 0x0003495A, 0x00094E58, 0x00064553, 0x00043A4B, 0x00002C41, 0x00001E2E,
        0x00001821, 0x00031F20, 0x00112A26, 0x0031322E, 0x00333430, 0x00363733, 0x003A3A34, 0x003A3A34, 0x003A3B32, 0x003C3A32, 0x003C3A32, 0x00353C35, 0x003E3E38, 0x00413B34, 0x00413730, 0x0040372E, 0x00403B32, 0x003B3C32, 0x00323B2E, 0x003B3A2E,
        0x003D3B30, 0x003F3D32, 0x003D3C32, 0x003D3C32, 0x003B3C32, 0x003E3F36, 0x00404138, 0x00373933, 0x003B3C38, 0x00343533, 0x003A3838, 0x00383636, 0x00333432, 0x003C3D39, 0x00343630, 0x003B392E, 0x003A382E, 0x0037352D, 0x00413E3A, 0x00373430,
        0x003F3835, 0x003C312D, 0x004A3E38, 0x003F3629, 0x00494032, 0x00463F30, 0x00393523, 0x00464634, 0x00393B27, 0x00383E27, 0x00343C25, 0x003C352C, 0x003C372E, 0x003E3B33, 0x003C3934, 0x003C3A32, 0x003E3C31, 0x00413B2E, 0x00463F30, 0x003A3C30,
        0x003C3C30, 0x003E3D2F, 0x003B382A, 0x00433C2D, 0x00443C2B, 0x00463C2B, 0x00453B2A, 0x003F382F, 0x00423B32, 0x00423B32, 0x003D382F, 0x003C372E, 0x003C3A30, 0x003C3A30, 0x0037352B, 0x00262423, 0x000E201F, 0x0000191A, 0x00002E37, 0x000B4E63,
        0x00004869, 0x00248AB5, 0x001A91C2, 0x001E8DBD, 0x002389B9, 0x002380AD, 0x001978A3, 0x002B94BF, 0x001584B2, 0x001D88BB, 0x002F93CA, 0x00126487, 0x002B3743, 0x005B574C, 0x00495948, 0x0054584C, 0x005D5947, 0x00565C4B, 0x004B4C48, 0x0069675C,
        0x00525045, 0x00616155, 0x0056564A, 0x0057594D, 0x00525448, 0x005A5C50, 0x00515549, 0x0066655B, 0x005A594F, 0x005B5A50, 0x005A594F, 0x0059584E, 0x0058574D, 0x0055544A, 0x00616056, 0x00565A4F, 0x005A5E53, 0x0052564B, 0x00585C51, 0x004C5045,
        0x005E6257, 0x0052564B, 0x0063675C, 0x00535248, 0x00626157, 0x0056554B, 0x0065665C, 0x00525349, 0x00606459, 0x0052564B, 0x005F6358, 0x004C5045, 0x005B5F54, 0x005A5E53, 0x005B5F54, 0x005A5E53, 0x005A5E53, 0x0061655A, 0x00575B50, 0x00616859,
        0x005C5951, 0x0066625D, 0x0053594E, 0x00636958, 0x0046463A, 0x0027393A, 0x00103D4A, 0x000A485A, 0x000A485A, 0x0009465A, 0x00064559, 0x00064559, 0x0004455A, 0x0005465B, 0x0005465B, 0x00044358, 0x000B455B, 0x000F485D, 0x00004153, 0x00004253,
        0x00488F9D, 0x002F6675, 0x003D6577, 0x004A949A, 0x0039858B, 0x001F535F, 0x00508491, 0x0052A7B1, 0x0045909E, 0x002C4F63, 0x003B576F, 0x003D5571, 0x003B526C, 0x003F697C, 0x004A909D, 0x00004A55, 0x00074857, 0x000A4358, 0x00014257, 0x0004435F,
        0x0005404A, 0x000B5E84, 0x0030AFEC, 0x00035A86, 0x000A4860, 0x00006267, 0x0039786F, 0x00202A2A, 0x00000300, 0x000C0E08, 0x00212016, 0x0028261B, 0x0026261A, 0x002A2B21, 0x001B1E15, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFDFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00171C1A, 0x00141813, 0x000D0805, 0x00091009, 0x00121813, 0x001D1F1F, 0x00346361, 0x000D6461, 0x000B5256, 0x00224248, 0x00173B43, 0x0016404C, 0x00114250, 0x000E4657,
        0x00094257, 0x000A435C, 0x00083F5A, 0x00054459, 0x000F4556, 0x00143C48, 0x00113137, 0x00143138, 0x00153B47, 0x000E4052, 0x00094258, 0x0008405B, 0x0008405B, 0x0008405B, 0x0008405B, 0x0009415C, 0x0009415C, 0x0009415C, 0x0009415C, 0x000D4961,
        0x000D465F, 0x00073F58, 0x00074056, 0x0000354A, 0x0030788A, 0x0082D2E3, 0x0058B0BE, 0x0004383E, 0x00072C28, 0x00061A0D, 0x00131F13, 0x00151F19, 0x00171E1B, 0x001B1E15, 0x00221F10, 0x00101611, 0x00656A61, 0x00F6FEED, 0x00EFFAE6, 0x0097A795,
        0x00BAD2C8, 0x00B4D3D2, 0x00A2C4CA, 0x0040ABD1, 0x0037B0E1, 0x0016A2DD, 0x0024B7F5, 0x002FC1FB, 0x0023B2EB, 0x002CBAFB, 0x0022B2FA, 0x0028B8FF, 0x0025B5F6, 0x00238BBA, 0x00082C4A, 0x00191423, 0x00211F1F, 0x00141C15, 0x001B1E15, 0x00161F0A,
        0x00131D0D, 0x00465149, 0x00B3BCB9, 0x00DBE2DF, 0x0093978C, 0x00D6D5C1, 0x00CECAB1, 0x00BEDCD1, 0x003B6865, 0x0009474D, 0x00003D50, 0x0003405A, 0x0009415C, 0x000D495F, 0x00004556, 0x0000424F, 0x00003D4D, 0x0001394C, 0x00053449, 0x00082E40,
        0x00062633, 0x00052227, 0x00052121, 0x000C1C15, 0x000B1B14, 0x000B1913, 0x000B1A12, 0x000D1A12, 0x000D1B10, 0x000C1A0F, 0x000D180E, 0x000A140E, 0x000D140F, 0x00121912, 0x00141812, 0x0011160D, 0x000F170D, 0x00111A10, 0x00101C10, 0x00162014,
        0x00111D11, 0x000D190D, 0x000E1A0E, 0x000F1D12, 0x00101E13, 0x000B1A12, 0x0007160E, 0x00181A14, 0x00171B15, 0x00121611, 0x00161917, 0x00151816, 0x00121611, 0x00191D17, 0x00141610, 0x000E180B, 0x00111B0F, 0x000D180E, 0x00141F17, 0x000D1810,
        0x000F1A12, 0x00121A13, 0x000E150E, 0x00041C12, 0x00021911, 0x000A1C15, 0x0003130C, 0x000C1714, 0x000D1613, 0x000B1110, 0x000C1110, 0x00121A0F, 0x0010190F, 0x000F1A10, 0x000B160C, 0x000A150B, 0x000A160A, 0x000D170A, 0x00111B0B, 0x000B1915,
        0x000A1814, 0x000C1A16, 0x000A1814, 0x00081612, 0x000C1714, 0x0006110E, 0x0006110E, 0x000D160C, 0x0010190F, 0x00111A10, 0x00111A10, 0x00121D13, 0x00121D13, 0x000D1B10, 0x000B190E, 0x00111919, 0x00001614, 0x00002928, 0x00336D72, 0x003A7F90,
        0x000E607F, 0x002991BC, 0x00158EC0, 0x001E8FBA, 0x00258EBA, 0x001E7DAE, 0x00036999, 0x002190C0, 0x001C92C3, 0x001C8CC1, 0x00238DC3, 0x00085A7D, 0x00192531, 0x00403C31, 0x002E3E2D, 0x00383C30, 0x003E3A28, 0x003A402F, 0x00353632, 0x003D3D31,
        0x002F2F23, 0x003D3D31, 0x0035372B, 0x003B3D31, 0x002F3327, 0x0035392D, 0x002D3125, 0x003A3B31, 0x00313228, 0x0034352B, 0x0033342A, 0x0033342A, 0x0033342A, 0x00303127, 0x0037382E, 0x0032362B, 0x0035392E, 0x00303429, 0x0033372C, 0x002B2F24,
        0x0033372C, 0x00262A1F, 0x00303429, 0x002D2C22, 0x0037362C, 0x002D2C22, 0x0038392F, 0x002D2E24, 0x00363A2F, 0x00292D22, 0x0031352A, 0x002C3025, 0x0032362B, 0x002D3126, 0x002E3227, 0x002D3126, 0x002A2E23, 0x0031352A, 0x002D3126, 0x002B3223,
        0x0028251D, 0x002D2924, 0x0022281D, 0x00343A29, 0x002B2B1F, 0x00182A2B, 0x000D3A47, 0x00084658, 0x00084658, 0x00094759, 0x0007465A, 0x0007465A, 0x0005465B, 0x0004455A, 0x00034459, 0x00024156, 0x000A445A, 0x000E475C, 0x00003F51, 0x00003C4D,
        0x00458C9A, 0x003D7483, 0x00598193, 0x007AC2C9, 0x005FABB1, 0x00386A76, 0x00588C99, 0x0052A7B1, 0x00458E9C, 0x002A4D61, 0x003C566E, 0x003D5571, 0x003A516B, 0x003E687B, 0x00498F9C, 0x00004954, 0x00064756, 0x000A4358, 0x00024358, 0x0004435F,
        0x0005404A, 0x000B5E84, 0x0030AFEC, 0x00045B87, 0x000A4860, 0x00006267, 0x003A7970, 0x00212B2B, 0x00000200, 0x000A0C06, 0x001E1D13, 0x0027251A, 0x0029291D, 0x002E2F25, 0x001D2017, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFDFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00171C1A, 0x00171B16, 0x000B0603, 0x00091009, 0x00131914, 0x001D1F1F, 0x00396866, 0x0019706D, 0x00165D61, 0x00080808, 0x00000404, 0x00081B20, 0x001E3F48, 0x0015404F,
        0x00124559, 0x000C455B, 0x00023D57, 0x000B4558, 0x00194552, 0x00182F31, 0x00000A04, 0x00000400, 0x0013282A, 0x001B4250, 0x000D4256, 0x0008405B, 0x0008405B, 0x0008405B, 0x0008405B, 0x0009415C, 0x0009415C, 0x0009415C, 0x0009415C, 0x000C425B,
        0x000E465F, 0x00043C55, 0x000D4961, 0x00004057, 0x0018647B, 0x0059AEC4, 0x003A93A8, 0x00003745, 0x00012E31, 0x000E261C, 0x00141D10, 0x00191C13, 0x00161715, 0x001A1C1D, 0x00181719, 0x0020161C, 0x003C3434, 0x00939189, 0x007D8475, 0x00465847,
        0x00728C7F, 0x0053756E, 0x0039605E, 0x000C4B59, 0x00004062, 0x00017AB2, 0x0024B4FB, 0x001CB3FE, 0x001CB0FE, 0x0027B9FF, 0x0027B7FF, 0x0018B2FF, 0x0021B3FF, 0x001985BF, 0x0000254E, 0x00080F23, 0x001E1F23, 0x0023241B, 0x00241E13, 0x00171D06,
        0x001F2814, 0x00353E34, 0x00A2B0AC, 0x00A8B6B4, 0x0066766F, 0x009FB0A3, 0x007B8B7A, 0x00759A96, 0x001F4F50, 0x000F4C56, 0x0007495C, 0x0004425A, 0x0009455D, 0x0009485D, 0x00014355, 0x00024456, 0x00044155, 0x00023B51, 0x0004354B, 0x00083147,
        0x000B2F3F, 0x000B2B36, 0x0008282E, 0x004F605C, 0x004B5C58, 0x00485851, 0x0045554E, 0x0045534D, 0x0045544C, 0x0045544C, 0x0046534B, 0x00525756, 0x004A4F4D, 0x0049524F, 0x0046514E, 0x00434F49, 0x00434D47, 0x004B524B, 0x004A514A, 0x00445247,
        0x00445247, 0x0045524A, 0x0043524A, 0x0044524C, 0x0043534C, 0x0043534C, 0x0043524E, 0x00454C45, 0x00485049, 0x004A554D, 0x0047524A, 0x00424D45, 0x003C473F, 0x00454D46, 0x00464D46, 0x00444F45, 0x00455046, 0x003D483E, 0x003C4941, 0x003D4C44,
        0x0034423C, 0x003F4F48, 0x003D4D46, 0x00364A45, 0x0031423E, 0x00394A47, 0x003C4848, 0x003B4447, 0x00444B4E, 0x003F4348, 0x003F4348, 0x003F4C44, 0x003E4B43, 0x003F4C44, 0x003C4A3F, 0x003C4A3F, 0x0038483D, 0x003C4C41, 0x003B4B40, 0x003F4643,
        0x003C4443, 0x003D4846, 0x003B4846, 0x00354342, 0x003B4B4A, 0x00314344, 0x00344647, 0x003A4941, 0x003B4A42, 0x0035443C, 0x0033443B, 0x00394A41, 0x00394A41, 0x00384B42, 0x003F5249, 0x00081C21, 0x00001E1F, 0x00366563, 0x009EDADA, 0x0077BDCA,
        0x001F7691, 0x002591BB, 0x000D8ABD, 0x002394BC, 0x001B84B0, 0x00248ABB, 0x00107EB2, 0x001892C8, 0x000E8DC4, 0x00118AC2, 0x001D8CC4, 0x0015678A, 0x00212D39, 0x00413D32, 0x00314130, 0x003A3E32, 0x003F3B29, 0x00393F2E, 0x003A3B37, 0x003E3E32,
        0x003B3B2F, 0x003E4034, 0x003B3D31, 0x0042463A, 0x0035392D, 0x00394033, 0x00333A2D, 0x003A3E33, 0x00363A2F, 0x00373B30, 0x00363A2F, 0x00373B30, 0x00393D32, 0x0034382D, 0x0034382D, 0x00363A2F, 0x00363A2F, 0x0035392E, 0x0034382D, 0x00363A2F,
        0x00393D32, 0x0032362B, 0x00373B30, 0x0035342A, 0x0039382E, 0x00333228, 0x0036372D, 0x0036372D, 0x00383C31, 0x00303429, 0x0034382D, 0x00373B30, 0x0034382D, 0x002F3328, 0x0032362B, 0x0032362B, 0x002F3328, 0x00303429, 0x00303429, 0x00313829,
        0x00343129, 0x00332F2A, 0x002D3328, 0x00343A29, 0x00313125, 0x00182A2B, 0x00103D4A, 0x00064454, 0x00074557, 0x00094759, 0x0008485A, 0x0008475B, 0x0005475A, 0x00034459, 0x00024358, 0x00034257, 0x00084258, 0x000E475C, 0x0006485A, 0x00004556,
        0x00458C9A, 0x00346B7A, 0x00466E80, 0x00499198, 0x003C868C, 0x00225460, 0x0050818F, 0x0051A6B0, 0x0047909E, 0x002B4E62, 0x003B556D, 0x003D5571, 0x003A516B, 0x003D677A, 0x00478D9A, 0x00004752, 0x00054655, 0x00094257, 0x00024358, 0x0003425E,
        0x0005404A, 0x000B5E84, 0x002FAEEB, 0x00045B87, 0x000A4860, 0x00006166, 0x003B7A71, 0x00242E2E, 0x00000200, 0x000A0C06, 0x001D1C12, 0x00262419, 0x002B2B1F, 0x002E2F25, 0x001C1F16, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFCFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00181D1B, 0x001D211C, 0x000D0805, 0x00080F08, 0x00101611, 0x00181A1A, 0x003A6967, 0x00217875, 0x0020676B, 0x00100E06, 0x00000200, 0x00000F0E, 0x0027454A, 0x00123F4A,
        0x000F4354, 0x000D465C, 0x000C4761, 0x00044252, 0x001B4950, 0x001C322D, 0x00000500, 0x00020500, 0x00132722, 0x001F4950, 0x00084051, 0x0008405B, 0x0008405B, 0x0008405B, 0x0008405B, 0x0009415C, 0x0009415C, 0x0009415C, 0x0009415C, 0x00044056,
        0x000C455E, 0x00063E57, 0x00124762, 0x00063B56, 0x00023654, 0x00184B6B, 0x00013454, 0x0000354A, 0x00042A36, 0x00041917, 0x00060D08, 0x00101615, 0x00010F1B, 0x0009203A, 0x00000E31, 0x00091B3A, 0x00000E29, 0x00001125, 0x00012330, 0x00012C35,
        0x0012434B, 0x00033944, 0x00003844, 0x0000444F, 0x00107895, 0x002AB1E7, 0x001CB8F9, 0x0021BCFB, 0x0024B1EA, 0x000988BF, 0x001087BF, 0x00258DC8, 0x002C8BBD, 0x001E6489, 0x00002038, 0x0003141D, 0x000D1A18, 0x000D1D12, 0x00081709, 0x00141414,
        0x0011171C, 0x00000D19, 0x002C4958, 0x00325D70, 0x00256070, 0x00367A87, 0x00246F78, 0x0070A0A2, 0x001F5359, 0x0004404C, 0x00024155, 0x00004156, 0x0006445C, 0x00044358, 0x00044357, 0x00014257, 0x00044358, 0x00053F55, 0x0000354A, 0x00002E43,
        0x00032D40, 0x00062D3C, 0x00042937, 0x003B4545, 0x00373F3F, 0x002F3736, 0x002A3231, 0x002C3330, 0x002C3330, 0x002D3230, 0x002C322D, 0x00313034, 0x00272A2E, 0x00273131, 0x00243231, 0x00293735, 0x00293432, 0x002F3433, 0x002A2B29, 0x002A302B,
        0x002C312F, 0x002D3230, 0x002B322F, 0x00292F2E, 0x00262E2D, 0x00262E2E, 0x00272F2F, 0x0025332F, 0x0024342D, 0x00263930, 0x001F3327, 0x00263A2E, 0x0024372E, 0x0025352E, 0x0024322E, 0x00323735, 0x002F3530, 0x00313530, 0x00293029, 0x00383F3A,
        0x002B3633, 0x002A3836, 0x00213130, 0x00383031, 0x00373231, 0x00363231, 0x00383733, 0x0031332D, 0x00313831, 0x00273026, 0x00273026, 0x002C3231, 0x002C3330, 0x00313732, 0x00303631, 0x00303631, 0x00262F2C, 0x002A3735, 0x00233130, 0x0037342C,
        0x0034322A, 0x0033342B, 0x0030342E, 0x00272E27, 0x00303B33, 0x0029352F, 0x00283630, 0x00293130, 0x002E3635, 0x002C3735, 0x002F3A38, 0x0035403E, 0x00303D3B, 0x00313E3C, 0x003D4A48, 0x0005272D, 0x0000272A, 0x00386A6A, 0x0088C5C7, 0x004A96A2,
        0x0016758F, 0x002292BD, 0x001490C6, 0x001581A3, 0x00076E95, 0x003197C7, 0x002393C8, 0x000D8CC3, 0x00098EC7, 0x001997D1, 0x001188C0, 0x000D5F82, 0x001E2A36, 0x003D392E, 0x002F3F2E, 0x00393D31, 0x00413D2B, 0x00373D2C, 0x00363733, 0x003E3F35,
        0x00424339, 0x003A3E33, 0x00373B30, 0x0040463B, 0x00353B30, 0x003E4439, 0x003A4237, 0x003D4136, 0x003D4136, 0x003C4035, 0x00393D32, 0x003B3F34, 0x003E4237, 0x00393D32, 0x0034382D, 0x003F4337, 0x003C4034, 0x00373B2F, 0x002F3327, 0x00363A2E,
        0x0034382C, 0x0032362A, 0x0034382C, 0x0038372D, 0x0038372D, 0x0037362C, 0x0033342A, 0x00393A30, 0x0034382D, 0x0035392E, 0x003B3F34, 0x00373D32, 0x00343A2F, 0x00343A2F, 0x00343A2F, 0x00363C31, 0x00373D32, 0x0031372C, 0x002E3429, 0x002D3425,
        0x0037342C, 0x0034302B, 0x00353B30, 0x00313726, 0x00323226, 0x00102223, 0x00093643, 0x00064454, 0x00074555, 0x00084658, 0x0008485A, 0x0008485A, 0x0005475A, 0x00044659, 0x00034558, 0x0006455A, 0x00064056, 0x00074055, 0x00024456, 0x00004455,
        0x00448B99, 0x00366D7C, 0x00487082, 0x0064ACB3, 0x0058A2A8, 0x00346672, 0x00528391, 0x004C9EA9, 0x003F8896, 0x00294A5E, 0x003D576F, 0x003D5571, 0x003A516B, 0x003C6679, 0x00468C99, 0x00004651, 0x00044554, 0x00094257, 0x00024358, 0x0003425E,
        0x0006414B, 0x000A5D83, 0x002FAEEB, 0x00055C88, 0x0009475F, 0x00006065, 0x003E7D74, 0x00242E2E, 0x00000200, 0x000E100A, 0x001F1E14, 0x00242217, 0x0029291D, 0x002A2B21, 0x00181B12, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFCFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00181D1B, 0x001F231E, 0x000E0906, 0x00091009, 0x00111712, 0x00181A1A, 0x00396866, 0x001F7673, 0x001D6468, 0x00152D21, 0x0000170E, 0x00022723, 0x00255557, 0x00104450,
        0x00094154, 0x00043C55, 0x00013954, 0x00004654, 0x0010474E, 0x001B3D36, 0x000F2213, 0x00051809, 0x0011322B, 0x0014484F, 0x00024554, 0x0008405B, 0x0008405B, 0x0008405B, 0x0008405B, 0x0009415C, 0x0009415C, 0x0009415C, 0x0009415C, 0x0009415A,
        0x000B435C, 0x0009415A, 0x000B425D, 0x00124762, 0x0011435F, 0x000E3D59, 0x000D3C58, 0x0007394F, 0x000C3746, 0x0005272D, 0x00042125, 0x00082C34, 0x0000314A, 0x00186494, 0x000B65A5, 0x000F69A4, 0x001B78AF, 0x000F70A2, 0x00187EA9, 0x00137CA5,
        0x000C759C, 0x00137AA1, 0x001B82A9, 0x001588B5, 0x002DB0E9, 0x0026B6FD, 0x002DBCFF, 0x00259FD3, 0x00005B7C, 0x00004258, 0x00064155, 0x00003D50, 0x00023849, 0x00012C3B, 0x0000262F, 0x000F3137, 0x0007252A, 0x00022025, 0x00001D21, 0x00001122,
        0x00031B2F, 0x0004233C, 0x001B4A66, 0x00266482, 0x00226F8F, 0x00106A89, 0x00187A98, 0x0056919B, 0x00235D69, 0x00084353, 0x000D4A5E, 0x0007485D, 0x0004475C, 0x00044659, 0x000C495D, 0x00003F57, 0x0006455A, 0x0009465A, 0x00074154, 0x00063B4F,
        0x00053749, 0x00013143, 0x00002A3C, 0x00273B40, 0x00203439, 0x001A2D32, 0x0016292C, 0x0018292C, 0x00192B2C, 0x00192B2C, 0x001B2A2C, 0x00182C31, 0x00132C30, 0x00102F32, 0x00002424, 0x00042828, 0x00062625, 0x00142E2E, 0x00182D2E, 0x00172928,
        0x00162827, 0x00152726, 0x00142829, 0x0015292A, 0x00152A2C, 0x0014292B, 0x0013282A, 0x00112F30, 0x000F2C29, 0x00132D27, 0x00051E14, 0x00162F25, 0x00142E28, 0x000F2C29, 0x001A3839, 0x001B3136, 0x000C2122, 0x00233633, 0x001D2E2A, 0x00132623,
        0x000B2021, 0x00132E32, 0x00153239, 0x000F2433, 0x001C323E, 0x0010262C, 0x000F2628, 0x00152C28, 0x00142C22, 0x00182F21, 0x001A3220, 0x0014272E, 0x0014272A, 0x001B2B2A, 0x00182926, 0x00172928, 0x000B2224, 0x00153138, 0x000C2A35, 0x00102C2C,
        0x00112D2D, 0x00143030, 0x0013302D, 0x000D2C29, 0x0013322F, 0x0012322D, 0x00072722, 0x0014292A, 0x00162B2C, 0x0014292A, 0x00132B2B, 0x00162E2E, 0x00163030, 0x001E3838, 0x002F4949, 0x0006343C, 0x0006353D, 0x000B4247, 0x00135861, 0x00005565,
        0x00167E9B, 0x001E91BC, 0x001991C5, 0x001D819D, 0x00004667, 0x000E6B92, 0x00258EBA, 0x00219ACB, 0x00098DC2, 0x001091C8, 0x00138EC8, 0x000A5C7F, 0x00232F3B, 0x00413D32, 0x0030402F, 0x003A3E32, 0x00484432, 0x003A402F, 0x00373834, 0x0041453A,
        0x00484C41, 0x003F453A, 0x003D4338, 0x00464E43, 0x0040483D, 0x00495146, 0x00495347, 0x0043493E, 0x00454B40, 0x0042483D, 0x0040463B, 0x0042483D, 0x00464C41, 0x00464C41, 0x003F453A, 0x00414539, 0x003D4135, 0x0035392D, 0x002F3327, 0x00363A2E,
        0x0033372B, 0x0035392D, 0x0035392D, 0x0039382E, 0x003A392F, 0x003B3A30, 0x0034352B, 0x00393A30, 0x0032362B, 0x003B3F34, 0x00464A3F, 0x003F453A, 0x003C4237, 0x003F453A, 0x00383E33, 0x003B4136, 0x0043493E, 0x00373D32, 0x0030362B, 0x002C3324,
        0x0037342C, 0x00312D28, 0x0032382D, 0x002D3322, 0x00353529, 0x0017292A, 0x0013404D, 0x00074555, 0x00074555, 0x00084656, 0x00074759, 0x00074759, 0x00054759, 0x0005475A, 0x00044659, 0x0007465B, 0x000A445A, 0x000C455A, 0x00054759, 0x00003F50,
        0x003F8694, 0x003D7483, 0x00537B8D, 0x004C9299, 0x004C949B, 0x0032626E, 0x0050818F, 0x004B9DA8, 0x00408795, 0x00294A5E, 0x00415971, 0x003D5571, 0x0039506A, 0x003C6679, 0x00468C99, 0x00004651, 0x00044554, 0x00094257, 0x00014257, 0x0002415D,
        0x0006414B, 0x00095C82, 0x002EADEA, 0x00055C88, 0x0008465E, 0x00005F64, 0x00407F76, 0x00242E2E, 0x00000200, 0x000F110B, 0x00222117, 0x00242217, 0x0027271B, 0x0028291F, 0x00181B12, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFBFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00191E1C, 0x001A1E19, 0x000E0906, 0x000B120B, 0x00131914, 0x001B1D1D, 0x00366563, 0x00156C69, 0x0011585C, 0x00376965, 0x001E5350, 0x00114A4B, 0x00104B54, 0x00003C4A,
        0x00054054, 0x000D445F, 0x000C405E, 0x00004256, 0x00084551, 0x00225454, 0x00365F58, 0x00325B54, 0x001D4D4E, 0x000B4551, 0x00004257, 0x0008405B, 0x0008405B, 0x0008405B, 0x0008405B, 0x0009415C, 0x0009415C, 0x0009415C, 0x0009415C, 0x00103E5D,
        0x000D3F5B, 0x0009425B, 0x00003D52, 0x0002475A, 0x00044C5D, 0x00003E4F, 0x00004654, 0x00003D4D, 0x00074251, 0x000B434E, 0x00073E45, 0x0000353F, 0x0000465F, 0x001E8DBF, 0x0033B6F9, 0x001EB4FB, 0x001EB7FC, 0x0020BCFE, 0x0022BEFF, 0x001EBBFA,
        0x0027C0FD, 0x0028BCF8, 0x0025B7F1, 0x002FB6FF, 0x0021AAFF, 0x002DB1FF, 0x0033A2E8, 0x000E5F8C, 0x0002344B, 0x001E3D4C, 0x002C4352, 0x00004C4F, 0x00053E47, 0x000D3D49, 0x001B5665, 0x00296779, 0x00113F57, 0x000C334F, 0x0006324F, 0x00053B46,
        0x00083C49, 0x00043849, 0x00093E53, 0x00003851, 0x0000324E, 0x00004261, 0x00004260, 0x00094E5F, 0x00125062, 0x00033D50, 0x00094759, 0x00004356, 0x00004255, 0x00024457, 0x000D485C, 0x00084860, 0x00044659, 0x00014251, 0x0001404E, 0x0004414F,
        0x00074252, 0x00053D50, 0x0002374C, 0x0012434D, 0x00103F47, 0x000C3B43, 0x000A3941, 0x000B3940, 0x000C3A41, 0x000E3B3F, 0x000D3A3E, 0x00083B3E, 0x00205558, 0x00377072, 0x002C6566, 0x00265F60, 0x00074140, 0x00063C3C, 0x000E4242, 0x000D3E3C,
        0x000F403E, 0x00104041, 0x000D3F3F, 0x000A3B3D, 0x00073A3C, 0x00093C3E, 0x000C3F42, 0x0005373D, 0x00275354, 0x00517470, 0x0047635D, 0x0044605A, 0x001C3F3B, 0x00123E3F, 0x003F7177, 0x004F8D97, 0x000A4449, 0x00083C3C, 0x00194845, 0x000D3C3A,
        0x000D3D41, 0x00356A74, 0x00578E9D, 0x0010667C, 0x001A6D82, 0x000B5769, 0x00003A47, 0x00003B44, 0x00003336, 0x000C3839, 0x00123A39, 0x00093B47, 0x00083941, 0x000F3C3F, 0x000E3A3B, 0x000E3E40, 0x00053A44, 0x001E5A6A, 0x0017586D, 0x000C5F75,
        0x0014647B, 0x001B687B, 0x00165E6F, 0x00195F6C, 0x0015565F, 0x001B595F, 0x0000373D, 0x000A3C3C, 0x00093B3B, 0x000A3C3C, 0x00083C3C, 0x00073B3B, 0x00063C3C, 0x000B4141, 0x00104646, 0x00093D49, 0x000F4350, 0x00003848, 0x00004359, 0x000B6C88,
        0x002796BC, 0x001C8EBD, 0x002696CA, 0x002E88A0, 0x00004861, 0x00004A63, 0x00166E8C, 0x0035A1C3, 0x001D96C0, 0x00108CC2, 0x001690CC, 0x00095B7E, 0x0025313D, 0x003F3B30, 0x002C3C2B, 0x0034382C, 0x0045412F, 0x00373D2C, 0x00353632, 0x00383E33,
        0x0041473C, 0x003F473C, 0x003F473C, 0x00474F44, 0x00444E42, 0x00475145, 0x00485246, 0x003C4439, 0x003F473C, 0x003B4338, 0x003C4439, 0x0040483D, 0x00434B40, 0x00474F44, 0x0041493E, 0x00383D2E, 0x003A3F30, 0x002F3425, 0x00303526, 0x0034392A,
        0x0034392A, 0x00333829, 0x0034392A, 0x0036352B, 0x0038372D, 0x0038372D, 0x0034352B, 0x0035362C, 0x00303429, 0x003A3E33, 0x004A4E43, 0x00454D42, 0x003E463B, 0x00424A3F, 0x00343C31, 0x00353D32, 0x00424A3F, 0x00323A2F, 0x002D352A, 0x00313829,
        0x003A372F, 0x0034302B, 0x002F352A, 0x002C3221, 0x00313125, 0x00152728, 0x000E3B48, 0x00074654, 0x00074654, 0x00084656, 0x00074857, 0x0008485A, 0x0006485A, 0x0006485A, 0x0006485B, 0x0008475C, 0x000D475D, 0x000E475C, 0x000A4C5E, 0x00004253,
        0x003E8593, 0x00336A79, 0x003C6476, 0x001A6067, 0x00287077, 0x001E4E5A, 0x004B7A88, 0x0050A0AB, 0x00478E9C, 0x002D4C61, 0x00415971, 0x003D5571, 0x0039506A, 0x003D677A, 0x00468C99, 0x00004651, 0x00044554, 0x00094257, 0x00014257, 0x0002415D,
        0x0006414B, 0x00095C82, 0x002EADEA, 0x00065D89, 0x0007455D, 0x00005E63, 0x00438279, 0x002B3535, 0x00000200, 0x000C0E08, 0x00222117, 0x00242217, 0x0028281C, 0x0028291F, 0x001B1E15, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFEFF, 0xFFFDFCFE, 0xFFFBFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00191E1C, 0x00151914, 0x000E0906, 0x000A110A, 0x00111712, 0x001C1E1E, 0x0032615F, 0x000A615E, 0x00074E52, 0x00165056, 0x000F4A53, 0x000B4854, 0x00004150, 0x00014054,
        0x00044058, 0x000D4361, 0x00083B5B, 0x0006435D, 0x00043F53, 0x00043E4A, 0x000E454C, 0x0012484F, 0x000A3F4C, 0x00063D52, 0x000E4560, 0x0008405B, 0x0008405B, 0x0008405B, 0x0008405B, 0x0009415C, 0x0009415C, 0x0009415C, 0x0009415C, 0x00113E5F,
        0x00114361, 0x00124B64, 0x00064559, 0x00014453, 0x000A4D56, 0x000D4D52, 0x000B484C, 0x000F4C56, 0x0000333F, 0x0008424E, 0x001A5156, 0x001E5455, 0x00236872, 0x000E6988, 0x000E7BAE, 0x002EBAFD, 0x0026B5F9, 0x0026BBFE, 0x001EB7FA, 0x001BB7F9,
        0x0023BDFE, 0x0022B7F7, 0x0028BBF9, 0x0018B3F8, 0x002EC3FF, 0x002BADE9, 0x00005883, 0x00003A51, 0x000A4C57, 0x00003F4D, 0x00004559, 0x00004A5B, 0x0000394E, 0x00053950, 0x00125E74, 0x001A6D83, 0x00024058, 0x00093A54, 0x00093F56, 0x00003E49,
        0x00013E4A, 0x00063F4E, 0x0009404F, 0x000D4256, 0x00114C60, 0x004E8EA6, 0x002E718A, 0x00327E94, 0x0044879E, 0x0010495E, 0x00074255, 0x00004455, 0x00004656, 0x00004457, 0x00043D52, 0x0006485B, 0x00024456, 0x0000444E, 0x0003474E, 0x00064A51,
        0x00074752, 0x00034153, 0x00013E52, 0x00024050, 0x00033F4F, 0x0003404E, 0x0003404E, 0x0004404C, 0x0004404C, 0x00053F4B, 0x0005404A, 0x000B454B, 0x003A7178, 0x00689EA5, 0x00679BA1, 0x00598D93, 0x00164D50, 0x0000383A, 0x000A4547, 0x000B4B4B,
        0x00064646, 0x00004140, 0x00003F40, 0x00004243, 0x00004346, 0x00004144, 0x00003E41, 0x00003C46, 0x001D575D, 0x00588183, 0x0064817E, 0x006C8986, 0x00295254, 0x000D474D, 0x004C939D, 0x005FB7C7, 0x00287A85, 0x00003C40, 0x00074747, 0x000E494B,
        0x0006414A, 0x001D5B6D, 0x00387890, 0x003C8D9C, 0x00368494, 0x00347F8D, 0x00004553, 0x00014251, 0x00003C4C, 0x00053E4D, 0x0007404F, 0x00004154, 0x00003E4E, 0x0008434C, 0x00064046, 0x0009474F, 0x00024758, 0x0024718B, 0x00217392, 0x000A7198,
        0x001C7DA1, 0x002883A2, 0x00247590, 0x003F869A, 0x00387785, 0x00558C95, 0x00255960, 0x000B484A, 0x00003E3F, 0x00003B3C, 0x00003C3D, 0x00003E3F, 0x00064748, 0x00094C4D, 0x00024546, 0x000B414C, 0x00083E4F, 0x00054460, 0x001C6E91, 0x002791BA,
        0x002297C4, 0x001F90C2, 0x00248ABE, 0x00004D5E, 0x001D6271, 0x00276470, 0x00024855, 0x00177386, 0x002594B4, 0x00259CCD, 0x000F87C3, 0x00095B7E, 0x0025313D, 0x003F3B30, 0x0030402F, 0x00363A2E, 0x0045412F, 0x00353B2A, 0x003A3B37, 0x00353D33,
        0x003B4339, 0x0041493F, 0x003E473D, 0x0040493F, 0x003F4A40, 0x003B463C, 0x003C473D, 0x00384236, 0x003C463A, 0x00384236, 0x003B4539, 0x003E483C, 0x003D473B, 0x00434D41, 0x003F493D, 0x00424738, 0x00464B3C, 0x00313627, 0x00373C2D, 0x002F3425,
        0x00333829, 0x002E3324, 0x00313627, 0x0038372D, 0x003A392F, 0x0035342A, 0x0038392F, 0x0035362C, 0x0035392E, 0x00383C31, 0x00494D42, 0x00464E43, 0x003B4338, 0x00444C41, 0x00363E33, 0x00373F34, 0x00424A3F, 0x002E362B, 0x002E362B, 0x002E3526,
        0x0037342C, 0x0037332E, 0x002E3429, 0x00323827, 0x00323226, 0x00182A2B, 0x000C3946, 0x00064553, 0x00074654, 0x00094856, 0x00094A59, 0x000A4B5A, 0x0007495B, 0x0006485A, 0x00054759, 0x0009485D, 0x00094359, 0x00043D52, 0x00004254, 0x00003F50,
        0x00408795, 0x00356C7B, 0x00335B6D, 0x006BAFB6, 0x0064ACB3, 0x003E6B78, 0x00507F8D, 0x004B9BA6, 0x00438897, 0x002B4A5F, 0x00415770, 0x003C5470, 0x0039506A, 0x003D677A, 0x00488E9B, 0x00004853, 0x00054655, 0x00094257, 0x00004156, 0x0001405C,
        0x0006414B, 0x00085B81, 0x002DACE9, 0x00075E8A, 0x0007455D, 0x00005D62, 0x0044837A, 0x00404A4A, 0x00000200, 0x00080A04, 0x001E1D13, 0x00242217, 0x002A2A1E, 0x0028291F, 0x001D2017, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFEFF, 0xFFFDFCFE, 0xFFFBFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00191E1C, 0x00141813, 0x000F0A07, 0x00070E07, 0x000D130E, 0x00191B1B, 0x002E5D5B, 0x00045B58, 0x00044B4F, 0x000D4152, 0x00093E52, 0x000B4459, 0x00023E54, 0x000C4963,
        0x00023C59, 0x00063D5C, 0x00053B5A, 0x000B3A5A, 0x000C415C, 0x00063F55, 0x00023D50, 0x00094457, 0x000E445B, 0x000E3E5A, 0x00123D5E, 0x0008405B, 0x0008405B, 0x0008405B, 0x0008405B, 0x0009415C, 0x0009415C, 0x0009415C, 0x0009415C, 0x00083E5F,
        0x00083F5E, 0x00074059, 0x00073F52, 0x0012414F, 0x00466A72, 0x0081999F, 0x008B9C9F, 0x007EAFB9, 0x002B5A68, 0x00315B68, 0x00628085, 0x0094ABA7, 0x00A8C4C4, 0x00376374, 0x00003959, 0x000868A4, 0x002A8FCD, 0x002997D7, 0x002096D7, 0x001E9ADA,
        0x001591D1, 0x001994D2, 0x0028A2DE, 0x0037ADD6, 0x002C97BD, 0x00085E7C, 0x00044252, 0x00154547, 0x00134649, 0x0003495A, 0x00003E5E, 0x0000405D, 0x00163C5E, 0x00234C6C, 0x00307C92, 0x002D8B98, 0x000A555E, 0x0009484C, 0x00004544, 0x000A4359,
        0x0002384F, 0x000D4459, 0x00063F54, 0x00054458, 0x00003E54, 0x0035889E, 0x000D6780, 0x0014667F, 0x003B7E97, 0x00053E54, 0x00054053, 0x0003495A, 0x00004757, 0x00034859, 0x000C455A, 0x00004053, 0x00094D5A, 0x001A5F68, 0x00286F73, 0x00256C70,
        0x00145962, 0x00044756, 0x00003F52, 0x00023B50, 0x00033C51, 0x00073E53, 0x00084053, 0x00084053, 0x00093F50, 0x00083E4F, 0x00073D4E, 0x0013404B, 0x002C5460, 0x0040626F, 0x00395560, 0x003B5762, 0x0012363E, 0x00275158, 0x005C8A91, 0x00357274,
        0x001E5B5D, 0x000D4A4C, 0x0019585C, 0x0039787C, 0x0049898E, 0x003D7D82, 0x0029696E, 0x00146776, 0x00004049, 0x00063336, 0x00203C3C, 0x005E7A7A, 0x00356265, 0x00165962, 0x0058ABBA, 0x00076277, 0x0058AABB, 0x00357D85, 0x002A676B, 0x00235960,
        0x00093D4D, 0x00073D56, 0x00144B6A, 0x007C9290, 0x00637D7D, 0x00708F92, 0x001E454D, 0x0012404B, 0x000B4251, 0x00023D50, 0x00003E52, 0x00003F58, 0x00033E52, 0x000C4352, 0x000A3E4A, 0x00094251, 0x00004055, 0x00236C8C, 0x001D6C93, 0x00197098,
        0x002B7CA1, 0x00337A9B, 0x002B647D, 0x004D798A, 0x0042636C, 0x006B8284, 0x00324443, 0x00084045, 0x0010484D, 0x0031696E, 0x004E888D, 0x00579196, 0x00518E92, 0x00367377, 0x000D4A4E, 0x00114752, 0x00063D52, 0x0019597B, 0x00388FBB, 0x002291C3,
        0x00118ABC, 0x002190C2, 0x00005486, 0x002B7686, 0x0072AFB9, 0x008EC3C6, 0x004C8888, 0x0022737B, 0x000F768F, 0x002093C0, 0x00188DCA, 0x00005275, 0x001B2733, 0x00373328, 0x0030402F, 0x00363A2E, 0x00403C2A, 0x002E3423, 0x00373834, 0x003D453B,
        0x003D453B, 0x00454E44, 0x003D463C, 0x00374238, 0x003B463C, 0x00333E34, 0x00324035, 0x003D473B, 0x00414B3F, 0x003C463A, 0x00414B3F, 0x00404A3E, 0x003B4539, 0x00414B3F, 0x003D473B, 0x00454A3B, 0x004B5041, 0x002F3425, 0x00383D2E, 0x002B3021,
        0x00363B2C, 0x00333829, 0x003A3F30, 0x003A392F, 0x003B3A30, 0x00323127, 0x003B3C32, 0x0037382E, 0x00393D32, 0x00363A2F, 0x0045493E, 0x0040483D, 0x00333B30, 0x0041493E, 0x003B4338, 0x003D453A, 0x00424A3F, 0x002B3328, 0x002F372C, 0x002F3627,
        0x00343129, 0x0035312C, 0x00282E23, 0x00313726, 0x00303024, 0x001A2C2D, 0x000D3A47, 0x00054452, 0x00074654, 0x000B4A58, 0x000B4C5B, 0x000C4D5C, 0x00094C5B, 0x0006485A, 0x00044658, 0x0006455A, 0x000D475D, 0x000C455A, 0x00054759, 0x00003E4F,
        0x003E8593, 0x00376E7D, 0x00375F71, 0x0065A9B0, 0x005BA3AA, 0x00315E6B, 0x00437280, 0x004696A1, 0x00448998, 0x002B4A5F, 0x0040566F, 0x003C5470, 0x0039506A, 0x003E687B, 0x00498F9C, 0x00004954, 0x00054655, 0x00094257, 0x00004156, 0x0001405C,
        0x0006414B, 0x00085B81, 0x002DACE9, 0x00075E8A, 0x0006445C, 0x00005D62, 0x0045847B, 0x00535D5D, 0x00000502, 0x00050701, 0x001B1A10, 0x00232116, 0x002A2A1E, 0x0026271D, 0x001B1E15, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFDFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00181D1B, 0x00101611, 0x00010D07, 0x00080C06, 0x001F1613, 0x000F1413, 0x002D5554, 0x00125256, 0x00084C53, 0x000D4556, 0x00094356, 0x00034054, 0x00003F54, 0x0003435B,
        0x00014257, 0x00003F52, 0x00064658, 0x00073F5A, 0x000B4660, 0x0009445E, 0x0008445C, 0x00074359, 0x00003F54, 0x00014054, 0x0009485C, 0x00044358, 0x0008445A, 0x00053E57, 0x00084059, 0x000D4560, 0x0008435D, 0x00033F5C, 0x0002415D, 0x000C4156,
        0x000B425D, 0x000B4162, 0x00063A5E, 0x00133D5A, 0x00355562, 0x0096A9A6, 0x00A8B3A3, 0x008DA599, 0x0055838E, 0x00487C8D, 0x0077908C, 0x009C9D89, 0x008E968B, 0x002A5055, 0x00003B47, 0x0004414D, 0x00004155, 0x000F6180, 0x00116686, 0x00003853,
        0x000C3E4A, 0x0089A29E, 0x00DEE8DB, 0x00D6E4D9, 0x00CBE0CA, 0x002E7075, 0x00004F6C, 0x001C7883, 0x00055052, 0x00054759, 0x000F475A, 0x0000395C, 0x00174452, 0x00B0C7C9, 0x00D7EAED, 0x00D6E8E9, 0x00CBE0DD, 0x00325A5F, 0x00003D55, 0x0000414B,
        0x000B4959, 0x00094257, 0x00073E59, 0x00074060, 0x00003B5E, 0x00257398, 0x00004E72, 0x00155574, 0x00236A7E, 0x00004956, 0x00004757, 0x0000465F, 0x00044B66, 0x00034250, 0x001C5556, 0x00105A66, 0x0021555B, 0x00BFD4D2, 0x00E8E8E2, 0x00EBEAE6,
        0x00B1C7CC, 0x000A4150, 0x0000475E, 0x00054054, 0x00054054, 0x00064155, 0x00064155, 0x00054054, 0x00054054, 0x00043F53, 0x00043F53, 0x0006444A, 0x000B444D, 0x0000343F, 0x0000313B, 0x00003E43, 0x00074244, 0x002C5453, 0x009BB1AF, 0x0096958B,
        0x006F918B, 0x00518B86, 0x006E998C, 0x00909E8C, 0x009DB4B0, 0x007BCEEA, 0x0027B2EA, 0x0045B6D7, 0x002A7EA1, 0x00003251, 0x000C4155, 0x0032737B, 0x00115953, 0x001D554A, 0x0075998D, 0x003C5557, 0x0091ACB0, 0x0067888B, 0x00678D91, 0x00235256,
        0x000F4349, 0x00074147, 0x0008434C, 0x000E394A, 0x000F3A4B, 0x000D3B4C, 0x000C3D4D, 0x000B3F4F, 0x000A4150, 0x00094251, 0x00084352, 0x00084352, 0x00034054, 0x00064258, 0x00084053, 0x000D424F, 0x00024050, 0x00237090, 0x000F6D98, 0x001A6F9D,
        0x001F77A0, 0x002D84A4, 0x00045067, 0x00003141, 0x00083F46, 0x00478787, 0x000B554F, 0x0005424C, 0x00004041, 0x007F9593, 0x00FFFDF9, 0x00FFFFF9, 0x00FFFFFB, 0x00A5B1B5, 0x00073E41, 0x00093B47, 0x00003E51, 0x000A5F7F, 0x003095BC, 0x002C93BA,
        0x00308DAC, 0x000E5E6F, 0x0000454E, 0x00C6D7C9, 0x00FAFFF0, 0x00F6F3E4, 0x00F8FFF8, 0x0074A09F, 0x00003C4B, 0x000A7596, 0x001B90BD, 0x00196795, 0x00103444, 0x003A3830, 0x00443B31, 0x003A3A2E, 0x00383925, 0x003B3C2C, 0x00343A35, 0x0038392F,
        0x0036372D, 0x003A3B31, 0x00393A30, 0x0036372D, 0x0037382E, 0x0037382E, 0x00393A30, 0x00363A2F, 0x00383C31, 0x0035362C, 0x0037382E, 0x0037382E, 0x0036352B, 0x003A392F, 0x0037362C, 0x0035362C, 0x00313228, 0x0034352B, 0x0035362C, 0x0033342A,
        0x0036372D, 0x003A3B31, 0x0035362C, 0x002F3328, 0x0034382D, 0x0034382D, 0x00353B30, 0x002F352A, 0x002C3429, 0x002E362B, 0x00373F34, 0x00333B30, 0x0030382D, 0x0030382D, 0x00343A2F, 0x00343A2F, 0x00363A2F, 0x00363A2F, 0x0035392E, 0x00373A2A,
        0x0032362B, 0x0030332A, 0x00323025, 0x00353525, 0x00243024, 0x000A2C2C, 0x000B3F4C, 0x00104952, 0x0000414A, 0x00054F5B, 0x00004C5C, 0x00004A5A, 0x00094F60, 0x00074255, 0x000E4455, 0x00194C60, 0x00084053, 0x0006485A, 0x00004554, 0x0000404E,
        0x003B8696, 0x003C697E, 0x0041556E, 0x003E5771, 0x002D5169, 0x0013485D, 0x003E7D91, 0x005C9BAF, 0x004F8499, 0x00294D65, 0x003B546E, 0x003D5474, 0x0035526D, 0x00376076, 0x00538B9C, 0x00004652, 0x00004B58, 0x00003F51, 0x00004559, 0x00114250,
        0x0000395B, 0x000065A0, 0x0035ACEA, 0x00055A8C, 0x000E3D58, 0x001C5C5D, 0x001E8979, 0x004A4E4F, 0x00010604, 0x00080C06, 0x001B1F14, 0x001D2115, 0x0024261A, 0x002D2C22, 0x00201E16, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFEFDFF, 0xFFFEFDFF, 0xFFFDFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00181D1B, 0x0010110D, 0x00000802, 0x00090D07, 0x00211A17, 0x00212726, 0x003F6968, 0x000F5253, 0x0000464A, 0x000F3E53, 0x000F4056, 0x000C4057, 0x00084059, 0x000B435E,
        0x0006415B, 0x00003D55, 0x00034159, 0x00043D56, 0x00043D56, 0x0000364F, 0x00033B54, 0x000D435C, 0x000E445D, 0x000E415B, 0x00073A54, 0x000F3F5B, 0x0011415D, 0x000C3C58, 0x000B3D59, 0x000D425D, 0x0009405B, 0x0009445E, 0x0006435D, 0x000D4158,
        0x000A415C, 0x00053E5E, 0x00073C5D, 0x00123E56, 0x001D3842, 0x0066706A, 0x007D7D6D, 0x00354F42, 0x000F3D48, 0x00093748, 0x003D514C, 0x00857F6C, 0x00565C51, 0x00183E42, 0x0003404A, 0x00033F4F, 0x00003E53, 0x001B627D, 0x0029728E, 0x00003F54,
        0x000E434D, 0x00719896, 0x00B7D3C6, 0x00B2D1D4, 0x00B2D1C8, 0x007DBFD2, 0x0067C5F0, 0x0065B5CC, 0x00276674, 0x0001415A, 0x00033B54, 0x0000475E, 0x001A4D50, 0x0095ABA6, 0x00BBC9C5, 0x00C4D0CA, 0x00B4CABE, 0x00285A56, 0x0000414F, 0x001B475F,
        0x00093B52, 0x00054054, 0x00074759, 0x00033F4B, 0x0014484F, 0x0082A8AC, 0x005E7D80, 0x004D798A, 0x0082B9C2, 0x0005474C, 0x00004755, 0x00003D58, 0x0000425F, 0x00033F55, 0x004A8692, 0x0071B1B6, 0x00598B8B, 0x00A8C8BD, 0x00B6C9B8, 0x00BFD2C3,
        0x008DAFA9, 0x00134950, 0x00004251, 0x00054054, 0x00054054, 0x00064155, 0x00064155, 0x00064155, 0x00054054, 0x00043F53, 0x00043F53, 0x00013C4F, 0x00073C51, 0x000A3F54, 0x00053E53, 0x00003D4D, 0x000A454F, 0x00285259, 0x0071898F, 0x00838D8D,
        0x0089A9AE, 0x00619094, 0x0086A9A5, 0x00849489, 0x00698585, 0x007FC8E6, 0x0034A9E0, 0x004BB3D6, 0x003B89AD, 0x000C3F5F, 0x000A3B51, 0x000A454F, 0x00003D3D, 0x002E5C56, 0x004F6B65, 0x002F4A46, 0x00789592, 0x00274B4B, 0x00204A4F, 0x000D3E46,
        0x000D4451, 0x00033F4F, 0x00004052, 0x000D3B4C, 0x000D3B4C, 0x000C3D4D, 0x000A3E4E, 0x0009404F, 0x00064150, 0x00064351, 0x00074452, 0x00074251, 0x00023F53, 0x00054157, 0x00084053, 0x000D424F, 0x00034151, 0x00227291, 0x0012709B, 0x00126D9A,
        0x001C79A0, 0x00328BA6, 0x00004759, 0x0009414C, 0x0043777D, 0x00629FA1, 0x00054D4D, 0x00094555, 0x0000363F, 0x00909CA2, 0x00E9D9DA, 0x00B7B7B1, 0x00AFB4B2, 0x00889194, 0x00396062, 0x00505D6B, 0x0041596D, 0x003D6480, 0x004B80A1, 0x00457B9C,
        0x0051829C, 0x00456B7D, 0x0048646F, 0x0091A6A4, 0x00C0C7C4, 0x00B9B3AE, 0x00C2BFBA, 0x00708583, 0x00073540, 0x000C4A62, 0x00286D8E, 0x001C6388, 0x0015363F, 0x003B392E, 0x003E382D, 0x0038392F, 0x00393B28, 0x003A3C29, 0x00323630, 0x00363A2F,
        0x00393D32, 0x00373B30, 0x00383C31, 0x00393D32, 0x00383C31, 0x00393D32, 0x003A3E33, 0x003A4035, 0x003C4237, 0x00373D32, 0x00393D32, 0x00383C31, 0x0036372D, 0x003A3B31, 0x0038392F, 0x003B3F34, 0x0035392E, 0x0035392E, 0x00373B30, 0x0034382D,
        0x00363A2F, 0x00373B30, 0x0034382D, 0x00373B30, 0x0031352A, 0x0032362B, 0x0032382D, 0x00363C31, 0x00323A2F, 0x00343C31, 0x00333B30, 0x00323A2F, 0x00333B30, 0x0031392E, 0x0031372C, 0x0033392E, 0x0034382D, 0x0031352A, 0x00303429, 0x00373A2A,
        0x0032362B, 0x0030332A, 0x00333126, 0x00363626, 0x00243024, 0x000B2D2D, 0x000C404D, 0x00094652, 0x00114E5C, 0x00054353, 0x00054557, 0x00115064, 0x0006485B, 0x00014355, 0x000B5061, 0x00003C4F, 0x0019566A, 0x00074256, 0x00044456, 0x00054A5B,
        0x00397A89, 0x003C6C7E, 0x002B4A5F, 0x00385068, 0x0034546B, 0x00224F64, 0x0044778B, 0x0056899D, 0x004E7B90, 0x00305067, 0x003C546C, 0x003C4B6C, 0x00385671, 0x002D5F71, 0x00488893, 0x0000444E, 0x00044252, 0x000A4359, 0x000E435E, 0x00003E54,
        0x00016588, 0x002AA0D3, 0x002B95CA, 0x00004A71, 0x000A4E61, 0x000C5857, 0x00358E85, 0x00383C3D, 0x00000100, 0x00050903, 0x001B1F14, 0x00202418, 0x0026281C, 0x002B2A20, 0x001E1C14, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFDFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00181D1B, 0x00161211, 0x00020904, 0x000C100B, 0x0015100D, 0x00182320, 0x003C6B68, 0x0008514F, 0x00044E50, 0x00144058, 0x0014415C, 0x000E3D58, 0x000B3B57, 0x000C3E5C,
        0x00083E5C, 0x00073E5D, 0x00083F5E, 0x00094258, 0x000F4760, 0x000D435C, 0x000E415B, 0x000A3C58, 0x000B3B57, 0x00154362, 0x00134160, 0x00103D5E, 0x00113F5E, 0x000E3D5C, 0x000F415F, 0x000D425D, 0x00063B56, 0x0009405B, 0x00043C55, 0x000C3D57,
        0x00093F5D, 0x00033F5D, 0x0009415E, 0x00174257, 0x00172D33, 0x0054524A, 0x007A6A59, 0x000E2C21, 0x0007353D, 0x00002A37, 0x002B3731, 0x00867C6A, 0x00353A31, 0x0013393D, 0x00034149, 0x0009425B, 0x00074056, 0x002C6378, 0x0042798E, 0x00003A4A,
        0x00033E47, 0x00599291, 0x009CD1C8, 0x0086C4C4, 0x006A9C92, 0x0034798C, 0x00186E98, 0x002B6F82, 0x00215B61, 0x00003D4C, 0x00044554, 0x00004053, 0x00194A48, 0x0072998A, 0x00A1C9BD, 0x00A3CCBD, 0x007EA694, 0x001A4F46, 0x00004350, 0x000E3B5C,
        0x000D4060, 0x0004405D, 0x00004259, 0x00024050, 0x000E4249, 0x0099BFBF, 0x0099B7B2, 0x00A0C0BF, 0x009BC6C3, 0x000B4443, 0x0003414B, 0x00104E66, 0x00064260, 0x00023E5B, 0x00145368, 0x00255D62, 0x001E544F, 0x0072A694, 0x009BCEB3, 0x00A5D7BF,
        0x005D9284, 0x00124849, 0x00083E49, 0x00064155, 0x00064155, 0x00064155, 0x00064155, 0x00064155, 0x00054054, 0x00054054, 0x00043F53, 0x00043D5C, 0x00073A5A, 0x000E3F5F, 0x0008405D, 0x00003C54, 0x00043F52, 0x001B4855, 0x004F6F7A, 0x006B888F,
        0x00507180, 0x00446574, 0x00556F75, 0x00485F5B, 0x003B6163, 0x004789A2, 0x002E86B5, 0x0041A2C4, 0x00095677, 0x00033958, 0x00114159, 0x000B4251, 0x00094047, 0x00264C50, 0x00586F71, 0x00476E66, 0x00618984, 0x001E4A4B, 0x0034646A, 0x001E5360,
        0x00074255, 0x00003D55, 0x00003C58, 0x000B3F4F, 0x000B3F4F, 0x0009404F, 0x00084150, 0x00074251, 0x00064351, 0x00054452, 0x00034452, 0x000A4554, 0x00054256, 0x00074359, 0x00094154, 0x000B424F, 0x00013F4F, 0x00217190, 0x0012709B, 0x001576A2,
        0x001B7C9E, 0x002C8397, 0x002F747E, 0x0007393F, 0x0058878B, 0x003D777D, 0x00003E46, 0x0000455A, 0x000F5D6D, 0x007E9CA7, 0x00889397, 0x0088B1AA, 0x00B8E7DE, 0x00B6DAD4, 0x00A8DBD4, 0x00ACD5D8, 0x00A5D3DA, 0x009AD1DA, 0x009CDAE4, 0x0093D4DD,
        0x00A3E1E9, 0x00A1D9DE, 0x00A8DEDF, 0x00A6DDE2, 0x00D9FEFF, 0x00BFD3D4, 0x008B9B9A, 0x00486562, 0x001A4548, 0x000C3F49, 0x00104451, 0x000C4B60, 0x0018302E, 0x00403B2C, 0x003B362D, 0x00333731, 0x00353B2A, 0x00383A27, 0x0035362C, 0x003A4237,
        0x00434B40, 0x00394136, 0x003B4338, 0x0041493E, 0x003B4338, 0x003D453A, 0x003D453A, 0x003C463A, 0x00414B3F, 0x0040483D, 0x00424A3F, 0x0040483D, 0x003C4237, 0x003F453A, 0x003D4338, 0x00434B40, 0x003B4338, 0x003A4237, 0x003F473C, 0x0041493E,
        0x0040483D, 0x0041493E, 0x0040483D, 0x00464A3F, 0x0034382D, 0x00383C31, 0x0032382D, 0x003D4338, 0x00384035, 0x00434B40, 0x003B4338, 0x00384035, 0x0040483D, 0x003C4439, 0x00393F34, 0x003E4439, 0x00404439, 0x00383C31, 0x00363A2F, 0x00373A2A,
        0x0032362B, 0x0031342B, 0x00343227, 0x00373727, 0x00253125, 0x000B2D2D, 0x000C404D, 0x00003E52, 0x00144F63, 0x00063E51, 0x00154D60, 0x00144E61, 0x00003E50, 0x00085363, 0x00004C5C, 0x00004F5F, 0x00004456, 0x00144B60, 0x0015485C, 0x00023A4B,
        0x00377282, 0x00396D7D, 0x002A5667, 0x002C4C5F, 0x00315265, 0x00224A5C, 0x002D5869, 0x002F5A6B, 0x00345C6E, 0x002A4B5E, 0x00305063, 0x00374667, 0x00244A62, 0x00266671, 0x00428B8F, 0x0009474F, 0x00124353, 0x000D3C58, 0x00073A5B, 0x0000597F,
        0x0032ACD4, 0x002098C2, 0x00004D75, 0x00003D58, 0x0002575F, 0x00055D5D, 0x002F7777, 0x002D3132, 0x00000100, 0x00050903, 0x001A1E13, 0x0022261A, 0x0027291D, 0x002A291F, 0x00201E16, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFDFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00171C1A, 0x00170F0F, 0x00010604, 0x000E120D, 0x00100D09, 0x001A2522, 0x003D6D67, 0x00084D4A, 0x00064C4B, 0x00033C55, 0x00084059, 0x00073D54, 0x000B3E58, 0x000E405C,
        0x000D3E5E, 0x000B3D61, 0x0004385D, 0x000A4057, 0x000A4057, 0x00073D54, 0x00084059, 0x00073F58, 0x00053D58, 0x000B435E, 0x00063E59, 0x00114966, 0x0008425F, 0x00053F5C, 0x0007425C, 0x000B4660, 0x00063E57, 0x00134660, 0x0012435D, 0x000D3D59,
        0x000A425F, 0x0006425F, 0x00033F57, 0x000E3C4D, 0x00192F34, 0x0057544C, 0x00857061, 0x0003251E, 0x00103F47, 0x000B313D, 0x002B322D, 0x007E7567, 0x003B423D, 0x001D454A, 0x0007454B, 0x00003455, 0x000E3E56, 0x00487180, 0x0069939F, 0x00114551,
        0x00003842, 0x003A8282, 0x007DC6BE, 0x006AC3B3, 0x0042896F, 0x00003A3D, 0x00003854, 0x0037767A, 0x0050887D, 0x00054F4B, 0x0004524B, 0x00184059, 0x000F3C3F, 0x00448274, 0x0076CEBE, 0x006BC4B4, 0x00367C6B, 0x000A4040, 0x00114359, 0x00094650,
        0x00114A59, 0x000B4054, 0x000C4156, 0x0013485C, 0x000E4A56, 0x008FD2D5, 0x0097DDDC, 0x00B6DDD5, 0x007CAFA8, 0x00135250, 0x0007464E, 0x00094454, 0x00023B50, 0x00074458, 0x00004254, 0x00063E49, 0x00003737, 0x00489385, 0x0079CCB3, 0x007ECFB6,
        0x002A7265, 0x00003A3C, 0x000C404D, 0x00044155, 0x00044155, 0x00054256, 0x00054256, 0x00044155, 0x00044155, 0x00034054, 0x00034054, 0x0004405D, 0x00073D5C, 0x00083959, 0x00043C59, 0x0001445B, 0x00003D4C, 0x000A424D, 0x00457079, 0x003B6F76,
        0x00001725, 0x00001B2A, 0x00001C23, 0x0000211D, 0x000A3C3A, 0x00003341, 0x00064665, 0x003293B4, 0x00035475, 0x00003A59, 0x00003B55, 0x00064559, 0x00054250, 0x001F4F5B, 0x007395A2, 0x00195456, 0x00124C51, 0x001B545D, 0x004B8591, 0x00124D5D,
        0x00003549, 0x0008445A, 0x00054159, 0x00074252, 0x00074252, 0x00064252, 0x00044252, 0x00054353, 0x00034453, 0x00014453, 0x00014453, 0x00094654, 0x00014054, 0x00044358, 0x00084255, 0x000C4350, 0x00004150, 0x00247493, 0x001675A0, 0x000B7096,
        0x00147793, 0x00308692, 0x005E9E9F, 0x000F3B3C, 0x003D676C, 0x00164D5A, 0x00004556, 0x00278EA9, 0x00368BA1, 0x00628396, 0x0070848F, 0x009BD7D1, 0x009EE0D4, 0x0090B9B1, 0x00A3CDC2, 0x00B3CDCD, 0x00B8D3D0, 0x00B1CFCA, 0x00B2D0CB, 0x00ABCBC6,
        0x00B3D5CF, 0x00ACCDC9, 0x00ADCDCC, 0x0088C0C1, 0x00A3CCCE, 0x009BB6BA, 0x00738E92, 0x002B5457, 0x00094043, 0x00054147, 0x00074049, 0x00002D3A, 0x0012271E, 0x00443E2B, 0x003B392F, 0x00303732, 0x00313829, 0x00373926, 0x003A3A2E, 0x003A4438,
        0x00475145, 0x00353F33, 0x00394337, 0x00424C40, 0x00374135, 0x003A4438, 0x00394337, 0x002F3B2F, 0x00384438, 0x003B473B, 0x00434D41, 0x00414B3F, 0x003B4338, 0x003C4439, 0x00394136, 0x003A4438, 0x00303A2E, 0x002F392D, 0x00374135, 0x003B4539,
        0x00374135, 0x00353F33, 0x00364034, 0x0043473C, 0x00303429, 0x003C4035, 0x0032382D, 0x003D4338, 0x00333B30, 0x00454D42, 0x003B4338, 0x00323A2F, 0x0040483D, 0x003D453A, 0x00393F34, 0x0041473C, 0x0042463B, 0x00363A2F, 0x0033372C, 0x00363929,
        0x0032362B, 0x0031342B, 0x00343227, 0x00383828, 0x00263226, 0x000C2E2E, 0x000D414E, 0x000B475F, 0x00024156, 0x00115365, 0x00084E5B, 0x00044C56, 0x0009545D, 0x00004A56, 0x000B5763, 0x00015763, 0x00034C5A, 0x00033F4F, 0x00124F5D, 0x0000424F,
        0x00307683, 0x006CA9B7, 0x00548797, 0x005C8D9B, 0x00608F9D, 0x00558492, 0x0054818F, 0x004A7785, 0x00548391, 0x00558492, 0x00598A98, 0x005F7D9A, 0x0053869A, 0x00489098, 0x0034797C, 0x00083F48, 0x0015485C, 0x0000405D, 0x00085E82, 0x0035A2D5,
        0x002B96C2, 0x00005176, 0x00003752, 0x000D505F, 0x00004D4F, 0x00055B5B, 0x00368082, 0x00313536, 0x00000200, 0x00070B05, 0x00171B10, 0x00202418, 0x0026281C, 0x002A291F, 0x0024221A, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFDFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00171C1A, 0x0023181A, 0x00060907, 0x000D110C, 0x00120F0A, 0x00202A24, 0x00436D66, 0x0013514B, 0x000A4443, 0x00004759, 0x00044A5B, 0x00044252, 0x000B4453, 0x000D4054,
        0x00093C56, 0x000E4162, 0x00093D62, 0x00083E55, 0x000A4057, 0x000A4359, 0x0009465A, 0x00064559, 0x00034557, 0x00024758, 0x00004657, 0x0002475A, 0x00004153, 0x00004658, 0x00004357, 0x00084B60, 0x0006425A, 0x000F425C, 0x00113E59, 0x000D3F5B,
        0x000A425D, 0x00094660, 0x00023E54, 0x000C3D4D, 0x0027434A, 0x005D6362, 0x007E766F, 0x00426A68, 0x0040727E, 0x00325C68, 0x004A5A53, 0x0074756B, 0x004C5E5F, 0x001A4850, 0x00024046, 0x00073F62, 0x00123E55, 0x003C6068, 0x0053797E, 0x00053945,
        0x00003741, 0x00469094, 0x009BE8E1, 0x0083DFCE, 0x00539A80, 0x00004548, 0x0011617E, 0x004A878B, 0x00437B70, 0x0000423E, 0x00004A43, 0x00124062, 0x00073945, 0x004A928C, 0x0091F1EA, 0x007DE1DB, 0x00388B83, 0x00074954, 0x00003557, 0x00004347,
        0x00003E46, 0x00004554, 0x00094B5E, 0x00003C4B, 0x0000444B, 0x0083CDC9, 0x0090DCD0, 0x008DC7C6, 0x005CA2A2, 0x00003E44, 0x00054E56, 0x000C4751, 0x00134852, 0x00023C42, 0x00003F44, 0x00003E56, 0x00004652, 0x0059AEAA, 0x0083E0D1, 0x007FDBCA,
        0x002D7F7A, 0x00003B46, 0x000D435A, 0x00054256, 0x00054256, 0x00054256, 0x00054256, 0x00054256, 0x00044155, 0x00044155, 0x00034054, 0x00004051, 0x00043F53, 0x00073E53, 0x00034054, 0x00024859, 0x00003F48, 0x00004247, 0x002A6267, 0x002A7174,
        0x00013640, 0x00082E3A, 0x00163C41, 0x000A3F3C, 0x00044641, 0x0006444A, 0x000E4455, 0x00056785, 0x00328CAA, 0x000F5F7C, 0x00004C67, 0x00004D64, 0x0000485B, 0x00307285, 0x00548C9F, 0x00005166, 0x00065268, 0x002D798C, 0x003D8295, 0x00034355,
        0x00033F4F, 0x000E4756, 0x00013847, 0x00054353, 0x00054353, 0x00034453, 0x00034453, 0x00034453, 0x00034453, 0x00014453, 0x00014453, 0x00054250, 0x00003D51, 0x00024156, 0x00084255, 0x000B4551, 0x00034453, 0x00287A99, 0x001F7EA9, 0x0010708E,
        0x001F7D90, 0x002D7E85, 0x00266563, 0x00194342, 0x00315B60, 0x004A8190, 0x004C93A7, 0x00276E8F, 0x000A425F, 0x001A2540, 0x00585A6C, 0x00779EA0, 0x00658B85, 0x008A918A, 0x0093948B, 0x00868A85, 0x008B8B85, 0x00848278, 0x00898378, 0x00868075,
        0x00858379, 0x007A7C76, 0x00737974, 0x00758076, 0x00828681, 0x007B7C80, 0x00636F7B, 0x00204454, 0x00033B4E, 0x0005465B, 0x00003F54, 0x0000313E, 0x00152A21, 0x0044402D, 0x003C3A30, 0x002F3933, 0x00303A2A, 0x00373926, 0x0037392D, 0x003A4438,
        0x00475145, 0x00364034, 0x003A4438, 0x00434D41, 0x003A4438, 0x003D473B, 0x003C463A, 0x00303C30, 0x00384438, 0x003A463A, 0x00404A3E, 0x003E483C, 0x00394136, 0x003C4439, 0x00394136, 0x00434D41, 0x003C463A, 0x00394337, 0x00404A3E, 0x00434D41,
        0x003B4539, 0x00343E32, 0x00343E32, 0x0041453A, 0x00303429, 0x003F4338, 0x00383E33, 0x0043493E, 0x00373F34, 0x00454D42, 0x00373F34, 0x0030382D, 0x003F473C, 0x003D453A, 0x003A4035, 0x0040463B, 0x003F4338, 0x0031352A, 0x002C3025, 0x00353828,
        0x0031352A, 0x0031342B, 0x00353328, 0x00383828, 0x00263226, 0x000C2E2E, 0x000D414E, 0x000C455E, 0x00084C5F, 0x0000464E, 0x00005353, 0x002D8683, 0x0067B6B3, 0x008ACACB, 0x0097CED3, 0x0085C5C6, 0x006FACB0, 0x00327277, 0x00105A60, 0x00004147,
        0x00004850, 0x00327683, 0x00508494, 0x00468B95, 0x00498994, 0x00498692, 0x004C8692, 0x00427C88, 0x0045828E, 0x0044848F, 0x0040858F, 0x00498297, 0x00488998, 0x002C717B, 0x001A5862, 0x00054050, 0x00004159, 0x00005F7F, 0x002CADCE, 0x003797CC,
        0x0000406F, 0x00073A5B, 0x000A4559, 0x00054A54, 0x00074B52, 0x00105A5E, 0x002F8787, 0x00363A3B, 0x00000402, 0x00070B05, 0x00161A0F, 0x001F2317, 0x0024261A, 0x0029281E, 0x0024221A, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFDFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00161B19, 0x00160B0E, 0x00020404, 0x000F130E, 0x001E1A15, 0x001C231C, 0x00385A53, 0x002D5D57, 0x00285350, 0x001B5C65, 0x0026616A, 0x00285F64, 0x002C5F62, 0x001B4C54,
        0x00083A4C, 0x000C435E, 0x00094262, 0x0009415C, 0x000A435C, 0x000D465B, 0x00003B4B, 0x000C4652, 0x00245D66, 0x00266066, 0x00316B70, 0x00286268, 0x00205E64, 0x001E616A, 0x0000424F, 0x00014657, 0x0008445A, 0x000D3F5B, 0x0011405C, 0x000C425B,
        0x000A415C, 0x000C4860, 0x00064258, 0x000F4458, 0x0027515E, 0x00456168, 0x00576A6F, 0x003F6B72, 0x00225B6A, 0x00215762, 0x00496964, 0x005B6F6A, 0x004C6D76, 0x00154959, 0x0008474F, 0x00073F62, 0x000B394B, 0x0021464A, 0x002D5659, 0x0008424E,
        0x00024758, 0x0052969B, 0x00A3E2D9, 0x0093D9D8, 0x00689D93, 0x00195B6E, 0x002A78A3, 0x00165165, 0x000E444B, 0x00004352, 0x00034C5A, 0x00004062, 0x0007434F, 0x006DA3A4, 0x00AAE4E9, 0x0098D8DD, 0x006DB0B3, 0x003B8E9D, 0x00167C9F, 0x00197AAC,
        0x00107BAE, 0x000379AD, 0x000079AA, 0x00007098, 0x0012718B, 0x0074BECA, 0x009CD7D9, 0x009AE1E5, 0x0063B4BC, 0x00005564, 0x0017606E, 0x003E747B, 0x00366263, 0x004C7E7C, 0x00316F6F, 0x00014463, 0x00135A6E, 0x0070BABE, 0x008AD7D0, 0x0088D3CB,
        0x00559B9B, 0x00084656, 0x0008405B, 0x00034256, 0x00034256, 0x00044357, 0x00044357, 0x00044357, 0x00034256, 0x00024155, 0x00024155, 0x0000474D, 0x0002424D, 0x000A4352, 0x00003E50, 0x00004151, 0x00003F4C, 0x0000444D, 0x0001444D, 0x00035459,
        0x00044651, 0x000B3F4F, 0x000E4552, 0x00004049, 0x00004047, 0x00044952, 0x00033E4D, 0x00003755, 0x00207694, 0x002E84A2, 0x00338CA7, 0x00318CA7, 0x00268099, 0x003586A1, 0x002E7894, 0x002882A4, 0x002781A0, 0x00257995, 0x0004516B, 0x00003B4F,
        0x00074857, 0x00084450, 0x000B4650, 0x00034355, 0x00034355, 0x00034355, 0x00034355, 0x00034355, 0x00054355, 0x00054355, 0x00054355, 0x00084755, 0x00004154, 0x00024358, 0x00084356, 0x000B4551, 0x00004251, 0x00267897, 0x001F7EA9, 0x002B8193,
        0x003E929E, 0x001A676A, 0x00003E3C, 0x00113F40, 0x0019444D, 0x00164E5F, 0x00135A6E, 0x0004405D, 0x00003B53, 0x000E293E, 0x00566575, 0x006A9395, 0x00678580, 0x009A9792, 0x00827D74, 0x00657E70, 0x00617568, 0x005D6D5C, 0x006A7565, 0x006C7666,
        0x00677464, 0x00607265, 0x00586F61, 0x0086866E, 0x00908F81, 0x006D7275, 0x00496576, 0x001F5C76, 0x00156C8C, 0x0016789A, 0x00086C8E, 0x00125368, 0x001A3734, 0x003E3E2E, 0x003A382E, 0x00323A33, 0x00353D2C, 0x00373B28, 0x0031372C, 0x003A4237,
        0x00434B40, 0x003A4237, 0x003E463B, 0x00454D42, 0x003F473C, 0x00424A3F, 0x00424A3F, 0x003C463A, 0x00404A3E, 0x003D453A, 0x003E463B, 0x003B4338, 0x00393F34, 0x003E4439, 0x003D4338, 0x0040483D, 0x003D453A, 0x003B4338, 0x003F473C, 0x00434B40,
        0x003C4439, 0x00353D32, 0x00363E33, 0x004A4E43, 0x00373B30, 0x003B3F34, 0x00363C31, 0x0042483D, 0x003D453A, 0x00464E43, 0x003D453A, 0x00394136, 0x00434B40, 0x00424A3F, 0x0040463B, 0x0043493E, 0x00404439, 0x0034382D, 0x002F3328, 0x00343727,
        0x0031352A, 0x0031342B, 0x00353328, 0x00383828, 0x00263226, 0x000C2E2E, 0x000D414E, 0x00044658, 0x0000434D, 0x00247370, 0x0074C6B9, 0x0085CEBE, 0x0072A89B, 0x008EB0A9, 0x00ACC2C0, 0x00B9D4CA, 0x00C0DFD8, 0x00BAE3DE, 0x0062A19F, 0x00136566,
        0x0003575D, 0x00004554, 0x000B4356, 0x00004652, 0x0000434F, 0x00004451, 0x00054B58, 0x00004653, 0x00014754, 0x00004753, 0x00004551, 0x00004654, 0x00034655, 0x00003446, 0x00003F54, 0x0000425C, 0x00026482, 0x002EA4C7, 0x001A9FC4, 0x00004874,
        0x0009365C, 0x00183E5C, 0x00064357, 0x00004253, 0x000E4657, 0x00195D62, 0x001D8078, 0x00393D3E, 0x00000100, 0x00030701, 0x00181C11, 0x00202418, 0x0024261A, 0x0028271D, 0x00211F17, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFDFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00161B19, 0x001C1316, 0x00090B0B, 0x000C100B, 0x001D1712, 0x00171711, 0x00526961, 0x00A2C2BD, 0x00C6DEDC, 0x00CDE7ED, 0x00D6EFF1, 0x00CEE6E4, 0x00B3CECA, 0x006B908E,
        0x00215058, 0x00084356, 0x00003C55, 0x0001425E, 0x00003D57, 0x000A4156, 0x00154452, 0x006C9199, 0x00CAE8E9, 0x00D9F0EC, 0x00E2F6F1, 0x00E2F5F2, 0x00CEEBE8, 0x00A2CBCD, 0x002D636A, 0x00094453, 0x00094359, 0x00043954, 0x000D3E5E, 0x000D435A,
        0x000B445A, 0x000C455E, 0x00054159, 0x00094359, 0x000A4255, 0x000F4354, 0x00275A6A, 0x002D5C6A, 0x00003A4E, 0x00034452, 0x00285957, 0x002F5756, 0x00275868, 0x00043F53, 0x00023F4B, 0x00013A59, 0x00184957, 0x00386261, 0x0044777A, 0x004B8B9D,
        0x00377C90, 0x00578E95, 0x0078A297, 0x00608C8D, 0x006B8E84, 0x00316C80, 0x002F80AD, 0x0000415A, 0x000C4756, 0x00074D65, 0x00004057, 0x00004A64, 0x0013474E, 0x00708F8E, 0x00779296, 0x0066868B, 0x006D9D9F, 0x0056A6B1, 0x0042B0CE, 0x0039A4DE,
        0x003DAEEC, 0x0029A4E6, 0x002CACED, 0x0045BBF6, 0x0046A6D4, 0x004089A7, 0x00568FA4, 0x0062A1A5, 0x003D8793, 0x002F7B91, 0x00317084, 0x00537C85, 0x00244346, 0x0060898B, 0x00447C81, 0x003C7F9A, 0x003D788B, 0x00619195, 0x00628B84, 0x006E958D,
        0x00689291, 0x00144652, 0x00053D50, 0x00034256, 0x00044357, 0x00044357, 0x00044357, 0x00044357, 0x00034256, 0x00034256, 0x00024155, 0x0000494F, 0x00034550, 0x00084255, 0x00054159, 0x000B556F, 0x00095873, 0x000A5772, 0x00094E69, 0x000B5A6D,
        0x0009546A, 0x000F5671, 0x00094F6D, 0x00074E6F, 0x000E5674, 0x000C526F, 0x00105570, 0x00155877, 0x00034C6C, 0x000B5B7A, 0x00075F7D, 0x00005A79, 0x00015B7A, 0x00015578, 0x000D5F82, 0x00025B7D, 0x00085F7F, 0x00045A78, 0x00004E68, 0x000D5B72,
        0x000F5A70, 0x00085264, 0x00064E60, 0x00014153, 0x00014153, 0x00044254, 0x00044254, 0x00064154, 0x00074255, 0x00074255, 0x00084255, 0x000B4A58, 0x00024457, 0x0004455A, 0x00094457, 0x000B4551, 0x00004150, 0x00247796, 0x001D7FA9, 0x00489097,
        0x003A8488, 0x0000484A, 0x00044445, 0x000F4147, 0x001A4B55, 0x00003A48, 0x00004656, 0x00004052, 0x00004451, 0x00002B37, 0x003B666F, 0x00659B9B, 0x006E8E89, 0x00898B85, 0x008E988C, 0x00748577, 0x00697668, 0x00697265, 0x007F8378, 0x007E8277,
        0x0072796C, 0x00727B6E, 0x006B786A, 0x00778867, 0x0070806E, 0x004F6A6E, 0x003B7085, 0x0022799B, 0x001486AE, 0x000E8AB2, 0x001995BD, 0x001E6A8E, 0x00153A42, 0x0036382C, 0x0039372C, 0x00353B30, 0x00373D2A, 0x00383E2B, 0x00303831, 0x00373B30,
        0x003A3E33, 0x00383C31, 0x00393D32, 0x003B3F34, 0x003A3E33, 0x003B3F34, 0x003C4035, 0x003A4035, 0x003C4237, 0x00363C31, 0x00393D32, 0x00373B30, 0x0035362C, 0x003A3B31, 0x0038392F, 0x003A3E33, 0x003C4035, 0x00393D32, 0x003B3F34, 0x00404439,
        0x003E4237, 0x003A3E33, 0x003B3F34, 0x0041453A, 0x0035392E, 0x00303429, 0x002E3429, 0x00343A2F, 0x00353D32, 0x00394136, 0x00394136, 0x00363E33, 0x00384035, 0x00384035, 0x00393F34, 0x00393F34, 0x00383C31, 0x0032362B, 0x002F3328, 0x00333626,
        0x00303429, 0x0030332A, 0x00343227, 0x00383828, 0x00263226, 0x000C2E2E, 0x000C404D, 0x0000444C, 0x002D7575, 0x0094CDC4, 0x0093BBA9, 0x0097AF97, 0x00CCD8C2, 0x00DDDECE, 0x00FFFFF7, 0x00F6F4DC, 0x00DED8C5, 0x00DCDCD0, 0x00D0E4DF, 0x0082B0B1,
        0x00195860, 0x00004251, 0x000A495D, 0x00034557, 0x00004354, 0x00004354, 0x00014657, 0x00004556, 0x00014657, 0x00024758, 0x00034557, 0x00003E47, 0x000C4757, 0x00073F5A, 0x00003C5E, 0x00005F85, 0x002A9CC4, 0x003EA6D1, 0x00004A78, 0x00013756,
        0x00154E67, 0x00003C4F, 0x0002485F, 0x000E4E6C, 0x00013F57, 0x000B555B, 0x002E8A7D, 0x00434748, 0x00000100, 0x00020600, 0x001C2015, 0x0022261A, 0x0025271B, 0x0029281E, 0x00201E16, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFDFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00161B19, 0x001B1215, 0x00060B0A, 0x00090D08, 0x00241B17, 0x0016120D, 0x0053635C, 0x00C1D8D3, 0x00E7F4F2, 0x00FFFBFF, 0x00FFFEFF, 0x00F6F3EE, 0x00C8D1C7, 0x0069847B,
        0x00124041, 0x0001414C, 0x0000495B, 0x00004562, 0x0004445D, 0x00114458, 0x00173A48, 0x007C8F94, 0x00EEF0F0, 0x00F4EAE3, 0x00F5E4DB, 0x00FFF4ED, 0x00FCF8F3, 0x00CCDCDB, 0x00365B5F, 0x00083A46, 0x000F465B, 0x00073D5B, 0x00114667, 0x00094257,
        0x000B445A, 0x000D455E, 0x000A435C, 0x0008455F, 0x00003A53, 0x00003C53, 0x00286E85, 0x00336579, 0x00003C53, 0x00004351, 0x000A4645, 0x000C4243, 0x00094257, 0x00004058, 0x00054452, 0x00003A54, 0x0010454F, 0x00174846, 0x000E484D, 0x0032798D,
        0x0033768F, 0x0056858D, 0x00718B7E, 0x0069877A, 0x008EAA93, 0x003A767C, 0x00156F8E, 0x00003D4A, 0x000A4F52, 0x00004657, 0x00004657, 0x00063D52, 0x0024474A, 0x00899C93, 0x00768B88, 0x005C7976, 0x00577F7A, 0x0014555D, 0x0000445F, 0x00004A4F,
        0x00004757, 0x00004461, 0x0000476F, 0x00064B73, 0x00216082, 0x00286278, 0x004D8793, 0x00356567, 0x000D4955, 0x0003465D, 0x000A3C52, 0x002C4755, 0x0013262D, 0x001A3B44, 0x000E3F4D, 0x0000364B, 0x00174955, 0x004E6A6A, 0x006A786D, 0x007E887C,
        0x007A908B, 0x0022484C, 0x000F414D, 0x00044357, 0x00044357, 0x00044357, 0x00044357, 0x00044357, 0x00044357, 0x00034256, 0x00024155, 0x0000414A, 0x00034453, 0x00063E57, 0x00144E71, 0x003D82A9, 0x003581AB, 0x0029759F, 0x003479A4, 0x00337E9E,
        0x002D7B9F, 0x00227198, 0x002A77A5, 0x00317AAC, 0x002B72A4, 0x002C72A1, 0x002C779E, 0x0046799A, 0x003A7595, 0x002D7495, 0x002A7B9C, 0x0023779A, 0x0022769A, 0x002A7DA3, 0x00207399, 0x00247793, 0x00237692, 0x00277A96, 0x002A7E98, 0x00297D97,
        0x00247993, 0x00358AA4, 0x001B708A, 0x00024052, 0x00024052, 0x00034153, 0x00054053, 0x00074154, 0x00074154, 0x000A4255, 0x000A4255, 0x00054452, 0x00003E51, 0x00004156, 0x00084356, 0x000B4753, 0x00024554, 0x002A7D9C, 0x002587B1, 0x00488686,
        0x00145758, 0x00004545, 0x0006464A, 0x00114650, 0x00104451, 0x00084755, 0x0000404E, 0x00123F4D, 0x00003E47, 0x00002F38, 0x00385F67, 0x00658B8D, 0x00798383, 0x00726360, 0x007B7A70, 0x00829281, 0x006F7C6E, 0x006E766C, 0x00838781, 0x007E807A,
        0x006B7067, 0x0070776A, 0x006C7666, 0x006E7658, 0x00727B6E, 0x0064767F, 0x00396584, 0x0018648E, 0x002489B6, 0x00228DB9, 0x00238CB8, 0x00156A97, 0x00083342, 0x002E332A, 0x0039372C, 0x00363A2E, 0x00363D28, 0x003B4130, 0x00333D37, 0x003A3B31,
        0x0038392F, 0x003B3C32, 0x00393A30, 0x0035362C, 0x0035362C, 0x0035362C, 0x0036372D, 0x0035392E, 0x00383C31, 0x0038392F, 0x003B3C32, 0x003B3C32, 0x0039382E, 0x003B3A30, 0x0036352B, 0x0037382E, 0x003A3B31, 0x0036372D, 0x0033342A, 0x0038392F,
        0x0038392F, 0x0034352B, 0x0036372D, 0x00303429, 0x0033372C, 0x0033372C, 0x00373D32, 0x00353B30, 0x0031392E, 0x002E362B, 0x00333B30, 0x002F372C, 0x002C3429, 0x002C3429, 0x0031372C, 0x0030362B, 0x0032362B, 0x0033372C, 0x0031352A, 0x00323525,
        0x002F3328, 0x0030332A, 0x00343227, 0x00383828, 0x00263226, 0x000B2D2D, 0x000C404D, 0x00005455, 0x0075BDB7, 0x00799D8F, 0x0091937D, 0x00ECD9BE, 0x00FFEED4, 0x00FFF1DB, 0x00FFF5E2, 0x00FFFFE2, 0x00FADBC2, 0x00E7C4B6, 0x00B49E99, 0x00A4ADB1,
        0x0066909C, 0x00084658, 0x00004256, 0x000E455A, 0x000B465A, 0x0009465A, 0x00044659, 0x00034558, 0x00074458, 0x00094458, 0x000B4257, 0x00134A53, 0x000D3D4F, 0x00063C5D, 0x0000567E, 0x0035AEDA, 0x00229AC8, 0x00003B6C, 0x000B3167, 0x00174255,
        0x00004752, 0x00004C58, 0x0000435E, 0x0000375E, 0x00054E6A, 0x0001555A, 0x00318475, 0x00515556, 0x00000402, 0x00030701, 0x001E2217, 0x0023271B, 0x0025271B, 0x002C2B21, 0x00222018, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFEFF, 0xFFFDFCFE, 0xFFFCFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00191E1C, 0x0007150A, 0x00000802, 0x00111615, 0x000D0C10, 0x001A1C1D, 0x00656C67, 0x00B6C4B8, 0x00B5C8B7, 0x00DAEDDA, 0x00DFDCCE, 0x00D0C1BE, 0x008C9194, 0x0064959F,
        0x0049A0B4, 0x004CAFCF, 0x003C9CC4, 0x0050ADC4, 0x0034A4B6, 0x004DC7D3, 0x0043A5AF, 0x006FA3A9, 0x00B3BDBD, 0x00B3ADA6, 0x00AFAC9E, 0x00D6DACE, 0x00C4D7C4, 0x00A0C7B2, 0x00275D56, 0x00023F4D, 0x0004435F, 0x00003B5A, 0x0003425E, 0x00084357,
        0x00124369, 0x00083C64, 0x0003475A, 0x00034452, 0x000D3A4F, 0x001C4050, 0x002C5857, 0x0049544A, 0x002E4334, 0x0088AC9C, 0x00639994, 0x00064A51, 0x00084C59, 0x0048868C, 0x00538787, 0x00508484, 0x00397075, 0x00003A43, 0x001E5963, 0x00689FA6,
        0x0054898C, 0x002F6264, 0x00194C4E, 0x00104646, 0x00296A7F, 0x002176A4, 0x000C78B2, 0x00007EB5, 0x000C8AC0, 0x00087CB7, 0x001179BE, 0x00207BAE, 0x002180B1, 0x00268BB8, 0x001985AF, 0x000E7EA9, 0x001484B2, 0x001786B8, 0x001581B7, 0x000086B1,
        0x00006086, 0x00003A5A, 0x0018455A, 0x00144848, 0x00004440, 0x001482A0, 0x0028B4FF, 0x00005D80, 0x0000485D, 0x0009474D, 0x00184D50, 0x000A414E, 0x00003E53, 0x00014159, 0x00003D50, 0x00024254, 0x00024254, 0x00014153, 0x00014153, 0x00014153,
        0x00014153, 0x00024254, 0x00024254, 0x00024457, 0x00024457, 0x00014356, 0x00014356, 0x00014356, 0x00014356, 0x00004255, 0x00004255, 0x00004753, 0x00074055, 0x0006415B, 0x000C5270, 0x003C7999, 0x00254D6A, 0x002F4D66, 0x002B5163, 0x003F4D6A,
        0x0040526F, 0x00374E68, 0x00334E68, 0x002B475F, 0x0036526A, 0x00365068, 0x0031495F, 0x002C4D5D, 0x00334D5E, 0x002F5161, 0x00295465, 0x002A4A5D, 0x0045526A, 0x00384D63, 0x00224D60, 0x00254E57, 0x00284659, 0x00203E57, 0x0022475D, 0x00305161,
        0x00344C60, 0x00325576, 0x003D78A0, 0x00034355, 0x00034355, 0x00034355, 0x00034355, 0x00034355, 0x00034355, 0x00034355, 0x00034355, 0x00104256, 0x000F4256, 0x00084051, 0x00024050, 0x00014453, 0x0000414D, 0x0034838E, 0x0043939E, 0x001E646B,
        0x00054B52, 0x0000444D, 0x00074856, 0x00014153, 0x00044155, 0x00074359, 0x00013D55, 0x00084664, 0x00003A42, 0x00093332, 0x00325962, 0x005C919B, 0x006B827D, 0x00544C3B, 0x00626653, 0x00859486, 0x00738274, 0x006B7A6C, 0x00818E80, 0x007B887A,
        0x00778174, 0x00717B6E, 0x00727C6F, 0x008D7B74, 0x006B7471, 0x004F6F75, 0x00406B7E, 0x000D5069, 0x0011728C, 0x00127B96, 0x000F6E88, 0x001A555F, 0x001D3939, 0x0030392C, 0x0034402E, 0x00323A2F, 0x003B3732, 0x003B352E, 0x00353A2B, 0x00393D32,
        0x00393D32, 0x00383C31, 0x00373B30, 0x0035392E, 0x0035392E, 0x00373B30, 0x00393D32, 0x00383C30, 0x00373B2F, 0x00363A2E, 0x0035392D, 0x0035392D, 0x0035392D, 0x0035392D, 0x00363A2E, 0x00393B2F, 0x0037392D, 0x0036382C, 0x0036382C, 0x00383A2E,
        0x00383A2E, 0x0036382C, 0x0034362A, 0x003A3A2E, 0x0039392D, 0x0039392D, 0x0038382C, 0x0038382C, 0x0038382C, 0x0038382C, 0x0039392D, 0x0030362B, 0x0032382D, 0x0032382D, 0x0031352A, 0x0031352A, 0x0035362C, 0x0035362C, 0x0033342A, 0x00313229,
        0x003C352C, 0x00353328, 0x002E362B, 0x002B3329, 0x002C2F2D, 0x000E2926, 0x0005403C, 0x002F8B7E, 0x007BA79A, 0x00696D62, 0x00CCC9C4, 0x00FBFCFA, 0x00FBF3EC, 0x009F917A, 0x00A19670, 0x00ECE7D2, 0x00FEFEE6, 0x00E4DAC2, 0x00B09783, 0x00867E71,
        0x0083A5A4, 0x0022656E, 0x00004457, 0x0000405B, 0x0004445D, 0x00094055, 0x00104657, 0x000A4554, 0x000A4656, 0x00114C5F, 0x00033C51, 0x00004260, 0x00043A5B, 0x000E5C81, 0x002CA9D6, 0x001B95CB, 0x00004276, 0x00033B56, 0x00004E4C, 0x0001445B,
        0x00014356, 0x00044554, 0x00044358, 0x00054460, 0x0003485C, 0x00095A5D, 0x002C8476, 0x00444E4E, 0x00000401, 0x00020600, 0x001E1F15, 0x0028281C, 0x00282A1E, 0x002C3025, 0x001D2219, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFCFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00191E1C, 0x00101B11, 0x00050B06, 0x00111111, 0x00141113, 0x001C1B1D, 0x00646B68, 0x00BACABF, 0x00AAC1B2, 0x00D0DCC8, 0x00BCB7A8, 0x00B09F96, 0x008B8A86, 0x006F9698,
        0x00448F98, 0x00388EA2, 0x00277B95, 0x006191AD, 0x00397D94, 0x003D8C9F, 0x00367A87, 0x00698994, 0x009FA4A7, 0x009D9D97, 0x009FA89B, 0x00BFBDB5, 0x00B1BEB0, 0x0095B4A5, 0x002D5A57, 0x000C4050, 0x000A405F, 0x00033A59, 0x000C435E, 0x00004A5C,
        0x00114165, 0x001F4067, 0x000E4052, 0x00013F49, 0x00083C4C, 0x0014343F, 0x00213530, 0x00263432, 0x005A5C5C, 0x00C3C6CA, 0x008DADB8, 0x0033788B, 0x00468FA3, 0x0084ABB9, 0x0093959D, 0x008FAEAB, 0x00487279, 0x00124353, 0x004B7B87, 0x0097C0C2,
        0x0090B8B3, 0x00487578, 0x00063C47, 0x0018677A, 0x00137496, 0x00188BBD, 0x0037B1E7, 0x0035A8DA, 0x002FA0C8, 0x0031A6CD, 0x0035B1D9, 0x0028A7EB, 0x0029A9EA, 0x001D9FDB, 0x0027AAE3, 0x002FB0E9, 0x00219FD9, 0x00269FDD, 0x0032A8E9, 0x0044B7E4,
        0x0046B0D5, 0x00056283, 0x00002E4A, 0x00174B57, 0x00114145, 0x00004766, 0x0029ABEE, 0x00117AAB, 0x00086285, 0x00004759, 0x00003A48, 0x00003C4E, 0x0002425A, 0x0004455A, 0x00034355, 0x00034355, 0x00024254, 0x00024254, 0x00014153, 0x00014153,
        0x00024254, 0x00024254, 0x00034355, 0x00014356, 0x00014356, 0x00014356, 0x00014356, 0x00014356, 0x00004255, 0x00004255, 0x00004255, 0x0000424F, 0x00074055, 0x00003953, 0x000C5270, 0x004283A2, 0x00285471, 0x004E7088, 0x00729DAE, 0x00375B6D,
        0x00285160, 0x0054818F, 0x0070A4B1, 0x0068A3AD, 0x005E9CA6, 0x00397D84, 0x0050949B, 0x00357782, 0x00629EAA, 0x005A97A3, 0x0065A6B4, 0x00558898, 0x00344F64, 0x003F5C71, 0x005C8EA0, 0x005EA1A4, 0x005C96A2, 0x005D95A8, 0x005391A1, 0x006197A2,
        0x00385F6E, 0x002F5B78, 0x0037789E, 0x00034355, 0x00034355, 0x00034355, 0x00034355, 0x00034355, 0x00034355, 0x00034355, 0x00034355, 0x000A4255, 0x00084255, 0x000D485B, 0x00074555, 0x00004352, 0x00084E5B, 0x00408894, 0x002F7985, 0x000A5057,
        0x0000444D, 0x0000454F, 0x00094A58, 0x00054557, 0x00064357, 0x0008445A, 0x00044058, 0x0001405C, 0x00034347, 0x002A5552, 0x004D767F, 0x005E97A0, 0x006E8E89, 0x0046463A, 0x0059685A, 0x0083877B, 0x00696D61, 0x0064665A, 0x00838579, 0x00898B7F,
        0x007F7F73, 0x00707064, 0x00737367, 0x007E705D, 0x007C8374, 0x00637772, 0x00576D73, 0x002F5462, 0x000E4F5E, 0x00003B4C, 0x00054355, 0x0000333F, 0x00112627, 0x00303428, 0x0038402F, 0x00343C31, 0x003E3A35, 0x003E3831, 0x00363B2C, 0x00383C31,
        0x00393D32, 0x00383C31, 0x00373B30, 0x0035392E, 0x0034382D, 0x0035392E, 0x00363A2F, 0x00363A2E, 0x00363A2E, 0x0035392D, 0x0035392D, 0x0034382C, 0x0034382C, 0x0034382C, 0x0034382C, 0x00383A2E, 0x0036382C, 0x0035372B, 0x0035372B, 0x0036382C,
        0x0037392D, 0x0036382C, 0x0035372B, 0x00393B2F, 0x0037392D, 0x0036382C, 0x0034362A, 0x0034362A, 0x0034362A, 0x0036382C, 0x0036382C, 0x002F352A, 0x0031372C, 0x0032382D, 0x0031352A, 0x0031352A, 0x0035362C, 0x0035362C, 0x00323329, 0x002F3229,
        0x003B342B, 0x00333126, 0x002C3429, 0x002B3027, 0x002D302E, 0x00122A28, 0x0008433F, 0x0047998D, 0x0044685C, 0x00908E83, 0x00FFFBF2, 0x00FFFFFB, 0x00E3D9CF, 0x008B7C62, 0x008F855D, 0x00948F76, 0x00CAC9AD, 0x00C3B89D, 0x009E8571, 0x00736A5D,
        0x006E8F8B, 0x00488991, 0x00004152, 0x00084A5C, 0x00004356, 0x0000445B, 0x00054760, 0x0006415B, 0x0002384F, 0x00073D4E, 0x00174E5B, 0x00053E57, 0x00005F82, 0x002DA9D9, 0x003497C9, 0x00014A70, 0x00003D55, 0x0001465A, 0x00073D56, 0x0001445B,
        0x00024457, 0x00044554, 0x00044358, 0x00054460, 0x0003485C, 0x00095A5D, 0x002C8476, 0x00465050, 0x00000200, 0x00000400, 0x00212218, 0x002C2C20, 0x00292B1F, 0x00292D22, 0x001B2017, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFCFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00171C1A, 0x000C130C, 0x00090A06, 0x00140F10, 0x001C1518, 0x001A1818, 0x005E6562, 0x00B5C8BF, 0x00A5C1B4, 0x00899686, 0x007B786A, 0x00988980, 0x00909188, 0x0053726F,
        0x00043F41, 0x00004049, 0x00014251, 0x0003344E, 0x00074359, 0x0000394D, 0x000D394A, 0x007C8593, 0x0087767F, 0x00503C3B, 0x0063564E, 0x008F8F8F, 0x00A2AEA8, 0x009DB8AF, 0x00396062, 0x000F3C51, 0x00114060, 0x000F3E5E, 0x000F405A, 0x00003A4E,
        0x000B4261, 0x00153F5C, 0x00133D4A, 0x004C7780, 0x00608391, 0x00354B51, 0x00374436, 0x00242E35, 0x003D4048, 0x00535E6C, 0x001F4960, 0x00004866, 0x00074C67, 0x00314858, 0x00977F85, 0x00636B64, 0x002B4551, 0x000F3851, 0x00234A59, 0x0028423C,
        0x002E4739, 0x00153F44, 0x0011506C, 0x00206D9E, 0x003C97CE, 0x0041A5DF, 0x001870A6, 0x00003E69, 0x000B4764, 0x00004657, 0x0000525E, 0x00004D62, 0x00025164, 0x00004D5A, 0x00004652, 0x00044B55, 0x000F5360, 0x00104E5E, 0x00003E50, 0x00065986,
        0x003AA5CA, 0x0032B2D5, 0x00006084, 0x00003953, 0x00234251, 0x00154F6C, 0x002F99CE, 0x0052C0FA, 0x004AA8D5, 0x0025718E, 0x0003465D, 0x0000425B, 0x00034661, 0x00014257, 0x00054353, 0x00044456, 0x00034355, 0x00034355, 0x00024254, 0x00024254,
        0x00034355, 0x00034355, 0x00044456, 0x00014356, 0x00014356, 0x00014356, 0x00004255, 0x00004255, 0x00004255, 0x00004255, 0x00004255, 0x00034754, 0x000C455A, 0x00023D57, 0x000F5775, 0x00377D9B, 0x00174763, 0x00436D84, 0x006699A9, 0x00538A93,
        0x004E888E, 0x004D8B91, 0x005CA0A5, 0x005AA7A9, 0x0069C1C1, 0x0079DDD8, 0x0089F0EB, 0x0086E4EA, 0x00469BA3, 0x0052A7B1, 0x0067BCC6, 0x002B6979, 0x0022475B, 0x004B6E82, 0x00679CB0, 0x00368D8A, 0x007FCDD4, 0x007DCBD8, 0x00469BA5, 0x0072BCC2,
        0x00437783, 0x002B617A, 0x0030789A, 0x00024254, 0x00024254, 0x00024254, 0x00024254, 0x00024254, 0x00024254, 0x00024254, 0x00024254, 0x00034256, 0x00044456, 0x0008485A, 0x00034355, 0x00024352, 0x00094A59, 0x002B6C7A, 0x000E4F5D, 0x00004049,
        0x0000434D, 0x00024653, 0x00064756, 0x00054557, 0x00074458, 0x00074359, 0x0008445A, 0x000C3954, 0x00124244, 0x007F9A96, 0x009EB9C3, 0x0063919C, 0x0071888A, 0x00615C59, 0x006B7674, 0x00828E82, 0x00707C70, 0x006A766A, 0x00838D81, 0x008B9589,
        0x007F877C, 0x006F776C, 0x00757D72, 0x0074806A, 0x00677F6D, 0x00516C63, 0x00708184, 0x0079929C, 0x0052828E, 0x00124C58, 0x00093C4C, 0x00002835, 0x001A2729, 0x003D3B30, 0x003C3F2F, 0x0032362B, 0x003A3631, 0x003B3830, 0x00303A2A, 0x00373B30,
        0x00383C31, 0x00383C31, 0x00373B30, 0x0035392E, 0x0034382D, 0x0035392E, 0x00363A2F, 0x00363A2E, 0x00363A2E, 0x00363A2E, 0x00363A2E, 0x00363A2E, 0x0035392D, 0x0035392D, 0x0034382C, 0x0036382C, 0x0035372B, 0x0035372B, 0x0035372B, 0x0037392D,
        0x00383A2E, 0x00383A2E, 0x0037392D, 0x00363D30, 0x00353C2F, 0x0032392C, 0x0030372A, 0x0030372A, 0x0031382B, 0x00333A2D, 0x00343B2E, 0x0030362B, 0x0033392E, 0x0032382D, 0x0032362B, 0x0031352A, 0x0035362C, 0x0034352B, 0x00313228, 0x0030332A,
        0x0039342B, 0x00323025, 0x002C3227, 0x002B3027, 0x002F302E, 0x00152B29, 0x000D4441, 0x0046877E, 0x004F675D, 0x00988F82, 0x00DCD0C4, 0x00F5ECE2, 0x00CDBDB0, 0x00AD9C81, 0x00C4BD92, 0x00AEA387, 0x009C9778, 0x009F9175, 0x00977C67, 0x00665A4E,
        0x004E6E69, 0x005A999D, 0x00044855, 0x00134E58, 0x00004152, 0x00004B62, 0x00004363, 0x00033B5E, 0x00134563, 0x00074154, 0x0000414B, 0x000F5D82, 0x002CABD1, 0x001295BD, 0x00014F74, 0x00103F5A, 0x0000424F, 0x00003E53, 0x00193D61, 0x0001445B,
        0x00024457, 0x00054655, 0x00044358, 0x00054460, 0x0003485C, 0x00095A5D, 0x002C8476, 0x00515B5B, 0x00000502, 0x00010500, 0x00202117, 0x002B2B1F, 0x00292B1F, 0x002A2E23, 0x0021261D, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFCFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00151A18, 0x000A0E09, 0x000A0905, 0x00181010, 0x001F1717, 0x001C1A1A, 0x00525D5B, 0x00809A94, 0x006C9189, 0x00405955, 0x00495452, 0x00838585, 0x008C9696, 0x004B6D6D,
        0x000C4046, 0x000E4953, 0x00104756, 0x0006455A, 0x0004495D, 0x00024457, 0x00184356, 0x005C6A7C, 0x00655E6B, 0x0050464C, 0x00585453, 0x00424E58, 0x00607779, 0x006C8E8D, 0x0029555C, 0x000B3C52, 0x000F4060, 0x000D4060, 0x000D435C, 0x0016485F,
        0x00044561, 0x00004053, 0x001B4A52, 0x009DB0BD, 0x00C4CDE1, 0x00546268, 0x00263D27, 0x0044494C, 0x000E2729, 0x002B6269, 0x00338295, 0x0030809D, 0x003A738C, 0x00243A40, 0x00514C43, 0x00343728, 0x00344E5A, 0x00487796, 0x00568194, 0x00415C52,
        0x00476350, 0x0021575E, 0x002D7EA4, 0x001187C2, 0x003AA8E2, 0x001173A9, 0x00003F6C, 0x00003F5C, 0x00054D59, 0x0007595A, 0x002D8A83, 0x006FB9B5, 0x006EB6B0, 0x0083CDC7, 0x0065AFA9, 0x002A7773, 0x000E6061, 0x00005157, 0x00004F59, 0x00003860,
        0x00005C7F, 0x0028A9D0, 0x002FAFDF, 0x000A5F87, 0x000A364D, 0x0014465C, 0x0008597E, 0x00005F8D, 0x000E6286, 0x00084E65, 0x00003B4F, 0x0001415A, 0x00034661, 0x00014459, 0x00094757, 0x00024457, 0x00024457, 0x00014356, 0x00014356, 0x00014356,
        0x00014356, 0x00024457, 0x00024457, 0x00014356, 0x00014356, 0x00014356, 0x00004255, 0x00004255, 0x00004255, 0x00004255, 0x00004255, 0x0000414E, 0x00053C51, 0x00003953, 0x000F5876, 0x00357D9B, 0x00285D78, 0x00618DA4, 0x006198A7, 0x002F5E6C,
        0x0041707E, 0x00326470, 0x00457B86, 0x005C9AA2, 0x007CC4CB, 0x0076C7CC, 0x00388F91, 0x0084D4DF, 0x00529CA8, 0x0068B2BE, 0x006FBAC8, 0x003B7686, 0x0033536A, 0x0044647B, 0x0070A3B7, 0x005FA5A5, 0x004D8B95, 0x00498C9B, 0x0059A8B3, 0x0071B5BC,
        0x002F616D, 0x00265A71, 0x0035799C, 0x00024254, 0x00024254, 0x00024254, 0x00024254, 0x00024254, 0x00024254, 0x00024254, 0x00024254, 0x00004558, 0x00014659, 0x00004354, 0x00003F51, 0x000B495B, 0x00033F4F, 0x000A4555, 0x00003B4B, 0x0000414E,
        0x00054956, 0x00034655, 0x00004150, 0x00034355, 0x00064357, 0x00054256, 0x0008445A, 0x0012425A, 0x00093B3B, 0x008BAAA3, 0x009CBCC1, 0x004A818A, 0x0046696D, 0x00546062, 0x004B686D, 0x00646B66, 0x00656C67, 0x00646B66, 0x00646A65, 0x00656B66,
        0x00636762, 0x005B5F5A, 0x00656964, 0x00486C66, 0x00305E58, 0x00355C5E, 0x005A7480, 0x00446273, 0x00195164, 0x00004357, 0x00003F54, 0x00052635, 0x00232A2D, 0x00443B31, 0x00373727, 0x0025291E, 0x00332F2A, 0x003D3A32, 0x00354030, 0x00373B30,
        0x00373B30, 0x00373B30, 0x0035392E, 0x0034382D, 0x0035392E, 0x00373B30, 0x00393D32, 0x00363C31, 0x00373D32, 0x00373D32, 0x00383E33, 0x00383E33, 0x00373D32, 0x00363C31, 0x00353B30, 0x0033372C, 0x0034382D, 0x00363A2F, 0x00373B30, 0x00393D32,
        0x00393D32, 0x00383C31, 0x00373B30, 0x00343E32, 0x00343E32, 0x00323C30, 0x00323C30, 0x00323C30, 0x00333D31, 0x00353F33, 0x00364034, 0x00363C31, 0x00373D32, 0x00343A2F, 0x0032362B, 0x00303429, 0x0034352B, 0x0034352B, 0x00323329, 0x002F342B,
        0x0039342B, 0x00323025, 0x002D3328, 0x002D3027, 0x00312F2E, 0x00152A28, 0x000D423F, 0x00366866, 0x0047564E, 0x00998C7E, 0x00A39484, 0x00AA9D8F, 0x00836F5E, 0x00AA9879, 0x00DFDAAD, 0x00B9AA8A, 0x006A6342, 0x00A39579, 0x00B19682, 0x0074685C,
        0x0046645F, 0x004B8688, 0x0005444C, 0x000E4358, 0x00004457, 0x0000495C, 0x0000435C, 0x000C415C, 0x000C405E, 0x00004762, 0x00005F77, 0x002AA2DD, 0x002F96BD, 0x00005267, 0x00003C48, 0x00044659, 0x0005405A, 0x000E415B, 0x00093B4F, 0x0002455C,
        0x00024457, 0x00054655, 0x00044358, 0x00054460, 0x0002475B, 0x00095A5D, 0x002B8375, 0x004D5757, 0x00000401, 0x00040802, 0x001E1F15, 0x0026261A, 0x0027291D, 0x00282C21, 0x0022271E, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFCFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00131816, 0x0010140F, 0x000A0703, 0x00130C09, 0x00160F0C, 0x001A1B19, 0x00415451, 0x00294E4A, 0x000A3936, 0x00184150, 0x001B3C4B, 0x00294351, 0x00224150, 0x00164150,
        0x00094154, 0x000C465C, 0x00033B56, 0x000D4158, 0x00003C54, 0x00003E56, 0x00003A50, 0x00114355, 0x00164350, 0x00194C4F, 0x000E4847, 0x00193B4B, 0x00163F48, 0x001C4F52, 0x000F4752, 0x000C455E, 0x00013C5C, 0x00003C58, 0x00084A5D, 0x00154660,
        0x00003C55, 0x00084D5E, 0x00123D46, 0x00405A6B, 0x00708BA6, 0x0037515D, 0x00334332, 0x0039413A, 0x000A2620, 0x00296261, 0x000F6068, 0x00004B5D, 0x000A4755, 0x000C2521, 0x006D6C58, 0x0037432F, 0x00194049, 0x000D4A6A, 0x00114D63, 0x001F4D47,
        0x00568477, 0x002F7582, 0x00187AA4, 0x001FAAE3, 0x00228ABF, 0x00003865, 0x00054060, 0x00004355, 0x001E7075, 0x0097DEDB, 0x00C2FBF3, 0x00E4F4ED, 0x00ECFFF9, 0x00E0F9F5, 0x00D4F6F5, 0x00C7F7FD, 0x008CCBD9, 0x00307C92, 0x0000445E, 0x00164C6A,
        0x00003E5B, 0x0000537D, 0x00269FD7, 0x00309ECE, 0x00004962, 0x00054151, 0x00023F59, 0x0000374F, 0x00003F4E, 0x000D444D, 0x000E444F, 0x00064357, 0x00004059, 0x00004055, 0x00014251, 0x00024457, 0x00024457, 0x00024457, 0x00014356, 0x00014356,
        0x00024457, 0x00024457, 0x00024457, 0x00014356, 0x00014356, 0x00014356, 0x00014356, 0x00004255, 0x00004255, 0x00004255, 0x00004255, 0x00014754, 0x000A4358, 0x00003A54, 0x00095270, 0x002F7593, 0x0020506C, 0x00406A81, 0x00295C6C, 0x002F4561,
        0x00384E6A, 0x00455B77, 0x003D5670, 0x002E4963, 0x0043637A, 0x003A5C73, 0x00284D63, 0x002C5768, 0x004B7183, 0x00396376, 0x003F7183, 0x00436A80, 0x003A4E67, 0x00364E66, 0x0038677D, 0x0042666E, 0x00395566, 0x002B4D64, 0x00356577, 0x00366370,
        0x00365767, 0x00375F7B, 0x002F6E94, 0x00024254, 0x00024254, 0x00024254, 0x00024254, 0x00024254, 0x00024254, 0x00024254, 0x00024254, 0x00004658, 0x00004557, 0x00004255, 0x00014355, 0x000F4D5F, 0x00054053, 0x00084051, 0x000A4253, 0x00034557,
        0x000A4C5E, 0x00034557, 0x00003E50, 0x00034355, 0x00084559, 0x00044155, 0x00054256, 0x00003D54, 0x00064B48, 0x0078AA9E, 0x0077AAAC, 0x004C969C, 0x001F585A, 0x00365B5F, 0x002E656E, 0x002C5B58, 0x002F5E5B, 0x0034615E, 0x002A5754, 0x002F5A57,
        0x00325D5A, 0x002D5855, 0x00375F5D, 0x003E5459, 0x0026474A, 0x00405D62, 0x006C7A86, 0x00395060, 0x00043747, 0x00003F52, 0x0000394E, 0x00052635, 0x0022292C, 0x0040372D, 0x00313121, 0x001E2217, 0x002D2924, 0x0038352D, 0x00313C2C, 0x00363A2F,
        0x00363A2F, 0x0035392E, 0x0033372C, 0x0033372C, 0x0035392E, 0x00393D32, 0x003D4136, 0x00393F34, 0x00393F34, 0x003A4035, 0x003B4136, 0x003B4136, 0x003B4136, 0x003A4035, 0x00393F34, 0x0032362B, 0x0035392E, 0x00393D32, 0x003C4035, 0x003C4035,
        0x003B3F34, 0x003A3E33, 0x00383C31, 0x00343E32, 0x00353F33, 0x00364034, 0x00374135, 0x00384236, 0x00394337, 0x003A4438, 0x003A4438, 0x003B4136, 0x003A4035, 0x00363C31, 0x0031352A, 0x002F3328, 0x0033342A, 0x0034352B, 0x0033342A, 0x002F342B,
        0x0038362C, 0x00343227, 0x0030362B, 0x0032332A, 0x0034302F, 0x00152726, 0x000B3C3A, 0x00346367, 0x002B3A36, 0x00857C6E, 0x00837765, 0x00A89989, 0x00846C5A, 0x00846F53, 0x00CDCA9E, 0x009C8B6A, 0x007C7352, 0x00C0B197, 0x00A18878, 0x0060574E,
        0x00415E5B, 0x003C7577, 0x00034147, 0x00073B5F, 0x000A455F, 0x00034151, 0x000E4854, 0x000C4352, 0x0000344F, 0x00006184, 0x0028A6CF, 0x001891C9, 0x0000446C, 0x000F4153, 0x00064C53, 0x0000414D, 0x000D435C, 0x000F425C, 0x0000495B, 0x0002455C,
        0x00034558, 0x00054655, 0x00044358, 0x00054460, 0x0002475B, 0x0008595C, 0x002B8375, 0x003C4646, 0x00000200, 0x00050903, 0x00202117, 0x0027271B, 0x00282A1E, 0x0025291E, 0x001E231A, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFCFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00121715, 0x000D1210, 0x000A0905, 0x0017110C, 0x001A1611, 0x00222823, 0x004B6663, 0x001B4C4A, 0x00054343, 0x00093F58, 0x000F4158, 0x000C3A52, 0x000C3C54, 0x00134B64,
        0x00003C5A, 0x0000385B, 0x000E4D73, 0x0020637C, 0x001F6681, 0x00125F7A, 0x00004B65, 0x00004860, 0x00003D50, 0x00004350, 0x00054A53, 0x0014455F, 0x00083E4F, 0x0006414B, 0x00064252, 0x0005415E, 0x00003958, 0x00003C57, 0x00024A5C, 0x0001445D,
        0x00003C56, 0x001B4558, 0x001B3C4B, 0x0015536B, 0x00439ABC, 0x001A4F64, 0x003A3233, 0x002A382C, 0x00363E34, 0x00AFBCBA, 0x007EACB3, 0x0002505D, 0x0000434D, 0x00253D3B, 0x0069554A, 0x00334F3C, 0x000D3F45, 0x0000415F, 0x00003E5A, 0x0002444F,
        0x004A8A8F, 0x003D8EA3, 0x00187AA2, 0x0028A9E2, 0x001C70A0, 0x00042C4F, 0x00183D51, 0x00417F89, 0x008DD4D7, 0x00BAE2E1, 0x00A2A7A6, 0x00D4D0AD, 0x00E6E4C2, 0x00E4E9CA, 0x00ECFBE0, 0x00C2E3CE, 0x00A8DED1, 0x00A0EAE4, 0x00419593, 0x00044553,
        0x00134E5E, 0x00003A5F, 0x000B74AD, 0x0034AEE2, 0x0000546F, 0x00004957, 0x000B4054, 0x000B4854, 0x000C4348, 0x00164648, 0x00154852, 0x00064155, 0x00004059, 0x0002475B, 0x00074857, 0x00004358, 0x00004358, 0x00004257, 0x00004257, 0x00004257,
        0x00004257, 0x00004358, 0x00004358, 0x00024457, 0x00024457, 0x00024457, 0x00014356, 0x00014356, 0x00014356, 0x00014356, 0x00004255, 0x0000424E, 0x000D485C, 0x00023D57, 0x000C5472, 0x004283A2, 0x0026506D, 0x0034536C, 0x002A5264, 0x00505679,
        0x00434C6E, 0x00354465, 0x00435776, 0x00384C6B, 0x003C4E6B, 0x004D5B78, 0x004A5573, 0x003E4E65, 0x0049556D, 0x0030475D, 0x002B4D64, 0x00314D65, 0x00475670, 0x003F5670, 0x00194A60, 0x00374C5B, 0x004E5870, 0x003E4967, 0x0037506A, 0x00344D61,
        0x00405269, 0x00315476, 0x00407EA7, 0x00034355, 0x00034355, 0x00034355, 0x00034355, 0x00034355, 0x00034355, 0x00034355, 0x00034355, 0x0001465A, 0x00004256, 0x0005495C, 0x00034558, 0x00054355, 0x000A4558, 0x000F495C, 0x000A4555, 0x00034558,
        0x0007495C, 0x00034558, 0x00014054, 0x0007465A, 0x000B495B, 0x00054355, 0x00034153, 0x000A3B5B, 0x0014464C, 0x006C8B84, 0x00628085, 0x0040757F, 0x0012383D, 0x003E5158, 0x002A515F, 0x00403E4A, 0x00373541, 0x003D3B47, 0x00393541, 0x003F3B47,
        0x00403946, 0x0038313E, 0x003C3542, 0x004E4D51, 0x0030393C, 0x00333B3B, 0x005B575C, 0x00485155, 0x0021454B, 0x00154A54, 0x000E3E4A, 0x00032D3A, 0x001D2A2C, 0x003A382D, 0x00363929, 0x002A2E23, 0x00332F2A, 0x0036332B, 0x002D3727, 0x0035392E,
        0x0034382D, 0x0033372C, 0x0032362B, 0x0032362B, 0x0034382D, 0x00383C31, 0x003C4035, 0x00363E33, 0x00363E33, 0x00373F34, 0x00373F34, 0x00373F34, 0x00373F34, 0x00373F34, 0x00373F34, 0x002F352A, 0x0032382D, 0x00373D32, 0x00393F34, 0x003A4035,
        0x00383E33, 0x00363C31, 0x00353B30, 0x00333B30, 0x00343C31, 0x00363E33, 0x00384035, 0x003A4237, 0x003A4237, 0x003A4237, 0x003A4237, 0x003B4136, 0x003A4035, 0x00353B30, 0x0031352A, 0x002E3227, 0x0033342A, 0x0034352B, 0x0033342A, 0x002C342A,
        0x0038362C, 0x00353328, 0x0032382D, 0x0034352C, 0x0035302F, 0x00172524, 0x00093836, 0x00275965, 0x002F4445, 0x004B4D41, 0x00534F3D, 0x00ADA090, 0x00C0A696, 0x00AE9B80, 0x00BFBF97, 0x00928160, 0x009B9375, 0x00978A74, 0x00654F44, 0x0049423F,
        0x00405F60, 0x00336D72, 0x0005434B, 0x000A4063, 0x00093E59, 0x000C4253, 0x000A4450, 0x00003C4A, 0x000E6782, 0x0034A0CA, 0x001A90C4, 0x00004B68, 0x0000375A, 0x00003D5F, 0x00074458, 0x000A464C, 0x000D474C, 0x00004457, 0x00004A6A, 0x0003465D,
        0x00034558, 0x00054655, 0x00044358, 0x00054460, 0x0002475B, 0x0008595C, 0x002A8274, 0x00434D4D, 0x00000200, 0x00060A04, 0x00212218, 0x002A2A1E, 0x002E3024, 0x002C3025, 0x00292E25, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFCFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00101513, 0x00070F0E, 0x00090D08, 0x0014110C, 0x0017150D, 0x000C1812, 0x00325551, 0x000E4748, 0x0000484A, 0x00064056, 0x00033D53, 0x00033D53, 0x00053F55, 0x00013E58,
        0x0000405F, 0x00185980, 0x003879A5, 0x00317592, 0x0029718F, 0x003885A5, 0x002E7FA0, 0x00186383, 0x0000425F, 0x000B415A, 0x00103E56, 0x00093C5D, 0x000D435C, 0x000E4455, 0x00063D52, 0x00023857, 0x0003395C, 0x0006435D, 0x0006485A, 0x00003D58,
        0x00114964, 0x001F4158, 0x00183B4F, 0x00005372, 0x0029ABD6, 0x00004B6F, 0x00382C3E, 0x00202B28, 0x005D5C58, 0x00B4B4B4, 0x0077949D, 0x00357789, 0x003A8091, 0x004F737D, 0x00605F63, 0x00476B5B, 0x003C7076, 0x00377E99, 0x002C7DA0, 0x00287897,
        0x003E8BA5, 0x0030809D, 0x000E6285, 0x000C9AD7, 0x001E89B5, 0x0000455B, 0x00135C60, 0x0074C8C4, 0x0082CECA, 0x00648882, 0x00A8A69E, 0x00D8C8BB, 0x00FFFFF0, 0x00F5E6D6, 0x00F4EBDD, 0x00FEFFF4, 0x00C3D5CE, 0x00A9CCC8, 0x00AFDEDC, 0x00237471,
        0x0006464A, 0x00003E5A, 0x001674A9, 0x003EB2E7, 0x00004C68, 0x00004454, 0x00084051, 0x00004156, 0x00033E4E, 0x000D434E, 0x000D4556, 0x0001415A, 0x00003F5C, 0x00004357, 0x0002444F, 0x00004358, 0x00004358, 0x00004257, 0x00004257, 0x00004257,
        0x00004257, 0x00004358, 0x00004358, 0x00034558, 0x00024457, 0x00024457, 0x00024457, 0x00024457, 0x00014356, 0x00014356, 0x00014356, 0x00004450, 0x000A475B, 0x00003B55, 0x00084B6A, 0x00427D9D, 0x002C4E6C, 0x003B506B, 0x00435E72, 0x003E4965,
        0x00455A75, 0x00355971, 0x005C8EA2, 0x0043788C, 0x001F5163, 0x00375F72, 0x0027485C, 0x00344C62, 0x003A4F65, 0x003C5B70, 0x00457085, 0x003B6278, 0x00354C66, 0x0032546C, 0x004E899D, 0x00558091, 0x00334E68, 0x003B4E6F, 0x00415977, 0x00354B64,
        0x00455774, 0x003E658B, 0x00266D99, 0x00044456, 0x00044456, 0x00044456, 0x00044456, 0x00044456, 0x00044456, 0x00044456, 0x00044456, 0x0004455A, 0x00034459, 0x000A4B60, 0x00014054, 0x00003D51, 0x00094759, 0x00074557, 0x00044254, 0x00014257,
        0x0004455A, 0x00034459, 0x00044357, 0x00064559, 0x00084658, 0x00054355, 0x00044254, 0x00143C66, 0x00123947, 0x00526463, 0x00586A71, 0x00376069, 0x00273F45, 0x00373E47, 0x00001B29, 0x0007182B, 0x00041528, 0x00152639, 0x0009182B, 0x00051427,
        0x00051326, 0x00031124, 0x0009172A, 0x00233540, 0x00234349, 0x00264548, 0x003F5456, 0x00335054, 0x00083C42, 0x00003E48, 0x00083F4E, 0x00043844, 0x00152A2B, 0x00313529, 0x00363E2D, 0x0031392E, 0x003C3833, 0x003D3730, 0x00363B2C, 0x0032362B,
        0x0032362B, 0x0032362B, 0x0032362B, 0x0031352A, 0x0031352A, 0x0033372C, 0x0035392E, 0x0030382E, 0x0030382E, 0x0030382E, 0x0030382E, 0x0031392F, 0x0031392F, 0x00323A30, 0x00323A30, 0x002D3229, 0x002F342B, 0x0033382F, 0x00343930, 0x00343930,
        0x0032372E, 0x0032372E, 0x0033382F, 0x0032372E, 0x0033382F, 0x0033382F, 0x00343930, 0x00353A31, 0x00353A31, 0x00353A31, 0x00353A31, 0x00353B30, 0x00353B30, 0x0032382D, 0x00303429, 0x002F3328, 0x0033342A, 0x0034352B, 0x00323329, 0x002A3329,
        0x0036352B, 0x00343227, 0x0032382D, 0x0037352D, 0x00383030, 0x00182424, 0x000B3736, 0x00064053, 0x0025474D, 0x0025352A, 0x00313726, 0x00584F41, 0x009E8375, 0x00BFAA94, 0x0090926E, 0x008A7859, 0x007A7358, 0x00504634, 0x0057443F, 0x005A5759,
        0x0046696D, 0x001C585E, 0x00003E48, 0x00094555, 0x000B4054, 0x000C445D, 0x00004762, 0x0000617E, 0x00309EC0, 0x003B9AC5, 0x00004776, 0x0007454F, 0x0001465A, 0x00003E5C, 0x000F415D, 0x00073E4D, 0x0000444C, 0x00025060, 0x00003653, 0x0003465D,
        0x00034558, 0x00064756, 0x00044358, 0x00054460, 0x0002475B, 0x0008595C, 0x002A8274, 0x005E6868, 0x00000300, 0x00030701, 0x001F2016, 0x002C2C20, 0x002E3024, 0x002B2F24, 0x0030352C, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFCFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00101513, 0x0006110F, 0x000A100B, 0x0011110B, 0x001D1E15, 0x000A1A13, 0x002B534E, 0x000D4F4E, 0x00004548, 0x00104859, 0x00043F4F, 0x000B4757, 0x00024052, 0x00003A52,
        0x002A6886, 0x003D7CA2, 0x000E5079, 0x00344A73, 0x00294972, 0x0013446C, 0x0016577E, 0x003C88AC, 0x001B6385, 0x00003653, 0x000C4461, 0x00103D63, 0x000E3D5C, 0x000C3D53, 0x0014425A, 0x00143F60, 0x000E3B60, 0x0011435F, 0x000E4657, 0x000E3F5F,
        0x0008425F, 0x000A3D51, 0x000C4156, 0x0000466C, 0x002BB1E7, 0x0000507E, 0x000D142D, 0x001D2025, 0x00151E21, 0x002C4547, 0x0015424D, 0x0010495F, 0x0008425F, 0x00205269, 0x00204B5A, 0x002C5147, 0x002E5E62, 0x00155770, 0x00004C75, 0x00005380,
        0x00004F78, 0x00014B6F, 0x00004664, 0x0033A5EC, 0x001375A5, 0x00003C4E, 0x005DA5A5, 0x0083C3BD, 0x00527A75, 0x0093968D, 0x00E3C7BC, 0x00DCE4E4, 0x00F4F7F5, 0x00FBF8F3, 0x00F5F0E7, 0x00FFFFF8, 0x00E8F1E7, 0x00AABFB6, 0x009AB9B0, 0x005CBAAF,
        0x00004A43, 0x0000485A, 0x001571A2, 0x0043ABE0, 0x0000496A, 0x00004455, 0x00074353, 0x00003E63, 0x00004361, 0x0005455E, 0x0003425E, 0x00003F60, 0x00003D5E, 0x00004357, 0x00074851, 0x00004358, 0x00004257, 0x00004257, 0x00004156, 0x00004156,
        0x00004257, 0x00004257, 0x00004358, 0x00034558, 0x00034558, 0x00034558, 0x00024457, 0x00024457, 0x00024457, 0x00024457, 0x00024457, 0x0000434E, 0x00014054, 0x0007425C, 0x000F506F, 0x00427899, 0x00355170, 0x0046526E, 0x00405368, 0x003B5365,
        0x002C5362, 0x00387581, 0x006CBDC4, 0x0067C2C7, 0x0057AEB2, 0x0056A4AA, 0x00549AA1, 0x005C879A, 0x00365E71, 0x0019485D, 0x005F9AAD, 0x0057899F, 0x001E425A, 0x003E6A81, 0x0066A9BE, 0x0067B0BE, 0x00245771, 0x002B5072, 0x002E4F70, 0x0038536E,
        0x00324A68, 0x002E5C85, 0x002575A4, 0x00044456, 0x00044456, 0x00044456, 0x00044456, 0x00044456, 0x00044456, 0x00044456, 0x00044456, 0x00074359, 0x000D495F, 0x0009485D, 0x00003D51, 0x0008475B, 0x000B4A5E, 0x00003C4E, 0x0006485A, 0x0003435B,
        0x0003435B, 0x00034459, 0x00044358, 0x00014054, 0x00034153, 0x00054355, 0x00064456, 0x00003B65, 0x00004652, 0x00134642, 0x000E3F41, 0x00003E41, 0x000E4746, 0x00183E40, 0x00023E44, 0x00003849, 0x0003495A, 0x0024697A, 0x00064B5C, 0x00003748,
        0x00003D4F, 0x000A4C5E, 0x00155769, 0x0017556D, 0x00135F71, 0x0005545F, 0x001A5C67, 0x00206771, 0x00065F6D, 0x00056376, 0x000C607A, 0x00144F59, 0x00163232, 0x00293225, 0x002E3A28, 0x002C3429, 0x0034302B, 0x00342E27, 0x002F3425, 0x002F3328,
        0x0031352A, 0x0032362B, 0x0032362B, 0x00303429, 0x002F3328, 0x002E3227, 0x002F3328, 0x002B3329, 0x002B3329, 0x002A3228, 0x002A3228, 0x002A3228, 0x002B3329, 0x002D352B, 0x002E362C, 0x002B3027, 0x002D3229, 0x002F342B, 0x002F342B, 0x002E332A,
        0x002D3229, 0x002F342B, 0x0030352C, 0x0032352C, 0x0031342B, 0x0031342B, 0x0030332A, 0x0030332A, 0x0030332A, 0x0031342B, 0x0031342B, 0x002E3429, 0x0030362B, 0x0030362B, 0x002F3328, 0x002F3328, 0x0033342A, 0x0033342A, 0x00313228, 0x002A3329,
        0x0035342A, 0x00333126, 0x0033372C, 0x0037352D, 0x003B3131, 0x001A2626, 0x000F3938, 0x0000384D, 0x00214A53, 0x00223931, 0x00293626, 0x003E3629, 0x0070564A, 0x0063503B, 0x00646747, 0x00AE9E81, 0x00706B52, 0x00484132, 0x00716260, 0x005D5D63,
        0x00365B63, 0x00003B45, 0x0006434F, 0x00034540, 0x0018505B, 0x00013E5E, 0x00005380, 0x0027A4D1, 0x001F98C2, 0x0000446A, 0x0013385E, 0x00004B56, 0x00043F48, 0x001C4954, 0x000C3E52, 0x00004061, 0x00004D6D, 0x00004256, 0x00194E58, 0x0003465D,
        0x00034558, 0x00064756, 0x00044358, 0x00054460, 0x0001465A, 0x0007585B, 0x002A8274, 0x006C7676, 0x00000300, 0x00000200, 0x001E1F15, 0x002C2C20, 0x0027291D, 0x001E2217, 0x00242920, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFCFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00131816, 0x000C0D0B, 0x00010A07, 0x000F130E, 0x001E1714, 0x00161D1A, 0x0037625F, 0x000E504F, 0x0004474A, 0x000F425C, 0x000A4057, 0x000B3A56, 0x000A355C, 0x0012537F,
        0x00247DA2, 0x00074D64, 0x003C5666, 0x005C8BA6, 0x0079B6C4, 0x0067A7AB, 0x0030616B, 0x00225068, 0x004380A2, 0x00115376, 0x00043C5F, 0x000A425D, 0x0009415C, 0x0008405B, 0x00073F5A, 0x00073F5A, 0x0009415C, 0x000B435E, 0x000D4560, 0x00114661,
        0x000D435A, 0x000E4358, 0x00053F55, 0x00004362, 0x003688B2, 0x0040A1DB, 0x00004487, 0x00003038, 0x00003941, 0x0000424A, 0x0000474F, 0x0000464E, 0x0000454D, 0x0000464E, 0x00004850, 0x0007495B, 0x00024554, 0x0000454D, 0x00004C52, 0x00004B52,
        0x00004151, 0x00075369, 0x002A7792, 0x0040BDE9, 0x000E6083, 0x00114F61, 0x007EC7CB, 0x0048797B, 0x00655E61, 0x00CFB6AC, 0x0097967C, 0x006D724B, 0x00938A69, 0x00C0A890, 0x009C8674, 0x00EDE5D4, 0x00FFFFF4, 0x00C2BCAF, 0x00928478, 0x0069A9AA,
        0x00105C62, 0x0000465D, 0x000F729E, 0x0035A2D8, 0x00004374, 0x00123F54, 0x00004B3E, 0x00044358, 0x00024156, 0x00034257, 0x00044358, 0x00034257, 0x00014055, 0x00024156, 0x00054459, 0x00063F5E, 0x00063F5E, 0x00063F5E, 0x00063F5E, 0x00063F5E,
        0x00063F5E, 0x00063F5E, 0x00063F5E, 0x0002415D, 0x0003425E, 0x0003425E, 0x0004435F, 0x0004435F, 0x0003425E, 0x0003425E, 0x0002415D, 0x00004350, 0x0000485A, 0x0000415C, 0x00084D6E, 0x00417C9C, 0x001B4961, 0x00395A6A, 0x00405963, 0x003C5363,
        0x00335664, 0x003F7580, 0x006EB8BE, 0x0076CFD3, 0x008DEAEB, 0x0075D2D3, 0x0052AEAF, 0x0074A8B5, 0x00245865, 0x001D525F, 0x00528C98, 0x005794A0, 0x004B8B96, 0x00337580, 0x0066ABB5, 0x00578E9D, 0x002F596C, 0x00364E66, 0x00414C6A, 0x003E4D6D,
        0x00325374, 0x00225978, 0x00317997, 0x00054654, 0x00044553, 0x00044553, 0x00034452, 0x00034452, 0x00034452, 0x00034452, 0x00044553, 0x00054459, 0x00054459, 0x00054459, 0x00054459, 0x00054459, 0x00054459, 0x00054459, 0x00054459, 0x0002425A,
        0x0002425A, 0x0004425A, 0x0004425A, 0x0004425A, 0x0006425A, 0x0006425A, 0x0006425A, 0x00064150, 0x00064150, 0x00064150, 0x0005404F, 0x00043F4E, 0x00033E4D, 0x00023D4C, 0x00013C4B, 0x00074458, 0x00003B4E, 0x00307889, 0x00115463, 0x00003746,
        0x00468190, 0x001D6472, 0x00045864, 0x00125D77, 0x00105B75, 0x000E5973, 0x00105B75, 0x00135E78, 0x00145F79, 0x00115C76, 0x000F5A74, 0x00173B45, 0x00001C20, 0x00273834, 0x001A2316, 0x00363929, 0x001E2014, 0x00363C37, 0x0018201F, 0x0028382D,
        0x0016231B, 0x00343B36, 0x00191E1C, 0x002C2F2D, 0x00161D16, 0x00313B2F, 0x001A2417, 0x002D352E, 0x001F241B, 0x0031352A, 0x00212E20, 0x00182C1F, 0x00172C23, 0x001D2A28, 0x00292E2F, 0x001A271F, 0x00212E26, 0x0015221A, 0x00212C24, 0x001C271F,
        0x002E362F, 0x00131B14, 0x00333B34, 0x00191C13, 0x0034372E, 0x00161910, 0x0033362D, 0x0020231A, 0x002F3229, 0x0022251C, 0x002D3027, 0x001A261A, 0x00253125, 0x001F2B1F, 0x00243024, 0x00222E22, 0x001B271B, 0x00232F23, 0x001A261A, 0x00202E2A,
        0x001B261E, 0x002A2F20, 0x00272A1A, 0x002B3225, 0x001B2925, 0x00092028, 0x001E3B4A, 0x0004393C, 0x00114244, 0x001B4143, 0x001D3737, 0x00242F2D, 0x002E2D29, 0x00332721, 0x002D1D16, 0x00281E14, 0x00311E17, 0x004B3831, 0x0060645F, 0x00375F5D,
        0x00004144, 0x0000414D, 0x00004154, 0x0010445B, 0x00004053, 0x0000587A, 0x003799D9, 0x002990CE, 0x00004F68, 0x00004349, 0x00074752, 0x00054054, 0x00044155, 0x00064357, 0x00074458, 0x00064559, 0x00064559, 0x00044659, 0x00034558, 0x00004356,
        0x00014453, 0x00054654, 0x00054458, 0x0005455E, 0x0001465A, 0x00045558, 0x00257D70, 0x004C5350, 0x00010805, 0x00000300, 0x00151913, 0x0033362D, 0x00282C21, 0x002B2D21, 0x002D2D21, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFCFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00121715, 0x00111210, 0x00030C09, 0x000C100B, 0x001D1613, 0x00181F1C, 0x003C6764, 0x00125453, 0x0005484B, 0x00083D58, 0x000F4158, 0x000F455E, 0x00003658, 0x001E628D,
        0x0027638D, 0x00144964, 0x005E8F9D, 0x009FFAFF, 0x009EFFFF, 0x009DFBF7, 0x0071B5BA, 0x00225B71, 0x002B6B8D, 0x001F6184, 0x00003558, 0x0008405B, 0x0008405B, 0x0009415C, 0x0009415C, 0x0009415C, 0x000A425D, 0x000A425D, 0x000A425D, 0x00053D5A,
        0x000B415A, 0x000A4156, 0x000E475C, 0x00003B57, 0x00004A6F, 0x003089BB, 0x004CADE5, 0x00218FCB, 0x002694D0, 0x002C9AD6, 0x002D9BD7, 0x002B99D5, 0x002997D3, 0x002A98D4, 0x002C9AD6, 0x002795D1, 0x002A99D1, 0x00289BCE, 0x00279ACD, 0x002A9ED3,
        0x002FA2DB, 0x0033A5E4, 0x0037A7ED, 0x000D6593, 0x00003958, 0x00246B79, 0x008DCACE, 0x001F4246, 0x00565B5A, 0x00C8C0B3, 0x007E7D61, 0x0085855D, 0x00B3A785, 0x00B49F83, 0x00806E57, 0x009C9582, 0x00CFCDBB, 0x00A7A091, 0x0079695D, 0x004D8082,
        0x00195B60, 0x00003A4E, 0x002380A7, 0x003DBBEB, 0x00004C79, 0x00033F5C, 0x0007464E, 0x0000415C, 0x0000415C, 0x0000405B, 0x0000435E, 0x00014661, 0x0000405B, 0x00003D58, 0x0000435E, 0x00004054, 0x00004054, 0x00004054, 0x00004054, 0x00004054,
        0x00004054, 0x00004054, 0x00004054, 0x00004153, 0x00004153, 0x00004254, 0x00004254, 0x00004254, 0x00004254, 0x00004153, 0x00004153, 0x00054452, 0x00044357, 0x00003D59, 0x00104F71, 0x004880A3, 0x00214D6C, 0x0037566F, 0x00385165, 0x00495470,
        0x003C5069, 0x0033536A, 0x00416E83, 0x0035697A, 0x00447889, 0x003B6C7C, 0x00305C6D, 0x00406279, 0x001F445A, 0x00264D63, 0x00497288, 0x00457287, 0x00437287, 0x002D5F73, 0x0044778B, 0x00396E82, 0x00265067, 0x00354D69, 0x003F4E6E, 0x00384B6E,
        0x002D5072, 0x00275D7E, 0x00367B9C, 0x00004755, 0x00004654, 0x00004654, 0x00004654, 0x00004654, 0x00004755, 0x00014856, 0x00014856, 0x00004654, 0x00004654, 0x00004654, 0x00004654, 0x00004654, 0x00004654, 0x00004654, 0x00004654, 0x00004553,
        0x00004553, 0x00004553, 0x00004553, 0x00004553, 0x00004453, 0x00004453, 0x00004453, 0x00034453, 0x00044554, 0x00034453, 0x00034453, 0x00024352, 0x00014251, 0x0000404F, 0x0000404F, 0x00003C4E, 0x00033A49, 0x004F7581, 0x006D7F8A, 0x003A4350,
        0x006A828E, 0x0019515C, 0x00055A62, 0x00074752, 0x0004444F, 0x0002424D, 0x0002424D, 0x0004444F, 0x00054550, 0x0004444F, 0x0002424D, 0x00063D4C, 0x00002D35, 0x00476F6E, 0x00315349, 0x004E6C61, 0x002B4A43, 0x00496B6A, 0x00103638, 0x004F6F64,
        0x0026453C, 0x0044645F, 0x00224241, 0x004F6F6E, 0x00244443, 0x00446660, 0x00254740, 0x004A595B, 0x003E504F, 0x00455B56, 0x003A4F47, 0x0041534C, 0x0040514D, 0x003A4C4B, 0x003F5456, 0x00354D4B, 0x0049615F, 0x00253D3B, 0x00526866, 0x001F3533,
        0x00526765, 0x00233836, 0x00516664, 0x0023413C, 0x0046645F, 0x0022403B, 0x004A6863, 0x0023413C, 0x003E5C57, 0x002A4843, 0x003F5D58, 0x00354C48, 0x003F5652, 0x00384F4B, 0x003D5450, 0x003C534F, 0x00364D49, 0x003D5450, 0x00344B47, 0x003C6060,
        0x001C3D39, 0x00415E55, 0x001C372D, 0x004E6D66, 0x000A3032, 0x000B3843, 0x00053749, 0x00043D46, 0x000A3E45, 0x000F3B42, 0x0014373B, 0x00193032, 0x001C2828, 0x001D2221, 0x001D1E1C, 0x00142020, 0x001B272B, 0x00334550, 0x002F5060, 0x000D4054,
        0x00003D4F, 0x00074353, 0x000A4253, 0x00003E56, 0x00085F7F, 0x0036A7CF, 0x002A96C0, 0x00004A74, 0x0000385C, 0x00074458, 0x0000464A, 0x00034054, 0x00044155, 0x00054256, 0x00054458, 0x00054458, 0x00034558, 0x00034558, 0x00034558, 0x00014558,
        0x00024554, 0x00074856, 0x00064559, 0x00074760, 0x0003485C, 0x0007585B, 0x00288073, 0x003F4643, 0x00000200, 0x00040A05, 0x00161A14, 0x00282B22, 0x00292D22, 0x00404236, 0x0049493D, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFCFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00121715, 0x000D0E0C, 0x00050E0B, 0x0010140F, 0x001D1613, 0x00101714, 0x00305B58, 0x000C4E4D, 0x0006494C, 0x00023D57, 0x000B3D54, 0x000A465C, 0x00003754, 0x002F749C,
        0x00315381, 0x00284E6E, 0x008ED2DF, 0x0080FDFF, 0x0071F3EC, 0x0095FFFD, 0x008DDCDF, 0x001D5B73, 0x001A5C7F, 0x00317598, 0x00054060, 0x00073F5A, 0x0008405B, 0x000A425D, 0x000B435E, 0x000B435E, 0x000A425D, 0x0009415C, 0x0008405B, 0x00084160,
        0x000A425D, 0x0001384D, 0x000D4459, 0x0009455B, 0x00003451, 0x00014F74, 0x002779A3, 0x001E82BC, 0x001F83BD, 0x002084BE, 0x001F83BD, 0x001D81BB, 0x001C80BA, 0x001D81BB, 0x001E82BC, 0x001785B5, 0x001A89B7, 0x001989B4, 0x001481AD, 0x001380AC,
        0x001A83B4, 0x00137BB0, 0x000468A2, 0x0002345E, 0x00003B53, 0x002D7A83, 0x0081B6B9, 0x0025393E, 0x005E6463, 0x00959888, 0x00848668, 0x00ABA07A, 0x00EEDEBA, 0x00D6C4A5, 0x009E9276, 0x008A866E, 0x00CFCBB8, 0x00AFA698, 0x00705E53, 0x00406164,
        0x00286569, 0x00013D4D, 0x00075475, 0x003BBBE4, 0x00199BC6, 0x00004D76, 0x00003659, 0x00034151, 0x00084656, 0x00044252, 0x00003E4E, 0x00024050, 0x00024050, 0x00024050, 0x00094757, 0x0007454F, 0x0007454F, 0x0007454F, 0x0007454F, 0x0007454F,
        0x0007454F, 0x0007454F, 0x0007454F, 0x0007464E, 0x0007464E, 0x0007464E, 0x0007464E, 0x0007464E, 0x0007464E, 0x0007464E, 0x0007464E, 0x00154950, 0x000E4350, 0x00043B50, 0x00164C6A, 0x00487B9B, 0x00265271, 0x00335670, 0x002F4E65, 0x00495273,
        0x00425171, 0x00354B67, 0x0037526D, 0x0027425C, 0x00354A65, 0x003A4761, 0x003F4863, 0x003B4C67, 0x00364964, 0x00364964, 0x00394E69, 0x0029425C, 0x00324D67, 0x002A455F, 0x002E4C65, 0x00205465, 0x00224B61, 0x002F4E65, 0x003B506B, 0x00344C6A,
        0x002A4E6C, 0x002A5F7A, 0x002F708C, 0x0004414F, 0x0004414F, 0x0004414F, 0x00054250, 0x00064351, 0x00074452, 0x00084553, 0x00094654, 0x00094850, 0x00094850, 0x00094850, 0x00094850, 0x00094850, 0x00094850, 0x00094850, 0x00094850, 0x00074A4D,
        0x00074A4D, 0x0009494D, 0x0009494D, 0x000A494D, 0x000A494D, 0x000A494D, 0x000C494D, 0x000C4352, 0x000D4453, 0x000D4453, 0x000C4352, 0x000B4251, 0x000A4150, 0x0009404F, 0x00083F4E, 0x000C404D, 0x000C4046, 0x003B6D69, 0x0033584E, 0x0016332A,
        0x00365955, 0x001D5556, 0x00438C90, 0x000B4A58, 0x00084755, 0x00044351, 0x00034250, 0x00044351, 0x00054452, 0x00054452, 0x00044351, 0x00084150, 0x00063B45, 0x0039696B, 0x00234E4B, 0x00325E57, 0x00204F4C, 0x00437678, 0x00245B60, 0x00456455,
        0x00204438, 0x00426C65, 0x002D5C59, 0x004B7D7D, 0x0019494A, 0x00376362, 0x00285251, 0x002E5B5F, 0x002C6564, 0x0033716B, 0x002A5C52, 0x003B5A51, 0x003D5751, 0x00345E57, 0x0039766C, 0x00396062, 0x003D6365, 0x002D5355, 0x003D6365, 0x0025484B,
        0x003D6063, 0x00294A4D, 0x00446568, 0x00214D4C, 0x003B6766, 0x00214D4C, 0x003D6968, 0x00214D4C, 0x00325E5D, 0x00255150, 0x00356160, 0x00335757, 0x003A5E5E, 0x00335757, 0x00365A5A, 0x00365A5A, 0x00335757, 0x00395D5D, 0x00305454, 0x003D6A6D,
        0x00275052, 0x003A625D, 0x00294F49, 0x0048706E, 0x001D4C50, 0x00104450, 0x00094152, 0x000B4249, 0x00053B42, 0x0008383E, 0x000F3A3D, 0x00123538, 0x000F2B2B, 0x000D2523, 0x00132825, 0x00162D2F, 0x000E3238, 0x000E3F4F, 0x00003249, 0x0003324E,
        0x001B445D, 0x00184354, 0x0016434E, 0x00005B75, 0x003FA4D1, 0x002490BA, 0x00005468, 0x00004457, 0x000F395E, 0x000C395A, 0x000E5154, 0x00034054, 0x00044155, 0x00034256, 0x00044357, 0x00044357, 0x00034558, 0x00024457, 0x00024457, 0x00014558,
        0x00024554, 0x00064755, 0x00064559, 0x00074760, 0x0002475B, 0x0007585B, 0x00288073, 0x00454C49, 0x00000200, 0x00000601, 0x0011150F, 0x0023261D, 0x002D3126, 0x003D3F33, 0x00323226, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFCFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00101513, 0x000A0B09, 0x00050E0B, 0x000F130E, 0x001D1613, 0x000F1613, 0x002D5855, 0x000D4F4E, 0x0006494C, 0x0000425B, 0x00023B51, 0x00044155, 0x00003E59, 0x0034759C,
        0x002E5B87, 0x001F4A6B, 0x0087BED3, 0x008EF9FC, 0x0089F8F4, 0x009AFAF4, 0x0078B9C2, 0x001B4D69, 0x001F5F82, 0x002C7093, 0x00003856, 0x00073F5A, 0x0008405B, 0x000A425D, 0x000B435E, 0x000B435E, 0x000A425D, 0x0009415C, 0x0008405B, 0x00054361,
        0x000D4862, 0x00084157, 0x000B4356, 0x000A4457, 0x00013D53, 0x00003E57, 0x00003855, 0x00003E52, 0x00003D51, 0x00003D51, 0x00003D51, 0x00003D51, 0x00003D51, 0x00003E52, 0x00003E52, 0x00003F4B, 0x00003D49, 0x0000424B, 0x0000434D, 0x0000404A,
        0x0000404E, 0x00034151, 0x00003949, 0x00164063, 0x00003347, 0x002C787E, 0x0074B1B3, 0x00183036, 0x006D686A, 0x00968F80, 0x00838869, 0x00D2BE9B, 0x00E0CDAA, 0x00C9BD9B, 0x00CAC3A2, 0x00B0AD91, 0x00AEAB96, 0x00887F71, 0x0074665A, 0x004C6365,
        0x000A4A4B, 0x0004414D, 0x00003550, 0x00006689, 0x0038B4DD, 0x002D93C4, 0x00004377, 0x00003D54, 0x0000435A, 0x00034960, 0x0002485F, 0x0000445B, 0x0000445B, 0x0000435A, 0x00003D54, 0x00003F5C, 0x00003F5C, 0x00003F5C, 0x00003F5C, 0x00003F5C,
        0x00003F5C, 0x00003F5C, 0x00003F5C, 0x0000415C, 0x0000405B, 0x00003F5A, 0x00003F5A, 0x00003F5A, 0x00003F5A, 0x0000405B, 0x0000415C, 0x00004259, 0x0000405B, 0x00003E60, 0x00065178, 0x003780AC, 0x002D719E, 0x003E7FAB, 0x003D7BA4, 0x002677A4,
        0x00287CA6, 0x002278A2, 0x00267EA7, 0x0020779F, 0x002B7BA4, 0x002B769D, 0x003278A0, 0x0031789E, 0x00347BA1, 0x002D749A, 0x002F789E, 0x00257096, 0x002C779D, 0x00267499, 0x00257398, 0x00257293, 0x002E7597, 0x00327096, 0x00397299, 0x0037729A,
        0x002F719A, 0x002B7AA1, 0x00227A9F, 0x0000405F, 0x0000405F, 0x00003F5E, 0x0000405F, 0x0000405F, 0x00014160, 0x00024261, 0x00034362, 0x0001415F, 0x0001415F, 0x0001415F, 0x0001415F, 0x0001415F, 0x0001415F, 0x0001415F, 0x0001415F, 0x0000455C,
        0x0000455C, 0x0001455C, 0x0001455C, 0x0002455C, 0x0002455C, 0x0004445C, 0x0004445C, 0x00013E5E, 0x00013E5E, 0x00013E5E, 0x00013E5E, 0x00013E5E, 0x00003D5D, 0x00003C5C, 0x00003B5B, 0x0000395C, 0x0000445C, 0x00337F8B, 0x001D6565, 0x00044143,
        0x001C5A64, 0x00196276, 0x00227994, 0x000B4856, 0x00094654, 0x00064351, 0x00054250, 0x00064351, 0x00074452, 0x00074452, 0x00064351, 0x00003C60, 0x00003D5D, 0x00003953, 0x00013B51, 0x00003A4E, 0x0005465B, 0x00165C74, 0x002F7894, 0x00496E72,
        0x00356069, 0x003E7582, 0x0037768A, 0x0022667D, 0x00003B54, 0x00003B55, 0x00053A55, 0x00003C5B, 0x0002566F, 0x00197685, 0x00236D79, 0x003D6A78, 0x00416674, 0x0030717A, 0x00248288, 0x00094C65, 0x00003851, 0x0000425B, 0x00002F48, 0x0000425B,
        0x0000364F, 0x00003D56, 0x00003B54, 0x00003850, 0x00023E56, 0x00003A52, 0x00003850, 0x00054159, 0x0000374F, 0x00013D55, 0x00003B53, 0x00003E55, 0x00004259, 0x00003B52, 0x00003D54, 0x00003F56, 0x00003E55, 0x00004259, 0x00003B52, 0x00023A55,
        0x00073F58, 0x00033950, 0x00093E53, 0x00043A51, 0x000D4764, 0x00003557, 0x00003B61, 0x00004561, 0x00003D58, 0x00003C54, 0x00004057, 0x00003F54, 0x0000384C, 0x00003649, 0x00003A4A, 0x000D423F, 0x00004542, 0x00004A48, 0x00004D50, 0x0014525C,
        0x000B4453, 0x00003D4B, 0x00055A69, 0x002AA5C9, 0x003294C2, 0x00024D73, 0x00003948, 0x00084B54, 0x00054157, 0x000B415A, 0x000E3E50, 0x00044155, 0x00024155, 0x00034256, 0x00034256, 0x00024457, 0x00024457, 0x00004457, 0x00004457, 0x00004255,
        0x00004251, 0x00034452, 0x00034256, 0x0004445D, 0x00004559, 0x00055659, 0x00277F72, 0x004F5653, 0x00000502, 0x00050B06, 0x00191D17, 0x00373A31, 0x0043473C, 0x003F4135, 0x00222216, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFEFF, 0xFFFEFDFF, 0xFFFDFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x000F1412, 0x000F100E, 0x00050E0B, 0x00080C07, 0x001D1613, 0x001A211E, 0x003A6562, 0x00165857, 0x0005484B, 0x0000445C, 0x00004255, 0x00073F52, 0x00103F5B, 0x00236187,
        0x0026739A, 0x0009486A, 0x004A6A87, 0x00A6DAEB, 0x00AEEFF8, 0x0090CBD4, 0x004A7285, 0x00294E70, 0x003B749B, 0x001F6485, 0x00003F58, 0x0009415C, 0x0009415C, 0x000A425D, 0x000A425D, 0x0009415C, 0x0009415C, 0x0009415C, 0x0009415C, 0x00003E5A,
        0x0004415B, 0x00094359, 0x000B4459, 0x000A4255, 0x00084053, 0x000D475A, 0x000A4559, 0x000E4554, 0x000D4453, 0x000D4453, 0x000D4453, 0x000E4554, 0x000F4655, 0x000F4655, 0x000E4554, 0x000A4D5C, 0x00064658, 0x000B495B, 0x000F4A5D, 0x000A4255,
        0x000B4152, 0x00124657, 0x00144757, 0x0000374F, 0x000F4353, 0x0024666B, 0x0064B1B3, 0x001D4950, 0x0044363C, 0x0071584E, 0x00717355, 0x00AA9579, 0x008D7E5E, 0x00827957, 0x0095906F, 0x00918D70, 0x00615C47, 0x003F3C2E, 0x00535048, 0x00405C5D,
        0x00004746, 0x00003C46, 0x00163F58, 0x00003150, 0x00126D94, 0x0043AADB, 0x001D97CD, 0x00135462, 0x00003F4D, 0x00003E4C, 0x00074856, 0x00034452, 0x00034452, 0x00064755, 0x00014250, 0x00064456, 0x00064456, 0x00064456, 0x00064456, 0x00064456,
        0x00064456, 0x00064456, 0x00064456, 0x00064456, 0x00054355, 0x00044254, 0x00044254, 0x00044254, 0x00044254, 0x00054355, 0x00064456, 0x00054A4D, 0x00044A51, 0x0000454F, 0x00004354, 0x00115A6E, 0x000A5369, 0x00115A70, 0x000C5269, 0x000E546C,
        0x00105870, 0x0009536B, 0x000B576D, 0x0007566B, 0x000C596C, 0x00085467, 0x000D566A, 0x00145669, 0x0015596C, 0x00105467, 0x001A5E71, 0x00195D70, 0x00195D70, 0x0014596C, 0x0013586B, 0x000C5765, 0x00155E6C, 0x000F5463, 0x00135665, 0x00135867,
        0x000E5667, 0x000C5A6A, 0x0000505F, 0x00074450, 0x00074450, 0x0006434F, 0x0006434F, 0x0006434F, 0x0006434F, 0x00074450, 0x00074450, 0x00074255, 0x00074255, 0x00074255, 0x00074255, 0x00074255, 0x00074255, 0x00074255, 0x00074255, 0x00064651,
        0x00064651, 0x00064651, 0x00084551, 0x00084551, 0x00084551, 0x00094551, 0x00094551, 0x000A4652, 0x000A4652, 0x000A4652, 0x000A4652, 0x000A4652, 0x00094551, 0x00084450, 0x0007434F, 0x00003D56, 0x00023D4D, 0x003F6C70, 0x00627D7A, 0x002F4140,
        0x005B797E, 0x0035717D, 0x00015769, 0x0000475B, 0x0000465A, 0x0000465A, 0x0000465A, 0x0000475B, 0x0000475B, 0x0000475B, 0x0000465A, 0x000F4556, 0x00124958, 0x00083D4A, 0x00164E59, 0x0007424C, 0x0004424C, 0x00003C47, 0x00145865, 0x003F6157,
        0x002C5550, 0x00285C5C, 0x00236266, 0x0015565F, 0x00084551, 0x000C4350, 0x000F434F, 0x000F3C51, 0x00074654, 0x00185F63, 0x00295C5E, 0x00455C5E, 0x004A5B5E, 0x00386261, 0x0019605D, 0x000C4F58, 0x0002454E, 0x0002434C, 0x0003444D, 0x0004454E,
        0x00094751, 0x0005434D, 0x0004424C, 0x00094650, 0x0008454F, 0x000B4852, 0x00013E48, 0x00104D57, 0x00023F49, 0x000C4953, 0x0007444E, 0x00044651, 0x00064853, 0x0002444F, 0x0002444F, 0x00044651, 0x00054752, 0x00074954, 0x0002444F, 0x000C414E,
        0x000C424D, 0x00164B55, 0x00043943, 0x00104651, 0x000B4251, 0x000E4959, 0x000D485B, 0x000A4A55, 0x00074752, 0x0004454E, 0x0007464E, 0x0008474F, 0x0008484D, 0x0007474B, 0x0008474B, 0x00184658, 0x0008434C, 0x00014243, 0x00155151, 0x0016474F,
        0x0000364D, 0x00126287, 0x0037A0CB, 0x002896C6, 0x00004E6D, 0x00054459, 0x0013455C, 0x00053F52, 0x00034A4E, 0x0000424C, 0x00063C5B, 0x00034256, 0x00034256, 0x00034256, 0x00014356, 0x00014356, 0x00004356, 0x00004457, 0x00004457, 0x00004255,
        0x00004251, 0x00034452, 0x00034256, 0x0004445D, 0x00004559, 0x0006575A, 0x00288073, 0x0039403D, 0x00000200, 0x00010702, 0x000C100A, 0x0025281F, 0x002C3025, 0x00282A1E, 0x0018180C, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFDFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x000E1311, 0x000F100E, 0x0009120F, 0x000B0F0A, 0x001F1815, 0x00161D1A, 0x00325D5A, 0x00115352, 0x00014447, 0x00003F5A, 0x00024659, 0x000C3E50, 0x00123751, 0x000A4769,
        0x002482A6, 0x00186586, 0x002E4C6F, 0x00435878, 0x00496F87, 0x00365F75, 0x00274463, 0x003E6189, 0x00356D96, 0x00004461, 0x00054458, 0x000B435E, 0x000A425D, 0x0009415C, 0x0008405B, 0x0008405B, 0x0008405B, 0x0009415C, 0x0009415C, 0x00004059,
        0x0004425A, 0x00023E56, 0x00064056, 0x000E475C, 0x000D4558, 0x00084051, 0x000D4354, 0x000B455B, 0x000A445A, 0x000A445A, 0x000A445A, 0x000A445A, 0x000A445A, 0x00094359, 0x00084258, 0x0001435C, 0x0003425E, 0x0003415F, 0x00033E5E, 0x00013D5B,
        0x0007425C, 0x0008445A, 0x00044155, 0x00004A5C, 0x00083947, 0x00083F48, 0x0062B9BD, 0x00387982, 0x0014151F, 0x00735A56, 0x00B1B096, 0x0082725B, 0x00756A4F, 0x00757051, 0x007C7758, 0x00867D62, 0x004A4332, 0x0030332A, 0x003B4945, 0x0020494B,
        0x00004548, 0x00003D4C, 0x001A435A, 0x000E3C54, 0x00013D5B, 0x000F648C, 0x0030B3DE, 0x003689A8, 0x00095C7B, 0x00004867, 0x00005170, 0x00004D6C, 0x00004766, 0x00004B6A, 0x00004F6E, 0x00004E62, 0x00004E62, 0x00004E62, 0x00004E62, 0x00004E62,
        0x00004E62, 0x00004E62, 0x00004E62, 0x00004D63, 0x00004D63, 0x00004D63, 0x00004C62, 0x00004C62, 0x00004D63, 0x00004D63, 0x00004D63, 0x00004E55, 0x0002515A, 0x0002525D, 0x00004C58, 0x00015463, 0x00005162, 0x00005565, 0x00004E60, 0x000A4B60,
        0x000D4E63, 0x000A4E61, 0x00094E61, 0x00044C5D, 0x00044F5F, 0x00014E5E, 0x00055360, 0x00024F5F, 0x00075464, 0x00014E5E, 0x00004D5D, 0x00004C5C, 0x00024D5D, 0x00024D5D, 0x00034E5E, 0x00004D59, 0x00035763, 0x0000505C, 0x0000525C, 0x0000545E,
        0x0000515B, 0x0000545E, 0x00004F59, 0x0000535B, 0x0000535B, 0x0000525A, 0x00005159, 0x00005159, 0x00005159, 0x0000525A, 0x0000525A, 0x00005264, 0x00005264, 0x00005264, 0x00005264, 0x00005264, 0x00005264, 0x00005264, 0x00005264, 0x00005362,
        0x00005362, 0x00005362, 0x00005362, 0x00005362, 0x00005362, 0x00025362, 0x00025362, 0x0002555D, 0x0003565E, 0x0003565E, 0x0003565E, 0x0003565E, 0x0002555D, 0x0001545C, 0x0001545C, 0x00084D68, 0x0000465A, 0x00044B55, 0x000D4A4E, 0x00003336,
        0x001A535C, 0x00095162, 0x00004B5F, 0x000A4E55, 0x000B4F56, 0x000B4F56, 0x000C5057, 0x000C5057, 0x000C5057, 0x000C5057, 0x000B4F56, 0x00024C58, 0x00004450, 0x0004505C, 0x0003525D, 0x00014F5C, 0x00004754, 0x00004F5B, 0x00004A56, 0x00074644,
        0x00003C3D, 0x00014B4F, 0x0000444C, 0x0000515D, 0x00004652, 0x0004535E, 0x00004955, 0x00044C5E, 0x00004652, 0x00004F54, 0x00004548, 0x00023F43, 0x00063E43, 0x00054C50, 0x0000494D, 0x00004A56, 0x00034D59, 0x00004753, 0x000A525E, 0x00004450,
        0x00074D5A, 0x00004552, 0x00044A57, 0x00004A55, 0x0000525D, 0x00004C57, 0x0001535E, 0x00004853, 0x00004D58, 0x00004B56, 0x0000525D, 0x00004957, 0x00014C5A, 0x00004A58, 0x00004957, 0x00004A58, 0x00014C5A, 0x00024D5B, 0x00004A58, 0x0004535E,
        0x00004853, 0x0005545F, 0x00004651, 0x00095865, 0x00004451, 0x00004F5C, 0x00004D5D, 0x00054C5A, 0x00064E5A, 0x00044C58, 0x00004854, 0x00004854, 0x00024A54, 0x00004B54, 0x00004851, 0x0000406D, 0x00004166, 0x0000415C, 0x0002455C, 0x0000425D,
        0x0012618A, 0x003293CB, 0x002192D4, 0x00004773, 0x00004152, 0x00004450, 0x00093F5D, 0x0012415D, 0x000E484E, 0x0000434A, 0x00004663, 0x00044357, 0x00024457, 0x00014356, 0x00014356, 0x00004356, 0x00004356, 0x00004457, 0x00004457, 0x00014558,
        0x00024554, 0x00054654, 0x00054458, 0x0006465F, 0x0003485C, 0x00095A5D, 0x002C8477, 0x00454C49, 0x00000401, 0x00070D08, 0x000A0E08, 0x00181B12, 0x001C2015, 0x00212317, 0x00252519, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFDFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x000D1210, 0x00090A08, 0x000A1310, 0x0010140F, 0x001F1815, 0x000C1310, 0x00224D4A, 0x00094B4A, 0x00024548, 0x00003C59, 0x00094055, 0x00124353, 0x0006384E, 0x00003C5E,
        0x001A658C, 0x002C7399, 0x002F698C, 0x00294C78, 0x00185073, 0x00175573, 0x00366A8F, 0x004376A1, 0x001B5981, 0x00003C58, 0x00114555, 0x000B435E, 0x000A425D, 0x000A425D, 0x0009415C, 0x0008405B, 0x0008405B, 0x0009415C, 0x0009415C, 0x00044358,
        0x000B4961, 0x0008465E, 0x00013F57, 0x0007435B, 0x000B455B, 0x000C4358, 0x0010485B, 0x000A4457, 0x000A4457, 0x000A4457, 0x00094356, 0x00094356, 0x00084255, 0x00094356, 0x000A4457, 0x00084353, 0x000D485B, 0x000C465C, 0x0006425A, 0x0008445A,
        0x00094A59, 0x00074A53, 0x0003474C, 0x00014758, 0x00174856, 0x00033541, 0x00307C88, 0x006AB8C5, 0x00214955, 0x001A1F1D, 0x007A7360, 0x00635B4A, 0x005B5641, 0x005D5A3E, 0x008B8366, 0x008F816B, 0x00383023, 0x001C2724, 0x00254344, 0x00094244,
        0x00002D37, 0x00064056, 0x000C4156, 0x00003C4B, 0x00093D4E, 0x00033B58, 0x00046787, 0x003AB2FA, 0x002EA6EE, 0x0028A0E8, 0x002EA6EE, 0x0033ABF3, 0x002FA7EF, 0x002DA5ED, 0x0031A9F1, 0x002EA7E5, 0x002EA7E5, 0x002EA7E5, 0x002EA7E5, 0x002EA7E5,
        0x002EA7E5, 0x002EA7E5, 0x002EA7E5, 0x002FA5E6, 0x002FA5E6, 0x002FA5E6, 0x0030A6E7, 0x0030A6E7, 0x002FA5E6, 0x002FA5E6, 0x002FA5E6, 0x002FA2DF, 0x002FA1DD, 0x0032A5DE, 0x0032A3DC, 0x0031A4DD, 0x002FA3DC, 0x002CA2DD, 0x002CA2DD, 0x001DA7DC,
        0x001FA6DC, 0x0021A7DD, 0x0022A9DD, 0x001DA4D8, 0x001CA3D5, 0x001BA4D5, 0x0018A3D4, 0x0014A2D7, 0x001AA8DD, 0x001AA8DD, 0x0019A5DA, 0x001BA7DC, 0x001FA9DE, 0x001AA4D9, 0x001CA6DB, 0x002B9BD7, 0x002FA1DD, 0x00289FD7, 0x0027A0D8, 0x002AA4DA,
        0x0029A0D7, 0x002DA1D6, 0x0032A4D9, 0x002AA1D8, 0x002AA1D8, 0x0029A0D7, 0x0029A0D7, 0x0029A0D7, 0x002AA1D8, 0x002AA1D8, 0x002BA2D9, 0x00289ED9, 0x00289ED9, 0x00289ED9, 0x00289ED9, 0x00289ED9, 0x00289ED9, 0x00289ED9, 0x00289ED9, 0x00249FD9,
        0x00249FD9, 0x00269ED9, 0x00269ED9, 0x00289ED9, 0x00289ED9, 0x002A9ED9, 0x002A9ED9, 0x00269DD4, 0x00279ED5, 0x00279ED5, 0x00279ED5, 0x00279ED5, 0x00279ED5, 0x00269DD4, 0x00259CD3, 0x003192D4, 0x002B9FDA, 0x001DA4D8, 0x000C98C7, 0x0026A8D7,
        0x00219ACC, 0x002297CE, 0x002AA2DD, 0x002599D8, 0x002498D7, 0x002498D7, 0x002498D7, 0x002397D6, 0x002397D6, 0x002498D7, 0x002498D7, 0x002B9FD6, 0x002296CD, 0x00279BD4, 0x002094CF, 0x001E91CE, 0x001F92CF, 0x002C9EDA, 0x002494D0, 0x002497CA,
        0x002195CA, 0x002C9FD8, 0x002598D5, 0x00269BD8, 0x002096D1, 0x00299DD6, 0x002297CE, 0x001C96D2, 0x001A97D1, 0x0021A1D6, 0x001D9CCF, 0x001E9BCE, 0x001793C9, 0x001E9BD5, 0x001E9ED9, 0x002294D0, 0x002496D2, 0x002898D4, 0x002494D0, 0x002A98D4,
        0x002492CE, 0x002994D1, 0x002994D1, 0x001B96CE, 0x00219CD4, 0x001C97CF, 0x00249FD7, 0x001691C9, 0x00209BD3, 0x001C97CF, 0x00249FD7, 0x002596D4, 0x002899D7, 0x00299AD8, 0x002798D6, 0x002697D5, 0x00299AD8, 0x00299AD8, 0x002899D7, 0x00279ED6,
        0x00229BD3, 0x001E99D3, 0x0028A2DE, 0x0026A0DC, 0x00229DD7, 0x00219AD2, 0x00269DD5, 0x00299AD8, 0x002A9BD9, 0x00299AD8, 0x002596D4, 0x002394D2, 0x002495D3, 0x002495D3, 0x002293D1, 0x001696C6, 0x00219ACB, 0x002A9ECD, 0x002098C6, 0x001A9AC5,
        0x002BAED6, 0x001B97C0, 0x0000537C, 0x00004051, 0x00044756, 0x00014054, 0x0008405B, 0x00063B56, 0x00033E52, 0x00084D5E, 0x00003B4D, 0x00034558, 0x00034558, 0x00024457, 0x00004356, 0x00004356, 0x00004356, 0x00004457, 0x00004558, 0x00024659,
        0x00034655, 0x00064755, 0x00054458, 0x00074760, 0x0004495D, 0x000A5B5E, 0x002D8578, 0x00575E5B, 0x00000200, 0x00030904, 0x000E120C, 0x001F2219, 0x00282C21, 0x00313327, 0x00333327, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFFFEFF, 0xFFFFFEFF, 0xFFFDFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x000D1210, 0x00060705, 0x0007100D, 0x000B0F0A, 0x001D1613, 0x000E1512, 0x00285350, 0x00125453, 0x00094C4F, 0x00093F5D, 0x000F3950, 0x00174B58, 0x00004455, 0x00003E5D,
        0x0000315D, 0x001F5D85, 0x00207B9A, 0x004385B5, 0x00267BA1, 0x001C7698, 0x0026739A, 0x00084C79, 0x0000365E, 0x00084861, 0x00103D4A, 0x000A425D, 0x000A425D, 0x000A425D, 0x000A425D, 0x0009415C, 0x0009415C, 0x0008405B, 0x0008405B, 0x0009465A,
        0x00014055, 0x0003435B, 0x0004445D, 0x00003F58, 0x0007425C, 0x000C465C, 0x000C4259, 0x0005455E, 0x0005455E, 0x0005455E, 0x0004445D, 0x0003435C, 0x0004445D, 0x00074760, 0x00094962, 0x00084853, 0x000C4D5C, 0x00094B5E, 0x0005485D, 0x00054A5D,
        0x00004955, 0x0000494C, 0x00005250, 0x000C4659, 0x000E424F, 0x00184957, 0x00002B3B, 0x0058AABB, 0x0071BFC6, 0x001F413B, 0x00221505, 0x00302D1F, 0x00393A26, 0x0036371D, 0x007B7257, 0x007E6D5A, 0x00393026, 0x00243433, 0x00193F43, 0x00003639,
        0x000B4757, 0x000D3C58, 0x00003C51, 0x00004C53, 0x0007424B, 0x001D4054, 0x0001415A, 0x0000527E, 0x0012739F, 0x001879A5, 0x000B6C98, 0x000F709C, 0x001374A0, 0x000E6F9B, 0x0010719D, 0x00116F9D, 0x00116F9D, 0x00116F9D, 0x00116F9D, 0x00116F9D,
        0x00116F9D, 0x00116F9D, 0x00116F9D, 0x00116D9E, 0x00116D9E, 0x00126E9F, 0x00136FA0, 0x00136FA0, 0x00126E9F, 0x00116D9E, 0x00116D9E, 0x001074AA, 0x00086B9F, 0x000E6D9E, 0x0013719E, 0x0011709B, 0x000E709A, 0x000B6F99, 0x000F75A0, 0x000C759C,
        0x00096E95, 0x000D6D95, 0x00136E95, 0x00146C94, 0x00187095, 0x00197196, 0x00126B90, 0x001B729E, 0x00166B98, 0x00146996, 0x00136693, 0x001A6D9A, 0x001D6E9B, 0x00176593, 0x001E6C9A, 0x00116D9C, 0x000D6D9B, 0x00056A97, 0x00036C97, 0x0009739C,
        0x00086E98, 0x000A6A92, 0x00136F98, 0x000F6A97, 0x000F6A97, 0x000F6A97, 0x000F6A97, 0x00106B98, 0x00116C99, 0x00126D9A, 0x00136E9B, 0x00106D94, 0x00106D94, 0x00106D94, 0x00106D94, 0x00106D94, 0x00106D94, 0x00106D94, 0x00106D94, 0x000E6D95,
        0x000E6D95, 0x000E6D95, 0x00106C95, 0x00106C95, 0x00126C95, 0x00126C95, 0x00126C95, 0x00146C9A, 0x00146C9A, 0x00146C9A, 0x00156D9B, 0x00156D9B, 0x00146C9A, 0x00146C9A, 0x00136B99, 0x00146D99, 0x000F6E96, 0x0005688E, 0x001F7A9F, 0x00106389,
        0x00196E94, 0x000C6D91, 0x00006E90, 0x000E7295, 0x000D7194, 0x000B6F92, 0x00096D90, 0x00086C8F, 0x00086C8F, 0x00096D90, 0x000B6F92, 0x00096890, 0x0013719C, 0x00096795, 0x001673A4, 0x000C679A, 0x001972A5, 0x000F6496, 0x00176B9B, 0x000B709D,
        0x001072A2, 0x00066295, 0x001670A6, 0x00065E94, 0x001975A6, 0x000E6C99, 0x0012719C, 0x001F6EA1, 0x00206D9E, 0x001E6A9A, 0x00176A97, 0x001976A3, 0x000D6A9B, 0x001368A0, 0x001D69A4, 0x000B73A2, 0x000F75A5, 0x00096F9F, 0x00096F9F, 0x001074A4,
        0x000F73A3, 0x000E70A0, 0x000D6F9F, 0x001476A4, 0x000D6F9D, 0x001375A3, 0x000B6D9B, 0x001476A4, 0x001072A0, 0x001577A5, 0x001274A2, 0x001077A8, 0x00137AAB, 0x00177EAF, 0x00147BAC, 0x001279AA, 0x00157CAD, 0x00157CAD, 0x00157CAD, 0x001A7DA9,
        0x001E83B0, 0x001C82B2, 0x001E84B5, 0x00197FB0, 0x001D81B1, 0x002083AF, 0x002082AC, 0x001581B1, 0x001581B1, 0x001581B1, 0x00157EAF, 0x00137CAD, 0x001379AD, 0x00147AAE, 0x00157BAF, 0x001881AA, 0x001473A5, 0x001B72B2, 0x001775B6, 0x001980B7,
        0x0017769D, 0x000C4F64, 0x00173F4B, 0x00005147, 0x00074154, 0x00204567, 0x000C3D53, 0x00004157, 0x00004566, 0x00003753, 0x001D4D53, 0x00044659, 0x00034558, 0x00024457, 0x00004356, 0x00004356, 0x00004457, 0x00004457, 0x00004558, 0x00004457,
        0x00014453, 0x00044553, 0x00034256, 0x0005455E, 0x0002475B, 0x00095A5D, 0x002C8477, 0x00626966, 0x00000200, 0x00040A05, 0x00181C16, 0x00282B22, 0x0031352A, 0x00393B2F, 0x00333327, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
  0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFCFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x000A1211, 0x0013130D, 0x0013100C, 0x0015100F, 0x00101213, 0x00061A1B, 0x00315F60, 0x0007504E, 0x00004D4A, 0x00034256, 0x00034256, 0x00034256, 0x00034256, 0x00024155,
        0x00014054, 0x00014054, 0x00003F53, 0x000E5062, 0x000C4E60, 0x00084A5C, 0x00044658, 0x00014355, 0x00004254, 0x00004254, 0x00014355, 0x000A4558, 0x00094457, 0x00084356, 0x00084356, 0x00084356, 0x00094457, 0x000A4558, 0x000B4659, 0x00084454,
        0x00094555, 0x000A4656, 0x00094555, 0x00084454, 0x00084454, 0x000A4656, 0x000C4858, 0x00004553, 0x00004A58, 0x00024D5B, 0x00014C5A, 0x00004B59, 0x00014C5A, 0x00024D5B, 0x00014C5A, 0x000C4E61, 0x000D4F62, 0x000E5063, 0x000D4F62, 0x000C4E61,
        0x000C4E61, 0x000C4E61, 0x000D4F62, 0x00035657, 0x00055052, 0x000A494D, 0x000F464B, 0x00285F64, 0x0067A6AA, 0x0069B4B6, 0x000F6263, 0x001D2B31, 0x00191F26, 0x0028232C, 0x004A414B, 0x0054515A, 0x002D3D43, 0x000B3133, 0x000D4141, 0x00053E4D,
        0x000A4352, 0x000C4554, 0x00074251, 0x00064150, 0x00074452, 0x00074452, 0x0004414F, 0x00003F47, 0x00003F47, 0x00003F47, 0x00003F47, 0x00003F47, 0x00003F47, 0x00003F47, 0x00003F47, 0x00063F48, 0x00063F48, 0x00053E47, 0x00053E47, 0x00053E47,
        0x00063F48, 0x00074049, 0x00074049, 0x00014048, 0x00014048, 0x00003F47, 0x00003E46, 0x00003D45, 0x00003F47, 0x0003424A, 0x0005444C, 0x00094349, 0x00084248, 0x00074147, 0x00084248, 0x00094349, 0x000A444A, 0x00094349, 0x00084248, 0x0002434C,
        0x0001424B, 0x0001424B, 0x0001424B, 0x0000414A, 0x00004049, 0x00004049, 0x00004049, 0x00004447, 0x00004548, 0x00004548, 0x00004548, 0x00004548, 0x00004649, 0x00004649, 0x00004649, 0x0000474A, 0x0000474A, 0x0000474A, 0x0000474A, 0x0000474A,
        0x00004649, 0x00004649, 0x00004649, 0x00004448, 0x00004448, 0x00004549, 0x00004549, 0x0000464A, 0x0000474B, 0x0000474B, 0x0000484C, 0x00054549, 0x00044448, 0x00044448, 0x00034347, 0x00044448, 0x00054549, 0x0006464A, 0x0007474B, 0x0008484D,
        0x0008484D, 0x0008484D, 0x0008484D, 0x0008484D, 0x0008484D, 0x0008484D, 0x0008484D, 0x0006464A, 0x0006464A, 0x0006464A, 0x0006464A, 0x0006464A, 0x0006464A, 0x0006464A, 0x0006464A, 0x00034348, 0x00034348, 0x00034348, 0x00044449, 0x00044449,
        0x00044449, 0x00044449, 0x0005454A, 0x00054549, 0x00044448, 0x00044448, 0x00034347, 0x00024246, 0x00014145, 0x00014145, 0x00004044, 0x00024246, 0x00024246, 0x00024246, 0x00024246, 0x00024246, 0x00024246, 0x00024246, 0x00024246, 0x00054349,
        0x0006444A, 0x0006444A, 0x0005454A, 0x0005454A, 0x0003454A, 0x0003454A, 0x00024449, 0x00054549, 0x00054549, 0x00054549, 0x00054549, 0x00054549, 0x00054549, 0x00054549, 0x00054549, 0x0006464A, 0x0006464A, 0x00054549, 0x00054549, 0x00044448,
        0x00044448, 0x00044448, 0x00044448, 0x0003444C, 0x0003444C, 0x0005464E, 0x0006474F, 0x0006474F, 0x0006474F, 0x0006474F, 0x0005464E, 0x00004A4B, 0x00004E4F, 0x00004E4F, 0x00044F51, 0x00095355, 0x00074E51, 0x00064B4E, 0x00105356, 0x000A5155,
        0x000A5155, 0x000B5256, 0x000D5458, 0x000F565A, 0x000F565A, 0x000C5357, 0x00095054, 0x000D4E56, 0x000D4E56, 0x00084C53, 0x0001474E, 0x0000474E, 0x00014B51, 0x00014B51, 0x0000474D, 0x0006444C, 0x0006444C, 0x0007454D, 0x0008464E, 0x0009474F,
        0x000A4850, 0x000B4951, 0x000C4A52, 0x000B4856, 0x000B4856, 0x000A4755, 0x00094654, 0x00094654, 0x000A4755, 0x000B4856, 0x000B4856, 0x00054353, 0x00054353, 0x00054353, 0x00054353, 0x00064454, 0x00064454, 0x00074555, 0x00084656, 0x000D4858,
        0x00074353, 0x00084656, 0x00084958, 0x00074C5B, 0x00004755, 0x0005505E, 0x002E7987, 0x00474F48, 0x00000400, 0x0010150C, 0x002E3128, 0x00313026, 0x00343228, 0x003D392E, 0x003A3429, 0x0034352B, 0xFFCCCDC4, 0xFFFFFFFC, 0xFFFDFBFA,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00091110, 0x000C0D09, 0x0012110D, 0x00100C0B, 0x000E0E0E, 0x000D1C1E, 0x003F6565, 0x00215D5C, 0x00075250, 0x00054654, 0x00044553, 0x00044553, 0x00044553, 0x00044553,
        0x00054654, 0x00064755, 0x00074856, 0x00074752, 0x00074752, 0x00064651, 0x00064651, 0x00054550, 0x00064651, 0x00064651, 0x00064651, 0x00004650, 0x00004751, 0x00004751, 0x00004751, 0x00014852, 0x00014852, 0x00014852, 0x00014852, 0x00024A52,
        0x00034B53, 0x00044C54, 0x00034B53, 0x00024A52, 0x00014951, 0x00024A52, 0x00044C54, 0x000F4D57, 0x000B4953, 0x000A4852, 0x000B4953, 0x0007454F, 0x00084650, 0x00185660, 0x002D6B75, 0x002B686C, 0x002D6A6E, 0x002E6B6F, 0x002D6A6E, 0x002B686C,
        0x0029666A, 0x00286569, 0x00286569, 0x00267370, 0x002B7472, 0x00337373, 0x003A7374, 0x00336C6D, 0x00377777, 0x004D9694, 0x00519E9B, 0x00366965, 0x0023504D, 0x00123735, 0x000B2D2C, 0x000C3331, 0x000C3F3B, 0x0001453E, 0x0000443B, 0x0000434A,
        0x0001454C, 0x0000464D, 0x0000444B, 0x0000444B, 0x0000464D, 0x0000474E, 0x0000484E, 0x00004A4C, 0x00004A4C, 0x00004A4C, 0x00004A4C, 0x00004A4C, 0x00014B4D, 0x00014B4D, 0x00014B4D, 0x00004B4B, 0x00004B4B, 0x00004C4C, 0x00004C4C, 0x00004B4B,
        0x00004A4A, 0x00004949, 0x00004949, 0x00044E50, 0x00054F51, 0x00054F51, 0x00034D4F, 0x00004A4C, 0x0000494B, 0x0000494B, 0x00004A4C, 0x00004D4A, 0x00004C49, 0x00004B48, 0x00004C49, 0x00004D4A, 0x00004E4B, 0x00004D4A, 0x00004C49, 0x00084B4E,
        0x00084B4E, 0x00074A4D, 0x00074A4D, 0x0006494C, 0x0006494C, 0x0005484B, 0x0005484B, 0x00034948, 0x00034948, 0x00034948, 0x00034948, 0x00034948, 0x00034948, 0x00034948, 0x00034948, 0x00074D4C, 0x00074D4C, 0x00074D4C, 0x00074D4C, 0x00074D4C,
        0x00064C4B, 0x00064C4B, 0x00064C4B, 0x00014B4B, 0x00014B4B, 0x00014B4B, 0x00014B4B, 0x00024C4C, 0x00024C4C, 0x00024C4C, 0x00024C4C, 0x00084A49, 0x00084A49, 0x00094B4A, 0x00094B4A, 0x000A4C4B, 0x000A4C4B, 0x000A4C4B, 0x000A4C4B, 0x00094A4B,
        0x00094A4B, 0x00094A4B, 0x00094A4B, 0x00094A4B, 0x00094A4B, 0x00094A4B, 0x00094A4B, 0x00084A49, 0x00084A49, 0x00084A49, 0x00084A49, 0x00084A49, 0x00084A49, 0x00084A49, 0x00084A49, 0x00064748, 0x00064748, 0x00064748, 0x00074849, 0x00074849,
        0x00074849, 0x00074849, 0x00074849, 0x00074948, 0x00074948, 0x00074948, 0x00064847, 0x00064847, 0x00054746, 0x00054746, 0x00044645, 0x00004B49, 0x00004B49, 0x00004B49, 0x00004B49, 0x00004B49, 0x00004B49, 0x00004B49, 0x00004B49, 0x00024A4A,
        0x00034B4B, 0x00054D4D, 0x00044E4E, 0x00044E4E, 0x00024E4D, 0x00004C4B, 0x00004B4A, 0x00094B4A, 0x00094B4A, 0x00094B4A, 0x00094B4A, 0x00094B4A, 0x00094B4A, 0x00094B4A, 0x00094B4A, 0x00084A49, 0x00084A49, 0x00084A49, 0x00074948, 0x00074948,
        0x00064847, 0x00064847, 0x00064847, 0x0006494C, 0x0006494C, 0x00074A4D, 0x00074A4D, 0x00084B4E, 0x00084B4E, 0x00084B4E, 0x00084B4E, 0x00135756, 0x00185A59, 0x00175958, 0x00185A59, 0x001C5C5C, 0x001E5C5C, 0x001D5B5B, 0x001E5C5C, 0x001C595B,
        0x001C595B, 0x001D5A5C, 0x00205D5F, 0x00236062, 0x00246163, 0x00236062, 0x00215E60, 0x001A5F62, 0x00175C5F, 0x00115659, 0x000C5154, 0x000A4F52, 0x000A4F52, 0x00084D50, 0x00054A4D, 0x00074E51, 0x00064D50, 0x00064D50, 0x00054C4F, 0x00054C4F,
        0x00044B4E, 0x00044B4E, 0x00034A4D, 0x0000484F, 0x0000484F, 0x0000474E, 0x0000474E, 0x0000474E, 0x0000474E, 0x0000484F, 0x0000484F, 0x0000474F, 0x0000474F, 0x0000474F, 0x0000474F, 0x00004850, 0x00004951, 0x00014A52, 0x00024B53, 0x00084D56,
        0x00014951, 0x00004850, 0x00004850, 0x00004F56, 0x00005158, 0x000C6066, 0x0031878D, 0x00454D46, 0x000A110A, 0x001E231A, 0x003A3D34, 0x00434238, 0x00444238, 0x004B473C, 0x00433D32, 0x003F4036, 0xFF9D9E95, 0xFFCFCECA, 0xFFFFFFFE,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFCFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x0008100F, 0x00050B06, 0x0010110D, 0x000F0B0A, 0x00141212, 0x00161E1E, 0x003D5555, 0x002B5554, 0x00154A47, 0x0018484C, 0x0017474B, 0x00144448, 0x00124246, 0x00114145,
        0x00114145, 0x00124246, 0x00134347, 0x000E4445, 0x000E4445, 0x000F4546, 0x000F4546, 0x000F4546, 0x000E4445, 0x000D4344, 0x000D4344, 0x000A4646, 0x000B4747, 0x000D4949, 0x000E4A4A, 0x000E4A4A, 0x000D4949, 0x000C4848, 0x000C4848, 0x000C4847,
        0x000D4948, 0x000D4948, 0x000D4948, 0x000C4847, 0x000B4746, 0x000C4847, 0x000D4948, 0x00204C4D, 0x001B4748, 0x001A4647, 0x001D494A, 0x001C4849, 0x001E4A4B, 0x002E5A5B, 0x00416D6E, 0x003A6C60, 0x003C6E62, 0x003F7165, 0x003F7165, 0x003E7064,
        0x003C6E62, 0x003B6D61, 0x003A6C60, 0x00537372, 0x004C6C6B, 0x00426261, 0x00466564, 0x00466564, 0x003B5B5A, 0x00345453, 0x002C4C4B, 0x001B3F39, 0x0013352F, 0x000B2B26, 0x00072520, 0x000D2D28, 0x00234943, 0x0026524B, 0x000F4038, 0x000D4643,
        0x000A4340, 0x0009423F, 0x000A4541, 0x000B4642, 0x000A4743, 0x000B4844, 0x000D4A46, 0x000B4844, 0x000B4844, 0x000C4945, 0x000C4945, 0x000C4945, 0x000D4A46, 0x000D4A46, 0x000D4A46, 0x000B4642, 0x000E4945, 0x00114C48, 0x00124D49, 0x00124D49,
        0x00114C48, 0x00124D49, 0x00144F4B, 0x000A4743, 0x000C4945, 0x000E4B47, 0x000E4B47, 0x000C4945, 0x000A4743, 0x00094642, 0x00094642, 0x000C4743, 0x000B4642, 0x000A4541, 0x000B4642, 0x000C4743, 0x000C4743, 0x000C4743, 0x000A4541, 0x000D3F3B,
        0x000D3F3B, 0x000C3E3A, 0x000C3E3A, 0x000B3D39, 0x000B3D39, 0x000A3C38, 0x000A3C38, 0x000B3E3A, 0x000B3E3A, 0x000B3E3A, 0x000B3E3A, 0x000B3E3A, 0x000B3E3A, 0x000A3D39, 0x000A3D39, 0x000F423E, 0x000F423E, 0x000F423E, 0x000E413D, 0x000E413D,
        0x000E413D, 0x000E413D, 0x000E413D, 0x00114643, 0x00114643, 0x00114643, 0x00114643, 0x00104542, 0x00104542, 0x00104542, 0x00104542, 0x00054942, 0x00064A43, 0x00074B44, 0x00084C45, 0x00094D46, 0x00084C45, 0x00074B44, 0x00064A43, 0x00064944,
        0x00064944, 0x00064944, 0x00064944, 0x00064944, 0x00064944, 0x00064944, 0x00064944, 0x00064A43, 0x00064A43, 0x00064A43, 0x00064A43, 0x00064A43, 0x00064A43, 0x00064A43, 0x00064A43, 0x00054843, 0x00054843, 0x00054843, 0x00064944, 0x00064944,
        0x00064944, 0x00064944, 0x00064944, 0x00064A43, 0x00064A43, 0x00064A43, 0x00064A43, 0x00064A43, 0x00054942, 0x00054942, 0x00054942, 0x000A4842, 0x000A4842, 0x000A4842, 0x000A4842, 0x000A4842, 0x000A4842, 0x000A4842, 0x000A4842, 0x000A4541,
        0x000B4642, 0x000D4844, 0x000D4A46, 0x000D4A46, 0x000B4844, 0x00074742, 0x00064641, 0x0000443D, 0x0000443D, 0x0000443D, 0x0000443D, 0x0000443D, 0x0000443D, 0x0000443D, 0x0000443D, 0x00034740, 0x00034740, 0x00034740, 0x00034740, 0x0002463F,
        0x0002463F, 0x0002463F, 0x0002463F, 0x00024842, 0x00024842, 0x00024842, 0x00024842, 0x00024842, 0x00034943, 0x00034943, 0x00044A44, 0x001C4946, 0x001E4B48, 0x00224F4C, 0x0022514E, 0x001E504C, 0x001F514D, 0x00215450, 0x0020534F, 0x00225351,
        0x00215250, 0x00215250, 0x00225351, 0x00235452, 0x00235452, 0x00235452, 0x00225351, 0x0011534E, 0x0011514C, 0x000F4F4A, 0x00104D49, 0x00114A47, 0x00104744, 0x00114845, 0x00164B48, 0x000D4844, 0x000D4844, 0x000D4844, 0x000D4844, 0x000D4844,
        0x000D4844, 0x000D4844, 0x000D4844, 0x000C4945, 0x000C4945, 0x000B4844, 0x000B4844, 0x000B4844, 0x000B4844, 0x000C4945, 0x000C4945, 0x000C4847, 0x000C4847, 0x000C4847, 0x000D4948, 0x000E4A49, 0x000F4B4A, 0x00104C4B, 0x00114D4C, 0x00175150,
        0x00134F4E, 0x00124E4D, 0x000E4D4B, 0x00105250, 0x00115653, 0x001A5F5C, 0x00327976, 0x00353C35, 0x00111811, 0x00171C13, 0x0023261D, 0x0025261D, 0x00242319, 0x00312F25, 0x0028231A, 0x0024251B, 0x003E3F36, 0x004E4D49, 0x00C0BEBD,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00070F0E, 0x00070E09, 0x000D110C, 0x000B0908, 0x0014100F, 0x00040606, 0x00081414, 0x00001311, 0x00000F0C, 0x0001120F, 0x0000110E, 0x00000F0C, 0x00000D0A, 0x00000C09,
        0x00000B08, 0x00000C09, 0x00000C09, 0x00001009, 0x00001009, 0x00001009, 0x00001009, 0x00000F08, 0x00000E07, 0x00000E07, 0x00000D06, 0x00000B07, 0x00000B07, 0x00000C08, 0x00000D09, 0x00000D09, 0x00000D09, 0x00000D09, 0x00000C08, 0x00000A05,
        0x00000B06, 0x00000C07, 0x00000D08, 0x00000D08, 0x00000D08, 0x00000D08, 0x00000D08, 0x00000D08, 0x00000B06, 0x00000904, 0x00000A05, 0x0000110C, 0x00011813, 0x00021914, 0x00001611, 0x00032213, 0x00052415, 0x00072617, 0x00092819, 0x00092819,
        0x00092819, 0x00092819, 0x000A291A, 0x000E1311, 0x0019221F, 0x001E2D29, 0x00152622, 0x000A1B17, 0x00101F1B, 0x00141D1A, 0x00010604, 0x00070B05, 0x00050903, 0x00050C05, 0x00040B04, 0x00010801, 0x00070B05, 0x00090B05, 0x00010300, 0x00000B03,
        0x00000700, 0x00000700, 0x00000B03, 0x00000D05, 0x00000B03, 0x00000901, 0x00000B03, 0x00000A06, 0x00000A06, 0x00000A06, 0x00000B07, 0x00000B07, 0x00000C08, 0x00000C08, 0x00000C08, 0x00000301, 0x00000604, 0x00000806, 0x00000806, 0x00000604,
        0x00000604, 0x00000806, 0x00000B09, 0x00000703, 0x00000905, 0x00000C08, 0x00000C08, 0x00000B07, 0x00000905, 0x00000804, 0x00000804, 0x00000705, 0x00000604, 0x00000503, 0x00000604, 0x00000705, 0x00000705, 0x00000604, 0x00000503, 0x00000A03,
        0x00000902, 0x00000902, 0x00000801, 0x00000801, 0x00000700, 0x00000700, 0x00000700, 0x00000804, 0x00000804, 0x00000905, 0x00000905, 0x00000905, 0x00000A06, 0x00000A06, 0x00000A06, 0x00000905, 0x00000905, 0x00000905, 0x00000905, 0x00000804,
        0x00000804, 0x00000804, 0x00000804, 0x00000805, 0x00000805, 0x00000805, 0x00000805, 0x00000805, 0x00000805, 0x00000805, 0x00000805, 0x00000805, 0x00000805, 0x00000906, 0x00000906, 0x00000906, 0x00000906, 0x00000906, 0x00000805, 0x00000807,
        0x00000807, 0x00000807, 0x00000807, 0x00000807, 0x00000807, 0x00000807, 0x00000807, 0x00000906, 0x00000906, 0x00000906, 0x00000906, 0x00000906, 0x00000906, 0x00000906, 0x00000906, 0x00010908, 0x00010908, 0x00010908, 0x00010908, 0x00010908,
        0x00020A09, 0x00020A09, 0x00020A09, 0x00010A07, 0x00010A07, 0x00010A07, 0x00020B08, 0x00020B08, 0x00020B08, 0x00030C09, 0x00030C09, 0x00000B06, 0x00000B06, 0x00000B06, 0x00000B06, 0x00000B06, 0x00000B06, 0x00000B06, 0x00000B06, 0x00000704,
        0x00000805, 0x00000906, 0x00000A07, 0x00000B07, 0x00000A06, 0x00000905, 0x00000804, 0x00000906, 0x00000906, 0x00000906, 0x00000906, 0x00000906, 0x00000906, 0x00000906, 0x00000906, 0x00000906, 0x00000906, 0x00000906, 0x00000906, 0x00000805,
        0x00000805, 0x00000805, 0x00000805, 0x00000B09, 0x00000A08, 0x00000A08, 0x00000907, 0x00000907, 0x00000A08, 0x00000B09, 0x00000B09, 0x00000F0A, 0x00000E09, 0x00011B15, 0x0007231D, 0x00001F18, 0x00001D16, 0x0000241C, 0x0001261E, 0x0003251E,
        0x0002241D, 0x0001231C, 0x0001231C, 0x0001231C, 0x0002241D, 0x0003251E, 0x0003251E, 0x0000261E, 0x0000251E, 0x0000261F, 0x0000221C, 0x00001913, 0x00000E09, 0x00000C08, 0x0000120E, 0x00000C09, 0x00000C09, 0x00000C09, 0x00000D0A, 0x00000D0A,
        0x00000D0A, 0x00000D0A, 0x00000D0A, 0x0000100B, 0x0000100B, 0x0000100B, 0x0000100B, 0x0000100B, 0x0000100B, 0x0000100B, 0x0000100B, 0x00000F0A, 0x00000F0A, 0x00000F0A, 0x0000100B, 0x0000110C, 0x0000130E, 0x0000140F, 0x00001510, 0x00021813,
        0x00061C17, 0x00061D18, 0x00031D17, 0x00041E18, 0x00001711, 0x0000130D, 0x00001C15, 0x00141812, 0x000E120C, 0x00171913, 0x001C1D14, 0x001B1C13, 0x0015130B, 0x00212016, 0x00141208, 0x000F1308, 0x0010130A, 0x00020300, 0x00838482,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFBFDFD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00070F0E, 0x00050F09, 0x000D130E, 0x0012130F, 0x001E1A19, 0x00151312, 0x00131614, 0x00111C19, 0x000B1A16, 0x00000300, 0x00000400, 0x00000400, 0x00020600, 0x00030701,
        0x00040802, 0x00050903, 0x00050903, 0x00060B02, 0x00060B02, 0x00060B02, 0x00070C03, 0x00080D04, 0x00090E05, 0x000A0F06, 0x000A0F06, 0x00100D08, 0x000F0C07, 0x000F0C07, 0x000F0C07, 0x000F0C07, 0x00100D08, 0x00100D08, 0x00110E09, 0x00110F07,
        0x00110F07, 0x00131109, 0x0014120A, 0x0016140C, 0x0016140C, 0x0016140C, 0x0016140C, 0x000E190F, 0x000B160C, 0x000B160C, 0x00162117, 0x002A352B, 0x003A453B, 0x003B463C, 0x00354036, 0x0037453F, 0x00384640, 0x00394741, 0x00394741, 0x00394741,
        0x003B4943, 0x003D4B45, 0x00404E48, 0x00485B52, 0x00364F45, 0x002C4C41, 0x002A5044, 0x002A5044, 0x0026463B, 0x00112A20, 0x00000F06, 0x000D140D, 0x000F1710, 0x000D1810, 0x000D1A12, 0x000F1A12, 0x000C130C, 0x0012140E, 0x0023201B, 0x00231E15,
        0x00211C13, 0x00211F15, 0x0027251B, 0x0027261C, 0x00212016, 0x001D1C12, 0x001C1B11, 0x001A1713, 0x001A1713, 0x001A1713, 0x001A1713, 0x001B1814, 0x001B1814, 0x001B1814, 0x001B1814, 0x00242B26, 0x00252C27, 0x00242B26, 0x00202722, 0x001B221D,
        0x0019201B, 0x001A211C, 0x001D241F, 0x0026231F, 0x0026231F, 0x0026231F, 0x0024211D, 0x00211E1A, 0x001F1C18, 0x001E1B17, 0x001E1B17, 0x00171E19, 0x00161D18, 0x00151C17, 0x00151C17, 0x00151C17, 0x00161D18, 0x00141B16, 0x00131A15, 0x001C1F16,
        0x001C1F16, 0x001C1F16, 0x001B1E15, 0x001B1E15, 0x001A1D14, 0x001A1D14, 0x001A1D14, 0x00171913, 0x00181A14, 0x00191B15, 0x001A1C16, 0x001B1D17, 0x001C1E18, 0x001C1E18, 0x001D1F19, 0x00181A14, 0x00181A14, 0x00181A14, 0x00171913, 0x00171913,
        0x00171913, 0x00171913, 0x00171913, 0x00191A16, 0x00191A16, 0x001A1B17, 0x001A1B17, 0x001B1C18, 0x001B1C18, 0x001C1D19, 0x001C1D19, 0x00151D16, 0x00151D16, 0x00141C15, 0x00131B14, 0x00131B14, 0x00141C15, 0x00141C15, 0x00151D16, 0x00141B16,
        0x00141B16, 0x00141B16, 0x00141B16, 0x00141B16, 0x00141B16, 0x00141B16, 0x00141B16, 0x00141C15, 0x00141C15, 0x00141C15, 0x00141C15, 0x00141C15, 0x00141C15, 0x00141C15, 0x00141C15, 0x00151C17, 0x00151C17, 0x00151C17, 0x00161D18, 0x00161D18,
        0x00161D18, 0x00161D18, 0x00171E19, 0x00151D16, 0x00161E17, 0x00161E17, 0x00161E17, 0x00171F18, 0x00171F18, 0x00182019, 0x00182019, 0x00211E19, 0x00211E19, 0x00211E19, 0x00211E19, 0x00211E19, 0x00211E19, 0x00211E19, 0x00211E19, 0x00201B18,
        0x00201B18, 0x00201B18, 0x001F1C18, 0x001F1C18, 0x001D1C18, 0x001D1C18, 0x001D1C18, 0x0019211A, 0x0019211A, 0x0019211A, 0x0019211A, 0x0019211A, 0x0019211A, 0x0019211A, 0x0019211A, 0x00182019, 0x0019211A, 0x0019211A, 0x0019211A, 0x0019211A,
        0x0019211A, 0x0019211A, 0x0019211A, 0x0019231D, 0x0019231D, 0x0018221C, 0x0017211B, 0x0017211B, 0x0017211B, 0x0018221C, 0x0019231D, 0x001A211A, 0x00131B14, 0x001C271F, 0x002C3931, 0x00293A31, 0x0023362D, 0x00243930, 0x00243C32, 0x0024372E,
        0x0025382F, 0x00263930, 0x00273A31, 0x00293C33, 0x002C3F36, 0x002F4239, 0x0031443B, 0x002C4139, 0x002B3D36, 0x002C3C35, 0x002E3A34, 0x002A312C, 0x00212520, 0x001B1C18, 0x001C1B17, 0x00241F1C, 0x00241F1C, 0x00231E1B, 0x00211C19, 0x00201B18,
        0x001E1916, 0x001D1815, 0x001C1714, 0x001B1911, 0x001B1911, 0x001C1A12, 0x001C1A12, 0x001C1A12, 0x001C1A12, 0x001B1911, 0x001B1911, 0x001B1911, 0x001A1810, 0x001B1911, 0x001B1911, 0x001C1A12, 0x001D1B13, 0x001E1C14, 0x001F1D15, 0x00201E16,
        0x0026241C, 0x0027251D, 0x0028261E, 0x0024251C, 0x001C1D14, 0x0014150C, 0x0012130A, 0x0011120E, 0x00181915, 0x001D1D17, 0x001D1D17, 0x0021211B, 0x001A1B12, 0x0023241B, 0x0014120A, 0x000B0F04, 0x00181B12, 0x00060703, 0x0080817F,
  0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFCFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x0008100F, 0x00040E08, 0x000E140F, 0x00181915, 0x001A1915, 0x001D1C18, 0x00262723, 0x002D312B, 0x00222922, 0x00202B21, 0x00212C22, 0x00232E24, 0x00242F25, 0x00253026,
        0x00253026, 0x00263127, 0x00263127, 0x0039342B, 0x0039342B, 0x0038332A, 0x0038332A, 0x0038332A, 0x0039342B, 0x0039342B, 0x003A352C, 0x00373731, 0x00363630, 0x00363630, 0x0035352F, 0x0035352F, 0x00363630, 0x00383832, 0x00393933, 0x0037382F,
        0x0037382F, 0x00383930, 0x003A3B32, 0x003C3D34, 0x003D3E35, 0x003C3D34, 0x003B3C33, 0x003E4138, 0x0035382F, 0x00363930, 0x0044473E, 0x004E5148, 0x004D5047, 0x004A4D44, 0x004B4E45, 0x00444948, 0x00444948, 0x00434847, 0x00424746, 0x00414645,
        0x00424746, 0x00444948, 0x00474C4B, 0x003D463C, 0x003A483D, 0x00394D41, 0x003D574A, 0x00415B4E, 0x00293D31, 0x000B190E, 0x00111A10, 0x002A2E28, 0x00353933, 0x00353C35, 0x00353D36, 0x003D453E, 0x003D443D, 0x003F413B, 0x00494943, 0x0047453B,
        0x0047463C, 0x004A493F, 0x004E4D43, 0x004B4C42, 0x0047483E, 0x003F4338, 0x003D4136, 0x003A3A34, 0x003A3A34, 0x003A3A34, 0x003A3A34, 0x003A3A34, 0x003A3A34, 0x003A3A34, 0x003A3A34, 0x00424640, 0x0041453F, 0x003F433D, 0x003B3F39, 0x00373B35,
        0x00363A34, 0x00383C36, 0x003A3E38, 0x00383832, 0x00383832, 0x00383832, 0x00363630, 0x0034342E, 0x0033332D, 0x0034342E, 0x0035352F, 0x0031352F, 0x002F332D, 0x002E322C, 0x002E322C, 0x002F332D, 0x002F332D, 0x002E322C, 0x002C302A, 0x00262F25,
        0x00262F25, 0x00262F25, 0x00252E24, 0x00252E24, 0x00242D23, 0x00242D23, 0x00242D23, 0x00222B21, 0x00222B21, 0x00232C22, 0x00242D23, 0x00252E24, 0x00262F25, 0x00262F25, 0x00273026, 0x00232C22, 0x00232C22, 0x00232C22, 0x00232C22, 0x00222B21,
        0x00222B21, 0x00222B21, 0x00222B21, 0x001D251E, 0x001D251E, 0x001E261F, 0x001F2720, 0x00202821, 0x00202821, 0x00212922, 0x00212922, 0x00332922, 0x00322821, 0x00312720, 0x0030261F, 0x002F251E, 0x0030261F, 0x00312720, 0x00322821, 0x002F2420,
        0x002F2420, 0x002F2420, 0x002F2420, 0x002F2420, 0x002F2420, 0x002F2420, 0x002F2420, 0x002F251E, 0x002F251E, 0x002F251E, 0x002F251E, 0x002F251E, 0x002F251E, 0x002F251E, 0x002F251E, 0x002F2420, 0x002F2420, 0x002F2420, 0x002F2420, 0x00302521,
        0x00302521, 0x00302521, 0x00302521, 0x002F251E, 0x002F251E, 0x0030261F, 0x0030261F, 0x0030261F, 0x00312720, 0x00312720, 0x00312720, 0x0027281F, 0x0027281F, 0x0027281F, 0x0027281F, 0x0027281F, 0x0027281F, 0x0027281F, 0x0027281F, 0x00292621,
        0x00292621, 0x0025251F, 0x0025251F, 0x0023251F, 0x0023251F, 0x00252721, 0x00232721, 0x002E241D, 0x002E241D, 0x002E241D, 0x002E241D, 0x002E241D, 0x002E241D, 0x002E241D, 0x002E241D, 0x002F251E, 0x002F251E, 0x0030261F, 0x0030261F, 0x0030261F,
        0x0030261F, 0x00312720, 0x00312720, 0x00302723, 0x00302723, 0x002F2622, 0x002F2622, 0x002F2622, 0x002F2622, 0x002F2622, 0x002F2622, 0x0022251C, 0x00191C13, 0x001F2219, 0x0030352C, 0x00384036, 0x00353D33, 0x0030392F, 0x002E372D, 0x0031392F,
        0x00323A30, 0x00333B31, 0x00333B31, 0x00343C32, 0x00363E34, 0x00394137, 0x003B4339, 0x00484A44, 0x0042443E, 0x0041413B, 0x0047443F, 0x004C4843, 0x004A443F, 0x00413834, 0x00382F2B, 0x0031312B, 0x0031312B, 0x0030302A, 0x0030302A, 0x002F2F29,
        0x002E2E28, 0x002D2D27, 0x002D2D27, 0x002E2E28, 0x002E2E28, 0x002E2E28, 0x002F2F29, 0x002F2F29, 0x002E2E28, 0x002E2E28, 0x002E2E28, 0x002F3027, 0x002F3027, 0x002F3027, 0x002F3027, 0x002F3027, 0x00303128, 0x00313229, 0x0032332A, 0x00303128,
        0x0034352C, 0x00313229, 0x0033342B, 0x0034352C, 0x00313229, 0x0034352C, 0x0032332A, 0x0031302C, 0x0033322E, 0x002C2B27, 0x0025251F, 0x002D2D27, 0x002A2A24, 0x002E2F26, 0x00191C13, 0x00060C01, 0x00151A11, 0x00020601, 0x00717472,
  0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFCFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x000A1211, 0x000A110C, 0x000F1510, 0x001C201B, 0x00161812, 0x00181A14, 0x001F2219, 0x0023281F, 0x001B2017, 0x0018231B, 0x0018231B, 0x0017221A, 0x0017221A, 0x00162119,
        0x00162119, 0x00162119, 0x00162119, 0x00202118, 0x00212219, 0x00212219, 0x00202118, 0x00202118, 0x001F2017, 0x001D1E15, 0x001D1E15, 0x00151C17, 0x00151C17, 0x00151C17, 0x00151C17, 0x00151C17, 0x00161D18, 0x00161D18, 0x00161D18, 0x00141C15,
        0x00141C15, 0x00151D16, 0x00161E17, 0x00182019, 0x00182019, 0x00151D16, 0x00131B14, 0x00181A14, 0x00151711, 0x001A1C16, 0x00292B25, 0x0031332D, 0x002F312B, 0x002D2F29, 0x0031332D, 0x0030312D, 0x0031322E, 0x0030312D, 0x002F302C, 0x002E2F2B,
        0x002E2F2B, 0x0030312D, 0x0032332F, 0x003A2F2B, 0x003A312D, 0x0037332E, 0x0034312C, 0x00393631, 0x002D2924, 0x00170E0A, 0x001A0F0B, 0x00110C09, 0x00191411, 0x001B1613, 0x0016110E, 0x0017120F, 0x001A1713, 0x00161511, 0x000F100C, 0x0010150C,
        0x0013180F, 0x00151A11, 0x00141910, 0x0011190F, 0x0011190F, 0x000E170D, 0x000B140A, 0x000E160F, 0x000D150E, 0x000D150E, 0x000C140D, 0x000C140D, 0x000B130C, 0x000B130C, 0x000B130C, 0x00120E09, 0x00120E09, 0x00110D08, 0x00100C07, 0x000F0B06,
        0x00100C07, 0x00130F0A, 0x0014100B, 0x000A120B, 0x000A120B, 0x000B130C, 0x000B130C, 0x000A120B, 0x000B130C, 0x000D150E, 0x000E160F, 0x0016110E, 0x0015100D, 0x00130E0B, 0x00130E0B, 0x00140F0C, 0x00140F0C, 0x00120D0A, 0x00110C09, 0x0009120F,
        0x0009120F, 0x0008110E, 0x0008110E, 0x0007100D, 0x0007100D, 0x0007100D, 0x00060F0C, 0x00081009, 0x00081009, 0x00081009, 0x0009110A, 0x0009110A, 0x0009110A, 0x0009110A, 0x0009110A, 0x00081009, 0x00081009, 0x00081009, 0x00081009, 0x00070F08,
        0x00070F08, 0x00070F08, 0x00070F08, 0x00080E09, 0x00080E09, 0x00080E09, 0x00090F0A, 0x00090F0A, 0x00090F0A, 0x000A100B, 0x000A100B, 0x0009160E, 0x0008150D, 0x0008150D, 0x0007140C, 0x0007140C, 0x0007140C, 0x0007140C, 0x0007140C, 0x0005110B,
        0x0005110B, 0x0005110B, 0x0005110B, 0x0005110B, 0x0005110B, 0x0005110B, 0x0005110B, 0x0005120A, 0x0005120A, 0x0005120A, 0x0005120A, 0x0005120A, 0x0005120A, 0x0005120A, 0x0005120A, 0x0004100A, 0x0004100A, 0x0005110B, 0x0005110B, 0x0005110B,
        0x0005110B, 0x0006120C, 0x0006120C, 0x0005120A, 0x0005120A, 0x0005120A, 0x0005120A, 0x0005120A, 0x0006130B, 0x0006130B, 0x0006130B, 0x00081009, 0x00081009, 0x00081009, 0x00081009, 0x00081009, 0x00081009, 0x00081009, 0x00081009, 0x000B0F0A,
        0x00080E09, 0x00080E09, 0x00070D08, 0x00060D08, 0x00070E09, 0x00050F09, 0x0006100A, 0x0005120A, 0x0005120A, 0x0005120A, 0x0005120A, 0x0005120A, 0x0005120A, 0x0005120A, 0x0005120A, 0x00020F07, 0x00020F07, 0x00020F07, 0x00031008, 0x00031008,
        0x00041109, 0x00041109, 0x00041109, 0x00021109, 0x0003120A, 0x0003120A, 0x0003120A, 0x00021109, 0x00011008, 0x00011008, 0x00000F07, 0x000D110B, 0x000D110B, 0x0012140E, 0x001B1D17, 0x00272721, 0x00292621, 0x0023201B, 0x00211E19, 0x0025231B,
        0x0026241C, 0x0027251D, 0x0026241C, 0x0024221A, 0x0024221A, 0x0025231B, 0x0026241C, 0x0026231E, 0x0024211C, 0x00211E19, 0x0023201B, 0x0027241F, 0x0026231E, 0x001B1B15, 0x0011110B, 0x00080F08, 0x00080F08, 0x00080F08, 0x00091009, 0x00091009,
        0x00091009, 0x000A110A, 0x000A110A, 0x000B120D, 0x000C130E, 0x000C130E, 0x000D140F, 0x000D140F, 0x000C130E, 0x000C130E, 0x000B120D, 0x000D140D, 0x000D140D, 0x000C130C, 0x000C130C, 0x000C130C, 0x000C130C, 0x000D140D, 0x000D140D, 0x000A120B,
        0x000C130C, 0x00070E07, 0x000C130C, 0x000F130D, 0x000E120C, 0x00171913, 0x00171913, 0x001B1716, 0x001C1817, 0x00201D19, 0x00191814, 0x0021211B, 0x001C1E18, 0x001F211B, 0x0010130A, 0x0012180D, 0x00141910, 0x00000400, 0x00686B69,
  0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFCFEFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x000B1312, 0x000B120D, 0x000C120D, 0x0020241F, 0x00161A14, 0x0010150C, 0x00070C03, 0x00090F04, 0x000A1005, 0x0013120E, 0x0012110D, 0x0011100C, 0x00100F0B, 0x0011100C,
        0x0012110D, 0x0014130F, 0x00151410, 0x0008120C, 0x000A140E, 0x000D1711, 0x000F1913, 0x00111B15, 0x00111B15, 0x00101A14, 0x000F1913, 0x00141616, 0x00151717, 0x00161818, 0x00171919, 0x00181A1A, 0x00171919, 0x00161818, 0x00161818, 0x00191C1A,
        0x00181B19, 0x00181B19, 0x00191C1A, 0x001A1D1B, 0x00191C1A, 0x00161917, 0x00131614, 0x00151816, 0x00141715, 0x00131614, 0x00121513, 0x00151816, 0x00181B19, 0x00151816, 0x00111412, 0x00161910, 0x00171A11, 0x00191C13, 0x00191C13, 0x00191C13,
        0x00191C13, 0x001A1D14, 0x001C1F16, 0x001B1D17, 0x00141610, 0x0020221C, 0x001D1F19, 0x00141610, 0x00161812, 0x000F110B, 0x000B0D07, 0x00060D0A, 0x000E1311, 0x001B1E1C, 0x001D201E, 0x00171C1A, 0x00192421, 0x00182925, 0x000D211C, 0x0023221E,
        0x00272622, 0x00262723, 0x0021221E, 0x001E221D, 0x00212520, 0x00232722, 0x001F2520, 0x00242822, 0x00242822, 0x00232721, 0x00222620, 0x0021251F, 0x0020241E, 0x0020241E, 0x001F231D, 0x001C271F, 0x001B261E, 0x001A251D, 0x001A251D, 0x001B261E,
        0x001B261E, 0x001C271F, 0x001C271F, 0x0020241E, 0x0021251F, 0x0021251F, 0x0020241E, 0x001E221C, 0x001D211B, 0x001D211B, 0x001E221C, 0x0017211B, 0x0016201A, 0x00141E18, 0x00141E18, 0x00151F19, 0x00141E18, 0x00131D17, 0x00121C16, 0x001C1718,
        0x001C1718, 0x001C1718, 0x001B1617, 0x001B1617, 0x001A1516, 0x001A1516, 0x001A1516, 0x0018140F, 0x0017130E, 0x0017130E, 0x0016120D, 0x0016120D, 0x0015110C, 0x0015110C, 0x0014100B, 0x0014100B, 0x0014100B, 0x0014100B, 0x0014100B, 0x00130F0A,
        0x00130F0A, 0x00130F0A, 0x00130F0A, 0x00140D0A, 0x00140D0A, 0x00140D0A, 0x00140D0A, 0x00140D0A, 0x00140D0A, 0x00130C09, 0x00130C09, 0x000D0D07, 0x000D0D07, 0x000E0E08, 0x000E0E08, 0x000E0E08, 0x000D0D07, 0x000B0B05, 0x000A0A04, 0x000D0C08,
        0x000D0C08, 0x000D0C08, 0x000D0C08, 0x000D0C08, 0x000D0C08, 0x000D0C08, 0x000D0C08, 0x000D0D07, 0x000D0D07, 0x000D0D07, 0x000D0D07, 0x000D0D07, 0x000D0D07, 0x000D0D07, 0x000D0D07, 0x000B0A06, 0x000B0A06, 0x000C0B07, 0x000C0B07, 0x000C0B07,
        0x000C0B07, 0x000C0B07, 0x000D0C08, 0x000C0C06, 0x000C0C06, 0x000C0C06, 0x000C0C06, 0x000C0C06, 0x000C0C06, 0x000C0C06, 0x000C0C06, 0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000A0E08, 0x000B0C08,
        0x000B0C08, 0x000A0E09, 0x000A0E09, 0x000A0E09, 0x00080E09, 0x00070D08, 0x00070D08, 0x000E0E08, 0x000E0E08, 0x000E0E08, 0x000E0E08, 0x000E0E08, 0x000E0E08, 0x000E0E08, 0x000E0E08, 0x000D0D07, 0x000E0E08, 0x000E0E08, 0x000E0E08, 0x000F0F09,
        0x000F0F09, 0x0010100A, 0x0010100A, 0x000E100A, 0x000E100A, 0x000F110B, 0x000F110B, 0x000E100A, 0x000D0F09, 0x000C0E08, 0x000B0D07, 0x00000600, 0x000E150E, 0x00131711, 0x0011130D, 0x00191611, 0x001B1712, 0x001B1510, 0x001F1612, 0x0018150D,
        0x0019160E, 0x001B1810, 0x001A170F, 0x0019160E, 0x0019160E, 0x001B1810, 0x001C1911, 0x00151711, 0x00191B15, 0x00191D17, 0x00141B14, 0x00121912, 0x00111912, 0x000C170F, 0x0008130B, 0x0013150F, 0x0012140E, 0x0012140E, 0x0011130D, 0x0010120C,
        0x000F110B, 0x000F110B, 0x000E100A, 0x000C0E0E, 0x000C0E0E, 0x000D0F0F, 0x000E1010, 0x000E1010, 0x000D0F0F, 0x000C0E0E, 0x000C0E0E, 0x000D0E0C, 0x000C0D0B, 0x000B0C0A, 0x000B0C0A, 0x000A0B09, 0x000A0B09, 0x000B0C0A, 0x000B0C0A, 0x00060907,
        0x00080B09, 0x00040705, 0x000D0E0C, 0x000E0C0B, 0x00070504, 0x000D0908, 0x000A0605, 0x000C0706, 0x000C0807, 0x0013100C, 0x00080703, 0x000A0A04, 0x000A0C06, 0x00151913, 0x00161B12, 0x00171D12, 0x0013180F, 0x00070B06, 0x00717472,
  0xFFFFFFFF, 0xFFFEFEFE, 0xFFFFFFFF, 0xFFFEF9FA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x00000503, 0x000F1A17, 0x0016211E, 0x00202926, 0x001B2421, 0x001C2522, 0x00272E2B, 0x0029302D, 0x002F3633, 0x002E362C, 0x002E362C, 0x002F372D, 0x0030382E, 0x00323A30,
        0x00363E34, 0x00394137, 0x003B4339, 0x0044423A, 0x0044423A, 0x0044423A, 0x0043443B, 0x0045463D, 0x00484940, 0x00494C43, 0x004B4E45, 0x004C4F40, 0x004E5142, 0x004F5243, 0x004E5142, 0x004D5041, 0x004C4F40, 0x004C4F40, 0x004C4F40, 0x004B4D41,
        0x00494B3F, 0x0047493D, 0x0045473B, 0x00434539, 0x003F4135, 0x003A3C30, 0x0036382C, 0x002B322B, 0x002A312A, 0x001C231C, 0x000F160F, 0x00121912, 0x001B221B, 0x001D241D, 0x001C231C, 0x001A211A, 0x001C231C, 0x001E251E, 0x001F261F, 0x001E251E,
        0x001E251E, 0x001E251E, 0x001E251E, 0x001C201A, 0x0020241E, 0x0021251F, 0x001C231C, 0x00171E17, 0x00121912, 0x000C140D, 0x00070F08, 0x0017180F, 0x002E2F26, 0x003C3D34, 0x003E3F36, 0x00414239, 0x0043443B, 0x0042433A, 0x0043443B, 0x0044463A,
        0x00434539, 0x00424438, 0x00424438, 0x00424438, 0x00434539, 0x0044463A, 0x0044463A, 0x004B4B3F, 0x0049493D, 0x00454539, 0x00414135, 0x003F3F33, 0x003D3D31, 0x003C3C30, 0x003C3C30, 0x003F3F33, 0x003C3C30, 0x003D3D31, 0x00414135, 0x00424236,
        0x003E3E32, 0x003C3C30, 0x003D3D31, 0x003F3E34, 0x003D3C32, 0x003B3A30, 0x0039382E, 0x0039382E, 0x0037362C, 0x0035342A, 0x00333228, 0x00303022, 0x00313123, 0x00313123, 0x00313123, 0x00313123, 0x00313123, 0x00313123, 0x00313123, 0x002C2D23,
        0x002B2C22, 0x00292A20, 0x00292A20, 0x00292A20, 0x0028291F, 0x0027281E, 0x0025261C, 0x0026251B, 0x00242319, 0x00222117, 0x00212016, 0x00212016, 0x00201F15, 0x001E1D13, 0x001C1B11, 0x001A1A0C, 0x001B1B0D, 0x001D1D0F, 0x001D1D0F, 0x001C1C0E,
        0x001C1C0E, 0x001D1D0F, 0x001E1E10, 0x001C1B11, 0x001B1A10, 0x001A190F, 0x0019180E, 0x0019180E, 0x0019180E, 0x001A190F, 0x001B1A10, 0x0018190F, 0x00191A10, 0x00191A10, 0x001A1B11, 0x001B1C12, 0x001B1C12, 0x001A1B11, 0x001A1B11, 0x001A1B12,
        0x001A1B12, 0x001A1B12, 0x001A1B12, 0x00191A11, 0x00191A11, 0x00191A11, 0x00191A11, 0x00191A10, 0x00191A10, 0x00191A10, 0x00191A10, 0x0018190F, 0x0018190F, 0x0018190F, 0x0018190F, 0x00191A10, 0x00191A10, 0x00191A10, 0x00191A10, 0x0018190F,
        0x0018190F, 0x0018190F, 0x0018190F, 0x0018190F, 0x00191A10, 0x00191A10, 0x00191A10, 0x00191A10, 0x0018190F, 0x0017180E, 0x0017180E, 0x0017150D, 0x0017150D, 0x0017150D, 0x0017150D, 0x0017150D, 0x0017150D, 0x0017150D, 0x0017150D, 0x0016170D,
        0x0018190F, 0x00191A10, 0x00191A10, 0x0018190F, 0x0017180E, 0x0017180E, 0x0018190F, 0x0018190F, 0x0018190F, 0x00191A10, 0x00191A10, 0x00191A10, 0x00191A10, 0x00191A10, 0x001A1B11, 0x001C1D13, 0x001B1C12, 0x001A1B11, 0x001B1C12, 0x001D1E14,
        0x001E1F15, 0x001E1F15, 0x001C1D13, 0x001A1B11, 0x001B1C12, 0x001C1D13, 0x001D1E14, 0x001D1E14, 0x001C1D13, 0x001B1C12, 0x001A1B11, 0x0012170E, 0x00161B12, 0x001A1F16, 0x001B2017, 0x00191E15, 0x00191E15, 0x001B2017, 0x001D2219, 0x00131D17,
        0x0017211B, 0x0018221C, 0x00151F19, 0x00141E18, 0x0016201A, 0x00141E18, 0x000F1913, 0x001B1911, 0x0016140C, 0x0017150D, 0x001F1D15, 0x00201E16, 0x001C1A12, 0x001D1B13, 0x00232119, 0x00252519, 0x00252519, 0x0026261A, 0x0027271B, 0x0028281C,
        0x0028281C, 0x0028281C, 0x0028281C, 0x001D2A1C, 0x001D2A1C, 0x001E2B1D, 0x001E2B1D, 0x001E2B1D, 0x001D2A1C, 0x001C291B, 0x001B281A, 0x00232C22, 0x00252E24, 0x00252E24, 0x00232C22, 0x001F281E, 0x001C251B, 0x001D261C, 0x001F281E, 0x0023271C,
        0x0021251A, 0x001E2217, 0x001C2015, 0x001C2015, 0x001D2116, 0x001C2015, 0x001C2015, 0x001D1F19, 0x001A1C16, 0x00181A14, 0x00191B15, 0x001C1E18, 0x0020221C, 0x0023251F, 0x00242620, 0x00191F1A, 0x00161C17, 0x00060C07, 0xFFBBC1BC,
  0xFFFFFFFF, 0xFFFDFDFD, 0xFFFFFFFF, 0xFFFFFEFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x000B1614, 0x000F100E, 0x001B1C1A, 0x002E2C2B, 0x002B2928, 0x002B2726, 0x0034302F, 0x00363130, 0x00373231, 0x00322E29, 0x00322E29, 0x00302C27, 0x002F2B26, 0x002F2B26,
        0x002F2B26, 0x00302C27, 0x00312D28, 0x002E2E28, 0x002F2F29, 0x0030302A, 0x002F312B, 0x002E302A, 0x002B2F29, 0x002A2E28, 0x00292D27, 0x002C2F26, 0x002D3027, 0x002E3128, 0x002D3027, 0x002A2D24, 0x00282B22, 0x00282B22, 0x00282B22, 0x0022251C,
        0x0022251C, 0x0021241B, 0x0021241B, 0x0020231A, 0x001F2219, 0x001C1F16, 0x001A1D14, 0x00141A15, 0x001A201B, 0x00191F1A, 0x00171D18, 0x00202621, 0x00262C27, 0x00262C27, 0x00252B26, 0x00212722, 0x00222823, 0x00242A25, 0x00232924, 0x00222823,
        0x00202621, 0x001F2520, 0x001F2520, 0x00262A25, 0x00252924, 0x00222621, 0x001D231E, 0x001C221D, 0x001A201B, 0x00111813, 0x00080F0A, 0x000A0C06, 0x0012140E, 0x0011130D, 0x0010120C, 0x00171913, 0x001C1E18, 0x001C1E18, 0x001C1E18, 0x00191C13,
        0x00181B12, 0x00181B12, 0x00171A11, 0x00171A11, 0x00161910, 0x00161910, 0x00171A11, 0x00161910, 0x00161910, 0x0015180F, 0x0014170E, 0x0013160D, 0x0013160D, 0x0014170E, 0x0015180F, 0x00191C13, 0x00171A11, 0x00171A11, 0x00191C13, 0x001A1D14,
        0x00181B12, 0x00161910, 0x0015180F, 0x0013150F, 0x0012140E, 0x0012140E, 0x0012140E, 0x00141610, 0x00141610, 0x00141610, 0x0013150F, 0x00141910, 0x00141910, 0x00141910, 0x00151A11, 0x00161B12, 0x00161B12, 0x00171C13, 0x00171C13, 0x00141B14,
        0x00131A13, 0x00121912, 0x00121912, 0x00141B14, 0x00141B14, 0x00131A13, 0x00121912, 0x00151913, 0x00141812, 0x00131711, 0x00131711, 0x00141812, 0x00141812, 0x00131711, 0x00121610, 0x0013180F, 0x0012170E, 0x0011160D, 0x000F140B, 0x000D1209,
        0x000F140B, 0x0012170E, 0x00151A11, 0x00121610, 0x00121610, 0x0011150F, 0x0011150F, 0x0011150F, 0x0011150F, 0x0011150F, 0x0011150F, 0x00101710, 0x00111811, 0x00111811, 0x00121912, 0x00121912, 0x00121912, 0x00121912, 0x00111811, 0x00111712,
        0x00111712, 0x00111712, 0x00111712, 0x00101611, 0x00101611, 0x00101611, 0x00101611, 0x000F160F, 0x000F160F, 0x000F160F, 0x000F160F, 0x000E150E, 0x000E150E, 0x000E150E, 0x000E150E, 0x000C130C, 0x000C130C, 0x000C130C, 0x000D140D, 0x000D140D,
        0x000E150E, 0x000E150E, 0x000E150E, 0x00101710, 0x00111811, 0x00111811, 0x00111811, 0x00111811, 0x00101710, 0x00101710, 0x000F160F, 0x0010140F, 0x0010140F, 0x0010140F, 0x0010140F, 0x0010140F, 0x0010140F, 0x0010140F, 0x0010140F, 0x000C130C,
        0x000D140D, 0x000E150E, 0x000E150E, 0x000E150E, 0x000D140D, 0x000D140D, 0x000D140D, 0x00101710, 0x00101710, 0x00101710, 0x00101710, 0x00101710, 0x00111811, 0x00111811, 0x00111811, 0x00131A13, 0x00121912, 0x00121912, 0x00121912, 0x00131A13,
        0x00131A13, 0x00131A13, 0x00131A13, 0x00151C15, 0x00161D16, 0x00171E17, 0x00171E17, 0x00171E17, 0x00171E17, 0x00161D16, 0x00151C15, 0x00101611, 0x00181E19, 0x00222823, 0x00282E29, 0x002A302B, 0x00282E29, 0x00262C27, 0x00242A25, 0x002E2A29,
        0x002F2B2A, 0x00302C2B, 0x002F2B2A, 0x002E2A29, 0x002F2B2A, 0x002E2A29, 0x002B2726, 0x001D211C, 0x001E221D, 0x00242823, 0x002C302B, 0x00292D28, 0x001E221D, 0x00141813, 0x00111510, 0x00191C13, 0x00191C13, 0x001A1D14, 0x001B1E15, 0x001B1E15,
        0x001B1E15, 0x001B1E15, 0x001B1E15, 0x00211F17, 0x00211F17, 0x00222018, 0x00222018, 0x00211F17, 0x00201E16, 0x001E1C14, 0x001D1B13, 0x001E1B16, 0x00201D18, 0x00211E19, 0x00201D18, 0x001E1B16, 0x001D1A15, 0x001F1C17, 0x00201D18, 0x001C2118,
        0x001A1F16, 0x00191E15, 0x00191E15, 0x001A1F16, 0x001B2017, 0x001B2017, 0x001A1F16, 0x001B1F19, 0x001B1F19, 0x001C201A, 0x001D211B, 0x001E221C, 0x001F231D, 0x001F231D, 0x001E221C, 0x00161C17, 0x00000200, 0x00666C67, 0xFFF8FEF9,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFDFE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x0086918F, 0x00181B19, 0x00060907, 0x00040705, 0x00030402, 0x00000100, 0x00040503, 0x00060403, 0x00050302, 0x00030100, 0x00030100, 0x00030100, 0x00030100, 0x00040201,
        0x00040201, 0x00050302, 0x00050302, 0x00000200, 0x00000301, 0x00020503, 0x00020705, 0x00030806, 0x00030806, 0x00010805, 0x00000704, 0x00050A08, 0x00070C0A, 0x00080D0B, 0x00090E0C, 0x00090E0C, 0x00080D0B, 0x00090E0C, 0x000A0F0D, 0x000B100F,
        0x000C1110, 0x000E1312, 0x00101514, 0x00111615, 0x00131817, 0x00141918, 0x00161B1A, 0x00151A19, 0x001B201F, 0x001D2221, 0x001F2423, 0x00262B2A, 0x00262B2A, 0x00232827, 0x00242928, 0x00262B2A, 0x00282D2C, 0x002A2F2E, 0x00292E2D, 0x00282D2C,
        0x00262B2A, 0x00252A29, 0x00252A29, 0x00262828, 0x00252727, 0x00232525, 0x00202524, 0x00232827, 0x00252A29, 0x00212726, 0x001A201F, 0x001A1F1D, 0x001C211F, 0x001A1F1D, 0x001C211F, 0x00242927, 0x00242927, 0x001D2220, 0x00191E1C, 0x001F2423,
        0x001F2423, 0x001E2322, 0x001E2322, 0x001C2120, 0x001B201F, 0x001A1F1E, 0x00191E1D, 0x00192120, 0x001A2221, 0x00192120, 0x00161E1D, 0x00121A19, 0x000F1716, 0x000F1716, 0x000F1716, 0x00111918, 0x00111918, 0x00101817, 0x000E1615, 0x000F1716,
        0x00101817, 0x000E1615, 0x000B1312, 0x000D1515, 0x000C1414, 0x000C1414, 0x000C1414, 0x000E1616, 0x000F1717, 0x000E1616, 0x000D1515, 0x000C1316, 0x000B1215, 0x000B1215, 0x000B1215, 0x000B1215, 0x000B1215, 0x000B1215, 0x000B1215, 0x00081212,
        0x00071111, 0x00061010, 0x00071111, 0x00081212, 0x00081212, 0x00081212, 0x00061010, 0x000A1212, 0x00091111, 0x00091111, 0x00091111, 0x000B1313, 0x000B1313, 0x000B1313, 0x000A1212, 0x00091013, 0x00091013, 0x00091013, 0x00091013, 0x000A1114,
        0x000D1417, 0x0012191C, 0x00161D20, 0x00081010, 0x00081010, 0x00081010, 0x00081010, 0x00081010, 0x00081010, 0x00070F0F, 0x00070F0F, 0x00061010, 0x00061010, 0x00061010, 0x00071111, 0x00061010, 0x00061010, 0x00050F0F, 0x00050F0F, 0x00040D10,
        0x00040D10, 0x00040D10, 0x00040D10, 0x00040D10, 0x00030C0F, 0x00030C0F, 0x00030C0F, 0x00061010, 0x00061010, 0x00061010, 0x00050F0F, 0x00050F0F, 0x00050F0F, 0x00050F0F, 0x00040E0E, 0x00061010, 0x00061010, 0x00061010, 0x00061010, 0x00061010,
        0x00061010, 0x00061010, 0x00061010, 0x00081212, 0x00091313, 0x00091313, 0x00091313, 0x00091313, 0x00081212, 0x00071111, 0x00071111, 0x00080F12, 0x00080F12, 0x00080F12, 0x00080F12, 0x00080F12, 0x00080F12, 0x00080F12, 0x00080F12, 0x000A1414,
        0x00091313, 0x000A1414, 0x000A1414, 0x000B1515, 0x000B1515, 0x000A1414, 0x00091313, 0x00061010, 0x00061010, 0x00071111, 0x00071111, 0x00071111, 0x00071111, 0x00081212, 0x00081212, 0x00081212, 0x00081212, 0x00091313, 0x00081212, 0x00071111,
        0x00071111, 0x00071111, 0x00081212, 0x00061010, 0x00061010, 0x00061010, 0x00071111, 0x00071111, 0x00061010, 0x00061010, 0x00061010, 0x00060D10, 0x00060D10, 0x00040B0E, 0x0002090C, 0x00000609, 0x00000407, 0x00000306, 0x00000205, 0x00020103,
        0x00010002, 0x00010002, 0x00020103, 0x00020103, 0x00010002, 0x00010002, 0x00020103, 0x00000003, 0x00000104, 0x00000407, 0x0001080B, 0x00030A0D, 0x00030A0D, 0x0002090C, 0x00030A0D, 0x00010908, 0x00020A09, 0x00020A09, 0x00030B0A, 0x00030B0A,
        0x00030B0A, 0x00020A09, 0x00020A09, 0x000B0B0B, 0x000B0B0B, 0x000B0B0B, 0x000B0B0B, 0x000A0A0A, 0x00090909, 0x00070707, 0x00060606, 0x00040201, 0x00050302, 0x00050302, 0x00050302, 0x00050302, 0x00050302, 0x00050302, 0x00060403, 0x00000500,
        0x00000400, 0x00000300, 0x00000300, 0x00000400, 0x00000500, 0x00000400, 0x00000300, 0x00000300, 0x00000300, 0x00000500, 0x00000500, 0x00000600, 0x00000600, 0x00000700, 0x00000700, 0x00181E19, 0xFF5F6560, 0xFFE5EBE6, 0xFFFBFFFC,
  0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF
};

static GUI_CONST_STORAGE GUI_BITMAP _bmJTrace_300x164 = {
  300,                                 // XSize
  164,                                 // YSize
  1200,                                // BytesPerLine
  32,                                  // BitsPerPixel
  (unsigned char *)_acJTrace_300x164,  // Pointer to picture data
  NULL,                                // Pointer to palette
  GUI_DRAW_BMP8888
};
#endif

/*********************************************************************
*
*       _bmSeggerLogo_300x181
*/
#if (GUI_USE_ARGB)
static GUI_CONST_STORAGE unsigned long _acSeggerLogo_300x181[] = {
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x50292929, 0x9F292929, 0xDF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xDF292929, 0x9F292929, 0x50292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x40292929, 0xCF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xCF292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x9F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x9F292929, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x70292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x70292929, 0x00000000, 0x00000000,
  0x00000000, 0x30292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x30292929, 0x00000000,
  0x00000000, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF363636, 0xFF797979, 0xFFA1A1A1, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA,
        0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA,
        0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA,
        0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA,
        0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA,
        0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA,
        0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA,
        0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA,
        0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA,
        0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA,
        0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA,
        0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA,
        0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA,
        0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA,
        0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFA1A1A1, 0xFF797979, 0xFF363636, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0x00000000,
  0x30292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF5F5F5F, 0xFFF2F2F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF2F2F2, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x30292929,
  0x8F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF5F5F5F, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x8F292929,
  0xCF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFE4E4E4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE4E4E4, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xCF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF6C6C6C, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF6C6C6C, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFBCBCBC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFBCBCBC, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFC2C4DE, 0xFF9EA1CB, 0xFF9EA1CB,
        0xFFB6B9D8, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFC2C4DE, 0xFF9EA1CB, 0xFF9EA1CB, 0xFFC2C4DE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF3F3F8, 0xFF6E72B0, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF6166AA, 0xFFF3F3F8, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFDBDCEB, 0xFF6166AA, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6E72B0, 0xFFF3F3F8, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF6166AA, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF6166AA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFF3F3F8, 0xFF555BA3, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF7A7EB7, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFC2C4DE, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF9295C4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFAAADD1, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFAAADD1, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFC2C4DE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFF7A7EB7, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFDBDCEB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFE7E8F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFF7A7EB7, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFF3F3F8, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFC2C4DE, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6166AA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFAAADD1, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6E72B0, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF6E72B0, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF9295C4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFF3F3F8, 0xFF555BA3, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF9EA1CB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE7E8F2, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFC2C4DE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFDBDCEB, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFC2C4DE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFC2C4DE, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFE7E8F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFB6B9D8, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFE7E8F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9295C4, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6166AA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF868ABD, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6166AA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF6166AA, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF9295C4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF3F3F8, 0xFF6166AA, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF868ABD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE7E8F2, 0xFF494F9D,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFC2C4DE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE7E8F2, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFB6B9D8, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFC2C4DE,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFE7E8F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFC2C4DE, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFDBDCEB, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFF9295C4, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6166AA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9295C4, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6166AA, 0xFFF3F3F8,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFF6166AA, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF9295C4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF6E72B0, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF7A7EB7,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFE7E8F2, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFC2C4DE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF3F3F8, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFFAAADD1, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFC2C4DE, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFE7E8F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCFD0E5, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFFDBDCEB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9295C4, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6166AA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFAAADD1, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF494F9D, 0xFFF3F3F8, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF6166AA, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF9295C4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF7A7EB7, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF6E72B0, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE7E8F2, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFC2C4DE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF3F3F8, 0xFF555BA3, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF9EA1CB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFC2C4DE, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFE7E8F2, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFDBDCEB, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFC2C4DE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9295C4, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6166AA, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFB6B9D8, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFE7E8F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF6166AA, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF9295C4, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF868ABD, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6166AA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE7E8F2, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFC2C4DE,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF3F3F8, 0xFF6166AA, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF868ABD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFC2C4DE, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D,
        0xFFE7E8F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE7E8F2, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFB6B9D8, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9295C4, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF6166AA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFC2C4DE, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFDBDCEB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF6166AA, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF9295C4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9295C4, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6166AA, 0xFFF3F3F8, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE7E8F2, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFFC2C4DE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF6E72B0, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF7A7EB7, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF3F3F8, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFC2C4DE, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFE7E8F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF3F3F8, 0xFF494F9D,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFAAADD1, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFFDBDCEB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9295C4, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6166AA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCFD0E5,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFCFD0E5, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF555BA3, 0xFFF3F3F8, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF6166AA, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF9295C4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFAAADD1, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFF3F3F8, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF7A7EB7, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE7E8F2, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFC2C4DE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFF7A7EB7, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6E72B0, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFFAAADD1, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFC2C4DE, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFE7E8F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFF3F3F8, 0xFF6166AA, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF9295C4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFDBDCEB, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9295C4, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6166AA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFDBDCEB, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFC2C4DE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF555BA3, 0xFFF3F3F8,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF6166AA, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF9295C4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFB6B9D8, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFE7E8F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF7A7EB7,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE7E8F2, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFC2C4DE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF868ABD, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6166AA, 0xFFF3F3F8, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFFAAADD1, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFC2C4DE, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFE7E8F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF6166AA, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF868ABD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFFDBDCEB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9295C4, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6166AA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE7E8F2, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFB6B9D8, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF555BA3, 0xFFF3F3F8, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF6166AA,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF9295C4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFC2C4DE, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFDBDCEB, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF7A7EB7, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE7E8F2,
        0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFC2C4DE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF555BA3, 0xFFF3F3F8,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFAAADD1, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFC2C4DE, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFE7E8F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF6E72B0, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF555BA3, 0xFF6E72B0, 0xFFC2C4DE,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFDBDCEB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFF9295C4, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6166AA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF3F3F8, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF9295C4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF555BA3, 0xFFF3F3F8, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFF6166AA, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF9295C4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFDBDCEB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFFDBDCEB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF7A7EB7, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFE7E8F2, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFC2C4DE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFAAADD1, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF9EA1CB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF9EA1CB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFC2C4DE, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFE7E8F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF7A7EB7, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF7A7EB7, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFCFD0E5, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9295C4, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6166AA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF3F3F8, 0xFF6166AA, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF9EA1CB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFF3F3F8, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF6166AA, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF9295C4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFDBDCEB, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFFCFD0E5, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6E72B0, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE7E8F2, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFC2C4DE, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF6E72B0, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF9EA1CB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFC2C4DE, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFE7E8F2, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFAAADD1, 0xFF555BA3, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFF9295C4,
        0xFFF3F3F8, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFCFD0E5, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9295C4, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6166AA, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFDBDCEB, 0xFFDBDCEB, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFF3F3F8, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF6166AA, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF9295C4,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6E72B0, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE7E8F2, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFFC2C4DE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF9EA1CB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFC2C4DE, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF555BA3, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFCFD0E5, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9295C4, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFDBDCEB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9295C4, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFAAADD1, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFC2C4DE, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF6166AA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF7A7EB7, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE7E8F2, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFFC2C4DE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF555BA3, 0xFFF3F3F8, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF6166AA, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF9295C4,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFDBDCEB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9295C4, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6166AA, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFDBDCEB, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5,
        0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5,
        0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5,
        0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5,
        0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5,
        0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5,
        0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5,
        0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5,
        0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5,
        0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5,
        0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFCFD0E5, 0xFFDBDCEB, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFAAADD1, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFC2C4DE, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFE7E8F2, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFAAADD1, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF9295C4,
        0xFFF3F3F8, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF7A7EB7, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE7E8F2, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFC2C4DE, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF6E72B0, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF555BA3, 0xFFF3F3F8, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF6166AA, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF9295C4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFDBDCEB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFFC2C4DE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFDBDCEB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9295C4, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6166AA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF3F3F8, 0xFF555BA3, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF9EA1CB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFAAADD1, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFC2C4DE, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFE7E8F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF7A7EB7, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF7A7EB7, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF7A7EB7, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFE7E8F2, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFC2C4DE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFAAADD1, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF9EA1CB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF555BA3, 0xFFF3F3F8, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFF6166AA, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF9295C4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCFD0E5, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFFE7E8F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFDBDCEB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFF9295C4, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6166AA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF3F3F8, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF9295C4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFAAADD1, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFC2C4DE, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFE7E8F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF6E72B0, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF555BA3,
        0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0,
        0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0,
        0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0,
        0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0,
        0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0,
        0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0,
        0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0,
        0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0,
        0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0,
        0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF6E72B0, 0xFF7A7EB7, 0xFFC2C4DE,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF7A7EB7, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE7E8F2,
        0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFC2C4DE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9295C4, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6166AA, 0xFFF3F3F8,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF555BA3, 0xFFF3F3F8, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF6166AA,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF9295C4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFC2C4DE, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFDBDCEB, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFFDBDCEB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9295C4, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6166AA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE7E8F2, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFB6B9D8, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFFAAADD1, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFC2C4DE, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFE7E8F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF3F3F8, 0xFF6166AA, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF868ABD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF7A7EB7,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE7E8F2, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFC2C4DE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF868ABD, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6166AA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF555BA3, 0xFFF3F3F8,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF6166AA, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF9295C4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFB6B9D8, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFE7E8F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFDBDCEB, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9295C4, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6166AA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFDBDCEB, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFC2C4DE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFFAAADD1, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFC2C4DE, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFE7E8F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFF3F3F8, 0xFF555BA3, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF9EA1CB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF7A7EB7, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE7E8F2, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFC2C4DE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFF7A7EB7, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6E72B0, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF555BA3, 0xFFF3F3F8, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF6166AA, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF9295C4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFAAADD1, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFF3F3F8, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFFDBDCEB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9295C4, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6166AA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCFD0E5,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFDBDCEB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF2F2F2, 0xFFCACACA, 0xFFE4E4E4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFD7D7D7, 0xFFE4E4E4,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE4E4E4, 0xFFD7D7D7, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFC2C4DE, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFE7E8F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF3F3F8, 0xFF494F9D,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFAAADD1, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE4E4E4, 0xFFA1A1A1, 0xFF6C6C6C, 0xFF515151, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF444444, 0xFF6C6C6C, 0xFFA1A1A1, 0xFFE4E4E4,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF2F2F2, 0xFFCACACA, 0xFFAFAFAF, 0xFF949494, 0xFF949494, 0xFF949494,
        0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFFAFAFAF, 0xFFCACACA, 0xFFF2F2F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFBCBCBC, 0xFF878787, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF444444, 0xFF6C6C6C, 0xFF949494, 0xFFD7D7D7, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF949494, 0xFF5F5F5F, 0xFF363636, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF363636,
        0xFF5F5F5F, 0xFF949494, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFD7D7D7, 0xFFCACACA, 0xFFA1A1A1, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494,
        0xFFCACACA, 0xFFD7D7D7, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE4E4E4, 0xFFCACACA, 0xFFAFAFAF, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494,
        0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFFCACACA, 0xFFCACACA, 0xFFD7D7D7, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE7E8F2, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFFC2C4DE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF6E72B0, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF7A7EB7, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFD7D7D7, 0xFF6C6C6C, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF5F5F5F, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFAFAFAF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF2F2F2, 0xFF949494, 0xFF363636, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF444444, 0xFF949494, 0xFFF2F2F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF2F2F2, 0xFF949494, 0xFF444444, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF363636, 0xFF878787, 0xFFE4E4E4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF2F2F2, 0xFF515151,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF515151, 0xFFE4E4E4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF797979, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF5F5F5F, 0xFFA1A1A1, 0xFFF2F2F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF6166AA, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF9295C4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9295C4, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6166AA, 0xFFF3F3F8, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFF878787, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF6C6C6C, 0xFFF2F2F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE4E4E4, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFA1A1A1, 0xFF363636, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF363636, 0xFFAFAFAF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFBCBCBC, 0xFF363636, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF363636, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF797979, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF797979, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFAFAFAF, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF363636, 0xFFBCBCBC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9295C4, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF6166AA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFC2C4DE, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFDBDCEB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFF797979, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF515151, 0xFFF2F2F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFA1A1A1, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF2F2F2, 0xFF6C6C6C, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF797979, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF878787, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF515151, 0xFFF2F2F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF515151, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF5F5F5F, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF797979, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFAFAFAF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFC2C4DE, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D,
        0xFFE7E8F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE7E8F2, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFB6B9D8, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFAFAFAF, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF6C6C6C, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF515151, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF2F2F2, 0xFF515151, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF797979, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF797979, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF515151, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFFA1A1A1, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFD7D7D7, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE7E8F2, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFC2C4DE,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF3F3F8, 0xFF6166AA, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF9295C4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFF444444, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF6C6C6C, 0xFFBCBCBC, 0xFFF2F2F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFD7D7D7, 0xFFA1A1A1, 0xFF444444, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFD7D7D7, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF515151, 0xFF797979, 0xFFE4E4E4, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF6C6C6C, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF444444, 0xFF878787, 0xFFAFAFAF, 0xFFCACACA, 0xFFCACACA,
        0xFFAFAFAF, 0xFF878787, 0xFF363636, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFF878787, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF444444, 0xFF797979, 0xFFA1A1A1, 0xFFCACACA, 0xFFCACACA, 0xFFBCBCBC, 0xFF878787, 0xFF444444,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF5F5F5F, 0xFFAFAFAF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF797979, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF6166AA, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF9295C4, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF868ABD, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6166AA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFAFAFAF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF797979, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFAFAFAF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF5F5F5F,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFA1A1A1, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF444444, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFF2F2F2, 0xFF949494, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF797979, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF363636, 0xFFAFAFAF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFA1A1A1, 0xFF363636, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF5F5F5F, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFBCBCBC, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9295C4, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6166AA, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFB6B9D8, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFE7E8F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF515151, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF797979, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF5F5F5F,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF2F2F2, 0xFF363636, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF515151, 0xFFF2F2F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF363636, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF878787, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFF444444, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF515151, 0xFFE4E4E4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFD7D7D7, 0xFF363636, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF5F5F5F, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFD7D7D7, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFC2C4DE, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFE7E8F2, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFDBDCEB, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFCFD0E5, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF878787, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF878787,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF797979, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF5F5F5F,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF515151, 0xFFF2F2F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFD7D7D7, 0xFF363636, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFBCBCBC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFAFAFAF, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF363636, 0xFFE4E4E4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFD7D7D7, 0xFF363636, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFBCBCBC, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE7E8F2, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFC2C4DE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF3F3F8, 0xFF555BA3, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF9EA1CB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF363636, 0xFFD7D7D7, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFE4E4E4, 0xFFA1A1A1, 0xFF949494, 0xFFD7D7D7, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF5F5F5F,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF444444, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFD7D7D7, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFD7D7D7, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF363636, 0xFFAFAFAF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFF6C6C6C, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFAFAFAF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF2F2F2, 0xFF797979, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF878787, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF6166AA, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF9295C4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF7A7EB7, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF6E72B0, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFAFAFAF,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF363636, 0xFF949494, 0xFFF2F2F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF5F5F5F,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFD7D7D7, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF6C6C6C, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF2F2F2,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF444444, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFD7D7D7, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFD7D7D7, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9295C4, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6166AA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFAAADD1, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF555BA3, 0xFFF3F3F8, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE4E4E4,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF5F5F5F, 0xFF949494, 0xFFCACACA, 0xFFF2F2F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF5F5F5F,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFA1A1A1, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFAFAFAF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF363636, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFC2C4DE, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFE7E8F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCFD0E5, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFFDBDCEB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF515151, 0xFF797979, 0xFFA1A1A1, 0xFFD7D7D7, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF363636, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFF878787, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFF2F2F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF515151, 0xFFD7D7D7, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF797979, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFE7E8F2, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFC2C4DE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF3F3F8, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFFAAADD1, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFD7D7D7, 0xFF363636, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF363636, 0xFF6C6C6C,
        0xFFA1A1A1, 0xFFE4E4E4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFA1A1A1, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFF2F2F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF363636, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF5F5F5F, 0xFF949494, 0xFF949494, 0xFF949494,
        0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF5F5F5F, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF363636, 0xFFF2F2F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFF6166AA, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF9295C4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF6E72B0, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF868ABD,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFD7D7D7, 0xFF444444, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF5F5F5F, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF5F5F5F, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFF878787, 0xFF363636, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF5F5F5F, 0xFFBCBCBC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494, 0xFF444444, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF5F5F5F, 0xFFAFAFAF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF363636, 0xFFD7D7D7, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFF9295C4, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6166AA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9295C4, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6166AA, 0xFFF3F3F8,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFF2F2F2, 0xFF949494, 0xFF363636, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF878787, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFA1A1A1, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF5F5F5F, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFA1A1A1,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFE4E4E4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFAFAFAF, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF363636, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF363636, 0xFF949494, 0xFFF2F2F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFC2C4DE,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFE7E8F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFC2C4DE, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFDBDCEB, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFBCBCBC, 0xFF797979, 0xFF444444, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF797979, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF363636, 0xFF878787, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF444444, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF444444, 0xFFBCBCBC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF797979, 0xFFF2F2F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE7E8F2, 0xFF494F9D,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFC2C4DE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE7E8F2, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFB6B9D8, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFD7D7D7, 0xFFAFAFAF, 0xFF878787, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFAFAFAF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF515151,
        0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFE4E4E4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF878787,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFD7D7D7, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFAFAFAF, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFA1A1A1, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF363636, 0xFFE4E4E4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF6166AA, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF9295C4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF3F3F8, 0xFF6166AA, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF9295C4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF949494, 0xFF515151, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF515151, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF5F5F5F,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFD7D7D7, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF2F2F2,
        0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFAFAFAF,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFAFAFAF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF797979, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF444444, 0xFF5F5F5F, 0xFF5F5F5F, 0xFF5F5F5F,
        0xFF5F5F5F, 0xFF5F5F5F, 0xFF5F5F5F, 0xFF5F5F5F, 0xFF5F5F5F, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF6C6C6C, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9295C4, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6166AA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF868ABD, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6166AA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE4E4E4, 0xFF949494, 0xFF363636, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF5F5F5F,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFAFAFAF, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFD7D7D7, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFF797979, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFD7D7D7,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF6C6C6C, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE4E4E4, 0xFFCACACA,
        0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFFCACACA, 0xFF949494, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF2F2F2, 0xFFA1A1A1, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFF2F2F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFC2C4DE, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFE7E8F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFB6B9D8, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFE7E8F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFF2F2F2, 0xFF878787, 0xFF5F5F5F, 0xFF5F5F5F, 0xFFA1A1A1, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE4E4E4, 0xFF515151,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF5F5F5F,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFE4E4E4, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF363636, 0xFFF2F2F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF515151, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFF363636, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFE4E4E4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF797979, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE7E8F2, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFC2C4DE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFDBDCEB, 0xFF494F9D, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFCFD0E5, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFF515151, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF515151, 0xFFF2F2F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE4E4E4,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF5F5F5F,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFF797979, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF6C6C6C, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE4E4E4, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFAFAFAF, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFAFAFAF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF6E72B0, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF9295C4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFF3F3F8, 0xFF555BA3, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF9EA1CB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFBCBCBC,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF797979, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF2F2F2,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFD7D7D7, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF5F5F5F,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFAFAFAF, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFAFAFAF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF2F2F2, 0xFF363636, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFE4E4E4, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFA1A1A1, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFF2F2F2, 0xFF515151, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFC2C4DE, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6166AA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFAAADD1, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6E72B0, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFA1A1A1,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF797979, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFA1A1A1,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF363636, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF5F5F5F,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFA1A1A1, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFD7D7D7, 0xFF444444, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFF797979, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF878787, 0xFFF2F2F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFE4E4E4, 0xFF515151, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF494F9D, 0xFFE7E8F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFF7A7EB7, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF555BA3, 0xFFF3F3F8, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFD7D7D7,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF6C6C6C, 0xFFE4E4E4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF2F2F2, 0xFF949494, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF797979, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF444444,
        0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFFD7D7D7, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFD7D7D7, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF515151, 0xFFAFAFAF, 0xFFF2F2F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFD7D7D7, 0xFF797979, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFF2F2F2, 0xFF363636, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF444444, 0xFFA1A1A1, 0xFFE4E4E4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE4E4E4,
        0xFF949494, 0xFF363636, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF797979, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494,
        0xFF949494, 0xFF949494, 0xFFAFAFAF, 0xFFF2F2F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF878787, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF9EA1CB, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFC2C4DE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFF7A7EB7, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFDBDCEB, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF515151, 0xFF878787, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFAFAFAF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFAFAFAF, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF515151, 0xFF5F5F5F, 0xFF5F5F5F,
        0xFF5F5F5F, 0xFF363636, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF444444, 0xFF5F5F5F, 0xFF5F5F5F, 0xFF5F5F5F, 0xFF444444, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF363636, 0xFFE4E4E4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF444444, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF515151, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFC2C4DE, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF9295C4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFAAADD1, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFFAAADD1, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFD7D7D7, 0xFF363636, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF878787, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF363636, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFAFAFAF, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFAFAFAF, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF6166AA, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF3D4396, 0xFF6E72B0, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFF3F3F8, 0xFF555BA3, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF868ABD, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFD7D7D7, 0xFF444444, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF797979, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFAFAFAF, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFAFAFAF, 0xFF363636, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF515151, 0xFFAFAFAF, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFD7D7D7, 0xFF363636, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF515151, 0xFFBCBCBC, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF3F3F8, 0xFF6E72B0, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396,
        0xFF3D4396, 0xFF6166AA, 0xFFF3F3F8, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFDBDCEB, 0xFF6166AA, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF3D4396, 0xFF6E72B0, 0xFFF3F3F8, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFF2F2F2, 0xFF878787, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF363636, 0xFFAFAFAF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF2F2F2, 0xFF363636, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF5F5F5F, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE4E4E4, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF878787, 0xFFFFFFFF, 0xFFF2F2F2, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF2F2F2, 0xFF797979, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF5F5F5F, 0xFFF2F2F2, 0xFFFFFFFF, 0xFF363636, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFA1A1A1, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFBCBCBC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF878787, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFAFAFAF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFD7D7D7, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFE4E4E4, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCFD0E5, 0xFF9EA1CB, 0xFF9EA1CB,
        0xFFC2C4DE, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFC2C4DE, 0xFF9EA1CB, 0xFF9EA1CB, 0xFFCFD0E5, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE4E4E4, 0xFF949494, 0xFF444444, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF444444, 0xFF949494, 0xFFF2F2F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFD7D7D7, 0xFF6C6C6C, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF363636, 0xFF797979, 0xFFF2F2F2, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF797979, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF5F5F5F, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFE4E4E4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE4E4E4, 0xFF797979, 0xFF363636, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF444444, 0xFFBCBCBC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494,
        0xFF444444, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF444444, 0xFFA1A1A1, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF2F2F2, 0xFF515151, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF5F5F5F, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF878787, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFAFAFAF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFD7D7D7, 0xFFA1A1A1, 0xFF878787, 0xFF5F5F5F, 0xFF5F5F5F, 0xFF5F5F5F, 0xFF5F5F5F, 0xFF5F5F5F, 0xFF5F5F5F, 0xFF5F5F5F, 0xFF878787, 0xFFA1A1A1, 0xFFD7D7D7,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF2F2F2, 0xFFBCBCBC, 0xFF949494, 0xFF6C6C6C, 0xFF5F5F5F, 0xFF5F5F5F, 0xFF5F5F5F, 0xFF5F5F5F,
        0xFF5F5F5F, 0xFF797979, 0xFFA1A1A1, 0xFFE4E4E4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF2F2F2, 0xFF797979, 0xFF5F5F5F, 0xFF5F5F5F, 0xFFBCBCBC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF949494, 0xFF797979, 0xFF5F5F5F, 0xFF5F5F5F, 0xFF5F5F5F, 0xFF5F5F5F, 0xFF5F5F5F, 0xFF6C6C6C, 0xFFA1A1A1,
        0xFFD7D7D7, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494, 0xFF5F5F5F, 0xFF5F5F5F, 0xFFAFAFAF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFFA1A1A1, 0xFFA1A1A1, 0xFFD7D7D7, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE4E4E4, 0xFFAFAFAF, 0xFF949494, 0xFFAFAFAF, 0xFFE4E4E4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFCACACA, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFCACACA, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFBCBCBC, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFBCBCBC, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF949494, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF949494, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF6C6C6C, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF6C6C6C, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0xCF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFFE4E4E4, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFE4E4E4, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xCF292929,
  0x8F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF515151, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFF515151, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x8F292929,
  0x20292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF5F5F5F, 0xFFF2F2F2, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,
        0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFF2F2F2, 0xFF5F5F5F, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x20292929,
  0x00000000, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF363636, 0xFF797979, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494,
        0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494,
        0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494,
        0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494,
        0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494,
        0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494,
        0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494,
        0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494,
        0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494,
        0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494,
        0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494,
        0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494,
        0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494,
        0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494,
        0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF949494, 0xFF797979, 0xFF363636, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0x00000000,
  0x00000000, 0x20292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x20292929, 0x00000000,
  0x00000000, 0x00000000, 0x70292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x60292929, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x9F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x9F292929, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x40292929, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x40292929, 0x9F292929, 0xDF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xDF292929, 0x9F292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x10292929, 0x40292929, 0x10292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x20292929, 0x40292929, 0x20292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x20292929, 0x40292929, 0x10292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x10292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x30292929, 0x40292929, 0x10292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x30292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x20292929,
        0x40292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x40292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x40292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x30292929, 0x50292929, 0x80292929, 0x80292929, 0x80292929, 0x70292929, 0x40292929, 0x10292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x10292929, 0x40292929, 0x80292929, 0x80292929, 0x80292929, 0x70292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x20292929, 0x50292929, 0x80292929, 0x80292929, 0x80292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x10292929, 0x40292929, 0x20292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x20292929, 0x60292929, 0x80292929, 0x80292929, 0x80292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x10292929, 0x40292929, 0x20292929, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x40292929, 0x70292929, 0x80292929, 0x80292929, 0x80292929, 0x40292929, 0x20292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x40292929, 0x30292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x40292929, 0x30292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x30292929, 0x60292929, 0x80292929, 0x80292929, 0x80292929, 0x50292929, 0x30292929, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x20292929, 0x50292929, 0x80292929, 0x80292929, 0x80292929, 0x80292929, 0x40292929,
        0x10292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x30292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x10292929, 0x50292929,
        0x80292929, 0x80292929, 0x70292929, 0x30292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x40292929, 0x80292929, 0x80292929, 0x80292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x60292929, 0xEF292929, 0xFF292929, 0xFF292929, 0x60292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x20292929, 0xDF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xCF292929, 0x10292929, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x70292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x60292929, 0x00000000, 0x00000000, 0x8F292929, 0xFF292929, 0xFF292929, 0xEF292929, 0x50292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x50292929,
        0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x9F292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x9F292929, 0xFF292929, 0xFF292929, 0xEF292929, 0x30292929, 0x00000000, 0x00000000, 0xBF292929, 0xFF292929,
        0xFF292929, 0xDF292929, 0x30292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x60292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x60292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x10292929,
        0xCF292929, 0xFF292929, 0xFF292929, 0xCF292929, 0x10292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x70292929, 0xDF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xAF292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x30292929,
        0xAF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x9F292929, 0x20292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x30292929, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xEF292929, 0x80292929, 0x00000000, 0x60292929, 0xEF292929, 0xFF292929, 0xFF292929, 0x9F292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x30292929, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xEF292929, 0x70292929, 0x00000000, 0x60292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x9F292929, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x10292929, 0x80292929, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0x50292929, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x30292929, 0xEF292929, 0xFF292929, 0xFF292929, 0xCF292929, 0x10292929, 0x00000000, 0x70292929, 0xEF292929, 0xFF292929, 0xFF292929, 0xDF292929, 0x30292929, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xDF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xDF292929,
        0x60292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x60292929, 0xDF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xAF292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x10292929, 0xBF292929, 0xFF292929, 0xFF292929, 0xEF292929, 0x30292929, 0x10292929, 0x8F292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xDF292929, 0x40292929, 0x00000000, 0x00000000, 0x80292929, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xEF292929, 0x80292929, 0x00000000, 0x00000000,
  0x00000000, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x20292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x9F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x20292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xDF292929, 0x00000000, 0x30292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xDF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xDF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x50292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x60292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xAF292929, 0x00000000, 0x60292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xAF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x10292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x20292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x8F292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x70292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x10292929, 0xCF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x9F292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x70292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x60292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x60292929,
        0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x9F292929, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x30292929, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x60292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x9F292929, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x30292929,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x30292929, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xAF292929, 0x10292929,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xAF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x50292929, 0x8F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xAF292929, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x20292929, 0xCF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xCF292929, 0x20292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x30292929, 0xCF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0x9F292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x60292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xAF292929, 0xCF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x60292929, 0x9F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x9F292929, 0x00000000,
  0x00000000, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x70292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xCF292929, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x70292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xEF292929, 0x00000000, 0x40292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x30292929, 0x00000000, 0x00000000, 0x00000000, 0x20292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x8F292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x9F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x60292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x60292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xDF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x60292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x70292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x60292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x30292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x40292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x30292929, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xCF292929,
        0x10292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x10292929, 0xDF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xDF292929, 0x10292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x30292929, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x60292929,
  0x00000000, 0xDF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xAF292929, 0x00000000, 0x00000000, 0x00000000, 0x20292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x10292929, 0x00000000, 0x00000000,
        0x00000000, 0xAF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xAF292929, 0x00000000, 0x20292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x70292929, 0x00000000, 0x00000000, 0x00000000, 0x60292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xCF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xDF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x50292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000, 0x8F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x9F292929, 0x00000000, 0x00000000, 0x00000000, 0x20292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x50292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x50292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xEF292929, 0x9F292929, 0x80292929, 0x9F292929, 0xDF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x30292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xDF292929, 0xDF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xDF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000,
        0x10292929, 0xDF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xCF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xEF292929, 0xCF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0x8F292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x9F292929, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xAF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xCF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xDF292929,
  0x00000000, 0x9F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xEF292929, 0x00000000, 0x00000000, 0x00000000, 0x60292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929, 0x00000000, 0x00000000,
        0x00000000, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x60292929, 0x00000000, 0x00000000, 0xCF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0x00000000, 0x00000000, 0x00000000, 0x8F292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x10292929, 0x00000000, 0x00000000, 0x20292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x30292929, 0x00000000, 0x00000000, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x00000000, 0x00000000, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xDF292929, 0x00000000, 0x00000000, 0x00000000, 0x60292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xEF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x8F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xEF292929, 0x10292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x70292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x60292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xAF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xAF292929, 0x20292929, 0x00000000, 0x00000000, 0x20292929, 0x9F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0x00000000, 0x00000000, 0x00000000, 0x60292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x20292929, 0x00000000, 0x00000000, 0x60292929, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000,
        0x60292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x10292929, 0x00000000, 0x10292929, 0x60292929, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929,
        0x00000000, 0x00000000, 0x00000000, 0x60292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xDF292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000, 0x70292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0x20292929, 0x00000000, 0x00000000, 0x00000000, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xCF292929, 0x9F292929, 0x10292929, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x40292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x70292929, 0x10292929, 0x00000000, 0x10292929, 0x80292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0x00000000, 0x00000000, 0x00000000, 0x70292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x10292929, 0x00000000, 0x00000000, 0x20292929, 0xAF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x20292929, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x20292929, 0x00000000,
        0x50292929, 0xCF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xCF292929, 0x40292929, 0x00000000, 0x10292929, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0x00000000, 0x50292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x30292929, 0x00000000, 0x00000000, 0x8F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x00000000,
        0x30292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x10292929, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x00000000, 0x00000000, 0x00000000, 0xCF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x50292929, 0x00000000, 0x00000000, 0x60292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xDF292929, 0x00000000, 0x00000000, 0x00000000, 0xAF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0x00000000, 0x00000000, 0x00000000, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x10292929, 0x00000000, 0x00000000, 0x9F292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0x9F292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xEF292929, 0x10292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x20292929, 0x8F292929, 0xBF292929, 0x60292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x20292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xCF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x20292929, 0x00000000, 0x00000000, 0xAF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x30292929, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000,
        0xCF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x70292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x30292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929,
        0x00000000, 0x00000000, 0x00000000, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x30292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x70292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0x70292929, 0x00000000, 0x00000000, 0x00000000, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x10292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x9F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x60292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0x9F292929, 0x00000000, 0x00000000, 0x00000000, 0xDF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x60292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x40292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xEF292929, 0x10292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x9F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0x00000000, 0x00000000, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x70292929, 0x00000000, 0x00000000, 0xCF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xCF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0x00000000, 0x00000000,
        0x70292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xCF292929, 0x00000000, 0x00000000, 0x00000000, 0x30292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929, 0x00000000, 0x00000000, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xCF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x00000000, 0x9F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x8F292929, 0x00000000, 0x00000000, 0x00000000, 0x60292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x00000000, 0x00000000, 0x40292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xDF292929, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x50292929, 0x00000000, 0x00000000, 0xDF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0x60292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x9F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xEF292929, 0x9F292929, 0x60292929, 0x30292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x50292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0x60292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x60292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x60292929, 0x00000000, 0x00000000, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xEF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x8F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xDF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x9F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929,
        0x00000000, 0x00000000, 0x00000000, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x10292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xBF292929, 0x00000000, 0x00000000, 0x00000000, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x70292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xDF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x10292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x50292929,
        0xCF292929, 0xEF292929, 0x9F292929, 0x10292929, 0x00000000, 0x00000000, 0x20292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xCF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x20292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x10292929, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x8F292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0x00000000, 0x00000000, 0xAF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xAF292929, 0x00000000, 0x00000000, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x60292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x00000000, 0x00000000,
        0xAF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x70292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xDF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x40292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xCF292929, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0x00000000, 0x00000000, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x50292929, 0x00000000, 0x00000000, 0x00000000, 0x20292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x50292929, 0x00000000, 0x70292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x9F292929, 0xAF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x8F292929, 0x00000000, 0x20292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0x10292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x60292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0x80292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xCF292929, 0xBF292929, 0xBF292929, 0xBF292929, 0xBF292929, 0xBF292929, 0xBF292929, 0xCF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x20292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0x9F292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x40292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929, 0x00000000, 0x00000000, 0x30292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x8F292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x50292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929,
        0x00000000, 0x00000000, 0x30292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xDF292929, 0xBF292929, 0xBF292929, 0xBF292929, 0xBF292929, 0xBF292929, 0xBF292929, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xBF292929, 0x00000000, 0x00000000, 0x00000000, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xCF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x40292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0xCF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0x00000000, 0x00000000, 0x60292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xEF292929, 0x00000000, 0x40292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xCF292929, 0x10292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x30292929, 0x00000000,
        0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x20292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x8F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0x00000000, 0x70292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0x9F292929, 0x40292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x00000000, 0x30292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xEF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xCF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x8F292929, 0x00000000, 0xAF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x60292929, 0x70292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0x00000000, 0x60292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xAF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xCF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xDF292929, 0x80292929, 0x10292929, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x40292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x10292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929, 0x00000000, 0x00000000, 0x40292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x40292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929,
        0x00000000, 0x00000000, 0x40292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xBF292929, 0x00000000, 0x00000000, 0x00000000, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x20292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0x00000000, 0x00000000, 0x10292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x30292929, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x8F292929, 0x00000000, 0xCF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x70292929, 0x30292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xDF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x50292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x00000000, 0xAF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0x60292929, 0x00000000, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929, 0x70292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x9F292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x70292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xCF292929, 0x00000000, 0xDF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x20292929, 0x30292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x00000000, 0x9F292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0x70292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x10292929, 0x9F292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xCF292929, 0x10292929, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x40292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x10292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929, 0x00000000, 0x00000000, 0x40292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x40292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929,
        0x00000000, 0x00000000, 0x40292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xBF292929, 0x00000000, 0x00000000, 0x00000000, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x40292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0xAF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0x00000000, 0x00000000, 0x00000000, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0xAF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x50292929, 0x00000000, 0x8F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xAF292929, 0x80292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x8F292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929, 0xDF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0x20292929, 0x00000000, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x70292929, 0xAF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x60292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x30292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x20292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xEF292929, 0x00000000, 0x00000000, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929, 0xDF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0x20292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x20292929, 0x8F292929,
        0xDF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x9F292929, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xCF292929, 0xBF292929, 0xBF292929, 0xBF292929, 0xBF292929, 0xBF292929, 0xBF292929, 0xBF292929, 0xBF292929, 0xBF292929, 0xBF292929, 0xBF292929, 0x9F292929, 0x10292929, 0x00000000, 0x30292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0x8F292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x40292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929, 0x00000000, 0x00000000, 0x40292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x40292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929,
        0x00000000, 0x00000000, 0x40292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xDF292929, 0xBF292929, 0xBF292929, 0xBF292929, 0xBF292929, 0xBF292929, 0xBF292929, 0xBF292929, 0xBF292929, 0xBF292929, 0xBF292929, 0xBF292929,
        0xBF292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x10292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x70292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0x00000000, 0x00000000, 0x00000000, 0x70292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0xDF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x20292929, 0x00000000, 0x50292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xDF292929, 0xBF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x30292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x9F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xAF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xDF292929, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xAF292929, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x10292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0xDF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x9F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xAF292929, 0x00000000, 0x00000000, 0xAF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x9F292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xCF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x30292929, 0x60292929, 0x9F292929, 0xCF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x10292929, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xCF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x70292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929, 0x00000000, 0x00000000, 0x10292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xCF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x8F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929,
        0x00000000, 0x00000000, 0x20292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x9F292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xDF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x10292929,
        0x70292929, 0x80292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000, 0x40292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0xDF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xEF292929, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0x00000000, 0x00000000, 0x00000000, 0x20292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xDF292929, 0x00000000, 0x00000000, 0x00000000, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xEF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x60292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xAF292929, 0x00000000, 0x00000000, 0x40292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xAF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x8F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x70292929, 0x00000000, 0x00000000, 0x70292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0x80292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x60292929, 0xAF292929, 0x8F292929, 0x20292929,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x10292929, 0x80292929, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929, 0x00000000, 0x40292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0x8F292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x50292929, 0x60292929, 0x20292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xCF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x10292929, 0xDF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000,
        0xDF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x10292929, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929,
        0x00000000, 0x00000000, 0x00000000, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xDF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x30292929, 0x80292929, 0x30292929,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xCF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x10292929, 0xCF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0x50292929, 0x00000000, 0x00000000, 0x10292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xDF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x40292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xCF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x9F292929, 0x00000000, 0x00000000, 0x00000000, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0x9F292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x10292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0x70292929, 0x00000000, 0x00000000, 0x00000000, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x70292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x40292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x30292929, 0x00000000, 0x00000000, 0x30292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0x30292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xEF292929, 0xFF292929, 0xCF292929, 0x50292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x70292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xEF292929,
        0x60292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x8F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x30292929, 0x00000000, 0x00000000, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xEF292929, 0x30292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x30292929, 0xCF292929, 0xFF292929, 0xFF292929, 0xEF292929, 0x20292929, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xEF292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000, 0x10292929, 0xAF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000,
        0x8F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xEF292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000, 0x20292929, 0xCF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929,
        0x00000000, 0x00000000, 0x00000000, 0x9F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x70292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x10292929, 0x9F292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0x70292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xEF292929, 0xFF292929,
        0xCF292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x8F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xCF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xCF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0x00000000, 0x00000000, 0x00000000, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x9F292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x10292929,
        0xCF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x70292929, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x8F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x60292929, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0x50292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xAF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0x30292929, 0x00000000, 0x00000000, 0x00000000, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x20292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xDF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x8F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x30292929, 0x00000000, 0x00000000, 0x00000000, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0x70292929, 0x10292929, 0x00000000, 0x00000000, 0x00000000, 0x20292929, 0xDF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xEF292929, 0x60292929, 0x20292929, 0x00000000, 0x20292929, 0x80292929, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x70292929, 0x00000000, 0x00000000, 0x00000000, 0x10292929, 0xEF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xDF292929, 0xBF292929, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000,
        0x20292929, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xCF292929, 0xBF292929, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929,
        0x00000000, 0x00000000, 0x00000000, 0x40292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x8F292929, 0x30292929, 0x00000000, 0x10292929, 0x60292929, 0xCF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xBF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x8F292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0x20292929, 0x00000000, 0x00000000, 0x00000000, 0x20292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xCF292929, 0x60292929, 0x40292929, 0x50292929, 0xCF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xAF292929, 0x00000000, 0x00000000, 0x00000000, 0x60292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xAF292929, 0x40292929, 0x00000000, 0x20292929, 0x60292929, 0xDF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xEF292929, 0x10292929, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x30292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x30292929, 0x00000000, 0x00000000, 0x00000000, 0x40292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x70292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xEF292929,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xCF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x9F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xAF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0x8F292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x00000000, 0x00000000, 0x9F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0xBF292929, 0xDF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x9F292929, 0x00000000, 0x00000000, 0x00000000, 0x10292929, 0xEF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x30292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x60292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x70292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x8F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0x8F292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x8F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x60292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xDF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xEF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xAF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x20292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x40292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x70292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x50292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x70292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0x50292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x00000000, 0x00000000, 0x20292929, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xEF292929, 0x20292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x30292929, 0xEF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x9F292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x80292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xCF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x9F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x10292929, 0xCF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xCF292929,
        0x10292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x10292929, 0xCF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x10292929, 0xCF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x9F292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x80292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x70292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x9F292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x9F292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0x50292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x60292929,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xDF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x20292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x30292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x20292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xDF292929,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x8F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x20292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x30292929, 0xDF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xCF292929, 0x30292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x30292929, 0xDF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xEF292929, 0x70292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x50292929, 0xDF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0x10292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x50292929, 0xDF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xBF292929, 0x50292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x10292929, 0xAF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xAF292929, 0x10292929,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x9F292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x9F292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x8F292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0x20292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x10292929, 0xAF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xEF292929, 0x60292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x10292929, 0xAF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xEF292929, 0x60292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x60292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xCF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0xCF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x50292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x30292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xEF292929,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x10292929, 0x9F292929, 0xEF292929, 0xFF292929, 0xFF292929, 0xAF292929, 0x10292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x10292929, 0xAF292929, 0xFF292929, 0xFF292929,
        0xEF292929, 0x8F292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x30292929, 0xAF292929, 0xFF292929, 0xFF292929, 0xEF292929, 0x8F292929, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x30292929, 0xCF292929, 0xFF292929, 0xFF292929, 0xDF292929, 0x60292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x40292929, 0xCF292929, 0xFF292929, 0xFF292929, 0xDF292929, 0x70292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x60292929, 0xDF292929, 0xFF292929, 0xFF292929, 0xCF292929, 0x30292929,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x8F292929, 0xEF292929, 0xFF292929, 0xCF292929, 0x50292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x60292929, 0xBF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xEF292929, 0xAF292929, 0x60292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x10292929,
        0x70292929, 0xCF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xCF292929, 0x80292929, 0x20292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x40292929, 0x80292929, 0x9F292929, 0xBF292929, 0x9F292929, 0x80292929, 0x30292929, 0x00000000, 0x30292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x50292929, 0x80292929, 0xAF292929, 0xBF292929, 0x9F292929, 0x80292929, 0x30292929, 0x00000000, 0x40292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x40292929, 0xAF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xEF292929, 0x9F292929, 0x40292929, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x10292929, 0xAF292929, 0xFF292929, 0xFF292929, 0xAF292929, 0x10292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x80292929, 0xEF292929, 0xFF292929,
        0xCF292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x40292929, 0xAF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xDF292929, 0x80292929,
        0x10292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x40292929, 0x9F292929, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929,
        0xDF292929, 0x80292929, 0x10292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x8F292929, 0xEF292929, 0xFF292929, 0xCF292929, 0x30292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x30292929, 0xCF292929, 0xFF292929, 0xEF292929, 0x8F292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x60292929, 0xDF292929, 0xFF292929, 0xCF292929, 0x40292929,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x30292929, 0x40292929, 0x40292929, 0x40292929, 0x40292929, 0x20292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x10292929, 0x40292929, 0x40292929, 0x40292929, 0x40292929, 0x10292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x20292929,
        0x40292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x50292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x20292929, 0x40292929, 0x40292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x60292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x30292929,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x30292929, 0x40292929, 0x40292929, 0x40292929, 0x20292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x40292929, 0x40292929, 0x40292929, 0x40292929, 0x20292929, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x20292929, 0x40292929, 0x40292929, 0x40292929, 0x40292929, 0x10292929,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x50292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xCF292929, 0x30292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xAF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x60292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xCF292929, 0x30292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xBF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x8F292929, 0x30292929, 0x00000000, 0x00000000, 0x40292929, 0xAF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xCF292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xEF292929, 0x80292929, 0x20292929, 0x00000000, 0x00000000, 0x40292929, 0xAF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xAF292929, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xBF292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x60292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x40292929, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x50292929, 0xFF292929,
        0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x9F292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x70292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x9F292929, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x60292929,
        0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x8F292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x70292929, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0x8F292929, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x10292929, 0x70292929, 0xBF292929, 0xEF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xEF292929, 0xBF292929, 0x80292929, 0x20292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x10292929, 0x70292929, 0xBF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xFF292929, 0xEF292929, 0xBF292929, 0x70292929, 0x20292929, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000
};

static GUI_CONST_STORAGE GUI_BITMAP _bmSeggerLogo_300x181 = {
  300,                                     // XSize
  181,                                     // YSize
  1200,                                    // BytesPerLine
  32,                                      // BitsPerPixel
  (unsigned char *)_acSeggerLogo_300x181,  // Pointer to picture data
  NULL,                                    // Pointer to palette
  GUI_DRAW_BMPM8888I
};
#else
static GUI_CONST_STORAGE unsigned long _acSeggerLogo_300x181[] = {
  0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xAF292929, 0x60292929, 0x20292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x20292929, 0x60292929, 0xAF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
  0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xBF292929, 0x30292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x30292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
  0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
  0xFF000000, 0xFF000000, 0xFF000000, 0x60292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x60292929, 0xFF000000, 0xFF000000, 0xFF000000,
  0xFF000000, 0xFF000000, 0x8F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x8F292929, 0xFF000000, 0xFF000000,
  0xFF000000, 0xCF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xCF292929, 0xFF000000,
  0xFF000000, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00363636, 0x00797979, 0x00A1A1A1, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA,
        0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA,
        0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA,
        0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA,
        0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA,
        0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA,
        0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA,
        0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA,
        0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA,
        0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA,
        0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA,
        0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA,
        0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA,
        0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA,
        0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00A1A1A1, 0x00797979, 0x00363636, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x40292929, 0xFF000000,
  0xCF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x005F5F5F, 0x00F2F2F2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2F2F2, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xCF292929,
  0x70292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x005F5F5F, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x70292929,
  0x30292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00E4E4E4, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00E4E4E4, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x30292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x006C6C6C, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x006C6C6C, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00BCBCBC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00BCBCBC, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00DEC4C2, 0x00CBA19E, 0x00CBA19E,
        0x00D8B9B6, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00DEC4C2, 0x00CBA19E, 0x00CBA19E, 0x00DEC4C2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F8F3F3, 0x00B0726E, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x00AA6661, 0x00F8F3F3, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00EBDCDB, 0x00AA6661, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00B0726E, 0x00F8F3F3, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00AA6661, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x00AA6661, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00F8F3F3, 0x00A35B55, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00B77E7A, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00DEC4C2, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x00C49592, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00D1ADAA, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00D1ADAA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00DEC4C2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00B77E7A, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00EBDCDB, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00F2E8E7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00B77E7A, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00F8F3F3, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00DEC4C2, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00AA6661, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00D1ADAA, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00B0726E, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00B0726E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00C49592, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00F8F3F3, 0x00A35B55, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00CBA19E, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2E8E7, 0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00DEC4C2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00EBDCDB, 0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00DEC4C2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00DEC4C2, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00F2E8E7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00D8B9B6, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00F2E8E7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00C49592, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00AA6661, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00BD8A86, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00AA6661, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00AA6661, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00C49592, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F8F3F3, 0x00AA6661, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00BD8A86, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2E8E7, 0x009D4F49,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00DEC4C2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2E8E7, 0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00D8B9B6, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00DEC4C2,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00F2E8E7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00DEC4C2, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00EBDCDB, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00C49592, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00AA6661, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00C49592, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00AA6661, 0x00F8F3F3,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00AA6661, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00C49592, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00B0726E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00B77E7A,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00F2E8E7, 0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00DEC4C2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F8F3F3, 0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x00D1ADAA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00DEC4C2, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00F2E8E7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00E5D0CF, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x00EBDCDB, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00C49592, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00AA6661, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00D1ADAA, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x009D4F49, 0x00F8F3F3, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00AA6661, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00C49592, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00B77E7A, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x00B0726E, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2E8E7, 0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00DEC4C2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F8F3F3, 0x00A35B55, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x00CBA19E, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00DEC4C2, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00F2E8E7, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00EBDCDB, 0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00DEC4C2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00C49592, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00AA6661, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00D8B9B6, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00F2E8E7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00AA6661, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00C49592, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00BD8A86, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00AA6661, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2E8E7, 0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00DEC4C2,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F8F3F3, 0x00AA6661, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00BD8A86, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00DEC4C2, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49,
        0x00F2E8E7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2E8E7, 0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00D8B9B6, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00C49592, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x00AA6661, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00DEC4C2, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00EBDCDB, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00AA6661, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x00C49592, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00C49592, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00AA6661, 0x00F8F3F3, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2E8E7, 0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x00DEC4C2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00B0726E, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00B77E7A, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F8F3F3, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00DEC4C2, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x009D4F49, 0x00F2E8E7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F8F3F3, 0x009D4F49,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00D1ADAA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x00EBDCDB, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00C49592, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x00AA6661, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00E5D0CF,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00E5D0CF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x00A35B55, 0x00F8F3F3, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00AA6661, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00C49592, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00D1ADAA, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00F8F3F3, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x00B77E7A, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2E8E7, 0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00DEC4C2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00B77E7A, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00B0726E, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x00D1ADAA, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00DEC4C2, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00F2E8E7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00F8F3F3, 0x00AA6661, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00C49592, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x00EBDCDB, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00C49592, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00AA6661, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00EBDCDB, 0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00DEC4C2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x00A35B55, 0x00F8F3F3,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00AA6661, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00C49592, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00D8B9B6, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00F2E8E7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00B77E7A,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2E8E7, 0x009D4F49, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00DEC4C2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00BD8A86, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00AA6661, 0x00F8F3F3, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x00D1ADAA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00DEC4C2, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00F2E8E7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00AA6661, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00BD8A86, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x00EBDCDB, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00C49592, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00AA6661, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2E8E7, 0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00D8B9B6, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x00A35B55, 0x00F8F3F3, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00AA6661,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00C49592, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00DEC4C2, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00EBDCDB, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x00B77E7A, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2E8E7,
        0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00DEC4C2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00A35B55, 0x00F8F3F3,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x00D1ADAA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00DEC4C2, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00F2E8E7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00B0726E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00A35B55, 0x00B0726E, 0x00DEC4C2,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00EBDCDB, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00C49592, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00AA6661, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F8F3F3, 0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x00C49592, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00A35B55, 0x00F8F3F3, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00AA6661, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00C49592, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00EBDCDB, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x00EBDCDB, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00B77E7A, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00F2E8E7, 0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00DEC4C2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00D1ADAA, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x00CBA19E, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00CBA19E, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00DEC4C2, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00F2E8E7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00B77E7A, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x00B77E7A, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00E5D0CF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00C49592, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00AA6661, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F8F3F3, 0x00AA6661, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x00CBA19E, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00F8F3F3, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00AA6661, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00C49592, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00EBDCDB, 0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x00E5D0CF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00B0726E, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2E8E7, 0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00DEC4C2, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x00B0726E, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00CBA19E, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00DEC4C2, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00F2E8E7, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00D1ADAA, 0x00A35B55, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00C49592,
        0x00F8F3F3, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00E5D0CF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00C49592, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00AA6661, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00EBDCDB, 0x00EBDCDB, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00F8F3F3, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00AA6661, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00C49592,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00B0726E, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2E8E7, 0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x00DEC4C2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00CBA19E, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00DEC4C2, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x00A35B55, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00E5D0CF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00C49592, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00EBDCDB, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00C49592, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00D1ADAA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00DEC4C2, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x00AA6661, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00B77E7A, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2E8E7, 0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x00DEC4C2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00A35B55, 0x00F8F3F3, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00AA6661, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00C49592,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00EBDCDB, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00C49592, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00AA6661, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00EBDCDB, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF,
        0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF,
        0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF,
        0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF,
        0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF,
        0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF,
        0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF,
        0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF,
        0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF,
        0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF,
        0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00E5D0CF, 0x00EBDCDB, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00D1ADAA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00DEC4C2, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00F2E8E7, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00D1ADAA, 0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00C49592,
        0x00F8F3F3, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00B77E7A, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2E8E7, 0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00DEC4C2, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x00B0726E, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00A35B55, 0x00F8F3F3, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00AA6661, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00C49592, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00EBDCDB, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x00DEC4C2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00EBDCDB, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00C49592, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00AA6661, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F8F3F3, 0x00A35B55, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x00CBA19E, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00D1ADAA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00DEC4C2, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00F2E8E7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00B77E7A, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x00B77E7A, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00B77E7A, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00F2E8E7, 0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00DEC4C2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00D1ADAA, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x00CBA19E, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00A35B55, 0x00F8F3F3, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00AA6661, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00C49592, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00E5D0CF, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x00F2E8E7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00EBDCDB, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00C49592, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00AA6661, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F8F3F3, 0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x00C49592, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x00D1ADAA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00DEC4C2, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00F2E8E7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00B0726E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00A35B55,
        0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E,
        0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E,
        0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E,
        0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E,
        0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E,
        0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E,
        0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E,
        0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E,
        0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E,
        0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B0726E, 0x00B77E7A, 0x00DEC4C2,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x00B77E7A, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2E8E7,
        0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00DEC4C2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00C49592, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00AA6661, 0x00F8F3F3,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x00A35B55, 0x00F8F3F3, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00AA6661,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00C49592, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00DEC4C2, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00EBDCDB, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x00EBDCDB, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00C49592, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00AA6661, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2E8E7, 0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00D8B9B6, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x00D1ADAA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00DEC4C2, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00F2E8E7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F8F3F3, 0x00AA6661, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00BD8A86, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00B77E7A,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2E8E7, 0x009D4F49, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00DEC4C2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00BD8A86, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00AA6661, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x00A35B55, 0x00F8F3F3,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00AA6661, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00C49592, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00D8B9B6, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00F2E8E7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x00EBDCDB, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00C49592, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00AA6661, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00EBDCDB, 0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00DEC4C2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x00D1ADAA, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00DEC4C2, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00F2E8E7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00F8F3F3, 0x00A35B55, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00CBA19E, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x00B77E7A, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2E8E7, 0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00DEC4C2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00B77E7A, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00B0726E, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x00A35B55, 0x00F8F3F3, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00AA6661, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00C49592, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00D1ADAA, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00F8F3F3, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x00EBDCDB, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00C49592, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x00AA6661, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00E5D0CF,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00EBDCDB, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2F2F2, 0x00CACACA, 0x00E4E4E4, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00D7D7D7, 0x00E4E4E4,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00E4E4E4, 0x00D7D7D7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00DEC4C2, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x009D4F49, 0x00F2E8E7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F8F3F3, 0x009D4F49,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00D1ADAA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00E4E4E4, 0x00A1A1A1, 0x006C6C6C, 0x00515151, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00444444, 0x006C6C6C, 0x00A1A1A1, 0x00E4E4E4,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2F2F2, 0x00CACACA, 0x00AFAFAF, 0x00949494, 0x00949494, 0x00949494,
        0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00AFAFAF, 0x00CACACA, 0x00F2F2F2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00BCBCBC, 0x00878787, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00444444, 0x006C6C6C, 0x00949494, 0x00D7D7D7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00949494, 0x005F5F5F, 0x00363636, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00363636,
        0x005F5F5F, 0x00949494, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00D7D7D7, 0x00CACACA, 0x00A1A1A1, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494,
        0x00CACACA, 0x00D7D7D7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00E4E4E4, 0x00CACACA, 0x00AFAFAF, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494,
        0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00CACACA, 0x00CACACA, 0x00D7D7D7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2E8E7, 0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x00DEC4C2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00B0726E, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00B77E7A, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00D7D7D7, 0x006C6C6C, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x005F5F5F, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00AFAFAF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2F2F2, 0x00949494, 0x00363636, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00444444, 0x00949494, 0x00F2F2F2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2F2F2, 0x00949494, 0x00444444, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00363636, 0x00878787, 0x00E4E4E4, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2F2F2, 0x00515151,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00515151, 0x00E4E4E4, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00797979, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x005F5F5F, 0x00A1A1A1, 0x00F2F2F2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00AA6661, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x00C49592, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00C49592, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00AA6661, 0x00F8F3F3, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00878787, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x006C6C6C, 0x00F2F2F2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00E4E4E4, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00A1A1A1, 0x00363636, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00363636, 0x00AFAFAF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00BCBCBC, 0x00363636, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00363636, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00797979, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00797979, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00AFAFAF, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00363636, 0x00BCBCBC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00C49592, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x00AA6661, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00DEC4C2, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00EBDCDB, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00797979, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00515151, 0x00F2F2F2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00A1A1A1, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2F2F2, 0x006C6C6C, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00797979, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00878787, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00515151, 0x00F2F2F2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00515151, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x005F5F5F, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00797979, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00AFAFAF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00DEC4C2, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49,
        0x00F2E8E7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2E8E7, 0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00D8B9B6, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00AFAFAF, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x006C6C6C, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00515151, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2F2F2, 0x00515151, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00797979, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00797979, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00515151, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00A1A1A1, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00D7D7D7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2E8E7, 0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00DEC4C2,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F8F3F3, 0x00AA6661, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00C49592, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00444444, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x006C6C6C, 0x00BCBCBC, 0x00F2F2F2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00D7D7D7, 0x00A1A1A1, 0x00444444, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00D7D7D7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00515151, 0x00797979, 0x00E4E4E4, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x006C6C6C, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00444444, 0x00878787, 0x00AFAFAF, 0x00CACACA, 0x00CACACA,
        0x00AFAFAF, 0x00878787, 0x00363636, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00878787, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00444444, 0x00797979, 0x00A1A1A1, 0x00CACACA, 0x00CACACA, 0x00BCBCBC, 0x00878787, 0x00444444,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x005F5F5F, 0x00AFAFAF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00797979, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00AA6661, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00C49592, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00BD8A86, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00AA6661, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00AFAFAF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00797979, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00AFAFAF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x005F5F5F,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00A1A1A1, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00444444, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00F2F2F2, 0x00949494, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00797979, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00363636, 0x00AFAFAF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00A1A1A1, 0x00363636, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x005F5F5F, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00BCBCBC, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00C49592, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00AA6661, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00D8B9B6, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00F2E8E7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00515151, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00797979, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x005F5F5F,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2F2F2, 0x00363636, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00515151, 0x00F2F2F2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00363636, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00878787, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00444444, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00515151, 0x00E4E4E4, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00D7D7D7, 0x00363636, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x005F5F5F, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00D7D7D7, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00DEC4C2, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00F2E8E7, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00EBDCDB, 0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00E5D0CF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00878787, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00878787,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00797979, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x005F5F5F,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00515151, 0x00F2F2F2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00D7D7D7, 0x00363636, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00BCBCBC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00AFAFAF, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00363636, 0x00E4E4E4, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00D7D7D7, 0x00363636, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00BCBCBC, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2E8E7, 0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00DEC4C2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F8F3F3, 0x00A35B55, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x00CBA19E, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00363636, 0x00D7D7D7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00E4E4E4, 0x00A1A1A1, 0x00949494, 0x00D7D7D7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x005F5F5F,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00444444, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00D7D7D7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00D7D7D7, 0x005F5F5F, 0x00292929, 0x00292929, 0x00363636, 0x00AFAFAF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x006C6C6C, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00AFAFAF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2F2F2, 0x00797979, 0x00292929, 0x00292929, 0x00292929, 0x00878787, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00AA6661, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00C49592, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00B77E7A, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x00B0726E, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00AFAFAF,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00363636, 0x00949494, 0x00F2F2F2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x005F5F5F,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00D7D7D7, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x006C6C6C, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2F2F2,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00444444, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00D7D7D7, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00D7D7D7, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00C49592, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00AA6661, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00D1ADAA, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x00A35B55, 0x00F8F3F3, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00E4E4E4,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x005F5F5F, 0x00949494, 0x00CACACA, 0x00F2F2F2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x005F5F5F,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00A1A1A1, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00AFAFAF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00363636, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00DEC4C2, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00F2E8E7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00E5D0CF, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x00EBDCDB, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00515151, 0x00797979, 0x00A1A1A1, 0x00D7D7D7, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00363636, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00878787, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00F2F2F2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00515151, 0x00D7D7D7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00797979, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00F2E8E7, 0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00DEC4C2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F8F3F3, 0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x00D1ADAA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00D7D7D7, 0x00363636, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00363636, 0x006C6C6C,
        0x00A1A1A1, 0x00E4E4E4, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00A1A1A1, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00F2F2F2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00363636, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x005F5F5F, 0x00949494, 0x00949494, 0x00949494,
        0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x005F5F5F, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00363636, 0x00F2F2F2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00AA6661, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00C49592, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00B0726E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00BD8A86,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00D7D7D7, 0x00444444, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x005F5F5F, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x005F5F5F, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00878787, 0x00363636, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x005F5F5F, 0x00BCBCBC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494, 0x00444444, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x005F5F5F, 0x00AFAFAF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00363636, 0x00D7D7D7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00C49592, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00AA6661, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00C49592, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00AA6661, 0x00F8F3F3,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00F2F2F2, 0x00949494, 0x00363636, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00878787, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00A1A1A1, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x005F5F5F, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00A1A1A1,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00E4E4E4, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00AFAFAF, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00363636, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00363636, 0x00949494, 0x00F2F2F2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00DEC4C2,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00F2E8E7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00DEC4C2, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00EBDCDB, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00BCBCBC, 0x00797979, 0x00444444, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00797979, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00363636, 0x00878787, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00444444, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00444444, 0x00BCBCBC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00797979, 0x00F2F2F2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2E8E7, 0x009D4F49,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00DEC4C2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2E8E7, 0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00D8B9B6, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00D7D7D7, 0x00AFAFAF, 0x00878787, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00AFAFAF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00515151,
        0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00E4E4E4, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00878787,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00D7D7D7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00AFAFAF, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00A1A1A1, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00363636, 0x00E4E4E4, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00AA6661, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00C49592, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F8F3F3, 0x00AA6661, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00C49592, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00949494, 0x00515151, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00515151, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x005F5F5F,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00949494, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00D7D7D7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2F2F2,
        0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00AFAFAF,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00AFAFAF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00797979, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00444444, 0x005F5F5F, 0x005F5F5F, 0x005F5F5F,
        0x005F5F5F, 0x005F5F5F, 0x005F5F5F, 0x005F5F5F, 0x005F5F5F, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x006C6C6C, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00C49592, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00AA6661, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00BD8A86, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00AA6661, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00E4E4E4, 0x00949494, 0x00363636, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x005F5F5F,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00AFAFAF, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00D7D7D7, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00797979, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00D7D7D7,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x006C6C6C, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00E4E4E4, 0x00CACACA,
        0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00CACACA, 0x00949494, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2F2F2, 0x00A1A1A1, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00F2F2F2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00DEC4C2, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00F2E8E7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00D8B9B6, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00F2E8E7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00F2F2F2, 0x00878787, 0x005F5F5F, 0x005F5F5F, 0x00A1A1A1, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00E4E4E4, 0x00515151,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x005F5F5F,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00E4E4E4, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00363636, 0x00F2F2F2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00515151, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00363636, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00E4E4E4, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00797979, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2E8E7, 0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00DEC4C2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00EBDCDB, 0x009D4F49, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00E5D0CF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00515151, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00515151, 0x00F2F2F2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00E4E4E4,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x005F5F5F,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00797979, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x006C6C6C, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00E4E4E4, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00AFAFAF, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00AFAFAF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00B0726E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00C49592, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00F8F3F3, 0x00A35B55, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00CBA19E, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00BCBCBC,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00797979, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2F2F2,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00D7D7D7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x005F5F5F,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00AFAFAF, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00AFAFAF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2F2F2, 0x00363636, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00E4E4E4, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00A1A1A1, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00F2F2F2, 0x00515151, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00DEC4C2, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00AA6661, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00D1ADAA, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00B0726E, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00A1A1A1,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00797979, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00A1A1A1,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00363636, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x005F5F5F,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00A1A1A1, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00D7D7D7, 0x00444444, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00797979, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00878787, 0x00F2F2F2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00E4E4E4, 0x00515151, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x009D4F49, 0x00F2E8E7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00B77E7A, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00A35B55, 0x00F8F3F3, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00D7D7D7,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x006C6C6C, 0x00E4E4E4, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2F2F2, 0x00949494, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00797979, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00444444,
        0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00D7D7D7, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00D7D7D7, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00515151, 0x00AFAFAF, 0x00F2F2F2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00D7D7D7, 0x00797979, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00F2F2F2, 0x00363636, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00444444, 0x00A1A1A1, 0x00E4E4E4, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00E4E4E4,
        0x00949494, 0x00363636, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00797979, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494,
        0x00949494, 0x00949494, 0x00AFAFAF, 0x00F2F2F2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00878787, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CBA19E, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00DEC4C2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00B77E7A, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00EBDCDB, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00515151, 0x00878787, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00AFAFAF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00AFAFAF, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00515151, 0x005F5F5F, 0x005F5F5F,
        0x005F5F5F, 0x00363636, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00444444, 0x005F5F5F, 0x005F5F5F, 0x005F5F5F, 0x00444444, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00363636, 0x00E4E4E4, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00444444, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00515151, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00DEC4C2, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x0096433D, 0x00C49592, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00D1ADAA, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00D1ADAA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00D7D7D7, 0x00363636, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00878787, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00363636, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00AFAFAF, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00AFAFAF, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00AA6661, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x0096433D, 0x00B0726E, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00F8F3F3, 0x00A35B55, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00BD8A86, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00D7D7D7, 0x00444444, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00797979, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00AFAFAF, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00AFAFAF, 0x00363636, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00515151, 0x00AFAFAF, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00D7D7D7, 0x00363636, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00515151, 0x00BCBCBC, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F8F3F3, 0x00B0726E, 0x0096433D, 0x0096433D, 0x0096433D,
        0x0096433D, 0x00AA6661, 0x00F8F3F3, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00EBDCDB, 0x00AA6661, 0x0096433D, 0x0096433D, 0x0096433D, 0x0096433D, 0x00B0726E, 0x00F8F3F3, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00F2F2F2, 0x00878787, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00363636, 0x00AFAFAF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2F2F2, 0x00363636, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x005F5F5F, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00E4E4E4, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00878787, 0x00FFFFFF, 0x00F2F2F2, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2F2F2, 0x00797979, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x005F5F5F, 0x00F2F2F2, 0x00FFFFFF, 0x00363636, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00A1A1A1, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00BCBCBC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00878787, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00AFAFAF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00D7D7D7, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00E4E4E4, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00E5D0CF, 0x00CBA19E, 0x00CBA19E,
        0x00DEC4C2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00DEC4C2, 0x00CBA19E, 0x00CBA19E, 0x00E5D0CF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00E4E4E4, 0x00949494, 0x00444444, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00444444, 0x00949494, 0x00F2F2F2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00D7D7D7, 0x006C6C6C, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00363636, 0x00797979, 0x00F2F2F2, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00797979, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x005F5F5F, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00E4E4E4, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00E4E4E4, 0x00797979, 0x00363636, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00444444, 0x00BCBCBC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494,
        0x00444444, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00444444, 0x00A1A1A1, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2F2F2, 0x00515151, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x005F5F5F, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00878787, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00AFAFAF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00D7D7D7, 0x00A1A1A1, 0x00878787, 0x005F5F5F, 0x005F5F5F, 0x005F5F5F, 0x005F5F5F, 0x005F5F5F, 0x005F5F5F, 0x005F5F5F, 0x00878787, 0x00A1A1A1, 0x00D7D7D7,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2F2F2, 0x00BCBCBC, 0x00949494, 0x006C6C6C, 0x005F5F5F, 0x005F5F5F, 0x005F5F5F, 0x005F5F5F,
        0x005F5F5F, 0x00797979, 0x00A1A1A1, 0x00E4E4E4, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2F2F2, 0x00797979, 0x005F5F5F, 0x005F5F5F, 0x00BCBCBC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00949494, 0x00797979, 0x005F5F5F, 0x005F5F5F, 0x005F5F5F, 0x005F5F5F, 0x005F5F5F, 0x006C6C6C, 0x00A1A1A1,
        0x00D7D7D7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494, 0x005F5F5F, 0x005F5F5F, 0x00AFAFAF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00A1A1A1, 0x00A1A1A1, 0x00D7D7D7, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00E4E4E4, 0x00AFAFAF, 0x00949494, 0x00AFAFAF, 0x00E4E4E4, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00CACACA, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00CACACA, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00BCBCBC, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00BCBCBC, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00949494, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00949494, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x006C6C6C, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x006C6C6C, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0x30292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00E4E4E4, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00E4E4E4, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x30292929,
  0x70292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00515151, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00515151, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x70292929,
  0xDF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x005F5F5F, 0x00F2F2F2, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF,
        0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00FFFFFF, 0x00F2F2F2, 0x005F5F5F, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xDF292929,
  0xFF000000, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00363636, 0x00797979, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494,
        0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494,
        0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494,
        0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494,
        0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494,
        0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494,
        0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494,
        0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494,
        0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494,
        0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494,
        0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494,
        0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494,
        0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494,
        0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494,
        0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00949494, 0x00797979, 0x00363636, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x40292929, 0xFF000000,
  0xFF000000, 0xDF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xDF292929, 0xFF000000,
  0xFF000000, 0xFF000000, 0x8F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x9F292929, 0xFF000000, 0xFF000000,
  0xFF000000, 0xFF000000, 0xFF000000, 0x60292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x60292929, 0xFF000000, 0xFF000000, 0xFF000000,
  0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
  0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xBF292929, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x40292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
  0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xBF292929, 0x60292929, 0x20292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x20292929, 0x60292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
  0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
  0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
  0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
  0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
  0xFF000000, 0xFF000000, 0xEF292929, 0xBF292929, 0xEF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xDF292929, 0xBF292929, 0xDF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xDF292929, 0xBF292929, 0xEF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xEF292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xCF292929, 0xBF292929, 0xEF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xCF292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xDF292929,
        0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xBF292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xBF292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xCF292929, 0xAF292929, 0x7F292929, 0x7F292929, 0x7F292929, 0x8F292929, 0xBF292929, 0xEF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xEF292929, 0xBF292929, 0x7F292929, 0x7F292929, 0x7F292929, 0x8F292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xDF292929, 0xAF292929, 0x7F292929, 0x7F292929, 0x7F292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xEF292929, 0xBF292929, 0xDF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xDF292929, 0x9F292929, 0x7F292929, 0x7F292929, 0x7F292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xEF292929, 0xBF292929, 0xDF292929, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xBF292929, 0x8F292929, 0x7F292929, 0x7F292929, 0x7F292929, 0xBF292929, 0xDF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xBF292929, 0xCF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xBF292929, 0xCF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xCF292929, 0x9F292929, 0x7F292929, 0x7F292929, 0x7F292929, 0xAF292929, 0xCF292929, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xDF292929, 0xAF292929, 0x7F292929, 0x7F292929, 0x7F292929, 0x7F292929, 0xBF292929,
        0xEF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xCF292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xEF292929, 0xAF292929,
        0x7F292929, 0x7F292929, 0x8F292929, 0xCF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xBF292929, 0x7F292929, 0x7F292929, 0x7F292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
  0xFF000000, 0x9F292929, 0x10292929, 0x00292929, 0x00292929, 0x9F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xDF292929, 0x20292929, 0x00292929, 0x00292929, 0x00292929, 0x30292929, 0xEF292929, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0x8F292929, 0x00292929, 0x00292929, 0x00292929, 0x9F292929, 0xFF000000, 0xFF000000, 0x70292929, 0x00292929, 0x00292929, 0x10292929, 0xAF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xAF292929,
        0x10292929, 0x00292929, 0x00292929, 0x00292929, 0x60292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x60292929, 0x00292929, 0x00292929, 0x10292929, 0xCF292929, 0xFF000000, 0xFF000000, 0x40292929, 0x00292929,
        0x00292929, 0x20292929, 0xCF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x9F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x9F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xEF292929,
        0x30292929, 0x00292929, 0x00292929, 0x30292929, 0xEF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x8F292929, 0x20292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x50292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xCF292929,
        0x50292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x60292929, 0xDF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xCF292929, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x10292929, 0x7F292929, 0xFF000000, 0x9F292929, 0x10292929, 0x00292929, 0x00292929, 0x60292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xCF292929, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x10292929, 0x8F292929, 0xFF000000, 0x9F292929, 0x00292929, 0x00292929, 0x00292929, 0x60292929, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xEF292929, 0x7F292929, 0x10292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x40292929, 0xAF292929, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xCF292929, 0x10292929, 0x00292929, 0x00292929, 0x30292929, 0xEF292929, 0xFF000000, 0x8F292929, 0x10292929, 0x00292929, 0x00292929, 0x20292929, 0xCF292929, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x20292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x20292929,
        0x9F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x9F292929, 0x20292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x50292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xEF292929, 0x40292929, 0x00292929, 0x00292929, 0x10292929, 0xCF292929, 0xEF292929, 0x70292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x20292929, 0xBF292929, 0xFF000000, 0xFF000000, 0x7F292929, 0x10292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x10292929, 0x7F292929, 0xFF000000, 0xFF000000,
  0xFF000000, 0x10292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xDF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x60292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xDF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x20292929, 0xFF000000, 0xCF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x20292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x20292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xAF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x9F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x50292929, 0xFF000000, 0x9F292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x50292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xEF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xDF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x70292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x8F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xEF292929, 0x30292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x60292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x8F292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x9F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x9F292929,
        0x10292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x60292929, 0x10292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xCF292929, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0x9F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x60292929, 0x10292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xCF292929,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xCF292929, 0x10292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x50292929, 0xEF292929,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x50292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xAF292929, 0x70292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x50292929, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xDF292929, 0x30292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x30292929, 0xDF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xCF292929, 0x30292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x60292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x9F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x50292929, 0x30292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x9F292929, 0x60292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x60292929, 0xFF000000,
  0xFF000000, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x8F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x10292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x30292929, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0x8F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x10292929, 0xFF000000, 0xBF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xCF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xDF292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x70292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x60292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x40292929, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x9F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x9F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x20292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x40292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x9F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x8F292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x9F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xCF292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xBF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xCF292929, 0x10292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x30292929,
        0xEF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x40292929, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xEF292929, 0x20292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x20292929, 0xEF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xCF292929, 0x10292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x40292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x9F292929,
  0xFF000000, 0x20292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x50292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xDF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xEF292929, 0xFF000000, 0xFF000000,
        0xFF000000, 0x50292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x50292929, 0xFF000000, 0xDF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x8F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x9F292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x30292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x20292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xAF292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x70292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x60292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xDF292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xAF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xAF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x10292929, 0x60292929, 0x7F292929, 0x60292929, 0x20292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xCF292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x20292929, 0x20292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x20292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000,
        0xEF292929, 0x20292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x30292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x10292929, 0x30292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x70292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x60292929, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x50292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x10292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x30292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x10292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x20292929,
  0xFF000000, 0x60292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x10292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x9F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929, 0xFF000000, 0xFF000000,
        0xFF000000, 0x10292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x9F292929, 0xFF000000, 0xFF000000, 0x30292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x40292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x70292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xEF292929, 0xFF000000, 0xFF000000, 0xDF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xCF292929, 0xFF000000, 0xFF000000, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x20292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x9F292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x10292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x70292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x10292929, 0xEF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x8F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x9F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x50292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x50292929, 0xDF292929, 0xFF000000, 0xFF000000, 0xDF292929, 0x60292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x40292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x9F292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xDF292929, 0xFF000000, 0xFF000000, 0x9F292929, 0x10292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000,
        0x9F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xEF292929, 0xFF000000, 0xEF292929, 0x9F292929, 0x10292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929,
        0xFF000000, 0xFF000000, 0xFF000000, 0x9F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x20292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x8F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0xDF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x30292929, 0x60292929, 0xEF292929, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xBF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x8F292929, 0xEF292929, 0xFF000000, 0xEF292929, 0x7F292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x40292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x8F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xEF292929, 0xFF000000, 0xFF000000, 0xDF292929, 0x50292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xDF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xDF292929, 0xFF000000,
        0xAF292929, 0x30292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x30292929, 0xBF292929, 0xFF000000, 0xEF292929, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0xFF000000, 0xAF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xCF292929, 0xFF000000, 0xFF000000, 0x70292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xFF000000,
        0xCF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xEF292929, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x30292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xAF292929, 0xFF000000, 0xFF000000, 0x9F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x20292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x50292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x40292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xEF292929, 0xFF000000, 0xFF000000, 0x60292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x60292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x10292929, 0xEF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xDF292929, 0x70292929, 0x40292929, 0x9F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xDF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x30292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xDF292929, 0xFF000000, 0xFF000000, 0x50292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xCF292929, 0x10292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000,
        0x30292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x8F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xCF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929,
        0xFF000000, 0xFF000000, 0xFF000000, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xCF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x8F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x8F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xEF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x60292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x9F292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x60292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x20292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x9F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xBF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x10292929, 0xEF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x60292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0xFF000000, 0xFF000000, 0x10292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x8F292929, 0xFF000000, 0xFF000000, 0x30292929, 0x00292929, 0x00292929, 0x00292929, 0x30292929, 0x00292929, 0x00292929, 0x00292929, 0x40292929, 0xFF000000, 0xFF000000,
        0x8F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x30292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xCF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929, 0xFF000000, 0xFF000000, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x30292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xFF000000, 0x60292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x70292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x9F292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xFF000000, 0xFF000000, 0xBF292929, 0x00292929, 0x00292929, 0x00292929, 0x20292929, 0x10292929, 0x00292929, 0x00292929, 0x00292929, 0xAF292929, 0xFF000000, 0xFF000000, 0x20292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x9F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x60292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x10292929, 0x60292929, 0x9F292929, 0xCF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xAF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x9F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x9F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x9F292929, 0xFF000000, 0xFF000000, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x10292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x70292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x20292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x60292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929,
        0xFF000000, 0xFF000000, 0xFF000000, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x40292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xEF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x40292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x8F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x20292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xEF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xAF292929,
        0x30292929, 0x10292929, 0x60292929, 0xEF292929, 0xFF000000, 0xFF000000, 0xDF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x30292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xDF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x40292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xEF292929, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x70292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0xFF000000, 0xFF000000, 0x50292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x50292929, 0xFF000000, 0xFF000000, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x9F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xFF000000, 0xFF000000,
        0x50292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x8F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x20292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xBF292929, 0x00292929, 0x00292929,
        0x00292929, 0x30292929, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x40292929, 0xFF000000, 0xFF000000, 0x10292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xAF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xDF292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xAF292929, 0xFF000000, 0x8F292929, 0x00292929, 0x00292929, 0x00292929, 0x60292929, 0x50292929, 0x00292929, 0x00292929, 0x00292929, 0x70292929, 0xFF000000, 0xDF292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0xEF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x9F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x40292929, 0x7F292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x30292929, 0x40292929, 0x40292929, 0x40292929, 0x40292929, 0x40292929, 0x40292929, 0x30292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xDF292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x60292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xBF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xCF292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x70292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xAF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929,
        0xFF000000, 0xFF000000, 0xCF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x20292929, 0x40292929, 0x40292929, 0x40292929, 0x40292929, 0x40292929, 0x40292929, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x40292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x30292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xBF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0x30292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0xFF000000, 0xFF000000, 0x9F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x10292929, 0xFF000000, 0xBF292929, 0x00292929, 0x00292929, 0x00292929, 0x30292929, 0xEF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xCF292929, 0xFF000000,
        0x10292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xDF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x70292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x40292929, 0xFF000000, 0x8F292929, 0x00292929, 0x00292929,
        0x00292929, 0x60292929, 0xBF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xFF000000, 0xCF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x10292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x30292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x70292929, 0xFF000000, 0x50292929, 0x00292929, 0x00292929, 0x00292929, 0x9F292929, 0x8F292929, 0x00292929, 0x00292929, 0x00292929, 0x40292929, 0xFF000000, 0x9F292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x50292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x30292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x20292929, 0x7F292929, 0xEF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xBF292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xEF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xBF292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xBF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929,
        0xFF000000, 0xFF000000, 0xBF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x40292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x40292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xDF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x40292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0xFF000000, 0xFF000000, 0xEF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xCF292929, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x70292929, 0xFF000000, 0x30292929, 0x00292929, 0x00292929, 0x00292929, 0x8F292929, 0xCF292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x20292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xAF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xFF000000, 0x50292929, 0x00292929, 0x00292929,
        0x00292929, 0x9F292929, 0xFF000000, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929, 0x8F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x60292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x8F292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x30292929, 0xFF000000, 0x20292929, 0x00292929, 0x00292929, 0x00292929, 0xDF292929, 0xCF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xFF000000, 0x60292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x8F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xEF292929, 0x60292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x30292929, 0xEF292929, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xBF292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xEF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xBF292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xBF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929,
        0xFF000000, 0xFF000000, 0xBF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x40292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x40292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xBF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x40292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0x50292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0xFF000000, 0xFF000000, 0xFF000000, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0x50292929, 0x00292929, 0x00292929, 0x00292929, 0xAF292929, 0xFF000000, 0x70292929, 0x00292929, 0x00292929, 0x00292929, 0x50292929, 0x7F292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x70292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x10292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929, 0x20292929, 0x00292929, 0x00292929,
        0x00292929, 0xDF292929, 0xFF000000, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x8F292929, 0x50292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x9F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xCF292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xDF292929, 0x00292929, 0x00292929, 0x00292929, 0x10292929, 0xFF000000, 0xFF000000, 0x10292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929, 0x20292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0xDF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xDF292929, 0x70292929,
        0x20292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x60292929, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x30292929, 0x40292929, 0x40292929, 0x40292929, 0x40292929, 0x40292929, 0x40292929, 0x40292929, 0x40292929, 0x40292929, 0x40292929, 0x40292929, 0x60292929, 0xEF292929, 0xFF000000, 0xCF292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x70292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xBF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xBF292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xBF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929,
        0xFF000000, 0xFF000000, 0xBF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x20292929, 0x40292929, 0x40292929, 0x40292929, 0x40292929, 0x40292929, 0x40292929, 0x40292929, 0x40292929, 0x40292929, 0x40292929, 0x40292929,
        0x40292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x40292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xEF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x40292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x8F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0xFF000000, 0xFF000000, 0xFF000000, 0x8F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x40292929, 0x20292929, 0x00292929, 0x00292929, 0x00292929, 0xDF292929, 0xFF000000, 0xAF292929, 0x00292929, 0x00292929, 0x00292929, 0x20292929, 0x40292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xCF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x60292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x50292929, 0x00292929, 0x00292929, 0x00292929,
        0x20292929, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x50292929, 0x10292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xEF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0x20292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x60292929, 0x00292929, 0x00292929, 0x00292929, 0x50292929, 0xFF000000, 0xFF000000, 0x50292929, 0x00292929, 0x00292929, 0x00292929, 0x60292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x30292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xCF292929, 0x9F292929, 0x60292929, 0x30292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xEF292929, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x30292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x8F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xEF292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x30292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x70292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929,
        0xFF000000, 0xFF000000, 0xDF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x60292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x40292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x20292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xEF292929,
        0x8F292929, 0x7F292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xBF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0x20292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x10292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0xFF000000, 0xFF000000, 0xFF000000, 0xDF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x20292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x10292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x9F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x50292929, 0xFF000000, 0xFF000000, 0xBF292929, 0x00292929, 0x00292929, 0x00292929, 0x10292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x50292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0x70292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x8F292929, 0xFF000000, 0xFF000000, 0x8F292929, 0x00292929, 0x00292929, 0x00292929, 0x10292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x7F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x9F292929, 0x50292929, 0x70292929, 0xDF292929,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xEF292929, 0x7F292929, 0x10292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929, 0xFF000000, 0xBF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x70292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xAF292929, 0x9F292929, 0xDF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x30292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xEF292929, 0x20292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000,
        0x20292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xEF292929, 0x10292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929,
        0xFF000000, 0xFF000000, 0xFF000000, 0x10292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x20292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xCF292929, 0x7F292929, 0xCF292929,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x40292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x30292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xEF292929, 0x30292929,
        0x00292929, 0x00292929, 0x00292929, 0xAF292929, 0xFF000000, 0xFF000000, 0xEF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x20292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xBF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x40292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x30292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x60292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x60292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xEF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x8F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x8F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xBF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xCF292929, 0xFF000000, 0xFF000000, 0xCF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0xCF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x10292929, 0x00292929, 0x30292929, 0xAF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x8F292929, 0x00292929, 0x00292929, 0x00292929, 0x10292929,
        0x9F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x70292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xCF292929, 0xFF000000, 0xFF000000, 0x10292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x10292929, 0xCF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xCF292929, 0x30292929, 0x00292929, 0x00292929, 0x10292929, 0xDF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x10292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xEF292929, 0x50292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000,
        0x70292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x10292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xDF292929, 0x30292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929,
        0xFF000000, 0xFF000000, 0xFF000000, 0x60292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x8F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xEF292929, 0x60292929, 0x00292929, 0x00292929, 0x00292929,
        0x8F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x40292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x10292929, 0x00292929,
        0x30292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x70292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x30292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x30292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x40292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x60292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xEF292929,
        0x30292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x8F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x70292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x9F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0xAF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x50292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0xCF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xDF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0x10292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x10292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x20292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x70292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xCF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x8F292929, 0xEF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xDF292929, 0x20292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x10292929, 0x9F292929, 0xDF292929, 0xFF000000, 0xDF292929, 0x7F292929, 0x10292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x8F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xEF292929, 0x10292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x20292929, 0x40292929, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000,
        0xDF292929, 0x10292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x30292929, 0x40292929, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929,
        0xFF000000, 0xFF000000, 0xFF000000, 0xBF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x70292929, 0xCF292929, 0xFF000000, 0xEF292929, 0x9F292929, 0x30292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x40292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x40292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x70292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0xDF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xDF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x30292929, 0x9F292929, 0xBF292929, 0xAF292929, 0x30292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x50292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x9F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x50292929, 0xBF292929, 0xFF000000, 0xDF292929, 0x9F292929, 0x20292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x10292929, 0xEF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xCF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xCF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xBF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x8F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x10292929,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x30292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0x60292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x40292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x50292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x70292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x10292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0x60292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x40292929, 0x40292929, 0x20292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x60292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xEF292929, 0x10292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xCF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x9F292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0x8F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x70292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x70292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x40292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x10292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x70292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x9F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x20292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x10292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x50292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xDF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x40292929,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xBF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x8F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xAF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x8F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0xAF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x10292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xDF292929, 0x10292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x10292929, 0xDF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xCF292929, 0x10292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x60292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x30292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0x60292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x10292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xEF292929, 0x30292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x30292929,
        0xEF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x40292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x10292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xEF292929, 0x30292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xEF292929, 0x30292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x60292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x7F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x8F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
  0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x60292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x60292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0xAF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x9F292929,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x20292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xDF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0x10292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xCF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xDF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x20292929,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x70292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xDF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xCF292929, 0x20292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x30292929, 0xCF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xCF292929, 0x20292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x10292929, 0x8F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xAF292929, 0x20292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x40292929, 0xEF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xAF292929, 0x20292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x40292929, 0xAF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xEF292929, 0x50292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x50292929, 0xEF292929,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x60292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x60292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x70292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0xDF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xEF292929, 0x50292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x10292929, 0x9F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xEF292929, 0x50292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x00292929, 0x00292929, 0x10292929, 0x9F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x9F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x30292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0x30292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xAF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xCF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x10292929,
  0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xEF292929, 0x60292929, 0x10292929, 0x00292929, 0x00292929, 0x50292929, 0xEF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xEF292929, 0x50292929, 0x00292929, 0x00292929,
        0x10292929, 0x70292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xCF292929, 0x50292929, 0x00292929, 0x00292929, 0x10292929, 0x70292929, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xCF292929, 0x30292929, 0x00292929, 0x00292929, 0x20292929, 0x9F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xBF292929, 0x30292929, 0x00292929, 0x00292929, 0x20292929, 0x8F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x9F292929, 0x20292929, 0x00292929, 0x00292929, 0x30292929, 0xCF292929,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x70292929, 0x10292929, 0x00292929, 0x30292929, 0xAF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x9F292929, 0x40292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x10292929, 0x50292929, 0x9F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xEF292929,
        0x8F292929, 0x30292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x30292929, 0x7F292929, 0xDF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xBF292929, 0x7F292929, 0x60292929, 0x40292929, 0x60292929, 0x7F292929, 0xCF292929, 0xFF000000, 0xCF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xAF292929, 0x7F292929, 0x50292929, 0x40292929, 0x60292929, 0x7F292929, 0xCF292929, 0xFF000000, 0xBF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xBF292929, 0x50292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x10292929, 0x60292929, 0xBF292929, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xEF292929, 0x50292929, 0x00292929, 0x00292929, 0x50292929, 0xEF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x7F292929, 0x10292929, 0x00292929,
        0x30292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xBF292929, 0x50292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x20292929, 0x7F292929,
        0xEF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xBF292929, 0x60292929, 0x10292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929,
        0x20292929, 0x7F292929, 0xEF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x70292929, 0x10292929, 0x00292929, 0x30292929, 0xCF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xCF292929, 0x30292929, 0x00292929, 0x10292929, 0x70292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x9F292929, 0x20292929, 0x00292929, 0x30292929, 0xBF292929,
  0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xCF292929, 0xBF292929, 0xBF292929, 0xBF292929, 0xBF292929, 0xDF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xEF292929, 0xBF292929, 0xBF292929, 0xBF292929, 0xBF292929, 0xEF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xDF292929,
        0xBF292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xAF292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xDF292929, 0xBF292929, 0xBF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x9F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xCF292929,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xCF292929, 0xBF292929, 0xBF292929, 0xBF292929, 0xDF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xBF292929, 0xBF292929, 0xBF292929, 0xBF292929, 0xDF292929, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xDF292929, 0xBF292929, 0xBF292929, 0xBF292929, 0xBF292929, 0xEF292929,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
  0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xAF292929, 0x00292929,
        0x00292929, 0x00292929, 0x30292929, 0xCF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x50292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0x9F292929, 0x00292929, 0x00292929, 0x00292929, 0x30292929, 0xCF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
  0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x40292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x70292929, 0xCF292929, 0xFF000000, 0xFF000000, 0xBF292929, 0x50292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x30292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x10292929, 0x7F292929, 0xDF292929, 0xFF000000, 0xFF000000, 0xBF292929, 0x50292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x50292929, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
  0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x40292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x9F292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0xBF292929, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
  0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xAF292929, 0x00292929,
        0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x60292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0x8F292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x60292929, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
  0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0x9F292929,
        0x10292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x70292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0x8F292929, 0x10292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x70292929, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
  0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xEF292929, 0x8F292929, 0x40292929, 0x10292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x10292929, 0x40292929, 0x7F292929, 0xDF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xEF292929, 0x8F292929, 0x40292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x00292929, 0x10292929, 0x40292929, 0x8F292929, 0xDF292929, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000,
        0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000, 0xFF000000
};

static GUI_CONST_STORAGE GUI_BITMAP _bmSeggerLogo_300x181 = {
  300,                                     // XSize
  181,                                     // YSize
  1200,                                    // BytesPerLine
  32,                                      // BitsPerPixel
  (unsigned char *)_acSeggerLogo_300x181,  // Pointer to picture data
  NULL,                                    // Pointer to palette
  GUI_DRAW_BMP8888
};
#endif

/*********************************************************************
*
*       Types
*
**********************************************************************
*/
typedef struct {
  const GUI_BITMAP GUI_UNI_PTR * pBitmap;
  GUI_MEMDEV_Handle              hMem;
  int                            xSize;
  int                            ySize;
} IMAGE;

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/
static IMAGE _aImage[] = {
  { &_bmSeggerLogo_300x181 },
  { &_bmJTrace_300x164     },
  { &_bmJLink_300x132      },
  { &_bmPlatine_250x200    }
};

static int _NumImages = sizeof(_aImage) / sizeof(_aImage[0]);

/*********************************************************************
*
*       Static code
*
**********************************************************************
*/
/*********************************************************************
*
*       _GetImage
*/
static int _GetImage(IMAGE * pImage) {
  pImage->xSize = pImage->pBitmap->XSize;
  pImage->ySize = pImage->pBitmap->YSize;
  pImage->hMem  = GUI_MEMDEV_CreateFixed(0, 0, pImage->xSize, pImage->ySize, GUI_MEMDEV_NOTRANS, GUI_MEMDEV_APILIST_32, GUI_COLOR_CONV);
  if (pImage->hMem == 0) {
    return 1;
  }
  GUI_MEMDEV_Select(pImage->hMem);
  GUI_SetBkColor(GUI_TRANSPARENT);
  GUI_Clear();
  GUI_DrawBitmap(pImage->pBitmap, 0, 0);
  GUI_MEMDEV_Select(0);
  return 0;
}

/*********************************************************************
*
*       _CopyFromLCD
*/
static void _CopyFromLCD(GUI_MEMDEV_Handle hMem) {
#if GUIDEMO_SUPPORT_CURSOR
  GUIDEMO_HideCursor();
#endif
  GUI_MEMDEV_CopyFromLCD(hMem);
#if GUIDEMO_SUPPORT_CURSOR
  GUIDEMO_ShowCursor();
#endif
}

/*********************************************************************
*
*       _RotateAndZoomImage
*/
static int _RotateAndZoomImage(void) {
  static int           Index;
  GUI_MEMDEV_Handle    hLCD;
  GUI_MEMDEV_Handle    hDst;
  IMAGE              * pImage;
  void              (* pfRotate)(GUI_MEMDEV_Handle hSrc, GUI_MEMDEV_Handle hDst, int dx, int dy, int a, int Mag);
  int                  TimeContinue;
  int                  TimeNow;
  int                  xSize;
  int                  ySize;
  int                  Mag;
  int                  a;
  int                  i;

  pfRotate = GUI_MEMDEV_Rotate;
  pImage   = &_aImage[Index];
  xSize    = LCD_GetXSize();
  ySize    = LCD_GetYSize();
  //
  // Create memory devices if no exist
  //
  hLCD = GUI_MEMDEV_CreateFixed(0, 0, xSize, ySize, GUI_MEMDEV_NOTRANS, GUI_MEMDEV_APILIST_32, GUI_COLOR_CONV);
  if (hLCD == 0) {
    return 1;
  }
  _CopyFromLCD(hLCD);
  hDst = GUI_MEMDEV_CreateFixed(0, 0, xSize, ySize, GUI_MEMDEV_NOTRANS, GUI_MEMDEV_APILIST_32, GUI_COLOR_CONV);
  if (hDst == 0) {
    GUI_MEMDEV_Delete(hLCD);
    return 1;
  }
  ySize -= TITLE_SIZE;
  //
  // Rotate and enlarge while moving in.
  //
  for (i = 2; (i <= 100); i += 2) {
    TimeContinue = GUIDEMO_GetTime() + MIN_TIME_PER_FRAME_ENLARGE;
    GUI_MEMDEV_Select(hDst);
    GUI_MEMDEV_Write(hLCD);
    pfRotate(pImage->hMem, hDst, (xSize - pImage->xSize) / 2 - (xSize * (100 - i)) / 250, TITLE_SIZE + (ySize - pImage->ySize) / 2, (i - 100) * 450, i * 10);
    GUI_MEMDEV_Select(0);
    GUI_MULTIBUF_Begin();
    GUI_MEMDEV_CopyToLCD(hDst);
    GUI_MULTIBUF_End();
    //
    // Delay
    //
    TimeNow = GUIDEMO_GetTime();
    if (TimeContinue > TimeNow) {
      GUI_Delay(TimeContinue - TimeNow);
    } else {
      GUI_Exec();
    }
    if (GUIDEMO_CheckCancel()) {
      GUI_MEMDEV_Delete(hLCD);
      GUI_MEMDEV_Delete(hDst);
      return 1;
    }
  }
  //
  // Rotate, enlarge and shrink on spot.
  //
  Mag = 1000;
  for (a = 5; a <= 360; a += 5) {
    TimeContinue = GUIDEMO_GetTime() + MIN_TIME_PER_FRAME_ENLARGE;
    //
    // Calculate magnification
    //
    if (a <= 180) {
      Mag = (Mag * 1050) / 1000;
    } else {
      Mag = (Mag * 1000) / 1050;
    }
    //
    // Do animation
    //
    GUI_MEMDEV_Select(hDst);
    GUI_MEMDEV_Write(hLCD);
    pfRotate(pImage->hMem, hDst, (xSize - pImage->xSize) / 2, TITLE_SIZE + (ySize - pImage->ySize) / 2, a * 1000, Mag);
    GUI_MEMDEV_Select(0);
    GUI_MULTIBUF_Begin();
    GUI_MEMDEV_CopyToLCD(hDst);
    GUI_MULTIBUF_End();
    //
    // Delay
    //
    TimeNow = GUIDEMO_GetTime();
    if (TimeContinue > TimeNow) {
      GUI_Delay(TimeContinue - TimeNow);
    } else {
      GUI_Exec();
    }
    if (GUIDEMO_CheckCancel()) {
      GUI_MEMDEV_Delete(hLCD);
      GUI_MEMDEV_Delete(hDst);
      return 1;
    }
  }
  //
  // Rotate complete screen
  //
  GUI_MEMDEV_CopyFromLCD(hLCD);
  GUIDEMO_ConfigureDemo(NULL, NULL, GUIDEMO_SHOW_CURSOR);
  WM_ValidateWindow(WM_HBKWIN);
  for (a = 5; a <= 360; a += 5) {
    TimeContinue = GUI_GetTime() + 30;
    //
    // Calculate magnification
    //
    Mag = (a < 180)
        ? ((180 - a) * 800) / 180 + 200
        : ((a - 180) * 800) / 180 + 200;
    //
    // Do animation
    //
    GUI_MEMDEV_Select(hDst);
    GUI_SetBkColor(GUI_MAKE_COLOR(0xa56e3a));
    GUI_Clear();
    GUI_MEMDEV_Select(0);
    pfRotate(hLCD, hDst, 0, 0, a * 1000, Mag);
    GUI_MULTIBUF_Begin();
    GUI_MEMDEV_CopyToLCD(hDst);
    GUI_MULTIBUF_End();
    //
    // Delay
    //
    TimeNow = GUI_GetTime();
    if (TimeContinue > TimeNow) {
      GUI_Delay(TimeContinue - TimeNow);
    }
  }
  GUIDEMO_ConfigureDemo(NULL, NULL, GUIDEMO_SHOW_CURSOR | GUIDEMO_SHOW_CONTROL);
  //
  // Clean up LCD context
  //
  GUI_MEMDEV_Select(hLCD);
  GUI_DrawGradientV(0, 40, xSize - 1, ySize + TITLE_SIZE - 1, GUI_WHITE, GUI_LIGHTBLUE);
  //
  // Rotate and enlarge while moving out.
  //
  for (i = 2; (i <= 100); i += 2) {
    TimeContinue = GUIDEMO_GetTime() + MIN_TIME_PER_FRAME_ENLARGE;
    GUI_MEMDEV_Select(hDst);
    GUI_MEMDEV_Write(hLCD);
    pfRotate(pImage->hMem, hDst, (xSize - pImage->xSize) / 2 + (xSize * i) / 250, TITLE_SIZE + (ySize - pImage->ySize) / 2, i * 450, (100 - i) * 10);
    GUI_MEMDEV_Select(0);
    GUI_MULTIBUF_Begin();
    GUI_MEMDEV_CopyToLCD(hDst);
    GUI_MULTIBUF_End();
    //
    // Delay
    //
    TimeNow = GUIDEMO_GetTime();
    if (TimeContinue > TimeNow) {
      GUI_Delay(TimeContinue - TimeNow);
    } else {
      GUI_Exec();
    }
    if (GUIDEMO_CheckCancel()) {
      GUI_MEMDEV_Delete(hLCD);
      GUI_MEMDEV_Delete(hDst);
      return 1;
    }
  }
  GUI_MULTIBUF_Begin();
  GUI_MEMDEV_CopyToLCD(hLCD);
  GUI_MULTIBUF_End();
  Index = (Index + 1) % _NumImages;
  //
  // Remove memory devices
  //
  GUI_MEMDEV_Delete(hLCD);
  GUI_MEMDEV_Delete(hDst);
  return 0;
}

/*********************************************************************
*
*       _ShiftImage
*/
static int _ShiftImage(int CleanUp) {
  static GUI_MEMDEV_Handle    hMemHQ[GUI_COUNTOF(_aImage)];
  static GUI_MEMDEV_Handle    hDstEnlarge;
  static GUI_MEMDEV_Handle    hLCDEnlarge;
  static GUI_MEMDEV_Handle    hDstShift;
  static GUI_MEMDEV_Handle    hLCD;
  static int                  Index;
  IMAGE                     * pImage;
  void                     (* pfRotate)(GUI_MEMDEV_Handle hSrc, GUI_MEMDEV_Handle hDst, int dx, int dy, int a, int Mag);
  int                         xSizeImageMag;
  int                         ySizeImageMag;
  int                         TimeContinue;
  int                         DispSpaceX;
  int                         DispSpaceY;
  int                         TimeNow;
  int                         Index0;
  int                         Index1;
  int                         Index2;
  int                         Index3;
  int                         xSize;
  int                         ySize;
  int                         Step;
  int                         Add;
  int                         Mag;
  int                         a;
  int                         i;

  xSize      = LCD_GetXSize();
  ySize      = LCD_GetYSize() - TITLE_SIZE;
  DispSpaceX = xSize / 3;
  DispSpaceY = (ySize * 3) / 5;
  //
  // Create memory devices if not existing
  //
  if (hLCD == 0) {
    hLCD        = GUI_MEMDEV_CreateFixed(0,         TITLE_SIZE + ySize / 5, xSize,           DispSpaceY, GUI_MEMDEV_NOTRANS, GUI_MEMDEV_APILIST_32, GUI_COLOR_CONV);
    if (hLCD == 0) {
      return 1;
    }
    _CopyFromLCD(hLCD);
  }
  if (hDstShift == 0) {
    hDstShift   = GUI_MEMDEV_CreateFixed(0,         TITLE_SIZE + ySize / 5, xSize,           DispSpaceY, GUI_MEMDEV_NOTRANS, GUI_MEMDEV_APILIST_32, GUI_COLOR_CONV);
    if (hDstShift == 0) {
      GUI_MEMDEV_Delete(hLCD);
      return 1;
    }
  }
  if (hDstEnlarge == 0) {
    hDstEnlarge = GUI_MEMDEV_CreateFixed(xSize / 6, TITLE_SIZE,             (xSize * 2) / 3, ySize,      GUI_MEMDEV_NOTRANS, GUI_MEMDEV_APILIST_32, GUI_COLOR_CONV);
    if (hDstEnlarge == 0) {
      GUI_MEMDEV_Delete(hLCD);
      GUI_MEMDEV_Delete(hDstShift);
      return 1;
    }
  }
  if (hLCDEnlarge == 0) {
    hLCDEnlarge = GUI_MEMDEV_CreateFixed(xSize / 6, TITLE_SIZE,             (xSize * 2) / 3, ySize,      GUI_MEMDEV_NOTRANS, GUI_MEMDEV_APILIST_32, GUI_COLOR_CONV);
    if (hLCDEnlarge == 0) {
      GUI_MEMDEV_Delete(hLCD);
      GUI_MEMDEV_Delete(hDstShift);
      GUI_MEMDEV_Delete(hDstEnlarge);
      return 1;
    }
  }
  GUI_MEMDEV_Select(0);
  for (i = 0; i < _NumImages; i++) {
    if (hMemHQ[i] == 0) {
      pImage        = &_aImage[i];
      xSizeImageMag = pImage->xSize * MAG / 1000;
      ySizeImageMag = pImage->ySize * MAG / 1000;
      hMemHQ[i]     = GUI_MEMDEV_CreateFixed((DispSpaceX - xSizeImageMag) / 2 - BORDER_SIZE, TITLE_SIZE + ySize / 5 + (DispSpaceY - ySizeImageMag) / 2 - BORDER_SIZE, xSizeImageMag + BORDER_SIZE * 2, ySizeImageMag + BORDER_SIZE * 2, GUI_MEMDEV_NOTRANS, GUI_MEMDEV_APILIST_32, GUI_COLOR_CONV);
      _CopyFromLCD(hMemHQ[i]);
      GUI_MEMDEV_RotateHQ(pImage->hMem, hMemHQ[i], (xSizeImageMag - pImage->xSize) / 2 + BORDER_SIZE, (ySizeImageMag - pImage->ySize) / 2 + BORDER_SIZE, 0, MAG);
    }
  }
  //
  // Get image indices
  //
  Index0 = (Index + 0) % _NumImages;
  Index1 = (Index + 1) % _NumImages;
  Index2 = (Index + 2) % _NumImages;
  Index3 = (Index + 3) % _NumImages;
  //
  // Perform shifting
  //
  pfRotate = GUI_MEMDEV_Rotate;
  for (i = 1; i <= FINAL_STEP; i++) {
    TimeContinue = GUIDEMO_GetTime() + MIN_TIME_PER_FRAME_SHIFT;
    Step         = (DispSpaceX * i) / FINAL_STEP;
    //
    // Calculate rotation
    //
    a = (-Step * 360 * 1000) / DispSpaceX;
    //
    // Do animation
    //
    GUI_MEMDEV_Select(hDstShift);
    GUI_MEMDEV_Write(hLCD);
    if (Step == DispSpaceX) {
      pImage        = &_aImage[Index0];
      ySizeImageMag = pImage->ySize * MAG / 1000;
      xSizeImageMag = pImage->xSize * MAG / 1000;
      GUI_MEMDEV_WriteAt(hMemHQ[Index0], (DispSpaceX - xSizeImageMag) / 2            - BORDER_SIZE, TITLE_SIZE + ySize / 5 + (DispSpaceY - ySizeImageMag) / 2 - BORDER_SIZE);
      pImage        = &_aImage[Index1];
      ySizeImageMag = pImage->ySize * MAG / 1000;
      xSizeImageMag = pImage->xSize * MAG / 1000;
      GUI_MEMDEV_WriteAt(hMemHQ[Index1], (DispSpaceX - xSizeImageMag) / 2 + Step     - BORDER_SIZE, TITLE_SIZE + ySize / 5 + (DispSpaceY - ySizeImageMag) / 2 - BORDER_SIZE);
      pImage        = &_aImage[Index2];
      ySizeImageMag = pImage->ySize * MAG / 1000;
      xSizeImageMag = pImage->xSize * MAG / 1000;
      GUI_MEMDEV_WriteAt(hMemHQ[Index2], (DispSpaceX - xSizeImageMag) / 2 + Step * 2 - BORDER_SIZE, TITLE_SIZE + ySize / 5 + (DispSpaceY - ySizeImageMag) / 2 - BORDER_SIZE);
    } else {
      pImage        = &_aImage[Index0];
      pfRotate(pImage->hMem, hDstShift, ((xSize - pImage->xSize) / 2) - (DispSpaceX * 2) + Step, (DispSpaceY - pImage->ySize) / 2, a, MAG);
      pImage        = &_aImage[Index1];
      pfRotate(pImage->hMem, hDstShift, ((xSize - pImage->xSize) / 2) -  DispSpaceX      + Step, (DispSpaceY - pImage->ySize) / 2, a, MAG);
      pImage        = &_aImage[Index2];
      pfRotate(pImage->hMem, hDstShift, ((xSize - pImage->xSize) / 2)                    + Step, (DispSpaceY - pImage->ySize) / 2, a, MAG);
      pImage        = &_aImage[Index3];
      pfRotate(pImage->hMem, hDstShift, ((xSize - pImage->xSize) / 2) +  DispSpaceX      + Step, (DispSpaceY - pImage->ySize) / 2, a, MAG);
    }
    GUI_MULTIBUF_Begin();
    GUI_MEMDEV_CopyToLCD(hDstShift);
    GUI_MULTIBUF_End();
    //
    // Delay
    //
    TimeNow = GUIDEMO_GetTime();
    if (TimeContinue > TimeNow) {
      GUI_Delay(TimeContinue - TimeNow);
    } else {
      GUI_Exec();
    }
    if (GUIDEMO_CheckCancel()) {
      GUI_MEMDEV_Delete(hLCD);
      GUI_MEMDEV_Delete(hDstShift);
      GUI_MEMDEV_Delete(hDstEnlarge);
      GUI_MEMDEV_Delete(hLCDEnlarge);
      hLCD        = 0;
      hDstShift   = 0;
      hDstEnlarge = 0;
      hLCDEnlarge = 0;
      return 1;
    }
  }
  //
  // Enlarge & shrink
  //
  pImage = &_aImage[Index1];
  _CopyFromLCD(hLCDEnlarge);
  GUI_MEMDEV_Select(hLCDEnlarge);
  GUI_DrawGradientV(DispSpaceX, TITLE_SIZE, DispSpaceX * 2 - 1, ySize + TITLE_SIZE - 1, GUI_WHITE, GUI_LIGHTBLUE);
  GUI_MEMDEV_Select(hDstEnlarge);
  Add = 1;
  Mag = MAG;
  i   = 1;
  do {
    TimeContinue = GUIDEMO_GetTime() + MIN_TIME_PER_FRAME_ENLARGE;
    //
    // Use HQ version only for last step
    //
    if (i == 0) {
      pfRotate = GUI_MEMDEV_RotateHQ;
    } else {
      pfRotate = GUI_MEMDEV_Rotate;
    }
    //
    // Calculate magnification
    //
    if (Add > 0) {
      Mag = (Mag * 1050) / 1000;
    } else {
      Mag = (Mag * 1000) / 1050;
    }
    if (Mag < MAG) {
      Mag = MAG;
    }
    //
    // Do animation
    //
    GUI_MEMDEV_Write(hLCDEnlarge);
    pfRotate(pImage->hMem, hDstEnlarge, (((xSize * 2) / 3) - pImage->xSize) / 2, (ySize - pImage->ySize) / 2, 0, Mag);
    GUI_MULTIBUF_Begin();
    GUI_MEMDEV_CopyToLCD(hDstEnlarge);
    GUI_MULTIBUF_End();
    i += Add;
    if (i == 20) {
      Add = -Add;
    }
    //
    // Delay
    //
    TimeNow = GUIDEMO_GetTime();
    if (TimeContinue > TimeNow) {
      GUI_Delay(TimeContinue - TimeNow);
    }
    if (GUIDEMO_CheckCancel()) {
      GUI_MEMDEV_Delete(hLCD);
      GUI_MEMDEV_Delete(hDstShift);
      GUI_MEMDEV_Delete(hDstEnlarge);
      GUI_MEMDEV_Delete(hLCDEnlarge);
      GUI_MEMDEV_Select(0);
      hLCD        = 0;
      hDstShift   = 0;
      hDstEnlarge = 0;
      hLCDEnlarge = 0;
      return 1;
    }
  } while (i > 0);
  if (Index == 0) {
    Index = _NumImages - 1;
  } else {
    Index--;
  }
  //
  // Remove memory devices (if required)
  //
  if (CleanUp) {
    GUI_MEMDEV_Delete(hLCD);
    GUI_MEMDEV_Delete(hDstShift);
    GUI_MEMDEV_Delete(hDstEnlarge);
    GUI_MEMDEV_Delete(hLCDEnlarge);
    GUI_MEMDEV_Select(0);
    hLCD        = 0;
    hDstShift   = 0;
    hDstEnlarge = 0;
    hLCDEnlarge = 0;
  }
  return 0;
}

/*********************************************************************
*
*       _DemoZoomAndRotate
*/
static void _DemoZoomAndRotate(void) {
  int FontDistY;
  int xSize;
  int ySize;
  int i;

  xSize = LCD_GetXSize();
  ySize = LCD_GetYSize();
  //
  // Get images
  //
  for (i = 0; i < _NumImages; i++) {
    if (_GetImage(&_aImage[i])) {
      return;
    }
  }
  //
  // Set background and headline
  //
  GUI_DrawGradientV(0, 0,          xSize - 1, TITLE_SIZE - 1, GUI_LIGHTBLUE, GUI_WHITE);
  GUI_DrawGradientV(0, TITLE_SIZE, xSize - 1, ySize      - 1, GUI_WHITE,     GUI_LIGHTBLUE);
  GUI_DrawBitmap(&_bmSeggerLogo50, 6, (TITLE_SIZE - _bmSeggerLogo50.YSize) / 2);
  GUI_SetTextMode(GUI_TM_TRANS);
  GUI_SetFont(&GUI_FontRounded22);
  FontDistY = GUI_GetFontDistY();
  GUI_SetColor(GUI_BLACK);
  GUI_DispStringHCenterAt("Zoom and rotate", 62 + (xSize - 62) / 2,  (TITLE_SIZE - FontDistY) / 2);
  if (_RotateAndZoomImage() == 0) {
    if (_ShiftImage(0) == 0) {
      if (_ShiftImage(0) == 0) {
        _ShiftImage(1);
      }
    }
  }
  //
  // Remove memory devices
  //
  for (i = 0; i < _NumImages; i++) {
    GUI_MEMDEV_Delete(_aImage[i].hMem);
  }
}

/*********************************************************************
*
*       Public code
*
**********************************************************************
*/
/*********************************************************************
*
*       GUIDEMO_ZoomAndRotate
*/
void GUIDEMO_ZoomAndRotate(void) {
  GUIDEMO_ConfigureDemo("Zoom and rotate", "Zooming and rotating any\nkind of images with emWin.", GUIDEMO_SHOW_CURSOR | GUIDEMO_SHOW_CONTROL);
  _DemoZoomAndRotate();
}

#else

void GUIDEMO_ZoomAndRotate_C(void);
void GUIDEMO_ZoomAndRotate_C(void) {}

#endif  // SHOW_GUIDEMO_ZOOMANDROTATE && GUI_WINSUPPORT && GUI_SUPPORT_MEMDEV

/*************************** End of file ****************************/
